/*  
 *  - - - - - - - - - - - -
 *   g a l _ g m o p v 0 0  
 *  - - - - - - - - - - - -
 *  
 *  This routine is part of the General Astrodynamics Library  
 *  
 *  Description: 
 *  
 *  Moon position and velocity, with respect to the Geocentric Celestial 
 *  Reference Frame (GCRF).  
 *  
 *  Status:
 *  
 *     support routine.  
 *  
 *  Given:
 *  
 *     epoch1              d        TDB epoch part A (Note 1)  
 *     epoch2              d        TDB epoch part B (Note 1) 
 *     icor                i        correction type
 *                                  icor=0, the constants are fitted to LLR observations provided 
 *                                  from 1970 to 2001; it is the default value;
 *                                  icor=1, the constants are fitted to DE405 ephemeris over one 
 *                                  century (1950-2060); the lunar angles W1, W2, W3 receive 
 *                                  also additive corrections to the secular coefficients.
 *  
 *  Returned:
 *  
 *     pv               d[2][3]     Geocentric Moon position & velocity ( m, m/s )  
 *     gal_gmopv00         i        status: 
 *                                    0 = Success  
 *                                    1 = warning: date outside 1940-2060 CE  
 *  
 *  Notes:  
 *  
 *  1) The epoch epoch1+epoch2 is a Julian Date, apportioned in  
 *     any convenient way between the two arguments.  For example,  
 *     JD(TDB)=2450123.7 could be expressed in any of these ways,  
 *     among others:  
 *  
 *            epoch1       epoch2  
 *  
 *         2450123.7          0.0        (JD method)  
 *         2451545.0      -1421.3        (J2000 method)  
 *         2400000.5      50123.2        (MJD method)  
 *         2450123.5          0.2        (date & time method)  
 *  
 *     The JD method is the most natural and convenient to use in  
 *     cases where the loss of several decimal digits of resolution  
 *     is acceptable.  The J2000 method is best matched to the way  
 *     the argument is handled internally and will deliver the  
 *     optimum resolution.  The MJD method and the date & time methods  
 *     are both good compromises between resolution and convenience.  
 *     However, the accuracy of the result is more likely to be  
 *     limited by the algorithm itself than the way the epoch has been  
 *     expressed.  
 *  
 * 
 *  References:
 *
 *     LUNAR SOLUTION ELP version ELP/MPP02 
 *     Jean CHAPRONT and G´erard FRANCOU 
 *     Observatoire de Paris -SYRTE department - UMR 8630/CNRS 
 *     October 2002  
 *
 *  This revision:
 *  
 *     2008 September 21  
 *  
 *  Copyright (C) 2008 Paul C. L. Willmott. See notes at end.  
 *  
 *-----------------------------------------------------------------------  
 */  

#include <stdio.h>
#include <math.h>  
#include "gal_gmopv02.h"
#include "gal_const.h"
  
int  
gal_gmopv02  
 (  
    double epoch1,  
    double epoch2,
    int icor,  
    double pv[2][3] 
 ) 
 
{

  double t[5], v[6] ;

  double ppw, qpw, ppw2, qpw2, ppwqpw, rap, ppwra, qpwra,
         clamb, slamb, cbeta, sbeta, cw, sw, x1, x2, x3, xp1, xp2, xp3,
         x, y, yp, xp, pw, qw, ra, pwqw, pw2, qw2, pwra, qwra, tj ;
         
  int n, k, it, jstat, iv ;

  const double a405 = 384747.9613701725 ;
  const double aelp = 384747.980674318 ;
  
/*
 * --- Precession coefficients for P and Q (Laskar, 1986) ---------------
 */
 
  const double p1 =   0.10180391e-04 ;
  const double p2 =   0.47020439e-06 ;
  const double p3 =   -0.5417367e-09 ;
  const double p4 =   -0.2507948e-11 ;
  const double p5 =     0.463486e-14 ;

  const double q1 = -0.113469002e-03 ;
  const double q2 =   0.12372674e-06 ;
  const double q3 =    0.1265417e-08 ;
  const double q4 =   -0.1371808e-11 ;
  const double q5 =    -0.320334e-14 ;

#define MAX1 2645
#define MAX2 33256
    
  static const double w[2][5][3] = {  
    {  
      {    3.8103439203219089e+00,    1.4547893480700087e+00,    2.1824386755573189e+00 } , 
      {    8.3996847302074329e+03,    7.0993305482306610e+01,   -3.3781427433053146e+01 } , 
      {   -3.3191992975274604e-05,   -1.8550474361662286e-04,    3.0829301981755390e-05 } , 
      {    3.2017095500473753e-08,   -2.1839401892941265e-07,    3.6967043184602116e-08 } , 
      {   -1.5363745554361197e-10,    1.0327016221314225e-09,   -1.7385418604587960e-10 } , 
    } ,  
    {  
      {    3.8103440908308803e+00,    1.4547895404440776e+00,    2.1824388474237688e+00 } , 
      {    8.3996847300719292e+03,    7.0993305448479248e+01,   -3.3781427419672326e+01 } , 
      {   -3.3189520425500942e-05,   -1.8548192818782712e-04,    3.0816644950982666e-05 } , 
      {    3.1102494491060616e-08,   -2.1961637966359412e-07,    3.6447710769397583e-08 } , 
      {   -2.0328237648922845e-10,    1.0327016221314225e-09,   -1.7385418604587960e-10 } , 
    } ,  
  } ;  

  static const int nmpb[3][3] = {  
    {     1023,      918,      704 } , 
    {        0,     1023,     1941 } , 
    {     1022,     1940,     2644 } , 
  } ;  

  static const int nper[3][4][3] = {  
    {  
      {    11314,     6462,    12115 } , 
      {     1199,      516,     1165 } , 
      {      219,       52,      210 } , 
      {        2,        0,        2 } , 
    } , 
    {  
      {        0,    12734,    19764 } , 
      {    11314,    19196,    31879 } , 
      {    12513,    19712,    33044 } , 
      {    12732,    19764,    33254 } , 
    } , 
    {  
      {    11313,    19195,    31878 } , 
      {    12512,    19711,    33043 } , 
      {    12731,    19763,    33253 } , 
      {    12733,    19763,    33255 } , 
    } , 
  } ;  

  static const double cmpb[2][MAX1] = {  
    {  
      -4.1159573394034464e+02, 
       4.2032545247910946e-01, 
      -5.8996898355993734e-04, 
       1.5999224588998434e-04, 
      -8.0187327780507925e-02, 
       3.9533300966452579e+01, 
       2.2639585879999784e+04, 
      -4.5099609183190310e+01, 
       9.0916994547684135e-02, 
      -1.8999224588998434e-04, 
       6.7997934422036981e-04, 
      -1.3725700882644198e+00, 
       7.6902571867111135e+02, 
      -3.9976133688320399e+00, 
       1.2969592010282917e-02, 
      -4.0000000000000003e-05, 
       9.0000000000000006e-05, 
      -5.6839244828719195e-02, 
       3.6123814129090093e+01, 
      -3.2865586543214154e-01, 
       1.5299526792758415e-03, 
      -3.5399567052666740e-03, 
       1.9336824738488416e+00, 
      -2.6019715516762902e-02, 
       1.5999699270073610e-04, 
      -2.2999786245648000e-04, 
       1.1100089397044806e-01, 
      -2.0099807621083202e-03, 
       2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       6.6500634929455380e-03, 
      -1.5000173951148780e-04, 
       4.1000434877871956e-04, 
      -1.0000000000000001e-05, 
       3.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -2.3999647491972464e-04, 
       6.0000000000000002e-05, 
      -3.3199034030053608e-03, 
       6.3996817983555127e-04, 
      -4.7018579161268337e-02, 
       6.9999999999999994e-05, 
      -3.0000000000000001e-05, 
       6.2296940378636818e-03, 
      -6.6937872520878050e-01, 
       1.0599428920506420e-03, 
      -1.6998611154247725e-04, 
       5.1647380979449500e-02, 
      -9.6790567959049891e+00, 
       2.4288759165808704e-02, 
      -4.0000000000000003e-05, 
      -6.2995306964755209e-04, 
       3.0155423091948980e-01, 
      -1.4732138419770180e+02, 
      -8.0485756779276163e-02, 
       1.2999612294499217e-04, 
      -2.2998611193059368e-04, 
       7.5006159646341927e-02, 
      -6.6641753993238694e+02, 
       4.1339790248083047e-01, 
      -8.2994218887740977e-04, 
       1.0999612294499217e-04, 
      -8.2815647886971794e-02, 
      -1.0938036372441518e+02, 
       2.6336624054187213e-01, 
      -6.1995307003566847e-04, 
      -3.0000000000000001e-05, 
       2.6768637581412277e-02, 
      -7.6301553320624675e+00, 
       4.3377800674324000e-02, 
      -1.4999612294499216e-04, 
       1.6599154655578634e-03, 
      -5.4592266032453873e-01, 
       5.2597397476500012e-03, 
      -3.0000000000000001e-05, 
       1.4000086975574390e-04, 
      -3.9468809119166517e-02, 
       5.4997732148232242e-04, 
       1.0000000000000001e-05, 
      -2.8699183274130229e-03, 
       5.0000000000000002e-05, 
      -2.0999560516398073e-04, 
      -2.0000000000000002e-05, 
      -8.9993865652492929e-05, 
       2.0000000000000002e-05, 
      -1.1899219453841306e-03, 
       1.3998773130498583e-04, 
      -1.5138965730477152e-02, 
       3.0000000000000001e-05, 
       9.9991435746851027e-04, 
      -1.9309647372217842e-01, 
       5.9994716713768193e-04, 
       4.7295767471812293e-03, 
      -2.5679459329852983e+00, 
       1.2998772936440373e-04, 
      -3.5496785749220628e-03, 
      -7.4474952294939296e+00, 
       5.1895322388652177e-03, 
      -1.0000000000000001e-05, 
      -3.9996157725738041e-04, 
      -1.1616866904843302e+00, 
       2.6997615238340497e-03, 
       3.9996331288770395e-04, 
      -6.5125430804016088e-02, 
       3.4996857825569998e-04, 
       2.0000000000000002e-05, 
      -3.4097650171113440e-03, 
       3.0000000000000001e-05, 
      -1.4998860106072974e-04, 
      -3.0000000000000001e-05, 
      -3.4996493303832892e-04, 
       2.0000000000000002e-05, 
      -4.1595529644513811e-03, 
       2.0000000000000002e-05, 
       7.9993865652492929e-05, 
      -5.1414425180471221e-02, 
       3.0000000000000001e-05, 
      -1.5998159695747877e-04, 
      -1.0134887457024103e-01, 
       6.9993865652492917e-05, 
      -1.7958047296051272e-02, 
       4.0000000000000003e-05, 
      -9.4989832497149504e-04, 
      -5.0000000000000002e-05, 
      -8.9987731304985851e-05, 
      -1.0798422449238381e-03, 
      -1.2798159719034862e-03, 
      -3.1995092521994341e-04, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       1.0998159692189333e-04, 
       2.0000000000000002e-05, 
       1.9997546253880082e-04, 
       2.5397239523170926e-03, 
       6.9993865625803844e-05, 
      -1.0000000000000001e-05, 
       4.9999999982207283e-05, 
       2.4998246662425908e-04, 
      -3.9999999982207283e-05, 
       2.2898387182755538e-03, 
      -3.1997158544820760e-04, 
       4.3036852149411435e-02, 
      -6.9999999973310921e-05, 
      -7.9999999973310934e-05, 
      -3.2997632604020015e-04, 
      -3.6997720083256121e-04, 
      -4.9999999982207283e-05, 
       2.0000000000000002e-05, 
       1.4000086971126209e-04, 
      -2.0000000000000002e-05, 
       6.8997894061651268e-04, 
       2.0000000000000002e-05, 
      -1.0999999996441456e-04, 
      -6.9397623496818854e-03, 
       1.2999999995551818e-04, 
      -9.9994942485945619e-04, 
      -1.3635527134034794e-01, 
       1.2999999995551818e-04, 
       2.0000000000000002e-05, 
      -1.2809306653630607e-02, 
      -5.5692003453874517e-01, 
       1.5499176474605520e-03, 
      -2.1198888374956821e-03, 
      -1.2072586850794553e-01, 
       5.9996469983172645e-04, 
       1.1999999995551821e-04, 
      -1.4829514347480970e-02, 
       1.1999999995551821e-04, 
       1.0000000000000001e-05, 
      -1.4899517456855838e-03, 
       2.0000000000000002e-05, 
      -1.4000086971126209e-04, 
      -1.0000000000000001e-05, 
      -4.9999999982207283e-05, 
       1.0000000000000001e-05, 
      -7.2000521750913268e-04, 
       8.9999999964414566e-05, 
      -9.5500647457892728e-03, 
       2.0000000000000002e-05, 
       6.8999184684151752e-04, 
      -1.2812063098189327e-01, 
       3.7999485270396856e-04, 
       1.0000000000000001e-05, 
       2.8899630362230668e-03, 
      -1.7529659229032848e+00, 
      -1.4199772108449472e-03, 
       1.8999224582770983e-04, 
      -2.3789613927173655e-02, 
      -1.8584706832567544e+01, 
       1.5979758557932412e-02, 
      -1.0999999996441456e-04, 
       1.2299573515797723e-03, 
      -5.8330993741777948e-01, 
      -1.2498811853267013e+02, 
       2.5428571489518664e-01, 
      -5.3998061415002090e-04, 
       1.0999999996441456e-04, 
      -4.0119364730095274e-02, 
      -8.4530981740238644e+00, 
       4.4499320857384859e-02, 
      -1.4999612289161402e-04, 
      -7.6999097899966673e-04, 
      -5.8399224155828255e-01, 
       5.3999243392690577e-03, 
      -2.9999999991103641e-05, 
      -5.9999999982207282e-05, 
      -4.0740230310782430e-02, 
       5.5999485457337969e-04, 
      -2.8500182619425442e-03, 
       4.9999999982207283e-05, 
      -2.0000173944031695e-04, 
      -1.0000000000000001e-05, 
       1.1000086972015847e-04, 
      -1.0000000000000001e-05, 
       1.5499543568756049e-03, 
      -1.1999999995551821e-04, 
       2.2959263029679815e-02, 
      -6.0000000758440126e-05, 
      -7.0996644047197814e-04, 
       3.4964830675722081e-01, 
       6.2996557073402700e-04, 
      -7.9999999973310934e-05, 
       1.0776929071813302e+00, 
      -1.1643492701065334e-12, 
       1.7000608706358361e-04, 
       1.7954467429825353e+01, 
      -3.5868072876963038e-02, 
       6.9999999973310921e-05, 
      -1.5699450438443900e-03, 
       1.2618161650053286e+00, 
      -6.5896596303992002e-03, 
       2.0000000000000002e-05, 
       2.7998385298776872e-04, 
       9.2046948009017429e-02, 
      -8.3995642901608524e-04, 
       2.0000000000000002e-05, 
       6.7897815393456072e-03, 
      -8.9999999964414566e-05, 
       4.9998507580252722e-04, 
       3.9999999991103643e-05, 
       3.9999999991103643e-05, 
       3.7997106686427421e-04, 
       3.6097475224265135e-03, 
      -2.9997835042503917e-03, 
       1.0000000000000001e-05, 
       2.9993865643596560e-05, 
      -3.9677107744042192e-02, 
       1.0999386561690749e-04, 
       2.0000000388116424e-05, 
      -8.4893890732562621e-03, 
       4.9999999982207283e-05, 
      -1.0199298657208036e-03, 
       1.0000000000000001e-05, 
      -1.0999386561690749e-04, 
      -1.0000000000000001e-05, 
       6.9993865625803844e-05, 
      -1.0998773126940041e-04, 
      -7.5226604952611881e-09, 
      -5.5993865594109284e-04, 
      -8.9987731278296778e-05, 
      -1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.6995092521994339e-04, 
       4.9990798401116103e-04, 
       4.9993865652492925e-05, 
       2.0000000000000002e-05, 
       3.1995179458757094e-04, 
       8.1188229736049930e-03, 
      -2.0000000000000002e-05, 
       4.0000000000000003e-05, 
       1.3308050994493486e-02, 
      -3.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       1.3298063078183069e-03, 
       1.1998159695747877e-04, 
       1.0000000000000001e-05, 
       3.0000000000000001e-05, 
       4.8994739975155165e-04, 
      -4.0000000000000003e-05, 
       9.6489691892412861e-03, 
      -2.0000000000000002e-05, 
      -2.0997546260997170e-04, 
       2.4694321708693218e-01, 
      -5.1993703637430422e-04, 
       1.6297949645372397e-03, 
       3.3979270311502285e-01, 
      -7.4990862115303523e-04, 
      -6.4991562564440254e-04, 
       3.2996434438908130e-02, 
      -1.5998159695747877e-04, 
      -4.9993865652492925e-05, 
       2.9496844414616777e-03, 
      -3.0000000000000001e-05, 
       2.5997019763009213e-04, 
       2.0000000000000002e-05, 
       5.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       8.7994387467127638e-04, 
      -1.2998773130498583e-04, 
       1.5438938193938302e-02, 
      -1.1099043460659956e-03, 
       2.9419934557448169e-01, 
      -7.1993576897464459e-04, 
      -4.3396143001681068e-03, 
       7.3712011460145677e+00, 
      -1.5408628973371889e-02, 
       3.0000000000000001e-05, 
      -8.0000000000000007e-05, 
       6.3134271742407974e-02, 
       8.0501723580323876e+00, 
      -1.8158365218141039e-02, 
       4.0000000000000003e-05, 
       1.0000000000000001e-05, 
      -1.6438530709394418e-02, 
       7.5167636331432242e-01, 
      -3.7796658667951129e-03, 
       1.0000000000000001e-05, 
      -1.3298829357242100e-03, 
       6.4385513274555953e-02, 
      -5.4995104419268989e-04, 
      -1.0999386565249293e-04, 
       5.3796343552609304e-03, 
      -6.9999999999999994e-05, 
       4.3997193675346353e-04, 
       4.0000000000000003e-05, 
       1.0000086975574391e-04, 
      -3.0000000000000001e-05, 
       1.7799464378812082e-03, 
      -3.5998258646220195e-04, 
       2.9489101202063411e-02, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
      -3.6498204034846661e-03, 
       4.7840473082119511e-01, 
       3.0000000000000001e-05, 
       5.0000000000000002e-05, 
      -2.8038574353863834e-02, 
       8.6055317947092895e+00, 
      -2.1568901297389149e-02, 
       1.1999612294499217e-04, 
      -1.4999612294499216e-04, 
      -6.1026812375840983e-02, 
       2.0543595303766264e+02, 
      -4.2621768879515032e-01, 
       9.2993391621014993e-04, 
      -2.8597949118859432e-03, 
       2.1460741255442124e+00, 
       1.6472861853103672e+02, 
      -3.8212969058735435e-01, 
       8.8993918041379650e-04, 
       1.7998611154247725e-04, 
      -3.7451033600200928e-01, 
       1.4530277885386321e+01, 
      -7.3726190818842491e-02, 
       2.4997997719497018e-04, 
       1.0000000000000001e-05, 
      -2.8868533048228329e-02, 
       1.1770012792645836e+00, 
      -1.0069497864886401e-02, 
       5.0000000000000002e-05, 
      -2.3198866534689115e-03, 
       9.3197090253788403e-02, 
      -1.1699437610301531e-03, 
      -1.8999473540823683e-04, 
       7.2997779147964603e-03, 
      -1.2000000000000000e-04, 
      -1.0000000000000001e-05, 
       5.6998594495996549e-04, 
      -1.0000000000000001e-05, 
       4.0000000000000003e-05, 
       3.0000000000000001e-05, 
       3.6000434877871959e-04, 
      -1.1000086975574391e-04, 
       4.9700469629290062e-03, 
      -1.1099893115061672e-03, 
       6.8500554304412917e-02, 
      -6.9999999999999994e-05, 
       5.0000000000000002e-05, 
      -1.0769882602748599e-02, 
       9.4827608962253374e-01, 
      -5.3999184785795211e-04, 
       2.3999311564572824e-04, 
      -9.1538855566803426e-02, 
       1.3194063561797675e+01, 
       4.6699433182892736e-03, 
      -3.0000000000000001e-05, 
       3.6999010718211502e-04, 
      -5.6064210406550086e-01, 
       2.1165555237566161e+02, 
      -5.3571236892975038e-01, 
       2.8199117704060103e-03, 
       3.0000000000000001e-05, 
      -8.4997175203998191e-03, 
       1.7902697687415572e-01, 
       4.5864383203391226e+03, 
      -9.3658634662895146e+00, 
       1.9929343790710426e-02, 
      -4.0000000000000003e-05, 
       1.4999224588998431e-04, 
      -7.5177389373332126e-02, 
       5.5177057781935453e+01, 
       2.3699139356645110e+03, 
      -5.7416071078822393e+00, 
       1.3719545250575591e-02, 
      -3.0000000000000001e-05, 
       1.0000000000000001e-05, 
       5.5998148059954039e-04, 
      -6.3831490688693977e+00, 
       1.9195619729415327e+02, 
      -9.9021535743421063e-01, 
       3.4398885053591481e-03, 
      -1.0000000000000001e-05, 
       8.0000000000000007e-05, 
      -4.5379355183867159e-01, 
       1.4379697414390638e+01, 
      -1.2385836450364340e-01, 
       5.8998322321595980e-04, 
       2.0000000000000002e-05, 
      -3.3359573716410879e-02, 
       1.0594960024585396e+00, 
      -1.3319840819853742e-02, 
       8.0000000000000007e-05, 
      -2.4699730068339547e-03, 
       7.7650572853363117e-02, 
      -1.3099854336749263e-03, 
       1.0000000000000001e-05, 
      -1.8000173951148783e-04, 
       5.6800469606003087e-03, 
      -1.2000086975574391e-04, 
      -1.0000000000000001e-05, 
       4.1000434839060315e-04, 
      -1.0000000000000001e-05, 
       3.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.5000173912337137e-04, 
      -3.0000000000000001e-05, 
       1.7299525679594346e-03, 
      -1.9999473463200399e-04, 
       1.8189447730470487e-02, 
      -3.0012270247479851e-05, 
      -1.1599428889457106e-03, 
       1.7623440184335515e-01, 
      -2.3998559376146584e-04, 
       2.0000000000000002e-05, 
      -3.4598243130128988e-03, 
       2.4888871030915660e+00, 
      -9.9999868040416062e-06, 
       1.6998611154247725e-04, 
      -2.2798819841528891e-03, 
      -2.8397100818380377e+01, 
       6.7416467629333718e-02, 
      -1.5999612294499219e-04, 
       1.9798565996754247e-03, 
      -1.4371623834197103e+00, 
      -2.4358228257667186e+01, 
       6.5406557807381582e-02, 
      -1.6998611154247725e-04, 
       3.0000000000000001e-05, 
       8.4715547794242618e-02, 
      -2.9145403611714258e+00, 
       1.6049176218736080e-02, 
      -6.0000000000000002e-05, 
       8.8895484966045463e-03, 
      -2.9022054660535118e-01, 
       2.6198687662668191e-03, 
      -1.0000000000000001e-05, 
       8.4995816843303634e-04, 
      -2.6759174399409124e-02, 
       3.4998171631834158e-04, 
       8.0000000000000007e-05, 
      -2.3599297708571275e-03, 
       4.0000000000000003e-05, 
      -1.9999560516398073e-04, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -4.8996667293793312e-04, 
       4.0000000000000003e-05, 
      -9.4393467713475929e-03, 
       2.0000000000000002e-05, 
       6.0000000000000002e-05, 
      -2.4988251297490727e-01, 
       5.9994716713768193e-04, 
       2.0000000000000002e-05, 
      -6.5394169047861220e-03, 
      -2.5211989535398200e+00, 
       5.1495391891488405e-03, 
      -1.0000000000000001e-05, 
       4.0000000000000003e-05, 
      -2.3487921490233606e-02, 
      -1.8574704319388002e-01, 
       9.5991962206027335e-04, 
      -1.9998317659648854e-03, 
      -1.3429085736074224e-02, 
       1.1999386565249293e-04, 
      -1.3998773130498583e-04, 
      -6.5995614336629047e-04, 
      -1.0000000000000001e-05, 
      -8.9987731304985851e-05, 
      -9.2989832303091297e-04, 
      -3.0000000000000001e-05, 
      -1.8797940367945940e-03, 
       6.9987731304985840e-05, 
       3.7995879791458901e-04, 
       2.0000000000000002e-05, 
       4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       2.0000000000000002e-05, 
      -1.6998159689520425e-04, 
      -5.2994478990937995e-04, 
      -4.9999999982207283e-05, 
       2.9996932815570831e-04, 
       2.9999999991103641e-05, 
      -1.3998773125160769e-04, 
      -3.9897185923957190e-03, 
      -7.9999999973310934e-05, 
      -1.1819155037885465e-02, 
       2.9999999991103641e-05, 
      -1.0199070014227888e-03, 
       5.1796153179470908e-03, 
      -3.9999999982207283e-05, 
       3.6997546247652629e-04, 
      -1.2999999995551818e-04, 
      -2.9699034944760988e-03, 
      -8.3107217151914367e-02, 
       1.9999386558132204e-04, 
      -1.8899041135789242e-03, 
      -2.3140200974508465e-01, 
       7.4995943518658525e-04, 
       2.0000000000000002e-05, 
      -1.9478951091972996e-02, 
       6.5067436698430528e-02, 
       1.9999386480508920e-04, 
      -8.3995468950459737e-04, 
       3.2998552073227852e-03, 
       4.9999999982207283e-05, 
      -3.9999999982207283e-05, 
      -1.1000086777957636e-04, 
       1.0000000000000001e-05, 
      -4.9999999982207283e-05, 
      -4.9999999982207283e-05, 
       1.0000000000000001e-05, 
      -1.7400113062197183e-03, 
       1.0999999996441456e-04, 
      -4.4420192896920646e-02, 
       1.0999999996441456e-04, 
      -2.7999612362336504e-04, 
      -1.2241239597244933e+00, 
       2.9799591618492887e-03, 
      -1.0000000000000001e-05, 
       5.9999999982207282e-05, 
      -2.9789531293046706e-02, 
      -3.2096876062154860e+00, 
       1.2149816237149513e-02, 
      -3.9999999991103643e-05, 
       2.4999224580102075e-04, 
      -2.5303567070897559e-01, 
       4.0422723984270315e-01, 
       3.7499475238193950e-03, 
      -2.0000000000000002e-05, 
       4.9999999982207283e-05, 
      -1.0619833428403244e-02, 
       3.6963186019266567e-04, 
       8.2999097548882618e-04, 
      -5.0999398522354503e-04, 
      -3.8900503698152157e-03, 
       1.3999999995551818e-04, 
      -2.0000000000000002e-05, 
      -6.2000347686888091e-04, 
       2.0000000000000002e-05, 
      -6.9999999973310921e-05, 
      -5.9999999982207282e-05, 
      -5.8997894065209821e-04, 
       2.0000000000000002e-05, 
      -6.2298025870411193e-03, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
      -4.1888592249990420e-02, 
       6.9999999973310921e-05, 
       1.1599394126273301e-03, 
       2.7376051097820830e-01, 
      -1.3799302709093879e-03, 
       8.9095189191142985e-03, 
       1.4989475424557247e-01, 
      -9.5994855395715561e-04, 
       1.9999386558132204e-04, 
       2.2439244786308304e-02, 
      -2.1999386557242567e-04, 
      -1.0000000000000001e-05, 
       2.5599175481060099e-03, 
      -3.9999999991103643e-05, 
       2.5998947072751000e-04, 
       2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -1.9998860098955888e-04, 
      -3.5997632991246808e-04, 
      -1.0000000000000001e-05, 
      -1.2699105963355189e-03, 
      -8.9987731269400412e-05, 
      -1.9098606469199124e-03, 
       1.0000000000000001e-05, 
      -3.8996932774090282e-04, 
      -5.9999999982207282e-05, 
       1.2268721703235996e-08, 
      -6.4850867194194969e-09, 
       3.9999999991103643e-05, 
      -1.3998773126050403e-04, 
      -1.9985019202383471e-05, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       3.9993865652492926e-05, 
       3.0000000000000001e-05, 
       1.0000000000000001e-05, 
       3.2995179419945445e-04, 
       8.5987291627325705e-04, 
       5.3992025192994223e-04, 
       8.9987731304985851e-05, 
       1.0000000000000001e-05, 
       2.6997019763009216e-04, 
       7.5491962034395319e-03, 
      -4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       1.7408121234192286e-02, 
      -8.9987731304985851e-05, 
       6.9993865652492917e-05, 
       9.7089421656193827e-03, 
      -6.0000000000000002e-05, 
      -3.0000000000000001e-05, 
       1.5998255786759898e-03, 
      -1.0000000000000001e-05, 
       1.9997546222185527e-04, 
       2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       5.5996159506659432e-03, 
      -4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       1.5779894934692520e-01, 
      -8.9991962167215679e-04, 
       1.0799026077188568e-03, 
       3.0871801726609527e-01, 
      -1.5998597839303550e-03, 
       9.0991527172908798e-04, 
       1.5225907067252270e-01, 
      -8.8991875152829648e-04, 
      -5.1995630878445289e-04, 
       2.4108302281639689e-02, 
      -2.0998773091686942e-04, 
      -8.9993865652492929e-05, 
       2.8997965968995945e-03, 
      -4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
       3.0997720134522664e-04, 
       3.0000000000000001e-05, 
      -5.0000000000000002e-05, 
      -3.3999034018410119e-04, 
       4.0000000000000003e-05, 
      -1.9999473540823683e-04, 
       6.9999999999999994e-05, 
       9.9816821877271866e-02, 
      -7.7995729790105968e-04, 
       1.0000000000000001e-05, 
       4.2997558235895091e-04, 
       2.7318889692915320e+00, 
      -1.5329221553467349e-02, 
       6.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.2888805918616802e-02, 
       4.3740094938716609e+00, 
      -2.2728826686090872e-02, 
       8.0000000000000007e-05, 
      -1.0000000000000001e-05, 
       6.6596364444854103e-03, 
       1.8707647458517696e+00, 
      -1.1139425136911808e-02, 
       4.0000000000000003e-05, 
      -6.0000000000000002e-05, 
      -7.3196177906305606e-03, 
       2.8250054434222499e-01, 
      -2.5098722894065129e-03, 
       1.0000000000000001e-05, 
      -1.2099420211305488e-03, 
       3.2738956578608133e-02, 
      -4.0998171554210871e-04, 
      -1.3999999999999999e-04, 
       3.3598955697700357e-03, 
      -6.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       3.1999033979598473e-04, 
       3.0000000000000001e-05, 
      -6.9999999999999994e-05, 
       1.0000000000000001e-05, 
      -6.2000608712585816e-04, 
       1.0000000000000000e-04, 
      -4.3100339134879160e-03, 
       4.1999485438098326e-04, 
      -7.0000434994306872e-04, 
      -2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.2999699192450325e-04, 
       1.1868256998194013e+00, 
      -9.3398815099772318e-03, 
       6.9999999999999994e-05, 
       1.2999612294499217e-04, 
       6.6199084360495791e-03, 
       3.0772575081204977e+01, 
      -1.6976763932382027e-01, 
       6.0998235346021586e-04, 
       1.6999612255687576e-04, 
       3.0085540153188278e-01, 
       3.8429834552033142e+01, 
      -2.0133704790524942e-01, 
       7.1997847601709165e-04, 
      -3.0000000000000001e-05, 
      -2.3909624008531132e-02, 
       1.3899059643458889e+01, 
      -8.4998751180535084e-02, 
       3.4998836805874361e-04, 
      -6.0998148448070469e-04, 
      -6.6009003858817805e-02, 
       1.9777270053585612e+00, 
      -1.7809756222952736e-02, 
       9.0000000000000006e-05, 
       1.0000000000000001e-05, 
      -9.7998650559810805e-03, 
       2.1853112449899581e-01, 
      -2.7399656460985740e-03, 
       2.0000000000000002e-05, 
      -1.0899867014627026e-03, 
       2.1490147755081266e-02, 
      -3.5999398462523933e-04, 
      -1.1000000000000000e-04, 
       1.9800139118226218e-03, 
      -4.0000000000000003e-05, 
       1.7000173912337139e-04, 
       1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -4.0000000000000003e-05, 
       6.3997980905999266e-04, 
      -4.0000000000000003e-05, 
       2.6819163868398232e-02, 
      -2.9999999611883578e-05, 
      -3.8998084617448124e-04, 
      -3.5757788979026356e-01, 
       2.1998884421426308e-03, 
      -1.8749021572572678e-02, 
      -6.3520815370989836e-01, 
       3.5298208471858813e-03, 
      -1.0000000000000001e-05, 
       5.0996783912509163e-04, 
      -2.8910002713341587e-01, 
       1.8599058510230314e-03, 
       4.0000000000000003e-05, 
       1.8299049835959862e-03, 
      -5.0698322513450050e-02, 
       4.7997558119460167e-04, 
       3.1998472361760555e-04, 
      -6.6797901960926479e-03, 
       9.0000000000000006e-05, 
       4.0000000000000003e-05, 
      -7.6997367510060191e-04, 
       1.0000000000000001e-05, 
      -8.0000000000000007e-05, 
      -5.0000000000000002e-05, 
      -3.6897474745097724e-03, 
       2.0000000000000002e-05, 
       8.0000000000000007e-05, 
      -3.5137537313124745e-02, 
       1.8998773091686942e-04, 
       8.9991875114018010e-04, 
      -2.1708466218293903e-02, 
       1.1999386565249293e-04, 
       1.0000000000000001e-05, 
      -2.0798570721713870e-03, 
       2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -7.9994735408236837e-05, 
       1.0000000000000001e-05, 
       1.9997546260997170e-04, 
       2.2997546222185527e-04, 
       5.0000000000000002e-05, 
      -2.4536560417312939e-08, 
      -2.9999999991103641e-05, 
       1.0000000000000001e-05, 
      -1.2998773126050403e-04, 
      -5.1996493208416889e-04, 
      -2.9999999991103641e-05, 
      -1.0999386484067465e-04, 
      -2.0000000000000002e-05, 
       1.1999386560801113e-04, 
       2.0000000000000002e-05, 
      -6.9999999982207274e-05, 
      -1.8899412159804957e-03, 
       1.0000000000000001e-05, 
      -2.9999999991103641e-05, 
      -6.9297710874748784e-03, 
       3.9999999982207283e-05, 
      -3.9997997627639562e-04, 
      -1.5399556172891688e-03, 
       2.9999999991103641e-05, 
      -2.0999386558132204e-04, 
       1.0199614536594991e-03, 
      -2.9999999991103641e-05, 
       1.3999999995551818e-04, 
      -5.9000347765401017e-04, 
      -1.6860079983748045e-02, 
       9.9999999964414565e-05, 
      -3.0999699259397982e-04, 
      -5.7250247274650445e-02, 
       3.8999398449179391e-04, 
      -3.3499470572835388e-03, 
      -1.3500127298342856e-02, 
       2.3999612208869210e-04, 
      -2.0000000000000002e-05, 
      -1.3699767348697473e-03, 
       4.3899543106083695e-03, 
       5.9999999982207282e-05, 
      -1.8999999993772550e-04, 
       4.9999826147493424e-04, 
       1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
      -5.9999999982207282e-05, 
      -4.5998333436825001e-04, 
       1.0000000000000001e-05, 
       3.9198719503338525e-03, 
      -3.9999999991103643e-05, 
       1.5999999994662187e-04, 
       4.7498425414331160e-03, 
      -4.9999999982207283e-05, 
       4.9999999982207283e-05, 
       1.8099368660641159e-03, 
      -2.0000000000000002e-05, 
       3.4998860015994786e-04, 
       4.9999999982207283e-05, 
       1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -7.9993865625803856e-05, 
      -3.9999999982207283e-05, 
      -1.0000000000000001e-05, 
      -1.7792718719392219e-14, 
       1.0000000000000001e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       1.9998246632510624e-04, 
       5.8993426013644425e-04, 
       5.8993425974832785e-04, 
       2.3997546183373886e-04, 
       5.0000000000000002e-05, 
       1.3998773130498583e-04, 
       3.1197878076156724e-03, 
      -3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       8.3594152442899722e-03, 
      -6.9999999999999994e-05, 
       3.0000000000000001e-05, 
       7.4394761613223986e-03, 
      -6.9999999999999994e-05, 
      -2.0000000000000002e-05, 
       2.8497975103839398e-03, 
      -3.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       5.5995879713835613e-04, 
       8.0000000000000007e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       1.7599438247328123e-03, 
      -2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       3.8998764490379520e-02, 
      -3.8998171593022512e-04, 
       4.3997558197083453e-04, 
       9.0327054369161802e-02, 
      -7.9995642736908294e-04, 
       1.7999386487626008e-04, 
       7.1417617507999423e-02, 
      -6.3996556901585390e-04, 
      -2.4999085835322905e-04, 
       2.5099162687542434e-02, 
      -2.5998385347374511e-04, 
      -1.6000000000000001e-04, 
       4.7698477546273101e-03, 
      -6.0000000000000002e-05, 
      -3.0000000000000001e-05, 
       6.8997893930424869e-04, 
      -1.0000000000000001e-05, 
       9.0000000000000006e-05, 
       2.0000173912337139e-04, 
       1.3760086941420144e-02, 
      -1.7000086975574392e-04, 
       1.8000000000000001e-04, 
       2.9255151717808109e-01, 
      -2.8699635081669379e-03, 
       2.0000000000000002e-05, 
       3.6399492688990323e-03, 
       5.7156244661304834e-01, 
      -5.0899338262202351e-03, 
       3.0000000000000001e-05, 
       6.9997387627836866e-05, 
       3.9451146453452329e-01, 
      -3.5899492634654021e-03, 
       2.0000000000000002e-05, 
      -8.0000000000000007e-05, 
      -2.1399673150233143e-03, 
       1.2616048387604764e-01, 
      -1.3199810821793918e-03, 
      -1.0000000000000001e-05, 
      -9.3998709910661830e-04, 
       2.2850114603891289e-02, 
      -3.0999699192450325e-04, 
      -1.7999999961188358e-04, 
       3.1800156451242470e-03, 
      -5.0000000000000002e-05, 
      -3.0000000000000001e-05, 
       3.8000260810288250e-04, 
       4.0000000000000003e-05, 
       1.0000000000000001e-05, 
       2.7998946887589147e-04, 
      -4.7798447195122190e-03, 
       5.0000000000000002e-05, 
      -2.5998385386186157e-04, 
      -1.2819580618409182e-02, 
       1.2000000000000000e-04, 
      -6.9999999611883575e-05, 
      -1.0639642962437685e-02, 
       1.0000000000000000e-04, 
       8.0000000000000007e-05, 
      -3.9598670777521560e-03, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
      -8.2997193481288139e-04, 
       1.0000000000000001e-05, 
      -1.2999999961188356e-04, 
      -2.0000000000000002e-05, 
      -6.0000000000000002e-05, 
      -5.0996580162972357e-04, 
       1.0000000000000001e-05, 
      -6.0995966650598363e-04, 
       1.0000000000000001e-05, 
      -2.0998159618124593e-04, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -3.9999999991103643e-05, 
      -1.5999386521099834e-04, 
      -1.1999999956740178e-04, 
       2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -2.5000173864629140e-04, 
      -9.8000521547958680e-04, 
       1.0000000000000001e-05, 
      -4.9999999982207283e-05, 
      -6.8000260631911153e-04, 
       1.0000000000000001e-05, 
      -3.9999999982207283e-05, 
      -4.9999999205974438e-05, 
      -1.0000000000000001e-05, 
       4.9999999982207283e-05, 
       5.9999999982207282e-05, 
       1.0999999996441456e-04, 
       6.9999999973310921e-05, 
       2.9999999991103641e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       6.0000000000000002e-05, 
       1.8998860028449693e-04, 
       2.4998246554887342e-04, 
       1.4998773052875299e-04, 
       5.0000000000000002e-05, 
       1.0000000000000001e-05, 
       3.0000000000000001e-05, 
       5.7998507442798871e-04, 
       1.6999473482606218e-03, 
      -2.0000000000000002e-05, 
       1.9599342082573983e-03, 
      -3.0000000000000001e-05, 
       1.1199605339330290e-03, 
      -1.0000000000000001e-05, 
       3.6998946926400795e-04, 
       8.0000000000000007e-05, 
       1.0000000000000001e-05, 
       1.8000173912337139e-04, 
       3.2400208644349430e-03, 
      -5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       8.4000464671420763e-03, 
      -1.1000000000000000e-04, 
       1.0000000000000001e-05, 
       8.6600425121261086e-03, 
      -1.0999999961188359e-04, 
      -5.0000000000000002e-05, 
       4.5500195144169264e-03, 
      -6.0000000000000002e-05, 
      -4.0000000000000003e-05, 
       1.3900052127127172e-03, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       2.9000173834713855e-04, 
       5.0000000000000002e-05, 
      -6.9999999999999994e-05, 
      -2.3999473502012039e-04, 
      -2.8998860028449689e-04, 
      -1.6999386487626008e-04, 
      -6.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       3.0000000000000001e-05, 
       5.0000000000000002e-05, 
       4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       4.0000000000000003e-05, 
       1.1999999961188358e-04, 
       1.7000086897951108e-04, 
       1.2999999922376715e-04, 
       6.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.8461240060000968e+04, 
      -6.2964772977841825e+00, 
       5.9197458538809708e-03, 
      -2.3599035461247487e-03, 
       2.7986413324884101e+00, 
       9.9969365545185065e+02, 
       1.0101671484113945e+03, 
      -1.0193852023009262e+00, 
       1.5599366993320986e-03, 
      -4.3998235423644866e-04, 
      -1.3034701080366828e-01, 
       3.1759670884809800e+01, 
       6.1911950388791020e+01, 
      -1.1786731681604115e-01, 
       2.5998923859072039e-04, 
       3.0000000000000001e-05, 
      -1.4599681890483385e-03, 
       1.5813035948086931e+00, 
       3.9840545271942500e+00, 
      -1.1809750247165853e-02, 
       4.0000000000000003e-05, 
      -6.0000000000000002e-05, 
       9.1569770719508434e-02, 
       2.6324939977672279e-01, 
      -1.0899798186608965e-03, 
       5.6599939577684196e-03, 
       1.7679988827081935e-02, 
      -1.0000000000000000e-04, 
       3.6000347902297565e-04, 
       1.1999996756596471e-03, 
       2.0000000000000002e-05, 
       8.0000869755743914e-05, 
      -4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
      -5.2998206829307402e-04, 
      -1.4000086975574390e-04, 
       3.0000000000000001e-05, 
      -6.4597473547634478e-03, 
      -1.8599291897615730e-03, 
       2.2999085835322900e-04, 
      -7.4786974647102392e-02, 
      -2.3649035623645471e-02, 
       2.0000000000000002e-05, 
       1.5399112376924722e-03, 
      -7.9318664391493254e-01, 
      -3.0127728954834437e-01, 
       1.9098820720763220e-03, 
      -1.0000000000000001e-05, 
       6.7895861954236428e-03, 
      -6.7314362777559564e+00, 
      -5.6323537674990405e+00, 
      -9.1094391491054628e-03, 
       1.0000522513244267e-04, 
      -4.8396143029776244e+00, 
      -6.4600720929713926e+00, 
       1.1569281673480687e-02, 
      -2.0000000000000002e-05, 
      -7.0295714410915401e-03, 
      -5.0759179266592636e+00, 
      -5.3112784086527398e+00, 
       6.6795914615678711e-03, 
      -1.0000000000000001e-05, 
       1.4099203789511266e-03, 
      -3.1290679708702374e-01, 
      -6.3881314739994810e-01, 
       1.3499203797273595e-03, 
       3.0000000000000001e-05, 
      -2.4189016822182795e-02, 
      -6.1757502558210935e-02, 
       1.9999085835322900e-04, 
      -1.9099291893734565e-03, 
      -5.4797858278326480e-03, 
       2.0000000000000002e-05, 
      -1.5000086975574389e-04, 
      -4.5998119892544658e-04, 
      -1.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
      -1.5998860106072977e-04, 
      -4.0000000000000003e-05, 
      -1.6898696306909405e-03, 
      -4.8996192573906496e-04, 
       3.0000000000000001e-05, 
      -1.5708764479858101e-02, 
      -5.3595782675569343e-03, 
       4.0000000000000003e-05, 
       9.9993865652492928e-05, 
      -1.1334089571385463e-01, 
      -9.5102356895458595e-02, 
      -8.9993865652492929e-05, 
      -1.3998773130498583e-04, 
      -1.8008571604879082e-02, 
      -5.7285376738349582e-02, 
       1.5998385424997804e-04, 
      -6.9999999999999994e-05, 
      -6.1865021524293219e-02, 
      -5.5035578764496822e-02, 
       6.9999999999999994e-05, 
       1.0000000000000001e-05, 
      -2.8397768666359184e-03, 
      -5.2795835344773955e-03, 
       1.0000000000000001e-05, 
      -1.5998773130498586e-04, 
      -3.6997720212145950e-04, 
      -2.0000000000000002e-05, 
      -4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
      -3.2996406328258504e-04, 
      -1.0998773130498585e-04, 
      -2.1597448514783452e-03, 
      -1.8997785305151448e-03, 
       6.5991862945061870e-04, 
      -3.0996319701988892e-04, 
      -1.0898774320226047e-03, 
      -8.2990196980074966e-04, 
      -4.0000000000000003e-05, 
      -7.9993865652492929e-05, 
      -3.9993865652492926e-05, 
      -4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       2.9999999991103641e-05, 
       1.0998773126940041e-04, 
       1.1998773126050406e-04, 
      -2.0000000000000002e-05, 
       6.9999999973310921e-05, 
      -2.9999999991103641e-05, 
       6.3994791769827043e-04, 
      -1.0000000000000001e-05, 
       2.0398333889057457e-03, 
       2.1398211209513792e-03, 
      -5.9995405167545372e-04, 
       4.9999999982207283e-05, 
      -5.9999999973310922e-05, 
      -2.9999999991103641e-05, 
       2.9999999991103641e-05, 
       1.7999473534596232e-04, 
      -3.9999999991103643e-05, 
       9.5995891849875222e-04, 
      -1.3699436411977828e-03, 
      -2.9999999991103641e-05, 
       1.0309547887020005e-02, 
       7.9995769526918826e-04, 
      -5.1996996561452802e-04, 
      -1.3459392256837954e-02, 
      -1.8289191079292447e-02, 
       5.9999999982207282e-05, 
      -7.9999999973310934e-05, 
      -2.0119122438275329e-02, 
      -8.4496360037975109e-03, 
       2.0000000000000002e-05, 
      -1.7499283646819433e-03, 
      -1.4599383750179688e-03, 
      -1.4999999994662184e-04, 
      -1.7999473534596232e-04, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -1.1000086972015847e-04, 
      -3.9999999991103643e-05, 
      -1.2199961946660772e-03, 
      -4.3000260911599361e-04, 
      -1.2549955313959386e-02, 
      -5.3699786917397428e-03, 
       2.0000000000000002e-05, 
      -2.4999224580102075e-04, 
      -1.0963943472731864e-01, 
      -7.8459564258628559e-02, 
      -1.9999612287382131e-04, 
      -3.6899091590814701e-03, 
      -4.2988727861871229e-01, 
       1.3927888273461300e-01, 
       2.3399414433926996e-03, 
       2.9999999991103641e-05, 
      -3.2259166406104510e-02, 
      -4.8057401256423802e+00, 
      -5.3683960751123054e+00, 
       6.4498350180914191e-03, 
      -1.0000000000000001e-05, 
       1.0000000000000001e-05, 
      -1.8999612327083411e-04, 
      -5.8892588698557768e-01, 
      -6.6740564069239361e-01, 
       1.4199677141185122e-03, 
      -3.6359802477709945e-02, 
      -6.3829693184392300e-02, 
       2.0999612287382131e-04, 
      -2.4699927896750921e-03, 
      -5.5299795618302255e-03, 
       2.0000000000000002e-05, 
      -1.7000086969346941e-04, 
      -4.5000260910709724e-04, 
      -1.0000000000000001e-05, 
      -3.9999999991103643e-05, 
       1.0000000000000001e-05, 
       1.6999473534596232e-04, 
       6.9999999973310921e-05, 
       1.8599257100980592e-03, 
       9.2995978826339244e-04, 
      -1.0000000000000001e-05, 
       1.5969326708015971e-02, 
       1.6799286143175867e-02, 
       5.9999999973310922e-05, 
       6.9999999973310921e-05, 
       6.3497694317794879e-03, 
      -5.5897523059121990e-02, 
      -2.0998998852631422e-04, 
       7.9994994271163835e-04, 
       8.0422378129851813e-01, 
       8.0259386654890708e-01, 
      -8.2994606564773402e-04, 
      -2.0998998891443065e-04, 
       3.4648501148112923e-02, 
       1.0175558253112336e-01, 
      -1.9998998852631422e-04, 
       1.0000000000000001e-05, 
       3.1698667439869857e-03, 
       1.0159573555631754e-02, 
      -2.9999999991103641e-05, 
       2.6998947071861366e-04, 
       9.2995978826339244e-04, 
       2.0000000000000002e-05, 
       7.9999999973310934e-05, 
       2.9999999991103641e-05, 
       1.0000000000000001e-05, 
       1.6998773125160771e-04, 
       1.5998773125160771e-04, 
      -7.9993865625803856e-05, 
      -3.8996932735278636e-04, 
      -1.6998618030783180e-03, 
      -2.0398372655726369e-03, 
      -2.1998159687741152e-04, 
      -7.1993790588095014e-04, 
      -2.9999999991103641e-05, 
      -1.1999386560801113e-04, 
      -1.0000000000000001e-05, 
      -2.9999999991103641e-05, 
      -2.9999999991103641e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       3.9993865652492926e-05, 
       2.0000000000000002e-05, 
       3.0000000000000001e-05, 
       3.0995705956745045e-04, 
       3.3995092521994341e-04, 
       1.1798166059404291e-03, 
       6.1990410656803674e-04, 
       7.9987731304985852e-05, 
       9.9987731304985851e-05, 
       1.0000000000000001e-05, 
       5.9993865652492924e-05, 
       7.6990897351588415e-04, 
      -5.0000000000000002e-05, 
       9.1389312020534186e-03, 
       1.0418780220407221e-02, 
      -2.0000000000000002e-05, 
       7.9993865652492929e-05, 
       3.6465675799467634e-02, 
       1.6028102805463934e-02, 
      -2.0000000000000002e-05, 
      -3.0000000000000001e-05, 
       2.3997211838440607e-03, 
       2.5596989021940664e-03, 
       2.0997546260997170e-04, 
       3.0996406328258504e-04, 
       2.0000000000000002e-05, 
       3.0000000000000001e-05, 
       1.1998773130498585e-04, 
       2.0000000000000002e-05, 
       1.7798643668754098e-03, 
       2.9997720212145948e-04, 
       2.2848199762481741e-02, 
      -1.9198494900775132e-03, 
      -8.9993865652492929e-05, 
       2.3997384284746309e-04, 
       2.6862839641005326e-01, 
       3.1471470172070526e-01, 
      -4.4995769709744109e-04, 
       2.7297277783777758e-03, 
       1.0857809713365896e+00, 
       3.8349873471131041e-01, 
      -4.3995769670932472e-04, 
      -7.1992627457690827e-04, 
       6.9144455331716245e-02, 
       5.8475325330337197e-02, 
      -1.0999386565249293e-04, 
      -3.0000000000000001e-05, 
       5.5495668670651989e-03, 
       6.7694684349527516e-03, 
      -2.0000000000000002e-05, 
       4.5996580279407287e-04, 
       6.9994664896855414e-04, 
       4.0000000000000003e-05, 
       6.9999999999999994e-05, 
       2.0000000000000002e-05, 
       2.8999034057221757e-04, 
       6.0000000000000002e-05, 
       4.1598378381608701e-03, 
       7.9996679268691247e-04, 
      -4.0000000000000003e-05, 
       5.5017766679959598e-02, 
       9.3496194112699816e-03, 
       6.9999999999999994e-05, 
       3.2998084733883051e-04, 
       6.5022258581976156e-01, 
      -6.2077378785298051e-02, 
      -2.6698407598428351e-03, 
      -1.0000000000000001e-05, 
       1.0339366937177970e-02, 
       7.4345550490542482e+00, 
       8.8681426389130262e+00, 
      -1.1769285173540063e-02, 
       2.0000000000000002e-05, 
      -9.0000000000000006e-05, 
       8.8144468391549075e-02, 
       2.9576600341313640e+01, 
       7.9585541680948806e+00, 
      -9.4394217785200331e-03, 
       2.0000000000000002e-05, 
      -3.0000000000000001e-05, 
      -1.6688979285872985e-02, 
       1.7659831729967461e+00, 
       1.1346109855797795e+00, 
      -2.2398673134749387e-03, 
      -6.0996169351331178e-04, 
       1.2896461784123028e-01, 
       1.2386484200071173e-01, 
      -3.5998171631834161e-04, 
      -6.0000000000000002e-05, 
       9.8496044825930398e-03, 
       1.2109518389374237e-02, 
      -5.0000000000000002e-05, 
       7.5996679191067966e-04, 
       1.1099562619509204e-03, 
       6.0000000000000002e-05, 
       1.0000000000000000e-04, 
       6.9999999999999994e-05, 
       2.0000000000000002e-05, 
       9.2000094267119070e-04, 
       2.0000173951148782e-04, 
      -3.0000000000000001e-05, 
       1.1269992649539751e-02, 
       2.2999988784312333e-03, 
      -1.4000086975574390e-04, 
       1.3380970029605976e-01, 
       2.4959941165328410e-02, 
      -2.0000000000000002e-05, 
       2.2999612372122502e-04, 
       1.5156341378557210e+00, 
       2.5407898124276790e-01, 
       2.3199492720039636e-03, 
       2.0000000000000002e-05, 
      -5.0000000000000002e-05, 
       2.0449529722484837e-02, 
       1.5566265447597038e+01, 
      -1.6244211538357360e+00, 
      -6.5608497190723633e-02, 
       1.1999612294499217e-04, 
      -4.5998061472496085e-04, 
       3.2906196151824524e-01, 
       1.6657411527804530e+02, 
       1.9948375963301740e+02, 
      -2.4483408725828171e-01, 
       3.7998449177996868e-04, 
      -2.2099030728485714e-03, 
       2.1863131782292413e+00, 
       6.2365247461499462e+02, 
       1.1726069509568450e+02, 
      -1.4452645294360375e-01, 
       2.5998836883497645e-04, 
      -7.4996985331568121e-04, 
      -2.9115288606638323e-01, 
       3.3357202739468264e+01, 
       1.5121554269599795e+01, 
      -3.0379290732301105e-02, 
       6.9999999999999994e-05, 
       8.0000000000000007e-05, 
      -9.9497676668088494e-03, 
       2.1461691684088535e+00, 
       1.5197527792844909e+00, 
      -4.5099010788072465e-03, 
       1.0000000000000001e-05, 
      -9.2998108644867252e-04, 
       1.4641952065136907e-01, 
       1.3794956624262233e-01, 
      -5.5998710027096760e-04, 
      -8.0000000000000007e-05, 
       1.0269980800016502e-02, 
       1.1859978331534651e-02, 
      -6.0000000000000002e-05, 
       7.2999833379207529e-04, 
       9.9000007252733027e-04, 
       5.0000000000000002e-05, 
       8.0000000000000007e-05, 
       3.0000000000000001e-05, 
       2.7999033979598468e-04, 
       5.0000000000000002e-05, 
       2.4699042477415332e-03, 
       3.3999033940786832e-04, 
      -2.0000000000000002e-05, 
       1.8179263934687134e-02, 
       1.6199348465636217e-03, 
      -1.4999999999999999e-04, 
       7.9126693357347011e-02, 
       5.4287721682792306e-02, 
       1.6999612410934146e-04, 
      -3.4197933612597239e-03, 
      -7.9101527799402760e-01, 
      -1.3178223061528569e+00, 
       1.9198856006496453e-03, 
       5.0000000000000002e-05, 
      -5.4566578036660512e-02, 
      -1.2094151145054150e+01, 
      -1.2642738981658888e+00, 
       1.7698886052320943e-03, 
       2.0000000000000002e-05, 
       4.6497158765900206e-03, 
      -8.2271407173140498e-01, 
      -2.3700982134666665e-01, 
       5.1997170452771030e-04, 
       2.0998998859748510e-04, 
      -6.2827365794592221e-02, 
      -3.1418700872567075e-02, 
       1.0000000000000000e-04, 
       2.0000000000000002e-05, 
      -4.9597976700850172e-03, 
      -3.5498584216992809e-03, 
       2.0000000000000002e-05, 
      -3.9999034018410118e-04, 
      -3.6999033979598470e-04, 
      -3.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -6.9999999999999994e-05, 
      -1.0000000000000001e-05, 
      -1.0199238510900136e-03, 
      -3.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -1.2619005146508959e-02, 
      -5.0396058064244792e-03, 
       4.0000000000000003e-05, 
      -4.5995156197370114e-04, 
      -1.0534151720387072e-01, 
      -1.1329088105326769e-01, 
       1.1999386565249293e-04, 
      -1.0299013852121475e-03, 
      -1.3413898599894702e-01, 
      -1.4818830097282552e-02, 
       3.0000000000000001e-05, 
      -6.0000000000000002e-05, 
      -6.1695030832341060e-03, 
      -1.4498910410327252e-03, 
      -2.4998246671322269e-04, 
      -8.9993865652492929e-05, 
      -5.9993865652492924e-05, 
      -2.0000000000000002e-05, 
      -8.9987731304985851e-05, 
      -5.0000000000000002e-05, 
       3.6995705917933404e-04, 
       4.0000000000000003e-05, 
       5.9993865652492924e-05, 
      -3.9999999982207283e-05, 
      -2.9999999991103641e-05, 
      -5.9999999982207282e-05, 
      -1.0000000000000001e-05, 
      -1.3998773125160769e-04, 
      -1.5998773125160771e-04, 
      -1.0099203717200797e-03, 
      -6.2994791653392119e-04, 
      -3.9999999982207283e-05, 
      -1.3098902168805669e-03, 
       7.9999999973310934e-05, 
      -1.3998773125160769e-04, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -2.1999473533706595e-04, 
      -2.8198811476243570e-03, 
      -3.2898606068781813e-03, 
      -8.9999999964414566e-05, 
      -2.1039078743255359e-02, 
      -1.3559414140626997e-02, 
       2.0000000000000002e-05, 
      -7.5994994233241826e-04, 
      -2.5718834864441851e-02, 
      -4.1999642134587118e-04, 
       1.0000000000000001e-05, 
      -2.7498838014231117e-03, 
      -2.9998946876913518e-04, 
      -2.6998860058364974e-04, 
      -6.9999999973310921e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -1.6000086970236577e-04, 
      -2.9999999991103641e-05, 
      -2.8699906512129978e-03, 
       1.4999999994662184e-04, 
       1.0000000000000001e-05, 
      -7.9999999973310934e-05, 
      -3.9409787893257976e-02, 
      -4.8519742921784306e-02, 
       1.5999612289161404e-04, 
      -1.2799668451834829e-03, 
      -3.0516792782752111e-01, 
      -2.0592872345902902e-01, 
       3.7999224498030612e-04, 
      -1.0089736351726580e-02, 
      -3.5182706067923597e-01, 
      -2.8399990406986774e-02, 
       1.5999612289161404e-04, 
       2.0000000000000002e-05, 
      -3.6109761777764510e-02, 
      -6.5799909429395051e-03, 
       3.9999999982207283e-05, 
      -3.2199841616701742e-03, 
      -1.0799974609458692e-03, 
      -2.7000173902551143e-04, 
      -1.3999999995551818e-04, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -6.9999999973310921e-05, 
      -1.0000000000000001e-05, 
      -4.3998333554149565e-04, 
       3.9999999991103643e-05, 
      -1.5199383742239435e-03, 
      -2.6098864103467085e-03, 
       2.9999999991103641e-05, 
       1.3209424682345903e-02, 
       2.0829098772341424e-02, 
      -4.9999999982207283e-05, 
       3.0997997708821388e-04, 
       3.4358451715049691e-02, 
       1.3509405449297214e-02, 
      -2.9999999991103641e-05, 
       4.2598195706491041e-03, 
       2.5798946838079947e-03, 
       4.3998333631772846e-04, 
       3.4998947030380820e-04, 
       3.9999999982207283e-05, 
       3.9999999982207283e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -4.9999999982207283e-05, 
      -9.9993865616907488e-05, 
      -5.5995318231672256e-04, 
      -1.5998773125160771e-04, 
      -8.9993865625803855e-05, 
      -3.9999999982207283e-05, 
      -1.0000000000000001e-05, 
       1.0004394809330180e-05, 
       1.1399318490311418e-08, 
      -1.0000000000000001e-05, 
       3.0000000000000001e-05, 
       6.9987731304985840e-05, 
       6.9987731304985840e-05, 
       7.9987731304985852e-05, 
       3.9993865652492926e-05, 
       1.0000000000000001e-05, 
       1.2998773130498583e-04, 
       5.8993038346955289e-04, 
       1.6498030679051731e-03, 
       1.3398437510452233e-03, 
       1.0000000000000001e-05, 
       1.6098065925973320e-03, 
       7.9990723284004703e-04, 
       2.3997546222185527e-04, 
       1.6998159656936233e-04, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
      -3.0000000000000001e-05, 
      -6.0000000000000002e-05, 
       1.0999386565249293e-04, 
      -2.0000000000000002e-05, 
       2.8297768631428705e-03, 
       1.2299032690033901e-02, 
      -4.0000000000000003e-05, 
      -5.0000000000000002e-05, 
       3.4617218330952798e-02, 
       2.3798096152639951e-02, 
      -5.0000000000000002e-05, 
       2.8996770849995605e-04, 
       2.8987641108066016e-02, 
       1.2698980263147391e-02, 
      -3.0000000000000001e-05, 
      -4.0000000000000003e-05, 
       4.1696670785365698e-03, 
       2.5497996586927129e-03, 
       4.6996105520708822e-04, 
       3.6997720095711023e-04, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -9.0000000000000006e-05, 
      -5.7998119814921375e-04, 
       2.0000000000000002e-05, 
      -5.0000000000000002e-05, 
      -8.9996592176681920e-04, 
       1.9099257068574329e-03, 
      -1.1000000000000000e-04, 
      -3.2998084656259770e-04, 
       5.2507790992369532e-02, 
       2.1375103006616214e-01, 
      -5.9996169312519535e-04, 
      -6.4996082259321847e-04, 
       5.9577402963628567e-01, 
       3.3880542150545523e-01, 
      -7.4995781529395462e-04, 
      -1.0000000000000001e-05, 
       5.3396641218539775e-03, 
       4.1494151238801513e-01, 
       1.5790314258793667e-01, 
      -3.6997997641873730e-04, 
      -6.1996082336945142e-04, 
       5.6857563148343747e-02, 
       3.0088730759479598e-02, 
      -9.0000000000000006e-05, 
      -1.0000000000000000e-04, 
       6.1397443596772410e-03, 
       4.2098291328411036e-03, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       5.9997506341359012e-04, 
       4.9998032800535348e-04, 
       6.0000000000000002e-05, 
       5.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -1.5000173951148780e-04, 
      -1.0000000000000001e-05, 
      -1.1599979342075775e-03, 
      -6.9999999999999994e-05, 
      -6.7999853210948171e-03, 
       2.9000173951148782e-04, 
       3.0000000000000001e-05, 
      -6.5998797002671152e-04, 
      -7.7099706085507350e-03, 
       2.1739908960198003e-02, 
      -1.2699711932426712e-03, 
      -3.8999113677165805e-03, 
       6.3370660783864674e-01, 
       2.4138773743315882e+00, 
      -6.3198580285127315e-03, 
       1.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -5.5398652333261906e-03, 
       6.5795749896877664e+00, 
       2.9984814500368664e+00, 
      -6.6398459229071805e-03, 
       2.0000000000000002e-05, 
      -1.2999612294499217e-04, 
       6.2568390379347902e-02, 
       3.6744619049828264e+00, 
       1.1918723349900475e+00, 
      -2.8799332881790565e-03, 
      -5.5198622334011382e-03, 
       4.7337715948955844e-01, 
       2.1258889481278204e-01, 
      -6.7998709949473478e-04, 
      -8.9997934616095187e-04, 
       4.8339796415044024e-02, 
       2.8279898739832832e-02, 
      -1.2000000000000000e-04, 
      -1.0000000000000000e-04, 
       4.4899885821476871e-03, 
       3.2299911184994713e-03, 
      -2.0000000000000002e-05, 
       4.0000260849099890e-04, 
       3.4000260849099891e-04, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
       6.2997419288161340e-04, 
       1.9199257041406178e-03, 
       3.0000000000000001e-05, 
      -2.0299139827234637e-03, 
      -2.9568742278036846e-02, 
       9.0000000000000006e-05, 
      -1.4999999961188355e-04, 
      -1.7190245161696016e-01, 
      -5.0967800333467587e-02, 
       1.2000000000000000e-04, 
      -3.0398120371516683e-03, 
      -1.1307496074008443e-01, 
      -2.5488897262257004e-02, 
       6.9999999999999994e-05, 
       1.1000000000000000e-04, 
      -1.6919273478946548e-02, 
      -5.5797675548115806e-03, 
       2.0000000000000002e-05, 
       3.0000000000000001e-05, 
      -1.9599178337746711e-03, 
      -8.7996505123484245e-04, 
      -1.9999473502012039e-04, 
      -1.2000000000000000e-04, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -3.0000000000000001e-05, 
      -1.9998860067261333e-04, 
       1.0000000000000001e-05, 
      -1.1399098413310458e-03, 
      -2.7697829955497955e-03, 
       5.0000000000000002e-05, 
      -4.3796530687776024e-03, 
      -1.7898600146036424e-03, 
      -5.5995492085958117e-04, 
      -2.4998246593698983e-04, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       7.9987731304985852e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
      -3.9999999982207283e-05, 
      -3.9999999982207283e-05, 
      -9.9993865616907488e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -3.9999999991103643e-05, 
      -1.3999999995551818e-04, 
      -5.6997945844200601e-04, 
      -6.0997332330047329e-04, 
      -1.0000000000000001e-05, 
      -1.2699419001605187e-03, 
      -2.8998859979852051e-04, 
      -4.9999999982207283e-05, 
      -3.1998773003388028e-04, 
      -5.9999999982207282e-05, 
      -1.0000000000000001e-05, 
      -3.3000173900771869e-04, 
      -1.2399944547575763e-03, 
      -5.0499738678599160e-03, 
      -5.1899747344573508e-03, 
       2.0000000000000002e-05, 
      -1.0999999996441456e-04, 
      -1.0489932195065683e-02, 
      -2.4499884335310567e-03, 
       1.0000000000000001e-05, 
      -4.0998836830451830e-04, 
      -2.5399828177052092e-03, 
      -2.7000260723768606e-04, 
      -2.9999999991103641e-05, 
      -4.4999786114089258e-04, 
      -5.9999999594090863e-05, 
      -5.9999999982207282e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -4.9999999982207283e-05, 
       1.3999999994662184e-04, 
       4.0998947028601546e-04, 
       9.6995717742590244e-04, 
       5.2997858830704193e-04, 
       3.0000000000000001e-05, 
       5.0997771894831093e-04, 
       2.2999386518430926e-04, 
       9.9999999964414565e-05, 
       4.9999999982207283e-05, 
       1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.9993865652492924e-05, 
       9.9987731304985851e-05, 
       5.9993865652492924e-05, 
       5.0000000000000002e-05, 
       3.0000000000000001e-05, 
       1.0000000000000001e-05, 
       6.9999999999999994e-05, 
       3.2997720134522664e-04, 
       8.7993350974156326e-04, 
       8.6993437910919070e-04, 
       1.0000000000000001e-05, 
       1.3698928250490470e-03, 
       7.7993877394520999e-04, 
       6.6994791598009727e-04, 
       3.1997546144562245e-04, 
       1.2998773130498583e-04, 
       6.9999999999999994e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       9.0000000000000006e-05, 
       8.8996592099058625e-04, 
       4.1498317428845686e-03, 
      -2.0000000000000002e-05, 
       1.0909542985422739e-02, 
       9.3896119613182637e-03, 
      -3.0000000000000001e-05, 
       1.3999999999999999e-04, 
       1.4859362638795979e-02, 
       7.5696827727210420e-03, 
      -3.0000000000000001e-05, 
       1.0000000000000000e-04, 
       6.5197169005045694e-03, 
       2.8798802722492336e-03, 
      -1.0000000000000001e-05, 
       1.2199489039323547e-03, 
       6.2997419249349700e-04, 
       1.6999386526437649e-04, 
       1.0000000000000000e-04, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       1.4000086975574390e-04, 
       7.1999659272812178e-04, 
      -2.0000000000000002e-05, 
       7.0199748793684947e-03, 
       3.1179885019795069e-02, 
      -1.3999999999999999e-04, 
       6.9999999999999994e-05, 
       8.0959591305152970e-02, 
       5.9629735945832305e-02, 
      -2.0999612255687575e-04, 
       1.0699745985739673e-03, 
       9.4029434590640532e-02, 
       4.2169792378650837e-02, 
      -1.3999999961188356e-04, 
       5.9998449022750293e-04, 
       3.6739771632229627e-02, 
       1.4649926777976373e-02, 
      -5.0000000000000002e-05, 
      -5.0000000000000002e-05, 
       6.5399648380029011e-03, 
       3.0499880347917134e-03, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       8.7999572258426148e-04, 
       4.7999873065975826e-04, 
       1.0000000000000000e-04, 
       6.0000000000000002e-05, 
       1.0000000000000001e-05, 
       3.0000000000000001e-05, 
      -3.0000000000000001e-05, 
      -5.3998032916970269e-04, 
      -2.5598925471541109e-03, 
      -1.3699445106488011e-03, 
      -6.9999999999999994e-05, 
      -3.4998471129340284e-03, 
      -1.1599527817635954e-03, 
      -4.0000000000000003e-05, 
      -1.5299357650934802e-03, 
      -4.6998333491650088e-04, 
      -3.0998773052875295e-04, 
      -1.1000000000000000e-04, 
      -5.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -5.0000000000000002e-05, 
      -8.9993865652492929e-05, 
      -6.0000000000000002e-05, 
      -5.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -3.9999999982207283e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -3.0000000000000001e-05, 
      -8.9999999973310933e-05, 
      -1.0999999996441456e-04, 
      -2.3999999914369992e-04, 
      -9.9999999964414565e-05, 
      -1.0000000000000001e-05, 
      -1.2999999956740178e-04, 
      -2.9999999991103641e-05, 
      -2.9999999991103641e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       4.0000000000000003e-05, 
       3.0000000000000001e-05, 
       4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       6.9999999999999994e-05, 
       1.8999473502012039e-04, 
       2.1999473463200399e-04, 
       3.7998946965212438e-04, 
       2.4999473463200401e-04, 
       2.8998773014063655e-04, 
       1.4999999922376715e-04, 
       1.0999999961188359e-04, 
       5.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       9.0000000000000006e-05, 
       4.2000260810288250e-04, 
       1.0699974613195163e-03, 
       1.0799974609313999e-03, 
       2.0000000000000002e-05, 
       1.8899914440041736e-03, 
       1.1099965903994232e-03, 
       2.0000000000000002e-05, 
       1.2999918432005551e-03, 
       5.9999872988352528e-04, 
       4.5000086820327825e-04, 
       1.9999999922376717e-04, 
       9.0000000000000006e-05, 
       4.0000000000000003e-05, 
       1.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -3.0000000000000001e-05, 
      -8.0000000000000007e-05, 
      -4.0000000000000003e-05, 
      -6.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       3.8500052903963596e+05, 
      -3.1483019999591910e+00, 
      -2.9999999997311145e-05, 
       3.7998575836933668e-04, 
       7.9660568469778482e+01, 
      -2.0905355137759419e+04, 
      -1.0325754056746472e-01, 
      -8.3697361852690996e-03, 
      -4.4211770490387590e+00, 
      -5.6992512638489313e+02, 
      -1.0199305524721176e-03, 
       3.2999010831688688e-04, 
      -1.1100043486792319e-03, 
      -2.3210430497874526e+01, 
       2.6000521851116001e-04, 
       1.9999999998207433e-05, 
       2.7000173948728815e-04, 
      -1.1169371044179983e+00, 
       3.9999999996414865e-05, 
       1.9999999998207433e-05, 
      -5.8880465319867410e-02, 
      -3.2900313109119031e-03, 
      -1.9000173949445842e-04, 
      -1.0999999999014087e-04, 
      -1.7399525712965289e-03, 
      -2.6739187212881998e-02, 
      -4.2240656115839154e-01, 
       1.4999999998655572e-04, 
      -1.8999999998297061e-04, 
      -7.0026960617373870e+00, 
      -5.0197444216470780e-02, 
      -6.9999999993726000e-05, 
      -7.6895894726563933e-03, 
      -1.2962022424070292e+02, 
       3.3463243448180796e-01, 
       2.9999999997311145e-05, 
      -1.8566991736964211e-01, 
       4.8888328449310244e+01, 
      -1.5802146652025906e-01, 
       2.9999999997311145e-05, 
      -2.4809697371058767e-01, 
       1.0475529435395759e+02, 
      -6.8196430863427648e-03, 
       4.9999999995518581e-05, 
       3.2648339848703190e-02, 
       5.7508578538436126e+00, 
      -2.9999999997311143e-04, 
      -7.9995729744124049e-04, 
       3.5507869597491121e-01, 
      -5.9999999994622291e-05, 
       2.3039299558702490e-02, 
       1.5299569661513701e-03, 
       9.9999999991037163e-05, 
      -4.9999999995518581e-05, 
      -6.5995527316327525e-04, 
      -9.3793572985260542e-03, 
      -1.3617046100022664e-01, 
      -9.8991435737977806e-04, 
      -3.0997546258218684e-04, 
      -2.1171282632583841e+00, 
       4.4995981571196867e-03, 
      -5.5794920664080727e-03, 
       1.0656722797003297e+00, 
      -3.4196903329037475e-03, 
      -2.3697890694752716e-03, 
       1.1655363838529489e+00, 
      -1.0999999999014087e-04, 
       1.7998773128885274e-04, 
       5.0716438193917468e-02, 
       2.2798456792376266e-03, 
       8.9999999991933447e-05, 
      -2.0997546259114971e-04, 
      -2.8696958416961800e-03, 
      -1.9999999998207433e-05, 
      -3.8935785427179993e-02, 
       6.9999999993726000e-05, 
      -1.6998159694224193e-04, 
       2.1267663634227500e-02, 
      -6.9999999993726000e-05, 
      -2.9999999997311145e-05, 
       1.9117926343405874e-02, 
       7.7991672833222241e-04, 
       2.9999999997311145e-05, 
      -5.9999999994622291e-05, 
      -7.2989571525071807e-04, 
       3.3995092518946974e-04, 
       3.6994479083927371e-04, 
      -5.9999999976829571e-05, 
       3.4996319376793487e-04, 
      -1.6098221032175221e-03, 
       3.9999999996414865e-05, 
       2.8997546248611943e-04, 
       8.4393952091937965e-03, 
      -2.3998159685590067e-04, 
      -2.9477843313093360e-02, 
      -2.9999999997311145e-05, 
      -1.9999999998207433e-05, 
       1.5898860497606509e-03, 
       3.5997546128880718e-04, 
       3.9999999996414865e-05, 
       9.9999999955451723e-05, 
       6.4997894135227971e-04, 
       1.9999999998207433e-05, 
      -4.2198518026374575e-03, 
      -7.9999999966140657e-05, 
      -1.0999999995455543e-04, 
       1.1410603726168141e-01, 
       2.0999999991000714e-04, 
      -1.3659260469050206e-02, 
       4.9755215819351722e-01, 
      -7.3996382995627932e-04, 
       4.3997384265678845e-04, 
       1.0997620924828107e-01, 
      -6.9999999967036927e-05, 
      -2.9999999997311145e-05, 
       1.1209625503392669e-02, 
       9.6996754071621005e-04, 
       7.9999999966140657e-05, 
      -2.9999999997311145e-05, 
      -4.3000347883319725e-04, 
      -6.3600399990106872e-03, 
      -1.8999999992069610e-04, 
      -9.9380484219865714e-02, 
      -4.7999485494516363e-04, 
      -2.7799621701349053e-03, 
      -1.7385258483230053e+00, 
       4.5699363718342441e-03, 
      -4.2319341920191239e-02, 
      -8.3791067416664688e+00, 
      -2.4709621256653429e-02, 
      -7.9562635561038542e-01, 
       1.0874270141177936e+02, 
      -1.8739689483299998e-02, 
       1.6999999993138503e-04, 
       6.0549063918332549e-02, 
       6.3220032203434355e+00, 
      -1.6399763369809811e-03, 
       2.0299711939103788e-03, 
       3.7852142052101295e-01, 
      -1.2999999994386648e-04, 
       1.0999999995455543e-04, 
       2.3680120794187423e-02, 
       1.5200095654877134e-03, 
       9.9999999955451723e-05, 
       5.9999999976829571e-05, 
       1.0099692817676822e-03, 
       1.7289444638976460e-02, 
       1.3999999994297019e-04, 
       7.9999999966140657e-05, 
       3.3224889132530938e-01, 
      -1.4799219996174619e-03, 
       1.5599202600721149e-03, 
       8.5124035002281906e-01, 
       3.3098243750911368e-03, 
       3.3788196608704767e-02, 
      -1.6674723862047578e+01, 
      -5.9999999976829571e-05, 
      -5.6197040706796285e-03, 
      -9.3331756600538507e-01, 
      -3.6998773037104138e-04, 
      -5.9038043138950086e-02, 
      -1.9999999998207433e-05, 
      -3.9198780429301125e-03, 
      -2.7000173939832454e-04, 
       1.9999999998207433e-05, 
       2.7998159683452278e-04, 
       3.2397711906711835e-03, 
       4.3796879646369998e-03, 
      -1.9999999998207433e-05, 
       3.7287275626590614e-02, 
       1.9999999998207433e-05, 
       6.4595389545972423e-03, 
       6.6995353419581881e-04, 
       5.9999999976829571e-05, 
       5.9999999976829571e-05, 
       7.9999999966140657e-05, 
       5.3993865628970640e-04, 
       6.9999999967036927e-05, 
      -1.9999999998207433e-05, 
      -1.1997546259921626e-04, 
      -5.7989571487604592e-04, 
      -3.9999999996414865e-05, 
       3.5994479084017005e-04, 
      -4.2593812511134803e-03, 
       2.9999999997311145e-05, 
      -1.5777681189612303e-02, 
      -1.1298361100869345e-03, 
      -8.9987731296919292e-05, 
       1.9999999998207433e-05, 
       4.5995266469020209e-04, 
       1.1158808779392805e-02, 
       2.9999999997311145e-05, 
       2.5996932823916120e-04, 
      -1.4806394212356289e-01, 
       6.9999999993726000e-05, 
       1.0398640608647635e-03, 
      -4.1071486233056914e-01, 
       2.6996932823826497e-04, 
      -2.8546907871265171e-02, 
       2.9999999997311145e-05, 
      -2.0897738487721945e-03, 
      -1.5998159694313822e-04, 
       2.9999999997311145e-05, 
       6.6995527393861188e-04, 
       1.9999999998207433e-05, 
       1.5168961862541621e-02, 
       4.0996932822571694e-04, 
       3.4301592316629376e-01, 
       9.8991435854412727e-04, 
       9.5691426166157386e-03, 
      -4.9501349067196063e+00, 
       1.6798497739297302e-03, 
       2.8387412335730777e-02, 
      -9.8844667465718103e+00, 
      -2.9999999997311145e-05, 
       7.9192898353840548e-03, 
      -6.5753296501894665e-01, 
       8.3992575672060896e-04, 
      -4.6076772975214793e-02, 
       5.9999999994622291e-05, 
      -3.3497711457249660e-03, 
      -2.4998333644655948e-04, 
      -1.9999999998207433e-05, 
       5.9999999994622291e-05, 
       1.1899666267739756e-03, 
       3.9999999996414865e-05, 
       2.3339289036866290e-02, 
       7.8995729899460239e-04, 
       4.9504418153977070e-01, 
      -4.5997558231772182e-04, 
       1.5179225093487955e-02, 
       1.0056203332336867e+01, 
       3.0238458060069424e-02, 
       3.2334304362267785e-01, 
      -1.5213781181555626e+02, 
       3.7338054588554326e-02, 
      -3.4997610010859231e-04, 
       6.5706430206093058e-01, 
      -2.0458611789480557e+02, 
       1.0799620769067898e-03, 
      -8.4993831174621766e-04, 
       2.0940899652925385e-01, 
      -1.2831403548880711e+01, 
       1.7395114878210153e-09, 
       1.2999999998834828e-04, 
       1.8039080838079844e-02, 
      -8.4880184799015856e-01, 
       1.2699337499019261e-03, 
      -5.8498148654635040e-02, 
       8.9999999991933447e-05, 
      -4.1298735975484717e-03, 
      -3.0000260924034318e-04, 
      -1.9999999998207433e-05, 
       2.0000260924930605e-04, 
       3.0400286993383803e-03, 
       1.1999999998924458e-04, 
       4.7400375694138144e-02, 
      -1.9999999998207433e-05, 
       1.8799811588075014e-03, 
       7.7854495144615832e-01, 
       4.7999572409370545e-04, 
       2.9099632514815468e-02, 
       1.4402688991954181e+01, 
      -2.0079749726991641e-02, 
      -1.7999999998386689e-04, 
      -1.9999999998207433e-05, 
       4.7262330622706727e-01, 
       2.4615847968480679e+02, 
       7.7403904019864478e-01, 
      -1.3999999998745200e-04, 
      -3.8998449174501354e-04, 
       8.7515625232254681e+00, 
      -3.6991109330180198e+03, 
       5.9632093121812468e-01, 
      -7.4497489299349016e-03, 
       1.0321107088258564e+01, 
      -2.9559675626057124e+03, 
       3.1429705662064322e-02, 
      -1.8659381227811495e-02, 
       4.1311145454994769e+00, 
      -1.7073307906648640e+02, 
       1.8099968202389119e-03, 
       2.1899294167495083e-03, 
       2.8398596260369768e-01, 
      -1.0444757788459562e+01, 
       1.4000260925468372e-04, 
       1.9999999998207433e-05, 
       1.8109768178310625e-02, 
      -6.6968351866645881e-01, 
       1.1899884424199983e-03, 
      -4.4220292185840150e-02, 
       7.9999999992829730e-05, 
      -2.9800243497888055e-03, 
      -2.0000173949356214e-04, 
       8.9999999991933447e-05, 
       1.1599666236717329e-03, 
       2.9999999997311145e-05, 
       1.4169568122116911e-02, 
       2.0999999920494518e-04, 
       1.6857465048617706e-01, 
       2.9998773127809725e-04, 
       5.1997383232171292e-04, 
       1.4367547210982884e-01, 
      -2.0298945871754519e-03, 
      -4.7357510174271755e-02, 
       2.4208498493055888e+01, 
      -5.8796966665792842e-03, 
       1.1999999998924458e-04, 
      -1.3571262386357466e-01, 
       3.0823859907219550e+01, 
      -1.4299250058847307e-03, 
       3.9997222304910313e-04, 
      -5.5267093531528186e-02, 
       2.6164091717727307e+00, 
      -1.4999999998655572e-04, 
      -4.9999999995518581e-05, 
      -5.3697280450288086e-03, 
       2.1251296661321492e-01, 
      -4.3997558231951439e-04, 
       1.7039460374453742e-02, 
      -3.9999999996414865e-05, 
       1.3499596203276855e-03, 
       1.0999999999014087e-04, 
      -4.0997193749294869e-04, 
      -2.9999999997311145e-05, 
      -1.0359283640210833e-02, 
      -5.4995104491962707e-04, 
      -1.0887233898445145e-01, 
      -4.5995543976371276e-04, 
      -1.0719047352460249e-02, 
       2.3536309843524257e+00, 
      -3.9999999996414865e-05, 
       7.9992662647993809e-04, 
       1.4762959370247397e-01, 
       1.8698379005329033e-03, 
       9.1193713528487444e-03, 
       9.9999999991037163e-05, 
       3.5997807184493723e-04, 
      -9.9999999991037163e-05, 
      -5.1994652956108452e-04, 
      -2.9999999997311145e-05, 
       2.0797756327894119e-03, 
      -2.3997546181222802e-04, 
       1.9999999998207433e-05, 
       2.9999999997311145e-05, 
      -2.9999999997311145e-05, 
       7.9999999966140657e-05, 
       6.4992638638489825e-04, 
       2.9999999997311145e-05, 
      -5.4993865628881011e-04, 
      -3.9999999996414865e-05, 
      -1.5998773123726716e-04, 
       2.2398413285241595e-03, 
      -4.9999999977725862e-05, 
       1.3889004284567789e-02, 
       6.9993929375197815e-04, 
      -1.0479229714051836e-02, 
       1.9999999998207433e-05, 
      -6.6995092492858706e-04, 
      -2.9999999997311145e-05, 
      -1.1999999994476279e-04, 
      -3.3398929652567809e-03, 
      -1.3999999994297019e-04, 
       5.2928223215939367e-02, 
      -5.9999999976829571e-05, 
      -8.8995381966018767e-04, 
       2.5560112901794679e-01, 
       1.4409224300161040e-02, 
      -1.5707413356650621e-01, 
       4.8997384263451435e-04, 
      -8.8696765680077155e-03, 
       2.9999999997311145e-05, 
      -2.9999999986635514e-04, 
      -2.9999999997311145e-05, 
      -1.7000104363293989e-03, 
      -4.9999999995518581e-05, 
      -4.8430212033192685e-02, 
      -1.2999999994386648e-04, 
      -2.0599711938987936e-03, 
       8.6243278904298004e-01, 
      -1.0099879692011123e-03, 
      -8.6798609676501642e-03, 
       3.2582370807619241e+00, 
      -5.9999999976829571e-05, 
      -1.7999999992159238e-04, 
       2.0098656931416145e-01, 
      -1.4189284087900234e+00, 
      -1.4999999993317757e-04, 
       4.9999999977725862e-05, 
       6.7098941217191028e-03, 
      -6.5449921583248111e-02, 
      -1.9999999998207433e-05, 
       3.4999999984408099e-04, 
      -1.1099973984011820e-03, 
       1.5999999876793204e-04, 
       2.9999999997311145e-05, 
      -4.9999999995518581e-05, 
      -6.4997980994367438e-04, 
      -6.7997806585012458e-03, 
      -1.9999999998207433e-05, 
      -8.9999999956348007e-05, 
       3.8098714295570907e-02, 
       6.9999999967036927e-05, 
      -2.2999999989931820e-04, 
      -2.1258262555232793e-01, 
       5.9999999976829571e-05, 
      -8.0795659691643273e-03, 
      -1.0765620290507359e-01, 
      -1.7999999992159238e-04, 
      -1.4479504603693322e-02, 
      -1.4999508765716298e-03, 
      -1.3999999993407385e-04, 
      -6.9999999967036927e-05, 
       3.1997546053395219e-04, 
       9.8992812884629469e-04, 
       8.9987731261333853e-05, 
       1.6198834451392309e-03, 
       2.7998159683452278e-04, 
       2.9999999997311145e-05, 
      -2.6080406079458980e-09, 
      -3.9999999996414865e-05, 
       9.9987731260437569e-05, 
       1.9980272777955583e-05, 
      -3.9999999996414865e-05, 
      -1.9999999998207433e-05, 
      -2.1996932824274633e-04, 
      -7.7988344538687309e-04, 
      -4.4993252097274005e-04, 
      -5.9999999994622291e-05, 
      -1.0998773129512671e-04, 
      -5.1294521563901270e-03, 
      -5.9999999994622291e-05, 
      -1.5708297041256869e-02, 
      -2.1997546259025342e-04, 
      -8.1691051872718589e-03, 
      -1.1398737847624751e-03, 
      -1.2998773129333413e-04, 
      -1.9999999998207433e-05, 
      -1.1999999998924458e-04, 
      -2.9999999997311145e-05, 
      -2.3898334058753573e-03, 
       3.9999999996414865e-05, 
      -3.5997546257770548e-04, 
      -1.0833240876766437e-01, 
       1.0999999999014087e-04, 
      -1.4498750613471372e-03, 
      -2.7904006579942225e-01, 
       4.9999999995518581e-05, 
      -4.0296357569822210e-03, 
      -1.2805075092513285e-01, 
       2.2998159693686420e-04, 
      -1.7278775418016080e-02, 
       4.9999999995518581e-05, 
      -1.8398719975784819e-03, 
      -1.7998773128885274e-04, 
      -3.9999999996414865e-05, 
      -4.0999121029121376e-04, 
      -4.9999999995518581e-05, 
      -2.2999289024477035e-03, 
      -5.8996643988306631e-04, 
      -4.3158624131595380e-02, 
       7.6996117527516992e-04, 
      -6.6096630553469891e-03, 
      -1.8970367926847447e+00, 
       1.5999167328141174e-03, 
      -5.9999999994622291e-05, 
      -2.6978590582815640e-02, 
      -3.9579827434849597e+00, 
       6.8996643948598696e-04, 
       2.9999999997311145e-05, 
      -5.7886881100952171e-02, 
      -1.5713943841003077e+00, 
       7.9999999992829730e-05, 
       4.9999999995518581e-05, 
       2.7198614091688139e-03, 
      -2.0285309945865274e-01, 
       5.4997558153342262e-04, 
      -2.0789319306507164e-02, 
       5.9999999994622291e-05, 
      -1.9399368200675657e-03, 
      -1.6999999998476318e-04, 
      -4.9999999995518581e-05, 
      -5.5000521770893508e-04, 
      -5.9999999994622291e-05, 
      -5.4200426093952208e-03, 
      -7.1999271638481434e-04, 
      -2.9390186091328888e-02, 
      -8.9999999991933447e-05, 
      -7.4399077060242633e-03, 
      -5.1423243497062621e-01, 
       9.2498845164474333e-03, 
      -1.3999999998745200e-04, 
      -7.9818880501352676e-02, 
      -2.1636343875076179e+01, 
       1.4829797144853687e-02, 
      -7.9997285976700968e-04, 
      -3.2175506886962035e-01, 
      -3.4782523745558798e+01, 
       5.4999213282995590e-03, 
       3.1998836880629540e-04, 
      -5.0792130896835275e-01, 
      -1.1649947819443046e+01, 
       6.6999271677741217e-04, 
       4.9998449173515452e-04, 
       1.9379704486947858e-02, 
      -1.4225541276068021e+00, 
       6.9999999993726000e-05, 
       3.9999449922891294e-03, 
      -1.3922061869363414e-01, 
       4.3999485511777957e-04, 
      -1.2430067812277675e-02, 
       3.9999999996414865e-05, 
      -1.0600078253779905e-03, 
      -8.9999999991933447e-05, 
      -3.9999999996414865e-05, 
       9.2996928018122837e-04, 
      -2.3989247389315162e-02, 
      -6.9999999993726000e-05, 
       3.5998773321330174e-04, 
       2.3695196810801203e-01, 
      -2.1999999998028173e-04, 
       1.5509193325354421e-02, 
       5.7878980681430592e-01, 
      -1.1999999998924458e-04, 
       1.9688938804572637e-02, 
       2.4452136289194804e-01, 
      -2.9999999997311145e-05, 
      -2.9999999997311145e-05, 
       1.2999999882399901e-04, 
       3.6818751174001671e-02, 
      -7.9999999992829730e-05, 
       4.2998578912151475e-03, 
       4.4998420502002841e-04, 
       3.9999999996414865e-05, 
      -2.9999999997311145e-05, 
       1.9999999998207433e-05, 
       2.5198281393313594e-03, 
       2.0998159693865677e-04, 
       2.6578133580483304e-02, 
      -6.3993929244509197e-04, 
       1.8678664992649377e-02, 
       4.9999999995518581e-05, 
       1.3599096738504068e-03, 
       2.9999999997311145e-05, 
      -1.4998159694403447e-04, 
      -1.9997546259204602e-04, 
      -2.9999999997311145e-05, 
       2.9999999997311145e-05, 
       8.9999999965244374e-05, 
       4.3996932690744069e-04, 
       1.9999999998207433e-05, 
      -5.9999999976829571e-05, 
      -1.5998773123726716e-04, 
      -1.9999999998207433e-05, 
       2.9999999997311145e-05, 
       1.2799570106219098e-03, 
       2.9999999997311145e-05, 
       5.7298105518939404e-03, 
       3.3998773115885951e-04, 
      -5.5997372556938687e-04, 
       2.0999999991000714e-04, 
      -1.5299447878728246e-03, 
       2.9999999997311145e-05, 
      -1.8999999992069610e-04, 
       2.6000173939922077e-04, 
       3.9999999996414865e-05, 
       1.1490054770587659e-02, 
      -1.9999999998207433e-05, 
       2.9999999986635514e-04, 
       4.5910184261982304e-02, 
       2.8099578190159021e-03, 
      -2.4799485949198968e-03, 
       1.4199767343365658e-03, 
      -8.1399938984571592e-03, 
       1.9999999991090345e-04, 
      -8.8999999960885808e-04, 
      -5.9999999976829571e-05, 
       2.9999999997311145e-05, 
       3.7998946909650378e-04, 
      -2.5599166785089333e-03, 
      -1.4999999993317757e-04, 
      -3.1098903978782504e-03, 
      -8.9999999965244374e-05, 
      -1.1099596656358362e-03, 
      -1.9999999991090345e-04, 
      -2.9999999997311145e-05, 
       6.9999999967036927e-05, 
       2.9999999997311145e-05, 
      -2.9999999997311145e-05, 
      -2.9999999997311145e-05, 
      -1.1998773129423043e-04, 
      -4.3995092401615758e-04, 
      -4.2995092362893745e-04, 
      -1.6998159694224193e-04, 
      -2.9999999997311145e-05, 
      -6.9999999993726000e-05, 
      -1.9498684724883475e-03, 
      -2.9999999997311145e-05, 
      -6.1795669147913027e-03, 
      -1.0999999999014087e-04, 
      -5.4096134264304419e-03, 
      -4.9999999995518581e-05, 
      -1.9398623822584546e-03, 
      -3.4997546141425244e-04, 
      -4.9999999995518581e-05, 
      -8.4997454439204531e-04, 
      -2.9999999997311145e-05, 
      -2.4359226139441411e-02, 
       3.9999999996414865e-05, 
      -4.0998773126823813e-04, 
      -6.6777799989498579e-02, 
       3.9999999996414865e-05, 
      -1.2099394109786337e-03, 
      -5.1908239459320625e-02, 
      -4.8997384125107942e-04, 
      -1.7089416902058167e-02, 
      -2.9598991387017283e-03, 
      -3.8998946922905284e-04, 
      -4.9999999995518581e-05, 
      -8.9999999991933447e-05, 
      -6.5900417302198174e-03, 
       1.1999999998924458e-04, 
      -2.6999999997580034e-04, 
      -1.8316094826847287e-01, 
       2.7999999997490400e-04, 
      -3.2599561529611147e-03, 
      -4.2250165827248998e-01, 
       1.9999999998207433e-04, 
      -7.8498755921186901e-03, 
      -2.8663087781865987e-01, 
       5.9999999994622291e-05, 
      -3.9999999996414865e-05, 
      -2.6999534677236972e-03, 
      -8.5870257737473116e-02, 
      -1.4200047788719221e-02, 
       3.9999999996414865e-05, 
      -1.8200086911844525e-03, 
      -1.9999999998207433e-04, 
      -1.9999999998207433e-05, 
      -2.3999999842602346e-04, 
       2.7799087645080695e-03, 
       1.8999999998297061e-04, 
       9.4796825087208689e-03, 
       4.1998773010299258e-04, 
       7.7997308005767260e-03, 
       1.5999999998565946e-04, 
       2.7199079303358601e-03, 
       5.1998333486989409e-04, 
       7.9999999992829730e-05, 
       3.9999999996414865e-05, 
       3.3997720092663659e-04, 
       4.3996932667056242e-04, 
       1.3999999998745200e-04, 
       1.9999999998207433e-05, 
       1.0999999995455543e-04, 
       5.9999999976829571e-05, 
      -1.9999999998207433e-05, 
       1.4999999993317757e-04, 
       6.6000347679193334e-04, 
       3.9999999996414865e-05, 
       3.1999999830320053e-04, 
       3.9999999996414865e-05, 
      -7.9999999966140657e-05, 
      -6.9999999967036927e-05, 
      -2.9999999997311145e-05, 
      -6.9999999967036927e-05, 
      -3.9999999996414865e-05, 
      -2.9999999997311145e-05, 
      -1.2999999998834828e-04, 
      -1.5998773129064531e-04, 
      -9.9999999991037163e-05, 
      -2.9999999997311145e-05, 
      -3.2998946962254695e-04, 
      -1.0899622741230546e-03, 
      -1.9999999998207433e-05, 
      -1.2599561389840779e-03, 
      -1.9999999998207433e-05, 
      -6.9997806870953207e-04, 
      -2.1999999920404889e-04, 
      -3.9999999996414865e-05, 
      -8.9999999991933447e-05, 
      -1.8400113012261245e-03, 
      -3.9999999996414865e-05, 
      -5.3900269440916079e-03, 
      -1.1999999998924458e-04, 
      -5.5900217228224049e-03, 
      -8.9999999991933447e-05, 
      -2.8500095558023656e-03, 
      -1.9999999998207433e-05, 
      -8.3000260569979234e-04, 
      -1.5999999998565946e-04, 
      -1.9999999998207433e-05, 
       3.9999999996414865e-05, 
       1.4999999998655572e-04, 
       1.8999999998297061e-04, 
       1.0999999999014087e-04, 
       3.9999999996414865e-05, 
      -2.9999999997311145e-05, 
      -1.9999999998207433e-05, 
      -1.9999999998207433e-05, 
      -6.9999999993726000e-05, 
      -9.9999999991037163e-05, 
      -6.9999999993726000e-05, 
      -2.9999999997311145e-05, 
    } ,  
    {  
      -4.1159574817515249e+02, 
       4.2032548138143505e-01, 
      -5.8996904561608857e-04, 
       1.5999226140402215e-04, 
      -8.0187332604691075e-02, 
       3.9533301970600597e+01, 
       2.2639585659999782e+04, 
      -4.5099610307254984e+01, 
       9.0916999939499463e-02, 
      -1.8999226140402215e-04, 
       6.7997937456124263e-04, 
      -1.3725701099805798e+00, 
       7.6902570369870182e+02, 
      -3.9976134296908530e+00, 
       1.2969592654144399e-02, 
      -4.0000000000000003e-05, 
       9.0000000000000006e-05, 
      -5.6839245216214383e-02, 
       3.6123813073253892e+01, 
      -3.2865586724190660e-01, 
       1.5299527452105020e-03, 
      -3.5399566985949336e-03, 
       1.9336823984636129e+00, 
      -2.6019715403432151e-02, 
       1.5999699512480448e-04, 
      -2.2999785954759791e-04, 
       1.1100088855380499e-01, 
      -2.0099807359283814e-03, 
       2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       6.6500631036979776e-03, 
      -1.5000172884558683e-04, 
       4.1000432211396706e-04, 
      -1.0000000000000001e-05, 
       3.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -2.3999645795124582e-04, 
       6.0000000000000002e-05, 
      -3.3199032234219098e-03, 
       6.3996817253443345e-04, 
      -4.7018577037686374e-02, 
       6.9999999999999994e-05, 
      -3.0000000000000001e-05, 
       6.2296940374589041e-03, 
      -6.6937870149957934e-01, 
       1.0599428919928163e-03, 
      -1.6998612608688771e-04, 
       5.1647381444881534e-02, 
      -9.6790565471940120e+00, 
       2.4288759392986029e-02, 
      -4.0000000000000003e-05, 
      -6.2995310694929300e-04, 
       3.0155423650436874e-01, 
      -1.4732138184362128e+02, 
      -8.0485758387155673e-02, 
       1.2999613070201104e-04, 
      -2.2998612650391686e-04, 
       7.5006161138150437e-02, 
      -6.6641753555245816e+02, 
       4.1339791352394223e-01, 
      -8.2994223832840526e-04, 
       1.0999613070201107e-04, 
      -8.2815649505433631e-02, 
      -1.0938036196605900e+02, 
       2.6336624535315567e-01, 
      -6.1995310736632212e-04, 
      -3.0000000000000001e-05, 
       2.6768637836637570e-02, 
      -7.6301551348229362e+00, 
       4.3377801050621556e-02, 
      -1.4999613070201104e-04, 
       1.6599154697477100e-03, 
      -5.4592264087203346e-01, 
       5.2597397400886703e-03, 
      -3.0000000000000001e-05, 
       1.4000086442279340e-04, 
      -3.9468807322558065e-02, 
       5.4997731272676332e-04, 
       1.0000000000000001e-05, 
      -2.8699181720991682e-03, 
       5.0000000000000002e-05, 
      -2.0999559352845242e-04, 
      -2.0000000000000002e-05, 
      -8.9993864682865569e-05, 
       2.0000000000000002e-05, 
      -1.1899218836971422e-03, 
       1.3998772936573111e-04, 
      -1.5138965103137053e-02, 
       3.0000000000000001e-05, 
       9.9991435544251718e-04, 
      -1.9309646760617927e-01, 
       5.9994717289762068e-04, 
       4.7295768064663591e-03, 
      -2.5679458767733752e+00, 
       1.2998772728058534e-04, 
      -3.5496786682241857e-03, 
      -7.4474951357555064e+00, 
       5.1895323465805934e-03, 
      -1.0000000000000001e-05, 
      -3.9996158815458320e-04, 
      -1.1616866651267410e+00, 
       2.6997615591387108e-03, 
       3.9996331282987846e-04, 
      -6.5125428755333420e-02, 
       3.4996858455827779e-04, 
       2.0000000000000002e-05, 
      -3.4097648760305628e-03, 
       3.0000000000000001e-05, 
      -1.4998859378852452e-04, 
      -3.0000000000000001e-05, 
      -3.4996491652575104e-04, 
       2.0000000000000002e-05, 
      -4.1595528113013806e-03, 
       2.0000000000000002e-05, 
       7.9993864682865570e-05, 
      -5.1414423754705015e-02, 
       3.0000000000000001e-05, 
      -1.5998159404859670e-04, 
      -1.0134887267215568e-01, 
       6.9993864682865557e-05, 
      -1.7958046806006885e-02, 
       4.0000000000000003e-05, 
      -9.4989829246006562e-04, 
      -5.0000000000000002e-05, 
      -8.9987729365731119e-05, 
      -1.0798422089319747e-03, 
      -1.2798159429881417e-03, 
      -3.1995091746292448e-04, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       1.0998159401036031e-04, 
       2.0000000000000002e-05, 
       1.9997545865498950e-04, 
       2.5397239085315128e-03, 
       6.9993864654188274e-05, 
      -1.0000000000000001e-05, 
       4.9999999980881818e-05, 
       2.4998245837579917e-04, 
      -3.9999999980881819e-05, 
       2.2898386651984026e-03, 
      -3.1997158931876420e-04, 
       4.3036851617944766e-02, 
      -6.9999999971322711e-05, 
      -7.9999999971322724e-05, 
      -3.2997631635752096e-04, 
      -3.6997718619213426e-04, 
      -4.9999999980881818e-05, 
       2.0000000000000002e-05, 
       1.4000086437499792e-04, 
      -2.0000000000000002e-05, 
       6.8997891533048204e-04, 
       2.0000000000000002e-05, 
      -1.0999999996176364e-04, 
      -6.9397621853979179e-03, 
       1.2999999995220451e-04, 
      -9.9994943937801984e-04, 
      -1.3635526910265638e-01, 
       1.2999999995220451e-04, 
       2.0000000000000002e-05, 
      -1.2809307020413472e-02, 
      -5.5692002982416444e-01, 
       1.5499176927330593e-03, 
      -2.1198888732590626e-03, 
      -1.2072586632235355e-01, 
       5.9996471378934223e-04, 
       1.1999999995220454e-04, 
      -1.4829513936803736e-02, 
       1.1999999995220454e-04, 
       1.0000000000000001e-05, 
      -1.4899516940545057e-03, 
       2.0000000000000002e-05, 
      -1.4000086437499792e-04, 
      -1.0000000000000001e-05, 
      -4.9999999980881818e-05, 
       1.0000000000000001e-05, 
      -7.2000518543504765e-04, 
       8.9999999961763637e-05, 
      -9.5500643636702888e-03, 
       2.0000000000000002e-05, 
       6.8999184870505300e-04, 
      -1.2812062711325589e-01, 
       3.7999485203640013e-04, 
       1.0000000000000001e-05, 
       2.8899630701658338e-03, 
      -1.7529658870917373e+00, 
      -1.4199772385874402e-03, 
       1.8999226133710849e-04, 
      -2.3789614548438715e-02, 
      -1.8584706631265686e+01, 
       1.5979758931841438e-02, 
      -1.0999999996176364e-04, 
       1.2299574368462561e-03, 
      -5.8330995747445824e-01, 
      -1.2498811836918333e+02, 
       2.5428572335145849e-01, 
      -5.3998065289228517e-04, 
       1.0999999996176364e-04, 
      -4.0119365740716291e-02, 
      -8.4530980809119285e+00, 
       4.4499321910298706e-02, 
      -1.4999613064465648e-04, 
      -7.6999098633872816e-04, 
      -5.8399222946181695e-01, 
       5.3999244182923751e-03, 
      -2.9999999990440909e-05, 
      -5.9999999980881817e-05, 
      -4.0740229068206957e-02, 
       5.5999485404507313e-04, 
      -2.8500181497324566e-03, 
       4.9999999980881818e-05, 
      -2.0000172876911412e-04, 
      -1.0000000000000001e-05, 
       1.1000086438455706e-04, 
      -1.0000000000000001e-05, 
       1.5499542893889137e-03, 
      -1.1999999995220454e-04, 
       2.2959262222460274e-02, 
      -6.0000000814940131e-05, 
      -7.0996644384778028e-04, 
       3.4964829777384165e-01, 
       6.2996557944410502e-04, 
      -7.9999999971322724e-05, 
       1.0776928885449957e+00, 
      -1.2510874740473208e-12, 
       1.7000604964155280e-04, 
       1.7954467304506036e+01, 
      -3.5868073866512869e-02, 
       6.9999999971322711e-05, 
      -1.5699450477570172e-03, 
       1.2618161438829738e+00, 
      -6.5896597480392654e-03, 
       2.0000000000000002e-05, 
       2.7998385871150466e-04, 
       9.2046945572244743e-02, 
      -8.3995643917530224e-04, 
       2.0000000000000002e-05, 
       6.7897812967059792e-03, 
      -8.9999999961763637e-05, 
       4.9998505154858841e-04, 
       3.9999999990440911e-05, 
       3.9999999990440911e-05, 
       3.7997105128246991e-04, 
       3.6097474108487937e-03, 
      -2.9997834420817477e-03, 
       1.0000000000000001e-05, 
       2.9993864673306469e-05, 
      -3.9677107212324189e-02, 
       1.0999386464462920e-04, 
       2.0000000417029158e-05, 
      -8.4893888830128075e-03, 
       4.9999999980881818e-05, 
      -1.0199298326691384e-03, 
       1.0000000000000001e-05, 
      -1.0999386464462920e-04, 
      -1.0000000000000001e-05, 
       6.9993864654188274e-05, 
      -1.0998772932749475e-04, 
      -7.5377475448448789e-09, 
      -5.5993864620132634e-04, 
      -8.9987729337053836e-05, 
      -1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.6995091746292446e-04, 
       4.9990796940892505e-04, 
       4.9993864682865559e-05, 
       2.0000000000000002e-05, 
       3.1995178146868876e-04, 
       8.1188227048091203e-03, 
      -2.0000000000000002e-05, 
       4.0000000000000003e-05, 
       1.3308050673809144e-02, 
      -3.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       1.3298062606179031e-03, 
       1.1998159404859668e-04, 
       1.0000000000000001e-05, 
       3.0000000000000001e-05, 
       4.8994737499714121e-04, 
      -4.0000000000000003e-05, 
       9.6489688340907539e-03, 
      -2.0000000000000002e-05, 
      -2.0997545873146223e-04, 
       2.4694321033747732e-01, 
      -5.1993704313278298e-04, 
       1.6297949907171786e-03, 
       3.3979269667574064e-01, 
      -7.4990863173219800e-04, 
      -6.4991563188915508e-04, 
       3.2996433487961413e-02, 
      -1.5998159404859670e-04, 
      -4.9993864682865559e-05, 
       2.9496843310041938e-03, 
      -3.0000000000000001e-05, 
       2.5997018742009208e-04, 
       2.0000000000000002e-05, 
       5.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       8.7994383294838702e-04, 
      -1.2998772936573112e-04, 
       1.5438937565750930e-02, 
      -1.1099043508273937e-03, 
       2.9419933639500490e-01, 
      -7.1993576752020364e-04, 
      -4.3396143538889835e-03, 
       7.3712009873208606e+00, 
      -1.5408629174585072e-02, 
       3.0000000000000001e-05, 
      -8.0000000000000007e-05, 
       6.3134273217907197e-02, 
       8.0501722501471242e+00, 
      -1.8158365570262437e-02, 
       4.0000000000000003e-05, 
       1.0000000000000001e-05, 
      -1.6438530931310025e-02, 
       7.5167634592978361e-01, 
      -3.7796659066298708e-03, 
       1.0000000000000001e-05, 
      -1.3298829400007950e-03, 
       6.4385511148316102e-02, 
      -5.4995104219560966e-04, 
      -1.0999386468286557e-04, 
       5.3796341272183221e-03, 
      -6.9999999999999994e-05, 
       4.3997191584864979e-04, 
       4.0000000000000003e-05, 
       1.0000086442279343e-04, 
      -3.0000000000000001e-05, 
       1.7799463418591865e-03, 
      -3.5998258403813351e-04, 
       2.9489099882669595e-02, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
      -3.6498203993748542e-03, 
       4.7840471403548945e-01, 
       3.0000000000000001e-05, 
       5.0000000000000002e-05, 
      -2.8038574609160361e-02, 
       8.6055315759708346e+00, 
      -2.1568901502827795e-02, 
       1.1999613070201107e-04, 
      -1.4999613070201104e-04, 
      -6.1026813470894330e-02, 
       2.0543594980827189e+02, 
      -4.2621769681760546e-01, 
       9.2993396172481055e-04, 
      -2.8597950970269438e-03, 
       2.1460741878253438e+00, 
       1.6472861720988391e+02, 
      -3.8212969982592837e-01, 
       8.8993923220212229e-04, 
       1.7998612608688771e-04, 
      -3.7451034323166538e-01, 
       1.4530277628654355e+01, 
      -7.3726191969978733e-02, 
       2.4997999076975324e-04, 
       1.0000000000000001e-05, 
      -2.8868533323010846e-02, 
       1.1770012469305144e+00, 
      -1.0069497923021725e-02, 
       5.0000000000000002e-05, 
      -2.3198866479913968e-03, 
       9.3197086776599547e-02, 
      -1.1699437555815514e-03, 
      -1.8999472910565898e-04, 
       7.2997775780399348e-03, 
      -1.2000000000000000e-04, 
      -1.0000000000000001e-05, 
       5.6998591532850544e-04, 
      -1.0000000000000001e-05, 
       4.0000000000000003e-05, 
       3.0000000000000001e-05, 
       3.6000432211396710e-04, 
      -1.1000086442279343e-04, 
       4.9700466746605528e-03, 
      -1.1099892969039312e-03, 
       6.8500550949332994e-02, 
      -6.9999999999999994e-05, 
       5.0000000000000002e-05, 
      -1.0769882553088510e-02, 
       9.4827605247099755e-01, 
      -5.3999184979720686e-04, 
       2.3999312582681555e-04, 
      -9.1538856043092448e-02, 
       1.3194063173790582e+01, 
       4.6699433443401547e-03, 
      -3.0000000000000001e-05, 
       3.6999011970053258e-04, 
      -5.6064211237340766e-01, 
       2.1165554821401352e+02, 
      -5.3571237723129372e-01, 
       2.8199119109441522e-03, 
       3.0000000000000001e-05, 
      -8.4997180352719484e-03, 
       1.7902698262118480e-01, 
       4.5864382748764046e+03, 
      -9.3658636971804885e+00, 
       1.9929344962433863e-02, 
      -4.0000000000000003e-05, 
       1.4999226140402212e-04, 
      -7.5177394616022644e-02, 
       5.5177059700241536e+01, 
       2.3699139285772667e+03, 
      -5.7416072726469185e+00, 
       1.3719546083927990e-02, 
      -3.0000000000000001e-05, 
       1.0000000000000001e-05, 
       5.5998151376255711e-04, 
      -6.3831492299232613e+00, 
       1.9195619489527536e+02, 
      -9.9021537775935742e-01, 
       3.4398886922370146e-03, 
      -1.0000000000000001e-05, 
       8.0000000000000007e-05, 
      -4.5379355888279105e-01, 
       1.4379697094884525e+01, 
      -1.2385836590823918e-01, 
       5.8998324594437736e-04, 
       2.0000000000000002e-05, 
      -3.3359573910778845e-02, 
       1.0594959685741081e+00, 
      -1.3319840849812047e-02, 
       8.0000000000000007e-05, 
      -2.4699729960813159e-03, 
       7.7650569609901368e-02, 
      -1.3099854267718839e-03, 
       1.0000000000000001e-05, 
      -1.8000172884558686e-04, 
       5.6800466721583787e-03, 
      -1.2000086442279343e-04, 
      -1.0000000000000001e-05, 
       4.1000432169693791e-04, 
      -1.0000000000000001e-05, 
       3.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.5000172842855768e-04, 
      -3.0000000000000001e-05, 
       1.7299524725890006e-03, 
      -1.9999472827160066e-04, 
       1.8189446900386926e-02, 
      -3.0012272302385518e-05, 
      -1.1599428886565832e-03, 
       1.7623439547424488e-01, 
      -2.3998558891332903e-04, 
       2.0000000000000002e-05, 
      -3.4598243384344036e-03, 
       2.4888870370572809e+00, 
      -9.9999858210086288e-06, 
       1.6998612608688771e-04, 
      -2.2798820324075567e-03, 
      -2.8397100387825329e+01, 
       6.7416468914439420e-02, 
      -1.5999613070201107e-04, 
       1.9798567275794987e-03, 
      -1.4371624246554999e+00, 
      -2.4358228023531897e+01, 
       6.5406559242570148e-02, 
      -1.6998612608688771e-04, 
       3.0000000000000001e-05, 
       8.4715549444192614e-02, 
      -2.9145403053891537e+00, 
       1.6049176443184703e-02, 
      -6.0000000000000002e-05, 
       8.8895485793052936e-03, 
      -2.9022053823049210e-01, 
       2.6198687771284064e-03, 
      -1.0000000000000001e-05, 
       8.4995816791930991e-04, 
      -2.6759173365087850e-02, 
       3.4998171919831090e-04, 
       8.0000000000000007e-05, 
      -2.3599296610001077e-03, 
       4.0000000000000003e-05, 
      -1.9999559352845242e-04, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -4.8996664578836700e-04, 
       4.0000000000000003e-05, 
      -9.4393463774119692e-03, 
       2.0000000000000002e-05, 
       6.0000000000000002e-05, 
      -2.4988250505490764e-01, 
       5.9994717289762068e-04, 
       2.0000000000000002e-05, 
      -6.5394170054182748e-03, 
      -2.5211988988567753e+00, 
       5.1495392536223573e-03, 
      -1.0000000000000001e-05, 
       4.0000000000000003e-05, 
      -2.3487922182196540e-02, 
      -1.8574703726238279e-01, 
       9.5991962633685830e-04, 
      -1.9998317976156360e-03, 
      -1.3429085176727668e-02, 
       1.1999386468286557e-04, 
      -1.3998772936573111e-04, 
      -6.5995610358265586e-04, 
      -1.0000000000000001e-05, 
      -8.9987729365731119e-05, 
      -9.2989829037491993e-04, 
      -3.0000000000000001e-05, 
      -1.8797939874814593e-03, 
       6.9987729365731107e-05, 
       3.7995878037455828e-04, 
       2.0000000000000002e-05, 
       4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       2.0000000000000002e-05, 
      -1.6998159398168305e-04, 
      -5.2994478111099074e-04, 
      -4.9999999980881818e-05, 
       2.9996932329961866e-04, 
       2.9999999990440909e-05, 
      -1.3998772930837655e-04, 
      -3.9897184654674967e-03, 
      -7.9999999971322724e-05, 
      -1.1819154776759650e-02, 
       2.9999999990440909e-05, 
      -1.0199070284880199e-03, 
       5.1796152842171735e-03, 
      -3.9999999980881819e-05, 
       3.6997545858807586e-04, 
      -1.2999999995220451e-04, 
      -2.9699033914452446e-03, 
      -8.3107215024644659e-02, 
       1.9999386460639284e-04, 
      -1.8899041488641176e-03, 
      -2.3140200583971432e-01, 
       7.4995944283764678e-04, 
       2.0000000000000002e-05, 
      -1.9478951652872143e-02, 
       6.5067436994949865e-02, 
       1.9999386377233451e-04, 
      -8.3995471032971536e-04, 
       3.2998552375344079e-03, 
       4.9999999980881818e-05, 
      -3.9999999980881819e-05, 
      -1.1000086229941127e-04, 
       1.0000000000000001e-05, 
      -4.9999999980881818e-05, 
      -4.9999999980881818e-05, 
       1.0000000000000001e-05, 
      -1.7400112368462959e-03, 
       1.0999999996176364e-04, 
      -4.4420191594329188e-02, 
       1.0999999996176364e-04, 
      -2.7999613143091934e-04, 
      -1.2241239354170774e+00, 
       2.9799592037488120e-03, 
      -1.0000000000000001e-05, 
       5.9999999980881817e-05, 
      -2.9789532034855189e-02, 
      -3.2096875702680525e+00, 
       1.2149816513247130e-02, 
      -3.9999999990440911e-05, 
       2.4999226130843115e-04, 
      -2.5303567949529321e-01, 
       4.0422724909322255e-01, 
       3.7499475884785766e-03, 
      -2.0000000000000002e-05, 
       4.9999999980881818e-05, 
      -1.0619833696571749e-02, 
       3.6963285764371392e-04, 
       8.2999098256634758e-04, 
      -5.0999399005842710e-04, 
      -3.8900502005239719e-03, 
       1.3999999995220451e-04, 
      -2.0000000000000002e-05, 
      -6.2000345537660962e-04, 
       2.0000000000000002e-05, 
      -6.9999999971322711e-05, 
      -5.9999999980881817e-05, 
      -5.8997891536871844e-04, 
       2.0000000000000002e-05, 
      -6.2298023688168265e-03, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
      -4.1888591181851030e-02, 
       6.9999999971322711e-05, 
       1.1599394338714835e-03, 
       2.7376050640024469e-01, 
      -1.3799302936026807e-03, 
       8.9095191827036676e-03, 
       1.4989475229298219e-01, 
      -9.5994857372325713e-04, 
       1.9999386460639284e-04, 
       2.2439244308103623e-02, 
      -2.1999386459683372e-04, 
      -1.0000000000000001e-05, 
       2.5599174746015686e-03, 
      -3.9999999990440911e-05, 
       2.5998945811572696e-04, 
       2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -1.9998859371205181e-04, 
      -3.5997632051825346e-04, 
      -1.0000000000000001e-05, 
      -1.2699105659621721e-03, 
      -8.9987729327494750e-05, 
      -1.9098606137589759e-03, 
       1.0000000000000001e-05, 
      -3.8996932285391227e-04, 
      -5.9999999980881817e-05, 
       1.2270662946163423e-08, 
      -6.4612388089665551e-09, 
       3.9999999990440911e-05, 
      -1.3998772931793563e-04, 
      -1.9984914225727413e-05, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       3.9993864682865560e-05, 
       3.0000000000000001e-05, 
       1.0000000000000001e-05, 
       3.2995178105165959e-04, 
       8.5987288510061782e-04, 
       5.3992023920913562e-04, 
       8.9987729365731119e-05, 
       1.0000000000000001e-05, 
       2.6997018742009211e-04, 
       7.5491959222631043e-03, 
      -4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       1.7408120729120773e-02, 
      -8.9987729365731119e-05, 
       6.9993864682865557e-05, 
       9.7089419363624394e-03, 
      -6.0000000000000002e-05, 
      -3.0000000000000001e-05, 
       1.5998255289069543e-03, 
      -1.0000000000000001e-05, 
       1.9997545831443309e-04, 
       2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       5.5996157195988012e-03, 
      -4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       1.5779894437490621e-01, 
      -8.9991962591982910e-04, 
       1.0799026232328943e-03, 
       3.0871801000134380e-01, 
      -1.5998597982434636e-03, 
       9.0991530255477438e-04, 
       1.5225906774649178e-01, 
      -8.8991876108000648e-04, 
      -5.1995631308995058e-04, 
       2.4108301622573006e-02, 
      -2.0998772894870199e-04, 
      -8.9993864682865569e-05, 
       2.8997964930560258e-03, 
      -4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
       3.0997718674299077e-04, 
       3.0000000000000001e-05, 
      -5.0000000000000002e-05, 
      -3.3999032221708226e-04, 
       4.0000000000000003e-05, 
      -1.9999472910565898e-04, 
       6.9999999999999994e-05, 
       9.9816818341372293e-02, 
      -7.7995730266245821e-04, 
       1.0000000000000001e-05, 
       4.2997558429820561e-04, 
       2.7318888991130463e+00, 
      -1.5329221686054889e-02, 
       6.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.2888806358530470e-02, 
       4.3740094149881275e+00, 
      -2.2728827028039987e-02, 
       8.0000000000000007e-05, 
      -1.0000000000000001e-05, 
       6.6596367501723175e-03, 
       1.8707647192162218e+00, 
      -1.1139425328502052e-02, 
       4.0000000000000003e-05, 
      -6.0000000000000002e-05, 
      -7.3196179071459144e-03, 
       2.8250053804221126e-01, 
      -2.5098723170920152e-03, 
       1.0000000000000001e-05, 
      -1.2099420263189352e-03, 
       3.2738955542451958e-02, 
      -4.0998171836425259e-04, 
      -1.3999999999999999e-04, 
       3.3598954377783608e-03, 
      -6.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       3.1999032180005311e-04, 
       3.0000000000000001e-05, 
      -6.9999999999999994e-05, 
       1.0000000000000001e-05, 
      -6.2000604970846650e-04, 
       1.0000000000000000e-04, 
      -4.3100337049824184e-03, 
       4.1999485383834408e-04, 
      -7.0000432336505452e-04, 
      -2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.2999699429074614e-04, 
       1.1868256646571000e+00, 
      -9.3398815629530830e-03, 
       6.9999999999999994e-05, 
       1.2999613070201104e-04, 
       6.6199085327521010e-03, 
       3.0772574469538196e+01, 
      -1.6976764181870591e-01, 
       6.0998238152158387e-04, 
       1.6999613028498192e-04, 
       3.0085540903233987e-01, 
       3.8429834065082716e+01, 
      -2.0133705199593427e-01, 
       7.1997851180656590e-04, 
      -3.0000000000000001e-05, 
      -2.3909623818604515e-02, 
       1.3899059511228872e+01, 
      -8.4998752997522520e-02, 
       3.4998839127197491e-04, 
      -6.0998151793284874e-04, 
      -6.6009005303252169e-02, 
       1.9777269709780243e+00, 
      -1.7809756481404950e-02, 
       9.0000000000000006e-05, 
       1.0000000000000001e-05, 
      -9.7998651776269961e-03, 
       2.1853111872294204e-01, 
      -2.7399656640878030e-03, 
       2.0000000000000002e-05, 
      -1.0899867028014926e-03, 
       2.1490146986460099e-02, 
      -3.5999398941555065e-04, 
      -1.1000000000000000e-04, 
       1.9800138261773740e-03, 
      -4.0000000000000003e-05, 
       1.7000172842855771e-04, 
       1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -4.0000000000000003e-05, 
       6.3997977834325448e-04, 
      -4.0000000000000003e-05, 
       2.6819162891074159e-02, 
      -2.9999999582970844e-05, 
      -3.8998085435848836e-04, 
      -3.5757788086640147e-01, 
       2.1998884645307976e-03, 
      -1.8749021919187647e-02, 
      -6.3520814175288576e-01, 
       3.5298208949222213e-03, 
      -1.0000000000000001e-05, 
       5.0996785738860332e-04, 
      -2.8910002243214866e-01, 
       1.8599058754883142e-03, 
       4.0000000000000003e-05, 
       1.8299050135676957e-03, 
      -5.0698321291315837e-02, 
       4.7997558304711818e-04, 
       3.1998472407350649e-04, 
      -6.6797899699247648e-03, 
       9.0000000000000006e-05, 
       4.0000000000000003e-05, 
      -7.6997364344314910e-04, 
       1.0000000000000001e-05, 
      -8.0000000000000007e-05, 
      -5.0000000000000002e-05, 
      -3.6897473244932536e-03, 
       2.0000000000000002e-05, 
       8.0000000000000007e-05, 
      -3.5137536195548782e-02, 
       1.8998772894870199e-04, 
       8.9991876066297736e-04, 
      -2.1708465634576391e-02, 
       1.1999386468286557e-04, 
       1.0000000000000001e-05, 
      -2.0798569834726936e-03, 
       2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -7.9994729105658993e-05, 
       1.0000000000000001e-05, 
       1.9997545873146224e-04, 
       2.2997545831443308e-04, 
       5.0000000000000002e-05, 
      -2.4540377124886545e-08, 
      -2.9999999990440909e-05, 
       1.0000000000000001e-05, 
      -1.2998772931793563e-04, 
      -5.1996491550051083e-04, 
      -2.9999999990440909e-05, 
      -1.0999386381057087e-04, 
      -2.0000000000000002e-05, 
       1.1999386463507010e-04, 
       2.0000000000000002e-05, 
      -6.9999999980881810e-05, 
      -1.8899411517054080e-03, 
       1.0000000000000001e-05, 
      -2.9999999990440909e-05, 
      -6.9297709077279961e-03, 
       3.9999999980881819e-05, 
      -3.9997998978274943e-04, 
      -1.5399555574949681e-03, 
       2.9999999990440909e-05, 
      -2.0999386460639284e-04, 
       1.0199614584522137e-03, 
      -2.9999999990440909e-05, 
       1.3999999995220451e-04, 
      -5.9000345622022704e-04, 
      -1.6860079490600130e-02, 
       9.9999999961763637e-05, 
      -3.0999699501009536e-04, 
      -5.7250246051946750e-02, 
       3.8999398927216418e-04, 
      -3.3499471412369239e-03, 
      -1.3500126888151383e-02, 
       2.3999612978192091e-04, 
      -2.0000000000000002e-05, 
      -1.3699767812032594e-03, 
       4.3899544269948880e-03, 
       5.9999999980881817e-05, 
      -1.8999999993308636e-04, 
       4.9999827221431877e-04, 
       1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
      -5.9999999980881817e-05, 
      -4.5998332063697402e-04, 
       1.0000000000000001e-05, 
       3.9198718475904771e-03, 
      -3.9999999990440911e-05, 
       1.5999999994264545e-04, 
       4.7498424417146126e-03, 
      -4.9999999980881818e-05, 
       4.9999999980881818e-05, 
       1.8099368283690308e-03, 
      -2.0000000000000002e-05, 
       3.4998859282063896e-04, 
       4.9999999980881818e-05, 
       1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -7.9993864654188287e-05, 
      -3.9999999980881819e-05, 
      -1.0000000000000001e-05, 
      -1.9118187377780694e-14, 
       1.0000000000000001e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       1.9998245805436096e-04, 
       5.8993423868899150e-04, 
       5.8993423827196235e-04, 
       2.3997545789740390e-04, 
       5.0000000000000002e-05, 
       1.3998772936573111e-04, 
       3.1197876805299598e-03, 
      -3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       8.3594149696340162e-03, 
      -6.9999999999999994e-05, 
       3.0000000000000001e-05, 
       7.4394759535129076e-03, 
      -6.9999999999999994e-05, 
      -2.0000000000000002e-05, 
       2.8497974393342246e-03, 
      -3.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       5.5995877954050002e-04, 
       8.0000000000000007e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       1.7599437444205147e-03, 
      -2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       3.8998763101098795e-02, 
      -3.8998171878128175e-04, 
       4.3997558388117648e-04, 
       9.0327051829633215e-02, 
      -7.9995643740560663e-04, 
       1.7999386384880725e-04, 
       7.1417615893261940e-02, 
      -6.3996557759793665e-04, 
      -2.4999085980767006e-04, 
       2.5099162146062979e-02, 
      -2.5998385923368386e-04, 
      -1.6000000000000001e-04, 
       4.7698476225132812e-03, 
      -6.0000000000000002e-05, 
      -3.0000000000000001e-05, 
       6.8997891392046096e-04, 
      -1.0000000000000001e-05, 
       9.0000000000000006e-05, 
       2.0000172842855771e-04, 
       1.3760086405580775e-02, 
      -1.7000086442279342e-04, 
       1.8000000000000001e-04, 
       2.9255150843605809e-01, 
      -2.8699635232183720e-03, 
       2.0000000000000002e-05, 
       3.6399493233071124e-03, 
       5.7156243369584214e-01, 
      -5.0899338785667068e-03, 
       3.0000000000000001e-05, 
       6.9997403395386472e-05, 
       3.9451145761016654e-01, 
      -3.5899493174687039e-03, 
       2.0000000000000002e-05, 
      -8.0000000000000007e-05, 
      -2.1399673550604602e-03, 
       1.2616048172676975e-01, 
      -1.3199811017387127e-03, 
      -1.0000000000000001e-05, 
      -9.3998711399127875e-04, 
       2.2850114068409776e-02, 
      -3.0999699429074619e-04, 
      -1.7999999958297086e-04, 
       3.1800155483504944e-03, 
      -5.0000000000000002e-05, 
      -3.0000000000000001e-05, 
       3.8000259201729283e-04, 
       4.0000000000000003e-05, 
       1.0000000000000001e-05, 
       2.7998945612617212e-04, 
      -4.7798445528808757e-03, 
       5.0000000000000002e-05, 
      -2.5998385965071301e-04, 
      -1.2819580249561514e-02, 
       1.2000000000000000e-04, 
      -6.9999999582970830e-05, 
      -1.0639642712304840e-02, 
       1.0000000000000000e-04, 
       8.0000000000000007e-05, 
      -3.9598669818323795e-03, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
      -8.2997191376350398e-04, 
       1.0000000000000001e-05, 
      -1.2999999958297084e-04, 
      -2.0000000000000002e-05, 
      -6.0000000000000002e-05, 
      -5.0996577969745713e-04, 
       1.0000000000000001e-05, 
      -6.0995964354626433e-04, 
       1.0000000000000001e-05, 
      -2.0998159321453837e-04, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -3.9999999990440911e-05, 
      -1.5999386420848186e-04, 
      -1.1999999953517537e-04, 
       2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -2.5000172791593756e-04, 
      -9.8000518325431097e-04, 
       1.0000000000000001e-05, 
      -4.9999999980881818e-05, 
      -6.8000259010063977e-04, 
       1.0000000000000001e-05, 
      -3.9999999980881819e-05, 
      -4.9999999146823498e-05, 
      -1.0000000000000001e-05, 
       4.9999999980881818e-05, 
       5.9999999980881817e-05, 
       1.0999999996176364e-04, 
       6.9999999971322711e-05, 
       2.9999999990440909e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       6.0000000000000002e-05, 
       1.8998859295446622e-04, 
       2.4998245722030268e-04, 
       1.4998772853167279e-04, 
       5.0000000000000002e-05, 
       1.0000000000000001e-05, 
       3.0000000000000001e-05, 
       5.7998505007165373e-04, 
       1.6999472848011523e-03, 
      -2.0000000000000002e-05, 
       1.9599341480759733e-03, 
      -3.0000000000000001e-05, 
       1.1199605054668827e-03, 
      -1.0000000000000001e-05, 
       3.6998945654320129e-04, 
       8.0000000000000007e-05, 
       1.0000000000000001e-05, 
       1.8000172842855771e-04, 
       3.2400207357213130e-03, 
      -5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       8.4000461959887664e-03, 
      -1.1000000000000000e-04, 
       1.0000000000000001e-05, 
       8.6600422808942937e-03, 
      -1.0999999958297084e-04, 
      -5.0000000000000002e-05, 
       4.5500194093078998e-03, 
      -6.0000000000000002e-05, 
      -4.0000000000000003e-05, 
       1.3900051802813232e-03, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       2.9000172759449937e-04, 
       5.0000000000000002e-05, 
      -6.9999999999999994e-05, 
      -2.3999472868862983e-04, 
      -2.8998859295446626e-04, 
      -1.6999386384880725e-04, 
      -6.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       3.0000000000000001e-05, 
       5.0000000000000002e-05, 
       4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       4.0000000000000003e-05, 
       1.1999999958297084e-04, 
       1.7000086358873509e-04, 
       1.2999999916594166e-04, 
       6.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.8461240380000967e+04, 
      -6.2964776186139018e+00, 
       5.9197463551783165e-03, 
      -2.3599037212003134e-03, 
       2.7986414518273106e+00, 
       9.9969366315481670e+02, 
       1.0101671561153941e+03, 
      -1.0193852447752909e+00, 
       1.5599368152025683e-03, 
      -4.3998238235564216e-04, 
      -1.3034701506935700e-01, 
       3.1759670828363294e+01, 
       6.1911950259640300e+01, 
      -1.1786732060032998e-01, 
       2.5998925652882663e-04, 
       3.0000000000000001e-05, 
      -1.4599682240705745e-03, 
       1.5813035766718755e+00, 
       3.9840544801889854e+00, 
      -1.1809750513357475e-02, 
       4.0000000000000003e-05, 
      -6.0000000000000002e-05, 
       9.1569768780338345e-02, 
       2.6324939411177511e-01, 
      -1.0899798302964247e-03, 
       5.6599937867737541e-03, 
       1.7679988271116290e-02, 
      -1.0000000000000000e-04, 
       3.6000345769117371e-04, 
       1.1999996296023475e-03, 
       2.0000000000000002e-05, 
       8.0000864422793429e-05, 
      -4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
      -5.2998204644754570e-04, 
      -1.4000086442279340e-04, 
       3.0000000000000001e-05, 
      -6.4597471744927944e-03, 
      -1.8599291354046150e-03, 
       2.2999085980767003e-04, 
      -7.4786973296373349e-02, 
      -2.3649035208117827e-02, 
       2.0000000000000002e-05, 
       1.5399112744515736e-03, 
      -7.9318663729777539e-01, 
      -3.0127728711866791e-01, 
       1.9098821237779097e-03, 
      -1.0000000000000001e-05, 
       6.7895864296686854e-03, 
      -6.7314362866293189e+00, 
      -5.6323537756655035e+00, 
      -9.1094394883815977e-03, 
       1.0000519362625620e-04, 
      -4.8396143488170438e+00, 
      -6.4600721505279202e+00, 
       1.1569282182361574e-02, 
      -2.0000000000000002e-05, 
      -7.0295716955030710e-03, 
      -5.0759179342248544e+00, 
      -5.3112784145560372e+00, 
       6.6795917022311434e-03, 
      -1.0000000000000001e-05, 
       1.4099204142268746e-03, 
      -3.1290679453673825e-01, 
      -6.3881314193420602e-01, 
       1.3499204150609330e-03, 
       3.0000000000000001e-05, 
      -2.4189016390142579e-02, 
      -6.1757501429886924e-02, 
       1.9999085980767004e-04, 
      -1.9099291349875857e-03, 
      -5.4797856752831116e-03, 
       2.0000000000000002e-05, 
      -1.5000086442279340e-04, 
      -4.5998118244178145e-04, 
      -1.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
      -1.5998859378852455e-04, 
      -4.0000000000000003e-05, 
      -1.6898695941853505e-03, 
      -4.8996191165055563e-04, 
       3.0000000000000001e-05, 
      -1.5708764255911896e-02, 
      -5.3595781915790472e-03, 
       4.0000000000000003e-05, 
       9.9993864682865568e-05, 
      -1.1334089519200992e-01, 
      -9.5102356472747188e-02, 
      -8.9993864682865569e-05, 
      -1.3998772936573111e-04, 
      -1.8008571587748830e-02, 
      -5.7285376796267398e-02, 
       1.5998386006774219e-04, 
      -6.9999999999999994e-05, 
      -6.1865021262157795e-02, 
      -5.5035578507142899e-02, 
       6.9999999999999994e-05, 
       1.0000000000000001e-05, 
      -2.8397768268233711e-03, 
      -5.2795834649755633e-03, 
       1.0000000000000001e-05, 
      -1.5998772936573114e-04, 
      -3.6997718757704907e-04, 
      -2.0000000000000002e-05, 
      -4.0000000000000003e-05, 
      -1.0000000000000001e-05, 
      -3.2996405210295766e-04, 
      -1.0998772936573112e-04, 
      -2.1597448309137817e-03, 
      -1.8997785123746492e-03, 
       6.5991863301108811e-04, 
      -3.0996319143342663e-04, 
      -1.0898774183899964e-03, 
      -8.2990196007556318e-04, 
      -4.0000000000000003e-05, 
      -7.9993864682865570e-05, 
      -3.9993864682865560e-05, 
      -4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       2.9999999990440909e-05, 
       1.0998772932749475e-04, 
       1.1998772931793565e-04, 
      -2.0000000000000002e-05, 
       6.9999999971322711e-05, 
      -2.9999999990440909e-05, 
       6.3994791234875175e-04, 
      -1.0000000000000001e-05, 
       2.0398334043096023e-03, 
       2.1398211344711548e-03, 
      -5.9995404726797511e-04, 
       4.9999999980881818e-05, 
      -5.9999999971322719e-05, 
      -2.9999999990440909e-05, 
       2.9999999990440909e-05, 
       1.7999472903874533e-04, 
      -3.9999999990440911e-05, 
       9.5995890683870267e-04, 
      -1.3699436299252053e-03, 
      -2.9999999990440909e-05, 
       1.0309547898187386e-02, 
       7.9995771355591250e-04, 
      -5.1996998596344794e-04, 
      -1.3459392383327749e-02, 
      -1.8289191222779496e-02, 
       5.9999999980881817e-05, 
      -7.9999999971322724e-05, 
      -2.0119122466600518e-02, 
      -8.4496359940365653e-03, 
       2.0000000000000002e-05, 
      -1.7499283538553255e-03, 
      -1.4599383573245114e-03, 
      -1.4999999994264542e-04, 
      -1.7999472903874533e-04, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -1.1000086438455706e-04, 
      -3.9999999990440911e-05, 
      -1.2199961697937800e-03, 
      -4.3000259310587563e-04, 
      -1.2549955154032127e-02, 
      -5.3699786296829413e-03, 
       2.0000000000000002e-05, 
      -2.4999226130843115e-04, 
      -1.0963943437153732e-01, 
      -7.8459564051565581e-02, 
      -1.9999613062553833e-04, 
      -3.6899093154652042e-03, 
      -4.2988728107179575e-01, 
       1.3927888402397687e-01, 
       2.3399415459683273e-03, 
       2.9999999990440909e-05, 
      -3.2259168071847082e-02, 
      -4.8057402037493979e+00, 
      -5.3683961612559825e+00, 
       6.4498353403772924e-03, 
      -1.0000000000000001e-05, 
       1.0000000000000001e-05, 
      -1.8999613105212656e-04, 
      -5.8892589129933504e-01, 
      -6.6740564496553156e-01, 
       1.4199677678096423e-03, 
      -3.6359802389319490e-02, 
      -6.3829692974737604e-02, 
       2.0999613062553833e-04, 
      -2.4699927536774319e-03, 
      -5.5299794944654096e-03, 
       2.0000000000000002e-05, 
      -1.7000086435587977e-04, 
      -4.5000259309631654e-04, 
      -1.0000000000000001e-05, 
      -3.9999999990440911e-05, 
       1.0000000000000001e-05, 
       1.6999472903874533e-04, 
       6.9999999971322711e-05, 
       1.8599256770251865e-03, 
       9.2995977127105522e-04, 
      -1.0000000000000001e-05, 
       1.5969326570793512e-02, 
       1.6799286013602484e-02, 
       5.9999999971322719e-05, 
       6.9999999971322711e-05, 
       6.3497693454486765e-03, 
      -5.5897523214725221e-02, 
      -2.0998999530840389e-04, 
       7.9994997662805132e-04, 
       8.0422378961740737e-01, 
       8.0259387492276002e-01, 
      -8.2994610732050331e-04, 
      -2.0998999572543304e-04, 
       3.4648501149383414e-02, 
       1.0175558259351709e-01, 
      -1.9998999530840389e-04, 
       1.0000000000000001e-05, 
       3.1698667157862761e-03, 
       1.0159573464138262e-02, 
      -2.9999999990440909e-05, 
       2.6998945810616790e-04, 
       9.2995977127105522e-04, 
       2.0000000000000002e-05, 
       7.9999999971322724e-05, 
       2.9999999990440909e-05, 
       1.0000000000000001e-05, 
       1.6998772930837657e-04, 
       1.5998772930837657e-04, 
      -7.9993864654188287e-05, 
      -3.8996932243688307e-04, 
      -1.6998618145833678e-03, 
      -2.0398372731905623e-03, 
      -2.1998159396256485e-04, 
      -7.1993790728792190e-04, 
      -2.9999999990440909e-05, 
      -1.1999386463507010e-04, 
      -1.0000000000000001e-05, 
      -2.9999999990440909e-05, 
      -2.9999999990440909e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       3.9993864682865560e-05, 
       2.0000000000000002e-05, 
       3.0000000000000001e-05, 
       3.0995705278005889e-04, 
       3.3995091746292448e-04, 
       1.1798166019751813e-03, 
       6.1990409969390700e-04, 
       7.9987729365731119e-05, 
       9.9987729365731118e-05, 
       1.0000000000000001e-05, 
       5.9993864682865558e-05, 
       7.6990895939846185e-04, 
      -5.0000000000000002e-05, 
       9.1389311067919324e-03, 
       1.0418780115449458e-02, 
      -2.0000000000000002e-05, 
       7.9993864682865570e-05, 
       3.6465675780352910e-02, 
       1.6028102763916620e-02, 
      -2.0000000000000002e-05, 
      -3.0000000000000001e-05, 
       2.3997211540680370e-03, 
       2.5596988771505289e-03, 
       2.0997545873146223e-04, 
       3.0996405210295767e-04, 
       2.0000000000000002e-05, 
       3.0000000000000001e-05, 
       1.1998772936573112e-04, 
       2.0000000000000002e-05, 
       1.7798643241250674e-03, 
       2.9997718757704905e-04, 
       2.2848199449214172e-02, 
      -1.9198494614735061e-03, 
      -8.9993864682865569e-05, 
       2.3997385545261883e-04, 
       2.6862839528785964e-01, 
       3.1471470039065103e-01, 
      -4.4995771552036102e-04, 
       2.7297278859196752e-03, 
       1.0857809772361875e+00, 
       3.8349873586206878e-01, 
      -4.3995771510333190e-04, 
      -7.1992629924458058e-04, 
       6.9144454997641505e-02, 
       5.8475324967037876e-02, 
      -1.0999386468286557e-04, 
      -3.0000000000000001e-05, 
       5.5495667836994557e-03, 
       6.7694683279979108e-03, 
      -2.0000000000000002e-05, 
       4.5996578094854450e-04, 
       6.9994663530703279e-04, 
       4.0000000000000003e-05, 
       6.9999999999999994e-05, 
       2.0000000000000002e-05, 
       2.8999032263411138e-04, 
       6.0000000000000002e-05, 
       4.1598377235713656e-03, 
       7.9996677135511047e-04, 
      -4.0000000000000003e-05, 
       5.5017765706985064e-02, 
       9.3496192475408194e-03, 
       6.9999999999999994e-05, 
       3.2998085560957582e-04, 
       6.5022258058474325e-01, 
      -6.2077378299581376e-02, 
      -2.6698408305388952e-03, 
      -1.0000000000000001e-05, 
       1.0339367321461741e-02, 
       7.4345550611201423e+00, 
       8.8681426530612608e+00, 
      -1.1769285589814833e-02, 
       2.0000000000000002e-05, 
      -9.0000000000000006e-05, 
       8.8144472523691508e-02, 
       2.9576600674946288e+01, 
       7.9585542335066854e+00, 
      -9.4394221534410804e-03, 
       2.0000000000000002e-05, 
      -3.0000000000000001e-05, 
      -1.6688979891507517e-02, 
       1.7659831748780288e+00, 
       1.1346109846345060e+00, 
      -2.2398673850249746e-03, 
      -6.0996170996806417e-04, 
       1.2896461667223058e-01, 
       1.2386484072228615e-01, 
      -3.5998171919831093e-04, 
      -6.0000000000000002e-05, 
       9.8496042942762384e-03, 
       1.2109518146664864e-02, 
      -5.0000000000000002e-05, 
       7.5996677052105218e-04, 
       1.1099562278983118e-03, 
       6.0000000000000002e-05, 
       1.0000000000000000e-04, 
       6.9999999999999994e-05, 
       2.0000000000000002e-05, 
       9.2000090479789798e-04, 
       2.0000172884558686e-04, 
      -3.0000000000000001e-05, 
       1.1269992293506328e-02, 
       2.2999988051376295e-03, 
      -1.4000086442279340e-04, 
       1.3380969738450182e-01, 
       2.4959940627996887e-02, 
      -2.0000000000000002e-05, 
       2.2999613153606939e-04, 
       1.5156341196347713e+00, 
       2.5407897826836784e-01, 
       2.3199493266433456e-03, 
       2.0000000000000002e-05, 
      -5.0000000000000002e-05, 
       2.0449530411600761e-02, 
       1.5566265411600057e+01, 
      -1.6244211504417858e+00, 
      -6.5608499340100476e-02, 
       1.1999613070201107e-04, 
      -4.5998065351005534e-04, 
       3.2906197559064598e-01, 
       1.6657411651352464e+02, 
       1.9948376111198255e+02, 
      -2.4483409732634207e-01, 
       3.7998452280804429e-04, 
      -2.2099032667162182e-03, 
       2.1863132930572466e+00, 
       6.2365248530516510e+02, 
       1.1726069661889859e+02, 
      -1.4452645931932773e-01, 
       2.5998839210603319e-04, 
      -7.4996991003888199e-04, 
      -2.9115289842970743e-01, 
       3.3357202973080064e+01, 
       1.5121554333970961e+01, 
      -3.0379291843547287e-02, 
       6.9999999999999994e-05, 
       8.0000000000000007e-05, 
      -9.9497680019529668e-03, 
       2.1461691617920584e+00, 
       1.5197527715089008e+00, 
      -4.5099012045118504e-03, 
       1.0000000000000001e-05, 
      -9.2998110632603356e-04, 
       1.4641951872491479e-01, 
       1.3794956422367508e-01, 
      -5.5998711524236618e-04, 
      -8.0000000000000007e-05, 
       1.0269980564904076e-02, 
       1.1859978046964528e-02, 
      -6.0000000000000002e-05, 
       7.2999831194654687e-04, 
       9.9000003995807547e-04, 
       5.0000000000000002e-05, 
       8.0000000000000007e-05, 
       3.0000000000000001e-05, 
       2.7999032180005306e-04, 
       5.0000000000000002e-05, 
       2.4699041754397662e-03, 
       3.3999032138302396e-04, 
      -2.0000000000000002e-05, 
       1.8179263590004310e-02, 
       1.6199348116503339e-03, 
      -1.4999999999999999e-04, 
       7.9126692560293324e-02, 
       5.4287721206722483e-02, 
       1.6999613195309854e-04, 
      -3.4197934884254719e-03, 
      -7.9101527986961173e-01, 
      -1.3178223087945189e+00, 
       1.9198856695799263e-03, 
       5.0000000000000002e-05, 
      -5.4566580553490521e-02, 
      -1.2094151282860922e+01, 
      -1.2642739063331361e+00, 
       1.7698886715359176e-03, 
       2.0000000000000002e-05, 
       4.6497160418536602e-03, 
      -8.2271407280647579e-01, 
      -2.3700982081636493e-01, 
       5.1997171416615843e-04, 
       2.0998999538487663e-04, 
      -6.2827365240563343e-02, 
      -3.1418700507997430e-02, 
       1.0000000000000000e-04, 
       2.0000000000000002e-05, 
      -4.9597975766249259e-03, 
      -3.5498583509965187e-03, 
       2.0000000000000002e-05, 
      -3.9999032221708225e-04, 
      -3.6999032180005308e-04, 
      -3.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -6.9999999999999994e-05, 
      -1.0000000000000001e-05, 
      -1.0199238228840821e-03, 
      -3.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -1.2619004970617137e-02, 
      -5.0396057314673424e-03, 
       4.0000000000000003e-05, 
      -4.5995157936916831e-04, 
      -1.0534151676616781e-01, 
      -1.1329088055547008e-01, 
       1.1999386468286557e-04, 
      -1.0299014307268081e-03, 
      -1.3413898665326490e-01, 
      -1.4818829888765636e-02, 
       3.0000000000000001e-05, 
      -6.0000000000000002e-05, 
      -6.1695030505202883e-03, 
      -1.4498910050119490e-03, 
      -2.4998245847139013e-04, 
      -8.9993864682865569e-05, 
      -5.9993864682865558e-05, 
      -2.0000000000000002e-05, 
      -8.9987729365731119e-05, 
      -5.0000000000000002e-05, 
       3.6995705236302973e-04, 
       4.0000000000000003e-05, 
       5.9993864682865558e-05, 
      -3.9999999980881819e-05, 
      -2.9999999990440909e-05, 
      -5.9999999980881817e-05, 
      -1.0000000000000001e-05, 
      -1.3998772930837655e-04, 
      -1.5998772930837657e-04, 
      -1.0099203648201037e-03, 
      -6.2994791109766428e-04, 
      -3.9999999980881819e-05, 
      -1.3098902244592227e-03, 
       7.9999999971322724e-05, 
      -1.3998772930837655e-04, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -2.1999472902918624e-04, 
      -2.8198811240780107e-03, 
      -3.2898605775021419e-03, 
      -8.9999999961763637e-05, 
      -2.1039078775602640e-02, 
      -1.3559414141111607e-02, 
       2.0000000000000002e-05, 
      -7.5994997622058126e-04, 
      -2.5718835140798812e-02, 
      -4.1999636551716586e-04, 
       1.0000000000000001e-05, 
      -2.7498838000645318e-03, 
      -2.9998945601146298e-04, 
      -2.6998859327590445e-04, 
      -6.9999999971322711e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -1.6000086436543886e-04, 
      -2.9999999990440909e-05, 
      -2.8699906122380261e-03, 
       1.4999999994264542e-04, 
       1.0000000000000001e-05, 
      -7.9999999971322724e-05, 
      -3.9409787796614117e-02, 
      -4.8519742795838566e-02, 
       1.5999613064465650e-04, 
      -1.2799669042687029e-03, 
      -3.0516793001571280e-01, 
      -2.0592872468806117e-01, 
       3.7999226042657739e-04, 
      -1.0089736878568208e-02, 
      -3.5182706645119044e-01, 
      -2.8399990057422268e-02, 
       1.5999613064465650e-04, 
       2.0000000000000002e-05, 
      -3.6109762019961189e-02, 
      -6.5799908815186506e-03, 
       3.9999999980881819e-05, 
      -3.2199841462111560e-03, 
      -1.0799974442030676e-03, 
      -2.7000172832340763e-04, 
      -1.3999999995220451e-04, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -6.9999999971322711e-05, 
      -1.0000000000000001e-05, 
      -4.3998332189762061e-04, 
       3.9999999990440911e-05, 
      -1.5199383564713348e-03, 
      -2.6098863929636786e-03, 
       2.9999999990440909e-05, 
       1.3209424696369491e-02, 
       2.0829098791581825e-02, 
      -4.9999999980881818e-05, 
       3.0997999065504410e-04, 
       3.4358452062718495e-02, 
       1.3509405503575258e-02, 
      -2.9999999990440909e-05, 
       4.2598195678548063e-03, 
       2.5798946704204497e-03, 
       4.3998332273167891e-04, 
       3.4998945766046149e-04, 
       3.9999999980881819e-05, 
       3.9999999980881819e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -4.9999999980881818e-05, 
      -9.9993864644629200e-05, 
      -5.5995318327176991e-04, 
      -1.5998772930837657e-04, 
      -8.9993864654188286e-05, 
      -3.9999999980881819e-05, 
      -1.0000000000000001e-05, 
       1.0004406442870324e-05, 
       1.1406618945528380e-08, 
      -1.0000000000000001e-05, 
       3.0000000000000001e-05, 
       6.9987729365731107e-05, 
       6.9987729365731107e-05, 
       7.9987729365731119e-05, 
       3.9993864682865560e-05, 
       1.0000000000000001e-05, 
       1.2998772936573112e-04, 
       5.8993036980803176e-04, 
       1.6498030536431869e-03, 
       1.3398437348439828e-03, 
       1.0000000000000001e-05, 
       1.6098065952749123e-03, 
       7.9990722930178761e-04, 
       2.3997545831443308e-04, 
       1.6998159363156755e-04, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
      -3.0000000000000001e-05, 
      -6.0000000000000002e-05, 
       1.0999386468286557e-04, 
      -2.0000000000000002e-05, 
       2.8297768230701090e-03, 
       1.2299032515509782e-02, 
      -4.0000000000000003e-05, 
      -5.0000000000000002e-05, 
       3.4617218176446002e-02, 
       2.3798095993400669e-02, 
      -5.0000000000000002e-05, 
       2.8996772013548436e-04, 
       2.8987641176711369e-02, 
       1.2698980216260428e-02, 
      -3.0000000000000001e-05, 
      -4.0000000000000003e-05, 
       4.1696670459316987e-03, 
       2.5497996329908851e-03, 
       4.6996104639370379e-04, 
       3.6997718632596162e-04, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -9.0000000000000006e-05, 
      -5.7998118160772314e-04, 
       2.0000000000000002e-05, 
      -5.0000000000000002e-05, 
      -8.9996590568122952e-04, 
       1.9099256735431497e-03, 
      -1.1000000000000000e-04, 
      -3.2998085477551752e-04, 
       5.2507790558040605e-02, 
       2.1375102827470735e-01, 
      -5.9996170955103499e-04, 
      -6.4996084429418328e-04, 
       5.9577403047192357e-01, 
       3.3880542107838019e-01, 
      -7.4995783941898787e-04, 
      -1.0000000000000001e-05, 
       5.3396643697027181e-03, 
       4.1494151556630338e-01, 
       1.5790314283804993e-01, 
      -3.6997998993569493e-04, 
      -6.1996084512824161e-04, 
       5.6857563054754208e-02, 
       3.0088730588867105e-02, 
      -9.0000000000000006e-05, 
      -1.0000000000000000e-04, 
       6.1397442904511292e-03, 
       4.2098290710028486e-03, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       5.9997504587355950e-04, 
       4.9998031676790067e-04, 
       6.0000000000000002e-05, 
       5.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -1.5000172884558683e-04, 
      -1.0000000000000001e-05, 
      -1.1599978986716151e-03, 
      -6.9999999999999994e-05, 
      -6.7999851699930194e-03, 
       2.9000172884558682e-04, 
       3.0000000000000001e-05, 
      -6.5998797966515955e-04, 
      -7.7099705155976653e-03, 
       2.1739908684458877e-02, 
      -1.2699712256095371e-03, 
      -3.8999114943329871e-03, 
       6.3370660627407005e-01, 
       2.4138773681885621e+00, 
      -6.3198582256228080e-03, 
       1.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -5.5398654660233524e-03, 
       6.5795750374310860e+00, 
       2.9984814622084768e+00, 
      -6.6398461619936577e-03, 
       2.0000000000000002e-05, 
      -1.2999613070201104e-04, 
       6.2568393649412343e-02, 
       3.6744619522344233e+00, 
       1.1918723424246769e+00, 
      -2.8799333924651202e-03, 
      -5.5198624690717107e-03, 
       4.7337716127890955e-01, 
       2.1258889463466799e-01, 
      -6.7998711440830798e-04, 
      -8.9997937664638842e-04, 
       4.8339796122932280e-02, 
       2.8279898471952964e-02, 
      -1.2000000000000000e-04, 
      -1.0000000000000000e-04, 
       4.4899885073929379e-03, 
       3.2299910602862142e-03, 
      -2.0000000000000002e-05, 
       4.0000259243432197e-04, 
       3.4000259243432198e-04, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
       6.2997418061670795e-04, 
       1.9199256706239455e-03, 
       3.0000000000000001e-05, 
      -2.0299139688574517e-03, 
      -2.9568742050766070e-02, 
       9.0000000000000006e-05, 
      -1.4999999958297084e-04, 
      -1.7190245191781617e-01, 
      -5.0967800232571393e-02, 
       1.2000000000000000e-04, 
      -3.0398121745785389e-03, 
      -1.1307496152169749e-01, 
      -2.5488897251826612e-02, 
       6.9999999999999994e-05, 
       1.1000000000000000e-04, 
      -1.6919273441755856e-02, 
      -5.5797675136468380e-03, 
       2.0000000000000002e-05, 
       3.0000000000000001e-05, 
      -1.9599178102144869e-03, 
      -8.7996504042437784e-04, 
      -1.9999472868862983e-04, 
      -1.2000000000000000e-04, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -3.0000000000000001e-05, 
      -1.9998859337149540e-04, 
       1.0000000000000001e-05, 
      -1.1399098217361096e-03, 
      -2.7697829563020976e-03, 
       5.0000000000000002e-05, 
      -4.3796530447837269e-03, 
      -1.7898599982578381e-03, 
      -5.5995491107656942e-04, 
      -2.4998245763733183e-04, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       7.9987729365731119e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
      -3.9999999980881819e-05, 
      -3.9999999980881819e-05, 
      -9.9993864644629200e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -3.9999999990440911e-05, 
      -1.3999999995220451e-04, 
      -5.6997945255183625e-04, 
      -6.0997331638152534e-04, 
      -1.0000000000000001e-05, 
      -1.2699418994401791e-03, 
      -2.8998859243228707e-04, 
      -4.9999999980881818e-05, 
      -3.1998772799993457e-04, 
      -5.9999999980881817e-05, 
      -1.0000000000000001e-05, 
      -3.3000172830428945e-04, 
      -1.2399944405216048e-03, 
      -5.0499738511240013e-03, 
      -5.1899747121532079e-03, 
       2.0000000000000002e-05, 
      -1.0999999996176364e-04, 
      -1.0489932263047171e-02, 
      -2.4499884236494698e-03, 
       1.0000000000000001e-05, 
      -4.0998839153605853e-04, 
      -2.5399828263017159e-03, 
      -2.7000259108764355e-04, 
      -2.9999999990440909e-05, 
      -4.4999785813400580e-04, 
      -5.9999999563852654e-05, 
      -5.9999999980881817e-05, 
      -1.0000000000000001e-05, 
      -1.0000000000000001e-05, 
      -4.9999999980881818e-05, 
       1.3999999994264542e-04, 
       4.0998945764134326e-04, 
       9.6995717631544025e-04, 
       5.2997858772157276e-04, 
       3.0000000000000001e-05, 
       5.0997772372536761e-04, 
       2.2999386417980457e-04, 
       9.9999999961763637e-05, 
       4.9999999980881818e-05, 
       1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.9993864682865558e-05, 
       9.9987729365731118e-05, 
       5.9993864682865558e-05, 
       5.0000000000000002e-05, 
       3.0000000000000001e-05, 
       1.0000000000000001e-05, 
       6.9999999999999994e-05, 
       3.2997718674299077e-04, 
       8.7993349941591233e-04, 
       8.6993436342167648e-04, 
       1.0000000000000001e-05, 
       1.3698928165469999e-03, 
       7.7993876989322414e-04, 
       6.6994791050258332e-04, 
       3.1997545748037483e-04, 
       1.2998772936573112e-04, 
       6.9999999999999994e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       9.0000000000000006e-05, 
       8.8996590484717120e-04, 
       4.1498316651052873e-03, 
      -2.0000000000000002e-05, 
       1.0909542893541542e-02, 
       9.3896118576569758e-03, 
      -3.0000000000000001e-05, 
       1.3999999999999999e-04, 
       1.4859362620021506e-02, 
       7.5696827235034493e-03, 
      -3.0000000000000001e-05, 
       1.0000000000000000e-04, 
       6.5197169059330621e-03, 
       2.8798802536172223e-03, 
      -1.0000000000000001e-05, 
       1.2199488988240033e-03, 
       6.2997418019967880e-04, 
       1.6999386426583643e-04, 
       1.0000000000000000e-04, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       1.4000086442279340e-04, 
       7.1999658143284348e-04, 
      -2.0000000000000002e-05, 
       7.0199747919151498e-03, 
       3.1179884630220266e-02, 
      -1.3999999999999999e-04, 
       6.9999999999999994e-05, 
       8.0959591085278074e-02, 
       5.9629735618255422e-02, 
      -2.0999613028498191e-04, 
       1.0699746420915477e-03, 
       9.4029434978090118e-02, 
       4.2169792332154843e-02, 
      -1.3999999958297084e-04, 
       5.9998452113992757e-04, 
       3.6739771859050800e-02, 
       1.4649926765324299e-02, 
      -5.0000000000000002e-05, 
      -5.0000000000000002e-05, 
       6.5399648279462685e-03, 
       3.0499880112826523e-03, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
       8.7999571659302088e-04, 
       4.7999872230227467e-04, 
       1.0000000000000000e-04, 
       6.0000000000000002e-05, 
       1.0000000000000001e-05, 
       3.0000000000000001e-05, 
      -3.0000000000000001e-05, 
      -5.3998031801898811e-04, 
      -2.5598925309239578e-03, 
      -1.3699444940205724e-03, 
      -6.9999999999999994e-05, 
      -3.4998471114150585e-03, 
      -1.1599527689560506e-03, 
      -4.0000000000000003e-05, 
      -1.5299357633499118e-03, 
      -4.6998332122606684e-04, 
      -3.0998772853167283e-04, 
      -1.1000000000000000e-04, 
      -5.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -5.0000000000000002e-05, 
      -8.9993864682865569e-05, 
      -6.0000000000000002e-05, 
      -5.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -1.0000000000000001e-05, 
      -2.0000000000000002e-05, 
      -3.9999999980881819e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -3.0000000000000001e-05, 
      -8.9999999971322723e-05, 
      -1.0999999996176364e-04, 
      -2.3999999907990986e-04, 
      -9.9999999961763637e-05, 
      -1.0000000000000001e-05, 
      -1.2999999953517536e-04, 
      -2.9999999990440909e-05, 
      -2.9999999990440909e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       4.0000000000000003e-05, 
       3.0000000000000001e-05, 
       4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       6.9999999999999994e-05, 
       1.8999472868862983e-04, 
       2.1999472827160065e-04, 
       3.7998945696023052e-04, 
       2.4999472827160070e-04, 
       2.8998772811464368e-04, 
       1.4999999916594166e-04, 
       1.0999999958297084e-04, 
       5.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       9.0000000000000006e-05, 
       4.2000259201729282e-04, 
       1.0699974446045496e-03, 
       1.0799974441875204e-03, 
       2.0000000000000002e-05, 
       1.8899914319349541e-03, 
       1.1099965789306686e-03, 
       2.0000000000000002e-05, 
       1.2999918447928571e-03, 
       5.9999872146821647e-04, 
       4.5000086275467677e-04, 
       1.9999999916594168e-04, 
       9.0000000000000006e-05, 
       4.0000000000000003e-05, 
       1.0000000000000001e-05, 
      -4.0000000000000003e-05, 
      -3.0000000000000001e-05, 
      -8.0000000000000007e-05, 
      -4.0000000000000003e-05, 
      -6.0000000000000002e-05, 
      -2.0000000000000002e-05, 
      -2.0000000000000002e-05, 
       1.0000000000000001e-05, 
       2.0000000000000002e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       3.8500052903228404e+05, 
      -3.1483021922752781e+00, 
      -2.9999999997633791e-05, 
       3.7998579755659352e-04, 
       7.9660570476435268e+01, 
      -2.0905354935205090e+04, 
      -1.0325754836510391e-01, 
      -8.3697366020164584e-03, 
      -4.4211771171071561e+00, 
      -5.6992511533509469e+02, 
      -1.0199309130843224e-03, 
       3.2999012092559170e-04, 
      -1.1100043220264174e-03, 
      -2.3210429822877586e+01, 
       2.6000518651625334e-04, 
       1.9999999998422528e-05, 
       2.7000172882429092e-04, 
      -1.1169370611059974e+00, 
       3.9999999996845056e-05, 
       1.9999999998422528e-05, 
      -5.8880462467805837e-02, 
      -3.2900311189610690e-03, 
      -1.9000172883060085e-04, 
      -1.0999999999132389e-04, 
      -1.7399524762050226e-03, 
      -2.6739186021817699e-02, 
      -4.2240654624048563e-01, 
       1.4999999998816893e-04, 
      -1.8999999998501401e-04, 
      -7.0026958822396912e+00, 
      -5.0197444683412334e-02, 
      -6.9999999994478830e-05, 
      -7.6895896701093900e-03, 
      -1.2962022217205057e+02, 
       3.3463244089062183e-01, 
       2.9999999997633791e-05, 
      -1.8566992299900853e-01, 
       4.8888328059174839e+01, 
      -1.5802147102879510e-01, 
       2.9999999997633791e-05, 
      -2.4809697845850509e-01, 
       1.0475529267427027e+02, 
      -6.8196431963242548e-03, 
       4.9999999996056319e-05, 
       3.2648340142780188e-02, 
       5.7508577057015726e+00, 
      -2.9999999997633786e-04, 
      -7.9995730218233017e-04, 
       3.5507868338050058e-01, 
      -5.9999999995267581e-05, 
       2.3039298527847120e-02, 
       1.5299568826931335e-03, 
       9.9999999992112637e-05, 
      -4.9999999996056319e-05, 
      -6.5995523869077661e-04, 
      -9.3793569172097267e-03, 
      -1.3617045669341821e-01, 
      -9.8991435536443227e-04, 
      -3.0997545870701143e-04, 
      -2.1171282169327208e+00, 
       4.4995982169380262e-03, 
      -5.5794921950237464e-03, 
       1.0656722650829162e+00, 
      -3.4196904093387018e-03, 
      -2.3697891063466005e-03, 
       1.1655363585022664e+00, 
      -1.0999999999132389e-04, 
       1.7998772935153386e-04, 
       5.0716436611946462e-02, 
       2.2798455837538525e-03, 
       8.9999999992901368e-05, 
      -2.0997545871489877e-04, 
      -2.8696957386574184e-03, 
      -1.9999999998422528e-05, 
      -3.8935784344622658e-02, 
       6.9999999994478830e-05, 
      -1.6998159403518818e-04, 
       2.1267663221308259e-02, 
      -6.9999999994478830e-05, 
      -2.9999999997633791e-05, 
       1.9117925823248460e-02, 
       7.7991669876697665e-04, 
       2.9999999997633791e-05, 
      -5.9999999995267581e-05, 
      -7.2989569871707838e-04, 
       3.3995091743610744e-04, 
       3.6994478211660676e-04, 
      -5.9999999976149397e-05, 
       3.4996318794531937e-04, 
      -1.6098220750738919e-03, 
       3.9999999996845056e-05, 
       2.8997545860343885e-04, 
       8.4393950201855413e-03, 
      -2.3998159394363516e-04, 
      -2.9477842955863667e-02, 
      -2.9999999997633791e-05, 
      -1.9999999998422528e-05, 
       1.5898860148522863e-03, 
       3.5997545731815292e-04, 
       3.9999999996845056e-05, 
       9.9999999953876269e-05, 
       6.4997891613239060e-04, 
       1.9999999998422528e-05, 
      -4.2198517073415813e-03, 
      -7.9999999965012830e-05, 
      -1.0999999995308752e-04, 
       1.1410603535319587e-01, 
       2.0999999990696381e-04, 
      -1.3659260861906745e-02, 
       4.9755215400672276e-01, 
      -7.3996384925082781e-04, 
       4.3997385525540976e-04, 
       1.0997620731914327e-01, 
      -6.9999999965801547e-05, 
      -2.9999999997633791e-05, 
       1.1209625212771860e-02, 
       9.6996750810329085e-04, 
       7.9999999965012830e-05, 
      -2.9999999997633791e-05, 
      -4.3000345749475338e-04, 
      -6.3600397530792401e-03, 
      -1.8999999991810035e-04, 
      -9.9380481230041715e-02, 
      -4.7999485445292023e-04, 
      -2.7799622097322992e-03, 
      -1.7385258128996706e+00, 
       4.5699364414918571e-03, 
      -4.2319342957746774e-02, 
      -8.3791066502014502e+00, 
      -2.4709621847278974e-02, 
      -7.9562638278438003e-01, 
       1.0874270127246305e+02, 
      -1.8739690056048076e-02, 
       1.6999999992923693e-04, 
       6.0549065378075952e-02, 
       6.3220031516918844e+00, 
      -1.6399763697789705e-03, 
       2.0299712263623713e-03, 
       3.7852141276755424e-01, 
      -1.2999999994195094e-04, 
       1.0999999995308752e-04, 
       2.3680120083704129e-02, 
       1.5200095067157658e-03, 
       9.9999999953876269e-05, 
       5.9999999976149397e-05, 
       1.0099692386042803e-03, 
       1.7289444030638344e-02, 
       1.3999999994116218e-04, 
       7.9999999965012830e-05, 
       3.3224888279882397e-01, 
      -1.4799220185026741e-03, 
       1.5599202896221000e-03, 
       8.5124033619353401e-01, 
       3.3098244400641009e-03, 
       3.3788197501593308e-02, 
      -1.6674723746336664e+01, 
      -5.9999999976149397e-05, 
      -5.6197041728912808e-03, 
      -9.3331755043545517e-01, 
      -3.6998772836866227e-04, 
      -5.9038041580494909e-02, 
      -1.9999999998422528e-05, 
      -3.9198779032451961e-03, 
      -2.7000172872869996e-04, 
       1.9999999998422528e-05, 
       2.7998159392136196e-04, 
       3.2397710884068728e-03, 
       4.3796878710824355e-03, 
      -1.9999999998422528e-05, 
       3.7287275130270782e-02, 
       1.9999999998422528e-05, 
       6.4595388101637554e-03, 
       6.6995351042992309e-04, 
       5.9999999976149397e-05, 
       5.9999999976149397e-05, 
       7.9999999965012830e-05, 
       5.3993864658532283e-04, 
       6.9999999965801547e-05, 
      -1.9999999998422528e-05, 
      -1.1997545872199740e-04, 
      -5.7989569831188030e-04, 
      -3.9999999996845056e-05, 
       3.5994478211739549e-04, 
      -4.2593811093623989e-03, 
       2.9999999997633791e-05, 
      -1.5777680821354629e-02, 
      -1.1298360731086838e-03, 
      -8.9987729358632481e-05, 
       1.9999999998422528e-05, 
       4.5995264627222938e-04, 
       1.1158808366184655e-02, 
       2.9999999997633791e-05, 
       2.5996932339382065e-04, 
      -1.4806393807175233e-01, 
       6.9999999994478830e-05, 
       1.0398640811513846e-03, 
      -4.1071485480076358e-01, 
       2.6996932339303192e-04, 
      -2.8546907073008249e-02, 
       2.9999999997633791e-05, 
      -2.0897737689961008e-03, 
      -1.5998159403597691e-04, 
       2.9999999997633791e-05, 
       6.6995523952404629e-04, 
       1.9999999998422528e-05, 
       1.5168961243902336e-02, 
       4.0996932338198958e-04, 
       3.4301591248131019e-01, 
       9.8991435661551982e-04, 
       9.5691427477273047e-03, 
      -4.9501348002130721e+00, 
       1.6798497951639492e-03, 
       2.8387413047911467e-02, 
      -9.8844666217709616e+00, 
      -2.9999999997633791e-05, 
       7.9192899476857939e-03, 
      -6.5753295021873692e-01, 
       8.3992576200476803e-04, 
      -4.6076771497468638e-02, 
       5.9999999995267581e-05, 
      -3.3497710052450623e-03, 
      -2.4998332287446507e-04, 
      -1.9999999998422528e-05, 
       5.9999999995267581e-05, 
       1.1899665613369270e-03, 
       3.9999999996845056e-05, 
       2.3339287993958329e-02, 
       7.8995730385123550e-04, 
       4.9504416423055553e-01, 
      -4.5997558426192373e-04, 
       1.5179225244421564e-02, 
       1.0056203077190446e+01, 
       3.0238458352181256e-02, 
       3.2334304986755269e-01, 
      -1.5213780942589071e+02, 
       3.7338055220178717e-02, 
      -3.4997612144415855e-04, 
       6.5706432249497604e-01, 
      -2.0458611646087303e+02, 
       1.0799619981539893e-03, 
      -8.4993836896337369e-04, 
       2.0940900061519149e-01, 
      -1.2831403332461623e+01, 
       1.7288455868366050e-09, 
       1.2999999998974642e-04, 
       1.8039081008914867e-02, 
      -8.4880182533646864e-01, 
       1.2699337512832878e-03, 
      -5.8498146534920061e-02, 
       8.9999999992901368e-05, 
      -4.1298734075970499e-03, 
      -3.0000259324471811e-04, 
      -1.9999999998422528e-05, 
       2.0000259325260553e-04, 
       3.0400285232102318e-03, 
       1.1999999999053516e-04, 
       4.7400373388124431e-02, 
      -1.9999999998422528e-05, 
       1.8799811461358263e-03, 
       7.7854492106106743e-01, 
       4.7999571822327815e-04, 
       2.9099632684995498e-02, 
       1.4402688569714263e+01, 
      -2.0079749835047428e-02, 
      -1.7999999998580274e-04, 
      -1.9999999998422528e-05, 
       4.7262331352426273e-01, 
       2.4615847485355792e+02, 
       7.7403905203800971e-01, 
      -1.3999999998895766e-04, 
      -3.8998452277728355e-04, 
       8.7515627423090230e+00, 
      -3.6991108963980764e+03, 
       5.9632094424355830e-01, 
      -7.4497493960877454e-03, 
       1.0321107477029065e+01, 
      -2.9559675579724144e+03, 
       3.1429705179098821e-02, 
      -1.8659382347128925e-02, 
       4.1311146504675733e+00, 
      -1.7073307712477057e+02, 
       1.8099967120070661e-03, 
       2.1899295292498355e-03, 
       2.8398596703683637e-01, 
      -1.0444757567465345e+01, 
       1.4000259325733791e-04, 
       1.9999999998422528e-05, 
       1.8109768286069500e-02, 
      -6.6968349796415183e-01, 
       1.1899884332213365e-03, 
      -4.4220290390859181e-02, 
       7.9999999993690113e-05, 
      -2.9800242002669388e-03, 
      -2.0000172882981212e-04, 
       8.9999999992901368e-05, 
       1.1599665580030599e-03, 
       2.9999999997633791e-05, 
       1.4169567487434505e-02, 
       2.0999999914937822e-04, 
       1.6857464438507141e-01, 
       2.9998772934206898e-04, 
       5.1997384415181720e-04, 
       1.4367546693920624e-01, 
      -2.0298946113646351e-03, 
      -4.7357511109689159e-02, 
       2.4208498126462157e+01, 
      -5.8796967594376198e-03, 
       1.1999999999053516e-04, 
      -1.3571262803566506e-01, 
       3.0823859669312185e+01, 
      -1.4299250222681243e-03, 
       3.9997225214222595e-04, 
      -5.5267094626872112e-02, 
       2.6164091256911894e+00, 
      -1.4999999998816893e-04, 
      -4.9999999996056319e-05, 
      -5.3697280934811922e-03, 
       2.1251296078252663e-01, 
      -4.3997558426350119e-04, 
       1.7039459745806843e-02, 
      -3.9999999996845056e-05, 
       1.3499595590822043e-03, 
       1.0999999999132389e-04, 
      -4.0997191665036983e-04, 
      -2.9999999997633791e-05, 
      -1.0359283208969236e-02, 
      -5.4995104298628740e-04, 
      -1.0887233560054604e-01, 
      -4.5995544946493360e-04, 
      -1.0719047501313196e-02, 
       2.3536309333839593e+00, 
      -3.9999999996845056e-05, 
       7.9992662643071628e-04, 
       1.4762958932276996e-01, 
       1.8698379332023052e-03, 
       9.1193710011059064e-03, 
       9.9999999992112637e-05, 
       3.5997805197144798e-04, 
      -9.9999999992112637e-05, 
      -5.1994651011630431e-04, 
      -2.9999999997633791e-05, 
       2.0797755805319405e-03, 
      -2.3997545787847422e-04, 
       1.9999999998422528e-05, 
       2.9999999997633791e-05, 
      -2.9999999997633791e-05, 
       7.9999999965012830e-05, 
       6.4992637465305402e-04, 
       2.9999999997633791e-05, 
      -5.4993864658453410e-04, 
      -3.9999999996845056e-05, 
      -1.5998772929575679e-04, 
       2.2398412595603114e-03, 
      -4.9999999976938134e-05, 
       1.3889003983154241e-02, 
       6.9993930925565061e-04, 
      -1.0479229613358202e-02, 
       1.9999999998422528e-05, 
      -6.6995091716154278e-04, 
      -2.9999999997633791e-05, 
      -1.1999999994273969e-04, 
      -3.3398928496763268e-03, 
      -1.3999999994116218e-04, 
       5.2928221875853429e-02, 
      -5.9999999976149397e-05, 
      -8.8995384582716522e-04, 
       2.5560112481644193e-01, 
       1.4409224712398310e-02, 
      -1.5707413311609475e-01, 
       4.8997385523234793e-04, 
      -8.8696764674312967e-03, 
       2.9999999997633791e-05, 
      -2.9999999986162873e-04, 
      -2.9999999997633791e-05, 
      -1.7000103723085354e-03, 
      -4.9999999996056319e-05, 
      -4.8430210614243123e-02, 
      -1.2999999994195094e-04, 
      -2.0599712263504460e-03, 
       8.6243277191744838e-01, 
      -1.0099879787956634e-03, 
      -8.6798611953819403e-03, 
       3.2582370457252896e+00, 
      -5.9999999976149397e-05, 
      -1.7999999991888908e-04, 
       2.0098657627468239e-01, 
      -1.4189284149249448e+00, 
      -1.4999999993081436e-04, 
       4.9999999976938134e-05, 
       6.7098942902756283e-03, 
      -6.5449921480596931e-02, 
      -1.9999999998422528e-05, 
       3.4999999983856690e-04, 
      -1.1099974149906414e-03, 
       1.5999999867893818e-04, 
       2.9999999997633791e-05, 
      -4.9999999996056319e-05, 
      -6.4997977930409659e-04, 
      -6.7997804204870567e-03, 
      -1.9999999998422528e-05, 
      -8.9999999954665000e-05, 
       3.8098713336470330e-02, 
       6.9999999965801547e-05, 
      -2.2999999989582721e-04, 
      -2.1258262202509592e-01, 
       5.9999999976149397e-05, 
      -8.0795662014067418e-03, 
      -1.0765620159727074e-01, 
      -1.7999999991888908e-04, 
      -1.4479504321496651e-02, 
      -1.4999508303474602e-03, 
      -1.3999999993160309e-04, 
      -6.9999999965801547e-05, 
       3.1997545650636777e-04, 
       9.8992810667422994e-04, 
       8.9987729320396113e-05, 
       1.6198834265763306e-03, 
       2.7998159392136196e-04, 
       2.9999999997633791e-05, 
      -2.5919503791250381e-09, 
      -3.9999999996845056e-05, 
       9.9987729319607382e-05, 
       1.9980180920396742e-05, 
      -3.9999999996845056e-05, 
      -1.9999999998422528e-05, 
      -2.1996932339697557e-04, 
      -7.7988342682777840e-04, 
      -4.4993251022494053e-04, 
      -5.9999999995267581e-05, 
      -1.0998772935705501e-04, 
      -5.1294519652774432e-03, 
      -5.9999999995267581e-05, 
      -1.5708296600154038e-02, 
      -2.1997545871411001e-04, 
      -8.1691050005996118e-03, 
      -1.1398737481824078e-03, 
      -1.2998772935547754e-04, 
      -1.9999999998422528e-05, 
      -1.1999999999053516e-04, 
      -2.9999999997633791e-05, 
      -2.3898333081065581e-03, 
       3.9999999996845056e-05, 
      -3.5997545870306773e-04, 
      -1.0833240536198517e-01, 
       1.0999999999132389e-04, 
      -1.4498750753066776e-03, 
      -2.7904005944795895e-01, 
       4.9999999996056319e-05, 
      -4.0296358495893600e-03, 
      -1.2805074866085511e-01, 
       2.2998159403045574e-04, 
      -1.7278774975208779e-02, 
       4.9999999996056319e-05, 
      -1.8398719332559123e-03, 
      -1.7998772935153386e-04, 
      -3.9999999996845056e-05, 
      -4.0999118702456658e-04, 
      -4.9999999996056319e-05, 
      -2.2999287980640212e-03, 
      -5.8996644322528203e-04, 
      -4.3158622600503828e-02, 
       7.6996117231674355e-04, 
      -6.6096631128729029e-03, 
      -1.8970367440031213e+00, 
       1.5999167452340917e-03, 
      -5.9999999995267581e-05, 
      -2.6978591089253386e-02, 
      -3.9579826757930441e+00, 
       6.8996644280036547e-04, 
       2.9999999997633791e-05, 
      -5.7886882828892962e-02, 
      -1.5713943644518369e+00, 
       7.9999999993690113e-05, 
       4.9999999996056319e-05, 
       2.7198614490906497e-03, 
      -2.0285309525301440e-01, 
       5.4997558342076685e-04, 
      -2.0789318692135777e-02, 
       5.9999999995267581e-05, 
      -1.9399367441105431e-03, 
      -1.6999999998659149e-04, 
      -4.9999999996056319e-05, 
      -5.5000518565932169e-04, 
      -5.9999999995267581e-05, 
      -5.4200423475317698e-03, 
      -7.1999271291212379e-04, 
      -2.9390184947878158e-02, 
      -8.9999999992901368e-05, 
      -7.4399077426542878e-03, 
      -5.1423241968031275e-01, 
       9.2498845670986977e-03, 
      -1.3999999998895766e-04, 
      -7.9818881687893625e-02, 
      -2.1636343445651164e+01, 
       1.4829797329931681e-02, 
      -7.9997291401692023e-04, 
      -3.2175507677622006e-01, 
      -3.4782523344715486e+01, 
       5.4999214095694049e-03, 
       3.1998839208079367e-04, 
      -5.0792132716165672e-01, 
      -1.1649947731911631e+01, 
       6.6999271333309670e-04, 
       4.9998452276860735e-04, 
       1.9379704871355419e-02, 
      -1.4225541054826598e+00, 
       6.9999999994478830e-05, 
       3.9999450408357258e-03, 
      -1.3922061526530222e-01, 
       4.3999485463769793e-04, 
      -1.2430067394388417e-02, 
       3.9999999996845056e-05, 
      -1.0600077772193598e-03, 
      -8.9999999992901368e-05, 
      -3.9999999996845056e-05, 
       9.2996923689824906e-04, 
      -2.3989246527191473e-02, 
      -6.9999999994478830e-05, 
       3.5998773142248240e-04, 
       2.3695196220855216e-01, 
      -2.1999999998264779e-04, 
       1.5509193615448615e-02, 
       5.7878979678519848e-01, 
      -1.1999999999053516e-04, 
       1.9688939362482307e-02, 
       2.4452135952649218e-01, 
      -2.9999999997633791e-05, 
      -2.9999999997633791e-05, 
       1.2999999873865894e-04, 
       3.6818750368665838e-02, 
      -7.9999999993690113e-05, 
       4.2998577586114287e-03, 
       4.4998418603039634e-04, 
       3.9999999996845056e-05, 
      -2.9999999997633791e-05, 
       1.9999999998422528e-05, 
       2.5198280351168174e-03, 
       2.0998159403203323e-04, 
       2.6578132750592169e-02, 
      -6.3993930785036202e-04, 
       1.8678664531883410e-02, 
       4.9999999996056319e-05, 
       1.3599096100364010e-03, 
       2.9999999997633791e-05, 
      -1.4998159403676562e-04, 
      -1.9997545871568750e-04, 
      -2.9999999997633791e-05, 
       2.9999999997633791e-05, 
       8.9999999964224085e-05, 
       4.3996932196603133e-04, 
       1.9999999998422528e-05, 
      -5.9999999976149397e-05, 
      -1.5998772929575679e-04, 
      -1.9999999998422528e-05, 
       2.9999999997633791e-05, 
       1.2799569653413907e-03, 
       2.9999999997633791e-05, 
       5.7298104005903266e-03, 
       3.3998772921464590e-04, 
      -5.5997373255061008e-04, 
       2.0999999990696381e-04, 
      -1.5299447789493925e-03, 
       2.9999999997633791e-05, 
      -1.8999999991810035e-04, 
       2.6000172872948869e-04, 
       3.9999999996845056e-05, 
       1.1490054433022383e-02, 
      -1.9999999998422528e-05, 
       2.9999999986162873e-04, 
       4.5910183303345259e-02, 
       2.8099578851042317e-03, 
      -2.4799487254914114e-03, 
       1.4199767806551116e-03, 
      -8.1399939349424515e-03, 
       1.9999999990775254e-04, 
      -8.8999999959523410e-04, 
      -5.9999999976149397e-05, 
       2.9999999997633791e-05, 
       3.7998945636984291e-04, 
      -2.5599166103938880e-03, 
      -1.4999999993081436e-04, 
      -3.1098903362823793e-03, 
      -8.9999999964224085e-05, 
      -1.1099596426306442e-03, 
      -1.9999999990775254e-04, 
      -2.9999999997633791e-05, 
       6.9999999965801547e-05, 
       2.9999999997633791e-05, 
      -2.9999999997633791e-05, 
      -2.9999999997633791e-05, 
      -1.1998772935626627e-04, 
      -4.3995091617713257e-04, 
      -4.2995091576089210e-04, 
      -1.6998159403518818e-04, 
      -2.9999999997633791e-05, 
      -6.9999999994478830e-05, 
      -1.9498683911984820e-03, 
      -2.9999999997633791e-05, 
      -6.1795667138423716e-03, 
      -1.0999999999132389e-04, 
      -5.4096132875293794e-03, 
      -4.9999999996056319e-05, 
      -1.9398623381545711e-03, 
      -3.4997545745276901e-04, 
      -4.9999999996056319e-05, 
      -8.4997450738187076e-04, 
      -2.9999999997633791e-05, 
      -2.4359225285690501e-02, 
       3.9999999996845056e-05, 
      -4.0998772933339289e-04, 
      -6.6777798173381128e-02, 
       3.9999999996845056e-05, 
      -1.2099394321210596e-03, 
      -5.1908238367859794e-02, 
      -4.8997385374585406e-04, 
      -1.7089416576075513e-02, 
      -2.9598990618660832e-03, 
      -3.8998945651244057e-04, 
      -4.9999999996056319e-05, 
      -8.9999999992901368e-05, 
      -6.5900414730079944e-03, 
       1.1999999999053516e-04, 
      -2.6999999997870410e-04, 
      -1.8316094280828832e-01, 
       2.7999999997791532e-04, 
      -3.2599561972231688e-03, 
      -4.2250164916974597e-01, 
       1.9999999998422527e-04, 
      -7.8498757930982582e-03, 
      -2.8663087332152248e-01, 
       5.9999999995267581e-05, 
      -3.9999999996845056e-05, 
      -2.6999535602876198e-03, 
      -8.5870256485955698e-02, 
      -1.4200047492090133e-02, 
       3.9999999996845056e-05, 
      -1.8200086374119174e-03, 
      -1.9999999998422527e-04, 
      -1.9999999998422528e-05, 
      -2.3999999831295370e-04, 
       2.7799086682335217e-03, 
       1.8999999998501401e-04, 
       9.4796822552229501e-03, 
       4.1998772808151676e-04, 
       7.7997306363464932e-03, 
       1.5999999998738023e-04, 
       2.7199078769132544e-03, 
       5.1998332118505263e-04, 
       7.9999999993690113e-05, 
       3.9999999996845056e-05, 
       3.3997718629914461e-04, 
       4.3996932171150673e-04, 
       1.3999999998895766e-04, 
       1.9999999998422528e-05, 
       1.0999999995308752e-04, 
       5.9999999976149397e-05, 
      -1.9999999998422528e-05, 
       1.4999999993081436e-04, 
       6.6000345530543478e-04, 
       3.9999999996845056e-05, 
       3.1999999818237558e-04, 
       3.9999999996845056e-05, 
      -7.9999999965012830e-05, 
      -6.9999999965801547e-05, 
      -2.9999999997633791e-05, 
      -6.9999999965801547e-05, 
      -3.9999999996845056e-05, 
      -2.9999999997633791e-05, 
      -1.2999999998974642e-04, 
      -1.5998772935311135e-04, 
      -9.9999999992112637e-05, 
      -2.9999999997633791e-05, 
      -3.2998945693420216e-04, 
      -1.0899622350605557e-03, 
      -1.9999999998422528e-05, 
      -1.2599560988959546e-03, 
      -1.9999999998422528e-05, 
      -6.9997804860839766e-04, 
      -2.1999999914858946e-04, 
      -3.9999999996845056e-05, 
      -8.9999999992901368e-05, 
      -1.8400112315127785e-03, 
      -3.9999999996845056e-05, 
      -5.3900267774981249e-03, 
      -1.1999999999053516e-04, 
      -5.5900215880263860e-03, 
      -8.9999999992901368e-05, 
      -2.8500094963320923e-03, 
      -1.9999999998422528e-05, 
      -8.3000258944965276e-04, 
      -1.5999999998738023e-04, 
      -1.9999999998422528e-05, 
       3.9999999996845056e-05, 
       1.4999999998816893e-04, 
       1.8999999998501401e-04, 
       1.0999999999132389e-04, 
       3.9999999996845056e-05, 
      -2.9999999997633791e-05, 
      -1.9999999998422528e-05, 
      -1.9999999998422528e-05, 
      -6.9999999994478830e-05, 
      -9.9999999992112637e-05, 
      -6.9999999994478830e-05, 
      -2.9999999997633791e-05, 
    } ,  
  } ;  

  static const double fmpb[2][MAX1][5] = {  
    {  
      {    3.2558104895291802e+00,     1.6866932315280974e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    6.5116209790583603e+00,     3.3733864630561948e+04,    -2.5608517982811998e-04,    -1.9799790736513449e-08,     8.0866922009070521e-11, } , 
      {    9.7674314685875405e+00,     5.0600796945842922e+04,    -3.8412776974217999e-04,    -2.9699686104770174e-08,     1.2130038301360578e-10, } , 
      {   -7.4118768963356398e+00,    -4.2272105521117795e+04,     5.3644052038352827e-04,     2.8011080053465659e-07,    -1.3076394606886402e-09, } , 
      {   -4.1560664068064597e+00,    -2.5405173205836822e+04,     4.0839793046946820e-04,     2.7021090516639987e-07,    -1.2672059996841049e-09, } , 
      {   -9.0025591727727994e-01,    -8.5382408905558477e+03,     2.8035534055540824e-04,     2.6031100979814315e-07,    -1.2267725386795698e-09, } , 
      {    2.3555545722519002e+00,     8.3286914247251261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6113650617810809e+00,     2.5195623740006100e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    8.8671755513102610e+00,     4.2062556055287074e+04,    -1.0377242918677173e-04,     2.3061132369337297e-07,    -1.1054721556659640e-09, } , 
      {    1.2122986040839441e+01,     5.8929488370568048e+04,    -2.3181501910083174e-04,     2.2071142832511625e-07,    -1.0650386946614287e-09, } , 
      {   -1.8005118345545599e+00,    -1.7076481781111695e+04,     5.6071068111081647e-04,     5.2062201959628629e-07,    -2.4535450773591395e-09, } , 
      {    1.4552986549746203e+00,    -2.0954946583072160e+02,     4.3266809119675651e-04,     5.1072212422802957e-07,    -2.4131116163546042e-09, } , 
      {    4.7111091445038005e+00,     1.6657382849450252e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.9669196340329806e+00,     3.3524315164731226e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.1222730123562162e+01,     5.0391247480012200e+04,     4.8540321454576519e-05,     4.8102243812325939e-07,    -2.2918112333409983e-09, } , 
      {    1.4478540613091340e+01,     6.7258179795293167e+04,    -7.9502268459483497e-05,     4.7112254275500267e-07,    -2.2513777723364630e-09, } , 
      {    5.5504273769733992e-01,    -8.7477903563865693e+03,     7.1302343175216475e-04,     7.7103313402617271e-07,    -3.6398841550341742e-09, } , 
      {    3.8108532272265201e+00,     8.1191419588944045e+03,     5.8498084183810468e-04,     7.6113323865791599e-07,    -3.5994506940296389e-09, } , 
      {    7.0666637167557003e+00,     2.4986074274175378e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0322474206284880e+01,     4.1853006589456352e+04,     3.2889566200998476e-04,     7.4133344792140254e-07,    -3.5185837720205683e-09, } , 
      {    1.3578284695814061e+01,     5.8719938904737326e+04,     2.0085307209592474e-04,     7.3143355255314581e-07,    -3.4781503110160329e-09, } , 
      {    6.1664077994784208e+00,     1.6447833383619531e+04,     7.3729359247945295e-04,     1.0115443530878024e-06,    -4.7857897717046731e-09, } , 
      {    9.4222182890076009e+00,     3.3314765698900505e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.2678028778536781e+01,     5.0181698014181478e+04,     4.8120841265133303e-04,     9.9174456235128896e-07,    -4.7049228496956025e-09, } , 
      {    1.5933839268065961e+01,     6.7048630329462452e+04,     3.5316582273727301e-04,     9.8184466698303223e-07,    -4.6644893886910672e-09, } , 
      {    8.5219623717303215e+00,     2.4776524808344657e+04,     8.8960634312080123e-04,     1.2619554675176888e-06,    -5.9721288493797074e-09, } , 
      {    1.1777772861259502e+01,     4.1643457123625631e+04,     7.6156375320674127e-04,     1.2520555721494321e-06,    -5.9316953883751721e-09, } , 
      {    1.5033583350788682e+01,     5.8510389438906604e+04,     6.3352116329268131e-04,     1.2421556767811754e-06,    -5.8912619273706367e-09, } , 
      {    1.8289393840317864e+01,     7.5377321754187578e+04,     5.0547857337862124e-04,     1.2322557814129187e-06,    -5.8508284663661014e-09, } , 
      {    1.0877516943982220e+01,     3.3105216233069783e+04,     1.0419190937621494e-03,     1.5123665819475752e-06,    -7.1584679270547424e-09, } , 
      {    1.4133327433511401e+01,     4.9972148548350757e+04,     9.1387650384808943e-04,     1.5024666865793185e-06,    -7.1180344660502071e-09, } , 
      {    1.7389137923040579e+01,     6.6839080863631738e+04,     7.8583391393402947e-04,     1.4925667912110618e-06,    -7.0776010050456718e-09, } , 
      {    1.6488882005763301e+01,     5.8300839973075883e+04,     1.0661892544894377e-03,     1.7528778010092049e-06,    -8.3043735437252414e-09, } , 
      {    1.9744692495292483e+01,     7.5167772288356849e+04,     9.3814666457537775e-04,     1.7429779056409482e-06,    -8.2639400827207061e-09, } , 
      {    1.8844436578015202e+01,     6.6629531397801009e+04,     1.2185020051307860e-03,     2.0032889154390914e-06,    -9.4907126214002756e-09, } , 
      {   -1.6532007687511179e+01,    -5.7672538017827399e+04,    -1.0688531359823463e-03,    -1.7522614088950423e-06,     8.2499338154734521e-09, } , 
      {   -1.4176453115259278e+01,    -4.9343846593102273e+04,    -9.1654038534099791e-04,    -1.5018502944651558e-06,     7.0635947377984171e-09, } , 
      {   -1.5076709032536559e+01,    -5.7882087483658121e+04,    -6.3618504478558978e-04,    -1.2415392846670127e-06,     5.8368221991188467e-09, } , 
      {   -1.1820898543007379e+01,    -4.1015155168377147e+04,    -7.6422763469964974e-04,    -1.2514391800352694e-06,     5.8772556601233820e-09, } , 
      {   -1.2721154460284659e+01,    -4.9553396058932994e+04,    -4.8387229414424156e-04,    -9.9112817023712628e-07,     4.6504831214438124e-09, } , 
      {   -9.4653439707554785e+00,    -3.2686463743652021e+04,    -6.1191488405830147e-04,    -1.0010280656053830e-06,     4.6909165824483477e-09, } , 
      {   -6.2095334812262992e+00,    -1.5819531428371049e+04,    -7.3995747397236143e-04,    -1.0109279609736397e-06,     4.7313500434528830e-09, } , 
      {   -1.3621410377561938e+01,    -5.8091636949488842e+04,    -2.0351695358883324e-04,    -7.3081716043898313e-07,     3.4237105827642433e-09, } , 
      {   -1.0365599888032758e+01,    -4.1224704634207868e+04,    -3.3155954350289329e-04,    -7.4071705580723986e-07,     3.4641440437687786e-09, } , 
      {   -7.1097893985035787e+00,    -2.4357772318926895e+04,    -4.5960213341695325e-04,    -7.5061695117549658e-07,     3.5045775047733139e-09, } , 
      {   -3.8539789089743985e+00,    -7.4908400036459225e+03,    -5.8764472333101316e-04,    -7.6051684654375331e-07,     3.5450109657778492e-09, } , 
      {   -1.1265855805310039e+01,    -4.9762945524763716e+04,    -5.1204202947485030e-05,    -4.8040604600909671e-07,     2.2373715050892086e-09, } , 
      {   -8.0100453157808591e+00,    -3.2896013209482742e+04,    -1.7924679286154501e-04,    -4.9030594137735344e-07,     2.2778049660937439e-09, } , 
      {   -4.7542348262516789e+00,    -1.6029080894201770e+04,    -3.0728938277560503e-04,    -5.0020583674561016e-07,     2.3182384270982792e-09, } , 
      {   -1.4984243367224985e+00,     8.3785142107920387e+02,    -4.3533197268966504e-04,    -5.1010573211386689e-07,     2.3586718881028146e-09, } , 
      {    1.7573861528066816e+00,     1.7704783736360179e+04,    -5.6337456260372495e-04,    -5.2000562748212361e-07,     2.3991053491073499e-09, } , 
      {   -8.9103012330581386e+00,    -4.1434254100038590e+04,     1.0110854769386321e-04,    -2.2999493157921032e-07,     1.0510324274141742e-09, } , 
      {   -5.6544907435289593e+00,    -2.4567321784757616e+04,    -2.6934042220196770e-05,    -2.3989482694746702e-07,     1.0914658884187093e-09, } , 
      {   -2.3986802539997782e+00,    -7.7003894694766441e+03,    -1.5497663213425675e-04,    -2.4979472231572374e-07,     1.1318993494232446e-09, } , 
      {    8.5713023552940171e-01,     9.1665428458043298e+03,    -2.8301922204831677e-04,    -2.5969461768398047e-07,     1.1723328104277799e-09, } , 
      {    4.1129407250585812e+00,     2.6033475161085305e+04,    -4.1106181196237673e-04,    -2.6959451305223719e-07,     1.2127662714323150e-09, } , 
      {   -6.5547466608062388e+00,    -3.3105562675313464e+04,     2.5342129833521145e-04,     2.0416182850676113e-08,    -1.3530665026086031e-10, } , 
      {   -3.2989361712770586e+00,    -1.6238630360032492e+04,     1.2537870842115148e-04,     1.0516287482419388e-08,    -9.4873189256325051e-11, } , 
      {   -4.3125681747878231e-02,     6.2830195524848227e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    3.2126848077813017e+00,     1.7495234270529458e+04,    -1.3070647140696849e-04,    -9.2835032540940612e-09,    -1.4006267247254537e-11, } , 
      {    6.4684952973104819e+00,     3.4362166585810432e+04,    -2.5874906132102851e-04,    -1.9183398622350786e-08,     2.6427193757280724e-11, } , 
      {   -4.1991920885543381e+00,    -2.4776871250588338e+04,     4.0573404897655967e-04,     2.7082729728056255e-07,    -1.3216457279358947e-09, } , 
      {   -9.4338159902515817e-01,    -7.9099389353073657e+03,     2.7769145906249971e-04,     2.6092740191230583e-07,    -1.2812122669313596e-09, } , 
      {    2.3124288905040222e+00,     8.9569933799736082e+03,     1.4964886914843974e-04,     2.5102750654404910e-07,    -1.2407788059268243e-09, } , 
      {    5.5682393800332024e+00,     2.5823925695254584e+04,     2.1606279234379749e-05,     2.4112761117579238e-07,    -1.2003453449222890e-09, } , 
      {    8.8240498695623835e+00,     4.2690858010535558e+04,    -1.0643631067968025e-04,     2.3122771580753563e-07,    -1.1599118839177539e-09, } , 
      {   -1.8436375163024381e+00,    -1.6448179825863212e+04,     5.5804679961790800e-04,     5.2123841171044897e-07,    -2.5079848056109292e-09, } , 
      {    1.4121729732267421e+00,     4.1875248941776067e+02,     4.3000420970384798e-04,     5.1133851634219225e-07,    -2.4675513446063939e-09, } , 
      {    4.6679834627559220e+00,     1.7285684804698736e+04,     3.0196161978978796e-04,     5.0143862097393552e-07,    -2.4271178836018586e-09, } , 
      {    7.9237939522851022e+00,     3.4152617119979710e+04,     1.7391902987572800e-04,     4.9153872560567880e-07,    -2.3866844225973233e-09, } , 
      {    1.1179604441814284e+01,     5.1019549435260684e+04,     4.5876439961668008e-05,     4.8163883023742207e-07,    -2.3462509615927879e-09, } , 
      {    3.7677275454786416e+00,     8.7474439141428866e+03,     5.8231696034519620e-04,     7.6174963077207867e-07,    -3.6538904222814285e-09, } , 
      {    7.0235380350078218e+00,     2.5614376229423862e+04,     4.5427437043113619e-04,     7.5184973540382194e-07,    -3.6134569612768932e-09, } , 
      {    1.0279348524537003e+01,     4.2481308544704836e+04,     3.2623178051707622e-04,     7.4194984003556522e-07,    -3.5730235002723579e-09, } , 
      {    1.3535159014066183e+01,     5.9348240859985810e+04,     1.9818919060301624e-04,     7.3204994466730849e-07,    -3.5325900392678226e-09, } , 
      {    6.1232821177305423e+00,     1.7076135338868015e+04,     7.3462971098654448e-04,     1.0121607452019651e-06,    -4.8402294999564632e-09, } , 
      {    9.3790926072597234e+00,     3.3943067654148988e+04,     6.0658712107248451e-04,     1.0022608498337084e-06,    -4.7997960389519279e-09, } , 
      {    1.2634903096788904e+01,     5.0809999969429962e+04,     4.7854453115842450e-04,     9.9236095446545164e-07,    -4.7593625779473926e-09, } , 
      {    8.4788366899824439e+00,     2.5404826763593141e+04,     8.8694246162789275e-04,     1.2625718596318515e-06,    -6.0265685776314974e-09, } , 
      {    1.1734647179511624e+01,     4.2271759078874114e+04,     7.5889987171383279e-04,     1.2526719642635948e-06,    -5.9861351166269621e-09, } , 
      {    1.4990457669040804e+01,     5.9138691394155088e+04,     6.3085728179977283e-04,     1.2427720688953381e-06,    -5.9457016556224268e-09, } , 
      {    1.4090201751763523e+01,     5.0600450503599241e+04,     9.1121262235518096e-04,     1.5030830786934812e-06,    -7.1724741943019972e-09, } , 
      {    1.6445756324015424e+01,     5.8929141928324367e+04,     1.0635253729965291e-03,     1.7534941931233676e-06,    -8.3588132719770306e-09, } , 
      {   -1.1864024224755259e+01,    -4.0386853213128663e+04,    -7.6689151619255833e-04,    -1.2508227879211067e-06,     5.8228159318715927e-09, } , 
      {   -1.2764280142032538e+01,    -4.8925094103684511e+04,    -4.8653617563715004e-04,    -9.9051177812296360e-07,     4.5960433931920232e-09, } , 
      {   -9.5084696525033578e+00,    -3.2058161788403540e+04,    -6.1457876555121005e-04,    -1.0004116734912203e-06,     4.6364768541965585e-09, } , 
      {   -1.0408725569780637e+01,    -4.0596402678959385e+04,    -3.3422342499580176e-04,    -7.4010066369307718e-07,     3.4097043155169885e-09, } , 
      {   -7.1529150802514572e+00,    -2.3729470363678414e+04,    -4.6226601490986172e-04,    -7.5000055906133390e-07,     3.4501377765215238e-09, } , 
      {   -3.8971045907222766e+00,    -6.8625380483974404e+03,    -5.9030860482392174e-04,    -7.5990045442959063e-07,     3.4905712375260591e-09, } , 
      {   -8.0531709975287367e+00,    -3.2267711254234262e+04,    -1.8191067435445354e-04,    -4.8968954926319076e-07,     2.2233652378419539e-09, } , 
      {   -4.7973605079995565e+00,    -1.5400778938953288e+04,    -3.0995326426851350e-04,    -4.9958944463144748e-07,     2.2637986988464892e-09, } , 
      {   -1.5415500184703768e+00,     1.4661533763276861e+03,    -4.3799585418257352e-04,    -5.0948933999970421e-07,     2.3042321598510245e-09, } , 
      {   -5.6976164252768378e+00,    -2.3939019829509136e+04,    -2.9597923713105281e-05,    -2.3927843483330439e-07,     1.0370261601669196e-09, } , 
      {   -2.4418059357476567e+00,    -7.0720875142281620e+03,    -1.5764051362716528e-04,    -2.4917833020156111e-07,     1.0774596211714549e-09, } , 
      {    8.1400455378152348e-01,     9.7948448010528118e+03,    -2.8568310354122524e-04,    -2.5907822556981784e-07,     1.1178930821759902e-09, } , 
      {   -3.3420618530249366e+00,    -1.5610328404784010e+04,     1.2271482692824295e-04,     1.1132679596582053e-08,    -1.4931291750811484e-10, } , 
      {   -8.6251363495756461e-02,     1.2566039104969645e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    3.1695591260334237e+00,     1.8123536225777938e+04,    -1.3337035289987702e-04,    -8.6671111399313961e-09,    -6.8445995499044334e-11, } , 
      {    6.4253696155626034e+00,     3.4990468541058915e+04,    -2.6141294281393699e-04,    -1.8567006508188122e-08,    -2.8012534494509073e-11, } , 
      {   -9.8650728077303640e-01,    -7.2816369800588836e+03,     2.7502757756959123e-04,     2.6154379402646845e-07,    -1.3356519951831493e-09, } , 
      {    2.2693032087561438e+00,     9.5852953352220902e+03,     1.4698498765553121e-04,     2.5164389865821173e-07,    -1.2952185341786140e-09, } , 
      {    5.5251136982853239e+00,     2.6452227650503064e+04,     1.8942397741471238e-05,     2.4174400328995500e-07,    -1.2547850731740787e-09, } , 
      {    1.3690472914788638e+00,     1.0470544446662429e+03,     4.2734032821093950e-04,     5.1195490845635493e-07,    -2.5219910728581840e-09, } , 
      {    4.6248577810080445e+00,     1.7913986759947216e+04,     2.9929773829687949e-04,     5.0205501308809820e-07,    -2.4815576118536486e-09, } , 
      {    7.8806682705372246e+00,     3.4780919075228194e+04,     1.7125514838281947e-04,     4.9215511771984148e-07,    -2.4411241508491133e-09, } , 
      {    3.7246018637307636e+00,     9.3757458693913686e+03,     5.7965307885228762e-04,     7.6236602288624135e-07,    -3.7083301505332186e-09, } , 
      {    6.9804123532599434e+00,     2.6242678184672342e+04,     4.5161048893822771e-04,     7.5246612751798462e-07,    -3.6678966895286833e-09, } , 
      {    1.0236222842789124e+01,     4.3109610499953320e+04,     3.2356789902416775e-04,     7.4256623214972790e-07,    -3.6274632285241480e-09, } , 
      {    9.3359669255118440e+00,     3.4571369609397472e+04,     6.0392323957957593e-04,     1.0028772419478710e-06,    -4.8542357672037171e-09, } , 
      {   -9.5515953342512354e+00,    -3.1429859833155057e+04,    -6.1724264704411853e-04,    -9.9979528137705764e-07,     4.5820371259447684e-09, } , 
      {   -7.1960407619993347e+00,    -2.3101168408429930e+04,    -4.6492989640277026e-04,    -7.4938416694717122e-07,     3.3956980482697342e-09, } , 
      {   -8.0962966792766160e+00,    -3.1639409298985778e+04,    -1.8457455584736205e-04,    -4.8907315714902808e-07,     2.1689255095901642e-09, } , 
      {   -4.8404861897474349e+00,    -1.4772476983704806e+04,    -3.1261714576142204e-04,    -4.9897305251728480e-07,     2.2093589705946995e-09, } , 
      {   -1.5846757002182550e+00,     2.0944553315761686e+03,    -4.4065973567548205e-04,    -5.0887294788554153e-07,     2.2497924315992348e-09, } , 
      {   -5.7407421070247153e+00,    -2.3310717874260652e+04,    -3.2261805206013799e-05,    -2.3866204271914171e-07,     9.8258643191512974e-10, } , 
      {   -2.4849316174955351e+00,    -6.4437855589796791e+03,    -1.6030439512007379e-04,    -2.4856193808739843e-07,     1.0230198929196651e-09, } , 
      {    7.7087887203364525e-01,     1.0423146756301294e+04,    -2.8834698503413378e-04,    -2.5846183345565516e-07,     1.0634533539242004e-09, } , 
      {   -3.3851875347728146e+00,    -1.4982026449535528e+04,     1.2005094543533445e-04,     1.1749071710744717e-08,    -2.0375264575990466e-10, } , 
      {   -1.2937704524363469e-01,     1.8849058657454468e+03,    -7.9916444787255359e-06,     1.8491763424879922e-09,    -1.6331918475536940e-10, } , 
      {    3.1264334442855457e+00,     1.8751838181026422e+04,    -1.3603423439278553e-04,    -8.0507190257687326e-09,    -1.2288572375083414e-10, } , 
      {    2.2261775270082653e+00,     1.0213597290470572e+04,     1.4432110616262271e-04,     2.5226029077237441e-07,    -1.3496582624304039e-09, } , 
      {    5.4819880165374464e+00,     2.7080529605751548e+04,     1.6278516248562724e-05,     2.4236039540411768e-07,    -1.3092248014258685e-09, } , 
      {    4.5817320992601660e+00,     1.8542288715195700e+04,     2.9663385680397096e-04,     5.0267140520226088e-07,    -2.5359973401054383e-09, } , 
      {    6.9372866715120658e+00,     2.6870980139920826e+04,     4.4894660744531918e-04,     7.5308251963214730e-07,    -3.7223364177804730e-09, } , 
      {   -4.8836118714953134e+00,    -1.4144175028456324e+04,    -3.1528102725433057e-04,    -4.9835666040312223e-07,     2.1549192423429098e-09, } , 
      {   -2.5280572992434132e+00,    -5.8154836037311970e+03,    -1.6296827661298229e-04,    -2.4794554597323575e-07,     9.6858016466787519e-10, } , 
      {   -1.7250272699151292e-01,     2.5132078209939291e+03,    -1.0655525971634047e-05,     2.4655684566506563e-09,    -2.1775891300715919e-10, } , 
      {    2.1830518452603873e+00,     1.0841899245719054e+04,     1.4165722466971420e-04,     2.5287668288653709e-07,    -1.4040979906821937e-09, } , 
      {    4.5386064175122875e+00,     1.9170590670444180e+04,     2.9396997531106243e-04,     5.0328779731642346e-07,    -2.5904370683572280e-09, } , 
      {   -2.5711829809912912e+00,    -5.1871816484827150e+03,    -1.6563215810589080e-04,    -2.4732915385907313e-07,     9.1414043641608552e-10, } , 
      {    5.3709695485465092e+00,     5.2581693240980503e+03,    -2.2438534640056430e-05,     2.9507893812523238e-08,     6.3394236941882920e-11, } , 
      {    6.1673472229483051e-01,    -1.0770911570103719e+04,    -3.2972791741566143e-04,    -4.7069794293308694e-07,     2.3816326640401619e-09, } , 
      {    2.9722892945467310e+00,    -2.4422201453785938e+03,    -1.7741516677431319e-04,    -2.2028682850320052e-07,     1.1952935863651277e-09, } , 
      {    5.3278438667986308e+00,     5.8864712793465324e+03,    -2.5102416132964941e-05,     3.0124285926685901e-08,     8.9545086900931297e-12, } , 
      {    7.6833984390505314e+00,     1.4215162704071659e+04,     1.2721033450838331e-04,     2.8053540035657232e-07,    -1.1773845689849412e-09, } , 
      {    1.0038953011302430e+01,     2.2543854128796786e+04,     2.7952308514973156e-04,     5.3094651478645869e-07,    -2.3637236466599759e-09, } , 
      {   -1.7819455317049475e+00,    -1.8471301039580361e+04,    -4.8470454954991819e-04,    -7.2049266524881073e-07,     3.5135320134634069e-09, } , 
      {    5.7360904054695228e-01,    -1.0142609614855237e+04,    -3.3239179890856996e-04,    -4.7008155081892426e-07,     2.3271929357883722e-09, } , 
      {   -3.2664687673032766e-01,    -1.8680850505411083e+04,    -5.2036458353161714e-05,    -2.0977054102078114e-07,     1.1004203971088025e-09, } , 
      {    2.9291636127988525e+00,    -1.8139181901301113e+03,    -1.8007904826722169e-04,    -2.1967043638903787e-07,     1.1408538581133378e-09, } , 
      {    2.0289076955215726e+00,    -1.0352159080685959e+04,     1.0027629228818653e-04,     4.0640573409105293e-08,    -8.5918680566231935e-11, } , 
      {    5.2847181850507532e+00,     6.5147732345950153e+03,    -2.7766297625873455e-05,     3.0740678040848568e-08,    -4.5485219561696674e-11, } , 
      {    8.5405286745799334e+00,     2.3381705549875991e+04,    -1.5580888753993343e-04,     2.0840782672591843e-08,    -5.0517585571614133e-12, } , 
      {    4.3844622677734737e+00,    -2.0234676559608329e+03,     2.5258904292953477e-04,     2.9105168783899173e-07,    -1.2722577582412664e-09, } , 
      {    7.6402727573026539e+00,     1.4843464659320141e+04,     1.2454645301547481e-04,     2.8115179247073500e-07,    -1.2318242972367311e-09, } , 
      {    9.9958273295545528e+00,     2.3172156084045269e+04,     2.7685920365682303e-04,     5.3156290690062137e-07,    -2.4181633749117656e-09, } , 
      {    1.2351381901806453e+01,     3.1500847508770396e+04,     4.2917195429817125e-04,     7.8197402133050779e-07,    -3.6045024525868002e-09, } , 
      {   -6.5361803579566269e+00,    -3.4500381933782133e+04,    -7.9199393232552327e-04,    -1.2206985019944209e-06,     5.8317704405616857e-09, } , 
      {   -4.1806257857047262e+00,    -2.6171690509057007e+04,    -6.3968118168417499e-04,    -9.7028738756453448e-07,     4.6454313628866515e-09, } , 
      {   -5.0808817029820057e+00,    -3.4709931399612855e+04,    -3.5932584112876670e-04,    -7.0997637776639133e-07,     3.4186588242070815e-09, } , 
      {   -1.8250712134528257e+00,    -1.7842999084331881e+04,    -4.8736843104282666e-04,    -7.1987627313464805e-07,     3.4590922852116168e-09, } , 
      {    1.4307392760763544e+00,    -9.7606676905090558e+02,    -6.1541102095688668e-04,    -7.2977616850290478e-07,     3.4995257462161521e-09, } , 
      {   -2.7253271307301059e+00,    -2.6381239974887729e+04,    -2.0701309048741848e-04,    -4.5956526333650491e-07,     2.2323197465320469e-09, } , 
      {    5.3048335879907405e-01,    -9.5143076596067549e+03,    -3.3505568040147844e-04,    -4.6946515870476163e-07,     2.2727532075365822e-09, } , 
      {    3.7862938483282544e+00,     7.3526246556742208e+03,    -4.6309827031553846e-04,    -4.7936505407301836e-07,     2.3131866685411175e-09, } , 
      {   -3.6977255847820589e-01,    -1.8052548550162603e+04,    -5.4700339846070225e-05,    -2.0915414890661846e-07,     1.0459806688570128e-09, } , 
      {    2.8860379310509741e+00,    -1.1856162348816290e+03,    -1.8274292976013022e-04,    -2.1905404427487519e-07,     1.0864141298615481e-09, } , 
      {    6.1418484205801542e+00,     1.5681316080399347e+04,    -3.1078551967419018e-04,    -2.2895393964313191e-07,     1.1268475908660832e-09, } , 
      {   -1.2700284757554858e+00,    -2.6590789440718450e+04,     2.2565500070933800e-04,     5.1156860891524677e-08,    -1.8079186982255702e-10, } , 
      {    1.9857820137736943e+00,    -9.7238571254374765e+03,     9.7612410795278029e-05,     4.1256965523267953e-08,    -1.4035840881802174e-10, } , 
      {    5.2415925033028747e+00,     7.1430751898434974e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    8.4974029928320540e+00,     2.4010007505124471e+04,    -1.5847276903284196e-04,     2.1457174786754507e-08,    -5.9491486808951217e-11, } , 
      {    4.3413365860255952e+00,    -1.3951657007123506e+03,     2.4992516143662629e-04,     2.9166807995315435e-07,    -1.3266974864930561e-09, } , 
      {    7.5971470755547754e+00,     1.5471766614568623e+04,     1.2188257152256629e-04,     2.8176818458489763e-07,    -1.2862640254885208e-09, } , 
      {    1.0852957565083955e+01,     3.2338698929849597e+04,    -6.1600183914937057e-06,     2.7186828921664090e-07,    -1.2458305644839857e-09, } , 
      {    6.6968911582774950e+00,     6.9335257240127758e+03,     4.0223791207797457e-04,     5.4207919438304077e-07,    -2.5130365641680910e-09, } , 
      {    9.9527016478066734e+00,     2.3800458039293750e+04,     2.7419532216391455e-04,     5.3217929901478405e-07,    -2.4726031031635556e-09, } , 
      {    1.3208512137335855e+01,     4.0667390354574723e+04,     1.4615273224985453e-04,     5.2227940364652733e-07,    -2.4321696421590203e-09, } , 
      {    9.0524457305293939e+00,     1.5262217148737902e+04,     5.5455066271932268e-04,     7.9249030881292720e-07,    -3.6993756418431256e-09, } , 
      {    1.2308256220058574e+01,     3.2129149464018876e+04,     4.2650807280526277e-04,     7.8259041344467047e-07,    -3.6589421808385903e-09, } , 
      {    1.5564066709587754e+01,     4.8996081779299850e+04,     2.9846548289120281e-04,     7.7269051807641375e-07,    -3.6185087198340550e-09, } , 
      {    1.4663810792310475e+01,     4.0457840888744002e+04,     5.7882082344661099e-04,     1.0330015278745569e-06,    -4.8452812585136241e-09, } , 
      {    1.7019365364562375e+01,     4.8786532313469128e+04,     7.3113357408795927e-04,     1.2834126423044433e-06,    -6.0316203361886584e-09, } , 
      {   -8.9348606119564042e+00,    -4.2200771403258776e+04,    -9.4697056445977991e-04,    -1.4704932243101446e-06,     6.9636697899849307e-09, } , 
      {   -9.8351165292336855e+00,    -5.0739012293814623e+04,    -6.6661522390437178e-04,    -1.2101822145120015e-06,     5.7368972513053603e-09, } , 
      {   -6.5793060397045053e+00,    -3.3872079978533649e+04,    -7.9465781381843174e-04,    -1.2200821098802582e-06,     5.7773307123098957e-09, } , 
      {   -7.4795619569817848e+00,    -4.2410320869089497e+04,    -5.1430247326302351e-04,    -9.5977110008211507e-07,     4.5505581736303261e-09, } , 
      {   -4.2237514674526047e+00,    -2.5543388553808523e+04,    -6.4234506317708347e-04,    -9.6967099545037180e-07,     4.5909916346348614e-09, } , 
      {   -9.6794097792342448e-01,    -8.6764562385275494e+03,    -7.7038765309114343e-04,    -9.7957089081862852e-07,     4.6314250956393967e-09, } , 
      {   -5.1240073847298842e+00,    -3.4081629444364371e+04,    -3.6198972262167523e-04,    -7.0935998565222865e-07,     3.3642190959552919e-09, } , 
      {   -1.8681968952007040e+00,    -1.7214697129083397e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.3876135943284762e+00,    -3.4776481380242330e+02,    -6.1807490244979515e-04,    -7.2915977638874210e-07,     3.4450860179643625e-09, } , 
      {   -6.0242633020071645e+00,    -4.2619870334920219e+04,    -8.1634382066266995e-05,    -4.4904897585408550e-07,     2.1374465572757223e-09, } , 
      {   -2.7684528124779844e+00,    -2.5752938019639245e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    4.8735767705119581e-01,    -8.8860057043582729e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.7431681665803760e+00,     7.9809266109227028e+03,    -4.6576215180844699e-04,    -4.7874866195885568e-07,     2.2587469402893278e-09, } , 
      {   -3.6687087297552643e+00,    -3.4291178910195093e+04,     7.0678368575081252e-05,    -1.9863786142419908e-07,     9.5110747960068764e-10, } , 
      {   -4.1289824022608412e-01,    -1.7424246594914119e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    2.8429122493030961e+00,    -5.5731427963314673e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.0987227388322758e+00,     1.6309618035647829e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    9.3545332283614560e+00,     3.3176550350928803e+04,    -4.4149199108115867e-04,    -2.3823744289722598e-07,     1.1128413236188287e-09, } , 
      {   -1.3131541575033641e+00,    -2.5962487485469966e+04,     2.2299111921642950e-04,     5.1773253005687344e-08,    -2.3523159807434682e-10, } , 
      {    1.9426563320258161e+00,    -9.0955551701889945e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    5.1984668215549963e+00,     7.7713771450919794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.4542773110841765e+00,     2.4638309460372955e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.1710087800613357e+01,     4.1505241775653929e+04,    -2.8917924043981045e-04,     1.2173671532660445e-08,    -7.3497754056205747e-11, } , 
      {    1.0424004147485362e+00,    -1.7633796060744840e+04,     3.7530386985777772e-04,     3.0218436743557376e-07,    -1.4215706757493813e-09, } , 
      {    4.2982109042777168e+00,    -7.6686374546386833e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    7.5540213938068970e+00,     1.6100068569817106e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0809831883336077e+01,     3.2967000885098081e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    1.4065642372865257e+01,     4.9833933200379055e+04,    -1.3686648979846221e-04,     2.6258478596254686e-07,    -1.2598368317312402e-09, } , 
      {    6.6537654765296166e+00,     7.5618276792612578e+03,     3.9957403058506603e-04,     5.4269558649720345e-07,    -2.5674762924198806e-09, } , 
      {    9.9095759660587959e+00,     2.4428759994542233e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.3165386455587978e+01,     4.1295692309823207e+04,     1.4348885075694603e-04,     5.2289579576069001e-07,    -2.4866093704108100e-09, } , 
      {    1.6421196945117156e+01,     5.8162624625104181e+04,     1.5446260842886042e-05,     5.1299590039243328e-07,    -2.4461759094062747e-09, } , 
      {    9.0093200487815164e+00,     1.5890519103986384e+04,     5.5188678122641420e-04,     7.9310670092708988e-07,    -3.7538153700949153e-09, } , 
      {    1.2265130538310697e+01,     3.2757451419267360e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    1.5520941027839877e+01,     4.9624383734548333e+04,     2.9580160139829428e-04,     7.7330691019057643e-07,    -3.6729484480858447e-09, } , 
      {    1.4620685110562597e+01,     4.1086142843992486e+04,     5.7615694195370252e-04,     1.0336179199887196e-06,    -4.8997209867654142e-09, } , 
      {    1.7876495600091779e+01,     5.7953075159273460e+04,     4.4811435203964255e-04,     1.0237180246204628e-06,    -4.8592875257608789e-09, } , 
      {    1.6976239682814498e+01,     4.9414834268717612e+04,     7.2846969259505079e-04,     1.2840290344186060e-06,    -6.0860600644404485e-09, } , 
      {    1.9331794255066399e+01,     5.7743525693442738e+04,     8.8078244323639896e-04,     1.5344401488484924e-06,    -7.2723991421154835e-09, } , 
      {   -6.6224317214523838e+00,    -3.3243778023285166e+04,    -7.9732169531134022e-04,    -1.2194657177660955e-06,     5.7228909840581056e-09, } , 
      {   -7.5226876387296633e+00,    -4.1782018913841013e+04,    -5.1696635475593198e-04,    -9.5915470796795239e-07,     4.4961184453785360e-09, } , 
      {   -4.2668771492004831e+00,    -2.4915086598560039e+04,    -6.4500894466999195e-04,    -9.6905460333620912e-07,     4.5365519063830713e-09, } , 
      {   -5.1671330664777626e+00,    -3.3453327489115887e+04,    -3.6465360411458376e-04,    -7.0874359353806597e-07,     3.3097793677035022e-09, } , 
      {   -1.9113225769485822e+00,    -1.6586395173834913e+04,    -4.9269619402864367e-04,    -7.1864348890632269e-07,     3.3502128287080375e-09, } , 
      {    1.3444879125805980e+00,     2.8053714144605897e+02,    -6.2073878394270363e-04,    -7.2854338427457942e-07,     3.3906462897125728e-09, } , 
      {   -2.8115784942258628e+00,    -2.5124636064390761e+04,    -2.1234085347323549e-04,    -4.5833247910817955e-07,     2.1234402900284675e-09, } , 
      {    4.4423199530331758e-01,    -8.2577037491097908e+03,    -3.4038344338729550e-04,    -4.6823237447643627e-07,     2.1638737510330028e-09, } , 
      {    3.7000424848324975e+00,     8.6092285661711849e+03,    -4.6842603330135552e-04,    -4.7813226984469300e-07,     2.2043072120375382e-09, } , 
      {   -4.5602392197396235e-01,    -1.6795944639665635e+04,    -6.0028102831887247e-05,    -2.0792136467829315e-07,     9.3710121235343308e-10, } , 
      {    2.7997865675552180e+00,     7.0987675615335547e+01,    -1.8807069274594723e-04,    -2.1782126004654988e-07,     9.7753467335796839e-10, } , 
      {    6.0555970570843973e+00,     1.6937919990896313e+04,    -3.1611328266000724e-04,    -2.2772115541480660e-07,     1.0179681343625035e-09, } , 
      {    1.8995306502779379e+00,    -8.4672532149405124e+03,     9.2284647809460994e-05,     4.2489749751593286e-08,    -2.4923786532160132e-10, } , 
      {    5.1553411398071178e+00,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    8.4111516293362989e+00,     2.5266611415621439e+04,    -1.6380053201865897e-04,     2.2689959015079834e-08,    -1.6837094331253080e-10, } , 
      {    1.1666962118865479e+01,     4.2133543730902413e+04,    -2.9184312193271898e-04,     1.2790063646823109e-08,    -1.2793748230799554e-10, } , 
      {    4.2550852225298383e+00,    -1.3856179021538605e+02,     2.4459739845080923e-04,     2.9290086418147971e-07,    -1.4355769429966358e-09, } , 
      {    7.5108957120590185e+00,     1.6728370525065588e+04,     1.1655480853674925e-04,     2.8300096881322299e-07,    -1.3951434819921005e-09, } , 
      {    1.0766706201588200e+01,     3.3595302840346565e+04,    -1.1487781377310728e-05,     2.7310107344496626e-07,    -1.3547100209875654e-09, } , 
      {    1.4022516691117380e+01,     5.0462235155627539e+04,    -1.3953037129137071e-04,     2.6320117807670954e-07,    -1.3142765599830301e-09, } , 
      {    6.6106397947817381e+00,     8.1901296345097398e+03,     3.9691014909215750e-04,     5.4331197861136613e-07,    -2.6219160206716703e-09, } , 
      {    9.8664502843109183e+00,     2.5057061949790717e+04,     2.6886755917809749e-04,     5.3341208324310941e-07,    -2.5814825596671350e-09, } , 
      {    1.3122260773840100e+01,     4.1923994265071691e+04,     1.4082496926403753e-04,     5.2351218787485269e-07,    -2.5410490986625997e-09, } , 
      {    8.9661943670336388e+00,     1.6518821059234866e+04,     5.4922289973350572e-04,     7.9372309304125256e-07,    -3.8082550983467054e-09, } , 
      {    1.2222004856562819e+01,     3.3385753374515843e+04,     4.2118030981944571e-04,     7.8382319767299583e-07,    -3.7678216373421700e-09, } , 
      {    1.5477815346091999e+01,     5.0252685689796817e+04,     2.9313771990538575e-04,     7.7392330230473911e-07,    -3.7273881763376347e-09, } , 
      {    1.4577559428814720e+01,     4.1714444799240970e+04,     5.7349306046079404e-04,     1.0342343121028823e-06,    -4.9541607150172043e-09, } , 
      {    1.6933114001066620e+01,     5.0043136223966096e+04,     7.2580581110214231e-04,     1.2846454265327687e-06,    -6.1404997926922385e-09, } , 
      {   -4.3100028309483616e+00,    -2.4286784643311559e+04,    -6.4767282616290053e-04,    -9.6843821122204644e-07,     4.4821121781312821e-09, } , 
      {   -1.9544482586964604e+00,    -1.5958093218586433e+04,    -4.9536007552155226e-04,    -7.1802709679216001e-07,     3.2957731004562474e-09, } , 
      {    4.0110631355543935e-01,    -7.6294017938613088e+03,    -3.4304732488020398e-04,    -4.6761598236227365e-07,     2.1094340227812128e-09, } , 
      {    2.7566608858073396e+00,     6.9928963086381782e+02,    -1.9073457423885576e-04,    -2.1720486793238720e-07,     9.2309494510617863e-10, } , 
      {    6.0124713753365189e+00,     1.7566221946144793e+04,    -3.1877716415291572e-04,    -2.2710476330064392e-07,     9.6352840611071384e-10, } , 
      {    1.8564049685300597e+00,    -7.8389512596920304e+03,     8.9620766316552490e-05,     4.3106141865755946e-08,    -3.0367759357339114e-10, } , 
      {    5.1122154580592394e+00,     9.0279810555889435e+03,    -3.8421823597507498e-05,     3.3206246497499222e-08,    -2.6324413256885588e-10, } , 
      {    8.3680259475884196e+00,     2.5894913370869919e+04,    -1.6646441351156750e-04,     2.3306351129242497e-08,    -2.2281067156432062e-10, } , 
      {    4.2119595407819599e+00,     4.8974016503309622e+02,     2.4193351695790072e-04,     2.9351725629564234e-07,    -1.4900166712484255e-09, } , 
      {    7.4677700303111401e+00,     1.7356672480314071e+04,     1.1389092704384075e-04,     2.8361736092738562e-07,    -1.4495832102438902e-09, } , 
      {    1.0723580519840320e+01,     3.4223604795595049e+04,    -1.4151662870219241e-05,     2.7371746555912889e-07,    -1.4091497492393551e-09, } , 
      {    9.8233246025630390e+00,     2.5685363905039198e+04,     2.6620367768518901e-04,     5.3402847535727209e-07,    -2.6359222879189250e-09, } , 
      {    1.3079135092092221e+01,     4.2552296220320175e+04,     1.3816108777112900e-04,     5.2412857998901537e-07,    -2.5954888269143897e-09, } , 
      {    1.2178879174814940e+01,     3.4014055329764327e+04,     4.1851642832653723e-04,     7.8443958978715851e-07,    -3.8222613655939593e-09, } , 
      {    1.4534433747066840e+01,     4.2342746754489453e+04,     5.7082917896788545e-04,     1.0348507042170449e-06,    -5.0086004432689935e-09, } , 
      {    3.5798063180756112e-01,    -7.0010998386128258e+03,    -3.4571120637311251e-04,    -4.6699959024811097e-07,     2.0549942945294231e-09, } , 
      {    2.7135352040594611e+00,     1.3275915861123001e+03,    -1.9339845573176426e-04,    -2.1658847581822455e-07,     8.6865521685438886e-10, } , 
      {    1.8132792867821814e+00,    -7.2106493044435474e+03,     8.6956884823643973e-05,     4.3722533979918613e-08,    -3.5811732182518090e-10, } , 
      {    5.0690897763113618e+00,     9.6562830108374255e+03,    -4.1085705090416016e-05,     3.3822638611661889e-08,    -3.1768386082064569e-10, } , 
      {    7.4246443485632625e+00,     1.7984974435562552e+04,     1.1122704555093223e-04,     2.8423375304154830e-07,    -1.5040229384956800e-09, } , 
      {    5.0259640945634834e+00,     1.0284584966085909e+04,    -4.3749586583324520e-05,     3.4439030725824549e-08,    -3.7212358907243546e-10, } , 
      {    1.0655687733597262e+01,     1.1772942558693065e+04,    -5.0204832265929882e-05,     6.0248571853371802e-08,     1.7909017380186259e-11, } , 
      {    5.9014529073455835e+00,    -4.2561383355087046e+03,    -3.5749421504153490e-04,    -4.3995726489223836e-07,     2.3361474444784657e-09, } , 
      {    8.2570074795974833e+00,     4.0725530892164215e+03,    -2.0518146440018665e-04,    -1.8954615046235197e-07,     1.1498083668034308e-09, } , 
      {    1.0612562051849384e+01,     1.2401244513941547e+04,    -5.2868713758838393e-05,     6.0864963967534469e-08,    -3.6530710871603557e-11, } , 
      {    1.2968116624101285e+01,     2.0729935938666673e+04,     9.9444036882509855e-05,     3.1127607839742090e-07,    -1.2228697885466381e-09, } , 
      {    3.5027726533458052e+00,    -1.1956527804985348e+04,    -5.1247084717579165e-04,    -6.8975198720796210e-07,     3.4680467939017102e-09, } , 
      {    5.8583272255977050e+00,    -3.6278363802602225e+03,    -3.6015809653444338e-04,    -4.3934087277807573e-07,     2.2817077162266756e-09, } , 
      {    8.2138817978496057e+00,     4.7008550444649036e+03,    -2.0784534589309516e-04,    -1.8892975834818931e-07,     1.0953686385516411e-09, } , 
      {    1.1469692287378784e+01,     2.1567787359745878e+04,    -3.3588793580715512e-04,    -1.9882965371644604e-07,     1.1358020995561764e-09, } , 
      {    7.3136258805723253e+00,    -3.8373858460909441e+03,     7.2509994662313078e-05,     7.1381251449953861e-08,    -1.3140390012792861e-10, } , 
      {    1.0569436370101506e+01,     1.3029546469190031e+04,    -5.5532595251746910e-05,     6.1481356081697136e-08,    -9.0970439123393348e-11, } , 
      {    1.3825246859630685e+01,     2.9896478784471004e+04,    -1.8357518516580690e-04,     5.1581460713440411e-08,    -5.0536978118858087e-11, } , 
      {    9.6691804528242251e+00,     4.4913055786341820e+03,     2.2482274530366133e-04,     3.2179236587984025e-07,    -1.3177429778029631e-09, } , 
      {    1.2924990942353407e+01,     2.1358237893915157e+04,     9.6780155389601338e-05,     3.1189247051158353e-07,    -1.2773095167984278e-09, } , 
      {    1.5280545514605308e+01,     2.9686929318640283e+04,     2.4909290603094961e-04,     5.6230358494147000e-07,    -2.4636485944734622e-09, } , 
      {    1.7636100086857208e+01,     3.8015620743365413e+04,     4.0140565667229783e-04,     8.1271469937135642e-07,    -3.6499876721484973e-09, } , 
      {    1.1040923993460263e+00,    -1.9656917274461994e+04,    -6.6744747931004841e-04,    -9.3954670952368595e-07,     4.5999461433249544e-09, } , 
      {    3.4596469715979268e+00,    -1.1328225849736866e+04,    -5.1513472866870013e-04,    -6.8913559509379953e-07,     3.4136070656499206e-09, } , 
      {    2.5593910543206464e+00,    -1.9866466740292715e+04,    -2.3477938811329192e-04,    -4.2882458529565633e-07,     2.1868345269703506e-09, } , 
      {    5.8152015438498266e+00,    -2.9995344250117405e+03,    -3.6282197802735191e-04,    -4.3872448066391305e-07,     2.2272679879748859e-09, } , 
      {    9.0710120333790076e+00,     1.3867397890269234e+04,    -4.9086456794141193e-04,    -4.4862437603216978e-07,     2.2677014489794212e-09, } , 
      {    4.9149456265725462e+00,    -1.1537775315567587e+04,    -8.2466637471943674e-05,    -1.7841347086576991e-07,     1.0004954492953159e-09, } , 
      {    8.1707561161017264e+00,     5.3291569997133856e+03,    -2.1050922738600366e-04,    -1.8831336623402663e-07,     1.0409289102998513e-09, } , 
      {    1.1426566605630907e+01,     2.2196089314994359e+04,    -3.3855181730006365e-04,    -1.9821326160228336e-07,     1.0813623713043866e-09, } , 
      {    7.2705001988244469e+00,    -3.2090838908424621e+03,     6.9846113169404574e-05,     7.1997643564116527e-08,    -1.8584362837971840e-10, } , 
      {    1.0526310688353627e+01,     1.3657848424438513e+04,    -5.8196476744655414e-05,     6.2097748195859803e-08,    -1.4541016737518314e-10, } , 
      {    1.3782121177882807e+01,     3.0524780739719485e+04,    -1.8623906665871540e-04,     5.2197852827603071e-08,    -1.0497670637064788e-10, } , 
      {    9.6260547710763475e+00,     5.1196075338826640e+03,     2.2215886381075282e-04,     3.2240875799400293e-07,    -1.3721827060547530e-09, } , 
      {    1.2881865260605528e+01,     2.1986539849163637e+04,     9.4116273896692834e-05,     3.1250886262574621e-07,    -1.3317492450502176e-09, } , 
      {    1.6137675750134708e+01,     3.8853472164444611e+04,    -3.3926316017367155e-05,     3.0260896725748949e-07,    -1.2913157840456823e-09, } , 
      {    1.1981609343328246e+01,     1.3448298958607791e+04,     3.7447161445210110e-04,     5.7281987242388930e-07,    -2.5585217837297872e-09, } , 
      {    1.5237419832857428e+01,     3.0315231273888763e+04,     2.4642902453804108e-04,     5.6291997705563258e-07,    -2.5180883227252519e-09, } , 
      {    1.8493230322386609e+01,     4.7182163589169737e+04,     1.1838643462398109e-04,     5.5302008168737585e-07,    -2.4776548617207166e-09, } , 
      {    1.7592974405109331e+01,     3.8643922698613897e+04,     3.9874177517938930e-04,     8.1333109148551900e-07,    -3.7044274004002870e-09, } , 
      {    1.9948528977361228e+01,     4.6972614123339023e+04,     5.5105452582073758e-04,     1.0637422059154054e-06,    -4.8907664780753212e-09, } , 
      {   -1.2945878546537526e+00,    -2.7357306743938636e+04,    -8.2242411144430516e-04,    -1.1893414318394097e-06,     5.7318454927481986e-09, } , 
      {   -2.1948437719310321e+00,    -3.5895547634494491e+04,    -5.4206877088889692e-04,    -9.2903042204126654e-07,     4.5050729540686290e-09, } , 
      {    1.0609667175981481e+00,    -1.9028615319213510e+04,    -6.7011136080295688e-04,    -9.3893031740952327e-07,     4.5455064150731643e-09, } , 
      {    1.6071080032086860e-01,    -2.7566856209769358e+04,    -3.8975602024754870e-04,    -6.7861930761138012e-07,     3.3187338763935956e-09, } , 
      {    3.4165212898500488e+00,    -1.0699923894488384e+04,    -5.1779861016160861e-04,    -6.8851920297963685e-07,     3.3591673373981309e-09, } , 
      {    2.5162653725727684e+00,    -1.9238164785044231e+04,    -2.3744326960620043e-04,    -4.2820819318149365e-07,     2.1323947987185609e-09, } , 
      {    5.7720758621019481e+00,    -2.3712324697632580e+03,    -3.6548585952026044e-04,    -4.3810808854975037e-07,     2.1728282597230963e-09, } , 
      {    9.0278863516311301e+00,     1.4495699845517716e+04,    -4.9352844943432040e-04,    -4.4800798391800710e-07,     2.2132617207276316e-09, } , 
      {    4.8718199448246686e+00,    -1.0909473360319105e+04,    -8.5130518964852191e-05,    -1.7779707875160725e-07,     9.4605572104352629e-10, } , 
      {    8.1276304343538488e+00,     5.9574589549618686e+03,    -2.1317310887891217e-04,    -1.8769697411986398e-07,     9.8648918204806139e-10, } , 
      {    1.1383440923883029e+01,     2.2824391270242842e+04,    -3.4121569879297218e-04,    -1.9759686948812070e-07,     1.0269226430525967e-09, } , 
      {    1.4639251413412209e+01,     3.9691323585523816e+04,    -4.6925828870703214e-04,    -2.0749676485637743e-07,     1.0673561040571320e-09, } , 
      {    3.9715640275473887e+00,    -1.9447714250874953e+04,     1.9522482159055606e-04,     8.2513931046535905e-08,    -2.8071681763604348e-10, } , 
      {    7.2273745170765693e+00,    -2.5807819355939796e+03,     6.7182231676496057e-05,     7.2614035678279181e-08,    -2.4028335663150822e-10, } , 
      {    1.0483185006605749e+01,     1.4286150379686995e+04,    -6.0860358237563932e-05,     6.2714140310022456e-08,    -1.9984989562697296e-10, } , 
      {    1.3738995496134930e+01,     3.1153082694967969e+04,    -1.8890294815162391e-04,     5.2814244941765738e-08,    -1.5941643462243769e-10, } , 
      {    1.6994805985664108e+01,     4.8020015010248942e+04,    -3.1694553806568392e-04,     4.2914349573509013e-08,    -1.1898297361790243e-10, } , 
      {    6.3271185997992898e+00,    -1.1119022826149827e+04,     3.4753757223190425e-04,     3.3292504547642234e-07,    -1.4670558953110779e-09, } , 
      {    9.5829290893284700e+00,     5.7479094891311470e+03,     2.1949498231784432e-04,     3.2302515010816561e-07,    -1.4266224343065428e-09, } , 
      {    1.2838739578857650e+01,     2.2614841804412121e+04,     9.1452392403784316e-05,     3.1312525473990889e-07,    -1.3861889733020075e-09, } , 
      {    1.6094550068386827e+01,     3.9481774119693087e+04,    -3.6590197510275672e-05,     3.0322535937165217e-07,    -1.3457555122974722e-09, } , 
      {    1.9350360557916009e+01,     5.6348706434974069e+04,    -1.6463278742433565e-04,     2.9332546400339544e-07,    -1.3053220512929369e-09, } , 
      {    1.1938483661580369e+01,     1.4076600913856273e+04,     3.7180773295919257e-04,     5.7343626453805198e-07,    -2.6129615119815769e-09, } , 
      {    1.5194294151109551e+01,     3.0943533229137247e+04,     2.4376514304513258e-04,     5.6353636916979526e-07,    -2.5725280509770416e-09, } , 
      {    1.8450104640638727e+01,     4.7810465544418214e+04,     1.1572255313107258e-04,     5.5363647380153853e-07,    -2.5320945899725062e-09, } , 
      {    1.4294038233832270e+01,     2.2405292338581399e+04,     5.2412048360054079e-04,     8.2384737896793840e-07,    -3.7993005896566124e-09, } , 
      {    1.7549848723361450e+01,     3.9272224653862373e+04,     3.9607789368648077e-04,     8.1394748359968168e-07,    -3.7588671286520770e-09, } , 
      {    2.0805659212890628e+01,     5.6139156969143340e+04,     2.6803530377242081e-04,     8.0404758823142495e-07,    -3.7184336676475413e-09, } , 
      {    1.9905403295613347e+01,     4.7600916078587499e+04,     5.4839064432782910e-04,     1.0643585980295681e-06,    -4.9452062063271113e-09, } , 
      {    2.2260957867865251e+01,     5.5929607503312625e+04,     7.0070339496917738e-04,     1.3147697124594545e-06,    -6.1315452840021455e-09, } , 
      {   -3.6932681086535295e+00,    -3.5057696213415278e+04,    -9.7740074357856180e-04,    -1.4391361541551334e-06,     6.8637448421714444e-09, } , 
      {   -4.5935240259308108e+00,    -4.3595937103971133e+04,    -6.9704540302315378e-04,    -1.1788251443569903e-06,     5.6369723034918740e-09, } , 
      {   -1.3377135364016308e+00,    -2.6729004788690156e+04,    -8.2508799293721374e-04,    -1.1887250397252470e-06,     5.6774057644964093e-09, } , 
      {   -2.2379694536789101e+00,    -3.5267245679246007e+04,    -5.4473265238180551e-04,    -9.2841402992710386e-07,     4.4506332258168398e-09, } , 
      {    1.0178410358502699e+00,    -1.8400313363965030e+04,    -6.7277524229586547e-04,    -9.3831392529536059e-07,     4.4910666868213751e-09, } , 
      {    4.2736515253794503e+00,    -1.5333810486840541e+03,    -8.0081783220992543e-04,    -9.4821382066361731e-07,     4.5315001478259104e-09, } , 
      {   -3.1382253709561896e+00,    -4.3805486569801848e+04,    -2.6437731182639716e-04,    -6.6810302012896072e-07,     3.2238606871372702e-09, } , 
      {    1.1758511857299037e-01,    -2.6938554254520877e+04,    -3.9241990174045718e-04,    -6.7800291549721744e-07,     3.2642941481418055e-09, } , 
      {    3.3733956081021708e+00,    -1.0071621939239902e+04,    -5.2046249165451719e-04,    -6.8790281086547417e-07,     3.3047276091463408e-09, } , 
      {    6.6292060976313509e+00,     6.7953103760410722e+03,    -6.4850508156857715e-04,    -6.9780270623373089e-07,     3.3451610701508757e-09, } , 
      {   -7.8267079870429002e-01,    -3.5476795145076721e+04,    -1.1206456118504895e-04,    -4.1769190569907430e-07,     2.0375216094622356e-09, } , 
      {    2.4731396908248904e+00,    -1.8609862829795751e+04,    -2.4010715109910896e-04,    -4.2759180106733102e-07,     2.0779550704667709e-09, } , 
      {    5.7289501803540706e+00,    -1.7429305145147757e+03,    -3.6814974101316892e-04,    -4.3749169643558775e-07,     2.1183885314713062e-09, } , 
      {    8.9847606698832507e+00,     1.5124001800766198e+04,    -4.9619233092722899e-04,    -4.4739159180384447e-07,     2.1588219924758415e-09, } , 
      {    1.2240571159412429e+01,     3.1990934116047174e+04,    -6.2423492084128895e-04,    -4.5729148717210119e-07,     2.1992554534803764e-09, } , 
      {    1.5728837735476102e+00,    -2.7148103720351599e+04,     4.0248189456299287e-05,    -1.6728079126918785e-07,     8.5118253178720121e-10, } , 
      {    4.8286942630767902e+00,    -1.0281171405070623e+04,    -8.7794400457760695e-05,    -1.7718068663744457e-07,     8.9161599279173642e-10, } , 
      {    8.0845047526059695e+00,     6.5857609102103506e+03,    -2.1583699037182070e-04,    -1.8708058200570130e-07,     9.3204945379627173e-10, } , 
      {    1.1340315242135150e+01,     2.3452693225491323e+04,    -3.4387958028588066e-04,    -1.9698047737395802e-07,     9.7248291480080704e-10, } , 
      {    1.4596125731664330e+01,     4.0319625540772300e+04,    -4.7192217019994062e-04,    -2.0688037274221475e-07,     1.0129163758053424e-09, } , 
      {    3.9284383457995107e+00,    -1.8819412295626473e+04,     1.9256094009764753e-04,     8.3130323160698572e-08,    -3.3515654588783324e-10, } , 
      {    7.1842488353286909e+00,    -1.9524799803454973e+03,     6.4518350183587553e-05,     7.3230427792441848e-08,    -2.9472308488329798e-10, } , 
      {    1.0440059324857870e+01,     1.4914452334935477e+04,    -6.3524239730472436e-05,     6.3330532424185123e-08,    -2.5428962387876272e-10, } , 
      {    1.3695869814387050e+01,     3.1781384650216449e+04,    -1.9156682964453244e-04,     5.3430637055928398e-08,    -2.1385616287422749e-10, } , 
      {    1.6951680303916230e+01,     4.8648316965497426e+04,    -3.1960941955859240e-04,     4.3530741687671674e-08,    -1.7342270186969222e-10, } , 
      {    6.2839929180514114e+00,    -1.0490720870901345e+04,     3.4487369073899578e-04,     3.3354143759058497e-07,    -1.5214956235628676e-09, } , 
      {    9.5398034075805906e+00,     6.3762114443796290e+03,     2.1683110082493579e-04,     3.2364154222232824e-07,    -1.4810621625583325e-09, } , 
      {    1.2795613897109771e+01,     2.3243143759660601e+04,     8.8788510910875812e-05,     3.1374164685407152e-07,    -1.4406287015537972e-09, } , 
      {    1.6051424386638949e+01,     4.0110076074941571e+04,    -3.9254079003184183e-05,     3.0384175148581479e-07,    -1.4001952405492619e-09, } , 
      {    1.9307234876168131e+01,     5.6977008390222552e+04,    -1.6729666891724418e-04,     2.9394185611755807e-07,    -1.3597617795447265e-09, } , 
      {    8.6395474903033112e+00,    -2.1620294461762187e+03,     4.9718644138034400e-04,     5.8395255202047139e-07,    -2.7078347012379023e-09, } , 
      {    1.1895357979832490e+01,     1.4704902869104755e+04,     3.6914385146628409e-04,     5.7405265665221466e-07,    -2.6674012402333669e-09, } , 
      {    1.5151168469361671e+01,     3.1571835184385727e+04,     2.4110126155222405e-04,     5.6415276128395794e-07,    -2.6269677792288316e-09, } , 
      {    1.8406978958890850e+01,     4.8438767499666697e+04,     1.1305867163816407e-04,     5.5425286591570121e-07,    -2.5865343182242963e-09, } , 
      {    2.1662789448420032e+01,     6.5305699814947679e+04,    -1.4983918275895924e-05,     5.4435297054744449e-07,    -2.5461008572197614e-09, } , 
      {    1.4250912552084390e+01,     2.3033594293829879e+04,     5.2145660210763220e-04,     8.2446377108210108e-07,    -3.8537403179084016e-09, } , 
      {    1.7506723041613572e+01,     3.9900526609110857e+04,     3.9341401219357229e-04,     8.1456387571384436e-07,    -3.8133068569038663e-09, } , 
      {    2.0762533531142751e+01,     5.6767458924391824e+04,     2.6537142227951233e-04,     8.0466398034558763e-07,    -3.7728733958993310e-09, } , 
      {    1.6606467124336291e+01,     3.1362285718555006e+04,     6.7376935274898048e-04,     1.0748748855119875e-06,    -5.0400793955834358e-09, } , 
      {    1.9862277613865469e+01,     4.8229218033835983e+04,     5.4572676283492052e-04,     1.0649749901437308e-06,    -4.9996459345789005e-09, } , 
      {    2.3118088103394651e+01,     6.5096150349116950e+04,     4.1768417292086061e-04,     1.0550750947754741e-06,    -4.9592124735743652e-09, } , 
      {    1.8962021696588192e+01,     3.9690977143280128e+04,     8.2608210339032875e-04,     1.3252859999418739e-06,    -6.2264184732584701e-09, } , 
      {    2.2217832186117374e+01,     5.6557909458561109e+04,     6.9803951347626879e-04,     1.3153861045736172e-06,    -6.1859850122539348e-09, } , 
      {    2.5473642675646552e+01,     7.3424841773842083e+04,     5.6999692356220883e-04,     1.3054862092053605e-06,    -6.1455515512493995e-09, } , 
      {    2.4573386758369271e+01,     6.4886600883286235e+04,     8.5035226411761696e-04,     1.5657972190035036e-06,    -7.3723240899289699e-09, } , 
      {   -8.4475029349052093e+00,    -5.1086777107617047e+04,    -1.2846901263541669e-03,    -1.9393419909007436e-06,     9.1819832692697228e-09, } , 
      {   -6.0919483626533086e+00,    -4.2758085682891920e+04,    -1.1323773757128187e-03,    -1.6889308764708572e-06,     7.9956441915946886e-09, } , 
      {   -6.9922042799305881e+00,    -5.1296326573447775e+04,    -8.5202203515741043e-04,    -1.4286198666727140e-06,     6.7688716529151190e-09, } , 
      {   -3.7363937904014080e+00,    -3.4429394258166794e+04,    -9.8006462507147039e-04,    -1.4385197620409707e-06,     6.8093051139196543e-09, } , 
      {   -4.6366497076786892e+00,    -4.2967635148722649e+04,    -6.9970928451606226e-04,    -1.1782087522428276e-06,     5.5825325752400839e-09, } , 
      {   -1.3808392181495091e+00,    -2.6100702833441672e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {    1.8749712713796711e+00,    -9.2337705181606980e+03,    -9.5579446434418218e-04,    -1.1980085429793411e-06,     5.6633994972491546e-09, } , 
      {   -5.5369056249559687e+00,    -5.1505876039278490e+04,    -4.1935394396065397e-04,    -9.1789774244468446e-07,     4.3557600365605144e-09, } , 
      {   -2.2810951354267885e+00,    -3.4638943723997523e+04,    -5.4739653387471398e-04,    -9.2779763781294118e-07,     4.3961934975650497e-09, } , 
      {    9.7471535410239163e-01,    -1.7772011408716546e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    4.2305258436315718e+00,    -9.0507909343557185e+02,    -8.0348171370283391e-04,    -9.4759742854945463e-07,     4.4770604195741203e-09, } , 
      {   -3.1813510527040680e+00,    -4.3177184614553364e+04,    -2.6704119331930569e-04,    -6.6748662801479804e-07,     3.1694209588854806e-09, } , 
      {    7.4459436825112135e-02,    -2.6310252299272393e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    3.3302699263542923e+00,    -9.4433199839914196e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.5860804158834725e+00,     7.4236123312895543e+03,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {    9.8418909054126527e+00,     2.4290544646570532e+04,    -7.7921155297554570e-04,    -7.0708620948782494e-07,     3.3311548029036214e-09, } , 
      {   -8.2579648045216825e-01,    -3.4848493189828238e+04,    -1.1472844267795747e-04,    -4.1707551358491162e-07,     1.9830818812104459e-09, } , 
      {    2.4300140090770119e+00,    -1.7981560874547267e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    5.6858244986061921e+00,    -1.1146285592662935e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.9416349881353732e+00,     1.5752303756014680e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.2197445477664552e+01,     3.2619236071295658e+04,    -6.2689880233419743e-04,    -4.5667509505793851e-07,     2.1448157252285867e-09, } , 
      {   -1.7260523977294482e+00,    -4.3386734080384093e+04,     1.6562689787745079e-04,    -1.5676450378676847e-07,     7.5630934253087613e-10, } , 
      {    1.5297580917997320e+00,    -2.6519801765103115e+04,     3.7584307963390776e-05,    -1.6666439915502520e-07,     7.9674280353541145e-10, } , 
      {    4.7855685813289117e+00,    -9.6528694498221412e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    8.0413790708580919e+00,     7.2140628654588327e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1297189560387272e+01,     2.4080995180739807e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.4553000049916452e+01,     4.0947927496020784e+04,    -4.7458605169284915e-04,    -2.0626398062805209e-07,     9.5847664755355249e-10, } , 
      {    1.7808810539445634e+01,     5.7814859811301751e+04,    -6.0262864160690922e-04,    -2.1616387599630882e-07,     9.9891010855808780e-10, } , 
      {    6.2950217452245205e-01,    -3.5058042655658966e+04,     3.1793964851879904e-04,     9.3646610643117964e-08,    -4.3002973514415832e-10, } , 
      {    3.8853126640516322e+00,    -1.8191110340377989e+04,     1.8989705860473902e-04,     8.3746715274861239e-08,    -3.8959627413962306e-10, } , 
      {    7.1411231535808124e+00,    -1.3241780250970151e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0396933643109993e+01,     1.5542754290183959e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3652744132639173e+01,     3.2409686605464933e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.6908554622168353e+01,     4.9276618920745910e+04,    -3.2227330105150093e-04,     4.4147133801834340e-08,    -2.2786243012148202e-10, } , 
      {    2.0164365111697535e+01,     6.6143551236026877e+04,    -4.5031589096556095e-04,     3.4247238433577616e-08,    -1.8742896911694675e-10, } , 
      {    2.9850567467743523e+00,    -2.6729351230933840e+04,     4.7025239916014731e-04,     3.4405772507300437e-07,    -1.6163688128191928e-09, } , 
      {    6.2408672363035329e+00,    -9.8624189156528628e+03,     3.4220980924608724e-04,     3.3415782970474765e-07,    -1.5759353518146575e-09, } , 
      {    9.4966777258327131e+00,     7.0045133996281111e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.2752488215361893e+01,     2.3871445714909085e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6008298704891072e+01,     4.0738378030190055e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    1.9264109194420254e+01,     5.7605310345471036e+04,    -1.6996055041015268e-04,     2.9455824823172075e-07,    -1.4142015077965164e-09, } , 
      {    2.2519919683949436e+01,     7.4472242660752003e+04,    -2.9800314032421267e-04,     2.8465835286346402e-07,    -1.3737680467919811e-09, } , 
      {    8.5964218085554336e+00,    -1.5337274909277367e+03,     4.9452255988743552e-04,     5.8456894413463407e-07,    -2.7622744294896919e-09, } , 
      {    1.1852232298084612e+01,     1.5333204824353237e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    1.5108042787613794e+01,     3.2200137139634211e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.8363853277142972e+01,     4.9067069454915181e+04,     1.1039479014525555e-04,     5.5486925802986389e-07,    -2.6409740464760860e-09, } , 
      {    2.1619663766672154e+01,     6.5934001770196162e+04,    -1.7647799768804435e-05,     5.4496936266160717e-07,    -2.6005405854715511e-09, } , 
      {    1.0951976380807332e+01,     6.7949639337973895e+03,     6.4683531052878379e-04,     8.3498005856452049e-07,    -3.9486135071647270e-09, } , 
      {    1.4207786870336513e+01,     2.3661896249078363e+04,     5.1879272061472372e-04,     8.2508016319626376e-07,    -3.9081800461601917e-09, } , 
      {    1.7463597359865695e+01,     4.0528828564359341e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.0719407849394873e+01,     5.7395760879640307e+04,     2.6270754078660380e-04,     8.0528037245975031e-07,    -3.8273131241511210e-09, } , 
      {    2.3975218338924051e+01,     7.4262693194921289e+04,     1.3466495087254379e-04,     7.9538047709149359e-07,    -3.7868796631465857e-09, } , 
      {    1.6563341442588413e+01,     3.1990587673803489e+04,     6.7110547125607200e-04,     1.0754912776261502e-06,    -5.0945191238352259e-09, } , 
      {    1.9819151932117592e+01,     4.8857519989084467e+04,     5.4306288134201204e-04,     1.0655913822578935e-06,    -5.0540856628306906e-09, } , 
      {    2.3074962421646774e+01,     6.5724452304365434e+04,     4.1502029142795208e-04,     1.0556914868896367e-06,    -5.0136522018261553e-09, } , 
      {    2.6330772911175956e+01,     8.2591384619646415e+04,     2.8697770151389206e-04,     1.0457915915213800e-06,    -4.9732187408216200e-09, } , 
      {    1.8918896014840314e+01,     4.0319279098528612e+04,     8.2341822189742027e-04,     1.3259023920560366e-06,    -6.2808582015102602e-09, } , 
      {    2.2174706504369496e+01,     5.7186211413809593e+04,     6.9537563198336031e-04,     1.3160024966877799e-06,    -6.2404247405057249e-09, } , 
      {    2.5430516993898674e+01,     7.4053143729090560e+04,     5.6733304206930035e-04,     1.3061026013195232e-06,    -6.1999912795011895e-09, } , 
      {    2.1274450587092211e+01,     4.8647970523253738e+04,     9.7573097253876844e-04,     1.5763135064859230e-06,    -7.4671972791852952e-09, } , 
      {    2.4530261076621393e+01,     6.5514902838534719e+04,     8.4768838262470848e-04,     1.5664136111176663e-06,    -7.4267638181807599e-09, } , 
      {    2.7786071566150575e+01,     8.2381835153815686e+04,     7.1964579271064852e-04,     1.5565137157494096e-06,    -7.3863303571762246e-09, } , 
      {    2.6885815648873294e+01,     7.3843594263259845e+04,     1.0000011332660568e-03,     1.8168247255475527e-06,    -8.6131028958557942e-09, } , 
      {   -6.1350740444011871e+00,    -4.2129783727643437e+04,    -1.1350412572057272e-03,    -1.6883144843566945e-06,     7.9412044633428993e-09, } , 
      {   -3.7795194721492864e+00,    -3.3801092302918310e+04,    -9.8272850656437897e-04,    -1.4379033699268081e-06,     6.7548653856678643e-09, } , 
      {   -4.6797753894265677e+00,    -4.2339333193474165e+04,    -7.0237316600897074e-04,    -1.1775923601286649e-06,     5.5280928469882939e-09, } , 
      {   -1.4239648998973873e+00,    -2.5472400878193188e+04,    -8.3041575592303070e-04,    -1.1874922554969216e-06,     5.5685263079928292e-09, } , 
      {   -2.3242208171746670e+00,    -3.4010641768749039e+04,    -5.5006041536762246e-04,    -9.2718124569877850e-07,     4.3417537693132596e-09, } , 
      {    9.3158967235451340e-01,    -1.7143709453468062e+04,    -6.7810300528168242e-04,    -9.3708114106703523e-07,     4.3821872303177949e-09, } , 
      {    4.1874001618836934e+00,    -2.7677713818708958e+02,    -8.0614559519574238e-04,    -9.4698103643529195e-07,     4.4226206913223303e-09, } , 
      {    3.1333755077233905e-02,    -2.5681950344023910e+04,    -3.9774766472627424e-04,    -6.7677013126889208e-07,     3.1554146916382262e-09, } , 
      {    3.2871442446064139e+00,    -8.8150180287429375e+03,    -5.2579025464033415e-04,    -6.8667002663714881e-07,     3.1958481526427615e-09, } , 
      {    6.5429547341355940e+00,     8.0519142865380363e+03,    -6.5383284455439411e-04,    -6.9656992200540553e-07,     3.2362816136472964e-09, } , 
      {   -8.6892216220004648e-01,    -3.4220191234579754e+04,    -1.1739232417086599e-04,    -4.1645912147074894e-07,     1.9286421529586562e-09, } , 
      {    2.3868883273291335e+00,    -1.7353258919298783e+04,    -2.4543491408492596e-04,    -4.2635901683900566e-07,     1.9690756139631916e-09, } , 
      {    5.6426988168583136e+00,    -4.8632660401781118e+02,    -3.7347750399898598e-04,    -4.3625891220726239e-07,     2.0095090749677269e-09, } , 
      {    8.8985093063874956e+00,     1.6380605711263162e+04,    -5.0152009391304594e-04,    -4.4615880757551911e-07,     2.0499425359722622e-09, } , 
      {    1.4866324100518538e+00,    -2.5891499809854631e+04,     3.4920426470482265e-05,    -1.6604800704086254e-07,     7.4230307528362168e-10, } , 
      {    4.7424428995810333e+00,    -9.0245674945736591e+03,    -9.3122163443577730e-05,    -1.7594790240911927e-07,     7.8273653628815689e-10, } , 
      {    7.9982533891102134e+00,     7.8423648207073147e+03,    -2.2116475335763771e-04,    -1.8584779777737599e-07,     8.2316999729269210e-10, } , 
      {    1.1254063878639395e+01,     2.4709297135988290e+04,    -3.4920734327169772e-04,    -1.9574769314563272e-07,     8.6360345829722741e-10, } , 
      {    1.4509874368168575e+01,     4.1576229451269268e+04,    -4.7724993318575768e-04,    -2.0564758851388944e-07,     9.0403691930176272e-10, } , 
      {    3.8421869823037538e+00,    -1.7562808385129505e+04,     1.8723317711183052e-04,     8.4363107389023906e-08,    -4.4403600239141287e-10, } , 
      {    7.0979974718329339e+00,    -6.9587606984853278e+02,     5.9190587197770524e-05,     7.4463212020767181e-08,    -4.0360254138687761e-10, } , 
      {    1.0353807961362115e+01,     1.6171056245432441e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {    1.3609618450891295e+01,     3.3037988560713413e+04,    -1.9689459263034945e-04,     5.4663421284253732e-08,    -3.2273561937780709e-10, } , 
      {    1.6865428940420475e+01,     4.9904920875994394e+04,    -3.2493718254440946e-04,     4.4763525915997007e-08,    -2.8230215837327183e-10, } , 
      {    6.1977415545556545e+00,    -9.2341169604043807e+03,     3.3954592775317871e-04,     3.3477422181891033e-07,    -1.6303750800664473e-09, } , 
      {    9.4535520440848355e+00,     7.6328153548765931e+03,     2.1150333783911878e-04,     3.2487432645065360e-07,    -1.5899416190619122e-09, } , 
      {    1.2709362533614016e+01,     2.4499747670157569e+04,     8.3460747925058777e-05,     3.1497443108239688e-07,    -1.5495081580573769e-09, } , 
      {    1.5965173023143194e+01,     4.1366679985438539e+04,    -4.4581841989001205e-05,     3.0507453571414015e-07,    -1.5090746970528416e-09, } , 
      {    1.9220983512672376e+01,     5.8233612300719520e+04,    -1.7262443190306119e-04,     2.9517464034588343e-07,    -1.4686412360483063e-09, } , 
      {    1.1809106616336734e+01,     1.5961506779601719e+04,     3.6381608848046703e-04,     5.7528544088054002e-07,    -2.7762806967369463e-09, } , 
      {    1.5064917105865916e+01,     3.2828439094882691e+04,     2.3577349856640704e-04,     5.6538554551228330e-07,    -2.7358472357324109e-09, } , 
      {    1.8320727595395095e+01,     4.9695371410163665e+04,     1.0773090865234704e-04,     5.5548565014402657e-07,    -2.6954137747278756e-09, } , 
      {    2.1576538084924277e+01,     6.6562303725444639e+04,    -2.0311681261712945e-05,     5.4558575477576985e-07,    -2.6549803137233407e-09, } , 
      {    1.4164661188588635e+01,     2.4290198204326847e+04,     5.1612883912181525e-04,     8.2569655531042644e-07,    -3.9626197744119817e-09, } , 
      {    1.7420471678117817e+01,     4.1157130519607825e+04,     3.8808624920775523e-04,     8.1579665994216972e-07,    -3.9221863134074464e-09, } , 
      {    2.0676282167646995e+01,     5.8024062834888791e+04,     2.6004365929369527e-04,     8.0589676457391299e-07,    -3.8817528524029111e-09, } , 
      {    1.6520215760840536e+01,     3.2618889629051973e+04,     6.6844158976316352e-04,     1.0761076697403129e-06,    -5.1489588520870160e-09, } , 
      {    1.9776026250369714e+01,     4.9485821944332951e+04,     5.4039899984910356e-04,     1.0662077743720561e-06,    -5.1085253910824807e-09, } , 
      {    2.3031836739898896e+01,     6.6352754259613910e+04,     4.1235640993504355e-04,     1.0563078790037994e-06,    -5.0680919300779454e-09, } , 
      {    2.2131580822621618e+01,     5.7814513369058077e+04,     6.9271175049045184e-04,     1.3166188888019426e-06,    -6.2948644687575149e-09, } , 
      {    2.4487135394873516e+01,     6.6143204793783196e+04,     8.4502450113180000e-04,     1.5670300032318290e-06,    -7.4812035464325492e-09, } , 
      {   -1.4670905816452655e+00,    -2.4844098922944708e+04,    -8.3307963741593928e-04,    -1.1868758633827590e-06,     5.5140865797410399e-09, } , 
      {    8.8846399060663517e-01,    -1.6515407498219582e+04,    -6.8076688677459101e-04,    -9.3646474895287255e-07,     4.3277475020660057e-09, } , 
      {   -1.1791926670644326e-02,    -2.5053648388775429e+04,    -4.0041154621918272e-04,    -6.7615373915472940e-07,     3.1009749633864361e-09, } , 
      {    3.2440185628585358e+00,    -8.1867160734944555e+03,    -5.2845413613324273e-04,    -6.8605363452298613e-07,     3.1414084243909714e-09, } , 
      {    6.4998290523877156e+00,     8.6802162417865184e+03,    -6.5649672604730269e-04,    -6.9595352989124285e-07,     3.1818418853955063e-09, } , 
      {    2.3437626455812555e+00,    -1.6724956964050303e+04,    -2.4809879557783450e-04,    -4.2574262472484303e-07,     1.9146358857114015e-09, } , 
      {    5.5995731351104361e+00,     1.4197535123067109e+02,    -3.7614138549189446e-04,    -4.3564252009309976e-07,     1.9550693467159368e-09, } , 
      {    8.8553836246396163e+00,     1.7008907666511644e+04,    -5.0418397540595453e-04,    -4.4554241546135648e-07,     1.9955028077204721e-09, } , 
      {    1.4435067283039755e+00,    -2.5263197854606151e+04,     3.2256544977573754e-05,    -1.6543161492669986e-07,     6.8786334703183181e-10, } , 
      {    4.6993172178331548e+00,    -8.3962655393251771e+03,    -9.5786044936486234e-05,    -1.7533151029495659e-07,     7.2829680803636702e-10, } , 
      {    7.9551277073623350e+00,     8.4706667759557968e+03,    -2.2382863485054624e-04,    -1.8523140566321331e-07,     7.6873026904090223e-10, } , 
      {    1.1210938196891515e+01,     2.5337599091236771e+04,    -3.5187122476460620e-04,    -1.9513130103147004e-07,     8.0916373004543754e-10, } , 
      {    1.4466748686420695e+01,     4.2204531406517752e+04,    -4.7991381467866616e-04,    -2.0503119639972676e-07,     8.4959719104997286e-10, } , 
      {    3.7990613005558758e+00,    -1.6934506429881025e+04,     1.8456929561892199e-04,     8.4979499503186573e-08,    -4.9847573064320264e-10, } , 
      {    7.0548717900850555e+00,    -6.7574114600050507e+01,     5.6526705704862013e-05,     7.5079604134929848e-08,    -4.5804226963866738e-10, } , 
      {    1.0310682279614236e+01,     1.6799358200680923e+04,    -7.1515884209197975e-05,     6.5179708766673123e-08,    -4.1760880863413212e-10, } , 
      {    1.3566492769143416e+01,     3.3666290515961897e+04,    -1.9955847412325798e-04,     5.5279813398416392e-08,    -3.7717534762959686e-10, } , 
      {    9.4104263623369562e+00,     8.2611173101250752e+03,     2.0883945634621025e-04,     3.2549071856481623e-07,    -1.6443813473137019e-09, } , 
      {    1.2666236851866136e+01,     2.5128049625406049e+04,     8.0796866432150273e-05,     3.1559082319655950e-07,    -1.6039478863091666e-09, } , 
      {    1.5922047341395315e+01,     4.1994981940687023e+04,    -4.7245723481909716e-05,     3.0569092782830278e-07,    -1.5635144253046313e-09, } , 
      {    1.1765980934588855e+01,     1.6589808734850201e+04,     3.6115220698755855e-04,     5.7590183299470270e-07,    -2.8307204249887363e-09, } , 
      {    1.5021791424118037e+01,     3.3456741050131175e+04,     2.3310961707349851e-04,     5.6600193762644598e-07,    -2.7902869639842010e-09, } , 
      {    1.4121535506840756e+01,     2.4918500159575327e+04,     5.1346495762890666e-04,     8.2631294742458912e-07,    -4.0170595026637710e-09, } , 
      {    3.2008928811106578e+00,    -7.5584141182459725e+03,    -5.3111801762615121e-04,    -6.8543724240882345e-07,     3.0869686961391818e-09, } , 
      {    5.5564474533625576e+00,     7.7027730647915337e+02,    -3.7880526698480299e-04,    -4.3502612797893708e-07,     1.9006296184641471e-09, } , 
      {    4.6561915360852772e+00,    -7.7679635840766941e+03,    -9.8449926429394752e-05,    -1.7471511818079393e-07,     6.7385707978457726e-10, } , 
      {    7.9120020256144574e+00,     9.0989687312042788e+03,    -2.2649251634345474e-04,    -1.8461501354905066e-07,     7.1429054078911247e-10, } , 
      {    7.0117461083371779e+00,     5.6072784064843177e+02,     5.3862824211953496e-05,     7.5695996249092501e-08,    -5.1248199789045720e-10, } , 
      {    1.0267556597866358e+01,     1.7427660155929407e+04,    -7.4179765702106493e-05,     6.5796100880835777e-08,    -4.7204853688592199e-10, } , 
      {    9.3673006805890786e+00,     8.8894192653735572e+03,     2.0617557485330174e-04,     3.2610711067897891e-07,    -1.6988210755654918e-09, } , 
      {    1.2623111170118259e+01,     2.5756351580654533e+04,     7.8132984939241755e-05,     3.1620721531072218e-07,    -1.6583876145609564e-09, } , 
      {    5.5133217716146792e+00,     1.3985792617276356e+03,    -3.8146914847771152e-04,    -4.3440973586477440e-07,     1.8461898902123573e-09, } , 
      {    7.8688763438665790e+00,     9.7272706864527609e+03,    -2.2915639783636324e-04,    -1.8399862143488798e-07,     6.5985081253732270e-10, } , 
      {    1.3412348619404600e+01,     1.2472232189556882e+04,    -2.4093940650478566e-04,    -1.5695629607901540e-07,     9.4100396248636494e-10, } , 
      {    1.5767903191656501e+01,     2.0800923614282008e+04,    -8.8626655863437387e-05,     9.3454818350871037e-08,    -2.4533511518866962e-10, } , 
      {    1.1013668365404824e+01,     4.7718427200802371e+03,    -3.9591603863904239e-04,    -4.0675101839473916e-07,     2.0729033119096095e-09, } , 
      {    1.3369222937656723e+01,     1.3100534144805364e+04,    -2.4360328799769417e-04,    -1.5633990396485272e-07,     8.8656423423457507e-10, } , 
      {    1.2468967020379443e+01,     4.5622932542495155e+03,     3.6752052557714098e-05,     1.0397110583329043e-07,    -3.4020830444499469e-10, } , 
      {    1.5724777509908623e+01,     2.1429225569530488e+04,    -9.1290537356345891e-05,     9.4071210465033704e-08,    -2.9977484344045938e-10, } , 
      {    1.8080332082160524e+01,     2.9757916994255615e+04,     6.1022213285002357e-05,     3.4448232489492010e-07,    -1.4861139211154938e-09, } , 
      {    8.6149881114050455e+00,    -2.9285467493964065e+03,    -5.5089267077329908e-04,    -6.5654574071046296e-07,     3.2048026613328545e-09, } , 
      {    1.0970542683656946e+01,     5.4001446753287200e+03,    -3.9857992013195092e-04,    -4.0613462628057648e-07,     2.0184635836578199e-09, } , 
      {    1.0070286766379665e+01,    -3.1380962152271281e+03,    -1.1822457957654267e-04,    -1.4582361648243334e-07,     7.9169104497824999e-10, } , 
      {    1.3326097255908845e+01,     1.3728836100053846e+04,    -2.4626716949060270e-04,    -1.5572351185069006e-07,     8.3212450598278531e-10, } , 
      {    1.6581907745438023e+01,     3.0595768415334820e+04,    -3.7430975940466266e-04,    -1.6562340721894679e-07,     8.7255796698732062e-10, } , 
      {    1.2425841338631566e+01,     5.1905952094979984e+03,     3.4088171064805580e-05,     1.0458749794745308e-07,    -3.9464803269678451e-10, } , 
      {    1.5681651828160746e+01,     2.2057527524778972e+04,    -9.3954418849254408e-05,     9.4687602579196358e-08,    -3.5421457169224920e-10, } , 
      {    1.4781395910883466e+01,     1.3519286634223125e+04,     1.8640092170615384e-04,     3.5499861237733950e-07,    -1.5809871103718190e-09, } , 
      {    1.8037206400412643e+01,     3.0386218949504098e+04,     5.8358331792093839e-05,     3.4509871700908278e-07,    -1.5405536493672837e-09, } , 
      {    6.2163078574052664e+00,    -1.0628936218873050e+04,    -7.0586930290755595e-04,    -9.0634046302618670e-07,     4.3367020107560987e-09, } , 
      {    8.5718624296571662e+00,    -2.3002447941479240e+03,    -5.5355655226620767e-04,    -6.5592934859630028e-07,     3.1503629330810644e-09, } , 
      {    1.0927417001909067e+01,     6.0284466305772021e+03,    -4.0124380162485939e-04,    -4.0551823416641386e-07,     1.9640238554060298e-09, } , 
      {    1.4183227491438245e+01,     2.2895378945858174e+04,    -5.2928639153891946e-04,    -4.1541812953467058e-07,     2.0044573164105651e-09, } , 
      {    1.0027161084631786e+01,    -2.5097942599786456e+03,    -1.2088846106945117e-04,    -1.4520722436827066e-07,     7.3725131672646013e-10, } , 
      {    1.3282971574160966e+01,     1.4357138055302328e+04,    -2.4893105098351117e-04,    -1.5510711973652738e-07,     7.7768477773099544e-10, } , 
      {    1.6538782063690146e+01,     3.1224070370583300e+04,    -3.7697364089757114e-04,    -1.6500701510478411e-07,     8.1811823873553075e-10, } , 
      {    9.1269051673545061e+00,    -1.1048035150534493e+04,     1.5946687948595705e-04,     1.1510378542987247e-07,    -4.8952122195310954e-10, } , 
      {    1.2382715656883686e+01,     5.8188971647464805e+03,     3.1424289571897069e-05,     1.0520389006161575e-07,    -4.4908776094857428e-10, } , 
      {    1.5638526146412866e+01,     2.2685829480027453e+04,    -9.6618300342162913e-05,     9.5303994693359024e-08,    -4.0865429994403896e-10, } , 
      {    1.8894336635942047e+01,     3.9552761795308426e+04,    -2.2466089025622291e-04,     8.5404099325102300e-08,    -3.6822083893950370e-10, } , 
      {    1.4738270229135587e+01,     1.4147588589471607e+04,     1.8373704021324531e-04,     3.5561500449150213e-07,    -1.6354268386236087e-09, } , 
      {    1.7994080718664765e+01,     3.1014520904752579e+04,     5.5694450299185329e-05,     3.4571510912324541e-07,    -1.5949933776190734e-09, } , 
      {    2.1249891208193947e+01,     4.7881453220033552e+04,    -7.2348139614874653e-05,     3.3581521375498868e-07,    -1.5545599166145381e-09, } , 
      {    1.7093824801387488e+01,     2.2476280014196731e+04,     3.3604979085459361e-04,     6.0602611892138855e-07,    -2.8217659162986433e-09, } , 
      {    2.0349635290916666e+01,     3.9343212329477705e+04,     2.0800720094053357e-04,     5.9612622355313183e-07,    -2.7813324552941080e-09, } , 
      {    2.3605445780445848e+01,     5.6210144644758679e+04,     7.9964611026473595e-05,     5.8622632818487510e-07,    -2.7408989942895727e-09, } , 
      {    2.2705189863168567e+01,     4.7671903754202831e+04,     3.6031995158188182e-04,     8.4653733798301825e-07,    -3.9676715329691427e-09, } , 
      {    3.8176276034054872e+00,    -1.8329325688349694e+04,    -8.6084593504181270e-04,    -1.1561351853419104e-06,     5.4686013601793429e-09, } , 
      {    2.9173716861282077e+00,    -2.6867566578905542e+04,    -5.8049059448640446e-04,    -8.9582417554376729e-07,     4.2418288214997733e-09, } , 
      {    6.1731821756573879e+00,    -1.0000634263624568e+04,    -7.0853318440046442e-04,    -9.0572407091202402e-07,     4.2822622825043086e-09, } , 
      {    5.2729262583801084e+00,    -1.8538875154180416e+04,    -4.2817784384505619e-04,    -6.4541306111388087e-07,     3.0554897438247395e-09, } , 
      {    8.5287367479092886e+00,    -1.6719428388994420e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    1.1784547237438469e+01,     1.5194989476381532e+04,    -6.8426302367317611e-04,    -6.6521285185039432e-07,     3.1363566658338101e-09, } , 
      {    7.6284808306320082e+00,    -1.0210183729455290e+04,    -2.7586509320370791e-04,    -3.9500194668399445e-07,     1.8691506661497048e-09, } , 
      {    1.0884291320161189e+01,     6.6567485858256841e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.4140101809690368e+01,     2.3523680901106658e+04,    -5.3195027303182794e-04,    -4.1480173742050790e-07,     1.9500175881587754e-09, } , 
      {    1.7395912299219546e+01,     4.0390613216387632e+04,    -6.5999286294588790e-04,    -4.2470163278876463e-07,     1.9904510491633108e-09, } , 
      {    6.7282249133547278e+00,    -1.8748424620011137e+04,     4.4902473517002990e-06,    -1.3469093688585128e-07,     6.4237812747013515e-10, } , 
      {    9.9840354028839080e+00,    -1.8814923047301636e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.3239845892413088e+01,     1.4985440010550810e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6495656381942268e+01,     3.1852372325831784e+04,    -3.7963752239047967e-04,    -1.6439062299062145e-07,     7.6367851048374099e-10, } , 
      {    1.9751466871471447e+01,     4.8719304641112758e+04,    -5.0768011230453963e-04,    -1.7429051835887818e-07,     8.0411197148827619e-10, } , 
      {    9.0837794856066285e+00,    -1.0419733195286011e+04,     1.5680299799304855e-04,     1.1572017754403514e-07,    -5.4396095020489930e-10, } , 
      {    1.2339589975135809e+01,     6.4471991199949625e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.5595400464664989e+01,     2.3314131435275936e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.8851210954194169e+01,     4.0181063750556910e+04,    -2.2732477174913142e-04,     8.6020491439264966e-08,    -4.2266056719129352e-10, } , 
      {    2.2107021443723347e+01,     5.7047996065837884e+04,    -3.5536736166319141e-04,     7.6120596071008242e-08,    -3.8222710618675826e-10, } , 
      {    1.1439334057858529e+01,    -2.0910417705608852e+03,     3.0911574863439677e-04,     3.6613129197392153e-07,    -1.7303000278799337e-09, } , 
      {    1.4695144547387709e+01,     1.4775890544720089e+04,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {    1.7950955036916888e+01,     3.1642822860001063e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.1206765526446070e+01,     4.8509755175282036e+04,    -7.5012021107783171e-05,     3.3643160586915136e-07,    -1.6089996448663279e-09, } , 
      {    1.7050699119639610e+01,     2.3104581969445215e+04,     3.3338590936168508e-04,     6.0664251103555123e-07,    -2.8762056445504330e-09, } , 
      {    2.0306509609168788e+01,     3.9971514284726189e+04,     2.0534331944762507e-04,     5.9674261566729451e-07,    -2.8357721835458977e-09, } , 
      {    2.3562320098697970e+01,     5.6838446600007162e+04,     7.7300729533565077e-05,     5.8684272029903778e-07,    -2.7953387225413624e-09, } , 
      {    1.9406253691891507e+01,     3.1433273394170341e+04,     4.8569866000303325e-04,     8.5705362546543765e-07,    -4.0625447222254681e-09, } , 
      {    2.2662064181420689e+01,     4.8300205709451315e+04,     3.5765607008897329e-04,     8.4715373009718093e-07,    -4.0221112612209328e-09, } , 
      {    2.5917874670949871e+01,     6.5167138024732289e+04,     2.2961348017491330e-04,     8.3725383472892420e-07,    -3.9816778002163974e-09, } , 
      {    2.5017618753672590e+01,     5.6628897134176441e+04,     5.0996882073032156e-04,     1.0975648445270673e-06,    -5.2084503388959670e-09, } , 
      {    3.7745019216576088e+00,    -1.7701023733101210e+04,    -8.6350981653472117e-04,    -1.1555187932277478e-06,     5.4141616319275528e-09, } , 
      {    6.1300564939095095e+00,    -9.3723323083760861e+03,    -7.1119706589337290e-04,    -9.0510767879786134e-07,     4.2278225542525185e-09, } , 
      {    5.2298005766322300e+00,    -1.7910573198931932e+04,    -4.3084172533796472e-04,    -6.4479666899971819e-07,     3.0010500155729498e-09, } , 
      {    8.4856110661614110e+00,    -1.0436408836509597e+03,    -5.5888431525202462e-04,    -6.5469656436797492e-07,     3.0414834765774851e-09, } , 
      {    1.1741421555690591e+01,     1.5823291431630014e+04,    -6.8692690516608459e-04,    -6.6459645973623164e-07,     3.0819169375820204e-09, } , 
      {    7.5853551488841298e+00,    -9.5818817742068077e+03,    -2.7852897469661644e-04,    -3.9438555456983177e-07,     1.8147109378979149e-09, } , 
      {    1.0841165638413312e+01,     7.2850505410741662e+03,    -4.0657156461067646e-04,    -4.0428544993808850e-07,     1.8551443989024503e-09, } , 
      {    1.4096976127942490e+01,     2.4151982856355142e+04,    -5.3461415452473642e-04,    -4.1418534530634522e-07,     1.8955778599069858e-09, } , 
      {    9.9409097211360304e+00,    -1.2531903494816813e+03,    -1.2621622405526819e-04,    -1.4397444013994535e-07,     6.2837186022288060e-10, } , 
      {    1.3196720210665211e+01,     1.5613741965799292e+04,    -2.5425881396932824e-04,    -1.5387433550820208e-07,     6.6880532122741591e-10, } , 
      {    1.6452530700194391e+01,     3.2480674281080268e+04,    -3.8230140388338820e-04,    -1.6377423087645880e-07,     7.0923878223195122e-10, } , 
      {    1.2296464293387931e+01,     7.0755010752434446e+03,     2.6096526586080048e-05,     1.0643667428994108e-07,    -5.5796721745215386e-10, } , 
      {    1.5552274782917111e+01,     2.3942433390524420e+04,    -1.0194606332797995e-04,     9.6536778921684358e-08,    -5.1753375644761854e-10, } , 
      {    1.8808085272446291e+01,     4.0809365705805394e+04,    -2.2998865324203992e-04,     8.6636883553427633e-08,    -4.7710029544308334e-10, } , 
      {    1.4652018865639832e+01,     1.5404192499968571e+04,     1.7840927722742830e-04,     3.5684778871982749e-07,    -1.7443062951271884e-09, } , 
      {    1.7907829355169010e+01,     3.2271124815249546e+04,     5.0366687313368307e-05,     3.4694789335157077e-07,    -1.7038728341226531e-09, } , 
      {    2.1163639844698192e+01,     4.9138057130530520e+04,    -7.7675902600691688e-05,     3.3704799798331404e-07,    -1.6634393731181178e-09, } , 
      {    1.7007573437891732e+01,     2.3732883924693699e+04,     3.3072202786877655e-04,     6.0725890314971391e-07,    -2.9306453728022227e-09, } , 
      {    2.0263383927420911e+01,     4.0599816239974672e+04,     2.0267943795471656e-04,     5.9735900778145719e-07,    -2.8902119117976873e-09, } , 
      {    2.3519194416950093e+01,     5.7466748555255646e+04,     7.4636848040656560e-05,     5.8745911241320046e-07,    -2.8497784507931520e-09, } , 
      {    2.2618938499672812e+01,     4.8928507664699799e+04,     3.5499218859606476e-04,     8.4777012221134361e-07,    -4.0765509894727228e-09, } , 
      {    2.4974493071924712e+01,     5.7257199089424925e+04,     5.0730493923741308e-04,     1.0981812366412300e-06,    -5.2628900671477571e-09, } , 
      {    6.0869308121616310e+00,    -8.7440303531276040e+03,    -7.1386094738628148e-04,    -9.0449128668369866e-07,     4.1733828260007293e-09, } , 
      {    8.4424853844135317e+00,    -4.1533892840247745e+02,    -5.6154819674493321e-04,    -6.5408017225381224e-07,     2.9870437483256950e-09, } , 
      {    1.0798039956665432e+01,     7.9133524963226482e+03,    -4.0923544610358493e-04,    -4.0366905782392587e-07,     1.8007046706506606e-09, } , 
      {    9.8977840393881511e+00,    -6.2488839423319905e+02,    -1.2888010554817672e-04,    -1.4335804802578267e-07,     5.7393213197109073e-10, } , 
      {    1.3153594528917331e+01,     1.6242043921047774e+04,    -2.5692269546223671e-04,    -1.5325794339403940e-07,     6.1436559297562604e-10, } , 
      {    1.2253338611640052e+01,     7.7038030304919266e+03,     2.3432645093171537e-05,     1.0705306640410375e-07,    -6.1240694570394373e-10, } , 
      {    1.5509149101169232e+01,     2.4570735345772900e+04,    -1.0460994482088845e-04,     9.7153171035847025e-08,    -5.7197348469940841e-10, } , 
      {    1.8764959590698414e+01,     4.1437667661053878e+04,    -2.3265253473494845e-04,     8.7253275667590300e-08,    -5.3154002369487310e-10, } , 
      {    1.7864703673421133e+01,     3.2899426770498030e+04,     4.7702805820459796e-05,     3.4756428546573339e-07,    -1.7583125623744428e-09, } , 
      {    2.0220258245673033e+01,     4.1228118195223156e+04,     2.0001555646180803e-04,     5.9797539989561987e-07,    -2.9446516400494774e-09, } , 
      {    8.3993597026656541e+00,     2.1296302684600482e+02,    -5.6421207823784169e-04,    -6.5346378013964956e-07,     2.9326040200739054e-09, } , 
      {    7.4991037853883737e+00,    -8.3252778637098436e+03,    -2.8385673768243345e-04,    -3.9315277034150647e-07,     1.7058314813943354e-09, } , 
      {    1.0754914274917555e+01,     8.5416544515711303e+03,    -4.1189932759649347e-04,    -4.0305266570976319e-07,     1.7462649423988707e-09, } , 
      {    1.3110468847169454e+01,     1.6870345876296258e+04,    -2.5958657695514524e-04,    -1.5264155127987674e-07,     5.5992586472383628e-10, } , 
      {    1.2210212929892174e+01,     8.3321049857404087e+03,     2.0768763600263023e-05,     1.0766945851826640e-07,    -6.6684667395573349e-10, } , 
      {    1.5466023419421354e+01,     2.5199037301021384e+04,    -1.0727382631379697e-04,     9.7769563150009678e-08,    -6.2641321295119818e-10, } , 
      {    1.6298386550455575e+01,     1.1286615954675253e+04,    -4.2368233626491586e-04,    -3.7601034035389058e-07,     2.0274180923479129e-09, } , 
      {    1.8653941122707476e+01,     1.9615307379400379e+04,    -2.7136958562356763e-04,    -1.2559922592400416e-07,     8.4107901467287841e-10, } , 
      {    2.1009495694959377e+01,     2.7943998804125506e+04,    -1.1905683498221935e-04,     1.2481188850588226e-07,    -3.4526006300215609e-10, } , 
      {    1.3899706296455797e+01,     3.5862264851986101e+03,    -5.7865896839917261e-04,    -6.2580506266961432e-07,     3.1593174417711574e-09, } , 
      {    1.6255260868707698e+01,     1.1914917909923737e+04,    -4.2634621775782433e-04,    -3.7539394823972796e-07,     1.9729783640961228e-09, } , 
      {    1.8610815440959598e+01,     2.0243609334648863e+04,    -2.7403346711647611e-04,    -1.2498283380984151e-07,     7.8663928642108854e-10, } , 
      {    2.0966370013211499e+01,     2.8572300759373989e+04,    -1.2172071647512786e-04,     1.2542828062004491e-07,    -3.9969979125394591e-10, } , 
      {    2.3321924585463400e+01,     3.6900992184099116e+04,     3.0592034166220384e-05,     3.7583939504993131e-07,    -1.5860388689289802e-09, } , 
      {    2.5677479157715300e+01,     4.5229683608824242e+04,     1.8290478480756863e-04,     6.2625050947981778e-07,    -2.7723779466040150e-09, } , 
      {    1.3856580614707919e+01,     4.2145284404470922e+03,    -5.8132284989208108e-04,    -6.2518867055545175e-07,     3.1048777135193678e-09, } , 
      {    1.6212135186959820e+01,     1.2543219865172219e+04,    -4.2901009925073286e-04,    -3.7477755612556528e-07,     1.9185386358443331e-09, } , 
      {    1.9467945676489002e+01,     2.9410152180453191e+04,    -5.5705268916479288e-04,    -3.8467745149382200e-07,     1.9589720968488684e-09, } , 
      {    1.5311879269682537e+01,     4.0049789746163706e+03,    -1.4865475869532463e-04,    -1.1446654632742210e-07,     6.9176609716476357e-10, } , 
      {    1.8567689759211721e+01,     2.0871911289897344e+04,    -2.7669734860938464e-04,    -1.2436644169567883e-07,     7.3219955816929878e-10, } , 
      {    2.1823500248740903e+01,     3.7738843605178321e+04,    -4.0473993852344460e-04,    -1.3426633706393555e-07,     7.7263301917383399e-10, } , 
      {    1.7667433841934439e+01,     1.2333670399341498e+04,     3.6579919460236176e-06,     1.3594456810246432e-07,    -4.9457298051027088e-10, } , 
      {    2.0923244331463621e+01,     2.9200602714622470e+04,    -1.2438459796803637e-04,     1.2604467273420759e-07,    -4.5413951950573568e-10, } , 
      {    2.4179054820992803e+01,     4.6067535029903447e+04,    -2.5242718788209633e-04,     1.1614477736595087e-07,    -4.1370605850120047e-10, } , 
      {    2.0022988414186340e+01,     2.0662361824066622e+04,     1.5597074258737187e-04,     3.8635568253235071e-07,    -1.6809120581853053e-09, } , 
      {    2.3278798903715522e+01,     3.7529294139347599e+04,     2.7928152673311877e-05,     3.7645578716409399e-07,    -1.6404785971807700e-09, } , 
      {    2.6534609393244704e+01,     5.4396226454628573e+04,    -1.0011443724074808e-04,     3.6655589179583726e-07,    -1.6000451361762349e-09, } , 
      {    2.5634353475967423e+01,     4.5857985564072726e+04,     1.8024090331466013e-04,     6.2686690159398035e-07,    -2.8268176748558047e-09, } , 
      {    2.7989908048219320e+01,     5.4186676988797852e+04,     3.3255365395600835e-04,     8.7727801602386678e-07,    -4.0131567525308389e-09, } , 
      {    9.1023457884562404e+00,    -1.1814552453754677e+04,    -8.8861223266768611e-04,    -1.1253945073010619e-06,     5.4231161406176466e-09, } , 
      {    1.1457900360708141e+01,    -3.4858610290295514e+03,    -7.3629948202633784e-04,    -8.7498339287117549e-07,     4.2367770629426124e-09, } , 
      {    1.3813454932960042e+01,     4.8428303956955751e+03,    -5.8398673138498956e-04,    -6.2457227844128907e-07,     3.0504379852675781e-09, } , 
      {    1.7069265422489224e+01,     2.1709762710976549e+04,    -7.1202932129904952e-04,    -6.3447217380954579e-07,     3.0908714462721134e-09, } , 
      {    1.2913199015682759e+01,    -3.6954104948602730e+03,    -3.0363139082958138e-04,    -3.6426126864314587e-07,     1.8236654465880082e-09, } , 
      {    1.6169009505211939e+01,     1.3171521820420701e+04,    -4.3167398074364139e-04,    -3.7416116401140260e-07,     1.8640989075925435e-09, } , 
      {    1.9424819994741121e+01,     3.0038454135701675e+04,    -5.5971657065770136e-04,    -3.8406105937965932e-07,     1.9045323685970788e-09, } , 
      {    1.5268753587934659e+01,     4.6332809298648535e+03,    -1.5131864018823313e-04,    -1.1385015421325945e-07,     6.3732636891297380e-10, } , 
      {    1.8524564077463840e+01,     2.1500213245145827e+04,    -2.7936123010229317e-04,    -1.2375004958151618e-07,     6.7775982991750901e-10, } , 
      {    2.1780374566993022e+01,     3.8367145560426798e+04,    -4.0740382001635314e-04,    -1.3364994494977290e-07,     7.1819329092204422e-10, } , 
      {    1.7624308160186558e+01,     1.2961972354589980e+04,     9.9411045311510506e-07,     1.3656096021662697e-07,    -5.4901270876206065e-10, } , 
      {    2.0880118649715740e+01,     2.9828904669870954e+04,    -1.2704847946094487e-04,     1.2666106484837025e-07,    -5.0857924775752544e-10, } , 
      {    2.4135929139244922e+01,     4.6695836985151924e+04,    -2.5509106937500486e-04,     1.1676116948011352e-07,    -4.6814578675299023e-10, } , 
      {    1.9979862732438459e+01,     2.1290663779315106e+04,     1.5330686109446336e-04,     3.8697207464651339e-07,    -1.7353517864370952e-09, } , 
      {    2.3235673221967641e+01,     3.8157596094596076e+04,     2.5264271180403363e-05,     3.7707217927825667e-07,    -1.6949183254325599e-09, } , 
      {    2.6491483711496823e+01,     5.5024528409877050e+04,    -1.0277831873365660e-04,     3.6717228390999994e-07,    -1.6544848644280248e-09, } , 
      {    2.2335417304690360e+01,     2.9619355204040232e+04,     3.0561961173581161e-04,     6.3738318907639976e-07,    -2.9216908641121297e-09, } , 
      {    2.5591227794219542e+01,     4.6486287519321202e+04,     1.7757702182175162e-04,     6.2748329370814303e-07,    -2.8812574031075943e-09, } , 
      {    2.8847038283748724e+01,     6.3353219834602176e+04,     4.9534431907691649e-05,     6.1758339833988631e-07,    -2.8408239421030590e-09, } , 
      {    2.4690971876942257e+01,     3.7948046628765354e+04,     4.5793236237715978e-04,     8.8779430350628618e-07,    -4.1080299417871643e-09, } , 
      {    2.7946782366471439e+01,     5.4814978944046328e+04,     3.2988977246309982e-04,     8.7789440813802946e-07,    -4.0675964807826290e-09, } , 
      {    3.0302336938723343e+01,     6.3143670368771454e+04,     4.8220252310444809e-04,     1.1283055225679159e-06,    -5.2539355584576633e-09, } , 
      {    6.7036655344564631e+00,    -1.9514941923231323e+04,    -1.0435888648019428e-03,    -1.3751892296167857e-06,     6.5550154900408924e-09, } , 
      {    9.0592201067083611e+00,    -1.1186250498506195e+04,    -8.9127611416059470e-04,    -1.1247781151868992e-06,     5.3686764123658574e-09, } , 
      {    8.1589641894310798e+00,    -1.9724491389062045e+04,    -6.1092077360518646e-04,    -8.6446710538875609e-07,     4.1419038736862878e-09, } , 
      {    1.1414774678960262e+01,    -2.8575590737810689e+03,    -7.3896336351924642e-04,    -8.7436700075701281e-07,     4.1823373346908231e-09, } , 
      {    1.0514518761682980e+01,    -1.1395799964336917e+04,    -4.5860802296383813e-04,    -6.1405599095886966e-07,     2.9555647960112532e-09, } , 
      {    1.3770329251212162e+01,     5.4711323509440572e+03,    -5.8665061287789815e-04,    -6.2395588632712639e-07,     2.9959982570157880e-09, } , 
      {    1.7026139740741346e+01,     2.2338064666225029e+04,    -7.1469320279195811e-04,    -6.3385578169538311e-07,     3.0364317180203234e-09, } , 
      {    9.6142628444057010e+00,    -1.9934040854892766e+04,    -1.7825268240842992e-04,    -3.5374498116072652e-07,     1.7287922573316832e-09, } , 
      {    1.2870073333934879e+01,    -3.0671085396117905e+03,    -3.0629527232248986e-04,    -3.6364487652898324e-07,     1.7692257183362185e-09, } , 
      {    1.6125883823464061e+01,     1.3799823775669183e+04,    -4.3433786223654987e-04,    -3.7354477189723997e-07,     1.8096591793407536e-09, } , 
      {    1.9381694312993243e+01,     3.0666756090950155e+04,    -5.6238045215060994e-04,    -3.8344466726549669e-07,     1.8500926403452889e-09, } , 
      {    2.2637504802522422e+01,     4.7533688406231129e+04,    -6.9042304206466990e-04,    -3.9334456263375342e-07,     1.8905261013498240e-09, } , 
      {    1.1969817416657602e+01,    -1.1605349430167638e+04,    -2.5939931767081667e-05,    -1.0333386673084006e-07,     5.4245317965664862e-10, } , 
      {    1.5225627906186780e+01,     5.2615828851133356e+03,    -1.5398252168114166e-04,    -1.1323376209909678e-07,     5.8288664066118393e-10, } , 
      {    1.8481438395715962e+01,     2.2128515200394308e+04,    -2.8202511159520165e-04,    -1.2313365746735350e-07,     6.2332010166571914e-10, } , 
      {    2.1737248885245144e+01,     3.8995447515675281e+04,    -4.1006770150926161e-04,    -1.3303355283561022e-07,     6.6375356267025435e-10, } , 
      {    2.4993059374774322e+01,     5.5862379830956255e+04,    -5.3811029142332163e-04,    -1.4293344820386694e-07,     7.0418702367478967e-10, } , 
      {    1.4325371988909502e+01,    -3.2766580054425121e+03,     1.2637281887426657e-04,     1.4707724769904638e-07,    -6.4388589801838583e-10, } , 
      {    1.7581182478438681e+01,     1.3590274309838462e+04,    -1.6697710397934066e-06,     1.3717735233078965e-07,    -6.0345243701385052e-10, } , 
      {    2.0836992967967863e+01,     3.0457206625119434e+04,    -1.2971236095385340e-04,     1.2727745696253293e-07,    -5.6301897600931531e-10, } , 
      {    2.4092803457497045e+01,     4.7324138940400408e+04,    -2.5775495086791334e-04,     1.1737756159427619e-07,    -5.2258551500478010e-10, } , 
      {    2.7348613947026223e+01,     6.4191071255681381e+04,    -3.8579754078197335e-04,     1.0747766622601946e-07,    -4.8215205400024479e-10, } , 
      {    1.6680926561161403e+01,     5.0520334192826140e+03,     2.7868556951561482e-04,     3.9748836212893274e-07,    -1.8302249756934202e-09, } , 
      {    1.9936737050690581e+01,     2.1918965734563586e+04,     1.5064297960155483e-04,     3.8758846676067602e-07,    -1.7897915146888849e-09, } , 
      {    2.3192547540219763e+01,     3.8785898049844560e+04,     2.2600389687494852e-05,     3.7768857139241929e-07,    -1.7493580536843496e-09, } , 
      {    2.6448358029748945e+01,     5.5652830365125534e+04,    -1.0544220022656510e-04,     3.6778867602416257e-07,    -1.7089245926798144e-09, } , 
      {    2.9704168519278124e+01,     7.2519762680406522e+04,    -2.3348479014062513e-04,     3.5788878065590584e-07,    -1.6684911316752791e-09, } , 
      {    2.2292291622942482e+01,     3.0247657159288712e+04,     3.0295573024290314e-04,     6.3799958119056244e-07,    -2.9761305923639197e-09, } , 
      {    2.5548102112471664e+01,     4.7114589474569686e+04,     1.7491314032884309e-04,     6.2809968582230571e-07,    -2.9356971313593844e-09, } , 
      {    2.8803912602000846e+01,     6.3981521789850660e+04,     4.6870550414783138e-05,     6.1819979045404899e-07,    -2.8952636703548491e-09, } , 
      {    2.4647846195194379e+01,     3.8576348584013838e+04,     4.5526848088425130e-04,     8.8841069562044886e-07,    -4.1624696700389536e-09, } , 
      {    2.7903656684723561e+01,     5.5443280899294812e+04,     3.2722589097019134e-04,     8.7851080025219214e-07,    -4.1220362090344183e-09, } , 
      {    3.1159467174252743e+01,     7.2310213214575793e+04,     1.9918330105613135e-04,     8.6861090488393541e-07,    -4.0816027480298829e-09, } , 
      {    2.7003400767446283e+01,     4.6905040008738964e+04,     6.0758123152559952e-04,     1.1388218100503353e-06,    -5.3488087477139878e-09, } , 
      {    3.0259211256975465e+01,     6.3771972324019938e+04,     4.7953864161153962e-04,     1.1289219146820786e-06,    -5.3083752867094525e-09, } , 
      {    3.2614765829227366e+01,     7.2100663748745064e+04,     6.3185139225288784e-04,     1.3793330291119650e-06,    -6.4947143643844867e-09, } , 
      {    4.3049852804566839e+00,    -2.7215331392707965e+04,    -1.1985654969361996e-03,    -1.6249839519325094e-06,     7.6869148394641358e-09, } , 
      {    3.4047293631794027e+00,    -3.5753572283263813e+04,    -9.1821015638079138e-04,    -1.3646729421343662e-06,     6.4601423007845671e-09, } , 
      {    6.6605398527085846e+00,    -1.8886639967982839e+04,    -1.0462527462948513e-03,    -1.3745728375026230e-06,     6.5005757617891024e-09, } , 
      {    5.7602839354313016e+00,    -2.7424880858538687e+04,    -7.6589740573944321e-04,    -1.1142618277044798e-06,     5.2738032231095320e-09, } , 
      {    9.0160944249604835e+00,    -1.0557948543257713e+04,    -8.9393999565350317e-04,    -1.1241617230727366e-06,     5.3142366841140673e-09, } , 
      {    8.1158385076832023e+00,    -1.9096189433813561e+04,    -6.1358465509809494e-04,    -8.6385071327459341e-07,     4.0874641454344977e-09, } , 
      {    1.1371648997212384e+01,    -2.2292571185325869e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    1.4627459486741566e+01,     1.4637675196748387e+04,    -8.6966983492621486e-04,    -8.8365050401110686e-07,     4.1683310674435684e-09, } , 
      {    7.2155825904059245e+00,    -2.7634430324369408e+04,    -3.3322931454268665e-04,    -6.0353970347645026e-07,     2.8606916067549282e-09, } , 
      {    1.0471393079935103e+01,    -1.0767498009088435e+04,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {    1.3727203569464285e+01,     6.0994343061925392e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.6983014058993469e+01,     2.2966366621473513e+04,    -7.1735708428486659e-04,    -6.3323938958122043e-07,     2.9819919897685337e-09, } , 
      {    2.0238824548522643e+01,     3.9833298936754487e+04,    -8.4539967419892665e-04,    -6.4313928494947716e-07,     3.0224254507730690e-09, } , 
      {    9.5711371626578234e+00,    -1.9305738899644282e+04,    -1.8091656390133843e-04,    -3.5312858904656384e-07,     1.6743525290798933e-09, } , 
      {    1.2826947652187002e+01,    -2.4388065843633085e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    1.6082758141716184e+01,     1.4428125730917665e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9338568631245366e+01,     3.1295058046198639e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    2.2594379120774544e+01,     4.8161990361479613e+04,    -6.9308692355757838e-04,    -3.9272817051959074e-07,     1.8360863730980344e-09, } , 
      {    1.1926691734909724e+01,    -1.0977047474919156e+04,    -2.8603813259990178e-05,    -1.0271747461667739e-07,     4.8801345140485886e-10, } , 
      {    1.5182502224438903e+01,     5.8898848403618176e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.8438312713968084e+01,     2.2756817155642791e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1694123203497266e+01,     3.9623749470923765e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    2.4949933693026445e+01,     5.6490681786204739e+04,    -5.4077417291623010e-04,    -1.4231705608970429e-07,     6.4974729542299990e-10, } , 
      {    1.4282246307161625e+01,    -2.6483560501940301e+03,     1.2370893738135807e-04,     1.4769363981320903e-07,    -6.9832562627017560e-10, } , 
      {    1.7538056796690803e+01,     1.4218576265086944e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.0793867286219985e+01,     3.1085508580367918e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.4049677775749167e+01,     4.7952440895648891e+04,    -2.6041883236082187e-04,     1.1799395370843885e-07,    -5.7702524325656986e-10, } , 
      {    2.7305488265278345e+01,     6.4819373210929865e+04,    -3.8846142227488188e-04,     1.0809405834018213e-07,    -5.3659178225203455e-10, } , 
      {    1.6637800879413525e+01,     5.6803353745310960e+03,     2.7602168802270629e-04,     3.9810475424309542e-07,    -1.8846647039452100e-09, } , 
      {    1.9893611368942704e+01,     2.2547267689812070e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {    2.3149421858471886e+01,     3.9414200005093044e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.6405232348001068e+01,     5.6281132320374018e+04,    -1.0810608171947362e-04,     3.6840506813832525e-07,    -1.7633643209316043e-09, } , 
      {    2.9661042837530246e+01,     7.3148064635654999e+04,    -2.3614867163353364e-04,     3.5850517277006852e-07,    -1.7229308599270690e-09, } , 
      {    1.8993355451665426e+01,     1.4009026799256222e+04,     4.2833443866405457e-04,     6.4851586867298184e-07,    -3.0710037816202447e-09, } , 
      {    2.2249165941194605e+01,     3.0875959114537196e+04,     3.0029184874999460e-04,     6.3861597330472512e-07,    -3.0305703206157094e-09, } , 
      {    2.5504976430723787e+01,     4.7742891429818170e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    2.8760786920252968e+01,     6.4609823745099144e+04,     4.4206668921874628e-05,     6.1881618256821167e-07,    -2.9497033986066388e-09, } , 
      {    3.2016597409782143e+01,     8.1476756060380110e+04,    -8.3835920992185388e-05,     6.0891628719995495e-07,    -2.9092699376021035e-09, } , 
      {    2.4604720513446502e+01,     3.9204650539262322e+04,     4.5260459939134277e-04,     8.8902708773461154e-07,    -4.2169093982907436e-09, } , 
      {    2.7860531002975684e+01,     5.6071582854543296e+04,     3.2456200947728281e-04,     8.7912719236635482e-07,    -4.1764759372862083e-09, } , 
      {    3.1116341492504866e+01,     7.2938515169824270e+04,     1.9651941956322285e-04,     8.6922729699809809e-07,    -4.1360424762816730e-09, } , 
      {    2.6960275085698406e+01,     4.7533341963987448e+04,     6.0491735003269105e-04,     1.1394382021644980e-06,    -5.4032484759657779e-09, } , 
      {    3.0216085575227588e+01,     6.4400274279268422e+04,     4.7687476011863108e-04,     1.1295383067962412e-06,    -5.3628150149612426e-09, } , 
      {    3.3471896064756770e+01,     8.1267206594549396e+04,     3.4883217020457112e-04,     1.1196384114279845e-06,    -5.3223815539567073e-09, } , 
      {    3.2571640147479485e+01,     7.2728965703993541e+04,     6.2918751075997936e-04,     1.3799494212261277e-06,    -6.5491540926362768e-09, } , 
      {    3.4927194719731389e+01,     8.1057657128718682e+04,     7.8150026140132753e-04,     1.6303605356560141e-06,    -7.7354931703113127e-09, } , 
      {    6.6174141709607062e+00,    -1.8258338012734355e+04,    -1.0489166277877599e-03,    -1.3739564453884603e-06,     6.4461360335373123e-09, } , 
      {    8.9729687432126060e+00,    -9.9296465880092310e+03,    -8.9660387714641165e-04,    -1.1235453309585739e-06,     5.2597969558622772e-09, } , 
      {    1.1328523315464507e+01,    -1.6009551632841046e+03,    -7.4429112650506338e-04,    -8.7313421652868745e-07,     4.0734578781872430e-09, } , 
      {    1.0428267398187225e+01,    -1.0139196053839953e+04,    -4.6393578594965519e-04,    -6.1282320673054430e-07,     2.8466853395076738e-09, } , 
      {    1.3684077887716407e+01,     6.7277362614410213e+03,    -5.9197837586371510e-04,    -6.2272310209880103e-07,     2.8871188005122087e-09, } , 
      {    1.6939888377245591e+01,     2.3594668576721997e+04,    -7.2002096577777506e-04,    -6.3262299746705775e-07,     2.9275522615167440e-09, } , 
      {    1.2783821970439124e+01,    -1.8105046291148262e+03,    -3.1162303530830692e-04,    -3.6241209230065788e-07,     1.6603462618326388e-09, } , 
      {    1.6039632459968306e+01,     1.5056427686166147e+04,    -4.3966562522236693e-04,    -3.7231198766891461e-07,     1.7007797228371739e-09, } , 
      {    1.9295442949497488e+01,     3.1923360001447123e+04,    -5.6770821513642690e-04,    -3.8221188303717133e-07,     1.7412131838417092e-09, } , 
      {    1.5139376542691025e+01,     6.5181867956102997e+03,    -1.5931028466695867e-04,    -1.1200097787077145e-07,     4.7400718415760441e-10, } , 
      {    1.8395187032220207e+01,     2.3385119110891275e+04,    -2.8735287458101871e-04,    -1.2190087323902819e-07,     5.1444064516213961e-10, } , 
      {    2.1650997521749389e+01,     4.0252051426172249e+04,    -4.1539546449507867e-04,    -1.3180076860728491e-07,     5.5487410616667482e-10, } , 
      {    2.4906808011278567e+01,     5.7118983741453223e+04,    -5.4343805440913858e-04,    -1.4170066397554164e-07,     5.9530756717121014e-10, } , 
      {    1.7494931114942926e+01,     1.4846878220335426e+04,    -6.9975340256104300e-06,     1.3841013655911496e-07,    -7.1233189351743005e-10, } , 
      {    2.0750741604472108e+01,     3.1713810535616401e+04,    -1.3504012393967041e-04,     1.2851024119085823e-07,    -6.7189843251289484e-10, } , 
      {    2.4006552094001290e+01,     4.8580742850897375e+04,    -2.6308271385373040e-04,     1.1861034582260152e-07,    -6.3146497150835963e-10, } , 
      {    1.6594675197665648e+01,     6.3086373297795781e+03,     2.7335780652979776e-04,     3.9872114635725810e-07,    -1.9391044321969997e-09, } , 
      {    1.9850485687194826e+01,     2.3175569645060554e+04,     1.4531521661573783e-04,     3.8882125098900138e-07,    -1.8986709711924644e-09, } , 
      {    2.3106296176724008e+01,     4.0042501960341528e+04,     1.7272626701677830e-05,     3.7892135562074465e-07,    -1.8582375101879293e-09, } , 
      {    2.6362106666253190e+01,     5.6909434275622501e+04,    -1.1076996321238214e-04,     3.6902146025248793e-07,    -1.8178040491833942e-09, } , 
      {    2.2206040259446727e+01,     3.1504261069785680e+04,     2.9762796725708607e-04,     6.3923236541888780e-07,    -3.0850100488674991e-09, } , 
      {    2.5461850748975909e+01,     4.8371193385066654e+04,     1.6958537734302609e-04,     6.2933247005063107e-07,    -3.0445765878629637e-09, } , 
      {    2.8717661238505091e+01,     6.5238125700347628e+04,     4.1542787428966117e-05,     6.1943257468237435e-07,    -3.0041431268584284e-09, } , 
      {    2.4561594831698624e+01,     3.9832952494510806e+04,     4.4994071789843424e-04,     8.8964347984877422e-07,    -4.2713491265425337e-09, } , 
      {    2.7817405321227806e+01,     5.6699884809791780e+04,     3.2189812798437428e-04,     8.7974358448051750e-07,    -4.2309156655379984e-09, } , 
      {    3.1073215810756988e+01,     7.3566817125072746e+04,     1.9385553807031434e-04,     8.6984368911226077e-07,    -4.1904822045334631e-09, } , 
      {    3.0172959893479710e+01,     6.5028576234516906e+04,     4.7421087862572255e-04,     1.1301546989104039e-06,    -5.4172547432130327e-09, } , 
      {    1.1285397633716627e+01,    -9.7265320803562236e+02,    -7.4695500799797196e-04,    -8.7251782441452477e-07,     4.0190181499354537e-09, } , 
      {    1.3640952205968528e+01,     7.3560382166895033e+03,    -5.9464225735662369e-04,    -6.2210670998463835e-07,     2.8326790722604187e-09, } , 
      {    1.6896762695497713e+01,     2.4222970531970477e+04,    -7.2268484727068365e-04,    -6.3200660535289507e-07,     2.8731125332649540e-09, } , 
      {    1.2740696288691245e+01,    -1.1822026738663440e+03,    -3.1428691680121540e-04,    -3.6179570018649526e-07,     1.6059065335808491e-09, } , 
      {    1.5996506778220427e+01,     1.5684729641414629e+04,    -4.4232950671527541e-04,    -3.7169559555475198e-07,     1.6463399945853842e-09, } , 
      {    1.9252317267749611e+01,     3.2551661956695603e+04,    -5.7037209662933548e-04,    -3.8159549092300871e-07,     1.6867734555899195e-09, } , 
      {    1.5096250860943146e+01,     7.1464887508587817e+03,    -1.6197416615986720e-04,    -1.1138458575660878e-07,     4.1956745590581459e-10, } , 
      {    1.8352061350472329e+01,     2.4013421066139756e+04,    -2.9001675607392719e-04,    -1.2128448112486551e-07,     4.6000091691034980e-10, } , 
      {    2.1607871840001511e+01,     4.0880353381420733e+04,    -4.1805934598798715e-04,    -1.3118437649312223e-07,     5.0043437791488495e-10, } , 
      {    1.7451805433195048e+01,     1.5475180175583908e+04,    -9.6614155185189416e-06,     1.3902652867327764e-07,    -7.6677162176921992e-10, } , 
      {    2.0707615922724230e+01,     3.2342112490864882e+04,    -1.3770400543257894e-04,     1.2912663330502091e-07,    -7.2633816076468471e-10, } , 
      {    2.3963426412253412e+01,     4.9209044806145859e+04,    -2.6574659534663888e-04,     1.1922673793676419e-07,    -6.8590469976014950e-10, } , 
      {    1.9807360005446949e+01,     2.3803871600309034e+04,     1.4265133512282929e-04,     3.8943764310316401e-07,    -1.9531106994442545e-09, } , 
      {    2.3063170494976131e+01,     4.0670803915590011e+04,     1.4608745208769318e-05,     3.7953774773490728e-07,    -1.9126772384397192e-09, } , 
      {    2.5418725067228031e+01,     4.8999495340315138e+04,     1.6692149585011755e-04,     6.2994886216479375e-07,    -3.0990163161147538e-09, } , 
      {    1.5953381096472549e+01,     1.6313031596663111e+04,    -4.4499338820818394e-04,    -3.7107920344058930e-07,     1.5919002663335943e-09, } , 
      {    1.8308935668724448e+01,     2.4641723021388239e+04,    -2.9268063756683572e-04,    -1.2066808901070285e-07,     4.0556118865855998e-10, } , 
      {    2.0664490240976349e+01,     3.2970414446113362e+04,    -1.4036788692548745e-04,     1.2974302541918357e-07,    -7.8077788901647447e-10, } , 
      {    1.2654444925195488e+01,     7.4401236630620588e+01,    -3.1961467978703246e-04,    -3.6056291595816990e-07,     1.4970270770772694e-09, } , 
      {    2.1410602008514815e+01,     2.0314597010264199e+04,    -4.6210415986242334e-04,    -3.4280409385639139e-07,     1.7641739597790569e-09, } , 
      {    2.6121711153018616e+01,     3.6971979859714455e+04,    -1.5747865857972684e-04,     1.5801813500338148e-07,    -6.0850419557101197e-10, } , 
      {    1.9011921754515036e+01,     1.2614207540787556e+04,    -6.1708079199668004e-04,    -5.9259881617211518e-07,     2.8960733092023021e-09, } , 
      {    2.1367476326766933e+01,     2.0942898965512682e+04,    -4.6476804135533187e-04,    -3.4218770174222871e-07,     1.7097342315272671e-09, } , 
      {    2.0467220409489652e+01,     1.2404658074956835e+04,    -1.8441270079992361e-04,    -8.1876691944085562e-08,     4.8296169284769741e-10, } , 
      {    2.3723030899018834e+01,     2.9271590390237809e+04,    -3.1245529071398365e-04,    -9.1776587312342287e-08,     5.2339515385223272e-10, } , 
      {    2.2822774981741553e+01,     2.0733349499681961e+04,    -3.2099950158575373e-05,     1.6853442248580086e-07,    -7.0337738482733705e-10, } , 
      {    2.6078585471270735e+01,     3.7600281814962931e+04,    -1.6014254007263535e-04,     1.5863452711754413e-07,    -6.6294392382280173e-10, } , 
      {    2.8434140043522635e+01,     4.5928973239688057e+04,    -7.8297894312871123e-06,     4.0904564154743055e-07,    -1.8492830014978363e-09, } , 
      {    1.6613241500515255e+01,     4.9138180713109123e+03,    -7.7205742413093690e-04,    -8.4239353848783892e-07,     4.0279726586255467e-09, } , 
      {    1.8968796072767159e+01,     1.3242509496036038e+04,    -6.1974467348958862e-04,    -5.9198242405795250e-07,     2.8416335809505121e-09, } , 
      {    2.2224606562296341e+01,     3.0109441811317010e+04,    -7.4778726340364858e-04,    -6.0188231942620923e-07,     2.8820670419550474e-09, } , 
      {    1.8068540155489874e+01,     4.7042686054801907e+03,    -3.3938933293418033e-04,    -3.3167141425980936e-07,     1.6148610422709421e-09, } , 
      {    2.1324350645019056e+01,     2.1571200920761163e+04,    -4.6743192284824035e-04,    -3.4157130962806608e-07,     1.6552945032754774e-09, } , 
      {    2.4580161134548238e+01,     3.8438133236042137e+04,    -5.9547451276230031e-04,    -3.5147120499632281e-07,     1.6957279642800127e-09, } , 
      {    2.0424094727741775e+01,     1.3032960030205317e+04,    -1.8707658229283214e-04,    -8.1260299829922895e-08,     4.2852196459590759e-10, } , 
      {    2.3679905217270957e+01,     2.9899892345486289e+04,    -3.1511917220689213e-04,    -9.1160195198179620e-08,     4.6895542560044285e-10, } , 
      {    2.6935715706800138e+01,     4.6766824660767263e+04,    -4.4316176212095209e-04,    -1.0106009056643634e-07,     5.0938888660497816e-10, } , 
      {    2.2779649299993675e+01,     2.1361651454930441e+04,    -3.4763831651483884e-05,     1.6915081459996354e-07,    -7.5781711307912692e-10, } , 
      {    2.6035459789522857e+01,     3.8228583770211415e+04,    -1.6280642156554388e-04,     1.5925091923170681e-07,    -7.1738365207459160e-10, } , 
      {    2.5135203872245576e+01,     2.9690342879655567e+04,     1.1754891898986437e-04,     4.1956192902984991e-07,    -1.9441561907541615e-09, } , 
      {    2.8391014361774758e+01,     4.6557275194936541e+04,    -1.0493670924195625e-05,     4.0966203366159318e-07,    -1.9037227297496262e-09, } , 
      {    1.6570115818767377e+01,     5.5421200265593943e+03,    -7.7472130562384538e-04,    -8.4177714637367624e-07,     3.9735329303737567e-09, } , 
      {    1.8925670391019281e+01,     1.3870811451284520e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    2.2181480880548463e+01,     3.0737743766565494e+04,    -7.5045114489655706e-04,    -6.0126592731204655e-07,     2.8276273137032577e-09, } , 
      {    1.8025414473741996e+01,     5.3325705607286727e+03,    -3.4205321442708886e-04,    -3.3105502214564668e-07,     1.5604213140191522e-09, } , 
      {    2.1281224963271178e+01,     2.2199502876009647e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    2.4537035452800360e+01,     3.9066435191290620e+04,    -5.9813839425520879e-04,    -3.5085481288216013e-07,     1.6412882360282228e-09, } , 
      {    2.0380969045993897e+01,     1.3661261985453799e+04,    -1.8974046378574064e-04,    -8.0643907715760229e-08,     3.7408223634411777e-10, } , 
      {    2.3636779535523079e+01,     3.0528194300734773e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {    2.6892590025052261e+01,     4.7395126616015747e+04,    -4.4582564361386062e-04,    -1.0044369845227368e-07,     4.5494915835318840e-10, } , 
      {    1.9480713128716619e+01,     5.1230210948979511e+03,     9.0614876769667594e-05,     1.7966710208238292e-07,    -8.5269030233545189e-10, } , 
      {    2.2736523618245798e+01,     2.1989953410178925e+04,    -3.7427713144392395e-05,     1.6976720671412619e-07,    -8.1225684133091668e-10, } , 
      {    2.5992334107774980e+01,     3.8856885725459899e+04,    -1.6547030305845238e-04,     1.5986731134586947e-07,    -7.7182338032638137e-10, } , 
      {    2.9248144597304162e+01,     5.5723818040740873e+04,    -2.9351289297251235e-04,     1.4996741597761274e-07,    -7.3138991932184606e-10, } , 
      {    2.5092078190497698e+01,     3.0318644834904051e+04,     1.1488503749695585e-04,     4.2017832114401259e-07,    -1.9985959190059511e-09, } , 
      {    2.8347888680026880e+01,     4.7185577150185025e+04,    -1.3157552417104136e-05,     4.1027842577575586e-07,    -1.9581624580014158e-09, } , 
      {    3.1603699169556062e+01,     6.4052509465465999e+04,    -1.4120014233116410e-04,     4.0037853040749914e-07,    -1.9177289969968805e-09, } , 
      {    2.7447632762749599e+01,     3.8647336259629177e+04,     2.6719778813830413e-04,     6.7058943557389901e-07,    -3.1849349966809858e-09, } , 
      {    3.0703443252278781e+01,     5.5514268574910151e+04,     1.3915519822424411e-04,     6.6068954020564228e-07,    -3.1445015356764505e-09, } , 
      {    1.6526990137019499e+01,     6.1704219818078764e+03,    -7.7738518711675385e-04,    -8.4116075425951356e-07,     3.9190932021219666e-09, } , 
      {    1.8882544709271404e+01,     1.4499113406533002e+04,    -6.2507243647540558e-04,    -5.9074963982962714e-07,     2.7327541244469327e-09, } , 
      {    1.7982288791994119e+01,     5.9608725159771548e+03,    -3.4471709591999739e-04,    -3.3043863003148400e-07,     1.5059815857673624e-09, } , 
      {    2.1238099281523301e+01,     2.2827804831258130e+04,    -4.7275968583405741e-04,    -3.4033852539974072e-07,     1.5464150467718977e-09, } , 
      {    2.4493909771052483e+01,     3.9694737146539104e+04,    -6.0080227574811726e-04,    -3.5023842076799745e-07,     1.5868485077764330e-09, } , 
      {    2.0337843364246019e+01,     1.4289563940702281e+04,    -1.9240434527864915e-04,    -8.0027515601597562e-08,     3.1964250809232796e-10, } , 
      {    2.3593653853775201e+01,     3.1156496255983257e+04,    -3.2044693519270919e-04,    -8.9927410969854286e-08,     3.6007596909686327e-10, } , 
      {    2.6849464343304383e+01,     4.8023428571264230e+04,    -4.4848952510676915e-04,    -9.9827306338111011e-08,     4.0050943010139858e-10, } , 
      {    2.2693397936497920e+01,     2.2618255365427409e+04,    -4.0091594637300906e-05,     1.7038359882828885e-07,    -8.6669656958270645e-10, } , 
      {    2.5949208426027102e+01,     3.9485187680708383e+04,    -1.6813418455136089e-04,     1.6048370346003212e-07,    -8.2626310857817113e-10, } , 
      {    2.9205018915556284e+01,     5.6352119995989357e+04,    -2.9617677446542088e-04,     1.5058380809177540e-07,    -7.8582964757363582e-10, } , 
      {    2.8304762998279003e+01,     4.7813879105433509e+04,    -1.5821433910012646e-05,     4.1089481788991854e-07,    -2.0126021862532055e-09, } , 
      {    3.0660317570530903e+01,     5.6142570530158635e+04,     1.3649131673133561e-04,     6.6130593231980496e-07,    -3.1989412639282401e-09, } , 
      {    1.8839419027523526e+01,     1.5127415361781485e+04,    -6.2773631796831416e-04,    -5.9013324771546446e-07,     2.6783143961951427e-09, } , 
      {    2.1194973599775423e+01,     2.3456106786506611e+04,    -4.7542356732696589e-04,    -3.3972213328557809e-07,     1.4919753185201080e-09, } , 
      {    2.3550528172027324e+01,     3.1784798211231737e+04,    -3.2311081668561767e-04,    -8.9311018855691620e-08,     3.0563624084507350e-10, } , 
      {    2.5906082744279225e+01,     4.0113489635956867e+04,    -1.7079806604426942e-04,     1.6110009557419480e-07,    -8.8070283682996100e-10, } , 
      {    2.8261637316531125e+01,     4.8442181060681993e+04,    -1.8485315402921157e-05,     4.1151121000408117e-07,    -2.0670419145049956e-09, } , 
      {    1.7896037428498360e+01,     7.2174764264741198e+03,    -3.5004485890581440e-04,    -3.2920584580315869e-07,     1.3971021292637828e-09, } , 
      {    2.4296639939565789e+01,     1.9128980775382566e+04,    -6.4484708962255356e-04,    -5.6185813813126655e-07,     2.8505880896406051e-09, } , 
      {    2.6652194511817690e+01,     2.7457672200107692e+04,    -4.9253433898120539e-04,    -3.1144702370138013e-07,     1.6642490119655706e-09, } , 
      {    2.9007749084069591e+01,     3.5786363624832818e+04,    -3.4022158833985706e-04,    -6.1035909271493692e-08,     4.7790993429053606e-10, } , 
      {    3.1363303656321492e+01,     4.4115055049557945e+04,    -1.8790883769850882e-04,     1.8937520515839272e-07,    -7.0842914338449840e-10, } , 
      {    2.4253514257817912e+01,     1.9757282730631046e+04,    -6.4751097111546204e-04,    -5.6124174601710397e-07,     2.7961483613888154e-09, } , 
      {    2.6609068830069813e+01,     2.8085974155356173e+04,    -4.9519822047411387e-04,    -3.1083063158721745e-07,     1.6098092837137807e-09, } , 
      {    2.8964623402321713e+01,     3.6414665580081302e+04,    -3.4288546983276560e-04,    -6.0419517157331026e-08,     4.2347020603874629e-10, } , 
      {    3.1320177974573614e+01,     4.4743357004806428e+04,    -1.9057271919141732e-04,     1.8999159727255540e-07,    -7.6286887163628816e-10, } , 
      {    3.3675732546825515e+01,     5.3072048429531555e+04,    -3.8259968550069073e-05,     4.4040271170244182e-07,    -1.9492079493113224e-09, } , 
      {    2.1854834003818134e+01,     1.2056893261154404e+04,    -8.0248760324971879e-04,    -8.1103646833282771e-07,     3.9280477108120596e-09, } , 
      {    2.4210388576070031e+01,     2.0385584685879530e+04,    -6.5017485260837051e-04,    -5.6062535390294129e-07,     2.7417086331370257e-09, } , 
      {    2.7466199065599213e+01,     3.7252517001160501e+04,    -7.7821744252243048e-04,    -5.7052524927119802e-07,     2.7821420941415611e-09, } , 
      {    2.3310132658792753e+01,     1.1847343795323683e+04,    -3.6981951205296233e-04,    -3.0031434410479804e-07,     1.5149360944574556e-09, } , 
      {    2.6565943148321931e+01,     2.8714276110604656e+04,    -4.9786210196702235e-04,    -3.1021423947305477e-07,     1.5553695554619909e-09, } , 
      {    2.9821753637851110e+01,     4.5581208425885627e+04,    -6.2590469188108231e-04,    -3.2011413484131149e-07,     1.5958030164665262e-09, } , 
      {    2.5665687231044654e+01,     2.0176035220048809e+04,    -2.1750676141161408e-04,    -4.9903229674911641e-08,     3.2859701678242116e-10, } , 
      {    2.8921497720573832e+01,     3.7042967535329779e+04,    -3.4554935132567413e-04,    -5.9803125043168372e-08,     3.6903047778695648e-10, } , 
      {    3.2177308210103014e+01,     5.3909899850610753e+04,    -4.7359194123973409e-04,    -6.9703020411425097e-08,     4.0946393879149179e-10, } , 
      {    2.8021241803296554e+01,     2.8504726644773935e+04,    -6.5194010770265850e-05,     2.0050788475497477e-07,    -8.5774206089261324e-10, } , 
      {    3.1277052292825733e+01,     4.5371658960054905e+04,    -1.9323660068432583e-04,     1.9060798938671805e-07,    -8.1730859988807793e-10, } , 
      {    3.4532862782354911e+01,     6.2238591275335879e+04,    -3.2127919059838581e-04,     1.8070809401846132e-07,    -7.7687513888354261e-10, } , 
      {    3.0376796375548455e+01,     3.6833418069499057e+04,     8.7118739871082398e-05,     4.5091899918486122e-07,    -2.0440811385676474e-09, } , 
      {    3.3632606865077634e+01,     5.3700350384780031e+04,    -4.0923850042977591e-05,     4.4101910381660450e-07,    -2.0036476775631121e-09, } , 
      {    3.5988161437329531e+01,     6.2029041809505157e+04,     1.1138890059837066e-04,     6.9143021824649081e-07,    -3.1899867552381467e-09, } , 
      {    3.8343716009581435e+01,     7.0357733234230283e+04,     2.6370165123971886e-04,     9.4184133267637723e-07,    -4.3763258329131818e-09, } , 
      {    1.9456153749818352e+01,     4.3565037916777601e+03,    -9.5746423538397565e-04,    -1.0608311906485515e-06,     5.0599470602353046e-09, } , 
      {    2.1811708322070256e+01,     1.2685195216402886e+04,    -8.0515148474262738e-04,    -8.1042007621866503e-07,     3.8736079825602703e-09, } , 
      {    2.5067518811599431e+01,     2.9552127531683858e+04,    -9.3319407465668734e-04,    -8.2031997158692176e-07,     3.9140414435648056e-09, } , 
      {    2.0911452404792978e+01,     4.1469543258470385e+03,    -5.2479614418721914e-04,    -5.5010906642052189e-07,     2.6468354438807004e-09, } , 
      {    2.4167262894322153e+01,     2.1013886641128011e+04,    -6.5283873410127910e-04,    -5.6000896178877861e-07,     2.6872689048852357e-09, } , 
      {    2.7423073383851335e+01,     3.7880818956408984e+04,    -7.8088132401533906e-04,    -5.6990885715703534e-07,     2.7277023658897710e-09, } , 
      {    2.3267006977044876e+01,     1.2475645750572165e+04,    -3.7248339354587081e-04,    -2.9969795199063541e-07,     1.4604963662056659e-09, } , 
      {    2.6522817466574054e+01,     2.9342578065853137e+04,    -5.0052598345993093e-04,    -3.0959784735889214e-07,     1.5009298272102012e-09, } , 
      {    2.9778627956103232e+01,     4.6209510381134111e+04,    -6.2856857337399089e-04,    -3.1949774272714886e-07,     1.5413632882147365e-09, } , 
      {    2.5622561549296776e+01,     2.0804337175297289e+04,    -2.2017064290452262e-04,    -4.9286837560748981e-08,     2.7415728853063140e-10, } , 
      {    2.8878372038825955e+01,     3.7671269490578263e+04,    -3.4821323281858260e-04,    -5.9186732929005705e-08,     3.1459074953516671e-10, } , 
      {    3.2134182528355140e+01,     5.4538201805859237e+04,    -4.7625582273264257e-04,    -6.9086628297262430e-08,     3.5502421053970202e-10, } , 
      {    2.7978116121548677e+01,     2.9133028600022415e+04,    -6.7857892263174354e-05,     2.0112427686913745e-07,    -9.1218178914440311e-10, } , 
      {    3.1233926611077855e+01,     4.5999960915303389e+04,    -1.9590048217723436e-04,     1.9122438150088073e-07,    -8.7174832813986779e-10, } , 
      {    3.4489737100607037e+01,     6.2866893230584363e+04,    -3.2394307209129429e-04,     1.8132448613262400e-07,    -8.3131486713533248e-10, } , 
      {    3.0333670693800578e+01,     3.7461720024747541e+04,     8.4454858378173894e-05,     4.5153539129902385e-07,    -2.0985208668194375e-09, } , 
      {    3.3589481183329760e+01,     5.4328652340028515e+04,    -4.3587731535886102e-05,     4.4163549593076712e-07,    -2.0580874058149021e-09, } , 
      {    3.6845291672858934e+01,     7.1195584655309489e+04,    -1.7163032144994607e-04,     4.3173560056251040e-07,    -2.0176539448103668e-09, } , 
      {    3.2689225266052482e+01,     4.5790411449472667e+04,     2.3676760901952213e-04,     7.0194650572891022e-07,    -3.2848599444944721e-09, } , 
      {    3.5945035755581657e+01,     6.2657343764753641e+04,     1.0872501910546215e-04,     6.9204661036065349e-07,    -3.2444264834899368e-09, } , 
      {    3.9200846245110839e+01,     7.9524276080034630e+04,    -1.9317570808597815e-05,     6.8214671499239677e-07,    -3.2039930224854015e-09, } , 
      {    3.8300590327833561e+01,     7.0986035189478775e+04,     2.6103776974681039e-04,     9.4245772479053991e-07,    -4.4307655611649710e-09, } , 
      {    1.9413028068070474e+01,     4.9848057469262421e+03,    -9.6012811687688413e-04,    -1.0602147985343888e-06,     5.0055073319835145e-09, } , 
      {    2.1768582640322379e+01,     1.3313497171651368e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {    2.5024393129851553e+01,     3.0180429486932342e+04,    -9.3585795614959581e-04,    -8.1970357947275908e-07,     3.8596017153130156e-09, } , 
      {    2.0868326723045101e+01,     4.7752562810955205e+03,    -5.2746002568012762e-04,    -5.4949267430635921e-07,     2.5923957156289107e-09, } , 
      {    2.4124137212574276e+01,     2.1642188596376494e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    2.7379947702103458e+01,     3.8509120911657468e+04,    -7.8354520550824754e-04,    -5.6929246504287266e-07,     2.6732626376379813e-09, } , 
      {    3.0635758191632640e+01,     5.5376053226938442e+04,    -9.1158779542230761e-04,    -5.7919236041112938e-07,     2.7136960986425166e-09, } , 
      {    2.3223881295296998e+01,     1.3103947705820647e+04,    -3.7514727503877934e-04,    -2.9908155987647273e-07,     1.4060566379538760e-09, } , 
      {    2.6479691784826176e+01,     2.9970880021101621e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.9735502274355355e+01,     4.6837812336382594e+04,    -6.3123245486689937e-04,    -3.1888135061298618e-07,     1.4869235599629467e-09, } , 
      {    3.2991312763884537e+01,     6.3704744651663568e+04,    -7.5927504478095933e-04,    -3.2878124598124291e-07,     1.5273570209674820e-09, } , 
      {    2.5579435867548899e+01,     2.1432639130545773e+04,    -2.2283452439743112e-04,    -4.8670445446586314e-08,     2.1971756027884158e-10, } , 
      {    2.8835246357078077e+01,     3.8299571445826747e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.2091056846607259e+01,     5.5166503761107720e+04,    -4.7891970422555110e-04,    -6.8470236183099763e-08,     3.0058448228791221e-10, } , 
      {    3.5346867336136441e+01,     7.2033436076388694e+04,    -6.0696229413961106e-04,    -7.8370131551356488e-08,     3.4101794329244742e-10, } , 
      {    2.4679179950271617e+01,     1.2894398239989925e+04,     5.7520816157977117e-05,     2.1164056435155683e-07,    -1.0070549784007282e-09, } , 
      {    2.7934990439800799e+01,     2.9761330555270899e+04,    -7.0521773756082872e-05,     2.0174066898330011e-07,    -9.6662151739619287e-10, } , 
      {    3.1190800929329978e+01,     4.6628262870551873e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    3.4446611418859156e+01,     6.3495195185832847e+04,    -3.2660695358420282e-04,     1.8194087824678666e-07,    -8.8575459538712225e-10, } , 
      {    2.7034734522523518e+01,     2.1223089664715051e+04,     2.0983356679932536e-04,     4.6205167878144325e-07,    -2.1933940560757624e-09, } , 
      {    3.0290545012052700e+01,     3.8090021979996025e+04,     8.1790976885265376e-05,     4.5215178341318653e-07,    -2.1529605950712271e-09, } , 
      {    3.3546355501581878e+01,     5.4956954295276999e+04,    -4.6251613028794612e-05,     4.4225188804492980e-07,    -2.1125271340666918e-09, } , 
      {    3.6802165991111053e+01,     7.1823886610557965e+04,    -1.7429420294285457e-04,     4.3235199267667308e-07,    -2.0720936730621565e-09, } , 
      {    3.2646099584304601e+01,     4.6418713404721151e+04,     2.3410372752661362e-04,     7.0256289784307290e-07,    -3.3392996727462618e-09, } , 
      {    3.5901910073833776e+01,     6.3285645720002125e+04,     1.0606113761255364e-04,     6.9266300247481617e-07,    -3.2988662117417265e-09, } , 
      {    3.9157720563362957e+01,     8.0152578035283106e+04,    -2.1981452301506326e-05,     6.8276310710655945e-07,    -3.2584327507371912e-09, } , 
      {    3.5001654156556498e+01,     5.4747404829446277e+04,     3.8641647816796182e-04,     9.5297401227295932e-07,    -4.5256387504212964e-09, } , 
      {    3.8257464646085680e+01,     7.1614337144727251e+04,     2.5837388825390186e-04,     9.4307411690470259e-07,    -4.4852052894167611e-09, } , 
      {    4.0613019218337577e+01,     7.9943028569452377e+04,     4.1068663889525013e-04,     1.1934852313345890e-06,    -5.6715443670917954e-09, } , 
      {    1.9369902386322597e+01,     5.6131077021747242e+03,    -9.6279199836979260e-04,    -1.0595984064202261e-06,     4.9510676037317244e-09, } , 
      {    2.1725456958574501e+01,     1.3941799126899850e+04,    -8.1047924772844433e-04,    -8.0918729199033967e-07,     3.7647285260566902e-09, } , 
      {    2.4081011530826398e+01,     2.2270490551624978e+04,    -6.5816649708709605e-04,    -5.5877617756045325e-07,     2.5783894483816563e-09, } , 
      {    2.7336822020355580e+01,     3.9137422866905952e+04,    -7.8620908700115602e-04,    -5.6867607292870998e-07,     2.6188229093861917e-09, } , 
      {    2.3180755613549120e+01,     1.3732249661069129e+04,    -3.7781115653168787e-04,    -2.9846516776231005e-07,     1.3516169097020862e-09, } , 
      {    2.6436566103078299e+01,     3.0599181976350104e+04,    -5.0585374644574789e-04,    -3.0836506313056678e-07,     1.3920503707066215e-09, } , 
      {    2.9692376592607477e+01,     4.7466114291631078e+04,    -6.3389633635980785e-04,    -3.1826495849882350e-07,     1.4324838317111568e-09, } , 
      {    2.5536310185801021e+01,     2.2060941085794257e+04,    -2.2549840589033962e-04,    -4.8054053332423647e-08,     1.6527783202705179e-10, } , 
      {    2.8792120675330199e+01,     3.8927873401075231e+04,    -3.5354099580439967e-04,    -5.7953948700680372e-08,     2.0571129303158710e-10, } , 
      {    3.2047931164859378e+01,     5.5794805716356204e+04,    -4.8158358571845963e-04,    -6.7853844068937096e-08,     2.4614475403612239e-10, } , 
      {    2.7891864758052922e+01,     3.0389632510519383e+04,    -7.3185655248991389e-05,     2.0235706109746276e-07,    -1.0210612456479827e-09, } , 
      {    3.1147675247582100e+01,     4.7256564825800357e+04,    -2.0122824516305136e-04,     1.9245716572920604e-07,    -9.8062778464344743e-10, } , 
      {    3.4403485737111275e+01,     6.4123497141081330e+04,    -3.2927083507711135e-04,     1.8255727036094931e-07,    -9.4019432363891211e-10, } , 
      {    3.0247419330304822e+01,     3.8718323935244509e+04,     7.9127095392356859e-05,     4.5276817552734921e-07,    -2.2074003233230168e-09, } , 
      {    3.3503229819833997e+01,     5.5585256250525483e+04,    -4.8915494521703123e-05,     4.4286828015909248e-07,    -2.1669668623184815e-09, } , 
      {    3.6759040309363172e+01,     7.2452188565806442e+04,    -1.7695808443576308e-04,     4.3296838479083576e-07,    -2.1265334013139462e-09, } , 
      {    3.5858784392085894e+01,     6.3913947675250609e+04,     1.0339725611964512e-04,     6.9327939458897885e-07,    -3.3533059399935161e-09, } , 
      {    3.8214338964337799e+01,     7.2242639099975728e+04,     2.5571000676099333e-04,     9.4369050901886527e-07,    -4.5396450176685512e-09, } , 
      {    2.1682331276826623e+01,     1.4570101082148332e+04,    -8.1314312922135291e-04,    -8.0857089987617699e-07,     3.7102887978049005e-09, } , 
      {    2.4037885849078521e+01,     2.2898792506873458e+04,    -6.6083037858000464e-04,    -5.5815978544629057e-07,     2.5239497201298663e-09, } , 
      {    2.3137629931801243e+01,     1.4360551616317611e+04,    -3.8047503802459635e-04,    -2.9784877564814743e-07,     1.2971771814502965e-09, } , 
      {    2.6393440421330421e+01,     3.1227483931598585e+04,    -5.0851762793865647e-04,    -3.0774867101640415e-07,     1.3376106424548318e-09, } , 
      {    2.5493184504053144e+01,     2.2689243041042737e+04,    -2.2816228738324815e-04,    -4.7437661218260987e-08,     1.1083810377526199e-10, } , 
      {    2.8748994993582322e+01,     3.9556175356323714e+04,    -3.5620487729730814e-04,    -5.7337556586517712e-08,     1.5127156477979731e-10, } , 
      {    3.1104549565834223e+01,     4.7884866781048840e+04,    -2.0389212665595990e-04,     1.9307355784336872e-07,    -1.0350675128952371e-09, } , 
      {    2.6350314739582540e+01,     3.1855785886847068e+04,    -5.1118150943156495e-04,    -3.0713227890224147e-07,     1.2831709142030419e-09, } , 
      {    2.9408855397625032e+01,     2.8156961830971512e+04,    -6.8326891322006099e-04,    -5.2865189163376740e-07,     2.5873439570717493e-09, } , 
      {    3.1764409969876930e+01,     3.6485653255696634e+04,    -5.3095616257871272e-04,    -2.7824077720388088e-07,     1.4010048793967145e-09, } , 
      {    2.7010175143625251e+01,     2.0456572361494866e+04,    -8.3824554535431785e-04,    -7.7844661394949115e-07,     3.7192433064949935e-09, } , 
      {    2.9365729715877155e+01,     2.8785263786219992e+04,    -6.8593279471296958e-04,    -5.2803549951960472e-07,     2.5329042288199593e-09, } , 
      {    3.1721284288129052e+01,     3.7113955210945118e+04,    -5.3362004407162130e-04,    -2.7762438508971825e-07,     1.3465651511449248e-09, } , 
      {    3.6432393432632857e+01,     5.3771338060395370e+04,    -2.2899454278892483e-04,     2.2319784377005462e-07,    -1.0261130042051441e-09, } , 
      {    2.4611494889625476e+01,     1.2756182892018223e+04,    -9.9322217748857460e-04,    -1.0282413362652149e-06,     4.8511426559182381e-09, } , 
      {    2.6967049461877373e+01,     2.1084874316743349e+04,    -8.4090942684722633e-04,    -7.7783022183532847e-07,     3.6648035782432039e-09, } , 
      {    2.9322604034129277e+01,     2.9413565741468476e+04,    -6.8859667620587805e-04,    -5.2741910740544204e-07,     2.4784645005681696e-09, } , 
      {    3.2578414523658459e+01,     4.6280498056749449e+04,    -8.1663926611993802e-04,    -5.3731900277369877e-07,     2.5188979615727049e-09, } , 
      {    2.8422348116851992e+01,     2.0875324850912628e+04,    -4.0824133565046982e-04,    -2.6710809760729885e-07,     1.2516919618885996e-09, } , 
      {    3.1678158606381174e+01,     3.7742257166193602e+04,    -5.3628392556452978e-04,    -2.7700799297555557e-07,     1.2921254228931349e-09, } , 
      {    3.4933969095910356e+01,     5.4609189481474576e+04,    -6.6432651547858974e-04,    -2.8690788834381230e-07,     1.3325588838976703e-09, } , 
      {    3.0777902689103893e+01,     2.9204016275637754e+04,    -2.5592858500912160e-04,    -1.6696983177412426e-08,     6.5352884213565184e-11, } , 
      {    3.4033713178633079e+01,     4.6070948590918728e+04,    -3.8397117492318161e-04,    -2.6596878545669150e-08,     1.0578634521810050e-10, } , 
      {    3.3133457261355794e+01,     3.7532707700362880e+04,    -1.0361583436777335e-04,     2.3371413125247400e-07,    -1.1209861934614693e-09, } , 
      {    3.6389267750884976e+01,     5.4399640015643854e+04,    -2.3165842428183334e-04,     2.2381423588421727e-07,    -1.0805527324569340e-09, } , 
      {    2.9279478352381400e+01,     3.0041867696716959e+04,    -6.9126055769878653e-04,    -5.2680271529127936e-07,     2.4240247723163799e-09, } , 
      {    3.1635032924633297e+01,     3.8370559121442086e+04,    -5.3894780705743826e-04,    -2.7639160086139289e-07,     1.2376856946413451e-09, } , 
      {    3.3990587496885198e+01,     4.6699250546167212e+04,    -3.8663505641609014e-04,    -2.5980486431506487e-08,     5.1346616966310699e-11, } , 
      {    3.6346142069137095e+01,     5.5027941970892338e+04,    -2.3432230577474184e-04,     2.2443062799837992e-07,    -1.1349924607087238e-09, } , 
      {    3.4650447900927908e+01,     3.5300037020815013e+04,    -7.1369909233884299e-04,    -4.9729482147875620e-07,     2.4874190092582626e-09, } , 
      {    3.7006002473179805e+01,     4.3628728445540139e+04,    -5.6138634169749482e-04,    -2.4688370704886967e-07,     1.3010799315832282e-09, } , 
      {    3.9361557045431709e+01,     5.1957419870265265e+04,    -4.0907359105614655e-04,     3.5274073810167458e-09,     1.1474085390819370e-10, } , 
      {    3.2251767646928123e+01,     2.7599647551338367e+04,    -8.6867572447309974e-04,    -7.4708954379447994e-07,     3.6193183586815072e-09, } , 
      {    3.4607322219180027e+01,     3.5928338976063489e+04,    -7.1636297383175147e-04,    -4.9667842936459352e-07,     2.4329792810064729e-09, } , 
      {    3.6962876791431924e+01,     4.4257030400788615e+04,    -5.6405022319040330e-04,    -2.4626731493470699e-07,     1.2466402033314383e-09, } , 
      {    3.9318431363683828e+01,     5.2585721825513741e+04,    -4.1173747254905508e-04,     4.1437994951794093e-09,     6.0301125656403900e-11, } , 
      {    4.1673985935935725e+01,     6.0914413250238867e+04,    -2.5942472190770681e-04,     2.5455491392506585e-07,    -1.1260379520186306e-09, } , 
      {    4.4029540508187623e+01,     6.9243104674963994e+04,    -1.0711197126635854e-04,     5.0496602835495217e-07,    -2.3123770296936649e-09, } , 
      {    2.9853087392928348e+01,     1.9899258081861721e+04,    -1.0236523566073564e-03,    -9.9688426611020368e-07,     4.7512177081047518e-09, } , 
      {    3.2208641965180249e+01,     2.8227949506586847e+04,    -8.7133960596600833e-04,    -7.4647315168031726e-07,     3.5648786304297171e-09, } , 
      {    3.4564196537432153e+01,     3.6556640931311973e+04,    -7.1902685532466005e-04,    -4.9606203725043084e-07,     2.3785395527546829e-09, } , 
      {    3.7820007026961335e+01,     5.3423573246592940e+04,    -8.4706944523872001e-04,    -5.0596193261868756e-07,     2.4189730137592182e-09, } , 
      {    3.6919751109684050e+01,     4.4885332356037099e+04,    -5.6671410468331189e-04,    -2.4565092282054436e-07,     1.1922004750796486e-09, } , 
      {    4.0175561599213232e+01,     6.1752264671318066e+04,    -6.9475669459737185e-04,    -2.5555081818880109e-07,     1.2326339360841839e-09, } , 
      {    3.9275305681935954e+01,     5.3214023780762225e+04,    -4.1440135404196356e-04,     4.7601916093420736e-09,     5.8613974046141025e-12, } , 
      {    4.2531116171465136e+01,     7.0080956096043199e+04,    -5.4244394395602357e-04,    -5.1397037589146511e-09,     4.6294858409149415e-11, } , 
      {    4.1630860254187851e+01,     6.1542715205487351e+04,    -2.6208860340061528e-04,     2.5517130603922848e-07,    -1.1804776802704203e-09, } , 
      {    4.3986414826439749e+01,     6.9871406630212485e+04,    -1.0977585275926705e-04,     5.0558242046911485e-07,    -2.3668167579454549e-09, } , 
      {    4.6341969398691653e+01,     7.8200098054937611e+04,     4.2536897882081193e-05,     7.5599353489900127e-07,    -3.5531558356204892e-09, } , 
      {    2.9809961711180470e+01,     2.0527560037110205e+04,    -1.0263162381002650e-03,    -9.9626787399604100e-07,     4.6967779798529617e-09, } , 
      {    3.2165516283432368e+01,     2.8856251461835331e+04,    -8.7400348745891681e-04,    -7.4585675956615458e-07,     3.5104389021779275e-09, } , 
      {    3.5421326772961550e+01,     4.5723183777116297e+04,    -1.0020460773729769e-03,    -7.5575665493441130e-07,     3.5508723631824628e-09, } , 
      {    3.1265260366155090e+01,     2.0318010571279483e+04,    -5.9364814690350857e-04,    -4.8554574976801143e-07,     2.2836663634983579e-09, } , 
      {    3.4521070855684272e+01,     3.7184942886560457e+04,    -7.2169073681756853e-04,    -4.9544564513626816e-07,     2.3240998245028932e-09, } , 
      {    3.7776881345213454e+01,     5.4051875201841423e+04,    -8.4973332673162849e-04,    -5.0534554050452488e-07,     2.3645332855074285e-09, } , 
      {    3.3620814938406987e+01,     2.8646701996004609e+04,    -4.4133539626216029e-04,    -2.3513463533812496e-07,     1.0973272858233234e-09, } , 
      {    3.6876625427936169e+01,     4.5513634311285583e+04,    -5.6937798617622036e-04,    -2.4503453070638168e-07,     1.1377607468278588e-09, } , 
      {    4.0132435917465351e+01,     6.2380566626566550e+04,    -6.9742057609028033e-04,    -2.5493442607463841e-07,     1.1781942078323941e-09, } , 
      {    3.5976369510658891e+01,     3.6975393420729735e+04,    -2.8902264562081207e-04,     1.5276479091761463e-08,    -8.9011791851711007e-11, } , 
      {    3.9232180000188073e+01,     5.3842325736010709e+04,    -4.1706523553487209e-04,     5.3765837235047379e-09,    -4.8578330847175695e-11, } , 
      {    4.2487990489717255e+01,     7.0709258051291676e+04,    -5.4510782544893205e-04,    -4.5233116447519868e-09,    -8.1448698426403824e-12, } , 
      {    3.8331924082910788e+01,     4.5304084845454861e+04,    -1.3670989497946383e-04,     2.6568759352164788e-07,    -1.2753508695267455e-09, } , 
      {    4.1587734572439970e+01,     6.2171017160735835e+04,    -2.6475248489352381e-04,     2.5578769815339116e-07,    -1.2349174085222101e-09, } , 
      {    4.4843545061969152e+01,     7.9037949476016802e+04,    -3.9279507480758378e-04,     2.4588780278513443e-07,    -1.1944839475176748e-09, } , 
      {    4.0687478655162685e+01,     5.3632776270179987e+04,     1.5602855661884423e-05,     5.1609870795153425e-07,    -2.4616899472017799e-09, } , 
      {    4.3943289144691867e+01,     7.0499708585460961e+04,    -1.1243973425217557e-04,     5.0619881258327753e-07,    -2.4212564861972446e-09, } , 
      {    4.6298843716943772e+01,     7.8828400010186087e+04,     3.9873016389172682e-05,     7.5660992701316395e-07,    -3.6075955638722788e-09, } , 
      {    3.2122390601684486e+01,     2.9484553417083815e+04,    -8.7666736895182528e-04,    -7.4524036745199190e-07,     3.4559991739261378e-09, } , 
      {    3.4477945173936391e+01,     3.7813244841808941e+04,    -7.2435461831047701e-04,    -4.9482925302210548e-07,     2.2696600962511035e-09, } , 
      {    3.6833499746188288e+01,     4.6141936266534067e+04,    -5.7204186766912884e-04,    -2.4441813859221900e-07,     1.0833210185760689e-09, } , 
      {    3.9189054318440192e+01,     5.4470627691259193e+04,    -4.1972911702778062e-04,     5.9929758376674022e-09,    -1.0301805909896549e-10, } , 
      {    4.1544608890692089e+01,     6.2799319115984319e+04,    -2.6741636638643234e-04,     2.5640409026755384e-07,    -1.2893571367740000e-09, } , 
      {    4.3900163462943986e+01,     7.1128010540709438e+04,    -1.1510361574508408e-04,     5.0681520469744021e-07,    -2.4756962144490343e-09, } , 
      {    3.4434819492188517e+01,     3.8441546797057425e+04,    -7.2701849980338559e-04,    -4.9421286090794280e-07,     2.2152203679993135e-09, } , 
      {    3.7363983104987362e+01,     3.6627628606927305e+04,    -9.0709754807060728e-04,    -7.1388329729698069e-07,     3.3560742261126515e-09, } , 
      {    3.9719537677239266e+01,     4.4956320031652431e+04,    -7.5478479742925901e-04,    -4.6347218286709422e-07,     2.1697351484376172e-09, } , 
      {    4.2605575608290238e+01,     4.3770703796770809e+04,    -9.3752772718938928e-04,    -6.8252622714196948e-07,     3.2561492782991647e-09, } , 
      {    4.4961130180542142e+01,     5.2099395221495935e+04,    -7.8521497654804101e-04,    -4.3211511271208301e-07,     2.0698102006241305e-09, } , 
      {    4.7316684752794039e+01,     6.0428086646221062e+04,    -6.3290222590669284e-04,    -1.8170399828219656e-07,     8.8347112294909593e-10, } , 
      {    4.9672239325045943e+01,     6.8756778070946195e+04,    -4.8058947526534451e-04,     6.8707116147689848e-08,    -3.0286795472593847e-10, } , 
      {    4.0206895354290459e+01,     3.6070314327294167e+04,    -1.0925043593236461e-03,    -9.3232094945769322e-07,     4.3880486277224089e-09, } , 
      {    4.2562449926542357e+01,     4.4399005752019293e+04,    -9.4019160868229776e-04,    -6.8190983502780680e-07,     3.2017095500473751e-09, } , 
      {    4.4918004498794261e+01,     5.2727697176744419e+04,    -7.8787885804094948e-04,    -4.3149872059792033e-07,     2.0153704723723408e-09, } , 
      {    4.7273559071046158e+01,     6.1056388601469545e+04,    -6.3556610739960132e-04,    -1.8108760616803391e-07,     8.2903139469730617e-10, } , 
      {    4.9629113643298062e+01,     6.9385080026194672e+04,    -4.8325335675825304e-04,     6.9323508261852514e-08,    -3.5730768297772828e-10, } , 
      {    5.1984668215549959e+01,     7.7713771450919798e+04,    -3.3094060611690477e-04,     3.1973462269173894e-07,    -1.5436467606527627e-09, } , 
      {    1.6279052447645901e+00,     8.4334661576404869e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.8837157342937703e+00,     2.5300398472921461e+04,    -1.9206388487109000e-04,    -1.4849843052385087e-08,     6.0650191506802891e-11, } , 
      {    8.1395262238229513e+00,     4.2167330788202438e+04,    -3.2010647478514996e-04,    -2.4749738420641812e-08,     1.0108365251133815e-10, } , 
      {   -5.7839716515710506e+00,    -3.3838639363477312e+04,     4.7241922542649818e-04,     2.7516085285052823e-07,    -1.2874227301863725e-09, } , 
      {   -2.5281611620418700e+00,    -1.6971707048196335e+04,     3.4437663551243822e-04,     2.6526095748227151e-07,    -1.2469892691818374e-09, } , 
      {    7.2764932748731015e-01,    -1.0477473291536080e+02,     2.1633404559837826e-04,     2.5536106211401478e-07,    -1.2065558081773021e-09, } , 
      {    3.9834598170164903e+00,     1.6762157582365613e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.2392703065456701e+00,     3.3629089897646583e+04,    -3.9751134229741748e-05,     2.3556127137750133e-07,    -1.1256888861682315e-09, } , 
      {    1.0495080796074852e+01,     5.0496022212927564e+04,    -1.6779372414380171e-04,     2.2566137600924461e-07,    -1.0852554251636964e-09, } , 
      {   -3.4284170793191509e+00,    -2.5509947938752186e+04,     6.2473197606784645e-04,     5.2557196728041465e-07,    -2.4737618078614072e-09, } , 
      {   -1.7260658978996979e-01,    -8.6430156234712085e+03,     4.9668938615378649e-04,     5.1567207191215793e-07,    -2.4333283468568719e-09, } , 
      {    3.0832038997392104e+00,     8.2239166918097653e+03,     3.6864679623972648e-04,     5.0577217654390120e-07,    -2.3928948858523366e-09, } , 
      {    6.3390143892683906e+00,     2.5090849007090739e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    9.5948248787975707e+00,     4.1957781322371709e+04,     1.1256161641160650e-04,     4.8597238580738775e-07,    -2.3120279638432659e-09, } , 
      {    1.2850635368326753e+01,     5.8824713637652691e+04,    -1.5480973502453462e-05,     4.7607249043913103e-07,    -2.2715945028387306e-09, } , 
      {   -1.0728625070672511e+00,    -1.7181256514027060e+04,     7.7704472670919473e-04,     7.7598308171030107e-07,    -3.6601008855364418e-09, } , 
      {    2.1829479824619300e+00,    -3.1432419874608240e+02,     6.4900213679513477e-04,     7.6608318634204435e-07,    -3.6196674245319065e-09, } , 
      {    5.4387584719911102e+00,     1.6552608116534891e+04,     5.2095954688107470e-04,     7.5618329097378762e-07,    -3.5792339635273712e-09, } , 
      {    8.6945689615202895e+00,     3.3419540431815869e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.1950379451049471e+01,     5.0286472747096836e+04,     2.6487436705295472e-04,     7.3638350023727418e-07,    -3.4983670415183006e-09, } , 
      {    1.5206189940578652e+01,     6.7153405062377817e+04,     1.3683177713889476e-04,     7.2648360486901745e-07,    -3.4579335805137653e-09, } , 
      {    4.5385025547138307e+00,     8.0143672259790437e+03,     8.0131488743648304e-04,     1.0164943007719308e-06,    -4.8060065022069408e-09, } , 
      {    7.7943130442430109e+00,     2.4881299541260018e+04,     6.7327229752242297e-04,     1.0065944054036740e-06,    -4.7655730412024055e-09, } , 
      {    1.1050123533772190e+01,     4.1748231856540995e+04,     5.4522970760836301e-04,     9.9669451003541732e-07,    -4.7251395801978702e-09, } , 
      {    1.4305934023301372e+01,     5.8615164171821962e+04,     4.1718711769430300e-04,     9.8679461466716060e-07,    -4.6847061191933348e-09, } , 
      {    1.0149867616494912e+01,     3.3209990965985140e+04,     8.2558504816377125e-04,     1.2570055198335605e-06,    -5.9519121188774397e-09, } , 
      {    1.3405678106024091e+01,     5.0076923281266121e+04,     6.9754245824971129e-04,     1.2471056244653037e-06,    -5.9114786578729044e-09, } , 
      {    1.6661488595553273e+01,     6.6943855596547088e+04,     5.6949986833565122e-04,     1.2372057290970470e-06,    -5.8710451968683691e-09, } , 
      {    1.2505422188746810e+01,     4.1538682390710266e+04,     9.7789779880511941e-04,     1.5074166342634469e-06,    -7.1382511965524748e-09, } , 
      {    1.5761232678275992e+01,     5.8405614705991247e+04,     8.4985520889105945e-04,     1.4975167388951902e-06,    -7.0978177355479395e-09, } , 
      {    1.4860976760998710e+01,     4.9867373815435392e+04,     1.1302105494464677e-03,     1.7578277486933333e-06,    -8.3245902742275090e-09, } , 
      {    1.8116787250527892e+01,     6.6734306130716373e+04,     1.0021679595324077e-03,     1.7479278533250766e-06,    -8.2841568132229737e-09, } , 
      {   -1.5804358360023869e+01,    -5.7777312750742763e+04,    -8.5251909038396793e-04,    -1.4969003467810275e-06,     7.0433780072961494e-09, } , 
      {   -1.2548547870494687e+01,    -4.0910380435461782e+04,    -9.8056168029802800e-04,    -1.5068002421492842e-06,     7.0838114683006847e-09, } , 
      {   -1.3448803787771968e+01,    -4.9448621326017637e+04,    -7.0020633974261976e-04,    -1.2464892323511411e-06,     5.8570389296211143e-09, } , 
      {   -1.0192993298242790e+01,    -3.2581689010736656e+04,    -8.2824892965667972e-04,    -1.2563891277193978e-06,     5.8974723906256496e-09, } , 
      {   -1.4349059705049250e+01,    -5.7986862216573478e+04,    -4.1985099918721153e-04,    -9.8617822255299792e-07,     4.6302663909415448e-09, } , 
      {   -1.1093249215520068e+01,    -4.1119929901292511e+04,    -5.4789358910127149e-04,    -9.9607811792125464e-07,     4.6706998519460801e-09, } , 
      {   -7.8374387259908893e+00,    -2.4252997586011534e+04,    -6.7593617901533145e-04,    -1.0059780132895114e-06,     4.7111333129506154e-09, } , 
      {   -1.1993505132797349e+01,    -4.9658170791848352e+04,    -2.6753824854586325e-04,    -7.3576710812311150e-07,     3.4439273132665109e-09, } , 
      {   -8.7376946432681670e+00,    -3.2791238476567385e+04,    -3.9558083845992327e-04,    -7.4566700349136822e-07,     3.4843607742710462e-09, } , 
      {   -5.4818841537389886e+00,    -1.5924306161286409e+04,    -5.2362342837398317e-04,    -7.5556689885962494e-07,     3.5247942352755816e-09, } , 
      {   -2.2260736642098085e+00,     9.4262615399456467e+02,    -6.5166601828804324e-04,    -7.6546679422788167e-07,     3.5652276962801169e-09, } , 
      {   -9.6379505605454483e+00,    -4.1329479367123226e+04,    -1.1522549790451502e-04,    -4.8535599369322507e-07,     2.2575882355914763e-09, } , 
      {   -6.3821400710162690e+00,    -2.4462547051842255e+04,    -2.4326808781857502e-04,    -4.9525588906148180e-07,     2.2980216965960116e-09, } , 
      {   -3.1263295814870888e+00,    -7.5956147365612833e+03,    -3.7131067773263501e-04,    -5.0515578442973852e-07,     2.3384551576005469e-09, } , 
      {    1.2948090804209156e-01,     9.2713175787196906e+03,    -4.9935326764669497e-04,    -5.1505567979799525e-07,     2.3788886186050822e-09, } , 
      {   -1.0538206477822730e+01,    -4.9867720257679081e+04,     1.6512984265089321e-04,    -2.2504498389508196e-07,     1.0308156969119065e-09, } , 
      {   -7.2823959882935485e+00,    -3.3000787942398099e+04,     3.7087252736833238e-05,    -2.3494487926333868e-07,     1.0712491579164416e-09, } , 
      {   -4.0265854987643683e+00,    -1.6133855627117131e+04,    -9.0955337177226771e-05,    -2.4484477463159538e-07,     1.1116826189209769e-09, } , 
      {   -7.7077500923518838e-01,     7.3307668816384307e+02,    -2.1899792709128676e-04,    -2.5474466999985210e-07,     1.1521160799255122e-09, } , 
      {    2.4850354802939920e+00,     1.7600009003444819e+04,    -3.4704051700534675e-04,    -2.6464456536810883e-07,     1.1925495409300476e-09, } , 
      {   -4.9268414160416487e+00,    -2.4672096517672977e+04,     1.8940000337818149e-04,     1.5466235166547750e-08,    -1.1508991975859268e-10, } , 
      {   -1.6710309265124683e+00,    -7.8051642023920049e+03,     6.1357413464121477e-05,     5.5663397982910266e-09,    -7.4656458754057421e-11, } , 
      {    1.5847795630167119e+00,     9.0617681128889690e+03,    -6.6685176449938512e-05,    -4.3335555699656980e-09,    -3.4222997749522167e-11, } , 
      {    4.8405900525458918e+00,     2.5928700428169945e+04,    -1.9472776636399850e-04,    -1.4233450938222424e-08,     6.2104632550130938e-12, } , 
      {    8.0964005420750738e+00,     4.2795632743450922e+04,    -3.2277035627805849e-04,    -2.4133346306479148e-08,     4.6643924259548354e-11, } , 
      {   -2.5712868437897480e+00,    -1.6343405092947853e+04,     3.4171275401952969e-04,     2.6587734959643419e-07,    -1.3014289974336273e-09, } , 
      {    6.8452364573943192e-01,     5.2352722233312147e+02,     2.1367016410546975e-04,     2.5597745422817746e-07,    -1.2609955364290920e-09, } , 
      {    3.9403341352686123e+00,     1.7390459537614097e+04,     8.5627574191409736e-05,     2.4607755885992074e-07,    -1.2205620754245567e-09, } , 
      {    7.1961446247977916e+00,     3.4257391852895067e+04,    -4.2415015722650259e-05,     2.3617766349166399e-07,    -1.1801286144200214e-09, } , 
      {    1.0451955114326974e+01,     5.1124324168176048e+04,    -1.7045760563671021e-04,     2.2627776812340726e-07,    -1.1396951534154862e-09, } , 
      {   -2.1573227153784802e-01,    -8.0147136682227265e+03,     4.9402550466087802e-04,     5.1628846402632061e-07,    -2.4877680751086615e-09, } , 
      {    3.0400782179913319e+00,     8.8522186470582474e+03,     3.6598291474681795e-04,     5.0638856865806388e-07,    -2.4473346141041262e-09, } , 
      {    6.2958887075205121e+00,     2.5719150962339223e+04,     2.3794032483275801e-04,     4.9648867328980716e-07,    -2.4069011530995909e-09, } , 
      {    9.5516991970496932e+00,     4.2586083277620193e+04,     1.0989773491869798e-04,     4.8658877792155043e-07,    -2.3664676920950556e-09, } , 
      {    2.1398223007140516e+00,     3.1397775650239987e+02,     6.4633825530222629e-04,     7.6669957845620703e-07,    -3.6741071527836962e-09, } , 
      {    5.3956327902432317e+00,     1.7180910071783375e+04,     5.1829566538816622e-04,     7.5679968308795030e-07,    -3.6336736917791609e-09, } , 
      {    8.6514432797724119e+00,     3.4047842387064353e+04,     3.9025307547410620e-04,     7.4689978771969358e-07,    -3.5932402307746256e-09, } , 
      {    1.1907253769301594e+01,     5.0914774702345319e+04,     2.6221048556004619e-04,     7.3699989235143686e-07,    -3.5528067697700903e-09, } , 
      {    7.7511873624951324e+00,     2.5509601496508501e+04,     6.7060841602951450e-04,     1.0072107975178367e-06,    -4.8200127694541955e-09, } , 
      {    1.1006997852024313e+01,     4.2376533811789479e+04,     5.4256582611545453e-04,     9.9731090214958000e-07,    -4.7795793084496602e-09, } , 
      {    1.4262808341553495e+01,     5.9243466127070446e+04,     4.1452323620139446e-04,     9.8741100678132328e-07,    -4.7391458474451249e-09, } , 
      {    1.0106741934747035e+01,     3.3838292921233624e+04,     8.2292116667086277e-04,     1.2576219119477231e-06,    -6.0063518471292298e-09, } , 
      {    1.3362552424276213e+01,     5.0705225236514605e+04,     6.9487857675680281e-04,     1.2477220165794664e-06,    -5.9659183861246945e-09, } , 
      {    1.2462296506998932e+01,     4.2166984345958750e+04,     9.7523391731221094e-04,     1.5080330263776096e-06,    -7.1926909248042649e-09, } , 
      {    1.5718106996528114e+01,     5.9033916661239731e+04,     8.4719132739815098e-04,     1.4981331310093528e-06,    -7.1522574637997295e-09, } , 
      {   -1.3491929469519848e+01,    -4.8820319370769153e+04,    -7.0287022123552835e-04,    -1.2458728402369784e-06,     5.8025992013693251e-09, } , 
      {   -1.1136374897267947e+01,    -4.0491627946044027e+04,    -5.5055747059418007e-04,    -9.9546172580709196e-07,     4.6162601236942908e-09, } , 
      {   -7.8805644077387669e+00,    -2.3624695630763053e+04,    -6.7860006050824003e-04,    -1.0053616211753487e-06,     4.6566935846988261e-09, } , 
      {   -8.7808203250160464e+00,    -3.2162936521318905e+04,    -3.9824471995283174e-04,    -7.4505061137720554e-07,     3.4299210460192562e-09, } , 
      {   -5.5250098354868662e+00,    -1.5296004206037927e+04,    -5.2628730986689176e-04,    -7.5495050674546226e-07,     3.4703545070237915e-09, } , 
      {   -9.6810762422933276e+00,    -4.0701177411874742e+04,    -1.1788937939742352e-04,    -4.8473960157906239e-07,     2.2031485073396862e-09, } , 
      {   -6.4252657527641475e+00,    -2.3834245096593775e+04,    -2.4593196931148352e-04,    -4.9463949694731912e-07,     2.2435819683442215e-09, } , 
      {   -3.1694552632349668e+00,    -6.9673127813128012e+03,    -3.7397455922554348e-04,    -5.0453939231557584e-07,     2.2840154293487568e-09, } , 
      {    8.6355226294213328e-02,     9.8996195339681726e+03,    -5.0201714913960355e-04,    -5.1443928768383257e-07,     2.3244488903532921e-09, } , 
      {   -7.3255216700414270e+00,    -3.2372485987149619e+04,     3.4423371243924727e-05,    -2.3432848714917600e-07,     1.0168094296646520e-09, } , 
      {   -4.0697111805122468e+00,    -1.5505553671868649e+04,    -9.3619218670135275e-05,    -2.4422838251743275e-07,     1.0572428906691873e-09, } , 
      {   -8.1390069098306661e-01,     1.3613786434123253e+03,    -2.2166180858419529e-04,    -2.5412827788568948e-07,     1.0976763516737226e-09, } , 
      {    2.4419097985461136e+00,     1.8228310958693299e+04,    -3.4970439849825523e-04,    -2.6402817325394620e-07,     1.1381098126782579e-09, } , 
      {   -4.9699670977895263e+00,    -2.4043794562424497e+04,     1.8673612188527296e-04,     1.6082627280710414e-08,    -1.6952964801038250e-10, } , 
      {   -1.7141566082603465e+00,    -7.1768622471435228e+03,     5.8693531971212973e-05,     6.1827319124536909e-09,    -1.2909618700584724e-10, } , 
      {    1.5416538812688336e+00,     9.6900700681374510e+03,    -6.9349057942847016e-05,    -3.7171634558030342e-09,    -8.8662726001311964e-11, } , 
      {    4.7974643707980142e+00,     2.6557002383418425e+04,    -1.9739164785690703e-04,    -1.3617058824059758e-08,    -4.8229264996776703e-11, } , 
      {   -2.6144125255376265e+00,    -1.5715103137699371e+04,     3.3904887252662121e-04,     2.6649374171059681e-07,    -1.3558687256854170e-09, } , 
      {    6.4139796399155369e-01,     1.1518291775816037e+03,     2.1100628261256122e-04,     2.5659384634234009e-07,    -1.3154352646808816e-09, } , 
      {    3.8972084535207339e+00,     1.8018761492862577e+04,     8.2963692698501232e-05,     2.4669395097408337e-07,    -1.2750018036763463e-09, } , 
      {    7.1530189430499131e+00,     3.4885693808143551e+04,    -4.5078897215558770e-05,     2.3679405560582667e-07,    -1.2345683426718110e-09, } , 
      {   -2.5885795328572625e-01,    -7.3864117129742444e+03,     4.9136162316796943e-04,     5.1690485614048329e-07,    -2.5422078033604516e-09, } , 
      {    2.9969525362434539e+00,     9.4805206023067294e+03,     3.6331903325390947e-04,     5.0700496077222656e-07,    -2.5017743423559163e-09, } , 
      {    6.2527630257726337e+00,     2.6347452917587703e+04,     2.3527644333984948e-04,     4.9710506540396984e-07,    -2.4613408813513810e-09, } , 
      {    9.5085735153018138e+00,     4.3214385232868677e+04,     1.0723385342578948e-04,     4.8720517003571311e-07,    -2.4209074203468457e-09, } , 
      {    5.3525071084953542e+00,     1.7809212027031856e+04,     5.1563178389525764e-04,     7.5741607520211298e-07,    -3.6881134200309510e-09, } , 
      {    8.6083175980245326e+00,     3.4676144342312837e+04,     3.8758919398119773e-04,     7.4751617983385626e-07,    -3.6476799590264156e-09, } , 
      {    1.0963872170276433e+01,     4.3004835767037963e+04,     5.3990194462254595e-04,     9.9792729426374268e-07,    -4.8340190367014495e-09, } , 
      {   -8.8239460067639239e+00,    -3.1534634566070421e+04,    -4.0090860144574028e-04,    -7.4443421926304286e-07,     3.3754813177674665e-09, } , 
      {   -5.5681355172347446e+00,    -1.4667702250789445e+04,    -5.2895119135980024e-04,    -7.5433411463129958e-07,     3.4159147787720018e-09, } , 
      {   -6.4683914345120250e+00,    -2.3205943141345291e+04,    -2.4859585080439205e-04,    -4.9402310483315644e-07,     2.1891422400924319e-09, } , 
      {   -3.2125809449828449e+00,    -6.3390108260643183e+03,    -3.7663844071845202e-04,    -5.0392300020141316e-07,     2.2295757010969672e-09, } , 
      {   -4.1128368622601252e+00,    -1.4877251716620167e+04,    -9.6283100163043793e-05,    -2.4361199040327007e-07,     1.0028031624173974e-09, } , 
      {   -8.5702637273094484e-01,     1.9896805986608076e+03,    -2.2432569007710380e-04,    -2.5351188577152680e-07,     1.0432366234219327e-09, } , 
      {   -1.7572822900082248e+00,    -6.5485602918950399e+03,     5.6029650478304455e-05,     6.7991240266163544e-09,    -1.8353591525763703e-10, } , 
      {    1.4985281995209554e+00,     1.0318372023385933e+04,    -7.2012939435755534e-05,    -3.1007713416403703e-09,    -1.4310245425310177e-10, } , 
      {    5.9827228224367546e-01,     1.7801311328300860e+03,     2.0834240111965272e-04,     2.5721023845650277e-07,    -1.3698749929326715e-09, } , 
      {    3.8540827717728554e+00,     1.8647063448111061e+04,     8.0299811205592714e-05,     2.4731034308824605e-07,    -1.3294415319281362e-09, } , 
      {    2.9538268544955759e+00,     1.0108822557555211e+04,     3.6065515176100094e-04,     5.0762135288638924e-07,    -2.5562140706077060e-09, } , 
      {    6.2096373440247561e+00,     2.6975754872836187e+04,     2.3261256184694098e-04,     4.9772145751813252e-07,    -2.5157806096031706e-09, } , 
      {   -4.1559625440080037e+00,    -1.4248949761371685e+04,    -9.8946981655952297e-05,    -2.4299559828910739e-07,     9.4836343416560753e-10, } , 
      {   -9.0015205447882307e-01,     2.6179825539092899e+03,    -2.2698957157001230e-04,    -2.5289549365736412e-07,     9.8879689517014284e-10, } , 
      {   -1.8004079717561030e+00,    -5.9202583366465578e+03,     5.3365768985395951e-05,     7.4155161407790187e-09,    -2.3797564350942682e-10, } , 
      {    1.4554025177730772e+00,     1.0946673978634415e+04,    -7.4676820928664038e-05,    -2.4843792274777060e-09,    -1.9754218250489156e-10, } , 
      {    5.5514660049579723e-01,     2.4084330880785683e+03,     2.0567851962674421e-04,     2.5782663057066545e-07,    -1.4243147211844614e-09, } , 
      {    3.8109570900249774e+00,     1.9275365403359541e+04,     7.7635929712684210e-05,     2.4792673520240873e-07,    -1.3838812601799261e-09, } , 
      {    1.3443840497821407e+00,    -1.0875686303019080e+04,    -1.1339387181728319e-04,    -2.1533688081907216e-07,     1.1750768558628600e-09, } , 
      {    3.6999386220340407e+00,    -2.5469948782939546e+03,     3.8918878824065057e-05,     3.5074233610814263e-08,    -1.1262221812174501e-11, } , 
      {    6.9557491115632208e+00,     1.4319937436987020e+04,    -8.9123711089994932e-05,     2.5174338242557539e-08,     2.9171239192360760e-11, } , 
      {    6.0554931942859405e+00,     5.7816965464311716e+03,     1.9123162946541332e-04,     2.8548534804070068e-07,    -1.1976012994872089e-09, } , 
      {   -1.0542962042176378e+00,    -1.8576075772495726e+04,    -2.6837050395153998e-04,    -4.6513160313479590e-07,     2.3069762052861046e-09, } , 
      {    2.2015142853115424e+00,    -1.7091434572147505e+03,    -3.9641309386559995e-04,    -4.7503149850305262e-07,     2.3474096662906399e-09, } , 
      {    1.3012583680342624e+00,    -1.0247384347770598e+04,    -1.1605775331019171e-04,    -2.1472048870490950e-07,     1.1206371276110701e-09, } , 
      {    4.5570688575634435e+00,     6.6195479675103761e+03,    -2.4410034322425170e-04,    -2.2462038407316623e-07,     1.1610705886156054e-09, } , 
      {    3.6568129402861627e+00,    -1.9186929230454721e+03,     3.6254997331156539e-05,     3.5690625724976930e-08,    -6.5701950063964304e-11, } , 
      {    6.9126234298153424e+00,     1.4948239392235502e+04,    -9.1787592582903449e-05,     2.5790730356720206e-08,    -2.5268489059429044e-11, } , 
      {    6.0123675125380629e+00,     6.4099985016796545e+03,     1.8856774797250481e-04,     2.8610174015486336e-07,    -1.2520410277389988e-09, } , 
      {    9.2681780020672431e+00,     2.3276930816960627e+04,     6.0525158058444799e-05,     2.7620184478660664e-07,    -1.2116075667344635e-09, } , 
      {    8.3679220847899636e+00,     1.4738689926404781e+04,     3.4088049861385301e-04,     5.3651285458474973e-07,    -2.4383801054140332e-09, } , 
      {    1.1623732574319144e+01,     3.1605622241685753e+04,     2.1283790869979307e-04,     5.2661295921649301e-07,    -2.3979466444094979e-09, } , 
      {   -5.8085310304693163e+00,    -3.4605156666697498e+04,    -5.7565988672714501e-04,    -9.6533743988040611e-07,     4.6252146323843838e-09, } , 
      {   -3.4529764582174156e+00,    -2.6276465241972372e+04,    -4.2334713608579668e-04,    -7.1492632545051969e-07,     3.4388755547093492e-09, } , 
      {   -1.9716596868823566e-01,    -9.4095329266913941e+03,    -5.5138972599985670e-04,    -7.2482622081877642e-07,     3.4793090157138845e-09, } , 
      {   -1.0974218859655160e+00,    -1.7947773817247245e+04,    -2.7103438544444846e-04,    -4.6451521102063327e-07,     2.2525364770343145e-09, } , 
      {    2.1583886035636644e+00,    -1.0808415019662682e+03,    -3.9907697535850842e-04,    -4.7441510638889000e-07,     2.2929699380388498e-09, } , 
      {   -1.9976778032427960e+00,    -2.6486014707803093e+04,     9.3209551109597826e-06,    -2.0420420122249010e-07,     1.0257639383547452e-09, } , 
      {    1.2581326862863842e+00,    -9.6190823925221157e+03,    -1.1872163480310021e-04,    -2.1410409659074682e-07,     1.0661973993592805e-09, } , 
      {    4.5139431758155650e+00,     7.2478499227588582e+03,    -2.4676422471716020e-04,    -2.2400399195900355e-07,     1.1066308603638158e-09, } , 
      {    3.5787676900910426e-01,    -1.8157323283077967e+04,     1.6163370575230802e-04,     4.6206913207396315e-08,    -1.6057513932028937e-10, } , 
      {    3.6136872585382847e+00,    -1.2903909677969898e+03,     3.3591115838248028e-05,     3.6307017839139590e-08,    -1.2014167831575411e-10, } , 
      {    6.8694977480674648e+00,     1.5576541347483984e+04,    -9.4451474075811953e-05,     2.6407122470882869e-08,    -7.9708217311218847e-11, } , 
      {    1.0125308237596645e+01,     3.2443473662764954e+04,    -2.2249406398987197e-04,     1.6507227102626144e-08,    -3.9274756306683580e-11, } , 
      {    2.7134313412610043e+00,    -9.8286318583528391e+03,     3.1394645639365627e-04,     2.9661802763728272e-07,    -1.3469142169953237e-09, } , 
      {    5.9692418307901844e+00,     7.0383004569281366e+03,     1.8590386647959628e-04,     2.8671813226902599e-07,    -1.3064807559907884e-09, } , 
      {    9.2250523203193637e+00,     2.3905232772209107e+04,     5.7861276565536288e-05,     2.7681823690076927e-07,    -1.2660472949862531e-09, } , 
      {    1.2480862809848546e+01,     4.0772165087490081e+04,    -7.0181313348523708e-05,     2.6691834153251254e-07,    -1.2256138339817180e-09, } , 
      {    8.3247964030420842e+00,     1.5366991881653263e+04,     3.3821661712094453e-04,     5.3712924669891241e-07,    -2.4928198336658233e-09, } , 
      {    1.1580606892571264e+01,     3.2233924196934233e+04,     2.1017402720688454e-04,     5.2722935133065569e-07,    -2.4523863726612880e-09, } , 
      {    1.0680350975293983e+01,     2.3695683306378385e+04,     4.9052936776229270e-04,     7.8754036112879883e-07,    -3.6791589113408580e-09, } , 
      {    1.3936161464823165e+01,     4.0562615621659359e+04,     3.6248677784823279e-04,     7.7764046576054211e-07,    -3.6387254503363226e-09, } , 
      {    1.3035905547545884e+01,     3.2024374731103511e+04,     6.4284211840364097e-04,     1.0379514755586853e-06,    -4.8654979890158918e-09, } , 
      {    1.6291716037075066e+01,     4.8891307046384485e+04,     5.1479952848958101e-04,     1.0280515801904285e-06,    -4.8250645280113565e-09, } , 
      {   -8.2072112844690963e+00,    -4.2305546136174140e+04,    -7.3063651886140176e-04,    -1.2151321621961299e-06,     5.7571139818076280e-09, } , 
      {   -4.9514007949399153e+00,    -2.5438613820893166e+04,    -8.5867910877546172e-04,    -1.2250320575643866e-06,     5.7975474428121633e-09, } , 
      {   -5.8516567122171947e+00,    -3.3976854711449014e+04,    -5.7832376822005349e-04,    -9.6472104776624343e-07,     4.5707749041325938e-09, } , 
      {   -2.5958462226880146e+00,    -1.7109922396168040e+04,    -7.0636635813411345e-04,    -9.7462094313450016e-07,     4.6112083651371291e-09, } , 
      {   -3.4961021399652941e+00,    -2.5648163286723888e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {   -2.4029165043611389e-01,    -8.7812309714429121e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    3.0155188390930672e+00,     8.0857013438380600e+03,    -6.8209619740682514e-04,    -7.3410972407287046e-07,     3.4653027484666301e-09, } , 
      {   -4.3963580572425744e+00,    -3.4186404177279735e+04,    -1.4565567702329698e-04,    -4.5399892353821387e-07,     2.1576632877779900e-09, } , 
      {   -1.1405475677133943e+00,    -1.7319471861998762e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.1152629218157859e+00,    -4.5253954671778592e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    5.3710734113449670e+00,     1.6414392768563186e+04,    -5.2978344676547697e-04,    -4.8369860964298404e-07,     2.2789636707915955e-09, } , 
      {   -2.0408034849906742e+00,    -2.5857712752554609e+04,     6.6570736180512717e-06,    -2.0358780910832745e-07,     9.7132421010295529e-10, } , 
      {    1.2150070045385060e+00,    -8.9907804372736337e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.4708174940676866e+00,     7.8761518780073402e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.7266279835968668e+00,     2.4743084193288312e+04,    -3.7747069612412869e-04,    -2.3328749521309762e-07,     1.0926245931165610e-09, } , 
      {   -2.9410594022679550e+00,    -3.4395953643110457e+04,     2.8701241417345948e-04,     5.6723200689815707e-08,    -2.5544832857661442e-10, } , 
      {    3.1475108726122603e-01,    -1.7529021327829483e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    3.5705615767904062e+00,    -6.6208901254850753e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.8263720663195864e+00,     1.6204843302732466e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0082182555848767e+01,     3.3071775618013438e+04,    -2.2515794548278047e-04,     1.7123619216788808e-08,    -9.3714484558473377e-11, } , 
      {    1.3337993045377948e+01,     4.9938707933294419e+04,    -3.5320053539684043e-04,     7.2237238485320832e-09,    -5.3281023553938117e-11, } , 
      {   -5.8550483001605480e-01,    -2.6067262218385331e+04,     4.3932516481480770e-04,     3.0713431511970212e-07,    -1.4417874062516489e-09, } , 
      {    2.6703056595131263e+00,    -9.2003299031043571e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    5.9261161490423060e+00,     7.6666024121766186e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.1819266385714862e+00,     2.4533534727457591e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.2437737128100668e+01,     4.1400467042738564e+04,    -7.2845194841432225e-05,     2.6753473364667522e-07,    -1.2800535622335079e-09, } , 
      {    8.2816707212942067e+00,     1.5995293836901745e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.1537481210823387e+01,     3.2862226152182717e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.4793291700352569e+01,     4.9729158467463691e+04,     7.9467555799916023e-05,     5.1794584807656164e-07,    -2.4663926399085423e-09, } , 
      {    1.0637225293546106e+01,     2.4323985261626869e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    1.3893035783075288e+01,     4.1190917576907843e+04,     3.5982289635532426e-04,     7.7825685787470479e-07,    -3.6931651785881123e-09, } , 
      {    1.7148846272604466e+01,     5.8057849892188817e+04,     2.3178030644126424e-04,     7.6835696250644806e-07,    -3.6527317175835770e-09, } , 
      {    1.2992779865798006e+01,     3.2652676686351995e+04,     6.4017823691073250e-04,     1.0385678676728479e-06,    -4.9199377172676819e-09, } , 
      {    1.6248590355327188e+01,     4.9519609001632969e+04,     5.1213564699667253e-04,     1.0286679723045912e-06,    -4.8795042562631466e-09, } , 
      {    1.5348334438049907e+01,     4.0981368111077121e+04,     7.9249098755208077e-04,     1.2889789821027344e-06,    -6.1062767949427161e-09, } , 
      {    1.8604144927579089e+01,     5.7848300426358095e+04,     6.6444839763802081e-04,     1.2790790867344776e-06,    -6.0658433339381808e-09, } , 
      {   -8.2503369662169739e+00,    -4.1677244180925656e+04,    -7.3330040035431024e-04,    -1.2145157700819672e-06,     5.7026742535558379e-09, } , 
      {   -5.8947823939650732e+00,    -3.3348552756200530e+04,    -5.8098764971296196e-04,    -9.6410465565208075e-07,     4.5163351758808037e-09, } , 
      {   -2.6389719044358930e+00,    -1.6481620440919556e+04,    -7.0903023962702193e-04,    -9.7400455102033748e-07,     4.5567686368853390e-09, } , 
      {   -3.5392278217131725e+00,    -2.5019861331475404e+04,    -4.2867489907161374e-04,    -7.1369354122219433e-07,     3.3299960982057698e-09, } , 
      {   -2.8341733218399212e-01,    -8.1529290161944300e+03,    -5.5671748898567365e-04,    -7.2359343659045106e-07,     3.3704295592103052e-09, } , 
      {   -4.4394837389904529e+00,    -3.3558102222031252e+04,    -1.4831955851620548e-04,    -4.5338253142405119e-07,     2.1032235595262003e-09, } , 
      {   -1.1836732494612725e+00,    -1.6691169906750278e+04,    -2.7636214843026552e-04,    -4.6328242679230791e-07,     2.1436570205307352e-09, } , 
      {    2.0721372400679074e+00,     1.7576240853069635e+02,    -4.0440473834432548e-04,    -4.7318232216056464e-07,     2.1840904815352705e-09, } , 
      {    5.3279477295970885e+00,     1.7042694723811670e+04,    -5.3244732825838545e-04,    -4.8308221752882136e-07,     2.2245239425398058e-09, } , 
      {   -2.0839291667385522e+00,    -2.5229410797306125e+04,     3.9931921251427601e-06,    -2.0297141699416479e-07,     9.1688448185116553e-10, } , 
      {    1.1718813227906277e+00,    -8.3624784820251516e+03,    -1.2404939778891725e-04,    -2.1287131236242152e-07,     9.5731794285570074e-10, } , 
      {    4.4276918123198081e+00,     8.5044538332558222e+03,    -2.5209198770297726e-04,    -2.2277120773067824e-07,     9.9775140386023605e-10, } , 
      {    7.6835023018489883e+00,     2.5371386148536796e+04,    -3.8013457761703723e-04,    -2.3267110309893497e-07,     1.0381848648647712e-09, } , 
      {    2.7162540551334780e-01,    -1.6900719372580999e+04,     1.5630594276649102e-04,     4.7439697435721649e-08,    -2.6945459582386898e-10, } , 
      {    3.5274358950425277e+00,    -3.3787057300025253e+01,     2.8263352852431007e-05,     3.7539802067464924e-08,    -2.2902113481933369e-10, } , 
      {    6.7832463845717079e+00,     1.6833145257980948e+04,    -9.9779237061628989e-05,     2.7639906699208196e-08,    -1.8858767381479843e-10, } , 
      {    1.0039056874100890e+01,     3.3700077573261922e+04,    -2.2782182697568898e-04,     1.7740011330951471e-08,    -1.4815421281026317e-10, } , 
      {    2.6271799777652483e+00,    -8.5720279478558750e+03,     3.0861869340783921e-04,     2.9785081186560808e-07,    -1.4557936734989035e-09, } , 
      {    5.8829904672944275e+00,     8.2949043674251006e+03,     1.8057610349377928e-04,     2.8795091649735135e-07,    -1.4153602124943682e-09, } , 
      {    9.1388009568236086e+00,     2.5161836682706074e+04,     5.2533513579719266e-05,     2.7805102112909463e-07,    -1.3749267514898329e-09, } , 
      {    1.2394611446352791e+01,     4.2028768997987048e+04,    -7.5509076334340743e-05,     2.6815112576083790e-07,    -1.3344932904852978e-09, } , 
      {    4.9827345500171480e+00,    -2.4333652313074867e+02,     4.6093144404918748e-04,     5.4826192629549450e-07,    -2.6421327511739375e-09, } , 
      {    8.2385450395463291e+00,     1.6623595792150227e+04,     3.3288885413512747e-04,     5.3836203092723777e-07,    -2.6016992901694026e-09, } , 
      {    1.1494355529075509e+01,     3.3490528107431201e+04,     2.0484626422106753e-04,     5.2846213555898105e-07,    -2.5612658291648673e-09, } , 
      {    1.4750166018604691e+01,     5.0357460422712174e+04,     7.6803674307007505e-05,     5.1856224019072432e-07,    -2.5208323681603320e-09, } , 
      {    1.0594099611798228e+01,     2.4952287216875353e+04,     4.8520160477647569e-04,     7.8877314535712419e-07,    -3.7880383678444377e-09, } , 
      {    1.3849910101327410e+01,     4.1819219532156327e+04,     3.5715901486241573e-04,     7.7887324998886747e-07,    -3.7476049068399024e-09, } , 
      {    1.2949654184050129e+01,     3.3280978641600479e+04,     6.3751435541782402e-04,     1.0391842597870106e-06,    -4.9743774455194719e-09, } , 
      {    1.6205464673579311e+01,     5.0147910956881453e+04,     5.0947176550376406e-04,     1.0292843644187539e-06,    -4.9339439845149366e-09, } , 
      {   -3.5823535034610505e+00,    -2.4391559376226924e+04,    -4.3133878056452222e-04,    -7.1307714910803165e-07,     3.2755563699539798e-09, } , 
      {   -3.2654301393187035e-01,    -7.5246270609459480e+03,    -5.5938137047858224e-04,    -7.2297704447628838e-07,     3.3159898309585151e-09, } , 
      {   -1.2267989312091507e+00,    -1.6062867951501798e+04,    -2.7902602992317400e-04,    -4.6266603467814528e-07,     2.0892172922789451e-09, } , 
      {    2.0290115583200294e+00,     8.0406436377917862e+02,    -4.0706861983723396e-04,    -4.7256593004640201e-07,     2.1296507532834804e-09, } , 
      {    1.1287556410427495e+00,    -7.7341765267766696e+03,    -1.2671327928182575e-04,    -2.1225492024825884e-07,     9.0287821460391097e-10, } , 
      {    4.3845661305719297e+00,     9.1327557885043043e+03,    -2.5475586919588574e-04,    -2.2215481561651556e-07,     9.4331167560844639e-10, } , 
      {    3.4843102132946497e+00,     5.9451489794845702e+02,     2.5599471359522496e-05,     3.8156194181627584e-08,    -2.8346086307112348e-10, } , 
      {    6.7401207028238304e+00,     1.7461447213229432e+04,    -1.0244311855453749e-04,     2.8256298813370859e-08,    -2.4302740206658822e-10, } , 
      {    5.8398647855465491e+00,     8.9232063226735827e+03,     1.7791222200087074e-04,     2.8856730861151398e-07,    -1.4697999407461578e-09, } , 
      {    9.0956752750757293e+00,     2.5790138637954555e+04,     4.9869632086810755e-05,     2.7866741324325725e-07,    -1.4293664797416225e-09, } , 
      {    8.1954193577984498e+00,     1.7251897747398711e+04,     3.3022497264221899e-04,     5.3897842304140045e-07,    -2.6561390184211927e-09, } , 
      {    1.1451229847327630e+01,     3.4118830062679684e+04,     2.0218238272815900e-04,     5.2907852767314373e-07,    -2.6157055574166574e-09, } , 
      {    1.3806784419579531e+01,     4.2447521487404811e+04,     3.5449513336950725e-04,     7.7948964210303015e-07,    -3.8020446350916916e-09, } , 
      {    3.4411845315467717e+00,     1.2228168531969393e+03,     2.2935589866613982e-05,     3.8772586295790251e-08,    -3.3790059132291330e-10, } , 
      {    6.6969950210759519e+00,     1.8089749168477912e+04,    -1.0510700004744601e-04,     2.8872690927533526e-08,    -2.9746713031837804e-10, } , 
      {    6.6291022348328923e+00,    -4.3609130684240654e+03,    -1.4116016944315665e-04,    -1.8459620277822360e-07,     1.1295916363011633e-09, } , 
      {    9.8849127243620742e+00,     1.2506019246856908e+04,    -2.6920275935721666e-04,    -1.9449609814648033e-07,     1.1700250973056984e-09, } , 
      {    8.9846568070847930e+00,     3.9677783563010607e+03,     1.1152581198191598e-05,     6.5814911651662831e-08,    -5.6747441373871162e-11, } , 
      {    1.2240467296613975e+01,     2.0834710671582034e+04,    -1.1689000871586837e-04,     5.5915016283406107e-08,    -1.6313980369335953e-11, } , 
      {    4.2304219808331140e+00,    -1.2061302537900709e+04,    -2.9613680157741340e-04,    -4.3439092509394737e-07,     2.2614909857244079e-09, } , 
      {    6.5859765530850138e+00,    -3.7326111131755833e+03,    -1.4382405093606515e-04,    -1.8397981066406095e-07,     1.0751519080493735e-09, } , 
      {    9.8417870426141967e+00,     1.3134321202105391e+04,    -2.7186664085012514e-04,    -1.9387970603231767e-07,     1.1155853690539088e-09, } , 
      {    8.9415311253369154e+00,     4.5960803115495428e+03,     8.4886997052830875e-06,     6.6431303765825498e-08,    -1.1118716962566095e-10, } , 
      {    1.2197341614866097e+01,     2.1463012626830518e+04,    -1.1955389020877689e-04,     5.6531408397568773e-08,    -7.0753708621125744e-11, } , 
      {    1.1297085697588816e+01,     1.2924771736274670e+04,     1.6080145034663135e-04,     3.1684241819571189e-07,    -1.2975262473006954e-09, } , 
      {    1.4552896187117998e+01,     2.9791704051555644e+04,     3.2758860432571357e-05,     3.0694252282745517e-07,    -1.2570927862961601e-09, } , 
      {    1.6908450759369899e+01,     3.8120395476280770e+04,     1.8507161107391961e-04,     5.5735363725734164e-07,    -2.4434318639711946e-09, } , 
      {    1.8317417268333360e+00,    -1.9761692007377354e+04,    -4.5111343371167015e-04,    -6.8418564740967116e-07,     3.3933903351476529e-09, } , 
      {    4.1872962990852356e+00,    -1.1433000582652227e+04,    -2.9880068307032193e-04,    -4.3377453297978469e-07,     2.2070512574726183e-09, } , 
      {    7.4431067886144175e+00,     5.4339317326287464e+03,    -4.2684327298438189e-04,    -4.4367442834804141e-07,     2.2474847184771536e-09, } , 
      {    6.5428508713371354e+00,    -3.1043091579271013e+03,    -1.4648793242897365e-04,    -1.8336341854989827e-07,     1.0207121797975836e-09, } , 
      {    9.7986613608663173e+00,     1.3762623157353873e+04,    -2.7453052234303367e-04,    -1.9326331391815499e-07,     1.0611456408021189e-09, } , 
      {    1.3054471850395496e+01,     3.0629555472634846e+04,    -4.0257311225709363e-04,    -2.0316320928641172e-07,     1.1015791018066542e-09, } , 
      {    5.6425949540598568e+00,    -1.1642550048482948e+04,     1.3386740812643458e-04,     7.6947591248244890e-08,    -2.0606035888198606e-10, } , 
      {    8.8984054435890361e+00,     5.2243822667980248e+03,     5.8248182123745767e-06,     6.7047695879988165e-08,    -1.6562689787745074e-10, } , 
      {    1.2154215933118218e+01,     2.2091314582078998e+04,    -1.2221777170168539e-04,     5.7147800511731434e-08,    -1.2519343687291553e-10, } , 
      {    1.5410026422647396e+01,     3.8958246897359975e+04,    -2.5026036161574541e-04,     4.7247905143474709e-08,    -8.4759975868380222e-11, } , 
      {    7.9981495263117566e+00,    -3.3138586237578229e+03,     2.8618015876778280e-04,     3.2735870567813130e-07,    -1.3923994365570206e-09, } , 
      {    1.1253960015840937e+01,     1.3553073691523152e+04,     1.5813756885372284e-04,     3.1745881030987457e-07,    -1.3519659755524853e-09, } , 
      {    1.4509770505370119e+01,     3.0420006006804124e+04,     3.0094978939662850e-05,     3.0755891494161785e-07,    -1.3115325145479500e-09, } , 
      {    1.3609514588092837e+01,     2.1881765116248276e+04,     3.1045031949507106e-04,     5.6786992473976094e-07,    -2.5383050532275195e-09, } , 
      {    1.6865325077622021e+01,     3.8748697431529254e+04,     1.8240772958101110e-04,     5.5797002937150422e-07,    -2.4978715922229842e-09, } , 
      {    1.5965069160344736e+01,     3.0210456540973402e+04,     4.6276307013641928e-04,     8.1828103916964736e-07,    -3.7246441309025546e-09, } , 
      {    1.9220879649873922e+01,     4.7077388856254380e+04,     3.3472048022235932e-04,     8.0838114380139064e-07,    -3.6842106698980193e-09, } , 
      {   -5.6693852716644288e-01,    -2.7462081476853997e+04,    -6.0609006584592690e-04,    -9.3398036972539490e-07,     4.5252896845708967e-09, } , 
      {    2.6888719623627391e+00,    -1.0595149161573023e+04,    -7.3413265575998686e-04,    -9.4388026509365163e-07,     4.5657231455754320e-09, } , 
      {    1.7886160450854578e+00,    -1.9133390052128871e+04,    -4.5377731520457868e-04,    -6.8356925529550848e-07,     3.3389506068958633e-09, } , 
      {    5.0444265346146402e+00,    -2.2664577368478972e+03,    -5.8181990511863859e-04,    -6.9346915066376521e-07,     3.3793840679003982e-09, } , 
      {    4.1441706173373571e+00,    -1.0804698627403744e+04,    -3.0146456456323046e-04,    -4.3315814086562201e-07,     2.1526115292208286e-09, } , 
      {    7.3999811068665391e+00,     6.0622336878772294e+03,    -4.2950715447729042e-04,    -4.4305803623387873e-07,     2.1930449902253639e-09, } , 
      {    1.0655791596395717e+01,     2.2929166003158203e+04,    -5.5754974439135038e-04,    -4.5295793160213546e-07,     2.2334784512298992e-09, } , 
      {    3.2439147000600785e+00,    -1.9342939517959592e+04,    -2.1109224007822183e-05,    -1.7284713106747889e-07,     9.2583899054125863e-10, } , 
      {    6.4997251895892578e+00,    -2.4760072026786188e+03,    -1.4915181392188216e-04,    -1.8274702643573562e-07,     9.6627245154579394e-10, } , 
      {    9.7555356791184398e+00,     1.4390925112602356e+04,    -2.7719440383594220e-04,    -1.9264692180399234e-07,     1.0067059125503291e-09, } , 
      {    1.3011346168647618e+01,     3.1257857427883329e+04,    -4.0523699375000216e-04,    -2.0254681717224906e-07,     1.0471393735548644e-09, } , 
      {    5.5994692723119783e+00,    -1.1014248093234466e+04,     1.3120352663352608e-04,     7.7563983362407543e-08,    -2.6050008713377587e-10, } , 
      {    8.8552797618411585e+00,     5.8526842220465078e+03,     3.1609367194660642e-06,     6.7664087994150818e-08,    -2.2006662612924056e-10, } , 
      {    1.2111090251370340e+01,     2.2719616537327482e+04,    -1.2488165319459390e-04,     5.7764192625894100e-08,    -1.7963316512470535e-10, } , 
      {    1.5366900740899519e+01,     3.9586548852608452e+04,    -2.5292424310865394e-04,     4.7864297257637376e-08,    -1.3919970412017004e-10, } , 
      {    7.9550238445638790e+00,    -2.6855566685093404e+03,     2.8351627727487427e-04,     3.2797509779229398e-07,    -1.4468391648088105e-09, } , 
      {    1.1210834334093059e+01,     1.4181375646771634e+04,     1.5547368736081434e-04,     3.1807520242403725e-07,    -1.4064057038042752e-09, } , 
      {    1.4466644823622241e+01,     3.1048307962052608e+04,     2.7431097446754335e-05,     3.0817530705578053e-07,    -1.3659722427997399e-09, } , 
      {    1.7722455313151418e+01,     4.7915240277333578e+04,    -1.0061149246730568e-04,     2.9827541168752380e-07,    -1.3255387817952045e-09, } , 
      {    1.0310578416815780e+01,     5.6431347562157862e+03,     4.3582902791622255e-04,     5.7838621222218035e-07,    -2.6331782424838445e-09, } , 
      {    1.3566388906344960e+01,     2.2510067071496760e+04,     3.0778643800216253e-04,     5.6848631685392362e-07,    -2.5927447814793092e-09, } , 
      {    1.6822199395874140e+01,     3.9376999386777730e+04,     1.7974384808810260e-04,     5.5858642148566690e-07,    -2.5523113204747739e-09, } , 
      {    2.0078009885403318e+01,     5.6243931702058704e+04,     5.1701258174042568e-05,     5.4868652611741017e-07,    -2.5118778594702386e-09, } , 
      {    1.5921943478596859e+01,     3.0838758496221886e+04,     4.6009918864351075e-04,     8.1889743128381004e-07,    -3.7790838591543447e-09, } , 
      {    1.9177753968126041e+01,     4.7705690811502856e+04,     3.3205659872945079e-04,     8.0899753591555332e-07,    -3.7386503981498094e-09, } , 
      {    1.8277498050848756e+01,     3.9167449920947009e+04,     6.1241193928485908e-04,     1.0693085457136965e-06,    -4.9654229368293789e-09, } , 
      {    2.1533308540377938e+01,     5.6034382236227983e+04,     4.8436934937079907e-04,     1.0594086503454397e-06,    -4.9249894758248436e-09, } , 
      {   -5.3211733534181205e+00,    -4.3491162371055769e+04,    -9.1337944862153193e-04,    -1.4341862064710051e-06,     6.8435281116691768e-09, } , 
      {   -2.9656187811662216e+00,    -3.5162470946330643e+04,    -7.6106669798018376e-04,    -1.1837750920411186e-06,     5.6571890339941417e-09, } , 
      {    2.9019170836296015e-01,    -1.8295538631049669e+04,    -8.8910928789424372e-04,    -1.1936749874093754e-06,     5.6976224949986770e-09, } , 
      {   -6.1006420891432112e-01,    -2.6833779521605516e+04,    -6.0875394733883549e-04,    -9.3336397761123222e-07,     4.4708499563191074e-09, } , 
      {    2.6457462806148611e+00,    -9.9668472063245408e+03,    -7.3679653725289545e-04,    -9.4326387297948895e-07,     4.5112834173236427e-09, } , 
      {   -1.5103201261915997e+00,    -3.5372020412161364e+04,    -3.2839860678342714e-04,    -6.7305296781308908e-07,     3.2440774176395379e-09, } , 
      {    1.7454903633375796e+00,    -1.8505088096880390e+04,    -4.5644119669748716e-04,    -6.8295286318134580e-07,     3.2845108786440732e-09, } , 
      {    5.0013008528667617e+00,    -1.6381557815994149e+03,    -5.8448378661154717e-04,    -6.9285275854960253e-07,     3.3249443396486081e-09, } , 
      {    8.2571113423959392e+00,     1.5228776533681559e+04,    -7.1252637652560724e-04,    -7.0275265391785925e-07,     3.3653778006531434e-09, } , 
      {    8.4523444606030007e-01,    -2.7043328987436238e+04,    -1.7608585614207895e-04,    -4.2264185338320266e-07,     2.0577383399645032e-09, } , 
      {    4.1010449355894796e+00,    -1.0176396672155262e+04,    -3.0412844605613894e-04,    -4.3254174875145938e-07,     2.0981718009690385e-09, } , 
      {    7.3568554251186615e+00,     6.6905356431257114e+03,    -4.3217103597019890e-04,    -4.4244164411971611e-07,     2.1386052619735738e-09, } , 
      {    1.0612665914647838e+01,     2.3557467958406683e+04,    -5.6021362588425897e-04,    -4.5234153948797283e-07,     2.1790387229781092e-09, } , 
      {   -5.5021471216980755e-02,    -3.5581569877992086e+04,     1.0426948441332927e-04,    -1.6233084358505949e-07,     8.3096580128493356e-10, } , 
      {    3.2007890183122001e+00,    -1.8714637562711112e+04,    -2.3773105500730694e-05,    -1.7223073895331621e-07,     8.7139926228946876e-10, } , 
      {    6.4565995078413794e+00,    -1.8477052474301365e+03,    -1.5181569541479069e-04,    -1.8213063432157294e-07,     9.1183272329400408e-10, } , 
      {    9.7124099973705604e+00,     1.5019227067850838e+04,    -2.7985828532885068e-04,    -1.9203052968982966e-07,     9.5226618429853939e-10, } , 
      {    1.2968220486899739e+01,     3.1886159383131810e+04,    -4.0790087524291064e-04,    -2.0193042505808638e-07,     9.9269964530307470e-10, } , 
      {    1.6224030976428921e+01,     4.8753091698412791e+04,    -5.3594346515697066e-04,    -2.1183032042634311e-07,     1.0331331063076100e-09, } , 
      {    2.3005331010349197e+00,    -2.7252878453266963e+04,     2.5658223505467754e-04,     8.8080270844826935e-08,    -3.5537327639010085e-10, } , 
      {    5.5563435905641008e+00,    -1.0385946137985984e+04,     1.2853964514061755e-04,     7.8180375476570210e-08,    -3.1493981538556564e-10, } , 
      {    8.8121540800932792e+00,     6.4809861772949898e+03,     4.9705522655755253e-07,     6.8280480108313485e-08,    -2.7450635438103032e-10, } , 
      {    1.2067964569622461e+01,     2.3347918492575962e+04,    -1.2754553468750243e-04,     5.8380584740056761e-08,    -2.3407289337649512e-10, } , 
      {    1.5323775059151640e+01,     4.0214850807856936e+04,    -2.5558812460156242e-04,     4.8480689371800036e-08,    -1.9363943237195983e-10, } , 
      {    1.8579585548680821e+01,     5.7081783123137917e+04,    -3.8363071451562238e-04,     3.8580794003543311e-08,    -1.5320597136742459e-10, } , 
      {    4.6560876732868204e+00,    -1.8924187028541837e+04,     4.0889498569602576e-04,     3.3849138527471333e-07,    -1.5417123540651352e-09, } , 
      {    7.9118981628160006e+00,    -2.0572547132608579e+03,     2.8085239578196579e-04,     3.2859148990645660e-07,    -1.5012788930606001e-09, } , 
      {    1.1167708652345180e+01,     1.4809677602020116e+04,     1.5280980586790581e-04,     3.1869159453819988e-07,    -1.4608454320560648e-09, } , 
      {    1.4423519141874362e+01,     3.1676609917301088e+04,     2.4767215953845825e-05,     3.0879169916994315e-07,    -1.4204119710515295e-09, } , 
      {    1.7679329631403540e+01,     4.8543542232582062e+04,    -1.0327537396021418e-04,     2.9889180380168643e-07,    -1.3799785100469942e-09, } , 
      {    1.0267452735067900e+01,     6.2714367114642682e+03,     4.3316514642331407e-04,     5.7900260433634303e-07,    -2.6876179707356346e-09, } , 
      {    1.3523263224597081e+01,     2.3138369026745240e+04,     3.0512255650925405e-04,     5.6910270896808630e-07,    -2.6471845097310993e-09, } , 
      {    1.6779073714126262e+01,     4.0005301342026214e+04,     1.7707996659519407e-04,     5.5920281359982958e-07,    -2.6067510487265640e-09, } , 
      {    2.0034884203655441e+01,     5.6872233657307188e+04,     4.9037376681134057e-05,     5.4930291823157285e-07,    -2.5663175877220287e-09, } , 
      {    1.2623007307319799e+01,     1.4600128136189394e+04,     5.8547789706466229e-04,     8.2941371876622945e-07,    -3.8739570484106693e-09, } , 
      {    1.5878817796848979e+01,     3.1467060451470366e+04,     4.5743530715060228e-04,     8.1951382339797272e-07,    -3.8335235874061339e-09, } , 
      {    1.9134628286378163e+01,     4.8333992766751340e+04,     3.2939271723654231e-04,     8.0961392802971600e-07,    -3.7930901264015986e-09, } , 
      {    2.2390438775907338e+01,     6.5200925082032314e+04,     2.0135012732248227e-04,     7.9971403266145927e-07,    -3.7526566653970633e-09, } , 
      {    1.8234372369100878e+01,     3.9795751876195493e+04,     6.0974805779195050e-04,     1.0699249378278591e-06,    -5.0198626650811682e-09, } , 
      {    2.1490182858630060e+01,     5.6662684191476466e+04,     4.8170546787789059e-04,     1.0600250424596024e-06,    -4.9794292040766329e-09, } , 
      {    2.0589926941352783e+01,     4.8124443300920619e+04,     7.6206080843329877e-04,     1.3203360522577456e-06,    -6.2062017427562024e-09, } , 
      {    2.3845737430881965e+01,     6.4991375616201585e+04,     6.3401821851923881e-04,     1.3104361568894888e-06,    -6.1657682817516671e-09, } , 
      {   -7.7198536074178996e+00,    -5.1191551840532411e+04,    -1.0683560807557887e-03,    -1.6839809287867288e-06,     7.9754274610924209e-09, } , 
      {   -4.4640431178887177e+00,    -3.4324619525251437e+04,    -1.1963986706698486e-03,    -1.6938808241549855e-06,     8.0158609220969562e-09, } , 
      {   -5.3642990351659989e+00,    -4.2862860415807285e+04,    -9.1604333011444041e-04,    -1.4335698143568424e-06,     6.7890883834173867e-09, } , 
      {   -2.1084885456368170e+00,    -2.5995928100526311e+04,    -1.0440859200285004e-03,    -1.4434697097250991e-06,     6.8295218444219220e-09, } , 
      {   -6.2645549524432793e+00,    -5.1401101306363133e+04,    -6.3568798955903217e-04,    -1.1732588045586992e-06,     5.5623158447378163e-09, } , 
      {   -3.0087444629141000e+00,    -3.4534168991082159e+04,    -7.6373057947309224e-04,    -1.1831586999269560e-06,     5.6027493057423516e-09, } , 
      {    2.4706602661508192e-01,    -1.7667236675801185e+04,    -8.9177316938715220e-04,    -1.1930585952952127e-06,     5.6431827667468869e-09, } , 
      {   -3.9090003801913786e+00,    -4.3072409881638006e+04,    -4.8337523891768395e-04,    -9.2284769012881282e-07,     4.3759767670627820e-09, } , 
      {   -6.5318989066219935e-01,    -2.6205477566357033e+04,    -6.1141782883174396e-04,    -9.3274758549706954e-07,     4.4164102280673174e-09, } , 
      {    2.6026205988669826e+00,    -9.3385452510760588e+03,    -7.3946041874580393e-04,    -9.4264748086532627e-07,     4.4568436890718527e-09, } , 
      {    5.8584310883961610e+00,     7.5283870642049151e+03,    -8.6750300865986400e-04,    -9.5254737623358299e-07,     4.4972771500763880e-09, } , 
      {   -1.5534458079394780e+00,    -3.4743718456912880e+04,    -3.3106248827633567e-04,    -6.7243657569892640e-07,     3.1896376893877482e-09, } , 
      {    1.7023646815897013e+00,    -1.7876786141631907e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    4.9581751711188833e+00,    -1.0098538263509326e+03,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    8.2139856606480617e+00,     1.5857078488930041e+04,    -7.1519025801851572e-04,    -7.0213626180369657e-07,     3.3109380724013537e-09, } , 
      {    1.1469796150177244e+01,     3.2724010804211022e+04,    -8.4323284793257568e-04,    -7.1203615717195330e-07,     3.3513715334058891e-09, } , 
      {   -2.4537017252167592e+00,    -4.3281959347468728e+04,    -5.0707147720927491e-05,    -4.1212556590078325e-07,     1.9628651507081782e-09, } , 
      {    8.0210876431242184e-01,    -2.6415027032187754e+04,    -1.7874973763498745e-04,    -4.2202546126903998e-07,     2.0032986117127136e-09, } , 
      {    4.0579192538416011e+00,    -9.5480947169067804e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    7.3137297433707831e+00,     7.3188375983741935e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.0569540232899961e+01,     2.4185769913655167e+04,    -5.6287750737716745e-04,    -4.5172514737381015e-07,     2.1245989947263195e-09, } , 
      {    1.3825350722429143e+01,     4.1052702228936148e+04,    -6.9092009729122741e-04,    -4.6162504274206688e-07,     2.1650324557308544e-09, } , 
      {   -9.8147152964858986e-02,    -3.4953267922743602e+04,     1.0160560292042076e-04,    -1.6171445147089683e-07,     7.7652607303314379e-10, } , 
      {    3.1576633365643221e+00,    -1.8086335607462628e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    6.4134738260935009e+00,    -1.2194032921816543e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.6692843156226829e+00,     1.5647529023099320e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.2925094805151861e+01,     3.2514461338380293e+04,    -4.1056475673581917e-04,    -2.0131403294392373e-07,     9.3825991705128483e-10, } , 
      {    1.6180905294681043e+01,     4.9381393653661275e+04,    -5.3860734664987913e-04,    -2.1121392831218046e-07,     9.7869337805582015e-10, } , 
      {    2.2574074192870413e+00,    -2.6624576498018479e+04,     2.5391835356176900e-04,     8.8696662958989601e-08,    -4.0981300464189066e-10, } , 
      {    5.5132179088162223e+00,    -9.7576441827375020e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    8.7690283983454016e+00,     7.1092881325434719e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2024838887874584e+01,     2.3976220447824446e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5280649377403762e+01,     4.0843152763105420e+04,    -2.5825200609447095e-04,     4.9097081485962703e-08,    -2.4807916062374962e-10, } , 
      {    1.8536459866932944e+01,     5.7710085078386401e+04,    -3.8629459600853091e-04,     3.9197186117705978e-08,    -2.0764569961921439e-10, } , 
      {    4.6129619915389419e+00,    -1.8295885073293353e+04,     4.0623110420311723e-04,     3.3910777738887601e-07,    -1.5961520823169251e-09, } , 
      {    7.8687724810681221e+00,    -1.4289527580123759e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {    1.1124582970597302e+01,     1.5437979557268598e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.4380393460126484e+01,     3.2304911872549572e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.7636203949655663e+01,     4.9171844187830546e+04,    -1.0593925545312270e-04,     2.9950819591584911e-07,    -1.4344182382987841e-09, } , 
      {    2.0892014439184845e+01,     6.6038776503111527e+04,    -2.3398184536718266e-04,     2.8960830054759239e-07,    -1.3939847772942488e-09, } , 
      {    6.9685165637908417e+00,    -9.9671936485682272e+03,     5.5854385484446550e-04,     5.8951889181876243e-07,    -2.7824911599919596e-09, } , 
      {    1.0224327053320023e+01,     6.8997386667127503e+03,     4.3050126493040554e-04,     5.7961899645050571e-07,    -2.7420576989874243e-09, } , 
      {    1.3480137542849203e+01,     2.3766670981993724e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.6735948032378385e+01,     4.0633603297274698e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.9991758521907563e+01,     5.7500535612555672e+04,     4.6373495188225546e-05,     5.4991931034573553e-07,    -2.6207573159738183e-09, } , 
      {    2.3247569011436745e+01,     7.4367467927836653e+04,    -8.1669094725834415e-05,     5.4001941497747881e-07,    -2.5803238549692834e-09, } , 
      {    1.2579881625571922e+01,     1.5228430091437876e+04,     5.8281401557175381e-04,     8.3003011088039213e-07,    -3.9283967766624593e-09, } , 
      {    1.5835692115101102e+01,     3.2095362406718850e+04,     4.5477142565769374e-04,     8.2013021551213540e-07,    -3.8879633156579240e-09, } , 
      {    1.9091502604630286e+01,     4.8962294721999824e+04,     3.2672883574363378e-04,     8.1023032014387868e-07,    -3.8475298546533887e-09, } , 
      {    2.2347313094159460e+01,     6.5829227037280798e+04,     1.9868624582957377e-04,     8.0033042477562195e-07,    -3.8070963936488534e-09, } , 
      {    1.4935436197823822e+01,     2.3557121516163003e+04,     7.3512676621310209e-04,     1.0804412253102785e-06,    -5.1147358543374936e-09, } , 
      {    1.8191246687353001e+01,     4.0424053831443976e+04,     6.0708417629904202e-04,     1.0705413299420218e-06,    -5.0743023933329583e-09, } , 
      {    2.1447057176882183e+01,     5.7290986146724950e+04,     4.7904158638498206e-04,     1.0606414345737651e-06,    -5.0338689323284230e-09, } , 
      {    2.4702867666411365e+01,     7.4157918462005924e+04,     3.5099899647092204e-04,     1.0507415392055084e-06,    -4.9934354713238876e-09, } , 
      {    2.0546801259604905e+01,     4.8752745256169103e+04,     7.5939692694039029e-04,     1.3209524443719082e-06,    -6.2606414710079925e-09, } , 
      {    2.3802611749134087e+01,     6.5619677571450069e+04,     6.3135433702633033e-04,     1.3110525490036515e-06,    -6.2202080100034572e-09, } , 
      {    2.2902355831856802e+01,     5.7081436680894229e+04,     9.1170967758173846e-04,     1.5713635588017947e-06,    -7.4469805486830276e-09, } , 
      {    2.6158166321385984e+01,     7.3948368996175210e+04,     7.8366708766767850e-04,     1.5614636634335379e-06,    -7.4065470876784923e-09, } , 
      {   -5.4074247169138774e+00,    -4.2234558460558801e+04,    -9.1870721160734888e-04,    -1.4329534222426797e-06,     6.7346486551655966e-09, } , 
      {   -3.0518701446619785e+00,    -3.3905867035833675e+04,    -7.6639446096600072e-04,    -1.1825423078127933e-06,     5.5483095774905615e-09, } , 
      {    2.0394034486720369e-01,    -1.7038934720552701e+04,    -8.9443705088006068e-04,    -1.1924422031810500e-06,     5.5887430384950968e-09, } , 
      {   -6.9631557241007758e-01,    -2.5577175611108549e+04,    -6.1408171032465244e-04,    -9.3213119338290686e-07,     4.3619704998155273e-09, } , 
      {    2.5594949171191042e+00,    -8.7102432958275767e+03,    -7.4212430023871240e-04,    -9.4203108875116359e-07,     4.4024039608200626e-09, } , 
      {   -1.5965714896873562e+00,    -3.4115416501664396e+04,    -3.3372636976924420e-04,    -6.7182018358476372e-07,     3.1351979611359586e-09, } , 
      {    1.6592389998418231e+00,    -1.7248484186383423e+04,    -4.6176895968330422e-04,    -6.8172007895302044e-07,     3.1756314221404939e-09, } , 
      {    4.9150494893710048e+00,    -3.8155187110245038e+02,    -5.8981154959736413e-04,    -6.9161997432127717e-07,     3.2160648831450288e-09, } , 
      {    7.5898308256454361e-01,    -2.5786725076939270e+04,    -1.8141361912789596e-04,    -4.2140906915487730e-07,     1.9488588834609239e-09, } , 
      {    4.0147935720937227e+00,    -8.9197927616582983e+03,    -3.0945620904195600e-04,    -4.3130896452313402e-07,     1.9892923444654592e-09, } , 
      {    7.2706040616229046e+00,     7.9471395536226755e+03,    -4.3749879895601596e-04,    -4.4120885989139075e-07,     2.0297258054699945e-09, } , 
      {    1.0526414551152083e+01,     2.4814071868903651e+04,    -5.6554138887007592e-04,    -4.5110875525964747e-07,     2.0701592664745298e-09, } , 
      {    3.1145376548164441e+00,    -1.7458033652214144e+04,    -2.9100868486547716e-05,    -1.7099795472499090e-07,     7.6251980578588923e-10, } , 
      {    6.3703481443456225e+00,    -5.9110133693317198e+02,    -1.5714345840060770e-04,    -1.8089785009324763e-07,     8.0295326679042455e-10, } , 
      {    9.6261586338748053e+00,     1.6275830978347802e+04,    -2.8518604831466774e-04,    -1.9079774546150435e-07,     8.4338672779495976e-10, } , 
      {    1.2881969123403984e+01,     3.3142763293628777e+04,    -4.1322863822872770e-04,    -2.0069764082976108e-07,     8.8382018879949507e-10, } , 
      {    2.2142817375391628e+00,    -2.5996274542769996e+04,     2.5125447206886047e-04,     8.9313055073152268e-08,    -4.6425273289368048e-10, } , 
      {    5.4700922270683439e+00,    -9.1293422274890199e+03,     1.2321188215480054e-04,     7.9413159704895543e-08,    -4.2381927188914527e-10, } , 
      {    8.7259027165975240e+00,     7.7375900877919539e+03,    -4.8307077592594708e-06,     6.9513264336638819e-08,    -3.8338581088460996e-10, } , 
      {    1.1981713206126706e+01,     2.4604522403072930e+04,    -1.3287329767331944e-04,     5.9613368968382094e-08,    -3.4295234988007475e-10, } , 
      {    1.5237523695655884e+01,     4.1471454718353903e+04,    -2.6091588758737948e-04,     4.9713473600125369e-08,    -3.0251888887553944e-10, } , 
      {    4.5698363097910635e+00,    -1.7667583118044869e+04,     4.0356722271020869e-04,     3.3972416950303869e-07,    -1.6505918105687150e-09, } , 
      {    7.8256467993202437e+00,    -8.0065080276389358e+02,     2.7552463279614873e-04,     3.2982427413478196e-07,    -1.6101583495641799e-09, } , 
      {    1.1081457288849425e+01,     1.6066281512517080e+04,     1.4748204288208880e-04,     3.1992437876652524e-07,    -1.5697248885596446e-09, } , 
      {    1.4337267778378607e+01,     3.2933213827798056e+04,     1.9439452968028803e-05,     3.1002448339826851e-07,    -1.5292914275551093e-09, } , 
      {    1.7593078267907785e+01,     4.9800146143079030e+04,    -1.0860313694603122e-04,     3.0012458803001179e-07,    -1.4888579665505739e-09, } , 
      {    1.0181201371572145e+01,     7.5280406219612323e+03,     4.2783738343749701e-04,     5.8023538856466839e-07,    -2.7964974272392139e-09, } , 
      {    1.3437011861101325e+01,     2.4394972937242208e+04,     2.9979479352343699e-04,     5.7033549319641166e-07,    -2.7560639662346786e-09, } , 
      {    1.6692822350630507e+01,     4.1261905252523182e+04,     1.7175220360937706e-04,     5.6043559782815494e-07,    -2.7156305052301433e-09, } , 
      {    1.9948632840159686e+01,     5.8128837567804156e+04,     4.3709613695317035e-05,     5.5053570245989821e-07,    -2.6751970442256080e-09, } , 
      {    1.2536755943824044e+01,     1.5856732046686358e+04,     5.8015013407884534e-04,     8.3064650299455481e-07,    -3.9828365049142494e-09, } , 
      {    1.5792566433353224e+01,     3.2723664361967334e+04,     4.5210754416478521e-04,     8.2074660762629808e-07,    -3.9424030439097141e-09, } , 
      {    1.9048376922882408e+01,     4.9590596677248308e+04,     3.2406495425072525e-04,     8.1084671225804136e-07,    -3.9019695829051788e-09, } , 
      {    2.2304187412411583e+01,     6.6457528992529275e+04,     1.9602236433666526e-04,     8.0094681688978463e-07,    -3.8615361219006435e-09, } , 
      {    1.8148121005605123e+01,     4.1052355786692460e+04,     6.0442029480613354e-04,     1.0711577220561845e-06,    -5.1287421215847483e-09, } , 
      {    2.1403931495134305e+01,     5.7919288101973434e+04,     4.7637770489207353e-04,     1.0612578266879278e-06,    -5.0883086605802130e-09, } , 
      {    2.0503675577857027e+01,     4.9381047211417586e+04,     7.5673304544748182e-04,     1.3215688364860709e-06,    -6.3150811992597826e-09, } , 
      {    2.3759486067386209e+01,     6.6247979526698546e+04,     6.2869045553342186e-04,     1.3116689411178142e-06,    -6.2746477382552473e-09, } , 
      {   -7.3944125415795581e-01,    -2.4948873655860069e+04,    -6.1674559181756103e-04,    -9.3151480126874418e-07,     4.3075307715637380e-09, } , 
      {    2.5163692353712261e+00,    -8.0819413405790947e+03,    -7.4478818173162099e-04,    -9.4141469663700091e-07,     4.3479642325682733e-09, } , 
      {    1.6161133180939449e+00,    -1.6620182231134942e+04,    -4.6443284117621270e-04,    -6.8110368683885776e-07,     3.1211916938887038e-09, } , 
      {    4.8719238076231264e+00,     2.4675008414603190e+02,    -5.9247543109027271e-04,    -6.9100358220711449e-07,     3.1616251548932387e-09, } , 
      {    7.1585740081666538e-01,    -2.5158423121690790e+04,    -1.8407750062080449e-04,    -4.2079267704071467e-07,     1.8944191552091338e-09, } , 
      {    3.9716678903458447e+00,    -8.2914908064098163e+03,    -3.1212009053486448e-04,    -4.3069257240897140e-07,     1.9348526162136691e-09, } , 
      {    7.2274783798750271e+00,     8.5754415088711576e+03,    -4.4016268044892444e-04,    -4.4059246777722812e-07,     1.9752860772182044e-09, } , 
      {    1.0483288869404204e+01,     2.5442373824152131e+04,    -5.6820527036298451e-04,    -4.5049236314548485e-07,     2.0157195382227398e-09, } , 
      {    3.0714119730685656e+00,    -1.6829731696965664e+04,    -3.1764749979456227e-05,    -1.7038156261082822e-07,     7.0808007753409937e-10, } , 
      {    6.3272224625977440e+00,     3.7200618315310294e+01,    -1.5980733989351623e-04,    -1.8028145797908495e-07,     7.4851353853863468e-10, } , 
      {    9.5830329521269260e+00,     1.6904132933596284e+04,    -2.8784992980757622e-04,    -1.9018135334734167e-07,     7.8894699954316989e-10, } , 
      {    1.2838843441656104e+01,     3.3771065248877261e+04,    -4.1589251972163618e-04,    -2.0008124871559840e-07,     8.2938046054770520e-10, } , 
      {    5.4269665453204663e+00,    -8.5010402722405379e+03,     1.2054800066189202e-04,     8.0029551819058210e-08,    -4.7825900014093509e-10, } , 
      {    8.6827770348496447e+00,     8.3658920430404360e+03,    -7.4945892521679825e-06,     7.0129656450801486e-08,    -4.3782553913639972e-10, } , 
      {    1.1938587524378827e+01,     2.5232824358321410e+04,    -1.3553717916622797e-04,     6.0229761082544761e-08,    -3.9739207813186451e-10, } , 
      {    1.5194398013908005e+01,     4.2099756673602387e+04,    -2.6357976908028796e-04,     5.0329865714288030e-08,    -3.5695861712732920e-10, } , 
      {    7.7825211175723652e+00,    -1.7234884751541131e+02,     2.7286075130324026e-04,     3.3044066624894459e-07,    -1.6645980778159695e-09, } , 
      {    1.1038331607101545e+01,     1.6694583467765562e+04,     1.4481816138918027e-04,     3.2054077088068787e-07,    -1.6241646168114342e-09, } , 
      {    1.4294142096630727e+01,     3.3561515783046540e+04,     1.6775571475120292e-05,     3.1064087551243114e-07,    -1.5837311558068989e-09, } , 
      {    1.3393886179353446e+01,     2.5023274892490688e+04,     2.9713091203052852e-04,     5.7095188531057434e-07,    -2.8105036944864687e-09, } , 
      {    1.6649696668882630e+01,     4.1890207207771666e+04,     1.6908832211646853e-04,     5.6105198994231762e-07,    -2.7700702334819334e-09, } , 
      {    3.9285422085979667e+00,    -7.6631888511613333e+03,    -3.1478397202777301e-04,    -4.3007618029480872e-07,     1.8804128879618795e-09, } , 
      {    7.1843526981271486e+00,     9.2037434641196396e+03,    -4.4282656194183297e-04,    -4.3997607566306544e-07,     1.9208463489664148e-09, } , 
      {    6.2840967808498664e+00,     6.6550257356379257e+02,    -1.6247122138642473e-04,    -1.7966506586492229e-07,     6.9407381028684491e-10, } , 
      {    9.5399072703790484e+00,     1.7532434888844768e+04,    -2.9051381130048475e-04,    -1.8956496123317902e-07,     7.3450727129138012e-10, } , 
      {    8.6396513531017671e+00,     8.9941939982889180e+03,    -1.0158470745076495e-05,     7.0746048564964139e-08,    -4.9226526738818954e-10, } , 
      {    1.1895461842630949e+01,     2.5861126313569894e+04,    -1.3820106065913647e-04,     6.0846153196707414e-08,    -4.5183180638365433e-10, } , 
      {    1.0995205925353668e+01,     1.7322885423014046e+04,     1.4215427989627176e-04,     3.2115716299485055e-07,    -1.6786043450632241e-09, } , 
      {    1.1741317692892133e+01,     4.6670679871648763e+03,    -1.7958199304066413e-04,    -1.5138995628072435e-07,     8.6634750373230742e-10, } , 
      {    1.4997128182421312e+01,     2.1534000302445849e+04,    -3.0762458295472415e-04,    -1.6128985164898108e-07,     9.0678096473684273e-10, } , 
      {    1.4096872265144034e+01,     1.2995759411890003e+04,    -2.7269242399315900e-05,     9.9021158149162066e-08,    -3.1999157394272704e-10, } , 
      {    6.9870828666404563e+00,    -1.1362012907036893e+04,    -4.8687137581626916e-04,    -6.5159579302633460e-07,     3.1845859308305869e-09, } , 
      {    9.3426374388923552e+00,    -3.0333214823117673e+03,    -3.3455862517492094e-04,    -4.0118467859644812e-07,     1.9982468531555522e-09, } , 
      {    1.2598447928421534e+01,     1.3833610832969207e+04,    -4.6260121508898090e-04,    -4.1108457396470485e-07,     2.0386803141600875e-09, } , 
      {    1.1698192011144256e+01,     5.2953699424133592e+03,    -1.8224587453357264e-04,    -1.5077356416656170e-07,     8.1190777548051765e-10, } , 
      {    1.4954002500673434e+01,     2.2162302257694333e+04,    -3.1028846444763268e-04,    -1.6067345953481843e-07,     8.5234123648505296e-10, } , 
      {    1.0797936093866975e+01,    -3.2428709481424889e+03,     9.8109466021835588e-05,     1.0953744563158144e-07,    -4.1486476319905206e-10, } , 
      {    1.4053746583396157e+01,     1.3624061367138485e+04,    -2.9933123892224414e-05,     9.9637550263324720e-08,    -3.7443130219451685e-10, } , 
      {    1.7309557072925333e+01,     3.0490993682419459e+04,    -1.5797571380628438e-04,     8.9737654895067995e-08,    -3.3399784118998154e-10, } , 
      {    1.6409301155648055e+01,     2.1952752791863611e+04,     1.2237962674912383e-04,     3.5004866469321114e-07,    -1.5607703798695514e-09, } , 
      {    1.8764855727899956e+01,     3.0281444216588738e+04,     2.7469237739047205e-04,     6.0045977912309751e-07,    -2.7471094575445856e-09, } , 
      {    4.5884026126406772e+00,    -1.9062402376513539e+04,    -6.4184800795052596e-04,    -9.0139051534205834e-07,     4.3164852802538310e-09, } , 
      {    6.9439571848925778e+00,    -1.0733710951788411e+04,    -4.8953525730917769e-04,    -6.5097940091217192e-07,     3.1301462025787968e-09, } , 
      {    9.2995117571444759e+00,    -2.4050195270632848e+03,    -3.3722250666782941e-04,    -4.0056828648228549e-07,     1.9438071249037621e-09, } , 
      {    1.2555322246673654e+01,     1.4461912788217689e+04,    -4.6526509658188938e-04,    -4.1046818185054222e-07,     1.9842405859082974e-09, } , 
      {    8.3992558398671946e+00,    -1.0943260417619133e+04,    -5.6867166112421171e-05,    -1.4025727668414230e-07,     7.1703458622419257e-10, } , 
      {    1.1655066329396377e+01,     5.9236718976618413e+03,    -1.8490975602648117e-04,    -1.5015717205239902e-07,     7.5746804722872778e-10, } , 
      {    1.4910876818925555e+01,     2.2790604212942813e+04,    -3.1295234594054115e-04,    -1.6005706742065575e-07,     7.9790150823326309e-10, } , 
      {    1.8166687308454737e+01,     3.9657536528223784e+04,    -4.4099493585460112e-04,    -1.6995696278891247e-07,     8.3833496923779830e-10, } , 
      {    1.0754810412119095e+01,    -2.6145689928940064e+03,     9.5445584528927084e-05,     1.1015383774574411e-07,    -4.6930449145084188e-10, } , 
      {    1.4010620901648277e+01,     1.4252363322386967e+04,    -3.2597005385132925e-05,     1.0025394237748739e-07,    -4.2887103044630662e-10, } , 
      {    1.7266431391177456e+01,     3.1119295637667939e+04,    -1.6063959529919291e-04,     9.0354047009230662e-08,    -3.8843756944177131e-10, } , 
      {    2.0522241880706638e+01,     4.7986227952948910e+04,    -2.8868218521325290e-04,     8.0454151640973937e-08,    -3.4800410843723610e-10, } , 
      {    1.6366175473900178e+01,     2.2581054747112092e+04,     1.1971574525621533e-04,     3.5066505680737377e-07,    -1.6152101081213410e-09, } , 
      {    1.9621985963429356e+01,     3.9447987062393069e+04,    -8.3268446578446521e-06,     3.4076516143911704e-07,    -1.5747766471168057e-09, } , 
      {    1.8721730046152079e+01,     3.0909746171837218e+04,     2.7202849589756358e-04,     6.0107617123726019e-07,    -2.8015491857963757e-09, } , 
      {    2.1977540535681257e+01,     4.7776678487118195e+04,     1.4398590598350359e-04,     5.9117627586900346e-07,    -2.7611157247918404e-09, } , 
      {    2.1077284618403976e+01,     3.9238437596562340e+04,     4.2434124653891180e-04,     8.5148728566714661e-07,    -3.9878882634714103e-09, } , 
      {    2.4333095107933158e+01,     5.6105369911843321e+04,     2.9629865662485184e-04,     8.4158739029888988e-07,    -3.9474548024668750e-09, } , 
      {    4.5452769308927987e+00,    -1.8434100421265055e+04,    -6.4451188944343444e-04,    -9.0077412322789566e-07,     4.2620455520020410e-09, } , 
      {    7.8010874204219771e+00,    -1.5671681059840812e+03,    -7.7255447935749440e-04,    -9.1067401859615238e-07,     4.3024790130065763e-09, } , 
      {    6.9008315031446994e+00,    -1.0105408996539929e+04,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {    1.0156641992673878e+01,     6.7615233187410449e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {    1.3412452482203060e+01,     2.3628455634022015e+04,    -7.4828431863020609e-04,    -6.7016279953452268e-07,     3.1565733963360778e-09, } , 
      {    6.0005755858674172e+00,    -1.8643649887095777e+04,    -2.1184379824667793e-04,    -3.9005199899986609e-07,     1.8489339356474372e-09, } , 
      {    9.2563860753965983e+00,    -1.7767175718148028e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.2512196564925777e+01,     1.5090214743466171e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.5768007054454959e+01,     3.1957147058747141e+04,    -5.9597156798885792e-04,    -4.1975168510463626e-07,     1.9702343186610431e-09, } , 
      {    8.3561301581193170e+00,    -1.0314958462370651e+04,    -5.9531047605329682e-05,    -1.3964088456997964e-07,     6.6259485797240281e-10, } , 
      {    1.1611940647648499e+01,     6.5519738529103233e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.4867751137177677e+01,     2.3418906168191297e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.8123561626706859e+01,     4.0285838483472267e+04,    -4.4365881734750965e-04,    -1.6934057067474982e-07,     7.8389524098600854e-10, } , 
      {    1.0711684730371218e+01,    -1.9862670376455244e+03,     9.2781703036018566e-05,     1.1077022985990678e-07,    -5.2374421970263164e-10, } , 
      {    1.3967495219900400e+01,     1.4880665277635449e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.7223305709429578e+01,     3.1747597592916423e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    2.0479116198958760e+01,     4.8614529908197394e+04,    -2.9134606670616143e-04,     8.1070543755136604e-08,    -4.0244383668902591e-10, } , 
      {    1.3067239302623118e+01,     6.3424243870796017e+03,     2.4509445367736679e-04,     3.6118134428979317e-07,    -1.7100832973776660e-09, } , 
      {    1.6323049792152300e+01,     2.3209356702360576e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    1.9578860281681479e+01,     4.0076289017641553e+04,    -1.0990726150753163e-05,     3.4138155355327972e-07,    -1.6292163753685956e-09, } , 
      {    2.2834670771210661e+01,     5.6943221332922520e+04,    -1.3903331606481318e-04,     3.3148165818502300e-07,    -1.5887829143640605e-09, } , 
      {    1.8678604364404201e+01,     3.1538048127085702e+04,     2.6936461440465505e-04,     6.0169256335142287e-07,    -2.8559889140481653e-09, } , 
      {    2.1934414853933379e+01,     4.8404980442366679e+04,     1.4132202449059509e-04,     5.9179266798316614e-07,    -2.8155554530436300e-09, } , 
      {    2.1034158936656098e+01,     3.9866739551810824e+04,     4.2167736504600327e-04,     8.5210367778130929e-07,    -4.0423279917232004e-09, } , 
      {    2.4289969426185280e+01,     5.6733671867091805e+04,     2.9363477513194331e-04,     8.4220378241305256e-07,    -4.0018945307186651e-09, } , 
      {    2.3389713508908002e+01,     4.8195430976535950e+04,     5.7399011568735154e-04,     1.1025147922111957e-06,    -5.2286670693982347e-09, } , 
      {    2.6645523998437177e+01,     6.5062363291816931e+04,     4.4594752577329158e-04,     1.0926148968429390e-06,    -5.1882336083936993e-09, } , 
      {    4.5021512491449203e+00,    -1.7805798466016571e+04,    -6.4717577093634292e-04,    -9.0015773111373298e-07,     4.2076058237502509e-09, } , 
      {    7.7579617386740987e+00,    -9.3886615073559892e+02,    -7.7521836085040288e-04,    -9.1005762648198970e-07,     4.2480392847547862e-09, } , 
      {    6.8577058213968209e+00,    -9.4771070412914469e+03,    -4.9486302029499464e-04,    -6.4974661668384656e-07,     3.0212667460752175e-09, } , 
      {    1.0113516310926000e+01,     7.3898252739895270e+03,    -6.2290561020905460e-04,    -6.5964651205210328e-07,     3.0617002070797528e-09, } , 
      {    9.2132603936487207e+00,    -1.1484156165663205e+03,    -3.4255026965364648e-04,    -3.9933550225396013e-07,     1.8349276684001826e-09, } , 
      {    1.2469070883177899e+01,     1.5718516698714653e+04,    -4.7059285956770644e-04,    -4.0923539762221686e-07,     1.8753611294047181e-09, } , 
      {    8.3130044763714395e+00,    -9.6866565071221685e+03,    -6.2194929098238199e-05,    -1.3902449245581699e-07,     6.0815512972061304e-10, } , 
      {    1.1568814965900621e+01,     7.1802758081588054e+03,    -1.9023751901229817e-04,    -1.4892438782407371e-07,     6.4858859072514825e-10, } , 
      {    1.4824625455429800e+01,     2.4047208123439781e+04,    -3.1828010892635822e-04,    -1.5882428319233044e-07,     6.8902205172968356e-10, } , 
      {    1.8080435944958982e+01,     4.0914140438720751e+04,    -4.4632269884041818e-04,    -1.6872417856058716e-07,     7.2945551273421877e-10, } , 
      {    1.0668559048623340e+01,    -1.3579650823970421e+03,     9.0117821543110049e-05,     1.1138662197406944e-07,    -5.7818394795442141e-10, } , 
      {    1.3924369538152522e+01,     1.5508967232883932e+04,    -3.7924768370949947e-05,     1.0148672660581272e-07,    -5.3775048694988620e-10, } , 
      {    1.7180180027681700e+01,     3.2375899548164907e+04,    -1.6596735828500991e-04,     9.1586831237555996e-08,    -4.9731702594535089e-10, } , 
      {    2.0435990517210882e+01,     4.9242831863445877e+04,    -2.9400994819906996e-04,     8.1686935869299271e-08,    -4.5688356494081573e-10, } , 
      {    1.6279924110404423e+01,     2.3837658657609059e+04,     1.1438798227039829e-04,     3.5189784103569913e-07,    -1.7240895646249208e-09, } , 
      {    1.9535734599933601e+01,     4.0704590972890037e+04,    -1.3654607643661674e-05,     3.4199794566744240e-07,    -1.6836561036203854e-09, } , 
      {    1.8635478682656323e+01,     3.2166350082334186e+04,     2.6670073291174652e-04,     6.0230895546558555e-07,    -2.9104286422999550e-09, } , 
      {    2.1891289172185502e+01,     4.9033282397615163e+04,     1.3865814299768658e-04,     5.9240906009732882e-07,    -2.8699951812954197e-09, } , 
      {    2.0991033254908221e+01,     4.0495041507059308e+04,     4.1901348355309474e-04,     8.5272006989547197e-07,    -4.0967677199749905e-09, } , 
      {    2.4246843744437403e+01,     5.7361973822340289e+04,     2.9097089363903477e-04,     8.4282017452721524e-07,    -4.0563342589704552e-09, } , 
      {    9.1701347119008414e+00,    -5.2011366131783825e+02,    -3.4521415114655495e-04,    -3.9871911013979751e-07,     1.7804879401483929e-09, } , 
      {    1.2425945201430020e+01,     1.6346818653963135e+04,    -4.7325674106061491e-04,    -4.0861900550805423e-07,     1.8209214011529283e-09, } , 
      {    1.1525689284152742e+01,     7.8085777634072874e+03,    -1.9290140050520671e-04,    -1.4830799570991103e-07,     5.9414886247335838e-10, } , 
      {    1.4781499773681920e+01,     2.4675510078688261e+04,    -3.2094399041926669e-04,    -1.5820789107816776e-07,     6.3458232347789370e-10, } , 
      {    1.3881243856404643e+01,     1.6137269188132414e+04,    -4.0588649863858458e-05,     1.0210311871997539e-07,    -5.9219021520167607e-10, } , 
      {    1.7137054345933823e+01,     3.3004201503413387e+04,    -1.6863123977791845e-04,     9.2203223351718662e-08,    -5.5175675419714076e-10, } , 
      {    1.6236798428656545e+01,     2.4465960612857540e+04,     1.1172410077748979e-04,     3.5251423314986175e-07,    -1.7785292928767104e-09, } , 
      {    1.9492608918185724e+01,     4.1332892928138521e+04,    -1.6318489136570185e-05,     3.4261433778160503e-07,    -1.7380958318721751e-09, } , 
      {    1.8592353000908446e+01,     3.2794652037582666e+04,     2.6403685141883804e-04,     6.0292534757974823e-07,    -2.9648683705517451e-09, } , 
      {    8.2267531128756826e+00,    -8.4300525966252044e+03,    -6.7522692084055221e-05,    -1.3779170822749166e-07,     4.9927567321703341e-10, } , 
      {    1.1482563602404865e+01,     8.4368797186557695e+03,    -1.9556528199811521e-04,    -1.4769160359574838e-07,     5.3970913422156862e-10, } , 
      {    1.4738374091934043e+01,     2.5303812033936745e+04,    -3.2360787191217523e-04,    -1.5759149896400510e-07,     5.8014259522610393e-10, } , 
      {    1.7883166113472289e+01,     2.0348384067564220e+04,    -4.9036751271485442e-04,    -3.8034389592385632e-07,     1.9931950945983904e-09, } , 
      {    1.6982910196195007e+01,     1.1810143177008376e+04,    -2.1001217215944610e-04,    -1.2003288612571315e-07,     7.6642255591882089e-10, } , 
      {    2.0238720685724189e+01,     2.8677075492289347e+04,    -3.3805476207350609e-04,    -1.2993278149396987e-07,     8.0685601692335620e-10, } , 
      {    1.9338464768446908e+01,     2.0138834601733502e+04,    -5.7699421518097869e-05,     1.3037822830417327e-07,    -4.1991652175621357e-10, } , 
      {    2.2594275257976090e+01,     3.7005766917014473e+04,    -1.8574201143215784e-04,     1.2047833293591655e-07,    -3.7948306075167825e-10, } , 
      {    2.1694019340698809e+01,     2.8467526026458629e+04,     9.4613329123250379e-05,     3.8078934273405967e-07,    -1.6062555994312478e-09, } , 
      {    1.4584229942195227e+01,     4.1097537075317314e+03,    -3.6498880429370288e-04,    -3.6982760844143691e-07,     1.8983219053420655e-09, } , 
      {    1.7840040431724411e+01,     2.0976686022812701e+04,    -4.9303139420776290e-04,    -3.7972750380969364e-07,     1.9387553663466008e-09, } , 
      {    1.6939784514447130e+01,     1.2438445132256858e+04,    -2.1267605365235461e-04,    -1.1941649401155047e-07,     7.1198282766703112e-10, } , 
      {    2.0195595003976312e+01,     2.9305377447537827e+04,    -3.4071864356641462e-04,    -1.2931638937980719e-07,     7.5241628867156643e-10, } , 
      {    1.6039528597169852e+01,     3.9002042417010098e+03,     6.7679286903053629e-05,     1.4089451578659268e-07,    -5.1478971101253854e-10, } , 
      {    1.9295339086699030e+01,     2.0767136556981983e+04,    -6.0363303011006373e-05,     1.3099462041833595e-07,    -4.7435625000800333e-10, } , 
      {    2.2551149576228212e+01,     3.7634068872262957e+04,    -1.8840589292506635e-04,     1.2109472505007923e-07,    -4.3392278900346802e-10, } , 
      {    2.1650893658950931e+01,     2.9095827981707109e+04,     9.1949447630341875e-05,     3.8140573484822235e-07,    -1.6606953276830377e-09, } , 
      {    2.4906704148480113e+01,     4.5962760296988083e+04,    -3.6093142283718100e-05,     3.7150583947996562e-07,    -1.6202618666785024e-09, } , 
      {    2.4006448231202832e+01,     3.7424519406432242e+04,     2.4426219827169011e-04,     6.3181684927810872e-07,    -2.8470344053580723e-09, } , 
      {    2.7262258720732014e+01,     5.4291451721713209e+04,     1.1621960835763015e-04,     6.2191695390985199e-07,    -2.8066009443535370e-09, } , 
      {    9.8299951159435501e+00,    -1.1919327186670038e+04,    -6.7227818706930786e-04,    -8.7003344518704713e-07,     4.2165603324403447e-09, } , 
      {    1.2185549688195451e+01,    -3.5906357619449122e+03,    -5.1996543642795958e-04,    -6.1962233075716071e-07,     3.0302212547653105e-09, } , 
      {    1.5441360177724633e+01,     1.3276296553336058e+04,    -6.4800802634201954e-04,    -6.2952222612541743e-07,     3.0706547157698458e-09, } , 
      {    1.1285293770918171e+01,    -1.2128876652500760e+04,    -2.3961009587255137e-04,    -3.5931132095901751e-07,     1.8034487160857405e-09, } , 
      {    1.4541104260447350e+01,     4.7380556627802143e+03,    -3.6765268578661141e-04,    -3.6921121632727423e-07,     1.8438821770902756e-09, } , 
      {    1.7796914749976530e+01,     2.1604987978061185e+04,    -4.9569527570067138e-04,    -3.7911111169553096e-07,     1.8843156380948111e-09, } , 
      {    2.1052725239505708e+01,     3.8471920293342162e+04,    -6.2373786561473134e-04,    -3.8901100706378768e-07,     1.9247490990993464e-09, } , 
      {    1.3640848343170072e+01,    -3.8001852277756338e+03,    -8.7297345231203137e-05,    -1.0890020652913109e-07,     6.1710963841070615e-10, } , 
      {    1.6896658832699249e+01,     1.3066747087505340e+04,    -2.1533993514526311e-04,    -1.1880010189738781e-07,     6.5754309941524136e-10, } , 
      {    2.0152469322228431e+01,     2.9933679402786311e+04,    -3.4338252505932315e-04,    -1.2869999726564454e-07,     6.9797656041977667e-10, } , 
      {    2.3408279811757609e+01,     4.6800611718067288e+04,    -4.7142511497338312e-04,    -1.3859989263390126e-07,     7.3841002142431188e-10, } , 
      {    1.5996402915421973e+01,     4.5285061969494927e+03,     6.5015405410145111e-05,     1.4151090790075533e-07,    -5.6922943926432831e-10, } , 
      {    1.9252213404951149e+01,     2.1395438512230467e+04,    -6.3027184503914891e-05,     1.3161101253249861e-07,    -5.2879597825979310e-10, } , 
      {    2.2508023894480331e+01,     3.8262370827511433e+04,    -1.9106977441797485e-04,     1.2171111716424188e-07,    -4.8836251725525778e-10, } , 
      {    2.5763834384009510e+01,     5.5129303142792414e+04,    -3.1911236433203490e-04,     1.1181122179598516e-07,    -4.4792905625072263e-10, } , 
      {    1.8351957487673872e+01,     1.2857197621674619e+04,     2.1732815605149337e-04,     3.9192202233064175e-07,    -1.7555685169393629e-09, } , 
      {    2.1607767977203050e+01,     2.9724129936955593e+04,     8.9285566137433357e-05,     3.8202212696238503e-07,    -1.7151350559348275e-09, } , 
      {    2.4863578466732232e+01,     4.6591062252236559e+04,    -3.8757023776626618e-05,     3.7212223159412830e-07,    -1.6747015949302922e-09, } , 
      {    2.3963322549454951e+01,     3.8052821361680719e+04,     2.4159831677878160e-04,     6.3243324139227140e-07,    -2.9014741336098620e-09, } , 
      {    2.7219133038984133e+01,     5.4919753676961685e+04,     1.1355572686472163e-04,     6.2253334602401467e-07,    -2.8610406726053267e-09, } , 
      {    2.6318877121706848e+01,     4.6381512786405845e+04,     3.9391106742012980e-04,     8.8284435582215782e-07,    -4.0878132112848967e-09, } , 
      {    2.9574687611236030e+01,     6.3248445101686812e+04,     2.6586847750606984e-04,     8.7294446045390109e-07,    -4.0473797502803613e-09, } , 
      {    5.0757602896918721e+00,    -2.7948408080871810e+04,    -9.7956756984491277e-04,    -1.3702392819326573e-06,     6.5347987595386248e-09, } , 
      {    7.4313148619437710e+00,    -1.9619716656146684e+04,    -8.2725481920356472e-04,    -1.1198281675027709e-06,     5.3484596818635897e-09, } , 
      {    9.7868694341956708e+00,    -1.1291025231421556e+04,    -6.7494206856221644e-04,    -8.6941705307288445e-07,     4.1621206041885555e-09, } , 
      {    1.3042679923724853e+01,     5.5759070838594143e+03,    -8.0298465847627640e-04,    -8.7931694844114117e-07,     4.2025540651930908e-09, } , 
      {    8.8866135169183931e+00,    -1.9829266121977405e+04,    -3.9458672800680810e-04,    -6.0910604327474130e-07,     2.9353480655089855e-09, } , 
      {    1.2142424006447571e+01,    -2.9623338066964297e+03,    -5.2262931792086817e-04,    -6.1900593864299803e-07,     2.9757815265135204e-09, } , 
      {    1.5398234495976753e+01,     1.3904598508584540e+04,    -6.5067190783492813e-04,    -6.2890583401125475e-07,     3.0162149875180557e-09, } , 
      {    1.8654044985505930e+01,     3.0771530823865520e+04,    -7.7871449774898820e-04,    -6.3880572937951148e-07,     3.0566484485225910e-09, } , 
      {    1.1242168089170292e+01,    -1.1500574697252277e+04,    -2.4227397736545990e-04,    -3.5869492884485488e-07,     1.7490089878339508e-09, } , 
      {    1.4497978578699470e+01,     5.3663576180286964e+03,    -3.7031656727951989e-04,    -3.6859482421311161e-07,     1.7894424488384859e-09, } , 
      {    1.7753789068228652e+01,     2.2233289933309665e+04,    -4.9835915719357996e-04,    -3.7849471958136833e-07,     1.8298759098430213e-09, } , 
      {    2.1009599557757831e+01,     3.9100222248590646e+04,    -6.2640174710763992e-04,    -3.8839461494962506e-07,     1.8703093708475564e-09, } , 
      {    1.3597722661422193e+01,    -3.1718832725271513e+03,    -8.9961226724111641e-05,    -1.0828381441496842e-07,     5.6266991015891628e-10, } , 
      {    1.6853533150951371e+01,     1.3695049042753823e+04,    -2.1800381663817164e-04,    -1.1818370978322515e-07,     6.0310337116345149e-10, } , 
      {    2.0109343640480553e+01,     3.0561981358034791e+04,    -3.4604640655223163e-04,    -1.2808360515148186e-07,     6.4353683216798680e-10, } , 
      {    2.3365154130009731e+01,     4.7428913673315772e+04,    -4.7408899646629159e-04,    -1.3798350051973858e-07,     6.8397029317252201e-10, } , 
      {    1.2697466744144911e+01,    -1.1710124163083003e+04,     1.9039411383129656e-04,     1.5202719538317474e-07,    -6.6410262852065349e-10, } , 
      {    1.5953277233674093e+01,     5.1568081521979748e+03,     6.2351523917236607e-05,     1.4212730001491801e-07,    -6.2366916751611817e-10, } , 
      {    1.9209087723203272e+01,     2.2023740467478947e+04,    -6.5691065996823395e-05,     1.3222740464666129e-07,    -5.8323570651158297e-10, } , 
      {    2.2464898212732454e+01,     3.8890672782759917e+04,    -1.9373365591088338e-04,     1.2232750927840456e-07,    -5.4280224550704765e-10, } , 
      {    2.5720708702261632e+01,     5.5757605098040898e+04,    -3.2177624582494337e-04,     1.1242761391014783e-07,    -5.0236878450251244e-10, } , 
      {    1.8308831805925994e+01,     1.3485499576923101e+04,     2.1466427455858484e-04,     3.9253841444480438e-07,    -1.8100082451911525e-09, } , 
      {    2.1564642295455172e+01,     3.0352431892204073e+04,     8.6621684644524853e-05,     3.8263851907654766e-07,    -1.7695747841866172e-09, } , 
      {    2.4820452784984354e+01,     4.7219364207485043e+04,    -4.1420905269535129e-05,     3.7273862370829093e-07,    -1.7291413231820819e-09, } , 
      {    2.8076263274513533e+01,     6.4086296522766024e+04,    -1.6946349518359515e-04,     3.6283872834003421e-07,    -1.6887078621775468e-09, } , 
      {    2.0664386378177895e+01,     2.1814191001648225e+04,     3.6697702519993312e-04,     6.4294952887469080e-07,    -2.9963473228661874e-09, } , 
      {    2.3920196867707073e+01,     3.8681123316929203e+04,     2.3893443528587307e-04,     6.3304963350643408e-07,    -2.9559138618616521e-09, } , 
      {    2.7176007357236255e+01,     5.5548055632210169e+04,     1.1089184537181313e-04,     6.2314973813817735e-07,    -2.9154804008571168e-09, } , 
      {    3.0431817846765433e+01,     7.2414987947491158e+04,    -1.7150744542246897e-05,     6.1324984276992063e-07,    -2.8750469398525815e-09, } , 
      {    2.3019940950429795e+01,     3.0142882426373351e+04,     5.1928977584128134e-04,     8.9336064330457722e-07,    -4.1826864005412212e-09, } , 
      {    2.6275751439958970e+01,     4.7009814741654329e+04,     3.9124718592722132e-04,     8.8346074793632050e-07,    -4.1422529395366859e-09, } , 
      {    2.9531561929488152e+01,     6.3876747056935295e+04,     2.6320459601316136e-04,     8.7356085256806377e-07,    -4.1018194785321506e-09, } , 
      {    2.8631306012210874e+01,     5.5338506166379455e+04,     5.4355993656856954e-04,     1.1338718623662069e-06,    -5.3285920172117202e-09, } , 
      {    3.1887116501740056e+01,     7.2205438481660429e+04,     4.1551734665450964e-04,     1.1239719669979502e-06,    -5.2881585562071848e-09, } , 
      {    2.6770800356920930e+00,    -3.5648797550348449e+04,    -1.1345442019791696e-03,    -1.6200340042483810e-06,     7.6666981089618681e-09, } , 
      {    5.0326346079439936e+00,    -2.7320106125623326e+04,    -9.8223145133782136e-04,    -1.3696228898184946e-06,     6.4803590312868347e-09, } , 
      {    8.2884450974731756e+00,    -1.0453173810342356e+04,    -1.1102740412518813e-03,    -1.3795227851867513e-06,     6.5207924922913700e-09, } , 
      {    7.3881891801958925e+00,    -1.8991414700898200e+04,    -8.2991870069647319e-04,    -1.1192117753886082e-06,     5.2940199536117996e-09, } , 
      {    6.4879332629186148e+00,    -2.7529655591454048e+04,    -5.4956336014106485e-04,    -8.5890076559046504e-07,     4.0672474149322301e-09, } , 
      {    9.7437437524477932e+00,    -1.0662723276173074e+04,    -6.7760595005512492e-04,    -8.6880066095872177e-07,     4.1076808759367654e-09, } , 
      {    1.2999554241976975e+01,     6.2042090391078964e+03,    -8.0564853996918488e-04,    -8.7870055632697849e-07,     4.1481143369413007e-09, } , 
      {    1.6255364731506155e+01,     2.3071141354388878e+04,    -9.3369112988324495e-04,    -8.8860045169523522e-07,     4.1885477979458360e-09, } , 
      {    8.8434878351705155e+00,    -1.9200964166728922e+04,    -3.9725060949971663e-04,    -6.0848965116057862e-07,     2.8809083372571958e-09, } , 
      {    1.2099298324699694e+01,    -2.3340318514479477e+03,    -5.2529319941377664e-04,    -6.1838954652883535e-07,     2.9213417982617307e-09, } , 
      {    1.5355108814228876e+01,     1.4532900463833023e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {    1.8610919303758052e+01,     3.1399832779114004e+04,    -7.8137837924189667e-04,    -6.3818933726534880e-07,     3.0022087202708014e-09, } , 
      {    1.1199042407422414e+01,    -1.0872272742003795e+04,    -2.4493785885836841e-04,    -3.5807853673069220e-07,     1.6945692595821610e-09, } , 
      {    1.4454852896951593e+01,     5.9946595732771784e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.7710663386480775e+01,     2.2861591888558149e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.0966473876009953e+01,     3.9728524203839130e+04,    -6.2906562860054840e-04,    -3.8777822283546238e-07,     1.8158696425957667e-09, } , 
      {    2.4222284365539135e+01,     5.6595456519120111e+04,    -7.5710821851460836e-04,    -3.9767811820371910e-07,     1.8563031036003020e-09, } , 
      {    1.0298786490145133e+01,    -1.9410513632559647e+04,     3.5417481697039803e-05,    -9.7767526932549030e-08,     4.6779672090259120e-10, } , 
      {    1.3554596979674315e+01,    -2.5435813172786693e+03,    -9.2625108217020158e-05,    -1.0766742230080575e-07,     5.0823018190712651e-10, } , 
      {    1.6810407469203493e+01,     1.4323350998002305e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.0066217958732675e+01,     3.1190283313283275e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    2.3322028448261854e+01,     4.8057215628564256e+04,    -4.7675287795920012e-04,    -1.3736710840557593e-07,     6.2953056492073224e-10, } , 
      {    2.6577838937791036e+01,     6.4924147943845237e+04,    -6.0479546787326009e-04,    -1.4726700377383265e-07,     6.6996402592526756e-10, } , 
      {    1.2654341062397034e+01,    -1.1081822207834521e+04,     1.8773023233838805e-04,     1.5264358749733739e-07,    -7.1854235677244325e-10, } , 
      {    1.5910151551926216e+01,     5.7851101074464568e+03,     5.9687642424328090e-05,     1.4274369212908067e-07,    -6.7810889576790794e-10, } , 
      {    1.9165962041455394e+01,     2.2652042422727431e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.2421772530984576e+01,     3.9518974738008401e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    2.5677583020513755e+01,     5.6385907053289382e+04,    -3.2444012731785190e-04,     1.1304400602431049e-07,    -5.5680851275430221e-10, } , 
      {    1.8265706124178116e+01,     1.4113801532171583e+04,     2.1200039306567634e-04,     3.9315480655896706e-07,    -1.8644479734429424e-09, } , 
      {    2.1521516613707295e+01,     3.0980733847452557e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.4777327103236477e+01,     4.7847666162733527e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {    2.8033137592765655e+01,     6.4714598478014508e+04,    -1.7212737667650366e-04,     3.6345512045419689e-07,    -1.7431475904293367e-09, } , 
      {    2.0621260696430017e+01,     2.2442492956896709e+04,     3.6431314370702459e-04,     6.4356592098885348e-07,    -3.0507870511179771e-09, } , 
      {    2.3877071185959196e+01,     3.9309425272177687e+04,     2.3627055379296457e-04,     6.3366602562059676e-07,    -3.0103535901134417e-09, } , 
      {    2.7132881675488377e+01,     5.6176357587458653e+04,     1.0822796387890461e-04,     6.2376613025234003e-07,    -2.9699201291089064e-09, } , 
      {    3.0388692165017556e+01,     7.3043289902739634e+04,    -1.9814626035155407e-05,     6.1386623488408331e-07,    -2.9294866681043711e-09, } , 
      {    2.2976815268681918e+01,     3.0771184381621835e+04,     5.1662589434837286e-04,     8.9397703541873990e-07,    -4.2371261287930113e-09, } , 
      {    2.6232625758211093e+01,     4.7638116696902813e+04,     3.8858330443431279e-04,     8.8407714005048318e-07,    -4.1966926677884760e-09, } , 
      {    2.9488436247740275e+01,     6.4505049012183779e+04,     2.6054071452025283e-04,     8.7417724468222645e-07,    -4.1562592067839407e-09, } , 
      {    3.2744246737269457e+01,     8.1371981327464760e+04,     1.3249812460619281e-04,     8.6427734931396973e-07,    -4.1158257457794054e-09, } , 
      {    2.8588180330462997e+01,     5.5966808121627939e+04,     5.4089605507566107e-04,     1.1344882544803696e-06,    -5.3830317454635102e-09, } , 
      {    3.1843990819992179e+01,     7.2833740436908905e+04,     4.1285346516160110e-04,     1.1245883591121129e-06,    -5.3425982844589749e-09, } , 
      {    3.0943734902714894e+01,     6.4295499546353065e+04,     6.9320880571700934e-04,     1.3848993689102560e-06,    -6.5693708231385445e-09, } , 
      {    3.4199545392244076e+01,     8.1162431861634032e+04,     5.6516621580294938e-04,     1.3749994735419993e-06,    -6.5289373621340092e-09, } , 
      {    7.3450634984480141e+00,    -1.8363112745649716e+04,    -8.3258258218938167e-04,    -1.1185953832744455e-06,     5.2395802253600096e-09, } , 
      {    9.7006180706999157e+00,    -1.0034421320924592e+04,    -6.8026983154803339e-04,    -8.6818426884455909e-07,     4.0532411476849753e-09, } , 
      {    1.2956428560229098e+01,     6.8325109943563784e+03,    -8.0831242146209336e-04,    -8.7808416421281581e-07,     4.0936746086895106e-09, } , 
      {    1.2056172642951816e+01,    -1.7057298961994654e+03,    -5.2795708090668512e-04,    -6.1777315441467267e-07,     2.8669020700099411e-09, } , 
      {    1.5311983132480998e+01,     1.5161202419081505e+04,    -6.5599967082074508e-04,    -6.2767304978292939e-07,     2.9073355310144764e-09, } , 
      {    1.1155916725674537e+01,    -1.0243970786755313e+04,    -2.4760174035127694e-04,    -3.5746214461652952e-07,     1.6401295313303711e-09, } , 
      {    1.4411727215203715e+01,     6.6229615285256605e+03,    -3.7564433026533695e-04,    -3.6736203998478625e-07,     1.6805629923349062e-09, } , 
      {    1.7667537704732897e+01,     2.3489893843806633e+04,    -5.0368692017939691e-04,    -3.7726193535304297e-07,     1.7209964533394415e-09, } , 
      {    2.0923348194262076e+01,     4.0356826159087614e+04,    -6.3172951009345688e-04,    -3.8716183072129970e-07,     1.7614299143439768e-09, } , 
      {    1.3511471297926438e+01,    -1.9152793620301870e+03,    -9.5288989709928676e-05,    -1.0705103018664309e-07,     4.5379045365533670e-10, } , 
      {    1.6767281787455616e+01,     1.4951652953250787e+04,    -2.2333157962398865e-04,    -1.1695092555489981e-07,     4.9422391465987196e-10, } , 
      {    2.0023092276984798e+01,     3.1818585268531759e+04,    -3.5137416953804869e-04,    -1.2685082092315655e-07,     5.3465737566440727e-10, } , 
      {    2.3278902766513976e+01,     4.8685517583812740e+04,    -4.7941675945210866e-04,    -1.3675071629141327e-07,     5.7509083666894248e-10, } , 
      {    1.5867025870178338e+01,     6.4134120626949389e+03,     5.7023760931419579e-05,     1.4336008424324332e-07,    -7.3254862401969770e-10, } , 
      {    1.9122836359707517e+01,     2.3280344377975915e+04,    -7.1018828982640430e-05,     1.3346018887498660e-07,    -6.9211516301516249e-10, } , 
      {    2.2378646849236699e+01,     4.0147276693256885e+04,    -1.9906141889670039e-04,     1.2356029350672987e-07,    -6.5168170201062718e-10, } , 
      {    2.5634457338765877e+01,     5.7014209008537866e+04,    -3.2710400881076043e-04,     1.1366039813847316e-07,    -6.1124824100609197e-10, } , 
      {    1.8222580442430239e+01,     1.4742103487420065e+04,     2.0933651157276783e-04,     3.9377119867312974e-07,    -1.9188877016947321e-09, } , 
      {    2.1478390931959417e+01,     3.1609035802701041e+04,     8.1293921658707818e-05,     3.8387130330487302e-07,    -1.8784542406901967e-09, } , 
      {    2.4734201421488599e+01,     4.8475968117982011e+04,    -4.6748668255352150e-05,     3.7397140793661629e-07,    -1.8380207796856616e-09, } , 
      {    2.7990011911017778e+01,     6.5342900433262992e+04,    -1.7479125816941216e-04,     3.6407151256835957e-07,    -1.7975873186811265e-09, } , 
      {    2.0578135014682140e+01,     2.3070794912145193e+04,     3.6164926221411605e-04,     6.4418231310301616e-07,    -3.1052267793697667e-09, } , 
      {    2.3833945504211318e+01,     3.9937727227426170e+04,     2.3360667230005607e-04,     6.3428241773475944e-07,    -3.0647933183652314e-09, } , 
      {    2.7089755993740500e+01,     5.6804659542707137e+04,     1.0556408238599609e-04,     6.2438252236650271e-07,    -3.0243598573606961e-09, } , 
      {    2.6189500076463215e+01,     4.8266418652151297e+04,     3.8591942294140426e-04,     8.8469353216464586e-07,    -4.2511323960402661e-09, } , 
      {    2.9445310565992397e+01,     6.5133350967432263e+04,     2.5787683302734430e-04,     8.7479363679638913e-07,    -4.2106989350357307e-09, } , 
      {    2.8545054648715119e+01,     5.6595110076876423e+04,     5.3823217358275259e-04,     1.1351046465945323e-06,    -5.4374714737153003e-09, } , 
      {    3.1800865138244301e+01,     7.3462042392157382e+04,     4.1018958366869257e-04,     1.1252047512262756e-06,    -5.3970380127107650e-09, } , 
      {    1.2013046961203937e+01,    -1.0774279409509832e+03,    -5.3062096239959371e-04,    -6.1715676230050999e-07,     2.8124623417581510e-09, } , 
      {    1.5268857450733119e+01,     1.5789504374329987e+04,    -6.5866355231365367e-04,    -6.2705665766876671e-07,     2.8528958027626863e-09, } , 
      {    1.1112791043926658e+01,    -9.6156688315068313e+03,    -2.5026562184418541e-04,    -3.5684575250236689e-07,     1.5856898030785814e-09, } , 
      {    1.4368601533455836e+01,     7.2512634837741425e+03,    -3.7830821175824543e-04,    -3.6674564787062362e-07,     1.6261232640831165e-09, } , 
      {    1.7624412022985020e+01,     2.4118195799055113e+04,    -5.0635080167230550e-04,    -3.7664554323888034e-07,     1.6665567250876519e-09, } , 
      {    1.3468345616178558e+01,    -1.2869774067817048e+03,    -9.7952871202837180e-05,    -1.0643463807248042e-07,     3.9935072540354693e-10, } , 
      {    1.6724156105707738e+01,     1.5579954908499269e+04,    -2.2599546111689718e-04,    -1.1633453344073715e-07,     4.3978418640808214e-10, } , 
      {    1.9979966595236920e+01,     3.2446887223780239e+04,    -3.5403805103095717e-04,    -1.2623442880899387e-07,     4.8021764741261740e-10, } , 
      {    1.9079710677959639e+01,     2.3908646333224395e+04,    -7.3682710475548934e-05,     1.3407658098914928e-07,    -7.4655489126695236e-10, } , 
      {    2.2335521167488821e+01,     4.0775578648505369e+04,    -2.0172530038960892e-04,     1.2417668562089255e-07,    -7.0612143026241705e-10, } , 
      {    2.1435265250211540e+01,     3.2237337757949521e+04,     7.8630040165799314e-05,     3.8448769541903564e-07,    -1.9328939689419868e-09, } , 
      {    2.4691075739740722e+01,     4.9104270073230495e+04,    -4.9412549748260661e-05,     3.7458780005077892e-07,    -1.8924605079374515e-09, } , 
      {    1.7581286341237139e+01,     2.4746497754303597e+04,    -5.0901468316521398e-04,    -3.7602915112471766e-07,     1.6121169968358620e-09, } , 
      {    1.6681030423959857e+01,     1.6208256863747751e+04,    -2.2865934260980569e-04,    -1.1571814132657449e-07,     3.8534445815629232e-10, } , 
      {    1.9936840913489039e+01,     3.3075189179028719e+04,    -3.5670193252386570e-04,    -1.2561803669483122e-07,     4.2577791916082764e-10, } , 
      {    1.9036584996211758e+01,     2.4536948288472879e+04,    -7.6346591968457452e-05,     1.3469297310331193e-07,    -8.0099461951874213e-10, } , 
      {    1.9739571082002342e+01,     1.2509432807872196e+04,    -4.0074674639830189e-04,    -3.3723775405810035e-07,     1.6895175010249994e-09, } , 
      {    2.2995381571531524e+01,     2.9376365123153173e+04,    -5.2878933631236174e-04,    -3.4713764942635707e-07,     1.7299509620295347e-09, } , 
      {    2.2095125654254243e+01,     2.0838124232597322e+04,    -2.4843399575695362e-04,    -8.6826639628213924e-08,     5.0317842334996506e-10, } , 
      {    2.5350936143783425e+01,     3.7705056547878296e+04,    -3.7647658567101363e-04,    -9.6726534996470649e-08,     5.4361188435450037e-10, } , 
      {    2.4450680226506144e+01,     2.9166815657322448e+04,    -9.6121245115605368e-05,     1.6358447480167250e-07,    -6.8316065432506939e-10, } , 
      {    1.7340890828002568e+01,     4.8090433383955515e+03,    -5.5572337853255864e-04,    -5.8703247637382414e-07,     2.8214168504482444e-09, } , 
      {    2.0596701317531750e+01,     2.1675975653676527e+04,    -6.8376596844661860e-04,    -5.9693237174208086e-07,     2.8618503114527797e-09, } , 
      {    1.9696445400254465e+01,     1.3137734763120678e+04,    -4.0341062789121037e-04,    -3.3662136194393772e-07,     1.6350777727732097e-09, } , 
      {    2.2952255889783647e+01,     3.0004667078401653e+04,    -5.3145321780527033e-04,    -3.4652125731219444e-07,     1.6755112337777451e-09, } , 
      {    1.8796189482977187e+01,     4.5994938725648299e+03,    -1.2305528733580213e-04,    -7.6310352145794533e-08,     4.0830523409364004e-10, } , 
      {    2.2051999972506366e+01,     2.1466426187845802e+04,    -2.5109787724986209e-04,    -8.6210247514051258e-08,     4.4873869509817525e-10, } , 
      {    2.5307810462035548e+01,     3.8333358503126779e+04,    -3.7914046716392211e-04,    -9.6110142882307982e-08,     4.8917215610271051e-10, } , 
      {    2.1151744055229088e+01,     1.2928185297289956e+04,     2.9257463305546124e-05,     1.7410076228409190e-07,    -7.7803384358139447e-10, } , 
      {    2.4407554544758266e+01,     2.9795117612570928e+04,    -9.8785126608513872e-05,     1.6420086691583518e-07,    -7.3760038257685926e-10, } , 
      {    2.6763109117010167e+01,     3.8123809037296051e+04,     5.3527624032834369e-05,     4.1461198134572154e-07,    -1.9239394602518938e-09, } , 
      {    1.8198021063531968e+01,     1.3975586184199885e+04,    -8.3874260058087536e-04,    -8.4672709405780460e-07,     3.9937496608760243e-09, } , 
      {    1.7297765146254690e+01,     5.4373452936440335e+03,    -5.5838726002546712e-04,    -5.8641608425966146e-07,     2.7669771221964547e-09, } , 
      {    2.0553575635783872e+01,     2.2304277608925011e+04,    -6.8642984993952708e-04,    -5.9631597962791818e-07,     2.8074105832009901e-09, } , 
      {    1.9653319718506587e+01,     1.3766036718369160e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    2.2909130208035769e+01,     3.0632969033650137e+04,    -5.3411709929817881e-04,    -3.4590486519803176e-07,     1.6210715055259552e-09, } , 
      {    2.6164940697564948e+01,     4.7499901348931104e+04,    -6.6215968921223888e-04,    -3.5580476056628849e-07,     1.6615049665304903e-09, } , 
      {    1.8753063801229310e+01,     5.2277958278133119e+03,    -1.2571916882871064e-04,    -7.5693960031631866e-08,     3.5386550584185022e-10, } , 
      {    2.2008874290758488e+01,     2.2094728143094286e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {    2.5264684780287670e+01,     3.8961660458375263e+04,    -3.8180434865683064e-04,    -9.5493750768145316e-08,     4.3473242785092074e-10, } , 
      {    2.8520495269816848e+01,     5.5828592773656230e+04,    -5.0984693857089060e-04,    -1.0539364613640204e-07,     4.7516588885545595e-10, } , 
      {    2.1108618373481210e+01,     1.3556487252538438e+04,     2.6593581812637613e-05,     1.7471715439825455e-07,    -8.3247357183318423e-10, } , 
      {    2.4364428863010389e+01,     3.0423419567819412e+04,    -1.0144900810142239e-04,     1.6481725902999783e-07,    -7.9204011082864902e-10, } , 
      {    2.7620239352539571e+01,     4.7290351883100389e+04,    -2.2949159801548236e-04,     1.5491736366174111e-07,    -7.5160664982411371e-10, } , 
      {    2.3464172945733111e+01,     2.1885178677263564e+04,     1.7890633245398586e-04,     4.2512826882814095e-07,    -2.0188126495082188e-09, } , 
      {    2.6719983435262289e+01,     3.8752110992544534e+04,     5.0863742539925859e-05,     4.1522837345988422e-07,    -1.9783791885036835e-09, } , 
      {    2.9975793924791471e+01,     5.5619043307825516e+04,    -7.7178847374134116e-05,     4.0532847809162750e-07,    -1.9379457274991482e-09, } , 
      {    2.9075538007514190e+01,     4.7080802417269661e+04,     2.0317649318127409e-04,     6.6563948788977065e-07,    -3.1647182661787181e-09, } , 
      {    3.2331348497043372e+01,     6.3947734732550642e+04,     7.5133903267214132e-05,     6.5573959252151392e-07,    -3.1242848051741828e-09, } , 
      {    1.7254639464506813e+01,     6.0656472488925156e+03,    -5.6105114151837560e-04,    -5.8579969214549878e-07,     2.7125373939446651e-09, } , 
      {    2.0510449954035995e+01,     2.2932579564173495e+04,    -6.8909373143243556e-04,    -5.9569958751375550e-07,     2.7529708549492004e-09, } , 
      {    1.9610194036758710e+01,     1.4394338673617642e+04,    -4.0873839087702743e-04,    -3.3538857771561236e-07,     1.5261983162696300e-09, } , 
      {    2.2866004526287892e+01,     3.1261270988898621e+04,    -5.3678098079108728e-04,    -3.4528847308386908e-07,     1.5666317772741653e-09, } , 
      {    2.1965748609010610e+01,     2.2723030098342770e+04,    -2.5642564023567915e-04,    -8.4977463285725924e-08,     3.3985923859459561e-10, } , 
      {    2.5221559098539792e+01,     3.9589962413623747e+04,    -3.8446823014973917e-04,    -9.4877358653982649e-08,     3.8029269959913092e-10, } , 
      {    2.1065492691733333e+01,     1.4184789207786920e+04,     2.3929700319729102e-05,     1.7533354651241721e-07,    -8.8691330008497400e-10, } , 
      {    2.4321303181262511e+01,     3.1051721523067896e+04,    -1.0411288959433091e-04,     1.6543365114416048e-07,    -8.4647983908043879e-10, } , 
      {    2.7577113670791693e+01,     4.7918653838348873e+04,    -2.3215547950839087e-04,     1.5553375577590376e-07,    -8.0604637807590348e-10, } , 
      {    2.6676857753514412e+01,     3.9380412947793018e+04,     4.8199861047017348e-05,     4.1584476557404690e-07,    -2.0328189167554731e-09, } , 
      {    2.9932668243043594e+01,     5.6247345263073999e+04,    -7.9842728867042634e-05,     4.0594487020579018e-07,    -1.9923854557509378e-09, } , 
      {    2.9032412325766312e+01,     4.7709104372518144e+04,     2.0051261168836559e-04,     6.6625588000393333e-07,    -3.2191579944305078e-09, } , 
      {    2.1922622927262733e+01,     2.3351332053591250e+04,    -2.5908952172858763e-04,    -8.4361071171563257e-08,     2.8541951034280585e-10, } , 
      {    2.4278177499514634e+01,     3.1680023478316376e+04,    -1.0677677108723941e-04,     1.6605004325832316e-07,    -9.0091956733222866e-10, } , 
      {    2.5881419502582503e+01,     2.8190748888271537e+04,    -7.1153226607249202e-04,    -5.6619169370123234e-07,     2.8163650918910831e-09, } , 
      {    2.4981163585305222e+01,     1.9652507997715682e+04,    -4.3117692551708384e-04,    -3.0588068390308908e-07,     1.5895925532115131e-09, } , 
      {    2.8236974074834400e+01,     3.6519440312996667e+04,    -5.5921951543114385e-04,    -3.1578057927134581e-07,     1.6300260142160484e-09, } , 
      {    2.7336718157557122e+01,     2.7981199422440808e+04,    -2.7886417487573562e-04,    -5.5469569473202670e-08,     4.0325347553647864e-10, } , 
      {    3.0592528647086301e+01,     4.4848131737721793e+04,    -4.0690676478979558e-04,    -6.5369464841459388e-08,     4.4368693654101395e-10, } , 
      {    2.9692272729809023e+01,     3.6309890847165938e+04,    -1.2655142423438734e-04,     1.9494154495668376e-07,    -7.8308560213855582e-10, } , 
      {    3.2948083219338201e+01,     5.3176823162446919e+04,    -2.5459401414844730e-04,     1.8504164958842703e-07,    -7.4265214113402050e-10, } , 
      {    3.2047827302060924e+01,     4.4638582271891064e+04,     2.5761326406960904e-05,     4.4535265938657018e-07,    -1.9694246798135901e-09, } , 
      {    2.2582483331305440e+01,     1.1952118528239040e+04,    -5.8615355765134053e-04,    -5.5567540621881293e-07,     2.7214919026347581e-09, } , 
      {    2.5838293820834622e+01,     2.8819050843520021e+04,    -7.1419614756540050e-04,    -5.6557530158706966e-07,     2.7619253636392934e-09, } , 
      {    2.4938037903557341e+01,     2.0280809952964166e+04,    -4.3384080700999237e-04,    -3.0526429178892640e-07,     1.5351528249597232e-09, } , 
      {    2.8193848393086519e+01,     3.7147742268245143e+04,    -5.6188339692405233e-04,    -3.1516418715718313e-07,     1.5755862859642585e-09, } , 
      {    2.4037781986280063e+01,     1.1742569062408322e+04,    -1.5348546645458408e-04,    -4.4953281990783278e-08,     3.0838028628015351e-10, } , 
      {    2.7293592475809241e+01,     2.8609501377689292e+04,    -2.8152805636864415e-04,    -5.4853177359040003e-08,     3.4881374728468882e-10, } , 
      {    3.0549402965338420e+01,     4.5476433692970269e+04,    -4.0957064628270411e-04,    -6.4753072727296734e-08,     3.8924720828922413e-10, } , 
      {    2.9649147048061142e+01,     3.6938192802414414e+04,    -1.2921530572729584e-04,     1.9555793707084641e-07,    -8.3752533039034558e-10, } , 
      {    3.2904957537590320e+01,     5.3805125117695396e+04,    -2.5725789564135583e-04,     1.8565804170258969e-07,    -7.9709186938581027e-10, } , 
      {    3.2004701620313043e+01,     4.5266884227139541e+04,     2.3097444914052390e-05,     4.4596905150073286e-07,    -2.0238644080653797e-09, } , 
      {    3.5260512109842217e+01,     6.2133816542420522e+04,    -1.0494514500000757e-04,     4.3606915613247613e-07,    -1.9834309470608444e-09, } , 
      {    3.4360256192564940e+01,     5.3595575651864667e+04,     1.7541019555540065e-04,     6.9638016593061917e-07,    -3.2102034857404144e-09, } , 
      {    3.7616066682094122e+01,     7.0462507967145648e+04,     4.7367605641340677e-05,     6.8648027056236245e-07,    -3.1697700247358791e-09, } , 
      {    2.0183803077305665e+01,     4.2517290587623957e+03,    -7.4113018978559739e-04,    -8.0547012853453667e-07,     3.8533912520580027e-09, } , 
      {    2.3439613566834840e+01,     2.1118661374043375e+04,    -8.6917277969965736e-04,    -8.1537002390279340e-07,     3.8938247130625380e-09, } , 
      {    2.2539357649557562e+01,     1.2580420483487522e+04,    -5.8881743914424912e-04,    -5.5505901410465025e-07,     2.6670521743829680e-09, } , 
      {    2.5795168139086744e+01,     2.9447352798768501e+04,    -7.1686002905830908e-04,    -5.6495890947290698e-07,     2.7074856353875033e-09, } , 
      {    2.9050978628615926e+01,     4.6314285114049468e+04,    -8.4490261897236915e-04,    -5.7485880484116370e-07,     2.7479190963920386e-09, } , 
      {    2.4894912221809463e+01,     2.0909111908212646e+04,    -4.3650468850290084e-04,    -3.0464789967476378e-07,     1.4807130967079336e-09, } , 
      {    2.8150722711338641e+01,     3.7776044223493627e+04,    -5.6454727841696091e-04,    -3.1454779504302050e-07,     1.5211465577124689e-09, } , 
      {    3.1406533200867823e+01,     5.4642976538774594e+04,    -6.9258986833102088e-04,    -3.2444769041127723e-07,     1.5615800187170042e-09, } , 
      {    2.3994656304532185e+01,     1.2370871017656804e+04,    -1.5614934794749261e-04,    -4.4336889876620618e-08,     2.5394055802836374e-10, } , 
      {    2.7250466794061364e+01,     2.9237803332937772e+04,    -2.8419193786155262e-04,    -5.4236785244877343e-08,     2.9437401903289905e-10, } , 
      {    3.0506277283590542e+01,     4.6104735648218753e+04,    -4.1223452777561258e-04,    -6.4136680613134068e-08,     3.3480748003743437e-10, } , 
      {    3.3762087773119731e+01,     6.2971667963499720e+04,    -5.4027711768967260e-04,    -7.4036575981390792e-08,     3.7524094104196958e-10, } , 
      {    2.6350210876784086e+01,     2.0699562442381928e+04,    -3.8365973061443522e-06,     2.0607422455326582e-07,    -9.3239851964667066e-10, } , 
      {    2.9606021366313264e+01,     3.7566494757662898e+04,    -1.3187918722020438e-04,     1.9617432918500909e-07,    -8.9196505864213545e-10, } , 
      {    3.2861831855842446e+01,     5.4433427072943879e+04,    -2.5992177713426431e-04,     1.8627443381675237e-07,    -8.5153159763760014e-10, } , 
      {    3.6117642345371628e+01,     7.1300359388224853e+04,    -3.8796436704832433e-04,     1.7637453844849564e-07,    -8.1109813663306493e-10, } , 
      {    3.1961575938565165e+01,     4.5895186182388024e+04,     2.0433563421143879e-05,     4.4658544361489549e-07,    -2.0783041363171698e-09, } , 
      {    3.5217386428094343e+01,     6.2762118497669006e+04,    -1.0760902649291608e-04,     4.3668554824663876e-07,    -2.0378706753126345e-09, } , 
      {    3.4317130510817066e+01,     5.4223877607113151e+04,     1.7274631406249212e-04,     6.9699655804478185e-07,    -3.2646432139922045e-09, } , 
      {    3.7572941000346248e+01,     7.1090809922394139e+04,     4.4703724148432166e-05,     6.8709666267652513e-07,    -3.2242097529876691e-09, } , 
      {    3.6672685083068970e+01,     6.2552569031838277e+04,     3.2505906470384037e-04,     9.4740767247466827e-07,    -4.4509822916672387e-09, } , 
      {    3.9928495572598145e+01,     7.9419501347119265e+04,     1.9701647478978038e-04,     9.3750777710641155e-07,    -4.4105488306627034e-09, } , 
      {    2.0140677395557788e+01,     4.8800310140108777e+03,    -7.4379407127850587e-04,    -8.0485373642037399e-07,     3.7989515238062126e-09, } , 
      {    2.3396487885086962e+01,     2.1746963329291859e+04,    -8.7183666119256583e-04,    -8.1475363178863072e-07,     3.8393849848107479e-09, } , 
      {    2.6652298374616144e+01,     3.8613895644572825e+04,    -9.9987925110662590e-04,    -8.2465352715688744e-07,     3.8798184458152832e-09, } , 
      {    2.2496231967809685e+01,     1.3208722438736004e+04,    -5.9148132063715760e-04,    -5.5444262199048757e-07,     2.6126124461311784e-09, } , 
      {    2.5752042457338867e+01,     3.0075654754016985e+04,    -7.1952391055121756e-04,    -5.6434251735874430e-07,     2.6530459071357137e-09, } , 
      {    2.9007852946868049e+01,     4.6942587069297952e+04,    -8.4756650046527763e-04,    -5.7424241272700102e-07,     2.6934793681402490e-09, } , 
      {    2.1595976050532407e+01,     4.6704815481801597e+03,    -3.1112598008174936e-04,    -2.9413161219234437e-07,     1.3858399074516084e-09, } , 
      {    2.4851786540061585e+01,     2.1537413863461130e+04,    -4.3916856999580938e-04,    -3.0403150756060110e-07,     1.4262733684561437e-09, } , 
      {    2.8107597029590764e+01,     3.8404346178742111e+04,    -5.6721115990986939e-04,    -3.1393140292885782e-07,     1.4667068294606790e-09, } , 
      {    3.1363407519119946e+01,     5.5271278494023078e+04,    -6.9525374982392935e-04,    -3.2383129829711455e-07,     1.5071402904652143e-09, } , 
      {    2.3951530622784308e+01,     1.2999172972905286e+04,    -1.5881322944040111e-04,    -4.3720497762457952e-08,     1.9950082977657393e-10, } , 
      {    2.7207341112313486e+01,     2.9866105288186256e+04,    -2.8685581935446115e-04,    -5.3620393130714676e-08,     2.3993429078110924e-10, } , 
      {    3.0463151601842664e+01,     4.6733037603467237e+04,    -4.1489840926852112e-04,    -6.3520288498971401e-08,     2.8036775178564455e-10, } , 
      {    3.3718962091371850e+01,     6.3599969918748204e+04,    -5.4294099918258108e-04,    -7.3420183867228126e-08,     3.2080121279017976e-10, } , 
      {    2.6307085195036208e+01,     2.1327864397630412e+04,    -6.5004787990528639e-06,     2.0669061666742847e-07,    -9.8683824789846053e-10, } , 
      {    2.9562895684565387e+01,     3.8194796712911382e+04,    -1.3454306871311288e-04,     1.9679072129917174e-07,    -9.4640478689392522e-10, } , 
      {    3.2818706174094565e+01,     5.5061729028192363e+04,    -2.6258565862717284e-04,     1.8689082593091502e-07,    -9.0597132588938990e-10, } , 
      {    3.6074516663623747e+01,     7.1928661343473330e+04,    -3.9062824854123286e-04,     1.7699093056265830e-07,    -8.6553786488485469e-10, } , 
      {    2.8662639767288109e+01,     2.9656555822355538e+04,     1.4581227184229538e-04,     4.5710173109731489e-07,    -2.1731773255734948e-09, } , 
      {    3.1918450256817287e+01,     4.6523488137636508e+04,     1.7769681928235368e-05,     4.4720183572905817e-07,    -2.1327438645689595e-09, } , 
      {    3.5174260746346462e+01,     6.3390420452917489e+04,    -1.1027290798582459e-04,     4.3730194036080144e-07,    -2.0923104035644241e-09, } , 
      {    3.8430071235875644e+01,     8.0257352768198456e+04,    -2.3831549789988461e-04,     4.2740204499254472e-07,    -2.0518769425598893e-09, } , 
      {    3.1018194339540010e+01,     3.7985247247080668e+04,     2.9812502248364363e-04,     7.0751284552720126e-07,    -3.3595164032485294e-09, } , 
      {    3.4274004829069185e+01,     5.4852179562361634e+04,     1.7008243256958362e-04,     6.9761295015894453e-07,    -3.3190829422439941e-09, } , 
      {    3.7529815318598367e+01,     7.1719111877642616e+04,     4.2039842655523655e-05,     6.8771305479068781e-07,    -3.2786494812394588e-09, } , 
      {    3.6629559401321089e+01,     6.3180870987086761e+04,     3.2239518321093184e-04,     9.4802406458883095e-07,    -4.5054220199190288e-09, } , 
      {    3.9885369890850264e+01,     8.0047803302367742e+04,     1.9435259329687188e-04,     9.3812416922057423e-07,    -4.4649885589144935e-09, } , 
      {    3.8985113973572986e+01,     7.1509562411811887e+04,     4.7470793385228011e-04,     1.1984351790187174e-06,    -5.6917610975940630e-09, } , 
      {    2.3353362203339085e+01,     2.2375265284540343e+04,    -8.7450054268547431e-04,    -8.1413723967446804e-07,     3.7849452565589578e-09, } , 
      {    2.2453106286061807e+01,     1.3837024393984486e+04,    -5.9414520213006607e-04,    -5.5382622987632489e-07,     2.5581727178793887e-09, } , 
      {    2.5708916775590989e+01,     3.0703956709265469e+04,    -7.2218779204412603e-04,    -5.6372612524458162e-07,     2.5986061788839240e-09, } , 
      {    2.4808660858313708e+01,     2.2165715818709614e+04,    -4.4183245148871791e-04,    -3.0341511544643842e-07,     1.3718336402043538e-09, } , 
      {    2.8064471347842886e+01,     3.9032648133990595e+04,    -5.6987504140277787e-04,    -3.1331501081469514e-07,     1.4122671012088891e-09, } , 
      {    2.3908404941036430e+01,     1.3627474928153768e+04,    -1.6147711093330962e-04,    -4.3104105648295285e-08,     1.4506110152478413e-10, } , 
      {    2.7164215430565609e+01,     3.0494407243434740e+04,    -2.8951970084736969e-04,    -5.3004001016552009e-08,     1.8549456252931945e-10, } , 
      {    3.0420025920094787e+01,     4.7361339558715721e+04,    -4.1756229076142965e-04,    -6.2903896384808734e-08,     2.2592802353385476e-10, } , 
      {    2.6263959513288331e+01,     2.1956166352878896e+04,    -9.1643602919613747e-06,     2.0730700878159112e-07,    -1.0412779761502504e-09, } , 
      {    2.9519770002817509e+01,     3.8823098668159866e+04,    -1.3720695020602138e-04,     1.9740711341333440e-07,    -1.0008445151457151e-09, } , 
      {    3.2775580492346684e+01,     5.5690030983440847e+04,    -2.6524954012008137e-04,     1.8750721804507767e-07,    -9.6041105414117977e-10, } , 
      {    3.1875324575069410e+01,     4.7151790092884992e+04,     1.5105800435326858e-05,     4.4781822784322085e-07,    -2.1871835928207491e-09, } , 
      {    3.5131135064598581e+01,     6.4018722408165973e+04,    -1.1293678947873311e-04,     4.3791833247496412e-07,    -2.1467501318162138e-09, } , 
      {    3.4230879147321303e+01,     5.5480481517610118e+04,     1.6741855107667511e-04,     6.9822934227310721e-07,    -3.3735226704957838e-09, } , 
      {    3.7486689636850485e+01,     7.2347413832891092e+04,     3.9375961162615144e-05,     6.8832944690485049e-07,    -3.3330892094912485e-09, } , 
      {    2.2409980604313930e+01,     1.4465326349232968e+04,    -5.9680908362297466e-04,    -5.5320983776216221e-07,     2.5037329896275986e-09, } , 
      {    2.5665791093843112e+01,     3.1332258664513949e+04,    -7.2485167353703462e-04,    -5.6310973313041894e-07,     2.5441664506321339e-09, } , 
      {    2.4765535176565830e+01,     2.2794017773958094e+04,    -4.4449633298162638e-04,    -3.0279872333227579e-07,     1.3173939119525642e-09, } , 
      {    2.8021345666095009e+01,     3.9660950089239079e+04,    -5.7253892289568645e-04,    -3.1269861870053251e-07,     1.3578273729570995e-09, } , 
      {    2.7121089748817731e+01,     3.1122709198683220e+04,    -2.9218358234027816e-04,    -5.2387608902389349e-08,     1.3105483427752966e-10, } , 
      {    3.0376900238346909e+01,     4.7989641513964205e+04,    -4.2022617225433812e-04,    -6.2287504270646067e-08,     1.7148829528206497e-10, } , 
      {    2.7737824471112564e+01,     2.0351797628579508e+04,    -6.2191149975593960e-04,    -5.2308555183547636e-07,     2.5126874983176916e-09, } , 
      {    3.0993634960641746e+01,     3.7218729943860475e+04,    -7.4995408966999956e-04,    -5.3298544720373309e-07,     2.5531209593222269e-09, } , 
      {    3.0093379043364461e+01,     2.8680489053304635e+04,    -4.6959874911459132e-04,    -2.7267443740558989e-07,     1.3263484206426572e-09, } , 
      {    3.3349189532893647e+01,     4.5547421368585601e+04,    -5.9764133902865128e-04,    -2.8257433277384661e-07,     1.3667818816471925e-09, } , 
      {    3.2448933615616369e+01,     3.7009180478029761e+04,    -3.1728599847324310e-04,    -2.2263322975703451e-08,     1.4000934296762263e-10, } , 
      {    2.8594954706641964e+01,     2.9518340474383833e+04,    -9.0493072180425631e-04,    -7.8278016951945683e-07,     3.6850203087454715e-09, } , 
      {    2.7694698789364686e+01,     2.0980099583827992e+04,    -6.2457538124884807e-04,    -5.2246915972131368e-07,     2.4582477700659020e-09, } , 
      {    3.0950509278893868e+01,     3.7847031899108959e+04,    -7.5261797116290803e-04,    -5.3236905508957041e-07,     2.4986812310704373e-09, } , 
      {    3.0050253361616583e+01,     2.9308791008553118e+04,    -4.7226263060749985e-04,    -2.7205804529142721e-07,     1.2719086923908673e-09, } , 
      {    3.3306063851145765e+01,     4.6175723323834085e+04,    -6.0030522052155976e-04,    -2.8195794065968393e-07,     1.3123421533954026e-09, } , 
      {    2.9149997444339306e+01,     2.0770550117997267e+04,    -1.9190729005209159e-04,    -1.1747035493284063e-08,     4.5136153711297528e-11, } , 
      {    3.2405807933868488e+01,     3.7637482433278245e+04,    -3.1994987996615163e-04,    -2.1646930861540788e-08,     8.5569614715832840e-11, } , 
      {    3.5661618423397670e+01,     5.4504414748559211e+04,    -4.4799246988021159e-04,    -3.1546826229797513e-08,     1.2600307572036815e-10, } , 
      {    3.4761362506120385e+01,     4.5966173858003371e+04,    -1.6763712932480336e-04,     2.2876418356834563e-07,    -1.1007694629592016e-09, } , 
      {    3.0007127679868706e+01,     2.9937092963801602e+04,    -4.7492651210040838e-04,    -2.7144165317726453e-07,     1.2174689641390774e-09, } , 
      {    3.3262938169397884e+01,     4.6804025279082569e+04,    -6.0296910201446824e-04,    -2.8134154854552125e-07,     1.2579024251436127e-09, } , 
      {    3.2362682252120607e+01,     3.8265784388526728e+04,    -3.2261376145906016e-04,    -2.1030538747378125e-08,     3.1129886464043043e-11, } , 
      {    3.5618492741649789e+01,     5.5132716703807695e+04,    -4.5065635137312012e-04,    -3.0930434115634846e-08,     7.1563347468578362e-11, } , 
      {    3.2979416974415436e+01,     2.7494872818423002e+04,    -6.5234167887472149e-04,    -4.9172848168046515e-07,     2.4127625505042053e-09, } , 
      {    3.6235227463944618e+01,     4.4361805133703980e+04,    -7.8038426878878145e-04,    -5.0162837704872188e-07,     2.4531960115087406e-09, } , 
      {    3.5334971546667333e+01,     3.5823564243148125e+04,    -5.0002892823337332e-04,    -2.4131736725057863e-07,     1.2264234728291706e-09, } , 
      {    3.8590782036196515e+01,     5.2690496558429106e+04,    -6.2807151814743328e-04,    -2.5121726261883535e-07,     1.2668569338337059e-09, } , 
      {    3.7690526118919237e+01,     4.4152255667873251e+04,    -3.4771617759202510e-04,     9.0937471793077716e-09,     4.0084395154136244e-11, } , 
      {    4.0946336608448419e+01,     6.1019187983154232e+04,    -4.7575876750608506e-04,    -8.0614818894895263e-10,     8.0517856158671556e-11, } , 
      {    4.0046080691171134e+01,     5.2480947092598377e+04,    -1.9540342695067680e-04,     2.5950486160919421e-07,    -1.1462546825208983e-09, } , 
      {    3.0580736720415654e+01,     1.9794483348946356e+04,    -8.0731831100897835e-04,    -7.4152320399618890e-07,     3.5446618999274495e-09, } , 
      {    3.3836547209944840e+01,     3.6661415664227337e+04,    -9.3536090092303831e-04,    -7.5142309936444562e-07,     3.5850953609319848e-09, } , 
      {    3.2936291292667562e+01,     2.8123174773671482e+04,    -6.5500556036763007e-04,    -4.9111208956630247e-07,     2.3583228222524152e-09, } , 
      {    3.6192101782196744e+01,     4.4990107088952464e+04,    -7.8304815028169003e-04,    -5.0101198493455920e-07,     2.3987562832569505e-09, } , 
      {    3.5291845864919459e+01,     3.6451866198396609e+04,    -5.0269280972628191e-04,    -2.4070097513641600e-07,     1.1719837445773810e-09, } , 
      {    3.8547656354448641e+01,     5.3318798513677590e+04,    -6.3073539964034187e-04,    -2.5060087050467273e-07,     1.2124172055819163e-09, } , 
      {    3.7647400437171363e+01,     4.4780557623121735e+04,    -3.5038005908493358e-04,     9.7101392934704367e-09,    -1.4355333097653554e-11, } , 
      {    4.0903210926700545e+01,     6.1647489938402716e+04,    -4.7842264899899354e-04,    -1.8975607478628855e-10,     2.6078127906881759e-11, } , 
      {    4.0002955009423260e+01,     5.3109249047846861e+04,    -1.9806730844358533e-04,     2.6012125372335684e-07,    -1.2006944107726879e-09, } , 
      {    4.3258765498952442e+01,     6.9976181363127849e+04,    -3.2610989835764526e-04,     2.5022135835510012e-07,    -1.1602609497681526e-09, } , 
      {    4.2358509581675158e+01,     6.1437940472571987e+04,    -4.5754557802237074e-05,     5.1053236815324321e-07,    -2.3870334884477226e-09, } , 
      {    4.5614320071204340e+01,     7.8304872787852975e+04,    -1.7379714771629704e-04,     5.0063247278498648e-07,    -2.3466000274431873e-09, } , 
      {    3.1437866955945061e+01,     2.8961026194750695e+04,    -1.0903375330572952e-03,    -1.0012178216801694e-06,     4.7169947103552294e-09, } , 
      {    3.0537611038667777e+01,     2.0422785304194840e+04,    -8.0998219250188682e-04,    -7.4090681188202622e-07,     3.4902221716756598e-09, } , 
      {    3.3793421528196959e+01,     3.7289717619475821e+04,    -9.3802478241594679e-04,    -7.5080670725028294e-07,     3.5306556326801951e-09, } , 
      {    3.2893165610919681e+01,     2.8751476728919966e+04,    -6.5766944186053855e-04,    -4.9049569745213979e-07,     2.3038830940006256e-09, } , 
      {    3.6148976100448863e+01,     4.5618409044200947e+04,    -7.8571203177459851e-04,    -5.0039559282039652e-07,     2.3443165550051609e-09, } , 
      {    3.1992909693642396e+01,     2.0213235838364126e+04,    -3.7731410130513031e-04,    -2.3018468765399660e-07,     1.0771105553210558e-09, } , 
      {    3.5248720183171578e+01,     3.7080168153645092e+04,    -5.0535669121919038e-04,    -2.4008458302225332e-07,     1.1175440163255911e-09, } , 
      {    3.8504530672700760e+01,     5.3947100468926074e+04,    -6.3339928113325034e-04,    -2.4998447839051005e-07,     1.1579774773301264e-09, } , 
      {    3.4348464265894300e+01,     2.8541927263089252e+04,    -2.2500135066378207e-04,     2.0226426775889825e-08,    -1.0922852235397866e-10, } , 
      {    3.7604274755423482e+01,     4.5408859578370219e+04,    -3.5304394057784211e-04,     1.0326531407633100e-08,    -6.8795061349443351e-11, } , 
      {    4.0860085244952664e+01,     6.2275791893651200e+04,    -4.8108653049190207e-04,     4.2663603937637554e-10,    -2.8361600344908039e-11, } , 
      {    3.9959829327675379e+01,     5.3737551003095345e+04,    -2.0073118993649383e-04,     2.6073764583751952e-07,    -1.2551341390244778e-09, } , 
      {    4.3215639817204561e+01,     7.0604483318376326e+04,    -3.2877377985055379e-04,     2.5083775046926280e-07,    -1.2147006780199425e-09, } , 
      {    4.2315383899927276e+01,     6.2066242427820471e+04,    -4.8418439295145585e-05,     5.1114876026740589e-07,    -2.4414732166995123e-09, } , 
      {    4.5571194389456458e+01,     7.8933174743101452e+04,    -1.7646102920920555e-04,     5.0124886489914916e-07,    -2.4010397556949769e-09, } , 
      {    4.4670938472179181e+01,     7.0394933852545597e+04,     1.0389431134620266e-04,     7.6155987469729231e-07,    -3.6278122943745465e-09, } , 
      {    3.2850039929171800e+01,     2.9379778684168450e+04,    -6.6033332335344703e-04,    -4.8987930533797711e-07,     2.2494433657488359e-09, } , 
      {    3.6105850418700982e+01,     4.6246710999449431e+04,    -7.8837591326750699e-04,    -4.9977920070623384e-07,     2.2898768267533712e-09, } , 
      {    3.5205594501423697e+01,     3.7708470108893576e+04,    -5.0802057271209886e-04,    -2.3946819090809064e-07,     1.0631042880738012e-09, } , 
      {    3.8461404990952879e+01,     5.4575402424174557e+04,    -6.3606316262615882e-04,    -2.4936808627634737e-07,     1.1035377490783365e-09, } , 
      {    3.7561149073675601e+01,     4.6037161533618702e+04,    -3.5570782207075064e-04,     1.0942923521795764e-08,    -1.2323478960123314e-10, } , 
      {    4.0816959563204783e+01,     6.2904093848899684e+04,    -4.8375041198481060e-04,     1.0430281535390396e-09,    -8.2801328596697829e-11, } , 
      {    3.9916703645927498e+01,     5.4365852958343829e+04,    -2.0339507142940234e-04,     2.6135403795168220e-07,    -1.3095738672762677e-09, } , 
      {    4.0934544681777766e+01,     3.5965539594378803e+04,    -8.7617031372526778e-04,    -6.7695988734367844e-07,     3.1814928195451074e-09, } , 
      {    4.4190355171306948e+01,     5.2832471909659784e+04,    -1.0042129036393278e-03,    -6.8685978271193516e-07,     3.2219262805496427e-09, } , 
      {    4.3290099254029670e+01,     4.4294231019103929e+04,    -7.2385756308391950e-04,    -4.2654877291379197e-07,     1.9951537418700732e-09, } , 
      {    4.6545909743558852e+01,     6.1161163334384910e+04,    -8.5190015299797946e-04,    -4.3644866828204869e-07,     2.0355872028746085e-09, } , 
      {    4.5645653826281567e+01,     5.2622922443829055e+04,    -5.7154481244257134e-04,    -1.7613765848390554e-07,     8.0881466419503851e-10, } , 
      {    4.8901464315810749e+01,     6.9489854759110036e+04,    -6.9958740235663130e-04,    -1.8603755385216227e-07,     8.4924812519957383e-10, } , 
      {    4.8001208398533471e+01,     6.0951613868554181e+04,    -4.1923206180122306e-04,     7.4273455945980877e-08,    -3.7752441347999594e-10, } , 
      {    1.5707963267948966e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8266068163240767e+00,     1.6866932315280974e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    8.0824173058532569e+00,     3.3733864630561948e+04,    -2.5608517982811998e-04,    -1.9799790736513449e-08,     8.0866922009070521e-11, } , 
      {   -2.5852700800115631e+00,    -2.5405173205836822e+04,     4.0839793046946820e-04,     2.7021090516639987e-07,    -1.2672059996841049e-09, } , 
      {    6.7054040951761662e-01,    -8.5382408905558477e+03,     2.8035534055540824e-04,     2.6031100979814315e-07,    -1.2267725386795698e-09, } , 
      {    3.9263508990467968e+00,     8.3286914247251261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1821613885759774e+00,     2.5195623740006100e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -2.2971550775966332e-01,    -1.7076481781111695e+04,     5.6071068111081647e-04,     5.2062201959628629e-07,    -2.4535450773591395e-09, } , 
      {    3.0260949817695169e+00,    -2.0954946583072160e+02,     4.3266809119675651e-04,     5.1072212422802957e-07,    -2.4131116163546042e-09, } , 
      {    6.2819054712986970e+00,     1.6657382849450252e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.5377159608278781e+00,     3.3524315164731226e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    2.1258390644922365e+00,    -8.7477903563865693e+03,     7.1302343175216475e-04,     7.7103313402617271e-07,    -3.6398841550341742e-09, } , 
      {    5.3816495540214166e+00,     8.1191419588944045e+03,     5.8498084183810468e-04,     7.6113323865791599e-07,    -3.5994506940296389e-09, } , 
      {    8.6374600435505968e+00,     2.4986074274175378e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1893270533079777e+01,     4.1853006589456352e+04,     3.2889566200998476e-04,     7.4133344792140254e-07,    -3.5185837720205683e-09, } , 
      {    4.4813936367441372e+00,    -4.1909893166144320e+02,     8.6533618239351302e-04,     1.0214442484560591e-06,    -4.8262232327092084e-09, } , 
      {    7.7372041262733173e+00,     1.6447833383619531e+04,     7.3729359247945295e-04,     1.0115443530878024e-06,    -4.7857897717046731e-09, } , 
      {    1.0993014615802498e+01,     3.3314765698900505e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.4248825105331678e+01,     5.0181698014181478e+04,     4.8120841265133303e-04,     9.9174456235128896e-07,    -4.7049228496956025e-09, } , 
      {    1.0092758698525218e+01,     2.4776524808344657e+04,     8.8960634312080123e-04,     1.2619554675176888e-06,    -5.9721288493797074e-09, } , 
      {    1.3348569188054398e+01,     4.1643457123625631e+04,     7.6156375320674127e-04,     1.2520555721494321e-06,    -5.9316953883751721e-09, } , 
      {    1.5704123760306297e+01,     4.9972148548350757e+04,     9.1387650384808943e-04,     1.5024666865793185e-06,    -7.1180344660502071e-09, } , 
      {    1.8059678332558200e+01,     5.8300839973075883e+04,     1.0661892544894377e-03,     1.7528778010092049e-06,    -8.3043735437252414e-09, } , 
      {   -1.2605656788464382e+01,    -4.9343846593102273e+04,    -9.1654038534099791e-04,    -1.5018502944651558e-06,     7.0635947377984171e-09, } , 
      {   -1.0250102216212483e+01,    -4.1015155168377147e+04,    -7.6422763469964974e-04,    -1.2514391800352694e-06,     5.8772556601233820e-09, } , 
      {   -7.8945476439605820e+00,    -3.2686463743652021e+04,    -6.1191488405830147e-04,    -1.0010280656053830e-06,     4.6909165824483477e-09, } , 
      {   -5.5389930717086822e+00,    -2.4357772318926895e+04,    -4.5960213341695325e-04,    -7.5061695117549658e-07,     3.5045775047733139e-09, } , 
      {   -2.2831825821795020e+00,    -7.4908400036459225e+03,    -5.8764472333101316e-04,    -7.6051684654375331e-07,     3.5450109657778492e-09, } , 
      {   -6.4392489889859625e+00,    -3.2896013209482742e+04,    -1.7924679286154501e-04,    -4.9030594137735344e-07,     2.2778049660937439e-09, } , 
      {   -3.1834384994567824e+00,    -1.6029080894201770e+04,    -3.0728938277560503e-04,    -5.0020583674561016e-07,     2.3182384270982792e-09, } , 
      {    7.2371990072398029e-02,     8.3785142107920387e+02,    -4.3533197268966504e-04,    -5.1010573211386689e-07,     2.3586718881028146e-09, } , 
      {    3.3281824796015780e+00,     1.7704783736360179e+04,    -5.6337456260372495e-04,    -5.2000562748212361e-07,     2.3991053491073499e-09, } , 
      {   -4.0836944167340627e+00,    -2.4567321784757616e+04,    -2.6934042220196770e-05,    -2.3989482694746702e-07,     1.0914658884187093e-09, } , 
      {   -8.2788392720488169e-01,    -7.7003894694766441e+03,    -1.5497663213425675e-04,    -2.4979472231572374e-07,     1.1318993494232446e-09, } , 
      {    2.4279265623242985e+00,     9.1665428458043298e+03,    -2.8301922204831677e-04,    -2.5969461768398047e-07,     1.1723328104277799e-09, } , 
      {    5.6837370518534778e+00,     2.6033475161085305e+04,    -4.1106181196237673e-04,    -2.6959451305223719e-07,     1.2127662714323150e-09, } , 
      {   -1.7281398444821621e+00,    -1.6238630360032492e+04,     1.2537870842115148e-04,     1.0516287482419388e-08,    -9.4873189256325051e-11, } , 
      {    1.5276706450470183e+00,     6.2830195524848227e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    4.7834811345761983e+00,     1.7495234270529458e+04,    -1.3070647140696849e-04,    -9.2835032540940612e-09,    -1.4006267247254537e-11, } , 
      {   -2.6283957617594416e+00,    -2.4776871250588338e+04,     4.0573404897655967e-04,     2.7082729728056255e-07,    -1.3216457279358947e-09, } , 
      {    6.2741472776973839e-01,    -7.9099389353073657e+03,     2.7769145906249971e-04,     2.6092740191230583e-07,    -1.2812122669313596e-09, } , 
      {    3.8832252172989188e+00,     8.9569933799736082e+03,     1.4964886914843974e-04,     2.5102750654404910e-07,    -1.2407788059268243e-09, } , 
      {    7.1390357068280990e+00,     2.5823925695254584e+04,     2.1606279234379749e-05,     2.4112761117579238e-07,    -1.2003453449222890e-09, } , 
      {   -2.7284118950754155e-01,    -1.6448179825863212e+04,     5.5804679961790800e-04,     5.2123841171044897e-07,    -2.5079848056109292e-09, } , 
      {    2.9829693000216384e+00,     4.1875248941776067e+02,     4.3000420970384798e-04,     5.1133851634219225e-07,    -2.4675513446063939e-09, } , 
      {    6.2387797895508186e+00,     1.7285684804698736e+04,     3.0196161978978796e-04,     5.0143862097393552e-07,    -2.4271178836018586e-09, } , 
      {    9.4945902790799988e+00,     3.4152617119979710e+04,     1.7391902987572800e-04,     4.9153872560567880e-07,    -2.3866844225973233e-09, } , 
      {    5.3385238722735382e+00,     8.7474439141428866e+03,     5.8231696034519620e-04,     7.6174963077207867e-07,    -3.6538904222814285e-09, } , 
      {    8.5943343618027193e+00,     2.5614376229423862e+04,     4.5427437043113619e-04,     7.5184973540382194e-07,    -3.6134569612768932e-09, } , 
      {    7.6940784445254389e+00,     1.7076135338868015e+04,     7.3462971098654448e-04,     1.0121607452019651e-06,    -4.8402294999564632e-09, } , 
      {    1.0949888934054620e+01,     3.3943067654148988e+04,     6.0658712107248451e-04,     1.0022608498337084e-06,    -4.7997960389519279e-09, } , 
      {    1.3305443506306521e+01,     4.2271759078874114e+04,     7.5889987171383279e-04,     1.2526719642635948e-06,    -5.9861351166269621e-09, } , 
      {    1.5660998078558420e+01,     5.0600450503599241e+04,     9.1121262235518096e-04,     1.5030830786934812e-06,    -7.1724741943019972e-09, } , 
      {   -1.0293227897960362e+01,    -4.0386853213128663e+04,    -7.6689151619255833e-04,    -1.2508227879211067e-06,     5.8228159318715927e-09, } , 
      {   -7.9376733257084613e+00,    -3.2058161788403540e+04,    -6.1457876555121005e-04,    -1.0004116734912203e-06,     4.6364768541965585e-09, } , 
      {   -5.5821187534565606e+00,    -2.3729470363678414e+04,    -4.6226601490986172e-04,    -7.5000055906133390e-07,     3.4501377765215238e-09, } , 
      {   -3.2265641812046599e+00,    -1.5400778938953288e+04,    -3.0995326426851350e-04,    -4.9958944463144748e-07,     2.2637986988464892e-09, } , 
      {    2.9246308324519799e-02,     1.4661533763276861e+03,    -4.3799585418257352e-04,    -5.0948933999970421e-07,     2.3042321598510245e-09, } , 
      {   -4.1268200984819412e+00,    -2.3939019829509136e+04,    -2.9597923713105281e-05,    -2.3927843483330439e-07,     1.0370261601669196e-09, } , 
      {   -8.7100960895276014e-01,    -7.0720875142281620e+03,    -1.5764051362716528e-04,    -2.4917833020156111e-07,     1.0774596211714549e-09, } , 
      {    2.3848008805764200e+00,     9.7948448010528118e+03,    -2.8568310354122524e-04,    -2.5907822556981784e-07,     1.1178930821759902e-09, } , 
      {   -1.7712655262300401e+00,    -1.5610328404784010e+04,     1.2271482692824295e-04,     1.1132679596582053e-08,    -1.4931291750811484e-10, } , 
      {    1.4845449632991401e+00,     1.2566039104969645e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    4.7403554528283198e+00,     1.8123536225777938e+04,    -1.3337035289987702e-04,    -8.6671111399313961e-09,    -6.8445995499044334e-11, } , 
      {    5.8428904602186016e-01,    -7.2816369800588836e+03,     2.7502757756959123e-04,     2.6154379402646845e-07,    -1.3356519951831493e-09, } , 
      {    3.8400995355510403e+00,     9.5852953352220902e+03,     1.4698498765553121e-04,     2.5164389865821173e-07,    -1.2952185341786140e-09, } , 
      {    7.0959100250802205e+00,     2.6452227650503064e+04,     1.8942397741471238e-05,     2.4174400328995500e-07,    -1.2547850731740787e-09, } , 
      {    2.9398436182737604e+00,     1.0470544446662429e+03,     4.2734032821093950e-04,     5.1195490845635493e-07,    -2.5219910728581840e-09, } , 
      {    6.1956541078029410e+00,     1.7913986759947216e+04,     2.9929773829687949e-04,     5.0205501308809820e-07,    -2.4815576118536486e-09, } , 
      {    8.5512086800548399e+00,     2.6242678184672342e+04,     4.5161048893822771e-04,     7.5246612751798462e-07,    -3.6678966895286833e-09, } , 
      {    1.0906763252306741e+01,     3.4571369609397472e+04,     6.0392323957957593e-04,     1.0028772419478710e-06,    -4.8542357672037171e-09, } , 
      {   -5.6252444352044382e+00,    -2.3101168408429930e+04,    -4.6492989640277026e-04,    -7.4938416694717122e-07,     3.3956980482697342e-09, } , 
      {   -3.2696898629525384e+00,    -1.4772476983704806e+04,    -3.1261714576142204e-04,    -4.9897305251728480e-07,     2.2093589705946995e-09, } , 
      {   -1.3879373423358432e-02,     2.0944553315761686e+03,    -4.4065973567548205e-04,    -5.0887294788554153e-07,     2.2497924315992348e-09, } , 
      {   -9.1413529070063859e-01,    -6.4437855589796791e+03,    -1.6030439512007379e-04,    -2.4856193808739843e-07,     1.0230198929196651e-09, } , 
      {    2.3416751988285416e+00,     1.0423146756301294e+04,    -2.8834698503413378e-04,    -2.5846183345565516e-07,     1.0634533539242004e-09, } , 
      {   -1.8143912079779181e+00,    -1.4982026449535528e+04,     1.2005094543533445e-04,     1.1749071710744717e-08,    -2.0375264575990466e-10, } , 
      {    1.4414192815512619e+00,     1.8849058657454468e+03,    -7.9916444787255359e-06,     1.8491763424879922e-09,    -1.6331918475536940e-10, } , 
      {    4.6972297710804423e+00,     1.8751838181026422e+04,    -1.3603423439278553e-04,    -8.0507190257687326e-09,    -1.2288572375083414e-10, } , 
      {    5.4116336427398193e-01,    -6.6533350248104007e+03,     2.7236369607668270e-04,     2.6216018614063113e-07,    -1.3900917234349392e-09, } , 
      {    3.7969738538031619e+00,     1.0213597290470572e+04,     1.4432110616262271e-04,     2.5226029077237441e-07,    -1.3496582624304039e-09, } , 
      {    6.1525284260550626e+00,     1.8542288715195700e+04,     2.9663385680397096e-04,     5.0267140520226088e-07,    -2.5359973401054383e-09, } , 
      {    8.5080829983069624e+00,     2.6870980139920826e+04,     4.4894660744531918e-04,     7.5308251963214730e-07,    -3.7223364177804730e-09, } , 
      {   -3.3128155447004168e+00,    -1.4144175028456324e+04,    -3.1528102725433057e-04,    -4.9835666040312223e-07,     2.1549192423429098e-09, } , 
      {   -9.5726097244851660e-01,    -5.8154836037311970e+03,    -1.6296827661298229e-04,    -2.4794554597323575e-07,     9.6858016466787519e-10, } , 
      {    1.3982935998033836e+00,     2.5132078209939291e+03,    -1.0655525971634047e-05,     2.4655684566506563e-09,    -2.1775891300715919e-10, } , 
      {    3.7538481720552839e+00,     1.0841899245719054e+04,     1.4165722466971420e-04,     2.5287668288653709e-07,    -1.4040979906821937e-09, } , 
      {    6.9417658753414058e+00,     5.2581693240980503e+03,    -2.2438534640056430e-05,     2.9507893812523238e-08,     6.3394236941882920e-11, } , 
      {    4.5430856213416275e+00,    -2.4422201453785938e+03,    -1.7741516677431319e-04,    -2.2028682850320052e-07,     1.1952935863651277e-09, } , 
      {    6.8986401935935273e+00,     5.8864712793465324e+03,    -2.5102416132964941e-05,     3.0124285926685901e-08,     8.9545086900931297e-12, } , 
      {   -2.1114920491005096e-01,    -1.8471301039580361e+04,    -4.8470454954991819e-04,    -7.2049266524881073e-07,     3.5135320134634069e-09, } , 
      {    2.1444053673418488e+00,    -1.0142609614855237e+04,    -3.3239179890856996e-04,    -4.7008155081892426e-07,     2.3271929357883722e-09, } , 
      {    4.4999599395937491e+00,    -1.8139181901301113e+03,    -1.8007904826722169e-04,    -2.1967043638903787e-07,     1.1408538581133378e-09, } , 
      {    3.5997040223164691e+00,    -1.0352159080685959e+04,     1.0027629228818653e-04,     4.0640573409105293e-08,    -8.5918680566231935e-11, } , 
      {    6.8555145118456497e+00,     6.5147732345950153e+03,    -2.7766297625873455e-05,     3.0740678040848568e-08,    -4.5485219561696674e-11, } , 
      {    1.0111325001374830e+01,     2.3381705549875991e+04,    -1.5580888753993343e-04,     2.0840782672591843e-08,    -5.0517585571614133e-12, } , 
      {    5.9552585945683703e+00,    -2.0234676559608329e+03,     2.5258904292953477e-04,     2.9105168783899173e-07,    -1.2722577582412664e-09, } , 
      {    9.2110690840975504e+00,     1.4843464659320141e+04,     1.2454645301547481e-04,     2.8115179247073500e-07,    -1.2318242972367311e-09, } , 
      {    1.1566623656349449e+01,     2.3172156084045269e+04,     2.7685920365682303e-04,     5.3156290690062137e-07,    -2.4181633749117656e-09, } , 
      {    1.3922178228601350e+01,     3.1500847508770396e+04,     4.2917195429817125e-04,     7.8197402133050779e-07,    -3.6045024525868002e-09, } , 
      {   -2.6098294589098296e+00,    -2.6171690509057007e+04,    -6.3968118168417499e-04,    -9.7028738756453448e-07,     4.6454313628866515e-09, } , 
      {   -2.5427488665792919e-01,    -1.7842999084331881e+04,    -4.8736843104282666e-04,    -7.1987627313464805e-07,     3.4590922852116168e-09, } , 
      {   -1.1545308039352093e+00,    -2.6381239974887729e+04,    -2.0701309048741848e-04,    -4.5956526333650491e-07,     2.2323197465320469e-09, } , 
      {    2.1012796855939708e+00,    -9.5143076596067549e+03,    -3.3505568040147844e-04,    -4.6946515870476163e-07,     2.2727532075365822e-09, } , 
      {    5.3570901751231510e+00,     7.3526246556742208e+03,    -4.6309827031553846e-04,    -4.7936505407301836e-07,     2.3131866685411175e-09, } , 
      {    1.2010237683166907e+00,    -1.8052548550162603e+04,    -5.4700339846070225e-05,    -2.0915414890661846e-07,     1.0459806688570128e-09, } , 
      {    4.4568342578458706e+00,    -1.1856162348816290e+03,    -1.8274292976013022e-04,    -2.1905404427487519e-07,     1.0864141298615481e-09, } , 
      {    7.7126447473750508e+00,     1.5681316080399347e+04,    -3.1078551967419018e-04,    -2.2895393964313191e-07,     1.1268475908660832e-09, } , 
      {    3.5565783405685911e+00,    -9.7238571254374765e+03,     9.7612410795278029e-05,     4.1256965523267953e-08,    -1.4035840881802174e-10, } , 
      {    6.8123888300977713e+00,     7.1430751898434974e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    1.0068199319626951e+01,     2.4010007505124471e+04,    -1.5847276903284196e-04,     2.1457174786754507e-08,    -5.9491486808951217e-11, } , 
      {    5.9121329128204918e+00,    -1.3951657007123506e+03,     2.4992516143662629e-04,     2.9166807995315435e-07,    -1.3266974864930561e-09, } , 
      {    9.1679434023496711e+00,     1.5471766614568623e+04,     1.2188257152256629e-04,     2.8176818458489763e-07,    -1.2862640254885208e-09, } , 
      {    1.2423753891878851e+01,     3.2338698929849597e+04,    -6.1600183914937057e-06,     2.7186828921664090e-07,    -1.2458305644839857e-09, } , 
      {    8.2676874850723916e+00,     6.9335257240127758e+03,     4.0223791207797457e-04,     5.4207919438304077e-07,    -2.5130365641680910e-09, } , 
      {    1.1523497974601570e+01,     2.3800458039293750e+04,     2.7419532216391455e-04,     5.3217929901478405e-07,    -2.4726031031635556e-09, } , 
      {    1.3879052546853471e+01,     3.2129149464018876e+04,     4.2650807280526277e-04,     7.8259041344467047e-07,    -3.6589421808385903e-09, } , 
      {    1.6234607119105370e+01,     4.0457840888744002e+04,     5.7882082344661099e-04,     1.0330015278745569e-06,    -4.8452812585136241e-09, } , 
      {   -7.3640642851615077e+00,    -4.2200771403258776e+04,    -9.4697056445977991e-04,    -1.4704932243101446e-06,     6.9636697899849307e-09, } , 
      {   -5.0085097129096088e+00,    -3.3872079978533649e+04,    -7.9465781381843174e-04,    -1.2200821098802582e-06,     5.7773307123098957e-09, } , 
      {   -2.6529551406577081e+00,    -2.5543388553808523e+04,    -6.4234506317708347e-04,    -9.6967099545037180e-07,     4.5909916346348614e-09, } , 
      {   -3.5532110579349876e+00,    -3.4081629444364371e+04,    -3.6198972262167523e-04,    -7.0935998565222865e-07,     3.3642190959552919e-09, } , 
      {   -2.9740056840580742e-01,    -1.7214697129083397e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.9584099211233728e+00,    -3.4776481380242330e+02,    -6.1807490244979515e-04,    -7.2915977638874210e-07,     3.4450860179643625e-09, } , 
      {   -1.1976564856830878e+00,    -2.5752938019639245e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    2.0581540038460924e+00,    -8.8860057043582729e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    5.3139644933752725e+00,     7.9809266109227028e+03,    -4.6576215180844699e-04,    -4.7874866195885568e-07,     2.2587469402893278e-09, } , 
      {    1.1578980865688124e+00,    -1.7424246594914119e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    4.4137085760979922e+00,    -5.5731427963314673e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.6695190656271723e+00,     1.6309618035647829e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    3.5134526588207127e+00,    -9.0955551701889945e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.7692631483498928e+00,     7.7713771450919794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0025073637879073e+01,     2.4638309460372955e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    2.6131967415434327e+00,    -1.7633796060744840e+04,     3.7530386985777772e-04,     3.0218436743557376e-07,    -1.4215706757493813e-09, } , 
      {    5.8690072310726134e+00,    -7.6686374546386833e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    9.1248177206017935e+00,     1.6100068569817106e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2380628210130974e+01,     3.2967000885098081e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    8.2245618033245123e+00,     7.5618276792612578e+03,     3.9957403058506603e-04,     5.4269558649720345e-07,    -2.5674762924198806e-09, } , 
      {    1.1480372292853692e+01,     2.4428759994542233e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.4736182782382874e+01,     4.1295692309823207e+04,     1.4348885075694603e-04,     5.2289579576069001e-07,    -2.4866093704108100e-09, } , 
      {    1.0580116375576413e+01,     1.5890519103986384e+04,     5.5188678122641420e-04,     7.9310670092708988e-07,    -3.7538153700949153e-09, } , 
      {    1.3835926865105593e+01,     3.2757451419267360e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    1.6191481437357496e+01,     4.1086142843992486e+04,     5.7615694195370252e-04,     1.0336179199887196e-06,    -4.8997209867654142e-09, } , 
      {    1.8547036009609393e+01,     4.9414834268717612e+04,     7.2846969259505079e-04,     1.2840290344186060e-06,    -6.0860600644404485e-09, } , 
      {   -5.0516353946574872e+00,    -3.3243778023285166e+04,    -7.9732169531134022e-04,    -1.2194657177660955e-06,     5.7228909840581056e-09, } , 
      {   -2.6960808224055866e+00,    -2.4915086598560039e+04,    -6.4500894466999195e-04,    -9.6905460333620912e-07,     4.5365519063830713e-09, } , 
      {   -3.4052625015368565e-01,    -1.6586395173834913e+04,    -4.9269619402864367e-04,    -7.1864348890632269e-07,     3.3502128287080375e-09, } , 
      {    2.9152842393754943e+00,     2.8053714144605897e+02,    -6.2073878394270363e-04,    -7.2854338427457942e-07,     3.3906462897125728e-09, } , 
      {   -1.2407821674309663e+00,    -2.5124636064390761e+04,    -2.1234085347323549e-04,    -4.5833247910817955e-07,     2.1234402900284675e-09, } , 
      {    2.0150283220982139e+00,    -8.2577037491097908e+03,    -3.4038344338729550e-04,    -4.6823237447643627e-07,     2.1638737510330028e-09, } , 
      {    5.2708388116273941e+00,     8.6092285661711849e+03,    -4.6842603330135552e-04,    -4.7813226984469300e-07,     2.2043072120375382e-09, } , 
      {    1.1147724048209342e+00,    -1.6795944639665635e+04,    -6.0028102831887247e-05,    -2.0792136467829315e-07,     9.3710121235343308e-10, } , 
      {    4.3705828943501146e+00,     7.0987675615335547e+01,    -1.8807069274594723e-04,    -2.1782126004654988e-07,     9.7753467335796839e-10, } , 
      {    7.6263933838792939e+00,     1.6937919990896313e+04,    -3.1611328266000724e-04,    -2.2772115541480660e-07,     1.0179681343625035e-09, } , 
      {    3.4703269770728342e+00,    -8.4672532149405124e+03,     9.2284647809460994e-05,     4.2489749751593286e-08,    -2.4923786532160132e-10, } , 
      {    6.7261374666020144e+00,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    9.9819479561311955e+00,     2.5266611415621439e+04,    -1.6380053201865897e-04,     2.2689959015079834e-08,    -1.6837094331253080e-10, } , 
      {    5.8258815493247349e+00,    -1.3856179021538605e+02,     2.4459739845080923e-04,     2.9290086418147971e-07,    -1.4355769429966358e-09, } , 
      {    9.0816920388539160e+00,     1.6728370525065588e+04,     1.1655480853674925e-04,     2.8300096881322299e-07,    -1.3951434819921005e-09, } , 
      {    8.1814361215766347e+00,     8.1901296345097398e+03,     3.9691014909215750e-04,     5.4331197861136613e-07,    -2.6219160206716703e-09, } , 
      {    1.1437246611105815e+01,     2.5057061949790717e+04,     2.6886755917809749e-04,     5.3341208324310941e-07,    -2.5814825596671350e-09, } , 
      {    1.0536990693828535e+01,     1.6518821059234866e+04,     5.4922289973350572e-04,     7.9372309304125256e-07,    -3.8082550983467054e-09, } , 
      {    1.3792801183357716e+01,     3.3385753374515843e+04,     4.2118030981944571e-04,     7.8382319767299583e-07,    -3.7678216373421700e-09, } , 
      {    1.6148355755609614e+01,     4.1714444799240970e+04,     5.7349306046079404e-04,     1.0342343121028823e-06,    -4.9541607150172043e-09, } , 
      {   -2.7392065041534650e+00,    -2.4286784643311559e+04,    -6.4767282616290053e-04,    -9.6843821122204644e-07,     4.4821121781312821e-09, } , 
      {   -3.8365193190156388e-01,    -1.5958093218586433e+04,    -4.9536007552155226e-04,    -7.1802709679216001e-07,     3.2957731004562474e-09, } , 
      {    1.9719026403503359e+00,    -7.6294017938613088e+03,    -3.4304732488020398e-04,    -4.6761598236227365e-07,     2.1094340227812128e-09, } , 
      {    4.3274572126022361e+00,     6.9928963086381782e+02,    -1.9073457423885576e-04,    -2.1720486793238720e-07,     9.2309494510617863e-10, } , 
      {    3.4272012953249562e+00,    -7.8389512596920304e+03,     8.9620766316552490e-05,     4.3106141865755946e-08,    -3.0367759357339114e-10, } , 
      {    6.6830117848541359e+00,     9.0279810555889435e+03,    -3.8421823597507498e-05,     3.3206246497499222e-08,    -2.6324413256885588e-10, } , 
      {    5.7827558675768564e+00,     4.8974016503309622e+02,     2.4193351695790072e-04,     2.9351725629564234e-07,    -1.4900166712484255e-09, } , 
      {    9.0385663571060366e+00,     1.7356672480314071e+04,     1.1389092704384075e-04,     2.8361736092738562e-07,    -1.4495832102438902e-09, } , 
      {    1.1394120929357936e+01,     2.5685363905039198e+04,     2.6620367768518901e-04,     5.3402847535727209e-07,    -2.6359222879189250e-09, } , 
      {    1.3749675501609836e+01,     3.4014055329764327e+04,     4.1851642832653723e-04,     7.8443958978715851e-07,    -3.8222613655939593e-09, } , 
      {    1.9287769586024577e+00,    -7.0010998386128258e+03,    -3.4571120637311251e-04,    -4.6699959024811097e-07,     2.0549942945294231e-09, } , 
      {    4.2843315308543577e+00,     1.3275915861123001e+03,    -1.9339845573176426e-04,    -2.1658847581822455e-07,     8.6865521685438886e-10, } , 
      {    6.6398861031062584e+00,     9.6562830108374255e+03,    -4.1085705090416016e-05,     3.3822638611661889e-08,    -3.1768386082064569e-10, } , 
      {    8.9954406753581591e+00,     1.7984974435562552e+04,     1.1122704555093223e-04,     2.8423375304154830e-07,    -1.5040229384956800e-09, } , 
      {    1.2226484060392158e+01,     1.1772942558693065e+04,    -5.0204832265929882e-05,     6.0248571853371802e-08,     1.7909017380186259e-11, } , 
      {    9.8278038063923798e+00,     4.0725530892164215e+03,    -2.0518146440018665e-04,    -1.8954615046235197e-07,     1.1498083668034308e-09, } , 
      {    1.2183358378644281e+01,     1.2401244513941547e+04,    -5.2868713758838393e-05,     6.0864963967534469e-08,    -3.6530710871603557e-11, } , 
      {    1.4538912950896181e+01,     2.0729935938666673e+04,     9.9444036882509855e-05,     3.1127607839742090e-07,    -1.2228697885466381e-09, } , 
      {    7.4291235523926016e+00,    -3.6278363802602225e+03,    -3.6015809653444338e-04,    -4.3934087277807573e-07,     2.2817077162266756e-09, } , 
      {    9.7846781246445023e+00,     4.7008550444649036e+03,    -2.0784534589309516e-04,    -1.8892975834818931e-07,     1.0953686385516411e-09, } , 
      {    8.8844222073672228e+00,    -3.8373858460909441e+03,     7.2509994662313078e-05,     7.1381251449953861e-08,    -1.3140390012792861e-10, } , 
      {    1.2140232696896403e+01,     1.3029546469190031e+04,    -5.5532595251746910e-05,     6.1481356081697136e-08,    -9.0970439123393348e-11, } , 
      {    1.4495787269148304e+01,     2.1358237893915157e+04,     9.6780155389601338e-05,     3.1189247051158353e-07,    -1.2773095167984278e-09, } , 
      {    1.6851341841400206e+01,     2.9686929318640283e+04,     2.4909290603094961e-04,     5.6230358494147000e-07,    -2.4636485944734622e-09, } , 
      {    2.6748887261409227e+00,    -1.9656917274461994e+04,    -6.6744747931004841e-04,    -9.3954670952368595e-07,     4.5999461433249544e-09, } , 
      {    5.0304432983928233e+00,    -1.1328225849736866e+04,    -5.1513472866870013e-04,    -6.8913559509379953e-07,     3.4136070656499206e-09, } , 
      {    7.3859978706447231e+00,    -2.9995344250117405e+03,    -3.6282197802735191e-04,    -4.3872448066391305e-07,     2.2272679879748859e-09, } , 
      {    1.0641808360173904e+01,     1.3867397890269234e+04,    -4.9086456794141193e-04,    -4.4862437603216978e-07,     2.2677014489794212e-09, } , 
      {    6.4857419533674427e+00,    -1.1537775315567587e+04,    -8.2466637471943674e-05,    -1.7841347086576991e-07,     1.0004954492953159e-09, } , 
      {    9.7415524428966229e+00,     5.3291569997133856e+03,    -2.1050922738600366e-04,    -1.8831336623402663e-07,     1.0409289102998513e-09, } , 
      {    1.2997362932425803e+01,     2.2196089314994359e+04,    -3.3855181730006365e-04,    -1.9821326160228336e-07,     1.0813623713043866e-09, } , 
      {    8.8412965256193434e+00,    -3.2090838908424621e+03,     6.9846113169404574e-05,     7.1997643564116527e-08,    -1.8584362837971840e-10, } , 
      {    1.2097107015148524e+01,     1.3657848424438513e+04,    -5.8196476744655414e-05,     6.2097748195859803e-08,    -1.4541016737518314e-10, } , 
      {    1.1196851097871244e+01,     5.1196075338826640e+03,     2.2215886381075282e-04,     3.2240875799400293e-07,    -1.3721827060547530e-09, } , 
      {    1.4452661587400424e+01,     2.1986539849163637e+04,     9.4116273896692834e-05,     3.1250886262574621e-07,    -1.3317492450502176e-09, } , 
      {    1.3552405670123143e+01,     1.3448298958607791e+04,     3.7447161445210110e-04,     5.7281987242388930e-07,    -2.5585217837297872e-09, } , 
      {    1.6808216159652325e+01,     3.0315231273888763e+04,     2.4642902453804108e-04,     5.6291997705563258e-07,    -2.5180883227252519e-09, } , 
      {    1.9163770731904229e+01,     3.8643922698613897e+04,     3.9874177517938930e-04,     8.1333109148551900e-07,    -3.7044274004002870e-09, } , 
      {    2.7620847214114397e-01,    -2.7357306743938636e+04,    -8.2242411144430516e-04,    -1.1893414318394097e-06,     5.7318454927481986e-09, } , 
      {    2.6317630443930446e+00,    -1.9028615319213510e+04,    -6.7011136080295688e-04,    -9.3893031740952327e-07,     4.5455064150731643e-09, } , 
      {    1.7315071271157652e+00,    -2.7566856209769358e+04,    -3.8975602024754870e-04,    -6.7861930761138012e-07,     3.3187338763935956e-09, } , 
      {    4.9873176166449458e+00,    -1.0699923894488384e+04,    -5.1779861016160861e-04,    -6.8851920297963685e-07,     3.3591673373981309e-09, } , 
      {    4.0870616993676645e+00,    -1.9238164785044231e+04,    -2.3744326960620043e-04,    -4.2820819318149365e-07,     2.1323947987185609e-09, } , 
      {    7.3428721888968447e+00,    -2.3712324697632580e+03,    -3.6548585952026044e-04,    -4.3810808854975037e-07,     2.1728282597230963e-09, } , 
      {    1.0598682678426027e+01,     1.4495699845517716e+04,    -4.9352844943432040e-04,    -4.4800798391800710e-07,     2.2132617207276316e-09, } , 
      {    6.4426162716195652e+00,    -1.0909473360319105e+04,    -8.5130518964852191e-05,    -1.7779707875160725e-07,     9.4605572104352629e-10, } , 
      {    9.6984267611487454e+00,     5.9574589549618686e+03,    -2.1317310887891217e-04,    -1.8769697411986398e-07,     9.8648918204806139e-10, } , 
      {    1.2954237250677926e+01,     2.2824391270242842e+04,    -3.4121569879297218e-04,    -1.9759686948812070e-07,     1.0269226430525967e-09, } , 
      {    8.7981708438714659e+00,    -2.5807819355939796e+03,     6.7182231676496057e-05,     7.2614035678279181e-08,    -2.4028335663150822e-10, } , 
      {    1.2053981333400646e+01,     1.4286150379686995e+04,    -6.0860358237563932e-05,     6.2714140310022456e-08,    -1.9984989562697296e-10, } , 
      {    7.8979149265941864e+00,    -1.1119022826149827e+04,     3.4753757223190425e-04,     3.3292504547642234e-07,    -1.4670558953110779e-09, } , 
      {    1.1153725416123367e+01,     5.7479094891311470e+03,     2.1949498231784432e-04,     3.2302515010816561e-07,    -1.4266224343065428e-09, } , 
      {    1.4409535905652547e+01,     2.2614841804412121e+04,     9.1452392403784316e-05,     3.1312525473990889e-07,    -1.3861889733020075e-09, } , 
      {    1.3509279988375265e+01,     1.4076600913856273e+04,     3.7180773295919257e-04,     5.7343626453805198e-07,    -2.6129615119815769e-09, } , 
      {    1.6765090477904447e+01,     3.0943533229137247e+04,     2.4376514304513258e-04,     5.6353636916979526e-07,    -2.5725280509770416e-09, } , 
      {    1.5864834560627166e+01,     2.2405292338581399e+04,     5.2412048360054079e-04,     8.2384737896793840e-07,    -3.7993005896566124e-09, } , 
      {    1.9120645050156348e+01,     3.9272224653862373e+04,     3.9607789368648077e-04,     8.1394748359968168e-07,    -3.7588671286520770e-09, } , 
      {    2.1476199622408245e+01,     4.7600916078587499e+04,     5.4839064432782910e-04,     1.0643585980295681e-06,    -4.9452062063271113e-09, } , 
      {    2.3831754194660149e+01,     5.5929607503312625e+04,     7.0070339496917738e-04,     1.3147697124594545e-06,    -6.1315452840021455e-09, } , 
      {   -2.1224717818586329e+00,    -3.5057696213415278e+04,    -9.7740074357856180e-04,    -1.4391361541551334e-06,     6.8637448421714444e-09, } , 
      {    2.3308279039326574e-01,    -2.6729004788690156e+04,    -8.2508799293721374e-04,    -1.1887250397252470e-06,     5.6774057644964093e-09, } , 
      {   -6.6717312688401353e-01,    -3.5267245679246007e+04,    -5.4473265238180551e-04,    -9.2841402992710386e-07,     4.4506332258168398e-09, } , 
      {    2.5886373626451666e+00,    -1.8400313363965030e+04,    -6.7277524229586547e-04,    -9.3831392529536059e-07,     4.4910666868213751e-09, } , 
      {    1.6883814453678869e+00,    -2.6938554254520877e+04,    -3.9241990174045718e-04,    -6.7800291549721744e-07,     3.2642941481418055e-09, } , 
      {    4.9441919348970673e+00,    -1.0071621939239902e+04,    -5.2046249165451719e-04,    -6.8790281086547417e-07,     3.3047276091463408e-09, } , 
      {    8.2000024244262484e+00,     6.7953103760410722e+03,    -6.4850508156857715e-04,    -6.9780270623373089e-07,     3.3451610701508757e-09, } , 
      {    4.0439360176197869e+00,    -1.8609862829795751e+04,    -2.4010715109910896e-04,    -4.2759180106733102e-07,     2.0779550704667709e-09, } , 
      {    7.2997465071489671e+00,    -1.7429305145147757e+03,    -3.6814974101316892e-04,    -4.3749169643558775e-07,     2.1183885314713062e-09, } , 
      {    1.0555556996678147e+01,     1.5124001800766198e+04,    -4.9619233092722899e-04,    -4.4739159180384447e-07,     2.1588219924758415e-09, } , 
      {    6.3994905898716867e+00,    -1.0281171405070623e+04,    -8.7794400457760695e-05,    -1.7718068663744457e-07,     8.9161599279173642e-10, } , 
      {    9.6553010794008660e+00,     6.5857609102103506e+03,    -2.1583699037182070e-04,    -1.8708058200570130e-07,     9.3204945379627173e-10, } , 
      {    1.2911111568930046e+01,     2.3452693225491323e+04,    -3.4387958028588066e-04,    -1.9698047737395802e-07,     9.7248291480080704e-10, } , 
      {    5.4992346725944072e+00,    -1.8819412295626473e+04,     1.9256094009764753e-04,     8.3130323160698572e-08,    -3.3515654588783324e-10, } , 
      {    8.7550451621235865e+00,    -1.9524799803454973e+03,     6.4518350183587553e-05,     7.3230427792441848e-08,    -2.9472308488329798e-10, } , 
      {    1.2010855651652767e+01,     1.4914452334935477e+04,    -6.3524239730472436e-05,     6.3330532424185123e-08,    -2.5428962387876272e-10, } , 
      {    1.5266666141181947e+01,     3.1781384650216449e+04,    -1.9156682964453244e-04,     5.3430637055928398e-08,    -2.1385616287422749e-10, } , 
      {    7.8547892448463079e+00,    -1.0490720870901345e+04,     3.4487369073899578e-04,     3.3354143759058497e-07,    -1.5214956235628676e-09, } , 
      {    1.1110599734375487e+01,     6.3762114443796290e+03,     2.1683110082493579e-04,     3.2364154222232824e-07,    -1.4810621625583325e-09, } , 
      {    1.4366410223904667e+01,     2.3243143759660601e+04,     8.8788510910875812e-05,     3.1374164685407152e-07,    -1.4406287015537972e-09, } , 
      {    1.7622220713433848e+01,     4.0110076074941571e+04,    -3.9254079003184183e-05,     3.0384175148581479e-07,    -1.4001952405492619e-09, } , 
      {    1.0210343817098208e+01,    -2.1620294461762187e+03,     4.9718644138034400e-04,     5.8395255202047139e-07,    -2.7078347012379023e-09, } , 
      {    1.3466154306627386e+01,     1.4704902869104755e+04,     3.6914385146628409e-04,     5.7405265665221466e-07,    -2.6674012402333669e-09, } , 
      {    1.6721964796156570e+01,     3.1571835184385727e+04,     2.4110126155222405e-04,     5.6415276128395794e-07,    -2.6269677792288316e-09, } , 
      {    1.5821708878879287e+01,     2.3033594293829879e+04,     5.2145660210763220e-04,     8.2446377108210108e-07,    -3.8537403179084016e-09, } , 
      {    1.9077519368408467e+01,     3.9900526609110857e+04,     3.9341401219357229e-04,     8.1456387571384436e-07,    -3.8133068569038663e-09, } , 
      {    1.8177263451131189e+01,     3.1362285718555006e+04,     6.7376935274898048e-04,     1.0748748855119875e-06,    -5.0400793955834358e-09, } , 
      {    2.1433073940660364e+01,     4.8229218033835983e+04,     5.4572676283492052e-04,     1.0649749901437308e-06,    -4.9996459345789005e-09, } , 
      {    2.3788628512912268e+01,     5.6557909458561109e+04,     6.9803951347626879e-04,     1.3153861045736172e-06,    -6.1859850122539348e-09, } , 
      {    2.6144183085164165e+01,     6.4886600883286235e+04,     8.5035226411761696e-04,     1.5657972190035036e-06,    -7.3723240899289699e-09, } , 
      {   -4.5211520358584121e+00,    -4.2758085682891920e+04,    -1.1323773757128187e-03,    -1.6889308764708572e-06,     7.9956441915946886e-09, } , 
      {   -2.1655974636065114e+00,    -3.4429394258166794e+04,    -9.8006462507147039e-04,    -1.4385197620409707e-06,     6.8093051139196543e-09, } , 
      {   -3.0658533808837927e+00,    -4.2967635148722649e+04,    -6.9970928451606226e-04,    -1.1782087522428276e-06,     5.5825325752400839e-09, } , 
      {    1.8995710864538751e-01,    -2.6100702833441672e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {    3.4457675981745677e+00,    -9.2337705181606980e+03,    -9.5579446434418218e-04,    -1.1980085429793411e-06,     5.6633994972491546e-09, } , 
      {   -7.1029880863189199e-01,    -3.4638943723997523e+04,    -5.4739653387471398e-04,    -9.2779763781294118e-07,     4.3961934975650497e-09, } , 
      {    2.5455116808972882e+00,    -1.7772011408716546e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    5.8013221704264684e+00,    -9.0507909343557185e+02,    -8.0348171370283391e-04,    -9.4759742854945463e-07,     4.4770604195741203e-09, } , 
      {    1.6452557636200087e+00,    -2.6310252299272393e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    4.9010662531491889e+00,    -9.4433199839914196e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.1568767426783690e+00,     7.4236123312895543e+03,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {    1.1412687232207549e+01,     2.4290544646570532e+04,    -7.7921155297554570e-04,    -7.0708620948782494e-07,     3.3311548029036214e-09, } , 
      {    7.4499984634272831e-01,    -3.4848493189828238e+04,    -1.1472844267795747e-04,    -4.1707551358491162e-07,     1.9830818812104459e-09, } , 
      {    4.0008103358719085e+00,    -1.7981560874547267e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    7.2566208254010887e+00,    -1.1146285592662935e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0512431314930270e+01,     1.5752303756014680e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.3768241804459448e+01,     3.2619236071295658e+04,    -6.2689880233419743e-04,    -4.5667509505793851e-07,     2.1448157252285867e-09, } , 
      {    3.1005544185946285e+00,    -2.6519801765103115e+04,     3.7584307963390776e-05,    -1.6666439915502520e-07,     7.9674280353541145e-10, } , 
      {    6.3563649081238083e+00,    -9.6528694498221412e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.6121753976529885e+00,     7.2140628654588327e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2867985887182169e+01,     2.4080995180739807e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.4561089908465288e+00,    -1.8191110340377989e+04,     1.8989705860473902e-04,     8.3746715274861239e-08,    -3.8959627413962306e-10, } , 
      {    8.7119194803757090e+00,    -1.3241780250970151e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1967729969904889e+01,     1.5542754290183959e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5223540459434069e+01,     3.2409686605464933e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    7.8116635630984295e+00,    -9.8624189156528628e+03,     3.4220980924608724e-04,     3.3415782970474765e-07,    -1.5759353518146575e-09, } , 
      {    1.1067474052627610e+01,     7.0045133996281111e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.4323284542156790e+01,     2.3871445714909085e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7579095031685966e+01,     4.0738378030190055e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    1.0167218135350330e+01,    -1.5337274909277367e+03,     4.9452255988743552e-04,     5.8456894413463407e-07,    -2.7622744294896919e-09, } , 
      {    1.3423028624879509e+01,     1.5333204824353237e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    1.6678839114408689e+01,     3.2200137139634211e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9934649603937871e+01,     4.9067069454915181e+04,     1.1039479014525555e-04,     5.5486925802986389e-07,    -2.6409740464760860e-09, } , 
      {    1.2522772707602229e+01,     6.7949639337973895e+03,     6.4683531052878379e-04,     8.3498005856452049e-07,    -3.9486135071647270e-09, } , 
      {    1.5778583197131409e+01,     2.3661896249078363e+04,     5.1879272061472372e-04,     8.2508016319626376e-07,    -3.9081800461601917e-09, } , 
      {    1.9034393686660593e+01,     4.0528828564359341e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.8134137769383308e+01,     3.1990587673803489e+04,     6.7110547125607200e-04,     1.0754912776261502e-06,    -5.0945191238352259e-09, } , 
      {    2.1389948258912490e+01,     4.8857519989084467e+04,     5.4306288134201204e-04,     1.0655913822578935e-06,    -5.0540856628306906e-09, } , 
      {    2.0489692341635212e+01,     4.0319279098528612e+04,     8.2341822189742027e-04,     1.3259023920560366e-06,    -6.2808582015102602e-09, } , 
      {    2.3745502831164394e+01,     5.7186211413809593e+04,     6.9537563198336031e-04,     1.3160024966877799e-06,    -6.2404247405057249e-09, } , 
      {    2.6101057403416291e+01,     6.5514902838534719e+04,     8.4768838262470848e-04,     1.5664136111176663e-06,    -7.4267638181807599e-09, } , 
      {   -2.2087231453543898e+00,    -3.3801092302918310e+04,    -9.8272850656437897e-04,    -1.4379033699268081e-06,     6.7548653856678643e-09, } , 
      {    1.4683142689750928e-01,    -2.5472400878193188e+04,    -8.3041575592303070e-04,    -1.1874922554969216e-06,     5.5685263079928292e-09, } , 
      {   -7.5342449037977044e-01,    -3.4010641768749039e+04,    -5.5006041536762246e-04,    -9.2718124569877850e-07,     4.3417537693132596e-09, } , 
      {    2.5023859991494097e+00,    -1.7143709453468062e+04,    -6.7810300528168242e-04,    -9.3708114106703523e-07,     4.3821872303177949e-09, } , 
      {    1.6021300818721305e+00,    -2.5681950344023910e+04,    -3.9774766472627424e-04,    -6.7677013126889208e-07,     3.1554146916382262e-09, } , 
      {    4.8579405714013104e+00,    -8.8150180287429375e+03,    -5.2579025464033415e-04,    -6.8667002663714881e-07,     3.1958481526427615e-09, } , 
      {    8.1137510609304897e+00,     8.0519142865380363e+03,    -6.5383284455439411e-04,    -6.9656992200540553e-07,     3.2362816136472964e-09, } , 
      {    3.9576846541240300e+00,    -1.7353258919298783e+04,    -2.4543491408492596e-04,    -4.2635901683900566e-07,     1.9690756139631916e-09, } , 
      {    7.2134951436532102e+00,    -4.8632660401781118e+02,    -3.7347750399898598e-04,    -4.3625891220726239e-07,     2.0095090749677269e-09, } , 
      {    1.0469305633182392e+01,     1.6380605711263162e+04,    -5.0152009391304594e-04,    -4.4615880757551911e-07,     2.0499425359722622e-09, } , 
      {    6.3132392263759298e+00,    -9.0245674945736591e+03,    -9.3122163443577730e-05,    -1.7594790240911927e-07,     7.8273653628815689e-10, } , 
      {    9.5690497159051091e+00,     7.8423648207073147e+03,    -2.2116475335763771e-04,    -1.8584779777737599e-07,     8.2316999729269210e-10, } , 
      {    1.2824860205434291e+01,     2.4709297135988290e+04,    -3.4920734327169772e-04,    -1.9574769314563272e-07,     8.6360345829722741e-10, } , 
      {    5.4129833090986503e+00,    -1.7562808385129505e+04,     1.8723317711183052e-04,     8.4363107389023906e-08,    -4.4403600239141287e-10, } , 
      {    8.6687937986278314e+00,    -6.9587606984853278e+02,     5.9190587197770524e-05,     7.4463212020767181e-08,    -4.0360254138687761e-10, } , 
      {    1.1924604288157012e+01,     1.6171056245432441e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {    1.5180414777686192e+01,     3.3037988560713413e+04,    -1.9689459263034945e-04,     5.4663421284253732e-08,    -3.2273561937780709e-10, } , 
      {    7.7685378813505510e+00,    -9.2341169604043807e+03,     3.3954592775317871e-04,     3.3477422181891033e-07,    -1.6303750800664473e-09, } , 
      {    1.1024348370879732e+01,     7.6328153548765931e+03,     2.1150333783911878e-04,     3.2487432645065360e-07,    -1.5899416190619122e-09, } , 
      {    1.4280158860408912e+01,     2.4499747670157569e+04,     8.3460747925058777e-05,     3.1497443108239688e-07,    -1.5495081580573769e-09, } , 
      {    1.7535969349938092e+01,     4.1366679985438539e+04,    -4.4581841989001205e-05,     3.0507453571414015e-07,    -1.5090746970528416e-09, } , 
      {    1.0124092453602453e+01,    -9.0542553567925438e+02,     4.9185867839452704e-04,     5.8518533624879675e-07,    -2.8167141577414816e-09, } , 
      {    1.3379902943131631e+01,     1.5961506779601719e+04,     3.6381608848046703e-04,     5.7528544088054002e-07,    -2.7762806967369463e-09, } , 
      {    1.6635713432660815e+01,     3.2828439094882691e+04,     2.3577349856640704e-04,     5.6538554551228330e-07,    -2.7358472357324109e-09, } , 
      {    1.5735457515383532e+01,     2.4290198204326847e+04,     5.1612883912181525e-04,     8.2569655531042644e-07,    -3.9626197744119817e-09, } , 
      {    1.8991268004912712e+01,     4.1157130519607825e+04,     3.8808624920775523e-04,     8.1579665994216972e-07,    -3.9221863134074464e-09, } , 
      {    1.8091012087635434e+01,     3.2618889629051973e+04,     6.6844158976316352e-04,     1.0761076697403129e-06,    -5.1489588520870160e-09, } , 
      {    2.1346822577164609e+01,     4.9485821944332951e+04,     5.4039899984910356e-04,     1.0662077743720561e-06,    -5.1085253910824807e-09, } , 
      {    2.3702377149416513e+01,     5.7814513369058077e+04,     6.9271175049045184e-04,     1.3166188888019426e-06,    -6.2948644687575149e-09, } , 
      {    2.4592603174015317e+00,    -1.6515407498219582e+04,    -6.8076688677459101e-04,    -9.3646474895287255e-07,     4.3277475020660057e-09, } , 
      {    1.5590044001242522e+00,    -2.5053648388775429e+04,    -4.0041154621918272e-04,    -6.7615373915472940e-07,     3.1009749633864361e-09, } , 
      {    4.8148148896534320e+00,    -8.1867160734944555e+03,    -5.2845413613324273e-04,    -6.8605363452298613e-07,     3.1414084243909714e-09, } , 
      {    3.9145589723761520e+00,    -1.6724956964050303e+04,    -2.4809879557783450e-04,    -4.2574262472484303e-07,     1.9146358857114015e-09, } , 
      {    7.1703694619053326e+00,     1.4197535123067109e+02,    -3.7614138549189446e-04,    -4.3564252009309976e-07,     1.9550693467159368e-09, } , 
      {    1.0426179951434513e+01,     1.7008907666511644e+04,    -5.0418397540595453e-04,    -4.4554241546135648e-07,     1.9955028077204721e-09, } , 
      {    6.2701135446280514e+00,    -8.3962655393251771e+03,    -9.5786044936486234e-05,    -1.7533151029495659e-07,     7.2829680803636702e-10, } , 
      {    9.5259240341572315e+00,     8.4706667759557968e+03,    -2.2382863485054624e-04,    -1.8523140566321331e-07,     7.6873026904090223e-10, } , 
      {    1.2781734523686412e+01,     2.5337599091236771e+04,    -3.5187122476460620e-04,    -1.9513130103147004e-07,     8.0916373004543754e-10, } , 
      {    8.6256681168799521e+00,    -6.7574114600050507e+01,     5.6526705704862013e-05,     7.5079604134929848e-08,    -4.5804226963866738e-10, } , 
      {    1.1881478606409132e+01,     1.6799358200680923e+04,    -7.1515884209197975e-05,     6.5179708766673123e-08,    -4.1760880863413212e-10, } , 
      {    1.0981222689131853e+01,     8.2611173101250752e+03,     2.0883945634621025e-04,     3.2549071856481623e-07,    -1.6443813473137019e-09, } , 
      {    1.4237033178661033e+01,     2.5128049625406049e+04,     8.0796866432150273e-05,     3.1559082319655950e-07,    -1.6039478863091666e-09, } , 
      {    1.3336777261383752e+01,     1.6589808734850201e+04,     3.6115220698755855e-04,     5.7590183299470270e-07,    -2.8307204249887363e-09, } , 
      {    1.6592587750912934e+01,     3.3456741050131175e+04,     2.3310961707349851e-04,     5.6600193762644598e-07,    -2.7902869639842010e-09, } , 
      {    4.7716892079055544e+00,    -7.5584141182459725e+03,    -5.3111801762615121e-04,    -6.8543724240882345e-07,     3.0869686961391818e-09, } , 
      {    7.1272437801574542e+00,     7.7027730647915337e+02,    -3.7880526698480299e-04,    -4.3502612797893708e-07,     1.9006296184641471e-09, } , 
      {    6.2269878628801738e+00,    -7.7679635840766941e+03,    -9.8449926429394752e-05,    -1.7471511818079393e-07,     6.7385707978457726e-10, } , 
      {    9.4827983524093540e+00,     9.0989687312042788e+03,    -2.2649251634345474e-04,    -1.8461501354905066e-07,     7.1429054078911247e-10, } , 
      {    1.1838352924661255e+01,     1.7427660155929407e+04,    -7.4179765702106493e-05,     6.5796100880835777e-08,    -4.7204853688592199e-10, } , 
      {    9.4396726706614764e+00,     9.7272706864527609e+03,    -2.2915639783636324e-04,    -1.8399862143488798e-07,     6.5985081253732270e-10, } , 
      {    1.4983144946199497e+01,     1.2472232189556882e+04,    -2.4093940650478566e-04,    -1.5695629607901540e-07,     9.4100396248636494e-10, } , 
      {    1.7338699518451399e+01,     2.0800923614282008e+04,    -8.8626655863437387e-05,     9.3454818350871037e-08,    -2.4533511518866962e-10, } , 
      {    1.2584464692199720e+01,     4.7718427200802371e+03,    -3.9591603863904239e-04,    -4.0675101839473916e-07,     2.0729033119096095e-09, } , 
      {    1.4940019264451619e+01,     1.3100534144805364e+04,    -2.4360328799769417e-04,    -1.5633990396485272e-07,     8.8656423423457507e-10, } , 
      {    1.4039763347174340e+01,     4.5622932542495155e+03,     3.6752052557714098e-05,     1.0397110583329043e-07,    -3.4020830444499469e-10, } , 
      {    1.7295573836703518e+01,     2.1429225569530488e+04,    -9.1290537356345891e-05,     9.4071210465033704e-08,    -2.9977484344045938e-10, } , 
      {    1.9651128408955422e+01,     2.9757916994255615e+04,     6.1022213285002357e-05,     3.4448232489492010e-07,    -1.4861139211154938e-09, } , 
      {    1.0185784438199942e+01,    -2.9285467493964065e+03,    -5.5089267077329908e-04,    -6.5654574071046296e-07,     3.2048026613328545e-09, } , 
      {    1.2541339010451843e+01,     5.4001446753287200e+03,    -3.9857992013195092e-04,    -4.0613462628057648e-07,     2.0184635836578199e-09, } , 
      {    1.1641083093174561e+01,    -3.1380962152271281e+03,    -1.1822457957654267e-04,    -1.4582361648243334e-07,     7.9169104497824999e-10, } , 
      {    1.4896893582703742e+01,     1.3728836100053846e+04,    -2.4626716949060270e-04,    -1.5572351185069006e-07,     8.3212450598278531e-10, } , 
      {    1.3996637665426462e+01,     5.1905952094979984e+03,     3.4088171064805580e-05,     1.0458749794745308e-07,    -3.9464803269678451e-10, } , 
      {    1.7252448154955644e+01,     2.2057527524778972e+04,    -9.3954418849254408e-05,     9.4687602579196358e-08,    -3.5421457169224920e-10, } , 
      {    1.6352192237678363e+01,     1.3519286634223125e+04,     1.8640092170615384e-04,     3.5499861237733950e-07,    -1.5809871103718190e-09, } , 
      {    1.9608002727207541e+01,     3.0386218949504098e+04,     5.8358331792093839e-05,     3.4509871700908278e-07,    -1.5405536493672837e-09, } , 
      {    2.1963557299459438e+01,     3.8714910374229221e+04,     2.1067108243344210e-04,     5.9550983143896915e-07,    -2.7268927270423179e-09, } , 
      {    7.7871041842001629e+00,    -1.0628936218873050e+04,    -7.0586930290755595e-04,    -9.0634046302618670e-07,     4.3367020107560987e-09, } , 
      {    1.0142658756452063e+01,    -2.3002447941479240e+03,    -5.5355655226620767e-04,    -6.5592934859630028e-07,     3.1503629330810644e-09, } , 
      {    9.2424028391747832e+00,    -1.0838485684703772e+04,    -2.7320121171079938e-04,    -3.9561833879815713e-07,     1.9235903944014945e-09, } , 
      {    1.2498213328703963e+01,     6.0284466305772021e+03,    -4.0124380162485939e-04,    -4.0551823416641386e-07,     1.9640238554060298e-09, } , 
      {    1.5754023818233142e+01,     2.2895378945858174e+04,    -5.2928639153891946e-04,    -4.1541812953467058e-07,     2.0044573164105651e-09, } , 
      {    1.1597957411426682e+01,    -2.5097942599786456e+03,    -1.2088846106945117e-04,    -1.4520722436827066e-07,     7.3725131672646013e-10, } , 
      {    1.4853767900955862e+01,     1.4357138055302328e+04,    -2.4893105098351117e-04,    -1.5510711973652738e-07,     7.7768477773099544e-10, } , 
      {    1.3953511983678583e+01,     5.8188971647464805e+03,     3.1424289571897069e-05,     1.0520389006161575e-07,    -4.4908776094857428e-10, } , 
      {    1.7209322473207763e+01,     2.2685829480027453e+04,    -9.6618300342162913e-05,     9.5303994693359024e-08,    -4.0865429994403896e-10, } , 
      {    1.6309066555930485e+01,     1.4147588589471607e+04,     1.8373704021324531e-04,     3.5561500449150213e-07,    -1.6354268386236087e-09, } , 
      {    1.9564877045459660e+01,     3.1014520904752579e+04,     5.5694450299185329e-05,     3.4571510912324541e-07,    -1.5949933776190734e-09, } , 
      {    1.8664621128182382e+01,     2.2476280014196731e+04,     3.3604979085459361e-04,     6.0602611892138855e-07,    -2.8217659162986433e-09, } , 
      {    2.1920431617711564e+01,     3.9343212329477705e+04,     2.0800720094053357e-04,     5.9612622355313183e-07,    -2.7813324552941080e-09, } , 
      {    5.3884239302003838e+00,    -1.8329325688349694e+04,    -8.6084593504181270e-04,    -1.1561351853419104e-06,     5.4686013601793429e-09, } , 
      {    7.7439785024522845e+00,    -1.0000634263624568e+04,    -7.0853318440046442e-04,    -9.0572407091202402e-07,     4.2822622825043086e-09, } , 
      {    6.8437225851750050e+00,    -1.8538875154180416e+04,    -4.2817784384505619e-04,    -6.4541306111388087e-07,     3.0554897438247395e-09, } , 
      {    1.0099533074704185e+01,    -1.6719428388994420e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    1.3355343564233365e+01,     1.5194989476381532e+04,    -6.8426302367317611e-04,    -6.6521285185039432e-07,     3.1363566658338101e-09, } , 
      {    9.1992771574269057e+00,    -1.0210183729455290e+04,    -2.7586509320370791e-04,    -3.9500194668399445e-07,     1.8691506661497048e-09, } , 
      {    1.2455087646956086e+01,     6.6567485858256841e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.5710898136485264e+01,     2.3523680901106658e+04,    -5.3195027303182794e-04,    -4.1480173742050790e-07,     1.9500175881587754e-09, } , 
      {    1.1554831729678805e+01,    -1.8814923047301636e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.4810642219207985e+01,     1.4985440010550810e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.8066452708737167e+01,     3.1852372325831784e+04,    -3.7963752239047967e-04,    -1.6439062299062145e-07,     7.6367851048374099e-10, } , 
      {    1.0654575812401525e+01,    -1.0419733195286011e+04,     1.5680299799304855e-04,     1.1572017754403514e-07,    -5.4396095020489930e-10, } , 
      {    1.3910386301930705e+01,     6.4471991199949625e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.7166196791459885e+01,     2.3314131435275936e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.0422007280989064e+01,     4.0181063750556910e+04,    -2.2732477174913142e-04,     8.6020491439264966e-08,    -4.2266056719129352e-10, } , 
      {    1.3010130384653426e+01,    -2.0910417705608852e+03,     3.0911574863439677e-04,     3.6613129197392153e-07,    -1.7303000278799337e-09, } , 
      {    1.6265940874182604e+01,     1.4775890544720089e+04,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {    1.9521751363711786e+01,     3.1642822860001063e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.2777561853240968e+01,     4.8509755175282036e+04,    -7.5012021107783171e-05,     3.3643160586915136e-07,    -1.6089996448663279e-09, } , 
      {    1.8621495446434508e+01,     2.3104581969445215e+04,     3.3338590936168508e-04,     6.0664251103555123e-07,    -2.8762056445504330e-09, } , 
      {    2.1877305935963683e+01,     3.9971514284726189e+04,     2.0534331944762507e-04,     5.9674261566729451e-07,    -2.8357721835458977e-09, } , 
      {    2.4232860508215587e+01,     4.8300205709451315e+04,     3.5765607008897329e-04,     8.4715373009718093e-07,    -4.0221112612209328e-09, } , 
      {    2.6588415080467485e+01,     5.6628897134176441e+04,     5.0996882073032156e-04,     1.0975648445270673e-06,    -5.2084503388959670e-09, } , 
      {    5.3452982484525053e+00,    -1.7701023733101210e+04,    -8.6350981653472117e-04,    -1.1555187932277478e-06,     5.4141616319275528e-09, } , 
      {    7.7008528207044060e+00,    -9.3723323083760861e+03,    -7.1119706589337290e-04,    -9.0510767879786134e-07,     4.2278225542525185e-09, } , 
      {    1.0056407392956308e+01,    -1.0436408836509597e+03,    -5.5888431525202462e-04,    -6.5469656436797492e-07,     3.0414834765774851e-09, } , 
      {    1.3312217882485488e+01,     1.5823291431630014e+04,    -6.8692690516608459e-04,    -6.6459645973623164e-07,     3.0819169375820204e-09, } , 
      {    9.1561514756790263e+00,    -9.5818817742068077e+03,    -2.7852897469661644e-04,    -3.9438555456983177e-07,     1.8147109378979149e-09, } , 
      {    1.2411961965208208e+01,     7.2850505410741662e+03,    -4.0657156461067646e-04,    -4.0428544993808850e-07,     1.8551443989024503e-09, } , 
      {    1.5667772454737387e+01,     2.4151982856355142e+04,    -5.3461415452473642e-04,    -4.1418534530634522e-07,     1.8955778599069858e-09, } , 
      {    1.1511706047930927e+01,    -1.2531903494816813e+03,    -1.2621622405526819e-04,    -1.4397444013994535e-07,     6.2837186022288060e-10, } , 
      {    1.4767516537460107e+01,     1.5613741965799292e+04,    -2.5425881396932824e-04,    -1.5387433550820208e-07,     6.6880532122741591e-10, } , 
      {    1.8023327026989286e+01,     3.2480674281080268e+04,    -3.8230140388338820e-04,    -1.6377423087645880e-07,     7.0923878223195122e-10, } , 
      {    1.3867260620182828e+01,     7.0755010752434446e+03,     2.6096526586080048e-05,     1.0643667428994108e-07,    -5.5796721745215386e-10, } , 
      {    1.7123071109712008e+01,     2.3942433390524420e+04,    -1.0194606332797995e-04,     9.6536778921684358e-08,    -5.1753375644761854e-10, } , 
      {    1.6222815192434730e+01,     1.5404192499968571e+04,     1.7840927722742830e-04,     3.5684778871982749e-07,    -1.7443062951271884e-09, } , 
      {    1.9478625681963905e+01,     3.2271124815249546e+04,     5.0366687313368307e-05,     3.4694789335157077e-07,    -1.7038728341226531e-09, } , 
      {    2.1834180254215809e+01,     4.0599816239974672e+04,     2.0267943795471656e-04,     5.9735900778145719e-07,    -2.8902119117976873e-09, } , 
      {    2.4189734826467706e+01,     4.8928507664699799e+04,     3.5499218859606476e-04,     8.4777012221134361e-07,    -4.0765509894727228e-09, } , 
      {    1.0013281711208428e+01,    -4.1533892840247745e+02,    -5.6154819674493321e-04,    -6.5408017225381224e-07,     2.9870437483256950e-09, } , 
      {    1.2368836283460329e+01,     7.9133524963226482e+03,    -4.0923544610358493e-04,    -4.0366905782392587e-07,     1.8007046706506606e-09, } , 
      {    1.4724390855712228e+01,     1.6242043921047774e+04,    -2.5692269546223671e-04,    -1.5325794339403940e-07,     6.1436559297562604e-10, } , 
      {    1.3824134938434948e+01,     7.7038030304919266e+03,     2.3432645093171537e-05,     1.0705306640410375e-07,    -6.1240694570394373e-10, } , 
      {    1.7079945427964127e+01,     2.4570735345772900e+04,    -1.0460994482088845e-04,     9.7153171035847025e-08,    -5.7197348469940841e-10, } , 
      {    1.9435500000216031e+01,     3.2899426770498030e+04,     4.7702805820459796e-05,     3.4756428546573339e-07,    -1.7583125623744428e-09, } , 
      {    2.1791054572467928e+01,     4.1228118195223156e+04,     2.0001555646180803e-04,     5.9797539989561987e-07,    -2.9446516400494774e-09, } , 
      {    9.0699001121832694e+00,    -8.3252778637098436e+03,    -2.8385673768243345e-04,    -3.9315277034150647e-07,     1.7058314813943354e-09, } , 
      {    1.2325710601712451e+01,     8.5416544515711303e+03,    -4.1189932759649347e-04,    -4.0305266570976319e-07,     1.7462649423988707e-09, } , 
      {    1.4681265173964350e+01,     1.6870345876296258e+04,    -2.5958657695514524e-04,    -1.5264155127987674e-07,     5.5992586472383628e-10, } , 
      {    1.3781009256687071e+01,     8.3321049857404087e+03,     2.0768763600263023e-05,     1.0766945851826640e-07,    -6.6684667395573349e-10, } , 
      {    2.0224737449502371e+01,     1.9615307379400379e+04,    -2.7136958562356763e-04,    -1.2559922592400416e-07,     8.4107901467287841e-10, } , 
      {    2.2580292021754275e+01,     2.7943998804125506e+04,    -1.1905683498221935e-04,     1.2481188850588226e-07,    -3.4526006300215609e-10, } , 
      {    1.7826057195502592e+01,     1.1914917909923737e+04,    -4.2634621775782433e-04,    -3.7539394823972796e-07,     1.9729783640961228e-09, } , 
      {    2.0181611767754497e+01,     2.0243609334648863e+04,    -2.7403346711647611e-04,    -1.2498283380984151e-07,     7.8663928642108854e-10, } , 
      {    2.2537166340006394e+01,     2.8572300759373989e+04,    -1.2172071647512786e-04,     1.2542828062004491e-07,    -3.9969979125394591e-10, } , 
      {    2.4892720912258298e+01,     3.6900992184099116e+04,     3.0592034166220384e-05,     3.7583939504993131e-07,    -1.5860388689289802e-09, } , 
      {    1.5427376941502816e+01,     4.2145284404470922e+03,    -5.8132284989208108e-04,    -6.2518867055545175e-07,     3.1048777135193678e-09, } , 
      {    1.7782931513754718e+01,     1.2543219865172219e+04,    -4.2901009925073286e-04,    -3.7477755612556528e-07,     1.9185386358443331e-09, } , 
      {    1.6882675596477434e+01,     4.0049789746163706e+03,    -1.4865475869532463e-04,    -1.1446654632742210e-07,     6.9176609716476357e-10, } , 
      {    2.0138486086006615e+01,     2.0871911289897344e+04,    -2.7669734860938464e-04,    -1.2436644169567883e-07,     7.3219955816929878e-10, } , 
      {    1.9238230168729338e+01,     1.2333670399341498e+04,     3.6579919460236176e-06,     1.3594456810246432e-07,    -4.9457298051027088e-10, } , 
      {    2.2494040658258520e+01,     2.9200602714622470e+04,    -1.2438459796803637e-04,     1.2604467273420759e-07,    -4.5413951950573568e-10, } , 
      {    2.4849595230510417e+01,     3.7529294139347599e+04,     2.7928152673311877e-05,     3.7645578716409399e-07,    -1.6404785971807700e-09, } , 
      {    2.7205149802762321e+01,     4.5857985564072726e+04,     1.8024090331466013e-04,     6.2686690159398035e-07,    -2.8268176748558047e-09, } , 
      {    1.0673142115251137e+01,    -1.1814552453754677e+04,    -8.8861223266768611e-04,    -1.1253945073010619e-06,     5.4231161406176466e-09, } , 
      {    1.3028696687503038e+01,    -3.4858610290295514e+03,    -7.3629948202633784e-04,    -8.7498339287117549e-07,     4.2367770629426124e-09, } , 
      {    1.2128440770225756e+01,    -1.2024101919585399e+04,    -4.5594414147092965e-04,    -6.1467238307303234e-07,     3.0100045242630432e-09, } , 
      {    1.5384251259754938e+01,     4.8428303956955751e+03,    -5.8398673138498956e-04,    -6.2457227844128907e-07,     3.0504379852675781e-09, } , 
      {    1.8640061749284122e+01,     2.1709762710976549e+04,    -7.1202932129904952e-04,    -6.3447217380954579e-07,     3.0908714462721134e-09, } , 
      {    1.4483995342477655e+01,    -3.6954104948602730e+03,    -3.0363139082958138e-04,    -3.6426126864314587e-07,     1.8236654465880082e-09, } , 
      {    1.7739805832006837e+01,     1.3171521820420701e+04,    -4.3167398074364139e-04,    -3.7416116401140260e-07,     1.8640989075925435e-09, } , 
      {    2.0995616321536019e+01,     3.0038454135701675e+04,    -5.5971657065770136e-04,    -3.8406105937965932e-07,     1.9045323685970788e-09, } , 
      {    1.6839549914729556e+01,     4.6332809298648535e+03,    -1.5131864018823313e-04,    -1.1385015421325945e-07,     6.3732636891297380e-10, } , 
      {    2.0095360404258734e+01,     2.1500213245145827e+04,    -2.7936123010229317e-04,    -1.2375004958151618e-07,     6.7775982991750901e-10, } , 
      {    2.3351170893787916e+01,     3.8367145560426798e+04,    -4.0740382001635314e-04,    -1.3364994494977290e-07,     7.1819329092204422e-10, } , 
      {    1.9195104486981457e+01,     1.2961972354589980e+04,     9.9411045311510506e-07,     1.3656096021662697e-07,    -5.4901270876206065e-10, } , 
      {    2.2450914976510639e+01,     2.9828904669870954e+04,    -1.2704847946094487e-04,     1.2666106484837025e-07,    -5.0857924775752544e-10, } , 
      {    2.1550659059233354e+01,     2.1290663779315106e+04,     1.5330686109446336e-04,     3.8697207464651339e-07,    -1.7353517864370952e-09, } , 
      {    2.4806469548762536e+01,     3.8157596094596076e+04,     2.5264271180403363e-05,     3.7707217927825667e-07,    -1.6949183254325599e-09, } , 
      {    2.3906213631485258e+01,     2.9619355204040232e+04,     3.0561961173581161e-04,     6.3738318907639976e-07,    -2.9216908641121297e-09, } , 
      {    2.7162024121014440e+01,     4.6486287519321202e+04,     1.7757702182175162e-04,     6.2748329370814303e-07,    -2.8812574031075943e-09, } , 
      {    2.9517578693266337e+01,     5.4814978944046328e+04,     3.2988977246309982e-04,     8.7789440813802946e-07,    -4.0675964807826290e-09, } , 
      {    8.2744618612513605e+00,    -1.9514941923231323e+04,    -1.0435888648019428e-03,    -1.3751892296167857e-06,     6.5550154900408924e-09, } , 
      {    1.0630016433503258e+01,    -1.1186250498506195e+04,    -8.9127611416059470e-04,    -1.1247781151868992e-06,     5.3686764123658574e-09, } , 
      {    9.7297605162259764e+00,    -1.9724491389062045e+04,    -6.1092077360518646e-04,    -8.6446710538875609e-07,     4.1419038736862878e-09, } , 
      {    1.2985571005755158e+01,    -2.8575590737810689e+03,    -7.3896336351924642e-04,    -8.7436700075701281e-07,     4.1823373346908231e-09, } , 
      {    1.2085315088477877e+01,    -1.1395799964336917e+04,    -4.5860802296383813e-04,    -6.1405599095886966e-07,     2.9555647960112532e-09, } , 
      {    1.5341125578007059e+01,     5.4711323509440572e+03,    -5.8665061287789815e-04,    -6.2395588632712639e-07,     2.9959982570157880e-09, } , 
      {    1.8596936067536241e+01,     2.2338064666225029e+04,    -7.1469320279195811e-04,    -6.3385578169538311e-07,     3.0364317180203234e-09, } , 
      {    1.4440869660729776e+01,    -3.0671085396117905e+03,    -3.0629527232248986e-04,    -3.6364487652898324e-07,     1.7692257183362185e-09, } , 
      {    1.7696680150258956e+01,     1.3799823775669183e+04,    -4.3433786223654987e-04,    -3.7354477189723997e-07,     1.8096591793407536e-09, } , 
      {    2.0952490639788138e+01,     3.0666756090950155e+04,    -5.6238045215060994e-04,    -3.8344466726549669e-07,     1.8500926403452889e-09, } , 
      {    1.3540613743452498e+01,    -1.1605349430167638e+04,    -2.5939931767081667e-05,    -1.0333386673084006e-07,     5.4245317965664862e-10, } , 
      {    1.6796424232981678e+01,     5.2615828851133356e+03,    -1.5398252168114166e-04,    -1.1323376209909678e-07,     5.8288664066118393e-10, } , 
      {    2.0052234722510860e+01,     2.2128515200394308e+04,    -2.8202511159520165e-04,    -1.2313365746735350e-07,     6.2332010166571914e-10, } , 
      {    2.3308045212040042e+01,     3.8995447515675281e+04,    -4.1006770150926161e-04,    -1.3303355283561022e-07,     6.6375356267025435e-10, } , 
      {    1.5896168315704399e+01,    -3.2766580054425121e+03,     1.2637281887426657e-04,     1.4707724769904638e-07,    -6.4388589801838583e-10, } , 
      {    1.9151978805233576e+01,     1.3590274309838462e+04,    -1.6697710397934066e-06,     1.3717735233078965e-07,    -6.0345243701385052e-10, } , 
      {    2.2407789294762757e+01,     3.0457206625119434e+04,    -1.2971236095385340e-04,     1.2727745696253293e-07,    -5.6301897600931531e-10, } , 
      {    2.5663599784291939e+01,     4.7324138940400408e+04,    -2.5775495086791334e-04,     1.1737756159427619e-07,    -5.2258551500478010e-10, } , 
      {    1.8251722887956298e+01,     5.0520334192826140e+03,     2.7868556951561482e-04,     3.9748836212893274e-07,    -1.8302249756934202e-09, } , 
      {    2.1507533377485480e+01,     2.1918965734563586e+04,     1.5064297960155483e-04,     3.8758846676067602e-07,    -1.7897915146888849e-09, } , 
      {    2.4763343867014662e+01,     3.8785898049844560e+04,     2.2600389687494852e-05,     3.7768857139241929e-07,    -1.7493580536843496e-09, } , 
      {    2.3863087949737377e+01,     3.0247657159288712e+04,     3.0295573024290314e-04,     6.3799958119056244e-07,    -2.9761305923639197e-09, } , 
      {    2.7118898439266559e+01,     4.7114589474569686e+04,     1.7491314032884309e-04,     6.2809968582230571e-07,    -2.9356971313593844e-09, } , 
      {    2.6218642521989274e+01,     3.8576348584013838e+04,     4.5526848088425130e-04,     8.8841069562044886e-07,    -4.1624696700389536e-09, } , 
      {    2.9474453011518456e+01,     5.5443280899294812e+04,     3.2722589097019134e-04,     8.7851080025219214e-07,    -4.1220362090344183e-09, } , 
      {    3.1830007583770360e+01,     6.3771972324019938e+04,     4.7953864161153962e-04,     1.1289219146820786e-06,    -5.3083752867094525e-09, } , 
      {    5.8757816072515805e+00,    -2.7215331392707965e+04,    -1.1985654969361996e-03,    -1.6249839519325094e-06,     7.6869148394641358e-09, } , 
      {    8.2313361795034812e+00,    -1.8886639967982839e+04,    -1.0462527462948513e-03,    -1.3745728375026230e-06,     6.5005757617891024e-09, } , 
      {    7.3310802622261981e+00,    -2.7424880858538687e+04,    -7.6589740573944321e-04,    -1.1142618277044798e-06,     5.2738032231095320e-09, } , 
      {    1.0586890751755380e+01,    -1.0557948543257713e+04,    -8.9393999565350317e-04,    -1.1241617230727366e-06,     5.3142366841140673e-09, } , 
      {    9.6866348344780988e+00,    -1.9096189433813561e+04,    -6.1358465509809494e-04,    -8.6385071327459341e-07,     4.0874641454344977e-09, } , 
      {    1.2942445324007281e+01,    -2.2292571185325869e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    1.6198255813536463e+01,     1.4637675196748387e+04,    -8.6966983492621486e-04,    -8.8365050401110686e-07,     4.1683310674435684e-09, } , 
      {    1.2042189406729999e+01,    -1.0767498009088435e+04,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {    1.5297999896259181e+01,     6.0994343061925392e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.8553810385788367e+01,     2.2966366621473513e+04,    -7.1735708428486659e-04,    -6.3323938958122043e-07,     2.9819919897685337e-09, } , 
      {    1.1141933489452720e+01,    -1.9305738899644282e+04,    -1.8091656390133843e-04,    -3.5312858904656384e-07,     1.6743525290798933e-09, } , 
      {    1.4397743978981898e+01,    -2.4388065843633085e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    1.7653554468511082e+01,     1.4428125730917665e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0909364958040264e+01,     3.1295058046198639e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    1.3497488061704621e+01,    -1.0977047474919156e+04,    -2.8603813259990178e-05,    -1.0271747461667739e-07,     4.8801345140485886e-10, } , 
      {    1.6753298551233797e+01,     5.8898848403618176e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.0009109040762979e+01,     2.2756817155642791e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.3264919530292161e+01,     3.9623749470923765e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    1.5853042633956521e+01,    -2.6483560501940301e+03,     1.2370893738135807e-04,     1.4769363981320903e-07,    -6.9832562627017560e-10, } , 
      {    1.9108853123485702e+01,     1.4218576265086944e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.2364663613014883e+01,     3.1085508580367918e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5620474102544065e+01,     4.7952440895648891e+04,    -2.6041883236082187e-04,     1.1799395370843885e-07,    -5.7702524325656986e-10, } , 
      {    1.8208597206208424e+01,     5.6803353745310960e+03,     2.7602168802270629e-04,     3.9810475424309542e-07,    -1.8846647039452100e-09, } , 
      {    2.1464407695737599e+01,     2.2547267689812070e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {    2.4720218185266781e+01,     3.9414200005093044e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.7976028674795963e+01,     5.6281132320374018e+04,    -1.0810608171947362e-04,     3.6840506813832525e-07,    -1.7633643209316043e-09, } , 
      {    2.3819962267989503e+01,     3.0875959114537196e+04,     3.0029184874999460e-04,     6.3861597330472512e-07,    -3.0305703206157094e-09, } , 
      {    2.7075772757518685e+01,     4.7742891429818170e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    2.6175516840241400e+01,     3.9204650539262322e+04,     4.5260459939134277e-04,     8.8902708773461154e-07,    -4.2169093982907436e-09, } , 
      {    2.9431327329770582e+01,     5.6071582854543296e+04,     3.2456200947728281e-04,     8.7912719236635482e-07,    -4.1764759372862083e-09, } , 
      {    2.8531071412493304e+01,     4.7533341963987448e+04,     6.0491735003269105e-04,     1.1394382021644980e-06,    -5.4032484759657779e-09, } , 
      {    3.1786881902022486e+01,     6.4400274279268422e+04,     4.7687476011863108e-04,     1.1295383067962412e-06,    -5.3628150149612426e-09, } , 
      {    3.4142436474274383e+01,     7.2728965703993541e+04,     6.2918751075997936e-04,     1.3799494212261277e-06,    -6.5491540926362768e-09, } , 
      {    1.0543765070007503e+01,    -9.9296465880092310e+03,    -8.9660387714641165e-04,    -1.1235453309585739e-06,     5.2597969558622772e-09, } , 
      {    1.2899319642259403e+01,    -1.6009551632841046e+03,    -7.4429112650506338e-04,    -8.7313421652868745e-07,     4.0734578781872430e-09, } , 
      {    1.5254874214511304e+01,     6.7277362614410213e+03,    -5.9197837586371510e-04,    -6.2272310209880103e-07,     2.8871188005122087e-09, } , 
      {    1.8510684704040486e+01,     2.3594668576721997e+04,    -7.2002096577777506e-04,    -6.3262299746705775e-07,     2.9275522615167440e-09, } , 
      {    1.4354618297234021e+01,    -1.8105046291148262e+03,    -3.1162303530830692e-04,    -3.6241209230065788e-07,     1.6603462618326388e-09, } , 
      {    1.7610428786763201e+01,     1.5056427686166147e+04,    -4.3966562522236693e-04,    -3.7231198766891461e-07,     1.7007797228371739e-09, } , 
      {    2.0866239276292383e+01,     3.1923360001447123e+04,    -5.6770821513642690e-04,    -3.8221188303717133e-07,     1.7412131838417092e-09, } , 
      {    1.6710172869485923e+01,     6.5181867956102997e+03,    -1.5931028466695867e-04,    -1.1200097787077145e-07,     4.7400718415760441e-10, } , 
      {    1.9965983359015105e+01,     2.3385119110891275e+04,    -2.8735287458101871e-04,    -1.2190087323902819e-07,     5.1444064516213961e-10, } , 
      {    2.3221793848544287e+01,     4.0252051426172249e+04,    -4.1539546449507867e-04,    -1.3180076860728491e-07,     5.5487410616667482e-10, } , 
      {    1.9065727441737820e+01,     1.4846878220335426e+04,    -6.9975340256104300e-06,     1.3841013655911496e-07,    -7.1233189351743005e-10, } , 
      {    2.2321537931267002e+01,     3.1713810535616401e+04,    -1.3504012393967041e-04,     1.2851024119085823e-07,    -6.7189843251289484e-10, } , 
      {    2.5577348420796184e+01,     4.8580742850897375e+04,    -2.6308271385373040e-04,     1.1861034582260152e-07,    -6.3146497150835963e-10, } , 
      {    1.8165471524460543e+01,     6.3086373297795781e+03,     2.7335780652979776e-04,     3.9872114635725810e-07,    -1.9391044321969997e-09, } , 
      {    2.1421282013989725e+01,     2.3175569645060554e+04,     1.4531521661573783e-04,     3.8882125098900138e-07,    -1.8986709711924644e-09, } , 
      {    2.4677092503518907e+01,     4.0042501960341528e+04,     1.7272626701677830e-05,     3.7892135562074465e-07,    -1.8582375101879293e-09, } , 
      {    2.3776836586241622e+01,     3.1504261069785680e+04,     2.9762796725708607e-04,     6.3923236541888780e-07,    -3.0850100488674991e-09, } , 
      {    2.7032647075770804e+01,     4.8371193385066654e+04,     1.6958537734302609e-04,     6.2933247005063107e-07,    -3.0445765878629637e-09, } , 
      {    2.9388201648022701e+01,     5.6699884809791780e+04,     3.2189812798437428e-04,     8.7974358448051750e-07,    -4.2309156655379984e-09, } , 
      {    3.1743756220274605e+01,     6.5028576234516906e+04,     4.7421087862572255e-04,     1.1301546989104039e-06,    -5.4172547432130327e-09, } , 
      {    1.2856193960511524e+01,    -9.7265320803562236e+02,    -7.4695500799797196e-04,    -8.7251782441452477e-07,     4.0190181499354537e-09, } , 
      {    1.1955938043234243e+01,    -9.5108940985914705e+03,    -4.6659966744256367e-04,    -6.1220681461638162e-07,     2.7922456112558838e-09, } , 
      {    1.5211748532763425e+01,     7.3560382166895033e+03,    -5.9464225735662369e-04,    -6.2210670998463835e-07,     2.8326790722604187e-09, } , 
      {    1.4311492615486141e+01,    -1.1822026738663440e+03,    -3.1428691680121540e-04,    -3.6179570018649526e-07,     1.6059065335808491e-09, } , 
      {    1.7567303105015323e+01,     1.5684729641414629e+04,    -4.4232950671527541e-04,    -3.7169559555475198e-07,     1.6463399945853842e-09, } , 
      {    1.6667047187738042e+01,     7.1464887508587817e+03,    -1.6197416615986720e-04,    -1.1138458575660878e-07,     4.1956745590581459e-10, } , 
      {    1.9922857677267224e+01,     2.4013421066139756e+04,    -2.9001675607392719e-04,    -1.2128448112486551e-07,     4.6000091691034980e-10, } , 
      {    1.9022601759989946e+01,     1.5475180175583908e+04,    -9.6614155185189416e-06,     1.3902652867327764e-07,    -7.6677162176921992e-10, } , 
      {    2.2278412249519128e+01,     3.2342112490864882e+04,    -1.3770400543257894e-04,     1.2912663330502091e-07,    -7.2633816076468471e-10, } , 
      {    2.4633966821771025e+01,     4.0670803915590011e+04,     1.4608745208769318e-05,     3.7953774773490728e-07,    -1.9126772384397192e-09, } , 
      {    1.7524177423267446e+01,     1.6313031596663111e+04,    -4.4499338820818394e-04,    -3.7107920344058930e-07,     1.5919002663335943e-09, } , 
      {    1.9879731995519343e+01,     2.4641723021388239e+04,    -2.9268063756683572e-04,    -1.2066808901070285e-07,     4.0556118865855998e-10, } , 
      {    2.2235286567771247e+01,     3.2970414446113362e+04,    -1.4036788692548745e-04,     1.2974302541918357e-07,    -7.8077788901647447e-10, } , 
      {    2.2981398335309713e+01,     2.0314597010264199e+04,    -4.6210415986242334e-04,    -3.4280409385639139e-07,     1.7641739597790569e-09, } , 
      {    2.0582718081309935e+01,     1.2614207540787556e+04,    -6.1708079199668004e-04,    -5.9259881617211518e-07,     2.8960733092023021e-09, } , 
      {    2.2938272653561832e+01,     2.0942898965512682e+04,    -4.6476804135533187e-04,    -3.4218770174222871e-07,     1.7097342315272671e-09, } , 
      {    2.2038016736284547e+01,     1.2404658074956835e+04,    -1.8441270079992361e-04,    -8.1876691944085562e-08,     4.8296169284769741e-10, } , 
      {    2.5293827225813729e+01,     2.9271590390237809e+04,    -3.1245529071398365e-04,    -9.1776587312342287e-08,     5.2339515385223272e-10, } , 
      {    2.7649381798065633e+01,     3.7600281814962931e+04,    -1.6014254007263535e-04,     1.5863452711754413e-07,    -6.6294392382280173e-10, } , 
      {    3.0004936370317530e+01,     4.5928973239688057e+04,    -7.8297894312871123e-06,     4.0904564154743055e-07,    -1.8492830014978363e-09, } , 
      {    1.8184037827310149e+01,     4.9138180713109123e+03,    -7.7205742413093690e-04,    -8.4239353848783892e-07,     4.0279726586255467e-09, } , 
      {    2.0539592399562054e+01,     1.3242509496036038e+04,    -6.1974467348958862e-04,    -5.9198242405795250e-07,     2.8416335809505121e-09, } , 
      {    1.9639336482284769e+01,     4.7042686054801907e+03,    -3.3938933293418033e-04,    -3.3167141425980936e-07,     1.6148610422709421e-09, } , 
      {    2.2895146971813951e+01,     2.1571200920761163e+04,    -4.6743192284824035e-04,    -3.4157130962806608e-07,     1.6552945032754774e-09, } , 
      {    2.1994891054536673e+01,     1.3032960030205317e+04,    -1.8707658229283214e-04,    -8.1260299829922895e-08,     4.2852196459590759e-10, } , 
      {    2.5250701544065855e+01,     2.9899892345486289e+04,    -3.1511917220689213e-04,    -9.1160195198179620e-08,     4.6895542560044285e-10, } , 
      {    2.4350445626788570e+01,     2.1361651454930441e+04,    -3.4763831651483884e-05,     1.6915081459996354e-07,    -7.5781711307912692e-10, } , 
      {    2.7606256116317752e+01,     3.8228583770211415e+04,    -1.6280642156554388e-04,     1.5925091923170681e-07,    -7.1738365207459160e-10, } , 
      {    2.6706000199040474e+01,     2.9690342879655567e+04,     1.1754891898986437e-04,     4.1956192902984991e-07,    -1.9441561907541615e-09, } , 
      {    2.9961810688569656e+01,     4.6557275194936541e+04,    -1.0493670924195625e-05,     4.0966203366159318e-07,    -1.9037227297496262e-09, } , 
      {    1.8140912145562275e+01,     5.5421200265593943e+03,    -7.7472130562384538e-04,    -8.4177714637367624e-07,     3.9735329303737567e-09, } , 
      {    1.7240656228284994e+01,    -2.9961208639964534e+03,    -4.9436596506843714e-04,    -5.8146613657553310e-07,     2.7467603916941871e-09, } , 
      {    2.0496466717814180e+01,     1.3870811451284520e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    2.3752277207343361e+01,     3.0737743766565494e+04,    -7.5045114489655706e-04,    -6.0126592731204655e-07,     2.8276273137032577e-09, } , 
      {    1.9596210800536895e+01,     5.3325705607286727e+03,    -3.4205321442708886e-04,    -3.3105502214564668e-07,     1.5604213140191522e-09, } , 
      {    2.2852021290066077e+01,     2.2199502876009647e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    2.1951765372788792e+01,     1.3661261985453799e+04,    -1.8974046378574064e-04,    -8.0643907715760229e-08,     3.7408223634411777e-10, } , 
      {    2.5207575862317974e+01,     3.0528194300734773e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {    2.4307319945040696e+01,     2.1989953410178925e+04,    -3.7427713144392395e-05,     1.6976720671412619e-07,    -8.1225684133091668e-10, } , 
      {    2.7563130434569878e+01,     3.8856885725459899e+04,    -1.6547030305845238e-04,     1.5986731134586947e-07,    -7.7182338032638137e-10, } , 
      {    2.6662874517292593e+01,     3.0318644834904051e+04,     1.1488503749695585e-04,     4.2017832114401259e-07,    -1.9985959190059511e-09, } , 
      {    2.9918685006821775e+01,     4.7185577150185025e+04,    -1.3157552417104136e-05,     4.1027842577575586e-07,    -1.9581624580014158e-09, } , 
      {    3.2274239579073679e+01,     5.5514268574910151e+04,     1.3915519822424411e-04,     6.6068954020564228e-07,    -3.1445015356764505e-09, } , 
      {    1.8097786463814394e+01,     6.1704219818078764e+03,    -7.7738518711675385e-04,    -8.4116075425951356e-07,     3.9190932021219666e-09, } , 
      {    2.0453341036066298e+01,     1.4499113406533002e+04,    -6.2507243647540558e-04,    -5.9074963982962714e-07,     2.7327541244469327e-09, } , 
      {    2.2808895608318196e+01,     2.2827804831258130e+04,    -4.7275968583405741e-04,    -3.4033852539974072e-07,     1.5464150467718977e-09, } , 
      {    2.1908639691040918e+01,     1.4289563940702281e+04,    -1.9240434527864915e-04,    -8.0027515601597562e-08,     3.1964250809232796e-10, } , 
      {    2.5164450180570100e+01,     3.1156496255983257e+04,    -3.2044693519270919e-04,    -8.9927410969854286e-08,     3.6007596909686327e-10, } , 
      {    2.4264194263292815e+01,     2.2618255365427409e+04,    -4.0091594637300906e-05,     1.7038359882828885e-07,    -8.6669656958270645e-10, } , 
      {    2.7520004752821997e+01,     3.9485187680708383e+04,    -1.6813418455136089e-04,     1.6048370346003212e-07,    -8.2626310857817113e-10, } , 
      {    2.9875559325073901e+01,     4.7813879105433509e+04,    -1.5821433910012646e-05,     4.1089481788991854e-07,    -2.0126021862532055e-09, } , 
      {    3.2231113897325798e+01,     5.6142570530158635e+04,     1.3649131673133561e-04,     6.6130593231980496e-07,    -3.1989412639282401e-09, } , 
      {    2.5121324498822219e+01,     3.1784798211231737e+04,    -3.2311081668561767e-04,    -8.9311018855691620e-08,     3.0563624084507350e-10, } , 
      {    2.7476879071074123e+01,     4.0113489635956867e+04,    -1.7079806604426942e-04,     1.6110009557419480e-07,    -8.8070283682996100e-10, } , 
      {    2.8222990838612589e+01,     2.7457672200107692e+04,    -4.9253433898120539e-04,    -3.1144702370138013e-07,     1.6642490119655706e-09, } , 
      {    3.0578545410864486e+01,     3.5786363624832818e+04,    -3.4022158833985706e-04,    -6.1035909271493692e-08,     4.7790993429053606e-10, } , 
      {    2.5824310584612810e+01,     1.9757282730631046e+04,    -6.4751097111546204e-04,    -5.6124174601710397e-07,     2.7961483613888154e-09, } , 
      {    2.8179865156864707e+01,     2.8085974155356173e+04,    -4.9519822047411387e-04,    -3.1083063158721745e-07,     1.6098092837137807e-09, } , 
      {    3.0535419729116612e+01,     3.6414665580081302e+04,    -3.4288546983276560e-04,    -6.0419517157331026e-08,     4.2347020603874629e-10, } , 
      {    3.2890974301368509e+01,     4.4743357004806428e+04,    -1.9057271919141732e-04,     1.8999159727255540e-07,    -7.6286887163628816e-10, } , 
      {    3.5246528873620413e+01,     5.3072048429531555e+04,    -3.8259968550069073e-05,     4.4040271170244182e-07,    -1.9492079493113224e-09, } , 
      {    2.3425630330613032e+01,     1.2056893261154404e+04,    -8.0248760324971879e-04,    -8.1103646833282771e-07,     3.9280477108120596e-09, } , 
      {    2.5781184902864929e+01,     2.0385584685879530e+04,    -6.5017485260837051e-04,    -5.6062535390294129e-07,     2.7417086331370257e-09, } , 
      {    2.4880928985587651e+01,     1.1847343795323683e+04,    -3.6981951205296233e-04,    -3.0031434410479804e-07,     1.5149360944574556e-09, } , 
      {    2.8136739475116826e+01,     2.8714276110604656e+04,    -4.9786210196702235e-04,    -3.1021423947305477e-07,     1.5553695554619909e-09, } , 
      {    2.7236483557839549e+01,     2.0176035220048809e+04,    -2.1750676141161408e-04,    -4.9903229674911641e-08,     3.2859701678242116e-10, } , 
      {    3.0492294047368731e+01,     3.7042967535329779e+04,    -3.4554935132567413e-04,    -5.9803125043168372e-08,     3.6903047778695648e-10, } , 
      {    2.9592038130091453e+01,     2.8504726644773935e+04,    -6.5194010770265850e-05,     2.0050788475497477e-07,    -8.5774206089261324e-10, } , 
      {    3.2847848619620628e+01,     4.5371658960054905e+04,    -1.9323660068432583e-04,     1.9060798938671805e-07,    -8.1730859988807793e-10, } , 
      {    3.5203403191872532e+01,     5.3700350384780031e+04,    -4.0923850042977591e-05,     4.4101910381660450e-07,    -2.0036476775631121e-09, } , 
      {    3.7558957764124429e+01,     6.2029041809505157e+04,     1.1138890059837066e-04,     6.9143021824649081e-07,    -3.1899867552381467e-09, } , 
      {    2.3382504648865151e+01,     1.2685195216402886e+04,    -8.0515148474262738e-04,    -8.1042007621866503e-07,     3.8736079825602703e-09, } , 
      {    2.2482248731587873e+01,     4.1469543258470385e+03,    -5.2479614418721914e-04,    -5.5010906642052189e-07,     2.6468354438807004e-09, } , 
      {    2.5738059221117048e+01,     2.1013886641128011e+04,    -6.5283873410127910e-04,    -5.6000896178877861e-07,     2.6872689048852357e-09, } , 
      {    2.8993869710646230e+01,     3.7880818956408984e+04,    -7.8088132401533906e-04,    -5.6990885715703534e-07,     2.7277023658897710e-09, } , 
      {    2.4837803303839770e+01,     1.2475645750572165e+04,    -3.7248339354587081e-04,    -2.9969795199063541e-07,     1.4604963662056659e-09, } , 
      {    2.8093613793368952e+01,     2.9342578065853137e+04,    -5.0052598345993093e-04,    -3.0959784735889214e-07,     1.5009298272102012e-09, } , 
      {    3.1349424282898127e+01,     4.6209510381134111e+04,    -6.2856857337399089e-04,    -3.1949774272714886e-07,     1.5413632882147365e-09, } , 
      {    2.7193357876091675e+01,     2.0804337175297289e+04,    -2.2017064290452262e-04,    -4.9286837560748981e-08,     2.7415728853063140e-10, } , 
      {    3.0449168365620849e+01,     3.7671269490578263e+04,    -3.4821323281858260e-04,    -5.9186732929005705e-08,     3.1459074953516671e-10, } , 
      {    2.9548912448343572e+01,     2.9133028600022415e+04,    -6.7857892263174354e-05,     2.0112427686913745e-07,    -9.1218178914440311e-10, } , 
      {    3.2804722937872754e+01,     4.5999960915303389e+04,    -1.9590048217723436e-04,     1.9122438150088073e-07,    -8.7174832813986779e-10, } , 
      {    3.5160277510124658e+01,     5.4328652340028515e+04,    -4.3587731535886102e-05,     4.4163549593076712e-07,    -2.0580874058149021e-09, } , 
      {    3.7515832082376555e+01,     6.2657343764753641e+04,     1.0872501910546215e-04,     6.9204661036065349e-07,    -3.2444264834899368e-09, } , 
      {    3.9871386654628459e+01,     7.0986035189478775e+04,     2.6103776974681039e-04,     9.4245772479053991e-07,    -4.4307655611649710e-09, } , 
      {    2.0983824394865373e+01,     4.9848057469262421e+03,    -9.6012811687688413e-04,    -1.0602147985343888e-06,     5.0055073319835145e-09, } , 
      {    2.3339378967117277e+01,     1.3313497171651368e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {    2.6595189456646452e+01,     3.0180429486932342e+04,    -9.3585795614959581e-04,    -8.1970357947275908e-07,     3.8596017153130156e-09, } , 
      {    2.2439123049839999e+01,     4.7752562810955205e+03,    -5.2746002568012762e-04,    -5.4949267430635921e-07,     2.5923957156289107e-09, } , 
      {    2.5694933539369174e+01,     2.1642188596376494e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    2.8950744028898356e+01,     3.8509120911657468e+04,    -7.8354520550824754e-04,    -5.6929246504287266e-07,     2.6732626376379813e-09, } , 
      {    2.4794677622091896e+01,     1.3103947705820647e+04,    -3.7514727503877934e-04,    -2.9908155987647273e-07,     1.4060566379538760e-09, } , 
      {    2.8050488111621071e+01,     2.9970880021101621e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    3.1306298601150253e+01,     4.6837812336382594e+04,    -6.3123245486689937e-04,    -3.1888135061298618e-07,     1.4869235599629467e-09, } , 
      {    2.7150232194343793e+01,     2.1432639130545773e+04,    -2.2283452439743112e-04,    -4.8670445446586314e-08,     2.1971756027884158e-10, } , 
      {    3.0406042683872975e+01,     3.8299571445826747e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.3661853173402157e+01,     5.5166503761107720e+04,    -4.7891970422555110e-04,    -6.8470236183099763e-08,     3.0058448228791221e-10, } , 
      {    2.6249976277066516e+01,     1.2894398239989925e+04,     5.7520816157977117e-05,     2.1164056435155683e-07,    -1.0070549784007282e-09, } , 
      {    2.9505786766595698e+01,     2.9761330555270899e+04,    -7.0521773756082872e-05,     2.0174066898330011e-07,    -9.6662151739619287e-10, } , 
      {    3.2761597256124873e+01,     4.6628262870551873e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    3.5117151828376777e+01,     5.4956954295276999e+04,    -4.6251613028794612e-05,     4.4225188804492980e-07,    -2.1125271340666918e-09, } , 
      {    3.4216895911099499e+01,     4.6418713404721151e+04,     2.3410372752661362e-04,     7.0256289784307290e-07,    -3.3392996727462618e-09, } , 
      {    3.7472706400628674e+01,     6.3285645720002125e+04,     1.0606113761255364e-04,     6.9266300247481617e-07,    -3.2988662117417265e-09, } , 
      {    3.9828260972880578e+01,     7.1614337144727251e+04,     2.5837388825390186e-04,     9.4307411690470259e-07,    -4.4852052894167611e-09, } , 
      {    4.2183815545132475e+01,     7.9943028569452377e+04,     4.1068663889525013e-04,     1.1934852313345890e-06,    -5.6715443670917954e-09, } , 
      {    2.3296253285369396e+01,     1.3941799126899850e+04,    -8.1047924772844433e-04,    -8.0918729199033967e-07,     3.7647285260566902e-09, } , 
      {    2.5651807857621293e+01,     2.2270490551624978e+04,    -6.5816649708709605e-04,    -5.5877617756045325e-07,     2.5783894483816563e-09, } , 
      {    2.4751551940344015e+01,     1.3732249661069129e+04,    -3.7781115653168787e-04,    -2.9846516776231005e-07,     1.3516169097020862e-09, } , 
      {    2.8007362429873197e+01,     3.0599181976350104e+04,    -5.0585374644574789e-04,    -3.0836506313056678e-07,     1.3920503707066215e-09, } , 
      {    2.7107106512595919e+01,     2.2060941085794257e+04,    -2.2549840589033962e-04,    -4.8054053332423647e-08,     1.6527783202705179e-10, } , 
      {    3.0362917002125094e+01,     3.8927873401075231e+04,    -3.5354099580439967e-04,    -5.7953948700680372e-08,     2.0571129303158710e-10, } , 
      {    2.9462661084847817e+01,     3.0389632510519383e+04,    -7.3185655248991389e-05,     2.0235706109746276e-07,    -1.0210612456479827e-09, } , 
      {    3.2718471574376998e+01,     4.7256564825800357e+04,    -2.0122824516305136e-04,     1.9245716572920604e-07,    -9.8062778464344743e-10, } , 
      {    3.5074026146628896e+01,     5.5585256250525483e+04,    -4.8915494521703123e-05,     4.4286828015909248e-07,    -2.1669668623184815e-09, } , 
      {    3.7429580718880793e+01,     6.3913947675250609e+04,     1.0339725611964512e-04,     6.9327939458897885e-07,    -3.3533059399935161e-09, } , 
      {    2.3253127603621522e+01,     1.4570101082148332e+04,    -8.1314312922135291e-04,    -8.0857089987617699e-07,     3.7102887978049005e-09, } , 
      {    2.5608682175873419e+01,     2.2898792506873458e+04,    -6.6083037858000464e-04,    -5.5815978544629057e-07,     2.5239497201298663e-09, } , 
      {    2.7964236748125316e+01,     3.1227483931598585e+04,    -5.0851762793865647e-04,    -3.0774867101640415e-07,     1.3376106424548318e-09, } , 
      {    3.0319791320377220e+01,     3.9556175356323714e+04,    -3.5620487729730814e-04,    -5.7337556586517712e-08,     1.5127156477979731e-10, } , 
      {    2.8580971470420145e+01,     2.0456572361494866e+04,    -8.3824554535431785e-04,    -7.7844661394949115e-07,     3.7192433064949935e-09, } , 
      {    3.0936526042672050e+01,     2.8785263786219992e+04,    -6.8593279471296958e-04,    -5.2803549951960472e-07,     2.5329042288199593e-09, } , 
      {    3.3292080614923947e+01,     3.7113955210945118e+04,    -5.3362004407162130e-04,    -2.7762438508971825e-07,     1.3465651511449248e-09, } , 
      {    3.5647635187175858e+01,     4.5442646635670244e+04,    -3.8130729343027308e-04,    -2.7213270659831814e-08,     1.6022607346989029e-10, } , 
      {    2.8537845788672271e+01,     2.1084874316743349e+04,    -8.4090942684722633e-04,    -7.7783022183532847e-07,     3.6648035782432039e-09, } , 
      {    3.0893400360924176e+01,     2.9413565741468476e+04,    -6.8859667620587805e-04,    -5.2741910740544204e-07,     2.4784645005681696e-09, } , 
      {    2.9993144443646891e+01,     2.0875324850912628e+04,    -4.0824133565046982e-04,    -2.6710809760729885e-07,     1.2516919618885996e-09, } , 
      {    3.3248954933176073e+01,     3.7742257166193602e+04,    -5.3628392556452978e-04,    -2.7700799297555557e-07,     1.2921254228931349e-09, } , 
      {    3.2348699015898788e+01,     2.9204016275637754e+04,    -2.5592858500912160e-04,    -1.6696983177412426e-08,     6.5352884213565184e-11, } , 
      {    3.5604509505427977e+01,     4.6070948590918728e+04,    -3.8397117492318161e-04,    -2.6596878545669150e-08,     1.0578634521810050e-10, } , 
      {    3.7960064077679874e+01,     5.4399640015643854e+04,    -2.3165842428183334e-04,     2.2381423588421727e-07,    -1.0805527324569340e-09, } , 
      {    3.0850274679176295e+01,     3.0041867696716959e+04,    -6.9126055769878653e-04,    -5.2680271529127936e-07,     2.4240247723163799e-09, } , 
      {    3.3205829251428192e+01,     3.8370559121442086e+04,    -5.3894780705743826e-04,    -2.7639160086139289e-07,     1.2376856946413451e-09, } , 
      {    3.5561383823680096e+01,     4.6699250546167212e+04,    -3.8663505641609014e-04,    -2.5980486431506487e-08,     5.1346616966310699e-11, } , 
      {    3.3822563973723021e+01,     2.7599647551338367e+04,    -8.6867572447309974e-04,    -7.4708954379447994e-07,     3.6193183586815072e-09, } , 
      {    3.6178118545974925e+01,     3.5928338976063489e+04,    -7.1636297383175147e-04,    -4.9667842936459352e-07,     2.4329792810064729e-09, } , 
      {    3.8533673118226822e+01,     4.4257030400788615e+04,    -5.6405022319040330e-04,    -2.4626731493470699e-07,     1.2466402033314383e-09, } , 
      {    4.0889227690478727e+01,     5.2585721825513741e+04,    -4.1173747254905508e-04,     4.1437994951794093e-09,     6.0301125656403900e-11, } , 
      {    4.3244782262730624e+01,     6.0914413250238867e+04,    -2.5942472190770681e-04,     2.5455491392506585e-07,    -1.1260379520186306e-09, } , 
      {    3.3779438291975147e+01,     2.8227949506586847e+04,    -8.7133960596600833e-04,    -7.4647315168031726e-07,     3.5648786304297171e-09, } , 
      {    3.6134992864227051e+01,     3.6556640931311973e+04,    -7.1902685532466005e-04,    -4.9606203725043084e-07,     2.3785395527546829e-09, } , 
      {    3.5234736946949766e+01,     2.8018400040756125e+04,    -4.3867151476925176e-04,    -2.3575102745228761e-07,     1.1517670140751133e-09, } , 
      {    3.8490547436478948e+01,     4.4885332356037099e+04,    -5.6671410468331189e-04,    -2.4565092282054436e-07,     1.1922004750796486e-09, } , 
      {    3.7590291519201671e+01,     3.6347091465481251e+04,    -2.8635876412790354e-04,     1.4660086977598799e-08,    -3.4572063599921210e-11, } , 
      {    4.0846102008730853e+01,     5.3214023780762225e+04,    -4.1440135404196356e-04,     4.7601916093420736e-09,     5.8613974046141025e-12, } , 
      {    4.3201656580982750e+01,     6.1542715205487351e+04,    -2.6208860340061528e-04,     2.5517130603922848e-07,    -1.1804776802704203e-09, } , 
      {    4.5557211153234647e+01,     6.9871406630212485e+04,    -1.0977585275926705e-04,     5.0558242046911485e-07,    -2.3668167579454549e-09, } , 
      {    3.1380758037975369e+01,     2.0527560037110205e+04,    -1.0263162381002650e-03,    -9.9626787399604100e-07,     4.6967779798529617e-09, } , 
      {    3.3736312610227266e+01,     2.8856251461835331e+04,    -8.7400348745891681e-04,    -7.4585675956615458e-07,     3.5104389021779275e-09, } , 
      {    3.2836056692949988e+01,     2.0318010571279483e+04,    -5.9364814690350857e-04,    -4.8554574976801143e-07,     2.2836663634983579e-09, } , 
      {    3.6091867182479170e+01,     3.7184942886560457e+04,    -7.2169073681756853e-04,    -4.9544564513626816e-07,     2.3240998245028932e-09, } , 
      {    3.5191611265201885e+01,     2.8646701996004609e+04,    -4.4133539626216029e-04,    -2.3513463533812496e-07,     1.0973272858233234e-09, } , 
      {    3.8447421754731067e+01,     4.5513634311285583e+04,    -5.6937798617622036e-04,    -2.4503453070638168e-07,     1.1377607468278588e-09, } , 
      {    3.7547165837453790e+01,     3.6975393420729735e+04,    -2.8902264562081207e-04,     1.5276479091761463e-08,    -8.9011791851711007e-11, } , 
      {    4.0802976326982972e+01,     5.3842325736010709e+04,    -4.1706523553487209e-04,     5.3765837235047379e-09,    -4.8578330847175695e-11, } , 
      {    3.9902720409705687e+01,     4.5304084845454861e+04,    -1.3670989497946383e-04,     2.6568759352164788e-07,    -1.2753508695267455e-09, } , 
      {    4.3158530899234869e+01,     6.2171017160735835e+04,    -2.6475248489352381e-04,     2.5578769815339116e-07,    -1.2349174085222101e-09, } , 
      {    4.5514085471486766e+01,     7.0499708585460961e+04,    -1.1243973425217557e-04,     5.0619881258327753e-07,    -2.4212564861972446e-09, } , 
      {    4.7869640043738670e+01,     7.8828400010186087e+04,     3.9873016389172682e-05,     7.5660992701316395e-07,    -3.6075955638722788e-09, } , 
      {    3.3693186928479385e+01,     2.9484553417083815e+04,    -8.7666736895182528e-04,    -7.4524036745199190e-07,     3.4559991739261378e-09, } , 
      {    3.6048741500731289e+01,     3.7813244841808941e+04,    -7.2435461831047701e-04,    -4.9482925302210548e-07,     2.2696600962511035e-09, } , 
      {    3.8404296072983186e+01,     4.6141936266534067e+04,    -5.7204186766912884e-04,    -2.4441813859221900e-07,     1.0833210185760689e-09, } , 
      {    4.0759850645235090e+01,     5.4470627691259193e+04,    -4.1972911702778062e-04,     5.9929758376674022e-09,    -1.0301805909896549e-10, } , 
      {    4.3115405217486988e+01,     6.2799319115984319e+04,    -2.6741636638643234e-04,     2.5640409026755384e-07,    -1.2893571367740000e-09, } , 
      {    4.6531926507337040e+01,     5.2099395221495935e+04,    -7.8521497654804101e-04,    -4.3211511271208301e-07,     2.0698102006241305e-09, } , 
      {    4.8887481079588937e+01,     6.0428086646221062e+04,    -6.3290222590669284e-04,    -1.8170399828219656e-07,     8.8347112294909593e-10, } , 
      {    4.1777691681085358e+01,     3.6070314327294167e+04,    -1.0925043593236461e-03,    -9.3232094945769322e-07,     4.3880486277224089e-09, } , 
      {    4.4133246253337255e+01,     4.4399005752019293e+04,    -9.4019160868229776e-04,    -6.8190983502780680e-07,     3.2017095500473751e-09, } , 
      {    4.6488800825589159e+01,     5.2727697176744419e+04,    -7.8787885804094948e-04,    -4.3149872059792033e-07,     2.0153704723723408e-09, } , 
      {    4.8844355397841056e+01,     6.1056388601469545e+04,    -6.3556610739960132e-04,    -1.8108760616803391e-07,     8.2903139469730617e-10, } , 
      {    5.1199909970092961e+01,     6.9385080026194672e+04,    -4.8325335675825304e-04,     6.9323508261852514e-08,    -3.5730768297772828e-10, } , 
    } ,  
    {  
      {    3.2558104868142230e+00,     1.6866932314983202e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    6.5116209736284461e+00,     3.3733864629966403e+04,    -2.5602466150593443e-04,    -2.1380865113347869e-08,    -1.1771276177339540e-10, } , 
      {    9.7674314604426691e+00,     5.0600796944949601e+04,    -3.8403699225890165e-04,    -3.2071297670021804e-08,    -1.7656914266009310e-10, } , 
      {   -7.4118769100558666e+00,    -4.2272105520326149e+04,     5.3632940002122783e-04,     2.8279017182467651e-07,    -1.0594148559605577e-09, } , 
      {   -4.1560664232416435e+00,    -2.5405173205342951e+04,     4.0831706926826062e-04,     2.7209973926800261e-07,    -1.1182712368472555e-09, } , 
      {   -9.0025593642742052e-01,    -8.5382408903597516e+03,     2.8030473851529340e-04,     2.6140930671132867e-07,    -1.1771276177339532e-09, } , 
      {    2.3555545503868025e+00,     8.3286914246234501e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6113650372010255e+00,     2.5195623739606650e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    8.8671755240152486e+00,     4.2062556054589855e+04,    -1.0373225374360825e-04,     2.2933800904130685e-07,    -1.3536967603940461e-09, } , 
      {    1.2122986010829472e+01,     5.8929488369573053e+04,    -2.3174458449657547e-04,     2.1864757648463293e-07,    -1.4125531412807440e-09, } , 
      {   -1.8005118728548410e+00,    -1.7076481780719503e+04,     5.6060947703058680e-04,     5.2281861342265733e-07,    -2.3542552354679064e-09, } , 
      {    1.4552986139593820e+00,    -2.0954946573630150e+02,     4.3259714627761958e-04,     5.1212818086598339e-07,    -2.4131116163546038e-09, } , 
      {    4.7111091007736050e+00,     1.6657382849246900e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.9669195875878280e+00,     3.3524315164230102e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.1222730074402051e+01,     5.0391247479213300e+04,     4.8560154018717928e-05,     4.8005688319596154e-07,    -2.5896807590146970e-09, } , 
      {    1.4478540561216274e+01,     6.7258179794196505e+04,    -7.9452176734249290e-05,     4.6936645063928765e-07,    -2.6485371399013948e-09, } , 
      {    5.5504267753196146e-01,    -8.7477903560960549e+03,     7.1290188479291292e-04,     7.7353748757731205e-07,    -3.5902392340885572e-09, } , 
      {    3.8108531643461845e+00,     8.1191419588871468e+03,     5.8488955403994576e-04,     7.6284705502063811e-07,    -3.6490956149752546e-09, } , 
      {    7.0666636511604075e+00,     2.4986074273870348e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0322474137974631e+01,     4.1853006588853546e+04,     3.2886489253401133e-04,     7.4146618990729021e-07,    -3.7668083767486503e-09, } , 
      {    1.3578284624788854e+01,     5.8719938903836752e+04,     2.0085256178104411e-04,     7.3077575735061626e-07,    -3.8256647576353482e-09, } , 
      {    6.1664077147329870e+00,     1.6447833383510599e+04,     7.3718196180227189e-04,     1.0135659291752927e-06,    -4.8850796135959055e-09, } , 
      {    9.4222182015472100e+00,     3.3314765698493800e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.2678028688361433e+01,     5.0181698013476998e+04,     4.8115730029633751e-04,     9.9218506406194504e-07,    -5.0027923753693012e-09, } , 
      {    1.5933839175175656e+01,     6.7048630328460204e+04,     3.5314496954337029e-04,     9.8149463150527098e-07,    -5.0616487562559990e-09, } , 
      {    8.5219622651197895e+00,     2.4776524808134051e+04,     8.8947436956459801e-04,     1.2642848033299474e-06,    -6.1210636122165563e-09, } , 
      {    1.1777772751934013e+01,     4.1643457123117252e+04,     7.6146203881163085e-04,     1.2535943707732736e-06,    -6.1799199931032542e-09, } , 
      {    1.5033583238748236e+01,     5.8510389438100450e+04,     6.3344970805866369e-04,     1.2429039382165998e-06,    -6.2387763739899520e-09, } , 
      {    1.8289393725562459e+01,     7.5377321753083655e+04,     5.0543737730569642e-04,     1.2322135056599257e-06,    -6.2976327548766499e-09, } , 
      {    1.0877516815506592e+01,     3.3105216232757492e+04,     1.0417667773269244e-03,     1.5150036774846022e-06,    -7.3570476108372071e-09, } , 
      {    1.4133327302320815e+01,     4.9972148547740697e+04,     9.1375444657395709e-04,     1.5043132449279283e-06,    -7.4159039917239050e-09, } , 
      {    1.7389137789135038e+01,     6.6839080862723902e+04,     7.8574211582098982e-04,     1.4936228123712545e-06,    -7.4747603726106028e-09, } , 
      {    1.6488881852707618e+01,     5.8300839972364149e+04,     1.0660468543362833e-03,     1.7550321190825830e-06,    -8.6518879903445566e-09, } , 
      {    1.9744692339521841e+01,     7.5167772287347354e+04,     9.3803452358331616e-04,     1.7443416865259092e-06,    -8.7107443712312545e-09, } , 
      {    1.8844436403094420e+01,     6.6629531396987601e+04,     1.2183392620986094e-03,     2.0057509932372378e-06,    -9.8878719889652066e-09, } , 
      {   -1.6532007540564148e+01,    -5.7672538017150087e+04,    -1.0687107358291919e-03,    -1.7544157269684204e-06,     8.5974482620927674e-09, } , 
      {   -1.4176452990177346e+01,    -4.9343846592526636e+04,    -9.1641832806686556e-04,    -1.5036968528137656e-06,     7.3614642634721149e-09, } , 
      {   -1.5076708926604766e+01,    -5.7882087482886389e+04,    -6.3611358955157217e-04,    -1.2422875461024371e-06,     6.1843366457381619e-09, } , 
      {   -1.1820898439790543e+01,    -4.1015155167903191e+04,    -7.6412592030453933e-04,    -1.2529779786591109e-06,     6.1254802648514641e-09, } , 
      {   -1.2721154376217964e+01,    -4.9553396058262937e+04,    -4.8382118178924604e-04,    -9.9156867194778236e-07,     4.9483526471175111e-09, } , 
      {   -9.4653438894037407e+00,    -3.2686463743279739e+04,    -6.1183351254221320e-04,    -1.0022591045044562e-06,     4.8894962662308132e-09, } , 
      {   -6.2095334025895177e+00,    -1.5819531428296537e+04,    -7.3984584329518036e-04,    -1.0129495370611300e-06,     4.8306398853441154e-09, } , 
      {   -1.3621410312645384e+01,    -5.8091636948622690e+04,    -2.0351644327395261e-04,    -7.3015936523645358e-07,     3.7712250293835581e-09, } , 
      {   -1.0365599825831161e+01,    -4.1224704633639485e+04,    -3.3152877402691986e-04,    -7.4084979779312753e-07,     3.7123686484968607e-09, } , 
      {   -7.1097893390169382e+00,    -2.4357772318656287e+04,    -4.5954110477988707e-04,    -7.5154023034980148e-07,     3.6535122676101628e-09, } , 
      {   -3.8539788522027152e+00,    -7.4908400036730864e+03,    -5.8755343553285424e-04,    -7.6223066290647543e-07,     3.5946558867234650e-09, } , 
      {   -1.1265855762258582e+01,    -4.9762945523999239e+04,    -5.1224035511626439e-05,    -4.7944049108179886e-07,     2.5352410307629073e-09, } , 
      {   -8.0100452754443587e+00,    -3.2896013209016040e+04,    -1.7923636626459365e-04,    -4.9013092363847281e-07,     2.4763846498762099e-09, } , 
      {   -4.7542347886301357e+00,    -1.6029080894032839e+04,    -3.0724869701756089e-04,    -5.0082135619514676e-07,     2.4175282689895120e-09, } , 
      {   -1.4984243018159125e+00,     8.3785142095036190e+02,    -4.3526102777052811e-04,    -5.1151178875182071e-07,     2.3586718881028141e-09, } , 
      {    1.7573861849983106e+00,     1.7704783735933564e+04,    -5.6327335852349527e-04,    -5.2220222130849465e-07,     2.2998155072161167e-09, } , 
      {   -8.9103012118717793e+00,    -4.1434254099375794e+04,     1.0106837225069974e-04,    -2.2872161692714420e-07,     1.2992570321422563e-09, } , 
      {   -5.6544907250575562e+00,    -2.4567321784392589e+04,    -2.6943958502267475e-05,    -2.3941204948381809e-07,     1.2404006512555586e-09, } , 
      {   -2.3986802382433332e+00,    -7.7003894694093897e+03,    -1.5495628925523469e-04,    -2.5010248204049204e-07,     1.1815442703688610e-09, } , 
      {    8.5713024857089004e-01,     9.1665428455738111e+03,    -2.8296862000820193e-04,    -2.6079291459716599e-07,     1.1226878894821633e-09, } , 
      {    4.1129407353851128e+00,     2.6033475160557013e+04,    -4.1098095076116915e-04,    -2.7148334715383993e-07,     1.0638315085954657e-09, } , 
      {   -6.5547466614849768e+00,    -3.3105562674752342e+04,     2.5336078001302590e-04,     2.1997257227510533e-08,     6.3273033521605610e-11, } , 
      {   -3.2989361746707537e+00,    -1.6238630359769140e+04,     1.2534844926005871e-04,     1.1306824670836598e-08,     4.4166526349079032e-12, } , 
      {   -4.3125687856530481e-02,     6.2830195521406040e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    3.2126847989576923e+00,     1.7495234270197263e+04,    -1.3067621224587572e-04,    -1.0074040442511271e-08,    -1.1329610913848749e-10, } , 
      {    6.4684952857719153e+00,     3.4362166585180465e+04,    -2.5868854299884297e-04,    -2.0764472999185206e-08,    -1.7215249002518519e-10, } , 
      {   -4.1991921110981743e+00,    -2.4776871250128890e+04,     4.0565318777535208e-04,     2.7271613138216529e-07,    -1.1727109650990454e-09, } , 
      {   -9.4338162428395100e-01,    -7.9099389351456912e+03,     2.7764085702238487e-04,     2.6202569882549135e-07,    -1.2315673459857430e-09, } , 
      {    2.3124288625302718e+00,     8.9569933798375096e+03,     1.4962852626941768e-04,     2.5133526626881740e-07,    -1.2904237268724407e-09, } , 
      {    5.5682393493444948e+00,     2.5823925694820711e+04,     2.1616195516450453e-05,     2.4064483371214345e-07,    -1.3492801077591383e-09, } , 
      {    8.8240498361587179e+00,     4.2690858009803917e+04,    -1.0639613523651677e-04,     2.2995440115546950e-07,    -1.4081364886458360e-09, } , 
      {   -1.8436375607113715e+00,    -1.6448179825505442e+04,     5.5794559553767832e-04,     5.2343500553682001e-07,    -2.4086949637196960e-09, } , 
      {    1.4121729261028515e+00,     4.1875248947775890e+02,     4.2993326478471105e-04,     5.1274457298014607e-07,    -2.4675513446063935e-09, } , 
      {    4.6679834129170743e+00,     1.7285684804460961e+04,     3.0192093403174383e-04,     5.0205414042347212e-07,    -2.5264077254930913e-09, } , 
      {    7.9237938997312973e+00,     3.4152617119444163e+04,     1.7390860327877664e-04,     4.9136370786679817e-07,    -2.5852641063797892e-09, } , 
      {    1.1179604386545520e+01,     5.1019549434427361e+04,     4.5896272525809417e-05,     4.8067327531012422e-07,    -2.6441204872664866e-09, } , 
      {    3.7677274764896538e+00,     8.7474439141012081e+03,     5.8222567254703728e-04,     7.6346344713480079e-07,    -3.7035353432270443e-09, } , 
      {    7.0235379633038768e+00,     2.5614376229084410e+04,     4.5421334179407001e-04,     7.5277301457812684e-07,    -3.7623917241137426e-09, } , 
      {    1.0279348450118100e+01,     4.2481308544067608e+04,     3.2620101104110280e-04,     7.4208258202145289e-07,    -3.8212481050004404e-09, } , 
      {    1.3535158936932323e+01,     5.9348240859050813e+04,     1.9818868028813560e-04,     7.3139214946477894e-07,    -3.8801044858871383e-09, } , 
      {    6.1232820268764563e+00,     1.7076135338724660e+04,     7.3451808030936341e-04,     1.0141823212894554e-06,    -4.9395193418476955e-09, } , 
      {    9.3790925136906793e+00,     3.3943067653707862e+04,     6.0650574955639625e-04,     1.0034918887327816e-06,    -4.9983757227343934e-09, } , 
      {    1.2634903000504902e+01,     5.0809999968691060e+04,     4.7849341880342898e-04,     9.9280145617610772e-07,    -5.0572321036210913e-09, } , 
      {    8.4788365772632588e+00,     2.5404826763348112e+04,     8.8681048807168954e-04,     1.2649011954441101e-06,    -6.1755033404683464e-09, } , 
      {    1.1734647064077482e+01,     4.2271759078331314e+04,     7.5879815731872237e-04,     1.2542107628874363e-06,    -6.2343597213550442e-09, } , 
      {    1.4990457550891705e+01,     5.9138691393314512e+04,     6.3078582656575521e-04,     1.2435203303307624e-06,    -6.2932161022417421e-09, } , 
      {    1.4090201614464284e+01,     5.0600450502954758e+04,     9.1109056508104861e-04,     1.5049296370420910e-06,    -7.4703437199756942e-09, } , 
      {    1.6445756164851087e+01,     5.8929141927578210e+04,     1.0633829728433747e-03,     1.7556485111967457e-06,    -8.7063277185963459e-09, } , 
      {   -1.1864024127647074e+01,    -4.0386853212689130e+04,    -7.6678980179744791e-04,    -1.2523615865449482e-06,     6.0710405365996748e-09, } , 
      {   -1.2764280064074494e+01,    -4.8925094103048876e+04,    -4.8648506328215452e-04,    -9.9095227983361968e-07,     4.8939129188657219e-09, } , 
      {   -9.5084695772602714e+00,    -3.2058161788065678e+04,    -6.1449739403512179e-04,    -1.0016427123902935e-06,     4.8350565379790240e-09, } , 
      {   -1.0408725513687692e+01,    -4.0596402678425424e+04,    -3.3419265551982833e-04,    -7.4023340567896485e-07,     3.6579289202450706e-09, } , 
      {   -7.1529150268734689e+00,    -2.3729470363442226e+04,    -4.6220498627279555e-04,    -7.5092383823563880e-07,     3.5990725393583728e-09, } , 
      {   -3.8971045400592454e+00,    -6.8625380484590260e+03,    -5.9021731702576282e-04,    -7.6161427079231275e-07,     3.5402161584716749e-09, } , 
      {   -8.0531709633008894e+00,    -3.2267711253801979e+04,    -1.8190024775750218e-04,    -4.8951453152431013e-07,     2.4219449216244198e-09, } , 
      {   -4.7973604764866664e+00,    -1.5400778938818779e+04,    -3.0991257851046937e-04,    -5.0020496408098408e-07,     2.3630885407377219e-09, } , 
      {   -1.5415499896724429e+00,     1.4661533761644223e+03,    -4.3792490926343659e-04,    -5.1089539663765803e-07,     2.3042321598510241e-09, } , 
      {   -5.6976164129140869e+00,    -2.3939019829178527e+04,    -2.9607839995175986e-05,    -2.3879565736965546e-07,     1.1859609230037689e-09, } , 
      {   -2.4418059260998635e+00,    -7.0720875141953293e+03,    -1.5762017074814322e-04,    -2.4948608992632941e-07,     1.1271045421170713e-09, } , 
      {    8.1400456071435956e-01,     9.7948448007878724e+03,    -2.8563250150111041e-04,    -2.6017652248300336e-07,     1.0682481612303737e-09, } , 
      {   -3.3420618625272840e+00,    -1.5610328404555081e+04,     1.2268456776715018e-04,     1.1923216784999263e-08,    -5.0023075616881894e-11, } , 
      {   -8.6251375713060963e-02,     1.2566039104281208e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    3.1695591111011621e+00,     1.8123536225411321e+04,    -1.3334009373878425e-04,    -9.4576483283486060e-09,    -1.6773583739027728e-10, } , 
      {    6.4253695979153846e+00,     3.4990468540394526e+04,    -2.6135242449175144e-04,    -2.0148080885022542e-08,    -2.2659221827697498e-10, } , 
      {   -9.8650731214048148e-01,    -7.2816369799316308e+03,     2.7497697552947639e-04,     2.6264209093965397e-07,    -1.2860070742375327e-09, } , 
      {    2.2693031746737415e+00,     9.5852953350515709e+03,     1.4696464477650915e-04,     2.5195165838298003e-07,    -1.3448634551242304e-09, } , 
      {    5.5251136614879641e+00,     2.6452227650034773e+04,     1.8952314023541942e-05,     2.4126122582630608e-07,    -1.4037198360109280e-09, } , 
      {    1.3690472382463210e+00,     1.0470544446918193e+03,     4.2726938329180257e-04,     5.1336096509430875e-07,    -2.5219910728581835e-09, } , 
      {    4.6248577250605436e+00,     1.7913986759675019e+04,     2.9925705253883535e-04,     5.0267053253763480e-07,    -2.5808474537448814e-09, } , 
      {    7.8806682118747666e+00,     3.4780919074658224e+04,     1.7124472178586811e-04,     4.9198009998096085e-07,    -2.6397038346315793e-09, } , 
      {    3.7246017886331235e+00,     9.3757458693152676e+03,     5.7956179105412870e-04,     7.6407983924896347e-07,    -3.7579750714788340e-09, } , 
      {    6.9804122754473461e+00,     2.6242678184298471e+04,     4.5154946030116154e-04,     7.5338940669228952e-07,    -3.8168314523655318e-09, } , 
      {    1.0236222762261569e+01,     4.3109610499281669e+04,     3.2353712954819432e-04,     7.4269897413561557e-07,    -3.8756878332522297e-09, } , 
      {    9.3359668258341486e+00,     3.4571369608921923e+04,     6.0384186806348766e-04,     1.0041082808469442e-06,    -5.0528154509861826e-09, } , 
      {   -9.5515952651168021e+00,    -3.1429859832851620e+04,    -6.1716127552803026e-04,    -1.0010263202761308e-06,     4.7806168097272339e-09, } , 
      {   -7.1960407147299987e+00,    -2.3101168408228168e+04,    -4.6486886776570408e-04,    -7.5030744612147612e-07,     3.5446328111065831e-09, } , 
      {   -8.0962966511574201e+00,    -3.1639409298587922e+04,    -1.8456412925041068e-04,    -4.8889813941014745e-07,     2.3675051933726301e-09, } , 
      {   -4.8404861643431962e+00,    -1.4772476983604720e+04,    -3.1257646000337790e-04,    -4.9958857196682140e-07,     2.3086488124859323e-09, } , 
      {   -1.5846756775289734e+00,     2.0944553313784827e+03,    -4.4058879075634512e-04,    -5.1027900452349535e-07,     2.2497924315992344e-09, } , 
      {   -5.7407421007706168e+00,    -2.3310717873964470e+04,    -3.2271721488084503e-05,    -2.3817926525549278e-07,     1.1315211947519791e-09, } , 
      {   -2.4849316139563937e+00,    -6.4437855589812689e+03,    -1.6028405224105172e-04,    -2.4886969781216673e-07,     1.0726648138652814e-09, } , 
      {    7.7087887285782908e-01,     1.0423146756001934e+04,    -2.8829638299401894e-04,    -2.5956013036884068e-07,     1.0138084329785838e-09, } , 
      {   -3.3851875503838142e+00,    -1.4982026449341021e+04,     1.2002068627424168e-04,     1.2539608899161927e-08,    -1.0446280386867170e-10, } , 
      {   -1.2937706356959144e-01,     1.8849058656421812e+03,    -7.9916444787255359e-06,     1.8491763424879922e-09,    -1.6331918475536940e-10, } , 
      {    3.1264334232446318e+00,     1.8751838180625382e+04,    -1.3600397523169276e-04,    -8.8412562141859425e-09,    -2.2217556564206710e-10, } , 
      {    2.2261774868172113e+00,     1.0213597290265632e+04,     1.4430076328360064e-04,     2.5256805049714271e-07,    -1.3993031833760202e-09, } , 
      {    5.4819879736314343e+00,     2.7080529605248830e+04,     1.6288432530633428e-05,     2.4187761794046876e-07,    -1.4581595642627179e-09, } , 
      {    4.5817320372040138e+00,     1.8542288714889080e+04,     2.9659317104592682e-04,     5.0328692465179748e-07,    -2.6352871819966711e-09, } , 
      {    6.9372865875908163e+00,     2.6870980139512529e+04,     4.4888557880825300e-04,     7.5400579880645220e-07,    -3.8712711806173219e-09, } , 
      {   -4.8836118521997269e+00,    -1.4144175028390659e+04,    -3.1524034149628643e-04,    -4.9897217985265882e-07,     2.2542090842341426e-09, } , 
      {   -2.5280573018129244e+00,    -5.8154836037672085e+03,    -1.6294793373396022e-04,    -2.4825330569800405e-07,     1.0182250856134916e-09, } , 
      {   -1.7250275142612193e-01,     2.5132078208562416e+03,    -1.0655525971634047e-05,     2.4655684566506563e-09,    -2.1775891300715919e-10, } , 
      {    2.1830517989606806e+00,     1.0841899245479692e+04,     1.4163688179069214e-04,     2.5318444261130539e-07,    -1.4537429116278101e-09, } , 
      {    4.5386063493474831e+00,     1.9170590670103142e+04,     2.9392928955301829e-04,     5.0390331676596005e-07,    -2.6897269102484607e-09, } , 
      {   -2.5711829896694551e+00,    -5.1871816485531481e+03,    -1.6561181522686873e-04,    -2.4763691358384143e-07,     9.6378535736170190e-10, } , 
      {    5.3709695505827257e+00,     5.2581693241346557e+03,    -2.2436062090282768e-05,     2.8593292803110101e-08,     1.3749315996266440e-11, } , 
      {    6.1673476195259025e-01,    -1.0770911569898184e+04,    -3.2968475910784365e-04,    -4.7222806339203665e-07,     2.4312775849857785e-09, } , 
      {    2.9722893123393925e+00,    -2.4422201452747340e+03,    -1.7739235134551747e-04,    -2.2150918923738194e-07,     1.1952935863651275e-09, } , 
      {    5.3278438627261950e+00,     5.8864712793487161e+03,    -2.5099943583191279e-05,     2.9209684917272764e-08,    -4.0690412255523351e-11, } , 
      {    7.6833984131129975e+00,     1.4215162703972168e+04,     1.2719246417913489e-04,     2.7992855907192750e-07,    -1.2766744108761742e-09, } , 
      {    1.0038952963499801e+01,     2.2543854128595616e+04,     2.7948487194146107e-04,     5.3064743322658212e-07,    -2.5126584094968248e-09, } , 
      {   -1.7819454762907427e+00,    -1.8471301039307575e+04,    -4.8464104836307831e-04,    -7.2233054543252880e-07,     3.6128218553546397e-09, } , 
      {    5.7360907409605977e-01,    -1.0142609614684125e+04,    -3.3234864060075218e-04,    -4.7161167127787397e-07,     2.3768378567339888e-09, } , 
      {   -3.2664686233136075e-01,    -1.8680850505043876e+04,    -5.2043902085458757e-05,    -2.1020236456654536e-07,     1.1997102390000352e-09, } , 
      {    2.9291636244828623e+00,    -1.8139181900606736e+03,    -1.8005623283842597e-04,    -2.2089279712321928e-07,     1.1408538581133376e-09, } , 
      {    2.0289076880554417e+00,    -1.0352159080420426e+04,     1.0024850567686742e-04,     4.0516509588109365e-08,    -3.6273759620615454e-11, } , 
      {    5.2847181748696652e+00,     6.5147732345627765e+03,    -2.7763825076099793e-05,     2.9826077031435431e-08,    -9.5130140507313155e-11, } , 
      {    8.5405286616838882e+00,     2.3381705549545979e+04,    -1.5577615582906701e-04,     1.9135644474761496e-08,    -1.5398652139401084e-10, } , 
      {    4.3844622384422447e+00,    -2.0234676557969760e+03,     2.5254091343919358e-04,     2.9123538374276408e-07,    -1.2722577582412664e-09, } , 
      {    7.6402727252564677e+00,     1.4843464659186227e+04,     1.2452858268622639e-04,     2.8054495118609018e-07,    -1.3311141391279641e-09, } , 
      {    9.9958272756432702e+00,     2.3172156083809678e+04,     2.7682099044855254e-04,     5.3126382534074480e-07,    -2.5670981377486145e-09, } , 
      {    1.2351381826030073e+01,     3.1500847508433122e+04,     4.2911339821087872e-04,     7.8198269949539952e-07,    -3.8030821363692661e-09, } , 
      {   -6.5361802649208780e+00,    -3.4500381933340417e+04,    -7.9188974538063920e-04,    -1.2231519016276755e-06,     6.0303501243441512e-09, } , 
      {   -4.1806257145340755e+00,    -2.6171690508716965e+04,    -6.3959733761831308e-04,    -9.7243302747302073e-07,     4.7943661257235004e-09, } , 
      {   -5.0808816509614960e+00,    -3.4709931399076711e+04,    -3.5929259910301962e-04,    -7.1102372076169217e-07,     3.6172385079895470e-09, } , 
      {   -1.8250711641472732e+00,    -1.7842999084093513e+04,    -4.8730492985598679e-04,    -7.2171415331836612e-07,     3.5583821271028496e-09, } , 
      {    1.4307393226669498e+00,    -9.7606676911031082e+02,    -6.1531726060895411e-04,    -7.3240458587504007e-07,     3.4995257462161517e-09, } , 
      {   -2.7253271005746935e+00,    -2.6381239974453267e+04,    -2.0700019134069347e-04,    -4.6030484660703745e-07,     2.3812545093688962e-09, } , 
      {    5.3048338623952929e-01,    -9.5143076594700651e+03,    -3.3501252209366066e-04,    -4.7099527916371135e-07,     2.3223981284821988e-09, } , 
      {    3.7862938730537525e+00,     7.3526246555131374e+03,    -4.6302485284662788e-04,    -4.8168571172038535e-07,     2.2635417475955009e-09, } , 
      {   -3.6977255018789124e-01,    -1.8052548549829815e+04,    -5.4707783578367268e-05,    -2.0958597245238268e-07,     1.1452705107482456e-09, } , 
      {    2.8860379366263320e+00,    -1.1856162348466132e+03,    -1.8272011433133451e-04,    -2.2027640500905660e-07,     1.0864141298615479e-09, } , 
      {    6.1418484234405550e+00,     1.5681316080136588e+04,    -3.1073244508430170e-04,    -2.3096683756573055e-07,     1.0275577489748503e-09, } , 
      {   -1.2700284866153118e+00,    -2.6590789440189568e+04,     2.2559695493692611e-04,     5.1823334258945960e-08,    -3.1857106985707551e-11, } , 
      {    1.9857820001989113e+00,    -9.7238571252063666e+03,     9.7584624183958920e-05,     4.1132901702272025e-08,    -9.0713487872405258e-11, } , 
      {    5.2415924870131345e+00,     7.1430751897768369e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    8.4974029738273575e+00,     2.4010007504760037e+04,    -1.5844003732197554e-04,     1.9752036588924160e-08,    -2.0842624964580063e-10, } , 
      {    4.3413365505857140e+00,    -1.3951657005829156e+03,     2.4987703194628510e-04,     2.9185177585692671e-07,    -1.3266974864930561e-09, } , 
      {    7.5971470373999370e+00,     1.5471766614400287e+04,     1.2186470119331788e-04,     2.8116134330025281e-07,    -1.3855538673797537e-09, } , 
      {    1.0852957524214160e+01,     3.2338698929383485e+04,    -6.1476295596493528e-06,     2.7047091074357886e-07,    -1.4444102482664514e-09, } , 
      {    6.6968911009725165e+00,     6.9335257240405344e+03,     4.0216943970861128e-04,     5.4257065001158142e-07,    -2.5626814851137071e-09, } , 
      {    9.9527015877867395e+00,     2.3800458039023735e+04,     2.7415710895564407e-04,     5.3188021745490748e-07,    -2.6215378660004046e-09, } , 
      {    1.3208512074600963e+01,     4.0667390354006937e+04,     1.4614477820267682e-04,     5.2118978489823353e-07,    -2.6803942468871024e-09, } , 
      {    9.0524456513593190e+00,     1.5262217148663982e+04,     5.5446184747093741e-04,     7.9328952416623614e-07,    -3.7986654837343575e-09, } , 
      {    1.2308256138173542e+01,     3.2129149463647183e+04,     4.2644951671797025e-04,     7.8259909160956220e-07,    -3.8575218646210554e-09, } , 
      {    1.5564066624987765e+01,     4.8996081778630381e+04,     2.9843718596500303e-04,     7.7190865905288825e-07,    -3.9163782455077524e-09, } , 
      {    1.4663810688560345e+01,     4.0457840888270635e+04,     5.7874192448029637e-04,     1.0333179657642170e-06,    -5.0935058632417062e-09, } , 
      {    1.7019365238947145e+01,     4.8786532312894087e+04,     7.3103433224262250e-04,     1.2840368399188717e-06,    -6.3294898618623571e-09, } , 
      {   -8.9348605031642112e+00,    -4.2200771402749800e+04,    -9.4684603463587392e-04,    -1.4732543836681675e-06,     7.2118943947130120e-09, } , 
      {   -9.8351164395916317e+00,    -5.0739012293109554e+04,    -6.6654129612058041e-04,    -1.2118450769568389e-06,     6.0347667769790590e-09, } , 
      {   -6.5793059527774087e+00,    -3.3872079978126356e+04,    -7.9455362687354768e-04,    -1.2225355095135128e-06,     5.9759103960923612e-09, } , 
      {   -7.4795618892048292e+00,    -4.2410320868486102e+04,    -5.1424888835825428e-04,    -9.6112620280218421e-07,     4.7987827783584082e-09, } , 
      {   -4.2237514023906062e+00,    -2.5543388553502904e+04,    -6.4226121911122155e-04,    -9.7181663535885805e-07,     4.7399263974717103e-09, } , 
      {   -9.6794091557638318e-01,    -8.6764562385197023e+03,    -7.7027354986418882e-04,    -9.8250706791553211e-07,     4.6810700165850125e-09, } , 
      {   -5.1240073388180267e+00,    -3.4081629443862650e+04,    -3.6195648059592815e-04,    -7.1040732864752949e-07,     3.5627987797377574e-09, } , 
      {   -1.8681968520038037e+00,    -1.7214697128879452e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.3876136348104193e+00,    -3.4776481389625042e+02,    -6.1798114210186259e-04,    -7.3178819376087739e-07,     3.4450860179643621e-09, } , 
      {   -6.0242632752454472e+00,    -4.2619870334222403e+04,    -8.1651742080634756e-05,    -4.4899802193620083e-07,     2.3856711620038044e-09, } , 
      {   -2.7684527884312242e+00,    -2.5752938019239205e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    4.8735769838299881e-01,    -8.8860057042560038e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.7431681851972218e+00,     7.9809266107271978e+03,    -4.6568873433953641e-04,    -4.8106931960622267e-07,     2.2091020193437112e-09, } , 
      {   -3.6687087248586447e+00,    -3.4291178909598959e+04,     7.0640665681691425e-05,    -1.9827914778154611e-07,     1.1496871633831536e-09, } , 
      {   -4.1289823804442172e-01,    -1.7424246594615754e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    2.8429122487698013e+00,    -5.5731427963255283e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.0987227355840243e+00,     1.6309618035350648e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    9.3545332223982474e+00,     3.3176550350333848e+04,    -4.4140865733017744e-04,    -2.4104087800824184e-07,     9.1426163983636265e-10, } , 
      {   -1.3131541744718422e+00,    -2.5962487484975507e+04,     2.2293307344401761e-04,     5.2439726373108627e-08,    -8.6296835237497348e-11, } , 
      {    1.9426563123423808e+00,    -9.0955551699923053e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    5.1984667991566038e+00,     7.7713771449908972e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.4542772859708268e+00,     2.4638309459974098e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.1710087772785050e+01,     4.1505241774957300e+04,    -2.8911624956785126e-04,     9.6779961464128886e-09,    -3.2172235878428818e-10, } , 
      {    1.0424003759149603e+00,    -1.7633796060352055e+04,     3.7522548120634379e-04,     3.0315860052776333e-07,    -1.3222808338581481e-09, } , 
      {    4.2982108627291833e+00,    -7.6686374536885523e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    7.5540213495434063e+00,     1.6100068569614348e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0809831836357629e+01,     3.2967000884597546e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    1.4065642323171852e+01,     4.9833933199580752e+04,    -1.3682384180552508e-04,     2.6039687030106759e-07,    -1.5577063574049391e-09, } , 
      {    6.6537654131159858e+00,     7.5618276792545948e+03,     3.9950555821570275e-04,     5.4318704212574410e-07,    -2.6171212133654968e-09, } , 
      {    9.9095758999302088e+00,     2.4428759994237796e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.3165386386744432e+01,     4.1295692309220998e+04,     1.4348089670976832e-04,     5.2180617701239621e-07,    -2.7348339751388921e-09, } , 
      {    1.6421196873558657e+01,     5.8162624624204196e+04,     1.5468565956801099e-05,     5.1111574445572237e-07,    -2.7936903560255899e-09, } , 
      {    9.0093199635027883e+00,     1.5890519103878043e+04,     5.5179796597802893e-04,     7.9390591628039882e-07,    -3.8531052119861476e-09, } , 
      {    1.2265130450317011e+01,     3.2757451418861245e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    1.5520940937131234e+01,     4.9624383733844443e+04,     2.9577330447209450e-04,     7.7252505116705093e-07,    -3.9708179737595425e-09, } , 
      {    1.4620685000703814e+01,     4.1086142843484697e+04,     5.7607804298738790e-04,     1.0339343578783797e-06,    -5.1479455914934963e-09, } , 
      {    1.7876495487518035e+01,     5.7953075158467895e+04,     4.4806571223442068e-04,     1.0232439253217056e-06,    -5.2068019723801942e-09, } , 
      {    1.6976239551090615e+01,     4.9414834268108149e+04,     7.2837045074971402e-04,     1.2846532320330344e-06,    -6.3839295901141471e-09, } , 
      {    1.9331794101477421e+01,     5.7743525692731593e+04,     8.8066285851204026e-04,     1.5353721061876891e-06,    -7.6199135887347980e-09, } , 
      {   -6.6224316406339394e+00,    -3.3243778022912295e+04,    -7.9721750836645616e-04,    -1.2219191173993501e-06,     5.9214706678405711e-09, } , 
      {   -7.5226875770613599e+00,    -4.1782018913272041e+04,    -5.1691276985116276e-04,    -9.6050981068802153e-07,     4.7443430501066181e-09, } , 
      {   -4.2668770902471369e+00,    -2.4915086598288843e+04,    -6.4492510060413003e-04,    -9.7120024324469537e-07,     4.6854866692199203e-09, } , 
      {   -5.1671330266745574e+00,    -3.3453327488648589e+04,    -3.6462036208883669e-04,    -7.0979093653336681e-07,     3.5083590514859677e-09, } , 
      {   -1.9113225398603342e+00,    -1.6586395173665391e+04,    -4.9263269284180379e-04,    -7.2048136909004076e-07,     3.4495026705992703e-09, } , 
      {    1.3444879469538888e+00,     2.8053714131780998e+02,    -6.2064502359477107e-04,    -7.3117180164671471e-07,     3.3906462897125724e-09, } , 
      {   -2.8115784762877549e+00,    -2.5124636064025144e+04,    -2.1232795432651048e-04,    -4.5907206237871209e-07,     2.2723750528653169e-09, } , 
      {    4.4423201052646832e-01,    -8.2577037490419425e+03,    -3.4034028507947772e-04,    -4.6976249493538599e-07,     2.2135186719786194e-09, } , 
      {    3.7000424973406911e+00,     8.6092285659412591e+03,    -4.6835261583244494e-04,    -4.8045292749205999e-07,     2.1546622910919216e-09, } , 
      {   -4.5602392590095220e-01,    -1.6795944639401692e+04,    -6.0035546564184289e-05,    -2.0835318822405737e-07,     1.0363910542446658e-09, } , 
      {    2.7997865609132706e+00,     7.0987675581507574e+01,    -1.8804787731715151e-04,    -2.1904362078073129e-07,     9.7753467335796819e-10, } , 
      {    6.0555970477274936e+00,     1.6937919990564707e+04,    -3.1606020807011876e-04,    -2.2973405333740524e-07,     9.1867829247127064e-10, } , 
      {    1.8995306244858503e+00,    -8.4672532147782440e+03,     9.2256861198141885e-05,     4.2365685930597359e-08,    -1.9959294437598484e-10, } , 
      {    5.1553411113000731e+00,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    8.4111515981142961e+00,     2.5266611415188159e+04,    -1.6376780030779255e-04,     2.0984820817249487e-08,    -3.1730570614938024e-10, } , 
      {    1.1666962084928519e+01,     4.2133543730171361e+04,    -2.9178013106075979e-04,     1.0294388260575552e-08,    -3.7616208703607799e-10, } , 
      {    4.2550851748726526e+00,    -1.3856179015479483e+02,     2.4454926896046804e-04,     2.9308456008525207e-07,    -1.4355769429966358e-09, } , 
      {    7.5108956616868756e+00,     1.6728370524828410e+04,     1.1653693820750085e-04,     2.8239412752857817e-07,    -1.4944333238833335e-09, } , 
      {    1.0766706148501099e+01,     3.3595302839811608e+04,    -1.1475392545466375e-05,     2.7170369497190422e-07,    -1.5532897047700311e-09, } , 
      {    1.4022516635315322e+01,     5.0462235154794813e+04,    -1.3948772329843359e-04,     2.6101326241523027e-07,    -1.6121460856567290e-09, } , 
      {    6.6106397252594551e+00,     8.1901296344686552e+03,     3.9684167672279422e-04,     5.4380343423990678e-07,    -2.6715609416172864e-09, } , 
      {    9.8664502120736781e+00,     2.5057061949451858e+04,     2.6882934596982700e-04,     5.3311300168323284e-07,    -2.7304173225039839e-09, } , 
      {    1.3122260698887901e+01,     4.1923994264435059e+04,     1.4081701521685981e-04,     5.2242256912655889e-07,    -2.7892737033906817e-09, } , 
      {    8.9661942756462576e+00,     1.6518821059092104e+04,     5.4913408448512046e-04,     7.9452230839456150e-07,    -3.9075449402379377e-09, } , 
      {    1.2222004762460481e+01,     3.3385753374075306e+04,     4.2112175373215318e-04,     7.8383187583788756e-07,    -3.9664013211246355e-09, } , 
      {    1.5477815249274704e+01,     5.0252685689058504e+04,     2.9310942297918597e-04,     7.7314144328121361e-07,    -4.0252577020113326e-09, } , 
      {    1.4577559312847283e+01,     4.1714444798698758e+04,     5.7341416149447942e-04,     1.0345507499925424e-06,    -5.2023853197452864e-09, } , 
      {    1.6933113863234084e+01,     5.0043136223322210e+04,     7.2570656925680555e-04,     1.2852696241471971e-06,    -6.4383693183659372e-09, } , 
      {   -4.3100027781036676e+00,    -2.4286784643074781e+04,    -6.4758898209703862e-04,    -9.7058385113053269e-07,     4.6310469409681310e-09, } , 
      {   -1.9544482277168647e+00,    -1.5958093218451331e+04,    -4.9529657433471238e-04,    -7.1986497697587808e-07,     3.3950629423474802e-09, } , 
      {    4.0110632266993784e-01,    -7.6294017938278830e+03,    -3.4300416657238620e-04,    -4.6914610282122336e-07,     2.1590789437268294e-09, } , 
      {    2.7566608730567403e+00,     6.9928963079556797e+02,    -1.9071175881006004e-04,    -2.1842722866656861e-07,     9.2309494510617842e-10, } , 
      {    6.0124713598709629e+00,     1.7566221945778769e+04,    -3.1872408956302724e-04,    -2.2911766122324256e-07,     8.6423856421948077e-10, } , 
      {    1.8564049366293198e+00,    -7.8389512595641845e+03,     8.9592979705233381e-05,     4.2982078044760019e-08,    -2.5403267262777466e-10, } , 
      {    5.1122154234435424e+00,     9.0279810554190190e+03,    -3.8419351047733836e-05,     3.2291645488086085e-08,    -3.1288905351447236e-10, } , 
      {    8.3680259102577654e+00,     2.5894913370402217e+04,    -1.6643168180070108e-04,     2.1601212931412150e-08,    -3.7174543440117001e-10, } , 
      {    4.2119594870161219e+00,     4.8974016505926556e+02,     2.4188538746755954e-04,     2.9370095219941469e-07,    -1.4900166712484255e-09, } , 
      {    7.4677699738303449e+00,     1.7356672480042471e+04,     1.1387305671459234e-04,     2.8301051964274080e-07,    -1.5488730521351231e-09, } , 
      {    1.0723580460644568e+01,     3.4223604795025669e+04,    -1.4139274038374888e-05,     2.7232008708606685e-07,    -1.6077294330218208e-09, } , 
      {    9.8233245242171474e+00,     2.5685363904665915e+04,     2.6616546447691853e-04,     5.3372939379739552e-07,    -2.7848570507557740e-09, } , 
      {    1.3079135011031370e+01,     4.2552296219649121e+04,     1.3815313372395128e-04,     5.2303896124072157e-07,    -2.8437134316424718e-09, } , 
      {    1.2178879074603950e+01,     3.4014055329289367e+04,     4.1845787223924471e-04,     7.8444826795205024e-07,    -4.0208410493764248e-09, } , 
      {    1.4534433624990752e+01,     4.2342746753912819e+04,     5.7075028000157083e-04,     1.0351671421067051e-06,    -5.2568250479970756e-09, } , 
      {    3.5798063481340736e-01,    -7.0010998386138226e+03,    -3.4566804806529473e-04,    -4.6852971070706068e-07,     2.1046392154750397e-09, } , 
      {    2.7135351852002101e+00,     1.3275915860096284e+03,    -1.9337564030296855e-04,    -2.1781083655240596e-07,     8.6865521685438866e-10, } , 
      {    1.8132792487727893e+00,    -7.2106493043501241e+03,     8.6929098212324863e-05,     4.3598470158922686e-08,    -3.0847240087956442e-10, } , 
      {    5.0690897355870126e+00,     9.6562830106330784e+03,    -4.1083232540642354e-05,     3.2908037602248751e-08,    -3.6732878176626217e-10, } , 
      {    7.4246442859738151e+00,     1.7984974435256529e+04,     1.1120917522168383e-04,     2.8362691175690348e-07,    -1.6033127803869130e-09, } , 
      {    5.0259640477304819e+00,     1.0284584965847138e+04,    -4.3747114033550858e-05,     3.3524429716411412e-08,    -4.2176851001805194e-10, } , 
      {    1.0655687725452390e+01,     1.1772942558697432e+04,    -5.0199887166382558e-05,     5.8419369834545528e-08,    -8.1380824511046702e-11, } , 
      {    5.9014529368222552e+00,    -4.2561383353354076e+03,    -3.5744858418394347e-04,    -4.4240198636060119e-07,     2.3361474444784652e-09, } , 
      {    8.2570074872090569e+00,     4.0725530892880424e+03,    -2.0515617642161726e-04,    -1.9168311220594652e-07,     1.1001634458578142e-09, } , 
      {    1.0612562037595859e+01,     1.2401244513911493e+04,    -5.2863768659291069e-05,     5.9035761948708195e-08,    -1.3582055276283652e-10, } , 
      {    1.2968116587982662e+01,     2.0729935938534942e+04,     9.9428639103035113e-05,     3.0975463610336297e-07,    -1.3718045513834875e-09, } , 
      {    3.5027726985789220e+00,    -1.1956527804744796e+04,    -5.1240487343917813e-04,    -6.9250446840109323e-07,     3.5176917148473260e-09, } , 
      {    5.8583272489657245e+00,    -3.6278363801213472e+03,    -3.6011246567685195e-04,    -4.4178559424643856e-07,     2.2817077162266752e-09, } , 
      {    8.2138817993525279e+00,     4.7008550445021028e+03,    -2.0782005791452577e-04,    -1.9106672009178387e-07,     1.0457237176060245e-09, } , 
      {    1.1469692286166751e+01,     2.1567787359485304e+04,    -3.3583238866749298e-04,    -2.0175715264845779e-07,     9.8686733671932690e-10, } , 
      {    7.3136258629251065e+00,    -3.8373858458576487e+03,     7.2484680600767631e-05,     7.0342586619544796e-08,    -1.3140390012792861e-10, } , 
      {    1.0569436349739330e+01,     1.3029546469125553e+04,    -5.5527650152199586e-05,     5.9652154062870862e-08,    -1.9026028101462631e-10, } , 
      {    1.3825246836553553e+01,     2.9896478784108756e+04,    -1.8353998090516680e-04,     4.8961721506196927e-08,    -2.4911666190132404e-10, } , 
      {    9.6691804133119099e+00,     4.4913055787658013e+03,     2.2477708836309381e-04,     3.2106146077419949e-07,    -1.3673878987485793e-09, } , 
      {    1.2924990900126133e+01,     2.1358237893749003e+04,     9.6764757610126595e-05,     3.1037102821752559e-07,    -1.4262442796352771e-09, } , 
      {    1.5280545450512935e+01,     2.9686929318372455e+04,     2.4905716537245278e-04,     5.6108990237218037e-07,    -2.6622282782559282e-09, } , 
      {    1.7636100000899738e+01,     3.8015620742995903e+04,     4.0134957313477901e-04,     8.1180877652683509e-07,    -3.8982122768765782e-09, } , 
      {    1.1040924603355891e+00,    -1.9656917274154188e+04,    -6.6736116269441284e-04,    -9.4260695044158538e-07,     4.6992359852161876e-09, } , 
      {    3.4596470107223913e+00,    -1.1328225849530736e+04,    -5.1506875493208661e-04,    -6.9188807628693066e-07,     3.4632519865955363e-09, } , 
      {    2.5593910742949708e+00,    -1.9866466739890489e+04,    -2.3476401641679324e-04,    -4.3047876957560199e-07,     2.2861243688615834e-09, } , 
      {    5.8152015611091938e+00,    -2.9995344249072868e+03,    -3.6277634716976048e-04,    -4.4116920213227588e-07,     2.2272679879748855e-09, } , 
      {    9.0710120479234178e+00,     1.3867397890075918e+04,    -4.9078867792272764e-04,    -4.5185963468894983e-07,     2.1684116070881881e-09, } , 
      {    4.9149456246817733e+00,    -1.1537775315267038e+04,    -8.2471608654467054e-05,    -1.7975989542094727e-07,     1.0501403702409325e-09, } , 
      {    8.1707561114959972e+00,     5.3291569997161632e+03,    -2.1048393940743427e-04,    -1.9045032797762119e-07,     9.9128398935423468e-10, } , 
      {    1.1426566598310220e+01,     2.2196089314699366e+04,    -3.3849627016040152e-04,    -2.0114076053429511e-07,     9.3242760846753703e-10, } , 
      {    7.2705001750685758e+00,    -3.2090838906435883e+03,     6.9820799107859127e-05,     7.0958978733707463e-08,    -1.8584362837971840e-10, } , 
      {    1.0526310661882800e+01,     1.3657848424339612e+04,    -5.8191531645108090e-05,     6.0268546177033528e-08,    -2.4470000926641610e-10, } , 
      {    1.3782121148697023e+01,     3.0524780739322818e+04,    -1.8620386239807531e-04,     4.9578113620359587e-08,    -3.0355639015311380e-10, } , 
      {    9.6260547254553792e+00,     5.1196075339798617e+03,     2.2211320687018531e-04,     3.2167785288836217e-07,    -1.4218276270003691e-09, } , 
      {    1.2881865212269602e+01,     2.1986539848963064e+04,     9.4100876117218091e-05,     3.1098742033168827e-07,    -1.4806840078870670e-09, } , 
      {    1.6137675699083825e+01,     3.8853472163946266e+04,    -3.3911454635749126e-05,     3.0029698777501433e-07,    -1.5395403887737646e-09, } , 
      {    1.1981609275842182e+01,     1.3448298958603311e+04,     3.7440561463251152e-04,     5.7239672704301689e-07,    -2.6578116256210200e-09, } , 
      {    1.5237419762656405e+01,     3.0315231273586516e+04,     2.4639328387954424e-04,     5.6170629448634294e-07,    -2.7166680065077178e-09, } , 
      {    1.8493230249470628e+01,     4.7182163588569718e+04,     1.1838095312657705e-04,     5.5101586192966899e-07,    -2.7755243873944153e-09, } , 
      {    1.7592974313043207e+01,     3.8643922698209957e+04,     3.9868569164187048e-04,     8.1242516864099766e-07,    -3.9526520051283682e-09, } , 
      {    1.9948528863430010e+01,     4.6972614122833409e+04,     5.5097809940419661e-04,     1.0631440427956523e-06,    -5.1886360037490191e-09, } , 
      {   -1.2945877779077439e+00,    -2.7357306743563582e+04,    -8.2231745194964745e-04,    -1.1927094324820775e-06,     5.8807802555850483e-09, } , 
      {   -2.1948437143351645e+00,    -3.5895547633923328e+04,    -5.4201271343435405e-04,    -9.3130012577074875e-07,     4.7036526378510945e-09, } , 
      {    1.0609667724790586e+00,    -1.9028615318940130e+04,    -6.7002504418732132e-04,    -9.4199055832742270e-07,     4.6447962569643975e-09, } , 
      {    1.6071083605163805e-01,    -2.7566856209299876e+04,    -3.8972030567202797e-04,    -6.8058125161609403e-07,     3.4676686392304445e-09, } , 
      {    3.4165213228658611e+00,    -1.0699923894316675e+04,    -5.1773263642499508e-04,    -6.9127168417276798e-07,     3.4088122583437467e-09, } , 
      {    2.5162653864384406e+00,    -1.9238164784676432e+04,    -2.3742789790970174e-04,    -4.2986237746143931e-07,     2.2316846406097937e-09, } , 
      {    5.7720758732526640e+00,    -2.3712324696932264e+03,    -3.6544022866266901e-04,    -4.4055281001811320e-07,     2.1728282597230958e-09, } , 
      {    9.0278863600668871e+00,     1.4495699845289977e+04,    -4.9345255941563612e-04,    -4.5124324257478715e-07,     2.1139718788363984e-09, } , 
      {    4.8718199368252435e+00,    -1.0909473360052978e+04,    -8.5135490147375572e-05,    -1.7914350330678462e-07,     9.9570064198914267e-10, } , 
      {    8.1276304236394665e+00,     5.9574589549302236e+03,    -2.1314782090034278e-04,    -1.8983393586345854e-07,     9.3684426110244481e-10, } , 
      {    1.1383440910453690e+01,     2.2824391269913423e+04,    -3.4116015165331005e-04,    -2.0052436842013246e-07,     8.7798788021574726e-10, } , 
      {    1.4639251397267913e+01,     3.9691323584896621e+04,    -4.6917248240627726e-04,    -2.1121480097680641e-07,     8.1913149932904962e-10, } , 
      {    3.9715640003978225e+00,    -1.9447714250412733e+04,     1.9516924836791784e-04,     8.2265803404544051e-08,    -1.8142697574481052e-10, } , 
      {    7.2273744872120460e+00,    -2.5807819354295279e+03,     6.7156917614950609e-05,     7.1575370847870116e-08,    -2.4028335663150822e-10, } , 
      {    1.0483184974026269e+01,     1.4286150379553674e+04,    -6.0855413138016608e-05,     6.0884938291196182e-08,    -2.9913973751820592e-10, } , 
      {    1.3738995460840492e+01,     3.1153082694536875e+04,    -1.8886774389098381e-04,     5.0194505734522254e-08,    -3.5799611840490362e-10, } , 
      {    1.6994805947654715e+01,     4.8020015009520073e+04,    -3.1688007464395108e-04,     3.9504073177848319e-08,    -4.1685249929160127e-10, } , 
      {    6.3271185507846255e+00,    -1.1119022825789281e+04,     3.4746165613024402e-04,     3.3298467755919880e-07,    -1.4174109743654613e-09, } , 
      {    9.5829290375988485e+00,     5.7479094891939221e+03,     2.1944932537727680e-04,     3.2229424500252485e-07,    -1.4762673552521590e-09, } , 
      {    1.2838739524413072e+01,     2.2614841804177122e+04,     9.1436994624309573e-05,     3.1160381244585095e-07,    -1.5351237361388568e-09, } , 
      {    1.6094550011227295e+01,     3.9481774119160327e+04,    -3.6575336128657644e-05,     3.0091337988917701e-07,    -1.5939801170255545e-09, } , 
      {    1.9350360498041518e+01,     5.6348706434143525e+04,    -1.6458766688162487e-04,     2.9022294733250306e-07,    -1.6528364979122521e-09, } , 
      {    1.1938483587985651e+01,     1.4076600913817372e+04,     3.7174173313960299e-04,     5.7301311915717957e-07,    -2.7122513538728096e-09, } , 
      {    1.5194294074799874e+01,     3.0943533228800574e+04,     2.4372940238663577e-04,     5.6232268660050562e-07,    -2.7711077347595075e-09, } , 
      {    1.8450104561614097e+01,     4.7810465543783779e+04,     1.1571707163366854e-04,     5.5163225404383167e-07,    -2.8299641156462049e-09, } , 
      {    1.4294038138372454e+01,     2.2405292338440820e+04,     5.2403414090192917e-04,     8.2373199331183429e-07,    -3.9482353524934613e-09, } , 
      {    1.7549848625186677e+01,     3.9272224653424018e+04,     3.9602181014896195e-04,     8.1304156075516034e-07,    -4.0070917333801583e-09, } , 
      {    2.0805659112000900e+01,     5.6139156968407224e+04,     2.6800947939599468e-04,     8.0235112819848639e-07,    -4.0659481142668562e-09, } , 
      {    1.9905403175573479e+01,     4.7600916078047470e+04,     5.4831421791128813e-04,     1.0637604349098150e-06,    -5.2430757320008091e-09, } , 
      {    2.2260957725960282e+01,     5.5929607502670922e+04,     7.0060662567361426e-04,     1.3144793090644697e-06,    -6.4790597306214600e-09, } , 
      {   -3.6932680161510767e+00,    -3.5057696212972965e+04,    -9.7727374120488205e-04,    -1.4428119145225696e-06,     7.0623245259539099e-09, } , 
      {   -4.5935239525784972e+00,    -4.3595937103332719e+04,    -6.9696900268958876e-04,    -1.1814026078112408e-06,     5.8851969082199561e-09, } , 
      {   -1.3377134657642744e+00,    -2.6729004788349521e+04,    -8.2498133344255603e-04,    -1.1920930403679148e-06,     5.8263405273332591e-09, } , 
      {   -2.2379694021916947e+00,    -3.5267245678709267e+04,    -5.4467659492726263e-04,    -9.3068373365658607e-07,     4.6492129095993053e-09, } , 
      {    1.0178410846225281e+00,    -1.8400313363726069e+04,    -6.7268892568022990e-04,    -9.4137416621326002e-07,     4.5903565287126083e-09, } , 
      {    4.2736515714367513e+00,    -1.5333810487428627e+03,    -8.0070125643319707e-04,    -9.5206459876993396e-07,     4.5315001478259104e-09, } , 
      {   -3.1382253386191152e+00,    -4.3805486569069020e+04,    -2.6437185641196918e-04,    -6.6927442694525751e-07,     3.4720852918653519e-09, } , 
      {    1.1758514819510757e-01,    -2.6938554254085815e+04,    -3.9238418716493645e-04,    -6.7996485950193135e-07,     3.4132289109786545e-09, } , 
      {    3.3733956350093308e+00,    -1.0071621939102613e+04,    -5.2039651791790367e-04,    -6.9065529205860530e-07,     3.3543725300919566e-09, } , 
      {    6.6292061218235538e+00,     6.7953103758805892e+03,    -6.4840884867087094e-04,    -7.0134572461527924e-07,     3.2955161492052592e-09, } , 
      {   -7.8267078823231295e-01,    -3.5476795144445568e+04,    -1.1207944864964304e-04,    -4.1855555279060273e-07,     2.2361012932447011e-09, } , 
      {    2.4731396985819103e+00,    -1.8609862829462370e+04,    -2.4009177940261027e-04,    -4.2924598534727668e-07,     2.1772449123580036e-09, } , 
      {    5.7289501853961333e+00,    -1.7429305144791660e+03,    -3.6810411015557749e-04,    -4.3993641790395058e-07,     2.1183885314713058e-09, } , 
      {    8.9847606722103563e+00,     1.5124001800504037e+04,    -4.9611644090854471e-04,    -4.5062685046062452e-07,     2.0595321505846083e-09, } , 
      {    1.2240571159024579e+01,     3.1990934115487235e+04,    -6.2412877166151198e-04,    -4.6131728301729847e-07,     2.0006757696979105e-09, } , 
      {    1.5728837621544896e+00,    -2.7148103719822124e+04,     4.0212959112683135e-05,    -1.6783667863594799e-07,     1.0001172946240507e-09, } , 
      {    4.8286942489687128e+00,    -1.0281171404838919e+04,    -8.7799371640284076e-05,    -1.7852711119262194e-07,     9.4126091373735280e-10, } , 
      {    8.0845047357829358e+00,     6.5857609101442840e+03,    -2.1581170239325131e-04,    -1.8921754374929586e-07,     8.8240453285065505e-10, } , 
      {    1.1340315222597159e+01,     2.3452693225127485e+04,    -3.4382403314621852e-04,    -1.9990797630596978e-07,     8.2354815196395740e-10, } , 
      {    1.4596125709411382e+01,     4.0319625540110683e+04,    -4.7183636389918574e-04,    -2.1059840886264373e-07,     7.6469177107725975e-10, } , 
      {    3.9284383125412923e+00,    -1.8819412295198672e+04,     1.9250536687500931e-04,     8.2882195518706718e-08,    -2.3586670399660028e-10, } , 
      {    7.1842487993555153e+00,    -1.9524799802154675e+03,     6.4493036122042105e-05,     7.2191762962032783e-08,    -2.9472308488329798e-10, } , 
      {    1.0440059286169738e+01,     1.4914452334767735e+04,    -6.3519294630925112e-05,     6.1501330405358849e-08,    -3.5357946576999568e-10, } , 
      {    1.3695869772983961e+01,     3.1781384649750937e+04,    -1.9153162538389234e-04,     5.0810897848684914e-08,    -4.1243584665669338e-10, } , 
      {    1.6951680259798184e+01,     4.8648316964734135e+04,    -3.1954395613685956e-04,     4.0120465292010979e-08,    -4.7129222754339108e-10, } , 
      {    6.2839928629280948e+00,    -1.0490720870575220e+04,     3.4479777463733554e-04,     3.3360106967336142e-07,    -1.4718507026172510e-09, } , 
      {    9.5398033497423178e+00,     6.3762114444079825e+03,     2.1678544388436827e-04,     3.2291063711668748e-07,    -1.5307070835039487e-09, } , 
      {    1.2795613836556541e+01,     2.3243143759391183e+04,     8.8773113131401069e-05,     3.1222020456001358e-07,    -1.5895634643906465e-09, } , 
      {    1.6051424323370764e+01,     4.0110076074374389e+04,    -3.9239217621566155e-05,     3.0152977200333963e-07,    -1.6484198452773442e-09, } , 
      {    1.9307234810184987e+01,     5.6977008389357587e+04,    -1.6725154837453341e-04,     2.9083933944666569e-07,    -1.7072762261640418e-09, } , 
      {    8.6395474133148973e+00,    -2.1620294459517709e+03,     4.9709018239966162e-04,     5.8431994382801609e-07,    -2.7078347012379023e-09, } , 
      {    1.1895357900129120e+01,     1.4704902869031434e+04,     3.6907785164669451e-04,     5.7362951127134225e-07,    -2.7666910821245997e-09, } , 
      {    1.5151168386943343e+01,     3.1571835184014635e+04,     2.4106552089372724e-04,     5.6293907871466830e-07,    -2.8255474630112975e-09, } , 
      {    1.8406978873757566e+01,     4.8438767498997840e+04,     1.1305319014076003e-04,     5.5224864615799435e-07,    -2.8844038438979950e-09, } , 
      {    2.1662789360571789e+01,     6.5305699813981031e+04,    -1.4959140612207218e-05,     5.4155821360132041e-07,    -2.9432602247846928e-09, } , 
      {    1.4250912450515923e+01,     2.3033594293654882e+04,     5.2137025940902058e-04,     8.2434838542599697e-07,    -4.0026750807452505e-09, } , 
      {    1.7506722937330146e+01,     3.9900526608638080e+04,     3.9335792865605347e-04,     8.1365795286932302e-07,    -4.0615314616319476e-09, } , 
      {    2.0762533424144369e+01,     5.6767458923621285e+04,     2.6534559790308620e-04,     8.0296752031264907e-07,    -4.1203878425186454e-09, } , 
      {    1.6606467000902725e+01,     3.1362285718278334e+04,     6.7366266717134682e-04,     1.0750672595806517e-06,    -5.2386590793659014e-09, } , 
      {    1.9862277487716948e+01,     4.8229218033261532e+04,     5.4565033641837955e-04,     1.0643768270239776e-06,    -5.2975154602525984e-09, } , 
      {    2.3118087974531171e+01,     6.5096150348244730e+04,     4.1763800566541244e-04,     1.0536863944673038e-06,    -5.3563718411392962e-09, } , 
      {    1.8962021551289528e+01,     3.9690977142901786e+04,     8.2595507493367294e-04,     1.3257861337353064e-06,    -6.4746430779865522e-09, } , 
      {    2.2217832038103751e+01,     5.6557909457884984e+04,     6.9794274418070567e-04,     1.3150957011786324e-06,    -6.5334994588732492e-09, } , 
      {    2.5473642524917974e+01,     7.3424841772868182e+04,     5.6993041342773851e-04,     1.3044052686219585e-06,    -6.5923558397599471e-09, } , 
      {    2.4573386588490553e+01,     6.4886600882508428e+04,     8.5023515194303191e-04,     1.5658145753332871e-06,    -7.7694834574939017e-09, } , 
      {   -8.4475028047812124e+00,    -5.1086777107005808e+04,    -1.2845224382224431e-03,    -1.9436332707177161e-06,     9.4798527949434207e-09, } , 
      {   -6.0919482543944099e+00,    -4.2758085682382356e+04,    -1.1322300304601170e-03,    -1.6929143965630616e-06,     8.2438687963227707e-09, } , 
      {   -6.9922041908218304e+00,    -5.1296326572742102e+04,    -8.5192529194482347e-04,    -1.4315050898517328e-06,     7.0667411785888169e-09, } , 
      {   -3.7363937040076074e+00,    -3.4429394257758904e+04,    -9.7993762269779064e-04,    -1.4421955224084069e-06,     7.0078847977021198e-09, } , 
      {   -4.6366496404350279e+00,    -4.2967635148118658e+04,    -6.9963288418249724e-04,    -1.1807862156970781e-06,     5.8307571799681660e-09, } , 
      {   -1.3808391536208049e+00,    -2.6100702833135460e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {    1.8749713331934181e+00,    -9.2337705181522542e+03,    -9.5565754568843167e-04,    -1.2021670808104260e-06,     5.7130444181947712e-09, } , 
      {   -5.5369055768624484e+00,    -5.1505876038478411e+04,    -4.1932814566720395e-04,    -9.1937690898574955e-07,     4.6536295622342131e-09, } , 
      {   -2.2810950900482254e+00,    -3.4638943723495206e+04,    -5.4734047642017111e-04,    -9.3006734154242339e-07,     4.5947731813475152e-09, } , 
      {    9.7471539676599761e-01,    -1.7772011408512008e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    4.2305258835802206e+00,    -9.0507909352880233e+02,    -8.0336513792610554e-04,    -9.5144820665577128e-07,     4.4770604195741203e-09, } , 
      {   -3.1813510264756459e+00,    -4.3177184613854959e+04,    -2.6703573790487771e-04,    -6.6865803483109483e-07,     3.4176455636135622e-09, } , 
      {    7.4459460338577088e-02,    -2.6310252298871754e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    3.3302699471528001e+00,    -9.4433199838885539e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.5860804339670231e+00,     7.4236123310946496e+03,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {    9.8418909207812462e+00,     2.4290544646077848e+04,    -7.7908506091674658e-04,    -7.1141976505779051e-07,     3.1822200400667716e-09, } , 
      {   -8.2579647608884343e-01,    -3.4848493189231507e+04,    -1.1474333014255156e-04,    -4.1793916067644005e-07,     2.1816615649929114e-09, } , 
      {    2.4300140107253796e+00,    -1.7981560874248309e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    5.6858244975396026e+00,    -1.1146285592651057e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.9416349843538256e+00,     1.5752303755718098e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.2197445471168049e+01,     3.2619236070701296e+04,    -6.2679265315442045e-04,    -4.6070089090313579e-07,     1.9462360414461208e-09, } , 
      {   -1.7260524125162640e+00,    -4.3386734079591261e+04,     1.6556140837274187e-04,    -1.5652985396511139e-07,     1.0045339472589584e-09, } , 
      {    1.5297580742979591e+00,    -2.6519801764608063e+04,     3.7549077619774624e-05,    -1.6722028652178533e-07,     9.4567756637226078e-10, } , 
      {    4.7855685611121821e+00,    -9.6528694496248572e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    8.0413790479264051e+00,     7.2140628653583444e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1297189534740628e+01,     2.4080995180341546e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.4553000021554851e+01,     4.0947927495324744e+04,    -4.7450024539209427e-04,    -2.0998201674848107e-07,     7.1025204282546998e-10, } , 
      {    1.7808810508369074e+01,     5.7814859810307942e+04,    -6.0251257614506149e-04,    -2.2067244930515502e-07,     6.5139566193877223e-10, } , 
      {    6.2950213787053855e-01,    -3.5058042654967809e+04,     3.1785381613506805e-04,     9.4189020189543319e-08,    -2.3145005136169240e-10, } , 
      {    3.8853126246847616e+00,    -1.8191110339984611e+04,     1.8984148538210081e-04,     8.3498587632869385e-08,    -2.9030643224839010e-10, } , 
      {    7.1411231114989846e+00,    -1.3241780250014071e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0396933598313208e+01,     1.5542754289981794e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3652744085127431e+01,     3.2409686604964998e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.6908554571941654e+01,     4.9276618919948196e+04,    -3.2220783762976809e-04,     4.0736857406173646e-08,    -5.2573195579518085e-10, } , 
      {    2.0164365058755877e+01,     6.6143551234931394e+04,    -4.5022016838273525e-04,     3.0046424849499712e-08,    -5.8458833668187860e-10, } , 
      {    2.9850566882573411e+00,    -2.6729351230344357e+04,     4.7014622389739423e-04,     3.4490789434419805e-07,    -1.4674340499823432e-09, } , 
      {    6.2408671750715641e+00,    -9.8624189153611605e+03,     3.4213389314442701e-04,     3.3421746178752410e-07,    -1.5262904308690409e-09, } , 
      {    9.4966776618857871e+00,     7.0045133996220429e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.2752488148700010e+01,     2.3871445714605245e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6008298635514233e+01,     4.0738378029588450e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    1.9264109122328456e+01,     5.7605310344571648e+04,    -1.6991542986744191e-04,     2.9145573156082837e-07,    -1.7617159544158317e-09, } , 
      {    2.2519919609142679e+01,     7.4472242659554846e+04,    -2.9792776062040907e-04,     2.8076529900415442e-07,    -1.8205723353025295e-09, } , 
      {    8.5964217254583666e+00,    -1.5337274907377105e+03,     4.9442630090675314e-04,     5.8493633594217877e-07,    -2.7622744294896919e-09, } , 
      {    1.1852232212272590e+01,     1.5333204824245493e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    1.5108042699086813e+01,     3.2200137139228696e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.8363853185901036e+01,     4.9067069454211902e+04,     1.1038930864785152e-04,     5.5286503827215703e-07,    -2.9388435721497847e-09, } , 
      {    2.1619663672715259e+01,     6.5934001769195092e+04,    -1.7623022105115729e-05,     5.4217460571548309e-07,    -2.9976999530364825e-09, } , 
      {    1.0951976275845169e+01,     6.7949639338857378e+03,     6.4671870866907938e-04,     8.3565521009683349e-07,    -3.9982584281103427e-09, } , 
      {    1.4207786762659392e+01,     2.3661896248868943e+04,     5.1870637791611210e-04,     8.2496477754015965e-07,    -4.0571148089970406e-09, } , 
      {    1.7463597249473615e+01,     4.0528828563852141e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.0719407736287838e+01,     5.7395760878835346e+04,     2.6268171641017767e-04,     8.0358391242681175e-07,    -4.1748275707704355e-09, } , 
      {    2.3975218223102061e+01,     7.4262693193818544e+04,     1.3466938565721045e-04,     7.9289347987013780e-07,    -4.2336839516571333e-09, } , 
      {    1.6563341313046195e+01,     3.1990587673492395e+04,     6.7099878567843834e-04,     1.0756836516948144e-06,    -5.2930988076176914e-09, } , 
      {    1.9819151799860418e+01,     4.8857519988475593e+04,     5.4298645492547107e-04,     1.0649932191381403e-06,    -5.3519551885043885e-09, } , 
      {    2.3074962286674641e+01,     6.5724452303458791e+04,     4.1497412417250391e-04,     1.0543027865814665e-06,    -5.4108115693910863e-09, } , 
      {    2.6330772773488864e+01,     8.2591384618441996e+04,     2.8696179341953663e-04,     1.0436123540247924e-06,    -5.4696679502777842e-09, } , 
      {    1.8918895863432997e+01,     4.0319279098115847e+04,     8.2329119344076447e-04,     1.3264025258494691e-06,    -6.5290828062383423e-09, } , 
      {    2.2174706350247220e+01,     5.7186211413099045e+04,     6.9527886268779720e-04,     1.3157120932927950e-06,    -6.5879391871250393e-09, } , 
      {    2.5430516837061443e+01,     7.4053143728082243e+04,     5.6726653193483003e-04,     1.3050216607361212e-06,    -6.6467955680117371e-09, } , 
      {    2.1274450413819800e+01,     4.8647970522739291e+04,     9.7558360120309070e-04,     1.5771214000041238e-06,    -7.7650668048589931e-09, } , 
      {    2.4530260900634023e+01,     6.5514902837722489e+04,     8.4757127045012343e-04,     1.5664309674474498e-06,    -7.8239231857456909e-09, } , 
      {    2.7786071387448246e+01,     8.2381835152705695e+04,     7.1955893969715627e-04,     1.5557405348907759e-06,    -7.8827795666323888e-09, } , 
      {    2.6885815451020825e+01,     7.3843594262345941e+04,     9.9986367821244967e-04,     1.8171498416021045e-06,    -9.0599071843663426e-09, } , 
      {   -6.1350739422509406e+00,    -4.2129783727168295e+04,    -1.1348939119530256e-03,    -1.6922980044488989e-06,     8.1894290680709814e-09, } , 
      {   -3.7795193918641381e+00,    -3.3801092302544843e+04,    -9.8260150419069922e-04,    -1.4415791302942442e-06,     6.9534450694503298e-09, } , 
      {   -4.6797753282915586e+00,    -4.2339333192904596e+04,    -7.0229676567540571e-04,    -1.1801698235829154e-06,     5.7763174517163760e-09, } , 
      {   -1.4239648414773354e+00,    -2.5472400877921398e+04,    -8.3030909642837299e-04,    -1.1908602561395895e-06,     5.7174610708296789e-09, } , 
      {   -2.3242207779047561e+00,    -3.4010641768281144e+04,    -5.5000435791307959e-04,    -9.2945094942826071e-07,     4.5403334530957251e-09, } , 
      {    9.3158970890946713e-01,    -1.7143709453297946e+04,    -6.7801668866604686e-04,    -9.4014138198493466e-07,     4.4814770722090281e-09, } , 
      {    4.1874001957236899e+00,    -2.7677713831474193e+02,    -8.0602901941901402e-04,    -9.5083181454160860e-07,     4.4226206913223303e-09, } , 
      {    3.1333772482046607e-02,    -2.5681950343657692e+04,    -3.9771195015075351e-04,    -6.7873207527360599e-07,     3.3043494544750751e-09, } , 
      {    3.2871442592962694e+00,    -8.8150180286744944e+03,    -5.2572428090372062e-04,    -6.8942250783027994e-07,     3.2454930735883773e-09, } , 
      {    6.5429547461104924e+00,     8.0519142863087100e+03,    -6.5373661165668789e-04,    -7.0011294038695388e-07,     3.1866366927016798e-09, } , 
      {   -8.6892216394537392e-01,    -3.4220191234017446e+04,    -1.1740721163546007e-04,    -4.1732276856227737e-07,     2.1272218367411217e-09, } , 
      {    2.3868883228688489e+00,    -1.7353258919034248e+04,    -2.4541954238842728e-04,    -4.2801320111895132e-07,     2.0683654558544243e-09, } , 
      {    5.6426988096830719e+00,    -4.8632660405104525e+02,    -3.7343187314139455e-04,    -4.3870363367562522e-07,     2.0095090749677264e-09, } , 
      {    8.8985092964972949e+00,     1.6380605710932159e+04,    -5.0144420389436166e-04,    -4.4939406623229916e-07,     1.9506526940810290e-09, } , 
      {    1.4866323864414286e+00,    -2.5891499809394001e+04,     3.4885196126866113e-05,    -1.6660389440762268e-07,     8.9123783812047102e-10, } , 
      {    4.7424428732556514e+00,    -9.0245674944107959e+03,    -9.3127134626101111e-05,    -1.7729432696429663e-07,     8.3238145723377327e-10, } , 
      {    7.9982533600698744e+00,     7.8423648205724048e+03,    -2.2113946537906831e-04,    -1.8798475952097055e-07,     7.7352507634707552e-10, } , 
      {    1.1254063846884097e+01,     2.4709297135555607e+04,    -3.4915179613203559e-04,    -1.9867519207764447e-07,     7.1466869546037787e-10, } , 
      {    1.4509874333698320e+01,     4.1576229450538805e+04,    -4.7716412688500280e-04,    -2.0936562463431842e-07,     6.5581231457368022e-10, } , 
      {    3.8421869368282309e+00,    -1.7562808384770549e+04,     1.8717760388919230e-04,     8.4114979747032051e-08,    -3.4474616050017991e-10, } , 
      {    7.0979974236424539e+00,    -6.9587606978734675e+02,     5.9165273136225077e-05,     7.3424547190358117e-08,    -4.0360254138687761e-10, } , 
      {    1.0353807910456677e+01,     1.6171056245195854e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {    1.3609618397270900e+01,     3.3037988560179059e+04,    -1.9685938836970935e-04,     5.2043682077010248e-08,    -5.2131530316027296e-10, } , 
      {    1.6865428884085123e+01,     4.9904920875162257e+04,    -3.2487171912267662e-04,     4.1353249520336313e-08,    -5.8017168404697061e-10, } , 
      {    6.1977414872150334e+00,    -9.2341169601471011e+03,     3.3947001165151848e-04,     3.3483385390168678e-07,    -1.5807301591208307e-09, } , 
      {    9.4535519740292564e+00,     7.6328153548361033e+03,     2.1145768089855126e-04,     3.2414342134501284e-07,    -1.6395865400075284e-09, } , 
      {    1.2709362460843479e+01,     2.4499747669819306e+04,     8.3445350145584034e-05,     3.1345298878833894e-07,    -1.6984429208942262e-09, } , 
      {    1.5965172947657702e+01,     4.1366679984802511e+04,    -4.4566980607383177e-05,     3.0276255623166499e-07,    -1.7572993017809239e-09, } , 
      {    1.9220983434471925e+01,     5.8233612299785709e+04,    -1.7257931136035041e-04,     2.9207212367499105e-07,    -1.8161556826676215e-09, } , 
      {    1.1809106524416059e+01,     1.5961506779459552e+04,     3.6375008866087745e-04,     5.7486229549966761e-07,    -2.8755705386281790e-09, } , 
      {    1.5064917011230282e+01,     3.2828439094442758e+04,     2.3573775790791023e-04,     5.6417186294299366e-07,    -2.9344269195148769e-09, } , 
      {    1.8320727498044505e+01,     4.9695371409425963e+04,     1.0772542715494300e-04,     5.5348143038631971e-07,    -2.9932833004015743e-09, } , 
      {    2.1576537984858728e+01,     6.6562303724409154e+04,    -2.0286903598024240e-05,     5.4279099782964577e-07,    -3.0521396812882722e-09, } , 
      {    1.4164661074802861e+01,     2.4290198204083004e+04,     5.1604249642320363e-04,     8.2558116965432233e-07,    -4.1115545372488307e-09, } , 
      {    1.7420471561617084e+01,     4.1157130519066202e+04,     3.8803016567023641e-04,     8.1489073709764838e-07,    -4.1704109181355277e-09, } , 
      {    2.0676282048431307e+01,     5.8024062834049408e+04,     2.6001783491726914e-04,     8.0420030454097443e-07,    -4.2292672990222256e-09, } , 
      {    1.6520215625189664e+01,     3.2618889628706456e+04,     6.6833490418552986e-04,     1.0763000438089770e-06,    -5.3475385358694815e-09, } , 
      {    1.9776026112003887e+01,     4.9485821943689654e+04,     5.4032257343256259e-04,     1.0656096112523030e-06,    -5.4063949167561785e-09, } , 
      {    2.3031836598818110e+01,     6.6352754258672852e+04,     4.1231024267959537e-04,     1.0549191786956292e-06,    -5.4652512976428764e-09, } , 
      {    2.2131580662390689e+01,     5.7814513368313106e+04,     6.9261498119488872e-04,     1.3163284854069577e-06,    -6.6423789153768294e-09, } , 
      {    2.4487135212777492e+01,     6.6143204792936551e+04,     8.4490738895721495e-04,     1.5670473595616124e-06,    -7.8783629139974802e-09, } , 
      {   -1.4670905293338659e+00,    -2.4844098922707337e+04,    -8.3297297792128157e-04,    -1.1902438640254268e-06,     5.6630213425778897e-09, } , 
      {    8.8846402105293665e-01,    -1.6515407498083885e+04,    -6.8068057015895544e-04,    -9.3952498987077198e-07,     4.4270373439572389e-09, } , 
      {   -1.1791915374483875e-02,    -2.5053648388443631e+04,    -4.0037583164366199e-04,    -6.7811568315944331e-07,     3.2499097262232851e-09, } , 
      {    3.2440185714397392e+00,    -8.1867160734604331e+03,    -5.2838816239662921e-04,    -6.8880611571611726e-07,     3.1910533453365872e-09, } , 
      {    6.4998290582539617e+00,     8.6802162415227704e+03,    -6.5640049314959648e-04,    -6.9949654827279120e-07,     3.1321969644498898e-09, } , 
      {    2.3437626350123186e+00,    -1.6724956963820187e+04,    -2.4808342388133581e-04,    -4.2739680900478869e-07,     2.0139257276026342e-09, } , 
      {    5.5995731218265412e+00,     1.4197535116301515e+02,    -3.7609575463430303e-04,    -4.3808724156146259e-07,     1.9550693467159364e-09, } , 
      {    8.8553836086407642e+00,     1.7008907666146217e+04,    -5.0410808538727024e-04,    -4.4877767411813654e-07,     1.8962129658292389e-09, } , 
      {    1.4435066985848981e+00,    -2.5263197854179940e+04,     3.2221314633957602e-05,    -1.6598750229346000e-07,     8.3679810986868115e-10, } , 
      {    4.6993171853991207e+00,    -8.3962655391967364e+03,    -9.5791016119009615e-05,    -1.7667793485013395e-07,     7.7794172898198340e-10, } , 
      {    7.9551276722133437e+00,     8.4706667757864652e+03,    -2.2380334687197685e-04,    -1.8736836740680787e-07,     7.1908534809528565e-10, } , 
      {    1.1210938159027567e+01,     2.5337599090769669e+04,    -3.5181567762494406e-04,    -1.9805879996348179e-07,     6.6022896720858800e-10, } , 
      {    1.4466748645841790e+01,     4.2204531405752867e+04,    -4.7982800837791128e-04,    -2.0874923252015574e-07,     6.0137258632189035e-10, } , 
      {    3.7990612489717006e+00,    -1.6934506429556488e+04,     1.8451372239628377e-04,     8.4731371861194718e-08,    -3.9918588875196968e-10, } , 
      {    7.0548717357859232e+00,    -6.7574114573286352e+01,     5.6501391643316566e-05,     7.4040939304520783e-08,    -4.5804226963866738e-10, } , 
      {    1.0310682222600146e+01,     1.6799358200409915e+04,    -7.1510939109650651e-05,     6.3350506747846849e-08,    -5.1689865052536508e-10, } , 
      {    1.3566492709414369e+01,     3.3666290515393121e+04,    -1.9952326986261788e-04,     5.2660074191172908e-08,    -5.7575503141206283e-10, } , 
      {    9.4104262861727257e+00,     8.2611173100501637e+03,     2.0879379940564273e-04,     3.2475981345917546e-07,    -1.6940262682593181e-09, } , 
      {    1.2666236772986949e+01,     2.5128049625033367e+04,     8.0781468652675530e-05,     3.1406938090250157e-07,    -1.7528826491460159e-09, } , 
      {    1.5922047259801172e+01,     4.1994981940016572e+04,    -4.7230862100291687e-05,     3.0337894834582762e-07,    -1.8117390300327136e-09, } , 
      {    1.1765980836559528e+01,     1.6589808734673614e+04,     3.6108620716796897e-04,     5.7547868761383029e-07,    -2.9300102668799691e-09, } , 
      {    1.5021791323373751e+01,     3.3456741049656819e+04,     2.3307387641500170e-04,     5.6478825505715634e-07,    -2.9888666477666669e-09, } , 
      {    1.4121535386946331e+01,     2.4918500159297066e+04,     5.1337861493029504e-04,     8.2619756176848501e-07,    -4.1659942655006199e-09, } , 
      {    3.2008928835832089e+00,    -7.5584141182463727e+03,    -5.3105204388953769e-04,    -6.8818972360195458e-07,     3.1366136170847975e-09, } , 
      {    5.5564474339700114e+00,     7.7027730637707555e+02,    -3.7875963612721156e-04,    -4.3747084944729991e-07,     1.9006296184641467e-09, } , 
      {    4.6561914975425909e+00,    -7.7679635839826760e+03,    -9.8454897611918133e-05,    -1.7606154273597130e-07,     7.2350200073019363e-10, } , 
      {    7.9120019843568139e+00,     9.0989687310005247e+03,    -2.2646722836488535e-04,    -1.8675197529264522e-07,     6.6464561984349588e-10, } , 
      {    7.0117460479293934e+00,     5.6072784064077405e+02,     5.3837510150408049e-05,     7.4657331418683437e-08,    -5.1248199789045720e-10, } , 
      {    1.0267556534743616e+01,     1.7427660155623977e+04,    -7.4174820602559169e-05,     6.3966898862009502e-08,    -5.7133837877715495e-10, } , 
      {    9.3673005983161950e+00,     8.8894192652642232e+03,     2.0612991791273423e-04,     3.2537620557333814e-07,    -1.7484659965111079e-09, } , 
      {    1.2623111085130418e+01,     2.5756351580247425e+04,     7.8117587159767012e-05,     3.1468577301666425e-07,    -1.8073223773978058e-09, } , 
      {    5.5133217461134807e+00,     1.3985792615911359e+03,    -3.8142351762012009e-04,    -4.3685445733313723e-07,     1.8461898902123568e-09, } , 
      {    7.8688762965002832e+00,     9.7272706862145860e+03,    -2.2913110985779385e-04,    -1.8613558317848254e-07,     6.1020589159170612e-10, } , 
      {    1.3412348598509130e+01,     1.2472232189493001e+04,    -2.4091164597644257e-04,    -1.6000785883202313e-07,     8.4171412059513177e-10, } , 
      {    1.5767903148895932e+01,     2.0800923614116451e+04,    -8.8619238214116401e-05,     9.0711015322631619e-08,    -3.9426987802551906e-10, } , 
      {    1.1013668360265799e+01,     4.7718427200836113e+03,    -3.9586793523167725e-04,    -4.1011034087251517e-07,     2.0232583909639929e-09, } , 
      {    1.3369222910652601e+01,     1.3100534144707060e+04,    -2.4357552746935107e-04,    -1.5939146671786045e-07,     7.8727439234334201e-10, } , 
      {    1.2468966974225181e+01,     4.5622932543473098e+03,     3.6729211045942312e-05,     1.0201783999346822e-07,    -3.8985322539061107e-10, } , 
      {    1.5724777461039404e+01,     2.1429225569330512e+04,    -9.1283119707024905e-05,     9.1327407436794286e-08,    -4.4870960627730882e-10, } , 
      {    1.8080332011426208e+01,     2.9757916993953961e+04,     6.1009288055301276e-05,     3.4204628159144899e-07,    -1.6846936048979595e-09, } , 
      {    8.6149881220224653e+00,    -2.9285467493257765e+03,    -5.5082422448691191e-04,    -6.6021282291300726e-07,     3.2048026613328541e-09, } , 
      {    1.0970542672409268e+01,     5.4001446752976717e+03,    -3.9853181672458579e-04,    -4.0949394875835249e-07,     1.9688186627122033e-09, } , 
      {    1.0070286735981847e+01,    -3.1380962150620799e+03,    -1.1822707820929237e-04,    -1.4808464204702385e-07,     7.9169104497824999e-10, } , 
      {    1.3326097222796070e+01,     1.3728836099921122e+04,    -2.4623940896225960e-04,    -1.5877507460369779e-07,     7.3283466409155224e-10, } , 
      {    1.6581907709610295e+01,     3.0595768414904320e+04,    -3.7425173971522682e-04,    -1.6946550716037171e-07,     6.7397828320485449e-10, } , 
      {    1.2425841286368650e+01,     5.1905952095613702e+03,     3.4065329553033795e-05,     1.0263423210763087e-07,    -4.4429295364240089e-10, } , 
      {    1.5681651773182873e+01,     2.2057527524544574e+04,    -9.3947001199933423e-05,     9.1943799550956940e-08,    -5.0314933452909859e-10, } , 
      {    1.4781395836755452e+01,     1.3519286634184820e+04,     1.8635773731536000e-04,     3.5335310626228562e-07,    -1.6802769522630518e-09, } , 
      {    1.8037206323569677e+01,     3.0386218949168018e+04,     5.8345406562392758e-05,     3.4266267370561167e-07,    -1.7391333331497494e-09, } , 
      {    6.2163078837791321e+00,    -1.0628936218735169e+04,    -7.0578051374214673e-04,    -9.1031530495349930e-07,     4.3863469317017153e-09, } , 
      {    8.5718624341659346e+00,    -2.3002447941117161e+03,    -5.5348810597982050e-04,    -6.5959643079884458e-07,     3.1503629330810640e-09, } , 
      {    1.0927416984552737e+01,     6.0284466305117321e+03,    -4.0119569821749426e-04,    -4.0887755664418986e-07,     1.9143789344604132e-09, } , 
      {    1.4183227471366962e+01,     2.2895378945494933e+04,    -5.2920802897046153e-04,    -4.1956798920086381e-07,     1.8555225535737156e-09, } , 
      {    1.0027161048125317e+01,    -2.5097942598480195e+03,    -1.2089095970220088e-04,    -1.4746824993286117e-07,     7.3725131672646013e-10, } , 
      {    1.3282971534939540e+01,     1.4357138055135183e+04,    -2.4890329045516808e-04,    -1.5815868248953511e-07,     6.7839493583976237e-10, } , 
      {    1.6538782021753764e+01,     3.1224070370118381e+04,    -3.7691562120813530e-04,    -1.6884911504620903e-07,     6.1953855495306462e-10, } , 
      {    9.1269051116978961e+00,    -1.1048035150207772e+04,     1.5941377881309251e-04,     1.1394105677846748e-07,    -4.3987630100749300e-10, } , 
      {    1.2382715598512119e+01,     5.8188971647754306e+03,     3.1401448060125284e-05,     1.0325062422179354e-07,    -4.9873268189419070e-10, } , 
      {    1.5638526085326342e+01,     2.2685829479758631e+04,    -9.6610882692841927e-05,     9.2560191665119606e-08,    -5.5758906278088846e-10, } , 
      {    1.8894336572140567e+01,     3.9552761794741833e+04,    -2.2462321344580914e-04,     8.1869759108445672e-08,    -6.1644544366758621e-10, } , 
      {    1.4738270148898922e+01,     1.4147588589398882e+04,     1.8369385582245147e-04,     3.5396949837644825e-07,    -1.7347166805148414e-09, } , 
      {    1.7994080635713146e+01,     3.1014520904382080e+04,     5.5681525069484248e-05,     3.4327906581977430e-07,    -1.7935730614015391e-09, } , 
      {    2.1249891122527369e+01,     4.7881453219365285e+04,    -7.2330805683482949e-05,     3.3258863326310035e-07,    -1.8524294422882369e-09, } , 
      {    1.7093824699285726e+01,     2.2476280014022330e+04,     3.3598626358477768e-04,     6.0468837253110297e-07,    -2.9707006791354923e-09, } , 
      {    2.0349635186099945e+01,     3.9343212329005532e+04,     2.0797393283181044e-04,     5.9399793997442902e-07,    -3.0295570600221901e-09, } , 
      {    2.3605445672914172e+01,     5.6210144643988737e+04,     7.9961602078843232e-05,     5.8330750741775507e-07,    -3.0884134409088880e-09, } , 
      {    2.2705189736486751e+01,     4.7671903753628983e+04,     3.6026634059413664e-04,     8.4471681412908374e-07,    -4.2655410586428405e-09, } , 
      {    3.8176276455357989e+00,    -1.8329325688144560e+04,    -8.6073680299738134e-04,    -1.1604177869939913e-06,     5.5678912020705760e-09, } , 
      {    2.9173717091083784e+00,    -2.6867566578504309e+04,    -5.8043206448208794e-04,    -8.9900848028266267e-07,     4.3907635843366231e-09, } , 
      {    6.1731821959226014e+00,    -1.0000634263521108e+04,    -7.0844439523505521e-04,    -9.0969891283933662e-07,     4.3319072034499252e-09, } , 
      {    5.2729262594951809e+00,    -1.8538875153880857e+04,    -4.2813965671976176e-04,    -6.4828960612800795e-07,     3.1547795857159722e-09, } , 
      {    8.5287367463094039e+00,    -1.6719428388976557e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    1.1784547233123629e+01,     1.5194989476085546e+04,    -6.8416431822569614e-04,    -6.6967047124135585e-07,     3.0370668239425765e-09, } , 
      {    7.6284808098819834e+00,    -1.0210183729257409e+04,    -2.7584724895743558e-04,    -3.9757073197335323e-07,     1.9187955870953214e-09, } , 
      {    1.0884291296696206e+01,     6.6567485857257925e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.4140101783510431e+01,     2.3523680900708994e+04,    -5.3187191046337001e-04,    -4.1895159708670113e-07,     1.8010828253219257e-09, } , 
      {    1.7395912270324651e+01,     4.0390613215692196e+04,    -6.5988424121633717e-04,    -4.2964202964337502e-07,     1.7422264444352280e-09, } , 
      {    6.7282248734545629e+00,    -1.8748424619617159e+04,     4.4574895578578227e-06,    -1.3616142526202456e-07,     7.4166796936136801e-10, } , 
      {    9.9840353602687859e+00,    -1.8814923046339591e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.3239845847083009e+01,     1.4985440010349243e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6495656333897234e+01,     3.1852372325332442e+04,    -3.7957950270104383e-04,    -1.6823272293204638e-07,     5.6509882670127486e-10, } , 
      {    1.9751466820711453e+01,     4.8719304640315648e+04,    -5.0759183345401105e-04,    -1.7892315548872030e-07,     5.0624244581457721e-10, } , 
      {    9.0837794238413654e+00,    -1.0419733194993711e+04,     1.5674989732018400e-04,     1.1455744889263014e-07,    -4.9431602925928282e-10, } , 
      {    1.2339589910655588e+01,     6.4471991199894910e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.5595400397469811e+01,     2.3314131434972693e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.8851210884284036e+01,     4.0181063749955894e+04,    -2.2728709493871765e-04,     8.2486151222608339e-08,    -6.7088517191937597e-10, } , 
      {    2.2107021371098256e+01,     5.7047996064939100e+04,    -3.5529942569168487e-04,     7.1795718665934404e-08,    -7.2974155280607362e-10, } , 
      {    1.1439333974228168e+01,    -2.0910417703702606e+03,     3.0904230508251019e-04,     3.6527632304728487e-07,    -1.7303000278799337e-09, } , 
      {    1.4695144461042391e+01,     1.4775890544612941e+04,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {    1.7950954947856616e+01,     3.1642822859596141e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.1206765434670839e+01,     4.8509755174579346e+04,    -7.4994687176391467e-05,     3.3320502537726303e-07,    -1.9068691705400268e-09, } , 
      {    1.7050699011429195e+01,     2.3104581969236391e+04,     3.3332238209186915e-04,     6.0530476464526565e-07,    -3.0251404073872819e-09, } , 
      {    2.0306509498243415e+01,     3.9971514284219593e+04,     2.0531005133890193e-04,     5.9461433208859170e-07,    -3.0839967882739798e-09, } , 
      {    2.3562319985057641e+01,     5.6838446599202798e+04,     7.7297720585934714e-05,     5.8392389953191775e-07,    -3.1428531691606776e-09, } , 
      {    1.9406253561815994e+01,     3.1433273393859839e+04,     4.8561478985419533e-04,     8.5602363879992037e-07,    -4.2611244060079328e-09, } , 
      {    2.2662064048630221e+01,     4.8300205708843045e+04,     3.5760245910122811e-04,     8.4533320624324642e-07,    -4.3199807868946306e-09, } , 
      {    2.5917874535444444e+01,     6.5167138023826243e+04,     2.2959012834826090e-04,     8.3464277368657247e-07,    -4.3788371677813285e-09, } , 
      {    2.5017618599017020e+01,     5.6628897133466497e+04,     5.0989486686355424e-04,     1.0960520803979011e-06,    -5.5559647855152814e-09, } , 
      {    3.7745019576792682e+00,    -1.7701023732930498e+04,    -8.6340068449028981e-04,    -1.1598013948798287e-06,     5.5134514738187860e-09, } , 
      {    6.1300565080660707e+00,    -9.3723323083070463e+03,    -7.1110827672796369e-04,    -9.0908252072517394e-07,     4.2774674751981351e-09, } , 
      {    5.2298005716386502e+00,    -1.7910573198666796e+04,    -4.3080353821267029e-04,    -6.4767321401384527e-07,     3.1003398574641826e-09, } , 
      {    8.4856110584528732e+00,    -1.0436408836835953e+03,    -5.5881586896563745e-04,    -6.5836364657051922e-07,     3.0414834765774847e-09, } , 
      {    1.1741421545267098e+01,     1.5823291431299607e+04,    -6.8682819971860461e-04,    -6.6905407912719317e-07,     2.9826270956907869e-09, } , 
      {    7.5853551220254527e+00,    -9.5818817740433478e+03,    -2.7851113045034411e-04,    -3.9695433985919055e-07,     1.8643558588435317e-09, } , 
      {    1.0841165608839676e+01,     7.2850505409398529e+03,    -4.0652346120331132e-04,    -4.0764477241586450e-07,     1.8054994779568337e-09, } , 
      {    1.4096976095653901e+01,     2.4151982855923055e+04,    -5.3453579195627849e-04,    -4.1833520497253845e-07,     1.7466430970701358e-09, } , 
      {    9.9409096724122552e+00,    -1.2531903494198987e+03,    -1.2621872268801790e-04,    -1.4623546570453586e-07,     6.2837186022288060e-10, } , 
      {    1.3196720159226478e+01,     1.5613741965563302e+04,    -2.5423105344098514e-04,    -1.5692589826120981e-07,     5.6951547933618284e-10, } , 
      {    1.6452530646040703e+01,     3.2480674280546504e+04,    -3.8224338419395236e-04,    -1.6761633081788373e-07,     5.1065909844948509e-10, } , 
      {    1.2296464222799058e+01,     7.0755010752035514e+03,     2.6073685074308262e-05,     1.0448340845011887e-07,    -6.0761213839777023e-10, } , 
      {    1.5552274709613281e+01,     2.3942433390186754e+04,    -1.0193864567865896e-04,     9.3792975893444940e-08,    -6.6646851928446799e-10, } , 
      {    1.8808085196427506e+01,     4.0809365705169956e+04,    -2.2995097643162615e-04,     8.3102543336771005e-08,    -7.2532490017116574e-10, } , 
      {    1.4652018773185860e+01,     1.5404192499827001e+04,     1.7836609283663446e-04,     3.5520228260477361e-07,    -1.8435961370184212e-09, } , 
      {    1.7907829260000085e+01,     3.2271124814810202e+04,     5.0353762083667226e-05,     3.4451185004809966e-07,    -1.9024525179051186e-09, } , 
      {    2.1163639746814308e+01,     4.9138057129793408e+04,    -7.7658568669299985e-05,     3.3382141749142571e-07,    -1.9613088987918165e-09, } , 
      {    1.7007573323572664e+01,     2.3732883924450452e+04,     3.3065850059896062e-04,     6.0592115675942833e-07,    -3.0795801356390716e-09, } , 
      {    2.0263383810386884e+01,     4.0599816239433654e+04,     2.0264616984599343e-04,     5.9523072420275438e-07,    -3.1384365165257694e-09, } , 
      {    2.3519194297201111e+01,     5.7466748554416859e+04,     7.4633839093026197e-05,     5.8454029164608043e-07,    -3.1972928974124673e-09, } , 
      {    2.2618938360773690e+01,     4.8928507664057106e+04,     3.5493857760831958e-04,     8.4594959835740910e-07,    -4.3744205151464207e-09, } , 
      {    2.4974492911160489e+01,     5.7257199088680558e+04,     5.0723098537064576e-04,     1.0966684725120638e-06,    -5.6104045137670715e-09, } , 
      {    6.0869308202095400e+00,    -8.7440303530929868e+03,    -7.1377215822087227e-04,    -9.0846612861101126e-07,     4.2230277469463459e-09, } , 
      {    8.4424853705963425e+00,    -4.1533892846953495e+02,    -5.6147975045854604e-04,    -6.5774725445635654e-07,     2.9870437483256946e-09, } , 
      {    1.0798039920983145e+01,     7.9133524961539133e+03,    -4.0918734269621980e-04,    -4.0702838030170187e-07,     1.7510597497050440e-09, } , 
      {    9.8977839845557245e+00,    -6.2488839420583827e+02,    -1.2888260418092643e-04,    -1.4561907359037318e-07,     5.7393213197109073e-10, } , 
      {    1.3153594471369948e+01,     1.6242043920777363e+04,    -2.5689493493389362e-04,    -1.5630950614704713e-07,     5.1507575108439298e-10, } , 
      {    1.2253338534942527e+01,     7.7038030304176118e+03,     2.3409803581399751e-05,     1.0509980056428154e-07,    -6.6205186664956010e-10, } , 
      {    1.5509149021756750e+01,     2.4570735345400812e+04,    -1.0460252717156747e-04,     9.4409368007607607e-08,    -7.2090824753625785e-10, } , 
      {    1.8764959508570975e+01,     4.1437667660384017e+04,    -2.3261485792453468e-04,     8.3718935450933672e-08,    -7.7976462842295561e-10, } , 
      {    1.7864703572143554e+01,     3.2899426770024264e+04,     4.7689880590758715e-05,     3.4512824216226229e-07,    -1.9568922461569087e-09, } , 
      {    2.0220258122530353e+01,     4.1228118194647715e+04,     1.9998228835308490e-04,     5.9584711631691706e-07,    -3.1928762447775595e-09, } , 
      {    8.3993596827398118e+00,     2.1296302674452545e+02,    -5.6414363195145451e-04,    -6.5713086234219386e-07,     2.9326040200739050e-09, } , 
      {    7.4991037463123922e+00,    -8.3252778636152289e+03,    -2.8383889343616112e-04,    -3.9572155563086524e-07,     1.7554764023399520e-09, } , 
      {    1.0754914233126614e+01,     8.5416544513679728e+03,    -4.1185122418912833e-04,    -4.0641198818753919e-07,     1.6966200214532541e-09, } , 
      {    1.3110468783513417e+01,     1.6870345875991425e+04,    -2.5955881642680215e-04,    -1.5569311403288447e-07,     4.6063602283260321e-10, } , 
      {    1.2210212847085996e+01,     8.3321049856316713e+03,     2.0745922088491237e-05,     1.0571619267844419e-07,    -7.1649159490134987e-10, } , 
      {    1.5466023333900219e+01,     2.5199037300614873e+04,    -1.0726640866447598e-04,     9.5025760121770260e-08,    -7.7534797578804762e-10, } , 
      {    1.6298386535135464e+01,     1.1286615954646386e+04,    -4.2363176030777707e-04,    -3.8028426384107976e-07,     1.9281282504566797e-09, } , 
      {    1.8653941085522266e+01,     1.9615307379269838e+04,    -2.7133935254545089e-04,    -1.2956538968642501e-07,     6.9214425183602887e-10, } , 
      {    2.1009495635909069e+01,     2.7943998803893286e+04,    -1.1904694478312470e-04,     1.2115348446822971e-07,    -5.4383974678462196e-10, } , 
      {    1.3899706296892131e+01,     3.5862264852369990e+03,    -5.7858804956301179e-04,    -6.3038674588157169e-07,     3.1096725208255405e-09, } , 
      {    1.6255260847278933e+01,     1.1914917909860447e+04,    -4.2629564180068555e-04,    -3.7966787172691708e-07,     1.8736885222048896e-09, } , 
      {    1.8610815397665736e+01,     2.0243609334483899e+04,    -2.7400323403835937e-04,    -1.2894899757226236e-07,     6.3770452358423900e-10, } , 
      {    2.0966369948052538e+01,     2.8572300759107347e+04,    -1.2171082627603322e-04,     1.2176987658239236e-07,    -5.9827947503641183e-10, } , 
      {    2.3321924498439341e+01,     3.6900992183730792e+04,     3.0581581486292965e-05,     3.7248875073704708e-07,    -1.8342634736570627e-09, } , 
      {    2.5677479048826143e+01,     4.5229683608354244e+04,     1.8287398924861915e-04,     6.2320762489170191e-07,    -3.0702474722777137e-09, } , 
      {    1.3856580609035600e+01,     4.2145284404510594e+03,    -5.8125193105592026e-04,    -6.2977035376740911e-07,     3.0552327925737508e-09, } , 
      {    1.6212135159422402e+01,     1.2543219865074509e+04,    -4.2895952329359408e-04,    -3.7905147961275440e-07,     1.8192487939531000e-09, } , 
      {    1.9467945646236625e+01,     2.9410152180057710e+04,    -5.5697185404656130e-04,    -3.8974191216942834e-07,     1.7603924130664023e-09, } , 
      {    1.5311879222994982e+01,     4.0049789747147561e+03,    -1.4865478477830068e-04,    -1.1764217290142573e-07,     6.4212117621914698e-10, } , 
      {    1.8567689709809205e+01,     2.0871911289697960e+04,    -2.7666711553126790e-04,    -1.2833260545809968e-07,     5.8326479533244923e-10, } , 
      {    2.1823500196623428e+01,     3.7738843604681155e+04,    -4.0467944628423512e-04,    -1.3902303801477362e-07,     5.2440841444575148e-10, } , 
      {    1.7667433773381784e+01,     1.2333670399338207e+04,     3.6376229840254941e-06,     1.3307670125322899e-07,    -5.9386282240150385e-10, } , 
      {    2.0923244260196007e+01,     2.9200602714321409e+04,    -1.2437470776894172e-04,     1.2238626869655504e-07,    -6.5271920328820160e-10, } , 
      {    2.4179054747010230e+01,     4.6067535029304607e+04,    -2.5238703852190894e-04,     1.1169583613988111e-07,    -7.1157558417489935e-10, } , 
      {    2.0022988323768587e+01,     2.0662361823961659e+04,     1.5593003074635168e-04,     3.8379557540788371e-07,    -1.8298468210221547e-09, } , 
      {    2.3278798810582810e+01,     3.7529294138944853e+04,     2.7917699993384458e-05,     3.7310514285120976e-07,    -1.8887032019088525e-09, } , 
      {    2.6534609297397033e+01,     5.4396226453928059e+04,    -1.0009463075958276e-04,     3.6241471029453582e-07,    -1.9475595827955504e-09, } , 
      {    2.5634353360969612e+01,     4.5857985563568305e+04,     1.8021010775571064e-04,     6.2382401700586448e-07,    -3.1246872005295034e-09, } , 
      {    2.7989907911356415e+01,     5.4186676988191757e+04,     3.3250251551803682e-04,     8.7454289116051920e-07,    -4.3606711991501542e-09, } , 
      {    9.1023458204054641e+00,    -1.1814552453581784e+04,    -8.8850062807348110e-04,    -1.1305917099625559e-06,     5.4727610615632624e-09, } , 
      {    1.1457900370792267e+01,    -3.4858610289583321e+03,    -7.3620822031115498e-04,    -8.7987283580790115e-07,     4.2367770629426115e-09, } , 
      {    1.3813454921179069e+01,     4.8428303956651198e+03,    -5.8391581254882874e-04,    -6.2915396165324643e-07,     3.0007930643219611e-09, } , 
      {    1.7069265407993292e+01,     2.1709762710648320e+04,    -7.1192814330179601e-04,    -6.3984439420992038e-07,     2.9419366834352633e-09, } , 
      {    1.2913198984751649e+01,    -3.6954104946946336e+03,    -3.0361107403353540e-04,    -3.6774465494191777e-07,     1.8236654465880079e-09, } , 
      {    1.6169009471565872e+01,     1.3171521820288568e+04,    -4.3162340478650261e-04,    -3.7843508749859172e-07,     1.7648090657013101e-09, } , 
      {    1.9424819958380095e+01,     3.0038454135271772e+04,    -5.5963573553946978e-04,    -3.8912552005526566e-07,     1.7059526848146124e-09, } , 
      {    1.5268753535138451e+01,     4.6332809299288165e+03,    -1.5131866627120919e-04,    -1.1702578078726307e-07,     5.8768144796735722e-10, } , 
      {    1.8524564021952674e+01,     2.1500213244912018e+04,    -2.7933099702417643e-04,    -1.2771621334393702e-07,     5.2882506708065947e-10, } , 
      {    2.1780374508766897e+01,     3.8367145559895216e+04,    -4.0734332777714365e-04,    -1.3840664590061097e-07,     4.6996868619396172e-10, } , 
      {    1.7624308085525254e+01,     1.2961972354552267e+04,     9.7374149111698154e-07,     1.3369309336739165e-07,    -6.4830255065329361e-10, } , 
      {    2.0880118572339477e+01,     2.9828904669535470e+04,    -1.2703858926185022e-04,     1.2300266081071770e-07,    -7.0715893153999136e-10, } , 
      {    2.4135929059153700e+01,     4.6695836984518668e+04,    -2.5505092001481747e-04,     1.1231222825404376e-07,    -7.6601531242668911e-10, } , 
      {    1.9979862635912056e+01,     2.1290663779175717e+04,     1.5326614925344317e-04,     3.8441196752204639e-07,    -1.8842865492739443e-09, } , 
      {    2.3235673122726279e+01,     3.8157596094158915e+04,     2.5253818500475944e-05,     3.7372153496537244e-07,    -1.9431429301606422e-09, } , 
      {    2.6491483609540502e+01,     5.5024528409142120e+04,    -1.0275851225249127e-04,     3.6303110240869850e-07,    -2.0019993110473400e-09, } , 
      {    2.2335417186298859e+01,     2.9619355203799169e+04,     3.0555855701576933e-04,     6.3513084167670100e-07,    -3.1202705478945952e-09, } , 
      {    2.5591227673113082e+01,     4.6486287518782367e+04,     1.7754622626280214e-04,     6.2444040912002716e-07,    -3.1791269287812930e-09, } , 
      {    2.8847038159927305e+01,     6.3353219833765572e+04,     4.9533895509834908e-05,     6.1374997656335321e-07,    -3.2379833096679909e-09, } , 
      {    2.4690971736685661e+01,     3.7948046628422613e+04,     4.5785096477809551e-04,     8.8584971583135572e-07,    -4.3562545465152464e-09, } , 
      {    2.7946782223499884e+01,     5.4814978943405818e+04,     3.2983863402512829e-04,     8.7515928327468188e-07,    -4.4151109274019443e-09, } , 
      {    3.0302336773886687e+01,     6.3143670368029270e+04,     4.8213104178745447e-04,     1.1258781574293366e-06,    -5.6510949260225951e-09, } , 
      {    6.7036655821621309e+00,    -1.9514941922991169e+04,    -1.0434569173287157e-03,    -1.3806941920030479e-06,     6.6543053319321240e-09, } , 
      {    9.0592201325489334e+00,    -1.1186250498367725e+04,    -8.9116450956638969e-04,    -1.1299753178483932e-06,     5.4183213333114731e-09, } , 
      {    8.1589641961215129e+00,    -1.9724491388727474e+04,    -6.1085977105109629e-04,    -8.6856601113706463e-07,     4.2411937155775202e-09, } , 
      {    1.1414774682935736e+01,    -2.8575590737442717e+03,    -7.3887210180406356e-04,    -8.7925644369373847e-07,     4.1823373346908223e-09, } , 
      {    1.0514518746508315e+01,    -1.1395799964104022e+04,    -4.5856736328877005e-04,    -6.1784713698240991e-07,     3.0052097169568689e-09, } , 
      {    1.3770329233322538e+01,     5.4711323508791802e+03,    -5.8657969404173733e-04,    -6.2853756953908375e-07,     2.9463533360701711e-09, } , 
      {    1.7026139720136761e+01,     2.2338064665862381e+04,    -7.1459202479470460e-04,    -6.3922800209575770e-07,     2.8874969551834732e-09, } , 
      {    9.6142628100808949e+00,    -1.9934040854463776e+04,    -1.7826262477347668e-04,    -3.5643783027108125e-07,     1.8280820992229159e-09, } , 
      {    1.2870073296895118e+01,    -3.0671085394805732e+03,    -3.0627495552644387e-04,    -3.6712826282775514e-07,     1.7692257183362183e-09, } , 
      {    1.6125883783709341e+01,     1.3799823775502628e+04,    -4.3428728627941109e-04,    -3.7781869538442909e-07,     1.7103693374495204e-09, } , 
      {    1.9381694270523564e+01,     3.0666756090485829e+04,    -5.6229961703237836e-04,    -3.8850912794110304e-07,     1.6515129565628228e-09, } , 
      {    2.2637504757337787e+01,     4.7533688405469031e+04,    -6.9031194778534563e-04,    -3.9919956049777693e-07,     1.5926565756761251e-09, } , 
      {    1.1969817360467697e+01,    -1.1605349429840324e+04,    -2.5970217011150495e-05,    -1.0571895611642649e-07,     5.9209810060226500e-10, } , 
      {    1.5225627847281920e+01,     5.2615828851428769e+03,    -1.5398254776411772e-04,    -1.1640938867310041e-07,     5.3324171971556735e-10, } , 
      {    1.8481438334096143e+01,     2.2128515200126079e+04,    -2.8199487851708491e-04,    -1.2709982122977434e-07,     4.7438533882886960e-10, } , 
      {    2.1737248820910366e+01,     3.8995447515109277e+04,    -4.1000720927005213e-04,    -1.3779025378644829e-07,     4.1552895794217190e-10, } , 
      {    2.4993059307724590e+01,     5.5862379830092483e+04,    -5.3801954002301940e-04,    -1.4848068634312221e-07,     3.5667257705547425e-10, } , 
      {    1.4325371910854500e+01,    -3.2766580052168747e+03,     1.2632219075117568e-04,     1.4499991803822825e-07,    -6.4388589801838583e-10, } , 
      {    1.7581182397668723e+01,     1.3590274309766326e+04,    -1.6901400017915301e-06,     1.3430948548155433e-07,    -7.0274227890508348e-10, } , 
      {    2.0836992884482946e+01,     3.0457206624749528e+04,    -1.2970247075475876e-04,     1.2361905292488038e-07,    -7.6159865979178123e-10, } , 
      {    2.4092803371297169e+01,     4.7324138939732729e+04,    -2.5771480150772595e-04,     1.1292862036820643e-07,    -8.2045504067847898e-10, } , 
      {    2.7348613858111392e+01,     6.4191071254715935e+04,    -3.8572713226069316e-04,     1.0223818781153249e-07,    -8.7931142156517663e-10, } , 
      {    1.6680926461241302e+01,     5.0520334194065754e+03,     2.7861459851350189e-04,     3.9571879219288291e-07,    -1.8798698966390366e-09, } , 
      {    1.9936736948055525e+01,     2.1918965734389778e+04,     1.5060226776053464e-04,     3.8502835963620902e-07,    -1.9387262775257344e-09, } , 
      {    2.3192547434869748e+01,     3.8785898049372976e+04,     2.2589937007567433e-05,     3.7433792707953507e-07,    -1.9975826584124323e-09, } , 
      {    2.6448357921683971e+01,     5.5652830364356181e+04,    -1.0542239374539978e-04,     3.6364749452286112e-07,    -2.0564390392991301e-09, } , 
      {    2.9704168408498195e+01,     7.2519762679339387e+04,    -2.3343472449836701e-04,     3.5295706196618723e-07,    -2.1152954201858276e-09, } , 
      {    2.2292291498442328e+01,     3.0247657159013226e+04,     3.0289467552286085e-04,     6.3574723379086368e-07,    -3.1747102761463852e-09, } , 
      {    2.5548101985256551e+01,     4.7114589473996428e+04,     1.7488234476989361e-04,     6.2505680123418984e-07,    -3.2335666570330831e-09, } , 
      {    2.8803912472070774e+01,     6.3981521788979633e+04,     4.6870014016926397e-05,     6.1436636867751589e-07,    -3.2924230379197810e-09, } , 
      {    2.4647846048829130e+01,     3.8576348583636674e+04,     4.5518708328518703e-04,     8.8646610794551840e-07,    -4.4106942747670357e-09, } , 
      {    2.7903656535643353e+01,     5.5443280898619880e+04,     3.2717475253221982e-04,     8.7577567538884456e-07,    -4.4695506556537335e-09, } , 
      {    3.1159467022457576e+01,     7.2310213213603070e+04,     1.9916242177925257e-04,     8.6508524283217061e-07,    -4.5284070365404314e-09, } , 
      {    2.7003400599215933e+01,     4.6905040008260126e+04,     6.0747949104751316e-04,     1.1371849821001731e-06,    -5.6466782733876865e-09, } , 
      {    3.0259211086030156e+01,     6.3771972323243332e+04,     4.7946716029454600e-04,     1.1264945495434993e-06,    -5.7055346542743843e-09, } , 
      {    3.2614765636416962e+01,     7.2100663747866784e+04,     6.3175956805687202e-04,     1.3772134236981540e-06,    -6.9415186528950352e-09, } , 
      {    4.3049853439187977e+00,    -2.7215331392400560e+04,    -1.1984132065839506e-03,    -1.6307966740435400e-06,     7.8358496023009864e-09, } , 
      {    3.4047294074913772e+00,    -3.5753572282760309e+04,    -9.1810846806865713e-04,    -1.3693873673322114e-06,     6.6587219845670317e-09, } , 
      {    6.6605398943056002e+00,    -1.8886639967777108e+04,    -1.0461207988216243e-03,    -1.3800777998888852e-06,     6.5998656036803339e-09, } , 
      {    5.7602839578781797e+00,    -2.7424880858136865e+04,    -7.6581606030633089e-04,    -1.1186684931775567e-06,     5.4227379859463809e-09, } , 
      {    9.0160944446924027e+00,    -1.0557948543153663e+04,    -8.9382839105929817e-04,    -1.1293589257342305e-06,     5.3638816050596831e-09, } , 
      {    8.1158385082649822e+00,    -1.9096189433513413e+04,    -6.1352365254400477e-04,    -8.6794961902290195e-07,     4.1867539873257301e-09, } , 
      {    1.1371648995079205e+01,    -2.2292571185302113e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    1.4627459481893428e+01,     1.4637675196452990e+04,    -8.6954831404993920e-04,    -8.8933048413624974e-07,     4.0690412255523344e-09, } , 
      {    7.2155825718375617e+00,    -2.7634430323873163e+04,    -3.3321891402871137e-04,    -6.0654031231157328e-07,     3.0096263695917771e-09, } , 
      {    1.0471393058651785e+01,    -1.0767498008889961e+04,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {    1.3727203545466008e+01,     6.0994343060932406e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.6983014032280231e+01,     2.2966366621076442e+04,    -7.1725590628761307e-04,    -6.3861160998159502e-07,     2.8330572269316835e-09, } , 
      {    2.0238824519094454e+01,     3.9833298936059648e+04,    -8.4526823704058024e-04,    -6.4930204253826897e-07,     2.7742008460449861e-09, } , 
      {    9.5711371222243642e+00,    -1.9305738899249714e+04,    -1.8092650626638519e-04,    -3.5582143815691857e-07,     1.7736423709711261e-09, } , 
      {    1.2826947609038587e+01,    -2.4388065842665128e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    1.6082758095852810e+01,     1.4428125730716689e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9338568582667033e+01,     3.1295058045699890e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    2.2594379069481256e+01,     4.8161990360683092e+04,    -6.9297582927825411e-04,    -3.9858316838361425e-07,     1.5382168474243353e-09, } , 
      {    1.1926691672611167e+01,    -1.0977047474626264e+04,    -2.8634098504059006e-05,    -1.0510256400226382e-07,     5.3765837235047523e-10, } , 
      {    1.5182502159425390e+01,     5.8898848403569373e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.8438312646239613e+01,     2.2756817155340141e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1694123133053836e+01,     3.9623749470323339e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    2.4949933619868059e+01,     5.6490681785306544e+04,    -5.4068342151592788e-04,    -1.4786429422895956e-07,     3.0223284880368443e-10, } , 
      {    1.4282246222997969e+01,    -2.6483560500028143e+03,     1.2365830925826718e-04,     1.4561631015239090e-07,    -6.9832562627017560e-10, } , 
      {    1.7538056709812192e+01,     1.4218576264980387e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.0793867196626415e+01,     3.1085508579963589e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.4049677683440638e+01,     4.7952440894946791e+04,    -2.6037868300063448e-04,     1.1354501248236910e-07,    -8.7489476893026875e-10, } , 
      {    2.7305488170254861e+01,     6.4819373209929996e+04,    -3.8839101375360169e-04,     1.0285457992569516e-07,    -9.3375114981696640e-10, } , 
      {    1.6637800773384772e+01,     5.6803353746206358e+03,     2.7595071702059336e-04,     3.9633518430704559e-07,    -1.9343096248908262e-09, } , 
      {    1.9893611260198995e+01,     2.2547267689603839e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {    2.3149421747013218e+01,     3.9414200004587037e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.6405232233827441e+01,     5.6281132319570243e+04,    -1.0808627523830830e-04,     3.6426388663702380e-07,    -2.1108787675509198e-09, } , 
      {    2.9661042720641664e+01,     7.3148064634553448e+04,    -2.3609860599127551e-04,     3.5357345408034991e-07,    -2.1697351484376172e-09, } , 
      {    1.8993355323771574e+01,     1.4009026799244086e+04,     4.2824312478291954e-04,     6.4705405846170031e-07,    -3.1702936235114770e-09, } , 
      {    2.2249165810585797e+01,     3.0875959114227288e+04,     3.0023079402995232e-04,     6.3636362590502636e-07,    -3.2291500043981749e-09, } , 
      {    2.5504976297400020e+01,     4.7742891429210489e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    2.8760786784214243e+01,     6.4609823744193694e+04,     4.4206132524017886e-05,     6.1498276079167857e-07,    -3.3468627661715706e-09, } , 
      {    3.2016597271028466e+01,     8.1476756059176900e+04,    -8.3806198228949331e-05,     6.0429232823500463e-07,    -3.4057191470582681e-09, } , 
      {    2.4604720360972600e+01,     3.9204650538850736e+04,     4.5252320179227850e-04,     8.8708250005968108e-07,    -4.4651340030188257e-09, } , 
      {    2.7860530847786823e+01,     5.6071582853833941e+04,     3.2451087103931128e-04,     8.7639206750300724e-07,    -4.5239903839055236e-09, } , 
      {    3.1116341334601046e+01,     7.2938515168817132e+04,     1.9649854028634407e-04,     8.6570163494633329e-07,    -4.5828467647922214e-09, } , 
      {    2.6960274911359402e+01,     4.7533341963474188e+04,     6.0481560955460468e-04,     1.1378013742143358e-06,    -5.7011180016394766e-09, } , 
      {    3.0216085398173625e+01,     6.4400274278457393e+04,     4.7680327880163747e-04,     1.1271109416576620e-06,    -5.7599743825261744e-09, } , 
      {    3.3471895884987845e+01,     8.1267206593440584e+04,     3.4879094804867025e-04,     1.1164205091009879e-06,    -5.8188307634128723e-09, } , 
      {    3.2571639948560431e+01,     7.2728965703080845e+04,     6.2909568656396354e-04,     1.3778298158123167e-06,    -6.9959583811468252e-09, } , 
      {    3.4927194498947230e+01,     8.1057657127704282e+04,     7.8138809432628988e-04,     1.6285486899669714e-06,    -8.2319423797674753e-09, } , 
      {    6.6174142064490695e+00,    -1.8258338012563047e+04,    -1.0487846803145329e-03,    -1.3794614077747226e-06,     6.5454258754285438e-09, } , 
      {    8.9729687568358720e+00,    -9.9296465879396019e+03,    -8.9649227255220664e-04,    -1.1287425336200678e-06,     5.3094418768078930e-09, } , 
      {    1.1328523307222675e+01,    -1.6009551633161509e+03,    -7.4419986478988052e-04,    -8.7802365946541311e-07,     4.0734578781872421e-09, } , 
      {    1.0428267370795254e+01,    -1.0139196053675900e+04,    -4.6389512627458712e-04,    -6.1661435275408455e-07,     2.8963302604532896e-09, } , 
      {    1.3684077857609477e+01,     6.7277362613073010e+03,    -5.9190745702755428e-04,    -6.2730478531075839e-07,     2.8374738795665917e-09, } , 
      {    1.6939888344423700e+01,     2.3594668576290504e+04,    -7.1991978778052155e-04,    -6.3799521786743234e-07,     2.7786174986798939e-09, } , 
      {    1.2783821921182057e+01,    -1.8105046290524524e+03,    -3.1160271851226094e-04,    -3.6589547859942978e-07,     1.6603462618326385e-09, } , 
      {    1.6039632407996280e+01,     1.5056427685930750e+04,    -4.3961504926522815e-04,    -3.7658591115610373e-07,     1.6014898809459407e-09, } , 
      {    1.9295442894810503e+01,     3.1923360000913952e+04,    -5.6762738001819532e-04,    -3.8727634371277768e-07,     1.5426335000592430e-09, } , 
      {    1.5139376471568859e+01,     6.5181867955709977e+03,    -1.5931031074993473e-04,    -1.1517660444477507e-07,     4.2436226321198777e-10, } , 
      {    1.8395186958383082e+01,     2.3385119110554202e+04,    -2.8732264150290197e-04,    -1.2586703700144904e-07,     3.6550588232529002e-10, } , 
      {    2.1650997445197305e+01,     4.0252051425537400e+04,    -4.1533497225586919e-04,    -1.3655746955812298e-07,     3.0664950143859227e-10, } , 
      {    2.4906807932011528e+01,     5.7118983740520605e+04,    -5.4334730300883635e-04,    -1.4724790211479690e-07,     2.4779312055189462e-10, } , 
      {    1.7494931021955662e+01,     1.4846878220194449e+04,    -7.0179029876085535e-06,     1.3554226970987963e-07,    -8.1162173540866301e-10, } , 
      {    2.0750741508769885e+01,     3.1713810535177650e+04,    -1.3503023374057576e-04,     1.2485183715320568e-07,    -8.7047811629536076e-10, } , 
      {    2.4006551995584108e+01,     4.8580742850160852e+04,    -2.6304256449354301e-04,     1.1416140459653176e-07,    -9.2933449718205851e-10, } , 
      {    1.6594675085528241e+01,     6.3086373298346962e+03,     2.7328683552768483e-04,     3.9695157642120827e-07,    -1.9887493531426159e-09, } , 
      {    1.9850485572342464e+01,     2.3175569644817901e+04,     1.4527450477471764e-04,     3.8626114386453438e-07,    -2.0476057340293137e-09, } , 
      {    2.3106296059156687e+01,     4.0042501959801099e+04,     1.7262174021750411e-05,     3.7557071130786043e-07,    -2.1064621149160116e-09, } , 
      {    2.6362106545970910e+01,     5.6909434274784304e+04,    -1.1075015673121681e-04,     3.6488027875118648e-07,    -2.1653184958027094e-09, } , 
      {    2.2206040122729267e+01,     3.1504261069441349e+04,     2.9756691253704379e-04,     6.3698001801918904e-07,    -3.2835897326499646e-09, } , 
      {    2.5461850609543490e+01,     4.8371193384424550e+04,     1.6955458178407660e-04,     6.2628958546251520e-07,    -3.3424461135366624e-09, } , 
      {    2.8717661096357713e+01,     6.5238125699407756e+04,     4.1542251031109375e-05,     6.1559915290584125e-07,    -3.4013024944233603e-09, } , 
      {    2.4561594673116069e+01,     3.9832952494064797e+04,     4.4985932029936997e-04,     8.8769889217384376e-07,    -4.5195737312706158e-09, } , 
      {    2.7817405159930292e+01,     5.6699884809048002e+04,     3.2184698954640275e-04,     8.7700845961716992e-07,    -4.5784301121573137e-09, } , 
      {    3.1073215646744515e+01,     7.3566817124031193e+04,     1.9383465879343556e-04,     8.6631802706049597e-07,    -4.6372864930440115e-09, } , 
      {    3.0172959710317095e+01,     6.5028576233671454e+04,     4.7413939730872893e-04,     1.1277273337718246e-06,    -5.8144141107779645e-09, } , 
      {    1.1285397619366144e+01,    -9.7265320810209050e+02,    -7.4686374628278910e-04,    -8.7740726735125043e-07,     4.0190181499354529e-09, } , 
      {    1.3640952169752946e+01,     7.3560382165213614e+03,    -5.9457133852046287e-04,    -6.2668839319659571e-07,     2.7830341513148017e-09, } , 
      {    1.6896762656567169e+01,     2.4222970531504565e+04,    -7.2258366927343014e-04,    -6.3737882575326966e-07,     2.7241777704281038e-09, } , 
      {    1.2740696233325526e+01,    -1.1822026738383920e+03,    -3.1426660000516941e-04,    -3.6527908648526715e-07,     1.6059065335808489e-09, } , 
      {    1.5996506720139749e+01,     1.5684729641144810e+04,    -4.4227893075813663e-04,    -3.7596951904194110e-07,     1.5470501526941510e-09, } , 
      {    1.9252317206953972e+01,     3.2551661956128009e+04,    -5.7029126151110390e-04,    -3.8665995159861505e-07,     1.4881937718074534e-09, } , 
      {    1.5096250783712328e+01,     7.1464887507850581e+03,    -1.6197419224284326e-04,    -1.1456021233061241e-07,     3.6992253496019800e-10, } , 
      {    1.8352061270526551e+01,     2.4013421065768263e+04,    -2.8998652299581045e-04,    -1.2525064488728636e-07,     3.1106615407350025e-10, } , 
      {    2.1607871757340774e+01,     4.0880353380751461e+04,    -4.1799885374877767e-04,    -1.3594107744396030e-07,     2.5220977318680250e-10, } , 
      {    1.7451805334099131e+01,     1.5475180175408508e+04,    -9.6817844805170651e-06,     1.3615866182404231e-07,    -8.6606146366045288e-10, } , 
      {    2.0707615820913354e+01,     3.2342112490391708e+04,    -1.3769411523348429e-04,     1.2546822926736836e-07,    -9.2491784454715063e-10, } , 
      {    2.3963426307727577e+01,     4.9209044805374913e+04,    -2.6570644598645148e-04,     1.1477779671069443e-07,    -9.8377422543384828e-10, } , 
      {    1.9807359884485933e+01,     2.3803871600031962e+04,     1.4261062328180910e-04,     3.8687753597869701e-07,    -2.1020454622811038e-09, } , 
      {    2.3063170371300156e+01,     4.0670803915015160e+04,     1.4598292528841899e-05,     3.7618710342202306e-07,    -2.1609018431678017e-09, } , 
      {    2.5418724921686959e+01,     4.8999495339638612e+04,     1.6689070029116807e-04,     6.2690597757667788e-07,    -3.3968858417884525e-09, } , 
      {    1.5953381032283218e+01,     1.6313031596358869e+04,    -4.4494281225104516e-04,    -3.7535312692777842e-07,     1.4926104244423612e-09, } , 
      {    1.8308935582670021e+01,     2.4641723020982321e+04,    -2.9265040448871898e-04,    -1.2463425277312370e-07,     2.5662642582171044e-10, } , 
      {    2.0664490133056823e+01,     3.2970414445605769e+04,    -1.4035799672639280e-04,     1.2608462138153102e-07,    -9.7935757279894050e-10, } , 
      {    1.2654444857612464e+01,     7.4401236589728796e+01,    -3.1959436299098647e-04,    -3.6404630225694179e-07,     1.4970270770772691e-09, } , 
      {    2.1410601958579008e+01,     2.0314597010065409e+04,    -4.6205111135551091e-04,    -3.4799261835299362e-07,     1.6152391969422074e-09, } , 
      {    2.6121711059352613e+01,     3.6971979859312305e+04,    -1.5746629583085855e-04,     1.5344512995631581e-07,    -8.5672880029909427e-10, } , 
      {    1.9011921720335675e+01,     1.2614207540656020e+04,    -6.1700740061074557e-04,    -5.9809510039348572e-07,     2.7967834673110686e-09, } , 
      {    2.1367476270722477e+01,     2.0942898965279470e+04,    -4.6471499284841944e-04,    -3.4737622623883094e-07,     1.5607994686904175e-09, } , 
      {    2.0467220334295057e+01,     1.2404658074919716e+04,    -1.8441025433312602e-04,    -8.5966919527502303e-08,     3.8367185095646439e-10, } , 
      {    2.3723030821109280e+01,     2.9271590389902914e+04,    -3.1242258508609326e-04,    -9.6657352084176251e-08,     3.2481547006976674e-10, } , 
      {    2.2822774884681859e+01,     2.0733349499543168e+04,    -3.2117846570799848e-05,     1.6475195462715242e-07,    -8.5231214766418639e-10, } , 
      {    2.6078585371496082e+01,     3.7600281814526366e+04,    -1.6013017732376705e-04,     1.5406152207047847e-07,    -9.1116852855088403e-10, } , 
      {    2.8434139921882885e+01,     4.5928973239149818e+04,    -7.8377695614408829e-06,     4.0478039622513321e-07,    -2.1471525271715348e-09, } , 
      {    1.6613241482092342e+01,     4.9138180712466283e+03,    -7.7196368986598028e-04,    -8.4819758243397776e-07,     3.9783277376799301e-09, } , 
      {    1.8968796032479144e+01,     1.3242509495870079e+04,    -6.1967128210365415e-04,    -5.9747870827932304e-07,     2.7423437390592785e-09, } , 
      {    2.2224606519293367e+01,     3.0109441810853277e+04,    -7.4768361285662132e-04,    -6.0816914083599698e-07,     2.6834873581725810e-09, } , 
      {    1.8068540096051724e+01,     4.7042686055103268e+03,    -3.3936654358836070e-04,    -3.3606940156799437e-07,     1.5652161213253255e-09, } , 
      {    2.1324350582865947e+01,     2.1571200920493528e+04,    -4.6737887434132792e-04,    -3.4675983412466832e-07,     1.5063597404386279e-09, } , 
      {    2.4580161069680170e+01,     3.8438133235476722e+04,    -5.9539120509429519e-04,    -3.5745026668134226e-07,     1.4475033595519302e-09, } , 
      {    2.0424094646438526e+01,     1.3032960030133778e+04,    -1.8707413582603455e-04,    -8.5350527413339637e-08,     3.2923212270467463e-10, } , 
      {    2.3679905133252749e+01,     2.9899892345116976e+04,    -3.1508646657900174e-04,    -9.6040959970013584e-08,     2.7037574181797698e-10, } , 
      {    2.6935715620066972e+01,     4.6766824660100174e+04,    -4.4309879733196895e-04,    -1.0673139252668753e-07,     2.1151936093127930e-10, } , 
      {    2.2779649196825329e+01,     2.1361651454757226e+04,    -3.4781728063708359e-05,     1.6536834674131510e-07,    -9.0675187591597625e-10, } , 
      {    2.6035459683639552e+01,     3.8228583769740428e+04,    -1.6279405881667558e-04,     1.5467791418464115e-07,    -9.6560825680267380e-10, } , 
      {    2.5135203747212131e+01,     2.9690342879380674e+04,     1.1751067969861783e-04,     4.1608722089596979e-07,    -2.1427358745366274e-09, } , 
      {    2.8391014234026354e+01,     4.6557275194363880e+04,    -1.0501651054349395e-05,     4.0539678833929584e-07,    -2.2015922554233248e-09, } , 
      {    1.6570115794235811e+01,     5.5421200264606887e+03,    -7.7462757135888876e-04,    -8.4758119031981508e-07,     3.9238880094281401e-09, } , 
      {    1.8925670344622613e+01,     1.3870811451084141e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    2.2181480831436836e+01,     3.0737743766067339e+04,    -7.5034749434952979e-04,    -6.0755274872183430e-07,     2.6290476299207914e-09, } , 
      {    1.8025414408195193e+01,     5.3325705607243872e+03,    -3.4203042508126923e-04,    -3.3545300945383169e-07,     1.5107763930735356e-09, } , 
      {    2.1281224895009416e+01,     2.2199502875707589e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    2.4537035381823639e+01,     3.9066435190690783e+04,    -5.9805508658720367e-04,    -3.5683387456717958e-07,     1.3930636313001403e-09, } , 
      {    2.0380968958581995e+01,     1.3661261985347837e+04,    -1.8973801731894305e-04,    -8.4734135299176970e-08,     2.7479239445288481e-10, } , 
      {    2.3636779445396218e+01,     3.0528194300331037e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {    2.6892589932210441e+01,     4.7395126615314235e+04,    -4.4576267882487749e-04,    -1.0611500041252487e-07,     1.5707963267948951e-10, } , 
      {    1.9480713022154575e+01,     5.1230210949880857e+03,     9.0566721196350347e-05,     1.7667517141215167e-07,    -9.0233522328106827e-10, } , 
      {    2.2736523508968798e+01,     2.1989953409971287e+04,    -3.7445609556616870e-05,     1.6598473885547775e-07,    -9.6119160416776602e-10, } , 
      {    2.5992333995783021e+01,     3.8856885724954489e+04,    -1.6545794030958409e-04,     1.5529430629880380e-07,    -1.0200479850544637e-09, } , 
      {    2.9248144482597244e+01,     5.5723818039937687e+04,    -2.9347027106255130e-04,     1.4460387374212985e-07,    -1.0789043659411613e-09, } , 
      {    2.5092078059355600e+01,     3.0318644834594736e+04,     1.1484679820570931e-04,     4.1670361301013247e-07,    -2.1971756027884171e-09, } , 
      {    2.8347888546169823e+01,     4.7185577149577941e+04,    -1.3165532547257906e-05,     4.0601318045345852e-07,    -2.2560319836751145e-09, } , 
      {    3.1603699032984046e+01,     6.4052509464561139e+04,    -1.4117786330022512e-04,     3.9532274789678457e-07,    -2.3148883645618119e-09, } , 
      {    2.7447632609742403e+01,     3.8647336259218187e+04,     2.6713920596803549e-04,     6.6742248716478719e-07,    -3.4331596014090679e-09, } , 
      {    3.0703443096556626e+01,     5.5514268574201385e+04,     1.3912687521506827e-04,     6.5673205460811324e-07,    -3.4920159822957653e-09, } , 
      {    1.6526990106379280e+01,     6.1704219816747491e+03,    -7.7729145285179724e-04,    -8.4696479820565240e-07,     3.8694482811763500e-09, } , 
      {    1.8882544656766083e+01,     1.4499113406298202e+04,    -6.2499904508947111e-04,    -5.9624592405099768e-07,     2.6334642825556992e-09, } , 
      {    1.7982288720338662e+01,     5.9608725159384476e+03,    -3.4469430657417776e-04,    -3.3483661733966901e-07,     1.4563366648217458e-09, } , 
      {    2.1238099207152885e+01,     2.2827804830921650e+04,    -4.7270663732714498e-04,    -3.4552704989634296e-07,     1.3974802839350481e-09, } , 
      {    2.4493909693967108e+01,     3.9694737145904845e+04,    -6.0071896808011214e-04,    -3.5621748245301690e-07,     1.3386239030483505e-09, } , 
      {    2.0337843270725465e+01,     1.4289563940561897e+04,    -1.9240189881185156e-04,    -8.4117743185014303e-08,     2.2035266620109502e-10, } , 
      {    2.3593653757539688e+01,     3.1156496255545098e+04,    -3.2041422956481880e-04,    -9.4808175741688251e-08,     1.6149628531439737e-10, } , 
      {    2.6849464244353911e+01,     4.8023428570528296e+04,    -4.4842656031778602e-04,    -1.0549860829836220e-07,     1.0263990442769972e-10, } , 
      {    2.2693397821112267e+01,     2.2618255365185349e+04,    -4.0109491049525381e-05,     1.6660113096964040e-07,    -1.0156313324195559e-09, } , 
      {    2.5949208307926490e+01,     3.9485187680168550e+04,    -1.6812182180249259e-04,     1.5591069841296646e-07,    -1.0744877133062535e-09, } , 
      {    2.9205018794740713e+01,     5.6352119995151748e+04,    -2.9613415255545984e-04,     1.4522026585629251e-07,    -1.1333440941929512e-09, } , 
      {    2.8304762858313293e+01,     4.7813879104792002e+04,    -1.5829414040166417e-05,     4.0662957256762120e-07,    -2.3104717119269042e-09, } , 
      {    3.0660317408700095e+01,     5.6142570529415447e+04,     1.3646299372215977e-04,     6.5734844672227592e-07,    -3.5464557105475550e-09, } , 
      {    1.8839418968909552e+01,     1.5127415361512261e+04,    -6.2766292658237969e-04,    -5.9562953193683500e-07,     2.5790245543039091e-09, } , 
      {    2.1194973519296354e+01,     2.3456106786135708e+04,    -4.7537051882005346e-04,    -3.4491065778218033e-07,     1.3430405556832585e-09, } , 
      {    2.3550528069683157e+01,     3.1784798210759160e+04,    -3.2307811105772728e-04,    -9.4191783627525584e-08,     1.0705655706260757e-10, } , 
      {    2.5906082620069959e+01,     4.0113489635382612e+04,    -1.7078570329540112e-04,     1.5652709052712914e-07,    -1.1289274415580432e-09, } , 
      {    2.8261637170456762e+01,     4.8442181060006063e+04,    -1.8493295533074928e-05,     4.0724596468178383e-07,    -2.3649114401786942e-09, } , 
      {    1.7896037344625601e+01,     7.2174764263665684e+03,    -3.5002206955999477e-04,    -3.3360383311134370e-07,     1.3474572083181662e-09, } , 
      {    2.4296639895205338e+01,     1.9128980775218795e+04,    -6.4477122568684544e-04,    -5.6826902336205025e-07,     2.7016533268037553e-09, } , 
      {    2.6652194445592141e+01,     2.7457672199842244e+04,    -4.9247881792451921e-04,    -3.1755014920739559e-07,     1.4656693281831045e-09, } , 
      {    2.9007748995978943e+01,     3.5786363624465695e+04,    -3.4018641016219303e-04,    -6.6831275052740827e-08,     2.2968532956245358e-10, } , 
      {    3.1363303546365746e+01,     4.4115055049089147e+04,    -1.8789400239986685e-04,     1.8388759910191388e-07,    -1.0062986690581972e-09, } , 
      {    2.4253514207348807e+01,     1.9757282730432857e+04,    -6.4743510717975392e-04,    -5.6765263124788768e-07,     2.6472135985519657e-09, } , 
      {    2.6609068757735610e+01,     2.8085974155056305e+04,    -4.9514269941742768e-04,    -3.1693375709323291e-07,     1.4112295999313146e-09, } , 
      {    2.8964623308122412e+01,     3.6414665579679749e+04,    -3.4285029165510156e-04,    -6.6214882938578160e-08,     1.7524560131066379e-10, } , 
      {    3.1320177858509215e+01,     4.4743357004303201e+04,    -1.9055788389277535e-04,     1.8450399121607656e-07,    -1.0607383973099870e-09, } , 
      {    3.3675732408896017e+01,     5.3072048428926653e+04,    -3.8265476130449168e-05,     4.3522286537073125e-07,    -2.2967223959306377e-09, } , 
      {    2.1854833969105474e+01,     1.2056893261023464e+04,    -8.0239139643498852e-04,    -8.1775511328837972e-07,     3.8287578689208264e-09, } , 
      {    2.4210388519492277e+01,     2.0385584685646914e+04,    -6.5009898867266240e-04,    -5.6703623913372500e-07,     2.5927738703001760e-09, } , 
      {    2.7466199006306500e+01,     3.7252517000630112e+04,    -7.7811131942562967e-04,    -5.7772667169039884e-07,     2.5339174894134781e-09, } , 
      {    2.3310132583064856e+01,     1.1847343795287163e+04,    -3.6979425015736905e-04,    -3.0562693242239628e-07,     1.4156462525662224e-09, } , 
      {    2.6565943069879079e+01,     2.8714276110270366e+04,    -4.9780658091033616e-04,    -3.1631736497907023e-07,     1.3567898716795247e-09, } , 
      {    2.9821753556693302e+01,     4.5581208425253557e+04,    -6.2581891166330343e-04,    -3.2700779753574412e-07,     1.2979334907928271e-09, } , 
      {    2.5665687133451659e+01,     2.0176035219910613e+04,    -2.1750184239504282e-04,    -5.4908058267741579e-08,     1.7966225394557162e-10, } , 
      {    2.8921497620265882e+01,     3.7042967534893811e+04,    -3.4551417314801009e-04,    -6.5598490824415507e-08,     1.2080587305887397e-10, } , 
      {    3.2177308107080108e+01,     5.3909899849877009e+04,    -4.7352650390097731e-04,    -7.6288923381089428e-08,     6.1949492172176333e-11, } , 
      {    2.8021241683838461e+01,     2.8504726644534065e+04,    -6.5209434632716650e-05,     1.9581081588691313e-07,    -1.0563217446750793e-09, } , 
      {    3.1277052170652684e+01,     4.5371658959517263e+04,    -1.9322176538568385e-04,     1.8512038333023921e-07,    -1.1151781255617769e-09, } , 
      {    3.4532862657466907e+01,     6.2238591274500461e+04,    -3.2123409613865113e-04,     1.7442995077356529e-07,    -1.1740345064484746e-09, } , 
      {    3.0376796234225264e+01,     3.6833418069157509e+04,     8.7082973129609531e-05,     4.4652969004156788e-07,    -2.2923057432957299e-09, } , 
      {    3.3632606721039487e+01,     5.3700350384140715e+04,    -4.0929357623357686e-05,     4.3583925748489393e-07,    -2.3511621241824273e-09, } , 
      {    3.5988161271426293e+01,     6.2029041808764159e+04,     1.1136305013896850e-04,     6.8655813163954860e-07,    -3.5871461228030782e-09, } , 
      {    3.8343715821813092e+01,     7.0357733233387611e+04,     2.6365545790129464e-04,     9.3727700579420332e-07,    -4.8231301214237294e-09, } , 
      {    1.9456153730862141e+01,     4.3565037916140727e+03,    -9.5734768569022324e-04,    -1.0678575953288718e-06,     5.0103021392896880e-09, } , 
      {    2.1811708281248944e+01,     1.2685195216237524e+04,    -8.0505527792789711e-04,    -8.1713872117421704e-07,     3.7743181406690372e-09, } , 
      {    2.5067518768063167e+01,     2.9552127531220722e+04,    -9.3306760868086438e-04,    -8.2782915373089088e-07,     3.7154617597823389e-09, } , 
      {    2.0911452344821523e+01,     4.1469543258777749e+03,    -5.2475053941260371e-04,    -5.5572941446288837e-07,     2.5971905229350834e-09, } , 
      {    2.4167262831635746e+01,     2.1013886640860976e+04,    -6.5276287016557098e-04,    -5.6641984701956232e-07,     2.5383341420483859e-09, } , 
      {    2.7423073318449969e+01,     3.7880818955844174e+04,    -7.8077520091853825e-04,    -5.7711027957623616e-07,     2.4794777611616881e-09, } , 
      {    2.3267006895208326e+01,     1.2475645750501222e+04,    -3.7245813165027753e-04,    -3.0501054030823365e-07,     1.3612065243144327e-09, } , 
      {    2.6522817382022549e+01,     2.9342578065484424e+04,    -5.0047046240324475e-04,    -3.1570097286490760e-07,     1.3023501434277351e-09, } , 
      {    2.9778627868836772e+01,     4.6209510380467618e+04,    -6.2848279315621202e-04,    -3.2639140542158149e-07,     1.2434937625410374e-09, } , 
      {    2.5622561445595128e+01,     2.0804337175124674e+04,    -2.2016572388795135e-04,    -5.4291666153578919e-08,     1.2522252569378183e-10, } , 
      {    2.8878371932409351e+01,     3.7671269490107872e+04,    -3.4817805464091857e-04,    -6.4982098710252840e-08,     6.6366144807084178e-11, } , 
      {    3.2134182419223578e+01,     5.4538201805091070e+04,    -4.7619038539388578e-04,    -7.5672531266926761e-08,     7.5097639203865362e-12, } , 
      {    2.7978115995981931e+01,     2.9133028599748122e+04,    -6.7873316125625154e-05,     1.9642720800107581e-07,    -1.1107614729268689e-09, } , 
      {    3.1233926482796154e+01,     4.5999960914731324e+04,    -1.9588564687859238e-04,     1.8573677544440189e-07,    -1.1696178538135666e-09, } , 
      {    3.4489736969610377e+01,     6.2866893229714522e+04,    -3.2389797763155960e-04,     1.7504634288772797e-07,    -1.2284742347002642e-09, } , 
      {    3.0333670546368733e+01,     3.7461720024371571e+04,     8.4419091636701027e-05,     4.4714608215573051e-07,    -2.3467454715475200e-09, } , 
      {    3.3589481033182956e+01,     5.4328652339354776e+04,    -4.3593239116266197e-05,     4.3645564959905656e-07,    -2.4056018524342174e-09, } , 
      {    3.6845291519997176e+01,     7.1195584654337974e+04,    -1.7160556986923345e-04,     4.2576521704238266e-07,    -2.4644582333209153e-09, } , 
      {    3.2689225096755536e+01,     4.5790411448995023e+04,     2.3671149939902720e-04,     6.9786495631038523e-07,    -3.5827294701681708e-09, } , 
      {    3.5945035583569762e+01,     6.2657343763978221e+04,     1.0869916864605999e-04,     6.8717452375371128e-07,    -3.6415858510548682e-09, } , 
      {    3.9200846070383982e+01,     7.9524276078961426e+04,    -1.9313162106907260e-05,     6.7648409119703744e-07,    -3.7004422319415661e-09, } , 
      {    3.8300590133956561e+01,     7.0986035188601672e+04,     2.6099157640838616e-04,     9.3789339790836600e-07,    -4.8775698496755187e-09, } , 
      {    1.9413028043005610e+01,     4.9848057468281331e+03,    -9.6001156718313171e-04,    -1.0672412032147091e-06,     4.9558624110378979e-09, } , 
      {    2.1768582593392413e+01,     1.3313497171451585e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {    2.5024393080206636e+01,     3.0180429486434783e+04,    -9.3573149017377286e-04,    -8.2721276161672820e-07,     3.6610220315305492e-09, } , 
      {    2.0868326656964992e+01,     4.7752562810918353e+03,    -5.2741442090551219e-04,    -5.5511302234872569e-07,     2.5427507946832937e-09, } , 
      {    2.4124137143779215e+01,     2.1642188596075037e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    2.7379947630593438e+01,     3.8509120911058235e+04,    -7.8343908241144673e-04,    -5.7649388746207348e-07,     2.4250380329098984e-09, } , 
      {    3.0635758117407665e+01,     5.5376053226041440e+04,    -9.1145141316441389e-04,    -5.8718432001874753e-07,     2.3661816520232005e-09, } , 
      {    2.3223881207351795e+01,     1.3103947705715284e+04,    -3.7512201314318606e-04,    -3.0439414819407097e-07,     1.3067667960626429e-09, } , 
      {    2.6479691694166018e+01,     2.9970880020698485e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.9735502180980241e+01,     4.6837812335681680e+04,    -6.3114667464912050e-04,    -3.2577501330741881e-07,     1.1890540342892476e-09, } , 
      {    3.2991312667794467e+01,     6.3704744650664885e+04,    -7.5915900540208766e-04,    -3.3646544586409276e-07,     1.1301976534025497e-09, } , 
      {    2.5579435757738597e+01,     2.1432639130338735e+04,    -2.2282960538085985e-04,    -5.3675274039416252e-08,     7.0782797441992036e-11, } , 
      {    2.8835246244552820e+01,     3.8299571445321933e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.2091056731367047e+01,     5.5166503760305131e+04,    -4.7885426688679431e-04,    -7.5056139152764095e-08,    -4.6929964331403261e-11, } , 
      {    3.5346867218181274e+01,     7.2033436075288337e+04,    -6.0686659763976142e-04,    -8.5746571709438042e-08,    -1.0578634521810112e-10, } , 
      {    2.4679179821311177e+01,     1.2894398239978982e+04,     5.7475133134433546e-05,     2.0773403267191242e-07,    -1.1063448202919609e-09, } , 
      {    2.7934990308125400e+01,     2.9761330554962184e+04,    -7.0537197618533671e-05,     1.9704360011523847e-07,    -1.1652012011786588e-09, } , 
      {    3.1190800794939623e+01,     4.6628262869945385e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    3.4446611281753846e+01,     6.3495195184928583e+04,    -3.2656185912446813e-04,     1.7566273500189062e-07,    -1.2829139629520541e-09, } , 
      {    2.7034734371697979e+01,     2.1223089664602434e+04,     2.0976754089675973e-04,     4.5845290682656713e-07,    -2.3423288189126118e-09, } , 
      {    3.0290544858512202e+01,     3.8090021979585632e+04,     8.1755210143792510e-05,     4.4776247426989319e-07,    -2.4011851997993096e-09, } , 
      {    3.3546355345326425e+01,     5.4956954294568837e+04,    -4.6257120609174708e-05,     4.3707204171321924e-07,    -2.4600415806860071e-09, } , 
      {    3.6802165832140645e+01,     7.1823886609552035e+04,    -1.7426945136214195e-04,     4.2638160915654534e-07,    -2.5188979615727049e-09, } , 
      {    3.2646099408899005e+01,     4.6418713404209084e+04,     2.3404761790611869e-04,     6.9848134842454791e-07,    -3.6371691984199605e-09, } , 
      {    3.5901909895713231e+01,     6.3285645719192282e+04,     1.0603528715315147e-04,     6.8779091586787396e-07,    -3.6960255793066579e-09, } , 
      {    3.9157720382527451e+01,     8.0152578034175487e+04,    -2.1977043599815771e-05,     6.7710048331120012e-07,    -3.7548819601933558e-09, } , 
      {    3.5001653959285804e+01,     5.4747404828832528e+04,     3.8634002566844490e-04,     9.4920022257920263e-07,    -4.8731531970406117e-09, } , 
      {    3.8257464446100030e+01,     7.1614337143815734e+04,     2.5832769491547763e-04,     9.3850979002252868e-07,    -4.9320095779273087e-09, } , 
      {    4.0613018996486829e+01,     7.9943028568439186e+04,     4.1062010267780386e-04,     1.1892286641771834e-06,    -6.1679935765479596e-09, } , 
      {    1.9369902355149080e+01,     5.6131077020421935e+03,    -9.6267544867604019e-04,    -1.0666248111005464e-06,     4.9014226827861078e-09, } , 
      {    2.1725456905535882e+01,     1.3941799126665646e+04,    -8.1038304091371406e-04,    -8.1590593694589168e-07,     3.6654386841654574e-09, } , 
      {    2.4081011455922685e+01,     2.2270490551289098e+04,    -6.5809063315138794e-04,    -5.6518706279123696e-07,     2.4294546855448066e-09, } , 
      {    2.7336821942736908e+01,     3.9137422866272296e+04,    -7.8610296390435521e-04,    -5.7587749534791080e-07,     2.3705983046581087e-09, } , 
      {    2.3180755519495264e+01,     1.3732249660929345e+04,    -3.7778589463609459e-04,    -3.0377775607990829e-07,     1.2523270678108530e-09, } , 
      {    2.6436566006309487e+01,     3.0599181975912546e+04,    -5.0579822538906170e-04,    -3.1446818863658224e-07,     1.1934706869241553e-09, } , 
      {    2.9692376493123710e+01,     4.7466114290895741e+04,    -6.3381055614202897e-04,    -3.2515862119325613e-07,     1.1346143060374577e-09, } , 
      {    2.5536310069882067e+01,     2.2060941085552797e+04,    -2.2549348687376836e-04,    -5.3058881925253585e-08,     1.6343069190202239e-11, } , 
      {    2.8792120556696290e+01,     3.8927873400535995e+04,    -3.5350581762673563e-04,    -6.3749314481927506e-08,    -4.2513311696495410e-11, } , 
      {    3.2047931043510516e+01,     5.5794805715519193e+04,    -4.8151814837970285e-04,    -7.4439747038601428e-08,    -1.0136969258319305e-10, } , 
      {    2.7891864620268869e+01,     3.0389632510176245e+04,    -7.3201079111442189e-05,     1.9765999222940112e-07,    -1.2196409294304487e-09, } , 
      {    3.1147675107083092e+01,     4.7256564825159447e+04,    -2.0121340986440939e-04,     1.8696955967272720e-07,    -1.2784973103171463e-09, } , 
      {    3.4403485593897315e+01,     6.4123497140142645e+04,    -3.2922574061737666e-04,     1.7627912711605328e-07,    -1.3373536912038440e-09, } , 
      {    3.0247419170655672e+01,     3.8718323934799693e+04,     7.9091328650883992e-05,     4.4837886638405587e-07,    -2.4556249280510993e-09, } , 
      {    3.3503229657469895e+01,     5.5585256249782899e+04,    -4.8921002102083218e-05,     4.3768843382738192e-07,    -2.5144813089377967e-09, } , 
      {    3.6759040144284114e+01,     7.2452188564766097e+04,    -1.7693333285505046e-04,     4.2699800127070802e-07,    -2.5733376898244946e-09, } , 
      {    3.5858784207856701e+01,     6.3913947674406343e+04,     1.0337140566024296e-04,     6.8840730798203664e-07,    -3.7504653075584480e-09, } , 
      {    3.8214338758243500e+01,     7.2242639099029795e+04,     2.5566381342256910e-04,     9.3912618213669136e-07,    -4.9864493061790988e-09, } , 
      {    2.1682331217679351e+01,     1.4570101081879706e+04,    -8.1304692240662265e-04,    -8.1528954483172900e-07,     3.6109989559136673e-09, } , 
      {    2.4037885768066154e+01,     2.2898792506503160e+04,    -6.6075451464429652e-04,    -5.6457067067707428e-07,     2.3750149572930165e-09, } , 
      {    2.3137629831638733e+01,     1.4360551616143404e+04,    -3.8044977612900307e-04,    -3.0316136396574566e-07,     1.1978873395590633e-09, } , 
      {    2.6393440318452956e+01,     3.1227483931126604e+04,    -5.0846210688197029e-04,    -3.1385179652241961e-07,     1.1390309586723657e-09, } , 
      {    2.5493184382025536e+01,     2.2689243040766858e+04,    -2.2815736836667689e-04,    -5.2442489811090925e-08,    -3.8096659061587558e-11, } , 
      {    2.8748994868839759e+01,     3.9556175355750056e+04,    -3.5616969911964411e-04,    -6.3132922367764840e-08,    -9.6953039948285207e-11, } , 
      {    3.1104549419226561e+01,     4.7884866780373508e+04,    -2.0387729135731792e-04,     1.8758595178688988e-07,    -1.3329370385689360e-09, } , 
      {    2.6350314630596426e+01,     3.1855785886340665e+04,    -5.1112598837487876e-04,    -3.1323540440825693e-07,     1.0845912304205758e-09, } , 
      {    2.9408855318648879e+01,     2.8156961830637811e+04,    -6.8319057673457923e-04,    -5.3597737787396417e-07,     2.3887642732892830e-09, } , 
      {    3.1764409869035681e+01,     3.6485653255261263e+04,    -5.3089816897225299e-04,    -2.8525850371930946e-07,     1.1527802746686320e-09, } , 
      {    2.7010175080405546e+01,     2.0456572361228420e+04,    -8.3814686598981394e-04,    -7.8607985991445621e-07,     3.5703085436581438e-09, } , 
      {    2.9365729630792348e+01,     2.8785263785851872e+04,    -6.8585445822748781e-04,    -5.3536098575980149e-07,     2.3343245450374929e-09, } , 
      {    3.1721284181179151e+01,     3.7113955210475324e+04,    -5.3356205046516158e-04,    -2.8464211160514683e-07,     1.0983405464168423e-09, } , 
      {    3.6432393281952756e+01,     5.3771338059722220e+04,    -2.2897723494050919e-04,     2.1679563670416264e-07,    -1.3736274508244593e-09, } , 
      {    2.4611494842162212e+01,     1.2756182891819029e+04,    -9.9310315524504854e-04,    -1.0361823419549483e-06,     4.7518528140270049e-09, } , 
      {    2.6967049392549015e+01,     2.1084874316442481e+04,    -8.4081074748272241e-04,    -7.8546346780029353e-07,     3.5158688154063541e-09, } , 
      {    2.9322603942935817e+01,     2.9413565741065933e+04,    -6.8851833972039629e-04,    -5.3474459364563881e-07,     2.2798848167857033e-09, } , 
      {    3.2578414429750040e+01,     4.6280498056049131e+04,    -8.1653067047336345e-04,    -5.4543502620231287e-07,     2.2210284358990054e-09, } , 
      {    2.8422348006508393e+01,     2.0875324850706183e+04,    -4.0821360120510284e-04,    -2.7333528693431020e-07,     1.1027571990517501e-09, } , 
      {    3.1678158493322620e+01,     3.7742257165689385e+04,    -5.3622593195807005e-04,    -2.8402571949098415e-07,     1.0439008181650524e-09, } , 
      {    3.4933968980136846e+01,     5.4609189480672576e+04,    -6.6423826271103732e-04,    -2.9471615204765810e-07,     9.8504443727835479e-10, } , 
      {    3.0777902556895196e+01,     2.9204016275329632e+04,    -2.5592119344277665e-04,    -2.2616412779655481e-08,    -1.3322679956890074e-10, } , 
      {    3.4033713043709426e+01,     4.6070948590312830e+04,    -3.8393352419574387e-04,    -3.3306845336329429e-08,    -1.9208318045559839e-10, } , 
      {    3.3133457107281998e+01,     3.7532707699953084e+04,    -1.0362878568045047e-04,     2.2810246137499924e-07,    -1.3692107981895516e-09, } , 
      {    3.6389267594096225e+01,     5.4399640014936282e+04,    -2.3164111643341769e-04,     2.1741202881832529e-07,    -1.4280671790762492e-09, } , 
      {    2.9279478255079287e+01,     3.0041867696279995e+04,    -6.9118222121330476e-04,    -5.3412820153147613e-07,     2.2254450885339136e-09, } , 
      {    3.1635032805466089e+01,     3.8370559120903446e+04,    -5.3888981345097853e-04,    -2.8340932737682147e-07,     9.8946108991326257e-10, } , 
      {    3.3990587355852895e+01,     4.6699250545526891e+04,    -3.8659740568865240e-04,    -3.2690453222166762e-08,    -2.4652290870738820e-10, } , 
      {    3.6346141906239694e+01,     5.5027941970150343e+04,    -2.3430499792632619e-04,     2.1802842093248794e-07,    -1.4825069073280391e-09, } , 
      {    3.4650447805662012e+01,     3.5300037020414646e+04,    -7.1361828330358747e-04,    -5.0553490872836614e-07,     2.2391944045301797e-09, } , 
      {    3.7006002356048818e+01,     4.3628728445038098e+04,    -5.6132587554126134e-04,    -2.5481603457371136e-07,     1.0032104059095291e-09, } , 
      {    3.9361556906435624e+01,     5.1957419869661542e+04,    -4.0903346777893516e-04,    -4.0971604190566499e-09,    -2.3277359271112176e-10, } , 
      {    3.2251767567418682e+01,     2.7599647551005255e+04,    -8.6857457255882218e-04,    -7.5563739076885818e-07,     3.4207386748990409e-09, } , 
      {    3.4607322117805481e+01,     3.5928338975628707e+04,    -7.1628216479649595e-04,    -5.0491851661420346e-07,     2.1847546762783900e-09, } , 
      {    3.6962876668192287e+01,     4.4257030400252159e+04,    -5.6398975703416982e-04,    -2.5419964245954868e-07,     9.4877067765773920e-10, } , 
      {    3.9318431218579093e+01,     5.2585721824875603e+04,    -4.1169734927184369e-04,    -3.4807683048939860e-09,    -2.8721332096291158e-10, } , 
      {    4.1673985768965892e+01,     6.0914413249499055e+04,    -2.5940494150951751e-04,     2.4723810584976075e-07,    -1.5231973195835625e-09, } , 
      {    4.4029540319352691e+01,     6.9243104674122500e+04,    -1.0711253374719132e-04,     4.9795698000441542e-07,    -2.7591813182042133e-09, } , 
      {    2.9853087329175349e+01,     1.9899258081595864e+04,    -1.0235308618140567e-03,    -1.0057398728093501e-06,     4.6022829452679020e-09, } , 
      {    3.2208641879562151e+01,     2.8227949506219316e+04,    -8.7123845405173077e-04,    -7.5502099865469550e-07,     3.3662989466472508e-09, } , 
      {    3.4564196429948950e+01,     3.6556640930842768e+04,    -7.1894604628940453e-04,    -5.0430212450004078e-07,     2.1303149480266000e-09, } , 
      {    3.7820006916763177e+01,     5.3423573245825966e+04,    -8.4695837704237180e-04,    -5.1499255705671472e-07,     2.0714585671399025e-09, } , 
      {    3.6919750980335756e+01,     4.4885332355466220e+04,    -5.6665363852707840e-04,    -2.5358325034538606e-07,     8.9433094940594943e-10, } , 
      {    4.0175561467149983e+01,     6.1752264670449411e+04,    -6.9466596928004557e-04,    -2.6427368290206001e-07,     8.3547456851925178e-10, } , 
      {    3.9275305530722562e+01,     5.3214023780089665e+04,    -4.1436123076475217e-04,    -2.8643761907313221e-09,    -3.4165304921470135e-10, } , 
      {    4.2531116017536789e+01,     7.0080956095072863e+04,    -5.4237356151771944e-04,    -1.3554808747405269e-08,    -4.0050943010139899e-10, } , 
      {    4.1630860081109361e+01,     6.1542715204713117e+04,    -2.6206882300242599e-04,     2.4785449796392338e-07,    -1.5776370478353521e-09, } , 
      {    4.3986414631496160e+01,     6.9871406629336561e+04,    -1.0977641524009982e-04,     4.9857337211857810e-07,    -2.8136210464560034e-09, } , 
      {    4.6341969181882966e+01,     7.8200098053960013e+04,     4.2515992522226355e-05,     7.4929224627323282e-07,    -4.0496050450766538e-09, } , 
      {    2.9809961641318818e+01,     2.0527560036809926e+04,    -1.0261947433069653e-03,    -1.0051234806951874e-06,     4.5478432170161120e-09, } , 
      {    3.2165516191705620e+01,     2.8856251461433378e+04,    -8.7390233554463924e-04,    -7.5440460654053282e-07,     3.3118592183954611e-09, } , 
      {    3.5421326678519847e+01,     4.5723183776416576e+04,    -1.0019146662976065e-03,    -7.6509503909720676e-07,     3.2530028375087637e-09, } , 
      {    3.1265260255278196e+01,     2.0318010571073632e+04,    -5.9359759702934584e-04,    -4.9299529982920415e-07,     2.1347316006615081e-09, } , 
      {    3.4521070742092419e+01,     3.7184942886056830e+04,    -7.2160992778231301e-04,    -5.0368573238587810e-07,     2.0758752197748103e-09, } , 
      {    3.7776881228906646e+01,     5.4051875201040028e+04,    -8.4962225853528028e-04,    -5.1437616494255204e-07,     2.0170188388881128e-09, } , 
      {    3.3620814805664999e+01,     2.8646701995697080e+04,    -4.4130518926701966e-04,    -2.4227642567454943e-07,     8.9874760204085732e-10, } , 
      {    3.6876625292479225e+01,     4.5513634310680281e+04,    -5.6931752001998688e-04,    -2.5296685823122338e-07,     8.3989122115415967e-10, } , 
      {    4.0132435779293452e+01,     6.2380566625663472e+04,    -6.9732985077295404e-04,    -2.6365729078789733e-07,     7.8103484026746202e-10, } , 
      {    3.5976369356051805e+01,     3.6975393420320528e+04,    -2.8901278150469348e-04,     8.4424484801052899e-09,    -3.3723639657979351e-10, } , 
      {    3.9232179842866032e+01,     5.3842325735303726e+04,    -4.1702511225766070e-04,    -2.2479840765686579e-09,    -3.9609277746649116e-10, } , 
      {    4.2487990329680258e+01,     7.0709258050286924e+04,    -5.4503744301062792e-04,    -1.2938416633242606e-08,    -4.5494915835318881e-10, } , 
      {    3.8331923906438604e+01,     4.5304084844943980e+04,    -1.3672037374236730e-04,     2.5916132263476001e-07,    -1.5732203952004443e-09, } , 
      {    4.1587734393252831e+01,     6.2171017159927178e+04,    -2.6473270449533452e-04,     2.4847089007808606e-07,    -1.6320767760871420e-09, } , 
      {    4.4843544880067057e+01,     7.9037949474910376e+04,    -3.9274503524830174e-04,     2.3778045752141211e-07,    -1.6909331569738396e-09, } , 
      {    4.0687478456825403e+01,     5.3632776269567432e+04,     1.5572034019958880e-05,     5.0988019678941473e-07,    -2.8092043938210952e-09, } , 
      {    4.3943288943639629e+01,     7.0499708584550623e+04,    -1.1244029673300834e-04,     4.9918976423274078e-07,    -2.8680607747077930e-09, } , 
      {    4.6298843494026436e+01,     7.8828400009174075e+04,     3.9852111029317844e-05,     7.4990863838739550e-07,    -4.1040447733284439e-09, } , 
      {    3.2122390503849090e+01,     2.9484553416647439e+04,    -8.7656621703754772e-04,    -7.5378821442637014e-07,     3.2574194901436715e-09, } , 
      {    3.4477945054235889e+01,     3.7813244841270891e+04,    -7.2427380927522148e-04,    -5.0306934027171542e-07,     2.0214354915230206e-09, } , 
      {    3.6833499604622695e+01,     4.6141936265894343e+04,    -5.7198140151289536e-04,    -2.5235046611706070e-07,     7.8545149290236990e-10, } , 
      {    3.9189054155009501e+01,     5.4470627690517787e+04,    -4.1968899375056923e-04,    -1.6315919624059938e-09,    -4.5053250571828098e-10, } , 
      {    4.1544608705396300e+01,     6.2799319115141239e+04,    -2.6739658598824305e-04,     2.4908728219224874e-07,    -1.6865165043389319e-09, } , 
      {    4.3900163255783099e+01,     7.1128010539764684e+04,    -1.1510417822591685e-04,     4.9980615634690346e-07,    -2.9225005029595827e-09, } , 
      {    3.4434819366379358e+01,     3.8441546796484952e+04,    -7.2693769076813007e-04,    -5.0245294815755274e-07,     1.9669957632712306e-09, } , 
      {    3.7363982990862226e+01,     3.6627628606424281e+04,    -9.0699392360655607e-04,    -7.2334574528077210e-07,     3.1078496213845686e-09, } , 
      {    3.9719537541249025e+01,     4.4956320031047733e+04,    -7.5470151584422994e-04,    -4.7262687112611733e-07,     1.8718656227639177e-09, } , 
      {    4.2605575477875362e+01,     4.3770703796201116e+04,    -9.3742163017556442e-04,    -6.9290327613517395e-07,     2.9582797526254656e-09, } , 
      {    4.4961130028262161e+01,     5.2099395220824568e+04,    -7.8512922241323819e-04,    -4.4218440198051923e-07,     1.7222957540048150e-09, } , 
      {    4.7316684578648967e+01,     6.0428086645448013e+04,    -6.3283681465091206e-04,    -1.9146552782586449e-07,     4.8631175538416409e-10, } , 
      {    4.9672239129035773e+01,     6.8756778070071465e+04,    -4.8054440688858583e-04,     5.9253346328790217e-08,    -7.4967224323648674e-10, } , 
      {    4.0206895239632033e+01,     3.6070314326791726e+04,    -1.0923779194307991e-03,    -9.4300575817566599e-07,     4.1398240229943268e-09, } , 
      {    4.2562449790018832e+01,     4.4399005751415178e+04,    -9.4008551166847290e-04,    -6.9228688402101127e-07,     2.9038400243736760e-09, } , 
      {    4.4918004340405631e+01,     5.2727697176038630e+04,    -7.8779310390614666e-04,    -4.4156800986635655e-07,     1.6678560257530252e-09, } , 
      {    4.7273558890792437e+01,     6.1056388600662074e+04,    -6.3550069614382054e-04,    -1.9084913571170184e-07,     4.3187202713237433e-10, } , 
      {    4.9629113441179243e+01,     6.9385080025285526e+04,    -4.8320828838149436e-04,     5.9869738442952884e-08,    -8.0411197148827650e-10, } , 
      {    5.1984667991566042e+01,     7.7713771449908978e+04,    -3.3091588061916817e-04,     3.1058861259760760e-07,    -2.0400959701089273e-09, } , 
      {    1.6279052434071115e+00,     8.4334661574916008e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.8837157302213345e+00,     2.5300398472474801e+04,    -1.9201849612945083e-04,    -1.6035648835010902e-08,    -8.8284571330046551e-11, } , 
      {    8.1395262170355576e+00,     4.2167330787458006e+04,    -3.2003082688241802e-04,    -2.6726081391684836e-08,    -1.4714095221674425e-10, } , 
      {   -5.7839716666487551e+00,    -3.3838639362834554e+04,     4.7232323464474420e-04,     2.7744495554633954e-07,    -1.0888430464039066e-09, } , 
      {   -2.5281611798345320e+00,    -1.6971707047851349e+04,     3.4431090389177698e-04,     2.6675452298966564e-07,    -1.1476994272906043e-09, } , 
      {    7.2764930697969099e-01,    -1.0477473286815075e+02,     2.1629857313880979e-04,     2.5606409043299169e-07,    -1.2065558081773019e-09, } , 
      {    3.9834597937939140e+00,     1.6762157582115051e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.2392702806081370e+00,     3.3629089897098253e+04,    -3.9726088367124645e-05,     2.3468322531964382e-07,    -1.3242685699506974e-09, } , 
      {    1.0495080767422360e+01,     5.0496022212081458e+04,    -1.6773841912009184e-04,     2.2399279276296988e-07,    -1.3831249508373951e-09, } , 
      {   -3.4284171162619526e+00,    -2.5509947938211106e+04,     6.2461564240707038e-04,     5.2816382970099426e-07,    -2.3248270450245574e-09, } , 
      {   -1.7260662944772953e-01,    -8.6430156232279005e+03,     4.9660331165410322e-04,     5.1747339714432031e-07,    -2.3836834259112553e-09, } , 
      {    3.0832038573664935e+00,     8.2239166917552993e+03,     3.6859098090113594e-04,     5.0678296458764636e-07,    -2.4425398067979527e-09, } , 
      {    6.3390143441807165e+00,     2.5090849006738499e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    9.5948248309949395e+00,     4.1957781321721704e+04,     1.1256631939520154e-04,     4.8540209947429857e-07,    -2.5602525685713480e-09, } , 
      {    1.2850635317809163e+01,     5.8824713636704910e+04,    -1.5446011357765654e-05,     4.7471166691762462e-07,    -2.6191089494580459e-09, } , 
      {   -1.0728625658751501e+00,    -1.7181256513587658e+04,     7.7690805016939661e-04,     7.7888270385564897e-07,    -3.5608110436452083e-09, } , 
      {    2.1829479209390730e+00,    -3.1432419860445225e+02,     6.4889571941642934e-04,     7.6819227129897503e-07,    -3.6196674245319061e-09, } , 
      {    5.4387584077532960e+00,     1.6552608116378746e+04,     5.2088338866346218e-04,     7.5750183874230108e-07,    -3.6785238054186036e-09, } , 
      {    8.6945688945675190e+00,     3.3419540431361951e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.1950379381381742e+01,     5.0286472746345149e+04,     2.6485872715752774e-04,     7.3612097362895329e-07,    -3.7962365671919993e-09, } , 
      {    1.5206189868195965e+01,     6.7153405061328347e+04,     1.3684639640456053e-04,     7.2543054107227934e-07,    -3.8550929480786971e-09, } , 
      {    4.5385024713258755e+00,     8.0143672260189996e+03,     8.0118812717875558e-04,     1.0189111454536297e-06,    -4.8556514231525565e-09, } , 
      {    7.7943129581400985e+00,     2.4881299541002198e+04,     6.7317579642578831e-04,     1.0082207128969559e-06,    -4.9145078040392544e-09, } , 
      {    1.1050123444954322e+01,     4.1748231855985403e+04,     5.4516346567282114e-04,     9.9753028034028185e-07,    -4.9733641849259522e-09, } , 
      {    1.4305933931768545e+01,     5.8615164170968601e+04,     4.1715113491985387e-04,     9.8683984778360801e-07,    -5.0322205658126501e-09, } , 
      {    1.0149867508526901e+01,     3.3209990965625650e+04,     8.2546820418811443e-04,     1.2589395870516106e-06,    -6.1504918026599052e-09, } , 
      {    1.3405677995341124e+01,     5.0076923280608855e+04,     6.9745587343514727e-04,     1.2482491544949366e-06,    -6.2093481835466031e-09, } , 
      {    1.6661488482155349e+01,     6.6943855595592060e+04,     5.6944354268218000e-04,     1.2375587219382627e-06,    -6.2682045644333009e-09, } , 
      {    1.2505422058913704e+01,     4.1538682390249094e+04,     9.7776061195044067e-04,     1.5096584612062653e-06,    -7.3864758012805561e-09, } , 
      {    1.5761232545727927e+01,     5.8405614705232299e+04,     8.4974828119747351e-04,     1.4989680286495913e-06,    -7.4453321821672539e-09, } , 
      {    1.4860976609300506e+01,     4.9867373814872546e+04,     1.1300530197127670e-03,     1.7603773353609201e-06,    -8.6224597999012085e-09, } , 
      {    1.8116787096114727e+01,     6.6734306129855744e+04,     1.0020406889597996e-03,     1.7496869028042460e-06,    -8.6813161807879047e-09, } , 
      {   -1.5804358233584457e+01,    -5.7777312750018238e+04,    -8.5241216269038198e-04,    -1.4983516365354286e-06,     7.3908924539154638e-09, } , 
      {   -1.2548547746770234e+01,    -4.0910380435035033e+04,    -9.8042449344334925e-04,    -1.5090420690921027e-06,     7.3320360730287660e-09, } , 
      {   -1.3448803683197655e+01,    -4.9448621325394794e+04,    -7.0011975492805575e-04,    -1.2476327623807739e-06,     6.1549084552948130e-09, } , 
      {   -1.0192993196383432e+01,    -3.2581689010411588e+04,    -8.2813208568102291e-04,    -1.2583231949374479e-06,     6.0960520744081151e-09, } , 
      {   -1.4349059619625075e+01,    -5.7986862215754540e+04,    -4.1981501641276240e-04,    -9.8622345566944533e-07,     4.9777808375608600e-09, } , 
      {   -1.1093249132810852e+01,    -4.1119929900771342e+04,    -5.4782734716572962e-04,    -9.9691388822611917e-07,     4.9189244566741622e-09, } , 
      {   -7.8374386459966292e+00,    -2.4252997585788136e+04,    -6.7583967791869678e-04,    -1.0076043207827932e-06,     4.8600680757874643e-09, } , 
      {   -1.1993505069238273e+01,    -4.9658170791131088e+04,    -2.6752260865043628e-04,    -7.3550458151479061e-07,     3.7417968389402092e-09, } , 
      {   -8.7376945824240497e+00,    -3.2791238476147890e+04,    -3.9553493940340344e-04,    -7.4619501407146456e-07,     3.6829404580535118e-09, } , 
      {   -5.4818840956098267e+00,    -1.5924306161164684e+04,    -5.2354727015637066e-04,    -7.5688544662813840e-07,     3.6240840771668139e-09, } , 
      {   -2.2260736087956037e+00,     9.4262615381851265e+02,    -6.5155960090933782e-04,    -7.6757587918481235e-07,     3.5652276962801165e-09, } , 
      {   -9.6379505188514702e+00,    -4.1329479366507643e+04,    -1.1523020088811005e-04,    -4.8478570736013589e-07,     2.5058128403195584e-09, } , 
      {   -6.3821400320372472e+00,    -2.4462547051524438e+04,    -2.4324253164107726e-04,    -4.9547613991680984e-07,     2.4469564594328609e-09, } , 
      {   -3.1263295452230242e+00,    -7.5956147365412389e+03,    -3.7125486239404448e-04,    -5.0616657247348368e-07,     2.3881000785461631e-09, } , 
      {    1.2948094159119905e-01,     9.2713175784419618e+03,    -4.9926719314701169e-04,    -5.1685700503015763e-07,     2.3292436976594656e-09, } , 
      {   -1.0538206455278891e+01,    -4.9867720256867397e+04,     1.6507453762718333e-04,    -2.2337640064880722e-07,     1.3286852225856052e-09, } , 
      {   -7.2823959684646677e+00,    -3.3000787941884191e+04,     3.7062206874216134e-05,    -2.3406683320548117e-07,     1.2698288416989075e-09, } , 
      {   -4.0265854816504447e+00,    -1.6133855626900990e+04,    -9.0950123878751090e-05,    -2.4475726576215507e-07,     1.2109724608122099e-09, } , 
      {   -7.7077499483622147e-01,     7.3307668808221115e+02,    -2.1896245463171829e-04,    -2.5544769831882901e-07,     1.1521160799255120e-09, } , 
      {    2.4850354919780013e+00,     1.7600009003065410e+04,    -3.4697478538468551e-04,    -2.6613813087550296e-07,     1.0932596990388144e-09, } , 
      {   -4.9268414180778652e+00,    -2.4672096517260739e+04,     1.8935461463654232e-04,     1.6652040949173565e-08,     3.3844843078256753e-11, } , 
      {   -1.6710309312636420e+00,    -7.8051642022775404e+03,     6.1342283883575091e-05,     5.9616083924996316e-09,    -2.5011537808440947e-11, } , 
      {    1.5847795555505810e+00,     9.0617681127056603e+03,    -6.6670046869392126e-05,    -4.7288241641743030e-09,    -8.3867918695138641e-11, } , 
      {    4.8405900423648038e+00,     2.5928700427688862e+04,    -1.9468237762235933e-04,    -1.5419256720848238e-08,    -1.4272429958183634e-10, } , 
      {    8.0964005291790269e+00,     4.2795632742672067e+04,    -3.2269470837532655e-04,    -2.6109689277522173e-08,    -2.0158068046853404e-10, } , 
      {   -2.5712868676910627e+00,    -1.6343405092637287e+04,     3.4164702239886845e-04,     2.6737091510382832e-07,    -1.2021391555423941e-09, } , 
      {    6.8452361912316051e-01,     5.2352722234590965e+02,     2.1363469164590129e-04,     2.5668048254715437e-07,    -1.2609955364290918e-09, } , 
      {    3.9403341059373833e+00,     1.7390459537329112e+04,     8.5622360892934055e-05,     2.4599004999048043e-07,    -1.3198519173157896e-09, } , 
      {    7.1961445927516063e+00,     3.4257391852312314e+04,    -4.2389969860033156e-05,     2.3529961743380648e-07,    -1.3787082982024873e-09, } , 
      {    1.0451955079565829e+01,     5.1124324167295519e+04,    -1.7040230061300034e-04,     2.2460918487713253e-07,    -1.4375646790891849e-09, } , 
      {   -2.1573231730426001e-01,    -8.0147136680138401e+03,     4.9393943016119474e-04,     5.1808978925848299e-07,    -2.4381231541630450e-09, } , 
      {    3.0400781695099628e+00,     8.8522186469693588e+03,     3.6592709940822741e-04,     5.0739935670180904e-07,    -2.4969795350497424e-09, } , 
      {    6.2958886563241858e+00,     2.5719150961952560e+04,     2.3791476865526025e-04,     4.9670892414513520e-07,    -2.5558359159364402e-09, } , 
      {    9.5516991431384088e+00,     4.2586083276935766e+04,     1.0990243790229302e-04,     4.8601849158846125e-07,    -2.6146922968231377e-09, } , 
      {    2.1398222330825423e+00,     3.1397775660960815e+02,     6.4623183792352087e-04,     7.6880866341313771e-07,    -3.6741071527836958e-09, } , 
      {    5.3956327198967653e+00,     1.7180910071592807e+04,     5.1821950717055370e-04,     7.5811823085646376e-07,    -3.7329635336703936e-09, } , 
      {    8.6514432067109883e+00,     3.4047842386576012e+04,     3.9020717641758638e-04,     7.4742779829978992e-07,    -3.7918199145570915e-09, } , 
      {    1.1907253693525211e+01,     5.0914774701559210e+04,     2.6219484566461921e-04,     7.3673736574311597e-07,    -3.8506762954437893e-09, } , 
      {    7.7511872702835678e+00,     2.5509601496216259e+04,     6.7051191493287983e-04,     1.0088371050111186e-06,    -4.9689475322910445e-09, } , 
      {    1.1006997757097791e+01,     4.2376533811199464e+04,     5.4249958417991267e-04,     9.9814667245444453e-07,    -5.0278039131777423e-09, } , 
      {    1.4262808243912014e+01,     5.9243466126182662e+04,     4.1448725342694534e-04,     9.8745623989777069e-07,    -5.0866602940644402e-09, } , 
      {    1.0106741820670370e+01,     3.3838292920839711e+04,     8.2280432269520596e-04,     1.2595559791657733e-06,    -6.2049315309116953e-09, } , 
      {    1.3362552307484593e+01,     5.0705225235822916e+04,     6.9479199194223879e-04,     1.2488655466090993e-06,    -6.2637879117983932e-09, } , 
      {    1.2462296371057173e+01,     4.2166984345463155e+04,     9.7509673045753219e-04,     1.5102748533204280e-06,    -7.4409155295323461e-09, } , 
      {    1.5718106857871396e+01,     5.9033916660446361e+04,     8.4708439970456503e-04,     1.4995844207637540e-06,    -7.4997719104190440e-09, } , 
      {   -1.3491929371054185e+01,    -4.8820319370180732e+04,    -7.0278363642096433e-04,    -1.2470163702666112e-06,     6.1004687270430238e-09, } , 
      {   -1.1136374820667383e+01,    -4.0491627945557280e+04,    -5.5049122865863821e-04,    -9.9629749611195649e-07,     4.8644847284223729e-09, } , 
      {   -7.8805643338531599e+00,    -2.3624695630574075e+04,    -6.7850355941160537e-04,    -1.0069879286686305e-06,     4.8056283475356751e-09, } , 
      {   -8.7808202702805804e+00,    -3.2162936520933828e+04,    -3.9819882089631192e-04,    -7.4557862195730188e-07,     3.6285007298017217e-09, } , 
      {   -5.5250097834663574e+00,    -1.5296004205950625e+04,    -5.2621115164927924e-04,    -7.5626905451397572e-07,     3.5696443489150238e-09, } , 
      {   -9.6810762067080010e+00,    -4.0701177411293582e+04,    -1.1789408238101856e-04,    -4.8416931524597321e-07,     2.4513731120677683e-09, } , 
      {   -6.4252657198937779e+00,    -2.3834245096310377e+04,    -2.4590641313398579e-04,    -4.9485974780264716e-07,     2.3925167311810709e-09, } , 
      {   -3.1694552330795545e+00,    -6.9673127813271785e+03,    -3.7391874388695295e-04,    -5.0555018035932100e-07,     2.3336603502943730e-09, } , 
      {    8.6355253734668569e-02,     9.8996195336560213e+03,    -5.0193107463992028e-04,    -5.1624061291599495e-07,     2.2748039694076756e-09, } , 
      {   -7.3255216563211984e+00,    -3.2372485986670130e+04,     3.4398325381307623e-05,    -2.3345044109131849e-07,     1.2153891134471179e-09, } , 
      {   -4.0697111695069754e+00,    -1.5505553671686930e+04,    -9.3614005371659594e-05,    -2.4414087364799244e-07,     1.1565327325604202e-09, } , 
      {   -8.1390068269275195e-01,     1.3613786432962715e+03,    -2.2162633612462683e-04,    -2.5483130620466639e-07,     1.0976763516737224e-09, } , 
      {    2.4419098041214711e+00,     1.8228310958279471e+04,    -3.4963866687759399e-04,    -2.6552173876134033e-07,     1.0388199707870247e-09, } , 
      {   -4.9699671059343959e+00,    -2.4043794562046678e+04,     1.8669073314363379e-04,     1.7268433063336229e-08,    -2.0594885173533044e-11, } , 
      {   -1.7141566191201725e+00,    -7.1768622470634800e+03,     5.8678402390666587e-05,     6.5780005066622959e-09,    -7.9451266060230744e-11, } , 
      {    1.5416538676940506e+00,     9.6900700679197216e+03,    -6.9333928362300630e-05,    -4.1124320500116387e-09,    -1.3830764694692846e-10, } , 
      {    4.7974643545082731e+00,     2.6557002382902923e+04,    -1.9734625911526786e-04,    -1.4802864606685573e-08,    -1.9716402783362616e-10, } , 
      {   -2.6144125555475930e+00,    -1.5715103137423228e+04,     3.3898314090595997e-04,     2.6798730721799095e-07,    -1.2565788837941838e-09, } , 
      {    6.4139793126663003e-01,     1.1518291775599700e+03,     2.1097081015299275e-04,     2.5729687466131700e-07,    -1.3154352646808814e-09, } , 
      {    3.8972084180808531e+00,     1.8018761492543170e+04,     8.2958479400025551e-05,     2.4660644210464305e-07,    -1.3742916455675793e-09, } , 
      {    7.1530189048950756e+00,     3.4885693807526375e+04,    -4.5053851352941666e-05,     2.3591600954796916e-07,    -1.4331480264542769e-09, } , 
      {   -2.5885800516079049e-01,    -7.3864117127997797e+03,     4.9127554866828615e-04,     5.1870618137264567e-07,    -2.4925628824148350e-09, } , 
      {    2.9969524816534325e+00,     9.4805206021834201e+03,     3.6326321791531894e-04,     5.0801574881597172e-07,    -2.5514192633015325e-09, } , 
      {    6.2527629684676551e+00,     2.6347452917166622e+04,     2.3525088716235172e-04,     4.9732531625929788e-07,    -2.6102756441882303e-09, } , 
      {    9.5085734552818781e+00,     4.3214385232149827e+04,     1.0723855640938451e-04,     4.8663488370262393e-07,    -2.6691320250749278e-09, } , 
      {    5.3525070320402346e+00,     1.7809212026806868e+04,     5.1555562567764512e-04,     7.5873462297062644e-07,    -3.7874032619221829e-09, } , 
      {    8.6083175188544576e+00,     3.4676144341790074e+04,     3.8754329492467790e-04,     7.4804419041395260e-07,    -3.8462596428088807e-09, } , 
      {    1.0963872069241260e+01,     4.3004835766413526e+04,     5.3983570268700408e-04,     9.9876306456860721e-07,    -5.0822436414295316e-09, } , 
      {   -8.8239459581371111e+00,    -3.1534634565719771e+04,    -4.0086270238922045e-04,    -7.4496222984313920e-07,     3.5740610015499320e-09, } , 
      {   -5.5681354713228872e+00,    -1.4667702250736565e+04,    -5.2887503314218772e-04,    -7.5565266239981304e-07,     3.5152046206632342e-09, } , 
      {   -6.4683914077503077e+00,    -2.3205943141096319e+04,    -2.4857029462689427e-04,    -4.9424335568848448e-07,     2.3380770029292812e-09, } , 
      {   -3.2125809209360847e+00,    -6.3390108261131181e+03,    -3.7658262537986148e-04,    -5.0493378824515832e-07,     2.2792206220425833e-09, } , 
      {   -4.1128368573635052e+00,    -1.4877251716472871e+04,    -9.6277886864568112e-05,    -2.4352448153382976e-07,     1.1020930043086304e-09, } , 
      {   -8.5702637054928243e-01,     1.9896805985103319e+03,    -2.2429021761753533e-04,    -2.5421491409050371e-07,     1.0432366234219325e-09, } , 
      {   -1.7572823069767030e+00,    -6.5485602918494196e+03,     5.6014520897758069e-05,     7.1943926208249593e-09,    -1.3389099431202055e-10, } , 
      {    1.4985281798375201e+00,     1.0318372023133783e+04,    -7.1997809855209148e-05,    -3.4960399358489752e-09,    -1.9274737519871825e-10, } , 
      {    5.9827224341009955e-01,     1.7801311327740304e+03,     2.0830692866008425e-04,     2.5791326677547968e-07,    -1.3698749929326713e-09, } , 
      {    3.8540827302243228e+00,     1.8647063447757231e+04,     8.0294597907117033e-05,     2.4722283421880573e-07,    -1.4287313738193692e-09, } , 
      {    2.9538267937969023e+00,     1.0108822557397481e+04,     3.6059933642241040e-04,     5.0863214093013440e-07,    -2.6058589915533221e-09, } , 
      {    6.2096372806111253e+00,     2.6975754872380679e+04,     2.3258700566944321e-04,     4.9794170837346056e-07,    -2.6647153724400200e-09, } , 
      {   -4.1559625452200359e+00,    -1.4248949761258809e+04,    -9.8941768357476616e-05,    -2.4290808941966708e-07,     1.0476532760568405e-09, } , 
      {   -9.0015205840581292e-01,     2.6179825537243923e+03,    -2.2695409911044383e-04,    -2.5359852197634103e-07,     9.8879689517014264e-10, } , 
      {   -1.8004079948332334e+00,    -5.9202583366353592e+03,     5.3350639404849565e-05,     7.8107847349876244e-09,    -1.8833072256381034e-10, } , 
      {    1.4554024919809896e+00,     1.0946673978347842e+04,    -7.4661691348117652e-05,    -2.8796478216863110e-09,    -2.4718710345050804e-10, } , 
      {    5.5514655555356907e-01,     2.4084330879880908e+03,     2.0564304716717575e-04,     2.5852965888964236e-07,    -1.4243147211844612e-09, } , 
      {    3.8109570423677921e+00,     1.9275365402971292e+04,     7.7630716414208529e-05,     2.4783922633296841e-07,    -1.4831711020711590e-09, } , 
      {    1.3443840689322812e+00,    -1.0875686302766335e+04,    -1.1338618596903385e-04,    -2.1616397295904499e-07,     1.2247217768084764e-09, } , 
      {    3.6999386193190835e+00,    -2.5469948781428848e+03,     3.8906221793292333e-05,     3.4554901195609731e-08,    -1.1262221812174501e-11, } , 
      {    6.9557491061333065e+00,     1.4319937436840319e+04,    -8.9106108959674884e-05,     2.3864468638935797e-08,    -7.0118602698872201e-11, } , 
      {    6.0554931697058860e+00,     5.7816965464805653e+03,     1.9119862955561853e-04,     2.8527377535026448e-07,    -1.2472462204328253e-09, } , 
      {   -1.0542961693110517e+00,    -1.8576075772175725e+04,    -2.6834247522426855e-04,    -4.6626645499953700e-07,     2.4062660471773373e-09, } , 
      {    2.2015143175031713e+00,    -1.7091434571925220e+03,    -3.9635480597723576e-04,    -4.7695688755621100e-07,     2.3474096662906399e-09, } , 
      {    1.3012583810757508e+00,    -1.0247384347552275e+04,    -1.1605006746194237e-04,    -2.1554758084488233e-07,     1.1702820485566865e-09, } , 
      {    4.5570688678899742e+00,     6.6195479674309281e+03,    -2.4406239821490956e-04,    -2.2623801340155626e-07,     1.1114256676699889e-09, } , 
      {    3.6568129314625533e+00,    -1.9186929229288244e+03,     3.6242340300383816e-05,     3.5171293309772398e-08,    -6.5701950063964304e-11, } , 
      {    6.9126234182767767e+00,     1.4948239392054378e+04,    -9.1769990452583402e-05,     2.4480860753098463e-08,    -1.2455833095066202e-10, } , 
      {    6.0123674818493562e+00,     6.4099985016946257e+03,     1.8853474806271002e-04,     2.8589016746442716e-07,    -1.3016859486846151e-09, } , 
      {    9.2681779686635792e+00,     2.3276930816677828e+04,     6.0522417309742779e-05,     2.7519973490775321e-07,    -1.3605423295713128e-09, } , 
      {    8.3679220322361587e+00,     1.4738689926318077e+04,     3.4082715582503618e-04,     5.3660904161908182e-07,    -2.5376699473052660e-09, } , 
      {    1.1623732519050382e+01,     3.1605622241301273e+04,     2.1281482507206899e-04,     5.2591860906240788e-07,    -2.5965263281919634e-09, } , 
      {   -5.8085309579411870e+00,    -3.4605156666208568e+04,    -5.7559117224182949e-04,    -9.6708781119468392e-07,     4.8237943161668493e-09, } , 
      {   -3.4529764075543845e+00,    -2.6276465241585112e+04,    -4.2329876447950321e-04,    -7.1636893704002909e-07,     3.5878103175461981e-09, } , 
      {   -1.9716592074016170e-01,    -9.4095329266019107e+03,    -5.5131109523247053e-04,    -7.2705936959670304e-07,     3.5289539366595007e-09, } , 
      {   -1.0974218571675822e+00,    -1.7947773816961664e+04,    -2.7100635671717702e-04,    -4.6565006288537437e-07,     2.3518263189255473e-09, } , 
      {    2.1583886296466410e+00,    -1.0808415019784616e+03,    -3.9901868747014424e-04,    -4.7634049544204832e-07,     2.2929699380388498e-09, } , 
      {   -1.9976777935950027e+00,    -2.6486014707321418e+04,     9.2983817981163544e-06,    -2.0424075617404571e-07,     1.1746987011915945e-09, } , 
      {    1.2581326932192203e+00,    -9.6190823923382159e+03,    -1.1871394895485087e-04,    -2.1493118873071965e-07,     1.1158423203048969e-09, } , 
      {    4.5139431800334435e+00,     7.2478499226449885e+03,    -2.4672627970781806e-04,    -2.2562162128739358e-07,     1.0569859394181992e-09, } , 
      {    3.5787675679179975e-01,    -1.8157323282697966e+04,     1.6159078956044253e-04,     4.6478117980608993e-08,    -6.1285297429056408e-11, } , 
      {    3.6136872436060230e+00,    -1.2903909677147640e+03,     3.3578458807475305e-05,     3.5787685423935058e-08,    -1.2014167831575411e-10, } , 
      {    6.8694977304202460e+00,     1.5576541347268438e+04,    -9.4433871945491906e-05,     2.5097252867261127e-08,    -1.7899805920245181e-10, } , 
      {    1.0125308217234469e+01,     3.2443473662251636e+04,    -2.2244620269845912e-04,     1.4406820310587192e-08,    -2.3785444008914948e-10, } , 
      {    2.7134313071786025e+00,    -9.8286318580745137e+03,     3.1388319732276874e-04,     2.9719699213526368e-07,    -1.2972692960497072e-09, } , 
      {    5.9692417939928255e+00,     7.0383004569086861e+03,     1.8587086656980149e-04,     2.8650655957858978e-07,    -1.3561256769364048e-09, } , 
      {    9.2250522808070485e+00,     2.3905232771891890e+04,     5.7858535816834269e-05,     2.7581612702191584e-07,    -1.4149820578231025e-09, } , 
      {    1.2480862767621272e+01,     4.0772165086875088e+04,    -7.0153794936132928e-05,     2.6512569446524189e-07,    -1.4738384387098003e-09, } , 
      {    8.3247963443796280e+00,     1.5366991881532136e+04,     3.3816327433212770e-04,     5.3722543373324450e-07,    -2.5921096755570561e-09, } , 
      {    1.1580606831193851e+01,     3.2233924196515334e+04,     2.1015094357916046e-04,     5.2653500117657056e-07,    -2.6509660564437535e-09, } , 
      {    1.0680350894766431e+01,     2.3695683306155584e+04,     4.9045568209445383e-04,     7.8794430788789922e-07,    -3.8280936741777065e-09, } , 
      {    1.3936161381580654e+01,     4.0562615621138786e+04,     3.6244335134148666e-04,     7.7725387533122528e-07,    -3.8869500550644043e-09, } , 
      {    1.3035905445153233e+01,     3.2024374730779036e+04,     6.4274808985677996e-04,     1.0386631820425538e-06,    -5.0640776727983573e-09, } , 
      {    1.6291715931967456e+01,     4.8891307045762238e+04,     5.1473575910381279e-04,     1.0279727494858800e-06,    -5.1229340536850552e-09, } , 
      {   -8.2072111961845202e+00,    -4.2305546135617958e+04,    -7.3054746149706410e-04,    -1.2171902932351760e-06,     6.0053385865357101e-09, } , 
      {   -4.9514007093702972e+00,    -2.5438613820634753e+04,    -8.5855979225003126e-04,    -1.2278807257918498e-06,     5.9464822056490122e-09, } , 
      {   -5.8516566457977177e+00,    -3.3976854710994507e+04,    -5.7825505373473797e-04,    -9.6647141908052124e-07,     4.7693545879150593e-09, } , 
      {   -2.5958461589834947e+00,    -1.7109922396011301e+04,    -7.0626738448770513e-04,    -9.7716185163719508e-07,     4.7104982070283614e-09, } , 
      {   -3.4961020954109152e+00,    -2.5648163286371051e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {   -2.4029160859669219e-01,    -8.7812309713878494e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    3.0155188782175308e+00,     8.0857013435953486e+03,    -6.8198730747834617e-04,    -7.3713341003921431e-07,     3.4156578275210131e-09, } , 
      {   -4.3963580318383357e+00,    -3.4186404176730808e+04,    -1.4565790745711834e-04,    -4.5434323821453780e-07,     2.3562429715604555e-09, } , 
      {   -1.1405475450241127e+00,    -1.7319471861747603e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.1152629417901103e+00,    -4.5253954676440117e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    5.3710734286043333e+00,     1.6414392768218797e+04,    -5.2969489971602004e-04,    -4.8641453588455959e-07,     2.1796738289003623e-09, } , 
      {   -2.0408034814515332e+00,    -2.5857712752107356e+04,     6.6345003052078436e-06,    -2.0362436405988305e-07,     1.1202589729398046e-09, } , 
      {    1.2150070053626898e+00,    -8.9907804371241546e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.4708174921769128e+00,     7.8761518778590489e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.7266279789911358e+00,     2.4743084192842245e+04,    -3.7740249195369381e-04,    -2.3569566172990487e-07,     9.4368983027971148e-10, } , 
      {   -2.9410594178789538e+00,    -3.4395953642467110e+04,     2.8693923882050119e-04,     5.7784942651445594e-08,    -5.6868644794148498e-11, } , 
      {    3.1475106893526927e-01,    -1.7529021327483904e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    3.5705615557494923e+00,    -6.6208901250070357e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.8263720425637153e+00,     1.6204843302482499e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0082182529377938e+01,     3.3071775617465697e+04,    -2.2511008419136763e-04,     1.5023212424749856e-08,    -2.9229416834093930e-10, } , 
      {    1.3337993016192161e+01,     4.9938707932448902e+04,    -3.5312241494433484e-04,     4.3327798680759213e-09,    -3.5115054922763700e-10, } , 
      {   -5.8550486749215125e-01,    -2.6067262217843658e+04,     4.3923164658282737e-04,     3.0850381680610031e-07,    -1.2928526434147994e-09, } , 
      {    2.6703056193220718e+00,    -9.2003299028604542e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    5.9261161061362948e+00,     7.6666024121227465e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.1819265929505178e+00,     2.4533534727105951e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.2437737079764741e+01,     4.1400467042089149e+04,    -7.2817676429041446e-05,     2.6574208657940457e-07,    -1.5282781669615902e-09, } , 
      {    8.2816706565230973e+00,     1.5995293836746197e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.1537481143337320e+01,     3.2862226151729395e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.4793291630151543e+01,     4.9729158466712601e+04,     7.9474731333284735e-05,     5.1646096073405929e-07,    -2.7642621655822410e-09, } , 
      {    1.0637225206909900e+01,     2.4323985261369646e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    1.3893035693724123e+01,     4.1190917576352847e+04,     3.5977946984857813e-04,     7.7787026744538796e-07,    -3.9413897833161944e-09, } , 
      {    1.7148846180538346e+01,     5.8057849891336045e+04,     2.3176713909561092e-04,     7.6717983488871401e-07,    -4.0002461642028914e-09, } , 
      {    1.2992779757296702e+01,     3.2652676685993098e+04,     6.4008420836387148e-04,     1.0392795741567165e-06,    -5.1185174010501474e-09, } , 
      {    1.6248590244110925e+01,     4.9519609000976299e+04,     5.1207187761090431e-04,     1.0285891416000427e-06,    -5.1773737819368452e-09, } , 
      {    1.5348334307683505e+01,     4.0981368110616546e+04,     7.9237661612619761e-04,     1.2899984483113712e-06,    -6.3545013996707982e-09, } , 
      {    1.8604144794497728e+01,     5.7848300425599751e+04,     6.6436428537323044e-04,     1.2793080157546974e-06,    -6.4133577805574961e-09, } , 
      {   -8.2503368840410509e+00,    -4.1677244180403897e+04,    -7.3321134298997258e-04,    -1.2165739011210133e-06,     5.9508988582839200e-09, } , 
      {   -5.8947823336542484e+00,    -3.3348552755780445e+04,    -5.8091893522764645e-04,    -9.6585502696635856e-07,     4.7149148596632692e-09, } , 
      {   -2.6389718468400254e+00,    -1.6481620440797240e+04,    -7.0893126598061361e-04,    -9.7654545952303240e-07,     4.6560584787765713e-09, } , 
      {   -3.5392277832674459e+00,    -2.5019861331156990e+04,    -4.2862652746532027e-04,    -7.1513615281170373e-07,     3.4789308610426188e-09, } , 
      {   -2.8341729645322267e-01,    -8.1529290161737890e+03,    -5.5663885821828748e-04,    -7.2582658536837768e-07,     3.4200744801559213e-09, } , 
      {   -4.4394837196948664e+00,    -3.3558102221516747e+04,    -1.4832178895002684e-04,    -4.5372684610037512e-07,     2.3018032433086658e-09, } , 
      {   -1.1836732328806432e+00,    -1.6691169906533542e+04,    -2.7633411970299409e-04,    -4.6441727865704901e-07,     2.2429468624219679e-09, } , 
      {    2.0721372539335796e+00,     1.7576240844965923e+02,    -4.0434645045596130e-04,    -4.7510771121372296e-07,     2.1840904815352705e-09, } , 
      {    5.3279477407478026e+00,     1.7042694723432858e+04,    -5.3235878120892852e-04,    -4.8579814377039691e-07,     2.1252341006485726e-09, } , 
      {   -2.0839291693080639e+00,    -2.5229410796893295e+04,     3.9706188122993319e-06,    -2.0300797194572040e-07,     1.0658192446880148e-09, } , 
      {    1.1718813175061593e+00,    -8.3624784819100933e+03,    -1.2404171194066790e-04,    -2.1369840450239435e-07,     1.0069628638013171e-09, } , 
      {    4.4276918043203821e+00,     8.5044538330731084e+03,    -2.5205404269363512e-04,    -2.2438883705906827e-07,     9.4810648291461947e-10, } , 
      {    7.6835022911346051e+00,     2.5371386148056306e+04,    -3.8006637344660234e-04,    -2.3507926961574222e-07,     8.8925010202792171e-10, } , 
      {    2.7162538107873879e-01,    -1.6900719372269843e+04,     1.5626302657462552e-04,     4.7710902208934326e-08,    -1.7016475393263599e-10, } , 
      {    3.5274358678929616e+00,    -3.3787057286643176e+01,     2.8250695821658283e-05,     3.7020469652260392e-08,    -2.2902113481933369e-10, } , 
      {    6.7832463547071846e+00,     1.6833145257696560e+04,    -9.9761634931308941e-05,     2.6330037095586454e-08,    -2.8787751570603142e-10, } , 
      {    1.0039056841521408e+01,     3.3700077572679758e+04,    -2.2777396568427613e-04,     1.5639604538912519e-08,    -3.4673389659272912e-10, } , 
      {    2.6271799314655411e+00,    -8.5720279476463948e+03,     3.0855543433695167e-04,     2.9842977636358904e-07,    -1.4061487525532869e-09, } , 
      {    5.8829904182797641e+00,     8.2949043673368069e+03,     1.8054310358398448e-04,     2.8773934380691514e-07,    -1.4650051334399845e-09, } , 
      {    9.1388009050939871e+00,     2.5161836682320012e+04,     5.2530772831017247e-05,     2.7704891125024120e-07,    -1.5238615143266822e-09, } , 
      {    1.2394611391908210e+01,     4.2028768997303210e+04,    -7.5481557921949964e-05,     2.6635847869356725e-07,    -1.5827178952133800e-09, } , 
      {    4.9827344818523436e+00,    -2.4333652302294377e+02,     4.6084784209927786e-04,     5.4914865051824381e-07,    -2.6421327511739375e-09, } , 
      {    8.2385449686665666e+00,     1.6623595791960259e+04,     3.3283551134631064e-04,     5.3845821796156986e-07,    -2.7009891320606354e-09, } , 
      {    1.1494355455480790e+01,     3.3490528106943457e+04,     2.0482318059334345e-04,     5.2776778540489592e-07,    -2.7598455129473328e-09, } , 
      {    1.4750165942295013e+01,     5.0357460421926662e+04,     7.6810849840376218e-05,     5.1707735284822197e-07,    -2.8187018938340307e-09, } , 
      {    1.0594099519053369e+01,     2.4952287216583707e+04,     4.8512791910863682e-04,     7.8917709211622458e-07,    -3.9369731306812866e-09, } , 
      {    1.3849910005867592e+01,     4.1819219531566909e+04,     3.5711558835566960e-04,     7.7848665955955064e-07,    -3.9958295115679845e-09, } , 
      {    1.2949654069440172e+01,     3.3280978641207155e+04,     6.3742032687096300e-04,     1.0398959662708792e-06,    -5.1729571293019374e-09, } , 
      {    1.6205464556254395e+01,     5.0147910956190361e+04,     5.0940799611799584e-04,     1.0292055337142054e-06,    -5.2318135101886353e-09, } , 
      {   -3.5823534711239762e+00,    -2.4391559375942932e+04,    -4.3129040895822874e-04,    -7.1451976069754105e-07,     3.4244911327908287e-09, } , 
      {   -3.2654298430975315e-01,    -7.5246270609597286e+03,    -5.5930273971119607e-04,    -7.2521019325421500e-07,     3.3656347519041313e-09, } , 
      {   -1.2267989207371737e+00,    -1.6062867951319482e+04,    -2.7899800119590256e-04,    -4.6380088654288639e-07,     2.1885071341701779e-09, } , 
      {    2.0290115660770494e+00,     8.0406436366371963e+02,    -4.0701033194886978e-04,    -4.7449131909956033e-07,     2.1296507532834804e-09, } , 
      {    1.1287556296496288e+00,    -7.7341765266960338e+03,    -1.2670559343357641e-04,    -2.1308201238823167e-07,     9.5252313554952745e-10, } , 
      {    4.3845661164638514e+00,     9.1327557882871697e+03,    -2.5471792418654360e-04,    -2.2377244494490559e-07,     8.9366675466282970e-10, } , 
      {    3.4843101800364313e+00,     5.9451489792741722e+02,     2.5586814328749772e-05,     3.7636861766423052e-08,    -2.8346086307112348e-10, } , 
      {    6.7401206668506539e+00,     1.7461447212910622e+04,    -1.0242551642421745e-04,     2.6946429209749117e-08,    -3.4231724395782118e-10, } , 
      {    5.8398647304232334e+00,     8.9232063225508682e+03,     1.7787922209107595e-04,     2.8835573592107777e-07,    -1.5194448616917742e-09, } , 
      {    9.0956752172374564e+00,     2.5790138637534073e+04,     4.9866891338108736e-05,     2.7766530336440382e-07,    -1.5783012425784719e-09, } , 
      {    8.1954192808100359e+00,     1.7251897747174320e+04,     3.3017162985340216e-04,     5.3907461007573254e-07,    -2.7554288603124255e-09, } , 
      {    1.1451229767624259e+01,     3.4118830062157518e+04,     2.0215929910043492e-04,     5.2838417751905860e-07,    -2.8142852411991229e-09, } , 
      {    1.3806784318011061e+01,     4.2447521486780970e+04,     3.5445170686276113e-04,     7.7910305167371332e-07,    -4.0502692398197737e-09, } , 
      {    3.4411844921799011e+00,     1.2228168531414776e+03,     2.2922932835841258e-05,     3.8253253880585719e-08,    -3.3790059132291330e-10, } , 
      {    6.6969949789941241e+00,     1.8089749168124679e+04,    -1.0508939791712596e-04,     2.7562821323911784e-08,    -3.9675697220961100e-10, } , 
      {    6.6291022438019462e+00,    -4.3609130682035584e+03,    -1.4115001104513365e-04,    -1.8633789592760955e-07,     1.1295916363011631e-09, } , 
      {    9.8849127306161684e+00,     1.2506019246779644e+04,    -2.6916234179810087e-04,    -1.9702832848428350e-07,     1.0707352554144655e-09, } , 
      {    8.9846567941887479e+00,     3.9677783564198917e+03,     1.1142396717192537e-05,     6.4380978227045169e-08,    -1.0639236231948764e-10, } , 
      {    1.2240467281002971e+01,     2.0834710671403092e+04,    -1.1686993403577468e-04,     5.3690545670371227e-08,    -1.6524874320618539e-10, } , 
      {    4.2304220055586130e+00,    -1.2061302537612948e+04,    -2.9610630030036831e-04,    -4.3644037796810164e-07,     2.3111359066700241e-09, } , 
      {    6.5859765559454155e+00,    -3.7326111129894980e+03,    -1.4381389253804216e-04,    -1.8572150381344690e-07,     1.0751519080493735e-09, } , 
      {    9.8417870427596377e+00,     1.3134321201993704e+04,    -2.7182622329100935e-04,    -1.9641193637012084e-07,     1.0162955271626756e-09, } , 
      {    8.9415311063322171e+00,     4.5960803116339521e+03,     8.4785152242840258e-06,     6.4997370341207835e-08,    -1.6083209057127743e-10, } , 
      {    1.2197341593146440e+01,     2.1463012626617154e+04,    -1.1953381552868319e-04,     5.4306937784533894e-08,    -2.1968847145797519e-10, } , 
      {    1.1297085656719020e+01,     1.2924771736257402e+04,     1.6077092298661020e-04,     3.1571624449586252e-07,    -1.3968160891919282e-09, } , 
      {    1.4552896143533243e+01,     2.9791704051240606e+04,     3.2758592233642986e-05,     3.0502581193918862e-07,    -1.4556724700786260e-09, } , 
      {    1.6908450693920045e+01,     3.8120395475864047e+04,     1.8505099999596917e-04,     5.5574468609384334e-07,    -2.6916564686992771e-09, } , 
      {    1.8317417673152800e+00,    -1.9761692007022335e+04,    -4.5106258955560303e-04,    -6.8654286000859374e-07,     3.4926801770388853e-09, } , 
      {    4.1872963177020823e+00,    -1.1433000582398887e+04,    -2.9877018179327684e-04,    -4.3582398585393896e-07,     2.2566961784182344e-09, } , 
      {    7.4431068045163054e+00,     5.4339317325843140e+03,    -4.2678251254624406e-04,    -4.4651441841061286e-07,     2.1978397975315366e-09, } , 
      {    6.5428508680888848e+00,    -3.1043091577754376e+03,    -1.4647777403095066e-04,    -1.8510511169928422e-07,     1.0207121797975836e-09, } , 
      {    9.7986613549031087e+00,     1.3762623157207763e+04,    -2.7449010478391788e-04,    -1.9579554425595816e-07,     9.6185579891088575e-10, } , 
      {    1.3054471841717332e+01,     3.0629555472190961e+04,    -4.0250243553688510e-04,    -2.0648597681263209e-07,     9.0299941802418821e-10, } , 
      {    5.6425949316614643e+00,    -1.1642550048135188e+04,     1.3382696448434274e-04,     7.6304195012044424e-08,    -1.5641543793636958e-10, } , 
      {    8.8984054184756882e+00,     5.2243822668480125e+03,     5.8146337313755150e-06,     6.5613762455370502e-08,    -2.1527181882306722e-10, } , 
      {    1.2154215905289911e+01,     2.2091314581831215e+04,    -1.2219769702159170e-04,     5.4923329898696554e-08,    -2.7412819970976498e-10, } , 
      {    1.5410026392104134e+01,     3.8958246896814409e+04,    -2.5021002777455892e-04,     4.4232897342022620e-08,    -3.3298458059646262e-10, } , 
      {    7.9981494820482668e+00,    -3.3138586235117373e+03,     2.8611937224666892e-04,     3.2702306916669914e-07,    -1.3923994365570204e-09, } , 
      {    1.1253959968862491e+01,     1.3553073691471462e+04,     1.5810704149370170e-04,     3.1633263661002520e-07,    -1.4512558174437181e-09, } , 
      {    1.4509770455676714e+01,     3.0420006006454667e+04,     3.0094710740734479e-05,     3.0564220405335130e-07,    -1.5101121983304159e-09, } , 
      {    1.3609514519249293e+01,     2.1881765116094914e+04,     3.1039944925602788e-04,     5.6705151076467986e-07,    -2.6872398160643689e-09, } , 
      {    1.6865325006063514e+01,     3.8748697431078108e+04,     1.8238711850306066e-04,     5.5636107820800591e-07,    -2.7460961969510667e-09, } , 
      {    1.5965069069636096e+01,     3.0210456540718362e+04,     4.6269185701835406e-04,     8.1777038491933458e-07,    -3.9232238146850201e-09, } , 
      {    1.9220879556450321e+01,     4.7077388855701560e+04,     3.3467952626538684e-04,     8.0707995236266063e-07,    -3.9820801955717172e-09, } , 
      {   -5.6693847092805294e-01,    -2.7462081476431726e+04,    -6.0601887881083763e-04,    -9.3664534204908577e-07,     4.6742244474077464e-09, } , 
      {    2.6888720158861701e+00,    -1.0595149161448526e+04,    -7.3403120956380490e-04,    -9.4733577460575972e-07,     4.6153680665210486e-09, } , 
      {    1.7886160794587496e+00,    -1.9133390051808274e+04,    -4.5372647104851156e-04,    -6.8592646789443106e-07,     3.4382404487870956e-09, } , 
      {    5.0444265662729730e+00,    -2.2664577368250739e+03,    -5.8173880180147867e-04,    -6.9661690045110500e-07,     3.3793840679003977e-09, } , 
      {    4.1441706298455525e+00,    -1.0804698627184827e+04,    -3.0143406328618538e-04,    -4.3520759373977628e-07,     2.2022564501664448e-09, } , 
      {    7.3999811166597755e+00,     6.0622336877983744e+03,    -4.2944639403915259e-04,    -4.4589802629645018e-07,     2.1434000692797469e-09, } , 
      {    1.0655791603473999e+01,     2.2929166002781574e+04,    -5.5745872479211970e-04,    -4.5658845885312413e-07,     2.0845436883930495e-09, } , 
      {    3.2439146934181315e+00,    -1.9342939517544575e+04,    -2.1129324770891963e-05,    -1.7379828702844764e-07,     1.0251288324324914e-09, } , 
      {    6.4997251802323550e+00,    -2.4760072025613772e+03,    -1.4914165552385917e-04,    -1.8448871958512156e-07,     9.6627245154579374e-10, } , 
      {    9.7555356670465780e+00,     1.4390925112421824e+04,    -2.7715398627682641e-04,    -1.9517915214179551e-07,     9.0741607065909609e-10, } , 
      {    1.3011346153860801e+01,     3.1257857427405019e+04,    -4.0516631702979363e-04,    -2.0586958469846943e-07,     8.4855968977239844e-10, } , 
      {    5.5994692438049345e+00,    -1.1014248092921127e+04,     1.3116308299143423e-04,     7.6920587126207077e-08,    -2.1085516618815939e-10, } , 
      {    8.8552797306191575e+00,     5.8526842220620729e+03,     3.1507522384670024e-06,     6.6230154569533156e-08,    -2.6971154707485704e-10, } , 
      {    1.2111090217433381e+01,     2.2719616537045273e+04,    -1.2486157851450020e-04,     5.5539722012859221e-08,    -3.2856792796155479e-10, } , 
      {    1.5366900704247604e+01,     3.9586548852028471e+04,    -2.5287390926746745e-04,     4.4849289456185287e-08,    -3.8742430884825244e-10, } , 
      {    7.9550237941917370e+00,    -2.6855566682976769e+03,     2.8345549075376039e-04,     3.2763946128086182e-07,    -1.4468391648088103e-09, } , 
      {    1.1210834281005960e+01,     1.4181375646685523e+04,     1.5544316000079320e-04,     3.1694902872418788e-07,    -1.5056955456955079e-09, } , 
      {    1.4466644767820183e+01,     3.1048307961668725e+04,     2.7430829247825965e-05,     3.0625859616751398e-07,    -1.5645519265822058e-09, } , 
      {    1.7722455254634404e+01,     4.7915240276651923e+04,    -1.0058150150514125e-04,     2.9556816361084003e-07,    -1.6234083074689032e-09, } , 
      {    1.0310578344578540e+01,     5.6431347563257732e+03,     4.3574789851608657e-04,     5.7835833543551649e-07,    -2.6828231634294611e-09, } , 
      {    1.3566388831392763e+01,     2.2510067071308971e+04,     3.0773556776311935e-04,     5.6766790287884254e-07,    -2.7416795443161585e-09, } , 
      {    1.6822199318206984e+01,     3.9376999386292169e+04,     1.7972323701015216e-04,     5.5697747032216859e-07,    -2.8005359252028564e-09, } , 
      {    2.0078009805021210e+01,     5.6243931701275375e+04,     5.1710906257184928e-05,     5.4628703776549465e-07,    -2.8593923060895538e-09, } , 
      {    1.5921943381779565e+01,     3.0838758495932423e+04,     4.6002797552544553e-04,     8.1838677703349726e-07,    -3.9776635429368102e-09, } , 
      {    1.9177753868593790e+01,     4.7705690810915621e+04,     3.3201564477247831e-04,     8.0769634447682331e-07,    -4.0365199238235072e-09, } , 
      {    1.8277497932166369e+01,     3.9167449920555868e+04,     6.1232038328777182e-04,     1.0691056511881520e-06,    -5.2136475415574602e-09, } , 
      {    2.1533308418980589e+01,     5.6034382235539073e+04,     4.8430805253480449e-04,     1.0584152186314781e-06,    -5.2725039224441581e-09, } , 
      {   -5.3211732595581882e+00,    -4.3491162370464561e+04,    -9.1326757582839869e-04,    -1.4374666982442325e-06,     7.0917527163972588e-09, } , 
      {   -2.9656187091713857e+00,    -3.5162470945841123e+04,    -7.6097516806607234e-04,    -1.1867478240895778e-06,     5.8557687177766072e-09, } , 
      {    2.9019177764283710e-01,    -1.8295538630857918e+04,    -8.8898749881903961e-04,    -1.1974382566462517e-06,     5.7969123368899102e-09, } , 
      {   -6.1006415878458342e-01,    -2.6833779521217668e+04,    -6.0868276030374621e-04,    -9.3602894993492309e-07,     4.6197847191559572e-09, } , 
      {    2.6457463280296398e+00,    -9.9668472062344663e+03,    -7.3669509105671349e-04,    -9.4671938249159704e-07,     4.5609283382692593e-09, } , 
      {   -1.5103200952120039e+00,    -3.5372020411577418e+04,    -3.2837802178845282e-04,    -6.7461964322359443e-07,     3.4426571014220030e-09, } , 
      {    1.7454903916022191e+00,    -1.8505088096594216e+04,    -4.5639035254142003e-04,    -6.8531007578026838e-07,     3.3838007205353055e-09, } , 
      {    5.0013008784164423e+00,    -1.6381557816110135e+03,    -5.8440268329438725e-04,    -6.9600050833694232e-07,     3.3249443396486077e-09, } , 
      {    8.2571113652306654e+00,     1.5228776533372184e+04,    -7.1241501404735452e-04,    -7.0669094089361627e-07,     3.2660879587619102e-09, } , 
      {    8.4523445517479856e-01,    -2.7043328986953966e+04,    -1.7608561402612666e-04,    -4.2390076906893971e-07,     2.2066731028013521e-09, } , 
      {    4.1010449419890218e+00,    -1.0176396671970768e+04,    -3.0409794477909385e-04,    -4.3459120162561366e-07,     2.1478167219146547e-09, } , 
      {    7.3568554288032448e+00,     6.6905356430124348e+03,    -4.3211027553206107e-04,    -4.4528163418228755e-07,     2.0889603410279568e-09, } , 
      {    1.0612665915617468e+01,     2.3557467957995632e+04,    -5.6012260628502829e-04,    -4.5597206673896150e-07,     2.0301039601412594e-09, } , 
      {   -5.5021481252621962e-02,    -3.5581569877313726e+04,     1.0421912448916670e-04,    -1.6249146235761101e-07,     1.0295454850673994e-09, } , 
      {    3.2007890055616013e+00,    -1.8714637562330518e+04,    -2.3793206263800474e-05,    -1.7318189491428496e-07,     9.7068910418070173e-10, } , 
      {    6.4565994923758243e+00,    -1.8477052473473168e+03,    -1.5180553701676770e-04,    -1.8387232747095888e-07,     9.1183272329400397e-10, } , 
      {    9.7124099791900473e+00,     1.5019227067635886e+04,    -2.7981786776973489e-04,    -1.9456276002763283e-07,     8.5297634240730622e-10, } , 
      {    1.2968220466004270e+01,     3.1886159382619080e+04,    -4.0783019852270211e-04,    -2.0525319258430675e-07,     7.9411996152060857e-10, } , 
      {    1.6224030952818492e+01,     4.8753091697602285e+04,    -5.3584252927566932e-04,    -2.1594362514098070e-07,     7.3526358063391082e-10, } , 
      {    2.3005330691341808e+00,    -2.7252878452690275e+04,     2.5651153225149289e-04,     8.8227411797043692e-08,    -2.0643851355325143e-10, } , 
      {    5.5563435559484038e+00,    -1.0385946137707066e+04,     1.2849920149852570e-04,     7.7536979240369744e-08,    -2.6529489443994916e-10, } , 
      {    8.8121540427626268e+00,     6.4809861772761333e+03,     4.8687074555849077e-07,     6.6846546683695823e-08,    -3.2415127532664681e-10, } , 
      {    1.2067964529576850e+01,     2.3347918492259334e+04,    -1.2752546000740873e-04,     5.6156114127021881e-08,    -3.8300765621334456e-10, } , 
      {    1.5323775016391073e+01,     4.0214850807242532e+04,    -2.5553779076037592e-04,     4.5465681570347947e-08,    -4.4186403710004221e-10, } , 
      {    1.8579585503205294e+01,     5.7081783122225737e+04,    -3.8355012151334314e-04,     3.4775249013674012e-08,    -5.0072041798674001e-10, } , 
      {    4.6560876195209833e+00,    -1.8924187028066823e+04,     4.0880394001381907e-04,     3.3894628595169840e-07,    -1.4424225121739023e-09, } , 
      {    7.9118981063352063e+00,    -2.0572547130836165e+03,     2.8079160926085191e-04,     3.2825585339502445e-07,    -1.5012788930605999e-09, } , 
      {    1.1167708593149429e+01,     1.4809677601899584e+04,     1.5277927850788466e-04,     3.1756542083835050e-07,    -1.5601352739472976e-09, } , 
      {    1.4423519079963652e+01,     3.1676609916882786e+04,     2.4766947754917454e-05,     3.0687498828167661e-07,    -1.6189916548339954e-09, } , 
      {    1.7679329566777874e+01,     4.8543542231865984e+04,    -1.0324538299804976e-04,     2.9618455572500266e-07,    -1.6778480357206929e-09, } , 
      {    1.0267452656722009e+01,     6.2714367115398336e+03,     4.3308401702317809e-04,     5.7897472754967917e-07,    -2.7372628916812512e-09, } , 
      {    1.3523263143536232e+01,     2.3138369026523033e+04,     3.0507168627021087e-04,     5.6828429499300522e-07,    -2.7961192725679486e-09, } , 
      {    1.6779073630350453e+01,     4.0005301341506231e+04,     1.7705935551724363e-04,     5.5759386243633127e-07,    -2.8549756534546465e-09, } , 
      {    2.0034884117164680e+01,     5.6872233656489436e+04,     4.9047024764276418e-05,     5.4690342987965733e-07,    -2.9138320343413439e-09, } , 
      {    1.2623007207108811e+01,     1.4600128136163283e+04,     5.8537642478550427e-04,     8.2969360170433389e-07,    -3.9732468903019016e-09, } , 
      {    1.5878817693923034e+01,     3.1467060451146481e+04,     4.5736409403253705e-04,     8.1900316914765994e-07,    -4.0321032711885995e-09, } , 
      {    1.9134628180737259e+01,     4.8333992766129682e+04,     3.2935176327956984e-04,     8.0831273659098599e-07,    -4.0909596520752965e-09, } , 
      {    2.2390438667551479e+01,     6.5200925081112888e+04,     2.0133943252660259e-04,     7.9762230403431205e-07,    -4.1498160329619943e-09, } , 
      {    1.8234372244309839e+01,     3.9795751875769929e+04,     6.0965650179486324e-04,     1.0697220433023147e-06,    -5.2680872698092495e-09, } , 
      {    2.1490182731124058e+01,     5.6662684190753134e+04,     4.8164417104189602e-04,     1.0590316107456408e-06,    -5.3269436506959473e-09, } , 
      {    2.0589926794696638e+01,     4.8124443300393381e+04,     7.6194890955718936e-04,     1.3204409174569694e-06,    -6.5040712684299011e-09, } , 
      {    2.3845737281510864e+01,     6.4991375615376586e+04,     6.3393657880422209e-04,     1.3097504849002955e-06,    -6.5629276493165981e-09, } , 
      {   -7.7198534978015214e+00,    -5.1191551839873951e+04,    -1.0682238650836333e-03,    -1.6875691802847246e-06,     8.2732969867661204e-09, } , 
      {   -4.4640430109872984e+00,    -3.4324619524890753e+04,    -1.1962361958366007e-03,    -1.6982596128413984e-06,     8.2144406058794226e-09, } , 
      {   -5.3642989474147189e+00,    -4.2862860415250499e+04,    -9.1593145732130716e-04,    -1.4368503061300699e-06,     7.0373129881454688e-09, } , 
      {   -2.1084884606004959e+00,    -2.5995928100267301e+04,    -1.0439437880742743e-03,    -1.4475407386867437e-06,     6.9784566072587709e-09, } , 
      {   -6.2645548838421394e+00,    -5.1401101305610260e+04,    -6.3562671880601366e-04,    -1.1754409994187413e-06,     5.8601853704115150e-09, } , 
      {   -3.0087443970279164e+00,    -3.4534168990627062e+04,    -7.6363904955898082e-04,    -1.1861314319754151e-06,     5.8013289895248171e-09, } , 
      {    2.4706608978630662e-01,    -1.7667236675643857e+04,    -8.9165138031194809e-04,    -1.1968218645320890e-06,     5.7424726086381201e-09, } , 
      {   -3.9090003334553369e+00,    -4.3072409880986808e+04,    -4.8333431104368753e-04,    -9.2472212526408647e-07,     4.6242013717908641e-09, } , 
      {   -6.5318984664111390e-01,    -2.6205477566003607e+04,    -6.1134664179665469e-04,    -9.3541255782076041e-07,     4.5653449909041671e-09, } , 
      {    2.6026206401731091e+00,    -9.3385452510204050e+03,    -7.3935897254962196e-04,    -9.4610299037743436e-07,     4.5064886100174693e-09, } , 
      {    5.8584311269873321e+00,     7.5283870639627930e+03,    -8.6737130330258923e-04,    -9.5679342293410831e-07,     4.4476322291307714e-09, } , 
      {   -1.5534457830685344e+00,    -3.4743718456363356e+04,    -3.3104190328136135e-04,    -6.7400325110943175e-07,     3.3882173731702133e-09, } , 
      {    1.7023647037456886e+00,    -1.7876786141380155e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    4.9581751905599116e+00,    -1.0098538263969531e+03,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    8.2139856773741347e+00,     1.5857078488586245e+04,    -7.1507889554026300e-04,    -7.0607454877945359e-07,     3.2116482305101206e-09, } , 
      {    1.1469796164188356e+01,     3.2724010803569450e+04,    -8.4309122629323027e-04,    -7.1676498133612743e-07,     3.1527918496234227e-09, } , 
      {   -2.4537017194959549e+00,    -4.3281959346723117e+04,    -5.0737164766068003e-05,    -4.1259394439810308e-07,     2.2110897554362603e-09, } , 
      {    8.0210876731826808e-01,    -2.6415027031739904e+04,    -1.7874949551903517e-04,    -4.2328437695477703e-07,     2.1522333745495625e-09, } , 
      {    4.0579192541324911e+00,    -9.5480947167567065e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    7.3137297409467141e+00,     7.3188375982264952e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.0569540227760937e+01,     2.4185769913209693e+04,    -5.6278648777793676e-04,    -4.5535567462479882e-07,     1.9756642318894697e-09, } , 
      {    1.3825350714575158e+01,     4.1052702228192895e+04,    -6.9079881853090404e-04,    -4.6604610718147277e-07,     1.9168078510027719e-09, } , 
      {   -9.8147169109152443e-02,    -3.4953267922099665e+04,     1.0155524299625818e-04,    -1.6187507024344836e-07,     9.7510575681560951e-10, } , 
      {    3.1576633177050706e+00,    -1.8086335607116456e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    6.4134738045192936e+00,    -1.2194032921332564e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.6692842913335166e+00,     1.5647529022849945e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.2925094778147740e+01,     3.2514461337833141e+04,    -4.1049408001561064e-04,    -2.0463680047014410e-07,     7.3968023326881881e-10, } , 
      {    1.6180905264961961e+01,     4.9381393652816347e+04,    -5.3850641076857780e-04,    -2.1532723302681805e-07,     6.8082385238212106e-10, } , 
      {    2.2574073812776501e+00,    -2.6624576497476213e+04,     2.5384765075858436e-04,     8.8843803911206358e-08,    -2.6087824180504122e-10, } , 
      {    5.5132178680918731e+00,    -9.7576441824930062e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    8.7690283549060961e+00,     7.1092881324901937e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2024838841720319e+01,     2.3976220447473395e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5280649328534542e+01,     4.0843152762456593e+04,    -2.5820167225328446e-04,     4.6082073684510614e-08,    -4.9630376535183202e-10, } , 
      {    1.8536459815348763e+01,     5.7710085077439799e+04,    -3.8621400300625167e-04,     3.5391641127836679e-08,    -5.5516014623852977e-10, } , 
      {    4.6129619316644526e+00,    -1.8295885072852761e+04,     4.0614005852091054e-04,     3.3956267806586108e-07,    -1.4968622404256922e-09, } , 
      {    7.8687724184786756e+00,    -1.4289527578695561e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {    1.1124582905292899e+01,     1.5437979557113644e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.4380393392107122e+01,     3.2304911872096847e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.7636203878921343e+01,     4.9171844187080045e+04,    -1.0590926449095827e-04,     2.9680094783916534e-07,    -1.7322877639724827e-09, } , 
      {    2.0892014365735566e+01,     6.6038776502063251e+04,    -2.3392159524392549e-04,     2.8611051528249139e-07,    -1.7911441448591806e-09, } , 
      {    6.9685164820512551e+00,    -9.9671936482293131e+03,     5.5843246628323672e-04,     5.9028155222051580e-07,    -2.7328462390463430e-09, } , 
      {    1.0224326968865478e+01,     6.8997386667538940e+03,     4.3042013553026956e-04,     5.7959111966384185e-07,    -2.7917026199330408e-09, } , 
      {    1.3480137455679701e+01,     2.3766670981737094e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.6735947942493922e+01,     4.0633603296720292e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.9991758429308149e+01,     5.7500535611703497e+04,     4.6383143271367907e-05,     5.4751982199382001e-07,    -2.9682717625931336e-09, } , 
      {    2.3247568916122368e+01,     7.4367467926686702e+04,    -8.1629187481599311e-05,     5.3682938943714616e-07,    -3.0271281434798314e-09, } , 
      {    1.2579881519252281e+01,     1.5228430091377342e+04,     5.8271254329259579e-04,     8.3030999381849657e-07,    -4.0276866185536917e-09, } , 
      {    1.5835692006066504e+01,     3.2095362406360542e+04,     4.5470021253962852e-04,     8.1961956126182262e-07,    -4.0865429994403895e-09, } , 
      {    1.9091502492880728e+01,     4.8962294721343744e+04,     3.2668788178666131e-04,     8.0892912870514867e-07,    -4.1453993803270866e-09, } , 
      {    2.2347312979694948e+01,     6.5829227036326949e+04,     1.9867555103369409e-04,     7.9823869614847473e-07,    -4.2042557612137844e-09, } , 
      {    1.4935436069639083e+01,     2.3557121516000792e+04,     7.3500495105492192e-04,     1.0810288679731512e-06,    -5.2636706171743425e-09, } , 
      {    1.8191246556453308e+01,     4.0424053830983990e+04,     6.0699262030195476e-04,     1.0703384354164773e-06,    -5.3225269980610395e-09, } , 
      {    2.1447057043267527e+01,     5.7290986145967196e+04,     4.7898028954898749e-04,     1.0596480028598035e-06,    -5.3813833789477374e-09, } , 
      {    2.4702867530081754e+01,     7.4157918460950401e+04,     3.5096795879602027e-04,     1.0489575703031294e-06,    -5.4402397598344352e-09, } , 
      {    2.0546801106840107e+01,     4.8752745255607442e+04,     7.5928502806428089e-04,     1.3210573095711321e-06,    -6.5585109966816912e-09, } , 
      {    2.3802611593654333e+01,     6.5619677570590648e+04,     6.3127269731131361e-04,     1.3103668770144582e-06,    -6.6173673775683882e-09, } , 
      {    2.2902355657226913e+01,     5.7081436680230894e+04,     9.1157743582660701e-04,     1.5717761837257868e-06,    -7.7944949953023412e-09, } , 
      {    2.6158166144041132e+01,     7.3948368995214085e+04,     7.8356510507363985e-04,     1.5610857511691129e-06,    -7.8533513761890390e-09, } , 
      {   -5.4074246352712496e+00,    -4.2234558460036438e+04,    -9.1859533881421564e-04,    -1.4362339140159072e-06,     6.9828732598936787e-09, } , 
      {   -3.0518700848844471e+00,    -3.3905867035413001e+04,    -7.6630293105188930e-04,    -1.1855150398612525e-06,     5.7468892612730270e-09, } , 
      {    2.0394040192977614e-01,    -1.7038934720429796e+04,    -8.9431526180485657e-04,    -1.1962054724179263e-06,     5.6880328803863300e-09, } , 
      {   -6.9631553449764438e-01,    -2.5577175610789545e+04,    -6.1401052328956317e-04,    -9.3479616570659773e-07,     4.5109052626523770e-09, } , 
      {    2.5594949523165784e+00,    -8.7102432958063437e+03,    -7.4202285404253044e-04,    -9.4548659826327168e-07,     4.4520488817656792e-09, } , 
      {   -1.5965714709250649e+00,    -3.4115416501149295e+04,    -3.3370578477426988e-04,    -6.7338685899526907e-07,     3.3337776449184236e-09, } , 
      {    1.6592390158891581e+00,    -1.7248484186166093e+04,    -4.6171811552723710e-04,    -6.8407729155194302e-07,     3.2749212640317262e-09, } , 
      {    4.9150495027033809e+00,    -3.8155187118289268e+02,    -5.8973044628020420e-04,    -6.9476772410861696e-07,     3.2160648831450284e-09, } , 
      {    7.5898307946173760e-01,    -2.5786725076525843e+04,    -1.8141337701194367e-04,    -4.2266798484061435e-07,     2.0977936462977728e-09, } , 
      {    4.0147935662759604e+00,    -8.9197927615426452e+03,    -3.0942570776491091e-04,    -4.3335841739728830e-07,     2.0389372654110754e-09, } , 
      {    7.2706040530901834e+00,     7.9471395534405556e+03,    -4.3743803851787813e-04,    -4.4404884995396219e-07,     1.9800808845243775e-09, } , 
      {    1.0526414539904406e+01,     2.4814071868423754e+04,    -5.6545036927084524e-04,    -4.5473928251063614e-07,     1.9212245036376801e-09, } , 
      {    3.1145376298485399e+00,    -1.7458033651902395e+04,    -2.9120969249617496e-05,    -1.7194911068595966e-07,     8.6180964767712209e-10, } , 
      {    6.3703481166627629e+00,    -5.9110133691919600e+02,    -1.5713330000258471e-04,    -1.8263954324263358e-07,     8.0295326679042444e-10, } , 
      {    9.6261586034769859e+00,     1.6275830978064005e+04,    -2.8514563075555195e-04,    -1.9332997579930752e-07,     7.4409688590372669e-10, } , 
      {    1.2881969090291209e+01,     3.3142763293047203e+04,    -4.1315796150851917e-04,    -2.0402040835598145e-07,     6.8524050501702904e-10, } , 
      {    2.2142816934211194e+00,    -2.5996274542262152e+04,     2.5118376926567583e-04,     8.9460196025369025e-08,    -3.1531797005683104e-10, } , 
      {    5.4700921802353424e+00,    -9.1293422272789467e+03,     1.2317143851270869e-04,     7.8769763468695077e-08,    -3.7417435094352879e-10, } , 
      {    8.7259026670495654e+00,     7.7375900877042541e+03,    -4.8408922402585326e-06,     6.8079330912021156e-08,    -4.3303073183022644e-10, } , 
      {    1.1981713153863788e+01,     2.4604522402687457e+04,    -1.3285322299322574e-04,     5.7388898355347215e-08,    -4.9188711271692414e-10, } , 
      {    1.5237523640678011e+01,     4.1471454717670655e+04,    -2.6086555374619299e-04,     4.6698465798673280e-08,    -5.5074349360362179e-10, } , 
      {    4.5698362438079219e+00,    -1.7667583117638700e+04,     4.0347617702800201e-04,     3.4017907018002376e-07,    -1.5513019686774820e-09, } , 
      {    7.8256467306221449e+00,    -8.0065080265549568e+02,     2.7546384627503485e-04,     3.2948863762334981e-07,    -1.6101583495641797e-09, } , 
      {    1.1081457217436368e+01,     1.6066281512327703e+04,     1.4745151552206766e-04,     3.1879820506667586e-07,    -1.6690147304508773e-09, } , 
      {    1.4337267704250591e+01,     3.2933213827310909e+04,     1.9439184769100432e-05,     3.0810777251000197e-07,    -1.7278711113375752e-09, } , 
      {    1.7593078191064812e+01,     4.9800146142294107e+04,    -1.0857314598386679e-04,     2.9741733995332802e-07,    -1.7867274922242726e-09, } , 
      {    1.0181201281008947e+01,     7.5280406219679544e+03,     4.2775625403736103e-04,     5.8020751177800453e-07,    -2.8461423481848305e-09, } , 
      {    1.3437011767823170e+01,     2.4394972936951155e+04,     2.9974392328439381e-04,     5.6951707922133058e-07,    -2.9049987290715279e-09, } , 
      {    1.6692822254637392e+01,     4.1261905251934353e+04,     1.7173159253142662e-04,     5.5882664666465663e-07,    -2.9638551099582258e-09, } , 
      {    1.9948632741451618e+01,     5.8128837566917558e+04,     4.3719261778459396e-05,     5.4813621410798269e-07,    -3.0227114908449232e-09, } , 
      {    1.2536755831395750e+01,     1.5856732046591402e+04,     5.8004866179968732e-04,     8.3092638593265925e-07,    -4.0821263468054817e-09, } , 
      {    1.5792566318209973e+01,     3.2723664361574603e+04,     4.5203633104671999e-04,     8.2023595337598530e-07,    -4.1409827276921796e-09, } , 
      {    1.9048376805024198e+01,     4.9590596676557805e+04,     3.2402400029375277e-04,     8.0954552081931135e-07,    -4.1998391085788766e-09, } , 
      {    2.2304187291838417e+01,     6.6457528991541010e+04,     1.9601166954078558e-04,     7.9885508826263741e-07,    -4.2586954894655745e-09, } , 
      {    1.8148120868596777e+01,     4.1052355786198052e+04,     6.0432873880904628e-04,     1.0709548275306400e-06,    -5.3769667263128296e-09, } , 
      {    2.1403931355410997e+01,     5.7919288101181257e+04,     4.7631640805607896e-04,     1.0602643949739662e-06,    -5.4358231071995275e-09, } , 
      {    2.0503675418983576e+01,     4.9381047210821504e+04,     7.5662114657137241e-04,     1.3216737016852947e-06,    -6.6129507249334813e-09, } , 
      {    2.3759485905797803e+01,     6.6247979525804709e+04,     6.2860881581840514e-04,     1.3109832691286209e-06,    -6.6718071058201783e-09, } , 
      {   -7.3944122235417487e-01,    -2.4948873655575488e+04,    -6.1667440478247175e-04,    -9.3417977359243505e-07,     4.4564655344005878e-09, } , 
      {    2.5163692644600482e+00,    -8.0819413405922842e+03,    -7.4468673553543903e-04,    -9.4487020614910900e-07,     4.3976091535138899e-09, } , 
      {    1.6161133280326276e+00,    -1.6620182230952036e+04,    -4.6438199702014557e-04,    -6.8346089943778034e-07,     3.2204815357799361e-09, } , 
      {    4.8719238148468502e+00,     2.4675008403116772e+02,    -5.9239432777311279e-04,    -6.9415133199445428e-07,     3.1616251548932383e-09, } , 
      {    7.1585739160520712e-01,    -2.5158423121311782e+04,    -1.8407725850485220e-04,    -4.2205159272645172e-07,     2.0433539180459827e-09, } , 
      {    3.9716678784194301e+00,    -8.2914908063285857e+03,    -3.1208958925781939e-04,    -4.3274202528312567e-07,     1.9844975371592853e-09, } , 
      {    7.2274783652336527e+00,     8.5754415086546160e+03,    -4.4010192001078661e-04,    -4.4343245783979956e-07,     1.9256411562725874e-09, } , 
      {    1.0483288852047876e+01,     2.5442373823637812e+04,    -5.6811425076375383e-04,    -4.5412289039647351e-07,     1.8667847753858900e-09, } , 
      {    3.0714119419920096e+00,    -1.6829731696688337e+04,    -3.1784850742526006e-05,    -1.7133271857179698e-07,     8.0736991942533222e-10, } , 
      {    6.3272224288062322e+00,     3.7200618294864398e+01,    -1.5979718149549324e-04,    -1.8202315112847090e-07,     7.4851353853863457e-10, } , 
      {    9.5830329156204552e+00,     1.6904132933278066e+04,    -2.8780951224846043e-04,    -1.9271358368514484e-07,     6.8965715765193682e-10, } , 
      {    1.2838843402434678e+01,     3.3771065248261264e+04,    -4.1582184300142764e-04,    -2.0340401624181877e-07,     6.3080077676523917e-10, } , 
      {    5.4269664923788117e+00,    -8.5010402720648854e+03,     1.2050755701980017e-04,     7.9386155582857744e-08,    -4.2861407919531855e-10, } , 
      {    8.6827769791930347e+00,     8.3658920429183145e+03,    -7.5047737331670442e-06,     6.8695723026183823e-08,    -4.8747046008201626e-10, } , 
      {    1.1938587466007258e+01,     2.5232824357901518e+04,    -1.3551710448613427e-04,     5.8005290469509875e-08,    -5.4632684096871401e-10, } , 
      {    1.5194397952821481e+01,     4.2099756672884716e+04,    -2.6352943523910146e-04,     4.7314857912835940e-08,    -6.0518322185541166e-10, } , 
      {    7.7825210427656142e+00,    -1.7234884744143528e+02,     2.7279996478212637e-04,     3.3010502973751244e-07,    -1.6645980778159693e-09, } , 
      {    1.1038331529579837e+01,     1.6694583467541765e+04,     1.4478763402915913e-04,     3.1941459718083849e-07,    -1.7234544587026670e-09, } , 
      {    1.4294142016394060e+01,     3.3561515782524970e+04,     1.6775303276191921e-05,     3.0872416462416459e-07,    -1.7823108395893648e-09, } , 
      {    1.3393886079966640e+01,     2.5023274892165216e+04,     2.9708004179148533e-04,     5.7013347133549326e-07,    -2.9594384573233180e-09, } , 
      {    1.6649696566780861e+01,     4.1890207207148414e+04,     1.6906771103851809e-04,     5.5944303877881931e-07,    -3.0182948382100159e-09, } , 
      {    3.9285421905628999e+00,    -7.6631888511145253e+03,    -3.1475347075072792e-04,    -4.3212563316896299e-07,     1.9300578089074956e-09, } , 
      {    7.1843526773771229e+00,     9.2037434638686755e+03,    -4.4276580150369514e-04,    -4.4281606572563688e-07,     1.8712014280207978e-09, } , 
      {    6.2840967409497024e+00,     6.6550257350892480e+02,    -1.6246106298840174e-04,    -1.8140675901430824e-07,     6.9407381028684481e-10, } , 
      {    9.5399072277639245e+00,     1.7532434888492127e+04,    -2.9047339374136896e-04,    -1.9209719157098219e-07,     6.3521742940014706e-10, } , 
      {    8.6396512913365040e+00,     8.9941939981323740e+03,    -1.0168655226075557e-05,     6.9312115140346476e-08,    -5.4191018833380602e-10, } , 
      {    1.1895461778150727e+01,     2.5861126313115576e+04,    -1.3818098597904278e-04,     5.8621682583672542e-08,    -6.0076656922050377e-10, } , 
      {    1.0995205841723307e+01,     1.7322885422755826e+04,     1.4212375253625062e-04,     3.2003098929500117e-07,    -1.7778941869544569e-09, } , 
      {    1.1741317667245490e+01,     4.6670679872154606e+03,    -1.7956936209286749e-04,    -1.5404625043952347e-07,     8.1670258278669083e-10, } , 
      {    1.4997128154059713e+01,     2.1534000302198659e+04,    -3.0758169284583471e-04,    -1.6473668299619739e-07,     7.5784620189999318e-10, } , 
      {    1.4096872217632292e+01,     1.2995759411838910e+04,    -2.7276954330541300e-05,     9.6672623715131247e-08,    -4.1928141583396000e-10, } , 
      {    6.9870828786153538e+00,    -1.1362012906817377e+04,    -4.8681805911042838e-04,    -6.5486760663467023e-07,     3.2342308517762030e-09, } , 
      {    9.3426374290021563e+00,    -3.0333214821939291e+03,    -3.3452565134810220e-04,    -4.0414873248001551e-07,     1.9982468531555522e-09, } , 
      {    1.2598447915816379e+01,     1.3833610832789271e+04,    -4.6253798210106942e-04,    -4.1483916503668941e-07,     1.9393904722688544e-09, } , 
      {    1.1698191979388959e+01,     5.2953699424295210e+03,    -1.8223324358577600e-04,    -1.5342985832536082e-07,     7.6226285453490107e-10, } , 
      {    1.4954002466203182e+01,     2.2162302257412717e+04,    -3.1024557433874324e-04,    -1.6412029088203474e-07,     7.0340647364820342e-10, } , 
      {    1.0797936042961538e+01,    -3.2428709479302288e+03,     9.8071494929517403e-05,     1.0797944838596785e-07,    -4.1486476319905206e-10, } , 
      {    1.4053746529775761e+01,     1.3624061367052971e+04,    -2.9940835823449814e-05,     9.7289015829293900e-08,    -4.7372114408574976e-10, } , 
      {    1.7309557016589984e+01,     3.0490993682036169e+04,    -1.5795316657641703e-04,     8.6598583272619979e-08,    -5.3257752497244741e-10, } , 
      {    1.6409301080162564e+01,     2.1952752791676423e+04,     1.2235157193887637e-04,     3.4800788998394865e-07,    -1.7097051427064007e-09, } , 
      {    1.8764855630549366e+01,     3.0281444216299868e+04,     2.7464397970120252e-04,     5.9872676413860337e-07,    -2.9456891413270511e-09, } , 
      {    4.5884026403720206e+00,    -1.9062402376226768e+04,    -6.4177434836566304e-04,    -9.0497008867516227e-07,     4.4157751221450642e-09, } , 
      {    6.9439571907588231e+00,    -1.0733710951603316e+04,    -4.8948194060333692e-04,    -6.5425121452050755e-07,     3.1797911235244130e-09, } , 
      {    9.2995117411456256e+00,    -2.4050195269798687e+03,    -3.3718953284101068e-04,    -4.0353234036585289e-07,     1.9438071249037621e-09, } , 
      {    1.2555322227959849e+01,     1.4461912788003330e+04,    -4.6520186359397790e-04,    -4.1422277292252678e-07,     1.8849507440170643e-09, } , 
      {    8.3992558047182051e+00,    -1.0943260417339618e+04,    -5.6884794325717289e-05,    -1.4212303365452419e-07,     7.6667950716980895e-10, } , 
      {    1.1655066291532428e+01,     5.9236718976435814e+03,    -1.8489712507868453e-04,    -1.5281346621119814e-07,     7.0782312628311120e-10, } , 
      {    1.4910876778346651e+01,     2.2790604212626778e+04,    -3.1290945583165172e-04,    -1.6350389876787206e-07,     6.4896674539641355e-10, } , 
      {    1.8166687265160874e+01,     3.9657536527609976e+04,    -4.4092178658461893e-04,    -1.7419433132454601e-07,     5.9011036450971580e-10, } , 
      {    1.0754810355105008e+01,    -2.6145689927161684e+03,     9.5407613436608899e-05,     1.0859584050013051e-07,    -4.6930449145084188e-10, } , 
      {    1.4010620841919231e+01,     1.4252363322267032e+04,    -3.2604717316358325e-05,     9.7905407943456567e-08,    -5.2816087233753963e-10, } , 
      {    1.7266431328733454e+01,     3.1119295637250230e+04,    -1.6061704806932556e-04,     8.7214975386782646e-08,    -5.8701725322423728e-10, } , 
      {    2.0522241815547677e+01,     4.7986227952233428e+04,    -2.8862937882229275e-04,     7.6524542830108698e-08,    -6.4587363411093503e-10, } , 
      {    1.6366175392306033e+01,     2.2581054746890481e+04,     1.1968769044596786e-04,     3.4862428209811127e-07,    -1.7641448709581904e-09, } , 
      {    1.9621985879120256e+01,     3.9447987061873682e+04,    -8.3246403069993746e-06,     3.3793384954143738e-07,    -1.8230012518448880e-09, } , 
      {    1.8721729942692836e+01,     3.0909746171513929e+04,     2.7198009820829404e-04,     5.9934315625276605e-07,    -3.0001288695788412e-09, } , 
      {    2.1977540429507059e+01,     4.7776678486497127e+04,     1.4396776745532680e-04,     5.8865272369609210e-07,    -3.0589852504655390e-09, } , 
      {    2.1077284493079638e+01,     3.9238437596137381e+04,     4.2427250597062023e-04,     8.5006203040742076e-07,    -4.2361128681994916e-09, } , 
      {    2.4333094979893861e+01,     5.6105369911120579e+04,     2.9626017521765301e-04,     8.3937159785074682e-07,    -4.2949692490861895e-09, } , 
      {    4.5452769525154899e+00,    -1.8434100421012707e+04,    -6.4443822985857152e-04,    -9.0435369656099959e-07,     4.3613353938932741e-09, } , 
      {    7.8010874393297129e+00,    -1.5671681060295086e+03,    -7.7245056061153879e-04,    -9.1504412911767354e-07,     4.3024790130065763e-09, } , 
      {    6.9008315029022924e+00,    -1.0105408996389257e+04,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {    1.0156641989716515e+01,     6.7615233185939433e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {    1.3412452476530738e+01,     2.3628455633577141e+04,    -7.4817048360217983e-04,    -6.7501568751969277e-07,     3.0076386334992276e-09, } , 
      {    6.0005755664748719e+00,    -1.8643649886749008e+04,    -2.1184108358095199e-04,    -3.9222551569501626e-07,     1.9482237775386699e-09, } , 
      {    9.2563860532890949e+00,    -1.7767175717658083e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.2512196540103318e+01,     1.5090214743217392e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.5768007026917541e+01,     3.1957147058200590e+04,    -5.9587807583985370e-04,    -4.2429681336503805e-07,     1.7716546348785768e-09, } , 
      {    8.3561301168616744e+00,    -1.0314958462125558e+04,    -5.9548675818625800e-05,    -1.4150664154036154e-07,     7.1223977891801919e-10, } , 
      {    1.1611940603675897e+01,     6.5519738528576418e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.4867751090490120e+01,     2.3418906167840840e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.8123561577304343e+01,     4.0285838482824038e+04,    -4.4358566807752747e-04,    -1.7357793921038335e-07,     5.3567063625792603e-10, } , 
      {    1.0711684667248477e+01,    -1.9862670375021080e+03,     9.2743731943700382e-05,     1.0921223261429318e-07,    -5.2374421970263164e-10, } , 
      {    1.3967495154062700e+01,     1.4880665277481092e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.7223305640876923e+01,     3.1747597592464292e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    2.0479116127691146e+01,     4.8614529907447490e+04,    -2.9129326031520128e-04,     7.7140934944271365e-08,    -7.0031336236272480e-10, } , 
      {    1.3067239217635279e+01,     6.3424243871213421e+03,     2.4503613970602655e-04,     3.5993110676894790e-07,    -1.7597282183232826e-09, } , 
      {    1.6323049704449502e+01,     2.3209356702104542e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    1.9578860191263725e+01,     4.0076289017087744e+04,    -1.0988521799907885e-05,     3.3855024165560006e-07,    -1.8774409800966779e-09, } , 
      {    2.2834670678077948e+01,     5.6943221332070942e+04,    -1.3900085255287510e-04,     3.2785980909892611e-07,    -1.9362973609833757e-09, } , 
      {    1.8678604254836305e+01,     3.1538048126727990e+04,     2.6931621671538551e-04,     5.9995954836692873e-07,    -3.0545685978306309e-09, } , 
      {    2.1934414741650528e+01,     4.8404980441711188e+04,     1.4130388596241830e-04,     5.8926911581025478e-07,    -3.1134249787173287e-09, } , 
      {    2.1034158805223107e+01,     3.9866739551351442e+04,     4.2160862447771169e-04,     8.5067842252158344e-07,    -4.2905525964512817e-09, } , 
      {    2.4289969292037330e+01,     5.6733671866334640e+04,     2.9359629372474448e-04,     8.3998798996490950e-07,    -4.3494089773379795e-09, } , 
      {    2.3389713355609910e+01,     4.8195430975974894e+04,     5.7390103224003793e-04,     1.1013972966762382e-06,    -5.5265365950719325e-09, } , 
      {    2.6645523842424133e+01,     6.5062363290958092e+04,     4.4588870148707066e-04,     1.0907068641195641e-06,    -5.5853929759586304e-09, } , 
      {    4.5021512646589592e+00,    -1.7805798465798645e+04,    -6.4710211135148000e-04,    -9.0373730444683691e-07,     4.3068956656414841e-09, } , 
      {    7.7579617514731822e+00,    -9.3886615081544824e+02,    -7.7511444210444727e-04,    -9.1442773700351086e-07,     4.2480392847547862e-09, } , 
      {    6.8577058150457617e+00,    -9.4771070411751971e+03,    -4.9480970358915387e-04,    -6.5301843029218219e-07,     3.0709116670208336e-09, } , 
      {    1.0113516301859985e+01,     7.3898252738080037e+03,    -6.2282203434212114e-04,    -6.6370886284885614e-07,     3.0120552861341358e-09, } , 
      {    9.2132603654325642e+00,    -1.1484156165517479e+03,    -3.4251729582682774e-04,    -4.0229955613752753e-07,     1.8349276684001826e-09, } , 
      {    1.2469070852246787e+01,     1.5718516698431453e+04,    -4.7052962657979496e-04,    -4.1298998869420142e-07,     1.7760712875134847e-09, } , 
      {    8.3130044290051437e+00,    -9.6866565069114986e+03,    -6.2212557311534317e-05,    -1.4089024942619888e-07,     6.5780005066622942e-10, } , 
      {    1.1568814915819367e+01,     7.1802758080717022e+03,    -1.9022488806450153e-04,    -1.5158068198287283e-07,     5.9894366977953167e-10, } , 
      {    1.4824625402633590e+01,     2.4047208123054901e+04,    -3.1823721881746878e-04,    -1.6227111453954675e-07,     5.4008728889283402e-10, } , 
      {    1.8080435889447813e+01,     4.0914140438038099e+04,    -4.4624954957043600e-04,    -1.7296154709622070e-07,     4.8123090800613627e-10, } , 
      {    1.0668558979391946e+01,    -1.3579650822880476e+03,     9.0079850450791864e-05,     1.0982862472845585e-07,    -5.7818394795442141e-10, } , 
      {    1.3924369466206169e+01,     1.5508967232695151e+04,    -3.7932480302175347e-05,     9.9138192171781901e-08,    -6.3704032884111916e-10, } , 
      {    1.7180179953020392e+01,     3.2375899547678353e+04,    -1.6594481105514257e-04,     8.8447759615107979e-08,    -6.9589670972781681e-10, } , 
      {    2.0435990439834615e+01,     4.9242831862661551e+04,    -2.9395714180810982e-04,     7.7757327058434031e-08,    -7.5475309061451456e-10, } , 
      {    1.6279924016592972e+01,     2.3837658657318603e+04,     1.1435992746015083e-04,     3.4985706632643663e-07,    -1.8730243274617701e-09, } , 
      {    1.9535734503407195e+01,     4.0704590972301805e+04,    -1.3652403292816396e-05,     3.3916663376976274e-07,    -1.9318807083484675e-09, } , 
      {    1.8635478566979774e+01,     3.2166350081942051e+04,     2.6665233522247698e-04,     6.0057594048109141e-07,    -3.1090083260824205e-09, } , 
      {    2.1891289053793997e+01,     4.9033282396925249e+04,     1.3864000446950979e-04,     5.8988550792441746e-07,    -3.1678647069691184e-09, } , 
      {    2.0991033117366577e+01,     4.0495041506565503e+04,     4.1894474298480316e-04,     8.5129481463574612e-07,    -4.3449923247030718e-09, } , 
      {    2.4246843604180800e+01,     5.7361973821548701e+04,     2.9093241223183595e-04,     8.4060438207907218e-07,    -4.4038487055897696e-09, } , 
      {    9.1701346775760335e+00,    -5.2011366133768752e+02,    -3.4518117731973622e-04,    -4.0168316402336490e-07,     1.7804879401483929e-09, } , 
      {    1.2425945164390257e+01,     1.6346818653645512e+04,    -4.7319350807270344e-04,    -4.1237359658003879e-07,     1.7216315592616951e-09, } , 
      {    1.1525689227962836e+01,     7.8085777632857626e+03,    -1.9288876955741007e-04,    -1.5096428986871015e-07,     5.4450394152774180e-10, } , 
      {    1.4781499714777059e+01,     2.4675510078268962e+04,    -3.2090110031037726e-04,    -1.6165472242538407e-07,     4.8564756064104415e-10, } , 
      {    1.3881243778349639e+01,     1.6137269187909213e+04,    -4.0596361795083857e-05,     9.9754584285944567e-08,    -6.9148005709290903e-10, } , 
      {    1.7137054265163862e+01,     3.3004201502892414e+04,    -1.6860869254805110e-04,     8.9064151729270646e-08,    -7.5033643797960668e-10, } , 
      {    1.6236798328736441e+01,     2.4465960612532661e+04,     1.1169604596724232e-04,     3.5047345844059926e-07,    -1.9274640557135598e-09, } , 
      {    1.9492608815550664e+01,     4.1332892927515866e+04,    -1.6316284785724907e-05,     3.3978302588392537e-07,    -1.9863204366002576e-09, } , 
      {    1.8592352879123244e+01,     3.2794652037156113e+04,     2.6398845372956851e-04,     6.0119233259525409e-07,    -3.1634480543342106e-09, } , 
      {    8.2267530532920823e+00,    -8.4300525964833760e+03,    -6.7540320297351339e-05,    -1.3965746519787355e-07,     5.4892059416264979e-10, } , 
      {    1.1482563540106305e+01,     8.4368797184998220e+03,    -1.9555265105031857e-04,    -1.5034789775454750e-07,     4.9006421327595204e-10, } , 
      {    1.4738374026920528e+01,     2.5303812033483020e+04,    -3.2356498180328579e-04,    -1.6103833031122142e-07,     4.3120783238925439e-10, } , 
      {    1.7883166090686046e+01,     2.0348384067352046e+04,    -4.9030180717716924e-04,    -3.8501308800525405e-07,     1.8442603317615411e-09, } , 
      {    1.6982910154258622e+01,     1.1810143176992298e+04,    -2.0999706866187579e-04,    -1.2360378129392538e-07,     6.6713271402758782e-10, } , 
      {    2.0238720641072849e+01,     2.8677075491975495e+04,    -3.3800939941484295e-04,    -1.3429421385059933e-07,     6.0827633314089017e-10, } , 
      {    1.9338464704645425e+01,     2.0138834601615748e+04,    -5.7704660899549607e-05,     1.2711509286072934e-07,    -5.6885128459306301e-10, } , 
      {    2.2594275191459651e+01,     3.7005766916598950e+04,    -1.8571699165251682e-04,     1.1642466030405540e-07,    -6.2770766547976066e-10, } , 
      {    2.1694019255032227e+01,     2.8467526026239200e+04,     9.4587746862776574e-05,     3.7783396701538406e-07,    -1.8048352832137137e-09, } , 
      {    1.4584229916015289e+01,     4.1097537075829086e+03,    -3.6495335791711045e-04,    -3.7370626333441742e-07,     1.8486769843964489e-09, } , 
      {    1.7840040402829516e+01,     2.0976686022566108e+04,    -4.9296568867007772e-04,    -3.8439669589109137e-07,     1.7898206035097512e-09, } , 
      {    1.6939784466402092e+01,     1.2438445132206358e+04,    -2.1266095015478429e-04,    -1.2298738917976270e-07,     6.1269298577579806e-10, } , 
      {    2.0195594953216318e+01,     2.9305377447189556e+04,    -3.4067328090775148e-04,    -1.3367782173643665e-07,     5.5383660488910041e-10, } , 
      {    1.6039528529974671e+01,     3.9002042418466071e+03,     6.7643788360509106e-05,     1.3842191753156594e-07,    -5.6443463195815502e-10, } , 
      {    1.9295339016788894e+01,     2.0767136556829810e+04,    -6.0368542392458111e-05,     1.2773148497489202e-07,    -6.2329101284485277e-10, } , 
      {    2.2551149503603121e+01,     3.7634068871813004e+04,    -1.8838087314542533e-04,     1.1704105241821807e-07,    -6.8214739373155042e-10, } , 
      {    2.1650893567175697e+01,     2.9095827981453262e+04,     9.1923865369868070e-05,     3.7845035912954674e-07,    -1.8592750114655036e-09, } , 
      {    2.4906704053989923e+01,     4.5962760296436456e+04,    -3.6088465383099147e-05,     3.6775992657287279e-07,    -1.9181313923522015e-09, } , 
      {    2.4006448117562499e+01,     3.7424519406076710e+04,     2.4421627313219422e-04,     6.2916923328420140e-07,    -3.0952590100861544e-09, } , 
      {    2.7262258604376726e+01,     5.4291451721059901e+04,     1.1620394237922703e-04,     6.1847880072752746e-07,    -3.1541153909728523e-09, } , 
      {    9.8299951273851534e+00,    -1.1919327186449931e+04,    -6.7220205493467129e-04,    -8.7452761952956423e-07,     4.2662052533859605e-09, } , 
      {    1.2185549677771956e+01,    -3.5906357618264792e+03,    -5.1990964717234516e-04,    -6.2380874537490951e-07,     3.0302212547653101e-09, } , 
      {    1.5441360164586182e+01,     1.3276296553156719e+04,    -6.4792197792531232e-04,    -6.3449917793158346e-07,     2.9713648738786122e-09, } , 
      {    1.1285293741344537e+01,    -1.2128876652186233e+04,    -2.3960490865705179e-04,    -3.6239943866358079e-07,     1.8530936370313569e-09, } , 
      {    1.4541104228158758e+01,     4.7380556627969690e+03,    -3.6761723941001898e-04,    -3.7308987122025474e-07,     1.7942372561446590e-09, } , 
      {    1.7796914714972985e+01,     2.1604987977780169e+04,    -4.9562957016298620e-04,    -3.8378030377692869e-07,     1.7353808752579614e-09, } , 
      {    2.1052725201787204e+01,     3.8471920292763360e+04,    -6.2364190091595336e-04,    -3.9447073633360264e-07,     1.6765244943712637e-09, } , 
      {    1.3640848291731340e+01,    -3.8001852275627825e+03,    -8.7312500894725593e-05,    -1.1168056450892613e-07,     6.1710963841070604e-10, } , 
      {    1.6896658778545561e+01,     1.3066747087420419e+04,    -2.1532483164769280e-04,    -1.2237099706560005e-07,     5.5825325752400829e-10, } , 
      {    2.0152469265359787e+01,     2.9933679402403613e+04,    -3.4333716240066001e-04,    -1.3306142962227400e-07,     4.9939687663731064e-10, } , 
      {    2.3408279752174007e+01,     4.6800611717386812e+04,    -4.7134949315362723e-04,    -1.4375186217894792e-07,     4.4054049575061284e-10, } , 
      {    1.5996402842118142e+01,     4.5285061970606675e+03,     6.4979906867600589e-05,     1.3903830964572859e-07,    -6.1887436020994479e-10, } , 
      {    1.9252213328932363e+01,     2.1395438512043867e+04,    -6.3032423885366629e-05,     1.2834787708905467e-07,    -6.7773074109664254e-10, } , 
      {    2.2508023815746590e+01,     3.8262370827027065e+04,    -1.9104475463833383e-04,     1.1765744453238072e-07,    -7.3658712198334019e-10, } , 
      {    2.5763834302560809e+01,     5.5129303142010263e+04,    -3.1905708539130105e-04,     1.0696701197570680e-07,    -7.9544350287003794e-10, } , 
      {    1.8351957392504943e+01,     1.2857197621684118e+04,     2.1727231462992678e-04,     3.8975718380038336e-07,    -1.8548583588305956e-09, } , 
      {    2.1607767879319166e+01,     2.9724129936667319e+04,     8.9259983876959552e-05,     3.7906675124370942e-07,    -1.9137147397172933e-09, } , 
      {    2.4863578366133392e+01,     4.6591062251650517e+04,    -3.8752346876007665e-05,     3.6837631868703547e-07,    -1.9725711206039911e-09, } , 
      {    2.3963322429705968e+01,     3.8052821361290771e+04,     2.4155239163928575e-04,     6.2978562539836408e-07,    -3.1496987383379441e-09, } , 
      {    2.7219132916520195e+01,     5.4919753676273962e+04,     1.1354006088631852e-04,     6.1909519284169014e-07,    -3.2085551192246420e-09, } , 
      {    2.6318876980092771e+01,     4.6381512785914216e+04,     3.9384479940161187e-04,     8.8050449955301880e-07,    -4.3856827369585953e-09, } , 
      {    2.9574687466906997e+01,     6.3248445100897414e+04,     2.6583246864864471e-04,     8.6981406699634486e-07,    -4.4445391178452932e-09, } , 
      {    5.0757603387550176e+00,    -2.7948408080482768e+04,    -9.7945075195223223e-04,    -1.3753489757247111e-06,     6.6837335223754729e-09, } , 
      {    7.4313148891418201e+00,    -1.9619716655859324e+04,    -8.2715834418990600e-04,    -1.1246301015700564e-06,     5.4477495237548221e-09, } , 
      {    9.7868694395286226e+00,    -1.1291025231235872e+04,    -6.7486593642757987e-04,    -8.7391122741540155e-07,     4.2117655251341712e-09, } , 
      {    1.3042679926342849e+01,     5.5759070837473273e+03,    -8.0287826718054714e-04,    -8.8460165997207550e-07,     4.1529091442474734e-09, } , 
      {    8.8866135031012039e+00,    -1.9829266121595621e+04,    -3.9456119791228642e-04,    -6.1250192070407289e-07,     3.0346379074002178e-09, } , 
      {    1.2142423989915425e+01,    -2.9623338066124188e+03,    -5.2257352866525374e-04,    -6.2319235326074683e-07,     2.9757815265135200e-09, } , 
      {    1.5398234476729652e+01,     1.3904598508370778e+04,    -6.5058585941822091e-04,    -6.3388278581742078e-07,     2.9169251456268221e-09, } , 
      {    1.8654044963543871e+01,     3.0771530823353976e+04,    -7.7859819017118807e-04,    -6.4457321837409473e-07,     2.8580687647401247e-09, } , 
      {    1.1242168053488006e+01,    -1.1500574696972173e+04,    -2.4226879014996032e-04,    -3.6178304654941817e-07,     1.7986539087795672e-09, } , 
      {    1.4497978540302228e+01,     5.3663576180110294e+03,    -3.7028112090292745e-04,    -3.7247347910609211e-07,     1.7397975278928694e-09, } , 
      {    1.7753789027116454e+01,     2.2233289932994227e+04,    -4.9829345165589478e-04,    -3.8316391166276606e-07,     1.6809411470061717e-09, } , 
      {    2.1009599513930674e+01,     3.9100222247977421e+04,    -6.2630578240886194e-04,    -3.9385434421944001e-07,     1.6220847661194741e-09, } , 
      {    1.3597722603874809e+01,    -3.1718832723487221e+03,    -8.9976382387634097e-05,    -1.1106417239476346e-07,     5.6266991015891618e-10, } , 
      {    1.6853533090689030e+01,     1.3695049042634480e+04,    -2.1798871314060133e-04,    -1.2175460495143737e-07,     5.0381352927221842e-10, } , 
      {    2.0109343577503257e+01,     3.0561981357617675e+04,    -3.4600104389356849e-04,    -1.3244503750811132e-07,     4.4495714838552083e-10, } , 
      {    2.3365154064317476e+01,     4.7428913672600873e+04,    -4.7401337464653571e-04,    -1.4313547006478524e-07,     3.8610076749882307e-10, } , 
      {    1.2697466667447388e+01,    -1.1710124162708478e+04,     1.9032835612765926e-04,     1.5034513431656522e-07,    -6.1445770757503701e-10, } , 
      {    1.5953277154261611e+01,     5.1568081522747279e+03,     6.2316025374692084e-05,     1.3965470175989127e-07,    -6.7331408846173465e-10, } , 
      {    1.9209087641075833e+01,     2.2023740467257929e+04,    -6.5696305378275133e-05,     1.2896426920321735e-07,    -7.3217046934843241e-10, } , 
      {    2.2464898127890059e+01,     3.8890672782241127e+04,    -1.9370863613124236e-04,     1.1827383664654339e-07,    -7.9102685023513006e-10, } , 
      {    2.5720708614704279e+01,     5.5757605097224325e+04,    -3.2172096688420953e-04,     1.0758340408986947e-07,    -8.4988323112182781e-10, } , 
      {    1.8308831704648412e+01,     1.3485499576898179e+04,     2.1460843313701825e-04,     3.9037357591454599e-07,    -1.9092980870823855e-09, } , 
      {    2.1564642191462635e+01,     3.0352431891881381e+04,     8.6596102384051048e-05,     3.7968314335787204e-07,    -1.9681544679690833e-09, } , 
      {    2.4820452678276862e+01,     4.7219364206864579e+04,    -4.1416228368916176e-05,     3.6899271080119810e-07,    -2.0270108488557812e-09, } , 
      {    2.8076263165091081e+01,     6.4086296521847777e+04,    -1.6942855912188337e-04,     3.5830227824452415e-07,    -2.0858672297424786e-09, } , 
      {    2.0664386255035218e+01,     2.1814191001521627e+04,     3.6690084089934443e-04,     6.4109245006920071e-07,    -3.1452820857030363e-09, } , 
      {    2.3920196741849438e+01,     3.8681123316504832e+04,     2.3888851014637722e-04,     6.3040201751252676e-07,    -3.2041384665897342e-09, } , 
      {    2.7176007228663664e+01,     5.5548055631488023e+04,     1.1087617939341001e-04,     6.1971158495585282e-07,    -3.2629948474764320e-09, } , 
      {    3.0431817715477884e+01,     7.2414987946471228e+04,    -1.7136151359557185e-05,     6.0902115239917887e-07,    -3.3218512283631295e-09, } , 
      {    2.3019940805422017e+01,     3.0142882426145075e+04,     5.1919324866167061e-04,     8.9181132422385543e-07,    -4.3812660843236867e-09, } , 
      {    2.6275751292236240e+01,     4.7009814741128277e+04,     3.9118091790870340e-04,     8.8112089166718148e-07,    -4.4401224652103846e-09, } , 
      {    2.9531561779050467e+01,     6.3876747056111475e+04,     2.6316858715573623e-04,     8.7043045911050754e-07,    -4.4989788460970824e-09, } , 
      {    2.8631305842623043e+01,     5.5338506165751729e+04,     5.4347332567102958e-04,     1.1318397658218361e-06,    -5.6761064638310354e-09, } , 
      {    3.1887116329437269e+01,     7.2205438480734927e+04,     4.1546099491806236e-04,     1.1211493332651623e-06,    -5.7349628447177333e-09, } , 
      {    2.6770801005116844e+00,    -3.5648797549892159e+04,    -1.1344070412074669e-03,    -1.6254514577652031e-06,     7.8652777927443345e-09, } , 
      {    5.0326346508984869e+00,    -2.7320106125268707e+04,    -9.8211463344514082e-04,    -1.3747325836105484e-06,     6.6292937941236828e-09, } , 
      {    8.2884451377127135e+00,    -1.0453173810285509e+04,    -1.1101269641981080e-03,    -1.3854230161672223e-06,     6.5704374132369850e-09, } , 
      {    7.3881892012852894e+00,    -1.8991414700645262e+04,    -8.2982222568281448e-04,    -1.1240137094558937e-06,     5.3933097955030320e-09, } , 
      {    6.4879332648578707e+00,    -2.7529655591005012e+04,    -5.4951748716752119e-04,    -8.6260440274456493e-07,     4.2161821777690790e-09, } , 
      {    9.7437437516720919e+00,    -1.0662723276021810e+04,    -6.7752981792048835e-04,    -8.7329483530123887e-07,     4.1573257968823811e-09, } , 
      {    1.2999554238486319e+01,     6.2042090389613877e+03,    -8.0554214867345562e-04,    -8.8398526785791282e-07,     4.0984694159956833e-09, } , 
      {    1.6255364725300538e+01,     2.3071141353944586e+04,    -9.3355447942642278e-04,    -8.9467570041458677e-07,     4.0396130351089854e-09, } , 
      {    8.8434878152446732e+00,    -1.9200964166381560e+04,    -3.9722507940519495e-04,    -6.1188552858991021e-07,     2.9801981791484282e-09, } , 
      {    1.2099298302058894e+01,    -2.3340318513983584e+03,    -5.2523741015816222e-04,    -6.2257596114658415e-07,     2.9213417982617303e-09, } , 
      {    1.5355108788873121e+01,     1.4532900463584840e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {    1.8610919275687341e+01,     3.1399832778568038e+04,    -7.8126207166409655e-04,    -6.4395682625993205e-07,     2.8036290364883350e-09, } , 
      {    1.1199042365631476e+01,    -1.0872272741758112e+04,    -2.4493267164286882e-04,    -3.6116665443525549e-07,     1.7442141805277773e-09, } , 
      {    1.4454852852445697e+01,     5.9946595732250898e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.7710663339259924e+01,     2.2861591888208288e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.0966473826074143e+01,     3.9728524203191482e+04,    -6.2896966390177042e-04,    -3.9323795210527733e-07,     1.5676450378676842e-09, } , 
      {    2.4222284312888370e+01,     5.6595456518174688e+04,    -7.5698199465473758e-04,    -4.0392838466195128e-07,     1.5087886569809863e-09, } , 
      {    1.0298786429204055e+01,    -1.9410513632117865e+04,     3.5372066872424576e-05,    -9.9757347723926846e-08,     5.6708656279382406e-10, } , 
      {    1.3554596916018278e+01,    -2.5435813171346617e+03,    -9.2640263880542614e-05,    -1.1044778028060079e-07,     5.0823018190712641e-10, } , 
      {    1.6810407402832499e+01,     1.4323350997848540e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.0066217889646726e+01,     3.1190283312831736e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    2.3322028376460946e+01,     4.8057215627814934e+04,    -4.7667725613944424e-04,    -1.4251907795062258e-07,     3.3166103924703326e-10, } , 
      {    2.6577838863275172e+01,     6.4924147942798139e+04,    -6.0468958689241146e-04,    -1.5320951050729653e-07,     2.7280465836033551e-10, } , 
      {    1.2654340979590858e+01,    -1.1081822207494417e+04,     1.8766447463475076e-04,     1.5096152643072787e-07,    -6.6889743582682677e-10, } , 
      {    1.5910151466405081e+01,     5.7851101074887883e+03,     5.9652143881783567e-05,     1.4027109387405393e-07,    -7.2775381671352442e-10, } , 
      {    1.9165961953219302e+01,     2.2652042422471990e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.2421772440033529e+01,     3.9518974737455188e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    2.5677582926847748e+01,     5.6385907052438386e+04,    -3.2438484837711806e-04,     1.0819979620403214e-07,    -9.0432295937361757e-10, } , 
      {    1.8265706016791881e+01,     1.4113801532112238e+04,     2.1194455164410975e-04,     3.9098996802870867e-07,    -1.9637378153341751e-09, } , 
      {    2.1521516503606104e+01,     3.0980733847095442e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.4777326990420331e+01,     4.7847666162078640e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {    2.8033137477234551e+01,     6.4714598477061838e+04,    -1.7209244061479188e-04,     3.5891867035868683e-07,    -2.1403069579942683e-09, } , 
      {    2.0621260567178687e+01,     2.2442492956735688e+04,     3.6423695940643590e-04,     6.4170884218336339e-07,    -3.1997218139548260e-09, } , 
      {    2.3877071053992907e+01,     3.9309425271718894e+04,     2.3622462865346871e-04,     6.3101840962668944e-07,    -3.2585781948415238e-09, } , 
      {    2.7132881540807134e+01,     5.6176357586702085e+04,     1.0821229790050149e-04,     6.2032797707001550e-07,    -3.3174345757282217e-09, } , 
      {    3.0388692027621353e+01,     7.3043289901685290e+04,    -1.9800032852465696e-05,     6.0963754451334155e-07,    -3.3762909566149191e-09, } , 
      {    2.2976815117565486e+01,     3.0771184381359137e+04,     5.1652936716876214e-04,     8.9242771633801811e-07,    -4.4357058125754768e-09, } , 
      {    2.6232625604379709e+01,     4.7638116696342338e+04,     3.8851703641579487e-04,     8.8173728378134416e-07,    -4.4945621934621747e-09, } , 
      {    2.9488436091193936e+01,     6.4505049011325536e+04,     2.6050470566282770e-04,     8.7104685122467022e-07,    -4.5534185743488725e-09, } , 
      {    3.2744246578008159e+01,     8.1371981326308742e+04,     1.3249237490986049e-04,     8.6035641866799627e-07,    -4.6122749552355704e-09, } , 
      {    2.8588180154766512e+01,     5.5966808120965790e+04,     5.4080944417812110e-04,     1.1324561579359988e-06,    -5.7305461920828255e-09, } , 
      {    3.1843990641580739e+01,     7.2833740435948988e+04,     4.1279711342515383e-04,     1.1217657253793249e-06,    -5.7894025729695233e-09, } , 
      {    3.0943734705153314e+01,     6.4295499545589242e+04,     6.9310185194044723e-04,     1.3831750320906535e-06,    -6.9665301907034763e-09, } , 
      {    3.4199545191967545e+01,     8.1162431860572440e+04,     5.6508952118747996e-04,     1.3724845995339797e-06,    -7.0253865715901742e-09, } , 
      {    7.3450635134287587e+00,    -1.8363112745431201e+04,    -8.3248610717572295e-04,    -1.1233973173417310e-06,     5.3388700672512419e-09, } , 
      {    9.7006180638155612e+00,    -1.0034421320807749e+04,    -6.8019369941339683e-04,    -8.7267844318707619e-07,     4.1028860686305911e-09, } , 
      {    1.2956428550629788e+01,     6.8325109941754481e+03,    -8.0820603016636410e-04,    -8.8336887574375014e-07,     4.0440296877438932e-09, } , 
      {    1.2056172614202364e+01,    -1.7057298961842980e+03,    -5.2790129165107070e-04,    -6.2195956903242147e-07,     2.8669020700099407e-09, } , 
      {    1.5311983101016590e+01,     1.5161202418798901e+04,    -6.5591362240403786e-04,    -6.3265000158909542e-07,     2.8080456891232428e-09, } , 
      {    1.1155916677774945e+01,    -1.0243970786544050e+04,    -2.4759655313577735e-04,    -3.6055026232109281e-07,     1.6897744522759875e-09, } , 
      {    1.4411727164589166e+01,     6.6229615284391502e+03,    -3.7560888388874452e-04,    -3.7124069487776675e-07,     1.6309180713892896e-09, } , 
      {    1.7667537651403393e+01,     2.3489893843422349e+04,    -5.0362121464171173e-04,    -3.8193112743444070e-07,     1.5720616905025920e-09, } , 
      {    2.0923348138217612e+01,     4.0356826158405544e+04,    -6.3163354539467890e-04,    -3.9262155999111465e-07,     1.5132053096158943e-09, } , 
      {    1.3511471228161748e+01,    -1.9152793619206013e+03,    -9.5304145373451132e-05,    -1.0983138816643813e-07,     4.5379045365533659e-10, } , 
      {    1.6767281714975969e+01,     1.4951652953062599e+04,    -2.2331647612641834e-04,    -1.2052182072311206e-07,     3.9493407276863884e-10, } , 
      {    2.0023092201790195e+01,     3.1818585268045797e+04,    -3.5132880687938555e-04,    -1.3121225327978601e-07,     3.3607769188194119e-10, } , 
      {    2.3278902688604415e+01,     4.8685517583028995e+04,    -4.7934113763235277e-04,    -1.4190268583645993e-07,     2.7722131099524344e-10, } , 
      {    1.5867025778548550e+01,     6.4134120627028487e+03,     5.6988262388875056e-05,     1.4088748598821658e-07,    -7.8219354496531418e-10, } , 
      {    1.9122836265362771e+01,     2.3280344377686051e+04,    -7.1024068364092168e-05,     1.3019705343154266e-07,    -8.4104992585201194e-10, } , 
      {    2.2378646752176998e+01,     4.0147276692669249e+04,    -1.9903639911705937e-04,     1.1950662087486871e-07,    -8.9990630673870959e-10, } , 
      {    2.5634457238991217e+01,     5.7014209007652447e+04,    -3.2704872987002659e-04,     1.0881618831819480e-07,    -9.5876268762540734e-10, } , 
      {    1.8222580328935351e+01,     1.4742103487326298e+04,     2.0928067015120124e-04,     3.9160636014287135e-07,    -2.0181775435859648e-09, } , 
      {    2.1478390815749574e+01,     3.1609035802309503e+04,     8.1268339398234013e-05,     3.8091592758619740e-07,    -2.0770339244726627e-09, } , 
      {    2.4734201302563800e+01,     4.8475968117292701e+04,    -4.6743991354733197e-05,     3.7022549502952346e-07,    -2.1358903053593605e-09, } , 
      {    2.7990011789378020e+01,     6.5342900432275899e+04,    -1.7475632210770038e-04,     3.5953506247284951e-07,    -2.1947466862460580e-09, } , 
      {    2.0578134879322157e+01,     2.3070794911949750e+04,     3.6157307791352737e-04,     6.4232523429752607e-07,    -3.2541615422066156e-09, } , 
      {    2.3833945366136376e+01,     3.9937727226932955e+04,     2.3356074716056021e-04,     6.3163480174085212e-07,    -3.3130179230933135e-09, } , 
      {    2.7089755852950603e+01,     5.6804659541916146e+04,     1.0554841640759298e-04,     6.2094436918417818e-07,    -3.3718743039800113e-09, } , 
      {    2.6189499916523179e+01,     4.8266418651556400e+04,     3.8585315492288633e-04,     8.8235367589550684e-07,    -4.5490019217139647e-09, } , 
      {    2.9445310403337405e+01,     6.5133350966539598e+04,     2.5784082416991917e-04,     8.7166324333883290e-07,    -4.6078583026006626e-09, } , 
      {    2.8545054466909981e+01,     5.6595110076179852e+04,     5.3814556268521262e-04,     1.1330725500501615e-06,    -5.7849859203346156e-09, } , 
      {    3.1800864953724208e+01,     7.3462042391163050e+04,     4.1013323193224530e-04,     1.1223821174934876e-06,    -5.8438423012213134e-09, } , 
      {    1.2013046926345833e+01,    -1.0774279409702376e+03,    -5.3056517314397928e-04,    -6.2134317691825879e-07,     2.8124623417581506e-09, } , 
      {    1.5268857413160060e+01,     1.5789504374012960e+04,    -6.5857750389694645e-04,    -6.3203360947493274e-07,     2.7536059608714527e-09, } , 
      {    1.1112790989918414e+01,    -9.6156688313299910e+03,    -2.5026043462868583e-04,    -3.5993387020693018e-07,     1.6353347240241978e-09, } , 
      {    1.4368601476732636e+01,     7.2512634836532106e+03,    -3.7827276538165299e-04,    -3.7062430276360413e-07,     1.5764783431375000e-09, } , 
      {    1.7624411963546862e+01,     2.4118195798636407e+04,    -5.0628509613462032e-04,    -3.8131473532027807e-07,     1.5176219622508023e-09, } , 
      {    1.3468345540305217e+01,    -1.2869774067065409e+03,    -9.7968026866359636e-05,    -1.0921499605227546e-07,     3.9935072540354683e-10, } , 
      {    1.6724156027119438e+01,     1.5579954908276661e+04,    -2.2598035761932687e-04,    -1.1990542860894938e-07,     3.4049434451684908e-10, } , 
      {    1.9979966513933665e+01,     3.2446887223259859e+04,    -3.5399268837229403e-04,    -1.3059586116562333e-07,     2.8163796363015143e-10, } , 
      {    1.9079710577506241e+01,     2.3908646332900113e+04,    -7.3687949857000672e-05,     1.3081344554570534e-07,    -8.9548965410380180e-10, } , 
      {    2.2335521064320467e+01,     4.0775578647883311e+04,    -2.0170028060996790e-04,     1.2012301298903139e-07,    -9.5434603499049935e-10, } , 
      {    2.1435265127893043e+01,     3.2237337757523565e+04,     7.8604457905325509e-05,     3.8153231970036003e-07,    -2.1314736527244527e-09, } , 
      {    2.4691075614707270e+01,     4.9104270072506763e+04,    -4.9407872847641708e-05,     3.7084188714368608e-07,    -2.1903300336111506e-09, } , 
      {    1.7581286275690331e+01,     2.4746497753850468e+04,    -5.0894897762752880e-04,    -3.8069834320611539e-07,     1.4631822339990124e-09, } , 
      {    1.6681030339262907e+01,     1.6208256863490722e+04,    -2.2864423911223537e-04,    -1.1928903649478673e-07,     2.8605461626505926e-10, } , 
      {    1.9936840826077134e+01,     3.3075189178473920e+04,    -3.5665656986520256e-04,    -1.2997946905146068e-07,     2.2719823537836161e-10, } , 
      {    1.9036584889649710e+01,     2.4536948288114170e+04,    -7.6351831349909190e-05,     1.3142983765986799e-07,    -9.4992938235559157e-10, } , 
      {    1.9739571027315364e+01,     1.2509432807787865e+04,    -4.0070882747193581e-04,    -3.4203100996049397e-07,     1.5902276591337665e-09, } , 
      {    2.2995381514129591e+01,     2.9376365122771072e+04,    -5.2872115822490302e-04,    -3.5272144251716792e-07,     1.5313712782470686e-09, } , 
      {    2.2095125577702166e+01,     2.0838124232411312e+04,    -2.4841641970960968e-04,    -9.1312135805839277e-08,     3.5424366051311567e-10, } , 
      {    2.5350936064516393e+01,     3.7705056547394517e+04,    -3.7642875046257684e-04,    -1.0200256836251323e-07,     2.9538727962641782e-10, } , 
      {    2.4450680128088969e+01,     2.9166815657034764e+04,    -9.6124011947283457e-05,     1.5940673834881544e-07,    -8.8174033810753511e-10, } , 
      {    1.7340890789072031e+01,     4.8090433383784775e+03,    -5.5566511672717057e-04,    -5.9213349200098612e-07,     2.7717719295026274e-09, } , 
      {    2.0596701275886257e+01,     2.1675975653361682e+04,    -6.8367744748013774e-04,    -6.0282392455765996e-07,     2.7129155486159296e-09, } , 
      {    1.9696445339458833e+01,     1.3137734763001925e+04,    -4.0337270896484428e-04,    -3.4141461784633134e-07,     1.5357879308819768e-09, } , 
      {    2.2952255826273060e+01,     3.0004667077985130e+04,    -5.3138503971781161e-04,    -3.5210505040300529e-07,     1.4769315499952789e-09, } , 
      {    1.8796189403031416e+01,     4.5994938726421778e+03,    -1.2306797044955094e-04,    -8.0005311135002689e-08,     3.5866031314802356e-10, } , 
      {    2.2051999889845636e+01,     2.1466426187625373e+04,    -2.5108030120251816e-04,    -9.0695743691676611e-08,     2.9980393226132591e-10, } , 
      {    2.5307810376659862e+01,     3.8333358502608578e+04,    -3.7909263195548532e-04,    -1.0138617624835056e-07,     2.4094755137462805e-10, } , 
      {    2.1151743953418219e+01,     1.2928185297265627e+04,     2.9224437312775249e-05,     1.7071356301965204e-07,    -8.7732368547262733e-10, } , 
      {    2.4407554440232438e+01,     2.9795117612248825e+04,    -9.8787893440191961e-05,     1.6002313046297812e-07,    -9.3618006635932487e-10, } , 
      {    2.6763108990619241e+01,     3.8123809036872277e+04,     5.3504514322134213e-05,     4.1074200461763281e-07,    -2.1721640649799759e-09, } , 
      {    1.8198021037642924e+01,     1.3975586183952291e+04,    -8.3863373673537245e-04,    -8.5292640659815200e-07,     3.8944598189847903e-09, } , 
      {    1.7297765101215500e+01,     5.4373452935925379e+03,    -5.5832899822007905e-04,    -5.9151709988682344e-07,     2.7173322012508378e-09, } , 
      {    2.0553575588029727e+01,     2.2304277608575743e+04,    -6.8634132897304621e-04,    -6.0220753244349728e-07,     2.6584758203641399e-09, } , 
      {    1.9653319651602303e+01,     1.3766036718215986e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    2.2909130138416529e+01,     3.0632969033199191e+04,    -5.3404892121072009e-04,    -3.5148865828884261e-07,     1.4224918217434891e-09, } , 
      {    2.6164940625230749e+01,     4.7499901348182393e+04,    -6.6206125196368725e-04,    -3.6217909084551651e-07,     1.3636354408567914e-09, } , 
      {    1.8753063715174886e+01,     5.2277958278562382e+03,    -1.2573185194245944e-04,    -7.9388919020840022e-08,     3.0422058489623374e-10, } , 
      {    2.2008874201989105e+01,     2.2094728142839434e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {    2.5264684688803332e+01,     3.8961660457822640e+04,    -3.8175651344839385e-04,    -1.0076978413418789e-07,     1.8650782312283823e-10, } , 
      {    2.8520495175617551e+01,     5.5828592772805838e+04,    -5.0976884420136101e-04,    -1.1146021669086181e-07,     1.2765144223614059e-10, } , 
      {    2.1108618265561688e+01,     1.3556487252479688e+04,     2.6560555819866739e-05,     1.7132995513381470e-07,    -9.3176341372441709e-10, } , 
      {    2.4364428752375908e+01,     3.0423419567462886e+04,    -1.0145177493310048e-04,     1.6063952257714078e-07,    -9.9061979461111474e-10, } , 
      {    2.7620239239190134e+01,     4.7290351882446092e+04,    -2.2946410568606770e-04,     1.4994909002046683e-07,    -1.0494761754978126e-09, } , 
      {    2.3464172815948491e+01,     2.1885178677103140e+04,     1.7885296358219292e-04,     4.2204882928846939e-07,    -2.1677474123450681e-09, } , 
      {    2.6719983302762710e+01,     3.8752110992086338e+04,     5.0840632829225702e-05,     4.1135839673179549e-07,    -2.2266037932317656e-09, } , 
      {    2.9975793789576937e+01,     5.5619043307069544e+04,    -7.7171697923741515e-05,     4.0066796417512155e-07,    -2.2854601741184634e-09, } , 
      {    2.9075537853149513e+01,     4.7080802416709790e+04,     2.0313304059155188e-04,     6.6207727088645016e-07,    -3.4625877918524164e-09, } , 
      {    3.2331348339963739e+01,     6.3947734731692995e+04,     7.5120709838584666e-05,     6.5138683832977632e-07,    -3.5214441727391143e-09, } , 
      {    1.7254639413358969e+01,     6.0656472488065983e+03,    -5.6099287971298753e-04,    -5.9090070777266076e-07,     2.6628924729990481e-09, } , 
      {    2.0510449900173196e+01,     2.2932579563789805e+04,    -6.8900521046595469e-04,    -6.0159114032933460e-07,     2.6040360921123502e-09, } , 
      {    1.9610193963745772e+01,     1.4394338673430047e+04,    -4.0870047195066135e-04,    -3.4018183361800598e-07,     1.4269084743783971e-09, } , 
      {    2.2866004450559998e+01,     3.1261270988413253e+04,    -5.3671280270362856e-04,    -3.5087226617467993e-07,     1.3680520934916992e-09, } , 
      {    2.1965748514132574e+01,     2.2723030098053496e+04,    -2.5640806418833522e-04,    -8.9462959463351277e-08,     1.9092447575774630e-10, } , 
      {    2.5221559000946801e+01,     3.9589962413036701e+04,    -3.8442039494130238e-04,    -1.0015339202002522e-07,     1.3206809487104844e-10, } , 
      {    2.1065492577705157e+01,     1.4184789207693750e+04,     2.3896674326958228e-05,     1.7194634724797735e-07,    -9.8620314197620696e-10, } , 
      {    2.4321303064519377e+01,     3.1051721522676948e+04,    -1.0411565642600900e-04,     1.6125591469130343e-07,    -1.0450595228629046e-09, } , 
      {    2.7577113551333603e+01,     4.7918653837660153e+04,    -2.3212798717897620e-04,     1.5056548213462948e-07,    -1.1039159037496025e-09, } , 
      {    2.6676857614906179e+01,     3.9380412947300400e+04,     4.8176751336317192e-05,     4.1197478884595817e-07,    -2.2810435214835552e-09, } , 
      {    2.9932668101720406e+01,     5.6247345262283605e+04,    -7.9835579416650033e-05,     4.0128435628928423e-07,    -2.3398999023702531e-09, } , 
      {    2.9032412165292982e+01,     4.7709104371923851e+04,     2.0046915909864338e-04,     6.6269366300061284e-07,    -3.5170275201042061e-09, } , 
      {    2.1922622826276044e+01,     2.3351332053267557e+04,    -2.5907194568124370e-04,    -8.8846567349188610e-08,     1.3648474750595651e-10, } , 
      {    2.4278177376662846e+01,     3.1680023477891009e+04,    -1.0677953791891750e-04,     1.6187230680546611e-07,    -1.0994992511146943e-09, } , 
      {    2.5881419450755917e+01,     2.8190748887924459e+04,    -7.1144127255623750e-04,    -5.7299784752622460e-07,     2.6177854081086167e-09, } , 
      {    2.4981163514328500e+01,     1.9652507997564702e+04,    -4.3113653404094410e-04,    -3.1158854081489588e-07,     1.4406577903746635e-09, } , 
      {    2.8236974001142720e+01,     3.6519440312547900e+04,    -5.5914886479391137e-04,    -3.2227897337156983e-07,     1.3818014094879657e-09, } , 
      {    2.7336718064715303e+01,     2.7981199422188150e+04,    -2.7884412627861792e-04,    -6.0869666660241186e-08,     2.0467379175401271e-10, } , 
      {    3.0592528551529522e+01,     4.4848131737171352e+04,    -4.0685645703158514e-04,    -7.1560099216915134e-08,     1.4581741086731486e-10, } , 
      {    2.9692272615102105e+01,     3.6309890846811599e+04,    -1.2655171851629174e-04,     1.8984920749441353e-07,    -1.0313102068666381e-09, } , 
      {    3.2948083101916325e+01,     5.3176823161794804e+04,    -2.5456404926925896e-04,     1.7915877493773959e-07,    -1.0901665877533360e-09, } , 
      {    3.2047827165488904e+01,     4.4638582271435051e+04,     2.5740689246034437e-05,     4.4056808164906828e-07,    -2.2672942054872887e-09, } , 
      {    2.2582483276085167e+01,     1.1952118528155313e+04,    -5.8609282329617882e-04,    -5.6169102285538797e-07,     2.6222020607435249e-09, } , 
      {    2.5838293762899387e+01,     2.8819050843138517e+04,    -7.1410515404914598e-04,    -5.7238145541206192e-07,     2.5633456798568271e-09, } , 
      {    2.4938037826471970e+01,     2.0280809952778764e+04,    -4.3380041553385263e-04,    -3.1097214870073320e-07,     1.3862180621228737e-09, } , 
      {    2.8193848313286189e+01,     3.7147742267761962e+04,    -5.6181274628681985e-04,    -3.2166258125740715e-07,     1.3273616812361758e-09, } , 
      {    2.4037781890044545e+01,     1.1742569062419014e+04,    -1.5349567701855921e-04,    -4.9562841989404605e-08,     2.0909044438892055e-10, } , 
      {    2.7293592376858772e+01,     2.8609501377402208e+04,    -2.8150800777152645e-04,    -6.0253274546078533e-08,     1.5023406350222290e-10, } , 
      {    3.0549402863672992e+01,     4.5476433692385413e+04,    -4.0952033852449367e-04,    -7.0943707102752481e-08,     9.1377682615525054e-11, } , 
      {    2.9649146927245575e+01,     3.6938192802025660e+04,    -1.2921560000920024e-04,     1.9046559960857619e-07,    -1.0857499351184280e-09, } , 
      {    3.2904957414059794e+01,     5.3805125117008865e+04,    -2.5722793076216749e-04,     1.7977516705190224e-07,    -1.1446063160051258e-09, } , 
      {    3.2004701477632373e+01,     4.5266884226649112e+04,     2.3076807753125923e-05,     4.4118447376323096e-07,    -2.3217339337390784e-09, } , 
      {    3.5260511964446593e+01,     6.2133816541632317e+04,    -1.0493552299984129e-04,     4.3049404120655701e-07,    -2.3805903146257763e-09, } , 
      {    3.4360256028019180e+01,     5.3595575651272564e+04,     1.7536921551545212e-04,     6.9190334791788563e-07,    -3.5577179323597292e-09, } , 
      {    3.7616066514833399e+01,     7.0462507966255769e+04,     4.7356884762484887e-05,     6.8121291536121168e-07,    -3.6165743132464271e-09, } , 
      {    2.0183803037841834e+01,     4.2517290587459220e+03,    -7.4104911255141353e-04,    -8.1179350489588001e-07,     3.8037463311123861e-09, } , 
      {    2.3439613524656053e+01,     2.1118661373729126e+04,    -8.6906144330438069e-04,    -8.2248393745255396e-07,     3.7448899502256882e-09, } , 
      {    2.2539357588228636e+01,     1.2580420483369373e+04,    -5.8875670478908740e-04,    -5.6107463074122529e-07,     2.5677623324917348e-09, } , 
      {    2.5795168075042856e+01,     2.9447352798352578e+04,    -7.1676903554205456e-04,    -5.7176506329789924e-07,     2.5089059516050370e-09, } , 
      {    2.9050978561857082e+01,     4.6314285113335769e+04,    -8.4478136629502173e-04,    -5.8245549585457319e-07,     2.4500495707183391e-09, } , 
      {    2.4894912138615439e+01,     2.0909111907992821e+04,    -4.3646429702676111e-04,    -3.1035575658657057e-07,     1.3317783338710840e-09, } , 
      {    2.8150722625429658e+01,     3.7776044222976023e+04,    -5.6447662777972844e-04,    -3.2104618914324452e-07,     1.2729219529843862e-09, } , 
      {    3.1406533112243885e+01,     5.4642976537959214e+04,    -6.9248895853269560e-04,    -3.3173662169991847e-07,     1.2140655720976885e-09, } , 
      {    2.3994656202188015e+01,     1.2370871017633075e+04,    -1.5615955851146774e-04,    -4.8946449875241945e-08,     1.5465071613713075e-10, } , 
      {    2.7250466689002241e+01,     2.9237803332616269e+04,    -2.8417188926443493e-04,    -5.9636882431915866e-08,     9.5794335250433106e-11, } , 
      {    3.0506277175816461e+01,     4.6104735647599475e+04,    -4.1218422001740215e-04,    -7.0327314988589814e-08,     3.6937954363735257e-11, } , 
      {    3.3762087662630691e+01,     6.2971667962582665e+04,    -5.4019655077036947e-04,    -8.1017747545263735e-08,    -2.1918426522962392e-11, } , 
      {    2.6350210752574817e+01,     2.0699562442256523e+04,    -3.8671507491415510e-06,     2.0177242427941279e-07,    -1.0813332824835200e-09, } , 
      {    2.9606021239389044e+01,     3.7566494757239721e+04,    -1.3187948150210878e-04,     1.9108199172273887e-07,    -1.1401896633702176e-09, } , 
      {    3.2861831726203263e+01,     5.4433427072222927e+04,    -2.5989181225507597e-04,     1.8039155916606492e-07,    -1.1990460442569155e-09, } , 
      {    3.6117642213017490e+01,     7.1300359387206117e+04,    -3.8790414300804318e-04,     1.6970112660939100e-07,    -1.2579024251436132e-09, } , 
      {    3.1961575789775846e+01,     4.5895186181863173e+04,     2.0412926260217412e-05,     4.4180086587739359e-07,    -2.3761736619908685e-09, } , 
      {    3.5217386276590062e+01,     6.2762118496846379e+04,    -1.0759940449274980e-04,     4.3111043332071964e-07,    -2.4350300428775663e-09, } , 
      {    3.4317130340162649e+01,     5.4223877606486625e+04,     1.7270533402254359e-04,     6.9251974003204831e-07,    -3.6121576606115193e-09, } , 
      {    3.7572940826976868e+01,     7.1090809921469830e+04,     4.4693003269576376e-05,     6.8182930747537436e-07,    -3.6710140414982172e-09, } , 
      {    3.6672684890549448e+01,     6.2552569031110070e+04,     3.2499774178486979e-04,     9.4323861418670302e-07,    -4.8481416592321697e-09, } , 
      {    3.9928495377363674e+01,     7.9419501346093282e+04,     1.9698541103190255e-04,     9.3254818163002908e-07,    -4.9069980401188676e-09, } , 
      {    2.0140677349985303e+01,     4.8800310139599824e+03,    -7.4371299404432200e-04,    -8.1117711278171733e-07,     3.7493066028605960e-09, } , 
      {    2.3396487836799523e+01,     2.1746963328943188e+04,    -8.7172532479728917e-04,    -8.2186754533839128e-07,     3.6904502219738982e-09, } , 
      {    2.6652298323613749e+01,     3.8613895643926378e+04,    -9.9973765555025655e-04,    -8.3255797789506523e-07,     3.6315938410872003e-09, } , 
      {    2.2496231900372106e+01,     1.3208722438583434e+04,    -5.9142058628199588e-04,    -5.6045823862706261e-07,     2.5133226042399452e-09, } , 
      {    2.5752042387186325e+01,     3.0075654753566640e+04,    -7.1943291703496304e-04,    -5.7114867118373656e-07,     2.4544662233532473e-09, } , 
      {    2.9007852874000552e+01,     4.6942587068549830e+04,    -8.4744524778793020e-04,    -5.8183910374041051e-07,     2.3956098424665495e-09, } , 
      {    2.1595975963944682e+01,     4.6704815482236845e+03,    -3.1111584776670243e-04,    -2.9904893191573400e-07,     1.3361949865059918e-09, } , 
      {    2.4851786450758908e+01,     2.1537413863206883e+04,    -4.3912817851966964e-04,    -3.0973936447240789e-07,     1.2773386056192941e-09, } , 
      {    2.8107596937573128e+01,     3.8404346178190084e+04,    -5.6714050927263691e-04,    -3.2042979702908184e-07,     1.2184822247325963e-09, } , 
      {    3.1363407424387354e+01,     5.5271278493173275e+04,    -6.9515284002560408e-04,    -3.3112022958575579e-07,     1.1596258438458986e-09, } , 
      {    2.3951530514331484e+01,     1.2999172972847135e+04,    -1.5882344000437624e-04,    -4.8330057761079278e-08,     1.0021098788534096e-10, } , 
      {    2.7207341001145711e+01,     2.9866105287830331e+04,    -2.8683577075734346e-04,    -5.9020490317753199e-08,     4.1354606998643315e-11, } , 
      {    3.0463151487959930e+01,     4.6733037602813536e+04,    -4.1484810151031068e-04,    -6.9710922874427147e-08,    -1.7501773888054540e-11, } , 
      {    3.3718961974774160e+01,     6.3599969917796727e+04,    -5.4286043226327795e-04,    -8.0401355431101068e-08,    -7.6358154774752189e-11, } , 
      {    2.6307085064718287e+01,     2.1327864397470585e+04,    -6.5310322420500627e-06,     2.0238881639357544e-07,    -1.1357730107353099e-09, } , 
      {    2.9562895551532513e+01,     3.8194796712453783e+04,    -1.3454336299501728e-04,     1.9169838383690152e-07,    -1.1946293916220075e-09, } , 
      {    3.2818706038346733e+01,     5.5061729027436988e+04,    -2.6255569374798450e-04,     1.8100795128022757e-07,    -1.2534857725087054e-09, } , 
      {    3.6074516525160959e+01,     7.1928661342420179e+04,    -3.9056802450095171e-04,     1.7031751872355365e-07,    -1.3123421533954030e-09, } , 
      {    2.8662639615105089e+01,     2.9656555822094037e+04,     1.4576137552027612e-04,     4.5310769054823016e-07,    -2.3717570093559607e-09, } , 
      {    3.1918450101919316e+01,     4.6523488137077235e+04,     1.7749044767308901e-05,     4.4241725799155627e-07,    -2.4306133902426581e-09, } , 
      {    3.5174260588733532e+01,     6.3390420452060440e+04,    -1.1026328598565832e-04,     4.3172682543488232e-07,    -2.4894697711293560e-09, } , 
      {    3.8430071075547758e+01,     8.0257352767043631e+04,    -2.3827561673862553e-04,     4.2103639287820837e-07,    -2.5483261520160538e-09, } , 
      {    3.1018194165491892e+01,     3.7985247246717481e+04,     2.9805378328260230e-04,     7.0382656470288483e-07,    -3.6077410079766115e-09, } , 
      {    3.4274004652306118e+01,     5.4852179561700686e+04,     1.7004145252963508e-04,     6.9313613214621099e-07,    -3.6665973888633090e-09, } , 
      {    3.7529815139120338e+01,     7.1719111876683892e+04,     4.2029121776667865e-05,     6.8244569958953704e-07,    -3.7254537697500068e-09, } , 
      {    3.6629559202692917e+01,     6.3180870986324131e+04,     3.2233386029196126e-04,     9.4385500630086570e-07,    -4.9025813874839598e-09, } , 
      {    3.9885369689507144e+01,     8.0047803301307344e+04,     1.9432152953899405e-04,     9.3316457374419176e-07,    -4.9614377683706577e-09, } , 
      {    3.8985113753079723e+01,     7.1509562410947576e+04,     4.7462626805428744e-04,     1.1945738804555204e-06,    -6.1385653861046106e-09, } , 
      {    2.3353362148942992e+01,     2.2375265284157249e+04,    -8.7438920629019764e-04,    -8.2125115322422860e-07,     3.6360104937221085e-09, } , 
      {    2.2453106212515575e+01,     1.3837024393797496e+04,    -5.9408446777490435e-04,    -5.5984184651289993e-07,     2.4588828759881555e-09, } , 
      {    2.5708916699329794e+01,     3.0703956708780701e+04,    -7.2209679852787152e-04,    -5.7053227906957388e-07,     2.4000264951014577e-09, } , 
      {    2.4808660762902377e+01,     2.2165715818420944e+04,    -4.4179206001257817e-04,    -3.0912297235824521e-07,     1.2228988773675043e-09, } , 
      {    2.8064471249716597e+01,     3.9032648133404145e+04,    -5.6980439076554539e-04,    -3.1981340491491916e-07,     1.1640424964808064e-09, } , 
      {    2.3908404826474953e+01,     1.3627474928061194e+04,    -1.6148732149728475e-04,    -4.7713665646916611e-08,     4.5771259633551167e-11, } , 
      {    2.7164215313289180e+01,     3.0494407243044392e+04,    -2.8949965225025199e-04,    -5.8404098203590533e-08,    -1.3085121253146482e-11, } , 
      {    3.0420025800103399e+01,     4.7361339558027597e+04,    -4.1751198300321921e-04,    -6.9094530760264480e-08,    -7.1941502139844331e-11, } , 
      {    2.6263959376861756e+01,     2.1956166352684646e+04,    -9.1949137349585735e-06,     2.0300520850773809e-07,    -1.1902127389870997e-09, } , 
      {    2.9519769863675982e+01,     3.8823098667667844e+04,    -1.3720724448792578e-04,     1.9231477595106417e-07,    -1.2490691198737974e-09, } , 
      {    3.2775580350490202e+01,     5.5690030982651049e+04,    -2.6521957524089303e-04,     1.8162434339439023e-07,    -1.3079255007604952e-09, } , 
      {    3.1875324414062785e+01,     4.7151790092291296e+04,     1.5085163274400390e-05,     4.4303365010571895e-07,    -2.4850531184944478e-09, } , 
      {    3.5131134900877001e+01,     6.4018722407274501e+04,    -1.1292716747856683e-04,     4.3234321754904500e-07,    -2.5439094993811457e-09, } , 
      {    3.4230878964449587e+01,     5.5480481516914748e+04,     1.6737757103672658e-04,     6.9375252426037367e-07,    -3.7210371171150986e-09, } , 
      {    3.7486689451263807e+01,     7.2347413831897953e+04,     3.9365240283759354e-05,     6.8306209170369972e-07,    -3.7798934980017969e-09, } , 
      {    2.2409980524659044e+01,     1.4465326349011555e+04,    -5.9674834926781294e-04,    -5.5922545439873725e-07,     2.4044431477363654e-09, } , 
      {    2.5665791011473264e+01,     3.1332258663994762e+04,    -7.2476068002078010e-04,    -5.6991588695541120e-07,     2.3455867668496676e-09, } , 
      {    2.4765535075045847e+01,     2.2794017773635002e+04,    -4.4445594150548665e-04,    -3.0850658024408259e-07,     1.1684591491157146e-09, } , 
      {    2.8021345561860066e+01,     3.9660950088618207e+04,    -5.7246827225845398e-04,    -3.1919701280075653e-07,     1.1096027682290168e-09, } , 
      {    2.7121089625432649e+01,     3.1122709198258453e+04,    -2.9216353374316047e-04,    -5.7787706089427872e-08,    -6.7524849504936279e-11, } , 
      {    3.0376900112246869e+01,     4.7989641513241659e+04,    -4.2017586449612769e-04,    -6.8478138646101814e-08,    -1.2638123039163412e-10, } , 
      {    2.7737824387385235e+01,     2.0351797628360269e+04,    -6.2184829285100412e-04,    -5.3001576948146457e-07,     2.3637527354808419e-09, } , 
      {    3.0993634874199461e+01,     3.7218729943343475e+04,    -7.4986062360397128e-04,    -5.4070620203813852e-07,     2.3048963545941440e-09, } , 
      {    3.0093378937772037e+01,     2.8680489052983718e+04,    -4.6955588508867794e-04,    -2.7929689532680985e-07,     1.1277687368601912e-09, } , 
      {    3.3349189424586264e+01,     4.5547421367966919e+04,    -5.9756821584164516e-04,    -2.8998732788348380e-07,     1.0689123559734934e-09, } , 
      {    3.2448933488158843e+01,     3.7009180477607166e+04,    -3.1726347732635170e-04,    -2.8578021172155118e-08,    -1.0821526176045967e-10, } , 
      {    2.8594954635956128e+01,     2.9518340473934084e+04,    -9.0481691285920600e-04,    -7.9080868407863056e-07,     3.4864406249630052e-09, } , 
      {    2.7694698699528704e+01,     2.0980099583574331e+04,    -6.2451217434391260e-04,    -5.2939937736730189e-07,     2.3093130072290522e-09, } , 
      {    3.0950509186342931e+01,     3.7847031898557536e+04,    -7.5252450509687976e-04,    -5.4008980992397584e-07,     2.2504566263423543e-09, } , 
      {    3.0050253249915507e+01,     2.9308791008197779e+04,    -4.7221976658158647e-04,    -2.7868050321264717e-07,     1.0733290086084014e-09, } , 
      {    3.3306063736729733e+01,     4.6175723323180981e+04,    -6.0023209733455363e-04,    -2.8937093576932112e-07,     1.0144726277217035e-09, } , 
      {    2.9149997313488090e+01,     2.0770550117838029e+04,    -1.9191502806629305e-04,    -1.7271196501318534e-08,    -1.0379860912555181e-10, } , 
      {    3.2405807800302313e+01,     3.7637482432821227e+04,    -3.1992735881926024e-04,    -2.7961629057992455e-08,    -1.6265499001224946e-10, } , 
      {    3.5661618287116539e+01,     5.4504414747804432e+04,    -4.4793968957222745e-04,    -3.8652061614666403e-08,    -2.2151137089894731e-10, } , 
      {    3.4761362350689112e+01,     4.5966173857444679e+04,    -1.6763495105693408e-04,     2.2275724509666226e-07,    -1.3986389886329003e-09, } , 
      {    3.0007127562058976e+01,     2.9937092963411840e+04,    -4.7488364807449500e-04,    -2.7806411109848449e-07,     1.0188892803566115e-09, } , 
      {    3.3262938048873202e+01,     4.6804025278395042e+04,    -6.0289597882746211e-04,    -2.8875454365515844e-07,     9.6003289946991365e-10, } , 
      {    3.2362682112445782e+01,     3.8265784388035288e+04,    -3.2259124031216877e-04,    -2.7345236943829791e-08,    -2.1709471826403925e-10, } , 
      {    3.5618492599260009e+01,     5.5132716703018494e+04,    -4.5060357106513598e-04,    -3.8035669500503736e-08,    -2.7595109915073713e-10, } , 
      {    3.2979416874398368e+01,     2.7494872818137104e+04,    -6.5227599942001236e-04,    -4.9957330033586654e-07,     2.2141828667217390e-09, } , 
      {    3.6235227361212594e+01,     4.4361805133120310e+04,    -7.8028833017297964e-04,    -5.1026373289254038e-07,     2.1553264858350411e-09, } , 
      {    3.5334971424785174e+01,     3.5823564242760549e+04,    -4.9998359165768624e-04,    -2.4885442618121171e-07,     9.7819886810108813e-10, } , 
      {    3.8590781911599400e+01,     5.2690496557743754e+04,    -6.2799592241065340e-04,    -2.5954485873788566e-07,     9.1934248721439037e-10, } , 
      {    3.7690525975171980e+01,     4.4152255667384001e+04,    -3.4769118389536006e-04,     1.8644479734429879e-09,    -2.5778513051956265e-10, } , 
      {    4.0946336461986206e+01,     6.1019187982367206e+04,    -4.7570351464832727e-04,    -8.8259845832309603e-09,    -3.1664151140626051e-10, } , 
      {    4.0046080525558779e+01,     5.2480947092007453e+04,    -1.9539877613303387e-04,     2.5258332212809773e-07,    -1.4937691291402135e-09, } , 
      {    3.0580736636155038e+01,     1.9794483348727714e+04,    -8.0723228867524718e-04,    -7.4967578237635858e-07,     3.3957271370905997e-09, } , 
      {    3.3836547122969264e+01,     3.6661415663710919e+04,    -9.3524461942821435e-04,    -7.6036621493303242e-07,     3.3368707562039019e-09, } , 
      {    3.2936291186541837e+01,     2.8123174773351166e+04,    -6.5493988091292095e-04,    -4.9895690822170386e-07,     2.1597431384699489e-09, } , 
      {    3.6192101673356063e+01,     4.4990107088334371e+04,    -7.8295221166588822e-04,    -5.0964734077837770e-07,     2.1008867575832510e-09, } , 
      {    3.5291845736928643e+01,     3.6451866197974610e+04,    -5.0264747315059482e-04,    -2.4823803406704908e-07,     9.2375913984929836e-10, } , 
      {    3.8547656223742869e+01,     5.3318798512957816e+04,    -6.3065980390356199e-04,    -2.5892846662372303e-07,     8.6490275896260051e-10, } , 
      {    3.7647400287315449e+01,     4.4780557622598062e+04,    -3.5035506538826853e-04,     2.4808400876056518e-09,    -3.1222485877135242e-10, } , 
      {    4.0903210774129676e+01,     6.1647489937581267e+04,    -4.7836739614123575e-04,    -8.2095924690682952e-09,    -3.7108123965805027e-10, } , 
      {    4.0002954837702248e+01,     5.3109249047221514e+04,    -1.9806265762594241e-04,     2.5319971424226036e-07,    -1.5482088573920032e-09, } , 
      {    4.3258765324516474e+01,     6.9976181362204719e+04,    -3.2607498837890957e-04,     2.4250928168558641e-07,    -1.6070652382787011e-09, } , 
      {    4.2358509388089047e+01,     6.1437940471844966e+04,    -4.5770249863616218e-05,     5.0391858839691502e-07,    -2.7841928560126544e-09, } , 
      {    4.5614319874903273e+01,     7.8304872786828171e+04,    -1.7378258061658341e-04,     4.9322815584024118e-07,    -2.8430492368993523e-09, } , 
      {    3.1437866884725931e+01,     2.8961026194301528e+04,    -1.0902009086834490e-03,    -1.0104686969735245e-06,     4.5184150265727630e-09, } , 
      {    3.0537610948298507e+01,     2.0422785303941775e+04,    -8.0989617016815566e-04,    -7.4905939026219590e-07,     3.3412874088388100e-09, } , 
      {    3.3793421435112734e+01,     3.7289717618924980e+04,    -9.3790850092112282e-04,    -7.5974982281886974e-07,     3.2824310279521122e-09, } , 
      {    3.2893165498685306e+01,     2.8751476728565227e+04,    -6.5760376240582943e-04,    -4.9834051610754118e-07,     2.1053034102181592e-09, } , 
      {    3.6148975985499533e+01,     4.5618409043548432e+04,    -7.8561609315879670e-04,    -5.0903094866421502e-07,     2.0464470293314614e-09, } , 
      {    3.1992909562257893e+01,     2.0213235838205477e+04,    -3.7729902389053603e-04,    -2.3693120939621246e-07,     9.2817579248420624e-10, } , 
      {    3.5248720049072112e+01,     3.7080168153188672e+04,    -5.0531135464350330e-04,    -2.4762164195288640e-07,     8.6931941159750860e-10, } , 
      {    3.8504530535886339e+01,     5.3947100468171877e+04,    -6.3332368539647046e-04,    -2.5831207450956035e-07,     8.1046303071081074e-10, } , 
      {    3.4348464112644692e+01,     2.8541927262828925e+04,    -2.2500661612820987e-04,     1.3787664758442264e-08,    -3.0780820613644459e-10, } , 
      {    3.7604274599458918e+01,     4.5408859577812123e+04,    -3.5301894688117706e-04,     3.0972322017683160e-09,    -3.6666458702314223e-10, } , 
      {    4.0860085086273145e+01,     6.2275791892795329e+04,    -4.8103127763414428e-04,    -7.5932003549056318e-09,    -4.2552096790984009e-10, } , 
      {    3.9959829149845717e+01,     5.3737551002435575e+04,    -2.0072653911885091e-04,     2.5381610635642304e-07,    -1.6026485856437931e-09, } , 
      {    4.3215639636659944e+01,     7.0604483317418781e+04,    -3.2873886987181810e-04,     2.4312567379974909e-07,    -1.6615049665304909e-09, } , 
      {    4.2315383700232516e+01,     6.2066242427059027e+04,    -4.8434131356524728e-05,     5.0453498051107770e-07,    -2.8386325842644441e-09, } , 
      {    4.5571194187046743e+01,     7.8933174742042233e+04,    -1.7644646210949192e-04,     4.9384454795440386e-07,    -2.8974889651511420e-09, } , 
      {    4.4670938250619322e+01,     7.0394933851682479e+04,     1.0385827640580145e-04,     7.5525385466573242e-07,    -4.0746165828850949e-09, } , 
      {    3.2850039810828775e+01,     2.9379778683779288e+04,    -6.6026764389873790e-04,    -4.9772412399337850e-07,     2.0508636819663696e-09, } , 
      {    3.6105850297643002e+01,     4.6246710998762494e+04,    -7.8827997465170517e-04,    -5.0841455655005234e-07,     1.9920073010796717e-09, } , 
      {    3.5205594361215582e+01,     3.7708470108402733e+04,    -5.0797523613641178e-04,    -2.4700524983872372e-07,     8.1487968334571883e-10, } , 
      {    3.8461404848029808e+01,     5.4575402423385938e+04,    -6.3598756688937894e-04,    -2.5769568239539767e-07,     7.5602330245902098e-10, } , 
      {    3.7561148911602388e+01,     4.6037161533026185e+04,    -3.5568282837408559e-04,     3.7136243159309803e-09,    -4.2110431527493205e-10, } , 
      {    4.0816959398416614e+01,     6.2904093848009390e+04,    -4.8369515912705281e-04,    -6.9768082407429675e-09,    -4.7996069616162985e-10, } , 
      {    3.9916703461989187e+01,     5.4365852957649637e+04,    -2.0339042061175941e-04,     2.5443249847058572e-07,    -1.6570883138955829e-09, } , 
      {    4.0934544546611718e+01,     3.5965539593923582e+04,    -8.7607934629198932e-04,    -6.8694166774267425e-07,     2.9332682148170249e-09, } , 
      {    4.4190355033425945e+01,     5.2832471908906773e+04,    -1.0040916770449565e-03,    -6.9763210029934830e-07,     2.8744118339303271e-09, } , 
      {    4.3290099096998517e+01,     4.4294231018547034e+04,    -7.2378693852966319e-04,    -4.3622279358801958e-07,     1.6972842161963741e-09, } , 
      {    4.6545909583812744e+01,     6.1161163333530225e+04,    -8.5179926928263035e-04,    -4.4691322614469353e-07,     1.6384278353096762e-09, } , 
      {    4.5645653647385323e+01,     5.2622922443170479e+04,    -5.7149453076733696e-04,    -1.8550391943336486e-07,     4.6130021757572325e-10, } , 
      {    4.8901464134199550e+01,     6.9489854758153670e+04,    -6.9950686152030412e-04,    -1.9619435199003881e-07,     4.0244383668902540e-10, } , 
      {    4.8001208197772129e+01,     6.0951613867793931e+04,    -4.1920212300501077e-04,     6.5214954721289858e-08,    -7.7468378104492758e-10, } , 
      {    1.5707963267948966e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8266068136091196e+00,     1.6866932314983202e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    8.0824173004233426e+00,     3.3733864629966403e+04,    -2.5602466150593443e-04,    -2.1380865113347869e-08,    -1.1771276177339540e-10, } , 
      {   -2.5852700964467470e+00,    -2.5405173205342951e+04,     4.0831706926826062e-04,     2.7209973926800261e-07,    -1.1182712368472555e-09, } , 
      {    6.7054039036747604e-01,    -8.5382408903597516e+03,     2.8030473851529340e-04,     2.6140930671132867e-07,    -1.1771276177339532e-09, } , 
      {    3.9263508771816991e+00,     8.3286914246234501e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1821613639959221e+00,     2.5195623739606650e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -2.2971554605994449e-01,    -1.7076481780719503e+04,     5.6060947703058680e-04,     5.2281861342265733e-07,    -2.3542552354679064e-09, } , 
      {    3.0260949407542785e+00,    -2.0954946573630150e+02,     4.3259714627761958e-04,     5.1212818086598339e-07,    -2.4131116163546038e-09, } , 
      {    6.2819054275685016e+00,     1.6657382849246900e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.5377159143827246e+00,     3.3524315164230102e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    2.1258390043268580e+00,    -8.7477903560960549e+03,     7.1290188479291292e-04,     7.7353748757731205e-07,    -3.5902392340885572e-09, } , 
      {    5.3816494911410810e+00,     8.1191419588871468e+03,     5.8488955403994576e-04,     7.6284705502063811e-07,    -3.6490956149752546e-09, } , 
      {    8.6374599779553041e+00,     2.4986074273870348e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1893270464769527e+01,     4.1853006588853546e+04,     3.2886489253401133e-04,     7.4146618990729021e-07,    -3.7668083767486503e-09, } , 
      {    4.4813935547136605e+00,    -4.1909893147260300e+02,     8.6519429255523916e-04,     1.0242563617319668e-06,    -4.8262232327092076e-09, } , 
      {    7.7372040415278835e+00,     1.6447833383510599e+04,     7.3718196180227189e-04,     1.0135659291752927e-06,    -4.8850796135959055e-09, } , 
      {    1.0993014528342107e+01,     3.3314765698493800e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.4248825015156330e+01,     5.0181698013476998e+04,     4.8115730029633751e-04,     9.9218506406194504e-07,    -5.0027923753693012e-09, } , 
      {    1.0092758591914686e+01,     2.4776524808134051e+04,     8.8947436956459801e-04,     1.2642848033299474e-06,    -6.1210636122165563e-09, } , 
      {    1.3348569078728909e+01,     4.1643457123117252e+04,     7.6146203881163085e-04,     1.2535943707732736e-06,    -6.1799199931032542e-09, } , 
      {    1.5704123629115712e+01,     4.9972148547740697e+04,     9.1375444657395709e-04,     1.5043132449279283e-06,    -7.4159039917239050e-09, } , 
      {    1.8059678179502512e+01,     5.8300839972364149e+04,     1.0660468543362833e-03,     1.7550321190825830e-06,    -8.6518879903445566e-09, } , 
      {   -1.2605656663382449e+01,    -4.9343846592526636e+04,    -9.1641832806686556e-04,    -1.5036968528137656e-06,     7.3614642634721149e-09, } , 
      {   -1.0250102112995647e+01,    -4.1015155167903191e+04,    -7.6412592030453933e-04,    -1.2529779786591109e-06,     6.1254802648514641e-09, } , 
      {   -7.8945475626088442e+00,    -3.2686463743279739e+04,    -6.1183351254221320e-04,    -1.0022591045044562e-06,     4.8894962662308132e-09, } , 
      {   -5.5389930122220417e+00,    -2.4357772318656287e+04,    -4.5954110477988707e-04,    -7.5154023034980148e-07,     3.6535122676101628e-09, } , 
      {   -2.2831825254078186e+00,    -7.4908400036730864e+03,    -5.8755343553285424e-04,    -7.6223066290647543e-07,     3.5946558867234650e-09, } , 
      {   -6.4392489486494622e+00,    -3.2896013209016040e+04,    -1.7923636626459365e-04,    -4.9013092363847281e-07,     2.4763846498762099e-09, } , 
      {   -3.1834384618352392e+00,    -1.6029080894032839e+04,    -3.0724869701756089e-04,    -5.0082135619514676e-07,     2.4175282689895120e-09, } , 
      {    7.2372024978984095e-02,     8.3785142095036190e+02,    -4.3526102777052811e-04,    -5.1151178875182071e-07,     2.3586718881028141e-09, } , 
      {    3.3281825117932069e+00,     1.7704783735933564e+04,    -5.6327335852349527e-04,    -5.2220222130849465e-07,     2.2998155072161167e-09, } , 
      {   -4.0836943982626597e+00,    -2.4567321784392589e+04,    -2.6943958502267475e-05,    -2.3941204948381809e-07,     1.2404006512555586e-09, } , 
      {   -8.2788391144843665e-01,    -7.7003894694093897e+03,    -1.5495628925523469e-04,    -2.5010248204049204e-07,     1.1815442703688610e-09, } , 
      {    2.4279265753657864e+00,     9.1665428455738111e+03,    -2.8296862000820193e-04,    -2.6079291459716599e-07,     1.1226878894821633e-09, } , 
      {    5.6837370621800094e+00,     2.6033475160557013e+04,    -4.1098095076116915e-04,    -2.7148334715383993e-07,     1.0638315085954657e-09, } , 
      {   -1.7281398478758572e+00,    -1.6238630359769140e+04,     1.2534844926005871e-04,     1.1306824670836598e-08,     4.4166526349079032e-12, } , 
      {    1.5276706389383661e+00,     6.2830195521406040e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    4.7834811257525889e+00,     1.7495234270197263e+04,    -1.3067621224587572e-04,    -1.0074040442511271e-08,    -1.1329610913848749e-10, } , 
      {   -2.6283957843032777e+00,    -2.4776871250128890e+04,     4.0565318777535208e-04,     2.7271613138216529e-07,    -1.1727109650990454e-09, } , 
      {    6.2741470251094555e-01,    -7.9099389351456912e+03,     2.7764085702238487e-04,     2.6202569882549135e-07,    -1.2315673459857430e-09, } , 
      {    3.8832251893251684e+00,     8.9569933798375096e+03,     1.4962852626941768e-04,     2.5133526626881740e-07,    -1.2904237268724407e-09, } , 
      {    7.1390356761393914e+00,     2.5823925694820711e+04,     2.1616195516450453e-05,     2.4064483371214345e-07,    -1.3492801077591383e-09, } , 
      {   -2.7284123391647497e-01,    -1.6448179825505442e+04,     5.5794559553767832e-04,     5.2343500553682001e-07,    -2.4086949637196960e-09, } , 
      {    2.9829692528977478e+00,     4.1875248947775890e+02,     4.2993326478471105e-04,     5.1274457298014607e-07,    -2.4675513446063935e-09, } , 
      {    6.2387797397119709e+00,     1.7285684804460961e+04,     3.0192093403174383e-04,     5.0205414042347212e-07,    -2.5264077254930913e-09, } , 
      {    9.4945902265261939e+00,     3.4152617119444163e+04,     1.7390860327877664e-04,     4.9136370786679817e-07,    -2.5852641063797892e-09, } , 
      {    5.3385238032845503e+00,     8.7474439141012081e+03,     5.8222567254703728e-04,     7.6346344713480079e-07,    -3.7035353432270443e-09, } , 
      {    8.5943342900987734e+00,     2.5614376229084410e+04,     4.5421334179407001e-04,     7.5277301457812684e-07,    -3.7623917241137426e-09, } , 
      {    7.6940783536713528e+00,     1.7076135338724660e+04,     7.3451808030936341e-04,     1.0141823212894554e-06,    -4.9395193418476955e-09, } , 
      {    1.0949888840485576e+01,     3.3943067653707862e+04,     6.0650574955639625e-04,     1.0034918887327816e-06,    -4.9983757227343934e-09, } , 
      {    1.3305443390872378e+01,     4.2271759078331314e+04,     7.5879815731872237e-04,     1.2542107628874363e-06,    -6.2343597213550442e-09, } , 
      {    1.5660997941259181e+01,     5.0600450502954758e+04,     9.1109056508104861e-04,     1.5049296370420910e-06,    -7.4703437199756942e-09, } , 
      {   -1.0293227800852177e+01,    -4.0386853212689130e+04,    -7.6678980179744791e-04,    -1.2523615865449482e-06,     6.0710405365996748e-09, } , 
      {   -7.9376732504653749e+00,    -3.2058161788065678e+04,    -6.1449739403512179e-04,    -1.0016427123902935e-06,     4.8350565379790240e-09, } , 
      {   -5.5821187000785724e+00,    -2.3729470363442226e+04,    -4.6220498627279555e-04,    -7.5092383823563880e-07,     3.5990725393583728e-09, } , 
      {   -3.2265641496917699e+00,    -1.5400778938818779e+04,    -3.0991257851046937e-04,    -5.0020496408098408e-07,     2.3630885407377219e-09, } , 
      {    2.9246337122453614e-02,     1.4661533761644223e+03,    -4.3792490926343659e-04,    -5.1089539663765803e-07,     2.3042321598510241e-09, } , 
      {   -4.1268200861191904e+00,    -2.3939019829178527e+04,    -2.9607839995175986e-05,    -2.3879565736965546e-07,     1.1859609230037689e-09, } , 
      {   -8.7100959930496691e-01,    -7.0720875141953293e+03,    -1.5762017074814322e-04,    -2.4948608992632941e-07,     1.1271045421170713e-09, } , 
      {    2.3848008875092561e+00,     9.7948448007878724e+03,    -2.8563250150111041e-04,    -2.6017652248300336e-07,     1.0682481612303737e-09, } , 
      {   -1.7712655357323874e+00,    -1.5610328404555081e+04,     1.2268456776715018e-04,     1.1923216784999263e-08,    -5.0023075616881894e-11, } , 
      {    1.4845449510818356e+00,     1.2566039104281208e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    4.7403554378960582e+00,     1.8123536225411321e+04,    -1.3334009373878425e-04,    -9.4576483283486060e-09,    -1.6773583739027728e-10, } , 
      {    5.8428901465441507e-01,    -7.2816369799316308e+03,     2.7497697552947639e-04,     2.6264209093965397e-07,    -1.2860070742375327e-09, } , 
      {    3.8400995014686381e+00,     9.5852953350515709e+03,     1.4696464477650915e-04,     2.5195165838298003e-07,    -1.3448634551242304e-09, } , 
      {    7.0959099882828607e+00,     2.6452227650034773e+04,     1.8952314023541942e-05,     2.4126122582630608e-07,    -1.4037198360109280e-09, } , 
      {    2.9398435650412176e+00,     1.0470544446918193e+03,     4.2726938329180257e-04,     5.1336096509430875e-07,    -2.5219910728581835e-09, } , 
      {    6.1956540518554402e+00,     1.7913986759675019e+04,     2.9925705253883535e-04,     5.0267053253763480e-07,    -2.5808474537448814e-09, } , 
      {    8.5512086022422427e+00,     2.6242678184298471e+04,     4.5154946030116154e-04,     7.5338940669228952e-07,    -3.8168314523655318e-09, } , 
      {    1.0906763152629045e+01,     3.4571369608921923e+04,     6.0384186806348766e-04,     1.0041082808469442e-06,    -5.0528154509861826e-09, } , 
      {   -5.6252443879351022e+00,    -2.3101168408228168e+04,    -4.6486886776570408e-04,    -7.5030744612147612e-07,     3.5446328111065831e-09, } , 
      {   -3.2696898375482997e+00,    -1.4772476983604720e+04,    -3.1257646000337790e-04,    -4.9958857196682140e-07,     2.3086488124859323e-09, } , 
      {   -1.3879350734076867e-02,     2.0944553313784827e+03,    -4.4058879075634512e-04,    -5.1027900452349535e-07,     2.2497924315992344e-09, } , 
      {   -9.1413528716149717e-01,    -6.4437855589812689e+03,    -1.6028405224105172e-04,    -2.4886969781216673e-07,     1.0726648138652814e-09, } , 
      {    2.3416751996527259e+00,     1.0423146756001934e+04,    -2.8829638299401894e-04,    -2.5956013036884068e-07,     1.0138084329785838e-09, } , 
      {   -1.8143912235889177e+00,    -1.4982026449341021e+04,     1.2002068627424168e-04,     1.2539608899161927e-08,    -1.0446280386867170e-10, } , 
      {    1.4414192632253051e+00,     1.8849058656421812e+03,    -7.9916444787255359e-06,     1.8491763424879922e-09,    -1.6331918475536940e-10, } , 
      {    4.6972297500395284e+00,     1.8751838180625382e+04,    -1.3600397523169276e-04,    -8.8412562141859425e-09,    -2.2217556564206710e-10, } , 
      {    5.4116332679788459e-01,    -6.6533350247175704e+03,     2.7231309403656786e-04,     2.6325848305381665e-07,    -1.3404468024893226e-09, } , 
      {    3.7969738136121078e+00,     1.0213597290265632e+04,     1.4430076328360064e-04,     2.5256805049714271e-07,    -1.3993031833760202e-09, } , 
      {    6.1525283639989103e+00,     1.8542288714889080e+04,     2.9659317104592682e-04,     5.0328692465179748e-07,    -2.6352871819966711e-09, } , 
      {    8.5080829143857137e+00,     2.6870980139512529e+04,     4.4888557880825300e-04,     7.5400579880645220e-07,    -3.8712711806173219e-09, } , 
      {   -3.3128155254048304e+00,    -1.4144175028390659e+04,    -3.1524034149628643e-04,    -4.9897217985265882e-07,     2.2542090842341426e-09, } , 
      {   -9.5726097501802787e-01,    -5.8154836037672085e+03,    -1.6294793373396022e-04,    -2.4825330569800405e-07,     1.0182250856134916e-09, } , 
      {    1.3982935753687746e+00,     2.5132078208562416e+03,    -1.0655525971634047e-05,     2.4655684566506563e-09,    -2.1775891300715919e-10, } , 
      {    3.7538481257555771e+00,     1.0841899245479692e+04,     1.4163688179069214e-04,     2.5318444261130539e-07,    -1.4537429116278101e-09, } , 
      {    6.9417658773776223e+00,     5.2581693241346557e+03,    -2.2436062090282768e-05,     2.8593292803110101e-08,     1.3749315996266440e-11, } , 
      {    4.5430856391342891e+00,    -2.4422201452747340e+03,    -1.7739235134551747e-04,    -2.2150918923738194e-07,     1.1952935863651275e-09, } , 
      {    6.8986401895210916e+00,     5.8864712793487161e+03,    -2.5099943583191279e-05,     2.9209684917272764e-08,    -4.0690412255523351e-11, } , 
      {   -2.1114914949584618e-01,    -1.8471301039307575e+04,    -4.8464104836307831e-04,    -7.2233054543252880e-07,     3.6128218553546397e-09, } , 
      {    2.1444054008909563e+00,    -1.0142609614684125e+04,    -3.3234864060075218e-04,    -4.7161167127787397e-07,     2.3768378567339888e-09, } , 
      {    4.4999599512777593e+00,    -1.8139181900606736e+03,    -1.8005623283842597e-04,    -2.2089279712321928e-07,     1.1408538581133376e-09, } , 
      {    3.5997040148503383e+00,    -1.0352159080420426e+04,     1.0024850567686742e-04,     4.0516509588109365e-08,    -3.6273759620615454e-11, } , 
      {    6.8555145016645618e+00,     6.5147732345627765e+03,    -2.7763825076099793e-05,     2.9826077031435431e-08,    -9.5130140507313155e-11, } , 
      {    1.0111324988478785e+01,     2.3381705549545979e+04,    -1.5577615582906701e-04,     1.9135644474761496e-08,    -1.5398652139401084e-10, } , 
      {    5.9552585652371413e+00,    -2.0234676557969760e+03,     2.5254091343919358e-04,     2.9123538374276408e-07,    -1.2722577582412664e-09, } , 
      {    9.2110690520513643e+00,     1.4843464659186227e+04,     1.2452858268622639e-04,     2.8054495118609018e-07,    -1.3311141391279641e-09, } , 
      {    1.1566623602438167e+01,     2.3172156083809678e+04,     2.7682099044855254e-04,     5.3126382534074480e-07,    -2.5670981377486145e-09, } , 
      {    1.3922178152824969e+01,     3.1500847508433122e+04,     4.2911339821087872e-04,     7.8198269949539952e-07,    -3.8030821363692661e-09, } , 
      {   -2.6098293877391789e+00,    -2.6171690508716965e+04,    -6.3959733761831308e-04,    -9.7243302747302073e-07,     4.7943661257235004e-09, } , 
      {   -2.5427483735237666e-01,    -1.7842999084093513e+04,    -4.8730492985598679e-04,    -7.2171415331836612e-07,     3.5583821271028496e-09, } , 
      {   -1.1545307737797970e+00,    -2.6381239974453267e+04,    -2.0700019134069347e-04,    -4.6030484660703745e-07,     2.3812545093688962e-09, } , 
      {    2.1012797130344261e+00,    -9.5143076594700651e+03,    -3.3501252209366066e-04,    -4.7099527916371135e-07,     2.3223981284821988e-09, } , 
      {    5.3570901998486491e+00,     7.3526246555131374e+03,    -4.6302485284662788e-04,    -4.8168571172038535e-07,     2.2635417475955009e-09, } , 
      {    1.2010237766070053e+00,    -1.8052548549829815e+04,    -5.4707783578367268e-05,    -2.0958597245238268e-07,     1.1452705107482456e-09, } , 
      {    4.4568342634212286e+00,    -1.1856162348466132e+03,    -1.8272011433133451e-04,    -2.2027640500905660e-07,     1.0864141298615479e-09, } , 
      {    7.7126447502354516e+00,     1.5681316080136588e+04,    -3.1073244508430170e-04,    -2.3096683756573055e-07,     1.0275577489748503e-09, } , 
      {    3.5565783269938080e+00,    -9.7238571252063666e+03,     9.7584624183958920e-05,     4.1132901702272025e-08,    -9.0713487872405258e-11, } , 
      {    6.8123888138080311e+00,     7.1430751897768369e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    1.0068199300622254e+01,     2.4010007504760037e+04,    -1.5844003732197554e-04,     1.9752036588924160e-08,    -2.0842624964580063e-10, } , 
      {    5.9121328773806106e+00,    -1.3951657005829156e+03,     2.4987703194628510e-04,     2.9185177585692671e-07,    -1.3266974864930561e-09, } , 
      {    9.1679433641948336e+00,     1.5471766614400287e+04,     1.2186470119331788e-04,     2.8116134330025281e-07,    -1.3855538673797537e-09, } , 
      {    1.2423753851009057e+01,     3.2338698929383485e+04,    -6.1476295596493528e-06,     2.7047091074357886e-07,    -1.4444102482664514e-09, } , 
      {    8.2676874277674131e+00,     6.9335257240405344e+03,     4.0216943970861128e-04,     5.4257065001158142e-07,    -2.5626814851137071e-09, } , 
      {    1.1523497914581636e+01,     2.3800458039023735e+04,     2.7415710895564407e-04,     5.3188021745490748e-07,    -2.6215378660004046e-09, } , 
      {    1.3879052464968439e+01,     3.2129149463647183e+04,     4.2644951671797025e-04,     7.8259909160956220e-07,    -3.8575218646210554e-09, } , 
      {    1.6234607015355241e+01,     4.0457840888270635e+04,     5.7874192448029637e-04,     1.0333179657642170e-06,    -5.0935058632417062e-09, } , 
      {   -7.3640641763693147e+00,    -4.2200771402749800e+04,    -9.4684603463587392e-04,    -1.4732543836681675e-06,     7.2118943947130120e-09, } , 
      {   -5.0085096259825121e+00,    -3.3872079978126356e+04,    -7.9455362687354768e-04,    -1.2225355095135128e-06,     5.9759103960923612e-09, } , 
      {   -2.6529550755957096e+00,    -2.5543388553502904e+04,    -6.4226121911122155e-04,    -9.7181663535885805e-07,     4.7399263974717103e-09, } , 
      {   -3.5532110120231302e+00,    -3.4081629443862650e+04,    -3.6195648059592815e-04,    -7.1040732864752949e-07,     3.5627987797377574e-09, } , 
      {   -2.9740052520890714e-01,    -1.7214697128879452e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.9584099616053159e+00,    -3.4776481389625042e+02,    -6.1798114210186259e-04,    -7.3178819376087739e-07,     3.4450860179643621e-09, } , 
      {   -1.1976564616363277e+00,    -2.5752938019239205e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    2.0581540251778954e+00,    -8.8860057042560038e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    5.3139645119921184e+00,     7.9809266107271978e+03,    -4.6568873433953641e-04,    -4.8106931960622267e-07,     2.2091020193437112e-09, } , 
      {    1.1578980887504748e+00,    -1.7424246594615754e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    4.4137085755646979e+00,    -5.5731427963255283e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.6695190623789209e+00,     1.6309618035350648e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    3.5134526391372773e+00,    -9.0955551699923053e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.7692631259515004e+00,     7.7713771449908972e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0025073612765723e+01,     2.4638309459974098e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    2.6131967027098568e+00,    -1.7633796060352055e+04,     3.7522548120634379e-04,     3.0315860052776333e-07,    -1.3222808338581481e-09, } , 
      {    5.8690071895240798e+00,    -7.6686374536885523e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    9.1248176763383029e+00,     1.6100068569614348e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2380628163152526e+01,     3.2967000884597546e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    8.2245617399108824e+00,     7.5618276792545948e+03,     3.9950555821570275e-04,     5.4318704212574410e-07,    -2.6171212133654968e-09, } , 
      {    1.1480372226725105e+01,     2.4428759994237796e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.4736182713539328e+01,     4.1295692309220998e+04,     1.4348089670976832e-04,     5.2180617701239621e-07,    -2.7348339751388921e-09, } , 
      {    1.0580116290297685e+01,     1.5890519103878043e+04,     5.5179796597802893e-04,     7.9390591628039882e-07,    -3.8531052119861476e-09, } , 
      {    1.3835926777111908e+01,     3.2757451418861245e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    1.6191481327498710e+01,     4.1086142843484697e+04,     5.7607804298738790e-04,     1.0339343578783797e-06,    -5.1479455914934963e-09, } , 
      {    1.8547035877885513e+01,     4.9414834268108149e+04,     7.2837045074971402e-04,     1.2846532320330344e-06,    -6.3839295901141471e-09, } , 
      {   -5.0516353138390429e+00,    -3.3243778022912295e+04,    -7.9721750836645616e-04,    -1.2219191173993501e-06,     5.9214706678405711e-09, } , 
      {   -2.6960807634522403e+00,    -2.4915086598288843e+04,    -6.4492510060413003e-04,    -9.7120024324469537e-07,     4.6854866692199203e-09, } , 
      {   -3.4052621306543762e-01,    -1.6586395173665391e+04,    -4.9263269284180379e-04,    -7.2048136909004076e-07,     3.4495026705992703e-09, } , 
      {    2.9152842737487852e+00,     2.8053714131780998e+02,    -6.2064502359477107e-04,    -7.3117180164671471e-07,     3.3906462897125724e-09, } , 
      {   -1.2407821494928584e+00,    -2.5124636064025144e+04,    -2.1232795432651048e-04,    -4.5907206237871209e-07,     2.2723750528653169e-09, } , 
      {    2.0150283373213647e+00,    -8.2577037490419425e+03,    -3.4034028507947772e-04,    -4.6976249493538599e-07,     2.2135186719786194e-09, } , 
      {    5.2708388241355877e+00,     8.6092285659412591e+03,    -4.6835261583244494e-04,    -4.8045292749205999e-07,     2.1546622910919216e-09, } , 
      {    1.1147724008939444e+00,    -1.6795944639401692e+04,    -6.0035546564184289e-05,    -2.0835318822405737e-07,     1.0363910542446658e-09, } , 
      {    4.3705828877081672e+00,     7.0987675581507574e+01,    -1.8804787731715151e-04,    -2.1904362078073129e-07,     9.7753467335796819e-10, } , 
      {    7.6263933745223902e+00,     1.6937919990564707e+04,    -3.1606020807011876e-04,    -2.2973405333740524e-07,     9.1867829247127064e-10, } , 
      {    3.4703269512807466e+00,    -8.4672532147782440e+03,     9.2256861198141885e-05,     4.2365685930597359e-08,    -1.9959294437598484e-10, } , 
      {    6.7261374380949697e+00,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    9.9819479249091927e+00,     2.5266611415188159e+04,    -1.6376780030779255e-04,     2.0984820817249487e-08,    -3.1730570614938024e-10, } , 
      {    5.8258815016675491e+00,    -1.3856179015479483e+02,     2.4454926896046804e-04,     2.9308456008525207e-07,    -1.4355769429966358e-09, } , 
      {    9.0816919884817722e+00,     1.6728370524828410e+04,     1.1653693820750085e-04,     2.8239412752857817e-07,    -1.4944333238833335e-09, } , 
      {    8.1814360520543516e+00,     8.1901296344686552e+03,     3.9684167672279422e-04,     5.4380343423990678e-07,    -2.6715609416172864e-09, } , 
      {    1.1437246538868575e+01,     2.5057061949451858e+04,     2.6882934596982700e-04,     5.3311300168323284e-07,    -2.7304173225039839e-09, } , 
      {    1.0536990602441154e+01,     1.6518821059092104e+04,     5.4913408448512046e-04,     7.9452230839456150e-07,    -3.9075449402379377e-09, } , 
      {    1.3792801089255377e+01,     3.3385753374075306e+04,     4.2112175373215318e-04,     7.8383187583788756e-07,    -3.9664013211246355e-09, } , 
      {    1.6148355639642180e+01,     4.1714444798698758e+04,     5.7341416149447942e-04,     1.0345507499925424e-06,    -5.2023853197452864e-09, } , 
      {   -2.7392064513087711e+00,    -2.4286784643074781e+04,    -6.4758898209703862e-04,    -9.7058385113053269e-07,     4.6310469409681310e-09, } , 
      {   -3.8365190092196810e-01,    -1.5958093218451331e+04,    -4.9529657433471238e-04,    -7.1986497697587808e-07,     3.3950629423474802e-09, } , 
      {    1.9719026494648344e+00,    -7.6294017938278830e+03,    -3.4300416657238620e-04,    -4.6914610282122336e-07,     2.1590789437268294e-09, } , 
      {    4.3274571998516365e+00,     6.9928963079556797e+02,    -1.9071175881006004e-04,    -2.1842722866656861e-07,     9.2309494510617842e-10, } , 
      {    3.4272012634242164e+00,    -7.8389512595641845e+03,     8.9592979705233381e-05,     4.2982078044760019e-08,    -2.5403267262777466e-10, } , 
      {    6.6830117502384390e+00,     9.0279810554190190e+03,    -3.8419351047733836e-05,     3.2291645488086085e-08,    -3.1288905351447236e-10, } , 
      {    5.7827558138110184e+00,     4.8974016505926556e+02,     2.4188538746755954e-04,     2.9370095219941469e-07,    -1.4900166712484255e-09, } , 
      {    9.0385663006252415e+00,     1.7356672480042471e+04,     1.1387305671459234e-04,     2.8301051964274080e-07,    -1.5488730521351231e-09, } , 
      {    1.1394120851012044e+01,     2.5685363904665915e+04,     2.6616546447691853e-04,     5.3372939379739552e-07,    -2.7848570507557740e-09, } , 
      {    1.3749675401398846e+01,     3.4014055329289367e+04,     4.1845787223924471e-04,     7.8444826795205024e-07,    -4.0208410493764248e-09, } , 
      {    1.9287769616083039e+00,    -7.0010998386138226e+03,    -3.4566804806529473e-04,    -4.6852971070706068e-07,     2.1046392154750397e-09, } , 
      {    4.2843315119951066e+00,     1.3275915860096284e+03,    -1.9337564030296855e-04,    -2.1781083655240596e-07,     8.6865521685438866e-10, } , 
      {    6.6398860623819091e+00,     9.6562830106330784e+03,    -4.1083232540642354e-05,     3.2908037602248751e-08,    -3.6732878176626217e-10, } , 
      {    8.9954406127687108e+00,     1.7984974435256529e+04,     1.1120917522168383e-04,     2.8362691175690348e-07,    -1.6033127803869130e-09, } , 
      {    1.2226484052247287e+01,     1.1772942558697432e+04,    -5.0199887166382558e-05,     5.8419369834545528e-08,    -8.1380824511046702e-11, } , 
      {    9.8278038140039534e+00,     4.0725530892880424e+03,    -2.0515617642161726e-04,    -1.9168311220594652e-07,     1.1001634458578142e-09, } , 
      {    1.2183358364390756e+01,     1.2401244513911493e+04,    -5.2863768659291069e-05,     5.9035761948708195e-08,    -1.3582055276283652e-10, } , 
      {    1.4538912914777558e+01,     2.0729935938534942e+04,     9.9428639103035113e-05,     3.0975463610336297e-07,    -1.3718045513834875e-09, } , 
      {    7.4291235757606211e+00,    -3.6278363801213472e+03,    -3.6011246567685195e-04,    -4.4178559424643856e-07,     2.2817077162266752e-09, } , 
      {    9.7846781261474245e+00,     4.7008550445021028e+03,    -2.0782005791452577e-04,    -1.9106672009178387e-07,     1.0457237176060245e-09, } , 
      {    8.8844221897200022e+00,    -3.8373858458576487e+03,     7.2484680600767631e-05,     7.0342586619544796e-08,    -1.3140390012792861e-10, } , 
      {    1.2140232676534227e+01,     1.3029546469125553e+04,    -5.5527650152199586e-05,     5.9652154062870862e-08,    -1.9026028101462631e-10, } , 
      {    1.4495787226921030e+01,     2.1358237893749003e+04,     9.6764757610126595e-05,     3.1037102821752559e-07,    -1.4262442796352771e-09, } , 
      {    1.6851341777307830e+01,     2.9686929318372455e+04,     2.4905716537245278e-04,     5.6108990237218037e-07,    -2.6622282782559282e-09, } , 
      {    2.6748887871304854e+00,    -1.9656917274154188e+04,    -6.6736116269441284e-04,    -9.4260695044158538e-07,     4.6992359852161876e-09, } , 
      {    5.0304433375172879e+00,    -1.1328225849530736e+04,    -5.1506875493208661e-04,    -6.9188807628693066e-07,     3.4632519865955363e-09, } , 
      {    7.3859978879040904e+00,    -2.9995344249072868e+03,    -3.6277634716976048e-04,    -4.4116920213227588e-07,     2.2272679879748855e-09, } , 
      {    1.0641808374718314e+01,     1.3867397890075918e+04,    -4.9078867792272764e-04,    -4.5185963468894983e-07,     2.1684116070881881e-09, } , 
      {    6.4857419514766699e+00,    -1.1537775315267038e+04,    -8.2471608654467054e-05,    -1.7975989542094727e-07,     1.0501403702409325e-09, } , 
      {    9.7415524382908938e+00,     5.3291569997161632e+03,    -2.1048393940743427e-04,    -1.9045032797762119e-07,     9.9128398935423468e-10, } , 
      {    1.2997362925105117e+01,     2.2196089314699366e+04,    -3.3849627016040152e-04,    -2.0114076053429511e-07,     9.3242760846753703e-10, } , 
      {    8.8412965018634715e+00,    -3.2090838906435883e+03,     6.9820799107859127e-05,     7.0958978733707463e-08,    -1.8584362837971840e-10, } , 
      {    1.2097106988677696e+01,     1.3657848424339612e+04,    -5.8191531645108090e-05,     6.0268546177033528e-08,    -2.4470000926641610e-10, } , 
      {    1.1196851052250276e+01,     5.1196075339798617e+03,     2.2211320687018531e-04,     3.2167785288836217e-07,    -1.4218276270003691e-09, } , 
      {    1.4452661539064499e+01,     2.1986539848963064e+04,     9.4100876117218091e-05,     3.1098742033168827e-07,    -1.4806840078870670e-09, } , 
      {    1.3552405602637078e+01,     1.3448298958603311e+04,     3.7440561463251152e-04,     5.7239672704301689e-07,    -2.6578116256210200e-09, } , 
      {    1.6808216089451300e+01,     3.0315231273586516e+04,     2.4639328387954424e-04,     5.6170629448634294e-07,    -2.7166680065077178e-09, } , 
      {    1.9163770639838106e+01,     3.8643922698209957e+04,     3.9868569164187048e-04,     8.1242516864099766e-07,    -3.9526520051283682e-09, } , 
      {    2.7620854888715263e-01,    -2.7357306743563582e+04,    -8.2231745194964745e-04,    -1.1927094324820775e-06,     5.8807802555850483e-09, } , 
      {    2.6317630992739551e+00,    -1.9028615318940130e+04,    -6.7002504418732132e-04,    -9.4199055832742270e-07,     4.6447962569643975e-09, } , 
      {    1.7315071628465346e+00,    -2.7566856209299876e+04,    -3.8972030567202797e-04,    -6.8058125161609403e-07,     3.4676686392304445e-09, } , 
      {    4.9873176496607581e+00,    -1.0699923894316675e+04,    -5.1773263642499508e-04,    -6.9127168417276798e-07,     3.4088122583437467e-09, } , 
      {    4.0870617132333376e+00,    -1.9238164784676432e+04,    -2.3742789790970174e-04,    -4.2986237746143931e-07,     2.2316846406097937e-09, } , 
      {    7.3428722000475606e+00,    -2.3712324696932264e+03,    -3.6544022866266901e-04,    -4.4055281001811320e-07,     2.1728282597230958e-09, } , 
      {    1.0598682686861784e+01,     1.4495699845289977e+04,    -4.9345255941563612e-04,    -4.5124324257478715e-07,     2.1139718788363984e-09, } , 
      {    6.4426162636201401e+00,    -1.0909473360052978e+04,    -8.5135490147375572e-05,    -1.7914350330678462e-07,     9.9570064198914267e-10, } , 
      {    9.6984267504343631e+00,     5.9574589549302236e+03,    -2.1314782090034278e-04,    -1.8983393586345854e-07,     9.3684426110244481e-10, } , 
      {    1.2954237237248586e+01,     2.2824391269913423e+04,    -3.4116015165331005e-04,    -2.0052436842013246e-07,     8.7798788021574726e-10, } , 
      {    8.7981708140069426e+00,    -2.5807819354295279e+03,     6.7156917614950609e-05,     7.1575370847870116e-08,    -2.4028335663150822e-10, } , 
      {    1.2053981300821166e+01,     1.4286150379553674e+04,    -6.0855413138016608e-05,     6.0884938291196182e-08,    -2.9913973751820592e-10, } , 
      {    7.8979148775795220e+00,    -1.1119022825789281e+04,     3.4746165613024402e-04,     3.3298467755919880e-07,    -1.4174109743654613e-09, } , 
      {    1.1153725364393745e+01,     5.7479094891939221e+03,     2.1944932537727680e-04,     3.2229424500252485e-07,    -1.4762673552521590e-09, } , 
      {    1.4409535851207968e+01,     2.2614841804177122e+04,     9.1436994624309573e-05,     3.1160381244585095e-07,    -1.5351237361388568e-09, } , 
      {    1.3509279914780548e+01,     1.4076600913817372e+04,     3.7174173313960299e-04,     5.7301311915717957e-07,    -2.7122513538728096e-09, } , 
      {    1.6765090401594769e+01,     3.0943533228800574e+04,     2.4372940238663577e-04,     5.6232268660050562e-07,    -2.7711077347595075e-09, } , 
      {    1.5864834465167350e+01,     2.2405292338440820e+04,     5.2403414090192917e-04,     8.2373199331183429e-07,    -3.9482353524934613e-09, } , 
      {    1.9120644951981575e+01,     3.9272224653424018e+04,     3.9602181014896195e-04,     8.1304156075516034e-07,    -4.0070917333801583e-09, } , 
      {    2.1476199502368374e+01,     4.7600916078047470e+04,     5.4831421791128813e-04,     1.0637604349098150e-06,    -5.2430757320008091e-09, } , 
      {    2.3831754052755180e+01,     5.5929607502670922e+04,     7.0060662567361426e-04,     1.3144793090644697e-06,    -6.4790597306214600e-09, } , 
      {   -2.1224716893561801e+00,    -3.5057696212972965e+04,    -9.7727374120488205e-04,    -1.4428119145225696e-06,     7.0623245259539099e-09, } , 
      {    2.3308286103062215e-01,    -2.6729004788349521e+04,    -8.2498133344255603e-04,    -1.1920930403679148e-06,     5.8263405273332591e-09, } , 
      {   -6.6717307539679815e-01,    -3.5267245678709267e+04,    -5.4467659492726263e-04,    -9.3068373365658607e-07,     4.6492129095993053e-09, } , 
      {    2.5886374114174249e+00,    -1.8400313363726069e+04,    -6.7268892568022990e-04,    -9.4137416621326002e-07,     4.5903565287126083e-09, } , 
      {    1.6883814749900041e+00,    -2.6938554254085815e+04,    -3.9238418716493645e-04,    -6.7996485950193135e-07,     3.4132289109786545e-09, } , 
      {    4.9441919618042274e+00,    -1.0071621939102613e+04,    -5.2039651791790367e-04,    -6.9065529205860530e-07,     3.3543725300919566e-09, } , 
      {    8.2000024486184504e+00,     6.7953103758805892e+03,    -6.4840884867087094e-04,    -7.0134572461527924e-07,     3.2955161492052592e-09, } , 
      {    4.0439360253768069e+00,    -1.8609862829462370e+04,    -2.4009177940261027e-04,    -4.2924598534727668e-07,     2.1772449123580036e-09, } , 
      {    7.2997465121910299e+00,    -1.7429305144791660e+03,    -3.6810411015557749e-04,    -4.3993641790395058e-07,     2.1183885314713058e-09, } , 
      {    1.0555556999005253e+01,     1.5124001800504037e+04,    -4.9611644090854471e-04,    -4.5062685046062452e-07,     2.0595321505846083e-09, } , 
      {    6.3994905757636094e+00,    -1.0281171404838919e+04,    -8.7799371640284076e-05,    -1.7852711119262194e-07,     9.4126091373735280e-10, } , 
      {    9.6553010625778324e+00,     6.5857609101442840e+03,    -2.1581170239325131e-04,    -1.8921754374929586e-07,     8.8240453285065505e-10, } , 
      {    1.2911111549392055e+01,     2.3452693225127485e+04,    -3.4382403314621852e-04,    -1.9990797630596978e-07,     8.2354815196395740e-10, } , 
      {    5.4992346393361888e+00,    -1.8819412295198672e+04,     1.9250536687500931e-04,     8.2882195518706718e-08,    -2.3586670399660028e-10, } , 
      {    8.7550451261504119e+00,    -1.9524799802154675e+03,     6.4493036122042105e-05,     7.2191762962032783e-08,    -2.9472308488329798e-10, } , 
      {    1.2010855612964635e+01,     1.4914452334767735e+04,    -6.3519294630925112e-05,     6.1501330405358849e-08,    -3.5357946576999568e-10, } , 
      {    1.5266666099778858e+01,     3.1781384649750937e+04,    -1.9153162538389234e-04,     5.0810897848684914e-08,    -4.1243584665669338e-10, } , 
      {    7.8547891897229913e+00,    -1.0490720870575220e+04,     3.4479777463733554e-04,     3.3360106967336142e-07,    -1.4718507026172510e-09, } , 
      {    1.1110599676537214e+01,     6.3762114444079825e+03,     2.1678544388436827e-04,     3.2291063711668748e-07,    -1.5307070835039487e-09, } , 
      {    1.4366410163351437e+01,     2.3243143759391183e+04,     8.8773113131401069e-05,     3.1222020456001358e-07,    -1.5895634643906465e-09, } , 
      {    1.7622220650165659e+01,     4.0110076074374389e+04,    -3.9239217621566155e-05,     3.0152977200333963e-07,    -1.6484198452773442e-09, } , 
      {    1.0210343740109794e+01,    -2.1620294459517709e+03,     4.9709018239966162e-04,     5.8431994382801609e-07,    -2.7078347012379023e-09, } , 
      {    1.3466154226924017e+01,     1.4704902869031434e+04,     3.6907785164669451e-04,     5.7362951127134225e-07,    -2.7666910821245997e-09, } , 
      {    1.6721964713738238e+01,     3.1571835184014635e+04,     2.4106552089372724e-04,     5.6293907871466830e-07,    -2.8255474630112975e-09, } , 
      {    1.5821708777310819e+01,     2.3033594293654882e+04,     5.2137025940902058e-04,     8.2434838542599697e-07,    -4.0026750807452505e-09, } , 
      {    1.9077519264125044e+01,     3.9900526608638080e+04,     3.9335792865605347e-04,     8.1365795286932302e-07,    -4.0615314616319476e-09, } , 
      {    1.8177263327697624e+01,     3.1362285718278334e+04,     6.7366266717134682e-04,     1.0750672595806517e-06,    -5.2386590793659014e-09, } , 
      {    2.1433073814511843e+01,     4.8229218033261532e+04,     5.4565033641837955e-04,     1.0643768270239776e-06,    -5.2975154602525984e-09, } , 
      {    2.3788628364898649e+01,     5.6557909457884984e+04,     6.9794274418070567e-04,     1.3150957011786324e-06,    -6.5334994588732492e-09, } , 
      {    2.6144182915285448e+01,     6.4886600882508428e+04,     8.5023515194303191e-04,     1.5658145753332871e-06,    -7.7694834574939017e-09, } , 
      {   -4.5211519275995133e+00,    -4.2758085682382356e+04,    -1.1322300304601170e-03,    -1.6929143965630616e-06,     8.2438687963227707e-09, } , 
      {   -2.1655973772127108e+00,    -3.4429394257758904e+04,    -9.7993762269779064e-04,    -1.4421955224084069e-06,     7.0078847977021198e-09, } , 
      {   -3.0658533136401314e+00,    -4.2967635148118658e+04,    -6.9963288418249724e-04,    -1.1807862156970781e-06,     5.8307571799681660e-09, } , 
      {    1.8995717317409166e-01,    -2.6100702833135460e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {    3.4457676599883147e+00,    -9.2337705181522542e+03,    -9.5565754568843167e-04,    -1.2021670808104260e-06,     5.7130444181947712e-09, } , 
      {   -7.1029876325332886e-01,    -3.4638943723495206e+04,    -5.4734047642017111e-04,    -9.3006734154242339e-07,     4.5947731813475152e-09, } , 
      {    2.5455117235608942e+00,    -1.7772011408512008e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    5.8013222103751172e+00,    -9.0507909352880233e+02,    -8.0336513792610554e-04,    -9.5144820665577128e-07,     4.4770604195741203e-09, } , 
      {    1.6452557871334736e+00,    -2.6310252298871754e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    4.9010662739476967e+00,    -9.4433199838885539e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.1568767607619197e+00,     7.4236123310946496e+03,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {    1.1412687247576143e+01,     2.4290544646077848e+04,    -7.7908506091674658e-04,    -7.1141976505779051e-07,     3.1822200400667716e-09, } , 
      {    7.4499985070605312e-01,    -3.4848493189231507e+04,    -1.1474333014255156e-04,    -4.1793916067644005e-07,     2.1816615649929114e-09, } , 
      {    4.0008103375202761e+00,    -1.7981560874248309e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    7.2566208243344992e+00,    -1.1146285592651057e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0512431311148722e+01,     1.5752303755718098e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.3768241797962945e+01,     3.2619236070701296e+04,    -6.2679265315442045e-04,    -4.6070089090313579e-07,     1.9462360414461208e-09, } , 
      {    3.1005544010928556e+00,    -2.6519801764608063e+04,     3.7549077619774624e-05,    -1.6722028652178533e-07,     9.4567756637226078e-10, } , 
      {    6.3563648879070787e+00,    -9.6528694496248572e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.6121753747213017e+00,     7.2140628653583444e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2867985861535525e+01,     2.4080995180341546e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.4561089514796581e+00,    -1.8191110339984611e+04,     1.8984148538210081e-04,     8.3498587632869385e-08,    -2.9030643224839010e-10, } , 
      {    8.7119194382938812e+00,    -1.3241780250014071e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1967729925108104e+01,     1.5542754289981794e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5223540411922327e+01,     3.2409686604964998e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    7.8116635018664606e+00,    -9.8624189153611605e+03,     3.4213389314442701e-04,     3.3421746178752410e-07,    -1.5262904308690409e-09, } , 
      {    1.1067473988680684e+01,     7.0045133996220429e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.4323284475494907e+01,     2.3871445714605245e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7579094962309128e+01,     4.0738378029588450e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    1.0167218052253263e+01,    -1.5337274907377105e+03,     4.9442630090675314e-04,     5.8493633594217877e-07,    -2.7622744294896919e-09, } , 
      {    1.3423028539067486e+01,     1.5333204824245493e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    1.6678839025881707e+01,     3.2200137139228696e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9934649512695934e+01,     4.9067069454211902e+04,     1.1038930864785152e-04,     5.5286503827215703e-07,    -2.9388435721497847e-09, } , 
      {    1.2522772602640066e+01,     6.7949639338857378e+03,     6.4671870866907938e-04,     8.3565521009683349e-07,    -3.9982584281103427e-09, } , 
      {    1.5778583089454289e+01,     2.3661896248868943e+04,     5.1870637791611210e-04,     8.2496477754015965e-07,    -4.0571148089970406e-09, } , 
      {    1.9034393576268513e+01,     4.0528828563852141e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.8134137639841093e+01,     3.1990587673492395e+04,     6.7099878567843834e-04,     1.0756836516948144e-06,    -5.2930988076176914e-09, } , 
      {    2.1389948126655312e+01,     4.8857519988475593e+04,     5.4298645492547107e-04,     1.0649932191381403e-06,    -5.3519551885043885e-09, } , 
      {    2.0489692190227892e+01,     4.0319279098115847e+04,     8.2329119344076447e-04,     1.3264025258494691e-06,    -6.5290828062383423e-09, } , 
      {    2.3745502677042118e+01,     5.7186211413099045e+04,     6.9527886268779720e-04,     1.3157120932927950e-06,    -6.5879391871250393e-09, } , 
      {    2.6101057227428917e+01,     6.5514902837722489e+04,     8.4757127045012343e-04,     1.5664309674474498e-06,    -7.8239231857456909e-09, } , 
      {   -2.2087230650692415e+00,    -3.3801092302544843e+04,    -9.8260150419069922e-04,    -1.4415791302942442e-06,     6.9534450694503298e-09, } , 
      {    1.4683148531756118e-01,    -2.5472400877921398e+04,    -8.3030909642837299e-04,    -1.1908602561395895e-06,     5.7174610708296789e-09, } , 
      {   -7.5342445110985956e-01,    -3.4010641768281144e+04,    -5.5000435791307959e-04,    -9.2945094942826071e-07,     4.5403334530957251e-09, } , 
      {    2.5023860357043635e+00,    -1.7143709453297946e+04,    -6.7801668866604686e-04,    -9.4014138198493466e-07,     4.4814770722090281e-09, } , 
      {    1.6021300992769432e+00,    -2.5681950343657692e+04,    -3.9771195015075351e-04,    -6.7873207527360599e-07,     3.3043494544750751e-09, } , 
      {    4.8579405860911660e+00,    -8.8150180286744944e+03,    -5.2572428090372062e-04,    -6.8942250783027994e-07,     3.2454930735883773e-09, } , 
      {    8.1137510729053890e+00,     8.0519142863087100e+03,    -6.5373661165668789e-04,    -7.0011294038695388e-07,     3.1866366927016798e-09, } , 
      {    3.9576846496637454e+00,    -1.7353258919034248e+04,    -2.4541954238842728e-04,    -4.2801320111895132e-07,     2.0683654558544243e-09, } , 
      {    7.2134951364779685e+00,    -4.8632660405104525e+02,    -3.7343187314139455e-04,    -4.3870363367562522e-07,     2.0095090749677264e-09, } , 
      {    1.0469305623292191e+01,     1.6380605710932159e+04,    -5.0144420389436166e-04,    -4.4939406623229916e-07,     1.9506526940810290e-09, } , 
      {    6.3132392000505480e+00,    -9.0245674944107959e+03,    -9.3127134626101111e-05,    -1.7729432696429663e-07,     8.3238145723377327e-10, } , 
      {    9.5690496868647710e+00,     7.8423648205724048e+03,    -2.2113946537906831e-04,    -1.8798475952097055e-07,     7.7352507634707552e-10, } , 
      {    1.2824860173678994e+01,     2.4709297135555607e+04,    -3.4915179613203559e-04,    -1.9867519207764447e-07,     7.1466869546037787e-10, } , 
      {    5.4129832636231274e+00,    -1.7562808384770549e+04,     1.8717760388919230e-04,     8.4114979747032051e-08,    -3.4474616050017991e-10, } , 
      {    8.6687937504373505e+00,    -6.9587606978734675e+02,     5.9165273136225077e-05,     7.3424547190358117e-08,    -4.0360254138687761e-10, } , 
      {    1.1924604237251573e+01,     1.6171056245195854e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {    1.5180414724065797e+01,     3.3037988560179059e+04,    -1.9685938836970935e-04,     5.2043682077010248e-08,    -5.2131530316027296e-10, } , 
      {    7.7685378140099299e+00,    -9.2341169601471011e+03,     3.3947001165151848e-04,     3.3483385390168678e-07,    -1.5807301591208307e-09, } , 
      {    1.1024348300824153e+01,     7.6328153548361033e+03,     2.1145768089855126e-04,     3.2414342134501284e-07,    -1.6395865400075284e-09, } , 
      {    1.4280158787638376e+01,     2.4499747669819306e+04,     8.3445350145584034e-05,     3.1345298878833894e-07,    -1.6984429208942262e-09, } , 
      {    1.7535969274452597e+01,     4.1366679984802511e+04,    -4.4566980607383177e-05,     3.0276255623166499e-07,    -1.7572993017809239e-09, } , 
      {    1.0124092364396732e+01,    -9.0542553552365007e+02,     4.9176241941384466e-04,     5.8555272805634145e-07,    -2.8167141577414816e-09, } , 
      {    1.3379902851210955e+01,     1.5961506779459552e+04,     3.6375008866087745e-04,     5.7486229549966761e-07,    -2.8755705386281790e-09, } , 
      {    1.6635713338025177e+01,     3.2828439094442758e+04,     2.3573775790791023e-04,     5.6417186294299366e-07,    -2.9344269195148769e-09, } , 
      {    1.5735457401597758e+01,     2.4290198204083004e+04,     5.1604249642320363e-04,     8.2558116965432233e-07,    -4.1115545372488307e-09, } , 
      {    1.8991267888411983e+01,     4.1157130519066202e+04,     3.8803016567023641e-04,     8.1489073709764838e-07,    -4.1704109181355277e-09, } , 
      {    1.8091011951984562e+01,     3.2618889628706456e+04,     6.6833490418552986e-04,     1.0763000438089770e-06,    -5.3475385358694815e-09, } , 
      {    2.1346822438798782e+01,     4.9485821943689654e+04,     5.4032257343256259e-04,     1.0656096112523030e-06,    -5.4063949167561785e-09, } , 
      {    2.3702376989185588e+01,     5.7814513368313106e+04,     6.9261498119488872e-04,     1.3163284854069577e-06,    -6.6423789153768294e-09, } , 
      {    2.4592603478478332e+00,    -1.6515407498083885e+04,    -6.8068057015895544e-04,    -9.3952498987077198e-07,     4.4270373439572389e-09, } , 
      {    1.5590044114204127e+00,    -2.5053648388443631e+04,    -4.0037583164366199e-04,    -6.7811568315944331e-07,     3.2499097262232851e-09, } , 
      {    4.8148148982346353e+00,    -8.1867160734604331e+03,    -5.2838816239662921e-04,    -6.8880611571611726e-07,     3.1910533453365872e-09, } , 
      {    3.9145589618072152e+00,    -1.6724956963820187e+04,    -2.4808342388133581e-04,    -4.2739680900478869e-07,     2.0139257276026342e-09, } , 
      {    7.1703694486214378e+00,     1.4197535116301515e+02,    -3.7609575463430303e-04,    -4.3808724156146259e-07,     1.9550693467159364e-09, } , 
      {    1.0426179935435661e+01,     1.7008907666146217e+04,    -5.0410808538727024e-04,    -4.4877767411813654e-07,     1.8962129658292389e-09, } , 
      {    6.2701135121940172e+00,    -8.3962655391967364e+03,    -9.5791016119009615e-05,    -1.7667793485013395e-07,     7.7794172898198340e-10, } , 
      {    9.5259239990082403e+00,     8.4706667757864652e+03,    -2.2380334687197685e-04,    -1.8736836740680787e-07,     7.1908534809528565e-10, } , 
      {    1.2781734485822463e+01,     2.5337599090769669e+04,    -3.5181567762494406e-04,    -1.9805879996348179e-07,     6.6022896720858800e-10, } , 
      {    8.6256680625808198e+00,    -6.7574114573286352e+01,     5.6501391643316566e-05,     7.4040939304520783e-08,    -4.5804226963866738e-10, } , 
      {    1.1881478549395043e+01,     1.6799358200409915e+04,    -7.1510939109650651e-05,     6.3350506747846849e-08,    -5.1689865052536508e-10, } , 
      {    1.0981222612967622e+01,     8.2611173100501637e+03,     2.0879379940564273e-04,     3.2475981345917546e-07,    -1.6940262682593181e-09, } , 
      {    1.4237033099781845e+01,     2.5128049625033367e+04,     8.0781468652675530e-05,     3.1406938090250157e-07,    -1.7528826491460159e-09, } , 
      {    1.3336777163354425e+01,     1.6589808734673614e+04,     3.6108620716796897e-04,     5.7547868761383029e-07,    -2.9300102668799691e-09, } , 
      {    1.6592587650168646e+01,     3.3456741049656819e+04,     2.3307387641500170e-04,     5.6478825505715634e-07,    -2.9888666477666669e-09, } , 
      {    4.7716892103781055e+00,    -7.5584141182463727e+03,    -5.3105204388953769e-04,    -6.8818972360195458e-07,     3.1366136170847975e-09, } , 
      {    7.1272437607649080e+00,     7.7027730637707555e+02,    -3.7875963612721156e-04,    -4.3747084944729991e-07,     1.9006296184641467e-09, } , 
      {    6.2269878243374874e+00,    -7.7679635839826760e+03,    -9.8454897611918133e-05,    -1.7606154273597130e-07,     7.2350200073019363e-10, } , 
      {    9.4827983111517113e+00,     9.0989687310005247e+03,    -2.2646722836488535e-04,    -1.8675197529264522e-07,     6.6464561984349588e-10, } , 
      {    1.1838352861538512e+01,     1.7427660155623977e+04,    -7.4174820602559169e-05,     6.3966898862009502e-08,    -5.7133837877715495e-10, } , 
      {    9.4396726232951806e+00,     9.7272706862145860e+03,    -2.2913110985779385e-04,    -1.8613558317848254e-07,     6.1020589159170612e-10, } , 
      {    1.4983144925304027e+01,     1.2472232189493001e+04,    -2.4091164597644257e-04,    -1.6000785883202313e-07,     8.4171412059513177e-10, } , 
      {    1.7338699475690831e+01,     2.0800923614116451e+04,    -8.8619238214116401e-05,     9.0711015322631619e-08,    -3.9426987802551906e-10, } , 
      {    1.2584464687060695e+01,     4.7718427200836113e+03,    -3.9586793523167725e-04,    -4.1011034087251517e-07,     2.0232583909639929e-09, } , 
      {    1.4940019237447498e+01,     1.3100534144707060e+04,    -2.4357552746935107e-04,    -1.5939146671786045e-07,     7.8727439234334201e-10, } , 
      {    1.4039763301020077e+01,     4.5622932543473098e+03,     3.6729211045942312e-05,     1.0201783999346822e-07,    -3.8985322539061107e-10, } , 
      {    1.7295573787834300e+01,     2.1429225569330512e+04,    -9.1283119707024905e-05,     9.1327407436794286e-08,    -4.4870960627730882e-10, } , 
      {    1.9651128338221106e+01,     2.9757916993953961e+04,     6.1009288055301276e-05,     3.4204628159144899e-07,    -1.6846936048979595e-09, } , 
      {    1.0185784448817362e+01,    -2.9285467493257765e+03,    -5.5082422448691191e-04,    -6.6021282291300726e-07,     3.2048026613328541e-09, } , 
      {    1.2541338999204164e+01,     5.4001446752976717e+03,    -3.9853181672458579e-04,    -4.0949394875835249e-07,     1.9688186627122033e-09, } , 
      {    1.1641083062776744e+01,    -3.1380962150620799e+03,    -1.1822707820929237e-04,    -1.4808464204702385e-07,     7.9169104497824999e-10, } , 
      {    1.4896893549590967e+01,     1.3728836099921122e+04,    -2.4623940896225960e-04,    -1.5877507460369779e-07,     7.3283466409155224e-10, } , 
      {    1.3996637613163546e+01,     5.1905952095613702e+03,     3.4065329553033795e-05,     1.0263423210763087e-07,    -4.4429295364240089e-10, } , 
      {    1.7252448099977769e+01,     2.2057527524544574e+04,    -9.3947001199933423e-05,     9.1943799550956940e-08,    -5.0314933452909859e-10, } , 
      {    1.6352192163550349e+01,     1.3519286634184820e+04,     1.8635773731536000e-04,     3.5335310626228562e-07,    -1.6802769522630518e-09, } , 
      {    1.9608002650364575e+01,     3.0386218949168018e+04,     5.8345406562392758e-05,     3.4266267370561167e-07,    -1.7391333331497494e-09, } , 
      {    2.1963557200751374e+01,     3.8714910373791470e+04,     2.1063781432471897e-04,     5.9338154786026634e-07,    -2.9751173317704000e-09, } , 
      {    7.7871042105740287e+00,    -1.0628936218735169e+04,    -7.0578051374214673e-04,    -9.1031530495349930e-07,     4.3863469317017153e-09, } , 
      {    1.0142658760960831e+01,    -2.3002447941117161e+03,    -5.5348810597982050e-04,    -6.5959643079884458e-07,     3.1503629330810640e-09, } , 
      {    9.2424028245334107e+00,    -1.0838485684471470e+04,    -2.7318336746452704e-04,    -3.9818712408751591e-07,     1.9732353153471111e-09, } , 
      {    1.2498213311347634e+01,     6.0284466305117321e+03,    -4.0119569821749426e-04,    -4.0887755664418986e-07,     1.9143789344604132e-09, } , 
      {    1.5754023798161858e+01,     2.2895378945494933e+04,    -5.2920802897046153e-04,    -4.1956798920086381e-07,     1.8555225535737156e-09, } , 
      {    1.1597957374920213e+01,    -2.5097942598480195e+03,    -1.2089095970220088e-04,    -1.4746824993286117e-07,     7.3725131672646013e-10, } , 
      {    1.4853767861734436e+01,     1.4357138055135183e+04,    -2.4890329045516808e-04,    -1.5815868248953511e-07,     6.7839493583976237e-10, } , 
      {    1.3953511925307016e+01,     5.8188971647754306e+03,     3.1401448060125284e-05,     1.0325062422179354e-07,    -4.9873268189419070e-10, } , 
      {    1.7209322412121239e+01,     2.2685829479758631e+04,    -9.6610882692841927e-05,     9.2560191665119606e-08,    -5.5758906278088846e-10, } , 
      {    1.6309066475693818e+01,     1.4147588589398882e+04,     1.8369385582245147e-04,     3.5396949837644825e-07,    -1.7347166805148414e-09, } , 
      {    1.9564876962508045e+01,     3.1014520904382080e+04,     5.5681525069484248e-05,     3.4327906581977430e-07,    -1.7935730614015391e-09, } , 
      {    1.8664621026080624e+01,     2.2476280014022330e+04,     3.3598626358477768e-04,     6.0468837253110297e-07,    -2.9707006791354923e-09, } , 
      {    2.1920431512894844e+01,     3.9343212329005532e+04,     2.0797393283181044e-04,     5.9399793997442902e-07,    -3.0295570600221901e-09, } , 
      {    5.3884239723306955e+00,    -1.8329325688144560e+04,    -8.6073680299738134e-04,    -1.1604177869939913e-06,     5.5678912020705760e-09, } , 
      {    7.7439785227174980e+00,    -1.0000634263521108e+04,    -7.0844439523505521e-04,    -9.0969891283933662e-07,     4.3319072034499252e-09, } , 
      {    6.8437225862900775e+00,    -1.8538875153880857e+04,    -4.2813965671976176e-04,    -6.4828960612800795e-07,     3.1547795857159722e-09, } , 
      {    1.0099533073104300e+01,    -1.6719428388976557e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    1.3355343559918525e+01,     1.5194989476085546e+04,    -6.8416431822569614e-04,    -6.6967047124135585e-07,     3.0370668239425765e-09, } , 
      {    9.1992771366768800e+00,    -1.0210183729257409e+04,    -2.7584724895743558e-04,    -3.9757073197335323e-07,     1.9187955870953214e-09, } , 
      {    1.2455087623491103e+01,     6.6567485857257925e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.5710898110305328e+01,     2.3523680900708994e+04,    -5.3187191046337001e-04,    -4.1895159708670113e-07,     1.8010828253219257e-09, } , 
      {    1.1554831687063682e+01,    -1.8814923046339591e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.4810642173877905e+01,     1.4985440010349243e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.8066452660692129e+01,     3.1852372325332442e+04,    -3.7957950270104383e-04,    -1.6823272293204638e-07,     5.6509882670127486e-10, } , 
      {    1.0654575750636262e+01,    -1.0419733194993711e+04,     1.5674989732018400e-04,     1.1455744889263014e-07,    -4.9431602925928282e-10, } , 
      {    1.3910386237450485e+01,     6.4471991199894910e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.7166196724264708e+01,     2.3314131434972693e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.0422007211078935e+01,     4.0181063749955894e+04,    -2.2728709493871765e-04,     8.2486151222608339e-08,    -6.7088517191937597e-10, } , 
      {    1.3010130301023064e+01,    -2.0910417703702606e+03,     3.0904230508251019e-04,     3.6527632304728487e-07,    -1.7303000278799337e-09, } , 
      {    1.6265940787837287e+01,     1.4775890544612941e+04,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {    1.9521751274651514e+01,     3.1642822859596141e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.2777561761465734e+01,     4.8509755174579346e+04,    -7.4994687176391467e-05,     3.3320502537726303e-07,    -1.9068691705400268e-09, } , 
      {    1.8621495338224094e+01,     2.3104581969236391e+04,     3.3332238209186915e-04,     6.0530476464526565e-07,    -3.0251404073872819e-09, } , 
      {    2.1877305825038313e+01,     3.9971514284219593e+04,     2.0531005133890193e-04,     5.9461433208859170e-07,    -3.0839967882739798e-09, } , 
      {    2.4232860375425119e+01,     4.8300205708843045e+04,     3.5760245910122811e-04,     8.4533320624324642e-07,    -4.3199807868946306e-09, } , 
      {    2.6588414925811918e+01,     5.6628897133466497e+04,     5.0989486686355424e-04,     1.0960520803979011e-06,    -5.5559647855152814e-09, } , 
      {    5.3452982844741648e+00,    -1.7701023732930498e+04,    -8.6340068449028981e-04,    -1.1598013948798287e-06,     5.5134514738187860e-09, } , 
      {    7.7008528348609673e+00,    -9.3723323083070463e+03,    -7.1110827672796369e-04,    -9.0908252072517394e-07,     4.2774674751981351e-09, } , 
      {    1.0056407385247770e+01,    -1.0436408836835953e+03,    -5.5881586896563745e-04,    -6.5836364657051922e-07,     3.0414834765774847e-09, } , 
      {    1.3312217872061995e+01,     1.5823291431299607e+04,    -6.8682819971860461e-04,    -6.6905407912719317e-07,     2.9826270956907869e-09, } , 
      {    9.1561514488203493e+00,    -9.5818817740433478e+03,    -2.7851113045034411e-04,    -3.9695433985919055e-07,     1.8643558588435317e-09, } , 
      {    1.2411961935634572e+01,     7.2850505409398529e+03,    -4.0652346120331132e-04,    -4.0764477241586450e-07,     1.8054994779568337e-09, } , 
      {    1.5667772422448797e+01,     2.4151982855923055e+04,    -5.3453579195627849e-04,    -4.1833520497253845e-07,     1.7466430970701358e-09, } , 
      {    1.1511705999207152e+01,    -1.2531903494198987e+03,    -1.2621872268801790e-04,    -1.4623546570453586e-07,     6.2837186022288060e-10, } , 
      {    1.4767516486021375e+01,     1.5613741965563302e+04,    -2.5423105344098514e-04,    -1.5692589826120981e-07,     5.6951547933618284e-10, } , 
      {    1.8023326972835598e+01,     3.2480674280546504e+04,    -3.8224338419395236e-04,    -1.6761633081788373e-07,     5.1065909844948509e-10, } , 
      {    1.3867260549593954e+01,     7.0755010752035514e+03,     2.6073685074308262e-05,     1.0448340845011887e-07,    -6.0761213839777023e-10, } , 
      {    1.7123071036408177e+01,     2.3942433390186754e+04,    -1.0193864567865896e-04,     9.3792975893444940e-08,    -6.6646851928446799e-10, } , 
      {    1.6222815099980757e+01,     1.5404192499827001e+04,     1.7836609283663446e-04,     3.5520228260477361e-07,    -1.8435961370184212e-09, } , 
      {    1.9478625586794983e+01,     3.2271124814810202e+04,     5.0353762083667226e-05,     3.4451185004809966e-07,    -1.9024525179051186e-09, } , 
      {    2.1834180137181782e+01,     4.0599816239433654e+04,     2.0264616984599343e-04,     5.9523072420275438e-07,    -3.1384365165257694e-09, } , 
      {    2.4189734687568588e+01,     4.8928507664057106e+04,     3.5493857760831958e-04,     8.4594959835740910e-07,    -4.3744205151464207e-09, } , 
      {    1.0013281697391239e+01,    -4.1533892846953495e+02,    -5.6147975045854604e-04,    -6.5774725445635654e-07,     2.9870437483256946e-09, } , 
      {    1.2368836247778042e+01,     7.9133524961539133e+03,    -4.0918734269621980e-04,    -4.0702838030170187e-07,     1.7510597497050440e-09, } , 
      {    1.4724390798164844e+01,     1.6242043920777363e+04,    -2.5689493493389362e-04,    -1.5630950614704713e-07,     5.1507575108439298e-10, } , 
      {    1.3824134861737424e+01,     7.7038030304176118e+03,     2.3409803581399751e-05,     1.0509980056428154e-07,    -6.6205186664956010e-10, } , 
      {    1.7079945348551647e+01,     2.4570735345400812e+04,    -1.0460252717156747e-04,     9.4409368007607607e-08,    -7.2090824753625785e-10, } , 
      {    1.9435499898938453e+01,     3.2899426770024264e+04,     4.7689880590758715e-05,     3.4512824216226229e-07,    -1.9568922461569087e-09, } , 
      {    2.1791054449325252e+01,     4.1228118194647715e+04,     1.9998228835308490e-04,     5.9584711631691706e-07,    -3.1928762447775595e-09, } , 
      {    9.0699000731072879e+00,    -8.3252778636152289e+03,    -2.8383889343616112e-04,    -3.9572155563086524e-07,     1.7554764023399520e-09, } , 
      {    1.2325710559921511e+01,     8.5416544513679728e+03,    -4.1185122418912833e-04,    -4.0641198818753919e-07,     1.6966200214532541e-09, } , 
      {    1.4681265110308313e+01,     1.6870345875991425e+04,    -2.5955881642680215e-04,    -1.5569311403288447e-07,     4.6063602283260321e-10, } , 
      {    1.3781009173880893e+01,     8.3321049856316713e+03,     2.0745922088491237e-05,     1.0571619267844419e-07,    -7.1649159490134987e-10, } , 
      {    2.0224737412317161e+01,     1.9615307379269838e+04,    -2.7133935254545089e-04,    -1.2956538968642501e-07,     6.9214425183602887e-10, } , 
      {    2.2580291962703967e+01,     2.7943998803893286e+04,    -1.1904694478312470e-04,     1.2115348446822971e-07,    -5.4383974678462196e-10, } , 
      {    1.7826057174073831e+01,     1.1914917909860447e+04,    -4.2629564180068555e-04,    -3.7966787172691708e-07,     1.8736885222048896e-09, } , 
      {    2.0181611724460630e+01,     2.0243609334483899e+04,    -2.7400323403835937e-04,    -1.2894899757226236e-07,     6.3770452358423900e-10, } , 
      {    2.2537166274847436e+01,     2.8572300759107347e+04,    -1.2171082627603322e-04,     1.2176987658239236e-07,    -5.9827947503641183e-10, } , 
      {    2.4892720825234235e+01,     3.6900992183730792e+04,     3.0581581486292965e-05,     3.7248875073704708e-07,    -1.8342634736570627e-09, } , 
      {    1.5427376935830496e+01,     4.2145284404510594e+03,    -5.8125193105592026e-04,    -6.2977035376740911e-07,     3.0552327925737508e-09, } , 
      {    1.7782931486217301e+01,     1.2543219865074509e+04,    -4.2895952329359408e-04,    -3.7905147961275440e-07,     1.8192487939531000e-09, } , 
      {    1.6882675549789880e+01,     4.0049789747147561e+03,    -1.4865478477830068e-04,    -1.1764217290142573e-07,     6.4212117621914698e-10, } , 
      {    2.0138486036604100e+01,     2.0871911289697960e+04,    -2.7666711553126790e-04,    -1.2833260545809968e-07,     5.8326479533244923e-10, } , 
      {    1.9238230100176679e+01,     1.2333670399338207e+04,     3.6376229840254941e-06,     1.3307670125322899e-07,    -5.9386282240150385e-10, } , 
      {    2.2494040586990906e+01,     2.9200602714321409e+04,    -1.2437470776894172e-04,     1.2238626869655504e-07,    -6.5271920328820160e-10, } , 
      {    2.4849595137377705e+01,     3.7529294138944853e+04,     2.7917699993384458e-05,     3.7310514285120976e-07,    -1.8887032019088525e-09, } , 
      {    2.7205149687764511e+01,     4.5857985563568305e+04,     1.8021010775571064e-04,     6.2382401700586448e-07,    -3.1246872005295034e-09, } , 
      {    1.0673142147200361e+01,    -1.1814552453581784e+04,    -8.8850062807348110e-04,    -1.1305917099625559e-06,     5.4727610615632624e-09, } , 
      {    1.3028696697587163e+01,    -3.4858610289583321e+03,    -7.3620822031115498e-04,    -8.7987283580790115e-07,     4.2367770629426115e-09, } , 
      {    1.2128440761159743e+01,    -1.2024101919318082e+04,    -4.5590348179586158e-04,    -6.1846352909657259e-07,     3.0596494452086590e-09, } , 
      {    1.5384251247973966e+01,     4.8428303956651198e+03,    -5.8391581254882874e-04,    -6.2915396165324643e-07,     3.0007930643219611e-09, } , 
      {    1.8640061734788191e+01,     2.1709762710648320e+04,    -7.1192814330179601e-04,    -6.3984439420992038e-07,     2.9419366834352633e-09, } , 
      {    1.4483995311546545e+01,    -3.6954104946946336e+03,    -3.0361107403353540e-04,    -3.6774465494191777e-07,     1.8236654465880079e-09, } , 
      {    1.7739805798360770e+01,     1.3171521820288568e+04,    -4.3162340478650261e-04,    -3.7843508749859172e-07,     1.7648090657013101e-09, } , 
      {    2.0995616285174989e+01,     3.0038454135271772e+04,    -5.5963573553946978e-04,    -3.8912552005526566e-07,     1.7059526848146124e-09, } , 
      {    1.6839549861933349e+01,     4.6332809299288165e+03,    -1.5131866627120919e-04,    -1.1702578078726307e-07,     5.8768144796735722e-10, } , 
      {    2.0095360348747569e+01,     2.1500213244912018e+04,    -2.7933099702417643e-04,    -1.2771621334393702e-07,     5.2882506708065947e-10, } , 
      {    2.3351170835561796e+01,     3.8367145559895216e+04,    -4.0734332777714365e-04,    -1.3840664590061097e-07,     4.6996868619396172e-10, } , 
      {    1.9195104412320148e+01,     1.2961972354552267e+04,     9.7374149111698154e-07,     1.3369309336739165e-07,    -6.4830255065329361e-10, } , 
      {    2.2450914899134375e+01,     2.9828904669535470e+04,    -1.2703858926185022e-04,     1.2300266081071770e-07,    -7.0715893153999136e-10, } , 
      {    2.1550658962706954e+01,     2.1290663779175717e+04,     1.5326614925344317e-04,     3.8441196752204639e-07,    -1.8842865492739443e-09, } , 
      {    2.4806469449521174e+01,     3.8157596094158915e+04,     2.5253818500475944e-05,     3.7372153496537244e-07,    -1.9431429301606422e-09, } , 
      {    2.3906213513093753e+01,     2.9619355203799169e+04,     3.0555855701576933e-04,     6.3513084167670100e-07,    -3.1202705478945952e-09, } , 
      {    2.7162023999907980e+01,     4.6486287518782367e+04,     1.7754622626280214e-04,     6.2444040912002716e-07,    -3.1791269287812930e-09, } , 
      {    2.9517578550294779e+01,     5.4814978943405818e+04,     3.2983863402512829e-04,     8.7515928327468188e-07,    -4.4151109274019443e-09, } , 
      {    8.2744619089570275e+00,    -1.9514941922991169e+04,    -1.0434569173287157e-03,    -1.3806941920030479e-06,     6.6543053319321240e-09, } , 
      {    1.0630016459343830e+01,    -1.1186250498367725e+04,    -8.9116450956638969e-04,    -1.1299753178483932e-06,     5.4183213333114731e-09, } , 
      {    9.7297605229164095e+00,    -1.9724491388727474e+04,    -6.1085977105109629e-04,    -8.6856601113706463e-07,     4.2411937155775202e-09, } , 
      {    1.2985571009730632e+01,    -2.8575590737442717e+03,    -7.3887210180406356e-04,    -8.7925644369373847e-07,     4.1823373346908223e-09, } , 
      {    1.2085315073303212e+01,    -1.1395799964104022e+04,    -4.5856736328877005e-04,    -6.1784713698240991e-07,     3.0052097169568689e-09, } , 
      {    1.5341125560117435e+01,     5.4711323508791802e+03,    -5.8657969404173733e-04,    -6.2853756953908375e-07,     2.9463533360701711e-09, } , 
      {    1.8596936046931660e+01,     2.2338064665862381e+04,    -7.1459202479470460e-04,    -6.3922800209575770e-07,     2.8874969551834732e-09, } , 
      {    1.4440869623690014e+01,    -3.0671085394805732e+03,    -3.0627495552644387e-04,    -3.6712826282775514e-07,     1.7692257183362183e-09, } , 
      {    1.7696680110504239e+01,     1.3799823775502628e+04,    -4.3428728627941109e-04,    -3.7781869538442909e-07,     1.7103693374495204e-09, } , 
      {    2.0952490597318459e+01,     3.0666756090485829e+04,    -5.6229961703237836e-04,    -3.8850912794110304e-07,     1.6515129565628228e-09, } , 
      {    1.3540613687262594e+01,    -1.1605349429840324e+04,    -2.5970217011150495e-05,    -1.0571895611642649e-07,     5.9209810060226500e-10, } , 
      {    1.6796424174076819e+01,     5.2615828851428769e+03,    -1.5398254776411772e-04,    -1.1640938867310041e-07,     5.3324171971556735e-10, } , 
      {    2.0052234660891038e+01,     2.2128515200126079e+04,    -2.8199487851708491e-04,    -1.2709982122977434e-07,     4.7438533882886960e-10, } , 
      {    2.3308045147705265e+01,     3.8995447515109277e+04,    -4.1000720927005213e-04,    -1.3779025378644829e-07,     4.1552895794217190e-10, } , 
      {    1.5896168237649396e+01,    -3.2766580052168747e+03,     1.2632219075117568e-04,     1.4499991803822825e-07,    -6.4388589801838583e-10, } , 
      {    1.9151978724463618e+01,     1.3590274309766326e+04,    -1.6901400017915301e-06,     1.3430948548155433e-07,    -7.0274227890508348e-10, } , 
      {    2.2407789211277844e+01,     3.0457206624749528e+04,    -1.2970247075475876e-04,     1.2361905292488038e-07,    -7.6159865979178123e-10, } , 
      {    2.5663599698092064e+01,     4.7324138939732729e+04,    -2.5771480150772595e-04,     1.1292862036820643e-07,    -8.2045504067847898e-10, } , 
      {    1.8251722788036197e+01,     5.0520334194065754e+03,     2.7861459851350189e-04,     3.9571879219288291e-07,    -1.8798698966390366e-09, } , 
      {    2.1507533274850424e+01,     2.1918965734389778e+04,     1.5060226776053464e-04,     3.8502835963620902e-07,    -1.9387262775257344e-09, } , 
      {    2.4763343761664643e+01,     3.8785898049372976e+04,     2.2589937007567433e-05,     3.7433792707953507e-07,    -1.9975826584124323e-09, } , 
      {    2.3863087825237223e+01,     3.0247657159013226e+04,     3.0289467552286085e-04,     6.3574723379086368e-07,    -3.1747102761463852e-09, } , 
      {    2.7118898312051449e+01,     4.7114589473996428e+04,     1.7488234476989361e-04,     6.2505680123418984e-07,    -3.2335666570330831e-09, } , 
      {    2.6218642375624029e+01,     3.8576348583636674e+04,     4.5518708328518703e-04,     8.8646610794551840e-07,    -4.4106942747670357e-09, } , 
      {    2.9474452862438248e+01,     5.5443280898619880e+04,     3.2717475253221982e-04,     8.7577567538884456e-07,    -4.4695506556537335e-09, } , 
      {    3.1830007412825054e+01,     6.3771972323243332e+04,     4.7946716029454600e-04,     1.1264945495434993e-06,    -5.7055346542743843e-09, } , 
      {    5.8757816707136943e+00,    -2.7215331392400560e+04,    -1.1984132065839506e-03,    -1.6307966740435400e-06,     7.8358496023009864e-09, } , 
      {    8.2313362211004968e+00,    -1.8886639967777108e+04,    -1.0461207988216243e-03,    -1.3800777998888852e-06,     6.5998656036803339e-09, } , 
      {    7.3310802846730763e+00,    -2.7424880858136865e+04,    -7.6581606030633089e-04,    -1.1186684931775567e-06,     5.4227379859463809e-09, } , 
      {    1.0586890771487299e+01,    -1.0557948543153663e+04,    -8.9382839105929817e-04,    -1.1293589257342305e-06,     5.3638816050596831e-09, } , 
      {    9.6866348350598788e+00,    -1.9096189433513413e+04,    -6.1352365254400477e-04,    -8.6794961902290195e-07,     4.1867539873257301e-09, } , 
      {    1.2942445321874102e+01,    -2.2292571185302113e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    1.6198255808688323e+01,     1.4637675196452990e+04,    -8.6954831404993920e-04,    -8.8933048413624974e-07,     4.0690412255523344e-09, } , 
      {    1.2042189385446681e+01,    -1.0767498008889961e+04,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {    1.5297999872260904e+01,     6.0994343060932406e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.8553810359075129e+01,     2.2966366621076442e+04,    -7.1725590628761307e-04,    -6.3861160998159502e-07,     2.8330572269316835e-09, } , 
      {    1.1141933449019261e+01,    -1.9305738899249714e+04,    -1.8092650626638519e-04,    -3.5582143815691857e-07,     1.7736423709711261e-09, } , 
      {    1.4397743935833484e+01,    -2.4388065842665128e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    1.7653554422647709e+01,     1.4428125730716689e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0909364909461928e+01,     3.1295058045699890e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    1.3497487999406063e+01,    -1.0977047474626264e+04,    -2.8634098504059006e-05,    -1.0510256400226382e-07,     5.3765837235047523e-10, } , 
      {    1.6753298486220288e+01,     5.8898848403569373e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.0009108973034508e+01,     2.2756817155340141e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.3264919459848734e+01,     3.9623749470323339e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    1.5853042549792866e+01,    -2.6483560500028143e+03,     1.2365830925826718e-04,     1.4561631015239090e-07,    -6.9832562627017560e-10, } , 
      {    1.9108853036607087e+01,     1.4218576264980387e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.2364663523421314e+01,     3.1085508579963589e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5620474010235533e+01,     4.7952440894946791e+04,    -2.6037868300063448e-04,     1.1354501248236910e-07,    -8.7489476893026875e-10, } , 
      {    1.8208597100179666e+01,     5.6803353746206358e+03,     2.7595071702059336e-04,     3.9633518430704559e-07,    -1.9343096248908262e-09, } , 
      {    2.1464407586993893e+01,     2.2547267689603839e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {    2.4720218073808113e+01,     3.9414200004587037e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.7976028560622339e+01,     5.6281132319570243e+04,    -1.0808627523830830e-04,     3.6426388663702380e-07,    -2.1108787675509198e-09, } , 
      {    2.3819962137380692e+01,     3.0875959114227288e+04,     3.0023079402995232e-04,     6.3636362590502636e-07,    -3.2291500043981749e-09, } , 
      {    2.7075772624194919e+01,     4.7742891429210489e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    2.6175516687767498e+01,     3.9204650538850736e+04,     4.5252320179227850e-04,     8.8708250005968108e-07,    -4.4651340030188257e-09, } , 
      {    2.9431327174581718e+01,     5.6071582853833941e+04,     3.2451087103931128e-04,     8.7639206750300724e-07,    -4.5239903839055236e-09, } , 
      {    2.8531071238154297e+01,     4.7533341963474188e+04,     6.0481560955460468e-04,     1.1378013742143358e-06,    -5.7011180016394766e-09, } , 
      {    3.1786881724968524e+01,     6.4400274278457393e+04,     4.7680327880163747e-04,     1.1271109416576620e-06,    -5.7599743825261744e-09, } , 
      {    3.4142436275355330e+01,     7.2728965703080845e+04,     6.2909568656396354e-04,     1.3778298158123167e-06,    -6.9959583811468252e-09, } , 
      {    1.0543765083630769e+01,    -9.9296465879396019e+03,    -8.9649227255220664e-04,    -1.1287425336200678e-06,     5.3094418768078930e-09, } , 
      {    1.2899319634017571e+01,    -1.6009551633161509e+03,    -7.4419986478988052e-04,    -8.7802365946541311e-07,     4.0734578781872421e-09, } , 
      {    1.5254874184404374e+01,     6.7277362613073010e+03,    -5.9190745702755428e-04,    -6.2730478531075839e-07,     2.8374738795665917e-09, } , 
      {    1.8510684671218598e+01,     2.3594668576290504e+04,    -7.1991978778052155e-04,    -6.3799521786743234e-07,     2.7786174986798939e-09, } , 
      {    1.4354618247976953e+01,    -1.8105046290524524e+03,    -3.1160271851226094e-04,    -3.6589547859942978e-07,     1.6603462618326385e-09, } , 
      {    1.7610428734791178e+01,     1.5056427685930750e+04,    -4.3961504926522815e-04,    -3.7658591115610373e-07,     1.6014898809459407e-09, } , 
      {    2.0866239221605397e+01,     3.1923360000913952e+04,    -5.6762738001819532e-04,    -3.8727634371277768e-07,     1.5426335000592430e-09, } , 
      {    1.6710172798363757e+01,     6.5181867955709977e+03,    -1.5931031074993473e-04,    -1.1517660444477507e-07,     4.2436226321198777e-10, } , 
      {    1.9965983285177977e+01,     2.3385119110554202e+04,    -2.8732264150290197e-04,    -1.2586703700144904e-07,     3.6550588232529002e-10, } , 
      {    2.3221793771992203e+01,     4.0252051425537400e+04,    -4.1533497225586919e-04,    -1.3655746955812298e-07,     3.0664950143859227e-10, } , 
      {    1.9065727348750556e+01,     1.4846878220194449e+04,    -7.0179029876085535e-06,     1.3554226970987963e-07,    -8.1162173540866301e-10, } , 
      {    2.2321537835564783e+01,     3.1713810535177650e+04,    -1.3503023374057576e-04,     1.2485183715320568e-07,    -8.7047811629536076e-10, } , 
      {    2.5577348322379002e+01,     4.8580742850160852e+04,    -2.6304256449354301e-04,     1.1416140459653176e-07,    -9.2933449718205851e-10, } , 
      {    1.8165471412323136e+01,     6.3086373298346962e+03,     2.7328683552768483e-04,     3.9695157642120827e-07,    -1.9887493531426159e-09, } , 
      {    2.1421281899137362e+01,     2.3175569644817901e+04,     1.4527450477471764e-04,     3.8626114386453438e-07,    -2.0476057340293137e-09, } , 
      {    2.4677092385951582e+01,     4.0042501959801099e+04,     1.7262174021750411e-05,     3.7557071130786043e-07,    -2.1064621149160116e-09, } , 
      {    2.3776836449524161e+01,     3.1504261069441349e+04,     2.9756691253704379e-04,     6.3698001801918904e-07,    -3.2835897326499646e-09, } , 
      {    2.7032646936338388e+01,     4.8371193384424550e+04,     1.6955458178407660e-04,     6.2628958546251520e-07,    -3.3424461135366624e-09, } , 
      {    2.9388201486725187e+01,     5.6699884809048002e+04,     3.2184698954640275e-04,     8.7700845961716992e-07,    -4.5784301121573137e-09, } , 
      {    3.1743756037111993e+01,     6.5028576233671454e+04,     4.7413939730872893e-04,     1.1277273337718246e-06,    -5.8144141107779645e-09, } , 
      {    1.2856193946161040e+01,    -9.7265320810209050e+02,    -7.4686374628278910e-04,    -8.7740726735125043e-07,     4.0190181499354529e-09, } , 
      {    1.1955938009733620e+01,    -9.5108940984618403e+03,    -4.6655900776749559e-04,    -6.1599796063992187e-07,     2.8418905322014995e-09, } , 
      {    1.5211748496547843e+01,     7.3560382165213614e+03,    -5.9457133852046287e-04,    -6.2668839319659571e-07,     2.7830341513148017e-09, } , 
      {    1.4311492560120422e+01,    -1.1822026738383920e+03,    -3.1426660000516941e-04,    -3.6527908648526715e-07,     1.6059065335808489e-09, } , 
      {    1.7567303046934647e+01,     1.5684729641144810e+04,    -4.4227893075813663e-04,    -3.7596951904194110e-07,     1.5470501526941510e-09, } , 
      {    1.6667047110507227e+01,     7.1464887507850581e+03,    -1.6197419224284326e-04,    -1.1456021233061241e-07,     3.6992253496019800e-10, } , 
      {    1.9922857597321446e+01,     2.4013421065768263e+04,    -2.8998652299581045e-04,    -1.2525064488728636e-07,     3.1106615407350025e-10, } , 
      {    1.9022601660894026e+01,     1.5475180175408508e+04,    -9.6817844805170651e-06,     1.3615866182404231e-07,    -8.6606146366045288e-10, } , 
      {    2.2278412147708252e+01,     3.2342112490391708e+04,    -1.3769411523348429e-04,     1.2546822926736836e-07,    -9.2491784454715063e-10, } , 
      {    2.4633966698095051e+01,     4.0670803915015160e+04,     1.4598292528841899e-05,     3.7618710342202306e-07,    -2.1609018431678017e-09, } , 
      {    1.7524177359078116e+01,     1.6313031596358869e+04,    -4.4494281225104516e-04,    -3.7535312692777842e-07,     1.4926104244423612e-09, } , 
      {    1.9879731909464915e+01,     2.4641723020982321e+04,    -2.9265040448871898e-04,    -1.2463425277312370e-07,     2.5662642582171044e-10, } , 
      {    2.2235286459851721e+01,     3.2970414445605769e+04,    -1.4035799672639280e-04,     1.2608462138153102e-07,    -9.7935757279894050e-10, } , 
      {    2.2981398285373906e+01,     2.0314597010065409e+04,    -4.6205111135551091e-04,    -3.4799261835299362e-07,     1.6152391969422074e-09, } , 
      {    2.0582718047130570e+01,     1.2614207540656020e+04,    -6.1700740061074557e-04,    -5.9809510039348572e-07,     2.7967834673110686e-09, } , 
      {    2.2938272597517376e+01,     2.0942898965279470e+04,    -4.6471499284841944e-04,    -3.4737622623883094e-07,     1.5607994686904175e-09, } , 
      {    2.2038016661089955e+01,     1.2404658074919716e+04,    -1.8441025433312602e-04,    -8.5966919527502303e-08,     3.8367185095646439e-10, } , 
      {    2.5293827147904175e+01,     2.9271590389902914e+04,    -3.1242258508609326e-04,    -9.6657352084176251e-08,     3.2481547006976674e-10, } , 
      {    2.7649381698290981e+01,     3.7600281814526366e+04,    -1.6013017732376705e-04,     1.5406152207047847e-07,    -9.1116852855088403e-10, } , 
      {    3.0004936248677780e+01,     4.5928973239149818e+04,    -7.8377695614408829e-06,     4.0478039622513321e-07,    -2.1471525271715348e-09, } , 
      {    1.8184037808887240e+01,     4.9138180712466283e+03,    -7.7196368986598028e-04,    -8.4819758243397776e-07,     3.9783277376799301e-09, } , 
      {    2.0539592359274039e+01,     1.3242509495870079e+04,    -6.1967128210365415e-04,    -5.9747870827932304e-07,     2.7423437390592785e-09, } , 
      {    1.9639336422846618e+01,     4.7042686055103268e+03,    -3.3936654358836070e-04,    -3.3606940156799437e-07,     1.5652161213253255e-09, } , 
      {    2.2895146909660845e+01,     2.1571200920493528e+04,    -4.6737887434132792e-04,    -3.4675983412466832e-07,     1.5063597404386279e-09, } , 
      {    2.1994890973233424e+01,     1.3032960030133778e+04,    -1.8707413582603455e-04,    -8.5350527413339637e-08,     3.2923212270467463e-10, } , 
      {    2.5250701460047644e+01,     2.9899892345116976e+04,    -3.1508646657900174e-04,    -9.6040959970013584e-08,     2.7037574181797698e-10, } , 
      {    2.4350445523620223e+01,     2.1361651454757226e+04,    -3.4781728063708359e-05,     1.6536834674131510e-07,    -9.0675187591597625e-10, } , 
      {    2.7606256010434450e+01,     3.8228583769740428e+04,    -1.6279405881667558e-04,     1.5467791418464115e-07,    -9.6560825680267380e-10, } , 
      {    2.6706000074007029e+01,     2.9690342879380674e+04,     1.1751067969861783e-04,     4.1608722089596979e-07,    -2.1427358745366274e-09, } , 
      {    2.9961810560821249e+01,     4.6557275194363880e+04,    -1.0501651054349395e-05,     4.0539678833929584e-07,    -2.2015922554233248e-09, } , 
      {    1.8140912121030709e+01,     5.5421200264606887e+03,    -7.7462757135888876e-04,    -8.4758119031981508e-07,     3.9238880094281401e-09, } , 
      {    1.7240656184603289e+01,    -2.9961208638990611e+03,    -4.9432283284359547e-04,    -5.8617188360848641e-07,     2.7467603916941863e-09, } , 
      {    2.0496466671417508e+01,     1.3870811451084141e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    2.3752277158231735e+01,     3.0737743766067339e+04,    -7.5034749434952979e-04,    -6.0755274872183430e-07,     2.6290476299207914e-09, } , 
      {    1.9596210734990088e+01,     5.3325705607243872e+03,    -3.4203042508126923e-04,    -3.3545300945383169e-07,     1.5107763930735356e-09, } , 
      {    2.2852021221804314e+01,     2.2199502875707589e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    2.1951765285376894e+01,     1.3661261985347837e+04,    -1.8973801731894305e-04,    -8.4734135299176970e-08,     2.7479239445288481e-10, } , 
      {    2.5207575772191113e+01,     3.0528194300331037e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {    2.4307319835763693e+01,     2.1989953409971287e+04,    -3.7445609556616870e-05,     1.6598473885547775e-07,    -9.6119160416776602e-10, } , 
      {    2.7563130322577919e+01,     3.8856885724954489e+04,    -1.6545794030958409e-04,     1.5529430629880380e-07,    -1.0200479850544637e-09, } , 
      {    2.6662874386150499e+01,     3.0318644834594736e+04,     1.1484679820570931e-04,     4.1670361301013247e-07,    -2.1971756027884171e-09, } , 
      {    2.9918684872964718e+01,     4.7185577149577941e+04,    -1.3165532547257906e-05,     4.0601318045345852e-07,    -2.2560319836751145e-09, } , 
      {    3.2274239423351524e+01,     5.5514268574201385e+04,     1.3912687521506827e-04,     6.5673205460811324e-07,    -3.4920159822957653e-09, } , 
      {    1.8097786433174178e+01,     6.1704219816747491e+03,    -7.7729145285179724e-04,    -8.4696479820565240e-07,     3.8694482811763500e-09, } , 
      {    2.0453340983560977e+01,     1.4499113406298202e+04,    -6.2499904508947111e-04,    -5.9624592405099768e-07,     2.6334642825556992e-09, } , 
      {    2.2808895533947783e+01,     2.2827804830921650e+04,    -4.7270663732714498e-04,    -3.4552704989634296e-07,     1.3974802839350481e-09, } , 
      {    2.1908639597520363e+01,     1.4289563940561897e+04,    -1.9240189881185156e-04,    -8.4117743185014303e-08,     2.2035266620109502e-10, } , 
      {    2.5164450084334582e+01,     3.1156496255545098e+04,    -3.2041422956481880e-04,    -9.4808175741688251e-08,     1.6149628531439737e-10, } , 
      {    2.4264194147907162e+01,     2.2618255365185349e+04,    -4.0109491049525381e-05,     1.6660113096964040e-07,    -1.0156313324195559e-09, } , 
      {    2.7520004634721388e+01,     3.9485187680168550e+04,    -1.6812182180249259e-04,     1.5591069841296646e-07,    -1.0744877133062535e-09, } , 
      {    2.9875559185108187e+01,     4.7813879104792002e+04,    -1.5829414040166417e-05,     4.0662957256762120e-07,    -2.3104717119269042e-09, } , 
      {    3.2231113735494993e+01,     5.6142570529415447e+04,     1.3646299372215977e-04,     6.5734844672227592e-07,    -3.5464557105475550e-09, } , 
      {    2.5121324396478052e+01,     3.1784798210759160e+04,    -3.2307811105772728e-04,    -9.4191783627525584e-08,     1.0705655706260757e-10, } , 
      {    2.7476878946864858e+01,     4.0113489635382612e+04,    -1.7078570329540112e-04,     1.5652709052712914e-07,    -1.1289274415580432e-09, } , 
      {    2.8222990772387035e+01,     2.7457672199842244e+04,    -4.9247881792451921e-04,    -3.1755014920739559e-07,     1.4656693281831045e-09, } , 
      {    3.0578545322773842e+01,     3.5786363624465695e+04,    -3.4018641016219303e-04,    -6.6831275052740827e-08,     2.2968532956245358e-10, } , 
      {    2.5824310534143706e+01,     1.9757282730432857e+04,    -6.4743510717975392e-04,    -5.6765263124788768e-07,     2.6472135985519657e-09, } , 
      {    2.8179865084530505e+01,     2.8085974155056305e+04,    -4.9514269941742768e-04,    -3.1693375709323291e-07,     1.4112295999313146e-09, } , 
      {    3.0535419634917311e+01,     3.6414665579679749e+04,    -3.4285029165510156e-04,    -6.6214882938578160e-08,     1.7524560131066379e-10, } , 
      {    3.2890974185304110e+01,     4.4743357004303201e+04,    -1.9055788389277535e-04,     1.8450399121607656e-07,    -1.0607383973099870e-09, } , 
      {    3.5246528735690916e+01,     5.3072048428926653e+04,    -3.8265476130449168e-05,     4.3522286537073125e-07,    -2.2967223959306377e-09, } , 
      {    2.3425630295900369e+01,     1.2056893261023464e+04,    -8.0239139643498852e-04,    -8.1775511328837972e-07,     3.8287578689208264e-09, } , 
      {    2.5781184846287175e+01,     2.0385584685646914e+04,    -6.5009898867266240e-04,    -5.6703623913372500e-07,     2.5927738703001760e-09, } , 
      {    2.4880928909859755e+01,     1.1847343795287163e+04,    -3.6979425015736905e-04,    -3.0562693242239628e-07,     1.4156462525662224e-09, } , 
      {    2.8136739396673974e+01,     2.8714276110270366e+04,    -4.9780658091033616e-04,    -3.1631736497907023e-07,     1.3567898716795247e-09, } , 
      {    2.7236483460246554e+01,     2.0176035219910613e+04,    -2.1750184239504282e-04,    -5.4908058267741579e-08,     1.7966225394557162e-10, } , 
      {    3.0492293947060780e+01,     3.7042967534893811e+04,    -3.4551417314801009e-04,    -6.5598490824415507e-08,     1.2080587305887397e-10, } , 
      {    2.9592038010633360e+01,     2.8504726644534065e+04,    -6.5209434632716650e-05,     1.9581081588691313e-07,    -1.0563217446750793e-09, } , 
      {    3.2847848497447579e+01,     4.5371658959517263e+04,    -1.9322176538568385e-04,     1.8512038333023921e-07,    -1.1151781255617769e-09, } , 
      {    3.5203403047834385e+01,     5.3700350384140715e+04,    -4.0929357623357686e-05,     4.3583925748489393e-07,    -2.3511621241824273e-09, } , 
      {    3.7558957598221191e+01,     6.2029041808764159e+04,     1.1136305013896850e-04,     6.8655813163954860e-07,    -3.5871461228030782e-09, } , 
      {    2.3382504608043838e+01,     1.2685195216237524e+04,    -8.0505527792789711e-04,    -8.1713872117421704e-07,     3.7743181406690372e-09, } , 
      {    2.2482248671616418e+01,     4.1469543258777749e+03,    -5.2475053941260371e-04,    -5.5572941446288837e-07,     2.5971905229350834e-09, } , 
      {    2.5738059158430644e+01,     2.1013886640860976e+04,    -6.5276287016557098e-04,    -5.6641984701956232e-07,     2.5383341420483859e-09, } , 
      {    2.8993869645244864e+01,     3.7880818955844174e+04,    -7.8077520091853825e-04,    -5.7711027957623616e-07,     2.4794777611616881e-09, } , 
      {    2.4837803222003224e+01,     1.2475645750501222e+04,    -3.7245813165027753e-04,    -3.0501054030823365e-07,     1.3612065243144327e-09, } , 
      {    2.8093613708817443e+01,     2.9342578065484424e+04,    -5.0047046240324475e-04,    -3.1570097286490760e-07,     1.3023501434277351e-09, } , 
      {    3.1349424195631670e+01,     4.6209510380467618e+04,    -6.2848279315621202e-04,    -3.2639140542158149e-07,     1.2434937625410374e-09, } , 
      {    2.7193357772390023e+01,     2.0804337175124674e+04,    -2.2016572388795135e-04,    -5.4291666153578919e-08,     1.2522252569378183e-10, } , 
      {    3.0449168259204249e+01,     3.7671269490107872e+04,    -3.4817805464091857e-04,    -6.4982098710252840e-08,     6.6366144807084178e-11, } , 
      {    2.9548912322776829e+01,     2.9133028599748122e+04,    -6.7873316125625154e-05,     1.9642720800107581e-07,    -1.1107614729268689e-09, } , 
      {    3.2804722809591048e+01,     4.5999960914731324e+04,    -1.9588564687859238e-04,     1.8573677544440189e-07,    -1.1696178538135666e-09, } , 
      {    3.5160277359977854e+01,     5.4328652339354776e+04,    -4.3593239116266197e-05,     4.3645564959905656e-07,    -2.4056018524342174e-09, } , 
      {    3.7515831910364660e+01,     6.2657343763978221e+04,     1.0869916864605999e-04,     6.8717452375371128e-07,    -3.6415858510548682e-09, } , 
      {    3.9871386460751459e+01,     7.0986035188601672e+04,     2.6099157640838616e-04,     9.3789339790836600e-07,    -4.8775698496755187e-09, } , 
      {    2.0983824369800509e+01,     4.9848057468281331e+03,    -9.6001156718313171e-04,    -1.0672412032147091e-06,     4.9558624110378979e-09, } , 
      {    2.3339378920187308e+01,     1.3313497171451585e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {    2.6595189407001534e+01,     3.0180429486434783e+04,    -9.3573149017377286e-04,    -8.2721276161672820e-07,     3.6610220315305492e-09, } , 
      {    2.2439122983759887e+01,     4.7752562810918353e+03,    -5.2741442090551219e-04,    -5.5511302234872569e-07,     2.5427507946832937e-09, } , 
      {    2.5694933470574114e+01,     2.1642188596075037e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    2.8950743957388333e+01,     3.8509120911058235e+04,    -7.8343908241144673e-04,    -5.7649388746207348e-07,     2.4250380329098984e-09, } , 
      {    2.4794677534146693e+01,     1.3103947705715284e+04,    -3.7512201314318606e-04,    -3.0439414819407097e-07,     1.3067667960626429e-09, } , 
      {    2.8050488020960913e+01,     2.9970880020698485e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    3.1306298507775139e+01,     4.6837812335681680e+04,    -6.3114667464912050e-04,    -3.2577501330741881e-07,     1.1890540342892476e-09, } , 
      {    2.7150232084533492e+01,     2.1432639130338735e+04,    -2.2282960538085985e-04,    -5.3675274039416252e-08,     7.0782797441992036e-11, } , 
      {    3.0406042571347719e+01,     3.8299571445321933e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.3661853058161945e+01,     5.5166503760305131e+04,    -4.7885426688679431e-04,    -7.5056139152764095e-08,    -4.6929964331403261e-11, } , 
      {    2.6249976148106072e+01,     1.2894398239978982e+04,     5.7475133134433546e-05,     2.0773403267191242e-07,    -1.1063448202919609e-09, } , 
      {    2.9505786634920298e+01,     2.9761330554962184e+04,    -7.0537197618533671e-05,     1.9704360011523847e-07,    -1.1652012011786588e-09, } , 
      {    3.2761597121734518e+01,     4.6628262869945385e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    3.5117151672121324e+01,     5.4956954294568837e+04,    -4.6257120609174708e-05,     4.3707204171321924e-07,    -2.4600415806860071e-09, } , 
      {    3.4216895735693903e+01,     4.6418713404209084e+04,     2.3404761790611869e-04,     6.9848134842454791e-07,    -3.6371691984199605e-09, } , 
      {    3.7472706222508130e+01,     6.3285645719192282e+04,     1.0603528715315147e-04,     6.8779091586787396e-07,    -3.6960255793066579e-09, } , 
      {    3.9828260772894929e+01,     7.1614337143815734e+04,     2.5832769491547763e-04,     9.3850979002252868e-07,    -4.9320095779273087e-09, } , 
      {    4.2183815323281728e+01,     7.9943028568439186e+04,     4.1062010267780386e-04,     1.1892286641771834e-06,    -6.1679935765479596e-09, } , 
      {    2.3296253232330777e+01,     1.3941799126665646e+04,    -8.1038304091371406e-04,    -8.1590593694589168e-07,     3.6654386841654574e-09, } , 
      {    2.5651807782717583e+01,     2.2270490551289098e+04,    -6.5809063315138794e-04,    -5.6518706279123696e-07,     2.4294546855448066e-09, } , 
      {    2.4751551846290162e+01,     1.3732249660929345e+04,    -3.7778589463609459e-04,    -3.0377775607990829e-07,     1.2523270678108530e-09, } , 
      {    2.8007362333104382e+01,     3.0599181975912546e+04,    -5.0579822538906170e-04,    -3.1446818863658224e-07,     1.1934706869241553e-09, } , 
      {    2.7107106396676961e+01,     2.2060941085552797e+04,    -2.2549348687376836e-04,    -5.3058881925253585e-08,     1.6343069190202239e-11, } , 
      {    3.0362916883491188e+01,     3.8927873400535995e+04,    -3.5350581762673563e-04,    -6.3749314481927506e-08,    -4.2513311696495410e-11, } , 
      {    2.9462660947063767e+01,     3.0389632510176245e+04,    -7.3201079111442189e-05,     1.9765999222940112e-07,    -1.2196409294304487e-09, } , 
      {    3.2718471433877987e+01,     4.7256564825159447e+04,    -2.0121340986440939e-04,     1.8696955967272720e-07,    -1.2784973103171463e-09, } , 
      {    3.5074025984264793e+01,     5.5585256249782899e+04,    -4.8921002102083218e-05,     4.3768843382738192e-07,    -2.5144813089377967e-09, } , 
      {    3.7429580534651599e+01,     6.3913947674406343e+04,     1.0337140566024296e-04,     6.8840730798203664e-07,    -3.7504653075584480e-09, } , 
      {    2.3253127544474246e+01,     1.4570101081879706e+04,    -8.1304692240662265e-04,    -8.1528954483172900e-07,     3.6109989559136673e-09, } , 
      {    2.5608682094861052e+01,     2.2898792506503160e+04,    -6.6075451464429652e-04,    -5.6457067067707428e-07,     2.3750149572930165e-09, } , 
      {    2.7964236645247851e+01,     3.1227483931126604e+04,    -5.0846210688197029e-04,    -3.1385179652241961e-07,     1.1390309586723657e-09, } , 
      {    3.0319791195634657e+01,     3.9556175355750056e+04,    -3.5616969911964411e-04,    -6.3132922367764840e-08,    -9.6953039948285207e-11, } , 
      {    2.8580971407200444e+01,     2.0456572361228420e+04,    -8.3814686598981394e-04,    -7.8607985991445621e-07,     3.5703085436581438e-09, } , 
      {    3.0936525957587243e+01,     2.8785263785851872e+04,    -6.8585445822748781e-04,    -5.3536098575980149e-07,     2.3343245450374929e-09, } , 
      {    3.3292080507974049e+01,     3.7113955210475324e+04,    -5.3356205046516158e-04,    -2.8464211160514683e-07,     1.0983405464168423e-09, } , 
      {    3.5647635058360855e+01,     4.5442646635098768e+04,    -3.8126964270283534e-04,    -3.3923237450492096e-08,    -1.3764345220380860e-10, } , 
      {    2.8537845719343913e+01,     2.1084874316442481e+04,    -8.4081074748272241e-04,    -7.8546346780029353e-07,     3.5158688154063541e-09, } , 
      {    3.0893400269730712e+01,     2.9413565741065933e+04,    -6.8851833972039629e-04,    -5.3474459364563881e-07,     2.2798848167857033e-09, } , 
      {    2.9993144333303292e+01,     2.0875324850706183e+04,    -4.0821360120510284e-04,    -2.7333528693431020e-07,     1.1027571990517501e-09, } , 
      {    3.3248954820117518e+01,     3.7742257165689385e+04,    -5.3622593195807005e-04,    -2.8402571949098415e-07,     1.0439008181650524e-09, } , 
      {    3.2348698883690091e+01,     2.9204016275329632e+04,    -2.5592119344277665e-04,    -2.2616412779655481e-08,    -1.3322679956890074e-10, } , 
      {    3.5604509370504324e+01,     4.6070948590312830e+04,    -3.8393352419574387e-04,    -3.3306845336329429e-08,    -1.9208318045559839e-10, } , 
      {    3.7960063920891123e+01,     5.4399640014936282e+04,    -2.3164111643341769e-04,     2.1741202881832529e-07,    -1.4280671790762492e-09, } , 
      {    3.0850274581874181e+01,     3.0041867696279995e+04,    -6.9118222121330476e-04,    -5.3412820153147613e-07,     2.2254450885339136e-09, } , 
      {    3.3205829132260988e+01,     3.8370559120903446e+04,    -5.3888981345097853e-04,    -2.8340932737682147e-07,     9.8946108991326257e-10, } , 
      {    3.5561383682647794e+01,     4.6699250545526891e+04,    -3.8659740568865240e-04,    -3.2690453222166762e-08,    -2.4652290870738820e-10, } , 
      {    3.3822563894213580e+01,     2.7599647551005255e+04,    -8.6857457255882218e-04,    -7.5563739076885818e-07,     3.4207386748990409e-09, } , 
      {    3.6178118444600379e+01,     3.5928338975628707e+04,    -7.1628216479649595e-04,    -5.0491851661420346e-07,     2.1847546762783900e-09, } , 
      {    3.8533672994987185e+01,     4.4257030400252159e+04,    -5.6398975703416982e-04,    -2.5419964245954868e-07,     9.4877067765773920e-10, } , 
      {    4.0889227545373991e+01,     5.2585721824875603e+04,    -4.1169734927184369e-04,    -3.4807683048939860e-09,    -2.8721332096291158e-10, } , 
      {    4.3244782095760790e+01,     6.0914413249499055e+04,    -2.5940494150951751e-04,     2.4723810584976075e-07,    -1.5231973195835625e-09, } , 
      {    3.3779438206357050e+01,     2.8227949506219316e+04,    -8.7123845405173077e-04,    -7.5502099865469550e-07,     3.3662989466472508e-09, } , 
      {    3.6134992756743848e+01,     3.6556640930842768e+04,    -7.1894604628940453e-04,    -5.0430212450004078e-07,     2.1303149480266000e-09, } , 
      {    3.5234736820316428e+01,     2.8018400040483019e+04,    -4.3864130777411113e-04,    -2.4289281778871211e-07,     9.5318733029264719e-10, } , 
      {    3.8490547307130655e+01,     4.4885332355466220e+04,    -5.6665363852707840e-04,    -2.5358325034538606e-07,     8.9433094940594943e-10, } , 
      {    3.7590291370703234e+01,     3.6347091465106467e+04,    -2.8634890001178495e-04,     7.8260563659426265e-09,    -2.8279666832800370e-10, } , 
      {    4.0846101857517461e+01,     5.3214023780089665e+04,    -4.1436123076475217e-04,    -2.8643761907313221e-09,    -3.4165304921470135e-10, } , 
      {    4.3201656407904260e+01,     6.1542715204713117e+04,    -2.6206882300242599e-04,     2.4785449796392338e-07,    -1.5776370478353521e-09, } , 
      {    4.5557210958291058e+01,     6.9871406629336561e+04,    -1.0977641524009982e-04,     4.9857337211857810e-07,    -2.8136210464560034e-09, } , 
      {    3.1380757968113713e+01,     2.0527560036809926e+04,    -1.0261947433069653e-03,    -1.0051234806951874e-06,     4.5478432170161120e-09, } , 
      {    3.3736312518500519e+01,     2.8856251461433378e+04,    -8.7390233554463924e-04,    -7.5440460654053282e-07,     3.3118592183954611e-09, } , 
      {    3.2836056582073091e+01,     2.0318010571073632e+04,    -5.9359759702934584e-04,    -4.9299529982920415e-07,     2.1347316006615081e-09, } , 
      {    3.6091867068887318e+01,     3.7184942886056830e+04,    -7.2160992778231301e-04,    -5.0368573238587810e-07,     2.0758752197748103e-09, } , 
      {    3.5191611132459897e+01,     2.8646701995697080e+04,    -4.4130518926701966e-04,    -2.4227642567454943e-07,     8.9874760204085732e-10, } , 
      {    3.8447421619274124e+01,     4.5513634310680281e+04,    -5.6931752001998688e-04,    -2.5296685823122338e-07,     8.3989122115415967e-10, } , 
      {    3.7547165682846703e+01,     3.6975393420320528e+04,    -2.8901278150469348e-04,     8.4424484801052899e-09,    -3.3723639657979351e-10, } , 
      {    4.0802976169660930e+01,     5.3842325735303726e+04,    -4.1702511225766070e-04,    -2.2479840765686579e-09,    -3.9609277746649116e-10, } , 
      {    3.9902720233233502e+01,     4.5304084844943980e+04,    -1.3672037374236730e-04,     2.5916132263476001e-07,    -1.5732203952004443e-09, } , 
      {    4.3158530720047729e+01,     6.2171017159927178e+04,    -2.6473270449533452e-04,     2.4847089007808606e-07,    -1.6320767760871420e-09, } , 
      {    4.5514085270434528e+01,     7.0499708584550623e+04,    -1.1244029673300834e-04,     4.9918976423274078e-07,    -2.8680607747077930e-09, } , 
      {    4.7869639820821334e+01,     7.8828400009174075e+04,     3.9852111029317844e-05,     7.4990863838739550e-07,    -4.1040447733284439e-09, } , 
      {    3.3693186830643988e+01,     2.9484553416647439e+04,    -8.7656621703754772e-04,    -7.5378821442637014e-07,     3.2574194901436715e-09, } , 
      {    3.6048741381030787e+01,     3.7813244841270891e+04,    -7.2427380927522148e-04,    -5.0306934027171542e-07,     2.0214354915230206e-09, } , 
      {    3.8404295931417593e+01,     4.6141936265894343e+04,    -5.7198140151289536e-04,    -2.5235046611706070e-07,     7.8545149290236990e-10, } , 
      {    4.0759850481804399e+01,     5.4470627690517787e+04,    -4.1968899375056923e-04,    -1.6315919624059938e-09,    -4.5053250571828098e-10, } , 
      {    4.3115405032191198e+01,     6.2799319115141239e+04,    -2.6739658598824305e-04,     2.4908728219224874e-07,    -1.6865165043389319e-09, } , 
      {    4.6531926355057060e+01,     5.2099395220824568e+04,    -7.8512922241323819e-04,    -4.4218440198051923e-07,     1.7222957540048150e-09, } , 
      {    4.8887480905443866e+01,     6.0428086645448013e+04,    -6.3283681465091206e-04,    -1.9146552782586449e-07,     4.8631175538416409e-10, } , 
      {    4.1777691566426931e+01,     3.6070314326791726e+04,    -1.0923779194307991e-03,    -9.4300575817566599e-07,     4.1398240229943268e-09, } , 
      {    4.4133246116813730e+01,     4.4399005751415178e+04,    -9.4008551166847290e-04,    -6.9228688402101127e-07,     2.9038400243736760e-09, } , 
      {    4.6488800667200529e+01,     5.2727697176038630e+04,    -7.8779310390614666e-04,    -4.4156800986635655e-07,     1.6678560257530252e-09, } , 
      {    4.8844355217587335e+01,     6.1056388600662074e+04,    -6.3550069614382054e-04,    -1.9084913571170184e-07,     4.3187202713237433e-10, } , 
      {    5.1199909767974141e+01,     6.9385080025285526e+04,    -4.8320828838149436e-04,     5.9869738442952884e-08,    -8.0411197148827650e-10, } , 
    } ,  
  } ;  

  static const double cper[2][MAX2] = {  
    {  
       1.4251457573243711e+01, 
       7.0629899999995374e+00, 
       1.1429948417379998e+00, 
       8.7610901832922305e-01, 
       7.8834808167402515e-01, 
       7.3956629204145363e-01, 
       8.2163174079366497e-01, 
       5.6494697986916509e-01, 
       6.3883742809878896e-01, 
       6.4374935185772142e-01, 
       4.4531067486023307e-01, 
       4.9329999999998347e-01, 
       4.9141000000001916e-01, 
       3.2460110501742634e-01, 
       3.6059999999998649e-01, 
       3.4352576250971745e-01, 
       2.8287457443256769e-01, 
       2.2664304493312151e-01, 
       2.8521515919988760e-01, 
       3.0155682206429113e-01, 
       2.4515729711994211e-01, 
       1.6423577186510677e-01, 
       1.6706959796702761e-01, 
       1.8108346839433509e-01, 
       1.8105721584359097e-01, 
       1.6443067841638262e-01, 
       1.6305920449678701e-01, 
       2.1109035099372481e-01, 
       1.5712074733991802e-01, 
       1.9446267060830916e-01, 
       1.5130313162744510e-01, 
       1.8463096480431115e-01, 
       1.8253758685858937e-01, 
       1.7511173403858782e-01, 
       1.3698478142959328e-01, 
       1.3686130489957518e-01, 
       1.6492075407333007e-01, 
       1.6083728625006383e-01, 
       1.5379311894940678e-01, 
       1.4360786177981982e-01, 
       1.3600276545444179e-01, 
       1.3427285125047461e-01, 
       1.3124068032388905e-01, 
       1.0499695279212787e-01, 
       1.2746242965502491e-01, 
       9.1785288730646369e-02, 
       1.2518199631527549e-01, 
       1.1006702587118962e-01, 
       8.9731966474748848e-02, 
       8.9637489703927903e-02, 
       9.5192286936762571e-02, 
       1.0629677251117418e-01, 
       8.6330413300043263e-02, 
       1.0404948516145857e-01, 
       8.2776127104447164e-02, 
       9.9285549280719032e-02, 
       9.6419999999995343e-02, 
       7.6755546476328512e-02, 
       6.4090626460624453e-02, 
       8.9095434709956053e-02, 
       6.3240144410331142e-02, 
       8.4034986945598827e-02, 
       6.8314028536393123e-02, 
       6.8303077183213837e-02, 
       8.2398944957090450e-02, 
       5.8449272719104599e-02, 
       6.3688009132578050e-02, 
       6.7536848532955648e-02, 
       7.2073300760869777e-02, 
       5.3556313475687803e-02, 
       5.9471106047972226e-02, 
       6.8664003491278658e-02, 
       6.7132924186440693e-02, 
       5.0245652440359882e-02, 
       6.1450150620116467e-02, 
       6.0830070372690781e-02, 
       6.5689999999995169e-02, 
       6.5200132132263888e-02, 
       6.4560000000004544e-02, 
       4.6688226262111013e-02, 
       4.5629591483185773e-02, 
       6.1805954665041365e-02, 
       6.0551814029700807e-02, 
       5.9038968920819626e-02, 
       5.1642974894268674e-02, 
       4.2712246544913401e-02, 
       5.7543101158813247e-02, 
       5.7399968673301077e-02, 
       5.5543844262644110e-02, 
       5.6158345419588578e-02, 
       5.2733808282278270e-02, 
       5.2717323320383182e-02, 
       4.0118635508609217e-02, 
       3.9001503765104729e-02, 
       3.8941618259706892e-02, 
       4.4785365077861607e-02, 
       4.0516769087353709e-02, 
       4.0475158015038100e-02, 
       3.8691468531472956e-02, 
       3.8397166125741559e-02, 
       3.7826248445116568e-02, 
       4.2767434723004295e-02, 
       5.0957800836379630e-02, 
       5.0583843138394910e-02, 
       5.0350000000001262e-02, 
       3.5468402657254532e-02, 
       4.9629999999995719e-02, 
       3.4818106018154361e-02, 
       3.4772973458037502e-02, 
       4.7459999999995221e-02, 
       4.6132545605275931e-02, 
       4.6588652666909529e-02, 
       4.6320711703107115e-02, 
       3.3133228444480294e-02, 
       3.1927899397583115e-02, 
       3.2611784056505090e-02, 
       4.3415375804109618e-02, 
       4.2188005894385046e-02, 
       4.2190750666371270e-02, 
       3.1539100658404894e-02, 
       3.1290014701064087e-02, 
       3.9566393966254762e-02, 
       2.8888069539323193e-02, 
       3.8987938960590385e-02, 
       3.8380000000004348e-02, 
       3.7705952404347515e-02, 
       2.8110548890319713e-02, 
       2.8108234605514738e-02, 
       3.6711933892690105e-02, 
       3.1714467359975455e-02, 
       3.6380000000001071e-02, 
       3.3021973999511997e-02, 
       3.5553189662374737e-02, 
       2.5104780754900673e-02, 
       3.4289439605918105e-02, 
       2.6070980098433071e-02, 
       2.8423149551153059e-02, 
       3.4010000000003794e-02, 
       2.5168234793525160e-02, 
       3.3408400933185263e-02, 
       3.1865472208710700e-02, 
       2.4106559462598230e-02, 
       3.3038099104849832e-02, 
       2.3866119692193755e-02, 
       3.2790000000002505e-02, 
       3.2667317510966284e-02, 
       2.3007619460476997e-02, 
       3.2421606338338557e-02, 
       2.9338162240744110e-02, 
       2.5858913855993395e-02, 
       2.3128699035052842e-02, 
       3.1027262021275717e-02, 
       2.5402215278098387e-02, 
       2.9715822221178807e-02, 
       3.0244349364247369e-02, 
       2.9119850618888238e-02, 
       2.2927797268912212e-02, 
       2.0216692691565879e-02, 
       2.3037952745140203e-02, 
       2.0092715545876751e-02, 
       1.9995328068746640e-02, 
       1.9472192625090038e-02, 
       1.8291649037549606e-02, 
       1.8184608314744891e-02, 
       2.1678397967257811e-02, 
       1.7653100740524553e-02, 
       2.3353077071363824e-02, 
       2.2824418591756091e-02, 
       1.9219851765689255e-02, 
       1.7061611182681052e-02, 
       1.7246354594809988e-02, 
       1.6771945339708338e-02, 
       2.2605735743909162e-02, 
       2.1924137390063780e-02, 
       2.1642757174307643e-02, 
       1.8672338502699275e-02, 
       1.8441108184902938e-02, 
       2.1273253950073061e-02, 
       1.8086561164565584e-02, 
       1.8130724229693963e-02, 
       1.7044540435060838e-02, 
       1.7044477417501532e-02, 
       2.0085482859434386e-02, 
       1.4274335694944601e-02, 
       1.4107359915571025e-02, 
       1.4267884442134482e-02, 
       1.4690675026163877e-02, 
       1.8763498286656501e-02, 
       1.7072880889663928e-02, 
       1.3099975445280646e-02, 
       1.3961014802560323e-02, 
       1.6374616250274730e-02, 
       1.2866997345753910e-02, 
       1.4441647209289192e-02, 
       1.4253412642324525e-02, 
       1.6734154951521151e-02, 
       1.4592019601026889e-02, 
       1.3102093412735239e-02, 
       1.5309486351060199e-02, 
       1.4508937895664301e-02, 
       1.3401961260460497e-02, 
       1.1336904980371118e-02, 
       1.5874360340771023e-02, 
       1.5890009389356627e-02, 
       1.4984936262666060e-02, 
       1.4815413545237275e-02, 
       1.5445974681861178e-02, 
       1.1266242535087706e-02, 
       1.1163898989060715e-02, 
       1.4544535280541294e-02, 
       1.0955204989456752e-02, 
       1.2296882556946316e-02, 
       1.2295502235573874e-02, 
       1.4500494863280781e-02, 
       1.4920000000000700e-02, 
       1.4375525984192825e-02, 
       1.4289302962828048e-02, 
       1.0814548003023025e-02, 
       1.2256997209160753e-02, 
       1.4356791151826224e-02, 
       1.1961952362634452e-02, 
       1.0235634212674503e-02, 
       1.1418764288339248e-02, 
       1.1820688058740524e-02, 
       9.9328117453114789e-03, 
       1.0921300410710584e-02, 
       1.0898434934046814e-02, 
       1.0928950731086273e-02, 
       1.0226221815351147e-02, 
       1.3141502978360422e-02, 
       1.0680472776416540e-02, 
       9.5760088268442966e-03, 
       1.3002310276079674e-02, 
       9.5017794040907263e-03, 
       1.2297949403945159e-02, 
       1.2592493465141108e-02, 
       1.2339999999995421e-02, 
       1.2276588583219322e-02, 
       8.9964420062321627e-03, 
       1.2093760855596099e-02, 
       1.2196107287576665e-02, 
       1.0867962968870925e-02, 
       8.7221437601413855e-03, 
       8.7377387165816726e-03, 
       1.1993327642168065e-02, 
       1.1941092500481457e-02, 
       9.2203010106494641e-03, 
       9.8821115930483854e-03, 
       8.4358991827079323e-03, 
       8.1164727977721344e-03, 
       9.2651103990500534e-03, 
       8.3694365898899278e-03, 
       1.1233065299030327e-02, 
       1.1003741194030168e-02, 
       9.3382855276331316e-03, 
       8.0895990222381465e-03, 
       8.7309503110737027e-03, 
       8.8020024347721954e-03, 
       8.7944588315124524e-03, 
       9.3618103055817959e-03, 
       8.7385815470108034e-03, 
       7.9088607385084612e-03, 
       8.9403125641649202e-03, 
       1.0522621197301553e-02, 
       1.0648382836180623e-02, 
       8.6003996075399037e-03, 
       8.6003288838716104e-03, 
       8.7715455587944729e-03, 
       8.5722352546708219e-03, 
       8.9722870518977524e-03, 
       1.0368090089815611e-02, 
       7.5434677414822461e-03, 
       9.4781841791835297e-03, 
       1.0164625304698017e-02, 
       1.0253092019446068e-02, 
       7.3736562184639268e-03, 
       7.2060381225442643e-03, 
       7.0367235435489723e-03, 
       9.7894616966225246e-03, 
       8.4946926695488659e-03, 
       8.1370793879972959e-03, 
       6.9795098885043613e-03, 
       8.4983289449398567e-03, 
       8.4859288574260540e-03, 
       9.5415347761849574e-03, 
       9.4477773940359853e-03, 
       8.9139888594979826e-03, 
       9.5669254787275216e-03, 
       7.8111718245907841e-03, 
       8.1480996322073219e-03, 
       6.9018976856702495e-03, 
       8.1885149882452400e-03, 
       9.3250624472567198e-03, 
       9.3160649213792201e-03, 
       6.7796717582263716e-03, 
       9.1315731545499341e-03, 
       9.1399999999995738e-03, 
       6.6089180137541160e-03, 
       8.8673379590845664e-03, 
       7.4850898517008204e-03, 
       7.9138893585915041e-03, 
       6.5487551452522805e-03, 
       8.7404399887153709e-03, 
       8.8600000000000952e-03, 
       8.7839550957260395e-03, 
       8.7993441570939461e-03, 
       8.6286267667268265e-03, 
       8.6474635538930922e-03, 
       6.7302390281485866e-03, 
       7.0389455346703961e-03, 
       6.5204579028863017e-03, 
       6.5133883699855606e-03, 
       8.4979770119313017e-03, 
       8.2805359410635087e-03, 
       8.1811026017887881e-03, 
       8.2897116313251553e-03, 
       7.7544015315446305e-03, 
       5.8521313172268212e-03, 
       6.2731478810823639e-03, 
       8.1700000000002829e-03, 
       6.2218927926720978e-03, 
       7.2829573576385920e-03, 
       6.5015069613570883e-03, 
       5.8358368036995683e-03, 
       5.8293286349401221e-03, 
       7.3939759607444923e-03, 
       7.2808655277002733e-03, 
       7.6796266064201729e-03, 
       5.9526732724390049e-03, 
       6.5858013785186856e-03, 
       5.7393996906342703e-03, 
       7.4444208832347017e-03, 
       7.3685068466279478e-03, 
       7.6476703024961224e-03, 
       7.5822428230606287e-03, 
       7.4766600866554211e-03, 
       6.7494436994987323e-03, 
       7.4267611810604458e-03, 
       6.5546467027685455e-03, 
       5.3458627960205922e-03, 
       5.3502199618913806e-03, 
       5.3197264217081999e-03, 
       6.3545948428592163e-03, 
       7.2824944837132227e-03, 
       5.9429317719508126e-03, 
       7.1040682863384788e-03, 
       5.8068799047874039e-03, 
       4.9510454389606251e-03, 
       5.0822150663320451e-03, 
       5.0481687931395970e-03, 
       6.9004428260513008e-03, 
       6.9404182965898138e-03, 
       5.3523915998995196e-03, 
       5.3515205975683718e-03, 
       5.9155924782595803e-03, 
       4.8162536109563709e-03, 
       5.0851723443103153e-03, 
       5.5273918693944076e-03, 
       5.0138462413465154e-03, 
       4.8603936553017176e-03, 
       4.9798617606231742e-03, 
       6.6532508656314644e-03, 
       5.5539146439224016e-03, 
       6.5671958230554906e-03, 
       6.0218437633064812e-03, 
       4.8222362430869201e-03, 
       5.4707720131492096e-03, 
       6.3068150119425593e-03, 
       4.7614523485600716e-03, 
       6.4599999999997793e-03, 
       6.3501628777587792e-03, 
       5.1462490539057751e-03, 
       4.5882135932955098e-03, 
       4.6503690343716294e-03, 
       5.1335599911527400e-03, 
       4.7377382124284793e-03, 
       5.0345606116760193e-03, 
       4.3324620295761337e-03, 
       4.5528580052894355e-03, 
       5.9254830898884297e-03, 
       4.3344604709025280e-03, 
       4.3157453797296810e-03, 
       5.9437972208986152e-03, 
       4.5373081046183030e-03, 
       4.8577501986451672e-03, 
       4.2565499367579812e-03, 
       4.3827245442165236e-03, 
       4.2496210330118800e-03, 
       5.8334423367587063e-03, 
       5.5418416289868362e-03, 
       5.8267409090115288e-03, 
       4.1500845058207950e-03, 
       4.1857546034400435e-03, 
       4.6336555318820761e-03, 
       4.6261860194072614e-03, 
       4.3262254343199615e-03, 
       4.3117237239847835e-03, 
       4.1440269630977451e-03, 
       5.7642442951513729e-03, 
       5.7407821764311013e-03, 
       4.6433348722167951e-03, 
       4.6384138935456265e-03, 
       5.7099834780819604e-03, 
       5.5860551739747326e-03, 
       4.0103196478021028e-03, 
       4.0298304214019410e-03, 
       4.5537327985462504e-03, 
       4.0951045592801626e-03, 
       4.2769747057371460e-03, 
       5.5412259473931981e-03, 
       4.5069136908714317e-03, 
       5.4910889212198225e-03, 
       5.4266394339452517e-03, 
       4.5444205487340603e-03, 
       5.4647383663730658e-03, 
       4.0651306653275137e-03, 
       4.3694250381110088e-03, 
       4.2349664265049361e-03, 
       5.4210467090225131e-03, 
       4.0688231027326003e-03, 
       4.1228004202513756e-03, 
       5.2092306391171650e-03, 
       5.2442444025377543e-03, 
       3.8120639642060588e-03, 
       4.4019685958540875e-03, 
       4.4250852829542447e-03, 
       3.9620685051879080e-03, 
       4.4394697653243869e-03, 
       4.1122452473073643e-03, 
       5.0296863913236300e-03, 
       3.9388402741186886e-03, 
       5.2424800204011485e-03, 
       4.4558157037252301e-03, 
       3.9683003044486149e-03, 
       4.2645386696497558e-03, 
       4.4610685795312254e-03, 
       4.2354678726022931e-03, 
       5.1125464093673741e-03, 
       3.7426392620832261e-03, 
       4.4117376800659027e-03, 
       3.7647455639516652e-03, 
       3.9364904695202568e-03, 
       4.2289390911729426e-03, 
       3.7735459921122783e-03, 
       3.7420133383358816e-03, 
       5.0273393347649966e-03, 
       4.1462438457142738e-03, 
       4.1789998103965702e-03, 
       3.6756145645106641e-03, 
       4.9905181183983929e-03, 
       4.2694069567301481e-03, 
       4.4510046103813077e-03, 
       3.7187638067171951e-03, 
       3.7995023643886548e-03, 
       3.6470245878938126e-03, 
       4.7663151064038850e-03, 
       4.7761839295230434e-03, 
       4.6451681345157422e-03, 
       4.6164208272463821e-03, 
       4.8299999999995187e-03, 
       3.6869107057904533e-03, 
       4.4300757978325495e-03, 
       4.7900000000004840e-03, 
       4.5271007134015167e-03, 
       3.9528511362587979e-03, 
       3.5404056156400393e-03, 
       4.0236788545375286e-03, 
       3.7041755573327196e-03, 
       3.3347071079631394e-03, 
       3.4274573516895656e-03, 
       4.6450414268349475e-03, 
       3.4074236785597194e-03, 
       3.3964737298713492e-03, 
       4.6023035426816161e-03, 
       4.6034337802553305e-03, 
       3.3759524111725663e-03, 
       4.2601266272364142e-03, 
       3.3518300456396050e-03, 
       4.4315500680550809e-03, 
       4.6100000033045020e-03, 
       4.5260423631291345e-03, 
       3.2905829724293794e-03, 
       4.0311885453820240e-03, 
       4.5375037397479071e-03, 
       4.5081916540116998e-03, 
       3.3034540092128895e-03, 
       4.2862959294371556e-03, 
       4.4161281293007629e-03, 
       3.3680125342573069e-03, 
       3.2548943313755246e-03, 
       3.2548943313755246e-03, 
       3.6497612144552618e-03, 
       4.3373467382246287e-03, 
       4.0300106814112951e-03, 
       3.8253101071344942e-03, 
       3.1081893635989768e-03, 
       3.5719646557067740e-03, 
       4.0655404354012969e-03, 
       4.2889185809626357e-03, 
       3.4216352006252778e-03, 
       3.6132675456107487e-03, 
       3.0550437224292669e-03, 
       4.0851551597584565e-03, 
       3.1717776225515474e-03, 
       4.1024226920472751e-03, 
       4.0353511436252874e-03, 
       3.1193102234358039e-03, 
       3.9196213894919030e-03, 
       3.3893039511980099e-03, 
       3.1600000000003834e-03, 
       3.9834820381993777e-03, 
       4.0593726395106227e-03, 
       3.0753077467888901e-03, 
       2.9486491078091893e-03, 
       3.8684773071257315e-03, 
       3.8947443071590997e-03, 
       3.6494449816163485e-03, 
       3.9649586962442951e-03, 
       3.8465318038624625e-03, 
       2.8974794568920759e-03, 
       3.5737281914646740e-03, 
       3.8262418604544475e-03, 
       3.7742375854299937e-03, 
       3.8699999999997287e-03, 
       3.8699999999999516e-03, 
       3.0069769359888253e-03, 
       3.1025331569056254e-03, 
       3.1524399402067726e-03, 
       2.7310207339922457e-03, 
       3.1310471785689496e-03, 
       3.7686905968307301e-03, 
       2.6611686171571455e-03, 
       3.1638130363504800e-03, 
       3.5923165603289784e-03, 
       3.0868165258771201e-03, 
       3.7501379583092698e-03, 
       3.2062463018620911e-03, 
       2.8259732036758512e-03, 
       2.6491799266507333e-03, 
       2.6450728329575168e-03, 
       3.6595992035653312e-03, 
       2.7968230711208730e-03, 
       3.3357503320279587e-03, 
       3.3897093237140632e-03, 
       2.7039573010546251e-03, 
       3.6050160595059639e-03, 
       2.5551519093759776e-03, 
       3.5880983104539751e-03, 
       2.9764461555292204e-03, 
       3.5790981751215485e-03, 
       2.6315137181520541e-03, 
       2.9847819957473593e-03, 
       3.5700342772485272e-03, 
       2.7282046370829863e-03, 
       3.5399468521004319e-03, 
       2.6198704294402046e-03, 
       2.5853041098974909e-03, 
       3.5369153631672843e-03, 
       2.8551844009306643e-03, 
       2.5984519683424643e-03, 
       2.6759948788926385e-03, 
       2.8275792481972269e-03, 
       3.3697494353653449e-03, 
       3.4589093862194215e-03, 
       2.7215857773519527e-03, 
       3.4397306740520609e-03, 
       2.7003319636169218e-03, 
       2.6998703990646447e-03, 
       3.3953292840951149e-03, 
       3.4200000000003815e-03, 
       2.4165801032213583e-03, 
       2.9011833024086288e-03, 
       3.2438745908509141e-03, 
       2.6515975750444226e-03, 
       2.5554048711513488e-03, 
       3.3264021630466656e-03, 
       2.4171139545802454e-03, 
       2.7688821010570020e-03, 
       2.6025260292265101e-03, 
       2.5194166763877713e-03, 
       3.3100000000000221e-03, 
       3.2636986459409038e-03, 
       2.6360466263729509e-03, 
       3.0386338644987966e-03, 
       2.5350239215405231e-03, 
       2.9370910455737680e-03, 
       2.8647748236371585e-03, 
       3.2775138354655966e-03, 
       2.5648514129616240e-03, 
       2.3804813839570071e-03, 
       2.6028294329002609e-03, 
       2.6007217353894592e-03, 
       2.5079775343310336e-03, 
       2.2847346856131809e-03, 
       3.0988677762894940e-03, 
       2.3064261104338156e-03, 
       2.9832218266940655e-03, 
       3.0439717380525044e-03, 
       2.5239351022626161e-03, 
       2.7314712752791854e-03, 
       2.5273518822850330e-03, 
       2.4614861440655214e-03, 
       2.6371836328727386e-03, 
       3.1027920797688825e-03, 
       2.4712656067106004e-03, 
       2.2744613312596479e-03, 
       2.6950511922503146e-03, 
       2.7040258163243797e-03, 
       2.2181952671622273e-03, 
       2.2535395162962932e-03, 
       3.0984425272880109e-03, 
       3.0011756690357301e-03, 
       2.2799106698382786e-03, 
       2.2918168296919444e-03, 
       2.1733633008863412e-03, 
       2.4012359641419037e-03, 
       2.9743258649099773e-03, 
       2.2521458842179299e-03, 
       3.0161616298876861e-03, 
       2.4297543613005092e-03, 
       2.7766976018804355e-03, 
       2.4199999999995414e-03, 
       2.4211629601735537e-03, 
       2.9419421173978318e-03, 
       2.3870133119518432e-03, 
       2.3821102666809289e-03, 
       2.8236889350369380e-03, 
       2.9300000000004930e-03, 
       2.9016840840167379e-03, 
       2.2787979018738650e-03, 
       2.2983666240586507e-03, 
       2.8899999999999928e-03, 
       2.3972947016620730e-03, 
       2.3723601486148813e-03, 
       2.8716272435513755e-03, 
       2.2476611171650248e-03, 
       2.1170667630863076e-03, 
       2.0355248606954697e-03, 
       2.8199999999998070e-03, 
       2.0524913316505968e-03, 
       2.1409532809406780e-03, 
       2.0831395316330058e-03, 
       2.7543957385140917e-03, 
       2.6842665989240074e-03, 
       2.1239895125103113e-03, 
       2.0237146891879127e-03, 
       2.3682725805927824e-03, 
       2.6895008524175706e-03, 
       1.9579294100652467e-03, 
       2.7066895296675004e-03, 
       2.6654961858512656e-03, 
       1.9872779054874853e-03, 
       1.9833063248019590e-03, 
       2.7185308300393288e-03, 
       2.7169512771363399e-03, 
       2.7028189079458492e-03, 
       2.5758232558686259e-03, 
       2.0068850024268967e-03, 
       2.4877748872556703e-03, 
       1.9219330432312276e-03, 
       2.4603415953445549e-03, 
       1.8931716016517451e-03, 
       2.0299999999998895e-03, 
       2.6472296335201986e-03, 
       1.9306363068383426e-03, 
       2.6164932604254300e-03, 
       2.5385705278607924e-03, 
       2.3928975288987730e-03, 
       2.1829795392251609e-03, 
       2.3182015988616558e-03, 
       2.6099999999995159e-03, 
       1.9421205255195234e-03, 
       1.9295149205727218e-03, 
       1.9365989912645330e-03, 
       2.3522546450178202e-03, 
       2.5858359733643499e-03, 
       2.2180990258365246e-03, 
       1.8221640774989003e-03, 
       1.9148473204572969e-03, 
       2.5658080098315350e-03, 
       1.8590326216774008e-03, 
       2.3457244205617460e-03, 
       1.9544455231295028e-03, 
       1.9522253530728719e-03, 
       2.0478872312459898e-03, 
       2.0468487318125667e-03, 
       2.1900245045798198e-03, 
       2.4896289906873523e-03, 
       2.5009123956874252e-03, 
       2.4999999999996401e-03, 
       2.4899999999997819e-03, 
       2.0702521242089576e-03, 
       1.9007870719109836e-03, 
       1.9780462846281208e-03, 
       1.9779336392554050e-03, 
       2.4318789125325042e-03, 
       1.8257966975204971e-03, 
       1.7625690462027088e-03, 
       1.7203878629902841e-03, 
       2.4199999999999174e-03, 
       2.4100000000002869e-03, 
       2.0007672667032837e-03, 
       2.3999999999999183e-03, 
       2.4000000000001338e-03, 
       1.7780516869793483e-03, 
       1.6983333676617875e-03, 
       2.3900000000002673e-03, 
       1.9807717103669104e-03, 
       2.2825691160982494e-03, 
       1.8191742338448909e-03, 
       1.8177788561484056e-03, 
       2.3007602462709500e-03, 
       2.0241995238632522e-03, 
       1.7830723828296887e-03, 
       2.2779806431451667e-03, 
       1.7509536248692258e-03, 
       2.2841914134650206e-03, 
       1.9480166452778796e-03, 
       2.3269518237402430e-03, 
       1.7338656980588098e-03, 
       2.2549876988810563e-03, 
       1.6869328314518073e-03, 
       1.8850192593118588e-03, 
       1.7335109636342030e-03, 
       1.9277037674457933e-03, 
       2.2937134493193167e-03, 
       2.2347239648000481e-03, 
       2.2899606564136419e-03, 
       1.8386076181676812e-03, 
       1.8091544383037517e-03, 
       1.8365890862521257e-03, 
       1.9632720077398563e-03, 
       2.2900000000004163e-03, 
       2.2606146438506084e-03, 
       2.1947260323806192e-03, 
       1.8366273892713070e-03, 
       2.2499999999999330e-03, 
       1.6503258949841859e-03, 
       1.5898618697840392e-03, 
       1.6132678133232736e-03, 
       1.7285710479875043e-03, 
       2.1819364536674727e-03, 
       1.7996990520162686e-03, 
       2.1424765074165306e-03, 
       2.1745838065229685e-03, 
       1.5768150137680436e-03, 
       2.1720670113321329e-03, 
       2.1799999999998622e-03, 
       1.6190812466956824e-03, 
       2.1722802848129449e-03, 
       1.8417832480108378e-03, 
       1.5528621807483862e-03, 
       2.1350106715931566e-03, 
       1.7350064538951375e-03, 
       1.7269363987423281e-03, 
       1.8711146861466682e-03, 
       1.5056759373217388e-03, 
       2.1001693125734843e-03, 
       2.0942332572060143e-03, 
       1.7982870075673900e-03, 
       1.9068667899253867e-03, 
       1.6503467998176468e-03, 
       2.0674873764758880e-03, 
       1.5325878350525264e-03, 
       2.0238367641259290e-03, 
       2.0550317770209876e-03, 
       2.0252491950527971e-03, 
       1.5308577695473096e-03, 
       1.5306973495298570e-03, 
       1.5793882496227105e-03, 
       1.6822554722101719e-03, 
       2.0265195270580318e-03, 
       1.8952629003556988e-03, 
       1.6407558163041862e-03, 
       1.9945984214087714e-03, 
       2.0015540468618395e-03, 
       1.4139060995160618e-03, 
       1.6617098435285356e-03, 
       1.9254248642200243e-03, 
       1.4062371807204413e-03, 
       1.6424171632766569e-03, 
       1.6495245913847024e-03, 
       1.6327609792276482e-03, 
       1.4635225266377311e-03, 
       1.4308955088630821e-03, 
       1.4018723847740997e-03, 
       1.8834339489351927e-03, 
       1.8611571534887577e-03, 
       1.8766687581784499e-03, 
       1.4179850367390420e-03, 
       1.4526550849287504e-03, 
       1.7244562643124190e-03, 
       1.9121356810601030e-03, 
       1.9231596069735166e-03, 
       1.6508087464653458e-03, 
       1.3777607252862863e-03, 
       1.6454618074853227e-03, 
       1.3766830169104562e-03, 
       1.8971842998242090e-03, 
       1.6777851843509054e-03, 
       1.5532247111761557e-03, 
       1.4034210878908348e-03, 
       1.8816394698278698e-03, 
       1.4014309909259250e-03, 
       1.3968544754312036e-03, 
       1.8700000000002452e-03, 
       1.8700000000002098e-03, 
       1.5087639524448795e-03, 
       1.3551651688856578e-03, 
       1.8223261810681351e-03, 
       1.7661833013445389e-03, 
       1.7838676718915363e-03, 
       1.8502113047060027e-03, 
       1.7577010502631578e-03, 
       1.5192144070342052e-03, 
       1.5708135120443797e-03, 
       1.8256453107601853e-03, 
       1.8254830921044806e-03, 
       1.2927516222434193e-03, 
       1.2954914079126398e-03, 
       1.7771201082909371e-03, 
       1.5149735245188185e-03, 
       1.2914713653732102e-03, 
       1.6445712380612244e-03, 
       1.2696070728118419e-03, 
       1.7843300701110296e-03, 
       1.3157549197217332e-03, 
       1.7832165136524272e-03, 
       1.3295767037098355e-03, 
       1.7000046378086108e-03, 
       1.5961206364084437e-03, 
       1.3561459724833172e-03, 
       1.7546449247051949e-03, 
       1.4570134449760693e-03, 
       1.3898727089682165e-03, 
       1.6191994377977149e-03, 
       1.5645293549003910e-03, 
       1.7223071664046474e-03, 
       1.6712992823120559e-03, 
       1.6207839964921986e-03, 
       1.3094477817975597e-03, 
       1.2569383607199749e-03, 
       1.6750126456623153e-03, 
       1.7067708412457268e-03, 
       1.2980850105653310e-03, 
       1.2979746762003005e-03, 
       1.5598484156674466e-03, 
       1.4098389180114096e-03, 
       1.3216291382943553e-03, 
       1.2428152303766366e-03, 
       1.5161105960777152e-03, 
       1.2420958168221062e-03, 
       1.3704625817213046e-03, 
       1.4504155532753240e-03, 
       1.4820334665109822e-03, 
       1.6496425196643361e-03, 
       1.6493818304308737e-03, 
       1.2114249995077357e-03, 
       1.3501186496078378e-03, 
       1.1920755562586532e-03, 
       1.3144748278246994e-03, 
       1.5876732120418174e-03, 
       1.5801546923187657e-03, 
       1.1869047774034163e-03, 
       1.5610172709767339e-03, 
       1.1225551325039886e-03, 
       1.1654554959813562e-03, 
       1.4890867100912824e-03, 
       1.2952018864439596e-03, 
       1.5636525516754310e-03, 
       1.5612119188761532e-03, 
       1.2073283840861419e-03, 
       1.2959527207559171e-03, 
       1.1376373878078253e-03, 
       1.2809689517142826e-03, 
       1.3249997525258835e-03, 
       1.5208287290748687e-03, 
       1.1000974537092906e-03, 
       1.3312629386195314e-03, 
       1.5204386359646618e-03, 
       1.4605374243384219e-03, 
       1.4754772135261614e-03, 
       1.5224156235756640e-03, 
       1.0864582301641662e-03, 
       1.0919910429324818e-03, 
       1.4542710429248334e-03, 
       1.4934959953747521e-03, 
       1.2490629610300006e-03, 
       1.1235046413893206e-03, 
       1.1321764201033899e-03, 
       1.4979530210164555e-03, 
       1.2444390624686773e-03, 
       1.4769897596576510e-03, 
       1.2172989807796421e-03, 
       1.0543785792185990e-03, 
       1.0812161219740978e-03, 
       1.2915562700134325e-03, 
       1.2891322235375040e-03, 
       1.4567401583301628e-03, 
       1.0890437945736370e-03, 
       1.1961248214479132e-03, 
       1.1138217529705540e-03, 
       1.1138217529705540e-03, 
       1.2961763385410602e-03, 
       1.3930029893194852e-03, 
       1.0370454804015058e-03, 
       1.1907394547140478e-03, 
       1.1103995892176044e-03, 
       1.4300000000002479e-03, 
       1.1925623463937486e-03, 
       1.1756587464652275e-03, 
       1.2585521050246628e-03, 
       1.3445309344465039e-03, 
       1.4204728198582424e-03, 
       1.4133346909035878e-03, 
       1.1775964492347218e-03, 
       1.1573027029038131e-03, 
       1.0855891597727570e-03, 
       1.4100000000004435e-03, 
       1.2648833334875436e-03, 
       9.8642469430611506e-04, 
       1.0275813540482407e-03, 
       1.0555013041040728e-03, 
       9.9819389362156897e-04, 
       1.2433506915278511e-03, 
       1.3765263557285331e-03, 
       9.9034240658264750e-04, 
       9.7279040163917403e-04, 
       1.3086182703963350e-03, 
       9.7158939871699564e-04, 
       1.1456493092330097e-03, 
       9.7418356003669540e-04, 
       1.3534111534617562e-03, 
       1.3351384686283842e-03, 
       9.8299324608544405e-04, 
       1.2038556933835489e-03, 
       1.0456741298643434e-03, 
       1.2952249410552608e-03, 
       1.1101262691606509e-03, 
       9.6753915639061889e-04, 
       1.2797035021220309e-03, 
       9.5094563299321202e-04, 
       9.9317960984859617e-04, 
       9.6534608775443627e-04, 
       9.5889018735018257e-04, 
       9.8512633706832617e-04, 
       1.0321970354880074e-03, 
       9.8975835786419475e-04, 
       9.6003395975713076e-04, 
       1.2765920184703047e-03, 
       9.3662067866448462e-04, 
       9.3000000000001133e-04, 
       1.3127530969139346e-03, 
       1.0865463154909878e-03, 
       1.0526862570578663e-03, 
       1.0516170403895252e-03, 
       9.3738508179694703e-04, 
       1.1303347363034800e-03, 
       9.2830149508325778e-04, 
       1.0052433311216577e-03, 
       1.2661045548038187e-03, 
       1.2922303583327006e-03, 
       9.6571632936494182e-04, 
       9.6447271286049521e-04, 
       1.2532086289933225e-03, 
       1.2909691719810109e-03, 
       1.0554029197475286e-03, 
       9.2978576200816591e-04, 
       1.2785993921210614e-03, 
       1.1834576057478997e-03, 
       1.2284959753454082e-03, 
       1.2699954126648093e-03, 
       9.4808121513936035e-04, 
       1.0024432138946654e-03, 
       1.2298178671611793e-03, 
       1.2113818681951645e-03, 
       9.4801346639570596e-04, 
       1.2142671101691000e-03, 
       1.2599999999998426e-03, 
       9.1800344261085555e-04, 
       1.0725286188973898e-03, 
       1.1599999999998159e-03, 
       1.0629071580385585e-03, 
       9.1232033840239421e-04, 
       1.1630885854882781e-03, 
       1.0794296412070786e-03, 
       1.0665972878419192e-03, 
       1.2329089067928292e-03, 
       1.2368080226265218e-03, 
       9.0939710877795479e-04, 
       1.2334740040695577e-03, 
       9.7351107849066691e-04, 
       1.1615715172142421e-03, 
       1.2061014874807255e-03, 
       1.0094002078523878e-03, 
       1.2300000000000737e-03, 
       9.3613845607067958e-04, 
       9.4097148850665199e-04, 
       1.2246944295565530e-03, 
       8.8008692332644956e-04, 
       8.6905442097273146e-04, 
       1.1858449620058234e-03, 
       1.2200002989522589e-03, 
       1.0148453228840163e-03, 
       1.0346230494303092e-03, 
       8.8040461890963458e-04, 
       1.0496273169325839e-03, 
       8.9777141655545366e-04, 
       1.0837776849863744e-03, 
       9.6859699326533311e-04, 
       1.1711318896361004e-03, 
       1.0711541643075523e-03, 
       1.1665183621811495e-03, 
       1.1490295100208825e-03, 
       1.1931935008622368e-03, 
       8.6052672601022863e-04, 
       1.1910957854165158e-03, 
       1.0389706735662884e-03, 
       1.0887497206317741e-03, 
       1.0308327779286342e-03, 
       1.1376350361910154e-03, 
       9.8092139282933660e-04, 
       8.4704049425449353e-04, 
       8.6690511533307085e-04, 
       9.3104157411509365e-04, 
       8.3759190208746467e-04, 
       1.0130114866484790e-03, 
       9.7726959591331371e-04, 
       9.5821244153830912e-04, 
       8.5454905270546078e-04, 
       9.1107071421944546e-04, 
       1.1712173367357662e-03, 
       8.3282677021752165e-04, 
       8.2659589346905618e-04, 
       1.1100358709307235e-03, 
       8.5223741674347387e-04, 
       8.4352696642236947e-04, 
       1.0116640678454482e-03, 
       1.1148682129775349e-03, 
       1.1600000000001093e-03, 
       8.1768916582058689e-04, 
       8.3600261687934857e-04, 
       1.1251200408447881e-03, 
       1.1456217508327799e-03, 
       1.1409139220836179e-03, 
       9.6545907568095619e-04, 
       1.1323239041273871e-03, 
       1.1315650878399993e-03, 
       1.1008358997167133e-03, 
       1.1300000000002812e-03, 
       9.4577781453270807e-04, 
       8.2279441518732426e-04, 
       1.1266993471448887e-03, 
       1.0903003804041259e-03, 
       8.0766078585498111e-04, 
       9.3478822209424185e-04, 
       1.0784929157195769e-03, 
       8.3006864829028310e-04, 
       1.0583221130535429e-03, 
       1.1094376852816847e-03, 
       7.9565216415007894e-04, 
       9.7244429779145181e-04, 
       7.8742599447722639e-04, 
       9.0648699947584060e-04, 
       1.0196915543615637e-03, 
       9.1809201721812712e-04, 
       1.0184813192613900e-03, 
       9.1522674398520644e-04, 
       1.0347315398800887e-03, 
       7.8997684950622476e-04, 
       1.0805440784325776e-03, 
       8.9479169604199708e-04, 
       1.0569965439996669e-03, 
       8.8991848703764726e-04, 
       1.0338625420254464e-03, 
       7.9573873837153854e-04, 
       9.6788962565278554e-04, 
       1.0246959253880103e-03, 
       8.5646651058429355e-04, 
       9.1563890352915761e-04, 
       1.0433697927345999e-03, 
       9.2213590099923630e-04, 
       7.7892305449145762e-04, 
       7.8856094614640200e-04, 
       8.6653114663670660e-04, 
       8.6646841693019134e-04, 
       8.9776978951531009e-04, 
       7.4168254552519470e-04, 
       7.7678684589418663e-04, 
       1.0306283206436698e-03, 
       7.9310913548351900e-04, 
       7.9053477314145436e-04, 
       8.3632264149889764e-04, 
       7.6144073851644076e-04, 
       1.0277153730335461e-03, 
       9.7440456489347000e-04, 
       9.2651204943898242e-04, 
       7.5588008391139249e-04, 
       9.7314615577283217e-04, 
       8.1982166626856211e-04, 
       9.7221328004828435e-04, 
       1.0002815602761206e-03, 
       7.4710708260124204e-04, 
       8.6568435755354930e-04, 
       9.0792895455544490e-04, 
       8.4300307694644655e-04, 
       7.8263328678369382e-04, 
       7.9713079621560740e-04, 
       7.6491305876090108e-04, 
       8.4378017076718785e-04, 
       8.1280990192064521e-04, 
       1.0022993250033661e-03, 
       8.3313400417962509e-04, 
       7.3939323541613350e-04, 
       8.2891418235444972e-04, 
       7.3455348180202806e-04, 
       8.2733113817355425e-04, 
       7.4409063843852848e-04, 
       8.0476678209954675e-04, 
       9.8999846744550566e-04, 
       8.0461005977376795e-04, 
       9.1018299110811522e-04, 
       7.0269820453712994e-04, 
       7.0669437355649748e-04, 
       7.1593528598210704e-04, 
       9.4365224177194416e-04, 
       7.2872433665334597e-04, 
       9.5424193532565827e-04, 
       7.0886162250229860e-04, 
       7.7795907707655927e-04, 
       9.4612316554480908e-04, 
       7.2804472880866062e-04, 
       9.7513557462950958e-04, 
       9.1821237842902614e-04, 
       6.9411276658204696e-04, 
       8.3142037398157859e-04, 
       7.1514495929499687e-04, 
       9.7000000000001382e-04, 
       7.1410394686497629e-04, 
       7.0813256168191180e-04, 
       9.0246164118255217e-04, 
       7.9980334316487268e-04, 
       8.2021783439170908e-04, 
       6.8161438271516206e-04, 
       6.9703861141072910e-04, 
       6.9943800764273132e-04, 
       7.1254736449391238e-04, 
       7.9531256715734654e-04, 
       7.2585222270890842e-04, 
       9.3134205004369156e-04, 
       7.7424818449768661e-04, 
       6.9841205197373579e-04, 
       7.7356785474675402e-04, 
       6.7863709647214977e-04, 
       8.6247022067978776e-04, 
       7.7879579799354102e-04, 
       9.4282042185140659e-04, 
       9.3678706388349063e-04, 
       8.4069150607142290e-04, 
       8.1314945375800067e-04, 
       8.1202647999096373e-04, 
       8.9785571152928409e-04, 
       7.8069838174116204e-04, 
       8.0333658894100174e-04, 
       7.6946118729107353e-04, 
       9.3000276244877326e-04, 
       7.4966076705902803e-04, 
       8.4971651518809274e-04, 
       6.9384560363747812e-04, 
       7.7604739092342078e-04, 
       6.7085871989980696e-04, 
       7.7197054848402715e-04, 
       7.2310282250962442e-04, 
       7.0955882061521120e-04, 
       8.8123764978376075e-04, 
       7.4465296825665817e-04, 
       6.6602981941966965e-04, 
       6.6690291217751780e-04, 
       6.8476968075501237e-04, 
       7.3908300356547363e-04, 
       7.3827546823965824e-04, 
       6.7156223298071925e-04, 
       6.7156223298071925e-04, 
       7.5174811240776224e-04, 
       9.1000000000004163e-04, 
       9.0779438677038012e-04, 
       8.3875719032176321e-04, 
       6.5868461063574735e-04, 
       7.6652500688921771e-04, 
       6.6681709180859013e-04, 
       6.4132284763147222e-04, 
       7.7690706557248653e-04, 
       6.4028864315425016e-04, 
       6.6750025715394269e-04, 
       8.9669739221255389e-04, 
       8.7967803816434501e-04, 
       6.4282206692254989e-04, 
       6.5130022039831001e-04, 
       6.3362931377866356e-04, 
       6.6573379846505477e-04, 
       6.9858794980904638e-04, 
       8.5660551968810925e-04, 
       7.4112391615269797e-04, 
       8.8677022639395680e-04, 
       6.4239790753865302e-04, 
       7.8711877804800965e-04, 
       6.5123281582353282e-04, 
       8.8000000000004318e-04, 
       6.2068431423636407e-04, 
       8.6920357265883846e-04, 
       8.7391428680221278e-04, 
       6.3143772308258653e-04, 
       8.6080515051304720e-04, 
       6.6371044248596527e-04, 
       6.5232962650301970e-04, 
       6.4026482505134819e-04, 
       6.4026482505134819e-04, 
       8.4321985992577282e-04, 
       6.1574345397563354e-04, 
       7.2100600217030848e-04, 
       8.6372702755716305e-04, 
       7.6966335784238861e-04, 
       8.6200654040554476e-04, 
       6.1158742150439923e-04, 
       7.1498381869119067e-04, 
       7.3534640458563518e-04, 
       6.9859436687440268e-04, 
       7.8191855263581378e-04, 
       6.5339877875688441e-04, 
       6.5240500092743976e-04, 
       6.1982505678272865e-04, 
       7.2340099106230266e-04, 
       7.8013613963673247e-04, 
       8.3616656788067559e-04, 
       6.8841056731480720e-04, 
       6.0029944242958016e-04, 
       6.9669644193107855e-04, 
       6.3257632304299831e-04, 
       7.3372048311659067e-04, 
       8.2220510869049530e-04, 
       8.3669455650854110e-04, 
       6.0316170414996930e-04, 
       6.1844176599117659e-04, 
       6.1872219584747782e-04, 
       6.9593362765576149e-04, 
       6.0834812220768666e-04, 
       6.2659570991868262e-04, 
       6.2661757041552330e-04, 
       7.3716058184034137e-04, 
       6.6496810260512247e-04, 
       6.8029633458562000e-04, 
       6.8000665693606331e-04, 
       8.3021291865708463e-04, 
       8.1767271090127706e-04, 
       6.1424330134125956e-04, 
       8.3000000293367827e-04, 
       6.0985571426827188e-04, 
       7.0823811775475867e-04, 
       6.6459371623357136e-04, 
       8.2196621697132029e-04, 
       6.1415282177970740e-04, 
       5.9972825136989907e-04, 
       6.5786260815004296e-04, 
       6.0119573645925502e-04, 
       8.1867900238362162e-04, 
       7.9000455583676233e-04, 
       8.1550591305644177e-04, 
       6.7711639966669271e-04, 
       6.3511126394072892e-04, 
       7.8289967941845085e-04, 
       5.7487147684041635e-04, 
       5.7431009692424117e-04, 
       7.8859062915458578e-04, 
       8.1000000000001459e-04, 
       5.7540540131339752e-04, 
       6.7950379247712106e-04, 
       6.7925481916929404e-04, 
       7.8631447703771964e-04, 
       5.8797821302234506e-04, 
       8.0111938525046628e-04, 
       6.0930544311687488e-04, 
       8.0000000000001142e-04, 
       7.7288081669481701e-04, 
       7.9012586086609870e-04, 
       5.6557066786845839e-04, 
       6.8829949818700236e-04, 
       7.8851225383130114e-04, 
       6.1811910089021197e-04, 
       6.8261425868056970e-04, 
       6.2192664561587768e-04, 
       6.2172657571608967e-04, 
       6.0335562376424043e-04, 
       6.3546634149962795e-04, 
       7.9230202111598769e-04, 
       7.3426841537285108e-04, 
       6.2775549002145616e-04, 
       6.5813170686152402e-04, 
       6.2000630240997682e-04, 
       5.8935810241948696e-04, 
       7.8989703736756891e-04, 
       6.5878391473622028e-04, 
       5.5825477734482143e-04, 
       6.3566083634935600e-04, 
       5.5696838578971346e-04, 
       5.7832240504527405e-04, 
       6.5821564761185225e-04, 
       6.5269722726035663e-04, 
       7.4626628412528622e-04, 
       7.6708975560699168e-04, 
       5.5817198725447098e-04, 
       5.5514695714526275e-04, 
       7.7721443263189540e-04, 
       5.7038000587658676e-04, 
       7.7531541309847269e-04, 
       7.7385038885484695e-04, 
       6.3689254218488077e-04, 
       5.4664932984583008e-04, 
       5.6472954698030711e-04, 
       5.4374441643537910e-04, 
       5.6595635183830388e-04, 
       5.4844452368383071e-04, 
       6.3787512410984760e-04, 
       6.6403608061930540e-04, 
       7.0780011383870828e-04, 
       5.4392453756790110e-04, 
       5.7070485353315561e-04, 
       5.7337620358155798e-04, 
       7.2490198671951145e-04, 
       7.3008052763913521e-04, 
       7.5999999999995841e-04, 
       7.0858008587138769e-04, 
       5.4791595179014686e-04, 
       7.5720139545383514e-04, 
       6.0934502501120661e-04, 
       5.4678211549236849e-04, 
       5.9138941823573966e-04, 
       7.2950902809912029e-04, 
       5.5339662960562821e-04, 
       7.2491758837569750e-04, 
       5.5379459678166719e-04, 
       5.4379377660796228e-04, 
       5.3389817404765518e-04, 
       6.2159724839353165e-04, 
       5.5628309491086636e-04, 
       6.4523080264652303e-04, 
       7.0907733189560466e-04, 
       6.6033047672569591e-04, 
       5.5378490182944639e-04, 
       7.1834494993784322e-04, 
       7.4213105818267042e-04, 
       6.5374483268347040e-04, 
       6.5266557974401752e-04, 
       5.2942198338573436e-04, 
       7.3999999999997386e-04, 
       7.4000000000001636e-04, 
       6.6092824664835525e-04, 
       5.2961141452477305e-04, 
       6.6581858743919157e-04, 
       7.2999974919207096e-04, 
       7.2999963076437492e-04, 
       5.5467143565976636e-04, 
       7.2992228105441372e-04, 
       6.2840305700487416e-04, 
       6.1452398036947089e-04, 
       5.2069714748877063e-04, 
       6.2677091869370012e-04, 
       7.1302760465230075e-04, 
       5.4089622468011373e-04, 
       5.5898812751329979e-04, 
       6.0931847502289273e-04, 
       5.1230391635608300e-04, 
       5.1436648547147556e-04, 
       5.3639982834622583e-04, 
       5.7539865900659861e-04, 
       5.2055512710551103e-04, 
       7.1710356663660794e-04, 
       7.1470424192824093e-04, 
       5.0532362346978363e-04, 
       5.1643089703568088e-04, 
       5.1976297439435286e-04, 
       5.7465312508802441e-04, 
       6.8414297292901784e-04, 
       7.1000000000004989e-04, 
       5.0884318288048423e-04, 
       5.2812394449659153e-04, 
       6.3302759308513090e-04, 
       7.0547990856312346e-04, 
       7.0798695767560957e-04, 
       6.7483662338723132e-04, 
       6.3810409887621387e-04, 
       4.9810961364705488e-04, 
       7.0200172250489495e-04, 
       5.3409237709100622e-04, 
       6.8900336577227444e-04, 
       5.9815571122091512e-04, 
       5.1307067397418648e-04, 
       6.9522924638597825e-04, 
       6.9511357842438448e-04, 
       6.3383446109559845e-04, 
       5.7832170103207540e-04, 
       6.5150389111380742e-04, 
       4.9541729000781014e-04, 
       5.2058192795818804e-04, 
       5.9131148231798684e-04, 
       6.4302987438582659e-04, 
       5.1481996021592361e-04, 
       4.9157115149900898e-04, 
       6.0280769361011370e-04, 
       5.6745820624528687e-04, 
       4.8355910489993840e-04, 
       5.6507212990846432e-04, 
       5.0730049399833727e-04, 
       6.7217573143669778e-04, 
       6.7887416791616621e-04, 
       5.5636094139904042e-04, 
       5.4961124234598289e-04, 
       4.8602644827140559e-04, 
       5.0938091158283724e-04, 
       6.7069048117028758e-04, 
       5.0215882513564010e-04, 
       5.5123517318064481e-04, 
       6.5268631430349033e-04, 
       4.7391213756538703e-04, 
       5.8090593726414558e-04, 
       6.3924507325015180e-04, 
       5.5209438130806936e-04, 
       4.9240943424248417e-04, 
       6.6000000000003708e-04, 
       4.7051462645494776e-04, 
       4.6692903419592245e-04, 
       5.4752816366597230e-04, 
       5.4630854112946374e-04, 
       5.3004132164538099e-04, 
       6.3062808974917711e-04, 
       6.3028007427644067e-04, 
       6.1056563291240335e-04, 
       5.7386396146040638e-04, 
       5.3772207757803831e-04, 
       6.3814635694454383e-04, 
       5.1617078394295822e-04, 
       6.4964955709223939e-04, 
       5.5424895413455763e-04, 
       5.0704652872991406e-04, 
       4.7984389657520371e-04, 
       6.4895196151641135e-04, 
       6.5000000714259194e-04, 
       6.4999999999995021e-04, 
       6.5000000000002588e-04, 
       5.7974990830082080e-04, 
       4.6116893078348688e-04, 
       4.6105045835894044e-04, 
       5.6518561923369292e-04, 
       4.9330423240376362e-04, 
       6.4000000000003193e-04, 
       5.6726398281265779e-04, 
       6.2411138393112572e-04, 
       6.2000000000004467e-04, 
       6.2264949120963923e-04, 
       4.5139493731399403e-04, 
       4.5871717975028376e-04, 
       4.5282857003451141e-04, 
       5.3929780421029894e-04, 
       5.5114513657074729e-04, 
       5.1827429749445311e-04, 
       4.6606441991696559e-04, 
       4.6551664567783077e-04, 
       5.6395540495885648e-04, 
       5.4266723688563311e-04, 
       4.5096746609551899e-04, 
       6.2726107068161448e-04, 
       4.6891395226036995e-04, 
       6.2713013014844101e-04, 
       6.0460364724051802e-04, 
       4.6490475422134063e-04, 
       4.5044461434635253e-04, 
       4.6060775522130046e-04, 
       6.2588674405481708e-04, 
       4.7493404627172012e-04, 
       4.6555159101184832e-04, 
       4.6924140135041467e-04, 
       5.5454665832410622e-04, 
       6.0110125953051713e-04, 
       5.7744783644751306e-04, 
       4.9263300484923844e-04, 
       4.4992127698678994e-04, 
       5.4091321885663774e-04, 
       4.3916091955935142e-04, 
       5.4010379449741675e-04, 
       6.1780995838109408e-04, 
       4.7434008361137744e-04, 
       4.9176163203455312e-04, 
       5.9605366984609204e-04, 
       4.3585740086119303e-04, 
       6.1198223062870077e-04, 
       5.9945970877674009e-04, 
       6.1156960132301684e-04, 
       4.9900568742153231e-04, 
       5.0318564804537958e-04, 
       5.9166025314418577e-04, 
       6.0320966875775312e-04, 
       5.0563910821510767e-04, 
       4.7116886124364742e-04, 
       4.5210135635676240e-04, 
       5.0057580977442954e-04, 
       4.8085403098942811e-04, 
       4.2510263363900776e-04, 
       4.8605194091633355e-04, 
       5.0038026554613047e-04, 
       4.6009372196928933e-04, 
       5.4999999998350021e-04, 
       4.6325826776952409e-04, 
       5.3497409788637079e-04, 
       4.2212467855028943e-04, 
       4.2430517418729288e-04, 
       4.9443022705390369e-04, 
       4.3682611477321190e-04, 
       4.7664420717421916e-04, 
       4.3733684690350411e-04, 
       5.2124592134093989e-04, 
       5.6389643274233734e-04, 
       4.9077073757915343e-04, 
       5.5646605353301573e-04, 
       5.8134237311108521e-04, 
       4.2955244193296736e-04, 
       4.8798053777009793e-04, 
       4.3161980268238481e-04, 
       5.8658242785552693e-04, 
       4.1516608093631946e-04, 
       5.3850314401051409e-04, 
       5.2639323687619563e-04, 
       4.1779609220308833e-04, 
       4.8562229419491133e-04, 
       4.9398842806422068e-04, 
       4.7989239609001570e-04, 
       4.2539607353024524e-04, 
       5.6014907347665740e-04, 
       5.6870009610388906e-04, 
       4.4163276182798987e-04, 
       4.0961420111653823e-04, 
       5.7512767555965058e-04, 
       5.7512767555965058e-04, 
       4.9974422197164093e-04, 
       4.8162583062591328e-04, 
       4.7276044731840917e-04, 
       4.3530070837642876e-04, 
       4.2598368022415262e-04, 
       5.6999999999999564e-04, 
       5.6335651122615992e-04, 
       4.5916480942228568e-04, 
       4.9197581347584134e-04, 
       4.0513652243518236e-04, 
       4.9191181205398323e-04, 
       4.6719090694487188e-04, 
       5.6005244572578290e-04, 
       5.0620815917619313e-04, 
       4.3737874174964634e-04, 
       4.2254269754009305e-04, 
       5.0318624530192458e-04, 
       4.1384923309204538e-04, 
       5.1999999999999789e-04, 
       4.6982485597677111e-04, 
       4.6818222624197929e-04, 
       4.2396172632085181e-04, 
       4.0095936284658513e-04, 
       4.2528950639611398e-04, 
       5.5575929263323575e-04, 
       4.6450829740590451e-04, 
       4.3683809078369208e-04, 
       4.6364913655398405e-04, 
       4.8371086780118680e-04, 
       5.4989367502066846e-04, 
       5.5000000000002118e-04, 
       3.9407655958794791e-04, 
       5.1549151323945438e-04, 
       3.9388407483963519e-04, 
       5.3818082284043552e-04, 
       4.1338172836927819e-04, 
       4.0916263705879078e-04, 
       4.5546994045643491e-04, 
       4.8751942788748522e-04, 
       4.1256719381255034e-04, 
       4.1463506368527554e-04, 
       4.5761529986272447e-04, 
       4.5713328062794978e-04, 
       5.5000000000000426e-04, 
       5.4999999994610825e-04, 
       5.0079616399263942e-04, 
       5.4552038217683778e-04, 
       4.6955223180184246e-04, 
       4.4668331051542258e-04, 
       5.4487690620766112e-04, 
       5.3999625505848090e-04, 
       4.0353909453589752e-04, 
       4.0353909453589752e-04, 
       4.2045118153598201e-04, 
       4.2809558453515392e-04, 
       3.8727862826213629e-04, 
       3.9477406153796572e-04, 
       3.8457937853508220e-04, 
       3.9456717868927011e-04, 
       3.8739514086400656e-04, 
       3.8311386562380829e-04, 
       3.8560631202609521e-04, 
       4.5004151858375865e-04, 
       3.8918891237868638e-04, 
       3.8949857213963109e-04, 
       4.2720352876515788e-04, 
       5.1121048883316081e-04, 
       5.1506245415238881e-04, 
       5.3437298561299570e-04, 
       3.9455596091555431e-04, 
       4.9872215054401719e-04, 
       5.1432484411937274e-04, 
       4.4517766482624586e-04, 
       4.4508913866600031e-04, 
       5.3367800639666043e-04, 
       5.1208172912988728e-04, 
       4.6685783886018606e-04, 
       4.0426565444144456e-04, 
       5.2649942590219637e-04, 
       4.6584336911603881e-04, 
       4.8196179333207195e-04, 
       4.4183047549366738e-04, 
       3.7600638848939817e-04, 
       5.3136331995239257e-04, 
       4.3686480762941763e-04, 
       4.2536927435630643e-04, 
       5.2072883301246969e-04, 
       5.2999999999995607e-04, 
       5.3000000000003186e-04, 
       4.3350364938149851e-04, 
       4.3581569017768880e-04, 
       4.2632785289746224e-04, 
       4.0031050301883928e-04, 
       3.9046176630679768e-04, 
       5.2351934653807662e-04, 
       3.7999096020331261e-04, 
       5.0336446387489017e-04, 
       3.8574316297888031e-04, 
       3.7361793461851840e-04, 
       4.3484949987432489e-04, 
       3.8049491423536297e-04, 
       4.5113272172745881e-04, 
       4.5021998740240915e-04, 
       4.4410613277214315e-04, 
       3.7480734470893030e-04, 
       5.0573993152945324e-04, 
       5.1321508434525531e-04, 
       3.6969461457686276e-04, 
       5.1205037073482967e-04, 
       3.7738099630051222e-04, 
       3.9226843927499738e-04, 
       3.9858985269086353e-04, 
       5.1032824845737251e-04, 
       4.2957090555950330e-04, 
       4.6034227044106465e-04, 
       5.1285702633402754e-04, 
       4.5882818950568680e-04, 
       4.3872746110911620e-04, 
       5.1301161162934164e-04, 
       4.9311073252946628e-04, 
       3.7874492454802698e-04, 
       3.7847888246667641e-04, 
       3.8121610893305797e-04, 
       4.2416932385073183e-04, 
       5.1005632098722275e-04, 
       4.7000000000001804e-04, 
       5.0999999999995071e-04, 
       3.7282554233516806e-04, 
       5.0378624745095447e-04, 
       3.7551728321603858e-04, 
       4.4456657649998552e-04, 
       5.0449773567706876e-04, 
       3.9052176384063572e-04, 
       3.7372919323574294e-04, 
       5.0370485521478105e-04, 
       3.8429008336012145e-04, 
       3.5631024691232032e-04, 
       3.6901055077555273e-04, 
       3.6298017167151213e-04, 
       4.8863471197428232e-04, 
       3.6665941008667987e-04, 
       4.1154273458343833e-04, 
       4.5999999999997199e-04, 
       4.1496683020581654e-04, 
       3.7750039089675064e-04, 
       3.7941307629292461e-04, 
       3.6406988930652286e-04, 
       3.5249725987256174e-04, 
       4.7798043817635851e-04, 
       4.9610424971991297e-04, 
       3.6981261412529057e-04, 
       3.6365571152934857e-04, 
       4.9032687590989065e-04, 
       4.0552389038652170e-04, 
       4.8815765837684373e-04, 
       4.3130501842226896e-04, 
       4.0307702415944756e-04, 
       3.6811934556956442e-04, 
       4.9050611671695135e-04, 
       4.0411028404503527e-04, 
       3.5641780053367070e-04, 
       3.6649024425408584e-04, 
       4.8970312256994407e-04, 
       3.7395020785479399e-04, 
       4.9000000000001159e-04, 
       4.1282322173015164e-04, 
       3.6328856151315246e-04, 
       4.1473232608298680e-04, 
       3.6293816486253599e-04, 
       3.9804907286306322e-04, 
       4.3267325395127782e-04, 
       4.0306881151688583e-04, 
       4.4975164901477803e-04, 
       3.5378950226443059e-04, 
       3.4695208127852496e-04, 
       3.6004725795847229e-04, 
       4.1652396491693837e-04, 
       4.1528934577348889e-04, 
       3.9455003239842945e-04, 
       3.9454326991327730e-04, 
       4.8016855975957355e-04, 
       3.5048348735828682e-04, 
       3.8169147839310413e-04, 
       4.5252464946800411e-04, 
       4.7751641859749145e-04, 
       4.8000000000002300e-04, 
       3.4415483865269991e-04, 
       3.3929201713642730e-04, 
       3.4927088914891074e-04, 
       3.9784336526643742e-04, 
       4.0009861275768307e-04, 
       4.7620363192444389e-04, 
       3.7832406030260241e-04, 
       3.5749410021930934e-04, 
       3.4166967476132040e-04, 
       4.3394398627180018e-04, 
       4.7284955211432443e-04, 
       4.7326156131668529e-04, 
       3.8264763752479246e-04, 
       4.6655137308119262e-04, 
       3.4428533432109181e-04, 
       3.3585236615849939e-04, 
       4.4354223156941236e-04, 
       4.6449417792684039e-04, 
       3.4701173402479666e-04, 
       3.9078619421172370e-04, 
       4.7000000000004726e-04, 
       3.9348658868949962e-04, 
       4.6918652533681644e-04, 
       4.6770622330167308e-04, 
       3.8933883352883648e-04, 
       3.4379558038630443e-04, 
       4.6821882305589873e-04, 
       3.8655641165644258e-04, 
       3.7540854015647454e-04, 
       3.9824750700320142e-04, 
       3.8635205428395566e-04, 
       3.5809932858616328e-04, 
       3.4725769431896318e-04, 
       4.6104904959842373e-04, 
       3.3594472322145074e-04, 
       3.4506185832132316e-04, 
       3.4557319595285963e-04, 
       3.8351112216075933e-04, 
       3.9620144708372198e-04, 
       3.5887751746418458e-04, 
       3.7019497036024594e-04, 
       3.3379879055111947e-04, 
       3.7996195758138794e-04, 
       4.4375048002769855e-04, 
       3.6534344498313705e-04, 
       4.5750684671995792e-04, 
       4.4189201186050237e-04, 
       3.7830891006131819e-04, 
       3.7757558406389881e-04, 
       4.5624709131267396e-04, 
       4.5387657614120259e-04, 
       3.3965495657166384e-04, 
       4.2000000000004848e-04, 
       3.4556834386416114e-04, 
       4.0640416094396456e-04, 
       4.2633949564346781e-04, 
       3.9012731171036578e-04, 
       3.2346383424974650e-04, 
       4.3991871553919433e-04, 
       4.5179916835719685e-04, 
       3.3374218929458228e-04, 
       3.4362949801894203e-04, 
       3.3134463008924230e-04, 
       4.5258579210128639e-04, 
       3.3753565775720458e-04, 
       3.7111877867089727e-04, 
       4.2877542323290678e-04, 
       4.5111790804178874e-04, 
       3.2628388182772425e-04, 
       3.4999999999996015e-04, 
       3.2901684701477603e-04, 
       3.6869926052463762e-04, 
       3.9436106115056701e-04, 
       4.3031960609901161e-04, 
       3.4401063567123387e-04, 
       4.3192521035167325e-04, 
       3.2324945941226497e-04, 
       4.4465713565632910e-04, 
       4.1000000000001398e-04, 
       3.7699094017464095e-04, 
       3.7984686696956811e-04, 
       3.1409974057439264e-04, 
       3.4863451868035172e-04, 
       3.2566805028308866e-04, 
       3.6678925222661304e-04, 
       3.2126662424898975e-04, 
       3.7935896285261285e-04, 
       3.6911235561371464e-04, 
       3.9021637844761426e-04, 
       3.5638001706662622e-04, 
       4.0681847137523949e-04, 
       4.2544332006264637e-04, 
       4.3999999999998972e-04, 
       4.4000000000001276e-04, 
       3.2466211964181005e-04, 
       3.2779809007237758e-04, 
       3.1494606464935876e-04, 
       4.3703168634438117e-04, 
       3.8091814934187421e-04, 
       4.3283201788746768e-04, 
       3.8787651843681800e-04, 
       3.2561725011307733e-04, 
       3.4896724338478899e-04, 
       3.2495154158264464e-04, 
       3.6225093611148265e-04, 
       3.1568024549466760e-04, 
       3.6143260960818014e-04, 
       4.2510603427415265e-04, 
       4.3385960316882953e-04, 
       3.8531393278944847e-04, 
       3.5749477944703424e-04, 
       3.6870974187214574e-04, 
       3.3213261934906738e-04, 
       3.0560295338488426e-04, 
       3.9100461575506090e-04, 
       3.0431737424626486e-04, 
       4.2999999999999273e-04, 
       3.0498524526232577e-04, 
       3.1648420869383583e-04, 
       3.4310638158848431e-04, 
       3.0316722303243347e-04, 
       4.2468000007732322e-04, 
       3.6229437929608937e-04, 
       3.6229437929608937e-04, 
       4.1068159018833541e-04, 
       3.0233612767745829e-04, 
       3.5450063851940239e-04, 
       3.9102547429156249e-04, 
       3.5523308079675394e-04, 
       3.0920048684611500e-04, 
       4.0761764220489314e-04, 
       4.0518425844854084e-04, 
       3.1806916894985886e-04, 
       4.2274689338733698e-04, 
       3.8999999999999067e-04, 
       3.3020903378459761e-04, 
       3.9464665570459583e-04, 
       3.9523425635147450e-04, 
       3.4583428389883282e-04, 
       4.2190765138491450e-04, 
       2.9993749142012525e-04, 
       3.8054820594683221e-04, 
       3.4971193584077532e-04, 
       4.2000026855266260e-04, 
       3.0031509415520707e-04, 
       3.9089040791596519e-04, 
       4.1999999999998961e-04, 
       3.7006480393252946e-04, 
       3.0663182220302975e-04, 
       3.3526062193236714e-04, 
       4.1640772967593533e-04, 
       3.0308883146672834e-04, 
       3.1254090069540144e-04, 
       2.9472137788544712e-04, 
       3.6020895870213270e-04, 
       3.4000000000003472e-04, 
       3.2109655763174893e-04, 
       4.0250305957530597e-04, 
       3.3899636555752695e-04, 
       3.4794351690815720e-04, 
       3.4175621273027580e-04, 
       3.4218966809802966e-04, 
       2.9503886267983378e-04, 
       3.9954299653274301e-04, 
       4.0881484453096238e-04, 
       2.9462646372733635e-04, 
       3.1962967370253710e-04, 
       4.0916865740933791e-04, 
       4.0363701184460541e-04, 
       3.3246309748291461e-04, 
       2.9517166212159778e-04, 
       3.3211203730722368e-04, 
       2.9976127117852777e-04, 
       2.9962457100265864e-04, 
       3.0901888211628327e-04, 
       3.5134690413953369e-04, 
       3.8862221311176449e-04, 
       2.9647004899364444e-04, 
       3.2818275133477201e-04, 
       2.8891741220349071e-04, 
       3.1046815325612312e-04, 
       2.9113647037429160e-04, 
       3.8061239796370437e-04, 
       3.0867548025880506e-04, 
       2.8500346520093456e-04, 
       3.3276501370818232e-04, 
       2.8410819343969144e-04, 
       3.9987957019984704e-04, 
       3.8694621562037512e-04, 
       3.0143634726493835e-04, 
       3.8926025294616304e-04, 
       3.0710365045423190e-04, 
       3.6938699108321460e-04, 
       3.8368959017781286e-04, 
       3.5758997500633620e-04, 
       3.3081229147781274e-04, 
       3.0855474298931645e-04, 
       3.2817976596494479e-04, 
       2.7952638208509386e-04, 
       3.9381244240047796e-04, 
       3.8676878008614673e-04, 
       2.9381030647368116e-04, 
       3.2588111777348891e-04, 
       3.6635421217691980e-04, 
       2.7894220485200516e-04, 
       3.0649671849783964e-04, 
       3.9110586687464871e-04, 
       3.5381848906551811e-04, 
       2.9174441952968630e-04, 
       3.3415315102016438e-04, 
       3.6000000000000994e-04, 
       2.9999999999997737e-04, 
       3.8426121287768797e-04, 
       3.8073136561670991e-04, 
       3.8958927839118410e-04, 
       3.9000000000000287e-04, 
       2.8214784396446424e-04, 
       3.0088389436765611e-04, 
       3.8779687155438334e-04, 
       3.4963531643484788e-04, 
       2.8137422119012555e-04, 
       2.7421931764710065e-04, 
       3.0849228222692141e-04, 
       3.1965920373934470e-04, 
       3.8572753681836899e-04, 
       2.7448948477840714e-04, 
       3.3486206459730229e-04, 
       2.7663595294063087e-04, 
       3.7722244115784287e-04, 
       2.9281249700962995e-04, 
       2.7897608599952977e-04, 
       2.9131770117922983e-04, 
       2.8765955124756735e-04, 
       2.8906932668831240e-04, 
       3.0691812661420284e-04, 
       3.2099300859788482e-04, 
       3.0289971553123482e-04, 
       2.7160777313612190e-04, 
       3.6612342080485910e-04, 
       3.6612342080485910e-04, 
       2.9740316213562617e-04, 
       3.1889647362531094e-04, 
       3.0664754286146038e-04, 
       2.8016299609648435e-04, 
       2.7135893660496820e-04, 
       3.0625801213706274e-04, 
       3.8288858215674784e-04, 
       2.8267628860100230e-04, 
       3.1913651429971802e-04, 
       3.8150128921608708e-04, 
       3.6301116838797142e-04, 
       2.8151476821767802e-04, 
       2.7693572297969870e-04, 
       3.3149348581474751e-04, 
       2.8167691445009482e-04, 
       3.8000000000004333e-04, 
       3.7999999999995991e-04, 
       3.4561737666945446e-04, 
       2.8215031628324737e-04, 
       3.3037861917515102e-04, 
       2.8241555789986926e-04, 
       2.7458111881018026e-04, 
       3.0514562754763126e-04, 
       2.6771912489981480e-04, 
       2.9479614770925943e-04, 
       3.3564992873426813e-04, 
       3.7759116887779276e-04, 
       2.6715740721041981e-04, 
       2.6692470691398246e-04, 
       3.1299548262245691e-04, 
       3.1416260121360564e-04, 
       3.1169933577574214e-04, 
       3.0188191664553795e-04, 
       2.7480980144696526e-04, 
       2.9156714783784083e-04, 
       3.1848479672344032e-04, 
       3.4051495725083273e-04, 
       3.7382931166251269e-04, 
       3.2043127077373837e-04, 
       2.7919957888173302e-04, 
       3.6408584341824447e-04, 
       3.0650447884550424e-04, 
       3.7198119435295291e-04, 
       3.7302819997674222e-04, 
       2.7854053765192415e-04, 
       3.0844451609554944e-04, 
       3.7073895047366572e-04, 
       2.9478656945448211e-04, 
       3.6571251354616037e-04, 
       3.7000000000001832e-04, 
       3.7000000000000552e-04, 
       3.6999999999996693e-04, 
       3.5246577333101516e-04, 
       2.7492168972559447e-04, 
       3.0569055475332163e-04, 
       2.6078377123679247e-04, 
       3.1283694369977965e-04, 
       3.1694605116375132e-04, 
       3.5074808953910211e-04, 
       2.5965631163232540e-04, 
       2.7670948030145349e-04, 
       3.2806998490010993e-04, 
       3.0439119817780798e-04, 
       3.0495973810626230e-04, 
       3.0406448179541953e-04, 
       2.7659993539418218e-04, 
       3.4362802489696520e-04, 
       2.5859028565911691e-04, 
       3.6357865505130132e-04, 
       3.4737802662420828e-04, 
       2.7046637784566212e-04, 
       3.3935755022058083e-04, 
       2.6375306340236325e-04, 
       3.2934349697214590e-04, 
       2.9824123680018090e-04, 
       2.5599522820911016e-04, 
       2.9991806883574731e-04, 
       3.0007926531984820e-04, 
       3.5937201334924733e-04, 
       2.7899408933336370e-04, 
       3.4816528139835226e-04, 
       2.9844607762745352e-04, 
       2.8911871289531634e-04, 
       2.9662497999368646e-04, 
       3.5887096042948237e-04, 
       2.8020289237650479e-04, 
       3.4593780142489194e-04, 
       3.0444131323393951e-04, 
       2.9408002877267316e-04, 
       3.2728219531534905e-04, 
       3.4712901583598243e-04, 
       3.2999999999996926e-04, 
       3.2999999999999810e-04, 
       2.8713073941316272e-04, 
       2.8688388235973534e-04, 
       3.5606451267367533e-04, 
       2.9655072210399116e-04, 
       2.9341123096812130e-04, 
       3.5521480735609318e-04, 
       3.0099523643406951e-04, 
       3.1314529682185092e-04, 
       2.9272363300667230e-04, 
       3.5249238487734022e-04, 
       3.5313912012647879e-04, 
       3.5569994395628582e-04, 
       2.8754646991334946e-04, 
       2.5634625485473683e-04, 
       2.8731810711531776e-04, 
       2.6027226830195610e-04, 
       3.5437447034408905e-04, 
       3.1469807429686041e-04, 
       3.1912931645537555e-04, 
       2.5259661810087295e-04, 
       3.0214384600439371e-04, 
       3.5222101647049744e-04, 
       3.0195971737167473e-04, 
       3.4799739618528007e-04, 
       2.4898705213235051e-04, 
       3.5045890138366991e-04, 
       2.4978017164393475e-04, 
       2.5797853573040163e-04, 
       2.4915607823899075e-04, 
       3.1677859568339078e-04, 
       2.5803099608073988e-04, 
       2.9069671440391857e-04, 
       2.9066771018564124e-04, 
       3.1830700531178279e-04, 
       3.4999999769699385e-04, 
       2.4794726414471260e-04, 
       3.3732816087003471e-04, 
       2.6527961420913249e-04, 
       2.6051878351969851e-04, 
       3.3700471305838487e-04, 
       2.5536333344901139e-04, 
       2.8671790642328308e-04, 
       2.8669020442501558e-04, 
       3.4124573496499772e-04, 
       2.8582322632570068e-04, 
       3.0969902074370171e-04, 
       2.4721853023526336e-04, 
       2.8207641246605770e-04, 
       3.4682568826888946e-04, 
       2.8182053346277526e-04, 
       2.6814795231813163e-04, 
       2.5497960784263128e-04, 
       2.8909975066614103e-04, 
       2.4576159441761912e-04, 
       2.5638292691730592e-04, 
       2.8561026430724861e-04, 
       2.5492030402668187e-04, 
       2.6027403041737989e-04, 
       2.5838285059557768e-04, 
       2.8289158250208857e-04, 
       2.4274309260134423e-04, 
       2.8304774705408860e-04, 
       2.5507152515281636e-04, 
       2.4276001026134260e-04, 
       2.5376060733476935e-04, 
       3.0549910755969882e-04, 
       3.4000000000002903e-04, 
       3.2817309124352333e-04, 
       3.3910557185732638e-04, 
       3.2851443431367830e-04, 
       2.7336818130423428e-04, 
       3.2465002308557550e-04, 
       2.7738414440392576e-04, 
       2.4087147027726104e-04, 
       2.9150648952472362e-04, 
       3.3433376990893763e-04, 
       2.9951565710251496e-04, 
       2.7977205197426633e-04, 
       3.1000000000004955e-04, 
       2.4268232015889037e-04, 
       2.3802194797841364e-04, 
       2.6470237182952031e-04, 
       3.0825936058622713e-04, 
       2.8015356281742716e-04, 
       3.2147369102024126e-04, 
       3.3602549302036910e-04, 
       3.3484208437546984e-04, 
       3.1158679909061755e-04, 
       3.3391006414717767e-04, 
       3.3178100899778734e-04, 
       3.2693076077376101e-04, 
       2.4587742452009288e-04, 
       3.1812078968321283e-04, 
       3.2745255924903239e-04, 
       2.3700147459950475e-04, 
       2.5073292726093040e-04, 
       2.9887809333783869e-04, 
       3.3038772420484845e-04, 
       3.1707043076798988e-04, 
       2.4066309227554762e-04, 
       2.4777933127548166e-04, 
       2.4516787149068210e-04, 
       2.3511166738172704e-04, 
       3.2841301452185854e-04, 
       2.7869539892330132e-04, 
       3.2507895591814972e-04, 
       2.3357568538978686e-04, 
       2.4650579047360703e-04, 
       3.1140347920485920e-04, 
       2.6491554154498741e-04, 
       3.2376801506460082e-04, 
       2.7189686491752409e-04, 
       2.8229210335892663e-04, 
       3.1361141120216714e-04, 
       2.3082065544860486e-04, 
       2.4287971909090668e-04, 
       2.4522064685685241e-04, 
       2.3048933522265363e-04, 
       3.1575511677970583e-04, 
       3.0052560807098408e-04, 
       3.2459944798876280e-04, 
       2.4315299594055041e-04, 
       2.4174443663928835e-04, 
       2.2932777568192018e-04, 
       2.7890801758860827e-04, 
       2.9653001834201241e-04, 
       2.3094116500960404e-04, 
       2.6841527929792294e-04, 
       2.5277492173775626e-04, 
       2.5269835499338721e-04, 
       2.6717973560172616e-04, 
       3.2004608744167000e-04, 
       2.6632850404897712e-04, 
       3.0822882059857381e-04, 
       3.2000000000000653e-04, 
       3.1999999999998197e-04, 
       2.3089197065868895e-04, 
       2.3644409815086988e-04, 
       2.7074578661858339e-04, 
       2.6843930833790760e-04, 
       2.7962867368758703e-04, 
       3.1335435371885494e-04, 
       2.2450961610092723e-04, 
       2.6377300524355469e-04, 
       2.8084648123536245e-04, 
       2.4639082287876787e-04, 
       3.0337575027186285e-04, 
       2.2398101376436432e-04, 
       2.8020949346363569e-04, 
       2.2725948757268074e-04, 
       2.7428192640330051e-04, 
       3.0757570615075217e-04, 
       2.3225671386347513e-04, 
       3.1519259927797821e-04, 
       2.3488097833332126e-04, 
       2.6121542600020958e-04, 
       2.2291161769439906e-04, 
       2.8999999999998558e-04, 
       2.6052650098136622e-04, 
       2.5605953635785862e-04, 
       2.7237736828798721e-04, 
       2.6398979271447686e-04, 
       2.8236004881136142e-04, 
       3.1223286215005525e-04, 
       2.5539065788708751e-04, 
       3.0353943632847115e-04, 
       3.1038683893495087e-04, 
       2.5605996640160224e-04, 
       2.5896238956270355e-04, 
       2.9968534325790512e-04, 
       2.2365817475873470e-04, 
       2.2359923356646434e-04, 
       3.1015214665271510e-04, 
       3.0966685550498830e-04, 
       3.0999999999999930e-04, 
       2.5895803887252878e-04, 
       2.9851050676463529e-04, 
       2.2568114291484124e-04, 
       2.9709160883279633e-04, 
       2.4695987619984672e-04, 
       2.2972314695256108e-04, 
       2.9052133525822317e-04, 
       2.9904633646293203e-04, 
       3.0584688234212338e-04, 
       3.0360145645795468e-04, 
       3.0762337277592464e-04, 
       2.5274598496518799e-04, 
       2.2832578538887165e-04, 
       2.8738117183432993e-04, 
       2.5066580831472799e-04, 
       2.4832087121313978e-04, 
       2.1634505320006252e-04, 
       3.0538632106064405e-04, 
       3.0505566785926706e-04, 
       2.4230240622553020e-04, 
       2.3712398367251383e-04, 
       3.0343700828352343e-04, 
       2.8000000000002844e-04, 
       2.5737851901275920e-04, 
       2.3090345289614232e-04, 
       2.6203615285588111e-04, 
       2.1435480401654007e-04, 
       2.1924543379301094e-04, 
       2.8349422113087177e-04, 
       3.0141984120337067e-04, 
       2.2412405919456285e-04, 
       2.2207074604221875e-04, 
       2.1515266976788883e-04, 
       2.6106557502668062e-04, 
       2.5123036435317954e-04, 
       2.1725486168721526e-04, 
       2.9723781671907562e-04, 
       2.1238431668671234e-04, 
       2.2112537127914842e-04, 
       2.2112537127914842e-04, 
       2.1458115413079128e-04, 
       2.9392665032830972e-04, 
       2.1630592414038688e-04, 
       2.3461817624261140e-04, 
       2.9888191951770479e-04, 
       2.9860314870736341e-04, 
       2.8528510200418475e-04, 
       2.5311792670238483e-04, 
       2.1631506307678700e-04, 
       2.9814447282336485e-04, 
       2.9762722303811544e-04, 
       2.9566304561046548e-04, 
       2.4668199069082089e-04, 
       2.5347531919254313e-04, 
       2.9767807760372087e-04, 
       2.2147411477197186e-04, 
       2.1019704900739519e-04, 
       2.4651670128117799e-04, 
       2.7593190462736541e-04, 
       2.6501804249264466e-04, 
       2.1404698313689599e-04, 
       2.2617384467264227e-04, 
       2.0999775609770074e-04, 
       2.4739478111970598e-04, 
       2.8959766279204840e-04, 
       2.1609022425062596e-04, 
       2.9018886024225268e-04, 
       2.8886942852741866e-04, 
       2.7677290404531528e-04, 
       2.0992450583501480e-04, 
       2.7761865182942186e-04, 
       2.0978827262598020e-04, 
       2.4924106536414474e-04, 
       2.7303486864617855e-04, 
       2.0973602942049916e-04, 
       2.6785537737312554e-04, 
       2.1638339343200199e-04, 
       2.4327563008538471e-04, 
       2.0683234599483158e-04, 
       2.4841495134293111e-04, 
       2.4117361179612615e-04, 
       2.9000000000001079e-04, 
       2.6017217572516302e-04, 
       2.0515327863038685e-04, 
       2.0721931099828851e-04, 
       2.0556349803770266e-04, 
       2.0554151356968740e-04, 
       2.8528680801423055e-04, 
       2.3288250107527647e-04, 
       2.5028076284343997e-04, 
       2.1174982848556030e-04, 
       2.1434862295089240e-04, 
       2.3259812400328294e-04, 
       2.8397343763635383e-04, 
       2.1198630611400692e-04, 
       2.1115566613833081e-04, 
       2.2534745645503380e-04, 
       2.1450551845586980e-04, 
       2.8364337529678224e-04, 
       2.1431854552746748e-04, 
       2.0916524716954076e-04, 
       2.4302346306440269e-04, 
       2.0345120414716255e-04, 
       2.8331966226812102e-04, 
       2.6863593299367573e-04, 
       2.0024761566247504e-04, 
       2.0848933855210344e-04, 
       2.1091351656094356e-04, 
       2.2911849871256159e-04, 
       2.2911238583062327e-04, 
       2.2909496872899104e-04, 
       2.2906657422364164e-04, 
       2.8167477541661887e-04, 
       2.4338143406680001e-04, 
       2.1185468489916734e-04, 
       2.4218780750653082e-04, 
       1.9936919697178377e-04, 
       2.2314343876869976e-04, 
       2.3366023713747571e-04, 
       2.3343471968429743e-04, 
       2.3374951714629722e-04, 
       2.8017560111609003e-04, 
       2.2992715607177285e-04, 
       2.8000000000000550e-04, 
       2.0309289098210270e-04, 
       2.7715103831956562e-04, 
       1.9952169240559857e-04, 
       2.3330012969725899e-04, 
       2.7922095171093451e-04, 
       2.1364682217870062e-04, 
       2.1299978282829338e-04, 
       2.3046635563901333e-04, 
       2.0900159873150136e-04, 
       2.3214170847214434e-04, 
       2.7898160468441292e-04, 
       2.0514147114674712e-04, 
       2.5065691003149014e-04, 
       2.7874021935346929e-04, 
       2.0160487597765896e-04, 
       2.7818511283130178e-04, 
       2.3084941794465844e-04, 
       2.5204189942460521e-04, 
       1.9670133688614192e-04, 
       2.7698205436740039e-04, 
       2.7580624146710219e-04, 
       2.0000000000005403e-04, 
       2.3705611681340094e-04, 
       1.9895410695235319e-04, 
       2.0446126622335097e-04, 
       2.0105913674654178e-04, 
       2.3636704243516863e-04, 
       2.7621406998876229e-04, 
       2.2229769886413230e-04, 
       2.2208451003023378e-04, 
       2.6643808870793675e-04, 
       2.3025909167680733e-04, 
       1.9492550991013027e-04, 
       2.4192778008061972e-04, 
       2.0442435745484692e-04, 
       1.9962559246435989e-04, 
       2.1000000000003473e-04, 
       2.0905328438121455e-04, 
       2.6099572557923708e-04, 
       2.7310320761013933e-04, 
       2.5788766370069325e-04, 
       2.7291701595589103e-04, 
       1.9723761951432405e-04, 
       2.1394067147785132e-04, 
       2.1389756278305462e-04, 
       2.0702013604190729e-04, 
       2.6764001294480455e-04, 
       2.6520765864910634e-04, 
       2.5928552326029072e-04, 
       1.9861945820817082e-04, 
       2.7216351050362138e-04, 
       2.3331554898973269e-04, 
       2.7151342561362364e-04, 
       2.4000000000004302e-04, 
       2.6904339026614867e-04, 
       2.6505583274631140e-04, 
       2.2522115025084503e-04, 
       1.9500357488339188e-04, 
       2.6010834827173729e-04, 
       2.6504551557715052e-04, 
       2.2869717402480120e-04, 
       2.2485528379923383e-04, 
       2.7000000000000000e-04, 
       2.2386096534236785e-04, 
       2.5580623644758349e-04, 
       2.0464179668304926e-04, 
       2.6924910537960239e-04, 
       2.4254778282210198e-04, 
       1.9463148864945034e-04, 
       1.9781152382694246e-04, 
       2.0380731358099737e-04, 
       2.1973938385810423e-04, 
       1.9344008981482465e-04, 
       2.5132282861364618e-04, 
       1.9946490893906532e-04, 
       2.1578959310608205e-04, 
       2.5025050850719808e-04, 
       2.6200483995390442e-04, 
       2.0561131681610842e-04, 
       1.8883291333670043e-04, 
       1.9429893762068129e-04, 
       1.9272130201738273e-04, 
       1.9348718208694800e-04, 
       2.1554101245122405e-04, 
       2.6575518573939447e-04, 
       2.5486622334928587e-04, 
       2.6145686979963951e-04, 
       2.2502864101785677e-04, 
       2.5964453700109741e-04, 
       2.2101643408883648e-04, 
       2.4221237934686190e-04, 
       2.3778003530802399e-04, 
       2.5399332543976828e-04, 
       1.9716912722749682e-04, 
       1.9687219187547033e-04, 
       1.8959717071523780e-04, 
       2.5472928566045626e-04, 
       2.1724382860670638e-04, 
       1.8798867080580410e-04, 
       2.6423573849746802e-04, 
       1.9112777204590629e-04, 
       2.0566209860903517e-04, 
       2.4842904558404765e-04, 
       2.0146250440383298e-04, 
       2.2608830617710314e-04, 
       2.1931749048673642e-04, 
       2.1166511424257006e-04, 
       2.6227468709101451e-04, 
       2.1469533439434928e-04, 
       2.0045334002944911e-04, 
       1.9258916104623593e-04, 
       1.9706220650144118e-04, 
       1.9229210893188927e-04, 
       2.2240106596626248e-04, 
       1.9478037161768579e-04, 
       2.3980586111792431e-04, 
       2.5999999999999949e-04, 
       2.5999999999995368e-04, 
       2.5999999999997656e-04, 
       2.6000000000002209e-04, 
       1.8356215622032768e-04, 
       2.5859209219964373e-04, 
       2.5019722600032118e-04, 
       1.9000000000003536e-04, 
       1.9364581385713668e-04, 
       2.0201482444573603e-04, 
       1.8711443864015038e-04, 
       2.5839651898496480e-04, 
       2.1036175554386041e-04, 
       1.8310049852630499e-04, 
       2.1323628006723061e-04, 
       2.5734716057206281e-04, 
       2.5743089644910885e-04, 
       1.8768844466384484e-04, 
       2.5721041159143667e-04, 
       2.2873284046830283e-04, 
       1.8381419059783772e-04, 
       1.8373054105177259e-04, 
       1.9208715411208081e-04, 
       1.9136383030135762e-04, 
       2.4613648827803301e-04, 
       1.8398793526488660e-04, 
       1.9148170962265894e-04, 
       1.8991932211251348e-04, 
       1.8301718056369935e-04, 
       2.0916589911031916e-04, 
       2.0916589911031916e-04, 
       2.5465622481112135e-04, 
       2.1042920538520888e-04, 
       2.3357907449821666e-04, 
       2.0157446081531952e-04, 
       2.5293798147816871e-04, 
       2.4680136085885587e-04, 
       1.8962725726023034e-04, 
       2.5352214108298379e-04, 
       2.5401006307258008e-04, 
       1.8506585105422172e-04, 
       2.5329975650256362e-04, 
       2.4570329357452949e-04, 
       2.1040019047558639e-04, 
       1.8100710744048041e-04, 
       2.4894750380896900e-04, 
       1.9266079420220857e-04, 
       2.0101547414854456e-04, 
       1.9154588585395057e-04, 
       2.0976045664812903e-04, 
       1.9767801580194218e-04, 
       2.3173613411575305e-04, 
       2.4961140387494765e-04, 
       1.8484834758946816e-04, 
       1.8795832026021766e-04, 
       2.1694108841672688e-04, 
       2.0031017447221213e-04, 
       2.5166341147980797e-04, 
       2.5000000000002429e-04, 
       1.8772357666824939e-04, 
       2.3070723835942681e-04, 
       1.9138619915697245e-04, 
       2.1676097639129112e-04, 
       2.3388007789404950e-04, 
       1.9792845493405998e-04, 
       2.4016847055572836e-04, 
       1.8288080989428381e-04, 
       2.4487984569213452e-04, 
       1.8685500154867415e-04, 
       1.8481437984612945e-04, 
       2.0127925897985981e-04, 
       2.4999999999997138e-04, 
       2.5000000000003481e-04, 
       2.2525195662859084e-04, 
       2.3000000000001971e-04, 
       1.9013709593681318e-04, 
       2.4286708485678668e-04, 
       2.0180861813451584e-04, 
       2.0180672383886332e-04, 
       2.4847692963982462e-04, 
       1.7961297366195908e-04, 
       1.8027247927328904e-04, 
       1.8488801166964967e-04, 
       2.0572120417329773e-04, 
       2.3965680206806559e-04, 
       2.4084883706121680e-04, 
       2.1004901317977760e-04, 
       1.8153011301996617e-04, 
       1.8408379737573766e-04, 
       1.7676503122597041e-04, 
       2.2367105859910202e-04, 
       1.9160693559992729e-04, 
       2.3268751928464093e-04, 
       1.9739136053101354e-04, 
       1.7820931170299865e-04, 
       2.0299558651710127e-04, 
       2.1091076571287041e-04, 
       1.9663308896173030e-04, 
       1.8180407058903316e-04, 
       2.0211345099280587e-04, 
       1.9600352025286717e-04, 
       2.4248445796108396e-04, 
       2.0172764377250673e-04, 
       1.9606317040211064e-04, 
       1.7599469631064931e-04, 
       1.7551403376186970e-04, 
       1.8482625573995801e-04, 
       2.0168528173187205e-04, 
       2.2502757509467014e-04, 
       1.8432344184098528e-04, 
       2.2300736763798123e-04, 
       1.8482567181899991e-04, 
       1.9997105137443404e-04, 
       2.1068342965162808e-04, 
       1.7128717857942222e-04, 
       1.7606127134602666e-04, 
       1.8033206423979982e-04, 
       2.2406408205660682e-04, 
       1.7827688212606363e-04, 
       2.3999999999999374e-04, 
       2.4000000000003950e-04, 
       1.7194860158688147e-04, 
       2.3782175769209788e-04, 
       2.0580680802089358e-04, 
       2.2000000000002351e-04, 
       2.1999999999995057e-04, 
       1.8658551422499358e-04, 
       2.3848118139772933e-04, 
       1.9367870971396595e-04, 
       1.9359689611873494e-04, 
       1.7728799875442106e-04, 
       2.3757420529788774e-04, 
       2.3754082277284175e-04, 
       1.7113888392909134e-04, 
       1.9749632742612429e-04, 
       2.1386488450319434e-04, 
       1.6778577499292386e-04, 
       1.7037364839777219e-04, 
       2.2704032547892812e-04, 
       2.3586279707232444e-04, 
       1.7281029799006583e-04, 
       2.0240905965637266e-04, 
       1.8398817698172149e-04, 
       1.6700703291699362e-04, 
       1.9583744192206410e-04, 
       1.7014302685374860e-04, 
       2.0059481787996570e-04, 
       1.8343730957174562e-04, 
       1.8346386811259903e-04, 
       1.6657129575150018e-04, 
       1.9409851180722372e-04, 
       1.7253911035893676e-04, 
       1.9434302097054112e-04, 
       1.6754071133750080e-04, 
       2.1430277254518254e-04, 
       1.6578532583570052e-04, 
       2.0686107107174050e-04, 
       2.1639676152145618e-04, 
       1.6966422399977850e-04, 
       1.9856731168577834e-04, 
       1.7121011482933504e-04, 
       1.9311546664742466e-04, 
       1.9379311619335080e-04, 
       2.2524652452152175e-04, 
       1.9799343492063197e-04, 
       2.0928092440440630e-04, 
       2.1903565740537752e-04, 
       2.2796511814887800e-04, 
       1.7439721721815556e-04, 
       2.2192173248291965e-04, 
       1.6784231918726031e-04, 
       1.6779805311004681e-04, 
       1.7167751827408282e-04, 
       2.3000000000001670e-04, 
       2.2999999999998298e-04, 
       2.0663896975973316e-04, 
       2.2630973133981694e-04, 
       1.9036427087779122e-04, 
       1.9498745670566302e-04, 
       1.9902389589481720e-04, 
       1.6813993766169713e-04, 
       2.2846232950801391e-04, 
       1.6614311356140298e-04, 
       2.1569629773348850e-04, 
       2.2670972471170632e-04, 
       2.0308133740099354e-04, 
       2.1573006537370509e-04, 
       2.0397043023891146e-04, 
       1.6196613886578174e-04, 
       1.9628123524880882e-04, 
       1.9516646448487890e-04, 
       1.6106585346314643e-04, 
       2.2593342332910619e-04, 
       1.6875879843956349e-04, 
       1.6249695851851690e-04, 
       1.7949644697835934e-04, 
       1.6264942576091197e-04, 
       1.6596949809967229e-04, 
       1.8713871021210196e-04, 
       1.6133453946285078e-04, 
       1.7114258491218544e-04, 
       1.7114258491218544e-04, 
       2.0084523783786906e-04, 
       2.0760747986977047e-04, 
       1.6069187677590830e-04, 
       2.2123784216931255e-04, 
       2.1989015474474825e-04, 
       2.2047863461495537e-04, 
       1.5701756970806815e-04, 
       1.6341026310311629e-04, 
       2.1143416979387201e-04, 
       1.8968520117372876e-04, 
       1.8126305538718947e-04, 
       1.6837003599658271e-04, 
       1.8409924372147897e-04, 
       2.1372596846036524e-04, 
       1.7120706074330927e-04, 
       2.2078081790338541e-04, 
       1.9369576697416860e-04, 
       2.1999991951612594e-04, 
       1.8440862727032401e-04, 
       2.1333411544866904e-04, 
       2.1360376873581611e-04, 
       2.2000000000004161e-04, 
       2.1999999999999133e-04, 
       2.1999999999999133e-04, 
       2.1707322430076679e-04, 
       2.1298648841903698e-04, 
       1.5753144099904970e-04, 
       1.6597891921253449e-04, 
       1.8000065076192084e-04, 
       1.8801528663936920e-04, 
       2.1328540252956699e-04, 
       1.6197988358398314e-04, 
       1.8323293887972743e-04, 
       2.0869137654530633e-04, 
       1.8518844177768955e-04, 
       1.9393073650336311e-04, 
       1.6144558566296582e-04, 
       1.7846965619131784e-04, 
       2.0264525529399853e-04, 
       2.0805646986402654e-04, 
       1.9369900378675813e-04, 
       1.7837547554451263e-04, 
       2.0326193386622010e-04, 
       1.8128772472467955e-04, 
       1.8534740672349764e-04, 
       1.5347791268923233e-04, 
       2.1307517255897068e-04, 
       1.6336809708337758e-04, 
       1.8806922083444393e-04, 
       2.0767041307857584e-04, 
       1.5283361663515540e-04, 
       2.1537183432233329e-04, 
       1.7977673232277746e-04, 
       1.8438286946670374e-04, 
       1.6710998590861655e-04, 
       2.0427732878232208e-04, 
       1.6337922945254524e-04, 
       1.7572142335229527e-04, 
       2.1221657390811666e-04, 
       1.5233607284881686e-04, 
       1.5220885917372449e-04, 
       1.7689560729984970e-04, 
       1.5418200866316962e-04, 
       1.5227018320278137e-04, 
       1.5175073934814036e-04, 
       2.0533123985914958e-04, 
       1.7714158346072268e-04, 
       1.9180318237326400e-04, 
       1.5364753948768174e-04, 
       1.8119700716329369e-04, 
       1.5304767319383489e-04, 
       1.5127112547973673e-04, 
       1.7453450646194251e-04, 
       1.7716135584757781e-04, 
       2.0497458057686503e-04, 
       1.6296616040678482e-04, 
       1.9323800624503843e-04, 
       1.7572567081883630e-04, 
       1.8016309052955936e-04, 
       2.1000000000004831e-04, 
       2.0999999999999453e-04, 
       1.7335398996921352e-04, 
       1.7949889001366091e-04, 
       1.7314943335367959e-04, 
       1.5717959413720259e-04, 
       1.9720482675601776e-04, 
       1.5445888500134557e-04, 
       1.8065464652859621e-04, 
       1.5700951644076052e-04, 
       1.5735326243464506e-04, 
       1.8736512259755672e-04, 
       1.5745823773509072e-04, 
       1.9828663629513409e-04, 
       1.6125795868505551e-04, 
       1.5265868395262572e-04, 
       1.9828243267669697e-04, 
       1.4564441896196820e-04, 
       1.6606385481762100e-04, 
       1.5694357160350017e-04, 
       2.0521315031001060e-04, 
       1.4835067256444091e-04, 
       1.8511250137498648e-04, 
       1.6563857520539497e-04, 
       2.0355046987827080e-04, 
       1.7084012408324963e-04, 
       1.8545597254881769e-04, 
       1.6078200857911208e-04, 
       1.7703171465073157e-04, 
       1.6808093371687422e-04, 
       2.0195387893156945e-04, 
       1.5276384361833517e-04, 
       1.7048108205428955e-04, 
       2.0393319663821084e-04, 
       1.7143634319190904e-04, 
       1.7334440029080007e-04, 
       1.5283624272491108e-04, 
       1.9638733974860508e-04, 
       1.5064748209291922e-04, 
       1.7273926036486944e-04, 
       1.4965243828875060e-04, 
       1.6100114638768894e-04, 
       1.8695592187965840e-04, 
       1.4935776078734252e-04, 
       1.9509952131857431e-04, 
       1.7301214719603949e-04, 
       1.4921049122693192e-04, 
       1.6806253229984327e-04, 
       1.4582371638546099e-04, 
       1.4505387186485862e-04, 
       1.6832711986389666e-04, 
       1.5824284812600129e-04, 
       1.4741234619142368e-04, 
       1.5003215934331006e-04, 
       1.4974491042026082e-04, 
       1.5448484685818759e-04, 
       2.0032291558471434e-04, 
       1.6776577785265344e-04, 
       1.8136899974741127e-04, 
       1.5045160886940456e-04, 
       1.6809081098942115e-04, 
       1.7391542753809186e-04, 
       1.7787694873044356e-04, 
       1.8925965442797522e-04, 
       1.5204199940079330e-04, 
       1.9333779417638925e-04, 
       1.9491434041936817e-04, 
       1.7015273483895194e-04, 
       1.6405278639003708e-04, 
       1.9591287644365099e-04, 
       1.6048840250699017e-04, 
       1.7090614880101718e-04, 
       1.5750053342482282e-04, 
       1.6224700211592231e-04, 
       1.4094048254804532e-04, 
       1.6418655247579252e-04, 
       1.5191581236513751e-04, 
       1.5085155884586116e-04, 
       1.9580975158241037e-04, 
       1.3977335124626779e-04, 
       1.8323301557543326e-04, 
       1.5868862222345300e-04, 
       1.3947099445262327e-04, 
       1.5331469205838718e-04, 
       1.9142210207546905e-04, 
       1.8527434238516005e-04, 
       1.6423658423509181e-04, 
       1.6371179391080153e-04, 
       1.4479878831178113e-04, 
       1.5448344415224405e-04, 
       1.9568244897546656e-04, 
       1.4965676015682653e-04, 
       1.6013967350063873e-04, 
       1.4199376076226629e-04, 
       1.4585282526477858e-04, 
       1.6905484986574012e-04, 
       1.5717477095481759e-04, 
       1.6129215428418010e-04, 
       1.5697641123787976e-04, 
       1.4058099612704894e-04, 
       1.8000000000003544e-04, 
       1.3808969425964352e-04, 
       1.6861407831494856e-04, 
       1.5127906401009260e-04, 
       1.8472270766498755e-04, 
       1.4042563149564810e-04, 
       1.8790454708843344e-04, 
       1.5560631326421470e-04, 
       1.6847647587883840e-04, 
       1.3814552934722073e-04, 
       1.5664767574528427e-04, 
       1.7673924693338515e-04, 
       1.6808456829862716e-04, 
       1.4722811629480642e-04, 
       1.7236350374315779e-04, 
       1.8670858580513135e-04, 
       1.7350883799623247e-04, 
       1.9336633401777217e-04, 
       1.7146228469559210e-04, 
       1.6074939472316392e-04, 
       1.6533497982448300e-04, 
       1.8837247007247047e-04, 
       1.4267385893685446e-04, 
       1.3697985715381976e-04, 
       1.6061452591610686e-04, 
       1.6901020225214402e-04, 
       1.5557476908500950e-04, 
       1.4043569529280711e-04, 
       1.3952728582179396e-04, 
       1.6269071104143640e-04, 
       1.7717482328486722e-04, 
       1.5583047187098386e-04, 
       1.5963586966641111e-04, 
       1.3627273806984973e-04, 
       1.4478112724730402e-04, 
       1.3800044333467644e-04, 
       1.6869240673341650e-04, 
       1.8468641086114192e-04, 
       1.5425360568931346e-04, 
       1.8147721078687896e-04, 
       1.8698054862125639e-04, 
       1.8143273047194322e-04, 
       1.4279758432263700e-04, 
       1.7459135458627894e-04, 
       1.8144070405429834e-04, 
       1.8999999999997716e-04, 
       1.7235785412461206e-04, 
       1.9000000000003186e-04, 
       1.3831394326416246e-04, 
       1.3401160986803082e-04, 
       1.8534744373115895e-04, 
       1.8797330621865912e-04, 
       1.7762252426968862e-04, 
       1.3415369015985274e-04, 
       1.3884429998533781e-04, 
       1.5539414766437560e-04, 
       1.8542381956917657e-04, 
       1.5651328186031089e-04, 
       1.4378668797443772e-04, 
       1.8743270906176014e-04, 
       1.8727409477743440e-04, 
       1.6189903351130369e-04, 
       1.8472843876819723e-04, 
       1.3994167428429469e-04, 
       1.5514367441577396e-04, 
       1.4360481043503126e-04, 
       1.6668915002311486e-04, 
       1.7681397782802461e-04, 
       1.3301531827745097e-04, 
       1.5503167893186795e-04, 
       1.6118269620053196e-04, 
       1.7981435638704521e-04, 
       1.3358004611482271e-04, 
       1.5697837275173866e-04, 
       1.7272039144944563e-04, 
       1.4315926172950912e-04, 
       1.6366238677552582e-04, 
       1.7111469344669800e-04, 
       1.6069172127420233e-04, 
       1.3313544107346838e-04, 
       1.3830524638874900e-04, 
       1.8394363317028101e-04, 
       1.8469945883789560e-04, 
       1.6997257040558299e-04, 
       1.3254140787305666e-04, 
       1.3162838125625883e-04, 
       1.3402651286070901e-04, 
       1.6424848298203311e-04, 
       1.7155182767531637e-04, 
       1.3435871167122207e-04, 
       1.5864868114251216e-04, 
       1.3020900735620942e-04, 
       1.3748199961562916e-04, 
       1.8122640806682297e-04, 
       1.7797461268866533e-04, 
       1.2995120570590866e-04, 
       1.8224854038035373e-04, 
       1.5372929952838741e-04, 
       1.7147780525383379e-04, 
       1.5692299398344675e-04, 
       1.8000126461232199e-04, 
       1.3471820445830121e-04, 
       1.7265866611622849e-04, 
       1.6510178981336617e-04, 
       1.5167074939250041e-04, 
       1.3645675563697896e-04, 
       1.4905125226267015e-04, 
       1.8206135548343532e-04, 
       1.3150851948068703e-04, 
       1.3001821260680025e-04, 
       1.3304357075601605e-04, 
       1.8053164075882628e-04, 
       1.8149866232012888e-04, 
       1.3098901525830566e-04, 
       1.8135424313909349e-04, 
       1.6649003290360187e-04, 
       1.8067195417206176e-04, 
       1.3823177827332168e-04, 
       1.3382103744599041e-04, 
       1.7107543102580782e-04, 
       1.3998058491970205e-04, 
       1.3984730741739923e-04, 
       1.2961744980718995e-04, 
       1.8004326891682138e-04, 
       1.5110296839731299e-04, 
       1.4493623656794641e-04, 
       1.7947005326787721e-04, 
       1.7237121244733896e-04, 
       1.6539622910215375e-04, 
       1.4877349471269875e-04, 
       1.2673987228163389e-04, 
       1.3578653978647267e-04, 
       1.4868519039975059e-04, 
       1.5230974562581280e-04, 
       1.5017886330910655e-04, 
       1.3297488089593275e-04, 
       1.5899921607069256e-04, 
       1.5368384186934469e-04, 
       1.3191312718210553e-04, 
       1.2696407626514773e-04, 
       1.5170994093807651e-04, 
       1.3167433558691549e-04, 
       1.3746257606846103e-04, 
       1.4056239654250532e-04, 
       1.7395895491990708e-04, 
       1.5250659308393866e-04, 
       1.3851355897702617e-04, 
       1.7248987842557044e-04, 
       1.2545247401294949e-04, 
       1.7652812379601683e-04, 
       1.7639098637550879e-04, 
       1.2558842826441554e-04, 
       1.5258696157181574e-04, 
       1.6314205839531795e-04, 
       1.5010429102565870e-04, 
       1.6692316310591026e-04, 
       1.7623471534650823e-04, 
       1.3638480619105619e-04, 
       1.2698334822981296e-04, 
       1.4563891638638389e-04, 
       1.6316837348210196e-04, 
       1.5989132434140386e-04, 
       1.4594466510570853e-04, 
       1.3705092549703851e-04, 
       1.2699458511659080e-04, 
       1.2578804470588149e-04, 
       1.3459641849085757e-04, 
       1.5100790947549653e-04, 
       1.7503822150888995e-04, 
       1.6690659211152781e-04, 
       1.4933343583298399e-04, 
       1.5015103978596137e-04, 
       1.6997398980867585e-04, 
       1.7147897669109009e-04, 
       1.4078683743712621e-04, 
       1.2456807174991645e-04, 
       1.4897632606892931e-04, 
       1.2508584143665360e-04, 
       1.6733846333524217e-04, 
       1.4979503116421789e-04, 
       1.2471487068012180e-04, 
       1.7312076462215172e-04, 
       1.3377824080753448e-04, 
       1.5078179423595403e-04, 
       1.2357885193811562e-04, 
       1.2281095327576060e-04, 
       1.7317601628547564e-04, 
       1.7228636586531868e-04, 
       1.6662503829660577e-04, 
       1.4354841240928103e-04, 
       1.6329295804790758e-04, 
       1.5463155548125698e-04, 
       1.4272229011325714e-04, 
       1.7148373939618452e-04, 
       1.7044889280124891e-04, 
       1.2478870530050501e-04, 
       1.2526124135798093e-04, 
       1.6987321698639615e-04, 
       1.2205088605318080e-04, 
       1.2259412616567945e-04, 
       1.2624285360254373e-04, 
       1.2156779067705010e-04, 
       1.2260718398569476e-04, 
       1.6495748037921964e-04, 
       1.7050329921991183e-04, 
       1.5769812062799596e-04, 
       1.3922113718203529e-04, 
       1.2402815308644842e-04, 
       1.4142898124632093e-04, 
       1.3113536514707216e-04, 
       1.7000000000000671e-04, 
       1.6999999999995700e-04, 
       1.6999999999997120e-04, 
       1.6999999999995242e-04, 
       1.7000000000001316e-04, 
       1.7000000000004143e-04, 
       1.5836635948176370e-04, 
       1.4323182400550918e-04, 
       1.2051280209077070e-04, 
       1.6933020785728678e-04, 
       1.2214096917977907e-04, 
       1.4297238978008749e-04, 
       1.3255724451334550e-04, 
       1.4082411518446234e-04, 
       1.2104567899002264e-04, 
       1.1998948684221611e-04, 
       1.5124098756580422e-04, 
       1.6297769182487237e-04, 
       1.6108842966683373e-04, 
       1.6673915867429089e-04, 
       1.2583719669620791e-04, 
       1.3555217026789678e-04, 
       1.2100509060378924e-04, 
       1.4374892135746671e-04, 
       1.4034065971763686e-04, 
       1.1911023529752493e-04, 
       1.3246510902477780e-04, 
       1.2100694943255494e-04, 
       1.3235372098767631e-04, 
       1.6659741249805583e-04, 
       1.3235629857088934e-04, 
       1.4563045364439053e-04, 
       1.2282316378989561e-04, 
       1.6018428305646017e-04, 
       1.3660934692626177e-04, 
       1.1930483914899852e-04, 
       1.2672233416070899e-04, 
       1.5255282676010178e-04, 
       1.3791111490004613e-04, 
       1.5351099714353589e-04, 
       1.6512026502715405e-04, 
       1.3472763625454182e-04, 
       1.3461814473396763e-04, 
       1.2651014792078676e-04, 
       1.3221547772756609e-04, 
       1.5501714579662840e-04, 
       1.3765821596946134e-04, 
       1.1999999999996169e-04, 
       1.6212370885442713e-04, 
       1.2254781805196249e-04, 
       1.1875609529350585e-04, 
       1.6032233847649378e-04, 
       1.2137448150433576e-04, 
       1.3890794005146301e-04, 
       1.3646503948404294e-04, 
       1.2667737743892604e-04, 
       1.4293205659205322e-04, 
       1.3376305077388991e-04, 
       1.5410257724665451e-04, 
       1.3360368511948490e-04, 
       1.1948612455100513e-04, 
       1.6406438658236484e-04, 
       1.4341839693145545e-04, 
       1.2553963701827516e-04, 
       1.2080205470342870e-04, 
       1.1624139046844870e-04, 
       1.5009564732489745e-04, 
       1.2874764157934303e-04, 
       1.5515271639643104e-04, 
       1.6249546908462983e-04, 
       1.4537388368918969e-04, 
       1.3727164075298123e-04, 
       1.2134573984300316e-04, 
       1.2866004553685226e-04, 
       1.3137749334912304e-04, 
       1.6202024977959493e-04, 
       1.3447652426224572e-04, 
       1.2080868968142446e-04, 
       1.3307361871846092e-04, 
       1.3673285286282282e-04, 
       1.1916645625928418e-04, 
       1.2779200002936946e-04, 
       1.1504884334514827e-04, 
       1.3651700955581025e-04, 
       1.3918573043331052e-04, 
       1.1884542819529089e-04, 
       1.2354947509355887e-04, 
       1.2353154213487250e-04, 
       1.5078092705759824e-04, 
       1.3430740748761796e-04, 
       1.3385917438640268e-04, 
       1.6085911131692859e-04, 
       1.2917038788256934e-04, 
       1.5441962016020234e-04, 
       1.5806398769554927e-04, 
       1.3928849493649847e-04, 
       1.2729276498288231e-04, 
       1.2490473825607382e-04, 
       1.5794518797378760e-04, 
       1.5999999999996792e-04, 
       1.5999999999998429e-04, 
       1.6000000000004100e-04, 
       1.3283825786922186e-04, 
       1.5868968904286855e-04, 
       1.2237346123927174e-04, 
       1.3106978826400413e-04, 
       1.1710718530496936e-04, 
       1.5291352160660453e-04, 
       1.3234232024356509e-04, 
       1.2528599539701659e-04, 
       1.4031607694761682e-04, 
       1.2182659129210776e-04, 
       1.3240804547160070e-04, 
       1.3581708875018016e-04, 
       1.2755391665463244e-04, 
       1.2340626388959849e-04, 
       1.1469621817116149e-04, 
       1.1566233929243237e-04, 
       1.1409883448653354e-04, 
       1.4123653097838760e-04, 
       1.5306744284361730e-04, 
       1.1620277443036803e-04, 
       1.1620277443036803e-04, 
       1.1999999999997950e-04, 
       1.1831940008078334e-04, 
       1.3903738383606076e-04, 
       1.1649744245099836e-04, 
       1.2667072595207900e-04, 
       1.1684692351786564e-04, 
       1.5622430796997811e-04, 
       1.2982310334060259e-04, 
       1.4294858053958301e-04, 
       1.1812914898764387e-04, 
       1.3822158911563733e-04, 
       1.1084622685662520e-04, 
       1.5519342601126037e-04, 
       1.1452998891652252e-04, 
       1.5199086443017207e-04, 
       1.1261947678254614e-04, 
       1.1046683477145898e-04, 
       1.1317622865794402e-04, 
       1.1493464597422978e-04, 
       1.4129601915754080e-04, 
       1.4129071708555689e-04, 
       1.2636683167190177e-04, 
       1.2832091074941178e-04, 
       1.1794897989295466e-04, 
       1.1307728290494497e-04, 
       1.1754348014736108e-04, 
       1.5337477260875311e-04, 
       1.3255702051876312e-04, 
       1.1696579807786919e-04, 
       1.4014974873959748e-04, 
       1.4763496174399716e-04, 
       1.2110334345667254e-04, 
       1.1069535932945421e-04, 
       1.0857712965206902e-04, 
       1.4184203835657574e-04, 
       1.0944887232362502e-04, 
       1.2514575060135780e-04, 
       1.2725141563258173e-04, 
       1.2622924458508713e-04, 
       1.2482671455285517e-04, 
       1.3930521400124307e-04, 
       1.5127372387559298e-04, 
       1.1261112535771039e-04, 
       1.2669553007819232e-04, 
       1.4347562507722832e-04, 
       1.4347562507722832e-04, 
       1.5199242142969286e-04, 
       1.2538101989688930e-04, 
       1.1852693085166877e-04, 
       1.4735630410222568e-04, 
       1.2600890317514653e-04, 
       1.3999999999995732e-04, 
       1.0817077410181477e-04, 
       1.2470288317636150e-04, 
       1.5133687805832794e-04, 
       1.3563376561304285e-04, 
       1.1133216725428975e-04, 
       1.3195787263301729e-04, 
       1.2570346163300741e-04, 
       1.0693303412412506e-04, 
       1.0740299411521515e-04, 
       1.1481724495420005e-04, 
       1.1373300665408960e-04, 
       1.0747587535989394e-04, 
       1.5075339249831485e-04, 
       1.4975515597021114e-04, 
       1.2000000000001801e-04, 
       1.1388755294784691e-04, 
       1.2084706312818877e-04, 
       1.4862283122155624e-04, 
       1.1765421926424346e-04, 
       1.2966397099610125e-04, 
       1.0941624356534514e-04, 
       1.1250621571916749e-04, 
       1.2895476011064269e-04, 
       1.0639779180408200e-04, 
       1.1844757370983131e-04, 
       1.5000000000000552e-04, 
       1.5000000000001682e-04, 
       1.2014835262670070e-04, 
       1.3378991976003242e-04, 
       1.2892809094973849e-04, 
       1.4966869162338458e-04, 
       1.2539476347891399e-04, 
       1.3768223742472233e-04, 
       1.0774047392095315e-04, 
       1.4918791531666814e-04, 
       1.0720063407423534e-04, 
       1.4365571365526666e-04, 
       1.1956684471295402e-04, 
       1.2940163648088166e-04, 
       1.4651833680541941e-04, 
       1.1230975415157783e-04, 
       1.1372775570048073e-04, 
       1.2006199351089215e-04, 
       1.4076754474915380e-04, 
       1.3370040941257624e-04, 
       1.3918162591177656e-04, 
       1.4438313787837632e-04, 
       1.4000000000002617e-04, 
       1.0442461594807061e-04, 
       1.0467528665442617e-04, 
       1.2542789901083757e-04, 
       1.0514410533665720e-04, 
       1.0635545593445169e-04, 
       1.2521171451199475e-04, 
       1.0978616453647124e-04, 
       1.2412340541638551e-04, 
       1.4131568193382252e-04, 
       1.0534815337802291e-04, 
       1.0818835832184637e-04, 
       1.2141816739885772e-04, 
       1.3151650030644637e-04, 
       1.3408230692314239e-04, 
       1.4367011318689818e-04, 
       1.1623134547530660e-04, 
       1.1548496966378767e-04, 
       1.1837479082656069e-04, 
       1.4494905168930411e-04, 
       1.0660132758895862e-04, 
       1.0262261626823875e-04, 
       1.4049320350993343e-04, 
       1.0218658938222781e-04, 
       1.0211286719089522e-04, 
       1.3820003418191091e-04, 
       1.4114019639483374e-04, 
       1.0803782563324318e-04, 
       1.1197778404577340e-04, 
       1.4258455595401887e-04, 
       1.0770254138635894e-04, 
       1.1563941242635600e-04, 
       1.0362731980567512e-04, 
       1.0572664984200582e-04, 
       1.1283749431607226e-04, 
       1.0179076859023901e-04, 
       1.1219914886551592e-04, 
       1.1140518212099334e-04, 
       1.0324492131668983e-04, 
       1.1874615253062522e-04, 
       1.1686431873005142e-04, 
       1.2201521390140522e-04, 
       1.0107222858261959e-04, 
       1.2853823864404166e-04, 
       1.4018312132512908e-04, 
       1.3378133431988259e-04, 
       1.3705960204063000e-04, 
       1.2175333229378890e-04, 
       1.0203550124332017e-04, 
       1.0173692236646140e-04, 
       1.4130608227444114e-04, 
       1.0006839951802925e-04, 
       1.1831598494419209e-04, 
       1.3012619269019718e-04, 
       1.4111199720164762e-04, 
       1.2916151752623105e-04, 
       1.1808712619179572e-04, 
       1.3964999991624867e-04, 
       1.0576018239523485e-04, 
       1.2999999999998755e-04, 
       1.4011150203667561e-04, 
       1.0114166889765038e-04, 
       1.4091817388760367e-04, 
       1.0290765437197313e-04, 
       1.0631121218973794e-04, 
       1.0209735684289537e-04, 
       1.1621921026655905e-04, 
       1.0198260286364570e-04, 
       1.2238334351968982e-04, 
       1.0026892664720046e-04, 
       1.3935175795721396e-04, 
       9.9651867176816126e-05, 
       1.0446607232764233e-04, 
       1.0243272072023020e-04, 
       1.0996229910403271e-04, 
       1.0015578301497623e-04, 
       1.3859443636844601e-04, 
       1.4000000000002763e-04, 
       1.4000000000000110e-04, 
       1.4000000000001912e-04, 
       1.0047110311622120e-04, 
       1.3917725524309924e-04, 
       1.1332641770630674e-04, 
       9.9245283501956925e-05, 
       1.1332325299373979e-04, 
       1.0397150132229449e-04, 
       1.3931611227052532e-04, 
       1.0287003962071929e-04, 
       9.8936034130486810e-05, 
       1.0771618516862094e-04, 
       9.9236744127034727e-05, 
       1.3336761976014014e-04, 
       9.8454909262513253e-05, 
       1.1454772960476021e-04, 
       1.0382973756918650e-04, 
       1.0381703861675464e-04, 
       1.1647356190914754e-04, 
       1.0750380452019836e-04, 
       1.3796086552233250e-04, 
       1.3179912015192982e-04, 
       9.7650026447738004e-05, 
       1.0911505144740881e-04, 
       1.1474396948754765e-04, 
       1.3740054430499588e-04, 
       1.0107445344115580e-04, 
       1.0172119112808597e-04, 
       1.0589375505509555e-04, 
       1.3433879059708125e-04, 
       9.9520203257500860e-05, 
       1.0080972528431748e-04, 
       1.2738091164652687e-04, 
       1.0115041605310192e-04, 
       1.3699878276777838e-04, 
       9.9815435240150987e-05, 
       1.0233821099768191e-04, 
       1.3558265282569203e-04, 
       1.3443016841195907e-04, 
       9.9602814815851821e-05, 
       9.8896915254324149e-05, 
       9.8471117341948373e-05, 
       1.0238717490652405e-04, 
       1.1384100125443397e-04, 
       1.3565208292782194e-04, 
       1.3565208292782194e-04, 
       9.7936817245821477e-05, 
       1.0188947091343735e-04, 
       1.2172437555884551e-04, 
       1.1701997164908569e-04, 
       1.1730397504441889e-04, 
       1.2566732000685298e-04, 
       1.0223417494087606e-04, 
       1.0559602316238035e-04, 
       1.0216005169840250e-04, 
       9.8384888555681058e-05, 
       1.3432804857391742e-04, 
       1.2596679597297601e-04, 
       1.0149632435041840e-04, 
       1.3259954107970376e-04, 
       1.2698073621658672e-04, 
       1.3459334918594374e-04, 
       9.5514286559908107e-05, 
       9.6383899944435734e-05, 
       1.0425789005995188e-04, 
       1.2789815959195189e-04, 
       1.1000000000000223e-04, 
       1.1017384143681400e-04, 
       1.1442018486182442e-04, 
       1.0002549329913187e-04, 
       1.1486835674375766e-04, 
       1.0982579096878552e-04, 
       1.2910408027421078e-04, 
       9.8959572874275509e-05, 
       9.4256014989025375e-05, 
       9.4256014989025375e-05, 
       1.3031034043556151e-04, 
       1.3182004837130717e-04, 
       1.0951152827246828e-04, 
       9.4837594760299225e-05, 
       1.1597438149184454e-04, 
       1.2989818785394248e-04, 
       1.0013314962572331e-04, 
       1.2419342039766089e-04, 
       1.2535159090224424e-04, 
       1.0375089075046001e-04, 
       9.8356347606335729e-05, 
       1.1023233361014229e-04, 
       1.2848027774813530e-04, 
       1.0112286679496412e-04, 
       1.1610510461304639e-04, 
       1.0815559382922234e-04, 
       1.1705577910857745e-04, 
       9.8122592882688494e-05, 
       9.4998393472230520e-05, 
       1.1730450890099048e-04, 
       1.0179678117667963e-04, 
       1.0035429191784983e-04, 
       1.2112012132480230e-04, 
       9.7861231782973759e-05, 
       9.3480574828878500e-05, 
       9.3407186112010699e-05, 
       1.0232662022235787e-04, 
       1.1999999999996249e-04, 
       1.3014665208967025e-04, 
       9.4491927704513307e-05, 
       1.2999999999996090e-04, 
       1.2999999999998126e-04, 
       1.0795512588910247e-04, 
       1.1418421445733480e-04, 
       9.3758691321809199e-05, 
       1.0820998240774132e-04, 
       1.2853839836687069e-04, 
       1.0518839873352059e-04, 
       1.1985854916816670e-04, 
       1.2727040708528000e-04, 
       1.2140251106057449e-04, 
       1.0178451768194304e-04, 
       1.0548048010985225e-04, 
       1.2417365720128617e-04, 
       9.5771747637823004e-05, 
       9.6113228668302656e-05, 
       9.6392803088793920e-05, 
       1.0904133612419888e-04, 
       9.1251486322646526e-05, 
       1.2511240522102384e-04, 
       1.2156877228499928e-04, 
       1.0783892726215754e-04, 
       1.0673748842660715e-04, 
       1.2300138730199649e-04, 
       9.9042889148017181e-05, 
       1.1531533879515578e-04, 
       9.8108920269365056e-05, 
       1.2695171801742754e-04, 
       1.0674707791993266e-04, 
       9.0691564594846721e-05, 
       9.1465954854871149e-05, 
       1.2698730034943243e-04, 
       1.0639365224193430e-04, 
       1.2622047111983365e-04, 
       9.3314555674562971e-05, 
       1.0643414037643173e-04, 
       1.0006928492220438e-04, 
       1.0122842793427244e-04, 
       1.0413257167419686e-04, 
       1.2202908471071597e-04, 
       1.1558402970936500e-04, 
       1.1893902020235161e-04, 
       1.2686574144570441e-04, 
       1.0013522019726030e-04, 
       1.2625355951340728e-04, 
       1.0268138186086510e-04, 
       1.2659706031755208e-04, 
       1.0556417008166629e-04, 
       1.2215988009023376e-04, 
       1.0489786068872611e-04, 
       1.2554221357798077e-04, 
       1.2220540041224276e-04, 
       1.1998772877047616e-04, 
       1.0101631069754862e-04, 
       1.2587880531051972e-04, 
       1.2557299381695951e-04, 
       1.0797480542713050e-04, 
       9.0794169709534130e-05, 
       1.2476322855077966e-04, 
       9.2025290859875519e-05, 
       1.2317095979725711e-04, 
       1.2179540465821959e-04, 
       1.0422621574539337e-04, 
       1.0660463581203187e-04, 
       1.2481755282695871e-04, 
       1.1941182032651300e-04, 
       1.0634151950527153e-04, 
       1.2496644898502880e-04, 
       9.3272550474342295e-05, 
       1.2015602811342524e-04, 
       8.9587828052505017e-05, 
       1.2472434483245748e-04, 
       1.1607579462479840e-04, 
       1.0133778800569856e-04, 
       9.0397152930314905e-05, 
       9.5966708415305051e-05, 
       9.1711100148394165e-05, 
       9.5930965523164334e-05, 
       1.2311231044665933e-04, 
       1.2163505559235126e-04, 
       9.1477576381338320e-05, 
       1.0084261793262089e-04, 
       1.2121886649936345e-04, 
       1.1770932008867974e-04, 
       9.7954673428925570e-05, 
       8.9502160494969282e-05, 
       1.2054276675425823e-04, 
       1.1466236847341514e-04, 
       1.0585991117247721e-04, 
       1.0274609417556479e-04, 
       8.8600171845902605e-05, 
       8.7704092154334347e-05, 
       8.9315872249003326e-05, 
       1.2228800668007871e-04, 
       1.2293626684163351e-04, 
       1.2199752773816792e-04, 
       9.0946331255193082e-05, 
       1.1173650310176072e-04, 
       1.0982298476195096e-04, 
       1.1801708365256981e-04, 
       1.1000000000004272e-04, 
       1.0122364224804299e-04, 
       1.2236024967769176e-04, 
       8.9655078870332785e-05, 
       9.1097136627500856e-05, 
       1.2204140303290559e-04, 
       1.0161859758221363e-04, 
       8.7418873967152577e-05, 
       9.6245012584474110e-05, 
       8.7376072823676856e-05, 
       8.8407617824617024e-05, 
       9.1052290109196643e-05, 
       8.6419235147234828e-05, 
       1.0406454839978003e-04, 
       9.8654240994702175e-05, 
       1.0357545271392300e-04, 
       1.0113194940507710e-04, 
       1.2115457430816763e-04, 
       1.2019271712722973e-04, 
       1.0288756658645369e-04, 
       1.1625487917200916e-04, 
       9.9094884208793244e-05, 
       1.1369437051621789e-04, 
       8.5662713138400716e-05, 
       1.0012770037634402e-04, 
       1.2078056948940330e-04, 
       9.0170591389285076e-05, 
       9.9097382064558664e-05, 
       1.1609179274745751e-04, 
       8.9788167153167516e-05, 
       1.0041456237337234e-04, 
       1.0179873231550728e-04, 
       8.5229411248159475e-05, 
       8.8606037494956086e-05, 
       8.6952370494365158e-05, 
       1.1999999999995943e-04, 
       1.2000000000001524e-04, 
       1.2000000000003645e-04, 
       9.2877475705977646e-05, 
       1.0445423371330962e-04, 
       9.0334795464594496e-05, 
       9.7689497658498618e-05, 
       1.0106212616639630e-04, 
       9.9453914457817681e-05, 
       1.1930229298001375e-04, 
       1.0495967301273962e-04, 
       1.0477045024903581e-04, 
       1.1055556797614898e-04, 
       9.8903490204335903e-05, 
       1.0693133197076152e-04, 
       8.5677933643574724e-05, 
       8.5677933643574724e-05, 
       9.8134642287844878e-05, 
       9.8581826616870937e-05, 
       1.0593145891981754e-04, 
       8.4987567858036368e-05, 
       1.1761938464274281e-04, 
       9.0236594731326979e-05, 
       1.1776210857319727e-04, 
       9.8980467336778099e-05, 
       9.6954530401830686e-05, 
       8.7522010043932408e-05, 
       9.7956932164348784e-05, 
       9.2894242498151830e-05, 
       1.1015688448530337e-04, 
       8.4601829289378865e-05, 
       8.6705955591731183e-05, 
       1.0533480662038682e-04, 
       1.0248771679859613e-04, 
       1.1478419819692594e-04, 
       1.1728751373722064e-04, 
       1.1711596197689180e-04, 
       8.7108145512102727e-05, 
       9.8330174610838648e-05, 
       9.1627796293045007e-05, 
       9.1473315705062227e-05, 
       1.1369570140219362e-04, 
       9.7377243901028171e-05, 
       1.1189917411427915e-04, 
       8.9093270563762024e-05, 
       8.7625352839321998e-05, 
       1.0675475350728433e-04, 
       1.0078395211880273e-04, 
       8.2631681505521398e-05, 
       8.7784966417625435e-05, 
       8.9631518315566570e-05, 
       8.8033305434181625e-05, 
       9.6184779404241734e-05, 
       8.9076565155085408e-05, 
       1.1495753769498076e-04, 
       8.5239328469476067e-05, 
       1.0389061032995724e-04, 
       1.1565596842212520e-04, 
       9.3687855810380420e-05, 
       8.5605681302257711e-05, 
       9.5787118111795202e-05, 
       8.2020543727679475e-05, 
       1.0458952989839777e-04, 
       8.8131439106634856e-05, 
       8.8125796515598880e-05, 
       8.8502191574219840e-05, 
       1.1431073333110567e-04, 
       1.1297654847357828e-04, 
       8.1220362210218346e-05, 
       8.2118500965781840e-05, 
       1.1363894119894263e-04, 
       8.1967575615574678e-05, 
       1.1216900814591960e-04, 
       8.3773204189794954e-05, 
       9.1990809807271769e-05, 
       9.5633570613663671e-05, 
       8.9205427571849058e-05, 
       8.2196948733038349e-05, 
       9.8002247116442029e-05, 
       8.1664488244470945e-05, 
       8.3492719622401334e-05, 
       8.1714283930089601e-05, 
       8.4105776540539196e-05, 
       8.7803851265448200e-05, 
       8.7803851265448200e-05, 
       1.1384795390375729e-04, 
       1.0880962120120442e-04, 
       8.5498377678002642e-05, 
       8.1106674862278249e-05, 
       8.0554666725152248e-05, 
       8.4983452972775265e-05, 
       8.7112378448848881e-05, 
       8.3666828374008421e-05, 
       8.0735350330282394e-05, 
       9.2953156227155957e-05, 
       8.8868849880820806e-05, 
       8.8849497878280381e-05, 
       8.1539993300187700e-05, 
       1.1254317118712444e-04, 
       9.6014568238159657e-05, 
       8.0375086117522349e-05, 
       8.0308740731786080e-05, 
       1.0854498369639651e-04, 
       1.1188913633649453e-04, 
       9.4576917685468073e-05, 
       1.1060472072735106e-04, 
       7.9765619810371097e-05, 
       8.4141442889284281e-05, 
       9.3552228751237761e-05, 
       9.3307353380867191e-05, 
       7.9999999999995141e-05, 
       9.3147583987742016e-05, 
       7.9400752936796209e-05, 
       9.9731252719811059e-05, 
       9.7405426826906955e-05, 
       9.0037963246403463e-05, 
       8.0812754892571018e-05, 
       8.1081648711590351e-05, 
       7.9282360529885667e-05, 
       8.7077408171290841e-05, 
       1.0891494191428356e-04, 
       8.0448180345346309e-05, 
       8.9553197089364632e-05, 
       8.5462435155135100e-05, 
       1.1049555081569100e-04, 
       9.3716531664271644e-05, 
       1.0390609225783435e-04, 
       8.1992047494702450e-05, 
       8.1992047494702450e-05, 
       8.3866376340423709e-05, 
       9.2679181045595126e-05, 
       7.9451628765122808e-05, 
       8.1447750919663616e-05, 
       9.5109104604508283e-05, 
       8.2383854676251999e-05, 
       1.0384410671968337e-04, 
       8.0501389709615388e-05, 
       8.7469207866319981e-05, 
       9.2054941032143590e-05, 
       9.1663231372287154e-05, 
       7.8068005748901069e-05, 
       7.9010224898634157e-05, 
       9.1402425047733125e-05, 
       8.8769469340950493e-05, 
       9.1103236320504681e-05, 
       9.0990634039100690e-05, 
       9.0646939422335467e-05, 
       8.1599544067641026e-05, 
       1.1000000000000944e-04, 
       1.0999999999997696e-04, 
       1.0958092995340969e-04, 
       1.0474686971425728e-04, 
       9.1305309677866555e-05, 
       8.9920496540599483e-05, 
       8.7534933561809596e-05, 
       8.5900358355785856e-05, 
       8.1449895027852748e-05, 
       1.0027569953737355e-04, 
       9.7530978379340675e-05, 
       8.3552787437219531e-05, 
       1.0495715211555467e-04, 
       7.9290166264358301e-05, 
       8.1601002496451241e-05, 
       8.9574910359454805e-05, 
       1.0854808640327811e-04, 
       7.8834494569848411e-05, 
       9.7130497056889585e-05, 
       7.7423594325735673e-05, 
       9.5031734645498737e-05, 
       7.7722290231086667e-05, 
       8.8521362495953083e-05, 
       7.7332930546429989e-05, 
       9.4455242035968405e-05, 
       9.9357670871807628e-05, 
       1.0868015946459822e-04, 
       7.9007586742574853e-05, 
       8.2872427195313414e-05, 
       7.9209879527786600e-05, 
       1.0186600739615422e-04, 
       1.0000000000000221e-04, 
       1.0000000000000334e-04, 
       1.0000000000000296e-04, 
       1.0769368652830217e-04, 
       1.0841867129570088e-04, 
       7.6996943788860871e-05, 
       7.8984749459162727e-05, 
       1.0208566978379656e-04, 
       1.0530876896041489e-04, 
       9.2593739023413035e-05, 
       8.9576808198524039e-05, 
       1.0753789779578029e-04, 
       1.0809940778217144e-04, 
       1.0769906396535702e-04, 
       7.6511598328092326e-05, 
       1.0750431097856745e-04, 
       7.8642804779853839e-05, 
       8.1034375191387626e-05, 
       1.0614597904593044e-04, 
       8.2270520820083523e-05, 
       8.2749208289396693e-05, 
       8.2724047965198520e-05, 
       8.8080738175531535e-05, 
       1.0198645734688157e-04, 
       1.0482507710597043e-04, 
       1.0540567930512304e-04, 
       1.0540567930512304e-04, 
       8.0700768500193207e-05, 
       8.9391332492663283e-05, 
       1.0098715166292284e-04, 
       1.0372625515196713e-04, 
       1.0571803801507745e-04, 
       7.9064782209315631e-05, 
       8.9359887661122648e-05, 
       7.8871210961207349e-05, 
       7.7162027057154278e-05, 
       1.0498765247598911e-04, 
       8.0000000000002338e-05, 
       1.0634007089865870e-04, 
       1.0613675761326501e-04, 
       7.9426496276440808e-05, 
       7.9505763256152297e-05, 
       7.7210598505739897e-05, 
       1.0635622983306603e-04, 
       7.6081911435969251e-05, 
       8.6204453092237601e-05, 
       1.0049582382262412e-04, 
       1.0545754108933778e-04, 
       7.5951569526638461e-05, 
       8.1283575624385520e-05, 
       1.0275252852369799e-04, 
       1.0563494487360308e-04, 
       7.8328748657451489e-05, 
       9.0200161963040226e-05, 
       7.6865994572887281e-05, 
       1.0609076826160927e-04, 
       9.7885338834587777e-05, 
       1.0235963700913029e-04, 
       7.6916078204174019e-05, 
       7.9078988306317309e-05, 
       9.8139948713311121e-05, 
       1.0531435183628403e-04, 
       9.1120004604576155e-05, 
       7.8868698026167742e-05, 
       9.2431396962727239e-05, 
       1.0524625884475417e-04, 
       1.0011760070079590e-04, 
       1.0199926476280730e-04, 
       8.2617014935499166e-05, 
       7.7463139948846538e-05, 
       8.3031926848547400e-05, 
       7.9523091247022258e-05, 
       7.9338199162599327e-05, 
       7.7713931829944308e-05, 
       8.2085366331088942e-05, 
       1.0077457514058436e-04, 
       8.5865373262364490e-05, 
       7.4137928220764966e-05, 
       1.0470154553259903e-04, 
       8.8858817226593657e-05, 
       9.1210272847435118e-05, 
       7.8140118010677057e-05, 
       8.7126234829113802e-05, 
       8.7024804486610045e-05, 
       8.4241039748788701e-05, 
       8.9027636463239628e-05, 
       7.9428313928899422e-05, 
       1.0114503711711229e-04, 
       9.9154942551752406e-05, 
       8.9999999999996374e-05, 
       1.0385829291425154e-04, 
       7.4517529755463297e-05, 
       8.5689554153714628e-05, 
       8.5663470548377846e-05, 
       7.3996973670928200e-05, 
       7.5317401735263868e-05, 
       7.7898976578780359e-05, 
       7.9078566880419278e-05, 
       8.6128147825981612e-05, 
       8.6050275878035237e-05, 
       8.9799683953218604e-05, 
       1.0274958615134018e-04, 
       8.6608870733724254e-05, 
       8.3104397612028625e-05, 
       7.6488209326951379e-05, 
       1.0071069698531535e-04, 
       7.5617757945707219e-05, 
       8.6011759076686030e-05, 
       8.5080255660471057e-05, 
       8.6132541170162320e-05, 
       7.6868064724253568e-05, 
       7.5244233822924252e-05, 
       7.7094027202383660e-05, 
       1.0225341492496869e-04, 
       8.6320793989075605e-05, 
       8.0724066167492284e-05, 
       8.4507434240344199e-05, 
       8.2527218444737835e-05, 
       8.0840544414476911e-05, 
       8.1187980492611412e-05, 
       8.5078633348245649e-05, 
       7.5530947950415230e-05, 
       7.5948853899103741e-05, 
       7.6554343975876183e-05, 
       7.5413265378803586e-05, 
       8.6683426775063202e-05, 
       1.0230753758844400e-04, 
       7.5010868427174493e-05, 
       9.9245999853681921e-05, 
       1.0214942167969541e-04, 
       7.6593160197422781e-05, 
       9.7647372960502971e-05, 
       8.0984161436259931e-05, 
       7.2221745549041433e-05, 
       8.2476434803497814e-05, 
       7.3558928363637988e-05, 
       8.7257090688041652e-05, 
       7.7647516038693694e-05, 
       9.0769981444006710e-05, 
       7.5426248920700166e-05, 
       8.3194959267818326e-05, 
       8.4218697443701613e-05, 
       1.0147087217306007e-04, 
       7.2967411298811708e-05, 
       1.0028066967817269e-04, 
       9.8339172054595243e-05, 
       1.0136219450604877e-04, 
       7.2615397088188091e-05, 
       7.3123308375082967e-05, 
       8.4086233668145997e-05, 
       8.5782063090470280e-05, 
       9.7650038960778358e-05, 
       8.3100098261311618e-05, 
       9.3057255284051765e-05, 
       9.1110967683341538e-05, 
       8.9844430398412713e-05, 
       8.4048648548912998e-05, 
       8.3842244744117227e-05, 
       7.3903876420449466e-05, 
       7.5387599049816549e-05, 
       8.3493089063256372e-05, 
       7.2818394753666285e-05, 
       7.1097341678211861e-05, 
       7.1057618640941528e-05, 
       9.4688802303242504e-05, 
       9.3384875017835654e-05, 
       7.1154455127459121e-05, 
       9.9999999999997836e-05, 
       9.9999999999997769e-05, 
       1.0000000000000276e-04, 
       9.9999999999996183e-05, 
       9.9999999999996928e-05, 
       1.0000000000000000e-04, 
       7.0838690459119080e-05, 
       7.2697623399521388e-05, 
       9.8925305019139062e-05, 
       8.2421452373052730e-05, 
       7.6033344538281594e-05, 
       9.3125582258770997e-05, 
       7.6126181075916911e-05, 
       7.6126181075916911e-05, 
       9.6393872178302532e-05, 
       7.6628797360605980e-05, 
       7.0873983990174847e-05, 
       7.6895165477934539e-05, 
       7.2103900483248457e-05, 
       7.0538337919117487e-05, 
       7.0538337919117487e-05, 
       7.9188664614064563e-05, 
       8.3773270029698226e-05, 
       9.5920792352817259e-05, 
       7.2240766374646753e-05, 
       7.1844946665426942e-05, 
       7.1701426358310755e-05, 
       7.4591761224882571e-05, 
       8.2030605153689514e-05, 
       8.8337311312279007e-05, 
       7.1718666790569841e-05, 
       7.0823538865281616e-05, 
       9.5217815484665235e-05, 
       9.8464249044307445e-05, 
       9.7467507742600494e-05, 
       9.0295184259738220e-05, 
       9.8092681798626937e-05, 
       8.3838450295963785e-05, 
       8.2232967350668557e-05, 
       9.6042340982166005e-05, 
       7.3438085108317973e-05, 
       8.1492508977708530e-05, 
       9.6702778587745325e-05, 
       6.9339477233584961e-05, 
       7.4458164224992286e-05, 
       8.0934131135357586e-05, 
       6.9080688397379434e-05, 
       7.5200061978936887e-05, 
       9.3867240955738978e-05, 
       7.3941866442909040e-05, 
       7.1128469288057064e-05, 
       9.6366182792600682e-05, 
       9.4372265409115459e-05, 
       8.3746971644011622e-05, 
       7.1366498026140070e-05, 
       9.4288735459412151e-05, 
       9.7110160108635358e-05, 
       7.0313604739178441e-05, 
       9.5596731502767970e-05, 
       9.3934205097630962e-05, 
       9.2285568587099117e-05, 
       7.4926273494395554e-05, 
       6.8490744902700751e-05, 
       8.0083233833820849e-05, 
       9.3038341842542811e-05, 
       9.6479540091464521e-05, 
       7.1206731617662458e-05, 
       8.6429112490005370e-05, 
       7.2498050070721179e-05, 
       6.9412684833113116e-05, 
       8.5023719387783869e-05, 
       9.6027030777804940e-05, 
       8.9939612579272824e-05, 
       7.3201324653390901e-05, 
       9.4917560175284942e-05, 
       7.9763874475242573e-05, 
       6.7944665615777067e-05, 
       7.1830982320605197e-05, 
       7.9656698342298618e-05, 
       9.5734497366677576e-05, 
       6.9520991574164085e-05, 
       7.3405941567082256e-05, 
       7.9999999999996469e-05, 
       6.7984291436332957e-05, 
       8.5481827748780628e-05, 
       6.8850560377642096e-05, 
       7.8372000363968527e-05, 
       6.7551066765761245e-05, 
       6.9915642286157010e-05, 
       9.2978689827703833e-05, 
       7.7373921555125538e-05, 
       7.7670199117669530e-05, 
       8.0791113938625603e-05, 
       6.9000961869339708e-05, 
       6.8787039893559380e-05, 
       6.8849139418414562e-05, 
       9.0684483329836834e-05, 
       7.2213056613350856e-05, 
       7.8009679925042664e-05, 
       8.7719744348829174e-05, 
       9.4167349109264807e-05, 
       8.4647536004137943e-05, 
       7.8407629277069211e-05, 
       7.0623313207959884e-05, 
       6.6912142476392269e-05, 
       7.7773108234484162e-05, 
       8.3775733136956583e-05, 
       6.8734267425167754e-05, 
       7.1679816490409913e-05, 
       8.5332878764648304e-05, 
       7.8422320652832752e-05, 
       7.7870524970763852e-05, 
       6.8501660446526976e-05, 
       7.0432189478614865e-05, 
       9.0740239839094537e-05, 
       6.8134276641400782e-05, 
       7.3897523893817082e-05, 
       7.2771408046467151e-05, 
       7.9566929305947912e-05, 
       6.8180357551378182e-05, 
       7.2374836344731177e-05, 
       8.6183451104102719e-05, 
       6.6694747734294955e-05, 
       8.1203830541420890e-05, 
       9.0400986196571091e-05, 
       6.7018487836548014e-05, 
       7.6826073766118362e-05, 
       6.6887324559123296e-05, 
       7.0287125741985075e-05, 
       8.9634266858742057e-05, 
       7.7434470655690573e-05, 
       7.0687491777254711e-05, 
       9.1483183225009686e-05, 
       6.5992029644971022e-05, 
       7.1828848277819221e-05, 
       6.6024144457343226e-05, 
       8.1814185979065589e-05, 
       6.6708441432496295e-05, 
       9.3048884072254430e-05, 
       6.9450395278256626e-05, 
       7.2354488575329369e-05, 
       7.5603683537169141e-05, 
       6.6139794816543932e-05, 
       7.9695381330682500e-05, 
       9.2680884495699285e-05, 
       7.0000000000001037e-05, 
       7.7112176335805444e-05, 
       7.8808265134962767e-05, 
       8.4398966581856669e-05, 
       7.0664104776124571e-05, 
       9.2403708128072811e-05, 
       9.2483814201478796e-05, 
       7.8159563609077914e-05, 
       6.9999995725857403e-05, 
       7.2386998166301154e-05, 
       7.2375372024988504e-05, 
       7.0580791753289401e-05, 
       7.8923195366876055e-05, 
       7.2290826358230312e-05, 
       6.5529325150052040e-05, 
       6.9459347650055228e-05, 
       7.2253991999628709e-05, 
       7.6417733400203452e-05, 
       7.4468734402209448e-05, 
       7.2316044276501170e-05, 
       9.0347699250556499e-05, 
       6.9781852661697272e-05, 
       8.9036430592359504e-05, 
       6.9857849579582756e-05, 
       6.5115015051355116e-05, 
       9.1879818669929596e-05, 
       6.6958263209413010e-05, 
       6.9623492354649571e-05, 
       7.4638296698939749e-05, 
       7.6472055442017550e-05, 
       8.7633269112767978e-05, 
       6.8844234115298837e-05, 
       6.9169537085692599e-05, 
       9.1307512053629612e-05, 
       9.1704636199515963e-05, 
       7.7501995247135250e-05, 
       7.2265772271875491e-05, 
       8.0702796472607662e-05, 
       7.7829506679633650e-05, 
       7.5724728776048944e-05, 
       6.7988487536570130e-05, 
       7.6224206543554077e-05, 
       7.6285332804508878e-05, 
       6.8053693319656685e-05, 
       7.5827644550184646e-05, 
       7.0890301680331277e-05, 
       7.2566643127504020e-05, 
       7.0000000000000550e-05, 
       6.6951092031751458e-05, 
       8.9921255587574768e-05, 
       7.5562011808856958e-05, 
       6.7776482568645196e-05, 
       7.5584065184259628e-05, 
       6.5007952404470877e-05, 
       7.4975626967596905e-05, 
       7.6285662344655040e-05, 
       8.5001058696662936e-05, 
       8.8872441589296907e-05, 
       8.8872441589296907e-05, 
       6.7710192644087938e-05, 
       6.4194301943277661e-05, 
       6.4028689619885615e-05, 
       6.4037243230730476e-05, 
       6.9405859838583801e-05, 
       7.4778587707066140e-05, 
       8.9449111123297633e-05, 
       8.9999999999999152e-05, 
       6.9421421684302993e-05, 
       6.8505192387133550e-05, 
       8.9999999999995682e-05, 
       9.0000000000002662e-05, 
       9.0000000000001117e-05, 
       8.9999999999998284e-05, 
       7.3022679680835465e-05, 
       8.9850699652130260e-05, 
       6.6065813717384501e-05, 
       6.5733712696743854e-05, 
       8.5650538876354761e-05, 
       8.6989953771884798e-05, 
       6.3712296798195791e-05, 
       8.9492379556623561e-05, 
       6.4086517720247352e-05, 
       6.5100963215693048e-05, 
       6.3980596038823461e-05, 
       6.6847156239555289e-05, 
       8.9654444410343858e-05, 
       6.5146546424750624e-05, 
       6.5006848448260021e-05, 
       6.3411209917457989e-05, 
       6.9539640506425878e-05, 
       6.7439104279998932e-05, 
       6.9697519601601200e-05, 
       6.6761475106429533e-05, 
       7.4872423681596504e-05, 
       6.6642789971112779e-05, 
       7.5063739271118005e-05, 
       7.5668614406573573e-05, 
       7.5668614406573573e-05, 
       6.9082363160356483e-05, 
       8.8692909018035009e-05, 
       6.3208892640598179e-05, 
       6.3208892640598179e-05, 
       7.4118530053897053e-05, 
       8.6136619189994216e-05, 
       6.3930293145074939e-05, 
       7.3194520951362090e-05, 
       7.7129223040689690e-05, 
       7.4162009056880814e-05, 
       8.4944989332001242e-05, 
       8.8785966861099685e-05, 
       6.6694712995663834e-05, 
       7.4144116441629356e-05, 
       8.7820600951576909e-05, 
       7.2579100502984554e-05, 
       8.7728282052421293e-05, 
       7.4008967460611122e-05, 
       6.8827688580660884e-05, 
       7.5520067906771049e-05, 
       8.2920752116079986e-05, 
       6.4001027740969365e-05, 
       8.1971408683785779e-05, 
       7.9489486907729966e-05, 
       6.7842017841115092e-05, 
       6.6625153454191282e-05, 
       6.6314427503739415e-05, 
       7.0601321236640679e-05, 
       6.8333220343366841e-05, 
       8.6672932995993221e-05, 
       8.6702153233717853e-05, 
       7.2295429682206887e-05, 
       6.4940287358932505e-05, 
       6.2017363851122814e-05, 
       6.6616550807102444e-05, 
       6.6425366805987916e-05, 
       8.7298972510569867e-05, 
       7.5616021934285472e-05, 
       6.5478572499739509e-05, 
       8.4431565625076025e-05, 
       7.2851939938918349e-05, 
       6.9637041488781497e-05, 
       8.5592444010360315e-05, 
       8.3219149453353451e-05, 
       8.6910082264070264e-05, 
       8.0000000000002256e-05, 
       6.8847951826343464e-05, 
       6.1947627983547143e-05, 
       6.4025907217633927e-05, 
       8.0000000000003883e-05, 
       8.0000000000003842e-05, 
       7.9999999999999424e-05, 
       8.0000000000001402e-05, 
       8.6050685551009913e-05, 
       6.6661393447996992e-05, 
       6.1295529077665185e-05, 
       8.3480488384025015e-05, 
       7.4086597278268763e-05, 
       6.1295538805971914e-05, 
       7.7353299251904794e-05, 
       6.3789988696441729e-05, 
       6.1937001615840369e-05, 
       7.7216314500714467e-05, 
       6.6053762088363646e-05, 
       6.8613577483682634e-05, 
       6.7145641820334925e-05, 
       6.2675146919482201e-05, 
       7.1845328603144167e-05, 
       8.3963539468134981e-05, 
       7.3044169581359955e-05, 
       7.3044169581359955e-05, 
       7.0891796755842722e-05, 
       6.4663322900376753e-05, 
       6.2292627610583193e-05, 
       7.6993565690140658e-05, 
       6.6820933405635228e-05, 
       7.7935118878123821e-05, 
       6.3967923127303174e-05, 
       6.0795406487596570e-05, 
       6.2589369465452651e-05, 
       7.4191208586323095e-05, 
       6.6255892380265394e-05, 
       8.5597916421167183e-05, 
       6.8370895422703118e-05, 
       6.0901676639620634e-05, 
       6.8049036301050733e-05, 
       6.1041156844097077e-05, 
       6.5903689147345793e-05, 
       6.2124740647708518e-05, 
       6.2314562387189132e-05, 
       6.0512134001003855e-05, 
       6.8486535095525514e-05, 
       7.7827088202146093e-05, 
       7.9336914608998536e-05, 
       7.2014845490578732e-05, 
       8.1606317136676747e-05, 
       7.1178561305609511e-05, 
       8.4176926519562015e-05, 
       7.0745665740781107e-05, 
       6.1030200597279737e-05, 
       7.0221943185742050e-05, 
       7.1770141042899974e-05, 
       8.2876096735188054e-05, 
       6.3106827635935021e-05, 
       8.4049649961430252e-05, 
       8.3971878811891205e-05, 
       6.9713048722176116e-05, 
       6.1063856078847468e-05, 
       6.2111156153156380e-05, 
       6.8477596855495828e-05, 
       7.4728142395030656e-05, 
       7.1443683352790126e-05, 
       6.1444309061791814e-05, 
       6.9437823142425010e-05, 
       6.9999586568302643e-05, 
       6.0288803113892016e-05, 
       7.0886250601877193e-05, 
       5.9517344801579615e-05, 
       6.9862041896514635e-05, 
       6.1326266094570249e-05, 
       6.9358131171232454e-05, 
       8.2294043253059359e-05, 
       6.2262878845233683e-05, 
       8.3087104793811743e-05, 
       5.9755455151006992e-05, 
       5.9975911008270836e-05, 
       6.7248098076303051e-05, 
       5.9939807033004162e-05, 
       7.9421379076872930e-05, 
       7.2077394366280579e-05, 
       6.9422569282022131e-05, 
       6.3049311078183258e-05, 
       6.1022505729840919e-05, 
       6.2316047964951690e-05, 
       8.3398129306047779e-05, 
       6.5588570795712721e-05, 
       6.0328606386868346e-05, 
       7.8356315139209453e-05, 
       8.1367356128142244e-05, 
       5.8789103604822755e-05, 
       6.7186149006355164e-05, 
       8.1137483331954937e-05, 
       5.8966069701977884e-05, 
       7.4130131166358967e-05, 
       6.6171300152773554e-05, 
       6.1846928361594187e-05, 
       8.2867968724286204e-05, 
       7.0041079554833440e-05, 
       7.8106701429622922e-05, 
       7.1202314635040298e-05, 
       5.9836546443424792e-05, 
       5.8576497700564385e-05, 
       5.8486837245813019e-05, 
       8.1853789853467439e-05, 
       6.6465033211453886e-05, 
       6.7594751604494358e-05, 
       6.7589207947063777e-05, 
       6.2851451430143699e-05, 
       7.4908647797968555e-05, 
       7.1841311749797058e-05, 
       7.1058161837312346e-05, 
       6.7351961272134699e-05, 
       6.9578580425358293e-05, 
       7.5811559562980379e-05, 
       6.8796222004168626e-05, 
       8.1804119139069613e-05, 
       6.0722142946019726e-05, 
       8.0359886539278736e-05, 
       5.8575460937275274e-05, 
       7.0043238730644513e-05, 
       7.7559363916957679e-05, 
       6.1117434485299306e-05, 
       6.8174505248427543e-05, 
       7.8342957218475141e-05, 
       6.1056585235949836e-05, 
       5.9013364815193407e-05, 
       7.5647576757309964e-05, 
       6.1747986820439481e-05, 
       7.9999999999996429e-05, 
       7.8760913549864620e-05, 
       8.1072729486216712e-05, 
       5.9127926522721309e-05, 
       8.1563993742181891e-05, 
       8.1448560451748795e-05, 
       5.8234019696507641e-05, 
       5.8218634662979693e-05, 
       6.9903257322839711e-05, 
       7.8593987223039778e-05, 
       8.0962871072286420e-05, 
       5.8070771386830406e-05, 
       6.3691326346237344e-05, 
       6.3683426154923165e-05, 
       8.0834510747882036e-05, 
       6.5929358015072881e-05, 
       5.7487505501763549e-05, 
       7.7990377094070031e-05, 
       6.5878549684681461e-05, 
       8.1096674012214243e-05, 
       6.0195324520524503e-05, 
       8.1121133834423359e-05, 
       8.1012738218484432e-05, 
       7.1298459410059630e-05, 
       7.0246902049072446e-05, 
       6.0914837545958159e-05, 
       7.2206027188168539e-05, 
       5.8333284843342976e-05, 
       6.7398179778162525e-05, 
       5.9656683086010640e-05, 
       5.8519342502889724e-05, 
       7.6609799758899620e-05, 
       6.7179552756107489e-05, 
       7.3742197126784134e-05, 
       5.7076756726943410e-05, 
       6.0472131434879337e-05, 
       5.7281188951486858e-05, 
       5.7513253386424693e-05, 
       6.6563551182254927e-05, 
       5.8096360746475580e-05, 
       7.6539397911164366e-05, 
       7.0901726630843105e-05, 
       7.1338707457302390e-05, 
       8.0246610328105238e-05, 
       6.0409854001594292e-05, 
       7.7308754208739908e-05, 
       5.6694894372324928e-05, 
       8.0000000000001552e-05, 
       5.6739072122418531e-05, 
       6.6802605884038702e-05, 
       6.9722779843071529e-05, 
       7.9999964553520704e-05, 
       5.7551186655517369e-05, 
       7.9068402437872180e-05, 
       6.0903545856475739e-05, 
       7.9999999999995981e-05, 
       7.9999999999999600e-05, 
       7.9999999999999708e-05, 
       7.9999999999998245e-05, 
       7.9999999999997716e-05, 
       7.9999999999995914e-05, 
       6.5771268461172988e-05, 
       7.9900951576652331e-05, 
       5.9618739190768820e-05, 
       6.4911545489035156e-05, 
       7.7017309256409406e-05, 
       6.0406897445587954e-05, 
       5.6372051538452727e-05, 
       6.4399149744066476e-05, 
       7.7310648005134351e-05, 
       7.8550756286093726e-05, 
       7.8722678202331354e-05, 
       5.8359126534978765e-05, 
       6.5772459083018813e-05, 
       6.9489860853043009e-05, 
       6.2111277647789215e-05, 
       5.8277492382694714e-05, 
       7.7885743919497391e-05, 
       6.5874151818445158e-05, 
       5.8601977383260578e-05, 
       5.6040097629855852e-05, 
       5.7007370548025930e-05, 
       7.1944465606370219e-05, 
       7.5110161170259949e-05, 
       5.8387783722172441e-05, 
       5.8804700320789873e-05, 
       6.8707265051740977e-05, 
       5.6299204895686136e-05, 
       6.9188156620092115e-05, 
       5.7401869333348134e-05, 
       5.7673069369991239e-05, 
       6.5796944772588875e-05, 
       6.3947499301463676e-05, 
       7.1580467597377731e-05, 
       5.6200564772730648e-05, 
       7.0309247430800411e-05, 
       6.6047817857402421e-05, 
       5.9462726273928843e-05, 
       5.6200268252530903e-05, 
       6.2144721329275262e-05, 
       5.8491587187299773e-05, 
       5.7197483299444588e-05, 
       5.8472361839070634e-05, 
       5.7431867446371738e-05, 
       7.8319174608492101e-05, 
       5.9554526781387140e-05, 
       5.8283305534387508e-05, 
       6.8507473709744919e-05, 
       5.5950661848999256e-05, 
       6.4500471463740738e-05, 
       7.5826741926749761e-05, 
       6.1218018140755617e-05, 
       7.5398343432449687e-05, 
       6.7242079155482249e-05, 
       5.8350509548537691e-05, 
       7.0046521270643268e-05, 
       6.6966234610041920e-05, 
       7.5153153319304142e-05, 
       6.5963198452251897e-05, 
       5.5888382849423982e-05, 
       5.7616578782686680e-05, 
       5.8047756786255423e-05, 
       7.7733992715337813e-05, 
       7.7319539440982277e-05, 
       5.9093479652273457e-05, 
       6.5879117630712947e-05, 
       6.3546154797898015e-05, 
       5.8496049945965234e-05, 
       5.8682686865950203e-05, 
       7.5133817578064494e-05, 
       5.5739563629734406e-05, 
       5.8227037358834596e-05, 
       6.6583157477973162e-05, 
       5.5423122066201949e-05, 
       7.2844555761428912e-05, 
       5.5351408239329317e-05, 
       5.6509587337812714e-05, 
       6.8016585031002057e-05, 
       5.6589329206011907e-05, 
       6.6708441432477688e-05, 
       5.7654406207677711e-05, 
       5.7654406207677711e-05, 
       7.7126306535250762e-05, 
       5.4536752546926678e-05, 
       5.4823193799954275e-05, 
       7.1403447915943303e-05, 
       5.9338104204380171e-05, 
       5.6986126373813316e-05, 
       6.1452082460627906e-05, 
       6.2450324586617445e-05, 
       6.5100923227102064e-05, 
       6.1391636171454107e-05, 
       6.9435771217651375e-05, 
       5.5963038522147751e-05, 
       6.2530567864370408e-05, 
       6.9167268456462962e-05, 
       7.5578720613505594e-05, 
       6.0521133845602216e-05, 
       6.5651183505185899e-05, 
       6.0122758624293347e-05, 
       7.5621206728768937e-05, 
       5.5671178888067848e-05, 
       5.7020332332487721e-05, 
       6.9418078355315405e-05, 
       7.4153623926901809e-05, 
       5.5670772558977532e-05, 
       7.6181881669031085e-05, 
       5.4152934220157735e-05, 
       7.6154745925831969e-05, 
       7.1994130050647689e-05, 
       6.5469739881167610e-05, 
       6.3308852028665555e-05, 
       6.3859059070151439e-05, 
       6.9999999999996267e-05, 
       7.0000000000004819e-05, 
       7.0000000000003003e-05, 
       5.7013116798884947e-05, 
       6.3714560075732259e-05, 
       7.2968238602742216e-05, 
       7.4254888482717910e-05, 
       7.0260475542549764e-05, 
       6.1301915205820201e-05, 
       5.3655710866274421e-05, 
       7.5611308701239476e-05, 
       6.4752873971490833e-05, 
       6.3815821636799638e-05, 
       6.7999993840232371e-05, 
       7.2388091326709255e-05, 
       7.4581497981184727e-05, 
       5.5097642372555272e-05, 
       6.1815259975918130e-05, 
       7.2968952904771010e-05, 
       6.2525499379224294e-05, 
       6.1675544193305282e-05, 
       7.2609882424347496e-05, 
       6.2632345928638076e-05, 
       7.3631452325070371e-05, 
       7.1920900723578259e-05, 
       6.1278462998428556e-05, 
       7.0404126608007687e-05, 
       5.3374378207900417e-05, 
       5.2953810785053414e-05, 
       6.2676821565421591e-05, 
       5.4208107973188428e-05, 
       7.4350850952534678e-05, 
       5.5903012209264134e-05, 
       7.1530779052606105e-05, 
       7.0544125382895765e-05, 
       7.3981679840751222e-05, 
       5.4637817072863970e-05, 
       5.8581128575405999e-05, 
       5.4454942426221525e-05, 
       5.3351956835571826e-05, 
       5.3569041825410595e-05, 
       5.8399683667779928e-05, 
       7.0492553212102906e-05, 
       5.9389539375220665e-05, 
       5.5756460692640313e-05, 
       6.9135663102043470e-05, 
       5.4239830530774268e-05, 
       5.5823776564372720e-05, 
       7.3052507124475637e-05, 
       5.9391032188061110e-05, 
       6.9389731175117396e-05, 
       5.8371814444202151e-05, 
       5.8361695767184039e-05, 
       5.3140106183809239e-05, 
       5.2538731658421197e-05, 
       7.3155927072294868e-05, 
       5.2548845364478694e-05, 
       5.7747045335037558e-05, 
       5.6333633705751053e-05, 
       5.8551628022872519e-05, 
       5.3919661770740322e-05, 
       5.2695829225956221e-05, 
       7.0707404064373417e-05, 
       5.4880795652292745e-05, 
       5.2994897044107277e-05, 
       7.0772914207727119e-05, 
       7.3520049375375574e-05, 
       6.2712054185075960e-05, 
       7.3358539146435272e-05, 
       7.2647133135199640e-05, 
       7.3281502635766943e-05, 
       6.4710196356528228e-05, 
       6.7554460421587098e-05, 
       5.4837516854305764e-05, 
       7.2882404801545201e-05, 
       5.3331870043678936e-05, 
       7.2377642960768355e-05, 
       5.3107525299106938e-05, 
       6.2402323313350813e-05, 
       5.3746768640790939e-05, 
       5.4852051732919532e-05, 
       7.2976956235294181e-05, 
       5.5715504773136874e-05, 
       7.2712231373442965e-05, 
       5.7821691739406147e-05, 
       5.4644058604852959e-05, 
       6.5503852183698607e-05, 
       6.2918695366846796e-05, 
       6.9028110165359383e-05, 
       7.2927196993623780e-05, 
       6.7818810337131808e-05, 
       5.8180969815989230e-05, 
       5.2277229812538866e-05, 
       5.2885731984462212e-05, 
       6.0634003001819919e-05, 
       6.0723573407856219e-05, 
       6.5792903285665392e-05, 
       5.1643206135485051e-05, 
       5.5635496467614915e-05, 
       5.2306118988525681e-05, 
       5.1489874333883744e-05, 
       5.3657723309819772e-05, 
       7.0505460494449583e-05, 
       7.2280886575799746e-05, 
       7.2684356865616753e-05, 
       5.2950100050595731e-05, 
       6.6679341668687649e-05, 
       5.3457587331172690e-05, 
       5.5496361067673006e-05, 
       5.9701741459847299e-05, 
       6.9338819658801707e-05, 
       5.1515672843550892e-05, 
       5.6940404780319242e-05, 
       6.5415175245451474e-05, 
       5.8590638433293021e-05, 
       6.9441843921330172e-05, 
       5.8030029043378005e-05, 
       5.1856533421948444e-05, 
       5.0926597531812841e-05, 
       5.5786819531077208e-05, 
       6.1301348644750942e-05, 
       6.9142646805354932e-05, 
       5.9264042032049323e-05, 
       5.4301579588420742e-05, 
       5.7225957435414691e-05, 
       5.9652253757711611e-05, 
       5.7146905961426048e-05, 
       6.4893626094914659e-05, 
       5.0803174066605268e-05, 
       5.9746029741379681e-05, 
       6.2127560580308650e-05, 
       5.3811870048602273e-05, 
       5.1189720894006389e-05, 
       6.5415150955390328e-05, 
       5.0862101514447519e-05, 
       6.0464805383182640e-05, 
       5.4244623883109573e-05, 
       5.7886304211728332e-05, 
       5.2053784143214063e-05, 
       5.0744622132762096e-05, 
       5.9439518842259396e-05, 
       7.1005939005932352e-05, 
       6.9269255178954875e-05, 
       6.6433327714632374e-05, 
       5.8893186649181449e-05, 
       5.0773205580234588e-05, 
       5.8509510152704110e-05, 
       6.3549956890922603e-05, 
       5.6576973974391862e-05, 
       5.3120923661899176e-05, 
       7.0950371238204677e-05, 
       6.5378066649362742e-05, 
       5.7303238460781716e-05, 
       5.0851196335983792e-05, 
       7.0459725896174018e-05, 
       5.8693773796014286e-05, 
       6.3695331317576109e-05, 
       5.0473907673574630e-05, 
       5.2237118897768206e-05, 
       7.0938130395567761e-05, 
       5.8237563178617421e-05, 
       5.0192432490451184e-05, 
       5.4233794547780502e-05, 
       6.0000000000000862e-05, 
       6.6617042000594276e-05, 
       6.8310130128675591e-05, 
       5.6527273877266639e-05, 
       7.0031955131841952e-05, 
       5.9705357397423808e-05, 
       6.9818012339344324e-05, 
       5.3097116520689287e-05, 
       5.8682533186491593e-05, 
       5.0369949563792117e-05, 
       5.0818625851728102e-05, 
       5.6061561357567273e-05, 
       5.8488201816057606e-05, 
       5.2904242858111470e-05, 
       6.4737146596510647e-05, 
       5.3521182847414386e-05, 
       6.9999999999997907e-05, 
       6.9999999999995237e-05, 
       6.9999999999999370e-05, 
       6.9999999999995901e-05, 
       7.0000000000003016e-05, 
       7.0000000000004182e-05, 
       6.7507811295745498e-05, 
       6.9999999999999994e-05, 
       5.1904743481132965e-05, 
       6.9911850920685790e-05, 
       6.8774555393876250e-05, 
       5.0853941962980113e-05, 
       5.5974749885326160e-05, 
       5.2196258461919686e-05, 
       6.5888879986082785e-05, 
       6.9647035199035261e-05, 
       5.9318592053048362e-05, 
       4.9941989778631981e-05, 
       5.5070065180113226e-05, 
       6.9667928372789176e-05, 
       6.0294293146174899e-05, 
       6.9614617916649192e-05, 
       6.0249719517538580e-05, 
       6.9634717973311532e-05, 
       5.3406994566046547e-05, 
       6.0075649808176604e-05, 
       5.7856110701220593e-05, 
       5.4194517124298945e-05, 
       5.9939807033001485e-05, 
       5.7788985598938492e-05, 
       5.7654209187841779e-05, 
       5.3245664476029690e-05, 
       6.8294990629212064e-05, 
       5.1819830695422259e-05, 
       5.1793064135587944e-05, 
       5.5692871739847024e-05, 
       5.5638458220648930e-05, 
       5.5061501746463422e-05, 
       5.9418269234674275e-05, 
       5.9418269234674275e-05, 
       6.9102842187250158e-05, 
       6.9202106627658342e-05, 
       6.8059553882645187e-05, 
       6.7118875069030451e-05, 
       4.9429592954613257e-05, 
       4.9438691666407176e-05, 
       6.4574574496992495e-05, 
       4.9075947065865876e-05, 
       6.6372181595925032e-05, 
       5.4211980298756077e-05, 
       5.6282488491592738e-05, 
       4.9733566373602802e-05, 
       4.9281556165954021e-05, 
       5.2386447909112655e-05, 
       5.8339756928813070e-05, 
       4.9082128531763488e-05, 
       5.4547897393177602e-05, 
       6.7154585161798426e-05, 
       5.1180978976347045e-05, 
       5.0059468117206730e-05, 
       5.7035029892671697e-05, 
       5.6224138340778580e-05, 
       6.5694941003993014e-05, 
       6.8423786791217876e-05, 
       6.7401074282846720e-05, 
       6.1057578073188745e-05, 
       5.6929735774859507e-05, 
       5.3553166895624647e-05, 
       5.7135445583040911e-05, 
       4.9173252232845279e-05, 
       6.8124712001435878e-05, 
       6.1982253217940286e-05, 
       4.9986025733025579e-05, 
       5.6304678819758760e-05, 
       5.6077226078964582e-05, 
       6.1721699720896365e-05, 
       5.1949826657155597e-05, 
       6.6271437407158890e-05, 
       5.8727230508765282e-05, 
       6.8058160232902756e-05, 
       6.7467970174608425e-05, 
       5.5839685787154337e-05, 
       5.1527046071899540e-05, 
       6.7950392033506567e-05, 
       5.1439873685258851e-05, 
       6.6429942082994295e-05, 
       4.8441815683187511e-05, 
       5.7788836592711078e-05, 
       6.5547984280585531e-05, 
       6.4969454356405900e-05, 
       5.5486439714172744e-05, 
       5.6830797803405895e-05, 
       5.8422499988691722e-05, 
       6.7888603362576452e-05, 
       4.8755500361795478e-05, 
       5.7676820514249529e-05, 
       4.8890909345170752e-05, 
       5.1986824794424328e-05, 
       5.0516744969084998e-05, 
       6.5180862467621351e-05, 
       6.6768447452326203e-05, 
       5.6428375356962725e-05, 
       4.8411150360027856e-05, 
       4.8829215634594100e-05, 
       5.9012176974199628e-05, 
       4.8190863403155121e-05, 
       4.8532461585484179e-05, 
       5.7990513324048184e-05, 
       6.7210421847840339e-05, 
       4.9065488049903942e-05, 
       4.9131588130123420e-05, 
       5.6295777949839289e-05, 
       5.6112155712866247e-05, 
       5.5421443814026223e-05, 
       5.7495826607724812e-05, 
       6.6976872419193898e-05, 
       4.7798188035033005e-05, 
       5.3543030848884162e-05, 
       4.8068768999675456e-05, 
       5.1168250063573888e-05, 
       5.2562102491930520e-05, 
       5.8360583435439789e-05, 
       4.9823361044004298e-05, 
       5.1239583190533600e-05, 
       5.9607253389648816e-05, 
       4.8856689878798525e-05, 
       6.6840243607990866e-05, 
       4.9155784851950384e-05, 
       5.5230300624771765e-05, 
       4.9817461477581836e-05, 
       5.0980848232306575e-05, 
       6.3992345259856762e-05, 
       4.7403145917500183e-05, 
       5.2624898170209190e-05, 
       6.6342692928062432e-05, 
       5.4720833273606180e-05, 
       5.8860423443622196e-05, 
       4.8628639255658508e-05, 
       6.3960307193163421e-05, 
       5.5560927253584967e-05, 
       6.6387015201549505e-05, 
       6.3003597891985046e-05, 
       5.7322980292546794e-05, 
       5.4893961997201745e-05, 
       5.5210723935268091e-05, 
       6.0847182112127938e-05, 
       6.1191053389739221e-05, 
       5.4800471870091196e-05, 
       4.8899093720158621e-05, 
       5.4415035528300198e-05, 
       4.7176606649280219e-05, 
       5.0149155019542366e-05, 
       5.7461491671426802e-05, 
       4.7595154813169758e-05, 
       6.5930833081932947e-05, 
       4.6680851352347405e-05, 
       6.2836045975070653e-05, 
       6.2836045975070653e-05, 
       5.7117989316320992e-05, 
       5.8084648482367822e-05, 
       4.6595005113552475e-05, 
       6.4805751040175906e-05, 
       5.1628197703498566e-05, 
       5.4954100343644434e-05, 
       6.4509697287140613e-05, 
       5.2277501459207974e-05, 
       4.7363900372736404e-05, 
       6.5500318321853570e-05, 
       6.2986434129051402e-05, 
       5.4905022482281828e-05, 
       4.8783702524048636e-05, 
       6.5336635269507286e-05, 
       5.7482534836658004e-05, 
       5.5456155417069248e-05, 
       6.2110551122138797e-05, 
       5.5367691047924030e-05, 
       6.0394125750386610e-05, 
       5.6393531088359224e-05, 
       4.8277076033647455e-05, 
       6.1963447890564172e-05, 
       5.9999999999996180e-05, 
       6.0000000000004040e-05, 
       5.9999999999999866e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       6.0000000000004996e-05, 
       4.8919005728962156e-05, 
       4.8020646058948294e-05, 
       4.6075908501621943e-05, 
       5.5636134083119825e-05, 
       4.9255790464045804e-05, 
       5.3665314933835224e-05, 
       5.0587831557503732e-05, 
       5.0370830344259781e-05, 
       5.6885949512116897e-05, 
       5.2672638865111929e-05, 
       5.0044337706742301e-05, 
       5.3704790463499113e-05, 
       6.3986119981049801e-05, 
       6.4648429997868714e-05, 
       5.2232735224126247e-05, 
       4.6059236816611589e-05, 
       5.2525298630325122e-05, 
       4.8532066647182072e-05, 
       5.2624022791551378e-05, 
       5.8875311616523331e-05, 
       5.8644737820742914e-05, 
       6.4368374490639133e-05, 
       4.8200494403544152e-05, 
       5.0720410533912382e-05, 
       5.1656215335812869e-05, 
       5.6554325964458382e-05, 
       6.1931035142444632e-05, 
       4.7389002589603285e-05, 
       5.2015638070832189e-05, 
       5.3186552918579861e-05, 
       5.5126275743094082e-05, 
       5.6521964227868431e-05, 
       5.0162084659005960e-05, 
       4.9258018642439319e-05, 
       6.1199966769753114e-05, 
       6.2350132761167278e-05, 
       5.3331492340606802e-05, 
       4.8172436837364844e-05, 
       4.5468663550895849e-05, 
       5.1057102444195728e-05, 
       5.3811322428276546e-05, 
       5.5370020169628080e-05, 
       5.0823814561563740e-05, 
       4.6626525578432450e-05, 
       5.5199044945956439e-05, 
       4.5496098109817633e-05, 
       5.1633336126447875e-05, 
       6.2639501948292821e-05, 
       4.6815621176611117e-05, 
       4.5319500355535343e-05, 
       5.0729915650022426e-05, 
       5.4896094376021840e-05, 
       5.0867937525453185e-05, 
       4.9934020721311711e-05, 
       6.0349450426038604e-05, 
       5.9439584788231909e-05, 
       4.6462275554701517e-05, 
       6.3937238831808708e-05, 
       5.2120247240544914e-05, 
       6.3866136969769688e-05, 
       4.6182803705393720e-05, 
       6.0119680112954966e-05, 
       5.3153732340189933e-05, 
       6.2472058227759644e-05, 
       5.0973623751609602e-05, 
       5.2907412242889158e-05, 
       5.5242338632736270e-05, 
       4.7603016568030278e-05, 
       4.8581084845547012e-05, 
       6.1665049247742589e-05, 
       6.3461028263481515e-05, 
       6.3460363199534445e-05, 
       5.7161925993514645e-05, 
       5.4852632436185530e-05, 
       6.2316493850177404e-05, 
       4.6761215110134757e-05, 
       6.1892017862394814e-05, 
       5.5015505677616779e-05, 
       4.6239570968043748e-05, 
       6.0463965265838370e-05, 
       6.2835813834466727e-05, 
       4.5073610026391611e-05, 
       5.1697356441486444e-05, 
       6.3032402181799788e-05, 
       5.6187922975949822e-05, 
       5.3793905913886575e-05, 
       5.0874171686920067e-05, 
       5.3997177171280784e-05, 
       4.6985874249167178e-05, 
       4.8763907431357866e-05, 
       5.2343888002231213e-05, 
       5.4943735044086137e-05, 
       4.6230089486701635e-05, 
       4.7021951831774940e-05, 
       6.1949003404044624e-05, 
       6.0960842892658908e-05, 
       6.0489937684067887e-05, 
       6.1449521589411731e-05, 
       5.5709207923670473e-05, 
       4.5982682345389509e-05, 
       6.2589872005379315e-05, 
       6.2582958198720948e-05, 
       5.4200546958542493e-05, 
       5.4415096140113719e-05, 
       5.1181059138378605e-05, 
       4.8506650141339845e-05, 
       5.3886848493621039e-05, 
       5.0208019389186380e-05, 
       4.5736473110669535e-05, 
       4.5084412361438301e-05, 
       5.1395366811943550e-05, 
       5.0649091619831184e-05, 
       5.9937319091337540e-05, 
       5.2314328297484447e-05, 
       4.8433251536815609e-05, 
       4.6638335520830991e-05, 
       5.0006524759705309e-05, 
       6.2340328410111174e-05, 
       6.1077412136091285e-05, 
       4.6204623839822289e-05, 
       5.9977976175652429e-05, 
       5.4022648021469051e-05, 
       4.4156965075618014e-05, 
       4.6246180472292401e-05, 
       5.1757954173142231e-05, 
       4.8244733671149489e-05, 
       5.1620125404129000e-05, 
       5.1725960944374309e-05, 
       5.1180115907803281e-05, 
       4.8762279412665143e-05, 
       5.1641541890932526e-05, 
       5.8943117718298882e-05, 
       4.4000839163406419e-05, 
       4.6232680008718511e-05, 
       4.9623448059495636e-05, 
       4.3845596249394962e-05, 
       5.2493474454416795e-05, 
       4.3931435054792515e-05, 
       4.5499899545180348e-05, 
       4.8744009384053409e-05, 
       4.4768635481360915e-05, 
       5.1010745314152139e-05, 
       4.3891094761038179e-05, 
       6.0650641962860851e-05, 
       4.3620840744908696e-05, 
       5.4747940780180304e-05, 
       5.1128901078611637e-05, 
       5.6163479836108622e-05, 
       5.0454579442896844e-05, 
       5.0959804372513256e-05, 
       4.5808848202632811e-05, 
       4.8316786522032077e-05, 
       5.3947820151145772e-05, 
       4.9906266041516460e-05, 
       4.9903041618072622e-05, 
       5.5553630235743111e-05, 
       4.4709685269662970e-05, 
       4.7411841094515627e-05, 
       4.9646011883341925e-05, 
       4.3741602841152682e-05, 
       4.5088673379960427e-05, 
       5.2515552633123617e-05, 
       4.5740945497191634e-05, 
       4.5667250007802094e-05, 
       4.3492508872549069e-05, 
       4.3766582934701616e-05, 
       4.5127952633870439e-05, 
       4.3312680122956944e-05, 
       6.0744279709598826e-05, 
       5.9810354663154359e-05, 
       5.3078133849708067e-05, 
       4.7617362384289807e-05, 
       5.3204472630424375e-05, 
       5.1655198458111211e-05, 
       4.9486079152464172e-05, 
       5.2417380696785202e-05, 
       4.4260246306852084e-05, 
       5.3529752387855166e-05, 
       5.7775698063934512e-05, 
       5.3550349496016404e-05, 
       4.3390146961430143e-05, 
       5.7578787324899365e-05, 
       6.0379542999331506e-05, 
       6.0141308807401612e-05, 
       5.1263794073019253e-05, 
       5.4321434884439146e-05, 
       6.0606394476908003e-05, 
       4.3736925885953352e-05, 
       4.5918444007100729e-05, 
       5.8037474782733717e-05, 
       4.5029398657819523e-05, 
       5.5450947744660237e-05, 
       5.9999999999995550e-05, 
       4.8183469016338309e-05, 
       4.2864905074471364e-05, 
       5.7421694509391550e-05, 
       4.6999246590521435e-05, 
       4.3314673945254786e-05, 
       4.3826639630942426e-05, 
       4.5099790812813945e-05, 
       4.4608464834851515e-05, 
       4.9912183775591017e-05, 
       5.4774717392822652e-05, 
       4.6112863593300849e-05, 
       5.8165673534554548e-05, 
       4.4528566541860135e-05, 
       4.5238136185692460e-05, 
       4.3843161888724208e-05, 
       5.0030142072056702e-05, 
       4.3088401082979049e-05, 
       4.2647939732095430e-05, 
       4.3413155913303453e-05, 
       5.2427380340666423e-05, 
       5.9999999999999310e-05, 
       5.9999999999996546e-05, 
       6.0000000000003742e-05, 
       6.0000000000002556e-05, 
       5.9999999999999527e-05, 
       6.0000000000000422e-05, 
       5.9999999999997874e-05, 
       5.9999999999997874e-05, 
       6.0000000000000849e-05, 
       5.9999999999998551e-05, 
       6.0000000000004779e-05, 
       6.0000000000000002e-05, 
       4.9989525692923144e-05, 
       4.9477025590364511e-05, 
       4.2914044060904132e-05, 
       4.7654349608539087e-05, 
       4.9885184164457889e-05, 
       4.4170852386683296e-05, 
       4.6180437730363466e-05, 
       5.7784283654206190e-05, 
       4.2321376650598003e-05, 
       5.4742201976634431e-05, 
       4.4559611633559966e-05, 
       5.9479462526411957e-05, 
       4.5428048789755465e-05, 
       4.9340762952638152e-05, 
       4.9117557474239952e-05, 
       4.3228013945769331e-05, 
       5.8138216111392133e-05, 
       4.6936812289581863e-05, 
       4.3059115108043826e-05, 
       5.9555156319000111e-05, 
       5.9396006082526388e-05, 
       5.0374372456593259e-05, 
       4.6403384076973468e-05, 
       5.4639016987122717e-05, 
       4.9254955487342607e-05, 
       4.9021088218738301e-05, 
       5.6814828119280989e-05, 
       4.2431641445317442e-05, 
       4.5757139751888294e-05, 
       5.9343876890261234e-05, 
       4.2028676301942872e-05, 
       4.3723196396817918e-05, 
       4.8736183382274307e-05, 
       4.9737012615764975e-05, 
       4.4502557313300532e-05, 
       4.2955638760384177e-05, 
       4.9320193414117061e-05, 
       5.8709160707257089e-05, 
       4.2369607953665634e-05, 
       4.1912103048150174e-05, 
       4.9274741048537857e-05, 
       4.9079260267646955e-05, 
       4.7131408596580615e-05, 
       4.8529906815559230e-05, 
       5.0032365735497000e-05, 
       4.4216806516761800e-05, 
       5.0473824958167644e-05, 
       5.4990747884506360e-05, 
       4.2995825998694557e-05, 
       4.8956344907266142e-05, 
       4.1643488907769427e-05, 
       5.0000000000004027e-05, 
       4.9102565603596434e-05, 
       4.2056231593610980e-05, 
       5.6551370220277830e-05, 
       4.1473351769081861e-05, 
       4.2660196591858984e-05, 
       4.8466105959378858e-05, 
       4.3664491349537101e-05, 
       4.7019190130416136e-05, 
       4.2020703594997762e-05, 
       4.2197521725418601e-05, 
       4.4050240165943549e-05, 
       4.5173091710291490e-05, 
       5.7387871179809036e-05, 
       5.8242958955245922e-05, 
       4.1387373846525672e-05, 
       4.5658650850494859e-05, 
       5.8380061506390137e-05, 
       4.9400488295855784e-05, 
       5.5626860385196315e-05, 
       4.7591523025750123e-05, 
       4.5605417099392996e-05, 
       4.5827593137828634e-05, 
       4.5597046260994381e-05, 
       4.5938540269852517e-05, 
       4.1239694363495530e-05, 
       5.7390012273906227e-05, 
       4.2623090897413479e-05, 
       5.6879615881749153e-05, 
       4.9190272230195365e-05, 
       4.9999999999999792e-05, 
       4.6034348380751324e-05, 
       4.1306516225154326e-05, 
       4.1306516225154326e-05, 
       4.2419183164565412e-05, 
       5.7927127405167101e-05, 
       4.1355279233160817e-05, 
       5.7986910355144224e-05, 
       4.8580462608542503e-05, 
       4.7612948134855449e-05, 
       5.6949260027469635e-05, 
       4.6481445271418442e-05, 
       5.7838111169947167e-05, 
       5.1333192981771837e-05, 
       4.6656464285117813e-05, 
       4.1056045656931586e-05, 
       4.3080856617576129e-05, 
       4.2539910213325332e-05, 
       4.4316438860835246e-05, 
       4.2269531053290492e-05, 
       4.4289595602791237e-05, 
       5.7598221827485390e-05, 
       4.3027901919575666e-05, 
       4.2344777273661307e-05, 
       5.7691783165335331e-05, 
       4.1893098273142994e-05, 
       4.1378991254230658e-05, 
       4.6680851352351701e-05, 
       5.3344797584888999e-05, 
       4.6561259790826225e-05, 
       4.6521148979778008e-05, 
       5.7431995387824092e-05, 
       4.0743268883048198e-05, 
       5.1702184280872233e-05, 
       4.2589252683174096e-05, 
       5.5276504598079247e-05, 
       5.5138742277526770e-05, 
       5.6835586211109020e-05, 
       4.6937118773572440e-05, 
       4.6421109921223438e-05, 
       5.7237172261648808e-05, 
       4.1784757532711830e-05, 
       5.4828682688127821e-05, 
       4.2726541807463150e-05, 
       5.3871417177149840e-05, 
       4.4191057740170973e-05, 
       4.0654368970348233e-05, 
       4.8420669905112727e-05, 
       4.7074138740620341e-05, 
       4.5726308369385158e-05, 
       4.1698844107093833e-05, 
       4.0508321643157128e-05, 
       4.8589159881024081e-05, 
       4.7475244498203241e-05, 
       4.0422237972570740e-05, 
       4.0630103915450757e-05, 
       5.6853074152897302e-05, 
       4.8653027493779121e-05, 
       4.1181911521023013e-05, 
       4.1807599914309833e-05, 
       4.7131760382240903e-05, 
       4.1487245110593908e-05, 
       4.1392752854613889e-05, 
       5.3736873559307385e-05, 
       4.0169583561656173e-05, 
       4.1610480757795027e-05, 
       3.9986283148078992e-05, 
       4.8476446646087345e-05, 
       4.2181077020240292e-05, 
       5.5220532245943299e-05, 
       5.6325547402070667e-05, 
       4.8282448144517436e-05, 
       5.4097829665084227e-05, 
       5.4402092406479162e-05, 
       5.6198027089088832e-05, 
       4.8738662300079464e-05, 
       4.1190115314639731e-05, 
       4.0206985399988753e-05, 
       3.9822533051480654e-05, 
       5.3533889858325180e-05, 
       4.8810598171253817e-05, 
       4.6367848408419797e-05, 
       4.8473308285840820e-05, 
       5.3482105176086828e-05, 
       4.6746429035703899e-05, 
       3.9928326222434816e-05, 
       4.0446738261275096e-05, 
       3.9999999999999251e-05, 
       3.9825616636154748e-05, 
       4.3094739618661663e-05, 
       4.5869008981259229e-05, 
       4.3476026373925649e-05, 
       3.9999999999999149e-05, 
       4.6679723539361222e-05, 
       4.1793679441100703e-05, 
       4.0045345043157613e-05, 
       4.0620579343639523e-05, 
       4.5533050701295619e-05, 
       5.4865128331398789e-05, 
       5.4876726358108719e-05, 
       5.4876726358108719e-05, 
       5.4933362672606455e-05, 
       4.8403188023473836e-05, 
       4.0503154181217877e-05, 
       4.0747971692862886e-05, 
       3.9999999999999204e-05, 
       4.3053301163211905e-05, 
       5.2872649988757145e-05, 
       4.9413415375108390e-05, 
       5.3349872581457082e-05, 
       4.6609530599120183e-05, 
       4.1634647590973212e-05, 
       5.5566698936445724e-05, 
       4.7317523129035093e-05, 
       5.5415502577735319e-05, 
       4.0000000000004198e-05, 
       4.6872977944749447e-05, 
       4.6872977944749447e-05, 
       4.7551620231641707e-05, 
       5.2716549386309342e-05, 
       5.5420129129652183e-05, 
       4.7984379484198297e-05, 
       5.5231082239972103e-05, 
       4.1227820768257724e-05, 
       5.2766396882121334e-05, 
       5.5277396251773343e-05, 
       4.7694693288193728e-05, 
       4.8947934039797070e-05, 
       4.5971446371953517e-05, 
       4.8788722750148041e-05, 
       4.5840655752655039e-05, 
       5.5156359921801346e-05, 
       4.0237339154447129e-05, 
       4.8029213486533319e-05, 
       4.5762989514534655e-05, 
       4.1998963819514386e-05, 
       5.3553287300580787e-05, 
       5.3885912021023566e-05, 
       4.7268150785897822e-05, 
       4.9999999999996926e-05, 
       4.7222674191666060e-05, 
       4.5516656777274089e-05, 
       5.4927576134101640e-05, 
       4.2017845779234498e-05, 
       3.9217118803972750e-05, 
       4.0758562369121633e-05, 
       4.0758562369121633e-05, 
       3.9829763113082735e-05, 
       4.2016701592091930e-05, 
       4.5797852167756901e-05, 
       3.9777342969000458e-05, 
       3.9889276389257778e-05, 
       3.9883810021708140e-05, 
       4.6382659451069880e-05, 
       5.2936145234419369e-05, 
       5.3854612408325158e-05, 
       5.3681544034002672e-05, 
       3.8757084183542985e-05, 
       3.8757084183542985e-05, 
       5.3129947971102481e-05, 
       4.8210346693396730e-05, 
       3.8682632566986381e-05, 
       4.1907428291848185e-05, 
       3.8912556520324435e-05, 
       4.9405678723330799e-05, 
       4.9424947177337444e-05, 
       3.9555536339518008e-05, 
       4.1880495300854461e-05, 
       5.4460827879384862e-05, 
       5.2783399198899163e-05, 
       5.3344018500666361e-05, 
       4.5301328581767161e-05, 
       4.0624931956256581e-05, 
       3.8689854482162302e-05, 
       4.5252740925766792e-05, 
       4.6415029432686081e-05, 
       5.4340489586031172e-05, 
       4.2564967480046287e-05, 
       4.5130349353345287e-05, 
       5.0000000000004299e-05, 
       5.0000000000002733e-05, 
       5.0000000000002794e-05, 
       4.9999999999996086e-05, 
       4.9999999999996479e-05, 
       3.8367026359383460e-05, 
       4.1812135500619011e-05, 
       3.8399553086101836e-05, 
       5.2504843594545961e-05, 
       4.6566756486422909e-05, 
       4.5025160670482117e-05, 
       3.9332741397176787e-05, 
       4.4833940213453963e-05, 
       3.9909229550201967e-05, 
       3.9999999999998485e-05, 
       4.0673051949744794e-05, 
       4.1192008757856732e-05, 
       3.9244089817564084e-05, 
       4.6463944138561079e-05, 
       4.0609923515246993e-05, 
       3.8205542051530889e-05, 
       4.1758445988407313e-05, 
       4.2582790376141000e-05, 
       3.8690818687620413e-05, 
       5.1054341164156614e-05, 
       4.0184014857642677e-05, 
       4.3071388782582017e-05, 
       3.8556120392793670e-05, 
       4.1186902337199184e-05, 
       3.8125340577283374e-05, 
       4.4898060934481598e-05, 
       3.8607437743775037e-05, 
       3.8579122539990380e-05, 
       4.4102334302382840e-05, 
       4.4092800625148512e-05, 
       3.8165375988579617e-05, 
       3.8221594349099795e-05, 
       3.9217597300470891e-05, 
       4.2026673184816736e-05, 
       4.5798996007120656e-05, 
       4.7271580203303840e-05, 
       4.2169464403581484e-05, 
       4.4583013873295372e-05, 
       5.2507353929527501e-05, 
       4.1897194718015050e-05, 
       4.4101190080671204e-05, 
       4.7935538673683188e-05, 
       3.8719379231439572e-05, 
       5.1580267387044076e-05, 
       4.3848316668304945e-05, 
       4.1198369059005749e-05, 
       4.6700887184167944e-05, 
       4.4393980442063270e-05, 
       5.1775690705076777e-05, 
       3.7671434409166956e-05, 
       4.5465024101112944e-05, 
       4.1578124282161413e-05, 
       3.9012974169455260e-05, 
       4.1314061750878910e-05, 
       4.7428509199940162e-05, 
       4.9730818209690336e-05, 
       4.0198860611875665e-05, 
       3.7756745272042277e-05, 
       4.4089243669457669e-05, 
       4.5462921802828011e-05, 
       3.9499044748964214e-05, 
       4.2824123629750374e-05, 
       4.5737826905557407e-05, 
       4.8529081539453502e-05, 
       4.9588076603087392e-05, 
       5.1981660451376669e-05, 
       5.0871420957154051e-05, 
       5.1144548571629179e-05, 
       3.9490149743067477e-05, 
       3.9031905915225573e-05, 
       4.5831677211210600e-05, 
       4.1444965641118307e-05, 
       5.2756724081492818e-05, 
       4.3401840076277341e-05, 
       4.1262132449701108e-05, 
       4.7361048466102391e-05, 
       4.8148587905372334e-05, 
       4.5249921745478738e-05, 
       3.7379471877535428e-05, 
       3.8171256090277061e-05, 
       4.5420754043660291e-05, 
       3.9999999999999956e-05, 
       4.0913829527531320e-05, 
       3.9225465815228226e-05, 
       3.8862509784538389e-05, 
       4.7760596118884652e-05, 
       5.2589650454665371e-05, 
       4.3802123322492794e-05, 
       5.0455662279347686e-05, 
       5.1016698778043452e-05, 
       5.0973232291483467e-05, 
       3.7254871645390122e-05, 
       3.9636665763166410e-05, 
       4.4500780153024367e-05, 
       4.0617568523987939e-05, 
       3.7252201921401205e-05, 
       3.7149256373465534e-05, 
       4.5323288458288803e-05, 
       3.8855997946358206e-05, 
       5.2135608610241341e-05, 
       5.0586978896262118e-05, 
       3.9332831215316651e-05, 
       3.8919190428930292e-05, 
       4.5709179834956401e-05, 
       3.8527016130150091e-05, 
       5.1810373328764550e-05, 
       3.8352374870641786e-05, 
       5.0559596881809618e-05, 
       4.2827255512977396e-05, 
       3.9507410929693281e-05, 
       5.0222619983382302e-05, 
       3.8776413196605887e-05, 
       3.8771762343032301e-05, 
       3.7213843409197711e-05, 
       4.3116272454786127e-05, 
       3.7551982842666390e-05, 
       4.3144032977884826e-05, 
       3.9039453027332062e-05, 
       3.6882089171293840e-05, 
       5.0137421064268391e-05, 
       4.3071053584278631e-05, 
       4.7494456055885984e-05, 
       4.7869477351075516e-05, 
       4.1039403876113906e-05, 
       4.7737029508300656e-05, 
       3.9025088162159966e-05, 
       3.6585694275826116e-05, 
       5.0819542019124864e-05, 
       4.0000000000002443e-05, 
       4.3561658624952643e-05, 
       4.4621306052016770e-05, 
       5.1539034031555042e-05, 
       4.4267521473492807e-05, 
       4.9059004569314347e-05, 
       4.2481388019654305e-05, 
       3.6781291270641367e-05, 
       4.9466585659021100e-05, 
       3.6639145793717715e-05, 
       3.6481849489570408e-05, 
       3.6915195434491318e-05, 
       4.9897097790425700e-05, 
       5.1068642799919907e-05, 
       3.9213912996997664e-05, 
       3.8302494563673123e-05, 
       3.6908670353626897e-05, 
       3.7747095218466348e-05, 
       3.6507257051977693e-05, 
       5.0623393126359428e-05, 
       3.6294870987292694e-05, 
       5.0407804196197774e-05, 
       4.8626464090568084e-05, 
       4.5903871677841165e-05, 
       3.6305220859840208e-05, 
       3.9156004466396121e-05, 
       5.1079647754534503e-05, 
       4.1126183676543897e-05, 
       4.4256456102755829e-05, 
       5.1072772878516972e-05, 
       4.1088050080020714e-05, 
       3.8098824929514733e-05, 
       3.6292802712483733e-05, 
       3.7881831586116264e-05, 
       3.8011160779265816e-05, 
       3.6225151056930629e-05, 
       4.6732009248430430e-05, 
       4.2689216259987502e-05, 
       3.6176113956295823e-05, 
       4.8325881984923283e-05, 
       3.9703483036919981e-05, 
       3.9672283088894010e-05, 
       4.9150003984808674e-05, 
       4.8673415704524168e-05, 
       3.7337748916521062e-05, 
       3.6606600540253771e-05, 
       3.6606600540253771e-05, 
       4.4169456576088538e-05, 
       3.8698521120121164e-05, 
       4.1152193881715595e-05, 
       4.0000000000000789e-05, 
       3.7942729673331959e-05, 
       4.2442945199250477e-05, 
       3.5921531897822069e-05, 
       4.1782907660022691e-05, 
       4.8123251743939720e-05, 
       3.6415494337422460e-05, 
       4.0838629665093989e-05, 
       4.8546991242800412e-05, 
       5.0686307697402933e-05, 
       4.7748508360558765e-05, 
       4.0400442778724485e-05, 
       3.7836951681355312e-05, 
       5.0655816488747451e-05, 
       4.2163479568077072e-05, 
       4.0857128528670636e-05, 
       5.0550680007651935e-05, 
       4.5659202385955215e-05, 
       3.5948592860459729e-05, 
       3.9366194457792237e-05, 
       4.9699248776905651e-05, 
       4.8876354551017700e-05, 
       4.9362622579371397e-05, 
       4.3667019327052607e-05, 
       4.1246289596223742e-05, 
       3.9013566052538380e-05, 
       3.7110415917498557e-05, 
       4.0620062413580923e-05, 
       4.2667778713392756e-05, 
       3.9932816694817713e-05, 
       5.0406696139146041e-05, 
       4.3396413401811097e-05, 
       4.9702100826339363e-05, 
       5.0353951837400616e-05, 
       4.1825744852813942e-05, 
       3.8175300453073919e-05, 
       3.6637391558180805e-05, 
       4.5041749499095774e-05, 
       3.7791254807466405e-05, 
       4.6947102251870528e-05, 
       4.9271502059371876e-05, 
       4.9999800902817170e-05, 
       4.0386545168513681e-05, 
       4.8164222572487311e-05, 
       4.1327683193812462e-05, 
       4.9999999999999792e-05, 
       5.0000000000001107e-05, 
       4.2980472829982204e-05, 
       4.0141286768363031e-05, 
       4.4223338451110325e-05, 
       4.0171164336368798e-05, 
       3.7419109782125890e-05, 
       4.9999999999996384e-05, 
       4.9999999999996086e-05, 
       5.0000000000001364e-05, 
       4.9999999996266254e-05, 
       5.0000000000001392e-05, 
       4.9999999999999216e-05, 
       4.9999999999997848e-05, 
       4.9999999999997848e-05, 
       5.0000000000001378e-05, 
       4.9999999999999819e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       4.3693215212707962e-05, 
       4.7174481025077144e-05, 
       4.9871287121128939e-05, 
       3.5421382611125771e-05, 
       4.7753676166647604e-05, 
       3.8008409109077445e-05, 
       3.8008409109077445e-05, 
       4.1566081444236976e-05, 
       4.7533196668052104e-05, 
       3.6735639398153767e-05, 
       4.9742904497568775e-05, 
       4.9522709122576209e-05, 
       4.3197275492044433e-05, 
       4.2937241449752606e-05, 
       3.9187780023383066e-05, 
       4.7939155819036232e-05, 
       3.7683266551203620e-05, 
       3.6727043111546528e-05, 
       3.5695488712657605e-05, 
       4.9256753497276079e-05, 
       4.1560731014285226e-05, 
       4.9216370954988458e-05, 
       3.5071356445811760e-05, 
       3.6082568143536867e-05, 
       3.8529478570692213e-05, 
       3.7632341137863449e-05, 
       4.4407288932550977e-05, 
       3.6700633488085201e-05, 
       3.7197643616710230e-05, 
       3.9811070062031266e-05, 
       3.5440652903296097e-05, 
       4.9144327677729245e-05, 
       4.5368754043618605e-05, 
       3.7070075310274946e-05, 
       3.7879033009118545e-05, 
       3.7824005790794645e-05, 
       4.9250996958964661e-05, 
       4.0947043244973540e-05, 
       4.8978983987269091e-05, 
       4.7486787626063863e-05, 
       3.5109510528548566e-05, 
       3.6010377256048052e-05, 
       3.6869525089517571e-05, 
       4.1110914622485778e-05, 
       4.2994690261453100e-05, 
       4.9133388421174224e-05, 
       4.1160183034695845e-05, 
       4.0888575439857405e-05, 
       3.9848762004838509e-05, 
       4.0960967167305764e-05, 
       3.5377295576300438e-05, 
       3.6845818446800123e-05, 
       3.4892133030956602e-05, 
       4.1997326890760992e-05, 
       3.6385140237877915e-05, 
       3.8957918846048882e-05, 
       4.8790913105544509e-05, 
       3.8871921550046947e-05, 
       4.4001622505033822e-05, 
       3.9999999999995077e-05, 
       3.8116922979120940e-05, 
       3.6443967291686463e-05, 
       4.1269404017097563e-05, 
       4.2427313383739517e-05, 
       4.2455284234732741e-05, 
       3.5380644425587888e-05, 
       4.7188308956427515e-05, 
       4.0416564189908883e-05, 
       4.0809087851262582e-05, 
       4.8695273996632675e-05, 
       3.6315767622023942e-05, 
       4.8700468027484211e-05, 
       3.5101411679641292e-05, 
       3.5101411679641292e-05, 
       4.7410702519995419e-05, 
       4.0533970471310822e-05, 
       4.1685295592839673e-05, 
       3.7301410440252810e-05, 
       4.8587599942772118e-05, 
       3.7416580886410366e-05, 
       4.3072197197748938e-05, 
       4.8362630978951883e-05, 
       3.4566380536288469e-05, 
       3.8193249630209771e-05, 
       4.0312656651805728e-05, 
       3.5040657082742915e-05, 
       4.8265519258935261e-05, 
       4.1540410754525179e-05, 
       3.6365833631586140e-05, 
       3.5085918624153234e-05, 
       3.6799077068152171e-05, 
       4.7585585037454157e-05, 
       3.5857311796147378e-05, 
       4.3480494615414690e-05, 
       4.6203620805737069e-05, 
       3.9756700645838978e-05, 
       4.8003699574459899e-05, 
       3.5274007499668463e-05, 
       4.0234289386043077e-05, 
       3.8205301869610130e-05, 
       3.4907102358461323e-05, 
       3.9837751300035644e-05, 
       3.4171605336092669e-05, 
       3.9814218009852564e-05, 
       3.4906469719833722e-05, 
       4.6042218920033771e-05, 
       3.5116448860824046e-05, 
       3.5744734427003745e-05, 
       4.7701491599618533e-05, 
       4.0993447561416658e-05, 
       3.5116821165118841e-05, 
       3.4531050335229208e-05, 
       3.3966540657473428e-05, 
       3.4393570465377052e-05, 
       4.2294675673379252e-05, 
       3.5770372791129209e-05, 
       3.5948235345206717e-05, 
       3.6286428679262367e-05, 
       3.3814985871288216e-05, 
       4.7521252368772711e-05, 
       3.5882091139243545e-05, 
       4.1607851222958348e-05, 
       3.8009054455950987e-05, 
       3.6413843303132666e-05, 
       3.9125496684288854e-05, 
       3.9127168544911873e-05, 
       4.0505927545139951e-05, 
       3.9398768231552840e-05, 
       4.7621772757621480e-05, 
       4.2959130747837117e-05, 
       4.3320627284348344e-05, 
       3.4464527597372001e-05, 
       4.7582001065388140e-05, 
       3.3786464518053388e-05, 
       4.0701688482625825e-05, 
       3.5900547284314782e-05, 
       3.8553029520085075e-05, 
       4.1924666731585510e-05, 
       3.5394604767313878e-05, 
       4.2239530017303450e-05, 
       4.7337776421580997e-05, 
       3.3659420155075219e-05, 
       3.4287600037315041e-05, 
       3.5814826611991624e-05, 
       3.8185957442163977e-05, 
       3.3538421441383937e-05, 
       4.7004368864065993e-05, 
       3.5231692289262343e-05, 
       4.1076415345679411e-05, 
       3.8929462739837082e-05, 
       4.6862722004600786e-05, 
       3.4708757394615761e-05, 
       4.5491008197321513e-05, 
       3.3915429669994730e-05, 
       4.1088511913046882e-05, 
       3.5935351549874749e-05, 
       3.4006614022889939e-05, 
       4.6845926751187133e-05, 
       4.0373014922872028e-05, 
       4.7107097092065055e-05, 
       4.0402449368802867e-05, 
       4.7122458628594792e-05, 
       3.3325520663685238e-05, 
       3.9067872514407116e-05, 
       3.5674633458111762e-05, 
       3.5674506601213085e-05, 
       4.0894415854947020e-05, 
       3.3296919416264139e-05, 
       3.3271932497910629e-05, 
       3.8961969120147022e-05, 
       3.8027666632312266e-05, 
       3.8027666632312266e-05, 
       3.4682735970563644e-05, 
       4.6883791369232384e-05, 
       3.8428290786578761e-05, 
       3.4561980525756756e-05, 
       3.6921376344854777e-05, 
       3.4526906225512987e-05, 
       4.4797950417819652e-05, 
       3.4480464500246164e-05, 
       4.5407418911527978e-05, 
       3.9009178455334323e-05, 
       4.2575556082544586e-05, 
       4.4594004074830920e-05, 
       4.4594004074830920e-05, 
       3.6364591491455336e-05, 
       4.4385878132989232e-05, 
       4.2513035633243153e-05, 
       4.6537570189777452e-05, 
       3.5788247046931940e-05, 
       4.6596777773986708e-05, 
       3.3068164718251933e-05, 
       3.5026932969098637e-05, 
       4.5546739590187569e-05, 
       3.6952804491064363e-05, 
       3.4618017509774466e-05, 
       4.5075548402572612e-05, 
       4.0447710321298316e-05, 
       3.4923386735517238e-05, 
       4.0419775110989452e-05, 
       4.6293746012459068e-05, 
       3.2745041720002113e-05, 
       4.4591998575776437e-05, 
       4.6252413834934152e-05, 
       4.6206101594182737e-05, 
       3.8460097446942457e-05, 
       3.8817991984067315e-05, 
       3.3832317569061025e-05, 
       4.5035227505016242e-05, 
       4.6142816491464215e-05, 
       3.4537755841240126e-05, 
       4.0521043525931923e-05, 
       4.1448946781680447e-05, 
       4.6126033669354521e-05, 
       3.3260537641285438e-05, 
       4.5313186282086244e-05, 
       4.5301371894513224e-05, 
       3.2571481755159606e-05, 
       3.9063553684363544e-05, 
       3.2885145049368764e-05, 
       3.8119031368579614e-05, 
       3.8269338524865958e-05, 
       3.8396549443803920e-05, 
       4.5857080253412536e-05, 
       4.1977324710849492e-05, 
       3.8179638836043623e-05, 
       4.0081425918717041e-05, 
       3.9946127923399494e-05, 
       4.5751216887632890e-05, 
       3.2634167652354340e-05, 
       3.8046536131002415e-05, 
       3.7827084449215953e-05, 
       3.4296112163756613e-05, 
       3.4174248376746770e-05, 
       3.4174248376746770e-05, 
       3.5965142450094721e-05, 
       3.3741121062653508e-05, 
       3.9236828710639907e-05, 
       3.9608938417377338e-05, 
       3.7764623957631554e-05, 
       3.3320197340346388e-05, 
       3.7338700379328136e-05, 
       4.3669425886363513e-05, 
       3.4271137196442654e-05, 
       3.5433376771082274e-05, 
       3.3486118516582762e-05, 
       4.5250261952513739e-05, 
       3.3913462748601325e-05, 
       3.4730916239774143e-05, 
       3.3471588561025422e-05, 
       3.9267183192155905e-05, 
       3.9267183192155905e-05, 
       4.5371186199634334e-05, 
       3.2125024205432697e-05, 
       4.4069876909639000e-05, 
       3.4196786711891340e-05, 
       3.9977846844898209e-05, 
       3.6710572203415786e-05, 
       3.4010369103921501e-05, 
       3.3724409266210451e-05, 
       3.2760481035611278e-05, 
       3.2904537761383779e-05, 
       4.4568479276266993e-05, 
       4.5019472868030665e-05, 
       3.9961036222125687e-05, 
       4.3870483207295735e-05, 
       3.3903109737938297e-05, 
       4.2523477283951611e-05, 
       3.3223967255526518e-05, 
       4.5150316826228718e-05, 
       3.2625516598689567e-05, 
       3.2033188099407540e-05, 
       3.6277123400178068e-05, 
       3.8338621015750025e-05, 
       4.1178170825202329e-05, 
       4.5040085633317287e-05, 
       4.4728082521064298e-05, 
       3.6757929909937804e-05, 
       3.2091222618036264e-05, 
       3.3538116947814391e-05, 
       4.2309881828846819e-05, 
       3.3046106934695401e-05, 
       3.7370279376754190e-05, 
       3.2621174685303466e-05, 
       3.3062661983855973e-05, 
       4.2257570242165295e-05, 
       4.4836123008111322e-05, 
       3.2630549073688078e-05, 
       4.4800177385296906e-05, 
       3.2300772030072913e-05, 
       4.1095680902665975e-05, 
       3.6019523482944345e-05, 
       3.7223478283263789e-05, 
       3.2920345203836887e-05, 
       3.5561345757898780e-05, 
       3.9941119147191532e-05, 
       3.4857476393429068e-05, 
       3.4857476393429068e-05, 
       4.0778564768519577e-05, 
       3.2630383931740110e-05, 
       3.1730450843158971e-05, 
       3.6002109092395767e-05, 
       3.2224597398680615e-05, 
       3.4760711536439007e-05, 
       4.0859185193395717e-05, 
       3.7665952232727266e-05, 
       3.2987602803948624e-05, 
       3.1631742267254777e-05, 
       3.3887177837585555e-05, 
       4.0040122562525175e-05, 
       3.7083791842945269e-05, 
       3.4243169648869223e-05, 
       3.6102549527256003e-05, 
       4.2664384334338440e-05, 
       4.4272147651364855e-05, 
       4.3419086008538208e-05, 
       3.1665743515908261e-05, 
       4.4251575407203080e-05, 
       3.7111207883091069e-05, 
       3.8243511389102586e-05, 
       4.0370481932988642e-05, 
       3.9660732848977612e-05, 
       3.1713718158598758e-05, 
       3.2438012017225746e-05, 
       3.7409172994194450e-05, 
       3.5951174141606017e-05, 
       3.5592287270683499e-05, 
       3.3502927751480388e-05, 
       3.1464921463741575e-05, 
       3.2109239776612635e-05, 
       3.7317946598489292e-05, 
       4.0843149007119850e-05, 
       3.2753763807951162e-05, 
       4.1860624426415585e-05, 
       3.2970674790670760e-05, 
       3.3782491171559164e-05, 
       4.3821868344843722e-05, 
       3.1340627089470671e-05, 
       3.2735446925465831e-05, 
       3.6556506152729484e-05, 
       3.6687479054251329e-05, 
       3.1204131079621381e-05, 
       3.6441274476749582e-05, 
       4.1004039750008077e-05, 
       4.2648071026097820e-05, 
       3.7759303884191027e-05, 
       3.3273649181840837e-05, 
       3.5005580082325342e-05, 
       3.6450346560257059e-05, 
       4.1125712757855676e-05, 
       3.4950674393153032e-05, 
       4.2405073701867510e-05, 
       3.4339471043475212e-05, 
       3.6434036345276246e-05, 
       3.0990544093728622e-05, 
       3.6365064433554915e-05, 
       4.2930508638612651e-05, 
       4.2080265251830981e-05, 
       3.7130266685097364e-05, 
       3.8232203652429172e-05, 
       3.6359961900597668e-05, 
       3.2669192246649469e-05, 
       3.6405797690291121e-05, 
       3.3636907942984401e-05, 
       3.5547161978024754e-05, 
       3.0893294137756412e-05, 
       3.1972434799545717e-05, 
       3.5430134173731002e-05, 
       3.5429698783730162e-05, 
       3.3446935639601215e-05, 
       3.1808951487061366e-05, 
       3.2587868982492576e-05, 
       4.1127232708330871e-05, 
       3.9765272436467066e-05, 
       3.4721808162169342e-05, 
       3.0858957211447490e-05, 
       3.3021418654063261e-05, 
       3.2483493963702462e-05, 
       3.5544181339750911e-05, 
       3.5333932672663143e-05, 
       4.3267520458496746e-05, 
       3.5329228201965449e-05, 
       3.5810996273227576e-05, 
       3.6234461652868764e-05, 
       3.4966700344344709e-05, 
       4.0000000000003601e-05, 
       3.9999999999996425e-05, 
       4.2642231394564299e-05, 
       4.0000000000000525e-05, 
       3.9999999999999996e-05, 
       4.0000000000000850e-05, 
       4.0000000000000850e-05, 
       3.0687080051642160e-05, 
       4.0175820489051220e-05, 
       4.2967949281064946e-05, 
       3.5791639705738227e-05, 
       3.2695046713603666e-05, 
       4.1284915194251193e-05, 
       4.3227325676961756e-05, 
       3.6351134044658116e-05, 
       3.6351134044658116e-05, 
       4.3049980272711664e-05, 
       3.8412507270652266e-05, 
       3.2478494203517075e-05, 
       3.3360246438204735e-05, 
       3.6606860578581990e-05, 
       3.1344493459954840e-05, 
       3.6478814396940000e-05, 
       3.9976617987118969e-05, 
       3.5228487213962482e-05, 
       3.0676753497353083e-05, 
       3.3844862148111803e-05, 
       3.1918693308232495e-05, 
       3.2108403942735955e-05, 
       3.2108147324704287e-05, 
       3.5718903546128392e-05, 
       3.0567255613096953e-05, 
       3.8787873519139278e-05, 
       3.2861028505453873e-05, 
       4.2864904444903718e-05, 
       3.0865703495637819e-05, 
       3.4524179741337411e-05, 
       3.7216515401013755e-05, 
       3.1734552038443310e-05, 
       4.1102116342529652e-05, 
       3.5460940924610744e-05, 
       3.1183862419299843e-05, 
       3.1924401516496573e-05, 
       3.5392930422675153e-05, 
       3.2228120626777810e-05, 
       3.9146063286264332e-05, 
       3.5071184487368467e-05, 
       3.1842807324020201e-05, 
       3.1970168139514024e-05, 
       3.7937099126946580e-05, 
       3.0915129868124844e-05, 
       3.0508548935440445e-05, 
       3.0622265053961770e-05, 
       3.0648898063168937e-05, 
       3.9380484993190091e-05, 
       3.1826736983741053e-05, 
       4.0556897947875557e-05, 
       3.6870969331310807e-05, 
       3.4986726002250227e-05, 
       3.2769072482660232e-05, 
       3.1440201620966136e-05, 
       3.8907903181064482e-05, 
       4.2438179901222286e-05, 
       4.2023661248174083e-05, 
       3.0312423698696510e-05, 
       4.0514857214319071e-05, 
       3.7269076161601675e-05, 
       3.7269076161601675e-05, 
       3.5191790287807582e-05, 
       3.0048526852174058e-05, 
       4.2410583326262858e-05, 
       3.8847237360009878e-05, 
       4.2389194621609895e-05, 
       4.1212515784459371e-05, 
       3.1579744445800019e-05, 
       4.0307395661197584e-05, 
       3.3133149361985029e-05, 
       3.2047974973105546e-05, 
       3.4599182453513417e-05, 
       3.0335200404371127e-05, 
       3.5816400473673277e-05, 
       3.8018900764741985e-05, 
       3.6255635280558503e-05, 
       3.6517050232858369e-05, 
       2.9898232491456338e-05, 
       2.9999999999997544e-05, 
       2.9865522238895069e-05, 
       3.0386678297487825e-05, 
       3.0000000000001115e-05, 
       2.9930018971477712e-05, 
       3.5179186529604157e-05, 
       3.1013397047679805e-05, 
       4.2089693138072779e-05, 
       4.1579684287786180e-05, 
       3.1777282309974928e-05, 
       3.9824470495324094e-05, 
       3.1047783850745064e-05, 
       3.0304760938680064e-05, 
       3.9674493234127261e-05, 
       3.3497123017394477e-05, 
       3.5918861116744748e-05, 
       3.2156097743371124e-05, 
       3.5134289476827633e-05, 
       2.9999999999998537e-05, 
       4.1930957968432621e-05, 
       3.9653004118706503e-05, 
       3.0000000000000729e-05, 
       3.3695538621211971e-05, 
       2.9640018998138414e-05, 
       3.9127744124431358e-05, 
       3.6731674903732453e-05, 
       4.1569901886460826e-05, 
       3.4414606058479796e-05, 
       2.9612270451253607e-05, 
       2.9999999999996538e-05, 
       3.0344684133494233e-05, 
       3.1832123733942461e-05, 
       3.5627866755230715e-05, 
       3.2673960057977809e-05, 
       3.0645824701591572e-05, 
       3.4894116176437179e-05, 
       2.9634844391844729e-05, 
       3.1200166430293327e-05, 
       2.9673516774456471e-05, 
       4.1607779347426780e-05, 
       3.1819138156569431e-05, 
       3.0055082223753782e-05, 
       4.1660744075167262e-05, 
       4.1466580221324531e-05, 
       3.0872755395731534e-05, 
       2.9999999999998662e-05, 
       3.0393275905751711e-05, 
       3.4624016429586776e-05, 
       2.9485903121034184e-05, 
       4.0964802946960687e-05, 
       3.5708171496109527e-05, 
       2.9865832181733699e-05, 
       3.4321890721831801e-05, 
       4.1174396362775808e-05, 
       4.1239757453149332e-05, 
       3.6298303961602183e-05, 
       3.4134436748018775e-05, 
       3.2974361243417835e-05, 
       3.2360342214527998e-05, 
       3.5764132571464947e-05, 
       3.0065796860569196e-05, 
       2.9877525228162061e-05, 
       3.9176939969549332e-05, 
       3.4249939884026670e-05, 
       3.5176953962531646e-05, 
       3.0049063731339457e-05, 
       4.0594427008230429e-05, 
       3.5368975007625415e-05, 
       3.9793598342921186e-05, 
       3.9477412791113474e-05, 
       3.1864739928386620e-05, 
       3.0478014376098028e-05, 
       3.8002640290901333e-05, 
       3.0423376473216073e-05, 
       2.9999999999995217e-05, 
       2.9134911748206467e-05, 
       3.9948413522440165e-05, 
       3.0000000000002071e-05, 
       3.1343479359611563e-05, 
       3.2754659955283474e-05, 
       3.8877251773229903e-05, 
       3.4132464045913957e-05, 
       4.0736144732704707e-05, 
       3.4958329812063158e-05, 
       3.8727742895267361e-05, 
       3.4891950350432412e-05, 
       4.0339802010774952e-05, 
       3.3220558770762330e-05, 
       4.0988250698645330e-05, 
       3.7215634748122155e-05, 
       3.4834560375819646e-05, 
       3.2981440111114627e-05, 
       3.3764422593409163e-05, 
       3.4753764118036549e-05, 
       2.9245315115871251e-05, 
       3.5607991088550644e-05, 
       3.0551320352083212e-05, 
       3.1261272408491547e-05, 
       3.1261272408491547e-05, 
       3.4347123758541313e-05, 
       3.7017335901600962e-05, 
       3.6874969061329085e-05, 
       3.4631403876541824e-05, 
       2.9999999999999438e-05, 
       2.9427458040281012e-05, 
       3.1811184794633091e-05, 
       3.4991318932723151e-05, 
       3.4083085694072307e-05, 
       2.8900056859666726e-05, 
       3.3917562241378232e-05, 
       2.9805675789377176e-05, 
       3.4543374114494115e-05, 
       3.2027650421014231e-05, 
       3.1959111208989747e-05, 
       3.0058654954851915e-05, 
       3.3681230551520629e-05, 
       2.9710860854944819e-05, 
       3.1951380143894823e-05, 
       3.0404849856203801e-05, 
       3.1810645167395909e-05, 
       2.9672724338572232e-05, 
       3.5110268221259054e-05, 
       4.0717993876324803e-05, 
       4.0286150479599335e-05, 
       4.0305514578017195e-05, 
       3.3339429561963698e-05, 
       3.9143019557160810e-05, 
       3.4486627900978304e-05, 
       3.9254735696295185e-05, 
       3.1399255449256761e-05, 
       4.0663280769063733e-05, 
       3.4855922592502555e-05, 
       3.9596039957574974e-05, 
       3.1754534929542795e-05, 
       3.0744456739125412e-05, 
       3.3729669735472886e-05, 
       3.3913347332532783e-05, 
       2.8901607985854970e-05, 
       2.9017705558142954e-05, 
       3.1432518819307625e-05, 
       3.0200779238637072e-05, 
       2.8679823958508767e-05, 
       3.7089513371622241e-05, 
       2.9491248245211476e-05, 
       3.3283956616272910e-05, 
       3.1536567005246314e-05, 
       3.8619172992455671e-05, 
       3.8303105744797076e-05, 
       3.3043207623952497e-05, 
       3.1484633099524213e-05, 
       3.2506231446373348e-05, 
       3.0453721823432587e-05, 
       3.3461008278413813e-05, 
       3.3755057643275865e-05, 
       3.8625125148200007e-05, 
       3.2232221050196497e-05, 
       2.8983048499829336e-05, 
       3.9999999999995653e-05, 
       3.1213214158177114e-05, 
       3.0560856076039788e-05, 
       2.9992363883985074e-05, 
       3.5282293979060346e-05, 
       3.9871326315803349e-05, 
       3.3415178693867250e-05, 
       3.3540650107118455e-05, 
       2.9914262856188159e-05, 
       3.2482885329664131e-05, 
       3.2892428695817698e-05, 
       2.8358925785976223e-05, 
       2.8752095878955557e-05, 
       3.8997869515035493e-05, 
       2.9203676273401392e-05, 
       2.8441581847269676e-05, 
       3.8910616054939123e-05, 
       3.7629191124480454e-05, 
       3.9867401013212453e-05, 
       3.9999999369874880e-05, 
       3.9999999999997787e-05, 
       4.0000000000004726e-05, 
       3.9999999999995307e-05, 
       3.9999999999997855e-05, 
       3.9999999999999807e-05, 
       3.9999999999996385e-05, 
       3.9999999999996385e-05, 
       3.9999999999998316e-05, 
       3.9999999999998763e-05, 
       3.9999999999997022e-05, 
       4.0000000000002158e-05, 
       4.0000000000001148e-05, 
       3.9999999999995246e-05, 
       4.0000000000000566e-05, 
       4.0000000000004855e-05, 
       3.9999999999999407e-05, 
       3.9999999999997367e-05, 
       3.9999999999997367e-05, 
       3.9999999999999346e-05, 
       4.0000000000001935e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.9998072730920922e-05, 
       3.4101966653091486e-05, 
       3.0371495990061642e-05, 
       3.3691510332241079e-05, 
       3.3146176564241083e-05, 
       2.8352624797045927e-05, 
       3.9370097906425992e-05, 
       3.0914698791476434e-05, 
       2.9281548988547377e-05, 
       2.8383964019516530e-05, 
       3.1829118794917943e-05, 
       2.9056778368371660e-05, 
       2.8575019696887445e-05, 
       3.9707425423179707e-05, 
       3.3314260650146486e-05, 
       3.3245589839847759e-05, 
       2.8856044178950096e-05, 
       2.9999999999999659e-05, 
       2.9673660542242409e-05, 
       3.0852338257734745e-05, 
       3.2821110138972828e-05, 
       3.9063030334859538e-05, 
       2.9921555345213084e-05, 
       3.6980425701451305e-05, 
       2.8913479216370070e-05, 
       3.9521574689845276e-05, 
       3.0794876476941755e-05, 
       3.0367424944129759e-05, 
       3.7191960864011787e-05, 
       3.5382993151209656e-05, 
       3.0133944190861544e-05, 
       3.3862217646562098e-05, 
       2.9999999999994048e-05, 
       2.8111432292700875e-05, 
       3.5546811532154019e-05, 
       3.1762476135288943e-05, 
       3.9103375540051138e-05, 
       3.4049201938889247e-05, 
       3.6323099597367277e-05, 
       3.3820477143005490e-05, 
       2.9473525216751520e-05, 
       2.8077341188740352e-05, 
       2.8641597318945315e-05, 
       3.3052167482487881e-05, 
       3.0006977809097550e-05, 
       2.8734055098689035e-05, 
       3.2121965885823627e-05, 
       3.2121965885823627e-05, 
       3.8351822669135409e-05, 
       3.6664552622955772e-05, 
       2.9488722842464473e-05, 
       3.7617913802574932e-05, 
       3.1158144985993347e-05, 
       3.2500998209758420e-05, 
       3.9072027489543663e-05, 
       3.9072027489543663e-05, 
       2.7735598066203194e-05, 
       2.7764695223452325e-05, 
       3.9097892441677912e-05, 
       3.2976553477897266e-05, 
       3.1512281040062003e-05, 
       3.2509069531462060e-05, 
       3.2746205464973778e-05, 
       2.9680476630247505e-05, 
       3.0848085455829987e-05, 
       3.2583812867033344e-05, 
       2.8437285254930122e-05, 
       3.0008925984002347e-05, 
       3.8042884052841897e-05, 
       3.8379089311409152e-05, 
       3.3558481689605506e-05, 
       2.8418965718931595e-05, 
       2.9494167602064154e-05, 
       2.7637433230441698e-05, 
       2.8307255428469105e-05, 
       3.6623500956541377e-05, 
       3.8723976764437028e-05, 
       3.6652409712305939e-05, 
       2.8337045843303976e-05, 
       3.0927080102684384e-05, 
       3.7670865208058935e-05, 
       3.0152878696192717e-05, 
       2.7524714884471664e-05, 
       3.3688080934318146e-05, 
       3.8814990204682862e-05, 
       3.8835684022816053e-05, 
       2.8261746632243478e-05, 
       3.5797980262834686e-05, 
       3.1178034009362328e-05, 
       3.7906882212586071e-05, 
       2.8153042407781401e-05, 
       2.7828242059350935e-05, 
       3.7079373000424196e-05, 
       3.0000000000004930e-05, 
       3.0263305462685590e-05, 
       2.7427051258041797e-05, 
       3.0000000000005052e-05, 
       3.2376122010147133e-05, 
       3.2118621750710555e-05, 
       3.4552560355167389e-05, 
       3.6393073870424638e-05, 
       3.4255623018692846e-05, 
       2.8779571151293554e-05, 
       3.6980006812946069e-05, 
       2.8084763012237611e-05, 
       2.7273015773530585e-05, 
       2.7273015773530585e-05, 
       2.7593266588727539e-05, 
       3.2750681140646947e-05, 
       3.4800784269216361e-05, 
       2.7197247536869282e-05, 
       3.4760654429085798e-05, 
       3.2985405708083391e-05, 
       3.4115498745264903e-05, 
       2.8710575911387293e-05, 
       3.1717718045268014e-05, 
       2.7286759674813567e-05, 
       3.7812380667003381e-05, 
       3.7812380667003381e-05, 
       2.8592893001158859e-05, 
       3.0323834104484881e-05, 
       2.8080863309496687e-05, 
       2.7944511872129473e-05, 
       3.7583345916136524e-05, 
       2.7158777851167449e-05, 
       3.1839687959205633e-05, 
       2.7094021974703431e-05, 
       2.7171020118079076e-05, 
       3.5664461173770676e-05, 
       3.2722960716693627e-05, 
       3.5309694226877276e-05, 
       3.6967609784195869e-05, 
       2.9623026139865901e-05, 
       3.7877722468814330e-05, 
       3.1440415079202689e-05, 
       3.7883896786987826e-05, 
       3.8157882535370888e-05, 
       2.8056278765496161e-05, 
       3.3986073179846497e-05, 
       2.9907224511221717e-05, 
       2.9166341473750211e-05, 
       2.8065591424862517e-05, 
       3.1818964029316071e-05, 
       3.7765770455637448e-05, 
       2.8484579934894903e-05, 
       3.0777297346699949e-05, 
       2.9248152045879696e-05, 
       3.6752540599563022e-05, 
       3.1444359295932022e-05, 
       2.7487311212258861e-05, 
       3.2158066034617186e-05, 
       2.8006832604812145e-05, 
       3.0869558468247232e-05, 
       3.6376927317581486e-05, 
       2.8515190475710322e-05, 
       3.1012280610962080e-05, 
       2.8186097908298101e-05, 
       3.1373067469956108e-05, 
       3.7620970727936492e-05, 
       2.9664505959975816e-05, 
       2.7895681110320562e-05, 
       2.9459653284077161e-05, 
       3.1804693620182743e-05, 
       2.8131921598845085e-05, 
       3.3415185001258451e-05, 
       2.8354502709659490e-05, 
       3.7431684776689476e-05, 
       3.0000000000002816e-05, 
       3.5569001099781376e-05, 
       2.8205497492748706e-05, 
       2.8607705000421176e-05, 
       2.7443968825585090e-05, 
       3.2427585495882185e-05, 
       2.7719805887245327e-05, 
       3.1234448541447378e-05, 
       3.0761616833129976e-05, 
       3.2715242069734450e-05, 
       3.2709670811519337e-05, 
       2.7136451288924564e-05, 
       2.7805304218350495e-05, 
       3.0594252101971418e-05, 
       3.5447034326557840e-05, 
       3.7601526470764493e-05, 
       3.5295341386316663e-05, 
       3.1806493251033354e-05, 
       3.6135582599628770e-05, 
       3.6086893871662933e-05, 
       2.7376114968961502e-05, 
       3.0009824206239838e-05, 
       3.7496766586534236e-05, 
       2.7355518903843049e-05, 
       2.7243121963924633e-05, 
       3.2256475016679078e-05, 
       2.6840541795447096e-05, 
       2.6579114334319194e-05, 
       3.7402568608890108e-05, 
       3.1135519868122257e-05, 
       3.7346398928643819e-05, 
       2.8521904190905685e-05, 
       3.2261295604610622e-05, 
       2.6478938327836762e-05, 
       3.2220820255549972e-05, 
       3.7399259503341476e-05, 
       3.7361263746379754e-05, 
       3.6127348295208255e-05, 
       3.2217602660011996e-05, 
       2.7620070347298465e-05, 
       2.8782051543496251e-05, 
       2.7215867328130824e-05, 
       2.7223585310788654e-05, 
       3.7277285874189767e-05, 
       3.4672406364845564e-05, 
       2.8907478343366754e-05, 
       3.7210134825299807e-05, 
       3.1636014922887694e-05, 
       2.8291149916605230e-05, 
       3.6973137758065399e-05, 
       2.7735177636889611e-05, 
       3.0970971843819026e-05, 
       2.9486243616418117e-05, 
       3.6719146288476597e-05, 
       2.7847546085857943e-05, 
       3.0181985060019578e-05, 
       2.6482193611925169e-05, 
       2.8815807534761744e-05, 
       3.3807949936129490e-05, 
       2.7375490978217892e-05, 
       3.5243441453665731e-05, 
       3.7073338059968103e-05, 
       3.0900032098967309e-05, 
       3.0345729496005474e-05, 
       3.1748258798729004e-05, 
       3.1311727103519367e-05, 
       3.5602270007051821e-05, 
       3.2016490711050935e-05, 
       2.9217232164379110e-05, 
       3.1929020109247796e-05, 
       2.6487994875777422e-05, 
       2.6191952943439877e-05, 
       2.6880665873802498e-05, 
       2.6508511276386538e-05, 
       2.9394690649248106e-05, 
       3.5153258633961551e-05, 
       3.1235147581718718e-05, 
       3.0912636153448121e-05, 
       3.4058065997406065e-05, 
       3.5029905548271276e-05, 
       3.2568045374985023e-05, 
       2.9385263267032378e-05, 
       3.1577812600937473e-05, 
       2.7859594944653383e-05, 
       2.6529123979190486e-05, 
       3.4027755742130251e-05, 
       3.5588880550044506e-05, 
       2.6224454407935215e-05, 
       2.7722099988389238e-05, 
       2.8636722394636658e-05, 
       3.0670721746372807e-05, 
       3.6821497979660456e-05, 
       3.2544191150131346e-05, 
       3.0680391841310685e-05, 
       2.6178223135168246e-05, 
       3.6751161736298665e-05, 
       2.6400459622820941e-05, 
       3.0961856006158206e-05, 
       2.6764834198560157e-05, 
       2.6459860954323407e-05, 
       2.9212574946141627e-05, 
       2.7423522087061264e-05, 
       3.2440399380419423e-05, 
       3.0468742644225386e-05, 
       3.4797020202410598e-05, 
       3.6544531947032417e-05, 
       2.8368146842082890e-05, 
       2.8239993402518091e-05, 
       3.5498194550767828e-05, 
       3.6476709581843703e-05, 
       2.6562021739210490e-05, 
       2.5949092582143126e-05, 
       3.2974859278554567e-05, 
       3.2830037064574109e-05, 
       2.7789722747859991e-05, 
       3.2889207538864745e-05, 
       2.9456942423899567e-05, 
       2.6926973764862875e-05, 
       3.0429893573776079e-05, 
       3.0340338333332492e-05, 
       2.7247080058153878e-05, 
       3.4612610330185798e-05, 
       2.6770615985280530e-05, 
       2.7088599783173430e-05, 
       2.6374543704044544e-05, 
       2.6112247893333990e-05, 
       3.6287859028296548e-05, 
       2.6213204618459015e-05, 
       3.3445339397098061e-05, 
       2.6087882046491557e-05, 
       3.4822849282260654e-05, 
       3.4905847569910088e-05, 
       3.3801178583429075e-05, 
       3.5639546744166639e-05, 
       2.6765807884927219e-05, 
       2.6021684201039694e-05, 
       3.1587062398479871e-05, 
       2.6786650153324187e-05, 
       3.5057036568917409e-05, 
       2.8441873606354640e-05, 
       3.0668387309916572e-05, 
       2.8330356668001596e-05, 
       2.7607652335100470e-05, 
       2.9503795557109351e-05, 
       2.5518499255795921e-05, 
       2.5619292432600003e-05, 
       3.1068335511633065e-05, 
       3.1068335511633065e-05, 
       2.6958705850508612e-05, 
       2.7263349299276504e-05, 
       2.6547596492032995e-05, 
       2.9450887070232957e-05, 
       2.5544643062216664e-05, 
       3.0118263192990052e-05, 
       2.6138459220834942e-05, 
       2.5696698120434355e-05, 
       2.5971268422425596e-05, 
       2.6540868527769923e-05, 
       3.5192854407768160e-05, 
       2.9603739126994531e-05, 
       2.9810256204293699e-05, 
       2.5942780705216061e-05, 
       2.5622626663456258e-05, 
       2.6710917055127132e-05, 
       2.9967900084011413e-05, 
       2.8385156287673372e-05, 
       3.5805144182451467e-05, 
       3.4799260043352211e-05, 
       3.1509351894112576e-05, 
       2.9803614824199960e-05, 
       3.5541725360757025e-05, 
       3.3691461277249626e-05, 
       3.3691461277249626e-05, 
       2.7598855946451330e-05, 
       3.5605025050374659e-05, 
       2.8475596494380867e-05, 
       2.9912924062507994e-05, 
       2.5293439665451053e-05, 
       3.3961856653239450e-05, 
       3.5616392303967299e-05, 
       2.7001811912287017e-05, 
       2.9601128816841725e-05, 
       2.5984147079570722e-05, 
       2.9632426328468161e-05, 
       3.0236654270346675e-05, 
       3.4572432642505528e-05, 
       2.5266027091167520e-05, 
       3.0951660963260621e-05, 
       3.4373632482355902e-05, 
       2.9999999999995661e-05, 
       2.5369526271716473e-05, 
       3.4839110124151276e-05, 
       2.6132824382948814e-05, 
       2.9063327995832415e-05, 
       2.6472615655785992e-05, 
       2.7764810178307452e-05, 
       2.6144789724538160e-05, 
       3.0890981756160902e-05, 
       2.6110355117407854e-05, 
       2.9885242587375260e-05, 
       2.7679857230577843e-05, 
       2.9425082044892741e-05, 
       3.0214741684833754e-05, 
       2.8679823958497325e-05, 
       2.8962282301601835e-05, 
       2.6225859260341377e-05, 
       2.7435545015336062e-05, 
       2.7435545015336062e-05, 
       3.3082135259121834e-05, 
       3.3491180389950727e-05, 
       2.6663753906808478e-05, 
       3.4765545177206964e-05, 
       2.5534034958505820e-05, 
       3.1424812695407142e-05, 
       2.5612389910391011e-05, 
       3.5275377998565117e-05, 
       3.3837363414203747e-05, 
       2.9309628794658645e-05, 
       2.6821631738102243e-05, 
       3.4582175764566227e-05, 
       2.6261166396776510e-05, 
       2.6261166396776510e-05, 
       2.8914715281012325e-05, 
       3.0414269646515200e-05, 
       3.3471699511818368e-05, 
       2.9230822657698585e-05, 
       2.9218638649073042e-05, 
       3.0647180318210917e-05, 
       2.7008426397817923e-05, 
       3.1160267185058773e-05, 
       2.9869749083058493e-05, 
       2.9219048505920373e-05, 
       2.4849600073339911e-05, 
       2.6037799510803392e-05, 
       2.8439865618647064e-05, 
       3.4555822189791866e-05, 
       2.6483743963575529e-05, 
       3.1302480593303306e-05, 
       2.8329082517580672e-05, 
       2.5064519024489073e-05, 
       3.4998459810803798e-05, 
       2.9452237135893890e-05, 
       2.8503272400643855e-05, 
       3.4732361333598925e-05, 
       2.6048038337545929e-05, 
       2.6020340011205926e-05, 
       2.5532442911192760e-05, 
       2.8467085593134533e-05, 
       2.7844633179166775e-05, 
       2.8916218067269055e-05, 
       2.7525261530568518e-05, 
       3.3500820873700212e-05, 
       2.5236795057485878e-05, 
       2.9712944822632725e-05, 
       3.4795297202249628e-05, 
       2.5506949516865321e-05, 
       2.9271572485443801e-05, 
       2.7711138370157813e-05, 
       3.4186103796714707e-05, 
       2.9070509852859199e-05, 
       2.5674419988596945e-05, 
       3.2652772143246170e-05, 
       3.4493917848018113e-05, 
       2.9057863155478573e-05, 
       2.8025355394401530e-05, 
       2.8800441688691351e-05, 
       2.6506855316750697e-05, 
       2.5796455768486728e-05, 
       2.4585885507754087e-05, 
       3.4565565458318385e-05, 
       3.4567075152386372e-05, 
       2.9370462255910539e-05, 
       2.8910944513129732e-05, 
       3.2009112198960969e-05, 
       3.3131324568675408e-05, 
       2.9779375113424555e-05, 
       2.4478639931581370e-05, 
       3.4489899448492962e-05, 
       3.0324988760153031e-05, 
       3.1903309572245323e-05, 
       3.1903309572245323e-05, 
       2.8477087782275310e-05, 
       2.5754293512195188e-05, 
       3.2566852533668789e-05, 
       3.4340648036014342e-05, 
       2.7749558171350663e-05, 
       3.1924019355658394e-05, 
       2.4936069811422376e-05, 
       3.0476923242705787e-05, 
       2.4686938562324296e-05, 
       2.4342071007000229e-05, 
       2.4411420511066239e-05, 
       2.4405290505860593e-05, 
       3.4217763687172977e-05, 
       2.8553692616667581e-05, 
       2.4585433594285740e-05, 
       2.9343236890926187e-05, 
       3.2887355848329521e-05, 
       2.8507144590401653e-05, 
       2.8045852090980314e-05, 
       3.0286917674087125e-05, 
       2.6246399195426636e-05, 
       2.7281415298846323e-05, 
       2.5500732971808655e-05, 
       2.8352624797046195e-05, 
       2.9873918513434426e-05, 
       2.9730170434726155e-05, 
       2.9237275841035036e-05, 
       3.4188621053293948e-05, 
       2.4188682359499585e-05, 
       2.5209713601922852e-05, 
       2.8331604465864143e-05, 
       2.4707486366339261e-05, 
       2.4355719506852440e-05, 
       2.4749902788761841e-05, 
       2.5333180611686048e-05, 
       2.4187335403740857e-05, 
       3.2258817870144468e-05, 
       2.4438787635556412e-05, 
       2.6140710148556463e-05, 
       2.4291482734505766e-05, 
       3.3879750428102944e-05, 
       3.4018595780887331e-05, 
       3.3647339395032489e-05, 
       2.9775088780512990e-05, 
       2.4829213008134859e-05, 
       2.4045298416024582e-05, 
       2.9185210302983760e-05, 
       3.3949404460007427e-05, 
       3.1509096619240060e-05, 
       3.3955540639339127e-05, 
       2.7901839643375575e-05, 
       2.7344674044050312e-05, 
       2.9684586839422806e-05, 
       2.5870741827333551e-05, 
       3.2466724563955442e-05, 
       2.7871862319775655e-05, 
       2.5342692229464575e-05, 
       3.0512170281136161e-05, 
       3.3818535922091312e-05, 
       2.7827499786511347e-05, 
       2.3961241946183188e-05, 
       3.2303034902006041e-05, 
       2.4849758004142231e-05, 
       3.3449827633712639e-05, 
       3.3808023390332035e-05, 
       2.7165717949085221e-05, 
       3.0923005167221850e-05, 
       3.3638027051661989e-05, 
       3.0739282912032341e-05, 
       3.3592194079797473e-05, 
       2.7191856632721637e-05, 
       2.9226713044868983e-05, 
       2.9226713044868983e-05, 
       2.9157068993403251e-05, 
       2.5168509482996461e-05, 
       2.9181564139240500e-05, 
       3.1630333474835738e-05, 
       2.3862864384340234e-05, 
       2.4647600897910629e-05, 
       2.8026398400045466e-05, 
       2.3833921696396811e-05, 
       2.5359355698710509e-05, 
       2.5770133112329606e-05, 
       3.0585895483759674e-05, 
       2.8964505735649643e-05, 
       2.4927464175877677e-05, 
       2.5177754065335333e-05, 
       3.1894639176358783e-05, 
       2.6804373046434031e-05, 
       2.7209816442318557e-05, 
       3.3121351326259246e-05, 
       2.7938357058959506e-05, 
       3.2828890520187029e-05, 
       3.2770233936279608e-05, 
       2.4638322341815776e-05, 
       2.7419752884977669e-05, 
       2.7551620986824348e-05, 
       2.4717408738492065e-05, 
       2.4563168897390186e-05, 
       3.3509327979780881e-05, 
       2.5326275383638952e-05, 
       2.3799116281428056e-05, 
       2.5078777557785509e-05, 
       2.7797044458813704e-05, 
       2.7093497446702675e-05, 
       3.3386824173787324e-05, 
       2.4028524605538312e-05, 
       2.5241853807550540e-05, 
       3.3075257096191461e-05, 
       2.3940762638083593e-05, 
       2.5539549691732142e-05, 
       3.0298107282268505e-05, 
       2.5643172869294652e-05, 
       2.7801655062966039e-05, 
       2.7262720866021170e-05, 
       2.9297133812325343e-05, 
       2.4419010895282114e-05, 
       2.8753570159816621e-05, 
       2.3870787891065978e-05, 
       2.3949759645091643e-05, 
       3.1922406099002337e-05, 
       2.7145417761133343e-05, 
       2.7815890328425494e-05, 
       3.2767008263177699e-05, 
       3.0114034507210691e-05, 
       2.5253422774383424e-05, 
       3.0642727278611896e-05, 
       2.5824901150253911e-05, 
       2.3410591931027246e-05, 
       2.7068867471816275e-05, 
       3.2648895709794282e-05, 
       2.4672847676930602e-05, 
       2.3521448354927441e-05, 
       3.2518711624502546e-05, 
       3.2861158502551574e-05, 
       3.1844942524062685e-05, 
       3.2881728890569991e-05, 
       3.1764374761788188e-05, 
       2.9024300087927630e-05, 
       3.1848927648058240e-05, 
       2.5282329527292050e-05, 
       2.5029778423398572e-05, 
       3.2066513827816139e-05, 
       3.1608449532221451e-05, 
       3.0628291401627091e-05, 
       2.5823532110138717e-05, 
       2.7412843254975176e-05, 
       3.1409120820080540e-05, 
       2.8644311410063357e-05, 
       2.8421479340362332e-05, 
       3.1326045631095485e-05, 
       3.2231419140420244e-05, 
       2.7296652101255519e-05, 
       2.3630111405753731e-05, 
       3.2861327047380933e-05, 
       2.9712014048090544e-05, 
       2.3925798804251419e-05, 
       3.1342173101085250e-05, 
       3.0000000000004754e-05, 
       2.9226276659628415e-05, 
       3.0130530819280588e-05, 
       3.2117939065473566e-05, 
       2.6858777358858744e-05, 
       2.8870892942467787e-05, 
       2.4518909711126289e-05, 
       2.4412836780792744e-05, 
       2.6281263626433913e-05, 
       2.3639349392733013e-05, 
       2.3583956584488247e-05, 
       2.6253044776742218e-05, 
       2.5699148914549104e-05, 
       2.5699148914549104e-05, 
       2.3491457914644075e-05, 
       2.3314645847018726e-05, 
       2.9207144529710503e-05, 
       2.3640059687264301e-05, 
       2.8666666350483922e-05, 
       2.9586239836190514e-05, 
       2.4484525123260018e-05, 
       2.7343223212082859e-05, 
       3.2456652798194124e-05, 
       2.8378110328381696e-05, 
       2.7391739070760625e-05, 
       2.4306039405363230e-05, 
       2.4056732822544182e-05, 
       2.3095281581195130e-05, 
       2.5149677418359549e-05, 
       2.5574407376998318e-05, 
       2.4378657860132152e-05, 
       3.1993758386423529e-05, 
       2.7223142958714399e-05, 
       2.4233206767204053e-05, 
       2.6989436715820582e-05, 
       3.1987076575724620e-05, 
       3.0000000000004395e-05, 
       3.0000000000003175e-05, 
       3.0000000000001532e-05, 
       2.9999999999996718e-05, 
       2.9999999999997483e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999998727e-05, 
       2.6758860529685300e-05, 
       2.4208507609863603e-05, 
       2.3356403694532675e-05, 
       2.6533035133485840e-05, 
       2.3718753643780621e-05, 
       2.3631250496919172e-05, 
       3.2440097769823163e-05, 
       2.4649858560772045e-05, 
       2.9853590315916782e-05, 
       2.4187825442138748e-05, 
       2.6640559523824854e-05, 
       2.9125236549820838e-05, 
       2.4153943479388485e-05, 
       2.4803136279500559e-05, 
       2.4130287897950964e-05, 
       2.3478285487695365e-05, 
       2.3795274514457805e-05, 
       2.4515043771484238e-05, 
       2.4200953544176126e-05, 
       2.3574672895801181e-05, 
       3.2324110614769193e-05, 
       2.7105031494033407e-05, 
       2.2953778406406320e-05, 
       2.4229775483503690e-05, 
       2.3557520715966312e-05, 
       2.6788109297348951e-05, 
       2.7332334142113215e-05, 
       2.2839127833220500e-05, 
       2.4068093204499735e-05, 
       2.7596487872861401e-05, 
       2.5598385740589486e-05, 
       2.5684292612582737e-05, 
       2.4595847579348524e-05, 
       2.4595847579348524e-05, 
       3.0956762769961139e-05, 
       2.7942570891361763e-05, 
       2.2991096796153131e-05, 
       2.8513015651367395e-05, 
       2.6688545694479522e-05, 
       2.8102415001453398e-05, 
       2.4655169578557400e-05, 
       2.9794338332953304e-05, 
       2.2956581602696994e-05, 
       2.2931886952648440e-05, 
       2.4144187950297291e-05, 
       2.4029621090260527e-05, 
       3.0703370663606498e-05, 
       2.4229483928598198e-05, 
       2.3933366256948536e-05, 
       2.3982790606745475e-05, 
       2.3969868591592027e-05, 
       3.0317843506147008e-05, 
       2.6674576628837607e-05, 
       2.3367356361450269e-05, 
       3.2031640655859457e-05, 
       3.0469737523165958e-05, 
       2.2757945858743646e-05, 
       2.4043458285882960e-05, 
       2.6542271326007287e-05, 
       2.3374861431159261e-05, 
       2.6991000384326193e-05, 
       2.8445432000485446e-05, 
       2.6989834427639376e-05, 
       2.8518207218564777e-05, 
       2.5216668416586875e-05, 
       2.5265550372095493e-05, 
       2.6569198107656936e-05, 
       2.6069881770093472e-05, 
       2.2953313332721326e-05, 
       2.6523415795344397e-05, 
       2.3344000402472098e-05, 
       2.2673881814649344e-05, 
       2.3933550003904084e-05, 
       2.3425921182439749e-05, 
       2.3792196270274422e-05, 
       2.2549797917558873e-05, 
       2.3448472113271315e-05, 
       2.9736931505981631e-05, 
       2.8723867369330672e-05, 
       2.2650778794561397e-05, 
       2.8328680923543289e-05, 
       2.6058267342644204e-05, 
       2.6592908014785288e-05, 
       3.1429655936959337e-05, 
       2.6439406135915164e-05, 
       2.5352515241921984e-05, 
       2.2494846800343745e-05, 
       2.2610655526867572e-05, 
       2.6892453776254730e-05, 
       2.2452683362133069e-05, 
       2.7301361290992842e-05, 
       2.3366845393318476e-05, 
       3.1135331500622071e-05, 
       2.4455903170381747e-05, 
       2.2526824852320108e-05, 
       2.2911820874109461e-05, 
       3.1578470513111974e-05, 
       3.0928779491943031e-05, 
       2.6518867773819540e-05, 
       3.1641879052901912e-05, 
       2.3590873027809217e-05, 
       2.3533876412302279e-05, 
       2.5803952430325030e-05, 
       2.3481949138645131e-05, 
       2.3481949138645131e-05, 
       2.8650311534162916e-05, 
       3.1001081832557546e-05, 
       2.2428789971638705e-05, 
       2.9898044865364266e-05, 
       2.9898044865364266e-05, 
       3.1548666571319805e-05, 
       2.3744016468473627e-05, 
       2.2994072585300936e-05, 
       2.3669362019641366e-05, 
       3.1521701032825531e-05, 
       3.0921012361174026e-05, 
       2.7360023105400635e-05, 
       2.6151704190524080e-05, 
       3.0751001713465921e-05, 
       3.1095888488842015e-05, 
       2.5716451000296051e-05, 
       2.9485222679581076e-05, 
       2.5159823144131633e-05, 
       2.7358770837891393e-05, 
       3.0530666011066705e-05, 
       3.0530666011066705e-05, 
       2.5449544575104073e-05, 
       3.0257544841379307e-05, 
       2.3856647544495763e-05, 
       2.7939450677757918e-05, 
       2.3671541985041571e-05, 
       2.8884827226786955e-05, 
       2.6511318383138590e-05, 
       2.4060262420451600e-05, 
       3.0641464985197976e-05, 
       2.5963420292856160e-05, 
       2.6065484576414844e-05, 
       2.3323596281785612e-05, 
       3.1040418763977577e-05, 
       2.3598198909936021e-05, 
       2.5979661052908224e-05, 
       2.2432774048719995e-05, 
       3.1180582639743129e-05, 
       3.1102052418651771e-05, 
       2.2634589229571477e-05, 
       3.0282535675771443e-05, 
       2.8454865268473373e-05, 
       3.1177362330310649e-05, 
       2.8273044383323993e-05, 
       2.9423592863181931e-05, 
       2.2960727133023318e-05, 
       2.2246767066760323e-05, 
       3.1060122990885818e-05, 
       2.3517871577331563e-05, 
       2.6840541795438060e-05, 
       2.5668902765180792e-05, 
       2.2693446515808260e-05, 
       2.4529322826566388e-05, 
       2.2047351993588615e-05, 
       2.3968912046537010e-05, 
       3.1058413944483223e-05, 
       2.5223326169483067e-05, 
       2.4665068427518309e-05, 
       2.2649673538213300e-05, 
       2.2157035469453294e-05, 
       2.6180330469735683e-05, 
       2.3644029360033887e-05, 
       2.3202026565367760e-05, 
       2.5162018786713200e-05, 
       2.3987691355586738e-05, 
       2.3131084972117483e-05, 
       3.1008175381355799e-05, 
       2.9313320843176681e-05, 
       2.8903601511474440e-05, 
       2.9639029173233814e-05, 
       2.3135850915461947e-05, 
       2.4072322275920603e-05, 
       2.2501968006603904e-05, 
       2.2024431361496799e-05, 
       2.3116081764594719e-05, 
       2.2998232343133420e-05, 
       2.2118110942939362e-05, 
       3.0876353023653049e-05, 
       2.8301764688688739e-05, 
       3.0889114776607159e-05, 
       2.5778263102697021e-05, 
       3.0856737713956672e-05, 
       2.4823812133505298e-05, 
       2.3873222720876555e-05, 
       2.1833431454969353e-05, 
       2.6552667117472001e-05, 
       3.0823793873458243e-05, 
       2.2165776438755421e-05, 
       2.3328140347630406e-05, 
       2.5622001933565781e-05, 
       2.2200358561387213e-05, 
       2.2000721234050828e-05, 
       2.2781670504722637e-05, 
       2.9779706727095740e-05, 
       2.2266811169566457e-05, 
       2.2933683581686270e-05, 
       2.6383768176218102e-05, 
       2.3490016088401157e-05, 
       2.2370681954512725e-05, 
       2.1737856923475611e-05, 
       2.2575549820837218e-05, 
       2.2977010371368420e-05, 
       2.4366059144324251e-05, 
       2.3586278733158683e-05, 
       3.0585956995254478e-05, 
       2.1659009292546130e-05, 
       2.3648264828023183e-05, 
       2.1914539979488495e-05, 
       2.5401265396047047e-05, 
       2.2555914528845136e-05, 
       3.0429755882210373e-05, 
       2.5459999545576019e-05, 
       2.7188135062864157e-05, 
       2.2267230224301959e-05, 
       2.3294591229374253e-05, 
       2.3294591229374253e-05, 
       2.6953573066187078e-05, 
       2.8290081438676998e-05, 
       2.8290081438676998e-05, 
       2.3130028017194969e-05, 
       3.0194313956997775e-05, 
       2.4735936596314983e-05, 
       2.2043917412816432e-05, 
       2.1648551552999270e-05, 
       2.2106802004440897e-05, 
       2.3540983502337627e-05, 
       2.2822233971491185e-05, 
       2.5873627814288544e-05, 
       2.5071944627958816e-05, 
       2.9402387053321753e-05, 
       2.1853040963288492e-05, 
       2.6467454135017226e-05, 
       2.3065278955422418e-05, 
       2.2238613532393600e-05, 
       2.3329180401850149e-05, 
       2.3196085076249708e-05, 
       2.2373094312897225e-05, 
       2.9866956616811273e-05, 
       2.5922591923065017e-05, 
       2.5250645009954058e-05, 
       2.2524488970521550e-05, 
       2.5265532237725771e-05, 
       2.3730332366992355e-05, 
       2.2248564829250396e-05, 
       2.5697352832721036e-05, 
       2.5897815625199073e-05, 
       3.0315277840851936e-05, 
       2.4580305047934197e-05, 
       2.4960653368182458e-05, 
       2.5155507038084925e-05, 
       2.4625825675360571e-05, 
       2.1632897588302667e-05, 
       3.0024465305102980e-05, 
       2.2880606084721630e-05, 
       3.0112102204550933e-05, 
       3.0217606228385636e-05, 
       2.6954143836136093e-05, 
       2.2332393502665777e-05, 
       2.9425975060388507e-05, 
       2.9380986474664640e-05, 
       2.2521523034482854e-05, 
       2.1331277949814846e-05, 
       3.0091286988889271e-05, 
       2.3719212332704527e-05, 
       2.9853555372591540e-05, 
       2.6162844361068285e-05, 
       2.3612129708629224e-05, 
       2.1407265443088452e-05, 
       2.5606384124734948e-05, 
       2.9221015565257169e-05, 
       2.1926441957545874e-05, 
       3.0000000000004412e-05, 
       2.9999999999999692e-05, 
       3.0000000000002569e-05, 
       3.0000000000002054e-05, 
       3.0000000000002054e-05, 
       3.0000000000000529e-05, 
       3.0000000000004487e-05, 
       2.9999999999997609e-05, 
       3.0000000000001105e-05, 
       2.9999999999999262e-05, 
       3.0000000000003806e-05, 
       2.9999999999999310e-05, 
       2.9999999999999425e-05, 
       3.0000000000002372e-05, 
       2.9999999999999940e-05, 
       2.9999999999999113e-05, 
       2.9999999999999113e-05, 
       3.0000000000000211e-05, 
       2.9999999999998937e-05, 
       2.9999999999998937e-05, 
       3.0000000000002826e-05, 
       2.9999999999997080e-05, 
       3.0000000000002759e-05, 
       3.0000000000000001e-05, 
       2.9967376100969666e-05, 
       2.1258585642290856e-05, 
       2.9878936701497045e-05, 
       2.2730705198157709e-05, 
       2.3163023385621234e-05, 
       2.4812463451754450e-05, 
       2.5353282395461595e-05, 
       2.2249489624176713e-05, 
       2.2368220992200166e-05, 
       2.9764587560147846e-05, 
       2.2244504661178118e-05, 
       2.2010075790050645e-05, 
       2.1181177183619379e-05, 
       2.2312673646753555e-05, 
       2.7988631464862571e-05, 
       2.9463742257525140e-05, 
       2.9384212561671601e-05, 
       2.1274702521020825e-05, 
       2.1102969096694835e-05, 
       2.2811155946580890e-05, 
       2.4377873012068023e-05, 
       2.4774953526761876e-05, 
       2.9086370645898128e-05, 
       2.1106764841317570e-05, 
       2.2219273201087013e-05, 
       2.2507511707896874e-05, 
       2.8580847492409141e-05, 
       2.7984905149733606e-05, 
       2.2678784325977479e-05, 
       2.1500917337873341e-05, 
       2.9149393640904396e-05, 
       2.3693510630709877e-05, 
       2.4239299196179574e-05, 
       2.5619415815442632e-05, 
       2.4606503422581895e-05, 
       2.4232812774850916e-05, 
       2.1479616746356397e-05, 
       2.4658734737019408e-05, 
       2.2377877345045009e-05, 
       2.0975302634761681e-05, 
       2.5696588232389626e-05, 
       2.1074236127339265e-05, 
       2.2515595496432635e-05, 
       2.3345035645919719e-05, 
       2.1158828201940872e-05, 
       2.0892482302087414e-05, 
       2.1388043356319155e-05, 
       2.7720231634122997e-05, 
       2.7420514974535629e-05, 
       2.0971438064451802e-05, 
       2.1282081610973466e-05, 
       2.3457273705969547e-05, 
       2.5449544575112137e-05, 
       2.1716943357788268e-05, 
       2.2363242451000932e-05, 
       2.8204496935918308e-05, 
       2.2396538869570202e-05, 
       2.4987528007939380e-05, 
       2.4838944864421288e-05, 
       2.4838944864421288e-05, 
       2.1563470081071840e-05, 
       2.1370585230125476e-05, 
       2.2482179206950944e-05, 
       2.5470090470973020e-05, 
       2.1972545968161488e-05, 
       2.0782501298533299e-05, 
       2.0963483266505083e-05, 
       2.3559251366869484e-05, 
       2.0834141204981505e-05, 
       2.1833172299397551e-05, 
       2.1655164963762016e-05, 
       2.8817619302988359e-05, 
       2.6225005235145851e-05, 
       2.1369445279637196e-05, 
       2.5148629093081621e-05, 
       2.0770465547790713e-05, 
       2.4494103931738696e-05, 
       2.4209462171757982e-05, 
       2.3773107992724942e-05, 
       2.1363995710048967e-05, 
       2.2044626518182909e-05, 
       2.3759393415435389e-05, 
       2.4805081974584825e-05, 
       2.3179116064277322e-05, 
       2.1514401895830754e-05, 
       2.7616296217120512e-05, 
       2.2709046354466190e-05, 
       2.8749879027748582e-05, 
       2.8192232780338984e-05, 
       2.0641157365171598e-05, 
       2.0791988818258568e-05, 
       2.2673633041937346e-05, 
       2.4165052128570052e-05, 
       2.4554041627304931e-05, 
       2.4554041627304931e-05, 
       2.7578189541945367e-05, 
       2.2812129845507126e-05, 
       2.5525537580048646e-05, 
       2.6098638829645853e-05, 
       2.2412501953195292e-05, 
       2.3033545935114953e-05, 
       2.0666215024317245e-05, 
       2.0693914290865395e-05, 
       2.0595726018747476e-05, 
       2.1580073915734177e-05, 
       2.4426109735967227e-05, 
       2.3391625466620212e-05, 
       2.8729696770726635e-05, 
       2.3376016344286541e-05, 
       2.1661261675198909e-05, 
       2.3587605957316008e-05, 
       2.4809251086063175e-05, 
       2.5159903276305127e-05, 
       2.0966463382454883e-05, 
       2.2090190791695682e-05, 
       2.0874798315180848e-05, 
       2.7260535518770556e-05, 
       2.1555418181337062e-05, 
       2.0478511218294781e-05, 
       2.0673722153604741e-05, 
       2.0673722153604741e-05, 
       2.5594558139612343e-05, 
       2.1167664623335956e-05, 
       2.0377182125879216e-05, 
       2.1391018867755895e-05, 
       2.2015926449780460e-05, 
       2.3066860679774381e-05, 
       2.1044399250220063e-05, 
       2.0579226170623280e-05, 
       2.3883321091715731e-05, 
       2.1903314260049617e-05, 
       2.1903314260049617e-05, 
       2.3598474375245868e-05, 
       2.0573410466495585e-05, 
       2.1734658126821529e-05, 
       2.7773991371145030e-05, 
       2.5967557331454341e-05, 
       2.8132873998052460e-05, 
       2.3651379913604591e-05, 
       2.6776563934697187e-05, 
       2.3783603814403211e-05, 
       2.0966317914944670e-05, 
       2.8545353480027904e-05, 
       2.0297020183652112e-05, 
       2.1014108239154567e-05, 
       2.0204735124811388e-05, 
       2.8352944682601519e-05, 
       2.1619502830263783e-05, 
       2.3293885456198640e-05, 
       2.0652180205250836e-05, 
       2.3063330146987459e-05, 
       2.0342947951443127e-05, 
       2.8323424751774202e-05, 
       2.1382184363406343e-05, 
       2.1586278014501960e-05, 
       2.0898790363700101e-05, 
       2.3658093225887116e-05, 
       2.1207884684600929e-05, 
       2.8441862496031210e-05, 
       2.0919243927195111e-05, 
       2.4020699456820609e-05, 
       2.4535409561536329e-05, 
       2.7075029329950524e-05, 
       2.8119847590653811e-05, 
       2.1211440014521348e-05, 
       2.5197119576424604e-05, 
       2.5797501301016550e-05, 
       2.3275728849808974e-05, 
       2.3077675354490958e-05, 
       2.4086360757039374e-05, 
       2.3128265093259274e-05, 
       2.0028676292895690e-05, 
       2.3522889548463088e-05, 
       2.4177213013043984e-05, 
       2.0492459356269730e-05, 
       2.3512154471688490e-05, 
       2.0000000000002217e-05, 
       2.0428470091726012e-05, 
       2.1308814110592108e-05, 
       2.7816142445164482e-05, 
       2.7843214917612733e-05, 
       2.3956837111922531e-05, 
       2.0558819361950228e-05, 
       2.5748900607172417e-05, 
       2.4653736170723783e-05, 
       2.1066849907602962e-05, 
       2.0654506029328560e-05, 
       2.1995466457976440e-05, 
       2.1108296035024811e-05, 
       2.2997515178881452e-05, 
       2.0683641650317797e-05, 
       2.0299625122938209e-05, 
       2.1367696919782209e-05, 
       2.3470292930444889e-05, 
       2.3442989589177679e-05, 
       2.1448600708759867e-05, 
       2.5875790059004020e-05, 
       2.0206901013864554e-05, 
       2.2340762747346633e-05, 
       2.7668434375695498e-05, 
       1.9950397140628340e-05, 
       2.7987382645874545e-05, 
       2.4293432174721289e-05, 
       2.0000000000001635e-05, 
       2.3289522212324585e-05, 
       2.6411884857133361e-05, 
       1.9873280032980062e-05, 
       2.1714968767927616e-05, 
       2.1839395959091631e-05, 
       2.1839395959091631e-05, 
       2.3466193893882380e-05, 
       2.1371321669396220e-05, 
       1.9938647667130330e-05, 
       2.0579979780548632e-05, 
       2.0000000000002234e-05, 
       2.8028569936047660e-05, 
       2.1771973961456877e-05, 
       2.0865597099147342e-05, 
       2.0865597099147342e-05, 
       2.6509081981774738e-05, 
       2.0000000000000717e-05, 
       2.3312392036192551e-05, 
       2.0258657597617159e-05, 
       2.0312599848835227e-05, 
       2.0920446986977366e-05, 
       2.3578648602996724e-05, 
       2.2436677140611300e-05, 
       2.7961471003000168e-05, 
       2.1166956644026552e-05, 
       1.9816622275304193e-05, 
       2.3246323501454473e-05, 
       2.7599870125663251e-05, 
       2.7997160563574968e-05, 
       1.9800172644680534e-05, 
       2.3465473046836687e-05, 
       2.3466454990672666e-05, 
       2.0125642276699688e-05, 
       2.0989461740345620e-05, 
       2.7888274606794038e-05, 
       2.7044949730180104e-05, 
       2.0966102464808802e-05, 
       2.0581903307930400e-05, 
       2.7892539894698830e-05, 
       1.9737154177717354e-05, 
       1.9899679834429881e-05, 
       2.6918734450128765e-05, 
       2.1161418318711277e-05, 
       2.2501997422535255e-05, 
       2.2738960599759796e-05, 
       2.1220680407509679e-05, 
       2.7833813514487734e-05, 
       1.9762798536095193e-05, 
       2.2741211861016365e-05, 
       2.3222112650948485e-05, 
       2.3079346777615531e-05, 
       2.0291190701228842e-05, 
       1.9766670501007140e-05, 
       1.9766670501007140e-05, 
       2.0875764955551667e-05, 
       2.7496573607971389e-05, 
       2.7824980272308788e-05, 
       1.9851321667405355e-05, 
       2.6730772067331172e-05, 
       1.9692576643129609e-05, 
       2.1649674267789876e-05, 
       1.9731697228219266e-05, 
       2.7742305224850582e-05, 
       2.7417919438258271e-05, 
       2.0390893820573060e-05, 
       1.9932359809715495e-05, 
       2.7296124548866782e-05, 
       2.2695630930497120e-05, 
       2.0814759615587026e-05, 
       2.1687285062861226e-05, 
       2.6026686656087951e-05, 
       2.4822390716806992e-05, 
       1.9633408681435115e-05, 
       2.0000000000004044e-05, 
       2.4890159914209150e-05, 
       2.7567145184525361e-05, 
       2.2996908295505237e-05, 
       2.1302429968690023e-05, 
       2.3854713945038826e-05, 
       1.9715524650095351e-05, 
       2.7175854351842625e-05, 
       2.4191135362079194e-05, 
       2.0627872820108754e-05, 
       2.2919391864464396e-05, 
       2.6727338643468326e-05, 
       2.7254384117928532e-05, 
       2.0427343831901037e-05, 
       1.9507494707828872e-05, 
       2.3171225864097440e-05, 
       1.9999999999996590e-05, 
       1.9999999999996590e-05, 
       2.2986803073853592e-05, 
       2.1918124810525084e-05, 
       2.6884276333680600e-05, 
       2.0398485690050204e-05, 
       2.4849261494498854e-05, 
       2.6960534050138303e-05, 
       2.1421354160062961e-05, 
       2.4551814619669700e-05, 
       2.4314578972066763e-05, 
       2.6363367334252085e-05, 
       1.9793834371186968e-05, 
       1.9440818566363255e-05, 
       1.9459987741405260e-05, 
       1.9459987741405260e-05, 
       2.0193170237659376e-05, 
       2.6496086993112514e-05, 
       2.2344511393982552e-05, 
       2.6577328173695179e-05, 
       2.2427716074009642e-05, 
       2.4107810868212471e-05, 
       2.1284459172261010e-05, 
       2.2020722328864483e-05, 
       2.6607255959589445e-05, 
       2.7202002368497076e-05, 
       2.0166011832938444e-05, 
       2.5207862623641681e-05, 
       2.5207862623641681e-05, 
       1.9999999999995265e-05, 
       2.0450194783755439e-05, 
       2.2263441842113082e-05, 
       2.7406316216041685e-05, 
       1.9409515491409219e-05, 
       2.2039607298616913e-05, 
       2.1152152459107805e-05, 
       2.2230502365173873e-05, 
       1.9670319855560186e-05, 
       2.4722180326833094e-05, 
       2.2763295970793442e-05, 
       2.2774519638715391e-05, 
       2.6679869687682337e-05, 
       2.4756484656674434e-05, 
       2.0055294309148890e-05, 
       2.0897446815595530e-05, 
       2.4428532151668143e-05, 
       2.3990605543756118e-05, 
       2.7254286047169275e-05, 
       2.4278298363497046e-05, 
       2.2314581840559189e-05, 
       2.5743564664592442e-05, 
       1.9275105701808604e-05, 
       2.0049170001202115e-05, 
       1.9902356931750910e-05, 
       2.6313437919997652e-05, 
       2.7152726206478358e-05, 
       2.0401429930896030e-05, 
       2.0330718397024442e-05, 
       2.6165096318595719e-05, 
       1.9265299359219691e-05, 
       2.1568915069916871e-05, 
       2.0706080105933901e-05, 
       2.3303586545614864e-05, 
       1.9501731727464772e-05, 
       1.9559861810741433e-05, 
       2.1057046631608082e-05, 
       1.9502976870692780e-05, 
       2.4809720863124641e-05, 
       2.1408962260453609e-05, 
       2.1125619940789139e-05, 
       2.6828077807227632e-05, 
       2.0038477634749918e-05, 
       1.9130409289656855e-05, 
       1.9130409289656855e-05, 
       2.2346531757305858e-05, 
       2.1942710646256503e-05, 
       2.1008609943568522e-05, 
       2.1008609943568522e-05, 
       1.9277210464739012e-05, 
       2.2661137357054017e-05, 
       2.2516314479552214e-05, 
       2.6851072298362163e-05, 
       2.1908850916506237e-05, 
       2.2483068492197152e-05, 
       2.1696394476536286e-05, 
       2.2098923932636108e-05, 
       2.0187822485852769e-05, 
       2.5808682445505923e-05, 
       2.0017282176804168e-05, 
       2.0017282176804168e-05, 
       1.9126965962250800e-05, 
       1.9126965962250800e-05, 
       2.3006546974076691e-05, 
       2.3179533542101368e-05, 
       2.6890690896602348e-05, 
       2.6021653074393433e-05, 
       2.6181070741757018e-05, 
       1.9913283459284369e-05, 
       1.9636588737164034e-05, 
       2.2324367947449302e-05, 
       2.3865313956306112e-05, 
       2.1618881238829151e-05, 
       1.9535217125231259e-05, 
       1.9041443690995283e-05, 
       2.0039573023823696e-05, 
       2.2953513679398349e-05, 
       2.5567766231399125e-05, 
       1.9895906118055519e-05, 
       2.2271290300944949e-05, 
       2.6754209861454405e-05, 
       1.9007159132051157e-05, 
       1.8967818768193727e-05, 
       2.2691966842233759e-05, 
       1.8960610391852309e-05, 
       2.6716661520230744e-05, 
       2.5927823756213389e-05, 
       2.2844697314839551e-05, 
       2.3747869267251670e-05, 
       2.2145624115503147e-05, 
       1.9469447275291399e-05, 
       2.3456903323842540e-05, 
       2.0036159169612765e-05, 
       1.9400261141639878e-05, 
       1.8867912229973693e-05, 
       1.9059994910767383e-05, 
       1.8890362507229716e-05, 
       2.2017503441595469e-05, 
       2.4559321891593536e-05, 
       2.6338258067374874e-05, 
       1.9999999999998321e-05, 
       2.3950871968419052e-05, 
       2.1899470576429120e-05, 
       2.3545632007714398e-05, 
       1.8815345378508486e-05, 
       2.2113213212395635e-05, 
       2.2127291837682675e-05, 
       1.9273170719726031e-05, 
       2.6415429930153639e-05, 
       2.6239168240300300e-05, 
       1.8850214491991355e-05, 
       1.9768044576141600e-05, 
       1.9768044576141600e-05, 
       2.0820150284495630e-05, 
       2.0456223851454600e-05, 
       2.2395140178245569e-05, 
       2.6481718745530546e-05, 
       1.9778488240021942e-05, 
       1.8897410381751787e-05, 
       2.5684915178804853e-05, 
       1.8732589574812063e-05, 
       1.8925710859862174e-05, 
       1.9214149212320105e-05, 
       1.9999999999997667e-05, 
       1.9751377763641712e-05, 
       2.1459337858959660e-05, 
       1.9070305230096828e-05, 
       1.8771507026313736e-05, 
       1.9855656259784941e-05, 
       1.8749988728834255e-05, 
       2.2154281478803802e-05, 
       2.1850451281937156e-05, 
       2.2643318038647409e-05, 
       2.2008951276807387e-05, 
       2.0000000000002539e-05, 
       2.2880427612072051e-05, 
       1.9999999999995878e-05, 
       2.4877237016597618e-05, 
       2.2035984449286025e-05, 
       2.0000000000003109e-05, 
       1.9357035115452651e-05, 
       1.9530244037060011e-05, 
       2.2200628370302749e-05, 
       2.0062154492898264e-05, 
       2.3797117726057506e-05, 
       1.9999999999999697e-05, 
       1.8720368835276171e-05, 
       2.5213111253492232e-05, 
       1.8629141526322722e-05, 
       2.5875015718519001e-05, 
       1.9084191995397657e-05, 
       2.1858681197141754e-05, 
       2.6297689960513770e-05, 
       2.6106027196373846e-05, 
       2.2532828039759901e-05, 
       1.9603826032796683e-05, 
       2.1491473288966037e-05, 
       1.8922290352108163e-05, 
       2.6113247302592242e-05, 
       1.9149633976069442e-05, 
       1.8811843292880828e-05, 
       1.9999999999996088e-05, 
       1.9031035429947053e-05, 
       1.8940829007484389e-05, 
       1.9559353629394119e-05, 
       2.3107745574945384e-05, 
       2.6187273058042965e-05, 
       2.0552653667056900e-05, 
       2.1181177183621364e-05, 
       2.1735928533133341e-05, 
       1.9541597193925965e-05, 
       2.1297797499442487e-05, 
       2.3815495268834883e-05, 
       1.9522285792446204e-05, 
       2.2414619069298451e-05, 
       2.1210767134427090e-05, 
       2.1210267732373230e-05, 
       1.9493906790999241e-05, 
       2.4122252971414907e-05, 
       2.1626639836759702e-05, 
       2.2433786700830839e-05, 
       2.2344292372869138e-05, 
       2.2344292372869138e-05, 
       2.6077452515192849e-05, 
       2.5948213700606818e-05, 
       2.4560982150199448e-05, 
       1.8596696142305167e-05, 
       2.5871759928564313e-05, 
       1.9407277176706601e-05, 
       2.6005775386305672e-05, 
       1.9170122027778372e-05, 
       1.9170122027778372e-05, 
       2.2336586056727969e-05, 
       1.8756522821119444e-05, 
       2.0600324216915043e-05, 
       1.8391394104241385e-05, 
       1.9110730691336842e-05, 
       1.8841348400882591e-05, 
       1.9314907369897868e-05, 
       2.1314253102740545e-05, 
       2.3076103047365980e-05, 
       1.9484253497074381e-05, 
       2.5834814580752454e-05, 
       1.8365488924781366e-05, 
       2.5016541916426808e-05, 
       2.1012328915233477e-05, 
       1.9999999999996847e-05, 
       2.1496989977947132e-05, 
       2.5804511066017738e-05, 
       2.5762157378443724e-05, 
       2.5836435802320644e-05, 
       2.1860975195960274e-05, 
       2.5687587942747713e-05, 
       1.8981530758458973e-05, 
       1.9139111905999696e-05, 
       1.8280166318881625e-05, 
       2.4070343580249986e-05, 
       1.8693379445236393e-05, 
       2.3962516453856421e-05, 
       1.9020137080359865e-05, 
       1.9247958464256808e-05, 
       2.1447310559871313e-05, 
       1.9855861400691827e-05, 
       2.2468046196661062e-05, 
       2.0728155512884324e-05, 
       2.4060218809271120e-05, 
       2.1149639292928914e-05, 
       2.4517282613909728e-05, 
       2.2771292414867269e-05, 
       2.5579069717142981e-05, 
       2.1397066743789710e-05, 
       1.8961682355017992e-05, 
       1.8961682355017992e-05, 
       2.1655772967855787e-05, 
       2.5693726133351492e-05, 
       1.8163884442569935e-05, 
       1.9107983344487789e-05, 
       1.9114684912089114e-05, 
       1.9294009701245560e-05, 
       2.5660087323933798e-05, 
       1.8838086560794785e-05, 
       1.8858928509117386e-05, 
       1.8219051742317011e-05, 
       1.8728757426271528e-05, 
       2.2818810759957005e-05, 
       2.3715650740840214e-05, 
       1.8130283744853847e-05, 
       2.2938816829746240e-05, 
       1.8176890579316707e-05, 
       1.9952609681202569e-05, 
       1.8251530509771619e-05, 
       2.4254948895590587e-05, 
       1.8290989507197052e-05, 
       2.0376107544335592e-05, 
       1.9054448403108678e-05, 
       2.0352200663612104e-05, 
       1.8107252081351881e-05, 
       2.5531308981118411e-05, 
       2.5411294279712070e-05, 
       2.1633389361055961e-05, 
       1.9628235793110991e-05, 
       1.9180833451816496e-05, 
       1.9847621489528326e-05, 
       1.8585369921435439e-05, 
       1.9280398867673850e-05, 
       1.8308666362155669e-05, 
       2.2006291036435541e-05, 
       2.0242159615772351e-05, 
       2.1713381528779144e-05, 
       2.0934449436080176e-05, 
       1.8139101146347185e-05, 
       2.0731426911967222e-05, 
       1.8696409943926012e-05, 
       2.4544245828620170e-05, 
       2.2783430089359999e-05, 
       1.7993730535971711e-05, 
       2.5339301420864887e-05, 
       2.3338603348431365e-05, 
       2.4363468166549399e-05, 
       1.9122521503458406e-05, 
       1.8512114069329657e-05, 
       1.7958421814018464e-05, 
       2.1326923127170881e-05, 
       1.9199696490683087e-05, 
       2.1003649145343692e-05, 
       2.1989288523769274e-05, 
       1.7930229589536953e-05, 
       1.8555827680287684e-05, 
       1.7909710544758879e-05, 
       2.1385311528258373e-05, 
       2.1043458671072926e-05, 
       1.8526453187106103e-05, 
       1.9841228398451822e-05, 
       2.2256428064753554e-05, 
       1.8130641321224758e-05, 
       2.0449994146295535e-05, 
       2.3785604022540511e-05, 
       2.0731298984516647e-05, 
       2.0760416759699299e-05, 
       2.5167052823156837e-05, 
       1.8174308171080877e-05, 
       1.8585059002567886e-05, 
       2.2380340613526966e-05, 
       1.8869498634825147e-05, 
       1.8269361341749325e-05, 
       2.0893323738844832e-05, 
       2.0100910348089074e-05, 
       2.0094288114021303e-05, 
       2.2404243188656371e-05, 
       2.5016175558792949e-05, 
       2.0831810051257426e-05, 
       2.0259728632858026e-05, 
       1.9242038929860518e-05, 
       2.1289342496334620e-05, 
       2.0234278488705536e-05, 
       1.9379271630522181e-05, 
       1.8010372286510348e-05, 
       2.5144607602309949e-05, 
       2.0187915931443160e-05, 
       2.1996352765883427e-05, 
       2.0904858800077536e-05, 
       2.1720039102126300e-05, 
       2.3767208220916401e-05, 
       2.0865928634290329e-05, 
       2.4890723895812140e-05, 
       2.5047055332498873e-05, 
       2.4201236963873363e-05, 
       2.3169373330216857e-05, 
       2.3163038376964701e-05, 
       1.8855190323723966e-05, 
       1.8855190323723966e-05, 
       1.8671149418406665e-05, 
       1.7910303507043153e-05, 
       1.7763979523353320e-05, 
       1.7680410653021601e-05, 
       1.7687133192360568e-05, 
       1.7892962818897249e-05, 
       2.0333066309177655e-05, 
       2.2327869922090372e-05, 
       2.0094886283726992e-05, 
       2.0821716037595176e-05, 
       1.8614770868802281e-05, 
       1.7720496265061905e-05, 
       2.4900219533208990e-05, 
       1.7697729913895767e-05, 
       1.9844408131246652e-05, 
       1.9639872230426079e-05, 
       2.1388329725444331e-05, 
       1.7640375920275090e-05, 
       1.7803342763302024e-05, 
       2.4912020906200038e-05, 
       2.4590699583193215e-05, 
       2.0001499662828823e-05, 
       1.7691394246998973e-05, 
       1.7691394246998973e-05, 
       2.0101747579768383e-05, 
       2.3994530711384027e-05, 
       1.8965759925222337e-05, 
       2.2918838738801451e-05, 
       2.2918838738801451e-05, 
       2.0676047575015440e-05, 
       1.8568461860930664e-05, 
       2.0616313862486782e-05, 
       1.8005314258452920e-05, 
       2.4588443881671791e-05, 
       1.8886457529499383e-05, 
       2.0710696799016547e-05, 
       1.7699193284962316e-05, 
       2.4410728419558051e-05, 
       2.2977044570771224e-05, 
       2.0614114202624367e-05, 
       2.3861379884331902e-05, 
       2.4443744519722304e-05, 
       1.8738042318730059e-05, 
       2.0202721180008084e-05, 
       2.4805918098041182e-05, 
       2.0008714673284008e-05, 
       2.3977528418830052e-05, 
       1.9875094911181225e-05, 
       1.7841857479155481e-05, 
       1.7841857479155481e-05, 
       1.9913097446718887e-05, 
       1.7876435978992887e-05, 
       1.8143751657890148e-05, 
       2.4509662452613220e-05, 
       1.7532411669743742e-05, 
       1.8009989763961877e-05, 
       1.7744356854316162e-05, 
       1.8566432369396533e-05, 
       2.0116060720729326e-05, 
       1.8498218562056421e-05, 
       2.1463113121993310e-05, 
       1.9713299927463456e-05, 
       2.3052850846839306e-05, 
       1.7673110224656203e-05, 
       2.4239977584247124e-05, 
       2.0040883798321096e-05, 
       1.9417256525828080e-05, 
       1.9417256525828080e-05, 
       2.0955702972504675e-05, 
       1.8919347476838179e-05, 
       2.4432582009374769e-05, 
       1.8164088693106744e-05, 
       1.7520072035584252e-05, 
       1.9532365774056529e-05, 
       2.1382414917835685e-05, 
       1.9738472702713679e-05, 
       1.7629541808994239e-05, 
       2.0616644767350053e-05, 
       1.7703633377054495e-05, 
       1.8116772429845239e-05, 
       1.9879440855986523e-05, 
       2.0124772551965589e-05, 
       1.7864366935218176e-05, 
       1.8810990471941299e-05, 
       1.9743748452707123e-05, 
       2.3865065718840363e-05, 
       2.1158828201933256e-05, 
       1.9780547239243782e-05, 
       1.9722888630204921e-05, 
       2.1281486054161358e-05, 
       1.7352701970226175e-05, 
       1.9057764607714187e-05, 
       2.1854926101455391e-05, 
       2.1335342960777997e-05, 
       2.3905614432718570e-05, 
       2.4508257473243487e-05, 
       1.8105890428700859e-05, 
       1.8105890428700859e-05, 
       2.3425411593397924e-05, 
       1.8465966771970703e-05, 
       2.1329285074762914e-05, 
       1.9554010968489385e-05, 
       1.9554010968489385e-05, 
       1.7306685536057503e-05, 
       2.1740601870453343e-05, 
       1.8263059311301995e-05, 
       2.4316548339899093e-05, 
       2.3547674773871388e-05, 
       2.3480237790270722e-05, 
       2.0961556471640558e-05, 
       2.2754038596753759e-05, 
       1.9999999999996922e-05, 
       1.9999999999996922e-05, 
       1.7697016103600526e-05, 
       2.3535283663645454e-05, 
       1.7392481252393953e-05, 
       1.8206092048519223e-05, 
       2.4165255746693801e-05, 
       1.7987481553165490e-05, 
       1.9548511366159317e-05, 
       1.7635978005143752e-05, 
       2.0343072227592185e-05, 
       1.9525566385121238e-05, 
       1.7941370251689439e-05, 
       2.3313479894531704e-05, 
       1.7316200591392876e-05, 
       2.0256262953270805e-05, 
       2.2782267752081354e-05, 
       2.0722440633148783e-05, 
       2.2570865527763971e-05, 
       2.2060084096032708e-05, 
       1.8159538450873559e-05, 
       1.8536393920439748e-05, 
       1.7557962389639970e-05, 
       2.0002834694791437e-05, 
       2.3342952109573024e-05, 
       2.2086456562446664e-05, 
       2.0498343612431602e-05, 
       1.7351005763319398e-05, 
       1.8528906464525795e-05, 
       1.7697574838161988e-05, 
       2.3462174193718691e-05, 
       1.9415110018429218e-05, 
       2.1620357335974060e-05, 
       2.2891788457019663e-05, 
       2.1585633011509839e-05, 
       1.8404306776497948e-05, 
       1.8404306776497948e-05, 
       2.0166310355110907e-05, 
       2.1170275625818371e-05, 
       1.9370530680985174e-05, 
       1.7147050315030178e-05, 
       1.8107541003231476e-05, 
       2.2816831798848336e-05, 
       2.1564857025852960e-05, 
       1.7121195645320671e-05, 
       2.3830216908688118e-05, 
       1.7640066694329988e-05, 
       2.3572348406129228e-05, 
       1.8211033238230675e-05, 
       1.8732999796455037e-05, 
       2.0047349746630555e-05, 
       2.2128156456494762e-05, 
       2.1043440060757221e-05, 
       2.2995362270055442e-05, 
       1.9991878851623358e-05, 
       2.3862632834583936e-05, 
       2.4067298190273253e-05, 
       1.7626718557277661e-05, 
       1.7277189678332198e-05, 
       1.9995528903169129e-05, 
       2.0178856238418034e-05, 
       1.8663548651772293e-05, 
       1.7844257239569768e-05, 
       1.7708938371280653e-05, 
       1.7708938371280653e-05, 
       2.1100876216390546e-05, 
       1.8936351533887953e-05, 
       2.1909249848044001e-05, 
       2.1909249848044001e-05, 
       2.0681238638649732e-05, 
       2.0235006211901255e-05, 
       2.3771931379770966e-05, 
       1.7271821128373024e-05, 
       2.0887875915477799e-05, 
       1.8265922996359798e-05, 
       2.1330995819079070e-05, 
       1.8766059560146823e-05, 
       1.7264445253428378e-05, 
       1.7264445253428378e-05, 
       1.9940401050376650e-05, 
       2.3842389296058815e-05, 
       2.2790918071610945e-05, 
       2.0517006919138890e-05, 
       1.9187496467440822e-05, 
       2.0960358783439589e-05, 
       2.0640630344123610e-05, 
       2.0945678455290999e-05, 
       2.2745010809265887e-05, 
       1.6960407224583200e-05, 
       2.0167093769747340e-05, 
       1.7807585815741925e-05, 
       1.7102038575890068e-05, 
       1.8392257888280286e-05, 
       2.3834263613959718e-05, 
       2.1012673084139515e-05, 
       2.3522038035291879e-05, 
       1.9404443666463979e-05, 
       1.7586315175081062e-05, 
       1.6835823803580511e-05, 
       1.7156411319383199e-05, 
       1.7479589437298256e-05, 
       2.3597271640262422e-05, 
       1.7287133384322985e-05, 
       1.8479863436802956e-05, 
       1.7286762049032248e-05, 
       1.7416590758136627e-05, 
       1.6898364375521922e-05, 
       1.7692832868685998e-05, 
       1.7692832868685998e-05, 
       2.3720659507815452e-05, 
       1.8772310497314735e-05, 
       2.0000000000004633e-05, 
       1.7132608549578928e-05, 
       1.6842701071753262e-05, 
       1.8821576107333434e-05, 
       1.9535192737125097e-05, 
       2.0123839854511366e-05, 
       1.8991720201829508e-05, 
       2.2838295057065624e-05, 
       2.0350669413703838e-05, 
       2.0445322535878866e-05, 
       2.0445322535878866e-05, 
       2.0188604558457556e-05, 
       1.6907632508101273e-05, 
       1.9252757586184596e-05, 
       1.8894781575960861e-05, 
       1.7888036875733064e-05, 
       2.2662977395270788e-05, 
       1.8053450720011652e-05, 
       1.6881160349336304e-05, 
       1.6675781899278904e-05, 
       2.3477252566993164e-05, 
       1.6671791014294461e-05, 
       1.6777623147983242e-05, 
       1.8301536416555231e-05, 
       1.7688458205896298e-05, 
       1.8115006792659946e-05, 
       2.3115301780885825e-05, 
       1.9547969899974727e-05, 
       2.1232942097686476e-05, 
       1.7603971171241998e-05, 
       1.8544627601127559e-05, 
       2.3122379100758194e-05, 
       2.0258173778535597e-05, 
       1.6669912361551040e-05, 
       2.0115386471181266e-05, 
       1.6669237843083501e-05, 
       2.2192905489048603e-05, 
       1.9523128327740499e-05, 
       2.2117423760742725e-05, 
       2.1246485395735331e-05, 
       1.6902877538767291e-05, 
       2.0948203655519942e-05, 
       2.3293130831803150e-05, 
       1.6755912441012491e-05, 
       1.7382368362886235e-05, 
       1.9296487651582421e-05, 
       2.3282029915945936e-05, 
       1.9590558508559368e-05, 
       1.7433881448371084e-05, 
       2.3066800207303732e-05, 
       1.6522532115456080e-05, 
       1.6519869720739026e-05, 
       1.6523350943213977e-05, 
       2.1106453553785930e-05, 
       2.0855518262297264e-05, 
       2.1612473413096596e-05, 
       1.7934231852549327e-05, 
       1.9321267329180687e-05, 
       1.6541097702661042e-05, 
       2.2869787571877537e-05, 
       2.3154810789682062e-05, 
       2.2973975708738382e-05, 
       2.2961707807479175e-05, 
       2.1241731584823695e-05, 
       1.7001513226710508e-05, 
       1.6928372631101334e-05, 
       2.2156590296457815e-05, 
       1.9530984570005048e-05, 
       1.6442805337604780e-05, 
       1.9276688387829358e-05, 
       2.2342760509353491e-05, 
       1.6763425422484552e-05, 
       2.1488128728838385e-05, 
       2.3114813929650088e-05, 
       1.8781375513748516e-05, 
       1.8946871324177870e-05, 
       1.8808814873676904e-05, 
       2.1605403140124169e-05, 
       2.3102567029396645e-05, 
       2.0605355541645451e-05, 
       1.7213748976173022e-05, 
       1.7793395274541191e-05, 
       1.6371128677377743e-05, 
       1.7077926575575495e-05, 
       1.6391641230189141e-05, 
       1.6802612342596238e-05, 
       1.6963230126010973e-05, 
       2.2047351993580104e-05, 
       1.8918710499546012e-05, 
       1.7591270267590143e-05, 
       2.2311092638659019e-05, 
       1.6387351086933984e-05, 
       1.9524251927299393e-05, 
       1.8881642834623986e-05, 
       1.9077429636800265e-05, 
       1.8917688474233060e-05, 
       2.1804552128331417e-05, 
       2.2656255284805286e-05, 
       1.9943006491561431e-05, 
       1.7469282681897057e-05, 
       1.6974804183679965e-05, 
       2.2740256596326434e-05, 
       1.7207528739574835e-05, 
       1.6970042819011529e-05, 
       1.9850897826104042e-05, 
       1.9140106729782767e-05, 
       1.6676344305763090e-05, 
       1.9090537627228603e-05, 
       1.6736497529196532e-05, 
       2.0133000437335605e-05, 
       2.2757006468817194e-05, 
       1.6702664019797677e-05, 
       1.6315397773966979e-05, 
       2.0790900210589302e-05, 
       1.8524588975115479e-05, 
       1.6451802847666920e-05, 
       1.7365645610299736e-05, 
       2.2855177578230783e-05, 
       2.2819858675157721e-05, 
       2.0904725780274183e-05, 
       2.1288427944038591e-05, 
       1.6192000254433390e-05, 
       1.6245299440748172e-05, 
       1.6245299440748172e-05, 
       1.8127242507720723e-05, 
       2.2275779288788813e-05, 
       2.0773727929471758e-05, 
       2.2829793251445898e-05, 
       1.8921584142257576e-05, 
       1.6339652113304440e-05, 
       1.8975843734927459e-05, 
       2.2688254878616019e-05, 
       1.8905393105560276e-05, 
       1.7022317425458386e-05, 
       2.2019462623333437e-05, 
       1.6246538359845686e-05, 
       1.6216986312379648e-05, 
       1.6121927420355377e-05, 
       2.1956054374316435e-05, 
       1.6562279413404521e-05, 
       1.6298213345503501e-05, 
       1.9060170845724578e-05, 
       1.8922364839388372e-05, 
       1.7061149537075686e-05, 
       1.7094956165661685e-05, 
       2.0821138244668801e-05, 
       2.0174318884068203e-05, 
       1.9140445654777217e-05, 
       1.6081710891039304e-05, 
       1.6227456944695615e-05, 
       1.7666140914068323e-05, 
       1.6567806372912873e-05, 
       1.7453416883691020e-05, 
       2.2657645268488578e-05, 
       2.0431023949867525e-05, 
       1.6950145720708444e-05, 
       1.9890155960664404e-05, 
       1.6053402041731836e-05, 
       1.9403903446542360e-05, 
       1.6110160628986708e-05, 
       2.2612138573918036e-05, 
       1.6667605616919987e-05, 
       1.6644933137546988e-05, 
       1.5980449473580598e-05, 
       1.6163551770792984e-05, 
       1.6617055878780183e-05, 
       2.2562695973202133e-05, 
       1.9886332432387718e-05, 
       2.1722027768969165e-05, 
       1.9910140933530282e-05, 
       1.9528441425711874e-05, 
       1.7245527634880786e-05, 
       1.8161074703236701e-05, 
       1.8017076822875017e-05, 
       1.7015623552843490e-05, 
       2.2501134802554360e-05, 
       1.8220193544218455e-05, 
       1.9510271297828276e-05, 
       1.6147164518456417e-05, 
       1.8771757892139438e-05, 
       1.5990302660566169e-05, 
       1.6012919130152530e-05, 
       1.6549792915255268e-05, 
       1.8945554267631933e-05, 
       1.6200727414548244e-05, 
       1.6284763419791300e-05, 
       1.6131260434911045e-05, 
       1.7709260348775388e-05, 
       2.1710321179262857e-05, 
       1.5951107002733541e-05, 
       1.6265113750427505e-05, 
       1.7234577675678331e-05, 
       2.2331401934126915e-05, 
       2.2371301127586665e-05, 
       2.2217390650471838e-05, 
       2.0093188338363381e-05, 
       2.2092602864857998e-05, 
       2.2346528997350687e-05, 
       1.6100076294670998e-05, 
       1.5942587442484113e-05, 
       1.6194102205535764e-05, 
       1.5898226785692709e-05, 
       2.1155515311702435e-05, 
       1.6583252908457672e-05, 
       1.7278340779413192e-05, 
       1.8470997967287768e-05, 
       2.1856606817620013e-05, 
       1.5745667999998268e-05, 
       2.2222483777966504e-05, 
       1.7492088499757034e-05, 
       1.6560501187162072e-05, 
       1.5726195086615124e-05, 
       1.7510390518953370e-05, 
       1.7845196780493277e-05, 
       1.6334757830895036e-05, 
       1.6334757830895036e-05, 
       1.6076950486640756e-05, 
       1.6829246649323837e-05, 
       1.6315128457849984e-05, 
       1.9051886859255621e-05, 
       1.5728524163541113e-05, 
       1.7443462232679977e-05, 
       2.2092696552803133e-05, 
       1.8469657222836685e-05, 
       1.5691417502302559e-05, 
       1.9453709029190264e-05, 
       1.6511741573958431e-05, 
       1.7645208764304032e-05, 
       2.1595499629582765e-05, 
       1.8377882355999809e-05, 
       1.6561248364934335e-05, 
       1.7984672575260438e-05, 
       1.8549200797180666e-05, 
       2.2087304554959113e-05, 
       2.1828425648732798e-05, 
       2.0391063478211066e-05, 
       2.1279859781010948e-05, 
       2.1508616375209985e-05, 
       1.6660970869587527e-05, 
       1.8588780727374235e-05, 
       1.6565400381325721e-05, 
       1.5717566299634416e-05, 
       2.0535965288849930e-05, 
       1.6466581956504004e-05, 
       1.8647137458050032e-05, 
       1.5610352888002224e-05, 
       1.6179081609254638e-05, 
       2.1794906242845501e-05, 
       1.5577931509606555e-05, 
       1.9586511985099117e-05, 
       2.1909841500100040e-05, 
       2.1998579505542611e-05, 
       2.1493479302175947e-05, 
       1.5559493904853652e-05, 
       1.5941066616909995e-05, 
       1.7676337579382255e-05, 
       1.8062776040369259e-05, 
       1.6659653881123988e-05, 
       2.0000000000003393e-05, 
       1.7849568206700026e-05, 
       1.5965574229352592e-05, 
       1.6437317940038550e-05, 
       2.0765863952647323e-05, 
       2.1846633864365964e-05, 
       1.8424062803935875e-05, 
       1.7064953581099732e-05, 
       2.1263222644974115e-05, 
       2.1825539867019784e-05, 
       2.0186503194792118e-05, 
       1.6318294159138578e-05, 
       1.6565632659979917e-05, 
       2.1854141651488121e-05, 
       1.9721867338203134e-05, 
       1.6591714642297451e-05, 
       1.6591714642297451e-05, 
       1.5516381802169052e-05, 
       1.8755561655044175e-05, 
       1.5955873784711584e-05, 
       1.5774574480693046e-05, 
       2.1735051759274436e-05, 
       1.6066594092445572e-05, 
       1.5846906777686247e-05, 
       1.8868337038229099e-05, 
       1.6265232406221913e-05, 
       1.7782797412512916e-05, 
       1.7809191450193069e-05, 
       1.5599869105108737e-05, 
       2.1693795445600207e-05, 
       1.8985056752555413e-05, 
       1.8113328581290859e-05, 
       1.6021103607237798e-05, 
       2.1684010188073883e-05, 
       1.9957107652443819e-05, 
       1.9469171199475047e-05, 
       1.9886956122973244e-05, 
       1.6479758202524700e-05, 
       2.1385307632514737e-05, 
       1.6649743338875126e-05, 
       1.9999999999999219e-05, 
       1.9999999999999219e-05, 
       1.9999999999997972e-05, 
       1.9999999999999134e-05, 
       1.9999999999998982e-05, 
       2.0000000000001706e-05, 
       1.9999999999998365e-05, 
       2.0000000000000425e-05, 
       1.9999999999995062e-05, 
       1.9999999999998385e-05, 
       1.9999999999998385e-05, 
       1.5977290848338997e-05, 
       1.5977290848338997e-05, 
       1.5420271501855298e-05, 
       2.0863754748184489e-05, 
       1.6308730864178650e-05, 
       2.1544639578979746e-05, 
       1.7659009667905495e-05, 
       1.7739548753552605e-05, 
       1.5878542845562615e-05, 
       1.5877807013638059e-05, 
       1.8395025156768364e-05, 
       1.9810470929856084e-05, 
       1.9807995984286075e-05, 
       2.1597618017637537e-05, 
       2.1380717362667158e-05, 
       1.5728529446399404e-05, 
       1.5799744767817560e-05, 
       1.7679947029514624e-05, 
       1.7877447508622171e-05, 
       1.7801104938934124e-05, 
       1.7961378393018670e-05, 
       1.5299759945115771e-05, 
       2.1370152468451239e-05, 
       1.6510340956330097e-05, 
       1.5486302119327786e-05, 
       1.5455725955911955e-05, 
       2.0135264182178537e-05, 
       1.6853292001640325e-05, 
       1.6832442225826654e-05, 
       1.6967090687246489e-05, 
       1.9071013973279498e-05, 
       2.1538167031908605e-05, 
       1.5548412391258848e-05, 
       2.1237395492952342e-05, 
       1.8164508774457600e-05, 
       1.7861361836655403e-05, 
       1.8624999829026083e-05, 
       1.7033020620889857e-05, 
       1.6765228345600891e-05, 
       1.7878966106612229e-05, 
       1.6311695542618120e-05, 
       1.7879737954012614e-05, 
       1.6721225102334025e-05, 
       1.7874945417602016e-05, 
       1.6812566985582902e-05, 
       1.7252375527424996e-05, 
       1.7680886806664228e-05, 
       1.6806036523154016e-05, 
       1.6252260579936975e-05, 
       1.6122600836113829e-05, 
       1.6667517199983966e-05, 
       1.6193045281960557e-05, 
       1.5398488477955585e-05, 
       2.0406259702848420e-05, 
       1.5590941002720459e-05, 
       1.6329699181494415e-05, 
       1.5328559236814314e-05, 
       1.7370318758741554e-05, 
       1.8812901235652901e-05, 
       1.8812901235652901e-05, 
       1.7369065115322222e-05, 
       1.5591093902455968e-05, 
       2.0656984802022977e-05, 
       1.9591177295441521e-05, 
       1.6687695692916484e-05, 
       2.1130099301282708e-05, 
       2.1325073661143272e-05, 
       2.1157688524304195e-05, 
       1.7461179860679065e-05, 
       1.6423108276909969e-05, 
       1.8245319251193914e-05, 
       2.1025867185539587e-05, 
       1.8019242603315165e-05, 
       2.0248387147714210e-05, 
       1.8157936344016041e-05, 
       1.6389757351005462e-05, 
       1.5100347492838665e-05, 
       1.5100347492838665e-05, 
       1.5230262986091805e-05, 
       2.1215582948340956e-05, 
       1.5414961617522229e-05, 
       1.7595936685111189e-05, 
       2.0616962092432869e-05, 
       1.5432323397707398e-05, 
       1.8048638426624819e-05, 
       1.5363941806358455e-05, 
       1.9283751778808183e-05, 
       1.7100546614081047e-05, 
       2.0102533641941161e-05, 
       1.7384638123306500e-05, 
       1.9906283196170509e-05, 
       1.8814199491772168e-05, 
       1.5013082294691551e-05, 
       1.5580516979981153e-05, 
       1.5580516979981153e-05, 
       1.8620212778572103e-05, 
       1.5264870491652246e-05, 
       2.0135536757306594e-05, 
       1.6850842558128003e-05, 
       1.5208548688269298e-05, 
       1.7459444804946951e-05, 
       1.8797311920859079e-05, 
       1.5559069052397929e-05, 
       1.5088699598054669e-05, 
       2.1057493730659698e-05, 
       1.6434958530078212e-05, 
       1.7030566798391419e-05, 
       1.5067069593693518e-05, 
       1.6693029274941945e-05, 
       1.8596654607507702e-05, 
       1.5775316812798327e-05, 
       1.5193852193121605e-05, 
       2.0579104068435114e-05, 
       1.6419791920995258e-05, 
       2.1022166679239770e-05, 
       1.5405578998752347e-05, 
       1.5137724906236635e-05, 
       1.5173460081174190e-05, 
       1.4911920522673188e-05, 
       1.5526408715852300e-05, 
       1.6954508804696774e-05, 
       2.1003900928651508e-05, 
       1.5286404993118030e-05, 
       1.9922310076878961e-05, 
       1.4896210933481640e-05, 
       2.1031371405014781e-05, 
       1.5952865525235097e-05, 
       1.6544617095966181e-05, 
       1.7218331604476612e-05, 
       1.7373531888541584e-05, 
       2.0496606911620923e-05, 
       1.4852990397279256e-05, 
       1.5569865199886903e-05, 
       1.8516799849218081e-05, 
       1.4828457252575804e-05, 
       1.5624891989336155e-05, 
       1.5624891989336155e-05, 
       1.4820220294325903e-05, 
       1.8874653645544506e-05, 
       1.6038903768216540e-05, 
       1.7290995979226839e-05, 
       1.9999999999995753e-05, 
       1.7273542428415941e-05, 
       1.4821832951205098e-05, 
       1.6753200237546766e-05, 
       1.6835026339759140e-05, 
       1.8997761949861205e-05, 
       1.9872279006599585e-05, 
       1.5838127256524234e-05, 
       1.5311895804839845e-05, 
       1.7246606887827225e-05, 
       1.5028273905079650e-05, 
       2.0898021927152357e-05, 
       1.4971825628919935e-05, 
       1.5166686490918444e-05, 
       1.7347097119550183e-05, 
       2.0385052342525400e-05, 
       2.0385052342525400e-05, 
       1.7226647172434394e-05, 
       1.8707304316293873e-05, 
       1.4971493848252444e-05, 
       1.7331983462908899e-05, 
       2.0077240491880317e-05, 
       1.5553746090871645e-05, 
       1.9088499511825495e-05, 
       1.6026474807185141e-05, 
       1.5611076867388777e-05, 
       1.7323177062971019e-05, 
       2.0808307448607207e-05, 
       1.9774199270579913e-05, 
       2.0802383711941766e-05, 
       1.4876178559690130e-05, 
       1.4781188707033816e-05, 
       1.4712464152710686e-05, 
       2.0772897788380036e-05, 
       1.5155486796189431e-05, 
       1.5268673215336786e-05, 
       1.9041790271914958e-05, 
       1.9111524188071107e-05, 
       1.5564865986683139e-05, 
       1.7594038744631663e-05, 
       1.7892962818902775e-05, 
       1.7319090635615784e-05, 
       1.8674395040506046e-05, 
       1.6240252637876185e-05, 
       1.7032907904837461e-05, 
       1.6050867624039417e-05, 
       1.5175963048911486e-05, 
       1.4792337855245117e-05, 
       1.4652905497411430e-05, 
       2.0680988167426885e-05, 
       2.0119294660253976e-05, 
       1.6095729797305823e-05, 
       2.0625093907111433e-05, 
       1.9521098756728870e-05, 
       2.0595661400170565e-05, 
       1.7163151624926476e-05, 
       1.7586841823329863e-05, 
       1.4890445304719087e-05, 
       1.8117403341027752e-05, 
       2.0416214909593494e-05, 
       1.5452353127622199e-05, 
       1.5054024981533608e-05, 
       1.5207350771354807e-05, 
       1.4615593876244564e-05, 
       1.4720638515039225e-05, 
       1.7128828706261080e-05, 
       1.8277759505547590e-05, 
       1.4727151546726160e-05, 
       1.9774829801442692e-05, 
       1.4637347381722367e-05, 
       1.6897349439657455e-05, 
       2.0028722685522382e-05, 
       1.8131287278910380e-05, 
       1.7375893020347383e-05, 
       1.7375893020347383e-05, 
       1.4604133161637290e-05, 
       1.5314749164619713e-05, 
       1.6465255960680018e-05, 
       1.8067845275684433e-05, 
       1.7609693232163050e-05, 
       1.8299191964204232e-05, 
       1.8227708368623237e-05, 
       1.5186755906245460e-05, 
       2.0142451525239328e-05, 
       1.5337914335951559e-05, 
       1.7595208210728162e-05, 
       1.5863110941040158e-05, 
       1.8393660809236776e-05, 
       1.4969141844527169e-05, 
       1.5035586495924964e-05, 
       1.5363517569489903e-05, 
       1.4854304650086469e-05, 
       1.5623459703162102e-05, 
       1.4767998889265674e-05, 
       1.7159067231406097e-05, 
       1.4539948710924227e-05, 
       1.7429313642443375e-05, 
       1.5031823833576059e-05, 
       1.7539807742658661e-05, 
       1.4980268277844546e-05, 
       1.5011269893965837e-05, 
       1.5248557503425525e-05, 
       1.8580287333637601e-05, 
       1.5043552299635457e-05, 
       1.8168963708791143e-05, 
       1.8651388613578185e-05, 
       2.0189993095340191e-05, 
       1.5432236115146692e-05, 
       1.5593621552787403e-05, 
       1.4474509998140308e-05, 
       1.5785590202942033e-05, 
       1.9611071407936163e-05, 
       1.6527176552352358e-05, 
       1.6232588600915820e-05, 
       1.4681213271934059e-05, 
       1.6896558968491199e-05, 
       1.6094049781341862e-05, 
       1.8552136823778786e-05, 
       1.7183534692853691e-05, 
       1.6927102978064870e-05, 
       1.9911752099446130e-05, 
       1.5292577163550201e-05, 
       2.0327942947741167e-05, 
       2.0202000369750319e-05, 
       1.4554175151559585e-05, 
       1.4386665081500624e-05, 
       1.6895948089447729e-05, 
       1.4757355285903934e-05, 
       1.4733338471278170e-05, 
       1.7958138239126570e-05, 
       1.9119896815625364e-05, 
       1.5176898949600512e-05, 
       1.4539556056417575e-05, 
       1.9999999999997976e-05, 
       1.7705303703023824e-05, 
       1.6796812012481824e-05, 
       2.0219456420751133e-05, 
       1.9461929824423670e-05, 
       1.4914971941135734e-05, 
       1.4725464117338971e-05, 
       1.4905502791489821e-05, 
       1.6704245104031172e-05, 
       1.6346849763034155e-05, 
       2.0183850467384718e-05, 
       1.7328649070163523e-05, 
       1.9993058534651745e-05, 
       2.0155206632469547e-05, 
       1.7158161840755303e-05, 
       1.6180506087687834e-05, 
       1.5044975232053725e-05, 
       1.9707045736827601e-05, 
       1.8274161459051690e-05, 
       1.7221161176813659e-05, 
       1.4421755533291740e-05, 
       1.6646253155362778e-05, 
       1.4334748803919929e-05, 
       2.0121364742274587e-05, 
       1.4291639566070391e-05, 
       1.5670943520524227e-05, 
       1.6482971711744494e-05, 
       1.4576547625996510e-05, 
       1.4655517265658455e-05, 
       1.7757433416121555e-05, 
       1.7970663476776966e-05, 
       1.5742856862648214e-05, 
       1.7018774590597435e-05, 
       1.8645251871130552e-05, 
       1.8645185290879596e-05, 
       1.7335664817918533e-05, 
       1.4818482970319865e-05, 
       1.6811733070769529e-05, 
       1.4250193466941593e-05, 
       1.6093868620042199e-05, 
       1.9326975582424918e-05, 
       1.9941949295786189e-05, 
       2.0052822795065137e-05, 
       1.4459181220275343e-05, 
       1.9728304593322032e-05, 
       1.7638712898908959e-05, 
       2.0072723215681190e-05, 
       1.5006211187950023e-05, 
       1.5717891767448431e-05, 
       1.7127581819786330e-05, 
       1.5713156326138730e-05, 
       1.8781582483123569e-05, 
       1.4276969090582839e-05, 
       2.0000084035178723e-05, 
       1.7159237423769609e-05, 
       1.8712812857215246e-05, 
       1.6555061551292769e-05, 
       1.4879353459598888e-05, 
       1.7156299998227326e-05, 
       1.9999999999995990e-05, 
       1.9999999999995512e-05, 
       2.0000000000001137e-05, 
       2.0000000000004206e-05, 
       1.9999999999998375e-05, 
       1.9999999999998037e-05, 
       2.0000000000003796e-05, 
       2.0000000000003928e-05, 
       1.9999999999996492e-05, 
       1.9999999999999910e-05, 
       1.9999999999998931e-05, 
       1.9999999999998562e-05, 
       2.0000000000003054e-05, 
       2.0000000000002682e-05, 
       2.0000000000004742e-05, 
       1.9999999999995546e-05, 
       1.9999999999996146e-05, 
       1.9999999999997400e-05, 
       1.9999999999998955e-05, 
       1.9999999999998511e-05, 
       1.9999999999998596e-05, 
       1.9999999999998596e-05, 
       2.0000000000003295e-05, 
       2.0000000000003474e-05, 
       2.0000000000003305e-05, 
       2.0000000000003305e-05, 
       1.9999999999995577e-05, 
       2.0000000000002624e-05, 
       2.0000000000002624e-05, 
       1.9999999999997010e-05, 
       1.9999999999999029e-05, 
       2.0000000000004735e-05, 
       1.9999999999995668e-05, 
       2.0000000000003241e-05, 
       2.0000000000002739e-05, 
       2.0000000000355461e-05, 
       1.9999999999999497e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.5939276814151124e-05, 
       1.6625936318879249e-05, 
       1.5877031373873162e-05, 
       1.8140395049239783e-05, 
       1.8308062360460845e-05, 
       1.5199292512977197e-05, 
       1.9165615722705478e-05, 
       1.5935874394150922e-05, 
       1.8691594224387670e-05, 
       1.7759550229843893e-05, 
       1.5086318238923548e-05, 
       1.6586312590480193e-05, 
       1.5159660928851428e-05, 
       1.9748344662031826e-05, 
       1.4184927099610776e-05, 
       1.6508969673611119e-05, 
       1.6842490537408845e-05, 
       1.4169264681528034e-05, 
       1.4973203993347896e-05, 
       1.6357064236119631e-05, 
       1.4081312733097830e-05, 
       1.9653155872593739e-05, 
       1.6576768086592933e-05, 
       1.4763383057877282e-05, 
       1.7895354644045685e-05, 
       1.7177379033345189e-05, 
       1.9835479307516483e-05, 
       1.9760450887237208e-05, 
       1.5922008662989584e-05, 
       1.4192356165573130e-05, 
       1.4446796603526607e-05, 
       1.9360542573648798e-05, 
       1.4862500633760479e-05, 
       1.4130615854198479e-05, 
       1.7993224758436213e-05, 
       1.4110303572950454e-05, 
       1.4439742680236640e-05, 
       1.8664932428318519e-05, 
       1.9830006478665723e-05, 
       1.4059554246394452e-05, 
       1.8361853591135091e-05, 
       1.8568941237139572e-05, 
       1.5159350661795558e-05, 
       1.4538116772865378e-05, 
       1.4619017599940080e-05, 
       1.4734848775359021e-05, 
       1.4530144801751874e-05, 
       1.4530144801751874e-05, 
       1.8711782765589747e-05, 
       1.7315393781889078e-05, 
       1.4582369958389614e-05, 
       1.4051410418387655e-05, 
       1.4802867434486541e-05, 
       1.3997770115497577e-05, 
       1.6294247591339046e-05, 
       1.9743431711978656e-05, 
       1.5424937527466854e-05, 
       1.4213582477691444e-05, 
       1.4000502399119676e-05, 
       1.4364465923070075e-05, 
       1.5817637901761373e-05, 
       1.6439319421041781e-05, 
       1.6520754002690123e-05, 
       1.4913314906272624e-05, 
       1.4338555938038893e-05, 
       1.4197002360549593e-05, 
       1.4599823824599674e-05, 
       1.7636462865049696e-05, 
       1.8994561772210017e-05, 
       1.6643106215529342e-05, 
       1.4583722683031408e-05, 
       1.9653099091101223e-05, 
       1.7121883203115021e-05, 
       1.9086288472752674e-05, 
       1.5116441838818137e-05, 
       1.9705578316040376e-05, 
       1.6234026576853952e-05, 
       1.9257310179903845e-05, 
       1.6231048319121017e-05, 
       1.5832105455099860e-05, 
       1.8576099081583036e-05, 
       1.5106001958022986e-05, 
       1.3906246436919196e-05, 
       1.4058462464141080e-05, 
       1.8493797485177163e-05, 
       1.3921528037739497e-05, 
       1.4382249695317635e-05, 
       1.4320293209520982e-05, 
       1.6343137405858013e-05, 
       1.9634988456481449e-05, 
       1.4069188474811291e-05, 
       1.3938111000651239e-05, 
       1.8599369508915002e-05, 
       1.4742257213966482e-05, 
       1.3900685838893411e-05, 
       1.4063705351466365e-05, 
       1.9207170807620005e-05, 
       1.6259790000197429e-05, 
       1.7613010324938945e-05, 
       1.6518564385013581e-05, 
       1.6518564385013581e-05, 
       1.8644505450716566e-05, 
       1.4702018963999849e-05, 
       1.7268308311332900e-05, 
       1.6146018171990213e-05, 
       1.7773427066869232e-05, 
       1.9196006637171878e-05, 
       1.9393106466433335e-05, 
       1.5496877413252688e-05, 
       1.9429654555499584e-05, 
       1.4379712814157296e-05, 
       1.6079721609574042e-05, 
       1.6464831290779503e-05, 
       1.5871668585008463e-05, 
       1.7015510992110149e-05, 
       1.5868517312155221e-05, 
       1.7644942363623315e-05, 
       1.5553784934446578e-05, 
       1.4332250703755212e-05, 
       1.3966361503711811e-05, 
       1.7871188241571116e-05, 
       1.3890484649062350e-05, 
       1.4097161510313847e-05, 
       1.9325691016493326e-05, 
       1.4163356986958101e-05, 
       1.3792167111711009e-05, 
       1.5718276067109254e-05, 
       1.3898593889712613e-05, 
       1.5336605592104323e-05, 
       1.4185338323611284e-05, 
       1.9302224824416489e-05, 
       1.9274896091972504e-05, 
       1.7607548689973156e-05, 
       1.9424470755392661e-05, 
       1.9407444552125230e-05, 
       1.9292331000324735e-05, 
       1.6214171167468407e-05, 
       1.5749766451329129e-05, 
       1.3821746312906816e-05, 
       1.4111874837520725e-05, 
       1.9423182861496826e-05, 
       1.4716813008698554e-05, 
       1.5726195086620572e-05, 
       1.3875264504861185e-05, 
       1.6572601239953122e-05, 
       1.3975000138134530e-05, 
       1.7546829945059612e-05, 
       1.5988943726112353e-05, 
       1.6127142760437654e-05, 
       1.8501286555225882e-05, 
       1.9369100314506194e-05, 
       1.3749988119975567e-05, 
       1.5195083146998539e-05, 
       1.4806830773544685e-05, 
       1.3718401689414570e-05, 
       1.3819554292886459e-05, 
       1.3819554292886459e-05, 
       1.4226198016015102e-05, 
       1.4526254768102927e-05, 
       1.4130402919915216e-05, 
       1.5924384881513964e-05, 
       1.3776920622358407e-05, 
       1.7307937339705117e-05, 
       1.7157118751837243e-05, 
       1.3791515431416588e-05, 
       1.3791169411630304e-05, 
       1.7420417376566657e-05, 
       1.7409352824449136e-05, 
       1.4285093339852193e-05, 
       1.8795587682042484e-05, 
       1.4103654794039135e-05, 
       1.9180107068938916e-05, 
       1.8503274707320360e-05, 
       1.4703893042475059e-05, 
       1.5685763325220564e-05, 
       1.4364479686318036e-05, 
       1.8745947523462314e-05, 
       1.4459917608715566e-05, 
       1.4031358457578322e-05, 
       1.6495701150819696e-05, 
       1.4454621902324786e-05, 
       1.3656166987330204e-05, 
       1.6179948708548639e-05, 
       1.3742596961253355e-05, 
       1.3790596327845639e-05, 
       1.4183318934991936e-05, 
       1.7599213631918401e-05, 
       1.9246636067142738e-05, 
       1.4545046767621585e-05, 
       1.9199568698335498e-05, 
       1.5482489214826254e-05, 
       1.6633405599664425e-05, 
       1.3890861298346911e-05, 
       1.9204840480165535e-05, 
       1.3650788074327285e-05, 
       1.9057870021014847e-05, 
       1.5700088389794672e-05, 
       1.3797255190944416e-05, 
       1.3612537214898020e-05, 
       1.8992765122828241e-05, 
       1.5907063252288016e-05, 
       1.7563224233352980e-05, 
       1.4295373293358697e-05, 
       1.8686881908932990e-05, 
       1.7698050950329329e-05, 
       1.6638831034697928e-05, 
       1.5391637414589236e-05, 
       1.3567521679662418e-05, 
       1.4775426800283805e-05, 
       1.4713707950625158e-05, 
       1.5881304267426527e-05, 
       1.4811723828636502e-05, 
       1.5888800551981285e-05, 
       1.4445333559047477e-05, 
       1.7627097551466300e-05, 
       1.6392602211556929e-05, 
       1.8901538121242530e-05, 
       1.4125446770525143e-05, 
       1.3985992402754039e-05, 
       1.9026115327224947e-05, 
       1.7543671232633482e-05, 
       1.3624994770428607e-05, 
       1.4147596407706047e-05, 
       1.4188435164862451e-05, 
       1.8766800406333077e-05, 
       1.8501831042218248e-05, 
       1.5682141141589220e-05, 
       1.4176620742262519e-05, 
       1.5587248848180789e-05, 
       1.5587248848180789e-05, 
       1.9027498924865004e-05, 
       1.8471267669323802e-05, 
       1.3940539721045514e-05, 
       1.4708561463009807e-05, 
       1.5057309310234542e-05, 
       1.4320397625204029e-05, 
       1.4182636794792536e-05, 
       1.3841723775407177e-05, 
       1.8686062663425051e-05, 
       1.8957179279687248e-05, 
       1.3434160286195692e-05, 
       1.8295928886823704e-05, 
       1.5964674802579159e-05, 
       1.4646932496401925e-05, 
       1.3723941450162334e-05, 
       1.3719201260618938e-05, 
       1.4536406557225851e-05, 
       1.4161199991469600e-05, 
       1.6734960448185846e-05, 
       1.3459860884794278e-05, 
       1.7480723532045456e-05, 
       1.6008297559529445e-05, 
       1.3889197324595794e-05, 
       1.5704066095512283e-05, 
       1.5223195330055947e-05, 
       1.3646413175679166e-05, 
       1.8307490823398830e-05, 
       1.8779267767158738e-05, 
       1.6295973950219000e-05, 
       1.7374349660920962e-05, 
       1.7336397653638667e-05, 
       1.5517788553327095e-05, 
       1.3949937650194191e-05, 
       1.5558319320347971e-05, 
       1.3477655223454708e-05, 
       1.5502386177901978e-05, 
       1.5437012993269314e-05, 
       1.7031093811409717e-05, 
       1.8590234832537693e-05, 
       1.5658159152181815e-05, 
       1.5016023147869611e-05, 
       1.5600915570675881e-05, 
       1.7891282073988843e-05, 
       1.8025899658835437e-05, 
       1.7423977222875363e-05, 
       1.4564904067818549e-05, 
       1.6985230109100869e-05, 
       1.6610169242526198e-05, 
       1.3403574850167767e-05, 
       1.8249590643225570e-05, 
       1.5324571989758253e-05, 
       1.8696725726503051e-05, 
       1.3299975408258749e-05, 
       1.3856856279333476e-05, 
       1.6103098521957558e-05, 
       1.4538139114707123e-05, 
       1.3702158804425321e-05, 
       1.3293422518546922e-05, 
       1.3293422518546922e-05, 
       1.6236107818336177e-05, 
       1.8688175919733982e-05, 
       1.3697525267876854e-05, 
       1.3807723910051766e-05, 
       1.3549227031101011e-05, 
       1.5578267437098484e-05, 
       1.3738672790371903e-05, 
       1.3392045731643397e-05, 
       1.3826092916486223e-05, 
       1.4373870531358810e-05, 
       1.5728116278106659e-05, 
       1.3268800564976914e-05, 
       1.5699211935313036e-05, 
       1.5005045045111907e-05, 
       1.3641239870295874e-05, 
       1.5968800531482134e-05, 
       1.8526216652598709e-05, 
       1.5313220473191225e-05, 
       1.8590595920302667e-05, 
       1.5613121484848878e-05, 
       1.5184033436742415e-05, 
       1.5109188015506222e-05, 
       1.6541342499016076e-05, 
       1.8342411660323198e-05, 
       1.3710210614290806e-05, 
       1.3592628967242167e-05, 
       1.4679805660731030e-05, 
       1.4607901020552364e-05, 
       1.5482167551382916e-05, 
       1.3604362088514964e-05, 
       1.8274378611735281e-05, 
       1.5927268050048197e-05, 
       1.8635562830561640e-05, 
       1.3880921072938089e-05, 
       1.3674217494541382e-05, 
       1.3300799926292706e-05, 
       1.3221374349235393e-05, 
       1.3715731222727339e-05, 
       1.3968892429642264e-05, 
       1.6234740689667079e-05, 
       1.3521752285369215e-05, 
       1.3181599317919874e-05, 
       1.3320017990740753e-05, 
       1.5122535180108063e-05, 
       1.3144870293603119e-05, 
       1.5086394562496400e-05, 
       1.3661037390654270e-05, 
       1.4444117437753411e-05, 
       1.3140819380544898e-05, 
       1.3366097190993894e-05, 
       1.4900164566905099e-05, 
       1.3427494996439749e-05, 
       1.3376517566833076e-05, 
       1.4152889034460591e-05, 
       1.3756462610644304e-05, 
       1.6297301150818062e-05, 
       1.7438216409263375e-05, 
       1.5711475861301436e-05, 
       1.6387592570966102e-05, 
       1.6046470233233955e-05, 
       1.3839515795230890e-05, 
       1.5442873685000349e-05, 
       1.3296253922502915e-05, 
       1.3195862494657522e-05, 
       1.8484162962177201e-05, 
       1.3623021781198347e-05, 
       1.5402607812110785e-05, 
       1.4788063864603584e-05, 
       1.5343935846969094e-05, 
       1.5800692881391843e-05, 
       1.3507008808491725e-05, 
       1.3579031645493009e-05, 
       1.4513522432581221e-05, 
       1.4779965753132000e-05, 
       1.3487598579836920e-05, 
       1.5429379125266887e-05, 
       1.8255826676610677e-05, 
       1.3240974759158147e-05, 
       1.5100633284082102e-05, 
       1.6541643306030848e-05, 
       1.5556387560409246e-05, 
       1.4738715945058147e-05, 
       1.3071202965307305e-05, 
       1.3304028771584416e-05, 
       1.8094333222118067e-05, 
       1.8094333222118067e-05, 
       1.3893211365789147e-05, 
       1.7898810072099596e-05, 
       1.5005171424926672e-05, 
       1.6611643061007290e-05, 
       1.3505242807491324e-05, 
       1.3741418404899087e-05, 
       1.3068773442443617e-05, 
       1.7293629641634060e-05, 
       1.3238875556645516e-05, 
       1.7695620618533069e-05, 
       1.5800871822403320e-05, 
       1.7450532617276401e-05, 
       1.7629295688044348e-05, 
       1.8344278480222551e-05, 
       1.3482464545722240e-05, 
       1.4175949303994331e-05, 
       1.5300242775821022e-05, 
       1.4605786763793261e-05, 
       1.4823175802028637e-05, 
       1.5153074284780484e-05, 
       1.7270806194812235e-05, 
       1.6080133306108396e-05, 
       1.4297887788280208e-05, 
       1.3342300958242572e-05, 
       1.4723920391526192e-05, 
       1.3480818098463213e-05, 
       1.4302739786959192e-05, 
       1.4390758875441822e-05, 
       1.3492151541326705e-05, 
       1.3177085229433623e-05, 
       1.2967378536239500e-05, 
       1.7702284305701121e-05, 
       1.5067600327411624e-05, 
       1.3387627760160553e-05, 
       1.3319111531154694e-05, 
       1.5197873533285913e-05, 
       1.8267001590097889e-05, 
       1.7569159342544463e-05, 
       1.5749766451335516e-05, 
       1.4349329423887490e-05, 
       1.3089622520666603e-05, 
       1.8214848141663575e-05, 
       1.6272081542993111e-05, 
       1.5582308902415184e-05, 
       1.3442712314267953e-05, 
       1.3091425520619221e-05, 
       1.3686534979773073e-05, 
       1.7339594937549733e-05, 
       1.6489623618874409e-05, 
       1.2880757335664411e-05, 
       1.3347189867640593e-05, 
       1.4050696450398433e-05, 
       1.4884455625497334e-05, 
       1.6626873230913024e-05, 
       1.5823735076546908e-05, 
       1.3000147393809456e-05, 
       1.5104512912761696e-05, 
       1.4101850482913153e-05, 
       1.5387481839614083e-05, 
       1.2918909218040337e-05, 
       1.3874800678788756e-05, 
       1.3131491965021635e-05, 
       1.2995579399911916e-05, 
       1.5104833365039862e-05, 
       1.5507514274429569e-05, 
       1.3364894728145192e-05, 
       1.4087188449380208e-05, 
       1.3597640967494783e-05, 
       1.3403023880438764e-05, 
       1.8118521262418251e-05, 
       1.6655942493056282e-05, 
       1.5490964261189846e-05, 
       1.6897403442486508e-05, 
       1.8079928873607369e-05, 
       1.5293086098448552e-05, 
       1.5137324166602677e-05, 
       1.2806963570890892e-05, 
       1.4584372774774720e-05, 
       1.4584323137432891e-05, 
       1.3336479988577629e-05, 
       1.8062443265322889e-05, 
       1.5911658893412627e-05, 
       1.4721689727392325e-05, 
       1.3228335561013241e-05, 
       1.6843793423740440e-05, 
       1.5016356582037923e-05, 
       1.7091135092454502e-05, 
       1.3151659632130112e-05, 
       1.7539273363188547e-05, 
       1.7557235930031233e-05, 
       1.3589754140539485e-05, 
       1.8025329576105984e-05, 
       1.4870995520110517e-05, 
       1.7870923326151566e-05, 
       1.5175791519919189e-05, 
       1.7594851726460161e-05, 
       1.3176151108582834e-05, 
       1.2880414496024289e-05, 
       1.3160649662418772e-05, 
       1.7167559616731058e-05, 
       1.2811074572457455e-05, 
       1.4955141120491437e-05, 
       1.2814387999203995e-05, 
       1.4899014168377159e-05, 
       1.4908026788281043e-05, 
       1.3850128959065888e-05, 
       1.2796271078368136e-05, 
       1.7686427184695396e-05, 
       1.2794038523273086e-05, 
       1.2735607351813803e-05, 
       1.5568858998512495e-05, 
       1.3554006583698968e-05, 
       1.2894425123708972e-05, 
       1.4480766156028099e-05, 
       1.3480737178010092e-05, 
       1.4473303739024755e-05, 
       1.5908892149985761e-05, 
       1.2745537453819999e-05, 
       1.7511545949478018e-05, 
       1.6997748985273308e-05, 
       1.2696258063591309e-05, 
       1.3553738252085991e-05, 
       1.3385560597741876e-05, 
       1.5417264670316735e-05, 
       1.2847939832424163e-05, 
       1.7618113086773479e-05, 
       1.2775045048953953e-05, 
       1.7280640997730996e-05, 
       1.4071209168825452e-05, 
       1.5934573699874957e-05, 
       1.4499961885680836e-05, 
       1.7420380509283671e-05, 
       1.7390979251685042e-05, 
       1.7896046931245846e-05, 
       1.5272640197153067e-05, 
       1.3143483145613326e-05, 
       1.6055450182000986e-05, 
       1.3029096990884418e-05, 
       1.4737733676853335e-05, 
       1.2853346621457406e-05, 
       1.6888833098419228e-05, 
       1.7266837862881601e-05, 
       1.4880176564453315e-05, 
       1.3303578245791464e-05, 
       1.4389006253772257e-05, 
       1.4581306955086769e-05, 
       1.3324878637255527e-05, 
       1.4586962968713603e-05, 
       1.5439188167706442e-05, 
       1.7628540369685631e-05, 
       1.2710953800351990e-05, 
       1.2722287998991867e-05, 
       1.3764484331250662e-05, 
       1.7754836156249789e-05, 
       1.4340533715801329e-05, 
       1.2709142458374608e-05, 
       1.7136753978475371e-05, 
       1.4771980148076379e-05, 
       1.7668925127910735e-05, 
       1.6873946144221132e-05, 
       1.7178883635342540e-05, 
       1.3540177879095211e-05, 
       1.3540177879095211e-05, 
       1.7726555941484860e-05, 
       1.7126093323873628e-05, 
       1.7391291245074722e-05, 
       1.6897490550629808e-05, 
       1.2626559322756644e-05, 
       1.7427788090370450e-05, 
       1.4313104514457679e-05, 
       1.6979634636265375e-05, 
       1.2540855306414584e-05, 
       1.7710141673424804e-05, 
       1.4490389970816197e-05, 
       1.3391202413501763e-05, 
       1.3391202413501763e-05, 
       1.3036983750738302e-05, 
       1.7698937267527312e-05, 
       1.3027780304667242e-05, 
       1.7060837186240233e-05, 
       1.3283256794748671e-05, 
       1.4307722125350473e-05, 
       1.7466787973707888e-05, 
       1.4854005793521301e-05, 
       1.5661389093368380e-05, 
       1.4684027875682477e-05, 
       1.3858576012762380e-05, 
       1.6340392098367192e-05, 
       1.3085781450145574e-05, 
       1.2502787160953376e-05, 
       1.3488634947007141e-05, 
       1.3767192242706289e-05, 
       1.2820878434160408e-05, 
       1.3798296019634512e-05, 
       1.2665232093165140e-05, 
       1.5915052412978616e-05, 
       1.3399827582885484e-05, 
       1.6680469784689776e-05, 
       1.3643378838659664e-05, 
       1.3279481386164379e-05, 
       1.7504207846230492e-05, 
       1.5593840334213613e-05, 
       1.4508967471616001e-05, 
       1.4261633455839362e-05, 
       1.4260526372031217e-05, 
       1.6095184673308086e-05, 
       1.4567329784929392e-05, 
       1.2916034349897811e-05, 
       1.4102494158456044e-05, 
       1.4580830373188505e-05, 
       1.4360851731491847e-05, 
       1.6316408455780562e-05, 
       1.4318243849690795e-05, 
       1.3585143566485153e-05, 
       1.4348745972368084e-05, 
       1.2492627150387478e-05, 
       1.2442304049548416e-05, 
       1.4088979986723125e-05, 
       1.6775660393623092e-05, 
       1.2428959037144436e-05, 
       1.5875801930255483e-05, 
       1.5314968384048395e-05, 
       1.2386938181135311e-05, 
       1.4670980613850224e-05, 
       1.2389523137184474e-05, 
       1.2524192672057032e-05, 
       1.2494490006412363e-05, 
       1.4503628483898586e-05, 
       1.3562786969037975e-05, 
       1.4521787795138020e-05, 
       1.4435448392455718e-05, 
       1.2626414392450376e-05, 
       1.2469257022665821e-05, 
       1.4163047639485972e-05, 
       1.4203377880169526e-05, 
       1.7446308986520677e-05, 
       1.4987944051186263e-05, 
       1.3621729740483568e-05, 
       1.6960153082689459e-05, 
       1.2948875232581966e-05, 
       1.2545997984448476e-05, 
       1.4391555603622707e-05, 
       1.2691110441516361e-05, 
       1.3855844707227722e-05, 
       1.7359384566316086e-05, 
       1.6898861535444388e-05, 
       1.3453718136021118e-05, 
       1.4375558402724365e-05, 
       1.7403224908037797e-05, 
       1.6953829584309808e-05, 
       1.2870965835069754e-05, 
       1.3943837491000413e-05, 
       1.2344108032617478e-05, 
       1.2386069398627815e-05, 
       1.2484532124767221e-05, 
       1.4589910278901068e-05, 
       1.3010696110014875e-05, 
       1.7245742834484203e-05, 
       1.7245445108196030e-05, 
       1.4034855803560286e-05, 
       1.3822854927071606e-05, 
       1.2976603896862156e-05, 
       1.2471959762714729e-05, 
       1.2471959762714729e-05, 
       1.2335305587210792e-05, 
       1.3553726418031432e-05, 
       1.6800115964483792e-05, 
       1.2309173331743173e-05, 
       1.3537955235122096e-05, 
       1.2961072457293526e-05, 
       1.2710964121726577e-05, 
       1.2657021703406087e-05, 
       1.4325437482851584e-05, 
       1.7187753713868272e-05, 
       1.2607809372930020e-05, 
       1.2607809372930020e-05, 
       1.6788749265047764e-05, 
       1.6862076102243585e-05, 
       1.2278088143277375e-05, 
       1.2553654088597697e-05, 
       1.3581783324189881e-05, 
       1.3500002929733665e-05, 
       1.4809308484121882e-05, 
       1.5777197720231638e-05, 
       1.2442416536710257e-05, 
       1.2789808865151445e-05, 
       1.3138967632996221e-05, 
       1.2437337981554402e-05, 
       1.2270316966538834e-05, 
       1.6725231655122100e-05, 
       1.2768120295959958e-05, 
       1.4138259865022317e-05, 
       1.5454387441707252e-05, 
       1.3490975182646470e-05, 
       1.2294224671848139e-05, 
       1.6473024040458836e-05, 
       1.2498439961805670e-05, 
       1.3320805486952995e-05, 
       1.4159497493630926e-05, 
       1.3981275923430545e-05, 
       1.4557278735090200e-05, 
       1.7193357221498349e-05, 
       1.2299877382389635e-05, 
       1.5083418308035586e-05, 
       1.7155443087777133e-05, 
       1.3998765574030313e-05, 
       1.5156406202640975e-05, 
       1.5071096782752692e-05, 
       1.6999965577393064e-05, 
       1.2283926466294301e-05, 
       1.4260414705583137e-05, 
       1.2475899247148247e-05, 
       1.4481181432509998e-05, 
       1.6963051812999197e-05, 
       1.6693967239233020e-05, 
       1.6895933397921275e-05, 
       1.7067244968282568e-05, 
       1.6511609766917350e-05, 
       1.2342882256042360e-05, 
       1.2503447975852459e-05, 
       1.4837053480320502e-05, 
       1.3550338030219951e-05, 
       1.6424553768599611e-05, 
       1.4034811378824999e-05, 
       1.4572930529791567e-05, 
       1.5660814224652469e-05, 
       1.2838552643104507e-05, 
       1.4634370473094196e-05, 
       1.4917773651318327e-05, 
       1.5098922607870378e-05, 
       1.2112673093226221e-05, 
       1.2498062274450194e-05, 
       1.2799920476892023e-05, 
       1.7097475884677428e-05, 
       1.3629537207041920e-05, 
       1.3886607702411817e-05, 
       1.4607637107221816e-05, 
       1.2381988056314457e-05, 
       1.2932352729534196e-05, 
       1.4382446488281116e-05, 
       1.6828654080691419e-05, 
       1.6906542095612401e-05, 
       1.4061496842924871e-05, 
       1.6927157243742016e-05, 
       1.6962254898130914e-05, 
       1.7040178422091086e-05, 
       1.3217043253878353e-05, 
       1.2201799911787035e-05, 
       1.3004104001976357e-05, 
       1.4141689479707442e-05, 
       1.3879107047421109e-05, 
       1.6439198247525448e-05, 
       1.2051153145332652e-05, 
       1.2839523566517898e-05, 
       1.6885421914800625e-05, 
       1.3679089576825923e-05, 
       1.2394913505990004e-05, 
       1.6343323208637191e-05, 
       1.5609889988095738e-05, 
       1.6060123324295829e-05, 
       1.2951751462282618e-05, 
       1.6548645289356341e-05, 
       1.2267481770772798e-05, 
       1.6521556092748513e-05, 
       1.2062890661016386e-05, 
       1.5703848225067494e-05, 
       1.4793501401871683e-05, 
       1.2151098340172989e-05, 
       1.2369469662254540e-05, 
       1.3846599821602117e-05, 
       1.2283923578237718e-05, 
       1.3246559181469745e-05, 
       1.6515589836570601e-05, 
       1.6762720462200617e-05, 
       1.2331047004902269e-05, 
       1.5177281524227594e-05, 
       1.3681703599686345e-05, 
       1.6230305120936819e-05, 
       1.3591409024688596e-05, 
       1.6485934940041834e-05, 
       1.2284074227182152e-05, 
       1.2956461204339326e-05, 
       1.2152194763912620e-05, 
       1.6773307292160054e-05, 
       1.1967615650054214e-05, 
       1.1975505905783033e-05, 
       1.2328354550964664e-05, 
       1.6402023829457311e-05, 
       1.2017318192492095e-05, 
       1.4875824108569669e-05, 
       1.4411062269783630e-05, 
       1.4032785219502711e-05, 
       1.6677528590307853e-05, 
       1.3418565222523246e-05, 
       1.2162172897560513e-05, 
       1.4939058389778436e-05, 
       1.2638112356050480e-05, 
       1.6707024854103104e-05, 
       1.6074313361273201e-05, 
       1.3742617783383362e-05, 
       1.5409510500739345e-05, 
       1.5539092049979890e-05, 
       1.2143997447308365e-05, 
       1.2923700438071394e-05, 
       1.2841981824842158e-05, 
       1.5271024521834124e-05, 
       1.6746748218237813e-05, 
       1.1909602730243198e-05, 
       1.3542362766701390e-05, 
       1.3945594853167781e-05, 
       1.6591552297043637e-05, 
       1.3901966460264854e-05, 
       1.2552482636613237e-05, 
       1.1855662649301542e-05, 
       1.6511380287452582e-05, 
       1.6736024251672289e-05, 
       1.1845511249883099e-05, 
       1.2772454401685757e-05, 
       1.6444859774017816e-05, 
       1.1837078722907510e-05, 
       1.6331190648800634e-05, 
       1.4480976230554762e-05, 
       1.4650396000285066e-05, 
       1.4216187792345135e-05, 
       1.3088038476606967e-05, 
       1.3337286452725844e-05, 
       1.2523678926316022e-05, 
       1.4782461139809140e-05, 
       1.1972800210400306e-05, 
       1.2519711528858055e-05, 
       1.6125081896199361e-05, 
       1.6713298992144534e-05, 
       1.6678130045148253e-05, 
       1.6490733408937781e-05, 
       1.4426810859328770e-05, 
       1.2058447012652167e-05, 
       1.4886464507297263e-05, 
       1.2977124969328830e-05, 
       1.3923329897107152e-05, 
       1.2455497239420550e-05, 
       1.1873359680281745e-05, 
       1.6639364238063028e-05, 
       1.1801165510882129e-05, 
       1.1801165510882129e-05, 
       1.2537315706778508e-05, 
       1.2463219717063673e-05, 
       1.5454711096299853e-05, 
       1.3745579890106580e-05, 
       1.1773370292365622e-05, 
       1.6533277169563514e-05, 
       1.4689900333100438e-05, 
       1.1992446244021064e-05, 
       1.6610250634084828e-05, 
       1.1838087415967461e-05, 
       1.1860953182291961e-05, 
       1.6619005424077404e-05, 
       1.2029624982065926e-05, 
       1.4944882191809471e-05, 
       1.2652977391415335e-05, 
       1.6540315396306796e-05, 
       1.6036506868789826e-05, 
       1.3779211802409204e-05, 
       1.3759738104080438e-05, 
       1.3783836622326562e-05, 
       1.1755704371001792e-05, 
       1.3756888755834545e-05, 
       1.1740946677724562e-05, 
       1.1740946677724562e-05, 
       1.2509243542771845e-05, 
       1.2475709864121511e-05, 
       1.3573263252305218e-05, 
       1.6539875014811285e-05, 
       1.1698455754107560e-05, 
       1.3071390328927385e-05, 
       1.2255883138375145e-05, 
       1.1961040627349164e-05, 
       1.4026488168279134e-05, 
       1.4656488798993812e-05, 
       1.3576977009018623e-05, 
       1.6494185236488781e-05, 
       1.6317113509340670e-05, 
       1.3699964069023062e-05, 
       1.1677002613866072e-05, 
       1.3279018477965065e-05, 
       1.3497296118562936e-05, 
       1.4413941486804807e-05, 
       1.2120783480670440e-05, 
       1.3482265122977599e-05, 
       1.3687673948764869e-05, 
       1.4011768348804389e-05, 
       1.3263212176298742e-05, 
       1.2844775986296911e-05, 
       1.2409422598320708e-05, 
       1.2021216942585990e-05, 
       1.2768309311563218e-05, 
       1.2108460891919024e-05, 
       1.6243017234788246e-05, 
       1.4715629585181497e-05, 
       1.2055050537749066e-05, 
       1.5269784078196127e-05, 
       1.1626108412671910e-05, 
       1.5427267837226295e-05, 
       1.3688472411888766e-05, 
       1.4265879036331624e-05, 
       1.6280060006956205e-05, 
       1.1763423671016568e-05, 
       1.3063003799737662e-05, 
       1.1870438638559717e-05, 
       1.2173091482338591e-05, 
       1.1583697085280687e-05, 
       1.1887000154819520e-05, 
       1.6363027769810196e-05, 
       1.6043844214483004e-05, 
       1.1692453485639220e-05, 
       1.2129223522047978e-05, 
       1.3914863308321251e-05, 
       1.3657251234300196e-05, 
       1.3016875725231145e-05, 
       1.3575300025194662e-05, 
       1.3774201644007277e-05, 
       1.2090086248608373e-05, 
       1.6333337471457206e-05, 
       1.4568035462459632e-05, 
       1.1599793775919464e-05, 
       1.5762249770742242e-05, 
       1.1893687494150015e-05, 
       1.2530250343329092e-05, 
       1.3219392035726646e-05, 
       1.2968057635695589e-05, 
       1.2481291080850444e-05, 
       1.5850445928099265e-05, 
       1.1522415149673283e-05, 
       1.2969194686459368e-05, 
       1.2008472306768569e-05, 
       1.1548404921534808e-05, 
       1.2509030139655461e-05, 
       1.1987443003169530e-05, 
       1.2232579529806520e-05, 
       1.4134462807824190e-05, 
       1.3594471423609121e-05, 
       1.5829638151601610e-05, 
       1.1708680329747450e-05, 
       1.3999562549330479e-05, 
       1.3175816829170453e-05, 
       1.1488743339510251e-05, 
       1.2451638480871150e-05, 
       1.1487381860889677e-05, 
       1.2064187339489767e-05, 
       1.1476824243579039e-05, 
       1.1812292511541923e-05, 
       1.2429670499840731e-05, 
       1.5523842376759259e-05, 
       1.4355399612868856e-05, 
       1.1476562269216375e-05, 
       1.4884644722930229e-05, 
       1.1796323039204514e-05, 
       1.5919041856879116e-05, 
       1.3972315656778566e-05, 
       1.1781164840143850e-05, 
       1.1523276164360502e-05, 
       1.1453449992394640e-05, 
       1.3402932809675038e-05, 
       1.3469967678943343e-05, 
       1.3558421750825662e-05, 
       1.5751511430091618e-05, 
       1.6153692887444126e-05, 
       1.3567868841980870e-05, 
       1.5990651316969160e-05, 
       1.2505190789439619e-05, 
       1.4191315159746284e-05, 
       1.2763551207767700e-05, 
       1.3139022883137724e-05, 
       1.1837371971164738e-05, 
       1.4554252916811563e-05, 
       1.3854076524022812e-05, 
       1.2861023192041890e-05, 
       1.1626325100487196e-05, 
       1.1587610802613169e-05, 
       1.1404359297156563e-05, 
       1.5342921659673172e-05, 
       1.2807204217316127e-05, 
       1.1544730960155244e-05, 
       1.5099362504310848e-05, 
       1.6087621791069557e-05, 
       1.5476310540145731e-05, 
       1.3394308699124815e-05, 
       1.2784533483894596e-05, 
       1.3325423239195278e-05, 
       1.1379673462709054e-05, 
       1.2313159437453939e-05, 
       1.2276319429446716e-05, 
       1.3280966122500558e-05, 
       1.6036054550874546e-05, 
       1.1607868526682517e-05, 
       1.1591400444480794e-05, 
       1.1778406704184160e-05, 
       1.3336154921564371e-05, 
       1.3315229438692028e-05, 
       1.4764980571177641e-05, 
       1.4764980571177641e-05, 
       1.3005731369938051e-05, 
       1.3857468221593714e-05, 
       1.2601611945198697e-05, 
       1.1535642234712601e-05, 
       1.4385838370302758e-05, 
       1.1431409072020787e-05, 
       1.5546432469003097e-05, 
       1.1819478940763967e-05, 
       1.1594796087635752e-05, 
       1.4376384604886602e-05, 
       1.4699854886166361e-05, 
       1.4699854886166361e-05, 
       1.4396400266741322e-05, 
       1.4855497795269528e-05, 
       1.2704668369661561e-05, 
       1.1640622584927458e-05, 
       1.4829960101301243e-05, 
       1.3367905391044500e-05, 
       1.5730339965465210e-05, 
       1.3325732032530396e-05, 
       1.5910499795239197e-05, 
       1.1706431528776467e-05, 
       1.1793640490663601e-05, 
       1.1789457250824178e-05, 
       1.1370970085676693e-05, 
       1.5908326901209936e-05, 
       1.3236524848963257e-05, 
       1.3663415886131263e-05, 
       1.1504976578743271e-05, 
       1.1543923892534094e-05, 
       1.1279697603442828e-05, 
       1.1838538878837953e-05, 
       1.1987127670512582e-05, 
       1.1296766269345427e-05, 
       1.2959114971534154e-05, 
       1.3218124520169283e-05, 
       1.2941663212295570e-05, 
       1.4731205970450673e-05, 
       1.1695320843535913e-05, 
       1.1516129984297710e-05, 
       1.5804377992296439e-05, 
       1.2627139401248321e-05, 
       1.1255438195063090e-05, 
       1.5725219389215415e-05, 
       1.2610794816444050e-05, 
       1.1291185821420846e-05, 
       1.1347458430815556e-05, 
       1.4046129526921711e-05, 
       1.1195658159874779e-05, 
       1.3454085532832275e-05, 
       1.1381419963787335e-05, 
       1.1406896544225626e-05, 
       1.1294532890678640e-05, 
       1.1762875123448269e-05, 
       1.1331706585394882e-05, 
       1.5093834594196556e-05, 
       1.4739012246260433e-05, 
       1.3130911919431970e-05, 
       1.2776219510881027e-05, 
       1.1526377574446102e-05, 
       1.1991309522357288e-05, 
       1.2226211083826836e-05, 
       1.5175221405291258e-05, 
       1.1409237087040052e-05, 
       1.4784229181950738e-05, 
       1.2554914194232347e-05, 
       1.3560759521860001e-05, 
       1.3533736027604757e-05, 
       1.3105796811171029e-05, 
       1.2416762654598721e-05, 
       1.3070588015784730e-05, 
       1.4784480416251641e-05, 
       1.2084829361035023e-05, 
       1.1251724539764732e-05, 
       1.1174224509154538e-05, 
       1.1112371076041054e-05, 
       1.1520115843759457e-05, 
       1.5334086415174013e-05, 
       1.1160672547446487e-05, 
       1.4451302495466187e-05, 
       1.1789498666796832e-05, 
       1.1699591699811319e-05, 
       1.5614724789498533e-05, 
       1.4595757897294223e-05, 
       1.2463651078146501e-05, 
       1.1155286087400586e-05, 
       1.1357403717846669e-05, 
       1.5065512276052079e-05, 
       1.3058121209538081e-05, 
       1.5409553591095302e-05, 
       1.4022228235795803e-05, 
       1.3254078202592438e-05, 
       1.1590497891745144e-05, 
       1.5473139660636038e-05, 
       1.3330167724947918e-05, 
       1.4871794157426763e-05, 
       1.3128682627561413e-05, 
       1.2819211413794908e-05, 
       1.2971139573058469e-05, 
       1.1577470508168492e-05, 
       1.5610416448607364e-05, 
       1.5215733471324159e-05, 
       1.1957110588090610e-05, 
       1.1104124698394748e-05, 
       1.4431928381036290e-05, 
       1.2896102274761090e-05, 
       1.3097337808655840e-05, 
       1.1825895801799263e-05, 
       1.1524661511334883e-05, 
       1.2409409129774765e-05, 
       1.2159701957895705e-05, 
       1.1034743965615807e-05, 
       1.3915233306724783e-05, 
       1.1343597954337286e-05, 
       1.4914483200639213e-05, 
       1.5210500243942558e-05, 
       1.5260298435165760e-05, 
       1.3171765115161997e-05, 
       1.3047338974395501e-05, 
       1.4882816353497803e-05, 
       1.1059404205545469e-05, 
       1.4936865174490363e-05, 
       1.2766985023063518e-05, 
       1.2893925531166015e-05, 
       1.2368786192915663e-05, 
       1.3219500897702974e-05, 
       1.1002599954937517e-05, 
       1.1020868212417563e-05, 
       1.4199405601560050e-05, 
       1.3364992036319819e-05, 
       1.5435136575897798e-05, 
       1.1650928381232596e-05, 
       1.4048953285959872e-05, 
       1.5331984029457066e-05, 
       1.3165083799601319e-05, 
       1.5427938157995699e-05, 
       1.5249798292015534e-05, 
       1.1271908252755239e-05, 
       1.2875663488557887e-05, 
       1.5096693194239976e-05, 
       1.3142013420892148e-05, 
       1.3142013420892148e-05, 
       1.3613220654626027e-05, 
       1.5162725173979664e-05, 
       1.0960232019262796e-05, 
       1.3970312000698586e-05, 
       1.2720786844480341e-05, 
       1.2439576800744272e-05, 
       1.5434167899305453e-05, 
       1.1334595558805153e-05, 
       1.3130836031856248e-05, 
       1.5437485254243910e-05, 
       1.5252806126741022e-05, 
       1.2679806263590631e-05, 
       1.1051149529517730e-05, 
       1.5167607600879590e-05, 
       1.1362365442106242e-05, 
       1.2789142817727944e-05, 
       1.1307932390956171e-05, 
       1.1890698855806812e-05, 
       1.1376044665726076e-05, 
       1.0940102390403214e-05, 
       1.1581938286855885e-05, 
       1.0911917563619211e-05, 
       1.2829347364747459e-05, 
       1.1777629661435843e-05, 
       1.2944615431237900e-05, 
       1.1487073909389023e-05, 
       1.4917167854124414e-05, 
       1.5293161765525723e-05, 
       1.3209788303320664e-05, 
       1.5036999696857796e-05, 
       1.1197201014953809e-05, 
       1.4741172951403030e-05, 
       1.0880112711177723e-05, 
       1.4751233477104657e-05, 
       1.4416314356188729e-05, 
       1.0911607764474247e-05, 
       1.2114268406527672e-05, 
       1.5079605106049488e-05, 
       1.3601432200708229e-05, 
       1.1228971459185498e-05, 
       1.2602104378764006e-05, 
       1.2119505435735810e-05, 
       1.3135486265618616e-05, 
       1.3233213652183534e-05, 
       1.1356087833350652e-05, 
       1.4385570413798744e-05, 
       1.0880119892811605e-05, 
       1.3062107178395968e-05, 
       1.2211645162798186e-05, 
       1.1765816781040863e-05, 
       1.1001408024351938e-05, 
       1.2694437425427017e-05, 
       1.1018586978386502e-05, 
       1.2132665622803684e-05, 
       1.1071953261747843e-05, 
       1.4193513743233301e-05, 
       1.2511519833986496e-05, 
       1.4037535283825726e-05, 
       1.3660777921279991e-05, 
       1.5260512510540615e-05, 
       1.5012322681784472e-05, 
       1.5248361112992795e-05, 
       1.1641820369659849e-05, 
       1.2632827152893666e-05, 
       1.4907534823259538e-05, 
       1.5239692329646160e-05, 
       1.2777745489921347e-05, 
       1.2676180169540794e-05, 
       1.3510423823284396e-05, 
       1.0795487128499497e-05, 
       1.4488596246222101e-05, 
       1.4950049701366523e-05, 
       1.2202229493805757e-05, 
       1.2026293789691191e-05, 
       1.2982750774007872e-05, 
       1.1767880451418330e-05, 
       1.4220651019447639e-05, 
       1.0734476731730870e-05, 
       1.1357915378189778e-05, 
       1.0913645623177548e-05, 
       1.0960413680839423e-05, 
       1.2499788879032851e-05, 
       1.3380111343443414e-05, 
       1.2446109729477853e-05, 
       1.2827333352772899e-05, 
       1.1003822470070702e-05, 
       1.0728759686802338e-05, 
       1.0917018151866301e-05, 
       1.1067963392579116e-05, 
       1.0726998344004861e-05, 
       1.1040759581333051e-05, 
       1.2691379618478009e-05, 
       1.0959669581733495e-05, 
       1.1067319579787656e-05, 
       1.1377896894289661e-05, 
       1.2510633106626702e-05, 
       1.0891091901650885e-05, 
       1.0777584065949914e-05, 
       1.1324054378445322e-05, 
       1.1115020312170831e-05, 
       1.3497819104780639e-05, 
       1.1176701791142629e-05, 
       1.0734103838765258e-05, 
       1.3188035917466688e-05, 
       1.3800546246854002e-05, 
       1.2003818416916015e-05, 
       1.1241948336015302e-05, 
       1.2935692445501080e-05, 
       1.4610151409723593e-05, 
       1.2259661994790551e-05, 
       1.0912663366606487e-05, 
       1.2011859699603323e-05, 
       1.1233586332889624e-05, 
       1.5046559987525149e-05, 
       1.1173076007958938e-05, 
       1.1017203891298821e-05, 
       1.3129116666707593e-05, 
       1.1795499610887464e-05, 
       1.4458924151673350e-05, 
       1.4391690213843748e-05, 
       1.0946392933647906e-05, 
       1.0753260275564165e-05, 
       1.1429648995195049e-05, 
       1.0939625745705024e-05, 
       1.1742309149567105e-05, 
       1.3417643480241299e-05, 
       1.1740272789485190e-05, 
       1.2455077712833444e-05, 
       1.4082933638771194e-05, 
       1.4812848580092030e-05, 
       1.0636484163392076e-05, 
       1.2740138849976161e-05, 
       1.1080299397763669e-05, 
       1.4729875165272834e-05, 
       1.4652926064351335e-05, 
       1.2388452190993172e-05, 
       1.1609745189482309e-05, 
       1.4451440761200383e-05, 
       1.2437822170348280e-05, 
       1.0783453965436617e-05, 
       1.1158019940170399e-05, 
       1.4870788033346072e-05, 
       1.2472960200562931e-05, 
       1.0681904699412976e-05, 
       1.0615868154871845e-05, 
       1.2415244490356536e-05, 
       1.1109042099266781e-05, 
       1.1637484604806696e-05, 
       1.0619503241551794e-05, 
       1.0993848273884750e-05, 
       1.2091127701719708e-05, 
       1.0706959311734820e-05, 
       1.3415406696699663e-05, 
       1.3904188254785544e-05, 
       1.0565616680907563e-05, 
       1.0955434589074079e-05, 
       1.4817778274625170e-05, 
       1.3811722608146129e-05, 
       1.0648152663130800e-05, 
       1.2390394217438649e-05, 
       1.0974698395040515e-05, 
       1.3838727097500063e-05, 
       1.1061380224088071e-05, 
       1.4232833043570118e-05, 
       1.0649694828788931e-05, 
       1.0539059100085208e-05, 
       1.2329686298436526e-05, 
       1.0606533385873046e-05, 
       1.4864113112570085e-05, 
       1.1998992969886485e-05, 
       1.4751578226387234e-05, 
       1.3336164308754586e-05, 
       1.3567746747392274e-05, 
       1.1114539414726377e-05, 
       1.0598005793930420e-05, 
       1.0551765433219797e-05, 
       1.4616861103236607e-05, 
       1.1461448699036052e-05, 
       1.0977041683487981e-05, 
       1.3651794554066623e-05, 
       1.0715279091292816e-05, 
       1.1222718580944904e-05, 
       1.2222388883919755e-05, 
       1.3799930584381251e-05, 
       1.0544488217613670e-05, 
       1.4685799775340629e-05, 
       1.4816433203071156e-05, 
       1.2252178183499918e-05, 
       1.2847939832422902e-05, 
       1.0543616110196778e-05, 
       1.0487409265946741e-05, 
       1.1034187197077173e-05, 
       1.0550917307512756e-05, 
       1.0552877443664410e-05, 
       1.0895634494892817e-05, 
       1.1295251606471709e-05, 
       1.0658665803181129e-05, 
       1.4108681308786072e-05, 
       1.1034464564272159e-05, 
       1.2046478417446544e-05, 
       1.2712744957966272e-05, 
       1.4191561439457718e-05, 
       1.2066068276229354e-05, 
       1.0540008854222513e-05, 
       1.0599839928229458e-05, 
       1.1235251149060995e-05, 
       1.4753906265802196e-05, 
       1.0480684677724653e-05, 
       1.0438530216473421e-05, 
       1.1047981100137697e-05, 
       1.0998185193224648e-05, 
       1.1432558062255359e-05, 
       1.0527554364672971e-05, 
       1.0905019411988960e-05, 
       1.1242607667190877e-05, 
       1.2828517394046592e-05, 
       1.1180595006706165e-05, 
       1.1180595006706165e-05, 
       1.4185914684124900e-05, 
       1.4715196328684755e-05, 
       1.0919893747567160e-05, 
       1.1297780666473095e-05, 
       1.2194675089853316e-05, 
       1.1350707639988095e-05, 
       1.2300774000897797e-05, 
       1.1271898793708609e-05, 
       1.1117924482975310e-05, 
       1.1196864256343460e-05, 
       1.2439203825190964e-05, 
       1.2827436894527132e-05, 
       1.2750023771996639e-05, 
       1.3583431501139882e-05, 
       1.0853800750461916e-05, 
       1.1203824109936732e-05, 
       1.2206548479858695e-05, 
       1.1476730178424250e-05, 
       1.4658722913049990e-05, 
       1.4638907142435021e-05, 
       1.3207874939737303e-05, 
       1.0947980011342474e-05, 
       1.0610004698907262e-05, 
       1.3632779740319906e-05, 
       1.0662404488644941e-05, 
       1.2970575774498610e-05, 
       1.1161405211005428e-05, 
       1.0381314204025958e-05, 
       1.2217831820003956e-05, 
       1.2157343488940657e-05, 
       1.0369325143007269e-05, 
       1.2057404112405401e-05, 
       1.0915105104844207e-05, 
       1.2096523335024780e-05, 
       1.0929738402608463e-05, 
       1.0913093511718217e-05, 
       1.1180972623180034e-05, 
       1.1642993037587542e-05, 
       1.2859409430803178e-05, 
       1.3448184593848241e-05, 
       1.4381898891484474e-05, 
       1.1114956795116073e-05, 
       1.1141125678232446e-05, 
       1.2906384145128930e-05, 
       1.0991389553506523e-05, 
       1.0420575666642764e-05, 
       1.2322022966211623e-05, 
       1.0728496236894351e-05, 
       1.1713343965105547e-05, 
       1.1535512222470310e-05, 
       1.2183752692252095e-05, 
       1.1780215350614559e-05, 
       1.2521885328123999e-05, 
       1.0563194018715709e-05, 
       1.0479737161371500e-05, 
       1.0529089926756773e-05, 
       1.0870118654929953e-05, 
       1.0645383676090020e-05, 
       1.1022204237005723e-05, 
       1.1580693087682408e-05, 
       1.3078910958596138e-05, 
       1.3224450467597730e-05, 
       1.2093590680975748e-05, 
       1.0529798597256065e-05, 
       1.3950605783330427e-05, 
       1.0405155291295776e-05, 
       1.4532356645709372e-05, 
       1.1557887317785126e-05, 
       1.4331264654138884e-05, 
       1.3170552423658752e-05, 
       1.0560604063197804e-05, 
       1.2914511009516920e-05, 
       1.4463540925630009e-05, 
       1.4493238831829933e-05, 
       1.1262964441190372e-05, 
       1.1217443702188182e-05, 
       1.0485478705638342e-05, 
       1.0822361034888643e-05, 
       1.0677977066657689e-05, 
       1.3789704872286703e-05, 
       1.3128482137200564e-05, 
       1.3128482137200564e-05, 
       1.1300788157987383e-05, 
       1.0743462033899452e-05, 
       1.4113195084363403e-05, 
       1.1015177595948874e-05, 
       1.1240848953262118e-05, 
       1.4220598988835272e-05, 
       1.0251693494309371e-05, 
       1.0356942271676616e-05, 
       1.0310817643193292e-05, 
       1.0819757982859366e-05, 
       1.1181222095608433e-05, 
       1.1086519442023879e-05, 
       1.0405047370049182e-05, 
       1.3106964823418116e-05, 
       1.1648237857761962e-05, 
       1.1882114562741186e-05, 
       1.4212376331393727e-05, 
       1.2748597903242481e-05, 
       1.4062700807589073e-05, 
       1.4172316877647646e-05, 
       1.4192643872286560e-05, 
       1.0343460932099921e-05, 
       1.2836567213794866e-05, 
       1.3493684316348202e-05, 
       1.4095274188926477e-05, 
       1.0217351885321709e-05, 
       1.1887388815049288e-05, 
       1.0227999987175334e-05, 
       1.2016387655506428e-05, 
       1.2971797911614471e-05, 
       1.3445474046718865e-05, 
       1.2079935079262819e-05, 
       1.0819976918595336e-05, 
       1.1219142276377080e-05, 
       1.1396223920582028e-05, 
       1.3571867867413760e-05, 
       1.0440074451155300e-05, 
       1.1975378689078452e-05, 
       1.0524580143455178e-05, 
       1.1980279389708748e-05, 
       1.0269156897812358e-05, 
       1.1970288204529031e-05, 
       1.2576160972068977e-05, 
       1.1990423566327884e-05, 
       1.2179815631313550e-05, 
       1.0244582907604472e-05, 
       1.1918670759452220e-05, 
       1.0440668415209756e-05, 
       1.2457996718812353e-05, 
       1.4252083669744673e-05, 
       1.4257714289259674e-05, 
       1.0232147136043175e-05, 
       1.0362532560831333e-05, 
       1.2616749358082860e-05, 
       1.2217133877581721e-05, 
       1.0942241347660040e-05, 
       1.1995377579496135e-05, 
       1.3937639753039255e-05, 
       1.2052315192076341e-05, 
       1.0657909308048463e-05, 
       1.0984380242742437e-05, 
       1.1468399116738412e-05, 
       1.2672634348090212e-05, 
       1.4262014406743227e-05, 
       1.0702897124077525e-05, 
       1.2473674947314517e-05, 
       1.1980681223105419e-05, 
       1.2762042933529212e-05, 
       1.0246696592659159e-05, 
       1.1062006358732409e-05, 
       1.0541362827602487e-05, 
       1.1761721137424494e-05, 
       1.0193337481050889e-05, 
       1.4245773551838934e-05, 
       1.1121978731039584e-05, 
       1.0210997003739260e-05, 
       1.0261622470509316e-05, 
       1.1840748520204376e-05, 
       1.2223977087196286e-05, 
       1.1938039783335699e-05, 
       1.4218991417974111e-05, 
       1.2254839504274966e-05, 
       1.2254839504274966e-05, 
       1.0363599605257835e-05, 
       1.1091793278381049e-05, 
       1.0991394130522510e-05, 
       1.3767536684400838e-05, 
       1.2257392763722619e-05, 
       1.0091338692569694e-05, 
       1.1940986395201969e-05, 
       1.2728472975436712e-05, 
       1.4234519023296746e-05, 
       1.4179297486403456e-05, 
       1.4211292201448137e-05, 
       1.0807116501392373e-05, 
       1.1432153739019050e-05, 
       1.0588314605866533e-05, 
       1.0601215426119423e-05, 
       1.2474957896494427e-05, 
       1.0110950982454661e-05, 
       1.1765852442352391e-05, 
       1.3063215255467915e-05, 
       1.2097295464289595e-05, 
       1.3377317274418171e-05, 
       1.0256962663487446e-05, 
       1.0918286234002667e-05, 
       1.0040154454996167e-05, 
       1.2314910558417348e-05, 
       1.2381680462325893e-05, 
       1.2262271650170048e-05, 
       1.2500611476233270e-05, 
       1.4170372253249018e-05, 
       1.3860929699559270e-05, 
       1.0295876872724294e-05, 
       1.4150164951585038e-05, 
       1.4168669082428720e-05, 
       1.4168669082428720e-05, 
       1.0022795513429451e-05, 
       1.3811032014607832e-05, 
       1.4165287469393681e-05, 
       1.0400835662083885e-05, 
       1.2775656086646232e-05, 
       1.1465809149777593e-05, 
       1.1778244916515037e-05, 
       1.3640944817942931e-05, 
       1.2308665443426985e-05, 
       1.3911938834561814e-05, 
       1.0004030941717438e-05, 
       1.0383184088395749e-05, 
       1.1743217462010197e-05, 
       1.0309746749173572e-05, 
       1.0247770577424164e-05, 
       1.0605412830024893e-05, 
       1.1674255861494337e-05, 
       1.0547939428984845e-05, 
       1.0755911967618475e-05, 
       1.0755911967618475e-05, 
       1.1742758684484464e-05, 
       1.0378227960700910e-05, 
       1.0353677029262057e-05, 
       1.0065808317388327e-05, 
       1.3626788072873657e-05, 
       1.0056924523394971e-05, 
       1.3764560799280717e-05, 
       1.1744295938719913e-05, 
       1.1789628643451820e-05, 
       1.3823208696104777e-05, 
       1.3836771726742849e-05, 
       1.0011732520350094e-05, 
       1.0188398686554663e-05, 
       1.2105455992031732e-05, 
       1.0616900291928309e-05, 
       1.0891582039366153e-05, 
       1.4047804863415895e-05, 
       1.0723982414901454e-05, 
       1.0723982414901454e-05, 
       1.2195102371984048e-05, 
       1.0282270946052643e-05, 
       1.1712518041176429e-05, 
       1.3162071804922592e-05, 
       1.0535304716944216e-05, 
       1.0581162499063250e-05, 
       1.2014355872174439e-05, 
       1.2873987436884781e-05, 
       1.0000000000000023e-05, 
       1.0000000000000268e-05, 
       1.0000000000000268e-05, 
       1.0033592543098577e-05, 
       1.0800985765584415e-05, 
       1.1532991698213397e-05, 
       1.3526479268226931e-05, 
       1.0000000000000323e-05, 
       1.0209533010079571e-05, 
       1.1425515239113752e-05, 
       1.0672595273858708e-05, 
       1.0386428230457244e-05, 
       1.3996350046693657e-05, 
       1.3843064277409263e-05, 
       1.3510773089777084e-05, 
       1.2056171205435316e-05, 
       1.3041147945189136e-05, 
       1.1545379148887747e-05, 
       1.0669072968944727e-05, 
       1.3422342374721899e-05, 
       1.0335975632094324e-05, 
       1.1355108394263668e-05, 
       1.0201521764179614e-05, 
       1.0696988400802460e-05, 
       1.2562648157101754e-05, 
       1.3790198634945796e-05, 
       1.0816152447077934e-05, 
       1.0816152447077934e-05, 
       1.3693871290507112e-05, 
       1.0371421785939316e-05, 
       1.1049091633219835e-05, 
       1.1049091633219835e-05, 
       9.9999999999998026e-06, 
       9.8863414828475480e-06, 
       1.0842334354545805e-05, 
       1.1016553673155095e-05, 
       1.3806136990466827e-05, 
       1.0172321800882813e-05, 
       1.0852161581951677e-05, 
       1.2420685585964461e-05, 
       1.0272382108427438e-05, 
       1.1555529804645246e-05, 
       1.0230642211654634e-05, 
       1.1860806030675018e-05, 
       1.1972800210400375e-05, 
       1.0636661219404868e-05, 
       1.1337543309452617e-05, 
       1.0582387614387777e-05, 
       1.1690955108063830e-05, 
       1.0069854078196332e-05, 
       1.0575796050493584e-05, 
       1.0378017196842637e-05, 
       1.3734244839181230e-05, 
       1.0416475571937694e-05, 
       1.0404572494067358e-05, 
       1.1581954715254319e-05, 
       1.3849959962822386e-05, 
       1.2633992051665372e-05, 
       1.1517864245491153e-05, 
       1.3753117039886167e-05, 
       1.0084075968533105e-05, 
       1.3229163901084218e-05, 
       1.3827459930612570e-05, 
       1.1485796255562143e-05, 
       1.2463485321869251e-05, 
       1.0328189515704196e-05, 
       1.1705425399479901e-05, 
       1.0225050862827386e-05, 
       1.0583832519574678e-05, 
       1.1702613649141828e-05, 
       1.0918332262707825e-05, 
       1.2930036780897380e-05, 
       1.1464072437495110e-05, 
       1.0160436762029178e-05, 
       1.0442452292664705e-05, 
       1.0973096565909740e-05, 
       1.0836505316468286e-05, 
       1.1358432824729901e-05, 
       1.0263225946912435e-05, 
       1.3365430761733129e-05, 
       9.9799890045457438e-06, 
       1.2208053131607979e-05, 
       1.0030559344939961e-05, 
       1.1441834967558262e-05, 
       1.3646803393589440e-05, 
       1.1395454000872295e-05, 
       1.0449805574618379e-05, 
       1.0470554850828422e-05, 
       1.1948195445534538e-05, 
       1.1655612665031576e-05, 
       1.2885907093996807e-05, 
       1.3090450128476459e-05, 
       1.1100879367236872e-05, 
       1.1077682739016790e-05, 
       1.0652921081990114e-05, 
       1.0110335438793266e-05, 
       1.3683167452455347e-05, 
       1.0107142500523414e-05, 
       1.1642936341962837e-05, 
       1.2004178018172652e-05, 
       1.1444779616145984e-05, 
       1.3624996005764152e-05, 
       1.2874817466288836e-05, 
       1.3505913785834337e-05, 
       1.2700988828783988e-05, 
       1.1090490323178219e-05, 
       1.3636622274195857e-05, 
       1.2164703814587884e-05, 
       1.0301581246945533e-05, 
       1.0968053562467948e-05, 
       1.3240901082505807e-05, 
       1.3108089869196893e-05, 
       1.3137693969723944e-05, 
       1.1401138646089511e-05, 
       1.3570164833343339e-05, 
       1.2767501669234633e-05, 
       1.3385644848559539e-05, 
       1.2655776463543991e-05, 
       1.2964417236072301e-05, 
       1.0229194005516311e-05, 
       1.0883072595791353e-05, 
       1.1679234438147814e-05, 
       1.1679234438147814e-05, 
       1.2490013806370188e-05, 
       1.1745994425535471e-05, 
       1.3486364172694538e-05, 
       1.3528212282673858e-05, 
       1.3553187248982594e-05, 
       1.0375079999629666e-05, 
       1.3301925489004176e-05, 
       1.0242896568991389e-05, 
       1.2779438489267659e-05, 
       1.1241657558686918e-05, 
       1.0575834157958750e-05, 
       1.3500186587812073e-05, 
       1.3423660757942420e-05, 
       1.2928116373951288e-05, 
       1.0194224110811556e-05, 
       1.0586378131950798e-05, 
       1.3482629488277775e-05, 
       1.3476123108973351e-05, 
       1.0580794659944253e-05, 
       1.0865960194484819e-05, 
       1.3005561107178907e-05, 
       1.2013861686729904e-05, 
       1.0192595347934893e-05, 
       1.0295145161628820e-05, 
       1.0133772205900771e-05, 
       1.1159488928946389e-05, 
       1.3235194096015786e-05, 
       1.3342282408806637e-05, 
       1.1180049595241262e-05, 
       1.0175929685198419e-05, 
       1.2895519020831097e-05, 
       1.1798273344587275e-05, 
       1.0364329079291338e-05, 
       1.0256838686053355e-05, 
       1.2428105951296390e-05, 
       1.3345874543649668e-05, 
       1.1053170611947719e-05, 
       1.1000748706659975e-05, 
       1.1000748706659975e-05, 
       1.2310427348020285e-05, 
       1.0513032395675814e-05, 
       1.1884185445599024e-05, 
       1.0350964768616864e-05, 
       1.1262161674427384e-05, 
       1.2440603050303903e-05, 
       1.0801818030161721e-05, 
       1.2595514104754330e-05, 
       1.2326398375685728e-05, 
       1.3121462357231044e-05, 
       1.2051111135393631e-05, 
       9.6861944557425459e-06, 
       1.2847466052592783e-05, 
       1.1167242322871567e-05, 
       1.1116526292907975e-05, 
       1.0919008697364914e-05, 
       1.2820466908398491e-05, 
       1.0006634682572335e-05, 
       1.1799576701145489e-05, 
       1.3213155925409601e-05, 
       1.3240111960265754e-05, 
       1.1488586968185329e-05, 
       1.3048285481238764e-05, 
       1.3327145467522403e-05, 
       1.0729667180655719e-05, 
       1.1740840462466724e-05, 
       1.2120482424671358e-05, 
       1.1105434464188138e-05, 
       1.1415543058879631e-05, 
       1.1784324663312130e-05, 
       1.0359203405959379e-05, 
       1.0198098845117519e-05, 
       1.0727830680943782e-05, 
       1.3028763351597676e-05, 
       1.2394632676909523e-05, 
       1.2394632676909523e-05, 
       1.0104657847434613e-05, 
       1.1179510379559346e-05, 
       1.2827736738588087e-05, 
       1.1049125996792385e-05, 
       1.0118138447390444e-05, 
       1.0938863237606858e-05, 
       1.3284088339869708e-05, 
       1.0030471288876616e-05, 
       1.1465809149776834e-05, 
       1.2727723023151873e-05, 
       1.1063968864139535e-05, 
       1.0296427106331738e-05, 
       1.0413933044058707e-05, 
       1.3260080337262532e-05, 
       1.3252913880364026e-05, 
       1.2639527702349771e-05, 
       1.3124664698887093e-05, 
       1.1112226361145868e-05, 
       1.1324609052229203e-05, 
       1.0260673948150928e-05, 
       1.1378045706685741e-05, 
       1.1778752049701913e-05, 
       1.0529287717220461e-05, 
       9.9999999999996654e-06, 
       1.2566543426591820e-05, 
       1.3212829544740357e-05, 
       1.1437621428574644e-05, 
       1.0344629517573135e-05, 
       1.3184394044685354e-05, 
       1.0912589505804658e-05, 
       9.6880600984534400e-06, 
       1.0000000000000016e-05, 
       1.1808055748105135e-05, 
       9.9999999999996366e-06, 
       1.1168479109041615e-05, 
       1.1266174807010030e-05, 
       1.1088943114261005e-05, 
       1.0531901172657745e-05, 
       1.1975802412901960e-05, 
       9.5440433848949558e-06, 
       1.1157699109273153e-05, 
       1.3162535441487810e-05, 
       1.3144009559597298e-05, 
       1.0934163879949750e-05, 
       1.2030385567571521e-05, 
       1.2860220334494098e-05, 
       1.2465258485631167e-05, 
       1.0809789107247169e-05, 
       1.2483979141210206e-05, 
       1.0061794968809452e-05, 
       1.0718601593863509e-05, 
       1.0645033821438555e-05, 
       9.9321116486192328e-06, 
       1.2828634711558548e-05, 
       1.0901080254263570e-05, 
       1.0780892821658742e-05, 
       1.2492528783680332e-05, 
       1.0541630982379583e-05, 
       1.0583470070958790e-05, 
       1.3075268468044839e-05, 
       1.0865920465580460e-05, 
       1.2833127188939596e-05, 
       1.0836004047366933e-05, 
       1.2096826383260093e-05, 
       1.3051139882532289e-05, 
       1.1340249501081946e-05, 
       1.2853405433508901e-05, 
       1.0989910376443089e-05, 
       1.2611360976825520e-05, 
       1.0072804733493211e-05, 
       1.0000000000000102e-05, 
       1.0918875645461945e-05, 
       1.2467239289853937e-05, 
       1.2744350577852576e-05, 
       1.2259309466877248e-05, 
       1.2200874581673758e-05, 
       1.0778205181598157e-05, 
       9.9999999999999110e-06, 
       1.2972779964556739e-05, 
       1.0960577395688777e-05, 
       1.2762456251609355e-05, 
       1.2823548419072967e-05, 
       1.1519499364513228e-05, 
       1.0055262577626300e-05, 
       1.1364520851940623e-05, 
       1.0918209704670864e-05, 
       1.0492225339710271e-05, 
       1.0986575418765329e-05, 
       1.0708759334643327e-05, 
       1.2002906148886288e-05, 
       1.2500285764523185e-05, 
       1.0979883293440163e-05, 
       1.2893993524991976e-05, 
       1.0146693875938490e-05, 
       1.0146693875938490e-05, 
       1.2511123638863749e-05, 
       1.0764462423615642e-05, 
       1.2202636357993463e-05, 
       1.0307741245399329e-05, 
       1.2520590834186167e-05, 
       1.0421492432074835e-05, 
       1.0319865455529786e-05, 
       1.1360091841782575e-05, 
       1.0837429653674148e-05, 
       1.1623919055502374e-05, 
       1.1846934740013011e-05, 
       1.0889709012286301e-05, 
       1.0820849548411076e-05, 
       1.0688216114365459e-05, 
       1.0744630642435491e-05, 
       1.0516142503876665e-05, 
       1.2690810856557470e-05, 
       1.2453915066165369e-05, 
       1.0024401752492738e-05, 
       1.0399536098569722e-05, 
       1.2841554799162064e-05, 
       1.2371386953186853e-05, 
       1.2809527010425565e-05, 
       1.0696610597318611e-05, 
       1.0849138714719213e-05, 
       1.0778176715706983e-05, 
       1.2190331993269490e-05, 
       1.1725430873935922e-05, 
       1.2268706427768936e-05, 
       1.1083578457964460e-05, 
       1.0529499891400088e-05, 
       1.0408199320983197e-05, 
       1.0267425298825185e-05, 
       1.1102949974510588e-05, 
       1.0228556525520173e-05, 
       1.0822384486713595e-05, 
       1.1726431655036165e-05, 
       1.0622283055352958e-05, 
       1.0507116872230133e-05, 
       1.2508011499520140e-05, 
       1.0701480936194088e-05, 
       1.0858941383488884e-05, 
       1.0828342586633265e-05, 
       1.0868737606804769e-05, 
       1.0515856237448519e-05, 
       1.0862968014903321e-05, 
       1.2353519139857615e-05, 
       1.2691384339691330e-05, 
       1.0319655845241792e-05, 
       1.1420857041808283e-05, 
       1.0762032042260704e-05, 
       1.0231557242102373e-05, 
       1.0842835758030894e-05, 
       1.0569213931884323e-05, 
       1.1621722314993375e-05, 
       1.0534936312424333e-05, 
       1.2193195697120969e-05, 
       1.0215665961202269e-05, 
       1.2543609614379301e-05, 
       1.0605870145837993e-05, 
       1.0814859593310377e-05, 
       1.0888060708822404e-05, 
       1.0415183192091767e-05, 
       1.1198884113209313e-05, 
       1.0132178103724105e-05, 
       1.0675364608552892e-05, 
       1.0123843706689495e-05, 
       1.2131667974029125e-05, 
       1.0277639360913933e-05, 
       1.0488810596038888e-05, 
       1.0891154897709894e-05, 
       1.0131230498547337e-05, 
       1.0491837130241224e-05, 
       9.7735644636457358e-06, 
       1.0648342360139235e-05, 
       1.2518346581190158e-05, 
       1.0240618113256932e-05, 
       1.0277168100648207e-05, 
       1.1918107311041250e-05, 
       1.0391178957444035e-05, 
       1.0278392183878733e-05, 
       1.0278392183878733e-05, 
       1.1720904715688604e-05, 
       1.2210414082431252e-05, 
       1.0150850778128672e-05, 
       1.1553441605469168e-05, 
       1.1334665045753116e-05, 
       1.0677203457460138e-05, 
       1.2429794488038528e-05, 
       1.2205913188185127e-05, 
       1.2008345346664152e-05, 
       1.0157142412420406e-05, 
       1.2392435906262250e-05, 
       1.0435466995246321e-05, 
       1.0515409084517231e-05, 
       1.2195971432321853e-05, 
       1.2235711884014671e-05, 
       1.2292589076882048e-05, 
       1.2451993531768390e-05, 
       1.2501743542146667e-05, 
       1.0969058251643321e-05, 
       1.1178757165294488e-05, 
       1.2422510615194479e-05, 
       1.1368031979734734e-05, 
       1.1382720783756992e-05, 
       1.2485310554560600e-05, 
       1.1107335688654900e-05, 
       1.0355668138949830e-05, 
       1.2477518961862657e-05, 
       1.0631202532206513e-05, 
       1.2223159662942926e-05, 
       1.0242237262682289e-05, 
       1.2420697734083275e-05, 
       1.0434942316190652e-05, 
       1.1445932383622525e-05, 
       1.0171224485565190e-05, 
       1.1720131338961665e-05, 
       1.0327567585121191e-05, 
       1.2188033579879494e-05, 
       1.0779648076362077e-05, 
       1.0320481366151830e-05, 
       1.0436073830974198e-05, 
       1.0208397914965615e-05, 
       1.2065356413651382e-05, 
       1.2065356413651382e-05, 
       1.1274243583633306e-05, 
       1.2354013568792376e-05, 
       1.1295689393518569e-05, 
       1.0118517548772012e-05, 
       1.2350502129827803e-05, 
       1.2254751365195674e-05, 
       1.1878441154163293e-05, 
       1.2387603755462172e-05, 
       1.1623463077479740e-05, 
       1.2317193135841640e-05, 
       1.0078216330379143e-05, 
       1.0366183554054452e-05, 
       1.0487280624726505e-05, 
       1.1866795523912437e-05, 
       1.1899697738117376e-05, 
       1.0736487387057855e-05, 
       1.1950070976428671e-05, 
       1.1221080265170288e-05, 
       1.1697319550334386e-05, 
       1.1769674617668093e-05, 
       1.0943934790898658e-05, 
       1.1804992754280547e-05, 
       1.2263794458669887e-05, 
       1.2143834399481443e-05, 
       1.1411449496704077e-05, 
       1.1392848751274944e-05, 
       1.1366047363172596e-05, 
       1.2283669916598482e-05, 
       1.0340624561312349e-05, 
       1.2280866626799468e-05, 
       1.0173644722021488e-05, 
       1.2095338482765341e-05, 
       1.2036863138255337e-05, 
       1.1815858135741824e-05, 
       1.1815858135741824e-05, 
       1.2266954348323858e-05, 
       1.2044678583800169e-05, 
       1.0093783567833868e-05, 
       1.0136421901119502e-05, 
       1.1872768703119583e-05, 
       1.2226382891714440e-05, 
       1.2047532551853724e-05, 
       1.2192925558836002e-05, 
       1.1137101927630641e-05, 
       1.0049082437662472e-05, 
       1.0049082437662472e-05, 
       1.2226454785819145e-05, 
       1.0456657992935101e-05, 
       1.1746499882668840e-05, 
       1.0487829631785488e-05, 
       1.2193288145787433e-05, 
       1.0221126412831013e-05, 
       1.0601860689653090e-05, 
       1.1559269957116234e-05, 
       1.2189619299817823e-05, 
       1.2139763415502552e-05, 
       1.1497971921071812e-05, 
       1.2065540104242826e-05, 
       1.1235033336164941e-05, 
       1.0610767812585569e-05, 
       1.0794326490968710e-05, 
       1.0273325805973786e-05, 
       1.1343878200826428e-05, 
       1.1609328060759204e-05, 
       1.1306341753215061e-05, 
       1.0825536710383240e-05, 
       1.1005795704342938e-05, 
       1.2121060757588093e-05, 
       1.2106505031350982e-05, 
       8.5754234129220994e-06, 
       1.0815761881751580e-05, 
       1.0219010595921674e-05, 
       1.0270514675373261e-05, 
       1.0813708100617444e-05, 
       1.0105667861555246e-05, 
       1.0056928116534905e-05, 
       1.0679507236274683e-05, 
       1.0249886465693353e-05, 
       1.0630195034163127e-05, 
       1.1803059379380101e-05, 
       1.0809637084121703e-05, 
       1.1429606680863589e-05, 
       1.1696599587364543e-05, 
       1.1214113892499516e-05, 
       1.1115287854269790e-05, 
       1.0997164930048754e-05, 
       1.1838762179157279e-05, 
       1.1641041837148440e-05, 
       1.1823447647627574e-05, 
       1.1104117240007266e-05, 
       1.1517148443691452e-05, 
       9.1485220696467970e-06, 
       1.0238965638900136e-05, 
       8.8919292874191530e-06, 
       1.1896330057952346e-05, 
       1.0497030131876357e-05, 
       1.1931376034776510e-05, 
       1.1933623040098443e-05, 
       1.0008601757717923e-05, 
       1.0591796756262276e-05, 
       1.1808370660381053e-05, 
       1.0153248540206407e-05, 
       1.0583159446168757e-05, 
       1.1569757653558306e-05, 
       1.1169324836137592e-05, 
       1.0246696592656460e-05, 
       1.1634855361359325e-05, 
       1.1835975123834041e-05, 
       1.0386781596695411e-05, 
       1.0254535427430939e-05, 
       1.1781556043583744e-05, 
       1.0550275164110156e-05, 
       1.0673620790962177e-05, 
       1.0323547064690068e-05, 
       1.0304941240959837e-05, 
       1.1837892347508179e-05, 
       1.1610136318218489e-05, 
       1.1075039004490072e-05, 
       1.0201628826680417e-05, 
       1.1664572115206606e-05, 
       1.0350041204194961e-05, 
       1.0281257550234481e-05, 
       1.0183497996229007e-05, 
       1.1790867452521418e-05, 
       1.1631679243118851e-05, 
       1.1338406639534840e-05, 
       1.0176300122955356e-05, 
       1.0251446762046222e-05, 
       1.0958827008473277e-05, 
       1.1576712894601486e-05, 
       1.0683772784826339e-05, 
       1.1585407245460639e-05, 
       1.0967951697648741e-05, 
       1.1358912209348374e-05, 
       1.0398128628659206e-05, 
       1.0835359287821527e-05, 
       1.1346940508727855e-05, 
       1.0226529771569070e-05, 
       1.1611212997899574e-05, 
       1.0473237255046831e-05, 
       1.1288721322259441e-05, 
       1.1698333661076355e-05, 
       9.2370224114736929e-06, 
       1.1064033135139272e-05, 
       1.1027381989352890e-05, 
       1.1671777168690116e-05, 
       1.1643272554156096e-05, 
       1.1688194370409005e-05, 
       1.0999967204317031e-05, 
       1.1607986155636769e-05, 
       1.1682607882720718e-05, 
       1.1636888690997451e-05, 
       1.1546907529907148e-05, 
       1.0373814780092168e-05, 
       1.1205740540454186e-05, 
       1.1517353343776237e-05, 
       1.0844527101560402e-05, 
       1.0000000000000131e-05, 
       1.0107604450208000e-05, 
       1.1638821972318951e-05, 
       9.9999999999995366e-06, 
       1.0704849040951484e-05, 
       9.9885471276394164e-06, 
       1.1513813993823662e-05, 
       1.1393396355224885e-05, 
       1.1155008795590567e-05, 
       9.2586238336926927e-06, 
       1.1520535543541284e-05, 
       1.1472151178516296e-05, 
       1.0204638282290158e-05, 
       1.1041110004442580e-05, 
       1.1111133081360804e-05, 
       1.1511399872203585e-05, 
       1.0084716239022681e-05, 
       1.1338504819397106e-05, 
       1.0045158540498697e-05, 
       9.5863471412588328e-06, 
       1.0437127983706993e-05, 
       1.1505072339552708e-05, 
       1.0906688298653201e-05, 
       1.0602208009787852e-05, 
       1.0850364545656378e-05, 
       1.0946978336327034e-05, 
       1.1248986091004524e-05, 
       1.1053100797946898e-05, 
       1.1454865516766797e-05, 
       1.1407421854723259e-05, 
       1.1446701910336723e-05, 
       1.1288404556624391e-05, 
       1.0392249307434665e-05, 
       1.1370560671741503e-05, 
       1.1136702107910117e-05, 
       1.0571192903146808e-05, 
       1.0571192903146808e-05, 
       1.0035826633069446e-05, 
       1.0153506910273173e-05, 
       1.1292970679226960e-05, 
       1.1282465065604453e-05, 
       1.1357160832396014e-05, 
       1.0032210568173383e-05, 
       1.0903015001942314e-05, 
       1.1169750399560845e-05, 
       1.0169324841006118e-05, 
       1.1317743609076503e-05, 
       1.0441894575960025e-05, 
       1.0855867360557481e-05, 
       1.1003852832147987e-05, 
       1.1221111558544434e-05, 
       1.1229090984497455e-05, 
       1.1208141846847685e-05, 
       1.1047875195611803e-05, 
       1.0861470609398258e-05, 
       1.0230465364057099e-05, 
       1.0868146619565750e-05, 
       8.7573672075511444e-06, 
       1.1113716088480374e-05, 
       1.0487124692589549e-05, 
       1.1125459079521999e-05, 
       1.1089927485805763e-05, 
       1.0941880566651757e-05, 
       1.0984098754776277e-05, 
       1.0692711398637797e-05, 
       1.0790492830363194e-05, 
       1.0997608543055777e-05, 
       1.1058123076535649e-05, 
       1.0668058765095470e-05, 
       1.0088839272899624e-05, 
       1.0764745606081065e-05, 
       1.0952099235736701e-05, 
       1.0982488563152961e-05, 
       7.8837979404905654e-06, 
       1.0862229832361817e-05, 
       1.0541641529977805e-05, 
       1.0673604470606032e-05, 
       8.4200784579030288e-06, 
       1.0449246940496374e-05, 
       1.0451199173168662e-05, 
       1.0451199173168662e-05, 
       1.0765901784358514e-05, 
       1.0027420269936300e-05, 
       1.0690276818582648e-05, 
       1.0932588706010660e-05, 
       1.0242400449443212e-05, 
       1.0925597573154379e-05, 
       1.0526296249549063e-05, 
       1.0901892062286681e-05, 
       1.0892631794838299e-05, 
       1.0860212778975072e-05, 
       1.0901760409426162e-05, 
       1.0479923337425894e-05, 
       1.0319417750035747e-05, 
       1.0494016569488708e-05, 
       1.0882109539471085e-05, 
       9.0100892682608505e-06, 
       1.0551879609854271e-05, 
       1.0857297352979116e-05, 
       1.0601985153734728e-05, 
       1.0855113115728639e-05, 
       9.9999999999997670e-06, 
       9.9999999999996874e-06, 
       1.0000000000000477e-05, 
       9.9999999999999602e-06, 
       1.0000000000000265e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       9.9999999999995078e-06, 
       9.9999999999998263e-06, 
       1.0000000000000402e-05, 
       1.0000000000000058e-05, 
       1.0851775992997990e-05, 
       1.0777559534306887e-05, 
       1.0457837785484885e-05, 
       1.0337086705433358e-05, 
       1.0399030604689359e-05, 
       1.0554182453027874e-05, 
       1.0606808503216396e-05, 
       1.0469552074835209e-05, 
       1.0758187477315188e-05, 
       1.0764274253704218e-05, 
       1.0748503052501520e-05, 
       1.0055159804355488e-05, 
       1.0560590688131672e-05, 
       1.0531867662253147e-05, 
       1.0200606462791767e-05, 
       1.0435063084191800e-05, 
       1.0058662408922714e-05, 
       1.0463192453881937e-05, 
       1.0709339178497549e-05, 
       1.0248306633511604e-05, 
       1.0657939536519573e-05, 
       1.0032877344981024e-05, 
       1.0285698682298048e-05, 
       1.0339573555828056e-05, 
       1.0333301509618587e-05, 
       1.0430174001961241e-05, 
       1.0580486546586671e-05, 
       9.7335271290188978e-06, 
       1.0547214944993346e-05, 
       1.0178681727673101e-05, 
       1.0466926065755992e-05, 
       1.0582186412973638e-05, 
       1.0239764319536040e-05, 
       1.0223160230998520e-05, 
       1.0533003112420351e-05, 
       1.0298809874875783e-05, 
       1.0519401134910453e-05, 
       1.0531407751771536e-05, 
       1.0493063275361416e-05, 
       1.0470235844516859e-05, 
       1.0393490793909300e-05, 
       1.0457768047153454e-05, 
       1.0063106233817438e-05, 
       9.9999999999997975e-06, 
       9.6066279557647178e-06, 
       1.0091618588870042e-05, 
       1.0358770802625782e-05, 
       1.0389774252348478e-05, 
       1.0361234027520185e-05, 
       7.4402997717178416e-06, 
       1.0284713501300155e-05, 
       1.0279142377792454e-05, 
       1.0312178237078133e-05, 
       9.1579311574051531e-06, 
       1.0333965162543499e-05, 
       1.0316025344018487e-05, 
       1.0305496593024411e-05, 
       1.0187916711954165e-05, 
       1.0061483530566063e-05, 
       1.0235734916679891e-05, 
       1.0242991715124708e-05, 
       9.9999999999996722e-06, 
       1.0276460202281464e-05, 
       1.0021533364610812e-05, 
       1.0232542491765221e-05, 
       1.0246155073062100e-05, 
       1.0218592229360409e-05, 
       1.0029594617618908e-05, 
       9.7360476184160780e-06, 
       1.0124465322484542e-05, 
       9.7775140930810087e-06, 
       1.0044288737344210e-05, 
       1.0088411917600719e-05, 
       1.0142038186682246e-05, 
       1.0010592423648064e-05, 
       9.9999999999997281e-06, 
       1.0071131086643633e-05, 
       1.0122300585079421e-05, 
       1.0109949153041595e-05, 
       1.0107767308264768e-05, 
       1.0115076025184727e-05, 
       1.0041907279180073e-05, 
       1.0019880079227199e-05, 
       1.0050018788159446e-05, 
       7.2384455027785996e-06, 
       1.0000000000000418e-05, 
       1.0000000000000414e-05, 
       1.0000000000000197e-05, 
       1.0000000000000345e-05, 
       9.9999999999996671e-06, 
       9.9999999999996061e-06, 
       9.9999999999996061e-06, 
       1.0000000000000396e-05, 
       9.9999999999999415e-06, 
       1.0000000000000102e-05, 
       1.0000000000000018e-05, 
       9.9999999999998229e-06, 
       9.9999999999998229e-06, 
       9.9999999999998297e-06, 
       9.9999999999995604e-06, 
       1.0000000000000370e-05, 
       1.0000000000000370e-05, 
       1.0000000000000370e-05, 
       9.9999999999998433e-06, 
       1.0000000000000087e-05, 
       9.9999999999996773e-06, 
       9.9999999999995858e-06, 
       9.9999999999995858e-06, 
       1.0000000000000140e-05, 
       1.0000000000000140e-05, 
       1.0000000000000268e-05, 
       9.9999999999997755e-06, 
       9.9999999999995587e-06, 
       9.9999999999997365e-06, 
       1.0000000000000043e-05, 
       1.0000000000000043e-05, 
       1.0000000000000043e-05, 
       9.9999999999996518e-06, 
       9.9999999999997874e-06, 
       1.0000000000000313e-05, 
       1.0000000000000275e-05, 
       1.0000000000000275e-05, 
       9.9999999999997145e-06, 
       1.0000000000000187e-05, 
       9.9999999999997332e-06, 
       1.0000000000000153e-05, 
       1.0000000000000424e-05, 
       9.9999999999998704e-06, 
       9.9999999999996739e-06, 
       1.0000000000000031e-05, 
       9.9999999999996620e-06, 
       9.9999999999999331e-06, 
       9.9999999999998636e-06, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       9.9276769143992826e-06, 
       8.9214736527310248e-06, 
       8.9600403999452289e-06, 
       6.9972294257189717e-06, 
       8.4057015001877455e-06, 
       9.3308453861784518e-06, 
       8.2355962587577931e-06, 
       7.4347161580137081e-06, 
       6.3564941087378608e-06, 
       6.6098969561254629e-06, 
       8.8670988397329206e-06, 
       8.7130972391972014e-06, 
       7.6554779559906427e-06, 
       8.4184260702981578e-06, 
       7.8738939073247051e-06, 
       8.5206363862676961e-06, 
       7.1890470396666863e-06, 
       7.9390841266504732e-06, 
       6.0104460260625855e-06, 
       7.0454590767425861e-06, 
       6.0539712200894748e-06, 
       6.0292957950127514e-06, 
       7.6382707738750426e-06, 
       5.7668944303463087e-06, 
       7.5377624762324105e-06, 
       6.2732718608679188e-06, 
       6.0403016421698422e-06, 
       6.6479441112793904e-06, 
       4.7257396507225487e-06, 
       5.0446984949984104e-06, 
       4.4321483554784521e-06, 
       5.5319825328363708e-06, 
       4.3373982845981741e-06, 
       5.6717935704369968e-06, 
       4.2240003896380582e-06, 
       4.9598137452493484e-06, 
       5.7315597150291972e-06, 
       5.5970657068869918e-06, 
       4.6554052538427468e-06, 
       5.0651417314833270e-06, 
       4.0918340727629949e-06, 
       4.2247385020999436e-06, 
       4.6038208711109434e-06, 
       3.2235609800667443e-06, 
       3.1285179284457663e-06, 
       4.1993287195063803e-06, 
       3.0097698467174810e-06, 
       4.1551966185885370e-06, 
       3.2150773190843271e-06, 
       4.0884734528614543e-06, 
       4.1444937401816535e-06, 
       3.8676464986809128e-06, 
       3.9255086534156559e-06, 
       2.9488822684155869e-06, 
       2.7300435072767274e-06, 
       3.5230053979075721e-06, 
       2.8564573635535870e-06, 
       2.8916221191763487e-06, 
       2.3873418313482731e-06, 
       2.8452084361953237e-06, 
       2.4921849021287432e-06, 
       2.2786364864845360e-06, 
       1.8644900725049101e-06, 
       1.8805378072745463e-06, 
       1.7178166346072060e-06, 
       1.7072389730096216e-06, 
       1.4521989448006111e-06, 
       1.1210431561524651e-06, 
       1.0472480871178415e-06, 
       9.5638469964008220e-07, 
       8.8095299923467385e-07, 
       6.7058215537446876e-07, 
       5.6483898759171525e-07, 
       3.0146074863636613e-07, 
       3.9406941389153455e-07, 
       2.8861348174430666e-07, 
       2.8346666350137537e-07, 
       2.0873669967861465e-07, 
       1.2245229840726787e-08, 
       1.6768000000000001e+00, 
       5.1641999999999999e-01, 
       4.1382999999999998e-01, 
       3.7114999999999998e-01, 
       2.4598630519789147e-01, 
       2.7560000000000001e-01, 
       7.1179999999999993e-02, 
       6.1280000000000001e-02, 
       4.5159999999999999e-02, 
       4.0480000000000002e-02, 
       3.7470000000000003e-02, 
       3.7069999999999999e-02, 
       3.6490000000000002e-02, 
       2.4379999999999999e-02, 
       2.1649999999999999e-02, 
       1.6919100290757366e-02, 
       1.9230000000000001e-02, 
       1.3609297575554020e-02, 
       1.2763511856205752e-02, 
       1.2930000000000001e-02, 
       1.2699999999999999e-02, 
       1.0970000000000001e-02, 
       1.0877073284572179e-02, 
       8.3453599048365513e-03, 
       7.3400000000000002e-03, 
       6.8599999999999998e-03, 
       6.3099999999999996e-03, 
       5.6619807848686454e-03, 
       5.6611068538185228e-03, 
       5.0937252062072062e-03, 
       5.8500000000000002e-03, 
       3.8723575555701987e-03, 
       5.3899999999999998e-03, 
       4.6899999999999997e-03, 
       2.7755230992037357e-03, 
       2.8925319854490442e-03, 
       3.7799999999999999e-03, 
       2.8467780380650328e-03, 
       2.8229930855291322e-03, 
       3.0265170195963847e-03, 
       3.6200000000000000e-03, 
       2.7200641051918287e-03, 
       2.4350269406220870e-03, 
       2.3951159060627606e-03, 
       3.1700000000000001e-03, 
       3.0000000000000001e-03, 
       2.1566458249048541e-03, 
       2.9499999999999999e-03, 
       2.8320556956046273e-03, 
       2.3901365119155470e-03, 
       2.7000000000000001e-03, 
       2.5600000000000002e-03, 
       1.5216239435853550e-03, 
       1.5010567723298686e-03, 
       1.5046083924101031e-03, 
       1.6113451360524626e-03, 
       1.6108183401981411e-03, 
       1.5619738963180364e-03, 
       1.8600000000000001e-03, 
       1.2413671622961934e-03, 
       1.6900000000000001e-03, 
       1.1780629944789897e-03, 
       1.6000000000000001e-03, 
       1.5499999999999999e-03, 
       1.4800000000000000e-03, 
       1.4126173984728781e-03, 
       1.4100000000000000e-03, 
       1.3799999999999999e-03, 
       1.0270133643677396e-03, 
       1.2899999999999999e-03, 
       9.9890919804875846e-04, 
       1.2600000000000001e-03, 
       9.2726750800733787e-04, 
       8.8585603632539123e-04, 
       1.1999999999999999e-03, 
       9.4182386532185101e-04, 
       9.3446107676674296e-04, 
       8.6986385266733595e-04, 
       8.7655265964070030e-04, 
       8.7343326299490251e-04, 
       8.2358774904712801e-04, 
       1.0183723058731088e-03, 
       1.0174618776608048e-03, 
       1.0700000000000000e-03, 
       1.0399999999999999e-03, 
       9.7000000000000005e-04, 
       9.6000000000000002e-04, 
       9.3999999999999997e-04, 
       9.3999999999999997e-04, 
       8.9999999999999998e-04, 
       8.8000000000000003e-04, 
       6.6131741401725204e-04, 
       6.4928667044860603e-04, 
       6.2789883250667601e-04, 
       7.9000000000000001e-04, 
       7.6000000000000004e-04, 
       7.6000000000000004e-04, 
       7.6000000000000004e-04, 
       5.9634959771494474e-04, 
       7.2999999999999996e-04, 
       7.2999999999999996e-04, 
       5.2331766276262205e-04, 
       7.1000000000000002e-04, 
       5.9862731465607421e-04, 
       5.9779752173464199e-04, 
       5.1186591180592527e-04, 
       5.5690123962899346e-04, 
       6.8999999999999997e-04, 
       5.4295542940330894e-04, 
       5.2177512040737920e-04, 
       6.6000000000000000e-04, 
       4.9977063346190714e-04, 
       4.8669255186217368e-04, 
       4.8663574669448946e-04, 
       5.6555962414255522e-04, 
       5.6555962414255522e-04, 
       5.8214271549757439e-04, 
       4.7171612356060452e-04, 
       4.4341697038978347e-04, 
       4.7189578980749513e-04, 
       5.8000000000000000e-04, 
       5.4011132164644734e-04, 
       4.0619385423640872e-04, 
       4.0066018258212141e-04, 
       4.0434115514577867e-04, 
       3.6526073369289210e-04, 
       4.9221561879064899e-04, 
       4.8805361745649290e-04, 
       3.4567388253694333e-04, 
       3.4800682751414563e-04, 
       3.7336207178047720e-04, 
       3.3078554701819365e-04, 
       3.4618968938920366e-04, 
       3.2997652781516993e-04, 
       4.4999999999999999e-04, 
       3.3714940952784959e-04, 
       4.1936404886310316e-04, 
       3.4905150524750281e-04, 
       3.1894306471750943e-04, 
       4.1640319354152729e-04, 
       3.1910995128305090e-04, 
       3.1949749035901145e-04, 
       3.8461960341498323e-04, 
       3.8459937917686017e-04, 
       3.1206459231754920e-04, 
       3.0109596168038190e-04, 
       4.0999999999999999e-04, 
       3.1426843645236083e-04, 
       2.8444718222922461e-04, 
       2.8128276445245809e-04, 
       2.8842697941288998e-04, 
       2.8655576054174121e-04, 
       3.6370056247452117e-04, 
       3.8998653306694707e-04, 
       3.8999999999999999e-04, 
       3.6023995941403394e-04, 
       3.8000000000000002e-04, 
       2.7775061216462314e-04, 
       2.6601917250873884e-04, 
       2.6506343627055945e-04, 
       2.7300779726484336e-04, 
       2.6014628160666355e-04, 
       2.8942224559087474e-04, 
       2.7942794331434049e-04, 
       2.6400856264205468e-04, 
       2.5429174054891876e-04, 
       3.4000000000000002e-04, 
       2.7114859861095104e-04, 
       2.6222691416255051e-04, 
       3.1660008516397884e-04, 
       2.3559142637683757e-04, 
       3.3000000000000000e-04, 
       3.1356699598810258e-04, 
       2.4381791925552252e-04, 
       3.2000000000000003e-04, 
       3.2000000000000003e-04, 
       2.3669530305334046e-04, 
       2.9716501145289685e-04, 
       2.5357362956049298e-04, 
       2.2769041501651902e-04, 
       2.2328154968264302e-04, 
       2.9999999999999997e-04, 
       2.1733480299824208e-04, 
       2.0988274958516311e-04, 
       2.1201539315427644e-04, 
       2.9000000000000000e-04, 
       2.0645811463053187e-04, 
       2.0000000000002446e-04, 
       2.0607149296629815e-04, 
       2.0358337292124119e-04, 
       1.9663808352323310e-04, 
       1.9547861151905776e-04, 
       2.6326379378830680e-04, 
       2.7044064550301659e-04, 
       1.9468585055608108e-04, 
       2.0844192208560572e-04, 
       1.8652982378492198e-04, 
       2.5000000000000001e-04, 
       2.5000000000000001e-04, 
       2.0106652609994485e-04, 
       1.7789354614226090e-04, 
       1.7999999999998901e-04, 
       1.7637053507828666e-04, 
       1.9413057139118013e-04, 
       1.9393750936839033e-04, 
       1.7317967082008492e-04, 
       1.7000000000002470e-04, 
       1.9768251742299464e-04, 
       1.7059839547565799e-04, 
       1.8621092030117562e-04, 
       1.6893241023915442e-04, 
       1.8470034523232002e-04, 
       2.3000000000000001e-04, 
       2.3000000000000001e-04, 
       2.3000000000000001e-04, 
       1.6862007395843811e-04, 
       2.2000000000000001e-04, 
       2.2000000000000001e-04, 
       1.6905726242723751e-04, 
       1.6248410552047860e-04, 
       1.6925930610250630e-04, 
       2.1000000000000001e-04, 
       2.1000000000000001e-04, 
       2.1000000000000001e-04, 
       2.1000000000000001e-04, 
       2.1000000000000001e-04, 
       1.5349956926387037e-04, 
       1.4252037226299830e-04, 
       1.4208834255520730e-04, 
       2.0000000000000001e-04, 
       2.0000000000000001e-04, 
       1.4661472487471143e-04, 
       1.5996109747741101e-04, 
       1.4167795618556271e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.3474751238052969e-04, 
       1.4415543713572695e-04, 
       1.3393228561587800e-04, 
       1.6000000000001891e-04, 
       1.3680789649364819e-04, 
       1.4255600909036291e-04, 
       1.8000000000000001e-04, 
       1.8000000000000001e-04, 
       1.8000000000000001e-04, 
       1.2826568084658131e-04, 
       1.4291083563379114e-04, 
       1.7000000000004663e-04, 
       1.2568276489903371e-04, 
       1.3178635363618110e-04, 
       1.2186148945234961e-04, 
       1.2999999999995589e-04, 
       1.2879618145486221e-04, 
       1.7000000000000001e-04, 
       1.7000000000000001e-04, 
       1.2097025540466503e-04, 
       1.1914878791780433e-04, 
       1.2033340346009163e-04, 
       1.6000000000000001e-04, 
       1.1420738737952349e-04, 
       1.1283963797625469e-04, 
       1.1000000000000044e-04, 
       1.0967996329608863e-04, 
       1.2812422817087810e-04, 
       1.2961637125433751e-04, 
       1.1875361978764619e-04, 
       1.3281345689030116e-04, 
       1.4999999999999999e-04, 
       1.4999999999999999e-04, 
       1.1366156689966042e-04, 
       1.0871830976829956e-04, 
       1.1447339604716639e-04, 
       1.1000000000004498e-04, 
       1.0435397961434781e-04, 
       1.1564413828886771e-04, 
       1.0000000000000213e-04, 
       1.1461197219250756e-04, 
       1.3999999999999999e-04, 
       1.0040586467764777e-04, 
       1.1252481981754288e-04, 
       1.0951702489818630e-04, 
       1.0936720826743502e-04, 
       9.8075020491742633e-05, 
       9.3673145454986945e-05, 
       1.2284022501393469e-04, 
       1.2992680729989375e-04, 
       1.2991104638424312e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       9.5017386849256515e-05, 
       9.9628261775951707e-05, 
       9.5265258572458994e-05, 
       9.1074110401061071e-05, 
       9.9249933838426005e-05, 
       8.9999999999997973e-05, 
       1.1644659225540634e-04, 
       9.8265422289696723e-05, 
       9.8157689146640892e-05, 
       1.0221869074255284e-04, 
       9.0175959454846298e-05, 
       8.8363171301259480e-05, 
       9.0000000000003204e-05, 
       8.6513004813478517e-05, 
       8.6376910144344955e-05, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       8.4910396588031836e-05, 
       8.6303704158404820e-05, 
       8.6312507848137495e-05, 
       9.0814339125707783e-05, 
       8.5856237159871466e-05, 
       8.4923436292185973e-05, 
       8.4768989290665974e-05, 
       8.3514236766920862e-05, 
       8.5595949986654808e-05, 
       8.7115129572176429e-05, 
       8.4519952218010410e-05, 
       1.1341256634033527e-04, 
       1.0042167120100659e-04, 
       1.1229409600703813e-04, 
       8.1810869358911565e-05, 
       9.2203561255744461e-05, 
       1.0786006974930761e-04, 
       9.6914653774837097e-05, 
       8.9999999999996631e-05, 
       7.9999999999995358e-05, 
       8.3963438479949732e-05, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       8.1324132580347792e-05, 
       7.7723800571588637e-05, 
       9.4685760602060317e-05, 
       8.5240481314107931e-05, 
       8.0513901395703555e-05, 
       9.0000000000000372e-05, 
       7.4766200919708641e-05, 
       8.8881337061543178e-05, 
       7.9844727324863575e-05, 
       7.8831853948302144e-05, 
       7.5739797625197825e-05, 
       8.2204264767937732e-05, 
       8.8842686480300503e-05, 
       7.5124829484863738e-05, 
       7.4004126313657510e-05, 
       1.0000000000000000e-04, 
       1.0000000000000000e-04, 
       7.4913040616924936e-05, 
       7.4631825873718531e-05, 
       7.6914348510389156e-05, 
       7.0489702228121405e-05, 
       8.9999999999995032e-05, 
       7.4066777775357972e-05, 
       7.6860194763091091e-05, 
       6.8090620690681083e-05, 
       6.9811404197783587e-05, 
       7.0383318658959903e-05, 
       6.6108421526585336e-05, 
       6.5839249867146346e-05, 
       6.8593479633896170e-05, 
       6.5857903246735846e-05, 
       6.7997271498290260e-05, 
       6.5818794749948116e-05, 
       6.5304478926461420e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       6.5116191827263741e-05, 
       6.6969302137990261e-05, 
       6.3520546486350003e-05, 
       6.9630262712291316e-05, 
       6.6611160947724389e-05, 
       6.5334570018810956e-05, 
       6.8562911070376692e-05, 
       6.4185782239491426e-05, 
       6.4068177718352564e-05, 
       6.1956489351114834e-05, 
       6.5355096389469039e-05, 
       7.3693831536948213e-05, 
       6.6298211332711321e-05, 
       5.8609513281343427e-05, 
       6.5592390976587983e-05, 
       7.0000143094329890e-05, 
       6.3496251289136529e-05, 
       7.7416149432809895e-05, 
       5.9481479247912861e-05, 
       6.0000000000004711e-05, 
       6.1286567111902532e-05, 
       5.8133913919595236e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       6.3983848535223179e-05, 
       5.6252980054447966e-05, 
       6.3483946792380840e-05, 
       7.3743804657105854e-05, 
       5.8362662288080200e-05, 
       6.2201350429539292e-05, 
       6.0000000000004928e-05, 
       5.4837975243834762e-05, 
       5.4190935865195813e-05, 
       5.4950801582622904e-05, 
       5.5518312910777733e-05, 
       5.3141322058460953e-05, 
       5.8084859452123288e-05, 
       6.0000000000002732e-05, 
       5.3843584577924200e-05, 
       7.2405053586645018e-05, 
       5.9999999999997603e-05, 
       6.1962475600750705e-05, 
       5.9636031509552471e-05, 
       5.8055745969866972e-05, 
       5.3716580919742125e-05, 
       7.0000000000003490e-05, 
       5.1225471827403264e-05, 
       5.3032041676239814e-05, 
       4.9999999999999440e-05, 
       5.9998263853224002e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       5.1695887208660406e-05, 
       4.9917651955771599e-05, 
       5.2521674722951197e-05, 
       5.0891744929096196e-05, 
       5.0424492771427094e-05, 
       5.2522299843086927e-05, 
       4.8832809213480156e-05, 
       5.0000000000000484e-05, 
       4.9411935334493937e-05, 
       4.9914667875922268e-05, 
       4.9441445165295810e-05, 
       5.4622423595324378e-05, 
       4.8517591409920700e-05, 
       5.3213438709148877e-05, 
       5.5632659828300991e-05, 
       4.8779028130356905e-05, 
       4.8226885575954597e-05, 
       4.7290604197927614e-05, 
       4.6322731955791945e-05, 
       6.1019334132064444e-05, 
       4.5610460412600764e-05, 
       4.6810932600579872e-05, 
       4.9432863350380618e-05, 
       6.1165618259026536e-05, 
       4.5239111913056748e-05, 
       4.9572770479235370e-05, 
       4.9567767780568799e-05, 
       6.2071041454853299e-05, 
       4.6065029709778942e-05, 
       4.5520660575144482e-05, 
       4.5641214366622427e-05, 
       4.4499774160447670e-05, 
       4.3388086270669392e-05, 
       4.9999999999999616e-05, 
       4.5490932624958557e-05, 
       5.5949191173713798e-05, 
       4.4407426305429539e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       5.6526734049517311e-05, 
       4.3722767955524346e-05, 
       4.3375138364128834e-05, 
       4.3353996364705500e-05, 
       4.6151747179215837e-05, 
       4.2885466563169962e-05, 
       4.9198988001138746e-05, 
       4.2807671337071456e-05, 
       4.2322456366005044e-05, 
       4.1006575193870757e-05, 
       4.4701650541767954e-05, 
       4.6185945198740273e-05, 
       3.9999999999997374e-05, 
       5.0273782151808583e-05, 
       5.1130186863394855e-05, 
       3.9999999999998106e-05, 
       4.1462095936688671e-05, 
       4.1577067602735772e-05, 
       4.9999999999999745e-05, 
       4.4309398436484000e-05, 
       5.5563110389812007e-05, 
       4.2614590876150446e-05, 
       3.9999999999998567e-05, 
       4.0188488893225141e-05, 
       5.4858757344799994e-05, 
       3.9500366384842609e-05, 
       4.4385849642023203e-05, 
       3.9999999999997957e-05, 
       3.9721569863311231e-05, 
       5.3549060009056154e-05, 
       4.2551218088839800e-05, 
       4.2390417927403567e-05, 
       3.8114998318829779e-05, 
       3.9481352883082236e-05, 
       3.7381668111036079e-05, 
       4.0000000000001867e-05, 
       3.7804824580582850e-05, 
       4.9897913969667595e-05, 
       3.7323918002612622e-05, 
       3.7074415561541644e-05, 
       3.8818932106846331e-05, 
       4.0160781085369481e-05, 
       3.9999999999998025e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       3.7086545818127610e-05, 
       4.1514868637532843e-05, 
       3.9203684999759020e-05, 
       3.5545170892441466e-05, 
       3.6244032488950672e-05, 
       4.7559060300528307e-05, 
       4.0000000000001630e-05, 
       4.0000000000001630e-05, 
       3.8469172159509515e-05, 
       3.6772492194250403e-05, 
       3.6498133237025635e-05, 
       4.5047886036370986e-05, 
       3.5523438895331447e-05, 
       3.7895942419542980e-05, 
       3.5135503375330759e-05, 
       3.8998510028733821e-05, 
       4.0686628072756452e-05, 
       4.0654927071157235e-05, 
       3.4080557107344325e-05, 
       3.4048217492594074e-05, 
       3.5203819138706939e-05, 
       3.7851728154034492e-05, 
       3.3698259039289606e-05, 
       3.4594701395675658e-05, 
       3.6435466527443812e-05, 
       3.3486346346068847e-05, 
       3.4733895007143607e-05, 
       3.1644301930794078e-05, 
       3.2550327773817797e-05, 
       3.9999999999997699e-05, 
       3.7487412107968229e-05, 
       3.3483136199464865e-05, 
       3.1361564915572966e-05, 
       3.3359073133855373e-05, 
       3.5013747517300329e-05, 
       3.2113825450448902e-05, 
       3.2997684166789597e-05, 
       3.4415338026954451e-05, 
       3.1403866066294734e-05, 
       4.1820979318686044e-05, 
       3.1179453632449354e-05, 
       4.1412175602386598e-05, 
       4.0118335767405143e-05, 
       3.3821636682938705e-05, 
       3.3793087846911833e-05, 
       3.3812847297667250e-05, 
       2.9999999999998439e-05, 
       3.0014896462251848e-05, 
       3.2965804877890337e-05, 
       4.0884159708154115e-05, 
       3.0000000000000024e-05, 
       3.5280730711421344e-05, 
       2.9999999999995623e-05, 
       2.9968480986509737e-05, 
       3.5657404605298736e-05, 
       2.9715071549215198e-05, 
       3.1546329800869204e-05, 
       3.0000000000001004e-05, 
       4.0736730489329173e-05, 
       4.0816164569913127e-05, 
       3.0000000000001136e-05, 
       3.3299638964219901e-05, 
       3.3299638964219901e-05, 
       3.0457422680714119e-05, 
       3.0864077802037112e-05, 
       2.9386880885571449e-05, 
       2.9380218081358039e-05, 
       3.5202693395530846e-05, 
       3.2847351552011229e-05, 
       2.9999999999998808e-05, 
       3.0136650401193868e-05, 
       3.0000000000001891e-05, 
       2.9689739846549140e-05, 
       2.8773922855604120e-05, 
       3.3700495527407158e-05, 
       3.8773547281792445e-05, 
       2.9999999999995491e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.9504829693468627e-05, 
       3.1272117616158080e-05, 
       2.8862227430549798e-05, 
       3.1291484810656107e-05, 
       3.9191897971157215e-05, 
       2.7977630063447355e-05, 
       2.7977630063447355e-05, 
       2.9999999999998578e-05, 
       2.8089975368071030e-05, 
       2.9185216845573686e-05, 
       2.8927608788804419e-05, 
       2.8151518352048091e-05, 
       2.7011750800167191e-05, 
       3.1465009182393138e-05, 
       3.2173722130937267e-05, 
       2.9919960533406326e-05, 
       2.9580842541345168e-05, 
       2.6692244642173586e-05, 
       2.7019188599007298e-05, 
       2.6846571113176940e-05, 
       2.8204457784433526e-05, 
       3.7440317362358582e-05, 
       2.8659394650376446e-05, 
       2.7038416658924865e-05, 
       2.9335476415507328e-05, 
       3.5522153832264638e-05, 
       2.9999999999998395e-05, 
       2.7210311059071359e-05, 
       2.6074942955201523e-05, 
       2.6297383547147306e-05, 
       2.9782364454003838e-05, 
       3.2457358833389846e-05, 
       2.8472570612034648e-05, 
       3.1872929181959594e-05, 
       2.7131565796937800e-05, 
       2.7971472503886519e-05, 
       2.7832361599668259e-05, 
       2.7832361599668259e-05, 
       2.5737932320108286e-05, 
       3.0000000000001793e-05, 
       2.6199921818010489e-05, 
       2.5682484190201157e-05, 
       2.9265385385010686e-05, 
       2.6228294859266784e-05, 
       2.5236763805001472e-05, 
       3.0046823587142986e-05, 
       2.5592180923732137e-05, 
       2.5356010336502280e-05, 
       2.5122134504469778e-05, 
       2.9999999999998110e-05, 
       2.7500125045985584e-05, 
       2.5584307744602768e-05, 
       3.2637883499349776e-05, 
       3.2657745312255911e-05, 
       2.4633513126311541e-05, 
       2.8998943625240472e-05, 
       2.5773881605407260e-05, 
       2.6807179885281498e-05, 
       2.4477250189912038e-05, 
       2.4371838282645147e-05, 
       2.5917938372508873e-05, 
       2.4535058597207485e-05, 
       2.4285911235302098e-05, 
       2.5317669587790486e-05, 
       2.5462588311319546e-05, 
       2.3941240359989158e-05, 
       2.3800505219905217e-05, 
       2.4961550181028645e-05, 
       2.4226066300122850e-05, 
       2.4226066300122850e-05, 
       2.3569946573050225e-05, 
       2.9256067872345432e-05, 
       2.9999999999996714e-05, 
       3.0742484125328481e-05, 
       2.5134000662371254e-05, 
       2.3306748978850969e-05, 
       2.5264107776272909e-05, 
       3.0492415851752670e-05, 
       2.7623986466856209e-05, 
       2.7343805356189461e-05, 
       2.6356052978239335e-05, 
       2.9797929900666918e-05, 
       2.4200857469102265e-05, 
       2.7367378383460666e-05, 
       3.1088971422759722e-05, 
       2.2556861236520565e-05, 
       2.3571041255271092e-05, 
       2.2427889211832239e-05, 
       2.7014212896991393e-05, 
       2.7137824175701920e-05, 
       2.2097546107119309e-05, 
       2.4573600077090218e-05, 
       2.6185930649968901e-05, 
       2.2131240997308618e-05, 
       3.0000000000003491e-05, 
       2.4382405017780020e-05, 
       2.2917351752852338e-05, 
       2.1993540383034172e-05, 
       2.4753475601988021e-05, 
       2.7106587321094513e-05, 
       2.7096223107290237e-05, 
       2.2311562620296933e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.9191556212557046e-05, 
       2.3708407126237797e-05, 
       2.3632543720058086e-05, 
       2.2178655585035640e-05, 
       2.1260303009147572e-05, 
       2.0751564665685468e-05, 
       2.7123291419799243e-05, 
       2.1293597600214950e-05, 
       2.7072748310821861e-05, 
       2.1259004476358883e-05, 
       2.4840474607486825e-05, 
       2.0977104654190915e-05, 
       2.0426691562162788e-05, 
       2.0575742866782545e-05, 
       2.7809262755418969e-05, 
       1.9999999999997217e-05, 
       2.7639306492202163e-05, 
       2.0000000000000588e-05, 
       2.2563138035948259e-05, 
       2.6665080149674556e-05, 
       2.0223797299616227e-05, 
       1.9864695338257338e-05, 
       2.6719377696700612e-05, 
       1.9999999999999548e-05, 
       2.2295558324945083e-05, 
       1.9999999999997274e-05, 
       2.2481410309963971e-05, 
       2.0182826491743358e-05, 
       2.1845276102154460e-05, 
       1.9639347126474737e-05, 
       1.9948019575927538e-05, 
       1.9408611324717893e-05, 
       2.3216819158676505e-05, 
       2.0846856718329786e-05, 
       2.0572601516417116e-05, 
       2.1603384716430804e-05, 
       2.2327988619413193e-05, 
       2.3107947036613048e-05, 
       1.9659599877273180e-05, 
       2.0000000000001469e-05, 
       2.6563136895194646e-05, 
       2.3651313797591950e-05, 
       1.9808176914537860e-05, 
       2.3612171717668531e-05, 
       1.9186151206555596e-05, 
       1.9479667288488301e-05, 
       2.6406639289927003e-05, 
       2.6422711248488157e-05, 
       1.9603888455501442e-05, 
       1.9322708244094596e-05, 
       1.9304301195852198e-05, 
       1.8966502313068078e-05, 
       2.1413140376842291e-05, 
       1.8885929402898659e-05, 
       1.9478075949216492e-05, 
       1.8997342556295698e-05, 
       1.9999999999998633e-05, 
       1.9412913706816398e-05, 
       1.8730084639634875e-05, 
       2.1030775585407879e-05, 
       2.0000000000004596e-05, 
       1.8805715509531297e-05, 
       1.8681991499424406e-05, 
       2.2313943334219730e-05, 
       1.8838987389853461e-05, 
       1.8991950648419524e-05, 
       2.4706036334314775e-05, 
       2.4699738671751328e-05, 
       1.8579274982661298e-05, 
       1.9826184433531350e-05, 
       2.0534360716990844e-05, 
       2.0993626785568987e-05, 
       2.2166591464208083e-05, 
       1.8359136074659026e-05, 
       2.5010955169093653e-05, 
       2.4129536331827622e-05, 
       2.2370575466761592e-05, 
       2.0000000000003234e-05, 
       1.8806277083723237e-05, 
       1.8467457477055713e-05, 
       1.9999999999999256e-05, 
       1.9999999999999256e-05, 
       2.0768642561885622e-05, 
       2.0000000000004098e-05, 
       1.7867416925220239e-05, 
       1.9999999999999012e-05, 
       2.0231731409536032e-05, 
       1.8182569317993935e-05, 
       2.1293705580291823e-05, 
       2.2836971537309227e-05, 
       2.2836971537309227e-05, 
       2.0008703215793480e-05, 
       1.9466270069855385e-05, 
       1.7519970751792677e-05, 
       2.4207997163155264e-05, 
       1.7991832038449499e-05, 
       1.9356879194243652e-05, 
       1.7826185487479829e-05, 
       1.7353968538952078e-05, 
       2.0000000000001177e-05, 
       2.3787675039106025e-05, 
       1.7309993086786362e-05, 
       2.0000000000002319e-05, 
       1.7139749191630588e-05, 
       1.9287464289845343e-05, 
       1.8062383567360180e-05, 
       2.2267660896859265e-05, 
       1.9999999999996559e-05, 
       2.0008378646672091e-05, 
       1.8804112972805669e-05, 
       1.6764587150742134e-05, 
       1.6669873460121549e-05, 
       1.7205393383691882e-05, 
       1.6931509532418365e-05, 
       1.6607785766890799e-05, 
       2.1880392553228619e-05, 
       1.7081082427052981e-05, 
       1.6969166942220725e-05, 
       1.7715228276346213e-05, 
       1.9963562514332652e-05, 
       2.2554646420987262e-05, 
       1.6758979672385730e-05, 
       1.6873519010243131e-05, 
       1.6349496012902706e-05, 
       1.8510212586384688e-05, 
       1.5938319162468304e-05, 
       1.6984179487034477e-05, 
       1.7788024916872645e-05, 
       1.6001874200859676e-05, 
       1.6743649814993181e-05, 
       1.7205551348619966e-05, 
       1.7885452909703484e-05, 
       1.6625200370722534e-05, 
       1.6373809606661409e-05, 
       1.6627832668215887e-05, 
       1.5976866250368946e-05, 
       1.9999999999996783e-05, 
       1.7363674046188015e-05, 
       1.6083037376418131e-05, 
       1.6343114636448679e-05, 
       1.8343407491848204e-05, 
       1.6925594702878127e-05, 
       2.0454950671941970e-05, 
       1.5686002403094944e-05, 
       1.5838518495045479e-05, 
       1.7486940480617817e-05, 
       1.9883389265775696e-05, 
       1.6327109040235535e-05, 
       2.0424938883841393e-05, 
       2.0427807483103429e-05, 
       1.5644484327526850e-05, 
       1.5867501230803267e-05, 
       1.5110751322616457e-05, 
       2.0000000000003580e-05, 
       1.7734450653983901e-05, 
       2.0134039715363126e-05, 
       2.0058162731625443e-05, 
       1.4624414869076079e-05, 
       1.5230245245118650e-05, 
       1.5085591842953761e-05, 
       1.5262591415855119e-05, 
       1.4564744570149252e-05, 
       1.7043605231382382e-05, 
       1.4589690644569201e-05, 
       1.4451209281903315e-05, 
       1.6149680792502952e-05, 
       1.8768797894612757e-05, 
       1.4374725904905995e-05, 
       1.5533388178468646e-05, 
       1.6532466198151173e-05, 
       1.5960653163557965e-05, 
       1.4355958387083369e-05, 
       1.4373606616565149e-05, 
       1.4898311729491209e-05, 
       1.5271969420058310e-05, 
       1.4278026984889705e-05, 
       1.5844494624281379e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.4160578560629806e-05, 
       1.6910400558404366e-05, 
       1.6848394678110017e-05, 
       1.7512948292476493e-05, 
       1.9141804144763431e-05, 
       1.5210383125229497e-05, 
       1.6827791530076173e-05, 
       1.4908028976569549e-05, 
       1.7143200018554293e-05, 
       1.3933984783339975e-05, 
       1.3911760376041158e-05, 
       1.4494828744928540e-05, 
       1.7697799065676122e-05, 
       1.4342058135619077e-05, 
       1.6863656364408230e-05, 
       1.6863656364408230e-05, 
       1.6681432592032161e-05, 
       1.3857273068689528e-05, 
       1.4292768698319954e-05, 
       1.3529282455952913e-05, 
       1.3492697305830638e-05, 
       1.5720480371830671e-05, 
       1.3714430654745951e-05, 
       1.4296436309647095e-05, 
       1.5712067622906465e-05, 
       1.4199112071642915e-05, 
       1.3623509582802912e-05, 
       1.7851914031471932e-05, 
       1.3667926439330561e-05, 
       1.6934056376821289e-05, 
       1.3197982690463027e-05, 
       1.3198984372531700e-05, 
       1.7281585869567818e-05, 
       1.4664548460781849e-05, 
       1.5365138404873848e-05, 
       1.3091265937794375e-05, 
       1.3487468102635314e-05, 
       1.3718333684757451e-05, 
       1.3211014606491409e-05, 
       1.3203394544383078e-05, 
       1.5328591659589319e-05, 
       1.2836279153562491e-05, 
       1.3595715093240521e-05, 
       1.7227964339966451e-05, 
       1.2613423965688773e-05, 
       1.3400348495384164e-05, 
       1.3141561166335034e-05, 
       1.2842273245148677e-05, 
       1.2594092440623859e-05, 
       1.2931833999465078e-05, 
       1.7021062370777087e-05, 
       1.3611366772337191e-05, 
       1.7233826626863242e-05, 
       1.2775253827489913e-05, 
       1.2775201272766793e-05, 
       1.2500637403403971e-05, 
       1.2418051838696583e-05, 
       1.2517041137607847e-05, 
       1.2572177143160616e-05, 
       1.2523782300879124e-05, 
       1.2636546356090033e-05, 
       1.4330806605078814e-05, 
       1.3659503941344594e-05, 
       1.2193318101691071e-05, 
       1.3655586328179575e-05, 
       1.3570150463719608e-05, 
       1.4457535782031519e-05, 
       1.3184220984711498e-05, 
       1.4747343942508893e-05, 
       1.4167866443020033e-05, 
       1.3379902545627577e-05, 
       1.3983982122698263e-05, 
       1.5124414135593321e-05, 
       1.4866846025021259e-05, 
       1.2566840745002747e-05, 
       1.4164853489324051e-05, 
       1.4131523120344591e-05, 
       1.2145750809411923e-05, 
       1.1686729010294344e-05, 
       1.5021649709840194e-05, 
       1.2559672032698739e-05, 
       1.5863959957828723e-05, 
       1.3760734668291225e-05, 
       1.5602265287517245e-05, 
       1.2046397081482098e-05, 
       1.1554788933308400e-05, 
       1.2024639719371994e-05, 
       1.5793341943915352e-05, 
       1.2763071187150709e-05, 
       1.2035133671108810e-05, 
       1.1619793695329978e-05, 
       1.1997264817923060e-05, 
       1.1645445387257533e-05, 
       1.1257305353587646e-05, 
       1.5141131320754851e-05, 
       1.1255050774316778e-05, 
       1.3068772519681541e-05, 
       1.2499371065885147e-05, 
       1.1328415363346918e-05, 
       1.1287572743084131e-05, 
       1.1309792768699568e-05, 
       1.1210186362251582e-05, 
       1.2814606962227572e-05, 
       1.2328694191987037e-05, 
       1.1259193589328646e-05, 
       1.5607550973367016e-05, 
       1.2391949261508307e-05, 
       1.1398385844271362e-05, 
       1.1578772042345190e-05, 
       1.1342472706858968e-05, 
       1.4065447134165465e-05, 
       1.1095780508374372e-05, 
       1.1567455883813949e-05, 
       1.0919929046678284e-05, 
       1.3410519812722531e-05, 
       1.4244859653893168e-05, 
       1.1172809410242831e-05, 
       1.1367458641059072e-05, 
       1.1202953053180991e-05, 
       1.0778074539157855e-05, 
       1.4527782135003634e-05, 
       1.0811134767029974e-05, 
       1.1799596959139493e-05, 
       1.2085154703886210e-05, 
       1.4355532454623739e-05, 
       1.1754192674693249e-05, 
       1.2645546693696847e-05, 
       1.1789674958006327e-05, 
       1.3386228659374884e-05, 
       1.4836574568707225e-05, 
       1.2551554549025422e-05, 
       1.0834831575800659e-05, 
       1.0595015724654419e-05, 
       1.3233519021148861e-05, 
       1.1067743051788599e-05, 
       1.0739846770578214e-05, 
       1.0905841990328247e-05, 
       1.1429733642794347e-05, 
       1.1555827423784605e-05, 
       1.0410066671676389e-05, 
       1.0367627954712632e-05, 
       1.1361049724090651e-05, 
       1.0305521611141474e-05, 
       1.0876987121034973e-05, 
       1.0290682116939284e-05, 
       1.0817939495735511e-05, 
       1.3487468102631545e-05, 
       1.0144824009627658e-05, 
       1.1341663846566092e-05, 
       1.0132381420194130e-05, 
       1.0430663587230592e-05, 
       1.1248338244145324e-05, 
       1.0865447622654365e-05, 
       1.2056174381251658e-05, 
       1.2657480016838313e-05, 
       9.9999999999997789e-06, 
       1.1227612670564297e-05, 
       1.0551922945587939e-05, 
       1.3697644880916474e-05, 
       1.1058724041859068e-05, 
       1.0864171766429975e-05, 
       9.9999999999998382e-06, 
       1.0316037602035974e-05, 
       1.1671117261079531e-05, 
       1.3335555846781680e-05, 
       9.9999999999997941e-06, 
       1.3310044361603757e-05, 
       1.0042033974282505e-05, 
       1.0646251574422470e-05, 
       1.2862325661298231e-05, 
       1.0622116247348954e-05, 
       1.0156858739603583e-05, 
       1.1503405839438515e-05, 
       1.0249497591828719e-05, 
       1.0634924780917165e-05, 
       1.1730443311164356e-05, 
       1.1725433434331973e-05, 
       1.0000000000000118e-05, 
       1.0561374453280059e-05, 
       1.0437783631662281e-05, 
       1.0563851329157979e-05, 
       1.0693107046668232e-05, 
       1.0000000000000243e-05, 
       1.1046186347863405e-05, 
       1.0374498060985676e-05, 
       9.9999999999997061e-06, 
       9.9999999999997061e-06, 
       1.0082154656522444e-05, 
       1.2285868583690549e-05, 
       1.1945879343491163e-05, 
       1.0000000000000001e-05, 
       1.1917887384520912e-05, 
       9.9999999999997975e-06, 
       9.9999999999997975e-06, 
       1.0000000000000509e-05, 
       1.0000000000000260e-05, 
       1.0000000000000260e-05, 
       1.0716597322411585e-05, 
       9.9999999999995722e-06, 
       1.0307682682769742e-05, 
       1.0000000000000543e-05, 
       9.9999999999999839e-06, 
       1.1010192905863880e-05, 
       1.0651750099817600e-05, 
       1.0967491772412170e-05, 
       1.0539570125271512e-05, 
       9.9999999999999212e-06, 
       1.0238980082244761e-05, 
       1.0428046361947348e-05, 
       1.0581551584182459e-05, 
       1.0521791774912581e-05, 
       1.0155431128245339e-05, 
       9.9999999999997501e-06, 
       1.0017465930359139e-05, 
       1.1252572208021010e-05, 
       1.0586834029076454e-05, 
       1.0457992382191283e-05, 
       1.0457316590264931e-05, 
       1.0000000000000089e-05, 
       1.1049761317512434e-05, 
       1.1063789652223144e-05, 
       1.0019487777920026e-05, 
       1.0584302017696919e-05, 
       9.9999999999997908e-06, 
       9.9999999999997908e-06, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       7.8647112849786019e-06, 
       5.3918428906441289e-06, 
       4.4779771707978103e-06, 
       4.3382849084542031e-06, 
       1.6832590046073050e-06, 
       1.1677365667767590e-06, 
       6.5464032593233605e-07, 
       5.4489728975999753e-07, 
       3.3057967358587582e-07, 
       4.8700000000000002e-03, 
       2.2800067089002759e-03, 
       1.5000000000000000e-03, 
       1.1999999999999999e-03, 
       1.0800000000000000e-03, 
       8.0000000000000004e-04, 
       2.5432833591343596e-04, 
       2.1000000000000001e-04, 
       1.8000000000000001e-04, 
       1.2614958679615747e-04, 
       1.1834516968595136e-04, 
       1.2999999999999999e-04, 
       1.2000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       9.9112194058782873e-05, 
       6.9919728962168168e-05, 
       6.9902033593255316e-05, 
       5.3717692184977326e-05, 
       6.9999999999999994e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       4.2960813801287238e-05, 
       4.0760731318087878e-05, 
       3.5890118751212446e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.6724848476250366e-05, 
       2.6306839803605131e-05, 
       2.6087328165969658e-05, 
       2.5131372454877677e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.0058244061163230e-05, 
       1.8398914607636817e-05, 
       1.8388331895300730e-05, 
       1.5737160983390079e-05, 
       1.7810150428029185e-05, 
       2.0419037322826659e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.4942522878921874e-05, 
       1.4163911507218445e-05, 
       1.4057837825740613e-05, 
       1.1073221815805074e-05, 
       1.1705376277585410e-05, 
       1.0012521990011182e-05, 
       1.0124078272641884e-05, 
       1.0744121532851539e-05, 
       1.1742659850014562e-05, 
       1.1614462115282566e-05, 
       8.1939742174658654e-06, 
       8.1833717167189554e-06, 
       8.3697321313647369e-06, 
       8.5699736445673027e-06, 
       7.9560357175029567e-06, 
       7.9025234008052000e-06, 
       8.0396465870489176e-06, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       9.1397405083220960e-06, 
       6.9075957633964196e-06, 
       6.8576606267723811e-06, 
       6.9833115219091995e-06, 
       6.6614621542316807e-06, 
       6.6938208965713218e-06, 
       6.4093362080313739e-06, 
       6.0767279160024249e-06, 
       5.9398241637203295e-06, 
       6.6824832367931063e-06, 
       5.5417505020845502e-06, 
       6.8689870309759069e-06, 
       5.7042248057659961e-06, 
       5.2508188458905792e-06, 
       5.6332494780071564e-06, 
       4.8084176849401377e-06, 
       4.9413720078873615e-06, 
       4.7475168603198393e-06, 
       4.7468253712181261e-06, 
       4.4272343159771057e-06, 
       6.0097994586177987e-06, 
       6.0185621807312714e-06, 
       4.3900908058358539e-06, 
       4.4723672820577840e-06, 
       4.4971112910730152e-06, 
       4.4971112910730152e-06, 
       4.3037414762997439e-06, 
       4.1134270887317063e-06, 
       4.7456714143049886e-06, 
       4.6712543900327493e-06, 
       3.9522579793540212e-06, 
       4.0676237858412570e-06, 
       3.6679896208244717e-06, 
       3.7527300772082450e-06, 
       3.3422074151796619e-06, 
       4.1036447154383807e-06, 
       3.1985589266976676e-06, 
       3.1137600123078054e-06, 
       3.0612774917133335e-06, 
       4.0214007049372836e-06, 
       3.1041382358140926e-06, 
       3.7013683248663611e-06, 
       3.6963459549802337e-06, 
       3.0117153426417565e-06, 
       2.9221459434933440e-06, 
       2.9221459434933440e-06, 
       2.8283225857125967e-06, 
       2.8072506991466326e-06, 
       2.5346132034046632e-06, 
       2.7857583100391412e-06, 
       2.6049068870382515e-06, 
       2.5502175464830456e-06, 
       2.4822855690439085e-06, 
       3.2639689889364743e-06, 
       2.3672733741342127e-06, 
       2.2946728631185666e-06, 
       2.2695655757273502e-06, 
       2.2175156504254322e-06, 
       2.3075325410813941e-06, 
       2.5621692738600066e-06, 
       2.0091975056906337e-06, 
       2.0091975056906337e-06, 
       1.9532635649223557e-06, 
       1.9289087801578894e-06, 
       2.4273141307617603e-06, 
       1.9082043643515313e-06, 
       1.8989640274380810e-06, 
       2.5815012181800044e-06, 
       1.8842271482719631e-06, 
       2.1884383784239724e-06, 
       2.0590914335602073e-06, 
       2.5261256645845715e-06, 
       1.9681239688197191e-06, 
       2.4775498732004812e-06, 
       1.8161848238891322e-06, 
       1.9001450765774576e-06, 
       2.2212856497307918e-06, 
       2.3898473569249983e-06, 
       1.8385273170975891e-06, 
       2.2293876524677399e-06, 
       1.7401992889472876e-06, 
       1.6328084843931155e-06, 
       1.6059688306867012e-06, 
       1.7010144948532279e-06, 
       1.5919477905845017e-06, 
       1.6760525371688950e-06, 
       1.6165267374513807e-06, 
       1.5672325901393067e-06, 
       1.5337997115150326e-06, 
       1.6135805964507537e-06, 
       1.5534609798250411e-06, 
       2.1534318918718258e-06, 
       1.6002877817284642e-06, 
       1.8261068926552727e-06, 
       1.5928491425372567e-06, 
       1.6181121056680439e-06, 
       1.4279252508681194e-06, 
       1.5170080367837477e-06, 
       1.4098477658962358e-06, 
       1.4017915266285438e-06, 
       1.4589349832739293e-06, 
       1.4494014483437918e-06, 
       1.4109683044788175e-06, 
       1.4601116561129850e-06, 
       1.4417215955640911e-06, 
       1.3097939538030513e-06, 
       1.7061104736072293e-06, 
       1.6368945523076984e-06, 
       1.3750529019272454e-06, 
       1.3314776790155002e-06, 
       1.3170852450275343e-06, 
       1.2590828352492033e-06, 
       1.2905957631034034e-06, 
       1.2411395480896739e-06, 
       1.2141961975913281e-06, 
       1.2111878545746250e-06, 
       1.6040829801933439e-06, 
       1.2169141386605042e-06, 
       1.5814120166654337e-06, 
       1.5551936569743640e-06, 
       1.2445735250153835e-06, 
       1.1752835618881358e-06, 
       1.1462212954296164e-06, 
       1.1668833706132307e-06, 
       1.5869480697887829e-06, 
       1.1332165260527875e-06, 
       1.1395384443069783e-06, 
       1.1410186168695212e-06, 
       1.4095923973585548e-06, 
       1.4083994918522709e-06, 
       1.1261019248431757e-06, 
       1.1390217500161753e-06, 
       1.2268036979224070e-06, 
       1.1878875660183000e-06, 
       1.1266072137598194e-06, 
       1.3145636716878417e-06, 
       1.1149104293934902e-06, 
       1.0370449872187075e-06, 
       1.0671095468501128e-06, 
       1.0852729903569755e-06, 
       1.1531022000172839e-06, 
       1.1971321246393756e-06, 
       1.0366182832323851e-06, 
       1.3113549650198791e-06, 
       1.2984169074711275e-06, 
       1.0250483239477515e-06, 
       1.2685995482901698e-06, 
       1.1732213973684467e-06, 
       1.0734044952711292e-06, 
       1.0661093224983895e-06, 
       1.0632613970902311e-06, 
       1.0136915340685956e-06, 
       1.1382866286921644e-06, 
       1.1967338272879567e-05, 
       1.6915544728815255e-06, 
       8.0450399999996893e+00, 
       1.5102092344719256e+00, 
       6.3052043386419909e-01, 
       6.3027190683252610e-01, 
       4.5585999999995519e-01, 
       4.1571000000004482e-01, 
       3.2622000000004786e-01, 
       2.9854999999997256e-01, 
       8.5017127084974081e-02, 
       6.9662063044645464e-02, 
       6.7500923034176846e-02, 
       7.9912925338734744e-02, 
       8.3499999999997299e-02, 
       8.0429999999996546e-02, 
       7.3330000000000187e-02, 
       6.6635116671311070e-02, 
       4.9678395767206805e-02, 
       5.2194291683707844e-02, 
       5.0700430514062853e-02, 
       4.5980652489027940e-02, 
       3.8852677198868368e-02, 
       4.6380000000001399e-02, 
       3.7124711441699952e-02, 
       3.4829184944516471e-02, 
       4.2141964380685744e-02, 
       3.9779999999998511e-02, 
       2.8836871031646834e-02, 
       2.7346917105562352e-02, 
       2.4978969253369728e-02, 
       2.4977696134518040e-02, 
       3.0450519695192545e-02, 
       3.0947418055604470e-02, 
       2.1811858986225948e-02, 
       2.1778154529048611e-02, 
       2.3375291041355436e-02, 
       2.3361999715268939e-02, 
       2.8643268490474744e-02, 
       2.4040000000004950e-02, 
       2.3595756163855980e-02, 
       2.0124697541815798e-02, 
       2.2466902821482896e-02, 
       1.9141297923069737e-02, 
       2.1953424469820258e-02, 
       1.6443603972986810e-02, 
       2.0054726855769162e-02, 
       1.9991836623552660e-02, 
       1.3924441871189698e-02, 
       1.4076527427018740e-02, 
       1.3647499814346299e-02, 
       1.6491127339534124e-02, 
       1.7440000000002398e-02, 
       1.2845599386154885e-02, 
       1.3889626849855230e-02, 
       1.6470000000001046e-02, 
       1.2159077284451475e-02, 
       1.3493711459941075e-02, 
       1.2053011854260717e-02, 
       1.3900026092923670e-02, 
       1.1031298302007402e-02, 
       1.4074660869092722e-02, 
       1.4049999999999344e-02, 
       1.2305848126526638e-02, 
       1.0211681724059483e-02, 
       9.2600269409717634e-03, 
       9.3557877619183331e-03, 
       8.4520279274782010e-03, 
       8.4509147918548615e-03, 
       9.8590644489637350e-03, 
       1.0050000000002541e-02, 
       8.3249023131672702e-03, 
       9.0144032342984026e-03, 
       9.1013728644659415e-03, 
       7.7804818291646247e-03, 
       9.1173241123075552e-03, 
       7.6201730727199915e-03, 
       7.6197843017261502e-03, 
       6.5837328261816423e-03, 
       7.5414804467914117e-03, 
       8.3806000885695165e-03, 
       7.4728497163251955e-03, 
       6.3310780340135675e-03, 
       7.9638366959054671e-03, 
       6.1535124088335823e-03, 
       8.4587816087514575e-03, 
       6.2500951706438617e-03, 
       6.4252824928326315e-03, 
       6.8868022816102473e-03, 
       6.8856163150056438e-03, 
       6.2109705881950921e-03, 
       8.3400000000003350e-03, 
       8.3000585701522769e-03, 
       6.1596087744463131e-03, 
       7.9771728179224496e-03, 
       8.0877797029879801e-03, 
       7.9820918119618643e-03, 
       6.8062629712314464e-03, 
       7.6402262249203236e-03, 
       7.8319711677719370e-03, 
       6.4921524938037697e-03, 
       6.0325452824095639e-03, 
       6.0271868255321074e-03, 
       7.1834222111314197e-03, 
       6.1547079139979491e-03, 
       7.0808702667713346e-03, 
       6.8925081939619458e-03, 
       6.8415168030109960e-03, 
       5.6439778298486322e-03, 
       6.7199999999998433e-03, 
       6.5578329184788862e-03, 
       6.5475312541876645e-03, 
       6.4187950962505758e-03, 
       6.3322794557790418e-03, 
       6.2315328954581422e-03, 
       6.1956397940844208e-03, 
       4.9134701514221332e-03, 
       4.9087525052121149e-03, 
       4.7416885433351269e-03, 
       5.9146763797479187e-03, 
       6.0446090478922920e-03, 
       4.9863001521728871e-03, 
       5.9663424435565677e-03, 
       5.1005457394913715e-03, 
       5.8900000000002267e-03, 
       5.8406588260202716e-03, 
       4.2029703382669998e-03, 
       5.8360597600046549e-03, 
       5.7370148335486159e-03, 
       4.1503334277825507e-03, 
       4.6475127037543815e-03, 
       4.7051889946399294e-03, 
       4.6435729797841283e-03, 
       5.5537409462860041e-03, 
       5.6299999999999311e-03, 
       5.6200000000003322e-03, 
       5.6108515189776153e-03, 
       5.6100000000000056e-03, 
       4.4833314261677505e-03, 
       4.4787627899237955e-03, 
       5.3070086163460361e-03, 
       4.6249138603349834e-03, 
       5.3186020284332648e-03, 
       4.7218320461153360e-03, 
       5.1768255126553066e-03, 
       5.0752490843158542e-03, 
       4.3664906879989102e-03, 
       5.0047583139203695e-03, 
       4.7872272541596012e-03, 
       4.7795274543678519e-03, 
       4.8753105325883180e-03, 
       3.9667747624743252e-03, 
       4.0834601586299353e-03, 
       4.7194995279174479e-03, 
       3.3633980146553712e-03, 
       4.4986340646213140e-03, 
       4.5383517668322040e-03, 
       4.4200000000004808e-03, 
       4.3699999999996658e-03, 
       3.5632099786965668e-03, 
       3.5265668121967383e-03, 
       4.2399999999997978e-03, 
       2.9920742735207598e-03, 
       4.1937054610074074e-03, 
       2.9648248352641695e-03, 
       3.4805423859042084e-03, 
       3.4459824149985050e-03, 
       3.9812712784127423e-03, 
       4.0490275619448086e-03, 
       3.5147477042229941e-03, 
       3.4951253478704256e-03, 
       3.9001540892946870e-03, 
       3.2734531523206031e-03, 
       3.2828771658553130e-03, 
       3.8102895686142595e-03, 
       3.1029704299248479e-03, 
       3.1019111371112331e-03, 
       2.6763179314531808e-03, 
       2.6742245538194892e-03, 
       3.7699999999996491e-03, 
       3.0661751699877513e-03, 
       3.7199999999999352e-03, 
       3.7093722671759366e-03, 
       2.9620523661433198e-03, 
       2.9610874038662987e-03, 
       2.9464774002881623e-03, 
       2.7729505322125528e-03, 
       2.7618147586587066e-03, 
       2.5935115512648850e-03, 
       3.4737978420013673e-03, 
       2.5432992376284930e-03, 
       2.6059899677907117e-03, 
       2.6059899677907117e-03, 
       3.2000134299412035e-03, 
       2.8336375349085632e-03, 
       3.2767362440520929e-03, 
       2.7599678407250411e-03, 
       3.2600719658559857e-03, 
       3.2899999999996023e-03, 
       2.8127115328946961e-03, 
       2.8063669623972316e-03, 
       2.6117278322433158e-03, 
       3.2200000000002275e-03, 
       2.9400000000004518e-03, 
       2.4305858864021456e-03, 
       3.1500000000000885e-03, 
       2.3385956276788518e-03, 
       2.6762270453347028e-03, 
       3.0084112461197109e-03, 
       2.2642114574429675e-03, 
       3.0039751024900155e-03, 
       2.4940260597677562e-03, 
       2.9566001525902991e-03, 
       2.1609247543580585e-03, 
       2.1588822067784208e-03, 
       2.1473726829793491e-03, 
       2.5334964893740835e-03, 
       2.9300000000000571e-03, 
       2.8725245742743286e-03, 
       2.5123100451131244e-03, 
       2.4659311887304460e-03, 
       2.8782272704443051e-03, 
       2.0513748850137089e-03, 
       2.1017027713759510e-03, 
       2.7211251533056542e-03, 
       2.2670803765853786e-03, 
       2.2670028812015871e-03, 
       2.7737206755488529e-03, 
       2.0580418070640961e-03, 
       2.7600000000003802e-03, 
       1.9750791385266964e-03, 
       2.6761692272419917e-03, 
       2.7299999999995292e-03, 
       2.2178108206497727e-03, 
       2.4600599829471474e-03, 
       2.2157060016125599e-03, 
       2.6399988900983355e-03, 
       1.9429106551148926e-03, 
       1.9033427673979019e-03, 
       2.5543620385740642e-03, 
       1.8579751874508359e-03, 
       1.8553436305305553e-03, 
       2.5799990701061656e-03, 
       2.5700000000003489e-03, 
       1.8854401905715570e-03, 
       2.0939170957528877e-03, 
       2.5312619792720371e-03, 
       2.5260768440158589e-03, 
       2.5200000000001163e-03, 
       2.1511958045741112e-03, 
       2.4192459372183468e-03, 
       2.5028771271624422e-03, 
       2.4995244566824674e-03, 
       2.4218622585549161e-03, 
       2.2669927281225204e-03, 
       2.2699999999996659e-03, 
       2.0398009900639822e-03, 
       2.4394672137045925e-03, 
       2.2939999928823572e-03, 
       2.4332643170941336e-03, 
       2.3427276241295568e-03, 
       1.8258896283203348e-03, 
       1.8056745273500145e-03, 
       2.3899999999999750e-03, 
       2.3819309585425747e-03, 
       1.7283115043241614e-03, 
       1.6895314014568270e-03, 
       1.7740558261157276e-03, 
       1.7080215205394298e-03, 
       2.2558819584066179e-03, 
       1.7847784740755398e-03, 
       1.7467559728304514e-03, 
       1.6955949045712329e-03, 
       1.8880806642052126e-03, 
       2.1661752769415441e-03, 
       2.3049267393778971e-03, 
       1.9180400301056525e-03, 
       2.3174074072395872e-03, 
       1.6528062300360539e-03, 
       2.2401895309149488e-03, 
       2.2832165615322714e-03, 
       1.7204750345019961e-03, 
       2.2135008361028542e-03, 
       2.2700000000004791e-03, 
       1.6051936004990838e-03, 
       1.6039533057537290e-03, 
       1.9096742527073237e-03, 
       2.2266421915729359e-03, 
       1.6298102271186644e-03, 
       1.5678069662617177e-03, 
       1.5655555987052270e-03, 
       1.5681794156946821e-03, 
       1.6038920269342083e-03, 
       2.1393920371074403e-03, 
       2.1800000000003778e-03, 
       1.8598882598486971e-03, 
       2.1580724764917888e-03, 
       1.6188856074503821e-03, 
       2.0927256499066712e-03, 
       2.0513070862608891e-03, 
       2.1200000000002181e-03, 
       2.0906928544715663e-03, 
       2.1100000000002588e-03, 
       1.7309318807075640e-03, 
       1.7158000599232584e-03, 
       1.9206132034569909e-03, 
       1.4966354704157539e-03, 
       2.0600000000001646e-03, 
       1.6871753664911295e-03, 
       2.0473058556835021e-03, 
       1.7099512653058373e-03, 
       2.0289437608360804e-03, 
       2.0176442179995068e-03, 
       1.9181040103220556e-03, 
       1.9366957412522833e-03, 
       1.9461244693220717e-03, 
       2.0063594346941445e-03, 
       1.6628322536005702e-03, 
       1.4698997263811568e-03, 
       1.6236490657578274e-03, 
       1.4590212317863774e-03, 
       1.4234191823931084e-03, 
       1.4855360411583907e-03, 
       1.9783723834961974e-03, 
       1.4203977784325364e-03, 
       1.6186535645447164e-03, 
       1.9034159237977749e-03, 
       1.6075253782971664e-03, 
       1.9289438060777250e-03, 
       1.7203627006369658e-03, 
       1.7197423007509222e-03, 
       1.3878470782072341e-03, 
       1.3866460904434414e-03, 
       1.3772920661365383e-03, 
       1.5167786023666968e-03, 
       1.5160212184099188e-03, 
       1.8746036021060908e-03, 
       1.8699999999997792e-03, 
       1.8071572106252980e-03, 
       1.8483448144917406e-03, 
       1.8146065313403717e-03, 
       1.5347017181191454e-03, 
       1.5288494954866173e-03, 
       1.3203896454805188e-03, 
       1.7826852353935636e-03, 
       1.7875295794420570e-03, 
       1.2699680203323699e-03, 
       1.2558736950941613e-03, 
       1.4685817108934826e-03, 
       1.7581146195617900e-03, 
       1.6275545520797279e-03, 
       1.7321420937262857e-03, 
       1.7188044152816545e-03, 
       1.5637626437821834e-03, 
       1.4736585261326605e-03, 
       1.7142833436007753e-03, 
       1.5898035792796582e-03, 
       1.4101107212682147e-03, 
       1.6828169301036317e-03, 
       1.6359620795286952e-03, 
       1.6691843929568719e-03, 
       1.4207562679303858e-03, 
       1.2044268214813640e-03, 
       1.1928684434281760e-03, 
       1.3741730432671379e-03, 
       1.3743806473975240e-03, 
       1.6236444770857848e-03, 
       1.1573856177432137e-03, 
       1.6154642429390212e-03, 
       1.3085319210469153e-03, 
       1.3319760175267326e-03, 
       1.1613728434288458e-03, 
       1.5093248040852359e-03, 
       1.1757400418186087e-03, 
       1.1827176293615603e-03, 
       1.5926562504339991e-03, 
       1.1322180084913311e-03, 
       1.5800105157753295e-03, 
       1.5818905631670374e-03, 
       1.5315135336496054e-03, 
       1.5500000000002864e-03, 
       1.5140283904400132e-03, 
       1.1206776414070126e-03, 
       1.1284831635992728e-03, 
       1.5137397156679013e-03, 
       1.1320049986655604e-03, 
       1.1305719562407681e-03, 
       1.5050242913759292e-03, 
       1.1243938624824765e-03, 
       1.4091755229554441e-03, 
       1.4091755229554441e-03, 
       1.0359687625384605e-03, 
       1.3188611305463304e-03, 
       1.4499999999998939e-03, 
       1.4440161361697567e-03, 
       1.3169698453123892e-03, 
       1.4109098563604866e-03, 
       1.4320711260814335e-03, 
       1.4137350781321077e-03, 
       1.4026805155315633e-03, 
       1.4051804264805178e-03, 
       1.4000004549437825e-03, 
       1.2456150634575758e-03, 
       1.1917404056854724e-03, 
       1.1469858896632520e-03, 
       1.3799999999996573e-03, 
       1.1451599120823083e-03, 
       1.1208179706300851e-03, 
       1.3330975433635385e-03, 
       1.3355407765091320e-03, 
       1.3187888915508884e-03, 
       1.0013270525405741e-03, 
       1.1568211093476619e-03, 
       1.3199999999995132e-03, 
       1.3116782654570126e-03, 
       1.0109559475397340e-03, 
       1.1138205452454876e-03, 
       1.0832716574282366e-03, 
       1.2836457932664052e-03, 
       1.1557278458711781e-03, 
       1.0656307882811553e-03, 
       1.0653787308962750e-03, 
       1.2769410555455612e-03, 
       1.2469323329052427e-03, 
       1.0389232051345304e-03, 
       9.2531631016251181e-04, 
       1.0130478732382282e-03, 
       1.0129401730177223e-03, 
       1.2085776073983202e-03, 
       9.2520552122620230e-04, 
       8.7107551093525582e-04, 
       1.0512642559133106e-03, 
       9.2004712002292715e-04, 
       8.7727090406908943e-04, 
       1.0097040623480615e-03, 
       8.7354061917720041e-04, 
       9.8890871541422194e-04, 
       1.1727329917061958e-03, 
       8.4796485652031462e-04, 
       9.0037526345779152e-04, 
       1.0550721991563637e-03, 
       1.1700750449994070e-03, 
       1.1326401775287145e-03, 
       8.2180904067922058e-04, 
       9.5107217131404939e-04, 
       1.1399999999995279e-03, 
       8.2644788088203356e-04, 
       1.1327675195473863e-03, 
       8.6408623064426456e-04, 
       8.6408623064426456e-04, 
       9.3652942131664419e-04, 
       8.2724855086805413e-04, 
       1.0516041839648289e-03, 
       8.0509973763748125e-04, 
       1.1000000000003342e-03, 
       1.0979015183471543e-03, 
       8.0626437551830786e-04, 
       7.8806691077271151e-04, 
       9.7914063964809586e-04, 
       7.7558813880571553e-04, 
       1.0780160426095077e-03, 
       9.0637779322945747e-04, 
       7.7149457900131842e-04, 
       9.1219661994809426e-04, 
       1.0580965704999389e-03, 
       9.3999999999999965e-04, 
       1.0475411312257327e-03, 
       7.6071622346851348e-04, 
       7.6601372150003611e-04, 
       8.0071860618742259e-04, 
       8.7154633617450897e-04, 
       1.0051995379508089e-03, 
       7.6208020201374996e-04, 
       1.0411476076641849e-03, 
       9.6000000000002876e-04, 
       1.0418666456897920e-03, 
       1.0400000000003265e-03, 
       1.0052959621780673e-03, 
       8.8354408842634881e-04, 
       8.4520956981134316e-04, 
       1.0085064543444062e-03, 
       7.6534909762047542e-04, 
       8.2795062025050692e-04, 
       8.8274655010465595e-04, 
       7.4596361836866626e-04, 
       8.2827356375065545e-04, 
       9.7036552037568313e-04, 
       7.2799575261560704e-04, 
       9.5167889368277250e-04, 
       8.4307764517166351e-04, 
       9.7627497970236455e-04, 
       7.2150706230905378e-04, 
       8.0628206420315165e-04, 
       8.3900864630724625e-04, 
       9.7000000000000211e-04, 
       6.9891696710727735e-04, 
       8.0235740815248703e-04, 
       9.1204691317772757e-04, 
       7.9601176197354655e-04, 
       7.1172509887164651e-04, 
       7.7772253203650698e-04, 
       9.4047607309143133e-04, 
       9.0170696014470013e-04, 
       7.7315636309325479e-04, 
       8.0173142247866498e-04, 
       8.0659486705388268e-04, 
       9.2117008112883349e-04, 
       8.9428632429355067e-04, 
       7.5046930753465337e-04, 
       7.4932796526593450e-04, 
       7.4008811493377622e-04, 
       7.4008057300323270e-04, 
       7.4755093911109015e-04, 
       9.0999999999997084e-04, 
       7.1176495945039732e-04, 
       7.1165140011339499e-04, 
       6.7660718419984339e-04, 
       6.8866269194694531e-04, 
       8.9999979774096418e-04, 
       7.7499779337206805e-04, 
       7.4779782852409105e-04, 
       6.7849745328580973e-04, 
       7.8543832436192779e-04, 
       6.9672952946551612e-04, 
       7.4332191289190521e-04, 
       8.3268613035583161e-04, 
       6.7278011577212257e-04, 
       6.2332674111311370e-04, 
       6.5887219355865176e-04, 
       8.7172737477500648e-04, 
       7.5499582920487247e-04, 
       6.3780277018099995e-04, 
       6.1838297415487840e-04, 
       6.4839924332826164e-04, 
       6.4708431868200984e-04, 
       6.9910256259398730e-04, 
       6.9849956442391212e-04, 
       6.6668331374206320e-04, 
       8.5999999999997938e-04, 
       6.2367116284347288e-04, 
       6.0916199496203821e-04, 
       7.8433215323384391e-04, 
       7.0742413328175500e-04, 
       6.1857119610164734e-04, 
       7.3547010518451627e-04, 
       8.4639036360295897e-04, 
       8.4750463425420038e-04, 
       8.4000000000003083e-04, 
       6.9167157194570178e-04, 
       8.3821006008085125e-04, 
       5.8528461769237017e-04, 
       5.9634596194179030e-04, 
       6.0942556969476524e-04, 
       6.3601747582323141e-04, 
       7.4999999999998462e-04, 
       6.0242566224755786e-04, 
       7.0571523827042996e-04, 
       6.5215536999790153e-04, 
       7.7331231080491497e-04, 
       7.9000000000000793e-04, 
       7.4646830363452304e-04, 
       7.7876196939172834e-04, 
       5.7496171974028162e-04, 
       5.9761357383430981e-04, 
       5.6155562699090791e-04, 
       7.6700867444750589e-04, 
       7.7267189826444509e-04, 
       7.4260041786850613e-04, 
       7.6906702623812107e-04, 
       6.3896956121410034e-04, 
       6.6107606364882571e-04, 
       6.2016993775670763e-04, 
       6.2013880832658178e-04, 
       5.8338445120051125e-04, 
       6.2865676338615800e-04, 
       6.5526500701197318e-04, 
       7.5283346077168866e-04, 
       6.3284655053611242e-04, 
       6.5418437427937316e-04, 
       6.5418437427937316e-04, 
       6.4625238265299668e-04, 
       6.4026795325958295e-04, 
       6.2303184067540626e-04, 
       5.8465500207235390e-04, 
       5.5927852918626075e-04, 
       5.5455611606629795e-04, 
       5.3854055866391526e-04, 
       7.3817015844026721e-04, 
       6.3375115436815660e-04, 
       5.3485051181018128e-04, 
       6.1408798771561538e-04, 
       6.7247453200508685e-04, 
       7.3437025593731120e-04, 
       5.1909521875237452e-04, 
       5.9095077101115902e-04, 
       5.4256068005737097e-04, 
       6.7000000000002854e-04, 
       7.2040462815967587e-04, 
       6.3684908656681468e-04, 
       5.8911142084574228e-04, 
       5.5659861274247855e-04, 
       6.5563809286661094e-04, 
       5.1743060637528043e-04, 
       5.6417821596793703e-04, 
       7.0999999999999265e-04, 
       5.2870183104463829e-04, 
       5.4072418392741076e-04, 
       5.3852724312237273e-04, 
       6.0380526684405370e-04, 
       5.1456074279196407e-04, 
       6.9633113120495457e-04, 
       4.8727848323658021e-04, 
       6.6530329034951527e-04, 
       6.8217432002491989e-04, 
       5.6131137496598425e-04, 
       5.5792543104689785e-04, 
       5.2083872850192516e-04, 
       4.8700254003741387e-04, 
       6.7000000000000165e-04, 
       4.7866479906005002e-04, 
       5.3589250109373719e-04, 
       6.4381806951675866e-04, 
       5.5606789303066595e-04, 
       5.5606789303066595e-04, 
       6.3989972803039680e-04, 
       4.8294345770109084e-04, 
       4.6383616640867657e-04, 
       6.4075662225117596e-04, 
       5.3774270138911835e-04, 
       4.9699686219286364e-04, 
       4.7389232158706282e-04, 
       6.1701059888090621e-04, 
       6.4034302206218165e-04, 
       5.3476892129095263e-04, 
       6.3564004399259215e-04, 
       4.5177658757417050e-04, 
       4.5904898230094679e-04, 
       4.7352672495366621e-04, 
       6.2127375597302516e-04, 
       5.1799017046544283e-04, 
       4.7194082679770963e-04, 
       6.0433804380131907e-04, 
       4.5799054581540608e-04, 
       5.4163788872034111e-04, 
       6.3005395655346490e-04, 
       5.2150618773877486e-04, 
       5.8000000000003416e-04, 
       4.8004921494631115e-04, 
       4.7951955867804634e-04, 
       5.2429042804136294e-04, 
       6.2781865161840923e-04, 
       5.2000000000000516e-04, 
       5.2126739525711345e-04, 
       4.5469085339590966e-04, 
       5.0594250147680594e-04, 
       5.2842352789031477e-04, 
       5.3116611673165583e-04, 
       4.5698228121702117e-04, 
       6.0999999999995530e-04, 
       6.0822654840751006e-04, 
       5.0706454162519239e-04, 
       4.2931612956913765e-04, 
       5.5812332499745115e-04, 
       4.9600973658683975e-04, 
       4.9586418373057594e-04, 
       5.9880403975352543e-04, 
       5.0097298555063494e-04, 
       4.9867321496820051e-04, 
       4.2584124053027250e-04, 
       4.3544238199404091e-04, 
       5.0192876341085451e-04, 
       5.4899513676598464e-04, 
       4.2469516018298401e-04, 
       4.3743761295145554e-04, 
       4.5238203879695673e-04, 
       4.3442072644832194e-04, 
       5.0378274461263797e-04, 
       4.5699816137679969e-04, 
       4.8374571178774403e-04, 
       4.4730254569496286e-04, 
       5.6738211168987443e-04, 
       5.7918767914023688e-04, 
       5.0938272706205006e-04, 
       4.6278087591704596e-04, 
       5.3999999999996163e-04, 
       4.2393831258550669e-04, 
       4.0855596764195274e-04, 
       5.2868979439029556e-04, 
       5.4057708358920336e-04, 
       4.9241120042939259e-04, 
       5.7037111470895584e-04, 
       5.6936384879611014e-04, 
       5.1482131513020312e-04, 
       5.6114289250650711e-04, 
       4.1658451098365140e-04, 
       4.0267907531323414e-04, 
       4.2163772609149047e-04, 
       5.3964324558564971e-04, 
       5.4100561173851716e-04, 
       4.6636328334147492e-04, 
       3.9883690804463234e-04, 
       4.5861684155700557e-04, 
       4.8066922885953931e-04, 
       4.1297481278256308e-04, 
       4.3681867049981035e-04, 
       4.4312047356383748e-04, 
       4.4292186284246467e-04, 
       4.8315118993635427e-04, 
       5.4799381116081461e-04, 
       4.4534254160209216e-04, 
       5.4999999999997239e-04, 
       4.2824639760355681e-04, 
       4.5353257244750274e-04, 
       5.0000000000004761e-04, 
       5.2581817920284771e-04, 
       3.9100422490278251e-04, 
       4.4885849783463984e-04, 
       4.5118808627533859e-04, 
       4.3864011646667020e-04, 
       5.3896428408220065e-04, 
       4.5356764480798044e-04, 
       5.3591453771710074e-04, 
       3.8461183613504812e-04, 
       3.8872720786660805e-04, 
       3.8035965653608497e-04, 
       5.3237819155267825e-04, 
       3.8794513299381832e-04, 
       3.9023927259511035e-04, 
       4.4268689399803578e-04, 
       3.9158373698325213e-04, 
       3.8072181872957230e-04, 
       5.3000000000003923e-04, 
       4.9049889309418966e-04, 
       5.2286011675979573e-04, 
       5.1242909998845915e-04, 
       4.5145269694994879e-04, 
       3.8474794626108935e-04, 
       4.3860490718389223e-04, 
       4.3716058219328666e-04, 
       3.8764796655494049e-04, 
       5.0979050589011830e-04, 
       4.3883146260156121e-04, 
       3.9923967057894916e-04, 
       4.8024923913759615e-04, 
       4.2488787081819304e-04, 
       4.2487719915837549e-04, 
       4.7999999999997247e-04, 
       3.8239992339520227e-04, 
       3.7995382647102339e-04, 
       5.1898232449788589e-04, 
       5.1273549010162091e-04, 
       3.6810298164892337e-04, 
       5.0414366740333027e-04, 
       3.8577085168650497e-04, 
       3.7700343951807010e-04, 
       4.5489825687109240e-04, 
       3.6543362656160872e-04, 
       4.7307774905430553e-04, 
       3.8202835929834300e-04, 
       4.2419115161969800e-04, 
       3.8031360246802600e-04, 
       5.0999999999999429e-04, 
       3.6863258737530695e-04, 
       3.5694593331547374e-04, 
       3.5609324961655457e-04, 
       3.6987206428839523e-04, 
       4.6375729871233853e-04, 
       4.9540420375800804e-04, 
       4.9647582897080283e-04, 
       3.6057957445622405e-04, 
       3.7072925946169159e-04, 
       3.7005288269498411e-04, 
       4.4672901027357679e-04, 
       4.9526950035526363e-04, 
       4.0130925590775823e-04, 
       3.6323734565869110e-04, 
       3.9902159754255960e-04, 
       3.6105441012942898e-04, 
       4.1924424522192511e-04, 
       4.0748733236215174e-04, 
       4.9000000000003143e-04, 
       4.8999999999996930e-04, 
       4.9000000000000595e-04, 
       3.5737580086504989e-04, 
       4.1506143629176762e-04, 
       3.9203938955698401e-04, 
       3.6111706270897489e-04, 
       4.3626597159182069e-04, 
       4.6157603400965913e-04, 
       3.9550710423398279e-04, 
       4.7999999458706781e-04, 
       4.8000000000003557e-04, 
       3.8651919252167692e-04, 
       4.3928279098575968e-04, 
       4.4000000000000419e-04, 
       4.4000000000003449e-04, 
       4.7493012832872686e-04, 
       4.2844347309980226e-04, 
       3.8275311827265382e-04, 
       4.7189005332795857e-04, 
       4.7164345940869164e-04, 
       3.4372634755835068e-04, 
       3.4164402379399666e-04, 
       4.6132198636922371e-04, 
       3.4161690903960801e-04, 
       4.3358937588831506e-04, 
       3.8443412995188027e-04, 
       3.5547127231329837e-04, 
       4.5504838680506947e-04, 
       3.6098603967025546e-04, 
       3.9473291192216161e-04, 
       3.8233349229818784e-04, 
       4.5749890781806207e-04, 
       3.6843319166199621e-04, 
       3.7929391348948306e-04, 
       3.6808291450303720e-04, 
       3.3524747217176978e-04, 
       3.3282655560692824e-04, 
       3.7869919537842370e-04, 
       4.4911548805009544e-04, 
       4.4936167761971241e-04, 
       3.4531661765942943e-04, 
       3.8385730787900190e-04, 
       3.6209485379078637e-04, 
       3.2839161185861454e-04, 
       4.4390645020852800e-04, 
       4.1225084259223922e-04, 
       4.4186487652001465e-04, 
       3.5955632244303139e-04, 
       4.2834178091091893e-04, 
       3.7070879794726781e-04, 
       4.3826296465373783e-04, 
       4.2341928205127249e-04, 
       4.3776596361037222e-04, 
       4.3676254907322459e-04, 
       3.6599824827797611e-04, 
       3.6086455042784942e-04, 
       3.6403029325719663e-04, 
       3.4000000000002968e-04, 
       3.8948236290775384e-04, 
       3.8079669069189721e-04, 
       3.1174520041760085e-04, 
       3.3770390625988866e-04, 
       3.5948608948153710e-04, 
       3.7940862248331920e-04, 
       3.8845959770774800e-04, 
       3.5628376498476742e-04, 
       3.1414703326371895e-04, 
       3.5114822697561812e-04, 
       3.5114822697561812e-04, 
       4.2356415450734846e-04, 
       3.7387527627911793e-04, 
       3.7348158308752526e-04, 
       3.4722116638193605e-04, 
       3.6814779390133333e-04, 
       3.0452945398203100e-04, 
       3.4589305946926453e-04, 
       3.3498402199729182e-04, 
       3.3445257862680088e-04, 
       4.1999999999998945e-04, 
       4.1999999999997958e-04, 
       4.2000000000001601e-04, 
       3.4914441802883292e-04, 
       4.1713888958976215e-04, 
       4.0033494202084774e-04, 
       3.1410039272413841e-04, 
       4.1628362137998846e-04, 
       4.1838938144434537e-04, 
       3.8723641641423258e-04, 
       4.1688464004266862e-04, 
       3.0171898838629374e-04, 
       3.5000000000000477e-04, 
       4.1374349887792194e-04, 
       3.7999999999997270e-04, 
       3.4234229013904233e-04, 
       3.0829218744575661e-04, 
       4.0999999999997782e-04, 
       3.9984979764024441e-04, 
       3.9586787986204037e-04, 
       3.4611775765024133e-04, 
       3.0000114426819363e-04, 
       2.9347441592801751e-04, 
       2.9333214380561340e-04, 
       4.0267053205293089e-04, 
       3.6999999999999506e-04, 
       3.7000000000002325e-04, 
       3.6999999999998720e-04, 
       2.9554856373002005e-04, 
       3.0875466973137258e-04, 
       3.9400650488012655e-04, 
       3.0263571048639280e-04, 
       3.4399682695572135e-04, 
       3.9503070368061176e-04, 
       3.3882167801917883e-04, 
       2.9024380978747268e-04, 
       3.9081448462846075e-04, 
       2.8330351599141046e-04, 
       3.0065374635551874e-04, 
       2.8782807808483743e-04, 
       2.8782807808483743e-04, 
       3.8999999999997538e-04, 
       3.8975268340873720e-04, 
       2.7657660253066761e-04, 
       3.3635034015515135e-04, 
       3.2889529318056397e-04, 
       3.8310041747387398e-04, 
       3.8284563450906333e-04, 
       3.6990458707474060e-04, 
       3.0733204285691735e-04, 
       2.7063659493299110e-04, 
       3.8000000000002398e-04, 
       3.1630643747690375e-04, 
       2.7524028809268545e-04, 
       2.7507386173293298e-04, 
       3.7162683182224477e-04, 
       3.7560967609000294e-04, 
       2.7576480115788809e-04, 
       3.2109099241918161e-04, 
       3.7031400750790245e-04, 
       3.7298931038614225e-04, 
       3.6833530005254777e-04, 
       2.9646962536296323e-04, 
       3.6953876353780702e-04, 
       3.1848441839320480e-04, 
       3.4031708862045801e-04, 
       2.7080569475716255e-04, 
       3.5202574209943155e-04, 
       2.9810565621907914e-04, 
       3.0482734539511168e-04, 
       2.9945747846631907e-04, 
       3.6151706030215124e-04, 
       2.8030256357470856e-04, 
       3.0099038408159298e-04, 
       2.7175455819719439e-04, 
       2.5932678514569433e-04, 
       3.4533163964025633e-04, 
       3.6000000000001954e-04, 
       2.9853991092599263e-04, 
       3.5500341493261176e-04, 
       2.5349334466824081e-04, 
       2.8686075247475314e-04, 
       2.9614429454243762e-04, 
       3.5547419360323717e-04, 
       2.9393008264281200e-04, 
       2.6428554766464243e-04, 
       3.5267558244546365e-04, 
       2.9762504929802604e-04, 
       2.9896630095926642e-04, 
       2.7137088148696919e-04, 
       2.9154971372214318e-04, 
       3.0880268896971949e-04, 
       2.9022085775428323e-04, 
       2.8096988753829064e-04, 
       2.9610753880841004e-04, 
       2.5937830117374913e-04, 
       3.4113505818403030e-04, 
       2.8949570886925184e-04, 
       2.9417077907381020e-04, 
       2.6915660666774143e-04, 
       3.4529126342670028e-04, 
       3.4511364918308411e-04, 
       3.0809286824513431e-04, 
       2.5663372490069349e-04, 
       3.1759241951216774e-04, 
       2.7661106505778052e-04, 
       2.7649016805465868e-04, 
       3.3941507461833969e-04, 
       2.8357716900052280e-04, 
       2.5054341159205660e-04, 
       3.4000000000000013e-04, 
       3.3999999999995742e-04, 
       2.8177568993185178e-04, 
       2.4223639852188515e-04, 
       2.5764080860950374e-04, 
       2.4427156842369739e-04, 
       2.5613184275647907e-04, 
       2.9938226131296319e-04, 
       3.3370959145793174e-04, 
       2.5476719487528814e-04, 
       2.4891847619529635e-04, 
       2.9033317750130878e-04, 
       2.7444747722149939e-04, 
       3.2185706476030871e-04, 
       2.4242484117400007e-04, 
       2.8254690194629088e-04, 
       2.9317112264232691e-04, 
       2.8216246556595951e-04, 
       2.7049093636069099e-04, 
       3.2999999999998433e-04, 
       3.2999999999997582e-04, 
       3.3000000000002971e-04, 
       3.3000000000004483e-04, 
       3.2789231512658475e-04, 
       2.4036255053395375e-04, 
       2.3660186381740538e-04, 
       2.4030449909875185e-04, 
       2.4549169759636714e-04, 
       3.2683780901060201e-04, 
       3.2738532044983943e-04, 
       2.3723403297637845e-04, 
       3.2644933282577088e-04, 
       3.1785194532102525e-04, 
       2.3612481666414564e-04, 
       2.3410145788861126e-04, 
       2.3482993433245864e-04, 
       2.8183502217594785e-04, 
       3.2322526929990156e-04, 
       3.1482433850621872e-04, 
       3.1570987111836387e-04, 
       3.1882896738783298e-04, 
       2.6856965347746162e-04, 
       2.8884576993238096e-04, 
       2.4008857206194383e-04, 
       3.1427355522657029e-04, 
       3.2027583644669041e-04, 
       3.1999999999998935e-04, 
       3.0321715521636693e-04, 
       2.6233083511775432e-04, 
       2.3870567028145707e-04, 
       3.1877332219331310e-04, 
       2.4123841209723344e-04, 
       2.2880551303770062e-04, 
       2.2883793202198413e-04, 
       2.2711519287886344e-04, 
       2.6431168954183946e-04, 
       3.1716431086964609e-04, 
       2.3365084331869923e-04, 
       2.3355344395473780e-04, 
       2.7129457402386305e-04, 
       2.5443679338400436e-04, 
       3.1557299742775082e-04, 
       3.1252900830236035e-04, 
       2.2265480926931077e-04, 
       2.2781199798900566e-04, 
       2.5898677765155405e-04, 
       2.3304964904622530e-04, 
       3.0699700667121110e-04, 
       2.5622203333772051e-04, 
       2.2516631000076644e-04, 
       3.1000000000004863e-04, 
       2.1899598358972997e-04, 
       2.1899598358972997e-04, 
       2.3334138603325351e-04, 
       3.0445221619753766e-04, 
       3.0298084247796109e-04, 
       2.3333393444851950e-04, 
       2.5486657324804720e-04, 
       2.6671463457996733e-04, 
       2.2831799310785541e-04, 
       2.2183253007517417e-04, 
       2.9861872571147508e-04, 
       2.5037606082532765e-04, 
       2.6135870692283120e-04, 
       2.1690328970584946e-04, 
       2.1675790685402860e-04, 
       2.5051814644690487e-04, 
       2.2553144221469621e-04, 
       2.9886517693246926e-04, 
       2.5084167846164913e-04, 
       2.9724777194514045e-04, 
       2.4027399963606001e-04, 
       2.8855562901231117e-04, 
       2.1211869292314073e-04, 
       2.9815237508766155e-04, 
       2.1168566704878465e-04, 
       2.7437553296904551e-04, 
       2.9570155494861017e-04, 
       2.1949512610809906e-04, 
       2.5709683054586381e-04, 
       2.3265927439742606e-04, 
       2.3262218563862365e-04, 
       2.6151263723332729e-04, 
       2.1879088501459518e-04, 
       2.8447186232177525e-04, 
       2.9371318972023185e-04, 
       2.1975926854542907e-04, 
       2.5834853807682776e-04, 
       2.7001806439462853e-04, 
       2.2391482155277457e-04, 
       2.5359642465256607e-04, 
       2.1463434804135225e-04, 
       2.0949856795106304e-04, 
       2.5325963842871466e-04, 
       2.3809827874936770e-04, 
       2.3896853608055085e-04, 
       2.8909781517596063e-04, 
       2.2793457791660845e-04, 
       2.8979724647139609e-04, 
       2.8999999999999751e-04, 
       2.8999999999996585e-04, 
       2.9000000000004304e-04, 
       2.1577862145613148e-04, 
       2.4782435612165235e-04, 
       2.1087283623582510e-04, 
       2.1546311852216378e-04, 
       2.3979577282235096e-04, 
       2.0837903812109704e-04, 
       2.0338287658960952e-04, 
       2.1346508110662552e-04, 
       2.1855062540596573e-04, 
       2.1287748558370426e-04, 
       2.8449846704539715e-04, 
       2.3369250571113616e-04, 
       2.7992038515875208e-04, 
       2.7704503865523262e-04, 
       2.0257242724104076e-04, 
       2.1700753483245558e-04, 
       2.1537074519788882e-04, 
       2.0486715699956560e-04, 
       2.6000000000003028e-04, 
       2.3129838330930130e-04, 
       2.7927487727435796e-04, 
       2.8002083736495647e-04, 
       2.4259068001668036e-04, 
       2.0189564856414673e-04, 
       2.0399450420918643e-04, 
       2.8000000000004399e-04, 
       2.7928398627215560e-04, 
       2.6834742158805651e-04, 
       2.2456476838297252e-04, 
       2.2015279319370770e-04, 
       2.0763818701831612e-04, 
       2.3980716703292300e-04, 
       2.1258144273220034e-04, 
       2.7650281385474307e-04, 
       2.7631049982290874e-04, 
       2.4918966991793653e-04, 
       1.9949652648244300e-04, 
       2.2722677455915040e-04, 
       1.9497406723129672e-04, 
       2.0481037164540500e-04, 
       2.2108456570757863e-04, 
       2.3238995659228244e-04, 
       2.4073002870254632e-04, 
       2.6526766753376857e-04, 
       2.0541980020880791e-04, 
       1.9214717321901396e-04, 
       2.6282405295118845e-04, 
       2.0675763193031598e-04, 
       2.1384371188986169e-04, 
       1.9106212618840197e-04, 
       2.0634557443678474e-04, 
       1.9194061882541716e-04, 
       2.5374895208178763e-04, 
       2.6377788051469521e-04, 
       2.6557726031751475e-04, 
       1.9159598808228350e-04, 
       2.6563262367237977e-04, 
       2.6622867182037242e-04, 
       2.3300779468081247e-04, 
       2.2020161139969677e-04, 
       2.6514003730403586e-04, 
       2.6361744011957474e-04, 
       2.2044611826163189e-04, 
       1.9188158774982578e-04, 
       2.0099014642399938e-04, 
       2.3921355409428383e-04, 
       2.6060257429494664e-04, 
       2.5346220913230130e-04, 
       1.9077696558919460e-04, 
       1.8654035273723578e-04, 
       2.5789951635309317e-04, 
       1.9553402523629171e-04, 
       2.1019155088827362e-04, 
       1.8583124909994129e-04, 
       2.5550658975943195e-04, 
       2.5292089448151301e-04, 
       1.8944632441603875e-04, 
       1.8741703637204199e-04, 
       2.5813604864117953e-04, 
       2.1688515177499426e-04, 
       2.3999999999998499e-04, 
       2.5129581242015796e-04, 
       1.9002063819015930e-04, 
       1.8612775806626922e-04, 
       1.9020275741330617e-04, 
       2.6000000000001114e-04, 
       1.8376209105200252e-04, 
       2.5666539043135266e-04, 
       1.9997808969826329e-04, 
       2.1547510150625839e-04, 
       2.1538095407068917e-04, 
       1.8336556482163681e-04, 
       2.5589098769450441e-04, 
       2.2535987852880950e-04, 
       2.5731342020724189e-04, 
       2.5658361720593556e-04, 
       2.1182995514284922e-04, 
       2.5647495567456132e-04, 
       1.9239215027761565e-04, 
       2.2388373662529689e-04, 
       2.0991239162701126e-04, 
       1.9004968201448793e-04, 
       2.1109732842864134e-04, 
       2.0426233006376991e-04, 
       2.0366585553522798e-04, 
       2.0413824214512901e-04, 
       2.5208417850456521e-04, 
       1.7970480936429269e-04, 
       1.9443304440592836e-04, 
       2.1071307491541316e-04, 
       1.7952606487333492e-04, 
       1.8337310156482436e-04, 
       2.0954430142699367e-04, 
       2.4372762565021887e-04, 
       1.8214499936976508e-04, 
       1.9481771528542636e-04, 
       1.8647348339112134e-04, 
       1.8103218041522570e-04, 
       1.8287528048022979e-04, 
       2.4999995476779509e-04, 
       2.4999999999996190e-04, 
       2.4999999999999339e-04, 
       2.4999999999998759e-04, 
       2.3961996823550861e-04, 
       2.4545490123846307e-04, 
       1.9497580963566768e-04, 
       1.9494362642921616e-04, 
       1.7950289051323813e-04, 
       2.0691986281761604e-04, 
       1.7816701775965750e-04, 
       2.1240691128690574e-04, 
       2.0793414692099344e-04, 
       1.8116439328314353e-04, 
       1.7681929569676296e-04, 
       1.7456698453694310e-04, 
       2.0450666265044376e-04, 
       1.9802104879332080e-04, 
       1.9033625410847809e-04, 
       1.8967057222170946e-04, 
       2.4114777641494735e-04, 
       1.8824503590531382e-04, 
       1.8719283179624669e-04, 
       1.7694008669400584e-04, 
       2.1849841181933264e-04, 
       1.9663030265806616e-04, 
       1.9633210254753068e-04, 
       2.4052158850526936e-04, 
       1.7999999999997944e-04, 
       1.7000680051714351e-04, 
       2.3999999831054937e-04, 
       2.4000000000002296e-04, 
       2.3943850889870426e-04, 
       1.8620077175263819e-04, 
       1.7328681277532958e-04, 
       1.9091808611069764e-04, 
       1.7171059381928050e-04, 
       2.2928177418358244e-04, 
       1.7374685821447850e-04, 
       1.8119808773433584e-04, 
       2.2473925712061986e-04, 
       2.3662297016705509e-04, 
       2.3381954273796020e-04, 
       1.7125750788944525e-04, 
       1.8918019180266379e-04, 
       1.9545364521866338e-04, 
       1.8980489007967589e-04, 
       1.7771797352622728e-04, 
       1.9617738270222024e-04, 
       2.2210182364860943e-04, 
       1.6565575158043771e-04, 
       1.6899180881186325e-04, 
       2.3290350350486623e-04, 
       2.3044753064077787e-04, 
       2.3159524911006907e-04, 
       1.6785474304415306e-04, 
       1.9689253978544933e-04, 
       1.9795310256447206e-04, 
       2.2132255411853915e-04, 
       1.6724228569226502e-04, 
       2.2999999812864339e-04, 
       2.3000000000002630e-04, 
       2.2237444821182145e-04, 
       1.6905309656930210e-04, 
       2.2693221872010415e-04, 
       1.7441377807373418e-04, 
       1.6655291014074013e-04, 
       1.8913219747789100e-04, 
       1.9379285248247649e-04, 
       2.1857880045748424e-04, 
       2.2570077622680382e-04, 
       1.6590376545785364e-04, 
       1.8622764763576868e-04, 
       2.2165792444875539e-04, 
       1.7162089720546868e-04, 
       1.6670253578951065e-04, 
       2.1264704225394088e-04, 
       1.6928004494945866e-04, 
       2.1626508377679105e-04, 
       2.1849553126416480e-04, 
       2.2316133834256421e-04, 
       2.2276873744277748e-04, 
       1.5957002627403554e-04, 
       1.6218527689569072e-04, 
       1.9922763320919011e-04, 
       1.9740658072256258e-04, 
       1.8364995241294433e-04, 
       1.6174137375705517e-04, 
       1.6918321206302954e-04, 
       1.8259169411739872e-04, 
       1.6676738950838812e-04, 
       1.7629318495105125e-04, 
       2.1999999842919999e-04, 
       1.8779527169834919e-04, 
       1.8910548967555318e-04, 
       1.9213389425207111e-04, 
       2.1557540994966449e-04, 
       1.6461390351423238e-04, 
       1.5587036190292371e-04, 
       1.6157027389241378e-04, 
       2.1481618087290823e-04, 
       1.5939889079285088e-04, 
       1.8232594305681648e-04, 
       1.5688520201851360e-04, 
       1.5676321374122462e-04, 
       1.5884008775410822e-04, 
       1.8673210484504343e-04, 
       1.8091731131835657e-04, 
       2.0596839809775206e-04, 
       1.6766738741332818e-04, 
       1.7861009961421464e-04, 
       1.8536883390554281e-04, 
       2.1282397984584276e-04, 
       1.5761894317681789e-04, 
       1.7828276555033470e-04, 
       2.1112015900125400e-04, 
       2.0375196258571638e-04, 
       1.8932112471143885e-04, 
       1.5319434806925058e-04, 
       1.8802999335465617e-04, 
       1.5512132666444398e-04, 
       1.7769463067617662e-04, 
       1.7563036253159019e-04, 
       1.6470366591437183e-04, 
       1.6727161890494976e-04, 
       1.7643082417215570e-04, 
       1.8007489828472906e-04, 
       1.8919683883273532e-04, 
       1.6700318425309687e-04, 
       1.5957975666994539e-04, 
       2.1049161795459571e-04, 
       2.1000000000004470e-04, 
       2.1000000000000001e-04, 
       1.9636548086640710e-04, 
       1.5520732477062579e-04, 
       1.4781218520495913e-04, 
       1.5306876366423720e-04, 
       1.7884138556134357e-04, 
       1.7016700469450033e-04, 
       1.5033757569022353e-04, 
       1.9925899477560508e-04, 
       1.5065458072371574e-04, 
       1.5092659355487014e-04, 
       1.4972260780358988e-04, 
       1.7266766283531037e-04, 
       1.7135449867231289e-04, 
       1.7613260782150424e-04, 
       1.9372920927938445e-04, 
       1.5361834628743401e-04, 
       1.8843971661812553e-04, 
       1.5442910893508768e-04, 
       1.6477192426527719e-04, 
       1.9807154613362733e-04, 
       1.9742417994822686e-04, 
       1.9070567297420501e-04, 
       1.5942528075983599e-04, 
       1.9608375579301121e-04, 
       1.7399927995049861e-04, 
       1.5881595642723009e-04, 
       1.8099227914324678e-04, 
       1.6660376318135381e-04, 
       1.7200648643678738e-04, 
       1.8427288242319785e-04, 
       1.9568369574592194e-04, 
       2.0003952246627155e-04, 
       1.5567366624488952e-04, 
       1.9999999999997402e-04, 
       1.9999999999998548e-04, 
       1.9792172858593848e-04, 
       1.4310096252096249e-04, 
       1.4984179580354047e-04, 
       1.9817181809985834e-04, 
       1.6243934246327064e-04, 
       1.9068964996572972e-04, 
       1.9710637457513990e-04, 
       1.4017792896661152e-04, 
       1.6175486689114766e-04, 
       1.9694412761680837e-04, 
       1.6449598019312948e-04, 
       1.4451102023875325e-04, 
       1.6430428708238764e-04, 
       1.6494121164139716e-04, 
       1.6224089167058133e-04, 
       1.4724545241305841e-04, 
       1.7700221897636373e-04, 
       1.3793220244745120e-04, 
       1.6201190195347465e-04, 
       1.7077752612510639e-04, 
       1.3744505668805614e-04, 
       1.7059975165868726e-04, 
       1.9355173156224126e-04, 
       1.4773166157372117e-04, 
       1.6299668447241464e-04, 
       1.8503016457775020e-04, 
       1.5248417766524760e-04, 
       1.3584577950472484e-04, 
       1.8755090593651656e-04, 
       1.5759592002792308e-04, 
       1.5813081766732521e-04, 
       1.9000000000004072e-04, 
       1.9000000000001714e-04, 
       1.9000000000004682e-04, 
       1.8999999999997995e-04, 
       1.6004593629615139e-04, 
       1.8669587368764319e-04, 
       1.3711202240170283e-04, 
       1.4712002082325982e-04, 
       1.3588851359598421e-04, 
       1.4059354710901909e-04, 
       1.5686488258599232e-04, 
       1.8782843662318118e-04, 
       1.6885789433656608e-04, 
       1.5853135825487006e-04, 
       1.3512641270313269e-04, 
       1.3512641270313269e-04, 
       1.5352199669070514e-04, 
       1.3555064056720956e-04, 
       1.8521666212579883e-04, 
       1.8631503853401530e-04, 
       1.3716155826547303e-04, 
       1.5554189350402072e-04, 
       1.3822028587182194e-04, 
       1.3170787661492017e-04, 
       1.4165160791149682e-04, 
       1.3542087827989903e-04, 
       1.5424922274356079e-04, 
       1.5416425086238890e-04, 
       1.6999999999995225e-04, 
       1.5262635401566943e-04, 
       1.3554706894795349e-04, 
       1.7738487596821574e-04, 
       1.4000000000000628e-04, 
       1.7208637595502314e-04, 
       1.2981732081726060e-04, 
       1.4525078714468031e-04, 
       1.4514597007982876e-04, 
       1.6586658500066900e-04, 
       1.8000000000002926e-04, 
       1.7971978623108569e-04, 
       1.4921811645460435e-04, 
       1.7917078180827833e-04, 
       1.5999421229703585e-04, 
       1.4770219502029539e-04, 
       1.7766611164873254e-04, 
       1.4702906421841302e-04, 
       1.5092865476700032e-04, 
       1.7208584410787677e-04, 
       1.5565886252242534e-04, 
       1.7472641986576156e-04, 
       1.5212853517262404e-04, 
       1.6762051919568026e-04, 
       1.4604840258536795e-04, 
       1.2855306854428510e-04, 
       1.7640171712224201e-04, 
       1.5105182804750640e-04, 
       1.2999999999996931e-04, 
       1.3393090359136434e-04, 
       1.3086388312263361e-04, 
       1.7437735676919882e-04, 
       1.4142957186260652e-04, 
       1.3601883947562049e-04, 
       1.7474304402880636e-04, 
       1.2870575456825636e-04, 
       1.3067794378956220e-04, 
       1.6726170031722580e-04, 
       1.5887090538137578e-04, 
       1.6736339494216186e-04, 
       1.7198108488768876e-04, 
       1.7281531081100190e-04, 
       1.4387618209498135e-04, 
       1.4281745325288834e-04, 
       1.6868632327446585e-04, 
       1.7186198810893914e-04, 
       1.6748077973254883e-04, 
       1.4526695555911202e-04, 
       1.2813652685045304e-04, 
       1.4699179246219638e-04, 
       1.5067871296164465e-04, 
       1.6111899421599097e-04, 
       1.3350497708359164e-04, 
       1.3346974484113109e-04, 
       1.3983871436806233e-04, 
       1.6949711907905522e-04, 
       1.6999999999999099e-04, 
       1.6999999999998806e-04, 
       1.6999999999999809e-04, 
       1.7000000000002267e-04, 
       1.2329989904235158e-04, 
       1.2436923400862665e-04, 
       1.2854304808762446e-04, 
       1.4683006957681496e-04, 
       1.6160435582882870e-04, 
       1.3972255583952578e-04, 
       1.5000000000001365e-04, 
       1.2809105799428647e-04, 
       1.6325830733345122e-04, 
       1.2387870153750044e-04, 
       1.2202344907059997e-04, 
       1.2750426576269410e-04, 
       1.3265229466694804e-04, 
       1.2165809851567457e-04, 
       1.2590904930981378e-04, 
       1.5730482990178394e-04, 
       1.6721244284335744e-04, 
       1.4147442375478847e-04, 
       1.5979258108469221e-04, 
       1.5828763831641872e-04, 
       1.2079998870919375e-04, 
       1.3847140225600735e-04, 
       1.6069811335272437e-04, 
       1.3991956707201230e-04, 
       1.3991956707201230e-04, 
       1.1785429316960505e-04, 
       1.2559121865270815e-04, 
       1.3420901893586887e-04, 
       1.2600707028360226e-04, 
       1.2193087664311440e-04, 
       1.3501593612698566e-04, 
       1.5239991214222232e-04, 
       1.6380710186730566e-04, 
       1.5961613541018601e-04, 
       1.4976541415855974e-04, 
       1.2107075764626484e-04, 
       1.3967694340647830e-04, 
       1.3637746409475310e-04, 
       1.6176536389318850e-04, 
       1.3865760562742341e-04, 
       1.1640969957083990e-04, 
       1.5591417147236815e-04, 
       1.6316491386601618e-04, 
       1.6291805302016873e-04, 
       1.5553617221488657e-04, 
       1.3569625693131957e-04, 
       1.4999999999995179e-04, 
       1.3934932883532909e-04, 
       1.3068547120783893e-04, 
       1.3510168282832339e-04, 
       1.6006687656677202e-04, 
       1.1953472088720430e-04, 
       1.1514603537633790e-04, 
       1.4556648430814588e-04, 
       1.5999999999997166e-04, 
       1.2013553052150146e-04, 
       1.1365076779197476e-04, 
       1.2825640608698591e-04, 
       1.2445859498741413e-04, 
       1.3261308641423491e-04, 
       1.2057430544411213e-04, 
       1.4609306144570863e-04, 
       1.4363448616454966e-04, 
       1.3354653452029715e-04, 
       1.2708441985621207e-04, 
       1.4492435764777885e-04, 
       1.5183513091055403e-04, 
       1.1157066231419675e-04, 
       1.1455643074102387e-04, 
       1.3044841616669255e-04, 
       1.1627946392463456e-04, 
       1.2958060207053849e-04, 
       1.2459594844172186e-04, 
       1.1395629374304224e-04, 
       1.5298074827407043e-04, 
       1.4522461023768444e-04, 
       1.1855328739221928e-04, 
       1.5556219022281689e-04, 
       1.4272621215920974e-04, 
       1.1824397191824585e-04, 
       1.5486374243830391e-04, 
       1.5408379332462090e-04, 
       1.3346898269536072e-04, 
       1.5398272742149797e-04, 
       1.5261939641875454e-04, 
       1.1750217811738496e-04, 
       1.1288547734143903e-04, 
       1.5187176762491777e-04, 
       1.4501878605906594e-04, 
       1.3066879402842633e-04, 
       1.2805472334957558e-04, 
       1.4709146874589748e-04, 
       1.3999999999999402e-04, 
       1.3999999999998776e-04, 
       1.5170144409404205e-04, 
       1.1306582797405172e-04, 
       1.2663497483234330e-04, 
       1.2610702170147089e-04, 
       1.5000029300247646e-04, 
       1.0673961076427949e-04, 
       1.5029886353441169e-04, 
       1.0616691504363341e-04, 
       1.5000000000001064e-04, 
       1.4999999999999467e-04, 
       1.4999999999997711e-04, 
       1.1721758607610839e-04, 
       1.1016492558308764e-04, 
       1.1167423907362062e-04, 
       1.2380937223528338e-04, 
       1.1687815407368244e-04, 
       1.4700540270982287e-04, 
       1.1128946784418891e-04, 
       1.3853901522321453e-04, 
       1.1225821419508380e-04, 
       1.4816806519414650e-04, 
       1.0769904421691774e-04, 
       1.4530878484590040e-04, 
       1.1952437979404790e-04, 
       1.1553986707149173e-04, 
       1.0878469424929832e-04, 
       1.3026565757573444e-04, 
       1.4657129988399718e-04, 
       1.4639661398271650e-04, 
       1.1866825296514568e-04, 
       1.1866653046738359e-04, 
       1.3434106521251660e-04, 
       1.3455977556224222e-04, 
       1.4368268389771411e-04, 
       1.0404929304876943e-04, 
       1.1766433410374535e-04, 
       1.0390266643685797e-04, 
       1.0803246895783221e-04, 
       1.3930464464820760e-04, 
       1.0394265719651076e-04, 
       1.1907803026006660e-04, 
       1.0668772727353113e-04, 
       1.0277041863110158e-04, 
       1.4402613366547838e-04, 
       1.1173817034373829e-04, 
       1.2542924952745800e-04, 
       1.0999999999997557e-04, 
       1.0555347211440150e-04, 
       1.4264894567343699e-04, 
       1.0337831229809064e-04, 
       1.1223481154586532e-04, 
       1.2619243783710144e-04, 
       1.1888644875430759e-04, 
       1.4302004722046026e-04, 
       1.1567467617161917e-04, 
       1.4137643514245581e-04, 
       1.1643544203566237e-04, 
       1.1118063815297272e-04, 
       1.1540624323582928e-04, 
       1.0605790651186666e-04, 
       1.1495631387861764e-04, 
       1.0581377076477180e-04, 
       1.1492862472665980e-04, 
       1.0337639564955326e-04, 
       1.4167016534912945e-04, 
       1.1490537634514367e-04, 
       1.1490465720208656e-04, 
       1.1432626144347066e-04, 
       1.1567158249878285e-04, 
       1.1256208400029416e-04, 
       1.3790069990049093e-04, 
       1.3000000000002826e-04, 
       1.2610654530554975e-04, 
       1.3830513725065290e-04, 
       1.1701195551187363e-04, 
       1.4038801587165967e-04, 
       1.0236967909843781e-04, 
       1.1636043614633647e-04, 
       1.1020472392565392e-04, 
       1.4000000000001167e-04, 
       1.4000000000000110e-04, 
       1.4000000000003953e-04, 
       1.4000000000004688e-04, 
       1.4000000000004837e-04, 
       1.3999999999996977e-04, 
       1.1126780328095909e-04, 
       1.0528455616485060e-04, 
       9.9999999999999517e-05, 
       1.0258710752567322e-04, 
       1.0329855936178343e-04, 
       1.0656022804585908e-04, 
       1.1270639282585546e-04, 
       1.3865843258503843e-04, 
       1.1706653543617880e-04, 
       1.3453066606080832e-04, 
       1.1961816208963041e-04, 
       1.1961816208963041e-04, 
       1.0608890589779587e-04, 
       1.3324356181760967e-04, 
       9.8863100740601603e-05, 
       1.2380398039220521e-04, 
       1.1501570694013844e-04, 
       1.3817409413933006e-04, 
       1.1488971493324137e-04, 
       1.3350318586923779e-04, 
       1.3683523930368416e-04, 
       1.2211504289147564e-04, 
       1.1442229980198665e-04, 
       1.0664995872739121e-04, 
       1.3460701210031291e-04, 
       1.0105275111864593e-04, 
       1.3590614420380142e-04, 
       9.9680851026209065e-05, 
       1.2333271045750381e-04, 
       1.3424193658533295e-04, 
       1.3090800856853686e-04, 
       1.2486354615009237e-04, 
       1.0999999999999088e-04, 
       1.3652648410990384e-04, 
       1.0182121231367291e-04, 
       1.0181568044506513e-04, 
       1.3601104767715580e-04, 
       1.3252436965422991e-04, 
       1.1259232341171402e-04, 
       1.2833429090195435e-04, 
       1.3501847961500257e-04, 
       9.7064206050890913e-05, 
       1.0380326753653852e-04, 
       1.2482993974796293e-04, 
       1.2482993974796293e-04, 
       1.3333633488077004e-04, 
       9.7317747424658342e-05, 
       9.4944447367566363e-05, 
       1.1183559120678446e-04, 
       1.3307008907062124e-04, 
       1.0177462781199216e-04, 
       1.3328731888404051e-04, 
       1.0854144522169795e-04, 
       9.9677531861012359e-05, 
       1.0957407953662138e-04, 
       1.3227642927314537e-04, 
       1.0839853218224298e-04, 
       1.2076369513222895e-04, 
       1.3156361648438620e-04, 
       9.3935089024272398e-05, 
       1.2423774197045632e-04, 
       9.8007735155193610e-05, 
       1.3128434577344933e-04, 
       1.0868487569435066e-04, 
       1.1908837266829746e-04, 
       9.9762104330059225e-05, 
       1.3070819875595776e-04, 
       1.3039370210925910e-04, 
       1.2655070920172213e-04, 
       1.1999999999997877e-04, 
       9.5895156411998753e-05, 
       1.2526659646256303e-04, 
       1.3000000000000611e-04, 
       1.2999999999995147e-04, 
       1.2999999999998126e-04, 
       1.2999999999998828e-04, 
       9.8965821713440779e-05, 
       1.0592771586146755e-04, 
       9.2125989360311317e-05, 
       1.2876420937829654e-04, 
       1.0426893821952235e-04, 
       1.0751163758022518e-04, 
       1.2504034534700531e-04, 
       1.2508771128184014e-04, 
       1.0406091492675854e-04, 
       1.2841970783595400e-04, 
       1.2676525261580762e-04, 
       9.3816476234606625e-05, 
       9.2594170017619586e-05, 
       1.2233015585000115e-04, 
       1.1840802640216451e-04, 
       9.7720220597622898e-05, 
       9.5830213999810896e-05, 
       1.0967056697652411e-04, 
       1.1647885698491935e-04, 
       9.7139851346291362e-05, 
       9.1073565116616358e-05, 
       9.5490474162231840e-05, 
       9.0402741737421097e-05, 
       9.6698614648289922e-05, 
       1.0999999999999496e-04, 
       1.2354530635731942e-04, 
       1.0690030384042726e-04, 
       1.2348502060723871e-04, 
       9.7609420044041060e-05, 
       9.4884054812071692e-05, 
       9.3686945301308121e-05, 
       1.0942905820567188e-04, 
       1.0388627259974351e-04, 
       9.2656937549976567e-05, 
       1.0429912183679612e-04, 
       1.0289531435946407e-04, 
       1.0160177640963846e-04, 
       1.0243124929433718e-04, 
       9.6142780469253138e-05, 
       9.9788007305391522e-05, 
       1.1938082758237188e-04, 
       1.1844657409796066e-04, 
       9.0768911189065575e-05, 
       1.2000000000000110e-04, 
       1.0218942844361111e-04, 
       9.7532388687279805e-05, 
       8.7072275886642574e-05, 
       8.9346388260607023e-05, 
       1.0374451924344620e-04, 
       9.3866317583021606e-05, 
       9.5111371387572691e-05, 
       8.9890890722588049e-05, 
       9.2657462193259730e-05, 
       1.0163860398825262e-04, 
       1.0073560073914527e-04, 
       9.4715709626865601e-05, 
       8.7336825966675513e-05, 
       9.2275712385022534e-05, 
       1.0052207887893580e-04, 
       9.4884426848954831e-05, 
       8.8048937809629319e-05, 
       1.0859335783480130e-04, 
       8.8691148872930309e-05, 
       8.8691148872930309e-05, 
       1.0157820078176118e-04, 
       9.9863531589595587e-05, 
       1.1503531872714597e-04, 
       1.2000000000001347e-04, 
       1.2000000000000212e-04, 
       9.0706050385269343e-05, 
       1.0101266460683503e-04, 
       9.8212642396666068e-05, 
       8.4604828321030098e-05, 
       8.8331369910074175e-05, 
       1.0259445924119551e-04, 
       9.9015126749100717e-05, 
       8.6002858681654773e-05, 
       9.7121283938282575e-05, 
       9.7078103007281952e-05, 
       1.0026377726197009e-04, 
       1.1468414638557733e-04, 
       1.1326780211799721e-04, 
       9.0000000000000494e-05, 
       1.1792528732497590e-04, 
       1.1243004562140722e-04, 
       1.1312972535826639e-04, 
       8.7170160819477724e-05, 
       1.1750472248573592e-04, 
       9.0720528399696460e-05, 
       9.1936902918945553e-05, 
       9.6423613918820126e-05, 
       1.0017401789283045e-04, 
       9.7549362526801463e-05, 
       1.0005182621451147e-04, 
       1.1569407166342272e-04, 
       1.0927381990370473e-04, 
       9.7915157727556642e-05, 
       1.1649512920689498e-04, 
       9.1259090477585007e-05, 
       1.1183642622410782e-04, 
       9.4547267135741534e-05, 
       9.4514442342074912e-05, 
       9.9077361324955728e-05, 
       1.0949632490976281e-04, 
       1.1551787204159648e-04, 
       8.5007216853543363e-05, 
       8.8583614831828545e-05, 
       9.5860006163252302e-05, 
       8.2998800876797816e-05, 
       9.0093729328440624e-05, 
       1.0804249175936848e-04, 
       9.7548195809030159e-05, 
       1.0285914155937444e-04, 
       8.4952673868894885e-05, 
       9.6863539106596547e-05, 
       9.8115831252059744e-05, 
       9.3180757077268361e-05, 
       1.1387447284266037e-04, 
       1.1227705897581782e-04, 
       1.1000000000000221e-04, 
       1.1366293789917179e-04, 
       1.1346928425245814e-04, 
       1.0886591197482317e-04, 
       8.2962508615956368e-05, 
       8.9975558662558737e-05, 
       9.1358363477240685e-05, 
       9.0718903197863350e-05, 
       9.0628569225002556e-05, 
       8.1637288432446456e-05, 
       1.1221529563250229e-04, 
       9.4775739184071523e-05, 
       8.3419803678503693e-05, 
       9.0195943222689040e-05, 
       9.0015738432773842e-05, 
       7.9046627205076302e-05, 
       1.1134268484170219e-04, 
       9.1853994485333035e-05, 
       7.9055260827248737e-05, 
       9.9244702823123580e-05, 
       9.0886706246580339e-05, 
       1.1090415320194132e-04, 
       9.3556617106591667e-05, 
       7.9986966676777013e-05, 
       1.0884651519342423e-04, 
       7.8389886686311476e-05, 
       9.1447014888223907e-05, 
       1.1035458197225739e-04, 
       1.0979461619772101e-04, 
       1.0999997238627179e-04, 
       9.0337247362648837e-05, 
       7.9957940107900307e-05, 
       8.4538417525963750e-05, 
       1.1000000000003342e-04, 
       1.0953458842347013e-04, 
       7.9999999999997269e-05, 
       8.2256747867474702e-05, 
       9.6238196065946447e-05, 
       1.0074983723571350e-04, 
       8.2083054248371101e-05, 
       8.2083054248371101e-05, 
       8.8238574351569185e-05, 
       9.0510674978432467e-05, 
       7.7269029351689111e-05, 
       8.9247665926683094e-05, 
       8.1254746184664830e-05, 
       8.5041351185770502e-05, 
       8.4525525824895298e-05, 
       8.2837199999628194e-05, 
       8.2837199999628194e-05, 
       9.9999999999998961e-05, 
       9.9999999999996901e-05, 
       1.0000000000000425e-04, 
       1.0818295538802235e-04, 
       7.6891272224960801e-05, 
       7.6891272224960801e-05, 
       9.3392265155965533e-05, 
       8.4143602881121271e-05, 
       8.7603940796913323e-05, 
       9.4949909936234531e-05, 
       8.8355833079739609e-05, 
       7.6416742381228480e-05, 
       9.4757610270950515e-05, 
       8.9734311554509369e-05, 
       1.0609988887412997e-04, 
       7.8498515426990930e-05, 
       8.3511680358857638e-05, 
       9.7883804870268468e-05, 
       9.0469765735941605e-05, 
       8.4523921051035615e-05, 
       8.7915191869153460e-05, 
       8.8058149316865556e-05, 
       7.6228793366155150e-05, 
       8.4530611428621755e-05, 
       1.0254735274621737e-04, 
       9.0041809451396157e-05, 
       8.0654143056511682e-05, 
       8.0577883735269748e-05, 
       8.9031283389523757e-05, 
       7.6103370431981394e-05, 
       1.0543325991552761e-04, 
       1.0511999633821307e-04, 
       8.7463475908070099e-05, 
       9.0172344873275706e-05, 
       7.8888557710059335e-05, 
       7.6122823846716420e-05, 
       1.0484604133902631e-04, 
       9.9532335429972354e-05, 
       7.8011008461987874e-05, 
       9.5768505462793252e-05, 
       7.4898126808023798e-05, 
       1.0424873772958282e-04, 
       1.0287897008709013e-04, 
       7.8991074409490562e-05, 
       9.5905465000960098e-05, 
       8.0693979535059911e-05, 
       1.0281728898164558e-04, 
       1.0375330573368589e-04, 
       8.5804998040995694e-05, 
       1.0230414205021565e-04, 
       8.8468744528276766e-05, 
       1.0160755187428885e-04, 
       8.9959802532195725e-05, 
       7.2799436588802591e-05, 
       8.3262490394561103e-05, 
       7.4383445503191855e-05, 
       9.8929041810989866e-05, 
       7.2653277445675033e-05, 
       7.9233685560548385e-05, 
       8.4727106840491888e-05, 
       1.0209067981308031e-04, 
       8.9999999999997607e-05, 
       7.8418755017624627e-05, 
       7.4710843220595095e-05, 
       7.3727200163397136e-05, 
       7.2031924528618391e-05, 
       7.5503500167489672e-05, 
       7.5533495295449589e-05, 
       8.2378048159348460e-05, 
       7.2265303630274217e-05, 
       8.2311804506703882e-05, 
       7.5008273725721692e-05, 
       8.9880023489910754e-05, 
       7.5176435855341429e-05, 
       8.4178406510053449e-05, 
       1.0108486902181352e-04, 
       7.6722527844090988e-05, 
       9.7836569561700730e-05, 
       8.5923754323962638e-05, 
       9.0276669078779740e-05, 
       1.0083648566695826e-04, 
       8.3575638885037524e-05, 
       7.1850684357491695e-05, 
       8.3541563588056397e-05, 
       9.9999968496009969e-05, 
       8.1232134268478711e-05, 
       8.1202670249458283e-05, 
       8.3041883809145410e-05, 
       9.9999999999995126e-05, 
       7.4369805809064893e-05, 
       8.0405053360720682e-05, 
       8.2673908319192931e-05, 
       7.7935913435711501e-05, 
       8.0384092157402133e-05, 
       7.3377483288319953e-05, 
       7.3377483288319953e-05, 
       9.8575025324124915e-05, 
       7.9813992409684354e-05, 
       8.6282157756054203e-05, 
       7.8009428230691512e-05, 
       8.6149260115270907e-05, 
       6.9999999999999587e-05, 
       7.1241164172232562e-05, 
       8.1968520765248807e-05, 
       7.3915617691633654e-05, 
       8.0409836568867425e-05, 
       8.0962927570370761e-05, 
       7.0995837672725042e-05, 
       7.1519846462871642e-05, 
       9.0671780755698116e-05, 
       9.8230577501271630e-05, 
       7.3215763548154211e-05, 
       7.0950396904196375e-05, 
       7.3071977656194234e-05, 
       9.5032132373625171e-05, 
       9.4940325499723091e-05, 
       9.4389402245401437e-05, 
       7.5081071066808086e-05, 
       7.3220459462588724e-05, 
       9.4011676620450971e-05, 
       9.7689180305367675e-05, 
       9.0000000000001618e-05, 
       9.7440171714916011e-05, 
       8.2801654181278574e-05, 
       7.0000000000001607e-05, 
       8.2110073562116493e-05, 
       7.1508605742953715e-05, 
       9.2568961745064066e-05, 
       9.7004438428922339e-05, 
       6.9448139879616628e-05, 
       8.4437775258814059e-05, 
       9.5782620207817340e-05, 
       9.2394804417379113e-05, 
       7.2070991491597882e-05, 
       9.6317802500946100e-05, 
       9.4355648840122120e-05, 
       6.9330853850233494e-05, 
       6.9330853850233494e-05, 
       7.9753478529611763e-05, 
       9.5940258180110991e-05, 
       6.8602725422614859e-05, 
       9.5783662687275647e-05, 
       8.4446137433537394e-05, 
       8.6386548448219724e-05, 
       7.9590911777632184e-05, 
       7.0546710876504700e-05, 
       9.5185356194954666e-05, 
       7.1328980530686558e-05, 
       8.0172575989284050e-05, 
       7.0532004944074808e-05, 
       9.2297633059039286e-05, 
       7.1419875675855980e-05, 
       7.8889822156699447e-05, 
       7.2097468393137764e-05, 
       8.8497526031193529e-05, 
       8.9999999999999762e-05, 
       6.6680549642237356e-05, 
       6.6599413841636541e-05, 
       7.7779319082704989e-05, 
       7.5608647985682878e-05, 
       9.3287887983410953e-05, 
       8.8669084639081470e-05, 
       6.9396713943279412e-05, 
       6.7332636908491996e-05, 
       6.6381118878455176e-05, 
       7.7145611396563674e-05, 
       7.0198035314649528e-05, 
       7.0191382377160892e-05, 
       6.7565343581897167e-05, 
       8.4342742543146870e-05, 
       8.4342742543146870e-05, 
       6.6833844616994985e-05, 
       9.1075084775576686e-05, 
       6.8597039600113425e-05, 
       7.8552972061075339e-05, 
       6.6236597135901294e-05, 
       6.8749341104185833e-05, 
       9.1589779982594659e-05, 
       6.7802652582552394e-05, 
       9.0371840237216981e-05, 
       9.1266176359602354e-05, 
       9.1077206891008146e-05, 
       6.7913666560699470e-05, 
       6.5292573492295425e-05, 
       6.7160839485901489e-05, 
       6.6399251384022836e-05, 
       6.5531632519939007e-05, 
       6.4420830953975086e-05, 
       6.4129429260847037e-05, 
       8.7945663414557162e-05, 
       8.1879308673372947e-05, 
       7.5245429139408744e-05, 
       7.3953169493952322e-05, 
       6.5517093991671423e-05, 
       8.5309045582437394e-05, 
       9.0000075390378611e-05, 
       8.0867871270279743e-05, 
       6.5279834467930230e-05, 
       9.0000000000001280e-05, 
       8.9999999999996374e-05, 
       8.9999999999996821e-05, 
       9.0000000000003421e-05, 
       9.0000000000003421e-05, 
       9.0000000000001307e-05, 
       8.9999999999997200e-05, 
       7.9871785573518598e-05, 
       6.8759601048570679e-05, 
       7.5004930484056384e-05, 
       8.8981045691875218e-05, 
       8.5527845712701702e-05, 
       8.2596520811133877e-05, 
       6.3564660245551418e-05, 
       6.3564660245551418e-05, 
       7.3105142678787580e-05, 
       8.1940378161010771e-05, 
       8.4217606451497077e-05, 
       7.1635180985790199e-05, 
       6.3815697361101561e-05, 
       7.3540398758123503e-05, 
       8.8132084988347349e-05, 
       6.6187421951266064e-05, 
       7.6744550415559101e-05, 
       8.7492855807599112e-05, 
       6.4329471783434409e-05, 
       7.0707089994141343e-05, 
       6.5368112311815733e-05, 
       8.5072918195078416e-05, 
       8.4405284748117950e-05, 
       7.3993591220397544e-05, 
       6.5694168104627371e-05, 
       6.5690969241841222e-05, 
       6.5974556934037681e-05, 
       8.8178087417671824e-05, 
       6.5804275354932752e-05, 
       8.5781989007628653e-05, 
       8.7268594436841837e-05, 
       8.7698454254539745e-05, 
       6.3523716565456746e-05, 
       6.2782664805147059e-05, 
       6.2947602785371875e-05, 
       7.5709235111267941e-05, 
       7.8164946699223252e-05, 
       6.2926318353639850e-05, 
       8.7066602451116795e-05, 
       6.6188837300290080e-05, 
       8.6804307605521829e-05, 
       8.7503516752477958e-05, 
       7.2710892778552434e-05, 
       6.5526676671419604e-05, 
       8.7119692468934601e-05, 
       8.3845442423676018e-05, 
       6.2932161438017685e-05, 
       8.0000000000001701e-05, 
       7.9999999999998678e-05, 
       7.9999999999999519e-05, 
       7.9999999999998096e-05, 
       6.4725568976262672e-05, 
       6.4082397875402751e-05, 
       6.1420904361906656e-05, 
       8.0201523356305477e-05, 
       8.6472488265611856e-05, 
       6.3025606566483524e-05, 
       7.5739641872558686e-05, 
       6.2858842499579202e-05, 
       7.3433826750189549e-05, 
       6.9753042806339780e-05, 
       7.4867353158233655e-05, 
       7.6181658409797231e-05, 
       6.1095693571789109e-05, 
       6.0707077108593158e-05, 
       8.0874928264925387e-05, 
       7.2457126857521179e-05, 
       7.0985841490534086e-05, 
       6.3739694237638698e-05, 
       6.8658852308327409e-05, 
       6.6536670889729419e-05, 
       8.4027243113395111e-05, 
       8.4925274827432314e-05, 
       7.2063916263562911e-05, 
       8.1998361593912876e-05, 
       6.3387811245412895e-05, 
       6.0302312366378721e-05, 
       8.2010302439041621e-05, 
       8.4545604335347903e-05, 
       6.9566134181631807e-05, 
       7.1285518319432479e-05, 
       7.9889015743517770e-05, 
       6.2169939994170183e-05, 
       6.2621428328389327e-05, 
       8.3843794703167303e-05, 
       7.0071874893443744e-05, 
       6.9898342311314589e-05, 
       7.5423868390244627e-05, 
       8.3364490792987438e-05, 
       8.4023516073628051e-05, 
       7.0446884097595110e-05, 
       6.0477205953521636e-05, 
       8.3601944322890705e-05, 
       6.2532895695482482e-05, 
       6.0000000000003268e-05, 
       6.9337576616454661e-05, 
       7.2147399057708142e-05, 
       7.4613826550478139e-05, 
       7.2047001243602128e-05, 
       6.9971356922728973e-05, 
       8.0869501741526474e-05, 
       8.2975026884193681e-05, 
       8.2777263459580538e-05, 
       7.6785306772321593e-05, 
       5.9656552169501568e-05, 
       6.1970674452659349e-05, 
       5.9255230858190536e-05, 
       5.9253192194836492e-05, 
       8.2739116416066750e-05, 
       6.0869383833468857e-05, 
       8.0864406808699158e-05, 
       7.6960243256741473e-05, 
       8.1828442372698114e-05, 
       6.6132429931755368e-05, 
       8.0339524670957564e-05, 
       7.9520753605988049e-05, 
       6.0738332847183389e-05, 
       6.7801155166521664e-05, 
       7.1434776554814736e-05, 
       7.1782064819187207e-05, 
       5.7669016428387093e-05, 
       6.7988597178578357e-05, 
       6.5299283752224353e-05, 
       5.7770529099573146e-05, 
       7.2149499281289289e-05, 
       5.7326520308117141e-05, 
       6.7283647890583752e-05, 
       5.8751699953325959e-05, 
       7.9096346889344523e-05, 
       6.2317767938422928e-05, 
       6.6867814522925489e-05, 
       8.0335984704479112e-05, 
       8.0000000000002758e-05, 
       7.9999999999998963e-05, 
       8.0000000000002189e-05, 
       8.0000000000003164e-05, 
       7.9999999999997879e-05, 
       8.0000000000002297e-05, 
       7.9999999999999735e-05, 
       8.0000000000000007e-05, 
       6.6519252973622876e-05, 
       5.6536494466905784e-05, 
       7.8013605660462348e-05, 
       7.9837751306237033e-05, 
       5.8833382772887223e-05, 
       7.8150134036085822e-05, 
       7.9677015017955863e-05, 
       6.5969169353133803e-05, 
       5.8103048496305281e-05, 
       5.9999966503040292e-05, 
       5.6474108545434415e-05, 
       5.7062411157666192e-05, 
       7.8689724458564432e-05, 
       6.5351581379482213e-05, 
       6.1823173332318599e-05, 
       6.5307855315628914e-05, 
       5.7267331134879187e-05, 
       5.8673927663774992e-05, 
       6.8607501504733658e-05, 
       6.5250104884631173e-05, 
       7.3950687960325733e-05, 
       7.8375792244723459e-05, 
       6.1611714335937885e-05, 
       6.4236197044914876e-05, 
       5.5369853546733534e-05, 
       7.4607631394892288e-05, 
       6.7242234899581670e-05, 
       6.4947281043603957e-05, 
       5.6160646610726989e-05, 
       7.7708505356304907e-05, 
       6.4079022329303412e-05, 
       5.6204394380613141e-05, 
       5.6153911757088295e-05, 
       7.3068760669381080e-05, 
       5.5818049402023691e-05, 
       7.0000000000000739e-05, 
       7.7039602817215977e-05, 
       5.7909221249900485e-05, 
       6.3803387863523009e-05, 
       7.5706378051610521e-05, 
       5.7404166084540090e-05, 
       7.6204958127061594e-05, 
       5.5627875156501482e-05, 
       6.9249275528613638e-05, 
       5.5660180861356442e-05, 
       6.0363830281187977e-05, 
       6.6591387485163435e-05, 
       6.4563079684622858e-05, 
       5.5458190355394455e-05, 
       7.6225441001027440e-05, 
       5.6112149914041319e-05, 
       5.6112149914041319e-05, 
       5.9268583277828877e-05, 
       6.9999999999997121e-05, 
       7.5789643086375563e-05, 
       7.0000000000001945e-05, 
       7.1003146118943941e-05, 
       7.2078340116240059e-05, 
       6.1902134037346168e-05, 
       7.5359985714889467e-05, 
       5.4725877573315912e-05, 
       5.8900623380256271e-05, 
       5.3589100327708243e-05, 
       6.3566154651774413e-05, 
       5.3304736699708273e-05, 
       7.3525456285339046e-05, 
       6.2313633304552190e-05, 
       6.4537691770276095e-05, 
       5.3292099533635400e-05, 
       5.4807553965406274e-05, 
       7.2928319837938513e-05, 
       5.2865201337797986e-05, 
       5.2865201337797986e-05, 
       6.2916737412755850e-05, 
       5.3176717045459786e-05, 
       5.4533976744891498e-05, 
       5.5832659831989547e-05, 
       5.3510696584172764e-05, 
       7.1300231543291307e-05, 
       5.2974600144812072e-05, 
       5.4377847940313387e-05, 
       6.1351663755275518e-05, 
       5.6215032601271803e-05, 
       5.4800822851492676e-05, 
       6.1404942457538020e-05, 
       6.3451143302306910e-05, 
       5.9584996489456378e-05, 
       5.9569387431431119e-05, 
       5.5869535544463678e-05, 
       5.4535617322689182e-05, 
       5.4688780502637356e-05, 
       5.9850479214570943e-05, 
       5.3586275762554595e-05, 
       6.1266153829070432e-05, 
       6.1266153829070432e-05, 
       5.3050299057791493e-05, 
       7.0939031655639867e-05, 
       5.2362326849137299e-05, 
       6.7212976920482721e-05, 
       5.2194764550423150e-05, 
       6.0325120855691790e-05, 
       6.3112883135151368e-05, 
       6.7535019399359163e-05, 
       5.1784527975714110e-05, 
       5.0524041830332288e-05, 
       5.8352660442955614e-05, 
       5.0574510191146357e-05, 
       5.9297863186819411e-05, 
       5.9284366611368220e-05, 
       6.8897413945949608e-05, 
       6.1108456033044211e-05, 
       7.0209409649381424e-05, 
       7.0816857552466640e-05, 
       6.8219055309236485e-05, 
       6.7877929500971134e-05, 
       6.9779979705935448e-05, 
       6.9462599853447303e-05, 
       5.7144800770106535e-05, 
       5.0308870103953731e-05, 
       5.0173190848404817e-05, 
       4.9952019638466125e-05, 
       5.3419951825346200e-05, 
       5.8466031174966944e-05, 
       6.6358685035065285e-05, 
       5.5140248012791047e-05, 
       6.8347594297426200e-05, 
       5.6960875757725883e-05, 
       6.9999999999999303e-05, 
       6.9999999999999709e-05, 
       7.0000000000004791e-05, 
       7.0000000000003423e-05, 
       7.0000000000001268e-05, 
       7.0000000000001593e-05, 
       5.9831518723280486e-05, 
       6.0293451616680224e-05, 
       6.9682258005263951e-05, 
       6.7802971065954073e-05, 
       4.9843034076844770e-05, 
       5.0624743386964780e-05, 
       5.2133621255799013e-05, 
       5.4176502438370773e-05, 
       5.5798538944435063e-05, 
       5.1000950188381827e-05, 
       6.0038400144387023e-05, 
       5.2694940463419361e-05, 
       4.9686147411923310e-05, 
       4.9379684732393815e-05, 
       5.0293386364764924e-05, 
       5.4966104262637906e-05, 
       5.2505922357650699e-05, 
       6.8943640194868587e-05, 
       5.5604439994099130e-05, 
       5.5599815750212186e-05, 
       6.5239209473791438e-05, 
       5.8395985010709600e-05, 
       4.9452519071456424e-05, 
       5.7017490852978671e-05, 
       5.5978416248899198e-05, 
       5.0264698894438015e-05, 
       6.6665926295676975e-05, 
       6.8453401918264407e-05, 
       5.6569357883090364e-05, 
       6.8335212869132154e-05, 
       4.9647129202807830e-05, 
       5.0104045650719756e-05, 
       6.3486804806258320e-05, 
       6.8298814642482164e-05, 
       5.6592329302999660e-05, 
       5.9880908325565909e-05, 
       5.4814640645869430e-05, 
       5.0081583947383345e-05, 
       5.4809868395992217e-05, 
       6.3228615564588314e-05, 
       5.4840267693430367e-05, 
       6.7782269914526991e-05, 
       5.6401907126616625e-05, 
       5.1141848434313667e-05, 
       4.9904532523365142e-05, 
       5.6391812676907800e-05, 
       6.7679656515190180e-05, 
       5.6829676507965120e-05, 
       4.9842647965109901e-05, 
       4.9842647965109901e-05, 
       5.5920225838151660e-05, 
       6.7199948373296471e-05, 
       5.6543237651659965e-05, 
       6.7157657138038626e-05, 
       6.6379400086788429e-05, 
       4.8901692508048178e-05, 
       4.9484856004237974e-05, 
       5.9996105121258811e-05, 
       5.4331009496574316e-05, 
       5.5750291038992419e-05, 
       4.8644734164384468e-05, 
       5.5961190211878463e-05, 
       5.0006049186135127e-05, 
       5.5367158497658229e-05, 
       5.8525506121777416e-05, 
       4.9067032759471781e-05, 
       5.7572266403071556e-05, 
       4.7506199980117524e-05, 
       6.4218909607501599e-05, 
       5.5091965481506400e-05, 
       5.1108817122108481e-05, 
       6.3729933344069172e-05, 
       4.7534723533173651e-05, 
       4.8721156222038537e-05, 
       5.0023741698867944e-05, 
       6.5413837799685278e-05, 
       6.6031288751394396e-05, 
       6.5185135829264157e-05, 
       4.8639939451732586e-05, 
       5.3471543155147815e-05, 
       5.3470234202847621e-05, 
       5.4235599937334449e-05, 
       4.6696674737072938e-05, 
       5.4510445522446929e-05, 
       6.3092676413622730e-05, 
       5.4480746848402336e-05, 
       6.5162083144622830e-05, 
       6.4637049800461062e-05, 
       5.8226291510811822e-05, 
       6.1727192099590223e-05, 
       6.0000000000001885e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       5.9999999999999663e-05, 
       4.8222542418440093e-05, 
       5.4441070445223625e-05, 
       4.6567864860969004e-05, 
       6.3116790105446032e-05, 
       4.7811094180524836e-05, 
       6.4524095056913290e-05, 
       4.5849246173702267e-05, 
       4.7957233995658348e-05, 
       5.7735816181440496e-05, 
       4.6131248145815747e-05, 
       6.4533793363186074e-05, 
       5.0000000000000321e-05, 
       5.0000000000004712e-05, 
       5.5276263557192818e-05, 
       5.3480082940807484e-05, 
       4.5994394769164854e-05, 
       6.3954734509573069e-05, 
       5.5432228886451026e-05, 
       4.6764141061557171e-05, 
       6.1466027736689719e-05, 
       5.2895993062280609e-05, 
       4.6911290282061963e-05, 
       4.7690694162014693e-05, 
       5.5655058365349034e-05, 
       4.5381029945158757e-05, 
       6.3352082157043253e-05, 
       6.3779055751876667e-05, 
       5.4588935291063727e-05, 
       4.6891522943457257e-05, 
       4.6402637571805200e-05, 
       6.2056218560641756e-05, 
       4.6687331791065307e-05, 
       4.5627804808424103e-05, 
       4.9240192715254951e-05, 
       5.2898674187020502e-05, 
       6.1551490208796360e-05, 
       6.3037811552734289e-05, 
       4.9070222801629898e-05, 
       6.0987675671929428e-05, 
       5.2482560993622402e-05, 
       6.2693667708373256e-05, 
       4.5401556783972952e-05, 
       5.5439002342805138e-05, 
       4.5055104093155815e-05, 
       5.2426711298138298e-05, 
       5.3305595690661849e-05, 
       5.9752633549208924e-05, 
       5.0821434509652217e-05, 
       5.4279540398605293e-05, 
       4.7726625323203568e-05, 
       6.0691526144331518e-05, 
       4.9446545397466637e-05, 
       5.8993801931120903e-05, 
       5.1719168035984416e-05, 
       5.6845072614997615e-05, 
       4.4359350661943662e-05, 
       4.4359286421192773e-05, 
       4.4683099791801449e-05, 
       4.6767846170332439e-05, 
       5.3795497044884278e-05, 
       4.6195354719139865e-05, 
       4.6195354719139865e-05, 
       4.6461494876802746e-05, 
       6.2294893931289029e-05, 
       5.9513949740074086e-05, 
       5.2803638788575085e-05, 
       4.6932931108333560e-05, 
       4.8425237850651634e-05, 
       5.1688611403087938e-05, 
       4.6103121211759707e-05, 
       4.4646855408922731e-05, 
       6.1971106494966159e-05, 
       5.2277120400292120e-05, 
       5.1268125044547678e-05, 
       4.5127168589627302e-05, 
       4.5559762378008551e-05, 
       4.6163909833683394e-05, 
       4.7622202315681006e-05, 
       5.1982162966691362e-05, 
       5.5561372275647245e-05, 
       4.4656735501918160e-05, 
       5.1112817075278030e-05, 
       4.5312263980815190e-05, 
       4.5984452296017492e-05, 
       5.0232938537276863e-05, 
       5.1438616540721909e-05, 
       4.9029391646492582e-05, 
       4.5847551201734067e-05, 
       5.9081814719610968e-05, 
       4.4758322468941056e-05, 
       4.4527218154763130e-05, 
       6.1196370752057935e-05, 
       5.9423192614379616e-05, 
       5.8426261204146650e-05, 
       4.3895787312711132e-05, 
       5.0823674877389410e-05, 
       4.4159592683339312e-05, 
       4.3183102749350650e-05, 
       5.9136410236724295e-05, 
       4.3048086403028848e-05, 
       4.6179897675467623e-05, 
       5.8611348001854098e-05, 
       5.5893493755321943e-05, 
       5.0781068754357385e-05, 
       5.0780953224542056e-05, 
       6.0400631391043782e-05, 
       5.0341563564047152e-05, 
       4.3795232393121785e-05, 
       5.9902762742168291e-05, 
       4.4058926915834686e-05, 
       5.0803928317872478e-05, 
       5.0655147822811326e-05, 
       5.9999789296617017e-05, 
       5.7861267789368643e-05, 
       5.9999975703747769e-05, 
       5.9604915867661808e-05, 
       5.9999999999997521e-05, 
       6.0000000000002583e-05, 
       5.9999999999998145e-05, 
       5.9999999999999920e-05, 
       5.9999999999995651e-05, 
       6.0000000000004630e-05, 
       5.9944858705767063e-05, 
       4.3729611121582715e-05, 
       4.2763863510998176e-05, 
       4.2902994091444175e-05, 
       5.1454732779578809e-05, 
       4.3518194438287823e-05, 
       4.2888991335543819e-05, 
       4.9567611860653887e-05, 
       5.9533922178796983e-05, 
       4.5448074194076151e-05, 
       4.4405629696802045e-05, 
       5.3005444466418007e-05, 
       4.4921370867962235e-05, 
       4.8214499541383813e-05, 
       4.8213488773082908e-05, 
       4.3903468407753180e-05, 
       4.8915317934496880e-05, 
       4.9178017325850189e-05, 
       4.2520817149067737e-05, 
       4.2520504244822200e-05, 
       4.9344675971299254e-05, 
       5.8870317247688816e-05, 
       5.8399868947111986e-05, 
       4.3479357167338926e-05, 
       4.8728647427657396e-05, 
       5.7060797803891980e-05, 
       4.1644535146986210e-05, 
       4.1666276614238178e-05, 
       4.9559935505613789e-05, 
       5.8638330734578074e-05, 
       4.4745532451904581e-05, 
       4.4505303279567565e-05, 
       4.9681559965770856e-05, 
       4.8810769695560470e-05, 
       5.7722904115022887e-05, 
       4.2225036450746274e-05, 
       4.7257314886941546e-05, 
       4.9058219335361414e-05, 
       4.4520702901876963e-05, 
       4.9006220801975131e-05, 
       4.3582639489121810e-05, 
       4.3570056626258740e-05, 
       4.7775474352826803e-05, 
       4.8756172778324964e-05, 
       5.8294821702607921e-05, 
       5.4503569091204679e-05, 
       4.2571569169167861e-05, 
       4.1398880755584905e-05, 
       4.4746669370521632e-05, 
       5.7998520207417815e-05, 
       5.6473935260511194e-05, 
       4.2300274501921504e-05, 
       5.7574970358682534e-05, 
       5.0084073722433577e-05, 
       5.6172722192566461e-05, 
       5.7565227528944879e-05, 
       5.5171156018821212e-05, 
       4.5150424730827053e-05, 
       4.9594894324417839e-05, 
       4.3575408626722596e-05, 
       4.7614424518065227e-05, 
       5.7403818408117554e-05, 
       4.9786074368996327e-05, 
       4.4456846563521062e-05, 
       5.6184746781328476e-05, 
       4.6172751015155248e-05, 
       5.6499997081157638e-05, 
       4.7607030550006311e-05, 
       4.0980629850883083e-05, 
       4.8134523305034848e-05, 
       4.7496310285260817e-05, 
       5.5759141294071027e-05, 
       5.4694078524475342e-05, 
       4.1168613794556891e-05, 
       5.6668906413007280e-05, 
       4.7065722633256996e-05, 
       4.0931194581401073e-05, 
       5.4448599722087120e-05, 
       4.8245317363303676e-05, 
       5.5935545589140298e-05, 
       4.3518100129399014e-05, 
       4.5258864513686095e-05, 
       4.7263814418240710e-05, 
       4.1628071441651432e-05, 
       4.1732668982753145e-05, 
       5.0839724294963075e-05, 
       5.5643895616299083e-05, 
       4.0874114034633423e-05, 
       4.0213414711940769e-05, 
       4.0602556199225371e-05, 
       3.9798917532351760e-05, 
       4.3502164958705832e-05, 
       4.0631247379848673e-05, 
       4.0000000000002219e-05, 
       5.3113323259739849e-05, 
       5.5960958129507828e-05, 
       5.5945503817689012e-05, 
       4.6479271990845340e-05, 
       5.4205030096753215e-05, 
       4.8038892964860316e-05, 
       4.8038892964860316e-05, 
       4.1249531890938287e-05, 
       4.9665036416254743e-05, 
       4.6393396209071189e-05, 
       4.8586203699758706e-05, 
       4.7643822159193113e-05, 
       5.0113951824612399e-05, 
       4.5644259921143167e-05, 
       4.0773247314461056e-05, 
       3.9886220440567700e-05, 
       5.0324536252164542e-05, 
       5.3774498427009434e-05, 
       4.1452454878147792e-05, 
       4.2226452827185388e-05, 
       5.3606606757098600e-05, 
       5.5148734182742428e-05, 
       3.9603226822633545e-05, 
       3.9847876969610408e-05, 
       3.9253995717913027e-05, 
       5.5209158908184279e-05, 
       4.0171539049671001e-05, 
       4.0229147335329989e-05, 
       4.7710572602320027e-05, 
       4.7801983139927655e-05, 
       4.0561034351946743e-05, 
       4.1492018703479000e-05, 
       5.1984560871044604e-05, 
       4.4244819373534095e-05, 
       4.2299679283002287e-05, 
       3.9384739031196680e-05, 
       4.8030171539722245e-05, 
       4.8934955333675140e-05, 
       3.8824445198949964e-05, 
       3.9346225258795313e-05, 
       5.4545005410651440e-05, 
       4.2248437508809583e-05, 
       5.4129559070840305e-05, 
       5.4432246400521999e-05, 
       4.7098358680081894e-05, 
       4.2951615229805261e-05, 
       5.0000000000004854e-05, 
       4.9999999999996845e-05, 
       4.4762456978213309e-05, 
       4.4214161872150917e-05, 
       4.9999999999996086e-05, 
       5.0000000000001602e-05, 
       4.0055337265233531e-05, 
       5.1875721802107281e-05, 
       5.4173851804815632e-05, 
       5.2342087512995231e-05, 
       4.0000000000004150e-05, 
       4.3645280927087385e-05, 
       3.9904217891268223e-05, 
       4.2376994520283985e-05, 
       4.2297303009571453e-05, 
       4.8879613522080589e-05, 
       4.0301879896279338e-05, 
       5.1009666875063227e-05, 
       4.0291884609681158e-05, 
       4.3495256436175411e-05, 
       4.5730042942677488e-05, 
       5.3787158368192670e-05, 
       4.8822304653832097e-05, 
       3.8810779524411854e-05, 
       5.3302791238473550e-05, 
       4.6120431269476313e-05, 
       4.5423750287827508e-05, 
       4.4419468484612943e-05, 
       4.1842659261109005e-05, 
       5.3342683361897172e-05, 
       3.7831057841672347e-05, 
       4.4876542213077566e-05, 
       5.3339953067899664e-05, 
       5.1307576498520121e-05, 
       4.9795340273845722e-05, 
       4.0551968891482593e-05, 
       4.2942181427477144e-05, 
       4.6580825491456696e-05, 
       4.4170839478869739e-05, 
       5.3225469148860561e-05, 
       4.3552526954001138e-05, 
       3.9936472321541530e-05, 
       3.7856822941797808e-05, 
       4.1424703336531182e-05, 
       4.4109848575973033e-05, 
       5.2309653545515140e-05, 
       5.0650435375008185e-05, 
       5.1035434637487536e-05, 
       3.9622605062936089e-05, 
       3.8590028790683536e-05, 
       4.1438395651877721e-05, 
       4.0552969630143932e-05, 
       5.2489713088133530e-05, 
       3.9345854051111601e-05, 
       3.7758300027101911e-05, 
       5.0750461565384194e-05, 
       3.7402233097626411e-05, 
       4.5911510414821101e-05, 
       4.4320155766962862e-05, 
       4.3554590023744675e-05, 
       4.4584164971959899e-05, 
       4.4580957478613705e-05, 
       3.8434965776471489e-05, 
       4.6170046403484089e-05, 
       4.3373197223144499e-05, 
       4.7388291342434926e-05, 
       4.3353766181158540e-05, 
       3.9106412200694077e-05, 
       3.8355144926800230e-05, 
       3.8376808888131592e-05, 
       4.9800727126423536e-05, 
       5.2057090972482660e-05, 
       4.3435654224056656e-05, 
       5.0063937364252527e-05, 
       3.7185351056380096e-05, 
       4.0916337656372547e-05, 
       3.7096229741818396e-05, 
       3.6781744627271483e-05, 
       5.1811525370187312e-05, 
       3.7833741387392980e-05, 
       3.6894970698006109e-05, 
       3.7151459339555959e-05, 
       4.9346160114750382e-05, 
       3.6633695532275512e-05, 
       3.7762083455162853e-05, 
       3.7713261214658515e-05, 
       4.2905385005542004e-05, 
       4.2721777594869631e-05, 
       4.2871158898182830e-05, 
       4.2856075784532639e-05, 
       3.7585011634027380e-05, 
       4.2787150309464686e-05, 
       3.8282707689072949e-05, 
       3.8547816795841882e-05, 
       3.8729983163976672e-05, 
       4.3802596034923394e-05, 
       4.9156593596046735e-05, 
       4.6674306788075115e-05, 
       3.6793600117994928e-05, 
       3.7672256743152476e-05, 
       3.8982643011500457e-05, 
       4.2095568646817963e-05, 
       4.9156596716464992e-05, 
       5.0818749307970015e-05, 
       4.7638625828669324e-05, 
       4.8743388042410356e-05, 
       4.8743388042410356e-05, 
       5.0920316189094351e-05, 
       4.2501847283851815e-05, 
       4.2770545601811098e-05, 
       3.8452002514831642e-05, 
       4.1560207605759910e-05, 
       3.6810146127261362e-05, 
       4.5106844834294229e-05, 
       3.7543407113834333e-05, 
       3.6999743152688838e-05, 
       4.5228094725095993e-05, 
       3.8120433144414718e-05, 
       4.2373393125234178e-05, 
       3.7339244915619255e-05, 
       5.0606222122548995e-05, 
       5.0171211793907470e-05, 
       4.8103703637198708e-05, 
       4.8657954980057095e-05, 
       4.0706698873243219e-05, 
       4.3300414987776052e-05, 
       3.7990417315996674e-05, 
       3.6171231268334320e-05, 
       4.7449667996545013e-05, 
       4.9457418300433907e-05, 
       4.5212475188456699e-05, 
       3.6277581681391359e-05, 
       3.8263565084705065e-05, 
       3.6853405453953792e-05, 
       5.0000000000004759e-05, 
       4.9999973253849495e-05, 
       4.9999309915235958e-05, 
       4.1708997021255155e-05, 
       5.0000000000004814e-05, 
       5.0000000000000212e-05, 
       4.9999999999997848e-05, 
       4.9999999999995706e-05, 
       4.9999999999997285e-05, 
       5.0000000000003695e-05, 
       5.0000000000001554e-05, 
       5.0000000000001825e-05, 
       4.9999999999996851e-05, 
       3.5408176441764625e-05, 
       4.8187191630835852e-05, 
       4.8459644167773767e-05, 
       3.5859946054074073e-05, 
       4.8698843911162814e-05, 
       4.9792661553601065e-05, 
       4.1093125575671377e-05, 
       4.9757504353982201e-05, 
       4.9627132456014287e-05, 
       4.7884173780204098e-05, 
       4.1288674931018946e-05, 
       4.9658525568868731e-05, 
       3.8935284750652676e-05, 
       4.2159821135385924e-05, 
       4.4696201374075211e-05, 
       3.8442269209632602e-05, 
       4.8383706284272943e-05, 
       3.8924999276684364e-05, 
       3.9944878971996325e-05, 
       4.9349400786323520e-05, 
       4.9043796063861990e-05, 
       3.6121592269412959e-05, 
       4.9267370544174806e-05, 
       4.6975451485351150e-05, 
       4.2420704851971450e-05, 
       4.9034265060333103e-05, 
       4.1000495993128493e-05, 
       3.6802914218379274e-05, 
       4.0994805730348300e-05, 
       3.5221795706480916e-05, 
       3.8855941063243146e-05, 
       4.1781485763613834e-05, 
       3.9538020650094244e-05, 
       4.0405750604700326e-05, 
       4.8900676149172333e-05, 
       4.2481998373067597e-05, 
       3.9039416360268234e-05, 
       3.9506867683778882e-05, 
       4.0696066776815846e-05, 
       4.8901500731813548e-05, 
       4.3173906267770743e-05, 
       4.0790710937704446e-05, 
       3.5496172630263295e-05, 
       4.6955589107558505e-05, 
       3.5221231340790295e-05, 
       4.6516135729167509e-05, 
       3.8238364870826670e-05, 
       4.4237793935597831e-05, 
       4.0802853698156234e-05, 
       4.1243064236859773e-05, 
       3.4548359607475459e-05, 
       3.6197101228016460e-05, 
       4.7748596570621995e-05, 
       4.0291274483495406e-05, 
       4.4257207759580529e-05, 
       4.8222350422121991e-05, 
       3.5094377888614524e-05, 
       3.6656924485171226e-05, 
       4.4073511658702931e-05, 
       3.9637449702643444e-05, 
       4.7938399710402696e-05, 
       3.4979671298360411e-05, 
       4.0410673297021768e-05, 
       3.9035231630682992e-05, 
       4.8090271341060706e-05, 
       3.9942231822511489e-05, 
       3.5083336168095262e-05, 
       3.9879539664203457e-05, 
       3.4832545622196622e-05, 
       3.7611091268450171e-05, 
       4.7232898177139256e-05, 
       3.9929004153968073e-05, 
       3.9894524243583448e-05, 
       3.7656906083208395e-05, 
       3.9819448513341300e-05, 
       3.4932540161114768e-05, 
       4.0518355283772929e-05, 
       3.4240921757062046e-05, 
       4.4066943738975884e-05, 
       4.7423610948041679e-05, 
       3.9144823329152139e-05, 
       4.0908748760812218e-05, 
       3.9786816243739836e-05, 
       3.3798432919652239e-05, 
       3.6880076521315806e-05, 
       3.9622751844855506e-05, 
       3.7290121774435469e-05, 
       4.7438291226495707e-05, 
       3.7284906634344718e-05, 
       3.3720419377599246e-05, 
       4.1041161905783108e-05, 
       4.0980629850874382e-05, 
       4.7510617390791483e-05, 
       3.9780995073475945e-05, 
       4.7436785653119979e-05, 
       3.4110563425326465e-05, 
       4.6985681246361248e-05, 
       3.3579559186638764e-05, 
       4.5272077524089852e-05, 
       3.9258009246260173e-05, 
       3.9275724047458070e-05, 
       4.3233587279662087e-05, 
       3.3452342038935922e-05, 
       3.4643963026278155e-05, 
       3.7732727961902534e-05, 
       3.8557410131031713e-05, 
       4.6562965856360202e-05, 
       3.7496918797710279e-05, 
       4.2679478413180879e-05, 
       4.1980440831123302e-05, 
       3.9327164957137778e-05, 
       3.8539291729924192e-05, 
       3.5849749348037440e-05, 
       4.5256285029026452e-05, 
       3.9630041236095456e-05, 
       4.5953910310499518e-05, 
       4.5574874226989893e-05, 
       3.8625314259170787e-05, 
       3.3750070092269913e-05, 
       3.5148565179809681e-05, 
       3.7512951978688678e-05, 
       4.4999841703036331e-05, 
       4.3891583404918332e-05, 
       4.6758295575699226e-05, 
       3.6498940058986615e-05, 
       3.7468372359729450e-05, 
       3.8945032376840196e-05, 
       3.7999984347982119e-05, 
       3.8784192773063788e-05, 
       3.5740706722842011e-05, 
       3.8641321225121811e-05, 
       3.4905316698490314e-05, 
       3.3067235639361119e-05, 
       3.3067235639361119e-05, 
       4.4707889030744459e-05, 
       3.9793249231023071e-05, 
       4.4610283210327343e-05, 
       4.4619029328667574e-05, 
       4.5206494724860396e-05, 
       3.3597725751776402e-05, 
       4.5494440138931664e-05, 
       3.2948751530114775e-05, 
       3.6135019425927445e-05, 
       4.6165735096238155e-05, 
       3.4680960890658572e-05, 
       4.6121954759801873e-05, 
       3.9606224310459238e-05, 
       4.6040365011574947e-05, 
       4.5968564234313613e-05, 
       3.3216908878284660e-05, 
       3.6121696320554319e-05, 
       3.6118904596417354e-05, 
       3.9965886553234756e-05, 
       3.7960942959211673e-05, 
       3.3015728625472228e-05, 
       3.3129972365801607e-05, 
       3.2695181427336741e-05, 
       3.4250440130200798e-05, 
       4.4448736542906044e-05, 
       3.7852540849177154e-05, 
       3.2502809213086196e-05, 
       3.4171962459343609e-05, 
       3.8131071084777378e-05, 
       3.4566648187038441e-05, 
       3.5013399755891196e-05, 
       3.2701903314242563e-05, 
       3.2431439664343020e-05, 
       4.5552301774535584e-05, 
       3.4438876315432489e-05, 
       4.3878641516741261e-05, 
       3.2304560984191068e-05, 
       3.2304339024064865e-05, 
       4.5264105814598043e-05, 
       4.3547681545482941e-05, 
       3.8540889506387398e-05, 
       4.5436591216783690e-05, 
       4.5437702730735851e-05, 
       3.7922218768194438e-05, 
       3.2415214542006272e-05, 
       3.2564457829914071e-05, 
       3.7304745716350087e-05, 
       3.2323617004262415e-05, 
       3.7278195702878041e-05, 
       3.8373241105207387e-05, 
       3.2573701986389598e-05, 
       3.7646342196827161e-05, 
       3.9421272924813623e-05, 
       3.9501624254035637e-05, 
       3.7390695190544806e-05, 
       3.6069459201781000e-05, 
       3.6062786061320047e-05, 
       4.3389064988984674e-05, 
       3.2627976048381635e-05, 
       4.4284561057134100e-05, 
       4.3601160427165270e-05, 
       3.4635328753529157e-05, 
       3.2149093834813746e-05, 
       3.3689855536866871e-05, 
       4.3216426236862585e-05, 
       3.2855792616886550e-05, 
       3.6064336406735929e-05, 
       3.3239424588928948e-05, 
       3.2827854533171908e-05, 
       4.4989275875484362e-05, 
       3.3584452108383197e-05, 
       4.2939446871580752e-05, 
       3.9222848092840874e-05, 
       4.2901170371280392e-05, 
       3.2233595369570919e-05, 
       3.1728628126545885e-05, 
       4.2855442589356341e-05, 
       3.6565023474546315e-05, 
       3.8298606578543848e-05, 
       3.2950984186462133e-05, 
       3.6532315097328446e-05, 
       3.2711571986310915e-05, 
       3.1967181879025635e-05, 
       4.4603547965731295e-05, 
       3.2643724001595769e-05, 
       3.2877730296234751e-05, 
       3.8593381181162502e-05, 
       4.2050707116091333e-05, 
       3.8193056584885661e-05, 
       3.2078421068134895e-05, 
       3.1563250816349307e-05, 
       3.3464700965702173e-05, 
       3.2295557314115479e-05, 
       3.1986838654988278e-05, 
       3.9658619568854458e-05, 
       3.1662339414483392e-05, 
       3.7031531628240052e-05, 
       3.1456022157395817e-05, 
       3.2736656275658254e-05, 
       3.1971041941472359e-05, 
       3.7061890999114858e-05, 
       3.1589612751515528e-05, 
       4.0842903727528385e-05, 
       3.8693550208363229e-05, 
       3.2652593079599549e-05, 
       3.2426118877173239e-05, 
       3.6662990908906790e-05, 
       4.4059443264123390e-05, 
       3.6646883609913904e-05, 
       3.3306620110803306e-05, 
       3.3945017549259451e-05, 
       3.4136085370227129e-05, 
       3.6709397279896857e-05, 
       4.3125553242843388e-05, 
       3.1275604022377079e-05, 
       3.9257993709319027e-05, 
       3.9241051090370256e-05, 
       3.5394353850374905e-05, 
       3.2612747134150309e-05, 
       3.7224761328834978e-05, 
       3.2490770041774935e-05, 
       3.2490770041774935e-05, 
       3.6874231398616637e-05, 
       3.3254272551589700e-05, 
       3.9086867039923709e-05, 
       3.9920204316030868e-05, 
       3.1321268734910817e-05, 
       3.3197088231901820e-05, 
       3.2692032624632953e-05, 
       4.2014459126866369e-05, 
       3.8134282383394992e-05, 
       3.4234246414402742e-05, 
       3.0862267877250008e-05, 
       4.2128536942499734e-05, 
       4.3476692460793331e-05, 
       4.2162720180259138e-05, 
       3.6232159466694327e-05, 
       3.6145824384960646e-05, 
       4.1965305380788002e-05, 
       4.0365250021186470e-05, 
       4.0000000000000850e-05, 
       4.0000000000000850e-05, 
       3.9999999999999305e-05, 
       3.9999999999996453e-05, 
       4.0000000000002870e-05, 
       3.3986957773966361e-05, 
       3.5607724550697976e-05, 
       3.5737714106043158e-05, 
       3.9342411982573345e-05, 
       3.6871693082504237e-05, 
       3.4233897666132191e-05, 
       3.3649464144804624e-05, 
       3.8566780597453067e-05, 
       3.6434982670612244e-05, 
       4.1276516310770805e-05, 
       3.0532456307141396e-05, 
       3.8264347262882043e-05, 
       3.1635269277834769e-05, 
       3.5723356591098958e-05, 
       3.2102106252287437e-05, 
       3.0419722688384099e-05, 
       3.4221942049885680e-05, 
       3.1474261673403497e-05, 
       3.1752446217172570e-05, 
       3.1568797847569072e-05, 
       3.5685618997503639e-05, 
       3.0308628964463674e-05, 
       4.2650090718053424e-05, 
       4.1018583566474474e-05, 
       4.1989409445850875e-05, 
       3.5442604585587855e-05, 
       4.1804760524109131e-05, 
       3.5086256291839228e-05, 
       3.5810119152410999e-05, 
       3.6628483041529297e-05, 
       3.0907207572010308e-05, 
       3.2253429739559717e-05, 
       3.5249617570816589e-05, 
       3.7457071533236017e-05, 
       3.4624590675350692e-05, 
       2.9970807997730558e-05, 
       4.1941240331531230e-05, 
       4.0588555882643076e-05, 
       3.0510810776208460e-05, 
       4.1640392292448351e-05, 
       3.7482181996118599e-05, 
       3.7533477045684776e-05, 
       3.1399725230672576e-05, 
       4.1076066446429675e-05, 
       3.5471457575904934e-05, 
       3.5336312716016542e-05, 
       4.1365813328181145e-05, 
       3.0682845205326052e-05, 
       3.8084296033612795e-05, 
       3.1145184029437977e-05, 
       3.4544896126377564e-05, 
       4.1544997694196912e-05, 
       2.9856066937675739e-05, 
       3.4341734991271874e-05, 
       3.5934040914515704e-05, 
       4.1328180697925300e-05, 
       2.9420608768881635e-05, 
       2.9631070620672174e-05, 
       3.8163247198778232e-05, 
       3.3917607508924799e-05, 
       3.0401451076688072e-05, 
       2.9337676955112569e-05, 
       3.9360086855353777e-05, 
       3.0659718172078284e-05, 
       2.9560824323877005e-05, 
       3.5333635863312106e-05, 
       2.9100768296810072e-05, 
       2.9783808149289946e-05, 
       3.5065947122296143e-05, 
       3.7292062299485572e-05, 
       3.0277993161779557e-05, 
       3.3666400298233992e-05, 
       3.4914136181191511e-05, 
       4.0819206289891856e-05, 
       3.1849335917306130e-05, 
       3.3745003547145633e-05, 
       3.0447909355766206e-05, 
       4.0735350301540957e-05, 
       2.8983900069099673e-05, 
       3.8053646227503343e-05, 
       3.6958017338921532e-05, 
       3.5066206410744284e-05, 
       3.1878230350892937e-05, 
       3.3670823840325130e-05, 
       4.0190618964093548e-05, 
       2.9611762148052136e-05, 
       3.0215572942741212e-05, 
       3.0978705150100671e-05, 
       3.0975116670585716e-05, 
       3.9205890377581426e-05, 
       3.0281667020559267e-05, 
       3.9596006158142123e-05, 
       2.9506513717721074e-05, 
       3.3582847240993350e-05, 
       3.4531769840299485e-05, 
       3.9762088604433533e-05, 
       2.9269265439836080e-05, 
       3.3380022999946215e-05, 
       3.9926407533688928e-05, 
       4.0104613244331801e-05, 
       3.5653058167747033e-05, 
       3.3004955267642635e-05, 
       3.4392622906690786e-05, 
       3.0370417067300626e-05, 
       3.1890047488315731e-05, 
       3.0022017500562490e-05, 
       4.0000003317432134e-05, 
       3.3523408508543471e-05, 
       2.9536082766360954e-05, 
       4.0000000000001989e-05, 
       3.9999999999997374e-05, 
       3.9999999999997374e-05, 
       4.0000000000003371e-05, 
       3.9999999999997909e-05, 
       3.9999999999996947e-05, 
       3.9999999999996175e-05, 
       4.0000000000001148e-05, 
       4.0000000000001148e-05, 
       3.9999999999995246e-05, 
       3.9999999999995246e-05, 
       3.9999999999995246e-05, 
       4.0000000000003832e-05, 
       4.0000000000001894e-05, 
       3.9999999999999481e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       3.1039159892549503e-05, 
       3.3151722660290517e-05, 
       2.8807688058454221e-05, 
       3.9692599755261031e-05, 
       3.4680733919685062e-05, 
       3.2826256554884176e-05, 
       3.8288161925822885e-05, 
       3.4211304406796375e-05, 
       2.9097792994242360e-05, 
       2.8673230559331065e-05, 
       2.8811579142860116e-05, 
       2.9638784793380221e-05, 
       2.8065502476368636e-05, 
       3.7018431922052273e-05, 
       3.1869994579495824e-05, 
       3.7611367219734550e-05, 
       2.9524917489233808e-05, 
       3.8654230861269744e-05, 
       3.8813655096499063e-05, 
       2.8242134260049362e-05, 
       3.3856798589886510e-05, 
       3.6989711056837685e-05, 
       2.8097092194560257e-05, 
       3.2589239900482450e-05, 
       3.0839366780925382e-05, 
       3.0913455647600671e-05, 
       2.8818795374229811e-05, 
       2.7955639415557753e-05, 
       3.8757866071503815e-05, 
       3.2194194668433212e-05, 
       3.1832447493921153e-05, 
       3.2344070446398742e-05, 
       3.7832374205409138e-05, 
       3.4832277859316015e-05, 
       3.2463844769771561e-05, 
       3.2424248890195081e-05, 
       3.0355743865040782e-05, 
       2.7547836766078887e-05, 
       2.8129289982065617e-05, 
       3.0065679269866430e-05, 
       3.0779226484141891e-05, 
       3.4562999970220125e-05, 
       3.3652238989943404e-05, 
       2.7519212508309735e-05, 
       2.8865537325274782e-05, 
       3.4536959440905748e-05, 
       3.6507321778070506e-05, 
       3.1157695519207996e-05, 
       3.0483603651375193e-05, 
       3.7971213702066805e-05, 
       2.7407710915057017e-05, 
       3.8106781457880113e-05, 
       3.6722116726253600e-05, 
       3.8440947193767489e-05, 
       2.8442006127918854e-05, 
       2.8551101375556656e-05, 
       2.9364739526051272e-05, 
       2.7770689262710977e-05, 
       3.4948618872287616e-05, 
       3.2064281429115760e-05, 
       2.7240377824836826e-05, 
       2.7308570833989415e-05, 
       2.8963467376296397e-05, 
       3.4712231642814477e-05, 
       2.7160857684904483e-05, 
       2.9327116841256538e-05, 
       2.8283712735542613e-05, 
       3.6136978569258876e-05, 
       2.8668276433282417e-05, 
       2.7239632087237912e-05, 
       2.7719525839683526e-05, 
       3.3183686698092545e-05, 
       2.8824367755484238e-05, 
       3.4994505665292073e-05, 
       3.8208999710328840e-05, 
       2.8487098891718443e-05, 
       2.7048322588066142e-05, 
       2.7624506260606630e-05, 
       2.7407524675903619e-05, 
       3.0381424342910978e-05, 
       2.8076267860498583e-05, 
       2.8512040773140239e-05, 
       3.4594742813447289e-05, 
       3.6441362101213101e-05, 
       3.6985978886589346e-05, 
       3.2253721185282010e-05, 
       3.2253721185282010e-05, 
       3.0305771304502396e-05, 
       3.7126481627199639e-05, 
       2.7167060333542747e-05, 
       3.8015892034047229e-05, 
       2.7954033262396059e-05, 
       3.4834437483229113e-05, 
       2.6985426747145644e-05, 
       2.7649760287491335e-05, 
       2.9695116186657036e-05, 
       2.8511435627383066e-05, 
       2.9999999999999726e-05, 
       2.9661643064631994e-05, 
       2.8266445242005317e-05, 
       3.0170551527330453e-05, 
       3.3860016083471914e-05, 
       3.1354773595006093e-05, 
       3.1488710584772834e-05, 
       3.1871636737527572e-05, 
       3.2564457829911849e-05, 
       2.7084809742553798e-05, 
       3.6062606881400475e-05, 
       3.7582830542471488e-05, 
       2.7047751291354494e-05, 
       3.2797133827564757e-05, 
       3.1903572974689500e-05, 
       2.8050886898984705e-05, 
       3.1352959491326954e-05, 
       2.6784146102882057e-05, 
       3.7522219749239128e-05, 
       3.6463180721163240e-05, 
       2.9471994694710249e-05, 
       3.1108916789167925e-05, 
       2.7894556225965309e-05, 
       2.6820357129538413e-05, 
       2.7245005270820060e-05, 
       2.6811103636298072e-05, 
       2.7593033786732819e-05, 
       3.6762135698496261e-05, 
       2.8293699618172031e-05, 
       3.3353514605442349e-05, 
       2.6705205624747917e-05, 
       3.3798794399179892e-05, 
       3.2423489616891279e-05, 
       3.7160835699223923e-05, 
       3.0907835110374876e-05, 
       3.7058958692587823e-05, 
       2.7304455332863470e-05, 
       2.7304111819551494e-05, 
       2.6519729660628462e-05, 
       3.7062079460459438e-05, 
       3.0062882662813043e-05, 
       3.7058611877109611e-05, 
       2.8139906563482852e-05, 
       3.4713878905424213e-05, 
       2.9921452903837636e-05, 
       2.9433641374759845e-05, 
       3.5930641659530661e-05, 
       3.2670098265821272e-05, 
       2.9074316244459978e-05, 
       2.9359739277691220e-05, 
       3.5104646844348376e-05, 
       3.5646814842964232e-05, 
       3.2622747588318853e-05, 
       3.5065917243724365e-05, 
       2.8773241079992690e-05, 
       2.8731474658252576e-05, 
       2.7097738525304627e-05, 
       2.7782372773298685e-05, 
       3.2933332913816304e-05, 
       2.7056097000847204e-05, 
       3.6155579985282791e-05, 
       2.9799808496960990e-05, 
       2.6233988731786980e-05, 
       2.6130143230018705e-05, 
       3.6258588441386774e-05, 
       2.6163119095811521e-05, 
       2.8276312058127089e-05, 
       2.6146171841921433e-05, 
       2.6862760269538149e-05, 
       2.6735396114970181e-05, 
       2.7899427385733393e-05, 
       3.2918184537306320e-05, 
       3.1036130902228970e-05, 
       3.1285251208887923e-05, 
       2.9105560696208592e-05, 
       3.0336594799645449e-05, 
       2.7876501617199792e-05, 
       2.9398011659190779e-05, 
       2.7620316784509255e-05, 
       2.7189639453186989e-05, 
       2.6862415599288675e-05, 
       3.6316695117972320e-05, 
       3.0266322085404291e-05, 
       3.1191891642868792e-05, 
       3.0283657005292182e-05, 
       2.9706145186971074e-05, 
       3.5092031119106683e-05, 
       2.6055786019617233e-05, 
       3.1552377055574032e-05, 
       2.5944034925236034e-05, 
       2.9298939491059567e-05, 
       2.8813682520249190e-05, 
       3.0277973911366052e-05, 
       2.6265346091618312e-05, 
       3.4313801964971653e-05, 
       2.8292579250612668e-05, 
       2.5710834411363957e-05, 
       2.7239934780247225e-05, 
       3.0108920080324501e-05, 
       2.6574339259353398e-05, 
       2.6574339259353398e-05, 
       2.6603702800408181e-05, 
       2.9945973306640608e-05, 
       2.7206813548484243e-05, 
       2.9914109083881459e-05, 
       3.5939277874200750e-05, 
       3.5696300850418049e-05, 
       3.5924106303404742e-05, 
       3.5820888095548639e-05, 
       2.6025289810035557e-05, 
       2.9850375573323215e-05, 
       3.2598410800682624e-05, 
       2.9355508235389601e-05, 
       2.7471010759664385e-05, 
       2.9387690540557073e-05, 
       2.6316839759040683e-05, 
       2.6415309608286847e-05, 
       2.6904606101369186e-05, 
       3.5419608296105436e-05, 
       2.7672076424565236e-05, 
       3.3809375937864830e-05, 
       2.5425493738492722e-05, 
       3.4329154938284909e-05, 
       3.4656630397368111e-05, 
       2.7502645097905004e-05, 
       3.0494655538212031e-05, 
       2.9591574561031932e-05, 
       3.5190724841144002e-05, 
       2.9484112638212692e-05, 
       2.6786852929363294e-05, 
       2.6669375901811111e-05, 
       2.9510662695274036e-05, 
       2.9125017675692073e-05, 
       2.5741754881077874e-05, 
       3.3652278080096861e-05, 
       3.5320334181141426e-05, 
       2.8303395946093349e-05, 
       2.6906276959935654e-05, 
       2.6017119619940396e-05, 
       3.5324200559418312e-05, 
       2.6306490025824851e-05, 
       3.4974827291506094e-05, 
       3.4826313408925401e-05, 
       2.7734781787149045e-05, 
       2.5597195153969135e-05, 
       2.4918202035402118e-05, 
       2.8939703220608540e-05, 
       3.5093573708270734e-05, 
       3.4997661313409507e-05, 
       3.3008834750735440e-05, 
       2.7525276060311697e-05, 
       2.8277938055810536e-05, 
       2.8277938055810536e-05, 
       3.1207843012308419e-05, 
       2.6940578710871507e-05, 
       2.5579835732558207e-05, 
       2.6915905993071593e-05, 
       3.4975701650819561e-05, 
       2.4957367328132936e-05, 
       2.4957367328132936e-05, 
       2.5397844703296914e-05, 
       2.6602745558242253e-05, 
       2.5187278275559250e-05, 
       2.5467888641003731e-05, 
       3.0974890461460283e-05, 
       2.5339616967127634e-05, 
       2.9626916887394795e-05, 
       2.7619726290716077e-05, 
       3.4236061275192604e-05, 
       3.4632668262892203e-05, 
       2.8413896081275994e-05, 
       3.4773454537139780e-05, 
       2.5660063211378296e-05, 
       3.1816931570003146e-05, 
       2.8810928217630149e-05, 
       3.2402344758379857e-05, 
       2.4622803671398517e-05, 
       3.3884337582830962e-05, 
       2.4745841016557243e-05, 
       2.8726927520080040e-05, 
       3.1607938084618696e-05, 
       3.0619368734278299e-05, 
       2.5103214760719228e-05, 
       2.6115418378948737e-05, 
       2.5216345416957959e-05, 
       3.4433248607461847e-05, 
       3.4003150069056831e-05, 
       2.6340049545712366e-05, 
       2.8430479102709681e-05, 
       3.4371986165039712e-05, 
       2.5130572792497025e-05, 
       3.4053832022543250e-05, 
       2.8551589404528400e-05, 
       2.8439695471795520e-05, 
       2.9328249229205305e-05, 
       3.4276449161730333e-05, 
       3.4285934210744179e-05, 
       2.4832250539552716e-05, 
       2.4218334389904042e-05, 
       3.4158494666143246e-05, 
       2.8463467350339630e-05, 
       2.5128338253532572e-05, 
       2.8849586357006848e-05, 
       2.4239171304499631e-05, 
       2.5847121689263722e-05, 
       2.4185026488689110e-05, 
       2.4667869299451022e-05, 
       2.4346446619042764e-05, 
       3.0824640538398941e-05, 
       2.4498728162961109e-05, 
       2.8302190214678204e-05, 
       2.7394076866844276e-05, 
       3.1153783358348083e-05, 
       2.3968252203040222e-05, 
       2.6052604265088291e-05, 
       2.8126871028984526e-05, 
       2.5603416304878284e-05, 
       2.8985004808456365e-05, 
       3.3589667866692417e-05, 
       2.3977472847857841e-05, 
       3.2227122415746155e-05, 
       2.5558419731325131e-05, 
       3.3690049600477206e-05, 
       2.4381272702297991e-05, 
       3.2346196530799784e-05, 
       2.5097609460247766e-05, 
       2.9905857527565588e-05, 
       3.1439720562106301e-05, 
       3.0018041084816379e-05, 
       3.2250837542768495e-05, 
       2.4304876842231052e-05, 
       3.3099265112919020e-05, 
       3.3552675389070232e-05, 
       2.9733487432063609e-05, 
       2.5558621357365356e-05, 
       2.5558621357365356e-05, 
       2.3990190883500398e-05, 
       2.5729830495522413e-05, 
       2.9362739923969074e-05, 
       2.3770852707783487e-05, 
       2.5028438173173765e-05, 
       3.2319648557118898e-05, 
       2.7822681342999946e-05, 
       2.4895968693405063e-05, 
       2.4895968693405063e-05, 
       2.7009589427507814e-05, 
       2.7432225192304068e-05, 
       2.8082407773897595e-05, 
       2.3589132420774609e-05, 
       2.7690075248950416e-05, 
       2.8518180427709347e-05, 
       2.3569263208211297e-05, 
       3.2793897924720190e-05, 
       2.4100745386675573e-05, 
       2.4279370515244482e-05, 
       3.1609514834965040e-05, 
       2.4837349791581452e-05, 
       2.8070337592737301e-05, 
       3.3143920272160123e-05, 
       2.7592067227439496e-05, 
       3.0171198763932223e-05, 
       2.4287564525222680e-05, 
       2.3989320553293856e-05, 
       2.7390072147498527e-05, 
       3.2518549524648607e-05, 
       2.4312910540716258e-05, 
       2.7390933708753500e-05, 
       2.6836730589191729e-05, 
       2.4637614711543699e-05, 
       2.8937923332251048e-05, 
       2.8934320666911560e-05, 
       2.3443795183707881e-05, 
       3.2966884370293322e-05, 
       2.3499458233154702e-05, 
       2.6756094696910096e-05, 
       3.1622971315236877e-05, 
       2.7380720405324034e-05, 
       2.4552508401457934e-05, 
       2.4552508401457934e-05, 
       2.4592888205323805e-05, 
       2.3479089004450081e-05, 
       3.2033337234628308e-05, 
       3.2837736143775106e-05, 
       2.3494746002207493e-05, 
       3.1688839173662951e-05, 
       3.2558652334238170e-05, 
       2.4496665591143735e-05, 
       3.2453869714794841e-05, 
       3.0000168399366491e-05, 
       3.0389503000317718e-05, 
       2.5367567861720414e-05, 
       3.1013184082052318e-05, 
       2.7206863554059752e-05, 
       2.5631609024973996e-05, 
       3.2632379530041206e-05, 
       2.7173856454263550e-05, 
       2.3139042828271667e-05, 
       2.5910488673821892e-05, 
       2.3835090517009871e-05, 
       3.2676096095116880e-05, 
       2.3115738546502348e-05, 
       2.5149840529202073e-05, 
       3.0000000000003914e-05, 
       2.5103814782062271e-05, 
       2.5561453121161013e-05, 
       2.4011247562584379e-05, 
       2.8229759568981670e-05, 
       3.0954827358492883e-05, 
       2.9999999999995010e-05, 
       2.9999999999995972e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.6638076735884128e-05, 
       2.9800443358011636e-05, 
       2.9957569938508312e-05, 
       2.6644792797369336e-05, 
       3.2449119475693635e-05, 
       3.2307270591818477e-05, 
       2.3855033202891477e-05, 
       2.2919945327084375e-05, 
       3.1181947945255791e-05, 
       3.0966962655152635e-05, 
       2.3618829129352344e-05, 
       2.6729763253723063e-05, 
       2.6251435400914067e-05, 
       2.6992323055423699e-05, 
       2.8808352255079773e-05, 
       3.1674498987854625e-05, 
       2.8845868164023834e-05, 
       2.2855762949261972e-05, 
       2.4475093841118160e-05, 
       3.0469422424589034e-05, 
       3.2121103410085548e-05, 
       2.6738752853282381e-05, 
       2.6199303457985351e-05, 
       2.5917549322748828e-05, 
       2.3972067650946521e-05, 
       2.3642004655701465e-05, 
       3.0111710482434150e-05, 
       2.2690055869077828e-05, 
       2.3972371441272996e-05, 
       2.7429533393715458e-05, 
       2.6609783593678770e-05, 
       3.0418182789536737e-05, 
       3.0729933159896485e-05, 
       3.0519892781556383e-05, 
       3.0519892781556383e-05, 
       3.1927209051499637e-05, 
       3.1412944191601312e-05, 
       2.6695502529920774e-05, 
       2.2747197677377404e-05, 
       2.2747197677377404e-05, 
       2.6927910644424263e-05, 
       2.3159964613851685e-05, 
       2.3449501588639427e-05, 
       3.1129140282794502e-05, 
       2.2661199232478790e-05, 
       3.1756464109022012e-05, 
       3.0000000000003331e-05, 
       3.1715957258018557e-05, 
       3.0493385472890526e-05, 
       2.2533690673819412e-05, 
       2.8841698703111353e-05, 
       2.7915941786718987e-05, 
       2.2787211817273076e-05, 
       2.6298737908416582e-05, 
       2.5754281679001387e-05, 
       2.3252148712137995e-05, 
       2.7892934451784692e-05, 
       2.7951013451930176e-05, 
       2.2465698897257985e-05, 
       2.2440742103618684e-05, 
       2.9542188794899120e-05, 
       2.4290663709578578e-05, 
       2.6350256389484496e-05, 
       2.5734503827461567e-05, 
       2.2412536595775360e-05, 
       2.7124462690205231e-05, 
       2.7177674193646515e-05, 
       3.1618651481483873e-05, 
       2.2432426536989266e-05, 
       2.4189720357826928e-05, 
       2.6648169829749547e-05, 
       2.7164941871740675e-05, 
       2.8077366384963156e-05, 
       2.6578976457764752e-05, 
       2.6212878484818197e-05, 
       2.2788776745599077e-05, 
       2.3072473188098390e-05, 
       2.7429999248861801e-05, 
       2.9908124403016638e-05, 
       2.2874307841218250e-05, 
       2.9376128973940538e-05, 
       3.0834070661572225e-05, 
       3.0834070661572225e-05, 
       2.4923884977834289e-05, 
       3.1182792115445008e-05, 
       2.9391533845460121e-05, 
       2.2537043954458916e-05, 
       2.3663334500167565e-05, 
       2.4082184739369677e-05, 
       2.2059381617659331e-05, 
       2.7013731654013261e-05, 
       2.5866503261118416e-05, 
       3.1123506807834984e-05, 
       3.1058236474875583e-05, 
       2.2042690047119963e-05, 
       2.3501143967570068e-05, 
       2.5264475481345029e-05, 
       2.2291403303108609e-05, 
       2.5789041976159144e-05, 
       2.5752625404385004e-05, 
       2.2419785171064484e-05, 
       2.3032330547245072e-05, 
       2.5500808155918871e-05, 
       2.2007811230651764e-05, 
       2.8595489040144257e-05, 
       2.5743081743900583e-05, 
       3.0833856686922384e-05, 
       2.2953013898141161e-05, 
       2.8060586309824298e-05, 
       2.2049482271398904e-05, 
       2.5559309080125467e-05, 
       2.1906982419196531e-05, 
       2.1774947899687872e-05, 
       2.5850814625073328e-05, 
       2.8677002986771678e-05, 
       2.3973213706637462e-05, 
       2.5456386562570028e-05, 
       3.0686887089838080e-05, 
       2.9860940400375449e-05, 
       2.5914801880035124e-05, 
       3.0582199910042621e-05, 
       2.9436827774699843e-05, 
       2.5595288355060954e-05, 
       2.9210645747847836e-05, 
       2.7180281847628045e-05, 
       2.2483651761501816e-05, 
       2.4667152066401473e-05, 
       2.1893746690499035e-05, 
       2.1933041582336083e-05, 
       3.0058277854270268e-05, 
       2.2133560849025269e-05, 
       2.8096083071044486e-05, 
       2.5344389356141837e-05, 
       2.9443800220647262e-05, 
       2.9951036745156710e-05, 
       2.9945849344433220e-05, 
       2.5093598549897843e-05, 
       2.4843453873112511e-05, 
       2.7371136082427610e-05, 
       2.5072399592830789e-05, 
       2.5214185493224736e-05, 
       2.5214185035996223e-05, 
       2.5004525511801097e-05, 
       2.8979278302965828e-05, 
       2.5186051604522636e-05, 
       2.5348828265287213e-05, 
       2.5244359735749165e-05, 
       2.9326966998297700e-05, 
       2.2697014348075652e-05, 
       2.9394368400131825e-05, 
       2.1813612638002743e-05, 
       2.5046979639083853e-05, 
       2.4157479821597049e-05, 
       2.3222770682420255e-05, 
       2.3314403055625338e-05, 
       2.2570661138101929e-05, 
       2.4908783871480569e-05, 
       2.2505905708169625e-05, 
       2.6258638767916868e-05, 
       2.3434371640934816e-05, 
       3.0133423094276347e-05, 
       2.7591028764093684e-05, 
       2.2061710501047537e-05, 
       2.1628895259523690e-05, 
       2.1358429561149283e-05, 
       2.1427494407500420e-05, 
       2.3167503033159669e-05, 
       2.2930017223228041e-05, 
       3.0000000000001031e-05, 
       3.0000000000003592e-05, 
       2.9999999999998828e-05, 
       3.0000000000004216e-05, 
       2.9999999999999855e-05, 
       2.9999999999998398e-05, 
       3.0000000000004487e-05, 
       3.0000000000001681e-05, 
       3.0000000000001105e-05, 
       2.9999999999999310e-05, 
       2.9999999999998435e-05, 
       2.9999999999998473e-05, 
       3.0000000000001292e-05, 
       2.9999999999997765e-05, 
       3.0000000000001674e-05, 
       2.9999999999999940e-05, 
       2.9999999999999940e-05, 
       2.9999999999999113e-05, 
       2.9999999999999205e-05, 
       2.9999999999999205e-05, 
       2.9999999999999205e-05, 
       3.0000000000003369e-05, 
       3.0000000000003369e-05, 
       2.9999999999998937e-05, 
       2.9999999999996423e-05, 
       3.0000000000004934e-05, 
       3.0000000000004859e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.7977786049804914e-05, 
       2.4781209860024590e-05, 
       2.1192390191768415e-05, 
       2.1192390191768415e-05, 
       2.2681904047265785e-05, 
       2.1881636715936563e-05, 
       2.9360956315481421e-05, 
       2.1831868130968220e-05, 
       2.9707174814456383e-05, 
       2.8404473381637825e-05, 
       2.9380613966485565e-05, 
       2.9373491294223167e-05, 
       2.1185461954203410e-05, 
       2.7203812003774421e-05, 
       2.5018477899419453e-05, 
       2.8712262856748577e-05, 
       2.9431709214940752e-05, 
       2.1692084655893151e-05, 
       2.7655972980607960e-05, 
       2.1897724635915417e-05, 
       2.4093131058151122e-05, 
       2.5790479923422541e-05, 
       2.9550736953917446e-05, 
       2.9369626226073478e-05, 
       2.1394745953434777e-05, 
       2.1888100646980633e-05, 
       2.0904220768728736e-05, 
       2.9465166307386468e-05, 
       2.5334505925107406e-05, 
       2.1375703784200034e-05, 
       2.1290565166949428e-05, 
       2.7069699230712933e-05, 
       2.6902666915836786e-05, 
       2.1883213951128004e-05, 
       2.4074314693100887e-05, 
       2.8562172022423441e-05, 
       2.4375815007945829e-05, 
       2.6115525504441513e-05, 
       2.1264298669881058e-05, 
       2.3181063810889197e-05, 
       2.9195675414178763e-05, 
       2.1835144903334811e-05, 
       2.0933184905867349e-05, 
       2.9223290880005068e-05, 
       2.0767081796558923e-05, 
       2.2678513821181670e-05, 
       2.9022835142669393e-05, 
       2.0740605279931568e-05, 
       2.2959758470424977e-05, 
       2.1756379400670785e-05, 
       2.6227865079976737e-05, 
       2.4892524129874611e-05, 
       2.4204463163150561e-05, 
       2.4222973587626428e-05, 
       2.8954710823700344e-05, 
       2.1667908739356850e-05, 
       2.2257311143512702e-05, 
       2.9023120902266073e-05, 
       2.3962968444884479e-05, 
       2.0637003746839989e-05, 
       2.0707409205838624e-05, 
       2.4902832791433148e-05, 
       2.0729249339574016e-05, 
       2.0720092835467932e-05, 
       2.2159633949716909e-05, 
       2.8901253631823843e-05, 
       2.4013863790456743e-05, 
       2.8415383314476248e-05, 
       2.3837597752080925e-05, 
       2.2460361097944093e-05, 
       2.2460361097944093e-05, 
       2.1627859486552423e-05, 
       2.2026778583341010e-05, 
       2.2026778583341010e-05, 
       2.0409577908840788e-05, 
       2.0797163145969286e-05, 
       2.2026357651294454e-05, 
       2.1896156195968891e-05, 
       2.2493976131117383e-05, 
       2.3561936361104048e-05, 
       2.3826450639880401e-05, 
       2.3662155848727076e-05, 
       2.7313184250191240e-05, 
       2.1545807116821245e-05, 
       2.0663068717321193e-05, 
       2.1109864205007314e-05, 
       2.2297094153571563e-05, 
       2.8587967460179920e-05, 
       2.3942355574288436e-05, 
       2.6953238336552894e-05, 
       2.1769339705180965e-05, 
       2.3724586078780852e-05, 
       2.7749220345302695e-05, 
       2.1274716056221353e-05, 
       2.3660199491775747e-05, 
       2.1684275785027130e-05, 
       2.1127601084820226e-05, 
       2.2245465324112150e-05, 
       2.4134492556523332e-05, 
       2.4134492556523332e-05, 
       2.0702109877429871e-05, 
       2.5415065232737208e-05, 
       2.5049164336646217e-05, 
       2.3711982590933556e-05, 
       2.0903996480810332e-05, 
       2.5477118471368847e-05, 
       2.7066217879222983e-05, 
       2.7066217879222983e-05, 
       2.0087756354107088e-05, 
       2.7407872901961955e-05, 
       2.5752217941525236e-05, 
       2.3509414389390902e-05, 
       2.2137284082853892e-05, 
       2.7868825096435729e-05, 
       2.1103925210061340e-05, 
       2.0000000000003136e-05, 
       2.7960841711629158e-05, 
       2.0128222232666396e-05, 
       2.8218563122518728e-05, 
       2.5887484385715143e-05, 
       2.7080687592378696e-05, 
       2.0868424239093153e-05, 
       2.2393731783960902e-05, 
       2.1124097199525199e-05, 
       2.0646704939798299e-05, 
       2.1425648591584204e-05, 
       2.5913907913736285e-05, 
       2.0677363797620225e-05, 
       2.3510576371202193e-05, 
       2.7057327699278204e-05, 
       2.2231911984833502e-05, 
       2.0054709668727553e-05, 
       2.0931915255503046e-05, 
       2.6613011953124452e-05, 
       2.6896352990146805e-05, 
       1.9899446274643278e-05, 
       2.0352943326531300e-05, 
       1.9845156033951324e-05, 
       2.0841047026937218e-05, 
       1.9825607164398136e-05, 
       2.1032338146034649e-05, 
       2.7804096630990566e-05, 
       2.6708948925327193e-05, 
       2.0997523170847318e-05, 
       2.0251870492957298e-05, 
       2.3145181527697719e-05, 
       1.9735478473128204e-05, 
       1.9723699235144497e-05, 
       2.7789289624749064e-05, 
       2.3329997431946800e-05, 
       2.0124318481218082e-05, 
       2.6253986385102155e-05, 
       2.5954718992651673e-05, 
       2.3990190883495407e-05, 
       2.1847220710856864e-05, 
       1.9881665767775377e-05, 
       2.3837718355342495e-05, 
       2.4834849775167029e-05, 
       1.9845056688366133e-05, 
       2.2984918779398254e-05, 
       2.2051791107932350e-05, 
       2.7687978543504833e-05, 
       2.0124311182856182e-05, 
       2.2969720901820586e-05, 
       2.3379719824416396e-05, 
       2.3045673572194488e-05, 
       2.4401844797352267e-05, 
       2.3017821626053330e-05, 
       2.1009850269871334e-05, 
       2.4604439170047656e-05, 
       2.6502057660116060e-05, 
       2.0915044009404437e-05, 
       2.0557490844970100e-05, 
       2.2303791106947136e-05, 
       2.3568849536529308e-05, 
       2.7487887027958263e-05, 
       2.7243831302811930e-05, 
       2.7401549863970587e-05, 
       2.2844170635079498e-05, 
       2.5428459492323464e-05, 
       2.0449537282602713e-05, 
       2.7315086935866378e-05, 
       2.7321089736325289e-05, 
       2.2223161106287887e-05, 
       2.0761232511071340e-05, 
       2.2722968226404221e-05, 
       2.0375912450819512e-05, 
       2.1982535867854892e-05, 
       2.3760552111706295e-05, 
       2.4657303494521758e-05, 
       2.0009857162887701e-05, 
       2.3726374972784535e-05, 
       2.1342289921849532e-05, 
       2.2108625191178818e-05, 
       2.2107918319979281e-05, 
       2.0502905389639388e-05, 
       2.5119528436742323e-05, 
       1.9516726581836720e-05, 
       2.7191646189580940e-05, 
       2.7108638240423049e-05, 
       2.2581685122227867e-05, 
       2.4813088265878294e-05, 
       2.2251956490045338e-05, 
       2.1105202602793973e-05, 
       2.3565849863946431e-05, 
       2.3563061638380849e-05, 
       2.0984009709726171e-05, 
       2.7090345596650079e-05, 
       2.3092205076770237e-05, 
       2.2508397829033177e-05, 
       2.5237376640927738e-05, 
       2.6845718547956329e-05, 
       2.0269194115185238e-05, 
       1.9178006383137801e-05, 
       2.6943891374647128e-05, 
       2.4011762522962640e-05, 
       2.2198128590104049e-05, 
       2.6661800044791447e-05, 
       2.0246033691061394e-05, 
       2.3782960195173511e-05, 
       1.9077211035467425e-05, 
       2.0130524220147914e-05, 
       1.9055041365690265e-05, 
       2.0061290439785591e-05, 
       2.2352715474766112e-05, 
       1.9368651876453975e-05, 
       2.3069805257315336e-05, 
       2.6867279556170112e-05, 
       2.2539563108794425e-05, 
       1.9999999999999588e-05, 
       2.3426979083492160e-05, 
       1.9348046020852360e-05, 
       2.2268611352024889e-05, 
       2.3366362904907280e-05, 
       2.2134011819243862e-05, 
       2.5834600287792073e-05, 
       2.2215721280363147e-05, 
       2.6660259341866286e-05, 
       2.2955372524408110e-05, 
       2.4461292269898077e-05, 
       2.5742500761044551e-05, 
       2.2173339694577063e-05, 
       2.6201512269285688e-05, 
       1.9638095676474543e-05, 
       1.9638095676474543e-05, 
       2.6608693063682616e-05, 
       1.9851434587407395e-05, 
       1.9283086234511842e-05, 
       1.9767970879306379e-05, 
       1.9271186973558166e-05, 
       2.5946749959651857e-05, 
       2.5820386282369037e-05, 
       1.9896871203278959e-05, 
       2.2067358137384505e-05, 
       1.9270178024292573e-05, 
       2.6130655900719434e-05, 
       2.6507400532632928e-05, 
       2.6407499725398140e-05, 
       2.6407499725398140e-05, 
       1.9111331917637347e-05, 
       2.2381794194760051e-05, 
       2.0198999310674607e-05, 
       2.1939827897244711e-05, 
       2.2034500481798385e-05, 
       1.9334266572284064e-05, 
       2.2039202088095241e-05, 
       2.2555828107000100e-05, 
       1.8694500382365884e-05, 
       2.0000000000003627e-05, 
       1.9999999999999195e-05, 
       2.6397551980700827e-05, 
       1.9282667617978935e-05, 
       1.8682110802826321e-05, 
       2.0021454763490297e-05, 
       1.9369387123481113e-05, 
       2.1897888923757808e-05, 
       2.1886458127008223e-05, 
       1.8941901803643449e-05, 
       2.1575136822679403e-05, 
       2.1690264765648639e-05, 
       2.2546380808982550e-05, 
       1.9108770638817880e-05, 
       2.3793034923037692e-05, 
       1.9646049935609245e-05, 
       1.9646049935609245e-05, 
       2.4898340085266668e-05, 
       1.8702432222567177e-05, 
       2.1337110066930517e-05, 
       1.8754960889276679e-05, 
       2.1920838420412080e-05, 
       1.8745661317307797e-05, 
       2.2012993885403050e-05, 
       1.9662509837507125e-05, 
       2.5149343542068934e-05, 
       2.1835443517498511e-05, 
       2.1432576408466843e-05, 
       2.0890346652158772e-05, 
       2.0041285728824034e-05, 
       2.0201550973463700e-05, 
       2.2369988673688493e-05, 
       2.1082677410618358e-05, 
       1.9475892497164422e-05, 
       2.0430841162105277e-05, 
       2.5616881089262331e-05, 
       2.5561524976553051e-05, 
       2.2256720600395467e-05, 
       1.8575965995141460e-05, 
       2.1582267125339719e-05, 
       2.0380664050224007e-05, 
       2.4045915975920568e-05, 
       2.5413815641187998e-05, 
       1.9376536056028707e-05, 
       2.1736695350079003e-05, 
       1.8632468678587944e-05, 
       2.1373167301566084e-05, 
       2.5243812758073397e-05, 
       2.5514214161906033e-05, 
       2.5833897822691343e-05, 
       1.8603444773941460e-05, 
       1.8329881415074994e-05, 
       1.9763816058288280e-05, 
       1.9763816058288280e-05, 
       2.2641428627869301e-05, 
       2.1437640160066925e-05, 
       1.8618202683601985e-05, 
       2.3850057869665633e-05, 
       2.0257013634953510e-05, 
       2.5813702743402554e-05, 
       2.5161700961535786e-05, 
       2.4500308605386585e-05, 
       2.1581356787528601e-05, 
       1.9319643385351092e-05, 
       1.9195393822148234e-05, 
       1.8318699574533651e-05, 
       1.8322037504694979e-05, 
       1.9687517411603891e-05, 
       2.2092577492403505e-05, 
       2.1181450893396919e-05, 
       1.8441641969427584e-05, 
       2.0690315870433976e-05, 
       2.0767286104586677e-05, 
       2.5668860259396340e-05, 
       1.9999999999996691e-05, 
       1.8216488992965863e-05, 
       2.1448785391892901e-05, 
       1.9128160866352327e-05, 
       1.8192542188358763e-05, 
       1.9620107864103088e-05, 
       2.1331047324169167e-05, 
       2.1301162457843403e-05, 
       2.5094107345774041e-05, 
       2.5345688678480327e-05, 
       1.8545317730810682e-05, 
       2.1501412452381147e-05, 
       2.2142057021603587e-05, 
       2.2142057021603587e-05, 
       2.1135275541214730e-05, 
       1.8494265276753072e-05, 
       2.5550953038270339e-05, 
       1.9564063852232794e-05, 
       1.9000981440101349e-05, 
       1.8655156555063814e-05, 
       2.5445532659369853e-05, 
       2.3830599867500915e-05, 
       1.8028754047776348e-05, 
       2.4137725314921818e-05, 
       2.1125424240387695e-05, 
       2.1258657835825322e-05, 
       2.4059491545958614e-05, 
       2.0242709299960099e-05, 
       1.7949021871998564e-05, 
       2.0463012973560527e-05, 
       1.9292773873423911e-05, 
       2.3065646511066904e-05, 
       1.9073307490169593e-05, 
       1.7924314853027964e-05, 
       2.0817589538067585e-05, 
       1.7914729486010503e-05, 
       2.1375688443046898e-05, 
       1.9481609739293776e-05, 
       2.2195282689139113e-05, 
       2.5033573767916607e-05, 
       1.8806642782522944e-05, 
       1.8682168791402342e-05, 
       1.9078981517641491e-05, 
       2.0824183766240837e-05, 
       1.7707290961288495e-05, 
       2.4970512379188960e-05, 
       1.8185021483994586e-05, 
       1.8261401155073962e-05, 
       2.3069120608504299e-05, 
       2.3069120608504299e-05, 
       2.0682214549822705e-05, 
       1.8626575200708320e-05, 
       1.9916922034440399e-05, 
       1.8640395583295617e-05, 
       1.8640395583295617e-05, 
       1.7614400745337066e-05, 
       2.0416043178336156e-05, 
       2.4589376309026510e-05, 
       1.8217254129423565e-05, 
       1.9413476597831931e-05, 
       2.4643901058988932e-05, 
       1.7574523066846941e-05, 
       2.4640116028100371e-05, 
       2.4640116028100371e-05, 
       2.4696147490686706e-05, 
       2.0491678912308683e-05, 
       1.8957136937250149e-05, 
       2.4657848950645196e-05, 
       2.4652030414921975e-05, 
       2.0000000000002583e-05, 
       2.4611661546530794e-05, 
       2.1530717511554056e-05, 
       2.0470849578236086e-05, 
       2.4219917468853571e-05, 
       2.3621308320517317e-05, 
       2.4568622360301528e-05, 
       1.9589116629270218e-05, 
       2.2748107542386935e-05, 
       2.3791632997449517e-05, 
       2.1023325737590410e-05, 
       1.7911409449688488e-05, 
       1.7575495673165250e-05, 
       2.0433389575409986e-05, 
       2.2341400891196384e-05, 
       1.7307849619763936e-05, 
       1.7734620493079325e-05, 
       1.7521271150068636e-05, 
       2.4381782421029007e-05, 
       1.7792815146452841e-05, 
       2.2937659393765543e-05, 
       2.0549308294811750e-05, 
       2.0912790456574726e-05, 
       1.9708477221209908e-05, 
       2.0669294698547308e-05, 
       1.8278942905448100e-05, 
       1.7300223475597531e-05, 
       2.1258409994565152e-05, 
       1.7298936232489338e-05, 
       2.4230916131844008e-05, 
       2.0260814967144021e-05, 
       2.1574472545965845e-05, 
       1.7355504855513649e-05, 
       2.4253538317534649e-05, 
       2.4192694215193304e-05, 
       1.8538766686584639e-05, 
       2.3425112168981819e-05, 
       2.2870792928003896e-05, 
       2.0039949015281509e-05, 
       1.8071154484291160e-05, 
       2.0320444398046786e-05, 
       1.8628515855767538e-05, 
       2.0527350002612239e-05, 
       1.7425273847949847e-05, 
       2.3806116634567082e-05, 
       2.1361339222984295e-05, 
       1.9832363163017510e-05, 
       2.2980753572720978e-05, 
       1.7245410421208641e-05, 
       1.7245410421208641e-05, 
       1.7663116451282980e-05, 
       1.8171953776881295e-05, 
       1.8756189313190220e-05, 
       1.7244694892905224e-05, 
       1.8169556565617902e-05, 
       1.7683924920349020e-05, 
       1.8342545908721153e-05, 
       2.3579778641489104e-05, 
       2.3579778641489104e-05, 
       2.2905822401600713e-05, 
       2.0093647699069538e-05, 
       1.8408002083955171e-05, 
       2.3488438511504516e-05, 
       2.3642086720562421e-05, 
       1.6970077202054217e-05, 
       1.7898575231892403e-05, 
       2.3531961271458973e-05, 
       2.3887809854504896e-05, 
       1.6958551214762053e-05, 
       2.3872360119663840e-05, 
       1.9839895212892159e-05, 
       1.8458838042259074e-05, 
       2.3895184889918358e-05, 
       1.9780310285029106e-05, 
       1.8142992038481507e-05, 
       1.6904485802710315e-05, 
       1.7145380567773708e-05, 
       1.8678559660865059e-05, 
       1.8676933459739412e-05, 
       2.3644979735206557e-05, 
       1.7106401237428762e-05, 
       2.3782731311723148e-05, 
       1.9608438369471971e-05, 
       1.9152380286727763e-05, 
       1.8940503300687458e-05, 
       2.3692946635835630e-05, 
       2.1390713579329950e-05, 
       1.9136096999761468e-05, 
       1.9999999999998680e-05, 
       1.7068780125830173e-05, 
       1.9685800512948540e-05, 
       2.2061355733952665e-05, 
       1.6756109219293494e-05, 
       1.9017772303267000e-05, 
       2.1684262333435649e-05, 
       1.8012208786142678e-05, 
       1.8012208786142678e-05, 
       2.2730062662028911e-05, 
       2.3569072491019156e-05, 
       1.6684917332045796e-05, 
       1.7080947158174366e-05, 
       1.6784263583268978e-05, 
       2.0945619289330842e-05, 
       2.2529044614750575e-05, 
       1.9364933915967557e-05, 
       2.0319266578649607e-05, 
       1.6644766925135311e-05, 
       1.6681531455427336e-05, 
       1.6692472119204529e-05, 
       2.3033399633323115e-05, 
       1.9332705412142697e-05, 
       1.7519492748762508e-05, 
       1.9330753562258820e-05, 
       1.6999776530199162e-05, 
       1.6829521316593051e-05, 
       2.0428762420327374e-05, 
       1.8591388650631442e-05, 
       1.7364302853916416e-05, 
       1.6560156314027844e-05, 
       1.6560156314027844e-05, 
       1.7349040404026116e-05, 
       2.1016574098868176e-05, 
       1.6757367952506360e-05, 
       1.7393157970824468e-05, 
       1.7504296991640123e-05, 
       1.6985127267371464e-05, 
       2.3269985597394429e-05, 
       2.3319167888592631e-05, 
       1.9356801909263181e-05, 
       1.7498754834802578e-05, 
       1.6807251875556978e-05, 
       1.6777951013067581e-05, 
       1.9157356595498308e-05, 
       1.9997507073835037e-05, 
       1.6551709086558985e-05, 
       2.1906997649233641e-05, 
       1.9544192400014738e-05, 
       2.1043724346603977e-05, 
       1.8240446434156144e-05, 
       2.3250011793104764e-05, 
       1.9013248481614200e-05, 
       2.3074855563075498e-05, 
       1.9304039766842712e-05, 
       1.8501413240458147e-05, 
       1.6419466302874748e-05, 
       1.9306624858929919e-05, 
       1.9302420962345517e-05, 
       2.0099874790470865e-05, 
       2.0099119625085626e-05, 
       1.7181973285697232e-05, 
       1.7249835267180766e-05, 
       1.7788070523349194e-05, 
       2.2420219424819730e-05, 
       1.6675585728044082e-05, 
       1.7252988806554564e-05, 
       1.6780507453344162e-05, 
       1.7752341584529981e-05, 
       1.8829169324520366e-05, 
       1.7541492588550479e-05, 
       1.7261342986004134e-05, 
       1.6473552121557051e-05, 
       1.6726797339031451e-05, 
       1.9099278465665695e-05, 
       1.6508317787735880e-05, 
       1.6345299668428689e-05, 
       2.0636986590934491e-05, 
       2.2316664324812403e-05, 
       1.9361491496825064e-05, 
       1.9492763683807662e-05, 
       1.8661835328000411e-05, 
       2.2270138151235147e-05, 
       1.6261959569716738e-05, 
       2.2680281059748456e-05, 
       1.6409527881725882e-05, 
       1.7187706513412739e-05, 
       1.7747808708896180e-05, 
       2.1893576160653762e-05, 
       1.6368441116126211e-05, 
       2.1886823911959488e-05, 
       1.8950025164632148e-05, 
       1.7894628316429601e-05, 
       2.0268576500937218e-05, 
       2.0106056539561648e-05, 
       1.8908707048867289e-05, 
       1.6303884265495167e-05, 
       1.8307849129973548e-05, 
       2.2652702311329514e-05, 
       2.2563947078767160e-05, 
       2.0790476275472836e-05, 
       2.2631806185743912e-05, 
       2.2616595770205090e-05, 
       1.9056683565391087e-05, 
       1.9039176789753157e-05, 
       1.6063784402242326e-05, 
       1.8464144511990500e-05, 
       1.6980010174667729e-05, 
       1.7273639214441055e-05, 
       1.8316207380717642e-05, 
       1.8794390188883509e-05, 
       1.8793118845375037e-05, 
       1.9399915953460206e-05, 
       1.8191255489112355e-05, 
       1.7243911997532744e-05, 
       2.0510714603222904e-05, 
       1.8288067043859070e-05, 
       1.6843815475317242e-05, 
       1.6782111658767569e-05, 
       2.0467052476129889e-05, 
       2.2307021864782610e-05, 
       1.6361777316992570e-05, 
       1.8584062651128511e-05, 
       1.7327024723531550e-05, 
       1.6733489485492690e-05, 
       1.7047740426222983e-05, 
       2.0993967840435955e-05, 
       1.6500152445011802e-05, 
       1.8001140136188653e-05, 
       1.6842787612327909e-05, 
       2.2285413808576788e-05, 
       1.7304528192344530e-05, 
       1.9440809173745262e-05, 
       1.9408652860286046e-05, 
       2.2308965890963654e-05, 
       1.7059894269898029e-05, 
       1.7299463701415883e-05, 
       1.8215888997602690e-05, 
       1.9412749608311289e-05, 
       1.5987760378662708e-05, 
       1.8817898978253875e-05, 
       1.8817898978253875e-05, 
       1.7973693338197817e-05, 
       1.7959060371545117e-05, 
       2.2178007724241785e-05, 
       1.6485000701342114e-05, 
       2.1883837894146590e-05, 
       1.6370014778855305e-05, 
       1.7461419958754493e-05, 
       1.9009122775459467e-05, 
       2.1415671267411649e-05, 
       1.6251923361205683e-05, 
       1.7677216822713677e-05, 
       2.2119426582307980e-05, 
       1.7139566016459397e-05, 
       2.2127131281547109e-05, 
       1.6484733306001025e-05, 
       1.6009518260357110e-05, 
       2.2088606066161964e-05, 
       1.6843372422457200e-05, 
       1.9703838718187286e-05, 
       1.5729701112158979e-05, 
       1.5605336586737418e-05, 
       2.0334043829559569e-05, 
       1.8093457164273500e-05, 
       1.7018370486444907e-05, 
       1.8092780768013057e-05, 
       2.1935861477145441e-05, 
       1.7821825649641092e-05, 
       1.7820806697276662e-05, 
       2.0679450147518896e-05, 
       2.1807114602056485e-05, 
       1.5514078046660234e-05, 
       2.1448112769470992e-05, 
       1.9348952945848586e-05, 
       1.7001177995273558e-05, 
       2.1043924230317544e-05, 
       1.6789717514511149e-05, 
       1.8155898825021778e-05, 
       1.7859594553655210e-05, 
       1.6087138253532083e-05, 
       1.7400873911920310e-05, 
       1.5502385879772839e-05, 
       1.8138713559224012e-05, 
       1.8020200214705512e-05, 
       2.0682001770383132e-05, 
       2.0116860434379703e-05, 
       1.7027776481093546e-05, 
       1.8108603465259826e-05, 
       2.1776543849616935e-05, 
       2.1142785652008354e-05, 
       1.8095736251542919e-05, 
       1.7471878746700059e-05, 
       1.6209419085385878e-05, 
       1.5776045047487888e-05, 
       1.8614429820787796e-05, 
       2.1466794788251957e-05, 
       1.5824364094885964e-05, 
       1.9999999999995160e-05, 
       2.0000000000004284e-05, 
       1.9999999999995407e-05, 
       1.9999999999998504e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       1.9999999999999853e-05, 
       2.0000000000004603e-05, 
       1.9999999999996295e-05, 
       1.6175978944920478e-05, 
       1.6719774526569959e-05, 
       1.7686564612508700e-05, 
       2.0813687420850283e-05, 
       1.5660071132120860e-05, 
       1.7989800407954048e-05, 
       1.6131548965074918e-05, 
       2.1616983409057042e-05, 
       1.8598768300361032e-05, 
       1.9607223872270877e-05, 
       1.6963892560874945e-05, 
       2.1366772388284552e-05, 
       1.7359940005171894e-05, 
       1.8507199444334618e-05, 
       2.0750008388237536e-05, 
       1.6868254967829275e-05, 
       1.6423042036472158e-05, 
       1.6423042036472158e-05, 
       1.6864259144271627e-05, 
       1.5689642570227779e-05, 
       1.9050947387774732e-05, 
       1.7611896965069240e-05, 
       1.7745247024068698e-05, 
       1.8538580551107680e-05, 
       1.8379427706102670e-05, 
       1.5945888870380719e-05, 
       1.7305791447725026e-05, 
       2.0474582331640290e-05, 
       1.7492727383964233e-05, 
       2.1476177380312355e-05, 
       1.9106095035613618e-05, 
       1.5660259438659259e-05, 
       1.5222668335700431e-05, 
       1.8219155343956824e-05, 
       1.5642650945830570e-05, 
       1.6793143148081557e-05, 
       1.7141524658524282e-05, 
       1.8846714100393478e-05, 
       1.9326279863760890e-05, 
       2.0406891479708431e-05, 
       1.7688341537316285e-05, 
       1.5724445646149188e-05, 
       1.5540909059375424e-05, 
       1.8441641969427492e-05, 
       2.1178813931240924e-05, 
       1.5511727108972537e-05, 
       2.1342028735132333e-05, 
       1.6739066052948356e-05, 
       2.1350732414601160e-05, 
       1.7694089944982486e-05, 
       1.5829334698050776e-05, 
       1.5655025671618804e-05, 
       1.9714318587157159e-05, 
       1.5057035573659841e-05, 
       2.0192111541620821e-05, 
       1.7685943389918353e-05, 
       2.1212723019460474e-05, 
       1.9747980812227106e-05, 
       1.5527905123794615e-05, 
       1.5527905123794615e-05, 
       1.5417830060424720e-05, 
       1.7247065269713207e-05, 
       2.1119942171578811e-05, 
       1.8175254317032841e-05, 
       1.7324416311977315e-05, 
       1.9999999999999429e-05, 
       2.1137512979669648e-05, 
       2.1090103082829514e-05, 
       2.0886570032442421e-05, 
       2.1105596617233711e-05, 
       1.5743035286946835e-05, 
       2.1083679352776058e-05, 
       2.0757712453211110e-05, 
       1.5540797566770339e-05, 
       1.7531973970627391e-05, 
       1.9259698071998040e-05, 
       2.0208818797206283e-05, 
       1.5149256871057733e-05, 
       1.5467115962108733e-05, 
       1.5760440219226027e-05, 
       1.7162560484133913e-05, 
       1.7014309235359223e-05, 
       2.1019737107659137e-05, 
       1.7457605929503559e-05, 
       2.0994409641551910e-05, 
       1.9458440920805566e-05, 
       1.5603829640010367e-05, 
       1.9952679961171134e-05, 
       1.8335526455375638e-05, 
       1.6608792629156972e-05, 
       1.5626835986606559e-05, 
       2.0757576383008465e-05, 
       1.6442832343553638e-05, 
       1.5191233475722848e-05, 
       1.5043197353550041e-05, 
       2.0442887337734135e-05, 
       1.6628664127615279e-05, 
       1.8371707177007882e-05, 
       1.5913842135018836e-05, 
       1.7254322155783046e-05, 
       2.0839446184619425e-05, 
       1.5216164047457902e-05, 
       1.5029809033679622e-05, 
       1.5744511007656851e-05, 
       1.7024285329190686e-05, 
       1.4829247045808572e-05, 
       1.7309310070084839e-05, 
       2.0437293880977209e-05, 
       2.0021885185557937e-05, 
       1.8019111551194156e-05, 
       1.4831344181665965e-05, 
       2.0310977894737043e-05, 
       1.5009995172045378e-05, 
       1.7169158565946365e-05, 
       1.6165500240324501e-05, 
       1.9447983568464269e-05, 
       1.5090612779274327e-05, 
       1.4918050969900448e-05, 
       1.6476833830296272e-05, 
       2.0191724627259120e-05, 
       1.5421214338198483e-05, 
       1.7144943104701221e-05, 
       1.8098227541769401e-05, 
       1.5368499686398445e-05, 
       1.7177387307292020e-05, 
       2.0028358502386586e-05, 
       1.4858069728598857e-05, 
       1.4557718472553275e-05, 
       2.0391569074065477e-05, 
       1.8121570412691590e-05, 
       1.7132585552788231e-05, 
       1.5566873195342529e-05, 
       1.4556633968720358e-05, 
       1.5665010478850375e-05, 
       1.7601692675380615e-05, 
       2.0369753881770931e-05, 
       1.5001512518226317e-05, 
       1.9764482926694215e-05, 
       1.6400977286120477e-05, 
       1.5664816591402002e-05, 
       1.8243093368615829e-05, 
       1.6613644707846301e-05, 
       2.0299623746178761e-05, 
       2.0448445057820992e-05, 
       2.0263783081914792e-05, 
       2.0126702648095053e-05, 
       1.7004791999853136e-05, 
       1.6858620981037930e-05, 
       1.7768595734014653e-05, 
       1.9703524533953504e-05, 
       1.6920527940262901e-05, 
       1.5476650158324031e-05, 
       1.6299601104799197e-05, 
       1.6579802259591897e-05, 
       1.5042009613690328e-05, 
       1.5323701383613402e-05, 
       1.9676797461439080e-05, 
       1.6966041167571207e-05, 
       1.4719380667612877e-05, 
       1.7172069384776256e-05, 
       1.8398699817429276e-05, 
       1.4386265143752439e-05, 
       2.0261595272386628e-05, 
       1.4550935010314665e-05, 
       2.0266470679314170e-05, 
       1.5717671821020621e-05, 
       1.6852600658635932e-05, 
       1.6558022622144565e-05, 
       1.8019584552226510e-05, 
       1.6832558809015334e-05, 
       1.6861710188960205e-05, 
       1.8893615505926731e-05, 
       1.9557683747342080e-05, 
       1.4541700275218320e-05, 
       2.0238268450987529e-05, 
       1.6661293302781685e-05, 
       1.8526044807807572e-05, 
       1.4564188078851668e-05, 
       1.4677653873009543e-05, 
       1.7056497365353723e-05, 
       1.9539926389561244e-05, 
       1.7085173203672934e-05, 
       1.6209120421822010e-05, 
       1.5300826859808195e-05, 
       1.4567346867009612e-05, 
       1.6697222707461084e-05, 
       2.0086481563064009e-05, 
       1.4972744923786265e-05, 
       1.9957044818525361e-05, 
       1.6005810102042682e-05, 
       1.9283733192707659e-05, 
       1.4966509948399841e-05, 
       1.4415727513073371e-05, 
       1.4808249418141361e-05, 
       1.6715996725192163e-05, 
       1.6666076245336490e-05, 
       2.0003825464763571e-05, 
       1.7719507721485027e-05, 
       1.9369663830905708e-05, 
       1.7613112162347754e-05, 
       1.4759238116369718e-05, 
       1.4759238116369718e-05, 
       2.0011800829874513e-05, 
       1.4164793410740855e-05, 
       1.5946658441031332e-05, 
       1.9999999999996695e-05, 
       2.0000000000003224e-05, 
       1.9999999999999121e-05, 
       2.0000000000004206e-05, 
       1.9999999999999659e-05, 
       2.0000000000002682e-05, 
       1.9999999999999239e-05, 
       1.9999999999996173e-05, 
       1.9999999999996146e-05, 
       1.9999999999997356e-05, 
       2.0000000000001601e-05, 
       2.0000000000001601e-05, 
       2.0000000000001601e-05, 
       2.0000000000002875e-05, 
       1.9999999999998955e-05, 
       1.9999999999998955e-05, 
       2.0000000000002953e-05, 
       1.9999999999998511e-05, 
       1.9999999999999551e-05, 
       2.0000000000001116e-05, 
       1.9999999999996078e-05, 
       1.9999999999999470e-05, 
       2.0000000000003474e-05, 
       2.0000000000003474e-05, 
       2.0000000000003474e-05, 
       1.9999999999998087e-05, 
       1.9999999999998087e-05, 
       2.0000000000003305e-05, 
       1.9999999999999138e-05, 
       1.9999999999999138e-05, 
       1.9999999999995577e-05, 
       2.0000000000002624e-05, 
       2.0000000000000283e-05, 
       2.0000000000000283e-05, 
       1.9999999999997427e-05, 
       1.9999999999996915e-05, 
       1.9999999999997010e-05, 
       1.9999999999997715e-05, 
       2.0000000000000947e-05, 
       2.0000000000003481e-05, 
       2.0000000000001838e-05, 
       2.0000000000000002e-05, 
       1.4252697008525248e-05, 
       1.4706617789152532e-05, 
       1.4222562777881572e-05, 
       1.6361346949309436e-05, 
       1.4879257822647029e-05, 
       1.6062458346031203e-05, 
       1.6975532626770155e-05, 
       1.6384722202982934e-05, 
       1.6431911229925512e-05, 
       1.4628386208393741e-05, 
       1.6299854659894770e-05, 
       1.8359843299780902e-05, 
       1.6064531702894845e-05, 
       1.7101967390113963e-05, 
       1.9327278959677277e-05, 
       1.9893912026707649e-05, 
       1.9435801441053278e-05, 
       1.9295442619442662e-05, 
       1.4198402879160075e-05, 
       1.5601986750816583e-05, 
       1.5601986750816583e-05, 
       1.5442611079544925e-05, 
       1.8273101120279640e-05, 
       1.5212715709730862e-05, 
       1.9790572520707653e-05, 
       1.4046644829345020e-05, 
       1.6453534288840922e-05, 
       1.9290476812844101e-05, 
       1.4378875451023086e-05, 
       1.5094122994640211e-05, 
       1.5094122994640211e-05, 
       1.4142842633091691e-05, 
       1.6470574208214187e-05, 
       1.8950886223863053e-05, 
       1.5757625747503491e-05, 
       1.9712309404743041e-05, 
       1.8205817440024950e-05, 
       1.8205817440024950e-05, 
       1.4528308264275255e-05, 
       1.9573364967590550e-05, 
       1.4478764179994934e-05, 
       1.6375644730731441e-05, 
       1.7263992904413860e-05, 
       1.3974287163374690e-05, 
       1.7069877753966555e-05, 
       1.3979984726077016e-05, 
       1.6294138445065738e-05, 
       1.4988600322324814e-05, 
       1.8087520439615098e-05, 
       1.4428605082598419e-05, 
       1.5397726069238819e-05, 
       1.5395294214570601e-05, 
       1.5112464888943527e-05, 
       1.4514880645921092e-05, 
       1.9631504807082202e-05, 
       1.4655089846006943e-05, 
       1.3932174364934544e-05, 
       1.6680758473363207e-05, 
       1.8482397616888962e-05, 
       1.9375638950615900e-05, 
       1.6274656624678140e-05, 
       1.8377590927511385e-05, 
       1.6270479688158208e-05, 
       1.9508249566450005e-05, 
       1.9157204813098891e-05, 
       1.6335965397168821e-05, 
       1.6881997905733259e-05, 
       1.6564319840218979e-05, 
       1.8872918146885511e-05, 
       1.8074549305929563e-05, 
       1.9412951588592442e-05, 
       1.6263307324113935e-05, 
       1.4147500694586512e-05, 
       1.6757367952507248e-05, 
       1.9322351527415885e-05, 
       1.4573751251572777e-05, 
       1.4154859016617978e-05, 
       1.4340334740790183e-05, 
       1.4340334740790183e-05, 
       1.4071960371740698e-05, 
       1.6121666465218100e-05, 
       1.8437279532170882e-05, 
       1.4256593632430758e-05, 
       1.4556420819794034e-05, 
       1.4142244855213609e-05, 
       1.4417170150842718e-05, 
       1.3988376919673882e-05, 
       1.3783936244070711e-05, 
       1.5545702181325805e-05, 
       1.4487086535407894e-05, 
       1.9064889146728757e-05, 
       1.9006325421144988e-05, 
       1.9056748871438939e-05, 
       1.8773399897788679e-05, 
       1.4335056900673789e-05, 
       1.4335056900673789e-05, 
       1.8522465125382274e-05, 
       1.6386653896313993e-05, 
       1.5596762415637884e-05, 
       1.6005668289636209e-05, 
       1.3766017742529295e-05, 
       1.3848653128938670e-05, 
       1.5576755427267637e-05, 
       1.9100408442469076e-05, 
       1.3593457337984071e-05, 
       1.3874206683477119e-05, 
       1.4465040574081605e-05, 
       1.4652034746258144e-05, 
       1.8956486987217587e-05, 
       1.4614510097070012e-05, 
       1.4614510097070012e-05, 
       1.8008430901014635e-05, 
       1.8541903458107715e-05, 
       1.5650865089307326e-05, 
       1.8035119275958362e-05, 
       1.3761799690021450e-05, 
       1.3912423234045471e-05, 
       1.5804152496313920e-05, 
       1.4498211958542439e-05, 
       1.4220681470575931e-05, 
       1.8527474953553169e-05, 
       1.6610320576735356e-05, 
       1.4236935008604960e-05, 
       1.3561594886345028e-05, 
       1.3703215974651375e-05, 
       1.5192322155459611e-05, 
       1.4690494971901992e-05, 
       1.7429942889447511e-05, 
       1.3803523486348431e-05, 
       1.6575179186165403e-05, 
       1.8483263583129016e-05, 
       1.4317789249962279e-05, 
       1.7732674788684577e-05, 
       1.5651627359566928e-05, 
       1.7290318557184173e-05, 
       1.8996710211881576e-05, 
       1.5790212465426335e-05, 
       1.8966699858754369e-05, 
       1.6029256120213828e-05, 
       1.8679539859490891e-05, 
       1.8679539859490891e-05, 
       1.3830009977865649e-05, 
       1.6463233245324904e-05, 
       1.5799996450446853e-05, 
       1.7882317958138889e-05, 
       1.3585888483559118e-05, 
       1.3894097840287981e-05, 
       1.3787686670116759e-05, 
       1.3644422352472035e-05, 
       1.8774035932232420e-05, 
       1.7563953822318123e-05, 
       1.5856345491746732e-05, 
       1.6139604297705414e-05, 
       1.4074765159929460e-05, 
       1.4484758356553500e-05, 
       1.3670166002683302e-05, 
       1.8775891554910203e-05, 
       1.6802587081324032e-05, 
       1.5910421546831657e-05, 
       1.3698167598078695e-05, 
       1.3653131818738679e-05, 
       1.4888735311831039e-05, 
       1.7799888897948100e-05, 
       1.3624957205328564e-05, 
       1.6329939260001416e-05, 
       1.8762516187751245e-05, 
       1.7014405338271208e-05, 
       1.5581039856646170e-05, 
       1.8697234576615613e-05, 
       1.3972208543942194e-05, 
       1.3972208543942194e-05, 
       1.4442937029131723e-05, 
       1.4399782406159191e-05, 
       1.3736672508724698e-05, 
       1.4140165512352361e-05, 
       1.5231770743292131e-05, 
       1.4853962474344839e-05, 
       1.7618334074186727e-05, 
       1.5433677464118755e-05, 
       1.3305211232350297e-05, 
       1.4964053258880190e-05, 
       1.3921310448009076e-05, 
       1.5374689745796454e-05, 
       1.3641410384706774e-05, 
       1.8055064613690848e-05, 
       1.3360955606502185e-05, 
       1.3856726747044094e-05, 
       1.3627425378109574e-05, 
       1.4890056335115028e-05, 
       1.8526520276109926e-05, 
       1.5801608056782483e-05, 
       1.4876131597626108e-05, 
       1.4138365761711702e-05, 
       1.4825313646580885e-05, 
       1.3602238135309408e-05, 
       1.3796179854504099e-05, 
       1.7297656639459478e-05, 
       1.3147675452199926e-05, 
       1.8204562308233258e-05, 
       1.7815850850000629e-05, 
       1.5278119496228523e-05, 
       1.3240215118733179e-05, 
       1.5699687872250967e-05, 
       1.3374323164538733e-05, 
       1.7112296806910231e-05, 
       1.6498260600057044e-05, 
       1.8277692739113280e-05, 
       1.4079393768881114e-05, 
       1.3926600400371500e-05, 
       1.5273539828843144e-05, 
       1.5265176841358936e-05, 
       1.3322314722811133e-05, 
       1.8232455127632119e-05, 
       1.5244495762962463e-05, 
       1.3032933685357870e-05, 
       1.2974384091595749e-05, 
       1.5379789430354704e-05, 
       1.5379789430354704e-05, 
       1.8331736566769736e-05, 
       1.3037120214926734e-05, 
       1.5759621600579822e-05, 
       1.7574461976645667e-05, 
       1.8089452974609749e-05, 
       1.3606642222909142e-05, 
       1.5163483142953135e-05, 
       1.4994433687067182e-05, 
       1.5200368248783655e-05, 
       1.3310792113503426e-05, 
       1.4671350937450219e-05, 
       1.3684550993637766e-05, 
       1.5494308882524729e-05, 
       1.7638645171835623e-05, 
       1.2938155469159480e-05, 
       1.3744066984965754e-05, 
       1.7816407086215970e-05, 
       1.8150773800612311e-05, 
       1.4726544941961299e-05, 
       1.7986737031725168e-05, 
       1.8052485607192366e-05, 
       1.4812099570027929e-05, 
       1.4518517656199323e-05, 
       1.3990896235310456e-05, 
       1.5543947283905356e-05, 
       1.7577282839669293e-05, 
       1.3507379863828441e-05, 
       1.7488924566693299e-05, 
       1.5071685391860221e-05, 
       1.2791921429880872e-05, 
       1.8023910057525558e-05, 
       1.3605811071184698e-05, 
       1.8022134748298126e-05, 
       1.4394770804460141e-05, 
       1.3728539436706959e-05, 
       1.3733553493619727e-05, 
       1.5396018344123773e-05, 
       1.6117579412375021e-05, 
       1.2763076368191250e-05, 
       1.3451049438603334e-05, 
       1.3485220199497009e-05, 
       1.5904892647605164e-05, 
       1.7864247572255879e-05, 
       1.4667043332272065e-05, 
       1.7971366349273862e-05, 
       1.4661552630549338e-05, 
       1.7908044971456631e-05, 
       1.4928801764298160e-05, 
       1.4910251689453436e-05, 
       1.7204350483958699e-05, 
       1.7657159089503033e-05, 
       1.5074256166742058e-05, 
       1.3282048646588323e-05, 
       1.7271865789996836e-05, 
       1.5912057737424142e-05, 
       1.7054593378276863e-05, 
       1.5969037353564106e-05, 
       1.5524953603403964e-05, 
       1.7650763770924837e-05, 
       1.3944850141770251e-05, 
       1.5190932435174982e-05, 
       1.3789607788259273e-05, 
       1.2701786898881363e-05, 
       1.7076570796036098e-05, 
       1.7616211913199017e-05, 
       1.4071295575194016e-05, 
       1.3725672479509337e-05, 
       1.5320600940140889e-05, 
       1.4290138759118962e-05, 
       1.3647439612993676e-05, 
       1.6509359377876491e-05, 
       1.4520756131834291e-05, 
       1.3030639943745086e-05, 
       1.5088065308295757e-05, 
       1.2850530350206869e-05, 
       1.7649665142622546e-05, 
       1.3153542124574486e-05, 
       1.7711449414143951e-05, 
       1.3601666905827086e-05, 
       1.3435283052920611e-05, 
       1.7156003099018238e-05, 
       1.4975499849479458e-05, 
       1.7637939469122597e-05, 
       1.7558379694124883e-05, 
       1.4668551910176133e-05, 
       1.7084070941358651e-05, 
       1.3682645263022775e-05, 
       1.5981906534706543e-05, 
       1.7637817035649532e-05, 
       1.2890273588343544e-05, 
       1.5245509927831403e-05, 
       1.6758967052960054e-05, 
       1.7233754233301464e-05, 
       1.7520367723575716e-05, 
       1.3192309639059891e-05, 
       1.4419266314513079e-05, 
       1.4638127644720810e-05, 
       1.3909898954175807e-05, 
       1.4695654784553427e-05, 
       1.3048290781505763e-05, 
       1.3048290781505763e-05, 
       1.4557878607341690e-05, 
       1.4660430329794040e-05, 
       1.4597096659250177e-05, 
       1.3423358781637431e-05, 
       1.6976026283234938e-05, 
       1.6294144632581943e-05, 
       1.3845272419402378e-05, 
       1.4201681647182699e-05, 
       1.2706578085768076e-05, 
       1.4496581518191856e-05, 
       1.3303962337713564e-05, 
       1.3052231908621290e-05, 
       1.3379125996042708e-05, 
       1.2836175305829798e-05, 
       1.4470153842595127e-05, 
       1.4476736529632748e-05, 
       1.2938046521844049e-05, 
       1.2938046521844049e-05, 
       1.3627442406962703e-05, 
       1.3929888230266741e-05, 
       1.4754452407133853e-05, 
       1.3645484846918677e-05, 
       1.3483611504244313e-05, 
       1.5459574601845453e-05, 
       1.3333275993307890e-05, 
       1.7288009190477404e-05, 
       1.2477230783178808e-05, 
       1.6125270270647051e-05, 
       1.6098750340081555e-05, 
       1.2372326077540164e-05, 
       1.6251502136286907e-05, 
       1.2914876775410167e-05, 
       1.4886017190838218e-05, 
       1.7266381037946462e-05, 
       1.5056480330779489e-05, 
       1.3909423114964528e-05, 
       1.2555830630170498e-05, 
       1.2245937489779250e-05, 
       1.3861970848648388e-05, 
       1.4343507935153652e-05, 
       1.7242847974451225e-05, 
       1.3103856485897079e-05, 
       1.3103856485897079e-05, 
       1.2397289904446542e-05, 
       1.3446720656278218e-05, 
       1.4311110555410218e-05, 
       1.5717980401117605e-05, 
       1.4863154551277376e-05, 
       1.4328863233185172e-05, 
       1.6998547074026785e-05, 
       1.7175291575323660e-05, 
       1.6471152625733929e-05, 
       1.3114513918939321e-05, 
       1.2860330495408732e-05, 
       1.4683817738810251e-05, 
       1.2167919130941174e-05, 
       1.7069186952466700e-05, 
       1.2837893826301510e-05, 
       1.2837291811602049e-05, 
       1.3258128466507157e-05, 
       1.7166171907753879e-05, 
       1.3541079299135263e-05, 
       1.3890893255316942e-05, 
       1.2253429105000318e-05, 
       1.6814379251639918e-05, 
       1.6686371790053983e-05, 
       1.2308744618674266e-05, 
       1.4636327977612473e-05, 
       1.3066470074275391e-05, 
       1.4231459697256301e-05, 
       1.2598263937027711e-05, 
       1.2598263937027711e-05, 
       1.6029260899523525e-05, 
       1.2717143887558507e-05, 
       1.2526394277540726e-05, 
       1.3337117264024642e-05, 
       1.3337117264024642e-05, 
       1.2066906506521277e-05, 
       1.4622861848075122e-05, 
       1.3245897771755157e-05, 
       1.4113747366887348e-05, 
       1.5776030685851246e-05, 
       1.6840589361518967e-05, 
       1.6757062602835289e-05, 
       1.2312482768388078e-05, 
       1.2945937951306349e-05, 
       1.2312734900739979e-05, 
       1.4364209292227234e-05, 
       1.3390556900591869e-05, 
       1.6850786734237513e-05, 
       1.2417472982824590e-05, 
       1.2050728736363786e-05, 
       1.2121054877793193e-05, 
       1.2021842078897896e-05, 
       1.6914859326904161e-05, 
       1.6914859326904161e-05, 
       1.6049674572441781e-05, 
       1.5596869909457542e-05, 
       1.5786976168410679e-05, 
       1.4058808896797033e-05, 
       1.4303475321960484e-05, 
       1.2406664726228691e-05, 
       1.2587942888697610e-05, 
       1.6731244239970316e-05, 
       1.4144804399699325e-05, 
       1.6666138483921900e-05, 
       1.4550935010312244e-05, 
       1.4461870876485519e-05, 
       1.3994226033164611e-05, 
       1.5195555397875725e-05, 
       1.6729713132836927e-05, 
       1.4091489602312114e-05, 
       1.2019136694960872e-05, 
       1.1900254687958665e-05, 
       1.2427743607487307e-05, 
       1.4146131462608746e-05, 
       1.4181027470503725e-05, 
       1.3594604895580452e-05, 
       1.2177891934311316e-05, 
       1.2025784821732307e-05, 
       1.1917756983521074e-05, 
       1.6254652074131396e-05, 
       1.1895702147260868e-05, 
       1.6096719431448920e-05, 
       1.5917873693892506e-05, 
       1.5917873693892506e-05, 
       1.6122999578866428e-05, 
       1.6550723620444485e-05, 
       1.3053404989046549e-05, 
       1.1899219598038330e-05, 
       1.6556372876046441e-05, 
       1.3959526236808225e-05, 
       1.2144994647592413e-05, 
       1.3386073177119976e-05, 
       1.5134449596170500e-05, 
       1.6696225586389643e-05, 
       1.2031171987761573e-05, 
       1.6293459771790049e-05, 
       1.6293459771790049e-05, 
       1.6547803747264835e-05, 
       1.6697400297342063e-05, 
       1.6216049437156085e-05, 
       1.3933037853893806e-05, 
       1.2257163925048118e-05, 
       1.4010491493895888e-05, 
       1.6252977248923219e-05, 
       1.2101801105589593e-05, 
       1.1892920941077241e-05, 
       1.3235715098705824e-05, 
       1.6648172967064449e-05, 
       1.4892291719144730e-05, 
       1.2604359039019861e-05, 
       1.5421558404408979e-05, 
       1.6590948772789458e-05, 
       1.3220818327265675e-05, 
       1.4563479832099421e-05, 
       1.3748895926703433e-05, 
       1.2606073278651340e-05, 
       1.3777882492901675e-05, 
       1.1869521198062512e-05, 
       1.2416428439146307e-05, 
       1.1768856122296658e-05, 
       1.1858819054917719e-05, 
       1.2397461607421407e-05, 
       1.1893677184827856e-05, 
       1.5888709529634991e-05, 
       1.1690375557812716e-05, 
       1.6517676686398936e-05, 
       1.1700429175629419e-05, 
       1.3674402824641351e-05, 
       1.3926320221697239e-05, 
       1.2176653171116317e-05, 
       1.3687430993784603e-05, 
       1.3741803269687414e-05, 
       1.5922462564260811e-05, 
       1.2801374700097838e-05, 
       1.3130363667340645e-05, 
       1.2922579292438108e-05, 
       1.4105335722376046e-05, 
       1.4376105332733210e-05, 
       1.6407326862993805e-05, 
       1.5873453532599455e-05, 
       1.3703222617552004e-05, 
       1.2117885951517277e-05, 
       1.5614580438514598e-05, 
       1.1975720602925757e-05, 
       1.2708501264251740e-05, 
       1.2357430174878866e-05, 
       1.6399966829113250e-05, 
       1.3183846274234094e-05, 
       1.5338415995535773e-05, 
       1.4563188418429497e-05, 
       1.3605880830959589e-05, 
       1.4328212241143718e-05, 
       1.6363733125681598e-05, 
       1.4164855642925120e-05, 
       1.4164855642925120e-05, 
       1.2323655363808756e-05, 
       1.2210970936641173e-05, 
       1.3640407348496314e-05, 
       1.1814901904537399e-05, 
       1.3139510997259285e-05, 
       1.6322576093387930e-05, 
       1.3980065987600939e-05, 
       1.3475308442031163e-05, 
       1.5921832000744340e-05, 
       1.1826485091935282e-05, 
       1.2120901952412654e-05, 
       1.2114943572371832e-05, 
       1.3597009682566435e-05, 
       1.4581189231655004e-05, 
       1.3501358373618001e-05, 
       1.6065981678586259e-05, 
       1.2276480460318524e-05, 
       1.2003836137298747e-05, 
       1.1525322578344814e-05, 
       1.3517460118223849e-05, 
       1.3970690321599620e-05, 
       1.6161601619169139e-05, 
       1.1931658769035319e-05, 
       1.3964616244322158e-05, 
       1.3533717127469221e-05, 
       1.1904444298536165e-05, 
       1.2577743538942000e-05, 
       1.1508460020682763e-05, 
       1.1730974848875819e-05, 
       1.2477745528219772e-05, 
       1.6049351736991511e-05, 
       1.1464313500445488e-05, 
       1.3475378673897131e-05, 
       1.5927747760108988e-05, 
       1.3457534238306322e-05, 
       1.1935326835509630e-05, 
       1.2220331175923250e-05, 
       1.4317074492694330e-05, 
       1.2205539674595510e-05, 
       1.3072336703719005e-05, 
       1.2655866562070534e-05, 
       1.2655866562070534e-05, 
       1.3065085265743608e-05, 
       1.6112900946421414e-05, 
       1.1416702843928877e-05, 
       1.5672390858687492e-05, 
       1.2116629510574814e-05, 
       1.2032232871584167e-05, 
       1.5320346282703337e-05, 
       1.4481654901481538e-05, 
       1.3885559510227951e-05, 
       1.2044972708168932e-05, 
       1.5002928543636147e-05, 
       1.5082211806793467e-05, 
       1.2756413853865409e-05, 
       1.4497843397617043e-05, 
       1.3392113626936127e-05, 
       1.1389365726732977e-05, 
       1.1985420076909093e-05, 
       1.5196842325061426e-05, 
       1.1933621019020256e-05, 
       1.1807638449827242e-05, 
       1.3308936305146922e-05, 
       1.3298599083369901e-05, 
       1.1378444149920218e-05, 
       1.6048462299887861e-05, 
       1.5991135950797735e-05, 
       1.3285633812314531e-05, 
       1.1861522576797246e-05, 
       1.4224421894410120e-05, 
       1.1986113227578546e-05, 
       1.4216639432982047e-05, 
       1.3470047970805898e-05, 
       1.3470047970805898e-05, 
       1.4741088562136247e-05, 
       1.3059321000458832e-05, 
       1.3269348820725535e-05, 
       1.4479061291151282e-05, 
       1.5511542301208200e-05, 
       1.1813723822093969e-05, 
       1.5280970012462507e-05, 
       1.3054280075616977e-05, 
       1.5762080187927474e-05, 
       1.2918826683218768e-05, 
       1.2092610841784454e-05, 
       1.2354112418920712e-05, 
       1.3225241829851445e-05, 
       1.5421509424894833e-05, 
       1.5773410814205763e-05, 
       1.1642924087402093e-05, 
       1.3248940560185062e-05, 
       1.3284484931482563e-05, 
       1.3064209267144802e-05, 
       1.1879999601992830e-05, 
       1.1693198035836968e-05, 
       1.3094240945144193e-05, 
       1.5849108661994252e-05, 
       1.5802276527407762e-05, 
       1.4528569276352678e-05, 
       1.3887200926675147e-05, 
       1.5628238006873463e-05, 
       1.1391895340035019e-05, 
       1.4124703796517866e-05, 
       1.1807043137415146e-05, 
       1.1856920864233906e-05, 
       1.3257437021867844e-05, 
       1.5764864603397483e-05, 
       1.3112152733703945e-05, 
       1.3433709764932103e-05, 
       1.3493138783878672e-05, 
       1.5640243705340389e-05, 
       1.4897516853048267e-05, 
       1.1824671081427098e-05, 
       1.1918979748613804e-05, 
       1.5306807208564794e-05, 
       1.5329215830388740e-05, 
       1.4575506214112725e-05, 
       1.3938339965521361e-05, 
       1.3827512359644423e-05, 
       1.3826506665593992e-05, 
       1.1154287022865394e-05, 
       1.3104211740154260e-05, 
       1.3057059875474051e-05, 
       1.4557953177370790e-05, 
       1.1159088919781853e-05, 
       1.1801813846695364e-05, 
       1.3861689708102152e-05, 
       1.5637547646829787e-05, 
       1.1356472191127747e-05, 
       1.2317807113902109e-05, 
       1.4895818288670863e-05, 
       1.3850120802112967e-05, 
       1.3492607685528236e-05, 
       1.4981062960212821e-05, 
       1.3252357555360155e-05, 
       1.1104728466622570e-05, 
       1.4483242358871658e-05, 
       1.1578097276879643e-05, 
       1.5420703115194019e-05, 
       1.5033360827189572e-05, 
       1.5186932725687286e-05, 
       1.2013952315036721e-05, 
       1.4990221537768369e-05, 
       1.5556573953943968e-05, 
       1.5563407443876743e-05, 
       1.2971239202206968e-05, 
       1.1890022150710963e-05, 
       1.5537853150611444e-05, 
       1.5539998142774340e-05, 
       1.3856883481054222e-05, 
       1.2530939020271129e-05, 
       1.1637049120821975e-05, 
       1.2945648311349691e-05, 
       1.1031681376658956e-05, 
       1.3049309327874108e-05, 
       1.1940715012502777e-05, 
       1.1897661138472753e-05, 
       1.1335968607249550e-05, 
       1.3555327357810430e-05, 
       1.1779470985341316e-05, 
       1.3187383370416893e-05, 
       1.4539576556180759e-05, 
       1.2813827019497473e-05, 
       1.1444304102659144e-05, 
       1.4969920882945149e-05, 
       1.5517416798333600e-05, 
       1.1410798355908373e-05, 
       1.1526908858379218e-05, 
       1.1412154447914233e-05, 
       1.1060721686510390e-05, 
       1.1994395300910673e-05, 
       1.1455233575224207e-05, 
       1.4773815207526001e-05, 
       1.3861004405617579e-05, 
       1.3640295916394148e-05, 
       1.2852315404897019e-05, 
       1.1398772300010273e-05, 
       1.2798525948146793e-05, 
       1.1653871052141508e-05, 
       1.3632118683130851e-05, 
       1.1148406474139346e-05, 
       1.3733998257353609e-05, 
       1.3305005818390715e-05, 
       1.0998248694589623e-05, 
       1.0998248694589623e-05, 
       1.1637082991179322e-05, 
       1.5368410306380071e-05, 
       1.3192941165492487e-05, 
       1.1188647725158896e-05, 
       1.0935645280156292e-05, 
       1.4542949985434590e-05, 
       1.3179870361789488e-05, 
       1.4960353796963889e-05, 
       1.4915837093744389e-05, 
       1.2088326964332276e-05, 
       1.4563764170574967e-05, 
       1.0863742504531845e-05, 
       1.5313167867916095e-05, 
       1.1192641060427904e-05, 
       1.1948775482607372e-05, 
       1.4452860451827453e-05, 
       1.2741662513848429e-05, 
       1.1317217686696363e-05, 
       1.1793717616235744e-05, 
       1.4855051404153621e-05, 
       1.4240635417252080e-05, 
       1.3825001173800063e-05, 
       1.1332938053101302e-05, 
       1.2580925969486195e-05, 
       1.3773935002405795e-05, 
       1.5258911695267703e-05, 
       1.1840574609522966e-05, 
       1.1139851175239220e-05, 
       1.3393528685845125e-05, 
       1.1249787407348119e-05, 
       1.0860150975170697e-05, 
       1.0885184065242962e-05, 
       1.2670890151171056e-05, 
       1.1170914243864697e-05, 
       1.0801434278990760e-05, 
       1.2765339764892054e-05, 
       1.0803464142490780e-05, 
       1.2366155267475223e-05, 
       1.5161410642571402e-05, 
       1.1219118698651237e-05, 
       1.2166386870818340e-05, 
       1.1544391300269693e-05, 
       1.1685487812199201e-05, 
       1.3685283576603951e-05, 
       1.3273456653083252e-05, 
       1.1664554800437190e-05, 
       1.1291937457610047e-05, 
       1.2554115313255417e-05, 
       1.0921965061761196e-05, 
       1.1607722747098729e-05, 
       1.0963988595682829e-05, 
       1.3731446740896991e-05, 
       1.2527592038560479e-05, 
       1.4171055758210119e-05, 
       1.5111095674629706e-05, 
       1.2538575122121326e-05, 
       1.2037640370020864e-05, 
       1.0728270322101026e-05, 
       1.2678203484924697e-05, 
       1.1274157493675601e-05, 
       1.4871411465465572e-05, 
       1.2522508413807697e-05, 
       1.0659127960713351e-05, 
       1.5055603144628777e-05, 
       1.2518855054635743e-05, 
       1.0651886373215528e-05, 
       1.0651886373215528e-05, 
       1.4886407385742349e-05, 
       1.3918019086430911e-05, 
       1.4347075010925284e-05, 
       1.4347075010925284e-05, 
       1.4442270658604354e-05, 
       1.2952642168319032e-05, 
       1.2952642168319032e-05, 
       1.1660242166527791e-05, 
       1.5016282501593631e-05, 
       1.0831100220351874e-05, 
       1.1197738453633726e-05, 
       1.2570870031835301e-05, 
       1.1465574695326973e-05, 
       1.0665439726870978e-05, 
       1.3759046524795987e-05, 
       1.4412923394003059e-05, 
       1.0571717301273498e-05, 
       1.1154961514419220e-05, 
       1.0890198866829411e-05, 
       1.4778160118356734e-05, 
       1.2621970953511089e-05, 
       1.2832151227168138e-05, 
       1.2626677746179825e-05, 
       1.4837709127623125e-05, 
       1.0910721916515993e-05, 
       1.3742445935768050e-05, 
       1.3742445935768050e-05, 
       1.4683404986516526e-05, 
       1.2344215961474470e-05, 
       1.3127125977790372e-05, 
       1.0676552533051296e-05, 
       1.4161551654548738e-05, 
       1.2306883668434777e-05, 
       1.2303001215473267e-05, 
       1.1164975310819916e-05, 
       1.2194597169369334e-05, 
       1.2115055312501551e-05, 
       1.2170763628642250e-05, 
       1.4484924963431202e-05, 
       1.1252811890346518e-05, 
       1.0495937740821709e-05, 
       1.4140604224437157e-05, 
       1.4681525362788206e-05, 
       1.0512675903067279e-05, 
       1.4243496757924631e-05, 
       1.0595719419054424e-05, 
       1.0449406925214181e-05, 
       1.1456951762915769e-05, 
       1.4724446820686352e-05, 
       1.3434622479719439e-05, 
       1.4709892644663882e-05, 
       1.1836000707292320e-05, 
       1.4713335443331565e-05, 
       1.1082379972575165e-05, 
       1.2242769296151881e-05, 
       1.1602853573823651e-05, 
       1.0716602306352488e-05, 
       1.0534752474499265e-05, 
       1.1344205164660185e-05, 
       1.0401305166555006e-05, 
       1.1014401181863681e-05, 
       1.4659949622414877e-05, 
       1.0384773996917772e-05, 
       1.4036959989357961e-05, 
       1.2524136353664507e-05, 
       1.0936214361452002e-05, 
       1.2153357203736828e-05, 
       1.4066699953688203e-05, 
       1.4141498238269605e-05, 
       1.2392378097987866e-05, 
       1.4331445284315292e-05, 
       1.4586442636674197e-05, 
       1.1830139090588987e-05, 
       1.0958495499256982e-05, 
       1.2428462634778150e-05, 
       1.0921579700293800e-05, 
       1.1666150705640568e-05, 
       1.4010764386899351e-05, 
       1.1956788430085225e-05, 
       1.0344890790468721e-05, 
       1.3971773772161322e-05, 
       1.0549690635922559e-05, 
       1.0318738385705355e-05, 
       1.0898921519712717e-05, 
       1.0543194708015209e-05, 
       1.4145343480733565e-05, 
       1.2060006773585947e-05, 
       1.3792486401472005e-05, 
       1.3869927962826378e-05, 
       1.2416426872968311e-05, 
       1.0338576661525758e-05, 
       1.0559825004865625e-05, 
       1.0484910908408635e-05, 
       1.3039043275302748e-05, 
       1.1697142927916625e-05, 
       1.4450694127899086e-05, 
       1.1643815849938036e-05, 
       1.2007908226959582e-05, 
       1.4425539353924863e-05, 
       1.0506640473049503e-05, 
       1.0474942937423516e-05, 
       1.3889033386983225e-05, 
       1.1880747508122686e-05, 
       1.2489604700702411e-05, 
       1.2489604700702411e-05, 
       1.0392611058292722e-05, 
       1.2772052783093288e-05, 
       1.2772052783093288e-05, 
       1.1808845978552323e-05, 
       1.1808845978552323e-05, 
       1.1945022081057719e-05, 
       1.2512305861297867e-05, 
       1.0415226473659687e-05, 
       1.0737765500422271e-05, 
       1.4402345954977824e-05, 
       1.0887444579532782e-05, 
       1.1703899220320226e-05, 
       1.1959594093656916e-05, 
       1.0773236559004109e-05, 
       1.4360862432694292e-05, 
       1.2240427739044189e-05, 
       1.1216123372096402e-05, 
       1.1788566356216907e-05, 
       1.3948310641137338e-05, 
       1.3815750446755045e-05, 
       1.2920933560247753e-05, 
       1.4134879628969013e-05, 
       1.1160623781527150e-05, 
       1.1160623781527150e-05, 
       1.2233412682813135e-05, 
       1.0317150656727319e-05, 
       1.0317150656727319e-05, 
       1.4289570782305795e-05, 
       1.0196487438925396e-05, 
       1.2790905708513912e-05, 
       1.2060137848635450e-05, 
       1.3550361314499474e-05, 
       1.1836798241633949e-05, 
       1.0346391351992383e-05, 
       1.2046436088159806e-05, 
       1.4100372472216545e-05, 
       1.3359771035025645e-05, 
       1.0997381950406280e-05, 
       1.2069338066894263e-05, 
       1.3976482602792469e-05, 
       1.4284330144195686e-05, 
       1.2890366120256928e-05, 
       1.4033048240817880e-05, 
       1.3534855672027939e-05, 
       1.0451472677867640e-05, 
       1.3268856864578705e-05, 
       1.1649828295476208e-05, 
       1.2061549760662306e-05, 
       1.2038304402786417e-05, 
       1.2038304402786417e-05, 
       1.1571451762549295e-05, 
       1.0118941153482331e-05, 
       1.0618798067062287e-05, 
       1.4238500739643426e-05, 
       1.0319876860561089e-05, 
       1.1768856122295848e-05, 
       1.2940979908592388e-05, 
       1.0746593594366579e-05, 
       1.0273276574236062e-05, 
       1.2248882187734898e-05, 
       1.2902405785523740e-05, 
       1.4125490646593750e-05, 
       1.0675488628701313e-05, 
       1.1547106398575243e-05, 
       1.1904903197152804e-05, 
       1.4102460234349681e-05, 
       1.0170247993596534e-05, 
       1.0170247993596534e-05, 
       1.3915573420428839e-05, 
       1.1709741867561160e-05, 
       1.0927570149911177e-05, 
       1.1812442133948183e-05, 
       1.4098690081250948e-05, 
       1.4098690081250948e-05, 
       1.3482363114989975e-05, 
       1.3989348178459340e-05, 
       1.0362414106611471e-05, 
       1.0498922550965688e-05, 
       1.0000000000000279e-05, 
       1.0000000000000279e-05, 
       1.2645135303900096e-05, 
       1.3930243692899263e-05, 
       1.3934062965166813e-05, 
       1.1720257817511319e-05, 
       1.1409584860212466e-05, 
       1.2331497769339113e-05, 
       1.0450595613188668e-05, 
       1.1691106391661278e-05, 
       1.3144167343231384e-05, 
       1.2307362758157403e-05, 
       1.1642030449468130e-05, 
       1.3958305273500112e-05, 
       1.1811713207911753e-05, 
       1.1811713207911753e-05, 
       1.0439481800487469e-05, 
       1.0439481800487469e-05, 
       1.3984277874136380e-05, 
       1.3829935804982754e-05, 
       1.0838563709328192e-05, 
       1.0150728843913007e-05, 
       1.3602066078744027e-05, 
       1.3602066078744027e-05, 
       1.0680250448823534e-05, 
       1.3903475554934372e-05, 
       1.1370130817448192e-05, 
       1.0194294979848547e-05, 
       1.1565059803575839e-05, 
       1.1879800114594682e-05, 
       1.1335664568262363e-05, 
       1.0362255223359137e-05, 
       1.0390016845283681e-05, 
       1.1177819124039588e-05, 
       1.1311927842918881e-05, 
       1.3576590305187022e-05, 
       1.0028811440704425e-05, 
       1.1341182313497504e-05, 
       1.1938097032108912e-05, 
       1.1535052014800086e-05, 
       1.3388378207252262e-05, 
       1.1807400927882543e-05, 
       1.3733780865396033e-05, 
       1.0073761459710556e-05, 
       1.1268215155539663e-05, 
       1.0822684638795439e-05, 
       1.0116852966843347e-05, 
       1.1845855739428831e-05, 
       1.3815380458341364e-05, 
       1.0353519986385917e-05, 
       1.0067961931794342e-05, 
       1.1055772251224289e-05, 
       1.0016432349673548e-05, 
       1.3210432289812378e-05, 
       1.1446403711334912e-05, 
       1.0233904652838011e-05, 
       1.0018719607251077e-05, 
       1.2660729345573786e-05, 
       1.1834000877725536e-05, 
       1.1834000877725536e-05, 
       1.0994290883737941e-05, 
       1.0698225210999301e-05, 
       1.3685380851341690e-05, 
       1.3651032839328186e-05, 
       1.0359912752619529e-05, 
       1.0317716094179740e-05, 
       1.1784193740799748e-05, 
       1.1213299153867411e-05, 
       1.0713899196496290e-05, 
       1.3644861737491657e-05, 
       1.1115395959777903e-05, 
       1.2697872481339443e-05, 
       1.3657603597674202e-05, 
       1.3035310519502945e-05, 
       1.3212153974730906e-05, 
       1.1337604904769409e-05, 
       1.3449679888745507e-05, 
       1.0404820062256778e-05, 
       1.0404820062256778e-05, 
       1.1803952086063882e-05, 
       1.1076539212505333e-05, 
       1.2026156016208614e-05, 
       1.3200601008670166e-05, 
       1.3391327431721781e-05, 
       1.1855046325072819e-05, 
       1.0009598062220815e-05, 
       1.0163739277361600e-05, 
       9.9999999999998755e-06, 
       1.3518213507169691e-05, 
       1.2222460518575340e-05, 
       1.1572690721656101e-05, 
       1.0827124939977011e-05, 
       1.2190721148789807e-05, 
       1.3184863891068122e-05, 
       1.3184863891068122e-05, 
       1.3558044760643362e-05, 
       1.2159319615327873e-05, 
       1.3550324963493495e-05, 
       1.0127763570488329e-05, 
       1.1267956287192027e-05, 
       1.1244205646962845e-05, 
       1.1257517530190761e-05, 
       1.3347899136801808e-05, 
       1.3435666467546523e-05, 
       1.1138693608766828e-05, 
       1.0403185702723281e-05, 
       1.0411360157012967e-05, 
       1.1233359572429419e-05, 
       1.1339773152336635e-05, 
       1.0065718675181396e-05, 
       1.3328305577333125e-05, 
       1.1243525739781475e-05, 
       1.2807430383209726e-05, 
       1.1412327518150835e-05, 
       1.1240386522981550e-05, 
       1.3436414976920724e-05, 
       1.0117373966157046e-05, 
       1.1455433895513748e-05, 
       1.1455433895513748e-05, 
       1.1423648558252021e-05, 
       1.0060410898710771e-05, 
       1.1696808898257769e-05, 
       1.1696808898257769e-05, 
       1.0288919083048500e-05, 
       1.3416746058899971e-05, 
       1.1633192973200580e-05, 
       1.1175006644526828e-05, 
       1.2878143762739536e-05, 
       1.0218961177469847e-05, 
       1.0106218887709105e-05, 
       1.3391470593782954e-05, 
       1.3405400948871187e-05, 
       1.0348747319988554e-05, 
       1.1116419317269919e-05, 
       1.2855094063585698e-05, 
       1.3390423371862794e-05, 
       1.1088076901555095e-05, 
       1.1116934561021705e-05, 
       1.1955241851797231e-05, 
       1.1304320880906099e-05, 
       9.9550323628339216e-06, 
       1.2393819279685939e-05, 
       1.1193265754124183e-05, 
       1.1294229249108099e-05, 
       1.3295096665274059e-05, 
       1.1111830722654068e-05, 
       1.1076007966670047e-05, 
       1.0753402324570199e-05, 
       1.0297624798953658e-05, 
       1.1084236647218861e-05, 
       1.3256133325131781e-05, 
       1.0006270624895459e-05, 
       1.1911969898245642e-05, 
       1.1911969898245642e-05, 
       1.3170578888812826e-05, 
       1.2257422759529661e-05, 
       1.1001550308801094e-05, 
       1.2720525018207747e-05, 
       9.9999999999999415e-06, 
       1.0000000000000440e-05, 
       1.0715636096181351e-05, 
       9.9999999999997044e-06, 
       1.0000000000000428e-05, 
       1.0870910800703376e-05, 
       1.0749077446380380e-05, 
       1.3186147797601452e-05, 
       1.2719646280423556e-05, 
       1.3094705236605134e-05, 
       1.0000000000000377e-05, 
       1.2747419464837698e-05, 
       1.1232156746794980e-05, 
       1.2883378125676412e-05, 
       1.2251724674349591e-05, 
       1.3016958173967444e-05, 
       1.0222411768726092e-05, 
       1.2834305081252563e-05, 
       1.2968890800170580e-05, 
       1.2990915904436691e-05, 
       1.2083980017933565e-05, 
       1.0785472236506439e-05, 
       1.2203615024909420e-05, 
       1.2203615024909420e-05, 
       1.0981226683038296e-05, 
       1.0533411740158035e-05, 
       1.0830199438578192e-05, 
       1.1960671887137153e-05, 
       1.0785929048776913e-05, 
       1.3025335988897369e-05, 
       1.1136781492999438e-05, 
       1.1612336003839269e-05, 
       1.2988377962757502e-05, 
       1.1164984246547905e-05, 
       1.0210854727747511e-05, 
       1.2446618436692591e-05, 
       1.0078031489747915e-05, 
       1.1678049274259802e-05, 
       1.3025995798683719e-05, 
       1.0544724608204932e-05, 
       9.6794497300601840e-06, 
       1.1096749297320163e-05, 
       1.0084975690122371e-05, 
       1.1340258203439586e-05, 
       1.0219172695380650e-05, 
       1.0742345396415169e-05, 
       1.0641298291999593e-05, 
       1.2449030052083542e-05, 
       1.0587909295528927e-05, 
       1.1126537284836484e-05, 
       1.1014402227703725e-05, 
       1.1634600359898741e-05, 
       1.1952925409807471e-05, 
       1.2532304611125723e-05, 
       1.0006342479309403e-05, 
       1.2468375296210687e-05, 
       1.2399793328269677e-05, 
       1.0760154193391669e-05, 
       1.0347115243372036e-05, 
       1.1845462406893194e-05, 
       1.1845462406893194e-05, 
       1.0332953921133789e-05, 
       1.2777486500807533e-05, 
       1.0984214956873311e-05, 
       1.2392227816465758e-05, 
       1.0103732349625345e-05, 
       1.0597505518281205e-05, 
       1.2782310985670216e-05, 
       1.2798816149107451e-05, 
       9.5755252517464675e-06, 
       1.0930061735249371e-05, 
       1.1215532672171842e-05, 
       1.1215532672171842e-05, 
       1.0623627311694828e-05, 
       1.1747856010054313e-05, 
       1.2672824620875476e-05, 
       1.1675461920018449e-05, 
       1.1696275840969826e-05, 
       1.2721106820871759e-05, 
       1.0975663700983502e-05, 
       1.0417188511511460e-05, 
       1.1564473419440789e-05, 
       1.2374252902852461e-05, 
       1.2699751570011256e-05, 
       1.2300450326821395e-05, 
       1.0572107590056833e-05, 
       1.2653597338284156e-05, 
       1.0532358432657360e-05, 
       1.0508712107490871e-05, 
       1.2127941949950451e-05, 
       1.2566948973633547e-05, 
       1.1694817858472381e-05, 
       1.1310794626647904e-05, 
       1.1351314937667811e-05, 
       1.2506580372841982e-05, 
       1.2177663457467430e-05, 
       1.0043356751573709e-05, 
       1.1825744830123676e-05, 
       1.2619741042911400e-05, 
       1.0436018383848676e-05, 
       1.1736153279790776e-05, 
       1.0440200983722432e-05, 
       1.2581057553944129e-05, 
       1.0736840566740127e-05, 
       1.0987452393915333e-05, 
       1.2560404705040478e-05, 
       1.1669068649218330e-05, 
       1.0614556692125842e-05, 
       1.0614556692125842e-05, 
       1.1852608794110614e-05, 
       1.1234969511137279e-05, 
       1.2522999801233024e-05, 
       1.0925117180852579e-05, 
       1.2340345431203483e-05, 
       1.2177556172870390e-05, 
       1.2077354031971931e-05, 
       1.2203757229499747e-05, 
       1.2377389666457332e-05, 
       1.2143601259765221e-05, 
       1.1856611632928141e-05, 
       1.0400051323449531e-05, 
       1.0674788821728457e-05, 
       1.0399108779269436e-05, 
       1.0053744132178589e-05, 
       1.1625569696674678e-05, 
       1.0349662063680248e-05, 
       1.2002949800122252e-05, 
       1.2401007241727416e-05, 
       1.0342939153113044e-05, 
       9.3670097431455861e-06, 
       1.2377607250777407e-05, 
       1.1615683170861384e-05, 
       1.2250537626559507e-05, 
       1.2219696523602408e-05, 
       1.1874515054363922e-05, 
       1.0306232340991216e-05, 
       1.0432658144539479e-05, 
       1.0019291185314555e-05, 
       1.0302717880861760e-05, 
       1.2395893541423118e-05, 
       1.0030008371672095e-05, 
       1.0297812240413762e-05, 
       1.0208317787615753e-05, 
       1.1853376772846337e-05, 
       1.1491020166018797e-05, 
       1.1889100513578049e-05, 
       1.0407403538684043e-05, 
       1.1710351188532819e-05, 
       1.0515067683171620e-05, 
       9.9999999999999940e-06, 
       1.1566324598341433e-05, 
       1.1025205482800089e-05, 
       1.1025205482800089e-05, 
       1.0334363610980777e-05, 
       1.0189613441608596e-05, 
       1.0587194818543708e-05, 
       1.2229520628921372e-05, 
       1.2076292359073861e-05, 
       1.1741456861510894e-05, 
       1.0200865513927602e-05, 
       1.0475953590698195e-05, 
       1.1479892200014455e-05, 
       1.2051128764004300e-05, 
       1.2090608747023257e-05, 
       1.0127004370090950e-05, 
       1.1477608575747280e-05, 
       1.2160734109956506e-05, 
       1.0867992493628197e-05, 
       1.2113850192128568e-05, 
       1.1911988206350110e-05, 
       1.1948098404588587e-05, 
       1.2063963158768995e-05, 
       1.0312983318377638e-05, 
       1.1629094794586838e-05, 
       1.0022621621885319e-05, 
       1.1595536579347005e-05, 
       1.1695246407393041e-05, 
       1.1812296131280923e-05, 
       1.1300497158384471e-05, 
       1.2039192290143985e-05, 
       1.1606008434542128e-05, 
       1.1949360841369873e-05, 
       1.0050360783253772e-05, 
       1.1732673705138513e-05, 
       1.1853169992719367e-05, 
       1.0523392207894367e-05, 
       1.1470762308426742e-05, 
       1.1459202078066045e-05, 
       1.0186962561994858e-05, 
       1.0193686357858046e-05, 
       1.1520984510337517e-05, 
       1.0349309317390205e-05, 
       1.0269152918098087e-05, 
       8.8484650919961647e-06, 
       1.1354845515236625e-05, 
       1.1757800557543071e-05, 
       1.0240297804286224e-05, 
       1.0768466438576220e-05, 
       1.1370481931591041e-05, 
       1.0550888729578210e-05, 
       1.1752033074206899e-05, 
       1.0888696548090402e-05, 
       1.1800349753385263e-05, 
       1.0048735759995933e-05, 
       1.1309763070584687e-05, 
       1.0837666862013228e-05, 
       1.1644174202776711e-05, 
       1.0291293135718161e-05, 
       1.0272204771767398e-05, 
       1.0595571812835537e-05, 
       1.1091516617349417e-05, 
       1.1027046333736054e-05, 
       1.1353019638199176e-05, 
       1.0679995378434379e-05, 
       1.1144959444864566e-05, 
       1.1679238391135216e-05, 
       1.0773817407946089e-05, 
       1.0398844655399668e-05, 
       1.0398844655399668e-05, 
       1.1557842536867459e-05, 
       1.1531066476708649e-05, 
       1.0503387037515746e-05, 
       1.1374708756226850e-05, 
       9.1280753193642096e-06, 
       1.0021678281198925e-05, 
       1.0411308026971035e-05, 
       1.0568563641505513e-05, 
       1.0253009534947850e-05, 
       1.1541357863022209e-05, 
       1.1098433566914407e-05, 
       1.0108125320667679e-05, 
       9.3931513146355596e-06, 
       1.1072652257075415e-05, 
       9.5718606092017721e-06, 
       1.1214199276524564e-05, 
       9.9999999999996027e-06, 
       1.1450765332801129e-05, 
       1.1091869081008813e-05, 
       1.1287319872967942e-05, 
       1.0970261784340101e-05, 
       1.0998661295750510e-05, 
       1.0674522609164812e-05, 
       9.9999999999998297e-06, 
       9.9999999999999076e-06, 
       1.0134039232724425e-05, 
       1.1357172690822843e-05, 
       1.0865963429175878e-05, 
       1.1311608420053632e-05, 
       1.1305099762793722e-05, 
       1.0293697436279193e-05, 
       1.0000000000000169e-05, 
       1.0111510657551930e-05, 
       1.0162412064020230e-05, 
       1.1266143376596381e-05, 
       1.1133261164504937e-05, 
       1.1133261164504937e-05, 
       1.1012183529695346e-05, 
       1.0000000000000263e-05, 
       1.0000000000000263e-05, 
       1.0080407574724099e-05, 
       1.0080407574724099e-05, 
       1.1112532850200788e-05, 
       1.0993256584251860e-05, 
       1.0340052037401160e-05, 
       1.1123452260431328e-05, 
       1.0844070741306635e-05, 
       1.0225208581808673e-05, 
       1.1120765085876377e-05, 
       1.1088575746234184e-05, 
       1.0778394460207925e-05, 
       1.0778394460207925e-05, 
       1.1073059461357174e-05, 
       1.0960301490463042e-05, 
       1.0216391029781544e-05, 
       1.0991863651548847e-05, 
       1.0306373554340460e-05, 
       1.0330738841434769e-05, 
       1.0858748761691399e-05, 
       1.0849631402720742e-05, 
       1.0977045380432893e-05, 
       1.0975840841820361e-05, 
       1.0878729073320801e-05, 
       1.0871566471404104e-05, 
       1.0886766153139473e-05, 
       9.3273489759907413e-06, 
       1.0901075893622793e-05, 
       1.0455979399303563e-05, 
       9.0990434321070764e-06, 
       1.0824462423078507e-05, 
       8.0654569579004804e-06, 
       1.0000000000000468e-05, 
       9.9999999999999957e-06, 
       1.0000000000000170e-05, 
       1.0000000000000465e-05, 
       9.9999999999996281e-06, 
       1.0000000000000014e-05, 
       1.0000000000000209e-05, 
       1.0000000000000318e-05, 
       1.0000000000000450e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       9.9999999999999652e-06, 
       1.0000000000000426e-05, 
       1.0564065212031581e-05, 
       1.0288105753802190e-05, 
       1.0370575677714644e-05, 
       1.0556041315866993e-05, 
       1.0307439579020981e-05, 
       1.0715095308724441e-05, 
       1.0810349502745650e-05, 
       1.0264067040872218e-05, 
       1.0787179285690881e-05, 
       1.0785344832062089e-05, 
       1.0172753410981790e-05, 
       1.0720598654292347e-05, 
       1.0706982955864209e-05, 
       1.0687750516855111e-05, 
       1.0290101921696099e-05, 
       1.0686754909453875e-05, 
       1.0499263293434549e-05, 
       1.0351941646251096e-05, 
       1.0635177322745668e-05, 
       1.0440397063716049e-05, 
       1.0623649871596379e-05, 
       1.0610146193661149e-05, 
       1.0303841302466265e-05, 
       1.0153375238608839e-05, 
       1.0013914749421234e-05, 
       1.0094953744876053e-05, 
       1.0541846767509978e-05, 
       1.0194270448630770e-05, 
       1.0499712394779468e-05, 
       1.0473551813943907e-05, 
       1.0155056258731191e-05, 
       1.0282826434473411e-05, 
       1.0449145634349503e-05, 
       1.0377275934931858e-05, 
       1.0340072110039857e-05, 
       1.0065763365093438e-05, 
       1.0427275746230539e-05, 
       1.0285423930497767e-05, 
       1.0182418398486908e-05, 
       8.5832802682249529e-06, 
       1.0338219749427979e-05, 
       1.0130416766810636e-05, 
       1.0285725109203654e-05, 
       9.9999999999997111e-06, 
       9.9999999999997111e-06, 
       1.0290362655745962e-05, 
       1.0135450172217182e-05, 
       1.0082717409549143e-05, 
       1.0105133511442682e-05, 
       1.0000000000000202e-05, 
       1.0000000000000158e-05, 
       1.0000000000000069e-05, 
       1.0000000000000170e-05, 
       1.0000000000000170e-05, 
       9.9999999999995705e-06, 
       9.9999999999999517e-06, 
       9.9999999999997467e-06, 
       9.9999999999995790e-06, 
       1.0000000000000204e-05, 
       1.0000000000000370e-05, 
       9.9999999999997992e-06, 
       9.9999999999996993e-06, 
       1.0000000000000438e-05, 
       1.0000000000000255e-05, 
       9.9999999999997365e-06, 
       9.9999999999997365e-06, 
       1.0000000000000043e-05, 
       1.0000000000000043e-05, 
       9.9999999999996518e-06, 
       9.9999999999996518e-06, 
       9.9999999999995688e-06, 
       9.9999999999995688e-06, 
       9.9999999999995688e-06, 
       9.9999999999997874e-06, 
       9.9999999999997874e-06, 
       9.9999999999997874e-06, 
       1.0000000000000457e-05, 
       1.0000000000000457e-05, 
       9.9999999999995045e-06, 
       1.0000000000000473e-05, 
       9.9999999999997416e-06, 
       1.0000000000000311e-05, 
       9.9999999999996400e-06, 
       9.9999999999998433e-06, 
       9.9999999999996739e-06, 
       1.0000000000000079e-05, 
       1.0000000000000092e-05, 
       9.9999999999995722e-06, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       9.6947656195540415e-06, 
       9.6228574014668216e-06, 
       9.7034679746248499e-06, 
       7.3920536245222257e-06, 
       7.0274129384601532e-06, 
       8.9536969490748550e-06, 
       8.5126783097362612e-06, 
       6.5149018314254372e-06, 
       6.6969377474252021e-06, 
       5.5029010393726161e-06, 
       6.8751000241200378e-06, 
       5.8586255084856454e-06, 
       5.9122700370890593e-06, 
       5.5104703349470542e-06, 
       6.0489566729078503e-06, 
       4.7265633678330344e-06, 
       5.8323602227067933e-06, 
       6.3133457526778899e-06, 
       5.0223046031235508e-06, 
       5.0002790689580138e-06, 
       4.2994300730316646e-06, 
       5.0344946337972877e-06, 
       4.8415215306477271e-06, 
       5.1789303693454573e-06, 
       5.4300748049888902e-06, 
       5.2141598280559893e-06, 
       4.7455087054416432e-06, 
       4.6884863112960923e-06, 
       4.8678671067706894e-06, 
       4.7727031223995261e-06, 
       4.8387751602642047e-06, 
       3.7556461523087268e-06, 
       4.2199348874234166e-06, 
       3.6324177563669891e-06, 
       4.8692675598497281e-06, 
       4.2778472530681033e-06, 
       3.3656582427142031e-06, 
       3.2269262894011530e-06, 
       3.1799569455096466e-06, 
       3.5567328700216482e-06, 
       3.5689368010299873e-06, 
       2.7916620831483727e-06, 
       2.6982026257262879e-06, 
       3.4064074860640183e-06, 
       3.1067653024678614e-06, 
       2.6167326963182031e-06, 
       2.3716971203355896e-06, 
       2.5389747475265993e-06, 
       3.0639137397203202e-06, 
       2.4414799820587471e-06, 
       2.8030249172971114e-06, 
       2.1686230862676590e-06, 
       1.5878777924834358e-06, 
       2.1081204277837366e-06, 
       1.9314470035402226e-06, 
       2.0449555221301887e-06, 
       1.4729447210363646e-06, 
       1.6947966107605757e-06, 
       1.8010722950656040e-06, 
       1.8408263960734567e-06, 
       1.8438197083066655e-06, 
       1.6371654430150459e-06, 
       1.4340882255152124e-06, 
       1.6319399025586903e-06, 
       1.5245528968978507e-06, 
       1.1030452099890286e-06, 
       8.2708500992786734e-07, 
       6.6699390894942584e-07, 
       8.0763361059836817e-07, 
       4.9753649352562971e-07, 
       4.5490228422152818e-07, 
       2.6751288427370527e-07, 
       2.3421106498979940e-07, 
       2.1313725025835865e-07, 
       8.5491195928982287e-08, 
       5.4981671066789653e-08, 
       7.4300000000000005e-02, 
       3.0429999999999999e-02, 
       2.2290000000000001e-02, 
       1.9990000000000001e-02, 
       1.8689999999999998e-02, 
       1.6959999999999999e-02, 
       1.6230000000000001e-02, 
       1.3037558548713546e-02, 
       1.0883081458488691e-02, 
       1.0878769050655761e-02, 
       1.4189999999999999e-02, 
       1.3380000000000000e-02, 
       1.2789999999999999e-02, 
       1.2149999999999999e-02, 
       5.4599999999999996e-03, 
       4.4299999999999999e-03, 
       3.4199999999999999e-03, 
       3.3000000000000000e-03, 
       3.1800000000000001e-03, 
       2.9566112801382260e-03, 
       2.8500000000000001e-03, 
       2.0699999999999998e-03, 
       2.0200000000000001e-03, 
       2.0200000000000001e-03, 
       2.0000000000000000e-03, 
       1.9800000000000000e-03, 
       1.9300000000000001e-03, 
       1.6400000000000000e-03, 
       1.6100000000000001e-03, 
       1.2060890952148998e-03, 
       1.5800000000000000e-03, 
       1.1686083171023138e-03, 
       1.4900000000000000e-03, 
       1.3500000000000001e-03, 
       1.0399999999999999e-03, 
       7.4680407001616875e-04, 
       7.4676655249667683e-04, 
       7.2973569750647870e-04, 
       8.4999999999999995e-04, 
       6.8973737323774101e-04, 
       7.9000000000000001e-04, 
       7.6000000000000004e-04, 
       5.9744224230548612e-04, 
       6.8000000000000005e-04, 
       5.9999999999999995e-04, 
       5.6999999999999998e-04, 
       5.6999999999999998e-04, 
       5.4000000000000001e-04, 
       5.2999999999999998e-04, 
       5.2999999999999998e-04, 
       3.9776663812518353e-04, 
       3.7762891855149514e-04, 
       3.7702581747575459e-04, 
       3.8163471818196041e-04, 
       4.8000000000000001e-04, 
       4.2999999999999999e-04, 
       4.0000000000000002e-04, 
       3.6999999999999999e-04, 
       3.5000000000000000e-04, 
       2.4370788829310061e-04, 
       3.2000000000000003e-04, 
       2.3627285519262696e-04, 
       3.1000000000000000e-04, 
       3.1000000000000000e-04, 
       2.5942127348425718e-04, 
       2.5888426091302705e-04, 
       2.5886924283989409e-04, 
       2.7300268859169316e-04, 
       2.7291627567911913e-04, 
       2.9000000000000000e-04, 
       2.9000000000000000e-04, 
       2.3788732299128356e-04, 
       2.7999999999999998e-04, 
       2.7999999999999998e-04, 
       2.7999999999999998e-04, 
       2.1051633554646494e-04, 
       2.0866728104305629e-04, 
       2.4367274160316422e-04, 
       2.4365524728625894e-04, 
       2.0794897620685812e-04, 
       2.5999999999999998e-04, 
       2.5999999999999998e-04, 
       1.7171600883160595e-04, 
       1.7170411054768196e-04, 
       2.2000000000000001e-04, 
       2.2000000000000001e-04, 
       2.0000000000000001e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.5533250198832314e-04, 
       1.8000000000000001e-04, 
       1.7000000000000001e-04, 
       1.4058859547840189e-04, 
       1.6000000000000001e-04, 
       1.6000000000000001e-04, 
       1.6000000000000001e-04, 
       1.3000000000004154e-04, 
       1.0768464363767053e-04, 
       1.0768464363767053e-04, 
       1.4999999999999999e-04, 
       1.1125177649531631e-04, 
       1.0822709668576399e-04, 
       1.2338914480495807e-04, 
       1.1202816046217404e-04, 
       1.0662936127647476e-04, 
       1.0293151870100175e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       9.6955674761802201e-05, 
       9.6813612919886404e-05, 
       1.0512758152498566e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.0854129662748241e-04, 
       1.0000000000000323e-04, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       1.1000000000000000e-04, 
       9.3250503713334041e-05, 
       8.5009088244708902e-05, 
       8.2886632511872054e-05, 
       8.2558474534351651e-05, 
       1.0000000000000000e-04, 
       8.0000000000003259e-05, 
       8.0584601377312788e-05, 
       6.5524020893980520e-05, 
       7.6482276851296802e-05, 
       7.6465672125631959e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       7.5198128283420474e-05, 
       6.9291714827347215e-05, 
       6.9268994988211548e-05, 
       8.2402233907603384e-05, 
       6.4630394994564559e-05, 
       6.4616272204064322e-05, 
       8.1917723578008500e-05, 
       7.0000000000005266e-05, 
       7.9365343260021315e-05, 
       6.9772390506842597e-05, 
       6.0000000000003647e-05, 
       5.9999999999997928e-05, 
       5.9999999999997928e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       6.2916140622468645e-05, 
       6.6331209696967878e-05, 
       7.1389511748867854e-05, 
       5.8736195175162288e-05, 
       5.9999999999995651e-05, 
       5.3042837422496945e-05, 
       5.2967416357068542e-05, 
       5.0429429860771925e-05, 
       6.0000000000001831e-05, 
       6.0000000000003376e-05, 
       6.0425404444090088e-05, 
       6.3897869755598016e-05, 
       5.1611735176274870e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       5.0809716377248796e-05, 
       5.5145331172950521e-05, 
       5.5053685336879535e-05, 
       5.8684151219614443e-05, 
       4.9276780129911816e-05, 
       5.7341132258635974e-05, 
       5.0000000000000158e-05, 
       4.7481446363360433e-05, 
       5.4238958410724838e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       4.2476744935787153e-05, 
       4.8124917068804158e-05, 
       4.4363021086367883e-05, 
       3.9999999999999346e-05, 
       4.9427201300072842e-05, 
       4.9394586853485093e-05, 
       4.0316193439998853e-05, 
       3.9510200506935524e-05, 
       3.7774317278009673e-05, 
       3.8587510437276392e-05, 
       3.9037154783625315e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       3.6607343541813714e-05, 
       4.2473647468209782e-05, 
       4.0000000000001630e-05, 
       3.6437913099884315e-05, 
       3.6437913099884315e-05, 
       3.9999999999998872e-05, 
       4.2503529537477015e-05, 
       3.6230852690919030e-05, 
       3.8115043083519835e-05, 
       3.6785623129306869e-05, 
       3.6670707271088484e-05, 
       3.5476255008093015e-05, 
       3.9501604122313745e-05, 
       3.3673151590614655e-05, 
       3.4370803001368847e-05, 
       3.5638451843159527e-05, 
       4.2236799611545319e-05, 
       3.7303178640235137e-05, 
       4.0000000000003994e-05, 
       3.2974215585232872e-05, 
       3.2863419025276005e-05, 
       2.9983816215052012e-05, 
       3.4129102789038733e-05, 
       2.9999999999997687e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.9729572577651724e-05, 
       3.0000000000000285e-05, 
       2.8949364234034643e-05, 
       3.0000000000003379e-05, 
       2.8922016746195419e-05, 
       3.5761930564004351e-05, 
       2.4820186271723133e-05, 
       2.7505771804118539e-05, 
       2.6203440903550380e-05, 
       2.5622241719660553e-05, 
       2.6717085180325387e-05, 
       3.0429864703042079e-05, 
       3.0424433406478087e-05, 
       2.6750853665162942e-05, 
       2.3806059537227716e-05, 
       2.7911368558389395e-05, 
       2.7886005610531906e-05, 
       2.5833628253127539e-05, 
       2.5833628253127539e-05, 
       2.6925610904461553e-05, 
       2.8856589112952020e-05, 
       2.3141411091770025e-05, 
       2.1502482397030963e-05, 
       2.4093888675263956e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.1139297388069430e-05, 
       2.3738438575771808e-05, 
       2.5241203407279468e-05, 
       2.5227186305818006e-05, 
       2.1838002479472441e-05, 
       2.5546326190138833e-05, 
       2.4490522756024003e-05, 
       2.2281851648640559e-05, 
       2.2281851648640559e-05, 
       2.2542980929659354e-05, 
       2.0681033298082310e-05, 
       2.1706939917175129e-05, 
       2.3743958143640291e-05, 
       2.0000000000001787e-05, 
       2.0429899761154197e-05, 
       2.0314490792511520e-05, 
       2.1358537063817187e-05, 
       2.5709715048454072e-05, 
       2.0000000000002177e-05, 
       2.0000000000002177e-05, 
       2.0751242983137847e-05, 
       1.9522640651934186e-05, 
       2.1833080183892826e-05, 
       1.9999999999999778e-05, 
       2.0389459078440465e-05, 
       2.2824376824859196e-05, 
       2.2810661382132814e-05, 
       2.0771481673299070e-05, 
       2.4284014889381276e-05, 
       1.8444510856471818e-05, 
       1.7891690628402386e-05, 
       1.9693546676850977e-05, 
       1.8133988054219961e-05, 
       1.9605419457062955e-05, 
       1.9999999999995722e-05, 
       1.9999999999995722e-05, 
       1.9999999999995722e-05, 
       2.0000000000003129e-05, 
       1.6654628576726314e-05, 
       1.9441566184141674e-05, 
       1.8278957718691330e-05, 
       1.6477693308954065e-05, 
       2.2881877795479631e-05, 
       1.7542530628398943e-05, 
       2.2882445962267331e-05, 
       1.7254925141115230e-05, 
       1.6215177534801218e-05, 
       1.7920954471522880e-05, 
       1.7521104817646689e-05, 
       1.8640485850935727e-05, 
       1.7392269637706035e-05, 
       2.1667342008788285e-05, 
       1.6772734328430967e-05, 
       2.1494670141462031e-05, 
       1.8113249002410183e-05, 
       1.5540352289672671e-05, 
       1.5540352289672671e-05, 
       1.5003592868804477e-05, 
       1.7914925055106471e-05, 
       1.7713243756555757e-05, 
       1.6168105300603769e-05, 
       1.6103363992769436e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.4372368340300102e-05, 
       1.4186133759386960e-05, 
       1.4352151547141893e-05, 
       1.4095798120461432e-05, 
       1.4062232331749144e-05, 
       1.5240760770330997e-05, 
       1.4214586288609827e-05, 
       1.4027601482944790e-05, 
       1.4790448963432358e-05, 
       1.5387424384441902e-05, 
       1.3791715489902704e-05, 
       1.3731644871520615e-05, 
       1.4647967385860729e-05, 
       1.3751845478867160e-05, 
       1.3315240160250380e-05, 
       1.6368813243386031e-05, 
       1.3585574458850988e-05, 
       1.4651606809567796e-05, 
       1.3528291819554959e-05, 
       1.3911099935290847e-05, 
       1.3487659352217637e-05, 
       1.3585294646382596e-05, 
       1.3165620741512801e-05, 
       1.3280634984460224e-05, 
       1.3116683353679425e-05, 
       1.2932481849632644e-05, 
       1.3121556345327799e-05, 
       1.6179932926484389e-05, 
       1.6157554189723095e-05, 
       1.3187338425071071e-05, 
       1.7709221213291686e-05, 
       1.2829841309757566e-05, 
       1.4344039135810327e-05, 
       1.4344039135810327e-05, 
       1.7561032328014526e-05, 
       1.2580137620756254e-05, 
       1.2518388139364237e-05, 
       1.2931451733940866e-05, 
       1.2482770676828027e-05, 
       1.2833950559067186e-05, 
       1.2209315251853346e-05, 
       1.3537988816381454e-05, 
       1.2748232853975038e-05, 
       1.3535279845194488e-05, 
       1.2564127097208087e-05, 
       1.5773362520496820e-05, 
       1.4271139758125524e-05, 
       1.3079603232597115e-05, 
       1.3077623302636237e-05, 
       1.5618266512877338e-05, 
       1.1904707309409523e-05, 
       1.1868372842740387e-05, 
       1.5827830634012848e-05, 
       1.3932303653497273e-05, 
       1.2034743169603295e-05, 
       1.1989413604475069e-05, 
       1.5684758081397685e-05, 
       1.3030967904734999e-05, 
       1.1833810921141765e-05, 
       1.1901002660347376e-05, 
       1.4133064189559994e-05, 
       1.1384110184290020e-05, 
       1.3815740881573331e-05, 
       1.2276459140089718e-05, 
       1.1608332428358637e-05, 
       1.1679612035632459e-05, 
       1.2029435108224906e-05, 
       1.2415220706934840e-05, 
       1.3013184344661012e-05, 
       1.1383559274334398e-05, 
       1.1423430917571241e-05, 
       1.2842889803800579e-05, 
       1.1300144854427703e-05, 
       1.1253984542610072e-05, 
       1.0769933943114439e-05, 
       1.2872633098187266e-05, 
       1.2759283008884534e-05, 
       1.1167374569971501e-05, 
       1.1167374569971501e-05, 
       1.0980695929938487e-05, 
       1.0504652896427845e-05, 
       1.0532436620651496e-05, 
       1.1392881516369259e-05, 
       1.0217986612048792e-05, 
       1.0178406623445530e-05, 
       1.3256878290149601e-05, 
       1.3160862113254326e-05, 
       1.2006034749489678e-05, 
       1.0591357225369108e-05, 
       1.0028890157238654e-05, 
       1.0735149792831343e-05, 
       1.0441092420834546e-05, 
       1.2790202262021769e-05, 
       1.1058056740146791e-05, 
       1.0191828058226249e-05, 
       1.0172217841118659e-05, 
       1.0048667149972156e-05, 
       1.0089991466293971e-05, 
       1.0801299942415698e-05, 
       1.1430142406271294e-05, 
       1.0649242621603342e-05, 
       1.0446964347364076e-05, 
       1.0017296267836176e-05, 
       1.0019618595250704e-05, 
       1.0836278626856517e-05, 
       1.0143419544145963e-05, 
       1.0857729731242161e-05, 
       1.0579743789073739e-05, 
       1.0896689356192545e-05, 
       1.1444580107221192e-05, 
       9.9999999999999212e-06, 
       1.1729141856298901e-05, 
       1.0215757516093120e-05, 
       1.1078860683095621e-05, 
       1.0635545193756356e-05, 
       1.0205809284028449e-05, 
       9.9999999999995146e-06, 
       1.0110474338076040e-05, 
       1.0107719544353477e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       5.9745227469730624e-06, 
       5.0951155718873691e-06, 
       4.7618291081441396e-06, 
       2.2000000000000001e-04, 
       1.0087314571106695e-04, 
       1.0083334479271939e-04, 
       9.0000000000000006e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       1.1146003069080525e-05, 
       1.0800050286712896e-05, 
       1.1246653573297428e-05, 
       1.1246653573297428e-05, 
       1.0000000000000001e-05, 
       3.4902409205208069e-06, 
       3.4833260440384979e-06, 
       3.2362948121349587e-06, 
       3.2357566821195204e-06, 
       2.9690018846829778e-06, 
       2.9682706512494376e-06, 
       2.8072116463778653e-06, 
       2.7786644396826463e-06, 
       2.7784064857383409e-06, 
       2.2520425231980443e-06, 
       2.1833608955175073e-06, 
       1.9450551588159206e-06, 
       1.9284968462130885e-06, 
       1.8430097991819099e-06, 
       1.5988315025429415e-06, 
       1.5988315025429415e-06, 
       1.5213327210532660e-06, 
       1.4586075279245748e-06, 
       1.4586075279245748e-06, 
       1.3314436719903048e-06, 
       1.6743986361808660e-06, 
       1.6774389725163116e-06, 
       1.1832334624743139e-06, 
       1.1528463269626069e-06, 
       1.1464196783629788e-06, 
       1.2473860221672384e-06, 
       1.2426583459896085e-06, 
       1.1098055498383791e-06, 
       1.0824410253831882e-06, 
       1.0281543706378878e-06, 
       1.1050332336199652e-06, 
       1.0440587645220091e-06, 
       3.5757309351598726e-07, 
       1.0586189945469200e+00, 
       7.2805201953374787e-01, 
       6.8280880054648996e-01, 
       5.9826466595576788e-01, 
       4.1020630963524468e-01, 
       4.5646999999997689e-01, 
       4.5276999999998541e-01, 
       2.0476399084864377e-01, 
       2.0371387499613716e-01, 
       2.0498740218224085e-01, 
       1.6720763217250953e-01, 
       1.6718951595312512e-01, 
       1.3475149516881021e-01, 
       1.2671484836488850e-01, 
       1.6659269878102087e-01, 
       1.5747892040940964e-01, 
       1.4451289627322528e-01, 
       1.1192876215558543e-01, 
       1.1181140455066313e-01, 
       1.3828052365278862e-01, 
       1.1047692012599106e-01, 
       1.1038616847914615e-01, 
       1.2681900458160425e-01, 
       1.2353537343789060e-01, 
       1.2058657100742542e-01, 
       1.2028056662794633e-01, 
       1.1613190138640254e-01, 
       1.0524828944696524e-01, 
       1.0688999999997963e-01, 
       1.0510671048105651e-01, 
       1.0069486820979097e-01, 
       9.9414858210953250e-02, 
       9.5529999999999324e-02, 
       7.7225433880364042e-02, 
       8.5079999999995035e-02, 
       7.0724544331379507e-02, 
       5.9017169211197630e-02, 
       5.8553916215287917e-02, 
       7.9527568657309616e-02, 
       6.3085137780566874e-02, 
       6.3066812559668478e-02, 
       6.2043854085539162e-02, 
       6.0855887283131696e-02, 
       5.6760274224216332e-02, 
       6.1041780373860551e-02, 
       5.6130002727076030e-02, 
       6.0131480927786259e-02, 
       4.3213017855050009e-02, 
       5.7327620604158566e-02, 
       5.5357006725964497e-02, 
       5.2616624102980153e-02, 
       3.9508360570041971e-02, 
       5.3449999999995251e-02, 
       3.9692924601571006e-02, 
       5.1762400957608659e-02, 
       3.6006552060059871e-02, 
       3.5962990042790466e-02, 
       4.8855307372525685e-02, 
       3.7236248887429650e-02, 
       3.7563096221859084e-02, 
       3.7218290926347805e-02, 
       4.9030411986050570e-02, 
       4.8903147217186776e-02, 
       4.8579980199933252e-02, 
       3.4321570193623280e-02, 
       4.6079842075371358e-02, 
       4.5609760507664410e-02, 
       3.3497324033231260e-02, 
       4.2833221499412863e-02, 
       4.3972546879275584e-02, 
       4.1197792006362151e-02, 
       4.3694920131661520e-02, 
       3.1296757703246740e-02, 
       4.2897038688336810e-02, 
       4.3492047955079383e-02, 
       3.1194425230905414e-02, 
       3.5441743857117541e-02, 
       3.0977985293214511e-02, 
       4.2566573638858553e-02, 
       3.0515096235953068e-02, 
       3.0067356170249988e-02, 
       2.9766490277874622e-02, 
       2.8711951509417761e-02, 
       2.8474563087796573e-02, 
       3.8861805242770567e-02, 
       3.8560000000000441e-02, 
       2.9123484022077557e-02, 
       2.8892529312384712e-02, 
       2.7546259454840828e-02, 
       3.7781089462793793e-02, 
       3.7604118230415283e-02, 
       3.7245629674337949e-02, 
       2.5866656054236093e-02, 
       3.4995927445234186e-02, 
       3.4903973577319065e-02, 
       3.4504790190709296e-02, 
       3.2819158543384755e-02, 
       3.1230687116812633e-02, 
       2.9440000000000001e-02, 
       2.8153426058541760e-02, 
       2.4655337345933302e-02, 
       2.7775316583689071e-02, 
       2.8070000000000428e-02, 
       2.7737911365600075e-02, 
       2.6890930739175839e-02, 
       2.3584211831951040e-02, 
       2.7028045602510171e-02, 
       2.6055436951890359e-02, 
       2.2123374594184189e-02, 
       2.4870000000003698e-02, 
       2.4699999999997772e-02, 
       1.7773991724927683e-02, 
       2.1256384275186852e-02, 
       1.7322420862266562e-02, 
       2.2070346158363792e-02, 
       1.8394252562409022e-02, 
       2.0136071619831171e-02, 
       2.1265023883021140e-02, 
       1.6211175955531020e-02, 
       1.5692828194415732e-02, 
       1.7780420322466840e-02, 
       1.5837231213278020e-02, 
       1.4094510037679675e-02, 
       1.6407090837951589e-02, 
       1.8657709185428913e-02, 
       1.5654395597788209e-02, 
       1.3621206906145515e-02, 
       1.8440011409768566e-02, 
       1.7507748724426508e-02, 
       1.7497629777917821e-02, 
       1.3497022692622788e-02, 
       1.4849232643256069e-02, 
       1.4165326641825940e-02, 
       1.2193984070672469e-02, 
       1.3746145009265819e-02, 
       1.3745911991719062e-02, 
       1.2952638015733941e-02, 
       1.3310301946101498e-02, 
       1.4290029170608971e-02, 
       1.2005230953029848e-02, 
       1.1171710700502747e-02, 
       1.0892484440708176e-02, 
       1.0691369230488378e-02, 
       1.4581261713517446e-02, 
       1.0460892833448233e-02, 
       1.2466269547663882e-02, 
       1.4347134314226637e-02, 
       1.0277458233091862e-02, 
       1.0390367599609164e-02, 
       1.1664555767618207e-02, 
       1.3935959140827859e-02, 
       1.1502089434336709e-02, 
       1.0035074611922408e-02, 
       1.3332895883585449e-02, 
       1.2814405244782105e-02, 
       1.0724944736576871e-02, 
       1.0724942549692756e-02, 
       1.0532770769392014e-02, 
       1.2500696312718790e-02, 
       1.0084446830977221e-02, 
       9.1905665206305932e-03, 
       1.0063604306485967e-02, 
       8.7998793132223509e-03, 
       1.1849602013920687e-02, 
       1.1591269944928326e-02, 
       1.1069055779414416e-02, 
       1.1525366736383423e-02, 
       8.6129361937721232e-03, 
       1.1579997557221684e-02, 
       1.1375458404156701e-02, 
       8.3328350519636003e-03, 
       1.1466077160299076e-02, 
       8.4384727486650542e-03, 
       1.1254606790423116e-02, 
       1.0369315070912183e-02, 
       1.1093255446470662e-02, 
       1.0973729290755629e-02, 
       9.1129240375512292e-03, 
       9.1121055359147975e-03, 
       1.1082293567540669e-02, 
       8.6883213624022616e-03, 
       8.1309044266609527e-03, 
       8.1254775519283284e-03, 
       1.0357623199717381e-02, 
       7.6450667512031577e-03, 
       1.0385645321199127e-02, 
       8.9447549901228527e-03, 
       9.8721764024740904e-03, 
       7.5468973167172942e-03, 
       1.0228075128839329e-02, 
       7.4309410898846752e-03, 
       9.8924765093901714e-03, 
       9.9000000000004119e-03, 
       7.9337432731794322e-03, 
       7.9027973659683889e-03, 
       9.6334059574371897e-03, 
       7.0847542530796210e-03, 
       7.8280646746504319e-03, 
       7.8217834220058909e-03, 
       7.0910998915362953e-03, 
       8.1043023534402066e-03, 
       7.8038869586495889e-03, 
       8.0750077938751223e-03, 
       9.2199999999999609e-03, 
       9.1606472767959708e-03, 
       6.6954745217432604e-03, 
       8.9821635758456746e-03, 
       7.8523927501902083e-03, 
       6.4220634715684011e-03, 
       7.8313950900798859e-03, 
       8.9014344912121731e-03, 
       8.9000068391243536e-03, 
       8.5299999999998259e-03, 
       8.3866412217897205e-03, 
       8.3238695041319231e-03, 
       8.2554262942135127e-03, 
       8.0143500657427567e-03, 
       7.8460202885916406e-03, 
       7.8400000000001090e-03, 
       7.6035786341608572e-03, 
       7.0197045353720296e-03, 
       7.5771555622992813e-03, 
       5.3963327063935194e-03, 
       7.5537592241843249e-03, 
       7.5069594686505061e-03, 
       7.4500000000004094e-03, 
       5.3880699294922060e-03, 
       5.4443186877411376e-03, 
       5.3833397695102272e-03, 
       5.3484592960283388e-03, 
       6.1367549131050147e-03, 
       7.2419808324227499e-03, 
       5.2112265103333982e-03, 
       7.2048353619812051e-03, 
       6.9178847748125119e-03, 
       7.1154678341725881e-03, 
       5.9063015285011448e-03, 
       6.8038524220089183e-03, 
       6.2509246775152138e-03, 
       6.8467038945936457e-03, 
       5.3112617387627362e-03, 
       5.3861480837971163e-03, 
       5.3792584964366874e-03, 
       5.8860542738467178e-03, 
       5.5859951427428105e-03, 
       5.0496371004821365e-03, 
       6.1615254724534645e-03, 
       5.5429396430689663e-03, 
       6.6406942522889484e-03, 
       5.0589134875469021e-03, 
       5.7711855533176745e-03, 
       6.5825815219745839e-03, 
       6.5815552428157469e-03, 
       5.0174940503949000e-03, 
       6.5799999999996200e-03, 
       5.3635290751001671e-03, 
       6.4415138894880790e-03, 
       6.3362273685503281e-03, 
       6.3302268361631314e-03, 
       6.2864230363762650e-03, 
       6.3074606552462114e-03, 
       4.5824775510914083e-03, 
       4.4359282230097977e-03, 
       5.4955163866318366e-03, 
       6.2099999999998849e-03, 
       4.6255111857770159e-03, 
       4.4819112006673498e-03, 
       6.1658814560769425e-03, 
       4.6040827860291999e-03, 
       4.2370131485177055e-03, 
       4.3134331035689791e-03, 
       5.9170543583771211e-03, 
       4.2910891863148694e-03, 
       5.7855942534332004e-03, 
       5.6668757412662273e-03, 
       5.7920392111615072e-03, 
       4.2487815879904802e-03, 
       5.5904133977084467e-03, 
       5.7257183371995134e-03, 
       4.2114657748962715e-03, 
       4.6492154045220469e-03, 
       4.4081777084087875e-03, 
       4.1810932664565091e-03, 
       5.7635191029758292e-03, 
       4.1075472119086586e-03, 
       4.6469956228837811e-03, 
       5.4625161322060679e-03, 
       4.1097956526734069e-03, 
       4.9782480487089199e-03, 
       3.9637238746577947e-03, 
       4.0818046230106219e-03, 
       5.4960466610222847e-03, 
       5.5620163306775708e-03, 
       3.9819043236940192e-03, 
       4.3161791874148214e-03, 
       4.3155454777283383e-03, 
       5.5062924487427629e-03, 
       3.9872780819929588e-03, 
       3.8875552584753084e-03, 
       5.0499617861224861e-03, 
       4.5171121772086950e-03, 
       5.4093746395381514e-03, 
       5.3484391130435623e-03, 
       3.8173318692585850e-03, 
       4.3045132590095510e-03, 
       4.3424015952698422e-03, 
       4.3666534216189714e-03, 
       5.1829232166587308e-03, 
       4.8906088548139335e-03, 
       3.6515923709263720e-03, 
       4.1116293335541178e-03, 
       3.7254134160732790e-03, 
       3.8091123803841507e-03, 
       3.7651040996554436e-03, 
       5.0432776609903932e-03, 
       3.7504227753855845e-03, 
       4.8244938157068302e-03, 
       3.7736111653070537e-03, 
       4.8088695971662820e-03, 
       4.9427519540123695e-03, 
       3.6769611546644755e-03, 
       4.9121417251174514e-03, 
       4.8799999999997118e-03, 
       3.7784626460440748e-03, 
       4.6121131751077305e-03, 
       4.0297916523199589e-03, 
       3.5883354333237972e-03, 
       4.7156158548982575e-03, 
       3.9366995289245232e-03, 
       3.5056285480428392e-03, 
       3.9127003849842830e-03, 
       4.6866297829639180e-03, 
       3.6531887671828640e-03, 
       4.5841899738456376e-03, 
       3.8416666042538276e-03, 
       4.5453993319281216e-03, 
       3.4371992621797759e-03, 
       3.3805868423897783e-03, 
       4.4333298224658799e-03, 
       3.9125737098600535e-03, 
       3.4805018894940466e-03, 
       3.2076279217153705e-03, 
       3.2028848864407017e-03, 
       4.2950558476862995e-03, 
       4.3599210214565320e-03, 
       3.2957239771295101e-03, 
       4.4638922879847737e-03, 
       4.2354210699997227e-03, 
       4.4947521691785551e-03, 
       3.4970537049163913e-03, 
       3.8671268788900521e-03, 
       4.4700000000000685e-03, 
       4.3777607920648708e-03, 
       4.3903675415080225e-03, 
       3.2389415376227124e-03, 
       3.3368172477021346e-03, 
       4.3281005184580557e-03, 
       3.2084699062838325e-03, 
       3.3178915101349550e-03, 
       3.4118294204911860e-03, 
       4.3287152005234393e-03, 
       3.5722662977477376e-03, 
       3.3687689836534086e-03, 
       3.3678563427398788e-03, 
       3.5262145219161077e-03, 
       3.4411445239224624e-03, 
       3.4372510175075936e-03, 
       3.6041231946871712e-03, 
       4.2400000001636302e-03, 
       4.2202452653535586e-03, 
       4.2117585164912598e-03, 
       4.2093777774100255e-03, 
       3.2064187120530011e-03, 
       3.1875423203699899e-03, 
       4.0119287956154916e-03, 
       3.4036238702117136e-03, 
       3.4209094898837791e-03, 
       3.1124289412179456e-03, 
       3.0055638819827162e-03, 
       3.0055638819827162e-03, 
       3.8626740227089480e-03, 
       2.9779065697888045e-03, 
       3.0294852761129521e-03, 
       3.4667072239767771e-03, 
       3.9996029064628411e-03, 
       2.9694821568150291e-03, 
       3.9106160851391843e-03, 
       2.9688175123321683e-03, 
       3.2121142719604987e-03, 
       3.8900000000001043e-03, 
       2.7523136147068927e-03, 
       2.8106285979094146e-03, 
       3.2557814197086287e-03, 
       3.0289595430954834e-03, 
       2.7483477625341091e-03, 
       2.9814666928773348e-03, 
       3.1335110018929106e-03, 
       3.7715923510223334e-03, 
       2.8277297186310407e-03, 
       3.7914955718626915e-03, 
       3.0181274720775756e-03, 
       3.6996905317986128e-03, 
       3.5666021445065988e-03, 
       3.3491668679870891e-03, 
       3.5388143473619677e-03, 
       2.8658636879760704e-03, 
       2.6221132548375804e-03, 
       3.3102662828063552e-03, 
       2.6670845037548921e-03, 
       2.8020772501898619e-03, 
       3.1734406936635238e-03, 
       3.4938930920938834e-03, 
       3.4976477329445653e-03, 
       3.4082970421393552e-03, 
       2.8598412722383088e-03, 
       3.4300000000004158e-03, 
       2.7315031883881134e-03, 
       3.1450006687706446e-03, 
       2.5041051908106116e-03, 
       3.1005774801966205e-03, 
       3.3384918908145240e-03, 
       3.0705489398676626e-03, 
       2.4832677614024731e-03, 
       2.4062815559438878e-03, 
       3.2719395793966849e-03, 
       2.4243255974249586e-03, 
       3.2082153419301725e-03, 
       2.9140079914865480e-03, 
       2.2706563737608897e-03, 
       3.1104683701458666e-03, 
       2.3096002490656190e-03, 
       2.6400144241326396e-03, 
       2.2663972870403848e-03, 
       3.0922271462426737e-03, 
       3.0292617223108271e-03, 
       2.2128740519034669e-03, 
       2.5039862098161759e-03, 
       3.0274666181308537e-03, 
       3.0541503475918848e-03, 
       3.0623144167858940e-03, 
       2.2745434560819309e-03, 
       2.1513223463192977e-03, 
       2.9667488016870552e-03, 
       2.9920503942038371e-03, 
       2.8562228542088875e-03, 
       2.1362725675902424e-03, 
       2.9899999999996639e-03, 
       2.9414767157264878e-03, 
       2.8514042762181175e-03, 
       2.9652212180623001e-03, 
       2.9585189420428515e-03, 
       2.9064433346914788e-03, 
       2.6680471429555669e-03, 
       2.5198492114006524e-03, 
       2.8863713896311754e-03, 
       2.1357250359881211e-03, 
       2.3560073514574597e-03, 
       2.2560524458250944e-03, 
       2.8242902410177025e-03, 
       2.3538172788601106e-03, 
       2.5474756177860983e-03, 
       2.1725687216684821e-03, 
       2.9061613171955333e-03, 
       2.4910328428001094e-03, 
       2.2534800678002138e-03, 
       2.8865196520503136e-03, 
       2.8096731769514170e-03, 
       2.7860226920468720e-03, 
       2.2568429423960655e-03, 
       2.3114190410703402e-03, 
       2.0305779135085448e-03, 
       2.8099999999998187e-03, 
       2.7841626987549551e-03, 
       2.5692091946682197e-03, 
       2.7899999999998009e-03, 
       2.0123825172653036e-03, 
       2.7631406829313693e-03, 
       2.7654621648997182e-03, 
       2.6724362488201574e-03, 
       2.2442370808073426e-03, 
       1.9631926935946313e-03, 
       1.9608628710692938e-03, 
       2.7305994929778580e-03, 
       2.0790612358289945e-03, 
       2.6966957366308859e-03, 
       2.7028319935185172e-03, 
       2.6195159587763133e-03, 
       2.6551549685462027e-03, 
       2.0270615562351870e-03, 
       2.2001862380952907e-03, 
       1.9981611910333649e-03, 
       2.2687790989754431e-03, 
       2.2128249336333325e-03, 
       2.6200374500365171e-03, 
       2.1000000000001842e-03, 
       1.9134298735014564e-03, 
       2.1375312343416600e-03, 
       2.5364043928730339e-03, 
       1.8693458641944184e-03, 
       2.0585907274822172e-03, 
       1.8949490969125680e-03, 
       2.0567202553391970e-03, 
       2.1416533717555156e-03, 
       2.5699995960119042e-03, 
       2.5680741086709221e-03, 
       2.1234642455712663e-03, 
       2.0585855043685724e-03, 
       2.0537412389877471e-03, 
       1.8052342581872638e-03, 
       1.9299999999997975e-03, 
       2.3896474478606036e-03, 
       2.5006310868285067e-03, 
       1.9634220820486077e-03, 
       1.8408973855317944e-03, 
       1.9376484829262488e-03, 
       1.8372194575687188e-03, 
       2.4468998966982384e-03, 
       2.4230211567243744e-03, 
       1.7334733112882410e-03, 
       1.7608770532946652e-03, 
       1.7515430398928440e-03, 
       1.7871859027667595e-03, 
       2.1708831375069487e-03, 
       1.7077451812340990e-03, 
       1.8007437248079537e-03, 
       1.6560345365226948e-03, 
       1.7261961392716785e-03, 
       2.3234941511659150e-03, 
       2.2133783237660554e-03, 
       1.7989091496158835e-03, 
       1.9154810054940661e-03, 
       2.2315818883796960e-03, 
       1.6539828710631730e-03, 
       2.1273066988111464e-03, 
       1.5998309394343266e-03, 
       1.9414251018069419e-03, 
       1.8278968770219135e-03, 
       1.8276512878618292e-03, 
       1.6393303833125356e-03, 
       2.2125536357380394e-03, 
       2.2300000000000054e-03, 
       2.1220642339331752e-03, 
       2.0663953537167928e-03, 
       2.2000000000004919e-03, 
       2.0912839952728327e-03, 
       1.6798213788333681e-03, 
       1.6785349101515520e-03, 
       1.8310716607564462e-03, 
       2.1899999999998392e-03, 
       2.1183933999970057e-03, 
       2.1812488520033288e-03, 
       2.1799999999995035e-03, 
       2.0177493489599331e-03, 
       1.6196554554146635e-03, 
       1.5759829805236554e-03, 
       1.7559203792237360e-03, 
       2.0643782094814430e-03, 
       1.5994351503272079e-03, 
       1.5130640838427697e-03, 
       1.8269694170747201e-03, 
       2.0302060041770217e-03, 
       1.9012298852765380e-03, 
       1.5775472687580702e-03, 
       1.7440441556429451e-03, 
       1.8406261146662166e-03, 
       2.0422796826856989e-03, 
       1.5577534887345889e-03, 
       1.5228601278934530e-03, 
       2.0771035049388397e-03, 
       1.5866548732582132e-03, 
       2.0716967087883731e-03, 
       1.9180561983708714e-03, 
       1.5740952178952352e-03, 
       2.0513139621791011e-03, 
       1.9573459473061004e-03, 
       1.8634200241438696e-03, 
       2.0251656955611358e-03, 
       1.9979317998758054e-03, 
       1.9887350858284738e-03, 
       1.9372602716283358e-03, 
       1.4840360517477664e-03, 
       1.9455046277369781e-03, 
       1.4822101348284575e-03, 
       1.4053910786393886e-03, 
       1.9395823817937978e-03, 
       1.9177600636645495e-03, 
       1.5264015753866627e-03, 
       1.9603758840579643e-03, 
       1.9417499485921484e-03, 
       1.4511862031092969e-03, 
       1.8646742637309633e-03, 
       1.9252181815478510e-03, 
       1.3599999999999604e-03, 
       1.9119292767271641e-03, 
       1.8961110713132993e-03, 
       1.5333158329601207e-03, 
       1.4139230297736539e-03, 
       1.4137389044732742e-03, 
       1.3570103165572672e-03, 
       1.3395039043591094e-03, 
       1.4004387273964859e-03, 
       1.8648709830104543e-03, 
       1.5130971212553176e-03, 
       1.5393815011184632e-03, 
       1.4975052381680644e-03, 
       1.8100000000000770e-03, 
       1.8043253317345553e-03, 
       1.3279191165339382e-03, 
       1.7777360197437973e-03, 
       1.2722045352822973e-03, 
       1.2714458707312442e-03, 
       1.7700000000002776e-03, 
       1.2710032749901944e-03, 
       1.5135480540596940e-03, 
       1.7600000000000896e-03, 
       1.7511916513083719e-03, 
       1.5481382404551379e-03, 
       1.2975004898291829e-03, 
       1.7277585702093344e-03, 
       1.2501574469829507e-03, 
       1.2231019609888045e-03, 
       1.7243741281408810e-03, 
       1.6308845089451603e-03, 
       1.2793440737035740e-03, 
       1.5006048138909867e-03, 
       1.5005378897688807e-03, 
       1.7069255032667000e-03, 
       1.3620550508441410e-03, 
       1.6230642962504198e-03, 
       1.4553063554247243e-03, 
       1.6218995245217852e-03, 
       1.4552063322303874e-03, 
       1.6789555159447637e-03, 
       1.4449559701513368e-03, 
       1.2659318488345850e-03, 
       1.5209138572461723e-03, 
       1.5990350323246752e-03, 
       1.6098241486649750e-03, 
       1.5757189329287491e-03, 
       1.2033138231236421e-03, 
       1.1857708139754795e-03, 
       1.1895723261845439e-03, 
       1.5659897314885961e-03, 
       1.1526345228659893e-03, 
       1.3354002368328906e-03, 
       1.2924142812332251e-03, 
       1.5320719587495141e-03, 
       1.2910282445561675e-03, 
       1.6099999999996226e-03, 
       1.2201642050523411e-03, 
       1.5501524665558076e-03, 
       1.3514172345681704e-03, 
       1.1830294696117383e-03, 
       1.1607915875320699e-03, 
       1.1807697503357464e-03, 
       1.5046698143431768e-03, 
       1.1985413357551227e-03, 
       1.1984574962452439e-03, 
       1.2778856906560383e-03, 
       1.3238302111596436e-03, 
       1.2948415778532567e-03, 
       1.5231039169712336e-03, 
       1.5013362254606114e-03, 
       1.5387958732749451e-03, 
       1.1507676226198125e-03, 
       1.4228335250778086e-03, 
       1.0850858623308514e-03, 
       1.5200000000004012e-03, 
       1.5200000000004012e-03, 
       1.1143897348160124e-03, 
       1.2452860537254702e-03, 
       1.2710603182758938e-03, 
       1.2432739331407364e-03, 
       1.4899999999996333e-03, 
       1.2741457912882313e-03, 
       1.3269646348660563e-03, 
       1.0397532081773630e-03, 
       1.2781405635310262e-03, 
       1.4496656523910268e-03, 
       1.0527540996661140e-03, 
       1.3203696992965630e-03, 
       1.4070013779843304e-03, 
       1.3799659491676719e-03, 
       1.0165041437892341e-03, 
       1.0196281928890548e-03, 
       1.3680492913381859e-03, 
       1.0402369816443502e-03, 
       1.4147265674069212e-03, 
       1.4100171484122115e-03, 
       1.0908604293445509e-03, 
       1.3618890119170967e-03, 
       1.3237732202611872e-03, 
       9.9708221566243826e-04, 
       1.1611524198720947e-03, 
       1.1629517505211317e-03, 
       1.3438264426949174e-03, 
       1.1506131523817075e-03, 
       1.0057892056631562e-03, 
       1.1423111598481066e-03, 
       1.2599999999997630e-03, 
       1.3703270139586788e-03, 
       1.3577085754456367e-03, 
       1.0625687959321286e-03, 
       1.3447875739703355e-03, 
       1.1175452366220758e-03, 
       1.3211516130545097e-03, 
       1.1048034257136953e-03, 
       9.9423476377358136e-04, 
       1.1092430170577703e-03, 
       1.3250055223884415e-03, 
       1.1821189543462284e-03, 
       1.2202607426113781e-03, 
       1.3134405230703027e-03, 
       1.2982985229270032e-03, 
       1.2688784810029625e-03, 
       1.3019455977390916e-03, 
       1.0601239182870051e-03, 
       9.1680187664364371e-04, 
       1.2323539662452863e-03, 
       1.2832090815055569e-03, 
       1.0020648431733285e-03, 
       1.0722218072626629e-03, 
       9.7769795959281973e-04, 
       8.9936423896613100e-04, 
       9.2014616645642617e-04, 
       1.0602129138901862e-03, 
       1.0819653372863402e-03, 
       9.3634191499031836e-04, 
       1.2410215165679821e-03, 
       1.2399984290036193e-03, 
       1.0392305152043199e-03, 
       9.6969874816485880e-04, 
       9.1921728217785644e-04, 
       1.2272233412865226e-03, 
       1.2241823772437739e-03, 
       8.8811818233925440e-04, 
       9.1717303670487262e-04, 
       1.2029970299853347e-03, 
       1.1801869607692937e-03, 
       9.1314607487902766e-04, 
       9.2444165512776857e-04, 
       8.5920427200963055e-04, 
       8.7159747325664698e-04, 
       1.1917882632199806e-03, 
       9.6229157780246038e-04, 
       1.1477111910238483e-03, 
       1.1000000000000246e-03, 
       8.8230807141904586e-04, 
       1.0999999999997865e-03, 
       8.4562716509719638e-04, 
       8.6410583733641840e-04, 
       8.5829866982418425e-04, 
       9.8335366835765911e-04, 
       1.0258602675832567e-03, 
       8.5179158535410640e-04, 
       8.3986982354248001e-04, 
       9.4607544338334263e-04, 
       8.6566528239446919e-04, 
       8.4662067118532593e-04, 
       8.5316568121794308e-04, 
       1.1142237098212653e-03, 
       1.0881081634943864e-03, 
       1.0699961414163866e-03, 
       8.4729119898882424e-04, 
       1.1172161898206866e-03, 
       8.4348518982079671e-04, 
       8.6025065839148165e-04, 
       9.3649111108827694e-04, 
       9.3617426305743208e-04, 
       8.9701770478238247e-04, 
       1.1496347130973616e-03, 
       1.1500000000002513e-03, 
       1.1119848850061013e-03, 
       1.0992229172782368e-03, 
       9.3325783394147951e-04, 
       8.4192938444093465e-04, 
       1.0313229985191754e-03, 
       8.2884973030957789e-04, 
       8.3763246474184065e-04, 
       8.3763246474184065e-04, 
       9.6491045870014864e-04, 
       1.0321189515890520e-03, 
       1.0803740293975942e-03, 
       1.1124099740827261e-03, 
       8.3070031739222272e-04, 
       8.1966401955703536e-04, 
       8.9994174840728636e-04, 
       7.9371954109791281e-04, 
       8.8853370249132406e-04, 
       1.0659978474763878e-03, 
       1.0868974802450738e-03, 
       1.0966906158355978e-03, 
       8.8227759143595034e-04, 
       8.8144581068127093e-04, 
       7.8103048611061860e-04, 
       9.7010164581648592e-04, 
       1.0900003230755434e-03, 
       8.0304729628508965e-04, 
       8.9979417599935483e-04, 
       7.8262724349118862e-04, 
       1.0899999999997277e-03, 
       9.0170596842303983e-04, 
       8.2370871341711589e-04, 
       7.7092076805772823e-04, 
       8.4680452763058413e-04, 
       1.0418131549458239e-03, 
       8.6096820499749396e-04, 
       7.9928493053154238e-04, 
       9.1481837272593689e-04, 
       7.7332843519254009e-04, 
       8.8726698946568685e-04, 
       7.5719046098634863e-04, 
       8.7754050135777485e-04, 
       8.0193368655592518e-04, 
       1.0266668393735755e-03, 
       9.8632732892494799e-04, 
       9.7489709987612469e-04, 
       7.3379979969195798e-04, 
       7.9626124886624764e-04, 
       7.2862290150293193e-04, 
       8.1306119499879859e-04, 
       9.5021846561452715e-04, 
       1.0078398485438907e-03, 
       8.3309609297655242e-04, 
       9.5834562391903578e-04, 
       7.3007214635110573e-04, 
       7.0519655650750919e-04, 
       7.1384934407070207e-04, 
       9.9287919540578281e-04, 
       9.9169007084090512e-04, 
       8.3106697366481532e-04, 
       8.2282018600784458e-04, 
       9.7416286484639414e-04, 
       9.0999999999995577e-04, 
       9.6148998310662437e-04, 
       9.8041500425687404e-04, 
       9.6516270624856537e-04, 
       9.4035122255820252e-04, 
       9.7890469347745217e-04, 
       8.3192973548312104e-04, 
       9.4260168843323094e-04, 
       7.1437807437092760e-04, 
       7.1005488858935444e-04, 
       8.3675398153775361e-04, 
       7.4452602308348523e-04, 
       9.5999999999995297e-04, 
       9.5082972283379601e-04, 
       8.5799678760900414e-04, 
       9.5726088493600916e-04, 
       6.9742922299746506e-04, 
       9.2532078484820095e-04, 
       7.9345441572660231e-04, 
       9.4971014607161757e-04, 
       8.1159886064028069e-04, 
       9.5106189115032162e-04, 
       6.8143898598677869e-04, 
       9.0387209606499809e-04, 
       6.6464242232223091e-04, 
       7.4444840593396449e-04, 
       9.3078453845091963e-04, 
       6.8297381251342137e-04, 
       7.0536040326004551e-04, 
       8.1418525059597326e-04, 
       7.6629647553879137e-04, 
       6.8000896972691765e-04, 
       8.1305782721097263e-04, 
       7.0319267093032285e-04, 
       7.4835150677836490e-04, 
       7.1930848318228313e-04, 
       8.0034526842961344e-04, 
       6.6032960458127445e-04, 
       8.6726558610639333e-04, 
       6.9963057984986042e-04, 
       8.9471979726288362e-04, 
       7.3621238822671052e-04, 
       6.5109456331396260e-04, 
       9.0592824627149209e-04, 
       7.6210360955107886e-04, 
       6.8426355060787218e-04, 
       6.4699182335023096e-04, 
       6.5146688099540831e-04, 
       8.8786327978900112e-04, 
       7.9719117905839403e-04, 
       7.1101150176139806e-04, 
       6.4133572312437639e-04, 
       7.3841566196087270e-04, 
       6.4945020128667893e-04, 
       6.2782579577528739e-04, 
       6.3970131835345103e-04, 
       8.4176252244299947e-04, 
       6.3705554263248981e-04, 
       6.5286150446976189e-04, 
       8.4489184720920618e-04, 
       7.8062600162712690e-04, 
       8.1963905686600403e-04, 
       7.5136634889083869e-04, 
       7.4931236112842743e-04, 
       7.8940268946368344e-04, 
       6.1231694115196768e-04, 
       7.0958213374742215e-04, 
       7.8096702757666180e-04, 
       6.2987478580222775e-04, 
       8.5542456851813537e-04, 
       8.1547538167852773e-04, 
       6.6689473573856818e-04, 
       6.1550731857013994e-04, 
       6.2002415632235117e-04, 
       8.4807525814775914e-04, 
       7.9063518007592253e-04, 
       7.9636440318691824e-04, 
       8.5031666702191562e-04, 
       8.2869915565432300e-04, 
       6.8170403543912894e-04, 
       6.9631788551563153e-04, 
       6.2016289473829881e-04, 
       8.4000000000000665e-04, 
       8.3999999999998778e-04, 
       8.0292117711621422e-04, 
       5.8726803111830211e-04, 
       8.0353128786761843e-04, 
       7.5999999999998736e-04, 
       7.8413254753069927e-04, 
       6.4687160412571249e-04, 
       6.0597647539770974e-04, 
       7.8333655693941953e-04, 
       8.1825577207246211e-04, 
       5.9697450684105568e-04, 
       7.5749622246079986e-04, 
       5.8849423821111507e-04, 
       5.9756269924659405e-04, 
       7.8968921509867411e-04, 
       6.7411270063678886e-04, 
       5.7350505197181594e-04, 
       7.1869247980781864e-04, 
       5.9463187431608598e-04, 
       8.0301448588421622e-04, 
       5.9067863574418231e-04, 
       7.9958895513287779e-04, 
       5.7517880114199061e-04, 
       6.0860272298428949e-04, 
       6.0777498255043343e-04, 
       7.6061636240533706e-04, 
       5.6039561055042629e-04, 
       7.6485825086788114e-04, 
       7.9000000000004165e-04, 
       7.2825040323008783e-04, 
       5.7955568712952238e-04, 
       5.5779998065632859e-04, 
       6.7153444153707513e-04, 
       6.9477772938393795e-04, 
       5.6028827356726621e-04, 
       7.8121600020740207e-04, 
       7.5476246659545343e-04, 
       6.3853383528271143e-04, 
       5.7611286398155052e-04, 
       7.7999999999996085e-04, 
       5.7183426317650404e-04, 
       5.7318635216712733e-04, 
       6.4773453100489854e-04, 
       5.7066764260964157e-04, 
       6.4807423646908563e-04, 
       6.8591754070165356e-04, 
       5.7049608631716715e-04, 
       5.5889246672149236e-04, 
       5.4584649581698058e-04, 
       6.9974009997874969e-04, 
       6.5020042223484049e-04, 
       5.6405233343490783e-04, 
       5.5388267268236359e-04, 
       7.1248712770259875e-04, 
       6.1907071990519849e-04, 
       7.6000000000000015e-04, 
       7.5535647685746421e-04, 
       7.2355768561197344e-04, 
       5.4461785348019652e-04, 
       5.8348501595818379e-04, 
       6.1944140208039327e-04, 
       5.6602657899486601e-04, 
       7.4913743798783554e-04, 
       7.4999999999999394e-04, 
       7.4999999999999405e-04, 
       5.7766835175624212e-04, 
       7.3849396037308889e-04, 
       7.3000000000002078e-04, 
       6.2777617237240284e-04, 
       6.0455692124723390e-04, 
       6.0429726330500221e-04, 
       7.4367005224911865e-04, 
       6.2689788947778837e-04, 
       7.1518599181241910e-04, 
       5.5557069138651609e-04, 
       7.1227126133623574e-04, 
       6.5890293530411720e-04, 
       6.4161608249076933e-04, 
       5.8765383637214281e-04, 
       6.0555485606131039e-04, 
       5.1885242865576451e-04, 
       5.2914866184207367e-04, 
       5.7413744753413622e-04, 
       5.7377568222508491e-04, 
       5.1788343871384348e-04, 
       7.3095216607443143e-04, 
       7.2878611614416626e-04, 
       5.8923655852713278e-04, 
       5.3492812220260312e-04, 
       7.3000000000003531e-04, 
       6.7002172573886866e-04, 
       5.8882327818308739e-04, 
       5.1454012940534729e-04, 
       5.7784386895701374e-04, 
       5.1124765074919037e-04, 
       7.0750750422095136e-04, 
       7.0631186050202721e-04, 
       5.9863584523184093e-04, 
       7.1554468406279657e-04, 
       6.5999999999998146e-04, 
       5.2377091126056886e-04, 
       5.1198725383510917e-04, 
       7.1396186768038657e-04, 
       7.1343680159700374e-04, 
       5.1252070393127084e-04, 
       5.0446902630530538e-04, 
       5.6320744796654236e-04, 
       5.9798010233506644e-04, 
       4.9753264466863018e-04, 
       5.7762393013298017e-04, 
       5.7758461167172490e-04, 
       6.1597317619391245e-04, 
       5.1639429336711824e-04, 
       5.1639429336711824e-04, 
       6.9928221968222463e-04, 
       5.5803830287435576e-04, 
       4.9868676331598066e-04, 
       5.8237504517693705e-04, 
       5.7713007783281325e-04, 
       6.6485618626471125e-04, 
       6.8888674548972430e-04, 
       6.8704295240417605e-04, 
       5.6187501299076470e-04, 
       5.6116040505326448e-04, 
       6.9168151476712420e-04, 
       5.7483773256884589e-04, 
       4.9123135415785329e-04, 
       6.8999999999998045e-04, 
       6.0861889975609483e-04, 
       6.6601010436314798e-04, 
       5.0120526841662195e-04, 
       5.1192173189655349e-04, 
       4.9102548183979763e-04, 
       5.0471476540238217e-04, 
       4.8445200915290864e-04, 
       5.5002539033249245e-04, 
       5.4876511776007794e-04, 
       6.8000000409002036e-04, 
       5.8076654086288283e-04, 
       5.5639579232833380e-04, 
       5.5632873757778512e-04, 
       6.7605136518107992e-04, 
       5.7992589178201782e-04, 
       6.6548994786067614e-04, 
       6.6097188273867903e-04, 
       4.7897528438647268e-04, 
       4.7376289341636576e-04, 
       5.1683067703012979e-04, 
       5.0677247425818389e-04, 
       4.7256242075504126e-04, 
       5.8280935025750041e-04, 
       5.3598416080429261e-04, 
       4.8005546006144967e-04, 
       6.6000021336748481e-04, 
       5.4811967195184997e-04, 
       6.4755755460146544e-04, 
       5.4138073163812991e-04, 
       5.4663788512695577e-04, 
       6.5587104953685740e-04, 
       6.3131468875082407e-04, 
       4.6479039155858338e-04, 
       6.3277156072041462e-04, 
       6.4763937751077360e-04, 
       4.6456708958959148e-04, 
       5.4675348127092792e-04, 
       4.7370325236730887e-04, 
       5.6028827356739523e-04, 
       5.2218554397120336e-04, 
       4.9065836201968836e-04, 
       4.9669724742721310e-04, 
       4.5750680310662162e-04, 
       5.5261406438353669e-04, 
       4.7933561122234618e-04, 
       5.8606610428883613e-04, 
       4.5597973855156156e-04, 
       6.2940995487041867e-04, 
       6.4245591692016978e-04, 
       6.4236232249980057e-04, 
       6.1985075881894997e-04, 
       5.7228735507775560e-04, 
       6.3999999999999452e-04, 
       4.8630465915939619e-04, 
       4.5989956792110288e-04, 
       5.6926391790780714e-04, 
       6.0933454307930549e-04, 
       4.6662505685575895e-04, 
       5.1401343777502111e-04, 
       4.6031086945726597e-04, 
       6.2449176652015239e-04, 
       6.3000000000002637e-04, 
       4.7789901572731935e-04, 
       4.6706012290130577e-04, 
       5.1882404328284083e-04, 
       4.5617637121493594e-04, 
       6.0424568821553816e-04, 
       6.1999942927723570e-04, 
       5.1583008948361941e-04, 
       6.1566879345417388e-04, 
       5.9460486378112105e-04, 
       4.7179834447934246e-04, 
       4.4182186032471084e-04, 
       4.3640853697096227e-04, 
       6.1379862328916391e-04, 
       4.5565528191092702e-04, 
       4.5487145592387935e-04, 
       4.6492028120825217e-04, 
       6.0725188605008124e-04, 
       4.6598332022192228e-04, 
       5.8962396790333349e-04, 
       6.0583879341166171e-04, 
       5.6282583822521003e-04, 
       5.9694940412260912e-04, 
       5.3619493593579787e-04, 
       4.9963566885522870e-04, 
       4.4103855290032143e-04, 
       5.9899400302895159e-04, 
       5.9742024194650903e-04, 
       5.9765595813331943e-04, 
       5.6070491972700579e-04, 
       4.7094964690610954e-04, 
       4.2716419877300034e-04, 
       4.8991233588122161e-04, 
       4.4075059548504480e-04, 
       5.8990358144745455e-04, 
       5.9000769201299609e-04, 
       5.9000000000004296e-04, 
       5.8999999999998095e-04, 
       5.8750013155814754e-04, 
       4.2016663265200097e-04, 
       4.9777362164916408e-04, 
       4.3032297541430907e-04, 
       4.2952916163121948e-04, 
       5.1794020212985034e-04, 
       4.3579888208547052e-04, 
       5.6036838985983333e-04, 
       4.0987138414556027e-04, 
       5.7666811113706300e-04, 
       5.6658768265591922e-04, 
       4.8698028931076873e-04, 
       4.7252152845787777e-04, 
       4.7236002888165614e-04, 
       4.2728808526085247e-04, 
       5.7265370879162425e-04, 
       5.5091321729471216e-04, 
       4.5926026086021219e-04, 
       5.7000000000000193e-04, 
       4.0867473223185158e-04, 
       4.1533075832266724e-04, 
       4.1689211916132987e-04, 
       4.0242990893899698e-04, 
       4.8828480585771917e-04, 
       4.2105596823818205e-04, 
       4.2168513633840789e-04, 
       5.2644630726734760e-04, 
       4.1769971011662414e-04, 
       5.4641938589369008e-04, 
       5.6009993013137404e-04, 
       4.3224942094529642e-04, 
       5.5999999999996515e-04, 
       4.1634491693792969e-04, 
       4.2999999999997641e-04, 
       5.3672971698170513e-04, 
       5.5338564409599040e-04, 
       5.0138912481284689e-04, 
       4.2527198322579978e-04, 
       5.5174014649771890e-04, 
       4.2160288368167480e-04, 
       4.1164869182218697e-04, 
       4.0515553234701465e-04, 
       4.1388888787201891e-04, 
       4.5879653316744994e-04, 
       4.1481905706604300e-04, 
       5.2226707695044578e-04, 
       5.3765684742377846e-04, 
       4.6251441454208276e-04, 
       4.0229676852138288e-04, 
       4.5132452315943972e-04, 
       4.9788235936933740e-04, 
       4.8404217926740605e-04, 
       4.3903711165634252e-04, 
       5.3107191607676125e-04, 
       5.3107191607676125e-04, 
       3.8886655132896715e-04, 
       4.4238738735252005e-04, 
       3.8785809892244742e-04, 
       4.5550219880348197e-04, 
       4.0799149657821883e-04, 
       5.1336441495056738e-04, 
       5.1946551137508213e-04, 
       4.0873861456086315e-04, 
       4.6825200439304545e-04, 
       4.5441886082673091e-04, 
       4.5433428521862327e-04, 
       4.0593087721937189e-04, 
       3.6937335423199740e-04, 
       4.3334119392976397e-04, 
       3.6855492601527227e-04, 
       3.9069878356694503e-04, 
       5.1310381798813501e-04, 
       4.8153322718350451e-04, 
       3.8523209819074509e-04, 
       4.9040818017261696e-04, 
       4.6989012326654486e-04, 
       4.0658815994708161e-04, 
       4.2850258282290993e-04, 
       3.8326422332357320e-04, 
       5.1165532873487848e-04, 
       4.3178371561377243e-04, 
       4.2354719479179568e-04, 
       5.1043313063264633e-04, 
       3.9072991288166282e-04, 
       5.0991904739426934e-04, 
       4.9443737384068695e-04, 
       3.8741334224277321e-04, 
       4.8224672578947938e-04, 
       4.2254070924172227e-04, 
       3.6652951263539706e-04, 
       4.1869461908878561e-04, 
       4.2010333030759959e-04, 
       3.7842677984806282e-04, 
       4.2793159450751759e-04, 
       3.7384841866869149e-04, 
       5.0087295555180009e-04, 
       4.1506283130912446e-04, 
       4.3339550700365780e-04, 
       4.2828450027069620e-04, 
       4.1566856155783350e-04, 
       3.6569451786849755e-04, 
       4.1245706510823686e-04, 
       4.8365668460789170e-04, 
       4.8763961348729888e-04, 
       4.1943463645608575e-04, 
       3.6734593777867946e-04, 
       3.7262731260247862e-04, 
       3.7262731260247862e-04, 
       3.6874899091149615e-04, 
       4.3646020973984594e-04, 
       4.3588229819350929e-04, 
       3.6303598553427474e-04, 
       4.4811267843526773e-04, 
       4.9886767777191103e-04, 
       4.5999999999997101e-04, 
       4.9601343917257288e-04, 
       3.5139155083695493e-04, 
       4.0603320152325677e-04, 
       4.9455446785785751e-04, 
       3.7996153360287277e-04, 
       4.7663625051561504e-04, 
       3.5001949176853336e-04, 
       4.3069372788813801e-04, 
       4.4664562925356523e-04, 
       3.5283090806247167e-04, 
       4.8999976265301872e-04, 
       4.0761576635064929e-04, 
       4.8935244512509615e-04, 
       3.7849444071535164e-04, 
       4.8469669829598327e-04, 
       4.8999999999999998e-04, 
       4.8684562236870086e-04, 
       3.7113797634645313e-04, 
       3.6038581826182924e-04, 
       4.8827147481653817e-04, 
       4.0030607450085932e-04, 
       4.8772495020563014e-04, 
       4.8197160844513204e-04, 
       4.6978819288669892e-04, 
       4.2446731897975725e-04, 
       4.8352294217218273e-04, 
       4.6515044934806131e-04, 
       3.8931902425185210e-04, 
       3.5981522816320262e-04, 
       4.4461820065308690e-04, 
       3.8995976754890020e-04, 
       3.4814296503105877e-04, 
       3.4122566007805717e-04, 
       3.9826746735479240e-04, 
       4.1521056116211217e-04, 
       4.5666186737046194e-04, 
       4.5666186737046194e-04, 
       3.6246622320238104e-04, 
       3.4200668566398024e-04, 
       3.8293866857847908e-04, 
       4.6978274894565291e-04, 
       3.9476533793299766e-04, 
       3.8177242545776337e-04, 
       3.8249113504133813e-04, 
       4.3713346183631185e-04, 
       3.9325081961528937e-04, 
       4.1184358201791054e-04, 
       3.6999999999996666e-04, 
       4.7193868547978684e-04, 
       4.0051027361552951e-04, 
       4.1063003272274410e-04, 
       4.6952459630720628e-04, 
       4.7000000000000503e-04, 
       4.6999999999997510e-04, 
       4.6901904337739482e-04, 
       4.6428090886067663e-04, 
       4.5535334122565518e-04, 
       4.2577556802660078e-04, 
       3.4158622217863807e-04, 
       4.0190618443182463e-04, 
       3.3165270024157088e-04, 
       3.8138586772530674e-04, 
       4.1381515453920391e-04, 
       4.6426092928233037e-04, 
       3.6957750115449621e-04, 
       3.3061034570348673e-04, 
       3.4761384384529337e-04, 
       3.3156198088253819e-04, 
       4.1136417470181438e-04, 
       3.3801544789340129e-04, 
       3.8277054437841702e-04, 
       3.4329678344799250e-04, 
       4.5999999735529546e-04, 
       4.3048145482445504e-04, 
       3.2524149394451754e-04, 
       4.5473571808533008e-04, 
       4.5930848782244120e-04, 
       3.4559459210361539e-04, 
       3.3349557060676611e-04, 
       4.1566117929988164e-04, 
       3.7626153102920484e-04, 
       4.5569039618893211e-04, 
       3.2532986604872212e-04, 
       3.3406635752527723e-04, 
       3.7710413964717920e-04, 
       4.5069157397737884e-04, 
       3.6444066719910299e-04, 
       4.4999999999997348e-04, 
       4.2984431033180087e-04, 
       4.2773493880725875e-04, 
       3.3081030536378608e-04, 
       4.4647132167755454e-04, 
       3.2356918158221401e-04, 
       3.7022810554191305e-04, 
       3.2973924690423264e-04, 
       3.5659008730153883e-04, 
       3.9270692314014261e-04, 
       3.2938480582062068e-04, 
       3.6885752698371691e-04, 
       3.1367207213070221e-04, 
       3.2688136719308795e-04, 
       3.2252418742579735e-04, 
       3.1294648920414918e-04, 
       3.2309537020842850e-04, 
       3.1670261691948536e-04, 
       4.4000000000000381e-04, 
       4.3999999999998055e-04, 
       4.3907942601544806e-04, 
       4.2594248213401295e-04, 
       4.3654299510511332e-04, 
       3.6308844713053639e-04, 
       4.2416185430309018e-04, 
       3.0947662162855965e-04, 
       3.0950735195422720e-04, 
       3.1826480590831986e-04, 
       3.1838418274988045e-04, 
       3.6218176349486850e-04, 
       4.1117671007293559e-04, 
       3.0736467096239737e-04, 
       3.6682246917940430e-04, 
       3.2202045371549487e-04, 
       3.5922581566760845e-04, 
       3.5916264391864687e-04, 
       3.0606194614235619e-04, 
       4.2684327659405375e-04, 
       4.2822681050420756e-04, 
       3.5622833920347422e-04, 
       3.5680292094087551e-04, 
       3.7929041075784799e-04, 
       3.0372629093901500e-04, 
       4.0434069622467610e-04, 
       3.5779185101252070e-04, 
       3.4460902560420090e-04, 
       3.5307268890912507e-04, 
       3.2170637134458334e-04, 
       4.1326378551179702e-04, 
       3.0002948586576667e-04, 
       3.4001718709206555e-04, 
       3.8467289296057703e-04, 
       4.2042023952030600e-04, 
       3.4844840596656495e-04, 
       4.2000000000000376e-04, 
       4.1999999999997958e-04, 
       4.1864749408116895e-04, 
       3.4811448790028165e-04, 
       4.1766614137503578e-04, 
       3.0489530857289988e-04, 
       3.4309756172639050e-04, 
       4.0281912617011722e-04, 
       3.4239416185192210e-04, 
       3.4220735527200075e-04, 
       3.4640714418511818e-04, 
       3.0459326080784350e-04, 
       3.8974634285763763e-04, 
       3.4634319522724956e-04, 
       3.6491339987555213e-04, 
       4.1453926694554489e-04, 
       3.4331422600119367e-04, 
       3.9282497621314792e-04, 
       3.4186799779284795e-04, 
       4.1210285388468227e-04, 
       4.0908671516770573e-04, 
       3.0731549100465209e-04, 
       3.5469041310171040e-04, 
       4.0887919647896955e-04, 
       3.0385270701003718e-04, 
       3.8511402544468920e-04, 
       3.3839707440502082e-04, 
       2.9820659145179040e-04, 
       2.9867487234606612e-04, 
       3.0342171916419966e-04, 
       3.3370733704627531e-04, 
       3.6173396560894126e-04, 
       2.9830504098508735e-04, 
       4.0428877742219096e-04, 
       4.0330102795794299e-04, 
       3.4651517308395715e-04, 
       3.5960853835157900e-04, 
       3.2231102196190981e-04, 
       3.9709906075794413e-04, 
       3.5523311514918976e-04, 
       3.3513011328181475e-04, 
       3.7000000000002282e-04, 
       2.9259497846616727e-04, 
       2.9239459626627650e-04, 
       2.9895903899708026e-04, 
       3.9999999999995898e-04, 
       3.8078454708052282e-04, 
       2.8690939739396774e-04, 
       3.7712873187877743e-04, 
       3.1000000000004407e-04, 
       3.3105642649603411e-04, 
       2.8847303114726096e-04, 
       3.8292613410330643e-04, 
       3.2114564201895379e-04, 
       2.8304054321787848e-04, 
       3.8079938340616383e-04, 
       3.9556733408690028e-04, 
       3.9579459262805984e-04, 
       3.9401531826353423e-04, 
       2.8097462496759604e-04, 
       3.3454201279634819e-04, 
       3.3454201279634819e-04, 
       2.8636537383450530e-04, 
       2.8614038649050290e-04, 
       3.8927861915739583e-04, 
       2.9583121267734539e-04, 
       3.4207210171564964e-04, 
       3.2647467760697687e-04, 
       3.7717347844795824e-04, 
       2.8454452119998841e-04, 
       3.9198930105694026e-04, 
       2.7733118401955585e-04, 
       3.6255297270708115e-04, 
       3.2473542908114755e-04, 
       2.9135032342685611e-04, 
       3.8934249328830478e-04, 
       3.4058762887578928e-04, 
       2.7519732507388510e-04, 
       3.3853975374277317e-04, 
       3.4001363110565104e-04, 
       2.7663783533663676e-04, 
       3.4893649010062636e-04, 
       2.7761558501658752e-04, 
       3.4314164553020778e-04, 
       2.8463270593471812e-04, 
       3.1831153949116257e-04, 
       3.7064997439797807e-04, 
       3.8451057598346596e-04, 
       2.7354580058698785e-04, 
       3.7786337315421026e-04, 
       2.7581415183103091e-04, 
       3.5829630510651988e-04, 
       2.8775167995718321e-04, 
       2.7229498488724198e-04, 
       2.9280328532242341e-04, 
       3.8047913711606332e-04, 
       3.4999999999999626e-04, 
       3.7063631999494209e-04, 
       3.1496831514292591e-04, 
       3.3704411430660844e-04, 
       2.7511902160417938e-04, 
       3.1359975061424710e-04, 
       3.7290465536728395e-04, 
       3.7321382662523089e-04, 
       2.9227515573780020e-04, 
       3.7106323452044612e-04, 
       3.7555578329939412e-04, 
       2.8010517725244368e-04, 
       2.9424174037803459e-04, 
       3.2528964040418149e-04, 
       2.9419727736172549e-04, 
       3.6344135447651194e-04, 
       3.4979692310113463e-04, 
       3.1887808700253883e-04, 
       2.9804644876324770e-04, 
       3.5977332187777083e-04, 
       2.6866476615321352e-04, 
       2.7540586172352814e-04, 
       2.9466889504489182e-04, 
       3.1243323066047849e-04, 
       2.6585713707362566e-04, 
       3.2481216266130690e-04, 
       3.2885386870422730e-04, 
       3.6040920857010716e-04, 
       3.6624685236108826e-04, 
       3.5369938359066652e-04, 
       2.7730746375271659e-04, 
       3.1927194131035040e-04, 
       2.9695462787589515e-04, 
       2.6348264903191133e-04, 
       2.7200836951598662e-04, 
       3.5724989829834941e-04, 
       2.6363250823933189e-04, 
       2.7912684171211107e-04, 
       2.5967531295331318e-04, 
       3.6262410500142924e-04, 
       3.5865527641514705e-04, 
       2.6928662850448737e-04, 
       2.6948265789733956e-04, 
       3.1254680142837383e-04, 
       3.0018247406708417e-04, 
       3.6073891588606853e-04, 
       3.5127059277291126e-04, 
       2.9946441383487939e-04, 
       2.9497538747445405e-04, 
       2.6858477443360111e-04, 
       3.6000000000000908e-04, 
       2.9755950290098114e-04, 
       2.6189986106057338e-04, 
       3.0453449304052445e-04, 
       2.5320310620209992e-04, 
       2.8781457522357825e-04, 
       2.9707648588416354e-04, 
       3.3749802525536560e-04, 
       3.3225214191250600e-04, 
       3.5604005055667377e-04, 
       3.5372906690141930e-04, 
       3.3913692667461278e-04, 
       3.5243092878674791e-04, 
       2.9272006353230735e-04, 
       3.5099944546393186e-04, 
       3.3426161893637907e-04, 
       2.6913762139848813e-04, 
       2.7973975061277987e-04, 
       3.4710503068018003e-04, 
       2.4699301629553972e-04, 
       2.4719485643679009e-04, 
       2.5282255451128361e-04, 
       3.3807012415164577e-04, 
       2.5929335405473595e-04, 
       3.3509526863541745e-04, 
       2.4782803868245238e-04, 
       2.5108093918816158e-04, 
       3.0901882212811270e-04, 
       2.5568366756854752e-04, 
       2.9640659487743240e-04, 
       3.4100128466179474e-04, 
       3.0985435917605633e-04, 
       2.7633004621094043e-04, 
       3.1226978544487299e-04, 
       2.8427540403496115e-04, 
       2.6779733037916664e-04, 
       2.5535088485590095e-04, 
       2.9089408655607295e-04, 
       2.5283240419304365e-04, 
       2.5696816267963837e-04, 
       2.8192317912486810e-04, 
       2.4773479211039772e-04, 
       2.6539622011183847e-04, 
       2.5424749836417760e-04, 
       2.8305831958776374e-04, 
       2.7985834617378654e-04, 
       3.3999999999997677e-04, 
       3.3963085401933089e-04, 
       3.3849525474905038e-04, 
       2.3977674056949386e-04, 
       2.8213381136175058e-04, 
       2.4045005515574227e-04, 
       3.3250885629433771e-04, 
       2.5223041540520610e-04, 
       2.4968916515183398e-04, 
       3.1336824593203571e-04, 
       2.6013923924945038e-04, 
       2.4783537528821507e-04, 
       2.5096483522151115e-04, 
       3.2829473124502707e-04, 
       2.7939835411289925e-04, 
       2.7914201765398761e-04, 
       2.5257057486488321e-04, 
       2.5306669735605981e-04, 
       2.7791608691046625e-04, 
       3.2697848120397609e-04, 
       3.2616855813316647e-04, 
       3.2118023809172191e-04, 
       2.4801439023399791e-04, 
       2.5259898189532680e-04, 
       3.3155444988579292e-04, 
       3.0793877799634872e-04, 
       2.9806732489713156e-04, 
       2.8004474074230535e-04, 
       3.2472797944905688e-04, 
       2.3393727992128394e-04, 
       2.5619369910748272e-04, 
       3.2856594952090592e-04, 
       2.7837324545852070e-04, 
       2.3567042563236025e-04, 
       3.2705122134773124e-04, 
       2.7138752800579381e-04, 
       3.2728425687190022e-04, 
       2.4311807194284896e-04, 
       2.4272938538496468e-04, 
       2.3579935773756629e-04, 
       2.7904710628036481e-04, 
       2.7877768578102655e-04, 
       2.3586852469842040e-04, 
       2.4520301594953904e-04, 
       2.2905429566589364e-04, 
       2.6810160108816533e-04, 
       2.6789511472948815e-04, 
       2.4115458529217778e-04, 
       2.3324309697577038e-04, 
       2.9954418391687484e-04, 
       2.8605858470754183e-04, 
       3.2174039888201920e-04, 
       2.3719227166811747e-04, 
       2.6609533664127518e-04, 
       2.2767667529402782e-04, 
       3.1791696857265108e-04, 
       3.0327868682943770e-04, 
       2.8113853310792104e-04, 
       3.2000000000001770e-04, 
       3.2000000000004442e-04, 
       2.3430614085050237e-04, 
       3.1671680845461752e-04, 
       3.0392419932039641e-04, 
       3.1615547541455059e-04, 
       2.8658651451206151e-04, 
       2.6463592934351489e-04, 
       3.0822068437102014e-04, 
       2.2556274162091985e-04, 
       2.2840189398557067e-04, 
       2.3719584204666294e-04, 
       3.1216208620905725e-04, 
       2.5449890807285040e-04, 
       2.5436344588189902e-04, 
       2.3538735028597537e-04, 
       3.1147282775995089e-04, 
       2.5969341461671652e-04, 
       3.1317461948619161e-04, 
       2.6340862216843078e-04, 
       2.3393406641701825e-04, 
       2.6439172803325927e-04, 
       2.5233741110161395e-04, 
       2.4231171600978458e-04, 
       3.1482955754108863e-04, 
       2.5522768005128418e-04, 
       2.5521588245201190e-04, 
       3.1381688584376116e-04, 
       2.5648729188693093e-04, 
       3.0268426695426762e-04, 
       3.1280353282196467e-04, 
       3.1202382480161988e-04, 
       2.3317163880900898e-04, 
       3.1218266534576024e-04, 
       2.9103326145474897e-04, 
       3.1176540652953229e-04, 
       2.2989221748258721e-04, 
       2.3688476104629736e-04, 
       3.0915704969407265e-04, 
       2.5773419781500539e-04, 
       2.3540475442860749e-04, 
       2.2821530714031650e-04, 
       2.2435797862340739e-04, 
       3.0999999908870525e-04, 
       3.0999999999995533e-04, 
       2.9744160928408088e-04, 
       2.5268772103077776e-04, 
       2.5204317672044381e-04, 
       2.5620717315975026e-04, 
       2.2149901677291959e-04, 
       2.2708527329067755e-04, 
       2.2262192236674442e-04, 
       2.9797646808577178e-04, 
       2.1757222439219346e-04, 
       3.0566583290436217e-04, 
       2.1929546850625516e-04, 
       2.1992350851614080e-04, 
       2.2475263530723574e-04, 
       3.0205953360630490e-04, 
       2.2977239204617595e-04, 
       2.3049290797896250e-04, 
       2.1500091076442467e-04, 
       2.2524368222211189e-04, 
       2.2501900577335461e-04, 
       2.9133652360006354e-04, 
       2.5024423343436022e-04, 
       2.5009461831495931e-04, 
       2.2601035982896625e-04, 
       2.9999999999996853e-04, 
       2.2322428892762461e-04, 
       2.7793638310670186e-04, 
       2.9866512115810974e-04, 
       2.2323497157643709e-04, 
       2.4975541771191106e-04, 
       2.1197769128322312e-04, 
       2.5453294413861448e-04, 
       2.8427321643510907e-04, 
       2.4190221773151853e-04, 
       2.1819894203704577e-04, 
       2.6099551698108655e-04, 
       2.8289151125479719e-04, 
       2.1979857578763252e-04, 
       2.9447306383953919e-04, 
       2.1109721135205640e-04, 
       2.9430324605776964e-04, 
       2.6889400409187689e-04, 
       2.1623334791344513e-04, 
       2.7745577353132820e-04, 
       2.0803433172293591e-04, 
       2.2202143031365020e-04, 
       2.6427417648126059e-04, 
       2.4313642858022461e-04, 
       2.8154748879054162e-04, 
       2.6022102441427370e-04, 
       2.4263446590600505e-04, 
       2.8143328749111499e-04, 
       2.4052465845005117e-04, 
       2.2790752633461154e-04, 
       2.8999999999996585e-04, 
       2.9000000000004023e-04, 
       2.1136425109980801e-04, 
       2.8792400406770977e-04, 
       2.5784407148522168e-04, 
       2.0599743197571911e-04, 
       2.6823189698646277e-04, 
       2.3864624382814809e-04, 
       2.0654354463586750e-04, 
       2.0650824641764735e-04, 
       2.3807126317323553e-04, 
       2.2249936953832900e-04, 
       2.0486768115336303e-04, 
       2.8551174868051918e-04, 
       2.7045841141958558e-04, 
       2.4282088432341491e-04, 
       2.0188497525244395e-04, 
       2.3629356788566148e-04, 
       2.2342977990820112e-04, 
       2.3502030290252078e-04, 
       2.3600733707685161e-04, 
       2.0076078679283655e-04, 
       2.5770553059412474e-04, 
       2.8268685380193723e-04, 
       2.1943505033605501e-04, 
       2.5630450071446604e-04, 
       2.3412907149410224e-04, 
       2.4116841449521123e-04, 
       2.0734485834611233e-04, 
       2.0937758302858094e-04, 
       2.7628774519960564e-04, 
       2.6374089093327333e-04, 
       2.1948984907636736e-04, 
       2.2487793751790322e-04, 
       2.0913516848770290e-04, 
       2.0187775868518284e-04, 
       2.8000000000003868e-04, 
       2.6225653176257770e-04, 
       2.6509698256519672e-04, 
       2.3844322890897673e-04, 
       1.9665986766056101e-04, 
       2.0111732430850836e-04, 
       2.6338312827091582e-04, 
       2.6068781202255765e-04, 
       2.0705591105201691e-04, 
       2.5845458073120746e-04, 
       1.9837677368377229e-04, 
       1.9604612271938266e-04, 
       2.7611247564881912e-04, 
       2.0178811723063668e-04, 
       1.9496006978690384e-04, 
       2.6621632966843497e-04, 
       2.3992448146326796e-04, 
       2.6564838803582618e-04, 
       2.1453072026988854e-04, 
       2.0242795087595258e-04, 
       2.3347231881524477e-04, 
       2.6687870558351698e-04, 
       2.6323077289597366e-04, 
       2.0408123499937787e-04, 
       1.9651225116706819e-04, 
       1.9341462991786140e-04, 
       2.1439355610803512e-04, 
       1.9247980619437155e-04, 
       2.3144883337296209e-04, 
       2.0026319710512395e-04, 
       2.4999999999999903e-04, 
       2.5000000000003231e-04, 
       2.0259484227500721e-04, 
       2.6231438262293907e-04, 
       1.9354497264774596e-04, 
       1.9345920373048503e-04, 
       2.7000000000004109e-04, 
       2.2430020695866915e-04, 
       2.7000000000000412e-04, 
       2.2549765354080070e-04, 
       1.9164253959316806e-04, 
       2.2509494155806910e-04, 
       2.2525026686727825e-04, 
       2.2318511283523620e-04, 
       2.3349247328095880e-04, 
       2.0434082807836406e-04, 
       2.0834132992564555e-04, 
       1.9294170107195499e-04, 
       1.8967587950751355e-04, 
       2.5745810029448074e-04, 
       2.0685094522593633e-04, 
       1.8988320397823010e-04, 
       1.8986898960552298e-04, 
       2.5535056960332815e-04, 
       2.0267129345234516e-04, 
       1.9956288665435413e-04, 
       2.4036168123659263e-04, 
       1.8996084203244602e-04, 
       2.6257343072142104e-04, 
       2.1977086040144420e-04, 
       2.1916480413819022e-04, 
       2.2844154174104164e-04, 
       1.8791228730514038e-04, 
       2.6347830992844233e-04, 
       2.1968083976387103e-04, 
       1.9273682796051440e-04, 
       2.6111258340710239e-04, 
       2.2608965841521824e-04, 
       1.9751690946885058e-04, 
       2.5412264350696491e-04, 
       2.1399416926409822e-04, 
       2.5308071120466388e-04, 
       2.1368925103375670e-04, 
       2.0233968547629742e-04, 
       2.1783334265975033e-04, 
       2.1484436905901236e-04, 
       2.0454533427018013e-04, 
       2.4879992141209003e-04, 
       1.9222968870637766e-04, 
       1.9118386806846177e-04, 
       2.0542411524102660e-04, 
       2.1603266564374946e-04, 
       2.3890753774537379e-04, 
       1.8429851713084840e-04, 
       2.3515367181629464e-04, 
       1.9245860287752804e-04, 
       2.5999999999998838e-04, 
       2.6000000000002724e-04, 
       2.5999999999999998e-04, 
       2.1339403724271124e-04, 
       1.9385983907538516e-04, 
       2.0567975577988150e-04, 
       1.9378579460987993e-04, 
       1.9370530809177691e-04, 
       2.4004264701372042e-04, 
       1.8401429832986659e-04, 
       2.5706041756531743e-04, 
       1.9205336067977150e-04, 
       1.8641168799872219e-04, 
       1.9871596974680873e-04, 
       2.3346162171777741e-04, 
       2.5549597071476400e-04, 
       2.4506630116177223e-04, 
       2.5497502263371018e-04, 
       2.0019183798654542e-04, 
       2.1863970087327935e-04, 
       2.4215656804310847e-04, 
       1.9947881840077890e-04, 
       1.8122610024640332e-04, 
       2.1148664323111877e-04, 
       1.8183902339949064e-04, 
       1.7955063605081613e-04, 
       2.1964981884957840e-04, 
       2.0649505172753237e-04, 
       2.4751605075783932e-04, 
       2.5194141162151448e-04, 
       2.0943619884108503e-04, 
       2.4713645351721371e-04, 
       2.2520760913522170e-04, 
       1.9834096329994437e-04, 
       1.7751719847499997e-04, 
       2.0840194989387871e-04, 
       1.8654202289697302e-04, 
       1.8062638012133694e-04, 
       1.8256946700069264e-04, 
       2.4999999849476456e-04, 
       1.7900677423558474e-04, 
       2.4999999999997398e-04, 
       2.5000000000000001e-04, 
       2.1322514357739974e-04, 
       1.9668870019524937e-04, 
       1.7968536908475615e-04, 
       2.4308609999510795e-04, 
       2.4474445627074111e-04, 
       2.2462253295264285e-04, 
       1.7999999999998418e-04, 
       1.9674066213209079e-04, 
       2.2106027174145405e-04, 
       2.0763941342180273e-04, 
       2.0632523958685788e-04, 
       1.7932639278016573e-04, 
       1.8610790686288738e-04, 
       2.0567405580623879e-04, 
       2.4745557041549896e-04, 
       2.4378876326027171e-04, 
       1.8002786593308484e-04, 
       2.0210148182786298e-04, 
       1.7672629716127206e-04, 
       1.7632235701721456e-04, 
       1.8503001149529135e-04, 
       2.4348532202869423e-04, 
       2.4524914550459476e-04, 
       2.4466121487279303e-04, 
       1.7418089125860803e-04, 
       2.4494067754826350e-04, 
       1.8411289069474883e-04, 
       2.4382441495580942e-04, 
       2.4239839706128764e-04, 
       2.3595555269380691e-04, 
       2.2615210627401538e-04, 
       2.0989451395970300e-04, 
       2.0989451395970300e-04, 
       2.0099270166529039e-04, 
       1.9638073835901758e-04, 
       1.9634152022268928e-04, 
       1.7560861284123544e-04, 
       1.8922351013133323e-04, 
       1.7516930455856077e-04, 
       2.1804833065561008e-04, 
       2.1781961728480382e-04, 
       1.8914532283297668e-04, 
       2.4062256760033399e-04, 
       2.3999999938011359e-04, 
       2.4000000000003950e-04, 
       1.7773774901065794e-04, 
       2.2829547390698215e-04, 
       2.3370156236937273e-04, 
       1.9882069584392146e-04, 
       2.0654639365039665e-04, 
       2.2029074928855608e-04, 
       2.3783418033836821e-04, 
       2.2000000000002611e-04, 
       2.1880329653845720e-04, 
       1.9830188000751601e-04, 
       1.8759849489001984e-04, 
       1.8061492288644478e-04, 
       1.6878256380459418e-04, 
       1.7730656474631621e-04, 
       2.0387918463937786e-04, 
       1.7887695003655505e-04, 
       2.3263123250724243e-04, 
       2.3027501567002260e-04, 
       2.2541082404631396e-04, 
       2.3683182833298509e-04, 
       2.2723577686318660e-04, 
       2.3652420110730206e-04, 
       1.9729299459400409e-04, 
       1.6743033833002279e-04, 
       2.0066346598707449e-04, 
       1.7207766987645189e-04, 
       1.6706117561708883e-04, 
       1.9552665050008418e-04, 
       2.2709119239798802e-04, 
       2.2476973040367378e-04, 
       2.2451420751005340e-04, 
       1.9556443108675100e-04, 
       2.1509982510861100e-04, 
       1.8958718076528264e-04, 
       1.9460526412567515e-04, 
       2.3299875583095806e-04, 
       1.6719105060434582e-04, 
       1.6556682272633315e-04, 
       2.3247342785214517e-04, 
       2.2000000000003104e-04, 
       2.2406518169228550e-04, 
       2.2920242042894314e-04, 
       1.8653197006921946e-04, 
       1.8639809982404844e-04, 
       2.2478186245675758e-04, 
       1.9843842809095339e-04, 
       2.2548931449566900e-04, 
       1.8853905584557880e-04, 
       1.6203269795081330e-04, 
       1.6264942681795956e-04, 
       1.6664962869265858e-04, 
       2.0999999999995477e-04, 
       1.6609262223060798e-04, 
       2.2306508566916029e-04, 
       2.1198933398516918e-04, 
       2.0209149221359651e-04, 
       2.0267190103407011e-04, 
       2.2410662266353771e-04, 
       1.8475573229451833e-04, 
       2.0568120441227043e-04, 
       2.1668748372709662e-04, 
       1.8206579700248956e-04, 
       2.2423300995854555e-04, 
       1.8173933140342254e-04, 
       1.8170965532504499e-04, 
       1.8713620395521889e-04, 
       1.6824834436211903e-04, 
       2.2448320945265150e-04, 
       2.1765007028663841e-04, 
       2.2307187812426087e-04, 
       2.1093991019072498e-04, 
       1.6025138311865316e-04, 
       2.2287412117075222e-04, 
       2.2094061455424846e-04, 
       1.6594820210389250e-04, 
       1.6644064948736367e-04, 
       2.2269581292867462e-04, 
       2.2125652072816520e-04, 
       2.1992016418302367e-04, 
       1.6387606382490191e-04, 
       1.6387606382490191e-04, 
       2.2085647907655440e-04, 
       1.7326738331311869e-04, 
       1.5714929915227561e-04, 
       2.1910538948336330e-04, 
       1.5721279031039717e-04, 
       1.6312033268606581e-04, 
       1.5941622696788346e-04, 
       1.8303249044752322e-04, 
       1.8260384170897472e-04, 
       1.8101569030928630e-04, 
       1.7901421414621421e-04, 
       1.7896416889367592e-04, 
       2.1999999999997214e-04, 
       2.1999999999981873e-04, 
       2.1992052197674223e-04, 
       1.9675514272150436e-04, 
       1.8005840266708869e-04, 
       1.6451930988121432e-04, 
       1.6605556438789405e-04, 
       1.5894376709162786e-04, 
       1.5802178955935829e-04, 
       1.8198896526810433e-04, 
       2.1016143332279935e-04, 
       2.0726322085303121e-04, 
       1.7885026518711547e-04, 
       1.9204042605274754e-04, 
       2.1837455490796189e-04, 
       1.8104066755906517e-04, 
       2.1804715489596541e-04, 
       2.0531624510420490e-04, 
       1.5475920596201236e-04, 
       1.8016643583571024e-04, 
       1.8699862737107888e-04, 
       1.9578263509659665e-04, 
       1.8305500784621524e-04, 
       1.6249764582446324e-04, 
       2.0929667091805338e-04, 
       2.1582870340570160e-04, 
       1.6605551212964048e-04, 
       2.0716954084167583e-04, 
       2.1507069086757128e-04, 
       1.8475771193749287e-04, 
       1.7831875447095699e-04, 
       1.5849022773778927e-04, 
       1.7767010462171225e-04, 
       1.9030256055376209e-04, 
       1.5464559081375723e-04, 
       1.7647647198693029e-04, 
       2.0408406983552979e-04, 
       1.9205906405437353e-04, 
       1.7634862576236844e-04, 
       1.5498527962314593e-04, 
       1.5494446850651414e-04, 
       2.0604727917679542e-04, 
       1.9777089989585393e-04, 
       2.0977876886592097e-04, 
       2.0470882574515059e-04, 
       1.8476957104605053e-04, 
       1.5451341198577181e-04, 
       1.7873275764015799e-04, 
       1.5223656451413296e-04, 
       1.9512056310265060e-04, 
       1.7858548915195452e-04, 
       1.8010282269353056e-04, 
       2.1105580993237960e-04, 
       2.1028226997063743e-04, 
       1.5788474322783725e-04, 
       1.7052669296654981e-04, 
       2.0884475469785347e-04, 
       1.8339236624213706e-04, 
       1.4882153936831809e-04, 
       1.7028186227888265e-04, 
       1.7434512776768699e-04, 
       1.8092171327827123e-04, 
       1.5622488767547135e-04, 
       2.0859338132221138e-04, 
       2.0881609779865713e-04, 
       2.0025656943117707e-04, 
       2.0895988641599414e-04, 
       1.7970859669072208e-04, 
       1.8774959425509104e-04, 
       1.6484850097486862e-04, 
       1.5322829384781218e-04, 
       1.5297396408521079e-04, 
       1.7097089104609490e-04, 
       1.5249135802333427e-04, 
       2.0680741789482447e-04, 
       1.5591361198113529e-04, 
       1.4618126955077889e-04, 
       1.9000000000000259e-04, 
       1.9000000000000259e-04, 
       1.4839016971000663e-04, 
       1.4976375274512033e-04, 
       1.9192261913811949e-04, 
       1.7024614522762827e-04, 
       2.0347297702718616e-04, 
       1.7130009479180536e-04, 
       1.7019459092138404e-04, 
       1.5480162580400620e-04, 
       2.0265709441273327e-04, 
       1.5589225669549738e-04, 
       1.5811161405271450e-04, 
       1.4583043058910938e-04, 
       1.5275191591023984e-04, 
       1.6386759147379607e-04, 
       1.5649873055971272e-04, 
       2.0210330448043061e-04, 
       1.5673742734416898e-04, 
       1.8868186705622329e-04, 
       1.6853581307339272e-04, 
       1.5201154003508039e-04, 
       1.5002609397987582e-04, 
       1.9935176528461965e-04, 
       1.6925668572832545e-04, 
       1.5852872819336388e-04, 
       1.9211338644997480e-04, 
       1.6450494012466580e-04, 
       1.9948930314551822e-04, 
       1.4189911685380784e-04, 
       1.6386174795584598e-04, 
       2.0000000000003292e-04, 
       1.7038579739620565e-04, 
       1.4153030563987212e-04, 
       1.4922653782461313e-04, 
       1.4865592872318311e-04, 
       1.8140558847304982e-04, 
       1.6647439892511631e-04, 
       1.7123753807215216e-04, 
       1.4565639678960004e-04, 
       1.6131827162816075e-04, 
       1.6130062419559557e-04, 
       1.4655596342892557e-04, 
       1.4777127138669135e-04, 
       1.8323557599784652e-04, 
       1.5887786913816517e-04, 
       1.5880612631051874e-04, 
       1.4703599117121615e-04, 
       1.5396621220992117e-04, 
       1.5256192388537245e-04, 
       1.4475885337201480e-04, 
       1.7564620822911917e-04, 
       1.4658731516480235e-04, 
       1.6263882846477584e-04, 
       1.9007235950751759e-04, 
       1.4487943684404124e-04, 
       1.5797187059511290e-04, 
       1.4305270602086776e-04, 
       1.4104692972479634e-04, 
       1.4889820738663365e-04, 
       1.4889820738663365e-04, 
       1.6188695180629020e-04, 
       1.4531913062627651e-04, 
       1.3939772249568275e-04, 
       1.5199269830109031e-04, 
       1.3952832480440807e-04, 
       1.7948020866856711e-04, 
       1.4598068997115319e-04, 
       1.4638352527027390e-04, 
       1.4252105004144175e-04, 
       1.6703984108697837e-04, 
       1.6226098768177096e-04, 
       1.3984191175566446e-04, 
       1.6109085501545911e-04, 
       1.3917560096728221e-04, 
       1.4823399073103232e-04, 
       1.5917056510460700e-04, 
       1.6114494611487964e-04, 
       1.6094430309989609e-04, 
       1.6090858292786033e-04, 
       1.8586403340804404e-04, 
       1.4614023712254063e-04, 
       1.8722612635460367e-04, 
       1.5963222980164971e-04, 
       1.9258407914400820e-04, 
       1.5057156362152240e-04, 
       1.8231387657230216e-04, 
       1.5298614928851081e-04, 
       1.9180258387790046e-04, 
       1.8456411542339035e-04, 
       1.5760672372505650e-04, 
       1.4634116113870644e-04, 
       1.4131268587429870e-04, 
       1.5813213596671094e-04, 
       1.4234461109327207e-04, 
       1.5444728059196133e-04, 
       1.7391698052298776e-04, 
       1.3978729177619508e-04, 
       1.9000000000003985e-04, 
       1.8999999999996331e-04, 
       1.8543444693126871e-04, 
       1.8127371360462133e-04, 
       1.6393186684220067e-04, 
       1.6119617736567931e-04, 
       1.4120782633257397e-04, 
       1.4486855747275948e-04, 
       1.8565518721806655e-04, 
       1.5750451354046977e-04, 
       1.3566106770843454e-04, 
       1.3408159503427373e-04, 
       1.8840942442162066e-04, 
       1.3418773219158706e-04, 
       1.8355787696957358e-04, 
       1.8805342139972795e-04, 
       1.5664796201008679e-04, 
       1.3738560012912295e-04, 
       1.5742573826460271e-04, 
       1.3628009623921665e-04, 
       1.5628290943715172e-04, 
       1.4735028220591288e-04, 
       1.3352132243735313e-04, 
       1.8009899990636949e-04, 
       1.7995833071148570e-04, 
       1.8659117999617753e-04, 
       1.3540183094536501e-04, 
       1.3335621257032373e-04, 
       1.5144678391270535e-04, 
       1.3823210578779740e-04, 
       1.6442668361415136e-04, 
       1.4193732633142146e-04, 
       1.8645140303901687e-04, 
       1.5536043186832382e-04, 
       1.3790606748834616e-04, 
       1.5965898780879206e-04, 
       1.8613226350233769e-04, 
       1.4228127918744228e-04, 
       1.4228127918744228e-04, 
       1.7910694429395607e-04, 
       1.3886445534098125e-04, 
       1.5291585927182499e-04, 
       1.3702736801156679e-04, 
       1.4148963027882838e-04, 
       1.3626089522172935e-04, 
       1.5340780807074033e-04, 
       1.3086033775648668e-04, 
       1.3472747676253444e-04, 
       1.5388677799297988e-04, 
       1.5300727782187963e-04, 
       1.3893791039736324e-04, 
       1.3055202271219514e-04, 
       1.7869023289706230e-04, 
       1.8330734739237245e-04, 
       1.7954853120639836e-04, 
       1.4130018723299147e-04, 
       1.2964238938799338e-04, 
       1.3000000000000500e-04, 
       1.7448456336048519e-04, 
       1.5284264438161980e-04, 
       1.8241009476485897e-04, 
       1.5296792097227612e-04, 
       1.3455418110024713e-04, 
       1.3905941360776441e-04, 
       1.7914656227956832e-04, 
       1.7471604237557093e-04, 
       1.8184097379011637e-04, 
       1.8048553862764206e-04, 
       1.7157880938600221e-04, 
       1.3448798795938360e-04, 
       1.5075862181441099e-04, 
       1.8039208196318228e-04, 
       1.8029859724180144e-04, 
       1.2970876796068303e-04, 
       1.3245728493635982e-04, 
       1.8000000000002077e-04, 
       1.4216448340443382e-04, 
       1.8000000000002226e-04, 
       1.8000000000001275e-04, 
       1.3017621797024967e-04, 
       1.3443956368586123e-04, 
       1.2952759826074327e-04, 
       1.5387316335389024e-04, 
       1.5912604808692449e-04, 
       1.7455886127829326e-04, 
       1.4719608075528955e-04, 
       1.6316723134130706e-04, 
       1.3297058212563280e-04, 
       1.4335741709054479e-04, 
       1.3157080375089433e-04, 
       1.7716585667569063e-04, 
       1.3000000000001471e-04, 
       1.4785695918166319e-04, 
       1.5157711389416218e-04, 
       1.4686561901280682e-04, 
       1.7067842344961311e-04, 
       1.4653936132412600e-04, 
       1.2447760840523199e-04, 
       1.2459580892553528e-04, 
       1.2405372936183270e-04, 
       1.7014372302603348e-04, 
       1.3102710557645722e-04, 
       1.2643841474122445e-04, 
       1.2464807486674666e-04, 
       1.6483510297495805e-04, 
       1.2451522767356545e-04, 
       1.3517562639984299e-04, 
       1.3978305538656557e-04, 
       1.3370668305938399e-04, 
       1.4908730012117805e-04, 
       1.2281322566576729e-04, 
       1.2918416459710481e-04, 
       1.6908967219138671e-04, 
       1.6580496218566068e-04, 
       1.4339080594022024e-04, 
       1.4154095875788987e-04, 
       1.6339785093517301e-04, 
       1.3218778997232537e-04, 
       1.6583434988729504e-04, 
       1.7109875005711265e-04, 
       1.2208479655807610e-04, 
       1.7153848512945912e-04, 
       1.2264675167000320e-04, 
       1.4311927612881812e-04, 
       1.2806238289468123e-04, 
       1.2255569134591823e-04, 
       1.6963550548596991e-04, 
       1.2174584543555244e-04, 
       1.3755528165326024e-04, 
       1.7000000000002576e-04, 
       1.4694753634467622e-04, 
       1.2385642484404952e-04, 
       1.2060199576864490e-04, 
       1.3289378310202886e-04, 
       1.7000000000000099e-04, 
       1.6999999999995949e-04, 
       1.2347559307610947e-04, 
       1.3056430606538138e-04, 
       1.2756777418066747e-04, 
       1.2725421518942540e-04, 
       1.4225195534547599e-04, 
       1.2286527424750272e-04, 
       1.4865755944095356e-04, 
       1.6059486697790195e-04, 
       1.4021547929632110e-04, 
       1.4453980011956607e-04, 
       1.6180867436633235e-04, 
       1.3424684953348040e-04, 
       1.2159045515285430e-04, 
       1.6475150319801389e-04, 
       1.2878824779840171e-04, 
       1.4462478997558280e-04, 
       1.3879869157117265e-04, 
       1.6614099885489198e-04, 
       1.6185801959003497e-04, 
       1.1909943731536806e-04, 
       1.3550780264248998e-04, 
       1.6671018263061932e-04, 
       1.2650625575230391e-04, 
       1.2925854372029339e-04, 
       1.2913476548672627e-04, 
       1.4905605843806101e-04, 
       1.6630528932720849e-04, 
       1.1991508831452532e-04, 
       1.4112333148850589e-04, 
       1.5033782635389117e-04, 
       1.5014199107311798e-04, 
       1.6559684807582580e-04, 
       1.5267771353526924e-04, 
       1.3323118472225129e-04, 
       1.5056739787229956e-04, 
       1.1719132503332358e-04, 
       1.1804333115692082e-04, 
       1.1760131079247727e-04, 
       1.1756055417724166e-04, 
       1.3688605158480185e-04, 
       1.6380880965737163e-04, 
       1.2883650946927860e-04, 
       1.6237713369364722e-04, 
       1.3630539375573893e-04, 
       1.6381860609971792e-04, 
       1.6262014773449490e-04, 
       1.2069620876317821e-04, 
       1.3969029887618591e-04, 
       1.5235228325381566e-04, 
       1.1672148138939228e-04, 
       1.1658546710949908e-04, 
       1.3644479149737515e-04, 
       1.5209948769125145e-04, 
       1.1763742938814272e-04, 
       1.3919857330780473e-04, 
       1.6298589813169067e-04, 
       1.5053581070970960e-04, 
       1.5000000000001910e-04, 
       1.5069024567772744e-04, 
       1.4544735982563082e-04, 
       1.3158982808350426e-04, 
       1.5624371793753139e-04, 
       1.5838834046787132e-04, 
       1.3410235805684048e-04, 
       1.3238307621922009e-04, 
       1.3460666294570382e-04, 
       1.5880752538002921e-04, 
       1.3486178681172586e-04, 
       1.1850134478763180e-04, 
       1.5704146721262860e-04, 
       1.6065436212689589e-04, 
       1.5080638990430960e-04, 
       1.2002298997067175e-04, 
       1.3344053357597520e-04, 
       1.3762666613752428e-04, 
       1.3831279475731023e-04, 
       1.5872879914473334e-04, 
       1.1702022556971323e-04, 
       1.5999999999995391e-04, 
       1.6000000000000779e-04, 
       1.1331944199996376e-04, 
       1.5930875822672526e-04, 
       1.5351109597861923e-04, 
       1.5832024257360079e-04, 
       1.3251117294219608e-04, 
       1.5394681573615768e-04, 
       1.4428363416040311e-04, 
       1.2487632250111243e-04, 
       1.1888712238653807e-04, 
       1.3227494497380926e-04, 
       1.3651285580680534e-04, 
       1.3206640364101142e-04, 
       1.3627218234281901e-04, 
       1.5340886602111347e-04, 
       1.1448345063470422e-04, 
       1.3592060976283315e-04, 
       1.1851240154062151e-04, 
       1.1295430251754943e-04, 
       1.3086445416674380e-04, 
       1.5751810183404623e-04, 
       1.1462770344066444e-04, 
       1.4665142352712524e-04, 
       1.2208871501011412e-04, 
       1.1428018053889767e-04, 
       1.1499930693333800e-04, 
       1.2588953654026048e-04, 
       1.5699835465420617e-04, 
       1.4561514422524989e-04, 
       1.5255547493516980e-04, 
       1.3068549716880240e-04, 
       1.2388467316961461e-04, 
       1.1277502243189071e-04, 
       1.5700586922016876e-04, 
       1.2518122822048272e-04, 
       1.3505606027831694e-04, 
       1.1683775191893710e-04, 
       1.1166436603549957e-04, 
       1.5482387891183660e-04, 
       1.1132268571256807e-04, 
       1.5636715982280181e-04, 
       1.1234095448336817e-04, 
       1.2580202647069163e-04, 
       1.1832324759572731e-04, 
       1.4476595734953294e-04, 
       1.2909864667410602e-04, 
       1.1759139102050370e-04, 
       1.4439216008446139e-04, 
       1.1152195640475793e-04, 
       1.2927921775693200e-04, 
       1.5194913283033722e-04, 
       1.3300969286135706e-04, 
       1.3288862311422117e-04, 
       1.2260380650769523e-04, 
       1.5345250003478554e-04, 
       1.1075588558350213e-04, 
       1.1117618258523058e-04, 
       1.1254069193349568e-04, 
       1.2632911205967578e-04, 
       1.1486703135125598e-04, 
       1.2185660220151639e-04, 
       1.0910066659627562e-04, 
       1.3912362956192693e-04, 
       1.5261955325947604e-04, 
       1.1010075447259687e-04, 
       1.3402797639511683e-04, 
       1.4175482508578812e-04, 
       1.2580055537558685e-04, 
       1.1814971544949213e-04, 
       1.4323255470301811e-04, 
       1.1763832620088614e-04, 
       1.2676046176393377e-04, 
       1.4552646040123655e-04, 
       1.5275008841376291e-04, 
       1.1430410815036325e-04, 
       1.2599413969948415e-04, 
       1.5186123918611613e-04, 
       1.2954460197720776e-04, 
       1.1643813256308433e-04, 
       1.1794416741161970e-04, 
       1.3272116574203515e-04, 
       1.4000000000003967e-04, 
       1.3866962318668868e-04, 
       1.5027839277891486e-04, 
       1.2408214929729094e-04, 
       1.3136918225224007e-04, 
       1.3058218750917821e-04, 
       1.0900227995060493e-04, 
       1.0812486473705156e-04, 
       1.1000000000000477e-04, 
       1.2301501421590460e-04, 
       1.1221538520675204e-04, 
       1.1050516540022862e-04, 
       1.0886341379226622e-04, 
       1.2779982847049026e-04, 
       1.5000000000004620e-04, 
       1.2980106501344824e-04, 
       1.2980106501344824e-04, 
       1.2450866466208140e-04, 
       1.0607329251539465e-04, 
       1.1526734122477948e-04, 
       1.2930918068652445e-04, 
       1.1098463061708403e-04, 
       1.3987372690784072e-04, 
       1.2789019317099347e-04, 
       1.2197401212516218e-04, 
       1.4484942316613520e-04, 
       1.2367618622315753e-04, 
       1.2853154831903485e-04, 
       1.2776870203034564e-04, 
       1.4651213369541353e-04, 
       1.2918728289517642e-04, 
       1.0785500712910272e-04, 
       1.4813364912937558e-04, 
       1.4217426421116026e-04, 
       1.3689600627323823e-04, 
       1.1600472723107482e-04, 
       1.0769226148962623e-04, 
       1.1058547095259425e-04, 
       1.0969714610893606e-04, 
       1.0536772778078748e-04, 
       1.1679151521150670e-04, 
       1.3218636195589430e-04, 
       1.2101758820216473e-04, 
       1.0574030783565906e-04, 
       1.2306261788190418e-04, 
       1.3916954153768641e-04, 
       1.4623470193514220e-04, 
       1.3164098117362304e-04, 
       1.1602353451452037e-04, 
       1.0564102775582140e-04, 
       1.2921388712637788e-04, 
       1.4096924073079368e-04, 
       1.2251684245847343e-04, 
       1.0742540548134736e-04, 
       1.4571247290509837e-04, 
       1.1870094810708615e-04, 
       1.1857481691012595e-04, 
       1.0999999999999777e-04, 
       1.2942071276436017e-04, 
       1.0891526675758188e-04, 
       1.2183667546130603e-04, 
       1.2071273883791727e-04, 
       1.4561969672886060e-04, 
       1.2642214618780069e-04, 
       1.4358871043113362e-04, 
       1.4631824587741979e-04, 
       1.0441216353664519e-04, 
       1.0758310678498174e-04, 
       1.1722109433736283e-04, 
       1.0351293863515583e-04, 
       1.2417566669373627e-04, 
       1.1000000000003117e-04, 
       1.2772791051136360e-04, 
       1.4495150345010912e-04, 
       1.2335844638726359e-04, 
       1.0614733444542853e-04, 
       1.1101888361306983e-04, 
       1.4201194672341949e-04, 
       1.0752541603442476e-04, 
       1.0349048526868057e-04, 
       1.1948299121558004e-04, 
       1.4483394606147943e-04, 
       1.4313449224105597e-04, 
       1.4451949577048187e-04, 
       1.0677506532342738e-04, 
       1.0642295243891492e-04, 
       1.3930003683782711e-04, 
       1.0586591301502096e-04, 
       1.2841646078912031e-04, 
       1.4108572918725108e-04, 
       1.1323506373512079e-04, 
       1.1736735659276794e-04, 
       1.4025624931218717e-04, 
       1.0477536765615824e-04, 
       1.0706321074530425e-04, 
       1.0415296577244842e-04, 
       1.1837213698458455e-04, 
       1.3261502422008879e-04, 
       1.0576477927013387e-04, 
       1.3055209835826658e-04, 
       1.3054729632964022e-04, 
       1.0425305513720020e-04, 
       1.4237852759741573e-04, 
       1.0109610832278538e-04, 
       1.1826126915906035e-04, 
       1.3704896562040164e-04, 
       1.4093093866209729e-04, 
       1.0385972721385113e-04, 
       1.2610483248449864e-04, 
       1.4000000000004875e-04, 
       1.0980833694774068e-04, 
       1.2229754359906664e-04, 
       1.4122845576607592e-04, 
       1.0208209851727968e-04, 
       1.0234832591432975e-04, 
       9.9657635289861066e-05, 
       1.3248514783421544e-04, 
       1.3248514783421544e-04, 
       1.0377366624192343e-04, 
       1.1103403941539034e-04, 
       1.0154046291275607e-04, 
       1.0906959122769470e-04, 
       1.0987668992938946e-04, 
       1.0069531561556829e-04, 
       1.1354629003186272e-04, 
       9.9295752501816803e-05, 
       1.3036286207517575e-04, 
       1.1650074746065720e-04, 
       1.3999999999995990e-04, 
       1.4000000000000110e-04, 
       1.3999999999999890e-04, 
       1.3999999999995198e-04, 
       1.3564817425279778e-04, 
       1.0043021954543532e-04, 
       9.8955601795769620e-05, 
       1.0807437342299052e-04, 
       1.3542203561031402e-04, 
       1.1603019781220284e-04, 
       1.0064065551563254e-04, 
       1.0947346569794872e-04, 
       1.3444971469351975e-04, 
       1.0556037979060595e-04, 
       1.3829812157927548e-04, 
       1.2999999999998714e-04, 
       9.8429234511192070e-05, 
       1.0166981641428263e-04, 
       1.1478205072131659e-04, 
       1.1040980640617042e-04, 
       1.0851264020890535e-04, 
       1.0843069105958756e-04, 
       1.1092130330270697e-04, 
       1.3750860922979695e-04, 
       9.9656089565455891e-05, 
       1.1623577018041262e-04, 
       1.0467974832721055e-04, 
       1.1107459701207377e-04, 
       1.0273891534618994e-04, 
       1.0420787295947745e-04, 
       1.3697496735186256e-04, 
       1.3637557484326128e-04, 
       1.1194136049221840e-04, 
       1.1194136049221840e-04, 
       1.0133367269267899e-04, 
       1.1128831754655549e-04, 
       1.1747764593815278e-04, 
       9.9064662352325492e-05, 
       1.3624687853630552e-04, 
       1.3537163431206719e-04, 
       1.3615828642036677e-04, 
       1.0088081393884620e-04, 
       9.6304947622919727e-05, 
       1.1153708120011003e-04, 
       1.1230553903125473e-04, 
       1.1632769477829590e-04, 
       1.1248680259543406e-04, 
       1.0065541685252134e-04, 
       1.3495516404631613e-04, 
       9.8356333291848986e-05, 
       1.2164487871491115e-04, 
       9.6827183575219939e-05, 
       1.3252104357553397e-04, 
       9.5846227302292321e-05, 
       1.2363091705896843e-04, 
       1.3515209756859590e-04, 
       1.0377627372025054e-04, 
       1.2069071941825651e-04, 
       1.2903555962552683e-04, 
       1.1113112433489377e-04, 
       1.3403191385925837e-04, 
       1.1896349798563184e-04, 
       1.2810434362628813e-04, 
       9.4906448671834605e-05, 
       1.0566561636052870e-04, 
       9.6842548233186989e-05, 
       1.1905140142587465e-04, 
       1.0377804388182562e-04, 
       1.2550640066725060e-04, 
       9.7561915037576536e-05, 
       1.3093956194442865e-04, 
       1.1410020794503139e-04, 
       1.2718986759722498e-04, 
       1.2987998109300602e-04, 
       1.0231402397797340e-04, 
       1.2525713326394571e-04, 
       1.0412416749121179e-04, 
       9.5831123647240895e-05, 
       1.2671755524927864e-04, 
       1.2898753537336682e-04, 
       1.1455997230533565e-04, 
       1.3213829568422465e-04, 
       1.0984612006841703e-04, 
       1.0577944699864368e-04, 
       1.0842216281595205e-04, 
       9.4038079998852997e-05, 
       1.0856887987416502e-04, 
       9.2964632245068297e-05, 
       1.3074205028631395e-04, 
       9.3412601347815421e-05, 
       1.1049674394400671e-04, 
       1.1049674394400671e-04, 
       9.8068554084132712e-05, 
       1.2756147172683277e-04, 
       1.3056452019125173e-04, 
       1.2392332873368850e-04, 
       1.1251162461555109e-04, 
       9.3536488593400283e-05, 
       1.1842575013581634e-04, 
       9.4376460021816503e-05, 
       1.2978097511959626e-04, 
       1.2994245233419633e-04, 
       9.4306630330219065e-05, 
       1.0854535738323931e-04, 
       1.1554208279398457e-04, 
       1.2987544981282417e-04, 
       1.0103366110753686e-04, 
       9.3850990420747563e-05, 
       1.2511734246622864e-04, 
       1.0746409006099631e-04, 
       1.2540665695261451e-04, 
       1.2836901770054131e-04, 
       1.2417774990342668e-04, 
       1.0081498044052136e-04, 
       1.2751866875823051e-04, 
       9.1568277510188333e-05, 
       1.0659709653387339e-04, 
       1.0647527542724309e-04, 
       9.4330581591470892e-05, 
       1.0558106245986374e-04, 
       1.0601071329336298e-04, 
       1.0052419587885571e-04, 
       1.0178137618718769e-04, 
       9.1498517567628563e-05, 
       1.2708940120230672e-04, 
       9.6925535558581706e-05, 
       9.0426684819324191e-05, 
       1.0885913717100040e-04, 
       1.2676901182814726e-04, 
       1.2134697536355791e-04, 
       9.4571172294993715e-05, 
       1.0514104418691219e-04, 
       1.2638674453410565e-04, 
       1.2442248464244589e-04, 
       9.6158389456886184e-05, 
       9.4543497887224144e-05, 
       8.9363620613159311e-05, 
       9.4509090912438926e-05, 
       9.4464279263820072e-05, 
       1.2466455270596931e-04, 
       9.7506219036385476e-05, 
       1.2044255214556726e-04, 
       1.0335441333010665e-04, 
       1.2486393866962324e-04, 
       9.3722178080192541e-05, 
       1.2065519576638520e-04, 
       1.1919620842876061e-04, 
       1.2450837504746990e-04, 
       9.0090384122727468e-05, 
       1.1610790525159790e-04, 
       1.0346654698785405e-04, 
       1.0199577774775537e-04, 
       9.9771768052383795e-05, 
       1.1677818757918457e-04, 
       1.2370503759884309e-04, 
       9.6272527017114047e-05, 
       9.5143961837704921e-05, 
       8.7512734539831790e-05, 
       8.9531775900607755e-05, 
       9.2200860650585960e-05, 
       1.2244128365521056e-04, 
       1.1354368932134140e-04, 
       1.2140564631876083e-04, 
       1.1869844864661550e-04, 
       8.9345052070426576e-05, 
       9.8034692418431243e-05, 
       1.1447774947201518e-04, 
       1.0599345875627486e-04, 
       8.7035843707702291e-05, 
       8.7035843707702291e-05, 
       1.0439107303714172e-04, 
       1.2248662230820542e-04, 
       1.2244104338450512e-04, 
       1.1948375995483888e-04, 
       1.2063232203128406e-04, 
       9.2192610898947654e-05, 
       9.9275150921640813e-05, 
       1.0138903213781838e-04, 
       9.9006615903272943e-05, 
       1.2172546652734902e-04, 
       1.1556318203856009e-04, 
       1.0124609829233624e-04, 
       1.2156565490811653e-04, 
       8.9513409976290447e-05, 
       8.7331786324354487e-05, 
       8.9955843974294478e-05, 
       9.9928520142543412e-05, 
       9.9192527763147402e-05, 
       1.1462072554235926e-04, 
       9.1346618405053215e-05, 
       8.5951004611516685e-05, 
       1.0112073733496960e-04, 
       1.1116710871815987e-04, 
       1.1633127694946090e-04, 
       1.0013600070086803e-04, 
       1.0035429818448213e-04, 
       9.4425047974876463e-05, 
       1.0319534459946427e-04, 
       1.1839225015148166e-04, 
       8.5762514177716880e-05, 
       9.9765934269498006e-05, 
       9.9842035472299086e-05, 
       1.1514015988654480e-04, 
       1.2000000000002682e-04, 
       1.2000000000001061e-04, 
       1.1999999999999011e-04, 
       1.2000000000003645e-04, 
       1.1999999999995575e-04, 
       1.1940477085031192e-04, 
       1.1602678910316416e-04, 
       8.6276530873133497e-05, 
       1.1300226761680984e-04, 
       8.7127563208010768e-05, 
       9.3966642725029477e-05, 
       1.1615330055173764e-04, 
       9.1555899520077836e-05, 
       9.1555899520077836e-05, 
       9.9945528136703498e-05, 
       1.1362537921878585e-04, 
       1.1112902742103280e-04, 
       1.0211065647865741e-04, 
       1.1532975179064319e-04, 
       1.1000000000002025e-04, 
       1.0074743513481569e-04, 
       8.7867454246812627e-05, 
       1.1212758179156246e-04, 
       1.1431728130369256e-04, 
       8.4023961728743211e-05, 
       1.1793771348098868e-04, 
       9.3813914394615030e-05, 
       8.6130628860348768e-05, 
       1.0505025669671977e-04, 
       8.3622980031645457e-05, 
       1.0559895742983438e-04, 
       8.4293287369582029e-05, 
       9.7854623379588660e-05, 
       8.4415530883748811e-05, 
       9.6690454032227584e-05, 
       1.1770778920013142e-04, 
       9.7800668104001978e-05, 
       9.7805423034166199e-05, 
       9.7416718583801949e-05, 
       8.2955144511724589e-05, 
       9.9056905683387656e-05, 
       9.7385227558982390e-05, 
       1.0396415602121767e-04, 
       9.2951087446997179e-05, 
       9.4481520445565041e-05, 
       9.2561394538370002e-05, 
       1.1384389226161904e-04, 
       1.1553371189915702e-04, 
       1.1623831228798012e-04, 
       8.6391518808076354e-05, 
       1.1519303820306984e-04, 
       1.1589918770282245e-04, 
       1.1600366311653424e-04, 
       1.1509223601072721e-04, 
       8.8967815755936044e-05, 
       9.0749700293172499e-05, 
       1.1548223978556904e-04, 
       9.5524527112435408e-05, 
       8.5944313369419459e-05, 
       8.1724010569126905e-05, 
       1.1243584227505029e-04, 
       9.1221656927148136e-05, 
       8.5202237062151012e-05, 
       1.0941008363297610e-04, 
       9.7358407967880085e-05, 
       8.4685704590305039e-05, 
       9.4908316937072635e-05, 
       8.3037738680439419e-05, 
       1.1337543984622992e-04, 
       8.4470407317411444e-05, 
       1.1453587300632310e-04, 
       9.5471656748831216e-05, 
       1.1084921182039240e-04, 
       8.2788688099392062e-05, 
       1.0924870113103861e-04, 
       9.2436780118694233e-05, 
       1.1389458846203944e-04, 
       8.2244474887760931e-05, 
       8.3353302877787656e-05, 
       1.1047209249116994e-04, 
       8.5745476677760577e-05, 
       1.1345591171058293e-04, 
       8.1272904770083280e-05, 
       1.1045211548922276e-04, 
       8.4833353592981371e-05, 
       8.3685848159278971e-05, 
       8.5066706933864246e-05, 
       8.0890476077037662e-05, 
       9.2448192853909742e-05, 
       1.0858067026619031e-04, 
       9.3291574089670101e-05, 
       8.5104330691077363e-05, 
       1.0952768191227760e-04, 
       9.3800360305940211e-05, 
       1.0107512810484684e-04, 
       1.1228292543536248e-04, 
       1.0445731817299992e-04, 
       9.2893930070133695e-05, 
       1.0340820264427405e-04, 
       1.0768106105878452e-04, 
       9.8049152152108400e-05, 
       7.9572939252956745e-05, 
       9.4380661800547267e-05, 
       9.4380661800547267e-05, 
       1.0963775318673943e-04, 
       9.4256986698576360e-05, 
       9.5173526053481569e-05, 
       8.0868984060553502e-05, 
       8.5155356114443334e-05, 
       1.0381659204951080e-04, 
       1.1067527173261012e-04, 
       8.1159215371754963e-05, 
       1.0615819232929342e-04, 
       9.3615981350920669e-05, 
       8.5263930059071750e-05, 
       8.0340520419823385e-05, 
       8.2788422078563148e-05, 
       1.0359009487965980e-04, 
       1.0960141660273000e-04, 
       7.9055850269712895e-05, 
       8.1959537506551521e-05, 
       8.0461347404811909e-05, 
       8.3381645528957660e-05, 
       7.8893324423134453e-05, 
       1.0999999999996466e-04, 
       1.0999999999999050e-04, 
       1.1000000000002681e-04, 
       1.0867527579689083e-04, 
       7.9114858003150043e-05, 
       7.9114858003150043e-05, 
       1.0940779309819365e-04, 
       1.0940779309819365e-04, 
       9.1211107282342154e-05, 
       8.2890656864390816e-05, 
       1.0818824581912958e-04, 
       1.0546465356649532e-04, 
       1.0868322690043291e-04, 
       7.8668509264994595e-05, 
       8.1055071394576681e-05, 
       8.2027424771362743e-05, 
       8.8035574315166909e-05, 
       7.7779868496286229e-05, 
       8.8328257280755672e-05, 
       1.0850215531571734e-04, 
       1.0539588646391191e-04, 
       1.0000000000000293e-04, 
       9.9999999999997199e-05, 
       1.0000000000000213e-04, 
       1.0561781008158581e-04, 
       1.0681730710454259e-04, 
       8.4848863112152629e-05, 
       7.9403614781265675e-05, 
       1.0414488429680525e-04, 
       1.0775223350058697e-04, 
       9.6515286832137652e-05, 
       1.0658434060926229e-04, 
       9.8572466268698081e-05, 
       1.0197485998576104e-04, 
       9.3891335191718063e-05, 
       7.6214426003865768e-05, 
       1.0758372172587323e-04, 
       8.3884370989410171e-05, 
       1.0156307729652022e-04, 
       8.3114617639840183e-05, 
       8.1955403856173950e-05, 
       9.3317896251426767e-05, 
       8.8895642732941612e-05, 
       1.0162880199726895e-04, 
       7.9730346619432860e-05, 
       7.7966020727073871e-05, 
       7.5735647743168299e-05, 
       7.6429820632332321e-05, 
       1.0287132953243190e-04, 
       1.0380151677870940e-04, 
       1.0017464756616927e-04, 
       7.9999999999996646e-05, 
       8.8230286362698623e-05, 
       7.5661582392445379e-05, 
       1.0506710316707535e-04, 
       7.6738211588522158e-05, 
       1.0537354304223342e-04, 
       9.5502279778848141e-05, 
       8.1218815848468624e-05, 
       7.5366017361578195e-05, 
       7.4423303316714358e-05, 
       1.0483320576757968e-04, 
       7.4389955013166905e-05, 
       7.5840005617816426e-05, 
       7.6734307329256053e-05, 
       7.5727824581932156e-05, 
       1.0350317149136423e-04, 
       8.6966750882960743e-05, 
       9.3870636148536010e-05, 
       8.9312029262868700e-05, 
       8.4390949977764025e-05, 
       7.4278930589606739e-05, 
       9.6512527911488111e-05, 
       7.4154128754928101e-05, 
       7.7655812288651517e-05, 
       1.0286907776116645e-04, 
       1.0343697252000206e-04, 
       8.6228696586167710e-05, 
       8.5572049362252905e-05, 
       8.3904047023546598e-05, 
       1.0179459697039431e-04, 
       8.9016268345203483e-05, 
       9.9558843251468853e-05, 
       7.6000548394638433e-05, 
       7.3211179323460894e-05, 
       1.0294318674654857e-04, 
       1.0286615917185163e-04, 
       1.0162985949567205e-04, 
       8.7546704778422680e-05, 
       8.6164166721760784e-05, 
       9.9774097554845140e-05, 
       8.1714722077438990e-05, 
       1.0073090849571845e-04, 
       8.0349657037898887e-05, 
       8.0332281060801548e-05, 
       8.4878596453155566e-05, 
       1.0239093399838701e-04, 
       7.6685599993349466e-05, 
       7.6673263168761430e-05, 
       9.8244827391186885e-05, 
       1.0178196647139997e-04, 
       1.0179672767107139e-04, 
       7.3411521523460003e-05, 
       8.4854367101241479e-05, 
       7.2247389904331249e-05, 
       7.2586099024853055e-05, 
       1.0062959282437415e-04, 
       9.6070158464693373e-05, 
       1.0057105244584317e-04, 
       7.5843537765876018e-05, 
       8.7795311963420261e-05, 
       9.2599101466128955e-05, 
       8.4224632681681580e-05, 
       8.5195792862301475e-05, 
       7.2628150845082163e-05, 
       7.5415014946150270e-05, 
       1.0069621296776179e-04, 
       7.5145697462586790e-05, 
       9.4319717602733870e-05, 
       7.1647255648673130e-05, 
       7.5388134286489715e-05, 
       8.9005050774863446e-05, 
       7.1161584355092179e-05, 
       7.0953088056664590e-05, 
       7.0953088056664590e-05, 
       9.4311546715269355e-05, 
       1.0000000000000141e-04, 
       9.9999999999997647e-05, 
       9.5699383237257270e-05, 
       1.0000000000000427e-04, 
       1.0000000000000053e-04, 
       1.0000000000000341e-04, 
       9.9999999999996752e-05, 
       1.0000000000000297e-04, 
       1.0000000000000070e-04, 
       7.4146819782277087e-05, 
       9.9703495667117354e-05, 
       8.1464730418087807e-05, 
       9.7331424747502341e-05, 
       9.7331424747502341e-05, 
       8.0048655973706184e-05, 
       9.3480151763516690e-05, 
       8.1934800861325449e-05, 
       7.4422581191619977e-05, 
       7.1893852149892043e-05, 
       7.2862131336357596e-05, 
       7.1884501972677378e-05, 
       7.9778861206226335e-05, 
       7.2926842917400131e-05, 
       7.2926842917400131e-05, 
       9.7471854532309716e-05, 
       9.8080267013623746e-05, 
       8.3603059033530118e-05, 
       7.9497944942830744e-05, 
       7.2550369392349840e-05, 
       7.1465323252195269e-05, 
       9.1190481756276459e-05, 
       8.1743171367494244e-05, 
       8.1139721848425702e-05, 
       8.1688674650823659e-05, 
       7.2529679544255335e-05, 
       9.7094250011530917e-05, 
       7.9502988976741699e-05, 
       8.1299728503353869e-05, 
       9.0436037884926559e-05, 
       9.4020228180045088e-05, 
       7.3112487755776605e-05, 
       8.4415530883770983e-05, 
       7.1977926524152626e-05, 
       7.6855392126411998e-05, 
       8.9999999999999233e-05, 
       9.0000000000003678e-05, 
       7.2533271211803251e-05, 
       9.7245830656222566e-05, 
       8.3761347872436233e-05, 
       7.8092171037543636e-05, 
       7.7697550788231943e-05, 
       7.1550289932836533e-05, 
       8.1175009592478309e-05, 
       7.1955445493150587e-05, 
       8.3826263947488899e-05, 
       8.5729414802535750e-05, 
       7.2421866503454319e-05, 
       6.9383606549503288e-05, 
       8.6927614155445348e-05, 
       6.9382770287509417e-05, 
       8.5587289252643016e-05, 
       6.9326960789822397e-05, 
       9.2480855525770190e-05, 
       9.5728700169687869e-05, 
       8.0256883662350377e-05, 
       7.0070544836900786e-05, 
       9.4792870542342609e-05, 
       7.0366813033908153e-05, 
       8.1084415168745192e-05, 
       7.9735025743089905e-05, 
       9.1792997090828920e-05, 
       9.1053232063024393e-05, 
       9.5913061667411793e-05, 
       9.4042946916507789e-05, 
       6.7999276604758626e-05, 
       6.8474425480938143e-05, 
       7.9654938624268919e-05, 
       9.3723815080277940e-05, 
       9.5676534965772713e-05, 
       7.0388293468966594e-05, 
       8.2057384754935232e-05, 
       7.8161830877482674e-05, 
       8.2485753408697050e-05, 
       7.1370583321989726e-05, 
       6.8139088835959957e-05, 
       8.1552696896240850e-05, 
       8.5492938033150627e-05, 
       9.0907206423492112e-05, 
       9.2454676495149401e-05, 
       8.5717644317623824e-05, 
       7.9182735659086317e-05, 
       7.6144075259945352e-05, 
       9.4419891031294949e-05, 
       7.4250656158255553e-05, 
       6.9017418205090112e-05, 
       7.8887746098952089e-05, 
       6.9987379738301560e-05, 
       7.8002321254740586e-05, 
       6.6780561010405213e-05, 
       6.7186243639152869e-05, 
       7.2596467247276116e-05, 
       7.2596467247276116e-05, 
       6.7846243466845006e-05, 
       8.3891472763001937e-05, 
       9.0009321005519189e-05, 
       8.4277740342411772e-05, 
       6.9604237560139752e-05, 
       7.8196358749678193e-05, 
       9.3555487671977190e-05, 
       9.3640245851003744e-05, 
       8.7343269535561914e-05, 
       8.9484190877531987e-05, 
       7.3388374318265873e-05, 
       7.7676712477864107e-05, 
       6.8258486742265440e-05, 
       8.3337382537409618e-05, 
       8.8709501636487510e-05, 
       8.8709501636487510e-05, 
       9.1303188412297879e-05, 
       7.9082190066726991e-05, 
       9.2875913713771251e-05, 
       7.7332782299417965e-05, 
       7.9762421725201906e-05, 
       6.9257537033323008e-05, 
       7.7179975395809180e-05, 
       9.0002352116730071e-05, 
       9.0588739338419080e-05, 
       7.6806684541722348e-05, 
       6.9999999999998598e-05, 
       9.1618809721288733e-05, 
       6.8188990003593256e-05, 
       6.6063470530351045e-05, 
       7.0435074173813154e-05, 
       6.6215507826655241e-05, 
       6.6329897123408609e-05, 
       8.8424974330396480e-05, 
       6.7414583752201012e-05, 
       7.6363512437986278e-05, 
       9.1541920431189759e-05, 
       6.5131901563241865e-05, 
       6.5396030503433271e-05, 
       8.2846599380160412e-05, 
       6.5041196519487006e-05, 
       8.9213122415064103e-05, 
       6.5974163403416180e-05, 
       6.5864820262920133e-05, 
       6.7921683149407529e-05, 
       6.8530998012025638e-05, 
       8.2770997109351218e-05, 
       6.5098014267949214e-05, 
       9.1350761424749015e-05, 
       7.6120481420064928e-05, 
       8.2908977760718256e-05, 
       9.0914590593641081e-05, 
       6.4900782719687857e-05, 
       7.1398124834366351e-05, 
       6.6544306845790373e-05, 
       6.6899491668431904e-05, 
       7.2410783136780393e-05, 
       6.8174285466066463e-05, 
       8.7849265423576650e-05, 
       7.7663439392540613e-05, 
       7.5680774547335294e-05, 
       6.4872847496756528e-05, 
       7.6492434768168528e-05, 
       6.5281983308523772e-05, 
       9.0613840793301310e-05, 
       6.5327074988789341e-05, 
       6.4080288894462534e-05, 
       8.7233908758987938e-05, 
       8.6149437497267124e-05, 
       8.9999950570625675e-05, 
       7.7520724294355493e-05, 
       7.2562565519166720e-05, 
       6.7347704311890633e-05, 
       6.3916292012603852e-05, 
       8.9999999999995655e-05, 
       6.5920428750565452e-05, 
       8.7265413886742531e-05, 
       7.4076402830600799e-05, 
       6.4048238597748757e-05, 
       7.4532006012174019e-05, 
       8.3297140690299592e-05, 
       6.3821470204878408e-05, 
       8.7813948101613936e-05, 
       6.3209824057161115e-05, 
       6.8699214171282828e-05, 
       7.3762965169588650e-05, 
       8.8304517909717296e-05, 
       6.4051282924345180e-05, 
       8.3818350172659210e-05, 
       6.5549520568470768e-05, 
       6.2896412352349629e-05, 
       8.1645689767005552e-05, 
       6.2966309390052931e-05, 
       7.3252546280949007e-05, 
       6.3550976139069816e-05, 
       7.1260404003721731e-05, 
       8.5677888662320353e-05, 
       6.9588829920697972e-05, 
       6.5129804274759116e-05, 
       8.8599828855079199e-05, 
       6.7572204708764557e-05, 
       6.9038264025421601e-05, 
       8.5538617412263886e-05, 
       7.3874800587391456e-05, 
       7.4760276545282980e-05, 
       6.6276278488031911e-05, 
       8.4836322904792652e-05, 
       7.1497038929578213e-05, 
       8.4470504821376104e-05, 
       8.4867081212468593e-05, 
       7.2701087783134095e-05, 
       7.1910884467673791e-05, 
       7.1743269682464302e-05, 
       7.7662419315387821e-05, 
       6.5606467637641930e-05, 
       6.3676682746692588e-05, 
       8.1808987517328029e-05, 
       7.1803551197664121e-05, 
       6.3452453424297798e-05, 
       6.8588143177559274e-05, 
       6.4100516295068194e-05, 
       8.3177954187319727e-05, 
       7.3686164391551504e-05, 
       8.0000000000002866e-05, 
       7.3554524191097505e-05, 
       7.1268949777913092e-05, 
       8.6611142649908326e-05, 
       6.5184004601355258e-05, 
       8.2318628494050525e-05, 
       6.1349598018708282e-05, 
       8.6424304151061092e-05, 
       6.8265338605440031e-05, 
       7.1952145251717722e-05, 
       7.1878138642422702e-05, 
       8.5795819024782972e-05, 
       7.2190925825553915e-05, 
       6.1653385830927551e-05, 
       8.2606506546626261e-05, 
       6.4229169789835415e-05, 
       7.0770406394965791e-05, 
       7.2142130031235347e-05, 
       6.3557850259528639e-05, 
       6.4276263799240176e-05, 
       7.4024811178821279e-05, 
       6.2182150740272473e-05, 
       6.5366408238848363e-05, 
       6.2380054834357184e-05, 
       6.3929074551628953e-05, 
       6.5364505368102637e-05, 
       6.5356092432141181e-05, 
       8.1757751016047809e-05, 
       6.3820950798328710e-05, 
       8.5219313226147519e-05, 
       6.6919057255288334e-05, 
       6.6210452449539278e-05, 
       6.1875966647146984e-05, 
       6.2656048915290402e-05, 
       6.9455799336008616e-05, 
       6.3509142853422293e-05, 
       8.4165599336841567e-05, 
       6.5837873033036814e-05, 
       7.0334436211899184e-05, 
       6.5236523031053658e-05, 
       6.0099663041440669e-05, 
       7.0375958498903229e-05, 
       8.4534356391229598e-05, 
       5.9978676897205998e-05, 
       6.9352405675766616e-05, 
       8.2129373715994958e-05, 
       6.9250354323117412e-05, 
       8.3618278997218935e-05, 
       8.0262838317054431e-05, 
       6.2924942174253610e-05, 
       8.0056477126931925e-05, 
       6.5357253986208318e-05, 
       5.9785902636241289e-05, 
       6.6188061200899868e-05, 
       7.1789604838212270e-05, 
       6.6463620188607816e-05, 
       8.0000000000003124e-05, 
       5.9494194718290323e-05, 
       6.9269312237852683e-05, 
       6.2839486830890053e-05, 
       6.9892470254323018e-05, 
       6.4379924004785054e-05, 
       7.0000000000003775e-05, 
       6.3807666750067207e-05, 
       6.3807666750067207e-05, 
       7.9095219842891071e-05, 
       8.2064661199463269e-05, 
       8.2064661199463269e-05, 
       7.9379102557832601e-05, 
       8.2816845401768909e-05, 
       6.2508309958354602e-05, 
       7.3492051927928490e-05, 
       6.2781101965321787e-05, 
       6.8974160796969741e-05, 
       8.2294074131408675e-05, 
       5.8776261333645033e-05, 
       6.3824931788326081e-05, 
       8.0863470241769597e-05, 
       5.9431122093465811e-05, 
       7.1788642816617357e-05, 
       6.1675079650201726e-05, 
       7.5470561541733706e-05, 
       6.1744713543348279e-05, 
       6.8335733149314020e-05, 
       7.6288066887247814e-05, 
       7.1010012696606569e-05, 
       8.1997524637719208e-05, 
       5.8366983809388220e-05, 
       5.8366983809388220e-05, 
       6.8232768989836943e-05, 
       8.1813898007016036e-05, 
       6.1331950040077464e-05, 
       6.1205225652951112e-05, 
       6.3485478666154110e-05, 
       6.8544802861219726e-05, 
       8.1970632755317031e-05, 
       8.1032255079964386e-05, 
       6.6961079287711025e-05, 
       7.0905303170765307e-05, 
       7.8662426845532715e-05, 
       6.7102272783884330e-05, 
       6.9982376783404344e-05, 
       5.7975696182506025e-05, 
       6.3022708218244474e-05, 
       7.5898089094974302e-05, 
       8.1394156946261678e-05, 
       7.9051458386031077e-05, 
       5.7792895408829065e-05, 
       6.7118288969200677e-05, 
       6.7113672802829189e-05, 
       5.8487350733031008e-05, 
       5.9060272254824622e-05, 
       6.6685573746479691e-05, 
       8.0884363748094789e-05, 
       6.3406352412302854e-05, 
       7.9168902861447130e-05, 
       6.8101999415719065e-05, 
       6.9999999999997649e-05, 
       7.9756846428443462e-05, 
       5.9466997906205094e-05, 
       6.1798035990260014e-05, 
       5.7525640937074250e-05, 
       6.7598303827360432e-05, 
       8.1123107197590081e-05, 
       6.0857602220587515e-05, 
       5.7238536089913648e-05, 
       6.0296108823403312e-05, 
       8.0462708232992357e-05, 
       5.7138218636255578e-05, 
       6.6744343168005023e-05, 
       6.9861326793764752e-05, 
       6.6448058852273747e-05, 
       8.0590379451377070e-05, 
       5.9301420977033479e-05, 
       6.3606252338734910e-05, 
       7.1561444912891957e-05, 
       5.8971531988847866e-05, 
       5.9962708678099615e-05, 
       5.8424391314053509e-05, 
       7.5044372478503521e-05, 
       5.6640882457554580e-05, 
       6.0634628650502093e-05, 
       7.9999999999997811e-05, 
       8.0000000000003124e-05, 
       7.9999999999998041e-05, 
       5.8762804664599435e-05, 
       6.8300553729258890e-05, 
       7.9756170013425904e-05, 
       7.6111421953117375e-05, 
       6.6791153639730893e-05, 
       5.8711633022558357e-05, 
       7.7190981078900111e-05, 
       6.7565975060514434e-05, 
       7.4409631614740109e-05, 
       6.8019637361911608e-05, 
       6.0000638740918991e-05, 
       5.9683060808299142e-05, 
       6.0000000000001716e-05, 
       6.0656444536213920e-05, 
       6.3315654994675305e-05, 
       7.9117751964661310e-05, 
       6.2847507467736645e-05, 
       6.1127345864194760e-05, 
       7.7953012488288023e-05, 
       6.2726754227690335e-05, 
       6.4457252889868805e-05, 
       7.0603219934318304e-05, 
       7.1793466863569297e-05, 
       6.6246300211220808e-05, 
       6.4800508456150708e-05, 
       5.6820814389342633e-05, 
       7.8429482158662094e-05, 
       6.4466112565549793e-05, 
       5.9425728391087322e-05, 
       7.7760116444469343e-05, 
       6.0573059801935110e-05, 
       6.7982975409914406e-05, 
       7.6974700191106334e-05, 
       7.0682061975435999e-05, 
       5.8324262355301776e-05, 
       5.6861440151489735e-05, 
       5.9520686541217098e-05, 
       7.7808551080479926e-05, 
       7.3014516071128041e-05, 
       7.0318610983867639e-05, 
       5.8341079890452817e-05, 
       5.8038404263407569e-05, 
       6.4192756093646431e-05, 
       5.5722688434389153e-05, 
       6.0917992163542404e-05, 
       6.5511922452528984e-05, 
       6.7905103955939297e-05, 
       6.0887668518765243e-05, 
       5.6455953970968241e-05, 
       6.4089790272093156e-05, 
       6.0166703378302190e-05, 
       6.0349615780373958e-05, 
       5.6531897328182399e-05, 
       5.5284361232208792e-05, 
       6.9157879984307118e-05, 
       7.7250984899385485e-05, 
       5.5904387553900388e-05, 
       5.7329476689326737e-05, 
       7.7177017699276516e-05, 
       6.3677988053574579e-05, 
       5.6657218670243469e-05, 
       6.1500274095199829e-05, 
       6.3525714763286182e-05, 
       6.4047466377773604e-05, 
       6.8746615557381972e-05, 
       7.6796009738558499e-05, 
       5.7960663415198631e-05, 
       5.6616898916124508e-05, 
       5.8507179406186414e-05, 
       5.8507179406186414e-05, 
       6.4286001971403442e-05, 
       6.3426068153556473e-05, 
       5.7427829944274808e-05, 
       6.9482822627741069e-05, 
       5.6218833671767182e-05, 
       5.6244821096145556e-05, 
       7.6559378599703979e-05, 
       5.9882412663587851e-05, 
       7.1769351834248960e-05, 
       6.3528202858453226e-05, 
       6.3308141831010801e-05, 
       5.9748704434500374e-05, 
       6.3407501594750623e-05, 
       5.5297201724374376e-05, 
       5.5294165843614736e-05, 
       5.3890286553868552e-05, 
       6.2827180672443735e-05, 
       6.2672038745459056e-05, 
       6.7627502188829497e-05, 
       6.9999999999996646e-05, 
       6.9999999999996646e-05, 
       7.0000000000003233e-05, 
       6.9999999999996511e-05, 
       6.9999999999999316e-05, 
       6.2931919908768894e-05, 
       5.6019709267986371e-05, 
       7.2411777775890373e-05, 
       5.3893704701277225e-05, 
       5.4919084356384583e-05, 
       7.2443204584907620e-05, 
       7.5700210033304168e-05, 
       5.4039625586445665e-05, 
       6.0057962594511456e-05, 
       6.2994190541505941e-05, 
       5.6553487353165438e-05, 
       7.0416897561046345e-05, 
       6.9018146650023175e-05, 
       5.4298339545751332e-05, 
       5.7566113457668347e-05, 
       5.7566113457668347e-05, 
       5.3896434824613023e-05, 
       6.2324006726300040e-05, 
       5.6245938035163895e-05, 
       5.6262876609590483e-05, 
       6.2064385542956658e-05, 
       7.3325508121557261e-05, 
       5.5336294369211569e-05, 
       5.4688918171822151e-05, 
       6.3016566473149727e-05, 
       7.4044467066409106e-05, 
       5.3847918139291220e-05, 
       6.1608144883388853e-05, 
       7.3829185564994795e-05, 
       6.1005953027268596e-05, 
       7.4563937119334210e-05, 
       6.1510643188956386e-05, 
       5.5585613459319514e-05, 
       5.3127657690257386e-05, 
       5.6854379222837663e-05, 
       7.4175647619881930e-05, 
       6.5352207425744773e-05, 
       5.6796512516564551e-05, 
       7.4141386918140235e-05, 
       5.9204649968848753e-05, 
       5.9953192340170565e-05, 
       5.6410428452317926e-05, 
       7.3611779026043771e-05, 
       7.3702367280026061e-05, 
       6.4411599407668551e-05, 
       6.3237997602772381e-05, 
       6.1536848685836765e-05, 
       5.2594819126674162e-05, 
       7.3546635520325198e-05, 
       5.2931680708235823e-05, 
       5.2434336116942445e-05, 
       7.3862602501210007e-05, 
       6.2123229596222678e-05, 
       6.1848973622720174e-05, 
       6.3386964907555081e-05, 
       5.4880455407284498e-05, 
       6.9170969299737822e-05, 
       7.0824123912965892e-05, 
       6.5608553700060505e-05, 
       6.9475698371029295e-05, 
       6.2070855503402149e-05, 
       5.4079809617853081e-05, 
       6.3769022162777749e-05, 
       6.2041610638332377e-05, 
       7.3240232758743496e-05, 
       7.2387522457684276e-05, 
       5.4112252607867139e-05, 
       5.9938290045320255e-05, 
       6.1265594645847256e-05, 
       7.2902188087098116e-05, 
       5.1714355001555184e-05, 
       6.1534209538043489e-05, 
       5.1599521823857750e-05, 
       6.8392806341544348e-05, 
       6.8557027230420028e-05, 
       5.9928672066695453e-05, 
       6.2223668755738873e-05, 
       6.0284898499818994e-05, 
       6.0647853444238224e-05, 
       5.4503511645400286e-05, 
       6.1100329371434364e-05, 
       5.9977443388786832e-05, 
       5.7196899156338021e-05, 
       5.7246132412459700e-05, 
       6.0812071135376466e-05, 
       6.0147509446240176e-05, 
       5.8028346182608723e-05, 
       5.8200288752660364e-05, 
       7.0392074591777795e-05, 
       5.4374901298990524e-05, 
       6.8852269191558974e-05, 
       6.8363485470761663e-05, 
       5.3203764216720171e-05, 
       6.8735722966409624e-05, 
       6.0113304328114580e-05, 
       7.1798976822536655e-05, 
       5.5434889335857634e-05, 
       5.8057602646373637e-05, 
       6.0236948424941417e-05, 
       5.7334825816418568e-05, 
       5.9584571208184333e-05, 
       5.3882311409436307e-05, 
       5.7975696182496064e-05, 
       5.8642811210191921e-05, 
       5.1021322393804143e-05, 
       5.7375966704019330e-05, 
       5.2165052232240396e-05, 
       7.0405629308618333e-05, 
       5.5814974730323309e-05, 
       6.5115786385638128e-05, 
       6.3191980801011519e-05, 
       6.8302492030051913e-05, 
       5.2946253100227508e-05, 
       6.9864399118165513e-05, 
       5.3519200528919703e-05, 
       6.0916619645259132e-05, 
       5.3353886478213653e-05, 
       5.1644057940744707e-05, 
       5.3311394153092751e-05, 
       5.0602562952945810e-05, 
       7.0888653660611630e-05, 
       6.9937350784747459e-05, 
       5.1664075736193493e-05, 
       5.3144158950403131e-05, 
       5.2999415799077513e-05, 
       6.4934755082764894e-05, 
       5.2657387756615676e-05, 
       5.2653647353310184e-05, 
       5.1741199062695796e-05, 
       5.2653886622445735e-05, 
       5.6413937181778075e-05, 
       4.9834537437369338e-05, 
       5.8527060264594480e-05, 
       5.2784083479654844e-05, 
       4.9999999999996181e-05, 
       6.8656869243822620e-05, 
       5.4785954426778778e-05, 
       5.7572845968130187e-05, 
       5.1307851903693580e-05, 
       7.0000000000004656e-05, 
       6.9999999999996253e-05, 
       7.0000000000003816e-05, 
       6.9999999999997053e-05, 
       7.0000000000001241e-05, 
       7.0000000000004182e-05, 
       7.0000000000001634e-05, 
       5.8100337514727283e-05, 
       5.0617390845403608e-05, 
       5.7077093981928396e-05, 
       6.6510501506377508e-05, 
       6.8447159918870196e-05, 
       6.8447159918870196e-05, 
       4.9998744397459931e-05, 
       5.9054110827512336e-05, 
       5.9054110827512336e-05, 
       4.9389018096962586e-05, 
       4.9469735152469814e-05, 
       5.0342980099344789e-05, 
       5.4096185903121902e-05, 
       6.8685210102147794e-05, 
       6.0739405471969981e-05, 
       5.0429677673274736e-05, 
       5.1596283397158637e-05, 
       6.9249303104309163e-05, 
       5.8668153290639881e-05, 
       6.5011050600263540e-05, 
       5.7681362495392439e-05, 
       5.1684670303987461e-05, 
       5.4190116214411712e-05, 
       6.2124236458825180e-05, 
       5.9026051417254481e-05, 
       6.7160770062276668e-05, 
       4.9216667694980751e-05, 
       6.7431874955774484e-05, 
       6.8571958268171145e-05, 
       5.0168777849553552e-05, 
       5.1240079225168537e-05, 
       5.2522492438199609e-05, 
       6.4444120753412486e-05, 
       5.1267950200395691e-05, 
       6.8552900247067109e-05, 
       5.5656780677968868e-05, 
       5.5645455789924967e-05, 
       6.6998902364645202e-05, 
       6.6106509933870419e-05, 
       4.8621134524977781e-05, 
       5.8327317960217514e-05, 
       6.8301317710511161e-05, 
       5.7739296754118536e-05, 
       5.6733561854340648e-05, 
       5.1907945724487169e-05, 
       6.7283603364102861e-05, 
       5.6086116077882290e-05, 
       5.3779628583649319e-05, 
       5.3773679883132576e-05, 
       5.7686250100676412e-05, 
       5.0756612126910737e-05, 
       5.9319218617168347e-05, 
       5.0996911751667910e-05, 
       5.1141434156633527e-05, 
       4.8232975294252025e-05, 
       6.7428905515221459e-05, 
       5.8075042220353883e-05, 
       5.0432445481483273e-05, 
       4.9723118600349544e-05, 
       6.7183023427300041e-05, 
       5.4418520495354878e-05, 
       5.1443190104135106e-05, 
       6.6479891987814772e-05, 
       4.8154488972058231e-05, 
       5.5414685218746776e-05, 
       5.8494536689008982e-05, 
       5.8214610480763870e-05, 
       6.1003932903509196e-05, 
       5.3989681249441327e-05, 
       5.4586888863709764e-05, 
       5.8011255465076777e-05, 
       4.9364794583557686e-05, 
       5.1588024520070403e-05, 
       5.0856798221431948e-05, 
       6.2523876992712051e-05, 
       4.7506758944440729e-05, 
       4.9736843766018602e-05, 
       4.8065538513162212e-05, 
       4.7843284337940138e-05, 
       6.6473967767005488e-05, 
       5.2119362124862120e-05, 
       5.5300147961083892e-05, 
       6.6494360747320299e-05, 
       6.6335737702639760e-05, 
       5.7565817985701589e-05, 
       4.8227124170123190e-05, 
       4.7959157223835262e-05, 
       5.5645120143779840e-05, 
       4.7924452431526926e-05, 
       4.7423306455270235e-05, 
       4.7850512381542845e-05, 
       5.3317661523466849e-05, 
       4.6999213151127284e-05, 
       6.5821931330637358e-05, 
       4.7430464499435272e-05, 
       4.6749555086998883e-05, 
       6.4608627553019386e-05, 
       4.8817519166752012e-05, 
       4.9658352765876390e-05, 
       5.7645430296232884e-05, 
       4.6513084145452088e-05, 
       4.6838328945944628e-05, 
       4.7518495244190778e-05, 
       4.7618079388639201e-05, 
       5.0937290716147252e-05, 
       4.6362387121220958e-05, 
       5.3112616514234598e-05, 
       4.7879863419670559e-05, 
       4.9802434052588395e-05, 
       6.4047065533614714e-05, 
       5.9880919513138390e-05, 
       4.9348575576909331e-05, 
       6.3797636223218260e-05, 
       5.3536503033076941e-05, 
       5.9634135939101574e-05, 
       5.3097350538854118e-05, 
       5.9999999999996844e-05, 
       5.4109364968479185e-05, 
       6.5108120875489320e-05, 
       5.9999999999998911e-05, 
       5.9999999999999168e-05, 
       5.9999999999995238e-05, 
       4.8253657136784224e-05, 
       4.6109176409349282e-05, 
       5.3323032427169881e-05, 
       4.9999999999997468e-05, 
       5.7746632105877832e-05, 
       4.8377566650179844e-05, 
       5.4019740675487589e-05, 
       5.4000527635045577e-05, 
       4.6984094691538928e-05, 
       5.5319634681187180e-05, 
       5.3399425666767305e-05, 
       4.8307957224223141e-05, 
       4.7218545037903774e-05, 
       5.0631322423297977e-05, 
       5.0624074626294180e-05, 
       4.6850544447525787e-05, 
       4.8097482961055307e-05, 
       4.8765284896953920e-05, 
       4.7788423942490573e-05, 
       5.7455590908959220e-05, 
       4.6541788185137160e-05, 
       6.2021619410128123e-05, 
       4.6007769183253639e-05, 
       6.3937193326120650e-05, 
       4.7630012076023683e-05, 
       6.4196456516951337e-05, 
       5.3232603550631491e-05, 
       6.4087197217880954e-05, 
       5.0976349119103704e-05, 
       4.8875673172377815e-05, 
       4.8200134931586160e-05, 
       5.0124516705383487e-05, 
       5.4866728612423982e-05, 
       5.4866728612423982e-05, 
       5.9687931585041279e-05, 
       4.9330618157882605e-05, 
       6.1015843689727393e-05, 
       5.2630713005819828e-05, 
       5.2470751223487250e-05, 
       5.1073463061904187e-05, 
       4.5159503742061857e-05, 
       5.6154056129171438e-05, 
       5.0465236947646229e-05, 
       5.8434605489657229e-05, 
       4.7082997248536849e-05, 
       5.2198062759797356e-05, 
       5.2923930366113863e-05, 
       4.6060975567785310e-05, 
       5.2841420701956051e-05, 
       6.2491473727720797e-05, 
       5.7672817453169587e-05, 
       4.9404427366540868e-05, 
       5.8078493652076541e-05, 
       5.5926724226921882e-05, 
       6.3285114004472097e-05, 
       6.1784187357510179e-05, 
       4.7228985500174511e-05, 
       4.7113106982599450e-05, 
       5.3329567001219627e-05, 
       5.1871969822082802e-05, 
       4.6806993844226478e-05, 
       4.5818422311003863e-05, 
       4.4764314160833883e-05, 
       4.6456947431001861e-05, 
       6.0540078197131089e-05, 
       4.8129644499075344e-05, 
       6.2428807946059644e-05, 
       5.2720757616892463e-05, 
       5.1738068980384296e-05, 
       6.1358427502498385e-05, 
       6.0624179763475773e-05, 
       6.1021659638661413e-05, 
       6.0452373358306819e-05, 
       4.4438743967969367e-05, 
       4.7752626282834979e-05, 
       5.9874578568506739e-05, 
       5.8858099541360165e-05, 
       4.9339661552883210e-05, 
       6.0290955447593425e-05, 
       6.0309509062758366e-05, 
       4.5585460009912667e-05, 
       6.2440432538608045e-05, 
       5.3620357713741222e-05, 
       6.2315065419913561e-05, 
       4.9233958178254318e-05, 
       6.0076291355083193e-05, 
       6.2128921490484584e-05, 
       4.7738966871028020e-05, 
       4.3988800867721600e-05, 
       4.6944177263479723e-05, 
       6.2120087390109697e-05, 
       4.7251966613467354e-05, 
       6.0194835224410237e-05, 
       6.0867259815089058e-05, 
       4.5229456315023511e-05, 
       4.5157064301636784e-05, 
       6.1696455166936233e-05, 
       5.3514139841055439e-05, 
       4.6897377446184051e-05, 
       5.2113667852258651e-05, 
       4.6065840048162590e-05, 
       6.0823189549657414e-05, 
       5.1847350888108032e-05, 
       4.6381100856351667e-05, 
       5.2265540931804702e-05, 
       4.5052626372544749e-05, 
       6.0440976732303250e-05, 
       4.4027916341510338e-05, 
       5.1235609203907210e-05, 
       5.8710147614769819e-05, 
       4.4934320706960839e-05, 
       4.8119410817917368e-05, 
       5.5641523552435012e-05, 
       5.1101042289822581e-05, 
       5.2398200281830565e-05, 
       5.2931680708258720e-05, 
       4.9768711920962139e-05, 
       5.9234396024083023e-05, 
       6.1209533575310674e-05, 
       5.9729449251660013e-05, 
       4.5709644395046836e-05, 
       5.3762909074445581e-05, 
       4.4386569573943469e-05, 
       5.2241653813088457e-05, 
       4.4194772821727407e-05, 
       4.4699757371715209e-05, 
       5.0497012037485661e-05, 
       4.5619810615109919e-05, 
       5.0614725124806547e-05, 
       4.3042300421031882e-05, 
       4.9263696735815763e-05, 
       4.8597369319794204e-05, 
       4.8321010540245448e-05, 
       5.0429327235622207e-05, 
       4.3781619241207234e-05, 
       4.3781619241207234e-05, 
       5.9283570582830531e-05, 
       5.0351183872548351e-05, 
       4.8380534997354775e-05, 
       5.9530333672816625e-05, 
       5.5180526868101322e-05, 
       4.9013062915240597e-05, 
       6.0391161831427170e-05, 
       4.4383839567870722e-05, 
       4.2940746688550135e-05, 
       4.3474940166716923e-05, 
       5.0114361240644861e-05, 
       4.5343776830733037e-05, 
       5.2233863204528452e-05, 
       6.0239894088239741e-05, 
       4.3602646585199418e-05, 
       5.9989974563351431e-05, 
       5.7236046982533619e-05, 
       4.5061389909857739e-05, 
       5.6680429292107721e-05, 
       4.2719784714483235e-05, 
       5.1847467408943452e-05, 
       6.0000000000003891e-05, 
       6.0000000000003186e-05, 
       5.9999999999997413e-05, 
       5.9999999999995699e-05, 
       5.9999999999995699e-05, 
       5.9999999999997074e-05, 
       6.0000000000002848e-05, 
       6.0000000000004203e-05, 
       5.9999999999900750e-05, 
       5.9948083493238083e-05, 
       5.1538275823445037e-05, 
       4.4255687506578091e-05, 
       5.9687917190791767e-05, 
       4.2385805422990165e-05, 
       5.2561308939517306e-05, 
       4.2989777661969736e-05, 
       5.8866115201010490e-05, 
       4.2585151442718195e-05, 
       5.7806800652732524e-05, 
       5.8538519496124681e-05, 
       4.7419101157292518e-05, 
       5.6969026218376831e-05, 
       4.2904941056530595e-05, 
       4.9468965792524989e-05, 
       4.2884982051612175e-05, 
       5.9418379806964409e-05, 
       4.8038549640988393e-05, 
       5.1413515079533608e-05, 
       5.6414144854206245e-05, 
       5.6698254084957370e-05, 
       5.6859133973873823e-05, 
       4.2238588668818394e-05, 
       5.0523661233750111e-05, 
       4.1859565026545551e-05, 
       4.2300453603142560e-05, 
       5.8357816852680722e-05, 
       4.2896636966194901e-05, 
       4.2890995066705979e-05, 
       4.3605593670430775e-05, 
       4.9078952402252231e-05, 
       4.2547482406794759e-05, 
       4.6021705635717682e-05, 
       5.7628952713722120e-05, 
       5.8899780859377870e-05, 
       4.4889126964744352e-05, 
       5.1167149777947389e-05, 
       4.9305332930358910e-05, 
       5.8059534537816362e-05, 
       5.2312466563373815e-05, 
       5.8505893198456180e-05, 
       5.8505619902993483e-05, 
       5.2485315478166272e-05, 
       4.7151073993626523e-05, 
       4.8947919031059924e-05, 
       5.8665593834410141e-05, 
       5.2712147925044101e-05, 
       5.0129217704798939e-05, 
       5.6163478919564309e-05, 
       4.9046925053080419e-05, 
       5.5347711415919464e-05, 
       4.7132881047904530e-05, 
       4.8136558642189787e-05, 
       4.3567204893986439e-05, 
       5.1005642257679023e-05, 
       4.8942600397635363e-05, 
       5.5565091641099407e-05, 
       5.3598968719123090e-05, 
       5.5623545885036493e-05, 
       4.7516740701371410e-05, 
       4.1400926093465427e-05, 
       4.6181922045644870e-05, 
       4.2158746517562697e-05, 
       4.4091303286996512e-05, 
       5.0933370057951013e-05, 
       4.5149131671974665e-05, 
       4.9185432993972400e-05, 
       4.2454629609651406e-05, 
       4.7020820823160696e-05, 
       4.3156904316442969e-05, 
       4.7217784961223787e-05, 
       4.2249633424153557e-05, 
       5.6559207837114271e-05, 
       4.4753813000411455e-05, 
       4.1942096590134538e-05, 
       4.4878995494326498e-05, 
       5.1082037872257545e-05, 
       4.1370599272740181e-05, 
       4.5124591943925040e-05, 
       5.7708008312763574e-05, 
       5.5049821223998331e-05, 
       4.5036977994987573e-05, 
       4.9956201747369401e-05, 
       5.0054996184009183e-05, 
       4.7978382753679209e-05, 
       4.7969420649407912e-05, 
       5.6401804604434095e-05, 
       4.7177734946549771e-05, 
       4.4469918696663561e-05, 
       4.2841763948950139e-05, 
       4.7915081188819933e-05, 
       5.6747801600031691e-05, 
       5.7131826878926684e-05, 
       4.5405496784351432e-05, 
       4.8625757495847274e-05, 
       4.9110659013475059e-05, 
       4.5443529919864918e-05, 
       5.6601929593734138e-05, 
       4.0449617628153772e-05, 
       4.1915434938599752e-05, 
       5.7004515463664237e-05, 
       5.5305299041923107e-05, 
       5.0300022670240464e-05, 
       4.7494894750542188e-05, 
       5.6302032533584690e-05, 
       5.6641529826804034e-05, 
       4.2285597655103947e-05, 
       4.4989890723973792e-05, 
       4.7160673033362612e-05, 
       5.1696973321681622e-05, 
       4.7739698725242445e-05, 
       4.7540566308481340e-05, 
       4.2743314203602834e-05, 
       4.0262983115625070e-05, 
       4.9386451377604352e-05, 
       4.0140837214534677e-05, 
       5.1454725847116135e-05, 
       4.2722855581471416e-05, 
       5.5838535488226382e-05, 
       4.9076297639682089e-05, 
       4.7441798013771622e-05, 
       4.5468000876504170e-05, 
       4.0045406906395053e-05, 
       4.8184698246698909e-05, 
       4.7098313696511857e-05, 
       4.0746250395595987e-05, 
       4.6636441632854370e-05, 
       4.8941346549806990e-05, 
       4.1910580657598172e-05, 
       3.9800093001322042e-05, 
       4.0877082420779330e-05, 
       4.1650704997486722e-05, 
       5.2622782291357465e-05, 
       3.9999999999998031e-05, 
       4.1034222836968937e-05, 
       4.3064016324120847e-05, 
       4.0626714689814220e-05, 
       4.3145765424439943e-05, 
       5.5687005889193095e-05, 
       4.0992368987462888e-05, 
       4.7778752397519893e-05, 
       4.7639866042398090e-05, 
       4.7873569030883523e-05, 
       5.3971749688348641e-05, 
       5.4411539910846220e-05, 
       4.1603394095032366e-05, 
       4.2833403445612206e-05, 
       5.3251208267986931e-05, 
       5.5025640024275558e-05, 
       4.0000000000004340e-05, 
       5.5494098897059034e-05, 
       4.6139240744064400e-05, 
       4.5576472255746050e-05, 
       4.1611469644164119e-05, 
       5.2205504717807511e-05, 
       5.0791540425267534e-05, 
       4.2424390204096938e-05, 
       4.4758167671899552e-05, 
       5.3117345831586268e-05, 
       4.2891161374582391e-05, 
       4.8004886649059284e-05, 
       4.8876657057956124e-05, 
       5.3566000052228521e-05, 
       5.4090499508276584e-05, 
       5.3379005510433491e-05, 
       3.9612001846744409e-05, 
       4.5551545149589090e-05, 
       4.4784831355082788e-05, 
       3.9108989711492461e-05, 
       4.0217885604607888e-05, 
       4.2391122145023589e-05, 
       4.2686663146066188e-05, 
       4.4476303476468326e-05, 
       4.0302573238981099e-05, 
       4.0481598641152580e-05, 
       5.4428501946752183e-05, 
       3.9532007940809496e-05, 
       3.8889093760938791e-05, 
       4.0969260975792362e-05, 
       4.6633414975083388e-05, 
       3.9362044300533755e-05, 
       3.9486078947276614e-05, 
       4.2462852854935404e-05, 
       4.0096948140222940e-05, 
       4.6705500429090406e-05, 
       5.1855634004080115e-05, 
       5.1252570390837694e-05, 
       5.4128198700289432e-05, 
       4.3943022516342919e-05, 
       4.2688429020360587e-05, 
       5.0290260734504572e-05, 
       4.9919596171507846e-05, 
       4.6782337339548067e-05, 
       4.6880460523404891e-05, 
       4.3731240612891365e-05, 
       4.1450286808754173e-05, 
       5.4503851490524109e-05, 
       4.7109779582227649e-05, 
       4.0870069770793465e-05, 
       4.2989358898683530e-05, 
       3.9119879430034020e-05, 
       5.0000000000001934e-05, 
       4.9999999999996181e-05, 
       4.9999999999996086e-05, 
       4.9999999999998091e-05, 
       3.9779839354899834e-05, 
       3.8423856351481847e-05, 
       5.2585713569939680e-05, 
       5.1355987448947586e-05, 
       5.3352343143186026e-05, 
       4.4293675235150309e-05, 
       5.3776890587359726e-05, 
       4.5340384718832403e-05, 
       4.5340384718832403e-05, 
       3.9596292637217605e-05, 
       3.8174969058300859e-05, 
       4.4748080799978564e-05, 
       4.1151237941040358e-05, 
       4.7755455282975009e-05, 
       4.6239999925812488e-05, 
       5.2473309178642368e-05, 
       3.9555565339275648e-05, 
       3.8142366730662753e-05, 
       3.8142366730662753e-05, 
       4.3576118269370213e-05, 
       4.4163713822123261e-05, 
       3.8519337872669541e-05, 
       4.7484368567342283e-05, 
       3.9723295223953362e-05, 
       4.4312333030711276e-05, 
       4.7326555183203576e-05, 
       4.3944811997648134e-05, 
       4.3533866174062588e-05, 
       4.1128293337820854e-05, 
       4.4264398543342156e-05, 
       4.5231235678463149e-05, 
       3.7974214225209884e-05, 
       4.0816353711432150e-05, 
       3.8254467210912872e-05, 
       3.9612293794954706e-05, 
       5.1289969748955931e-05, 
       3.8204355001303550e-05, 
       5.1429020707299141e-05, 
       4.3865029279998410e-05, 
       4.2589526891569486e-05, 
       4.4070767503813843e-05, 
       3.8557692981571585e-05, 
       5.1663683487387737e-05, 
       3.8367436677400796e-05, 
       5.0849740797423859e-05, 
       4.0000000000000830e-05, 
       5.0898469445921683e-05, 
       4.3902023613929762e-05, 
       3.9155327739404640e-05, 
       4.3718353705104801e-05, 
       4.4468686884197946e-05, 
       4.4468686884197946e-05, 
       5.1030169856757265e-05, 
       3.9999999999994413e-05, 
       4.0784369845711627e-05, 
       5.2589590232619284e-05, 
       4.4544976562195326e-05, 
       4.7145326972658432e-05, 
       4.4844963114920864e-05, 
       5.2120749894353478e-05, 
       4.3430922795922550e-05, 
       3.7242177299827923e-05, 
       4.2603831393923695e-05, 
       4.3810479862621382e-05, 
       5.2466497157731773e-05, 
       3.7256022446790273e-05, 
       4.2988144478824435e-05, 
       3.7193453687580723e-05, 
       4.3238059411697453e-05, 
       3.7016537498390827e-05, 
       5.2234932880688905e-05, 
       4.5267813064039461e-05, 
       4.4712517755245851e-05, 
       3.7000148022141180e-05, 
       4.7615779704774728e-05, 
       3.8886250063827352e-05, 
       3.6899640180051325e-05, 
       5.0782384036866160e-05, 
       4.4393359892921203e-05, 
       4.1135690655529002e-05, 
       4.9141935157055799e-05, 
       3.7329087888352542e-05, 
       5.1910726772086912e-05, 
       4.2882121064123977e-05, 
       4.9030536298102691e-05, 
       4.3371776100530395e-05, 
       3.9471983870651639e-05, 
       3.7349716501486679e-05, 
       5.1746950144655407e-05, 
       4.0394638474618180e-05, 
       4.6630404355709410e-05, 
       4.1512075312213028e-05, 
       4.1041243074259562e-05, 
       4.1470642074475918e-05, 
       4.0473054842158006e-05, 
       4.0635957593095788e-05, 
       4.2449354969984348e-05, 
       4.3989186256937562e-05, 
       3.6720931065664631e-05, 
       4.2209893969995383e-05, 
       3.9708942725034822e-05, 
       3.6534787555024640e-05, 
       5.0610504843381534e-05, 
       4.2978997488036201e-05, 
       4.9725059033144498e-05, 
       4.5539627719646152e-05, 
       4.8658601913449700e-05, 
       4.3662347596505869e-05, 
       4.5236029450620008e-05, 
       3.9999999999999956e-05, 
       3.6289585155460508e-05, 
       4.3123337913771949e-05, 
       4.2900455249023130e-05, 
       5.1022865399885616e-05, 
       5.1056897295305156e-05, 
       5.0201679932823080e-05, 
       4.5035755516383509e-05, 
       4.5742678592749948e-05, 
       3.6504866171755416e-05, 
       4.3696555871514647e-05, 
       3.6054507801335057e-05, 
       3.7956799054945251e-05, 
       4.5320516591022333e-05, 
       3.6023644937291720e-05, 
       3.8894940278020020e-05, 
       4.8243389571334880e-05, 
       4.7270104165723254e-05, 
       3.7872248784148562e-05, 
       3.5990844969634747e-05, 
       4.1400377003497217e-05, 
       5.0762440896385007e-05, 
       3.7387501437768796e-05, 
       5.0533824346019972e-05, 
       3.7636387073252496e-05, 
       3.7636387073252496e-05, 
       3.6930192577231878e-05, 
       3.7143969475281460e-05, 
       4.7936270874557962e-05, 
       4.2117955653727522e-05, 
       4.3294910049706541e-05, 
       4.9523858616957549e-05, 
       3.5788225525521872e-05, 
       3.5788225525521872e-05, 
       4.0998361333740045e-05, 
       3.9374118396399936e-05, 
       4.1967932715282660e-05, 
       4.0950171851870644e-05, 
       4.3502961137135996e-05, 
       4.6706950447247980e-05, 
       4.7003431790866724e-05, 
       5.0211960719528261e-05, 
       4.8496757365635496e-05, 
       4.9383251163863035e-05, 
       4.1782909511288393e-05, 
       4.5226377126599348e-05, 
       4.3331301718420514e-05, 
       3.5607520520066936e-05, 
       4.3584694709283523e-05, 
       4.1860315866986073e-05, 
       3.9464747440740120e-05, 
       4.2839836375224111e-05, 
       3.8053233704079882e-05, 
       4.3102572869320559e-05, 
       3.8356422874175486e-05, 
       3.6332911632079705e-05, 
       4.9502101644929617e-05, 
       3.5963125627094130e-05, 
       4.1222812894943381e-05, 
       4.3888207697742621e-05, 
       4.5787608236944982e-05, 
       4.8525729897417938e-05, 
       5.0000000000002923e-05, 
       4.9999999999997129e-05, 
       5.0000000000002157e-05, 
       5.0000000000003452e-05, 
       5.0000000000001378e-05, 
       4.9999999999998464e-05, 
       5.0000000000003363e-05, 
       4.9999999999999318e-05, 
       4.9999999999995395e-05, 
       4.9999999999995164e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       4.9633443533143397e-05, 
       3.6146040313024670e-05, 
       3.7186596895147424e-05, 
       4.1434351842659767e-05, 
       4.9671232623830673e-05, 
       4.1086755648298457e-05, 
       3.7176566014276475e-05, 
       3.6751636140254799e-05, 
       4.6944283266381702e-05, 
       3.7165895702074299e-05, 
       3.6225040908503689e-05, 
       3.8811100700933701e-05, 
       4.1518995666847782e-05, 
       3.6093810752093189e-05, 
       4.1260527577609158e-05, 
       3.6368625575969663e-05, 
       3.5601094917615441e-05, 
       4.9493889965562862e-05, 
       4.9390770584311340e-05, 
       4.9434877379427027e-05, 
       3.5811564067132317e-05, 
       3.6809148280972258e-05, 
       4.7011955014847065e-05, 
       3.4941977009063790e-05, 
       3.4892160456408059e-05, 
       4.2030988622430125e-05, 
       4.2713454725133798e-05, 
       4.0187044306381169e-05, 
       4.1743879995633237e-05, 
       4.9248450632993391e-05, 
       4.9166442089218142e-05, 
       3.4969556426075690e-05, 
       3.6282312082767726e-05, 
       4.8464322045901105e-05, 
       3.8432721766409586e-05, 
       3.5083913374717896e-05, 
       4.1023984914919550e-05, 
       3.5881594253143910e-05, 
       4.8187367855616964e-05, 
       4.8993859646157952e-05, 
       4.6568319271275242e-05, 
       4.2273591652999149e-05, 
       4.8957568324896678e-05, 
       4.8728911755346141e-05, 
       3.7413193024207677e-05, 
       4.0805710255231961e-05, 
       4.2296555324443978e-05, 
       4.3744170080442670e-05, 
       4.8864927515826213e-05, 
       3.8945859482717501e-05, 
       3.4758231754809261e-05, 
       4.7063404474244042e-05, 
       4.2123198552018641e-05, 
       4.0392903234596108e-05, 
       4.6872510806139494e-05, 
       3.8100115863702311e-05, 
       4.8168472029125805e-05, 
       4.7997072200865581e-05, 
       3.7972044715614297e-05, 
       3.7750564708231690e-05, 
       3.4485404745062308e-05, 
       4.6543856886749265e-05, 
       3.4735257490871966e-05, 
       4.8568518935912545e-05, 
       4.6055982656744297e-05, 
       3.5550093171139102e-05, 
       4.1774266234048137e-05, 
       4.0334001637208215e-05, 
       3.5363093834407096e-05, 
       4.0366359598685416e-05, 
       3.4330078960330751e-05, 
       3.4451720747244228e-05, 
       3.8066042689854233e-05, 
       4.1097989466049727e-05, 
       3.6082679688447050e-05, 
       4.5786774902315612e-05, 
       3.5985816837075656e-05, 
       3.4409089928700468e-05, 
       3.6868224318568522e-05, 
       3.4493094354991217e-05, 
       3.4529184531971749e-05, 
       3.5329038916482944e-05, 
       4.3836217151760351e-05, 
       3.4346868454266494e-05, 
       3.4274093961437654e-05, 
       3.9630524450406251e-05, 
       4.1735657518023995e-05, 
       3.6779927403012487e-05, 
       3.8593768385457307e-05, 
       4.4168135556270210e-05, 
       3.9486983893443966e-05, 
       3.9389217212448893e-05, 
       3.8558157525674599e-05, 
       4.4936088371861442e-05, 
       3.7402776845119888e-05, 
       3.4973124993109781e-05, 
       3.5649897995381909e-05, 
       3.5649897995381909e-05, 
       3.9216573613098059e-05, 
       4.7705200275961463e-05, 
       4.1237985527924675e-05, 
       3.3932771690846749e-05, 
       4.7721066134404328e-05, 
       3.3851426899466486e-05, 
       4.5638081995442594e-05, 
       4.0840257476572915e-05, 
       4.7423543505636740e-05, 
       3.5719111906935536e-05, 
       4.5572686613313965e-05, 
       3.4052247117402352e-05, 
       4.0740048496475884e-05, 
       3.7525217758159063e-05, 
       3.5476270735535963e-05, 
       3.4481818606858959e-05, 
       4.6015467757075992e-05, 
       3.7037340173965451e-05, 
       4.3411946184019808e-05, 
       4.0260813962040006e-05, 
       3.4055869172199889e-05, 
       4.0940109944184912e-05, 
       4.0940109944184912e-05, 
       3.4250637955032855e-05, 
       3.4817914977387234e-05, 
       3.6307606035348190e-05, 
       3.5179241929085079e-05, 
       3.9475707740318510e-05, 
       3.9148557941318213e-05, 
       3.6799820595621291e-05, 
       4.4967245762079095e-05, 
       3.5144825161120897e-05, 
       4.0422731609176954e-05, 
       3.7864172622580213e-05, 
       4.5196076099778753e-05, 
       3.3802472591938855e-05, 
       3.3802472591938855e-05, 
       3.6646806552042508e-05, 
       3.4276871995850190e-05, 
       3.3173960785885519e-05, 
       4.4449011968136945e-05, 
       3.4291182918036221e-05, 
       4.4732487582257194e-05, 
       3.6618552393547212e-05, 
       3.4152068050445495e-05, 
       3.7289155024135254e-05, 
       3.9921231757574604e-05, 
       3.4760153222000421e-05, 
       4.2280574376393759e-05, 
       4.3234377780913248e-05, 
       4.4650198205885767e-05, 
       4.2924765859740912e-05, 
       3.7764182225863115e-05, 
       3.7763227562940555e-05, 
       4.0132310108969856e-05, 
       3.3706501038521069e-05, 
       4.1401363697285896e-05, 
       3.9556663477278989e-05, 
       3.5941651724123704e-05, 
       3.5941651724123704e-05, 
       3.4210120966294029e-05, 
       3.8277022844147885e-05, 
       4.4908664840505224e-05, 
       3.4357778184269766e-05, 
       4.6262428798046453e-05, 
       3.4067672764753477e-05, 
       4.4839968556063865e-05, 
       3.8547866681690799e-05, 
       3.9595103411516685e-05, 
       4.6101807395534712e-05, 
       3.8194338408432130e-05, 
       3.9406142771192216e-05, 
       3.4246035292597692e-05, 
       3.8397194000903380e-05, 
       3.3358498559749076e-05, 
       4.4529473298277279e-05, 
       3.3555139690032741e-05, 
       3.9353938472215678e-05, 
       4.4409439316398738e-05, 
       3.8100405601044617e-05, 
       3.9901076985856095e-05, 
       4.5582072391166627e-05, 
       4.5373061124512591e-05, 
       3.8481683320743657e-05, 
       3.8882566019829942e-05, 
       3.2732554727724230e-05, 
       3.6919623440596352e-05, 
       3.3127811884610454e-05, 
       4.5605273648995629e-05, 
       3.6883096917962499e-05, 
       3.8646018510438268e-05, 
       3.3349414877125529e-05, 
       3.3139629391697061e-05, 
       4.2886603364193917e-05, 
       4.5576671685786118e-05, 
       4.5361457569649943e-05, 
       3.7885936919343399e-05, 
       4.4811888092210247e-05, 
       4.4698893327048875e-05, 
       4.3918202571323494e-05, 
       3.7062422667364702e-05, 
       3.8632060047219603e-05, 
       4.0744976649541705e-05, 
       4.4634977635134732e-05, 
       3.5299040778033262e-05, 
       3.8291727383125503e-05, 
       3.7770718044458303e-05, 
       3.9455274047740725e-05, 
       4.4232365071854907e-05, 
       4.1787579097588931e-05, 
       3.8775187271074794e-05, 
       3.7674431674787115e-05, 
       3.2294846835653616e-05, 
       3.7108330099575195e-05, 
       4.5119420341760923e-05, 
       3.4996037817289307e-05, 
       3.3394308306189422e-05, 
       3.5518565752892662e-05, 
       3.8482338553436338e-05, 
       3.2272639992713039e-05, 
       3.4088521844591359e-05, 
       3.5077080022841553e-05, 
       3.5075598899362149e-05, 
       3.3573211959297057e-05, 
       3.1973382332524500e-05, 
       3.5840891222916174e-05, 
       3.8751206026778844e-05, 
       3.8751206026778844e-05, 
       3.7086597111063709e-05, 
       3.3096259382174962e-05, 
       3.3838700250631010e-05, 
       3.2618847110974430e-05, 
       3.4020508225717068e-05, 
       3.1843818509819005e-05, 
       3.8390188135025377e-05, 
       3.8810010462808655e-05, 
       3.3211378175664576e-05, 
       3.4869311968063722e-05, 
       3.2032700395950697e-05, 
       3.7811234785441171e-05, 
       4.4649453583111601e-05, 
       3.3583031038668679e-05, 
       3.1655365245265213e-05, 
       3.9967497657719340e-05, 
       4.2289960354588734e-05, 
       3.1631018804853388e-05, 
       4.4374183229811822e-05, 
       3.1799366549007982e-05, 
       3.1729880444632586e-05, 
       4.4407619179603196e-05, 
       4.3963940690826286e-05, 
       3.1600386724480620e-05, 
       3.7125809542202758e-05, 
       3.2531686361602757e-05, 
       3.2969916162530642e-05, 
       3.2969916162530642e-05, 
       3.2701051266413326e-05, 
       3.3056203684438582e-05, 
       3.1463116866752515e-05, 
       4.2466996232677248e-05, 
       3.5437423146589721e-05, 
       3.6772774001921257e-05, 
       3.1581270747609690e-05, 
       4.4133679269790426e-05, 
       3.8478619970294991e-05, 
       3.7698901535583698e-05, 
       3.6744197880479884e-05, 
       3.8486937536895026e-05, 
       3.7899811440157475e-05, 
       3.6481118895287630e-05, 
       4.3145115509460633e-05, 
       3.5810792340679481e-05, 
       3.5807562073949786e-05, 
       3.7351548445723314e-05, 
       3.6161055734250827e-05, 
       3.3459078306163453e-05, 
       3.7758535206845361e-05, 
       3.5074324019369783e-05, 
       3.8777583458437893e-05, 
       3.5052550073536175e-05, 
       3.2657632475121690e-05, 
       3.1722550486931950e-05, 
       3.1777163109859709e-05, 
       3.1128393080790415e-05, 
       4.2972575006060915e-05, 
       3.1623936050455644e-05, 
       3.3211228967468389e-05, 
       3.2219112346808056e-05, 
       4.3344935254243846e-05, 
       3.0767142438753858e-05, 
       3.9246352493641287e-05, 
       3.6265805758314331e-05, 
       4.3436259932608941e-05, 
       3.2519086391907427e-05, 
       3.5019963142609861e-05, 
       3.5552732137505110e-05, 
       4.0000000000000044e-05, 
       4.0000000000003256e-05, 
       3.9999999999998695e-05, 
       3.9999999999998695e-05, 
       3.9999999999995877e-05, 
       3.9999999999996947e-05, 
       4.0000000000003710e-05, 
       3.9999999999998641e-05, 
       3.9999999999998323e-05, 
       4.0000000000004604e-05, 
       4.0000000000004604e-05, 
       3.1520603523588699e-05, 
       3.8187700027478257e-05, 
       3.0655504273065068e-05, 
       3.5114682601046063e-05, 
       3.5114682601046063e-05, 
       3.0629212305899648e-05, 
       3.9340365155687039e-05, 
       3.0790902821695978e-05, 
       3.5479215584076588e-05, 
       3.5476396478559203e-05, 
       3.0550624230616264e-05, 
       3.0638378345440583e-05, 
       4.2895442200886316e-05, 
       3.1446869249003625e-05, 
       3.5988637916796449e-05, 
       3.0544748877238458e-05, 
       3.2441108279527417e-05, 
       4.2814356416683900e-05, 
       3.0480827923530931e-05, 
       4.2960992784964117e-05, 
       3.1205634424314559e-05, 
       3.2176290434396502e-05, 
       3.4861292552849490e-05, 
       3.9792660334406046e-05, 
       3.9792660334406046e-05, 
       4.0067539615737803e-05, 
       3.1520225277737094e-05, 
       3.9344044811382328e-05, 
       3.9160383529734912e-05, 
       3.5589712608140480e-05, 
       4.2136885109624920e-05, 
       4.2136885109624920e-05, 
       3.5800516909935126e-05, 
       3.1545489344280527e-05, 
       3.2051367584966136e-05, 
       3.9090739088999569e-05, 
       3.1816945549504981e-05, 
       4.1451797449941164e-05, 
       3.5676432089641460e-05, 
       3.4040340607583295e-05, 
       3.0752560636817230e-05, 
       4.0326944325697221e-05, 
       3.0961311707883677e-05, 
       3.5421085552023147e-05, 
       3.1448708787215871e-05, 
       3.7149270370099857e-05, 
       3.5534674789581067e-05, 
       3.0917513880140062e-05, 
       4.1181329337049383e-05, 
       3.2561787529455299e-05, 
       4.2602634000447563e-05, 
       3.5526838153295525e-05, 
       3.1766301141713461e-05, 
       3.1138123142915250e-05, 
       3.5352084970560701e-05, 
       3.0283125677505168e-05, 
       3.1296910168382224e-05, 
       4.2402176308274449e-05, 
       3.1235925595150562e-05, 
       3.1783354659371787e-05, 
       3.0831423973898561e-05, 
       3.2623115391230380e-05, 
       3.1266830259563386e-05, 
       3.2623853756686830e-05, 
       4.0000000000004171e-05, 
       3.0151503090034851e-05, 
       3.4115561186289944e-05, 
       3.0700382999754308e-05, 
       3.1687830719248371e-05, 
       4.1811574531025101e-05, 
       3.9935185934137698e-05, 
       3.3883660673652234e-05, 
       3.0056033030738733e-05, 
       3.0071986333635376e-05, 
       3.2293030300190461e-05, 
       3.1105521348951953e-05, 
       3.1556442746839226e-05, 
       3.1556442746839226e-05, 
       3.0960588468513537e-05, 
       3.3228570021988799e-05, 
       3.4213533216578949e-05, 
       3.1390879861648162e-05, 
       3.4928221367676138e-05, 
       4.2082652052599186e-05, 
       3.1454256467664598e-05, 
       3.5133637921966394e-05, 
       3.4906351278678537e-05, 
       4.0289584855503186e-05, 
       3.0000000000005696e-05, 
       3.0000000000003257e-05, 
       3.2665592510237427e-05, 
       3.0000000000002850e-05, 
       3.2610327726377131e-05, 
       2.9999999999998537e-05, 
       2.9999999999998537e-05, 
       3.3836451523498569e-05, 
       4.0373708207542985e-05, 
       3.1953995716624574e-05, 
       4.1602820865319293e-05, 
       4.0606951685279347e-05, 
       3.5988849208438583e-05, 
       4.1799785873359888e-05, 
       3.0202547506420863e-05, 
       4.1701605606512434e-05, 
       3.3245080297128400e-05, 
       3.0363063426439208e-05, 
       3.8486875719931199e-05, 
       2.9650761490452748e-05, 
       3.0125359633885684e-05, 
       3.5334815406176788e-05, 
       3.7282687177737835e-05, 
       4.0279247069626156e-05, 
       3.9798040404030383e-05, 
       4.1588235054095805e-05, 
       3.1492990325673574e-05, 
       4.1160470578291326e-05, 
       3.6351931201530698e-05, 
       3.6716831478262055e-05, 
       3.9564283766043665e-05, 
       3.9564283766043665e-05, 
       2.9372918642731359e-05, 
       3.7740652638366989e-05, 
       3.4228546616817536e-05, 
       3.3734390075500566e-05, 
       3.9782066726249878e-05, 
       3.3399863828928042e-05, 
       4.1328098805544524e-05, 
       2.9565615872071453e-05, 
       3.9763346035326985e-05, 
       4.1285477836314294e-05, 
       3.6972811087475271e-05, 
       3.6990578151268718e-05, 
       3.8193023676862647e-05, 
       3.0832013078021731e-05, 
       4.1244800999261019e-05, 
       3.2187334333793278e-05, 
       3.2187334333793278e-05, 
       3.2850848384872945e-05, 
       2.9816401789328107e-05, 
       3.4200872130658989e-05, 
       4.1102988943635879e-05, 
       3.4737574801587400e-05, 
       3.4737574801587400e-05, 
       3.0666894504126092e-05, 
       4.1071813861677277e-05, 
       3.0645350076576921e-05, 
       4.1070252544372064e-05, 
       4.1076324877475343e-05, 
       3.0996977406781032e-05, 
       4.1031868374573238e-05, 
       3.6020689277912352e-05, 
       3.7207170822787140e-05, 
       3.3932771690848700e-05, 
       2.8979178260506008e-05, 
       3.4616441795642464e-05, 
       2.9623291809170077e-05, 
       3.1082912024953075e-05, 
       3.4042137612941533e-05, 
       4.0603184187485268e-05, 
       3.5296338770245203e-05, 
       3.8295201009682578e-05, 
       3.9315755608348710e-05, 
       3.7161531723287246e-05, 
       2.9075252692255868e-05, 
       3.8205457155891319e-05, 
       2.9349786857907786e-05, 
       3.0000000000003799e-05, 
       2.9928702118651575e-05, 
       2.9701306352631196e-05, 
       2.9052798867166905e-05, 
       3.3070098669626784e-05, 
       3.7174932397282985e-05, 
       4.0419324820322041e-05, 
       3.0915297672439782e-05, 
       3.2090122233725596e-05, 
       3.0850098825009209e-05, 
       3.1249330033542872e-05, 
       2.8903793623760534e-05, 
       3.1394439697017604e-05, 
       4.0456745335075532e-05, 
       2.9701510602019764e-05, 
       3.4464562264029627e-05, 
       3.0927553069141961e-05, 
       2.9056149175103325e-05, 
       3.3712354842395815e-05, 
       3.0016120296635692e-05, 
       3.8449684263640226e-05, 
       3.7914245397879775e-05, 
       4.0261905754794565e-05, 
       3.3686053414424321e-05, 
       3.1736531080024320e-05, 
       3.3919277072243219e-05, 
       3.3333369538697630e-05, 
       4.0234791889229823e-05, 
       3.5729591088579886e-05, 
       3.4634201405336033e-05, 
       3.4683033109870199e-05, 
       3.3701145766668560e-05, 
       3.7639476947115350e-05, 
       3.4920534260712328e-05, 
       3.0044425549602490e-05, 
       4.0228911425485270e-05, 
       3.7210650310490927e-05, 
       3.5327437378899194e-05, 
       4.0208263599085097e-05, 
       2.8472522087666933e-05, 
       3.5204527044063818e-05, 
       3.8463102569177809e-05, 
       3.0794404473527768e-05, 
       3.5539496648845564e-05, 
       3.4226461824829729e-05, 
       3.3402023386649974e-05, 
       3.9999833733587459e-05, 
       4.0029826003979332e-05, 
       4.0029844154048853e-05, 
       3.0366110018074966e-05, 
       4.0012715551191984e-05, 
       2.9923443061167850e-05, 
       3.0000000000004290e-05, 
       3.8741427463576370e-05, 
       2.9170411237893291e-05, 
       3.9999999999999075e-05, 
       3.9999999999997679e-05, 
       3.9999999999997787e-05, 
       4.0000000000004957e-05, 
       4.0000000000001026e-05, 
       3.9999999999999414e-05, 
       4.0000000000002246e-05, 
       3.9999999999996818e-05, 
       3.9999999999998275e-05, 
       4.0000000000000566e-05, 
       4.0000000000001894e-05, 
       3.9999999999999407e-05, 
       4.0000000000004625e-05, 
       3.9999999999997638e-05, 
       4.0000000000001935e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.9650987941895117e-05, 
       3.5482870386269809e-05, 
       2.9622973978578160e-05, 
       3.6593164811460520e-05, 
       2.8575086252524803e-05, 
       3.2446315832416563e-05, 
       3.2445998443909133e-05, 
       2.9222570908625658e-05, 
       3.5897359386160824e-05, 
       2.8220584857159224e-05, 
       3.6687774840680397e-05, 
       3.6443879322673523e-05, 
       3.1244270277087970e-05, 
       3.3289332685652457e-05, 
       3.1230700689164850e-05, 
       3.1337411944442081e-05, 
       2.8521061297300755e-05, 
       3.9687554334761485e-05, 
       3.0254233466135556e-05, 
       3.1100594687884024e-05, 
       2.8850617147111457e-05, 
       2.8944511564133373e-05, 
       2.8082265116472246e-05, 
       2.8068528184765371e-05, 
       2.8077208884788131e-05, 
       3.0000000000003890e-05, 
       3.0000000000002623e-05, 
       2.8530676677170921e-05, 
       3.7275800419838399e-05, 
       3.6215545086320893e-05, 
       3.1365052532130788e-05, 
       3.9516476748865015e-05, 
       3.0000000000001698e-05, 
       3.7853102683148861e-05, 
       2.9432110253767269e-05, 
       2.8959495717187786e-05, 
       3.0543824997494746e-05, 
       2.9999999999999923e-05, 
       2.8568650787982516e-05, 
       2.9297579659319114e-05, 
       3.3499765174853511e-05, 
       2.9204637490925500e-05, 
       2.9318124522991120e-05, 
       2.7865298580209685e-05, 
       2.7988822053944336e-05, 
       3.0248660897675957e-05, 
       3.7341689134393539e-05, 
       3.4414201452375797e-05, 
       3.4414201452375797e-05, 
       3.4308219212227503e-05, 
       3.0383257698228503e-05, 
       3.8408828770963399e-05, 
       2.9386197789292617e-05, 
       3.1259015612606084e-05, 
       3.8658217901830363e-05, 
       3.9064243768141764e-05, 
       2.8712653641529668e-05, 
       3.3520119136562809e-05, 
       2.9572263932335081e-05, 
       2.9130791664867986e-05, 
       2.8160474893762232e-05, 
       3.3742104548615828e-05, 
       3.9008734495496870e-05, 
       3.7431286076416152e-05, 
       3.0654617331625450e-05, 
       3.5126907049644794e-05, 
       3.1607286960826823e-05, 
       2.8285746086841743e-05, 
       3.8541853781224040e-05, 
       2.9989459915217344e-05, 
       3.8467281057835360e-05, 
       3.3113867711355641e-05, 
       3.2347611867880527e-05, 
       2.7505781452199071e-05, 
       3.2323134450346488e-05, 
       3.3188587985101968e-05, 
       2.9969687710007455e-05, 
       2.9702866064993838e-05, 
       2.7976306513998781e-05, 
       3.7649197822990172e-05, 
       2.7814093744353112e-05, 
       3.8729535589708686e-05, 
       2.7814734299262401e-05, 
       3.8653908574207702e-05, 
       3.7582923524431794e-05, 
       2.7486323187209600e-05, 
       2.8162466834048949e-05, 
       2.8851249081728891e-05, 
       3.8695240199046960e-05, 
       3.5327127544927605e-05, 
       3.3040893491297512e-05, 
       2.9324329557977478e-05, 
       2.7374221715914724e-05, 
       3.2978908424775638e-05, 
       3.2043720776259829e-05, 
       2.7389734391356983e-05, 
       3.6547845291455098e-05, 
       2.9465349402272529e-05, 
       3.1787455009125469e-05, 
       3.1730730693507046e-05, 
       2.7976528120341016e-05, 
       3.8285833626464303e-05, 
       2.8948340932654851e-05, 
       2.9134190212726618e-05, 
       2.8673368417038018e-05, 
       2.9519846433190461e-05, 
       2.8578194731965850e-05, 
       2.7316504284669223e-05, 
       3.6331372720670647e-05, 
       3.6323511307096425e-05, 
       3.5713305796603942e-05, 
       3.2584746456569697e-05, 
       2.7071017134050915e-05, 
       3.4570049770267100e-05, 
       3.2989438459643273e-05, 
       2.8217452674773613e-05, 
       3.0026421008164881e-05, 
       3.2531280616438226e-05, 
       3.7257390920122403e-05, 
       3.2955583650009276e-05, 
       3.6680754730805116e-05, 
       3.3405109040810726e-05, 
       2.8382676175699329e-05, 
       3.5311993187856882e-05, 
       3.0539013882529363e-05, 
       3.1505554418710124e-05, 
       3.2843689428531188e-05, 
       3.4710405485303393e-05, 
       3.6281714511098339e-05, 
       3.1972625598174635e-05, 
       2.6952375059381419e-05, 
       2.7834170994420485e-05, 
       2.6849007007184411e-05, 
       3.7823612738472191e-05, 
       2.7695085583835227e-05, 
       2.8737751496506896e-05, 
       3.2315997158234298e-05, 
       3.2035312534696759e-05, 
       3.0555951315988259e-05, 
       3.3530224421913222e-05, 
       3.5782113514338267e-05, 
       3.1337038596283865e-05, 
       3.0000000000003209e-05, 
       2.8769334550289220e-05, 
       3.4499437752777667e-05, 
       3.2807943768833218e-05, 
       3.3126108059818784e-05, 
       2.7934328777687005e-05, 
       3.5580419669643663e-05, 
       3.2176283974695814e-05, 
       3.3298103528320674e-05, 
       3.6055431562855655e-05, 
       3.5609595365240585e-05, 
       2.6689551612567050e-05, 
       2.9115329692074404e-05, 
       3.0276756787263230e-05, 
       2.6775219220041727e-05, 
       2.7994865876308730e-05, 
       3.2096843417391421e-05, 
       3.7363428462315630e-05, 
       3.2084949845151920e-05, 
       2.6651871004129518e-05, 
       3.1763396696123271e-05, 
       2.7519057009068031e-05, 
       2.8563919742320334e-05, 
       3.1894204383735864e-05, 
       2.7458936442062691e-05, 
       3.0897729527835391e-05, 
       3.5273612550722127e-05, 
       3.1457485844386864e-05, 
       3.2446120342464429e-05, 
       2.8527782530677453e-05, 
       2.8080167794846691e-05, 
       2.8080167794846691e-05, 
       2.6827956180676950e-05, 
       3.7202714531507361e-05, 
       3.7199602705927552e-05, 
       2.6293008694190432e-05, 
       2.8990484485342919e-05, 
       2.9886850376486911e-05, 
       2.6320960512972062e-05, 
       2.9856951105215019e-05, 
       3.7049150835329941e-05, 
       2.8181324118350672e-05, 
       3.1068817939316885e-05, 
       2.8419576696686963e-05, 
       2.9772054084084639e-05, 
       3.6978786302679147e-05, 
       3.0817872042770107e-05, 
       2.7651810436792986e-05, 
       3.0472039441073314e-05, 
       3.0749033162695300e-05, 
       3.6877175023213065e-05, 
       3.6804827147161923e-05, 
       3.0366074177180129e-05, 
       3.6528104931409156e-05, 
       3.1616580804268080e-05, 
       2.8777848553049863e-05, 
       2.6295438224540838e-05, 
       2.6345438903566346e-05, 
       2.8480138056537372e-05, 
       3.4263847765449354e-05, 
       3.0982006598378774e-05, 
       3.4239074491246886e-05, 
       3.0632180601652828e-05, 
       2.7771942945472978e-05, 
       2.8934784541401819e-05, 
       3.0337024272243701e-05, 
       3.1720626572684824e-05, 
       2.7392062671394504e-05, 
       3.6002388331020245e-05, 
       3.1607286960826430e-05, 
       3.5801186193448936e-05, 
       2.7487744403928269e-05, 
       2.6677955032792055e-05, 
       3.0628741632799320e-05, 
       3.6510065153031226e-05, 
       3.6015452006704093e-05, 
       3.6357297291441801e-05, 
       3.6476683310775503e-05, 
       3.3611621798708118e-05, 
       3.5875659550603929e-05, 
       3.2634187209055610e-05, 
       2.7930778087845611e-05, 
       2.9290313237655716e-05, 
       2.5777677579179011e-05, 
       2.5928187390429477e-05, 
       2.7540875722982256e-05, 
       3.5469450524023458e-05, 
       3.5871783031862640e-05, 
       2.5667459950830651e-05, 
       2.9600884725684659e-05, 
       3.3064887819056622e-05, 
       2.8522896376403126e-05, 
       2.6429971651214224e-05, 
       3.0877422423057182e-05, 
       2.7225424370410121e-05, 
       2.5826760319285673e-05, 
       3.5762396041448836e-05, 
       2.7058305027219290e-05, 
       2.6396357425576386e-05, 
       3.0125333144341761e-05, 
       2.6963325232473530e-05, 
       2.6963325232473530e-05, 
       3.0889459469239398e-05, 
       2.6912393472690718e-05, 
       3.0273351280007716e-05, 
       3.3091362765132128e-05, 
       3.5936245272006098e-05, 
       2.7615675276231317e-05, 
       3.4659105311767249e-05, 
       3.1358466138024678e-05, 
       2.8803181620953417e-05, 
       2.9690143341845960e-05, 
       3.0805093199411554e-05, 
       2.5536594902026181e-05, 
       2.9970226582605102e-05, 
       2.8971946865796545e-05, 
       3.5274607453297510e-05, 
       2.5840498017201320e-05, 
       3.5771754859293081e-05, 
       2.6186282374133461e-05, 
       3.1211810801065112e-05, 
       2.9368704696853477e-05, 
       3.2053126566242743e-05, 
       2.6230692824546582e-05, 
       2.5779427165932097e-05, 
       3.5847076981616771e-05, 
       3.4321135515174934e-05, 
       3.5625144114530312e-05, 
       2.6260702955184550e-05, 
       3.3743727769820210e-05, 
       2.5496509510115764e-05, 
       3.2920047814375116e-05, 
       3.5276042413766353e-05, 
       2.9838628640169500e-05, 
       2.6180073852977485e-05, 
       2.6430193040288062e-05, 
       3.5540757566365029e-05, 
       2.5445362457155064e-05, 
       2.5131748360077843e-05, 
       2.5183856314941863e-05, 
       2.5183856314941863e-05, 
       3.2243154260855022e-05, 
       2.5413613970155223e-05, 
       3.2029548066166961e-05, 
       3.4915887907330485e-05, 
       3.4915887907330485e-05, 
       2.9121450554522224e-05, 
       2.5879768128818131e-05, 
       3.5204857364067725e-05, 
       2.5288919313416905e-05, 
       2.7163503719489365e-05, 
       2.9374918632531657e-05, 
       2.7094345674080616e-05, 
       3.3247343677741839e-05, 
       2.6368372183667267e-05, 
       2.6798105231056230e-05, 
       2.6248455264832591e-05, 
       3.1602086651328833e-05, 
       2.8507450648439152e-05, 
       2.7117876097687938e-05, 
       2.7435175913409345e-05, 
       2.6449669604885500e-05, 
       2.6837769560996346e-05, 
       3.2288093342801241e-05, 
       3.3112181753226338e-05, 
       2.6327828105466185e-05, 
       2.7604395014481555e-05, 
       2.6248274781019007e-05, 
       3.5075187226166501e-05, 
       2.6499287902139400e-05, 
       3.5048342316519342e-05, 
       2.5487772775955021e-05, 
       2.9107585376571788e-05, 
       2.7315182587174559e-05, 
       3.5026048669214905e-05, 
       2.7240047410789172e-05, 
       2.7240047410789172e-05, 
       3.4992291131616725e-05, 
       2.9301599397494444e-05, 
       2.7829450804557431e-05, 
       2.7517890164182404e-05, 
       2.7051289547636673e-05, 
       3.0700648712216066e-05, 
       2.5506036094101103e-05, 
       3.4817957224150159e-05, 
       2.9490278375770467e-05, 
       3.4844018242779629e-05, 
       2.6165365567694776e-05, 
       2.6165365567694776e-05, 
       2.8393523521984109e-05, 
       2.5428953482915565e-05, 
       2.5539217545176599e-05, 
       2.6802706074228607e-05, 
       3.4758237164937349e-05, 
       2.8927436514525534e-05, 
       2.5163502360372020e-05, 
       2.8961570281075600e-05, 
       2.6434114728889446e-05, 
       2.4654758886623465e-05, 
       2.4654758886623465e-05, 
       2.5001033971007190e-05, 
       2.5964479321303237e-05, 
       2.8836170219685999e-05, 
       2.6007485549697854e-05, 
       3.4599167819017538e-05, 
       2.5920207430770246e-05, 
       2.9846520473705149e-05, 
       2.8645809582644942e-05, 
       3.4377045575196747e-05, 
       3.4200684791627271e-05, 
       2.4426128399523059e-05, 
       2.5647224272868987e-05, 
       3.4476549336772421e-05, 
       2.8247894539198831e-05, 
       2.7438582780098892e-05, 
       3.4479778057852196e-05, 
       3.4251021526359262e-05, 
       2.5664206075342809e-05, 
       2.9121678859158691e-05, 
       2.5033201362671111e-05, 
       3.3364283037172436e-05, 
       2.4447835058470351e-05, 
       3.4294599299771525e-05, 
       3.3291065125318106e-05, 
       3.4089501744920831e-05, 
       2.4969954847476265e-05, 
       2.9024699719979002e-05, 
       2.8671196159773006e-05, 
       2.4630330652046270e-05, 
       3.2280424805061478e-05, 
       2.9522686564137383e-05, 
       3.4192396523913104e-05, 
       2.7114935325952179e-05, 
       3.3912427428238050e-05, 
       3.4316842850734913e-05, 
       2.8418999202795457e-05, 
       3.0233394826160935e-05, 
       2.8194171881190316e-05, 
       2.8404788785574209e-05, 
       2.4343139871393643e-05, 
       3.0191619504350694e-05, 
       3.0755917042513452e-05, 
       2.8027836970488926e-05, 
       3.4272269132251431e-05, 
       3.3120345686135500e-05, 
       2.4218683291086537e-05, 
       2.5944641231553954e-05, 
       2.8519726090785362e-05, 
       3.2754655590475162e-05, 
       2.6670566568781220e-05, 
       2.8667375144667282e-05, 
       2.6250974145638234e-05, 
       3.4179587676987727e-05, 
       2.8417846549915602e-05, 
       3.1915296966306216e-05, 
       2.7946890837173315e-05, 
       2.7946890837173315e-05, 
       2.5561360154550715e-05, 
       3.3523687032400315e-05, 
       3.3516198827054114e-05, 
       2.8885952754713721e-05, 
       2.5392466137233413e-05, 
       2.6697893488365707e-05, 
       2.7953335551113027e-05, 
       3.0385390315916287e-05, 
       2.6691373934654575e-05, 
       2.4891608425172016e-05, 
       2.9186893725728448e-05, 
       2.9720024317533491e-05, 
       2.4316073025063863e-05, 
       3.2072791174885115e-05, 
       2.8310123390552802e-05, 
       2.7681565392937294e-05, 
       2.8172209727582446e-05, 
       2.4717834974175119e-05, 
       2.8500031283951278e-05, 
       2.8151312716500778e-05, 
       2.7880937040111283e-05, 
       2.7049357159754555e-05, 
       2.9417677684036884e-05, 
       2.8328383928724001e-05, 
       2.5935271797444521e-05, 
       3.2755520485280344e-05, 
       2.4394581900398357e-05, 
       2.6867819495971078e-05, 
       2.9794057034115196e-05, 
       2.7440414384611052e-05, 
       3.3695842700976900e-05, 
       2.7432561829927438e-05, 
       3.0380485458353943e-05, 
       2.5186131576443482e-05, 
       3.3498529456835508e-05, 
       2.6893492175475148e-05, 
       2.8566549431284558e-05, 
       2.8566549431284558e-05, 
       2.6663138893977932e-05, 
       3.2461213637322961e-05, 
       2.6958626730782541e-05, 
       2.4930182552588714e-05, 
       2.4930182552588714e-05, 
       2.8686862129057285e-05, 
       2.4263861299639403e-05, 
       2.8605673870863314e-05, 
       2.3730701314903488e-05, 
       2.3903231254453783e-05, 
       2.5323838146909259e-05, 
       2.7089073128826942e-05, 
       2.7708527447814174e-05, 
       2.7743510227138512e-05, 
       2.7527281192965033e-05, 
       3.1486880033920773e-05, 
       2.4644722521115708e-05, 
       3.0367131000665230e-05, 
       3.3264524532519042e-05, 
       3.3092992729586531e-05, 
       2.7315479801981029e-05, 
       3.2950504333784331e-05, 
       2.7649710679235837e-05, 
       2.5261628588576502e-05, 
       3.2156559939064274e-05, 
       2.8527342620175341e-05, 
       2.8397158494519286e-05, 
       3.3153529898453979e-05, 
       2.4085492896186930e-05, 
       2.8105970117063629e-05, 
       3.1033444745727773e-05, 
       2.4807487784192627e-05, 
       2.4583567523299465e-05, 
       2.7437918633535890e-05, 
       2.6059703812951711e-05, 
       2.8669835086657525e-05, 
       2.3522274813803017e-05, 
       2.7546112682650883e-05, 
       2.4376002944147142e-05, 
       2.3809454159939669e-05, 
       3.2130895186351850e-05, 
       2.7134289388676936e-05, 
       2.8031167862592865e-05, 
       2.4330837799660043e-05, 
       2.3790586943871917e-05, 
       2.4618466039271922e-05, 
       3.2499801756115493e-05, 
       3.2415007070882617e-05, 
       3.1402259866988810e-05, 
       3.1361265532885596e-05, 
       3.2967247616230259e-05, 
       3.2926980145146655e-05, 
       3.2897957161648355e-05, 
       3.2948545011265201e-05, 
       2.8324228067536267e-05, 
       3.2801773635771439e-05, 
       2.4924362740452873e-05, 
       2.3586579648045455e-05, 
       2.6808095908631068e-05, 
       2.4024510227490845e-05, 
       2.7682180923534672e-05, 
       2.6090946790785796e-05, 
       3.2557167762135286e-05, 
       2.6799504014023414e-05, 
       3.2673818104157556e-05, 
       2.4122472530575361e-05, 
       2.9519237447149331e-05, 
       2.4135295232504095e-05, 
       3.2648974817421232e-05, 
       2.3600289507962191e-05, 
       3.2609486734809940e-05, 
       3.2438272513003899e-05, 
       2.7015383036214886e-05, 
       2.7238773284972113e-05, 
       2.7028509038268039e-05, 
       2.6668046581791002e-05, 
       2.3224137971530830e-05, 
       2.6750217843854205e-05, 
       2.4787134256384244e-05, 
       2.5890325553033500e-05, 
       2.3044499027859778e-05, 
       2.9999999999996823e-05, 
       2.9999999999997812e-05, 
       3.0000000000001190e-05, 
       3.0000000000001925e-05, 
       3.0000000000001241e-05, 
       2.9999999999996040e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.3889920703872291e-05, 
       2.5797298581868303e-05, 
       2.9068548551881954e-05, 
       2.7035369778148715e-05, 
       2.5461509891066295e-05, 
       2.5157720705493091e-05, 
       2.9296387254842345e-05, 
       2.6890413863725637e-05, 
       2.8410366991330650e-05, 
       2.7688211026207803e-05, 
       2.8417379778696077e-05, 
       2.6997498023266238e-05, 
       2.3482817963913375e-05, 
       2.4557840187249647e-05, 
       2.6940261705840389e-05, 
       2.2894511820809013e-05, 
       2.2908706346192163e-05, 
       3.1349347545628386e-05, 
       2.4321376606195986e-05, 
       2.2966859743536285e-05, 
       2.9147794246437171e-05, 
       3.0703859380416975e-05, 
       2.3402221596710644e-05, 
       2.7615709465128338e-05, 
       2.3650705692883611e-05, 
       2.3650705692883611e-05, 
       2.3904474655771749e-05, 
       3.1607364102157018e-05, 
       2.6600896328399041e-05, 
       3.1229248333623198e-05, 
       2.6321370303117079e-05, 
       2.7796535037283469e-05, 
       2.6641235987568230e-05, 
       2.3832959785462192e-05, 
       3.1649812539506376e-05, 
       3.0592741499987829e-05, 
       2.3096510237342673e-05, 
       2.2699653875450696e-05, 
       2.3249745470124507e-05, 
       2.6773023703665230e-05, 
       2.2970110172489182e-05, 
       3.0838358223896081e-05, 
       2.6139734478468064e-05, 
       2.3764794548079181e-05, 
       2.2947817061798552e-05, 
       2.2833053494474773e-05, 
       2.9999999999999235e-05, 
       2.9626090535365558e-05, 
       2.3486790295566854e-05, 
       2.3241267647300088e-05, 
       3.1836859885167991e-05, 
       2.6571641821906969e-05, 
       2.2834650174087768e-05, 
       2.4906174972015973e-05, 
       2.3022973518139672e-05, 
       2.8062723410813606e-05, 
       3.1809906127330802e-05, 
       2.5474798257939025e-05, 
       3.0350618415187151e-05, 
       2.4060360308950122e-05, 
       3.1708235945966868e-05, 
       2.8395459418629152e-05, 
       3.1498596333085101e-05, 
       3.1498596333085101e-05, 
       2.7018291849097191e-05, 
       2.2779391893054154e-05, 
       2.4024226804887041e-05, 
       2.8994313727000298e-05, 
       2.6988491048561361e-05, 
       2.3677471064899219e-05, 
       2.4107423881426563e-05, 
       2.6027661009359367e-05, 
       2.2385820822631361e-05, 
       2.6290319960895727e-05, 
       2.7460075917738829e-05, 
       2.3665446502189034e-05, 
       2.6987673330377177e-05, 
       2.9310579038495038e-05, 
       2.2646672164481440e-05, 
       2.2771857861919115e-05, 
       2.7179884624405099e-05, 
       2.3862224680109702e-05, 
       3.1500715622996577e-05, 
       2.6202096940542220e-05, 
       2.4102312040910398e-05, 
       2.7408329776064467e-05, 
       2.2316398705654424e-05, 
       3.1105320269090580e-05, 
       2.4065749760453949e-05, 
       2.4065749760453949e-05, 
       2.8476234504351562e-05, 
       2.6138437717666728e-05, 
       2.7557054771714884e-05, 
       2.2222437761925977e-05, 
       2.5761711979121882e-05, 
       2.2205564291773968e-05, 
       2.2410307074124568e-05, 
       2.8276523875806391e-05, 
       3.1302737334661764e-05, 
       2.3283726202759125e-05, 
       2.3304161748790386e-05, 
       3.1306975642616347e-05, 
       2.6375390025120789e-05, 
       2.2182938597154211e-05, 
       3.0960807001893467e-05, 
       2.2835906472531427e-05, 
       2.5884602200539683e-05, 
       2.9229347078953845e-05, 
       2.3496642104841726e-05, 
       2.3185542134062420e-05, 
       2.2946360701331529e-05, 
       2.5776194824604358e-05, 
       2.2454703100843991e-05, 
       2.8531063441147001e-05, 
       2.6074827207450302e-05, 
       2.5556299411245292e-05, 
       2.3144668520699665e-05, 
       2.8795506143643095e-05, 
       2.1986390120268598e-05, 
       3.0299289156082009e-05, 
       2.6564089538255877e-05, 
       2.3322930384472611e-05, 
       2.5342102881747371e-05, 
       2.2335720200635056e-05, 
       3.0258393603544679e-05, 
       2.2195165077980501e-05, 
       2.7397086408704430e-05, 
       2.6917607645641059e-05, 
       2.4710220326074705e-05, 
       2.2253959978517775e-05, 
       2.2253959978517775e-05, 
       2.2771073735115909e-05, 
       2.4479378788658450e-05, 
       2.2765888807037921e-05, 
       2.1895520882208273e-05, 
       2.2921622956386172e-05, 
       2.9707451838051642e-05, 
       2.5573072733358111e-05, 
       2.3058233823004184e-05, 
       2.3307036526590237e-05, 
       2.5092609836781439e-05, 
       2.2748515000489624e-05, 
       2.3150277437011079e-05, 
       3.0791274555545078e-05, 
       2.2900377411271002e-05, 
       2.8012605383694653e-05, 
       3.0412383982806414e-05, 
       3.0181374998003317e-05, 
       2.2276745666794063e-05, 
       2.9750928763819810e-05, 
       2.1662405221183852e-05, 
       2.2138372384155840e-05, 
       2.1996088220648256e-05, 
       2.1991383242162543e-05, 
       2.4564033802824913e-05, 
       2.6990098647192138e-05, 
       2.3780130967086489e-05, 
       2.5351466465478984e-05, 
       3.0443058101149356e-05, 
       3.0390926228646568e-05, 
       2.5368723327925288e-05, 
       2.1567682677648480e-05, 
       2.2292876913502037e-05, 
       2.2735671428963824e-05, 
       2.8647198936037716e-05, 
       2.9988827002107710e-05, 
       2.8577410507709898e-05, 
       2.5269777898027529e-05, 
       2.8216726269075040e-05, 
       2.3764465637506819e-05, 
       2.4782966485001782e-05, 
       2.2255096370630049e-05, 
       2.9164866735637960e-05, 
       2.2340046124967011e-05, 
       2.5329885223332629e-05, 
       2.3076594573793710e-05, 
       2.5272042295458624e-05, 
       2.4704698549571055e-05, 
       2.9454304430481464e-05, 
       2.7756201280397087e-05, 
       2.2572585828920256e-05, 
       2.2494388918449920e-05, 
       3.0137206527548647e-05, 
       2.8170724885628871e-05, 
       2.1413179031443492e-05, 
       2.8659232795270722e-05, 
       2.1346292775702525e-05, 
       2.6760840154110105e-05, 
       2.9204634832804709e-05, 
       2.5666949948466051e-05, 
       2.6061808553163991e-05, 
       3.0059956200694516e-05, 
       2.3290481993096621e-05, 
       2.5138412556286546e-05, 
       2.5532372454328029e-05, 
       2.8899931661099338e-05, 
       2.9907934239634957e-05, 
       3.0000000000004405e-05, 
       3.0000000000001478e-05, 
       2.9999999999995698e-05, 
       2.9999999999999855e-05, 
       2.9999999999999736e-05, 
       3.0000000000000529e-05, 
       2.9999999999998808e-05, 
       2.9999999999998740e-05, 
       2.9999999999997765e-05, 
       2.9999999999997893e-05, 
       3.0000000000001674e-05, 
       2.9999999999999940e-05, 
       3.0000000000002129e-05, 
       2.9999999999998706e-05, 
       2.9999999999998937e-05, 
       3.0000000000004934e-05, 
       2.9999999999999347e-05, 
       2.9999999999999242e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.1239152732205458e-05, 
       2.7327175427063343e-05, 
       2.7327175427063343e-05, 
       2.1890823686870737e-05, 
       2.6429974762185499e-05, 
       2.5876481913348810e-05, 
       2.3954620912600911e-05, 
       2.1540065046920755e-05, 
       2.1576992470412397e-05, 
       2.4661653433269915e-05, 
       2.6433030840338851e-05, 
       2.4922766670927189e-05, 
       2.1123041710218976e-05, 
       2.6064512283843695e-05, 
       2.5749495904983114e-05, 
       2.1310217221159616e-05, 
       2.1111653994676344e-05, 
       2.5077949988261698e-05, 
       2.9784471235163563e-05, 
       2.3056373773679594e-05, 
       2.9102145421663064e-05, 
       2.7483514799424028e-05, 
       2.8962947173945567e-05, 
       2.2711763764086440e-05, 
       2.2711763764086440e-05, 
       2.7200218741839871e-05, 
       2.9407851081748054e-05, 
       2.2205162355419221e-05, 
       2.4743849721618246e-05, 
       2.4532785214329851e-05, 
       2.8615733804263409e-05, 
       2.1740599758329869e-05, 
       2.7775020160739574e-05, 
       2.7231851287790759e-05, 
       2.4023947834927688e-05, 
       2.4613768342028950e-05, 
       2.1715685410280728e-05, 
       2.4704448207108885e-05, 
       2.0983761833621713e-05, 
       2.9268202828632838e-05, 
       2.9589588322323290e-05, 
       2.3819988431851243e-05, 
       2.9140865922690650e-05, 
       2.7016209725563762e-05, 
       2.4619807651610879e-05, 
       2.5340042480965205e-05, 
       2.3769275458079885e-05, 
       2.5389185845765271e-05, 
       2.6163251136164897e-05, 
       2.4037323377968369e-05, 
       2.2896673422365298e-05, 
       2.1062759073885588e-05, 
       2.8959262462343110e-05, 
       2.5692251912621091e-05, 
       2.6271725889237126e-05, 
       2.1650971366648380e-05, 
       2.0955979386201294e-05, 
       2.1116546838638918e-05, 
       2.4984552050094938e-05, 
       2.1437901128472993e-05, 
       2.9323671189211286e-05, 
       2.4692825644927512e-05, 
       2.6052954166854551e-05, 
       2.5214357617053483e-05, 
       2.6081102558987802e-05, 
       2.6475050434112720e-05, 
       2.2109725118767885e-05, 
       2.6001789423800533e-05, 
       2.8165089431936793e-05, 
       2.3352057237998144e-05, 
       2.4203662078715161e-05, 
       2.9127691585543378e-05, 
       2.9183274514529182e-05, 
       2.4295959119443233e-05, 
       2.2425610814530062e-05, 
       2.2030280284591683e-05, 
       2.8305956971834876e-05, 
       2.4830233957070959e-05, 
       2.7607803707495960e-05, 
       2.7607803707495960e-05, 
       2.1237476913355189e-05, 
       2.3586741764726801e-05, 
       2.1134841277494663e-05, 
       2.7810885568273507e-05, 
       2.2977942399483104e-05, 
       2.6428151577749415e-05, 
       2.1210739386215050e-05, 
       2.0848778399258320e-05, 
       2.1178276892523642e-05, 
       2.9087254520438652e-05, 
       2.4337536328356341e-05, 
       2.8566646731706197e-05, 
       2.4203451429410660e-05, 
       2.4703830364985675e-05, 
       2.1426201623577512e-05, 
       2.8956288614032272e-05, 
       2.8956288614032272e-05, 
       2.0636546521793716e-05, 
       2.0573885919396488e-05, 
       2.5401352461363626e-05, 
       2.5868753062338599e-05, 
       2.3027119709218912e-05, 
       2.3357590161797815e-05, 
       2.8191964995993427e-05, 
       2.8191964995993427e-05, 
       2.7058952232719377e-05, 
       2.6095050197318389e-05, 
       2.7813635560402981e-05, 
       2.6638296207266734e-05, 
       2.0662922795087871e-05, 
       2.1326781027017741e-05, 
       2.2295701266401901e-05, 
       2.1014264001594890e-05, 
       2.0757973111099345e-05, 
       2.1458026712368595e-05, 
       2.8274207074975589e-05, 
       2.4810695999304351e-05, 
       2.8323983624365901e-05, 
       2.5742038341567871e-05, 
       2.8508845781605500e-05, 
       2.0888701155268137e-05, 
       2.0529585232227509e-05, 
       2.5034772056485327e-05, 
       2.5030568775508550e-05, 
       2.5228473520783341e-05, 
       2.2251594537773418e-05, 
       2.3920430802565223e-05, 
       2.7173253407105435e-05, 
       2.7173253407105435e-05, 
       2.1532483995958935e-05, 
       2.3938053442539653e-05, 
       2.3921023616950308e-05, 
       2.2326028083594663e-05, 
       2.3155490562162931e-05, 
       2.8715113244671733e-05, 
       2.1717610844558287e-05, 
       2.0794380332537767e-05, 
       2.4581107671194017e-05, 
       2.3855107930500977e-05, 
       2.3931400718936076e-05, 
       2.6537028378583931e-05, 
       2.3382582675524286e-05, 
       2.7401846745912245e-05, 
       2.5487488713158126e-05, 
       2.1077974696978288e-05, 
       2.0288251412787598e-05, 
       2.7819966116245448e-05, 
       2.1919165600279391e-05, 
       2.0262590036713024e-05, 
       2.0262590036713024e-05, 
       2.2188504960183010e-05, 
       2.2127654568867594e-05, 
       2.2127654568867594e-05, 
       2.8180755148115264e-05, 
       2.8516499935546783e-05, 
       2.4835200329453488e-05, 
       2.4115984691999000e-05, 
       2.5046301541370833e-05, 
       2.2974631505232761e-05, 
       2.4013273726756287e-05, 
       2.7078990616854246e-05, 
       2.2102567056342413e-05, 
       2.0185751655454124e-05, 
       2.4316151966514346e-05, 
       2.7071017134061641e-05, 
       2.8348982008541965e-05, 
       2.3428860298172131e-05, 
       2.2196862156178476e-05, 
       2.6937453733626056e-05, 
       2.1150799509610855e-05, 
       2.3539659488581484e-05, 
       2.3482343892585202e-05, 
       2.7999252452952237e-05, 
       2.7628439205731162e-05, 
       2.4588039812213814e-05, 
       2.2701421608013592e-05, 
       2.7155021069766327e-05, 
       2.8121583758424230e-05, 
       2.0762602104814532e-05, 
       2.8193199377089744e-05, 
       2.5484025494064713e-05, 
       2.1457534367848610e-05, 
       2.6123013017904331e-05, 
       2.6123013017904331e-05, 
       2.2650471065666470e-05, 
       2.0384977972974928e-05, 
       2.0981830259274723e-05, 
       1.9999999999998111e-05, 
       2.4435962051764705e-05, 
       2.0560672706527655e-05, 
       2.0921608959167802e-05, 
       2.3320543730630196e-05, 
       2.7658989075695313e-05, 
       2.7983855192934815e-05, 
       2.0025529558097509e-05, 
       2.7924049859267542e-05, 
       2.7094752882854488e-05, 
       2.4750256870124958e-05, 
       1.9928238017228546e-05, 
       2.2697794512588630e-05, 
       2.7910623423382483e-05, 
       2.1803981359955448e-05, 
       2.2513133712243432e-05, 
       2.2487632562352765e-05, 
       2.6532785764768870e-05, 
       2.3247646278202567e-05, 
       1.9805385880804040e-05, 
       2.4020321050384608e-05, 
       2.0785188884872226e-05, 
       2.2168277952894067e-05, 
       1.9802802571600301e-05, 
       2.7302757843914607e-05, 
       1.9694814819074407e-05, 
       2.0438563368833077e-05, 
       2.3941827263979897e-05, 
       2.3080230150871152e-05, 
       2.0386440672725847e-05, 
       2.3196201706351199e-05, 
       2.3975270264584468e-05, 
       1.9664127420988511e-05, 
       2.2537078314622451e-05, 
       2.6440983083775817e-05, 
       2.6979270356365161e-05, 
       2.0736045787212078e-05, 
       2.3333914253345995e-05, 
       2.7007087635012078e-05, 
       2.0131096348224223e-05, 
       2.4244809465662699e-05, 
       2.7557277890439430e-05, 
       2.2490350813223323e-05, 
       2.0348301841789819e-05, 
       2.3626160000265987e-05, 
       2.1828982117440435e-05, 
       1.9827436590608716e-05, 
       1.9639964216823775e-05, 
       1.9728813156633697e-05, 
       2.0274162029293038e-05, 
       2.0603484843342109e-05, 
       1.9999999999998596e-05, 
       2.2248334806690697e-05, 
       2.2913328556012979e-05, 
       2.4238273249756075e-05, 
       2.7587174358110437e-05, 
       1.9888466296019138e-05, 
       2.0020788450740702e-05, 
       1.9533548444942853e-05, 
       2.2651202832857441e-05, 
       2.6446103004066458e-05, 
       2.1740974643964685e-05, 
       2.6901910246995046e-05, 
       2.4305129581516451e-05, 
       2.2488905444394759e-05, 
       2.1155955256729102e-05, 
       2.0703716797033260e-05, 
       2.2270526340128344e-05, 
       2.5843863748330956e-05, 
       2.6501376718398551e-05, 
       1.9944936673672440e-05, 
       2.2385726217078230e-05, 
       2.4696363978890080e-05, 
       2.7364476339361907e-05, 
       2.5812831975973625e-05, 
       2.4001103060774727e-05, 
       2.7340611114994528e-05, 
       2.1092902376393430e-05, 
       2.3213266729957517e-05, 
       2.2470683703778668e-05, 
       2.0392611886884281e-05, 
       1.9575513305912215e-05, 
       2.1356778847748766e-05, 
       2.6425080053357025e-05, 
       2.0751072619852740e-05, 
       2.7039588483966157e-05, 
       1.9280495550961758e-05, 
       2.1915014621335653e-05, 
       2.3466568508411861e-05, 
       2.7042029363104111e-05, 
       2.7192529976509706e-05, 
       1.9328067673915844e-05, 
       2.6989068563196528e-05, 
       2.6365586578272929e-05, 
       1.9224655318514847e-05, 
       2.7101016643272701e-05, 
       2.6346345686438382e-05, 
       2.2592616862561221e-05, 
       2.1388308055816657e-05, 
       2.6787463240450105e-05, 
       1.9418850589352804e-05, 
       2.0162229237477238e-05, 
       2.0744412428478471e-05, 
       2.6415827814525692e-05, 
       1.9477067696663569e-05, 
       2.5814645656456167e-05, 
       2.6826156528245559e-05, 
       2.6240325992772855e-05, 
       1.9586178829398509e-05, 
       2.6368433247245749e-05, 
       2.6790923230971601e-05, 
       2.3687207381010272e-05, 
       2.7005755157448488e-05, 
       2.3172311266359876e-05, 
       2.2254635078452681e-05, 
       2.3552660218191950e-05, 
       2.2398173304182567e-05, 
       2.3771650466816687e-05, 
       2.6917006544640887e-05, 
       2.3038848490733663e-05, 
       2.0101133478987235e-05, 
       2.0090020165552969e-05, 
       2.0816722050757227e-05, 
       2.2578505195366890e-05, 
       2.6934731795653595e-05, 
       2.0149226279840114e-05, 
       1.9725871970259640e-05, 
       1.9136164161308124e-05, 
       2.5371679533008806e-05, 
       2.1445739792044982e-05, 
       2.2985661158146827e-05, 
       2.0464658943518358e-05, 
       2.0464658943518358e-05, 
       2.2040326108959798e-05, 
       1.9068193289794883e-05, 
       2.3115999304309994e-05, 
       1.9179440609092591e-05, 
       1.9470251516125982e-05, 
       2.5562586340520214e-05, 
       1.9228055870160890e-05, 
       2.0008225281875002e-05, 
       1.9065959945762624e-05, 
       1.9447883614347173e-05, 
       2.0695028792671969e-05, 
       1.9138796796021523e-05, 
       2.6664007448672790e-05, 
       2.2982371947980182e-05, 
       2.1826380928158985e-05, 
       2.1426388751940305e-05, 
       2.2349770540388181e-05, 
       2.3196359932591713e-05, 
       2.6602166401022034e-05, 
       2.1400451297735498e-05, 
       2.5297577505044151e-05, 
       2.3189235287865066e-05, 
       2.2134251156477077e-05, 
       2.1063674826185046e-05, 
       2.2503202268728972e-05, 
       1.9090079825955896e-05, 
       1.9090079825955896e-05, 
       1.9819687896412248e-05, 
       1.9237567574377060e-05, 
       1.9491526676154471e-05, 
       2.5243153123067677e-05, 
       2.1505240942907195e-05, 
       2.2531502469511492e-05, 
       2.2012556906109524e-05, 
       2.1246945279320117e-05, 
       2.4966842938183963e-05, 
       1.9893459957904125e-05, 
       1.9707958363798394e-05, 
       1.9187586038552021e-05, 
       1.9181439651334718e-05, 
       1.9978335979757786e-05, 
       2.4865216543862584e-05, 
       1.9734538324161951e-05, 
       2.2572015976622474e-05, 
       2.0282776646234093e-05, 
       2.0084592507547901e-05, 
       2.1914426599946710e-05, 
       1.9673716799595695e-05, 
       2.1691931175728616e-05, 
       2.4372224247570914e-05, 
       2.4372224247570914e-05, 
       1.9006468975859341e-05, 
       2.4458949583607247e-05, 
       2.0538444088119941e-05, 
       2.6163678466691344e-05, 
       1.9747456291983756e-05, 
       1.8636187270806300e-05, 
       1.9009836508943332e-05, 
       2.1911165843983136e-05, 
       2.1624708666460605e-05, 
       1.8693752491082285e-05, 
       2.5787068643643150e-05, 
       2.5132587746355704e-05, 
       2.1239152732205980e-05, 
       1.8796481878728683e-05, 
       2.5776450751655573e-05, 
       1.8927684817619635e-05, 
       1.9557109881434541e-05, 
       2.2740069998641482e-05, 
       1.9567363171964758e-05, 
       2.1792521487500579e-05, 
       2.2813287725737651e-05, 
       2.1892987785546046e-05, 
       1.9375388272943816e-05, 
       1.8873864999461511e-05, 
       2.1357837712905821e-05, 
       1.8754340642006708e-05, 
       2.3152728096937984e-05, 
       1.9121671248429618e-05, 
       2.2327309484392186e-05, 
       2.2742171677133305e-05, 
       2.0728844413406723e-05, 
       1.9874339648149293e-05, 
       2.1869385036547042e-05, 
       2.6034821136249018e-05, 
       2.1995402891497991e-05, 
       2.1869850324289869e-05, 
       1.9556681695726242e-05, 
       2.1277419710744966e-05, 
       2.0715242061331388e-05, 
       2.1681874417413688e-05, 
       2.0007606984251554e-05, 
       1.8775064099226119e-05, 
       1.9078681243448876e-05, 
       2.2653326413630240e-05, 
       2.5225062678109871e-05, 
       1.8673774283287912e-05, 
       1.9050824102138936e-05, 
       1.8425960945520104e-05, 
       1.9734445336062810e-05, 
       2.1854409703071268e-05, 
       2.5705538340808325e-05, 
       2.5766723524056884e-05, 
       2.1498796117001881e-05, 
       2.5593522429008230e-05, 
       2.4388741541868519e-05, 
       2.1441527713789161e-05, 
       2.3410514219344735e-05, 
       1.8808368343945443e-05, 
       2.4254438336572184e-05, 
       1.8462885968707351e-05, 
       2.2871133218626609e-05, 
       2.5536695184870963e-05, 
       2.5739755318344120e-05, 
       1.9999999999997684e-05, 
       2.4376183099517822e-05, 
       2.1836704331587505e-05, 
       2.1365198219423314e-05, 
       2.1377118558596892e-05, 
       1.8525276165632930e-05, 
       2.4398119237534047e-05, 
       2.5666956243073106e-05, 
       2.2583614748870743e-05, 
       2.2158308025661018e-05, 
       1.9847157166394711e-05, 
       1.9468243403322062e-05, 
       2.5204343752716428e-05, 
       1.8880900621100187e-05, 
       2.2668065591133145e-05, 
       2.1425158584596436e-05, 
       2.1259834454645140e-05, 
       2.0016991355826771e-05, 
       1.9680388165063498e-05, 
       2.5470336005178848e-05, 
       2.4373783826419086e-05, 
       1.8098510402825817e-05, 
       1.8352430349971238e-05, 
       1.8594681754406291e-05, 
       2.2282838825220545e-05, 
       2.5502255093924968e-05, 
       2.5448910520566233e-05, 
       2.2037187161003800e-05, 
       1.8844221243828147e-05, 
       2.2952532144424310e-05, 
       2.3894831095443319e-05, 
       2.5441872124714022e-05, 
       2.1158092788023229e-05, 
       2.4322585026855575e-05, 
       1.8533930663034295e-05, 
       2.2998279765040865e-05, 
       2.4876667588630600e-05, 
       2.4981786707723841e-05, 
       2.5163273217196913e-05, 
       1.7969319536917142e-05, 
       1.7969319536917142e-05, 
       1.8909462693514672e-05, 
       2.1045781535115987e-05, 
       2.0719354432601130e-05, 
       2.0115437166001478e-05, 
       2.4601214841607067e-05, 
       2.1054952803658941e-05, 
       1.9581443679071778e-05, 
       2.3276897413604886e-05, 
       2.3276897413604886e-05, 
       1.9187780126464548e-05, 
       1.9311392199925316e-05, 
       2.5272550638326800e-05, 
       1.8300532878472574e-05, 
       2.4058511959979058e-05, 
       1.8702402504351326e-05, 
       1.9132021117095319e-05, 
       2.4903796606459513e-05, 
       1.9337408323683094e-05, 
       2.5172357235331020e-05, 
       2.4321284906538726e-05, 
       2.0520706678718408e-05, 
       2.3778894425124177e-05, 
       2.3676838813413018e-05, 
       1.7838204932429741e-05, 
       2.0830410808266415e-05, 
       2.4960731576209159e-05, 
       2.3566771526061548e-05, 
       2.0112552151498836e-05, 
       1.9123739417627946e-05, 
       1.8561524207988734e-05, 
       2.4673839508180660e-05, 
       1.9906903730058719e-05, 
       2.1168295386925452e-05, 
       2.0752916294115517e-05, 
       2.4709149553722373e-05, 
       1.7767541327249107e-05, 
       1.7763700054793674e-05, 
       2.0141943014697284e-05, 
       2.0268969440731765e-05, 
       2.1554692848064810e-05, 
       1.7991720512962049e-05, 
       2.1740473510379552e-05, 
       2.0815827632765828e-05, 
       1.8361873249861767e-05, 
       2.4830514986061490e-05, 
       2.1792649518925854e-05, 
       2.1047010497146118e-05, 
       1.9492208692842696e-05, 
       1.7664987355903558e-05, 
       1.7664987355903558e-05, 
       2.1251316770795840e-05, 
       2.2607753054010485e-05, 
       1.8608857124891965e-05, 
       2.4934759550667732e-05, 
       2.4925523841027443e-05, 
       1.8194042024081950e-05, 
       1.8460627084400566e-05, 
       2.0361623128963163e-05, 
       2.1141057200037726e-05, 
       2.4043785780287165e-05, 
       2.4833198239755762e-05, 
       1.8145610318622196e-05, 
       2.2228509624992713e-05, 
       2.1132024306760404e-05, 
       1.7642051084354999e-05, 
       1.8086666707132975e-05, 
       1.9249884487151741e-05, 
       1.8255487502303842e-05, 
       1.7956939739931842e-05, 
       1.7731134423061075e-05, 
       2.4754713874808710e-05, 
       2.0686297667369626e-05, 
       1.7517170134011922e-05, 
       1.7567707248893300e-05, 
       2.3371460974311176e-05, 
       1.8437092303982169e-05, 
       2.0372370831356654e-05, 
       1.8204426550607204e-05, 
       1.7553804130884323e-05, 
       1.8335748846230362e-05, 
       2.4617157935715516e-05, 
       1.8351476526719662e-05, 
       1.7831190145844371e-05, 
       1.7561766834944122e-05, 
       2.3204551666706484e-05, 
       2.0425201813119722e-05, 
       2.0953774414409753e-05, 
       1.7964182941566314e-05, 
       2.3938995201209839e-05, 
       2.4477283634413058e-05, 
       1.8127032897212113e-05, 
       1.7477988349187017e-05, 
       1.9134103353779846e-05, 
       2.4504287059597024e-05, 
       1.7645478902118700e-05, 
       2.0258348840739947e-05, 
       1.7815881636178072e-05, 
       2.0416046338673935e-05, 
       1.7400553241784334e-05, 
       1.7546115789295338e-05, 
       1.8332272741464264e-05, 
       1.8617529477667705e-05, 
       1.7500691077864170e-05, 
       2.2626327304652426e-05, 
       2.2227514768022570e-05, 
       1.8188667994350754e-05, 
       1.7437844563341825e-05, 
       2.0314220634938714e-05, 
       1.7691905454582359e-05, 
       1.8765674276470071e-05, 
       2.3082902752773052e-05, 
       1.7706569415143102e-05, 
       1.8228139491229842e-05, 
       2.1552573926884468e-05, 
       1.8519689604015046e-05, 
       2.3993102372972416e-05, 
       2.4288245220394133e-05, 
       2.2183634289793656e-05, 
       1.9552071132927956e-05, 
       2.0759479813238321e-05, 
       1.8457204441878411e-05, 
       1.7292463119650519e-05, 
       1.9596345964718906e-05, 
       2.4327606169512865e-05, 
       2.4339106425848476e-05, 
       2.0285040788588862e-05, 
       1.9862348129452729e-05, 
       1.7963271883623455e-05, 
       2.3958109454929556e-05, 
       2.0081950436156026e-05, 
       2.1879464641930097e-05, 
       2.3311196480404057e-05, 
       2.3664991134883837e-05, 
       1.7570857835264645e-05, 
       2.3816467195815987e-05, 
       2.3969057885338020e-05, 
       1.7684340607261483e-05, 
       1.7130623687063773e-05, 
       1.8004413730683098e-05, 
       2.4192829489753538e-05, 
       1.7642895397066091e-05, 
       1.8061073828648228e-05, 
       1.9744182333377942e-05, 
       1.7700876766767885e-05, 
       1.7687369787554525e-05, 
       1.7661716713810105e-05, 
       2.4112509382338131e-05, 
       1.7251645577806190e-05, 
       1.8318965404654398e-05, 
       1.7958747158205470e-05, 
       2.0141697344594615e-05, 
       2.4078595848649804e-05, 
       1.7158024940848134e-05, 
       2.3587342040018002e-05, 
       2.1353213766013239e-05, 
       1.7632049491289552e-05, 
       1.7553541999937986e-05, 
       2.0632681521166749e-05, 
       2.0632681521166749e-05, 
       2.0094466779403417e-05, 
       1.8242622188536006e-05, 
       2.2661751330096227e-05, 
       2.1655401501450158e-05, 
       2.2973915559528669e-05, 
       1.7857781946628933e-05, 
       2.3874064025257466e-05, 
       1.7466289208002739e-05, 
       1.7868190241998999e-05, 
       1.8522292841140415e-05, 
       1.9541454144670358e-05, 
       2.0360858386803245e-05, 
       1.7745433564095962e-05, 
       2.1450608391854456e-05, 
       1.7873924089632360e-05, 
       1.7873924089632360e-05, 
       1.7642268004839998e-05, 
       1.7007271490848372e-05, 
       2.3008785724228431e-05, 
       2.3012536452576482e-05, 
       1.8835388379737464e-05, 
       2.3888953016319224e-05, 
       1.8257329889199428e-05, 
       1.8056709382823948e-05, 
       1.9869298923376563e-05, 
       2.1964741924384022e-05, 
       1.9822422962427555e-05, 
       1.7723877559109750e-05, 
       1.9752375106067294e-05, 
       1.7562414468378501e-05, 
       2.3820159096627734e-05, 
       1.7849041965527568e-05, 
       1.8178132738886788e-05, 
       1.7933421962478748e-05, 
       2.0442580535800413e-05, 
       1.8737388710914070e-05, 
       1.8010655480464308e-05, 
       1.7499746820761987e-05, 
       1.9628184551172320e-05, 
       2.0135454702788720e-05, 
       1.9783768630407857e-05, 
       1.7027457476285222e-05, 
       2.0439839165013294e-05, 
       1.7668140506123545e-05, 
       2.2887222766642335e-05, 
       1.8625722075590894e-05, 
       1.6774244951824475e-05, 
       1.9355861887763674e-05, 
       1.9346576290592440e-05, 
       1.8886452560261844e-05, 
       2.3428397754813264e-05, 
       1.9147065979715298e-05, 
       1.7953202929927740e-05, 
       2.0126846824773085e-05, 
       2.0276715003319864e-05, 
       2.3371966026493449e-05, 
       1.7718215711264086e-05, 
       1.8327956841474887e-05, 
       2.3475965054884663e-05, 
       2.1220863521424081e-05, 
       1.8344353342725405e-05, 
       2.3379306655953676e-05, 
       1.7756953192344690e-05, 
       1.8927844511496484e-05, 
       2.1319615922844804e-05, 
       2.3149571951534485e-05, 
       1.7266925840009747e-05, 
       1.9727835207955215e-05, 
       1.9493125385176378e-05, 
       2.2569242370532613e-05, 
       1.6570225727416707e-05, 
       2.0761943236018171e-05, 
       1.9277691130070734e-05, 
       1.7412207746104813e-05, 
       1.6794022527098285e-05, 
       1.9911347833599989e-05, 
       2.1787554130159452e-05, 
       1.8588028392662681e-05, 
       1.6711214286223893e-05, 
       1.8880801601497662e-05, 
       1.7805419113968902e-05, 
       1.8199539146443171e-05, 
       1.8193725676805197e-05, 
       2.2137212536092185e-05, 
       1.9290095584840817e-05, 
       1.7483531629355937e-05, 
       1.7263911772972458e-05, 
       1.9291348274936961e-05, 
       2.0711584120144664e-05, 
       1.7711451284701733e-05, 
       1.9679286515370669e-05, 
       2.2966962280570155e-05, 
       1.9212163216954399e-05, 
       1.7255996582263717e-05, 
       1.7231387705174675e-05, 
       2.1930982813245604e-05, 
       1.7410850631496899e-05, 
       1.7410850631496899e-05, 
       1.8162260557585636e-05, 
       1.8838587001928264e-05, 
       1.7291093427628760e-05, 
       2.1562991469965018e-05, 
       2.2955954995703078e-05, 
       2.2138774233384432e-05, 
       1.6891504861024997e-05, 
       1.6924801108667900e-05, 
       1.8636187270804091e-05, 
       1.9854450989405549e-05, 
       2.2187440342377361e-05, 
       1.6336203316387930e-05, 
       1.6336203316387930e-05, 
       2.2125101210696940e-05, 
       1.7199656862921846e-05, 
       2.1163216656948455e-05, 
       2.1163216656948455e-05, 
       1.8459217397607133e-05, 
       1.6337360455581812e-05, 
       2.2125176236958186e-05, 
       2.2550082661318860e-05, 
       1.6234615335634816e-05, 
       2.1496712981959119e-05, 
       2.0485376621618394e-05, 
       1.7494093464794439e-05, 
       1.6487806552309386e-05, 
       2.1110132354203416e-05, 
       1.8702551269710746e-05, 
       2.2865396123724519e-05, 
       2.0350022676992984e-05, 
       2.2838498108133499e-05, 
       1.9784249880636262e-05, 
       2.0292191214828813e-05, 
       1.8685372482041398e-05, 
       2.2655086304048908e-05, 
       1.7304808887747554e-05, 
       1.6703681152645012e-05, 
       2.2788125298875008e-05, 
       2.2128691640789324e-05, 
       1.9293006539837878e-05, 
       1.8616979793947943e-05, 
       2.1754197699099749e-05, 
       2.2370830009755466e-05, 
       2.2370830009755466e-05, 
       2.1058322901314090e-05, 
       2.1082277701276365e-05, 
       2.2724673823789762e-05, 
       1.6722677059682963e-05, 
       2.2611698338101417e-05, 
       1.8820708615568329e-05, 
       1.9115688755658775e-05, 
       1.6234721814418811e-05, 
       1.7084719343943476e-05, 
       1.9366446164047562e-05, 
       1.6259222281809273e-05, 
       1.6335206644752317e-05, 
       1.7313040984674325e-05, 
       1.7313040984674325e-05, 
       1.9470156832045873e-05, 
       1.6122471032662713e-05, 
       1.6224475435896360e-05, 
       2.2642168998363644e-05, 
       2.2632537119189582e-05, 
       1.6592295820877356e-05, 
       2.1366640852304917e-05, 
       1.7087754208338704e-05, 
       1.6142464685406160e-05, 
       1.8212730203929584e-05, 
       1.6062205609472556e-05, 
       1.7317044300247156e-05, 
       1.6710506133544104e-05, 
       1.6718948384614613e-05, 
       1.6718948384614613e-05, 
       2.0992852238434180e-05, 
       2.1186438281349700e-05, 
       1.6373805636870852e-05, 
       1.6953638512254993e-05, 
       1.8072305094187577e-05, 
       1.8056140424669700e-05, 
       1.8056140424669700e-05, 
       1.6098907761922259e-05, 
       2.2234532934795830e-05, 
       1.9527558213273912e-05, 
       1.7716118858467923e-05, 
       2.2554816209846009e-05, 
       1.6405645902497258e-05, 
       1.6507591631663653e-05, 
       1.6028698901246867e-05, 
       1.6610357546445781e-05, 
       2.0891228865120608e-05, 
       1.7205477237655727e-05, 
       1.6285032053036909e-05, 
       1.6102190176239627e-05, 
       2.0038516713607140e-05, 
       1.8727638036245882e-05, 
       1.9418850589350212e-05, 
       2.0003273484876013e-05, 
       2.1906878577715012e-05, 
       1.6532627116963400e-05, 
       1.7667379260515993e-05, 
       1.7667379260515993e-05, 
       1.8996600645954346e-05, 
       1.6474853520918024e-05, 
       1.6754965910496321e-05, 
       1.9131977728321137e-05, 
       2.0359016436841823e-05, 
       1.6171135481124933e-05, 
       1.6994505531946206e-05, 
       1.6994505531946206e-05, 
       2.2407763531605025e-05, 
       1.8617808635099769e-05, 
       1.7970666720639915e-05, 
       1.7818094752935917e-05, 
       2.2278244153232194e-05, 
       1.7807560102282335e-05, 
       2.2292872493900075e-05, 
       1.6011178376679929e-05, 
       2.1130905491636213e-05, 
       1.8389816834979284e-05, 
       1.7518317671006143e-05, 
       1.8386936279129174e-05, 
       1.9289791732180916e-05, 
       1.8390802048013865e-05, 
       2.1841772793681454e-05, 
       1.7023838490886034e-05, 
       1.6430075484698597e-05, 
       1.5794344155936214e-05, 
       1.6383421816699133e-05, 
       2.1836304668254267e-05, 
       1.5947436384747194e-05, 
       1.6658611312546870e-05, 
       1.8996568006323307e-05, 
       1.5726667288329263e-05, 
       1.7203608654114123e-05, 
       1.6603196937560013e-05, 
       1.6420403016835570e-05, 
       1.8458044682778509e-05, 
       2.0504660640626798e-05, 
       2.1163618511902066e-05, 
       2.1735775776141427e-05, 
       1.9291869054352809e-05, 
       2.2156388020013238e-05, 
       1.5738661564135976e-05, 
       1.8892177115344497e-05, 
       1.7044119165090520e-05, 
       1.7873537991545251e-05, 
       1.7742036488274337e-05, 
       1.5974291757940324e-05, 
       1.7725308376978671e-05, 
       2.2107213301778072e-05, 
       1.8390227544060127e-05, 
       2.0029172972782206e-05, 
       1.7434739033936376e-05, 
       1.5653113198686018e-05, 
       1.6068993551798857e-05, 
       1.8932653957189339e-05, 
       2.0256523544959617e-05, 
       1.9252343903262409e-05, 
       1.6044104462389430e-05, 
       2.1089221346517688e-05, 
       1.6043761918339424e-05, 
       1.5997479462943513e-05, 
       1.8328406323002826e-05, 
       2.1954356616487514e-05, 
       1.9428513590515729e-05, 
       1.8019306688431413e-05, 
       1.5795121486224743e-05, 
       1.5579582249498965e-05, 
       1.7103957557751057e-05, 
       1.5570399547994975e-05, 
       2.1849104957647042e-05, 
       1.8073135630927389e-05, 
       1.6513389478517118e-05, 
       1.8273652355757713e-05, 
       2.1144803368417817e-05, 
       2.0270387868887738e-05, 
       1.6240298468458713e-05, 
       1.9746705821200908e-05, 
       1.8176373571861887e-05, 
       1.7468726387310578e-05, 
       1.8237445111701578e-05, 
       1.7451376992430054e-05, 
       1.7044135398653662e-05, 
       1.7044135398653662e-05, 
       1.9502404229276985e-05, 
       1.6069453006010668e-05, 
       1.6385605152059634e-05, 
       2.1787275947365716e-05, 
       1.6092117797190203e-05, 
       2.1808978138339834e-05, 
       1.8910890863139921e-05, 
       1.7320251751630813e-05, 
       1.6669209234786041e-05, 
       2.0002068595933454e-05, 
       2.0721517790829148e-05, 
       2.1175748194723008e-05, 
       1.6352279732357424e-05, 
       1.5588799981723169e-05, 
       1.5596929345692841e-05, 
       2.1598519054007094e-05, 
       1.6341032883464118e-05, 
       1.6037577515563460e-05, 
       1.8198648681853879e-05, 
       1.8198648681853879e-05, 
       1.6304655116409151e-05, 
       2.1106709550229482e-05, 
       2.1617499740261796e-05, 
       1.5429339135867125e-05, 
       1.5912453233245103e-05, 
       1.5478302911601509e-05, 
       1.6553374800561391e-05, 
       1.6570634637199210e-05, 
       1.7597865511431501e-05, 
       1.5439674044828063e-05, 
       1.9608929313707292e-05, 
       1.7060817982284103e-05, 
       1.6504583528432646e-05, 
       2.0423577083621921e-05, 
       1.5426516787865581e-05, 
       1.5426516787865581e-05, 
       1.8991887595935154e-05, 
       1.5432803594276111e-05, 
       1.9414015532927609e-05, 
       1.5492143234340858e-05, 
       1.6790153048861613e-05, 
       2.0000000000002919e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000002302e-05, 
       2.0000000000002302e-05, 
       2.0123214111927458e-05, 
       1.9134195875659283e-05, 
       2.1611367090571530e-05, 
       1.8053293945355630e-05, 
       1.5880767113893326e-05, 
       1.7723619065575681e-05, 
       1.7026402635406914e-05, 
       1.7941155666721253e-05, 
       2.1239128676142639e-05, 
       1.5532530466566838e-05, 
       1.6092455651163923e-05, 
       2.1614673492820541e-05, 
       1.5302442165784327e-05, 
       1.5371800463024751e-05, 
       1.6301383742497040e-05, 
       1.5372985229919656e-05, 
       1.6134008305294374e-05, 
       1.6335296778040991e-05, 
       1.6841040774595513e-05, 
       1.6073508541610085e-05, 
       1.5337011740924597e-05, 
       1.9327005042392512e-05, 
       2.0936607644920401e-05, 
       1.8438087968188434e-05, 
       1.6159618091167911e-05, 
       1.5691290674776830e-05, 
       1.7512542040997598e-05, 
       1.6912588626088647e-05, 
       1.5788332881296314e-05, 
       2.1503293331511378e-05, 
       1.8532975911561430e-05, 
       1.5200325041619105e-05, 
       1.8052713942789469e-05, 
       1.8536055581128455e-05, 
       1.9138944246306821e-05, 
       1.8197005931973759e-05, 
       1.8197005931973759e-05, 
       1.5326171036394532e-05, 
       1.6549433215411359e-05, 
       1.5277926910026729e-05, 
       1.6352611865435243e-05, 
       2.1408440169012478e-05, 
       2.1331593915833227e-05, 
       2.1162908630631478e-05, 
       1.7856444264198156e-05, 
       1.9819216586925235e-05, 
       1.5913074729844049e-05, 
       1.6959108126368575e-05, 
       1.9625463956564047e-05, 
       1.6022982921782222e-05, 
       2.1335613287615755e-05, 
       1.7336758653690069e-05, 
       1.7336417810835606e-05, 
       1.6014421515239101e-05, 
       1.8274039956081468e-05, 
       1.7850855189331426e-05, 
       1.5081305769075736e-05, 
       2.1232787288345027e-05, 
       1.5097430599505189e-05, 
       1.5160385257255164e-05, 
       2.0472366386909940e-05, 
       1.5840265349244938e-05, 
       1.8218932701100731e-05, 
       2.0493155196437179e-05, 
       1.7104780513039831e-05, 
       2.1237469975771126e-05, 
       1.5993331306734184e-05, 
       1.7144170276246923e-05, 
       1.8142196848276659e-05, 
       1.9056615108258974e-05, 
       2.1017973319543292e-05, 
       1.6630590717646387e-05, 
       1.8026372337631166e-05, 
       1.5663953166791747e-05, 
       1.8677585553408706e-05, 
       1.9005941589898054e-05, 
       2.0491367724795340e-05, 
       1.5398119269448739e-05, 
       1.7795917071237724e-05, 
       1.6525252283402562e-05, 
       2.0935352653952154e-05, 
       1.7020357703059911e-05, 
       1.6884866524177841e-05, 
       2.1049454017331673e-05, 
       2.0173245753953342e-05, 
       2.1006587041820796e-05, 
       1.5840673816212893e-05, 
       1.6560409888110331e-05, 
       1.5810346852933602e-05, 
       2.0800370930636108e-05, 
       2.0847348923073888e-05, 
       1.7067499214287063e-05, 
       1.5013179642766518e-05, 
       1.7714997793260417e-05, 
       1.6956928977768807e-05, 
       1.9999999999997342e-05, 
       1.9999999999997342e-05, 
       1.8456261803637364e-05, 
       1.5912687180576364e-05, 
       1.4960456400075546e-05, 
       1.4888882831800838e-05, 
       1.8184846829995311e-05, 
       1.8184846829995311e-05, 
       1.4853009727722440e-05, 
       2.0578529469769259e-05, 
       2.0977449158401122e-05, 
       1.7443887309016557e-05, 
       1.8909286191306884e-05, 
       1.6966481931461809e-05, 
       1.6556459995739591e-05, 
       1.6752037303715518e-05, 
       2.0018554255881626e-05, 
       1.5628571172107139e-05, 
       1.4842777408290094e-05, 
       1.7310397120016435e-05, 
       1.4947440150134089e-05, 
       1.5407206684160909e-05, 
       1.5407206684160909e-05, 
       1.6490443897643008e-05, 
       2.0884794716715465e-05, 
       1.8488805778599490e-05, 
       1.6898576170893853e-05, 
       1.4774242673951012e-05, 
       1.5632457593345169e-05, 
       1.7261613309425577e-05, 
       1.7155922347204155e-05, 
       1.5326543904164046e-05, 
       1.7326792718584700e-05, 
       1.5319825549543372e-05, 
       1.5463498838618996e-05, 
       1.5463498838618996e-05, 
       1.7316095803796125e-05, 
       1.5598099978153576e-05, 
       1.7298086056708285e-05, 
       2.0678661369356820e-05, 
       1.7099494813622297e-05, 
       1.5728453803341573e-05, 
       1.7432757133150035e-05, 
       1.5160168419881993e-05, 
       1.7816461753330562e-05, 
       1.9580880461032115e-05, 
       1.6472916584417822e-05, 
       2.0474582919533831e-05, 
       1.6492713123745396e-05, 
       1.5602647159461790e-05, 
       1.7417914327781255e-05, 
       1.7430518082606324e-05, 
       1.7200872561710810e-05, 
       1.7594505467611897e-05, 
       1.4593374291172440e-05, 
       1.4615829990183012e-05, 
       1.6970189109472224e-05, 
       1.4595680036653840e-05, 
       1.7158899171503057e-05, 
       2.0387654478843343e-05, 
       1.4978906698363142e-05, 
       2.0155956683865661e-05, 
       1.7187787360644636e-05, 
       1.4789011693835631e-05, 
       1.7666481865181118e-05, 
       1.6470282452882369e-05, 
       2.0284471522640487e-05, 
       1.5267740543673378e-05, 
       1.6462580885715711e-05, 
       1.5115179205604446e-05, 
       1.8278406388334036e-05, 
       1.6444101319305045e-05, 
       1.5539970999924486e-05, 
       1.7290020306619490e-05, 
       1.4581378540356226e-05, 
       1.5180179612361038e-05, 
       1.4942897866165691e-05, 
       1.9743080684653949e-05, 
       1.9989758912136145e-05, 
       1.5008135119909638e-05, 
       1.7373965946285542e-05, 
       1.4479154891499769e-05, 
       1.4485316501184317e-05, 
       1.7536488865533303e-05, 
       1.9623937145625572e-05, 
       1.4457394980177487e-05, 
       1.6319321257813999e-05, 
       1.4539641364728309e-05, 
       1.5276729671877384e-05, 
       1.7674272465634721e-05, 
       1.8579230583364200e-05, 
       1.7681898591113989e-05, 
       1.8838002625992734e-05, 
       1.7500734554702812e-05, 
       1.4430863000435895e-05, 
       1.4535642083636242e-05, 
       2.0031152627917139e-05, 
       1.4577317255433693e-05, 
       1.4715791150096691e-05, 
       2.0295526242962622e-05, 
       1.6377442125886164e-05, 
       2.0319378355387940e-05, 
       1.5185683324654169e-05, 
       1.8722989770466373e-05, 
       1.7157723980489144e-05, 
       1.9649753986370153e-05, 
       2.0246860646734095e-05, 
       1.7780798413728221e-05, 
       1.6378411618912062e-05, 
       1.7854347085060371e-05, 
       2.0280975871608747e-05, 
       1.6367406998208899e-05, 
       1.6765526193631551e-05, 
       1.6129318341749104e-05, 
       1.4459745708239373e-05, 
       1.7117545156381083e-05, 
       1.6850357142189432e-05, 
       1.6564115568313463e-05, 
       1.5252014388780991e-05, 
       1.4721371293476008e-05, 
       1.4458951834839838e-05, 
       1.6766171326926528e-05, 
       1.5083196282175192e-05, 
       1.5320761042333061e-05, 
       1.5320761042333061e-05, 
       1.9027424464127888e-05, 
       1.4749700525073902e-05, 
       1.6683798331944300e-05, 
       1.6791999751125377e-05, 
       1.7706857203696913e-05, 
       2.0005296688531713e-05, 
       1.6595973368406404e-05, 
       1.5034032999894572e-05, 
       1.5126322773709885e-05, 
       1.9986363807275739e-05, 
       1.6709325941348302e-05, 
       2.0053850375514413e-05, 
       1.4223540189186284e-05, 
       2.0037132798653616e-05, 
       1.4306005111882178e-05, 
       1.4784458429091257e-05, 
       1.4784458429091257e-05, 
       1.9792067331488810e-05, 
       1.4507994240964273e-05, 
       1.4570284655908726e-05, 
       1.4226306273688533e-05, 
       1.6175380175445676e-05, 
       1.4650543477103646e-05, 
       1.4695437694652338e-05, 
       1.4694844206046384e-05, 
       1.4753403157452437e-05, 
       1.4753403157452437e-05, 
       1.6369736954885356e-05, 
       1.9449524363557376e-05, 
       1.6025121652272348e-05, 
       2.0000000000000503e-05, 
       1.9999999999997362e-05, 
       1.9999999999997538e-05, 
       2.0000000000000852e-05, 
       1.9999999999998558e-05, 
       1.9999999999998558e-05, 
       2.0000000000004742e-05, 
       2.0000000000001686e-05, 
       2.0000000000001601e-05, 
       2.0000000000002875e-05, 
       2.0000000000000862e-05, 
       1.9999999999999382e-05, 
       1.9999999999998247e-05, 
       1.9999999999999551e-05, 
       2.0000000000001116e-05, 
       1.9999999999999138e-05, 
       2.0000000000000283e-05, 
       2.0000000000000947e-05, 
       2.0000000000001929e-05, 
       1.4275144858379013e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.8292226618068275e-05, 
       1.5084418188810893e-05, 
       1.8290229564389832e-05, 
       1.6578154038873046e-05, 
       1.4297554586141315e-05, 
       1.9496177904183695e-05, 
       1.8011077407310878e-05, 
       1.4898523869975374e-05, 
       1.4111929008584934e-05, 
       1.7128423186652288e-05, 
       1.4954562209032282e-05, 
       1.6188877412179246e-05, 
       1.6185805494386691e-05, 
       1.8582378366695840e-05, 
       1.4482294464459094e-05, 
       1.5458285924510495e-05, 
       1.5875604022001212e-05, 
       1.4726196961436846e-05, 
       1.4726196961436846e-05, 
       1.7644837065831166e-05, 
       1.4225061342693049e-05, 
       1.5357222718776597e-05, 
       1.4221112565537283e-05, 
       1.4649874679356803e-05, 
       1.9174587312672647e-05, 
       1.5452373965760110e-05, 
       1.9695206628413968e-05, 
       1.4559764239267428e-05, 
       1.5077726642019496e-05, 
       1.6341843706471940e-05, 
       1.4650695995497018e-05, 
       1.4874401355918175e-05, 
       1.4611813467941683e-05, 
       1.6536959336557455e-05, 
       1.4095585803147418e-05, 
       1.8110808860485055e-05, 
       1.8935788653751514e-05, 
       1.6405407101598671e-05, 
       1.9494713950402057e-05, 
       1.9818464645475865e-05, 
       1.5641623130648651e-05, 
       1.8965868148730674e-05, 
       1.4786853098410819e-05, 
       1.7511499766096822e-05, 
       1.7371800959598735e-05, 
       1.7371800959598735e-05, 
       1.9693059287692170e-05, 
       1.9525000342476939e-05, 
       1.6348177454292356e-05, 
       1.5412157021002226e-05, 
       1.6256106337729249e-05, 
       1.6915966439772749e-05, 
       1.5157066212291830e-05, 
       1.4031057972296246e-05, 
       1.3952467785550451e-05, 
       1.6306341402721227e-05, 
       1.4019159315126013e-05, 
       1.6934626541642875e-05, 
       1.6306443902052956e-05, 
       1.6795545556722046e-05, 
       1.6366777185526895e-05, 
       1.7257599135686726e-05, 
       1.8908723722688233e-05, 
       1.3902432785640216e-05, 
       1.6320478483971147e-05, 
       1.6214035080290748e-05, 
       1.4869089766760915e-05, 
       1.5366423107994453e-05, 
       1.6162850240229664e-05, 
       1.9065158632882171e-05, 
       1.7313274740991905e-05, 
       1.7223009077964974e-05, 
       1.9398913964943017e-05, 
       1.5087951775884419e-05, 
       1.7812042220933701e-05, 
       1.4347426818214703e-05, 
       1.7792253676004883e-05, 
       1.5090639868874618e-05, 
       1.5067922795395602e-05, 
       1.7423519498994054e-05, 
       1.6436411634818005e-05, 
       1.9338856410587927e-05, 
       1.5134194369259809e-05, 
       1.9520627370044779e-05, 
       1.8173420693764944e-05, 
       1.3897658431095295e-05, 
       1.3805705622813171e-05, 
       1.9509538609945451e-05, 
       1.4746339834957969e-05, 
       1.6196914275664551e-05, 
       1.9423555238125053e-05, 
       1.6255504681146960e-05, 
       1.4448467695138742e-05, 
       1.9416435738647281e-05, 
       1.5091471276995217e-05, 
       1.5826070551859609e-05, 
       1.6573016682227819e-05, 
       1.3796127577441248e-05, 
       1.9433281450590757e-05, 
       1.6004114690697736e-05, 
       1.5913985293590652e-05, 
       1.9134029090375728e-05, 
       1.9419992242537505e-05, 
       1.8399476874794269e-05, 
       1.5565638875817932e-05, 
       1.8175752840565924e-05, 
       1.9257476212652068e-05, 
       1.7320699429900624e-05, 
       1.9393995692260808e-05, 
       1.7736044064910271e-05, 
       1.9249993135876063e-05, 
       1.7332491817108567e-05, 
       1.4705820446896152e-05, 
       1.8873422116603438e-05, 
       1.3714174653088546e-05, 
       1.5053238176136231e-05, 
       1.6000793488734814e-05, 
       1.7471407309996013e-05, 
       1.9115617417499850e-05, 
       1.4035041327082734e-05, 
       1.5154780626558336e-05, 
       1.3924333585404814e-05, 
       1.3924333585404814e-05, 
       1.5501360377022928e-05, 
       1.3797425075843989e-05, 
       1.6690377565218823e-05, 
       1.4209392044125464e-05, 
       1.5192168921665653e-05, 
       1.9310310714502314e-05, 
       1.6026681801802033e-05, 
       1.5379446118655621e-05, 
       1.6182087572781853e-05, 
       1.4414278015070796e-05, 
       1.4403712747404126e-05, 
       1.9106139992569447e-05, 
       1.6279535540267637e-05, 
       1.8823522614061252e-05, 
       1.8823522614061252e-05, 
       1.3992514652115460e-05, 
       1.6417989200898137e-05, 
       1.9062044393849746e-05, 
       1.4444584898110234e-05, 
       1.4156201937858308e-05, 
       1.6980828150998573e-05, 
       1.8225599700613522e-05, 
       1.3842949214899192e-05, 
       1.3925324343753706e-05, 
       1.9199250441596477e-05, 
       1.6340884995954673e-05, 
       1.4097997925162449e-05, 
       1.5110802450527575e-05, 
       1.5110802450527575e-05, 
       1.4118188534189489e-05, 
       1.5972507348626299e-05, 
       1.4949117314399967e-05, 
       1.3634268769191239e-05, 
       1.9176981138413062e-05, 
       1.5747926190215031e-05, 
       1.4529697734581688e-05, 
       1.9035875295255347e-05, 
       1.8644242886533229e-05, 
       1.7019058967098475e-05, 
       1.8943571938968203e-05, 
       1.4070283982784185e-05, 
       1.6521567681950814e-05, 
       1.5736450260849093e-05, 
       1.4478110207350232e-05, 
       1.5891044823812513e-05, 
       1.8396317263515655e-05, 
       1.3654047047661890e-05, 
       1.4949344645606630e-05, 
       1.3950721742957454e-05, 
       1.3629239211560011e-05, 
       1.8770199721783211e-05, 
       1.5625283557334548e-05, 
       1.5333111283465042e-05, 
       1.6752068316668764e-05, 
       1.4307254727976837e-05, 
       1.7834629041524526e-05, 
       1.4638886206675482e-05, 
       1.4682563538763840e-05, 
       1.8007016967909183e-05, 
       1.4713241671169784e-05, 
       1.3856152861851609e-05, 
       1.5598944511325250e-05, 
       1.5406023337684634e-05, 
       1.8530468800527959e-05, 
       1.6489273021796442e-05, 
       1.7716550230181951e-05, 
       1.8594642013859483e-05, 
       1.3422890685954698e-05, 
       1.3589465871217270e-05, 
       1.3387344641485087e-05, 
       1.3880605412900500e-05, 
       1.8098814396644403e-05, 
       1.3778090850651112e-05, 
       1.6259222281811902e-05, 
       1.4175680983774898e-05, 
       1.4109463605866338e-05, 
       1.8385627478743485e-05, 
       1.3521687157485570e-05, 
       1.8651763803404701e-05, 
       1.3377877231170490e-05, 
       1.6225787460026322e-05, 
       1.5715779055579045e-05, 
       1.6034263328024173e-05, 
       1.4448337077626683e-05, 
       1.5350431184082312e-05, 
       1.8644281493184915e-05, 
       1.3766684015347864e-05, 
       1.8383946509146261e-05, 
       1.6834144086824390e-05, 
       1.8493761399307631e-05, 
       1.6490789150431061e-05, 
       1.7583798779864384e-05, 
       1.5272064378446065e-05, 
       1.5480765625171037e-05, 
       1.8393271799071932e-05, 
       1.4532019910555164e-05, 
       1.4149199393426439e-05, 
       1.3723138294747032e-05, 
       1.3823324093956162e-05, 
       1.3776209334212682e-05, 
       1.4839416795349127e-05, 
       1.8067420031682479e-05, 
       1.3468431700565305e-05, 
       1.3290548121645528e-05, 
       1.3593186316793604e-05, 
       1.4968574602007244e-05, 
       1.8210432925077731e-05, 
       1.5053902918481532e-05, 
       1.8555653900424379e-05, 
       1.8569914396785366e-05, 
       1.3180648660796089e-05, 
       1.5043968157884262e-05, 
       1.3206602859679263e-05, 
       1.3554147747746022e-05, 
       1.3860988911090755e-05, 
       1.3185581351947644e-05, 
       1.8513598646233649e-05, 
       1.7849408706365214e-05, 
       1.8499087714372173e-05, 
       1.3993618033684512e-05, 
       1.4170496345724529e-05, 
       1.3152985080876476e-05, 
       1.4377014272742440e-05, 
       1.5416237163161959e-05, 
       1.5415667785852915e-05, 
       1.7331183274922722e-05, 
       1.3210886754495558e-05, 
       1.5382921909012251e-05, 
       1.3431839782307517e-05, 
       1.5947436384756142e-05, 
       1.7884633014238202e-05, 
       1.3073642148904387e-05, 
       1.8440413867756214e-05, 
       1.6756083051915222e-05, 
       1.3492696200253430e-05, 
       1.8198691373090194e-05, 
       1.3135375943926957e-05, 
       1.3471624840180043e-05, 
       1.3656889977458795e-05, 
       1.3020158379448295e-05, 
       1.7666894613209809e-05, 
       1.5341635851169986e-05, 
       1.5679624049124903e-05, 
       1.8397222288501354e-05, 
       1.5728709149685157e-05, 
       1.3917938394849733e-05, 
       1.3193237808113149e-05, 
       1.3567494731282809e-05, 
       1.3153650027517457e-05, 
       1.8167269613865688e-05, 
       1.3983227712855635e-05, 
       1.3066869628852629e-05, 
       1.3435099973742444e-05, 
       1.8034381080621541e-05, 
       1.6707778403101021e-05, 
       1.6696937979988921e-05, 
       1.5361515935679014e-05, 
       1.4770516228118091e-05, 
       1.4970903524730745e-05, 
       1.5238537885689830e-05, 
       1.4842777408289981e-05, 
       1.4780444790771307e-05, 
       1.8094725257512659e-05, 
       1.4967704431308001e-05, 
       1.8281152666798779e-05, 
       1.3595710434242635e-05, 
       1.8197931748287858e-05, 
       1.5853880063817633e-05, 
       1.3663150845888761e-05, 
       1.5454779573915931e-05, 
       1.5454779573915931e-05, 
       1.6673457291542528e-05, 
       1.6939810631577229e-05, 
       1.6939525358680649e-05, 
       1.3417110962764424e-05, 
       1.3417110962764424e-05, 
       1.7932256526001101e-05, 
       1.5690038087088703e-05, 
       1.5208755176494659e-05, 
       1.3488650103220160e-05, 
       1.3012791257583799e-05, 
       1.7499930853069511e-05, 
       1.3150432078271197e-05, 
       1.7467308246123641e-05, 
       1.3612116310638862e-05, 
       1.8081444624459543e-05, 
       1.6038070826472913e-05, 
       1.3105512486918082e-05, 
       1.4876698672437411e-05, 
       1.5369073508425687e-05, 
       1.4496813536277749e-05, 
       1.7650753964143016e-05, 
       1.5663433034602309e-05, 
       1.5106033963424716e-05, 
       1.6871251797605927e-05, 
       1.3413837951967647e-05, 
       1.5125577788070745e-05, 
       1.3552799548837164e-05, 
       1.6002207175500304e-05, 
       1.5808465650411301e-05, 
       1.8034202182915985e-05, 
       1.4459188583216419e-05, 
       1.4398822522986248e-05, 
       1.7701252955455914e-05, 
       1.7608637086252329e-05, 
       1.6744852224104611e-05, 
       1.7376241656315121e-05, 
       1.7348163414595373e-05, 
       1.5253214219338255e-05, 
       1.5253214219338255e-05, 
       1.8098285110705944e-05, 
       1.7842467350696516e-05, 
       1.5559274863015924e-05, 
       1.4561998883880659e-05, 
       1.5036896330495797e-05, 
       1.7971341195498525e-05, 
       1.2808210408468525e-05, 
       1.7405777785703126e-05, 
       1.7007270547102020e-05, 
       1.7499582875733780e-05, 
       1.8032706326538459e-05, 
       1.3480381240299832e-05, 
       1.4784256759997464e-05, 
       1.4682439644155401e-05, 
       1.3287028623768412e-05, 
       1.4484149999385861e-05, 
       1.6086427499347982e-05, 
       1.4921600419341173e-05, 
       1.6050879829408097e-05, 
       1.6430875556361242e-05, 
       1.4926256281201423e-05, 
       1.4409678223866859e-05, 
       1.4604845285442554e-05, 
       1.5893647408508373e-05, 
       1.7770542439493338e-05, 
       1.4849712600315778e-05, 
       1.4867061603209816e-05, 
       1.4578200825941145e-05, 
       1.3442171322900818e-05, 
       1.5386454023643783e-05, 
       1.2734060603846992e-05, 
       1.3352918692079359e-05, 
       1.4094315592579050e-05, 
       1.5533355736466470e-05, 
       1.5010608987705953e-05, 
       1.2760952897130246e-05, 
       1.2760952897130246e-05, 
       1.4377882311814148e-05, 
       1.4271191270747151e-05, 
       1.7147380911302687e-05, 
       1.7829846918456326e-05, 
       1.2978113923474221e-05, 
       1.2978113923474221e-05, 
       1.2800607764655122e-05, 
       1.7213613475570351e-05, 
       1.3170890314236146e-05, 
       1.3130123945228694e-05, 
       1.4051891351076992e-05, 
       1.7747538488951913e-05, 
       1.3254199608808187e-05, 
       1.5170060201130070e-05, 
       1.4804926255523652e-05, 
       1.7600723611033809e-05, 
       1.5285331596776503e-05, 
       1.5285331596776503e-05, 
       1.2834340235367653e-05, 
       1.3115351271004138e-05, 
       1.3278168088532595e-05, 
       1.2590310682667907e-05, 
       1.2616539312197141e-05, 
       1.2616539312197141e-05, 
       1.6239502110115467e-05, 
       1.3410059869523268e-05, 
       1.2739244896656666e-05, 
       1.3221287591580966e-05, 
       1.2619655692590581e-05, 
       1.6876150779395045e-05, 
       1.2761254104092911e-05, 
       1.2548240730186572e-05, 
       1.5504036107356138e-05, 
       1.2883017742100525e-05, 
       1.5482703441841687e-05, 
       1.4271414314979747e-05, 
       1.6237712184524923e-05, 
       1.4713665597139593e-05, 
       1.7519490454569938e-05, 
       1.3394370224564338e-05, 
       1.6447363289564098e-05, 
       1.7493372789529711e-05, 
       1.7675957442536731e-05, 
       1.3773721019678660e-05, 
       1.3632809795481211e-05, 
       1.4245151176498330e-05, 
       1.2671713762754604e-05, 
       1.3984775134693882e-05, 
       1.6340529027817826e-05, 
       1.6205885007740020e-05, 
       1.5479307111382507e-05, 
       1.4979060718438190e-05, 
       1.3251537322325006e-05, 
       1.3245180432867855e-05, 
       1.7640603615650854e-05, 
       1.6875280249171112e-05, 
       1.6197893075766203e-05, 
       1.7629239460338853e-05, 
       1.3952383661868425e-05, 
       1.2706963798939207e-05, 
       1.6427492641259547e-05, 
       1.4044605099708595e-05, 
       1.4108645504857299e-05, 
       1.5714139308584642e-05, 
       1.3415735146379706e-05, 
       1.3415735146379706e-05, 
       1.3833928002165956e-05, 
       1.4757991614962459e-05, 
       1.2503567669085296e-05, 
       1.6978977378017918e-05, 
       1.7572824210603769e-05, 
       1.5250019145736625e-05, 
       1.4021692643629530e-05, 
       1.3302507036345570e-05, 
       1.7264052493257041e-05, 
       1.2497796004100066e-05, 
       1.2443401476728252e-05, 
       1.4590982869632107e-05, 
       1.7495991400571698e-05, 
       1.7333265420611655e-05, 
       1.4544240948629550e-05, 
       1.5699269394890567e-05, 
       1.4630215557433655e-05, 
       1.7300478351871982e-05, 
       1.2378329505747897e-05, 
       1.5049836477043053e-05, 
       1.4834978998291240e-05, 
       1.5819836908762031e-05, 
       1.2375339233550892e-05, 
       1.2612583977005754e-05, 
       1.2538371429661807e-05, 
       1.6413248799603736e-05, 
       1.5025524595439353e-05, 
       1.7272799465774889e-05, 
       1.5647927718461434e-05, 
       1.5647927718461434e-05, 
       1.4805763062424026e-05, 
       1.2382999632591006e-05, 
       1.3310339837886794e-05, 
       1.6878513430882560e-05, 
       1.7227940946222278e-05, 
       1.7344081411861175e-05, 
       1.5302814970989250e-05, 
       1.7213004986443475e-05, 
       1.6130088687624654e-05, 
       1.5842188695780215e-05, 
       1.2538095956927858e-05, 
       1.4849813330292949e-05, 
       1.2801565767518424e-05, 
       1.6744115574630633e-05, 
       1.4384897928145751e-05, 
       1.2275123712787654e-05, 
       1.2275123712787654e-05, 
       1.6739318170894267e-05, 
       1.2291458867866464e-05, 
       1.2275240011089254e-05, 
       1.2319686348472004e-05, 
       1.3935460415845843e-05, 
       1.2560747955146874e-05, 
       1.4076398503961912e-05, 
       1.2740319679690527e-05, 
       1.7256056442118583e-05, 
       1.3065879010733709e-05, 
       1.2317403565032562e-05, 
       1.3929757073636674e-05, 
       1.5237747244415033e-05, 
       1.4776501382106372e-05, 
       1.4365729474100232e-05, 
       1.6603264069635658e-05, 
       1.3622059822933727e-05, 
       1.4911043224232789e-05, 
       1.2230871633200690e-05, 
       1.7107076903889589e-05, 
       1.5220259368270058e-05, 
       1.2910286416092554e-05, 
       1.4203808534110436e-05, 
       1.3234174537463257e-05, 
       1.2224037420052983e-05, 
       1.6660962321400290e-05, 
       1.2867605302766192e-05, 
       1.5908573856228586e-05, 
       1.4527999453640038e-05, 
       1.2853281289329972e-05, 
       1.2501838912653071e-05, 
       1.2446649622681885e-05, 
       1.2178918734974544e-05, 
       1.2178918734974544e-05, 
       1.2267360122042968e-05, 
       1.6650278213161410e-05, 
       1.2578933501377602e-05, 
       1.2852426510684521e-05, 
       1.2621328299211735e-05, 
       1.4746748772933602e-05, 
       1.6506990813598154e-05, 
       1.2311117626207192e-05, 
       1.2375227362546670e-05, 
       1.7066928687530193e-05, 
       1.4591846588279915e-05, 
       1.2762711398992711e-05, 
       1.2422509540169615e-05, 
       1.7034831309899153e-05, 
       1.4836276164334514e-05, 
       1.2510095761094118e-05, 
       1.3897268735376902e-05, 
       1.2756431345564313e-05, 
       1.6666206861287346e-05, 
       1.3701806360771379e-05, 
       1.5240098743275998e-05, 
       1.6933157802861485e-05, 
       1.3987630597275828e-05, 
       1.6239460021718436e-05, 
       1.2087122914189682e-05, 
       1.6968706058066496e-05, 
       1.6527730736026860e-05, 
       1.2238341461892484e-05, 
       1.4137614289372472e-05, 
       1.4157354101469494e-05, 
       1.3161443364041454e-05, 
       1.2697887288799018e-05, 
       1.6989548777283794e-05, 
       1.2132162113290513e-05, 
       1.4825963926568935e-05, 
       1.6525633023303252e-05, 
       1.3982563665918434e-05, 
       1.3614131502338191e-05, 
       1.4559643828280083e-05, 
       1.5413447901425963e-05, 
       1.2088372027115803e-05, 
       1.6735773988068960e-05, 
       1.2622847057133571e-05, 
       1.3655651535491917e-05, 
       1.2456712871527126e-05, 
       1.5892031373226817e-05, 
       1.2047173019681641e-05, 
       1.6896154279726885e-05, 
       1.6874378094446402e-05, 
       1.4041409516085108e-05, 
       1.5097369288541178e-05, 
       1.2236326210569871e-05, 
       1.4498492314310596e-05, 
       1.2129747073421302e-05, 
       1.2369703786470514e-05, 
       1.5982592891834027e-05, 
       1.3769009649801223e-05, 
       1.4001743229159612e-05, 
       1.6540876044223736e-05, 
       1.6221164609751807e-05, 
       1.3763053020101112e-05, 
       1.4325095098802108e-05, 
       1.2633952474857119e-05, 
       1.6329013277347899e-05, 
       1.3123614651706734e-05, 
       1.5021231011704797e-05, 
       1.3219114816026737e-05, 
       1.6759926188128403e-05, 
       1.6592881323971881e-05, 
       1.3890328679378200e-05, 
       1.4793950089679507e-05, 
       1.4613577962848466e-05, 
       1.2703345119109875e-05, 
       1.5951380463504746e-05, 
       1.3991862677995752e-05, 
       1.2249058797105467e-05, 
       1.1972523661408510e-05, 
       1.6700376618316970e-05, 
       1.1969876721777264e-05, 
       1.4312469097455324e-05, 
       1.5967498384523299e-05, 
       1.5008487372870637e-05, 
       1.6552382744300849e-05, 
       1.2212314103295063e-05, 
       1.2565981052576954e-05, 
       1.5766650069032707e-05, 
       1.4362171464586172e-05, 
       1.1879011217201128e-05, 
       1.5473636336196578e-05, 
       1.2147704058296473e-05, 
       1.2760228433953444e-05, 
       1.2517464257710940e-05, 
       1.3896056333865877e-05, 
       1.3206583426179921e-05, 
       1.1788798968672102e-05, 
       1.6658976447284278e-05, 
       1.5051683809196574e-05, 
       1.2227159182147019e-05, 
       1.1964453171392055e-05, 
       1.3293035593964773e-05, 
       1.4239868457034409e-05, 
       1.3577023077152769e-05, 
       1.5210733389587490e-05, 
       1.4478818833415534e-05, 
       1.2114514986123191e-05, 
       1.2199923874453162e-05, 
       1.3488248717679212e-05, 
       1.5788557463872575e-05, 
       1.6554838609887707e-05, 
       1.1725848025827054e-05, 
       1.2436178609519201e-05, 
       1.5710961214449188e-05, 
       1.2406294336498330e-05, 
       1.2324917714639373e-05, 
       1.6530755275620778e-05, 
       1.3731959267300114e-05, 
       1.2397948700856795e-05, 
       1.5808884852920341e-05, 
       1.4996574873079029e-05, 
       1.2698437380966265e-05, 
       1.2820249545183314e-05, 
       1.6496189658827139e-05, 
       1.5666289734267274e-05, 
       1.3441240929757031e-05, 
       1.6084198893382013e-05, 
       1.3464354008009656e-05, 
       1.2923312047013518e-05, 
       1.4153434069785935e-05, 
       1.3692415858911692e-05, 
       1.2647929563223295e-05, 
       1.5707238963434554e-05, 
       1.4732038930163918e-05, 
       1.2303613663945354e-05, 
       1.6038563085680193e-05, 
       1.6440131848013768e-05, 
       1.2782956036827071e-05, 
       1.3681689919426739e-05, 
       1.1691671644629165e-05, 
       1.4499624017604651e-05, 
       1.6395388970284352e-05, 
       1.6268289868709545e-05, 
       1.5691960681611264e-05, 
       1.1820198074980815e-05, 
       1.2346581472258579e-05, 
       1.2434391892051689e-05, 
       1.5844056607382023e-05, 
       1.2365413501255700e-05, 
       1.2365413501255700e-05, 
       1.3586962442396367e-05, 
       1.2743115700108890e-05, 
       1.6329553554615786e-05, 
       1.1570739547506951e-05, 
       1.3206829560682617e-05, 
       1.4699065298021907e-05, 
       1.1943969004980533e-05, 
       1.5709386608091679e-05, 
       1.1976467870679928e-05, 
       1.2037110680872397e-05, 
       1.3252283484808413e-05, 
       1.1751907263236933e-05, 
       1.4695821628044451e-05, 
       1.1909909616669224e-05, 
       1.5429483851814471e-05, 
       1.3540485741379906e-05, 
       1.5015246910511515e-05, 
       1.3229899545695143e-05, 
       1.5181413541441026e-05, 
       1.4412180103396542e-05, 
       1.3190404474947619e-05, 
       1.1988832533212580e-05, 
       1.6185558154446363e-05, 
       1.3426714532296734e-05, 
       1.3869565860930604e-05, 
       1.4135381909291927e-05, 
       1.2606850455616096e-05, 
       1.2144157737387524e-05, 
       1.2445261049368796e-05, 
       1.4567667222832323e-05, 
       1.3693073542211198e-05, 
       1.5945804333257039e-05, 
       1.5097511323655867e-05, 
       1.3083451230784315e-05, 
       1.1656562503901878e-05, 
       1.3248369701287739e-05, 
       1.5059307448959791e-05, 
       1.1878061639952296e-05, 
       1.3749207965992576e-05, 
       1.4592582593190956e-05, 
       1.1791409574610544e-05, 
       1.1791409574610544e-05, 
       1.5897686053808388e-05, 
       1.3200346710388588e-05, 
       1.1491733677706403e-05, 
       1.5554715215553435e-05, 
       1.1805912385341901e-05, 
       1.1428469760813022e-05, 
       1.1473427980233798e-05, 
       1.5641819841991074e-05, 
       1.2674305212567148e-05, 
       1.5323952730433448e-05, 
       1.1437287703727674e-05, 
       1.2033933443160386e-05, 
       1.3514789530426399e-05, 
       1.6110115952677161e-05, 
       1.1496957380279707e-05, 
       1.3073495216324230e-05, 
       1.1610067832011640e-05, 
       1.3030248344053378e-05, 
       1.2418145207118873e-05, 
       1.1676042330679389e-05, 
       1.1806746906698654e-05, 
       1.5218638717593696e-05, 
       1.2264714382757942e-05, 
       1.1756912514758306e-05, 
       1.3870005976852957e-05, 
       1.6064855965009281e-05, 
       1.1997717874030617e-05, 
       1.4541108926322465e-05, 
       1.1385934298865472e-05, 
       1.1536830153690704e-05, 
       1.6046644649672756e-05, 
       1.1991518599098427e-05, 
       1.3126065397642909e-05, 
       1.5842218070209173e-05, 
       1.2805757786314474e-05, 
       1.1707112735086121e-05, 
       1.1903779516582374e-05, 
       1.1641802848545782e-05, 
       1.2631384096337930e-05, 
       1.4002302487247345e-05, 
       1.3327414429176635e-05, 
       1.3094907964503913e-05, 
       1.4584880159812993e-05, 
       1.1460871146783676e-05, 
       1.1788920941807358e-05, 
       1.4212354146342015e-05, 
       1.2124504294522087e-05, 
       1.1403082808497251e-05, 
       1.1805093320787671e-05, 
       1.1516586403133170e-05, 
       1.1516586403133170e-05, 
       1.1782432470851694e-05, 
       1.5687642170787301e-05, 
       1.1468094689081221e-05, 
       1.5646327836801596e-05, 
       1.1606360060081572e-05, 
       1.1746459586951272e-05, 
       1.3481216402351418e-05, 
       1.1306542119716975e-05, 
       1.1643740295214875e-05, 
       1.4264155037551276e-05, 
       1.5903588039561712e-05, 
       1.5036331088029241e-05, 
       1.1708627779604114e-05, 
       1.3450768538155453e-05, 
       1.2896341573252213e-05, 
       1.5927435455377457e-05, 
       1.2801393122403435e-05, 
       1.1917661158638408e-05, 
       1.5668815478302384e-05, 
       1.3466516587832193e-05, 
       1.1707057365782273e-05, 
       1.4121117094425142e-05, 
       1.1328603882568802e-05, 
       1.1308068145325968e-05, 
       1.1252187233513517e-05, 
       1.5199645314881461e-05, 
       1.5853025526967749e-05, 
       1.5313347729406159e-05, 
       1.4943648209940563e-05, 
       1.3088713696753645e-05, 
       1.3728329773655138e-05, 
       1.5189703733387558e-05, 
       1.1354250426422499e-05, 
       1.5843041372750021e-05, 
       1.1448615092421986e-05, 
       1.2125600347787393e-05, 
       1.1541750563452205e-05, 
       1.5816620432606382e-05, 
       1.4779759221480474e-05, 
       1.1936290366460016e-05, 
       1.3141568699492736e-05, 
       1.3376187761453536e-05, 
       1.4558536771582689e-05, 
       1.2850382793074335e-05, 
       1.5786156436101002e-05, 
       1.4711042705478195e-05, 
       1.3846017952778950e-05, 
       1.2329962791748679e-05, 
       1.1304216507208660e-05, 
       1.1761617043429116e-05, 
       1.4533402466654407e-05, 
       1.1184795576213820e-05, 
       1.2355016347426515e-05, 
       1.3868232904000190e-05, 
       1.5712421812686618e-05, 
       1.5274628319666322e-05, 
       1.1747215746359886e-05, 
       1.1110298822449085e-05, 
       1.3189853961366807e-05, 
       1.4873315195404058e-05, 
       1.5660190947536167e-05, 
       1.1113944966616766e-05, 
       1.4928415901558642e-05, 
       1.1709538471747310e-05, 
       1.5432092204515111e-05, 
       1.5601191169957125e-05, 
       1.2523359665147950e-05, 
       1.1515936533384390e-05, 
       1.5159361270082106e-05, 
       1.1637901880002912e-05, 
       1.1262262527624278e-05, 
       1.3738001311676133e-05, 
       1.5630116408341179e-05, 
       1.4003566990071585e-05, 
       1.4011563929128087e-05, 
       1.5570699567029073e-05, 
       1.1243402289275636e-05, 
       1.2202651906328915e-05, 
       1.1632835224451255e-05, 
       1.3077925768637599e-05, 
       1.1496020688915531e-05, 
       1.1225035725463698e-05, 
       1.1243885758371005e-05, 
       1.5389381913883833e-05, 
       1.1842930660028952e-05, 
       1.5264381754706096e-05, 
       1.1115315664648713e-05, 
       1.1653384767536996e-05, 
       1.5468816662612564e-05, 
       1.2253406756355439e-05, 
       1.1272289600893787e-05, 
       1.5522807778847407e-05, 
       1.4173567456488411e-05, 
       1.2267949558929904e-05, 
       1.1849773091019263e-05, 
       1.1849773091019263e-05, 
       1.1039016207471834e-05, 
       1.1199760475259692e-05, 
       1.0986523256542509e-05, 
       1.1196181141700119e-05, 
       1.4153368210771970e-05, 
       1.1348480770287634e-05, 
       1.5089187950494129e-05, 
       1.0967520958194335e-05, 
       1.0952010752608838e-05, 
       1.5115769554656917e-05, 
       1.1018112549010922e-05, 
       1.2428055255654019e-05, 
       1.2869710498104527e-05, 
       1.1171132744942016e-05, 
       1.1315865080764070e-05, 
       1.4663579251914919e-05, 
       1.4107151413661129e-05, 
       1.1059027997872198e-05, 
       1.1537827920369020e-05, 
       1.2735825688174792e-05, 
       1.5440009462433429e-05, 
       1.5270992835223876e-05, 
       1.1185437406002740e-05, 
       1.1352045364457742e-05, 
       1.2439626722135538e-05, 
       1.0967929408119725e-05, 
       1.3557333155104824e-05, 
       1.0934323250491465e-05, 
       1.1175654347406480e-05, 
       1.4912583062787027e-05, 
       1.5220031394833180e-05, 
       1.2697667714867147e-05, 
       1.2845462218534526e-05, 
       1.0897176133463497e-05, 
       1.0897176133463497e-05, 
       1.4073786409325840e-05, 
       1.0874609104551376e-05, 
       1.1508515859870161e-05, 
       1.1508515859870161e-05, 
       1.2734060603850802e-05, 
       1.1067005512295486e-05, 
       1.5337877145590250e-05, 
       1.5337877145590250e-05, 
       1.4624884438892826e-05, 
       1.5307137564677187e-05, 
       1.0996587230195566e-05, 
       1.0927985780817866e-05, 
       1.5006960783782638e-05, 
       1.3950980342708871e-05, 
       1.2518256502461746e-05, 
       1.1286889547465902e-05, 
       1.1286889547465902e-05, 
       1.2418006574986329e-05, 
       1.3020686125861597e-05, 
       1.3305410636865290e-05, 
       1.2520019019828464e-05, 
       1.4964993694528354e-05, 
       1.4527589466641050e-05, 
       1.0890367921632658e-05, 
       1.5031753928095082e-05, 
       1.2535173598128093e-05, 
       1.2725692351619874e-05, 
       1.5236367486478689e-05, 
       1.3183908284447587e-05, 
       1.3312196876400103e-05, 
       1.5067080981965494e-05, 
       1.1144693532560288e-05, 
       1.1189151896069598e-05, 
       1.5256502598029275e-05, 
       1.3805931020184647e-05, 
       1.0837804810568068e-05, 
       1.2485414595653036e-05, 
       1.4777074359356703e-05, 
       1.2382391337398714e-05, 
       1.4580128075709371e-05, 
       1.2885019884980843e-05, 
       1.2682417068049357e-05, 
       1.3257170988703858e-05, 
       1.1047785809995651e-05, 
       1.4399846579952405e-05, 
       1.3566597422146882e-05, 
       1.3124404547055084e-05, 
       1.3124404547055084e-05, 
       1.5077655569610467e-05, 
       1.2277398297783406e-05, 
       1.1326304583625107e-05, 
       1.4916032627462605e-05, 
       1.5166866833510470e-05, 
       1.4664819432685067e-05, 
       1.4771604348427269e-05, 
       1.2820765149677424e-05, 
       1.4617752897242361e-05, 
       1.4099604567113423e-05, 
       1.0909423648589213e-05, 
       1.2308778090971123e-05, 
       1.4499352619690369e-05, 
       1.1986266754807234e-05, 
       1.0706643184233675e-05, 
       1.2803978930077526e-05, 
       1.3145670200735626e-05, 
       1.0687074060935999e-05, 
       1.1483070592647357e-05, 
       1.4089477424144276e-05, 
       1.3540320035125067e-05, 
       1.4270909832045135e-05, 
       1.0781868765621849e-05, 
       1.2524379929047252e-05, 
       1.1569316407621641e-05, 
       1.0682416214828684e-05, 
       1.1227308808072031e-05, 
       1.5057881050155482e-05, 
       1.4784180429050506e-05, 
       1.2334791045062176e-05, 
       1.0931688471478746e-05, 
       1.4978560770722526e-05, 
       1.0645797797263889e-05, 
       1.0907301800509692e-05, 
       1.2245357589501916e-05, 
       1.1682483367378722e-05, 
       1.2536967804570898e-05, 
       1.2674431119390458e-05, 
       1.2500760479123041e-05, 
       1.2661436039349780e-05, 
       1.0748096343809930e-05, 
       1.0661241267353549e-05, 
       1.2906085811427898e-05, 
       1.1393258842823256e-05, 
       1.0934370358397742e-05, 
       1.3314085303905102e-05, 
       1.4893977145145620e-05, 
       1.1742098731337657e-05, 
       1.4898379300062665e-05, 
       1.1738503965747513e-05, 
       1.4960334989380520e-05, 
       1.0825665770186496e-05, 
       1.4892962356135938e-05, 
       1.0582864328384272e-05, 
       1.1876128172893532e-05, 
       1.4717111255904247e-05, 
       1.2416046454848009e-05, 
       1.0572774764081422e-05, 
       1.0629001008275446e-05, 
       1.0629001008275446e-05, 
       1.2989608020850853e-05, 
       1.0635451693907987e-05, 
       1.2416871252008860e-05, 
       1.1309479836273569e-05, 
       1.3441577510479102e-05, 
       1.2173619395365241e-05, 
       1.2272730109041874e-05, 
       1.1202943919641708e-05, 
       1.4827849245973207e-05, 
       1.1484453307605679e-05, 
       1.4721423390146512e-05, 
       1.3473508823247756e-05, 
       1.4356461362413227e-05, 
       1.2137163074763383e-05, 
       1.1452851846238364e-05, 
       1.4792230661840160e-05, 
       1.4862192739303246e-05, 
       1.1330698891247778e-05, 
       1.1341041684206632e-05, 
       1.1438869926346119e-05, 
       1.0985599373678655e-05, 
       1.1580380892111391e-05, 
       1.1693132431522686e-05, 
       1.4456293277566900e-05, 
       1.0897156582543343e-05, 
       1.3090127479297849e-05, 
       1.4616810496144469e-05, 
       1.3116734853383398e-05, 
       1.3241563968620226e-05, 
       1.4852177161157095e-05, 
       1.1268027648545931e-05, 
       1.2110637909259223e-05, 
       1.0983971588025755e-05, 
       1.3101101857249413e-05, 
       1.4302291641547505e-05, 
       1.2036897671629886e-05, 
       1.2088085575284726e-05, 
       1.2470940657654839e-05, 
       1.1353835712482653e-05, 
       1.2318200703793047e-05, 
       1.1561208327386016e-05, 
       1.0502266072465305e-05, 
       1.0726078868318596e-05, 
       1.3633957912284724e-05, 
       1.3633957912284724e-05, 
       1.1031766948423923e-05, 
       1.4756214215517396e-05, 
       1.4783993172475419e-05, 
       1.4265632235424737e-05, 
       1.1341234758415497e-05, 
       1.0455546620602337e-05, 
       1.4760754796311092e-05, 
       1.1379783376150733e-05, 
       1.1118145755479153e-05, 
       1.0779059583388129e-05, 
       1.3107552009535025e-05, 
       1.3573820965661135e-05, 
       1.3573820965661135e-05, 
       1.2964928274335566e-05, 
       1.0824900110190728e-05, 
       1.4249046018263897e-05, 
       1.1186842864810494e-05, 
       1.1002718625379305e-05, 
       1.2601212542019463e-05, 
       1.0401747429834436e-05, 
       1.3512024838147051e-05, 
       1.0659363588757335e-05, 
       1.0706524910972153e-05, 
       1.1267419351786995e-05, 
       1.2615801326915027e-05, 
       1.0372462058424156e-05, 
       1.1125956058060493e-05, 
       1.2396593323131289e-05, 
       1.0851091150722102e-05, 
       1.4638037150614476e-05, 
       1.1976140804757904e-05, 
       1.1976140804757904e-05, 
       1.4046939908507796e-05, 
       1.2495128346118907e-05, 
       1.0354329265691745e-05, 
       1.3593186316792134e-05, 
       1.2096016545599932e-05, 
       1.2235654928411608e-05, 
       1.0369261521023316e-05, 
       1.0713590219783965e-05, 
       1.4478955976275943e-05, 
       1.3058413759815817e-05, 
       1.0618612324834768e-05, 
       1.3981977480614612e-05, 
       1.3723817160305707e-05, 
       1.3603717863442891e-05, 
       1.0324597641693863e-05, 
       1.0789928402511669e-05, 
       1.0565284499678855e-05, 
       1.1921996299012355e-05, 
       1.0958471205144210e-05, 
       1.0536622016865853e-05, 
       1.0305727383443258e-05, 
       1.2382380832845453e-05, 
       1.0539096173643417e-05, 
       1.0884221606114679e-05, 
       1.0359597169923122e-05, 
       1.2083611228768885e-05, 
       1.4001995807347465e-05, 
       1.4357178868411115e-05, 
       1.2090636903173968e-05, 
       1.4511475405066456e-05, 
       1.1477522054824460e-05, 
       1.1850400761972084e-05, 
       1.2066733298351592e-05, 
       1.0514904496397342e-05, 
       1.1752157912646303e-05, 
       1.4474148208912567e-05, 
       1.0533462105683179e-05, 
       1.3916519850986633e-05, 
       1.0423082566069726e-05, 
       1.3478186026087543e-05, 
       1.0255779626614144e-05, 
       1.2482187415577462e-05, 
       1.4293843094331272e-05, 
       1.0962390480004999e-05, 
       1.0290444949957584e-05, 
       1.1504534691962995e-05, 
       1.4334106235450884e-05, 
       1.3306408690282276e-05, 
       1.4370237310148139e-05, 
       1.4078642185062700e-05, 
       1.4369702453050149e-05, 
       1.0452491693609763e-05, 
       1.0778451124246511e-05, 
       1.1852402892640105e-05, 
       1.0207994119854338e-05, 
       1.0250107976425917e-05, 
       1.4167849901597010e-05, 
       1.0201536244387168e-05, 
       1.4033737566722881e-05, 
       1.0640666094756368e-05, 
       1.1531383654357561e-05, 
       1.4385479716268984e-05, 
       1.0197191385776692e-05, 
       1.4151961089680895e-05, 
       1.0529160502369732e-05, 
       1.3891435537586878e-05, 
       1.1966728543004530e-05, 
       1.4372643406053173e-05, 
       1.4339635528439476e-05, 
       1.0446776561917997e-05, 
       1.2339808286648252e-05, 
       1.3240828610130460e-05, 
       1.3297687409586142e-05, 
       1.0232686493411448e-05, 
       1.0232467321738716e-05, 
       1.0769457889965293e-05, 
       1.0769457889965293e-05, 
       1.1233776810542011e-05, 
       1.1230618000829442e-05, 
       1.1393715805315966e-05, 
       1.0140699828897416e-05, 
       1.0191038029783340e-05, 
       1.0480206885599021e-05, 
       1.1704722679734985e-05, 
       1.3754455930786722e-05, 
       1.4259549301548854e-05, 
       1.1899471228164326e-05, 
       1.0855262973853046e-05, 
       1.0150205549888582e-05, 
       1.4275851808117710e-05, 
       1.3856538222947389e-05, 
       1.4141594015085124e-05, 
       1.2133007463131249e-05, 
       1.0086087376263686e-05, 
       1.0358004196978648e-05, 
       1.0148252957319420e-05, 
       1.0231964803194382e-05, 
       1.1662381400910250e-05, 
       1.0480274902075477e-05, 
       1.1844462285448935e-05, 
       1.0518551833511820e-05, 
       1.4175465863273368e-05, 
       1.0351382921913594e-05, 
       1.0345904758398967e-05, 
       1.4159137148875332e-05, 
       1.3995844960773261e-05, 
       1.3705111037037255e-05, 
       1.3783819765661786e-05, 
       1.4182962566498339e-05, 
       1.4175186838039334e-05, 
       1.1034237926638181e-05, 
       1.0941254008155762e-05, 
       1.0523948902187752e-05, 
       1.0192413359347300e-05, 
       1.3277067386134746e-05, 
       1.1601258009471674e-05, 
       1.2366330407442362e-05, 
       1.0164096791271234e-05, 
       1.3852707329658270e-05, 
       1.0944116714033461e-05, 
       1.1716036737641770e-05, 
       1.0127605453051655e-05, 
       1.2545008577836389e-05, 
       1.0719060500363479e-05, 
       1.1459250018088116e-05, 
       1.1203282772219203e-05, 
       1.0384421379242826e-05, 
       1.0046168905596495e-05, 
       1.0109426574253145e-05, 
       1.2954958561733972e-05, 
       1.0107896110300699e-05, 
       1.1481840674364551e-05, 
       1.2348459390545577e-05, 
       1.1544211853024134e-05, 
       1.0781553527906825e-05, 
       1.0443297817610015e-05, 
       1.0443297817610015e-05, 
       1.0741773271011750e-05, 
       9.9588492548859384e-06, 
       1.0772599095308152e-05, 
       1.0717318714635639e-05, 
       1.1562262967228524e-05, 
       1.1671304731552792e-05, 
       1.0427996792002121e-05, 
       1.0766993965580983e-05, 
       9.9639252780536248e-06, 
       1.0699631739427376e-05, 
       1.0734362119894454e-05, 
       1.0938622843211320e-05, 
       1.2509512401668630e-05, 
       1.0465518322655523e-05, 
       1.0369418297989408e-05, 
       1.1644271876959301e-05, 
       1.0756879532016300e-05, 
       1.3571781635163237e-05, 
       9.9066181154063978e-06, 
       1.0375729240048642e-05, 
       1.1350746187690665e-05, 
       1.0048140096828067e-05, 
       1.0169779087521093e-05, 
       1.0274727581595650e-05, 
       1.2098358739634541e-05, 
       1.1583528020663391e-05, 
       1.3716189943916849e-05, 
       1.3716189943916849e-05, 
       1.0273134656555220e-05, 
       1.3771343749510956e-05, 
       1.3511470796911996e-05, 
       1.2472753076264438e-05, 
       1.0163898887041306e-05, 
       1.1782940348591839e-05, 
       1.1932072754492584e-05, 
       1.1595036243800563e-05, 
       1.3290564297012587e-05, 
       1.1555697292284911e-05, 
       9.8531573836538121e-06, 
       1.0475872976903728e-05, 
       1.3891492169927510e-05, 
       1.0264615489127979e-05, 
       1.3662868048168734e-05, 
       1.3882906807495525e-05, 
       1.3632743678908576e-05, 
       9.9322116857103128e-06, 
       9.9676209493156017e-06, 
       1.1172525963628234e-05, 
       1.1157513459169266e-05, 
       9.8463506734846862e-06, 
       9.8589441723096154e-06, 
       1.1167394334040324e-05, 
       1.0764400560250339e-05, 
       9.9295732532001488e-06, 
       1.3857666475041970e-05, 
       9.8540545144493283e-06, 
       1.2391894679069700e-05, 
       9.8709408983938285e-06, 
       1.0265848360167324e-05, 
       1.3752051008159996e-05, 
       1.3465837193572501e-05, 
       1.0294149916100227e-05, 
       1.3553150957271262e-05, 
       1.0937738936159390e-05, 
       1.1652798280705539e-05, 
       9.8991856293166356e-06, 
       1.3680345509379824e-05, 
       1.3680232479877401e-05, 
       1.3778407793679142e-05, 
       9.8166427871913147e-06, 
       1.0266360710962645e-05, 
       1.1120224616402655e-05, 
       1.3479388353296501e-05, 
       1.2713057698686368e-05, 
       1.2400727021484068e-05, 
       1.1193169627032153e-05, 
       1.0293345836682840e-05, 
       1.0286900843820786e-05, 
       1.0287509587112707e-05, 
       9.7110557318157017e-06, 
       1.3584579796094864e-05, 
       9.7991190832840413e-06, 
       1.3581910128433880e-05, 
       1.3709792190589098e-05, 
       1.1140006391140550e-05, 
       1.3185749278468462e-05, 
       1.0558336312681318e-05, 
       1.0907261870507726e-05, 
       1.3308886757718064e-05, 
       1.1728424833290895e-05, 
       1.3677034841825666e-05, 
       1.1005444053608968e-05, 
       9.9181580570812516e-06, 
       1.2492448011625643e-05, 
       1.2730863681723320e-05, 
       1.2920167304930794e-05, 
       1.3409714538063165e-05, 
       1.3409714538063165e-05, 
       1.1354024047272605e-05, 
       1.2725660885446155e-05, 
       9.8953075569406875e-06, 
       1.1278233862602094e-05, 
       1.0521365032372457e-05, 
       1.1247028112192032e-05, 
       9.8727686935996752e-06, 
       1.1752157912643477e-05, 
       1.1299091244424666e-05, 
       1.3445703513279984e-05, 
       1.3223115866879419e-05, 
       1.0556804643202225e-05, 
       1.0556804643202225e-05, 
       1.0324142386844406e-05, 
       1.0324142386844406e-05, 
       1.2403292753983264e-05, 
       1.0020088063763115e-05, 
       1.3209683995720098e-05, 
       9.8997126994705673e-06, 
       9.6877317195667489e-06, 
       1.0741291799648876e-05, 
       9.8357648440326428e-06, 
       1.1301045296227273e-05, 
       9.6371606708548516e-06, 
       9.9188789453519585e-06, 
       1.3533870031930246e-05, 
       1.0288554358793279e-05, 
       1.1254408991385510e-05, 
       9.6538100147117608e-06, 
       1.1839614596608071e-05, 
       1.1576468942207957e-05, 
       9.6080930288863930e-06, 
       1.0325854012744955e-05, 
       1.2905913985529389e-05, 
       9.7542016527540027e-06, 
       1.2336471051698504e-05, 
       1.1204766900491367e-05, 
       1.3457481789545981e-05, 
       1.3082326440540041e-05, 
       9.9288021589010119e-06, 
       9.9423396243975264e-06, 
       1.1043116804061502e-05, 
       9.5470105352297831e-06, 
       1.3030394604068650e-05, 
       9.5630513997533024e-06, 
       9.8001029643319265e-06, 
       9.6546220414183026e-06, 
       9.6513717990655065e-06, 
       9.7255842240087050e-06, 
       9.8312539701814733e-06, 
       1.3358906659324441e-05, 
       1.1204631885711674e-05, 
       1.1417371644690850e-05, 
       9.5471562509718646e-06, 
       1.3356955723132384e-05, 
       9.8350477729461501e-06, 
       1.1703350164090343e-05, 
       1.0022851019675560e-05, 
       9.6933187761007210e-06, 
       1.1914518514727370e-05, 
       1.2092467310824198e-05, 
       9.8016029589222726e-06, 
       1.0043099465667155e-05, 
       1.1218004962993395e-05, 
       1.1433161928157302e-05, 
       9.5026622513660741e-06, 
       1.0437392199040041e-05, 
       1.1687549437028655e-05, 
       1.1441998243931732e-05, 
       1.3405385310560013e-05, 
       1.0207625149463528e-05, 
       1.0104401045019265e-05, 
       9.8864957878371508e-06, 
       1.1798593183640694e-05, 
       9.8656653331183074e-06, 
       9.8656653331183074e-06, 
       1.1127895645814130e-05, 
       1.2122818045564395e-05, 
       1.2122818045564395e-05, 
       1.1427349957727755e-05, 
       1.0943809308893678e-05, 
       1.2391450287923214e-05, 
       1.3186895349734087e-05, 
       1.1107318880802598e-05, 
       9.5026384260519855e-06, 
       1.1303850900876228e-05, 
       1.3239157353800872e-05, 
       1.3350268526375690e-05, 
       1.2635770437608012e-05, 
       1.3186067216009567e-05, 
       9.5105068645971677e-06, 
       9.6283361171834058e-06, 
       9.4432811927597549e-06, 
       1.0130424330690898e-05, 
       1.3109729013515758e-05, 
       9.5633359468702655e-06, 
       9.4785173521846500e-06, 
       1.2514020340884226e-05, 
       9.4511416572530357e-06, 
       1.2639813888453164e-05, 
       1.0602095869449240e-05, 
       9.9247045689405628e-06, 
       1.0769130946744692e-05, 
       1.2326572788999798e-05, 
       1.2258874428434726e-05, 
       9.3894207286489628e-06, 
       1.1162989531966972e-05, 
       1.3144905699325213e-05, 
       1.1016691630843322e-05, 
       9.4493428251548987e-06, 
       1.0770542529304339e-05, 
       1.1390225352955032e-05, 
       9.4171171286346109e-06, 
       1.0806720495068495e-05, 
       1.3105891423479931e-05, 
       1.0124985383816042e-05, 
       1.1186872677173530e-05, 
       1.1647913762635511e-05, 
       9.4736783241047553e-06, 
       1.0011519348709738e-05, 
       1.0804181581460961e-05, 
       1.3171782712334967e-05, 
       1.1023048174560538e-05, 
       1.2928282235854085e-05, 
       9.7078942855980961e-06, 
       1.0803244108138391e-05, 
       1.2109242877613827e-05, 
       1.2752397517035859e-05, 
       1.2221917979082999e-05, 
       1.2117846112147576e-05, 
       1.0567614747559902e-05, 
       1.1007610559107456e-05, 
       1.2983373656710785e-05, 
       9.4021952706926370e-06, 
       1.2106753394518955e-05, 
       1.1229722669348576e-05, 
       1.3153807692290633e-05, 
       1.1205800655026882e-05, 
       1.0954143051024719e-05, 
       1.3137960761351307e-05, 
       9.3168954657137443e-06, 
       1.0858590684983307e-05, 
       1.2290473544713559e-05, 
       1.2828144047638880e-05, 
       9.5832087676143862e-06, 
       9.8818302319605414e-06, 
       1.0491732248718483e-05, 
       1.1953922390953531e-05, 
       1.0918878942682853e-05, 
       1.2004128080271443e-05, 
       1.3017446398937332e-05, 
       1.0922067436222345e-05, 
       9.5499468237402093e-06, 
       1.1886034178225775e-05, 
       1.2030848723993907e-05, 
       1.0671480260562395e-05, 
       9.4398779849119620e-06, 
       1.2681549098845909e-05, 
       9.6975012974264742e-06, 
       1.0607255961229588e-05, 
       9.2677950455456232e-06, 
       1.0669405025341764e-05, 
       1.0837804810570888e-05, 
       9.7600592480075076e-06, 
       9.3376919690089264e-06, 
       1.3062806636467790e-05, 
       1.3062806636467790e-05, 
       1.1048775634624641e-05, 
       1.0739902383738782e-05, 
       1.0591587876995203e-05, 
       1.1145778771575602e-05, 
       1.0500205060680463e-05, 
       9.2321339976707295e-06, 
       9.3092463478716725e-06, 
       9.8759585353772033e-06, 
       9.2668813377319048e-06, 
       9.3775407997072199e-06, 
       1.0310541212589325e-05, 
       9.5918294218066813e-06, 
       9.2587164719816308e-06, 
       1.1304216507209693e-05, 
       9.9697715055097759e-06, 
       9.3148027575251408e-06, 
       9.3547467882339339e-06, 
       9.3781396000811463e-06, 
       1.1570674593632197e-05, 
       1.1210949235434507e-05, 
       9.7688027720322650e-06, 
       1.2627834612757489e-05, 
       1.2772207937889913e-05, 
       1.0703498427079553e-05, 
       9.2247734081576331e-06, 
       9.2818047193352773e-06, 
       1.2308412295258785e-05, 
       9.4085128010184797e-06, 
       9.1846790933436114e-06, 
       1.2630237888799176e-05, 
       9.3531527641751993e-06, 
       1.2220465496440703e-05, 
       9.2888880421973135e-06, 
       9.9317347715202342e-06, 
       1.0485547869184157e-05, 
       1.2955498244343441e-05, 
       1.1698216894219982e-05, 
       9.7665100201072385e-06, 
       1.0525702757590444e-05, 
       1.0523551832201159e-05, 
       9.7253751619105309e-06, 
       9.6578785903524936e-06, 
       9.8947536800422147e-06, 
       9.8784675056871213e-06, 
       1.0139019887189659e-05, 
       1.0137861699121923e-05, 
       1.1149270590659216e-05, 
       1.0768237818775314e-05, 
       1.0423542401133671e-05, 
       9.6199835875848645e-06, 
       9.3708820858261481e-06, 
       9.6089459488269225e-06, 
       9.4267873914124887e-06, 
       1.2830083882353740e-05, 
       1.0966169533015538e-05, 
       9.8521648788683639e-06, 
       1.2455490813311961e-05, 
       1.1231479999904232e-05, 
       9.6842656308280507e-06, 
       1.2433500153068158e-05, 
       1.1773305617921443e-05, 
       1.2033001514277886e-05, 
       1.1852795457209111e-05, 
       1.2570505041565146e-05, 
       1.2622725017399904e-05, 
       9.8337776002085453e-06, 
       1.0817707761469620e-05, 
       9.1230553059871431e-06, 
       1.1328954557311369e-05, 
       1.2371896701208038e-05, 
       1.0059840790181679e-05, 
       9.1478675568288740e-06, 
       1.2825661190244699e-05, 
       1.0449262547816364e-05, 
       1.0449049795165766e-05, 
       1.2743105654076809e-05, 
       9.9876167479811714e-06, 
       9.9876167479811714e-06, 
       1.2780497664426927e-05, 
       9.8805574510434289e-06, 
       1.0202712395731005e-05, 
       1.0202712395731005e-05, 
       1.1191677570474883e-05, 
       9.8993722027278463e-06, 
       1.0402341386775062e-05, 
       1.1071430896233756e-05, 
       1.2255945887280579e-05, 
       9.1022566900728773e-06, 
       1.1658378212952020e-05, 
       9.2494681641170340e-06, 
       9.1727039195827449e-06, 
       9.1727039195827449e-06, 
       1.0121869774851753e-05, 
       1.0178099871991463e-05, 
       9.9970367497889430e-06, 
       9.4469575938118905e-06, 
       1.2768852916673558e-05, 
       1.0757200347408972e-05, 
       1.2347740237962768e-05, 
       1.2563735955440907e-05, 
       9.5500591719759298e-06, 
       9.3436504438106613e-06, 
       9.3436504438106613e-06, 
       9.9866977973797680e-06, 
       1.0013612574915403e-05, 
       1.0815805915283423e-05, 
       9.4001893056016508e-06, 
       9.3424966200329927e-06, 
       1.2463336640932619e-05, 
       1.2654201733108691e-05, 
       1.0042257124373006e-05, 
       9.1022441532061181e-06, 
       1.2307315990743306e-05, 
       1.1872992088757272e-05, 
       1.0301533207928223e-05, 
       1.2778805979040705e-05, 
       1.0559094159024394e-05, 
       9.1998150324756944e-06, 
       9.7392855939157339e-06, 
       9.7392855939157339e-06, 
       1.2684148584956904e-05, 
       9.0368436568031782e-06, 
       1.1275154083769477e-05, 
       9.0746217170146251e-06, 
       1.0928542677687956e-05, 
       1.0781962041928323e-05, 
       1.0600707078360105e-05, 
       9.1671318570126391e-06, 
       9.5236702297173559e-06, 
       1.2738658977837161e-05, 
       9.1357389198660292e-06, 
       9.1248382381796373e-06, 
       1.0620995329562578e-05, 
       9.2780105511684983e-06, 
       1.1737788126103411e-05, 
       1.2564144906311051e-05, 
       9.6351536738725791e-06, 
       1.2676606841844208e-05, 
       1.0708949430844922e-05, 
       9.3331175295456527e-06, 
       9.0819214798173418e-06, 
       1.1165061773801044e-05, 
       1.2534492581613298e-05, 
       1.2110617539560772e-05, 
       9.1478512460921433e-06, 
       1.0037633078030742e-05, 
       1.0635264197800616e-05, 
       1.1815973088010024e-05, 
       1.2334185130685705e-05, 
       9.1100972426726350e-06, 
       9.7445044027918277e-06, 
       1.2670282504972080e-05, 
       1.0996587230191946e-05, 
       9.5965841699365205e-06, 
       1.0534533226667888e-05, 
       9.5819383088884107e-06, 
       1.0749523252631202e-05, 
       1.1953525186153542e-05, 
       1.0443074660427468e-05, 
       8.9958933306436060e-06, 
       1.2446000187454274e-05, 
       1.2397091642794731e-05, 
       9.9912873716180499e-06, 
       1.1509756812962129e-05, 
       9.7961727688561816e-06, 
       1.0653765509033395e-05, 
       1.1961500703302007e-05, 
       9.0804718562953469e-06, 
       1.2606717937057878e-05, 
       1.0774460659384311e-05, 
       1.2391582890951479e-05, 
       9.2227945383247079e-06, 
       9.1879717427088422e-06, 
       8.9268099626050993e-06, 
       1.2597584016415193e-05, 
       8.9814441206256559e-06, 
       1.1451624288040226e-05, 
       1.0689431338339348e-05, 
       8.9195566727446034e-06, 
       1.0474506716620784e-05, 
       9.4729164238296090e-06, 
       1.2561939692113902e-05, 
       9.8400799162374162e-06, 
       9.2811240200997808e-06, 
       9.2381699501942920e-06, 
       8.9941227169061446e-06, 
       1.2346581472266734e-05, 
       1.1945606285926167e-05, 
       1.0481620945989216e-05, 
       9.6625128449432227e-06, 
       1.1067654623883784e-05, 
       9.2473224187938725e-06, 
       1.2460145103289174e-05, 
       1.2091566040481341e-05, 
       1.0694173560187419e-05, 
       8.9574075892759891e-06, 
       1.2222873169944524e-05, 
       1.0443932151844147e-05, 
       1.2121883741920248e-05, 
       9.1851166488721981e-06, 
       9.4187543135471055e-06, 
       8.8543249880848631e-06, 
       1.0355983952848059e-05, 
       1.1143484055106893e-05, 
       1.0356967215095525e-05, 
       8.8832192040186158e-06, 
       1.2407701137751519e-05, 
       1.0896293850782069e-05, 
       1.1674929870624057e-05, 
       1.1169989751052750e-05, 
       1.0379883688042089e-05, 
       9.1063635490633534e-06, 
       1.2336138780579346e-05, 
       9.0001753881004004e-06, 
       1.1219340538269669e-05, 
       1.2235805909465814e-05, 
       9.1830790774495666e-06, 
       1.1200785234531888e-05, 
       1.0401995299288373e-05, 
       1.2459582760076553e-05, 
       1.0585377835422413e-05, 
       8.8956288824362054e-06, 
       1.0748096343807257e-05, 
       9.5744835955419582e-06, 
       9.6149983712680203e-06, 
       9.4107281440109504e-06, 
       1.2336108874312123e-05, 
       1.1513433387387463e-05, 
       9.8439063118872454e-06, 
       1.1982044708512600e-05, 
       1.2225867655223777e-05, 
       9.5952260509795880e-06, 
       9.0242975439403601e-06, 
       1.1122085553303411e-05, 
       1.2288443764546314e-05, 
       8.8670562925428283e-06, 
       1.0530901477910492e-05, 
       1.0337472656865706e-05, 
       9.0511085522554975e-06, 
       1.2401467377660168e-05, 
       8.7751352482030465e-06, 
       1.0851131390882867e-05, 
       8.7745824667157009e-06, 
       9.8245554415825270e-06, 
       1.2344993589865840e-05, 
       1.0327333199442356e-05, 
       8.8450662512124993e-06, 
       1.1536309285521137e-05, 
       1.1277111082430603e-05, 
       8.8375524337901501e-06, 
       9.8586702779874318e-06, 
       1.0292560042439935e-05, 
       8.7907778702964134e-06, 
       9.9090568073786265e-06, 
       9.0422170242966008e-06, 
       1.0293907396225010e-05, 
       8.7753222769101873e-06, 
       1.1831149635131640e-05, 
       1.0012746213256483e-05, 
       9.0544925483596595e-06, 
       1.1799471467373434e-05, 
       9.2177058552836243e-06, 
       1.2227785601250888e-05, 
       9.2044360456944574e-06, 
       1.1142567919245456e-05, 
       8.7348770311390610e-06, 
       8.7344957993144064e-06, 
       9.6801254345436389e-06, 
       1.0527087143944868e-05, 
       1.1602416612816817e-05, 
       9.0600700046074098e-06, 
       9.8743414406016297e-06, 
       1.0228212411451325e-05, 
       1.0937964482508904e-05, 
       1.2140971794259049e-05, 
       9.1305462765572713e-06, 
       1.0229423815091647e-05, 
       9.0513340917907232e-06, 
       1.2299220560215220e-05, 
       8.7088710442996347e-06, 
       8.7180403949976244e-06, 
       9.4875174364329943e-06, 
       8.6829906565761092e-06, 
       8.8259184291011369e-06, 
       9.8832892435092658e-06, 
       1.0522537900222211e-05, 
       9.8811350920512785e-06, 
       1.2264179452124367e-05, 
       1.0070795802198668e-05, 
       1.0291903019556673e-05, 
       1.1347234189430728e-05, 
       9.9612026493148042e-06, 
       9.1058306621528790e-06, 
       9.4909664734453217e-06, 
       9.4988971369013392e-06, 
       1.0784399995877553e-05, 
       1.2186232015751885e-05, 
       1.2254632984604563e-05, 
       8.7919730096768578e-06, 
       8.6604601825212244e-06, 
       9.3934058079061585e-06, 
       8.7113613066632600e-06, 
       1.0111915838943395e-05, 
       1.0158618198907516e-05, 
       9.9513140685793033e-06, 
       8.6868084682827813e-06, 
       9.8416161227922178e-06, 
       1.0389452780257481e-05, 
       1.1691230142425545e-05, 
       9.8303350376425648e-06, 
       1.1329199413064755e-05, 
       1.0169328768071783e-05, 
       1.2141504381133440e-05, 
       1.1354656249574024e-05, 
       1.1354656249574024e-05, 
       8.6935629358109298e-06, 
       9.0141586972807182e-06, 
       9.0282991780687111e-06, 
       8.8089813724129680e-06, 
       8.6399653361158122e-06, 
       1.1282516728773691e-05, 
       9.0823208079387150e-06, 
       1.0411312084603624e-05, 
       9.3090533406867956e-06, 
       1.2057244244393008e-05, 
       1.0148467603168878e-05, 
       8.8476369059254946e-06, 
       9.2209414277631789e-06, 
       1.1116078236253220e-05, 
       1.0552701260820029e-05, 
       1.0854122423359124e-05, 
       1.0067167443994695e-05, 
       1.0082122234113974e-05, 
       9.6775349443170425e-06, 
       9.1253680979815203e-06, 
       8.7662247296253884e-06, 
       1.0268029690399450e-05, 
       8.8566346690324586e-06, 
       9.9565108148315982e-06, 
       8.6081173272099283e-06, 
       1.0702017346930397e-05, 
       1.2030135211295566e-05, 
       9.6398361670274659e-06, 
       8.7961159887268427e-06, 
       8.8108829202063293e-06, 
       8.6081422551946095e-06, 
       8.8248027791409656e-06, 
       1.0005922587375624e-05, 
       9.0004283189170079e-06, 
       1.2043036907652049e-05, 
       1.1940055035649077e-05, 
       1.2062130119493855e-05, 
       8.9897317227603656e-06, 
       8.7992311869331345e-06, 
       1.1755737321336195e-05, 
       9.8402231567159156e-06, 
       9.1804230151786108e-06, 
       1.2038580769513963e-05, 
       1.0984991599234054e-05, 
       8.9800128477225336e-06, 
       8.9800128477225336e-06, 
       1.0250788450084232e-05, 
       8.7739980794217569e-06, 
       9.6448824020837030e-06, 
       8.6069665581148381e-06, 
       8.5528432668242652e-06, 
       1.1993494601139581e-05, 
       1.0319842355969517e-05, 
       1.1899174885544215e-05, 
       9.5555685320202960e-06, 
       9.2860009143264645e-06, 
       8.5172472667412958e-06, 
       9.5102993471382897e-06, 
       8.4836705119684425e-06, 
       1.0439499237609040e-05, 
       8.8535520676748101e-06, 
       1.1914666717700017e-05, 
       9.9669083791440136e-06, 
       1.1849196969717771e-05, 
       1.0900506162067672e-05, 
       1.1003000311280193e-05, 
       9.1020544083674957e-06, 
       1.0019021192034605e-05, 
       1.0186679587385188e-05, 
       1.0433379442175783e-05, 
       1.0166461505592848e-05, 
       9.3547459771763122e-06, 
       9.3547459771763122e-06, 
       8.4627990073382092e-06, 
       9.7928082886333503e-06, 
       8.7912306205002656e-06, 
       1.0795533149685783e-05, 
       8.5269013027321104e-06, 
       8.4632370910551085e-06, 
       8.4632370910551085e-06, 
       9.8032526866060710e-06, 
       8.6166194987115170e-06, 
       8.4177357674216400e-06, 
       9.5775617653283435e-06, 
       1.0266672870714404e-05, 
       9.4663483929623121e-06, 
       9.4701466977615695e-06, 
       8.8038046287815367e-06, 
       9.5954177025168152e-06, 
       8.5697616820432065e-06, 
       9.0194555960488053e-06, 
       9.0189255120466999e-06, 
       9.8975915928375960e-06, 
       9.6983787526857572e-06, 
       9.6983787526857572e-06, 
       8.9493644194257832e-06, 
       1.1796679581070828e-05, 
       8.5925237907150606e-06, 
       9.8223692341669995e-06, 
       1.0101177606579987e-05, 
       9.7885556248928885e-06, 
       1.1262356646469647e-05, 
       1.0067768894138080e-05, 
       8.9987364796765496e-06, 
       8.5958929885161994e-06, 
       8.7254187691508449e-06, 
       9.8138837041622698e-06, 
       8.5561429827420834e-06, 
       8.7732407973011242e-06, 
       1.1792957579922011e-05, 
       9.9504050081533011e-06, 
       8.3455299444919103e-06, 
       9.8030991965915884e-06, 
       9.7980809577219038e-06, 
       9.1010904758197969e-06, 
       8.9201321624907420e-06, 
       8.7352304763132800e-06, 
       9.0354187114852844e-06, 
       1.0406508362932448e-05, 
       9.9386650565623355e-06, 
       9.0765000960446460e-06, 
       1.0621560838042796e-05, 
       9.2241098964281614e-06, 
       9.7859468108526106e-06, 
       8.5244880032475073e-06, 
       9.3104902787219013e-06, 
       8.8523869460707942e-06, 
       8.7811899376470511e-06, 
       9.8455577063767873e-06, 
       1.0478510876457637e-05, 
       9.3169897622483651e-06, 
       1.0040816299170210e-05, 
       1.0612346558106938e-05, 
       8.5140757397043491e-06, 
       8.4412883936659743e-06, 
       8.4412883936659743e-06, 
       9.3065083495887255e-06, 
       8.3388935841647627e-06, 
       8.9228027136218589e-06, 
       8.9228027136218589e-06, 
       9.2094188948961765e-06, 
       9.4697161310466244e-06, 
       8.3205550488787154e-06, 
       1.1476082285179164e-05, 
       8.2679623736798372e-06, 
       9.8527546484523602e-06, 
       8.5659417122952512e-06, 
       8.3226816694482874e-06, 
       1.0549274143239734e-05, 
       9.7902580207581672e-06, 
       9.0122431521735610e-06, 
       1.1570129713218390e-05, 
       9.5834138137362068e-06, 
       9.8644425354725586e-06, 
       9.8643078018134197e-06, 
       1.0583440354826471e-05, 
       8.3596060383784413e-06, 
       8.9209508582935097e-06, 
       9.7143042723297784e-06, 
       1.0839125379946887e-05, 
       9.8250909155633992e-06, 
       9.6443928285969687e-06, 
       8.6302925255577577e-06, 
       1.1614219611710227e-05, 
       9.4408398883806297e-06, 
       1.0217710138223576e-05, 
       9.4910498368356787e-06, 
       9.4910498368356787e-06, 
       9.1795251127395000e-06, 
       1.0792253652749786e-05, 
       9.4527703640180113e-06, 
       9.9022339518048429e-06, 
       1.0012746213256557e-05, 
       9.6625091952213829e-06, 
       1.1308384871347993e-05, 
       8.3858523701088737e-06, 
       8.8185020240099527e-06, 
       8.2264569290775491e-06, 
       8.6809186463087251e-06, 
       9.6673269749644050e-06, 
       1.1578267211247012e-05, 
       8.3155317317234892e-06, 
       1.0316464829673697e-05, 
       1.1545806004323859e-05, 
       1.1572900583349127e-05, 
       9.6046259507971645e-06, 
       9.7203341959591670e-06, 
       8.3893145334266619e-06, 
       1.1564097364522789e-05, 
       9.7148378588876969e-06, 
       8.9600174446238185e-06, 
       1.0658104939939485e-05, 
       8.5997936680632638e-06, 
       1.1556106328449921e-05, 
       9.0883387754851259e-06, 
       9.3668416107336625e-06, 
       1.1346011814057129e-05, 
       1.0571400853246439e-05, 
       1.1315714128925886e-05, 
       9.5744046880814586e-06, 
       8.9254028337447856e-06, 
       1.0676456813870472e-05, 
       8.6248592934786310e-06, 
       1.0201665644283194e-05, 
       8.1620599283207719e-06, 
       1.1216832582278412e-05, 
       1.0112283694257502e-05, 
       8.4024959770880573e-06, 
       9.6320657887368971e-06, 
       8.6766526760894909e-06, 
       1.1507095700849713e-05, 
       9.7875825345123419e-06, 
       8.2393089766065277e-06, 
       9.5832620559615775e-06, 
       9.7402279933637225e-06, 
       1.1075964556524318e-05, 
       1.1500931311052911e-05, 
       8.4651859723759634e-06, 
       1.1440234668465594e-05, 
       8.5716686645809429e-06, 
       9.4104734359851716e-06, 
       1.0989103300252571e-05, 
       9.3990814767558021e-06, 
       8.2368158100908824e-06, 
       1.0163330422237313e-05, 
       1.1427580183596415e-05, 
       8.1297624970274687e-06, 
       9.6338841272753390e-06, 
       8.2895648963407689e-06, 
       1.0588611992121518e-05, 
       9.9261904060029401e-06, 
       1.1424365551130071e-05, 
       1.1019718323411165e-05, 
       1.1245865773948267e-05, 
       8.1240434828092947e-06, 
       8.1130599564122140e-06, 
       1.0911672544827132e-05, 
       9.6919587935911346e-06, 
       9.5573101777326521e-06, 
       1.0893715082688273e-05, 
       1.0337612987241769e-05, 
       1.1141129563125891e-05, 
       1.1141129563125891e-05, 
       9.2866362176704509e-06, 
       9.5228355615112211e-06, 
       1.0915522604313858e-05, 
       8.5314814252202339e-06, 
       1.0422335276293300e-05, 
       1.1301714487256171e-05, 
       8.1056814038934162e-06, 
       8.1821531769789903e-06, 
       8.8631485082008062e-06, 
       8.1816305763038414e-06, 
       1.1408665313321240e-05, 
       8.1483108725705809e-06, 
       1.0976290503418787e-05, 
       8.3916659944529794e-06, 
       8.3862245004199113e-06, 
       1.1334316144781269e-05, 
       1.1394636477489268e-05, 
       9.1840069072858722e-06, 
       9.1961890652711172e-06, 
       8.3837674422858028e-06, 
       9.1090075149421652e-06, 
       8.4011241886923444e-06, 
       9.4668553331241407e-06, 
       1.1072609903073088e-05, 
       9.3828097272308220e-06, 
       1.0682237802984417e-05, 
       8.2806711833994120e-06, 
       8.1372473964898334e-06, 
       8.4610574025488490e-06, 
       9.6530221453072238e-06, 
       8.4989915408250530e-06, 
       8.3581931331143700e-06, 
       9.5392252883409472e-06, 
       1.0194485083566727e-05, 
       8.2333607442203799e-06, 
       1.1306693487970252e-05, 
       1.1264565238219749e-05, 
       9.4387058293918144e-06, 
       8.5591727815723025e-06, 
       1.1195135132882532e-05, 
       1.0069192799702587e-05, 
       8.2084879324017224e-06, 
       9.5637548099358271e-06, 
       1.1237886591376780e-05, 
       9.5894323881536232e-06, 
       1.1049217377108872e-05, 
       9.5884790988969791e-06, 
       1.0070171713254139e-05, 
       8.4822439597244781e-06, 
       9.7640416673735818e-06, 
       8.0218585497777089e-06, 
       1.0004638378238634e-05, 
       8.2689266949638085e-06, 
       1.0893632222847776e-05, 
       1.0893632222847776e-05, 
       1.0680956178865357e-05, 
       8.3068017517819875e-06, 
       8.1784889777895286e-06, 
       8.6219463879132669e-06, 
       8.6219463879132669e-06, 
       1.0613855467132364e-05, 
       8.8218469693648963e-06, 
       9.2793056077332950e-06, 
       9.2793056077332950e-06, 
       8.4202574473891134e-06, 
       8.1151335533799126e-06, 
       8.1151335533799126e-06, 
       1.1274457755722832e-05, 
       1.1276257419551856e-05, 
       9.6092942491575890e-06, 
       9.5292290691375629e-06, 
       1.0316368507571364e-05, 
       8.7919882715844521e-06, 
       1.1012207864956154e-05, 
       9.3556432264183307e-06, 
       1.1220636024965189e-05, 
       9.9545484800477327e-06, 
       8.1081579450573246e-06, 
       7.9775799100225089e-06, 
       9.9015668775049225e-06, 
       1.1118898663356273e-05, 
       1.0176888539422512e-05, 
       8.1614176038338039e-06, 
       8.1383815434114593e-06, 
       8.2715596466002474e-06, 
       9.2572923721762516e-06, 
       1.0184262149425215e-05, 
       9.1027805643622562e-06, 
       9.3196616039386248e-06, 
       8.4897077314426014e-06, 
       9.3712856359736449e-06, 
       1.0558840685668337e-05, 
       7.9701544354216161e-06, 
       9.3752238505392224e-06, 
       9.3218889974755906e-06, 
       8.1666993175257805e-06, 
       8.3145078168011290e-06, 
       8.3121202645472809e-06, 
       9.0863819992738258e-06, 
       8.0665237825047248e-06, 
       8.0665237825047248e-06, 
       9.2668813377300616e-06, 
       7.9403300894960625e-06, 
       7.9423071954399177e-06, 
       1.1162899207149065e-05, 
       1.1128981634165691e-05, 
       9.3130325154253909e-06, 
       9.9447426443385946e-06, 
       7.9557724566108447e-06, 
       7.9557724566108447e-06, 
       9.1969239597557831e-06, 
       9.7137420484488393e-06, 
       9.6988226889307845e-06, 
       8.6084317975669163e-06, 
       8.2079364984215487e-06, 
       9.2664193044474990e-06, 
       9.3982271571577779e-06, 
       9.5640531537992826e-06, 
       9.5640531537992826e-06, 
       8.2449659480421945e-06, 
       1.1124283028070484e-05, 
       1.0230642787435756e-05, 
       1.0752795102935563e-05, 
       8.1338138265136051e-06, 
       9.8627448433822505e-06, 
       8.4650446148437809e-06, 
       9.2391110915947779e-06, 
       1.1106049283376227e-05, 
       9.8378477792810784e-06, 
       8.9389654608879446e-06, 
       9.8346640257821793e-06, 
       8.9382058328346428e-06, 
       8.3484270957921402e-06, 
       1.0233371573847037e-05, 
       1.1088021841344722e-05, 
       9.2783339610245754e-06, 
       8.3225209852060744e-06, 
       9.2128916416207421e-06, 
       7.9901962606480104e-06, 
       9.9016690306449276e-06, 
       7.8386061408966458e-06, 
       9.2005141561892868e-06, 
       8.2984346169051845e-06, 
       8.6539252123255984e-06, 
       9.2124512922854893e-06, 
       9.5115349495428261e-06, 
       9.0089720991261276e-06, 
       8.2100488070487949e-06, 
       8.4271242573932820e-06, 
       9.4131783256638764e-06, 
       7.9100925880461155e-06, 
       1.0847178748180059e-05, 
       9.0450837230742403e-06, 
       8.0075310697306971e-06, 
       8.8941358635953865e-06, 
       8.0420935970326870e-06, 
       9.1530907382480584e-06, 
       8.0362852081996535e-06, 
       8.5963193552956587e-06, 
       7.9555716617612778e-06, 
       9.1513943716710290e-06, 
       8.2216892573174026e-06, 
       7.8785993652022820e-06, 
       1.0681612575009408e-05, 
       9.4959527473528723e-06, 
       9.2302143646798746e-06, 
       9.2523306144933668e-06, 
       7.7994491477371091e-06, 
       8.0348031480158002e-06, 
       8.5346461244377665e-06, 
       1.0870129031629068e-05, 
       1.0719784547633156e-05, 
       1.0570725731885925e-05, 
       1.0254791872251513e-05, 
       1.1003608909046722e-05, 
       1.1002988321622837e-05, 
       1.0700791483295746e-05, 
       1.0677495842633823e-05, 
       9.0585154067311918e-06, 
       9.2687599562972307e-06, 
       9.4945450392505599e-06, 
       1.0508822285229782e-05, 
       8.3197492905251875e-06, 
       8.3788156141465420e-06, 
       1.0717755599924656e-05, 
       1.0917401695371128e-05, 
       7.8812828229993523e-06, 
       1.0889336079675683e-05, 
       9.9857422280705700e-06, 
       8.4230134506980076e-06, 
       8.6870954671172861e-06, 
       7.9550458896130485e-06, 
       1.0862220316611429e-05, 
       1.0597348878727284e-05, 
       8.1776104126236317e-06, 
       8.7995753931053764e-06, 
       8.8488374657337124e-06, 
       1.0274406831173117e-05, 
       1.0921586273018134e-05, 
       1.0617489836286677e-05, 
       7.9396086210649903e-06, 
       9.0555060309151793e-06, 
       7.7322114107149703e-06, 
       7.9568345156984707e-06, 
       7.7296661513056158e-06, 
       9.8567056480158617e-06, 
       1.0828355356545615e-05, 
       8.1632875317698573e-06, 
       9.9980436073042535e-06, 
       8.7756386298331642e-06, 
       9.5496381812452384e-06, 
       8.1997384717698682e-06, 
       9.1290559422702391e-06, 
       1.0069982616257073e-05, 
       9.8107269212403520e-06, 
       8.7745081171285365e-06, 
       9.2902573090105335e-06, 
       8.8080810145371141e-06, 
       8.5183774160555534e-06, 
       9.0119386789277543e-06, 
       9.3845757811806129e-06, 
       8.6727001937326769e-06, 
       7.7012325764527980e-06, 
       7.7709031664546681e-06, 
       9.3561672883305658e-06, 
       8.0433027691680343e-06, 
       8.5328585984970745e-06, 
       1.0245847881954835e-05, 
       9.0314913757531635e-06, 
       9.0268453426130139e-06, 
       8.5328502014238343e-06, 
       7.9047978874289847e-06, 
       9.0042641928965590e-06, 
       8.1802765907632405e-06, 
       9.2369147618282662e-06, 
       8.5973714904737580e-06, 
       7.7833564634743368e-06, 
       7.6958721290966612e-06, 
       7.6819578750149307e-06, 
       7.9003985346023638e-06, 
       9.1000345798889766e-06, 
       7.8437884963041966e-06, 
       7.6852760955990472e-06, 
       8.3293754165624045e-06, 
       1.0301855177730258e-05, 
       1.0411235492559563e-05, 
       9.7190533834427092e-06, 
       1.0665759078555350e-05, 
       8.7343770647866526e-06, 
       7.7247384834149740e-06, 
       8.9391403154481494e-06, 
       1.0438960006456072e-05, 
       7.6768868726883430e-06, 
       8.2049849145992558e-06, 
       8.7161004404253225e-06, 
       8.3514627692815660e-06, 
       8.3362377768541952e-06, 
       9.3954646876017580e-06, 
       8.7348770311387645e-06, 
       7.7703674294009935e-06, 
       7.8840869331188450e-06, 
       8.0832225024337957e-06, 
       9.5744946107751445e-06, 
       7.9859711956653885e-06, 
       8.9665073673582473e-06, 
       7.7522570552000642e-06, 
       7.9275241583138652e-06, 
       7.9744162316582475e-06, 
       1.0520314203986481e-05, 
       8.0108409499673174e-06, 
       7.8412835969871585e-06, 
       1.0635794769335427e-05, 
       9.0882293627284289e-06, 
       7.6864339919134233e-06, 
       9.6538772515057335e-06, 
       8.2689131120307599e-06, 
       7.9016347659010911e-06, 
       8.2033203665399499e-06, 
       8.6152172990149733e-06, 
       1.0195641252320373e-05, 
       8.5603586538935639e-06, 
       1.0383119443405808e-05, 
       8.9560245047916898e-06, 
       1.0549226444422211e-05, 
       7.7978952138079091e-06, 
       9.3775407997062509e-06, 
       7.9903078497188967e-06, 
       7.5747733294260965e-06, 
       8.2244928215429057e-06, 
       8.0130976294730647e-06, 
       7.7532604664496800e-06, 
       1.0183504612127337e-05, 
       8.6303187536373533e-06, 
       1.0674042092238093e-05, 
       1.0461883361577260e-05, 
       8.1668144868069544e-06, 
       1.0544989293039738e-05, 
       7.6640394542293375e-06, 
       1.0340164451212245e-05, 
       9.2198750334892818e-06, 
       9.2537332410885918e-06, 
       8.0093887382391771e-06, 
       7.7412763915784586e-06, 
       8.3816165834963131e-06, 
       9.6740786322865163e-06, 
       7.6098632375998587e-06, 
       8.6082039998256647e-06, 
       8.0668497052821541e-06, 
       8.0690616456404578e-06, 
       8.5154007010916605e-06, 
       7.5992216221021936e-06, 
       8.1620506992688687e-06, 
       8.2382536268793300e-06, 
       8.7401136645946120e-06, 
       7.7492106044145923e-06, 
       7.9342063839498765e-06, 
       8.0855420810425071e-06, 
       7.8501600772728562e-06, 
       8.8216392840787369e-06, 
       8.2417505156770137e-06, 
       8.1717314072238489e-06, 
       1.0529537367040197e-05, 
       8.1907584461971306e-06, 
       7.9061123553144055e-06, 
       8.6409523098516972e-06, 
       7.5388156540367476e-06, 
       9.2295502921108319e-06, 
       7.8734464299847550e-06, 
       7.6243967876995583e-06, 
       9.3087126958352223e-06, 
       9.2779255097197296e-06, 
       7.4938549443077470e-06, 
       1.0287262239378611e-05, 
       8.9521218321704762e-06, 
       8.5053047090478842e-06, 
       7.6768854336133028e-06, 
       8.8139990087388334e-06, 
       7.5812302051459350e-06, 
       9.1579781370846212e-06, 
       8.3525442477463480e-06, 
       8.9754630221939822e-06, 
       8.0398994293974298e-06, 
       7.8955052057535876e-06, 
       7.4726216040131691e-06, 
       7.5100997582979282e-06, 
       8.7229574002592308e-06, 
       7.6753502954028464e-06, 
       9.1357389198633441e-06, 
       7.6294121191425054e-06, 
       8.7437604477771877e-06, 
       1.0115174160072227e-05, 
       7.5802989770960316e-06, 
       7.4876012667325821e-06, 
       1.0060169217245969e-05, 
       1.0037132983848174e-05, 
       7.6156354674581140e-06, 
       7.4402296822310429e-06, 
       1.0464512661222532e-05, 
       9.2305340931117760e-06, 
       1.0474915173810910e-05, 
       7.9489144080405492e-06, 
       7.4990809942112595e-06, 
       8.4738663846739184e-06, 
       7.8254269818318990e-06, 
       8.8813083656097161e-06, 
       7.6501296179721846e-06, 
       7.7478829669640355e-06, 
       7.3913688321490851e-06, 
       9.2417469302986720e-06, 
       1.0445440324972980e-05, 
       7.4193606906722409e-06, 
       1.0099328646795399e-05, 
       7.5375142372720485e-06, 
       9.3145216951914641e-06, 
       9.2143327967594379e-06, 
       8.5370309756764049e-06, 
       7.6624900513929440e-06, 
       8.6678651573534391e-06, 
       9.0598042155435764e-06, 
       8.6113128087237438e-06, 
       9.1631530855462216e-06, 
       9.7245454299966835e-06, 
       1.0395937131233272e-05, 
       8.3333042324777245e-06, 
       1.0274837844931824e-05, 
       7.9255127079015939e-06, 
       8.9071752241408234e-06, 
       8.9071752241408234e-06, 
       8.6453032592448570e-06, 
       9.9336776929419432e-06, 
       8.1059382105455300e-06, 
       7.3915988762433960e-06, 
       9.0536726748379587e-06, 
       8.4095319761341917e-06, 
       7.6862391376933517e-06, 
       7.7044253283202774e-06, 
       7.7609188999259821e-06, 
       9.0409648321802874e-06, 
       8.6081173272119171e-06, 
       8.7418252208292769e-06, 
       8.7418252208292769e-06, 
       9.5212150102631109e-06, 
       7.5573176042884998e-06, 
       7.8633644573347822e-06, 
       7.8398359799655931e-06, 
       1.0042893138034025e-05, 
       7.3744521957410257e-06, 
       9.6848206154956413e-06, 
       1.0359576163545988e-05, 
       1.0350391092904932e-05, 
       1.0352055924613761e-05, 
       8.6129963153709672e-06, 
       1.0243803239763929e-05, 
       9.1673913199148270e-06, 
       7.7601281464880230e-06, 
       8.7779643287890737e-06, 
       7.3713503515549822e-06, 
       7.3713503515549822e-06, 
       8.7083699729784656e-06, 
       7.5980363877438524e-06, 
       9.1428760147405649e-06, 
       8.4719482054638628e-06, 
       7.6247335739739216e-06, 
       8.1889684015296987e-06, 
       7.5269127567406981e-06, 
       9.2622992301378082e-06, 
       1.0035642918459950e-05, 
       7.8707223685400925e-06, 
       8.7277681350533409e-06, 
       7.3451493477256893e-06, 
       9.2728425367906016e-06, 
       7.4614093461191979e-06, 
       7.7075727683536397e-06, 
       8.5959454642636525e-06, 
       9.0318784538119424e-06, 
       9.7788261886435019e-06, 
       1.0291934679547354e-05, 
       8.6011960295699683e-06, 
       7.5915387001339136e-06, 
       7.3195623289504928e-06, 
       8.1207291197665533e-06, 
       1.0302512364327620e-05, 
       8.8051830512342144e-06, 
       7.5404385394625655e-06, 
       9.4683598132942462e-06, 
       8.3019876432134816e-06, 
       8.6876036564257291e-06, 
       7.6696174387384350e-06, 
       1.0257201462673399e-05, 
       7.2852140668582012e-06, 
       7.8173847771761922e-06, 
       8.5298161718801100e-06, 
       7.3009019774590933e-06, 
       1.0155697241880736e-05, 
       7.6691235381770117e-06, 
       8.0471136583741190e-06, 
       9.2610071247805391e-06, 
       8.9621828855895392e-06, 
       7.7628049480058955e-06, 
       9.0469969945026715e-06, 
       1.0164066053341233e-05, 
       7.7619632611060494e-06, 
       7.2995643133299642e-06, 
       9.4939567882661035e-06, 
       7.3867114100195525e-06, 
       7.4035314210113230e-06, 
       1.0218059520758884e-05, 
       7.8700345022050083e-06, 
       7.2839547681303544e-06, 
       8.6834388122850430e-06, 
       7.6332952143167844e-06, 
       1.0161348600242670e-05, 
       8.2234785347685320e-06, 
       9.7205725339627478e-06, 
       9.3756585031684713e-06, 
       7.4713380739200797e-06, 
       1.0208921133790667e-05, 
       8.6805370358456099e-06, 
       7.6227730155946613e-06, 
       7.7387847229100630e-06, 
       7.8921539272629087e-06, 
       7.4502461498157010e-06, 
       8.1994657054351247e-06, 
       1.0181221721079982e-05, 
       7.3884764847084409e-06, 
       8.2410530143383462e-06, 
       9.7820539422396204e-06, 
       7.3442218566782460e-06, 
       7.1960192227982234e-06, 
       1.0136864191515192e-05, 
       8.4555878520634406e-06, 
       7.4257835245479381e-06, 
       8.1263302751145928e-06, 
       7.6841579379593006e-06, 
       1.0156269091877432e-05, 
       7.8213516604529091e-06, 
       7.3290925495992384e-06, 
       7.2823986026147364e-06, 
       7.5291771598954287e-06, 
       8.7799244618344682e-06, 
       7.4935611001727147e-06, 
       7.4912648651637252e-06, 
       7.4903252961214336e-06, 
       9.2636242397806070e-06, 
       7.3563649245148374e-06, 
       7.1986565167205011e-06, 
       7.5034079436264009e-06, 
       7.6675601425445385e-06, 
       7.5983589005594174e-06, 
       9.5966072688652656e-06, 
       9.9735328138779986e-06, 
       7.5069530377776847e-06, 
       7.8530422891891276e-06, 
       7.4604321377178627e-06, 
       7.2579046482994943e-06, 
       1.0120621440372561e-05, 
       8.9915304454425184e-06, 
       1.0093439660243728e-05, 
       8.1883193553535821e-06, 
       9.2840392333524554e-06, 
       7.9191577538986502e-06, 
       9.5590538408543123e-06, 
       8.9968656205260790e-06, 
       8.3096176131982932e-06, 
       7.7164384381443804e-06, 
       8.4936355911240060e-06, 
       7.2778495668730288e-06, 
       9.6110688738410582e-06, 
       8.0381419829550866e-06, 
       7.6861802837996033e-06, 
       8.3856990374699769e-06, 
       9.8299537980685379e-06, 
       7.9403649077885954e-06, 
       7.2954467465183168e-06, 
       8.3797149674630468e-06, 
       7.4248086665694123e-06, 
       9.5724833618397886e-06, 
       8.5702688031091007e-06, 
       8.5702688031091007e-06, 
       7.2797403433225919e-06, 
       8.3778500700876071e-06, 
       8.4324248743694243e-06, 
       7.2715569552598720e-06, 
       7.7580289807621388e-06, 
       8.1998293214437180e-06, 
       1.0033577001400010e-05, 
       7.2950433907370081e-06, 
       7.2942998560565074e-06, 
       9.8768428626717588e-06, 
       8.3448342469311900e-06, 
       7.6022904488171252e-06, 
       9.7454991896392173e-06, 
       7.7210622996243804e-06, 
       8.8264807025657653e-06, 
       7.9521900405611917e-06, 
       8.1650712336678060e-06, 
       9.8740593447710227e-06, 
       9.8740593447710227e-06, 
       9.6449601723023314e-06, 
       8.1588287999408652e-06, 
       9.4723107647543282e-06, 
       7.3761702053635474e-06, 
       8.1258377874538328e-06, 
       8.1256431244713322e-06, 
       7.2349402453334237e-06, 
       9.4275061385919854e-06, 
       8.1072772872498990e-06, 
       7.7976303675472471e-06, 
       9.6073814282074118e-06, 
       7.0758604436048121e-06, 
       7.0758604436048121e-06, 
       7.4186564220463898e-06, 
       7.5512353127705052e-06, 
       8.6517925198525297e-06, 
       8.0920101996311514e-06, 
       8.8763120998898750e-06, 
       7.5410748994443664e-06, 
       7.1286199751796116e-06, 
       7.4609823016266310e-06, 
       7.5191772934971114e-06, 
       8.6069665581161527e-06, 
       7.2782077190550146e-06, 
       7.4917849195692477e-06, 
       9.7196388123727519e-06, 
       9.9697199916447367e-06, 
       8.2928682886990817e-06, 
       8.2772573089046558e-06, 
       9.8086798052902393e-06, 
       7.1327035321631547e-06, 
       9.5757342656985402e-06, 
       7.0639503905372047e-06, 
       7.6897575476941053e-06, 
       8.2559756910233242e-06, 
       8.1997055593983706e-06, 
       9.2349159468474991e-06, 
       7.1956662051401743e-06, 
       7.1956662051401743e-06, 
       9.8679499392978896e-06, 
       9.8270500926272297e-06, 
       8.2876486069556001e-06, 
       8.5554930389289843e-06, 
       8.1985629601931549e-06, 
       7.1323774024794711e-06, 
       8.3312776173427957e-06, 
       8.2447165353865727e-06, 
       9.1956245273527530e-06, 
       9.1956245273527530e-06, 
       8.8090692126762870e-06, 
       7.3616308877820578e-06, 
       9.7256898932906996e-06, 
       9.7328930037142975e-06, 
       9.8083203169867580e-06, 
       9.6305724349251221e-06, 
       8.5912271533917025e-06, 
       9.7781133476709457e-06, 
       7.0844994599387306e-06, 
       9.6337617630833450e-06, 
       7.0311783752497355e-06, 
       9.8457077282660722e-06, 
       7.3401881967765916e-06, 
       9.8816836353625960e-06, 
       7.8144078171565622e-06, 
       8.3267294054764985e-06, 
       8.2041054121670727e-06, 
       9.7886383390755596e-06, 
       8.1304903494721963e-06, 
       9.7530866090488051e-06, 
       7.3794304395828917e-06, 
       8.1988392948123450e-06, 
       6.9708920314343526e-06, 
       7.5999379411477947e-06, 
       7.2695447279310253e-06, 
       8.5539667019671981e-06, 
       7.4620572273169090e-06, 
       9.8319109962910036e-06, 
       8.1091444806706032e-06, 
       7.5153599271950633e-06, 
       6.9863981007945262e-06, 
       7.5326525960982858e-06, 
       6.9753609673637478e-06, 
       7.2600523951827833e-06, 
       8.3463825467006557e-06, 
       7.3326980319849971e-06, 
       7.7183924030754683e-06, 
       9.7894715382345001e-06, 
       7.6962795301090991e-06, 
       9.7553875977186139e-06, 
       8.9598433263375079e-06, 
       8.3528111122777484e-06, 
       8.1457153620441806e-06, 
       7.8378057518544266e-06, 
       6.9819447889327140e-06, 
       9.4314976476721552e-06, 
       8.2723445659586661e-06, 
       7.0553177804164477e-06, 
       7.2965496146168832e-06, 
       8.2533591122562355e-06, 
       8.1170332083824554e-06, 
       9.7741631699649544e-06, 
       7.8211796525313481e-06, 
       8.7700068071272317e-06, 
       8.4943294136905936e-06, 
       9.2414115201871608e-06, 
       9.6906542939443421e-06, 
       8.8922999139681438e-06, 
       8.1348538305619821e-06, 
       7.2695512334270988e-06, 
       8.9039893920663026e-06, 
       8.3492574079326468e-06, 
       7.0929200648926815e-06, 
       7.1584122384943713e-06, 
       9.2019341833575309e-06, 
       9.6509862313190868e-06, 
       6.9376083257949937e-06, 
       7.2879794345286969e-06, 
       9.6946987688541102e-06, 
       6.9359331277862900e-06, 
       7.2631456512703167e-06, 
       9.5846130198507103e-06, 
       7.0123578707325427e-06, 
       8.0720724366374411e-06, 
       7.1408981820968414e-06, 
       9.0389909888344436e-06, 
       9.6828526620404540e-06, 
       6.9958365486258549e-06, 
       7.0594675875800848e-06, 
       8.4366020628842735e-06, 
       9.4127420836238821e-06, 
       7.2939525196515608e-06, 
       7.8253484974940789e-06, 
       7.4774724884137749e-06, 
       7.9031259249848090e-06, 
       9.3221081355370253e-06, 
       9.1818530247256396e-06, 
       7.0609278101675739e-06, 
       8.0403582403689009e-06, 
       6.8414073614937899e-06, 
       8.0371142262716750e-06, 
       6.9655204734358998e-06, 
       7.3131279198415955e-06, 
       6.9562856729521153e-06, 
       9.4065281218122541e-06, 
       7.3619075244112493e-06, 
       8.0655738756565442e-06, 
       8.0113891974961052e-06, 
       7.0463997603980868e-06, 
       7.4620419119801679e-06, 
       8.1769544433499168e-06, 
       8.1769544433499168e-06, 
       7.2541459096988303e-06, 
       7.3002442721849942e-06, 
       9.3602525278444049e-06, 
       8.7621550572143473e-06, 
       9.5587543300814092e-06, 
       8.0205676051001481e-06, 
       7.5648356011898606e-06, 
       9.6185693974811439e-06, 
       7.2740577344015332e-06, 
       9.2887238767578941e-06, 
       6.9573154279322068e-06, 
       7.4056419912235992e-06, 
       7.4056419912235992e-06, 
       8.7725388811248681e-06, 
       7.6651866035158580e-06, 
       7.0485646909336939e-06, 
       8.7722378125792570e-06, 
       8.6498391581864232e-06, 
       7.4995783105403575e-06, 
       8.0225029362850735e-06, 
       9.6112366408622248e-06, 
       9.3155322710559862e-06, 
       8.1020860055524957e-06, 
       7.3322963033713459e-06, 
       8.8422095295971853e-06, 
       8.3166421304567018e-06, 
       8.5922142292589426e-06, 
       7.1019906471807894e-06, 
       9.0948657825844880e-06, 
       7.1967065174912637e-06, 
       7.6841067043832354e-06, 
       9.5803440427786459e-06, 
       9.0405665292249316e-06, 
       6.8738472867025731e-06, 
       8.1382816103142463e-06, 
       7.7318304603220828e-06, 
       6.9744889729743760e-06, 
       9.1963561541976896e-06, 
       8.0621597026890802e-06, 
       8.0058682119296321e-06, 
       7.1341012664360027e-06, 
       7.1341012664360027e-06, 
       7.2561597411357307e-06, 
       6.9440398237390533e-06, 
       7.8107561287240352e-06, 
       6.7785646817114017e-06, 
       8.0130136286493813e-06, 
       9.4831266625608775e-06, 
       8.0581781457332993e-06, 
       7.5520164974274102e-06, 
       7.7473483871527097e-06, 
       7.6975254681945178e-06, 
       7.1132307101934392e-06, 
       7.9031386051158613e-06, 
       6.9053521317003693e-06, 
       7.9239112646334187e-06, 
       9.4561527580235451e-06, 
       9.5271632773332470e-06, 
       6.9984252377406487e-06, 
       7.4517070363776825e-06, 
       7.2639632698890872e-06, 
       7.8178559768832763e-06, 
       8.7697158820766970e-06, 
       8.1420230389920002e-06, 
       7.0334899732600404e-06, 
       7.1714930142710134e-06, 
       9.4330078740725921e-06, 
       8.8091701184626974e-06, 
       7.2706547785382884e-06, 
       7.0115161570043416e-06, 
       9.0423830955011811e-06, 
       9.3999744775062304e-06, 
       6.7425448450299707e-06, 
       8.8241773079381535e-06, 
       7.2002236700754120e-06, 
       6.8312840162122383e-06, 
       9.4345453310146607e-06, 
       7.3660770575756916e-06, 
       7.3418363857439904e-06, 
       8.1188860987120926e-06, 
       8.9185186984834552e-06, 
       9.4729908571243130e-06, 
       7.7557560782905311e-06, 
       9.1875365543623014e-06, 
       7.9733320674767990e-06, 
       8.2145122776506850e-06, 
       7.0333805808212549e-06, 
       8.4172920601199673e-06, 
       7.0555351115119565e-06, 
       7.0782694199661069e-06, 
       8.2311884686145158e-06, 
       8.3518479559606461e-06, 
       9.4236289155637985e-06, 
       9.4401107224254513e-06, 
       8.5418851893152376e-06, 
       9.2137575148251859e-06, 
       7.9264400671756219e-06, 
       7.8251546424924307e-06, 
       9.3718368389183097e-06, 
       6.9583678457639486e-06, 
       6.7349676129490696e-06, 
       6.7563397036981006e-06, 
       7.3294539024553638e-06, 
       6.6607173307959310e-06, 
       7.3590952960829240e-06, 
       7.3059731209727630e-06, 
       7.8922593944444480e-06, 
       8.4318575308163327e-06, 
       9.4011807194441671e-06, 
       8.9339173219913714e-06, 
       8.0839945736551128e-06, 
       6.7467625964429040e-06, 
       6.7467625964429040e-06, 
       8.3588442700307246e-06, 
       6.8317009949242571e-06, 
       7.0130639700049206e-06, 
       8.0920101996331639e-06, 
       7.1808964872536828e-06, 
       7.0571409558812126e-06, 
       6.6548233743813297e-06, 
       7.7044994213222037e-06, 
       6.6342955198375408e-06, 
       7.8790143897317364e-06, 
       6.8280299424638658e-06, 
       6.8556152754124818e-06, 
       6.9791557535340669e-06, 
       6.6248946547636529e-06, 
       9.2147722391755671e-06, 
       6.6505806544854734e-06, 
       6.6505806544854734e-06, 
       7.4778303668220133e-06, 
       9.2231494546389956e-06, 
       7.3725528621304886e-06, 
       7.7840117742958585e-06, 
       9.3318422593327715e-06, 
       7.4034329758736306e-06, 
       7.2214648167241815e-06, 
       7.0699867118935984e-06, 
       7.6462040038293282e-06, 
       9.2291686206828167e-06, 
       7.9159964893499741e-06, 
       6.9113012173785423e-06, 
       7.1353400647428976e-06, 
       6.6103222641621750e-06, 
       9.3102462488111512e-06, 
       9.0746291707480987e-06, 
       6.6268259049878282e-06, 
       7.0367349677029740e-06, 
       9.0786076153189231e-06, 
       7.8584827224449888e-06, 
       9.2531473044081081e-06, 
       9.2986245220478116e-06, 
       8.9417254999352876e-06, 
       6.7897802955711695e-06, 
       8.7688459716966336e-06, 
       7.2569612885574776e-06, 
       7.7163313472312421e-06, 
       8.7220978972782175e-06, 
       7.0998373539109099e-06, 
       7.5627052324265657e-06, 
       8.4650258913157174e-06, 
       6.9828998804383820e-06, 
       7.9746856665137826e-06, 
       6.9297543682532431e-06, 
       6.5771804244100982e-06, 
       6.5771804244100982e-06, 
       7.4890132312797471e-06, 
       7.3979498735569698e-06, 
       9.2725911981186198e-06, 
       9.0817363724073848e-06, 
       9.1432624788577325e-06, 
       7.2457460287555945e-06, 
       7.1963679331424395e-06, 
       7.2502253066581621e-06, 
       6.8920983307010275e-06, 
       7.7893943370099836e-06, 
       8.6433265963005241e-06, 
       7.2805967397125414e-06, 
       8.5244315921790046e-06, 
       7.7550473013884136e-06, 
       7.0567694936574281e-06, 
       7.8521916916546849e-06, 
       6.9072254622430874e-06, 
       7.8477381502237174e-06, 
       9.1766855908569807e-06, 
       7.9645294428216512e-06, 
       8.8291190718911000e-06, 
       9.2336847247763000e-06, 
       6.6015429233127112e-06, 
       7.7042302017165378e-06, 
       9.2288231303598275e-06, 
       7.9055938434267945e-06, 
       7.4592961950107909e-06, 
       6.5491929133676266e-06, 
       7.6620451945574120e-06, 
       8.9221300377110478e-06, 
       8.3010984150470864e-06, 
       6.5474248716219182e-06, 
       8.0798256190126098e-06, 
       8.0798256190126098e-06, 
       7.6308146864574621e-06, 
       9.1052830972083158e-06, 
       6.6356742232704856e-06, 
       7.6478204372606517e-06, 
       6.7877071224781175e-06, 
       6.7043707098962448e-06, 
       6.9696284110407584e-06, 
       6.8430889404345125e-06, 
       7.8145467752953302e-06, 
       7.5895290214197811e-06, 
       8.2979264840318532e-06, 
       7.0225138182682791e-06, 
       6.5598947379226666e-06, 
       6.5263582528281802e-06, 
       8.0185240534889043e-06, 
       6.8591369464065336e-06, 
       6.6246946532254745e-06, 
       7.5831469362897161e-06, 
       6.5760772155722983e-06, 
       6.5860489774443317e-06, 
       9.1619977610123552e-06, 
       6.8396081617743118e-06, 
       9.1607080319894974e-06, 
       7.7069453503989028e-06, 
       7.5255181879462910e-06, 
       7.2423229291722075e-06, 
       6.5535409390625275e-06, 
       6.8339596578175687e-06, 
       6.5521607800905488e-06, 
       6.8199063245202792e-06, 
       7.6166767767009324e-06, 
       7.1642574103318959e-06, 
       9.1368235071684835e-06, 
       8.7001839197489778e-06, 
       6.6742214994262911e-06, 
       7.0055951277443041e-06, 
       8.5909450277774095e-06, 
       7.4923095572379223e-06, 
       6.8897996141610245e-06, 
       8.2064513474718100e-06, 
       7.0183627537525882e-06, 
       6.8217802487833250e-06, 
       8.7339399319581167e-06, 
       7.6249760639406681e-06, 
       8.2845651555819609e-06, 
       6.4672227601571984e-06, 
       7.0055068108393741e-06, 
       7.7161503679644622e-06, 
       8.8286353597846351e-06, 
       8.9510162436432471e-06, 
       6.5684851092576428e-06, 
       6.5684851092576428e-06, 
       6.5508866889942678e-06, 
       6.9180462463258925e-06, 
       8.5140284645219045e-06, 
       7.6621449265000683e-06, 
       9.0602526508623823e-06, 
       7.1518686419180610e-06, 
       6.5506460651062299e-06, 
       6.7914480886844448e-06, 
       7.7286393475152642e-06, 
       7.7286393475152642e-06, 
       7.7938798587020840e-06, 
       6.6237503088493044e-06, 
       8.2824168058431387e-06, 
       6.5322330079088950e-06, 
       8.2952146977195562e-06, 
       7.6354298398456278e-06, 
       6.7797129833401708e-06, 
       7.8277446877155383e-06, 
       8.8513599619117221e-06, 
       7.5720400061854970e-06, 
       8.5175754060970068e-06, 
       9.0636993764960856e-06, 
       6.5667610439140813e-06, 
       7.9052856087990585e-06, 
       7.8898233996348284e-06, 
       6.7711524854142965e-06, 
       9.0665994778222575e-06, 
       7.1946378227210689e-06, 
       6.7036901087952936e-06, 
       6.4384530719176966e-06, 
       6.5301888219157093e-06, 
       7.0906931992334687e-06, 
       9.0465546796408257e-06, 
       7.4465974784187845e-06, 
       6.8144679288521984e-06, 
       7.2643321294304105e-06, 
       8.6656111308997235e-06, 
       6.8648583168668612e-06, 
       8.3691712041021568e-06, 
       8.3691712041021568e-06, 
       6.7620815394756523e-06, 
       6.6778598689149639e-06, 
       7.6543380767563227e-06, 
       8.7542739742999081e-06, 
       9.0393374809911484e-06, 
       8.5187046038331045e-06, 
       8.7412468174090400e-06, 
       8.8855512930596958e-06, 
       7.5091204148449903e-06, 
       7.6307304300467335e-06, 
       6.4096719389816361e-06, 
       8.9099621029462012e-06, 
       8.3246379116954724e-06, 
       6.5962157263513901e-06, 
       6.3933113114614037e-06, 
       7.3519486084261897e-06, 
       6.4969275509580127e-06, 
       6.6857146335327857e-06, 
       7.8546493713742808e-06, 
       8.0013724890205717e-06, 
       7.6421612553330652e-06, 
       6.6374329641449255e-06, 
       8.0623647794336033e-06, 
       8.8039277632004543e-06, 
       8.9963744768525235e-06, 
       8.9871604762801666e-06, 
       7.2834477113909146e-06, 
       6.5228085492378918e-06, 
       7.3498006447304801e-06, 
       6.6019437100402593e-06, 
       8.7742980251340573e-06, 
       8.9849436974476711e-06, 
       6.5309578916674192e-06, 
       6.3867657018400496e-06, 
       8.6185752280280464e-06, 
       7.4675639816437450e-06, 
       6.9563312052050034e-06, 
       7.8859551537378748e-06, 
       8.9754900345756206e-06, 
       7.4500106276723023e-06, 
       8.9688963519355511e-06, 
       6.5582768655317222e-06, 
       6.4242157115110067e-06, 
       6.7375196424337785e-06, 
       6.6024831789930687e-06, 
       6.6024831789930687e-06, 
       6.7341810314687135e-06, 
       6.6591381745472444e-06, 
       7.2641374856137321e-06, 
       7.2641374856137321e-06, 
       6.9497242991630664e-06, 
       7.0029522369466259e-06, 
       7.5768103314666172e-06, 
       7.3107897533005535e-06, 
       6.5225792984745791e-06, 
       7.1356366163167647e-06, 
       8.0692365433367166e-06, 
       7.4444256159581871e-06, 
       7.1460107286311461e-06, 
       8.4275147813363165e-06, 
       7.0167331257169006e-06, 
       8.5905769790151593e-06, 
       8.9242152432771606e-06, 
       6.8886601593604030e-06, 
       6.3474672375825963e-06, 
       7.1929088275143092e-06, 
       7.1929088275143092e-06, 
       6.5654030041431047e-06, 
       6.9297082384390362e-06, 
       6.9297082384390362e-06, 
       8.9174526297043432e-06, 
       7.4515543233904749e-06, 
       7.3364888995001921e-06, 
       8.2753648078383072e-06, 
       8.2753648078383072e-06, 
       6.5516713789820937e-06, 
       7.1295750680138787e-06, 
       6.3078553517831054e-06, 
       7.4103657809244833e-06, 
       8.3629971692932712e-06, 
       7.3616465907945938e-06, 
       6.3372398729124551e-06, 
       6.5706063501504345e-06, 
       6.6456744548685912e-06, 
       6.7757057733577256e-06, 
       7.4086594382659704e-06, 
       6.7505533631938813e-06, 
       6.5360404508770609e-06, 
       7.3150528154931461e-06, 
       8.7285503443286675e-06, 
       7.1561224822003936e-06, 
       6.7909187309139897e-06, 
       6.4355689958022936e-06, 
       6.4355689958022936e-06, 
       7.2020014942128767e-06, 
       7.5183185827913282e-06, 
       6.6064049470720472e-06, 
       6.3002053911673322e-06, 
       8.7862137373627173e-06, 
       6.5812627415038502e-06, 
       8.0189033321025132e-06, 
       8.7775055137521579e-06, 
       7.3997968472898606e-06, 
       7.4804915528694705e-06, 
       6.2686442548773163e-06, 
       7.6286438487275157e-06, 
       7.4441561761663887e-06, 
       6.3560278006408548e-06, 
       7.2319543526141497e-06, 
       7.2991791113879763e-06, 
       6.3254200073059143e-06, 
       6.6913621685037686e-06, 
       7.3866593829998167e-06, 
       6.6826615682511985e-06, 
       8.8438123135625688e-06, 
       7.4505820885705501e-06, 
       6.2588206990894915e-06, 
       8.1456444943191182e-06, 
       7.5222728635699052e-06, 
       6.2803341098763689e-06, 
       6.2602441708171866e-06, 
       8.4328009011942075e-06, 
       8.8311432645383168e-06, 
       6.2515316972227607e-06, 
       7.2108069833971247e-06, 
       6.2912568150644549e-06, 
       6.2460126475488882e-06, 
       8.4601248272563673e-06, 
       6.7695777696724625e-06, 
       7.1336281004901149e-06, 
       6.5815994036313524e-06, 
       8.7347914051599332e-06, 
       7.1428506898094357e-06, 
       6.2684736108602153e-06, 
       6.5343859108454189e-06, 
       6.5343859108454189e-06, 
       6.2757831161466615e-06, 
       6.7284843171276053e-06, 
       8.3997266957885311e-06, 
       8.3997266957885311e-06, 
       7.4157811085411235e-06, 
       8.6480482353852202e-06, 
       7.1208558260262440e-06, 
       6.7626071642231572e-06, 
       7.8799488566105269e-06, 
       8.3291560350584627e-06, 
       7.2475475861736808e-06, 
       6.5543031976112280e-06, 
       6.8115595953373345e-06, 
       7.3002588688526544e-06, 
       7.3975234915129323e-06, 
       8.4977761668622579e-06, 
       8.6231294225766718e-06, 
       6.4197123049953335e-06, 
       6.4197123049953335e-06, 
       8.7629367907792299e-06, 
       8.5908154202243397e-06, 
       8.5908154202243397e-06, 
       6.2353605710632741e-06, 
       7.2645486231084375e-06, 
       7.0322620696110055e-06, 
       6.2036627329208141e-06, 
       8.1272754545177550e-06, 
       6.8028685586738674e-06, 
       6.3498724064878148e-06, 
       7.3787077898494652e-06, 
       6.3723951624183599e-06, 
       6.8025496843978953e-06, 
       7.2119864764766553e-06, 
       7.3143418473540846e-06, 
       6.4179889974482465e-06, 
       6.1835817505237845e-06, 
       8.6336856396131326e-06, 
       7.6405947450649844e-06, 
       8.7251851985558355e-06, 
       8.6241585815222008e-06, 
       6.8835241358467028e-06, 
       7.2576878081917892e-06, 
       6.4854865965629602e-06, 
       8.5181092998597026e-06, 
       8.7227818054858713e-06, 
       6.7431547226799023e-06, 
       6.2023087452962673e-06, 
       6.2190136898280959e-06, 
       8.0517529083978908e-06, 
       7.7314946770714672e-06, 
       7.2185100316606121e-06, 
       6.3029787688162320e-06, 
       7.4929647548053749e-06, 
       7.3862276380618472e-06, 
       7.1929809476813677e-06, 
       7.4210924521698175e-06, 
       6.9845835557839355e-06, 
       7.2339623599891363e-06, 
       7.2980766945396167e-06, 
       8.7026353556718936e-06, 
       6.4647316007578916e-06, 
       7.6734902627708431e-06, 
       8.3065186011947007e-06, 
       8.2566268674679823e-06, 
       7.2494077172805987e-06, 
       7.2357705684560732e-06, 
       7.9992740120157997e-06, 
       6.5257031967176887e-06, 
       8.0672066659405771e-06, 
       6.2988898677024704e-06, 
       6.2984922292067476e-06, 
       7.3162518795899722e-06, 
       7.4335463672576897e-06, 
       6.6686373188145086e-06, 
       6.9745201458353265e-06, 
       8.3800912899299453e-06, 
       6.1472356474906756e-06, 
       6.1472356474906756e-06, 
       7.1565923928930791e-06, 
       7.2037210253070906e-06, 
       6.2465978486557630e-06, 
       8.3845452518161466e-06, 
       6.1915228536191935e-06, 
       8.6711183139505477e-06, 
       6.2369806080215025e-06, 
       8.6578060372424294e-06, 
       6.6783136515748090e-06, 
       7.4141226085344546e-06, 
       8.5216927834975987e-06, 
       7.8268616905179159e-06, 
       7.6463396581371580e-06, 
       8.4123595512830509e-06, 
       6.1638949566598170e-06, 
       6.1990793319867436e-06, 
       6.3181482506115036e-06, 
       6.9939253436335080e-06, 
       6.5324409026322684e-06, 
       6.5619591124630627e-06, 
       6.1524682206353792e-06, 
       7.0905174121898160e-06, 
       7.1288151843027341e-06, 
       6.1446167360678263e-06, 
       6.7301497602913626e-06, 
       6.5560817518133754e-06, 
       7.4271775398430279e-06, 
       6.4636228271698370e-06, 
       8.3106498673420008e-06, 
       7.1778968243807378e-06, 
       6.1822753990940506e-06, 
       8.2189031357146174e-06, 
       6.2512574404560243e-06, 
       7.0812799642936359e-06, 
       8.3412112430763813e-06, 
       8.4041652315430232e-06, 
       7.1813933911322022e-06, 
       7.6650721813987826e-06, 
       7.0604328833151582e-06, 
       8.3189547259836080e-06, 
       6.2751464385722669e-06, 
       6.0934149408327473e-06, 
       8.5986050107562305e-06, 
       8.3887845878644261e-06, 
       8.2418809165492090e-06, 
       6.9625758579767307e-06, 
       7.4353825551699999e-06, 
       7.9585870182116215e-06, 
       7.9585870182116215e-06, 
       8.4369630626473101e-06, 
       6.1734556018040296e-06, 
       7.5056861541878824e-06, 
       6.0783668890218329e-06, 
       7.9509980734048850e-06, 
       7.2901613995684128e-06, 
       6.4270647401612042e-06, 
       6.3049974903736912e-06, 
       8.1674320639395898e-06, 
       8.1329460571515801e-06, 
       8.4914168310770236e-06, 
       6.0825632765300573e-06, 
       6.4446768907984583e-06, 
       8.2424512034177528e-06, 
       6.0656163413195227e-06, 
       7.8261093991125811e-06, 
       6.1731412291248245e-06, 
       6.0660816018400033e-06, 
       6.0940640239179070e-06, 
       6.2433837713859445e-06, 
       6.1375235328924300e-06, 
       6.6776705464370996e-06, 
       7.6128317508891224e-06, 
       6.2056603881787041e-06, 
       6.3160420616000400e-06, 
       6.5742987544099159e-06, 
       7.9092953110400073e-06, 
       8.5284909185904371e-06, 
       6.6184495095073507e-06, 
       6.0576522769766956e-06, 
       7.4315152197510592e-06, 
       8.4779703869546865e-06, 
       6.1128876309425450e-06, 
       6.3703879958432397e-06, 
       7.7811102823324509e-06, 
       7.8750202562067952e-06, 
       7.2238162828457384e-06, 
       6.8948415927883536e-06, 
       8.4819216693938803e-06, 
       6.7026834853040547e-06, 
       6.1836968654195716e-06, 
       7.2262296594756434e-06, 
       7.7562435236739028e-06, 
       7.0713108416821150e-06, 
       8.4945452015843609e-06, 
       6.2601246886244100e-06, 
       6.2601246886244100e-06, 
       6.1746067175039328e-06, 
       6.1698152756815594e-06, 
       8.2583553190279654e-06, 
       6.1193385783123349e-06, 
       7.0005149266175923e-06, 
       6.4476117150199198e-06, 
       7.5418946531039307e-06, 
       8.1958285386800848e-06, 
       6.2252740761213526e-06, 
       7.0374422054511995e-06, 
       7.1836958738174674e-06, 
       6.9547061605343956e-06, 
       6.8646116799650280e-06, 
       7.0234915560450410e-06, 
       6.8579550878500184e-06, 
       8.0088908788046566e-06, 
       6.3717790147651571e-06, 
       8.0602157610638725e-06, 
       8.0602157610638725e-06, 
       8.2788596034278513e-06, 
       6.0005315897409951e-06, 
       8.3971474609980947e-06, 
       6.7370093815358890e-06, 
       6.1833571940134427e-06, 
       7.2747414815575755e-06, 
       7.2747414815575755e-06, 
       7.9913846801651110e-06, 
       6.3526933546381042e-06, 
       7.0306330494084037e-06, 
       7.0083104475583571e-06, 
       8.2834279063047264e-06, 
       8.4292375148169116e-06, 
       5.9918877153763712e-06, 
       6.2461249565140326e-06, 
       7.8524633572922532e-06, 
       8.3925346810958888e-06, 
       6.4081097893016474e-06, 
       7.8489647709845187e-06, 
       6.6856768207824598e-06, 
       6.3866921119764571e-06, 
       5.9892405905555438e-06, 
       6.0701932028368224e-06, 
       6.9712321049450497e-06, 
       6.1109418140015768e-06, 
       6.6012669730450818e-06, 
       6.0506203602609542e-06, 
       6.6736511403254016e-06, 
       8.0494660034072037e-06, 
       6.1205292162906291e-06, 
       8.0978505534266053e-06, 
       7.3812817444794255e-06, 
       6.4110196892760487e-06, 
       6.4110196892760487e-06, 
       6.7490130214699421e-06, 
       5.9712615862261181e-06, 
       6.4292076745542406e-06, 
       6.3130719625325062e-06, 
       6.1295861106979835e-06, 
       6.2637025182357376e-06, 
       6.3972976294907306e-06, 
       6.8813159639501813e-06, 
       7.5723629563698862e-06, 
       6.8625717617732620e-06, 
       8.1284194695849764e-06, 
       6.6479316370298611e-06, 
       6.2263761972644871e-06, 
       6.9708152592193059e-06, 
       6.7983349303912446e-06, 
       6.8703284751379293e-06, 
       6.0976748192703715e-06, 
       6.3608475042346744e-06, 
       7.8743041072102623e-06, 
       7.0345031277281029e-06, 
       6.7810659927237325e-06, 
       6.9347736888875799e-06, 
       6.7583587870433055e-06, 
       6.6319777344867035e-06, 
       6.5406443811060676e-06, 
       5.9169725475530489e-06, 
       7.0424778804689125e-06, 
       6.9260243933788475e-06, 
       6.9661061540103577e-06, 
       6.9961003659945388e-06, 
       6.1926937743323093e-06, 
       6.4429823386278980e-06, 
       6.1368810661376337e-06, 
       6.9216023301264990e-06, 
       6.0459918914417011e-06, 
       8.3242173023020648e-06, 
       6.0888987622463703e-06, 
       6.1310715485463229e-06, 
       6.1310715485463229e-06, 
       6.8223957501686080e-06, 
       6.8780900439435869e-06, 
       6.9122752959014334e-06, 
       7.7558589804353135e-06, 
       7.7558589804353135e-06, 
       6.8121686787746056e-06, 
       8.2394357110695498e-06, 
       6.0103545451384626e-06, 
       6.9112676985500028e-06, 
       6.8490000809427573e-06, 
       5.9478858413916054e-06, 
       6.6918009694643482e-06, 
       8.3067856191332953e-06, 
       6.5134515604246363e-06, 
       8.0053412612514628e-06, 
       7.6486185600385615e-06, 
       8.2175187069414970e-06, 
       5.9804848335056000e-06, 
       8.2134113302511345e-06, 
       6.9245866844205943e-06, 
       6.3290084134759874e-06, 
       6.3290084134759874e-06, 
       7.6721996046676507e-06, 
       7.6721996046676507e-06, 
       5.8648483127511116e-06, 
       5.9458163922010010e-06, 
       6.3107790695953235e-06, 
       5.9068665958937301e-06, 
       6.9935313475040688e-06, 
       6.0117711779888827e-06, 
       5.8580256296403368e-06, 
       8.2464337603668212e-06, 
       5.9000330282288397e-06, 
       6.7239790210275806e-06, 
       5.8647339778188461e-06, 
       6.3518259329211351e-06, 
       8.2639639079514514e-06, 
       6.1648806215504545e-06, 
       6.3958209581944024e-06, 
       6.1554199347364266e-06, 
       6.3267592137151926e-06, 
       6.3001486701815570e-06, 
       5.8388459020743659e-06, 
       6.2813181533750810e-06, 
       6.6288000738691927e-06, 
       7.8375324620365539e-06, 
       7.0750487145454557e-06, 
       6.9593370203302197e-06, 
       5.9700674882967042e-06, 
       5.8805998299486068e-06, 
       6.0180050424936755e-06, 
       5.8878890538961769e-06, 
       6.4824211789942815e-06, 
       6.0765911321755052e-06, 
       5.9967601821535024e-06, 
       7.0520722870666350e-06, 
       6.3176149866831145e-06, 
       7.3748711806243560e-06, 
       5.8189616333131275e-06, 
       7.6014493443479603e-06, 
       5.8233736629191989e-06, 
       6.8373681992870242e-06, 
       6.8893867413851278e-06, 
       5.9318576361326868e-06, 
       7.3134476890165212e-06, 
       8.1555369446671217e-06, 
       7.0122464557963211e-06, 
       6.3881590923954351e-06, 
       5.8398447070955233e-06, 
       6.3207942216425624e-06, 
       5.8332751990217967e-06, 
       8.1928479770367015e-06, 
       5.8770604104701679e-06, 
       8.0645650882193115e-06, 
       6.3034333371821745e-06, 
       5.8791713453888670e-06, 
       7.4084705330522823e-06, 
       8.0896799952733750e-06, 
       7.4120442912588264e-06, 
       7.3094391173652882e-06, 
       5.7979373306140173e-06, 
       6.7079521650846923e-06, 
       6.4037527534294436e-06, 
       6.1053186671584738e-06, 
       7.9308164370539397e-06, 
       6.1621818815777966e-06, 
       6.1566434379916117e-06, 
       6.7845263322245182e-06, 
       7.7980221149136742e-06, 
       7.5501020554867114e-06, 
       8.1512373358501598e-06, 
       5.8407734058995067e-06, 
       5.8400656589179236e-06, 
       6.4927170252261992e-06, 
       5.8322236659581226e-06, 
       5.9449373704409834e-06, 
       7.5999379411496277e-06, 
       7.0397198435935514e-06, 
       6.7823182210344033e-06, 
       6.0874095657918735e-06, 
       6.2049603864947197e-06, 
       5.9302654726162630e-06, 
       7.5961201142158137e-06, 
       5.8083048572853333e-06, 
       6.1532008705229079e-06, 
       7.6981518595158221e-06, 
       8.0389131218149023e-06, 
       6.2072050912998679e-06, 
       6.7747902997990278e-06, 
       6.7733298474800394e-06, 
       6.2768221739696796e-06, 
       6.6769342024508076e-06, 
       6.0477432292033850e-06, 
       5.7714161697798590e-06, 
       7.5284719740124397e-06, 
       6.5316724809324856e-06, 
       7.9623069978104681e-06, 
       5.8755946358505967e-06, 
       5.9163091637401178e-06, 
       6.0512281175250681e-06, 
       8.1017484342337481e-06, 
       6.0119256384045336e-06, 
       7.9561526216720767e-06, 
       8.0685433275937159e-06, 
       6.8420766310820408e-06, 
       8.0716495037554573e-06, 
       5.7289473493859735e-06, 
       6.0573140302087647e-06, 
       5.7395235886413482e-06, 
       6.2484738484557058e-06, 
       7.7972646991467567e-06, 
       8.0811787268130118e-06, 
       7.3988564796031372e-06, 
       5.9469531554759990e-06, 
       6.6506772899210516e-06, 
       6.7373197816786533e-06, 
       6.3679053471746231e-06, 
       5.7281206425357208e-06, 
       6.6157581857603464e-06, 
       7.1895041992855789e-06, 
       7.7175147111385762e-06, 
       7.8335209297753050e-06, 
       6.4474738351189731e-06, 
       6.1473590376885219e-06, 
       6.4462811753832024e-06, 
       6.9315983704938046e-06, 
       6.5934960330411564e-06, 
       7.6909102929830486e-06, 
       7.7382084477576520e-06, 
       7.8958765981027960e-06, 
       6.7010444446199084e-06, 
       7.0363867459435089e-06, 
       7.9488449063163525e-06, 
       6.1513222449936413e-06, 
       5.7514353877765550e-06, 
       5.7514353877765550e-06, 
       5.9715857229413774e-06, 
       6.7019987093615118e-06, 
       7.8075888571475219e-06, 
       7.3330234284166976e-06, 
       7.3330234284166976e-06, 
       5.9194422721262626e-06, 
       5.9194422721262626e-06, 
       6.9360382308100745e-06, 
       8.0270695148698613e-06, 
       6.2752805484684084e-06, 
       6.5215071537263699e-06, 
       6.0057877098106507e-06, 
       7.6706556038852359e-06, 
       7.3077098656291973e-06, 
       5.8229767964387140e-06, 
       7.0372824891309952e-06, 
       7.0372824891309952e-06, 
       6.0839371205176060e-06, 
       7.9066086530323954e-06, 
       5.6856416551133155e-06, 
       6.9353637843832049e-06, 
       6.5572033527013221e-06, 
       6.4514001338088802e-06, 
       6.6767303036216666e-06, 
       6.1175214978259281e-06, 
       6.1175214978259281e-06, 
       6.6575831375598063e-06, 
       8.0110207126909263e-06, 
       5.9099378205266888e-06, 
       5.9098300791842611e-06, 
       6.3701150726510827e-06, 
       7.9946757366469358e-06, 
       5.7893825844294062e-06, 
       7.0296094625248600e-06, 
       7.7028325447432575e-06, 
       6.5241433513711065e-06, 
       6.6071763689915883e-06, 
       7.6060322278667517e-06, 
       6.1477708627416665e-06, 
       6.0542347017678760e-06, 
       6.4735352353305927e-06, 
       6.0946528049094158e-06, 
       5.6647376191841529e-06, 
       7.3042566503805005e-06, 
       6.7143271916453903e-06, 
       6.0139650461696895e-06, 
       5.9842169302488418e-06, 
       5.9842169302488418e-06, 
       6.4299785419398079e-06, 
       6.2317807024477101e-06, 
       6.2262177149300052e-06, 
       5.7864124709614297e-06, 
       6.0301924055209276e-06, 
       6.4609454024067838e-06, 
       7.6969107109906059e-06, 
       7.9219023381550292e-06, 
       7.5863884577044585e-06, 
       7.2666436111268533e-06, 
       6.0851308685511238e-06, 
       7.7478829669651739e-06, 
       7.2488045749190594e-06, 
       6.6338462833589685e-06, 
       5.8919124089728720e-06, 
       6.2678476077611997e-06, 
       7.5147541677235143e-06, 
       7.3874730463023694e-06, 
       7.3874730463023694e-06, 
       6.7227909984932959e-06, 
       6.5866295301633824e-06, 
       5.6653002417601156e-06, 
       5.8302446078644253e-06, 
       5.8717080253342111e-06, 
       6.9437227914826160e-06, 
       7.9569976055179080e-06, 
       7.8498790395516665e-06, 
       7.1313566831986382e-06, 
       6.1332973158058391e-06, 
       6.8225786586583919e-06, 
       7.3384879520962539e-06, 
       7.3351227683695434e-06, 
       6.4459088014364410e-06, 
       7.6360177266591665e-06, 
       6.6049477963966903e-06, 
       7.1570096894811623e-06, 
       7.1841774515509554e-06, 
       6.0723232771644539e-06, 
       6.0723232771644539e-06, 
       5.9321793651896593e-06, 
       6.8220138345120145e-06, 
       6.1917424327761135e-06, 
       7.3839755161443332e-06, 
       5.6133427323961324e-06, 
       6.7229970697189676e-06, 
       6.2936294218716995e-06, 
       5.8377923414176172e-06, 
       6.3092551123587698e-06, 
       6.3693271990030199e-06, 
       6.3663575982817277e-06, 
       5.9243213065365439e-06, 
       5.6068254434507447e-06, 
       6.5689183661893125e-06, 
       6.5657493265859072e-06, 
       5.6993735776600820e-06, 
       6.4776597852050243e-06, 
       6.4776597852050243e-06, 
       5.7257643624273743e-06, 
       6.3593939696084577e-06, 
       5.6460360586571671e-06, 
       6.6770364984684734e-06, 
       5.8506440103248970e-06, 
       7.7184389661012812e-06, 
       5.9421062779038503e-06, 
       5.9421062779038503e-06, 
       6.8546566850660382e-06, 
       5.6723282087207926e-06, 
       6.5818602233431880e-06, 
       7.8869489038372725e-06, 
       7.1811186110926320e-06, 
       7.4372955415553001e-06, 
       7.5977223209783669e-06, 
       5.6928202452816236e-06, 
       7.8850645551993581e-06, 
       5.6625581909836898e-06, 
       5.8306605132059918e-06, 
       7.8789798532756016e-06, 
       7.7566069046830254e-06, 
       7.4910838759822679e-06, 
       5.5828519665234011e-06, 
       6.9278586241127189e-06, 
       7.6938811413423828e-06, 
       6.4112766801815158e-06, 
       7.0884042681500066e-06, 
       5.8806492669706556e-06, 
       7.1714331081377097e-06, 
       5.5985294380252112e-06, 
       6.3366455192755263e-06, 
       5.8778240938400084e-06, 
       7.4062681867521400e-06, 
       5.8641135301341802e-06, 
       6.5375684984458203e-06, 
       5.9402176325205669e-06, 
       6.6468985395814031e-06, 
       5.5668211270054751e-06, 
       5.8690810990016784e-06, 
       7.4984984025530162e-06, 
       5.6351178191807373e-06, 
       6.0133744164299965e-06, 
       7.6969290813172705e-06, 
       5.8789567852999778e-06, 
       6.4872040006237233e-06, 
       7.4227540129962615e-06, 
       6.3602999352574965e-06, 
       5.6523251102634305e-06, 
       6.3577452605491751e-06, 
       5.5371040560367191e-06, 
       5.5705088695345787e-06, 
       5.5361041581552081e-06, 
       7.2554233267141175e-06, 
       7.4373738922535855e-06, 
       7.8088952722643637e-06, 
       7.2830173818326053e-06, 
       6.9568999828649608e-06, 
       5.9140118219653703e-06, 
       5.8240186323996355e-06, 
       6.0970259878867018e-06, 
       6.4092862479087905e-06, 
       7.7784793313186358e-06, 
       7.7619573181044552e-06, 
       5.8730017161119043e-06, 
       5.8401850622854228e-06, 
       5.8235763918139724e-06, 
       5.7669563939944181e-06, 
       6.6022766558331848e-06, 
       7.1321619948413291e-06, 
       6.0368389773663254e-06, 
       6.6673008964104412e-06, 
       6.5493117199386200e-06, 
       5.5716230856974613e-06, 
       7.6799908115435664e-06, 
       7.7290513817957976e-06, 
       5.6099146253383544e-06, 
       5.5485933268136754e-06, 
       7.4525522040293373e-06, 
       6.7449051623924324e-06, 
       6.4953756911488208e-06, 
       7.2346209776431906e-06, 
       7.1292065314911228e-06, 
       5.9273508629383414e-06, 
       5.9273508629383414e-06, 
       5.7839268094209508e-06, 
       5.5680000741447003e-06, 
       7.6492018435427706e-06, 
       6.2844759672116925e-06, 
       7.1165594983431288e-06, 
       7.6916502679306475e-06, 
       6.8815559859282941e-06, 
       6.8815559859282941e-06, 
       5.6108584914714207e-06, 
       7.2824102475582366e-06, 
       6.6606419430841566e-06, 
       5.4987685858258669e-06, 
       6.8216817942333458e-06, 
       7.7667959472634922e-06, 
       7.1488937072269070e-06, 
       5.6116216692315893e-06, 
       6.3315460759110286e-06, 
       5.8270755096995049e-06, 
       7.4499916002647527e-06, 
       5.5211507504764959e-06, 
       5.5990383611166498e-06, 
       5.7293898980322094e-06, 
       6.0248802367462348e-06, 
       5.4987039912939833e-06, 
       5.5202340728467789e-06, 
       7.4301426307197604e-06, 
       5.5904900780697039e-06, 
       5.5042720306013878e-06, 
       5.5141979517683643e-06, 
       5.5269235920617102e-06, 
       6.0214572820884799e-06, 
       6.4865826138278284e-06, 
       7.6216301996646295e-06, 
       6.0358238377978600e-06, 
       7.4894777729884937e-06, 
       5.7974342735583936e-06, 
       7.5712199588158649e-06, 
       7.3206924896966611e-06, 
       7.1087810180296741e-06, 
       5.4698617244385636e-06, 
       7.4780109657501206e-06, 
       6.6320813301216291e-06, 
       6.1979183696187411e-06, 
       5.4734043257187459e-06, 
       6.7264912384989215e-06, 
       5.5360451687354046e-06, 
       6.2969860873987095e-06, 
       6.9740899334537264e-06, 
       6.6151806310918619e-06, 
       5.8341809562237005e-06, 
       6.3867657018405756e-06, 
       5.8185899516587961e-06, 
       7.4039834423309210e-06, 
       7.7013523877199202e-06, 
       6.0183424099203077e-06, 
       5.4542435306703439e-06, 
       7.3305289119770143e-06, 
       5.5811844266112707e-06, 
       5.6434823862782500e-06, 
       6.4929668990443809e-06, 
       5.4372467239796982e-06, 
       5.9726579061218906e-06, 
       5.8312507766350068e-06, 
       7.2236510002249492e-06, 
       7.2236510002249492e-06, 
       5.7141670995395799e-06, 
       5.7141670995395799e-06, 
       6.5034786740290710e-06, 
       7.0126583371818916e-06, 
       5.9660078903655377e-06, 
       7.1068545367516976e-06, 
       7.6602298658043909e-06, 
       5.4354355768316042e-06, 
       5.4978848314097929e-06, 
       6.5490024985775831e-06, 
       7.1874704758251506e-06, 
       6.6328622534756901e-06, 
       6.3053843994556907e-06, 
       5.5735529363233826e-06, 
       7.5042120962215634e-06, 
       6.2332311584478082e-06, 
       5.4412318464924084e-06, 
       6.6556685438093709e-06, 
       7.4367528489456366e-06, 
       6.3334701581461318e-06, 
       5.6790505656733595e-06, 
       7.1737151705823430e-06, 
       5.8435749372809484e-06, 
       5.8195704108619180e-06, 
       5.6176878292815404e-06, 
       5.4850296311379635e-06, 
       5.8287010682684441e-06, 
       5.8287010682684441e-06, 
       5.4738037276891584e-06, 
       5.6055901277558427e-06, 
       5.4675266565406102e-06, 
       5.6953850928486172e-06, 
       6.1663904937912220e-06, 
       5.4560019388275427e-06, 
       6.2268138566635983e-06, 
       6.7962333816708824e-06, 
       5.6546447487018889e-06, 
       6.9389468873509928e-06, 
       5.4577517309906123e-06, 
       6.8453771303658551e-06, 
       5.3968428111083162e-06, 
       6.1411821759672176e-06, 
       5.4050708321308964e-06, 
       5.5910215998506130e-06, 
       5.6206609822563032e-06, 
       5.4178990030535030e-06, 
       5.9608602327330703e-06, 
       7.4459940824368707e-06, 
       6.3127771923031804e-06, 
       6.8012510034988692e-06, 
       7.2413663755627394e-06, 
       6.3218599194809736e-06, 
       7.3856445382819625e-06, 
       6.4747231373514488e-06, 
       7.2107875488546526e-06, 
       5.3964591284108049e-06, 
       5.6219306846195998e-06, 
       5.6649535137684141e-06, 
       5.5133894026622594e-06, 
       7.5416625089222462e-06, 
       7.3079470537212988e-06, 
       6.3446360574820194e-06, 
       5.3656301911468203e-06, 
       5.4710651506518317e-06, 
       5.4710651506518317e-06, 
       5.4840420268021623e-06, 
       6.2748978648343780e-06, 
       7.2901978480760592e-06, 
       5.3927708541901849e-06, 
       7.1159973601950606e-06, 
       5.7153026356728936e-06, 
       5.6090909472119815e-06, 
       5.5785916958863474e-06, 
       7.5045253671804486e-06, 
       6.2693712495954386e-06, 
       5.4492635435953270e-06, 
       5.4492635435953270e-06, 
       6.2328120330743371e-06, 
       5.5759594053786031e-06, 
       7.5576678516437239e-06, 
       5.5673370723316717e-06, 
       5.7580359593978296e-06, 
       5.4856844222718987e-06, 
       5.7382760900903607e-06, 
       6.6524389859029789e-06, 
       5.3574339895901584e-06, 
       5.6719069383675439e-06, 
       6.2730897331371592e-06, 
       5.3458904277265951e-06, 
       5.8802830805125336e-06, 
       5.5713564450323236e-06, 
       5.6306620204435878e-06, 
       5.7053699802332208e-06, 
       5.6267657299126743e-06, 
       5.6302117785843122e-06, 
       6.6411939948092841e-06, 
       5.5580604017678049e-06, 
       5.5919946275602895e-06, 
       6.3789967109831761e-06, 
       5.6988958066157142e-06, 
       7.2846802435576741e-06, 
       5.4636368134338368e-06, 
       5.7673413517829630e-06, 
       5.6188064782030739e-06, 
       5.6188064782030739e-06, 
       6.0196062491091521e-06, 
       7.0309581977162748e-06, 
       5.6129535813662744e-06, 
       5.4967947110936696e-06, 
       6.7630353971120394e-06, 
       6.0463083708660807e-06, 
       5.4349722481497518e-06, 
       5.3986288969722914e-06, 
       5.6071254763456087e-06, 
       5.6071254763456087e-06, 
       6.2383096400005441e-06, 
       6.0445671933963983e-06, 
       5.3256019043161995e-06, 
       6.2497837976480703e-06, 
       5.7948908437514988e-06, 
       5.7948908437514988e-06, 
       6.7156070078061986e-06, 
       6.7156070078061986e-06, 
       5.6418828638371840e-06, 
       7.2403598629070618e-06, 
       5.8417567807660990e-06, 
       6.2820589589913953e-06, 
       6.1233772335214195e-06, 
       6.6065835442861159e-06, 
       6.3128895552476755e-06, 
       5.4834543918027496e-06, 
       6.3648115902874342e-06, 
       5.4643121356496141e-06, 
       5.7249126991342857e-06, 
       5.4399498756473093e-06, 
       5.2999463145210319e-06, 
       6.2283724913761078e-06, 
       6.4658032042281087e-06, 
       7.1768090046384347e-06, 
       5.5894564580435195e-06, 
       6.8840015723682449e-06, 
       5.9261397066392665e-06, 
       6.9779388588296297e-06, 
       5.3191498843007388e-06, 
       5.3191498843007388e-06, 
       5.9874063918786388e-06, 
       7.2347945395158138e-06, 
       6.5705133871353495e-06, 
       6.3588287186123169e-06, 
       5.4733277142645209e-06, 
       5.4816491031014475e-06, 
       5.4852136140988793e-06, 
       5.3009848819773374e-06, 
       6.3823084746243504e-06, 
       5.2855854277772946e-06, 
       5.3234782223854016e-06, 
       7.3910706731593719e-06, 
       6.1971752137126430e-06, 
       6.1685728059759624e-06, 
       6.1963396464175342e-06, 
       6.0991906015090449e-06, 
       5.4282886626571874e-06, 
       7.3076513564598581e-06, 
       6.2509626386402173e-06, 
       6.8161365134358455e-06, 
       7.3284180558334787e-06, 
       6.9616228786228841e-06, 
       5.2882033478535780e-06, 
       5.8265221983726228e-06, 
       6.2459638192776110e-06, 
       7.3586269895958738e-06, 
       5.8240311390494092e-06, 
       5.2633040321757291e-06, 
       5.2623643485908661e-06, 
       6.1715095997981540e-06, 
       6.1711126851757028e-06, 
       7.3739283032986618e-06, 
       7.1182205718370804e-06, 
       5.5409420826400983e-06, 
       5.5409420826400983e-06, 
       5.2530462099487961e-06, 
       5.8151331651033968e-06, 
       6.1996736076808754e-06, 
       5.8136914912325525e-06, 
       5.7945581272735897e-06, 
       5.6720608441733446e-06, 
       5.6567068798802168e-06, 
       5.6567068798802168e-06, 
       5.3119579867566664e-06, 
       6.1446167360689486e-06, 
       6.2413637792225165e-06, 
       6.3736368740059631e-06, 
       5.6866138459997124e-06, 
       5.2597505066372498e-06, 
       5.5351059762663311e-06, 
       5.2449725505910261e-06, 
       5.8019847348223856e-06, 
       5.3881324039450042e-06, 
       6.1743409956739766e-06, 
       7.0960200525735063e-06, 
       6.1878075213491113e-06, 
       5.6639276859468927e-06, 
       6.7453826502003142e-06, 
       5.2801946144824463e-06, 
       5.6419414277686788e-06, 
       5.4953495705117457e-06, 
       5.3470216952694643e-06, 
       5.3316654200498077e-06, 
       5.3316654200498077e-06, 
       6.1592265135701084e-06, 
       6.3212243141321063e-06, 
       5.9038794404820273e-06, 
       7.1796222453561282e-06, 
       5.3586947602444381e-06, 
       6.1303323342424041e-06, 
       5.2142611407450598e-06, 
       5.7984582423616250e-06, 
       6.6732112277201568e-06, 
       6.4845793806532715e-06, 
       5.3803109637429919e-06, 
       6.0775451105302589e-06, 
       5.2869273923983769e-06, 
       6.1960396089228197e-06, 
       5.7942254924912351e-06, 
       5.2961199851837655e-06, 
       5.2099046119477297e-06, 
       5.2099046119477297e-06, 
       5.6195241637639676e-06, 
       6.5366961733991931e-06, 
       5.9939304118486463e-06, 
       5.5883114042347507e-06, 
       5.5604161751531724e-06, 
       5.7123056861289184e-06, 
       7.3033461951278232e-06, 
       5.1922576373620115e-06, 
       5.2103437310014724e-06, 
       5.2853177283297270e-06, 
       7.1841291337644549e-06, 
       6.0994001401383625e-06, 
       7.1315920389691240e-06, 
       5.1952394044418878e-06, 
       5.1839625594298388e-06, 
       5.3984776986229278e-06, 
       5.9828411333733380e-06, 
       5.2542327582255395e-06, 
       6.9195065994403507e-06, 
       5.7406816037181469e-06, 
       5.1815702179236271e-06, 
       6.1717121505326583e-06, 
       7.2375563353334706e-06, 
       6.0959330530099474e-06, 
       5.2654385040045693e-06, 
       5.2742141602359554e-06, 
       5.5099875120466912e-06, 
       6.1380621913181044e-06, 
       6.3587559695713922e-06, 
       6.3500495306671978e-06, 
       6.5504362344820278e-06, 
       7.3085476534325835e-06, 
       6.9369761671272424e-06, 
       5.2083034429496105e-06, 
       6.0943673243299101e-06, 
       5.5452162003177375e-06, 
       7.1812823493272101e-06, 
       5.2070903653600036e-06, 
       5.5450326961784913e-06, 
       6.0298629082148001e-06, 
       6.0759416025091884e-06, 
       5.4376153940847482e-06, 
       6.2216373638310003e-06, 
       5.9527824870020783e-06, 
       7.2761201307871922e-06, 
       7.2536247179587770e-06, 
       5.6946663319159855e-06, 
       5.9161843364260264e-06, 
       5.6036960515758617e-06, 
       6.9060878839117946e-06, 
       7.0621578828587832e-06, 
       5.9150176619157414e-06, 
       5.8707747590809960e-06, 
       5.2936260448345143e-06, 
       5.7895872312718065e-06, 
       5.2293502442081073e-06, 
       7.0978011155268866e-06, 
       5.2140242422292580e-06, 
       5.6370831467498159e-06, 
       6.2434331450144373e-06, 
       5.6761121138607354e-06, 
       5.7938011047819470e-06, 
       6.9798429801371771e-06, 
       5.1376124617099431e-06, 
       5.6177931049612327e-06, 
       5.3853481303807970e-06, 
       5.6489731767423241e-06, 
       7.1787845043765107e-06, 
       5.1355079062054452e-06, 
       5.2680750918229941e-06, 
       5.1952897824955046e-06, 
       5.2177128000369585e-06, 
       6.8740213496231926e-06, 
       5.1350352771996024e-06, 
       6.0330876273482068e-06, 
       5.3674535970162317e-06, 
       6.1665294127595418e-06, 
       7.1646367260284260e-06, 
       5.5463496105284731e-06, 
       5.1156604485734757e-06, 
       6.4097739868846593e-06, 
       7.0001168880692733e-06, 
       5.3309827991589180e-06, 
       5.9370042008787239e-06, 
       7.1483257138584613e-06, 
       5.2241201855979548e-06, 
       7.1381170983540448e-06, 
       5.9854629512188670e-06, 
       6.1984606602516078e-06, 
       5.1697216816634835e-06, 
       6.0421869211088783e-06, 
       5.9467558687254566e-06, 
       5.1963011923969957e-06, 
       5.7955574635328155e-06, 
       5.2335008206552384e-06, 
       5.9222083023801718e-06, 
       5.9222083023801718e-06, 
       5.1345183928977579e-06, 
       5.1779226813815735e-06, 
       5.2809291593338167e-06, 
       5.2500313721013064e-06, 
       6.1213134347757487e-06, 
       7.1994616371103522e-06, 
       5.3870815770524110e-06, 
       6.7989978936838905e-06, 
       5.1062573826758132e-06, 
       5.1062573826758132e-06, 
       5.2093764944275705e-06, 
       5.9983728259356309e-06, 
       5.5057082333435576e-06, 
       5.2956046853318113e-06, 
       5.9999787111417625e-06, 
       5.9947276188692832e-06, 
       5.1797367038177675e-06, 
       5.5646532733676450e-06, 
       5.0970033874482051e-06, 
       5.8449895608308438e-06, 
       5.5128873345893522e-06, 
       5.3354432389580698e-06, 
       5.9794584177036992e-06, 
       5.5269001653082006e-06, 
       5.4301204974799552e-06, 
       6.5113828350671211e-06, 
       6.0579157490319682e-06, 
       7.1818094599419126e-06, 
       5.6448908607957191e-06, 
       5.2886239188818242e-06, 
       6.1822240881396097e-06, 
       6.5761609005233121e-06, 
       6.1295876217032792e-06, 
       7.0609465838038365e-06, 
       5.3295903793182630e-06, 
       6.2210612366522984e-06, 
       6.4318915984687154e-06, 
       7.1416731953154181e-06, 
       7.1367795218627426e-06, 
       7.1367795218627426e-06, 
       5.4167911684879703e-06, 
       5.0812188668055641e-06, 
       5.5722174034112043e-06, 
       5.0809930755755908e-06, 
       5.2396561704941294e-06, 
       6.3116745772156059e-06, 
       5.0886273372111876e-06, 
       5.1370691443946227e-06, 
       5.4453595210934112e-06, 
       5.3515178448995128e-06, 
       6.8853121788173181e-06, 
       5.9588431157775341e-06, 
       5.4551879361301095e-06, 
       6.0230421128834749e-06, 
       5.4190001622071678e-06, 
       6.1885102346695336e-06, 
       5.9500114608948415e-06, 
       6.1723099000514734e-06, 
       7.1326684221226177e-06, 
       7.1103993029403522e-06, 
       5.1194619670739964e-06, 
       5.7030143347136473e-06, 
       7.1129987148367658e-06, 
       6.4073357510512288e-06, 
       5.2481544397776283e-06, 
       5.8553983504745477e-06, 
       6.8160774596724606e-06, 
       7.1142287488917522e-06, 
       6.8750335736659288e-06, 
       5.1592520815993292e-06, 
       5.5857631097333356e-06, 
       5.5747027211627644e-06, 
       5.1541793113280835e-06, 
       5.1674512553663078e-06, 
       7.0837631504613443e-06, 
       5.9171008319952491e-06, 
       7.0848727141699949e-06, 
       5.7461365044403820e-06, 
       5.4731974440827196e-06, 
       5.9480663561064358e-06, 
       5.0905241699708477e-06, 
       5.4591811203725754e-06, 
       5.1214759364607740e-06, 
       6.8891917435359078e-06, 
       5.0947041703477098e-06, 
       5.2406346594873073e-06, 
       7.0923674204867306e-06, 
       6.5089817294301258e-06, 
       5.8096522733005506e-06, 
       6.8506476457098806e-06, 
       6.4531293182565535e-06, 
       5.0547738214144203e-06, 
       5.3905457215428139e-06, 
       5.1566242860057006e-06, 
       5.1221577350799645e-06, 
       6.9806046736001449e-06, 
       5.7716041108821939e-06, 
       6.9851767682472947e-06, 
       6.5648383967170602e-06, 
       7.0106066233526353e-06, 
       6.8060965397364521e-06, 
       5.0842766940022541e-06, 
       5.9537133428881028e-06, 
       6.9528503282128028e-06, 
       6.1506390515912310e-06, 
       6.3961767867015086e-06, 
       6.8486980863331425e-06, 
       5.3921273203882736e-06, 
       5.1444193299806858e-06, 
       6.6849750460592125e-06, 
       7.0068683119154729e-06, 
       5.2577032268142826e-06, 
       6.2249357085535062e-06, 
       5.0833168941234252e-06, 
       5.0085014117636185e-06, 
       5.9013590922268348e-06, 
       6.6198000279741446e-06, 
       5.1614666727166710e-06, 
       5.0106631202157810e-06, 
       5.8445371863336206e-06, 
       5.2330392318547598e-06, 
       5.3486881901950089e-06, 
       5.2915355720685562e-06, 
       6.3139887568711686e-06, 
       7.0022259858289227e-06, 
       5.0318981072577734e-06, 
       5.5261245698180585e-06, 
       5.2440811592908714e-06, 
       5.0269721974645215e-06, 
       6.3632394457536943e-06, 
       5.2397438910679939e-06, 
       5.0826630699598868e-06, 
       5.1208378584291366e-06, 
       5.0435708758945564e-06, 
       6.1047026104769068e-06, 
       5.4953131563619164e-06, 
       5.0064830931092391e-06, 
       6.9703899192539470e-06, 
       5.8104779663789147e-06, 
       5.7509491820541933e-06, 
       5.9919504635408999e-06, 
       6.3819638508089249e-06, 
       5.8826005274842328e-06, 
       6.3032715192769789e-06, 
       5.8832286313925278e-06, 
       6.6032982901123253e-06, 
       5.3371778107890469e-06, 
       5.3068475987420778e-06, 
       5.3100447903422174e-06, 
       6.4535617853097211e-06, 
       5.2555838842759139e-06, 
       6.9874361639068328e-06, 
       6.7201838066916409e-06, 
       6.0803771249479365e-06, 
       6.6615922658869639e-06, 
       5.8938156826297979e-06, 
       6.9714315271608122e-06, 
       5.8035111196522279e-06, 
       6.9083745673522458e-06, 
       5.9291885976173082e-06, 
       5.3725088472169186e-06, 
       5.3313664439602887e-06, 
       5.0465607183655812e-06, 
       6.6726946095327684e-06, 
       5.7871767749112381e-06, 
       5.0314130846051858e-06, 
       6.9268197169265925e-06, 
       5.6534952389373843e-06, 
       6.6557575679690944e-06, 
       5.7290491855400866e-06, 
       5.8828977537553643e-06, 
       6.5172483037598923e-06, 
       6.5686845820461960e-06, 
       5.1086268120768148e-06, 
       5.1086268120768148e-06, 
       6.7095936786334403e-06, 
       5.2694113996491418e-06, 
       5.8870372093967251e-06, 
       6.4988046288068995e-06, 
       5.0630430177468774e-06, 
       5.1995986081133840e-06, 
       6.8690895681881170e-06, 
       5.5937738269508193e-06, 
       5.6658307962802522e-06, 
       5.4965421396530949e-06, 
       5.0326691764202182e-06, 
       5.1869476181368466e-06, 
       5.4543895889618590e-06, 
       5.0823147383892382e-06, 
       6.5588743433599956e-06, 
       5.0701327663356127e-06, 
       5.0701327663356127e-06, 
       6.5258547298671280e-06, 
       5.4158018427215725e-06, 
       5.5151859597019291e-06, 
       6.9224298685875303e-06, 
       5.3315887446736730e-06, 
       5.2779231048842430e-06, 
       5.5393283092395559e-06, 
       5.1682324120205102e-06, 
       6.7004027236315605e-06, 
       5.5934016400940161e-06, 
       5.0884830288265021e-06, 
       5.7174565362622801e-06, 
       5.2598475833986776e-06, 
       5.2442553462784150e-06, 
       5.5421848688388462e-06, 
       6.8116396911470511e-06, 
       5.8826332724209201e-06, 
       5.6662567728503056e-06, 
       5.9902828602964845e-06, 
       5.2466019332563461e-06, 
       5.0570042134846688e-06, 
       5.7111689516064055e-06, 
       5.2101380513567342e-06, 
       5.5602047657187201e-06, 
       6.8742956284347077e-06, 
       5.6540495539382093e-06, 
       5.0460292258312460e-06, 
       5.1549116944486901e-06, 
       5.5775265455652450e-06, 
       5.1640475731820762e-06, 
       5.0197965666856556e-06, 
       5.1737491968416198e-06, 
       5.7018284607207900e-06, 
       5.0050621522429372e-06, 
       5.7449206046869171e-06, 
       5.3824847455501841e-06, 
       6.8041304210485603e-06, 
       5.2615249107631482e-06, 
       5.7208001534270193e-06, 
       5.0212954559275695e-06, 
       5.1607434354432520e-06, 
       5.4611633917527863e-06, 
       5.9602933657748334e-06, 
       5.6528955640162581e-06, 
       6.8550672890222821e-06, 
       5.6488054282400489e-06, 
       5.3912334193558601e-06, 
       5.1163811574024321e-06, 
       6.7983717580557833e-06, 
       5.1066358624911472e-06, 
       5.0836626150728225e-06, 
       5.2332037482620057e-06, 
       5.6305180201971218e-06, 
       5.2045884493554040e-06, 
       5.6833752711610136e-06, 
       5.2190897181656702e-06, 
       6.3370363119653598e-06, 
       5.8718657000322704e-06, 
       5.4737350134082627e-06, 
       5.8965344129845294e-06, 
       6.4288473987285681e-06, 
       5.0768917666474741e-06, 
       5.7189481189440884e-06, 
       5.5049766052956606e-06, 
       6.6361527963698073e-06, 
       6.8345628765601126e-06, 
       5.7737045442855177e-06, 
       6.5305944359090663e-06, 
       5.1493852082100133e-06, 
       5.3961424670948650e-06, 
       6.2129269227920473e-06, 
       6.6022330015227166e-06, 
       5.0760636462681580e-06, 
       5.8278508831585285e-06, 
       5.0218524394220919e-06, 
       6.0800577367124848e-06, 
       5.6672786189137055e-06, 
       6.7925566062889826e-06, 
       5.0242959108969123e-06, 
       5.2477587134107276e-06, 
       5.2641498657831820e-06, 
       5.4199274228467586e-06, 
       5.4199274228467586e-06, 
       5.5966656683064003e-06, 
       5.0689655704583934e-06, 
       5.1101297893866222e-06, 
       5.7199182075604264e-06, 
       5.2176755710980267e-06, 
       5.3206573454895828e-06, 
       5.6922236530038079e-06, 
       5.0141864109769346e-06, 
       6.6514175209917794e-06, 
       5.9996529757580357e-06, 
       6.5176158995257485e-06, 
       6.5317630734928191e-06, 
       6.7655290325922168e-06, 
       5.6761972123955106e-06, 
       5.0286035151825328e-06, 
       6.2658662024399494e-06, 
       5.6147863389075215e-06, 
       5.7173672777150223e-06, 
       5.5904677456826809e-06, 
       5.5831621281208631e-06, 
       5.5831621281208631e-06, 
       6.6378375664960152e-06, 
       5.1557302974680786e-06, 
       6.3964295219645038e-06, 
       6.6262593234978287e-06, 
       6.5795957726478845e-06, 
       4.7992435814737550e-06, 
       5.2107465524985579e-06, 
       5.2520007262578167e-06, 
       5.2520007262578167e-06, 
       5.3025824794307352e-06, 
       5.0954540557249336e-06, 
       6.5281574882995468e-06, 
       6.6795097007978265e-06, 
       6.3784640823086232e-06, 
       5.4976469302318790e-06, 
       5.1452517256750275e-06, 
       5.6074495087505588e-06, 
       5.8963969250732190e-06, 
       5.4542435306721075e-06, 
       5.3913355520480451e-06, 
       6.0594296119022806e-06, 
       5.5451621006613134e-06, 
       5.4873575611822610e-06, 
       6.2507834835932795e-06, 
       6.7228326983438489e-06, 
       5.7812268800423634e-06, 
       5.1534234143823262e-06, 
       6.4625368067707079e-06, 
       5.0027598680037044e-06, 
       6.3045354836100039e-06, 
       6.0560481901183896e-06, 
       6.0357014312552698e-06, 
       5.7792162728220610e-06, 
       5.6826491846723122e-06, 
       6.0302854355404565e-06, 
       5.0210275337924473e-06, 
       6.6915531796553851e-06, 
       5.8450356906509580e-06, 
       5.3600249703567839e-06, 
       5.4185496293013407e-06, 
       5.4895535867999497e-06, 
       6.0222640448217595e-06, 
       5.2745603253994991e-06, 
       6.5324409026307853e-06, 
       5.1630943567168660e-06, 
       5.4936450107345400e-06, 
       5.5807566772513599e-06, 
       6.4179001922208611e-06, 
       6.6421778162269562e-06, 
       5.2990588006790259e-06, 
       5.8965122723314421e-06, 
       5.2612428625586008e-06, 
       6.6555890823717292e-06, 
       5.5029842709465513e-06, 
       6.6377495152204413e-06, 
       5.1935351190436556e-06, 
       6.5870771052481835e-06, 
       5.9855054114687412e-06, 
       5.9855054114687412e-06, 
       5.1598176631595898e-06, 
       5.6201479494211058e-06, 
       5.5064073446163104e-06, 
       5.6143782395154250e-06, 
       6.3054898553332670e-06, 
       6.4180314564261719e-06, 
       5.5874585180665623e-06, 
       5.3644805486260753e-06, 
       5.5279324878914114e-06, 
       5.5719098697993120e-06, 
       6.2405685159652341e-06, 
       6.5865834752201634e-06, 
       5.4625522499349752e-06, 
       6.5246776380086168e-06, 
       6.5545309827368852e-06, 
       6.3485976630824233e-06, 
       6.2999163408555939e-06, 
       5.4902305215171285e-06, 
       6.4717968138826564e-06, 
       6.4717968138826564e-06, 
       5.2730927349920946e-06, 
       5.9187959838327633e-06, 
       5.9936842264385107e-06, 
       6.1215251667772025e-06, 
       5.5381073499036053e-06, 
       5.4828488921237593e-06, 
       5.4828488921237593e-06, 
       5.9863829980264074e-06, 
       5.2180912813171309e-06, 
       6.5241052992798866e-06, 
       5.4803733004894887e-06, 
       5.4462180943788180e-06, 
       5.5151481148291688e-06, 
       5.9663070643777142e-06, 
       5.7995837240048130e-06, 
       6.5639073451497502e-06, 
       5.4895228082230622e-06, 
       5.1754656925323139e-06, 
       6.2175972770242291e-06, 
       5.4715922215058293e-06, 
       6.3368153853075715e-06, 
       6.4284846485669782e-06, 
       5.6751283956075012e-06, 
       6.3948153522664443e-06, 
       5.3198910560943191e-06, 
       5.1664982288260777e-06, 
       4.7521991519815740e-06, 
       5.4893169089131109e-06, 
       6.3420379067258069e-06, 
       5.4494624428744065e-06, 
       6.0510447410791965e-06, 
       6.4091495271231018e-06, 
       6.3459107681675649e-06, 
       5.2095264923620327e-06, 
       5.8773718763879571e-06, 
       5.1778553585803506e-06, 
       6.2898322719490820e-06, 
       5.4270082791666147e-06, 
       5.2037289754794092e-06, 
       5.4168279835365792e-06, 
       5.9054988439086120e-06, 
       5.7428663536124496e-06, 
       5.9120408095561665e-06, 
       5.0218212520073213e-06, 
       5.0340836941962763e-06, 
       5.3072479568843563e-06, 
       5.7825726904972110e-06, 
       5.4268528063671242e-06, 
       5.0250997454352976e-06, 
       5.1176939841153357e-06, 
       6.1685598719457118e-06, 
       5.2896231274812721e-06, 
       5.0879620037536710e-06, 
       5.0034395699456692e-06, 
       5.3718490042720656e-06, 
       5.4186745523148162e-06, 
       5.5263220172550726e-06, 
       5.4626803850014608e-06, 
       5.5229254447647654e-06, 
       5.4001416351328001e-06, 
       6.2356049745909060e-06, 
       6.3388839870347919e-06, 
       4.6664212737616913e-06, 
       5.2536632370275706e-06, 
       5.3737484900741266e-06, 
       5.2169472815965601e-06, 
       5.6109682864543693e-06, 
       5.4101379481269568e-06, 
       5.8314861251021297e-06, 
       6.4737380295128058e-06, 
       5.0482104983694529e-06, 
       6.0939237863137028e-06, 
       6.3943979041996382e-06, 
       5.3316653083492868e-06, 
       5.2310059277683736e-06, 
       5.0587546690044736e-06, 
       5.0583775456638594e-06, 
       6.2341629298008892e-06, 
       5.8702649819192267e-06, 
       6.1534053114077470e-06, 
       6.1534053114077470e-06, 
       5.3871528531629385e-06, 
       5.9986524682632356e-06, 
       5.8531159010470763e-06, 
       5.8531159010470763e-06, 
       5.2804664671475744e-06, 
       5.2931526854921598e-06, 
       5.3767447969360239e-06, 
       6.1263820624324322e-06, 
       5.5746415204192447e-06, 
       6.2151697911937897e-06, 
       5.3537378056026867e-06, 
       6.2149863772245279e-06, 
       5.2725993944539913e-06, 
       5.3248524742817636e-06, 
       5.3391753195553864e-06, 
       6.2035846243055683e-06, 
       5.4763623584831728e-06, 
       6.0714972440823765e-06, 
       5.3746466992960332e-06, 
       5.1804352527809584e-06, 
       5.0870310103445580e-06, 
       5.2831805709472684e-06, 
       6.1291588560095151e-06, 
       6.3037809378175243e-06, 
       5.0783389516416873e-06, 
       5.3310616767475379e-06, 
       5.2770438798567332e-06, 
       5.3097475355513358e-06, 
       5.3097623642754956e-06, 
       5.4105514130946883e-06, 
       6.1267866011622665e-06, 
       5.9282669734497838e-06, 
       5.2420689542674930e-06, 
       5.2446812980520261e-06, 
       6.3170739927257570e-06, 
       5.1777041650908182e-06, 
       5.1776797897796081e-06, 
       5.5055257950663088e-06, 
       6.2242965256978093e-06, 
       5.4289534128697224e-06, 
       6.3518850474802251e-06, 
       5.2425727551385661e-06, 
       5.2354607754403214e-06, 
       6.3550408154357236e-06, 
       6.1845650996092003e-06, 
       6.3616741339881534e-06, 
       5.0196847702697732e-06, 
       5.3614456434472771e-06, 
       5.3812634519404583e-06, 
       5.7308393630989936e-06, 
       6.1201332253082622e-06, 
       6.3107648868426264e-06, 
       6.1677310431917398e-06, 
       5.5444796246049634e-06, 
       5.1796768606720741e-06, 
       5.4747765538091168e-06, 
       5.8455324207581975e-06, 
       5.4277700209869721e-06, 
       5.0024295208455143e-06, 
       5.2643842900094456e-06, 
       6.3303785320649327e-06, 
       6.0114559276627244e-06, 
       6.3256015793503870e-06, 
       6.2808127364632513e-06, 
       5.2762368743937764e-06, 
       6.0972649739483321e-06, 
       5.8300872368097120e-06, 
       5.8300872368097120e-06, 
       5.1598370897904015e-06, 
       5.4626146644626077e-06, 
       5.0996278109875364e-06, 
       5.9463396058431414e-06, 
       5.7536138244090496e-06, 
       6.0858145226710934e-06, 
       5.0954068610092392e-06, 
       6.0350406989189679e-06, 
       5.3592951436384879e-06, 
       5.1155353431921269e-06, 
       5.5119125694427228e-06, 
       5.3365177727165231e-06, 
       5.0581859382161608e-06, 
       5.4369362561508835e-06, 
       6.0177691585081710e-06, 
       5.3897817444824042e-06, 
       5.4429047005213112e-06, 
       6.1548312689348900e-06, 
       6.2559393177394249e-06, 
       5.6238225810619494e-06, 
       5.3330220345457473e-06, 
       5.6200269433882879e-06, 
       5.3054326963884223e-06, 
       5.2155845987621009e-06, 
       6.0325271446213796e-06, 
       6.0325271446213796e-06, 
       6.1782126098662529e-06, 
       6.1782126098662529e-06, 
       5.1757901891024100e-06, 
       5.1324257605761870e-06, 
       5.3936376763456650e-06, 
       5.2861094044231784e-06, 
       5.3498189020532625e-06, 
       6.2560180924793723e-06, 
       5.2565053150963198e-06, 
       5.3484195326053427e-06, 
       5.8455289353692215e-06, 
       6.0588282668225581e-06, 
       5.0379413483211976e-06, 
       6.2057535478895710e-06, 
       6.0057155141293581e-06, 
       5.3310955905386799e-06, 
       5.3291115562465479e-06, 
       5.0127295060414864e-06, 
       5.9749777363796055e-06, 
       5.0081515349133331e-06, 
       5.0146144317898749e-06, 
       6.1883210328814205e-06, 
       6.2071692176588892e-06, 
       5.3805868871736665e-06, 
       5.9836572025951664e-06, 
       5.0333025461794840e-06, 
       6.1523342664345888e-06, 
       6.0427403501075467e-06, 
       6.0530270978632636e-06, 
       5.8799005963263358e-06, 
       6.1883890754386447e-06, 
       5.2780674936391770e-06, 
       5.4437892384562048e-06, 
       5.9337392507855844e-06, 
       5.8382447128135589e-06, 
       5.7209016592709124e-06, 
       5.0113330671737226e-06, 
       5.8087148737957375e-06, 
       5.1595070186797060e-06, 
       5.1399457319934617e-06, 
       6.0178969399976503e-06, 
       5.7444661140035553e-06, 
       5.1486621761844994e-06, 
       5.9558537417361689e-06, 
       5.3524918510568683e-06, 
       5.9528406430842774e-06, 
       5.0656448100715895e-06, 
       5.2044046269480460e-06, 
       5.6186530142220227e-06, 
       5.4562655704896460e-06, 
       6.1440587956218229e-06, 
       5.5986714627720149e-06, 
       5.5898780152126773e-06, 
       6.1184045316402038e-06, 
       5.1226469337313348e-06, 
       5.0023908414153552e-06, 
       5.0857870525654636e-06, 
       5.8383893306633182e-06, 
       5.1000824279336001e-06, 
       5.0914411998711972e-06, 
       6.0107621790383463e-06, 
       6.1133813652809107e-06, 
       5.8268113858044288e-06, 
       5.9841089652204617e-06, 
       5.5043874270889355e-06, 
       6.0098143645202329e-06, 
       6.0812296172472494e-06, 
       5.9215775460626108e-06, 
       5.3607888136203843e-06, 
       5.1473876953804448e-06, 
       5.9585940214251379e-06, 
       5.3136288670390372e-06, 
       5.3136288670390372e-06, 
       5.2884613512650158e-06, 
       5.0869469743453589e-06, 
       5.0716693689100437e-06, 
       5.5989902629521520e-06, 
       5.9592307068080828e-06, 
       5.2461339793188327e-06, 
       6.0459189947077105e-06, 
       5.3743566457657728e-06, 
       5.1868241822810742e-06, 
       5.0737919503516130e-06, 
       5.1148031161402039e-06, 
       5.9580740092558554e-06, 
       5.4047125899594316e-06, 
       5.0752825518101136e-06, 
       5.5675548668729676e-06, 
       5.9489761361053251e-06, 
       5.9639904532365475e-06, 
       6.0187906790448153e-06, 
       5.6717230633106364e-06, 
       5.0503370431109422e-06, 
       6.0166729205228162e-06, 
       6.0610055956872700e-06, 
       5.1668783178540350e-06, 
       5.0602226827531717e-06, 
       5.0030545453146077e-06, 
       5.9527703788844209e-06, 
       5.1546695371713818e-06, 
       5.9863421046551326e-06, 
       5.3621423238508812e-06, 
       5.0368919279212390e-06, 
       5.0052550777560114e-06, 
       5.9301269746991164e-06, 
       5.9301269746991164e-06, 
       6.0302269457810001e-06, 
       5.5134269004199718e-06, 
       5.8700772806571592e-06, 
       5.1336753862608268e-06, 
       5.1952897824953165e-06, 
       5.5635477614355360e-06, 
       5.1477465836711558e-06, 
       5.1850800173310276e-06, 
       5.7362718768636548e-06, 
       5.6947134222493196e-06, 
       5.7449798931401584e-06, 
       5.9955761872355403e-06, 
       5.1587237221590708e-06, 
       5.9877277740763452e-06, 
       5.6446954884715188e-06, 
       5.9007229710252235e-06, 
       5.6647491616665125e-06, 
       5.7264001663513060e-06, 
       5.9410527690328562e-06, 
       5.3217152354915820e-06, 
       5.3217152354915820e-06, 
       5.9527618272326271e-06, 
       5.9667441212585740e-06, 
       5.6021974795037798e-06, 
       5.2635331453849485e-06, 
       5.0374919295276705e-06, 
       5.0809930755746811e-06, 
       5.6366760787676291e-06, 
       5.5083637395121758e-06, 
       5.8341219367879098e-06, 
       5.9406452489022741e-06, 
       5.9110492104708608e-06, 
       5.0378812630258421e-06, 
       5.7002020148408301e-06, 
       5.1386929685965699e-06, 
       5.0971950924986852e-06, 
       5.1761038326235829e-06, 
       5.0706020531530833e-06, 
       5.5765089760325255e-06, 
       5.8397052964406561e-06, 
       5.1208030381570945e-06, 
       5.1017017515123913e-06, 
       5.5412966474801593e-06, 
       5.2927184156121335e-06, 
       5.0158445186118359e-06, 
       5.8128671961918113e-06, 
       5.8398416966148020e-06, 
       5.8818599718788383e-06, 
       5.7351418318056513e-06, 
       5.8767966991916143e-06, 
       5.5834309364316821e-06, 
       5.6496390270471529e-06, 
       5.1111775084856188e-06, 
       5.0905241699695831e-06, 
       5.4485705131333322e-06, 
       5.0141494433966451e-06, 
       5.7741083738719545e-06, 
       5.7580931033362738e-06, 
       5.5570545952195046e-06, 
       5.0139846842892758e-06, 
       5.6488399521117475e-06, 
       5.8332966795243346e-06, 
       5.5781167524448547e-06, 
       5.6826025096072567e-06, 
       5.1675695745309290e-06, 
       5.7683578268900917e-06, 
       5.8033858471278421e-06, 
       5.6047892526645588e-06, 
       5.0080991041142102e-06, 
       5.1711592226692452e-06, 
       5.4197206120298153e-06, 
       5.1681540059604073e-06, 
       5.7958033305428470e-06, 
       5.0850739827986408e-06, 
       5.2305982181848773e-06, 
       5.7222684747191234e-06, 
       5.0988049745223930e-06, 
       5.5230836394820703e-06, 
       5.2328760747914639e-06, 
       5.0557660700306887e-06, 
       5.3121492095264738e-06, 
       5.2687993902927303e-06, 
       5.2687993902927303e-06, 
       5.7706132811397440e-06, 
       5.5123040747650722e-06, 
       5.5769498830767172e-06, 
       5.2061781259568613e-06, 
       5.4354116781069014e-06, 
       5.7639818777898807e-06, 
       5.7188654748385452e-06, 
       5.4828044959411630e-06, 
       4.1764615318466408e-06, 
       5.4316315949652049e-06, 
       5.5107028634243977e-06, 
       5.7394686998754322e-06, 
       5.7359904372353817e-06, 
       5.7402263124701256e-06, 
       5.0914822169357031e-06, 
       5.6655685038791128e-06, 
       5.6798481359192832e-06, 
       5.7303422585022632e-06, 
       5.7255355375710926e-06, 
       5.1137142088643486e-06, 
       5.7150255980480998e-06, 
       5.2082277271798792e-06, 
       5.6365689227533783e-06, 
       5.5536958900480318e-06, 
       5.1958150120846636e-06, 
       5.7054117003563376e-06, 
       5.2484155199182350e-06, 
       5.2839575326039288e-06, 
       5.6431276911212468e-06, 
       4.8818541165884776e-06, 
       5.3660177641301225e-06, 
       5.5986888890180555e-06, 
       5.4016191175648048e-06, 
       5.0350877214284590e-06, 
       5.0883793820649863e-06, 
       5.1328643055958233e-06, 
       5.5122679323975302e-06, 
       5.2756103776186083e-06, 
       5.1016338865315503e-06, 
       5.0889083072643050e-06, 
       5.0255201896258539e-06, 
       5.0114019559471120e-06, 
       5.4163573445536790e-06, 
       5.5827640096120511e-06, 
       5.3924816868528117e-06, 
       4.0107417700803471e-06, 
       5.1198320135288938e-06, 
       5.4356376596428968e-06, 
       5.3785799184178310e-06, 
       5.3785799184178310e-06, 
       5.3666902486106625e-06, 
       5.5489520802024997e-06, 
       5.3430324147748822e-06, 
       5.2004128589288835e-06, 
       5.2344845002324264e-06, 
       5.5916786106987390e-06, 
       5.1042960463452030e-06, 
       5.0058057433437249e-06, 
       5.5461975267482173e-06, 
       5.4509189869747298e-06, 
       5.3139329183145442e-06, 
       5.0959858930645435e-06, 
       5.2369201014007488e-06, 
       5.5170066223313800e-06, 
       5.0599405267161582e-06, 
       5.3981951069656541e-06, 
       5.4835083589949598e-06, 
       5.2373372488400130e-06, 
       5.0098594388383370e-06, 
       5.5340366549082862e-06, 
       5.3581334658876339e-06, 
       5.5445668642847338e-06, 
       5.3390805778782203e-06, 
       5.4967517777583481e-06, 
       5.4570444438223800e-06, 
       5.4237702116104440e-06, 
       5.4321369776985504e-06, 
       5.2798992878887659e-06, 
       5.2752831942193078e-06, 
       5.3459980893226038e-06, 
       5.5036303581537377e-06, 
       5.4763647998242889e-06, 
       5.2914631796811970e-06, 
       5.0569303299710301e-06, 
       5.4644570077575748e-06, 
       5.4574851531646112e-06, 
       5.3506488461072221e-06, 
       5.4255230620318967e-06, 
       5.4635550371603619e-06, 
       5.2026455310589493e-06, 
       5.2741898402235437e-06, 
       5.4031742231514186e-06, 
       5.2288365347478204e-06, 
       5.2073524265684458e-06, 
       5.2014224215640365e-06, 
       5.3689955616577516e-06, 
       5.1154003794623961e-06, 
       5.2433728403881844e-06, 
       5.2882509971059874e-06, 
       5.4289621281983504e-06, 
       4.0188561026472077e-06, 
       5.2014913049325679e-06, 
       5.1689124892622495e-06, 
       5.3257829930331397e-06, 
       5.3688968592956671e-06, 
       5.3069212302261274e-06, 
       5.3069212302261274e-06, 
       5.4149365980804110e-06, 
       5.3724860372220292e-06, 
       5.1993932169724536e-06, 
       5.3289096220349370e-06, 
       4.6979732785818350e-06, 
       5.3128076504315207e-06, 
       5.3703345792560846e-06, 
       5.4007275206599324e-06, 
       5.0953348323435475e-06, 
       5.1994644592379628e-06, 
       5.1890259328273346e-06, 
       5.3117953752267757e-06, 
       5.3686594188764639e-06, 
       5.0041988766673299e-06, 
       5.3644787454305108e-06, 
       5.3374067489281747e-06, 
       5.1629011880157555e-06, 
       5.3404349588708749e-06, 
       5.1736240070327872e-06, 
       5.2176889157484829e-06, 
       5.2176889157484829e-06, 
       5.2016664016103271e-06, 
       5.1692510662168345e-06, 
       5.3350275550941942e-06, 
       5.2940469610276675e-06, 
       5.1294401935810315e-06, 
       5.3295786762989356e-06, 
       5.0274451289247776e-06, 
       5.2953681099277361e-06, 
       5.1682544786000140e-06, 
       5.1480288055990563e-06, 
       5.3174348734126608e-06, 
       5.2867214134870111e-06, 
       5.2951483965307036e-06, 
       5.2707784294424629e-06, 
       5.1925527684970269e-06, 
       5.2810458312463196e-06, 
       5.2843808019004501e-06, 
       5.2651312697631949e-06, 
       5.2732545685315601e-06, 
       5.2756383548186310e-06, 
       4.2553133236938207e-06, 
       5.0132351763416044e-06, 
       4.0653322648608337e-06, 
       5.0540832211679705e-06, 
       5.2325435948903670e-06, 
       5.0581468764195084e-06, 
       5.0290456866366287e-06, 
       5.0403817435733359e-06, 
       5.1817374429881562e-06, 
       5.0695608853696723e-06, 
       5.1826883761894547e-06, 
       5.0308771467482412e-06, 
       5.0866674452058403e-06, 
       5.0101393907263346e-06, 
       5.0377559537885877e-06, 
       5.2048477030060932e-06, 
       5.1144041116000440e-06, 
       5.1606391537171966e-06, 
       5.0782952286673656e-06, 
       5.1563038159900740e-06, 
       5.1092128326741333e-06, 
       3.6445288565889857e-06, 
       5.1044741941917574e-06, 
       4.6817843072271943e-06, 
       5.0919802979617073e-06, 
       5.0812516315489175e-06, 
       5.0217176709760366e-06, 
       5.0669850127083357e-06, 
       5.0623150247998982e-06, 
       5.0623563277233133e-06, 
       5.0415134111404351e-06, 
       5.0294090149971936e-06, 
       5.0138605174322225e-06, 
       3.6061083905788575e-06, 
       5.0087066005281490e-06, 
       4.7157887370096078e-06, 
       4.7897188816553370e-06, 
       4.0190420555389312e-06, 
       3.4280937214479663e-06, 
       3.2683039369825603e-06, 
       4.5439360425481115e-06, 
       4.3906793360816393e-06, 
       4.4577096361213861e-06, 
       3.7606238731965770e-06, 
       4.2553268880521830e-06, 
       4.3692138195555612e-06, 
       3.7645550140875171e-06, 
       3.7959686286191428e-06, 
       3.9406323286473640e-06, 
       4.0848370257426016e-06, 
       3.0724108836804624e-06, 
       4.0058863910256172e-06, 
       2.9453054766729291e-06, 
       3.6740512664573890e-06, 
       3.3596222534086296e-06, 
       3.5457440681216754e-06, 
       2.8769414880236166e-06, 
       3.0827636387375175e-06, 
       2.9248679763402892e-06, 
       3.2812002729520182e-06, 
       3.1952025050945725e-06, 
       3.4352154086302833e-06, 
       2.5863369360047297e-06, 
       2.8472900011959834e-06, 
       3.5270539384196333e-06, 
       3.1155017456187914e-06, 
       3.1469511900523859e-06, 
       3.1131355704337638e-06, 
       3.4632191032818884e-06, 
       2.4894192284453514e-06, 
       2.4894192284453514e-06, 
       2.6036523611461552e-06, 
       3.1718967289293811e-06, 
       2.4549565088739915e-06, 
       3.2447866746631603e-06, 
       3.1602908500623040e-06, 
       3.3460338124861593e-06, 
       2.9454910322133524e-06, 
       2.9939527757406180e-06, 
       2.9808893923106758e-06, 
       3.1173806684887831e-06, 
       3.1468941190235020e-06, 
       2.2173826843207871e-06, 
       2.3574293577752759e-06, 
       2.7053704866996431e-06, 
       2.4138602986550301e-06, 
       2.3778626024340618e-06, 
       2.6475442173597309e-06, 
       2.1115446236870956e-06, 
       1.9668109382580050e-06, 
       1.9809797216841081e-06, 
       2.0359554775028884e-06, 
       2.6138378238767261e-06, 
       2.0841133606305947e-06, 
       2.3131519585527844e-06, 
       1.7456838155087883e-06, 
       2.3652463792342635e-06, 
       1.7553329487995013e-06, 
       1.9877944423922688e-06, 
       2.3095649050887859e-06, 
       2.0773367622062853e-06, 
       2.0435816749251404e-06, 
       1.9592015644998100e-06, 
       1.3912497124621834e-06, 
       1.8775644539483207e-06, 
       1.9237477726393112e-06, 
       1.3601182142183656e-06, 
       1.7133237645418347e-06, 
       1.3696626727948160e-06, 
       1.8955436227967992e-06, 
       1.5277402074724151e-06, 
       1.7713908614740323e-06, 
       1.4484934338608677e-06, 
       1.6327268799973691e-06, 
       1.6092177938411747e-06, 
       1.2992445975798179e-06, 
       1.2791313356930112e-06, 
       1.0304074300189669e-06, 
       1.2924496282158034e-06, 
       1.2107750826153107e-06, 
       1.1662244481932350e-06, 
       1.1333784194240194e-06, 
       9.4877435005246230e-07, 
       9.1465683905706348e-07, 
       8.3270552020066231e-07, 
       6.8584670715570804e-07, 
       7.5071849672608790e-07, 
       5.6213213495874768e-07, 
       6.5168860764867182e-07, 
       6.8313173358539717e-07, 
       6.7822379339035126e-07, 
       4.5142205489306838e-07, 
       5.4018484783610423e-07, 
       3.3812713272444742e-07, 
       3.1968879635580148e-07, 
       3.8275975987512881e-07, 
       3.2556977434010945e-07, 
       3.3377133367819638e-07, 
       2.8887629371463892e-07, 
       2.1808147756675140e-07, 
       1.9865652723404625e-07, 
       2.0431345209440644e-07, 
       1.6725764829710641e-07, 
       1.0998523475471594e-07, 
       9.2489351323682330e-08, 
       5.1395000000000002e-01, 
       3.8245000000000001e-01, 
       3.2654000000000000e-01, 
       2.6395999999999997e-01, 
       1.2302000000000000e-01, 
       7.7539999999999998e-02, 
       6.0679999999999998e-02, 
       4.9700000000000001e-02, 
       4.1939999999999998e-02, 
       3.2219999999999999e-02, 
       2.5290000000000000e-02, 
       2.4899999999999999e-02, 
       1.7639999999999999e-02, 
       1.2568406893972081e-02, 
       1.1784003379098401e-02, 
       1.4489999999999999e-02, 
       1.3690897470438234e-02, 
       1.1860000000000001e-02, 
       1.0659999999999999e-02, 
       9.9299999999999996e-03, 
       6.5799999999999999e-03, 
       6.4753622005591753e-03, 
       5.8700000000000002e-03, 
       5.2279423322279840e-03, 
       5.2275372671141095e-03, 
       5.3600000000000002e-03, 
       4.7600000000000003e-03, 
       3.5343517042719782e-03, 
       3.5161394648211041e-03, 
       3.9399999999999999e-03, 
       3.3100000000000000e-03, 
       3.0999999999999999e-03, 
       2.3260935648086235e-03, 
       2.2062438129107935e-03, 
       2.1868024776725554e-03, 
       2.3500000000000001e-03, 
       2.3500000000000001e-03, 
       2.3300000000000000e-03, 
       2.1400000000000000e-03, 
       2.1299999999999999e-03, 
       1.4878968187292880e-03, 
       1.4872760675947584e-03, 
       1.7300000000000000e-03, 
       1.6500000000000000e-03, 
       1.4599999999999999e-03, 
       1.4000000000000000e-03, 
       1.2699402436660329e-03, 
       1.2688350196431452e-03, 
       1.2600000000000001e-03, 
       1.2500000000000000e-03, 
       1.1400000000000000e-03, 
       1.1199999999999999e-03, 
       8.7825392277994995e-04, 
       8.7166795623049573e-04, 
       1.0600000000000000e-03, 
       6.8723565472967240e-04, 
       7.0684341392724824e-04, 
       7.0440859794383606e-04, 
       8.8999999999999995e-04, 
       6.4461674483570384e-04, 
       8.4000000000000003e-04, 
       8.4000000000000003e-04, 
       8.0999999999999996e-04, 
       5.9271753236239406e-04, 
       7.3999999999999999e-04, 
       5.7902852888044225e-04, 
       5.5679248941917117e-04, 
       6.8999999999999997e-04, 
       5.6117488115408232e-04, 
       5.6057577514191095e-04, 
       4.8193391383684933e-04, 
       6.4000000000000005e-04, 
       6.4000000000000005e-04, 
       6.2000000000000000e-04, 
       6.0999999999999997e-04, 
       5.9999999999999995e-04, 
       4.6537040858705362e-04, 
       5.2592823124983834e-04, 
       5.2592823124983834e-04, 
       4.1531973246123569e-04, 
       4.5000000000003354e-04, 
       5.5000000000000003e-04, 
       5.4000000000000001e-04, 
       4.2908044958095469e-04, 
       5.2999999999999998e-04, 
       5.2999999999999998e-04, 
       5.2999999999999998e-04, 
       4.9768189447461128e-04, 
       3.7454492888425476e-04, 
       3.7071638480686291e-04, 
       3.9708750105345888e-04, 
       5.1000000000000004e-04, 
       4.6999999999999999e-04, 
       3.3080690776920719e-04, 
       4.2999999999999999e-04, 
       3.0532549555169122e-04, 
       3.9497173643677320e-04, 
       3.9113420847551659e-04, 
       3.0772601464211665e-04, 
       3.9675208999735158e-04, 
       4.0000000000000002e-04, 
       4.0000000000000002e-04, 
       3.9365143818023950e-04, 
       2.9994785474000198e-04, 
       2.8000000000000935e-04, 
       3.0900123578542367e-04, 
       3.8999999999999999e-04, 
       3.0270902761540380e-04, 
       3.8000000000000002e-04, 
       2.5991094753513762e-04, 
       2.6053178823735282e-04, 
       2.5922019768501338e-04, 
       2.5186415461473494e-04, 
       2.4454534469128033e-04, 
       2.4988031352489069e-04, 
       3.4000000000000002e-04, 
       2.4166799235374786e-04, 
       2.5101543941009126e-04, 
       2.4501256647819770e-04, 
       2.3000000000005058e-04, 
       2.2841802372000470e-04, 
       2.5294714920320313e-04, 
       2.2829490021872978e-04, 
       3.2000000000000003e-04, 
       2.2790008645504975e-04, 
       2.3263758280866889e-04, 
       2.8511287761438266e-04, 
       2.8507847399412414e-04, 
       2.5019068052258085e-04, 
       2.9325496945684521e-04, 
       2.9046198675541311e-04, 
       2.5125325925129052e-04, 
       2.4709836201773508e-04, 
       2.4695737727719485e-04, 
       2.5110045651422196e-04, 
       2.3767051727543534e-04, 
       2.3485512641444902e-04, 
       2.4522738186651729e-04, 
       2.9000000000000000e-04, 
       2.9000000000000000e-04, 
       2.7999999999999998e-04, 
       1.9354233554890547e-04, 
       2.7000000000000000e-04, 
       1.9797077374057048e-04, 
       1.8923655014563631e-04, 
       2.5999999999999998e-04, 
       1.9140286245590093e-04, 
       1.9049823101475376e-04, 
       1.8137838533852755e-04, 
       1.8390377530360578e-04, 
       1.8281122273027361e-04, 
       1.9430328733263945e-04, 
       1.9000000000004777e-04, 
       1.7667572091763153e-04, 
       1.7889787186768205e-04, 
       1.7056857781076406e-04, 
       2.3000000000000001e-04, 
       1.6999999999998966e-04, 
       1.6447454584617039e-04, 
       1.5977613888343729e-04, 
       2.2000000000000001e-04, 
       1.4964434856036944e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.5017598457786838e-04, 
       1.4897594970445617e-04, 
       1.3723239055300529e-04, 
       1.3086431296222998e-04, 
       1.3557168198828488e-04, 
       1.3270684530295230e-04, 
       1.4359993940508937e-04, 
       1.2442503315285798e-04, 
       1.2367343471197728e-04, 
       1.2686173899535859e-04, 
       1.2884511326030930e-04, 
       1.7000000000000001e-04, 
       1.3513548092575971e-04, 
       1.3063050647139311e-04, 
       1.6000000000000001e-04, 
       1.6000000000000001e-04, 
       1.1642313303651290e-04, 
       1.5000000000004311e-04, 
       1.2749412219105222e-04, 
       1.1187488756414990e-04, 
       1.0755307430968702e-04, 
       1.4999999999999999e-04, 
       1.4999999999999999e-04, 
       1.4999999999999999e-04, 
       1.1299769972415393e-04, 
       1.1839141046751093e-04, 
       1.0999999999999572e-04, 
       1.0392399590770566e-04, 
       1.2307367730301260e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.0905085975014206e-04, 
       1.0001913494704076e-04, 
       1.0559937521497350e-04, 
       1.0000000000000570e-04, 
       9.9405048484051031e-05, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.0149101534921004e-04, 
       9.0000000000000792e-05, 
       9.0243828537369297e-05, 
       9.5165385561884030e-05, 
       8.9999999999997404e-05, 
       8.9999999999997404e-05, 
       1.1334847871294843e-04, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       1.1000000000003241e-04, 
       8.7710285957550918e-05, 
       8.4460857744159920e-05, 
       8.4761128208903357e-05, 
       9.2252495680553986e-05, 
       8.2342999936493421e-05, 
       1.1550314987776648e-04, 
       1.1546447826408749e-04, 
       8.4547368504826441e-05, 
       8.3152647071812804e-05, 
       9.5960734913279127e-05, 
       8.0000000000001186e-05, 
       1.1033330856068844e-04, 
       7.8886000801812084e-05, 
       1.1000000000000000e-04, 
       8.3043091216989743e-05, 
       7.9816681021397759e-05, 
       7.9622816564665214e-05, 
       7.5891301312984365e-05, 
       8.1459149995481456e-05, 
       7.3365123912174588e-05, 
       8.1679035965688031e-05, 
       7.4075634107649023e-05, 
       7.9168261898614610e-05, 
       7.9088096306455838e-05, 
       8.1194627099614026e-05, 
       1.0000000000000000e-04, 
       1.0000000000000000e-04, 
       8.1146666386748705e-05, 
       8.1040148542723745e-05, 
       7.0116217323202353e-05, 
       6.8465088072203978e-05, 
       6.8241291959534998e-05, 
       6.9210065860619323e-05, 
       7.2614126026657202e-05, 
       6.6967527743241344e-05, 
       7.8203016676591175e-05, 
       6.7398905880599206e-05, 
       6.4303073408889709e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       6.4979619015389175e-05, 
       6.3760429821466111e-05, 
       6.3069388092895171e-05, 
       6.2967914103979937e-05, 
       6.5929312836591415e-05, 
       8.2154476603235867e-05, 
       7.0000000000004344e-05, 
       6.2131835531902989e-05, 
       6.5187197546849373e-05, 
       6.3766869393550138e-05, 
       6.4914606257073944e-05, 
       6.0953568224508716e-05, 
       6.1452486986326326e-05, 
       6.3662204929734099e-05, 
       6.2125489845577463e-05, 
       6.0311746889320389e-05, 
       6.1890712814082030e-05, 
       6.8926078079690930e-05, 
       5.9120862622828267e-05, 
       5.8110994913895685e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       6.1707893043326067e-05, 
       6.1692059474636917e-05, 
       5.8361048514878264e-05, 
       6.0000000000000889e-05, 
       6.1656465940055846e-05, 
       6.0574034811264697e-05, 
       5.4683645659718058e-05, 
       7.1003664702539888e-05, 
       5.4780535305623308e-05, 
       5.1782835873059048e-05, 
       6.8526730205726194e-05, 
       5.5359418872701745e-05, 
       5.1036428582262125e-05, 
       5.0531453149911530e-05, 
       5.2575609598701979e-05, 
       4.9980419730081616e-05, 
       5.5594112775369728e-05, 
       5.9328689179248411e-05, 
       5.0000000000000958e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       5.6519647400985846e-05, 
       4.9885129577079217e-05, 
       5.0000000000000484e-05, 
       5.0000000000000484e-05, 
       4.9599812436233439e-05, 
       4.8632420095004347e-05, 
       4.9398645239422328e-05, 
       4.8096626752022035e-05, 
       4.8959874911656805e-05, 
       5.0000000000000829e-05, 
       4.8485255662400824e-05, 
       4.6513486503327878e-05, 
       4.6157010319438953e-05, 
       4.5392787589185880e-05, 
       4.6177513316823789e-05, 
       4.5555108923361914e-05, 
       4.8899035332652549e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       4.7418265235260363e-05, 
       4.3472723661292423e-05, 
       4.3400631161124156e-05, 
       4.7048147378972200e-05, 
       4.2184021451112264e-05, 
       3.9999999999998167e-05, 
       4.0000000000000057e-05, 
       4.2017656494641927e-05, 
       4.0000000000000844e-05, 
       4.1501398099675757e-05, 
       4.1978138299121989e-05, 
       4.0753305337013683e-05, 
       4.6191892594424316e-05, 
       3.9273783856895769e-05, 
       3.9802849971982503e-05, 
       5.2064486505768060e-05, 
       3.8800582781518947e-05, 
       3.8584568300673510e-05, 
       3.8284407720050025e-05, 
       4.6788648784923036e-05, 
       3.8172847651184046e-05, 
       4.0647446122550230e-05, 
       4.0805637271106639e-05, 
       3.8578230080372285e-05, 
       3.9378349820930966e-05, 
       3.7042812135612926e-05, 
       5.0482708280868738e-05, 
       3.7434534127795312e-05, 
       3.9999999999994840e-05, 
       3.7107575921643962e-05, 
       3.8491276509774090e-05, 
       3.6599474590053623e-05, 
       3.9999999999998025e-05, 
       3.9896459567427549e-05, 
       4.9561464705726884e-05, 
       3.9632905834956871e-05, 
       4.8957470931800587e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       3.5307894505969600e-05, 
       3.6706748173119010e-05, 
       4.7036822723610919e-05, 
       3.4896201836217627e-05, 
       3.4896201836217627e-05, 
       4.0000000000001630e-05, 
       3.6452796315661451e-05, 
       3.8083150687420347e-05, 
       3.4928978498854827e-05, 
       3.3867620534448810e-05, 
       3.7520259983707021e-05, 
       3.5668559013682856e-05, 
       4.6506152394088104e-05, 
       4.4604463882644200e-05, 
       3.2699023886874744e-05, 
       3.4007252950701781e-05, 
       3.3034564518192191e-05, 
       3.2341141198643524e-05, 
       3.9407048502644538e-05, 
       3.2551398735100988e-05, 
       4.4138556343625333e-05, 
       3.3420174369559843e-05, 
       3.1538258279122524e-05, 
       3.1496886654673129e-05, 
       3.1493970367684139e-05, 
       3.6761965623058505e-05, 
       3.1659196010350884e-05, 
       3.2568230167760899e-05, 
       3.4521648219744358e-05, 
       3.0596838357052694e-05, 
       3.9916210866537443e-05, 
       2.9999999999995623e-05, 
       3.0490353142401590e-05, 
       3.0241412231445859e-05, 
       3.0350932444682084e-05, 
       3.2857246986154037e-05, 
       3.0000000000001004e-05, 
       3.2261315146808669e-05, 
       3.7959026900473236e-05, 
       3.1667618604007313e-05, 
       3.7933411674007313e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.8589603978043228e-05, 
       3.1524492131290423e-05, 
       2.8362946479193587e-05, 
       3.8626525785678050e-05, 
       3.8093536670983630e-05, 
       3.1222477793243399e-05, 
       3.1202989880283919e-05, 
       2.9999999999998578e-05, 
       2.9999999999998578e-05, 
       2.7681872941054794e-05, 
       2.7230973358653330e-05, 
       2.8182660385394178e-05, 
       2.8352723334744815e-05, 
       3.1260586080407508e-05, 
       3.0402734122248371e-05, 
       2.9999999999998259e-05, 
       2.7287745829678278e-05, 
       2.7644668719605025e-05, 
       2.9904463035609902e-05, 
       2.9999999999995478e-05, 
       2.8031009929185742e-05, 
       2.7711795395081385e-05, 
       3.0000000000003379e-05, 
       3.0000000000003379e-05, 
       3.0000000000003379e-05, 
       3.0000000000003379e-05, 
       2.8200418782756053e-05, 
       2.9201060919921262e-05, 
       2.7291010521720810e-05, 
       2.6700850744264351e-05, 
       2.8424040236260078e-05, 
       3.0000000000002017e-05, 
       3.0000000000001793e-05, 
       2.5387971797372546e-05, 
       2.6055731549064188e-05, 
       2.6869479518744932e-05, 
       2.5348528635847772e-05, 
       3.0137498916115293e-05, 
       3.0113508740696198e-05, 
       2.5751594850855288e-05, 
       2.5641570033012689e-05, 
       2.9293257112649028e-05, 
       3.3656473252729510e-05, 
       3.3103433646800270e-05, 
       2.5471544590581578e-05, 
       3.3535038549939455e-05, 
       3.3064468029852999e-05, 
       2.4319205301166105e-05, 
       2.5673926407981536e-05, 
       3.3412806132221019e-05, 
       2.4600570094929900e-05, 
       3.2758111197503761e-05, 
       3.2804861329707739e-05, 
       2.5119103852087525e-05, 
       2.5700355298132588e-05, 
       2.5700355298132588e-05, 
       2.4455687401803118e-05, 
       2.5280258792097136e-05, 
       3.2010309558555405e-05, 
       2.3371881420904367e-05, 
       2.3191029991928993e-05, 
       2.3054601462705134e-05, 
       2.5667700509819839e-05, 
       2.4255482066990104e-05, 
       2.4404707724799157e-05, 
       3.1077293641236036e-05, 
       2.4628521461384366e-05, 
       2.2842897000823129e-05, 
       2.6664807061112853e-05, 
       2.5782177582851115e-05, 
       2.3651192529226165e-05, 
       2.6103195788865071e-05, 
       2.9784774926826150e-05, 
       2.2025540017736664e-05, 
       2.4910720027409912e-05, 
       2.2513791214272149e-05, 
       2.4312406084897530e-05, 
       2.2717852425910860e-05, 
       2.1785674730739426e-05, 
       2.2370308360634774e-05, 
       2.2370308360634774e-05, 
       2.1652804158264882e-05, 
       2.3909890694788322e-05, 
       2.4152512197972920e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.3788430104401198e-05, 
       2.0978090127768883e-05, 
       2.3719965812685840e-05, 
       2.1494168635770795e-05, 
       2.6099462241861174e-05, 
       2.2217013440853598e-05, 
       2.7253304578086529e-05, 
       2.0481436127107325e-05, 
       2.4839153453821431e-05, 
       2.6987389394127891e-05, 
       2.6889520563811129e-05, 
       2.0574098169193254e-05, 
       2.5293651134129159e-05, 
       2.0793289668495495e-05, 
       2.6941697375168136e-05, 
       2.0373438942022411e-05, 
       2.2640866923612534e-05, 
       1.9823607667748466e-05, 
       2.0000000000005372e-05, 
       2.1795645248532292e-05, 
       2.4143358489140004e-05, 
       2.3112689627928407e-05, 
       1.9999999999997288e-05, 
       2.0099751201744037e-05, 
       2.3319758926953415e-05, 
       1.8854812707935437e-05, 
       2.1647722281125278e-05, 
       1.9297068739862420e-05, 
       2.1085360621631260e-05, 
       2.4495860202666620e-05, 
       2.1021480119838357e-05, 
       1.8288369937107696e-05, 
       1.8592980633971474e-05, 
       2.0454394181672909e-05, 
       2.4802079123496686e-05, 
       1.8290334782186212e-05, 
       2.4978113595954295e-05, 
       1.8544196614098385e-05, 
       1.9799893344402930e-05, 
       1.7936792018330110e-05, 
       2.3165049274500912e-05, 
       1.9566354639868191e-05, 
       1.9753377571509464e-05, 
       2.3266934548824592e-05, 
       1.7743990459484160e-05, 
       1.8002785822586596e-05, 
       2.3216764265444699e-05, 
       1.7710368627631337e-05, 
       2.0000000000000537e-05, 
       1.8865230683183364e-05, 
       1.8001063923225192e-05, 
       1.7842555882918417e-05, 
       1.7825460844297778e-05, 
       1.7712757114896970e-05, 
       2.1451828402676773e-05, 
       1.7528642660685291e-05, 
       2.1414732482260364e-05, 
       1.7452575761645021e-05, 
       1.9652980405081511e-05, 
       2.1376184080307644e-05, 
       1.7269222747465106e-05, 
       2.1323411353491352e-05, 
       1.7094557185043193e-05, 
       1.7039779640373359e-05, 
       2.0333544285585613e-05, 
       1.7484167385865677e-05, 
       1.7190010149894823e-05, 
       1.9999999999996559e-05, 
       1.9999999999996559e-05, 
       1.7011808470777423e-05, 
       1.9002429201614061e-05, 
       1.6830951520407571e-05, 
       2.0411089720788116e-05, 
       1.6670920597545245e-05, 
       1.6571842208823096e-05, 
       1.8685397456088012e-05, 
       1.9403934891001417e-05, 
       1.8355071144565587e-05, 
       1.6642354989153762e-05, 
       1.6808152851161150e-05, 
       1.8276185070206446e-05, 
       1.8115458148729392e-05, 
       2.1087620622527917e-05, 
       2.1087620622527917e-05, 
       1.6279188799200311e-05, 
       1.6750880293239179e-05, 
       1.6216072144592670e-05, 
       1.6489329929888867e-05, 
       1.7991949977368835e-05, 
       1.6335716544553203e-05, 
       1.6216155011468388e-05, 
       1.7573744286687250e-05, 
       1.7547748994287195e-05, 
       1.9999999999997061e-05, 
       2.1391474193466786e-05, 
       1.7291576990228942e-05, 
       1.8431090852052867e-05, 
       2.0000000000004813e-05, 
       1.5304570455558684e-05, 
       1.5358621616874319e-05, 
       1.5660277408606414e-05, 
       1.9999999999996698e-05, 
       1.9999999999996698e-05, 
       1.8190120812147348e-05, 
       1.6165886412933463e-05, 
       1.5606222406256737e-05, 
       1.6685554873310344e-05, 
       1.5899430431353373e-05, 
       1.4875289298916494e-05, 
       1.4850802965767678e-05, 
       2.0082540826426291e-05, 
       1.4882863623829293e-05, 
       1.6501039814091209e-05, 
       2.0385008116287263e-05, 
       1.9848939511585388e-05, 
       1.5650635876016359e-05, 
       1.5093514612430268e-05, 
       1.6640607382535858e-05, 
       1.6447002477705602e-05, 
       1.4917804831480329e-05, 
       1.5978200045072543e-05, 
       1.4585863928786180e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.4570462038592828e-05, 
       1.4548663076605211e-05, 
       1.5685414700036578e-05, 
       1.4474125910208743e-05, 
       1.9365517980274570e-05, 
       1.8821586937772193e-05, 
       1.8807182851767827e-05, 
       1.4048284245040205e-05, 
       1.7385377344406373e-05, 
       1.7379015713368040e-05, 
       1.4088321779724864e-05, 
       1.3776952819239927e-05, 
       1.3990308702390204e-05, 
       1.4320532953379261e-05, 
       1.3501088620543262e-05, 
       1.7719856150355205e-05, 
       1.7709417175342579e-05, 
       1.7634153277693866e-05, 
       1.6390904418187451e-05, 
       1.3919229500910895e-05, 
       1.4338173253904613e-05, 
       1.3913927374749813e-05, 
       1.8469250275329990e-05, 
       1.3050630729572519e-05, 
       1.3613952334564977e-05, 
       1.3054152635182538e-05, 
       1.3405726778128933e-05, 
       1.3466072103855028e-05, 
       1.4373974059060835e-05, 
       1.2793964401976620e-05, 
       1.4159163904472759e-05, 
       1.3072426515244650e-05, 
       1.4595223222946117e-05, 
       1.3004172500496937e-05, 
       1.6750880293241520e-05, 
       1.5571871565364185e-05, 
       1.5571871565364185e-05, 
       1.3158900795072862e-05, 
       1.4328526581230689e-05, 
       1.2842855960269581e-05, 
       1.2576931002492702e-05, 
       1.2484543198423890e-05, 
       1.3399001468278735e-05, 
       1.4883740383401062e-05, 
       1.3155149403464286e-05, 
       1.2912947467300540e-05, 
       1.5592449403370351e-05, 
       1.2682396500829246e-05, 
       1.2236441354311879e-05, 
       1.4355160774092448e-05, 
       1.5841811442093344e-05, 
       1.2013877824227743e-05, 
       1.3620858279107597e-05, 
       1.2312265279682979e-05, 
       1.2216808093369565e-05, 
       1.2206843949530528e-05, 
       1.5770349181922546e-05, 
       1.2502644236347054e-05, 
       1.1844259935222618e-05, 
       1.1997676697147328e-05, 
       1.1837991875031265e-05, 
       1.2408266201573699e-05, 
       1.4022890600076445e-05, 
       1.4937707528793601e-05, 
       1.3039889740767648e-05, 
       1.2989125549696863e-05, 
       1.2204545186161065e-05, 
       1.1754400389805281e-05, 
       1.2099655400016914e-05, 
       1.1740206034148839e-05, 
       1.1738954996217235e-05, 
       1.1531923147655575e-05, 
       1.1584773043904730e-05, 
       1.2690199900280389e-05, 
       1.4235364985916107e-05, 
       1.1874281489299238e-05, 
       1.1329964783421689e-05, 
       1.2899869687755386e-05, 
       1.4391493753313829e-05, 
       1.1858175358032654e-05, 
       1.1759934567113864e-05, 
       1.1638428315735223e-05, 
       1.3401456710306259e-05, 
       1.4361919276825344e-05, 
       1.2170673549690623e-05, 
       1.2445876171938376e-05, 
       1.5181728277297751e-05, 
       1.1014838876694021e-05, 
       1.0903601780886999e-05, 
       1.3589561225417159e-05, 
       1.0982902513960771e-05, 
       1.2219076865613779e-05, 
       1.1462832579698723e-05, 
       1.0717430646007050e-05, 
       1.1190562103865615e-05, 
       1.1949004858055973e-05, 
       1.0717220743235374e-05, 
       1.2602434554336594e-05, 
       1.1193091379303580e-05, 
       1.0665946665475236e-05, 
       1.2189423617636098e-05, 
       1.1851867428322875e-05, 
       1.1492378436646006e-05, 
       1.2402437460254247e-05, 
       1.0501897353023273e-05, 
       1.0769361352568703e-05, 
       1.0660715299179784e-05, 
       1.0374449991422352e-05, 
       1.0369058316675662e-05, 
       1.3169266031338557e-05, 
       1.1319077525685141e-05, 
       1.0286753043817520e-05, 
       1.3634410117562983e-05, 
       1.1263356807985336e-05, 
       1.1689491022625012e-05, 
       1.0140840871030608e-05, 
       1.0370314950386196e-05, 
       1.0477076928911442e-05, 
       1.1373838594448410e-05, 
       9.8171802890085565e-06, 
       1.0929218512156398e-05, 
       9.9058879007358836e-06, 
       1.0907312010853471e-05, 
       9.9020587561811231e-06, 
       1.2329883850171766e-05, 
       1.2287846742225126e-05, 
       9.7601391118785833e-06, 
       1.0810989671155187e-05, 
       1.2826438668611414e-05, 
       1.0518105530445644e-05, 
       1.0467520347314968e-05, 
       9.4041025882412688e-06, 
       9.7838091661005748e-06, 
       9.6248840527469163e-06, 
       1.0375398206522494e-05, 
       9.5872968783310446e-06, 
       1.2608613236357157e-05, 
       9.3263932917629976e-06, 
       9.9310449206062467e-06, 
       1.2345438360638910e-05, 
       9.3433147664402195e-06, 
       9.6018533209568520e-06, 
       9.6180649223804651e-06, 
       1.0196186154948797e-05, 
       9.0548310137078983e-06, 
       1.0388469667100754e-05, 
       9.5980807834344346e-06, 
       9.3708193721244305e-06, 
       9.9371776464561289e-06, 
       9.5694966147049163e-06, 
       9.0966155162714480e-06, 
       9.1421652706040462e-06, 
       9.0602215610569848e-06, 
       9.7922551430125546e-06, 
       8.9260626300029436e-06, 
       9.5282931092297037e-06, 
       8.9590225639948553e-06, 
       9.3565070056060589e-06, 
       9.3186121677237623e-06, 
       8.9080038892701045e-06, 
       8.7543570600405882e-06, 
       1.0328187785966119e-05, 
       9.0325162377397451e-06, 
       8.7032329907912545e-06, 
       1.0833771441611283e-05, 
       1.0830820903337420e-05, 
       8.9132331306980121e-06, 
       8.8856451837987447e-06, 
       9.0957546797665316e-06, 
       9.7288650217690265e-06, 
       8.8403326850143654e-06, 
       1.1824711001772112e-05, 
       1.0165185739377282e-05, 
       8.5459251157449557e-06, 
       8.6725391624246413e-06, 
       8.4760960664675638e-06, 
       9.3669130719187560e-06, 
       1.1382089860841990e-05, 
       8.4361528444329976e-06, 
       9.4426513678278533e-06, 
       9.9735542821671082e-06, 
       8.6368010900088815e-06, 
       8.5837572488841785e-06, 
       8.3110585021256145e-06, 
       9.2525517275076938e-06, 
       8.8210997414247014e-06, 
       8.5344122617600387e-06, 
       8.5242264837865845e-06, 
       8.1833832546017160e-06, 
       8.1832145112202790e-06, 
       8.2112435963161936e-06, 
       8.2070696798167070e-06, 
       8.2349211126785980e-06, 
       8.1597053828882363e-06, 
       8.0185093665547183e-06, 
       9.1682244861165609e-06, 
       9.6635010610130547e-06, 
       8.9356673419142491e-06, 
       1.0868492926657315e-05, 
       9.3194228066974546e-06, 
       8.3189186031184482e-06, 
       1.0860785713890774e-05, 
       8.9192027120887603e-06, 
       8.7584731571415443e-06, 
       8.2663283767761240e-06, 
       7.9813698078336869e-06, 
       8.2293555424581714e-06, 
       7.8113145882220023e-06, 
       7.9973740233809380e-06, 
       7.6903816874505557e-06, 
       1.0539601170709950e-05, 
       8.0144730002726315e-06, 
       7.6995842152124265e-06, 
       8.4526566769240888e-06, 
       8.0062453251905186e-06, 
       7.8788314484962559e-06, 
       9.1080211200565109e-06, 
       8.3958281170095143e-06, 
       7.6104355949299831e-06, 
       8.6031187346043701e-06, 
       8.0517117893480499e-06, 
       9.9593032310541382e-06, 
       7.9321199367800675e-06, 
       9.9208134491744625e-06, 
       8.1791849230346208e-06, 
       7.6235860473217769e-06, 
       7.8072088469386917e-06, 
       9.0880906269530331e-06, 
       7.4961960723108302e-06, 
       8.3115242471540807e-06, 
       7.7296028991198358e-06, 
       8.2942225209006913e-06, 
       7.4391323999931147e-06, 
       8.3608347417200307e-06, 
       7.8020739612870757e-06, 
       7.8669021825709058e-06, 
       8.2369907808965320e-06, 
       7.5565087284292423e-06, 
       7.7018377912660353e-06, 
       7.6980867801185913e-06, 
       7.4395691839566759e-06, 
       7.4759465735550981e-06, 
       7.2540406206877551e-06, 
       8.0483136750243228e-06, 
       8.0483136750243228e-06, 
       7.5956879749604106e-06, 
       7.4097193479178969e-06, 
       7.0916411137885525e-06, 
       7.0693838551818292e-06, 
       8.4436596384628879e-06, 
       8.6389828507415969e-06, 
       9.6427639054843386e-06, 
       8.5553760380388220e-06, 
       7.1325489368607324e-06, 
       7.2553697159599648e-06, 
       7.0754050229257628e-06, 
       7.0695973231508207e-06, 
       9.6228969101457598e-06, 
       7.6531934404028063e-06, 
       7.3958434487168453e-06, 
       8.5692248872960298e-06, 
       7.3383706582692006e-06, 
       7.1854714019240426e-06, 
       7.5695005453400734e-06, 
       7.3534049644413207e-06, 
       8.9224062552253326e-06, 
       8.0440364611396338e-06, 
       6.9351579504160027e-06, 
       7.1731706648071492e-06, 
       7.2414232907511160e-06, 
       7.5914517616853656e-06, 
       7.1144438615740840e-06, 
       6.7881859574839110e-06, 
       8.2432295953023061e-06, 
       8.9369666098449463e-06, 
       8.5260400614014442e-06, 
       8.5306423584069898e-06, 
       8.1161924748027263e-06, 
       7.1830051066901237e-06, 
       7.8127449411731031e-06, 
       7.0870941545758944e-06, 
       6.9458209113742427e-06, 
       6.7636892687736263e-06, 
       6.7636892687736263e-06, 
       7.1394379159594234e-06, 
       6.8052713240863207e-06, 
       6.7966510516058743e-06, 
       6.7353906432321844e-06, 
       7.9178634525845773e-06, 
       6.6760335629827680e-06, 
       8.1368790951563903e-06, 
       9.2621059769026164e-06, 
       6.6455900426978548e-06, 
       7.5175700428122142e-06, 
       7.1621782109095060e-06, 
       7.1621782109095060e-06, 
       6.7472343586452060e-06, 
       7.5950482332312706e-06, 
       6.5338308507628183e-06, 
       6.5597495878337028e-06, 
       6.5213170430377275e-06, 
       6.9169630192050877e-06, 
       6.4902753335487729e-06, 
       8.4729920194510255e-06, 
       8.4729920194510255e-06, 
       7.7534866824578384e-06, 
       6.7395355866726167e-06, 
       6.8409174894260553e-06, 
       6.4999829569450186e-06, 
       6.4999829569450186e-06, 
       7.2034089576335261e-06, 
       7.1510840782560615e-06, 
       7.1241705664622865e-06, 
       7.1241705664622865e-06, 
       6.3222530748839867e-06, 
       6.4799563015104936e-06, 
       6.8080045890944089e-06, 
       8.7602520738025704e-06, 
       6.4578860935770902e-06, 
       6.8920567180715747e-06, 
       6.2384683098934866e-06, 
       6.2822798558249214e-06, 
       6.5780247373297760e-06, 
       6.7382527247518505e-06, 
       6.3827701255542561e-06, 
       6.1543963202956087e-06, 
       7.5183276575533373e-06, 
       8.3880151646012032e-06, 
       6.5045560304318092e-06, 
       7.1713681708993686e-06, 
       6.3437716282096654e-06, 
       8.0174566890029509e-06, 
       6.5054368741342887e-06, 
       7.5005437031529041e-06, 
       8.3163783593694593e-06, 
       6.3094168705288066e-06, 
       6.1356002839117095e-06, 
       6.3946997774153607e-06, 
       7.2399128418608813e-06, 
       6.3760734296361674e-06, 
       6.1230831531869733e-06, 
       8.4001409797736138e-06, 
       6.7835255895705884e-06, 
       6.9064471764360649e-06, 
       6.5596518342942182e-06, 
       8.2755937433931197e-06, 
       6.1492135127887412e-06, 
       6.1520176577604837e-06, 
       6.0226668186291452e-06, 
       6.7523292826438940e-06, 
       6.0666275178128038e-06, 
       6.3562982850042437e-06, 
       5.9967507767496810e-06, 
       7.0473615116974803e-06, 
       6.6714029894164123e-06, 
       5.8161734200203890e-06, 
       5.8100838021554864e-06, 
       5.8774046187868913e-06, 
       5.8691420639189546e-06, 
       6.1545441254885254e-06, 
       7.0068177970510037e-06, 
       6.4876817275913891e-06, 
       5.7958446823588512e-06, 
       5.7453321898965552e-06, 
       5.9448261711328567e-06, 
       5.9597742801824941e-06, 
       5.8838036299677197e-06, 
       6.4002514387297686e-06, 
       6.8963023034895994e-06, 
       5.9589844469564854e-06, 
       5.9455990434343156e-06, 
       5.9718716896313373e-06, 
       6.1740235380997552e-06, 
       5.5807457010752515e-06, 
       5.8022622185183938e-06, 
       6.1490838287004136e-06, 
       6.6931011957064057e-06, 
       6.4026776375959264e-06, 
       5.6086819124128343e-06, 
       6.2023096500389016e-06, 
       5.8924807938064745e-06, 
       6.0323439365576814e-06, 
       5.5333623990245080e-06, 
       6.1664880947978114e-06, 
       5.7188583147820323e-06, 
       5.7133969790055027e-06, 
       5.6544358741601256e-06, 
       7.2608850860695403e-06, 
       5.9179612057631273e-06, 
       5.5792238593155419e-06, 
       6.6024583872942040e-06, 
       5.9000159042981068e-06, 
       5.5628205001692157e-06, 
       6.2383764891975408e-06, 
       6.0065705098425825e-06, 
       7.2105422667616364e-06, 
       5.5297785453710923e-06, 
       5.8269152359354678e-06, 
       6.5034167644754156e-06, 
       5.7987387321087941e-06, 
       6.1244567299353366e-06, 
       5.4902248670982550e-06, 
       5.4381404142082752e-06, 
       5.3897061261285667e-06, 
       5.6537769330969066e-06, 
       5.8177954108434776e-06, 
       5.3511880178555835e-06, 
       5.9682279401161824e-06, 
       5.5531889501605182e-06, 
       6.1276104603660443e-06, 
       7.1829448073385632e-06, 
       5.8211208998089501e-06, 
       5.6075332458964771e-06, 
       5.5270128204197134e-06, 
       7.2438783991234791e-06, 
       6.5060578481571736e-06, 
       6.7949027083198405e-06, 
       5.4495255869135593e-06, 
       7.0780287595185447e-06, 
       5.3714182388177623e-06, 
       5.1993421118583157e-06, 
       5.1988368230429119e-06, 
       5.6522507644294218e-06, 
       6.1688520561651427e-06, 
       6.1753075974036657e-06, 
       5.3136884644573691e-06, 
       5.2736364496354909e-06, 
       6.0685638005720477e-06, 
       6.0640786267204043e-06, 
       5.1427248988323829e-06, 
       5.9984515111420417e-06, 
       5.0963400284239101e-06, 
       5.8708076861354811e-06, 
       5.8708076861354811e-06, 
       5.2297965961232114e-06, 
       5.7742602699776282e-06, 
       5.1430840907286563e-06, 
       5.3564923847103657e-06, 
       5.6764080388336965e-06, 
       5.3615112038371476e-06, 
       6.0518823783181513e-06, 
       5.5416494614751916e-06, 
       5.2440934785646912e-06, 
       6.9153676455002985e-06, 
       5.6200506928412139e-06, 
       6.5528189594751042e-06, 
       6.0570678577339580e-06, 
       5.2158300621543786e-06, 
       5.4881107063749253e-06, 
       5.4868925487152638e-06, 
       5.0736626595713733e-06, 
       5.4697639361438365e-06, 
       5.3659917032428393e-06, 
       5.0509042688983174e-06, 
       5.7040468038228717e-06, 
       5.3493546857912489e-06, 
       5.4123326868080220e-06, 
       5.1516637481318055e-06, 
       5.7817818478067093e-06, 
       5.2636369715464500e-06, 
       6.4671042069658231e-06, 
       5.0269987095309869e-06, 
       5.3747526771421389e-06, 
       5.5692530210595150e-06, 
       6.6520755264999482e-06, 
       6.3416798435693794e-06, 
       6.1520176577603210e-06, 
       6.3152716796667773e-06, 
       5.1169800781997018e-06, 
       5.1247027482866805e-06, 
       5.4547653230141280e-06, 
       6.2283629759942392e-06, 
       5.3575585972832832e-06, 
       5.3541602955320290e-06, 
       5.0417279061294094e-06, 
       6.0218225757066414e-06, 
       5.1189256413122862e-06, 
       5.1957810253843831e-06, 
       5.2362997084908111e-06, 
       5.6689435073143479e-06, 
       5.6523345053025765e-06, 
       5.9669516693967838e-06, 
       5.2184882583348902e-06, 
       5.5603111161946228e-06, 
       5.5560471620436252e-06, 
       5.3714182388160098e-06, 
       5.4395051384972718e-06, 
       5.2918898758861294e-06, 
       5.2771353456805135e-06, 
       5.4243350326745304e-06, 
       5.0401657310148476e-06, 
       4.0252578715627750e-06, 
       5.0064093343017481e-06, 
       1.7764796718998617e-06, 
       1.7612593025801785e-06, 
       1.4703465156921496e-06, 
       7.0102565801250459e-07, 
       5.4952125159578815e-07, 
       4.4413304680654396e-07, 
       2.6390632080274210e-07, 
       1.4900000000000000e-03, 
       1.1100000000000001e-03, 
       9.5000000000000000e-04, 
       7.6999999999999996e-04, 
       3.6000000000000002e-04, 
       2.3000000000000001e-04, 
       1.8000000000000001e-04, 
       1.1650112442953340e-04, 
       1.0925892459976593e-04, 
       1.3999999999999999e-04, 
       1.2000000000000000e-04, 
       9.0000000000000006e-05, 
       6.4565029040009129e-05, 
       6.4553285355187372e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       5.0000000000000002e-05, 
       3.2759437135047733e-05, 
       3.2589798205594919e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.1554666176653731e-05, 
       2.0267972565679051e-05, 
       1.6991900081786556e-05, 
       1.6980756748035617e-05, 
       1.4494491681495742e-05, 
       2.0000000000000002e-05, 
       1.3078854957329289e-05, 
       1.2981060310400874e-05, 
       1.0289303103144626e-05, 
       7.5676592890847977e-06, 
       7.5606399659211027e-06, 
       7.3427300098279983e-06, 
       7.3050782509370220e-06, 
       6.3510187923679768e-06, 
       5.9583746048616614e-06, 
       5.5622497613625651e-06, 
       5.5105058225924277e-06, 
       7.4822714577540800e-06, 
       5.5252858812979813e-06, 
       7.4986039240655044e-06, 
       5.4640457060693577e-06, 
       5.3565054791010650e-06, 
       5.3566441843630026e-06, 
       5.1724642728830022e-06, 
       5.1144350418702510e-06, 
       5.1313811293051870e-06, 
       4.8637366012252047e-06, 
       4.6040648752585503e-06, 
       4.5460361819040874e-06, 
       3.4584927137592618e-06, 
       3.5165959372625102e-06, 
       3.5147168986542081e-06, 
       3.6447617045410646e-06, 
       3.6447617045410646e-06, 
       3.8286227323576215e-06, 
       3.7691215018687954e-06, 
       2.7124915472117445e-06, 
       2.8741174442898025e-06, 
       3.7108982032601902e-06, 
       2.8630801502195835e-06, 
       2.8430128048376965e-06, 
       2.7735041521540279e-06, 
       2.7757899616820560e-06, 
       2.6945036493993428e-06, 
       2.2579552085063673e-06, 
       3.0520449246461583e-06, 
       2.1595063039457711e-06, 
       2.2641155882160216e-06, 
       2.2468271062172162e-06, 
       2.0582972483827890e-06, 
       2.6724949636628015e-06, 
       2.2651113789799418e-06, 
       1.8552895547679796e-06, 
       1.8552895547679796e-06, 
       2.6030091245393579e-06, 
       1.9778872001593187e-06, 
       2.0056884112992946e-06, 
       2.3680265510958201e-06, 
       1.7441728282254390e-06, 
       2.2903953182433812e-06, 
       1.7478132208700874e-06, 
       1.7049263690199195e-06, 
       2.0212043860516087e-06, 
       1.5581705149181762e-06, 
       1.5189119937452621e-06, 
       2.0527234569691342e-06, 
       1.5094967688334891e-06, 
       1.9904754673432901e-06, 
       1.4925720522507429e-06, 
       1.4814521746804570e-06, 
       1.5397295960846138e-06, 
       1.7991579567505197e-06, 
       1.3179786614497433e-06, 
       1.6437568925932885e-06, 
       1.3054820767242094e-06, 
       1.3754579538485801e-06, 
       1.3282369114948043e-06, 
       1.3178417256854783e-06, 
       1.2434519789256758e-06, 
       1.2854979399775411e-06, 
       1.2268053247015361e-06, 
       1.1965666862810739e-06, 
       1.2735227371049395e-06, 
       1.2761994777433087e-06, 
       1.1895170929407851e-06, 
       1.1018287841462368e-06, 
       1.1029955308837029e-06, 
       1.0875540790367915e-06, 
       1.0573990085997407e-06, 
       1.0460658066656411e-06, 
       1.0820788598592123e-06, 
       1.0464621256188476e-06, 
       1.0351449870859697e-06, 
       1.0589664488342222e-06, 
       9.8224697385682902e-07, 
       9.8069220831819925e-07, 
       9.7717699178510151e-07, 
       1.0336806273220803e-06, 
       9.7075603751990794e-07, 
       9.6088863455750400e-07, 
       9.8216114996507725e-07, 
       9.7856094801760923e-07, 
       1.0419838346575139e-06, 
       1.0387856340644167e-06, 
       9.2905083182433053e-07, 
       9.6955203792036565e-07, 
       9.2097201459277882e-07, 
       1.1256355488727203e-06, 
       1.1559937754997115e-06, 
       1.2051196138163077e-06, 
       1.1962045108381221e-06, 
       9.8866879036363509e-07, 
       8.9389008999263979e-07, 
       1.1748993296901304e-06, 
       1.0111935221460964e-06, 
       1.0403815419576801e-06, 
       1.0406329054678555e-06, 
       8.3398073854650977e-07, 
       9.3849455520494792e-07, 
       9.3556437278789948e-07, 
       9.3601947727292253e-07, 
       7.6331718933777414e-07, 
       9.0856895471284970e-07, 
       7.8303506526691024e-07, 
       8.8648830007897938e-07, 
       7.5922953949112471e-07, 
       6.9639579629548862e-07, 
       6.9294747632699254e-07, 
       9.1789295390847046e-07, 
       9.1273442188211514e-07, 
       6.6513446246233394e-07, 
       6.5215159048423341e-07, 
       7.1044843807000765e-07, 
       7.1044843807000765e-07, 
       6.4171657279271983e-07, 
       6.0754186409324534e-07, 
       6.4909375384922481e-07, 
       6.8119223559722105e-07, 
       6.2712506514131937e-07, 
       6.1013519200641486e-07, 
       7.2228528133403244e-07, 
       6.0616971630424106e-07, 
       8.0834696347463327e-07, 
       5.9033419547507954e-07, 
       5.8481169371658993e-07, 
       5.8383454590388789e-07, 
       6.1545147735764275e-07, 
       7.8062345651834652e-07, 
       5.8377410752089955e-07, 
       6.0100892570784439e-07, 
       6.0100892570784439e-07, 
       5.8706206593572406e-07, 
       5.8801051909872250e-07, 
       7.0266029726937314e-07, 
       5.7427888929582681e-07, 
       5.4431464619825336e-07, 
       6.7666356977204396e-07, 
       5.5688692077565137e-07, 
       5.5763251202411191e-07, 
       5.2412338472350442e-07, 
       5.1433879723248250e-07, 
       5.1433879723248250e-07, 
       5.9192476042725944e-07, 
       5.3776828990228831e-07, 
       5.3590183341312974e-07, 
       5.4978479827318256e-07, 
       6.6105082718853496e-07, 
       6.4968859756962652e-07, 
       5.1018346374810569e-07, 
       5.1496785029614463e-07, 
       5.1240960144310893e-07, 
       5.1212729607826611e-07, 
       5.0162673907769992e-07, 
       6.1667826486585486e-07, 
       6.1622409788405526e-07, 
       6.1581886529524829e-07, 
       6.1320666737290239e-07, 
       6.0981240883340311e-07, 
       5.4739175782054580e-07, 
       5.7264073939333235e-07, 
       5.8325269687776216e-07, 
       5.4855481105700098e-07, 
       5.4631323714147601e-07, 
       5.1767330415775869e-07, 
       5.1578299686586698e-07, 
       5.1709689763904068e-07, 
       6.1356443173398706e-07, 
       5.7559501445016441e-07, 
    } ,  
    {  
       1.4251457573243711e+01, 
       7.0629899999995374e+00, 
       1.1429948417379998e+00, 
       8.7610901832922305e-01, 
       7.8834808167402515e-01, 
       7.3956629204145363e-01, 
       8.2163174079366497e-01, 
       5.6494697986916509e-01, 
       6.3883742809878896e-01, 
       6.4374935185772142e-01, 
       4.4531067486023307e-01, 
       4.9329999999998347e-01, 
       4.9141000000001916e-01, 
       3.2460110501742634e-01, 
       3.6059999999998649e-01, 
       3.4352576250971745e-01, 
       2.8287457443256769e-01, 
       2.2664304493312151e-01, 
       2.8521515919988760e-01, 
       3.0155682206429113e-01, 
       2.4515729711994211e-01, 
       1.6423577186510677e-01, 
       1.6706959796702761e-01, 
       1.8108346839433509e-01, 
       1.8105721584359097e-01, 
       1.6443067841638262e-01, 
       1.6305920449678701e-01, 
       2.1109035099372481e-01, 
       1.5712074733991802e-01, 
       1.9446267060830916e-01, 
       1.5130313162744510e-01, 
       1.8463096480431115e-01, 
       1.8253758685858937e-01, 
       1.7511173403858782e-01, 
       1.3698478142959328e-01, 
       1.3686130489957518e-01, 
       1.6492075407333007e-01, 
       1.6083728625006383e-01, 
       1.5379311894940678e-01, 
       1.4360786177981982e-01, 
       1.3600276545444179e-01, 
       1.3427285125047461e-01, 
       1.3124068032388905e-01, 
       1.0499695279212787e-01, 
       1.2746242965502491e-01, 
       9.1785288730646369e-02, 
       1.2518199631527549e-01, 
       1.1006702587118962e-01, 
       8.9731966474748848e-02, 
       8.9637489703927903e-02, 
       9.5192286936762571e-02, 
       1.0629677251117418e-01, 
       8.6330413300043263e-02, 
       1.0404948516145857e-01, 
       8.2776127104447164e-02, 
       9.9285549280719032e-02, 
       9.6419999999995343e-02, 
       7.6755546476328512e-02, 
       6.4090626460624453e-02, 
       8.9095434709956053e-02, 
       6.3240144410331142e-02, 
       8.4034986945598827e-02, 
       6.8314028536393123e-02, 
       6.8303077183213837e-02, 
       8.2398944957090450e-02, 
       5.8449272719104599e-02, 
       6.3688009132578050e-02, 
       6.7536848532955648e-02, 
       7.2073300760869777e-02, 
       5.3556313475687803e-02, 
       5.9471106047972226e-02, 
       6.8664003491278658e-02, 
       6.7132924186440693e-02, 
       5.0245652440359882e-02, 
       6.1450150620116467e-02, 
       6.0830070372690781e-02, 
       6.5689999999995169e-02, 
       6.5200132132263888e-02, 
       6.4560000000004544e-02, 
       4.6688226262111013e-02, 
       4.5629591483185773e-02, 
       6.1805954665041365e-02, 
       6.0551814029700807e-02, 
       5.9038968920819626e-02, 
       5.1642974894268674e-02, 
       4.2712246544913401e-02, 
       5.7543101158813247e-02, 
       5.7399968673301077e-02, 
       5.5543844262644110e-02, 
       5.6158345419588578e-02, 
       5.2733808282278270e-02, 
       5.2717323320383182e-02, 
       4.0118635508609217e-02, 
       3.9001503765104729e-02, 
       3.8941618259706892e-02, 
       4.4785365077861607e-02, 
       4.0516769087353709e-02, 
       4.0475158015038100e-02, 
       3.8691468531472956e-02, 
       3.8397166125741559e-02, 
       3.7826248445116568e-02, 
       4.2767434723004295e-02, 
       5.0957800836379630e-02, 
       5.0583843138394910e-02, 
       5.0350000000001262e-02, 
       3.5468402657254532e-02, 
       4.9629999999995719e-02, 
       3.4818106018154361e-02, 
       3.4772973458037502e-02, 
       4.7459999999995221e-02, 
       4.6132545605275931e-02, 
       4.6588652666909529e-02, 
       4.6320711703107115e-02, 
       3.3133228444480294e-02, 
       3.1927899397583115e-02, 
       3.2611784056505090e-02, 
       4.3415375804109618e-02, 
       4.2188005894385046e-02, 
       4.2190750666371270e-02, 
       3.1539100658404894e-02, 
       3.1290014701064087e-02, 
       3.9566393966254762e-02, 
       2.8888069539323193e-02, 
       3.8987938960590385e-02, 
       3.8380000000004348e-02, 
       3.7705952404347515e-02, 
       2.8110548890319713e-02, 
       2.8108234605514738e-02, 
       3.6711933892690105e-02, 
       3.1714467359975455e-02, 
       3.6380000000001071e-02, 
       3.3021973999511997e-02, 
       3.5553189662374737e-02, 
       2.5104780754900673e-02, 
       3.4289439605918105e-02, 
       2.6070980098433071e-02, 
       2.8423149551153059e-02, 
       3.4010000000003794e-02, 
       2.5168234793525160e-02, 
       3.3408400933185263e-02, 
       3.1865472208710700e-02, 
       2.4106559462598230e-02, 
       3.3038099104849832e-02, 
       2.3866119692193755e-02, 
       3.2790000000002505e-02, 
       3.2667317510966284e-02, 
       2.3007619460476997e-02, 
       3.2421606338338557e-02, 
       2.9338162240744110e-02, 
       2.5858913855993395e-02, 
       2.3128699035052842e-02, 
       3.1027262021275717e-02, 
       2.5402215278098387e-02, 
       2.9715822221178807e-02, 
       3.0244349364247369e-02, 
       2.9119850618888238e-02, 
       2.2927797268912212e-02, 
       2.0216692691565879e-02, 
       2.3037952745140203e-02, 
       2.0092715545876751e-02, 
       1.9995328068746640e-02, 
       1.9472192625090038e-02, 
       1.8291649037549606e-02, 
       1.8184608314744891e-02, 
       2.1678397967257811e-02, 
       1.7653100740524553e-02, 
       2.3353077071363824e-02, 
       2.2824418591756091e-02, 
       1.9219851765689255e-02, 
       1.7061611182681052e-02, 
       1.7246354594809988e-02, 
       1.6771945339708338e-02, 
       2.2605735743909162e-02, 
       2.1924137390063780e-02, 
       2.1642757174307643e-02, 
       1.8672338502699275e-02, 
       1.8441108184902938e-02, 
       2.1273253950073061e-02, 
       1.8086561164565584e-02, 
       1.8130724229693963e-02, 
       1.7044540435060838e-02, 
       1.7044477417501532e-02, 
       2.0085482859434386e-02, 
       1.4274335694944601e-02, 
       1.4107359915571025e-02, 
       1.4267884442134482e-02, 
       1.4690675026163877e-02, 
       1.8763498286656501e-02, 
       1.7072880889663928e-02, 
       1.3099975445280646e-02, 
       1.3961014802560323e-02, 
       1.6374616250274730e-02, 
       1.2866997345753910e-02, 
       1.4441647209289192e-02, 
       1.4253412642324525e-02, 
       1.6734154951521151e-02, 
       1.4592019601026889e-02, 
       1.3102093412735239e-02, 
       1.5309486351060199e-02, 
       1.4508937895664301e-02, 
       1.3401961260460497e-02, 
       1.1336904980371118e-02, 
       1.5874360340771023e-02, 
       1.5890009389356627e-02, 
       1.4984936262666060e-02, 
       1.4815413545237275e-02, 
       1.5445974681861178e-02, 
       1.1266242535087706e-02, 
       1.1163898989060715e-02, 
       1.4544535280541294e-02, 
       1.0955204989456752e-02, 
       1.2296882556946316e-02, 
       1.2295502235573874e-02, 
       1.4500494863280781e-02, 
       1.4920000000000700e-02, 
       1.4375525984192825e-02, 
       1.4289302962828048e-02, 
       1.0814548003023025e-02, 
       1.2256997209160753e-02, 
       1.4356791151826224e-02, 
       1.1961952362634452e-02, 
       1.0235634212674503e-02, 
       1.1418764288339248e-02, 
       1.1820688058740524e-02, 
       9.9328117453114789e-03, 
       1.0921300410710584e-02, 
       1.0898434934046814e-02, 
       1.0928950731086273e-02, 
       1.0226221815351147e-02, 
       1.3141502978360422e-02, 
       1.0680472776416540e-02, 
       9.5760088268442966e-03, 
       1.3002310276079674e-02, 
       9.5017794040907263e-03, 
       1.2297949403945159e-02, 
       1.2592493465141108e-02, 
       1.2339999999995421e-02, 
       1.2276588583219322e-02, 
       8.9964420062321627e-03, 
       1.2093760855596099e-02, 
       1.2196107287576665e-02, 
       1.0867962968870925e-02, 
       8.7221437601413855e-03, 
       8.7377387165816726e-03, 
       1.1993327642168065e-02, 
       1.1941092500481457e-02, 
       9.2203010106494641e-03, 
       9.8821115930483854e-03, 
       8.4358991827079323e-03, 
       8.1164727977721344e-03, 
       9.2651103990500534e-03, 
       8.3694365898899278e-03, 
       1.1233065299030327e-02, 
       1.1003741194030168e-02, 
       9.3382855276331316e-03, 
       8.0895990222381465e-03, 
       8.7309503110737027e-03, 
       8.8020024347721954e-03, 
       8.7944588315124524e-03, 
       9.3618103055817959e-03, 
       8.7385815470108034e-03, 
       7.9088607385084612e-03, 
       8.9403125641649202e-03, 
       1.0522621197301553e-02, 
       1.0648382836180623e-02, 
       8.6003996075399037e-03, 
       8.6003288838716104e-03, 
       8.7715455587944729e-03, 
       8.5722352546708219e-03, 
       8.9722870518977524e-03, 
       1.0368090089815611e-02, 
       7.5434677414822461e-03, 
       9.4781841791835297e-03, 
       1.0164625304698017e-02, 
       1.0253092019446068e-02, 
       7.3736562184639268e-03, 
       7.2060381225442643e-03, 
       7.0367235435489723e-03, 
       9.7894616966225246e-03, 
       8.4946926695488659e-03, 
       8.1370793879972959e-03, 
       6.9795098885043613e-03, 
       8.4983289449398567e-03, 
       8.4859288574260540e-03, 
       9.5415347761849574e-03, 
       9.4477773940359853e-03, 
       8.9139888594979826e-03, 
       9.5669254787275216e-03, 
       7.8111718245907841e-03, 
       8.1480996322073219e-03, 
       6.9018976856702495e-03, 
       8.1885149882452400e-03, 
       9.3250624472567198e-03, 
       9.3160649213792201e-03, 
       6.7796717582263716e-03, 
       9.1315731545499341e-03, 
       9.1399999999995738e-03, 
       6.6089180137541160e-03, 
       8.8673379590845664e-03, 
       7.4850898517008204e-03, 
       7.9138893585915041e-03, 
       6.5487551452522805e-03, 
       8.7404399887153709e-03, 
       8.8600000000000952e-03, 
       8.7839550957260395e-03, 
       8.7993441570939461e-03, 
       8.6286267667268265e-03, 
       8.6474635538930922e-03, 
       6.7302390281485866e-03, 
       7.0389455346703961e-03, 
       6.5204579028863017e-03, 
       6.5133883699855606e-03, 
       8.4979770119313017e-03, 
       8.2805359410635087e-03, 
       8.1811026017887881e-03, 
       8.2897116313251553e-03, 
       7.7544015315446305e-03, 
       5.8521313172268212e-03, 
       6.2731478810823639e-03, 
       8.1700000000002829e-03, 
       6.2218927926720978e-03, 
       7.2829573576385920e-03, 
       6.5015069613570883e-03, 
       5.8358368036995683e-03, 
       5.8293286349401221e-03, 
       7.3939759607444923e-03, 
       7.2808655277002733e-03, 
       7.6796266064201729e-03, 
       5.9526732724390049e-03, 
       6.5858013785186856e-03, 
       5.7393996906342703e-03, 
       7.4444208832347017e-03, 
       7.3685068466279478e-03, 
       7.6476703024961224e-03, 
       7.5822428230606287e-03, 
       7.4766600866554211e-03, 
       6.7494436994987323e-03, 
       7.4267611810604458e-03, 
       6.5546467027685455e-03, 
       5.3458627960205922e-03, 
       5.3502199618913806e-03, 
       5.3197264217081999e-03, 
       6.3545948428592163e-03, 
       7.2824944837132227e-03, 
       5.9429317719508126e-03, 
       7.1040682863384788e-03, 
       5.8068799047874039e-03, 
       4.9510454389606251e-03, 
       5.0822150663320451e-03, 
       5.0481687931395970e-03, 
       6.9004428260513008e-03, 
       6.9404182965898138e-03, 
       5.3523915998995196e-03, 
       5.3515205975683718e-03, 
       5.9155924782595803e-03, 
       4.8162536109563709e-03, 
       5.0851723443103153e-03, 
       5.5273918693944076e-03, 
       5.0138462413465154e-03, 
       4.8603936553017176e-03, 
       4.9798617606231742e-03, 
       6.6532508656314644e-03, 
       5.5539146439224016e-03, 
       6.5671958230554906e-03, 
       6.0218437633064812e-03, 
       4.8222362430869201e-03, 
       5.4707720131492096e-03, 
       6.3068150119425593e-03, 
       4.7614523485600716e-03, 
       6.4599999999997793e-03, 
       6.3501628777587792e-03, 
       5.1462490539057751e-03, 
       4.5882135932955098e-03, 
       4.6503690343716294e-03, 
       5.1335599911527400e-03, 
       4.7377382124284793e-03, 
       5.0345606116760193e-03, 
       4.3324620295761337e-03, 
       4.5528580052894355e-03, 
       5.9254830898884297e-03, 
       4.3344604709025280e-03, 
       4.3157453797296810e-03, 
       5.9437972208986152e-03, 
       4.5373081046183030e-03, 
       4.8577501986451672e-03, 
       4.2565499367579812e-03, 
       4.3827245442165236e-03, 
       4.2496210330118800e-03, 
       5.8334423367587063e-03, 
       5.5418416289868362e-03, 
       5.8267409090115288e-03, 
       4.1500845058207950e-03, 
       4.1857546034400435e-03, 
       4.6336555318820761e-03, 
       4.6261860194072614e-03, 
       4.3262254343199615e-03, 
       4.3117237239847835e-03, 
       4.1440269630977451e-03, 
       5.7642442951513729e-03, 
       5.7407821764311013e-03, 
       4.6433348722167951e-03, 
       4.6384138935456265e-03, 
       5.7099834780819604e-03, 
       5.5860551739747326e-03, 
       4.0103196478021028e-03, 
       4.0298304214019410e-03, 
       4.5537327985462504e-03, 
       4.0951045592801626e-03, 
       4.2769747057371460e-03, 
       5.5412259473931981e-03, 
       4.5069136908714317e-03, 
       5.4910889212198225e-03, 
       5.4266394339452517e-03, 
       4.5444205487340603e-03, 
       5.4647383663730658e-03, 
       4.0651306653275137e-03, 
       4.3694250381110088e-03, 
       4.2349664265049361e-03, 
       5.4210467090225131e-03, 
       4.0688231027326003e-03, 
       4.1228004202513756e-03, 
       5.2092306391171650e-03, 
       5.2442444025377543e-03, 
       3.8120639642060588e-03, 
       4.4019685958540875e-03, 
       4.4250852829542447e-03, 
       3.9620685051879080e-03, 
       4.4394697653243869e-03, 
       4.1122452473073643e-03, 
       5.0296863913236300e-03, 
       3.9388402741186886e-03, 
       5.2424800204011485e-03, 
       4.4558157037252301e-03, 
       3.9683003044486149e-03, 
       4.2645386696497558e-03, 
       4.4610685795312254e-03, 
       4.2354678726022931e-03, 
       5.1125464093673741e-03, 
       3.7426392620832261e-03, 
       4.4117376800659027e-03, 
       3.7647455639516652e-03, 
       3.9364904695202568e-03, 
       4.2289390911729426e-03, 
       3.7735459921122783e-03, 
       3.7420133383358816e-03, 
       5.0273393347649966e-03, 
       4.1462438457142738e-03, 
       4.1789998103965702e-03, 
       3.6756145645106641e-03, 
       4.9905181183983929e-03, 
       4.2694069567301481e-03, 
       4.4510046103813077e-03, 
       3.7187638067171951e-03, 
       3.7995023643886548e-03, 
       3.6470245878938126e-03, 
       4.7663151064038850e-03, 
       4.7761839295230434e-03, 
       4.6451681345157422e-03, 
       4.6164208272463821e-03, 
       4.8299999999995187e-03, 
       3.6869107057904533e-03, 
       4.4300757978325495e-03, 
       4.7900000000004840e-03, 
       4.5271007134015167e-03, 
       3.9528511362587979e-03, 
       3.5404056156400393e-03, 
       4.0236788545375286e-03, 
       3.7041755573327196e-03, 
       3.3347071079631394e-03, 
       3.4274573516895656e-03, 
       4.6450414268349475e-03, 
       3.4074236785597194e-03, 
       3.3964737298713492e-03, 
       4.6023035426816161e-03, 
       4.6034337802553305e-03, 
       3.3759524111725663e-03, 
       4.2601266272364142e-03, 
       3.3518300456396050e-03, 
       4.4315500680550809e-03, 
       4.6100000033045020e-03, 
       4.5260423631291345e-03, 
       3.2905829724293794e-03, 
       4.0311885453820240e-03, 
       4.5375037397479071e-03, 
       4.5081916540116998e-03, 
       3.3034540092128895e-03, 
       4.2862959294371556e-03, 
       4.4161281293007629e-03, 
       3.3680125342573069e-03, 
       3.2548943313755246e-03, 
       3.2548943313755246e-03, 
       3.6497612144552618e-03, 
       4.3373467382246287e-03, 
       4.0300106814112951e-03, 
       3.8253101071344942e-03, 
       3.1081893635989768e-03, 
       3.5719646557067740e-03, 
       4.0655404354012969e-03, 
       4.2889185809626357e-03, 
       3.4216352006252778e-03, 
       3.6132675456107487e-03, 
       3.0550437224292669e-03, 
       4.0851551597584565e-03, 
       3.1717776225515474e-03, 
       4.1024226920472751e-03, 
       4.0353511436252874e-03, 
       3.1193102234358039e-03, 
       3.9196213894919030e-03, 
       3.3893039511980099e-03, 
       3.1600000000003834e-03, 
       3.9834820381993777e-03, 
       4.0593726395106227e-03, 
       3.0753077467888901e-03, 
       2.9486491078091893e-03, 
       3.8684773071257315e-03, 
       3.8947443071590997e-03, 
       3.6494449816163485e-03, 
       3.9649586962442951e-03, 
       3.8465318038624625e-03, 
       2.8974794568920759e-03, 
       3.5737281914646740e-03, 
       3.8262418604544475e-03, 
       3.7742375854299937e-03, 
       3.8699999999997287e-03, 
       3.8699999999999516e-03, 
       3.0069769359888253e-03, 
       3.1025331569056254e-03, 
       3.1524399402067726e-03, 
       2.7310207339922457e-03, 
       3.1310471785689496e-03, 
       3.7686905968307301e-03, 
       2.6611686171571455e-03, 
       3.1638130363504800e-03, 
       3.5923165603289784e-03, 
       3.0868165258771201e-03, 
       3.7501379583092698e-03, 
       3.2062463018620911e-03, 
       2.8259732036758512e-03, 
       2.6491799266507333e-03, 
       2.6450728329575168e-03, 
       3.6595992035653312e-03, 
       2.7968230711208730e-03, 
       3.3357503320279587e-03, 
       3.3897093237140632e-03, 
       2.7039573010546251e-03, 
       3.6050160595059639e-03, 
       2.5551519093759776e-03, 
       3.5880983104539751e-03, 
       2.9764461555292204e-03, 
       3.5790981751215485e-03, 
       2.6315137181520541e-03, 
       2.9847819957473593e-03, 
       3.5700342772485272e-03, 
       2.7282046370829863e-03, 
       3.5399468521004319e-03, 
       2.6198704294402046e-03, 
       2.5853041098974909e-03, 
       3.5369153631672843e-03, 
       2.8551844009306643e-03, 
       2.5984519683424643e-03, 
       2.6759948788926385e-03, 
       2.8275792481972269e-03, 
       3.3697494353653449e-03, 
       3.4589093862194215e-03, 
       2.7215857773519527e-03, 
       3.4397306740520609e-03, 
       2.7003319636169218e-03, 
       2.6998703990646447e-03, 
       3.3953292840951149e-03, 
       3.4200000000003815e-03, 
       2.4165801032213583e-03, 
       2.9011833024086288e-03, 
       3.2438745908509141e-03, 
       2.6515975750444226e-03, 
       2.5554048711513488e-03, 
       3.3264021630466656e-03, 
       2.4171139545802454e-03, 
       2.7688821010570020e-03, 
       2.6025260292265101e-03, 
       2.5194166763877713e-03, 
       3.3100000000000221e-03, 
       3.2636986459409038e-03, 
       2.6360466263729509e-03, 
       3.0386338644987966e-03, 
       2.5350239215405231e-03, 
       2.9370910455737680e-03, 
       2.8647748236371585e-03, 
       3.2775138354655966e-03, 
       2.5648514129616240e-03, 
       2.3804813839570071e-03, 
       2.6028294329002609e-03, 
       2.6007217353894592e-03, 
       2.5079775343310336e-03, 
       2.2847346856131809e-03, 
       3.0988677762894940e-03, 
       2.3064261104338156e-03, 
       2.9832218266940655e-03, 
       3.0439717380525044e-03, 
       2.5239351022626161e-03, 
       2.7314712752791854e-03, 
       2.5273518822850330e-03, 
       2.4614861440655214e-03, 
       2.6371836328727386e-03, 
       3.1027920797688825e-03, 
       2.4712656067106004e-03, 
       2.2744613312596479e-03, 
       2.6950511922503146e-03, 
       2.7040258163243797e-03, 
       2.2181952671622273e-03, 
       2.2535395162962932e-03, 
       3.0984425272880109e-03, 
       3.0011756690357301e-03, 
       2.2799106698382786e-03, 
       2.2918168296919444e-03, 
       2.1733633008863412e-03, 
       2.4012359641419037e-03, 
       2.9743258649099773e-03, 
       2.2521458842179299e-03, 
       3.0161616298876861e-03, 
       2.4297543613005092e-03, 
       2.7766976018804355e-03, 
       2.4199999999995414e-03, 
       2.4211629601735537e-03, 
       2.9419421173978318e-03, 
       2.3870133119518432e-03, 
       2.3821102666809289e-03, 
       2.8236889350369380e-03, 
       2.9300000000004930e-03, 
       2.9016840840167379e-03, 
       2.2787979018738650e-03, 
       2.2983666240586507e-03, 
       2.8899999999999928e-03, 
       2.3972947016620730e-03, 
       2.3723601486148813e-03, 
       2.8716272435513755e-03, 
       2.2476611171650248e-03, 
       2.1170667630863076e-03, 
       2.0355248606954697e-03, 
       2.8199999999998070e-03, 
       2.0524913316505968e-03, 
       2.1409532809406780e-03, 
       2.0831395316330058e-03, 
       2.7543957385140917e-03, 
       2.6842665989240074e-03, 
       2.1239895125103113e-03, 
       2.0237146891879127e-03, 
       2.3682725805927824e-03, 
       2.6895008524175706e-03, 
       1.9579294100652467e-03, 
       2.7066895296675004e-03, 
       2.6654961858512656e-03, 
       1.9872779054874853e-03, 
       1.9833063248019590e-03, 
       2.7185308300393288e-03, 
       2.7169512771363399e-03, 
       2.7028189079458492e-03, 
       2.5758232558686259e-03, 
       2.0068850024268967e-03, 
       2.4877748872556703e-03, 
       1.9219330432312276e-03, 
       2.4603415953445549e-03, 
       1.8931716016517451e-03, 
       2.0299999999998895e-03, 
       2.6472296335201986e-03, 
       1.9306363068383426e-03, 
       2.6164932604254300e-03, 
       2.5385705278607924e-03, 
       2.3928975288987730e-03, 
       2.1829795392251609e-03, 
       2.3182015988616558e-03, 
       2.6099999999995159e-03, 
       1.9421205255195234e-03, 
       1.9295149205727218e-03, 
       1.9365989912645330e-03, 
       2.3522546450178202e-03, 
       2.5858359733643499e-03, 
       2.2180990258365246e-03, 
       1.8221640774989003e-03, 
       1.9148473204572969e-03, 
       2.5658080098315350e-03, 
       1.8590326216774008e-03, 
       2.3457244205617460e-03, 
       1.9544455231295028e-03, 
       1.9522253530728719e-03, 
       2.0478872312459898e-03, 
       2.0468487318125667e-03, 
       2.1900245045798198e-03, 
       2.4896289906873523e-03, 
       2.5009123956874252e-03, 
       2.4999999999996401e-03, 
       2.4899999999997819e-03, 
       2.0702521242089576e-03, 
       1.9007870719109836e-03, 
       1.9780462846281208e-03, 
       1.9779336392554050e-03, 
       2.4318789125325042e-03, 
       1.8257966975204971e-03, 
       1.7625690462027088e-03, 
       1.7203878629902841e-03, 
       2.4199999999999174e-03, 
       2.4100000000002869e-03, 
       2.0007672667032837e-03, 
       2.3999999999999183e-03, 
       2.4000000000001338e-03, 
       1.7780516869793483e-03, 
       1.6983333676617875e-03, 
       2.3900000000002673e-03, 
       1.9807717103669104e-03, 
       2.2825691160982494e-03, 
       1.8191742338448909e-03, 
       1.8177788561484056e-03, 
       2.3007602462709500e-03, 
       2.0241995238632522e-03, 
       1.7830723828296887e-03, 
       2.2779806431451667e-03, 
       1.7509536248692258e-03, 
       2.2841914134650206e-03, 
       1.9480166452778796e-03, 
       2.3269518237402430e-03, 
       1.7338656980588098e-03, 
       2.2549876988810563e-03, 
       1.6869328314518073e-03, 
       1.8850192593118588e-03, 
       1.7335109636342030e-03, 
       1.9277037674457933e-03, 
       2.2937134493193167e-03, 
       2.2347239648000481e-03, 
       2.2899606564136419e-03, 
       1.8386076181676812e-03, 
       1.8091544383037517e-03, 
       1.8365890862521257e-03, 
       1.9632720077398563e-03, 
       2.2900000000004163e-03, 
       2.2606146438506084e-03, 
       2.1947260323806192e-03, 
       1.8366273892713070e-03, 
       2.2499999999999330e-03, 
       1.6503258949841859e-03, 
       1.5898618697840392e-03, 
       1.6132678133232736e-03, 
       1.7285710479875043e-03, 
       2.1819364536674727e-03, 
       1.7996990520162686e-03, 
       2.1424765074165306e-03, 
       2.1745838065229685e-03, 
       1.5768150137680436e-03, 
       2.1720670113321329e-03, 
       2.1799999999998622e-03, 
       1.6190812466956824e-03, 
       2.1722802848129449e-03, 
       1.8417832480108378e-03, 
       1.5528621807483862e-03, 
       2.1350106715931566e-03, 
       1.7350064538951375e-03, 
       1.7269363987423281e-03, 
       1.8711146861466682e-03, 
       1.5056759373217388e-03, 
       2.1001693125734843e-03, 
       2.0942332572060143e-03, 
       1.7982870075673900e-03, 
       1.9068667899253867e-03, 
       1.6503467998176468e-03, 
       2.0674873764758880e-03, 
       1.5325878350525264e-03, 
       2.0238367641259290e-03, 
       2.0550317770209876e-03, 
       2.0252491950527971e-03, 
       1.5308577695473096e-03, 
       1.5306973495298570e-03, 
       1.5793882496227105e-03, 
       1.6822554722101719e-03, 
       2.0265195270580318e-03, 
       1.8952629003556988e-03, 
       1.6407558163041862e-03, 
       1.9945984214087714e-03, 
       2.0015540468618395e-03, 
       1.4139060995160618e-03, 
       1.6617098435285356e-03, 
       1.9254248642200243e-03, 
       1.4062371807204413e-03, 
       1.6424171632766569e-03, 
       1.6495245913847024e-03, 
       1.6327609792276482e-03, 
       1.4635225266377311e-03, 
       1.4308955088630821e-03, 
       1.4018723847740997e-03, 
       1.8834339489351927e-03, 
       1.8611571534887577e-03, 
       1.8766687581784499e-03, 
       1.4179850367390420e-03, 
       1.4526550849287504e-03, 
       1.7244562643124190e-03, 
       1.9121356810601030e-03, 
       1.9231596069735166e-03, 
       1.6508087464653458e-03, 
       1.3777607252862863e-03, 
       1.6454618074853227e-03, 
       1.3766830169104562e-03, 
       1.8971842998242090e-03, 
       1.6777851843509054e-03, 
       1.5532247111761557e-03, 
       1.4034210878908348e-03, 
       1.8816394698278698e-03, 
       1.4014309909259250e-03, 
       1.3968544754312036e-03, 
       1.8700000000002452e-03, 
       1.8700000000002098e-03, 
       1.5087639524448795e-03, 
       1.3551651688856578e-03, 
       1.8223261810681351e-03, 
       1.7661833013445389e-03, 
       1.7838676718915363e-03, 
       1.8502113047060027e-03, 
       1.7577010502631578e-03, 
       1.5192144070342052e-03, 
       1.5708135120443797e-03, 
       1.8256453107601853e-03, 
       1.8254830921044806e-03, 
       1.2927516222434193e-03, 
       1.2954914079126398e-03, 
       1.7771201082909371e-03, 
       1.5149735245188185e-03, 
       1.2914713653732102e-03, 
       1.6445712380612244e-03, 
       1.2696070728118419e-03, 
       1.7843300701110296e-03, 
       1.3157549197217332e-03, 
       1.7832165136524272e-03, 
       1.3295767037098355e-03, 
       1.7000046378086108e-03, 
       1.5961206364084437e-03, 
       1.3561459724833172e-03, 
       1.7546449247051949e-03, 
       1.4570134449760693e-03, 
       1.3898727089682165e-03, 
       1.6191994377977149e-03, 
       1.5645293549003910e-03, 
       1.7223071664046474e-03, 
       1.6712992823120559e-03, 
       1.6207839964921986e-03, 
       1.3094477817975597e-03, 
       1.2569383607199749e-03, 
       1.6750126456623153e-03, 
       1.7067708412457268e-03, 
       1.2980850105653310e-03, 
       1.2979746762003005e-03, 
       1.5598484156674466e-03, 
       1.4098389180114096e-03, 
       1.3216291382943553e-03, 
       1.2428152303766366e-03, 
       1.5161105960777152e-03, 
       1.2420958168221062e-03, 
       1.3704625817213046e-03, 
       1.4504155532753240e-03, 
       1.4820334665109822e-03, 
       1.6496425196643361e-03, 
       1.6493818304308737e-03, 
       1.2114249995077357e-03, 
       1.3501186496078378e-03, 
       1.1920755562586532e-03, 
       1.3144748278246994e-03, 
       1.5876732120418174e-03, 
       1.5801546923187657e-03, 
       1.1869047774034163e-03, 
       1.5610172709767339e-03, 
       1.1225551325039886e-03, 
       1.1654554959813562e-03, 
       1.4890867100912824e-03, 
       1.2952018864439596e-03, 
       1.5636525516754310e-03, 
       1.5612119188761532e-03, 
       1.2073283840861419e-03, 
       1.2959527207559171e-03, 
       1.1376373878078253e-03, 
       1.2809689517142826e-03, 
       1.3249997525258835e-03, 
       1.5208287290748687e-03, 
       1.1000974537092906e-03, 
       1.3312629386195314e-03, 
       1.5204386359646618e-03, 
       1.4605374243384219e-03, 
       1.4754772135261614e-03, 
       1.5224156235756640e-03, 
       1.0864582301641662e-03, 
       1.0919910429324818e-03, 
       1.4542710429248334e-03, 
       1.4934959953747521e-03, 
       1.2490629610300006e-03, 
       1.1235046413893206e-03, 
       1.1321764201033899e-03, 
       1.4979530210164555e-03, 
       1.2444390624686773e-03, 
       1.4769897596576510e-03, 
       1.2172989807796421e-03, 
       1.0543785792185990e-03, 
       1.0812161219740978e-03, 
       1.2915562700134325e-03, 
       1.2891322235375040e-03, 
       1.4567401583301628e-03, 
       1.0890437945736370e-03, 
       1.1961248214479132e-03, 
       1.1138217529705540e-03, 
       1.1138217529705540e-03, 
       1.2961763385410602e-03, 
       1.3930029893194852e-03, 
       1.0370454804015058e-03, 
       1.1907394547140478e-03, 
       1.1103995892176044e-03, 
       1.4300000000002479e-03, 
       1.1925623463937486e-03, 
       1.1756587464652275e-03, 
       1.2585521050246628e-03, 
       1.3445309344465039e-03, 
       1.4204728198582424e-03, 
       1.4133346909035878e-03, 
       1.1775964492347218e-03, 
       1.1573027029038131e-03, 
       1.0855891597727570e-03, 
       1.4100000000004435e-03, 
       1.2648833334875436e-03, 
       9.8642469430611506e-04, 
       1.0275813540482407e-03, 
       1.0555013041040728e-03, 
       9.9819389362156897e-04, 
       1.2433506915278511e-03, 
       1.3765263557285331e-03, 
       9.9034240658264750e-04, 
       9.7279040163917403e-04, 
       1.3086182703963350e-03, 
       9.7158939871699564e-04, 
       1.1456493092330097e-03, 
       9.7418356003669540e-04, 
       1.3534111534617562e-03, 
       1.3351384686283842e-03, 
       9.8299324608544405e-04, 
       1.2038556933835489e-03, 
       1.0456741298643434e-03, 
       1.2952249410552608e-03, 
       1.1101262691606509e-03, 
       9.6753915639061889e-04, 
       1.2797035021220309e-03, 
       9.5094563299321202e-04, 
       9.9317960984859617e-04, 
       9.6534608775443627e-04, 
       9.5889018735018257e-04, 
       9.8512633706832617e-04, 
       1.0321970354880074e-03, 
       9.8975835786419475e-04, 
       9.6003395975713076e-04, 
       1.2765920184703047e-03, 
       9.3662067866448462e-04, 
       9.3000000000001133e-04, 
       1.3127530969139346e-03, 
       1.0865463154909878e-03, 
       1.0526862570578663e-03, 
       1.0516170403895252e-03, 
       9.3738508179694703e-04, 
       1.1303347363034800e-03, 
       9.2830149508325778e-04, 
       1.0052433311216577e-03, 
       1.2661045548038187e-03, 
       1.2922303583327006e-03, 
       9.6571632936494182e-04, 
       9.6447271286049521e-04, 
       1.2532086289933225e-03, 
       1.2909691719810109e-03, 
       1.0554029197475286e-03, 
       9.2978576200816591e-04, 
       1.2785993921210614e-03, 
       1.1834576057478997e-03, 
       1.2284959753454082e-03, 
       1.2699954126648093e-03, 
       9.4808121513936035e-04, 
       1.0024432138946654e-03, 
       1.2298178671611793e-03, 
       1.2113818681951645e-03, 
       9.4801346639570596e-04, 
       1.2142671101691000e-03, 
       1.2599999999998426e-03, 
       9.1800344261085555e-04, 
       1.0725286188973898e-03, 
       1.1599999999998159e-03, 
       1.0629071580385585e-03, 
       9.1232033840239421e-04, 
       1.1630885854882781e-03, 
       1.0794296412070786e-03, 
       1.0665972878419192e-03, 
       1.2329089067928292e-03, 
       1.2368080226265218e-03, 
       9.0939710877795479e-04, 
       1.2334740040695577e-03, 
       9.7351107849066691e-04, 
       1.1615715172142421e-03, 
       1.2061014874807255e-03, 
       1.0094002078523878e-03, 
       1.2300000000000737e-03, 
       9.3613845607067958e-04, 
       9.4097148850665199e-04, 
       1.2246944295565530e-03, 
       8.8008692332644956e-04, 
       8.6905442097273146e-04, 
       1.1858449620058234e-03, 
       1.2200002989522589e-03, 
       1.0148453228840163e-03, 
       1.0346230494303092e-03, 
       8.8040461890963458e-04, 
       1.0496273169325839e-03, 
       8.9777141655545366e-04, 
       1.0837776849863744e-03, 
       9.6859699326533311e-04, 
       1.1711318896361004e-03, 
       1.0711541643075523e-03, 
       1.1665183621811495e-03, 
       1.1490295100208825e-03, 
       1.1931935008622368e-03, 
       8.6052672601022863e-04, 
       1.1910957854165158e-03, 
       1.0389706735662884e-03, 
       1.0887497206317741e-03, 
       1.0308327779286342e-03, 
       1.1376350361910154e-03, 
       9.8092139282933660e-04, 
       8.4704049425449353e-04, 
       8.6690511533307085e-04, 
       9.3104157411509365e-04, 
       8.3759190208746467e-04, 
       1.0130114866484790e-03, 
       9.7726959591331371e-04, 
       9.5821244153830912e-04, 
       8.5454905270546078e-04, 
       9.1107071421944546e-04, 
       1.1712173367357662e-03, 
       8.3282677021752165e-04, 
       8.2659589346905618e-04, 
       1.1100358709307235e-03, 
       8.5223741674347387e-04, 
       8.4352696642236947e-04, 
       1.0116640678454482e-03, 
       1.1148682129775349e-03, 
       1.1600000000001093e-03, 
       8.1768916582058689e-04, 
       8.3600261687934857e-04, 
       1.1251200408447881e-03, 
       1.1456217508327799e-03, 
       1.1409139220836179e-03, 
       9.6545907568095619e-04, 
       1.1323239041273871e-03, 
       1.1315650878399993e-03, 
       1.1008358997167133e-03, 
       1.1300000000002812e-03, 
       9.4577781453270807e-04, 
       8.2279441518732426e-04, 
       1.1266993471448887e-03, 
       1.0903003804041259e-03, 
       8.0766078585498111e-04, 
       9.3478822209424185e-04, 
       1.0784929157195769e-03, 
       8.3006864829028310e-04, 
       1.0583221130535429e-03, 
       1.1094376852816847e-03, 
       7.9565216415007894e-04, 
       9.7244429779145181e-04, 
       7.8742599447722639e-04, 
       9.0648699947584060e-04, 
       1.0196915543615637e-03, 
       9.1809201721812712e-04, 
       1.0184813192613900e-03, 
       9.1522674398520644e-04, 
       1.0347315398800887e-03, 
       7.8997684950622476e-04, 
       1.0805440784325776e-03, 
       8.9479169604199708e-04, 
       1.0569965439996669e-03, 
       8.8991848703764726e-04, 
       1.0338625420254464e-03, 
       7.9573873837153854e-04, 
       9.6788962565278554e-04, 
       1.0246959253880103e-03, 
       8.5646651058429355e-04, 
       9.1563890352915761e-04, 
       1.0433697927345999e-03, 
       9.2213590099923630e-04, 
       7.7892305449145762e-04, 
       7.8856094614640200e-04, 
       8.6653114663670660e-04, 
       8.6646841693019134e-04, 
       8.9776978951531009e-04, 
       7.4168254552519470e-04, 
       7.7678684589418663e-04, 
       1.0306283206436698e-03, 
       7.9310913548351900e-04, 
       7.9053477314145436e-04, 
       8.3632264149889764e-04, 
       7.6144073851644076e-04, 
       1.0277153730335461e-03, 
       9.7440456489347000e-04, 
       9.2651204943898242e-04, 
       7.5588008391139249e-04, 
       9.7314615577283217e-04, 
       8.1982166626856211e-04, 
       9.7221328004828435e-04, 
       1.0002815602761206e-03, 
       7.4710708260124204e-04, 
       8.6568435755354930e-04, 
       9.0792895455544490e-04, 
       8.4300307694644655e-04, 
       7.8263328678369382e-04, 
       7.9713079621560740e-04, 
       7.6491305876090108e-04, 
       8.4378017076718785e-04, 
       8.1280990192064521e-04, 
       1.0022993250033661e-03, 
       8.3313400417962509e-04, 
       7.3939323541613350e-04, 
       8.2891418235444972e-04, 
       7.3455348180202806e-04, 
       8.2733113817355425e-04, 
       7.4409063843852848e-04, 
       8.0476678209954675e-04, 
       9.8999846744550566e-04, 
       8.0461005977376795e-04, 
       9.1018299110811522e-04, 
       7.0269820453712994e-04, 
       7.0669437355649748e-04, 
       7.1593528598210704e-04, 
       9.4365224177194416e-04, 
       7.2872433665334597e-04, 
       9.5424193532565827e-04, 
       7.0886162250229860e-04, 
       7.7795907707655927e-04, 
       9.4612316554480908e-04, 
       7.2804472880866062e-04, 
       9.7513557462950958e-04, 
       9.1821237842902614e-04, 
       6.9411276658204696e-04, 
       8.3142037398157859e-04, 
       7.1514495929499687e-04, 
       9.7000000000001382e-04, 
       7.1410394686497629e-04, 
       7.0813256168191180e-04, 
       9.0246164118255217e-04, 
       7.9980334316487268e-04, 
       8.2021783439170908e-04, 
       6.8161438271516206e-04, 
       6.9703861141072910e-04, 
       6.9943800764273132e-04, 
       7.1254736449391238e-04, 
       7.9531256715734654e-04, 
       7.2585222270890842e-04, 
       9.3134205004369156e-04, 
       7.7424818449768661e-04, 
       6.9841205197373579e-04, 
       7.7356785474675402e-04, 
       6.7863709647214977e-04, 
       8.6247022067978776e-04, 
       7.7879579799354102e-04, 
       9.4282042185140659e-04, 
       9.3678706388349063e-04, 
       8.4069150607142290e-04, 
       8.1314945375800067e-04, 
       8.1202647999096373e-04, 
       8.9785571152928409e-04, 
       7.8069838174116204e-04, 
       8.0333658894100174e-04, 
       7.6946118729107353e-04, 
       9.3000276244877326e-04, 
       7.4966076705902803e-04, 
       8.4971651518809274e-04, 
       6.9384560363747812e-04, 
       7.7604739092342078e-04, 
       6.7085871989980696e-04, 
       7.7197054848402715e-04, 
       7.2310282250962442e-04, 
       7.0955882061521120e-04, 
       8.8123764978376075e-04, 
       7.4465296825665817e-04, 
       6.6602981941966965e-04, 
       6.6690291217751780e-04, 
       6.8476968075501237e-04, 
       7.3908300356547363e-04, 
       7.3827546823965824e-04, 
       6.7156223298071925e-04, 
       6.7156223298071925e-04, 
       7.5174811240776224e-04, 
       9.1000000000004163e-04, 
       9.0779438677038012e-04, 
       8.3875719032176321e-04, 
       6.5868461063574735e-04, 
       7.6652500688921771e-04, 
       6.6681709180859013e-04, 
       6.4132284763147222e-04, 
       7.7690706557248653e-04, 
       6.4028864315425016e-04, 
       6.6750025715394269e-04, 
       8.9669739221255389e-04, 
       8.7967803816434501e-04, 
       6.4282206692254989e-04, 
       6.5130022039831001e-04, 
       6.3362931377866356e-04, 
       6.6573379846505477e-04, 
       6.9858794980904638e-04, 
       8.5660551968810925e-04, 
       7.4112391615269797e-04, 
       8.8677022639395680e-04, 
       6.4239790753865302e-04, 
       7.8711877804800965e-04, 
       6.5123281582353282e-04, 
       8.8000000000004318e-04, 
       6.2068431423636407e-04, 
       8.6920357265883846e-04, 
       8.7391428680221278e-04, 
       6.3143772308258653e-04, 
       8.6080515051304720e-04, 
       6.6371044248596527e-04, 
       6.5232962650301970e-04, 
       6.4026482505134819e-04, 
       6.4026482505134819e-04, 
       8.4321985992577282e-04, 
       6.1574345397563354e-04, 
       7.2100600217030848e-04, 
       8.6372702755716305e-04, 
       7.6966335784238861e-04, 
       8.6200654040554476e-04, 
       6.1158742150439923e-04, 
       7.1498381869119067e-04, 
       7.3534640458563518e-04, 
       6.9859436687440268e-04, 
       7.8191855263581378e-04, 
       6.5339877875688441e-04, 
       6.5240500092743976e-04, 
       6.1982505678272865e-04, 
       7.2340099106230266e-04, 
       7.8013613963673247e-04, 
       8.3616656788067559e-04, 
       6.8841056731480720e-04, 
       6.0029944242958016e-04, 
       6.9669644193107855e-04, 
       6.3257632304299831e-04, 
       7.3372048311659067e-04, 
       8.2220510869049530e-04, 
       8.3669455650854110e-04, 
       6.0316170414996930e-04, 
       6.1844176599117659e-04, 
       6.1872219584747782e-04, 
       6.9593362765576149e-04, 
       6.0834812220768666e-04, 
       6.2659570991868262e-04, 
       6.2661757041552330e-04, 
       7.3716058184034137e-04, 
       6.6496810260512247e-04, 
       6.8029633458562000e-04, 
       6.8000665693606331e-04, 
       8.3021291865708463e-04, 
       8.1767271090127706e-04, 
       6.1424330134125956e-04, 
       8.3000000293367827e-04, 
       6.0985571426827188e-04, 
       7.0823811775475867e-04, 
       6.6459371623357136e-04, 
       8.2196621697132029e-04, 
       6.1415282177970740e-04, 
       5.9972825136989907e-04, 
       6.5786260815004296e-04, 
       6.0119573645925502e-04, 
       8.1867900238362162e-04, 
       7.9000455583676233e-04, 
       8.1550591305644177e-04, 
       6.7711639966669271e-04, 
       6.3511126394072892e-04, 
       7.8289967941845085e-04, 
       5.7487147684041635e-04, 
       5.7431009692424117e-04, 
       7.8859062915458578e-04, 
       8.1000000000001459e-04, 
       5.7540540131339752e-04, 
       6.7950379247712106e-04, 
       6.7925481916929404e-04, 
       7.8631447703771964e-04, 
       5.8797821302234506e-04, 
       8.0111938525046628e-04, 
       6.0930544311687488e-04, 
       8.0000000000001142e-04, 
       7.7288081669481701e-04, 
       7.9012586086609870e-04, 
       5.6557066786845839e-04, 
       6.8829949818700236e-04, 
       7.8851225383130114e-04, 
       6.1811910089021197e-04, 
       6.8261425868056970e-04, 
       6.2192664561587768e-04, 
       6.2172657571608967e-04, 
       6.0335562376424043e-04, 
       6.3546634149962795e-04, 
       7.9230202111598769e-04, 
       7.3426841537285108e-04, 
       6.2775549002145616e-04, 
       6.5813170686152402e-04, 
       6.2000630240997682e-04, 
       5.8935810241948696e-04, 
       7.8989703736756891e-04, 
       6.5878391473622028e-04, 
       5.5825477734482143e-04, 
       6.3566083634935600e-04, 
       5.5696838578971346e-04, 
       5.7832240504527405e-04, 
       6.5821564761185225e-04, 
       6.5269722726035663e-04, 
       7.4626628412528622e-04, 
       7.6708975560699168e-04, 
       5.5817198725447098e-04, 
       5.5514695714526275e-04, 
       7.7721443263189540e-04, 
       5.7038000587658676e-04, 
       7.7531541309847269e-04, 
       7.7385038885484695e-04, 
       6.3689254218488077e-04, 
       5.4664932984583008e-04, 
       5.6472954698030711e-04, 
       5.4374441643537910e-04, 
       5.6595635183830388e-04, 
       5.4844452368383071e-04, 
       6.3787512410984760e-04, 
       6.6403608061930540e-04, 
       7.0780011383870828e-04, 
       5.4392453756790110e-04, 
       5.7070485353315561e-04, 
       5.7337620358155798e-04, 
       7.2490198671951145e-04, 
       7.3008052763913521e-04, 
       7.5999999999995841e-04, 
       7.0858008587138769e-04, 
       5.4791595179014686e-04, 
       7.5720139545383514e-04, 
       6.0934502501120661e-04, 
       5.4678211549236849e-04, 
       5.9138941823573966e-04, 
       7.2950902809912029e-04, 
       5.5339662960562821e-04, 
       7.2491758837569750e-04, 
       5.5379459678166719e-04, 
       5.4379377660796228e-04, 
       5.3389817404765518e-04, 
       6.2159724839353165e-04, 
       5.5628309491086636e-04, 
       6.4523080264652303e-04, 
       7.0907733189560466e-04, 
       6.6033047672569591e-04, 
       5.5378490182944639e-04, 
       7.1834494993784322e-04, 
       7.4213105818267042e-04, 
       6.5374483268347040e-04, 
       6.5266557974401752e-04, 
       5.2942198338573436e-04, 
       7.3999999999997386e-04, 
       7.4000000000001636e-04, 
       6.6092824664835525e-04, 
       5.2961141452477305e-04, 
       6.6581858743919157e-04, 
       7.2999974919207096e-04, 
       7.2999963076437492e-04, 
       5.5467143565976636e-04, 
       7.2992228105441372e-04, 
       6.2840305700487416e-04, 
       6.1452398036947089e-04, 
       5.2069714748877063e-04, 
       6.2677091869370012e-04, 
       7.1302760465230075e-04, 
       5.4089622468011373e-04, 
       5.5898812751329979e-04, 
       6.0931847502289273e-04, 
       5.1230391635608300e-04, 
       5.1436648547147556e-04, 
       5.3639982834622583e-04, 
       5.7539865900659861e-04, 
       5.2055512710551103e-04, 
       7.1710356663660794e-04, 
       7.1470424192824093e-04, 
       5.0532362346978363e-04, 
       5.1643089703568088e-04, 
       5.1976297439435286e-04, 
       5.7465312508802441e-04, 
       6.8414297292901784e-04, 
       7.1000000000004989e-04, 
       5.0884318288048423e-04, 
       5.2812394449659153e-04, 
       6.3302759308513090e-04, 
       7.0547990856312346e-04, 
       7.0798695767560957e-04, 
       6.7483662338723132e-04, 
       6.3810409887621387e-04, 
       4.9810961364705488e-04, 
       7.0200172250489495e-04, 
       5.3409237709100622e-04, 
       6.8900336577227444e-04, 
       5.9815571122091512e-04, 
       5.1307067397418648e-04, 
       6.9522924638597825e-04, 
       6.9511357842438448e-04, 
       6.3383446109559845e-04, 
       5.7832170103207540e-04, 
       6.5150389111380742e-04, 
       4.9541729000781014e-04, 
       5.2058192795818804e-04, 
       5.9131148231798684e-04, 
       6.4302987438582659e-04, 
       5.1481996021592361e-04, 
       4.9157115149900898e-04, 
       6.0280769361011370e-04, 
       5.6745820624528687e-04, 
       4.8355910489993840e-04, 
       5.6507212990846432e-04, 
       5.0730049399833727e-04, 
       6.7217573143669778e-04, 
       6.7887416791616621e-04, 
       5.5636094139904042e-04, 
       5.4961124234598289e-04, 
       4.8602644827140559e-04, 
       5.0938091158283724e-04, 
       6.7069048117028758e-04, 
       5.0215882513564010e-04, 
       5.5123517318064481e-04, 
       6.5268631430349033e-04, 
       4.7391213756538703e-04, 
       5.8090593726414558e-04, 
       6.3924507325015180e-04, 
       5.5209438130806936e-04, 
       4.9240943424248417e-04, 
       6.6000000000003708e-04, 
       4.7051462645494776e-04, 
       4.6692903419592245e-04, 
       5.4752816366597230e-04, 
       5.4630854112946374e-04, 
       5.3004132164538099e-04, 
       6.3062808974917711e-04, 
       6.3028007427644067e-04, 
       6.1056563291240335e-04, 
       5.7386396146040638e-04, 
       5.3772207757803831e-04, 
       6.3814635694454383e-04, 
       5.1617078394295822e-04, 
       6.4964955709223939e-04, 
       5.5424895413455763e-04, 
       5.0704652872991406e-04, 
       4.7984389657520371e-04, 
       6.4895196151641135e-04, 
       6.5000000714259194e-04, 
       6.4999999999995021e-04, 
       6.5000000000002588e-04, 
       5.7974990830082080e-04, 
       4.6116893078348688e-04, 
       4.6105045835894044e-04, 
       5.6518561923369292e-04, 
       4.9330423240376362e-04, 
       6.4000000000003193e-04, 
       5.6726398281265779e-04, 
       6.2411138393112572e-04, 
       6.2000000000004467e-04, 
       6.2264949120963923e-04, 
       4.5139493731399403e-04, 
       4.5871717975028376e-04, 
       4.5282857003451141e-04, 
       5.3929780421029894e-04, 
       5.5114513657074729e-04, 
       5.1827429749445311e-04, 
       4.6606441991696559e-04, 
       4.6551664567783077e-04, 
       5.6395540495885648e-04, 
       5.4266723688563311e-04, 
       4.5096746609551899e-04, 
       6.2726107068161448e-04, 
       4.6891395226036995e-04, 
       6.2713013014844101e-04, 
       6.0460364724051802e-04, 
       4.6490475422134063e-04, 
       4.5044461434635253e-04, 
       4.6060775522130046e-04, 
       6.2588674405481708e-04, 
       4.7493404627172012e-04, 
       4.6555159101184832e-04, 
       4.6924140135041467e-04, 
       5.5454665832410622e-04, 
       6.0110125953051713e-04, 
       5.7744783644751306e-04, 
       4.9263300484923844e-04, 
       4.4992127698678994e-04, 
       5.4091321885663774e-04, 
       4.3916091955935142e-04, 
       5.4010379449741675e-04, 
       6.1780995838109408e-04, 
       4.7434008361137744e-04, 
       4.9176163203455312e-04, 
       5.9605366984609204e-04, 
       4.3585740086119303e-04, 
       6.1198223062870077e-04, 
       5.9945970877674009e-04, 
       6.1156960132301684e-04, 
       4.9900568742153231e-04, 
       5.0318564804537958e-04, 
       5.9166025314418577e-04, 
       6.0320966875775312e-04, 
       5.0563910821510767e-04, 
       4.7116886124364742e-04, 
       4.5210135635676240e-04, 
       5.0057580977442954e-04, 
       4.8085403098942811e-04, 
       4.2510263363900776e-04, 
       4.8605194091633355e-04, 
       5.0038026554613047e-04, 
       4.6009372196928933e-04, 
       5.4999999998350021e-04, 
       4.6325826776952409e-04, 
       5.3497409788637079e-04, 
       4.2212467855028943e-04, 
       4.2430517418729288e-04, 
       4.9443022705390369e-04, 
       4.3682611477321190e-04, 
       4.7664420717421916e-04, 
       4.3733684690350411e-04, 
       5.2124592134093989e-04, 
       5.6389643274233734e-04, 
       4.9077073757915343e-04, 
       5.5646605353301573e-04, 
       5.8134237311108521e-04, 
       4.2955244193296736e-04, 
       4.8798053777009793e-04, 
       4.3161980268238481e-04, 
       5.8658242785552693e-04, 
       4.1516608093631946e-04, 
       5.3850314401051409e-04, 
       5.2639323687619563e-04, 
       4.1779609220308833e-04, 
       4.8562229419491133e-04, 
       4.9398842806422068e-04, 
       4.7989239609001570e-04, 
       4.2539607353024524e-04, 
       5.6014907347665740e-04, 
       5.6870009610388906e-04, 
       4.4163276182798987e-04, 
       4.0961420111653823e-04, 
       5.7512767555965058e-04, 
       5.7512767555965058e-04, 
       4.9974422197164093e-04, 
       4.8162583062591328e-04, 
       4.7276044731840917e-04, 
       4.3530070837642876e-04, 
       4.2598368022415262e-04, 
       5.6999999999999564e-04, 
       5.6335651122615992e-04, 
       4.5916480942228568e-04, 
       4.9197581347584134e-04, 
       4.0513652243518236e-04, 
       4.9191181205398323e-04, 
       4.6719090694487188e-04, 
       5.6005244572578290e-04, 
       5.0620815917619313e-04, 
       4.3737874174964634e-04, 
       4.2254269754009305e-04, 
       5.0318624530192458e-04, 
       4.1384923309204538e-04, 
       5.1999999999999789e-04, 
       4.6982485597677111e-04, 
       4.6818222624197929e-04, 
       4.2396172632085181e-04, 
       4.0095936284658513e-04, 
       4.2528950639611398e-04, 
       5.5575929263323575e-04, 
       4.6450829740590451e-04, 
       4.3683809078369208e-04, 
       4.6364913655398405e-04, 
       4.8371086780118680e-04, 
       5.4989367502066846e-04, 
       5.5000000000002118e-04, 
       3.9407655958794791e-04, 
       5.1549151323945438e-04, 
       3.9388407483963519e-04, 
       5.3818082284043552e-04, 
       4.1338172836927819e-04, 
       4.0916263705879078e-04, 
       4.5546994045643491e-04, 
       4.8751942788748522e-04, 
       4.1256719381255034e-04, 
       4.1463506368527554e-04, 
       4.5761529986272447e-04, 
       4.5713328062794978e-04, 
       5.5000000000000426e-04, 
       5.4999999994610825e-04, 
       5.0079616399263942e-04, 
       5.4552038217683778e-04, 
       4.6955223180184246e-04, 
       4.4668331051542258e-04, 
       5.4487690620766112e-04, 
       5.3999625505848090e-04, 
       4.0353909453589752e-04, 
       4.0353909453589752e-04, 
       4.2045118153598201e-04, 
       4.2809558453515392e-04, 
       3.8727862826213629e-04, 
       3.9477406153796572e-04, 
       3.8457937853508220e-04, 
       3.9456717868927011e-04, 
       3.8739514086400656e-04, 
       3.8311386562380829e-04, 
       3.8560631202609521e-04, 
       4.5004151858375865e-04, 
       3.8918891237868638e-04, 
       3.8949857213963109e-04, 
       4.2720352876515788e-04, 
       5.1121048883316081e-04, 
       5.1506245415238881e-04, 
       5.3437298561299570e-04, 
       3.9455596091555431e-04, 
       4.9872215054401719e-04, 
       5.1432484411937274e-04, 
       4.4517766482624586e-04, 
       4.4508913866600031e-04, 
       5.3367800639666043e-04, 
       5.1208172912988728e-04, 
       4.6685783886018606e-04, 
       4.0426565444144456e-04, 
       5.2649942590219637e-04, 
       4.6584336911603881e-04, 
       4.8196179333207195e-04, 
       4.4183047549366738e-04, 
       3.7600638848939817e-04, 
       5.3136331995239257e-04, 
       4.3686480762941763e-04, 
       4.2536927435630643e-04, 
       5.2072883301246969e-04, 
       5.2999999999995607e-04, 
       5.3000000000003186e-04, 
       4.3350364938149851e-04, 
       4.3581569017768880e-04, 
       4.2632785289746224e-04, 
       4.0031050301883928e-04, 
       3.9046176630679768e-04, 
       5.2351934653807662e-04, 
       3.7999096020331261e-04, 
       5.0336446387489017e-04, 
       3.8574316297888031e-04, 
       3.7361793461851840e-04, 
       4.3484949987432489e-04, 
       3.8049491423536297e-04, 
       4.5113272172745881e-04, 
       4.5021998740240915e-04, 
       4.4410613277214315e-04, 
       3.7480734470893030e-04, 
       5.0573993152945324e-04, 
       5.1321508434525531e-04, 
       3.6969461457686276e-04, 
       5.1205037073482967e-04, 
       3.7738099630051222e-04, 
       3.9226843927499738e-04, 
       3.9858985269086353e-04, 
       5.1032824845737251e-04, 
       4.2957090555950330e-04, 
       4.6034227044106465e-04, 
       5.1285702633402754e-04, 
       4.5882818950568680e-04, 
       4.3872746110911620e-04, 
       5.1301161162934164e-04, 
       4.9311073252946628e-04, 
       3.7874492454802698e-04, 
       3.7847888246667641e-04, 
       3.8121610893305797e-04, 
       4.2416932385073183e-04, 
       5.1005632098722275e-04, 
       4.7000000000001804e-04, 
       5.0999999999995071e-04, 
       3.7282554233516806e-04, 
       5.0378624745095447e-04, 
       3.7551728321603858e-04, 
       4.4456657649998552e-04, 
       5.0449773567706876e-04, 
       3.9052176384063572e-04, 
       3.7372919323574294e-04, 
       5.0370485521478105e-04, 
       3.8429008336012145e-04, 
       3.5631024691232032e-04, 
       3.6901055077555273e-04, 
       3.6298017167151213e-04, 
       4.8863471197428232e-04, 
       3.6665941008667987e-04, 
       4.1154273458343833e-04, 
       4.5999999999997199e-04, 
       4.1496683020581654e-04, 
       3.7750039089675064e-04, 
       3.7941307629292461e-04, 
       3.6406988930652286e-04, 
       3.5249725987256174e-04, 
       4.7798043817635851e-04, 
       4.9610424971991297e-04, 
       3.6981261412529057e-04, 
       3.6365571152934857e-04, 
       4.9032687590989065e-04, 
       4.0552389038652170e-04, 
       4.8815765837684373e-04, 
       4.3130501842226896e-04, 
       4.0307702415944756e-04, 
       3.6811934556956442e-04, 
       4.9050611671695135e-04, 
       4.0411028404503527e-04, 
       3.5641780053367070e-04, 
       3.6649024425408584e-04, 
       4.8970312256994407e-04, 
       3.7395020785479399e-04, 
       4.9000000000001159e-04, 
       4.1282322173015164e-04, 
       3.6328856151315246e-04, 
       4.1473232608298680e-04, 
       3.6293816486253599e-04, 
       3.9804907286306322e-04, 
       4.3267325395127782e-04, 
       4.0306881151688583e-04, 
       4.4975164901477803e-04, 
       3.5378950226443059e-04, 
       3.4695208127852496e-04, 
       3.6004725795847229e-04, 
       4.1652396491693837e-04, 
       4.1528934577348889e-04, 
       3.9455003239842945e-04, 
       3.9454326991327730e-04, 
       4.8016855975957355e-04, 
       3.5048348735828682e-04, 
       3.8169147839310413e-04, 
       4.5252464946800411e-04, 
       4.7751641859749145e-04, 
       4.8000000000002300e-04, 
       3.4415483865269991e-04, 
       3.3929201713642730e-04, 
       3.4927088914891074e-04, 
       3.9784336526643742e-04, 
       4.0009861275768307e-04, 
       4.7620363192444389e-04, 
       3.7832406030260241e-04, 
       3.5749410021930934e-04, 
       3.4166967476132040e-04, 
       4.3394398627180018e-04, 
       4.7284955211432443e-04, 
       4.7326156131668529e-04, 
       3.8264763752479246e-04, 
       4.6655137308119262e-04, 
       3.4428533432109181e-04, 
       3.3585236615849939e-04, 
       4.4354223156941236e-04, 
       4.6449417792684039e-04, 
       3.4701173402479666e-04, 
       3.9078619421172370e-04, 
       4.7000000000004726e-04, 
       3.9348658868949962e-04, 
       4.6918652533681644e-04, 
       4.6770622330167308e-04, 
       3.8933883352883648e-04, 
       3.4379558038630443e-04, 
       4.6821882305589873e-04, 
       3.8655641165644258e-04, 
       3.7540854015647454e-04, 
       3.9824750700320142e-04, 
       3.8635205428395566e-04, 
       3.5809932858616328e-04, 
       3.4725769431896318e-04, 
       4.6104904959842373e-04, 
       3.3594472322145074e-04, 
       3.4506185832132316e-04, 
       3.4557319595285963e-04, 
       3.8351112216075933e-04, 
       3.9620144708372198e-04, 
       3.5887751746418458e-04, 
       3.7019497036024594e-04, 
       3.3379879055111947e-04, 
       3.7996195758138794e-04, 
       4.4375048002769855e-04, 
       3.6534344498313705e-04, 
       4.5750684671995792e-04, 
       4.4189201186050237e-04, 
       3.7830891006131819e-04, 
       3.7757558406389881e-04, 
       4.5624709131267396e-04, 
       4.5387657614120259e-04, 
       3.3965495657166384e-04, 
       4.2000000000004848e-04, 
       3.4556834386416114e-04, 
       4.0640416094396456e-04, 
       4.2633949564346781e-04, 
       3.9012731171036578e-04, 
       3.2346383424974650e-04, 
       4.3991871553919433e-04, 
       4.5179916835719685e-04, 
       3.3374218929458228e-04, 
       3.4362949801894203e-04, 
       3.3134463008924230e-04, 
       4.5258579210128639e-04, 
       3.3753565775720458e-04, 
       3.7111877867089727e-04, 
       4.2877542323290678e-04, 
       4.5111790804178874e-04, 
       3.2628388182772425e-04, 
       3.4999999999996015e-04, 
       3.2901684701477603e-04, 
       3.6869926052463762e-04, 
       3.9436106115056701e-04, 
       4.3031960609901161e-04, 
       3.4401063567123387e-04, 
       4.3192521035167325e-04, 
       3.2324945941226497e-04, 
       4.4465713565632910e-04, 
       4.1000000000001398e-04, 
       3.7699094017464095e-04, 
       3.7984686696956811e-04, 
       3.1409974057439264e-04, 
       3.4863451868035172e-04, 
       3.2566805028308866e-04, 
       3.6678925222661304e-04, 
       3.2126662424898975e-04, 
       3.7935896285261285e-04, 
       3.6911235561371464e-04, 
       3.9021637844761426e-04, 
       3.5638001706662622e-04, 
       4.0681847137523949e-04, 
       4.2544332006264637e-04, 
       4.3999999999998972e-04, 
       4.4000000000001276e-04, 
       3.2466211964181005e-04, 
       3.2779809007237758e-04, 
       3.1494606464935876e-04, 
       4.3703168634438117e-04, 
       3.8091814934187421e-04, 
       4.3283201788746768e-04, 
       3.8787651843681800e-04, 
       3.2561725011307733e-04, 
       3.4896724338478899e-04, 
       3.2495154158264464e-04, 
       3.6225093611148265e-04, 
       3.1568024549466760e-04, 
       3.6143260960818014e-04, 
       4.2510603427415265e-04, 
       4.3385960316882953e-04, 
       3.8531393278944847e-04, 
       3.5749477944703424e-04, 
       3.6870974187214574e-04, 
       3.3213261934906738e-04, 
       3.0560295338488426e-04, 
       3.9100461575506090e-04, 
       3.0431737424626486e-04, 
       4.2999999999999273e-04, 
       3.0498524526232577e-04, 
       3.1648420869383583e-04, 
       3.4310638158848431e-04, 
       3.0316722303243347e-04, 
       4.2468000007732322e-04, 
       3.6229437929608937e-04, 
       3.6229437929608937e-04, 
       4.1068159018833541e-04, 
       3.0233612767745829e-04, 
       3.5450063851940239e-04, 
       3.9102547429156249e-04, 
       3.5523308079675394e-04, 
       3.0920048684611500e-04, 
       4.0761764220489314e-04, 
       4.0518425844854084e-04, 
       3.1806916894985886e-04, 
       4.2274689338733698e-04, 
       3.8999999999999067e-04, 
       3.3020903378459761e-04, 
       3.9464665570459583e-04, 
       3.9523425635147450e-04, 
       3.4583428389883282e-04, 
       4.2190765138491450e-04, 
       2.9993749142012525e-04, 
       3.8054820594683221e-04, 
       3.4971193584077532e-04, 
       4.2000026855266260e-04, 
       3.0031509415520707e-04, 
       3.9089040791596519e-04, 
       4.1999999999998961e-04, 
       3.7006480393252946e-04, 
       3.0663182220302975e-04, 
       3.3526062193236714e-04, 
       4.1640772967593533e-04, 
       3.0308883146672834e-04, 
       3.1254090069540144e-04, 
       2.9472137788544712e-04, 
       3.6020895870213270e-04, 
       3.4000000000003472e-04, 
       3.2109655763174893e-04, 
       4.0250305957530597e-04, 
       3.3899636555752695e-04, 
       3.4794351690815720e-04, 
       3.4175621273027580e-04, 
       3.4218966809802966e-04, 
       2.9503886267983378e-04, 
       3.9954299653274301e-04, 
       4.0881484453096238e-04, 
       2.9462646372733635e-04, 
       3.1962967370253710e-04, 
       4.0916865740933791e-04, 
       4.0363701184460541e-04, 
       3.3246309748291461e-04, 
       2.9517166212159778e-04, 
       3.3211203730722368e-04, 
       2.9976127117852777e-04, 
       2.9962457100265864e-04, 
       3.0901888211628327e-04, 
       3.5134690413953369e-04, 
       3.8862221311176449e-04, 
       2.9647004899364444e-04, 
       3.2818275133477201e-04, 
       2.8891741220349071e-04, 
       3.1046815325612312e-04, 
       2.9113647037429160e-04, 
       3.8061239796370437e-04, 
       3.0867548025880506e-04, 
       2.8500346520093456e-04, 
       3.3276501370818232e-04, 
       2.8410819343969144e-04, 
       3.9987957019984704e-04, 
       3.8694621562037512e-04, 
       3.0143634726493835e-04, 
       3.8926025294616304e-04, 
       3.0710365045423190e-04, 
       3.6938699108321460e-04, 
       3.8368959017781286e-04, 
       3.5758997500633620e-04, 
       3.3081229147781274e-04, 
       3.0855474298931645e-04, 
       3.2817976596494479e-04, 
       2.7952638208509386e-04, 
       3.9381244240047796e-04, 
       3.8676878008614673e-04, 
       2.9381030647368116e-04, 
       3.2588111777348891e-04, 
       3.6635421217691980e-04, 
       2.7894220485200516e-04, 
       3.0649671849783964e-04, 
       3.9110586687464871e-04, 
       3.5381848906551811e-04, 
       2.9174441952968630e-04, 
       3.3415315102016438e-04, 
       3.6000000000000994e-04, 
       2.9999999999997737e-04, 
       3.8426121287768797e-04, 
       3.8073136561670991e-04, 
       3.8958927839118410e-04, 
       3.9000000000000287e-04, 
       2.8214784396446424e-04, 
       3.0088389436765611e-04, 
       3.8779687155438334e-04, 
       3.4963531643484788e-04, 
       2.8137422119012555e-04, 
       2.7421931764710065e-04, 
       3.0849228222692141e-04, 
       3.1965920373934470e-04, 
       3.8572753681836899e-04, 
       2.7448948477840714e-04, 
       3.3486206459730229e-04, 
       2.7663595294063087e-04, 
       3.7722244115784287e-04, 
       2.9281249700962995e-04, 
       2.7897608599952977e-04, 
       2.9131770117922983e-04, 
       2.8765955124756735e-04, 
       2.8906932668831240e-04, 
       3.0691812661420284e-04, 
       3.2099300859788482e-04, 
       3.0289971553123482e-04, 
       2.7160777313612190e-04, 
       3.6612342080485910e-04, 
       3.6612342080485910e-04, 
       2.9740316213562617e-04, 
       3.1889647362531094e-04, 
       3.0664754286146038e-04, 
       2.8016299609648435e-04, 
       2.7135893660496820e-04, 
       3.0625801213706274e-04, 
       3.8288858215674784e-04, 
       2.8267628860100230e-04, 
       3.1913651429971802e-04, 
       3.8150128921608708e-04, 
       3.6301116838797142e-04, 
       2.8151476821767802e-04, 
       2.7693572297969870e-04, 
       3.3149348581474751e-04, 
       2.8167691445009482e-04, 
       3.8000000000004333e-04, 
       3.7999999999995991e-04, 
       3.4561737666945446e-04, 
       2.8215031628324737e-04, 
       3.3037861917515102e-04, 
       2.8241555789986926e-04, 
       2.7458111881018026e-04, 
       3.0514562754763126e-04, 
       2.6771912489981480e-04, 
       2.9479614770925943e-04, 
       3.3564992873426813e-04, 
       3.7759116887779276e-04, 
       2.6715740721041981e-04, 
       2.6692470691398246e-04, 
       3.1299548262245691e-04, 
       3.1416260121360564e-04, 
       3.1169933577574214e-04, 
       3.0188191664553795e-04, 
       2.7480980144696526e-04, 
       2.9156714783784083e-04, 
       3.1848479672344032e-04, 
       3.4051495725083273e-04, 
       3.7382931166251269e-04, 
       3.2043127077373837e-04, 
       2.7919957888173302e-04, 
       3.6408584341824447e-04, 
       3.0650447884550424e-04, 
       3.7198119435295291e-04, 
       3.7302819997674222e-04, 
       2.7854053765192415e-04, 
       3.0844451609554944e-04, 
       3.7073895047366572e-04, 
       2.9478656945448211e-04, 
       3.6571251354616037e-04, 
       3.7000000000001832e-04, 
       3.7000000000000552e-04, 
       3.6999999999996693e-04, 
       3.5246577333101516e-04, 
       2.7492168972559447e-04, 
       3.0569055475332163e-04, 
       2.6078377123679247e-04, 
       3.1283694369977965e-04, 
       3.1694605116375132e-04, 
       3.5074808953910211e-04, 
       2.5965631163232540e-04, 
       2.7670948030145349e-04, 
       3.2806998490010993e-04, 
       3.0439119817780798e-04, 
       3.0495973810626230e-04, 
       3.0406448179541953e-04, 
       2.7659993539418218e-04, 
       3.4362802489696520e-04, 
       2.5859028565911691e-04, 
       3.6357865505130132e-04, 
       3.4737802662420828e-04, 
       2.7046637784566212e-04, 
       3.3935755022058083e-04, 
       2.6375306340236325e-04, 
       3.2934349697214590e-04, 
       2.9824123680018090e-04, 
       2.5599522820911016e-04, 
       2.9991806883574731e-04, 
       3.0007926531984820e-04, 
       3.5937201334924733e-04, 
       2.7899408933336370e-04, 
       3.4816528139835226e-04, 
       2.9844607762745352e-04, 
       2.8911871289531634e-04, 
       2.9662497999368646e-04, 
       3.5887096042948237e-04, 
       2.8020289237650479e-04, 
       3.4593780142489194e-04, 
       3.0444131323393951e-04, 
       2.9408002877267316e-04, 
       3.2728219531534905e-04, 
       3.4712901583598243e-04, 
       3.2999999999996926e-04, 
       3.2999999999999810e-04, 
       2.8713073941316272e-04, 
       2.8688388235973534e-04, 
       3.5606451267367533e-04, 
       2.9655072210399116e-04, 
       2.9341123096812130e-04, 
       3.5521480735609318e-04, 
       3.0099523643406951e-04, 
       3.1314529682185092e-04, 
       2.9272363300667230e-04, 
       3.5249238487734022e-04, 
       3.5313912012647879e-04, 
       3.5569994395628582e-04, 
       2.8754646991334946e-04, 
       2.5634625485473683e-04, 
       2.8731810711531776e-04, 
       2.6027226830195610e-04, 
       3.5437447034408905e-04, 
       3.1469807429686041e-04, 
       3.1912931645537555e-04, 
       2.5259661810087295e-04, 
       3.0214384600439371e-04, 
       3.5222101647049744e-04, 
       3.0195971737167473e-04, 
       3.4799739618528007e-04, 
       2.4898705213235051e-04, 
       3.5045890138366991e-04, 
       2.4978017164393475e-04, 
       2.5797853573040163e-04, 
       2.4915607823899075e-04, 
       3.1677859568339078e-04, 
       2.5803099608073988e-04, 
       2.9069671440391857e-04, 
       2.9066771018564124e-04, 
       3.1830700531178279e-04, 
       3.4999999769699385e-04, 
       2.4794726414471260e-04, 
       3.3732816087003471e-04, 
       2.6527961420913249e-04, 
       2.6051878351969851e-04, 
       3.3700471305838487e-04, 
       2.5536333344901139e-04, 
       2.8671790642328308e-04, 
       2.8669020442501558e-04, 
       3.4124573496499772e-04, 
       2.8582322632570068e-04, 
       3.0969902074370171e-04, 
       2.4721853023526336e-04, 
       2.8207641246605770e-04, 
       3.4682568826888946e-04, 
       2.8182053346277526e-04, 
       2.6814795231813163e-04, 
       2.5497960784263128e-04, 
       2.8909975066614103e-04, 
       2.4576159441761912e-04, 
       2.5638292691730592e-04, 
       2.8561026430724861e-04, 
       2.5492030402668187e-04, 
       2.6027403041737989e-04, 
       2.5838285059557768e-04, 
       2.8289158250208857e-04, 
       2.4274309260134423e-04, 
       2.8304774705408860e-04, 
       2.5507152515281636e-04, 
       2.4276001026134260e-04, 
       2.5376060733476935e-04, 
       3.0549910755969882e-04, 
       3.4000000000002903e-04, 
       3.2817309124352333e-04, 
       3.3910557185732638e-04, 
       3.2851443431367830e-04, 
       2.7336818130423428e-04, 
       3.2465002308557550e-04, 
       2.7738414440392576e-04, 
       2.4087147027726104e-04, 
       2.9150648952472362e-04, 
       3.3433376990893763e-04, 
       2.9951565710251496e-04, 
       2.7977205197426633e-04, 
       3.1000000000004955e-04, 
       2.4268232015889037e-04, 
       2.3802194797841364e-04, 
       2.6470237182952031e-04, 
       3.0825936058622713e-04, 
       2.8015356281742716e-04, 
       3.2147369102024126e-04, 
       3.3602549302036910e-04, 
       3.3484208437546984e-04, 
       3.1158679909061755e-04, 
       3.3391006414717767e-04, 
       3.3178100899778734e-04, 
       3.2693076077376101e-04, 
       2.4587742452009288e-04, 
       3.1812078968321283e-04, 
       3.2745255924903239e-04, 
       2.3700147459950475e-04, 
       2.5073292726093040e-04, 
       2.9887809333783869e-04, 
       3.3038772420484845e-04, 
       3.1707043076798988e-04, 
       2.4066309227554762e-04, 
       2.4777933127548166e-04, 
       2.4516787149068210e-04, 
       2.3511166738172704e-04, 
       3.2841301452185854e-04, 
       2.7869539892330132e-04, 
       3.2507895591814972e-04, 
       2.3357568538978686e-04, 
       2.4650579047360703e-04, 
       3.1140347920485920e-04, 
       2.6491554154498741e-04, 
       3.2376801506460082e-04, 
       2.7189686491752409e-04, 
       2.8229210335892663e-04, 
       3.1361141120216714e-04, 
       2.3082065544860486e-04, 
       2.4287971909090668e-04, 
       2.4522064685685241e-04, 
       2.3048933522265363e-04, 
       3.1575511677970583e-04, 
       3.0052560807098408e-04, 
       3.2459944798876280e-04, 
       2.4315299594055041e-04, 
       2.4174443663928835e-04, 
       2.2932777568192018e-04, 
       2.7890801758860827e-04, 
       2.9653001834201241e-04, 
       2.3094116500960404e-04, 
       2.6841527929792294e-04, 
       2.5277492173775626e-04, 
       2.5269835499338721e-04, 
       2.6717973560172616e-04, 
       3.2004608744167000e-04, 
       2.6632850404897712e-04, 
       3.0822882059857381e-04, 
       3.2000000000000653e-04, 
       3.1999999999998197e-04, 
       2.3089197065868895e-04, 
       2.3644409815086988e-04, 
       2.7074578661858339e-04, 
       2.6843930833790760e-04, 
       2.7962867368758703e-04, 
       3.1335435371885494e-04, 
       2.2450961610092723e-04, 
       2.6377300524355469e-04, 
       2.8084648123536245e-04, 
       2.4639082287876787e-04, 
       3.0337575027186285e-04, 
       2.2398101376436432e-04, 
       2.8020949346363569e-04, 
       2.2725948757268074e-04, 
       2.7428192640330051e-04, 
       3.0757570615075217e-04, 
       2.3225671386347513e-04, 
       3.1519259927797821e-04, 
       2.3488097833332126e-04, 
       2.6121542600020958e-04, 
       2.2291161769439906e-04, 
       2.8999999999998558e-04, 
       2.6052650098136622e-04, 
       2.5605953635785862e-04, 
       2.7237736828798721e-04, 
       2.6398979271447686e-04, 
       2.8236004881136142e-04, 
       3.1223286215005525e-04, 
       2.5539065788708751e-04, 
       3.0353943632847115e-04, 
       3.1038683893495087e-04, 
       2.5605996640160224e-04, 
       2.5896238956270355e-04, 
       2.9968534325790512e-04, 
       2.2365817475873470e-04, 
       2.2359923356646434e-04, 
       3.1015214665271510e-04, 
       3.0966685550498830e-04, 
       3.0999999999999930e-04, 
       2.5895803887252878e-04, 
       2.9851050676463529e-04, 
       2.2568114291484124e-04, 
       2.9709160883279633e-04, 
       2.4695987619984672e-04, 
       2.2972314695256108e-04, 
       2.9052133525822317e-04, 
       2.9904633646293203e-04, 
       3.0584688234212338e-04, 
       3.0360145645795468e-04, 
       3.0762337277592464e-04, 
       2.5274598496518799e-04, 
       2.2832578538887165e-04, 
       2.8738117183432993e-04, 
       2.5066580831472799e-04, 
       2.4832087121313978e-04, 
       2.1634505320006252e-04, 
       3.0538632106064405e-04, 
       3.0505566785926706e-04, 
       2.4230240622553020e-04, 
       2.3712398367251383e-04, 
       3.0343700828352343e-04, 
       2.8000000000002844e-04, 
       2.5737851901275920e-04, 
       2.3090345289614232e-04, 
       2.6203615285588111e-04, 
       2.1435480401654007e-04, 
       2.1924543379301094e-04, 
       2.8349422113087177e-04, 
       3.0141984120337067e-04, 
       2.2412405919456285e-04, 
       2.2207074604221875e-04, 
       2.1515266976788883e-04, 
       2.6106557502668062e-04, 
       2.5123036435317954e-04, 
       2.1725486168721526e-04, 
       2.9723781671907562e-04, 
       2.1238431668671234e-04, 
       2.2112537127914842e-04, 
       2.2112537127914842e-04, 
       2.1458115413079128e-04, 
       2.9392665032830972e-04, 
       2.1630592414038688e-04, 
       2.3461817624261140e-04, 
       2.9888191951770479e-04, 
       2.9860314870736341e-04, 
       2.8528510200418475e-04, 
       2.5311792670238483e-04, 
       2.1631506307678700e-04, 
       2.9814447282336485e-04, 
       2.9762722303811544e-04, 
       2.9566304561046548e-04, 
       2.4668199069082089e-04, 
       2.5347531919254313e-04, 
       2.9767807760372087e-04, 
       2.2147411477197186e-04, 
       2.1019704900739519e-04, 
       2.4651670128117799e-04, 
       2.7593190462736541e-04, 
       2.6501804249264466e-04, 
       2.1404698313689599e-04, 
       2.2617384467264227e-04, 
       2.0999775609770074e-04, 
       2.4739478111970598e-04, 
       2.8959766279204840e-04, 
       2.1609022425062596e-04, 
       2.9018886024225268e-04, 
       2.8886942852741866e-04, 
       2.7677290404531528e-04, 
       2.0992450583501480e-04, 
       2.7761865182942186e-04, 
       2.0978827262598020e-04, 
       2.4924106536414474e-04, 
       2.7303486864617855e-04, 
       2.0973602942049916e-04, 
       2.6785537737312554e-04, 
       2.1638339343200199e-04, 
       2.4327563008538471e-04, 
       2.0683234599483158e-04, 
       2.4841495134293111e-04, 
       2.4117361179612615e-04, 
       2.9000000000001079e-04, 
       2.6017217572516302e-04, 
       2.0515327863038685e-04, 
       2.0721931099828851e-04, 
       2.0556349803770266e-04, 
       2.0554151356968740e-04, 
       2.8528680801423055e-04, 
       2.3288250107527647e-04, 
       2.5028076284343997e-04, 
       2.1174982848556030e-04, 
       2.1434862295089240e-04, 
       2.3259812400328294e-04, 
       2.8397343763635383e-04, 
       2.1198630611400692e-04, 
       2.1115566613833081e-04, 
       2.2534745645503380e-04, 
       2.1450551845586980e-04, 
       2.8364337529678224e-04, 
       2.1431854552746748e-04, 
       2.0916524716954076e-04, 
       2.4302346306440269e-04, 
       2.0345120414716255e-04, 
       2.8331966226812102e-04, 
       2.6863593299367573e-04, 
       2.0024761566247504e-04, 
       2.0848933855210344e-04, 
       2.1091351656094356e-04, 
       2.2911849871256159e-04, 
       2.2911238583062327e-04, 
       2.2909496872899104e-04, 
       2.2906657422364164e-04, 
       2.8167477541661887e-04, 
       2.4338143406680001e-04, 
       2.1185468489916734e-04, 
       2.4218780750653082e-04, 
       1.9936919697178377e-04, 
       2.2314343876869976e-04, 
       2.3366023713747571e-04, 
       2.3343471968429743e-04, 
       2.3374951714629722e-04, 
       2.8017560111609003e-04, 
       2.2992715607177285e-04, 
       2.8000000000000550e-04, 
       2.0309289098210270e-04, 
       2.7715103831956562e-04, 
       1.9952169240559857e-04, 
       2.3330012969725899e-04, 
       2.7922095171093451e-04, 
       2.1364682217870062e-04, 
       2.1299978282829338e-04, 
       2.3046635563901333e-04, 
       2.0900159873150136e-04, 
       2.3214170847214434e-04, 
       2.7898160468441292e-04, 
       2.0514147114674712e-04, 
       2.5065691003149014e-04, 
       2.7874021935346929e-04, 
       2.0160487597765896e-04, 
       2.7818511283130178e-04, 
       2.3084941794465844e-04, 
       2.5204189942460521e-04, 
       1.9670133688614192e-04, 
       2.7698205436740039e-04, 
       2.7580624146710219e-04, 
       2.0000000000005403e-04, 
       2.3705611681340094e-04, 
       1.9895410695235319e-04, 
       2.0446126622335097e-04, 
       2.0105913674654178e-04, 
       2.3636704243516863e-04, 
       2.7621406998876229e-04, 
       2.2229769886413230e-04, 
       2.2208451003023378e-04, 
       2.6643808870793675e-04, 
       2.3025909167680733e-04, 
       1.9492550991013027e-04, 
       2.4192778008061972e-04, 
       2.0442435745484692e-04, 
       1.9962559246435989e-04, 
       2.1000000000003473e-04, 
       2.0905328438121455e-04, 
       2.6099572557923708e-04, 
       2.7310320761013933e-04, 
       2.5788766370069325e-04, 
       2.7291701595589103e-04, 
       1.9723761951432405e-04, 
       2.1394067147785132e-04, 
       2.1389756278305462e-04, 
       2.0702013604190729e-04, 
       2.6764001294480455e-04, 
       2.6520765864910634e-04, 
       2.5928552326029072e-04, 
       1.9861945820817082e-04, 
       2.7216351050362138e-04, 
       2.3331554898973269e-04, 
       2.7151342561362364e-04, 
       2.4000000000004302e-04, 
       2.6904339026614867e-04, 
       2.6505583274631140e-04, 
       2.2522115025084503e-04, 
       1.9500357488339188e-04, 
       2.6010834827173729e-04, 
       2.6504551557715052e-04, 
       2.2869717402480120e-04, 
       2.2485528379923383e-04, 
       2.7000000000000000e-04, 
       2.2386096534236785e-04, 
       2.5580623644758349e-04, 
       2.0464179668304926e-04, 
       2.6924910537960239e-04, 
       2.4254778282210198e-04, 
       1.9463148864945034e-04, 
       1.9781152382694246e-04, 
       2.0380731358099737e-04, 
       2.1973938385810423e-04, 
       1.9344008981482465e-04, 
       2.5132282861364618e-04, 
       1.9946490893906532e-04, 
       2.1578959310608205e-04, 
       2.5025050850719808e-04, 
       2.6200483995390442e-04, 
       2.0561131681610842e-04, 
       1.8883291333670043e-04, 
       1.9429893762068129e-04, 
       1.9272130201738273e-04, 
       1.9348718208694800e-04, 
       2.1554101245122405e-04, 
       2.6575518573939447e-04, 
       2.5486622334928587e-04, 
       2.6145686979963951e-04, 
       2.2502864101785677e-04, 
       2.5964453700109741e-04, 
       2.2101643408883648e-04, 
       2.4221237934686190e-04, 
       2.3778003530802399e-04, 
       2.5399332543976828e-04, 
       1.9716912722749682e-04, 
       1.9687219187547033e-04, 
       1.8959717071523780e-04, 
       2.5472928566045626e-04, 
       2.1724382860670638e-04, 
       1.8798867080580410e-04, 
       2.6423573849746802e-04, 
       1.9112777204590629e-04, 
       2.0566209860903517e-04, 
       2.4842904558404765e-04, 
       2.0146250440383298e-04, 
       2.2608830617710314e-04, 
       2.1931749048673642e-04, 
       2.1166511424257006e-04, 
       2.6227468709101451e-04, 
       2.1469533439434928e-04, 
       2.0045334002944911e-04, 
       1.9258916104623593e-04, 
       1.9706220650144118e-04, 
       1.9229210893188927e-04, 
       2.2240106596626248e-04, 
       1.9478037161768579e-04, 
       2.3980586111792431e-04, 
       2.5999999999999949e-04, 
       2.5999999999995368e-04, 
       2.5999999999997656e-04, 
       2.6000000000002209e-04, 
       1.8356215622032768e-04, 
       2.5859209219964373e-04, 
       2.5019722600032118e-04, 
       1.9000000000003536e-04, 
       1.9364581385713668e-04, 
       2.0201482444573603e-04, 
       1.8711443864015038e-04, 
       2.5839651898496480e-04, 
       2.1036175554386041e-04, 
       1.8310049852630499e-04, 
       2.1323628006723061e-04, 
       2.5734716057206281e-04, 
       2.5743089644910885e-04, 
       1.8768844466384484e-04, 
       2.5721041159143667e-04, 
       2.2873284046830283e-04, 
       1.8381419059783772e-04, 
       1.8373054105177259e-04, 
       1.9208715411208081e-04, 
       1.9136383030135762e-04, 
       2.4613648827803301e-04, 
       1.8398793526488660e-04, 
       1.9148170962265894e-04, 
       1.8991932211251348e-04, 
       1.8301718056369935e-04, 
       2.0916589911031916e-04, 
       2.0916589911031916e-04, 
       2.5465622481112135e-04, 
       2.1042920538520888e-04, 
       2.3357907449821666e-04, 
       2.0157446081531952e-04, 
       2.5293798147816871e-04, 
       2.4680136085885587e-04, 
       1.8962725726023034e-04, 
       2.5352214108298379e-04, 
       2.5401006307258008e-04, 
       1.8506585105422172e-04, 
       2.5329975650256362e-04, 
       2.4570329357452949e-04, 
       2.1040019047558639e-04, 
       1.8100710744048041e-04, 
       2.4894750380896900e-04, 
       1.9266079420220857e-04, 
       2.0101547414854456e-04, 
       1.9154588585395057e-04, 
       2.0976045664812903e-04, 
       1.9767801580194218e-04, 
       2.3173613411575305e-04, 
       2.4961140387494765e-04, 
       1.8484834758946816e-04, 
       1.8795832026021766e-04, 
       2.1694108841672688e-04, 
       2.0031017447221213e-04, 
       2.5166341147980797e-04, 
       2.5000000000002429e-04, 
       1.8772357666824939e-04, 
       2.3070723835942681e-04, 
       1.9138619915697245e-04, 
       2.1676097639129112e-04, 
       2.3388007789404950e-04, 
       1.9792845493405998e-04, 
       2.4016847055572836e-04, 
       1.8288080989428381e-04, 
       2.4487984569213452e-04, 
       1.8685500154867415e-04, 
       1.8481437984612945e-04, 
       2.0127925897985981e-04, 
       2.4999999999997138e-04, 
       2.5000000000003481e-04, 
       2.2525195662859084e-04, 
       2.3000000000001971e-04, 
       1.9013709593681318e-04, 
       2.4286708485678668e-04, 
       2.0180861813451584e-04, 
       2.0180672383886332e-04, 
       2.4847692963982462e-04, 
       1.7961297366195908e-04, 
       1.8027247927328904e-04, 
       1.8488801166964967e-04, 
       2.0572120417329773e-04, 
       2.3965680206806559e-04, 
       2.4084883706121680e-04, 
       2.1004901317977760e-04, 
       1.8153011301996617e-04, 
       1.8408379737573766e-04, 
       1.7676503122597041e-04, 
       2.2367105859910202e-04, 
       1.9160693559992729e-04, 
       2.3268751928464093e-04, 
       1.9739136053101354e-04, 
       1.7820931170299865e-04, 
       2.0299558651710127e-04, 
       2.1091076571287041e-04, 
       1.9663308896173030e-04, 
       1.8180407058903316e-04, 
       2.0211345099280587e-04, 
       1.9600352025286717e-04, 
       2.4248445796108396e-04, 
       2.0172764377250673e-04, 
       1.9606317040211064e-04, 
       1.7599469631064931e-04, 
       1.7551403376186970e-04, 
       1.8482625573995801e-04, 
       2.0168528173187205e-04, 
       2.2502757509467014e-04, 
       1.8432344184098528e-04, 
       2.2300736763798123e-04, 
       1.8482567181899991e-04, 
       1.9997105137443404e-04, 
       2.1068342965162808e-04, 
       1.7128717857942222e-04, 
       1.7606127134602666e-04, 
       1.8033206423979982e-04, 
       2.2406408205660682e-04, 
       1.7827688212606363e-04, 
       2.3999999999999374e-04, 
       2.4000000000003950e-04, 
       1.7194860158688147e-04, 
       2.3782175769209788e-04, 
       2.0580680802089358e-04, 
       2.2000000000002351e-04, 
       2.1999999999995057e-04, 
       1.8658551422499358e-04, 
       2.3848118139772933e-04, 
       1.9367870971396595e-04, 
       1.9359689611873494e-04, 
       1.7728799875442106e-04, 
       2.3757420529788774e-04, 
       2.3754082277284175e-04, 
       1.7113888392909134e-04, 
       1.9749632742612429e-04, 
       2.1386488450319434e-04, 
       1.6778577499292386e-04, 
       1.7037364839777219e-04, 
       2.2704032547892812e-04, 
       2.3586279707232444e-04, 
       1.7281029799006583e-04, 
       2.0240905965637266e-04, 
       1.8398817698172149e-04, 
       1.6700703291699362e-04, 
       1.9583744192206410e-04, 
       1.7014302685374860e-04, 
       2.0059481787996570e-04, 
       1.8343730957174562e-04, 
       1.8346386811259903e-04, 
       1.6657129575150018e-04, 
       1.9409851180722372e-04, 
       1.7253911035893676e-04, 
       1.9434302097054112e-04, 
       1.6754071133750080e-04, 
       2.1430277254518254e-04, 
       1.6578532583570052e-04, 
       2.0686107107174050e-04, 
       2.1639676152145618e-04, 
       1.6966422399977850e-04, 
       1.9856731168577834e-04, 
       1.7121011482933504e-04, 
       1.9311546664742466e-04, 
       1.9379311619335080e-04, 
       2.2524652452152175e-04, 
       1.9799343492063197e-04, 
       2.0928092440440630e-04, 
       2.1903565740537752e-04, 
       2.2796511814887800e-04, 
       1.7439721721815556e-04, 
       2.2192173248291965e-04, 
       1.6784231918726031e-04, 
       1.6779805311004681e-04, 
       1.7167751827408282e-04, 
       2.3000000000001670e-04, 
       2.2999999999998298e-04, 
       2.0663896975973316e-04, 
       2.2630973133981694e-04, 
       1.9036427087779122e-04, 
       1.9498745670566302e-04, 
       1.9902389589481720e-04, 
       1.6813993766169713e-04, 
       2.2846232950801391e-04, 
       1.6614311356140298e-04, 
       2.1569629773348850e-04, 
       2.2670972471170632e-04, 
       2.0308133740099354e-04, 
       2.1573006537370509e-04, 
       2.0397043023891146e-04, 
       1.6196613886578174e-04, 
       1.9628123524880882e-04, 
       1.9516646448487890e-04, 
       1.6106585346314643e-04, 
       2.2593342332910619e-04, 
       1.6875879843956349e-04, 
       1.6249695851851690e-04, 
       1.7949644697835934e-04, 
       1.6264942576091197e-04, 
       1.6596949809967229e-04, 
       1.8713871021210196e-04, 
       1.6133453946285078e-04, 
       1.7114258491218544e-04, 
       1.7114258491218544e-04, 
       2.0084523783786906e-04, 
       2.0760747986977047e-04, 
       1.6069187677590830e-04, 
       2.2123784216931255e-04, 
       2.1989015474474825e-04, 
       2.2047863461495537e-04, 
       1.5701756970806815e-04, 
       1.6341026310311629e-04, 
       2.1143416979387201e-04, 
       1.8968520117372876e-04, 
       1.8126305538718947e-04, 
       1.6837003599658271e-04, 
       1.8409924372147897e-04, 
       2.1372596846036524e-04, 
       1.7120706074330927e-04, 
       2.2078081790338541e-04, 
       1.9369576697416860e-04, 
       2.1999991951612594e-04, 
       1.8440862727032401e-04, 
       2.1333411544866904e-04, 
       2.1360376873581611e-04, 
       2.2000000000004161e-04, 
       2.1999999999999133e-04, 
       2.1999999999999133e-04, 
       2.1707322430076679e-04, 
       2.1298648841903698e-04, 
       1.5753144099904970e-04, 
       1.6597891921253449e-04, 
       1.8000065076192084e-04, 
       1.8801528663936920e-04, 
       2.1328540252956699e-04, 
       1.6197988358398314e-04, 
       1.8323293887972743e-04, 
       2.0869137654530633e-04, 
       1.8518844177768955e-04, 
       1.9393073650336311e-04, 
       1.6144558566296582e-04, 
       1.7846965619131784e-04, 
       2.0264525529399853e-04, 
       2.0805646986402654e-04, 
       1.9369900378675813e-04, 
       1.7837547554451263e-04, 
       2.0326193386622010e-04, 
       1.8128772472467955e-04, 
       1.8534740672349764e-04, 
       1.5347791268923233e-04, 
       2.1307517255897068e-04, 
       1.6336809708337758e-04, 
       1.8806922083444393e-04, 
       2.0767041307857584e-04, 
       1.5283361663515540e-04, 
       2.1537183432233329e-04, 
       1.7977673232277746e-04, 
       1.8438286946670374e-04, 
       1.6710998590861655e-04, 
       2.0427732878232208e-04, 
       1.6337922945254524e-04, 
       1.7572142335229527e-04, 
       2.1221657390811666e-04, 
       1.5233607284881686e-04, 
       1.5220885917372449e-04, 
       1.7689560729984970e-04, 
       1.5418200866316962e-04, 
       1.5227018320278137e-04, 
       1.5175073934814036e-04, 
       2.0533123985914958e-04, 
       1.7714158346072268e-04, 
       1.9180318237326400e-04, 
       1.5364753948768174e-04, 
       1.8119700716329369e-04, 
       1.5304767319383489e-04, 
       1.5127112547973673e-04, 
       1.7453450646194251e-04, 
       1.7716135584757781e-04, 
       2.0497458057686503e-04, 
       1.6296616040678482e-04, 
       1.9323800624503843e-04, 
       1.7572567081883630e-04, 
       1.8016309052955936e-04, 
       2.1000000000004831e-04, 
       2.0999999999999453e-04, 
       1.7335398996921352e-04, 
       1.7949889001366091e-04, 
       1.7314943335367959e-04, 
       1.5717959413720259e-04, 
       1.9720482675601776e-04, 
       1.5445888500134557e-04, 
       1.8065464652859621e-04, 
       1.5700951644076052e-04, 
       1.5735326243464506e-04, 
       1.8736512259755672e-04, 
       1.5745823773509072e-04, 
       1.9828663629513409e-04, 
       1.6125795868505551e-04, 
       1.5265868395262572e-04, 
       1.9828243267669697e-04, 
       1.4564441896196820e-04, 
       1.6606385481762100e-04, 
       1.5694357160350017e-04, 
       2.0521315031001060e-04, 
       1.4835067256444091e-04, 
       1.8511250137498648e-04, 
       1.6563857520539497e-04, 
       2.0355046987827080e-04, 
       1.7084012408324963e-04, 
       1.8545597254881769e-04, 
       1.6078200857911208e-04, 
       1.7703171465073157e-04, 
       1.6808093371687422e-04, 
       2.0195387893156945e-04, 
       1.5276384361833517e-04, 
       1.7048108205428955e-04, 
       2.0393319663821084e-04, 
       1.7143634319190904e-04, 
       1.7334440029080007e-04, 
       1.5283624272491108e-04, 
       1.9638733974860508e-04, 
       1.5064748209291922e-04, 
       1.7273926036486944e-04, 
       1.4965243828875060e-04, 
       1.6100114638768894e-04, 
       1.8695592187965840e-04, 
       1.4935776078734252e-04, 
       1.9509952131857431e-04, 
       1.7301214719603949e-04, 
       1.4921049122693192e-04, 
       1.6806253229984327e-04, 
       1.4582371638546099e-04, 
       1.4505387186485862e-04, 
       1.6832711986389666e-04, 
       1.5824284812600129e-04, 
       1.4741234619142368e-04, 
       1.5003215934331006e-04, 
       1.4974491042026082e-04, 
       1.5448484685818759e-04, 
       2.0032291558471434e-04, 
       1.6776577785265344e-04, 
       1.8136899974741127e-04, 
       1.5045160886940456e-04, 
       1.6809081098942115e-04, 
       1.7391542753809186e-04, 
       1.7787694873044356e-04, 
       1.8925965442797522e-04, 
       1.5204199940079330e-04, 
       1.9333779417638925e-04, 
       1.9491434041936817e-04, 
       1.7015273483895194e-04, 
       1.6405278639003708e-04, 
       1.9591287644365099e-04, 
       1.6048840250699017e-04, 
       1.7090614880101718e-04, 
       1.5750053342482282e-04, 
       1.6224700211592231e-04, 
       1.4094048254804532e-04, 
       1.6418655247579252e-04, 
       1.5191581236513751e-04, 
       1.5085155884586116e-04, 
       1.9580975158241037e-04, 
       1.3977335124626779e-04, 
       1.8323301557543326e-04, 
       1.5868862222345300e-04, 
       1.3947099445262327e-04, 
       1.5331469205838718e-04, 
       1.9142210207546905e-04, 
       1.8527434238516005e-04, 
       1.6423658423509181e-04, 
       1.6371179391080153e-04, 
       1.4479878831178113e-04, 
       1.5448344415224405e-04, 
       1.9568244897546656e-04, 
       1.4965676015682653e-04, 
       1.6013967350063873e-04, 
       1.4199376076226629e-04, 
       1.4585282526477858e-04, 
       1.6905484986574012e-04, 
       1.5717477095481759e-04, 
       1.6129215428418010e-04, 
       1.5697641123787976e-04, 
       1.4058099612704894e-04, 
       1.8000000000003544e-04, 
       1.3808969425964352e-04, 
       1.6861407831494856e-04, 
       1.5127906401009260e-04, 
       1.8472270766498755e-04, 
       1.4042563149564810e-04, 
       1.8790454708843344e-04, 
       1.5560631326421470e-04, 
       1.6847647587883840e-04, 
       1.3814552934722073e-04, 
       1.5664767574528427e-04, 
       1.7673924693338515e-04, 
       1.6808456829862716e-04, 
       1.4722811629480642e-04, 
       1.7236350374315779e-04, 
       1.8670858580513135e-04, 
       1.7350883799623247e-04, 
       1.9336633401777217e-04, 
       1.7146228469559210e-04, 
       1.6074939472316392e-04, 
       1.6533497982448300e-04, 
       1.8837247007247047e-04, 
       1.4267385893685446e-04, 
       1.3697985715381976e-04, 
       1.6061452591610686e-04, 
       1.6901020225214402e-04, 
       1.5557476908500950e-04, 
       1.4043569529280711e-04, 
       1.3952728582179396e-04, 
       1.6269071104143640e-04, 
       1.7717482328486722e-04, 
       1.5583047187098386e-04, 
       1.5963586966641111e-04, 
       1.3627273806984973e-04, 
       1.4478112724730402e-04, 
       1.3800044333467644e-04, 
       1.6869240673341650e-04, 
       1.8468641086114192e-04, 
       1.5425360568931346e-04, 
       1.8147721078687896e-04, 
       1.8698054862125639e-04, 
       1.8143273047194322e-04, 
       1.4279758432263700e-04, 
       1.7459135458627894e-04, 
       1.8144070405429834e-04, 
       1.8999999999997716e-04, 
       1.7235785412461206e-04, 
       1.9000000000003186e-04, 
       1.3831394326416246e-04, 
       1.3401160986803082e-04, 
       1.8534744373115895e-04, 
       1.8797330621865912e-04, 
       1.7762252426968862e-04, 
       1.3415369015985274e-04, 
       1.3884429998533781e-04, 
       1.5539414766437560e-04, 
       1.8542381956917657e-04, 
       1.5651328186031089e-04, 
       1.4378668797443772e-04, 
       1.8743270906176014e-04, 
       1.8727409477743440e-04, 
       1.6189903351130369e-04, 
       1.8472843876819723e-04, 
       1.3994167428429469e-04, 
       1.5514367441577396e-04, 
       1.4360481043503126e-04, 
       1.6668915002311486e-04, 
       1.7681397782802461e-04, 
       1.3301531827745097e-04, 
       1.5503167893186795e-04, 
       1.6118269620053196e-04, 
       1.7981435638704521e-04, 
       1.3358004611482271e-04, 
       1.5697837275173866e-04, 
       1.7272039144944563e-04, 
       1.4315926172950912e-04, 
       1.6366238677552582e-04, 
       1.7111469344669800e-04, 
       1.6069172127420233e-04, 
       1.3313544107346838e-04, 
       1.3830524638874900e-04, 
       1.8394363317028101e-04, 
       1.8469945883789560e-04, 
       1.6997257040558299e-04, 
       1.3254140787305666e-04, 
       1.3162838125625883e-04, 
       1.3402651286070901e-04, 
       1.6424848298203311e-04, 
       1.7155182767531637e-04, 
       1.3435871167122207e-04, 
       1.5864868114251216e-04, 
       1.3020900735620942e-04, 
       1.3748199961562916e-04, 
       1.8122640806682297e-04, 
       1.7797461268866533e-04, 
       1.2995120570590866e-04, 
       1.8224854038035373e-04, 
       1.5372929952838741e-04, 
       1.7147780525383379e-04, 
       1.5692299398344675e-04, 
       1.8000126461232199e-04, 
       1.3471820445830121e-04, 
       1.7265866611622849e-04, 
       1.6510178981336617e-04, 
       1.5167074939250041e-04, 
       1.3645675563697896e-04, 
       1.4905125226267015e-04, 
       1.8206135548343532e-04, 
       1.3150851948068703e-04, 
       1.3001821260680025e-04, 
       1.3304357075601605e-04, 
       1.8053164075882628e-04, 
       1.8149866232012888e-04, 
       1.3098901525830566e-04, 
       1.8135424313909349e-04, 
       1.6649003290360187e-04, 
       1.8067195417206176e-04, 
       1.3823177827332168e-04, 
       1.3382103744599041e-04, 
       1.7107543102580782e-04, 
       1.3998058491970205e-04, 
       1.3984730741739923e-04, 
       1.2961744980718995e-04, 
       1.8004326891682138e-04, 
       1.5110296839731299e-04, 
       1.4493623656794641e-04, 
       1.7947005326787721e-04, 
       1.7237121244733896e-04, 
       1.6539622910215375e-04, 
       1.4877349471269875e-04, 
       1.2673987228163389e-04, 
       1.3578653978647267e-04, 
       1.4868519039975059e-04, 
       1.5230974562581280e-04, 
       1.5017886330910655e-04, 
       1.3297488089593275e-04, 
       1.5899921607069256e-04, 
       1.5368384186934469e-04, 
       1.3191312718210553e-04, 
       1.2696407626514773e-04, 
       1.5170994093807651e-04, 
       1.3167433558691549e-04, 
       1.3746257606846103e-04, 
       1.4056239654250532e-04, 
       1.7395895491990708e-04, 
       1.5250659308393866e-04, 
       1.3851355897702617e-04, 
       1.7248987842557044e-04, 
       1.2545247401294949e-04, 
       1.7652812379601683e-04, 
       1.7639098637550879e-04, 
       1.2558842826441554e-04, 
       1.5258696157181574e-04, 
       1.6314205839531795e-04, 
       1.5010429102565870e-04, 
       1.6692316310591026e-04, 
       1.7623471534650823e-04, 
       1.3638480619105619e-04, 
       1.2698334822981296e-04, 
       1.4563891638638389e-04, 
       1.6316837348210196e-04, 
       1.5989132434140386e-04, 
       1.4594466510570853e-04, 
       1.3705092549703851e-04, 
       1.2699458511659080e-04, 
       1.2578804470588149e-04, 
       1.3459641849085757e-04, 
       1.5100790947549653e-04, 
       1.7503822150888995e-04, 
       1.6690659211152781e-04, 
       1.4933343583298399e-04, 
       1.5015103978596137e-04, 
       1.6997398980867585e-04, 
       1.7147897669109009e-04, 
       1.4078683743712621e-04, 
       1.2456807174991645e-04, 
       1.4897632606892931e-04, 
       1.2508584143665360e-04, 
       1.6733846333524217e-04, 
       1.4979503116421789e-04, 
       1.2471487068012180e-04, 
       1.7312076462215172e-04, 
       1.3377824080753448e-04, 
       1.5078179423595403e-04, 
       1.2357885193811562e-04, 
       1.2281095327576060e-04, 
       1.7317601628547564e-04, 
       1.7228636586531868e-04, 
       1.6662503829660577e-04, 
       1.4354841240928103e-04, 
       1.6329295804790758e-04, 
       1.5463155548125698e-04, 
       1.4272229011325714e-04, 
       1.7148373939618452e-04, 
       1.7044889280124891e-04, 
       1.2478870530050501e-04, 
       1.2526124135798093e-04, 
       1.6987321698639615e-04, 
       1.2205088605318080e-04, 
       1.2259412616567945e-04, 
       1.2624285360254373e-04, 
       1.2156779067705010e-04, 
       1.2260718398569476e-04, 
       1.6495748037921964e-04, 
       1.7050329921991183e-04, 
       1.5769812062799596e-04, 
       1.3922113718203529e-04, 
       1.2402815308644842e-04, 
       1.4142898124632093e-04, 
       1.3113536514707216e-04, 
       1.7000000000000671e-04, 
       1.6999999999995700e-04, 
       1.6999999999997120e-04, 
       1.6999999999995242e-04, 
       1.7000000000001316e-04, 
       1.7000000000004143e-04, 
       1.5836635948176370e-04, 
       1.4323182400550918e-04, 
       1.2051280209077070e-04, 
       1.6933020785728678e-04, 
       1.2214096917977907e-04, 
       1.4297238978008749e-04, 
       1.3255724451334550e-04, 
       1.4082411518446234e-04, 
       1.2104567899002264e-04, 
       1.1998948684221611e-04, 
       1.5124098756580422e-04, 
       1.6297769182487237e-04, 
       1.6108842966683373e-04, 
       1.6673915867429089e-04, 
       1.2583719669620791e-04, 
       1.3555217026789678e-04, 
       1.2100509060378924e-04, 
       1.4374892135746671e-04, 
       1.4034065971763686e-04, 
       1.1911023529752493e-04, 
       1.3246510902477780e-04, 
       1.2100694943255494e-04, 
       1.3235372098767631e-04, 
       1.6659741249805583e-04, 
       1.3235629857088934e-04, 
       1.4563045364439053e-04, 
       1.2282316378989561e-04, 
       1.6018428305646017e-04, 
       1.3660934692626177e-04, 
       1.1930483914899852e-04, 
       1.2672233416070899e-04, 
       1.5255282676010178e-04, 
       1.3791111490004613e-04, 
       1.5351099714353589e-04, 
       1.6512026502715405e-04, 
       1.3472763625454182e-04, 
       1.3461814473396763e-04, 
       1.2651014792078676e-04, 
       1.3221547772756609e-04, 
       1.5501714579662840e-04, 
       1.3765821596946134e-04, 
       1.1999999999996169e-04, 
       1.6212370885442713e-04, 
       1.2254781805196249e-04, 
       1.1875609529350585e-04, 
       1.6032233847649378e-04, 
       1.2137448150433576e-04, 
       1.3890794005146301e-04, 
       1.3646503948404294e-04, 
       1.2667737743892604e-04, 
       1.4293205659205322e-04, 
       1.3376305077388991e-04, 
       1.5410257724665451e-04, 
       1.3360368511948490e-04, 
       1.1948612455100513e-04, 
       1.6406438658236484e-04, 
       1.4341839693145545e-04, 
       1.2553963701827516e-04, 
       1.2080205470342870e-04, 
       1.1624139046844870e-04, 
       1.5009564732489745e-04, 
       1.2874764157934303e-04, 
       1.5515271639643104e-04, 
       1.6249546908462983e-04, 
       1.4537388368918969e-04, 
       1.3727164075298123e-04, 
       1.2134573984300316e-04, 
       1.2866004553685226e-04, 
       1.3137749334912304e-04, 
       1.6202024977959493e-04, 
       1.3447652426224572e-04, 
       1.2080868968142446e-04, 
       1.3307361871846092e-04, 
       1.3673285286282282e-04, 
       1.1916645625928418e-04, 
       1.2779200002936946e-04, 
       1.1504884334514827e-04, 
       1.3651700955581025e-04, 
       1.3918573043331052e-04, 
       1.1884542819529089e-04, 
       1.2354947509355887e-04, 
       1.2353154213487250e-04, 
       1.5078092705759824e-04, 
       1.3430740748761796e-04, 
       1.3385917438640268e-04, 
       1.6085911131692859e-04, 
       1.2917038788256934e-04, 
       1.5441962016020234e-04, 
       1.5806398769554927e-04, 
       1.3928849493649847e-04, 
       1.2729276498288231e-04, 
       1.2490473825607382e-04, 
       1.5794518797378760e-04, 
       1.5999999999996792e-04, 
       1.5999999999998429e-04, 
       1.6000000000004100e-04, 
       1.3283825786922186e-04, 
       1.5868968904286855e-04, 
       1.2237346123927174e-04, 
       1.3106978826400413e-04, 
       1.1710718530496936e-04, 
       1.5291352160660453e-04, 
       1.3234232024356509e-04, 
       1.2528599539701659e-04, 
       1.4031607694761682e-04, 
       1.2182659129210776e-04, 
       1.3240804547160070e-04, 
       1.3581708875018016e-04, 
       1.2755391665463244e-04, 
       1.2340626388959849e-04, 
       1.1469621817116149e-04, 
       1.1566233929243237e-04, 
       1.1409883448653354e-04, 
       1.4123653097838760e-04, 
       1.5306744284361730e-04, 
       1.1620277443036803e-04, 
       1.1620277443036803e-04, 
       1.1999999999997950e-04, 
       1.1831940008078334e-04, 
       1.3903738383606076e-04, 
       1.1649744245099836e-04, 
       1.2667072595207900e-04, 
       1.1684692351786564e-04, 
       1.5622430796997811e-04, 
       1.2982310334060259e-04, 
       1.4294858053958301e-04, 
       1.1812914898764387e-04, 
       1.3822158911563733e-04, 
       1.1084622685662520e-04, 
       1.5519342601126037e-04, 
       1.1452998891652252e-04, 
       1.5199086443017207e-04, 
       1.1261947678254614e-04, 
       1.1046683477145898e-04, 
       1.1317622865794402e-04, 
       1.1493464597422978e-04, 
       1.4129601915754080e-04, 
       1.4129071708555689e-04, 
       1.2636683167190177e-04, 
       1.2832091074941178e-04, 
       1.1794897989295466e-04, 
       1.1307728290494497e-04, 
       1.1754348014736108e-04, 
       1.5337477260875311e-04, 
       1.3255702051876312e-04, 
       1.1696579807786919e-04, 
       1.4014974873959748e-04, 
       1.4763496174399716e-04, 
       1.2110334345667254e-04, 
       1.1069535932945421e-04, 
       1.0857712965206902e-04, 
       1.4184203835657574e-04, 
       1.0944887232362502e-04, 
       1.2514575060135780e-04, 
       1.2725141563258173e-04, 
       1.2622924458508713e-04, 
       1.2482671455285517e-04, 
       1.3930521400124307e-04, 
       1.5127372387559298e-04, 
       1.1261112535771039e-04, 
       1.2669553007819232e-04, 
       1.4347562507722832e-04, 
       1.4347562507722832e-04, 
       1.5199242142969286e-04, 
       1.2538101989688930e-04, 
       1.1852693085166877e-04, 
       1.4735630410222568e-04, 
       1.2600890317514653e-04, 
       1.3999999999995732e-04, 
       1.0817077410181477e-04, 
       1.2470288317636150e-04, 
       1.5133687805832794e-04, 
       1.3563376561304285e-04, 
       1.1133216725428975e-04, 
       1.3195787263301729e-04, 
       1.2570346163300741e-04, 
       1.0693303412412506e-04, 
       1.0740299411521515e-04, 
       1.1481724495420005e-04, 
       1.1373300665408960e-04, 
       1.0747587535989394e-04, 
       1.5075339249831485e-04, 
       1.4975515597021114e-04, 
       1.2000000000001801e-04, 
       1.1388755294784691e-04, 
       1.2084706312818877e-04, 
       1.4862283122155624e-04, 
       1.1765421926424346e-04, 
       1.2966397099610125e-04, 
       1.0941624356534514e-04, 
       1.1250621571916749e-04, 
       1.2895476011064269e-04, 
       1.0639779180408200e-04, 
       1.1844757370983131e-04, 
       1.5000000000000552e-04, 
       1.5000000000001682e-04, 
       1.2014835262670070e-04, 
       1.3378991976003242e-04, 
       1.2892809094973849e-04, 
       1.4966869162338458e-04, 
       1.2539476347891399e-04, 
       1.3768223742472233e-04, 
       1.0774047392095315e-04, 
       1.4918791531666814e-04, 
       1.0720063407423534e-04, 
       1.4365571365526666e-04, 
       1.1956684471295402e-04, 
       1.2940163648088166e-04, 
       1.4651833680541941e-04, 
       1.1230975415157783e-04, 
       1.1372775570048073e-04, 
       1.2006199351089215e-04, 
       1.4076754474915380e-04, 
       1.3370040941257624e-04, 
       1.3918162591177656e-04, 
       1.4438313787837632e-04, 
       1.4000000000002617e-04, 
       1.0442461594807061e-04, 
       1.0467528665442617e-04, 
       1.2542789901083757e-04, 
       1.0514410533665720e-04, 
       1.0635545593445169e-04, 
       1.2521171451199475e-04, 
       1.0978616453647124e-04, 
       1.2412340541638551e-04, 
       1.4131568193382252e-04, 
       1.0534815337802291e-04, 
       1.0818835832184637e-04, 
       1.2141816739885772e-04, 
       1.3151650030644637e-04, 
       1.3408230692314239e-04, 
       1.4367011318689818e-04, 
       1.1623134547530660e-04, 
       1.1548496966378767e-04, 
       1.1837479082656069e-04, 
       1.4494905168930411e-04, 
       1.0660132758895862e-04, 
       1.0262261626823875e-04, 
       1.4049320350993343e-04, 
       1.0218658938222781e-04, 
       1.0211286719089522e-04, 
       1.3820003418191091e-04, 
       1.4114019639483374e-04, 
       1.0803782563324318e-04, 
       1.1197778404577340e-04, 
       1.4258455595401887e-04, 
       1.0770254138635894e-04, 
       1.1563941242635600e-04, 
       1.0362731980567512e-04, 
       1.0572664984200582e-04, 
       1.1283749431607226e-04, 
       1.0179076859023901e-04, 
       1.1219914886551592e-04, 
       1.1140518212099334e-04, 
       1.0324492131668983e-04, 
       1.1874615253062522e-04, 
       1.1686431873005142e-04, 
       1.2201521390140522e-04, 
       1.0107222858261959e-04, 
       1.2853823864404166e-04, 
       1.4018312132512908e-04, 
       1.3378133431988259e-04, 
       1.3705960204063000e-04, 
       1.2175333229378890e-04, 
       1.0203550124332017e-04, 
       1.0173692236646140e-04, 
       1.4130608227444114e-04, 
       1.0006839951802925e-04, 
       1.1831598494419209e-04, 
       1.3012619269019718e-04, 
       1.4111199720164762e-04, 
       1.2916151752623105e-04, 
       1.1808712619179572e-04, 
       1.3964999991624867e-04, 
       1.0576018239523485e-04, 
       1.2999999999998755e-04, 
       1.4011150203667561e-04, 
       1.0114166889765038e-04, 
       1.4091817388760367e-04, 
       1.0290765437197313e-04, 
       1.0631121218973794e-04, 
       1.0209735684289537e-04, 
       1.1621921026655905e-04, 
       1.0198260286364570e-04, 
       1.2238334351968982e-04, 
       1.0026892664720046e-04, 
       1.3935175795721396e-04, 
       9.9651867176816126e-05, 
       1.0446607232764233e-04, 
       1.0243272072023020e-04, 
       1.0996229910403271e-04, 
       1.0015578301497623e-04, 
       1.3859443636844601e-04, 
       1.4000000000002763e-04, 
       1.4000000000000110e-04, 
       1.4000000000001912e-04, 
       1.0047110311622120e-04, 
       1.3917725524309924e-04, 
       1.1332641770630674e-04, 
       9.9245283501956925e-05, 
       1.1332325299373979e-04, 
       1.0397150132229449e-04, 
       1.3931611227052532e-04, 
       1.0287003962071929e-04, 
       9.8936034130486810e-05, 
       1.0771618516862094e-04, 
       9.9236744127034727e-05, 
       1.3336761976014014e-04, 
       9.8454909262513253e-05, 
       1.1454772960476021e-04, 
       1.0382973756918650e-04, 
       1.0381703861675464e-04, 
       1.1647356190914754e-04, 
       1.0750380452019836e-04, 
       1.3796086552233250e-04, 
       1.3179912015192982e-04, 
       9.7650026447738004e-05, 
       1.0911505144740881e-04, 
       1.1474396948754765e-04, 
       1.3740054430499588e-04, 
       1.0107445344115580e-04, 
       1.0172119112808597e-04, 
       1.0589375505509555e-04, 
       1.3433879059708125e-04, 
       9.9520203257500860e-05, 
       1.0080972528431748e-04, 
       1.2738091164652687e-04, 
       1.0115041605310192e-04, 
       1.3699878276777838e-04, 
       9.9815435240150987e-05, 
       1.0233821099768191e-04, 
       1.3558265282569203e-04, 
       1.3443016841195907e-04, 
       9.9602814815851821e-05, 
       9.8896915254324149e-05, 
       9.8471117341948373e-05, 
       1.0238717490652405e-04, 
       1.1384100125443397e-04, 
       1.3565208292782194e-04, 
       1.3565208292782194e-04, 
       9.7936817245821477e-05, 
       1.0188947091343735e-04, 
       1.2172437555884551e-04, 
       1.1701997164908569e-04, 
       1.1730397504441889e-04, 
       1.2566732000685298e-04, 
       1.0223417494087606e-04, 
       1.0559602316238035e-04, 
       1.0216005169840250e-04, 
       9.8384888555681058e-05, 
       1.3432804857391742e-04, 
       1.2596679597297601e-04, 
       1.0149632435041840e-04, 
       1.3259954107970376e-04, 
       1.2698073621658672e-04, 
       1.3459334918594374e-04, 
       9.5514286559908107e-05, 
       9.6383899944435734e-05, 
       1.0425789005995188e-04, 
       1.2789815959195189e-04, 
       1.1000000000000223e-04, 
       1.1017384143681400e-04, 
       1.1442018486182442e-04, 
       1.0002549329913187e-04, 
       1.1486835674375766e-04, 
       1.0982579096878552e-04, 
       1.2910408027421078e-04, 
       9.8959572874275509e-05, 
       9.4256014989025375e-05, 
       9.4256014989025375e-05, 
       1.3031034043556151e-04, 
       1.3182004837130717e-04, 
       1.0951152827246828e-04, 
       9.4837594760299225e-05, 
       1.1597438149184454e-04, 
       1.2989818785394248e-04, 
       1.0013314962572331e-04, 
       1.2419342039766089e-04, 
       1.2535159090224424e-04, 
       1.0375089075046001e-04, 
       9.8356347606335729e-05, 
       1.1023233361014229e-04, 
       1.2848027774813530e-04, 
       1.0112286679496412e-04, 
       1.1610510461304639e-04, 
       1.0815559382922234e-04, 
       1.1705577910857745e-04, 
       9.8122592882688494e-05, 
       9.4998393472230520e-05, 
       1.1730450890099048e-04, 
       1.0179678117667963e-04, 
       1.0035429191784983e-04, 
       1.2112012132480230e-04, 
       9.7861231782973759e-05, 
       9.3480574828878500e-05, 
       9.3407186112010699e-05, 
       1.0232662022235787e-04, 
       1.1999999999996249e-04, 
       1.3014665208967025e-04, 
       9.4491927704513307e-05, 
       1.2999999999996090e-04, 
       1.2999999999998126e-04, 
       1.0795512588910247e-04, 
       1.1418421445733480e-04, 
       9.3758691321809199e-05, 
       1.0820998240774132e-04, 
       1.2853839836687069e-04, 
       1.0518839873352059e-04, 
       1.1985854916816670e-04, 
       1.2727040708528000e-04, 
       1.2140251106057449e-04, 
       1.0178451768194304e-04, 
       1.0548048010985225e-04, 
       1.2417365720128617e-04, 
       9.5771747637823004e-05, 
       9.6113228668302656e-05, 
       9.6392803088793920e-05, 
       1.0904133612419888e-04, 
       9.1251486322646526e-05, 
       1.2511240522102384e-04, 
       1.2156877228499928e-04, 
       1.0783892726215754e-04, 
       1.0673748842660715e-04, 
       1.2300138730199649e-04, 
       9.9042889148017181e-05, 
       1.1531533879515578e-04, 
       9.8108920269365056e-05, 
       1.2695171801742754e-04, 
       1.0674707791993266e-04, 
       9.0691564594846721e-05, 
       9.1465954854871149e-05, 
       1.2698730034943243e-04, 
       1.0639365224193430e-04, 
       1.2622047111983365e-04, 
       9.3314555674562971e-05, 
       1.0643414037643173e-04, 
       1.0006928492220438e-04, 
       1.0122842793427244e-04, 
       1.0413257167419686e-04, 
       1.2202908471071597e-04, 
       1.1558402970936500e-04, 
       1.1893902020235161e-04, 
       1.2686574144570441e-04, 
       1.0013522019726030e-04, 
       1.2625355951340728e-04, 
       1.0268138186086510e-04, 
       1.2659706031755208e-04, 
       1.0556417008166629e-04, 
       1.2215988009023376e-04, 
       1.0489786068872611e-04, 
       1.2554221357798077e-04, 
       1.2220540041224276e-04, 
       1.1998772877047616e-04, 
       1.0101631069754862e-04, 
       1.2587880531051972e-04, 
       1.2557299381695951e-04, 
       1.0797480542713050e-04, 
       9.0794169709534130e-05, 
       1.2476322855077966e-04, 
       9.2025290859875519e-05, 
       1.2317095979725711e-04, 
       1.2179540465821959e-04, 
       1.0422621574539337e-04, 
       1.0660463581203187e-04, 
       1.2481755282695871e-04, 
       1.1941182032651300e-04, 
       1.0634151950527153e-04, 
       1.2496644898502880e-04, 
       9.3272550474342295e-05, 
       1.2015602811342524e-04, 
       8.9587828052505017e-05, 
       1.2472434483245748e-04, 
       1.1607579462479840e-04, 
       1.0133778800569856e-04, 
       9.0397152930314905e-05, 
       9.5966708415305051e-05, 
       9.1711100148394165e-05, 
       9.5930965523164334e-05, 
       1.2311231044665933e-04, 
       1.2163505559235126e-04, 
       9.1477576381338320e-05, 
       1.0084261793262089e-04, 
       1.2121886649936345e-04, 
       1.1770932008867974e-04, 
       9.7954673428925570e-05, 
       8.9502160494969282e-05, 
       1.2054276675425823e-04, 
       1.1466236847341514e-04, 
       1.0585991117247721e-04, 
       1.0274609417556479e-04, 
       8.8600171845902605e-05, 
       8.7704092154334347e-05, 
       8.9315872249003326e-05, 
       1.2228800668007871e-04, 
       1.2293626684163351e-04, 
       1.2199752773816792e-04, 
       9.0946331255193082e-05, 
       1.1173650310176072e-04, 
       1.0982298476195096e-04, 
       1.1801708365256981e-04, 
       1.1000000000004272e-04, 
       1.0122364224804299e-04, 
       1.2236024967769176e-04, 
       8.9655078870332785e-05, 
       9.1097136627500856e-05, 
       1.2204140303290559e-04, 
       1.0161859758221363e-04, 
       8.7418873967152577e-05, 
       9.6245012584474110e-05, 
       8.7376072823676856e-05, 
       8.8407617824617024e-05, 
       9.1052290109196643e-05, 
       8.6419235147234828e-05, 
       1.0406454839978003e-04, 
       9.8654240994702175e-05, 
       1.0357545271392300e-04, 
       1.0113194940507710e-04, 
       1.2115457430816763e-04, 
       1.2019271712722973e-04, 
       1.0288756658645369e-04, 
       1.1625487917200916e-04, 
       9.9094884208793244e-05, 
       1.1369437051621789e-04, 
       8.5662713138400716e-05, 
       1.0012770037634402e-04, 
       1.2078056948940330e-04, 
       9.0170591389285076e-05, 
       9.9097382064558664e-05, 
       1.1609179274745751e-04, 
       8.9788167153167516e-05, 
       1.0041456237337234e-04, 
       1.0179873231550728e-04, 
       8.5229411248159475e-05, 
       8.8606037494956086e-05, 
       8.6952370494365158e-05, 
       1.1999999999995943e-04, 
       1.2000000000001524e-04, 
       1.2000000000003645e-04, 
       9.2877475705977646e-05, 
       1.0445423371330962e-04, 
       9.0334795464594496e-05, 
       9.7689497658498618e-05, 
       1.0106212616639630e-04, 
       9.9453914457817681e-05, 
       1.1930229298001375e-04, 
       1.0495967301273962e-04, 
       1.0477045024903581e-04, 
       1.1055556797614898e-04, 
       9.8903490204335903e-05, 
       1.0693133197076152e-04, 
       8.5677933643574724e-05, 
       8.5677933643574724e-05, 
       9.8134642287844878e-05, 
       9.8581826616870937e-05, 
       1.0593145891981754e-04, 
       8.4987567858036368e-05, 
       1.1761938464274281e-04, 
       9.0236594731326979e-05, 
       1.1776210857319727e-04, 
       9.8980467336778099e-05, 
       9.6954530401830686e-05, 
       8.7522010043932408e-05, 
       9.7956932164348784e-05, 
       9.2894242498151830e-05, 
       1.1015688448530337e-04, 
       8.4601829289378865e-05, 
       8.6705955591731183e-05, 
       1.0533480662038682e-04, 
       1.0248771679859613e-04, 
       1.1478419819692594e-04, 
       1.1728751373722064e-04, 
       1.1711596197689180e-04, 
       8.7108145512102727e-05, 
       9.8330174610838648e-05, 
       9.1627796293045007e-05, 
       9.1473315705062227e-05, 
       1.1369570140219362e-04, 
       9.7377243901028171e-05, 
       1.1189917411427915e-04, 
       8.9093270563762024e-05, 
       8.7625352839321998e-05, 
       1.0675475350728433e-04, 
       1.0078395211880273e-04, 
       8.2631681505521398e-05, 
       8.7784966417625435e-05, 
       8.9631518315566570e-05, 
       8.8033305434181625e-05, 
       9.6184779404241734e-05, 
       8.9076565155085408e-05, 
       1.1495753769498076e-04, 
       8.5239328469476067e-05, 
       1.0389061032995724e-04, 
       1.1565596842212520e-04, 
       9.3687855810380420e-05, 
       8.5605681302257711e-05, 
       9.5787118111795202e-05, 
       8.2020543727679475e-05, 
       1.0458952989839777e-04, 
       8.8131439106634856e-05, 
       8.8125796515598880e-05, 
       8.8502191574219840e-05, 
       1.1431073333110567e-04, 
       1.1297654847357828e-04, 
       8.1220362210218346e-05, 
       8.2118500965781840e-05, 
       1.1363894119894263e-04, 
       8.1967575615574678e-05, 
       1.1216900814591960e-04, 
       8.3773204189794954e-05, 
       9.1990809807271769e-05, 
       9.5633570613663671e-05, 
       8.9205427571849058e-05, 
       8.2196948733038349e-05, 
       9.8002247116442029e-05, 
       8.1664488244470945e-05, 
       8.3492719622401334e-05, 
       8.1714283930089601e-05, 
       8.4105776540539196e-05, 
       8.7803851265448200e-05, 
       8.7803851265448200e-05, 
       1.1384795390375729e-04, 
       1.0880962120120442e-04, 
       8.5498377678002642e-05, 
       8.1106674862278249e-05, 
       8.0554666725152248e-05, 
       8.4983452972775265e-05, 
       8.7112378448848881e-05, 
       8.3666828374008421e-05, 
       8.0735350330282394e-05, 
       9.2953156227155957e-05, 
       8.8868849880820806e-05, 
       8.8849497878280381e-05, 
       8.1539993300187700e-05, 
       1.1254317118712444e-04, 
       9.6014568238159657e-05, 
       8.0375086117522349e-05, 
       8.0308740731786080e-05, 
       1.0854498369639651e-04, 
       1.1188913633649453e-04, 
       9.4576917685468073e-05, 
       1.1060472072735106e-04, 
       7.9765619810371097e-05, 
       8.4141442889284281e-05, 
       9.3552228751237761e-05, 
       9.3307353380867191e-05, 
       7.9999999999995141e-05, 
       9.3147583987742016e-05, 
       7.9400752936796209e-05, 
       9.9731252719811059e-05, 
       9.7405426826906955e-05, 
       9.0037963246403463e-05, 
       8.0812754892571018e-05, 
       8.1081648711590351e-05, 
       7.9282360529885667e-05, 
       8.7077408171290841e-05, 
       1.0891494191428356e-04, 
       8.0448180345346309e-05, 
       8.9553197089364632e-05, 
       8.5462435155135100e-05, 
       1.1049555081569100e-04, 
       9.3716531664271644e-05, 
       1.0390609225783435e-04, 
       8.1992047494702450e-05, 
       8.1992047494702450e-05, 
       8.3866376340423709e-05, 
       9.2679181045595126e-05, 
       7.9451628765122808e-05, 
       8.1447750919663616e-05, 
       9.5109104604508283e-05, 
       8.2383854676251999e-05, 
       1.0384410671968337e-04, 
       8.0501389709615388e-05, 
       8.7469207866319981e-05, 
       9.2054941032143590e-05, 
       9.1663231372287154e-05, 
       7.8068005748901069e-05, 
       7.9010224898634157e-05, 
       9.1402425047733125e-05, 
       8.8769469340950493e-05, 
       9.1103236320504681e-05, 
       9.0990634039100690e-05, 
       9.0646939422335467e-05, 
       8.1599544067641026e-05, 
       1.1000000000000944e-04, 
       1.0999999999997696e-04, 
       1.0958092995340969e-04, 
       1.0474686971425728e-04, 
       9.1305309677866555e-05, 
       8.9920496540599483e-05, 
       8.7534933561809596e-05, 
       8.5900358355785856e-05, 
       8.1449895027852748e-05, 
       1.0027569953737355e-04, 
       9.7530978379340675e-05, 
       8.3552787437219531e-05, 
       1.0495715211555467e-04, 
       7.9290166264358301e-05, 
       8.1601002496451241e-05, 
       8.9574910359454805e-05, 
       1.0854808640327811e-04, 
       7.8834494569848411e-05, 
       9.7130497056889585e-05, 
       7.7423594325735673e-05, 
       9.5031734645498737e-05, 
       7.7722290231086667e-05, 
       8.8521362495953083e-05, 
       7.7332930546429989e-05, 
       9.4455242035968405e-05, 
       9.9357670871807628e-05, 
       1.0868015946459822e-04, 
       7.9007586742574853e-05, 
       8.2872427195313414e-05, 
       7.9209879527786600e-05, 
       1.0186600739615422e-04, 
       1.0000000000000221e-04, 
       1.0000000000000334e-04, 
       1.0000000000000296e-04, 
       1.0769368652830217e-04, 
       1.0841867129570088e-04, 
       7.6996943788860871e-05, 
       7.8984749459162727e-05, 
       1.0208566978379656e-04, 
       1.0530876896041489e-04, 
       9.2593739023413035e-05, 
       8.9576808198524039e-05, 
       1.0753789779578029e-04, 
       1.0809940778217144e-04, 
       1.0769906396535702e-04, 
       7.6511598328092326e-05, 
       1.0750431097856745e-04, 
       7.8642804779853839e-05, 
       8.1034375191387626e-05, 
       1.0614597904593044e-04, 
       8.2270520820083523e-05, 
       8.2749208289396693e-05, 
       8.2724047965198520e-05, 
       8.8080738175531535e-05, 
       1.0198645734688157e-04, 
       1.0482507710597043e-04, 
       1.0540567930512304e-04, 
       1.0540567930512304e-04, 
       8.0700768500193207e-05, 
       8.9391332492663283e-05, 
       1.0098715166292284e-04, 
       1.0372625515196713e-04, 
       1.0571803801507745e-04, 
       7.9064782209315631e-05, 
       8.9359887661122648e-05, 
       7.8871210961207349e-05, 
       7.7162027057154278e-05, 
       1.0498765247598911e-04, 
       8.0000000000002338e-05, 
       1.0634007089865870e-04, 
       1.0613675761326501e-04, 
       7.9426496276440808e-05, 
       7.9505763256152297e-05, 
       7.7210598505739897e-05, 
       1.0635622983306603e-04, 
       7.6081911435969251e-05, 
       8.6204453092237601e-05, 
       1.0049582382262412e-04, 
       1.0545754108933778e-04, 
       7.5951569526638461e-05, 
       8.1283575624385520e-05, 
       1.0275252852369799e-04, 
       1.0563494487360308e-04, 
       7.8328748657451489e-05, 
       9.0200161963040226e-05, 
       7.6865994572887281e-05, 
       1.0609076826160927e-04, 
       9.7885338834587777e-05, 
       1.0235963700913029e-04, 
       7.6916078204174019e-05, 
       7.9078988306317309e-05, 
       9.8139948713311121e-05, 
       1.0531435183628403e-04, 
       9.1120004604576155e-05, 
       7.8868698026167742e-05, 
       9.2431396962727239e-05, 
       1.0524625884475417e-04, 
       1.0011760070079590e-04, 
       1.0199926476280730e-04, 
       8.2617014935499166e-05, 
       7.7463139948846538e-05, 
       8.3031926848547400e-05, 
       7.9523091247022258e-05, 
       7.9338199162599327e-05, 
       7.7713931829944308e-05, 
       8.2085366331088942e-05, 
       1.0077457514058436e-04, 
       8.5865373262364490e-05, 
       7.4137928220764966e-05, 
       1.0470154553259903e-04, 
       8.8858817226593657e-05, 
       9.1210272847435118e-05, 
       7.8140118010677057e-05, 
       8.7126234829113802e-05, 
       8.7024804486610045e-05, 
       8.4241039748788701e-05, 
       8.9027636463239628e-05, 
       7.9428313928899422e-05, 
       1.0114503711711229e-04, 
       9.9154942551752406e-05, 
       8.9999999999996374e-05, 
       1.0385829291425154e-04, 
       7.4517529755463297e-05, 
       8.5689554153714628e-05, 
       8.5663470548377846e-05, 
       7.3996973670928200e-05, 
       7.5317401735263868e-05, 
       7.7898976578780359e-05, 
       7.9078566880419278e-05, 
       8.6128147825981612e-05, 
       8.6050275878035237e-05, 
       8.9799683953218604e-05, 
       1.0274958615134018e-04, 
       8.6608870733724254e-05, 
       8.3104397612028625e-05, 
       7.6488209326951379e-05, 
       1.0071069698531535e-04, 
       7.5617757945707219e-05, 
       8.6011759076686030e-05, 
       8.5080255660471057e-05, 
       8.6132541170162320e-05, 
       7.6868064724253568e-05, 
       7.5244233822924252e-05, 
       7.7094027202383660e-05, 
       1.0225341492496869e-04, 
       8.6320793989075605e-05, 
       8.0724066167492284e-05, 
       8.4507434240344199e-05, 
       8.2527218444737835e-05, 
       8.0840544414476911e-05, 
       8.1187980492611412e-05, 
       8.5078633348245649e-05, 
       7.5530947950415230e-05, 
       7.5948853899103741e-05, 
       7.6554343975876183e-05, 
       7.5413265378803586e-05, 
       8.6683426775063202e-05, 
       1.0230753758844400e-04, 
       7.5010868427174493e-05, 
       9.9245999853681921e-05, 
       1.0214942167969541e-04, 
       7.6593160197422781e-05, 
       9.7647372960502971e-05, 
       8.0984161436259931e-05, 
       7.2221745549041433e-05, 
       8.2476434803497814e-05, 
       7.3558928363637988e-05, 
       8.7257090688041652e-05, 
       7.7647516038693694e-05, 
       9.0769981444006710e-05, 
       7.5426248920700166e-05, 
       8.3194959267818326e-05, 
       8.4218697443701613e-05, 
       1.0147087217306007e-04, 
       7.2967411298811708e-05, 
       1.0028066967817269e-04, 
       9.8339172054595243e-05, 
       1.0136219450604877e-04, 
       7.2615397088188091e-05, 
       7.3123308375082967e-05, 
       8.4086233668145997e-05, 
       8.5782063090470280e-05, 
       9.7650038960778358e-05, 
       8.3100098261311618e-05, 
       9.3057255284051765e-05, 
       9.1110967683341538e-05, 
       8.9844430398412713e-05, 
       8.4048648548912998e-05, 
       8.3842244744117227e-05, 
       7.3903876420449466e-05, 
       7.5387599049816549e-05, 
       8.3493089063256372e-05, 
       7.2818394753666285e-05, 
       7.1097341678211861e-05, 
       7.1057618640941528e-05, 
       9.4688802303242504e-05, 
       9.3384875017835654e-05, 
       7.1154455127459121e-05, 
       9.9999999999997836e-05, 
       9.9999999999997769e-05, 
       1.0000000000000276e-04, 
       9.9999999999996183e-05, 
       9.9999999999996928e-05, 
       1.0000000000000000e-04, 
       7.0838690459119080e-05, 
       7.2697623399521388e-05, 
       9.8925305019139062e-05, 
       8.2421452373052730e-05, 
       7.6033344538281594e-05, 
       9.3125582258770997e-05, 
       7.6126181075916911e-05, 
       7.6126181075916911e-05, 
       9.6393872178302532e-05, 
       7.6628797360605980e-05, 
       7.0873983990174847e-05, 
       7.6895165477934539e-05, 
       7.2103900483248457e-05, 
       7.0538337919117487e-05, 
       7.0538337919117487e-05, 
       7.9188664614064563e-05, 
       8.3773270029698226e-05, 
       9.5920792352817259e-05, 
       7.2240766374646753e-05, 
       7.1844946665426942e-05, 
       7.1701426358310755e-05, 
       7.4591761224882571e-05, 
       8.2030605153689514e-05, 
       8.8337311312279007e-05, 
       7.1718666790569841e-05, 
       7.0823538865281616e-05, 
       9.5217815484665235e-05, 
       9.8464249044307445e-05, 
       9.7467507742600494e-05, 
       9.0295184259738220e-05, 
       9.8092681798626937e-05, 
       8.3838450295963785e-05, 
       8.2232967350668557e-05, 
       9.6042340982166005e-05, 
       7.3438085108317973e-05, 
       8.1492508977708530e-05, 
       9.6702778587745325e-05, 
       6.9339477233584961e-05, 
       7.4458164224992286e-05, 
       8.0934131135357586e-05, 
       6.9080688397379434e-05, 
       7.5200061978936887e-05, 
       9.3867240955738978e-05, 
       7.3941866442909040e-05, 
       7.1128469288057064e-05, 
       9.6366182792600682e-05, 
       9.4372265409115459e-05, 
       8.3746971644011622e-05, 
       7.1366498026140070e-05, 
       9.4288735459412151e-05, 
       9.7110160108635358e-05, 
       7.0313604739178441e-05, 
       9.5596731502767970e-05, 
       9.3934205097630962e-05, 
       9.2285568587099117e-05, 
       7.4926273494395554e-05, 
       6.8490744902700751e-05, 
       8.0083233833820849e-05, 
       9.3038341842542811e-05, 
       9.6479540091464521e-05, 
       7.1206731617662458e-05, 
       8.6429112490005370e-05, 
       7.2498050070721179e-05, 
       6.9412684833113116e-05, 
       8.5023719387783869e-05, 
       9.6027030777804940e-05, 
       8.9939612579272824e-05, 
       7.3201324653390901e-05, 
       9.4917560175284942e-05, 
       7.9763874475242573e-05, 
       6.7944665615777067e-05, 
       7.1830982320605197e-05, 
       7.9656698342298618e-05, 
       9.5734497366677576e-05, 
       6.9520991574164085e-05, 
       7.3405941567082256e-05, 
       7.9999999999996469e-05, 
       6.7984291436332957e-05, 
       8.5481827748780628e-05, 
       6.8850560377642096e-05, 
       7.8372000363968527e-05, 
       6.7551066765761245e-05, 
       6.9915642286157010e-05, 
       9.2978689827703833e-05, 
       7.7373921555125538e-05, 
       7.7670199117669530e-05, 
       8.0791113938625603e-05, 
       6.9000961869339708e-05, 
       6.8787039893559380e-05, 
       6.8849139418414562e-05, 
       9.0684483329836834e-05, 
       7.2213056613350856e-05, 
       7.8009679925042664e-05, 
       8.7719744348829174e-05, 
       9.4167349109264807e-05, 
       8.4647536004137943e-05, 
       7.8407629277069211e-05, 
       7.0623313207959884e-05, 
       6.6912142476392269e-05, 
       7.7773108234484162e-05, 
       8.3775733136956583e-05, 
       6.8734267425167754e-05, 
       7.1679816490409913e-05, 
       8.5332878764648304e-05, 
       7.8422320652832752e-05, 
       7.7870524970763852e-05, 
       6.8501660446526976e-05, 
       7.0432189478614865e-05, 
       9.0740239839094537e-05, 
       6.8134276641400782e-05, 
       7.3897523893817082e-05, 
       7.2771408046467151e-05, 
       7.9566929305947912e-05, 
       6.8180357551378182e-05, 
       7.2374836344731177e-05, 
       8.6183451104102719e-05, 
       6.6694747734294955e-05, 
       8.1203830541420890e-05, 
       9.0400986196571091e-05, 
       6.7018487836548014e-05, 
       7.6826073766118362e-05, 
       6.6887324559123296e-05, 
       7.0287125741985075e-05, 
       8.9634266858742057e-05, 
       7.7434470655690573e-05, 
       7.0687491777254711e-05, 
       9.1483183225009686e-05, 
       6.5992029644971022e-05, 
       7.1828848277819221e-05, 
       6.6024144457343226e-05, 
       8.1814185979065589e-05, 
       6.6708441432496295e-05, 
       9.3048884072254430e-05, 
       6.9450395278256626e-05, 
       7.2354488575329369e-05, 
       7.5603683537169141e-05, 
       6.6139794816543932e-05, 
       7.9695381330682500e-05, 
       9.2680884495699285e-05, 
       7.0000000000001037e-05, 
       7.7112176335805444e-05, 
       7.8808265134962767e-05, 
       8.4398966581856669e-05, 
       7.0664104776124571e-05, 
       9.2403708128072811e-05, 
       9.2483814201478796e-05, 
       7.8159563609077914e-05, 
       6.9999995725857403e-05, 
       7.2386998166301154e-05, 
       7.2375372024988504e-05, 
       7.0580791753289401e-05, 
       7.8923195366876055e-05, 
       7.2290826358230312e-05, 
       6.5529325150052040e-05, 
       6.9459347650055228e-05, 
       7.2253991999628709e-05, 
       7.6417733400203452e-05, 
       7.4468734402209448e-05, 
       7.2316044276501170e-05, 
       9.0347699250556499e-05, 
       6.9781852661697272e-05, 
       8.9036430592359504e-05, 
       6.9857849579582756e-05, 
       6.5115015051355116e-05, 
       9.1879818669929596e-05, 
       6.6958263209413010e-05, 
       6.9623492354649571e-05, 
       7.4638296698939749e-05, 
       7.6472055442017550e-05, 
       8.7633269112767978e-05, 
       6.8844234115298837e-05, 
       6.9169537085692599e-05, 
       9.1307512053629612e-05, 
       9.1704636199515963e-05, 
       7.7501995247135250e-05, 
       7.2265772271875491e-05, 
       8.0702796472607662e-05, 
       7.7829506679633650e-05, 
       7.5724728776048944e-05, 
       6.7988487536570130e-05, 
       7.6224206543554077e-05, 
       7.6285332804508878e-05, 
       6.8053693319656685e-05, 
       7.5827644550184646e-05, 
       7.0890301680331277e-05, 
       7.2566643127504020e-05, 
       7.0000000000000550e-05, 
       6.6951092031751458e-05, 
       8.9921255587574768e-05, 
       7.5562011808856958e-05, 
       6.7776482568645196e-05, 
       7.5584065184259628e-05, 
       6.5007952404470877e-05, 
       7.4975626967596905e-05, 
       7.6285662344655040e-05, 
       8.5001058696662936e-05, 
       8.8872441589296907e-05, 
       8.8872441589296907e-05, 
       6.7710192644087938e-05, 
       6.4194301943277661e-05, 
       6.4028689619885615e-05, 
       6.4037243230730476e-05, 
       6.9405859838583801e-05, 
       7.4778587707066140e-05, 
       8.9449111123297633e-05, 
       8.9999999999999152e-05, 
       6.9421421684302993e-05, 
       6.8505192387133550e-05, 
       8.9999999999995682e-05, 
       9.0000000000002662e-05, 
       9.0000000000001117e-05, 
       8.9999999999998284e-05, 
       7.3022679680835465e-05, 
       8.9850699652130260e-05, 
       6.6065813717384501e-05, 
       6.5733712696743854e-05, 
       8.5650538876354761e-05, 
       8.6989953771884798e-05, 
       6.3712296798195791e-05, 
       8.9492379556623561e-05, 
       6.4086517720247352e-05, 
       6.5100963215693048e-05, 
       6.3980596038823461e-05, 
       6.6847156239555289e-05, 
       8.9654444410343858e-05, 
       6.5146546424750624e-05, 
       6.5006848448260021e-05, 
       6.3411209917457989e-05, 
       6.9539640506425878e-05, 
       6.7439104279998932e-05, 
       6.9697519601601200e-05, 
       6.6761475106429533e-05, 
       7.4872423681596504e-05, 
       6.6642789971112779e-05, 
       7.5063739271118005e-05, 
       7.5668614406573573e-05, 
       7.5668614406573573e-05, 
       6.9082363160356483e-05, 
       8.8692909018035009e-05, 
       6.3208892640598179e-05, 
       6.3208892640598179e-05, 
       7.4118530053897053e-05, 
       8.6136619189994216e-05, 
       6.3930293145074939e-05, 
       7.3194520951362090e-05, 
       7.7129223040689690e-05, 
       7.4162009056880814e-05, 
       8.4944989332001242e-05, 
       8.8785966861099685e-05, 
       6.6694712995663834e-05, 
       7.4144116441629356e-05, 
       8.7820600951576909e-05, 
       7.2579100502984554e-05, 
       8.7728282052421293e-05, 
       7.4008967460611122e-05, 
       6.8827688580660884e-05, 
       7.5520067906771049e-05, 
       8.2920752116079986e-05, 
       6.4001027740969365e-05, 
       8.1971408683785779e-05, 
       7.9489486907729966e-05, 
       6.7842017841115092e-05, 
       6.6625153454191282e-05, 
       6.6314427503739415e-05, 
       7.0601321236640679e-05, 
       6.8333220343366841e-05, 
       8.6672932995993221e-05, 
       8.6702153233717853e-05, 
       7.2295429682206887e-05, 
       6.4940287358932505e-05, 
       6.2017363851122814e-05, 
       6.6616550807102444e-05, 
       6.6425366805987916e-05, 
       8.7298972510569867e-05, 
       7.5616021934285472e-05, 
       6.5478572499739509e-05, 
       8.4431565625076025e-05, 
       7.2851939938918349e-05, 
       6.9637041488781497e-05, 
       8.5592444010360315e-05, 
       8.3219149453353451e-05, 
       8.6910082264070264e-05, 
       8.0000000000002256e-05, 
       6.8847951826343464e-05, 
       6.1947627983547143e-05, 
       6.4025907217633927e-05, 
       8.0000000000003883e-05, 
       8.0000000000003842e-05, 
       7.9999999999999424e-05, 
       8.0000000000001402e-05, 
       8.6050685551009913e-05, 
       6.6661393447996992e-05, 
       6.1295529077665185e-05, 
       8.3480488384025015e-05, 
       7.4086597278268763e-05, 
       6.1295538805971914e-05, 
       7.7353299251904794e-05, 
       6.3789988696441729e-05, 
       6.1937001615840369e-05, 
       7.7216314500714467e-05, 
       6.6053762088363646e-05, 
       6.8613577483682634e-05, 
       6.7145641820334925e-05, 
       6.2675146919482201e-05, 
       7.1845328603144167e-05, 
       8.3963539468134981e-05, 
       7.3044169581359955e-05, 
       7.3044169581359955e-05, 
       7.0891796755842722e-05, 
       6.4663322900376753e-05, 
       6.2292627610583193e-05, 
       7.6993565690140658e-05, 
       6.6820933405635228e-05, 
       7.7935118878123821e-05, 
       6.3967923127303174e-05, 
       6.0795406487596570e-05, 
       6.2589369465452651e-05, 
       7.4191208586323095e-05, 
       6.6255892380265394e-05, 
       8.5597916421167183e-05, 
       6.8370895422703118e-05, 
       6.0901676639620634e-05, 
       6.8049036301050733e-05, 
       6.1041156844097077e-05, 
       6.5903689147345793e-05, 
       6.2124740647708518e-05, 
       6.2314562387189132e-05, 
       6.0512134001003855e-05, 
       6.8486535095525514e-05, 
       7.7827088202146093e-05, 
       7.9336914608998536e-05, 
       7.2014845490578732e-05, 
       8.1606317136676747e-05, 
       7.1178561305609511e-05, 
       8.4176926519562015e-05, 
       7.0745665740781107e-05, 
       6.1030200597279737e-05, 
       7.0221943185742050e-05, 
       7.1770141042899974e-05, 
       8.2876096735188054e-05, 
       6.3106827635935021e-05, 
       8.4049649961430252e-05, 
       8.3971878811891205e-05, 
       6.9713048722176116e-05, 
       6.1063856078847468e-05, 
       6.2111156153156380e-05, 
       6.8477596855495828e-05, 
       7.4728142395030656e-05, 
       7.1443683352790126e-05, 
       6.1444309061791814e-05, 
       6.9437823142425010e-05, 
       6.9999586568302643e-05, 
       6.0288803113892016e-05, 
       7.0886250601877193e-05, 
       5.9517344801579615e-05, 
       6.9862041896514635e-05, 
       6.1326266094570249e-05, 
       6.9358131171232454e-05, 
       8.2294043253059359e-05, 
       6.2262878845233683e-05, 
       8.3087104793811743e-05, 
       5.9755455151006992e-05, 
       5.9975911008270836e-05, 
       6.7248098076303051e-05, 
       5.9939807033004162e-05, 
       7.9421379076872930e-05, 
       7.2077394366280579e-05, 
       6.9422569282022131e-05, 
       6.3049311078183258e-05, 
       6.1022505729840919e-05, 
       6.2316047964951690e-05, 
       8.3398129306047779e-05, 
       6.5588570795712721e-05, 
       6.0328606386868346e-05, 
       7.8356315139209453e-05, 
       8.1367356128142244e-05, 
       5.8789103604822755e-05, 
       6.7186149006355164e-05, 
       8.1137483331954937e-05, 
       5.8966069701977884e-05, 
       7.4130131166358967e-05, 
       6.6171300152773554e-05, 
       6.1846928361594187e-05, 
       8.2867968724286204e-05, 
       7.0041079554833440e-05, 
       7.8106701429622922e-05, 
       7.1202314635040298e-05, 
       5.9836546443424792e-05, 
       5.8576497700564385e-05, 
       5.8486837245813019e-05, 
       8.1853789853467439e-05, 
       6.6465033211453886e-05, 
       6.7594751604494358e-05, 
       6.7589207947063777e-05, 
       6.2851451430143699e-05, 
       7.4908647797968555e-05, 
       7.1841311749797058e-05, 
       7.1058161837312346e-05, 
       6.7351961272134699e-05, 
       6.9578580425358293e-05, 
       7.5811559562980379e-05, 
       6.8796222004168626e-05, 
       8.1804119139069613e-05, 
       6.0722142946019726e-05, 
       8.0359886539278736e-05, 
       5.8575460937275274e-05, 
       7.0043238730644513e-05, 
       7.7559363916957679e-05, 
       6.1117434485299306e-05, 
       6.8174505248427543e-05, 
       7.8342957218475141e-05, 
       6.1056585235949836e-05, 
       5.9013364815193407e-05, 
       7.5647576757309964e-05, 
       6.1747986820439481e-05, 
       7.9999999999996429e-05, 
       7.8760913549864620e-05, 
       8.1072729486216712e-05, 
       5.9127926522721309e-05, 
       8.1563993742181891e-05, 
       8.1448560451748795e-05, 
       5.8234019696507641e-05, 
       5.8218634662979693e-05, 
       6.9903257322839711e-05, 
       7.8593987223039778e-05, 
       8.0962871072286420e-05, 
       5.8070771386830406e-05, 
       6.3691326346237344e-05, 
       6.3683426154923165e-05, 
       8.0834510747882036e-05, 
       6.5929358015072881e-05, 
       5.7487505501763549e-05, 
       7.7990377094070031e-05, 
       6.5878549684681461e-05, 
       8.1096674012214243e-05, 
       6.0195324520524503e-05, 
       8.1121133834423359e-05, 
       8.1012738218484432e-05, 
       7.1298459410059630e-05, 
       7.0246902049072446e-05, 
       6.0914837545958159e-05, 
       7.2206027188168539e-05, 
       5.8333284843342976e-05, 
       6.7398179778162525e-05, 
       5.9656683086010640e-05, 
       5.8519342502889724e-05, 
       7.6609799758899620e-05, 
       6.7179552756107489e-05, 
       7.3742197126784134e-05, 
       5.7076756726943410e-05, 
       6.0472131434879337e-05, 
       5.7281188951486858e-05, 
       5.7513253386424693e-05, 
       6.6563551182254927e-05, 
       5.8096360746475580e-05, 
       7.6539397911164366e-05, 
       7.0901726630843105e-05, 
       7.1338707457302390e-05, 
       8.0246610328105238e-05, 
       6.0409854001594292e-05, 
       7.7308754208739908e-05, 
       5.6694894372324928e-05, 
       8.0000000000001552e-05, 
       5.6739072122418531e-05, 
       6.6802605884038702e-05, 
       6.9722779843071529e-05, 
       7.9999964553520704e-05, 
       5.7551186655517369e-05, 
       7.9068402437872180e-05, 
       6.0903545856475739e-05, 
       7.9999999999995981e-05, 
       7.9999999999999600e-05, 
       7.9999999999999708e-05, 
       7.9999999999998245e-05, 
       7.9999999999997716e-05, 
       7.9999999999995914e-05, 
       6.5771268461172988e-05, 
       7.9900951576652331e-05, 
       5.9618739190768820e-05, 
       6.4911545489035156e-05, 
       7.7017309256409406e-05, 
       6.0406897445587954e-05, 
       5.6372051538452727e-05, 
       6.4399149744066476e-05, 
       7.7310648005134351e-05, 
       7.8550756286093726e-05, 
       7.8722678202331354e-05, 
       5.8359126534978765e-05, 
       6.5772459083018813e-05, 
       6.9489860853043009e-05, 
       6.2111277647789215e-05, 
       5.8277492382694714e-05, 
       7.7885743919497391e-05, 
       6.5874151818445158e-05, 
       5.8601977383260578e-05, 
       5.6040097629855852e-05, 
       5.7007370548025930e-05, 
       7.1944465606370219e-05, 
       7.5110161170259949e-05, 
       5.8387783722172441e-05, 
       5.8804700320789873e-05, 
       6.8707265051740977e-05, 
       5.6299204895686136e-05, 
       6.9188156620092115e-05, 
       5.7401869333348134e-05, 
       5.7673069369991239e-05, 
       6.5796944772588875e-05, 
       6.3947499301463676e-05, 
       7.1580467597377731e-05, 
       5.6200564772730648e-05, 
       7.0309247430800411e-05, 
       6.6047817857402421e-05, 
       5.9462726273928843e-05, 
       5.6200268252530903e-05, 
       6.2144721329275262e-05, 
       5.8491587187299773e-05, 
       5.7197483299444588e-05, 
       5.8472361839070634e-05, 
       5.7431867446371738e-05, 
       7.8319174608492101e-05, 
       5.9554526781387140e-05, 
       5.8283305534387508e-05, 
       6.8507473709744919e-05, 
       5.5950661848999256e-05, 
       6.4500471463740738e-05, 
       7.5826741926749761e-05, 
       6.1218018140755617e-05, 
       7.5398343432449687e-05, 
       6.7242079155482249e-05, 
       5.8350509548537691e-05, 
       7.0046521270643268e-05, 
       6.6966234610041920e-05, 
       7.5153153319304142e-05, 
       6.5963198452251897e-05, 
       5.5888382849423982e-05, 
       5.7616578782686680e-05, 
       5.8047756786255423e-05, 
       7.7733992715337813e-05, 
       7.7319539440982277e-05, 
       5.9093479652273457e-05, 
       6.5879117630712947e-05, 
       6.3546154797898015e-05, 
       5.8496049945965234e-05, 
       5.8682686865950203e-05, 
       7.5133817578064494e-05, 
       5.5739563629734406e-05, 
       5.8227037358834596e-05, 
       6.6583157477973162e-05, 
       5.5423122066201949e-05, 
       7.2844555761428912e-05, 
       5.5351408239329317e-05, 
       5.6509587337812714e-05, 
       6.8016585031002057e-05, 
       5.6589329206011907e-05, 
       6.6708441432477688e-05, 
       5.7654406207677711e-05, 
       5.7654406207677711e-05, 
       7.7126306535250762e-05, 
       5.4536752546926678e-05, 
       5.4823193799954275e-05, 
       7.1403447915943303e-05, 
       5.9338104204380171e-05, 
       5.6986126373813316e-05, 
       6.1452082460627906e-05, 
       6.2450324586617445e-05, 
       6.5100923227102064e-05, 
       6.1391636171454107e-05, 
       6.9435771217651375e-05, 
       5.5963038522147751e-05, 
       6.2530567864370408e-05, 
       6.9167268456462962e-05, 
       7.5578720613505594e-05, 
       6.0521133845602216e-05, 
       6.5651183505185899e-05, 
       6.0122758624293347e-05, 
       7.5621206728768937e-05, 
       5.5671178888067848e-05, 
       5.7020332332487721e-05, 
       6.9418078355315405e-05, 
       7.4153623926901809e-05, 
       5.5670772558977532e-05, 
       7.6181881669031085e-05, 
       5.4152934220157735e-05, 
       7.6154745925831969e-05, 
       7.1994130050647689e-05, 
       6.5469739881167610e-05, 
       6.3308852028665555e-05, 
       6.3859059070151439e-05, 
       6.9999999999996267e-05, 
       7.0000000000004819e-05, 
       7.0000000000003003e-05, 
       5.7013116798884947e-05, 
       6.3714560075732259e-05, 
       7.2968238602742216e-05, 
       7.4254888482717910e-05, 
       7.0260475542549764e-05, 
       6.1301915205820201e-05, 
       5.3655710866274421e-05, 
       7.5611308701239476e-05, 
       6.4752873971490833e-05, 
       6.3815821636799638e-05, 
       6.7999993840232371e-05, 
       7.2388091326709255e-05, 
       7.4581497981184727e-05, 
       5.5097642372555272e-05, 
       6.1815259975918130e-05, 
       7.2968952904771010e-05, 
       6.2525499379224294e-05, 
       6.1675544193305282e-05, 
       7.2609882424347496e-05, 
       6.2632345928638076e-05, 
       7.3631452325070371e-05, 
       7.1920900723578259e-05, 
       6.1278462998428556e-05, 
       7.0404126608007687e-05, 
       5.3374378207900417e-05, 
       5.2953810785053414e-05, 
       6.2676821565421591e-05, 
       5.4208107973188428e-05, 
       7.4350850952534678e-05, 
       5.5903012209264134e-05, 
       7.1530779052606105e-05, 
       7.0544125382895765e-05, 
       7.3981679840751222e-05, 
       5.4637817072863970e-05, 
       5.8581128575405999e-05, 
       5.4454942426221525e-05, 
       5.3351956835571826e-05, 
       5.3569041825410595e-05, 
       5.8399683667779928e-05, 
       7.0492553212102906e-05, 
       5.9389539375220665e-05, 
       5.5756460692640313e-05, 
       6.9135663102043470e-05, 
       5.4239830530774268e-05, 
       5.5823776564372720e-05, 
       7.3052507124475637e-05, 
       5.9391032188061110e-05, 
       6.9389731175117396e-05, 
       5.8371814444202151e-05, 
       5.8361695767184039e-05, 
       5.3140106183809239e-05, 
       5.2538731658421197e-05, 
       7.3155927072294868e-05, 
       5.2548845364478694e-05, 
       5.7747045335037558e-05, 
       5.6333633705751053e-05, 
       5.8551628022872519e-05, 
       5.3919661770740322e-05, 
       5.2695829225956221e-05, 
       7.0707404064373417e-05, 
       5.4880795652292745e-05, 
       5.2994897044107277e-05, 
       7.0772914207727119e-05, 
       7.3520049375375574e-05, 
       6.2712054185075960e-05, 
       7.3358539146435272e-05, 
       7.2647133135199640e-05, 
       7.3281502635766943e-05, 
       6.4710196356528228e-05, 
       6.7554460421587098e-05, 
       5.4837516854305764e-05, 
       7.2882404801545201e-05, 
       5.3331870043678936e-05, 
       7.2377642960768355e-05, 
       5.3107525299106938e-05, 
       6.2402323313350813e-05, 
       5.3746768640790939e-05, 
       5.4852051732919532e-05, 
       7.2976956235294181e-05, 
       5.5715504773136874e-05, 
       7.2712231373442965e-05, 
       5.7821691739406147e-05, 
       5.4644058604852959e-05, 
       6.5503852183698607e-05, 
       6.2918695366846796e-05, 
       6.9028110165359383e-05, 
       7.2927196993623780e-05, 
       6.7818810337131808e-05, 
       5.8180969815989230e-05, 
       5.2277229812538866e-05, 
       5.2885731984462212e-05, 
       6.0634003001819919e-05, 
       6.0723573407856219e-05, 
       6.5792903285665392e-05, 
       5.1643206135485051e-05, 
       5.5635496467614915e-05, 
       5.2306118988525681e-05, 
       5.1489874333883744e-05, 
       5.3657723309819772e-05, 
       7.0505460494449583e-05, 
       7.2280886575799746e-05, 
       7.2684356865616753e-05, 
       5.2950100050595731e-05, 
       6.6679341668687649e-05, 
       5.3457587331172690e-05, 
       5.5496361067673006e-05, 
       5.9701741459847299e-05, 
       6.9338819658801707e-05, 
       5.1515672843550892e-05, 
       5.6940404780319242e-05, 
       6.5415175245451474e-05, 
       5.8590638433293021e-05, 
       6.9441843921330172e-05, 
       5.8030029043378005e-05, 
       5.1856533421948444e-05, 
       5.0926597531812841e-05, 
       5.5786819531077208e-05, 
       6.1301348644750942e-05, 
       6.9142646805354932e-05, 
       5.9264042032049323e-05, 
       5.4301579588420742e-05, 
       5.7225957435414691e-05, 
       5.9652253757711611e-05, 
       5.7146905961426048e-05, 
       6.4893626094914659e-05, 
       5.0803174066605268e-05, 
       5.9746029741379681e-05, 
       6.2127560580308650e-05, 
       5.3811870048602273e-05, 
       5.1189720894006389e-05, 
       6.5415150955390328e-05, 
       5.0862101514447519e-05, 
       6.0464805383182640e-05, 
       5.4244623883109573e-05, 
       5.7886304211728332e-05, 
       5.2053784143214063e-05, 
       5.0744622132762096e-05, 
       5.9439518842259396e-05, 
       7.1005939005932352e-05, 
       6.9269255178954875e-05, 
       6.6433327714632374e-05, 
       5.8893186649181449e-05, 
       5.0773205580234588e-05, 
       5.8509510152704110e-05, 
       6.3549956890922603e-05, 
       5.6576973974391862e-05, 
       5.3120923661899176e-05, 
       7.0950371238204677e-05, 
       6.5378066649362742e-05, 
       5.7303238460781716e-05, 
       5.0851196335983792e-05, 
       7.0459725896174018e-05, 
       5.8693773796014286e-05, 
       6.3695331317576109e-05, 
       5.0473907673574630e-05, 
       5.2237118897768206e-05, 
       7.0938130395567761e-05, 
       5.8237563178617421e-05, 
       5.0192432490451184e-05, 
       5.4233794547780502e-05, 
       6.0000000000000862e-05, 
       6.6617042000594276e-05, 
       6.8310130128675591e-05, 
       5.6527273877266639e-05, 
       7.0031955131841952e-05, 
       5.9705357397423808e-05, 
       6.9818012339344324e-05, 
       5.3097116520689287e-05, 
       5.8682533186491593e-05, 
       5.0369949563792117e-05, 
       5.0818625851728102e-05, 
       5.6061561357567273e-05, 
       5.8488201816057606e-05, 
       5.2904242858111470e-05, 
       6.4737146596510647e-05, 
       5.3521182847414386e-05, 
       6.9999999999997907e-05, 
       6.9999999999995237e-05, 
       6.9999999999999370e-05, 
       6.9999999999995901e-05, 
       7.0000000000003016e-05, 
       7.0000000000004182e-05, 
       6.7507811295745498e-05, 
       6.9999999999999994e-05, 
       5.1904743481132965e-05, 
       6.9911850920685790e-05, 
       6.8774555393876250e-05, 
       5.0853941962980113e-05, 
       5.5974749885326160e-05, 
       5.2196258461919686e-05, 
       6.5888879986082785e-05, 
       6.9647035199035261e-05, 
       5.9318592053048362e-05, 
       4.9941989778631981e-05, 
       5.5070065180113226e-05, 
       6.9667928372789176e-05, 
       6.0294293146174899e-05, 
       6.9614617916649192e-05, 
       6.0249719517538580e-05, 
       6.9634717973311532e-05, 
       5.3406994566046547e-05, 
       6.0075649808176604e-05, 
       5.7856110701220593e-05, 
       5.4194517124298945e-05, 
       5.9939807033001485e-05, 
       5.7788985598938492e-05, 
       5.7654209187841779e-05, 
       5.3245664476029690e-05, 
       6.8294990629212064e-05, 
       5.1819830695422259e-05, 
       5.1793064135587944e-05, 
       5.5692871739847024e-05, 
       5.5638458220648930e-05, 
       5.5061501746463422e-05, 
       5.9418269234674275e-05, 
       5.9418269234674275e-05, 
       6.9102842187250158e-05, 
       6.9202106627658342e-05, 
       6.8059553882645187e-05, 
       6.7118875069030451e-05, 
       4.9429592954613257e-05, 
       4.9438691666407176e-05, 
       6.4574574496992495e-05, 
       4.9075947065865876e-05, 
       6.6372181595925032e-05, 
       5.4211980298756077e-05, 
       5.6282488491592738e-05, 
       4.9733566373602802e-05, 
       4.9281556165954021e-05, 
       5.2386447909112655e-05, 
       5.8339756928813070e-05, 
       4.9082128531763488e-05, 
       5.4547897393177602e-05, 
       6.7154585161798426e-05, 
       5.1180978976347045e-05, 
       5.0059468117206730e-05, 
       5.7035029892671697e-05, 
       5.6224138340778580e-05, 
       6.5694941003993014e-05, 
       6.8423786791217876e-05, 
       6.7401074282846720e-05, 
       6.1057578073188745e-05, 
       5.6929735774859507e-05, 
       5.3553166895624647e-05, 
       5.7135445583040911e-05, 
       4.9173252232845279e-05, 
       6.8124712001435878e-05, 
       6.1982253217940286e-05, 
       4.9986025733025579e-05, 
       5.6304678819758760e-05, 
       5.6077226078964582e-05, 
       6.1721699720896365e-05, 
       5.1949826657155597e-05, 
       6.6271437407158890e-05, 
       5.8727230508765282e-05, 
       6.8058160232902756e-05, 
       6.7467970174608425e-05, 
       5.5839685787154337e-05, 
       5.1527046071899540e-05, 
       6.7950392033506567e-05, 
       5.1439873685258851e-05, 
       6.6429942082994295e-05, 
       4.8441815683187511e-05, 
       5.7788836592711078e-05, 
       6.5547984280585531e-05, 
       6.4969454356405900e-05, 
       5.5486439714172744e-05, 
       5.6830797803405895e-05, 
       5.8422499988691722e-05, 
       6.7888603362576452e-05, 
       4.8755500361795478e-05, 
       5.7676820514249529e-05, 
       4.8890909345170752e-05, 
       5.1986824794424328e-05, 
       5.0516744969084998e-05, 
       6.5180862467621351e-05, 
       6.6768447452326203e-05, 
       5.6428375356962725e-05, 
       4.8411150360027856e-05, 
       4.8829215634594100e-05, 
       5.9012176974199628e-05, 
       4.8190863403155121e-05, 
       4.8532461585484179e-05, 
       5.7990513324048184e-05, 
       6.7210421847840339e-05, 
       4.9065488049903942e-05, 
       4.9131588130123420e-05, 
       5.6295777949839289e-05, 
       5.6112155712866247e-05, 
       5.5421443814026223e-05, 
       5.7495826607724812e-05, 
       6.6976872419193898e-05, 
       4.7798188035033005e-05, 
       5.3543030848884162e-05, 
       4.8068768999675456e-05, 
       5.1168250063573888e-05, 
       5.2562102491930520e-05, 
       5.8360583435439789e-05, 
       4.9823361044004298e-05, 
       5.1239583190533600e-05, 
       5.9607253389648816e-05, 
       4.8856689878798525e-05, 
       6.6840243607990866e-05, 
       4.9155784851950384e-05, 
       5.5230300624771765e-05, 
       4.9817461477581836e-05, 
       5.0980848232306575e-05, 
       6.3992345259856762e-05, 
       4.7403145917500183e-05, 
       5.2624898170209190e-05, 
       6.6342692928062432e-05, 
       5.4720833273606180e-05, 
       5.8860423443622196e-05, 
       4.8628639255658508e-05, 
       6.3960307193163421e-05, 
       5.5560927253584967e-05, 
       6.6387015201549505e-05, 
       6.3003597891985046e-05, 
       5.7322980292546794e-05, 
       5.4893961997201745e-05, 
       5.5210723935268091e-05, 
       6.0847182112127938e-05, 
       6.1191053389739221e-05, 
       5.4800471870091196e-05, 
       4.8899093720158621e-05, 
       5.4415035528300198e-05, 
       4.7176606649280219e-05, 
       5.0149155019542366e-05, 
       5.7461491671426802e-05, 
       4.7595154813169758e-05, 
       6.5930833081932947e-05, 
       4.6680851352347405e-05, 
       6.2836045975070653e-05, 
       6.2836045975070653e-05, 
       5.7117989316320992e-05, 
       5.8084648482367822e-05, 
       4.6595005113552475e-05, 
       6.4805751040175906e-05, 
       5.1628197703498566e-05, 
       5.4954100343644434e-05, 
       6.4509697287140613e-05, 
       5.2277501459207974e-05, 
       4.7363900372736404e-05, 
       6.5500318321853570e-05, 
       6.2986434129051402e-05, 
       5.4905022482281828e-05, 
       4.8783702524048636e-05, 
       6.5336635269507286e-05, 
       5.7482534836658004e-05, 
       5.5456155417069248e-05, 
       6.2110551122138797e-05, 
       5.5367691047924030e-05, 
       6.0394125750386610e-05, 
       5.6393531088359224e-05, 
       4.8277076033647455e-05, 
       6.1963447890564172e-05, 
       5.9999999999996180e-05, 
       6.0000000000004040e-05, 
       5.9999999999999866e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       6.0000000000004996e-05, 
       4.8919005728962156e-05, 
       4.8020646058948294e-05, 
       4.6075908501621943e-05, 
       5.5636134083119825e-05, 
       4.9255790464045804e-05, 
       5.3665314933835224e-05, 
       5.0587831557503732e-05, 
       5.0370830344259781e-05, 
       5.6885949512116897e-05, 
       5.2672638865111929e-05, 
       5.0044337706742301e-05, 
       5.3704790463499113e-05, 
       6.3986119981049801e-05, 
       6.4648429997868714e-05, 
       5.2232735224126247e-05, 
       4.6059236816611589e-05, 
       5.2525298630325122e-05, 
       4.8532066647182072e-05, 
       5.2624022791551378e-05, 
       5.8875311616523331e-05, 
       5.8644737820742914e-05, 
       6.4368374490639133e-05, 
       4.8200494403544152e-05, 
       5.0720410533912382e-05, 
       5.1656215335812869e-05, 
       5.6554325964458382e-05, 
       6.1931035142444632e-05, 
       4.7389002589603285e-05, 
       5.2015638070832189e-05, 
       5.3186552918579861e-05, 
       5.5126275743094082e-05, 
       5.6521964227868431e-05, 
       5.0162084659005960e-05, 
       4.9258018642439319e-05, 
       6.1199966769753114e-05, 
       6.2350132761167278e-05, 
       5.3331492340606802e-05, 
       4.8172436837364844e-05, 
       4.5468663550895849e-05, 
       5.1057102444195728e-05, 
       5.3811322428276546e-05, 
       5.5370020169628080e-05, 
       5.0823814561563740e-05, 
       4.6626525578432450e-05, 
       5.5199044945956439e-05, 
       4.5496098109817633e-05, 
       5.1633336126447875e-05, 
       6.2639501948292821e-05, 
       4.6815621176611117e-05, 
       4.5319500355535343e-05, 
       5.0729915650022426e-05, 
       5.4896094376021840e-05, 
       5.0867937525453185e-05, 
       4.9934020721311711e-05, 
       6.0349450426038604e-05, 
       5.9439584788231909e-05, 
       4.6462275554701517e-05, 
       6.3937238831808708e-05, 
       5.2120247240544914e-05, 
       6.3866136969769688e-05, 
       4.6182803705393720e-05, 
       6.0119680112954966e-05, 
       5.3153732340189933e-05, 
       6.2472058227759644e-05, 
       5.0973623751609602e-05, 
       5.2907412242889158e-05, 
       5.5242338632736270e-05, 
       4.7603016568030278e-05, 
       4.8581084845547012e-05, 
       6.1665049247742589e-05, 
       6.3461028263481515e-05, 
       6.3460363199534445e-05, 
       5.7161925993514645e-05, 
       5.4852632436185530e-05, 
       6.2316493850177404e-05, 
       4.6761215110134757e-05, 
       6.1892017862394814e-05, 
       5.5015505677616779e-05, 
       4.6239570968043748e-05, 
       6.0463965265838370e-05, 
       6.2835813834466727e-05, 
       4.5073610026391611e-05, 
       5.1697356441486444e-05, 
       6.3032402181799788e-05, 
       5.6187922975949822e-05, 
       5.3793905913886575e-05, 
       5.0874171686920067e-05, 
       5.3997177171280784e-05, 
       4.6985874249167178e-05, 
       4.8763907431357866e-05, 
       5.2343888002231213e-05, 
       5.4943735044086137e-05, 
       4.6230089486701635e-05, 
       4.7021951831774940e-05, 
       6.1949003404044624e-05, 
       6.0960842892658908e-05, 
       6.0489937684067887e-05, 
       6.1449521589411731e-05, 
       5.5709207923670473e-05, 
       4.5982682345389509e-05, 
       6.2589872005379315e-05, 
       6.2582958198720948e-05, 
       5.4200546958542493e-05, 
       5.4415096140113719e-05, 
       5.1181059138378605e-05, 
       4.8506650141339845e-05, 
       5.3886848493621039e-05, 
       5.0208019389186380e-05, 
       4.5736473110669535e-05, 
       4.5084412361438301e-05, 
       5.1395366811943550e-05, 
       5.0649091619831184e-05, 
       5.9937319091337540e-05, 
       5.2314328297484447e-05, 
       4.8433251536815609e-05, 
       4.6638335520830991e-05, 
       5.0006524759705309e-05, 
       6.2340328410111174e-05, 
       6.1077412136091285e-05, 
       4.6204623839822289e-05, 
       5.9977976175652429e-05, 
       5.4022648021469051e-05, 
       4.4156965075618014e-05, 
       4.6246180472292401e-05, 
       5.1757954173142231e-05, 
       4.8244733671149489e-05, 
       5.1620125404129000e-05, 
       5.1725960944374309e-05, 
       5.1180115907803281e-05, 
       4.8762279412665143e-05, 
       5.1641541890932526e-05, 
       5.8943117718298882e-05, 
       4.4000839163406419e-05, 
       4.6232680008718511e-05, 
       4.9623448059495636e-05, 
       4.3845596249394962e-05, 
       5.2493474454416795e-05, 
       4.3931435054792515e-05, 
       4.5499899545180348e-05, 
       4.8744009384053409e-05, 
       4.4768635481360915e-05, 
       5.1010745314152139e-05, 
       4.3891094761038179e-05, 
       6.0650641962860851e-05, 
       4.3620840744908696e-05, 
       5.4747940780180304e-05, 
       5.1128901078611637e-05, 
       5.6163479836108622e-05, 
       5.0454579442896844e-05, 
       5.0959804372513256e-05, 
       4.5808848202632811e-05, 
       4.8316786522032077e-05, 
       5.3947820151145772e-05, 
       4.9906266041516460e-05, 
       4.9903041618072622e-05, 
       5.5553630235743111e-05, 
       4.4709685269662970e-05, 
       4.7411841094515627e-05, 
       4.9646011883341925e-05, 
       4.3741602841152682e-05, 
       4.5088673379960427e-05, 
       5.2515552633123617e-05, 
       4.5740945497191634e-05, 
       4.5667250007802094e-05, 
       4.3492508872549069e-05, 
       4.3766582934701616e-05, 
       4.5127952633870439e-05, 
       4.3312680122956944e-05, 
       6.0744279709598826e-05, 
       5.9810354663154359e-05, 
       5.3078133849708067e-05, 
       4.7617362384289807e-05, 
       5.3204472630424375e-05, 
       5.1655198458111211e-05, 
       4.9486079152464172e-05, 
       5.2417380696785202e-05, 
       4.4260246306852084e-05, 
       5.3529752387855166e-05, 
       5.7775698063934512e-05, 
       5.3550349496016404e-05, 
       4.3390146961430143e-05, 
       5.7578787324899365e-05, 
       6.0379542999331506e-05, 
       6.0141308807401612e-05, 
       5.1263794073019253e-05, 
       5.4321434884439146e-05, 
       6.0606394476908003e-05, 
       4.3736925885953352e-05, 
       4.5918444007100729e-05, 
       5.8037474782733717e-05, 
       4.5029398657819523e-05, 
       5.5450947744660237e-05, 
       5.9999999999995550e-05, 
       4.8183469016338309e-05, 
       4.2864905074471364e-05, 
       5.7421694509391550e-05, 
       4.6999246590521435e-05, 
       4.3314673945254786e-05, 
       4.3826639630942426e-05, 
       4.5099790812813945e-05, 
       4.4608464834851515e-05, 
       4.9912183775591017e-05, 
       5.4774717392822652e-05, 
       4.6112863593300849e-05, 
       5.8165673534554548e-05, 
       4.4528566541860135e-05, 
       4.5238136185692460e-05, 
       4.3843161888724208e-05, 
       5.0030142072056702e-05, 
       4.3088401082979049e-05, 
       4.2647939732095430e-05, 
       4.3413155913303453e-05, 
       5.2427380340666423e-05, 
       5.9999999999999310e-05, 
       5.9999999999996546e-05, 
       6.0000000000003742e-05, 
       6.0000000000002556e-05, 
       5.9999999999999527e-05, 
       6.0000000000000422e-05, 
       5.9999999999997874e-05, 
       5.9999999999997874e-05, 
       6.0000000000000849e-05, 
       5.9999999999998551e-05, 
       6.0000000000004779e-05, 
       6.0000000000000002e-05, 
       4.9989525692923144e-05, 
       4.9477025590364511e-05, 
       4.2914044060904132e-05, 
       4.7654349608539087e-05, 
       4.9885184164457889e-05, 
       4.4170852386683296e-05, 
       4.6180437730363466e-05, 
       5.7784283654206190e-05, 
       4.2321376650598003e-05, 
       5.4742201976634431e-05, 
       4.4559611633559966e-05, 
       5.9479462526411957e-05, 
       4.5428048789755465e-05, 
       4.9340762952638152e-05, 
       4.9117557474239952e-05, 
       4.3228013945769331e-05, 
       5.8138216111392133e-05, 
       4.6936812289581863e-05, 
       4.3059115108043826e-05, 
       5.9555156319000111e-05, 
       5.9396006082526388e-05, 
       5.0374372456593259e-05, 
       4.6403384076973468e-05, 
       5.4639016987122717e-05, 
       4.9254955487342607e-05, 
       4.9021088218738301e-05, 
       5.6814828119280989e-05, 
       4.2431641445317442e-05, 
       4.5757139751888294e-05, 
       5.9343876890261234e-05, 
       4.2028676301942872e-05, 
       4.3723196396817918e-05, 
       4.8736183382274307e-05, 
       4.9737012615764975e-05, 
       4.4502557313300532e-05, 
       4.2955638760384177e-05, 
       4.9320193414117061e-05, 
       5.8709160707257089e-05, 
       4.2369607953665634e-05, 
       4.1912103048150174e-05, 
       4.9274741048537857e-05, 
       4.9079260267646955e-05, 
       4.7131408596580615e-05, 
       4.8529906815559230e-05, 
       5.0032365735497000e-05, 
       4.4216806516761800e-05, 
       5.0473824958167644e-05, 
       5.4990747884506360e-05, 
       4.2995825998694557e-05, 
       4.8956344907266142e-05, 
       4.1643488907769427e-05, 
       5.0000000000004027e-05, 
       4.9102565603596434e-05, 
       4.2056231593610980e-05, 
       5.6551370220277830e-05, 
       4.1473351769081861e-05, 
       4.2660196591858984e-05, 
       4.8466105959378858e-05, 
       4.3664491349537101e-05, 
       4.7019190130416136e-05, 
       4.2020703594997762e-05, 
       4.2197521725418601e-05, 
       4.4050240165943549e-05, 
       4.5173091710291490e-05, 
       5.7387871179809036e-05, 
       5.8242958955245922e-05, 
       4.1387373846525672e-05, 
       4.5658650850494859e-05, 
       5.8380061506390137e-05, 
       4.9400488295855784e-05, 
       5.5626860385196315e-05, 
       4.7591523025750123e-05, 
       4.5605417099392996e-05, 
       4.5827593137828634e-05, 
       4.5597046260994381e-05, 
       4.5938540269852517e-05, 
       4.1239694363495530e-05, 
       5.7390012273906227e-05, 
       4.2623090897413479e-05, 
       5.6879615881749153e-05, 
       4.9190272230195365e-05, 
       4.9999999999999792e-05, 
       4.6034348380751324e-05, 
       4.1306516225154326e-05, 
       4.1306516225154326e-05, 
       4.2419183164565412e-05, 
       5.7927127405167101e-05, 
       4.1355279233160817e-05, 
       5.7986910355144224e-05, 
       4.8580462608542503e-05, 
       4.7612948134855449e-05, 
       5.6949260027469635e-05, 
       4.6481445271418442e-05, 
       5.7838111169947167e-05, 
       5.1333192981771837e-05, 
       4.6656464285117813e-05, 
       4.1056045656931586e-05, 
       4.3080856617576129e-05, 
       4.2539910213325332e-05, 
       4.4316438860835246e-05, 
       4.2269531053290492e-05, 
       4.4289595602791237e-05, 
       5.7598221827485390e-05, 
       4.3027901919575666e-05, 
       4.2344777273661307e-05, 
       5.7691783165335331e-05, 
       4.1893098273142994e-05, 
       4.1378991254230658e-05, 
       4.6680851352351701e-05, 
       5.3344797584888999e-05, 
       4.6561259790826225e-05, 
       4.6521148979778008e-05, 
       5.7431995387824092e-05, 
       4.0743268883048198e-05, 
       5.1702184280872233e-05, 
       4.2589252683174096e-05, 
       5.5276504598079247e-05, 
       5.5138742277526770e-05, 
       5.6835586211109020e-05, 
       4.6937118773572440e-05, 
       4.6421109921223438e-05, 
       5.7237172261648808e-05, 
       4.1784757532711830e-05, 
       5.4828682688127821e-05, 
       4.2726541807463150e-05, 
       5.3871417177149840e-05, 
       4.4191057740170973e-05, 
       4.0654368970348233e-05, 
       4.8420669905112727e-05, 
       4.7074138740620341e-05, 
       4.5726308369385158e-05, 
       4.1698844107093833e-05, 
       4.0508321643157128e-05, 
       4.8589159881024081e-05, 
       4.7475244498203241e-05, 
       4.0422237972570740e-05, 
       4.0630103915450757e-05, 
       5.6853074152897302e-05, 
       4.8653027493779121e-05, 
       4.1181911521023013e-05, 
       4.1807599914309833e-05, 
       4.7131760382240903e-05, 
       4.1487245110593908e-05, 
       4.1392752854613889e-05, 
       5.3736873559307385e-05, 
       4.0169583561656173e-05, 
       4.1610480757795027e-05, 
       3.9986283148078992e-05, 
       4.8476446646087345e-05, 
       4.2181077020240292e-05, 
       5.5220532245943299e-05, 
       5.6325547402070667e-05, 
       4.8282448144517436e-05, 
       5.4097829665084227e-05, 
       5.4402092406479162e-05, 
       5.6198027089088832e-05, 
       4.8738662300079464e-05, 
       4.1190115314639731e-05, 
       4.0206985399988753e-05, 
       3.9822533051480654e-05, 
       5.3533889858325180e-05, 
       4.8810598171253817e-05, 
       4.6367848408419797e-05, 
       4.8473308285840820e-05, 
       5.3482105176086828e-05, 
       4.6746429035703899e-05, 
       3.9928326222434816e-05, 
       4.0446738261275096e-05, 
       3.9999999999999251e-05, 
       3.9825616636154748e-05, 
       4.3094739618661663e-05, 
       4.5869008981259229e-05, 
       4.3476026373925649e-05, 
       3.9999999999999149e-05, 
       4.6679723539361222e-05, 
       4.1793679441100703e-05, 
       4.0045345043157613e-05, 
       4.0620579343639523e-05, 
       4.5533050701295619e-05, 
       5.4865128331398789e-05, 
       5.4876726358108719e-05, 
       5.4876726358108719e-05, 
       5.4933362672606455e-05, 
       4.8403188023473836e-05, 
       4.0503154181217877e-05, 
       4.0747971692862886e-05, 
       3.9999999999999204e-05, 
       4.3053301163211905e-05, 
       5.2872649988757145e-05, 
       4.9413415375108390e-05, 
       5.3349872581457082e-05, 
       4.6609530599120183e-05, 
       4.1634647590973212e-05, 
       5.5566698936445724e-05, 
       4.7317523129035093e-05, 
       5.5415502577735319e-05, 
       4.0000000000004198e-05, 
       4.6872977944749447e-05, 
       4.6872977944749447e-05, 
       4.7551620231641707e-05, 
       5.2716549386309342e-05, 
       5.5420129129652183e-05, 
       4.7984379484198297e-05, 
       5.5231082239972103e-05, 
       4.1227820768257724e-05, 
       5.2766396882121334e-05, 
       5.5277396251773343e-05, 
       4.7694693288193728e-05, 
       4.8947934039797070e-05, 
       4.5971446371953517e-05, 
       4.8788722750148041e-05, 
       4.5840655752655039e-05, 
       5.5156359921801346e-05, 
       4.0237339154447129e-05, 
       4.8029213486533319e-05, 
       4.5762989514534655e-05, 
       4.1998963819514386e-05, 
       5.3553287300580787e-05, 
       5.3885912021023566e-05, 
       4.7268150785897822e-05, 
       4.9999999999996926e-05, 
       4.7222674191666060e-05, 
       4.5516656777274089e-05, 
       5.4927576134101640e-05, 
       4.2017845779234498e-05, 
       3.9217118803972750e-05, 
       4.0758562369121633e-05, 
       4.0758562369121633e-05, 
       3.9829763113082735e-05, 
       4.2016701592091930e-05, 
       4.5797852167756901e-05, 
       3.9777342969000458e-05, 
       3.9889276389257778e-05, 
       3.9883810021708140e-05, 
       4.6382659451069880e-05, 
       5.2936145234419369e-05, 
       5.3854612408325158e-05, 
       5.3681544034002672e-05, 
       3.8757084183542985e-05, 
       3.8757084183542985e-05, 
       5.3129947971102481e-05, 
       4.8210346693396730e-05, 
       3.8682632566986381e-05, 
       4.1907428291848185e-05, 
       3.8912556520324435e-05, 
       4.9405678723330799e-05, 
       4.9424947177337444e-05, 
       3.9555536339518008e-05, 
       4.1880495300854461e-05, 
       5.4460827879384862e-05, 
       5.2783399198899163e-05, 
       5.3344018500666361e-05, 
       4.5301328581767161e-05, 
       4.0624931956256581e-05, 
       3.8689854482162302e-05, 
       4.5252740925766792e-05, 
       4.6415029432686081e-05, 
       5.4340489586031172e-05, 
       4.2564967480046287e-05, 
       4.5130349353345287e-05, 
       5.0000000000004299e-05, 
       5.0000000000002733e-05, 
       5.0000000000002794e-05, 
       4.9999999999996086e-05, 
       4.9999999999996479e-05, 
       3.8367026359383460e-05, 
       4.1812135500619011e-05, 
       3.8399553086101836e-05, 
       5.2504843594545961e-05, 
       4.6566756486422909e-05, 
       4.5025160670482117e-05, 
       3.9332741397176787e-05, 
       4.4833940213453963e-05, 
       3.9909229550201967e-05, 
       3.9999999999998485e-05, 
       4.0673051949744794e-05, 
       4.1192008757856732e-05, 
       3.9244089817564084e-05, 
       4.6463944138561079e-05, 
       4.0609923515246993e-05, 
       3.8205542051530889e-05, 
       4.1758445988407313e-05, 
       4.2582790376141000e-05, 
       3.8690818687620413e-05, 
       5.1054341164156614e-05, 
       4.0184014857642677e-05, 
       4.3071388782582017e-05, 
       3.8556120392793670e-05, 
       4.1186902337199184e-05, 
       3.8125340577283374e-05, 
       4.4898060934481598e-05, 
       3.8607437743775037e-05, 
       3.8579122539990380e-05, 
       4.4102334302382840e-05, 
       4.4092800625148512e-05, 
       3.8165375988579617e-05, 
       3.8221594349099795e-05, 
       3.9217597300470891e-05, 
       4.2026673184816736e-05, 
       4.5798996007120656e-05, 
       4.7271580203303840e-05, 
       4.2169464403581484e-05, 
       4.4583013873295372e-05, 
       5.2507353929527501e-05, 
       4.1897194718015050e-05, 
       4.4101190080671204e-05, 
       4.7935538673683188e-05, 
       3.8719379231439572e-05, 
       5.1580267387044076e-05, 
       4.3848316668304945e-05, 
       4.1198369059005749e-05, 
       4.6700887184167944e-05, 
       4.4393980442063270e-05, 
       5.1775690705076777e-05, 
       3.7671434409166956e-05, 
       4.5465024101112944e-05, 
       4.1578124282161413e-05, 
       3.9012974169455260e-05, 
       4.1314061750878910e-05, 
       4.7428509199940162e-05, 
       4.9730818209690336e-05, 
       4.0198860611875665e-05, 
       3.7756745272042277e-05, 
       4.4089243669457669e-05, 
       4.5462921802828011e-05, 
       3.9499044748964214e-05, 
       4.2824123629750374e-05, 
       4.5737826905557407e-05, 
       4.8529081539453502e-05, 
       4.9588076603087392e-05, 
       5.1981660451376669e-05, 
       5.0871420957154051e-05, 
       5.1144548571629179e-05, 
       3.9490149743067477e-05, 
       3.9031905915225573e-05, 
       4.5831677211210600e-05, 
       4.1444965641118307e-05, 
       5.2756724081492818e-05, 
       4.3401840076277341e-05, 
       4.1262132449701108e-05, 
       4.7361048466102391e-05, 
       4.8148587905372334e-05, 
       4.5249921745478738e-05, 
       3.7379471877535428e-05, 
       3.8171256090277061e-05, 
       4.5420754043660291e-05, 
       3.9999999999999956e-05, 
       4.0913829527531320e-05, 
       3.9225465815228226e-05, 
       3.8862509784538389e-05, 
       4.7760596118884652e-05, 
       5.2589650454665371e-05, 
       4.3802123322492794e-05, 
       5.0455662279347686e-05, 
       5.1016698778043452e-05, 
       5.0973232291483467e-05, 
       3.7254871645390122e-05, 
       3.9636665763166410e-05, 
       4.4500780153024367e-05, 
       4.0617568523987939e-05, 
       3.7252201921401205e-05, 
       3.7149256373465534e-05, 
       4.5323288458288803e-05, 
       3.8855997946358206e-05, 
       5.2135608610241341e-05, 
       5.0586978896262118e-05, 
       3.9332831215316651e-05, 
       3.8919190428930292e-05, 
       4.5709179834956401e-05, 
       3.8527016130150091e-05, 
       5.1810373328764550e-05, 
       3.8352374870641786e-05, 
       5.0559596881809618e-05, 
       4.2827255512977396e-05, 
       3.9507410929693281e-05, 
       5.0222619983382302e-05, 
       3.8776413196605887e-05, 
       3.8771762343032301e-05, 
       3.7213843409197711e-05, 
       4.3116272454786127e-05, 
       3.7551982842666390e-05, 
       4.3144032977884826e-05, 
       3.9039453027332062e-05, 
       3.6882089171293840e-05, 
       5.0137421064268391e-05, 
       4.3071053584278631e-05, 
       4.7494456055885984e-05, 
       4.7869477351075516e-05, 
       4.1039403876113906e-05, 
       4.7737029508300656e-05, 
       3.9025088162159966e-05, 
       3.6585694275826116e-05, 
       5.0819542019124864e-05, 
       4.0000000000002443e-05, 
       4.3561658624952643e-05, 
       4.4621306052016770e-05, 
       5.1539034031555042e-05, 
       4.4267521473492807e-05, 
       4.9059004569314347e-05, 
       4.2481388019654305e-05, 
       3.6781291270641367e-05, 
       4.9466585659021100e-05, 
       3.6639145793717715e-05, 
       3.6481849489570408e-05, 
       3.6915195434491318e-05, 
       4.9897097790425700e-05, 
       5.1068642799919907e-05, 
       3.9213912996997664e-05, 
       3.8302494563673123e-05, 
       3.6908670353626897e-05, 
       3.7747095218466348e-05, 
       3.6507257051977693e-05, 
       5.0623393126359428e-05, 
       3.6294870987292694e-05, 
       5.0407804196197774e-05, 
       4.8626464090568084e-05, 
       4.5903871677841165e-05, 
       3.6305220859840208e-05, 
       3.9156004466396121e-05, 
       5.1079647754534503e-05, 
       4.1126183676543897e-05, 
       4.4256456102755829e-05, 
       5.1072772878516972e-05, 
       4.1088050080020714e-05, 
       3.8098824929514733e-05, 
       3.6292802712483733e-05, 
       3.7881831586116264e-05, 
       3.8011160779265816e-05, 
       3.6225151056930629e-05, 
       4.6732009248430430e-05, 
       4.2689216259987502e-05, 
       3.6176113956295823e-05, 
       4.8325881984923283e-05, 
       3.9703483036919981e-05, 
       3.9672283088894010e-05, 
       4.9150003984808674e-05, 
       4.8673415704524168e-05, 
       3.7337748916521062e-05, 
       3.6606600540253771e-05, 
       3.6606600540253771e-05, 
       4.4169456576088538e-05, 
       3.8698521120121164e-05, 
       4.1152193881715595e-05, 
       4.0000000000000789e-05, 
       3.7942729673331959e-05, 
       4.2442945199250477e-05, 
       3.5921531897822069e-05, 
       4.1782907660022691e-05, 
       4.8123251743939720e-05, 
       3.6415494337422460e-05, 
       4.0838629665093989e-05, 
       4.8546991242800412e-05, 
       5.0686307697402933e-05, 
       4.7748508360558765e-05, 
       4.0400442778724485e-05, 
       3.7836951681355312e-05, 
       5.0655816488747451e-05, 
       4.2163479568077072e-05, 
       4.0857128528670636e-05, 
       5.0550680007651935e-05, 
       4.5659202385955215e-05, 
       3.5948592860459729e-05, 
       3.9366194457792237e-05, 
       4.9699248776905651e-05, 
       4.8876354551017700e-05, 
       4.9362622579371397e-05, 
       4.3667019327052607e-05, 
       4.1246289596223742e-05, 
       3.9013566052538380e-05, 
       3.7110415917498557e-05, 
       4.0620062413580923e-05, 
       4.2667778713392756e-05, 
       3.9932816694817713e-05, 
       5.0406696139146041e-05, 
       4.3396413401811097e-05, 
       4.9702100826339363e-05, 
       5.0353951837400616e-05, 
       4.1825744852813942e-05, 
       3.8175300453073919e-05, 
       3.6637391558180805e-05, 
       4.5041749499095774e-05, 
       3.7791254807466405e-05, 
       4.6947102251870528e-05, 
       4.9271502059371876e-05, 
       4.9999800902817170e-05, 
       4.0386545168513681e-05, 
       4.8164222572487311e-05, 
       4.1327683193812462e-05, 
       4.9999999999999792e-05, 
       5.0000000000001107e-05, 
       4.2980472829982204e-05, 
       4.0141286768363031e-05, 
       4.4223338451110325e-05, 
       4.0171164336368798e-05, 
       3.7419109782125890e-05, 
       4.9999999999996384e-05, 
       4.9999999999996086e-05, 
       5.0000000000001364e-05, 
       4.9999999996266254e-05, 
       5.0000000000001392e-05, 
       4.9999999999999216e-05, 
       4.9999999999997848e-05, 
       4.9999999999997848e-05, 
       5.0000000000001378e-05, 
       4.9999999999999819e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       4.3693215212707962e-05, 
       4.7174481025077144e-05, 
       4.9871287121128939e-05, 
       3.5421382611125771e-05, 
       4.7753676166647604e-05, 
       3.8008409109077445e-05, 
       3.8008409109077445e-05, 
       4.1566081444236976e-05, 
       4.7533196668052104e-05, 
       3.6735639398153767e-05, 
       4.9742904497568775e-05, 
       4.9522709122576209e-05, 
       4.3197275492044433e-05, 
       4.2937241449752606e-05, 
       3.9187780023383066e-05, 
       4.7939155819036232e-05, 
       3.7683266551203620e-05, 
       3.6727043111546528e-05, 
       3.5695488712657605e-05, 
       4.9256753497276079e-05, 
       4.1560731014285226e-05, 
       4.9216370954988458e-05, 
       3.5071356445811760e-05, 
       3.6082568143536867e-05, 
       3.8529478570692213e-05, 
       3.7632341137863449e-05, 
       4.4407288932550977e-05, 
       3.6700633488085201e-05, 
       3.7197643616710230e-05, 
       3.9811070062031266e-05, 
       3.5440652903296097e-05, 
       4.9144327677729245e-05, 
       4.5368754043618605e-05, 
       3.7070075310274946e-05, 
       3.7879033009118545e-05, 
       3.7824005790794645e-05, 
       4.9250996958964661e-05, 
       4.0947043244973540e-05, 
       4.8978983987269091e-05, 
       4.7486787626063863e-05, 
       3.5109510528548566e-05, 
       3.6010377256048052e-05, 
       3.6869525089517571e-05, 
       4.1110914622485778e-05, 
       4.2994690261453100e-05, 
       4.9133388421174224e-05, 
       4.1160183034695845e-05, 
       4.0888575439857405e-05, 
       3.9848762004838509e-05, 
       4.0960967167305764e-05, 
       3.5377295576300438e-05, 
       3.6845818446800123e-05, 
       3.4892133030956602e-05, 
       4.1997326890760992e-05, 
       3.6385140237877915e-05, 
       3.8957918846048882e-05, 
       4.8790913105544509e-05, 
       3.8871921550046947e-05, 
       4.4001622505033822e-05, 
       3.9999999999995077e-05, 
       3.8116922979120940e-05, 
       3.6443967291686463e-05, 
       4.1269404017097563e-05, 
       4.2427313383739517e-05, 
       4.2455284234732741e-05, 
       3.5380644425587888e-05, 
       4.7188308956427515e-05, 
       4.0416564189908883e-05, 
       4.0809087851262582e-05, 
       4.8695273996632675e-05, 
       3.6315767622023942e-05, 
       4.8700468027484211e-05, 
       3.5101411679641292e-05, 
       3.5101411679641292e-05, 
       4.7410702519995419e-05, 
       4.0533970471310822e-05, 
       4.1685295592839673e-05, 
       3.7301410440252810e-05, 
       4.8587599942772118e-05, 
       3.7416580886410366e-05, 
       4.3072197197748938e-05, 
       4.8362630978951883e-05, 
       3.4566380536288469e-05, 
       3.8193249630209771e-05, 
       4.0312656651805728e-05, 
       3.5040657082742915e-05, 
       4.8265519258935261e-05, 
       4.1540410754525179e-05, 
       3.6365833631586140e-05, 
       3.5085918624153234e-05, 
       3.6799077068152171e-05, 
       4.7585585037454157e-05, 
       3.5857311796147378e-05, 
       4.3480494615414690e-05, 
       4.6203620805737069e-05, 
       3.9756700645838978e-05, 
       4.8003699574459899e-05, 
       3.5274007499668463e-05, 
       4.0234289386043077e-05, 
       3.8205301869610130e-05, 
       3.4907102358461323e-05, 
       3.9837751300035644e-05, 
       3.4171605336092669e-05, 
       3.9814218009852564e-05, 
       3.4906469719833722e-05, 
       4.6042218920033771e-05, 
       3.5116448860824046e-05, 
       3.5744734427003745e-05, 
       4.7701491599618533e-05, 
       4.0993447561416658e-05, 
       3.5116821165118841e-05, 
       3.4531050335229208e-05, 
       3.3966540657473428e-05, 
       3.4393570465377052e-05, 
       4.2294675673379252e-05, 
       3.5770372791129209e-05, 
       3.5948235345206717e-05, 
       3.6286428679262367e-05, 
       3.3814985871288216e-05, 
       4.7521252368772711e-05, 
       3.5882091139243545e-05, 
       4.1607851222958348e-05, 
       3.8009054455950987e-05, 
       3.6413843303132666e-05, 
       3.9125496684288854e-05, 
       3.9127168544911873e-05, 
       4.0505927545139951e-05, 
       3.9398768231552840e-05, 
       4.7621772757621480e-05, 
       4.2959130747837117e-05, 
       4.3320627284348344e-05, 
       3.4464527597372001e-05, 
       4.7582001065388140e-05, 
       3.3786464518053388e-05, 
       4.0701688482625825e-05, 
       3.5900547284314782e-05, 
       3.8553029520085075e-05, 
       4.1924666731585510e-05, 
       3.5394604767313878e-05, 
       4.2239530017303450e-05, 
       4.7337776421580997e-05, 
       3.3659420155075219e-05, 
       3.4287600037315041e-05, 
       3.5814826611991624e-05, 
       3.8185957442163977e-05, 
       3.3538421441383937e-05, 
       4.7004368864065993e-05, 
       3.5231692289262343e-05, 
       4.1076415345679411e-05, 
       3.8929462739837082e-05, 
       4.6862722004600786e-05, 
       3.4708757394615761e-05, 
       4.5491008197321513e-05, 
       3.3915429669994730e-05, 
       4.1088511913046882e-05, 
       3.5935351549874749e-05, 
       3.4006614022889939e-05, 
       4.6845926751187133e-05, 
       4.0373014922872028e-05, 
       4.7107097092065055e-05, 
       4.0402449368802867e-05, 
       4.7122458628594792e-05, 
       3.3325520663685238e-05, 
       3.9067872514407116e-05, 
       3.5674633458111762e-05, 
       3.5674506601213085e-05, 
       4.0894415854947020e-05, 
       3.3296919416264139e-05, 
       3.3271932497910629e-05, 
       3.8961969120147022e-05, 
       3.8027666632312266e-05, 
       3.8027666632312266e-05, 
       3.4682735970563644e-05, 
       4.6883791369232384e-05, 
       3.8428290786578761e-05, 
       3.4561980525756756e-05, 
       3.6921376344854777e-05, 
       3.4526906225512987e-05, 
       4.4797950417819652e-05, 
       3.4480464500246164e-05, 
       4.5407418911527978e-05, 
       3.9009178455334323e-05, 
       4.2575556082544586e-05, 
       4.4594004074830920e-05, 
       4.4594004074830920e-05, 
       3.6364591491455336e-05, 
       4.4385878132989232e-05, 
       4.2513035633243153e-05, 
       4.6537570189777452e-05, 
       3.5788247046931940e-05, 
       4.6596777773986708e-05, 
       3.3068164718251933e-05, 
       3.5026932969098637e-05, 
       4.5546739590187569e-05, 
       3.6952804491064363e-05, 
       3.4618017509774466e-05, 
       4.5075548402572612e-05, 
       4.0447710321298316e-05, 
       3.4923386735517238e-05, 
       4.0419775110989452e-05, 
       4.6293746012459068e-05, 
       3.2745041720002113e-05, 
       4.4591998575776437e-05, 
       4.6252413834934152e-05, 
       4.6206101594182737e-05, 
       3.8460097446942457e-05, 
       3.8817991984067315e-05, 
       3.3832317569061025e-05, 
       4.5035227505016242e-05, 
       4.6142816491464215e-05, 
       3.4537755841240126e-05, 
       4.0521043525931923e-05, 
       4.1448946781680447e-05, 
       4.6126033669354521e-05, 
       3.3260537641285438e-05, 
       4.5313186282086244e-05, 
       4.5301371894513224e-05, 
       3.2571481755159606e-05, 
       3.9063553684363544e-05, 
       3.2885145049368764e-05, 
       3.8119031368579614e-05, 
       3.8269338524865958e-05, 
       3.8396549443803920e-05, 
       4.5857080253412536e-05, 
       4.1977324710849492e-05, 
       3.8179638836043623e-05, 
       4.0081425918717041e-05, 
       3.9946127923399494e-05, 
       4.5751216887632890e-05, 
       3.2634167652354340e-05, 
       3.8046536131002415e-05, 
       3.7827084449215953e-05, 
       3.4296112163756613e-05, 
       3.4174248376746770e-05, 
       3.4174248376746770e-05, 
       3.5965142450094721e-05, 
       3.3741121062653508e-05, 
       3.9236828710639907e-05, 
       3.9608938417377338e-05, 
       3.7764623957631554e-05, 
       3.3320197340346388e-05, 
       3.7338700379328136e-05, 
       4.3669425886363513e-05, 
       3.4271137196442654e-05, 
       3.5433376771082274e-05, 
       3.3486118516582762e-05, 
       4.5250261952513739e-05, 
       3.3913462748601325e-05, 
       3.4730916239774143e-05, 
       3.3471588561025422e-05, 
       3.9267183192155905e-05, 
       3.9267183192155905e-05, 
       4.5371186199634334e-05, 
       3.2125024205432697e-05, 
       4.4069876909639000e-05, 
       3.4196786711891340e-05, 
       3.9977846844898209e-05, 
       3.6710572203415786e-05, 
       3.4010369103921501e-05, 
       3.3724409266210451e-05, 
       3.2760481035611278e-05, 
       3.2904537761383779e-05, 
       4.4568479276266993e-05, 
       4.5019472868030665e-05, 
       3.9961036222125687e-05, 
       4.3870483207295735e-05, 
       3.3903109737938297e-05, 
       4.2523477283951611e-05, 
       3.3223967255526518e-05, 
       4.5150316826228718e-05, 
       3.2625516598689567e-05, 
       3.2033188099407540e-05, 
       3.6277123400178068e-05, 
       3.8338621015750025e-05, 
       4.1178170825202329e-05, 
       4.5040085633317287e-05, 
       4.4728082521064298e-05, 
       3.6757929909937804e-05, 
       3.2091222618036264e-05, 
       3.3538116947814391e-05, 
       4.2309881828846819e-05, 
       3.3046106934695401e-05, 
       3.7370279376754190e-05, 
       3.2621174685303466e-05, 
       3.3062661983855973e-05, 
       4.2257570242165295e-05, 
       4.4836123008111322e-05, 
       3.2630549073688078e-05, 
       4.4800177385296906e-05, 
       3.2300772030072913e-05, 
       4.1095680902665975e-05, 
       3.6019523482944345e-05, 
       3.7223478283263789e-05, 
       3.2920345203836887e-05, 
       3.5561345757898780e-05, 
       3.9941119147191532e-05, 
       3.4857476393429068e-05, 
       3.4857476393429068e-05, 
       4.0778564768519577e-05, 
       3.2630383931740110e-05, 
       3.1730450843158971e-05, 
       3.6002109092395767e-05, 
       3.2224597398680615e-05, 
       3.4760711536439007e-05, 
       4.0859185193395717e-05, 
       3.7665952232727266e-05, 
       3.2987602803948624e-05, 
       3.1631742267254777e-05, 
       3.3887177837585555e-05, 
       4.0040122562525175e-05, 
       3.7083791842945269e-05, 
       3.4243169648869223e-05, 
       3.6102549527256003e-05, 
       4.2664384334338440e-05, 
       4.4272147651364855e-05, 
       4.3419086008538208e-05, 
       3.1665743515908261e-05, 
       4.4251575407203080e-05, 
       3.7111207883091069e-05, 
       3.8243511389102586e-05, 
       4.0370481932988642e-05, 
       3.9660732848977612e-05, 
       3.1713718158598758e-05, 
       3.2438012017225746e-05, 
       3.7409172994194450e-05, 
       3.5951174141606017e-05, 
       3.5592287270683499e-05, 
       3.3502927751480388e-05, 
       3.1464921463741575e-05, 
       3.2109239776612635e-05, 
       3.7317946598489292e-05, 
       4.0843149007119850e-05, 
       3.2753763807951162e-05, 
       4.1860624426415585e-05, 
       3.2970674790670760e-05, 
       3.3782491171559164e-05, 
       4.3821868344843722e-05, 
       3.1340627089470671e-05, 
       3.2735446925465831e-05, 
       3.6556506152729484e-05, 
       3.6687479054251329e-05, 
       3.1204131079621381e-05, 
       3.6441274476749582e-05, 
       4.1004039750008077e-05, 
       4.2648071026097820e-05, 
       3.7759303884191027e-05, 
       3.3273649181840837e-05, 
       3.5005580082325342e-05, 
       3.6450346560257059e-05, 
       4.1125712757855676e-05, 
       3.4950674393153032e-05, 
       4.2405073701867510e-05, 
       3.4339471043475212e-05, 
       3.6434036345276246e-05, 
       3.0990544093728622e-05, 
       3.6365064433554915e-05, 
       4.2930508638612651e-05, 
       4.2080265251830981e-05, 
       3.7130266685097364e-05, 
       3.8232203652429172e-05, 
       3.6359961900597668e-05, 
       3.2669192246649469e-05, 
       3.6405797690291121e-05, 
       3.3636907942984401e-05, 
       3.5547161978024754e-05, 
       3.0893294137756412e-05, 
       3.1972434799545717e-05, 
       3.5430134173731002e-05, 
       3.5429698783730162e-05, 
       3.3446935639601215e-05, 
       3.1808951487061366e-05, 
       3.2587868982492576e-05, 
       4.1127232708330871e-05, 
       3.9765272436467066e-05, 
       3.4721808162169342e-05, 
       3.0858957211447490e-05, 
       3.3021418654063261e-05, 
       3.2483493963702462e-05, 
       3.5544181339750911e-05, 
       3.5333932672663143e-05, 
       4.3267520458496746e-05, 
       3.5329228201965449e-05, 
       3.5810996273227576e-05, 
       3.6234461652868764e-05, 
       3.4966700344344709e-05, 
       4.0000000000003601e-05, 
       3.9999999999996425e-05, 
       4.2642231394564299e-05, 
       4.0000000000000525e-05, 
       3.9999999999999996e-05, 
       4.0000000000000850e-05, 
       4.0000000000000850e-05, 
       3.0687080051642160e-05, 
       4.0175820489051220e-05, 
       4.2967949281064946e-05, 
       3.5791639705738227e-05, 
       3.2695046713603666e-05, 
       4.1284915194251193e-05, 
       4.3227325676961756e-05, 
       3.6351134044658116e-05, 
       3.6351134044658116e-05, 
       4.3049980272711664e-05, 
       3.8412507270652266e-05, 
       3.2478494203517075e-05, 
       3.3360246438204735e-05, 
       3.6606860578581990e-05, 
       3.1344493459954840e-05, 
       3.6478814396940000e-05, 
       3.9976617987118969e-05, 
       3.5228487213962482e-05, 
       3.0676753497353083e-05, 
       3.3844862148111803e-05, 
       3.1918693308232495e-05, 
       3.2108403942735955e-05, 
       3.2108147324704287e-05, 
       3.5718903546128392e-05, 
       3.0567255613096953e-05, 
       3.8787873519139278e-05, 
       3.2861028505453873e-05, 
       4.2864904444903718e-05, 
       3.0865703495637819e-05, 
       3.4524179741337411e-05, 
       3.7216515401013755e-05, 
       3.1734552038443310e-05, 
       4.1102116342529652e-05, 
       3.5460940924610744e-05, 
       3.1183862419299843e-05, 
       3.1924401516496573e-05, 
       3.5392930422675153e-05, 
       3.2228120626777810e-05, 
       3.9146063286264332e-05, 
       3.5071184487368467e-05, 
       3.1842807324020201e-05, 
       3.1970168139514024e-05, 
       3.7937099126946580e-05, 
       3.0915129868124844e-05, 
       3.0508548935440445e-05, 
       3.0622265053961770e-05, 
       3.0648898063168937e-05, 
       3.9380484993190091e-05, 
       3.1826736983741053e-05, 
       4.0556897947875557e-05, 
       3.6870969331310807e-05, 
       3.4986726002250227e-05, 
       3.2769072482660232e-05, 
       3.1440201620966136e-05, 
       3.8907903181064482e-05, 
       4.2438179901222286e-05, 
       4.2023661248174083e-05, 
       3.0312423698696510e-05, 
       4.0514857214319071e-05, 
       3.7269076161601675e-05, 
       3.7269076161601675e-05, 
       3.5191790287807582e-05, 
       3.0048526852174058e-05, 
       4.2410583326262858e-05, 
       3.8847237360009878e-05, 
       4.2389194621609895e-05, 
       4.1212515784459371e-05, 
       3.1579744445800019e-05, 
       4.0307395661197584e-05, 
       3.3133149361985029e-05, 
       3.2047974973105546e-05, 
       3.4599182453513417e-05, 
       3.0335200404371127e-05, 
       3.5816400473673277e-05, 
       3.8018900764741985e-05, 
       3.6255635280558503e-05, 
       3.6517050232858369e-05, 
       2.9898232491456338e-05, 
       2.9999999999997544e-05, 
       2.9865522238895069e-05, 
       3.0386678297487825e-05, 
       3.0000000000001115e-05, 
       2.9930018971477712e-05, 
       3.5179186529604157e-05, 
       3.1013397047679805e-05, 
       4.2089693138072779e-05, 
       4.1579684287786180e-05, 
       3.1777282309974928e-05, 
       3.9824470495324094e-05, 
       3.1047783850745064e-05, 
       3.0304760938680064e-05, 
       3.9674493234127261e-05, 
       3.3497123017394477e-05, 
       3.5918861116744748e-05, 
       3.2156097743371124e-05, 
       3.5134289476827633e-05, 
       2.9999999999998537e-05, 
       4.1930957968432621e-05, 
       3.9653004118706503e-05, 
       3.0000000000000729e-05, 
       3.3695538621211971e-05, 
       2.9640018998138414e-05, 
       3.9127744124431358e-05, 
       3.6731674903732453e-05, 
       4.1569901886460826e-05, 
       3.4414606058479796e-05, 
       2.9612270451253607e-05, 
       2.9999999999996538e-05, 
       3.0344684133494233e-05, 
       3.1832123733942461e-05, 
       3.5627866755230715e-05, 
       3.2673960057977809e-05, 
       3.0645824701591572e-05, 
       3.4894116176437179e-05, 
       2.9634844391844729e-05, 
       3.1200166430293327e-05, 
       2.9673516774456471e-05, 
       4.1607779347426780e-05, 
       3.1819138156569431e-05, 
       3.0055082223753782e-05, 
       4.1660744075167262e-05, 
       4.1466580221324531e-05, 
       3.0872755395731534e-05, 
       2.9999999999998662e-05, 
       3.0393275905751711e-05, 
       3.4624016429586776e-05, 
       2.9485903121034184e-05, 
       4.0964802946960687e-05, 
       3.5708171496109527e-05, 
       2.9865832181733699e-05, 
       3.4321890721831801e-05, 
       4.1174396362775808e-05, 
       4.1239757453149332e-05, 
       3.6298303961602183e-05, 
       3.4134436748018775e-05, 
       3.2974361243417835e-05, 
       3.2360342214527998e-05, 
       3.5764132571464947e-05, 
       3.0065796860569196e-05, 
       2.9877525228162061e-05, 
       3.9176939969549332e-05, 
       3.4249939884026670e-05, 
       3.5176953962531646e-05, 
       3.0049063731339457e-05, 
       4.0594427008230429e-05, 
       3.5368975007625415e-05, 
       3.9793598342921186e-05, 
       3.9477412791113474e-05, 
       3.1864739928386620e-05, 
       3.0478014376098028e-05, 
       3.8002640290901333e-05, 
       3.0423376473216073e-05, 
       2.9999999999995217e-05, 
       2.9134911748206467e-05, 
       3.9948413522440165e-05, 
       3.0000000000002071e-05, 
       3.1343479359611563e-05, 
       3.2754659955283474e-05, 
       3.8877251773229903e-05, 
       3.4132464045913957e-05, 
       4.0736144732704707e-05, 
       3.4958329812063158e-05, 
       3.8727742895267361e-05, 
       3.4891950350432412e-05, 
       4.0339802010774952e-05, 
       3.3220558770762330e-05, 
       4.0988250698645330e-05, 
       3.7215634748122155e-05, 
       3.4834560375819646e-05, 
       3.2981440111114627e-05, 
       3.3764422593409163e-05, 
       3.4753764118036549e-05, 
       2.9245315115871251e-05, 
       3.5607991088550644e-05, 
       3.0551320352083212e-05, 
       3.1261272408491547e-05, 
       3.1261272408491547e-05, 
       3.4347123758541313e-05, 
       3.7017335901600962e-05, 
       3.6874969061329085e-05, 
       3.4631403876541824e-05, 
       2.9999999999999438e-05, 
       2.9427458040281012e-05, 
       3.1811184794633091e-05, 
       3.4991318932723151e-05, 
       3.4083085694072307e-05, 
       2.8900056859666726e-05, 
       3.3917562241378232e-05, 
       2.9805675789377176e-05, 
       3.4543374114494115e-05, 
       3.2027650421014231e-05, 
       3.1959111208989747e-05, 
       3.0058654954851915e-05, 
       3.3681230551520629e-05, 
       2.9710860854944819e-05, 
       3.1951380143894823e-05, 
       3.0404849856203801e-05, 
       3.1810645167395909e-05, 
       2.9672724338572232e-05, 
       3.5110268221259054e-05, 
       4.0717993876324803e-05, 
       4.0286150479599335e-05, 
       4.0305514578017195e-05, 
       3.3339429561963698e-05, 
       3.9143019557160810e-05, 
       3.4486627900978304e-05, 
       3.9254735696295185e-05, 
       3.1399255449256761e-05, 
       4.0663280769063733e-05, 
       3.4855922592502555e-05, 
       3.9596039957574974e-05, 
       3.1754534929542795e-05, 
       3.0744456739125412e-05, 
       3.3729669735472886e-05, 
       3.3913347332532783e-05, 
       2.8901607985854970e-05, 
       2.9017705558142954e-05, 
       3.1432518819307625e-05, 
       3.0200779238637072e-05, 
       2.8679823958508767e-05, 
       3.7089513371622241e-05, 
       2.9491248245211476e-05, 
       3.3283956616272910e-05, 
       3.1536567005246314e-05, 
       3.8619172992455671e-05, 
       3.8303105744797076e-05, 
       3.3043207623952497e-05, 
       3.1484633099524213e-05, 
       3.2506231446373348e-05, 
       3.0453721823432587e-05, 
       3.3461008278413813e-05, 
       3.3755057643275865e-05, 
       3.8625125148200007e-05, 
       3.2232221050196497e-05, 
       2.8983048499829336e-05, 
       3.9999999999995653e-05, 
       3.1213214158177114e-05, 
       3.0560856076039788e-05, 
       2.9992363883985074e-05, 
       3.5282293979060346e-05, 
       3.9871326315803349e-05, 
       3.3415178693867250e-05, 
       3.3540650107118455e-05, 
       2.9914262856188159e-05, 
       3.2482885329664131e-05, 
       3.2892428695817698e-05, 
       2.8358925785976223e-05, 
       2.8752095878955557e-05, 
       3.8997869515035493e-05, 
       2.9203676273401392e-05, 
       2.8441581847269676e-05, 
       3.8910616054939123e-05, 
       3.7629191124480454e-05, 
       3.9867401013212453e-05, 
       3.9999999369874880e-05, 
       3.9999999999997787e-05, 
       4.0000000000004726e-05, 
       3.9999999999995307e-05, 
       3.9999999999997855e-05, 
       3.9999999999999807e-05, 
       3.9999999999996385e-05, 
       3.9999999999996385e-05, 
       3.9999999999998316e-05, 
       3.9999999999998763e-05, 
       3.9999999999997022e-05, 
       4.0000000000002158e-05, 
       4.0000000000001148e-05, 
       3.9999999999995246e-05, 
       4.0000000000000566e-05, 
       4.0000000000004855e-05, 
       3.9999999999999407e-05, 
       3.9999999999997367e-05, 
       3.9999999999997367e-05, 
       3.9999999999999346e-05, 
       4.0000000000001935e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.9998072730920922e-05, 
       3.4101966653091486e-05, 
       3.0371495990061642e-05, 
       3.3691510332241079e-05, 
       3.3146176564241083e-05, 
       2.8352624797045927e-05, 
       3.9370097906425992e-05, 
       3.0914698791476434e-05, 
       2.9281548988547377e-05, 
       2.8383964019516530e-05, 
       3.1829118794917943e-05, 
       2.9056778368371660e-05, 
       2.8575019696887445e-05, 
       3.9707425423179707e-05, 
       3.3314260650146486e-05, 
       3.3245589839847759e-05, 
       2.8856044178950096e-05, 
       2.9999999999999659e-05, 
       2.9673660542242409e-05, 
       3.0852338257734745e-05, 
       3.2821110138972828e-05, 
       3.9063030334859538e-05, 
       2.9921555345213084e-05, 
       3.6980425701451305e-05, 
       2.8913479216370070e-05, 
       3.9521574689845276e-05, 
       3.0794876476941755e-05, 
       3.0367424944129759e-05, 
       3.7191960864011787e-05, 
       3.5382993151209656e-05, 
       3.0133944190861544e-05, 
       3.3862217646562098e-05, 
       2.9999999999994048e-05, 
       2.8111432292700875e-05, 
       3.5546811532154019e-05, 
       3.1762476135288943e-05, 
       3.9103375540051138e-05, 
       3.4049201938889247e-05, 
       3.6323099597367277e-05, 
       3.3820477143005490e-05, 
       2.9473525216751520e-05, 
       2.8077341188740352e-05, 
       2.8641597318945315e-05, 
       3.3052167482487881e-05, 
       3.0006977809097550e-05, 
       2.8734055098689035e-05, 
       3.2121965885823627e-05, 
       3.2121965885823627e-05, 
       3.8351822669135409e-05, 
       3.6664552622955772e-05, 
       2.9488722842464473e-05, 
       3.7617913802574932e-05, 
       3.1158144985993347e-05, 
       3.2500998209758420e-05, 
       3.9072027489543663e-05, 
       3.9072027489543663e-05, 
       2.7735598066203194e-05, 
       2.7764695223452325e-05, 
       3.9097892441677912e-05, 
       3.2976553477897266e-05, 
       3.1512281040062003e-05, 
       3.2509069531462060e-05, 
       3.2746205464973778e-05, 
       2.9680476630247505e-05, 
       3.0848085455829987e-05, 
       3.2583812867033344e-05, 
       2.8437285254930122e-05, 
       3.0008925984002347e-05, 
       3.8042884052841897e-05, 
       3.8379089311409152e-05, 
       3.3558481689605506e-05, 
       2.8418965718931595e-05, 
       2.9494167602064154e-05, 
       2.7637433230441698e-05, 
       2.8307255428469105e-05, 
       3.6623500956541377e-05, 
       3.8723976764437028e-05, 
       3.6652409712305939e-05, 
       2.8337045843303976e-05, 
       3.0927080102684384e-05, 
       3.7670865208058935e-05, 
       3.0152878696192717e-05, 
       2.7524714884471664e-05, 
       3.3688080934318146e-05, 
       3.8814990204682862e-05, 
       3.8835684022816053e-05, 
       2.8261746632243478e-05, 
       3.5797980262834686e-05, 
       3.1178034009362328e-05, 
       3.7906882212586071e-05, 
       2.8153042407781401e-05, 
       2.7828242059350935e-05, 
       3.7079373000424196e-05, 
       3.0000000000004930e-05, 
       3.0263305462685590e-05, 
       2.7427051258041797e-05, 
       3.0000000000005052e-05, 
       3.2376122010147133e-05, 
       3.2118621750710555e-05, 
       3.4552560355167389e-05, 
       3.6393073870424638e-05, 
       3.4255623018692846e-05, 
       2.8779571151293554e-05, 
       3.6980006812946069e-05, 
       2.8084763012237611e-05, 
       2.7273015773530585e-05, 
       2.7273015773530585e-05, 
       2.7593266588727539e-05, 
       3.2750681140646947e-05, 
       3.4800784269216361e-05, 
       2.7197247536869282e-05, 
       3.4760654429085798e-05, 
       3.2985405708083391e-05, 
       3.4115498745264903e-05, 
       2.8710575911387293e-05, 
       3.1717718045268014e-05, 
       2.7286759674813567e-05, 
       3.7812380667003381e-05, 
       3.7812380667003381e-05, 
       2.8592893001158859e-05, 
       3.0323834104484881e-05, 
       2.8080863309496687e-05, 
       2.7944511872129473e-05, 
       3.7583345916136524e-05, 
       2.7158777851167449e-05, 
       3.1839687959205633e-05, 
       2.7094021974703431e-05, 
       2.7171020118079076e-05, 
       3.5664461173770676e-05, 
       3.2722960716693627e-05, 
       3.5309694226877276e-05, 
       3.6967609784195869e-05, 
       2.9623026139865901e-05, 
       3.7877722468814330e-05, 
       3.1440415079202689e-05, 
       3.7883896786987826e-05, 
       3.8157882535370888e-05, 
       2.8056278765496161e-05, 
       3.3986073179846497e-05, 
       2.9907224511221717e-05, 
       2.9166341473750211e-05, 
       2.8065591424862517e-05, 
       3.1818964029316071e-05, 
       3.7765770455637448e-05, 
       2.8484579934894903e-05, 
       3.0777297346699949e-05, 
       2.9248152045879696e-05, 
       3.6752540599563022e-05, 
       3.1444359295932022e-05, 
       2.7487311212258861e-05, 
       3.2158066034617186e-05, 
       2.8006832604812145e-05, 
       3.0869558468247232e-05, 
       3.6376927317581486e-05, 
       2.8515190475710322e-05, 
       3.1012280610962080e-05, 
       2.8186097908298101e-05, 
       3.1373067469956108e-05, 
       3.7620970727936492e-05, 
       2.9664505959975816e-05, 
       2.7895681110320562e-05, 
       2.9459653284077161e-05, 
       3.1804693620182743e-05, 
       2.8131921598845085e-05, 
       3.3415185001258451e-05, 
       2.8354502709659490e-05, 
       3.7431684776689476e-05, 
       3.0000000000002816e-05, 
       3.5569001099781376e-05, 
       2.8205497492748706e-05, 
       2.8607705000421176e-05, 
       2.7443968825585090e-05, 
       3.2427585495882185e-05, 
       2.7719805887245327e-05, 
       3.1234448541447378e-05, 
       3.0761616833129976e-05, 
       3.2715242069734450e-05, 
       3.2709670811519337e-05, 
       2.7136451288924564e-05, 
       2.7805304218350495e-05, 
       3.0594252101971418e-05, 
       3.5447034326557840e-05, 
       3.7601526470764493e-05, 
       3.5295341386316663e-05, 
       3.1806493251033354e-05, 
       3.6135582599628770e-05, 
       3.6086893871662933e-05, 
       2.7376114968961502e-05, 
       3.0009824206239838e-05, 
       3.7496766586534236e-05, 
       2.7355518903843049e-05, 
       2.7243121963924633e-05, 
       3.2256475016679078e-05, 
       2.6840541795447096e-05, 
       2.6579114334319194e-05, 
       3.7402568608890108e-05, 
       3.1135519868122257e-05, 
       3.7346398928643819e-05, 
       2.8521904190905685e-05, 
       3.2261295604610622e-05, 
       2.6478938327836762e-05, 
       3.2220820255549972e-05, 
       3.7399259503341476e-05, 
       3.7361263746379754e-05, 
       3.6127348295208255e-05, 
       3.2217602660011996e-05, 
       2.7620070347298465e-05, 
       2.8782051543496251e-05, 
       2.7215867328130824e-05, 
       2.7223585310788654e-05, 
       3.7277285874189767e-05, 
       3.4672406364845564e-05, 
       2.8907478343366754e-05, 
       3.7210134825299807e-05, 
       3.1636014922887694e-05, 
       2.8291149916605230e-05, 
       3.6973137758065399e-05, 
       2.7735177636889611e-05, 
       3.0970971843819026e-05, 
       2.9486243616418117e-05, 
       3.6719146288476597e-05, 
       2.7847546085857943e-05, 
       3.0181985060019578e-05, 
       2.6482193611925169e-05, 
       2.8815807534761744e-05, 
       3.3807949936129490e-05, 
       2.7375490978217892e-05, 
       3.5243441453665731e-05, 
       3.7073338059968103e-05, 
       3.0900032098967309e-05, 
       3.0345729496005474e-05, 
       3.1748258798729004e-05, 
       3.1311727103519367e-05, 
       3.5602270007051821e-05, 
       3.2016490711050935e-05, 
       2.9217232164379110e-05, 
       3.1929020109247796e-05, 
       2.6487994875777422e-05, 
       2.6191952943439877e-05, 
       2.6880665873802498e-05, 
       2.6508511276386538e-05, 
       2.9394690649248106e-05, 
       3.5153258633961551e-05, 
       3.1235147581718718e-05, 
       3.0912636153448121e-05, 
       3.4058065997406065e-05, 
       3.5029905548271276e-05, 
       3.2568045374985023e-05, 
       2.9385263267032378e-05, 
       3.1577812600937473e-05, 
       2.7859594944653383e-05, 
       2.6529123979190486e-05, 
       3.4027755742130251e-05, 
       3.5588880550044506e-05, 
       2.6224454407935215e-05, 
       2.7722099988389238e-05, 
       2.8636722394636658e-05, 
       3.0670721746372807e-05, 
       3.6821497979660456e-05, 
       3.2544191150131346e-05, 
       3.0680391841310685e-05, 
       2.6178223135168246e-05, 
       3.6751161736298665e-05, 
       2.6400459622820941e-05, 
       3.0961856006158206e-05, 
       2.6764834198560157e-05, 
       2.6459860954323407e-05, 
       2.9212574946141627e-05, 
       2.7423522087061264e-05, 
       3.2440399380419423e-05, 
       3.0468742644225386e-05, 
       3.4797020202410598e-05, 
       3.6544531947032417e-05, 
       2.8368146842082890e-05, 
       2.8239993402518091e-05, 
       3.5498194550767828e-05, 
       3.6476709581843703e-05, 
       2.6562021739210490e-05, 
       2.5949092582143126e-05, 
       3.2974859278554567e-05, 
       3.2830037064574109e-05, 
       2.7789722747859991e-05, 
       3.2889207538864745e-05, 
       2.9456942423899567e-05, 
       2.6926973764862875e-05, 
       3.0429893573776079e-05, 
       3.0340338333332492e-05, 
       2.7247080058153878e-05, 
       3.4612610330185798e-05, 
       2.6770615985280530e-05, 
       2.7088599783173430e-05, 
       2.6374543704044544e-05, 
       2.6112247893333990e-05, 
       3.6287859028296548e-05, 
       2.6213204618459015e-05, 
       3.3445339397098061e-05, 
       2.6087882046491557e-05, 
       3.4822849282260654e-05, 
       3.4905847569910088e-05, 
       3.3801178583429075e-05, 
       3.5639546744166639e-05, 
       2.6765807884927219e-05, 
       2.6021684201039694e-05, 
       3.1587062398479871e-05, 
       2.6786650153324187e-05, 
       3.5057036568917409e-05, 
       2.8441873606354640e-05, 
       3.0668387309916572e-05, 
       2.8330356668001596e-05, 
       2.7607652335100470e-05, 
       2.9503795557109351e-05, 
       2.5518499255795921e-05, 
       2.5619292432600003e-05, 
       3.1068335511633065e-05, 
       3.1068335511633065e-05, 
       2.6958705850508612e-05, 
       2.7263349299276504e-05, 
       2.6547596492032995e-05, 
       2.9450887070232957e-05, 
       2.5544643062216664e-05, 
       3.0118263192990052e-05, 
       2.6138459220834942e-05, 
       2.5696698120434355e-05, 
       2.5971268422425596e-05, 
       2.6540868527769923e-05, 
       3.5192854407768160e-05, 
       2.9603739126994531e-05, 
       2.9810256204293699e-05, 
       2.5942780705216061e-05, 
       2.5622626663456258e-05, 
       2.6710917055127132e-05, 
       2.9967900084011413e-05, 
       2.8385156287673372e-05, 
       3.5805144182451467e-05, 
       3.4799260043352211e-05, 
       3.1509351894112576e-05, 
       2.9803614824199960e-05, 
       3.5541725360757025e-05, 
       3.3691461277249626e-05, 
       3.3691461277249626e-05, 
       2.7598855946451330e-05, 
       3.5605025050374659e-05, 
       2.8475596494380867e-05, 
       2.9912924062507994e-05, 
       2.5293439665451053e-05, 
       3.3961856653239450e-05, 
       3.5616392303967299e-05, 
       2.7001811912287017e-05, 
       2.9601128816841725e-05, 
       2.5984147079570722e-05, 
       2.9632426328468161e-05, 
       3.0236654270346675e-05, 
       3.4572432642505528e-05, 
       2.5266027091167520e-05, 
       3.0951660963260621e-05, 
       3.4373632482355902e-05, 
       2.9999999999995661e-05, 
       2.5369526271716473e-05, 
       3.4839110124151276e-05, 
       2.6132824382948814e-05, 
       2.9063327995832415e-05, 
       2.6472615655785992e-05, 
       2.7764810178307452e-05, 
       2.6144789724538160e-05, 
       3.0890981756160902e-05, 
       2.6110355117407854e-05, 
       2.9885242587375260e-05, 
       2.7679857230577843e-05, 
       2.9425082044892741e-05, 
       3.0214741684833754e-05, 
       2.8679823958497325e-05, 
       2.8962282301601835e-05, 
       2.6225859260341377e-05, 
       2.7435545015336062e-05, 
       2.7435545015336062e-05, 
       3.3082135259121834e-05, 
       3.3491180389950727e-05, 
       2.6663753906808478e-05, 
       3.4765545177206964e-05, 
       2.5534034958505820e-05, 
       3.1424812695407142e-05, 
       2.5612389910391011e-05, 
       3.5275377998565117e-05, 
       3.3837363414203747e-05, 
       2.9309628794658645e-05, 
       2.6821631738102243e-05, 
       3.4582175764566227e-05, 
       2.6261166396776510e-05, 
       2.6261166396776510e-05, 
       2.8914715281012325e-05, 
       3.0414269646515200e-05, 
       3.3471699511818368e-05, 
       2.9230822657698585e-05, 
       2.9218638649073042e-05, 
       3.0647180318210917e-05, 
       2.7008426397817923e-05, 
       3.1160267185058773e-05, 
       2.9869749083058493e-05, 
       2.9219048505920373e-05, 
       2.4849600073339911e-05, 
       2.6037799510803392e-05, 
       2.8439865618647064e-05, 
       3.4555822189791866e-05, 
       2.6483743963575529e-05, 
       3.1302480593303306e-05, 
       2.8329082517580672e-05, 
       2.5064519024489073e-05, 
       3.4998459810803798e-05, 
       2.9452237135893890e-05, 
       2.8503272400643855e-05, 
       3.4732361333598925e-05, 
       2.6048038337545929e-05, 
       2.6020340011205926e-05, 
       2.5532442911192760e-05, 
       2.8467085593134533e-05, 
       2.7844633179166775e-05, 
       2.8916218067269055e-05, 
       2.7525261530568518e-05, 
       3.3500820873700212e-05, 
       2.5236795057485878e-05, 
       2.9712944822632725e-05, 
       3.4795297202249628e-05, 
       2.5506949516865321e-05, 
       2.9271572485443801e-05, 
       2.7711138370157813e-05, 
       3.4186103796714707e-05, 
       2.9070509852859199e-05, 
       2.5674419988596945e-05, 
       3.2652772143246170e-05, 
       3.4493917848018113e-05, 
       2.9057863155478573e-05, 
       2.8025355394401530e-05, 
       2.8800441688691351e-05, 
       2.6506855316750697e-05, 
       2.5796455768486728e-05, 
       2.4585885507754087e-05, 
       3.4565565458318385e-05, 
       3.4567075152386372e-05, 
       2.9370462255910539e-05, 
       2.8910944513129732e-05, 
       3.2009112198960969e-05, 
       3.3131324568675408e-05, 
       2.9779375113424555e-05, 
       2.4478639931581370e-05, 
       3.4489899448492962e-05, 
       3.0324988760153031e-05, 
       3.1903309572245323e-05, 
       3.1903309572245323e-05, 
       2.8477087782275310e-05, 
       2.5754293512195188e-05, 
       3.2566852533668789e-05, 
       3.4340648036014342e-05, 
       2.7749558171350663e-05, 
       3.1924019355658394e-05, 
       2.4936069811422376e-05, 
       3.0476923242705787e-05, 
       2.4686938562324296e-05, 
       2.4342071007000229e-05, 
       2.4411420511066239e-05, 
       2.4405290505860593e-05, 
       3.4217763687172977e-05, 
       2.8553692616667581e-05, 
       2.4585433594285740e-05, 
       2.9343236890926187e-05, 
       3.2887355848329521e-05, 
       2.8507144590401653e-05, 
       2.8045852090980314e-05, 
       3.0286917674087125e-05, 
       2.6246399195426636e-05, 
       2.7281415298846323e-05, 
       2.5500732971808655e-05, 
       2.8352624797046195e-05, 
       2.9873918513434426e-05, 
       2.9730170434726155e-05, 
       2.9237275841035036e-05, 
       3.4188621053293948e-05, 
       2.4188682359499585e-05, 
       2.5209713601922852e-05, 
       2.8331604465864143e-05, 
       2.4707486366339261e-05, 
       2.4355719506852440e-05, 
       2.4749902788761841e-05, 
       2.5333180611686048e-05, 
       2.4187335403740857e-05, 
       3.2258817870144468e-05, 
       2.4438787635556412e-05, 
       2.6140710148556463e-05, 
       2.4291482734505766e-05, 
       3.3879750428102944e-05, 
       3.4018595780887331e-05, 
       3.3647339395032489e-05, 
       2.9775088780512990e-05, 
       2.4829213008134859e-05, 
       2.4045298416024582e-05, 
       2.9185210302983760e-05, 
       3.3949404460007427e-05, 
       3.1509096619240060e-05, 
       3.3955540639339127e-05, 
       2.7901839643375575e-05, 
       2.7344674044050312e-05, 
       2.9684586839422806e-05, 
       2.5870741827333551e-05, 
       3.2466724563955442e-05, 
       2.7871862319775655e-05, 
       2.5342692229464575e-05, 
       3.0512170281136161e-05, 
       3.3818535922091312e-05, 
       2.7827499786511347e-05, 
       2.3961241946183188e-05, 
       3.2303034902006041e-05, 
       2.4849758004142231e-05, 
       3.3449827633712639e-05, 
       3.3808023390332035e-05, 
       2.7165717949085221e-05, 
       3.0923005167221850e-05, 
       3.3638027051661989e-05, 
       3.0739282912032341e-05, 
       3.3592194079797473e-05, 
       2.7191856632721637e-05, 
       2.9226713044868983e-05, 
       2.9226713044868983e-05, 
       2.9157068993403251e-05, 
       2.5168509482996461e-05, 
       2.9181564139240500e-05, 
       3.1630333474835738e-05, 
       2.3862864384340234e-05, 
       2.4647600897910629e-05, 
       2.8026398400045466e-05, 
       2.3833921696396811e-05, 
       2.5359355698710509e-05, 
       2.5770133112329606e-05, 
       3.0585895483759674e-05, 
       2.8964505735649643e-05, 
       2.4927464175877677e-05, 
       2.5177754065335333e-05, 
       3.1894639176358783e-05, 
       2.6804373046434031e-05, 
       2.7209816442318557e-05, 
       3.3121351326259246e-05, 
       2.7938357058959506e-05, 
       3.2828890520187029e-05, 
       3.2770233936279608e-05, 
       2.4638322341815776e-05, 
       2.7419752884977669e-05, 
       2.7551620986824348e-05, 
       2.4717408738492065e-05, 
       2.4563168897390186e-05, 
       3.3509327979780881e-05, 
       2.5326275383638952e-05, 
       2.3799116281428056e-05, 
       2.5078777557785509e-05, 
       2.7797044458813704e-05, 
       2.7093497446702675e-05, 
       3.3386824173787324e-05, 
       2.4028524605538312e-05, 
       2.5241853807550540e-05, 
       3.3075257096191461e-05, 
       2.3940762638083593e-05, 
       2.5539549691732142e-05, 
       3.0298107282268505e-05, 
       2.5643172869294652e-05, 
       2.7801655062966039e-05, 
       2.7262720866021170e-05, 
       2.9297133812325343e-05, 
       2.4419010895282114e-05, 
       2.8753570159816621e-05, 
       2.3870787891065978e-05, 
       2.3949759645091643e-05, 
       3.1922406099002337e-05, 
       2.7145417761133343e-05, 
       2.7815890328425494e-05, 
       3.2767008263177699e-05, 
       3.0114034507210691e-05, 
       2.5253422774383424e-05, 
       3.0642727278611896e-05, 
       2.5824901150253911e-05, 
       2.3410591931027246e-05, 
       2.7068867471816275e-05, 
       3.2648895709794282e-05, 
       2.4672847676930602e-05, 
       2.3521448354927441e-05, 
       3.2518711624502546e-05, 
       3.2861158502551574e-05, 
       3.1844942524062685e-05, 
       3.2881728890569991e-05, 
       3.1764374761788188e-05, 
       2.9024300087927630e-05, 
       3.1848927648058240e-05, 
       2.5282329527292050e-05, 
       2.5029778423398572e-05, 
       3.2066513827816139e-05, 
       3.1608449532221451e-05, 
       3.0628291401627091e-05, 
       2.5823532110138717e-05, 
       2.7412843254975176e-05, 
       3.1409120820080540e-05, 
       2.8644311410063357e-05, 
       2.8421479340362332e-05, 
       3.1326045631095485e-05, 
       3.2231419140420244e-05, 
       2.7296652101255519e-05, 
       2.3630111405753731e-05, 
       3.2861327047380933e-05, 
       2.9712014048090544e-05, 
       2.3925798804251419e-05, 
       3.1342173101085250e-05, 
       3.0000000000004754e-05, 
       2.9226276659628415e-05, 
       3.0130530819280588e-05, 
       3.2117939065473566e-05, 
       2.6858777358858744e-05, 
       2.8870892942467787e-05, 
       2.4518909711126289e-05, 
       2.4412836780792744e-05, 
       2.6281263626433913e-05, 
       2.3639349392733013e-05, 
       2.3583956584488247e-05, 
       2.6253044776742218e-05, 
       2.5699148914549104e-05, 
       2.5699148914549104e-05, 
       2.3491457914644075e-05, 
       2.3314645847018726e-05, 
       2.9207144529710503e-05, 
       2.3640059687264301e-05, 
       2.8666666350483922e-05, 
       2.9586239836190514e-05, 
       2.4484525123260018e-05, 
       2.7343223212082859e-05, 
       3.2456652798194124e-05, 
       2.8378110328381696e-05, 
       2.7391739070760625e-05, 
       2.4306039405363230e-05, 
       2.4056732822544182e-05, 
       2.3095281581195130e-05, 
       2.5149677418359549e-05, 
       2.5574407376998318e-05, 
       2.4378657860132152e-05, 
       3.1993758386423529e-05, 
       2.7223142958714399e-05, 
       2.4233206767204053e-05, 
       2.6989436715820582e-05, 
       3.1987076575724620e-05, 
       3.0000000000004395e-05, 
       3.0000000000003175e-05, 
       3.0000000000001532e-05, 
       2.9999999999996718e-05, 
       2.9999999999997483e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999998727e-05, 
       2.6758860529685300e-05, 
       2.4208507609863603e-05, 
       2.3356403694532675e-05, 
       2.6533035133485840e-05, 
       2.3718753643780621e-05, 
       2.3631250496919172e-05, 
       3.2440097769823163e-05, 
       2.4649858560772045e-05, 
       2.9853590315916782e-05, 
       2.4187825442138748e-05, 
       2.6640559523824854e-05, 
       2.9125236549820838e-05, 
       2.4153943479388485e-05, 
       2.4803136279500559e-05, 
       2.4130287897950964e-05, 
       2.3478285487695365e-05, 
       2.3795274514457805e-05, 
       2.4515043771484238e-05, 
       2.4200953544176126e-05, 
       2.3574672895801181e-05, 
       3.2324110614769193e-05, 
       2.7105031494033407e-05, 
       2.2953778406406320e-05, 
       2.4229775483503690e-05, 
       2.3557520715966312e-05, 
       2.6788109297348951e-05, 
       2.7332334142113215e-05, 
       2.2839127833220500e-05, 
       2.4068093204499735e-05, 
       2.7596487872861401e-05, 
       2.5598385740589486e-05, 
       2.5684292612582737e-05, 
       2.4595847579348524e-05, 
       2.4595847579348524e-05, 
       3.0956762769961139e-05, 
       2.7942570891361763e-05, 
       2.2991096796153131e-05, 
       2.8513015651367395e-05, 
       2.6688545694479522e-05, 
       2.8102415001453398e-05, 
       2.4655169578557400e-05, 
       2.9794338332953304e-05, 
       2.2956581602696994e-05, 
       2.2931886952648440e-05, 
       2.4144187950297291e-05, 
       2.4029621090260527e-05, 
       3.0703370663606498e-05, 
       2.4229483928598198e-05, 
       2.3933366256948536e-05, 
       2.3982790606745475e-05, 
       2.3969868591592027e-05, 
       3.0317843506147008e-05, 
       2.6674576628837607e-05, 
       2.3367356361450269e-05, 
       3.2031640655859457e-05, 
       3.0469737523165958e-05, 
       2.2757945858743646e-05, 
       2.4043458285882960e-05, 
       2.6542271326007287e-05, 
       2.3374861431159261e-05, 
       2.6991000384326193e-05, 
       2.8445432000485446e-05, 
       2.6989834427639376e-05, 
       2.8518207218564777e-05, 
       2.5216668416586875e-05, 
       2.5265550372095493e-05, 
       2.6569198107656936e-05, 
       2.6069881770093472e-05, 
       2.2953313332721326e-05, 
       2.6523415795344397e-05, 
       2.3344000402472098e-05, 
       2.2673881814649344e-05, 
       2.3933550003904084e-05, 
       2.3425921182439749e-05, 
       2.3792196270274422e-05, 
       2.2549797917558873e-05, 
       2.3448472113271315e-05, 
       2.9736931505981631e-05, 
       2.8723867369330672e-05, 
       2.2650778794561397e-05, 
       2.8328680923543289e-05, 
       2.6058267342644204e-05, 
       2.6592908014785288e-05, 
       3.1429655936959337e-05, 
       2.6439406135915164e-05, 
       2.5352515241921984e-05, 
       2.2494846800343745e-05, 
       2.2610655526867572e-05, 
       2.6892453776254730e-05, 
       2.2452683362133069e-05, 
       2.7301361290992842e-05, 
       2.3366845393318476e-05, 
       3.1135331500622071e-05, 
       2.4455903170381747e-05, 
       2.2526824852320108e-05, 
       2.2911820874109461e-05, 
       3.1578470513111974e-05, 
       3.0928779491943031e-05, 
       2.6518867773819540e-05, 
       3.1641879052901912e-05, 
       2.3590873027809217e-05, 
       2.3533876412302279e-05, 
       2.5803952430325030e-05, 
       2.3481949138645131e-05, 
       2.3481949138645131e-05, 
       2.8650311534162916e-05, 
       3.1001081832557546e-05, 
       2.2428789971638705e-05, 
       2.9898044865364266e-05, 
       2.9898044865364266e-05, 
       3.1548666571319805e-05, 
       2.3744016468473627e-05, 
       2.2994072585300936e-05, 
       2.3669362019641366e-05, 
       3.1521701032825531e-05, 
       3.0921012361174026e-05, 
       2.7360023105400635e-05, 
       2.6151704190524080e-05, 
       3.0751001713465921e-05, 
       3.1095888488842015e-05, 
       2.5716451000296051e-05, 
       2.9485222679581076e-05, 
       2.5159823144131633e-05, 
       2.7358770837891393e-05, 
       3.0530666011066705e-05, 
       3.0530666011066705e-05, 
       2.5449544575104073e-05, 
       3.0257544841379307e-05, 
       2.3856647544495763e-05, 
       2.7939450677757918e-05, 
       2.3671541985041571e-05, 
       2.8884827226786955e-05, 
       2.6511318383138590e-05, 
       2.4060262420451600e-05, 
       3.0641464985197976e-05, 
       2.5963420292856160e-05, 
       2.6065484576414844e-05, 
       2.3323596281785612e-05, 
       3.1040418763977577e-05, 
       2.3598198909936021e-05, 
       2.5979661052908224e-05, 
       2.2432774048719995e-05, 
       3.1180582639743129e-05, 
       3.1102052418651771e-05, 
       2.2634589229571477e-05, 
       3.0282535675771443e-05, 
       2.8454865268473373e-05, 
       3.1177362330310649e-05, 
       2.8273044383323993e-05, 
       2.9423592863181931e-05, 
       2.2960727133023318e-05, 
       2.2246767066760323e-05, 
       3.1060122990885818e-05, 
       2.3517871577331563e-05, 
       2.6840541795438060e-05, 
       2.5668902765180792e-05, 
       2.2693446515808260e-05, 
       2.4529322826566388e-05, 
       2.2047351993588615e-05, 
       2.3968912046537010e-05, 
       3.1058413944483223e-05, 
       2.5223326169483067e-05, 
       2.4665068427518309e-05, 
       2.2649673538213300e-05, 
       2.2157035469453294e-05, 
       2.6180330469735683e-05, 
       2.3644029360033887e-05, 
       2.3202026565367760e-05, 
       2.5162018786713200e-05, 
       2.3987691355586738e-05, 
       2.3131084972117483e-05, 
       3.1008175381355799e-05, 
       2.9313320843176681e-05, 
       2.8903601511474440e-05, 
       2.9639029173233814e-05, 
       2.3135850915461947e-05, 
       2.4072322275920603e-05, 
       2.2501968006603904e-05, 
       2.2024431361496799e-05, 
       2.3116081764594719e-05, 
       2.2998232343133420e-05, 
       2.2118110942939362e-05, 
       3.0876353023653049e-05, 
       2.8301764688688739e-05, 
       3.0889114776607159e-05, 
       2.5778263102697021e-05, 
       3.0856737713956672e-05, 
       2.4823812133505298e-05, 
       2.3873222720876555e-05, 
       2.1833431454969353e-05, 
       2.6552667117472001e-05, 
       3.0823793873458243e-05, 
       2.2165776438755421e-05, 
       2.3328140347630406e-05, 
       2.5622001933565781e-05, 
       2.2200358561387213e-05, 
       2.2000721234050828e-05, 
       2.2781670504722637e-05, 
       2.9779706727095740e-05, 
       2.2266811169566457e-05, 
       2.2933683581686270e-05, 
       2.6383768176218102e-05, 
       2.3490016088401157e-05, 
       2.2370681954512725e-05, 
       2.1737856923475611e-05, 
       2.2575549820837218e-05, 
       2.2977010371368420e-05, 
       2.4366059144324251e-05, 
       2.3586278733158683e-05, 
       3.0585956995254478e-05, 
       2.1659009292546130e-05, 
       2.3648264828023183e-05, 
       2.1914539979488495e-05, 
       2.5401265396047047e-05, 
       2.2555914528845136e-05, 
       3.0429755882210373e-05, 
       2.5459999545576019e-05, 
       2.7188135062864157e-05, 
       2.2267230224301959e-05, 
       2.3294591229374253e-05, 
       2.3294591229374253e-05, 
       2.6953573066187078e-05, 
       2.8290081438676998e-05, 
       2.8290081438676998e-05, 
       2.3130028017194969e-05, 
       3.0194313956997775e-05, 
       2.4735936596314983e-05, 
       2.2043917412816432e-05, 
       2.1648551552999270e-05, 
       2.2106802004440897e-05, 
       2.3540983502337627e-05, 
       2.2822233971491185e-05, 
       2.5873627814288544e-05, 
       2.5071944627958816e-05, 
       2.9402387053321753e-05, 
       2.1853040963288492e-05, 
       2.6467454135017226e-05, 
       2.3065278955422418e-05, 
       2.2238613532393600e-05, 
       2.3329180401850149e-05, 
       2.3196085076249708e-05, 
       2.2373094312897225e-05, 
       2.9866956616811273e-05, 
       2.5922591923065017e-05, 
       2.5250645009954058e-05, 
       2.2524488970521550e-05, 
       2.5265532237725771e-05, 
       2.3730332366992355e-05, 
       2.2248564829250396e-05, 
       2.5697352832721036e-05, 
       2.5897815625199073e-05, 
       3.0315277840851936e-05, 
       2.4580305047934197e-05, 
       2.4960653368182458e-05, 
       2.5155507038084925e-05, 
       2.4625825675360571e-05, 
       2.1632897588302667e-05, 
       3.0024465305102980e-05, 
       2.2880606084721630e-05, 
       3.0112102204550933e-05, 
       3.0217606228385636e-05, 
       2.6954143836136093e-05, 
       2.2332393502665777e-05, 
       2.9425975060388507e-05, 
       2.9380986474664640e-05, 
       2.2521523034482854e-05, 
       2.1331277949814846e-05, 
       3.0091286988889271e-05, 
       2.3719212332704527e-05, 
       2.9853555372591540e-05, 
       2.6162844361068285e-05, 
       2.3612129708629224e-05, 
       2.1407265443088452e-05, 
       2.5606384124734948e-05, 
       2.9221015565257169e-05, 
       2.1926441957545874e-05, 
       3.0000000000004412e-05, 
       2.9999999999999692e-05, 
       3.0000000000002569e-05, 
       3.0000000000002054e-05, 
       3.0000000000002054e-05, 
       3.0000000000000529e-05, 
       3.0000000000004487e-05, 
       2.9999999999997609e-05, 
       3.0000000000001105e-05, 
       2.9999999999999262e-05, 
       3.0000000000003806e-05, 
       2.9999999999999310e-05, 
       2.9999999999999425e-05, 
       3.0000000000002372e-05, 
       2.9999999999999940e-05, 
       2.9999999999999113e-05, 
       2.9999999999999113e-05, 
       3.0000000000000211e-05, 
       2.9999999999998937e-05, 
       2.9999999999998937e-05, 
       3.0000000000002826e-05, 
       2.9999999999997080e-05, 
       3.0000000000002759e-05, 
       3.0000000000000001e-05, 
       2.9967376100969666e-05, 
       2.1258585642290856e-05, 
       2.9878936701497045e-05, 
       2.2730705198157709e-05, 
       2.3163023385621234e-05, 
       2.4812463451754450e-05, 
       2.5353282395461595e-05, 
       2.2249489624176713e-05, 
       2.2368220992200166e-05, 
       2.9764587560147846e-05, 
       2.2244504661178118e-05, 
       2.2010075790050645e-05, 
       2.1181177183619379e-05, 
       2.2312673646753555e-05, 
       2.7988631464862571e-05, 
       2.9463742257525140e-05, 
       2.9384212561671601e-05, 
       2.1274702521020825e-05, 
       2.1102969096694835e-05, 
       2.2811155946580890e-05, 
       2.4377873012068023e-05, 
       2.4774953526761876e-05, 
       2.9086370645898128e-05, 
       2.1106764841317570e-05, 
       2.2219273201087013e-05, 
       2.2507511707896874e-05, 
       2.8580847492409141e-05, 
       2.7984905149733606e-05, 
       2.2678784325977479e-05, 
       2.1500917337873341e-05, 
       2.9149393640904396e-05, 
       2.3693510630709877e-05, 
       2.4239299196179574e-05, 
       2.5619415815442632e-05, 
       2.4606503422581895e-05, 
       2.4232812774850916e-05, 
       2.1479616746356397e-05, 
       2.4658734737019408e-05, 
       2.2377877345045009e-05, 
       2.0975302634761681e-05, 
       2.5696588232389626e-05, 
       2.1074236127339265e-05, 
       2.2515595496432635e-05, 
       2.3345035645919719e-05, 
       2.1158828201940872e-05, 
       2.0892482302087414e-05, 
       2.1388043356319155e-05, 
       2.7720231634122997e-05, 
       2.7420514974535629e-05, 
       2.0971438064451802e-05, 
       2.1282081610973466e-05, 
       2.3457273705969547e-05, 
       2.5449544575112137e-05, 
       2.1716943357788268e-05, 
       2.2363242451000932e-05, 
       2.8204496935918308e-05, 
       2.2396538869570202e-05, 
       2.4987528007939380e-05, 
       2.4838944864421288e-05, 
       2.4838944864421288e-05, 
       2.1563470081071840e-05, 
       2.1370585230125476e-05, 
       2.2482179206950944e-05, 
       2.5470090470973020e-05, 
       2.1972545968161488e-05, 
       2.0782501298533299e-05, 
       2.0963483266505083e-05, 
       2.3559251366869484e-05, 
       2.0834141204981505e-05, 
       2.1833172299397551e-05, 
       2.1655164963762016e-05, 
       2.8817619302988359e-05, 
       2.6225005235145851e-05, 
       2.1369445279637196e-05, 
       2.5148629093081621e-05, 
       2.0770465547790713e-05, 
       2.4494103931738696e-05, 
       2.4209462171757982e-05, 
       2.3773107992724942e-05, 
       2.1363995710048967e-05, 
       2.2044626518182909e-05, 
       2.3759393415435389e-05, 
       2.4805081974584825e-05, 
       2.3179116064277322e-05, 
       2.1514401895830754e-05, 
       2.7616296217120512e-05, 
       2.2709046354466190e-05, 
       2.8749879027748582e-05, 
       2.8192232780338984e-05, 
       2.0641157365171598e-05, 
       2.0791988818258568e-05, 
       2.2673633041937346e-05, 
       2.4165052128570052e-05, 
       2.4554041627304931e-05, 
       2.4554041627304931e-05, 
       2.7578189541945367e-05, 
       2.2812129845507126e-05, 
       2.5525537580048646e-05, 
       2.6098638829645853e-05, 
       2.2412501953195292e-05, 
       2.3033545935114953e-05, 
       2.0666215024317245e-05, 
       2.0693914290865395e-05, 
       2.0595726018747476e-05, 
       2.1580073915734177e-05, 
       2.4426109735967227e-05, 
       2.3391625466620212e-05, 
       2.8729696770726635e-05, 
       2.3376016344286541e-05, 
       2.1661261675198909e-05, 
       2.3587605957316008e-05, 
       2.4809251086063175e-05, 
       2.5159903276305127e-05, 
       2.0966463382454883e-05, 
       2.2090190791695682e-05, 
       2.0874798315180848e-05, 
       2.7260535518770556e-05, 
       2.1555418181337062e-05, 
       2.0478511218294781e-05, 
       2.0673722153604741e-05, 
       2.0673722153604741e-05, 
       2.5594558139612343e-05, 
       2.1167664623335956e-05, 
       2.0377182125879216e-05, 
       2.1391018867755895e-05, 
       2.2015926449780460e-05, 
       2.3066860679774381e-05, 
       2.1044399250220063e-05, 
       2.0579226170623280e-05, 
       2.3883321091715731e-05, 
       2.1903314260049617e-05, 
       2.1903314260049617e-05, 
       2.3598474375245868e-05, 
       2.0573410466495585e-05, 
       2.1734658126821529e-05, 
       2.7773991371145030e-05, 
       2.5967557331454341e-05, 
       2.8132873998052460e-05, 
       2.3651379913604591e-05, 
       2.6776563934697187e-05, 
       2.3783603814403211e-05, 
       2.0966317914944670e-05, 
       2.8545353480027904e-05, 
       2.0297020183652112e-05, 
       2.1014108239154567e-05, 
       2.0204735124811388e-05, 
       2.8352944682601519e-05, 
       2.1619502830263783e-05, 
       2.3293885456198640e-05, 
       2.0652180205250836e-05, 
       2.3063330146987459e-05, 
       2.0342947951443127e-05, 
       2.8323424751774202e-05, 
       2.1382184363406343e-05, 
       2.1586278014501960e-05, 
       2.0898790363700101e-05, 
       2.3658093225887116e-05, 
       2.1207884684600929e-05, 
       2.8441862496031210e-05, 
       2.0919243927195111e-05, 
       2.4020699456820609e-05, 
       2.4535409561536329e-05, 
       2.7075029329950524e-05, 
       2.8119847590653811e-05, 
       2.1211440014521348e-05, 
       2.5197119576424604e-05, 
       2.5797501301016550e-05, 
       2.3275728849808974e-05, 
       2.3077675354490958e-05, 
       2.4086360757039374e-05, 
       2.3128265093259274e-05, 
       2.0028676292895690e-05, 
       2.3522889548463088e-05, 
       2.4177213013043984e-05, 
       2.0492459356269730e-05, 
       2.3512154471688490e-05, 
       2.0000000000002217e-05, 
       2.0428470091726012e-05, 
       2.1308814110592108e-05, 
       2.7816142445164482e-05, 
       2.7843214917612733e-05, 
       2.3956837111922531e-05, 
       2.0558819361950228e-05, 
       2.5748900607172417e-05, 
       2.4653736170723783e-05, 
       2.1066849907602962e-05, 
       2.0654506029328560e-05, 
       2.1995466457976440e-05, 
       2.1108296035024811e-05, 
       2.2997515178881452e-05, 
       2.0683641650317797e-05, 
       2.0299625122938209e-05, 
       2.1367696919782209e-05, 
       2.3470292930444889e-05, 
       2.3442989589177679e-05, 
       2.1448600708759867e-05, 
       2.5875790059004020e-05, 
       2.0206901013864554e-05, 
       2.2340762747346633e-05, 
       2.7668434375695498e-05, 
       1.9950397140628340e-05, 
       2.7987382645874545e-05, 
       2.4293432174721289e-05, 
       2.0000000000001635e-05, 
       2.3289522212324585e-05, 
       2.6411884857133361e-05, 
       1.9873280032980062e-05, 
       2.1714968767927616e-05, 
       2.1839395959091631e-05, 
       2.1839395959091631e-05, 
       2.3466193893882380e-05, 
       2.1371321669396220e-05, 
       1.9938647667130330e-05, 
       2.0579979780548632e-05, 
       2.0000000000002234e-05, 
       2.8028569936047660e-05, 
       2.1771973961456877e-05, 
       2.0865597099147342e-05, 
       2.0865597099147342e-05, 
       2.6509081981774738e-05, 
       2.0000000000000717e-05, 
       2.3312392036192551e-05, 
       2.0258657597617159e-05, 
       2.0312599848835227e-05, 
       2.0920446986977366e-05, 
       2.3578648602996724e-05, 
       2.2436677140611300e-05, 
       2.7961471003000168e-05, 
       2.1166956644026552e-05, 
       1.9816622275304193e-05, 
       2.3246323501454473e-05, 
       2.7599870125663251e-05, 
       2.7997160563574968e-05, 
       1.9800172644680534e-05, 
       2.3465473046836687e-05, 
       2.3466454990672666e-05, 
       2.0125642276699688e-05, 
       2.0989461740345620e-05, 
       2.7888274606794038e-05, 
       2.7044949730180104e-05, 
       2.0966102464808802e-05, 
       2.0581903307930400e-05, 
       2.7892539894698830e-05, 
       1.9737154177717354e-05, 
       1.9899679834429881e-05, 
       2.6918734450128765e-05, 
       2.1161418318711277e-05, 
       2.2501997422535255e-05, 
       2.2738960599759796e-05, 
       2.1220680407509679e-05, 
       2.7833813514487734e-05, 
       1.9762798536095193e-05, 
       2.2741211861016365e-05, 
       2.3222112650948485e-05, 
       2.3079346777615531e-05, 
       2.0291190701228842e-05, 
       1.9766670501007140e-05, 
       1.9766670501007140e-05, 
       2.0875764955551667e-05, 
       2.7496573607971389e-05, 
       2.7824980272308788e-05, 
       1.9851321667405355e-05, 
       2.6730772067331172e-05, 
       1.9692576643129609e-05, 
       2.1649674267789876e-05, 
       1.9731697228219266e-05, 
       2.7742305224850582e-05, 
       2.7417919438258271e-05, 
       2.0390893820573060e-05, 
       1.9932359809715495e-05, 
       2.7296124548866782e-05, 
       2.2695630930497120e-05, 
       2.0814759615587026e-05, 
       2.1687285062861226e-05, 
       2.6026686656087951e-05, 
       2.4822390716806992e-05, 
       1.9633408681435115e-05, 
       2.0000000000004044e-05, 
       2.4890159914209150e-05, 
       2.7567145184525361e-05, 
       2.2996908295505237e-05, 
       2.1302429968690023e-05, 
       2.3854713945038826e-05, 
       1.9715524650095351e-05, 
       2.7175854351842625e-05, 
       2.4191135362079194e-05, 
       2.0627872820108754e-05, 
       2.2919391864464396e-05, 
       2.6727338643468326e-05, 
       2.7254384117928532e-05, 
       2.0427343831901037e-05, 
       1.9507494707828872e-05, 
       2.3171225864097440e-05, 
       1.9999999999996590e-05, 
       1.9999999999996590e-05, 
       2.2986803073853592e-05, 
       2.1918124810525084e-05, 
       2.6884276333680600e-05, 
       2.0398485690050204e-05, 
       2.4849261494498854e-05, 
       2.6960534050138303e-05, 
       2.1421354160062961e-05, 
       2.4551814619669700e-05, 
       2.4314578972066763e-05, 
       2.6363367334252085e-05, 
       1.9793834371186968e-05, 
       1.9440818566363255e-05, 
       1.9459987741405260e-05, 
       1.9459987741405260e-05, 
       2.0193170237659376e-05, 
       2.6496086993112514e-05, 
       2.2344511393982552e-05, 
       2.6577328173695179e-05, 
       2.2427716074009642e-05, 
       2.4107810868212471e-05, 
       2.1284459172261010e-05, 
       2.2020722328864483e-05, 
       2.6607255959589445e-05, 
       2.7202002368497076e-05, 
       2.0166011832938444e-05, 
       2.5207862623641681e-05, 
       2.5207862623641681e-05, 
       1.9999999999995265e-05, 
       2.0450194783755439e-05, 
       2.2263441842113082e-05, 
       2.7406316216041685e-05, 
       1.9409515491409219e-05, 
       2.2039607298616913e-05, 
       2.1152152459107805e-05, 
       2.2230502365173873e-05, 
       1.9670319855560186e-05, 
       2.4722180326833094e-05, 
       2.2763295970793442e-05, 
       2.2774519638715391e-05, 
       2.6679869687682337e-05, 
       2.4756484656674434e-05, 
       2.0055294309148890e-05, 
       2.0897446815595530e-05, 
       2.4428532151668143e-05, 
       2.3990605543756118e-05, 
       2.7254286047169275e-05, 
       2.4278298363497046e-05, 
       2.2314581840559189e-05, 
       2.5743564664592442e-05, 
       1.9275105701808604e-05, 
       2.0049170001202115e-05, 
       1.9902356931750910e-05, 
       2.6313437919997652e-05, 
       2.7152726206478358e-05, 
       2.0401429930896030e-05, 
       2.0330718397024442e-05, 
       2.6165096318595719e-05, 
       1.9265299359219691e-05, 
       2.1568915069916871e-05, 
       2.0706080105933901e-05, 
       2.3303586545614864e-05, 
       1.9501731727464772e-05, 
       1.9559861810741433e-05, 
       2.1057046631608082e-05, 
       1.9502976870692780e-05, 
       2.4809720863124641e-05, 
       2.1408962260453609e-05, 
       2.1125619940789139e-05, 
       2.6828077807227632e-05, 
       2.0038477634749918e-05, 
       1.9130409289656855e-05, 
       1.9130409289656855e-05, 
       2.2346531757305858e-05, 
       2.1942710646256503e-05, 
       2.1008609943568522e-05, 
       2.1008609943568522e-05, 
       1.9277210464739012e-05, 
       2.2661137357054017e-05, 
       2.2516314479552214e-05, 
       2.6851072298362163e-05, 
       2.1908850916506237e-05, 
       2.2483068492197152e-05, 
       2.1696394476536286e-05, 
       2.2098923932636108e-05, 
       2.0187822485852769e-05, 
       2.5808682445505923e-05, 
       2.0017282176804168e-05, 
       2.0017282176804168e-05, 
       1.9126965962250800e-05, 
       1.9126965962250800e-05, 
       2.3006546974076691e-05, 
       2.3179533542101368e-05, 
       2.6890690896602348e-05, 
       2.6021653074393433e-05, 
       2.6181070741757018e-05, 
       1.9913283459284369e-05, 
       1.9636588737164034e-05, 
       2.2324367947449302e-05, 
       2.3865313956306112e-05, 
       2.1618881238829151e-05, 
       1.9535217125231259e-05, 
       1.9041443690995283e-05, 
       2.0039573023823696e-05, 
       2.2953513679398349e-05, 
       2.5567766231399125e-05, 
       1.9895906118055519e-05, 
       2.2271290300944949e-05, 
       2.6754209861454405e-05, 
       1.9007159132051157e-05, 
       1.8967818768193727e-05, 
       2.2691966842233759e-05, 
       1.8960610391852309e-05, 
       2.6716661520230744e-05, 
       2.5927823756213389e-05, 
       2.2844697314839551e-05, 
       2.3747869267251670e-05, 
       2.2145624115503147e-05, 
       1.9469447275291399e-05, 
       2.3456903323842540e-05, 
       2.0036159169612765e-05, 
       1.9400261141639878e-05, 
       1.8867912229973693e-05, 
       1.9059994910767383e-05, 
       1.8890362507229716e-05, 
       2.2017503441595469e-05, 
       2.4559321891593536e-05, 
       2.6338258067374874e-05, 
       1.9999999999998321e-05, 
       2.3950871968419052e-05, 
       2.1899470576429120e-05, 
       2.3545632007714398e-05, 
       1.8815345378508486e-05, 
       2.2113213212395635e-05, 
       2.2127291837682675e-05, 
       1.9273170719726031e-05, 
       2.6415429930153639e-05, 
       2.6239168240300300e-05, 
       1.8850214491991355e-05, 
       1.9768044576141600e-05, 
       1.9768044576141600e-05, 
       2.0820150284495630e-05, 
       2.0456223851454600e-05, 
       2.2395140178245569e-05, 
       2.6481718745530546e-05, 
       1.9778488240021942e-05, 
       1.8897410381751787e-05, 
       2.5684915178804853e-05, 
       1.8732589574812063e-05, 
       1.8925710859862174e-05, 
       1.9214149212320105e-05, 
       1.9999999999997667e-05, 
       1.9751377763641712e-05, 
       2.1459337858959660e-05, 
       1.9070305230096828e-05, 
       1.8771507026313736e-05, 
       1.9855656259784941e-05, 
       1.8749988728834255e-05, 
       2.2154281478803802e-05, 
       2.1850451281937156e-05, 
       2.2643318038647409e-05, 
       2.2008951276807387e-05, 
       2.0000000000002539e-05, 
       2.2880427612072051e-05, 
       1.9999999999995878e-05, 
       2.4877237016597618e-05, 
       2.2035984449286025e-05, 
       2.0000000000003109e-05, 
       1.9357035115452651e-05, 
       1.9530244037060011e-05, 
       2.2200628370302749e-05, 
       2.0062154492898264e-05, 
       2.3797117726057506e-05, 
       1.9999999999999697e-05, 
       1.8720368835276171e-05, 
       2.5213111253492232e-05, 
       1.8629141526322722e-05, 
       2.5875015718519001e-05, 
       1.9084191995397657e-05, 
       2.1858681197141754e-05, 
       2.6297689960513770e-05, 
       2.6106027196373846e-05, 
       2.2532828039759901e-05, 
       1.9603826032796683e-05, 
       2.1491473288966037e-05, 
       1.8922290352108163e-05, 
       2.6113247302592242e-05, 
       1.9149633976069442e-05, 
       1.8811843292880828e-05, 
       1.9999999999996088e-05, 
       1.9031035429947053e-05, 
       1.8940829007484389e-05, 
       1.9559353629394119e-05, 
       2.3107745574945384e-05, 
       2.6187273058042965e-05, 
       2.0552653667056900e-05, 
       2.1181177183621364e-05, 
       2.1735928533133341e-05, 
       1.9541597193925965e-05, 
       2.1297797499442487e-05, 
       2.3815495268834883e-05, 
       1.9522285792446204e-05, 
       2.2414619069298451e-05, 
       2.1210767134427090e-05, 
       2.1210267732373230e-05, 
       1.9493906790999241e-05, 
       2.4122252971414907e-05, 
       2.1626639836759702e-05, 
       2.2433786700830839e-05, 
       2.2344292372869138e-05, 
       2.2344292372869138e-05, 
       2.6077452515192849e-05, 
       2.5948213700606818e-05, 
       2.4560982150199448e-05, 
       1.8596696142305167e-05, 
       2.5871759928564313e-05, 
       1.9407277176706601e-05, 
       2.6005775386305672e-05, 
       1.9170122027778372e-05, 
       1.9170122027778372e-05, 
       2.2336586056727969e-05, 
       1.8756522821119444e-05, 
       2.0600324216915043e-05, 
       1.8391394104241385e-05, 
       1.9110730691336842e-05, 
       1.8841348400882591e-05, 
       1.9314907369897868e-05, 
       2.1314253102740545e-05, 
       2.3076103047365980e-05, 
       1.9484253497074381e-05, 
       2.5834814580752454e-05, 
       1.8365488924781366e-05, 
       2.5016541916426808e-05, 
       2.1012328915233477e-05, 
       1.9999999999996847e-05, 
       2.1496989977947132e-05, 
       2.5804511066017738e-05, 
       2.5762157378443724e-05, 
       2.5836435802320644e-05, 
       2.1860975195960274e-05, 
       2.5687587942747713e-05, 
       1.8981530758458973e-05, 
       1.9139111905999696e-05, 
       1.8280166318881625e-05, 
       2.4070343580249986e-05, 
       1.8693379445236393e-05, 
       2.3962516453856421e-05, 
       1.9020137080359865e-05, 
       1.9247958464256808e-05, 
       2.1447310559871313e-05, 
       1.9855861400691827e-05, 
       2.2468046196661062e-05, 
       2.0728155512884324e-05, 
       2.4060218809271120e-05, 
       2.1149639292928914e-05, 
       2.4517282613909728e-05, 
       2.2771292414867269e-05, 
       2.5579069717142981e-05, 
       2.1397066743789710e-05, 
       1.8961682355017992e-05, 
       1.8961682355017992e-05, 
       2.1655772967855787e-05, 
       2.5693726133351492e-05, 
       1.8163884442569935e-05, 
       1.9107983344487789e-05, 
       1.9114684912089114e-05, 
       1.9294009701245560e-05, 
       2.5660087323933798e-05, 
       1.8838086560794785e-05, 
       1.8858928509117386e-05, 
       1.8219051742317011e-05, 
       1.8728757426271528e-05, 
       2.2818810759957005e-05, 
       2.3715650740840214e-05, 
       1.8130283744853847e-05, 
       2.2938816829746240e-05, 
       1.8176890579316707e-05, 
       1.9952609681202569e-05, 
       1.8251530509771619e-05, 
       2.4254948895590587e-05, 
       1.8290989507197052e-05, 
       2.0376107544335592e-05, 
       1.9054448403108678e-05, 
       2.0352200663612104e-05, 
       1.8107252081351881e-05, 
       2.5531308981118411e-05, 
       2.5411294279712070e-05, 
       2.1633389361055961e-05, 
       1.9628235793110991e-05, 
       1.9180833451816496e-05, 
       1.9847621489528326e-05, 
       1.8585369921435439e-05, 
       1.9280398867673850e-05, 
       1.8308666362155669e-05, 
       2.2006291036435541e-05, 
       2.0242159615772351e-05, 
       2.1713381528779144e-05, 
       2.0934449436080176e-05, 
       1.8139101146347185e-05, 
       2.0731426911967222e-05, 
       1.8696409943926012e-05, 
       2.4544245828620170e-05, 
       2.2783430089359999e-05, 
       1.7993730535971711e-05, 
       2.5339301420864887e-05, 
       2.3338603348431365e-05, 
       2.4363468166549399e-05, 
       1.9122521503458406e-05, 
       1.8512114069329657e-05, 
       1.7958421814018464e-05, 
       2.1326923127170881e-05, 
       1.9199696490683087e-05, 
       2.1003649145343692e-05, 
       2.1989288523769274e-05, 
       1.7930229589536953e-05, 
       1.8555827680287684e-05, 
       1.7909710544758879e-05, 
       2.1385311528258373e-05, 
       2.1043458671072926e-05, 
       1.8526453187106103e-05, 
       1.9841228398451822e-05, 
       2.2256428064753554e-05, 
       1.8130641321224758e-05, 
       2.0449994146295535e-05, 
       2.3785604022540511e-05, 
       2.0731298984516647e-05, 
       2.0760416759699299e-05, 
       2.5167052823156837e-05, 
       1.8174308171080877e-05, 
       1.8585059002567886e-05, 
       2.2380340613526966e-05, 
       1.8869498634825147e-05, 
       1.8269361341749325e-05, 
       2.0893323738844832e-05, 
       2.0100910348089074e-05, 
       2.0094288114021303e-05, 
       2.2404243188656371e-05, 
       2.5016175558792949e-05, 
       2.0831810051257426e-05, 
       2.0259728632858026e-05, 
       1.9242038929860518e-05, 
       2.1289342496334620e-05, 
       2.0234278488705536e-05, 
       1.9379271630522181e-05, 
       1.8010372286510348e-05, 
       2.5144607602309949e-05, 
       2.0187915931443160e-05, 
       2.1996352765883427e-05, 
       2.0904858800077536e-05, 
       2.1720039102126300e-05, 
       2.3767208220916401e-05, 
       2.0865928634290329e-05, 
       2.4890723895812140e-05, 
       2.5047055332498873e-05, 
       2.4201236963873363e-05, 
       2.3169373330216857e-05, 
       2.3163038376964701e-05, 
       1.8855190323723966e-05, 
       1.8855190323723966e-05, 
       1.8671149418406665e-05, 
       1.7910303507043153e-05, 
       1.7763979523353320e-05, 
       1.7680410653021601e-05, 
       1.7687133192360568e-05, 
       1.7892962818897249e-05, 
       2.0333066309177655e-05, 
       2.2327869922090372e-05, 
       2.0094886283726992e-05, 
       2.0821716037595176e-05, 
       1.8614770868802281e-05, 
       1.7720496265061905e-05, 
       2.4900219533208990e-05, 
       1.7697729913895767e-05, 
       1.9844408131246652e-05, 
       1.9639872230426079e-05, 
       2.1388329725444331e-05, 
       1.7640375920275090e-05, 
       1.7803342763302024e-05, 
       2.4912020906200038e-05, 
       2.4590699583193215e-05, 
       2.0001499662828823e-05, 
       1.7691394246998973e-05, 
       1.7691394246998973e-05, 
       2.0101747579768383e-05, 
       2.3994530711384027e-05, 
       1.8965759925222337e-05, 
       2.2918838738801451e-05, 
       2.2918838738801451e-05, 
       2.0676047575015440e-05, 
       1.8568461860930664e-05, 
       2.0616313862486782e-05, 
       1.8005314258452920e-05, 
       2.4588443881671791e-05, 
       1.8886457529499383e-05, 
       2.0710696799016547e-05, 
       1.7699193284962316e-05, 
       2.4410728419558051e-05, 
       2.2977044570771224e-05, 
       2.0614114202624367e-05, 
       2.3861379884331902e-05, 
       2.4443744519722304e-05, 
       1.8738042318730059e-05, 
       2.0202721180008084e-05, 
       2.4805918098041182e-05, 
       2.0008714673284008e-05, 
       2.3977528418830052e-05, 
       1.9875094911181225e-05, 
       1.7841857479155481e-05, 
       1.7841857479155481e-05, 
       1.9913097446718887e-05, 
       1.7876435978992887e-05, 
       1.8143751657890148e-05, 
       2.4509662452613220e-05, 
       1.7532411669743742e-05, 
       1.8009989763961877e-05, 
       1.7744356854316162e-05, 
       1.8566432369396533e-05, 
       2.0116060720729326e-05, 
       1.8498218562056421e-05, 
       2.1463113121993310e-05, 
       1.9713299927463456e-05, 
       2.3052850846839306e-05, 
       1.7673110224656203e-05, 
       2.4239977584247124e-05, 
       2.0040883798321096e-05, 
       1.9417256525828080e-05, 
       1.9417256525828080e-05, 
       2.0955702972504675e-05, 
       1.8919347476838179e-05, 
       2.4432582009374769e-05, 
       1.8164088693106744e-05, 
       1.7520072035584252e-05, 
       1.9532365774056529e-05, 
       2.1382414917835685e-05, 
       1.9738472702713679e-05, 
       1.7629541808994239e-05, 
       2.0616644767350053e-05, 
       1.7703633377054495e-05, 
       1.8116772429845239e-05, 
       1.9879440855986523e-05, 
       2.0124772551965589e-05, 
       1.7864366935218176e-05, 
       1.8810990471941299e-05, 
       1.9743748452707123e-05, 
       2.3865065718840363e-05, 
       2.1158828201933256e-05, 
       1.9780547239243782e-05, 
       1.9722888630204921e-05, 
       2.1281486054161358e-05, 
       1.7352701970226175e-05, 
       1.9057764607714187e-05, 
       2.1854926101455391e-05, 
       2.1335342960777997e-05, 
       2.3905614432718570e-05, 
       2.4508257473243487e-05, 
       1.8105890428700859e-05, 
       1.8105890428700859e-05, 
       2.3425411593397924e-05, 
       1.8465966771970703e-05, 
       2.1329285074762914e-05, 
       1.9554010968489385e-05, 
       1.9554010968489385e-05, 
       1.7306685536057503e-05, 
       2.1740601870453343e-05, 
       1.8263059311301995e-05, 
       2.4316548339899093e-05, 
       2.3547674773871388e-05, 
       2.3480237790270722e-05, 
       2.0961556471640558e-05, 
       2.2754038596753759e-05, 
       1.9999999999996922e-05, 
       1.9999999999996922e-05, 
       1.7697016103600526e-05, 
       2.3535283663645454e-05, 
       1.7392481252393953e-05, 
       1.8206092048519223e-05, 
       2.4165255746693801e-05, 
       1.7987481553165490e-05, 
       1.9548511366159317e-05, 
       1.7635978005143752e-05, 
       2.0343072227592185e-05, 
       1.9525566385121238e-05, 
       1.7941370251689439e-05, 
       2.3313479894531704e-05, 
       1.7316200591392876e-05, 
       2.0256262953270805e-05, 
       2.2782267752081354e-05, 
       2.0722440633148783e-05, 
       2.2570865527763971e-05, 
       2.2060084096032708e-05, 
       1.8159538450873559e-05, 
       1.8536393920439748e-05, 
       1.7557962389639970e-05, 
       2.0002834694791437e-05, 
       2.3342952109573024e-05, 
       2.2086456562446664e-05, 
       2.0498343612431602e-05, 
       1.7351005763319398e-05, 
       1.8528906464525795e-05, 
       1.7697574838161988e-05, 
       2.3462174193718691e-05, 
       1.9415110018429218e-05, 
       2.1620357335974060e-05, 
       2.2891788457019663e-05, 
       2.1585633011509839e-05, 
       1.8404306776497948e-05, 
       1.8404306776497948e-05, 
       2.0166310355110907e-05, 
       2.1170275625818371e-05, 
       1.9370530680985174e-05, 
       1.7147050315030178e-05, 
       1.8107541003231476e-05, 
       2.2816831798848336e-05, 
       2.1564857025852960e-05, 
       1.7121195645320671e-05, 
       2.3830216908688118e-05, 
       1.7640066694329988e-05, 
       2.3572348406129228e-05, 
       1.8211033238230675e-05, 
       1.8732999796455037e-05, 
       2.0047349746630555e-05, 
       2.2128156456494762e-05, 
       2.1043440060757221e-05, 
       2.2995362270055442e-05, 
       1.9991878851623358e-05, 
       2.3862632834583936e-05, 
       2.4067298190273253e-05, 
       1.7626718557277661e-05, 
       1.7277189678332198e-05, 
       1.9995528903169129e-05, 
       2.0178856238418034e-05, 
       1.8663548651772293e-05, 
       1.7844257239569768e-05, 
       1.7708938371280653e-05, 
       1.7708938371280653e-05, 
       2.1100876216390546e-05, 
       1.8936351533887953e-05, 
       2.1909249848044001e-05, 
       2.1909249848044001e-05, 
       2.0681238638649732e-05, 
       2.0235006211901255e-05, 
       2.3771931379770966e-05, 
       1.7271821128373024e-05, 
       2.0887875915477799e-05, 
       1.8265922996359798e-05, 
       2.1330995819079070e-05, 
       1.8766059560146823e-05, 
       1.7264445253428378e-05, 
       1.7264445253428378e-05, 
       1.9940401050376650e-05, 
       2.3842389296058815e-05, 
       2.2790918071610945e-05, 
       2.0517006919138890e-05, 
       1.9187496467440822e-05, 
       2.0960358783439589e-05, 
       2.0640630344123610e-05, 
       2.0945678455290999e-05, 
       2.2745010809265887e-05, 
       1.6960407224583200e-05, 
       2.0167093769747340e-05, 
       1.7807585815741925e-05, 
       1.7102038575890068e-05, 
       1.8392257888280286e-05, 
       2.3834263613959718e-05, 
       2.1012673084139515e-05, 
       2.3522038035291879e-05, 
       1.9404443666463979e-05, 
       1.7586315175081062e-05, 
       1.6835823803580511e-05, 
       1.7156411319383199e-05, 
       1.7479589437298256e-05, 
       2.3597271640262422e-05, 
       1.7287133384322985e-05, 
       1.8479863436802956e-05, 
       1.7286762049032248e-05, 
       1.7416590758136627e-05, 
       1.6898364375521922e-05, 
       1.7692832868685998e-05, 
       1.7692832868685998e-05, 
       2.3720659507815452e-05, 
       1.8772310497314735e-05, 
       2.0000000000004633e-05, 
       1.7132608549578928e-05, 
       1.6842701071753262e-05, 
       1.8821576107333434e-05, 
       1.9535192737125097e-05, 
       2.0123839854511366e-05, 
       1.8991720201829508e-05, 
       2.2838295057065624e-05, 
       2.0350669413703838e-05, 
       2.0445322535878866e-05, 
       2.0445322535878866e-05, 
       2.0188604558457556e-05, 
       1.6907632508101273e-05, 
       1.9252757586184596e-05, 
       1.8894781575960861e-05, 
       1.7888036875733064e-05, 
       2.2662977395270788e-05, 
       1.8053450720011652e-05, 
       1.6881160349336304e-05, 
       1.6675781899278904e-05, 
       2.3477252566993164e-05, 
       1.6671791014294461e-05, 
       1.6777623147983242e-05, 
       1.8301536416555231e-05, 
       1.7688458205896298e-05, 
       1.8115006792659946e-05, 
       2.3115301780885825e-05, 
       1.9547969899974727e-05, 
       2.1232942097686476e-05, 
       1.7603971171241998e-05, 
       1.8544627601127559e-05, 
       2.3122379100758194e-05, 
       2.0258173778535597e-05, 
       1.6669912361551040e-05, 
       2.0115386471181266e-05, 
       1.6669237843083501e-05, 
       2.2192905489048603e-05, 
       1.9523128327740499e-05, 
       2.2117423760742725e-05, 
       2.1246485395735331e-05, 
       1.6902877538767291e-05, 
       2.0948203655519942e-05, 
       2.3293130831803150e-05, 
       1.6755912441012491e-05, 
       1.7382368362886235e-05, 
       1.9296487651582421e-05, 
       2.3282029915945936e-05, 
       1.9590558508559368e-05, 
       1.7433881448371084e-05, 
       2.3066800207303732e-05, 
       1.6522532115456080e-05, 
       1.6519869720739026e-05, 
       1.6523350943213977e-05, 
       2.1106453553785930e-05, 
       2.0855518262297264e-05, 
       2.1612473413096596e-05, 
       1.7934231852549327e-05, 
       1.9321267329180687e-05, 
       1.6541097702661042e-05, 
       2.2869787571877537e-05, 
       2.3154810789682062e-05, 
       2.2973975708738382e-05, 
       2.2961707807479175e-05, 
       2.1241731584823695e-05, 
       1.7001513226710508e-05, 
       1.6928372631101334e-05, 
       2.2156590296457815e-05, 
       1.9530984570005048e-05, 
       1.6442805337604780e-05, 
       1.9276688387829358e-05, 
       2.2342760509353491e-05, 
       1.6763425422484552e-05, 
       2.1488128728838385e-05, 
       2.3114813929650088e-05, 
       1.8781375513748516e-05, 
       1.8946871324177870e-05, 
       1.8808814873676904e-05, 
       2.1605403140124169e-05, 
       2.3102567029396645e-05, 
       2.0605355541645451e-05, 
       1.7213748976173022e-05, 
       1.7793395274541191e-05, 
       1.6371128677377743e-05, 
       1.7077926575575495e-05, 
       1.6391641230189141e-05, 
       1.6802612342596238e-05, 
       1.6963230126010973e-05, 
       2.2047351993580104e-05, 
       1.8918710499546012e-05, 
       1.7591270267590143e-05, 
       2.2311092638659019e-05, 
       1.6387351086933984e-05, 
       1.9524251927299393e-05, 
       1.8881642834623986e-05, 
       1.9077429636800265e-05, 
       1.8917688474233060e-05, 
       2.1804552128331417e-05, 
       2.2656255284805286e-05, 
       1.9943006491561431e-05, 
       1.7469282681897057e-05, 
       1.6974804183679965e-05, 
       2.2740256596326434e-05, 
       1.7207528739574835e-05, 
       1.6970042819011529e-05, 
       1.9850897826104042e-05, 
       1.9140106729782767e-05, 
       1.6676344305763090e-05, 
       1.9090537627228603e-05, 
       1.6736497529196532e-05, 
       2.0133000437335605e-05, 
       2.2757006468817194e-05, 
       1.6702664019797677e-05, 
       1.6315397773966979e-05, 
       2.0790900210589302e-05, 
       1.8524588975115479e-05, 
       1.6451802847666920e-05, 
       1.7365645610299736e-05, 
       2.2855177578230783e-05, 
       2.2819858675157721e-05, 
       2.0904725780274183e-05, 
       2.1288427944038591e-05, 
       1.6192000254433390e-05, 
       1.6245299440748172e-05, 
       1.6245299440748172e-05, 
       1.8127242507720723e-05, 
       2.2275779288788813e-05, 
       2.0773727929471758e-05, 
       2.2829793251445898e-05, 
       1.8921584142257576e-05, 
       1.6339652113304440e-05, 
       1.8975843734927459e-05, 
       2.2688254878616019e-05, 
       1.8905393105560276e-05, 
       1.7022317425458386e-05, 
       2.2019462623333437e-05, 
       1.6246538359845686e-05, 
       1.6216986312379648e-05, 
       1.6121927420355377e-05, 
       2.1956054374316435e-05, 
       1.6562279413404521e-05, 
       1.6298213345503501e-05, 
       1.9060170845724578e-05, 
       1.8922364839388372e-05, 
       1.7061149537075686e-05, 
       1.7094956165661685e-05, 
       2.0821138244668801e-05, 
       2.0174318884068203e-05, 
       1.9140445654777217e-05, 
       1.6081710891039304e-05, 
       1.6227456944695615e-05, 
       1.7666140914068323e-05, 
       1.6567806372912873e-05, 
       1.7453416883691020e-05, 
       2.2657645268488578e-05, 
       2.0431023949867525e-05, 
       1.6950145720708444e-05, 
       1.9890155960664404e-05, 
       1.6053402041731836e-05, 
       1.9403903446542360e-05, 
       1.6110160628986708e-05, 
       2.2612138573918036e-05, 
       1.6667605616919987e-05, 
       1.6644933137546988e-05, 
       1.5980449473580598e-05, 
       1.6163551770792984e-05, 
       1.6617055878780183e-05, 
       2.2562695973202133e-05, 
       1.9886332432387718e-05, 
       2.1722027768969165e-05, 
       1.9910140933530282e-05, 
       1.9528441425711874e-05, 
       1.7245527634880786e-05, 
       1.8161074703236701e-05, 
       1.8017076822875017e-05, 
       1.7015623552843490e-05, 
       2.2501134802554360e-05, 
       1.8220193544218455e-05, 
       1.9510271297828276e-05, 
       1.6147164518456417e-05, 
       1.8771757892139438e-05, 
       1.5990302660566169e-05, 
       1.6012919130152530e-05, 
       1.6549792915255268e-05, 
       1.8945554267631933e-05, 
       1.6200727414548244e-05, 
       1.6284763419791300e-05, 
       1.6131260434911045e-05, 
       1.7709260348775388e-05, 
       2.1710321179262857e-05, 
       1.5951107002733541e-05, 
       1.6265113750427505e-05, 
       1.7234577675678331e-05, 
       2.2331401934126915e-05, 
       2.2371301127586665e-05, 
       2.2217390650471838e-05, 
       2.0093188338363381e-05, 
       2.2092602864857998e-05, 
       2.2346528997350687e-05, 
       1.6100076294670998e-05, 
       1.5942587442484113e-05, 
       1.6194102205535764e-05, 
       1.5898226785692709e-05, 
       2.1155515311702435e-05, 
       1.6583252908457672e-05, 
       1.7278340779413192e-05, 
       1.8470997967287768e-05, 
       2.1856606817620013e-05, 
       1.5745667999998268e-05, 
       2.2222483777966504e-05, 
       1.7492088499757034e-05, 
       1.6560501187162072e-05, 
       1.5726195086615124e-05, 
       1.7510390518953370e-05, 
       1.7845196780493277e-05, 
       1.6334757830895036e-05, 
       1.6334757830895036e-05, 
       1.6076950486640756e-05, 
       1.6829246649323837e-05, 
       1.6315128457849984e-05, 
       1.9051886859255621e-05, 
       1.5728524163541113e-05, 
       1.7443462232679977e-05, 
       2.2092696552803133e-05, 
       1.8469657222836685e-05, 
       1.5691417502302559e-05, 
       1.9453709029190264e-05, 
       1.6511741573958431e-05, 
       1.7645208764304032e-05, 
       2.1595499629582765e-05, 
       1.8377882355999809e-05, 
       1.6561248364934335e-05, 
       1.7984672575260438e-05, 
       1.8549200797180666e-05, 
       2.2087304554959113e-05, 
       2.1828425648732798e-05, 
       2.0391063478211066e-05, 
       2.1279859781010948e-05, 
       2.1508616375209985e-05, 
       1.6660970869587527e-05, 
       1.8588780727374235e-05, 
       1.6565400381325721e-05, 
       1.5717566299634416e-05, 
       2.0535965288849930e-05, 
       1.6466581956504004e-05, 
       1.8647137458050032e-05, 
       1.5610352888002224e-05, 
       1.6179081609254638e-05, 
       2.1794906242845501e-05, 
       1.5577931509606555e-05, 
       1.9586511985099117e-05, 
       2.1909841500100040e-05, 
       2.1998579505542611e-05, 
       2.1493479302175947e-05, 
       1.5559493904853652e-05, 
       1.5941066616909995e-05, 
       1.7676337579382255e-05, 
       1.8062776040369259e-05, 
       1.6659653881123988e-05, 
       2.0000000000003393e-05, 
       1.7849568206700026e-05, 
       1.5965574229352592e-05, 
       1.6437317940038550e-05, 
       2.0765863952647323e-05, 
       2.1846633864365964e-05, 
       1.8424062803935875e-05, 
       1.7064953581099732e-05, 
       2.1263222644974115e-05, 
       2.1825539867019784e-05, 
       2.0186503194792118e-05, 
       1.6318294159138578e-05, 
       1.6565632659979917e-05, 
       2.1854141651488121e-05, 
       1.9721867338203134e-05, 
       1.6591714642297451e-05, 
       1.6591714642297451e-05, 
       1.5516381802169052e-05, 
       1.8755561655044175e-05, 
       1.5955873784711584e-05, 
       1.5774574480693046e-05, 
       2.1735051759274436e-05, 
       1.6066594092445572e-05, 
       1.5846906777686247e-05, 
       1.8868337038229099e-05, 
       1.6265232406221913e-05, 
       1.7782797412512916e-05, 
       1.7809191450193069e-05, 
       1.5599869105108737e-05, 
       2.1693795445600207e-05, 
       1.8985056752555413e-05, 
       1.8113328581290859e-05, 
       1.6021103607237798e-05, 
       2.1684010188073883e-05, 
       1.9957107652443819e-05, 
       1.9469171199475047e-05, 
       1.9886956122973244e-05, 
       1.6479758202524700e-05, 
       2.1385307632514737e-05, 
       1.6649743338875126e-05, 
       1.9999999999999219e-05, 
       1.9999999999999219e-05, 
       1.9999999999997972e-05, 
       1.9999999999999134e-05, 
       1.9999999999998982e-05, 
       2.0000000000001706e-05, 
       1.9999999999998365e-05, 
       2.0000000000000425e-05, 
       1.9999999999995062e-05, 
       1.9999999999998385e-05, 
       1.9999999999998385e-05, 
       1.5977290848338997e-05, 
       1.5977290848338997e-05, 
       1.5420271501855298e-05, 
       2.0863754748184489e-05, 
       1.6308730864178650e-05, 
       2.1544639578979746e-05, 
       1.7659009667905495e-05, 
       1.7739548753552605e-05, 
       1.5878542845562615e-05, 
       1.5877807013638059e-05, 
       1.8395025156768364e-05, 
       1.9810470929856084e-05, 
       1.9807995984286075e-05, 
       2.1597618017637537e-05, 
       2.1380717362667158e-05, 
       1.5728529446399404e-05, 
       1.5799744767817560e-05, 
       1.7679947029514624e-05, 
       1.7877447508622171e-05, 
       1.7801104938934124e-05, 
       1.7961378393018670e-05, 
       1.5299759945115771e-05, 
       2.1370152468451239e-05, 
       1.6510340956330097e-05, 
       1.5486302119327786e-05, 
       1.5455725955911955e-05, 
       2.0135264182178537e-05, 
       1.6853292001640325e-05, 
       1.6832442225826654e-05, 
       1.6967090687246489e-05, 
       1.9071013973279498e-05, 
       2.1538167031908605e-05, 
       1.5548412391258848e-05, 
       2.1237395492952342e-05, 
       1.8164508774457600e-05, 
       1.7861361836655403e-05, 
       1.8624999829026083e-05, 
       1.7033020620889857e-05, 
       1.6765228345600891e-05, 
       1.7878966106612229e-05, 
       1.6311695542618120e-05, 
       1.7879737954012614e-05, 
       1.6721225102334025e-05, 
       1.7874945417602016e-05, 
       1.6812566985582902e-05, 
       1.7252375527424996e-05, 
       1.7680886806664228e-05, 
       1.6806036523154016e-05, 
       1.6252260579936975e-05, 
       1.6122600836113829e-05, 
       1.6667517199983966e-05, 
       1.6193045281960557e-05, 
       1.5398488477955585e-05, 
       2.0406259702848420e-05, 
       1.5590941002720459e-05, 
       1.6329699181494415e-05, 
       1.5328559236814314e-05, 
       1.7370318758741554e-05, 
       1.8812901235652901e-05, 
       1.8812901235652901e-05, 
       1.7369065115322222e-05, 
       1.5591093902455968e-05, 
       2.0656984802022977e-05, 
       1.9591177295441521e-05, 
       1.6687695692916484e-05, 
       2.1130099301282708e-05, 
       2.1325073661143272e-05, 
       2.1157688524304195e-05, 
       1.7461179860679065e-05, 
       1.6423108276909969e-05, 
       1.8245319251193914e-05, 
       2.1025867185539587e-05, 
       1.8019242603315165e-05, 
       2.0248387147714210e-05, 
       1.8157936344016041e-05, 
       1.6389757351005462e-05, 
       1.5100347492838665e-05, 
       1.5100347492838665e-05, 
       1.5230262986091805e-05, 
       2.1215582948340956e-05, 
       1.5414961617522229e-05, 
       1.7595936685111189e-05, 
       2.0616962092432869e-05, 
       1.5432323397707398e-05, 
       1.8048638426624819e-05, 
       1.5363941806358455e-05, 
       1.9283751778808183e-05, 
       1.7100546614081047e-05, 
       2.0102533641941161e-05, 
       1.7384638123306500e-05, 
       1.9906283196170509e-05, 
       1.8814199491772168e-05, 
       1.5013082294691551e-05, 
       1.5580516979981153e-05, 
       1.5580516979981153e-05, 
       1.8620212778572103e-05, 
       1.5264870491652246e-05, 
       2.0135536757306594e-05, 
       1.6850842558128003e-05, 
       1.5208548688269298e-05, 
       1.7459444804946951e-05, 
       1.8797311920859079e-05, 
       1.5559069052397929e-05, 
       1.5088699598054669e-05, 
       2.1057493730659698e-05, 
       1.6434958530078212e-05, 
       1.7030566798391419e-05, 
       1.5067069593693518e-05, 
       1.6693029274941945e-05, 
       1.8596654607507702e-05, 
       1.5775316812798327e-05, 
       1.5193852193121605e-05, 
       2.0579104068435114e-05, 
       1.6419791920995258e-05, 
       2.1022166679239770e-05, 
       1.5405578998752347e-05, 
       1.5137724906236635e-05, 
       1.5173460081174190e-05, 
       1.4911920522673188e-05, 
       1.5526408715852300e-05, 
       1.6954508804696774e-05, 
       2.1003900928651508e-05, 
       1.5286404993118030e-05, 
       1.9922310076878961e-05, 
       1.4896210933481640e-05, 
       2.1031371405014781e-05, 
       1.5952865525235097e-05, 
       1.6544617095966181e-05, 
       1.7218331604476612e-05, 
       1.7373531888541584e-05, 
       2.0496606911620923e-05, 
       1.4852990397279256e-05, 
       1.5569865199886903e-05, 
       1.8516799849218081e-05, 
       1.4828457252575804e-05, 
       1.5624891989336155e-05, 
       1.5624891989336155e-05, 
       1.4820220294325903e-05, 
       1.8874653645544506e-05, 
       1.6038903768216540e-05, 
       1.7290995979226839e-05, 
       1.9999999999995753e-05, 
       1.7273542428415941e-05, 
       1.4821832951205098e-05, 
       1.6753200237546766e-05, 
       1.6835026339759140e-05, 
       1.8997761949861205e-05, 
       1.9872279006599585e-05, 
       1.5838127256524234e-05, 
       1.5311895804839845e-05, 
       1.7246606887827225e-05, 
       1.5028273905079650e-05, 
       2.0898021927152357e-05, 
       1.4971825628919935e-05, 
       1.5166686490918444e-05, 
       1.7347097119550183e-05, 
       2.0385052342525400e-05, 
       2.0385052342525400e-05, 
       1.7226647172434394e-05, 
       1.8707304316293873e-05, 
       1.4971493848252444e-05, 
       1.7331983462908899e-05, 
       2.0077240491880317e-05, 
       1.5553746090871645e-05, 
       1.9088499511825495e-05, 
       1.6026474807185141e-05, 
       1.5611076867388777e-05, 
       1.7323177062971019e-05, 
       2.0808307448607207e-05, 
       1.9774199270579913e-05, 
       2.0802383711941766e-05, 
       1.4876178559690130e-05, 
       1.4781188707033816e-05, 
       1.4712464152710686e-05, 
       2.0772897788380036e-05, 
       1.5155486796189431e-05, 
       1.5268673215336786e-05, 
       1.9041790271914958e-05, 
       1.9111524188071107e-05, 
       1.5564865986683139e-05, 
       1.7594038744631663e-05, 
       1.7892962818902775e-05, 
       1.7319090635615784e-05, 
       1.8674395040506046e-05, 
       1.6240252637876185e-05, 
       1.7032907904837461e-05, 
       1.6050867624039417e-05, 
       1.5175963048911486e-05, 
       1.4792337855245117e-05, 
       1.4652905497411430e-05, 
       2.0680988167426885e-05, 
       2.0119294660253976e-05, 
       1.6095729797305823e-05, 
       2.0625093907111433e-05, 
       1.9521098756728870e-05, 
       2.0595661400170565e-05, 
       1.7163151624926476e-05, 
       1.7586841823329863e-05, 
       1.4890445304719087e-05, 
       1.8117403341027752e-05, 
       2.0416214909593494e-05, 
       1.5452353127622199e-05, 
       1.5054024981533608e-05, 
       1.5207350771354807e-05, 
       1.4615593876244564e-05, 
       1.4720638515039225e-05, 
       1.7128828706261080e-05, 
       1.8277759505547590e-05, 
       1.4727151546726160e-05, 
       1.9774829801442692e-05, 
       1.4637347381722367e-05, 
       1.6897349439657455e-05, 
       2.0028722685522382e-05, 
       1.8131287278910380e-05, 
       1.7375893020347383e-05, 
       1.7375893020347383e-05, 
       1.4604133161637290e-05, 
       1.5314749164619713e-05, 
       1.6465255960680018e-05, 
       1.8067845275684433e-05, 
       1.7609693232163050e-05, 
       1.8299191964204232e-05, 
       1.8227708368623237e-05, 
       1.5186755906245460e-05, 
       2.0142451525239328e-05, 
       1.5337914335951559e-05, 
       1.7595208210728162e-05, 
       1.5863110941040158e-05, 
       1.8393660809236776e-05, 
       1.4969141844527169e-05, 
       1.5035586495924964e-05, 
       1.5363517569489903e-05, 
       1.4854304650086469e-05, 
       1.5623459703162102e-05, 
       1.4767998889265674e-05, 
       1.7159067231406097e-05, 
       1.4539948710924227e-05, 
       1.7429313642443375e-05, 
       1.5031823833576059e-05, 
       1.7539807742658661e-05, 
       1.4980268277844546e-05, 
       1.5011269893965837e-05, 
       1.5248557503425525e-05, 
       1.8580287333637601e-05, 
       1.5043552299635457e-05, 
       1.8168963708791143e-05, 
       1.8651388613578185e-05, 
       2.0189993095340191e-05, 
       1.5432236115146692e-05, 
       1.5593621552787403e-05, 
       1.4474509998140308e-05, 
       1.5785590202942033e-05, 
       1.9611071407936163e-05, 
       1.6527176552352358e-05, 
       1.6232588600915820e-05, 
       1.4681213271934059e-05, 
       1.6896558968491199e-05, 
       1.6094049781341862e-05, 
       1.8552136823778786e-05, 
       1.7183534692853691e-05, 
       1.6927102978064870e-05, 
       1.9911752099446130e-05, 
       1.5292577163550201e-05, 
       2.0327942947741167e-05, 
       2.0202000369750319e-05, 
       1.4554175151559585e-05, 
       1.4386665081500624e-05, 
       1.6895948089447729e-05, 
       1.4757355285903934e-05, 
       1.4733338471278170e-05, 
       1.7958138239126570e-05, 
       1.9119896815625364e-05, 
       1.5176898949600512e-05, 
       1.4539556056417575e-05, 
       1.9999999999997976e-05, 
       1.7705303703023824e-05, 
       1.6796812012481824e-05, 
       2.0219456420751133e-05, 
       1.9461929824423670e-05, 
       1.4914971941135734e-05, 
       1.4725464117338971e-05, 
       1.4905502791489821e-05, 
       1.6704245104031172e-05, 
       1.6346849763034155e-05, 
       2.0183850467384718e-05, 
       1.7328649070163523e-05, 
       1.9993058534651745e-05, 
       2.0155206632469547e-05, 
       1.7158161840755303e-05, 
       1.6180506087687834e-05, 
       1.5044975232053725e-05, 
       1.9707045736827601e-05, 
       1.8274161459051690e-05, 
       1.7221161176813659e-05, 
       1.4421755533291740e-05, 
       1.6646253155362778e-05, 
       1.4334748803919929e-05, 
       2.0121364742274587e-05, 
       1.4291639566070391e-05, 
       1.5670943520524227e-05, 
       1.6482971711744494e-05, 
       1.4576547625996510e-05, 
       1.4655517265658455e-05, 
       1.7757433416121555e-05, 
       1.7970663476776966e-05, 
       1.5742856862648214e-05, 
       1.7018774590597435e-05, 
       1.8645251871130552e-05, 
       1.8645185290879596e-05, 
       1.7335664817918533e-05, 
       1.4818482970319865e-05, 
       1.6811733070769529e-05, 
       1.4250193466941593e-05, 
       1.6093868620042199e-05, 
       1.9326975582424918e-05, 
       1.9941949295786189e-05, 
       2.0052822795065137e-05, 
       1.4459181220275343e-05, 
       1.9728304593322032e-05, 
       1.7638712898908959e-05, 
       2.0072723215681190e-05, 
       1.5006211187950023e-05, 
       1.5717891767448431e-05, 
       1.7127581819786330e-05, 
       1.5713156326138730e-05, 
       1.8781582483123569e-05, 
       1.4276969090582839e-05, 
       2.0000084035178723e-05, 
       1.7159237423769609e-05, 
       1.8712812857215246e-05, 
       1.6555061551292769e-05, 
       1.4879353459598888e-05, 
       1.7156299998227326e-05, 
       1.9999999999995990e-05, 
       1.9999999999995512e-05, 
       2.0000000000001137e-05, 
       2.0000000000004206e-05, 
       1.9999999999998375e-05, 
       1.9999999999998037e-05, 
       2.0000000000003796e-05, 
       2.0000000000003928e-05, 
       1.9999999999996492e-05, 
       1.9999999999999910e-05, 
       1.9999999999998931e-05, 
       1.9999999999998562e-05, 
       2.0000000000003054e-05, 
       2.0000000000002682e-05, 
       2.0000000000004742e-05, 
       1.9999999999995546e-05, 
       1.9999999999996146e-05, 
       1.9999999999997400e-05, 
       1.9999999999998955e-05, 
       1.9999999999998511e-05, 
       1.9999999999998596e-05, 
       1.9999999999998596e-05, 
       2.0000000000003295e-05, 
       2.0000000000003474e-05, 
       2.0000000000003305e-05, 
       2.0000000000003305e-05, 
       1.9999999999995577e-05, 
       2.0000000000002624e-05, 
       2.0000000000002624e-05, 
       1.9999999999997010e-05, 
       1.9999999999999029e-05, 
       2.0000000000004735e-05, 
       1.9999999999995668e-05, 
       2.0000000000003241e-05, 
       2.0000000000002739e-05, 
       2.0000000000355461e-05, 
       1.9999999999999497e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.5939276814151124e-05, 
       1.6625936318879249e-05, 
       1.5877031373873162e-05, 
       1.8140395049239783e-05, 
       1.8308062360460845e-05, 
       1.5199292512977197e-05, 
       1.9165615722705478e-05, 
       1.5935874394150922e-05, 
       1.8691594224387670e-05, 
       1.7759550229843893e-05, 
       1.5086318238923548e-05, 
       1.6586312590480193e-05, 
       1.5159660928851428e-05, 
       1.9748344662031826e-05, 
       1.4184927099610776e-05, 
       1.6508969673611119e-05, 
       1.6842490537408845e-05, 
       1.4169264681528034e-05, 
       1.4973203993347896e-05, 
       1.6357064236119631e-05, 
       1.4081312733097830e-05, 
       1.9653155872593739e-05, 
       1.6576768086592933e-05, 
       1.4763383057877282e-05, 
       1.7895354644045685e-05, 
       1.7177379033345189e-05, 
       1.9835479307516483e-05, 
       1.9760450887237208e-05, 
       1.5922008662989584e-05, 
       1.4192356165573130e-05, 
       1.4446796603526607e-05, 
       1.9360542573648798e-05, 
       1.4862500633760479e-05, 
       1.4130615854198479e-05, 
       1.7993224758436213e-05, 
       1.4110303572950454e-05, 
       1.4439742680236640e-05, 
       1.8664932428318519e-05, 
       1.9830006478665723e-05, 
       1.4059554246394452e-05, 
       1.8361853591135091e-05, 
       1.8568941237139572e-05, 
       1.5159350661795558e-05, 
       1.4538116772865378e-05, 
       1.4619017599940080e-05, 
       1.4734848775359021e-05, 
       1.4530144801751874e-05, 
       1.4530144801751874e-05, 
       1.8711782765589747e-05, 
       1.7315393781889078e-05, 
       1.4582369958389614e-05, 
       1.4051410418387655e-05, 
       1.4802867434486541e-05, 
       1.3997770115497577e-05, 
       1.6294247591339046e-05, 
       1.9743431711978656e-05, 
       1.5424937527466854e-05, 
       1.4213582477691444e-05, 
       1.4000502399119676e-05, 
       1.4364465923070075e-05, 
       1.5817637901761373e-05, 
       1.6439319421041781e-05, 
       1.6520754002690123e-05, 
       1.4913314906272624e-05, 
       1.4338555938038893e-05, 
       1.4197002360549593e-05, 
       1.4599823824599674e-05, 
       1.7636462865049696e-05, 
       1.8994561772210017e-05, 
       1.6643106215529342e-05, 
       1.4583722683031408e-05, 
       1.9653099091101223e-05, 
       1.7121883203115021e-05, 
       1.9086288472752674e-05, 
       1.5116441838818137e-05, 
       1.9705578316040376e-05, 
       1.6234026576853952e-05, 
       1.9257310179903845e-05, 
       1.6231048319121017e-05, 
       1.5832105455099860e-05, 
       1.8576099081583036e-05, 
       1.5106001958022986e-05, 
       1.3906246436919196e-05, 
       1.4058462464141080e-05, 
       1.8493797485177163e-05, 
       1.3921528037739497e-05, 
       1.4382249695317635e-05, 
       1.4320293209520982e-05, 
       1.6343137405858013e-05, 
       1.9634988456481449e-05, 
       1.4069188474811291e-05, 
       1.3938111000651239e-05, 
       1.8599369508915002e-05, 
       1.4742257213966482e-05, 
       1.3900685838893411e-05, 
       1.4063705351466365e-05, 
       1.9207170807620005e-05, 
       1.6259790000197429e-05, 
       1.7613010324938945e-05, 
       1.6518564385013581e-05, 
       1.6518564385013581e-05, 
       1.8644505450716566e-05, 
       1.4702018963999849e-05, 
       1.7268308311332900e-05, 
       1.6146018171990213e-05, 
       1.7773427066869232e-05, 
       1.9196006637171878e-05, 
       1.9393106466433335e-05, 
       1.5496877413252688e-05, 
       1.9429654555499584e-05, 
       1.4379712814157296e-05, 
       1.6079721609574042e-05, 
       1.6464831290779503e-05, 
       1.5871668585008463e-05, 
       1.7015510992110149e-05, 
       1.5868517312155221e-05, 
       1.7644942363623315e-05, 
       1.5553784934446578e-05, 
       1.4332250703755212e-05, 
       1.3966361503711811e-05, 
       1.7871188241571116e-05, 
       1.3890484649062350e-05, 
       1.4097161510313847e-05, 
       1.9325691016493326e-05, 
       1.4163356986958101e-05, 
       1.3792167111711009e-05, 
       1.5718276067109254e-05, 
       1.3898593889712613e-05, 
       1.5336605592104323e-05, 
       1.4185338323611284e-05, 
       1.9302224824416489e-05, 
       1.9274896091972504e-05, 
       1.7607548689973156e-05, 
       1.9424470755392661e-05, 
       1.9407444552125230e-05, 
       1.9292331000324735e-05, 
       1.6214171167468407e-05, 
       1.5749766451329129e-05, 
       1.3821746312906816e-05, 
       1.4111874837520725e-05, 
       1.9423182861496826e-05, 
       1.4716813008698554e-05, 
       1.5726195086620572e-05, 
       1.3875264504861185e-05, 
       1.6572601239953122e-05, 
       1.3975000138134530e-05, 
       1.7546829945059612e-05, 
       1.5988943726112353e-05, 
       1.6127142760437654e-05, 
       1.8501286555225882e-05, 
       1.9369100314506194e-05, 
       1.3749988119975567e-05, 
       1.5195083146998539e-05, 
       1.4806830773544685e-05, 
       1.3718401689414570e-05, 
       1.3819554292886459e-05, 
       1.3819554292886459e-05, 
       1.4226198016015102e-05, 
       1.4526254768102927e-05, 
       1.4130402919915216e-05, 
       1.5924384881513964e-05, 
       1.3776920622358407e-05, 
       1.7307937339705117e-05, 
       1.7157118751837243e-05, 
       1.3791515431416588e-05, 
       1.3791169411630304e-05, 
       1.7420417376566657e-05, 
       1.7409352824449136e-05, 
       1.4285093339852193e-05, 
       1.8795587682042484e-05, 
       1.4103654794039135e-05, 
       1.9180107068938916e-05, 
       1.8503274707320360e-05, 
       1.4703893042475059e-05, 
       1.5685763325220564e-05, 
       1.4364479686318036e-05, 
       1.8745947523462314e-05, 
       1.4459917608715566e-05, 
       1.4031358457578322e-05, 
       1.6495701150819696e-05, 
       1.4454621902324786e-05, 
       1.3656166987330204e-05, 
       1.6179948708548639e-05, 
       1.3742596961253355e-05, 
       1.3790596327845639e-05, 
       1.4183318934991936e-05, 
       1.7599213631918401e-05, 
       1.9246636067142738e-05, 
       1.4545046767621585e-05, 
       1.9199568698335498e-05, 
       1.5482489214826254e-05, 
       1.6633405599664425e-05, 
       1.3890861298346911e-05, 
       1.9204840480165535e-05, 
       1.3650788074327285e-05, 
       1.9057870021014847e-05, 
       1.5700088389794672e-05, 
       1.3797255190944416e-05, 
       1.3612537214898020e-05, 
       1.8992765122828241e-05, 
       1.5907063252288016e-05, 
       1.7563224233352980e-05, 
       1.4295373293358697e-05, 
       1.8686881908932990e-05, 
       1.7698050950329329e-05, 
       1.6638831034697928e-05, 
       1.5391637414589236e-05, 
       1.3567521679662418e-05, 
       1.4775426800283805e-05, 
       1.4713707950625158e-05, 
       1.5881304267426527e-05, 
       1.4811723828636502e-05, 
       1.5888800551981285e-05, 
       1.4445333559047477e-05, 
       1.7627097551466300e-05, 
       1.6392602211556929e-05, 
       1.8901538121242530e-05, 
       1.4125446770525143e-05, 
       1.3985992402754039e-05, 
       1.9026115327224947e-05, 
       1.7543671232633482e-05, 
       1.3624994770428607e-05, 
       1.4147596407706047e-05, 
       1.4188435164862451e-05, 
       1.8766800406333077e-05, 
       1.8501831042218248e-05, 
       1.5682141141589220e-05, 
       1.4176620742262519e-05, 
       1.5587248848180789e-05, 
       1.5587248848180789e-05, 
       1.9027498924865004e-05, 
       1.8471267669323802e-05, 
       1.3940539721045514e-05, 
       1.4708561463009807e-05, 
       1.5057309310234542e-05, 
       1.4320397625204029e-05, 
       1.4182636794792536e-05, 
       1.3841723775407177e-05, 
       1.8686062663425051e-05, 
       1.8957179279687248e-05, 
       1.3434160286195692e-05, 
       1.8295928886823704e-05, 
       1.5964674802579159e-05, 
       1.4646932496401925e-05, 
       1.3723941450162334e-05, 
       1.3719201260618938e-05, 
       1.4536406557225851e-05, 
       1.4161199991469600e-05, 
       1.6734960448185846e-05, 
       1.3459860884794278e-05, 
       1.7480723532045456e-05, 
       1.6008297559529445e-05, 
       1.3889197324595794e-05, 
       1.5704066095512283e-05, 
       1.5223195330055947e-05, 
       1.3646413175679166e-05, 
       1.8307490823398830e-05, 
       1.8779267767158738e-05, 
       1.6295973950219000e-05, 
       1.7374349660920962e-05, 
       1.7336397653638667e-05, 
       1.5517788553327095e-05, 
       1.3949937650194191e-05, 
       1.5558319320347971e-05, 
       1.3477655223454708e-05, 
       1.5502386177901978e-05, 
       1.5437012993269314e-05, 
       1.7031093811409717e-05, 
       1.8590234832537693e-05, 
       1.5658159152181815e-05, 
       1.5016023147869611e-05, 
       1.5600915570675881e-05, 
       1.7891282073988843e-05, 
       1.8025899658835437e-05, 
       1.7423977222875363e-05, 
       1.4564904067818549e-05, 
       1.6985230109100869e-05, 
       1.6610169242526198e-05, 
       1.3403574850167767e-05, 
       1.8249590643225570e-05, 
       1.5324571989758253e-05, 
       1.8696725726503051e-05, 
       1.3299975408258749e-05, 
       1.3856856279333476e-05, 
       1.6103098521957558e-05, 
       1.4538139114707123e-05, 
       1.3702158804425321e-05, 
       1.3293422518546922e-05, 
       1.3293422518546922e-05, 
       1.6236107818336177e-05, 
       1.8688175919733982e-05, 
       1.3697525267876854e-05, 
       1.3807723910051766e-05, 
       1.3549227031101011e-05, 
       1.5578267437098484e-05, 
       1.3738672790371903e-05, 
       1.3392045731643397e-05, 
       1.3826092916486223e-05, 
       1.4373870531358810e-05, 
       1.5728116278106659e-05, 
       1.3268800564976914e-05, 
       1.5699211935313036e-05, 
       1.5005045045111907e-05, 
       1.3641239870295874e-05, 
       1.5968800531482134e-05, 
       1.8526216652598709e-05, 
       1.5313220473191225e-05, 
       1.8590595920302667e-05, 
       1.5613121484848878e-05, 
       1.5184033436742415e-05, 
       1.5109188015506222e-05, 
       1.6541342499016076e-05, 
       1.8342411660323198e-05, 
       1.3710210614290806e-05, 
       1.3592628967242167e-05, 
       1.4679805660731030e-05, 
       1.4607901020552364e-05, 
       1.5482167551382916e-05, 
       1.3604362088514964e-05, 
       1.8274378611735281e-05, 
       1.5927268050048197e-05, 
       1.8635562830561640e-05, 
       1.3880921072938089e-05, 
       1.3674217494541382e-05, 
       1.3300799926292706e-05, 
       1.3221374349235393e-05, 
       1.3715731222727339e-05, 
       1.3968892429642264e-05, 
       1.6234740689667079e-05, 
       1.3521752285369215e-05, 
       1.3181599317919874e-05, 
       1.3320017990740753e-05, 
       1.5122535180108063e-05, 
       1.3144870293603119e-05, 
       1.5086394562496400e-05, 
       1.3661037390654270e-05, 
       1.4444117437753411e-05, 
       1.3140819380544898e-05, 
       1.3366097190993894e-05, 
       1.4900164566905099e-05, 
       1.3427494996439749e-05, 
       1.3376517566833076e-05, 
       1.4152889034460591e-05, 
       1.3756462610644304e-05, 
       1.6297301150818062e-05, 
       1.7438216409263375e-05, 
       1.5711475861301436e-05, 
       1.6387592570966102e-05, 
       1.6046470233233955e-05, 
       1.3839515795230890e-05, 
       1.5442873685000349e-05, 
       1.3296253922502915e-05, 
       1.3195862494657522e-05, 
       1.8484162962177201e-05, 
       1.3623021781198347e-05, 
       1.5402607812110785e-05, 
       1.4788063864603584e-05, 
       1.5343935846969094e-05, 
       1.5800692881391843e-05, 
       1.3507008808491725e-05, 
       1.3579031645493009e-05, 
       1.4513522432581221e-05, 
       1.4779965753132000e-05, 
       1.3487598579836920e-05, 
       1.5429379125266887e-05, 
       1.8255826676610677e-05, 
       1.3240974759158147e-05, 
       1.5100633284082102e-05, 
       1.6541643306030848e-05, 
       1.5556387560409246e-05, 
       1.4738715945058147e-05, 
       1.3071202965307305e-05, 
       1.3304028771584416e-05, 
       1.8094333222118067e-05, 
       1.8094333222118067e-05, 
       1.3893211365789147e-05, 
       1.7898810072099596e-05, 
       1.5005171424926672e-05, 
       1.6611643061007290e-05, 
       1.3505242807491324e-05, 
       1.3741418404899087e-05, 
       1.3068773442443617e-05, 
       1.7293629641634060e-05, 
       1.3238875556645516e-05, 
       1.7695620618533069e-05, 
       1.5800871822403320e-05, 
       1.7450532617276401e-05, 
       1.7629295688044348e-05, 
       1.8344278480222551e-05, 
       1.3482464545722240e-05, 
       1.4175949303994331e-05, 
       1.5300242775821022e-05, 
       1.4605786763793261e-05, 
       1.4823175802028637e-05, 
       1.5153074284780484e-05, 
       1.7270806194812235e-05, 
       1.6080133306108396e-05, 
       1.4297887788280208e-05, 
       1.3342300958242572e-05, 
       1.4723920391526192e-05, 
       1.3480818098463213e-05, 
       1.4302739786959192e-05, 
       1.4390758875441822e-05, 
       1.3492151541326705e-05, 
       1.3177085229433623e-05, 
       1.2967378536239500e-05, 
       1.7702284305701121e-05, 
       1.5067600327411624e-05, 
       1.3387627760160553e-05, 
       1.3319111531154694e-05, 
       1.5197873533285913e-05, 
       1.8267001590097889e-05, 
       1.7569159342544463e-05, 
       1.5749766451335516e-05, 
       1.4349329423887490e-05, 
       1.3089622520666603e-05, 
       1.8214848141663575e-05, 
       1.6272081542993111e-05, 
       1.5582308902415184e-05, 
       1.3442712314267953e-05, 
       1.3091425520619221e-05, 
       1.3686534979773073e-05, 
       1.7339594937549733e-05, 
       1.6489623618874409e-05, 
       1.2880757335664411e-05, 
       1.3347189867640593e-05, 
       1.4050696450398433e-05, 
       1.4884455625497334e-05, 
       1.6626873230913024e-05, 
       1.5823735076546908e-05, 
       1.3000147393809456e-05, 
       1.5104512912761696e-05, 
       1.4101850482913153e-05, 
       1.5387481839614083e-05, 
       1.2918909218040337e-05, 
       1.3874800678788756e-05, 
       1.3131491965021635e-05, 
       1.2995579399911916e-05, 
       1.5104833365039862e-05, 
       1.5507514274429569e-05, 
       1.3364894728145192e-05, 
       1.4087188449380208e-05, 
       1.3597640967494783e-05, 
       1.3403023880438764e-05, 
       1.8118521262418251e-05, 
       1.6655942493056282e-05, 
       1.5490964261189846e-05, 
       1.6897403442486508e-05, 
       1.8079928873607369e-05, 
       1.5293086098448552e-05, 
       1.5137324166602677e-05, 
       1.2806963570890892e-05, 
       1.4584372774774720e-05, 
       1.4584323137432891e-05, 
       1.3336479988577629e-05, 
       1.8062443265322889e-05, 
       1.5911658893412627e-05, 
       1.4721689727392325e-05, 
       1.3228335561013241e-05, 
       1.6843793423740440e-05, 
       1.5016356582037923e-05, 
       1.7091135092454502e-05, 
       1.3151659632130112e-05, 
       1.7539273363188547e-05, 
       1.7557235930031233e-05, 
       1.3589754140539485e-05, 
       1.8025329576105984e-05, 
       1.4870995520110517e-05, 
       1.7870923326151566e-05, 
       1.5175791519919189e-05, 
       1.7594851726460161e-05, 
       1.3176151108582834e-05, 
       1.2880414496024289e-05, 
       1.3160649662418772e-05, 
       1.7167559616731058e-05, 
       1.2811074572457455e-05, 
       1.4955141120491437e-05, 
       1.2814387999203995e-05, 
       1.4899014168377159e-05, 
       1.4908026788281043e-05, 
       1.3850128959065888e-05, 
       1.2796271078368136e-05, 
       1.7686427184695396e-05, 
       1.2794038523273086e-05, 
       1.2735607351813803e-05, 
       1.5568858998512495e-05, 
       1.3554006583698968e-05, 
       1.2894425123708972e-05, 
       1.4480766156028099e-05, 
       1.3480737178010092e-05, 
       1.4473303739024755e-05, 
       1.5908892149985761e-05, 
       1.2745537453819999e-05, 
       1.7511545949478018e-05, 
       1.6997748985273308e-05, 
       1.2696258063591309e-05, 
       1.3553738252085991e-05, 
       1.3385560597741876e-05, 
       1.5417264670316735e-05, 
       1.2847939832424163e-05, 
       1.7618113086773479e-05, 
       1.2775045048953953e-05, 
       1.7280640997730996e-05, 
       1.4071209168825452e-05, 
       1.5934573699874957e-05, 
       1.4499961885680836e-05, 
       1.7420380509283671e-05, 
       1.7390979251685042e-05, 
       1.7896046931245846e-05, 
       1.5272640197153067e-05, 
       1.3143483145613326e-05, 
       1.6055450182000986e-05, 
       1.3029096990884418e-05, 
       1.4737733676853335e-05, 
       1.2853346621457406e-05, 
       1.6888833098419228e-05, 
       1.7266837862881601e-05, 
       1.4880176564453315e-05, 
       1.3303578245791464e-05, 
       1.4389006253772257e-05, 
       1.4581306955086769e-05, 
       1.3324878637255527e-05, 
       1.4586962968713603e-05, 
       1.5439188167706442e-05, 
       1.7628540369685631e-05, 
       1.2710953800351990e-05, 
       1.2722287998991867e-05, 
       1.3764484331250662e-05, 
       1.7754836156249789e-05, 
       1.4340533715801329e-05, 
       1.2709142458374608e-05, 
       1.7136753978475371e-05, 
       1.4771980148076379e-05, 
       1.7668925127910735e-05, 
       1.6873946144221132e-05, 
       1.7178883635342540e-05, 
       1.3540177879095211e-05, 
       1.3540177879095211e-05, 
       1.7726555941484860e-05, 
       1.7126093323873628e-05, 
       1.7391291245074722e-05, 
       1.6897490550629808e-05, 
       1.2626559322756644e-05, 
       1.7427788090370450e-05, 
       1.4313104514457679e-05, 
       1.6979634636265375e-05, 
       1.2540855306414584e-05, 
       1.7710141673424804e-05, 
       1.4490389970816197e-05, 
       1.3391202413501763e-05, 
       1.3391202413501763e-05, 
       1.3036983750738302e-05, 
       1.7698937267527312e-05, 
       1.3027780304667242e-05, 
       1.7060837186240233e-05, 
       1.3283256794748671e-05, 
       1.4307722125350473e-05, 
       1.7466787973707888e-05, 
       1.4854005793521301e-05, 
       1.5661389093368380e-05, 
       1.4684027875682477e-05, 
       1.3858576012762380e-05, 
       1.6340392098367192e-05, 
       1.3085781450145574e-05, 
       1.2502787160953376e-05, 
       1.3488634947007141e-05, 
       1.3767192242706289e-05, 
       1.2820878434160408e-05, 
       1.3798296019634512e-05, 
       1.2665232093165140e-05, 
       1.5915052412978616e-05, 
       1.3399827582885484e-05, 
       1.6680469784689776e-05, 
       1.3643378838659664e-05, 
       1.3279481386164379e-05, 
       1.7504207846230492e-05, 
       1.5593840334213613e-05, 
       1.4508967471616001e-05, 
       1.4261633455839362e-05, 
       1.4260526372031217e-05, 
       1.6095184673308086e-05, 
       1.4567329784929392e-05, 
       1.2916034349897811e-05, 
       1.4102494158456044e-05, 
       1.4580830373188505e-05, 
       1.4360851731491847e-05, 
       1.6316408455780562e-05, 
       1.4318243849690795e-05, 
       1.3585143566485153e-05, 
       1.4348745972368084e-05, 
       1.2492627150387478e-05, 
       1.2442304049548416e-05, 
       1.4088979986723125e-05, 
       1.6775660393623092e-05, 
       1.2428959037144436e-05, 
       1.5875801930255483e-05, 
       1.5314968384048395e-05, 
       1.2386938181135311e-05, 
       1.4670980613850224e-05, 
       1.2389523137184474e-05, 
       1.2524192672057032e-05, 
       1.2494490006412363e-05, 
       1.4503628483898586e-05, 
       1.3562786969037975e-05, 
       1.4521787795138020e-05, 
       1.4435448392455718e-05, 
       1.2626414392450376e-05, 
       1.2469257022665821e-05, 
       1.4163047639485972e-05, 
       1.4203377880169526e-05, 
       1.7446308986520677e-05, 
       1.4987944051186263e-05, 
       1.3621729740483568e-05, 
       1.6960153082689459e-05, 
       1.2948875232581966e-05, 
       1.2545997984448476e-05, 
       1.4391555603622707e-05, 
       1.2691110441516361e-05, 
       1.3855844707227722e-05, 
       1.7359384566316086e-05, 
       1.6898861535444388e-05, 
       1.3453718136021118e-05, 
       1.4375558402724365e-05, 
       1.7403224908037797e-05, 
       1.6953829584309808e-05, 
       1.2870965835069754e-05, 
       1.3943837491000413e-05, 
       1.2344108032617478e-05, 
       1.2386069398627815e-05, 
       1.2484532124767221e-05, 
       1.4589910278901068e-05, 
       1.3010696110014875e-05, 
       1.7245742834484203e-05, 
       1.7245445108196030e-05, 
       1.4034855803560286e-05, 
       1.3822854927071606e-05, 
       1.2976603896862156e-05, 
       1.2471959762714729e-05, 
       1.2471959762714729e-05, 
       1.2335305587210792e-05, 
       1.3553726418031432e-05, 
       1.6800115964483792e-05, 
       1.2309173331743173e-05, 
       1.3537955235122096e-05, 
       1.2961072457293526e-05, 
       1.2710964121726577e-05, 
       1.2657021703406087e-05, 
       1.4325437482851584e-05, 
       1.7187753713868272e-05, 
       1.2607809372930020e-05, 
       1.2607809372930020e-05, 
       1.6788749265047764e-05, 
       1.6862076102243585e-05, 
       1.2278088143277375e-05, 
       1.2553654088597697e-05, 
       1.3581783324189881e-05, 
       1.3500002929733665e-05, 
       1.4809308484121882e-05, 
       1.5777197720231638e-05, 
       1.2442416536710257e-05, 
       1.2789808865151445e-05, 
       1.3138967632996221e-05, 
       1.2437337981554402e-05, 
       1.2270316966538834e-05, 
       1.6725231655122100e-05, 
       1.2768120295959958e-05, 
       1.4138259865022317e-05, 
       1.5454387441707252e-05, 
       1.3490975182646470e-05, 
       1.2294224671848139e-05, 
       1.6473024040458836e-05, 
       1.2498439961805670e-05, 
       1.3320805486952995e-05, 
       1.4159497493630926e-05, 
       1.3981275923430545e-05, 
       1.4557278735090200e-05, 
       1.7193357221498349e-05, 
       1.2299877382389635e-05, 
       1.5083418308035586e-05, 
       1.7155443087777133e-05, 
       1.3998765574030313e-05, 
       1.5156406202640975e-05, 
       1.5071096782752692e-05, 
       1.6999965577393064e-05, 
       1.2283926466294301e-05, 
       1.4260414705583137e-05, 
       1.2475899247148247e-05, 
       1.4481181432509998e-05, 
       1.6963051812999197e-05, 
       1.6693967239233020e-05, 
       1.6895933397921275e-05, 
       1.7067244968282568e-05, 
       1.6511609766917350e-05, 
       1.2342882256042360e-05, 
       1.2503447975852459e-05, 
       1.4837053480320502e-05, 
       1.3550338030219951e-05, 
       1.6424553768599611e-05, 
       1.4034811378824999e-05, 
       1.4572930529791567e-05, 
       1.5660814224652469e-05, 
       1.2838552643104507e-05, 
       1.4634370473094196e-05, 
       1.4917773651318327e-05, 
       1.5098922607870378e-05, 
       1.2112673093226221e-05, 
       1.2498062274450194e-05, 
       1.2799920476892023e-05, 
       1.7097475884677428e-05, 
       1.3629537207041920e-05, 
       1.3886607702411817e-05, 
       1.4607637107221816e-05, 
       1.2381988056314457e-05, 
       1.2932352729534196e-05, 
       1.4382446488281116e-05, 
       1.6828654080691419e-05, 
       1.6906542095612401e-05, 
       1.4061496842924871e-05, 
       1.6927157243742016e-05, 
       1.6962254898130914e-05, 
       1.7040178422091086e-05, 
       1.3217043253878353e-05, 
       1.2201799911787035e-05, 
       1.3004104001976357e-05, 
       1.4141689479707442e-05, 
       1.3879107047421109e-05, 
       1.6439198247525448e-05, 
       1.2051153145332652e-05, 
       1.2839523566517898e-05, 
       1.6885421914800625e-05, 
       1.3679089576825923e-05, 
       1.2394913505990004e-05, 
       1.6343323208637191e-05, 
       1.5609889988095738e-05, 
       1.6060123324295829e-05, 
       1.2951751462282618e-05, 
       1.6548645289356341e-05, 
       1.2267481770772798e-05, 
       1.6521556092748513e-05, 
       1.2062890661016386e-05, 
       1.5703848225067494e-05, 
       1.4793501401871683e-05, 
       1.2151098340172989e-05, 
       1.2369469662254540e-05, 
       1.3846599821602117e-05, 
       1.2283923578237718e-05, 
       1.3246559181469745e-05, 
       1.6515589836570601e-05, 
       1.6762720462200617e-05, 
       1.2331047004902269e-05, 
       1.5177281524227594e-05, 
       1.3681703599686345e-05, 
       1.6230305120936819e-05, 
       1.3591409024688596e-05, 
       1.6485934940041834e-05, 
       1.2284074227182152e-05, 
       1.2956461204339326e-05, 
       1.2152194763912620e-05, 
       1.6773307292160054e-05, 
       1.1967615650054214e-05, 
       1.1975505905783033e-05, 
       1.2328354550964664e-05, 
       1.6402023829457311e-05, 
       1.2017318192492095e-05, 
       1.4875824108569669e-05, 
       1.4411062269783630e-05, 
       1.4032785219502711e-05, 
       1.6677528590307853e-05, 
       1.3418565222523246e-05, 
       1.2162172897560513e-05, 
       1.4939058389778436e-05, 
       1.2638112356050480e-05, 
       1.6707024854103104e-05, 
       1.6074313361273201e-05, 
       1.3742617783383362e-05, 
       1.5409510500739345e-05, 
       1.5539092049979890e-05, 
       1.2143997447308365e-05, 
       1.2923700438071394e-05, 
       1.2841981824842158e-05, 
       1.5271024521834124e-05, 
       1.6746748218237813e-05, 
       1.1909602730243198e-05, 
       1.3542362766701390e-05, 
       1.3945594853167781e-05, 
       1.6591552297043637e-05, 
       1.3901966460264854e-05, 
       1.2552482636613237e-05, 
       1.1855662649301542e-05, 
       1.6511380287452582e-05, 
       1.6736024251672289e-05, 
       1.1845511249883099e-05, 
       1.2772454401685757e-05, 
       1.6444859774017816e-05, 
       1.1837078722907510e-05, 
       1.6331190648800634e-05, 
       1.4480976230554762e-05, 
       1.4650396000285066e-05, 
       1.4216187792345135e-05, 
       1.3088038476606967e-05, 
       1.3337286452725844e-05, 
       1.2523678926316022e-05, 
       1.4782461139809140e-05, 
       1.1972800210400306e-05, 
       1.2519711528858055e-05, 
       1.6125081896199361e-05, 
       1.6713298992144534e-05, 
       1.6678130045148253e-05, 
       1.6490733408937781e-05, 
       1.4426810859328770e-05, 
       1.2058447012652167e-05, 
       1.4886464507297263e-05, 
       1.2977124969328830e-05, 
       1.3923329897107152e-05, 
       1.2455497239420550e-05, 
       1.1873359680281745e-05, 
       1.6639364238063028e-05, 
       1.1801165510882129e-05, 
       1.1801165510882129e-05, 
       1.2537315706778508e-05, 
       1.2463219717063673e-05, 
       1.5454711096299853e-05, 
       1.3745579890106580e-05, 
       1.1773370292365622e-05, 
       1.6533277169563514e-05, 
       1.4689900333100438e-05, 
       1.1992446244021064e-05, 
       1.6610250634084828e-05, 
       1.1838087415967461e-05, 
       1.1860953182291961e-05, 
       1.6619005424077404e-05, 
       1.2029624982065926e-05, 
       1.4944882191809471e-05, 
       1.2652977391415335e-05, 
       1.6540315396306796e-05, 
       1.6036506868789826e-05, 
       1.3779211802409204e-05, 
       1.3759738104080438e-05, 
       1.3783836622326562e-05, 
       1.1755704371001792e-05, 
       1.3756888755834545e-05, 
       1.1740946677724562e-05, 
       1.1740946677724562e-05, 
       1.2509243542771845e-05, 
       1.2475709864121511e-05, 
       1.3573263252305218e-05, 
       1.6539875014811285e-05, 
       1.1698455754107560e-05, 
       1.3071390328927385e-05, 
       1.2255883138375145e-05, 
       1.1961040627349164e-05, 
       1.4026488168279134e-05, 
       1.4656488798993812e-05, 
       1.3576977009018623e-05, 
       1.6494185236488781e-05, 
       1.6317113509340670e-05, 
       1.3699964069023062e-05, 
       1.1677002613866072e-05, 
       1.3279018477965065e-05, 
       1.3497296118562936e-05, 
       1.4413941486804807e-05, 
       1.2120783480670440e-05, 
       1.3482265122977599e-05, 
       1.3687673948764869e-05, 
       1.4011768348804389e-05, 
       1.3263212176298742e-05, 
       1.2844775986296911e-05, 
       1.2409422598320708e-05, 
       1.2021216942585990e-05, 
       1.2768309311563218e-05, 
       1.2108460891919024e-05, 
       1.6243017234788246e-05, 
       1.4715629585181497e-05, 
       1.2055050537749066e-05, 
       1.5269784078196127e-05, 
       1.1626108412671910e-05, 
       1.5427267837226295e-05, 
       1.3688472411888766e-05, 
       1.4265879036331624e-05, 
       1.6280060006956205e-05, 
       1.1763423671016568e-05, 
       1.3063003799737662e-05, 
       1.1870438638559717e-05, 
       1.2173091482338591e-05, 
       1.1583697085280687e-05, 
       1.1887000154819520e-05, 
       1.6363027769810196e-05, 
       1.6043844214483004e-05, 
       1.1692453485639220e-05, 
       1.2129223522047978e-05, 
       1.3914863308321251e-05, 
       1.3657251234300196e-05, 
       1.3016875725231145e-05, 
       1.3575300025194662e-05, 
       1.3774201644007277e-05, 
       1.2090086248608373e-05, 
       1.6333337471457206e-05, 
       1.4568035462459632e-05, 
       1.1599793775919464e-05, 
       1.5762249770742242e-05, 
       1.1893687494150015e-05, 
       1.2530250343329092e-05, 
       1.3219392035726646e-05, 
       1.2968057635695589e-05, 
       1.2481291080850444e-05, 
       1.5850445928099265e-05, 
       1.1522415149673283e-05, 
       1.2969194686459368e-05, 
       1.2008472306768569e-05, 
       1.1548404921534808e-05, 
       1.2509030139655461e-05, 
       1.1987443003169530e-05, 
       1.2232579529806520e-05, 
       1.4134462807824190e-05, 
       1.3594471423609121e-05, 
       1.5829638151601610e-05, 
       1.1708680329747450e-05, 
       1.3999562549330479e-05, 
       1.3175816829170453e-05, 
       1.1488743339510251e-05, 
       1.2451638480871150e-05, 
       1.1487381860889677e-05, 
       1.2064187339489767e-05, 
       1.1476824243579039e-05, 
       1.1812292511541923e-05, 
       1.2429670499840731e-05, 
       1.5523842376759259e-05, 
       1.4355399612868856e-05, 
       1.1476562269216375e-05, 
       1.4884644722930229e-05, 
       1.1796323039204514e-05, 
       1.5919041856879116e-05, 
       1.3972315656778566e-05, 
       1.1781164840143850e-05, 
       1.1523276164360502e-05, 
       1.1453449992394640e-05, 
       1.3402932809675038e-05, 
       1.3469967678943343e-05, 
       1.3558421750825662e-05, 
       1.5751511430091618e-05, 
       1.6153692887444126e-05, 
       1.3567868841980870e-05, 
       1.5990651316969160e-05, 
       1.2505190789439619e-05, 
       1.4191315159746284e-05, 
       1.2763551207767700e-05, 
       1.3139022883137724e-05, 
       1.1837371971164738e-05, 
       1.4554252916811563e-05, 
       1.3854076524022812e-05, 
       1.2861023192041890e-05, 
       1.1626325100487196e-05, 
       1.1587610802613169e-05, 
       1.1404359297156563e-05, 
       1.5342921659673172e-05, 
       1.2807204217316127e-05, 
       1.1544730960155244e-05, 
       1.5099362504310848e-05, 
       1.6087621791069557e-05, 
       1.5476310540145731e-05, 
       1.3394308699124815e-05, 
       1.2784533483894596e-05, 
       1.3325423239195278e-05, 
       1.1379673462709054e-05, 
       1.2313159437453939e-05, 
       1.2276319429446716e-05, 
       1.3280966122500558e-05, 
       1.6036054550874546e-05, 
       1.1607868526682517e-05, 
       1.1591400444480794e-05, 
       1.1778406704184160e-05, 
       1.3336154921564371e-05, 
       1.3315229438692028e-05, 
       1.4764980571177641e-05, 
       1.4764980571177641e-05, 
       1.3005731369938051e-05, 
       1.3857468221593714e-05, 
       1.2601611945198697e-05, 
       1.1535642234712601e-05, 
       1.4385838370302758e-05, 
       1.1431409072020787e-05, 
       1.5546432469003097e-05, 
       1.1819478940763967e-05, 
       1.1594796087635752e-05, 
       1.4376384604886602e-05, 
       1.4699854886166361e-05, 
       1.4699854886166361e-05, 
       1.4396400266741322e-05, 
       1.4855497795269528e-05, 
       1.2704668369661561e-05, 
       1.1640622584927458e-05, 
       1.4829960101301243e-05, 
       1.3367905391044500e-05, 
       1.5730339965465210e-05, 
       1.3325732032530396e-05, 
       1.5910499795239197e-05, 
       1.1706431528776467e-05, 
       1.1793640490663601e-05, 
       1.1789457250824178e-05, 
       1.1370970085676693e-05, 
       1.5908326901209936e-05, 
       1.3236524848963257e-05, 
       1.3663415886131263e-05, 
       1.1504976578743271e-05, 
       1.1543923892534094e-05, 
       1.1279697603442828e-05, 
       1.1838538878837953e-05, 
       1.1987127670512582e-05, 
       1.1296766269345427e-05, 
       1.2959114971534154e-05, 
       1.3218124520169283e-05, 
       1.2941663212295570e-05, 
       1.4731205970450673e-05, 
       1.1695320843535913e-05, 
       1.1516129984297710e-05, 
       1.5804377992296439e-05, 
       1.2627139401248321e-05, 
       1.1255438195063090e-05, 
       1.5725219389215415e-05, 
       1.2610794816444050e-05, 
       1.1291185821420846e-05, 
       1.1347458430815556e-05, 
       1.4046129526921711e-05, 
       1.1195658159874779e-05, 
       1.3454085532832275e-05, 
       1.1381419963787335e-05, 
       1.1406896544225626e-05, 
       1.1294532890678640e-05, 
       1.1762875123448269e-05, 
       1.1331706585394882e-05, 
       1.5093834594196556e-05, 
       1.4739012246260433e-05, 
       1.3130911919431970e-05, 
       1.2776219510881027e-05, 
       1.1526377574446102e-05, 
       1.1991309522357288e-05, 
       1.2226211083826836e-05, 
       1.5175221405291258e-05, 
       1.1409237087040052e-05, 
       1.4784229181950738e-05, 
       1.2554914194232347e-05, 
       1.3560759521860001e-05, 
       1.3533736027604757e-05, 
       1.3105796811171029e-05, 
       1.2416762654598721e-05, 
       1.3070588015784730e-05, 
       1.4784480416251641e-05, 
       1.2084829361035023e-05, 
       1.1251724539764732e-05, 
       1.1174224509154538e-05, 
       1.1112371076041054e-05, 
       1.1520115843759457e-05, 
       1.5334086415174013e-05, 
       1.1160672547446487e-05, 
       1.4451302495466187e-05, 
       1.1789498666796832e-05, 
       1.1699591699811319e-05, 
       1.5614724789498533e-05, 
       1.4595757897294223e-05, 
       1.2463651078146501e-05, 
       1.1155286087400586e-05, 
       1.1357403717846669e-05, 
       1.5065512276052079e-05, 
       1.3058121209538081e-05, 
       1.5409553591095302e-05, 
       1.4022228235795803e-05, 
       1.3254078202592438e-05, 
       1.1590497891745144e-05, 
       1.5473139660636038e-05, 
       1.3330167724947918e-05, 
       1.4871794157426763e-05, 
       1.3128682627561413e-05, 
       1.2819211413794908e-05, 
       1.2971139573058469e-05, 
       1.1577470508168492e-05, 
       1.5610416448607364e-05, 
       1.5215733471324159e-05, 
       1.1957110588090610e-05, 
       1.1104124698394748e-05, 
       1.4431928381036290e-05, 
       1.2896102274761090e-05, 
       1.3097337808655840e-05, 
       1.1825895801799263e-05, 
       1.1524661511334883e-05, 
       1.2409409129774765e-05, 
       1.2159701957895705e-05, 
       1.1034743965615807e-05, 
       1.3915233306724783e-05, 
       1.1343597954337286e-05, 
       1.4914483200639213e-05, 
       1.5210500243942558e-05, 
       1.5260298435165760e-05, 
       1.3171765115161997e-05, 
       1.3047338974395501e-05, 
       1.4882816353497803e-05, 
       1.1059404205545469e-05, 
       1.4936865174490363e-05, 
       1.2766985023063518e-05, 
       1.2893925531166015e-05, 
       1.2368786192915663e-05, 
       1.3219500897702974e-05, 
       1.1002599954937517e-05, 
       1.1020868212417563e-05, 
       1.4199405601560050e-05, 
       1.3364992036319819e-05, 
       1.5435136575897798e-05, 
       1.1650928381232596e-05, 
       1.4048953285959872e-05, 
       1.5331984029457066e-05, 
       1.3165083799601319e-05, 
       1.5427938157995699e-05, 
       1.5249798292015534e-05, 
       1.1271908252755239e-05, 
       1.2875663488557887e-05, 
       1.5096693194239976e-05, 
       1.3142013420892148e-05, 
       1.3142013420892148e-05, 
       1.3613220654626027e-05, 
       1.5162725173979664e-05, 
       1.0960232019262796e-05, 
       1.3970312000698586e-05, 
       1.2720786844480341e-05, 
       1.2439576800744272e-05, 
       1.5434167899305453e-05, 
       1.1334595558805153e-05, 
       1.3130836031856248e-05, 
       1.5437485254243910e-05, 
       1.5252806126741022e-05, 
       1.2679806263590631e-05, 
       1.1051149529517730e-05, 
       1.5167607600879590e-05, 
       1.1362365442106242e-05, 
       1.2789142817727944e-05, 
       1.1307932390956171e-05, 
       1.1890698855806812e-05, 
       1.1376044665726076e-05, 
       1.0940102390403214e-05, 
       1.1581938286855885e-05, 
       1.0911917563619211e-05, 
       1.2829347364747459e-05, 
       1.1777629661435843e-05, 
       1.2944615431237900e-05, 
       1.1487073909389023e-05, 
       1.4917167854124414e-05, 
       1.5293161765525723e-05, 
       1.3209788303320664e-05, 
       1.5036999696857796e-05, 
       1.1197201014953809e-05, 
       1.4741172951403030e-05, 
       1.0880112711177723e-05, 
       1.4751233477104657e-05, 
       1.4416314356188729e-05, 
       1.0911607764474247e-05, 
       1.2114268406527672e-05, 
       1.5079605106049488e-05, 
       1.3601432200708229e-05, 
       1.1228971459185498e-05, 
       1.2602104378764006e-05, 
       1.2119505435735810e-05, 
       1.3135486265618616e-05, 
       1.3233213652183534e-05, 
       1.1356087833350652e-05, 
       1.4385570413798744e-05, 
       1.0880119892811605e-05, 
       1.3062107178395968e-05, 
       1.2211645162798186e-05, 
       1.1765816781040863e-05, 
       1.1001408024351938e-05, 
       1.2694437425427017e-05, 
       1.1018586978386502e-05, 
       1.2132665622803684e-05, 
       1.1071953261747843e-05, 
       1.4193513743233301e-05, 
       1.2511519833986496e-05, 
       1.4037535283825726e-05, 
       1.3660777921279991e-05, 
       1.5260512510540615e-05, 
       1.5012322681784472e-05, 
       1.5248361112992795e-05, 
       1.1641820369659849e-05, 
       1.2632827152893666e-05, 
       1.4907534823259538e-05, 
       1.5239692329646160e-05, 
       1.2777745489921347e-05, 
       1.2676180169540794e-05, 
       1.3510423823284396e-05, 
       1.0795487128499497e-05, 
       1.4488596246222101e-05, 
       1.4950049701366523e-05, 
       1.2202229493805757e-05, 
       1.2026293789691191e-05, 
       1.2982750774007872e-05, 
       1.1767880451418330e-05, 
       1.4220651019447639e-05, 
       1.0734476731730870e-05, 
       1.1357915378189778e-05, 
       1.0913645623177548e-05, 
       1.0960413680839423e-05, 
       1.2499788879032851e-05, 
       1.3380111343443414e-05, 
       1.2446109729477853e-05, 
       1.2827333352772899e-05, 
       1.1003822470070702e-05, 
       1.0728759686802338e-05, 
       1.0917018151866301e-05, 
       1.1067963392579116e-05, 
       1.0726998344004861e-05, 
       1.1040759581333051e-05, 
       1.2691379618478009e-05, 
       1.0959669581733495e-05, 
       1.1067319579787656e-05, 
       1.1377896894289661e-05, 
       1.2510633106626702e-05, 
       1.0891091901650885e-05, 
       1.0777584065949914e-05, 
       1.1324054378445322e-05, 
       1.1115020312170831e-05, 
       1.3497819104780639e-05, 
       1.1176701791142629e-05, 
       1.0734103838765258e-05, 
       1.3188035917466688e-05, 
       1.3800546246854002e-05, 
       1.2003818416916015e-05, 
       1.1241948336015302e-05, 
       1.2935692445501080e-05, 
       1.4610151409723593e-05, 
       1.2259661994790551e-05, 
       1.0912663366606487e-05, 
       1.2011859699603323e-05, 
       1.1233586332889624e-05, 
       1.5046559987525149e-05, 
       1.1173076007958938e-05, 
       1.1017203891298821e-05, 
       1.3129116666707593e-05, 
       1.1795499610887464e-05, 
       1.4458924151673350e-05, 
       1.4391690213843748e-05, 
       1.0946392933647906e-05, 
       1.0753260275564165e-05, 
       1.1429648995195049e-05, 
       1.0939625745705024e-05, 
       1.1742309149567105e-05, 
       1.3417643480241299e-05, 
       1.1740272789485190e-05, 
       1.2455077712833444e-05, 
       1.4082933638771194e-05, 
       1.4812848580092030e-05, 
       1.0636484163392076e-05, 
       1.2740138849976161e-05, 
       1.1080299397763669e-05, 
       1.4729875165272834e-05, 
       1.4652926064351335e-05, 
       1.2388452190993172e-05, 
       1.1609745189482309e-05, 
       1.4451440761200383e-05, 
       1.2437822170348280e-05, 
       1.0783453965436617e-05, 
       1.1158019940170399e-05, 
       1.4870788033346072e-05, 
       1.2472960200562931e-05, 
       1.0681904699412976e-05, 
       1.0615868154871845e-05, 
       1.2415244490356536e-05, 
       1.1109042099266781e-05, 
       1.1637484604806696e-05, 
       1.0619503241551794e-05, 
       1.0993848273884750e-05, 
       1.2091127701719708e-05, 
       1.0706959311734820e-05, 
       1.3415406696699663e-05, 
       1.3904188254785544e-05, 
       1.0565616680907563e-05, 
       1.0955434589074079e-05, 
       1.4817778274625170e-05, 
       1.3811722608146129e-05, 
       1.0648152663130800e-05, 
       1.2390394217438649e-05, 
       1.0974698395040515e-05, 
       1.3838727097500063e-05, 
       1.1061380224088071e-05, 
       1.4232833043570118e-05, 
       1.0649694828788931e-05, 
       1.0539059100085208e-05, 
       1.2329686298436526e-05, 
       1.0606533385873046e-05, 
       1.4864113112570085e-05, 
       1.1998992969886485e-05, 
       1.4751578226387234e-05, 
       1.3336164308754586e-05, 
       1.3567746747392274e-05, 
       1.1114539414726377e-05, 
       1.0598005793930420e-05, 
       1.0551765433219797e-05, 
       1.4616861103236607e-05, 
       1.1461448699036052e-05, 
       1.0977041683487981e-05, 
       1.3651794554066623e-05, 
       1.0715279091292816e-05, 
       1.1222718580944904e-05, 
       1.2222388883919755e-05, 
       1.3799930584381251e-05, 
       1.0544488217613670e-05, 
       1.4685799775340629e-05, 
       1.4816433203071156e-05, 
       1.2252178183499918e-05, 
       1.2847939832422902e-05, 
       1.0543616110196778e-05, 
       1.0487409265946741e-05, 
       1.1034187197077173e-05, 
       1.0550917307512756e-05, 
       1.0552877443664410e-05, 
       1.0895634494892817e-05, 
       1.1295251606471709e-05, 
       1.0658665803181129e-05, 
       1.4108681308786072e-05, 
       1.1034464564272159e-05, 
       1.2046478417446544e-05, 
       1.2712744957966272e-05, 
       1.4191561439457718e-05, 
       1.2066068276229354e-05, 
       1.0540008854222513e-05, 
       1.0599839928229458e-05, 
       1.1235251149060995e-05, 
       1.4753906265802196e-05, 
       1.0480684677724653e-05, 
       1.0438530216473421e-05, 
       1.1047981100137697e-05, 
       1.0998185193224648e-05, 
       1.1432558062255359e-05, 
       1.0527554364672971e-05, 
       1.0905019411988960e-05, 
       1.1242607667190877e-05, 
       1.2828517394046592e-05, 
       1.1180595006706165e-05, 
       1.1180595006706165e-05, 
       1.4185914684124900e-05, 
       1.4715196328684755e-05, 
       1.0919893747567160e-05, 
       1.1297780666473095e-05, 
       1.2194675089853316e-05, 
       1.1350707639988095e-05, 
       1.2300774000897797e-05, 
       1.1271898793708609e-05, 
       1.1117924482975310e-05, 
       1.1196864256343460e-05, 
       1.2439203825190964e-05, 
       1.2827436894527132e-05, 
       1.2750023771996639e-05, 
       1.3583431501139882e-05, 
       1.0853800750461916e-05, 
       1.1203824109936732e-05, 
       1.2206548479858695e-05, 
       1.1476730178424250e-05, 
       1.4658722913049990e-05, 
       1.4638907142435021e-05, 
       1.3207874939737303e-05, 
       1.0947980011342474e-05, 
       1.0610004698907262e-05, 
       1.3632779740319906e-05, 
       1.0662404488644941e-05, 
       1.2970575774498610e-05, 
       1.1161405211005428e-05, 
       1.0381314204025958e-05, 
       1.2217831820003956e-05, 
       1.2157343488940657e-05, 
       1.0369325143007269e-05, 
       1.2057404112405401e-05, 
       1.0915105104844207e-05, 
       1.2096523335024780e-05, 
       1.0929738402608463e-05, 
       1.0913093511718217e-05, 
       1.1180972623180034e-05, 
       1.1642993037587542e-05, 
       1.2859409430803178e-05, 
       1.3448184593848241e-05, 
       1.4381898891484474e-05, 
       1.1114956795116073e-05, 
       1.1141125678232446e-05, 
       1.2906384145128930e-05, 
       1.0991389553506523e-05, 
       1.0420575666642764e-05, 
       1.2322022966211623e-05, 
       1.0728496236894351e-05, 
       1.1713343965105547e-05, 
       1.1535512222470310e-05, 
       1.2183752692252095e-05, 
       1.1780215350614559e-05, 
       1.2521885328123999e-05, 
       1.0563194018715709e-05, 
       1.0479737161371500e-05, 
       1.0529089926756773e-05, 
       1.0870118654929953e-05, 
       1.0645383676090020e-05, 
       1.1022204237005723e-05, 
       1.1580693087682408e-05, 
       1.3078910958596138e-05, 
       1.3224450467597730e-05, 
       1.2093590680975748e-05, 
       1.0529798597256065e-05, 
       1.3950605783330427e-05, 
       1.0405155291295776e-05, 
       1.4532356645709372e-05, 
       1.1557887317785126e-05, 
       1.4331264654138884e-05, 
       1.3170552423658752e-05, 
       1.0560604063197804e-05, 
       1.2914511009516920e-05, 
       1.4463540925630009e-05, 
       1.4493238831829933e-05, 
       1.1262964441190372e-05, 
       1.1217443702188182e-05, 
       1.0485478705638342e-05, 
       1.0822361034888643e-05, 
       1.0677977066657689e-05, 
       1.3789704872286703e-05, 
       1.3128482137200564e-05, 
       1.3128482137200564e-05, 
       1.1300788157987383e-05, 
       1.0743462033899452e-05, 
       1.4113195084363403e-05, 
       1.1015177595948874e-05, 
       1.1240848953262118e-05, 
       1.4220598988835272e-05, 
       1.0251693494309371e-05, 
       1.0356942271676616e-05, 
       1.0310817643193292e-05, 
       1.0819757982859366e-05, 
       1.1181222095608433e-05, 
       1.1086519442023879e-05, 
       1.0405047370049182e-05, 
       1.3106964823418116e-05, 
       1.1648237857761962e-05, 
       1.1882114562741186e-05, 
       1.4212376331393727e-05, 
       1.2748597903242481e-05, 
       1.4062700807589073e-05, 
       1.4172316877647646e-05, 
       1.4192643872286560e-05, 
       1.0343460932099921e-05, 
       1.2836567213794866e-05, 
       1.3493684316348202e-05, 
       1.4095274188926477e-05, 
       1.0217351885321709e-05, 
       1.1887388815049288e-05, 
       1.0227999987175334e-05, 
       1.2016387655506428e-05, 
       1.2971797911614471e-05, 
       1.3445474046718865e-05, 
       1.2079935079262819e-05, 
       1.0819976918595336e-05, 
       1.1219142276377080e-05, 
       1.1396223920582028e-05, 
       1.3571867867413760e-05, 
       1.0440074451155300e-05, 
       1.1975378689078452e-05, 
       1.0524580143455178e-05, 
       1.1980279389708748e-05, 
       1.0269156897812358e-05, 
       1.1970288204529031e-05, 
       1.2576160972068977e-05, 
       1.1990423566327884e-05, 
       1.2179815631313550e-05, 
       1.0244582907604472e-05, 
       1.1918670759452220e-05, 
       1.0440668415209756e-05, 
       1.2457996718812353e-05, 
       1.4252083669744673e-05, 
       1.4257714289259674e-05, 
       1.0232147136043175e-05, 
       1.0362532560831333e-05, 
       1.2616749358082860e-05, 
       1.2217133877581721e-05, 
       1.0942241347660040e-05, 
       1.1995377579496135e-05, 
       1.3937639753039255e-05, 
       1.2052315192076341e-05, 
       1.0657909308048463e-05, 
       1.0984380242742437e-05, 
       1.1468399116738412e-05, 
       1.2672634348090212e-05, 
       1.4262014406743227e-05, 
       1.0702897124077525e-05, 
       1.2473674947314517e-05, 
       1.1980681223105419e-05, 
       1.2762042933529212e-05, 
       1.0246696592659159e-05, 
       1.1062006358732409e-05, 
       1.0541362827602487e-05, 
       1.1761721137424494e-05, 
       1.0193337481050889e-05, 
       1.4245773551838934e-05, 
       1.1121978731039584e-05, 
       1.0210997003739260e-05, 
       1.0261622470509316e-05, 
       1.1840748520204376e-05, 
       1.2223977087196286e-05, 
       1.1938039783335699e-05, 
       1.4218991417974111e-05, 
       1.2254839504274966e-05, 
       1.2254839504274966e-05, 
       1.0363599605257835e-05, 
       1.1091793278381049e-05, 
       1.0991394130522510e-05, 
       1.3767536684400838e-05, 
       1.2257392763722619e-05, 
       1.0091338692569694e-05, 
       1.1940986395201969e-05, 
       1.2728472975436712e-05, 
       1.4234519023296746e-05, 
       1.4179297486403456e-05, 
       1.4211292201448137e-05, 
       1.0807116501392373e-05, 
       1.1432153739019050e-05, 
       1.0588314605866533e-05, 
       1.0601215426119423e-05, 
       1.2474957896494427e-05, 
       1.0110950982454661e-05, 
       1.1765852442352391e-05, 
       1.3063215255467915e-05, 
       1.2097295464289595e-05, 
       1.3377317274418171e-05, 
       1.0256962663487446e-05, 
       1.0918286234002667e-05, 
       1.0040154454996167e-05, 
       1.2314910558417348e-05, 
       1.2381680462325893e-05, 
       1.2262271650170048e-05, 
       1.2500611476233270e-05, 
       1.4170372253249018e-05, 
       1.3860929699559270e-05, 
       1.0295876872724294e-05, 
       1.4150164951585038e-05, 
       1.4168669082428720e-05, 
       1.4168669082428720e-05, 
       1.0022795513429451e-05, 
       1.3811032014607832e-05, 
       1.4165287469393681e-05, 
       1.0400835662083885e-05, 
       1.2775656086646232e-05, 
       1.1465809149777593e-05, 
       1.1778244916515037e-05, 
       1.3640944817942931e-05, 
       1.2308665443426985e-05, 
       1.3911938834561814e-05, 
       1.0004030941717438e-05, 
       1.0383184088395749e-05, 
       1.1743217462010197e-05, 
       1.0309746749173572e-05, 
       1.0247770577424164e-05, 
       1.0605412830024893e-05, 
       1.1674255861494337e-05, 
       1.0547939428984845e-05, 
       1.0755911967618475e-05, 
       1.0755911967618475e-05, 
       1.1742758684484464e-05, 
       1.0378227960700910e-05, 
       1.0353677029262057e-05, 
       1.0065808317388327e-05, 
       1.3626788072873657e-05, 
       1.0056924523394971e-05, 
       1.3764560799280717e-05, 
       1.1744295938719913e-05, 
       1.1789628643451820e-05, 
       1.3823208696104777e-05, 
       1.3836771726742849e-05, 
       1.0011732520350094e-05, 
       1.0188398686554663e-05, 
       1.2105455992031732e-05, 
       1.0616900291928309e-05, 
       1.0891582039366153e-05, 
       1.4047804863415895e-05, 
       1.0723982414901454e-05, 
       1.0723982414901454e-05, 
       1.2195102371984048e-05, 
       1.0282270946052643e-05, 
       1.1712518041176429e-05, 
       1.3162071804922592e-05, 
       1.0535304716944216e-05, 
       1.0581162499063250e-05, 
       1.2014355872174439e-05, 
       1.2873987436884781e-05, 
       1.0000000000000023e-05, 
       1.0000000000000268e-05, 
       1.0000000000000268e-05, 
       1.0033592543098577e-05, 
       1.0800985765584415e-05, 
       1.1532991698213397e-05, 
       1.3526479268226931e-05, 
       1.0000000000000323e-05, 
       1.0209533010079571e-05, 
       1.1425515239113752e-05, 
       1.0672595273858708e-05, 
       1.0386428230457244e-05, 
       1.3996350046693657e-05, 
       1.3843064277409263e-05, 
       1.3510773089777084e-05, 
       1.2056171205435316e-05, 
       1.3041147945189136e-05, 
       1.1545379148887747e-05, 
       1.0669072968944727e-05, 
       1.3422342374721899e-05, 
       1.0335975632094324e-05, 
       1.1355108394263668e-05, 
       1.0201521764179614e-05, 
       1.0696988400802460e-05, 
       1.2562648157101754e-05, 
       1.3790198634945796e-05, 
       1.0816152447077934e-05, 
       1.0816152447077934e-05, 
       1.3693871290507112e-05, 
       1.0371421785939316e-05, 
       1.1049091633219835e-05, 
       1.1049091633219835e-05, 
       9.9999999999998026e-06, 
       9.8863414828475480e-06, 
       1.0842334354545805e-05, 
       1.1016553673155095e-05, 
       1.3806136990466827e-05, 
       1.0172321800882813e-05, 
       1.0852161581951677e-05, 
       1.2420685585964461e-05, 
       1.0272382108427438e-05, 
       1.1555529804645246e-05, 
       1.0230642211654634e-05, 
       1.1860806030675018e-05, 
       1.1972800210400375e-05, 
       1.0636661219404868e-05, 
       1.1337543309452617e-05, 
       1.0582387614387777e-05, 
       1.1690955108063830e-05, 
       1.0069854078196332e-05, 
       1.0575796050493584e-05, 
       1.0378017196842637e-05, 
       1.3734244839181230e-05, 
       1.0416475571937694e-05, 
       1.0404572494067358e-05, 
       1.1581954715254319e-05, 
       1.3849959962822386e-05, 
       1.2633992051665372e-05, 
       1.1517864245491153e-05, 
       1.3753117039886167e-05, 
       1.0084075968533105e-05, 
       1.3229163901084218e-05, 
       1.3827459930612570e-05, 
       1.1485796255562143e-05, 
       1.2463485321869251e-05, 
       1.0328189515704196e-05, 
       1.1705425399479901e-05, 
       1.0225050862827386e-05, 
       1.0583832519574678e-05, 
       1.1702613649141828e-05, 
       1.0918332262707825e-05, 
       1.2930036780897380e-05, 
       1.1464072437495110e-05, 
       1.0160436762029178e-05, 
       1.0442452292664705e-05, 
       1.0973096565909740e-05, 
       1.0836505316468286e-05, 
       1.1358432824729901e-05, 
       1.0263225946912435e-05, 
       1.3365430761733129e-05, 
       9.9799890045457438e-06, 
       1.2208053131607979e-05, 
       1.0030559344939961e-05, 
       1.1441834967558262e-05, 
       1.3646803393589440e-05, 
       1.1395454000872295e-05, 
       1.0449805574618379e-05, 
       1.0470554850828422e-05, 
       1.1948195445534538e-05, 
       1.1655612665031576e-05, 
       1.2885907093996807e-05, 
       1.3090450128476459e-05, 
       1.1100879367236872e-05, 
       1.1077682739016790e-05, 
       1.0652921081990114e-05, 
       1.0110335438793266e-05, 
       1.3683167452455347e-05, 
       1.0107142500523414e-05, 
       1.1642936341962837e-05, 
       1.2004178018172652e-05, 
       1.1444779616145984e-05, 
       1.3624996005764152e-05, 
       1.2874817466288836e-05, 
       1.3505913785834337e-05, 
       1.2700988828783988e-05, 
       1.1090490323178219e-05, 
       1.3636622274195857e-05, 
       1.2164703814587884e-05, 
       1.0301581246945533e-05, 
       1.0968053562467948e-05, 
       1.3240901082505807e-05, 
       1.3108089869196893e-05, 
       1.3137693969723944e-05, 
       1.1401138646089511e-05, 
       1.3570164833343339e-05, 
       1.2767501669234633e-05, 
       1.3385644848559539e-05, 
       1.2655776463543991e-05, 
       1.2964417236072301e-05, 
       1.0229194005516311e-05, 
       1.0883072595791353e-05, 
       1.1679234438147814e-05, 
       1.1679234438147814e-05, 
       1.2490013806370188e-05, 
       1.1745994425535471e-05, 
       1.3486364172694538e-05, 
       1.3528212282673858e-05, 
       1.3553187248982594e-05, 
       1.0375079999629666e-05, 
       1.3301925489004176e-05, 
       1.0242896568991389e-05, 
       1.2779438489267659e-05, 
       1.1241657558686918e-05, 
       1.0575834157958750e-05, 
       1.3500186587812073e-05, 
       1.3423660757942420e-05, 
       1.2928116373951288e-05, 
       1.0194224110811556e-05, 
       1.0586378131950798e-05, 
       1.3482629488277775e-05, 
       1.3476123108973351e-05, 
       1.0580794659944253e-05, 
       1.0865960194484819e-05, 
       1.3005561107178907e-05, 
       1.2013861686729904e-05, 
       1.0192595347934893e-05, 
       1.0295145161628820e-05, 
       1.0133772205900771e-05, 
       1.1159488928946389e-05, 
       1.3235194096015786e-05, 
       1.3342282408806637e-05, 
       1.1180049595241262e-05, 
       1.0175929685198419e-05, 
       1.2895519020831097e-05, 
       1.1798273344587275e-05, 
       1.0364329079291338e-05, 
       1.0256838686053355e-05, 
       1.2428105951296390e-05, 
       1.3345874543649668e-05, 
       1.1053170611947719e-05, 
       1.1000748706659975e-05, 
       1.1000748706659975e-05, 
       1.2310427348020285e-05, 
       1.0513032395675814e-05, 
       1.1884185445599024e-05, 
       1.0350964768616864e-05, 
       1.1262161674427384e-05, 
       1.2440603050303903e-05, 
       1.0801818030161721e-05, 
       1.2595514104754330e-05, 
       1.2326398375685728e-05, 
       1.3121462357231044e-05, 
       1.2051111135393631e-05, 
       9.6861944557425459e-06, 
       1.2847466052592783e-05, 
       1.1167242322871567e-05, 
       1.1116526292907975e-05, 
       1.0919008697364914e-05, 
       1.2820466908398491e-05, 
       1.0006634682572335e-05, 
       1.1799576701145489e-05, 
       1.3213155925409601e-05, 
       1.3240111960265754e-05, 
       1.1488586968185329e-05, 
       1.3048285481238764e-05, 
       1.3327145467522403e-05, 
       1.0729667180655719e-05, 
       1.1740840462466724e-05, 
       1.2120482424671358e-05, 
       1.1105434464188138e-05, 
       1.1415543058879631e-05, 
       1.1784324663312130e-05, 
       1.0359203405959379e-05, 
       1.0198098845117519e-05, 
       1.0727830680943782e-05, 
       1.3028763351597676e-05, 
       1.2394632676909523e-05, 
       1.2394632676909523e-05, 
       1.0104657847434613e-05, 
       1.1179510379559346e-05, 
       1.2827736738588087e-05, 
       1.1049125996792385e-05, 
       1.0118138447390444e-05, 
       1.0938863237606858e-05, 
       1.3284088339869708e-05, 
       1.0030471288876616e-05, 
       1.1465809149776834e-05, 
       1.2727723023151873e-05, 
       1.1063968864139535e-05, 
       1.0296427106331738e-05, 
       1.0413933044058707e-05, 
       1.3260080337262532e-05, 
       1.3252913880364026e-05, 
       1.2639527702349771e-05, 
       1.3124664698887093e-05, 
       1.1112226361145868e-05, 
       1.1324609052229203e-05, 
       1.0260673948150928e-05, 
       1.1378045706685741e-05, 
       1.1778752049701913e-05, 
       1.0529287717220461e-05, 
       9.9999999999996654e-06, 
       1.2566543426591820e-05, 
       1.3212829544740357e-05, 
       1.1437621428574644e-05, 
       1.0344629517573135e-05, 
       1.3184394044685354e-05, 
       1.0912589505804658e-05, 
       9.6880600984534400e-06, 
       1.0000000000000016e-05, 
       1.1808055748105135e-05, 
       9.9999999999996366e-06, 
       1.1168479109041615e-05, 
       1.1266174807010030e-05, 
       1.1088943114261005e-05, 
       1.0531901172657745e-05, 
       1.1975802412901960e-05, 
       9.5440433848949558e-06, 
       1.1157699109273153e-05, 
       1.3162535441487810e-05, 
       1.3144009559597298e-05, 
       1.0934163879949750e-05, 
       1.2030385567571521e-05, 
       1.2860220334494098e-05, 
       1.2465258485631167e-05, 
       1.0809789107247169e-05, 
       1.2483979141210206e-05, 
       1.0061794968809452e-05, 
       1.0718601593863509e-05, 
       1.0645033821438555e-05, 
       9.9321116486192328e-06, 
       1.2828634711558548e-05, 
       1.0901080254263570e-05, 
       1.0780892821658742e-05, 
       1.2492528783680332e-05, 
       1.0541630982379583e-05, 
       1.0583470070958790e-05, 
       1.3075268468044839e-05, 
       1.0865920465580460e-05, 
       1.2833127188939596e-05, 
       1.0836004047366933e-05, 
       1.2096826383260093e-05, 
       1.3051139882532289e-05, 
       1.1340249501081946e-05, 
       1.2853405433508901e-05, 
       1.0989910376443089e-05, 
       1.2611360976825520e-05, 
       1.0072804733493211e-05, 
       1.0000000000000102e-05, 
       1.0918875645461945e-05, 
       1.2467239289853937e-05, 
       1.2744350577852576e-05, 
       1.2259309466877248e-05, 
       1.2200874581673758e-05, 
       1.0778205181598157e-05, 
       9.9999999999999110e-06, 
       1.2972779964556739e-05, 
       1.0960577395688777e-05, 
       1.2762456251609355e-05, 
       1.2823548419072967e-05, 
       1.1519499364513228e-05, 
       1.0055262577626300e-05, 
       1.1364520851940623e-05, 
       1.0918209704670864e-05, 
       1.0492225339710271e-05, 
       1.0986575418765329e-05, 
       1.0708759334643327e-05, 
       1.2002906148886288e-05, 
       1.2500285764523185e-05, 
       1.0979883293440163e-05, 
       1.2893993524991976e-05, 
       1.0146693875938490e-05, 
       1.0146693875938490e-05, 
       1.2511123638863749e-05, 
       1.0764462423615642e-05, 
       1.2202636357993463e-05, 
       1.0307741245399329e-05, 
       1.2520590834186167e-05, 
       1.0421492432074835e-05, 
       1.0319865455529786e-05, 
       1.1360091841782575e-05, 
       1.0837429653674148e-05, 
       1.1623919055502374e-05, 
       1.1846934740013011e-05, 
       1.0889709012286301e-05, 
       1.0820849548411076e-05, 
       1.0688216114365459e-05, 
       1.0744630642435491e-05, 
       1.0516142503876665e-05, 
       1.2690810856557470e-05, 
       1.2453915066165369e-05, 
       1.0024401752492738e-05, 
       1.0399536098569722e-05, 
       1.2841554799162064e-05, 
       1.2371386953186853e-05, 
       1.2809527010425565e-05, 
       1.0696610597318611e-05, 
       1.0849138714719213e-05, 
       1.0778176715706983e-05, 
       1.2190331993269490e-05, 
       1.1725430873935922e-05, 
       1.2268706427768936e-05, 
       1.1083578457964460e-05, 
       1.0529499891400088e-05, 
       1.0408199320983197e-05, 
       1.0267425298825185e-05, 
       1.1102949974510588e-05, 
       1.0228556525520173e-05, 
       1.0822384486713595e-05, 
       1.1726431655036165e-05, 
       1.0622283055352958e-05, 
       1.0507116872230133e-05, 
       1.2508011499520140e-05, 
       1.0701480936194088e-05, 
       1.0858941383488884e-05, 
       1.0828342586633265e-05, 
       1.0868737606804769e-05, 
       1.0515856237448519e-05, 
       1.0862968014903321e-05, 
       1.2353519139857615e-05, 
       1.2691384339691330e-05, 
       1.0319655845241792e-05, 
       1.1420857041808283e-05, 
       1.0762032042260704e-05, 
       1.0231557242102373e-05, 
       1.0842835758030894e-05, 
       1.0569213931884323e-05, 
       1.1621722314993375e-05, 
       1.0534936312424333e-05, 
       1.2193195697120969e-05, 
       1.0215665961202269e-05, 
       1.2543609614379301e-05, 
       1.0605870145837993e-05, 
       1.0814859593310377e-05, 
       1.0888060708822404e-05, 
       1.0415183192091767e-05, 
       1.1198884113209313e-05, 
       1.0132178103724105e-05, 
       1.0675364608552892e-05, 
       1.0123843706689495e-05, 
       1.2131667974029125e-05, 
       1.0277639360913933e-05, 
       1.0488810596038888e-05, 
       1.0891154897709894e-05, 
       1.0131230498547337e-05, 
       1.0491837130241224e-05, 
       9.7735644636457358e-06, 
       1.0648342360139235e-05, 
       1.2518346581190158e-05, 
       1.0240618113256932e-05, 
       1.0277168100648207e-05, 
       1.1918107311041250e-05, 
       1.0391178957444035e-05, 
       1.0278392183878733e-05, 
       1.0278392183878733e-05, 
       1.1720904715688604e-05, 
       1.2210414082431252e-05, 
       1.0150850778128672e-05, 
       1.1553441605469168e-05, 
       1.1334665045753116e-05, 
       1.0677203457460138e-05, 
       1.2429794488038528e-05, 
       1.2205913188185127e-05, 
       1.2008345346664152e-05, 
       1.0157142412420406e-05, 
       1.2392435906262250e-05, 
       1.0435466995246321e-05, 
       1.0515409084517231e-05, 
       1.2195971432321853e-05, 
       1.2235711884014671e-05, 
       1.2292589076882048e-05, 
       1.2451993531768390e-05, 
       1.2501743542146667e-05, 
       1.0969058251643321e-05, 
       1.1178757165294488e-05, 
       1.2422510615194479e-05, 
       1.1368031979734734e-05, 
       1.1382720783756992e-05, 
       1.2485310554560600e-05, 
       1.1107335688654900e-05, 
       1.0355668138949830e-05, 
       1.2477518961862657e-05, 
       1.0631202532206513e-05, 
       1.2223159662942926e-05, 
       1.0242237262682289e-05, 
       1.2420697734083275e-05, 
       1.0434942316190652e-05, 
       1.1445932383622525e-05, 
       1.0171224485565190e-05, 
       1.1720131338961665e-05, 
       1.0327567585121191e-05, 
       1.2188033579879494e-05, 
       1.0779648076362077e-05, 
       1.0320481366151830e-05, 
       1.0436073830974198e-05, 
       1.0208397914965615e-05, 
       1.2065356413651382e-05, 
       1.2065356413651382e-05, 
       1.1274243583633306e-05, 
       1.2354013568792376e-05, 
       1.1295689393518569e-05, 
       1.0118517548772012e-05, 
       1.2350502129827803e-05, 
       1.2254751365195674e-05, 
       1.1878441154163293e-05, 
       1.2387603755462172e-05, 
       1.1623463077479740e-05, 
       1.2317193135841640e-05, 
       1.0078216330379143e-05, 
       1.0366183554054452e-05, 
       1.0487280624726505e-05, 
       1.1866795523912437e-05, 
       1.1899697738117376e-05, 
       1.0736487387057855e-05, 
       1.1950070976428671e-05, 
       1.1221080265170288e-05, 
       1.1697319550334386e-05, 
       1.1769674617668093e-05, 
       1.0943934790898658e-05, 
       1.1804992754280547e-05, 
       1.2263794458669887e-05, 
       1.2143834399481443e-05, 
       1.1411449496704077e-05, 
       1.1392848751274944e-05, 
       1.1366047363172596e-05, 
       1.2283669916598482e-05, 
       1.0340624561312349e-05, 
       1.2280866626799468e-05, 
       1.0173644722021488e-05, 
       1.2095338482765341e-05, 
       1.2036863138255337e-05, 
       1.1815858135741824e-05, 
       1.1815858135741824e-05, 
       1.2266954348323858e-05, 
       1.2044678583800169e-05, 
       1.0093783567833868e-05, 
       1.0136421901119502e-05, 
       1.1872768703119583e-05, 
       1.2226382891714440e-05, 
       1.2047532551853724e-05, 
       1.2192925558836002e-05, 
       1.1137101927630641e-05, 
       1.0049082437662472e-05, 
       1.0049082437662472e-05, 
       1.2226454785819145e-05, 
       1.0456657992935101e-05, 
       1.1746499882668840e-05, 
       1.0487829631785488e-05, 
       1.2193288145787433e-05, 
       1.0221126412831013e-05, 
       1.0601860689653090e-05, 
       1.1559269957116234e-05, 
       1.2189619299817823e-05, 
       1.2139763415502552e-05, 
       1.1497971921071812e-05, 
       1.2065540104242826e-05, 
       1.1235033336164941e-05, 
       1.0610767812585569e-05, 
       1.0794326490968710e-05, 
       1.0273325805973786e-05, 
       1.1343878200826428e-05, 
       1.1609328060759204e-05, 
       1.1306341753215061e-05, 
       1.0825536710383240e-05, 
       1.1005795704342938e-05, 
       1.2121060757588093e-05, 
       1.2106505031350982e-05, 
       8.5754234129220994e-06, 
       1.0815761881751580e-05, 
       1.0219010595921674e-05, 
       1.0270514675373261e-05, 
       1.0813708100617444e-05, 
       1.0105667861555246e-05, 
       1.0056928116534905e-05, 
       1.0679507236274683e-05, 
       1.0249886465693353e-05, 
       1.0630195034163127e-05, 
       1.1803059379380101e-05, 
       1.0809637084121703e-05, 
       1.1429606680863589e-05, 
       1.1696599587364543e-05, 
       1.1214113892499516e-05, 
       1.1115287854269790e-05, 
       1.0997164930048754e-05, 
       1.1838762179157279e-05, 
       1.1641041837148440e-05, 
       1.1823447647627574e-05, 
       1.1104117240007266e-05, 
       1.1517148443691452e-05, 
       9.1485220696467970e-06, 
       1.0238965638900136e-05, 
       8.8919292874191530e-06, 
       1.1896330057952346e-05, 
       1.0497030131876357e-05, 
       1.1931376034776510e-05, 
       1.1933623040098443e-05, 
       1.0008601757717923e-05, 
       1.0591796756262276e-05, 
       1.1808370660381053e-05, 
       1.0153248540206407e-05, 
       1.0583159446168757e-05, 
       1.1569757653558306e-05, 
       1.1169324836137592e-05, 
       1.0246696592656460e-05, 
       1.1634855361359325e-05, 
       1.1835975123834041e-05, 
       1.0386781596695411e-05, 
       1.0254535427430939e-05, 
       1.1781556043583744e-05, 
       1.0550275164110156e-05, 
       1.0673620790962177e-05, 
       1.0323547064690068e-05, 
       1.0304941240959837e-05, 
       1.1837892347508179e-05, 
       1.1610136318218489e-05, 
       1.1075039004490072e-05, 
       1.0201628826680417e-05, 
       1.1664572115206606e-05, 
       1.0350041204194961e-05, 
       1.0281257550234481e-05, 
       1.0183497996229007e-05, 
       1.1790867452521418e-05, 
       1.1631679243118851e-05, 
       1.1338406639534840e-05, 
       1.0176300122955356e-05, 
       1.0251446762046222e-05, 
       1.0958827008473277e-05, 
       1.1576712894601486e-05, 
       1.0683772784826339e-05, 
       1.1585407245460639e-05, 
       1.0967951697648741e-05, 
       1.1358912209348374e-05, 
       1.0398128628659206e-05, 
       1.0835359287821527e-05, 
       1.1346940508727855e-05, 
       1.0226529771569070e-05, 
       1.1611212997899574e-05, 
       1.0473237255046831e-05, 
       1.1288721322259441e-05, 
       1.1698333661076355e-05, 
       9.2370224114736929e-06, 
       1.1064033135139272e-05, 
       1.1027381989352890e-05, 
       1.1671777168690116e-05, 
       1.1643272554156096e-05, 
       1.1688194370409005e-05, 
       1.0999967204317031e-05, 
       1.1607986155636769e-05, 
       1.1682607882720718e-05, 
       1.1636888690997451e-05, 
       1.1546907529907148e-05, 
       1.0373814780092168e-05, 
       1.1205740540454186e-05, 
       1.1517353343776237e-05, 
       1.0844527101560402e-05, 
       1.0000000000000131e-05, 
       1.0107604450208000e-05, 
       1.1638821972318951e-05, 
       9.9999999999995366e-06, 
       1.0704849040951484e-05, 
       9.9885471276394164e-06, 
       1.1513813993823662e-05, 
       1.1393396355224885e-05, 
       1.1155008795590567e-05, 
       9.2586238336926927e-06, 
       1.1520535543541284e-05, 
       1.1472151178516296e-05, 
       1.0204638282290158e-05, 
       1.1041110004442580e-05, 
       1.1111133081360804e-05, 
       1.1511399872203585e-05, 
       1.0084716239022681e-05, 
       1.1338504819397106e-05, 
       1.0045158540498697e-05, 
       9.5863471412588328e-06, 
       1.0437127983706993e-05, 
       1.1505072339552708e-05, 
       1.0906688298653201e-05, 
       1.0602208009787852e-05, 
       1.0850364545656378e-05, 
       1.0946978336327034e-05, 
       1.1248986091004524e-05, 
       1.1053100797946898e-05, 
       1.1454865516766797e-05, 
       1.1407421854723259e-05, 
       1.1446701910336723e-05, 
       1.1288404556624391e-05, 
       1.0392249307434665e-05, 
       1.1370560671741503e-05, 
       1.1136702107910117e-05, 
       1.0571192903146808e-05, 
       1.0571192903146808e-05, 
       1.0035826633069446e-05, 
       1.0153506910273173e-05, 
       1.1292970679226960e-05, 
       1.1282465065604453e-05, 
       1.1357160832396014e-05, 
       1.0032210568173383e-05, 
       1.0903015001942314e-05, 
       1.1169750399560845e-05, 
       1.0169324841006118e-05, 
       1.1317743609076503e-05, 
       1.0441894575960025e-05, 
       1.0855867360557481e-05, 
       1.1003852832147987e-05, 
       1.1221111558544434e-05, 
       1.1229090984497455e-05, 
       1.1208141846847685e-05, 
       1.1047875195611803e-05, 
       1.0861470609398258e-05, 
       1.0230465364057099e-05, 
       1.0868146619565750e-05, 
       8.7573672075511444e-06, 
       1.1113716088480374e-05, 
       1.0487124692589549e-05, 
       1.1125459079521999e-05, 
       1.1089927485805763e-05, 
       1.0941880566651757e-05, 
       1.0984098754776277e-05, 
       1.0692711398637797e-05, 
       1.0790492830363194e-05, 
       1.0997608543055777e-05, 
       1.1058123076535649e-05, 
       1.0668058765095470e-05, 
       1.0088839272899624e-05, 
       1.0764745606081065e-05, 
       1.0952099235736701e-05, 
       1.0982488563152961e-05, 
       7.8837979404905654e-06, 
       1.0862229832361817e-05, 
       1.0541641529977805e-05, 
       1.0673604470606032e-05, 
       8.4200784579030288e-06, 
       1.0449246940496374e-05, 
       1.0451199173168662e-05, 
       1.0451199173168662e-05, 
       1.0765901784358514e-05, 
       1.0027420269936300e-05, 
       1.0690276818582648e-05, 
       1.0932588706010660e-05, 
       1.0242400449443212e-05, 
       1.0925597573154379e-05, 
       1.0526296249549063e-05, 
       1.0901892062286681e-05, 
       1.0892631794838299e-05, 
       1.0860212778975072e-05, 
       1.0901760409426162e-05, 
       1.0479923337425894e-05, 
       1.0319417750035747e-05, 
       1.0494016569488708e-05, 
       1.0882109539471085e-05, 
       9.0100892682608505e-06, 
       1.0551879609854271e-05, 
       1.0857297352979116e-05, 
       1.0601985153734728e-05, 
       1.0855113115728639e-05, 
       9.9999999999997670e-06, 
       9.9999999999996874e-06, 
       1.0000000000000477e-05, 
       9.9999999999999602e-06, 
       1.0000000000000265e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       9.9999999999995078e-06, 
       9.9999999999998263e-06, 
       1.0000000000000402e-05, 
       1.0000000000000058e-05, 
       1.0851775992997990e-05, 
       1.0777559534306887e-05, 
       1.0457837785484885e-05, 
       1.0337086705433358e-05, 
       1.0399030604689359e-05, 
       1.0554182453027874e-05, 
       1.0606808503216396e-05, 
       1.0469552074835209e-05, 
       1.0758187477315188e-05, 
       1.0764274253704218e-05, 
       1.0748503052501520e-05, 
       1.0055159804355488e-05, 
       1.0560590688131672e-05, 
       1.0531867662253147e-05, 
       1.0200606462791767e-05, 
       1.0435063084191800e-05, 
       1.0058662408922714e-05, 
       1.0463192453881937e-05, 
       1.0709339178497549e-05, 
       1.0248306633511604e-05, 
       1.0657939536519573e-05, 
       1.0032877344981024e-05, 
       1.0285698682298048e-05, 
       1.0339573555828056e-05, 
       1.0333301509618587e-05, 
       1.0430174001961241e-05, 
       1.0580486546586671e-05, 
       9.7335271290188978e-06, 
       1.0547214944993346e-05, 
       1.0178681727673101e-05, 
       1.0466926065755992e-05, 
       1.0582186412973638e-05, 
       1.0239764319536040e-05, 
       1.0223160230998520e-05, 
       1.0533003112420351e-05, 
       1.0298809874875783e-05, 
       1.0519401134910453e-05, 
       1.0531407751771536e-05, 
       1.0493063275361416e-05, 
       1.0470235844516859e-05, 
       1.0393490793909300e-05, 
       1.0457768047153454e-05, 
       1.0063106233817438e-05, 
       9.9999999999997975e-06, 
       9.6066279557647178e-06, 
       1.0091618588870042e-05, 
       1.0358770802625782e-05, 
       1.0389774252348478e-05, 
       1.0361234027520185e-05, 
       7.4402997717178416e-06, 
       1.0284713501300155e-05, 
       1.0279142377792454e-05, 
       1.0312178237078133e-05, 
       9.1579311574051531e-06, 
       1.0333965162543499e-05, 
       1.0316025344018487e-05, 
       1.0305496593024411e-05, 
       1.0187916711954165e-05, 
       1.0061483530566063e-05, 
       1.0235734916679891e-05, 
       1.0242991715124708e-05, 
       9.9999999999996722e-06, 
       1.0276460202281464e-05, 
       1.0021533364610812e-05, 
       1.0232542491765221e-05, 
       1.0246155073062100e-05, 
       1.0218592229360409e-05, 
       1.0029594617618908e-05, 
       9.7360476184160780e-06, 
       1.0124465322484542e-05, 
       9.7775140930810087e-06, 
       1.0044288737344210e-05, 
       1.0088411917600719e-05, 
       1.0142038186682246e-05, 
       1.0010592423648064e-05, 
       9.9999999999997281e-06, 
       1.0071131086643633e-05, 
       1.0122300585079421e-05, 
       1.0109949153041595e-05, 
       1.0107767308264768e-05, 
       1.0115076025184727e-05, 
       1.0041907279180073e-05, 
       1.0019880079227199e-05, 
       1.0050018788159446e-05, 
       7.2384455027785996e-06, 
       1.0000000000000418e-05, 
       1.0000000000000414e-05, 
       1.0000000000000197e-05, 
       1.0000000000000345e-05, 
       9.9999999999996671e-06, 
       9.9999999999996061e-06, 
       9.9999999999996061e-06, 
       1.0000000000000396e-05, 
       9.9999999999999415e-06, 
       1.0000000000000102e-05, 
       1.0000000000000018e-05, 
       9.9999999999998229e-06, 
       9.9999999999998229e-06, 
       9.9999999999998297e-06, 
       9.9999999999995604e-06, 
       1.0000000000000370e-05, 
       1.0000000000000370e-05, 
       1.0000000000000370e-05, 
       9.9999999999998433e-06, 
       1.0000000000000087e-05, 
       9.9999999999996773e-06, 
       9.9999999999995858e-06, 
       9.9999999999995858e-06, 
       1.0000000000000140e-05, 
       1.0000000000000140e-05, 
       1.0000000000000268e-05, 
       9.9999999999997755e-06, 
       9.9999999999995587e-06, 
       9.9999999999997365e-06, 
       1.0000000000000043e-05, 
       1.0000000000000043e-05, 
       1.0000000000000043e-05, 
       9.9999999999996518e-06, 
       9.9999999999997874e-06, 
       1.0000000000000313e-05, 
       1.0000000000000275e-05, 
       1.0000000000000275e-05, 
       9.9999999999997145e-06, 
       1.0000000000000187e-05, 
       9.9999999999997332e-06, 
       1.0000000000000153e-05, 
       1.0000000000000424e-05, 
       9.9999999999998704e-06, 
       9.9999999999996739e-06, 
       1.0000000000000031e-05, 
       9.9999999999996620e-06, 
       9.9999999999999331e-06, 
       9.9999999999998636e-06, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       9.9276769143992826e-06, 
       8.9214736527310248e-06, 
       8.9600403999452289e-06, 
       6.9972294257189717e-06, 
       8.4057015001877455e-06, 
       9.3308453861784518e-06, 
       8.2355962587577931e-06, 
       7.4347161580137081e-06, 
       6.3564941087378608e-06, 
       6.6098969561254629e-06, 
       8.8670988397329206e-06, 
       8.7130972391972014e-06, 
       7.6554779559906427e-06, 
       8.4184260702981578e-06, 
       7.8738939073247051e-06, 
       8.5206363862676961e-06, 
       7.1890470396666863e-06, 
       7.9390841266504732e-06, 
       6.0104460260625855e-06, 
       7.0454590767425861e-06, 
       6.0539712200894748e-06, 
       6.0292957950127514e-06, 
       7.6382707738750426e-06, 
       5.7668944303463087e-06, 
       7.5377624762324105e-06, 
       6.2732718608679188e-06, 
       6.0403016421698422e-06, 
       6.6479441112793904e-06, 
       4.7257396507225487e-06, 
       5.0446984949984104e-06, 
       4.4321483554784521e-06, 
       5.5319825328363708e-06, 
       4.3373982845981741e-06, 
       5.6717935704369968e-06, 
       4.2240003896380582e-06, 
       4.9598137452493484e-06, 
       5.7315597150291972e-06, 
       5.5970657068869918e-06, 
       4.6554052538427468e-06, 
       5.0651417314833270e-06, 
       4.0918340727629949e-06, 
       4.2247385020999436e-06, 
       4.6038208711109434e-06, 
       3.2235609800667443e-06, 
       3.1285179284457663e-06, 
       4.1993287195063803e-06, 
       3.0097698467174810e-06, 
       4.1551966185885370e-06, 
       3.2150773190843271e-06, 
       4.0884734528614543e-06, 
       4.1444937401816535e-06, 
       3.8676464986809128e-06, 
       3.9255086534156559e-06, 
       2.9488822684155869e-06, 
       2.7300435072767274e-06, 
       3.5230053979075721e-06, 
       2.8564573635535870e-06, 
       2.8916221191763487e-06, 
       2.3873418313482731e-06, 
       2.8452084361953237e-06, 
       2.4921849021287432e-06, 
       2.2786364864845360e-06, 
       1.8644900725049101e-06, 
       1.8805378072745463e-06, 
       1.7178166346072060e-06, 
       1.7072389730096216e-06, 
       1.4521989448006111e-06, 
       1.1210431561524651e-06, 
       1.0472480871178415e-06, 
       9.5638469964008220e-07, 
       8.8095299923467385e-07, 
       6.7058215537446876e-07, 
       5.6483898759171525e-07, 
       3.0146074863636613e-07, 
       3.9406941389153455e-07, 
       2.8861348174430666e-07, 
       2.8346666350137537e-07, 
       2.0873669967861465e-07, 
       1.2245229840726787e-08, 
       1.6768000000000001e+00, 
       5.1641999999999999e-01, 
       4.1382999999999998e-01, 
       3.7114999999999998e-01, 
       2.4598630519789147e-01, 
       2.7560000000000001e-01, 
       7.1179999999999993e-02, 
       6.1280000000000001e-02, 
       4.5159999999999999e-02, 
       4.0480000000000002e-02, 
       3.7470000000000003e-02, 
       3.7069999999999999e-02, 
       3.6490000000000002e-02, 
       2.4379999999999999e-02, 
       2.1649999999999999e-02, 
       1.6919100290757366e-02, 
       1.9230000000000001e-02, 
       1.3609297575554020e-02, 
       1.2763511856205752e-02, 
       1.2930000000000001e-02, 
       1.2699999999999999e-02, 
       1.0970000000000001e-02, 
       1.0877073284572179e-02, 
       8.3453599048365513e-03, 
       7.3400000000000002e-03, 
       6.8599999999999998e-03, 
       6.3099999999999996e-03, 
       5.6619807848686454e-03, 
       5.6611068538185228e-03, 
       5.0937252062072062e-03, 
       5.8500000000000002e-03, 
       3.8723575555701987e-03, 
       5.3899999999999998e-03, 
       4.6899999999999997e-03, 
       2.7755230992037357e-03, 
       2.8925319854490442e-03, 
       3.7799999999999999e-03, 
       2.8467780380650328e-03, 
       2.8229930855291322e-03, 
       3.0265170195963847e-03, 
       3.6200000000000000e-03, 
       2.7200641051918287e-03, 
       2.4350269406220870e-03, 
       2.3951159060627606e-03, 
       3.1700000000000001e-03, 
       3.0000000000000001e-03, 
       2.1566458249048541e-03, 
       2.9499999999999999e-03, 
       2.8320556956046273e-03, 
       2.3901365119155470e-03, 
       2.7000000000000001e-03, 
       2.5600000000000002e-03, 
       1.5216239435853550e-03, 
       1.5010567723298686e-03, 
       1.5046083924101031e-03, 
       1.6113451360524626e-03, 
       1.6108183401981411e-03, 
       1.5619738963180364e-03, 
       1.8600000000000001e-03, 
       1.2413671622961934e-03, 
       1.6900000000000001e-03, 
       1.1780629944789897e-03, 
       1.6000000000000001e-03, 
       1.5499999999999999e-03, 
       1.4800000000000000e-03, 
       1.4126173984728781e-03, 
       1.4100000000000000e-03, 
       1.3799999999999999e-03, 
       1.0270133643677396e-03, 
       1.2899999999999999e-03, 
       9.9890919804875846e-04, 
       1.2600000000000001e-03, 
       9.2726750800733787e-04, 
       8.8585603632539123e-04, 
       1.1999999999999999e-03, 
       9.4182386532185101e-04, 
       9.3446107676674296e-04, 
       8.6986385266733595e-04, 
       8.7655265964070030e-04, 
       8.7343326299490251e-04, 
       8.2358774904712801e-04, 
       1.0183723058731088e-03, 
       1.0174618776608048e-03, 
       1.0700000000000000e-03, 
       1.0399999999999999e-03, 
       9.7000000000000005e-04, 
       9.6000000000000002e-04, 
       9.3999999999999997e-04, 
       9.3999999999999997e-04, 
       8.9999999999999998e-04, 
       8.8000000000000003e-04, 
       6.6131741401725204e-04, 
       6.4928667044860603e-04, 
       6.2789883250667601e-04, 
       7.9000000000000001e-04, 
       7.6000000000000004e-04, 
       7.6000000000000004e-04, 
       7.6000000000000004e-04, 
       5.9634959771494474e-04, 
       7.2999999999999996e-04, 
       7.2999999999999996e-04, 
       5.2331766276262205e-04, 
       7.1000000000000002e-04, 
       5.9862731465607421e-04, 
       5.9779752173464199e-04, 
       5.1186591180592527e-04, 
       5.5690123962899346e-04, 
       6.8999999999999997e-04, 
       5.4295542940330894e-04, 
       5.2177512040737920e-04, 
       6.6000000000000000e-04, 
       4.9977063346190714e-04, 
       4.8669255186217368e-04, 
       4.8663574669448946e-04, 
       5.6555962414255522e-04, 
       5.6555962414255522e-04, 
       5.8214271549757439e-04, 
       4.7171612356060452e-04, 
       4.4341697038978347e-04, 
       4.7189578980749513e-04, 
       5.8000000000000000e-04, 
       5.4011132164644734e-04, 
       4.0619385423640872e-04, 
       4.0066018258212141e-04, 
       4.0434115514577867e-04, 
       3.6526073369289210e-04, 
       4.9221561879064899e-04, 
       4.8805361745649290e-04, 
       3.4567388253694333e-04, 
       3.4800682751414563e-04, 
       3.7336207178047720e-04, 
       3.3078554701819365e-04, 
       3.4618968938920366e-04, 
       3.2997652781516993e-04, 
       4.4999999999999999e-04, 
       3.3714940952784959e-04, 
       4.1936404886310316e-04, 
       3.4905150524750281e-04, 
       3.1894306471750943e-04, 
       4.1640319354152729e-04, 
       3.1910995128305090e-04, 
       3.1949749035901145e-04, 
       3.8461960341498323e-04, 
       3.8459937917686017e-04, 
       3.1206459231754920e-04, 
       3.0109596168038190e-04, 
       4.0999999999999999e-04, 
       3.1426843645236083e-04, 
       2.8444718222922461e-04, 
       2.8128276445245809e-04, 
       2.8842697941288998e-04, 
       2.8655576054174121e-04, 
       3.6370056247452117e-04, 
       3.8998653306694707e-04, 
       3.8999999999999999e-04, 
       3.6023995941403394e-04, 
       3.8000000000000002e-04, 
       2.7775061216462314e-04, 
       2.6601917250873884e-04, 
       2.6506343627055945e-04, 
       2.7300779726484336e-04, 
       2.6014628160666355e-04, 
       2.8942224559087474e-04, 
       2.7942794331434049e-04, 
       2.6400856264205468e-04, 
       2.5429174054891876e-04, 
       3.4000000000000002e-04, 
       2.7114859861095104e-04, 
       2.6222691416255051e-04, 
       3.1660008516397884e-04, 
       2.3559142637683757e-04, 
       3.3000000000000000e-04, 
       3.1356699598810258e-04, 
       2.4381791925552252e-04, 
       3.2000000000000003e-04, 
       3.2000000000000003e-04, 
       2.3669530305334046e-04, 
       2.9716501145289685e-04, 
       2.5357362956049298e-04, 
       2.2769041501651902e-04, 
       2.2328154968264302e-04, 
       2.9999999999999997e-04, 
       2.1733480299824208e-04, 
       2.0988274958516311e-04, 
       2.1201539315427644e-04, 
       2.9000000000000000e-04, 
       2.0645811463053187e-04, 
       2.0000000000002446e-04, 
       2.0607149296629815e-04, 
       2.0358337292124119e-04, 
       1.9663808352323310e-04, 
       1.9547861151905776e-04, 
       2.6326379378830680e-04, 
       2.7044064550301659e-04, 
       1.9468585055608108e-04, 
       2.0844192208560572e-04, 
       1.8652982378492198e-04, 
       2.5000000000000001e-04, 
       2.5000000000000001e-04, 
       2.0106652609994485e-04, 
       1.7789354614226090e-04, 
       1.7999999999998901e-04, 
       1.7637053507828666e-04, 
       1.9413057139118013e-04, 
       1.9393750936839033e-04, 
       1.7317967082008492e-04, 
       1.7000000000002470e-04, 
       1.9768251742299464e-04, 
       1.7059839547565799e-04, 
       1.8621092030117562e-04, 
       1.6893241023915442e-04, 
       1.8470034523232002e-04, 
       2.3000000000000001e-04, 
       2.3000000000000001e-04, 
       2.3000000000000001e-04, 
       1.6862007395843811e-04, 
       2.2000000000000001e-04, 
       2.2000000000000001e-04, 
       1.6905726242723751e-04, 
       1.6248410552047860e-04, 
       1.6925930610250630e-04, 
       2.1000000000000001e-04, 
       2.1000000000000001e-04, 
       2.1000000000000001e-04, 
       2.1000000000000001e-04, 
       2.1000000000000001e-04, 
       1.5349956926387037e-04, 
       1.4252037226299830e-04, 
       1.4208834255520730e-04, 
       2.0000000000000001e-04, 
       2.0000000000000001e-04, 
       1.4661472487471143e-04, 
       1.5996109747741101e-04, 
       1.4167795618556271e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.3474751238052969e-04, 
       1.4415543713572695e-04, 
       1.3393228561587800e-04, 
       1.6000000000001891e-04, 
       1.3680789649364819e-04, 
       1.4255600909036291e-04, 
       1.8000000000000001e-04, 
       1.8000000000000001e-04, 
       1.8000000000000001e-04, 
       1.2826568084658131e-04, 
       1.4291083563379114e-04, 
       1.7000000000004663e-04, 
       1.2568276489903371e-04, 
       1.3178635363618110e-04, 
       1.2186148945234961e-04, 
       1.2999999999995589e-04, 
       1.2879618145486221e-04, 
       1.7000000000000001e-04, 
       1.7000000000000001e-04, 
       1.2097025540466503e-04, 
       1.1914878791780433e-04, 
       1.2033340346009163e-04, 
       1.6000000000000001e-04, 
       1.1420738737952349e-04, 
       1.1283963797625469e-04, 
       1.1000000000000044e-04, 
       1.0967996329608863e-04, 
       1.2812422817087810e-04, 
       1.2961637125433751e-04, 
       1.1875361978764619e-04, 
       1.3281345689030116e-04, 
       1.4999999999999999e-04, 
       1.4999999999999999e-04, 
       1.1366156689966042e-04, 
       1.0871830976829956e-04, 
       1.1447339604716639e-04, 
       1.1000000000004498e-04, 
       1.0435397961434781e-04, 
       1.1564413828886771e-04, 
       1.0000000000000213e-04, 
       1.1461197219250756e-04, 
       1.3999999999999999e-04, 
       1.0040586467764777e-04, 
       1.1252481981754288e-04, 
       1.0951702489818630e-04, 
       1.0936720826743502e-04, 
       9.8075020491742633e-05, 
       9.3673145454986945e-05, 
       1.2284022501393469e-04, 
       1.2992680729989375e-04, 
       1.2991104638424312e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       9.5017386849256515e-05, 
       9.9628261775951707e-05, 
       9.5265258572458994e-05, 
       9.1074110401061071e-05, 
       9.9249933838426005e-05, 
       8.9999999999997973e-05, 
       1.1644659225540634e-04, 
       9.8265422289696723e-05, 
       9.8157689146640892e-05, 
       1.0221869074255284e-04, 
       9.0175959454846298e-05, 
       8.8363171301259480e-05, 
       9.0000000000003204e-05, 
       8.6513004813478517e-05, 
       8.6376910144344955e-05, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       8.4910396588031836e-05, 
       8.6303704158404820e-05, 
       8.6312507848137495e-05, 
       9.0814339125707783e-05, 
       8.5856237159871466e-05, 
       8.4923436292185973e-05, 
       8.4768989290665974e-05, 
       8.3514236766920862e-05, 
       8.5595949986654808e-05, 
       8.7115129572176429e-05, 
       8.4519952218010410e-05, 
       1.1341256634033527e-04, 
       1.0042167120100659e-04, 
       1.1229409600703813e-04, 
       8.1810869358911565e-05, 
       9.2203561255744461e-05, 
       1.0786006974930761e-04, 
       9.6914653774837097e-05, 
       8.9999999999996631e-05, 
       7.9999999999995358e-05, 
       8.3963438479949732e-05, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       8.1324132580347792e-05, 
       7.7723800571588637e-05, 
       9.4685760602060317e-05, 
       8.5240481314107931e-05, 
       8.0513901395703555e-05, 
       9.0000000000000372e-05, 
       7.4766200919708641e-05, 
       8.8881337061543178e-05, 
       7.9844727324863575e-05, 
       7.8831853948302144e-05, 
       7.5739797625197825e-05, 
       8.2204264767937732e-05, 
       8.8842686480300503e-05, 
       7.5124829484863738e-05, 
       7.4004126313657510e-05, 
       1.0000000000000000e-04, 
       1.0000000000000000e-04, 
       7.4913040616924936e-05, 
       7.4631825873718531e-05, 
       7.6914348510389156e-05, 
       7.0489702228121405e-05, 
       8.9999999999995032e-05, 
       7.4066777775357972e-05, 
       7.6860194763091091e-05, 
       6.8090620690681083e-05, 
       6.9811404197783587e-05, 
       7.0383318658959903e-05, 
       6.6108421526585336e-05, 
       6.5839249867146346e-05, 
       6.8593479633896170e-05, 
       6.5857903246735846e-05, 
       6.7997271498290260e-05, 
       6.5818794749948116e-05, 
       6.5304478926461420e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       6.5116191827263741e-05, 
       6.6969302137990261e-05, 
       6.3520546486350003e-05, 
       6.9630262712291316e-05, 
       6.6611160947724389e-05, 
       6.5334570018810956e-05, 
       6.8562911070376692e-05, 
       6.4185782239491426e-05, 
       6.4068177718352564e-05, 
       6.1956489351114834e-05, 
       6.5355096389469039e-05, 
       7.3693831536948213e-05, 
       6.6298211332711321e-05, 
       5.8609513281343427e-05, 
       6.5592390976587983e-05, 
       7.0000143094329890e-05, 
       6.3496251289136529e-05, 
       7.7416149432809895e-05, 
       5.9481479247912861e-05, 
       6.0000000000004711e-05, 
       6.1286567111902532e-05, 
       5.8133913919595236e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       6.3983848535223179e-05, 
       5.6252980054447966e-05, 
       6.3483946792380840e-05, 
       7.3743804657105854e-05, 
       5.8362662288080200e-05, 
       6.2201350429539292e-05, 
       6.0000000000004928e-05, 
       5.4837975243834762e-05, 
       5.4190935865195813e-05, 
       5.4950801582622904e-05, 
       5.5518312910777733e-05, 
       5.3141322058460953e-05, 
       5.8084859452123288e-05, 
       6.0000000000002732e-05, 
       5.3843584577924200e-05, 
       7.2405053586645018e-05, 
       5.9999999999997603e-05, 
       6.1962475600750705e-05, 
       5.9636031509552471e-05, 
       5.8055745969866972e-05, 
       5.3716580919742125e-05, 
       7.0000000000003490e-05, 
       5.1225471827403264e-05, 
       5.3032041676239814e-05, 
       4.9999999999999440e-05, 
       5.9998263853224002e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       5.1695887208660406e-05, 
       4.9917651955771599e-05, 
       5.2521674722951197e-05, 
       5.0891744929096196e-05, 
       5.0424492771427094e-05, 
       5.2522299843086927e-05, 
       4.8832809213480156e-05, 
       5.0000000000000484e-05, 
       4.9411935334493937e-05, 
       4.9914667875922268e-05, 
       4.9441445165295810e-05, 
       5.4622423595324378e-05, 
       4.8517591409920700e-05, 
       5.3213438709148877e-05, 
       5.5632659828300991e-05, 
       4.8779028130356905e-05, 
       4.8226885575954597e-05, 
       4.7290604197927614e-05, 
       4.6322731955791945e-05, 
       6.1019334132064444e-05, 
       4.5610460412600764e-05, 
       4.6810932600579872e-05, 
       4.9432863350380618e-05, 
       6.1165618259026536e-05, 
       4.5239111913056748e-05, 
       4.9572770479235370e-05, 
       4.9567767780568799e-05, 
       6.2071041454853299e-05, 
       4.6065029709778942e-05, 
       4.5520660575144482e-05, 
       4.5641214366622427e-05, 
       4.4499774160447670e-05, 
       4.3388086270669392e-05, 
       4.9999999999999616e-05, 
       4.5490932624958557e-05, 
       5.5949191173713798e-05, 
       4.4407426305429539e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       5.6526734049517311e-05, 
       4.3722767955524346e-05, 
       4.3375138364128834e-05, 
       4.3353996364705500e-05, 
       4.6151747179215837e-05, 
       4.2885466563169962e-05, 
       4.9198988001138746e-05, 
       4.2807671337071456e-05, 
       4.2322456366005044e-05, 
       4.1006575193870757e-05, 
       4.4701650541767954e-05, 
       4.6185945198740273e-05, 
       3.9999999999997374e-05, 
       5.0273782151808583e-05, 
       5.1130186863394855e-05, 
       3.9999999999998106e-05, 
       4.1462095936688671e-05, 
       4.1577067602735772e-05, 
       4.9999999999999745e-05, 
       4.4309398436484000e-05, 
       5.5563110389812007e-05, 
       4.2614590876150446e-05, 
       3.9999999999998567e-05, 
       4.0188488893225141e-05, 
       5.4858757344799994e-05, 
       3.9500366384842609e-05, 
       4.4385849642023203e-05, 
       3.9999999999997957e-05, 
       3.9721569863311231e-05, 
       5.3549060009056154e-05, 
       4.2551218088839800e-05, 
       4.2390417927403567e-05, 
       3.8114998318829779e-05, 
       3.9481352883082236e-05, 
       3.7381668111036079e-05, 
       4.0000000000001867e-05, 
       3.7804824580582850e-05, 
       4.9897913969667595e-05, 
       3.7323918002612622e-05, 
       3.7074415561541644e-05, 
       3.8818932106846331e-05, 
       4.0160781085369481e-05, 
       3.9999999999998025e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       3.7086545818127610e-05, 
       4.1514868637532843e-05, 
       3.9203684999759020e-05, 
       3.5545170892441466e-05, 
       3.6244032488950672e-05, 
       4.7559060300528307e-05, 
       4.0000000000001630e-05, 
       4.0000000000001630e-05, 
       3.8469172159509515e-05, 
       3.6772492194250403e-05, 
       3.6498133237025635e-05, 
       4.5047886036370986e-05, 
       3.5523438895331447e-05, 
       3.7895942419542980e-05, 
       3.5135503375330759e-05, 
       3.8998510028733821e-05, 
       4.0686628072756452e-05, 
       4.0654927071157235e-05, 
       3.4080557107344325e-05, 
       3.4048217492594074e-05, 
       3.5203819138706939e-05, 
       3.7851728154034492e-05, 
       3.3698259039289606e-05, 
       3.4594701395675658e-05, 
       3.6435466527443812e-05, 
       3.3486346346068847e-05, 
       3.4733895007143607e-05, 
       3.1644301930794078e-05, 
       3.2550327773817797e-05, 
       3.9999999999997699e-05, 
       3.7487412107968229e-05, 
       3.3483136199464865e-05, 
       3.1361564915572966e-05, 
       3.3359073133855373e-05, 
       3.5013747517300329e-05, 
       3.2113825450448902e-05, 
       3.2997684166789597e-05, 
       3.4415338026954451e-05, 
       3.1403866066294734e-05, 
       4.1820979318686044e-05, 
       3.1179453632449354e-05, 
       4.1412175602386598e-05, 
       4.0118335767405143e-05, 
       3.3821636682938705e-05, 
       3.3793087846911833e-05, 
       3.3812847297667250e-05, 
       2.9999999999998439e-05, 
       3.0014896462251848e-05, 
       3.2965804877890337e-05, 
       4.0884159708154115e-05, 
       3.0000000000000024e-05, 
       3.5280730711421344e-05, 
       2.9999999999995623e-05, 
       2.9968480986509737e-05, 
       3.5657404605298736e-05, 
       2.9715071549215198e-05, 
       3.1546329800869204e-05, 
       3.0000000000001004e-05, 
       4.0736730489329173e-05, 
       4.0816164569913127e-05, 
       3.0000000000001136e-05, 
       3.3299638964219901e-05, 
       3.3299638964219901e-05, 
       3.0457422680714119e-05, 
       3.0864077802037112e-05, 
       2.9386880885571449e-05, 
       2.9380218081358039e-05, 
       3.5202693395530846e-05, 
       3.2847351552011229e-05, 
       2.9999999999998808e-05, 
       3.0136650401193868e-05, 
       3.0000000000001891e-05, 
       2.9689739846549140e-05, 
       2.8773922855604120e-05, 
       3.3700495527407158e-05, 
       3.8773547281792445e-05, 
       2.9999999999995491e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.9504829693468627e-05, 
       3.1272117616158080e-05, 
       2.8862227430549798e-05, 
       3.1291484810656107e-05, 
       3.9191897971157215e-05, 
       2.7977630063447355e-05, 
       2.7977630063447355e-05, 
       2.9999999999998578e-05, 
       2.8089975368071030e-05, 
       2.9185216845573686e-05, 
       2.8927608788804419e-05, 
       2.8151518352048091e-05, 
       2.7011750800167191e-05, 
       3.1465009182393138e-05, 
       3.2173722130937267e-05, 
       2.9919960533406326e-05, 
       2.9580842541345168e-05, 
       2.6692244642173586e-05, 
       2.7019188599007298e-05, 
       2.6846571113176940e-05, 
       2.8204457784433526e-05, 
       3.7440317362358582e-05, 
       2.8659394650376446e-05, 
       2.7038416658924865e-05, 
       2.9335476415507328e-05, 
       3.5522153832264638e-05, 
       2.9999999999998395e-05, 
       2.7210311059071359e-05, 
       2.6074942955201523e-05, 
       2.6297383547147306e-05, 
       2.9782364454003838e-05, 
       3.2457358833389846e-05, 
       2.8472570612034648e-05, 
       3.1872929181959594e-05, 
       2.7131565796937800e-05, 
       2.7971472503886519e-05, 
       2.7832361599668259e-05, 
       2.7832361599668259e-05, 
       2.5737932320108286e-05, 
       3.0000000000001793e-05, 
       2.6199921818010489e-05, 
       2.5682484190201157e-05, 
       2.9265385385010686e-05, 
       2.6228294859266784e-05, 
       2.5236763805001472e-05, 
       3.0046823587142986e-05, 
       2.5592180923732137e-05, 
       2.5356010336502280e-05, 
       2.5122134504469778e-05, 
       2.9999999999998110e-05, 
       2.7500125045985584e-05, 
       2.5584307744602768e-05, 
       3.2637883499349776e-05, 
       3.2657745312255911e-05, 
       2.4633513126311541e-05, 
       2.8998943625240472e-05, 
       2.5773881605407260e-05, 
       2.6807179885281498e-05, 
       2.4477250189912038e-05, 
       2.4371838282645147e-05, 
       2.5917938372508873e-05, 
       2.4535058597207485e-05, 
       2.4285911235302098e-05, 
       2.5317669587790486e-05, 
       2.5462588311319546e-05, 
       2.3941240359989158e-05, 
       2.3800505219905217e-05, 
       2.4961550181028645e-05, 
       2.4226066300122850e-05, 
       2.4226066300122850e-05, 
       2.3569946573050225e-05, 
       2.9256067872345432e-05, 
       2.9999999999996714e-05, 
       3.0742484125328481e-05, 
       2.5134000662371254e-05, 
       2.3306748978850969e-05, 
       2.5264107776272909e-05, 
       3.0492415851752670e-05, 
       2.7623986466856209e-05, 
       2.7343805356189461e-05, 
       2.6356052978239335e-05, 
       2.9797929900666918e-05, 
       2.4200857469102265e-05, 
       2.7367378383460666e-05, 
       3.1088971422759722e-05, 
       2.2556861236520565e-05, 
       2.3571041255271092e-05, 
       2.2427889211832239e-05, 
       2.7014212896991393e-05, 
       2.7137824175701920e-05, 
       2.2097546107119309e-05, 
       2.4573600077090218e-05, 
       2.6185930649968901e-05, 
       2.2131240997308618e-05, 
       3.0000000000003491e-05, 
       2.4382405017780020e-05, 
       2.2917351752852338e-05, 
       2.1993540383034172e-05, 
       2.4753475601988021e-05, 
       2.7106587321094513e-05, 
       2.7096223107290237e-05, 
       2.2311562620296933e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.9191556212557046e-05, 
       2.3708407126237797e-05, 
       2.3632543720058086e-05, 
       2.2178655585035640e-05, 
       2.1260303009147572e-05, 
       2.0751564665685468e-05, 
       2.7123291419799243e-05, 
       2.1293597600214950e-05, 
       2.7072748310821861e-05, 
       2.1259004476358883e-05, 
       2.4840474607486825e-05, 
       2.0977104654190915e-05, 
       2.0426691562162788e-05, 
       2.0575742866782545e-05, 
       2.7809262755418969e-05, 
       1.9999999999997217e-05, 
       2.7639306492202163e-05, 
       2.0000000000000588e-05, 
       2.2563138035948259e-05, 
       2.6665080149674556e-05, 
       2.0223797299616227e-05, 
       1.9864695338257338e-05, 
       2.6719377696700612e-05, 
       1.9999999999999548e-05, 
       2.2295558324945083e-05, 
       1.9999999999997274e-05, 
       2.2481410309963971e-05, 
       2.0182826491743358e-05, 
       2.1845276102154460e-05, 
       1.9639347126474737e-05, 
       1.9948019575927538e-05, 
       1.9408611324717893e-05, 
       2.3216819158676505e-05, 
       2.0846856718329786e-05, 
       2.0572601516417116e-05, 
       2.1603384716430804e-05, 
       2.2327988619413193e-05, 
       2.3107947036613048e-05, 
       1.9659599877273180e-05, 
       2.0000000000001469e-05, 
       2.6563136895194646e-05, 
       2.3651313797591950e-05, 
       1.9808176914537860e-05, 
       2.3612171717668531e-05, 
       1.9186151206555596e-05, 
       1.9479667288488301e-05, 
       2.6406639289927003e-05, 
       2.6422711248488157e-05, 
       1.9603888455501442e-05, 
       1.9322708244094596e-05, 
       1.9304301195852198e-05, 
       1.8966502313068078e-05, 
       2.1413140376842291e-05, 
       1.8885929402898659e-05, 
       1.9478075949216492e-05, 
       1.8997342556295698e-05, 
       1.9999999999998633e-05, 
       1.9412913706816398e-05, 
       1.8730084639634875e-05, 
       2.1030775585407879e-05, 
       2.0000000000004596e-05, 
       1.8805715509531297e-05, 
       1.8681991499424406e-05, 
       2.2313943334219730e-05, 
       1.8838987389853461e-05, 
       1.8991950648419524e-05, 
       2.4706036334314775e-05, 
       2.4699738671751328e-05, 
       1.8579274982661298e-05, 
       1.9826184433531350e-05, 
       2.0534360716990844e-05, 
       2.0993626785568987e-05, 
       2.2166591464208083e-05, 
       1.8359136074659026e-05, 
       2.5010955169093653e-05, 
       2.4129536331827622e-05, 
       2.2370575466761592e-05, 
       2.0000000000003234e-05, 
       1.8806277083723237e-05, 
       1.8467457477055713e-05, 
       1.9999999999999256e-05, 
       1.9999999999999256e-05, 
       2.0768642561885622e-05, 
       2.0000000000004098e-05, 
       1.7867416925220239e-05, 
       1.9999999999999012e-05, 
       2.0231731409536032e-05, 
       1.8182569317993935e-05, 
       2.1293705580291823e-05, 
       2.2836971537309227e-05, 
       2.2836971537309227e-05, 
       2.0008703215793480e-05, 
       1.9466270069855385e-05, 
       1.7519970751792677e-05, 
       2.4207997163155264e-05, 
       1.7991832038449499e-05, 
       1.9356879194243652e-05, 
       1.7826185487479829e-05, 
       1.7353968538952078e-05, 
       2.0000000000001177e-05, 
       2.3787675039106025e-05, 
       1.7309993086786362e-05, 
       2.0000000000002319e-05, 
       1.7139749191630588e-05, 
       1.9287464289845343e-05, 
       1.8062383567360180e-05, 
       2.2267660896859265e-05, 
       1.9999999999996559e-05, 
       2.0008378646672091e-05, 
       1.8804112972805669e-05, 
       1.6764587150742134e-05, 
       1.6669873460121549e-05, 
       1.7205393383691882e-05, 
       1.6931509532418365e-05, 
       1.6607785766890799e-05, 
       2.1880392553228619e-05, 
       1.7081082427052981e-05, 
       1.6969166942220725e-05, 
       1.7715228276346213e-05, 
       1.9963562514332652e-05, 
       2.2554646420987262e-05, 
       1.6758979672385730e-05, 
       1.6873519010243131e-05, 
       1.6349496012902706e-05, 
       1.8510212586384688e-05, 
       1.5938319162468304e-05, 
       1.6984179487034477e-05, 
       1.7788024916872645e-05, 
       1.6001874200859676e-05, 
       1.6743649814993181e-05, 
       1.7205551348619966e-05, 
       1.7885452909703484e-05, 
       1.6625200370722534e-05, 
       1.6373809606661409e-05, 
       1.6627832668215887e-05, 
       1.5976866250368946e-05, 
       1.9999999999996783e-05, 
       1.7363674046188015e-05, 
       1.6083037376418131e-05, 
       1.6343114636448679e-05, 
       1.8343407491848204e-05, 
       1.6925594702878127e-05, 
       2.0454950671941970e-05, 
       1.5686002403094944e-05, 
       1.5838518495045479e-05, 
       1.7486940480617817e-05, 
       1.9883389265775696e-05, 
       1.6327109040235535e-05, 
       2.0424938883841393e-05, 
       2.0427807483103429e-05, 
       1.5644484327526850e-05, 
       1.5867501230803267e-05, 
       1.5110751322616457e-05, 
       2.0000000000003580e-05, 
       1.7734450653983901e-05, 
       2.0134039715363126e-05, 
       2.0058162731625443e-05, 
       1.4624414869076079e-05, 
       1.5230245245118650e-05, 
       1.5085591842953761e-05, 
       1.5262591415855119e-05, 
       1.4564744570149252e-05, 
       1.7043605231382382e-05, 
       1.4589690644569201e-05, 
       1.4451209281903315e-05, 
       1.6149680792502952e-05, 
       1.8768797894612757e-05, 
       1.4374725904905995e-05, 
       1.5533388178468646e-05, 
       1.6532466198151173e-05, 
       1.5960653163557965e-05, 
       1.4355958387083369e-05, 
       1.4373606616565149e-05, 
       1.4898311729491209e-05, 
       1.5271969420058310e-05, 
       1.4278026984889705e-05, 
       1.5844494624281379e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.4160578560629806e-05, 
       1.6910400558404366e-05, 
       1.6848394678110017e-05, 
       1.7512948292476493e-05, 
       1.9141804144763431e-05, 
       1.5210383125229497e-05, 
       1.6827791530076173e-05, 
       1.4908028976569549e-05, 
       1.7143200018554293e-05, 
       1.3933984783339975e-05, 
       1.3911760376041158e-05, 
       1.4494828744928540e-05, 
       1.7697799065676122e-05, 
       1.4342058135619077e-05, 
       1.6863656364408230e-05, 
       1.6863656364408230e-05, 
       1.6681432592032161e-05, 
       1.3857273068689528e-05, 
       1.4292768698319954e-05, 
       1.3529282455952913e-05, 
       1.3492697305830638e-05, 
       1.5720480371830671e-05, 
       1.3714430654745951e-05, 
       1.4296436309647095e-05, 
       1.5712067622906465e-05, 
       1.4199112071642915e-05, 
       1.3623509582802912e-05, 
       1.7851914031471932e-05, 
       1.3667926439330561e-05, 
       1.6934056376821289e-05, 
       1.3197982690463027e-05, 
       1.3198984372531700e-05, 
       1.7281585869567818e-05, 
       1.4664548460781849e-05, 
       1.5365138404873848e-05, 
       1.3091265937794375e-05, 
       1.3487468102635314e-05, 
       1.3718333684757451e-05, 
       1.3211014606491409e-05, 
       1.3203394544383078e-05, 
       1.5328591659589319e-05, 
       1.2836279153562491e-05, 
       1.3595715093240521e-05, 
       1.7227964339966451e-05, 
       1.2613423965688773e-05, 
       1.3400348495384164e-05, 
       1.3141561166335034e-05, 
       1.2842273245148677e-05, 
       1.2594092440623859e-05, 
       1.2931833999465078e-05, 
       1.7021062370777087e-05, 
       1.3611366772337191e-05, 
       1.7233826626863242e-05, 
       1.2775253827489913e-05, 
       1.2775201272766793e-05, 
       1.2500637403403971e-05, 
       1.2418051838696583e-05, 
       1.2517041137607847e-05, 
       1.2572177143160616e-05, 
       1.2523782300879124e-05, 
       1.2636546356090033e-05, 
       1.4330806605078814e-05, 
       1.3659503941344594e-05, 
       1.2193318101691071e-05, 
       1.3655586328179575e-05, 
       1.3570150463719608e-05, 
       1.4457535782031519e-05, 
       1.3184220984711498e-05, 
       1.4747343942508893e-05, 
       1.4167866443020033e-05, 
       1.3379902545627577e-05, 
       1.3983982122698263e-05, 
       1.5124414135593321e-05, 
       1.4866846025021259e-05, 
       1.2566840745002747e-05, 
       1.4164853489324051e-05, 
       1.4131523120344591e-05, 
       1.2145750809411923e-05, 
       1.1686729010294344e-05, 
       1.5021649709840194e-05, 
       1.2559672032698739e-05, 
       1.5863959957828723e-05, 
       1.3760734668291225e-05, 
       1.5602265287517245e-05, 
       1.2046397081482098e-05, 
       1.1554788933308400e-05, 
       1.2024639719371994e-05, 
       1.5793341943915352e-05, 
       1.2763071187150709e-05, 
       1.2035133671108810e-05, 
       1.1619793695329978e-05, 
       1.1997264817923060e-05, 
       1.1645445387257533e-05, 
       1.1257305353587646e-05, 
       1.5141131320754851e-05, 
       1.1255050774316778e-05, 
       1.3068772519681541e-05, 
       1.2499371065885147e-05, 
       1.1328415363346918e-05, 
       1.1287572743084131e-05, 
       1.1309792768699568e-05, 
       1.1210186362251582e-05, 
       1.2814606962227572e-05, 
       1.2328694191987037e-05, 
       1.1259193589328646e-05, 
       1.5607550973367016e-05, 
       1.2391949261508307e-05, 
       1.1398385844271362e-05, 
       1.1578772042345190e-05, 
       1.1342472706858968e-05, 
       1.4065447134165465e-05, 
       1.1095780508374372e-05, 
       1.1567455883813949e-05, 
       1.0919929046678284e-05, 
       1.3410519812722531e-05, 
       1.4244859653893168e-05, 
       1.1172809410242831e-05, 
       1.1367458641059072e-05, 
       1.1202953053180991e-05, 
       1.0778074539157855e-05, 
       1.4527782135003634e-05, 
       1.0811134767029974e-05, 
       1.1799596959139493e-05, 
       1.2085154703886210e-05, 
       1.4355532454623739e-05, 
       1.1754192674693249e-05, 
       1.2645546693696847e-05, 
       1.1789674958006327e-05, 
       1.3386228659374884e-05, 
       1.4836574568707225e-05, 
       1.2551554549025422e-05, 
       1.0834831575800659e-05, 
       1.0595015724654419e-05, 
       1.3233519021148861e-05, 
       1.1067743051788599e-05, 
       1.0739846770578214e-05, 
       1.0905841990328247e-05, 
       1.1429733642794347e-05, 
       1.1555827423784605e-05, 
       1.0410066671676389e-05, 
       1.0367627954712632e-05, 
       1.1361049724090651e-05, 
       1.0305521611141474e-05, 
       1.0876987121034973e-05, 
       1.0290682116939284e-05, 
       1.0817939495735511e-05, 
       1.3487468102631545e-05, 
       1.0144824009627658e-05, 
       1.1341663846566092e-05, 
       1.0132381420194130e-05, 
       1.0430663587230592e-05, 
       1.1248338244145324e-05, 
       1.0865447622654365e-05, 
       1.2056174381251658e-05, 
       1.2657480016838313e-05, 
       9.9999999999997789e-06, 
       1.1227612670564297e-05, 
       1.0551922945587939e-05, 
       1.3697644880916474e-05, 
       1.1058724041859068e-05, 
       1.0864171766429975e-05, 
       9.9999999999998382e-06, 
       1.0316037602035974e-05, 
       1.1671117261079531e-05, 
       1.3335555846781680e-05, 
       9.9999999999997941e-06, 
       1.3310044361603757e-05, 
       1.0042033974282505e-05, 
       1.0646251574422470e-05, 
       1.2862325661298231e-05, 
       1.0622116247348954e-05, 
       1.0156858739603583e-05, 
       1.1503405839438515e-05, 
       1.0249497591828719e-05, 
       1.0634924780917165e-05, 
       1.1730443311164356e-05, 
       1.1725433434331973e-05, 
       1.0000000000000118e-05, 
       1.0561374453280059e-05, 
       1.0437783631662281e-05, 
       1.0563851329157979e-05, 
       1.0693107046668232e-05, 
       1.0000000000000243e-05, 
       1.1046186347863405e-05, 
       1.0374498060985676e-05, 
       9.9999999999997061e-06, 
       9.9999999999997061e-06, 
       1.0082154656522444e-05, 
       1.2285868583690549e-05, 
       1.1945879343491163e-05, 
       1.0000000000000001e-05, 
       1.1917887384520912e-05, 
       9.9999999999997975e-06, 
       9.9999999999997975e-06, 
       1.0000000000000509e-05, 
       1.0000000000000260e-05, 
       1.0000000000000260e-05, 
       1.0716597322411585e-05, 
       9.9999999999995722e-06, 
       1.0307682682769742e-05, 
       1.0000000000000543e-05, 
       9.9999999999999839e-06, 
       1.1010192905863880e-05, 
       1.0651750099817600e-05, 
       1.0967491772412170e-05, 
       1.0539570125271512e-05, 
       9.9999999999999212e-06, 
       1.0238980082244761e-05, 
       1.0428046361947348e-05, 
       1.0581551584182459e-05, 
       1.0521791774912581e-05, 
       1.0155431128245339e-05, 
       9.9999999999997501e-06, 
       1.0017465930359139e-05, 
       1.1252572208021010e-05, 
       1.0586834029076454e-05, 
       1.0457992382191283e-05, 
       1.0457316590264931e-05, 
       1.0000000000000089e-05, 
       1.1049761317512434e-05, 
       1.1063789652223144e-05, 
       1.0019487777920026e-05, 
       1.0584302017696919e-05, 
       9.9999999999997908e-06, 
       9.9999999999997908e-06, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       7.8647112849786019e-06, 
       5.3918428906441289e-06, 
       4.4779771707978103e-06, 
       4.3382849084542031e-06, 
       1.6832590046073050e-06, 
       1.1677365667767590e-06, 
       6.5464032593233605e-07, 
       5.4489728975999753e-07, 
       3.3057967358587582e-07, 
       4.8700000000000002e-03, 
       2.2800067089002759e-03, 
       1.5000000000000000e-03, 
       1.1999999999999999e-03, 
       1.0800000000000000e-03, 
       8.0000000000000004e-04, 
       2.5432833591343596e-04, 
       2.1000000000000001e-04, 
       1.8000000000000001e-04, 
       1.2614958679615747e-04, 
       1.1834516968595136e-04, 
       1.2999999999999999e-04, 
       1.2000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       1.1000000000000000e-04, 
       9.9112194058782873e-05, 
       6.9919728962168168e-05, 
       6.9902033593255316e-05, 
       5.3717692184977326e-05, 
       6.9999999999999994e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       4.2960813801287238e-05, 
       4.0760731318087878e-05, 
       3.5890118751212446e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.6724848476250366e-05, 
       2.6306839803605131e-05, 
       2.6087328165969658e-05, 
       2.5131372454877677e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.0058244061163230e-05, 
       1.8398914607636817e-05, 
       1.8388331895300730e-05, 
       1.5737160983390079e-05, 
       1.7810150428029185e-05, 
       2.0419037322826659e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.4942522878921874e-05, 
       1.4163911507218445e-05, 
       1.4057837825740613e-05, 
       1.1073221815805074e-05, 
       1.1705376277585410e-05, 
       1.0012521990011182e-05, 
       1.0124078272641884e-05, 
       1.0744121532851539e-05, 
       1.1742659850014562e-05, 
       1.1614462115282566e-05, 
       8.1939742174658654e-06, 
       8.1833717167189554e-06, 
       8.3697321313647369e-06, 
       8.5699736445673027e-06, 
       7.9560357175029567e-06, 
       7.9025234008052000e-06, 
       8.0396465870489176e-06, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       9.1397405083220960e-06, 
       6.9075957633964196e-06, 
       6.8576606267723811e-06, 
       6.9833115219091995e-06, 
       6.6614621542316807e-06, 
       6.6938208965713218e-06, 
       6.4093362080313739e-06, 
       6.0767279160024249e-06, 
       5.9398241637203295e-06, 
       6.6824832367931063e-06, 
       5.5417505020845502e-06, 
       6.8689870309759069e-06, 
       5.7042248057659961e-06, 
       5.2508188458905792e-06, 
       5.6332494780071564e-06, 
       4.8084176849401377e-06, 
       4.9413720078873615e-06, 
       4.7475168603198393e-06, 
       4.7468253712181261e-06, 
       4.4272343159771057e-06, 
       6.0097994586177987e-06, 
       6.0185621807312714e-06, 
       4.3900908058358539e-06, 
       4.4723672820577840e-06, 
       4.4971112910730152e-06, 
       4.4971112910730152e-06, 
       4.3037414762997439e-06, 
       4.1134270887317063e-06, 
       4.7456714143049886e-06, 
       4.6712543900327493e-06, 
       3.9522579793540212e-06, 
       4.0676237858412570e-06, 
       3.6679896208244717e-06, 
       3.7527300772082450e-06, 
       3.3422074151796619e-06, 
       4.1036447154383807e-06, 
       3.1985589266976676e-06, 
       3.1137600123078054e-06, 
       3.0612774917133335e-06, 
       4.0214007049372836e-06, 
       3.1041382358140926e-06, 
       3.7013683248663611e-06, 
       3.6963459549802337e-06, 
       3.0117153426417565e-06, 
       2.9221459434933440e-06, 
       2.9221459434933440e-06, 
       2.8283225857125967e-06, 
       2.8072506991466326e-06, 
       2.5346132034046632e-06, 
       2.7857583100391412e-06, 
       2.6049068870382515e-06, 
       2.5502175464830456e-06, 
       2.4822855690439085e-06, 
       3.2639689889364743e-06, 
       2.3672733741342127e-06, 
       2.2946728631185666e-06, 
       2.2695655757273502e-06, 
       2.2175156504254322e-06, 
       2.3075325410813941e-06, 
       2.5621692738600066e-06, 
       2.0091975056906337e-06, 
       2.0091975056906337e-06, 
       1.9532635649223557e-06, 
       1.9289087801578894e-06, 
       2.4273141307617603e-06, 
       1.9082043643515313e-06, 
       1.8989640274380810e-06, 
       2.5815012181800044e-06, 
       1.8842271482719631e-06, 
       2.1884383784239724e-06, 
       2.0590914335602073e-06, 
       2.5261256645845715e-06, 
       1.9681239688197191e-06, 
       2.4775498732004812e-06, 
       1.8161848238891322e-06, 
       1.9001450765774576e-06, 
       2.2212856497307918e-06, 
       2.3898473569249983e-06, 
       1.8385273170975891e-06, 
       2.2293876524677399e-06, 
       1.7401992889472876e-06, 
       1.6328084843931155e-06, 
       1.6059688306867012e-06, 
       1.7010144948532279e-06, 
       1.5919477905845017e-06, 
       1.6760525371688950e-06, 
       1.6165267374513807e-06, 
       1.5672325901393067e-06, 
       1.5337997115150326e-06, 
       1.6135805964507537e-06, 
       1.5534609798250411e-06, 
       2.1534318918718258e-06, 
       1.6002877817284642e-06, 
       1.8261068926552727e-06, 
       1.5928491425372567e-06, 
       1.6181121056680439e-06, 
       1.4279252508681194e-06, 
       1.5170080367837477e-06, 
       1.4098477658962358e-06, 
       1.4017915266285438e-06, 
       1.4589349832739293e-06, 
       1.4494014483437918e-06, 
       1.4109683044788175e-06, 
       1.4601116561129850e-06, 
       1.4417215955640911e-06, 
       1.3097939538030513e-06, 
       1.7061104736072293e-06, 
       1.6368945523076984e-06, 
       1.3750529019272454e-06, 
       1.3314776790155002e-06, 
       1.3170852450275343e-06, 
       1.2590828352492033e-06, 
       1.2905957631034034e-06, 
       1.2411395480896739e-06, 
       1.2141961975913281e-06, 
       1.2111878545746250e-06, 
       1.6040829801933439e-06, 
       1.2169141386605042e-06, 
       1.5814120166654337e-06, 
       1.5551936569743640e-06, 
       1.2445735250153835e-06, 
       1.1752835618881358e-06, 
       1.1462212954296164e-06, 
       1.1668833706132307e-06, 
       1.5869480697887829e-06, 
       1.1332165260527875e-06, 
       1.1395384443069783e-06, 
       1.1410186168695212e-06, 
       1.4095923973585548e-06, 
       1.4083994918522709e-06, 
       1.1261019248431757e-06, 
       1.1390217500161753e-06, 
       1.2268036979224070e-06, 
       1.1878875660183000e-06, 
       1.1266072137598194e-06, 
       1.3145636716878417e-06, 
       1.1149104293934902e-06, 
       1.0370449872187075e-06, 
       1.0671095468501128e-06, 
       1.0852729903569755e-06, 
       1.1531022000172839e-06, 
       1.1971321246393756e-06, 
       1.0366182832323851e-06, 
       1.3113549650198791e-06, 
       1.2984169074711275e-06, 
       1.0250483239477515e-06, 
       1.2685995482901698e-06, 
       1.1732213973684467e-06, 
       1.0734044952711292e-06, 
       1.0661093224983895e-06, 
       1.0632613970902311e-06, 
       1.0136915340685956e-06, 
       1.1382866286921644e-06, 
       1.1967338272879567e-05, 
       1.6915544728815255e-06, 
       8.0450399999996893e+00, 
       1.5102092344719256e+00, 
       6.3052043386419909e-01, 
       6.3027190683252610e-01, 
       4.5585999999995519e-01, 
       4.1571000000004482e-01, 
       3.2622000000004786e-01, 
       2.9854999999997256e-01, 
       8.5017127084974081e-02, 
       6.9662063044645464e-02, 
       6.7500923034176846e-02, 
       7.9912925338734744e-02, 
       8.3499999999997299e-02, 
       8.0429999999996546e-02, 
       7.3330000000000187e-02, 
       6.6635116671311070e-02, 
       4.9678395767206805e-02, 
       5.2194291683707844e-02, 
       5.0700430514062853e-02, 
       4.5980652489027940e-02, 
       3.8852677198868368e-02, 
       4.6380000000001399e-02, 
       3.7124711441699952e-02, 
       3.4829184944516471e-02, 
       4.2141964380685744e-02, 
       3.9779999999998511e-02, 
       2.8836871031646834e-02, 
       2.7346917105562352e-02, 
       2.4978969253369728e-02, 
       2.4977696134518040e-02, 
       3.0450519695192545e-02, 
       3.0947418055604470e-02, 
       2.1811858986225948e-02, 
       2.1778154529048611e-02, 
       2.3375291041355436e-02, 
       2.3361999715268939e-02, 
       2.8643268490474744e-02, 
       2.4040000000004950e-02, 
       2.3595756163855980e-02, 
       2.0124697541815798e-02, 
       2.2466902821482896e-02, 
       1.9141297923069737e-02, 
       2.1953424469820258e-02, 
       1.6443603972986810e-02, 
       2.0054726855769162e-02, 
       1.9991836623552660e-02, 
       1.3924441871189698e-02, 
       1.4076527427018740e-02, 
       1.3647499814346299e-02, 
       1.6491127339534124e-02, 
       1.7440000000002398e-02, 
       1.2845599386154885e-02, 
       1.3889626849855230e-02, 
       1.6470000000001046e-02, 
       1.2159077284451475e-02, 
       1.3493711459941075e-02, 
       1.2053011854260717e-02, 
       1.3900026092923670e-02, 
       1.1031298302007402e-02, 
       1.4074660869092722e-02, 
       1.4049999999999344e-02, 
       1.2305848126526638e-02, 
       1.0211681724059483e-02, 
       9.2600269409717634e-03, 
       9.3557877619183331e-03, 
       8.4520279274782010e-03, 
       8.4509147918548615e-03, 
       9.8590644489637350e-03, 
       1.0050000000002541e-02, 
       8.3249023131672702e-03, 
       9.0144032342984026e-03, 
       9.1013728644659415e-03, 
       7.7804818291646247e-03, 
       9.1173241123075552e-03, 
       7.6201730727199915e-03, 
       7.6197843017261502e-03, 
       6.5837328261816423e-03, 
       7.5414804467914117e-03, 
       8.3806000885695165e-03, 
       7.4728497163251955e-03, 
       6.3310780340135675e-03, 
       7.9638366959054671e-03, 
       6.1535124088335823e-03, 
       8.4587816087514575e-03, 
       6.2500951706438617e-03, 
       6.4252824928326315e-03, 
       6.8868022816102473e-03, 
       6.8856163150056438e-03, 
       6.2109705881950921e-03, 
       8.3400000000003350e-03, 
       8.3000585701522769e-03, 
       6.1596087744463131e-03, 
       7.9771728179224496e-03, 
       8.0877797029879801e-03, 
       7.9820918119618643e-03, 
       6.8062629712314464e-03, 
       7.6402262249203236e-03, 
       7.8319711677719370e-03, 
       6.4921524938037697e-03, 
       6.0325452824095639e-03, 
       6.0271868255321074e-03, 
       7.1834222111314197e-03, 
       6.1547079139979491e-03, 
       7.0808702667713346e-03, 
       6.8925081939619458e-03, 
       6.8415168030109960e-03, 
       5.6439778298486322e-03, 
       6.7199999999998433e-03, 
       6.5578329184788862e-03, 
       6.5475312541876645e-03, 
       6.4187950962505758e-03, 
       6.3322794557790418e-03, 
       6.2315328954581422e-03, 
       6.1956397940844208e-03, 
       4.9134701514221332e-03, 
       4.9087525052121149e-03, 
       4.7416885433351269e-03, 
       5.9146763797479187e-03, 
       6.0446090478922920e-03, 
       4.9863001521728871e-03, 
       5.9663424435565677e-03, 
       5.1005457394913715e-03, 
       5.8900000000002267e-03, 
       5.8406588260202716e-03, 
       4.2029703382669998e-03, 
       5.8360597600046549e-03, 
       5.7370148335486159e-03, 
       4.1503334277825507e-03, 
       4.6475127037543815e-03, 
       4.7051889946399294e-03, 
       4.6435729797841283e-03, 
       5.5537409462860041e-03, 
       5.6299999999999311e-03, 
       5.6200000000003322e-03, 
       5.6108515189776153e-03, 
       5.6100000000000056e-03, 
       4.4833314261677505e-03, 
       4.4787627899237955e-03, 
       5.3070086163460361e-03, 
       4.6249138603349834e-03, 
       5.3186020284332648e-03, 
       4.7218320461153360e-03, 
       5.1768255126553066e-03, 
       5.0752490843158542e-03, 
       4.3664906879989102e-03, 
       5.0047583139203695e-03, 
       4.7872272541596012e-03, 
       4.7795274543678519e-03, 
       4.8753105325883180e-03, 
       3.9667747624743252e-03, 
       4.0834601586299353e-03, 
       4.7194995279174479e-03, 
       3.3633980146553712e-03, 
       4.4986340646213140e-03, 
       4.5383517668322040e-03, 
       4.4200000000004808e-03, 
       4.3699999999996658e-03, 
       3.5632099786965668e-03, 
       3.5265668121967383e-03, 
       4.2399999999997978e-03, 
       2.9920742735207598e-03, 
       4.1937054610074074e-03, 
       2.9648248352641695e-03, 
       3.4805423859042084e-03, 
       3.4459824149985050e-03, 
       3.9812712784127423e-03, 
       4.0490275619448086e-03, 
       3.5147477042229941e-03, 
       3.4951253478704256e-03, 
       3.9001540892946870e-03, 
       3.2734531523206031e-03, 
       3.2828771658553130e-03, 
       3.8102895686142595e-03, 
       3.1029704299248479e-03, 
       3.1019111371112331e-03, 
       2.6763179314531808e-03, 
       2.6742245538194892e-03, 
       3.7699999999996491e-03, 
       3.0661751699877513e-03, 
       3.7199999999999352e-03, 
       3.7093722671759366e-03, 
       2.9620523661433198e-03, 
       2.9610874038662987e-03, 
       2.9464774002881623e-03, 
       2.7729505322125528e-03, 
       2.7618147586587066e-03, 
       2.5935115512648850e-03, 
       3.4737978420013673e-03, 
       2.5432992376284930e-03, 
       2.6059899677907117e-03, 
       2.6059899677907117e-03, 
       3.2000134299412035e-03, 
       2.8336375349085632e-03, 
       3.2767362440520929e-03, 
       2.7599678407250411e-03, 
       3.2600719658559857e-03, 
       3.2899999999996023e-03, 
       2.8127115328946961e-03, 
       2.8063669623972316e-03, 
       2.6117278322433158e-03, 
       3.2200000000002275e-03, 
       2.9400000000004518e-03, 
       2.4305858864021456e-03, 
       3.1500000000000885e-03, 
       2.3385956276788518e-03, 
       2.6762270453347028e-03, 
       3.0084112461197109e-03, 
       2.2642114574429675e-03, 
       3.0039751024900155e-03, 
       2.4940260597677562e-03, 
       2.9566001525902991e-03, 
       2.1609247543580585e-03, 
       2.1588822067784208e-03, 
       2.1473726829793491e-03, 
       2.5334964893740835e-03, 
       2.9300000000000571e-03, 
       2.8725245742743286e-03, 
       2.5123100451131244e-03, 
       2.4659311887304460e-03, 
       2.8782272704443051e-03, 
       2.0513748850137089e-03, 
       2.1017027713759510e-03, 
       2.7211251533056542e-03, 
       2.2670803765853786e-03, 
       2.2670028812015871e-03, 
       2.7737206755488529e-03, 
       2.0580418070640961e-03, 
       2.7600000000003802e-03, 
       1.9750791385266964e-03, 
       2.6761692272419917e-03, 
       2.7299999999995292e-03, 
       2.2178108206497727e-03, 
       2.4600599829471474e-03, 
       2.2157060016125599e-03, 
       2.6399988900983355e-03, 
       1.9429106551148926e-03, 
       1.9033427673979019e-03, 
       2.5543620385740642e-03, 
       1.8579751874508359e-03, 
       1.8553436305305553e-03, 
       2.5799990701061656e-03, 
       2.5700000000003489e-03, 
       1.8854401905715570e-03, 
       2.0939170957528877e-03, 
       2.5312619792720371e-03, 
       2.5260768440158589e-03, 
       2.5200000000001163e-03, 
       2.1511958045741112e-03, 
       2.4192459372183468e-03, 
       2.5028771271624422e-03, 
       2.4995244566824674e-03, 
       2.4218622585549161e-03, 
       2.2669927281225204e-03, 
       2.2699999999996659e-03, 
       2.0398009900639822e-03, 
       2.4394672137045925e-03, 
       2.2939999928823572e-03, 
       2.4332643170941336e-03, 
       2.3427276241295568e-03, 
       1.8258896283203348e-03, 
       1.8056745273500145e-03, 
       2.3899999999999750e-03, 
       2.3819309585425747e-03, 
       1.7283115043241614e-03, 
       1.6895314014568270e-03, 
       1.7740558261157276e-03, 
       1.7080215205394298e-03, 
       2.2558819584066179e-03, 
       1.7847784740755398e-03, 
       1.7467559728304514e-03, 
       1.6955949045712329e-03, 
       1.8880806642052126e-03, 
       2.1661752769415441e-03, 
       2.3049267393778971e-03, 
       1.9180400301056525e-03, 
       2.3174074072395872e-03, 
       1.6528062300360539e-03, 
       2.2401895309149488e-03, 
       2.2832165615322714e-03, 
       1.7204750345019961e-03, 
       2.2135008361028542e-03, 
       2.2700000000004791e-03, 
       1.6051936004990838e-03, 
       1.6039533057537290e-03, 
       1.9096742527073237e-03, 
       2.2266421915729359e-03, 
       1.6298102271186644e-03, 
       1.5678069662617177e-03, 
       1.5655555987052270e-03, 
       1.5681794156946821e-03, 
       1.6038920269342083e-03, 
       2.1393920371074403e-03, 
       2.1800000000003778e-03, 
       1.8598882598486971e-03, 
       2.1580724764917888e-03, 
       1.6188856074503821e-03, 
       2.0927256499066712e-03, 
       2.0513070862608891e-03, 
       2.1200000000002181e-03, 
       2.0906928544715663e-03, 
       2.1100000000002588e-03, 
       1.7309318807075640e-03, 
       1.7158000599232584e-03, 
       1.9206132034569909e-03, 
       1.4966354704157539e-03, 
       2.0600000000001646e-03, 
       1.6871753664911295e-03, 
       2.0473058556835021e-03, 
       1.7099512653058373e-03, 
       2.0289437608360804e-03, 
       2.0176442179995068e-03, 
       1.9181040103220556e-03, 
       1.9366957412522833e-03, 
       1.9461244693220717e-03, 
       2.0063594346941445e-03, 
       1.6628322536005702e-03, 
       1.4698997263811568e-03, 
       1.6236490657578274e-03, 
       1.4590212317863774e-03, 
       1.4234191823931084e-03, 
       1.4855360411583907e-03, 
       1.9783723834961974e-03, 
       1.4203977784325364e-03, 
       1.6186535645447164e-03, 
       1.9034159237977749e-03, 
       1.6075253782971664e-03, 
       1.9289438060777250e-03, 
       1.7203627006369658e-03, 
       1.7197423007509222e-03, 
       1.3878470782072341e-03, 
       1.3866460904434414e-03, 
       1.3772920661365383e-03, 
       1.5167786023666968e-03, 
       1.5160212184099188e-03, 
       1.8746036021060908e-03, 
       1.8699999999997792e-03, 
       1.8071572106252980e-03, 
       1.8483448144917406e-03, 
       1.8146065313403717e-03, 
       1.5347017181191454e-03, 
       1.5288494954866173e-03, 
       1.3203896454805188e-03, 
       1.7826852353935636e-03, 
       1.7875295794420570e-03, 
       1.2699680203323699e-03, 
       1.2558736950941613e-03, 
       1.4685817108934826e-03, 
       1.7581146195617900e-03, 
       1.6275545520797279e-03, 
       1.7321420937262857e-03, 
       1.7188044152816545e-03, 
       1.5637626437821834e-03, 
       1.4736585261326605e-03, 
       1.7142833436007753e-03, 
       1.5898035792796582e-03, 
       1.4101107212682147e-03, 
       1.6828169301036317e-03, 
       1.6359620795286952e-03, 
       1.6691843929568719e-03, 
       1.4207562679303858e-03, 
       1.2044268214813640e-03, 
       1.1928684434281760e-03, 
       1.3741730432671379e-03, 
       1.3743806473975240e-03, 
       1.6236444770857848e-03, 
       1.1573856177432137e-03, 
       1.6154642429390212e-03, 
       1.3085319210469153e-03, 
       1.3319760175267326e-03, 
       1.1613728434288458e-03, 
       1.5093248040852359e-03, 
       1.1757400418186087e-03, 
       1.1827176293615603e-03, 
       1.5926562504339991e-03, 
       1.1322180084913311e-03, 
       1.5800105157753295e-03, 
       1.5818905631670374e-03, 
       1.5315135336496054e-03, 
       1.5500000000002864e-03, 
       1.5140283904400132e-03, 
       1.1206776414070126e-03, 
       1.1284831635992728e-03, 
       1.5137397156679013e-03, 
       1.1320049986655604e-03, 
       1.1305719562407681e-03, 
       1.5050242913759292e-03, 
       1.1243938624824765e-03, 
       1.4091755229554441e-03, 
       1.4091755229554441e-03, 
       1.0359687625384605e-03, 
       1.3188611305463304e-03, 
       1.4499999999998939e-03, 
       1.4440161361697567e-03, 
       1.3169698453123892e-03, 
       1.4109098563604866e-03, 
       1.4320711260814335e-03, 
       1.4137350781321077e-03, 
       1.4026805155315633e-03, 
       1.4051804264805178e-03, 
       1.4000004549437825e-03, 
       1.2456150634575758e-03, 
       1.1917404056854724e-03, 
       1.1469858896632520e-03, 
       1.3799999999996573e-03, 
       1.1451599120823083e-03, 
       1.1208179706300851e-03, 
       1.3330975433635385e-03, 
       1.3355407765091320e-03, 
       1.3187888915508884e-03, 
       1.0013270525405741e-03, 
       1.1568211093476619e-03, 
       1.3199999999995132e-03, 
       1.3116782654570126e-03, 
       1.0109559475397340e-03, 
       1.1138205452454876e-03, 
       1.0832716574282366e-03, 
       1.2836457932664052e-03, 
       1.1557278458711781e-03, 
       1.0656307882811553e-03, 
       1.0653787308962750e-03, 
       1.2769410555455612e-03, 
       1.2469323329052427e-03, 
       1.0389232051345304e-03, 
       9.2531631016251181e-04, 
       1.0130478732382282e-03, 
       1.0129401730177223e-03, 
       1.2085776073983202e-03, 
       9.2520552122620230e-04, 
       8.7107551093525582e-04, 
       1.0512642559133106e-03, 
       9.2004712002292715e-04, 
       8.7727090406908943e-04, 
       1.0097040623480615e-03, 
       8.7354061917720041e-04, 
       9.8890871541422194e-04, 
       1.1727329917061958e-03, 
       8.4796485652031462e-04, 
       9.0037526345779152e-04, 
       1.0550721991563637e-03, 
       1.1700750449994070e-03, 
       1.1326401775287145e-03, 
       8.2180904067922058e-04, 
       9.5107217131404939e-04, 
       1.1399999999995279e-03, 
       8.2644788088203356e-04, 
       1.1327675195473863e-03, 
       8.6408623064426456e-04, 
       8.6408623064426456e-04, 
       9.3652942131664419e-04, 
       8.2724855086805413e-04, 
       1.0516041839648289e-03, 
       8.0509973763748125e-04, 
       1.1000000000003342e-03, 
       1.0979015183471543e-03, 
       8.0626437551830786e-04, 
       7.8806691077271151e-04, 
       9.7914063964809586e-04, 
       7.7558813880571553e-04, 
       1.0780160426095077e-03, 
       9.0637779322945747e-04, 
       7.7149457900131842e-04, 
       9.1219661994809426e-04, 
       1.0580965704999389e-03, 
       9.3999999999999965e-04, 
       1.0475411312257327e-03, 
       7.6071622346851348e-04, 
       7.6601372150003611e-04, 
       8.0071860618742259e-04, 
       8.7154633617450897e-04, 
       1.0051995379508089e-03, 
       7.6208020201374996e-04, 
       1.0411476076641849e-03, 
       9.6000000000002876e-04, 
       1.0418666456897920e-03, 
       1.0400000000003265e-03, 
       1.0052959621780673e-03, 
       8.8354408842634881e-04, 
       8.4520956981134316e-04, 
       1.0085064543444062e-03, 
       7.6534909762047542e-04, 
       8.2795062025050692e-04, 
       8.8274655010465595e-04, 
       7.4596361836866626e-04, 
       8.2827356375065545e-04, 
       9.7036552037568313e-04, 
       7.2799575261560704e-04, 
       9.5167889368277250e-04, 
       8.4307764517166351e-04, 
       9.7627497970236455e-04, 
       7.2150706230905378e-04, 
       8.0628206420315165e-04, 
       8.3900864630724625e-04, 
       9.7000000000000211e-04, 
       6.9891696710727735e-04, 
       8.0235740815248703e-04, 
       9.1204691317772757e-04, 
       7.9601176197354655e-04, 
       7.1172509887164651e-04, 
       7.7772253203650698e-04, 
       9.4047607309143133e-04, 
       9.0170696014470013e-04, 
       7.7315636309325479e-04, 
       8.0173142247866498e-04, 
       8.0659486705388268e-04, 
       9.2117008112883349e-04, 
       8.9428632429355067e-04, 
       7.5046930753465337e-04, 
       7.4932796526593450e-04, 
       7.4008811493377622e-04, 
       7.4008057300323270e-04, 
       7.4755093911109015e-04, 
       9.0999999999997084e-04, 
       7.1176495945039732e-04, 
       7.1165140011339499e-04, 
       6.7660718419984339e-04, 
       6.8866269194694531e-04, 
       8.9999979774096418e-04, 
       7.7499779337206805e-04, 
       7.4779782852409105e-04, 
       6.7849745328580973e-04, 
       7.8543832436192779e-04, 
       6.9672952946551612e-04, 
       7.4332191289190521e-04, 
       8.3268613035583161e-04, 
       6.7278011577212257e-04, 
       6.2332674111311370e-04, 
       6.5887219355865176e-04, 
       8.7172737477500648e-04, 
       7.5499582920487247e-04, 
       6.3780277018099995e-04, 
       6.1838297415487840e-04, 
       6.4839924332826164e-04, 
       6.4708431868200984e-04, 
       6.9910256259398730e-04, 
       6.9849956442391212e-04, 
       6.6668331374206320e-04, 
       8.5999999999997938e-04, 
       6.2367116284347288e-04, 
       6.0916199496203821e-04, 
       7.8433215323384391e-04, 
       7.0742413328175500e-04, 
       6.1857119610164734e-04, 
       7.3547010518451627e-04, 
       8.4639036360295897e-04, 
       8.4750463425420038e-04, 
       8.4000000000003083e-04, 
       6.9167157194570178e-04, 
       8.3821006008085125e-04, 
       5.8528461769237017e-04, 
       5.9634596194179030e-04, 
       6.0942556969476524e-04, 
       6.3601747582323141e-04, 
       7.4999999999998462e-04, 
       6.0242566224755786e-04, 
       7.0571523827042996e-04, 
       6.5215536999790153e-04, 
       7.7331231080491497e-04, 
       7.9000000000000793e-04, 
       7.4646830363452304e-04, 
       7.7876196939172834e-04, 
       5.7496171974028162e-04, 
       5.9761357383430981e-04, 
       5.6155562699090791e-04, 
       7.6700867444750589e-04, 
       7.7267189826444509e-04, 
       7.4260041786850613e-04, 
       7.6906702623812107e-04, 
       6.3896956121410034e-04, 
       6.6107606364882571e-04, 
       6.2016993775670763e-04, 
       6.2013880832658178e-04, 
       5.8338445120051125e-04, 
       6.2865676338615800e-04, 
       6.5526500701197318e-04, 
       7.5283346077168866e-04, 
       6.3284655053611242e-04, 
       6.5418437427937316e-04, 
       6.5418437427937316e-04, 
       6.4625238265299668e-04, 
       6.4026795325958295e-04, 
       6.2303184067540626e-04, 
       5.8465500207235390e-04, 
       5.5927852918626075e-04, 
       5.5455611606629795e-04, 
       5.3854055866391526e-04, 
       7.3817015844026721e-04, 
       6.3375115436815660e-04, 
       5.3485051181018128e-04, 
       6.1408798771561538e-04, 
       6.7247453200508685e-04, 
       7.3437025593731120e-04, 
       5.1909521875237452e-04, 
       5.9095077101115902e-04, 
       5.4256068005737097e-04, 
       6.7000000000002854e-04, 
       7.2040462815967587e-04, 
       6.3684908656681468e-04, 
       5.8911142084574228e-04, 
       5.5659861274247855e-04, 
       6.5563809286661094e-04, 
       5.1743060637528043e-04, 
       5.6417821596793703e-04, 
       7.0999999999999265e-04, 
       5.2870183104463829e-04, 
       5.4072418392741076e-04, 
       5.3852724312237273e-04, 
       6.0380526684405370e-04, 
       5.1456074279196407e-04, 
       6.9633113120495457e-04, 
       4.8727848323658021e-04, 
       6.6530329034951527e-04, 
       6.8217432002491989e-04, 
       5.6131137496598425e-04, 
       5.5792543104689785e-04, 
       5.2083872850192516e-04, 
       4.8700254003741387e-04, 
       6.7000000000000165e-04, 
       4.7866479906005002e-04, 
       5.3589250109373719e-04, 
       6.4381806951675866e-04, 
       5.5606789303066595e-04, 
       5.5606789303066595e-04, 
       6.3989972803039680e-04, 
       4.8294345770109084e-04, 
       4.6383616640867657e-04, 
       6.4075662225117596e-04, 
       5.3774270138911835e-04, 
       4.9699686219286364e-04, 
       4.7389232158706282e-04, 
       6.1701059888090621e-04, 
       6.4034302206218165e-04, 
       5.3476892129095263e-04, 
       6.3564004399259215e-04, 
       4.5177658757417050e-04, 
       4.5904898230094679e-04, 
       4.7352672495366621e-04, 
       6.2127375597302516e-04, 
       5.1799017046544283e-04, 
       4.7194082679770963e-04, 
       6.0433804380131907e-04, 
       4.5799054581540608e-04, 
       5.4163788872034111e-04, 
       6.3005395655346490e-04, 
       5.2150618773877486e-04, 
       5.8000000000003416e-04, 
       4.8004921494631115e-04, 
       4.7951955867804634e-04, 
       5.2429042804136294e-04, 
       6.2781865161840923e-04, 
       5.2000000000000516e-04, 
       5.2126739525711345e-04, 
       4.5469085339590966e-04, 
       5.0594250147680594e-04, 
       5.2842352789031477e-04, 
       5.3116611673165583e-04, 
       4.5698228121702117e-04, 
       6.0999999999995530e-04, 
       6.0822654840751006e-04, 
       5.0706454162519239e-04, 
       4.2931612956913765e-04, 
       5.5812332499745115e-04, 
       4.9600973658683975e-04, 
       4.9586418373057594e-04, 
       5.9880403975352543e-04, 
       5.0097298555063494e-04, 
       4.9867321496820051e-04, 
       4.2584124053027250e-04, 
       4.3544238199404091e-04, 
       5.0192876341085451e-04, 
       5.4899513676598464e-04, 
       4.2469516018298401e-04, 
       4.3743761295145554e-04, 
       4.5238203879695673e-04, 
       4.3442072644832194e-04, 
       5.0378274461263797e-04, 
       4.5699816137679969e-04, 
       4.8374571178774403e-04, 
       4.4730254569496286e-04, 
       5.6738211168987443e-04, 
       5.7918767914023688e-04, 
       5.0938272706205006e-04, 
       4.6278087591704596e-04, 
       5.3999999999996163e-04, 
       4.2393831258550669e-04, 
       4.0855596764195274e-04, 
       5.2868979439029556e-04, 
       5.4057708358920336e-04, 
       4.9241120042939259e-04, 
       5.7037111470895584e-04, 
       5.6936384879611014e-04, 
       5.1482131513020312e-04, 
       5.6114289250650711e-04, 
       4.1658451098365140e-04, 
       4.0267907531323414e-04, 
       4.2163772609149047e-04, 
       5.3964324558564971e-04, 
       5.4100561173851716e-04, 
       4.6636328334147492e-04, 
       3.9883690804463234e-04, 
       4.5861684155700557e-04, 
       4.8066922885953931e-04, 
       4.1297481278256308e-04, 
       4.3681867049981035e-04, 
       4.4312047356383748e-04, 
       4.4292186284246467e-04, 
       4.8315118993635427e-04, 
       5.4799381116081461e-04, 
       4.4534254160209216e-04, 
       5.4999999999997239e-04, 
       4.2824639760355681e-04, 
       4.5353257244750274e-04, 
       5.0000000000004761e-04, 
       5.2581817920284771e-04, 
       3.9100422490278251e-04, 
       4.4885849783463984e-04, 
       4.5118808627533859e-04, 
       4.3864011646667020e-04, 
       5.3896428408220065e-04, 
       4.5356764480798044e-04, 
       5.3591453771710074e-04, 
       3.8461183613504812e-04, 
       3.8872720786660805e-04, 
       3.8035965653608497e-04, 
       5.3237819155267825e-04, 
       3.8794513299381832e-04, 
       3.9023927259511035e-04, 
       4.4268689399803578e-04, 
       3.9158373698325213e-04, 
       3.8072181872957230e-04, 
       5.3000000000003923e-04, 
       4.9049889309418966e-04, 
       5.2286011675979573e-04, 
       5.1242909998845915e-04, 
       4.5145269694994879e-04, 
       3.8474794626108935e-04, 
       4.3860490718389223e-04, 
       4.3716058219328666e-04, 
       3.8764796655494049e-04, 
       5.0979050589011830e-04, 
       4.3883146260156121e-04, 
       3.9923967057894916e-04, 
       4.8024923913759615e-04, 
       4.2488787081819304e-04, 
       4.2487719915837549e-04, 
       4.7999999999997247e-04, 
       3.8239992339520227e-04, 
       3.7995382647102339e-04, 
       5.1898232449788589e-04, 
       5.1273549010162091e-04, 
       3.6810298164892337e-04, 
       5.0414366740333027e-04, 
       3.8577085168650497e-04, 
       3.7700343951807010e-04, 
       4.5489825687109240e-04, 
       3.6543362656160872e-04, 
       4.7307774905430553e-04, 
       3.8202835929834300e-04, 
       4.2419115161969800e-04, 
       3.8031360246802600e-04, 
       5.0999999999999429e-04, 
       3.6863258737530695e-04, 
       3.5694593331547374e-04, 
       3.5609324961655457e-04, 
       3.6987206428839523e-04, 
       4.6375729871233853e-04, 
       4.9540420375800804e-04, 
       4.9647582897080283e-04, 
       3.6057957445622405e-04, 
       3.7072925946169159e-04, 
       3.7005288269498411e-04, 
       4.4672901027357679e-04, 
       4.9526950035526363e-04, 
       4.0130925590775823e-04, 
       3.6323734565869110e-04, 
       3.9902159754255960e-04, 
       3.6105441012942898e-04, 
       4.1924424522192511e-04, 
       4.0748733236215174e-04, 
       4.9000000000003143e-04, 
       4.8999999999996930e-04, 
       4.9000000000000595e-04, 
       3.5737580086504989e-04, 
       4.1506143629176762e-04, 
       3.9203938955698401e-04, 
       3.6111706270897489e-04, 
       4.3626597159182069e-04, 
       4.6157603400965913e-04, 
       3.9550710423398279e-04, 
       4.7999999458706781e-04, 
       4.8000000000003557e-04, 
       3.8651919252167692e-04, 
       4.3928279098575968e-04, 
       4.4000000000000419e-04, 
       4.4000000000003449e-04, 
       4.7493012832872686e-04, 
       4.2844347309980226e-04, 
       3.8275311827265382e-04, 
       4.7189005332795857e-04, 
       4.7164345940869164e-04, 
       3.4372634755835068e-04, 
       3.4164402379399666e-04, 
       4.6132198636922371e-04, 
       3.4161690903960801e-04, 
       4.3358937588831506e-04, 
       3.8443412995188027e-04, 
       3.5547127231329837e-04, 
       4.5504838680506947e-04, 
       3.6098603967025546e-04, 
       3.9473291192216161e-04, 
       3.8233349229818784e-04, 
       4.5749890781806207e-04, 
       3.6843319166199621e-04, 
       3.7929391348948306e-04, 
       3.6808291450303720e-04, 
       3.3524747217176978e-04, 
       3.3282655560692824e-04, 
       3.7869919537842370e-04, 
       4.4911548805009544e-04, 
       4.4936167761971241e-04, 
       3.4531661765942943e-04, 
       3.8385730787900190e-04, 
       3.6209485379078637e-04, 
       3.2839161185861454e-04, 
       4.4390645020852800e-04, 
       4.1225084259223922e-04, 
       4.4186487652001465e-04, 
       3.5955632244303139e-04, 
       4.2834178091091893e-04, 
       3.7070879794726781e-04, 
       4.3826296465373783e-04, 
       4.2341928205127249e-04, 
       4.3776596361037222e-04, 
       4.3676254907322459e-04, 
       3.6599824827797611e-04, 
       3.6086455042784942e-04, 
       3.6403029325719663e-04, 
       3.4000000000002968e-04, 
       3.8948236290775384e-04, 
       3.8079669069189721e-04, 
       3.1174520041760085e-04, 
       3.3770390625988866e-04, 
       3.5948608948153710e-04, 
       3.7940862248331920e-04, 
       3.8845959770774800e-04, 
       3.5628376498476742e-04, 
       3.1414703326371895e-04, 
       3.5114822697561812e-04, 
       3.5114822697561812e-04, 
       4.2356415450734846e-04, 
       3.7387527627911793e-04, 
       3.7348158308752526e-04, 
       3.4722116638193605e-04, 
       3.6814779390133333e-04, 
       3.0452945398203100e-04, 
       3.4589305946926453e-04, 
       3.3498402199729182e-04, 
       3.3445257862680088e-04, 
       4.1999999999998945e-04, 
       4.1999999999997958e-04, 
       4.2000000000001601e-04, 
       3.4914441802883292e-04, 
       4.1713888958976215e-04, 
       4.0033494202084774e-04, 
       3.1410039272413841e-04, 
       4.1628362137998846e-04, 
       4.1838938144434537e-04, 
       3.8723641641423258e-04, 
       4.1688464004266862e-04, 
       3.0171898838629374e-04, 
       3.5000000000000477e-04, 
       4.1374349887792194e-04, 
       3.7999999999997270e-04, 
       3.4234229013904233e-04, 
       3.0829218744575661e-04, 
       4.0999999999997782e-04, 
       3.9984979764024441e-04, 
       3.9586787986204037e-04, 
       3.4611775765024133e-04, 
       3.0000114426819363e-04, 
       2.9347441592801751e-04, 
       2.9333214380561340e-04, 
       4.0267053205293089e-04, 
       3.6999999999999506e-04, 
       3.7000000000002325e-04, 
       3.6999999999998720e-04, 
       2.9554856373002005e-04, 
       3.0875466973137258e-04, 
       3.9400650488012655e-04, 
       3.0263571048639280e-04, 
       3.4399682695572135e-04, 
       3.9503070368061176e-04, 
       3.3882167801917883e-04, 
       2.9024380978747268e-04, 
       3.9081448462846075e-04, 
       2.8330351599141046e-04, 
       3.0065374635551874e-04, 
       2.8782807808483743e-04, 
       2.8782807808483743e-04, 
       3.8999999999997538e-04, 
       3.8975268340873720e-04, 
       2.7657660253066761e-04, 
       3.3635034015515135e-04, 
       3.2889529318056397e-04, 
       3.8310041747387398e-04, 
       3.8284563450906333e-04, 
       3.6990458707474060e-04, 
       3.0733204285691735e-04, 
       2.7063659493299110e-04, 
       3.8000000000002398e-04, 
       3.1630643747690375e-04, 
       2.7524028809268545e-04, 
       2.7507386173293298e-04, 
       3.7162683182224477e-04, 
       3.7560967609000294e-04, 
       2.7576480115788809e-04, 
       3.2109099241918161e-04, 
       3.7031400750790245e-04, 
       3.7298931038614225e-04, 
       3.6833530005254777e-04, 
       2.9646962536296323e-04, 
       3.6953876353780702e-04, 
       3.1848441839320480e-04, 
       3.4031708862045801e-04, 
       2.7080569475716255e-04, 
       3.5202574209943155e-04, 
       2.9810565621907914e-04, 
       3.0482734539511168e-04, 
       2.9945747846631907e-04, 
       3.6151706030215124e-04, 
       2.8030256357470856e-04, 
       3.0099038408159298e-04, 
       2.7175455819719439e-04, 
       2.5932678514569433e-04, 
       3.4533163964025633e-04, 
       3.6000000000001954e-04, 
       2.9853991092599263e-04, 
       3.5500341493261176e-04, 
       2.5349334466824081e-04, 
       2.8686075247475314e-04, 
       2.9614429454243762e-04, 
       3.5547419360323717e-04, 
       2.9393008264281200e-04, 
       2.6428554766464243e-04, 
       3.5267558244546365e-04, 
       2.9762504929802604e-04, 
       2.9896630095926642e-04, 
       2.7137088148696919e-04, 
       2.9154971372214318e-04, 
       3.0880268896971949e-04, 
       2.9022085775428323e-04, 
       2.8096988753829064e-04, 
       2.9610753880841004e-04, 
       2.5937830117374913e-04, 
       3.4113505818403030e-04, 
       2.8949570886925184e-04, 
       2.9417077907381020e-04, 
       2.6915660666774143e-04, 
       3.4529126342670028e-04, 
       3.4511364918308411e-04, 
       3.0809286824513431e-04, 
       2.5663372490069349e-04, 
       3.1759241951216774e-04, 
       2.7661106505778052e-04, 
       2.7649016805465868e-04, 
       3.3941507461833969e-04, 
       2.8357716900052280e-04, 
       2.5054341159205660e-04, 
       3.4000000000000013e-04, 
       3.3999999999995742e-04, 
       2.8177568993185178e-04, 
       2.4223639852188515e-04, 
       2.5764080860950374e-04, 
       2.4427156842369739e-04, 
       2.5613184275647907e-04, 
       2.9938226131296319e-04, 
       3.3370959145793174e-04, 
       2.5476719487528814e-04, 
       2.4891847619529635e-04, 
       2.9033317750130878e-04, 
       2.7444747722149939e-04, 
       3.2185706476030871e-04, 
       2.4242484117400007e-04, 
       2.8254690194629088e-04, 
       2.9317112264232691e-04, 
       2.8216246556595951e-04, 
       2.7049093636069099e-04, 
       3.2999999999998433e-04, 
       3.2999999999997582e-04, 
       3.3000000000002971e-04, 
       3.3000000000004483e-04, 
       3.2789231512658475e-04, 
       2.4036255053395375e-04, 
       2.3660186381740538e-04, 
       2.4030449909875185e-04, 
       2.4549169759636714e-04, 
       3.2683780901060201e-04, 
       3.2738532044983943e-04, 
       2.3723403297637845e-04, 
       3.2644933282577088e-04, 
       3.1785194532102525e-04, 
       2.3612481666414564e-04, 
       2.3410145788861126e-04, 
       2.3482993433245864e-04, 
       2.8183502217594785e-04, 
       3.2322526929990156e-04, 
       3.1482433850621872e-04, 
       3.1570987111836387e-04, 
       3.1882896738783298e-04, 
       2.6856965347746162e-04, 
       2.8884576993238096e-04, 
       2.4008857206194383e-04, 
       3.1427355522657029e-04, 
       3.2027583644669041e-04, 
       3.1999999999998935e-04, 
       3.0321715521636693e-04, 
       2.6233083511775432e-04, 
       2.3870567028145707e-04, 
       3.1877332219331310e-04, 
       2.4123841209723344e-04, 
       2.2880551303770062e-04, 
       2.2883793202198413e-04, 
       2.2711519287886344e-04, 
       2.6431168954183946e-04, 
       3.1716431086964609e-04, 
       2.3365084331869923e-04, 
       2.3355344395473780e-04, 
       2.7129457402386305e-04, 
       2.5443679338400436e-04, 
       3.1557299742775082e-04, 
       3.1252900830236035e-04, 
       2.2265480926931077e-04, 
       2.2781199798900566e-04, 
       2.5898677765155405e-04, 
       2.3304964904622530e-04, 
       3.0699700667121110e-04, 
       2.5622203333772051e-04, 
       2.2516631000076644e-04, 
       3.1000000000004863e-04, 
       2.1899598358972997e-04, 
       2.1899598358972997e-04, 
       2.3334138603325351e-04, 
       3.0445221619753766e-04, 
       3.0298084247796109e-04, 
       2.3333393444851950e-04, 
       2.5486657324804720e-04, 
       2.6671463457996733e-04, 
       2.2831799310785541e-04, 
       2.2183253007517417e-04, 
       2.9861872571147508e-04, 
       2.5037606082532765e-04, 
       2.6135870692283120e-04, 
       2.1690328970584946e-04, 
       2.1675790685402860e-04, 
       2.5051814644690487e-04, 
       2.2553144221469621e-04, 
       2.9886517693246926e-04, 
       2.5084167846164913e-04, 
       2.9724777194514045e-04, 
       2.4027399963606001e-04, 
       2.8855562901231117e-04, 
       2.1211869292314073e-04, 
       2.9815237508766155e-04, 
       2.1168566704878465e-04, 
       2.7437553296904551e-04, 
       2.9570155494861017e-04, 
       2.1949512610809906e-04, 
       2.5709683054586381e-04, 
       2.3265927439742606e-04, 
       2.3262218563862365e-04, 
       2.6151263723332729e-04, 
       2.1879088501459518e-04, 
       2.8447186232177525e-04, 
       2.9371318972023185e-04, 
       2.1975926854542907e-04, 
       2.5834853807682776e-04, 
       2.7001806439462853e-04, 
       2.2391482155277457e-04, 
       2.5359642465256607e-04, 
       2.1463434804135225e-04, 
       2.0949856795106304e-04, 
       2.5325963842871466e-04, 
       2.3809827874936770e-04, 
       2.3896853608055085e-04, 
       2.8909781517596063e-04, 
       2.2793457791660845e-04, 
       2.8979724647139609e-04, 
       2.8999999999999751e-04, 
       2.8999999999996585e-04, 
       2.9000000000004304e-04, 
       2.1577862145613148e-04, 
       2.4782435612165235e-04, 
       2.1087283623582510e-04, 
       2.1546311852216378e-04, 
       2.3979577282235096e-04, 
       2.0837903812109704e-04, 
       2.0338287658960952e-04, 
       2.1346508110662552e-04, 
       2.1855062540596573e-04, 
       2.1287748558370426e-04, 
       2.8449846704539715e-04, 
       2.3369250571113616e-04, 
       2.7992038515875208e-04, 
       2.7704503865523262e-04, 
       2.0257242724104076e-04, 
       2.1700753483245558e-04, 
       2.1537074519788882e-04, 
       2.0486715699956560e-04, 
       2.6000000000003028e-04, 
       2.3129838330930130e-04, 
       2.7927487727435796e-04, 
       2.8002083736495647e-04, 
       2.4259068001668036e-04, 
       2.0189564856414673e-04, 
       2.0399450420918643e-04, 
       2.8000000000004399e-04, 
       2.7928398627215560e-04, 
       2.6834742158805651e-04, 
       2.2456476838297252e-04, 
       2.2015279319370770e-04, 
       2.0763818701831612e-04, 
       2.3980716703292300e-04, 
       2.1258144273220034e-04, 
       2.7650281385474307e-04, 
       2.7631049982290874e-04, 
       2.4918966991793653e-04, 
       1.9949652648244300e-04, 
       2.2722677455915040e-04, 
       1.9497406723129672e-04, 
       2.0481037164540500e-04, 
       2.2108456570757863e-04, 
       2.3238995659228244e-04, 
       2.4073002870254632e-04, 
       2.6526766753376857e-04, 
       2.0541980020880791e-04, 
       1.9214717321901396e-04, 
       2.6282405295118845e-04, 
       2.0675763193031598e-04, 
       2.1384371188986169e-04, 
       1.9106212618840197e-04, 
       2.0634557443678474e-04, 
       1.9194061882541716e-04, 
       2.5374895208178763e-04, 
       2.6377788051469521e-04, 
       2.6557726031751475e-04, 
       1.9159598808228350e-04, 
       2.6563262367237977e-04, 
       2.6622867182037242e-04, 
       2.3300779468081247e-04, 
       2.2020161139969677e-04, 
       2.6514003730403586e-04, 
       2.6361744011957474e-04, 
       2.2044611826163189e-04, 
       1.9188158774982578e-04, 
       2.0099014642399938e-04, 
       2.3921355409428383e-04, 
       2.6060257429494664e-04, 
       2.5346220913230130e-04, 
       1.9077696558919460e-04, 
       1.8654035273723578e-04, 
       2.5789951635309317e-04, 
       1.9553402523629171e-04, 
       2.1019155088827362e-04, 
       1.8583124909994129e-04, 
       2.5550658975943195e-04, 
       2.5292089448151301e-04, 
       1.8944632441603875e-04, 
       1.8741703637204199e-04, 
       2.5813604864117953e-04, 
       2.1688515177499426e-04, 
       2.3999999999998499e-04, 
       2.5129581242015796e-04, 
       1.9002063819015930e-04, 
       1.8612775806626922e-04, 
       1.9020275741330617e-04, 
       2.6000000000001114e-04, 
       1.8376209105200252e-04, 
       2.5666539043135266e-04, 
       1.9997808969826329e-04, 
       2.1547510150625839e-04, 
       2.1538095407068917e-04, 
       1.8336556482163681e-04, 
       2.5589098769450441e-04, 
       2.2535987852880950e-04, 
       2.5731342020724189e-04, 
       2.5658361720593556e-04, 
       2.1182995514284922e-04, 
       2.5647495567456132e-04, 
       1.9239215027761565e-04, 
       2.2388373662529689e-04, 
       2.0991239162701126e-04, 
       1.9004968201448793e-04, 
       2.1109732842864134e-04, 
       2.0426233006376991e-04, 
       2.0366585553522798e-04, 
       2.0413824214512901e-04, 
       2.5208417850456521e-04, 
       1.7970480936429269e-04, 
       1.9443304440592836e-04, 
       2.1071307491541316e-04, 
       1.7952606487333492e-04, 
       1.8337310156482436e-04, 
       2.0954430142699367e-04, 
       2.4372762565021887e-04, 
       1.8214499936976508e-04, 
       1.9481771528542636e-04, 
       1.8647348339112134e-04, 
       1.8103218041522570e-04, 
       1.8287528048022979e-04, 
       2.4999995476779509e-04, 
       2.4999999999996190e-04, 
       2.4999999999999339e-04, 
       2.4999999999998759e-04, 
       2.3961996823550861e-04, 
       2.4545490123846307e-04, 
       1.9497580963566768e-04, 
       1.9494362642921616e-04, 
       1.7950289051323813e-04, 
       2.0691986281761604e-04, 
       1.7816701775965750e-04, 
       2.1240691128690574e-04, 
       2.0793414692099344e-04, 
       1.8116439328314353e-04, 
       1.7681929569676296e-04, 
       1.7456698453694310e-04, 
       2.0450666265044376e-04, 
       1.9802104879332080e-04, 
       1.9033625410847809e-04, 
       1.8967057222170946e-04, 
       2.4114777641494735e-04, 
       1.8824503590531382e-04, 
       1.8719283179624669e-04, 
       1.7694008669400584e-04, 
       2.1849841181933264e-04, 
       1.9663030265806616e-04, 
       1.9633210254753068e-04, 
       2.4052158850526936e-04, 
       1.7999999999997944e-04, 
       1.7000680051714351e-04, 
       2.3999999831054937e-04, 
       2.4000000000002296e-04, 
       2.3943850889870426e-04, 
       1.8620077175263819e-04, 
       1.7328681277532958e-04, 
       1.9091808611069764e-04, 
       1.7171059381928050e-04, 
       2.2928177418358244e-04, 
       1.7374685821447850e-04, 
       1.8119808773433584e-04, 
       2.2473925712061986e-04, 
       2.3662297016705509e-04, 
       2.3381954273796020e-04, 
       1.7125750788944525e-04, 
       1.8918019180266379e-04, 
       1.9545364521866338e-04, 
       1.8980489007967589e-04, 
       1.7771797352622728e-04, 
       1.9617738270222024e-04, 
       2.2210182364860943e-04, 
       1.6565575158043771e-04, 
       1.6899180881186325e-04, 
       2.3290350350486623e-04, 
       2.3044753064077787e-04, 
       2.3159524911006907e-04, 
       1.6785474304415306e-04, 
       1.9689253978544933e-04, 
       1.9795310256447206e-04, 
       2.2132255411853915e-04, 
       1.6724228569226502e-04, 
       2.2999999812864339e-04, 
       2.3000000000002630e-04, 
       2.2237444821182145e-04, 
       1.6905309656930210e-04, 
       2.2693221872010415e-04, 
       1.7441377807373418e-04, 
       1.6655291014074013e-04, 
       1.8913219747789100e-04, 
       1.9379285248247649e-04, 
       2.1857880045748424e-04, 
       2.2570077622680382e-04, 
       1.6590376545785364e-04, 
       1.8622764763576868e-04, 
       2.2165792444875539e-04, 
       1.7162089720546868e-04, 
       1.6670253578951065e-04, 
       2.1264704225394088e-04, 
       1.6928004494945866e-04, 
       2.1626508377679105e-04, 
       2.1849553126416480e-04, 
       2.2316133834256421e-04, 
       2.2276873744277748e-04, 
       1.5957002627403554e-04, 
       1.6218527689569072e-04, 
       1.9922763320919011e-04, 
       1.9740658072256258e-04, 
       1.8364995241294433e-04, 
       1.6174137375705517e-04, 
       1.6918321206302954e-04, 
       1.8259169411739872e-04, 
       1.6676738950838812e-04, 
       1.7629318495105125e-04, 
       2.1999999842919999e-04, 
       1.8779527169834919e-04, 
       1.8910548967555318e-04, 
       1.9213389425207111e-04, 
       2.1557540994966449e-04, 
       1.6461390351423238e-04, 
       1.5587036190292371e-04, 
       1.6157027389241378e-04, 
       2.1481618087290823e-04, 
       1.5939889079285088e-04, 
       1.8232594305681648e-04, 
       1.5688520201851360e-04, 
       1.5676321374122462e-04, 
       1.5884008775410822e-04, 
       1.8673210484504343e-04, 
       1.8091731131835657e-04, 
       2.0596839809775206e-04, 
       1.6766738741332818e-04, 
       1.7861009961421464e-04, 
       1.8536883390554281e-04, 
       2.1282397984584276e-04, 
       1.5761894317681789e-04, 
       1.7828276555033470e-04, 
       2.1112015900125400e-04, 
       2.0375196258571638e-04, 
       1.8932112471143885e-04, 
       1.5319434806925058e-04, 
       1.8802999335465617e-04, 
       1.5512132666444398e-04, 
       1.7769463067617662e-04, 
       1.7563036253159019e-04, 
       1.6470366591437183e-04, 
       1.6727161890494976e-04, 
       1.7643082417215570e-04, 
       1.8007489828472906e-04, 
       1.8919683883273532e-04, 
       1.6700318425309687e-04, 
       1.5957975666994539e-04, 
       2.1049161795459571e-04, 
       2.1000000000004470e-04, 
       2.1000000000000001e-04, 
       1.9636548086640710e-04, 
       1.5520732477062579e-04, 
       1.4781218520495913e-04, 
       1.5306876366423720e-04, 
       1.7884138556134357e-04, 
       1.7016700469450033e-04, 
       1.5033757569022353e-04, 
       1.9925899477560508e-04, 
       1.5065458072371574e-04, 
       1.5092659355487014e-04, 
       1.4972260780358988e-04, 
       1.7266766283531037e-04, 
       1.7135449867231289e-04, 
       1.7613260782150424e-04, 
       1.9372920927938445e-04, 
       1.5361834628743401e-04, 
       1.8843971661812553e-04, 
       1.5442910893508768e-04, 
       1.6477192426527719e-04, 
       1.9807154613362733e-04, 
       1.9742417994822686e-04, 
       1.9070567297420501e-04, 
       1.5942528075983599e-04, 
       1.9608375579301121e-04, 
       1.7399927995049861e-04, 
       1.5881595642723009e-04, 
       1.8099227914324678e-04, 
       1.6660376318135381e-04, 
       1.7200648643678738e-04, 
       1.8427288242319785e-04, 
       1.9568369574592194e-04, 
       2.0003952246627155e-04, 
       1.5567366624488952e-04, 
       1.9999999999997402e-04, 
       1.9999999999998548e-04, 
       1.9792172858593848e-04, 
       1.4310096252096249e-04, 
       1.4984179580354047e-04, 
       1.9817181809985834e-04, 
       1.6243934246327064e-04, 
       1.9068964996572972e-04, 
       1.9710637457513990e-04, 
       1.4017792896661152e-04, 
       1.6175486689114766e-04, 
       1.9694412761680837e-04, 
       1.6449598019312948e-04, 
       1.4451102023875325e-04, 
       1.6430428708238764e-04, 
       1.6494121164139716e-04, 
       1.6224089167058133e-04, 
       1.4724545241305841e-04, 
       1.7700221897636373e-04, 
       1.3793220244745120e-04, 
       1.6201190195347465e-04, 
       1.7077752612510639e-04, 
       1.3744505668805614e-04, 
       1.7059975165868726e-04, 
       1.9355173156224126e-04, 
       1.4773166157372117e-04, 
       1.6299668447241464e-04, 
       1.8503016457775020e-04, 
       1.5248417766524760e-04, 
       1.3584577950472484e-04, 
       1.8755090593651656e-04, 
       1.5759592002792308e-04, 
       1.5813081766732521e-04, 
       1.9000000000004072e-04, 
       1.9000000000001714e-04, 
       1.9000000000004682e-04, 
       1.8999999999997995e-04, 
       1.6004593629615139e-04, 
       1.8669587368764319e-04, 
       1.3711202240170283e-04, 
       1.4712002082325982e-04, 
       1.3588851359598421e-04, 
       1.4059354710901909e-04, 
       1.5686488258599232e-04, 
       1.8782843662318118e-04, 
       1.6885789433656608e-04, 
       1.5853135825487006e-04, 
       1.3512641270313269e-04, 
       1.3512641270313269e-04, 
       1.5352199669070514e-04, 
       1.3555064056720956e-04, 
       1.8521666212579883e-04, 
       1.8631503853401530e-04, 
       1.3716155826547303e-04, 
       1.5554189350402072e-04, 
       1.3822028587182194e-04, 
       1.3170787661492017e-04, 
       1.4165160791149682e-04, 
       1.3542087827989903e-04, 
       1.5424922274356079e-04, 
       1.5416425086238890e-04, 
       1.6999999999995225e-04, 
       1.5262635401566943e-04, 
       1.3554706894795349e-04, 
       1.7738487596821574e-04, 
       1.4000000000000628e-04, 
       1.7208637595502314e-04, 
       1.2981732081726060e-04, 
       1.4525078714468031e-04, 
       1.4514597007982876e-04, 
       1.6586658500066900e-04, 
       1.8000000000002926e-04, 
       1.7971978623108569e-04, 
       1.4921811645460435e-04, 
       1.7917078180827833e-04, 
       1.5999421229703585e-04, 
       1.4770219502029539e-04, 
       1.7766611164873254e-04, 
       1.4702906421841302e-04, 
       1.5092865476700032e-04, 
       1.7208584410787677e-04, 
       1.5565886252242534e-04, 
       1.7472641986576156e-04, 
       1.5212853517262404e-04, 
       1.6762051919568026e-04, 
       1.4604840258536795e-04, 
       1.2855306854428510e-04, 
       1.7640171712224201e-04, 
       1.5105182804750640e-04, 
       1.2999999999996931e-04, 
       1.3393090359136434e-04, 
       1.3086388312263361e-04, 
       1.7437735676919882e-04, 
       1.4142957186260652e-04, 
       1.3601883947562049e-04, 
       1.7474304402880636e-04, 
       1.2870575456825636e-04, 
       1.3067794378956220e-04, 
       1.6726170031722580e-04, 
       1.5887090538137578e-04, 
       1.6736339494216186e-04, 
       1.7198108488768876e-04, 
       1.7281531081100190e-04, 
       1.4387618209498135e-04, 
       1.4281745325288834e-04, 
       1.6868632327446585e-04, 
       1.7186198810893914e-04, 
       1.6748077973254883e-04, 
       1.4526695555911202e-04, 
       1.2813652685045304e-04, 
       1.4699179246219638e-04, 
       1.5067871296164465e-04, 
       1.6111899421599097e-04, 
       1.3350497708359164e-04, 
       1.3346974484113109e-04, 
       1.3983871436806233e-04, 
       1.6949711907905522e-04, 
       1.6999999999999099e-04, 
       1.6999999999998806e-04, 
       1.6999999999999809e-04, 
       1.7000000000002267e-04, 
       1.2329989904235158e-04, 
       1.2436923400862665e-04, 
       1.2854304808762446e-04, 
       1.4683006957681496e-04, 
       1.6160435582882870e-04, 
       1.3972255583952578e-04, 
       1.5000000000001365e-04, 
       1.2809105799428647e-04, 
       1.6325830733345122e-04, 
       1.2387870153750044e-04, 
       1.2202344907059997e-04, 
       1.2750426576269410e-04, 
       1.3265229466694804e-04, 
       1.2165809851567457e-04, 
       1.2590904930981378e-04, 
       1.5730482990178394e-04, 
       1.6721244284335744e-04, 
       1.4147442375478847e-04, 
       1.5979258108469221e-04, 
       1.5828763831641872e-04, 
       1.2079998870919375e-04, 
       1.3847140225600735e-04, 
       1.6069811335272437e-04, 
       1.3991956707201230e-04, 
       1.3991956707201230e-04, 
       1.1785429316960505e-04, 
       1.2559121865270815e-04, 
       1.3420901893586887e-04, 
       1.2600707028360226e-04, 
       1.2193087664311440e-04, 
       1.3501593612698566e-04, 
       1.5239991214222232e-04, 
       1.6380710186730566e-04, 
       1.5961613541018601e-04, 
       1.4976541415855974e-04, 
       1.2107075764626484e-04, 
       1.3967694340647830e-04, 
       1.3637746409475310e-04, 
       1.6176536389318850e-04, 
       1.3865760562742341e-04, 
       1.1640969957083990e-04, 
       1.5591417147236815e-04, 
       1.6316491386601618e-04, 
       1.6291805302016873e-04, 
       1.5553617221488657e-04, 
       1.3569625693131957e-04, 
       1.4999999999995179e-04, 
       1.3934932883532909e-04, 
       1.3068547120783893e-04, 
       1.3510168282832339e-04, 
       1.6006687656677202e-04, 
       1.1953472088720430e-04, 
       1.1514603537633790e-04, 
       1.4556648430814588e-04, 
       1.5999999999997166e-04, 
       1.2013553052150146e-04, 
       1.1365076779197476e-04, 
       1.2825640608698591e-04, 
       1.2445859498741413e-04, 
       1.3261308641423491e-04, 
       1.2057430544411213e-04, 
       1.4609306144570863e-04, 
       1.4363448616454966e-04, 
       1.3354653452029715e-04, 
       1.2708441985621207e-04, 
       1.4492435764777885e-04, 
       1.5183513091055403e-04, 
       1.1157066231419675e-04, 
       1.1455643074102387e-04, 
       1.3044841616669255e-04, 
       1.1627946392463456e-04, 
       1.2958060207053849e-04, 
       1.2459594844172186e-04, 
       1.1395629374304224e-04, 
       1.5298074827407043e-04, 
       1.4522461023768444e-04, 
       1.1855328739221928e-04, 
       1.5556219022281689e-04, 
       1.4272621215920974e-04, 
       1.1824397191824585e-04, 
       1.5486374243830391e-04, 
       1.5408379332462090e-04, 
       1.3346898269536072e-04, 
       1.5398272742149797e-04, 
       1.5261939641875454e-04, 
       1.1750217811738496e-04, 
       1.1288547734143903e-04, 
       1.5187176762491777e-04, 
       1.4501878605906594e-04, 
       1.3066879402842633e-04, 
       1.2805472334957558e-04, 
       1.4709146874589748e-04, 
       1.3999999999999402e-04, 
       1.3999999999998776e-04, 
       1.5170144409404205e-04, 
       1.1306582797405172e-04, 
       1.2663497483234330e-04, 
       1.2610702170147089e-04, 
       1.5000029300247646e-04, 
       1.0673961076427949e-04, 
       1.5029886353441169e-04, 
       1.0616691504363341e-04, 
       1.5000000000001064e-04, 
       1.4999999999999467e-04, 
       1.4999999999997711e-04, 
       1.1721758607610839e-04, 
       1.1016492558308764e-04, 
       1.1167423907362062e-04, 
       1.2380937223528338e-04, 
       1.1687815407368244e-04, 
       1.4700540270982287e-04, 
       1.1128946784418891e-04, 
       1.3853901522321453e-04, 
       1.1225821419508380e-04, 
       1.4816806519414650e-04, 
       1.0769904421691774e-04, 
       1.4530878484590040e-04, 
       1.1952437979404790e-04, 
       1.1553986707149173e-04, 
       1.0878469424929832e-04, 
       1.3026565757573444e-04, 
       1.4657129988399718e-04, 
       1.4639661398271650e-04, 
       1.1866825296514568e-04, 
       1.1866653046738359e-04, 
       1.3434106521251660e-04, 
       1.3455977556224222e-04, 
       1.4368268389771411e-04, 
       1.0404929304876943e-04, 
       1.1766433410374535e-04, 
       1.0390266643685797e-04, 
       1.0803246895783221e-04, 
       1.3930464464820760e-04, 
       1.0394265719651076e-04, 
       1.1907803026006660e-04, 
       1.0668772727353113e-04, 
       1.0277041863110158e-04, 
       1.4402613366547838e-04, 
       1.1173817034373829e-04, 
       1.2542924952745800e-04, 
       1.0999999999997557e-04, 
       1.0555347211440150e-04, 
       1.4264894567343699e-04, 
       1.0337831229809064e-04, 
       1.1223481154586532e-04, 
       1.2619243783710144e-04, 
       1.1888644875430759e-04, 
       1.4302004722046026e-04, 
       1.1567467617161917e-04, 
       1.4137643514245581e-04, 
       1.1643544203566237e-04, 
       1.1118063815297272e-04, 
       1.1540624323582928e-04, 
       1.0605790651186666e-04, 
       1.1495631387861764e-04, 
       1.0581377076477180e-04, 
       1.1492862472665980e-04, 
       1.0337639564955326e-04, 
       1.4167016534912945e-04, 
       1.1490537634514367e-04, 
       1.1490465720208656e-04, 
       1.1432626144347066e-04, 
       1.1567158249878285e-04, 
       1.1256208400029416e-04, 
       1.3790069990049093e-04, 
       1.3000000000002826e-04, 
       1.2610654530554975e-04, 
       1.3830513725065290e-04, 
       1.1701195551187363e-04, 
       1.4038801587165967e-04, 
       1.0236967909843781e-04, 
       1.1636043614633647e-04, 
       1.1020472392565392e-04, 
       1.4000000000001167e-04, 
       1.4000000000000110e-04, 
       1.4000000000003953e-04, 
       1.4000000000004688e-04, 
       1.4000000000004837e-04, 
       1.3999999999996977e-04, 
       1.1126780328095909e-04, 
       1.0528455616485060e-04, 
       9.9999999999999517e-05, 
       1.0258710752567322e-04, 
       1.0329855936178343e-04, 
       1.0656022804585908e-04, 
       1.1270639282585546e-04, 
       1.3865843258503843e-04, 
       1.1706653543617880e-04, 
       1.3453066606080832e-04, 
       1.1961816208963041e-04, 
       1.1961816208963041e-04, 
       1.0608890589779587e-04, 
       1.3324356181760967e-04, 
       9.8863100740601603e-05, 
       1.2380398039220521e-04, 
       1.1501570694013844e-04, 
       1.3817409413933006e-04, 
       1.1488971493324137e-04, 
       1.3350318586923779e-04, 
       1.3683523930368416e-04, 
       1.2211504289147564e-04, 
       1.1442229980198665e-04, 
       1.0664995872739121e-04, 
       1.3460701210031291e-04, 
       1.0105275111864593e-04, 
       1.3590614420380142e-04, 
       9.9680851026209065e-05, 
       1.2333271045750381e-04, 
       1.3424193658533295e-04, 
       1.3090800856853686e-04, 
       1.2486354615009237e-04, 
       1.0999999999999088e-04, 
       1.3652648410990384e-04, 
       1.0182121231367291e-04, 
       1.0181568044506513e-04, 
       1.3601104767715580e-04, 
       1.3252436965422991e-04, 
       1.1259232341171402e-04, 
       1.2833429090195435e-04, 
       1.3501847961500257e-04, 
       9.7064206050890913e-05, 
       1.0380326753653852e-04, 
       1.2482993974796293e-04, 
       1.2482993974796293e-04, 
       1.3333633488077004e-04, 
       9.7317747424658342e-05, 
       9.4944447367566363e-05, 
       1.1183559120678446e-04, 
       1.3307008907062124e-04, 
       1.0177462781199216e-04, 
       1.3328731888404051e-04, 
       1.0854144522169795e-04, 
       9.9677531861012359e-05, 
       1.0957407953662138e-04, 
       1.3227642927314537e-04, 
       1.0839853218224298e-04, 
       1.2076369513222895e-04, 
       1.3156361648438620e-04, 
       9.3935089024272398e-05, 
       1.2423774197045632e-04, 
       9.8007735155193610e-05, 
       1.3128434577344933e-04, 
       1.0868487569435066e-04, 
       1.1908837266829746e-04, 
       9.9762104330059225e-05, 
       1.3070819875595776e-04, 
       1.3039370210925910e-04, 
       1.2655070920172213e-04, 
       1.1999999999997877e-04, 
       9.5895156411998753e-05, 
       1.2526659646256303e-04, 
       1.3000000000000611e-04, 
       1.2999999999995147e-04, 
       1.2999999999998126e-04, 
       1.2999999999998828e-04, 
       9.8965821713440779e-05, 
       1.0592771586146755e-04, 
       9.2125989360311317e-05, 
       1.2876420937829654e-04, 
       1.0426893821952235e-04, 
       1.0751163758022518e-04, 
       1.2504034534700531e-04, 
       1.2508771128184014e-04, 
       1.0406091492675854e-04, 
       1.2841970783595400e-04, 
       1.2676525261580762e-04, 
       9.3816476234606625e-05, 
       9.2594170017619586e-05, 
       1.2233015585000115e-04, 
       1.1840802640216451e-04, 
       9.7720220597622898e-05, 
       9.5830213999810896e-05, 
       1.0967056697652411e-04, 
       1.1647885698491935e-04, 
       9.7139851346291362e-05, 
       9.1073565116616358e-05, 
       9.5490474162231840e-05, 
       9.0402741737421097e-05, 
       9.6698614648289922e-05, 
       1.0999999999999496e-04, 
       1.2354530635731942e-04, 
       1.0690030384042726e-04, 
       1.2348502060723871e-04, 
       9.7609420044041060e-05, 
       9.4884054812071692e-05, 
       9.3686945301308121e-05, 
       1.0942905820567188e-04, 
       1.0388627259974351e-04, 
       9.2656937549976567e-05, 
       1.0429912183679612e-04, 
       1.0289531435946407e-04, 
       1.0160177640963846e-04, 
       1.0243124929433718e-04, 
       9.6142780469253138e-05, 
       9.9788007305391522e-05, 
       1.1938082758237188e-04, 
       1.1844657409796066e-04, 
       9.0768911189065575e-05, 
       1.2000000000000110e-04, 
       1.0218942844361111e-04, 
       9.7532388687279805e-05, 
       8.7072275886642574e-05, 
       8.9346388260607023e-05, 
       1.0374451924344620e-04, 
       9.3866317583021606e-05, 
       9.5111371387572691e-05, 
       8.9890890722588049e-05, 
       9.2657462193259730e-05, 
       1.0163860398825262e-04, 
       1.0073560073914527e-04, 
       9.4715709626865601e-05, 
       8.7336825966675513e-05, 
       9.2275712385022534e-05, 
       1.0052207887893580e-04, 
       9.4884426848954831e-05, 
       8.8048937809629319e-05, 
       1.0859335783480130e-04, 
       8.8691148872930309e-05, 
       8.8691148872930309e-05, 
       1.0157820078176118e-04, 
       9.9863531589595587e-05, 
       1.1503531872714597e-04, 
       1.2000000000001347e-04, 
       1.2000000000000212e-04, 
       9.0706050385269343e-05, 
       1.0101266460683503e-04, 
       9.8212642396666068e-05, 
       8.4604828321030098e-05, 
       8.8331369910074175e-05, 
       1.0259445924119551e-04, 
       9.9015126749100717e-05, 
       8.6002858681654773e-05, 
       9.7121283938282575e-05, 
       9.7078103007281952e-05, 
       1.0026377726197009e-04, 
       1.1468414638557733e-04, 
       1.1326780211799721e-04, 
       9.0000000000000494e-05, 
       1.1792528732497590e-04, 
       1.1243004562140722e-04, 
       1.1312972535826639e-04, 
       8.7170160819477724e-05, 
       1.1750472248573592e-04, 
       9.0720528399696460e-05, 
       9.1936902918945553e-05, 
       9.6423613918820126e-05, 
       1.0017401789283045e-04, 
       9.7549362526801463e-05, 
       1.0005182621451147e-04, 
       1.1569407166342272e-04, 
       1.0927381990370473e-04, 
       9.7915157727556642e-05, 
       1.1649512920689498e-04, 
       9.1259090477585007e-05, 
       1.1183642622410782e-04, 
       9.4547267135741534e-05, 
       9.4514442342074912e-05, 
       9.9077361324955728e-05, 
       1.0949632490976281e-04, 
       1.1551787204159648e-04, 
       8.5007216853543363e-05, 
       8.8583614831828545e-05, 
       9.5860006163252302e-05, 
       8.2998800876797816e-05, 
       9.0093729328440624e-05, 
       1.0804249175936848e-04, 
       9.7548195809030159e-05, 
       1.0285914155937444e-04, 
       8.4952673868894885e-05, 
       9.6863539106596547e-05, 
       9.8115831252059744e-05, 
       9.3180757077268361e-05, 
       1.1387447284266037e-04, 
       1.1227705897581782e-04, 
       1.1000000000000221e-04, 
       1.1366293789917179e-04, 
       1.1346928425245814e-04, 
       1.0886591197482317e-04, 
       8.2962508615956368e-05, 
       8.9975558662558737e-05, 
       9.1358363477240685e-05, 
       9.0718903197863350e-05, 
       9.0628569225002556e-05, 
       8.1637288432446456e-05, 
       1.1221529563250229e-04, 
       9.4775739184071523e-05, 
       8.3419803678503693e-05, 
       9.0195943222689040e-05, 
       9.0015738432773842e-05, 
       7.9046627205076302e-05, 
       1.1134268484170219e-04, 
       9.1853994485333035e-05, 
       7.9055260827248737e-05, 
       9.9244702823123580e-05, 
       9.0886706246580339e-05, 
       1.1090415320194132e-04, 
       9.3556617106591667e-05, 
       7.9986966676777013e-05, 
       1.0884651519342423e-04, 
       7.8389886686311476e-05, 
       9.1447014888223907e-05, 
       1.1035458197225739e-04, 
       1.0979461619772101e-04, 
       1.0999997238627179e-04, 
       9.0337247362648837e-05, 
       7.9957940107900307e-05, 
       8.4538417525963750e-05, 
       1.1000000000003342e-04, 
       1.0953458842347013e-04, 
       7.9999999999997269e-05, 
       8.2256747867474702e-05, 
       9.6238196065946447e-05, 
       1.0074983723571350e-04, 
       8.2083054248371101e-05, 
       8.2083054248371101e-05, 
       8.8238574351569185e-05, 
       9.0510674978432467e-05, 
       7.7269029351689111e-05, 
       8.9247665926683094e-05, 
       8.1254746184664830e-05, 
       8.5041351185770502e-05, 
       8.4525525824895298e-05, 
       8.2837199999628194e-05, 
       8.2837199999628194e-05, 
       9.9999999999998961e-05, 
       9.9999999999996901e-05, 
       1.0000000000000425e-04, 
       1.0818295538802235e-04, 
       7.6891272224960801e-05, 
       7.6891272224960801e-05, 
       9.3392265155965533e-05, 
       8.4143602881121271e-05, 
       8.7603940796913323e-05, 
       9.4949909936234531e-05, 
       8.8355833079739609e-05, 
       7.6416742381228480e-05, 
       9.4757610270950515e-05, 
       8.9734311554509369e-05, 
       1.0609988887412997e-04, 
       7.8498515426990930e-05, 
       8.3511680358857638e-05, 
       9.7883804870268468e-05, 
       9.0469765735941605e-05, 
       8.4523921051035615e-05, 
       8.7915191869153460e-05, 
       8.8058149316865556e-05, 
       7.6228793366155150e-05, 
       8.4530611428621755e-05, 
       1.0254735274621737e-04, 
       9.0041809451396157e-05, 
       8.0654143056511682e-05, 
       8.0577883735269748e-05, 
       8.9031283389523757e-05, 
       7.6103370431981394e-05, 
       1.0543325991552761e-04, 
       1.0511999633821307e-04, 
       8.7463475908070099e-05, 
       9.0172344873275706e-05, 
       7.8888557710059335e-05, 
       7.6122823846716420e-05, 
       1.0484604133902631e-04, 
       9.9532335429972354e-05, 
       7.8011008461987874e-05, 
       9.5768505462793252e-05, 
       7.4898126808023798e-05, 
       1.0424873772958282e-04, 
       1.0287897008709013e-04, 
       7.8991074409490562e-05, 
       9.5905465000960098e-05, 
       8.0693979535059911e-05, 
       1.0281728898164558e-04, 
       1.0375330573368589e-04, 
       8.5804998040995694e-05, 
       1.0230414205021565e-04, 
       8.8468744528276766e-05, 
       1.0160755187428885e-04, 
       8.9959802532195725e-05, 
       7.2799436588802591e-05, 
       8.3262490394561103e-05, 
       7.4383445503191855e-05, 
       9.8929041810989866e-05, 
       7.2653277445675033e-05, 
       7.9233685560548385e-05, 
       8.4727106840491888e-05, 
       1.0209067981308031e-04, 
       8.9999999999997607e-05, 
       7.8418755017624627e-05, 
       7.4710843220595095e-05, 
       7.3727200163397136e-05, 
       7.2031924528618391e-05, 
       7.5503500167489672e-05, 
       7.5533495295449589e-05, 
       8.2378048159348460e-05, 
       7.2265303630274217e-05, 
       8.2311804506703882e-05, 
       7.5008273725721692e-05, 
       8.9880023489910754e-05, 
       7.5176435855341429e-05, 
       8.4178406510053449e-05, 
       1.0108486902181352e-04, 
       7.6722527844090988e-05, 
       9.7836569561700730e-05, 
       8.5923754323962638e-05, 
       9.0276669078779740e-05, 
       1.0083648566695826e-04, 
       8.3575638885037524e-05, 
       7.1850684357491695e-05, 
       8.3541563588056397e-05, 
       9.9999968496009969e-05, 
       8.1232134268478711e-05, 
       8.1202670249458283e-05, 
       8.3041883809145410e-05, 
       9.9999999999995126e-05, 
       7.4369805809064893e-05, 
       8.0405053360720682e-05, 
       8.2673908319192931e-05, 
       7.7935913435711501e-05, 
       8.0384092157402133e-05, 
       7.3377483288319953e-05, 
       7.3377483288319953e-05, 
       9.8575025324124915e-05, 
       7.9813992409684354e-05, 
       8.6282157756054203e-05, 
       7.8009428230691512e-05, 
       8.6149260115270907e-05, 
       6.9999999999999587e-05, 
       7.1241164172232562e-05, 
       8.1968520765248807e-05, 
       7.3915617691633654e-05, 
       8.0409836568867425e-05, 
       8.0962927570370761e-05, 
       7.0995837672725042e-05, 
       7.1519846462871642e-05, 
       9.0671780755698116e-05, 
       9.8230577501271630e-05, 
       7.3215763548154211e-05, 
       7.0950396904196375e-05, 
       7.3071977656194234e-05, 
       9.5032132373625171e-05, 
       9.4940325499723091e-05, 
       9.4389402245401437e-05, 
       7.5081071066808086e-05, 
       7.3220459462588724e-05, 
       9.4011676620450971e-05, 
       9.7689180305367675e-05, 
       9.0000000000001618e-05, 
       9.7440171714916011e-05, 
       8.2801654181278574e-05, 
       7.0000000000001607e-05, 
       8.2110073562116493e-05, 
       7.1508605742953715e-05, 
       9.2568961745064066e-05, 
       9.7004438428922339e-05, 
       6.9448139879616628e-05, 
       8.4437775258814059e-05, 
       9.5782620207817340e-05, 
       9.2394804417379113e-05, 
       7.2070991491597882e-05, 
       9.6317802500946100e-05, 
       9.4355648840122120e-05, 
       6.9330853850233494e-05, 
       6.9330853850233494e-05, 
       7.9753478529611763e-05, 
       9.5940258180110991e-05, 
       6.8602725422614859e-05, 
       9.5783662687275647e-05, 
       8.4446137433537394e-05, 
       8.6386548448219724e-05, 
       7.9590911777632184e-05, 
       7.0546710876504700e-05, 
       9.5185356194954666e-05, 
       7.1328980530686558e-05, 
       8.0172575989284050e-05, 
       7.0532004944074808e-05, 
       9.2297633059039286e-05, 
       7.1419875675855980e-05, 
       7.8889822156699447e-05, 
       7.2097468393137764e-05, 
       8.8497526031193529e-05, 
       8.9999999999999762e-05, 
       6.6680549642237356e-05, 
       6.6599413841636541e-05, 
       7.7779319082704989e-05, 
       7.5608647985682878e-05, 
       9.3287887983410953e-05, 
       8.8669084639081470e-05, 
       6.9396713943279412e-05, 
       6.7332636908491996e-05, 
       6.6381118878455176e-05, 
       7.7145611396563674e-05, 
       7.0198035314649528e-05, 
       7.0191382377160892e-05, 
       6.7565343581897167e-05, 
       8.4342742543146870e-05, 
       8.4342742543146870e-05, 
       6.6833844616994985e-05, 
       9.1075084775576686e-05, 
       6.8597039600113425e-05, 
       7.8552972061075339e-05, 
       6.6236597135901294e-05, 
       6.8749341104185833e-05, 
       9.1589779982594659e-05, 
       6.7802652582552394e-05, 
       9.0371840237216981e-05, 
       9.1266176359602354e-05, 
       9.1077206891008146e-05, 
       6.7913666560699470e-05, 
       6.5292573492295425e-05, 
       6.7160839485901489e-05, 
       6.6399251384022836e-05, 
       6.5531632519939007e-05, 
       6.4420830953975086e-05, 
       6.4129429260847037e-05, 
       8.7945663414557162e-05, 
       8.1879308673372947e-05, 
       7.5245429139408744e-05, 
       7.3953169493952322e-05, 
       6.5517093991671423e-05, 
       8.5309045582437394e-05, 
       9.0000075390378611e-05, 
       8.0867871270279743e-05, 
       6.5279834467930230e-05, 
       9.0000000000001280e-05, 
       8.9999999999996374e-05, 
       8.9999999999996821e-05, 
       9.0000000000003421e-05, 
       9.0000000000003421e-05, 
       9.0000000000001307e-05, 
       8.9999999999997200e-05, 
       7.9871785573518598e-05, 
       6.8759601048570679e-05, 
       7.5004930484056384e-05, 
       8.8981045691875218e-05, 
       8.5527845712701702e-05, 
       8.2596520811133877e-05, 
       6.3564660245551418e-05, 
       6.3564660245551418e-05, 
       7.3105142678787580e-05, 
       8.1940378161010771e-05, 
       8.4217606451497077e-05, 
       7.1635180985790199e-05, 
       6.3815697361101561e-05, 
       7.3540398758123503e-05, 
       8.8132084988347349e-05, 
       6.6187421951266064e-05, 
       7.6744550415559101e-05, 
       8.7492855807599112e-05, 
       6.4329471783434409e-05, 
       7.0707089994141343e-05, 
       6.5368112311815733e-05, 
       8.5072918195078416e-05, 
       8.4405284748117950e-05, 
       7.3993591220397544e-05, 
       6.5694168104627371e-05, 
       6.5690969241841222e-05, 
       6.5974556934037681e-05, 
       8.8178087417671824e-05, 
       6.5804275354932752e-05, 
       8.5781989007628653e-05, 
       8.7268594436841837e-05, 
       8.7698454254539745e-05, 
       6.3523716565456746e-05, 
       6.2782664805147059e-05, 
       6.2947602785371875e-05, 
       7.5709235111267941e-05, 
       7.8164946699223252e-05, 
       6.2926318353639850e-05, 
       8.7066602451116795e-05, 
       6.6188837300290080e-05, 
       8.6804307605521829e-05, 
       8.7503516752477958e-05, 
       7.2710892778552434e-05, 
       6.5526676671419604e-05, 
       8.7119692468934601e-05, 
       8.3845442423676018e-05, 
       6.2932161438017685e-05, 
       8.0000000000001701e-05, 
       7.9999999999998678e-05, 
       7.9999999999999519e-05, 
       7.9999999999998096e-05, 
       6.4725568976262672e-05, 
       6.4082397875402751e-05, 
       6.1420904361906656e-05, 
       8.0201523356305477e-05, 
       8.6472488265611856e-05, 
       6.3025606566483524e-05, 
       7.5739641872558686e-05, 
       6.2858842499579202e-05, 
       7.3433826750189549e-05, 
       6.9753042806339780e-05, 
       7.4867353158233655e-05, 
       7.6181658409797231e-05, 
       6.1095693571789109e-05, 
       6.0707077108593158e-05, 
       8.0874928264925387e-05, 
       7.2457126857521179e-05, 
       7.0985841490534086e-05, 
       6.3739694237638698e-05, 
       6.8658852308327409e-05, 
       6.6536670889729419e-05, 
       8.4027243113395111e-05, 
       8.4925274827432314e-05, 
       7.2063916263562911e-05, 
       8.1998361593912876e-05, 
       6.3387811245412895e-05, 
       6.0302312366378721e-05, 
       8.2010302439041621e-05, 
       8.4545604335347903e-05, 
       6.9566134181631807e-05, 
       7.1285518319432479e-05, 
       7.9889015743517770e-05, 
       6.2169939994170183e-05, 
       6.2621428328389327e-05, 
       8.3843794703167303e-05, 
       7.0071874893443744e-05, 
       6.9898342311314589e-05, 
       7.5423868390244627e-05, 
       8.3364490792987438e-05, 
       8.4023516073628051e-05, 
       7.0446884097595110e-05, 
       6.0477205953521636e-05, 
       8.3601944322890705e-05, 
       6.2532895695482482e-05, 
       6.0000000000003268e-05, 
       6.9337576616454661e-05, 
       7.2147399057708142e-05, 
       7.4613826550478139e-05, 
       7.2047001243602128e-05, 
       6.9971356922728973e-05, 
       8.0869501741526474e-05, 
       8.2975026884193681e-05, 
       8.2777263459580538e-05, 
       7.6785306772321593e-05, 
       5.9656552169501568e-05, 
       6.1970674452659349e-05, 
       5.9255230858190536e-05, 
       5.9253192194836492e-05, 
       8.2739116416066750e-05, 
       6.0869383833468857e-05, 
       8.0864406808699158e-05, 
       7.6960243256741473e-05, 
       8.1828442372698114e-05, 
       6.6132429931755368e-05, 
       8.0339524670957564e-05, 
       7.9520753605988049e-05, 
       6.0738332847183389e-05, 
       6.7801155166521664e-05, 
       7.1434776554814736e-05, 
       7.1782064819187207e-05, 
       5.7669016428387093e-05, 
       6.7988597178578357e-05, 
       6.5299283752224353e-05, 
       5.7770529099573146e-05, 
       7.2149499281289289e-05, 
       5.7326520308117141e-05, 
       6.7283647890583752e-05, 
       5.8751699953325959e-05, 
       7.9096346889344523e-05, 
       6.2317767938422928e-05, 
       6.6867814522925489e-05, 
       8.0335984704479112e-05, 
       8.0000000000002758e-05, 
       7.9999999999998963e-05, 
       8.0000000000002189e-05, 
       8.0000000000003164e-05, 
       7.9999999999997879e-05, 
       8.0000000000002297e-05, 
       7.9999999999999735e-05, 
       8.0000000000000007e-05, 
       6.6519252973622876e-05, 
       5.6536494466905784e-05, 
       7.8013605660462348e-05, 
       7.9837751306237033e-05, 
       5.8833382772887223e-05, 
       7.8150134036085822e-05, 
       7.9677015017955863e-05, 
       6.5969169353133803e-05, 
       5.8103048496305281e-05, 
       5.9999966503040292e-05, 
       5.6474108545434415e-05, 
       5.7062411157666192e-05, 
       7.8689724458564432e-05, 
       6.5351581379482213e-05, 
       6.1823173332318599e-05, 
       6.5307855315628914e-05, 
       5.7267331134879187e-05, 
       5.8673927663774992e-05, 
       6.8607501504733658e-05, 
       6.5250104884631173e-05, 
       7.3950687960325733e-05, 
       7.8375792244723459e-05, 
       6.1611714335937885e-05, 
       6.4236197044914876e-05, 
       5.5369853546733534e-05, 
       7.4607631394892288e-05, 
       6.7242234899581670e-05, 
       6.4947281043603957e-05, 
       5.6160646610726989e-05, 
       7.7708505356304907e-05, 
       6.4079022329303412e-05, 
       5.6204394380613141e-05, 
       5.6153911757088295e-05, 
       7.3068760669381080e-05, 
       5.5818049402023691e-05, 
       7.0000000000000739e-05, 
       7.7039602817215977e-05, 
       5.7909221249900485e-05, 
       6.3803387863523009e-05, 
       7.5706378051610521e-05, 
       5.7404166084540090e-05, 
       7.6204958127061594e-05, 
       5.5627875156501482e-05, 
       6.9249275528613638e-05, 
       5.5660180861356442e-05, 
       6.0363830281187977e-05, 
       6.6591387485163435e-05, 
       6.4563079684622858e-05, 
       5.5458190355394455e-05, 
       7.6225441001027440e-05, 
       5.6112149914041319e-05, 
       5.6112149914041319e-05, 
       5.9268583277828877e-05, 
       6.9999999999997121e-05, 
       7.5789643086375563e-05, 
       7.0000000000001945e-05, 
       7.1003146118943941e-05, 
       7.2078340116240059e-05, 
       6.1902134037346168e-05, 
       7.5359985714889467e-05, 
       5.4725877573315912e-05, 
       5.8900623380256271e-05, 
       5.3589100327708243e-05, 
       6.3566154651774413e-05, 
       5.3304736699708273e-05, 
       7.3525456285339046e-05, 
       6.2313633304552190e-05, 
       6.4537691770276095e-05, 
       5.3292099533635400e-05, 
       5.4807553965406274e-05, 
       7.2928319837938513e-05, 
       5.2865201337797986e-05, 
       5.2865201337797986e-05, 
       6.2916737412755850e-05, 
       5.3176717045459786e-05, 
       5.4533976744891498e-05, 
       5.5832659831989547e-05, 
       5.3510696584172764e-05, 
       7.1300231543291307e-05, 
       5.2974600144812072e-05, 
       5.4377847940313387e-05, 
       6.1351663755275518e-05, 
       5.6215032601271803e-05, 
       5.4800822851492676e-05, 
       6.1404942457538020e-05, 
       6.3451143302306910e-05, 
       5.9584996489456378e-05, 
       5.9569387431431119e-05, 
       5.5869535544463678e-05, 
       5.4535617322689182e-05, 
       5.4688780502637356e-05, 
       5.9850479214570943e-05, 
       5.3586275762554595e-05, 
       6.1266153829070432e-05, 
       6.1266153829070432e-05, 
       5.3050299057791493e-05, 
       7.0939031655639867e-05, 
       5.2362326849137299e-05, 
       6.7212976920482721e-05, 
       5.2194764550423150e-05, 
       6.0325120855691790e-05, 
       6.3112883135151368e-05, 
       6.7535019399359163e-05, 
       5.1784527975714110e-05, 
       5.0524041830332288e-05, 
       5.8352660442955614e-05, 
       5.0574510191146357e-05, 
       5.9297863186819411e-05, 
       5.9284366611368220e-05, 
       6.8897413945949608e-05, 
       6.1108456033044211e-05, 
       7.0209409649381424e-05, 
       7.0816857552466640e-05, 
       6.8219055309236485e-05, 
       6.7877929500971134e-05, 
       6.9779979705935448e-05, 
       6.9462599853447303e-05, 
       5.7144800770106535e-05, 
       5.0308870103953731e-05, 
       5.0173190848404817e-05, 
       4.9952019638466125e-05, 
       5.3419951825346200e-05, 
       5.8466031174966944e-05, 
       6.6358685035065285e-05, 
       5.5140248012791047e-05, 
       6.8347594297426200e-05, 
       5.6960875757725883e-05, 
       6.9999999999999303e-05, 
       6.9999999999999709e-05, 
       7.0000000000004791e-05, 
       7.0000000000003423e-05, 
       7.0000000000001268e-05, 
       7.0000000000001593e-05, 
       5.9831518723280486e-05, 
       6.0293451616680224e-05, 
       6.9682258005263951e-05, 
       6.7802971065954073e-05, 
       4.9843034076844770e-05, 
       5.0624743386964780e-05, 
       5.2133621255799013e-05, 
       5.4176502438370773e-05, 
       5.5798538944435063e-05, 
       5.1000950188381827e-05, 
       6.0038400144387023e-05, 
       5.2694940463419361e-05, 
       4.9686147411923310e-05, 
       4.9379684732393815e-05, 
       5.0293386364764924e-05, 
       5.4966104262637906e-05, 
       5.2505922357650699e-05, 
       6.8943640194868587e-05, 
       5.5604439994099130e-05, 
       5.5599815750212186e-05, 
       6.5239209473791438e-05, 
       5.8395985010709600e-05, 
       4.9452519071456424e-05, 
       5.7017490852978671e-05, 
       5.5978416248899198e-05, 
       5.0264698894438015e-05, 
       6.6665926295676975e-05, 
       6.8453401918264407e-05, 
       5.6569357883090364e-05, 
       6.8335212869132154e-05, 
       4.9647129202807830e-05, 
       5.0104045650719756e-05, 
       6.3486804806258320e-05, 
       6.8298814642482164e-05, 
       5.6592329302999660e-05, 
       5.9880908325565909e-05, 
       5.4814640645869430e-05, 
       5.0081583947383345e-05, 
       5.4809868395992217e-05, 
       6.3228615564588314e-05, 
       5.4840267693430367e-05, 
       6.7782269914526991e-05, 
       5.6401907126616625e-05, 
       5.1141848434313667e-05, 
       4.9904532523365142e-05, 
       5.6391812676907800e-05, 
       6.7679656515190180e-05, 
       5.6829676507965120e-05, 
       4.9842647965109901e-05, 
       4.9842647965109901e-05, 
       5.5920225838151660e-05, 
       6.7199948373296471e-05, 
       5.6543237651659965e-05, 
       6.7157657138038626e-05, 
       6.6379400086788429e-05, 
       4.8901692508048178e-05, 
       4.9484856004237974e-05, 
       5.9996105121258811e-05, 
       5.4331009496574316e-05, 
       5.5750291038992419e-05, 
       4.8644734164384468e-05, 
       5.5961190211878463e-05, 
       5.0006049186135127e-05, 
       5.5367158497658229e-05, 
       5.8525506121777416e-05, 
       4.9067032759471781e-05, 
       5.7572266403071556e-05, 
       4.7506199980117524e-05, 
       6.4218909607501599e-05, 
       5.5091965481506400e-05, 
       5.1108817122108481e-05, 
       6.3729933344069172e-05, 
       4.7534723533173651e-05, 
       4.8721156222038537e-05, 
       5.0023741698867944e-05, 
       6.5413837799685278e-05, 
       6.6031288751394396e-05, 
       6.5185135829264157e-05, 
       4.8639939451732586e-05, 
       5.3471543155147815e-05, 
       5.3470234202847621e-05, 
       5.4235599937334449e-05, 
       4.6696674737072938e-05, 
       5.4510445522446929e-05, 
       6.3092676413622730e-05, 
       5.4480746848402336e-05, 
       6.5162083144622830e-05, 
       6.4637049800461062e-05, 
       5.8226291510811822e-05, 
       6.1727192099590223e-05, 
       6.0000000000001885e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       6.0000000000001275e-05, 
       5.9999999999999663e-05, 
       4.8222542418440093e-05, 
       5.4441070445223625e-05, 
       4.6567864860969004e-05, 
       6.3116790105446032e-05, 
       4.7811094180524836e-05, 
       6.4524095056913290e-05, 
       4.5849246173702267e-05, 
       4.7957233995658348e-05, 
       5.7735816181440496e-05, 
       4.6131248145815747e-05, 
       6.4533793363186074e-05, 
       5.0000000000000321e-05, 
       5.0000000000004712e-05, 
       5.5276263557192818e-05, 
       5.3480082940807484e-05, 
       4.5994394769164854e-05, 
       6.3954734509573069e-05, 
       5.5432228886451026e-05, 
       4.6764141061557171e-05, 
       6.1466027736689719e-05, 
       5.2895993062280609e-05, 
       4.6911290282061963e-05, 
       4.7690694162014693e-05, 
       5.5655058365349034e-05, 
       4.5381029945158757e-05, 
       6.3352082157043253e-05, 
       6.3779055751876667e-05, 
       5.4588935291063727e-05, 
       4.6891522943457257e-05, 
       4.6402637571805200e-05, 
       6.2056218560641756e-05, 
       4.6687331791065307e-05, 
       4.5627804808424103e-05, 
       4.9240192715254951e-05, 
       5.2898674187020502e-05, 
       6.1551490208796360e-05, 
       6.3037811552734289e-05, 
       4.9070222801629898e-05, 
       6.0987675671929428e-05, 
       5.2482560993622402e-05, 
       6.2693667708373256e-05, 
       4.5401556783972952e-05, 
       5.5439002342805138e-05, 
       4.5055104093155815e-05, 
       5.2426711298138298e-05, 
       5.3305595690661849e-05, 
       5.9752633549208924e-05, 
       5.0821434509652217e-05, 
       5.4279540398605293e-05, 
       4.7726625323203568e-05, 
       6.0691526144331518e-05, 
       4.9446545397466637e-05, 
       5.8993801931120903e-05, 
       5.1719168035984416e-05, 
       5.6845072614997615e-05, 
       4.4359350661943662e-05, 
       4.4359286421192773e-05, 
       4.4683099791801449e-05, 
       4.6767846170332439e-05, 
       5.3795497044884278e-05, 
       4.6195354719139865e-05, 
       4.6195354719139865e-05, 
       4.6461494876802746e-05, 
       6.2294893931289029e-05, 
       5.9513949740074086e-05, 
       5.2803638788575085e-05, 
       4.6932931108333560e-05, 
       4.8425237850651634e-05, 
       5.1688611403087938e-05, 
       4.6103121211759707e-05, 
       4.4646855408922731e-05, 
       6.1971106494966159e-05, 
       5.2277120400292120e-05, 
       5.1268125044547678e-05, 
       4.5127168589627302e-05, 
       4.5559762378008551e-05, 
       4.6163909833683394e-05, 
       4.7622202315681006e-05, 
       5.1982162966691362e-05, 
       5.5561372275647245e-05, 
       4.4656735501918160e-05, 
       5.1112817075278030e-05, 
       4.5312263980815190e-05, 
       4.5984452296017492e-05, 
       5.0232938537276863e-05, 
       5.1438616540721909e-05, 
       4.9029391646492582e-05, 
       4.5847551201734067e-05, 
       5.9081814719610968e-05, 
       4.4758322468941056e-05, 
       4.4527218154763130e-05, 
       6.1196370752057935e-05, 
       5.9423192614379616e-05, 
       5.8426261204146650e-05, 
       4.3895787312711132e-05, 
       5.0823674877389410e-05, 
       4.4159592683339312e-05, 
       4.3183102749350650e-05, 
       5.9136410236724295e-05, 
       4.3048086403028848e-05, 
       4.6179897675467623e-05, 
       5.8611348001854098e-05, 
       5.5893493755321943e-05, 
       5.0781068754357385e-05, 
       5.0780953224542056e-05, 
       6.0400631391043782e-05, 
       5.0341563564047152e-05, 
       4.3795232393121785e-05, 
       5.9902762742168291e-05, 
       4.4058926915834686e-05, 
       5.0803928317872478e-05, 
       5.0655147822811326e-05, 
       5.9999789296617017e-05, 
       5.7861267789368643e-05, 
       5.9999975703747769e-05, 
       5.9604915867661808e-05, 
       5.9999999999997521e-05, 
       6.0000000000002583e-05, 
       5.9999999999998145e-05, 
       5.9999999999999920e-05, 
       5.9999999999995651e-05, 
       6.0000000000004630e-05, 
       5.9944858705767063e-05, 
       4.3729611121582715e-05, 
       4.2763863510998176e-05, 
       4.2902994091444175e-05, 
       5.1454732779578809e-05, 
       4.3518194438287823e-05, 
       4.2888991335543819e-05, 
       4.9567611860653887e-05, 
       5.9533922178796983e-05, 
       4.5448074194076151e-05, 
       4.4405629696802045e-05, 
       5.3005444466418007e-05, 
       4.4921370867962235e-05, 
       4.8214499541383813e-05, 
       4.8213488773082908e-05, 
       4.3903468407753180e-05, 
       4.8915317934496880e-05, 
       4.9178017325850189e-05, 
       4.2520817149067737e-05, 
       4.2520504244822200e-05, 
       4.9344675971299254e-05, 
       5.8870317247688816e-05, 
       5.8399868947111986e-05, 
       4.3479357167338926e-05, 
       4.8728647427657396e-05, 
       5.7060797803891980e-05, 
       4.1644535146986210e-05, 
       4.1666276614238178e-05, 
       4.9559935505613789e-05, 
       5.8638330734578074e-05, 
       4.4745532451904581e-05, 
       4.4505303279567565e-05, 
       4.9681559965770856e-05, 
       4.8810769695560470e-05, 
       5.7722904115022887e-05, 
       4.2225036450746274e-05, 
       4.7257314886941546e-05, 
       4.9058219335361414e-05, 
       4.4520702901876963e-05, 
       4.9006220801975131e-05, 
       4.3582639489121810e-05, 
       4.3570056626258740e-05, 
       4.7775474352826803e-05, 
       4.8756172778324964e-05, 
       5.8294821702607921e-05, 
       5.4503569091204679e-05, 
       4.2571569169167861e-05, 
       4.1398880755584905e-05, 
       4.4746669370521632e-05, 
       5.7998520207417815e-05, 
       5.6473935260511194e-05, 
       4.2300274501921504e-05, 
       5.7574970358682534e-05, 
       5.0084073722433577e-05, 
       5.6172722192566461e-05, 
       5.7565227528944879e-05, 
       5.5171156018821212e-05, 
       4.5150424730827053e-05, 
       4.9594894324417839e-05, 
       4.3575408626722596e-05, 
       4.7614424518065227e-05, 
       5.7403818408117554e-05, 
       4.9786074368996327e-05, 
       4.4456846563521062e-05, 
       5.6184746781328476e-05, 
       4.6172751015155248e-05, 
       5.6499997081157638e-05, 
       4.7607030550006311e-05, 
       4.0980629850883083e-05, 
       4.8134523305034848e-05, 
       4.7496310285260817e-05, 
       5.5759141294071027e-05, 
       5.4694078524475342e-05, 
       4.1168613794556891e-05, 
       5.6668906413007280e-05, 
       4.7065722633256996e-05, 
       4.0931194581401073e-05, 
       5.4448599722087120e-05, 
       4.8245317363303676e-05, 
       5.5935545589140298e-05, 
       4.3518100129399014e-05, 
       4.5258864513686095e-05, 
       4.7263814418240710e-05, 
       4.1628071441651432e-05, 
       4.1732668982753145e-05, 
       5.0839724294963075e-05, 
       5.5643895616299083e-05, 
       4.0874114034633423e-05, 
       4.0213414711940769e-05, 
       4.0602556199225371e-05, 
       3.9798917532351760e-05, 
       4.3502164958705832e-05, 
       4.0631247379848673e-05, 
       4.0000000000002219e-05, 
       5.3113323259739849e-05, 
       5.5960958129507828e-05, 
       5.5945503817689012e-05, 
       4.6479271990845340e-05, 
       5.4205030096753215e-05, 
       4.8038892964860316e-05, 
       4.8038892964860316e-05, 
       4.1249531890938287e-05, 
       4.9665036416254743e-05, 
       4.6393396209071189e-05, 
       4.8586203699758706e-05, 
       4.7643822159193113e-05, 
       5.0113951824612399e-05, 
       4.5644259921143167e-05, 
       4.0773247314461056e-05, 
       3.9886220440567700e-05, 
       5.0324536252164542e-05, 
       5.3774498427009434e-05, 
       4.1452454878147792e-05, 
       4.2226452827185388e-05, 
       5.3606606757098600e-05, 
       5.5148734182742428e-05, 
       3.9603226822633545e-05, 
       3.9847876969610408e-05, 
       3.9253995717913027e-05, 
       5.5209158908184279e-05, 
       4.0171539049671001e-05, 
       4.0229147335329989e-05, 
       4.7710572602320027e-05, 
       4.7801983139927655e-05, 
       4.0561034351946743e-05, 
       4.1492018703479000e-05, 
       5.1984560871044604e-05, 
       4.4244819373534095e-05, 
       4.2299679283002287e-05, 
       3.9384739031196680e-05, 
       4.8030171539722245e-05, 
       4.8934955333675140e-05, 
       3.8824445198949964e-05, 
       3.9346225258795313e-05, 
       5.4545005410651440e-05, 
       4.2248437508809583e-05, 
       5.4129559070840305e-05, 
       5.4432246400521999e-05, 
       4.7098358680081894e-05, 
       4.2951615229805261e-05, 
       5.0000000000004854e-05, 
       4.9999999999996845e-05, 
       4.4762456978213309e-05, 
       4.4214161872150917e-05, 
       4.9999999999996086e-05, 
       5.0000000000001602e-05, 
       4.0055337265233531e-05, 
       5.1875721802107281e-05, 
       5.4173851804815632e-05, 
       5.2342087512995231e-05, 
       4.0000000000004150e-05, 
       4.3645280927087385e-05, 
       3.9904217891268223e-05, 
       4.2376994520283985e-05, 
       4.2297303009571453e-05, 
       4.8879613522080589e-05, 
       4.0301879896279338e-05, 
       5.1009666875063227e-05, 
       4.0291884609681158e-05, 
       4.3495256436175411e-05, 
       4.5730042942677488e-05, 
       5.3787158368192670e-05, 
       4.8822304653832097e-05, 
       3.8810779524411854e-05, 
       5.3302791238473550e-05, 
       4.6120431269476313e-05, 
       4.5423750287827508e-05, 
       4.4419468484612943e-05, 
       4.1842659261109005e-05, 
       5.3342683361897172e-05, 
       3.7831057841672347e-05, 
       4.4876542213077566e-05, 
       5.3339953067899664e-05, 
       5.1307576498520121e-05, 
       4.9795340273845722e-05, 
       4.0551968891482593e-05, 
       4.2942181427477144e-05, 
       4.6580825491456696e-05, 
       4.4170839478869739e-05, 
       5.3225469148860561e-05, 
       4.3552526954001138e-05, 
       3.9936472321541530e-05, 
       3.7856822941797808e-05, 
       4.1424703336531182e-05, 
       4.4109848575973033e-05, 
       5.2309653545515140e-05, 
       5.0650435375008185e-05, 
       5.1035434637487536e-05, 
       3.9622605062936089e-05, 
       3.8590028790683536e-05, 
       4.1438395651877721e-05, 
       4.0552969630143932e-05, 
       5.2489713088133530e-05, 
       3.9345854051111601e-05, 
       3.7758300027101911e-05, 
       5.0750461565384194e-05, 
       3.7402233097626411e-05, 
       4.5911510414821101e-05, 
       4.4320155766962862e-05, 
       4.3554590023744675e-05, 
       4.4584164971959899e-05, 
       4.4580957478613705e-05, 
       3.8434965776471489e-05, 
       4.6170046403484089e-05, 
       4.3373197223144499e-05, 
       4.7388291342434926e-05, 
       4.3353766181158540e-05, 
       3.9106412200694077e-05, 
       3.8355144926800230e-05, 
       3.8376808888131592e-05, 
       4.9800727126423536e-05, 
       5.2057090972482660e-05, 
       4.3435654224056656e-05, 
       5.0063937364252527e-05, 
       3.7185351056380096e-05, 
       4.0916337656372547e-05, 
       3.7096229741818396e-05, 
       3.6781744627271483e-05, 
       5.1811525370187312e-05, 
       3.7833741387392980e-05, 
       3.6894970698006109e-05, 
       3.7151459339555959e-05, 
       4.9346160114750382e-05, 
       3.6633695532275512e-05, 
       3.7762083455162853e-05, 
       3.7713261214658515e-05, 
       4.2905385005542004e-05, 
       4.2721777594869631e-05, 
       4.2871158898182830e-05, 
       4.2856075784532639e-05, 
       3.7585011634027380e-05, 
       4.2787150309464686e-05, 
       3.8282707689072949e-05, 
       3.8547816795841882e-05, 
       3.8729983163976672e-05, 
       4.3802596034923394e-05, 
       4.9156593596046735e-05, 
       4.6674306788075115e-05, 
       3.6793600117994928e-05, 
       3.7672256743152476e-05, 
       3.8982643011500457e-05, 
       4.2095568646817963e-05, 
       4.9156596716464992e-05, 
       5.0818749307970015e-05, 
       4.7638625828669324e-05, 
       4.8743388042410356e-05, 
       4.8743388042410356e-05, 
       5.0920316189094351e-05, 
       4.2501847283851815e-05, 
       4.2770545601811098e-05, 
       3.8452002514831642e-05, 
       4.1560207605759910e-05, 
       3.6810146127261362e-05, 
       4.5106844834294229e-05, 
       3.7543407113834333e-05, 
       3.6999743152688838e-05, 
       4.5228094725095993e-05, 
       3.8120433144414718e-05, 
       4.2373393125234178e-05, 
       3.7339244915619255e-05, 
       5.0606222122548995e-05, 
       5.0171211793907470e-05, 
       4.8103703637198708e-05, 
       4.8657954980057095e-05, 
       4.0706698873243219e-05, 
       4.3300414987776052e-05, 
       3.7990417315996674e-05, 
       3.6171231268334320e-05, 
       4.7449667996545013e-05, 
       4.9457418300433907e-05, 
       4.5212475188456699e-05, 
       3.6277581681391359e-05, 
       3.8263565084705065e-05, 
       3.6853405453953792e-05, 
       5.0000000000004759e-05, 
       4.9999973253849495e-05, 
       4.9999309915235958e-05, 
       4.1708997021255155e-05, 
       5.0000000000004814e-05, 
       5.0000000000000212e-05, 
       4.9999999999997848e-05, 
       4.9999999999995706e-05, 
       4.9999999999997285e-05, 
       5.0000000000003695e-05, 
       5.0000000000001554e-05, 
       5.0000000000001825e-05, 
       4.9999999999996851e-05, 
       3.5408176441764625e-05, 
       4.8187191630835852e-05, 
       4.8459644167773767e-05, 
       3.5859946054074073e-05, 
       4.8698843911162814e-05, 
       4.9792661553601065e-05, 
       4.1093125575671377e-05, 
       4.9757504353982201e-05, 
       4.9627132456014287e-05, 
       4.7884173780204098e-05, 
       4.1288674931018946e-05, 
       4.9658525568868731e-05, 
       3.8935284750652676e-05, 
       4.2159821135385924e-05, 
       4.4696201374075211e-05, 
       3.8442269209632602e-05, 
       4.8383706284272943e-05, 
       3.8924999276684364e-05, 
       3.9944878971996325e-05, 
       4.9349400786323520e-05, 
       4.9043796063861990e-05, 
       3.6121592269412959e-05, 
       4.9267370544174806e-05, 
       4.6975451485351150e-05, 
       4.2420704851971450e-05, 
       4.9034265060333103e-05, 
       4.1000495993128493e-05, 
       3.6802914218379274e-05, 
       4.0994805730348300e-05, 
       3.5221795706480916e-05, 
       3.8855941063243146e-05, 
       4.1781485763613834e-05, 
       3.9538020650094244e-05, 
       4.0405750604700326e-05, 
       4.8900676149172333e-05, 
       4.2481998373067597e-05, 
       3.9039416360268234e-05, 
       3.9506867683778882e-05, 
       4.0696066776815846e-05, 
       4.8901500731813548e-05, 
       4.3173906267770743e-05, 
       4.0790710937704446e-05, 
       3.5496172630263295e-05, 
       4.6955589107558505e-05, 
       3.5221231340790295e-05, 
       4.6516135729167509e-05, 
       3.8238364870826670e-05, 
       4.4237793935597831e-05, 
       4.0802853698156234e-05, 
       4.1243064236859773e-05, 
       3.4548359607475459e-05, 
       3.6197101228016460e-05, 
       4.7748596570621995e-05, 
       4.0291274483495406e-05, 
       4.4257207759580529e-05, 
       4.8222350422121991e-05, 
       3.5094377888614524e-05, 
       3.6656924485171226e-05, 
       4.4073511658702931e-05, 
       3.9637449702643444e-05, 
       4.7938399710402696e-05, 
       3.4979671298360411e-05, 
       4.0410673297021768e-05, 
       3.9035231630682992e-05, 
       4.8090271341060706e-05, 
       3.9942231822511489e-05, 
       3.5083336168095262e-05, 
       3.9879539664203457e-05, 
       3.4832545622196622e-05, 
       3.7611091268450171e-05, 
       4.7232898177139256e-05, 
       3.9929004153968073e-05, 
       3.9894524243583448e-05, 
       3.7656906083208395e-05, 
       3.9819448513341300e-05, 
       3.4932540161114768e-05, 
       4.0518355283772929e-05, 
       3.4240921757062046e-05, 
       4.4066943738975884e-05, 
       4.7423610948041679e-05, 
       3.9144823329152139e-05, 
       4.0908748760812218e-05, 
       3.9786816243739836e-05, 
       3.3798432919652239e-05, 
       3.6880076521315806e-05, 
       3.9622751844855506e-05, 
       3.7290121774435469e-05, 
       4.7438291226495707e-05, 
       3.7284906634344718e-05, 
       3.3720419377599246e-05, 
       4.1041161905783108e-05, 
       4.0980629850874382e-05, 
       4.7510617390791483e-05, 
       3.9780995073475945e-05, 
       4.7436785653119979e-05, 
       3.4110563425326465e-05, 
       4.6985681246361248e-05, 
       3.3579559186638764e-05, 
       4.5272077524089852e-05, 
       3.9258009246260173e-05, 
       3.9275724047458070e-05, 
       4.3233587279662087e-05, 
       3.3452342038935922e-05, 
       3.4643963026278155e-05, 
       3.7732727961902534e-05, 
       3.8557410131031713e-05, 
       4.6562965856360202e-05, 
       3.7496918797710279e-05, 
       4.2679478413180879e-05, 
       4.1980440831123302e-05, 
       3.9327164957137778e-05, 
       3.8539291729924192e-05, 
       3.5849749348037440e-05, 
       4.5256285029026452e-05, 
       3.9630041236095456e-05, 
       4.5953910310499518e-05, 
       4.5574874226989893e-05, 
       3.8625314259170787e-05, 
       3.3750070092269913e-05, 
       3.5148565179809681e-05, 
       3.7512951978688678e-05, 
       4.4999841703036331e-05, 
       4.3891583404918332e-05, 
       4.6758295575699226e-05, 
       3.6498940058986615e-05, 
       3.7468372359729450e-05, 
       3.8945032376840196e-05, 
       3.7999984347982119e-05, 
       3.8784192773063788e-05, 
       3.5740706722842011e-05, 
       3.8641321225121811e-05, 
       3.4905316698490314e-05, 
       3.3067235639361119e-05, 
       3.3067235639361119e-05, 
       4.4707889030744459e-05, 
       3.9793249231023071e-05, 
       4.4610283210327343e-05, 
       4.4619029328667574e-05, 
       4.5206494724860396e-05, 
       3.3597725751776402e-05, 
       4.5494440138931664e-05, 
       3.2948751530114775e-05, 
       3.6135019425927445e-05, 
       4.6165735096238155e-05, 
       3.4680960890658572e-05, 
       4.6121954759801873e-05, 
       3.9606224310459238e-05, 
       4.6040365011574947e-05, 
       4.5968564234313613e-05, 
       3.3216908878284660e-05, 
       3.6121696320554319e-05, 
       3.6118904596417354e-05, 
       3.9965886553234756e-05, 
       3.7960942959211673e-05, 
       3.3015728625472228e-05, 
       3.3129972365801607e-05, 
       3.2695181427336741e-05, 
       3.4250440130200798e-05, 
       4.4448736542906044e-05, 
       3.7852540849177154e-05, 
       3.2502809213086196e-05, 
       3.4171962459343609e-05, 
       3.8131071084777378e-05, 
       3.4566648187038441e-05, 
       3.5013399755891196e-05, 
       3.2701903314242563e-05, 
       3.2431439664343020e-05, 
       4.5552301774535584e-05, 
       3.4438876315432489e-05, 
       4.3878641516741261e-05, 
       3.2304560984191068e-05, 
       3.2304339024064865e-05, 
       4.5264105814598043e-05, 
       4.3547681545482941e-05, 
       3.8540889506387398e-05, 
       4.5436591216783690e-05, 
       4.5437702730735851e-05, 
       3.7922218768194438e-05, 
       3.2415214542006272e-05, 
       3.2564457829914071e-05, 
       3.7304745716350087e-05, 
       3.2323617004262415e-05, 
       3.7278195702878041e-05, 
       3.8373241105207387e-05, 
       3.2573701986389598e-05, 
       3.7646342196827161e-05, 
       3.9421272924813623e-05, 
       3.9501624254035637e-05, 
       3.7390695190544806e-05, 
       3.6069459201781000e-05, 
       3.6062786061320047e-05, 
       4.3389064988984674e-05, 
       3.2627976048381635e-05, 
       4.4284561057134100e-05, 
       4.3601160427165270e-05, 
       3.4635328753529157e-05, 
       3.2149093834813746e-05, 
       3.3689855536866871e-05, 
       4.3216426236862585e-05, 
       3.2855792616886550e-05, 
       3.6064336406735929e-05, 
       3.3239424588928948e-05, 
       3.2827854533171908e-05, 
       4.4989275875484362e-05, 
       3.3584452108383197e-05, 
       4.2939446871580752e-05, 
       3.9222848092840874e-05, 
       4.2901170371280392e-05, 
       3.2233595369570919e-05, 
       3.1728628126545885e-05, 
       4.2855442589356341e-05, 
       3.6565023474546315e-05, 
       3.8298606578543848e-05, 
       3.2950984186462133e-05, 
       3.6532315097328446e-05, 
       3.2711571986310915e-05, 
       3.1967181879025635e-05, 
       4.4603547965731295e-05, 
       3.2643724001595769e-05, 
       3.2877730296234751e-05, 
       3.8593381181162502e-05, 
       4.2050707116091333e-05, 
       3.8193056584885661e-05, 
       3.2078421068134895e-05, 
       3.1563250816349307e-05, 
       3.3464700965702173e-05, 
       3.2295557314115479e-05, 
       3.1986838654988278e-05, 
       3.9658619568854458e-05, 
       3.1662339414483392e-05, 
       3.7031531628240052e-05, 
       3.1456022157395817e-05, 
       3.2736656275658254e-05, 
       3.1971041941472359e-05, 
       3.7061890999114858e-05, 
       3.1589612751515528e-05, 
       4.0842903727528385e-05, 
       3.8693550208363229e-05, 
       3.2652593079599549e-05, 
       3.2426118877173239e-05, 
       3.6662990908906790e-05, 
       4.4059443264123390e-05, 
       3.6646883609913904e-05, 
       3.3306620110803306e-05, 
       3.3945017549259451e-05, 
       3.4136085370227129e-05, 
       3.6709397279896857e-05, 
       4.3125553242843388e-05, 
       3.1275604022377079e-05, 
       3.9257993709319027e-05, 
       3.9241051090370256e-05, 
       3.5394353850374905e-05, 
       3.2612747134150309e-05, 
       3.7224761328834978e-05, 
       3.2490770041774935e-05, 
       3.2490770041774935e-05, 
       3.6874231398616637e-05, 
       3.3254272551589700e-05, 
       3.9086867039923709e-05, 
       3.9920204316030868e-05, 
       3.1321268734910817e-05, 
       3.3197088231901820e-05, 
       3.2692032624632953e-05, 
       4.2014459126866369e-05, 
       3.8134282383394992e-05, 
       3.4234246414402742e-05, 
       3.0862267877250008e-05, 
       4.2128536942499734e-05, 
       4.3476692460793331e-05, 
       4.2162720180259138e-05, 
       3.6232159466694327e-05, 
       3.6145824384960646e-05, 
       4.1965305380788002e-05, 
       4.0365250021186470e-05, 
       4.0000000000000850e-05, 
       4.0000000000000850e-05, 
       3.9999999999999305e-05, 
       3.9999999999996453e-05, 
       4.0000000000002870e-05, 
       3.3986957773966361e-05, 
       3.5607724550697976e-05, 
       3.5737714106043158e-05, 
       3.9342411982573345e-05, 
       3.6871693082504237e-05, 
       3.4233897666132191e-05, 
       3.3649464144804624e-05, 
       3.8566780597453067e-05, 
       3.6434982670612244e-05, 
       4.1276516310770805e-05, 
       3.0532456307141396e-05, 
       3.8264347262882043e-05, 
       3.1635269277834769e-05, 
       3.5723356591098958e-05, 
       3.2102106252287437e-05, 
       3.0419722688384099e-05, 
       3.4221942049885680e-05, 
       3.1474261673403497e-05, 
       3.1752446217172570e-05, 
       3.1568797847569072e-05, 
       3.5685618997503639e-05, 
       3.0308628964463674e-05, 
       4.2650090718053424e-05, 
       4.1018583566474474e-05, 
       4.1989409445850875e-05, 
       3.5442604585587855e-05, 
       4.1804760524109131e-05, 
       3.5086256291839228e-05, 
       3.5810119152410999e-05, 
       3.6628483041529297e-05, 
       3.0907207572010308e-05, 
       3.2253429739559717e-05, 
       3.5249617570816589e-05, 
       3.7457071533236017e-05, 
       3.4624590675350692e-05, 
       2.9970807997730558e-05, 
       4.1941240331531230e-05, 
       4.0588555882643076e-05, 
       3.0510810776208460e-05, 
       4.1640392292448351e-05, 
       3.7482181996118599e-05, 
       3.7533477045684776e-05, 
       3.1399725230672576e-05, 
       4.1076066446429675e-05, 
       3.5471457575904934e-05, 
       3.5336312716016542e-05, 
       4.1365813328181145e-05, 
       3.0682845205326052e-05, 
       3.8084296033612795e-05, 
       3.1145184029437977e-05, 
       3.4544896126377564e-05, 
       4.1544997694196912e-05, 
       2.9856066937675739e-05, 
       3.4341734991271874e-05, 
       3.5934040914515704e-05, 
       4.1328180697925300e-05, 
       2.9420608768881635e-05, 
       2.9631070620672174e-05, 
       3.8163247198778232e-05, 
       3.3917607508924799e-05, 
       3.0401451076688072e-05, 
       2.9337676955112569e-05, 
       3.9360086855353777e-05, 
       3.0659718172078284e-05, 
       2.9560824323877005e-05, 
       3.5333635863312106e-05, 
       2.9100768296810072e-05, 
       2.9783808149289946e-05, 
       3.5065947122296143e-05, 
       3.7292062299485572e-05, 
       3.0277993161779557e-05, 
       3.3666400298233992e-05, 
       3.4914136181191511e-05, 
       4.0819206289891856e-05, 
       3.1849335917306130e-05, 
       3.3745003547145633e-05, 
       3.0447909355766206e-05, 
       4.0735350301540957e-05, 
       2.8983900069099673e-05, 
       3.8053646227503343e-05, 
       3.6958017338921532e-05, 
       3.5066206410744284e-05, 
       3.1878230350892937e-05, 
       3.3670823840325130e-05, 
       4.0190618964093548e-05, 
       2.9611762148052136e-05, 
       3.0215572942741212e-05, 
       3.0978705150100671e-05, 
       3.0975116670585716e-05, 
       3.9205890377581426e-05, 
       3.0281667020559267e-05, 
       3.9596006158142123e-05, 
       2.9506513717721074e-05, 
       3.3582847240993350e-05, 
       3.4531769840299485e-05, 
       3.9762088604433533e-05, 
       2.9269265439836080e-05, 
       3.3380022999946215e-05, 
       3.9926407533688928e-05, 
       4.0104613244331801e-05, 
       3.5653058167747033e-05, 
       3.3004955267642635e-05, 
       3.4392622906690786e-05, 
       3.0370417067300626e-05, 
       3.1890047488315731e-05, 
       3.0022017500562490e-05, 
       4.0000003317432134e-05, 
       3.3523408508543471e-05, 
       2.9536082766360954e-05, 
       4.0000000000001989e-05, 
       3.9999999999997374e-05, 
       3.9999999999997374e-05, 
       4.0000000000003371e-05, 
       3.9999999999997909e-05, 
       3.9999999999996947e-05, 
       3.9999999999996175e-05, 
       4.0000000000001148e-05, 
       4.0000000000001148e-05, 
       3.9999999999995246e-05, 
       3.9999999999995246e-05, 
       3.9999999999995246e-05, 
       4.0000000000003832e-05, 
       4.0000000000001894e-05, 
       3.9999999999999481e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       3.1039159892549503e-05, 
       3.3151722660290517e-05, 
       2.8807688058454221e-05, 
       3.9692599755261031e-05, 
       3.4680733919685062e-05, 
       3.2826256554884176e-05, 
       3.8288161925822885e-05, 
       3.4211304406796375e-05, 
       2.9097792994242360e-05, 
       2.8673230559331065e-05, 
       2.8811579142860116e-05, 
       2.9638784793380221e-05, 
       2.8065502476368636e-05, 
       3.7018431922052273e-05, 
       3.1869994579495824e-05, 
       3.7611367219734550e-05, 
       2.9524917489233808e-05, 
       3.8654230861269744e-05, 
       3.8813655096499063e-05, 
       2.8242134260049362e-05, 
       3.3856798589886510e-05, 
       3.6989711056837685e-05, 
       2.8097092194560257e-05, 
       3.2589239900482450e-05, 
       3.0839366780925382e-05, 
       3.0913455647600671e-05, 
       2.8818795374229811e-05, 
       2.7955639415557753e-05, 
       3.8757866071503815e-05, 
       3.2194194668433212e-05, 
       3.1832447493921153e-05, 
       3.2344070446398742e-05, 
       3.7832374205409138e-05, 
       3.4832277859316015e-05, 
       3.2463844769771561e-05, 
       3.2424248890195081e-05, 
       3.0355743865040782e-05, 
       2.7547836766078887e-05, 
       2.8129289982065617e-05, 
       3.0065679269866430e-05, 
       3.0779226484141891e-05, 
       3.4562999970220125e-05, 
       3.3652238989943404e-05, 
       2.7519212508309735e-05, 
       2.8865537325274782e-05, 
       3.4536959440905748e-05, 
       3.6507321778070506e-05, 
       3.1157695519207996e-05, 
       3.0483603651375193e-05, 
       3.7971213702066805e-05, 
       2.7407710915057017e-05, 
       3.8106781457880113e-05, 
       3.6722116726253600e-05, 
       3.8440947193767489e-05, 
       2.8442006127918854e-05, 
       2.8551101375556656e-05, 
       2.9364739526051272e-05, 
       2.7770689262710977e-05, 
       3.4948618872287616e-05, 
       3.2064281429115760e-05, 
       2.7240377824836826e-05, 
       2.7308570833989415e-05, 
       2.8963467376296397e-05, 
       3.4712231642814477e-05, 
       2.7160857684904483e-05, 
       2.9327116841256538e-05, 
       2.8283712735542613e-05, 
       3.6136978569258876e-05, 
       2.8668276433282417e-05, 
       2.7239632087237912e-05, 
       2.7719525839683526e-05, 
       3.3183686698092545e-05, 
       2.8824367755484238e-05, 
       3.4994505665292073e-05, 
       3.8208999710328840e-05, 
       2.8487098891718443e-05, 
       2.7048322588066142e-05, 
       2.7624506260606630e-05, 
       2.7407524675903619e-05, 
       3.0381424342910978e-05, 
       2.8076267860498583e-05, 
       2.8512040773140239e-05, 
       3.4594742813447289e-05, 
       3.6441362101213101e-05, 
       3.6985978886589346e-05, 
       3.2253721185282010e-05, 
       3.2253721185282010e-05, 
       3.0305771304502396e-05, 
       3.7126481627199639e-05, 
       2.7167060333542747e-05, 
       3.8015892034047229e-05, 
       2.7954033262396059e-05, 
       3.4834437483229113e-05, 
       2.6985426747145644e-05, 
       2.7649760287491335e-05, 
       2.9695116186657036e-05, 
       2.8511435627383066e-05, 
       2.9999999999999726e-05, 
       2.9661643064631994e-05, 
       2.8266445242005317e-05, 
       3.0170551527330453e-05, 
       3.3860016083471914e-05, 
       3.1354773595006093e-05, 
       3.1488710584772834e-05, 
       3.1871636737527572e-05, 
       3.2564457829911849e-05, 
       2.7084809742553798e-05, 
       3.6062606881400475e-05, 
       3.7582830542471488e-05, 
       2.7047751291354494e-05, 
       3.2797133827564757e-05, 
       3.1903572974689500e-05, 
       2.8050886898984705e-05, 
       3.1352959491326954e-05, 
       2.6784146102882057e-05, 
       3.7522219749239128e-05, 
       3.6463180721163240e-05, 
       2.9471994694710249e-05, 
       3.1108916789167925e-05, 
       2.7894556225965309e-05, 
       2.6820357129538413e-05, 
       2.7245005270820060e-05, 
       2.6811103636298072e-05, 
       2.7593033786732819e-05, 
       3.6762135698496261e-05, 
       2.8293699618172031e-05, 
       3.3353514605442349e-05, 
       2.6705205624747917e-05, 
       3.3798794399179892e-05, 
       3.2423489616891279e-05, 
       3.7160835699223923e-05, 
       3.0907835110374876e-05, 
       3.7058958692587823e-05, 
       2.7304455332863470e-05, 
       2.7304111819551494e-05, 
       2.6519729660628462e-05, 
       3.7062079460459438e-05, 
       3.0062882662813043e-05, 
       3.7058611877109611e-05, 
       2.8139906563482852e-05, 
       3.4713878905424213e-05, 
       2.9921452903837636e-05, 
       2.9433641374759845e-05, 
       3.5930641659530661e-05, 
       3.2670098265821272e-05, 
       2.9074316244459978e-05, 
       2.9359739277691220e-05, 
       3.5104646844348376e-05, 
       3.5646814842964232e-05, 
       3.2622747588318853e-05, 
       3.5065917243724365e-05, 
       2.8773241079992690e-05, 
       2.8731474658252576e-05, 
       2.7097738525304627e-05, 
       2.7782372773298685e-05, 
       3.2933332913816304e-05, 
       2.7056097000847204e-05, 
       3.6155579985282791e-05, 
       2.9799808496960990e-05, 
       2.6233988731786980e-05, 
       2.6130143230018705e-05, 
       3.6258588441386774e-05, 
       2.6163119095811521e-05, 
       2.8276312058127089e-05, 
       2.6146171841921433e-05, 
       2.6862760269538149e-05, 
       2.6735396114970181e-05, 
       2.7899427385733393e-05, 
       3.2918184537306320e-05, 
       3.1036130902228970e-05, 
       3.1285251208887923e-05, 
       2.9105560696208592e-05, 
       3.0336594799645449e-05, 
       2.7876501617199792e-05, 
       2.9398011659190779e-05, 
       2.7620316784509255e-05, 
       2.7189639453186989e-05, 
       2.6862415599288675e-05, 
       3.6316695117972320e-05, 
       3.0266322085404291e-05, 
       3.1191891642868792e-05, 
       3.0283657005292182e-05, 
       2.9706145186971074e-05, 
       3.5092031119106683e-05, 
       2.6055786019617233e-05, 
       3.1552377055574032e-05, 
       2.5944034925236034e-05, 
       2.9298939491059567e-05, 
       2.8813682520249190e-05, 
       3.0277973911366052e-05, 
       2.6265346091618312e-05, 
       3.4313801964971653e-05, 
       2.8292579250612668e-05, 
       2.5710834411363957e-05, 
       2.7239934780247225e-05, 
       3.0108920080324501e-05, 
       2.6574339259353398e-05, 
       2.6574339259353398e-05, 
       2.6603702800408181e-05, 
       2.9945973306640608e-05, 
       2.7206813548484243e-05, 
       2.9914109083881459e-05, 
       3.5939277874200750e-05, 
       3.5696300850418049e-05, 
       3.5924106303404742e-05, 
       3.5820888095548639e-05, 
       2.6025289810035557e-05, 
       2.9850375573323215e-05, 
       3.2598410800682624e-05, 
       2.9355508235389601e-05, 
       2.7471010759664385e-05, 
       2.9387690540557073e-05, 
       2.6316839759040683e-05, 
       2.6415309608286847e-05, 
       2.6904606101369186e-05, 
       3.5419608296105436e-05, 
       2.7672076424565236e-05, 
       3.3809375937864830e-05, 
       2.5425493738492722e-05, 
       3.4329154938284909e-05, 
       3.4656630397368111e-05, 
       2.7502645097905004e-05, 
       3.0494655538212031e-05, 
       2.9591574561031932e-05, 
       3.5190724841144002e-05, 
       2.9484112638212692e-05, 
       2.6786852929363294e-05, 
       2.6669375901811111e-05, 
       2.9510662695274036e-05, 
       2.9125017675692073e-05, 
       2.5741754881077874e-05, 
       3.3652278080096861e-05, 
       3.5320334181141426e-05, 
       2.8303395946093349e-05, 
       2.6906276959935654e-05, 
       2.6017119619940396e-05, 
       3.5324200559418312e-05, 
       2.6306490025824851e-05, 
       3.4974827291506094e-05, 
       3.4826313408925401e-05, 
       2.7734781787149045e-05, 
       2.5597195153969135e-05, 
       2.4918202035402118e-05, 
       2.8939703220608540e-05, 
       3.5093573708270734e-05, 
       3.4997661313409507e-05, 
       3.3008834750735440e-05, 
       2.7525276060311697e-05, 
       2.8277938055810536e-05, 
       2.8277938055810536e-05, 
       3.1207843012308419e-05, 
       2.6940578710871507e-05, 
       2.5579835732558207e-05, 
       2.6915905993071593e-05, 
       3.4975701650819561e-05, 
       2.4957367328132936e-05, 
       2.4957367328132936e-05, 
       2.5397844703296914e-05, 
       2.6602745558242253e-05, 
       2.5187278275559250e-05, 
       2.5467888641003731e-05, 
       3.0974890461460283e-05, 
       2.5339616967127634e-05, 
       2.9626916887394795e-05, 
       2.7619726290716077e-05, 
       3.4236061275192604e-05, 
       3.4632668262892203e-05, 
       2.8413896081275994e-05, 
       3.4773454537139780e-05, 
       2.5660063211378296e-05, 
       3.1816931570003146e-05, 
       2.8810928217630149e-05, 
       3.2402344758379857e-05, 
       2.4622803671398517e-05, 
       3.3884337582830962e-05, 
       2.4745841016557243e-05, 
       2.8726927520080040e-05, 
       3.1607938084618696e-05, 
       3.0619368734278299e-05, 
       2.5103214760719228e-05, 
       2.6115418378948737e-05, 
       2.5216345416957959e-05, 
       3.4433248607461847e-05, 
       3.4003150069056831e-05, 
       2.6340049545712366e-05, 
       2.8430479102709681e-05, 
       3.4371986165039712e-05, 
       2.5130572792497025e-05, 
       3.4053832022543250e-05, 
       2.8551589404528400e-05, 
       2.8439695471795520e-05, 
       2.9328249229205305e-05, 
       3.4276449161730333e-05, 
       3.4285934210744179e-05, 
       2.4832250539552716e-05, 
       2.4218334389904042e-05, 
       3.4158494666143246e-05, 
       2.8463467350339630e-05, 
       2.5128338253532572e-05, 
       2.8849586357006848e-05, 
       2.4239171304499631e-05, 
       2.5847121689263722e-05, 
       2.4185026488689110e-05, 
       2.4667869299451022e-05, 
       2.4346446619042764e-05, 
       3.0824640538398941e-05, 
       2.4498728162961109e-05, 
       2.8302190214678204e-05, 
       2.7394076866844276e-05, 
       3.1153783358348083e-05, 
       2.3968252203040222e-05, 
       2.6052604265088291e-05, 
       2.8126871028984526e-05, 
       2.5603416304878284e-05, 
       2.8985004808456365e-05, 
       3.3589667866692417e-05, 
       2.3977472847857841e-05, 
       3.2227122415746155e-05, 
       2.5558419731325131e-05, 
       3.3690049600477206e-05, 
       2.4381272702297991e-05, 
       3.2346196530799784e-05, 
       2.5097609460247766e-05, 
       2.9905857527565588e-05, 
       3.1439720562106301e-05, 
       3.0018041084816379e-05, 
       3.2250837542768495e-05, 
       2.4304876842231052e-05, 
       3.3099265112919020e-05, 
       3.3552675389070232e-05, 
       2.9733487432063609e-05, 
       2.5558621357365356e-05, 
       2.5558621357365356e-05, 
       2.3990190883500398e-05, 
       2.5729830495522413e-05, 
       2.9362739923969074e-05, 
       2.3770852707783487e-05, 
       2.5028438173173765e-05, 
       3.2319648557118898e-05, 
       2.7822681342999946e-05, 
       2.4895968693405063e-05, 
       2.4895968693405063e-05, 
       2.7009589427507814e-05, 
       2.7432225192304068e-05, 
       2.8082407773897595e-05, 
       2.3589132420774609e-05, 
       2.7690075248950416e-05, 
       2.8518180427709347e-05, 
       2.3569263208211297e-05, 
       3.2793897924720190e-05, 
       2.4100745386675573e-05, 
       2.4279370515244482e-05, 
       3.1609514834965040e-05, 
       2.4837349791581452e-05, 
       2.8070337592737301e-05, 
       3.3143920272160123e-05, 
       2.7592067227439496e-05, 
       3.0171198763932223e-05, 
       2.4287564525222680e-05, 
       2.3989320553293856e-05, 
       2.7390072147498527e-05, 
       3.2518549524648607e-05, 
       2.4312910540716258e-05, 
       2.7390933708753500e-05, 
       2.6836730589191729e-05, 
       2.4637614711543699e-05, 
       2.8937923332251048e-05, 
       2.8934320666911560e-05, 
       2.3443795183707881e-05, 
       3.2966884370293322e-05, 
       2.3499458233154702e-05, 
       2.6756094696910096e-05, 
       3.1622971315236877e-05, 
       2.7380720405324034e-05, 
       2.4552508401457934e-05, 
       2.4552508401457934e-05, 
       2.4592888205323805e-05, 
       2.3479089004450081e-05, 
       3.2033337234628308e-05, 
       3.2837736143775106e-05, 
       2.3494746002207493e-05, 
       3.1688839173662951e-05, 
       3.2558652334238170e-05, 
       2.4496665591143735e-05, 
       3.2453869714794841e-05, 
       3.0000168399366491e-05, 
       3.0389503000317718e-05, 
       2.5367567861720414e-05, 
       3.1013184082052318e-05, 
       2.7206863554059752e-05, 
       2.5631609024973996e-05, 
       3.2632379530041206e-05, 
       2.7173856454263550e-05, 
       2.3139042828271667e-05, 
       2.5910488673821892e-05, 
       2.3835090517009871e-05, 
       3.2676096095116880e-05, 
       2.3115738546502348e-05, 
       2.5149840529202073e-05, 
       3.0000000000003914e-05, 
       2.5103814782062271e-05, 
       2.5561453121161013e-05, 
       2.4011247562584379e-05, 
       2.8229759568981670e-05, 
       3.0954827358492883e-05, 
       2.9999999999995010e-05, 
       2.9999999999995972e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.6638076735884128e-05, 
       2.9800443358011636e-05, 
       2.9957569938508312e-05, 
       2.6644792797369336e-05, 
       3.2449119475693635e-05, 
       3.2307270591818477e-05, 
       2.3855033202891477e-05, 
       2.2919945327084375e-05, 
       3.1181947945255791e-05, 
       3.0966962655152635e-05, 
       2.3618829129352344e-05, 
       2.6729763253723063e-05, 
       2.6251435400914067e-05, 
       2.6992323055423699e-05, 
       2.8808352255079773e-05, 
       3.1674498987854625e-05, 
       2.8845868164023834e-05, 
       2.2855762949261972e-05, 
       2.4475093841118160e-05, 
       3.0469422424589034e-05, 
       3.2121103410085548e-05, 
       2.6738752853282381e-05, 
       2.6199303457985351e-05, 
       2.5917549322748828e-05, 
       2.3972067650946521e-05, 
       2.3642004655701465e-05, 
       3.0111710482434150e-05, 
       2.2690055869077828e-05, 
       2.3972371441272996e-05, 
       2.7429533393715458e-05, 
       2.6609783593678770e-05, 
       3.0418182789536737e-05, 
       3.0729933159896485e-05, 
       3.0519892781556383e-05, 
       3.0519892781556383e-05, 
       3.1927209051499637e-05, 
       3.1412944191601312e-05, 
       2.6695502529920774e-05, 
       2.2747197677377404e-05, 
       2.2747197677377404e-05, 
       2.6927910644424263e-05, 
       2.3159964613851685e-05, 
       2.3449501588639427e-05, 
       3.1129140282794502e-05, 
       2.2661199232478790e-05, 
       3.1756464109022012e-05, 
       3.0000000000003331e-05, 
       3.1715957258018557e-05, 
       3.0493385472890526e-05, 
       2.2533690673819412e-05, 
       2.8841698703111353e-05, 
       2.7915941786718987e-05, 
       2.2787211817273076e-05, 
       2.6298737908416582e-05, 
       2.5754281679001387e-05, 
       2.3252148712137995e-05, 
       2.7892934451784692e-05, 
       2.7951013451930176e-05, 
       2.2465698897257985e-05, 
       2.2440742103618684e-05, 
       2.9542188794899120e-05, 
       2.4290663709578578e-05, 
       2.6350256389484496e-05, 
       2.5734503827461567e-05, 
       2.2412536595775360e-05, 
       2.7124462690205231e-05, 
       2.7177674193646515e-05, 
       3.1618651481483873e-05, 
       2.2432426536989266e-05, 
       2.4189720357826928e-05, 
       2.6648169829749547e-05, 
       2.7164941871740675e-05, 
       2.8077366384963156e-05, 
       2.6578976457764752e-05, 
       2.6212878484818197e-05, 
       2.2788776745599077e-05, 
       2.3072473188098390e-05, 
       2.7429999248861801e-05, 
       2.9908124403016638e-05, 
       2.2874307841218250e-05, 
       2.9376128973940538e-05, 
       3.0834070661572225e-05, 
       3.0834070661572225e-05, 
       2.4923884977834289e-05, 
       3.1182792115445008e-05, 
       2.9391533845460121e-05, 
       2.2537043954458916e-05, 
       2.3663334500167565e-05, 
       2.4082184739369677e-05, 
       2.2059381617659331e-05, 
       2.7013731654013261e-05, 
       2.5866503261118416e-05, 
       3.1123506807834984e-05, 
       3.1058236474875583e-05, 
       2.2042690047119963e-05, 
       2.3501143967570068e-05, 
       2.5264475481345029e-05, 
       2.2291403303108609e-05, 
       2.5789041976159144e-05, 
       2.5752625404385004e-05, 
       2.2419785171064484e-05, 
       2.3032330547245072e-05, 
       2.5500808155918871e-05, 
       2.2007811230651764e-05, 
       2.8595489040144257e-05, 
       2.5743081743900583e-05, 
       3.0833856686922384e-05, 
       2.2953013898141161e-05, 
       2.8060586309824298e-05, 
       2.2049482271398904e-05, 
       2.5559309080125467e-05, 
       2.1906982419196531e-05, 
       2.1774947899687872e-05, 
       2.5850814625073328e-05, 
       2.8677002986771678e-05, 
       2.3973213706637462e-05, 
       2.5456386562570028e-05, 
       3.0686887089838080e-05, 
       2.9860940400375449e-05, 
       2.5914801880035124e-05, 
       3.0582199910042621e-05, 
       2.9436827774699843e-05, 
       2.5595288355060954e-05, 
       2.9210645747847836e-05, 
       2.7180281847628045e-05, 
       2.2483651761501816e-05, 
       2.4667152066401473e-05, 
       2.1893746690499035e-05, 
       2.1933041582336083e-05, 
       3.0058277854270268e-05, 
       2.2133560849025269e-05, 
       2.8096083071044486e-05, 
       2.5344389356141837e-05, 
       2.9443800220647262e-05, 
       2.9951036745156710e-05, 
       2.9945849344433220e-05, 
       2.5093598549897843e-05, 
       2.4843453873112511e-05, 
       2.7371136082427610e-05, 
       2.5072399592830789e-05, 
       2.5214185493224736e-05, 
       2.5214185035996223e-05, 
       2.5004525511801097e-05, 
       2.8979278302965828e-05, 
       2.5186051604522636e-05, 
       2.5348828265287213e-05, 
       2.5244359735749165e-05, 
       2.9326966998297700e-05, 
       2.2697014348075652e-05, 
       2.9394368400131825e-05, 
       2.1813612638002743e-05, 
       2.5046979639083853e-05, 
       2.4157479821597049e-05, 
       2.3222770682420255e-05, 
       2.3314403055625338e-05, 
       2.2570661138101929e-05, 
       2.4908783871480569e-05, 
       2.2505905708169625e-05, 
       2.6258638767916868e-05, 
       2.3434371640934816e-05, 
       3.0133423094276347e-05, 
       2.7591028764093684e-05, 
       2.2061710501047537e-05, 
       2.1628895259523690e-05, 
       2.1358429561149283e-05, 
       2.1427494407500420e-05, 
       2.3167503033159669e-05, 
       2.2930017223228041e-05, 
       3.0000000000001031e-05, 
       3.0000000000003592e-05, 
       2.9999999999998828e-05, 
       3.0000000000004216e-05, 
       2.9999999999999855e-05, 
       2.9999999999998398e-05, 
       3.0000000000004487e-05, 
       3.0000000000001681e-05, 
       3.0000000000001105e-05, 
       2.9999999999999310e-05, 
       2.9999999999998435e-05, 
       2.9999999999998473e-05, 
       3.0000000000001292e-05, 
       2.9999999999997765e-05, 
       3.0000000000001674e-05, 
       2.9999999999999940e-05, 
       2.9999999999999940e-05, 
       2.9999999999999113e-05, 
       2.9999999999999205e-05, 
       2.9999999999999205e-05, 
       2.9999999999999205e-05, 
       3.0000000000003369e-05, 
       3.0000000000003369e-05, 
       2.9999999999998937e-05, 
       2.9999999999996423e-05, 
       3.0000000000004934e-05, 
       3.0000000000004859e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.7977786049804914e-05, 
       2.4781209860024590e-05, 
       2.1192390191768415e-05, 
       2.1192390191768415e-05, 
       2.2681904047265785e-05, 
       2.1881636715936563e-05, 
       2.9360956315481421e-05, 
       2.1831868130968220e-05, 
       2.9707174814456383e-05, 
       2.8404473381637825e-05, 
       2.9380613966485565e-05, 
       2.9373491294223167e-05, 
       2.1185461954203410e-05, 
       2.7203812003774421e-05, 
       2.5018477899419453e-05, 
       2.8712262856748577e-05, 
       2.9431709214940752e-05, 
       2.1692084655893151e-05, 
       2.7655972980607960e-05, 
       2.1897724635915417e-05, 
       2.4093131058151122e-05, 
       2.5790479923422541e-05, 
       2.9550736953917446e-05, 
       2.9369626226073478e-05, 
       2.1394745953434777e-05, 
       2.1888100646980633e-05, 
       2.0904220768728736e-05, 
       2.9465166307386468e-05, 
       2.5334505925107406e-05, 
       2.1375703784200034e-05, 
       2.1290565166949428e-05, 
       2.7069699230712933e-05, 
       2.6902666915836786e-05, 
       2.1883213951128004e-05, 
       2.4074314693100887e-05, 
       2.8562172022423441e-05, 
       2.4375815007945829e-05, 
       2.6115525504441513e-05, 
       2.1264298669881058e-05, 
       2.3181063810889197e-05, 
       2.9195675414178763e-05, 
       2.1835144903334811e-05, 
       2.0933184905867349e-05, 
       2.9223290880005068e-05, 
       2.0767081796558923e-05, 
       2.2678513821181670e-05, 
       2.9022835142669393e-05, 
       2.0740605279931568e-05, 
       2.2959758470424977e-05, 
       2.1756379400670785e-05, 
       2.6227865079976737e-05, 
       2.4892524129874611e-05, 
       2.4204463163150561e-05, 
       2.4222973587626428e-05, 
       2.8954710823700344e-05, 
       2.1667908739356850e-05, 
       2.2257311143512702e-05, 
       2.9023120902266073e-05, 
       2.3962968444884479e-05, 
       2.0637003746839989e-05, 
       2.0707409205838624e-05, 
       2.4902832791433148e-05, 
       2.0729249339574016e-05, 
       2.0720092835467932e-05, 
       2.2159633949716909e-05, 
       2.8901253631823843e-05, 
       2.4013863790456743e-05, 
       2.8415383314476248e-05, 
       2.3837597752080925e-05, 
       2.2460361097944093e-05, 
       2.2460361097944093e-05, 
       2.1627859486552423e-05, 
       2.2026778583341010e-05, 
       2.2026778583341010e-05, 
       2.0409577908840788e-05, 
       2.0797163145969286e-05, 
       2.2026357651294454e-05, 
       2.1896156195968891e-05, 
       2.2493976131117383e-05, 
       2.3561936361104048e-05, 
       2.3826450639880401e-05, 
       2.3662155848727076e-05, 
       2.7313184250191240e-05, 
       2.1545807116821245e-05, 
       2.0663068717321193e-05, 
       2.1109864205007314e-05, 
       2.2297094153571563e-05, 
       2.8587967460179920e-05, 
       2.3942355574288436e-05, 
       2.6953238336552894e-05, 
       2.1769339705180965e-05, 
       2.3724586078780852e-05, 
       2.7749220345302695e-05, 
       2.1274716056221353e-05, 
       2.3660199491775747e-05, 
       2.1684275785027130e-05, 
       2.1127601084820226e-05, 
       2.2245465324112150e-05, 
       2.4134492556523332e-05, 
       2.4134492556523332e-05, 
       2.0702109877429871e-05, 
       2.5415065232737208e-05, 
       2.5049164336646217e-05, 
       2.3711982590933556e-05, 
       2.0903996480810332e-05, 
       2.5477118471368847e-05, 
       2.7066217879222983e-05, 
       2.7066217879222983e-05, 
       2.0087756354107088e-05, 
       2.7407872901961955e-05, 
       2.5752217941525236e-05, 
       2.3509414389390902e-05, 
       2.2137284082853892e-05, 
       2.7868825096435729e-05, 
       2.1103925210061340e-05, 
       2.0000000000003136e-05, 
       2.7960841711629158e-05, 
       2.0128222232666396e-05, 
       2.8218563122518728e-05, 
       2.5887484385715143e-05, 
       2.7080687592378696e-05, 
       2.0868424239093153e-05, 
       2.2393731783960902e-05, 
       2.1124097199525199e-05, 
       2.0646704939798299e-05, 
       2.1425648591584204e-05, 
       2.5913907913736285e-05, 
       2.0677363797620225e-05, 
       2.3510576371202193e-05, 
       2.7057327699278204e-05, 
       2.2231911984833502e-05, 
       2.0054709668727553e-05, 
       2.0931915255503046e-05, 
       2.6613011953124452e-05, 
       2.6896352990146805e-05, 
       1.9899446274643278e-05, 
       2.0352943326531300e-05, 
       1.9845156033951324e-05, 
       2.0841047026937218e-05, 
       1.9825607164398136e-05, 
       2.1032338146034649e-05, 
       2.7804096630990566e-05, 
       2.6708948925327193e-05, 
       2.0997523170847318e-05, 
       2.0251870492957298e-05, 
       2.3145181527697719e-05, 
       1.9735478473128204e-05, 
       1.9723699235144497e-05, 
       2.7789289624749064e-05, 
       2.3329997431946800e-05, 
       2.0124318481218082e-05, 
       2.6253986385102155e-05, 
       2.5954718992651673e-05, 
       2.3990190883495407e-05, 
       2.1847220710856864e-05, 
       1.9881665767775377e-05, 
       2.3837718355342495e-05, 
       2.4834849775167029e-05, 
       1.9845056688366133e-05, 
       2.2984918779398254e-05, 
       2.2051791107932350e-05, 
       2.7687978543504833e-05, 
       2.0124311182856182e-05, 
       2.2969720901820586e-05, 
       2.3379719824416396e-05, 
       2.3045673572194488e-05, 
       2.4401844797352267e-05, 
       2.3017821626053330e-05, 
       2.1009850269871334e-05, 
       2.4604439170047656e-05, 
       2.6502057660116060e-05, 
       2.0915044009404437e-05, 
       2.0557490844970100e-05, 
       2.2303791106947136e-05, 
       2.3568849536529308e-05, 
       2.7487887027958263e-05, 
       2.7243831302811930e-05, 
       2.7401549863970587e-05, 
       2.2844170635079498e-05, 
       2.5428459492323464e-05, 
       2.0449537282602713e-05, 
       2.7315086935866378e-05, 
       2.7321089736325289e-05, 
       2.2223161106287887e-05, 
       2.0761232511071340e-05, 
       2.2722968226404221e-05, 
       2.0375912450819512e-05, 
       2.1982535867854892e-05, 
       2.3760552111706295e-05, 
       2.4657303494521758e-05, 
       2.0009857162887701e-05, 
       2.3726374972784535e-05, 
       2.1342289921849532e-05, 
       2.2108625191178818e-05, 
       2.2107918319979281e-05, 
       2.0502905389639388e-05, 
       2.5119528436742323e-05, 
       1.9516726581836720e-05, 
       2.7191646189580940e-05, 
       2.7108638240423049e-05, 
       2.2581685122227867e-05, 
       2.4813088265878294e-05, 
       2.2251956490045338e-05, 
       2.1105202602793973e-05, 
       2.3565849863946431e-05, 
       2.3563061638380849e-05, 
       2.0984009709726171e-05, 
       2.7090345596650079e-05, 
       2.3092205076770237e-05, 
       2.2508397829033177e-05, 
       2.5237376640927738e-05, 
       2.6845718547956329e-05, 
       2.0269194115185238e-05, 
       1.9178006383137801e-05, 
       2.6943891374647128e-05, 
       2.4011762522962640e-05, 
       2.2198128590104049e-05, 
       2.6661800044791447e-05, 
       2.0246033691061394e-05, 
       2.3782960195173511e-05, 
       1.9077211035467425e-05, 
       2.0130524220147914e-05, 
       1.9055041365690265e-05, 
       2.0061290439785591e-05, 
       2.2352715474766112e-05, 
       1.9368651876453975e-05, 
       2.3069805257315336e-05, 
       2.6867279556170112e-05, 
       2.2539563108794425e-05, 
       1.9999999999999588e-05, 
       2.3426979083492160e-05, 
       1.9348046020852360e-05, 
       2.2268611352024889e-05, 
       2.3366362904907280e-05, 
       2.2134011819243862e-05, 
       2.5834600287792073e-05, 
       2.2215721280363147e-05, 
       2.6660259341866286e-05, 
       2.2955372524408110e-05, 
       2.4461292269898077e-05, 
       2.5742500761044551e-05, 
       2.2173339694577063e-05, 
       2.6201512269285688e-05, 
       1.9638095676474543e-05, 
       1.9638095676474543e-05, 
       2.6608693063682616e-05, 
       1.9851434587407395e-05, 
       1.9283086234511842e-05, 
       1.9767970879306379e-05, 
       1.9271186973558166e-05, 
       2.5946749959651857e-05, 
       2.5820386282369037e-05, 
       1.9896871203278959e-05, 
       2.2067358137384505e-05, 
       1.9270178024292573e-05, 
       2.6130655900719434e-05, 
       2.6507400532632928e-05, 
       2.6407499725398140e-05, 
       2.6407499725398140e-05, 
       1.9111331917637347e-05, 
       2.2381794194760051e-05, 
       2.0198999310674607e-05, 
       2.1939827897244711e-05, 
       2.2034500481798385e-05, 
       1.9334266572284064e-05, 
       2.2039202088095241e-05, 
       2.2555828107000100e-05, 
       1.8694500382365884e-05, 
       2.0000000000003627e-05, 
       1.9999999999999195e-05, 
       2.6397551980700827e-05, 
       1.9282667617978935e-05, 
       1.8682110802826321e-05, 
       2.0021454763490297e-05, 
       1.9369387123481113e-05, 
       2.1897888923757808e-05, 
       2.1886458127008223e-05, 
       1.8941901803643449e-05, 
       2.1575136822679403e-05, 
       2.1690264765648639e-05, 
       2.2546380808982550e-05, 
       1.9108770638817880e-05, 
       2.3793034923037692e-05, 
       1.9646049935609245e-05, 
       1.9646049935609245e-05, 
       2.4898340085266668e-05, 
       1.8702432222567177e-05, 
       2.1337110066930517e-05, 
       1.8754960889276679e-05, 
       2.1920838420412080e-05, 
       1.8745661317307797e-05, 
       2.2012993885403050e-05, 
       1.9662509837507125e-05, 
       2.5149343542068934e-05, 
       2.1835443517498511e-05, 
       2.1432576408466843e-05, 
       2.0890346652158772e-05, 
       2.0041285728824034e-05, 
       2.0201550973463700e-05, 
       2.2369988673688493e-05, 
       2.1082677410618358e-05, 
       1.9475892497164422e-05, 
       2.0430841162105277e-05, 
       2.5616881089262331e-05, 
       2.5561524976553051e-05, 
       2.2256720600395467e-05, 
       1.8575965995141460e-05, 
       2.1582267125339719e-05, 
       2.0380664050224007e-05, 
       2.4045915975920568e-05, 
       2.5413815641187998e-05, 
       1.9376536056028707e-05, 
       2.1736695350079003e-05, 
       1.8632468678587944e-05, 
       2.1373167301566084e-05, 
       2.5243812758073397e-05, 
       2.5514214161906033e-05, 
       2.5833897822691343e-05, 
       1.8603444773941460e-05, 
       1.8329881415074994e-05, 
       1.9763816058288280e-05, 
       1.9763816058288280e-05, 
       2.2641428627869301e-05, 
       2.1437640160066925e-05, 
       1.8618202683601985e-05, 
       2.3850057869665633e-05, 
       2.0257013634953510e-05, 
       2.5813702743402554e-05, 
       2.5161700961535786e-05, 
       2.4500308605386585e-05, 
       2.1581356787528601e-05, 
       1.9319643385351092e-05, 
       1.9195393822148234e-05, 
       1.8318699574533651e-05, 
       1.8322037504694979e-05, 
       1.9687517411603891e-05, 
       2.2092577492403505e-05, 
       2.1181450893396919e-05, 
       1.8441641969427584e-05, 
       2.0690315870433976e-05, 
       2.0767286104586677e-05, 
       2.5668860259396340e-05, 
       1.9999999999996691e-05, 
       1.8216488992965863e-05, 
       2.1448785391892901e-05, 
       1.9128160866352327e-05, 
       1.8192542188358763e-05, 
       1.9620107864103088e-05, 
       2.1331047324169167e-05, 
       2.1301162457843403e-05, 
       2.5094107345774041e-05, 
       2.5345688678480327e-05, 
       1.8545317730810682e-05, 
       2.1501412452381147e-05, 
       2.2142057021603587e-05, 
       2.2142057021603587e-05, 
       2.1135275541214730e-05, 
       1.8494265276753072e-05, 
       2.5550953038270339e-05, 
       1.9564063852232794e-05, 
       1.9000981440101349e-05, 
       1.8655156555063814e-05, 
       2.5445532659369853e-05, 
       2.3830599867500915e-05, 
       1.8028754047776348e-05, 
       2.4137725314921818e-05, 
       2.1125424240387695e-05, 
       2.1258657835825322e-05, 
       2.4059491545958614e-05, 
       2.0242709299960099e-05, 
       1.7949021871998564e-05, 
       2.0463012973560527e-05, 
       1.9292773873423911e-05, 
       2.3065646511066904e-05, 
       1.9073307490169593e-05, 
       1.7924314853027964e-05, 
       2.0817589538067585e-05, 
       1.7914729486010503e-05, 
       2.1375688443046898e-05, 
       1.9481609739293776e-05, 
       2.2195282689139113e-05, 
       2.5033573767916607e-05, 
       1.8806642782522944e-05, 
       1.8682168791402342e-05, 
       1.9078981517641491e-05, 
       2.0824183766240837e-05, 
       1.7707290961288495e-05, 
       2.4970512379188960e-05, 
       1.8185021483994586e-05, 
       1.8261401155073962e-05, 
       2.3069120608504299e-05, 
       2.3069120608504299e-05, 
       2.0682214549822705e-05, 
       1.8626575200708320e-05, 
       1.9916922034440399e-05, 
       1.8640395583295617e-05, 
       1.8640395583295617e-05, 
       1.7614400745337066e-05, 
       2.0416043178336156e-05, 
       2.4589376309026510e-05, 
       1.8217254129423565e-05, 
       1.9413476597831931e-05, 
       2.4643901058988932e-05, 
       1.7574523066846941e-05, 
       2.4640116028100371e-05, 
       2.4640116028100371e-05, 
       2.4696147490686706e-05, 
       2.0491678912308683e-05, 
       1.8957136937250149e-05, 
       2.4657848950645196e-05, 
       2.4652030414921975e-05, 
       2.0000000000002583e-05, 
       2.4611661546530794e-05, 
       2.1530717511554056e-05, 
       2.0470849578236086e-05, 
       2.4219917468853571e-05, 
       2.3621308320517317e-05, 
       2.4568622360301528e-05, 
       1.9589116629270218e-05, 
       2.2748107542386935e-05, 
       2.3791632997449517e-05, 
       2.1023325737590410e-05, 
       1.7911409449688488e-05, 
       1.7575495673165250e-05, 
       2.0433389575409986e-05, 
       2.2341400891196384e-05, 
       1.7307849619763936e-05, 
       1.7734620493079325e-05, 
       1.7521271150068636e-05, 
       2.4381782421029007e-05, 
       1.7792815146452841e-05, 
       2.2937659393765543e-05, 
       2.0549308294811750e-05, 
       2.0912790456574726e-05, 
       1.9708477221209908e-05, 
       2.0669294698547308e-05, 
       1.8278942905448100e-05, 
       1.7300223475597531e-05, 
       2.1258409994565152e-05, 
       1.7298936232489338e-05, 
       2.4230916131844008e-05, 
       2.0260814967144021e-05, 
       2.1574472545965845e-05, 
       1.7355504855513649e-05, 
       2.4253538317534649e-05, 
       2.4192694215193304e-05, 
       1.8538766686584639e-05, 
       2.3425112168981819e-05, 
       2.2870792928003896e-05, 
       2.0039949015281509e-05, 
       1.8071154484291160e-05, 
       2.0320444398046786e-05, 
       1.8628515855767538e-05, 
       2.0527350002612239e-05, 
       1.7425273847949847e-05, 
       2.3806116634567082e-05, 
       2.1361339222984295e-05, 
       1.9832363163017510e-05, 
       2.2980753572720978e-05, 
       1.7245410421208641e-05, 
       1.7245410421208641e-05, 
       1.7663116451282980e-05, 
       1.8171953776881295e-05, 
       1.8756189313190220e-05, 
       1.7244694892905224e-05, 
       1.8169556565617902e-05, 
       1.7683924920349020e-05, 
       1.8342545908721153e-05, 
       2.3579778641489104e-05, 
       2.3579778641489104e-05, 
       2.2905822401600713e-05, 
       2.0093647699069538e-05, 
       1.8408002083955171e-05, 
       2.3488438511504516e-05, 
       2.3642086720562421e-05, 
       1.6970077202054217e-05, 
       1.7898575231892403e-05, 
       2.3531961271458973e-05, 
       2.3887809854504896e-05, 
       1.6958551214762053e-05, 
       2.3872360119663840e-05, 
       1.9839895212892159e-05, 
       1.8458838042259074e-05, 
       2.3895184889918358e-05, 
       1.9780310285029106e-05, 
       1.8142992038481507e-05, 
       1.6904485802710315e-05, 
       1.7145380567773708e-05, 
       1.8678559660865059e-05, 
       1.8676933459739412e-05, 
       2.3644979735206557e-05, 
       1.7106401237428762e-05, 
       2.3782731311723148e-05, 
       1.9608438369471971e-05, 
       1.9152380286727763e-05, 
       1.8940503300687458e-05, 
       2.3692946635835630e-05, 
       2.1390713579329950e-05, 
       1.9136096999761468e-05, 
       1.9999999999998680e-05, 
       1.7068780125830173e-05, 
       1.9685800512948540e-05, 
       2.2061355733952665e-05, 
       1.6756109219293494e-05, 
       1.9017772303267000e-05, 
       2.1684262333435649e-05, 
       1.8012208786142678e-05, 
       1.8012208786142678e-05, 
       2.2730062662028911e-05, 
       2.3569072491019156e-05, 
       1.6684917332045796e-05, 
       1.7080947158174366e-05, 
       1.6784263583268978e-05, 
       2.0945619289330842e-05, 
       2.2529044614750575e-05, 
       1.9364933915967557e-05, 
       2.0319266578649607e-05, 
       1.6644766925135311e-05, 
       1.6681531455427336e-05, 
       1.6692472119204529e-05, 
       2.3033399633323115e-05, 
       1.9332705412142697e-05, 
       1.7519492748762508e-05, 
       1.9330753562258820e-05, 
       1.6999776530199162e-05, 
       1.6829521316593051e-05, 
       2.0428762420327374e-05, 
       1.8591388650631442e-05, 
       1.7364302853916416e-05, 
       1.6560156314027844e-05, 
       1.6560156314027844e-05, 
       1.7349040404026116e-05, 
       2.1016574098868176e-05, 
       1.6757367952506360e-05, 
       1.7393157970824468e-05, 
       1.7504296991640123e-05, 
       1.6985127267371464e-05, 
       2.3269985597394429e-05, 
       2.3319167888592631e-05, 
       1.9356801909263181e-05, 
       1.7498754834802578e-05, 
       1.6807251875556978e-05, 
       1.6777951013067581e-05, 
       1.9157356595498308e-05, 
       1.9997507073835037e-05, 
       1.6551709086558985e-05, 
       2.1906997649233641e-05, 
       1.9544192400014738e-05, 
       2.1043724346603977e-05, 
       1.8240446434156144e-05, 
       2.3250011793104764e-05, 
       1.9013248481614200e-05, 
       2.3074855563075498e-05, 
       1.9304039766842712e-05, 
       1.8501413240458147e-05, 
       1.6419466302874748e-05, 
       1.9306624858929919e-05, 
       1.9302420962345517e-05, 
       2.0099874790470865e-05, 
       2.0099119625085626e-05, 
       1.7181973285697232e-05, 
       1.7249835267180766e-05, 
       1.7788070523349194e-05, 
       2.2420219424819730e-05, 
       1.6675585728044082e-05, 
       1.7252988806554564e-05, 
       1.6780507453344162e-05, 
       1.7752341584529981e-05, 
       1.8829169324520366e-05, 
       1.7541492588550479e-05, 
       1.7261342986004134e-05, 
       1.6473552121557051e-05, 
       1.6726797339031451e-05, 
       1.9099278465665695e-05, 
       1.6508317787735880e-05, 
       1.6345299668428689e-05, 
       2.0636986590934491e-05, 
       2.2316664324812403e-05, 
       1.9361491496825064e-05, 
       1.9492763683807662e-05, 
       1.8661835328000411e-05, 
       2.2270138151235147e-05, 
       1.6261959569716738e-05, 
       2.2680281059748456e-05, 
       1.6409527881725882e-05, 
       1.7187706513412739e-05, 
       1.7747808708896180e-05, 
       2.1893576160653762e-05, 
       1.6368441116126211e-05, 
       2.1886823911959488e-05, 
       1.8950025164632148e-05, 
       1.7894628316429601e-05, 
       2.0268576500937218e-05, 
       2.0106056539561648e-05, 
       1.8908707048867289e-05, 
       1.6303884265495167e-05, 
       1.8307849129973548e-05, 
       2.2652702311329514e-05, 
       2.2563947078767160e-05, 
       2.0790476275472836e-05, 
       2.2631806185743912e-05, 
       2.2616595770205090e-05, 
       1.9056683565391087e-05, 
       1.9039176789753157e-05, 
       1.6063784402242326e-05, 
       1.8464144511990500e-05, 
       1.6980010174667729e-05, 
       1.7273639214441055e-05, 
       1.8316207380717642e-05, 
       1.8794390188883509e-05, 
       1.8793118845375037e-05, 
       1.9399915953460206e-05, 
       1.8191255489112355e-05, 
       1.7243911997532744e-05, 
       2.0510714603222904e-05, 
       1.8288067043859070e-05, 
       1.6843815475317242e-05, 
       1.6782111658767569e-05, 
       2.0467052476129889e-05, 
       2.2307021864782610e-05, 
       1.6361777316992570e-05, 
       1.8584062651128511e-05, 
       1.7327024723531550e-05, 
       1.6733489485492690e-05, 
       1.7047740426222983e-05, 
       2.0993967840435955e-05, 
       1.6500152445011802e-05, 
       1.8001140136188653e-05, 
       1.6842787612327909e-05, 
       2.2285413808576788e-05, 
       1.7304528192344530e-05, 
       1.9440809173745262e-05, 
       1.9408652860286046e-05, 
       2.2308965890963654e-05, 
       1.7059894269898029e-05, 
       1.7299463701415883e-05, 
       1.8215888997602690e-05, 
       1.9412749608311289e-05, 
       1.5987760378662708e-05, 
       1.8817898978253875e-05, 
       1.8817898978253875e-05, 
       1.7973693338197817e-05, 
       1.7959060371545117e-05, 
       2.2178007724241785e-05, 
       1.6485000701342114e-05, 
       2.1883837894146590e-05, 
       1.6370014778855305e-05, 
       1.7461419958754493e-05, 
       1.9009122775459467e-05, 
       2.1415671267411649e-05, 
       1.6251923361205683e-05, 
       1.7677216822713677e-05, 
       2.2119426582307980e-05, 
       1.7139566016459397e-05, 
       2.2127131281547109e-05, 
       1.6484733306001025e-05, 
       1.6009518260357110e-05, 
       2.2088606066161964e-05, 
       1.6843372422457200e-05, 
       1.9703838718187286e-05, 
       1.5729701112158979e-05, 
       1.5605336586737418e-05, 
       2.0334043829559569e-05, 
       1.8093457164273500e-05, 
       1.7018370486444907e-05, 
       1.8092780768013057e-05, 
       2.1935861477145441e-05, 
       1.7821825649641092e-05, 
       1.7820806697276662e-05, 
       2.0679450147518896e-05, 
       2.1807114602056485e-05, 
       1.5514078046660234e-05, 
       2.1448112769470992e-05, 
       1.9348952945848586e-05, 
       1.7001177995273558e-05, 
       2.1043924230317544e-05, 
       1.6789717514511149e-05, 
       1.8155898825021778e-05, 
       1.7859594553655210e-05, 
       1.6087138253532083e-05, 
       1.7400873911920310e-05, 
       1.5502385879772839e-05, 
       1.8138713559224012e-05, 
       1.8020200214705512e-05, 
       2.0682001770383132e-05, 
       2.0116860434379703e-05, 
       1.7027776481093546e-05, 
       1.8108603465259826e-05, 
       2.1776543849616935e-05, 
       2.1142785652008354e-05, 
       1.8095736251542919e-05, 
       1.7471878746700059e-05, 
       1.6209419085385878e-05, 
       1.5776045047487888e-05, 
       1.8614429820787796e-05, 
       2.1466794788251957e-05, 
       1.5824364094885964e-05, 
       1.9999999999995160e-05, 
       2.0000000000004284e-05, 
       1.9999999999995407e-05, 
       1.9999999999998504e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       1.9999999999999853e-05, 
       2.0000000000004603e-05, 
       1.9999999999996295e-05, 
       1.6175978944920478e-05, 
       1.6719774526569959e-05, 
       1.7686564612508700e-05, 
       2.0813687420850283e-05, 
       1.5660071132120860e-05, 
       1.7989800407954048e-05, 
       1.6131548965074918e-05, 
       2.1616983409057042e-05, 
       1.8598768300361032e-05, 
       1.9607223872270877e-05, 
       1.6963892560874945e-05, 
       2.1366772388284552e-05, 
       1.7359940005171894e-05, 
       1.8507199444334618e-05, 
       2.0750008388237536e-05, 
       1.6868254967829275e-05, 
       1.6423042036472158e-05, 
       1.6423042036472158e-05, 
       1.6864259144271627e-05, 
       1.5689642570227779e-05, 
       1.9050947387774732e-05, 
       1.7611896965069240e-05, 
       1.7745247024068698e-05, 
       1.8538580551107680e-05, 
       1.8379427706102670e-05, 
       1.5945888870380719e-05, 
       1.7305791447725026e-05, 
       2.0474582331640290e-05, 
       1.7492727383964233e-05, 
       2.1476177380312355e-05, 
       1.9106095035613618e-05, 
       1.5660259438659259e-05, 
       1.5222668335700431e-05, 
       1.8219155343956824e-05, 
       1.5642650945830570e-05, 
       1.6793143148081557e-05, 
       1.7141524658524282e-05, 
       1.8846714100393478e-05, 
       1.9326279863760890e-05, 
       2.0406891479708431e-05, 
       1.7688341537316285e-05, 
       1.5724445646149188e-05, 
       1.5540909059375424e-05, 
       1.8441641969427492e-05, 
       2.1178813931240924e-05, 
       1.5511727108972537e-05, 
       2.1342028735132333e-05, 
       1.6739066052948356e-05, 
       2.1350732414601160e-05, 
       1.7694089944982486e-05, 
       1.5829334698050776e-05, 
       1.5655025671618804e-05, 
       1.9714318587157159e-05, 
       1.5057035573659841e-05, 
       2.0192111541620821e-05, 
       1.7685943389918353e-05, 
       2.1212723019460474e-05, 
       1.9747980812227106e-05, 
       1.5527905123794615e-05, 
       1.5527905123794615e-05, 
       1.5417830060424720e-05, 
       1.7247065269713207e-05, 
       2.1119942171578811e-05, 
       1.8175254317032841e-05, 
       1.7324416311977315e-05, 
       1.9999999999999429e-05, 
       2.1137512979669648e-05, 
       2.1090103082829514e-05, 
       2.0886570032442421e-05, 
       2.1105596617233711e-05, 
       1.5743035286946835e-05, 
       2.1083679352776058e-05, 
       2.0757712453211110e-05, 
       1.5540797566770339e-05, 
       1.7531973970627391e-05, 
       1.9259698071998040e-05, 
       2.0208818797206283e-05, 
       1.5149256871057733e-05, 
       1.5467115962108733e-05, 
       1.5760440219226027e-05, 
       1.7162560484133913e-05, 
       1.7014309235359223e-05, 
       2.1019737107659137e-05, 
       1.7457605929503559e-05, 
       2.0994409641551910e-05, 
       1.9458440920805566e-05, 
       1.5603829640010367e-05, 
       1.9952679961171134e-05, 
       1.8335526455375638e-05, 
       1.6608792629156972e-05, 
       1.5626835986606559e-05, 
       2.0757576383008465e-05, 
       1.6442832343553638e-05, 
       1.5191233475722848e-05, 
       1.5043197353550041e-05, 
       2.0442887337734135e-05, 
       1.6628664127615279e-05, 
       1.8371707177007882e-05, 
       1.5913842135018836e-05, 
       1.7254322155783046e-05, 
       2.0839446184619425e-05, 
       1.5216164047457902e-05, 
       1.5029809033679622e-05, 
       1.5744511007656851e-05, 
       1.7024285329190686e-05, 
       1.4829247045808572e-05, 
       1.7309310070084839e-05, 
       2.0437293880977209e-05, 
       2.0021885185557937e-05, 
       1.8019111551194156e-05, 
       1.4831344181665965e-05, 
       2.0310977894737043e-05, 
       1.5009995172045378e-05, 
       1.7169158565946365e-05, 
       1.6165500240324501e-05, 
       1.9447983568464269e-05, 
       1.5090612779274327e-05, 
       1.4918050969900448e-05, 
       1.6476833830296272e-05, 
       2.0191724627259120e-05, 
       1.5421214338198483e-05, 
       1.7144943104701221e-05, 
       1.8098227541769401e-05, 
       1.5368499686398445e-05, 
       1.7177387307292020e-05, 
       2.0028358502386586e-05, 
       1.4858069728598857e-05, 
       1.4557718472553275e-05, 
       2.0391569074065477e-05, 
       1.8121570412691590e-05, 
       1.7132585552788231e-05, 
       1.5566873195342529e-05, 
       1.4556633968720358e-05, 
       1.5665010478850375e-05, 
       1.7601692675380615e-05, 
       2.0369753881770931e-05, 
       1.5001512518226317e-05, 
       1.9764482926694215e-05, 
       1.6400977286120477e-05, 
       1.5664816591402002e-05, 
       1.8243093368615829e-05, 
       1.6613644707846301e-05, 
       2.0299623746178761e-05, 
       2.0448445057820992e-05, 
       2.0263783081914792e-05, 
       2.0126702648095053e-05, 
       1.7004791999853136e-05, 
       1.6858620981037930e-05, 
       1.7768595734014653e-05, 
       1.9703524533953504e-05, 
       1.6920527940262901e-05, 
       1.5476650158324031e-05, 
       1.6299601104799197e-05, 
       1.6579802259591897e-05, 
       1.5042009613690328e-05, 
       1.5323701383613402e-05, 
       1.9676797461439080e-05, 
       1.6966041167571207e-05, 
       1.4719380667612877e-05, 
       1.7172069384776256e-05, 
       1.8398699817429276e-05, 
       1.4386265143752439e-05, 
       2.0261595272386628e-05, 
       1.4550935010314665e-05, 
       2.0266470679314170e-05, 
       1.5717671821020621e-05, 
       1.6852600658635932e-05, 
       1.6558022622144565e-05, 
       1.8019584552226510e-05, 
       1.6832558809015334e-05, 
       1.6861710188960205e-05, 
       1.8893615505926731e-05, 
       1.9557683747342080e-05, 
       1.4541700275218320e-05, 
       2.0238268450987529e-05, 
       1.6661293302781685e-05, 
       1.8526044807807572e-05, 
       1.4564188078851668e-05, 
       1.4677653873009543e-05, 
       1.7056497365353723e-05, 
       1.9539926389561244e-05, 
       1.7085173203672934e-05, 
       1.6209120421822010e-05, 
       1.5300826859808195e-05, 
       1.4567346867009612e-05, 
       1.6697222707461084e-05, 
       2.0086481563064009e-05, 
       1.4972744923786265e-05, 
       1.9957044818525361e-05, 
       1.6005810102042682e-05, 
       1.9283733192707659e-05, 
       1.4966509948399841e-05, 
       1.4415727513073371e-05, 
       1.4808249418141361e-05, 
       1.6715996725192163e-05, 
       1.6666076245336490e-05, 
       2.0003825464763571e-05, 
       1.7719507721485027e-05, 
       1.9369663830905708e-05, 
       1.7613112162347754e-05, 
       1.4759238116369718e-05, 
       1.4759238116369718e-05, 
       2.0011800829874513e-05, 
       1.4164793410740855e-05, 
       1.5946658441031332e-05, 
       1.9999999999996695e-05, 
       2.0000000000003224e-05, 
       1.9999999999999121e-05, 
       2.0000000000004206e-05, 
       1.9999999999999659e-05, 
       2.0000000000002682e-05, 
       1.9999999999999239e-05, 
       1.9999999999996173e-05, 
       1.9999999999996146e-05, 
       1.9999999999997356e-05, 
       2.0000000000001601e-05, 
       2.0000000000001601e-05, 
       2.0000000000001601e-05, 
       2.0000000000002875e-05, 
       1.9999999999998955e-05, 
       1.9999999999998955e-05, 
       2.0000000000002953e-05, 
       1.9999999999998511e-05, 
       1.9999999999999551e-05, 
       2.0000000000001116e-05, 
       1.9999999999996078e-05, 
       1.9999999999999470e-05, 
       2.0000000000003474e-05, 
       2.0000000000003474e-05, 
       2.0000000000003474e-05, 
       1.9999999999998087e-05, 
       1.9999999999998087e-05, 
       2.0000000000003305e-05, 
       1.9999999999999138e-05, 
       1.9999999999999138e-05, 
       1.9999999999995577e-05, 
       2.0000000000002624e-05, 
       2.0000000000000283e-05, 
       2.0000000000000283e-05, 
       1.9999999999997427e-05, 
       1.9999999999996915e-05, 
       1.9999999999997010e-05, 
       1.9999999999997715e-05, 
       2.0000000000000947e-05, 
       2.0000000000003481e-05, 
       2.0000000000001838e-05, 
       2.0000000000000002e-05, 
       1.4252697008525248e-05, 
       1.4706617789152532e-05, 
       1.4222562777881572e-05, 
       1.6361346949309436e-05, 
       1.4879257822647029e-05, 
       1.6062458346031203e-05, 
       1.6975532626770155e-05, 
       1.6384722202982934e-05, 
       1.6431911229925512e-05, 
       1.4628386208393741e-05, 
       1.6299854659894770e-05, 
       1.8359843299780902e-05, 
       1.6064531702894845e-05, 
       1.7101967390113963e-05, 
       1.9327278959677277e-05, 
       1.9893912026707649e-05, 
       1.9435801441053278e-05, 
       1.9295442619442662e-05, 
       1.4198402879160075e-05, 
       1.5601986750816583e-05, 
       1.5601986750816583e-05, 
       1.5442611079544925e-05, 
       1.8273101120279640e-05, 
       1.5212715709730862e-05, 
       1.9790572520707653e-05, 
       1.4046644829345020e-05, 
       1.6453534288840922e-05, 
       1.9290476812844101e-05, 
       1.4378875451023086e-05, 
       1.5094122994640211e-05, 
       1.5094122994640211e-05, 
       1.4142842633091691e-05, 
       1.6470574208214187e-05, 
       1.8950886223863053e-05, 
       1.5757625747503491e-05, 
       1.9712309404743041e-05, 
       1.8205817440024950e-05, 
       1.8205817440024950e-05, 
       1.4528308264275255e-05, 
       1.9573364967590550e-05, 
       1.4478764179994934e-05, 
       1.6375644730731441e-05, 
       1.7263992904413860e-05, 
       1.3974287163374690e-05, 
       1.7069877753966555e-05, 
       1.3979984726077016e-05, 
       1.6294138445065738e-05, 
       1.4988600322324814e-05, 
       1.8087520439615098e-05, 
       1.4428605082598419e-05, 
       1.5397726069238819e-05, 
       1.5395294214570601e-05, 
       1.5112464888943527e-05, 
       1.4514880645921092e-05, 
       1.9631504807082202e-05, 
       1.4655089846006943e-05, 
       1.3932174364934544e-05, 
       1.6680758473363207e-05, 
       1.8482397616888962e-05, 
       1.9375638950615900e-05, 
       1.6274656624678140e-05, 
       1.8377590927511385e-05, 
       1.6270479688158208e-05, 
       1.9508249566450005e-05, 
       1.9157204813098891e-05, 
       1.6335965397168821e-05, 
       1.6881997905733259e-05, 
       1.6564319840218979e-05, 
       1.8872918146885511e-05, 
       1.8074549305929563e-05, 
       1.9412951588592442e-05, 
       1.6263307324113935e-05, 
       1.4147500694586512e-05, 
       1.6757367952507248e-05, 
       1.9322351527415885e-05, 
       1.4573751251572777e-05, 
       1.4154859016617978e-05, 
       1.4340334740790183e-05, 
       1.4340334740790183e-05, 
       1.4071960371740698e-05, 
       1.6121666465218100e-05, 
       1.8437279532170882e-05, 
       1.4256593632430758e-05, 
       1.4556420819794034e-05, 
       1.4142244855213609e-05, 
       1.4417170150842718e-05, 
       1.3988376919673882e-05, 
       1.3783936244070711e-05, 
       1.5545702181325805e-05, 
       1.4487086535407894e-05, 
       1.9064889146728757e-05, 
       1.9006325421144988e-05, 
       1.9056748871438939e-05, 
       1.8773399897788679e-05, 
       1.4335056900673789e-05, 
       1.4335056900673789e-05, 
       1.8522465125382274e-05, 
       1.6386653896313993e-05, 
       1.5596762415637884e-05, 
       1.6005668289636209e-05, 
       1.3766017742529295e-05, 
       1.3848653128938670e-05, 
       1.5576755427267637e-05, 
       1.9100408442469076e-05, 
       1.3593457337984071e-05, 
       1.3874206683477119e-05, 
       1.4465040574081605e-05, 
       1.4652034746258144e-05, 
       1.8956486987217587e-05, 
       1.4614510097070012e-05, 
       1.4614510097070012e-05, 
       1.8008430901014635e-05, 
       1.8541903458107715e-05, 
       1.5650865089307326e-05, 
       1.8035119275958362e-05, 
       1.3761799690021450e-05, 
       1.3912423234045471e-05, 
       1.5804152496313920e-05, 
       1.4498211958542439e-05, 
       1.4220681470575931e-05, 
       1.8527474953553169e-05, 
       1.6610320576735356e-05, 
       1.4236935008604960e-05, 
       1.3561594886345028e-05, 
       1.3703215974651375e-05, 
       1.5192322155459611e-05, 
       1.4690494971901992e-05, 
       1.7429942889447511e-05, 
       1.3803523486348431e-05, 
       1.6575179186165403e-05, 
       1.8483263583129016e-05, 
       1.4317789249962279e-05, 
       1.7732674788684577e-05, 
       1.5651627359566928e-05, 
       1.7290318557184173e-05, 
       1.8996710211881576e-05, 
       1.5790212465426335e-05, 
       1.8966699858754369e-05, 
       1.6029256120213828e-05, 
       1.8679539859490891e-05, 
       1.8679539859490891e-05, 
       1.3830009977865649e-05, 
       1.6463233245324904e-05, 
       1.5799996450446853e-05, 
       1.7882317958138889e-05, 
       1.3585888483559118e-05, 
       1.3894097840287981e-05, 
       1.3787686670116759e-05, 
       1.3644422352472035e-05, 
       1.8774035932232420e-05, 
       1.7563953822318123e-05, 
       1.5856345491746732e-05, 
       1.6139604297705414e-05, 
       1.4074765159929460e-05, 
       1.4484758356553500e-05, 
       1.3670166002683302e-05, 
       1.8775891554910203e-05, 
       1.6802587081324032e-05, 
       1.5910421546831657e-05, 
       1.3698167598078695e-05, 
       1.3653131818738679e-05, 
       1.4888735311831039e-05, 
       1.7799888897948100e-05, 
       1.3624957205328564e-05, 
       1.6329939260001416e-05, 
       1.8762516187751245e-05, 
       1.7014405338271208e-05, 
       1.5581039856646170e-05, 
       1.8697234576615613e-05, 
       1.3972208543942194e-05, 
       1.3972208543942194e-05, 
       1.4442937029131723e-05, 
       1.4399782406159191e-05, 
       1.3736672508724698e-05, 
       1.4140165512352361e-05, 
       1.5231770743292131e-05, 
       1.4853962474344839e-05, 
       1.7618334074186727e-05, 
       1.5433677464118755e-05, 
       1.3305211232350297e-05, 
       1.4964053258880190e-05, 
       1.3921310448009076e-05, 
       1.5374689745796454e-05, 
       1.3641410384706774e-05, 
       1.8055064613690848e-05, 
       1.3360955606502185e-05, 
       1.3856726747044094e-05, 
       1.3627425378109574e-05, 
       1.4890056335115028e-05, 
       1.8526520276109926e-05, 
       1.5801608056782483e-05, 
       1.4876131597626108e-05, 
       1.4138365761711702e-05, 
       1.4825313646580885e-05, 
       1.3602238135309408e-05, 
       1.3796179854504099e-05, 
       1.7297656639459478e-05, 
       1.3147675452199926e-05, 
       1.8204562308233258e-05, 
       1.7815850850000629e-05, 
       1.5278119496228523e-05, 
       1.3240215118733179e-05, 
       1.5699687872250967e-05, 
       1.3374323164538733e-05, 
       1.7112296806910231e-05, 
       1.6498260600057044e-05, 
       1.8277692739113280e-05, 
       1.4079393768881114e-05, 
       1.3926600400371500e-05, 
       1.5273539828843144e-05, 
       1.5265176841358936e-05, 
       1.3322314722811133e-05, 
       1.8232455127632119e-05, 
       1.5244495762962463e-05, 
       1.3032933685357870e-05, 
       1.2974384091595749e-05, 
       1.5379789430354704e-05, 
       1.5379789430354704e-05, 
       1.8331736566769736e-05, 
       1.3037120214926734e-05, 
       1.5759621600579822e-05, 
       1.7574461976645667e-05, 
       1.8089452974609749e-05, 
       1.3606642222909142e-05, 
       1.5163483142953135e-05, 
       1.4994433687067182e-05, 
       1.5200368248783655e-05, 
       1.3310792113503426e-05, 
       1.4671350937450219e-05, 
       1.3684550993637766e-05, 
       1.5494308882524729e-05, 
       1.7638645171835623e-05, 
       1.2938155469159480e-05, 
       1.3744066984965754e-05, 
       1.7816407086215970e-05, 
       1.8150773800612311e-05, 
       1.4726544941961299e-05, 
       1.7986737031725168e-05, 
       1.8052485607192366e-05, 
       1.4812099570027929e-05, 
       1.4518517656199323e-05, 
       1.3990896235310456e-05, 
       1.5543947283905356e-05, 
       1.7577282839669293e-05, 
       1.3507379863828441e-05, 
       1.7488924566693299e-05, 
       1.5071685391860221e-05, 
       1.2791921429880872e-05, 
       1.8023910057525558e-05, 
       1.3605811071184698e-05, 
       1.8022134748298126e-05, 
       1.4394770804460141e-05, 
       1.3728539436706959e-05, 
       1.3733553493619727e-05, 
       1.5396018344123773e-05, 
       1.6117579412375021e-05, 
       1.2763076368191250e-05, 
       1.3451049438603334e-05, 
       1.3485220199497009e-05, 
       1.5904892647605164e-05, 
       1.7864247572255879e-05, 
       1.4667043332272065e-05, 
       1.7971366349273862e-05, 
       1.4661552630549338e-05, 
       1.7908044971456631e-05, 
       1.4928801764298160e-05, 
       1.4910251689453436e-05, 
       1.7204350483958699e-05, 
       1.7657159089503033e-05, 
       1.5074256166742058e-05, 
       1.3282048646588323e-05, 
       1.7271865789996836e-05, 
       1.5912057737424142e-05, 
       1.7054593378276863e-05, 
       1.5969037353564106e-05, 
       1.5524953603403964e-05, 
       1.7650763770924837e-05, 
       1.3944850141770251e-05, 
       1.5190932435174982e-05, 
       1.3789607788259273e-05, 
       1.2701786898881363e-05, 
       1.7076570796036098e-05, 
       1.7616211913199017e-05, 
       1.4071295575194016e-05, 
       1.3725672479509337e-05, 
       1.5320600940140889e-05, 
       1.4290138759118962e-05, 
       1.3647439612993676e-05, 
       1.6509359377876491e-05, 
       1.4520756131834291e-05, 
       1.3030639943745086e-05, 
       1.5088065308295757e-05, 
       1.2850530350206869e-05, 
       1.7649665142622546e-05, 
       1.3153542124574486e-05, 
       1.7711449414143951e-05, 
       1.3601666905827086e-05, 
       1.3435283052920611e-05, 
       1.7156003099018238e-05, 
       1.4975499849479458e-05, 
       1.7637939469122597e-05, 
       1.7558379694124883e-05, 
       1.4668551910176133e-05, 
       1.7084070941358651e-05, 
       1.3682645263022775e-05, 
       1.5981906534706543e-05, 
       1.7637817035649532e-05, 
       1.2890273588343544e-05, 
       1.5245509927831403e-05, 
       1.6758967052960054e-05, 
       1.7233754233301464e-05, 
       1.7520367723575716e-05, 
       1.3192309639059891e-05, 
       1.4419266314513079e-05, 
       1.4638127644720810e-05, 
       1.3909898954175807e-05, 
       1.4695654784553427e-05, 
       1.3048290781505763e-05, 
       1.3048290781505763e-05, 
       1.4557878607341690e-05, 
       1.4660430329794040e-05, 
       1.4597096659250177e-05, 
       1.3423358781637431e-05, 
       1.6976026283234938e-05, 
       1.6294144632581943e-05, 
       1.3845272419402378e-05, 
       1.4201681647182699e-05, 
       1.2706578085768076e-05, 
       1.4496581518191856e-05, 
       1.3303962337713564e-05, 
       1.3052231908621290e-05, 
       1.3379125996042708e-05, 
       1.2836175305829798e-05, 
       1.4470153842595127e-05, 
       1.4476736529632748e-05, 
       1.2938046521844049e-05, 
       1.2938046521844049e-05, 
       1.3627442406962703e-05, 
       1.3929888230266741e-05, 
       1.4754452407133853e-05, 
       1.3645484846918677e-05, 
       1.3483611504244313e-05, 
       1.5459574601845453e-05, 
       1.3333275993307890e-05, 
       1.7288009190477404e-05, 
       1.2477230783178808e-05, 
       1.6125270270647051e-05, 
       1.6098750340081555e-05, 
       1.2372326077540164e-05, 
       1.6251502136286907e-05, 
       1.2914876775410167e-05, 
       1.4886017190838218e-05, 
       1.7266381037946462e-05, 
       1.5056480330779489e-05, 
       1.3909423114964528e-05, 
       1.2555830630170498e-05, 
       1.2245937489779250e-05, 
       1.3861970848648388e-05, 
       1.4343507935153652e-05, 
       1.7242847974451225e-05, 
       1.3103856485897079e-05, 
       1.3103856485897079e-05, 
       1.2397289904446542e-05, 
       1.3446720656278218e-05, 
       1.4311110555410218e-05, 
       1.5717980401117605e-05, 
       1.4863154551277376e-05, 
       1.4328863233185172e-05, 
       1.6998547074026785e-05, 
       1.7175291575323660e-05, 
       1.6471152625733929e-05, 
       1.3114513918939321e-05, 
       1.2860330495408732e-05, 
       1.4683817738810251e-05, 
       1.2167919130941174e-05, 
       1.7069186952466700e-05, 
       1.2837893826301510e-05, 
       1.2837291811602049e-05, 
       1.3258128466507157e-05, 
       1.7166171907753879e-05, 
       1.3541079299135263e-05, 
       1.3890893255316942e-05, 
       1.2253429105000318e-05, 
       1.6814379251639918e-05, 
       1.6686371790053983e-05, 
       1.2308744618674266e-05, 
       1.4636327977612473e-05, 
       1.3066470074275391e-05, 
       1.4231459697256301e-05, 
       1.2598263937027711e-05, 
       1.2598263937027711e-05, 
       1.6029260899523525e-05, 
       1.2717143887558507e-05, 
       1.2526394277540726e-05, 
       1.3337117264024642e-05, 
       1.3337117264024642e-05, 
       1.2066906506521277e-05, 
       1.4622861848075122e-05, 
       1.3245897771755157e-05, 
       1.4113747366887348e-05, 
       1.5776030685851246e-05, 
       1.6840589361518967e-05, 
       1.6757062602835289e-05, 
       1.2312482768388078e-05, 
       1.2945937951306349e-05, 
       1.2312734900739979e-05, 
       1.4364209292227234e-05, 
       1.3390556900591869e-05, 
       1.6850786734237513e-05, 
       1.2417472982824590e-05, 
       1.2050728736363786e-05, 
       1.2121054877793193e-05, 
       1.2021842078897896e-05, 
       1.6914859326904161e-05, 
       1.6914859326904161e-05, 
       1.6049674572441781e-05, 
       1.5596869909457542e-05, 
       1.5786976168410679e-05, 
       1.4058808896797033e-05, 
       1.4303475321960484e-05, 
       1.2406664726228691e-05, 
       1.2587942888697610e-05, 
       1.6731244239970316e-05, 
       1.4144804399699325e-05, 
       1.6666138483921900e-05, 
       1.4550935010312244e-05, 
       1.4461870876485519e-05, 
       1.3994226033164611e-05, 
       1.5195555397875725e-05, 
       1.6729713132836927e-05, 
       1.4091489602312114e-05, 
       1.2019136694960872e-05, 
       1.1900254687958665e-05, 
       1.2427743607487307e-05, 
       1.4146131462608746e-05, 
       1.4181027470503725e-05, 
       1.3594604895580452e-05, 
       1.2177891934311316e-05, 
       1.2025784821732307e-05, 
       1.1917756983521074e-05, 
       1.6254652074131396e-05, 
       1.1895702147260868e-05, 
       1.6096719431448920e-05, 
       1.5917873693892506e-05, 
       1.5917873693892506e-05, 
       1.6122999578866428e-05, 
       1.6550723620444485e-05, 
       1.3053404989046549e-05, 
       1.1899219598038330e-05, 
       1.6556372876046441e-05, 
       1.3959526236808225e-05, 
       1.2144994647592413e-05, 
       1.3386073177119976e-05, 
       1.5134449596170500e-05, 
       1.6696225586389643e-05, 
       1.2031171987761573e-05, 
       1.6293459771790049e-05, 
       1.6293459771790049e-05, 
       1.6547803747264835e-05, 
       1.6697400297342063e-05, 
       1.6216049437156085e-05, 
       1.3933037853893806e-05, 
       1.2257163925048118e-05, 
       1.4010491493895888e-05, 
       1.6252977248923219e-05, 
       1.2101801105589593e-05, 
       1.1892920941077241e-05, 
       1.3235715098705824e-05, 
       1.6648172967064449e-05, 
       1.4892291719144730e-05, 
       1.2604359039019861e-05, 
       1.5421558404408979e-05, 
       1.6590948772789458e-05, 
       1.3220818327265675e-05, 
       1.4563479832099421e-05, 
       1.3748895926703433e-05, 
       1.2606073278651340e-05, 
       1.3777882492901675e-05, 
       1.1869521198062512e-05, 
       1.2416428439146307e-05, 
       1.1768856122296658e-05, 
       1.1858819054917719e-05, 
       1.2397461607421407e-05, 
       1.1893677184827856e-05, 
       1.5888709529634991e-05, 
       1.1690375557812716e-05, 
       1.6517676686398936e-05, 
       1.1700429175629419e-05, 
       1.3674402824641351e-05, 
       1.3926320221697239e-05, 
       1.2176653171116317e-05, 
       1.3687430993784603e-05, 
       1.3741803269687414e-05, 
       1.5922462564260811e-05, 
       1.2801374700097838e-05, 
       1.3130363667340645e-05, 
       1.2922579292438108e-05, 
       1.4105335722376046e-05, 
       1.4376105332733210e-05, 
       1.6407326862993805e-05, 
       1.5873453532599455e-05, 
       1.3703222617552004e-05, 
       1.2117885951517277e-05, 
       1.5614580438514598e-05, 
       1.1975720602925757e-05, 
       1.2708501264251740e-05, 
       1.2357430174878866e-05, 
       1.6399966829113250e-05, 
       1.3183846274234094e-05, 
       1.5338415995535773e-05, 
       1.4563188418429497e-05, 
       1.3605880830959589e-05, 
       1.4328212241143718e-05, 
       1.6363733125681598e-05, 
       1.4164855642925120e-05, 
       1.4164855642925120e-05, 
       1.2323655363808756e-05, 
       1.2210970936641173e-05, 
       1.3640407348496314e-05, 
       1.1814901904537399e-05, 
       1.3139510997259285e-05, 
       1.6322576093387930e-05, 
       1.3980065987600939e-05, 
       1.3475308442031163e-05, 
       1.5921832000744340e-05, 
       1.1826485091935282e-05, 
       1.2120901952412654e-05, 
       1.2114943572371832e-05, 
       1.3597009682566435e-05, 
       1.4581189231655004e-05, 
       1.3501358373618001e-05, 
       1.6065981678586259e-05, 
       1.2276480460318524e-05, 
       1.2003836137298747e-05, 
       1.1525322578344814e-05, 
       1.3517460118223849e-05, 
       1.3970690321599620e-05, 
       1.6161601619169139e-05, 
       1.1931658769035319e-05, 
       1.3964616244322158e-05, 
       1.3533717127469221e-05, 
       1.1904444298536165e-05, 
       1.2577743538942000e-05, 
       1.1508460020682763e-05, 
       1.1730974848875819e-05, 
       1.2477745528219772e-05, 
       1.6049351736991511e-05, 
       1.1464313500445488e-05, 
       1.3475378673897131e-05, 
       1.5927747760108988e-05, 
       1.3457534238306322e-05, 
       1.1935326835509630e-05, 
       1.2220331175923250e-05, 
       1.4317074492694330e-05, 
       1.2205539674595510e-05, 
       1.3072336703719005e-05, 
       1.2655866562070534e-05, 
       1.2655866562070534e-05, 
       1.3065085265743608e-05, 
       1.6112900946421414e-05, 
       1.1416702843928877e-05, 
       1.5672390858687492e-05, 
       1.2116629510574814e-05, 
       1.2032232871584167e-05, 
       1.5320346282703337e-05, 
       1.4481654901481538e-05, 
       1.3885559510227951e-05, 
       1.2044972708168932e-05, 
       1.5002928543636147e-05, 
       1.5082211806793467e-05, 
       1.2756413853865409e-05, 
       1.4497843397617043e-05, 
       1.3392113626936127e-05, 
       1.1389365726732977e-05, 
       1.1985420076909093e-05, 
       1.5196842325061426e-05, 
       1.1933621019020256e-05, 
       1.1807638449827242e-05, 
       1.3308936305146922e-05, 
       1.3298599083369901e-05, 
       1.1378444149920218e-05, 
       1.6048462299887861e-05, 
       1.5991135950797735e-05, 
       1.3285633812314531e-05, 
       1.1861522576797246e-05, 
       1.4224421894410120e-05, 
       1.1986113227578546e-05, 
       1.4216639432982047e-05, 
       1.3470047970805898e-05, 
       1.3470047970805898e-05, 
       1.4741088562136247e-05, 
       1.3059321000458832e-05, 
       1.3269348820725535e-05, 
       1.4479061291151282e-05, 
       1.5511542301208200e-05, 
       1.1813723822093969e-05, 
       1.5280970012462507e-05, 
       1.3054280075616977e-05, 
       1.5762080187927474e-05, 
       1.2918826683218768e-05, 
       1.2092610841784454e-05, 
       1.2354112418920712e-05, 
       1.3225241829851445e-05, 
       1.5421509424894833e-05, 
       1.5773410814205763e-05, 
       1.1642924087402093e-05, 
       1.3248940560185062e-05, 
       1.3284484931482563e-05, 
       1.3064209267144802e-05, 
       1.1879999601992830e-05, 
       1.1693198035836968e-05, 
       1.3094240945144193e-05, 
       1.5849108661994252e-05, 
       1.5802276527407762e-05, 
       1.4528569276352678e-05, 
       1.3887200926675147e-05, 
       1.5628238006873463e-05, 
       1.1391895340035019e-05, 
       1.4124703796517866e-05, 
       1.1807043137415146e-05, 
       1.1856920864233906e-05, 
       1.3257437021867844e-05, 
       1.5764864603397483e-05, 
       1.3112152733703945e-05, 
       1.3433709764932103e-05, 
       1.3493138783878672e-05, 
       1.5640243705340389e-05, 
       1.4897516853048267e-05, 
       1.1824671081427098e-05, 
       1.1918979748613804e-05, 
       1.5306807208564794e-05, 
       1.5329215830388740e-05, 
       1.4575506214112725e-05, 
       1.3938339965521361e-05, 
       1.3827512359644423e-05, 
       1.3826506665593992e-05, 
       1.1154287022865394e-05, 
       1.3104211740154260e-05, 
       1.3057059875474051e-05, 
       1.4557953177370790e-05, 
       1.1159088919781853e-05, 
       1.1801813846695364e-05, 
       1.3861689708102152e-05, 
       1.5637547646829787e-05, 
       1.1356472191127747e-05, 
       1.2317807113902109e-05, 
       1.4895818288670863e-05, 
       1.3850120802112967e-05, 
       1.3492607685528236e-05, 
       1.4981062960212821e-05, 
       1.3252357555360155e-05, 
       1.1104728466622570e-05, 
       1.4483242358871658e-05, 
       1.1578097276879643e-05, 
       1.5420703115194019e-05, 
       1.5033360827189572e-05, 
       1.5186932725687286e-05, 
       1.2013952315036721e-05, 
       1.4990221537768369e-05, 
       1.5556573953943968e-05, 
       1.5563407443876743e-05, 
       1.2971239202206968e-05, 
       1.1890022150710963e-05, 
       1.5537853150611444e-05, 
       1.5539998142774340e-05, 
       1.3856883481054222e-05, 
       1.2530939020271129e-05, 
       1.1637049120821975e-05, 
       1.2945648311349691e-05, 
       1.1031681376658956e-05, 
       1.3049309327874108e-05, 
       1.1940715012502777e-05, 
       1.1897661138472753e-05, 
       1.1335968607249550e-05, 
       1.3555327357810430e-05, 
       1.1779470985341316e-05, 
       1.3187383370416893e-05, 
       1.4539576556180759e-05, 
       1.2813827019497473e-05, 
       1.1444304102659144e-05, 
       1.4969920882945149e-05, 
       1.5517416798333600e-05, 
       1.1410798355908373e-05, 
       1.1526908858379218e-05, 
       1.1412154447914233e-05, 
       1.1060721686510390e-05, 
       1.1994395300910673e-05, 
       1.1455233575224207e-05, 
       1.4773815207526001e-05, 
       1.3861004405617579e-05, 
       1.3640295916394148e-05, 
       1.2852315404897019e-05, 
       1.1398772300010273e-05, 
       1.2798525948146793e-05, 
       1.1653871052141508e-05, 
       1.3632118683130851e-05, 
       1.1148406474139346e-05, 
       1.3733998257353609e-05, 
       1.3305005818390715e-05, 
       1.0998248694589623e-05, 
       1.0998248694589623e-05, 
       1.1637082991179322e-05, 
       1.5368410306380071e-05, 
       1.3192941165492487e-05, 
       1.1188647725158896e-05, 
       1.0935645280156292e-05, 
       1.4542949985434590e-05, 
       1.3179870361789488e-05, 
       1.4960353796963889e-05, 
       1.4915837093744389e-05, 
       1.2088326964332276e-05, 
       1.4563764170574967e-05, 
       1.0863742504531845e-05, 
       1.5313167867916095e-05, 
       1.1192641060427904e-05, 
       1.1948775482607372e-05, 
       1.4452860451827453e-05, 
       1.2741662513848429e-05, 
       1.1317217686696363e-05, 
       1.1793717616235744e-05, 
       1.4855051404153621e-05, 
       1.4240635417252080e-05, 
       1.3825001173800063e-05, 
       1.1332938053101302e-05, 
       1.2580925969486195e-05, 
       1.3773935002405795e-05, 
       1.5258911695267703e-05, 
       1.1840574609522966e-05, 
       1.1139851175239220e-05, 
       1.3393528685845125e-05, 
       1.1249787407348119e-05, 
       1.0860150975170697e-05, 
       1.0885184065242962e-05, 
       1.2670890151171056e-05, 
       1.1170914243864697e-05, 
       1.0801434278990760e-05, 
       1.2765339764892054e-05, 
       1.0803464142490780e-05, 
       1.2366155267475223e-05, 
       1.5161410642571402e-05, 
       1.1219118698651237e-05, 
       1.2166386870818340e-05, 
       1.1544391300269693e-05, 
       1.1685487812199201e-05, 
       1.3685283576603951e-05, 
       1.3273456653083252e-05, 
       1.1664554800437190e-05, 
       1.1291937457610047e-05, 
       1.2554115313255417e-05, 
       1.0921965061761196e-05, 
       1.1607722747098729e-05, 
       1.0963988595682829e-05, 
       1.3731446740896991e-05, 
       1.2527592038560479e-05, 
       1.4171055758210119e-05, 
       1.5111095674629706e-05, 
       1.2538575122121326e-05, 
       1.2037640370020864e-05, 
       1.0728270322101026e-05, 
       1.2678203484924697e-05, 
       1.1274157493675601e-05, 
       1.4871411465465572e-05, 
       1.2522508413807697e-05, 
       1.0659127960713351e-05, 
       1.5055603144628777e-05, 
       1.2518855054635743e-05, 
       1.0651886373215528e-05, 
       1.0651886373215528e-05, 
       1.4886407385742349e-05, 
       1.3918019086430911e-05, 
       1.4347075010925284e-05, 
       1.4347075010925284e-05, 
       1.4442270658604354e-05, 
       1.2952642168319032e-05, 
       1.2952642168319032e-05, 
       1.1660242166527791e-05, 
       1.5016282501593631e-05, 
       1.0831100220351874e-05, 
       1.1197738453633726e-05, 
       1.2570870031835301e-05, 
       1.1465574695326973e-05, 
       1.0665439726870978e-05, 
       1.3759046524795987e-05, 
       1.4412923394003059e-05, 
       1.0571717301273498e-05, 
       1.1154961514419220e-05, 
       1.0890198866829411e-05, 
       1.4778160118356734e-05, 
       1.2621970953511089e-05, 
       1.2832151227168138e-05, 
       1.2626677746179825e-05, 
       1.4837709127623125e-05, 
       1.0910721916515993e-05, 
       1.3742445935768050e-05, 
       1.3742445935768050e-05, 
       1.4683404986516526e-05, 
       1.2344215961474470e-05, 
       1.3127125977790372e-05, 
       1.0676552533051296e-05, 
       1.4161551654548738e-05, 
       1.2306883668434777e-05, 
       1.2303001215473267e-05, 
       1.1164975310819916e-05, 
       1.2194597169369334e-05, 
       1.2115055312501551e-05, 
       1.2170763628642250e-05, 
       1.4484924963431202e-05, 
       1.1252811890346518e-05, 
       1.0495937740821709e-05, 
       1.4140604224437157e-05, 
       1.4681525362788206e-05, 
       1.0512675903067279e-05, 
       1.4243496757924631e-05, 
       1.0595719419054424e-05, 
       1.0449406925214181e-05, 
       1.1456951762915769e-05, 
       1.4724446820686352e-05, 
       1.3434622479719439e-05, 
       1.4709892644663882e-05, 
       1.1836000707292320e-05, 
       1.4713335443331565e-05, 
       1.1082379972575165e-05, 
       1.2242769296151881e-05, 
       1.1602853573823651e-05, 
       1.0716602306352488e-05, 
       1.0534752474499265e-05, 
       1.1344205164660185e-05, 
       1.0401305166555006e-05, 
       1.1014401181863681e-05, 
       1.4659949622414877e-05, 
       1.0384773996917772e-05, 
       1.4036959989357961e-05, 
       1.2524136353664507e-05, 
       1.0936214361452002e-05, 
       1.2153357203736828e-05, 
       1.4066699953688203e-05, 
       1.4141498238269605e-05, 
       1.2392378097987866e-05, 
       1.4331445284315292e-05, 
       1.4586442636674197e-05, 
       1.1830139090588987e-05, 
       1.0958495499256982e-05, 
       1.2428462634778150e-05, 
       1.0921579700293800e-05, 
       1.1666150705640568e-05, 
       1.4010764386899351e-05, 
       1.1956788430085225e-05, 
       1.0344890790468721e-05, 
       1.3971773772161322e-05, 
       1.0549690635922559e-05, 
       1.0318738385705355e-05, 
       1.0898921519712717e-05, 
       1.0543194708015209e-05, 
       1.4145343480733565e-05, 
       1.2060006773585947e-05, 
       1.3792486401472005e-05, 
       1.3869927962826378e-05, 
       1.2416426872968311e-05, 
       1.0338576661525758e-05, 
       1.0559825004865625e-05, 
       1.0484910908408635e-05, 
       1.3039043275302748e-05, 
       1.1697142927916625e-05, 
       1.4450694127899086e-05, 
       1.1643815849938036e-05, 
       1.2007908226959582e-05, 
       1.4425539353924863e-05, 
       1.0506640473049503e-05, 
       1.0474942937423516e-05, 
       1.3889033386983225e-05, 
       1.1880747508122686e-05, 
       1.2489604700702411e-05, 
       1.2489604700702411e-05, 
       1.0392611058292722e-05, 
       1.2772052783093288e-05, 
       1.2772052783093288e-05, 
       1.1808845978552323e-05, 
       1.1808845978552323e-05, 
       1.1945022081057719e-05, 
       1.2512305861297867e-05, 
       1.0415226473659687e-05, 
       1.0737765500422271e-05, 
       1.4402345954977824e-05, 
       1.0887444579532782e-05, 
       1.1703899220320226e-05, 
       1.1959594093656916e-05, 
       1.0773236559004109e-05, 
       1.4360862432694292e-05, 
       1.2240427739044189e-05, 
       1.1216123372096402e-05, 
       1.1788566356216907e-05, 
       1.3948310641137338e-05, 
       1.3815750446755045e-05, 
       1.2920933560247753e-05, 
       1.4134879628969013e-05, 
       1.1160623781527150e-05, 
       1.1160623781527150e-05, 
       1.2233412682813135e-05, 
       1.0317150656727319e-05, 
       1.0317150656727319e-05, 
       1.4289570782305795e-05, 
       1.0196487438925396e-05, 
       1.2790905708513912e-05, 
       1.2060137848635450e-05, 
       1.3550361314499474e-05, 
       1.1836798241633949e-05, 
       1.0346391351992383e-05, 
       1.2046436088159806e-05, 
       1.4100372472216545e-05, 
       1.3359771035025645e-05, 
       1.0997381950406280e-05, 
       1.2069338066894263e-05, 
       1.3976482602792469e-05, 
       1.4284330144195686e-05, 
       1.2890366120256928e-05, 
       1.4033048240817880e-05, 
       1.3534855672027939e-05, 
       1.0451472677867640e-05, 
       1.3268856864578705e-05, 
       1.1649828295476208e-05, 
       1.2061549760662306e-05, 
       1.2038304402786417e-05, 
       1.2038304402786417e-05, 
       1.1571451762549295e-05, 
       1.0118941153482331e-05, 
       1.0618798067062287e-05, 
       1.4238500739643426e-05, 
       1.0319876860561089e-05, 
       1.1768856122295848e-05, 
       1.2940979908592388e-05, 
       1.0746593594366579e-05, 
       1.0273276574236062e-05, 
       1.2248882187734898e-05, 
       1.2902405785523740e-05, 
       1.4125490646593750e-05, 
       1.0675488628701313e-05, 
       1.1547106398575243e-05, 
       1.1904903197152804e-05, 
       1.4102460234349681e-05, 
       1.0170247993596534e-05, 
       1.0170247993596534e-05, 
       1.3915573420428839e-05, 
       1.1709741867561160e-05, 
       1.0927570149911177e-05, 
       1.1812442133948183e-05, 
       1.4098690081250948e-05, 
       1.4098690081250948e-05, 
       1.3482363114989975e-05, 
       1.3989348178459340e-05, 
       1.0362414106611471e-05, 
       1.0498922550965688e-05, 
       1.0000000000000279e-05, 
       1.0000000000000279e-05, 
       1.2645135303900096e-05, 
       1.3930243692899263e-05, 
       1.3934062965166813e-05, 
       1.1720257817511319e-05, 
       1.1409584860212466e-05, 
       1.2331497769339113e-05, 
       1.0450595613188668e-05, 
       1.1691106391661278e-05, 
       1.3144167343231384e-05, 
       1.2307362758157403e-05, 
       1.1642030449468130e-05, 
       1.3958305273500112e-05, 
       1.1811713207911753e-05, 
       1.1811713207911753e-05, 
       1.0439481800487469e-05, 
       1.0439481800487469e-05, 
       1.3984277874136380e-05, 
       1.3829935804982754e-05, 
       1.0838563709328192e-05, 
       1.0150728843913007e-05, 
       1.3602066078744027e-05, 
       1.3602066078744027e-05, 
       1.0680250448823534e-05, 
       1.3903475554934372e-05, 
       1.1370130817448192e-05, 
       1.0194294979848547e-05, 
       1.1565059803575839e-05, 
       1.1879800114594682e-05, 
       1.1335664568262363e-05, 
       1.0362255223359137e-05, 
       1.0390016845283681e-05, 
       1.1177819124039588e-05, 
       1.1311927842918881e-05, 
       1.3576590305187022e-05, 
       1.0028811440704425e-05, 
       1.1341182313497504e-05, 
       1.1938097032108912e-05, 
       1.1535052014800086e-05, 
       1.3388378207252262e-05, 
       1.1807400927882543e-05, 
       1.3733780865396033e-05, 
       1.0073761459710556e-05, 
       1.1268215155539663e-05, 
       1.0822684638795439e-05, 
       1.0116852966843347e-05, 
       1.1845855739428831e-05, 
       1.3815380458341364e-05, 
       1.0353519986385917e-05, 
       1.0067961931794342e-05, 
       1.1055772251224289e-05, 
       1.0016432349673548e-05, 
       1.3210432289812378e-05, 
       1.1446403711334912e-05, 
       1.0233904652838011e-05, 
       1.0018719607251077e-05, 
       1.2660729345573786e-05, 
       1.1834000877725536e-05, 
       1.1834000877725536e-05, 
       1.0994290883737941e-05, 
       1.0698225210999301e-05, 
       1.3685380851341690e-05, 
       1.3651032839328186e-05, 
       1.0359912752619529e-05, 
       1.0317716094179740e-05, 
       1.1784193740799748e-05, 
       1.1213299153867411e-05, 
       1.0713899196496290e-05, 
       1.3644861737491657e-05, 
       1.1115395959777903e-05, 
       1.2697872481339443e-05, 
       1.3657603597674202e-05, 
       1.3035310519502945e-05, 
       1.3212153974730906e-05, 
       1.1337604904769409e-05, 
       1.3449679888745507e-05, 
       1.0404820062256778e-05, 
       1.0404820062256778e-05, 
       1.1803952086063882e-05, 
       1.1076539212505333e-05, 
       1.2026156016208614e-05, 
       1.3200601008670166e-05, 
       1.3391327431721781e-05, 
       1.1855046325072819e-05, 
       1.0009598062220815e-05, 
       1.0163739277361600e-05, 
       9.9999999999998755e-06, 
       1.3518213507169691e-05, 
       1.2222460518575340e-05, 
       1.1572690721656101e-05, 
       1.0827124939977011e-05, 
       1.2190721148789807e-05, 
       1.3184863891068122e-05, 
       1.3184863891068122e-05, 
       1.3558044760643362e-05, 
       1.2159319615327873e-05, 
       1.3550324963493495e-05, 
       1.0127763570488329e-05, 
       1.1267956287192027e-05, 
       1.1244205646962845e-05, 
       1.1257517530190761e-05, 
       1.3347899136801808e-05, 
       1.3435666467546523e-05, 
       1.1138693608766828e-05, 
       1.0403185702723281e-05, 
       1.0411360157012967e-05, 
       1.1233359572429419e-05, 
       1.1339773152336635e-05, 
       1.0065718675181396e-05, 
       1.3328305577333125e-05, 
       1.1243525739781475e-05, 
       1.2807430383209726e-05, 
       1.1412327518150835e-05, 
       1.1240386522981550e-05, 
       1.3436414976920724e-05, 
       1.0117373966157046e-05, 
       1.1455433895513748e-05, 
       1.1455433895513748e-05, 
       1.1423648558252021e-05, 
       1.0060410898710771e-05, 
       1.1696808898257769e-05, 
       1.1696808898257769e-05, 
       1.0288919083048500e-05, 
       1.3416746058899971e-05, 
       1.1633192973200580e-05, 
       1.1175006644526828e-05, 
       1.2878143762739536e-05, 
       1.0218961177469847e-05, 
       1.0106218887709105e-05, 
       1.3391470593782954e-05, 
       1.3405400948871187e-05, 
       1.0348747319988554e-05, 
       1.1116419317269919e-05, 
       1.2855094063585698e-05, 
       1.3390423371862794e-05, 
       1.1088076901555095e-05, 
       1.1116934561021705e-05, 
       1.1955241851797231e-05, 
       1.1304320880906099e-05, 
       9.9550323628339216e-06, 
       1.2393819279685939e-05, 
       1.1193265754124183e-05, 
       1.1294229249108099e-05, 
       1.3295096665274059e-05, 
       1.1111830722654068e-05, 
       1.1076007966670047e-05, 
       1.0753402324570199e-05, 
       1.0297624798953658e-05, 
       1.1084236647218861e-05, 
       1.3256133325131781e-05, 
       1.0006270624895459e-05, 
       1.1911969898245642e-05, 
       1.1911969898245642e-05, 
       1.3170578888812826e-05, 
       1.2257422759529661e-05, 
       1.1001550308801094e-05, 
       1.2720525018207747e-05, 
       9.9999999999999415e-06, 
       1.0000000000000440e-05, 
       1.0715636096181351e-05, 
       9.9999999999997044e-06, 
       1.0000000000000428e-05, 
       1.0870910800703376e-05, 
       1.0749077446380380e-05, 
       1.3186147797601452e-05, 
       1.2719646280423556e-05, 
       1.3094705236605134e-05, 
       1.0000000000000377e-05, 
       1.2747419464837698e-05, 
       1.1232156746794980e-05, 
       1.2883378125676412e-05, 
       1.2251724674349591e-05, 
       1.3016958173967444e-05, 
       1.0222411768726092e-05, 
       1.2834305081252563e-05, 
       1.2968890800170580e-05, 
       1.2990915904436691e-05, 
       1.2083980017933565e-05, 
       1.0785472236506439e-05, 
       1.2203615024909420e-05, 
       1.2203615024909420e-05, 
       1.0981226683038296e-05, 
       1.0533411740158035e-05, 
       1.0830199438578192e-05, 
       1.1960671887137153e-05, 
       1.0785929048776913e-05, 
       1.3025335988897369e-05, 
       1.1136781492999438e-05, 
       1.1612336003839269e-05, 
       1.2988377962757502e-05, 
       1.1164984246547905e-05, 
       1.0210854727747511e-05, 
       1.2446618436692591e-05, 
       1.0078031489747915e-05, 
       1.1678049274259802e-05, 
       1.3025995798683719e-05, 
       1.0544724608204932e-05, 
       9.6794497300601840e-06, 
       1.1096749297320163e-05, 
       1.0084975690122371e-05, 
       1.1340258203439586e-05, 
       1.0219172695380650e-05, 
       1.0742345396415169e-05, 
       1.0641298291999593e-05, 
       1.2449030052083542e-05, 
       1.0587909295528927e-05, 
       1.1126537284836484e-05, 
       1.1014402227703725e-05, 
       1.1634600359898741e-05, 
       1.1952925409807471e-05, 
       1.2532304611125723e-05, 
       1.0006342479309403e-05, 
       1.2468375296210687e-05, 
       1.2399793328269677e-05, 
       1.0760154193391669e-05, 
       1.0347115243372036e-05, 
       1.1845462406893194e-05, 
       1.1845462406893194e-05, 
       1.0332953921133789e-05, 
       1.2777486500807533e-05, 
       1.0984214956873311e-05, 
       1.2392227816465758e-05, 
       1.0103732349625345e-05, 
       1.0597505518281205e-05, 
       1.2782310985670216e-05, 
       1.2798816149107451e-05, 
       9.5755252517464675e-06, 
       1.0930061735249371e-05, 
       1.1215532672171842e-05, 
       1.1215532672171842e-05, 
       1.0623627311694828e-05, 
       1.1747856010054313e-05, 
       1.2672824620875476e-05, 
       1.1675461920018449e-05, 
       1.1696275840969826e-05, 
       1.2721106820871759e-05, 
       1.0975663700983502e-05, 
       1.0417188511511460e-05, 
       1.1564473419440789e-05, 
       1.2374252902852461e-05, 
       1.2699751570011256e-05, 
       1.2300450326821395e-05, 
       1.0572107590056833e-05, 
       1.2653597338284156e-05, 
       1.0532358432657360e-05, 
       1.0508712107490871e-05, 
       1.2127941949950451e-05, 
       1.2566948973633547e-05, 
       1.1694817858472381e-05, 
       1.1310794626647904e-05, 
       1.1351314937667811e-05, 
       1.2506580372841982e-05, 
       1.2177663457467430e-05, 
       1.0043356751573709e-05, 
       1.1825744830123676e-05, 
       1.2619741042911400e-05, 
       1.0436018383848676e-05, 
       1.1736153279790776e-05, 
       1.0440200983722432e-05, 
       1.2581057553944129e-05, 
       1.0736840566740127e-05, 
       1.0987452393915333e-05, 
       1.2560404705040478e-05, 
       1.1669068649218330e-05, 
       1.0614556692125842e-05, 
       1.0614556692125842e-05, 
       1.1852608794110614e-05, 
       1.1234969511137279e-05, 
       1.2522999801233024e-05, 
       1.0925117180852579e-05, 
       1.2340345431203483e-05, 
       1.2177556172870390e-05, 
       1.2077354031971931e-05, 
       1.2203757229499747e-05, 
       1.2377389666457332e-05, 
       1.2143601259765221e-05, 
       1.1856611632928141e-05, 
       1.0400051323449531e-05, 
       1.0674788821728457e-05, 
       1.0399108779269436e-05, 
       1.0053744132178589e-05, 
       1.1625569696674678e-05, 
       1.0349662063680248e-05, 
       1.2002949800122252e-05, 
       1.2401007241727416e-05, 
       1.0342939153113044e-05, 
       9.3670097431455861e-06, 
       1.2377607250777407e-05, 
       1.1615683170861384e-05, 
       1.2250537626559507e-05, 
       1.2219696523602408e-05, 
       1.1874515054363922e-05, 
       1.0306232340991216e-05, 
       1.0432658144539479e-05, 
       1.0019291185314555e-05, 
       1.0302717880861760e-05, 
       1.2395893541423118e-05, 
       1.0030008371672095e-05, 
       1.0297812240413762e-05, 
       1.0208317787615753e-05, 
       1.1853376772846337e-05, 
       1.1491020166018797e-05, 
       1.1889100513578049e-05, 
       1.0407403538684043e-05, 
       1.1710351188532819e-05, 
       1.0515067683171620e-05, 
       9.9999999999999940e-06, 
       1.1566324598341433e-05, 
       1.1025205482800089e-05, 
       1.1025205482800089e-05, 
       1.0334363610980777e-05, 
       1.0189613441608596e-05, 
       1.0587194818543708e-05, 
       1.2229520628921372e-05, 
       1.2076292359073861e-05, 
       1.1741456861510894e-05, 
       1.0200865513927602e-05, 
       1.0475953590698195e-05, 
       1.1479892200014455e-05, 
       1.2051128764004300e-05, 
       1.2090608747023257e-05, 
       1.0127004370090950e-05, 
       1.1477608575747280e-05, 
       1.2160734109956506e-05, 
       1.0867992493628197e-05, 
       1.2113850192128568e-05, 
       1.1911988206350110e-05, 
       1.1948098404588587e-05, 
       1.2063963158768995e-05, 
       1.0312983318377638e-05, 
       1.1629094794586838e-05, 
       1.0022621621885319e-05, 
       1.1595536579347005e-05, 
       1.1695246407393041e-05, 
       1.1812296131280923e-05, 
       1.1300497158384471e-05, 
       1.2039192290143985e-05, 
       1.1606008434542128e-05, 
       1.1949360841369873e-05, 
       1.0050360783253772e-05, 
       1.1732673705138513e-05, 
       1.1853169992719367e-05, 
       1.0523392207894367e-05, 
       1.1470762308426742e-05, 
       1.1459202078066045e-05, 
       1.0186962561994858e-05, 
       1.0193686357858046e-05, 
       1.1520984510337517e-05, 
       1.0349309317390205e-05, 
       1.0269152918098087e-05, 
       8.8484650919961647e-06, 
       1.1354845515236625e-05, 
       1.1757800557543071e-05, 
       1.0240297804286224e-05, 
       1.0768466438576220e-05, 
       1.1370481931591041e-05, 
       1.0550888729578210e-05, 
       1.1752033074206899e-05, 
       1.0888696548090402e-05, 
       1.1800349753385263e-05, 
       1.0048735759995933e-05, 
       1.1309763070584687e-05, 
       1.0837666862013228e-05, 
       1.1644174202776711e-05, 
       1.0291293135718161e-05, 
       1.0272204771767398e-05, 
       1.0595571812835537e-05, 
       1.1091516617349417e-05, 
       1.1027046333736054e-05, 
       1.1353019638199176e-05, 
       1.0679995378434379e-05, 
       1.1144959444864566e-05, 
       1.1679238391135216e-05, 
       1.0773817407946089e-05, 
       1.0398844655399668e-05, 
       1.0398844655399668e-05, 
       1.1557842536867459e-05, 
       1.1531066476708649e-05, 
       1.0503387037515746e-05, 
       1.1374708756226850e-05, 
       9.1280753193642096e-06, 
       1.0021678281198925e-05, 
       1.0411308026971035e-05, 
       1.0568563641505513e-05, 
       1.0253009534947850e-05, 
       1.1541357863022209e-05, 
       1.1098433566914407e-05, 
       1.0108125320667679e-05, 
       9.3931513146355596e-06, 
       1.1072652257075415e-05, 
       9.5718606092017721e-06, 
       1.1214199276524564e-05, 
       9.9999999999996027e-06, 
       1.1450765332801129e-05, 
       1.1091869081008813e-05, 
       1.1287319872967942e-05, 
       1.0970261784340101e-05, 
       1.0998661295750510e-05, 
       1.0674522609164812e-05, 
       9.9999999999998297e-06, 
       9.9999999999999076e-06, 
       1.0134039232724425e-05, 
       1.1357172690822843e-05, 
       1.0865963429175878e-05, 
       1.1311608420053632e-05, 
       1.1305099762793722e-05, 
       1.0293697436279193e-05, 
       1.0000000000000169e-05, 
       1.0111510657551930e-05, 
       1.0162412064020230e-05, 
       1.1266143376596381e-05, 
       1.1133261164504937e-05, 
       1.1133261164504937e-05, 
       1.1012183529695346e-05, 
       1.0000000000000263e-05, 
       1.0000000000000263e-05, 
       1.0080407574724099e-05, 
       1.0080407574724099e-05, 
       1.1112532850200788e-05, 
       1.0993256584251860e-05, 
       1.0340052037401160e-05, 
       1.1123452260431328e-05, 
       1.0844070741306635e-05, 
       1.0225208581808673e-05, 
       1.1120765085876377e-05, 
       1.1088575746234184e-05, 
       1.0778394460207925e-05, 
       1.0778394460207925e-05, 
       1.1073059461357174e-05, 
       1.0960301490463042e-05, 
       1.0216391029781544e-05, 
       1.0991863651548847e-05, 
       1.0306373554340460e-05, 
       1.0330738841434769e-05, 
       1.0858748761691399e-05, 
       1.0849631402720742e-05, 
       1.0977045380432893e-05, 
       1.0975840841820361e-05, 
       1.0878729073320801e-05, 
       1.0871566471404104e-05, 
       1.0886766153139473e-05, 
       9.3273489759907413e-06, 
       1.0901075893622793e-05, 
       1.0455979399303563e-05, 
       9.0990434321070764e-06, 
       1.0824462423078507e-05, 
       8.0654569579004804e-06, 
       1.0000000000000468e-05, 
       9.9999999999999957e-06, 
       1.0000000000000170e-05, 
       1.0000000000000465e-05, 
       9.9999999999996281e-06, 
       1.0000000000000014e-05, 
       1.0000000000000209e-05, 
       1.0000000000000318e-05, 
       1.0000000000000450e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       1.0000000000000213e-05, 
       9.9999999999999652e-06, 
       1.0000000000000426e-05, 
       1.0564065212031581e-05, 
       1.0288105753802190e-05, 
       1.0370575677714644e-05, 
       1.0556041315866993e-05, 
       1.0307439579020981e-05, 
       1.0715095308724441e-05, 
       1.0810349502745650e-05, 
       1.0264067040872218e-05, 
       1.0787179285690881e-05, 
       1.0785344832062089e-05, 
       1.0172753410981790e-05, 
       1.0720598654292347e-05, 
       1.0706982955864209e-05, 
       1.0687750516855111e-05, 
       1.0290101921696099e-05, 
       1.0686754909453875e-05, 
       1.0499263293434549e-05, 
       1.0351941646251096e-05, 
       1.0635177322745668e-05, 
       1.0440397063716049e-05, 
       1.0623649871596379e-05, 
       1.0610146193661149e-05, 
       1.0303841302466265e-05, 
       1.0153375238608839e-05, 
       1.0013914749421234e-05, 
       1.0094953744876053e-05, 
       1.0541846767509978e-05, 
       1.0194270448630770e-05, 
       1.0499712394779468e-05, 
       1.0473551813943907e-05, 
       1.0155056258731191e-05, 
       1.0282826434473411e-05, 
       1.0449145634349503e-05, 
       1.0377275934931858e-05, 
       1.0340072110039857e-05, 
       1.0065763365093438e-05, 
       1.0427275746230539e-05, 
       1.0285423930497767e-05, 
       1.0182418398486908e-05, 
       8.5832802682249529e-06, 
       1.0338219749427979e-05, 
       1.0130416766810636e-05, 
       1.0285725109203654e-05, 
       9.9999999999997111e-06, 
       9.9999999999997111e-06, 
       1.0290362655745962e-05, 
       1.0135450172217182e-05, 
       1.0082717409549143e-05, 
       1.0105133511442682e-05, 
       1.0000000000000202e-05, 
       1.0000000000000158e-05, 
       1.0000000000000069e-05, 
       1.0000000000000170e-05, 
       1.0000000000000170e-05, 
       9.9999999999995705e-06, 
       9.9999999999999517e-06, 
       9.9999999999997467e-06, 
       9.9999999999995790e-06, 
       1.0000000000000204e-05, 
       1.0000000000000370e-05, 
       9.9999999999997992e-06, 
       9.9999999999996993e-06, 
       1.0000000000000438e-05, 
       1.0000000000000255e-05, 
       9.9999999999997365e-06, 
       9.9999999999997365e-06, 
       1.0000000000000043e-05, 
       1.0000000000000043e-05, 
       9.9999999999996518e-06, 
       9.9999999999996518e-06, 
       9.9999999999995688e-06, 
       9.9999999999995688e-06, 
       9.9999999999995688e-06, 
       9.9999999999997874e-06, 
       9.9999999999997874e-06, 
       9.9999999999997874e-06, 
       1.0000000000000457e-05, 
       1.0000000000000457e-05, 
       9.9999999999995045e-06, 
       1.0000000000000473e-05, 
       9.9999999999997416e-06, 
       1.0000000000000311e-05, 
       9.9999999999996400e-06, 
       9.9999999999998433e-06, 
       9.9999999999996739e-06, 
       1.0000000000000079e-05, 
       1.0000000000000092e-05, 
       9.9999999999995722e-06, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       9.6947656195540415e-06, 
       9.6228574014668216e-06, 
       9.7034679746248499e-06, 
       7.3920536245222257e-06, 
       7.0274129384601532e-06, 
       8.9536969490748550e-06, 
       8.5126783097362612e-06, 
       6.5149018314254372e-06, 
       6.6969377474252021e-06, 
       5.5029010393726161e-06, 
       6.8751000241200378e-06, 
       5.8586255084856454e-06, 
       5.9122700370890593e-06, 
       5.5104703349470542e-06, 
       6.0489566729078503e-06, 
       4.7265633678330344e-06, 
       5.8323602227067933e-06, 
       6.3133457526778899e-06, 
       5.0223046031235508e-06, 
       5.0002790689580138e-06, 
       4.2994300730316646e-06, 
       5.0344946337972877e-06, 
       4.8415215306477271e-06, 
       5.1789303693454573e-06, 
       5.4300748049888902e-06, 
       5.2141598280559893e-06, 
       4.7455087054416432e-06, 
       4.6884863112960923e-06, 
       4.8678671067706894e-06, 
       4.7727031223995261e-06, 
       4.8387751602642047e-06, 
       3.7556461523087268e-06, 
       4.2199348874234166e-06, 
       3.6324177563669891e-06, 
       4.8692675598497281e-06, 
       4.2778472530681033e-06, 
       3.3656582427142031e-06, 
       3.2269262894011530e-06, 
       3.1799569455096466e-06, 
       3.5567328700216482e-06, 
       3.5689368010299873e-06, 
       2.7916620831483727e-06, 
       2.6982026257262879e-06, 
       3.4064074860640183e-06, 
       3.1067653024678614e-06, 
       2.6167326963182031e-06, 
       2.3716971203355896e-06, 
       2.5389747475265993e-06, 
       3.0639137397203202e-06, 
       2.4414799820587471e-06, 
       2.8030249172971114e-06, 
       2.1686230862676590e-06, 
       1.5878777924834358e-06, 
       2.1081204277837366e-06, 
       1.9314470035402226e-06, 
       2.0449555221301887e-06, 
       1.4729447210363646e-06, 
       1.6947966107605757e-06, 
       1.8010722950656040e-06, 
       1.8408263960734567e-06, 
       1.8438197083066655e-06, 
       1.6371654430150459e-06, 
       1.4340882255152124e-06, 
       1.6319399025586903e-06, 
       1.5245528968978507e-06, 
       1.1030452099890286e-06, 
       8.2708500992786734e-07, 
       6.6699390894942584e-07, 
       8.0763361059836817e-07, 
       4.9753649352562971e-07, 
       4.5490228422152818e-07, 
       2.6751288427370527e-07, 
       2.3421106498979940e-07, 
       2.1313725025835865e-07, 
       8.5491195928982287e-08, 
       5.4981671066789653e-08, 
       7.4300000000000005e-02, 
       3.0429999999999999e-02, 
       2.2290000000000001e-02, 
       1.9990000000000001e-02, 
       1.8689999999999998e-02, 
       1.6959999999999999e-02, 
       1.6230000000000001e-02, 
       1.3037558548713546e-02, 
       1.0883081458488691e-02, 
       1.0878769050655761e-02, 
       1.4189999999999999e-02, 
       1.3380000000000000e-02, 
       1.2789999999999999e-02, 
       1.2149999999999999e-02, 
       5.4599999999999996e-03, 
       4.4299999999999999e-03, 
       3.4199999999999999e-03, 
       3.3000000000000000e-03, 
       3.1800000000000001e-03, 
       2.9566112801382260e-03, 
       2.8500000000000001e-03, 
       2.0699999999999998e-03, 
       2.0200000000000001e-03, 
       2.0200000000000001e-03, 
       2.0000000000000000e-03, 
       1.9800000000000000e-03, 
       1.9300000000000001e-03, 
       1.6400000000000000e-03, 
       1.6100000000000001e-03, 
       1.2060890952148998e-03, 
       1.5800000000000000e-03, 
       1.1686083171023138e-03, 
       1.4900000000000000e-03, 
       1.3500000000000001e-03, 
       1.0399999999999999e-03, 
       7.4680407001616875e-04, 
       7.4676655249667683e-04, 
       7.2973569750647870e-04, 
       8.4999999999999995e-04, 
       6.8973737323774101e-04, 
       7.9000000000000001e-04, 
       7.6000000000000004e-04, 
       5.9744224230548612e-04, 
       6.8000000000000005e-04, 
       5.9999999999999995e-04, 
       5.6999999999999998e-04, 
       5.6999999999999998e-04, 
       5.4000000000000001e-04, 
       5.2999999999999998e-04, 
       5.2999999999999998e-04, 
       3.9776663812518353e-04, 
       3.7762891855149514e-04, 
       3.7702581747575459e-04, 
       3.8163471818196041e-04, 
       4.8000000000000001e-04, 
       4.2999999999999999e-04, 
       4.0000000000000002e-04, 
       3.6999999999999999e-04, 
       3.5000000000000000e-04, 
       2.4370788829310061e-04, 
       3.2000000000000003e-04, 
       2.3627285519262696e-04, 
       3.1000000000000000e-04, 
       3.1000000000000000e-04, 
       2.5942127348425718e-04, 
       2.5888426091302705e-04, 
       2.5886924283989409e-04, 
       2.7300268859169316e-04, 
       2.7291627567911913e-04, 
       2.9000000000000000e-04, 
       2.9000000000000000e-04, 
       2.3788732299128356e-04, 
       2.7999999999999998e-04, 
       2.7999999999999998e-04, 
       2.7999999999999998e-04, 
       2.1051633554646494e-04, 
       2.0866728104305629e-04, 
       2.4367274160316422e-04, 
       2.4365524728625894e-04, 
       2.0794897620685812e-04, 
       2.5999999999999998e-04, 
       2.5999999999999998e-04, 
       1.7171600883160595e-04, 
       1.7170411054768196e-04, 
       2.2000000000000001e-04, 
       2.2000000000000001e-04, 
       2.0000000000000001e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.5533250198832314e-04, 
       1.8000000000000001e-04, 
       1.7000000000000001e-04, 
       1.4058859547840189e-04, 
       1.6000000000000001e-04, 
       1.6000000000000001e-04, 
       1.6000000000000001e-04, 
       1.3000000000004154e-04, 
       1.0768464363767053e-04, 
       1.0768464363767053e-04, 
       1.4999999999999999e-04, 
       1.1125177649531631e-04, 
       1.0822709668576399e-04, 
       1.2338914480495807e-04, 
       1.1202816046217404e-04, 
       1.0662936127647476e-04, 
       1.0293151870100175e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       9.6955674761802201e-05, 
       9.6813612919886404e-05, 
       1.0512758152498566e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.0854129662748241e-04, 
       1.0000000000000323e-04, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       1.1000000000000000e-04, 
       9.3250503713334041e-05, 
       8.5009088244708902e-05, 
       8.2886632511872054e-05, 
       8.2558474534351651e-05, 
       1.0000000000000000e-04, 
       8.0000000000003259e-05, 
       8.0584601377312788e-05, 
       6.5524020893980520e-05, 
       7.6482276851296802e-05, 
       7.6465672125631959e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       7.5198128283420474e-05, 
       6.9291714827347215e-05, 
       6.9268994988211548e-05, 
       8.2402233907603384e-05, 
       6.4630394994564559e-05, 
       6.4616272204064322e-05, 
       8.1917723578008500e-05, 
       7.0000000000005266e-05, 
       7.9365343260021315e-05, 
       6.9772390506842597e-05, 
       6.0000000000003647e-05, 
       5.9999999999997928e-05, 
       5.9999999999997928e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       6.2916140622468645e-05, 
       6.6331209696967878e-05, 
       7.1389511748867854e-05, 
       5.8736195175162288e-05, 
       5.9999999999995651e-05, 
       5.3042837422496945e-05, 
       5.2967416357068542e-05, 
       5.0429429860771925e-05, 
       6.0000000000001831e-05, 
       6.0000000000003376e-05, 
       6.0425404444090088e-05, 
       6.3897869755598016e-05, 
       5.1611735176274870e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       5.0809716377248796e-05, 
       5.5145331172950521e-05, 
       5.5053685336879535e-05, 
       5.8684151219614443e-05, 
       4.9276780129911816e-05, 
       5.7341132258635974e-05, 
       5.0000000000000158e-05, 
       4.7481446363360433e-05, 
       5.4238958410724838e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       4.2476744935787153e-05, 
       4.8124917068804158e-05, 
       4.4363021086367883e-05, 
       3.9999999999999346e-05, 
       4.9427201300072842e-05, 
       4.9394586853485093e-05, 
       4.0316193439998853e-05, 
       3.9510200506935524e-05, 
       3.7774317278009673e-05, 
       3.8587510437276392e-05, 
       3.9037154783625315e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       3.6607343541813714e-05, 
       4.2473647468209782e-05, 
       4.0000000000001630e-05, 
       3.6437913099884315e-05, 
       3.6437913099884315e-05, 
       3.9999999999998872e-05, 
       4.2503529537477015e-05, 
       3.6230852690919030e-05, 
       3.8115043083519835e-05, 
       3.6785623129306869e-05, 
       3.6670707271088484e-05, 
       3.5476255008093015e-05, 
       3.9501604122313745e-05, 
       3.3673151590614655e-05, 
       3.4370803001368847e-05, 
       3.5638451843159527e-05, 
       4.2236799611545319e-05, 
       3.7303178640235137e-05, 
       4.0000000000003994e-05, 
       3.2974215585232872e-05, 
       3.2863419025276005e-05, 
       2.9983816215052012e-05, 
       3.4129102789038733e-05, 
       2.9999999999997687e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.9729572577651724e-05, 
       3.0000000000000285e-05, 
       2.8949364234034643e-05, 
       3.0000000000003379e-05, 
       2.8922016746195419e-05, 
       3.5761930564004351e-05, 
       2.4820186271723133e-05, 
       2.7505771804118539e-05, 
       2.6203440903550380e-05, 
       2.5622241719660553e-05, 
       2.6717085180325387e-05, 
       3.0429864703042079e-05, 
       3.0424433406478087e-05, 
       2.6750853665162942e-05, 
       2.3806059537227716e-05, 
       2.7911368558389395e-05, 
       2.7886005610531906e-05, 
       2.5833628253127539e-05, 
       2.5833628253127539e-05, 
       2.6925610904461553e-05, 
       2.8856589112952020e-05, 
       2.3141411091770025e-05, 
       2.1502482397030963e-05, 
       2.4093888675263956e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.1139297388069430e-05, 
       2.3738438575771808e-05, 
       2.5241203407279468e-05, 
       2.5227186305818006e-05, 
       2.1838002479472441e-05, 
       2.5546326190138833e-05, 
       2.4490522756024003e-05, 
       2.2281851648640559e-05, 
       2.2281851648640559e-05, 
       2.2542980929659354e-05, 
       2.0681033298082310e-05, 
       2.1706939917175129e-05, 
       2.3743958143640291e-05, 
       2.0000000000001787e-05, 
       2.0429899761154197e-05, 
       2.0314490792511520e-05, 
       2.1358537063817187e-05, 
       2.5709715048454072e-05, 
       2.0000000000002177e-05, 
       2.0000000000002177e-05, 
       2.0751242983137847e-05, 
       1.9522640651934186e-05, 
       2.1833080183892826e-05, 
       1.9999999999999778e-05, 
       2.0389459078440465e-05, 
       2.2824376824859196e-05, 
       2.2810661382132814e-05, 
       2.0771481673299070e-05, 
       2.4284014889381276e-05, 
       1.8444510856471818e-05, 
       1.7891690628402386e-05, 
       1.9693546676850977e-05, 
       1.8133988054219961e-05, 
       1.9605419457062955e-05, 
       1.9999999999995722e-05, 
       1.9999999999995722e-05, 
       1.9999999999995722e-05, 
       2.0000000000003129e-05, 
       1.6654628576726314e-05, 
       1.9441566184141674e-05, 
       1.8278957718691330e-05, 
       1.6477693308954065e-05, 
       2.2881877795479631e-05, 
       1.7542530628398943e-05, 
       2.2882445962267331e-05, 
       1.7254925141115230e-05, 
       1.6215177534801218e-05, 
       1.7920954471522880e-05, 
       1.7521104817646689e-05, 
       1.8640485850935727e-05, 
       1.7392269637706035e-05, 
       2.1667342008788285e-05, 
       1.6772734328430967e-05, 
       2.1494670141462031e-05, 
       1.8113249002410183e-05, 
       1.5540352289672671e-05, 
       1.5540352289672671e-05, 
       1.5003592868804477e-05, 
       1.7914925055106471e-05, 
       1.7713243756555757e-05, 
       1.6168105300603769e-05, 
       1.6103363992769436e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.4372368340300102e-05, 
       1.4186133759386960e-05, 
       1.4352151547141893e-05, 
       1.4095798120461432e-05, 
       1.4062232331749144e-05, 
       1.5240760770330997e-05, 
       1.4214586288609827e-05, 
       1.4027601482944790e-05, 
       1.4790448963432358e-05, 
       1.5387424384441902e-05, 
       1.3791715489902704e-05, 
       1.3731644871520615e-05, 
       1.4647967385860729e-05, 
       1.3751845478867160e-05, 
       1.3315240160250380e-05, 
       1.6368813243386031e-05, 
       1.3585574458850988e-05, 
       1.4651606809567796e-05, 
       1.3528291819554959e-05, 
       1.3911099935290847e-05, 
       1.3487659352217637e-05, 
       1.3585294646382596e-05, 
       1.3165620741512801e-05, 
       1.3280634984460224e-05, 
       1.3116683353679425e-05, 
       1.2932481849632644e-05, 
       1.3121556345327799e-05, 
       1.6179932926484389e-05, 
       1.6157554189723095e-05, 
       1.3187338425071071e-05, 
       1.7709221213291686e-05, 
       1.2829841309757566e-05, 
       1.4344039135810327e-05, 
       1.4344039135810327e-05, 
       1.7561032328014526e-05, 
       1.2580137620756254e-05, 
       1.2518388139364237e-05, 
       1.2931451733940866e-05, 
       1.2482770676828027e-05, 
       1.2833950559067186e-05, 
       1.2209315251853346e-05, 
       1.3537988816381454e-05, 
       1.2748232853975038e-05, 
       1.3535279845194488e-05, 
       1.2564127097208087e-05, 
       1.5773362520496820e-05, 
       1.4271139758125524e-05, 
       1.3079603232597115e-05, 
       1.3077623302636237e-05, 
       1.5618266512877338e-05, 
       1.1904707309409523e-05, 
       1.1868372842740387e-05, 
       1.5827830634012848e-05, 
       1.3932303653497273e-05, 
       1.2034743169603295e-05, 
       1.1989413604475069e-05, 
       1.5684758081397685e-05, 
       1.3030967904734999e-05, 
       1.1833810921141765e-05, 
       1.1901002660347376e-05, 
       1.4133064189559994e-05, 
       1.1384110184290020e-05, 
       1.3815740881573331e-05, 
       1.2276459140089718e-05, 
       1.1608332428358637e-05, 
       1.1679612035632459e-05, 
       1.2029435108224906e-05, 
       1.2415220706934840e-05, 
       1.3013184344661012e-05, 
       1.1383559274334398e-05, 
       1.1423430917571241e-05, 
       1.2842889803800579e-05, 
       1.1300144854427703e-05, 
       1.1253984542610072e-05, 
       1.0769933943114439e-05, 
       1.2872633098187266e-05, 
       1.2759283008884534e-05, 
       1.1167374569971501e-05, 
       1.1167374569971501e-05, 
       1.0980695929938487e-05, 
       1.0504652896427845e-05, 
       1.0532436620651496e-05, 
       1.1392881516369259e-05, 
       1.0217986612048792e-05, 
       1.0178406623445530e-05, 
       1.3256878290149601e-05, 
       1.3160862113254326e-05, 
       1.2006034749489678e-05, 
       1.0591357225369108e-05, 
       1.0028890157238654e-05, 
       1.0735149792831343e-05, 
       1.0441092420834546e-05, 
       1.2790202262021769e-05, 
       1.1058056740146791e-05, 
       1.0191828058226249e-05, 
       1.0172217841118659e-05, 
       1.0048667149972156e-05, 
       1.0089991466293971e-05, 
       1.0801299942415698e-05, 
       1.1430142406271294e-05, 
       1.0649242621603342e-05, 
       1.0446964347364076e-05, 
       1.0017296267836176e-05, 
       1.0019618595250704e-05, 
       1.0836278626856517e-05, 
       1.0143419544145963e-05, 
       1.0857729731242161e-05, 
       1.0579743789073739e-05, 
       1.0896689356192545e-05, 
       1.1444580107221192e-05, 
       9.9999999999999212e-06, 
       1.1729141856298901e-05, 
       1.0215757516093120e-05, 
       1.1078860683095621e-05, 
       1.0635545193756356e-05, 
       1.0205809284028449e-05, 
       9.9999999999995146e-06, 
       1.0110474338076040e-05, 
       1.0107719544353477e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       1.0000000000000001e-05, 
       5.9745227469730624e-06, 
       5.0951155718873691e-06, 
       4.7618291081441396e-06, 
       2.2000000000000001e-04, 
       1.0087314571106695e-04, 
       1.0083334479271939e-04, 
       9.0000000000000006e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.0000000000000002e-05, 
       1.1146003069080525e-05, 
       1.0800050286712896e-05, 
       1.1246653573297428e-05, 
       1.1246653573297428e-05, 
       1.0000000000000001e-05, 
       3.4902409205208069e-06, 
       3.4833260440384979e-06, 
       3.2362948121349587e-06, 
       3.2357566821195204e-06, 
       2.9690018846829778e-06, 
       2.9682706512494376e-06, 
       2.8072116463778653e-06, 
       2.7786644396826463e-06, 
       2.7784064857383409e-06, 
       2.2520425231980443e-06, 
       2.1833608955175073e-06, 
       1.9450551588159206e-06, 
       1.9284968462130885e-06, 
       1.8430097991819099e-06, 
       1.5988315025429415e-06, 
       1.5988315025429415e-06, 
       1.5213327210532660e-06, 
       1.4586075279245748e-06, 
       1.4586075279245748e-06, 
       1.3314436719903048e-06, 
       1.6743986361808660e-06, 
       1.6774389725163116e-06, 
       1.1832334624743139e-06, 
       1.1528463269626069e-06, 
       1.1464196783629788e-06, 
       1.2473860221672384e-06, 
       1.2426583459896085e-06, 
       1.1098055498383791e-06, 
       1.0824410253831882e-06, 
       1.0281543706378878e-06, 
       1.1050332336199652e-06, 
       1.0440587645220091e-06, 
       3.5757309351598726e-07, 
       1.0586189945469200e+00, 
       7.2805201953374787e-01, 
       6.8280880054648996e-01, 
       5.9826466595576788e-01, 
       4.1020630963524468e-01, 
       4.5646999999997689e-01, 
       4.5276999999998541e-01, 
       2.0476399084864377e-01, 
       2.0371387499613716e-01, 
       2.0498740218224085e-01, 
       1.6720763217250953e-01, 
       1.6718951595312512e-01, 
       1.3475149516881021e-01, 
       1.2671484836488850e-01, 
       1.6659269878102087e-01, 
       1.5747892040940964e-01, 
       1.4451289627322528e-01, 
       1.1192876215558543e-01, 
       1.1181140455066313e-01, 
       1.3828052365278862e-01, 
       1.1047692012599106e-01, 
       1.1038616847914615e-01, 
       1.2681900458160425e-01, 
       1.2353537343789060e-01, 
       1.2058657100742542e-01, 
       1.2028056662794633e-01, 
       1.1613190138640254e-01, 
       1.0524828944696524e-01, 
       1.0688999999997963e-01, 
       1.0510671048105651e-01, 
       1.0069486820979097e-01, 
       9.9414858210953250e-02, 
       9.5529999999999324e-02, 
       7.7225433880364042e-02, 
       8.5079999999995035e-02, 
       7.0724544331379507e-02, 
       5.9017169211197630e-02, 
       5.8553916215287917e-02, 
       7.9527568657309616e-02, 
       6.3085137780566874e-02, 
       6.3066812559668478e-02, 
       6.2043854085539162e-02, 
       6.0855887283131696e-02, 
       5.6760274224216332e-02, 
       6.1041780373860551e-02, 
       5.6130002727076030e-02, 
       6.0131480927786259e-02, 
       4.3213017855050009e-02, 
       5.7327620604158566e-02, 
       5.5357006725964497e-02, 
       5.2616624102980153e-02, 
       3.9508360570041971e-02, 
       5.3449999999995251e-02, 
       3.9692924601571006e-02, 
       5.1762400957608659e-02, 
       3.6006552060059871e-02, 
       3.5962990042790466e-02, 
       4.8855307372525685e-02, 
       3.7236248887429650e-02, 
       3.7563096221859084e-02, 
       3.7218290926347805e-02, 
       4.9030411986050570e-02, 
       4.8903147217186776e-02, 
       4.8579980199933252e-02, 
       3.4321570193623280e-02, 
       4.6079842075371358e-02, 
       4.5609760507664410e-02, 
       3.3497324033231260e-02, 
       4.2833221499412863e-02, 
       4.3972546879275584e-02, 
       4.1197792006362151e-02, 
       4.3694920131661520e-02, 
       3.1296757703246740e-02, 
       4.2897038688336810e-02, 
       4.3492047955079383e-02, 
       3.1194425230905414e-02, 
       3.5441743857117541e-02, 
       3.0977985293214511e-02, 
       4.2566573638858553e-02, 
       3.0515096235953068e-02, 
       3.0067356170249988e-02, 
       2.9766490277874622e-02, 
       2.8711951509417761e-02, 
       2.8474563087796573e-02, 
       3.8861805242770567e-02, 
       3.8560000000000441e-02, 
       2.9123484022077557e-02, 
       2.8892529312384712e-02, 
       2.7546259454840828e-02, 
       3.7781089462793793e-02, 
       3.7604118230415283e-02, 
       3.7245629674337949e-02, 
       2.5866656054236093e-02, 
       3.4995927445234186e-02, 
       3.4903973577319065e-02, 
       3.4504790190709296e-02, 
       3.2819158543384755e-02, 
       3.1230687116812633e-02, 
       2.9440000000000001e-02, 
       2.8153426058541760e-02, 
       2.4655337345933302e-02, 
       2.7775316583689071e-02, 
       2.8070000000000428e-02, 
       2.7737911365600075e-02, 
       2.6890930739175839e-02, 
       2.3584211831951040e-02, 
       2.7028045602510171e-02, 
       2.6055436951890359e-02, 
       2.2123374594184189e-02, 
       2.4870000000003698e-02, 
       2.4699999999997772e-02, 
       1.7773991724927683e-02, 
       2.1256384275186852e-02, 
       1.7322420862266562e-02, 
       2.2070346158363792e-02, 
       1.8394252562409022e-02, 
       2.0136071619831171e-02, 
       2.1265023883021140e-02, 
       1.6211175955531020e-02, 
       1.5692828194415732e-02, 
       1.7780420322466840e-02, 
       1.5837231213278020e-02, 
       1.4094510037679675e-02, 
       1.6407090837951589e-02, 
       1.8657709185428913e-02, 
       1.5654395597788209e-02, 
       1.3621206906145515e-02, 
       1.8440011409768566e-02, 
       1.7507748724426508e-02, 
       1.7497629777917821e-02, 
       1.3497022692622788e-02, 
       1.4849232643256069e-02, 
       1.4165326641825940e-02, 
       1.2193984070672469e-02, 
       1.3746145009265819e-02, 
       1.3745911991719062e-02, 
       1.2952638015733941e-02, 
       1.3310301946101498e-02, 
       1.4290029170608971e-02, 
       1.2005230953029848e-02, 
       1.1171710700502747e-02, 
       1.0892484440708176e-02, 
       1.0691369230488378e-02, 
       1.4581261713517446e-02, 
       1.0460892833448233e-02, 
       1.2466269547663882e-02, 
       1.4347134314226637e-02, 
       1.0277458233091862e-02, 
       1.0390367599609164e-02, 
       1.1664555767618207e-02, 
       1.3935959140827859e-02, 
       1.1502089434336709e-02, 
       1.0035074611922408e-02, 
       1.3332895883585449e-02, 
       1.2814405244782105e-02, 
       1.0724944736576871e-02, 
       1.0724942549692756e-02, 
       1.0532770769392014e-02, 
       1.2500696312718790e-02, 
       1.0084446830977221e-02, 
       9.1905665206305932e-03, 
       1.0063604306485967e-02, 
       8.7998793132223509e-03, 
       1.1849602013920687e-02, 
       1.1591269944928326e-02, 
       1.1069055779414416e-02, 
       1.1525366736383423e-02, 
       8.6129361937721232e-03, 
       1.1579997557221684e-02, 
       1.1375458404156701e-02, 
       8.3328350519636003e-03, 
       1.1466077160299076e-02, 
       8.4384727486650542e-03, 
       1.1254606790423116e-02, 
       1.0369315070912183e-02, 
       1.1093255446470662e-02, 
       1.0973729290755629e-02, 
       9.1129240375512292e-03, 
       9.1121055359147975e-03, 
       1.1082293567540669e-02, 
       8.6883213624022616e-03, 
       8.1309044266609527e-03, 
       8.1254775519283284e-03, 
       1.0357623199717381e-02, 
       7.6450667512031577e-03, 
       1.0385645321199127e-02, 
       8.9447549901228527e-03, 
       9.8721764024740904e-03, 
       7.5468973167172942e-03, 
       1.0228075128839329e-02, 
       7.4309410898846752e-03, 
       9.8924765093901714e-03, 
       9.9000000000004119e-03, 
       7.9337432731794322e-03, 
       7.9027973659683889e-03, 
       9.6334059574371897e-03, 
       7.0847542530796210e-03, 
       7.8280646746504319e-03, 
       7.8217834220058909e-03, 
       7.0910998915362953e-03, 
       8.1043023534402066e-03, 
       7.8038869586495889e-03, 
       8.0750077938751223e-03, 
       9.2199999999999609e-03, 
       9.1606472767959708e-03, 
       6.6954745217432604e-03, 
       8.9821635758456746e-03, 
       7.8523927501902083e-03, 
       6.4220634715684011e-03, 
       7.8313950900798859e-03, 
       8.9014344912121731e-03, 
       8.9000068391243536e-03, 
       8.5299999999998259e-03, 
       8.3866412217897205e-03, 
       8.3238695041319231e-03, 
       8.2554262942135127e-03, 
       8.0143500657427567e-03, 
       7.8460202885916406e-03, 
       7.8400000000001090e-03, 
       7.6035786341608572e-03, 
       7.0197045353720296e-03, 
       7.5771555622992813e-03, 
       5.3963327063935194e-03, 
       7.5537592241843249e-03, 
       7.5069594686505061e-03, 
       7.4500000000004094e-03, 
       5.3880699294922060e-03, 
       5.4443186877411376e-03, 
       5.3833397695102272e-03, 
       5.3484592960283388e-03, 
       6.1367549131050147e-03, 
       7.2419808324227499e-03, 
       5.2112265103333982e-03, 
       7.2048353619812051e-03, 
       6.9178847748125119e-03, 
       7.1154678341725881e-03, 
       5.9063015285011448e-03, 
       6.8038524220089183e-03, 
       6.2509246775152138e-03, 
       6.8467038945936457e-03, 
       5.3112617387627362e-03, 
       5.3861480837971163e-03, 
       5.3792584964366874e-03, 
       5.8860542738467178e-03, 
       5.5859951427428105e-03, 
       5.0496371004821365e-03, 
       6.1615254724534645e-03, 
       5.5429396430689663e-03, 
       6.6406942522889484e-03, 
       5.0589134875469021e-03, 
       5.7711855533176745e-03, 
       6.5825815219745839e-03, 
       6.5815552428157469e-03, 
       5.0174940503949000e-03, 
       6.5799999999996200e-03, 
       5.3635290751001671e-03, 
       6.4415138894880790e-03, 
       6.3362273685503281e-03, 
       6.3302268361631314e-03, 
       6.2864230363762650e-03, 
       6.3074606552462114e-03, 
       4.5824775510914083e-03, 
       4.4359282230097977e-03, 
       5.4955163866318366e-03, 
       6.2099999999998849e-03, 
       4.6255111857770159e-03, 
       4.4819112006673498e-03, 
       6.1658814560769425e-03, 
       4.6040827860291999e-03, 
       4.2370131485177055e-03, 
       4.3134331035689791e-03, 
       5.9170543583771211e-03, 
       4.2910891863148694e-03, 
       5.7855942534332004e-03, 
       5.6668757412662273e-03, 
       5.7920392111615072e-03, 
       4.2487815879904802e-03, 
       5.5904133977084467e-03, 
       5.7257183371995134e-03, 
       4.2114657748962715e-03, 
       4.6492154045220469e-03, 
       4.4081777084087875e-03, 
       4.1810932664565091e-03, 
       5.7635191029758292e-03, 
       4.1075472119086586e-03, 
       4.6469956228837811e-03, 
       5.4625161322060679e-03, 
       4.1097956526734069e-03, 
       4.9782480487089199e-03, 
       3.9637238746577947e-03, 
       4.0818046230106219e-03, 
       5.4960466610222847e-03, 
       5.5620163306775708e-03, 
       3.9819043236940192e-03, 
       4.3161791874148214e-03, 
       4.3155454777283383e-03, 
       5.5062924487427629e-03, 
       3.9872780819929588e-03, 
       3.8875552584753084e-03, 
       5.0499617861224861e-03, 
       4.5171121772086950e-03, 
       5.4093746395381514e-03, 
       5.3484391130435623e-03, 
       3.8173318692585850e-03, 
       4.3045132590095510e-03, 
       4.3424015952698422e-03, 
       4.3666534216189714e-03, 
       5.1829232166587308e-03, 
       4.8906088548139335e-03, 
       3.6515923709263720e-03, 
       4.1116293335541178e-03, 
       3.7254134160732790e-03, 
       3.8091123803841507e-03, 
       3.7651040996554436e-03, 
       5.0432776609903932e-03, 
       3.7504227753855845e-03, 
       4.8244938157068302e-03, 
       3.7736111653070537e-03, 
       4.8088695971662820e-03, 
       4.9427519540123695e-03, 
       3.6769611546644755e-03, 
       4.9121417251174514e-03, 
       4.8799999999997118e-03, 
       3.7784626460440748e-03, 
       4.6121131751077305e-03, 
       4.0297916523199589e-03, 
       3.5883354333237972e-03, 
       4.7156158548982575e-03, 
       3.9366995289245232e-03, 
       3.5056285480428392e-03, 
       3.9127003849842830e-03, 
       4.6866297829639180e-03, 
       3.6531887671828640e-03, 
       4.5841899738456376e-03, 
       3.8416666042538276e-03, 
       4.5453993319281216e-03, 
       3.4371992621797759e-03, 
       3.3805868423897783e-03, 
       4.4333298224658799e-03, 
       3.9125737098600535e-03, 
       3.4805018894940466e-03, 
       3.2076279217153705e-03, 
       3.2028848864407017e-03, 
       4.2950558476862995e-03, 
       4.3599210214565320e-03, 
       3.2957239771295101e-03, 
       4.4638922879847737e-03, 
       4.2354210699997227e-03, 
       4.4947521691785551e-03, 
       3.4970537049163913e-03, 
       3.8671268788900521e-03, 
       4.4700000000000685e-03, 
       4.3777607920648708e-03, 
       4.3903675415080225e-03, 
       3.2389415376227124e-03, 
       3.3368172477021346e-03, 
       4.3281005184580557e-03, 
       3.2084699062838325e-03, 
       3.3178915101349550e-03, 
       3.4118294204911860e-03, 
       4.3287152005234393e-03, 
       3.5722662977477376e-03, 
       3.3687689836534086e-03, 
       3.3678563427398788e-03, 
       3.5262145219161077e-03, 
       3.4411445239224624e-03, 
       3.4372510175075936e-03, 
       3.6041231946871712e-03, 
       4.2400000001636302e-03, 
       4.2202452653535586e-03, 
       4.2117585164912598e-03, 
       4.2093777774100255e-03, 
       3.2064187120530011e-03, 
       3.1875423203699899e-03, 
       4.0119287956154916e-03, 
       3.4036238702117136e-03, 
       3.4209094898837791e-03, 
       3.1124289412179456e-03, 
       3.0055638819827162e-03, 
       3.0055638819827162e-03, 
       3.8626740227089480e-03, 
       2.9779065697888045e-03, 
       3.0294852761129521e-03, 
       3.4667072239767771e-03, 
       3.9996029064628411e-03, 
       2.9694821568150291e-03, 
       3.9106160851391843e-03, 
       2.9688175123321683e-03, 
       3.2121142719604987e-03, 
       3.8900000000001043e-03, 
       2.7523136147068927e-03, 
       2.8106285979094146e-03, 
       3.2557814197086287e-03, 
       3.0289595430954834e-03, 
       2.7483477625341091e-03, 
       2.9814666928773348e-03, 
       3.1335110018929106e-03, 
       3.7715923510223334e-03, 
       2.8277297186310407e-03, 
       3.7914955718626915e-03, 
       3.0181274720775756e-03, 
       3.6996905317986128e-03, 
       3.5666021445065988e-03, 
       3.3491668679870891e-03, 
       3.5388143473619677e-03, 
       2.8658636879760704e-03, 
       2.6221132548375804e-03, 
       3.3102662828063552e-03, 
       2.6670845037548921e-03, 
       2.8020772501898619e-03, 
       3.1734406936635238e-03, 
       3.4938930920938834e-03, 
       3.4976477329445653e-03, 
       3.4082970421393552e-03, 
       2.8598412722383088e-03, 
       3.4300000000004158e-03, 
       2.7315031883881134e-03, 
       3.1450006687706446e-03, 
       2.5041051908106116e-03, 
       3.1005774801966205e-03, 
       3.3384918908145240e-03, 
       3.0705489398676626e-03, 
       2.4832677614024731e-03, 
       2.4062815559438878e-03, 
       3.2719395793966849e-03, 
       2.4243255974249586e-03, 
       3.2082153419301725e-03, 
       2.9140079914865480e-03, 
       2.2706563737608897e-03, 
       3.1104683701458666e-03, 
       2.3096002490656190e-03, 
       2.6400144241326396e-03, 
       2.2663972870403848e-03, 
       3.0922271462426737e-03, 
       3.0292617223108271e-03, 
       2.2128740519034669e-03, 
       2.5039862098161759e-03, 
       3.0274666181308537e-03, 
       3.0541503475918848e-03, 
       3.0623144167858940e-03, 
       2.2745434560819309e-03, 
       2.1513223463192977e-03, 
       2.9667488016870552e-03, 
       2.9920503942038371e-03, 
       2.8562228542088875e-03, 
       2.1362725675902424e-03, 
       2.9899999999996639e-03, 
       2.9414767157264878e-03, 
       2.8514042762181175e-03, 
       2.9652212180623001e-03, 
       2.9585189420428515e-03, 
       2.9064433346914788e-03, 
       2.6680471429555669e-03, 
       2.5198492114006524e-03, 
       2.8863713896311754e-03, 
       2.1357250359881211e-03, 
       2.3560073514574597e-03, 
       2.2560524458250944e-03, 
       2.8242902410177025e-03, 
       2.3538172788601106e-03, 
       2.5474756177860983e-03, 
       2.1725687216684821e-03, 
       2.9061613171955333e-03, 
       2.4910328428001094e-03, 
       2.2534800678002138e-03, 
       2.8865196520503136e-03, 
       2.8096731769514170e-03, 
       2.7860226920468720e-03, 
       2.2568429423960655e-03, 
       2.3114190410703402e-03, 
       2.0305779135085448e-03, 
       2.8099999999998187e-03, 
       2.7841626987549551e-03, 
       2.5692091946682197e-03, 
       2.7899999999998009e-03, 
       2.0123825172653036e-03, 
       2.7631406829313693e-03, 
       2.7654621648997182e-03, 
       2.6724362488201574e-03, 
       2.2442370808073426e-03, 
       1.9631926935946313e-03, 
       1.9608628710692938e-03, 
       2.7305994929778580e-03, 
       2.0790612358289945e-03, 
       2.6966957366308859e-03, 
       2.7028319935185172e-03, 
       2.6195159587763133e-03, 
       2.6551549685462027e-03, 
       2.0270615562351870e-03, 
       2.2001862380952907e-03, 
       1.9981611910333649e-03, 
       2.2687790989754431e-03, 
       2.2128249336333325e-03, 
       2.6200374500365171e-03, 
       2.1000000000001842e-03, 
       1.9134298735014564e-03, 
       2.1375312343416600e-03, 
       2.5364043928730339e-03, 
       1.8693458641944184e-03, 
       2.0585907274822172e-03, 
       1.8949490969125680e-03, 
       2.0567202553391970e-03, 
       2.1416533717555156e-03, 
       2.5699995960119042e-03, 
       2.5680741086709221e-03, 
       2.1234642455712663e-03, 
       2.0585855043685724e-03, 
       2.0537412389877471e-03, 
       1.8052342581872638e-03, 
       1.9299999999997975e-03, 
       2.3896474478606036e-03, 
       2.5006310868285067e-03, 
       1.9634220820486077e-03, 
       1.8408973855317944e-03, 
       1.9376484829262488e-03, 
       1.8372194575687188e-03, 
       2.4468998966982384e-03, 
       2.4230211567243744e-03, 
       1.7334733112882410e-03, 
       1.7608770532946652e-03, 
       1.7515430398928440e-03, 
       1.7871859027667595e-03, 
       2.1708831375069487e-03, 
       1.7077451812340990e-03, 
       1.8007437248079537e-03, 
       1.6560345365226948e-03, 
       1.7261961392716785e-03, 
       2.3234941511659150e-03, 
       2.2133783237660554e-03, 
       1.7989091496158835e-03, 
       1.9154810054940661e-03, 
       2.2315818883796960e-03, 
       1.6539828710631730e-03, 
       2.1273066988111464e-03, 
       1.5998309394343266e-03, 
       1.9414251018069419e-03, 
       1.8278968770219135e-03, 
       1.8276512878618292e-03, 
       1.6393303833125356e-03, 
       2.2125536357380394e-03, 
       2.2300000000000054e-03, 
       2.1220642339331752e-03, 
       2.0663953537167928e-03, 
       2.2000000000004919e-03, 
       2.0912839952728327e-03, 
       1.6798213788333681e-03, 
       1.6785349101515520e-03, 
       1.8310716607564462e-03, 
       2.1899999999998392e-03, 
       2.1183933999970057e-03, 
       2.1812488520033288e-03, 
       2.1799999999995035e-03, 
       2.0177493489599331e-03, 
       1.6196554554146635e-03, 
       1.5759829805236554e-03, 
       1.7559203792237360e-03, 
       2.0643782094814430e-03, 
       1.5994351503272079e-03, 
       1.5130640838427697e-03, 
       1.8269694170747201e-03, 
       2.0302060041770217e-03, 
       1.9012298852765380e-03, 
       1.5775472687580702e-03, 
       1.7440441556429451e-03, 
       1.8406261146662166e-03, 
       2.0422796826856989e-03, 
       1.5577534887345889e-03, 
       1.5228601278934530e-03, 
       2.0771035049388397e-03, 
       1.5866548732582132e-03, 
       2.0716967087883731e-03, 
       1.9180561983708714e-03, 
       1.5740952178952352e-03, 
       2.0513139621791011e-03, 
       1.9573459473061004e-03, 
       1.8634200241438696e-03, 
       2.0251656955611358e-03, 
       1.9979317998758054e-03, 
       1.9887350858284738e-03, 
       1.9372602716283358e-03, 
       1.4840360517477664e-03, 
       1.9455046277369781e-03, 
       1.4822101348284575e-03, 
       1.4053910786393886e-03, 
       1.9395823817937978e-03, 
       1.9177600636645495e-03, 
       1.5264015753866627e-03, 
       1.9603758840579643e-03, 
       1.9417499485921484e-03, 
       1.4511862031092969e-03, 
       1.8646742637309633e-03, 
       1.9252181815478510e-03, 
       1.3599999999999604e-03, 
       1.9119292767271641e-03, 
       1.8961110713132993e-03, 
       1.5333158329601207e-03, 
       1.4139230297736539e-03, 
       1.4137389044732742e-03, 
       1.3570103165572672e-03, 
       1.3395039043591094e-03, 
       1.4004387273964859e-03, 
       1.8648709830104543e-03, 
       1.5130971212553176e-03, 
       1.5393815011184632e-03, 
       1.4975052381680644e-03, 
       1.8100000000000770e-03, 
       1.8043253317345553e-03, 
       1.3279191165339382e-03, 
       1.7777360197437973e-03, 
       1.2722045352822973e-03, 
       1.2714458707312442e-03, 
       1.7700000000002776e-03, 
       1.2710032749901944e-03, 
       1.5135480540596940e-03, 
       1.7600000000000896e-03, 
       1.7511916513083719e-03, 
       1.5481382404551379e-03, 
       1.2975004898291829e-03, 
       1.7277585702093344e-03, 
       1.2501574469829507e-03, 
       1.2231019609888045e-03, 
       1.7243741281408810e-03, 
       1.6308845089451603e-03, 
       1.2793440737035740e-03, 
       1.5006048138909867e-03, 
       1.5005378897688807e-03, 
       1.7069255032667000e-03, 
       1.3620550508441410e-03, 
       1.6230642962504198e-03, 
       1.4553063554247243e-03, 
       1.6218995245217852e-03, 
       1.4552063322303874e-03, 
       1.6789555159447637e-03, 
       1.4449559701513368e-03, 
       1.2659318488345850e-03, 
       1.5209138572461723e-03, 
       1.5990350323246752e-03, 
       1.6098241486649750e-03, 
       1.5757189329287491e-03, 
       1.2033138231236421e-03, 
       1.1857708139754795e-03, 
       1.1895723261845439e-03, 
       1.5659897314885961e-03, 
       1.1526345228659893e-03, 
       1.3354002368328906e-03, 
       1.2924142812332251e-03, 
       1.5320719587495141e-03, 
       1.2910282445561675e-03, 
       1.6099999999996226e-03, 
       1.2201642050523411e-03, 
       1.5501524665558076e-03, 
       1.3514172345681704e-03, 
       1.1830294696117383e-03, 
       1.1607915875320699e-03, 
       1.1807697503357464e-03, 
       1.5046698143431768e-03, 
       1.1985413357551227e-03, 
       1.1984574962452439e-03, 
       1.2778856906560383e-03, 
       1.3238302111596436e-03, 
       1.2948415778532567e-03, 
       1.5231039169712336e-03, 
       1.5013362254606114e-03, 
       1.5387958732749451e-03, 
       1.1507676226198125e-03, 
       1.4228335250778086e-03, 
       1.0850858623308514e-03, 
       1.5200000000004012e-03, 
       1.5200000000004012e-03, 
       1.1143897348160124e-03, 
       1.2452860537254702e-03, 
       1.2710603182758938e-03, 
       1.2432739331407364e-03, 
       1.4899999999996333e-03, 
       1.2741457912882313e-03, 
       1.3269646348660563e-03, 
       1.0397532081773630e-03, 
       1.2781405635310262e-03, 
       1.4496656523910268e-03, 
       1.0527540996661140e-03, 
       1.3203696992965630e-03, 
       1.4070013779843304e-03, 
       1.3799659491676719e-03, 
       1.0165041437892341e-03, 
       1.0196281928890548e-03, 
       1.3680492913381859e-03, 
       1.0402369816443502e-03, 
       1.4147265674069212e-03, 
       1.4100171484122115e-03, 
       1.0908604293445509e-03, 
       1.3618890119170967e-03, 
       1.3237732202611872e-03, 
       9.9708221566243826e-04, 
       1.1611524198720947e-03, 
       1.1629517505211317e-03, 
       1.3438264426949174e-03, 
       1.1506131523817075e-03, 
       1.0057892056631562e-03, 
       1.1423111598481066e-03, 
       1.2599999999997630e-03, 
       1.3703270139586788e-03, 
       1.3577085754456367e-03, 
       1.0625687959321286e-03, 
       1.3447875739703355e-03, 
       1.1175452366220758e-03, 
       1.3211516130545097e-03, 
       1.1048034257136953e-03, 
       9.9423476377358136e-04, 
       1.1092430170577703e-03, 
       1.3250055223884415e-03, 
       1.1821189543462284e-03, 
       1.2202607426113781e-03, 
       1.3134405230703027e-03, 
       1.2982985229270032e-03, 
       1.2688784810029625e-03, 
       1.3019455977390916e-03, 
       1.0601239182870051e-03, 
       9.1680187664364371e-04, 
       1.2323539662452863e-03, 
       1.2832090815055569e-03, 
       1.0020648431733285e-03, 
       1.0722218072626629e-03, 
       9.7769795959281973e-04, 
       8.9936423896613100e-04, 
       9.2014616645642617e-04, 
       1.0602129138901862e-03, 
       1.0819653372863402e-03, 
       9.3634191499031836e-04, 
       1.2410215165679821e-03, 
       1.2399984290036193e-03, 
       1.0392305152043199e-03, 
       9.6969874816485880e-04, 
       9.1921728217785644e-04, 
       1.2272233412865226e-03, 
       1.2241823772437739e-03, 
       8.8811818233925440e-04, 
       9.1717303670487262e-04, 
       1.2029970299853347e-03, 
       1.1801869607692937e-03, 
       9.1314607487902766e-04, 
       9.2444165512776857e-04, 
       8.5920427200963055e-04, 
       8.7159747325664698e-04, 
       1.1917882632199806e-03, 
       9.6229157780246038e-04, 
       1.1477111910238483e-03, 
       1.1000000000000246e-03, 
       8.8230807141904586e-04, 
       1.0999999999997865e-03, 
       8.4562716509719638e-04, 
       8.6410583733641840e-04, 
       8.5829866982418425e-04, 
       9.8335366835765911e-04, 
       1.0258602675832567e-03, 
       8.5179158535410640e-04, 
       8.3986982354248001e-04, 
       9.4607544338334263e-04, 
       8.6566528239446919e-04, 
       8.4662067118532593e-04, 
       8.5316568121794308e-04, 
       1.1142237098212653e-03, 
       1.0881081634943864e-03, 
       1.0699961414163866e-03, 
       8.4729119898882424e-04, 
       1.1172161898206866e-03, 
       8.4348518982079671e-04, 
       8.6025065839148165e-04, 
       9.3649111108827694e-04, 
       9.3617426305743208e-04, 
       8.9701770478238247e-04, 
       1.1496347130973616e-03, 
       1.1500000000002513e-03, 
       1.1119848850061013e-03, 
       1.0992229172782368e-03, 
       9.3325783394147951e-04, 
       8.4192938444093465e-04, 
       1.0313229985191754e-03, 
       8.2884973030957789e-04, 
       8.3763246474184065e-04, 
       8.3763246474184065e-04, 
       9.6491045870014864e-04, 
       1.0321189515890520e-03, 
       1.0803740293975942e-03, 
       1.1124099740827261e-03, 
       8.3070031739222272e-04, 
       8.1966401955703536e-04, 
       8.9994174840728636e-04, 
       7.9371954109791281e-04, 
       8.8853370249132406e-04, 
       1.0659978474763878e-03, 
       1.0868974802450738e-03, 
       1.0966906158355978e-03, 
       8.8227759143595034e-04, 
       8.8144581068127093e-04, 
       7.8103048611061860e-04, 
       9.7010164581648592e-04, 
       1.0900003230755434e-03, 
       8.0304729628508965e-04, 
       8.9979417599935483e-04, 
       7.8262724349118862e-04, 
       1.0899999999997277e-03, 
       9.0170596842303983e-04, 
       8.2370871341711589e-04, 
       7.7092076805772823e-04, 
       8.4680452763058413e-04, 
       1.0418131549458239e-03, 
       8.6096820499749396e-04, 
       7.9928493053154238e-04, 
       9.1481837272593689e-04, 
       7.7332843519254009e-04, 
       8.8726698946568685e-04, 
       7.5719046098634863e-04, 
       8.7754050135777485e-04, 
       8.0193368655592518e-04, 
       1.0266668393735755e-03, 
       9.8632732892494799e-04, 
       9.7489709987612469e-04, 
       7.3379979969195798e-04, 
       7.9626124886624764e-04, 
       7.2862290150293193e-04, 
       8.1306119499879859e-04, 
       9.5021846561452715e-04, 
       1.0078398485438907e-03, 
       8.3309609297655242e-04, 
       9.5834562391903578e-04, 
       7.3007214635110573e-04, 
       7.0519655650750919e-04, 
       7.1384934407070207e-04, 
       9.9287919540578281e-04, 
       9.9169007084090512e-04, 
       8.3106697366481532e-04, 
       8.2282018600784458e-04, 
       9.7416286484639414e-04, 
       9.0999999999995577e-04, 
       9.6148998310662437e-04, 
       9.8041500425687404e-04, 
       9.6516270624856537e-04, 
       9.4035122255820252e-04, 
       9.7890469347745217e-04, 
       8.3192973548312104e-04, 
       9.4260168843323094e-04, 
       7.1437807437092760e-04, 
       7.1005488858935444e-04, 
       8.3675398153775361e-04, 
       7.4452602308348523e-04, 
       9.5999999999995297e-04, 
       9.5082972283379601e-04, 
       8.5799678760900414e-04, 
       9.5726088493600916e-04, 
       6.9742922299746506e-04, 
       9.2532078484820095e-04, 
       7.9345441572660231e-04, 
       9.4971014607161757e-04, 
       8.1159886064028069e-04, 
       9.5106189115032162e-04, 
       6.8143898598677869e-04, 
       9.0387209606499809e-04, 
       6.6464242232223091e-04, 
       7.4444840593396449e-04, 
       9.3078453845091963e-04, 
       6.8297381251342137e-04, 
       7.0536040326004551e-04, 
       8.1418525059597326e-04, 
       7.6629647553879137e-04, 
       6.8000896972691765e-04, 
       8.1305782721097263e-04, 
       7.0319267093032285e-04, 
       7.4835150677836490e-04, 
       7.1930848318228313e-04, 
       8.0034526842961344e-04, 
       6.6032960458127445e-04, 
       8.6726558610639333e-04, 
       6.9963057984986042e-04, 
       8.9471979726288362e-04, 
       7.3621238822671052e-04, 
       6.5109456331396260e-04, 
       9.0592824627149209e-04, 
       7.6210360955107886e-04, 
       6.8426355060787218e-04, 
       6.4699182335023096e-04, 
       6.5146688099540831e-04, 
       8.8786327978900112e-04, 
       7.9719117905839403e-04, 
       7.1101150176139806e-04, 
       6.4133572312437639e-04, 
       7.3841566196087270e-04, 
       6.4945020128667893e-04, 
       6.2782579577528739e-04, 
       6.3970131835345103e-04, 
       8.4176252244299947e-04, 
       6.3705554263248981e-04, 
       6.5286150446976189e-04, 
       8.4489184720920618e-04, 
       7.8062600162712690e-04, 
       8.1963905686600403e-04, 
       7.5136634889083869e-04, 
       7.4931236112842743e-04, 
       7.8940268946368344e-04, 
       6.1231694115196768e-04, 
       7.0958213374742215e-04, 
       7.8096702757666180e-04, 
       6.2987478580222775e-04, 
       8.5542456851813537e-04, 
       8.1547538167852773e-04, 
       6.6689473573856818e-04, 
       6.1550731857013994e-04, 
       6.2002415632235117e-04, 
       8.4807525814775914e-04, 
       7.9063518007592253e-04, 
       7.9636440318691824e-04, 
       8.5031666702191562e-04, 
       8.2869915565432300e-04, 
       6.8170403543912894e-04, 
       6.9631788551563153e-04, 
       6.2016289473829881e-04, 
       8.4000000000000665e-04, 
       8.3999999999998778e-04, 
       8.0292117711621422e-04, 
       5.8726803111830211e-04, 
       8.0353128786761843e-04, 
       7.5999999999998736e-04, 
       7.8413254753069927e-04, 
       6.4687160412571249e-04, 
       6.0597647539770974e-04, 
       7.8333655693941953e-04, 
       8.1825577207246211e-04, 
       5.9697450684105568e-04, 
       7.5749622246079986e-04, 
       5.8849423821111507e-04, 
       5.9756269924659405e-04, 
       7.8968921509867411e-04, 
       6.7411270063678886e-04, 
       5.7350505197181594e-04, 
       7.1869247980781864e-04, 
       5.9463187431608598e-04, 
       8.0301448588421622e-04, 
       5.9067863574418231e-04, 
       7.9958895513287779e-04, 
       5.7517880114199061e-04, 
       6.0860272298428949e-04, 
       6.0777498255043343e-04, 
       7.6061636240533706e-04, 
       5.6039561055042629e-04, 
       7.6485825086788114e-04, 
       7.9000000000004165e-04, 
       7.2825040323008783e-04, 
       5.7955568712952238e-04, 
       5.5779998065632859e-04, 
       6.7153444153707513e-04, 
       6.9477772938393795e-04, 
       5.6028827356726621e-04, 
       7.8121600020740207e-04, 
       7.5476246659545343e-04, 
       6.3853383528271143e-04, 
       5.7611286398155052e-04, 
       7.7999999999996085e-04, 
       5.7183426317650404e-04, 
       5.7318635216712733e-04, 
       6.4773453100489854e-04, 
       5.7066764260964157e-04, 
       6.4807423646908563e-04, 
       6.8591754070165356e-04, 
       5.7049608631716715e-04, 
       5.5889246672149236e-04, 
       5.4584649581698058e-04, 
       6.9974009997874969e-04, 
       6.5020042223484049e-04, 
       5.6405233343490783e-04, 
       5.5388267268236359e-04, 
       7.1248712770259875e-04, 
       6.1907071990519849e-04, 
       7.6000000000000015e-04, 
       7.5535647685746421e-04, 
       7.2355768561197344e-04, 
       5.4461785348019652e-04, 
       5.8348501595818379e-04, 
       6.1944140208039327e-04, 
       5.6602657899486601e-04, 
       7.4913743798783554e-04, 
       7.4999999999999394e-04, 
       7.4999999999999405e-04, 
       5.7766835175624212e-04, 
       7.3849396037308889e-04, 
       7.3000000000002078e-04, 
       6.2777617237240284e-04, 
       6.0455692124723390e-04, 
       6.0429726330500221e-04, 
       7.4367005224911865e-04, 
       6.2689788947778837e-04, 
       7.1518599181241910e-04, 
       5.5557069138651609e-04, 
       7.1227126133623574e-04, 
       6.5890293530411720e-04, 
       6.4161608249076933e-04, 
       5.8765383637214281e-04, 
       6.0555485606131039e-04, 
       5.1885242865576451e-04, 
       5.2914866184207367e-04, 
       5.7413744753413622e-04, 
       5.7377568222508491e-04, 
       5.1788343871384348e-04, 
       7.3095216607443143e-04, 
       7.2878611614416626e-04, 
       5.8923655852713278e-04, 
       5.3492812220260312e-04, 
       7.3000000000003531e-04, 
       6.7002172573886866e-04, 
       5.8882327818308739e-04, 
       5.1454012940534729e-04, 
       5.7784386895701374e-04, 
       5.1124765074919037e-04, 
       7.0750750422095136e-04, 
       7.0631186050202721e-04, 
       5.9863584523184093e-04, 
       7.1554468406279657e-04, 
       6.5999999999998146e-04, 
       5.2377091126056886e-04, 
       5.1198725383510917e-04, 
       7.1396186768038657e-04, 
       7.1343680159700374e-04, 
       5.1252070393127084e-04, 
       5.0446902630530538e-04, 
       5.6320744796654236e-04, 
       5.9798010233506644e-04, 
       4.9753264466863018e-04, 
       5.7762393013298017e-04, 
       5.7758461167172490e-04, 
       6.1597317619391245e-04, 
       5.1639429336711824e-04, 
       5.1639429336711824e-04, 
       6.9928221968222463e-04, 
       5.5803830287435576e-04, 
       4.9868676331598066e-04, 
       5.8237504517693705e-04, 
       5.7713007783281325e-04, 
       6.6485618626471125e-04, 
       6.8888674548972430e-04, 
       6.8704295240417605e-04, 
       5.6187501299076470e-04, 
       5.6116040505326448e-04, 
       6.9168151476712420e-04, 
       5.7483773256884589e-04, 
       4.9123135415785329e-04, 
       6.8999999999998045e-04, 
       6.0861889975609483e-04, 
       6.6601010436314798e-04, 
       5.0120526841662195e-04, 
       5.1192173189655349e-04, 
       4.9102548183979763e-04, 
       5.0471476540238217e-04, 
       4.8445200915290864e-04, 
       5.5002539033249245e-04, 
       5.4876511776007794e-04, 
       6.8000000409002036e-04, 
       5.8076654086288283e-04, 
       5.5639579232833380e-04, 
       5.5632873757778512e-04, 
       6.7605136518107992e-04, 
       5.7992589178201782e-04, 
       6.6548994786067614e-04, 
       6.6097188273867903e-04, 
       4.7897528438647268e-04, 
       4.7376289341636576e-04, 
       5.1683067703012979e-04, 
       5.0677247425818389e-04, 
       4.7256242075504126e-04, 
       5.8280935025750041e-04, 
       5.3598416080429261e-04, 
       4.8005546006144967e-04, 
       6.6000021336748481e-04, 
       5.4811967195184997e-04, 
       6.4755755460146544e-04, 
       5.4138073163812991e-04, 
       5.4663788512695577e-04, 
       6.5587104953685740e-04, 
       6.3131468875082407e-04, 
       4.6479039155858338e-04, 
       6.3277156072041462e-04, 
       6.4763937751077360e-04, 
       4.6456708958959148e-04, 
       5.4675348127092792e-04, 
       4.7370325236730887e-04, 
       5.6028827356739523e-04, 
       5.2218554397120336e-04, 
       4.9065836201968836e-04, 
       4.9669724742721310e-04, 
       4.5750680310662162e-04, 
       5.5261406438353669e-04, 
       4.7933561122234618e-04, 
       5.8606610428883613e-04, 
       4.5597973855156156e-04, 
       6.2940995487041867e-04, 
       6.4245591692016978e-04, 
       6.4236232249980057e-04, 
       6.1985075881894997e-04, 
       5.7228735507775560e-04, 
       6.3999999999999452e-04, 
       4.8630465915939619e-04, 
       4.5989956792110288e-04, 
       5.6926391790780714e-04, 
       6.0933454307930549e-04, 
       4.6662505685575895e-04, 
       5.1401343777502111e-04, 
       4.6031086945726597e-04, 
       6.2449176652015239e-04, 
       6.3000000000002637e-04, 
       4.7789901572731935e-04, 
       4.6706012290130577e-04, 
       5.1882404328284083e-04, 
       4.5617637121493594e-04, 
       6.0424568821553816e-04, 
       6.1999942927723570e-04, 
       5.1583008948361941e-04, 
       6.1566879345417388e-04, 
       5.9460486378112105e-04, 
       4.7179834447934246e-04, 
       4.4182186032471084e-04, 
       4.3640853697096227e-04, 
       6.1379862328916391e-04, 
       4.5565528191092702e-04, 
       4.5487145592387935e-04, 
       4.6492028120825217e-04, 
       6.0725188605008124e-04, 
       4.6598332022192228e-04, 
       5.8962396790333349e-04, 
       6.0583879341166171e-04, 
       5.6282583822521003e-04, 
       5.9694940412260912e-04, 
       5.3619493593579787e-04, 
       4.9963566885522870e-04, 
       4.4103855290032143e-04, 
       5.9899400302895159e-04, 
       5.9742024194650903e-04, 
       5.9765595813331943e-04, 
       5.6070491972700579e-04, 
       4.7094964690610954e-04, 
       4.2716419877300034e-04, 
       4.8991233588122161e-04, 
       4.4075059548504480e-04, 
       5.8990358144745455e-04, 
       5.9000769201299609e-04, 
       5.9000000000004296e-04, 
       5.8999999999998095e-04, 
       5.8750013155814754e-04, 
       4.2016663265200097e-04, 
       4.9777362164916408e-04, 
       4.3032297541430907e-04, 
       4.2952916163121948e-04, 
       5.1794020212985034e-04, 
       4.3579888208547052e-04, 
       5.6036838985983333e-04, 
       4.0987138414556027e-04, 
       5.7666811113706300e-04, 
       5.6658768265591922e-04, 
       4.8698028931076873e-04, 
       4.7252152845787777e-04, 
       4.7236002888165614e-04, 
       4.2728808526085247e-04, 
       5.7265370879162425e-04, 
       5.5091321729471216e-04, 
       4.5926026086021219e-04, 
       5.7000000000000193e-04, 
       4.0867473223185158e-04, 
       4.1533075832266724e-04, 
       4.1689211916132987e-04, 
       4.0242990893899698e-04, 
       4.8828480585771917e-04, 
       4.2105596823818205e-04, 
       4.2168513633840789e-04, 
       5.2644630726734760e-04, 
       4.1769971011662414e-04, 
       5.4641938589369008e-04, 
       5.6009993013137404e-04, 
       4.3224942094529642e-04, 
       5.5999999999996515e-04, 
       4.1634491693792969e-04, 
       4.2999999999997641e-04, 
       5.3672971698170513e-04, 
       5.5338564409599040e-04, 
       5.0138912481284689e-04, 
       4.2527198322579978e-04, 
       5.5174014649771890e-04, 
       4.2160288368167480e-04, 
       4.1164869182218697e-04, 
       4.0515553234701465e-04, 
       4.1388888787201891e-04, 
       4.5879653316744994e-04, 
       4.1481905706604300e-04, 
       5.2226707695044578e-04, 
       5.3765684742377846e-04, 
       4.6251441454208276e-04, 
       4.0229676852138288e-04, 
       4.5132452315943972e-04, 
       4.9788235936933740e-04, 
       4.8404217926740605e-04, 
       4.3903711165634252e-04, 
       5.3107191607676125e-04, 
       5.3107191607676125e-04, 
       3.8886655132896715e-04, 
       4.4238738735252005e-04, 
       3.8785809892244742e-04, 
       4.5550219880348197e-04, 
       4.0799149657821883e-04, 
       5.1336441495056738e-04, 
       5.1946551137508213e-04, 
       4.0873861456086315e-04, 
       4.6825200439304545e-04, 
       4.5441886082673091e-04, 
       4.5433428521862327e-04, 
       4.0593087721937189e-04, 
       3.6937335423199740e-04, 
       4.3334119392976397e-04, 
       3.6855492601527227e-04, 
       3.9069878356694503e-04, 
       5.1310381798813501e-04, 
       4.8153322718350451e-04, 
       3.8523209819074509e-04, 
       4.9040818017261696e-04, 
       4.6989012326654486e-04, 
       4.0658815994708161e-04, 
       4.2850258282290993e-04, 
       3.8326422332357320e-04, 
       5.1165532873487848e-04, 
       4.3178371561377243e-04, 
       4.2354719479179568e-04, 
       5.1043313063264633e-04, 
       3.9072991288166282e-04, 
       5.0991904739426934e-04, 
       4.9443737384068695e-04, 
       3.8741334224277321e-04, 
       4.8224672578947938e-04, 
       4.2254070924172227e-04, 
       3.6652951263539706e-04, 
       4.1869461908878561e-04, 
       4.2010333030759959e-04, 
       3.7842677984806282e-04, 
       4.2793159450751759e-04, 
       3.7384841866869149e-04, 
       5.0087295555180009e-04, 
       4.1506283130912446e-04, 
       4.3339550700365780e-04, 
       4.2828450027069620e-04, 
       4.1566856155783350e-04, 
       3.6569451786849755e-04, 
       4.1245706510823686e-04, 
       4.8365668460789170e-04, 
       4.8763961348729888e-04, 
       4.1943463645608575e-04, 
       3.6734593777867946e-04, 
       3.7262731260247862e-04, 
       3.7262731260247862e-04, 
       3.6874899091149615e-04, 
       4.3646020973984594e-04, 
       4.3588229819350929e-04, 
       3.6303598553427474e-04, 
       4.4811267843526773e-04, 
       4.9886767777191103e-04, 
       4.5999999999997101e-04, 
       4.9601343917257288e-04, 
       3.5139155083695493e-04, 
       4.0603320152325677e-04, 
       4.9455446785785751e-04, 
       3.7996153360287277e-04, 
       4.7663625051561504e-04, 
       3.5001949176853336e-04, 
       4.3069372788813801e-04, 
       4.4664562925356523e-04, 
       3.5283090806247167e-04, 
       4.8999976265301872e-04, 
       4.0761576635064929e-04, 
       4.8935244512509615e-04, 
       3.7849444071535164e-04, 
       4.8469669829598327e-04, 
       4.8999999999999998e-04, 
       4.8684562236870086e-04, 
       3.7113797634645313e-04, 
       3.6038581826182924e-04, 
       4.8827147481653817e-04, 
       4.0030607450085932e-04, 
       4.8772495020563014e-04, 
       4.8197160844513204e-04, 
       4.6978819288669892e-04, 
       4.2446731897975725e-04, 
       4.8352294217218273e-04, 
       4.6515044934806131e-04, 
       3.8931902425185210e-04, 
       3.5981522816320262e-04, 
       4.4461820065308690e-04, 
       3.8995976754890020e-04, 
       3.4814296503105877e-04, 
       3.4122566007805717e-04, 
       3.9826746735479240e-04, 
       4.1521056116211217e-04, 
       4.5666186737046194e-04, 
       4.5666186737046194e-04, 
       3.6246622320238104e-04, 
       3.4200668566398024e-04, 
       3.8293866857847908e-04, 
       4.6978274894565291e-04, 
       3.9476533793299766e-04, 
       3.8177242545776337e-04, 
       3.8249113504133813e-04, 
       4.3713346183631185e-04, 
       3.9325081961528937e-04, 
       4.1184358201791054e-04, 
       3.6999999999996666e-04, 
       4.7193868547978684e-04, 
       4.0051027361552951e-04, 
       4.1063003272274410e-04, 
       4.6952459630720628e-04, 
       4.7000000000000503e-04, 
       4.6999999999997510e-04, 
       4.6901904337739482e-04, 
       4.6428090886067663e-04, 
       4.5535334122565518e-04, 
       4.2577556802660078e-04, 
       3.4158622217863807e-04, 
       4.0190618443182463e-04, 
       3.3165270024157088e-04, 
       3.8138586772530674e-04, 
       4.1381515453920391e-04, 
       4.6426092928233037e-04, 
       3.6957750115449621e-04, 
       3.3061034570348673e-04, 
       3.4761384384529337e-04, 
       3.3156198088253819e-04, 
       4.1136417470181438e-04, 
       3.3801544789340129e-04, 
       3.8277054437841702e-04, 
       3.4329678344799250e-04, 
       4.5999999735529546e-04, 
       4.3048145482445504e-04, 
       3.2524149394451754e-04, 
       4.5473571808533008e-04, 
       4.5930848782244120e-04, 
       3.4559459210361539e-04, 
       3.3349557060676611e-04, 
       4.1566117929988164e-04, 
       3.7626153102920484e-04, 
       4.5569039618893211e-04, 
       3.2532986604872212e-04, 
       3.3406635752527723e-04, 
       3.7710413964717920e-04, 
       4.5069157397737884e-04, 
       3.6444066719910299e-04, 
       4.4999999999997348e-04, 
       4.2984431033180087e-04, 
       4.2773493880725875e-04, 
       3.3081030536378608e-04, 
       4.4647132167755454e-04, 
       3.2356918158221401e-04, 
       3.7022810554191305e-04, 
       3.2973924690423264e-04, 
       3.5659008730153883e-04, 
       3.9270692314014261e-04, 
       3.2938480582062068e-04, 
       3.6885752698371691e-04, 
       3.1367207213070221e-04, 
       3.2688136719308795e-04, 
       3.2252418742579735e-04, 
       3.1294648920414918e-04, 
       3.2309537020842850e-04, 
       3.1670261691948536e-04, 
       4.4000000000000381e-04, 
       4.3999999999998055e-04, 
       4.3907942601544806e-04, 
       4.2594248213401295e-04, 
       4.3654299510511332e-04, 
       3.6308844713053639e-04, 
       4.2416185430309018e-04, 
       3.0947662162855965e-04, 
       3.0950735195422720e-04, 
       3.1826480590831986e-04, 
       3.1838418274988045e-04, 
       3.6218176349486850e-04, 
       4.1117671007293559e-04, 
       3.0736467096239737e-04, 
       3.6682246917940430e-04, 
       3.2202045371549487e-04, 
       3.5922581566760845e-04, 
       3.5916264391864687e-04, 
       3.0606194614235619e-04, 
       4.2684327659405375e-04, 
       4.2822681050420756e-04, 
       3.5622833920347422e-04, 
       3.5680292094087551e-04, 
       3.7929041075784799e-04, 
       3.0372629093901500e-04, 
       4.0434069622467610e-04, 
       3.5779185101252070e-04, 
       3.4460902560420090e-04, 
       3.5307268890912507e-04, 
       3.2170637134458334e-04, 
       4.1326378551179702e-04, 
       3.0002948586576667e-04, 
       3.4001718709206555e-04, 
       3.8467289296057703e-04, 
       4.2042023952030600e-04, 
       3.4844840596656495e-04, 
       4.2000000000000376e-04, 
       4.1999999999997958e-04, 
       4.1864749408116895e-04, 
       3.4811448790028165e-04, 
       4.1766614137503578e-04, 
       3.0489530857289988e-04, 
       3.4309756172639050e-04, 
       4.0281912617011722e-04, 
       3.4239416185192210e-04, 
       3.4220735527200075e-04, 
       3.4640714418511818e-04, 
       3.0459326080784350e-04, 
       3.8974634285763763e-04, 
       3.4634319522724956e-04, 
       3.6491339987555213e-04, 
       4.1453926694554489e-04, 
       3.4331422600119367e-04, 
       3.9282497621314792e-04, 
       3.4186799779284795e-04, 
       4.1210285388468227e-04, 
       4.0908671516770573e-04, 
       3.0731549100465209e-04, 
       3.5469041310171040e-04, 
       4.0887919647896955e-04, 
       3.0385270701003718e-04, 
       3.8511402544468920e-04, 
       3.3839707440502082e-04, 
       2.9820659145179040e-04, 
       2.9867487234606612e-04, 
       3.0342171916419966e-04, 
       3.3370733704627531e-04, 
       3.6173396560894126e-04, 
       2.9830504098508735e-04, 
       4.0428877742219096e-04, 
       4.0330102795794299e-04, 
       3.4651517308395715e-04, 
       3.5960853835157900e-04, 
       3.2231102196190981e-04, 
       3.9709906075794413e-04, 
       3.5523311514918976e-04, 
       3.3513011328181475e-04, 
       3.7000000000002282e-04, 
       2.9259497846616727e-04, 
       2.9239459626627650e-04, 
       2.9895903899708026e-04, 
       3.9999999999995898e-04, 
       3.8078454708052282e-04, 
       2.8690939739396774e-04, 
       3.7712873187877743e-04, 
       3.1000000000004407e-04, 
       3.3105642649603411e-04, 
       2.8847303114726096e-04, 
       3.8292613410330643e-04, 
       3.2114564201895379e-04, 
       2.8304054321787848e-04, 
       3.8079938340616383e-04, 
       3.9556733408690028e-04, 
       3.9579459262805984e-04, 
       3.9401531826353423e-04, 
       2.8097462496759604e-04, 
       3.3454201279634819e-04, 
       3.3454201279634819e-04, 
       2.8636537383450530e-04, 
       2.8614038649050290e-04, 
       3.8927861915739583e-04, 
       2.9583121267734539e-04, 
       3.4207210171564964e-04, 
       3.2647467760697687e-04, 
       3.7717347844795824e-04, 
       2.8454452119998841e-04, 
       3.9198930105694026e-04, 
       2.7733118401955585e-04, 
       3.6255297270708115e-04, 
       3.2473542908114755e-04, 
       2.9135032342685611e-04, 
       3.8934249328830478e-04, 
       3.4058762887578928e-04, 
       2.7519732507388510e-04, 
       3.3853975374277317e-04, 
       3.4001363110565104e-04, 
       2.7663783533663676e-04, 
       3.4893649010062636e-04, 
       2.7761558501658752e-04, 
       3.4314164553020778e-04, 
       2.8463270593471812e-04, 
       3.1831153949116257e-04, 
       3.7064997439797807e-04, 
       3.8451057598346596e-04, 
       2.7354580058698785e-04, 
       3.7786337315421026e-04, 
       2.7581415183103091e-04, 
       3.5829630510651988e-04, 
       2.8775167995718321e-04, 
       2.7229498488724198e-04, 
       2.9280328532242341e-04, 
       3.8047913711606332e-04, 
       3.4999999999999626e-04, 
       3.7063631999494209e-04, 
       3.1496831514292591e-04, 
       3.3704411430660844e-04, 
       2.7511902160417938e-04, 
       3.1359975061424710e-04, 
       3.7290465536728395e-04, 
       3.7321382662523089e-04, 
       2.9227515573780020e-04, 
       3.7106323452044612e-04, 
       3.7555578329939412e-04, 
       2.8010517725244368e-04, 
       2.9424174037803459e-04, 
       3.2528964040418149e-04, 
       2.9419727736172549e-04, 
       3.6344135447651194e-04, 
       3.4979692310113463e-04, 
       3.1887808700253883e-04, 
       2.9804644876324770e-04, 
       3.5977332187777083e-04, 
       2.6866476615321352e-04, 
       2.7540586172352814e-04, 
       2.9466889504489182e-04, 
       3.1243323066047849e-04, 
       2.6585713707362566e-04, 
       3.2481216266130690e-04, 
       3.2885386870422730e-04, 
       3.6040920857010716e-04, 
       3.6624685236108826e-04, 
       3.5369938359066652e-04, 
       2.7730746375271659e-04, 
       3.1927194131035040e-04, 
       2.9695462787589515e-04, 
       2.6348264903191133e-04, 
       2.7200836951598662e-04, 
       3.5724989829834941e-04, 
       2.6363250823933189e-04, 
       2.7912684171211107e-04, 
       2.5967531295331318e-04, 
       3.6262410500142924e-04, 
       3.5865527641514705e-04, 
       2.6928662850448737e-04, 
       2.6948265789733956e-04, 
       3.1254680142837383e-04, 
       3.0018247406708417e-04, 
       3.6073891588606853e-04, 
       3.5127059277291126e-04, 
       2.9946441383487939e-04, 
       2.9497538747445405e-04, 
       2.6858477443360111e-04, 
       3.6000000000000908e-04, 
       2.9755950290098114e-04, 
       2.6189986106057338e-04, 
       3.0453449304052445e-04, 
       2.5320310620209992e-04, 
       2.8781457522357825e-04, 
       2.9707648588416354e-04, 
       3.3749802525536560e-04, 
       3.3225214191250600e-04, 
       3.5604005055667377e-04, 
       3.5372906690141930e-04, 
       3.3913692667461278e-04, 
       3.5243092878674791e-04, 
       2.9272006353230735e-04, 
       3.5099944546393186e-04, 
       3.3426161893637907e-04, 
       2.6913762139848813e-04, 
       2.7973975061277987e-04, 
       3.4710503068018003e-04, 
       2.4699301629553972e-04, 
       2.4719485643679009e-04, 
       2.5282255451128361e-04, 
       3.3807012415164577e-04, 
       2.5929335405473595e-04, 
       3.3509526863541745e-04, 
       2.4782803868245238e-04, 
       2.5108093918816158e-04, 
       3.0901882212811270e-04, 
       2.5568366756854752e-04, 
       2.9640659487743240e-04, 
       3.4100128466179474e-04, 
       3.0985435917605633e-04, 
       2.7633004621094043e-04, 
       3.1226978544487299e-04, 
       2.8427540403496115e-04, 
       2.6779733037916664e-04, 
       2.5535088485590095e-04, 
       2.9089408655607295e-04, 
       2.5283240419304365e-04, 
       2.5696816267963837e-04, 
       2.8192317912486810e-04, 
       2.4773479211039772e-04, 
       2.6539622011183847e-04, 
       2.5424749836417760e-04, 
       2.8305831958776374e-04, 
       2.7985834617378654e-04, 
       3.3999999999997677e-04, 
       3.3963085401933089e-04, 
       3.3849525474905038e-04, 
       2.3977674056949386e-04, 
       2.8213381136175058e-04, 
       2.4045005515574227e-04, 
       3.3250885629433771e-04, 
       2.5223041540520610e-04, 
       2.4968916515183398e-04, 
       3.1336824593203571e-04, 
       2.6013923924945038e-04, 
       2.4783537528821507e-04, 
       2.5096483522151115e-04, 
       3.2829473124502707e-04, 
       2.7939835411289925e-04, 
       2.7914201765398761e-04, 
       2.5257057486488321e-04, 
       2.5306669735605981e-04, 
       2.7791608691046625e-04, 
       3.2697848120397609e-04, 
       3.2616855813316647e-04, 
       3.2118023809172191e-04, 
       2.4801439023399791e-04, 
       2.5259898189532680e-04, 
       3.3155444988579292e-04, 
       3.0793877799634872e-04, 
       2.9806732489713156e-04, 
       2.8004474074230535e-04, 
       3.2472797944905688e-04, 
       2.3393727992128394e-04, 
       2.5619369910748272e-04, 
       3.2856594952090592e-04, 
       2.7837324545852070e-04, 
       2.3567042563236025e-04, 
       3.2705122134773124e-04, 
       2.7138752800579381e-04, 
       3.2728425687190022e-04, 
       2.4311807194284896e-04, 
       2.4272938538496468e-04, 
       2.3579935773756629e-04, 
       2.7904710628036481e-04, 
       2.7877768578102655e-04, 
       2.3586852469842040e-04, 
       2.4520301594953904e-04, 
       2.2905429566589364e-04, 
       2.6810160108816533e-04, 
       2.6789511472948815e-04, 
       2.4115458529217778e-04, 
       2.3324309697577038e-04, 
       2.9954418391687484e-04, 
       2.8605858470754183e-04, 
       3.2174039888201920e-04, 
       2.3719227166811747e-04, 
       2.6609533664127518e-04, 
       2.2767667529402782e-04, 
       3.1791696857265108e-04, 
       3.0327868682943770e-04, 
       2.8113853310792104e-04, 
       3.2000000000001770e-04, 
       3.2000000000004442e-04, 
       2.3430614085050237e-04, 
       3.1671680845461752e-04, 
       3.0392419932039641e-04, 
       3.1615547541455059e-04, 
       2.8658651451206151e-04, 
       2.6463592934351489e-04, 
       3.0822068437102014e-04, 
       2.2556274162091985e-04, 
       2.2840189398557067e-04, 
       2.3719584204666294e-04, 
       3.1216208620905725e-04, 
       2.5449890807285040e-04, 
       2.5436344588189902e-04, 
       2.3538735028597537e-04, 
       3.1147282775995089e-04, 
       2.5969341461671652e-04, 
       3.1317461948619161e-04, 
       2.6340862216843078e-04, 
       2.3393406641701825e-04, 
       2.6439172803325927e-04, 
       2.5233741110161395e-04, 
       2.4231171600978458e-04, 
       3.1482955754108863e-04, 
       2.5522768005128418e-04, 
       2.5521588245201190e-04, 
       3.1381688584376116e-04, 
       2.5648729188693093e-04, 
       3.0268426695426762e-04, 
       3.1280353282196467e-04, 
       3.1202382480161988e-04, 
       2.3317163880900898e-04, 
       3.1218266534576024e-04, 
       2.9103326145474897e-04, 
       3.1176540652953229e-04, 
       2.2989221748258721e-04, 
       2.3688476104629736e-04, 
       3.0915704969407265e-04, 
       2.5773419781500539e-04, 
       2.3540475442860749e-04, 
       2.2821530714031650e-04, 
       2.2435797862340739e-04, 
       3.0999999908870525e-04, 
       3.0999999999995533e-04, 
       2.9744160928408088e-04, 
       2.5268772103077776e-04, 
       2.5204317672044381e-04, 
       2.5620717315975026e-04, 
       2.2149901677291959e-04, 
       2.2708527329067755e-04, 
       2.2262192236674442e-04, 
       2.9797646808577178e-04, 
       2.1757222439219346e-04, 
       3.0566583290436217e-04, 
       2.1929546850625516e-04, 
       2.1992350851614080e-04, 
       2.2475263530723574e-04, 
       3.0205953360630490e-04, 
       2.2977239204617595e-04, 
       2.3049290797896250e-04, 
       2.1500091076442467e-04, 
       2.2524368222211189e-04, 
       2.2501900577335461e-04, 
       2.9133652360006354e-04, 
       2.5024423343436022e-04, 
       2.5009461831495931e-04, 
       2.2601035982896625e-04, 
       2.9999999999996853e-04, 
       2.2322428892762461e-04, 
       2.7793638310670186e-04, 
       2.9866512115810974e-04, 
       2.2323497157643709e-04, 
       2.4975541771191106e-04, 
       2.1197769128322312e-04, 
       2.5453294413861448e-04, 
       2.8427321643510907e-04, 
       2.4190221773151853e-04, 
       2.1819894203704577e-04, 
       2.6099551698108655e-04, 
       2.8289151125479719e-04, 
       2.1979857578763252e-04, 
       2.9447306383953919e-04, 
       2.1109721135205640e-04, 
       2.9430324605776964e-04, 
       2.6889400409187689e-04, 
       2.1623334791344513e-04, 
       2.7745577353132820e-04, 
       2.0803433172293591e-04, 
       2.2202143031365020e-04, 
       2.6427417648126059e-04, 
       2.4313642858022461e-04, 
       2.8154748879054162e-04, 
       2.6022102441427370e-04, 
       2.4263446590600505e-04, 
       2.8143328749111499e-04, 
       2.4052465845005117e-04, 
       2.2790752633461154e-04, 
       2.8999999999996585e-04, 
       2.9000000000004023e-04, 
       2.1136425109980801e-04, 
       2.8792400406770977e-04, 
       2.5784407148522168e-04, 
       2.0599743197571911e-04, 
       2.6823189698646277e-04, 
       2.3864624382814809e-04, 
       2.0654354463586750e-04, 
       2.0650824641764735e-04, 
       2.3807126317323553e-04, 
       2.2249936953832900e-04, 
       2.0486768115336303e-04, 
       2.8551174868051918e-04, 
       2.7045841141958558e-04, 
       2.4282088432341491e-04, 
       2.0188497525244395e-04, 
       2.3629356788566148e-04, 
       2.2342977990820112e-04, 
       2.3502030290252078e-04, 
       2.3600733707685161e-04, 
       2.0076078679283655e-04, 
       2.5770553059412474e-04, 
       2.8268685380193723e-04, 
       2.1943505033605501e-04, 
       2.5630450071446604e-04, 
       2.3412907149410224e-04, 
       2.4116841449521123e-04, 
       2.0734485834611233e-04, 
       2.0937758302858094e-04, 
       2.7628774519960564e-04, 
       2.6374089093327333e-04, 
       2.1948984907636736e-04, 
       2.2487793751790322e-04, 
       2.0913516848770290e-04, 
       2.0187775868518284e-04, 
       2.8000000000003868e-04, 
       2.6225653176257770e-04, 
       2.6509698256519672e-04, 
       2.3844322890897673e-04, 
       1.9665986766056101e-04, 
       2.0111732430850836e-04, 
       2.6338312827091582e-04, 
       2.6068781202255765e-04, 
       2.0705591105201691e-04, 
       2.5845458073120746e-04, 
       1.9837677368377229e-04, 
       1.9604612271938266e-04, 
       2.7611247564881912e-04, 
       2.0178811723063668e-04, 
       1.9496006978690384e-04, 
       2.6621632966843497e-04, 
       2.3992448146326796e-04, 
       2.6564838803582618e-04, 
       2.1453072026988854e-04, 
       2.0242795087595258e-04, 
       2.3347231881524477e-04, 
       2.6687870558351698e-04, 
       2.6323077289597366e-04, 
       2.0408123499937787e-04, 
       1.9651225116706819e-04, 
       1.9341462991786140e-04, 
       2.1439355610803512e-04, 
       1.9247980619437155e-04, 
       2.3144883337296209e-04, 
       2.0026319710512395e-04, 
       2.4999999999999903e-04, 
       2.5000000000003231e-04, 
       2.0259484227500721e-04, 
       2.6231438262293907e-04, 
       1.9354497264774596e-04, 
       1.9345920373048503e-04, 
       2.7000000000004109e-04, 
       2.2430020695866915e-04, 
       2.7000000000000412e-04, 
       2.2549765354080070e-04, 
       1.9164253959316806e-04, 
       2.2509494155806910e-04, 
       2.2525026686727825e-04, 
       2.2318511283523620e-04, 
       2.3349247328095880e-04, 
       2.0434082807836406e-04, 
       2.0834132992564555e-04, 
       1.9294170107195499e-04, 
       1.8967587950751355e-04, 
       2.5745810029448074e-04, 
       2.0685094522593633e-04, 
       1.8988320397823010e-04, 
       1.8986898960552298e-04, 
       2.5535056960332815e-04, 
       2.0267129345234516e-04, 
       1.9956288665435413e-04, 
       2.4036168123659263e-04, 
       1.8996084203244602e-04, 
       2.6257343072142104e-04, 
       2.1977086040144420e-04, 
       2.1916480413819022e-04, 
       2.2844154174104164e-04, 
       1.8791228730514038e-04, 
       2.6347830992844233e-04, 
       2.1968083976387103e-04, 
       1.9273682796051440e-04, 
       2.6111258340710239e-04, 
       2.2608965841521824e-04, 
       1.9751690946885058e-04, 
       2.5412264350696491e-04, 
       2.1399416926409822e-04, 
       2.5308071120466388e-04, 
       2.1368925103375670e-04, 
       2.0233968547629742e-04, 
       2.1783334265975033e-04, 
       2.1484436905901236e-04, 
       2.0454533427018013e-04, 
       2.4879992141209003e-04, 
       1.9222968870637766e-04, 
       1.9118386806846177e-04, 
       2.0542411524102660e-04, 
       2.1603266564374946e-04, 
       2.3890753774537379e-04, 
       1.8429851713084840e-04, 
       2.3515367181629464e-04, 
       1.9245860287752804e-04, 
       2.5999999999998838e-04, 
       2.6000000000002724e-04, 
       2.5999999999999998e-04, 
       2.1339403724271124e-04, 
       1.9385983907538516e-04, 
       2.0567975577988150e-04, 
       1.9378579460987993e-04, 
       1.9370530809177691e-04, 
       2.4004264701372042e-04, 
       1.8401429832986659e-04, 
       2.5706041756531743e-04, 
       1.9205336067977150e-04, 
       1.8641168799872219e-04, 
       1.9871596974680873e-04, 
       2.3346162171777741e-04, 
       2.5549597071476400e-04, 
       2.4506630116177223e-04, 
       2.5497502263371018e-04, 
       2.0019183798654542e-04, 
       2.1863970087327935e-04, 
       2.4215656804310847e-04, 
       1.9947881840077890e-04, 
       1.8122610024640332e-04, 
       2.1148664323111877e-04, 
       1.8183902339949064e-04, 
       1.7955063605081613e-04, 
       2.1964981884957840e-04, 
       2.0649505172753237e-04, 
       2.4751605075783932e-04, 
       2.5194141162151448e-04, 
       2.0943619884108503e-04, 
       2.4713645351721371e-04, 
       2.2520760913522170e-04, 
       1.9834096329994437e-04, 
       1.7751719847499997e-04, 
       2.0840194989387871e-04, 
       1.8654202289697302e-04, 
       1.8062638012133694e-04, 
       1.8256946700069264e-04, 
       2.4999999849476456e-04, 
       1.7900677423558474e-04, 
       2.4999999999997398e-04, 
       2.5000000000000001e-04, 
       2.1322514357739974e-04, 
       1.9668870019524937e-04, 
       1.7968536908475615e-04, 
       2.4308609999510795e-04, 
       2.4474445627074111e-04, 
       2.2462253295264285e-04, 
       1.7999999999998418e-04, 
       1.9674066213209079e-04, 
       2.2106027174145405e-04, 
       2.0763941342180273e-04, 
       2.0632523958685788e-04, 
       1.7932639278016573e-04, 
       1.8610790686288738e-04, 
       2.0567405580623879e-04, 
       2.4745557041549896e-04, 
       2.4378876326027171e-04, 
       1.8002786593308484e-04, 
       2.0210148182786298e-04, 
       1.7672629716127206e-04, 
       1.7632235701721456e-04, 
       1.8503001149529135e-04, 
       2.4348532202869423e-04, 
       2.4524914550459476e-04, 
       2.4466121487279303e-04, 
       1.7418089125860803e-04, 
       2.4494067754826350e-04, 
       1.8411289069474883e-04, 
       2.4382441495580942e-04, 
       2.4239839706128764e-04, 
       2.3595555269380691e-04, 
       2.2615210627401538e-04, 
       2.0989451395970300e-04, 
       2.0989451395970300e-04, 
       2.0099270166529039e-04, 
       1.9638073835901758e-04, 
       1.9634152022268928e-04, 
       1.7560861284123544e-04, 
       1.8922351013133323e-04, 
       1.7516930455856077e-04, 
       2.1804833065561008e-04, 
       2.1781961728480382e-04, 
       1.8914532283297668e-04, 
       2.4062256760033399e-04, 
       2.3999999938011359e-04, 
       2.4000000000003950e-04, 
       1.7773774901065794e-04, 
       2.2829547390698215e-04, 
       2.3370156236937273e-04, 
       1.9882069584392146e-04, 
       2.0654639365039665e-04, 
       2.2029074928855608e-04, 
       2.3783418033836821e-04, 
       2.2000000000002611e-04, 
       2.1880329653845720e-04, 
       1.9830188000751601e-04, 
       1.8759849489001984e-04, 
       1.8061492288644478e-04, 
       1.6878256380459418e-04, 
       1.7730656474631621e-04, 
       2.0387918463937786e-04, 
       1.7887695003655505e-04, 
       2.3263123250724243e-04, 
       2.3027501567002260e-04, 
       2.2541082404631396e-04, 
       2.3683182833298509e-04, 
       2.2723577686318660e-04, 
       2.3652420110730206e-04, 
       1.9729299459400409e-04, 
       1.6743033833002279e-04, 
       2.0066346598707449e-04, 
       1.7207766987645189e-04, 
       1.6706117561708883e-04, 
       1.9552665050008418e-04, 
       2.2709119239798802e-04, 
       2.2476973040367378e-04, 
       2.2451420751005340e-04, 
       1.9556443108675100e-04, 
       2.1509982510861100e-04, 
       1.8958718076528264e-04, 
       1.9460526412567515e-04, 
       2.3299875583095806e-04, 
       1.6719105060434582e-04, 
       1.6556682272633315e-04, 
       2.3247342785214517e-04, 
       2.2000000000003104e-04, 
       2.2406518169228550e-04, 
       2.2920242042894314e-04, 
       1.8653197006921946e-04, 
       1.8639809982404844e-04, 
       2.2478186245675758e-04, 
       1.9843842809095339e-04, 
       2.2548931449566900e-04, 
       1.8853905584557880e-04, 
       1.6203269795081330e-04, 
       1.6264942681795956e-04, 
       1.6664962869265858e-04, 
       2.0999999999995477e-04, 
       1.6609262223060798e-04, 
       2.2306508566916029e-04, 
       2.1198933398516918e-04, 
       2.0209149221359651e-04, 
       2.0267190103407011e-04, 
       2.2410662266353771e-04, 
       1.8475573229451833e-04, 
       2.0568120441227043e-04, 
       2.1668748372709662e-04, 
       1.8206579700248956e-04, 
       2.2423300995854555e-04, 
       1.8173933140342254e-04, 
       1.8170965532504499e-04, 
       1.8713620395521889e-04, 
       1.6824834436211903e-04, 
       2.2448320945265150e-04, 
       2.1765007028663841e-04, 
       2.2307187812426087e-04, 
       2.1093991019072498e-04, 
       1.6025138311865316e-04, 
       2.2287412117075222e-04, 
       2.2094061455424846e-04, 
       1.6594820210389250e-04, 
       1.6644064948736367e-04, 
       2.2269581292867462e-04, 
       2.2125652072816520e-04, 
       2.1992016418302367e-04, 
       1.6387606382490191e-04, 
       1.6387606382490191e-04, 
       2.2085647907655440e-04, 
       1.7326738331311869e-04, 
       1.5714929915227561e-04, 
       2.1910538948336330e-04, 
       1.5721279031039717e-04, 
       1.6312033268606581e-04, 
       1.5941622696788346e-04, 
       1.8303249044752322e-04, 
       1.8260384170897472e-04, 
       1.8101569030928630e-04, 
       1.7901421414621421e-04, 
       1.7896416889367592e-04, 
       2.1999999999997214e-04, 
       2.1999999999981873e-04, 
       2.1992052197674223e-04, 
       1.9675514272150436e-04, 
       1.8005840266708869e-04, 
       1.6451930988121432e-04, 
       1.6605556438789405e-04, 
       1.5894376709162786e-04, 
       1.5802178955935829e-04, 
       1.8198896526810433e-04, 
       2.1016143332279935e-04, 
       2.0726322085303121e-04, 
       1.7885026518711547e-04, 
       1.9204042605274754e-04, 
       2.1837455490796189e-04, 
       1.8104066755906517e-04, 
       2.1804715489596541e-04, 
       2.0531624510420490e-04, 
       1.5475920596201236e-04, 
       1.8016643583571024e-04, 
       1.8699862737107888e-04, 
       1.9578263509659665e-04, 
       1.8305500784621524e-04, 
       1.6249764582446324e-04, 
       2.0929667091805338e-04, 
       2.1582870340570160e-04, 
       1.6605551212964048e-04, 
       2.0716954084167583e-04, 
       2.1507069086757128e-04, 
       1.8475771193749287e-04, 
       1.7831875447095699e-04, 
       1.5849022773778927e-04, 
       1.7767010462171225e-04, 
       1.9030256055376209e-04, 
       1.5464559081375723e-04, 
       1.7647647198693029e-04, 
       2.0408406983552979e-04, 
       1.9205906405437353e-04, 
       1.7634862576236844e-04, 
       1.5498527962314593e-04, 
       1.5494446850651414e-04, 
       2.0604727917679542e-04, 
       1.9777089989585393e-04, 
       2.0977876886592097e-04, 
       2.0470882574515059e-04, 
       1.8476957104605053e-04, 
       1.5451341198577181e-04, 
       1.7873275764015799e-04, 
       1.5223656451413296e-04, 
       1.9512056310265060e-04, 
       1.7858548915195452e-04, 
       1.8010282269353056e-04, 
       2.1105580993237960e-04, 
       2.1028226997063743e-04, 
       1.5788474322783725e-04, 
       1.7052669296654981e-04, 
       2.0884475469785347e-04, 
       1.8339236624213706e-04, 
       1.4882153936831809e-04, 
       1.7028186227888265e-04, 
       1.7434512776768699e-04, 
       1.8092171327827123e-04, 
       1.5622488767547135e-04, 
       2.0859338132221138e-04, 
       2.0881609779865713e-04, 
       2.0025656943117707e-04, 
       2.0895988641599414e-04, 
       1.7970859669072208e-04, 
       1.8774959425509104e-04, 
       1.6484850097486862e-04, 
       1.5322829384781218e-04, 
       1.5297396408521079e-04, 
       1.7097089104609490e-04, 
       1.5249135802333427e-04, 
       2.0680741789482447e-04, 
       1.5591361198113529e-04, 
       1.4618126955077889e-04, 
       1.9000000000000259e-04, 
       1.9000000000000259e-04, 
       1.4839016971000663e-04, 
       1.4976375274512033e-04, 
       1.9192261913811949e-04, 
       1.7024614522762827e-04, 
       2.0347297702718616e-04, 
       1.7130009479180536e-04, 
       1.7019459092138404e-04, 
       1.5480162580400620e-04, 
       2.0265709441273327e-04, 
       1.5589225669549738e-04, 
       1.5811161405271450e-04, 
       1.4583043058910938e-04, 
       1.5275191591023984e-04, 
       1.6386759147379607e-04, 
       1.5649873055971272e-04, 
       2.0210330448043061e-04, 
       1.5673742734416898e-04, 
       1.8868186705622329e-04, 
       1.6853581307339272e-04, 
       1.5201154003508039e-04, 
       1.5002609397987582e-04, 
       1.9935176528461965e-04, 
       1.6925668572832545e-04, 
       1.5852872819336388e-04, 
       1.9211338644997480e-04, 
       1.6450494012466580e-04, 
       1.9948930314551822e-04, 
       1.4189911685380784e-04, 
       1.6386174795584598e-04, 
       2.0000000000003292e-04, 
       1.7038579739620565e-04, 
       1.4153030563987212e-04, 
       1.4922653782461313e-04, 
       1.4865592872318311e-04, 
       1.8140558847304982e-04, 
       1.6647439892511631e-04, 
       1.7123753807215216e-04, 
       1.4565639678960004e-04, 
       1.6131827162816075e-04, 
       1.6130062419559557e-04, 
       1.4655596342892557e-04, 
       1.4777127138669135e-04, 
       1.8323557599784652e-04, 
       1.5887786913816517e-04, 
       1.5880612631051874e-04, 
       1.4703599117121615e-04, 
       1.5396621220992117e-04, 
       1.5256192388537245e-04, 
       1.4475885337201480e-04, 
       1.7564620822911917e-04, 
       1.4658731516480235e-04, 
       1.6263882846477584e-04, 
       1.9007235950751759e-04, 
       1.4487943684404124e-04, 
       1.5797187059511290e-04, 
       1.4305270602086776e-04, 
       1.4104692972479634e-04, 
       1.4889820738663365e-04, 
       1.4889820738663365e-04, 
       1.6188695180629020e-04, 
       1.4531913062627651e-04, 
       1.3939772249568275e-04, 
       1.5199269830109031e-04, 
       1.3952832480440807e-04, 
       1.7948020866856711e-04, 
       1.4598068997115319e-04, 
       1.4638352527027390e-04, 
       1.4252105004144175e-04, 
       1.6703984108697837e-04, 
       1.6226098768177096e-04, 
       1.3984191175566446e-04, 
       1.6109085501545911e-04, 
       1.3917560096728221e-04, 
       1.4823399073103232e-04, 
       1.5917056510460700e-04, 
       1.6114494611487964e-04, 
       1.6094430309989609e-04, 
       1.6090858292786033e-04, 
       1.8586403340804404e-04, 
       1.4614023712254063e-04, 
       1.8722612635460367e-04, 
       1.5963222980164971e-04, 
       1.9258407914400820e-04, 
       1.5057156362152240e-04, 
       1.8231387657230216e-04, 
       1.5298614928851081e-04, 
       1.9180258387790046e-04, 
       1.8456411542339035e-04, 
       1.5760672372505650e-04, 
       1.4634116113870644e-04, 
       1.4131268587429870e-04, 
       1.5813213596671094e-04, 
       1.4234461109327207e-04, 
       1.5444728059196133e-04, 
       1.7391698052298776e-04, 
       1.3978729177619508e-04, 
       1.9000000000003985e-04, 
       1.8999999999996331e-04, 
       1.8543444693126871e-04, 
       1.8127371360462133e-04, 
       1.6393186684220067e-04, 
       1.6119617736567931e-04, 
       1.4120782633257397e-04, 
       1.4486855747275948e-04, 
       1.8565518721806655e-04, 
       1.5750451354046977e-04, 
       1.3566106770843454e-04, 
       1.3408159503427373e-04, 
       1.8840942442162066e-04, 
       1.3418773219158706e-04, 
       1.8355787696957358e-04, 
       1.8805342139972795e-04, 
       1.5664796201008679e-04, 
       1.3738560012912295e-04, 
       1.5742573826460271e-04, 
       1.3628009623921665e-04, 
       1.5628290943715172e-04, 
       1.4735028220591288e-04, 
       1.3352132243735313e-04, 
       1.8009899990636949e-04, 
       1.7995833071148570e-04, 
       1.8659117999617753e-04, 
       1.3540183094536501e-04, 
       1.3335621257032373e-04, 
       1.5144678391270535e-04, 
       1.3823210578779740e-04, 
       1.6442668361415136e-04, 
       1.4193732633142146e-04, 
       1.8645140303901687e-04, 
       1.5536043186832382e-04, 
       1.3790606748834616e-04, 
       1.5965898780879206e-04, 
       1.8613226350233769e-04, 
       1.4228127918744228e-04, 
       1.4228127918744228e-04, 
       1.7910694429395607e-04, 
       1.3886445534098125e-04, 
       1.5291585927182499e-04, 
       1.3702736801156679e-04, 
       1.4148963027882838e-04, 
       1.3626089522172935e-04, 
       1.5340780807074033e-04, 
       1.3086033775648668e-04, 
       1.3472747676253444e-04, 
       1.5388677799297988e-04, 
       1.5300727782187963e-04, 
       1.3893791039736324e-04, 
       1.3055202271219514e-04, 
       1.7869023289706230e-04, 
       1.8330734739237245e-04, 
       1.7954853120639836e-04, 
       1.4130018723299147e-04, 
       1.2964238938799338e-04, 
       1.3000000000000500e-04, 
       1.7448456336048519e-04, 
       1.5284264438161980e-04, 
       1.8241009476485897e-04, 
       1.5296792097227612e-04, 
       1.3455418110024713e-04, 
       1.3905941360776441e-04, 
       1.7914656227956832e-04, 
       1.7471604237557093e-04, 
       1.8184097379011637e-04, 
       1.8048553862764206e-04, 
       1.7157880938600221e-04, 
       1.3448798795938360e-04, 
       1.5075862181441099e-04, 
       1.8039208196318228e-04, 
       1.8029859724180144e-04, 
       1.2970876796068303e-04, 
       1.3245728493635982e-04, 
       1.8000000000002077e-04, 
       1.4216448340443382e-04, 
       1.8000000000002226e-04, 
       1.8000000000001275e-04, 
       1.3017621797024967e-04, 
       1.3443956368586123e-04, 
       1.2952759826074327e-04, 
       1.5387316335389024e-04, 
       1.5912604808692449e-04, 
       1.7455886127829326e-04, 
       1.4719608075528955e-04, 
       1.6316723134130706e-04, 
       1.3297058212563280e-04, 
       1.4335741709054479e-04, 
       1.3157080375089433e-04, 
       1.7716585667569063e-04, 
       1.3000000000001471e-04, 
       1.4785695918166319e-04, 
       1.5157711389416218e-04, 
       1.4686561901280682e-04, 
       1.7067842344961311e-04, 
       1.4653936132412600e-04, 
       1.2447760840523199e-04, 
       1.2459580892553528e-04, 
       1.2405372936183270e-04, 
       1.7014372302603348e-04, 
       1.3102710557645722e-04, 
       1.2643841474122445e-04, 
       1.2464807486674666e-04, 
       1.6483510297495805e-04, 
       1.2451522767356545e-04, 
       1.3517562639984299e-04, 
       1.3978305538656557e-04, 
       1.3370668305938399e-04, 
       1.4908730012117805e-04, 
       1.2281322566576729e-04, 
       1.2918416459710481e-04, 
       1.6908967219138671e-04, 
       1.6580496218566068e-04, 
       1.4339080594022024e-04, 
       1.4154095875788987e-04, 
       1.6339785093517301e-04, 
       1.3218778997232537e-04, 
       1.6583434988729504e-04, 
       1.7109875005711265e-04, 
       1.2208479655807610e-04, 
       1.7153848512945912e-04, 
       1.2264675167000320e-04, 
       1.4311927612881812e-04, 
       1.2806238289468123e-04, 
       1.2255569134591823e-04, 
       1.6963550548596991e-04, 
       1.2174584543555244e-04, 
       1.3755528165326024e-04, 
       1.7000000000002576e-04, 
       1.4694753634467622e-04, 
       1.2385642484404952e-04, 
       1.2060199576864490e-04, 
       1.3289378310202886e-04, 
       1.7000000000000099e-04, 
       1.6999999999995949e-04, 
       1.2347559307610947e-04, 
       1.3056430606538138e-04, 
       1.2756777418066747e-04, 
       1.2725421518942540e-04, 
       1.4225195534547599e-04, 
       1.2286527424750272e-04, 
       1.4865755944095356e-04, 
       1.6059486697790195e-04, 
       1.4021547929632110e-04, 
       1.4453980011956607e-04, 
       1.6180867436633235e-04, 
       1.3424684953348040e-04, 
       1.2159045515285430e-04, 
       1.6475150319801389e-04, 
       1.2878824779840171e-04, 
       1.4462478997558280e-04, 
       1.3879869157117265e-04, 
       1.6614099885489198e-04, 
       1.6185801959003497e-04, 
       1.1909943731536806e-04, 
       1.3550780264248998e-04, 
       1.6671018263061932e-04, 
       1.2650625575230391e-04, 
       1.2925854372029339e-04, 
       1.2913476548672627e-04, 
       1.4905605843806101e-04, 
       1.6630528932720849e-04, 
       1.1991508831452532e-04, 
       1.4112333148850589e-04, 
       1.5033782635389117e-04, 
       1.5014199107311798e-04, 
       1.6559684807582580e-04, 
       1.5267771353526924e-04, 
       1.3323118472225129e-04, 
       1.5056739787229956e-04, 
       1.1719132503332358e-04, 
       1.1804333115692082e-04, 
       1.1760131079247727e-04, 
       1.1756055417724166e-04, 
       1.3688605158480185e-04, 
       1.6380880965737163e-04, 
       1.2883650946927860e-04, 
       1.6237713369364722e-04, 
       1.3630539375573893e-04, 
       1.6381860609971792e-04, 
       1.6262014773449490e-04, 
       1.2069620876317821e-04, 
       1.3969029887618591e-04, 
       1.5235228325381566e-04, 
       1.1672148138939228e-04, 
       1.1658546710949908e-04, 
       1.3644479149737515e-04, 
       1.5209948769125145e-04, 
       1.1763742938814272e-04, 
       1.3919857330780473e-04, 
       1.6298589813169067e-04, 
       1.5053581070970960e-04, 
       1.5000000000001910e-04, 
       1.5069024567772744e-04, 
       1.4544735982563082e-04, 
       1.3158982808350426e-04, 
       1.5624371793753139e-04, 
       1.5838834046787132e-04, 
       1.3410235805684048e-04, 
       1.3238307621922009e-04, 
       1.3460666294570382e-04, 
       1.5880752538002921e-04, 
       1.3486178681172586e-04, 
       1.1850134478763180e-04, 
       1.5704146721262860e-04, 
       1.6065436212689589e-04, 
       1.5080638990430960e-04, 
       1.2002298997067175e-04, 
       1.3344053357597520e-04, 
       1.3762666613752428e-04, 
       1.3831279475731023e-04, 
       1.5872879914473334e-04, 
       1.1702022556971323e-04, 
       1.5999999999995391e-04, 
       1.6000000000000779e-04, 
       1.1331944199996376e-04, 
       1.5930875822672526e-04, 
       1.5351109597861923e-04, 
       1.5832024257360079e-04, 
       1.3251117294219608e-04, 
       1.5394681573615768e-04, 
       1.4428363416040311e-04, 
       1.2487632250111243e-04, 
       1.1888712238653807e-04, 
       1.3227494497380926e-04, 
       1.3651285580680534e-04, 
       1.3206640364101142e-04, 
       1.3627218234281901e-04, 
       1.5340886602111347e-04, 
       1.1448345063470422e-04, 
       1.3592060976283315e-04, 
       1.1851240154062151e-04, 
       1.1295430251754943e-04, 
       1.3086445416674380e-04, 
       1.5751810183404623e-04, 
       1.1462770344066444e-04, 
       1.4665142352712524e-04, 
       1.2208871501011412e-04, 
       1.1428018053889767e-04, 
       1.1499930693333800e-04, 
       1.2588953654026048e-04, 
       1.5699835465420617e-04, 
       1.4561514422524989e-04, 
       1.5255547493516980e-04, 
       1.3068549716880240e-04, 
       1.2388467316961461e-04, 
       1.1277502243189071e-04, 
       1.5700586922016876e-04, 
       1.2518122822048272e-04, 
       1.3505606027831694e-04, 
       1.1683775191893710e-04, 
       1.1166436603549957e-04, 
       1.5482387891183660e-04, 
       1.1132268571256807e-04, 
       1.5636715982280181e-04, 
       1.1234095448336817e-04, 
       1.2580202647069163e-04, 
       1.1832324759572731e-04, 
       1.4476595734953294e-04, 
       1.2909864667410602e-04, 
       1.1759139102050370e-04, 
       1.4439216008446139e-04, 
       1.1152195640475793e-04, 
       1.2927921775693200e-04, 
       1.5194913283033722e-04, 
       1.3300969286135706e-04, 
       1.3288862311422117e-04, 
       1.2260380650769523e-04, 
       1.5345250003478554e-04, 
       1.1075588558350213e-04, 
       1.1117618258523058e-04, 
       1.1254069193349568e-04, 
       1.2632911205967578e-04, 
       1.1486703135125598e-04, 
       1.2185660220151639e-04, 
       1.0910066659627562e-04, 
       1.3912362956192693e-04, 
       1.5261955325947604e-04, 
       1.1010075447259687e-04, 
       1.3402797639511683e-04, 
       1.4175482508578812e-04, 
       1.2580055537558685e-04, 
       1.1814971544949213e-04, 
       1.4323255470301811e-04, 
       1.1763832620088614e-04, 
       1.2676046176393377e-04, 
       1.4552646040123655e-04, 
       1.5275008841376291e-04, 
       1.1430410815036325e-04, 
       1.2599413969948415e-04, 
       1.5186123918611613e-04, 
       1.2954460197720776e-04, 
       1.1643813256308433e-04, 
       1.1794416741161970e-04, 
       1.3272116574203515e-04, 
       1.4000000000003967e-04, 
       1.3866962318668868e-04, 
       1.5027839277891486e-04, 
       1.2408214929729094e-04, 
       1.3136918225224007e-04, 
       1.3058218750917821e-04, 
       1.0900227995060493e-04, 
       1.0812486473705156e-04, 
       1.1000000000000477e-04, 
       1.2301501421590460e-04, 
       1.1221538520675204e-04, 
       1.1050516540022862e-04, 
       1.0886341379226622e-04, 
       1.2779982847049026e-04, 
       1.5000000000004620e-04, 
       1.2980106501344824e-04, 
       1.2980106501344824e-04, 
       1.2450866466208140e-04, 
       1.0607329251539465e-04, 
       1.1526734122477948e-04, 
       1.2930918068652445e-04, 
       1.1098463061708403e-04, 
       1.3987372690784072e-04, 
       1.2789019317099347e-04, 
       1.2197401212516218e-04, 
       1.4484942316613520e-04, 
       1.2367618622315753e-04, 
       1.2853154831903485e-04, 
       1.2776870203034564e-04, 
       1.4651213369541353e-04, 
       1.2918728289517642e-04, 
       1.0785500712910272e-04, 
       1.4813364912937558e-04, 
       1.4217426421116026e-04, 
       1.3689600627323823e-04, 
       1.1600472723107482e-04, 
       1.0769226148962623e-04, 
       1.1058547095259425e-04, 
       1.0969714610893606e-04, 
       1.0536772778078748e-04, 
       1.1679151521150670e-04, 
       1.3218636195589430e-04, 
       1.2101758820216473e-04, 
       1.0574030783565906e-04, 
       1.2306261788190418e-04, 
       1.3916954153768641e-04, 
       1.4623470193514220e-04, 
       1.3164098117362304e-04, 
       1.1602353451452037e-04, 
       1.0564102775582140e-04, 
       1.2921388712637788e-04, 
       1.4096924073079368e-04, 
       1.2251684245847343e-04, 
       1.0742540548134736e-04, 
       1.4571247290509837e-04, 
       1.1870094810708615e-04, 
       1.1857481691012595e-04, 
       1.0999999999999777e-04, 
       1.2942071276436017e-04, 
       1.0891526675758188e-04, 
       1.2183667546130603e-04, 
       1.2071273883791727e-04, 
       1.4561969672886060e-04, 
       1.2642214618780069e-04, 
       1.4358871043113362e-04, 
       1.4631824587741979e-04, 
       1.0441216353664519e-04, 
       1.0758310678498174e-04, 
       1.1722109433736283e-04, 
       1.0351293863515583e-04, 
       1.2417566669373627e-04, 
       1.1000000000003117e-04, 
       1.2772791051136360e-04, 
       1.4495150345010912e-04, 
       1.2335844638726359e-04, 
       1.0614733444542853e-04, 
       1.1101888361306983e-04, 
       1.4201194672341949e-04, 
       1.0752541603442476e-04, 
       1.0349048526868057e-04, 
       1.1948299121558004e-04, 
       1.4483394606147943e-04, 
       1.4313449224105597e-04, 
       1.4451949577048187e-04, 
       1.0677506532342738e-04, 
       1.0642295243891492e-04, 
       1.3930003683782711e-04, 
       1.0586591301502096e-04, 
       1.2841646078912031e-04, 
       1.4108572918725108e-04, 
       1.1323506373512079e-04, 
       1.1736735659276794e-04, 
       1.4025624931218717e-04, 
       1.0477536765615824e-04, 
       1.0706321074530425e-04, 
       1.0415296577244842e-04, 
       1.1837213698458455e-04, 
       1.3261502422008879e-04, 
       1.0576477927013387e-04, 
       1.3055209835826658e-04, 
       1.3054729632964022e-04, 
       1.0425305513720020e-04, 
       1.4237852759741573e-04, 
       1.0109610832278538e-04, 
       1.1826126915906035e-04, 
       1.3704896562040164e-04, 
       1.4093093866209729e-04, 
       1.0385972721385113e-04, 
       1.2610483248449864e-04, 
       1.4000000000004875e-04, 
       1.0980833694774068e-04, 
       1.2229754359906664e-04, 
       1.4122845576607592e-04, 
       1.0208209851727968e-04, 
       1.0234832591432975e-04, 
       9.9657635289861066e-05, 
       1.3248514783421544e-04, 
       1.3248514783421544e-04, 
       1.0377366624192343e-04, 
       1.1103403941539034e-04, 
       1.0154046291275607e-04, 
       1.0906959122769470e-04, 
       1.0987668992938946e-04, 
       1.0069531561556829e-04, 
       1.1354629003186272e-04, 
       9.9295752501816803e-05, 
       1.3036286207517575e-04, 
       1.1650074746065720e-04, 
       1.3999999999995990e-04, 
       1.4000000000000110e-04, 
       1.3999999999999890e-04, 
       1.3999999999995198e-04, 
       1.3564817425279778e-04, 
       1.0043021954543532e-04, 
       9.8955601795769620e-05, 
       1.0807437342299052e-04, 
       1.3542203561031402e-04, 
       1.1603019781220284e-04, 
       1.0064065551563254e-04, 
       1.0947346569794872e-04, 
       1.3444971469351975e-04, 
       1.0556037979060595e-04, 
       1.3829812157927548e-04, 
       1.2999999999998714e-04, 
       9.8429234511192070e-05, 
       1.0166981641428263e-04, 
       1.1478205072131659e-04, 
       1.1040980640617042e-04, 
       1.0851264020890535e-04, 
       1.0843069105958756e-04, 
       1.1092130330270697e-04, 
       1.3750860922979695e-04, 
       9.9656089565455891e-05, 
       1.1623577018041262e-04, 
       1.0467974832721055e-04, 
       1.1107459701207377e-04, 
       1.0273891534618994e-04, 
       1.0420787295947745e-04, 
       1.3697496735186256e-04, 
       1.3637557484326128e-04, 
       1.1194136049221840e-04, 
       1.1194136049221840e-04, 
       1.0133367269267899e-04, 
       1.1128831754655549e-04, 
       1.1747764593815278e-04, 
       9.9064662352325492e-05, 
       1.3624687853630552e-04, 
       1.3537163431206719e-04, 
       1.3615828642036677e-04, 
       1.0088081393884620e-04, 
       9.6304947622919727e-05, 
       1.1153708120011003e-04, 
       1.1230553903125473e-04, 
       1.1632769477829590e-04, 
       1.1248680259543406e-04, 
       1.0065541685252134e-04, 
       1.3495516404631613e-04, 
       9.8356333291848986e-05, 
       1.2164487871491115e-04, 
       9.6827183575219939e-05, 
       1.3252104357553397e-04, 
       9.5846227302292321e-05, 
       1.2363091705896843e-04, 
       1.3515209756859590e-04, 
       1.0377627372025054e-04, 
       1.2069071941825651e-04, 
       1.2903555962552683e-04, 
       1.1113112433489377e-04, 
       1.3403191385925837e-04, 
       1.1896349798563184e-04, 
       1.2810434362628813e-04, 
       9.4906448671834605e-05, 
       1.0566561636052870e-04, 
       9.6842548233186989e-05, 
       1.1905140142587465e-04, 
       1.0377804388182562e-04, 
       1.2550640066725060e-04, 
       9.7561915037576536e-05, 
       1.3093956194442865e-04, 
       1.1410020794503139e-04, 
       1.2718986759722498e-04, 
       1.2987998109300602e-04, 
       1.0231402397797340e-04, 
       1.2525713326394571e-04, 
       1.0412416749121179e-04, 
       9.5831123647240895e-05, 
       1.2671755524927864e-04, 
       1.2898753537336682e-04, 
       1.1455997230533565e-04, 
       1.3213829568422465e-04, 
       1.0984612006841703e-04, 
       1.0577944699864368e-04, 
       1.0842216281595205e-04, 
       9.4038079998852997e-05, 
       1.0856887987416502e-04, 
       9.2964632245068297e-05, 
       1.3074205028631395e-04, 
       9.3412601347815421e-05, 
       1.1049674394400671e-04, 
       1.1049674394400671e-04, 
       9.8068554084132712e-05, 
       1.2756147172683277e-04, 
       1.3056452019125173e-04, 
       1.2392332873368850e-04, 
       1.1251162461555109e-04, 
       9.3536488593400283e-05, 
       1.1842575013581634e-04, 
       9.4376460021816503e-05, 
       1.2978097511959626e-04, 
       1.2994245233419633e-04, 
       9.4306630330219065e-05, 
       1.0854535738323931e-04, 
       1.1554208279398457e-04, 
       1.2987544981282417e-04, 
       1.0103366110753686e-04, 
       9.3850990420747563e-05, 
       1.2511734246622864e-04, 
       1.0746409006099631e-04, 
       1.2540665695261451e-04, 
       1.2836901770054131e-04, 
       1.2417774990342668e-04, 
       1.0081498044052136e-04, 
       1.2751866875823051e-04, 
       9.1568277510188333e-05, 
       1.0659709653387339e-04, 
       1.0647527542724309e-04, 
       9.4330581591470892e-05, 
       1.0558106245986374e-04, 
       1.0601071329336298e-04, 
       1.0052419587885571e-04, 
       1.0178137618718769e-04, 
       9.1498517567628563e-05, 
       1.2708940120230672e-04, 
       9.6925535558581706e-05, 
       9.0426684819324191e-05, 
       1.0885913717100040e-04, 
       1.2676901182814726e-04, 
       1.2134697536355791e-04, 
       9.4571172294993715e-05, 
       1.0514104418691219e-04, 
       1.2638674453410565e-04, 
       1.2442248464244589e-04, 
       9.6158389456886184e-05, 
       9.4543497887224144e-05, 
       8.9363620613159311e-05, 
       9.4509090912438926e-05, 
       9.4464279263820072e-05, 
       1.2466455270596931e-04, 
       9.7506219036385476e-05, 
       1.2044255214556726e-04, 
       1.0335441333010665e-04, 
       1.2486393866962324e-04, 
       9.3722178080192541e-05, 
       1.2065519576638520e-04, 
       1.1919620842876061e-04, 
       1.2450837504746990e-04, 
       9.0090384122727468e-05, 
       1.1610790525159790e-04, 
       1.0346654698785405e-04, 
       1.0199577774775537e-04, 
       9.9771768052383795e-05, 
       1.1677818757918457e-04, 
       1.2370503759884309e-04, 
       9.6272527017114047e-05, 
       9.5143961837704921e-05, 
       8.7512734539831790e-05, 
       8.9531775900607755e-05, 
       9.2200860650585960e-05, 
       1.2244128365521056e-04, 
       1.1354368932134140e-04, 
       1.2140564631876083e-04, 
       1.1869844864661550e-04, 
       8.9345052070426576e-05, 
       9.8034692418431243e-05, 
       1.1447774947201518e-04, 
       1.0599345875627486e-04, 
       8.7035843707702291e-05, 
       8.7035843707702291e-05, 
       1.0439107303714172e-04, 
       1.2248662230820542e-04, 
       1.2244104338450512e-04, 
       1.1948375995483888e-04, 
       1.2063232203128406e-04, 
       9.2192610898947654e-05, 
       9.9275150921640813e-05, 
       1.0138903213781838e-04, 
       9.9006615903272943e-05, 
       1.2172546652734902e-04, 
       1.1556318203856009e-04, 
       1.0124609829233624e-04, 
       1.2156565490811653e-04, 
       8.9513409976290447e-05, 
       8.7331786324354487e-05, 
       8.9955843974294478e-05, 
       9.9928520142543412e-05, 
       9.9192527763147402e-05, 
       1.1462072554235926e-04, 
       9.1346618405053215e-05, 
       8.5951004611516685e-05, 
       1.0112073733496960e-04, 
       1.1116710871815987e-04, 
       1.1633127694946090e-04, 
       1.0013600070086803e-04, 
       1.0035429818448213e-04, 
       9.4425047974876463e-05, 
       1.0319534459946427e-04, 
       1.1839225015148166e-04, 
       8.5762514177716880e-05, 
       9.9765934269498006e-05, 
       9.9842035472299086e-05, 
       1.1514015988654480e-04, 
       1.2000000000002682e-04, 
       1.2000000000001061e-04, 
       1.1999999999999011e-04, 
       1.2000000000003645e-04, 
       1.1999999999995575e-04, 
       1.1940477085031192e-04, 
       1.1602678910316416e-04, 
       8.6276530873133497e-05, 
       1.1300226761680984e-04, 
       8.7127563208010768e-05, 
       9.3966642725029477e-05, 
       1.1615330055173764e-04, 
       9.1555899520077836e-05, 
       9.1555899520077836e-05, 
       9.9945528136703498e-05, 
       1.1362537921878585e-04, 
       1.1112902742103280e-04, 
       1.0211065647865741e-04, 
       1.1532975179064319e-04, 
       1.1000000000002025e-04, 
       1.0074743513481569e-04, 
       8.7867454246812627e-05, 
       1.1212758179156246e-04, 
       1.1431728130369256e-04, 
       8.4023961728743211e-05, 
       1.1793771348098868e-04, 
       9.3813914394615030e-05, 
       8.6130628860348768e-05, 
       1.0505025669671977e-04, 
       8.3622980031645457e-05, 
       1.0559895742983438e-04, 
       8.4293287369582029e-05, 
       9.7854623379588660e-05, 
       8.4415530883748811e-05, 
       9.6690454032227584e-05, 
       1.1770778920013142e-04, 
       9.7800668104001978e-05, 
       9.7805423034166199e-05, 
       9.7416718583801949e-05, 
       8.2955144511724589e-05, 
       9.9056905683387656e-05, 
       9.7385227558982390e-05, 
       1.0396415602121767e-04, 
       9.2951087446997179e-05, 
       9.4481520445565041e-05, 
       9.2561394538370002e-05, 
       1.1384389226161904e-04, 
       1.1553371189915702e-04, 
       1.1623831228798012e-04, 
       8.6391518808076354e-05, 
       1.1519303820306984e-04, 
       1.1589918770282245e-04, 
       1.1600366311653424e-04, 
       1.1509223601072721e-04, 
       8.8967815755936044e-05, 
       9.0749700293172499e-05, 
       1.1548223978556904e-04, 
       9.5524527112435408e-05, 
       8.5944313369419459e-05, 
       8.1724010569126905e-05, 
       1.1243584227505029e-04, 
       9.1221656927148136e-05, 
       8.5202237062151012e-05, 
       1.0941008363297610e-04, 
       9.7358407967880085e-05, 
       8.4685704590305039e-05, 
       9.4908316937072635e-05, 
       8.3037738680439419e-05, 
       1.1337543984622992e-04, 
       8.4470407317411444e-05, 
       1.1453587300632310e-04, 
       9.5471656748831216e-05, 
       1.1084921182039240e-04, 
       8.2788688099392062e-05, 
       1.0924870113103861e-04, 
       9.2436780118694233e-05, 
       1.1389458846203944e-04, 
       8.2244474887760931e-05, 
       8.3353302877787656e-05, 
       1.1047209249116994e-04, 
       8.5745476677760577e-05, 
       1.1345591171058293e-04, 
       8.1272904770083280e-05, 
       1.1045211548922276e-04, 
       8.4833353592981371e-05, 
       8.3685848159278971e-05, 
       8.5066706933864246e-05, 
       8.0890476077037662e-05, 
       9.2448192853909742e-05, 
       1.0858067026619031e-04, 
       9.3291574089670101e-05, 
       8.5104330691077363e-05, 
       1.0952768191227760e-04, 
       9.3800360305940211e-05, 
       1.0107512810484684e-04, 
       1.1228292543536248e-04, 
       1.0445731817299992e-04, 
       9.2893930070133695e-05, 
       1.0340820264427405e-04, 
       1.0768106105878452e-04, 
       9.8049152152108400e-05, 
       7.9572939252956745e-05, 
       9.4380661800547267e-05, 
       9.4380661800547267e-05, 
       1.0963775318673943e-04, 
       9.4256986698576360e-05, 
       9.5173526053481569e-05, 
       8.0868984060553502e-05, 
       8.5155356114443334e-05, 
       1.0381659204951080e-04, 
       1.1067527173261012e-04, 
       8.1159215371754963e-05, 
       1.0615819232929342e-04, 
       9.3615981350920669e-05, 
       8.5263930059071750e-05, 
       8.0340520419823385e-05, 
       8.2788422078563148e-05, 
       1.0359009487965980e-04, 
       1.0960141660273000e-04, 
       7.9055850269712895e-05, 
       8.1959537506551521e-05, 
       8.0461347404811909e-05, 
       8.3381645528957660e-05, 
       7.8893324423134453e-05, 
       1.0999999999996466e-04, 
       1.0999999999999050e-04, 
       1.1000000000002681e-04, 
       1.0867527579689083e-04, 
       7.9114858003150043e-05, 
       7.9114858003150043e-05, 
       1.0940779309819365e-04, 
       1.0940779309819365e-04, 
       9.1211107282342154e-05, 
       8.2890656864390816e-05, 
       1.0818824581912958e-04, 
       1.0546465356649532e-04, 
       1.0868322690043291e-04, 
       7.8668509264994595e-05, 
       8.1055071394576681e-05, 
       8.2027424771362743e-05, 
       8.8035574315166909e-05, 
       7.7779868496286229e-05, 
       8.8328257280755672e-05, 
       1.0850215531571734e-04, 
       1.0539588646391191e-04, 
       1.0000000000000293e-04, 
       9.9999999999997199e-05, 
       1.0000000000000213e-04, 
       1.0561781008158581e-04, 
       1.0681730710454259e-04, 
       8.4848863112152629e-05, 
       7.9403614781265675e-05, 
       1.0414488429680525e-04, 
       1.0775223350058697e-04, 
       9.6515286832137652e-05, 
       1.0658434060926229e-04, 
       9.8572466268698081e-05, 
       1.0197485998576104e-04, 
       9.3891335191718063e-05, 
       7.6214426003865768e-05, 
       1.0758372172587323e-04, 
       8.3884370989410171e-05, 
       1.0156307729652022e-04, 
       8.3114617639840183e-05, 
       8.1955403856173950e-05, 
       9.3317896251426767e-05, 
       8.8895642732941612e-05, 
       1.0162880199726895e-04, 
       7.9730346619432860e-05, 
       7.7966020727073871e-05, 
       7.5735647743168299e-05, 
       7.6429820632332321e-05, 
       1.0287132953243190e-04, 
       1.0380151677870940e-04, 
       1.0017464756616927e-04, 
       7.9999999999996646e-05, 
       8.8230286362698623e-05, 
       7.5661582392445379e-05, 
       1.0506710316707535e-04, 
       7.6738211588522158e-05, 
       1.0537354304223342e-04, 
       9.5502279778848141e-05, 
       8.1218815848468624e-05, 
       7.5366017361578195e-05, 
       7.4423303316714358e-05, 
       1.0483320576757968e-04, 
       7.4389955013166905e-05, 
       7.5840005617816426e-05, 
       7.6734307329256053e-05, 
       7.5727824581932156e-05, 
       1.0350317149136423e-04, 
       8.6966750882960743e-05, 
       9.3870636148536010e-05, 
       8.9312029262868700e-05, 
       8.4390949977764025e-05, 
       7.4278930589606739e-05, 
       9.6512527911488111e-05, 
       7.4154128754928101e-05, 
       7.7655812288651517e-05, 
       1.0286907776116645e-04, 
       1.0343697252000206e-04, 
       8.6228696586167710e-05, 
       8.5572049362252905e-05, 
       8.3904047023546598e-05, 
       1.0179459697039431e-04, 
       8.9016268345203483e-05, 
       9.9558843251468853e-05, 
       7.6000548394638433e-05, 
       7.3211179323460894e-05, 
       1.0294318674654857e-04, 
       1.0286615917185163e-04, 
       1.0162985949567205e-04, 
       8.7546704778422680e-05, 
       8.6164166721760784e-05, 
       9.9774097554845140e-05, 
       8.1714722077438990e-05, 
       1.0073090849571845e-04, 
       8.0349657037898887e-05, 
       8.0332281060801548e-05, 
       8.4878596453155566e-05, 
       1.0239093399838701e-04, 
       7.6685599993349466e-05, 
       7.6673263168761430e-05, 
       9.8244827391186885e-05, 
       1.0178196647139997e-04, 
       1.0179672767107139e-04, 
       7.3411521523460003e-05, 
       8.4854367101241479e-05, 
       7.2247389904331249e-05, 
       7.2586099024853055e-05, 
       1.0062959282437415e-04, 
       9.6070158464693373e-05, 
       1.0057105244584317e-04, 
       7.5843537765876018e-05, 
       8.7795311963420261e-05, 
       9.2599101466128955e-05, 
       8.4224632681681580e-05, 
       8.5195792862301475e-05, 
       7.2628150845082163e-05, 
       7.5415014946150270e-05, 
       1.0069621296776179e-04, 
       7.5145697462586790e-05, 
       9.4319717602733870e-05, 
       7.1647255648673130e-05, 
       7.5388134286489715e-05, 
       8.9005050774863446e-05, 
       7.1161584355092179e-05, 
       7.0953088056664590e-05, 
       7.0953088056664590e-05, 
       9.4311546715269355e-05, 
       1.0000000000000141e-04, 
       9.9999999999997647e-05, 
       9.5699383237257270e-05, 
       1.0000000000000427e-04, 
       1.0000000000000053e-04, 
       1.0000000000000341e-04, 
       9.9999999999996752e-05, 
       1.0000000000000297e-04, 
       1.0000000000000070e-04, 
       7.4146819782277087e-05, 
       9.9703495667117354e-05, 
       8.1464730418087807e-05, 
       9.7331424747502341e-05, 
       9.7331424747502341e-05, 
       8.0048655973706184e-05, 
       9.3480151763516690e-05, 
       8.1934800861325449e-05, 
       7.4422581191619977e-05, 
       7.1893852149892043e-05, 
       7.2862131336357596e-05, 
       7.1884501972677378e-05, 
       7.9778861206226335e-05, 
       7.2926842917400131e-05, 
       7.2926842917400131e-05, 
       9.7471854532309716e-05, 
       9.8080267013623746e-05, 
       8.3603059033530118e-05, 
       7.9497944942830744e-05, 
       7.2550369392349840e-05, 
       7.1465323252195269e-05, 
       9.1190481756276459e-05, 
       8.1743171367494244e-05, 
       8.1139721848425702e-05, 
       8.1688674650823659e-05, 
       7.2529679544255335e-05, 
       9.7094250011530917e-05, 
       7.9502988976741699e-05, 
       8.1299728503353869e-05, 
       9.0436037884926559e-05, 
       9.4020228180045088e-05, 
       7.3112487755776605e-05, 
       8.4415530883770983e-05, 
       7.1977926524152626e-05, 
       7.6855392126411998e-05, 
       8.9999999999999233e-05, 
       9.0000000000003678e-05, 
       7.2533271211803251e-05, 
       9.7245830656222566e-05, 
       8.3761347872436233e-05, 
       7.8092171037543636e-05, 
       7.7697550788231943e-05, 
       7.1550289932836533e-05, 
       8.1175009592478309e-05, 
       7.1955445493150587e-05, 
       8.3826263947488899e-05, 
       8.5729414802535750e-05, 
       7.2421866503454319e-05, 
       6.9383606549503288e-05, 
       8.6927614155445348e-05, 
       6.9382770287509417e-05, 
       8.5587289252643016e-05, 
       6.9326960789822397e-05, 
       9.2480855525770190e-05, 
       9.5728700169687869e-05, 
       8.0256883662350377e-05, 
       7.0070544836900786e-05, 
       9.4792870542342609e-05, 
       7.0366813033908153e-05, 
       8.1084415168745192e-05, 
       7.9735025743089905e-05, 
       9.1792997090828920e-05, 
       9.1053232063024393e-05, 
       9.5913061667411793e-05, 
       9.4042946916507789e-05, 
       6.7999276604758626e-05, 
       6.8474425480938143e-05, 
       7.9654938624268919e-05, 
       9.3723815080277940e-05, 
       9.5676534965772713e-05, 
       7.0388293468966594e-05, 
       8.2057384754935232e-05, 
       7.8161830877482674e-05, 
       8.2485753408697050e-05, 
       7.1370583321989726e-05, 
       6.8139088835959957e-05, 
       8.1552696896240850e-05, 
       8.5492938033150627e-05, 
       9.0907206423492112e-05, 
       9.2454676495149401e-05, 
       8.5717644317623824e-05, 
       7.9182735659086317e-05, 
       7.6144075259945352e-05, 
       9.4419891031294949e-05, 
       7.4250656158255553e-05, 
       6.9017418205090112e-05, 
       7.8887746098952089e-05, 
       6.9987379738301560e-05, 
       7.8002321254740586e-05, 
       6.6780561010405213e-05, 
       6.7186243639152869e-05, 
       7.2596467247276116e-05, 
       7.2596467247276116e-05, 
       6.7846243466845006e-05, 
       8.3891472763001937e-05, 
       9.0009321005519189e-05, 
       8.4277740342411772e-05, 
       6.9604237560139752e-05, 
       7.8196358749678193e-05, 
       9.3555487671977190e-05, 
       9.3640245851003744e-05, 
       8.7343269535561914e-05, 
       8.9484190877531987e-05, 
       7.3388374318265873e-05, 
       7.7676712477864107e-05, 
       6.8258486742265440e-05, 
       8.3337382537409618e-05, 
       8.8709501636487510e-05, 
       8.8709501636487510e-05, 
       9.1303188412297879e-05, 
       7.9082190066726991e-05, 
       9.2875913713771251e-05, 
       7.7332782299417965e-05, 
       7.9762421725201906e-05, 
       6.9257537033323008e-05, 
       7.7179975395809180e-05, 
       9.0002352116730071e-05, 
       9.0588739338419080e-05, 
       7.6806684541722348e-05, 
       6.9999999999998598e-05, 
       9.1618809721288733e-05, 
       6.8188990003593256e-05, 
       6.6063470530351045e-05, 
       7.0435074173813154e-05, 
       6.6215507826655241e-05, 
       6.6329897123408609e-05, 
       8.8424974330396480e-05, 
       6.7414583752201012e-05, 
       7.6363512437986278e-05, 
       9.1541920431189759e-05, 
       6.5131901563241865e-05, 
       6.5396030503433271e-05, 
       8.2846599380160412e-05, 
       6.5041196519487006e-05, 
       8.9213122415064103e-05, 
       6.5974163403416180e-05, 
       6.5864820262920133e-05, 
       6.7921683149407529e-05, 
       6.8530998012025638e-05, 
       8.2770997109351218e-05, 
       6.5098014267949214e-05, 
       9.1350761424749015e-05, 
       7.6120481420064928e-05, 
       8.2908977760718256e-05, 
       9.0914590593641081e-05, 
       6.4900782719687857e-05, 
       7.1398124834366351e-05, 
       6.6544306845790373e-05, 
       6.6899491668431904e-05, 
       7.2410783136780393e-05, 
       6.8174285466066463e-05, 
       8.7849265423576650e-05, 
       7.7663439392540613e-05, 
       7.5680774547335294e-05, 
       6.4872847496756528e-05, 
       7.6492434768168528e-05, 
       6.5281983308523772e-05, 
       9.0613840793301310e-05, 
       6.5327074988789341e-05, 
       6.4080288894462534e-05, 
       8.7233908758987938e-05, 
       8.6149437497267124e-05, 
       8.9999950570625675e-05, 
       7.7520724294355493e-05, 
       7.2562565519166720e-05, 
       6.7347704311890633e-05, 
       6.3916292012603852e-05, 
       8.9999999999995655e-05, 
       6.5920428750565452e-05, 
       8.7265413886742531e-05, 
       7.4076402830600799e-05, 
       6.4048238597748757e-05, 
       7.4532006012174019e-05, 
       8.3297140690299592e-05, 
       6.3821470204878408e-05, 
       8.7813948101613936e-05, 
       6.3209824057161115e-05, 
       6.8699214171282828e-05, 
       7.3762965169588650e-05, 
       8.8304517909717296e-05, 
       6.4051282924345180e-05, 
       8.3818350172659210e-05, 
       6.5549520568470768e-05, 
       6.2896412352349629e-05, 
       8.1645689767005552e-05, 
       6.2966309390052931e-05, 
       7.3252546280949007e-05, 
       6.3550976139069816e-05, 
       7.1260404003721731e-05, 
       8.5677888662320353e-05, 
       6.9588829920697972e-05, 
       6.5129804274759116e-05, 
       8.8599828855079199e-05, 
       6.7572204708764557e-05, 
       6.9038264025421601e-05, 
       8.5538617412263886e-05, 
       7.3874800587391456e-05, 
       7.4760276545282980e-05, 
       6.6276278488031911e-05, 
       8.4836322904792652e-05, 
       7.1497038929578213e-05, 
       8.4470504821376104e-05, 
       8.4867081212468593e-05, 
       7.2701087783134095e-05, 
       7.1910884467673791e-05, 
       7.1743269682464302e-05, 
       7.7662419315387821e-05, 
       6.5606467637641930e-05, 
       6.3676682746692588e-05, 
       8.1808987517328029e-05, 
       7.1803551197664121e-05, 
       6.3452453424297798e-05, 
       6.8588143177559274e-05, 
       6.4100516295068194e-05, 
       8.3177954187319727e-05, 
       7.3686164391551504e-05, 
       8.0000000000002866e-05, 
       7.3554524191097505e-05, 
       7.1268949777913092e-05, 
       8.6611142649908326e-05, 
       6.5184004601355258e-05, 
       8.2318628494050525e-05, 
       6.1349598018708282e-05, 
       8.6424304151061092e-05, 
       6.8265338605440031e-05, 
       7.1952145251717722e-05, 
       7.1878138642422702e-05, 
       8.5795819024782972e-05, 
       7.2190925825553915e-05, 
       6.1653385830927551e-05, 
       8.2606506546626261e-05, 
       6.4229169789835415e-05, 
       7.0770406394965791e-05, 
       7.2142130031235347e-05, 
       6.3557850259528639e-05, 
       6.4276263799240176e-05, 
       7.4024811178821279e-05, 
       6.2182150740272473e-05, 
       6.5366408238848363e-05, 
       6.2380054834357184e-05, 
       6.3929074551628953e-05, 
       6.5364505368102637e-05, 
       6.5356092432141181e-05, 
       8.1757751016047809e-05, 
       6.3820950798328710e-05, 
       8.5219313226147519e-05, 
       6.6919057255288334e-05, 
       6.6210452449539278e-05, 
       6.1875966647146984e-05, 
       6.2656048915290402e-05, 
       6.9455799336008616e-05, 
       6.3509142853422293e-05, 
       8.4165599336841567e-05, 
       6.5837873033036814e-05, 
       7.0334436211899184e-05, 
       6.5236523031053658e-05, 
       6.0099663041440669e-05, 
       7.0375958498903229e-05, 
       8.4534356391229598e-05, 
       5.9978676897205998e-05, 
       6.9352405675766616e-05, 
       8.2129373715994958e-05, 
       6.9250354323117412e-05, 
       8.3618278997218935e-05, 
       8.0262838317054431e-05, 
       6.2924942174253610e-05, 
       8.0056477126931925e-05, 
       6.5357253986208318e-05, 
       5.9785902636241289e-05, 
       6.6188061200899868e-05, 
       7.1789604838212270e-05, 
       6.6463620188607816e-05, 
       8.0000000000003124e-05, 
       5.9494194718290323e-05, 
       6.9269312237852683e-05, 
       6.2839486830890053e-05, 
       6.9892470254323018e-05, 
       6.4379924004785054e-05, 
       7.0000000000003775e-05, 
       6.3807666750067207e-05, 
       6.3807666750067207e-05, 
       7.9095219842891071e-05, 
       8.2064661199463269e-05, 
       8.2064661199463269e-05, 
       7.9379102557832601e-05, 
       8.2816845401768909e-05, 
       6.2508309958354602e-05, 
       7.3492051927928490e-05, 
       6.2781101965321787e-05, 
       6.8974160796969741e-05, 
       8.2294074131408675e-05, 
       5.8776261333645033e-05, 
       6.3824931788326081e-05, 
       8.0863470241769597e-05, 
       5.9431122093465811e-05, 
       7.1788642816617357e-05, 
       6.1675079650201726e-05, 
       7.5470561541733706e-05, 
       6.1744713543348279e-05, 
       6.8335733149314020e-05, 
       7.6288066887247814e-05, 
       7.1010012696606569e-05, 
       8.1997524637719208e-05, 
       5.8366983809388220e-05, 
       5.8366983809388220e-05, 
       6.8232768989836943e-05, 
       8.1813898007016036e-05, 
       6.1331950040077464e-05, 
       6.1205225652951112e-05, 
       6.3485478666154110e-05, 
       6.8544802861219726e-05, 
       8.1970632755317031e-05, 
       8.1032255079964386e-05, 
       6.6961079287711025e-05, 
       7.0905303170765307e-05, 
       7.8662426845532715e-05, 
       6.7102272783884330e-05, 
       6.9982376783404344e-05, 
       5.7975696182506025e-05, 
       6.3022708218244474e-05, 
       7.5898089094974302e-05, 
       8.1394156946261678e-05, 
       7.9051458386031077e-05, 
       5.7792895408829065e-05, 
       6.7118288969200677e-05, 
       6.7113672802829189e-05, 
       5.8487350733031008e-05, 
       5.9060272254824622e-05, 
       6.6685573746479691e-05, 
       8.0884363748094789e-05, 
       6.3406352412302854e-05, 
       7.9168902861447130e-05, 
       6.8101999415719065e-05, 
       6.9999999999997649e-05, 
       7.9756846428443462e-05, 
       5.9466997906205094e-05, 
       6.1798035990260014e-05, 
       5.7525640937074250e-05, 
       6.7598303827360432e-05, 
       8.1123107197590081e-05, 
       6.0857602220587515e-05, 
       5.7238536089913648e-05, 
       6.0296108823403312e-05, 
       8.0462708232992357e-05, 
       5.7138218636255578e-05, 
       6.6744343168005023e-05, 
       6.9861326793764752e-05, 
       6.6448058852273747e-05, 
       8.0590379451377070e-05, 
       5.9301420977033479e-05, 
       6.3606252338734910e-05, 
       7.1561444912891957e-05, 
       5.8971531988847866e-05, 
       5.9962708678099615e-05, 
       5.8424391314053509e-05, 
       7.5044372478503521e-05, 
       5.6640882457554580e-05, 
       6.0634628650502093e-05, 
       7.9999999999997811e-05, 
       8.0000000000003124e-05, 
       7.9999999999998041e-05, 
       5.8762804664599435e-05, 
       6.8300553729258890e-05, 
       7.9756170013425904e-05, 
       7.6111421953117375e-05, 
       6.6791153639730893e-05, 
       5.8711633022558357e-05, 
       7.7190981078900111e-05, 
       6.7565975060514434e-05, 
       7.4409631614740109e-05, 
       6.8019637361911608e-05, 
       6.0000638740918991e-05, 
       5.9683060808299142e-05, 
       6.0000000000001716e-05, 
       6.0656444536213920e-05, 
       6.3315654994675305e-05, 
       7.9117751964661310e-05, 
       6.2847507467736645e-05, 
       6.1127345864194760e-05, 
       7.7953012488288023e-05, 
       6.2726754227690335e-05, 
       6.4457252889868805e-05, 
       7.0603219934318304e-05, 
       7.1793466863569297e-05, 
       6.6246300211220808e-05, 
       6.4800508456150708e-05, 
       5.6820814389342633e-05, 
       7.8429482158662094e-05, 
       6.4466112565549793e-05, 
       5.9425728391087322e-05, 
       7.7760116444469343e-05, 
       6.0573059801935110e-05, 
       6.7982975409914406e-05, 
       7.6974700191106334e-05, 
       7.0682061975435999e-05, 
       5.8324262355301776e-05, 
       5.6861440151489735e-05, 
       5.9520686541217098e-05, 
       7.7808551080479926e-05, 
       7.3014516071128041e-05, 
       7.0318610983867639e-05, 
       5.8341079890452817e-05, 
       5.8038404263407569e-05, 
       6.4192756093646431e-05, 
       5.5722688434389153e-05, 
       6.0917992163542404e-05, 
       6.5511922452528984e-05, 
       6.7905103955939297e-05, 
       6.0887668518765243e-05, 
       5.6455953970968241e-05, 
       6.4089790272093156e-05, 
       6.0166703378302190e-05, 
       6.0349615780373958e-05, 
       5.6531897328182399e-05, 
       5.5284361232208792e-05, 
       6.9157879984307118e-05, 
       7.7250984899385485e-05, 
       5.5904387553900388e-05, 
       5.7329476689326737e-05, 
       7.7177017699276516e-05, 
       6.3677988053574579e-05, 
       5.6657218670243469e-05, 
       6.1500274095199829e-05, 
       6.3525714763286182e-05, 
       6.4047466377773604e-05, 
       6.8746615557381972e-05, 
       7.6796009738558499e-05, 
       5.7960663415198631e-05, 
       5.6616898916124508e-05, 
       5.8507179406186414e-05, 
       5.8507179406186414e-05, 
       6.4286001971403442e-05, 
       6.3426068153556473e-05, 
       5.7427829944274808e-05, 
       6.9482822627741069e-05, 
       5.6218833671767182e-05, 
       5.6244821096145556e-05, 
       7.6559378599703979e-05, 
       5.9882412663587851e-05, 
       7.1769351834248960e-05, 
       6.3528202858453226e-05, 
       6.3308141831010801e-05, 
       5.9748704434500374e-05, 
       6.3407501594750623e-05, 
       5.5297201724374376e-05, 
       5.5294165843614736e-05, 
       5.3890286553868552e-05, 
       6.2827180672443735e-05, 
       6.2672038745459056e-05, 
       6.7627502188829497e-05, 
       6.9999999999996646e-05, 
       6.9999999999996646e-05, 
       7.0000000000003233e-05, 
       6.9999999999996511e-05, 
       6.9999999999999316e-05, 
       6.2931919908768894e-05, 
       5.6019709267986371e-05, 
       7.2411777775890373e-05, 
       5.3893704701277225e-05, 
       5.4919084356384583e-05, 
       7.2443204584907620e-05, 
       7.5700210033304168e-05, 
       5.4039625586445665e-05, 
       6.0057962594511456e-05, 
       6.2994190541505941e-05, 
       5.6553487353165438e-05, 
       7.0416897561046345e-05, 
       6.9018146650023175e-05, 
       5.4298339545751332e-05, 
       5.7566113457668347e-05, 
       5.7566113457668347e-05, 
       5.3896434824613023e-05, 
       6.2324006726300040e-05, 
       5.6245938035163895e-05, 
       5.6262876609590483e-05, 
       6.2064385542956658e-05, 
       7.3325508121557261e-05, 
       5.5336294369211569e-05, 
       5.4688918171822151e-05, 
       6.3016566473149727e-05, 
       7.4044467066409106e-05, 
       5.3847918139291220e-05, 
       6.1608144883388853e-05, 
       7.3829185564994795e-05, 
       6.1005953027268596e-05, 
       7.4563937119334210e-05, 
       6.1510643188956386e-05, 
       5.5585613459319514e-05, 
       5.3127657690257386e-05, 
       5.6854379222837663e-05, 
       7.4175647619881930e-05, 
       6.5352207425744773e-05, 
       5.6796512516564551e-05, 
       7.4141386918140235e-05, 
       5.9204649968848753e-05, 
       5.9953192340170565e-05, 
       5.6410428452317926e-05, 
       7.3611779026043771e-05, 
       7.3702367280026061e-05, 
       6.4411599407668551e-05, 
       6.3237997602772381e-05, 
       6.1536848685836765e-05, 
       5.2594819126674162e-05, 
       7.3546635520325198e-05, 
       5.2931680708235823e-05, 
       5.2434336116942445e-05, 
       7.3862602501210007e-05, 
       6.2123229596222678e-05, 
       6.1848973622720174e-05, 
       6.3386964907555081e-05, 
       5.4880455407284498e-05, 
       6.9170969299737822e-05, 
       7.0824123912965892e-05, 
       6.5608553700060505e-05, 
       6.9475698371029295e-05, 
       6.2070855503402149e-05, 
       5.4079809617853081e-05, 
       6.3769022162777749e-05, 
       6.2041610638332377e-05, 
       7.3240232758743496e-05, 
       7.2387522457684276e-05, 
       5.4112252607867139e-05, 
       5.9938290045320255e-05, 
       6.1265594645847256e-05, 
       7.2902188087098116e-05, 
       5.1714355001555184e-05, 
       6.1534209538043489e-05, 
       5.1599521823857750e-05, 
       6.8392806341544348e-05, 
       6.8557027230420028e-05, 
       5.9928672066695453e-05, 
       6.2223668755738873e-05, 
       6.0284898499818994e-05, 
       6.0647853444238224e-05, 
       5.4503511645400286e-05, 
       6.1100329371434364e-05, 
       5.9977443388786832e-05, 
       5.7196899156338021e-05, 
       5.7246132412459700e-05, 
       6.0812071135376466e-05, 
       6.0147509446240176e-05, 
       5.8028346182608723e-05, 
       5.8200288752660364e-05, 
       7.0392074591777795e-05, 
       5.4374901298990524e-05, 
       6.8852269191558974e-05, 
       6.8363485470761663e-05, 
       5.3203764216720171e-05, 
       6.8735722966409624e-05, 
       6.0113304328114580e-05, 
       7.1798976822536655e-05, 
       5.5434889335857634e-05, 
       5.8057602646373637e-05, 
       6.0236948424941417e-05, 
       5.7334825816418568e-05, 
       5.9584571208184333e-05, 
       5.3882311409436307e-05, 
       5.7975696182496064e-05, 
       5.8642811210191921e-05, 
       5.1021322393804143e-05, 
       5.7375966704019330e-05, 
       5.2165052232240396e-05, 
       7.0405629308618333e-05, 
       5.5814974730323309e-05, 
       6.5115786385638128e-05, 
       6.3191980801011519e-05, 
       6.8302492030051913e-05, 
       5.2946253100227508e-05, 
       6.9864399118165513e-05, 
       5.3519200528919703e-05, 
       6.0916619645259132e-05, 
       5.3353886478213653e-05, 
       5.1644057940744707e-05, 
       5.3311394153092751e-05, 
       5.0602562952945810e-05, 
       7.0888653660611630e-05, 
       6.9937350784747459e-05, 
       5.1664075736193493e-05, 
       5.3144158950403131e-05, 
       5.2999415799077513e-05, 
       6.4934755082764894e-05, 
       5.2657387756615676e-05, 
       5.2653647353310184e-05, 
       5.1741199062695796e-05, 
       5.2653886622445735e-05, 
       5.6413937181778075e-05, 
       4.9834537437369338e-05, 
       5.8527060264594480e-05, 
       5.2784083479654844e-05, 
       4.9999999999996181e-05, 
       6.8656869243822620e-05, 
       5.4785954426778778e-05, 
       5.7572845968130187e-05, 
       5.1307851903693580e-05, 
       7.0000000000004656e-05, 
       6.9999999999996253e-05, 
       7.0000000000003816e-05, 
       6.9999999999997053e-05, 
       7.0000000000001241e-05, 
       7.0000000000004182e-05, 
       7.0000000000001634e-05, 
       5.8100337514727283e-05, 
       5.0617390845403608e-05, 
       5.7077093981928396e-05, 
       6.6510501506377508e-05, 
       6.8447159918870196e-05, 
       6.8447159918870196e-05, 
       4.9998744397459931e-05, 
       5.9054110827512336e-05, 
       5.9054110827512336e-05, 
       4.9389018096962586e-05, 
       4.9469735152469814e-05, 
       5.0342980099344789e-05, 
       5.4096185903121902e-05, 
       6.8685210102147794e-05, 
       6.0739405471969981e-05, 
       5.0429677673274736e-05, 
       5.1596283397158637e-05, 
       6.9249303104309163e-05, 
       5.8668153290639881e-05, 
       6.5011050600263540e-05, 
       5.7681362495392439e-05, 
       5.1684670303987461e-05, 
       5.4190116214411712e-05, 
       6.2124236458825180e-05, 
       5.9026051417254481e-05, 
       6.7160770062276668e-05, 
       4.9216667694980751e-05, 
       6.7431874955774484e-05, 
       6.8571958268171145e-05, 
       5.0168777849553552e-05, 
       5.1240079225168537e-05, 
       5.2522492438199609e-05, 
       6.4444120753412486e-05, 
       5.1267950200395691e-05, 
       6.8552900247067109e-05, 
       5.5656780677968868e-05, 
       5.5645455789924967e-05, 
       6.6998902364645202e-05, 
       6.6106509933870419e-05, 
       4.8621134524977781e-05, 
       5.8327317960217514e-05, 
       6.8301317710511161e-05, 
       5.7739296754118536e-05, 
       5.6733561854340648e-05, 
       5.1907945724487169e-05, 
       6.7283603364102861e-05, 
       5.6086116077882290e-05, 
       5.3779628583649319e-05, 
       5.3773679883132576e-05, 
       5.7686250100676412e-05, 
       5.0756612126910737e-05, 
       5.9319218617168347e-05, 
       5.0996911751667910e-05, 
       5.1141434156633527e-05, 
       4.8232975294252025e-05, 
       6.7428905515221459e-05, 
       5.8075042220353883e-05, 
       5.0432445481483273e-05, 
       4.9723118600349544e-05, 
       6.7183023427300041e-05, 
       5.4418520495354878e-05, 
       5.1443190104135106e-05, 
       6.6479891987814772e-05, 
       4.8154488972058231e-05, 
       5.5414685218746776e-05, 
       5.8494536689008982e-05, 
       5.8214610480763870e-05, 
       6.1003932903509196e-05, 
       5.3989681249441327e-05, 
       5.4586888863709764e-05, 
       5.8011255465076777e-05, 
       4.9364794583557686e-05, 
       5.1588024520070403e-05, 
       5.0856798221431948e-05, 
       6.2523876992712051e-05, 
       4.7506758944440729e-05, 
       4.9736843766018602e-05, 
       4.8065538513162212e-05, 
       4.7843284337940138e-05, 
       6.6473967767005488e-05, 
       5.2119362124862120e-05, 
       5.5300147961083892e-05, 
       6.6494360747320299e-05, 
       6.6335737702639760e-05, 
       5.7565817985701589e-05, 
       4.8227124170123190e-05, 
       4.7959157223835262e-05, 
       5.5645120143779840e-05, 
       4.7924452431526926e-05, 
       4.7423306455270235e-05, 
       4.7850512381542845e-05, 
       5.3317661523466849e-05, 
       4.6999213151127284e-05, 
       6.5821931330637358e-05, 
       4.7430464499435272e-05, 
       4.6749555086998883e-05, 
       6.4608627553019386e-05, 
       4.8817519166752012e-05, 
       4.9658352765876390e-05, 
       5.7645430296232884e-05, 
       4.6513084145452088e-05, 
       4.6838328945944628e-05, 
       4.7518495244190778e-05, 
       4.7618079388639201e-05, 
       5.0937290716147252e-05, 
       4.6362387121220958e-05, 
       5.3112616514234598e-05, 
       4.7879863419670559e-05, 
       4.9802434052588395e-05, 
       6.4047065533614714e-05, 
       5.9880919513138390e-05, 
       4.9348575576909331e-05, 
       6.3797636223218260e-05, 
       5.3536503033076941e-05, 
       5.9634135939101574e-05, 
       5.3097350538854118e-05, 
       5.9999999999996844e-05, 
       5.4109364968479185e-05, 
       6.5108120875489320e-05, 
       5.9999999999998911e-05, 
       5.9999999999999168e-05, 
       5.9999999999995238e-05, 
       4.8253657136784224e-05, 
       4.6109176409349282e-05, 
       5.3323032427169881e-05, 
       4.9999999999997468e-05, 
       5.7746632105877832e-05, 
       4.8377566650179844e-05, 
       5.4019740675487589e-05, 
       5.4000527635045577e-05, 
       4.6984094691538928e-05, 
       5.5319634681187180e-05, 
       5.3399425666767305e-05, 
       4.8307957224223141e-05, 
       4.7218545037903774e-05, 
       5.0631322423297977e-05, 
       5.0624074626294180e-05, 
       4.6850544447525787e-05, 
       4.8097482961055307e-05, 
       4.8765284896953920e-05, 
       4.7788423942490573e-05, 
       5.7455590908959220e-05, 
       4.6541788185137160e-05, 
       6.2021619410128123e-05, 
       4.6007769183253639e-05, 
       6.3937193326120650e-05, 
       4.7630012076023683e-05, 
       6.4196456516951337e-05, 
       5.3232603550631491e-05, 
       6.4087197217880954e-05, 
       5.0976349119103704e-05, 
       4.8875673172377815e-05, 
       4.8200134931586160e-05, 
       5.0124516705383487e-05, 
       5.4866728612423982e-05, 
       5.4866728612423982e-05, 
       5.9687931585041279e-05, 
       4.9330618157882605e-05, 
       6.1015843689727393e-05, 
       5.2630713005819828e-05, 
       5.2470751223487250e-05, 
       5.1073463061904187e-05, 
       4.5159503742061857e-05, 
       5.6154056129171438e-05, 
       5.0465236947646229e-05, 
       5.8434605489657229e-05, 
       4.7082997248536849e-05, 
       5.2198062759797356e-05, 
       5.2923930366113863e-05, 
       4.6060975567785310e-05, 
       5.2841420701956051e-05, 
       6.2491473727720797e-05, 
       5.7672817453169587e-05, 
       4.9404427366540868e-05, 
       5.8078493652076541e-05, 
       5.5926724226921882e-05, 
       6.3285114004472097e-05, 
       6.1784187357510179e-05, 
       4.7228985500174511e-05, 
       4.7113106982599450e-05, 
       5.3329567001219627e-05, 
       5.1871969822082802e-05, 
       4.6806993844226478e-05, 
       4.5818422311003863e-05, 
       4.4764314160833883e-05, 
       4.6456947431001861e-05, 
       6.0540078197131089e-05, 
       4.8129644499075344e-05, 
       6.2428807946059644e-05, 
       5.2720757616892463e-05, 
       5.1738068980384296e-05, 
       6.1358427502498385e-05, 
       6.0624179763475773e-05, 
       6.1021659638661413e-05, 
       6.0452373358306819e-05, 
       4.4438743967969367e-05, 
       4.7752626282834979e-05, 
       5.9874578568506739e-05, 
       5.8858099541360165e-05, 
       4.9339661552883210e-05, 
       6.0290955447593425e-05, 
       6.0309509062758366e-05, 
       4.5585460009912667e-05, 
       6.2440432538608045e-05, 
       5.3620357713741222e-05, 
       6.2315065419913561e-05, 
       4.9233958178254318e-05, 
       6.0076291355083193e-05, 
       6.2128921490484584e-05, 
       4.7738966871028020e-05, 
       4.3988800867721600e-05, 
       4.6944177263479723e-05, 
       6.2120087390109697e-05, 
       4.7251966613467354e-05, 
       6.0194835224410237e-05, 
       6.0867259815089058e-05, 
       4.5229456315023511e-05, 
       4.5157064301636784e-05, 
       6.1696455166936233e-05, 
       5.3514139841055439e-05, 
       4.6897377446184051e-05, 
       5.2113667852258651e-05, 
       4.6065840048162590e-05, 
       6.0823189549657414e-05, 
       5.1847350888108032e-05, 
       4.6381100856351667e-05, 
       5.2265540931804702e-05, 
       4.5052626372544749e-05, 
       6.0440976732303250e-05, 
       4.4027916341510338e-05, 
       5.1235609203907210e-05, 
       5.8710147614769819e-05, 
       4.4934320706960839e-05, 
       4.8119410817917368e-05, 
       5.5641523552435012e-05, 
       5.1101042289822581e-05, 
       5.2398200281830565e-05, 
       5.2931680708258720e-05, 
       4.9768711920962139e-05, 
       5.9234396024083023e-05, 
       6.1209533575310674e-05, 
       5.9729449251660013e-05, 
       4.5709644395046836e-05, 
       5.3762909074445581e-05, 
       4.4386569573943469e-05, 
       5.2241653813088457e-05, 
       4.4194772821727407e-05, 
       4.4699757371715209e-05, 
       5.0497012037485661e-05, 
       4.5619810615109919e-05, 
       5.0614725124806547e-05, 
       4.3042300421031882e-05, 
       4.9263696735815763e-05, 
       4.8597369319794204e-05, 
       4.8321010540245448e-05, 
       5.0429327235622207e-05, 
       4.3781619241207234e-05, 
       4.3781619241207234e-05, 
       5.9283570582830531e-05, 
       5.0351183872548351e-05, 
       4.8380534997354775e-05, 
       5.9530333672816625e-05, 
       5.5180526868101322e-05, 
       4.9013062915240597e-05, 
       6.0391161831427170e-05, 
       4.4383839567870722e-05, 
       4.2940746688550135e-05, 
       4.3474940166716923e-05, 
       5.0114361240644861e-05, 
       4.5343776830733037e-05, 
       5.2233863204528452e-05, 
       6.0239894088239741e-05, 
       4.3602646585199418e-05, 
       5.9989974563351431e-05, 
       5.7236046982533619e-05, 
       4.5061389909857739e-05, 
       5.6680429292107721e-05, 
       4.2719784714483235e-05, 
       5.1847467408943452e-05, 
       6.0000000000003891e-05, 
       6.0000000000003186e-05, 
       5.9999999999997413e-05, 
       5.9999999999995699e-05, 
       5.9999999999995699e-05, 
       5.9999999999997074e-05, 
       6.0000000000002848e-05, 
       6.0000000000004203e-05, 
       5.9999999999900750e-05, 
       5.9948083493238083e-05, 
       5.1538275823445037e-05, 
       4.4255687506578091e-05, 
       5.9687917190791767e-05, 
       4.2385805422990165e-05, 
       5.2561308939517306e-05, 
       4.2989777661969736e-05, 
       5.8866115201010490e-05, 
       4.2585151442718195e-05, 
       5.7806800652732524e-05, 
       5.8538519496124681e-05, 
       4.7419101157292518e-05, 
       5.6969026218376831e-05, 
       4.2904941056530595e-05, 
       4.9468965792524989e-05, 
       4.2884982051612175e-05, 
       5.9418379806964409e-05, 
       4.8038549640988393e-05, 
       5.1413515079533608e-05, 
       5.6414144854206245e-05, 
       5.6698254084957370e-05, 
       5.6859133973873823e-05, 
       4.2238588668818394e-05, 
       5.0523661233750111e-05, 
       4.1859565026545551e-05, 
       4.2300453603142560e-05, 
       5.8357816852680722e-05, 
       4.2896636966194901e-05, 
       4.2890995066705979e-05, 
       4.3605593670430775e-05, 
       4.9078952402252231e-05, 
       4.2547482406794759e-05, 
       4.6021705635717682e-05, 
       5.7628952713722120e-05, 
       5.8899780859377870e-05, 
       4.4889126964744352e-05, 
       5.1167149777947389e-05, 
       4.9305332930358910e-05, 
       5.8059534537816362e-05, 
       5.2312466563373815e-05, 
       5.8505893198456180e-05, 
       5.8505619902993483e-05, 
       5.2485315478166272e-05, 
       4.7151073993626523e-05, 
       4.8947919031059924e-05, 
       5.8665593834410141e-05, 
       5.2712147925044101e-05, 
       5.0129217704798939e-05, 
       5.6163478919564309e-05, 
       4.9046925053080419e-05, 
       5.5347711415919464e-05, 
       4.7132881047904530e-05, 
       4.8136558642189787e-05, 
       4.3567204893986439e-05, 
       5.1005642257679023e-05, 
       4.8942600397635363e-05, 
       5.5565091641099407e-05, 
       5.3598968719123090e-05, 
       5.5623545885036493e-05, 
       4.7516740701371410e-05, 
       4.1400926093465427e-05, 
       4.6181922045644870e-05, 
       4.2158746517562697e-05, 
       4.4091303286996512e-05, 
       5.0933370057951013e-05, 
       4.5149131671974665e-05, 
       4.9185432993972400e-05, 
       4.2454629609651406e-05, 
       4.7020820823160696e-05, 
       4.3156904316442969e-05, 
       4.7217784961223787e-05, 
       4.2249633424153557e-05, 
       5.6559207837114271e-05, 
       4.4753813000411455e-05, 
       4.1942096590134538e-05, 
       4.4878995494326498e-05, 
       5.1082037872257545e-05, 
       4.1370599272740181e-05, 
       4.5124591943925040e-05, 
       5.7708008312763574e-05, 
       5.5049821223998331e-05, 
       4.5036977994987573e-05, 
       4.9956201747369401e-05, 
       5.0054996184009183e-05, 
       4.7978382753679209e-05, 
       4.7969420649407912e-05, 
       5.6401804604434095e-05, 
       4.7177734946549771e-05, 
       4.4469918696663561e-05, 
       4.2841763948950139e-05, 
       4.7915081188819933e-05, 
       5.6747801600031691e-05, 
       5.7131826878926684e-05, 
       4.5405496784351432e-05, 
       4.8625757495847274e-05, 
       4.9110659013475059e-05, 
       4.5443529919864918e-05, 
       5.6601929593734138e-05, 
       4.0449617628153772e-05, 
       4.1915434938599752e-05, 
       5.7004515463664237e-05, 
       5.5305299041923107e-05, 
       5.0300022670240464e-05, 
       4.7494894750542188e-05, 
       5.6302032533584690e-05, 
       5.6641529826804034e-05, 
       4.2285597655103947e-05, 
       4.4989890723973792e-05, 
       4.7160673033362612e-05, 
       5.1696973321681622e-05, 
       4.7739698725242445e-05, 
       4.7540566308481340e-05, 
       4.2743314203602834e-05, 
       4.0262983115625070e-05, 
       4.9386451377604352e-05, 
       4.0140837214534677e-05, 
       5.1454725847116135e-05, 
       4.2722855581471416e-05, 
       5.5838535488226382e-05, 
       4.9076297639682089e-05, 
       4.7441798013771622e-05, 
       4.5468000876504170e-05, 
       4.0045406906395053e-05, 
       4.8184698246698909e-05, 
       4.7098313696511857e-05, 
       4.0746250395595987e-05, 
       4.6636441632854370e-05, 
       4.8941346549806990e-05, 
       4.1910580657598172e-05, 
       3.9800093001322042e-05, 
       4.0877082420779330e-05, 
       4.1650704997486722e-05, 
       5.2622782291357465e-05, 
       3.9999999999998031e-05, 
       4.1034222836968937e-05, 
       4.3064016324120847e-05, 
       4.0626714689814220e-05, 
       4.3145765424439943e-05, 
       5.5687005889193095e-05, 
       4.0992368987462888e-05, 
       4.7778752397519893e-05, 
       4.7639866042398090e-05, 
       4.7873569030883523e-05, 
       5.3971749688348641e-05, 
       5.4411539910846220e-05, 
       4.1603394095032366e-05, 
       4.2833403445612206e-05, 
       5.3251208267986931e-05, 
       5.5025640024275558e-05, 
       4.0000000000004340e-05, 
       5.5494098897059034e-05, 
       4.6139240744064400e-05, 
       4.5576472255746050e-05, 
       4.1611469644164119e-05, 
       5.2205504717807511e-05, 
       5.0791540425267534e-05, 
       4.2424390204096938e-05, 
       4.4758167671899552e-05, 
       5.3117345831586268e-05, 
       4.2891161374582391e-05, 
       4.8004886649059284e-05, 
       4.8876657057956124e-05, 
       5.3566000052228521e-05, 
       5.4090499508276584e-05, 
       5.3379005510433491e-05, 
       3.9612001846744409e-05, 
       4.5551545149589090e-05, 
       4.4784831355082788e-05, 
       3.9108989711492461e-05, 
       4.0217885604607888e-05, 
       4.2391122145023589e-05, 
       4.2686663146066188e-05, 
       4.4476303476468326e-05, 
       4.0302573238981099e-05, 
       4.0481598641152580e-05, 
       5.4428501946752183e-05, 
       3.9532007940809496e-05, 
       3.8889093760938791e-05, 
       4.0969260975792362e-05, 
       4.6633414975083388e-05, 
       3.9362044300533755e-05, 
       3.9486078947276614e-05, 
       4.2462852854935404e-05, 
       4.0096948140222940e-05, 
       4.6705500429090406e-05, 
       5.1855634004080115e-05, 
       5.1252570390837694e-05, 
       5.4128198700289432e-05, 
       4.3943022516342919e-05, 
       4.2688429020360587e-05, 
       5.0290260734504572e-05, 
       4.9919596171507846e-05, 
       4.6782337339548067e-05, 
       4.6880460523404891e-05, 
       4.3731240612891365e-05, 
       4.1450286808754173e-05, 
       5.4503851490524109e-05, 
       4.7109779582227649e-05, 
       4.0870069770793465e-05, 
       4.2989358898683530e-05, 
       3.9119879430034020e-05, 
       5.0000000000001934e-05, 
       4.9999999999996181e-05, 
       4.9999999999996086e-05, 
       4.9999999999998091e-05, 
       3.9779839354899834e-05, 
       3.8423856351481847e-05, 
       5.2585713569939680e-05, 
       5.1355987448947586e-05, 
       5.3352343143186026e-05, 
       4.4293675235150309e-05, 
       5.3776890587359726e-05, 
       4.5340384718832403e-05, 
       4.5340384718832403e-05, 
       3.9596292637217605e-05, 
       3.8174969058300859e-05, 
       4.4748080799978564e-05, 
       4.1151237941040358e-05, 
       4.7755455282975009e-05, 
       4.6239999925812488e-05, 
       5.2473309178642368e-05, 
       3.9555565339275648e-05, 
       3.8142366730662753e-05, 
       3.8142366730662753e-05, 
       4.3576118269370213e-05, 
       4.4163713822123261e-05, 
       3.8519337872669541e-05, 
       4.7484368567342283e-05, 
       3.9723295223953362e-05, 
       4.4312333030711276e-05, 
       4.7326555183203576e-05, 
       4.3944811997648134e-05, 
       4.3533866174062588e-05, 
       4.1128293337820854e-05, 
       4.4264398543342156e-05, 
       4.5231235678463149e-05, 
       3.7974214225209884e-05, 
       4.0816353711432150e-05, 
       3.8254467210912872e-05, 
       3.9612293794954706e-05, 
       5.1289969748955931e-05, 
       3.8204355001303550e-05, 
       5.1429020707299141e-05, 
       4.3865029279998410e-05, 
       4.2589526891569486e-05, 
       4.4070767503813843e-05, 
       3.8557692981571585e-05, 
       5.1663683487387737e-05, 
       3.8367436677400796e-05, 
       5.0849740797423859e-05, 
       4.0000000000000830e-05, 
       5.0898469445921683e-05, 
       4.3902023613929762e-05, 
       3.9155327739404640e-05, 
       4.3718353705104801e-05, 
       4.4468686884197946e-05, 
       4.4468686884197946e-05, 
       5.1030169856757265e-05, 
       3.9999999999994413e-05, 
       4.0784369845711627e-05, 
       5.2589590232619284e-05, 
       4.4544976562195326e-05, 
       4.7145326972658432e-05, 
       4.4844963114920864e-05, 
       5.2120749894353478e-05, 
       4.3430922795922550e-05, 
       3.7242177299827923e-05, 
       4.2603831393923695e-05, 
       4.3810479862621382e-05, 
       5.2466497157731773e-05, 
       3.7256022446790273e-05, 
       4.2988144478824435e-05, 
       3.7193453687580723e-05, 
       4.3238059411697453e-05, 
       3.7016537498390827e-05, 
       5.2234932880688905e-05, 
       4.5267813064039461e-05, 
       4.4712517755245851e-05, 
       3.7000148022141180e-05, 
       4.7615779704774728e-05, 
       3.8886250063827352e-05, 
       3.6899640180051325e-05, 
       5.0782384036866160e-05, 
       4.4393359892921203e-05, 
       4.1135690655529002e-05, 
       4.9141935157055799e-05, 
       3.7329087888352542e-05, 
       5.1910726772086912e-05, 
       4.2882121064123977e-05, 
       4.9030536298102691e-05, 
       4.3371776100530395e-05, 
       3.9471983870651639e-05, 
       3.7349716501486679e-05, 
       5.1746950144655407e-05, 
       4.0394638474618180e-05, 
       4.6630404355709410e-05, 
       4.1512075312213028e-05, 
       4.1041243074259562e-05, 
       4.1470642074475918e-05, 
       4.0473054842158006e-05, 
       4.0635957593095788e-05, 
       4.2449354969984348e-05, 
       4.3989186256937562e-05, 
       3.6720931065664631e-05, 
       4.2209893969995383e-05, 
       3.9708942725034822e-05, 
       3.6534787555024640e-05, 
       5.0610504843381534e-05, 
       4.2978997488036201e-05, 
       4.9725059033144498e-05, 
       4.5539627719646152e-05, 
       4.8658601913449700e-05, 
       4.3662347596505869e-05, 
       4.5236029450620008e-05, 
       3.9999999999999956e-05, 
       3.6289585155460508e-05, 
       4.3123337913771949e-05, 
       4.2900455249023130e-05, 
       5.1022865399885616e-05, 
       5.1056897295305156e-05, 
       5.0201679932823080e-05, 
       4.5035755516383509e-05, 
       4.5742678592749948e-05, 
       3.6504866171755416e-05, 
       4.3696555871514647e-05, 
       3.6054507801335057e-05, 
       3.7956799054945251e-05, 
       4.5320516591022333e-05, 
       3.6023644937291720e-05, 
       3.8894940278020020e-05, 
       4.8243389571334880e-05, 
       4.7270104165723254e-05, 
       3.7872248784148562e-05, 
       3.5990844969634747e-05, 
       4.1400377003497217e-05, 
       5.0762440896385007e-05, 
       3.7387501437768796e-05, 
       5.0533824346019972e-05, 
       3.7636387073252496e-05, 
       3.7636387073252496e-05, 
       3.6930192577231878e-05, 
       3.7143969475281460e-05, 
       4.7936270874557962e-05, 
       4.2117955653727522e-05, 
       4.3294910049706541e-05, 
       4.9523858616957549e-05, 
       3.5788225525521872e-05, 
       3.5788225525521872e-05, 
       4.0998361333740045e-05, 
       3.9374118396399936e-05, 
       4.1967932715282660e-05, 
       4.0950171851870644e-05, 
       4.3502961137135996e-05, 
       4.6706950447247980e-05, 
       4.7003431790866724e-05, 
       5.0211960719528261e-05, 
       4.8496757365635496e-05, 
       4.9383251163863035e-05, 
       4.1782909511288393e-05, 
       4.5226377126599348e-05, 
       4.3331301718420514e-05, 
       3.5607520520066936e-05, 
       4.3584694709283523e-05, 
       4.1860315866986073e-05, 
       3.9464747440740120e-05, 
       4.2839836375224111e-05, 
       3.8053233704079882e-05, 
       4.3102572869320559e-05, 
       3.8356422874175486e-05, 
       3.6332911632079705e-05, 
       4.9502101644929617e-05, 
       3.5963125627094130e-05, 
       4.1222812894943381e-05, 
       4.3888207697742621e-05, 
       4.5787608236944982e-05, 
       4.8525729897417938e-05, 
       5.0000000000002923e-05, 
       4.9999999999997129e-05, 
       5.0000000000002157e-05, 
       5.0000000000003452e-05, 
       5.0000000000001378e-05, 
       4.9999999999998464e-05, 
       5.0000000000003363e-05, 
       4.9999999999999318e-05, 
       4.9999999999995395e-05, 
       4.9999999999995164e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       4.9633443533143397e-05, 
       3.6146040313024670e-05, 
       3.7186596895147424e-05, 
       4.1434351842659767e-05, 
       4.9671232623830673e-05, 
       4.1086755648298457e-05, 
       3.7176566014276475e-05, 
       3.6751636140254799e-05, 
       4.6944283266381702e-05, 
       3.7165895702074299e-05, 
       3.6225040908503689e-05, 
       3.8811100700933701e-05, 
       4.1518995666847782e-05, 
       3.6093810752093189e-05, 
       4.1260527577609158e-05, 
       3.6368625575969663e-05, 
       3.5601094917615441e-05, 
       4.9493889965562862e-05, 
       4.9390770584311340e-05, 
       4.9434877379427027e-05, 
       3.5811564067132317e-05, 
       3.6809148280972258e-05, 
       4.7011955014847065e-05, 
       3.4941977009063790e-05, 
       3.4892160456408059e-05, 
       4.2030988622430125e-05, 
       4.2713454725133798e-05, 
       4.0187044306381169e-05, 
       4.1743879995633237e-05, 
       4.9248450632993391e-05, 
       4.9166442089218142e-05, 
       3.4969556426075690e-05, 
       3.6282312082767726e-05, 
       4.8464322045901105e-05, 
       3.8432721766409586e-05, 
       3.5083913374717896e-05, 
       4.1023984914919550e-05, 
       3.5881594253143910e-05, 
       4.8187367855616964e-05, 
       4.8993859646157952e-05, 
       4.6568319271275242e-05, 
       4.2273591652999149e-05, 
       4.8957568324896678e-05, 
       4.8728911755346141e-05, 
       3.7413193024207677e-05, 
       4.0805710255231961e-05, 
       4.2296555324443978e-05, 
       4.3744170080442670e-05, 
       4.8864927515826213e-05, 
       3.8945859482717501e-05, 
       3.4758231754809261e-05, 
       4.7063404474244042e-05, 
       4.2123198552018641e-05, 
       4.0392903234596108e-05, 
       4.6872510806139494e-05, 
       3.8100115863702311e-05, 
       4.8168472029125805e-05, 
       4.7997072200865581e-05, 
       3.7972044715614297e-05, 
       3.7750564708231690e-05, 
       3.4485404745062308e-05, 
       4.6543856886749265e-05, 
       3.4735257490871966e-05, 
       4.8568518935912545e-05, 
       4.6055982656744297e-05, 
       3.5550093171139102e-05, 
       4.1774266234048137e-05, 
       4.0334001637208215e-05, 
       3.5363093834407096e-05, 
       4.0366359598685416e-05, 
       3.4330078960330751e-05, 
       3.4451720747244228e-05, 
       3.8066042689854233e-05, 
       4.1097989466049727e-05, 
       3.6082679688447050e-05, 
       4.5786774902315612e-05, 
       3.5985816837075656e-05, 
       3.4409089928700468e-05, 
       3.6868224318568522e-05, 
       3.4493094354991217e-05, 
       3.4529184531971749e-05, 
       3.5329038916482944e-05, 
       4.3836217151760351e-05, 
       3.4346868454266494e-05, 
       3.4274093961437654e-05, 
       3.9630524450406251e-05, 
       4.1735657518023995e-05, 
       3.6779927403012487e-05, 
       3.8593768385457307e-05, 
       4.4168135556270210e-05, 
       3.9486983893443966e-05, 
       3.9389217212448893e-05, 
       3.8558157525674599e-05, 
       4.4936088371861442e-05, 
       3.7402776845119888e-05, 
       3.4973124993109781e-05, 
       3.5649897995381909e-05, 
       3.5649897995381909e-05, 
       3.9216573613098059e-05, 
       4.7705200275961463e-05, 
       4.1237985527924675e-05, 
       3.3932771690846749e-05, 
       4.7721066134404328e-05, 
       3.3851426899466486e-05, 
       4.5638081995442594e-05, 
       4.0840257476572915e-05, 
       4.7423543505636740e-05, 
       3.5719111906935536e-05, 
       4.5572686613313965e-05, 
       3.4052247117402352e-05, 
       4.0740048496475884e-05, 
       3.7525217758159063e-05, 
       3.5476270735535963e-05, 
       3.4481818606858959e-05, 
       4.6015467757075992e-05, 
       3.7037340173965451e-05, 
       4.3411946184019808e-05, 
       4.0260813962040006e-05, 
       3.4055869172199889e-05, 
       4.0940109944184912e-05, 
       4.0940109944184912e-05, 
       3.4250637955032855e-05, 
       3.4817914977387234e-05, 
       3.6307606035348190e-05, 
       3.5179241929085079e-05, 
       3.9475707740318510e-05, 
       3.9148557941318213e-05, 
       3.6799820595621291e-05, 
       4.4967245762079095e-05, 
       3.5144825161120897e-05, 
       4.0422731609176954e-05, 
       3.7864172622580213e-05, 
       4.5196076099778753e-05, 
       3.3802472591938855e-05, 
       3.3802472591938855e-05, 
       3.6646806552042508e-05, 
       3.4276871995850190e-05, 
       3.3173960785885519e-05, 
       4.4449011968136945e-05, 
       3.4291182918036221e-05, 
       4.4732487582257194e-05, 
       3.6618552393547212e-05, 
       3.4152068050445495e-05, 
       3.7289155024135254e-05, 
       3.9921231757574604e-05, 
       3.4760153222000421e-05, 
       4.2280574376393759e-05, 
       4.3234377780913248e-05, 
       4.4650198205885767e-05, 
       4.2924765859740912e-05, 
       3.7764182225863115e-05, 
       3.7763227562940555e-05, 
       4.0132310108969856e-05, 
       3.3706501038521069e-05, 
       4.1401363697285896e-05, 
       3.9556663477278989e-05, 
       3.5941651724123704e-05, 
       3.5941651724123704e-05, 
       3.4210120966294029e-05, 
       3.8277022844147885e-05, 
       4.4908664840505224e-05, 
       3.4357778184269766e-05, 
       4.6262428798046453e-05, 
       3.4067672764753477e-05, 
       4.4839968556063865e-05, 
       3.8547866681690799e-05, 
       3.9595103411516685e-05, 
       4.6101807395534712e-05, 
       3.8194338408432130e-05, 
       3.9406142771192216e-05, 
       3.4246035292597692e-05, 
       3.8397194000903380e-05, 
       3.3358498559749076e-05, 
       4.4529473298277279e-05, 
       3.3555139690032741e-05, 
       3.9353938472215678e-05, 
       4.4409439316398738e-05, 
       3.8100405601044617e-05, 
       3.9901076985856095e-05, 
       4.5582072391166627e-05, 
       4.5373061124512591e-05, 
       3.8481683320743657e-05, 
       3.8882566019829942e-05, 
       3.2732554727724230e-05, 
       3.6919623440596352e-05, 
       3.3127811884610454e-05, 
       4.5605273648995629e-05, 
       3.6883096917962499e-05, 
       3.8646018510438268e-05, 
       3.3349414877125529e-05, 
       3.3139629391697061e-05, 
       4.2886603364193917e-05, 
       4.5576671685786118e-05, 
       4.5361457569649943e-05, 
       3.7885936919343399e-05, 
       4.4811888092210247e-05, 
       4.4698893327048875e-05, 
       4.3918202571323494e-05, 
       3.7062422667364702e-05, 
       3.8632060047219603e-05, 
       4.0744976649541705e-05, 
       4.4634977635134732e-05, 
       3.5299040778033262e-05, 
       3.8291727383125503e-05, 
       3.7770718044458303e-05, 
       3.9455274047740725e-05, 
       4.4232365071854907e-05, 
       4.1787579097588931e-05, 
       3.8775187271074794e-05, 
       3.7674431674787115e-05, 
       3.2294846835653616e-05, 
       3.7108330099575195e-05, 
       4.5119420341760923e-05, 
       3.4996037817289307e-05, 
       3.3394308306189422e-05, 
       3.5518565752892662e-05, 
       3.8482338553436338e-05, 
       3.2272639992713039e-05, 
       3.4088521844591359e-05, 
       3.5077080022841553e-05, 
       3.5075598899362149e-05, 
       3.3573211959297057e-05, 
       3.1973382332524500e-05, 
       3.5840891222916174e-05, 
       3.8751206026778844e-05, 
       3.8751206026778844e-05, 
       3.7086597111063709e-05, 
       3.3096259382174962e-05, 
       3.3838700250631010e-05, 
       3.2618847110974430e-05, 
       3.4020508225717068e-05, 
       3.1843818509819005e-05, 
       3.8390188135025377e-05, 
       3.8810010462808655e-05, 
       3.3211378175664576e-05, 
       3.4869311968063722e-05, 
       3.2032700395950697e-05, 
       3.7811234785441171e-05, 
       4.4649453583111601e-05, 
       3.3583031038668679e-05, 
       3.1655365245265213e-05, 
       3.9967497657719340e-05, 
       4.2289960354588734e-05, 
       3.1631018804853388e-05, 
       4.4374183229811822e-05, 
       3.1799366549007982e-05, 
       3.1729880444632586e-05, 
       4.4407619179603196e-05, 
       4.3963940690826286e-05, 
       3.1600386724480620e-05, 
       3.7125809542202758e-05, 
       3.2531686361602757e-05, 
       3.2969916162530642e-05, 
       3.2969916162530642e-05, 
       3.2701051266413326e-05, 
       3.3056203684438582e-05, 
       3.1463116866752515e-05, 
       4.2466996232677248e-05, 
       3.5437423146589721e-05, 
       3.6772774001921257e-05, 
       3.1581270747609690e-05, 
       4.4133679269790426e-05, 
       3.8478619970294991e-05, 
       3.7698901535583698e-05, 
       3.6744197880479884e-05, 
       3.8486937536895026e-05, 
       3.7899811440157475e-05, 
       3.6481118895287630e-05, 
       4.3145115509460633e-05, 
       3.5810792340679481e-05, 
       3.5807562073949786e-05, 
       3.7351548445723314e-05, 
       3.6161055734250827e-05, 
       3.3459078306163453e-05, 
       3.7758535206845361e-05, 
       3.5074324019369783e-05, 
       3.8777583458437893e-05, 
       3.5052550073536175e-05, 
       3.2657632475121690e-05, 
       3.1722550486931950e-05, 
       3.1777163109859709e-05, 
       3.1128393080790415e-05, 
       4.2972575006060915e-05, 
       3.1623936050455644e-05, 
       3.3211228967468389e-05, 
       3.2219112346808056e-05, 
       4.3344935254243846e-05, 
       3.0767142438753858e-05, 
       3.9246352493641287e-05, 
       3.6265805758314331e-05, 
       4.3436259932608941e-05, 
       3.2519086391907427e-05, 
       3.5019963142609861e-05, 
       3.5552732137505110e-05, 
       4.0000000000000044e-05, 
       4.0000000000003256e-05, 
       3.9999999999998695e-05, 
       3.9999999999998695e-05, 
       3.9999999999995877e-05, 
       3.9999999999996947e-05, 
       4.0000000000003710e-05, 
       3.9999999999998641e-05, 
       3.9999999999998323e-05, 
       4.0000000000004604e-05, 
       4.0000000000004604e-05, 
       3.1520603523588699e-05, 
       3.8187700027478257e-05, 
       3.0655504273065068e-05, 
       3.5114682601046063e-05, 
       3.5114682601046063e-05, 
       3.0629212305899648e-05, 
       3.9340365155687039e-05, 
       3.0790902821695978e-05, 
       3.5479215584076588e-05, 
       3.5476396478559203e-05, 
       3.0550624230616264e-05, 
       3.0638378345440583e-05, 
       4.2895442200886316e-05, 
       3.1446869249003625e-05, 
       3.5988637916796449e-05, 
       3.0544748877238458e-05, 
       3.2441108279527417e-05, 
       4.2814356416683900e-05, 
       3.0480827923530931e-05, 
       4.2960992784964117e-05, 
       3.1205634424314559e-05, 
       3.2176290434396502e-05, 
       3.4861292552849490e-05, 
       3.9792660334406046e-05, 
       3.9792660334406046e-05, 
       4.0067539615737803e-05, 
       3.1520225277737094e-05, 
       3.9344044811382328e-05, 
       3.9160383529734912e-05, 
       3.5589712608140480e-05, 
       4.2136885109624920e-05, 
       4.2136885109624920e-05, 
       3.5800516909935126e-05, 
       3.1545489344280527e-05, 
       3.2051367584966136e-05, 
       3.9090739088999569e-05, 
       3.1816945549504981e-05, 
       4.1451797449941164e-05, 
       3.5676432089641460e-05, 
       3.4040340607583295e-05, 
       3.0752560636817230e-05, 
       4.0326944325697221e-05, 
       3.0961311707883677e-05, 
       3.5421085552023147e-05, 
       3.1448708787215871e-05, 
       3.7149270370099857e-05, 
       3.5534674789581067e-05, 
       3.0917513880140062e-05, 
       4.1181329337049383e-05, 
       3.2561787529455299e-05, 
       4.2602634000447563e-05, 
       3.5526838153295525e-05, 
       3.1766301141713461e-05, 
       3.1138123142915250e-05, 
       3.5352084970560701e-05, 
       3.0283125677505168e-05, 
       3.1296910168382224e-05, 
       4.2402176308274449e-05, 
       3.1235925595150562e-05, 
       3.1783354659371787e-05, 
       3.0831423973898561e-05, 
       3.2623115391230380e-05, 
       3.1266830259563386e-05, 
       3.2623853756686830e-05, 
       4.0000000000004171e-05, 
       3.0151503090034851e-05, 
       3.4115561186289944e-05, 
       3.0700382999754308e-05, 
       3.1687830719248371e-05, 
       4.1811574531025101e-05, 
       3.9935185934137698e-05, 
       3.3883660673652234e-05, 
       3.0056033030738733e-05, 
       3.0071986333635376e-05, 
       3.2293030300190461e-05, 
       3.1105521348951953e-05, 
       3.1556442746839226e-05, 
       3.1556442746839226e-05, 
       3.0960588468513537e-05, 
       3.3228570021988799e-05, 
       3.4213533216578949e-05, 
       3.1390879861648162e-05, 
       3.4928221367676138e-05, 
       4.2082652052599186e-05, 
       3.1454256467664598e-05, 
       3.5133637921966394e-05, 
       3.4906351278678537e-05, 
       4.0289584855503186e-05, 
       3.0000000000005696e-05, 
       3.0000000000003257e-05, 
       3.2665592510237427e-05, 
       3.0000000000002850e-05, 
       3.2610327726377131e-05, 
       2.9999999999998537e-05, 
       2.9999999999998537e-05, 
       3.3836451523498569e-05, 
       4.0373708207542985e-05, 
       3.1953995716624574e-05, 
       4.1602820865319293e-05, 
       4.0606951685279347e-05, 
       3.5988849208438583e-05, 
       4.1799785873359888e-05, 
       3.0202547506420863e-05, 
       4.1701605606512434e-05, 
       3.3245080297128400e-05, 
       3.0363063426439208e-05, 
       3.8486875719931199e-05, 
       2.9650761490452748e-05, 
       3.0125359633885684e-05, 
       3.5334815406176788e-05, 
       3.7282687177737835e-05, 
       4.0279247069626156e-05, 
       3.9798040404030383e-05, 
       4.1588235054095805e-05, 
       3.1492990325673574e-05, 
       4.1160470578291326e-05, 
       3.6351931201530698e-05, 
       3.6716831478262055e-05, 
       3.9564283766043665e-05, 
       3.9564283766043665e-05, 
       2.9372918642731359e-05, 
       3.7740652638366989e-05, 
       3.4228546616817536e-05, 
       3.3734390075500566e-05, 
       3.9782066726249878e-05, 
       3.3399863828928042e-05, 
       4.1328098805544524e-05, 
       2.9565615872071453e-05, 
       3.9763346035326985e-05, 
       4.1285477836314294e-05, 
       3.6972811087475271e-05, 
       3.6990578151268718e-05, 
       3.8193023676862647e-05, 
       3.0832013078021731e-05, 
       4.1244800999261019e-05, 
       3.2187334333793278e-05, 
       3.2187334333793278e-05, 
       3.2850848384872945e-05, 
       2.9816401789328107e-05, 
       3.4200872130658989e-05, 
       4.1102988943635879e-05, 
       3.4737574801587400e-05, 
       3.4737574801587400e-05, 
       3.0666894504126092e-05, 
       4.1071813861677277e-05, 
       3.0645350076576921e-05, 
       4.1070252544372064e-05, 
       4.1076324877475343e-05, 
       3.0996977406781032e-05, 
       4.1031868374573238e-05, 
       3.6020689277912352e-05, 
       3.7207170822787140e-05, 
       3.3932771690848700e-05, 
       2.8979178260506008e-05, 
       3.4616441795642464e-05, 
       2.9623291809170077e-05, 
       3.1082912024953075e-05, 
       3.4042137612941533e-05, 
       4.0603184187485268e-05, 
       3.5296338770245203e-05, 
       3.8295201009682578e-05, 
       3.9315755608348710e-05, 
       3.7161531723287246e-05, 
       2.9075252692255868e-05, 
       3.8205457155891319e-05, 
       2.9349786857907786e-05, 
       3.0000000000003799e-05, 
       2.9928702118651575e-05, 
       2.9701306352631196e-05, 
       2.9052798867166905e-05, 
       3.3070098669626784e-05, 
       3.7174932397282985e-05, 
       4.0419324820322041e-05, 
       3.0915297672439782e-05, 
       3.2090122233725596e-05, 
       3.0850098825009209e-05, 
       3.1249330033542872e-05, 
       2.8903793623760534e-05, 
       3.1394439697017604e-05, 
       4.0456745335075532e-05, 
       2.9701510602019764e-05, 
       3.4464562264029627e-05, 
       3.0927553069141961e-05, 
       2.9056149175103325e-05, 
       3.3712354842395815e-05, 
       3.0016120296635692e-05, 
       3.8449684263640226e-05, 
       3.7914245397879775e-05, 
       4.0261905754794565e-05, 
       3.3686053414424321e-05, 
       3.1736531080024320e-05, 
       3.3919277072243219e-05, 
       3.3333369538697630e-05, 
       4.0234791889229823e-05, 
       3.5729591088579886e-05, 
       3.4634201405336033e-05, 
       3.4683033109870199e-05, 
       3.3701145766668560e-05, 
       3.7639476947115350e-05, 
       3.4920534260712328e-05, 
       3.0044425549602490e-05, 
       4.0228911425485270e-05, 
       3.7210650310490927e-05, 
       3.5327437378899194e-05, 
       4.0208263599085097e-05, 
       2.8472522087666933e-05, 
       3.5204527044063818e-05, 
       3.8463102569177809e-05, 
       3.0794404473527768e-05, 
       3.5539496648845564e-05, 
       3.4226461824829729e-05, 
       3.3402023386649974e-05, 
       3.9999833733587459e-05, 
       4.0029826003979332e-05, 
       4.0029844154048853e-05, 
       3.0366110018074966e-05, 
       4.0012715551191984e-05, 
       2.9923443061167850e-05, 
       3.0000000000004290e-05, 
       3.8741427463576370e-05, 
       2.9170411237893291e-05, 
       3.9999999999999075e-05, 
       3.9999999999997679e-05, 
       3.9999999999997787e-05, 
       4.0000000000004957e-05, 
       4.0000000000001026e-05, 
       3.9999999999999414e-05, 
       4.0000000000002246e-05, 
       3.9999999999996818e-05, 
       3.9999999999998275e-05, 
       4.0000000000000566e-05, 
       4.0000000000001894e-05, 
       3.9999999999999407e-05, 
       4.0000000000004625e-05, 
       3.9999999999997638e-05, 
       4.0000000000001935e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.9650987941895117e-05, 
       3.5482870386269809e-05, 
       2.9622973978578160e-05, 
       3.6593164811460520e-05, 
       2.8575086252524803e-05, 
       3.2446315832416563e-05, 
       3.2445998443909133e-05, 
       2.9222570908625658e-05, 
       3.5897359386160824e-05, 
       2.8220584857159224e-05, 
       3.6687774840680397e-05, 
       3.6443879322673523e-05, 
       3.1244270277087970e-05, 
       3.3289332685652457e-05, 
       3.1230700689164850e-05, 
       3.1337411944442081e-05, 
       2.8521061297300755e-05, 
       3.9687554334761485e-05, 
       3.0254233466135556e-05, 
       3.1100594687884024e-05, 
       2.8850617147111457e-05, 
       2.8944511564133373e-05, 
       2.8082265116472246e-05, 
       2.8068528184765371e-05, 
       2.8077208884788131e-05, 
       3.0000000000003890e-05, 
       3.0000000000002623e-05, 
       2.8530676677170921e-05, 
       3.7275800419838399e-05, 
       3.6215545086320893e-05, 
       3.1365052532130788e-05, 
       3.9516476748865015e-05, 
       3.0000000000001698e-05, 
       3.7853102683148861e-05, 
       2.9432110253767269e-05, 
       2.8959495717187786e-05, 
       3.0543824997494746e-05, 
       2.9999999999999923e-05, 
       2.8568650787982516e-05, 
       2.9297579659319114e-05, 
       3.3499765174853511e-05, 
       2.9204637490925500e-05, 
       2.9318124522991120e-05, 
       2.7865298580209685e-05, 
       2.7988822053944336e-05, 
       3.0248660897675957e-05, 
       3.7341689134393539e-05, 
       3.4414201452375797e-05, 
       3.4414201452375797e-05, 
       3.4308219212227503e-05, 
       3.0383257698228503e-05, 
       3.8408828770963399e-05, 
       2.9386197789292617e-05, 
       3.1259015612606084e-05, 
       3.8658217901830363e-05, 
       3.9064243768141764e-05, 
       2.8712653641529668e-05, 
       3.3520119136562809e-05, 
       2.9572263932335081e-05, 
       2.9130791664867986e-05, 
       2.8160474893762232e-05, 
       3.3742104548615828e-05, 
       3.9008734495496870e-05, 
       3.7431286076416152e-05, 
       3.0654617331625450e-05, 
       3.5126907049644794e-05, 
       3.1607286960826823e-05, 
       2.8285746086841743e-05, 
       3.8541853781224040e-05, 
       2.9989459915217344e-05, 
       3.8467281057835360e-05, 
       3.3113867711355641e-05, 
       3.2347611867880527e-05, 
       2.7505781452199071e-05, 
       3.2323134450346488e-05, 
       3.3188587985101968e-05, 
       2.9969687710007455e-05, 
       2.9702866064993838e-05, 
       2.7976306513998781e-05, 
       3.7649197822990172e-05, 
       2.7814093744353112e-05, 
       3.8729535589708686e-05, 
       2.7814734299262401e-05, 
       3.8653908574207702e-05, 
       3.7582923524431794e-05, 
       2.7486323187209600e-05, 
       2.8162466834048949e-05, 
       2.8851249081728891e-05, 
       3.8695240199046960e-05, 
       3.5327127544927605e-05, 
       3.3040893491297512e-05, 
       2.9324329557977478e-05, 
       2.7374221715914724e-05, 
       3.2978908424775638e-05, 
       3.2043720776259829e-05, 
       2.7389734391356983e-05, 
       3.6547845291455098e-05, 
       2.9465349402272529e-05, 
       3.1787455009125469e-05, 
       3.1730730693507046e-05, 
       2.7976528120341016e-05, 
       3.8285833626464303e-05, 
       2.8948340932654851e-05, 
       2.9134190212726618e-05, 
       2.8673368417038018e-05, 
       2.9519846433190461e-05, 
       2.8578194731965850e-05, 
       2.7316504284669223e-05, 
       3.6331372720670647e-05, 
       3.6323511307096425e-05, 
       3.5713305796603942e-05, 
       3.2584746456569697e-05, 
       2.7071017134050915e-05, 
       3.4570049770267100e-05, 
       3.2989438459643273e-05, 
       2.8217452674773613e-05, 
       3.0026421008164881e-05, 
       3.2531280616438226e-05, 
       3.7257390920122403e-05, 
       3.2955583650009276e-05, 
       3.6680754730805116e-05, 
       3.3405109040810726e-05, 
       2.8382676175699329e-05, 
       3.5311993187856882e-05, 
       3.0539013882529363e-05, 
       3.1505554418710124e-05, 
       3.2843689428531188e-05, 
       3.4710405485303393e-05, 
       3.6281714511098339e-05, 
       3.1972625598174635e-05, 
       2.6952375059381419e-05, 
       2.7834170994420485e-05, 
       2.6849007007184411e-05, 
       3.7823612738472191e-05, 
       2.7695085583835227e-05, 
       2.8737751496506896e-05, 
       3.2315997158234298e-05, 
       3.2035312534696759e-05, 
       3.0555951315988259e-05, 
       3.3530224421913222e-05, 
       3.5782113514338267e-05, 
       3.1337038596283865e-05, 
       3.0000000000003209e-05, 
       2.8769334550289220e-05, 
       3.4499437752777667e-05, 
       3.2807943768833218e-05, 
       3.3126108059818784e-05, 
       2.7934328777687005e-05, 
       3.5580419669643663e-05, 
       3.2176283974695814e-05, 
       3.3298103528320674e-05, 
       3.6055431562855655e-05, 
       3.5609595365240585e-05, 
       2.6689551612567050e-05, 
       2.9115329692074404e-05, 
       3.0276756787263230e-05, 
       2.6775219220041727e-05, 
       2.7994865876308730e-05, 
       3.2096843417391421e-05, 
       3.7363428462315630e-05, 
       3.2084949845151920e-05, 
       2.6651871004129518e-05, 
       3.1763396696123271e-05, 
       2.7519057009068031e-05, 
       2.8563919742320334e-05, 
       3.1894204383735864e-05, 
       2.7458936442062691e-05, 
       3.0897729527835391e-05, 
       3.5273612550722127e-05, 
       3.1457485844386864e-05, 
       3.2446120342464429e-05, 
       2.8527782530677453e-05, 
       2.8080167794846691e-05, 
       2.8080167794846691e-05, 
       2.6827956180676950e-05, 
       3.7202714531507361e-05, 
       3.7199602705927552e-05, 
       2.6293008694190432e-05, 
       2.8990484485342919e-05, 
       2.9886850376486911e-05, 
       2.6320960512972062e-05, 
       2.9856951105215019e-05, 
       3.7049150835329941e-05, 
       2.8181324118350672e-05, 
       3.1068817939316885e-05, 
       2.8419576696686963e-05, 
       2.9772054084084639e-05, 
       3.6978786302679147e-05, 
       3.0817872042770107e-05, 
       2.7651810436792986e-05, 
       3.0472039441073314e-05, 
       3.0749033162695300e-05, 
       3.6877175023213065e-05, 
       3.6804827147161923e-05, 
       3.0366074177180129e-05, 
       3.6528104931409156e-05, 
       3.1616580804268080e-05, 
       2.8777848553049863e-05, 
       2.6295438224540838e-05, 
       2.6345438903566346e-05, 
       2.8480138056537372e-05, 
       3.4263847765449354e-05, 
       3.0982006598378774e-05, 
       3.4239074491246886e-05, 
       3.0632180601652828e-05, 
       2.7771942945472978e-05, 
       2.8934784541401819e-05, 
       3.0337024272243701e-05, 
       3.1720626572684824e-05, 
       2.7392062671394504e-05, 
       3.6002388331020245e-05, 
       3.1607286960826430e-05, 
       3.5801186193448936e-05, 
       2.7487744403928269e-05, 
       2.6677955032792055e-05, 
       3.0628741632799320e-05, 
       3.6510065153031226e-05, 
       3.6015452006704093e-05, 
       3.6357297291441801e-05, 
       3.6476683310775503e-05, 
       3.3611621798708118e-05, 
       3.5875659550603929e-05, 
       3.2634187209055610e-05, 
       2.7930778087845611e-05, 
       2.9290313237655716e-05, 
       2.5777677579179011e-05, 
       2.5928187390429477e-05, 
       2.7540875722982256e-05, 
       3.5469450524023458e-05, 
       3.5871783031862640e-05, 
       2.5667459950830651e-05, 
       2.9600884725684659e-05, 
       3.3064887819056622e-05, 
       2.8522896376403126e-05, 
       2.6429971651214224e-05, 
       3.0877422423057182e-05, 
       2.7225424370410121e-05, 
       2.5826760319285673e-05, 
       3.5762396041448836e-05, 
       2.7058305027219290e-05, 
       2.6396357425576386e-05, 
       3.0125333144341761e-05, 
       2.6963325232473530e-05, 
       2.6963325232473530e-05, 
       3.0889459469239398e-05, 
       2.6912393472690718e-05, 
       3.0273351280007716e-05, 
       3.3091362765132128e-05, 
       3.5936245272006098e-05, 
       2.7615675276231317e-05, 
       3.4659105311767249e-05, 
       3.1358466138024678e-05, 
       2.8803181620953417e-05, 
       2.9690143341845960e-05, 
       3.0805093199411554e-05, 
       2.5536594902026181e-05, 
       2.9970226582605102e-05, 
       2.8971946865796545e-05, 
       3.5274607453297510e-05, 
       2.5840498017201320e-05, 
       3.5771754859293081e-05, 
       2.6186282374133461e-05, 
       3.1211810801065112e-05, 
       2.9368704696853477e-05, 
       3.2053126566242743e-05, 
       2.6230692824546582e-05, 
       2.5779427165932097e-05, 
       3.5847076981616771e-05, 
       3.4321135515174934e-05, 
       3.5625144114530312e-05, 
       2.6260702955184550e-05, 
       3.3743727769820210e-05, 
       2.5496509510115764e-05, 
       3.2920047814375116e-05, 
       3.5276042413766353e-05, 
       2.9838628640169500e-05, 
       2.6180073852977485e-05, 
       2.6430193040288062e-05, 
       3.5540757566365029e-05, 
       2.5445362457155064e-05, 
       2.5131748360077843e-05, 
       2.5183856314941863e-05, 
       2.5183856314941863e-05, 
       3.2243154260855022e-05, 
       2.5413613970155223e-05, 
       3.2029548066166961e-05, 
       3.4915887907330485e-05, 
       3.4915887907330485e-05, 
       2.9121450554522224e-05, 
       2.5879768128818131e-05, 
       3.5204857364067725e-05, 
       2.5288919313416905e-05, 
       2.7163503719489365e-05, 
       2.9374918632531657e-05, 
       2.7094345674080616e-05, 
       3.3247343677741839e-05, 
       2.6368372183667267e-05, 
       2.6798105231056230e-05, 
       2.6248455264832591e-05, 
       3.1602086651328833e-05, 
       2.8507450648439152e-05, 
       2.7117876097687938e-05, 
       2.7435175913409345e-05, 
       2.6449669604885500e-05, 
       2.6837769560996346e-05, 
       3.2288093342801241e-05, 
       3.3112181753226338e-05, 
       2.6327828105466185e-05, 
       2.7604395014481555e-05, 
       2.6248274781019007e-05, 
       3.5075187226166501e-05, 
       2.6499287902139400e-05, 
       3.5048342316519342e-05, 
       2.5487772775955021e-05, 
       2.9107585376571788e-05, 
       2.7315182587174559e-05, 
       3.5026048669214905e-05, 
       2.7240047410789172e-05, 
       2.7240047410789172e-05, 
       3.4992291131616725e-05, 
       2.9301599397494444e-05, 
       2.7829450804557431e-05, 
       2.7517890164182404e-05, 
       2.7051289547636673e-05, 
       3.0700648712216066e-05, 
       2.5506036094101103e-05, 
       3.4817957224150159e-05, 
       2.9490278375770467e-05, 
       3.4844018242779629e-05, 
       2.6165365567694776e-05, 
       2.6165365567694776e-05, 
       2.8393523521984109e-05, 
       2.5428953482915565e-05, 
       2.5539217545176599e-05, 
       2.6802706074228607e-05, 
       3.4758237164937349e-05, 
       2.8927436514525534e-05, 
       2.5163502360372020e-05, 
       2.8961570281075600e-05, 
       2.6434114728889446e-05, 
       2.4654758886623465e-05, 
       2.4654758886623465e-05, 
       2.5001033971007190e-05, 
       2.5964479321303237e-05, 
       2.8836170219685999e-05, 
       2.6007485549697854e-05, 
       3.4599167819017538e-05, 
       2.5920207430770246e-05, 
       2.9846520473705149e-05, 
       2.8645809582644942e-05, 
       3.4377045575196747e-05, 
       3.4200684791627271e-05, 
       2.4426128399523059e-05, 
       2.5647224272868987e-05, 
       3.4476549336772421e-05, 
       2.8247894539198831e-05, 
       2.7438582780098892e-05, 
       3.4479778057852196e-05, 
       3.4251021526359262e-05, 
       2.5664206075342809e-05, 
       2.9121678859158691e-05, 
       2.5033201362671111e-05, 
       3.3364283037172436e-05, 
       2.4447835058470351e-05, 
       3.4294599299771525e-05, 
       3.3291065125318106e-05, 
       3.4089501744920831e-05, 
       2.4969954847476265e-05, 
       2.9024699719979002e-05, 
       2.8671196159773006e-05, 
       2.4630330652046270e-05, 
       3.2280424805061478e-05, 
       2.9522686564137383e-05, 
       3.4192396523913104e-05, 
       2.7114935325952179e-05, 
       3.3912427428238050e-05, 
       3.4316842850734913e-05, 
       2.8418999202795457e-05, 
       3.0233394826160935e-05, 
       2.8194171881190316e-05, 
       2.8404788785574209e-05, 
       2.4343139871393643e-05, 
       3.0191619504350694e-05, 
       3.0755917042513452e-05, 
       2.8027836970488926e-05, 
       3.4272269132251431e-05, 
       3.3120345686135500e-05, 
       2.4218683291086537e-05, 
       2.5944641231553954e-05, 
       2.8519726090785362e-05, 
       3.2754655590475162e-05, 
       2.6670566568781220e-05, 
       2.8667375144667282e-05, 
       2.6250974145638234e-05, 
       3.4179587676987727e-05, 
       2.8417846549915602e-05, 
       3.1915296966306216e-05, 
       2.7946890837173315e-05, 
       2.7946890837173315e-05, 
       2.5561360154550715e-05, 
       3.3523687032400315e-05, 
       3.3516198827054114e-05, 
       2.8885952754713721e-05, 
       2.5392466137233413e-05, 
       2.6697893488365707e-05, 
       2.7953335551113027e-05, 
       3.0385390315916287e-05, 
       2.6691373934654575e-05, 
       2.4891608425172016e-05, 
       2.9186893725728448e-05, 
       2.9720024317533491e-05, 
       2.4316073025063863e-05, 
       3.2072791174885115e-05, 
       2.8310123390552802e-05, 
       2.7681565392937294e-05, 
       2.8172209727582446e-05, 
       2.4717834974175119e-05, 
       2.8500031283951278e-05, 
       2.8151312716500778e-05, 
       2.7880937040111283e-05, 
       2.7049357159754555e-05, 
       2.9417677684036884e-05, 
       2.8328383928724001e-05, 
       2.5935271797444521e-05, 
       3.2755520485280344e-05, 
       2.4394581900398357e-05, 
       2.6867819495971078e-05, 
       2.9794057034115196e-05, 
       2.7440414384611052e-05, 
       3.3695842700976900e-05, 
       2.7432561829927438e-05, 
       3.0380485458353943e-05, 
       2.5186131576443482e-05, 
       3.3498529456835508e-05, 
       2.6893492175475148e-05, 
       2.8566549431284558e-05, 
       2.8566549431284558e-05, 
       2.6663138893977932e-05, 
       3.2461213637322961e-05, 
       2.6958626730782541e-05, 
       2.4930182552588714e-05, 
       2.4930182552588714e-05, 
       2.8686862129057285e-05, 
       2.4263861299639403e-05, 
       2.8605673870863314e-05, 
       2.3730701314903488e-05, 
       2.3903231254453783e-05, 
       2.5323838146909259e-05, 
       2.7089073128826942e-05, 
       2.7708527447814174e-05, 
       2.7743510227138512e-05, 
       2.7527281192965033e-05, 
       3.1486880033920773e-05, 
       2.4644722521115708e-05, 
       3.0367131000665230e-05, 
       3.3264524532519042e-05, 
       3.3092992729586531e-05, 
       2.7315479801981029e-05, 
       3.2950504333784331e-05, 
       2.7649710679235837e-05, 
       2.5261628588576502e-05, 
       3.2156559939064274e-05, 
       2.8527342620175341e-05, 
       2.8397158494519286e-05, 
       3.3153529898453979e-05, 
       2.4085492896186930e-05, 
       2.8105970117063629e-05, 
       3.1033444745727773e-05, 
       2.4807487784192627e-05, 
       2.4583567523299465e-05, 
       2.7437918633535890e-05, 
       2.6059703812951711e-05, 
       2.8669835086657525e-05, 
       2.3522274813803017e-05, 
       2.7546112682650883e-05, 
       2.4376002944147142e-05, 
       2.3809454159939669e-05, 
       3.2130895186351850e-05, 
       2.7134289388676936e-05, 
       2.8031167862592865e-05, 
       2.4330837799660043e-05, 
       2.3790586943871917e-05, 
       2.4618466039271922e-05, 
       3.2499801756115493e-05, 
       3.2415007070882617e-05, 
       3.1402259866988810e-05, 
       3.1361265532885596e-05, 
       3.2967247616230259e-05, 
       3.2926980145146655e-05, 
       3.2897957161648355e-05, 
       3.2948545011265201e-05, 
       2.8324228067536267e-05, 
       3.2801773635771439e-05, 
       2.4924362740452873e-05, 
       2.3586579648045455e-05, 
       2.6808095908631068e-05, 
       2.4024510227490845e-05, 
       2.7682180923534672e-05, 
       2.6090946790785796e-05, 
       3.2557167762135286e-05, 
       2.6799504014023414e-05, 
       3.2673818104157556e-05, 
       2.4122472530575361e-05, 
       2.9519237447149331e-05, 
       2.4135295232504095e-05, 
       3.2648974817421232e-05, 
       2.3600289507962191e-05, 
       3.2609486734809940e-05, 
       3.2438272513003899e-05, 
       2.7015383036214886e-05, 
       2.7238773284972113e-05, 
       2.7028509038268039e-05, 
       2.6668046581791002e-05, 
       2.3224137971530830e-05, 
       2.6750217843854205e-05, 
       2.4787134256384244e-05, 
       2.5890325553033500e-05, 
       2.3044499027859778e-05, 
       2.9999999999996823e-05, 
       2.9999999999997812e-05, 
       3.0000000000001190e-05, 
       3.0000000000001925e-05, 
       3.0000000000001241e-05, 
       2.9999999999996040e-05, 
       2.9999999999995661e-05, 
       2.9999999999995661e-05, 
       2.3889920703872291e-05, 
       2.5797298581868303e-05, 
       2.9068548551881954e-05, 
       2.7035369778148715e-05, 
       2.5461509891066295e-05, 
       2.5157720705493091e-05, 
       2.9296387254842345e-05, 
       2.6890413863725637e-05, 
       2.8410366991330650e-05, 
       2.7688211026207803e-05, 
       2.8417379778696077e-05, 
       2.6997498023266238e-05, 
       2.3482817963913375e-05, 
       2.4557840187249647e-05, 
       2.6940261705840389e-05, 
       2.2894511820809013e-05, 
       2.2908706346192163e-05, 
       3.1349347545628386e-05, 
       2.4321376606195986e-05, 
       2.2966859743536285e-05, 
       2.9147794246437171e-05, 
       3.0703859380416975e-05, 
       2.3402221596710644e-05, 
       2.7615709465128338e-05, 
       2.3650705692883611e-05, 
       2.3650705692883611e-05, 
       2.3904474655771749e-05, 
       3.1607364102157018e-05, 
       2.6600896328399041e-05, 
       3.1229248333623198e-05, 
       2.6321370303117079e-05, 
       2.7796535037283469e-05, 
       2.6641235987568230e-05, 
       2.3832959785462192e-05, 
       3.1649812539506376e-05, 
       3.0592741499987829e-05, 
       2.3096510237342673e-05, 
       2.2699653875450696e-05, 
       2.3249745470124507e-05, 
       2.6773023703665230e-05, 
       2.2970110172489182e-05, 
       3.0838358223896081e-05, 
       2.6139734478468064e-05, 
       2.3764794548079181e-05, 
       2.2947817061798552e-05, 
       2.2833053494474773e-05, 
       2.9999999999999235e-05, 
       2.9626090535365558e-05, 
       2.3486790295566854e-05, 
       2.3241267647300088e-05, 
       3.1836859885167991e-05, 
       2.6571641821906969e-05, 
       2.2834650174087768e-05, 
       2.4906174972015973e-05, 
       2.3022973518139672e-05, 
       2.8062723410813606e-05, 
       3.1809906127330802e-05, 
       2.5474798257939025e-05, 
       3.0350618415187151e-05, 
       2.4060360308950122e-05, 
       3.1708235945966868e-05, 
       2.8395459418629152e-05, 
       3.1498596333085101e-05, 
       3.1498596333085101e-05, 
       2.7018291849097191e-05, 
       2.2779391893054154e-05, 
       2.4024226804887041e-05, 
       2.8994313727000298e-05, 
       2.6988491048561361e-05, 
       2.3677471064899219e-05, 
       2.4107423881426563e-05, 
       2.6027661009359367e-05, 
       2.2385820822631361e-05, 
       2.6290319960895727e-05, 
       2.7460075917738829e-05, 
       2.3665446502189034e-05, 
       2.6987673330377177e-05, 
       2.9310579038495038e-05, 
       2.2646672164481440e-05, 
       2.2771857861919115e-05, 
       2.7179884624405099e-05, 
       2.3862224680109702e-05, 
       3.1500715622996577e-05, 
       2.6202096940542220e-05, 
       2.4102312040910398e-05, 
       2.7408329776064467e-05, 
       2.2316398705654424e-05, 
       3.1105320269090580e-05, 
       2.4065749760453949e-05, 
       2.4065749760453949e-05, 
       2.8476234504351562e-05, 
       2.6138437717666728e-05, 
       2.7557054771714884e-05, 
       2.2222437761925977e-05, 
       2.5761711979121882e-05, 
       2.2205564291773968e-05, 
       2.2410307074124568e-05, 
       2.8276523875806391e-05, 
       3.1302737334661764e-05, 
       2.3283726202759125e-05, 
       2.3304161748790386e-05, 
       3.1306975642616347e-05, 
       2.6375390025120789e-05, 
       2.2182938597154211e-05, 
       3.0960807001893467e-05, 
       2.2835906472531427e-05, 
       2.5884602200539683e-05, 
       2.9229347078953845e-05, 
       2.3496642104841726e-05, 
       2.3185542134062420e-05, 
       2.2946360701331529e-05, 
       2.5776194824604358e-05, 
       2.2454703100843991e-05, 
       2.8531063441147001e-05, 
       2.6074827207450302e-05, 
       2.5556299411245292e-05, 
       2.3144668520699665e-05, 
       2.8795506143643095e-05, 
       2.1986390120268598e-05, 
       3.0299289156082009e-05, 
       2.6564089538255877e-05, 
       2.3322930384472611e-05, 
       2.5342102881747371e-05, 
       2.2335720200635056e-05, 
       3.0258393603544679e-05, 
       2.2195165077980501e-05, 
       2.7397086408704430e-05, 
       2.6917607645641059e-05, 
       2.4710220326074705e-05, 
       2.2253959978517775e-05, 
       2.2253959978517775e-05, 
       2.2771073735115909e-05, 
       2.4479378788658450e-05, 
       2.2765888807037921e-05, 
       2.1895520882208273e-05, 
       2.2921622956386172e-05, 
       2.9707451838051642e-05, 
       2.5573072733358111e-05, 
       2.3058233823004184e-05, 
       2.3307036526590237e-05, 
       2.5092609836781439e-05, 
       2.2748515000489624e-05, 
       2.3150277437011079e-05, 
       3.0791274555545078e-05, 
       2.2900377411271002e-05, 
       2.8012605383694653e-05, 
       3.0412383982806414e-05, 
       3.0181374998003317e-05, 
       2.2276745666794063e-05, 
       2.9750928763819810e-05, 
       2.1662405221183852e-05, 
       2.2138372384155840e-05, 
       2.1996088220648256e-05, 
       2.1991383242162543e-05, 
       2.4564033802824913e-05, 
       2.6990098647192138e-05, 
       2.3780130967086489e-05, 
       2.5351466465478984e-05, 
       3.0443058101149356e-05, 
       3.0390926228646568e-05, 
       2.5368723327925288e-05, 
       2.1567682677648480e-05, 
       2.2292876913502037e-05, 
       2.2735671428963824e-05, 
       2.8647198936037716e-05, 
       2.9988827002107710e-05, 
       2.8577410507709898e-05, 
       2.5269777898027529e-05, 
       2.8216726269075040e-05, 
       2.3764465637506819e-05, 
       2.4782966485001782e-05, 
       2.2255096370630049e-05, 
       2.9164866735637960e-05, 
       2.2340046124967011e-05, 
       2.5329885223332629e-05, 
       2.3076594573793710e-05, 
       2.5272042295458624e-05, 
       2.4704698549571055e-05, 
       2.9454304430481464e-05, 
       2.7756201280397087e-05, 
       2.2572585828920256e-05, 
       2.2494388918449920e-05, 
       3.0137206527548647e-05, 
       2.8170724885628871e-05, 
       2.1413179031443492e-05, 
       2.8659232795270722e-05, 
       2.1346292775702525e-05, 
       2.6760840154110105e-05, 
       2.9204634832804709e-05, 
       2.5666949948466051e-05, 
       2.6061808553163991e-05, 
       3.0059956200694516e-05, 
       2.3290481993096621e-05, 
       2.5138412556286546e-05, 
       2.5532372454328029e-05, 
       2.8899931661099338e-05, 
       2.9907934239634957e-05, 
       3.0000000000004405e-05, 
       3.0000000000001478e-05, 
       2.9999999999995698e-05, 
       2.9999999999999855e-05, 
       2.9999999999999736e-05, 
       3.0000000000000529e-05, 
       2.9999999999998808e-05, 
       2.9999999999998740e-05, 
       2.9999999999997765e-05, 
       2.9999999999997893e-05, 
       3.0000000000001674e-05, 
       2.9999999999999940e-05, 
       3.0000000000002129e-05, 
       2.9999999999998706e-05, 
       2.9999999999998937e-05, 
       3.0000000000004934e-05, 
       2.9999999999999347e-05, 
       2.9999999999999242e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.1239152732205458e-05, 
       2.7327175427063343e-05, 
       2.7327175427063343e-05, 
       2.1890823686870737e-05, 
       2.6429974762185499e-05, 
       2.5876481913348810e-05, 
       2.3954620912600911e-05, 
       2.1540065046920755e-05, 
       2.1576992470412397e-05, 
       2.4661653433269915e-05, 
       2.6433030840338851e-05, 
       2.4922766670927189e-05, 
       2.1123041710218976e-05, 
       2.6064512283843695e-05, 
       2.5749495904983114e-05, 
       2.1310217221159616e-05, 
       2.1111653994676344e-05, 
       2.5077949988261698e-05, 
       2.9784471235163563e-05, 
       2.3056373773679594e-05, 
       2.9102145421663064e-05, 
       2.7483514799424028e-05, 
       2.8962947173945567e-05, 
       2.2711763764086440e-05, 
       2.2711763764086440e-05, 
       2.7200218741839871e-05, 
       2.9407851081748054e-05, 
       2.2205162355419221e-05, 
       2.4743849721618246e-05, 
       2.4532785214329851e-05, 
       2.8615733804263409e-05, 
       2.1740599758329869e-05, 
       2.7775020160739574e-05, 
       2.7231851287790759e-05, 
       2.4023947834927688e-05, 
       2.4613768342028950e-05, 
       2.1715685410280728e-05, 
       2.4704448207108885e-05, 
       2.0983761833621713e-05, 
       2.9268202828632838e-05, 
       2.9589588322323290e-05, 
       2.3819988431851243e-05, 
       2.9140865922690650e-05, 
       2.7016209725563762e-05, 
       2.4619807651610879e-05, 
       2.5340042480965205e-05, 
       2.3769275458079885e-05, 
       2.5389185845765271e-05, 
       2.6163251136164897e-05, 
       2.4037323377968369e-05, 
       2.2896673422365298e-05, 
       2.1062759073885588e-05, 
       2.8959262462343110e-05, 
       2.5692251912621091e-05, 
       2.6271725889237126e-05, 
       2.1650971366648380e-05, 
       2.0955979386201294e-05, 
       2.1116546838638918e-05, 
       2.4984552050094938e-05, 
       2.1437901128472993e-05, 
       2.9323671189211286e-05, 
       2.4692825644927512e-05, 
       2.6052954166854551e-05, 
       2.5214357617053483e-05, 
       2.6081102558987802e-05, 
       2.6475050434112720e-05, 
       2.2109725118767885e-05, 
       2.6001789423800533e-05, 
       2.8165089431936793e-05, 
       2.3352057237998144e-05, 
       2.4203662078715161e-05, 
       2.9127691585543378e-05, 
       2.9183274514529182e-05, 
       2.4295959119443233e-05, 
       2.2425610814530062e-05, 
       2.2030280284591683e-05, 
       2.8305956971834876e-05, 
       2.4830233957070959e-05, 
       2.7607803707495960e-05, 
       2.7607803707495960e-05, 
       2.1237476913355189e-05, 
       2.3586741764726801e-05, 
       2.1134841277494663e-05, 
       2.7810885568273507e-05, 
       2.2977942399483104e-05, 
       2.6428151577749415e-05, 
       2.1210739386215050e-05, 
       2.0848778399258320e-05, 
       2.1178276892523642e-05, 
       2.9087254520438652e-05, 
       2.4337536328356341e-05, 
       2.8566646731706197e-05, 
       2.4203451429410660e-05, 
       2.4703830364985675e-05, 
       2.1426201623577512e-05, 
       2.8956288614032272e-05, 
       2.8956288614032272e-05, 
       2.0636546521793716e-05, 
       2.0573885919396488e-05, 
       2.5401352461363626e-05, 
       2.5868753062338599e-05, 
       2.3027119709218912e-05, 
       2.3357590161797815e-05, 
       2.8191964995993427e-05, 
       2.8191964995993427e-05, 
       2.7058952232719377e-05, 
       2.6095050197318389e-05, 
       2.7813635560402981e-05, 
       2.6638296207266734e-05, 
       2.0662922795087871e-05, 
       2.1326781027017741e-05, 
       2.2295701266401901e-05, 
       2.1014264001594890e-05, 
       2.0757973111099345e-05, 
       2.1458026712368595e-05, 
       2.8274207074975589e-05, 
       2.4810695999304351e-05, 
       2.8323983624365901e-05, 
       2.5742038341567871e-05, 
       2.8508845781605500e-05, 
       2.0888701155268137e-05, 
       2.0529585232227509e-05, 
       2.5034772056485327e-05, 
       2.5030568775508550e-05, 
       2.5228473520783341e-05, 
       2.2251594537773418e-05, 
       2.3920430802565223e-05, 
       2.7173253407105435e-05, 
       2.7173253407105435e-05, 
       2.1532483995958935e-05, 
       2.3938053442539653e-05, 
       2.3921023616950308e-05, 
       2.2326028083594663e-05, 
       2.3155490562162931e-05, 
       2.8715113244671733e-05, 
       2.1717610844558287e-05, 
       2.0794380332537767e-05, 
       2.4581107671194017e-05, 
       2.3855107930500977e-05, 
       2.3931400718936076e-05, 
       2.6537028378583931e-05, 
       2.3382582675524286e-05, 
       2.7401846745912245e-05, 
       2.5487488713158126e-05, 
       2.1077974696978288e-05, 
       2.0288251412787598e-05, 
       2.7819966116245448e-05, 
       2.1919165600279391e-05, 
       2.0262590036713024e-05, 
       2.0262590036713024e-05, 
       2.2188504960183010e-05, 
       2.2127654568867594e-05, 
       2.2127654568867594e-05, 
       2.8180755148115264e-05, 
       2.8516499935546783e-05, 
       2.4835200329453488e-05, 
       2.4115984691999000e-05, 
       2.5046301541370833e-05, 
       2.2974631505232761e-05, 
       2.4013273726756287e-05, 
       2.7078990616854246e-05, 
       2.2102567056342413e-05, 
       2.0185751655454124e-05, 
       2.4316151966514346e-05, 
       2.7071017134061641e-05, 
       2.8348982008541965e-05, 
       2.3428860298172131e-05, 
       2.2196862156178476e-05, 
       2.6937453733626056e-05, 
       2.1150799509610855e-05, 
       2.3539659488581484e-05, 
       2.3482343892585202e-05, 
       2.7999252452952237e-05, 
       2.7628439205731162e-05, 
       2.4588039812213814e-05, 
       2.2701421608013592e-05, 
       2.7155021069766327e-05, 
       2.8121583758424230e-05, 
       2.0762602104814532e-05, 
       2.8193199377089744e-05, 
       2.5484025494064713e-05, 
       2.1457534367848610e-05, 
       2.6123013017904331e-05, 
       2.6123013017904331e-05, 
       2.2650471065666470e-05, 
       2.0384977972974928e-05, 
       2.0981830259274723e-05, 
       1.9999999999998111e-05, 
       2.4435962051764705e-05, 
       2.0560672706527655e-05, 
       2.0921608959167802e-05, 
       2.3320543730630196e-05, 
       2.7658989075695313e-05, 
       2.7983855192934815e-05, 
       2.0025529558097509e-05, 
       2.7924049859267542e-05, 
       2.7094752882854488e-05, 
       2.4750256870124958e-05, 
       1.9928238017228546e-05, 
       2.2697794512588630e-05, 
       2.7910623423382483e-05, 
       2.1803981359955448e-05, 
       2.2513133712243432e-05, 
       2.2487632562352765e-05, 
       2.6532785764768870e-05, 
       2.3247646278202567e-05, 
       1.9805385880804040e-05, 
       2.4020321050384608e-05, 
       2.0785188884872226e-05, 
       2.2168277952894067e-05, 
       1.9802802571600301e-05, 
       2.7302757843914607e-05, 
       1.9694814819074407e-05, 
       2.0438563368833077e-05, 
       2.3941827263979897e-05, 
       2.3080230150871152e-05, 
       2.0386440672725847e-05, 
       2.3196201706351199e-05, 
       2.3975270264584468e-05, 
       1.9664127420988511e-05, 
       2.2537078314622451e-05, 
       2.6440983083775817e-05, 
       2.6979270356365161e-05, 
       2.0736045787212078e-05, 
       2.3333914253345995e-05, 
       2.7007087635012078e-05, 
       2.0131096348224223e-05, 
       2.4244809465662699e-05, 
       2.7557277890439430e-05, 
       2.2490350813223323e-05, 
       2.0348301841789819e-05, 
       2.3626160000265987e-05, 
       2.1828982117440435e-05, 
       1.9827436590608716e-05, 
       1.9639964216823775e-05, 
       1.9728813156633697e-05, 
       2.0274162029293038e-05, 
       2.0603484843342109e-05, 
       1.9999999999998596e-05, 
       2.2248334806690697e-05, 
       2.2913328556012979e-05, 
       2.4238273249756075e-05, 
       2.7587174358110437e-05, 
       1.9888466296019138e-05, 
       2.0020788450740702e-05, 
       1.9533548444942853e-05, 
       2.2651202832857441e-05, 
       2.6446103004066458e-05, 
       2.1740974643964685e-05, 
       2.6901910246995046e-05, 
       2.4305129581516451e-05, 
       2.2488905444394759e-05, 
       2.1155955256729102e-05, 
       2.0703716797033260e-05, 
       2.2270526340128344e-05, 
       2.5843863748330956e-05, 
       2.6501376718398551e-05, 
       1.9944936673672440e-05, 
       2.2385726217078230e-05, 
       2.4696363978890080e-05, 
       2.7364476339361907e-05, 
       2.5812831975973625e-05, 
       2.4001103060774727e-05, 
       2.7340611114994528e-05, 
       2.1092902376393430e-05, 
       2.3213266729957517e-05, 
       2.2470683703778668e-05, 
       2.0392611886884281e-05, 
       1.9575513305912215e-05, 
       2.1356778847748766e-05, 
       2.6425080053357025e-05, 
       2.0751072619852740e-05, 
       2.7039588483966157e-05, 
       1.9280495550961758e-05, 
       2.1915014621335653e-05, 
       2.3466568508411861e-05, 
       2.7042029363104111e-05, 
       2.7192529976509706e-05, 
       1.9328067673915844e-05, 
       2.6989068563196528e-05, 
       2.6365586578272929e-05, 
       1.9224655318514847e-05, 
       2.7101016643272701e-05, 
       2.6346345686438382e-05, 
       2.2592616862561221e-05, 
       2.1388308055816657e-05, 
       2.6787463240450105e-05, 
       1.9418850589352804e-05, 
       2.0162229237477238e-05, 
       2.0744412428478471e-05, 
       2.6415827814525692e-05, 
       1.9477067696663569e-05, 
       2.5814645656456167e-05, 
       2.6826156528245559e-05, 
       2.6240325992772855e-05, 
       1.9586178829398509e-05, 
       2.6368433247245749e-05, 
       2.6790923230971601e-05, 
       2.3687207381010272e-05, 
       2.7005755157448488e-05, 
       2.3172311266359876e-05, 
       2.2254635078452681e-05, 
       2.3552660218191950e-05, 
       2.2398173304182567e-05, 
       2.3771650466816687e-05, 
       2.6917006544640887e-05, 
       2.3038848490733663e-05, 
       2.0101133478987235e-05, 
       2.0090020165552969e-05, 
       2.0816722050757227e-05, 
       2.2578505195366890e-05, 
       2.6934731795653595e-05, 
       2.0149226279840114e-05, 
       1.9725871970259640e-05, 
       1.9136164161308124e-05, 
       2.5371679533008806e-05, 
       2.1445739792044982e-05, 
       2.2985661158146827e-05, 
       2.0464658943518358e-05, 
       2.0464658943518358e-05, 
       2.2040326108959798e-05, 
       1.9068193289794883e-05, 
       2.3115999304309994e-05, 
       1.9179440609092591e-05, 
       1.9470251516125982e-05, 
       2.5562586340520214e-05, 
       1.9228055870160890e-05, 
       2.0008225281875002e-05, 
       1.9065959945762624e-05, 
       1.9447883614347173e-05, 
       2.0695028792671969e-05, 
       1.9138796796021523e-05, 
       2.6664007448672790e-05, 
       2.2982371947980182e-05, 
       2.1826380928158985e-05, 
       2.1426388751940305e-05, 
       2.2349770540388181e-05, 
       2.3196359932591713e-05, 
       2.6602166401022034e-05, 
       2.1400451297735498e-05, 
       2.5297577505044151e-05, 
       2.3189235287865066e-05, 
       2.2134251156477077e-05, 
       2.1063674826185046e-05, 
       2.2503202268728972e-05, 
       1.9090079825955896e-05, 
       1.9090079825955896e-05, 
       1.9819687896412248e-05, 
       1.9237567574377060e-05, 
       1.9491526676154471e-05, 
       2.5243153123067677e-05, 
       2.1505240942907195e-05, 
       2.2531502469511492e-05, 
       2.2012556906109524e-05, 
       2.1246945279320117e-05, 
       2.4966842938183963e-05, 
       1.9893459957904125e-05, 
       1.9707958363798394e-05, 
       1.9187586038552021e-05, 
       1.9181439651334718e-05, 
       1.9978335979757786e-05, 
       2.4865216543862584e-05, 
       1.9734538324161951e-05, 
       2.2572015976622474e-05, 
       2.0282776646234093e-05, 
       2.0084592507547901e-05, 
       2.1914426599946710e-05, 
       1.9673716799595695e-05, 
       2.1691931175728616e-05, 
       2.4372224247570914e-05, 
       2.4372224247570914e-05, 
       1.9006468975859341e-05, 
       2.4458949583607247e-05, 
       2.0538444088119941e-05, 
       2.6163678466691344e-05, 
       1.9747456291983756e-05, 
       1.8636187270806300e-05, 
       1.9009836508943332e-05, 
       2.1911165843983136e-05, 
       2.1624708666460605e-05, 
       1.8693752491082285e-05, 
       2.5787068643643150e-05, 
       2.5132587746355704e-05, 
       2.1239152732205980e-05, 
       1.8796481878728683e-05, 
       2.5776450751655573e-05, 
       1.8927684817619635e-05, 
       1.9557109881434541e-05, 
       2.2740069998641482e-05, 
       1.9567363171964758e-05, 
       2.1792521487500579e-05, 
       2.2813287725737651e-05, 
       2.1892987785546046e-05, 
       1.9375388272943816e-05, 
       1.8873864999461511e-05, 
       2.1357837712905821e-05, 
       1.8754340642006708e-05, 
       2.3152728096937984e-05, 
       1.9121671248429618e-05, 
       2.2327309484392186e-05, 
       2.2742171677133305e-05, 
       2.0728844413406723e-05, 
       1.9874339648149293e-05, 
       2.1869385036547042e-05, 
       2.6034821136249018e-05, 
       2.1995402891497991e-05, 
       2.1869850324289869e-05, 
       1.9556681695726242e-05, 
       2.1277419710744966e-05, 
       2.0715242061331388e-05, 
       2.1681874417413688e-05, 
       2.0007606984251554e-05, 
       1.8775064099226119e-05, 
       1.9078681243448876e-05, 
       2.2653326413630240e-05, 
       2.5225062678109871e-05, 
       1.8673774283287912e-05, 
       1.9050824102138936e-05, 
       1.8425960945520104e-05, 
       1.9734445336062810e-05, 
       2.1854409703071268e-05, 
       2.5705538340808325e-05, 
       2.5766723524056884e-05, 
       2.1498796117001881e-05, 
       2.5593522429008230e-05, 
       2.4388741541868519e-05, 
       2.1441527713789161e-05, 
       2.3410514219344735e-05, 
       1.8808368343945443e-05, 
       2.4254438336572184e-05, 
       1.8462885968707351e-05, 
       2.2871133218626609e-05, 
       2.5536695184870963e-05, 
       2.5739755318344120e-05, 
       1.9999999999997684e-05, 
       2.4376183099517822e-05, 
       2.1836704331587505e-05, 
       2.1365198219423314e-05, 
       2.1377118558596892e-05, 
       1.8525276165632930e-05, 
       2.4398119237534047e-05, 
       2.5666956243073106e-05, 
       2.2583614748870743e-05, 
       2.2158308025661018e-05, 
       1.9847157166394711e-05, 
       1.9468243403322062e-05, 
       2.5204343752716428e-05, 
       1.8880900621100187e-05, 
       2.2668065591133145e-05, 
       2.1425158584596436e-05, 
       2.1259834454645140e-05, 
       2.0016991355826771e-05, 
       1.9680388165063498e-05, 
       2.5470336005178848e-05, 
       2.4373783826419086e-05, 
       1.8098510402825817e-05, 
       1.8352430349971238e-05, 
       1.8594681754406291e-05, 
       2.2282838825220545e-05, 
       2.5502255093924968e-05, 
       2.5448910520566233e-05, 
       2.2037187161003800e-05, 
       1.8844221243828147e-05, 
       2.2952532144424310e-05, 
       2.3894831095443319e-05, 
       2.5441872124714022e-05, 
       2.1158092788023229e-05, 
       2.4322585026855575e-05, 
       1.8533930663034295e-05, 
       2.2998279765040865e-05, 
       2.4876667588630600e-05, 
       2.4981786707723841e-05, 
       2.5163273217196913e-05, 
       1.7969319536917142e-05, 
       1.7969319536917142e-05, 
       1.8909462693514672e-05, 
       2.1045781535115987e-05, 
       2.0719354432601130e-05, 
       2.0115437166001478e-05, 
       2.4601214841607067e-05, 
       2.1054952803658941e-05, 
       1.9581443679071778e-05, 
       2.3276897413604886e-05, 
       2.3276897413604886e-05, 
       1.9187780126464548e-05, 
       1.9311392199925316e-05, 
       2.5272550638326800e-05, 
       1.8300532878472574e-05, 
       2.4058511959979058e-05, 
       1.8702402504351326e-05, 
       1.9132021117095319e-05, 
       2.4903796606459513e-05, 
       1.9337408323683094e-05, 
       2.5172357235331020e-05, 
       2.4321284906538726e-05, 
       2.0520706678718408e-05, 
       2.3778894425124177e-05, 
       2.3676838813413018e-05, 
       1.7838204932429741e-05, 
       2.0830410808266415e-05, 
       2.4960731576209159e-05, 
       2.3566771526061548e-05, 
       2.0112552151498836e-05, 
       1.9123739417627946e-05, 
       1.8561524207988734e-05, 
       2.4673839508180660e-05, 
       1.9906903730058719e-05, 
       2.1168295386925452e-05, 
       2.0752916294115517e-05, 
       2.4709149553722373e-05, 
       1.7767541327249107e-05, 
       1.7763700054793674e-05, 
       2.0141943014697284e-05, 
       2.0268969440731765e-05, 
       2.1554692848064810e-05, 
       1.7991720512962049e-05, 
       2.1740473510379552e-05, 
       2.0815827632765828e-05, 
       1.8361873249861767e-05, 
       2.4830514986061490e-05, 
       2.1792649518925854e-05, 
       2.1047010497146118e-05, 
       1.9492208692842696e-05, 
       1.7664987355903558e-05, 
       1.7664987355903558e-05, 
       2.1251316770795840e-05, 
       2.2607753054010485e-05, 
       1.8608857124891965e-05, 
       2.4934759550667732e-05, 
       2.4925523841027443e-05, 
       1.8194042024081950e-05, 
       1.8460627084400566e-05, 
       2.0361623128963163e-05, 
       2.1141057200037726e-05, 
       2.4043785780287165e-05, 
       2.4833198239755762e-05, 
       1.8145610318622196e-05, 
       2.2228509624992713e-05, 
       2.1132024306760404e-05, 
       1.7642051084354999e-05, 
       1.8086666707132975e-05, 
       1.9249884487151741e-05, 
       1.8255487502303842e-05, 
       1.7956939739931842e-05, 
       1.7731134423061075e-05, 
       2.4754713874808710e-05, 
       2.0686297667369626e-05, 
       1.7517170134011922e-05, 
       1.7567707248893300e-05, 
       2.3371460974311176e-05, 
       1.8437092303982169e-05, 
       2.0372370831356654e-05, 
       1.8204426550607204e-05, 
       1.7553804130884323e-05, 
       1.8335748846230362e-05, 
       2.4617157935715516e-05, 
       1.8351476526719662e-05, 
       1.7831190145844371e-05, 
       1.7561766834944122e-05, 
       2.3204551666706484e-05, 
       2.0425201813119722e-05, 
       2.0953774414409753e-05, 
       1.7964182941566314e-05, 
       2.3938995201209839e-05, 
       2.4477283634413058e-05, 
       1.8127032897212113e-05, 
       1.7477988349187017e-05, 
       1.9134103353779846e-05, 
       2.4504287059597024e-05, 
       1.7645478902118700e-05, 
       2.0258348840739947e-05, 
       1.7815881636178072e-05, 
       2.0416046338673935e-05, 
       1.7400553241784334e-05, 
       1.7546115789295338e-05, 
       1.8332272741464264e-05, 
       1.8617529477667705e-05, 
       1.7500691077864170e-05, 
       2.2626327304652426e-05, 
       2.2227514768022570e-05, 
       1.8188667994350754e-05, 
       1.7437844563341825e-05, 
       2.0314220634938714e-05, 
       1.7691905454582359e-05, 
       1.8765674276470071e-05, 
       2.3082902752773052e-05, 
       1.7706569415143102e-05, 
       1.8228139491229842e-05, 
       2.1552573926884468e-05, 
       1.8519689604015046e-05, 
       2.3993102372972416e-05, 
       2.4288245220394133e-05, 
       2.2183634289793656e-05, 
       1.9552071132927956e-05, 
       2.0759479813238321e-05, 
       1.8457204441878411e-05, 
       1.7292463119650519e-05, 
       1.9596345964718906e-05, 
       2.4327606169512865e-05, 
       2.4339106425848476e-05, 
       2.0285040788588862e-05, 
       1.9862348129452729e-05, 
       1.7963271883623455e-05, 
       2.3958109454929556e-05, 
       2.0081950436156026e-05, 
       2.1879464641930097e-05, 
       2.3311196480404057e-05, 
       2.3664991134883837e-05, 
       1.7570857835264645e-05, 
       2.3816467195815987e-05, 
       2.3969057885338020e-05, 
       1.7684340607261483e-05, 
       1.7130623687063773e-05, 
       1.8004413730683098e-05, 
       2.4192829489753538e-05, 
       1.7642895397066091e-05, 
       1.8061073828648228e-05, 
       1.9744182333377942e-05, 
       1.7700876766767885e-05, 
       1.7687369787554525e-05, 
       1.7661716713810105e-05, 
       2.4112509382338131e-05, 
       1.7251645577806190e-05, 
       1.8318965404654398e-05, 
       1.7958747158205470e-05, 
       2.0141697344594615e-05, 
       2.4078595848649804e-05, 
       1.7158024940848134e-05, 
       2.3587342040018002e-05, 
       2.1353213766013239e-05, 
       1.7632049491289552e-05, 
       1.7553541999937986e-05, 
       2.0632681521166749e-05, 
       2.0632681521166749e-05, 
       2.0094466779403417e-05, 
       1.8242622188536006e-05, 
       2.2661751330096227e-05, 
       2.1655401501450158e-05, 
       2.2973915559528669e-05, 
       1.7857781946628933e-05, 
       2.3874064025257466e-05, 
       1.7466289208002739e-05, 
       1.7868190241998999e-05, 
       1.8522292841140415e-05, 
       1.9541454144670358e-05, 
       2.0360858386803245e-05, 
       1.7745433564095962e-05, 
       2.1450608391854456e-05, 
       1.7873924089632360e-05, 
       1.7873924089632360e-05, 
       1.7642268004839998e-05, 
       1.7007271490848372e-05, 
       2.3008785724228431e-05, 
       2.3012536452576482e-05, 
       1.8835388379737464e-05, 
       2.3888953016319224e-05, 
       1.8257329889199428e-05, 
       1.8056709382823948e-05, 
       1.9869298923376563e-05, 
       2.1964741924384022e-05, 
       1.9822422962427555e-05, 
       1.7723877559109750e-05, 
       1.9752375106067294e-05, 
       1.7562414468378501e-05, 
       2.3820159096627734e-05, 
       1.7849041965527568e-05, 
       1.8178132738886788e-05, 
       1.7933421962478748e-05, 
       2.0442580535800413e-05, 
       1.8737388710914070e-05, 
       1.8010655480464308e-05, 
       1.7499746820761987e-05, 
       1.9628184551172320e-05, 
       2.0135454702788720e-05, 
       1.9783768630407857e-05, 
       1.7027457476285222e-05, 
       2.0439839165013294e-05, 
       1.7668140506123545e-05, 
       2.2887222766642335e-05, 
       1.8625722075590894e-05, 
       1.6774244951824475e-05, 
       1.9355861887763674e-05, 
       1.9346576290592440e-05, 
       1.8886452560261844e-05, 
       2.3428397754813264e-05, 
       1.9147065979715298e-05, 
       1.7953202929927740e-05, 
       2.0126846824773085e-05, 
       2.0276715003319864e-05, 
       2.3371966026493449e-05, 
       1.7718215711264086e-05, 
       1.8327956841474887e-05, 
       2.3475965054884663e-05, 
       2.1220863521424081e-05, 
       1.8344353342725405e-05, 
       2.3379306655953676e-05, 
       1.7756953192344690e-05, 
       1.8927844511496484e-05, 
       2.1319615922844804e-05, 
       2.3149571951534485e-05, 
       1.7266925840009747e-05, 
       1.9727835207955215e-05, 
       1.9493125385176378e-05, 
       2.2569242370532613e-05, 
       1.6570225727416707e-05, 
       2.0761943236018171e-05, 
       1.9277691130070734e-05, 
       1.7412207746104813e-05, 
       1.6794022527098285e-05, 
       1.9911347833599989e-05, 
       2.1787554130159452e-05, 
       1.8588028392662681e-05, 
       1.6711214286223893e-05, 
       1.8880801601497662e-05, 
       1.7805419113968902e-05, 
       1.8199539146443171e-05, 
       1.8193725676805197e-05, 
       2.2137212536092185e-05, 
       1.9290095584840817e-05, 
       1.7483531629355937e-05, 
       1.7263911772972458e-05, 
       1.9291348274936961e-05, 
       2.0711584120144664e-05, 
       1.7711451284701733e-05, 
       1.9679286515370669e-05, 
       2.2966962280570155e-05, 
       1.9212163216954399e-05, 
       1.7255996582263717e-05, 
       1.7231387705174675e-05, 
       2.1930982813245604e-05, 
       1.7410850631496899e-05, 
       1.7410850631496899e-05, 
       1.8162260557585636e-05, 
       1.8838587001928264e-05, 
       1.7291093427628760e-05, 
       2.1562991469965018e-05, 
       2.2955954995703078e-05, 
       2.2138774233384432e-05, 
       1.6891504861024997e-05, 
       1.6924801108667900e-05, 
       1.8636187270804091e-05, 
       1.9854450989405549e-05, 
       2.2187440342377361e-05, 
       1.6336203316387930e-05, 
       1.6336203316387930e-05, 
       2.2125101210696940e-05, 
       1.7199656862921846e-05, 
       2.1163216656948455e-05, 
       2.1163216656948455e-05, 
       1.8459217397607133e-05, 
       1.6337360455581812e-05, 
       2.2125176236958186e-05, 
       2.2550082661318860e-05, 
       1.6234615335634816e-05, 
       2.1496712981959119e-05, 
       2.0485376621618394e-05, 
       1.7494093464794439e-05, 
       1.6487806552309386e-05, 
       2.1110132354203416e-05, 
       1.8702551269710746e-05, 
       2.2865396123724519e-05, 
       2.0350022676992984e-05, 
       2.2838498108133499e-05, 
       1.9784249880636262e-05, 
       2.0292191214828813e-05, 
       1.8685372482041398e-05, 
       2.2655086304048908e-05, 
       1.7304808887747554e-05, 
       1.6703681152645012e-05, 
       2.2788125298875008e-05, 
       2.2128691640789324e-05, 
       1.9293006539837878e-05, 
       1.8616979793947943e-05, 
       2.1754197699099749e-05, 
       2.2370830009755466e-05, 
       2.2370830009755466e-05, 
       2.1058322901314090e-05, 
       2.1082277701276365e-05, 
       2.2724673823789762e-05, 
       1.6722677059682963e-05, 
       2.2611698338101417e-05, 
       1.8820708615568329e-05, 
       1.9115688755658775e-05, 
       1.6234721814418811e-05, 
       1.7084719343943476e-05, 
       1.9366446164047562e-05, 
       1.6259222281809273e-05, 
       1.6335206644752317e-05, 
       1.7313040984674325e-05, 
       1.7313040984674325e-05, 
       1.9470156832045873e-05, 
       1.6122471032662713e-05, 
       1.6224475435896360e-05, 
       2.2642168998363644e-05, 
       2.2632537119189582e-05, 
       1.6592295820877356e-05, 
       2.1366640852304917e-05, 
       1.7087754208338704e-05, 
       1.6142464685406160e-05, 
       1.8212730203929584e-05, 
       1.6062205609472556e-05, 
       1.7317044300247156e-05, 
       1.6710506133544104e-05, 
       1.6718948384614613e-05, 
       1.6718948384614613e-05, 
       2.0992852238434180e-05, 
       2.1186438281349700e-05, 
       1.6373805636870852e-05, 
       1.6953638512254993e-05, 
       1.8072305094187577e-05, 
       1.8056140424669700e-05, 
       1.8056140424669700e-05, 
       1.6098907761922259e-05, 
       2.2234532934795830e-05, 
       1.9527558213273912e-05, 
       1.7716118858467923e-05, 
       2.2554816209846009e-05, 
       1.6405645902497258e-05, 
       1.6507591631663653e-05, 
       1.6028698901246867e-05, 
       1.6610357546445781e-05, 
       2.0891228865120608e-05, 
       1.7205477237655727e-05, 
       1.6285032053036909e-05, 
       1.6102190176239627e-05, 
       2.0038516713607140e-05, 
       1.8727638036245882e-05, 
       1.9418850589350212e-05, 
       2.0003273484876013e-05, 
       2.1906878577715012e-05, 
       1.6532627116963400e-05, 
       1.7667379260515993e-05, 
       1.7667379260515993e-05, 
       1.8996600645954346e-05, 
       1.6474853520918024e-05, 
       1.6754965910496321e-05, 
       1.9131977728321137e-05, 
       2.0359016436841823e-05, 
       1.6171135481124933e-05, 
       1.6994505531946206e-05, 
       1.6994505531946206e-05, 
       2.2407763531605025e-05, 
       1.8617808635099769e-05, 
       1.7970666720639915e-05, 
       1.7818094752935917e-05, 
       2.2278244153232194e-05, 
       1.7807560102282335e-05, 
       2.2292872493900075e-05, 
       1.6011178376679929e-05, 
       2.1130905491636213e-05, 
       1.8389816834979284e-05, 
       1.7518317671006143e-05, 
       1.8386936279129174e-05, 
       1.9289791732180916e-05, 
       1.8390802048013865e-05, 
       2.1841772793681454e-05, 
       1.7023838490886034e-05, 
       1.6430075484698597e-05, 
       1.5794344155936214e-05, 
       1.6383421816699133e-05, 
       2.1836304668254267e-05, 
       1.5947436384747194e-05, 
       1.6658611312546870e-05, 
       1.8996568006323307e-05, 
       1.5726667288329263e-05, 
       1.7203608654114123e-05, 
       1.6603196937560013e-05, 
       1.6420403016835570e-05, 
       1.8458044682778509e-05, 
       2.0504660640626798e-05, 
       2.1163618511902066e-05, 
       2.1735775776141427e-05, 
       1.9291869054352809e-05, 
       2.2156388020013238e-05, 
       1.5738661564135976e-05, 
       1.8892177115344497e-05, 
       1.7044119165090520e-05, 
       1.7873537991545251e-05, 
       1.7742036488274337e-05, 
       1.5974291757940324e-05, 
       1.7725308376978671e-05, 
       2.2107213301778072e-05, 
       1.8390227544060127e-05, 
       2.0029172972782206e-05, 
       1.7434739033936376e-05, 
       1.5653113198686018e-05, 
       1.6068993551798857e-05, 
       1.8932653957189339e-05, 
       2.0256523544959617e-05, 
       1.9252343903262409e-05, 
       1.6044104462389430e-05, 
       2.1089221346517688e-05, 
       1.6043761918339424e-05, 
       1.5997479462943513e-05, 
       1.8328406323002826e-05, 
       2.1954356616487514e-05, 
       1.9428513590515729e-05, 
       1.8019306688431413e-05, 
       1.5795121486224743e-05, 
       1.5579582249498965e-05, 
       1.7103957557751057e-05, 
       1.5570399547994975e-05, 
       2.1849104957647042e-05, 
       1.8073135630927389e-05, 
       1.6513389478517118e-05, 
       1.8273652355757713e-05, 
       2.1144803368417817e-05, 
       2.0270387868887738e-05, 
       1.6240298468458713e-05, 
       1.9746705821200908e-05, 
       1.8176373571861887e-05, 
       1.7468726387310578e-05, 
       1.8237445111701578e-05, 
       1.7451376992430054e-05, 
       1.7044135398653662e-05, 
       1.7044135398653662e-05, 
       1.9502404229276985e-05, 
       1.6069453006010668e-05, 
       1.6385605152059634e-05, 
       2.1787275947365716e-05, 
       1.6092117797190203e-05, 
       2.1808978138339834e-05, 
       1.8910890863139921e-05, 
       1.7320251751630813e-05, 
       1.6669209234786041e-05, 
       2.0002068595933454e-05, 
       2.0721517790829148e-05, 
       2.1175748194723008e-05, 
       1.6352279732357424e-05, 
       1.5588799981723169e-05, 
       1.5596929345692841e-05, 
       2.1598519054007094e-05, 
       1.6341032883464118e-05, 
       1.6037577515563460e-05, 
       1.8198648681853879e-05, 
       1.8198648681853879e-05, 
       1.6304655116409151e-05, 
       2.1106709550229482e-05, 
       2.1617499740261796e-05, 
       1.5429339135867125e-05, 
       1.5912453233245103e-05, 
       1.5478302911601509e-05, 
       1.6553374800561391e-05, 
       1.6570634637199210e-05, 
       1.7597865511431501e-05, 
       1.5439674044828063e-05, 
       1.9608929313707292e-05, 
       1.7060817982284103e-05, 
       1.6504583528432646e-05, 
       2.0423577083621921e-05, 
       1.5426516787865581e-05, 
       1.5426516787865581e-05, 
       1.8991887595935154e-05, 
       1.5432803594276111e-05, 
       1.9414015532927609e-05, 
       1.5492143234340858e-05, 
       1.6790153048861613e-05, 
       2.0000000000002919e-05, 
       2.0000000000000425e-05, 
       2.0000000000000425e-05, 
       2.0000000000002302e-05, 
       2.0000000000002302e-05, 
       2.0123214111927458e-05, 
       1.9134195875659283e-05, 
       2.1611367090571530e-05, 
       1.8053293945355630e-05, 
       1.5880767113893326e-05, 
       1.7723619065575681e-05, 
       1.7026402635406914e-05, 
       1.7941155666721253e-05, 
       2.1239128676142639e-05, 
       1.5532530466566838e-05, 
       1.6092455651163923e-05, 
       2.1614673492820541e-05, 
       1.5302442165784327e-05, 
       1.5371800463024751e-05, 
       1.6301383742497040e-05, 
       1.5372985229919656e-05, 
       1.6134008305294374e-05, 
       1.6335296778040991e-05, 
       1.6841040774595513e-05, 
       1.6073508541610085e-05, 
       1.5337011740924597e-05, 
       1.9327005042392512e-05, 
       2.0936607644920401e-05, 
       1.8438087968188434e-05, 
       1.6159618091167911e-05, 
       1.5691290674776830e-05, 
       1.7512542040997598e-05, 
       1.6912588626088647e-05, 
       1.5788332881296314e-05, 
       2.1503293331511378e-05, 
       1.8532975911561430e-05, 
       1.5200325041619105e-05, 
       1.8052713942789469e-05, 
       1.8536055581128455e-05, 
       1.9138944246306821e-05, 
       1.8197005931973759e-05, 
       1.8197005931973759e-05, 
       1.5326171036394532e-05, 
       1.6549433215411359e-05, 
       1.5277926910026729e-05, 
       1.6352611865435243e-05, 
       2.1408440169012478e-05, 
       2.1331593915833227e-05, 
       2.1162908630631478e-05, 
       1.7856444264198156e-05, 
       1.9819216586925235e-05, 
       1.5913074729844049e-05, 
       1.6959108126368575e-05, 
       1.9625463956564047e-05, 
       1.6022982921782222e-05, 
       2.1335613287615755e-05, 
       1.7336758653690069e-05, 
       1.7336417810835606e-05, 
       1.6014421515239101e-05, 
       1.8274039956081468e-05, 
       1.7850855189331426e-05, 
       1.5081305769075736e-05, 
       2.1232787288345027e-05, 
       1.5097430599505189e-05, 
       1.5160385257255164e-05, 
       2.0472366386909940e-05, 
       1.5840265349244938e-05, 
       1.8218932701100731e-05, 
       2.0493155196437179e-05, 
       1.7104780513039831e-05, 
       2.1237469975771126e-05, 
       1.5993331306734184e-05, 
       1.7144170276246923e-05, 
       1.8142196848276659e-05, 
       1.9056615108258974e-05, 
       2.1017973319543292e-05, 
       1.6630590717646387e-05, 
       1.8026372337631166e-05, 
       1.5663953166791747e-05, 
       1.8677585553408706e-05, 
       1.9005941589898054e-05, 
       2.0491367724795340e-05, 
       1.5398119269448739e-05, 
       1.7795917071237724e-05, 
       1.6525252283402562e-05, 
       2.0935352653952154e-05, 
       1.7020357703059911e-05, 
       1.6884866524177841e-05, 
       2.1049454017331673e-05, 
       2.0173245753953342e-05, 
       2.1006587041820796e-05, 
       1.5840673816212893e-05, 
       1.6560409888110331e-05, 
       1.5810346852933602e-05, 
       2.0800370930636108e-05, 
       2.0847348923073888e-05, 
       1.7067499214287063e-05, 
       1.5013179642766518e-05, 
       1.7714997793260417e-05, 
       1.6956928977768807e-05, 
       1.9999999999997342e-05, 
       1.9999999999997342e-05, 
       1.8456261803637364e-05, 
       1.5912687180576364e-05, 
       1.4960456400075546e-05, 
       1.4888882831800838e-05, 
       1.8184846829995311e-05, 
       1.8184846829995311e-05, 
       1.4853009727722440e-05, 
       2.0578529469769259e-05, 
       2.0977449158401122e-05, 
       1.7443887309016557e-05, 
       1.8909286191306884e-05, 
       1.6966481931461809e-05, 
       1.6556459995739591e-05, 
       1.6752037303715518e-05, 
       2.0018554255881626e-05, 
       1.5628571172107139e-05, 
       1.4842777408290094e-05, 
       1.7310397120016435e-05, 
       1.4947440150134089e-05, 
       1.5407206684160909e-05, 
       1.5407206684160909e-05, 
       1.6490443897643008e-05, 
       2.0884794716715465e-05, 
       1.8488805778599490e-05, 
       1.6898576170893853e-05, 
       1.4774242673951012e-05, 
       1.5632457593345169e-05, 
       1.7261613309425577e-05, 
       1.7155922347204155e-05, 
       1.5326543904164046e-05, 
       1.7326792718584700e-05, 
       1.5319825549543372e-05, 
       1.5463498838618996e-05, 
       1.5463498838618996e-05, 
       1.7316095803796125e-05, 
       1.5598099978153576e-05, 
       1.7298086056708285e-05, 
       2.0678661369356820e-05, 
       1.7099494813622297e-05, 
       1.5728453803341573e-05, 
       1.7432757133150035e-05, 
       1.5160168419881993e-05, 
       1.7816461753330562e-05, 
       1.9580880461032115e-05, 
       1.6472916584417822e-05, 
       2.0474582919533831e-05, 
       1.6492713123745396e-05, 
       1.5602647159461790e-05, 
       1.7417914327781255e-05, 
       1.7430518082606324e-05, 
       1.7200872561710810e-05, 
       1.7594505467611897e-05, 
       1.4593374291172440e-05, 
       1.4615829990183012e-05, 
       1.6970189109472224e-05, 
       1.4595680036653840e-05, 
       1.7158899171503057e-05, 
       2.0387654478843343e-05, 
       1.4978906698363142e-05, 
       2.0155956683865661e-05, 
       1.7187787360644636e-05, 
       1.4789011693835631e-05, 
       1.7666481865181118e-05, 
       1.6470282452882369e-05, 
       2.0284471522640487e-05, 
       1.5267740543673378e-05, 
       1.6462580885715711e-05, 
       1.5115179205604446e-05, 
       1.8278406388334036e-05, 
       1.6444101319305045e-05, 
       1.5539970999924486e-05, 
       1.7290020306619490e-05, 
       1.4581378540356226e-05, 
       1.5180179612361038e-05, 
       1.4942897866165691e-05, 
       1.9743080684653949e-05, 
       1.9989758912136145e-05, 
       1.5008135119909638e-05, 
       1.7373965946285542e-05, 
       1.4479154891499769e-05, 
       1.4485316501184317e-05, 
       1.7536488865533303e-05, 
       1.9623937145625572e-05, 
       1.4457394980177487e-05, 
       1.6319321257813999e-05, 
       1.4539641364728309e-05, 
       1.5276729671877384e-05, 
       1.7674272465634721e-05, 
       1.8579230583364200e-05, 
       1.7681898591113989e-05, 
       1.8838002625992734e-05, 
       1.7500734554702812e-05, 
       1.4430863000435895e-05, 
       1.4535642083636242e-05, 
       2.0031152627917139e-05, 
       1.4577317255433693e-05, 
       1.4715791150096691e-05, 
       2.0295526242962622e-05, 
       1.6377442125886164e-05, 
       2.0319378355387940e-05, 
       1.5185683324654169e-05, 
       1.8722989770466373e-05, 
       1.7157723980489144e-05, 
       1.9649753986370153e-05, 
       2.0246860646734095e-05, 
       1.7780798413728221e-05, 
       1.6378411618912062e-05, 
       1.7854347085060371e-05, 
       2.0280975871608747e-05, 
       1.6367406998208899e-05, 
       1.6765526193631551e-05, 
       1.6129318341749104e-05, 
       1.4459745708239373e-05, 
       1.7117545156381083e-05, 
       1.6850357142189432e-05, 
       1.6564115568313463e-05, 
       1.5252014388780991e-05, 
       1.4721371293476008e-05, 
       1.4458951834839838e-05, 
       1.6766171326926528e-05, 
       1.5083196282175192e-05, 
       1.5320761042333061e-05, 
       1.5320761042333061e-05, 
       1.9027424464127888e-05, 
       1.4749700525073902e-05, 
       1.6683798331944300e-05, 
       1.6791999751125377e-05, 
       1.7706857203696913e-05, 
       2.0005296688531713e-05, 
       1.6595973368406404e-05, 
       1.5034032999894572e-05, 
       1.5126322773709885e-05, 
       1.9986363807275739e-05, 
       1.6709325941348302e-05, 
       2.0053850375514413e-05, 
       1.4223540189186284e-05, 
       2.0037132798653616e-05, 
       1.4306005111882178e-05, 
       1.4784458429091257e-05, 
       1.4784458429091257e-05, 
       1.9792067331488810e-05, 
       1.4507994240964273e-05, 
       1.4570284655908726e-05, 
       1.4226306273688533e-05, 
       1.6175380175445676e-05, 
       1.4650543477103646e-05, 
       1.4695437694652338e-05, 
       1.4694844206046384e-05, 
       1.4753403157452437e-05, 
       1.4753403157452437e-05, 
       1.6369736954885356e-05, 
       1.9449524363557376e-05, 
       1.6025121652272348e-05, 
       2.0000000000000503e-05, 
       1.9999999999997362e-05, 
       1.9999999999997538e-05, 
       2.0000000000000852e-05, 
       1.9999999999998558e-05, 
       1.9999999999998558e-05, 
       2.0000000000004742e-05, 
       2.0000000000001686e-05, 
       2.0000000000001601e-05, 
       2.0000000000002875e-05, 
       2.0000000000000862e-05, 
       1.9999999999999382e-05, 
       1.9999999999998247e-05, 
       1.9999999999999551e-05, 
       2.0000000000001116e-05, 
       1.9999999999999138e-05, 
       2.0000000000000283e-05, 
       2.0000000000000947e-05, 
       2.0000000000001929e-05, 
       1.4275144858379013e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.8292226618068275e-05, 
       1.5084418188810893e-05, 
       1.8290229564389832e-05, 
       1.6578154038873046e-05, 
       1.4297554586141315e-05, 
       1.9496177904183695e-05, 
       1.8011077407310878e-05, 
       1.4898523869975374e-05, 
       1.4111929008584934e-05, 
       1.7128423186652288e-05, 
       1.4954562209032282e-05, 
       1.6188877412179246e-05, 
       1.6185805494386691e-05, 
       1.8582378366695840e-05, 
       1.4482294464459094e-05, 
       1.5458285924510495e-05, 
       1.5875604022001212e-05, 
       1.4726196961436846e-05, 
       1.4726196961436846e-05, 
       1.7644837065831166e-05, 
       1.4225061342693049e-05, 
       1.5357222718776597e-05, 
       1.4221112565537283e-05, 
       1.4649874679356803e-05, 
       1.9174587312672647e-05, 
       1.5452373965760110e-05, 
       1.9695206628413968e-05, 
       1.4559764239267428e-05, 
       1.5077726642019496e-05, 
       1.6341843706471940e-05, 
       1.4650695995497018e-05, 
       1.4874401355918175e-05, 
       1.4611813467941683e-05, 
       1.6536959336557455e-05, 
       1.4095585803147418e-05, 
       1.8110808860485055e-05, 
       1.8935788653751514e-05, 
       1.6405407101598671e-05, 
       1.9494713950402057e-05, 
       1.9818464645475865e-05, 
       1.5641623130648651e-05, 
       1.8965868148730674e-05, 
       1.4786853098410819e-05, 
       1.7511499766096822e-05, 
       1.7371800959598735e-05, 
       1.7371800959598735e-05, 
       1.9693059287692170e-05, 
       1.9525000342476939e-05, 
       1.6348177454292356e-05, 
       1.5412157021002226e-05, 
       1.6256106337729249e-05, 
       1.6915966439772749e-05, 
       1.5157066212291830e-05, 
       1.4031057972296246e-05, 
       1.3952467785550451e-05, 
       1.6306341402721227e-05, 
       1.4019159315126013e-05, 
       1.6934626541642875e-05, 
       1.6306443902052956e-05, 
       1.6795545556722046e-05, 
       1.6366777185526895e-05, 
       1.7257599135686726e-05, 
       1.8908723722688233e-05, 
       1.3902432785640216e-05, 
       1.6320478483971147e-05, 
       1.6214035080290748e-05, 
       1.4869089766760915e-05, 
       1.5366423107994453e-05, 
       1.6162850240229664e-05, 
       1.9065158632882171e-05, 
       1.7313274740991905e-05, 
       1.7223009077964974e-05, 
       1.9398913964943017e-05, 
       1.5087951775884419e-05, 
       1.7812042220933701e-05, 
       1.4347426818214703e-05, 
       1.7792253676004883e-05, 
       1.5090639868874618e-05, 
       1.5067922795395602e-05, 
       1.7423519498994054e-05, 
       1.6436411634818005e-05, 
       1.9338856410587927e-05, 
       1.5134194369259809e-05, 
       1.9520627370044779e-05, 
       1.8173420693764944e-05, 
       1.3897658431095295e-05, 
       1.3805705622813171e-05, 
       1.9509538609945451e-05, 
       1.4746339834957969e-05, 
       1.6196914275664551e-05, 
       1.9423555238125053e-05, 
       1.6255504681146960e-05, 
       1.4448467695138742e-05, 
       1.9416435738647281e-05, 
       1.5091471276995217e-05, 
       1.5826070551859609e-05, 
       1.6573016682227819e-05, 
       1.3796127577441248e-05, 
       1.9433281450590757e-05, 
       1.6004114690697736e-05, 
       1.5913985293590652e-05, 
       1.9134029090375728e-05, 
       1.9419992242537505e-05, 
       1.8399476874794269e-05, 
       1.5565638875817932e-05, 
       1.8175752840565924e-05, 
       1.9257476212652068e-05, 
       1.7320699429900624e-05, 
       1.9393995692260808e-05, 
       1.7736044064910271e-05, 
       1.9249993135876063e-05, 
       1.7332491817108567e-05, 
       1.4705820446896152e-05, 
       1.8873422116603438e-05, 
       1.3714174653088546e-05, 
       1.5053238176136231e-05, 
       1.6000793488734814e-05, 
       1.7471407309996013e-05, 
       1.9115617417499850e-05, 
       1.4035041327082734e-05, 
       1.5154780626558336e-05, 
       1.3924333585404814e-05, 
       1.3924333585404814e-05, 
       1.5501360377022928e-05, 
       1.3797425075843989e-05, 
       1.6690377565218823e-05, 
       1.4209392044125464e-05, 
       1.5192168921665653e-05, 
       1.9310310714502314e-05, 
       1.6026681801802033e-05, 
       1.5379446118655621e-05, 
       1.6182087572781853e-05, 
       1.4414278015070796e-05, 
       1.4403712747404126e-05, 
       1.9106139992569447e-05, 
       1.6279535540267637e-05, 
       1.8823522614061252e-05, 
       1.8823522614061252e-05, 
       1.3992514652115460e-05, 
       1.6417989200898137e-05, 
       1.9062044393849746e-05, 
       1.4444584898110234e-05, 
       1.4156201937858308e-05, 
       1.6980828150998573e-05, 
       1.8225599700613522e-05, 
       1.3842949214899192e-05, 
       1.3925324343753706e-05, 
       1.9199250441596477e-05, 
       1.6340884995954673e-05, 
       1.4097997925162449e-05, 
       1.5110802450527575e-05, 
       1.5110802450527575e-05, 
       1.4118188534189489e-05, 
       1.5972507348626299e-05, 
       1.4949117314399967e-05, 
       1.3634268769191239e-05, 
       1.9176981138413062e-05, 
       1.5747926190215031e-05, 
       1.4529697734581688e-05, 
       1.9035875295255347e-05, 
       1.8644242886533229e-05, 
       1.7019058967098475e-05, 
       1.8943571938968203e-05, 
       1.4070283982784185e-05, 
       1.6521567681950814e-05, 
       1.5736450260849093e-05, 
       1.4478110207350232e-05, 
       1.5891044823812513e-05, 
       1.8396317263515655e-05, 
       1.3654047047661890e-05, 
       1.4949344645606630e-05, 
       1.3950721742957454e-05, 
       1.3629239211560011e-05, 
       1.8770199721783211e-05, 
       1.5625283557334548e-05, 
       1.5333111283465042e-05, 
       1.6752068316668764e-05, 
       1.4307254727976837e-05, 
       1.7834629041524526e-05, 
       1.4638886206675482e-05, 
       1.4682563538763840e-05, 
       1.8007016967909183e-05, 
       1.4713241671169784e-05, 
       1.3856152861851609e-05, 
       1.5598944511325250e-05, 
       1.5406023337684634e-05, 
       1.8530468800527959e-05, 
       1.6489273021796442e-05, 
       1.7716550230181951e-05, 
       1.8594642013859483e-05, 
       1.3422890685954698e-05, 
       1.3589465871217270e-05, 
       1.3387344641485087e-05, 
       1.3880605412900500e-05, 
       1.8098814396644403e-05, 
       1.3778090850651112e-05, 
       1.6259222281811902e-05, 
       1.4175680983774898e-05, 
       1.4109463605866338e-05, 
       1.8385627478743485e-05, 
       1.3521687157485570e-05, 
       1.8651763803404701e-05, 
       1.3377877231170490e-05, 
       1.6225787460026322e-05, 
       1.5715779055579045e-05, 
       1.6034263328024173e-05, 
       1.4448337077626683e-05, 
       1.5350431184082312e-05, 
       1.8644281493184915e-05, 
       1.3766684015347864e-05, 
       1.8383946509146261e-05, 
       1.6834144086824390e-05, 
       1.8493761399307631e-05, 
       1.6490789150431061e-05, 
       1.7583798779864384e-05, 
       1.5272064378446065e-05, 
       1.5480765625171037e-05, 
       1.8393271799071932e-05, 
       1.4532019910555164e-05, 
       1.4149199393426439e-05, 
       1.3723138294747032e-05, 
       1.3823324093956162e-05, 
       1.3776209334212682e-05, 
       1.4839416795349127e-05, 
       1.8067420031682479e-05, 
       1.3468431700565305e-05, 
       1.3290548121645528e-05, 
       1.3593186316793604e-05, 
       1.4968574602007244e-05, 
       1.8210432925077731e-05, 
       1.5053902918481532e-05, 
       1.8555653900424379e-05, 
       1.8569914396785366e-05, 
       1.3180648660796089e-05, 
       1.5043968157884262e-05, 
       1.3206602859679263e-05, 
       1.3554147747746022e-05, 
       1.3860988911090755e-05, 
       1.3185581351947644e-05, 
       1.8513598646233649e-05, 
       1.7849408706365214e-05, 
       1.8499087714372173e-05, 
       1.3993618033684512e-05, 
       1.4170496345724529e-05, 
       1.3152985080876476e-05, 
       1.4377014272742440e-05, 
       1.5416237163161959e-05, 
       1.5415667785852915e-05, 
       1.7331183274922722e-05, 
       1.3210886754495558e-05, 
       1.5382921909012251e-05, 
       1.3431839782307517e-05, 
       1.5947436384756142e-05, 
       1.7884633014238202e-05, 
       1.3073642148904387e-05, 
       1.8440413867756214e-05, 
       1.6756083051915222e-05, 
       1.3492696200253430e-05, 
       1.8198691373090194e-05, 
       1.3135375943926957e-05, 
       1.3471624840180043e-05, 
       1.3656889977458795e-05, 
       1.3020158379448295e-05, 
       1.7666894613209809e-05, 
       1.5341635851169986e-05, 
       1.5679624049124903e-05, 
       1.8397222288501354e-05, 
       1.5728709149685157e-05, 
       1.3917938394849733e-05, 
       1.3193237808113149e-05, 
       1.3567494731282809e-05, 
       1.3153650027517457e-05, 
       1.8167269613865688e-05, 
       1.3983227712855635e-05, 
       1.3066869628852629e-05, 
       1.3435099973742444e-05, 
       1.8034381080621541e-05, 
       1.6707778403101021e-05, 
       1.6696937979988921e-05, 
       1.5361515935679014e-05, 
       1.4770516228118091e-05, 
       1.4970903524730745e-05, 
       1.5238537885689830e-05, 
       1.4842777408289981e-05, 
       1.4780444790771307e-05, 
       1.8094725257512659e-05, 
       1.4967704431308001e-05, 
       1.8281152666798779e-05, 
       1.3595710434242635e-05, 
       1.8197931748287858e-05, 
       1.5853880063817633e-05, 
       1.3663150845888761e-05, 
       1.5454779573915931e-05, 
       1.5454779573915931e-05, 
       1.6673457291542528e-05, 
       1.6939810631577229e-05, 
       1.6939525358680649e-05, 
       1.3417110962764424e-05, 
       1.3417110962764424e-05, 
       1.7932256526001101e-05, 
       1.5690038087088703e-05, 
       1.5208755176494659e-05, 
       1.3488650103220160e-05, 
       1.3012791257583799e-05, 
       1.7499930853069511e-05, 
       1.3150432078271197e-05, 
       1.7467308246123641e-05, 
       1.3612116310638862e-05, 
       1.8081444624459543e-05, 
       1.6038070826472913e-05, 
       1.3105512486918082e-05, 
       1.4876698672437411e-05, 
       1.5369073508425687e-05, 
       1.4496813536277749e-05, 
       1.7650753964143016e-05, 
       1.5663433034602309e-05, 
       1.5106033963424716e-05, 
       1.6871251797605927e-05, 
       1.3413837951967647e-05, 
       1.5125577788070745e-05, 
       1.3552799548837164e-05, 
       1.6002207175500304e-05, 
       1.5808465650411301e-05, 
       1.8034202182915985e-05, 
       1.4459188583216419e-05, 
       1.4398822522986248e-05, 
       1.7701252955455914e-05, 
       1.7608637086252329e-05, 
       1.6744852224104611e-05, 
       1.7376241656315121e-05, 
       1.7348163414595373e-05, 
       1.5253214219338255e-05, 
       1.5253214219338255e-05, 
       1.8098285110705944e-05, 
       1.7842467350696516e-05, 
       1.5559274863015924e-05, 
       1.4561998883880659e-05, 
       1.5036896330495797e-05, 
       1.7971341195498525e-05, 
       1.2808210408468525e-05, 
       1.7405777785703126e-05, 
       1.7007270547102020e-05, 
       1.7499582875733780e-05, 
       1.8032706326538459e-05, 
       1.3480381240299832e-05, 
       1.4784256759997464e-05, 
       1.4682439644155401e-05, 
       1.3287028623768412e-05, 
       1.4484149999385861e-05, 
       1.6086427499347982e-05, 
       1.4921600419341173e-05, 
       1.6050879829408097e-05, 
       1.6430875556361242e-05, 
       1.4926256281201423e-05, 
       1.4409678223866859e-05, 
       1.4604845285442554e-05, 
       1.5893647408508373e-05, 
       1.7770542439493338e-05, 
       1.4849712600315778e-05, 
       1.4867061603209816e-05, 
       1.4578200825941145e-05, 
       1.3442171322900818e-05, 
       1.5386454023643783e-05, 
       1.2734060603846992e-05, 
       1.3352918692079359e-05, 
       1.4094315592579050e-05, 
       1.5533355736466470e-05, 
       1.5010608987705953e-05, 
       1.2760952897130246e-05, 
       1.2760952897130246e-05, 
       1.4377882311814148e-05, 
       1.4271191270747151e-05, 
       1.7147380911302687e-05, 
       1.7829846918456326e-05, 
       1.2978113923474221e-05, 
       1.2978113923474221e-05, 
       1.2800607764655122e-05, 
       1.7213613475570351e-05, 
       1.3170890314236146e-05, 
       1.3130123945228694e-05, 
       1.4051891351076992e-05, 
       1.7747538488951913e-05, 
       1.3254199608808187e-05, 
       1.5170060201130070e-05, 
       1.4804926255523652e-05, 
       1.7600723611033809e-05, 
       1.5285331596776503e-05, 
       1.5285331596776503e-05, 
       1.2834340235367653e-05, 
       1.3115351271004138e-05, 
       1.3278168088532595e-05, 
       1.2590310682667907e-05, 
       1.2616539312197141e-05, 
       1.2616539312197141e-05, 
       1.6239502110115467e-05, 
       1.3410059869523268e-05, 
       1.2739244896656666e-05, 
       1.3221287591580966e-05, 
       1.2619655692590581e-05, 
       1.6876150779395045e-05, 
       1.2761254104092911e-05, 
       1.2548240730186572e-05, 
       1.5504036107356138e-05, 
       1.2883017742100525e-05, 
       1.5482703441841687e-05, 
       1.4271414314979747e-05, 
       1.6237712184524923e-05, 
       1.4713665597139593e-05, 
       1.7519490454569938e-05, 
       1.3394370224564338e-05, 
       1.6447363289564098e-05, 
       1.7493372789529711e-05, 
       1.7675957442536731e-05, 
       1.3773721019678660e-05, 
       1.3632809795481211e-05, 
       1.4245151176498330e-05, 
       1.2671713762754604e-05, 
       1.3984775134693882e-05, 
       1.6340529027817826e-05, 
       1.6205885007740020e-05, 
       1.5479307111382507e-05, 
       1.4979060718438190e-05, 
       1.3251537322325006e-05, 
       1.3245180432867855e-05, 
       1.7640603615650854e-05, 
       1.6875280249171112e-05, 
       1.6197893075766203e-05, 
       1.7629239460338853e-05, 
       1.3952383661868425e-05, 
       1.2706963798939207e-05, 
       1.6427492641259547e-05, 
       1.4044605099708595e-05, 
       1.4108645504857299e-05, 
       1.5714139308584642e-05, 
       1.3415735146379706e-05, 
       1.3415735146379706e-05, 
       1.3833928002165956e-05, 
       1.4757991614962459e-05, 
       1.2503567669085296e-05, 
       1.6978977378017918e-05, 
       1.7572824210603769e-05, 
       1.5250019145736625e-05, 
       1.4021692643629530e-05, 
       1.3302507036345570e-05, 
       1.7264052493257041e-05, 
       1.2497796004100066e-05, 
       1.2443401476728252e-05, 
       1.4590982869632107e-05, 
       1.7495991400571698e-05, 
       1.7333265420611655e-05, 
       1.4544240948629550e-05, 
       1.5699269394890567e-05, 
       1.4630215557433655e-05, 
       1.7300478351871982e-05, 
       1.2378329505747897e-05, 
       1.5049836477043053e-05, 
       1.4834978998291240e-05, 
       1.5819836908762031e-05, 
       1.2375339233550892e-05, 
       1.2612583977005754e-05, 
       1.2538371429661807e-05, 
       1.6413248799603736e-05, 
       1.5025524595439353e-05, 
       1.7272799465774889e-05, 
       1.5647927718461434e-05, 
       1.5647927718461434e-05, 
       1.4805763062424026e-05, 
       1.2382999632591006e-05, 
       1.3310339837886794e-05, 
       1.6878513430882560e-05, 
       1.7227940946222278e-05, 
       1.7344081411861175e-05, 
       1.5302814970989250e-05, 
       1.7213004986443475e-05, 
       1.6130088687624654e-05, 
       1.5842188695780215e-05, 
       1.2538095956927858e-05, 
       1.4849813330292949e-05, 
       1.2801565767518424e-05, 
       1.6744115574630633e-05, 
       1.4384897928145751e-05, 
       1.2275123712787654e-05, 
       1.2275123712787654e-05, 
       1.6739318170894267e-05, 
       1.2291458867866464e-05, 
       1.2275240011089254e-05, 
       1.2319686348472004e-05, 
       1.3935460415845843e-05, 
       1.2560747955146874e-05, 
       1.4076398503961912e-05, 
       1.2740319679690527e-05, 
       1.7256056442118583e-05, 
       1.3065879010733709e-05, 
       1.2317403565032562e-05, 
       1.3929757073636674e-05, 
       1.5237747244415033e-05, 
       1.4776501382106372e-05, 
       1.4365729474100232e-05, 
       1.6603264069635658e-05, 
       1.3622059822933727e-05, 
       1.4911043224232789e-05, 
       1.2230871633200690e-05, 
       1.7107076903889589e-05, 
       1.5220259368270058e-05, 
       1.2910286416092554e-05, 
       1.4203808534110436e-05, 
       1.3234174537463257e-05, 
       1.2224037420052983e-05, 
       1.6660962321400290e-05, 
       1.2867605302766192e-05, 
       1.5908573856228586e-05, 
       1.4527999453640038e-05, 
       1.2853281289329972e-05, 
       1.2501838912653071e-05, 
       1.2446649622681885e-05, 
       1.2178918734974544e-05, 
       1.2178918734974544e-05, 
       1.2267360122042968e-05, 
       1.6650278213161410e-05, 
       1.2578933501377602e-05, 
       1.2852426510684521e-05, 
       1.2621328299211735e-05, 
       1.4746748772933602e-05, 
       1.6506990813598154e-05, 
       1.2311117626207192e-05, 
       1.2375227362546670e-05, 
       1.7066928687530193e-05, 
       1.4591846588279915e-05, 
       1.2762711398992711e-05, 
       1.2422509540169615e-05, 
       1.7034831309899153e-05, 
       1.4836276164334514e-05, 
       1.2510095761094118e-05, 
       1.3897268735376902e-05, 
       1.2756431345564313e-05, 
       1.6666206861287346e-05, 
       1.3701806360771379e-05, 
       1.5240098743275998e-05, 
       1.6933157802861485e-05, 
       1.3987630597275828e-05, 
       1.6239460021718436e-05, 
       1.2087122914189682e-05, 
       1.6968706058066496e-05, 
       1.6527730736026860e-05, 
       1.2238341461892484e-05, 
       1.4137614289372472e-05, 
       1.4157354101469494e-05, 
       1.3161443364041454e-05, 
       1.2697887288799018e-05, 
       1.6989548777283794e-05, 
       1.2132162113290513e-05, 
       1.4825963926568935e-05, 
       1.6525633023303252e-05, 
       1.3982563665918434e-05, 
       1.3614131502338191e-05, 
       1.4559643828280083e-05, 
       1.5413447901425963e-05, 
       1.2088372027115803e-05, 
       1.6735773988068960e-05, 
       1.2622847057133571e-05, 
       1.3655651535491917e-05, 
       1.2456712871527126e-05, 
       1.5892031373226817e-05, 
       1.2047173019681641e-05, 
       1.6896154279726885e-05, 
       1.6874378094446402e-05, 
       1.4041409516085108e-05, 
       1.5097369288541178e-05, 
       1.2236326210569871e-05, 
       1.4498492314310596e-05, 
       1.2129747073421302e-05, 
       1.2369703786470514e-05, 
       1.5982592891834027e-05, 
       1.3769009649801223e-05, 
       1.4001743229159612e-05, 
       1.6540876044223736e-05, 
       1.6221164609751807e-05, 
       1.3763053020101112e-05, 
       1.4325095098802108e-05, 
       1.2633952474857119e-05, 
       1.6329013277347899e-05, 
       1.3123614651706734e-05, 
       1.5021231011704797e-05, 
       1.3219114816026737e-05, 
       1.6759926188128403e-05, 
       1.6592881323971881e-05, 
       1.3890328679378200e-05, 
       1.4793950089679507e-05, 
       1.4613577962848466e-05, 
       1.2703345119109875e-05, 
       1.5951380463504746e-05, 
       1.3991862677995752e-05, 
       1.2249058797105467e-05, 
       1.1972523661408510e-05, 
       1.6700376618316970e-05, 
       1.1969876721777264e-05, 
       1.4312469097455324e-05, 
       1.5967498384523299e-05, 
       1.5008487372870637e-05, 
       1.6552382744300849e-05, 
       1.2212314103295063e-05, 
       1.2565981052576954e-05, 
       1.5766650069032707e-05, 
       1.4362171464586172e-05, 
       1.1879011217201128e-05, 
       1.5473636336196578e-05, 
       1.2147704058296473e-05, 
       1.2760228433953444e-05, 
       1.2517464257710940e-05, 
       1.3896056333865877e-05, 
       1.3206583426179921e-05, 
       1.1788798968672102e-05, 
       1.6658976447284278e-05, 
       1.5051683809196574e-05, 
       1.2227159182147019e-05, 
       1.1964453171392055e-05, 
       1.3293035593964773e-05, 
       1.4239868457034409e-05, 
       1.3577023077152769e-05, 
       1.5210733389587490e-05, 
       1.4478818833415534e-05, 
       1.2114514986123191e-05, 
       1.2199923874453162e-05, 
       1.3488248717679212e-05, 
       1.5788557463872575e-05, 
       1.6554838609887707e-05, 
       1.1725848025827054e-05, 
       1.2436178609519201e-05, 
       1.5710961214449188e-05, 
       1.2406294336498330e-05, 
       1.2324917714639373e-05, 
       1.6530755275620778e-05, 
       1.3731959267300114e-05, 
       1.2397948700856795e-05, 
       1.5808884852920341e-05, 
       1.4996574873079029e-05, 
       1.2698437380966265e-05, 
       1.2820249545183314e-05, 
       1.6496189658827139e-05, 
       1.5666289734267274e-05, 
       1.3441240929757031e-05, 
       1.6084198893382013e-05, 
       1.3464354008009656e-05, 
       1.2923312047013518e-05, 
       1.4153434069785935e-05, 
       1.3692415858911692e-05, 
       1.2647929563223295e-05, 
       1.5707238963434554e-05, 
       1.4732038930163918e-05, 
       1.2303613663945354e-05, 
       1.6038563085680193e-05, 
       1.6440131848013768e-05, 
       1.2782956036827071e-05, 
       1.3681689919426739e-05, 
       1.1691671644629165e-05, 
       1.4499624017604651e-05, 
       1.6395388970284352e-05, 
       1.6268289868709545e-05, 
       1.5691960681611264e-05, 
       1.1820198074980815e-05, 
       1.2346581472258579e-05, 
       1.2434391892051689e-05, 
       1.5844056607382023e-05, 
       1.2365413501255700e-05, 
       1.2365413501255700e-05, 
       1.3586962442396367e-05, 
       1.2743115700108890e-05, 
       1.6329553554615786e-05, 
       1.1570739547506951e-05, 
       1.3206829560682617e-05, 
       1.4699065298021907e-05, 
       1.1943969004980533e-05, 
       1.5709386608091679e-05, 
       1.1976467870679928e-05, 
       1.2037110680872397e-05, 
       1.3252283484808413e-05, 
       1.1751907263236933e-05, 
       1.4695821628044451e-05, 
       1.1909909616669224e-05, 
       1.5429483851814471e-05, 
       1.3540485741379906e-05, 
       1.5015246910511515e-05, 
       1.3229899545695143e-05, 
       1.5181413541441026e-05, 
       1.4412180103396542e-05, 
       1.3190404474947619e-05, 
       1.1988832533212580e-05, 
       1.6185558154446363e-05, 
       1.3426714532296734e-05, 
       1.3869565860930604e-05, 
       1.4135381909291927e-05, 
       1.2606850455616096e-05, 
       1.2144157737387524e-05, 
       1.2445261049368796e-05, 
       1.4567667222832323e-05, 
       1.3693073542211198e-05, 
       1.5945804333257039e-05, 
       1.5097511323655867e-05, 
       1.3083451230784315e-05, 
       1.1656562503901878e-05, 
       1.3248369701287739e-05, 
       1.5059307448959791e-05, 
       1.1878061639952296e-05, 
       1.3749207965992576e-05, 
       1.4592582593190956e-05, 
       1.1791409574610544e-05, 
       1.1791409574610544e-05, 
       1.5897686053808388e-05, 
       1.3200346710388588e-05, 
       1.1491733677706403e-05, 
       1.5554715215553435e-05, 
       1.1805912385341901e-05, 
       1.1428469760813022e-05, 
       1.1473427980233798e-05, 
       1.5641819841991074e-05, 
       1.2674305212567148e-05, 
       1.5323952730433448e-05, 
       1.1437287703727674e-05, 
       1.2033933443160386e-05, 
       1.3514789530426399e-05, 
       1.6110115952677161e-05, 
       1.1496957380279707e-05, 
       1.3073495216324230e-05, 
       1.1610067832011640e-05, 
       1.3030248344053378e-05, 
       1.2418145207118873e-05, 
       1.1676042330679389e-05, 
       1.1806746906698654e-05, 
       1.5218638717593696e-05, 
       1.2264714382757942e-05, 
       1.1756912514758306e-05, 
       1.3870005976852957e-05, 
       1.6064855965009281e-05, 
       1.1997717874030617e-05, 
       1.4541108926322465e-05, 
       1.1385934298865472e-05, 
       1.1536830153690704e-05, 
       1.6046644649672756e-05, 
       1.1991518599098427e-05, 
       1.3126065397642909e-05, 
       1.5842218070209173e-05, 
       1.2805757786314474e-05, 
       1.1707112735086121e-05, 
       1.1903779516582374e-05, 
       1.1641802848545782e-05, 
       1.2631384096337930e-05, 
       1.4002302487247345e-05, 
       1.3327414429176635e-05, 
       1.3094907964503913e-05, 
       1.4584880159812993e-05, 
       1.1460871146783676e-05, 
       1.1788920941807358e-05, 
       1.4212354146342015e-05, 
       1.2124504294522087e-05, 
       1.1403082808497251e-05, 
       1.1805093320787671e-05, 
       1.1516586403133170e-05, 
       1.1516586403133170e-05, 
       1.1782432470851694e-05, 
       1.5687642170787301e-05, 
       1.1468094689081221e-05, 
       1.5646327836801596e-05, 
       1.1606360060081572e-05, 
       1.1746459586951272e-05, 
       1.3481216402351418e-05, 
       1.1306542119716975e-05, 
       1.1643740295214875e-05, 
       1.4264155037551276e-05, 
       1.5903588039561712e-05, 
       1.5036331088029241e-05, 
       1.1708627779604114e-05, 
       1.3450768538155453e-05, 
       1.2896341573252213e-05, 
       1.5927435455377457e-05, 
       1.2801393122403435e-05, 
       1.1917661158638408e-05, 
       1.5668815478302384e-05, 
       1.3466516587832193e-05, 
       1.1707057365782273e-05, 
       1.4121117094425142e-05, 
       1.1328603882568802e-05, 
       1.1308068145325968e-05, 
       1.1252187233513517e-05, 
       1.5199645314881461e-05, 
       1.5853025526967749e-05, 
       1.5313347729406159e-05, 
       1.4943648209940563e-05, 
       1.3088713696753645e-05, 
       1.3728329773655138e-05, 
       1.5189703733387558e-05, 
       1.1354250426422499e-05, 
       1.5843041372750021e-05, 
       1.1448615092421986e-05, 
       1.2125600347787393e-05, 
       1.1541750563452205e-05, 
       1.5816620432606382e-05, 
       1.4779759221480474e-05, 
       1.1936290366460016e-05, 
       1.3141568699492736e-05, 
       1.3376187761453536e-05, 
       1.4558536771582689e-05, 
       1.2850382793074335e-05, 
       1.5786156436101002e-05, 
       1.4711042705478195e-05, 
       1.3846017952778950e-05, 
       1.2329962791748679e-05, 
       1.1304216507208660e-05, 
       1.1761617043429116e-05, 
       1.4533402466654407e-05, 
       1.1184795576213820e-05, 
       1.2355016347426515e-05, 
       1.3868232904000190e-05, 
       1.5712421812686618e-05, 
       1.5274628319666322e-05, 
       1.1747215746359886e-05, 
       1.1110298822449085e-05, 
       1.3189853961366807e-05, 
       1.4873315195404058e-05, 
       1.5660190947536167e-05, 
       1.1113944966616766e-05, 
       1.4928415901558642e-05, 
       1.1709538471747310e-05, 
       1.5432092204515111e-05, 
       1.5601191169957125e-05, 
       1.2523359665147950e-05, 
       1.1515936533384390e-05, 
       1.5159361270082106e-05, 
       1.1637901880002912e-05, 
       1.1262262527624278e-05, 
       1.3738001311676133e-05, 
       1.5630116408341179e-05, 
       1.4003566990071585e-05, 
       1.4011563929128087e-05, 
       1.5570699567029073e-05, 
       1.1243402289275636e-05, 
       1.2202651906328915e-05, 
       1.1632835224451255e-05, 
       1.3077925768637599e-05, 
       1.1496020688915531e-05, 
       1.1225035725463698e-05, 
       1.1243885758371005e-05, 
       1.5389381913883833e-05, 
       1.1842930660028952e-05, 
       1.5264381754706096e-05, 
       1.1115315664648713e-05, 
       1.1653384767536996e-05, 
       1.5468816662612564e-05, 
       1.2253406756355439e-05, 
       1.1272289600893787e-05, 
       1.5522807778847407e-05, 
       1.4173567456488411e-05, 
       1.2267949558929904e-05, 
       1.1849773091019263e-05, 
       1.1849773091019263e-05, 
       1.1039016207471834e-05, 
       1.1199760475259692e-05, 
       1.0986523256542509e-05, 
       1.1196181141700119e-05, 
       1.4153368210771970e-05, 
       1.1348480770287634e-05, 
       1.5089187950494129e-05, 
       1.0967520958194335e-05, 
       1.0952010752608838e-05, 
       1.5115769554656917e-05, 
       1.1018112549010922e-05, 
       1.2428055255654019e-05, 
       1.2869710498104527e-05, 
       1.1171132744942016e-05, 
       1.1315865080764070e-05, 
       1.4663579251914919e-05, 
       1.4107151413661129e-05, 
       1.1059027997872198e-05, 
       1.1537827920369020e-05, 
       1.2735825688174792e-05, 
       1.5440009462433429e-05, 
       1.5270992835223876e-05, 
       1.1185437406002740e-05, 
       1.1352045364457742e-05, 
       1.2439626722135538e-05, 
       1.0967929408119725e-05, 
       1.3557333155104824e-05, 
       1.0934323250491465e-05, 
       1.1175654347406480e-05, 
       1.4912583062787027e-05, 
       1.5220031394833180e-05, 
       1.2697667714867147e-05, 
       1.2845462218534526e-05, 
       1.0897176133463497e-05, 
       1.0897176133463497e-05, 
       1.4073786409325840e-05, 
       1.0874609104551376e-05, 
       1.1508515859870161e-05, 
       1.1508515859870161e-05, 
       1.2734060603850802e-05, 
       1.1067005512295486e-05, 
       1.5337877145590250e-05, 
       1.5337877145590250e-05, 
       1.4624884438892826e-05, 
       1.5307137564677187e-05, 
       1.0996587230195566e-05, 
       1.0927985780817866e-05, 
       1.5006960783782638e-05, 
       1.3950980342708871e-05, 
       1.2518256502461746e-05, 
       1.1286889547465902e-05, 
       1.1286889547465902e-05, 
       1.2418006574986329e-05, 
       1.3020686125861597e-05, 
       1.3305410636865290e-05, 
       1.2520019019828464e-05, 
       1.4964993694528354e-05, 
       1.4527589466641050e-05, 
       1.0890367921632658e-05, 
       1.5031753928095082e-05, 
       1.2535173598128093e-05, 
       1.2725692351619874e-05, 
       1.5236367486478689e-05, 
       1.3183908284447587e-05, 
       1.3312196876400103e-05, 
       1.5067080981965494e-05, 
       1.1144693532560288e-05, 
       1.1189151896069598e-05, 
       1.5256502598029275e-05, 
       1.3805931020184647e-05, 
       1.0837804810568068e-05, 
       1.2485414595653036e-05, 
       1.4777074359356703e-05, 
       1.2382391337398714e-05, 
       1.4580128075709371e-05, 
       1.2885019884980843e-05, 
       1.2682417068049357e-05, 
       1.3257170988703858e-05, 
       1.1047785809995651e-05, 
       1.4399846579952405e-05, 
       1.3566597422146882e-05, 
       1.3124404547055084e-05, 
       1.3124404547055084e-05, 
       1.5077655569610467e-05, 
       1.2277398297783406e-05, 
       1.1326304583625107e-05, 
       1.4916032627462605e-05, 
       1.5166866833510470e-05, 
       1.4664819432685067e-05, 
       1.4771604348427269e-05, 
       1.2820765149677424e-05, 
       1.4617752897242361e-05, 
       1.4099604567113423e-05, 
       1.0909423648589213e-05, 
       1.2308778090971123e-05, 
       1.4499352619690369e-05, 
       1.1986266754807234e-05, 
       1.0706643184233675e-05, 
       1.2803978930077526e-05, 
       1.3145670200735626e-05, 
       1.0687074060935999e-05, 
       1.1483070592647357e-05, 
       1.4089477424144276e-05, 
       1.3540320035125067e-05, 
       1.4270909832045135e-05, 
       1.0781868765621849e-05, 
       1.2524379929047252e-05, 
       1.1569316407621641e-05, 
       1.0682416214828684e-05, 
       1.1227308808072031e-05, 
       1.5057881050155482e-05, 
       1.4784180429050506e-05, 
       1.2334791045062176e-05, 
       1.0931688471478746e-05, 
       1.4978560770722526e-05, 
       1.0645797797263889e-05, 
       1.0907301800509692e-05, 
       1.2245357589501916e-05, 
       1.1682483367378722e-05, 
       1.2536967804570898e-05, 
       1.2674431119390458e-05, 
       1.2500760479123041e-05, 
       1.2661436039349780e-05, 
       1.0748096343809930e-05, 
       1.0661241267353549e-05, 
       1.2906085811427898e-05, 
       1.1393258842823256e-05, 
       1.0934370358397742e-05, 
       1.3314085303905102e-05, 
       1.4893977145145620e-05, 
       1.1742098731337657e-05, 
       1.4898379300062665e-05, 
       1.1738503965747513e-05, 
       1.4960334989380520e-05, 
       1.0825665770186496e-05, 
       1.4892962356135938e-05, 
       1.0582864328384272e-05, 
       1.1876128172893532e-05, 
       1.4717111255904247e-05, 
       1.2416046454848009e-05, 
       1.0572774764081422e-05, 
       1.0629001008275446e-05, 
       1.0629001008275446e-05, 
       1.2989608020850853e-05, 
       1.0635451693907987e-05, 
       1.2416871252008860e-05, 
       1.1309479836273569e-05, 
       1.3441577510479102e-05, 
       1.2173619395365241e-05, 
       1.2272730109041874e-05, 
       1.1202943919641708e-05, 
       1.4827849245973207e-05, 
       1.1484453307605679e-05, 
       1.4721423390146512e-05, 
       1.3473508823247756e-05, 
       1.4356461362413227e-05, 
       1.2137163074763383e-05, 
       1.1452851846238364e-05, 
       1.4792230661840160e-05, 
       1.4862192739303246e-05, 
       1.1330698891247778e-05, 
       1.1341041684206632e-05, 
       1.1438869926346119e-05, 
       1.0985599373678655e-05, 
       1.1580380892111391e-05, 
       1.1693132431522686e-05, 
       1.4456293277566900e-05, 
       1.0897156582543343e-05, 
       1.3090127479297849e-05, 
       1.4616810496144469e-05, 
       1.3116734853383398e-05, 
       1.3241563968620226e-05, 
       1.4852177161157095e-05, 
       1.1268027648545931e-05, 
       1.2110637909259223e-05, 
       1.0983971588025755e-05, 
       1.3101101857249413e-05, 
       1.4302291641547505e-05, 
       1.2036897671629886e-05, 
       1.2088085575284726e-05, 
       1.2470940657654839e-05, 
       1.1353835712482653e-05, 
       1.2318200703793047e-05, 
       1.1561208327386016e-05, 
       1.0502266072465305e-05, 
       1.0726078868318596e-05, 
       1.3633957912284724e-05, 
       1.3633957912284724e-05, 
       1.1031766948423923e-05, 
       1.4756214215517396e-05, 
       1.4783993172475419e-05, 
       1.4265632235424737e-05, 
       1.1341234758415497e-05, 
       1.0455546620602337e-05, 
       1.4760754796311092e-05, 
       1.1379783376150733e-05, 
       1.1118145755479153e-05, 
       1.0779059583388129e-05, 
       1.3107552009535025e-05, 
       1.3573820965661135e-05, 
       1.3573820965661135e-05, 
       1.2964928274335566e-05, 
       1.0824900110190728e-05, 
       1.4249046018263897e-05, 
       1.1186842864810494e-05, 
       1.1002718625379305e-05, 
       1.2601212542019463e-05, 
       1.0401747429834436e-05, 
       1.3512024838147051e-05, 
       1.0659363588757335e-05, 
       1.0706524910972153e-05, 
       1.1267419351786995e-05, 
       1.2615801326915027e-05, 
       1.0372462058424156e-05, 
       1.1125956058060493e-05, 
       1.2396593323131289e-05, 
       1.0851091150722102e-05, 
       1.4638037150614476e-05, 
       1.1976140804757904e-05, 
       1.1976140804757904e-05, 
       1.4046939908507796e-05, 
       1.2495128346118907e-05, 
       1.0354329265691745e-05, 
       1.3593186316792134e-05, 
       1.2096016545599932e-05, 
       1.2235654928411608e-05, 
       1.0369261521023316e-05, 
       1.0713590219783965e-05, 
       1.4478955976275943e-05, 
       1.3058413759815817e-05, 
       1.0618612324834768e-05, 
       1.3981977480614612e-05, 
       1.3723817160305707e-05, 
       1.3603717863442891e-05, 
       1.0324597641693863e-05, 
       1.0789928402511669e-05, 
       1.0565284499678855e-05, 
       1.1921996299012355e-05, 
       1.0958471205144210e-05, 
       1.0536622016865853e-05, 
       1.0305727383443258e-05, 
       1.2382380832845453e-05, 
       1.0539096173643417e-05, 
       1.0884221606114679e-05, 
       1.0359597169923122e-05, 
       1.2083611228768885e-05, 
       1.4001995807347465e-05, 
       1.4357178868411115e-05, 
       1.2090636903173968e-05, 
       1.4511475405066456e-05, 
       1.1477522054824460e-05, 
       1.1850400761972084e-05, 
       1.2066733298351592e-05, 
       1.0514904496397342e-05, 
       1.1752157912646303e-05, 
       1.4474148208912567e-05, 
       1.0533462105683179e-05, 
       1.3916519850986633e-05, 
       1.0423082566069726e-05, 
       1.3478186026087543e-05, 
       1.0255779626614144e-05, 
       1.2482187415577462e-05, 
       1.4293843094331272e-05, 
       1.0962390480004999e-05, 
       1.0290444949957584e-05, 
       1.1504534691962995e-05, 
       1.4334106235450884e-05, 
       1.3306408690282276e-05, 
       1.4370237310148139e-05, 
       1.4078642185062700e-05, 
       1.4369702453050149e-05, 
       1.0452491693609763e-05, 
       1.0778451124246511e-05, 
       1.1852402892640105e-05, 
       1.0207994119854338e-05, 
       1.0250107976425917e-05, 
       1.4167849901597010e-05, 
       1.0201536244387168e-05, 
       1.4033737566722881e-05, 
       1.0640666094756368e-05, 
       1.1531383654357561e-05, 
       1.4385479716268984e-05, 
       1.0197191385776692e-05, 
       1.4151961089680895e-05, 
       1.0529160502369732e-05, 
       1.3891435537586878e-05, 
       1.1966728543004530e-05, 
       1.4372643406053173e-05, 
       1.4339635528439476e-05, 
       1.0446776561917997e-05, 
       1.2339808286648252e-05, 
       1.3240828610130460e-05, 
       1.3297687409586142e-05, 
       1.0232686493411448e-05, 
       1.0232467321738716e-05, 
       1.0769457889965293e-05, 
       1.0769457889965293e-05, 
       1.1233776810542011e-05, 
       1.1230618000829442e-05, 
       1.1393715805315966e-05, 
       1.0140699828897416e-05, 
       1.0191038029783340e-05, 
       1.0480206885599021e-05, 
       1.1704722679734985e-05, 
       1.3754455930786722e-05, 
       1.4259549301548854e-05, 
       1.1899471228164326e-05, 
       1.0855262973853046e-05, 
       1.0150205549888582e-05, 
       1.4275851808117710e-05, 
       1.3856538222947389e-05, 
       1.4141594015085124e-05, 
       1.2133007463131249e-05, 
       1.0086087376263686e-05, 
       1.0358004196978648e-05, 
       1.0148252957319420e-05, 
       1.0231964803194382e-05, 
       1.1662381400910250e-05, 
       1.0480274902075477e-05, 
       1.1844462285448935e-05, 
       1.0518551833511820e-05, 
       1.4175465863273368e-05, 
       1.0351382921913594e-05, 
       1.0345904758398967e-05, 
       1.4159137148875332e-05, 
       1.3995844960773261e-05, 
       1.3705111037037255e-05, 
       1.3783819765661786e-05, 
       1.4182962566498339e-05, 
       1.4175186838039334e-05, 
       1.1034237926638181e-05, 
       1.0941254008155762e-05, 
       1.0523948902187752e-05, 
       1.0192413359347300e-05, 
       1.3277067386134746e-05, 
       1.1601258009471674e-05, 
       1.2366330407442362e-05, 
       1.0164096791271234e-05, 
       1.3852707329658270e-05, 
       1.0944116714033461e-05, 
       1.1716036737641770e-05, 
       1.0127605453051655e-05, 
       1.2545008577836389e-05, 
       1.0719060500363479e-05, 
       1.1459250018088116e-05, 
       1.1203282772219203e-05, 
       1.0384421379242826e-05, 
       1.0046168905596495e-05, 
       1.0109426574253145e-05, 
       1.2954958561733972e-05, 
       1.0107896110300699e-05, 
       1.1481840674364551e-05, 
       1.2348459390545577e-05, 
       1.1544211853024134e-05, 
       1.0781553527906825e-05, 
       1.0443297817610015e-05, 
       1.0443297817610015e-05, 
       1.0741773271011750e-05, 
       9.9588492548859384e-06, 
       1.0772599095308152e-05, 
       1.0717318714635639e-05, 
       1.1562262967228524e-05, 
       1.1671304731552792e-05, 
       1.0427996792002121e-05, 
       1.0766993965580983e-05, 
       9.9639252780536248e-06, 
       1.0699631739427376e-05, 
       1.0734362119894454e-05, 
       1.0938622843211320e-05, 
       1.2509512401668630e-05, 
       1.0465518322655523e-05, 
       1.0369418297989408e-05, 
       1.1644271876959301e-05, 
       1.0756879532016300e-05, 
       1.3571781635163237e-05, 
       9.9066181154063978e-06, 
       1.0375729240048642e-05, 
       1.1350746187690665e-05, 
       1.0048140096828067e-05, 
       1.0169779087521093e-05, 
       1.0274727581595650e-05, 
       1.2098358739634541e-05, 
       1.1583528020663391e-05, 
       1.3716189943916849e-05, 
       1.3716189943916849e-05, 
       1.0273134656555220e-05, 
       1.3771343749510956e-05, 
       1.3511470796911996e-05, 
       1.2472753076264438e-05, 
       1.0163898887041306e-05, 
       1.1782940348591839e-05, 
       1.1932072754492584e-05, 
       1.1595036243800563e-05, 
       1.3290564297012587e-05, 
       1.1555697292284911e-05, 
       9.8531573836538121e-06, 
       1.0475872976903728e-05, 
       1.3891492169927510e-05, 
       1.0264615489127979e-05, 
       1.3662868048168734e-05, 
       1.3882906807495525e-05, 
       1.3632743678908576e-05, 
       9.9322116857103128e-06, 
       9.9676209493156017e-06, 
       1.1172525963628234e-05, 
       1.1157513459169266e-05, 
       9.8463506734846862e-06, 
       9.8589441723096154e-06, 
       1.1167394334040324e-05, 
       1.0764400560250339e-05, 
       9.9295732532001488e-06, 
       1.3857666475041970e-05, 
       9.8540545144493283e-06, 
       1.2391894679069700e-05, 
       9.8709408983938285e-06, 
       1.0265848360167324e-05, 
       1.3752051008159996e-05, 
       1.3465837193572501e-05, 
       1.0294149916100227e-05, 
       1.3553150957271262e-05, 
       1.0937738936159390e-05, 
       1.1652798280705539e-05, 
       9.8991856293166356e-06, 
       1.3680345509379824e-05, 
       1.3680232479877401e-05, 
       1.3778407793679142e-05, 
       9.8166427871913147e-06, 
       1.0266360710962645e-05, 
       1.1120224616402655e-05, 
       1.3479388353296501e-05, 
       1.2713057698686368e-05, 
       1.2400727021484068e-05, 
       1.1193169627032153e-05, 
       1.0293345836682840e-05, 
       1.0286900843820786e-05, 
       1.0287509587112707e-05, 
       9.7110557318157017e-06, 
       1.3584579796094864e-05, 
       9.7991190832840413e-06, 
       1.3581910128433880e-05, 
       1.3709792190589098e-05, 
       1.1140006391140550e-05, 
       1.3185749278468462e-05, 
       1.0558336312681318e-05, 
       1.0907261870507726e-05, 
       1.3308886757718064e-05, 
       1.1728424833290895e-05, 
       1.3677034841825666e-05, 
       1.1005444053608968e-05, 
       9.9181580570812516e-06, 
       1.2492448011625643e-05, 
       1.2730863681723320e-05, 
       1.2920167304930794e-05, 
       1.3409714538063165e-05, 
       1.3409714538063165e-05, 
       1.1354024047272605e-05, 
       1.2725660885446155e-05, 
       9.8953075569406875e-06, 
       1.1278233862602094e-05, 
       1.0521365032372457e-05, 
       1.1247028112192032e-05, 
       9.8727686935996752e-06, 
       1.1752157912643477e-05, 
       1.1299091244424666e-05, 
       1.3445703513279984e-05, 
       1.3223115866879419e-05, 
       1.0556804643202225e-05, 
       1.0556804643202225e-05, 
       1.0324142386844406e-05, 
       1.0324142386844406e-05, 
       1.2403292753983264e-05, 
       1.0020088063763115e-05, 
       1.3209683995720098e-05, 
       9.8997126994705673e-06, 
       9.6877317195667489e-06, 
       1.0741291799648876e-05, 
       9.8357648440326428e-06, 
       1.1301045296227273e-05, 
       9.6371606708548516e-06, 
       9.9188789453519585e-06, 
       1.3533870031930246e-05, 
       1.0288554358793279e-05, 
       1.1254408991385510e-05, 
       9.6538100147117608e-06, 
       1.1839614596608071e-05, 
       1.1576468942207957e-05, 
       9.6080930288863930e-06, 
       1.0325854012744955e-05, 
       1.2905913985529389e-05, 
       9.7542016527540027e-06, 
       1.2336471051698504e-05, 
       1.1204766900491367e-05, 
       1.3457481789545981e-05, 
       1.3082326440540041e-05, 
       9.9288021589010119e-06, 
       9.9423396243975264e-06, 
       1.1043116804061502e-05, 
       9.5470105352297831e-06, 
       1.3030394604068650e-05, 
       9.5630513997533024e-06, 
       9.8001029643319265e-06, 
       9.6546220414183026e-06, 
       9.6513717990655065e-06, 
       9.7255842240087050e-06, 
       9.8312539701814733e-06, 
       1.3358906659324441e-05, 
       1.1204631885711674e-05, 
       1.1417371644690850e-05, 
       9.5471562509718646e-06, 
       1.3356955723132384e-05, 
       9.8350477729461501e-06, 
       1.1703350164090343e-05, 
       1.0022851019675560e-05, 
       9.6933187761007210e-06, 
       1.1914518514727370e-05, 
       1.2092467310824198e-05, 
       9.8016029589222726e-06, 
       1.0043099465667155e-05, 
       1.1218004962993395e-05, 
       1.1433161928157302e-05, 
       9.5026622513660741e-06, 
       1.0437392199040041e-05, 
       1.1687549437028655e-05, 
       1.1441998243931732e-05, 
       1.3405385310560013e-05, 
       1.0207625149463528e-05, 
       1.0104401045019265e-05, 
       9.8864957878371508e-06, 
       1.1798593183640694e-05, 
       9.8656653331183074e-06, 
       9.8656653331183074e-06, 
       1.1127895645814130e-05, 
       1.2122818045564395e-05, 
       1.2122818045564395e-05, 
       1.1427349957727755e-05, 
       1.0943809308893678e-05, 
       1.2391450287923214e-05, 
       1.3186895349734087e-05, 
       1.1107318880802598e-05, 
       9.5026384260519855e-06, 
       1.1303850900876228e-05, 
       1.3239157353800872e-05, 
       1.3350268526375690e-05, 
       1.2635770437608012e-05, 
       1.3186067216009567e-05, 
       9.5105068645971677e-06, 
       9.6283361171834058e-06, 
       9.4432811927597549e-06, 
       1.0130424330690898e-05, 
       1.3109729013515758e-05, 
       9.5633359468702655e-06, 
       9.4785173521846500e-06, 
       1.2514020340884226e-05, 
       9.4511416572530357e-06, 
       1.2639813888453164e-05, 
       1.0602095869449240e-05, 
       9.9247045689405628e-06, 
       1.0769130946744692e-05, 
       1.2326572788999798e-05, 
       1.2258874428434726e-05, 
       9.3894207286489628e-06, 
       1.1162989531966972e-05, 
       1.3144905699325213e-05, 
       1.1016691630843322e-05, 
       9.4493428251548987e-06, 
       1.0770542529304339e-05, 
       1.1390225352955032e-05, 
       9.4171171286346109e-06, 
       1.0806720495068495e-05, 
       1.3105891423479931e-05, 
       1.0124985383816042e-05, 
       1.1186872677173530e-05, 
       1.1647913762635511e-05, 
       9.4736783241047553e-06, 
       1.0011519348709738e-05, 
       1.0804181581460961e-05, 
       1.3171782712334967e-05, 
       1.1023048174560538e-05, 
       1.2928282235854085e-05, 
       9.7078942855980961e-06, 
       1.0803244108138391e-05, 
       1.2109242877613827e-05, 
       1.2752397517035859e-05, 
       1.2221917979082999e-05, 
       1.2117846112147576e-05, 
       1.0567614747559902e-05, 
       1.1007610559107456e-05, 
       1.2983373656710785e-05, 
       9.4021952706926370e-06, 
       1.2106753394518955e-05, 
       1.1229722669348576e-05, 
       1.3153807692290633e-05, 
       1.1205800655026882e-05, 
       1.0954143051024719e-05, 
       1.3137960761351307e-05, 
       9.3168954657137443e-06, 
       1.0858590684983307e-05, 
       1.2290473544713559e-05, 
       1.2828144047638880e-05, 
       9.5832087676143862e-06, 
       9.8818302319605414e-06, 
       1.0491732248718483e-05, 
       1.1953922390953531e-05, 
       1.0918878942682853e-05, 
       1.2004128080271443e-05, 
       1.3017446398937332e-05, 
       1.0922067436222345e-05, 
       9.5499468237402093e-06, 
       1.1886034178225775e-05, 
       1.2030848723993907e-05, 
       1.0671480260562395e-05, 
       9.4398779849119620e-06, 
       1.2681549098845909e-05, 
       9.6975012974264742e-06, 
       1.0607255961229588e-05, 
       9.2677950455456232e-06, 
       1.0669405025341764e-05, 
       1.0837804810570888e-05, 
       9.7600592480075076e-06, 
       9.3376919690089264e-06, 
       1.3062806636467790e-05, 
       1.3062806636467790e-05, 
       1.1048775634624641e-05, 
       1.0739902383738782e-05, 
       1.0591587876995203e-05, 
       1.1145778771575602e-05, 
       1.0500205060680463e-05, 
       9.2321339976707295e-06, 
       9.3092463478716725e-06, 
       9.8759585353772033e-06, 
       9.2668813377319048e-06, 
       9.3775407997072199e-06, 
       1.0310541212589325e-05, 
       9.5918294218066813e-06, 
       9.2587164719816308e-06, 
       1.1304216507209693e-05, 
       9.9697715055097759e-06, 
       9.3148027575251408e-06, 
       9.3547467882339339e-06, 
       9.3781396000811463e-06, 
       1.1570674593632197e-05, 
       1.1210949235434507e-05, 
       9.7688027720322650e-06, 
       1.2627834612757489e-05, 
       1.2772207937889913e-05, 
       1.0703498427079553e-05, 
       9.2247734081576331e-06, 
       9.2818047193352773e-06, 
       1.2308412295258785e-05, 
       9.4085128010184797e-06, 
       9.1846790933436114e-06, 
       1.2630237888799176e-05, 
       9.3531527641751993e-06, 
       1.2220465496440703e-05, 
       9.2888880421973135e-06, 
       9.9317347715202342e-06, 
       1.0485547869184157e-05, 
       1.2955498244343441e-05, 
       1.1698216894219982e-05, 
       9.7665100201072385e-06, 
       1.0525702757590444e-05, 
       1.0523551832201159e-05, 
       9.7253751619105309e-06, 
       9.6578785903524936e-06, 
       9.8947536800422147e-06, 
       9.8784675056871213e-06, 
       1.0139019887189659e-05, 
       1.0137861699121923e-05, 
       1.1149270590659216e-05, 
       1.0768237818775314e-05, 
       1.0423542401133671e-05, 
       9.6199835875848645e-06, 
       9.3708820858261481e-06, 
       9.6089459488269225e-06, 
       9.4267873914124887e-06, 
       1.2830083882353740e-05, 
       1.0966169533015538e-05, 
       9.8521648788683639e-06, 
       1.2455490813311961e-05, 
       1.1231479999904232e-05, 
       9.6842656308280507e-06, 
       1.2433500153068158e-05, 
       1.1773305617921443e-05, 
       1.2033001514277886e-05, 
       1.1852795457209111e-05, 
       1.2570505041565146e-05, 
       1.2622725017399904e-05, 
       9.8337776002085453e-06, 
       1.0817707761469620e-05, 
       9.1230553059871431e-06, 
       1.1328954557311369e-05, 
       1.2371896701208038e-05, 
       1.0059840790181679e-05, 
       9.1478675568288740e-06, 
       1.2825661190244699e-05, 
       1.0449262547816364e-05, 
       1.0449049795165766e-05, 
       1.2743105654076809e-05, 
       9.9876167479811714e-06, 
       9.9876167479811714e-06, 
       1.2780497664426927e-05, 
       9.8805574510434289e-06, 
       1.0202712395731005e-05, 
       1.0202712395731005e-05, 
       1.1191677570474883e-05, 
       9.8993722027278463e-06, 
       1.0402341386775062e-05, 
       1.1071430896233756e-05, 
       1.2255945887280579e-05, 
       9.1022566900728773e-06, 
       1.1658378212952020e-05, 
       9.2494681641170340e-06, 
       9.1727039195827449e-06, 
       9.1727039195827449e-06, 
       1.0121869774851753e-05, 
       1.0178099871991463e-05, 
       9.9970367497889430e-06, 
       9.4469575938118905e-06, 
       1.2768852916673558e-05, 
       1.0757200347408972e-05, 
       1.2347740237962768e-05, 
       1.2563735955440907e-05, 
       9.5500591719759298e-06, 
       9.3436504438106613e-06, 
       9.3436504438106613e-06, 
       9.9866977973797680e-06, 
       1.0013612574915403e-05, 
       1.0815805915283423e-05, 
       9.4001893056016508e-06, 
       9.3424966200329927e-06, 
       1.2463336640932619e-05, 
       1.2654201733108691e-05, 
       1.0042257124373006e-05, 
       9.1022441532061181e-06, 
       1.2307315990743306e-05, 
       1.1872992088757272e-05, 
       1.0301533207928223e-05, 
       1.2778805979040705e-05, 
       1.0559094159024394e-05, 
       9.1998150324756944e-06, 
       9.7392855939157339e-06, 
       9.7392855939157339e-06, 
       1.2684148584956904e-05, 
       9.0368436568031782e-06, 
       1.1275154083769477e-05, 
       9.0746217170146251e-06, 
       1.0928542677687956e-05, 
       1.0781962041928323e-05, 
       1.0600707078360105e-05, 
       9.1671318570126391e-06, 
       9.5236702297173559e-06, 
       1.2738658977837161e-05, 
       9.1357389198660292e-06, 
       9.1248382381796373e-06, 
       1.0620995329562578e-05, 
       9.2780105511684983e-06, 
       1.1737788126103411e-05, 
       1.2564144906311051e-05, 
       9.6351536738725791e-06, 
       1.2676606841844208e-05, 
       1.0708949430844922e-05, 
       9.3331175295456527e-06, 
       9.0819214798173418e-06, 
       1.1165061773801044e-05, 
       1.2534492581613298e-05, 
       1.2110617539560772e-05, 
       9.1478512460921433e-06, 
       1.0037633078030742e-05, 
       1.0635264197800616e-05, 
       1.1815973088010024e-05, 
       1.2334185130685705e-05, 
       9.1100972426726350e-06, 
       9.7445044027918277e-06, 
       1.2670282504972080e-05, 
       1.0996587230191946e-05, 
       9.5965841699365205e-06, 
       1.0534533226667888e-05, 
       9.5819383088884107e-06, 
       1.0749523252631202e-05, 
       1.1953525186153542e-05, 
       1.0443074660427468e-05, 
       8.9958933306436060e-06, 
       1.2446000187454274e-05, 
       1.2397091642794731e-05, 
       9.9912873716180499e-06, 
       1.1509756812962129e-05, 
       9.7961727688561816e-06, 
       1.0653765509033395e-05, 
       1.1961500703302007e-05, 
       9.0804718562953469e-06, 
       1.2606717937057878e-05, 
       1.0774460659384311e-05, 
       1.2391582890951479e-05, 
       9.2227945383247079e-06, 
       9.1879717427088422e-06, 
       8.9268099626050993e-06, 
       1.2597584016415193e-05, 
       8.9814441206256559e-06, 
       1.1451624288040226e-05, 
       1.0689431338339348e-05, 
       8.9195566727446034e-06, 
       1.0474506716620784e-05, 
       9.4729164238296090e-06, 
       1.2561939692113902e-05, 
       9.8400799162374162e-06, 
       9.2811240200997808e-06, 
       9.2381699501942920e-06, 
       8.9941227169061446e-06, 
       1.2346581472266734e-05, 
       1.1945606285926167e-05, 
       1.0481620945989216e-05, 
       9.6625128449432227e-06, 
       1.1067654623883784e-05, 
       9.2473224187938725e-06, 
       1.2460145103289174e-05, 
       1.2091566040481341e-05, 
       1.0694173560187419e-05, 
       8.9574075892759891e-06, 
       1.2222873169944524e-05, 
       1.0443932151844147e-05, 
       1.2121883741920248e-05, 
       9.1851166488721981e-06, 
       9.4187543135471055e-06, 
       8.8543249880848631e-06, 
       1.0355983952848059e-05, 
       1.1143484055106893e-05, 
       1.0356967215095525e-05, 
       8.8832192040186158e-06, 
       1.2407701137751519e-05, 
       1.0896293850782069e-05, 
       1.1674929870624057e-05, 
       1.1169989751052750e-05, 
       1.0379883688042089e-05, 
       9.1063635490633534e-06, 
       1.2336138780579346e-05, 
       9.0001753881004004e-06, 
       1.1219340538269669e-05, 
       1.2235805909465814e-05, 
       9.1830790774495666e-06, 
       1.1200785234531888e-05, 
       1.0401995299288373e-05, 
       1.2459582760076553e-05, 
       1.0585377835422413e-05, 
       8.8956288824362054e-06, 
       1.0748096343807257e-05, 
       9.5744835955419582e-06, 
       9.6149983712680203e-06, 
       9.4107281440109504e-06, 
       1.2336108874312123e-05, 
       1.1513433387387463e-05, 
       9.8439063118872454e-06, 
       1.1982044708512600e-05, 
       1.2225867655223777e-05, 
       9.5952260509795880e-06, 
       9.0242975439403601e-06, 
       1.1122085553303411e-05, 
       1.2288443764546314e-05, 
       8.8670562925428283e-06, 
       1.0530901477910492e-05, 
       1.0337472656865706e-05, 
       9.0511085522554975e-06, 
       1.2401467377660168e-05, 
       8.7751352482030465e-06, 
       1.0851131390882867e-05, 
       8.7745824667157009e-06, 
       9.8245554415825270e-06, 
       1.2344993589865840e-05, 
       1.0327333199442356e-05, 
       8.8450662512124993e-06, 
       1.1536309285521137e-05, 
       1.1277111082430603e-05, 
       8.8375524337901501e-06, 
       9.8586702779874318e-06, 
       1.0292560042439935e-05, 
       8.7907778702964134e-06, 
       9.9090568073786265e-06, 
       9.0422170242966008e-06, 
       1.0293907396225010e-05, 
       8.7753222769101873e-06, 
       1.1831149635131640e-05, 
       1.0012746213256483e-05, 
       9.0544925483596595e-06, 
       1.1799471467373434e-05, 
       9.2177058552836243e-06, 
       1.2227785601250888e-05, 
       9.2044360456944574e-06, 
       1.1142567919245456e-05, 
       8.7348770311390610e-06, 
       8.7344957993144064e-06, 
       9.6801254345436389e-06, 
       1.0527087143944868e-05, 
       1.1602416612816817e-05, 
       9.0600700046074098e-06, 
       9.8743414406016297e-06, 
       1.0228212411451325e-05, 
       1.0937964482508904e-05, 
       1.2140971794259049e-05, 
       9.1305462765572713e-06, 
       1.0229423815091647e-05, 
       9.0513340917907232e-06, 
       1.2299220560215220e-05, 
       8.7088710442996347e-06, 
       8.7180403949976244e-06, 
       9.4875174364329943e-06, 
       8.6829906565761092e-06, 
       8.8259184291011369e-06, 
       9.8832892435092658e-06, 
       1.0522537900222211e-05, 
       9.8811350920512785e-06, 
       1.2264179452124367e-05, 
       1.0070795802198668e-05, 
       1.0291903019556673e-05, 
       1.1347234189430728e-05, 
       9.9612026493148042e-06, 
       9.1058306621528790e-06, 
       9.4909664734453217e-06, 
       9.4988971369013392e-06, 
       1.0784399995877553e-05, 
       1.2186232015751885e-05, 
       1.2254632984604563e-05, 
       8.7919730096768578e-06, 
       8.6604601825212244e-06, 
       9.3934058079061585e-06, 
       8.7113613066632600e-06, 
       1.0111915838943395e-05, 
       1.0158618198907516e-05, 
       9.9513140685793033e-06, 
       8.6868084682827813e-06, 
       9.8416161227922178e-06, 
       1.0389452780257481e-05, 
       1.1691230142425545e-05, 
       9.8303350376425648e-06, 
       1.1329199413064755e-05, 
       1.0169328768071783e-05, 
       1.2141504381133440e-05, 
       1.1354656249574024e-05, 
       1.1354656249574024e-05, 
       8.6935629358109298e-06, 
       9.0141586972807182e-06, 
       9.0282991780687111e-06, 
       8.8089813724129680e-06, 
       8.6399653361158122e-06, 
       1.1282516728773691e-05, 
       9.0823208079387150e-06, 
       1.0411312084603624e-05, 
       9.3090533406867956e-06, 
       1.2057244244393008e-05, 
       1.0148467603168878e-05, 
       8.8476369059254946e-06, 
       9.2209414277631789e-06, 
       1.1116078236253220e-05, 
       1.0552701260820029e-05, 
       1.0854122423359124e-05, 
       1.0067167443994695e-05, 
       1.0082122234113974e-05, 
       9.6775349443170425e-06, 
       9.1253680979815203e-06, 
       8.7662247296253884e-06, 
       1.0268029690399450e-05, 
       8.8566346690324586e-06, 
       9.9565108148315982e-06, 
       8.6081173272099283e-06, 
       1.0702017346930397e-05, 
       1.2030135211295566e-05, 
       9.6398361670274659e-06, 
       8.7961159887268427e-06, 
       8.8108829202063293e-06, 
       8.6081422551946095e-06, 
       8.8248027791409656e-06, 
       1.0005922587375624e-05, 
       9.0004283189170079e-06, 
       1.2043036907652049e-05, 
       1.1940055035649077e-05, 
       1.2062130119493855e-05, 
       8.9897317227603656e-06, 
       8.7992311869331345e-06, 
       1.1755737321336195e-05, 
       9.8402231567159156e-06, 
       9.1804230151786108e-06, 
       1.2038580769513963e-05, 
       1.0984991599234054e-05, 
       8.9800128477225336e-06, 
       8.9800128477225336e-06, 
       1.0250788450084232e-05, 
       8.7739980794217569e-06, 
       9.6448824020837030e-06, 
       8.6069665581148381e-06, 
       8.5528432668242652e-06, 
       1.1993494601139581e-05, 
       1.0319842355969517e-05, 
       1.1899174885544215e-05, 
       9.5555685320202960e-06, 
       9.2860009143264645e-06, 
       8.5172472667412958e-06, 
       9.5102993471382897e-06, 
       8.4836705119684425e-06, 
       1.0439499237609040e-05, 
       8.8535520676748101e-06, 
       1.1914666717700017e-05, 
       9.9669083791440136e-06, 
       1.1849196969717771e-05, 
       1.0900506162067672e-05, 
       1.1003000311280193e-05, 
       9.1020544083674957e-06, 
       1.0019021192034605e-05, 
       1.0186679587385188e-05, 
       1.0433379442175783e-05, 
       1.0166461505592848e-05, 
       9.3547459771763122e-06, 
       9.3547459771763122e-06, 
       8.4627990073382092e-06, 
       9.7928082886333503e-06, 
       8.7912306205002656e-06, 
       1.0795533149685783e-05, 
       8.5269013027321104e-06, 
       8.4632370910551085e-06, 
       8.4632370910551085e-06, 
       9.8032526866060710e-06, 
       8.6166194987115170e-06, 
       8.4177357674216400e-06, 
       9.5775617653283435e-06, 
       1.0266672870714404e-05, 
       9.4663483929623121e-06, 
       9.4701466977615695e-06, 
       8.8038046287815367e-06, 
       9.5954177025168152e-06, 
       8.5697616820432065e-06, 
       9.0194555960488053e-06, 
       9.0189255120466999e-06, 
       9.8975915928375960e-06, 
       9.6983787526857572e-06, 
       9.6983787526857572e-06, 
       8.9493644194257832e-06, 
       1.1796679581070828e-05, 
       8.5925237907150606e-06, 
       9.8223692341669995e-06, 
       1.0101177606579987e-05, 
       9.7885556248928885e-06, 
       1.1262356646469647e-05, 
       1.0067768894138080e-05, 
       8.9987364796765496e-06, 
       8.5958929885161994e-06, 
       8.7254187691508449e-06, 
       9.8138837041622698e-06, 
       8.5561429827420834e-06, 
       8.7732407973011242e-06, 
       1.1792957579922011e-05, 
       9.9504050081533011e-06, 
       8.3455299444919103e-06, 
       9.8030991965915884e-06, 
       9.7980809577219038e-06, 
       9.1010904758197969e-06, 
       8.9201321624907420e-06, 
       8.7352304763132800e-06, 
       9.0354187114852844e-06, 
       1.0406508362932448e-05, 
       9.9386650565623355e-06, 
       9.0765000960446460e-06, 
       1.0621560838042796e-05, 
       9.2241098964281614e-06, 
       9.7859468108526106e-06, 
       8.5244880032475073e-06, 
       9.3104902787219013e-06, 
       8.8523869460707942e-06, 
       8.7811899376470511e-06, 
       9.8455577063767873e-06, 
       1.0478510876457637e-05, 
       9.3169897622483651e-06, 
       1.0040816299170210e-05, 
       1.0612346558106938e-05, 
       8.5140757397043491e-06, 
       8.4412883936659743e-06, 
       8.4412883936659743e-06, 
       9.3065083495887255e-06, 
       8.3388935841647627e-06, 
       8.9228027136218589e-06, 
       8.9228027136218589e-06, 
       9.2094188948961765e-06, 
       9.4697161310466244e-06, 
       8.3205550488787154e-06, 
       1.1476082285179164e-05, 
       8.2679623736798372e-06, 
       9.8527546484523602e-06, 
       8.5659417122952512e-06, 
       8.3226816694482874e-06, 
       1.0549274143239734e-05, 
       9.7902580207581672e-06, 
       9.0122431521735610e-06, 
       1.1570129713218390e-05, 
       9.5834138137362068e-06, 
       9.8644425354725586e-06, 
       9.8643078018134197e-06, 
       1.0583440354826471e-05, 
       8.3596060383784413e-06, 
       8.9209508582935097e-06, 
       9.7143042723297784e-06, 
       1.0839125379946887e-05, 
       9.8250909155633992e-06, 
       9.6443928285969687e-06, 
       8.6302925255577577e-06, 
       1.1614219611710227e-05, 
       9.4408398883806297e-06, 
       1.0217710138223576e-05, 
       9.4910498368356787e-06, 
       9.4910498368356787e-06, 
       9.1795251127395000e-06, 
       1.0792253652749786e-05, 
       9.4527703640180113e-06, 
       9.9022339518048429e-06, 
       1.0012746213256557e-05, 
       9.6625091952213829e-06, 
       1.1308384871347993e-05, 
       8.3858523701088737e-06, 
       8.8185020240099527e-06, 
       8.2264569290775491e-06, 
       8.6809186463087251e-06, 
       9.6673269749644050e-06, 
       1.1578267211247012e-05, 
       8.3155317317234892e-06, 
       1.0316464829673697e-05, 
       1.1545806004323859e-05, 
       1.1572900583349127e-05, 
       9.6046259507971645e-06, 
       9.7203341959591670e-06, 
       8.3893145334266619e-06, 
       1.1564097364522789e-05, 
       9.7148378588876969e-06, 
       8.9600174446238185e-06, 
       1.0658104939939485e-05, 
       8.5997936680632638e-06, 
       1.1556106328449921e-05, 
       9.0883387754851259e-06, 
       9.3668416107336625e-06, 
       1.1346011814057129e-05, 
       1.0571400853246439e-05, 
       1.1315714128925886e-05, 
       9.5744046880814586e-06, 
       8.9254028337447856e-06, 
       1.0676456813870472e-05, 
       8.6248592934786310e-06, 
       1.0201665644283194e-05, 
       8.1620599283207719e-06, 
       1.1216832582278412e-05, 
       1.0112283694257502e-05, 
       8.4024959770880573e-06, 
       9.6320657887368971e-06, 
       8.6766526760894909e-06, 
       1.1507095700849713e-05, 
       9.7875825345123419e-06, 
       8.2393089766065277e-06, 
       9.5832620559615775e-06, 
       9.7402279933637225e-06, 
       1.1075964556524318e-05, 
       1.1500931311052911e-05, 
       8.4651859723759634e-06, 
       1.1440234668465594e-05, 
       8.5716686645809429e-06, 
       9.4104734359851716e-06, 
       1.0989103300252571e-05, 
       9.3990814767558021e-06, 
       8.2368158100908824e-06, 
       1.0163330422237313e-05, 
       1.1427580183596415e-05, 
       8.1297624970274687e-06, 
       9.6338841272753390e-06, 
       8.2895648963407689e-06, 
       1.0588611992121518e-05, 
       9.9261904060029401e-06, 
       1.1424365551130071e-05, 
       1.1019718323411165e-05, 
       1.1245865773948267e-05, 
       8.1240434828092947e-06, 
       8.1130599564122140e-06, 
       1.0911672544827132e-05, 
       9.6919587935911346e-06, 
       9.5573101777326521e-06, 
       1.0893715082688273e-05, 
       1.0337612987241769e-05, 
       1.1141129563125891e-05, 
       1.1141129563125891e-05, 
       9.2866362176704509e-06, 
       9.5228355615112211e-06, 
       1.0915522604313858e-05, 
       8.5314814252202339e-06, 
       1.0422335276293300e-05, 
       1.1301714487256171e-05, 
       8.1056814038934162e-06, 
       8.1821531769789903e-06, 
       8.8631485082008062e-06, 
       8.1816305763038414e-06, 
       1.1408665313321240e-05, 
       8.1483108725705809e-06, 
       1.0976290503418787e-05, 
       8.3916659944529794e-06, 
       8.3862245004199113e-06, 
       1.1334316144781269e-05, 
       1.1394636477489268e-05, 
       9.1840069072858722e-06, 
       9.1961890652711172e-06, 
       8.3837674422858028e-06, 
       9.1090075149421652e-06, 
       8.4011241886923444e-06, 
       9.4668553331241407e-06, 
       1.1072609903073088e-05, 
       9.3828097272308220e-06, 
       1.0682237802984417e-05, 
       8.2806711833994120e-06, 
       8.1372473964898334e-06, 
       8.4610574025488490e-06, 
       9.6530221453072238e-06, 
       8.4989915408250530e-06, 
       8.3581931331143700e-06, 
       9.5392252883409472e-06, 
       1.0194485083566727e-05, 
       8.2333607442203799e-06, 
       1.1306693487970252e-05, 
       1.1264565238219749e-05, 
       9.4387058293918144e-06, 
       8.5591727815723025e-06, 
       1.1195135132882532e-05, 
       1.0069192799702587e-05, 
       8.2084879324017224e-06, 
       9.5637548099358271e-06, 
       1.1237886591376780e-05, 
       9.5894323881536232e-06, 
       1.1049217377108872e-05, 
       9.5884790988969791e-06, 
       1.0070171713254139e-05, 
       8.4822439597244781e-06, 
       9.7640416673735818e-06, 
       8.0218585497777089e-06, 
       1.0004638378238634e-05, 
       8.2689266949638085e-06, 
       1.0893632222847776e-05, 
       1.0893632222847776e-05, 
       1.0680956178865357e-05, 
       8.3068017517819875e-06, 
       8.1784889777895286e-06, 
       8.6219463879132669e-06, 
       8.6219463879132669e-06, 
       1.0613855467132364e-05, 
       8.8218469693648963e-06, 
       9.2793056077332950e-06, 
       9.2793056077332950e-06, 
       8.4202574473891134e-06, 
       8.1151335533799126e-06, 
       8.1151335533799126e-06, 
       1.1274457755722832e-05, 
       1.1276257419551856e-05, 
       9.6092942491575890e-06, 
       9.5292290691375629e-06, 
       1.0316368507571364e-05, 
       8.7919882715844521e-06, 
       1.1012207864956154e-05, 
       9.3556432264183307e-06, 
       1.1220636024965189e-05, 
       9.9545484800477327e-06, 
       8.1081579450573246e-06, 
       7.9775799100225089e-06, 
       9.9015668775049225e-06, 
       1.1118898663356273e-05, 
       1.0176888539422512e-05, 
       8.1614176038338039e-06, 
       8.1383815434114593e-06, 
       8.2715596466002474e-06, 
       9.2572923721762516e-06, 
       1.0184262149425215e-05, 
       9.1027805643622562e-06, 
       9.3196616039386248e-06, 
       8.4897077314426014e-06, 
       9.3712856359736449e-06, 
       1.0558840685668337e-05, 
       7.9701544354216161e-06, 
       9.3752238505392224e-06, 
       9.3218889974755906e-06, 
       8.1666993175257805e-06, 
       8.3145078168011290e-06, 
       8.3121202645472809e-06, 
       9.0863819992738258e-06, 
       8.0665237825047248e-06, 
       8.0665237825047248e-06, 
       9.2668813377300616e-06, 
       7.9403300894960625e-06, 
       7.9423071954399177e-06, 
       1.1162899207149065e-05, 
       1.1128981634165691e-05, 
       9.3130325154253909e-06, 
       9.9447426443385946e-06, 
       7.9557724566108447e-06, 
       7.9557724566108447e-06, 
       9.1969239597557831e-06, 
       9.7137420484488393e-06, 
       9.6988226889307845e-06, 
       8.6084317975669163e-06, 
       8.2079364984215487e-06, 
       9.2664193044474990e-06, 
       9.3982271571577779e-06, 
       9.5640531537992826e-06, 
       9.5640531537992826e-06, 
       8.2449659480421945e-06, 
       1.1124283028070484e-05, 
       1.0230642787435756e-05, 
       1.0752795102935563e-05, 
       8.1338138265136051e-06, 
       9.8627448433822505e-06, 
       8.4650446148437809e-06, 
       9.2391110915947779e-06, 
       1.1106049283376227e-05, 
       9.8378477792810784e-06, 
       8.9389654608879446e-06, 
       9.8346640257821793e-06, 
       8.9382058328346428e-06, 
       8.3484270957921402e-06, 
       1.0233371573847037e-05, 
       1.1088021841344722e-05, 
       9.2783339610245754e-06, 
       8.3225209852060744e-06, 
       9.2128916416207421e-06, 
       7.9901962606480104e-06, 
       9.9016690306449276e-06, 
       7.8386061408966458e-06, 
       9.2005141561892868e-06, 
       8.2984346169051845e-06, 
       8.6539252123255984e-06, 
       9.2124512922854893e-06, 
       9.5115349495428261e-06, 
       9.0089720991261276e-06, 
       8.2100488070487949e-06, 
       8.4271242573932820e-06, 
       9.4131783256638764e-06, 
       7.9100925880461155e-06, 
       1.0847178748180059e-05, 
       9.0450837230742403e-06, 
       8.0075310697306971e-06, 
       8.8941358635953865e-06, 
       8.0420935970326870e-06, 
       9.1530907382480584e-06, 
       8.0362852081996535e-06, 
       8.5963193552956587e-06, 
       7.9555716617612778e-06, 
       9.1513943716710290e-06, 
       8.2216892573174026e-06, 
       7.8785993652022820e-06, 
       1.0681612575009408e-05, 
       9.4959527473528723e-06, 
       9.2302143646798746e-06, 
       9.2523306144933668e-06, 
       7.7994491477371091e-06, 
       8.0348031480158002e-06, 
       8.5346461244377665e-06, 
       1.0870129031629068e-05, 
       1.0719784547633156e-05, 
       1.0570725731885925e-05, 
       1.0254791872251513e-05, 
       1.1003608909046722e-05, 
       1.1002988321622837e-05, 
       1.0700791483295746e-05, 
       1.0677495842633823e-05, 
       9.0585154067311918e-06, 
       9.2687599562972307e-06, 
       9.4945450392505599e-06, 
       1.0508822285229782e-05, 
       8.3197492905251875e-06, 
       8.3788156141465420e-06, 
       1.0717755599924656e-05, 
       1.0917401695371128e-05, 
       7.8812828229993523e-06, 
       1.0889336079675683e-05, 
       9.9857422280705700e-06, 
       8.4230134506980076e-06, 
       8.6870954671172861e-06, 
       7.9550458896130485e-06, 
       1.0862220316611429e-05, 
       1.0597348878727284e-05, 
       8.1776104126236317e-06, 
       8.7995753931053764e-06, 
       8.8488374657337124e-06, 
       1.0274406831173117e-05, 
       1.0921586273018134e-05, 
       1.0617489836286677e-05, 
       7.9396086210649903e-06, 
       9.0555060309151793e-06, 
       7.7322114107149703e-06, 
       7.9568345156984707e-06, 
       7.7296661513056158e-06, 
       9.8567056480158617e-06, 
       1.0828355356545615e-05, 
       8.1632875317698573e-06, 
       9.9980436073042535e-06, 
       8.7756386298331642e-06, 
       9.5496381812452384e-06, 
       8.1997384717698682e-06, 
       9.1290559422702391e-06, 
       1.0069982616257073e-05, 
       9.8107269212403520e-06, 
       8.7745081171285365e-06, 
       9.2902573090105335e-06, 
       8.8080810145371141e-06, 
       8.5183774160555534e-06, 
       9.0119386789277543e-06, 
       9.3845757811806129e-06, 
       8.6727001937326769e-06, 
       7.7012325764527980e-06, 
       7.7709031664546681e-06, 
       9.3561672883305658e-06, 
       8.0433027691680343e-06, 
       8.5328585984970745e-06, 
       1.0245847881954835e-05, 
       9.0314913757531635e-06, 
       9.0268453426130139e-06, 
       8.5328502014238343e-06, 
       7.9047978874289847e-06, 
       9.0042641928965590e-06, 
       8.1802765907632405e-06, 
       9.2369147618282662e-06, 
       8.5973714904737580e-06, 
       7.7833564634743368e-06, 
       7.6958721290966612e-06, 
       7.6819578750149307e-06, 
       7.9003985346023638e-06, 
       9.1000345798889766e-06, 
       7.8437884963041966e-06, 
       7.6852760955990472e-06, 
       8.3293754165624045e-06, 
       1.0301855177730258e-05, 
       1.0411235492559563e-05, 
       9.7190533834427092e-06, 
       1.0665759078555350e-05, 
       8.7343770647866526e-06, 
       7.7247384834149740e-06, 
       8.9391403154481494e-06, 
       1.0438960006456072e-05, 
       7.6768868726883430e-06, 
       8.2049849145992558e-06, 
       8.7161004404253225e-06, 
       8.3514627692815660e-06, 
       8.3362377768541952e-06, 
       9.3954646876017580e-06, 
       8.7348770311387645e-06, 
       7.7703674294009935e-06, 
       7.8840869331188450e-06, 
       8.0832225024337957e-06, 
       9.5744946107751445e-06, 
       7.9859711956653885e-06, 
       8.9665073673582473e-06, 
       7.7522570552000642e-06, 
       7.9275241583138652e-06, 
       7.9744162316582475e-06, 
       1.0520314203986481e-05, 
       8.0108409499673174e-06, 
       7.8412835969871585e-06, 
       1.0635794769335427e-05, 
       9.0882293627284289e-06, 
       7.6864339919134233e-06, 
       9.6538772515057335e-06, 
       8.2689131120307599e-06, 
       7.9016347659010911e-06, 
       8.2033203665399499e-06, 
       8.6152172990149733e-06, 
       1.0195641252320373e-05, 
       8.5603586538935639e-06, 
       1.0383119443405808e-05, 
       8.9560245047916898e-06, 
       1.0549226444422211e-05, 
       7.7978952138079091e-06, 
       9.3775407997062509e-06, 
       7.9903078497188967e-06, 
       7.5747733294260965e-06, 
       8.2244928215429057e-06, 
       8.0130976294730647e-06, 
       7.7532604664496800e-06, 
       1.0183504612127337e-05, 
       8.6303187536373533e-06, 
       1.0674042092238093e-05, 
       1.0461883361577260e-05, 
       8.1668144868069544e-06, 
       1.0544989293039738e-05, 
       7.6640394542293375e-06, 
       1.0340164451212245e-05, 
       9.2198750334892818e-06, 
       9.2537332410885918e-06, 
       8.0093887382391771e-06, 
       7.7412763915784586e-06, 
       8.3816165834963131e-06, 
       9.6740786322865163e-06, 
       7.6098632375998587e-06, 
       8.6082039998256647e-06, 
       8.0668497052821541e-06, 
       8.0690616456404578e-06, 
       8.5154007010916605e-06, 
       7.5992216221021936e-06, 
       8.1620506992688687e-06, 
       8.2382536268793300e-06, 
       8.7401136645946120e-06, 
       7.7492106044145923e-06, 
       7.9342063839498765e-06, 
       8.0855420810425071e-06, 
       7.8501600772728562e-06, 
       8.8216392840787369e-06, 
       8.2417505156770137e-06, 
       8.1717314072238489e-06, 
       1.0529537367040197e-05, 
       8.1907584461971306e-06, 
       7.9061123553144055e-06, 
       8.6409523098516972e-06, 
       7.5388156540367476e-06, 
       9.2295502921108319e-06, 
       7.8734464299847550e-06, 
       7.6243967876995583e-06, 
       9.3087126958352223e-06, 
       9.2779255097197296e-06, 
       7.4938549443077470e-06, 
       1.0287262239378611e-05, 
       8.9521218321704762e-06, 
       8.5053047090478842e-06, 
       7.6768854336133028e-06, 
       8.8139990087388334e-06, 
       7.5812302051459350e-06, 
       9.1579781370846212e-06, 
       8.3525442477463480e-06, 
       8.9754630221939822e-06, 
       8.0398994293974298e-06, 
       7.8955052057535876e-06, 
       7.4726216040131691e-06, 
       7.5100997582979282e-06, 
       8.7229574002592308e-06, 
       7.6753502954028464e-06, 
       9.1357389198633441e-06, 
       7.6294121191425054e-06, 
       8.7437604477771877e-06, 
       1.0115174160072227e-05, 
       7.5802989770960316e-06, 
       7.4876012667325821e-06, 
       1.0060169217245969e-05, 
       1.0037132983848174e-05, 
       7.6156354674581140e-06, 
       7.4402296822310429e-06, 
       1.0464512661222532e-05, 
       9.2305340931117760e-06, 
       1.0474915173810910e-05, 
       7.9489144080405492e-06, 
       7.4990809942112595e-06, 
       8.4738663846739184e-06, 
       7.8254269818318990e-06, 
       8.8813083656097161e-06, 
       7.6501296179721846e-06, 
       7.7478829669640355e-06, 
       7.3913688321490851e-06, 
       9.2417469302986720e-06, 
       1.0445440324972980e-05, 
       7.4193606906722409e-06, 
       1.0099328646795399e-05, 
       7.5375142372720485e-06, 
       9.3145216951914641e-06, 
       9.2143327967594379e-06, 
       8.5370309756764049e-06, 
       7.6624900513929440e-06, 
       8.6678651573534391e-06, 
       9.0598042155435764e-06, 
       8.6113128087237438e-06, 
       9.1631530855462216e-06, 
       9.7245454299966835e-06, 
       1.0395937131233272e-05, 
       8.3333042324777245e-06, 
       1.0274837844931824e-05, 
       7.9255127079015939e-06, 
       8.9071752241408234e-06, 
       8.9071752241408234e-06, 
       8.6453032592448570e-06, 
       9.9336776929419432e-06, 
       8.1059382105455300e-06, 
       7.3915988762433960e-06, 
       9.0536726748379587e-06, 
       8.4095319761341917e-06, 
       7.6862391376933517e-06, 
       7.7044253283202774e-06, 
       7.7609188999259821e-06, 
       9.0409648321802874e-06, 
       8.6081173272119171e-06, 
       8.7418252208292769e-06, 
       8.7418252208292769e-06, 
       9.5212150102631109e-06, 
       7.5573176042884998e-06, 
       7.8633644573347822e-06, 
       7.8398359799655931e-06, 
       1.0042893138034025e-05, 
       7.3744521957410257e-06, 
       9.6848206154956413e-06, 
       1.0359576163545988e-05, 
       1.0350391092904932e-05, 
       1.0352055924613761e-05, 
       8.6129963153709672e-06, 
       1.0243803239763929e-05, 
       9.1673913199148270e-06, 
       7.7601281464880230e-06, 
       8.7779643287890737e-06, 
       7.3713503515549822e-06, 
       7.3713503515549822e-06, 
       8.7083699729784656e-06, 
       7.5980363877438524e-06, 
       9.1428760147405649e-06, 
       8.4719482054638628e-06, 
       7.6247335739739216e-06, 
       8.1889684015296987e-06, 
       7.5269127567406981e-06, 
       9.2622992301378082e-06, 
       1.0035642918459950e-05, 
       7.8707223685400925e-06, 
       8.7277681350533409e-06, 
       7.3451493477256893e-06, 
       9.2728425367906016e-06, 
       7.4614093461191979e-06, 
       7.7075727683536397e-06, 
       8.5959454642636525e-06, 
       9.0318784538119424e-06, 
       9.7788261886435019e-06, 
       1.0291934679547354e-05, 
       8.6011960295699683e-06, 
       7.5915387001339136e-06, 
       7.3195623289504928e-06, 
       8.1207291197665533e-06, 
       1.0302512364327620e-05, 
       8.8051830512342144e-06, 
       7.5404385394625655e-06, 
       9.4683598132942462e-06, 
       8.3019876432134816e-06, 
       8.6876036564257291e-06, 
       7.6696174387384350e-06, 
       1.0257201462673399e-05, 
       7.2852140668582012e-06, 
       7.8173847771761922e-06, 
       8.5298161718801100e-06, 
       7.3009019774590933e-06, 
       1.0155697241880736e-05, 
       7.6691235381770117e-06, 
       8.0471136583741190e-06, 
       9.2610071247805391e-06, 
       8.9621828855895392e-06, 
       7.7628049480058955e-06, 
       9.0469969945026715e-06, 
       1.0164066053341233e-05, 
       7.7619632611060494e-06, 
       7.2995643133299642e-06, 
       9.4939567882661035e-06, 
       7.3867114100195525e-06, 
       7.4035314210113230e-06, 
       1.0218059520758884e-05, 
       7.8700345022050083e-06, 
       7.2839547681303544e-06, 
       8.6834388122850430e-06, 
       7.6332952143167844e-06, 
       1.0161348600242670e-05, 
       8.2234785347685320e-06, 
       9.7205725339627478e-06, 
       9.3756585031684713e-06, 
       7.4713380739200797e-06, 
       1.0208921133790667e-05, 
       8.6805370358456099e-06, 
       7.6227730155946613e-06, 
       7.7387847229100630e-06, 
       7.8921539272629087e-06, 
       7.4502461498157010e-06, 
       8.1994657054351247e-06, 
       1.0181221721079982e-05, 
       7.3884764847084409e-06, 
       8.2410530143383462e-06, 
       9.7820539422396204e-06, 
       7.3442218566782460e-06, 
       7.1960192227982234e-06, 
       1.0136864191515192e-05, 
       8.4555878520634406e-06, 
       7.4257835245479381e-06, 
       8.1263302751145928e-06, 
       7.6841579379593006e-06, 
       1.0156269091877432e-05, 
       7.8213516604529091e-06, 
       7.3290925495992384e-06, 
       7.2823986026147364e-06, 
       7.5291771598954287e-06, 
       8.7799244618344682e-06, 
       7.4935611001727147e-06, 
       7.4912648651637252e-06, 
       7.4903252961214336e-06, 
       9.2636242397806070e-06, 
       7.3563649245148374e-06, 
       7.1986565167205011e-06, 
       7.5034079436264009e-06, 
       7.6675601425445385e-06, 
       7.5983589005594174e-06, 
       9.5966072688652656e-06, 
       9.9735328138779986e-06, 
       7.5069530377776847e-06, 
       7.8530422891891276e-06, 
       7.4604321377178627e-06, 
       7.2579046482994943e-06, 
       1.0120621440372561e-05, 
       8.9915304454425184e-06, 
       1.0093439660243728e-05, 
       8.1883193553535821e-06, 
       9.2840392333524554e-06, 
       7.9191577538986502e-06, 
       9.5590538408543123e-06, 
       8.9968656205260790e-06, 
       8.3096176131982932e-06, 
       7.7164384381443804e-06, 
       8.4936355911240060e-06, 
       7.2778495668730288e-06, 
       9.6110688738410582e-06, 
       8.0381419829550866e-06, 
       7.6861802837996033e-06, 
       8.3856990374699769e-06, 
       9.8299537980685379e-06, 
       7.9403649077885954e-06, 
       7.2954467465183168e-06, 
       8.3797149674630468e-06, 
       7.4248086665694123e-06, 
       9.5724833618397886e-06, 
       8.5702688031091007e-06, 
       8.5702688031091007e-06, 
       7.2797403433225919e-06, 
       8.3778500700876071e-06, 
       8.4324248743694243e-06, 
       7.2715569552598720e-06, 
       7.7580289807621388e-06, 
       8.1998293214437180e-06, 
       1.0033577001400010e-05, 
       7.2950433907370081e-06, 
       7.2942998560565074e-06, 
       9.8768428626717588e-06, 
       8.3448342469311900e-06, 
       7.6022904488171252e-06, 
       9.7454991896392173e-06, 
       7.7210622996243804e-06, 
       8.8264807025657653e-06, 
       7.9521900405611917e-06, 
       8.1650712336678060e-06, 
       9.8740593447710227e-06, 
       9.8740593447710227e-06, 
       9.6449601723023314e-06, 
       8.1588287999408652e-06, 
       9.4723107647543282e-06, 
       7.3761702053635474e-06, 
       8.1258377874538328e-06, 
       8.1256431244713322e-06, 
       7.2349402453334237e-06, 
       9.4275061385919854e-06, 
       8.1072772872498990e-06, 
       7.7976303675472471e-06, 
       9.6073814282074118e-06, 
       7.0758604436048121e-06, 
       7.0758604436048121e-06, 
       7.4186564220463898e-06, 
       7.5512353127705052e-06, 
       8.6517925198525297e-06, 
       8.0920101996311514e-06, 
       8.8763120998898750e-06, 
       7.5410748994443664e-06, 
       7.1286199751796116e-06, 
       7.4609823016266310e-06, 
       7.5191772934971114e-06, 
       8.6069665581161527e-06, 
       7.2782077190550146e-06, 
       7.4917849195692477e-06, 
       9.7196388123727519e-06, 
       9.9697199916447367e-06, 
       8.2928682886990817e-06, 
       8.2772573089046558e-06, 
       9.8086798052902393e-06, 
       7.1327035321631547e-06, 
       9.5757342656985402e-06, 
       7.0639503905372047e-06, 
       7.6897575476941053e-06, 
       8.2559756910233242e-06, 
       8.1997055593983706e-06, 
       9.2349159468474991e-06, 
       7.1956662051401743e-06, 
       7.1956662051401743e-06, 
       9.8679499392978896e-06, 
       9.8270500926272297e-06, 
       8.2876486069556001e-06, 
       8.5554930389289843e-06, 
       8.1985629601931549e-06, 
       7.1323774024794711e-06, 
       8.3312776173427957e-06, 
       8.2447165353865727e-06, 
       9.1956245273527530e-06, 
       9.1956245273527530e-06, 
       8.8090692126762870e-06, 
       7.3616308877820578e-06, 
       9.7256898932906996e-06, 
       9.7328930037142975e-06, 
       9.8083203169867580e-06, 
       9.6305724349251221e-06, 
       8.5912271533917025e-06, 
       9.7781133476709457e-06, 
       7.0844994599387306e-06, 
       9.6337617630833450e-06, 
       7.0311783752497355e-06, 
       9.8457077282660722e-06, 
       7.3401881967765916e-06, 
       9.8816836353625960e-06, 
       7.8144078171565622e-06, 
       8.3267294054764985e-06, 
       8.2041054121670727e-06, 
       9.7886383390755596e-06, 
       8.1304903494721963e-06, 
       9.7530866090488051e-06, 
       7.3794304395828917e-06, 
       8.1988392948123450e-06, 
       6.9708920314343526e-06, 
       7.5999379411477947e-06, 
       7.2695447279310253e-06, 
       8.5539667019671981e-06, 
       7.4620572273169090e-06, 
       9.8319109962910036e-06, 
       8.1091444806706032e-06, 
       7.5153599271950633e-06, 
       6.9863981007945262e-06, 
       7.5326525960982858e-06, 
       6.9753609673637478e-06, 
       7.2600523951827833e-06, 
       8.3463825467006557e-06, 
       7.3326980319849971e-06, 
       7.7183924030754683e-06, 
       9.7894715382345001e-06, 
       7.6962795301090991e-06, 
       9.7553875977186139e-06, 
       8.9598433263375079e-06, 
       8.3528111122777484e-06, 
       8.1457153620441806e-06, 
       7.8378057518544266e-06, 
       6.9819447889327140e-06, 
       9.4314976476721552e-06, 
       8.2723445659586661e-06, 
       7.0553177804164477e-06, 
       7.2965496146168832e-06, 
       8.2533591122562355e-06, 
       8.1170332083824554e-06, 
       9.7741631699649544e-06, 
       7.8211796525313481e-06, 
       8.7700068071272317e-06, 
       8.4943294136905936e-06, 
       9.2414115201871608e-06, 
       9.6906542939443421e-06, 
       8.8922999139681438e-06, 
       8.1348538305619821e-06, 
       7.2695512334270988e-06, 
       8.9039893920663026e-06, 
       8.3492574079326468e-06, 
       7.0929200648926815e-06, 
       7.1584122384943713e-06, 
       9.2019341833575309e-06, 
       9.6509862313190868e-06, 
       6.9376083257949937e-06, 
       7.2879794345286969e-06, 
       9.6946987688541102e-06, 
       6.9359331277862900e-06, 
       7.2631456512703167e-06, 
       9.5846130198507103e-06, 
       7.0123578707325427e-06, 
       8.0720724366374411e-06, 
       7.1408981820968414e-06, 
       9.0389909888344436e-06, 
       9.6828526620404540e-06, 
       6.9958365486258549e-06, 
       7.0594675875800848e-06, 
       8.4366020628842735e-06, 
       9.4127420836238821e-06, 
       7.2939525196515608e-06, 
       7.8253484974940789e-06, 
       7.4774724884137749e-06, 
       7.9031259249848090e-06, 
       9.3221081355370253e-06, 
       9.1818530247256396e-06, 
       7.0609278101675739e-06, 
       8.0403582403689009e-06, 
       6.8414073614937899e-06, 
       8.0371142262716750e-06, 
       6.9655204734358998e-06, 
       7.3131279198415955e-06, 
       6.9562856729521153e-06, 
       9.4065281218122541e-06, 
       7.3619075244112493e-06, 
       8.0655738756565442e-06, 
       8.0113891974961052e-06, 
       7.0463997603980868e-06, 
       7.4620419119801679e-06, 
       8.1769544433499168e-06, 
       8.1769544433499168e-06, 
       7.2541459096988303e-06, 
       7.3002442721849942e-06, 
       9.3602525278444049e-06, 
       8.7621550572143473e-06, 
       9.5587543300814092e-06, 
       8.0205676051001481e-06, 
       7.5648356011898606e-06, 
       9.6185693974811439e-06, 
       7.2740577344015332e-06, 
       9.2887238767578941e-06, 
       6.9573154279322068e-06, 
       7.4056419912235992e-06, 
       7.4056419912235992e-06, 
       8.7725388811248681e-06, 
       7.6651866035158580e-06, 
       7.0485646909336939e-06, 
       8.7722378125792570e-06, 
       8.6498391581864232e-06, 
       7.4995783105403575e-06, 
       8.0225029362850735e-06, 
       9.6112366408622248e-06, 
       9.3155322710559862e-06, 
       8.1020860055524957e-06, 
       7.3322963033713459e-06, 
       8.8422095295971853e-06, 
       8.3166421304567018e-06, 
       8.5922142292589426e-06, 
       7.1019906471807894e-06, 
       9.0948657825844880e-06, 
       7.1967065174912637e-06, 
       7.6841067043832354e-06, 
       9.5803440427786459e-06, 
       9.0405665292249316e-06, 
       6.8738472867025731e-06, 
       8.1382816103142463e-06, 
       7.7318304603220828e-06, 
       6.9744889729743760e-06, 
       9.1963561541976896e-06, 
       8.0621597026890802e-06, 
       8.0058682119296321e-06, 
       7.1341012664360027e-06, 
       7.1341012664360027e-06, 
       7.2561597411357307e-06, 
       6.9440398237390533e-06, 
       7.8107561287240352e-06, 
       6.7785646817114017e-06, 
       8.0130136286493813e-06, 
       9.4831266625608775e-06, 
       8.0581781457332993e-06, 
       7.5520164974274102e-06, 
       7.7473483871527097e-06, 
       7.6975254681945178e-06, 
       7.1132307101934392e-06, 
       7.9031386051158613e-06, 
       6.9053521317003693e-06, 
       7.9239112646334187e-06, 
       9.4561527580235451e-06, 
       9.5271632773332470e-06, 
       6.9984252377406487e-06, 
       7.4517070363776825e-06, 
       7.2639632698890872e-06, 
       7.8178559768832763e-06, 
       8.7697158820766970e-06, 
       8.1420230389920002e-06, 
       7.0334899732600404e-06, 
       7.1714930142710134e-06, 
       9.4330078740725921e-06, 
       8.8091701184626974e-06, 
       7.2706547785382884e-06, 
       7.0115161570043416e-06, 
       9.0423830955011811e-06, 
       9.3999744775062304e-06, 
       6.7425448450299707e-06, 
       8.8241773079381535e-06, 
       7.2002236700754120e-06, 
       6.8312840162122383e-06, 
       9.4345453310146607e-06, 
       7.3660770575756916e-06, 
       7.3418363857439904e-06, 
       8.1188860987120926e-06, 
       8.9185186984834552e-06, 
       9.4729908571243130e-06, 
       7.7557560782905311e-06, 
       9.1875365543623014e-06, 
       7.9733320674767990e-06, 
       8.2145122776506850e-06, 
       7.0333805808212549e-06, 
       8.4172920601199673e-06, 
       7.0555351115119565e-06, 
       7.0782694199661069e-06, 
       8.2311884686145158e-06, 
       8.3518479559606461e-06, 
       9.4236289155637985e-06, 
       9.4401107224254513e-06, 
       8.5418851893152376e-06, 
       9.2137575148251859e-06, 
       7.9264400671756219e-06, 
       7.8251546424924307e-06, 
       9.3718368389183097e-06, 
       6.9583678457639486e-06, 
       6.7349676129490696e-06, 
       6.7563397036981006e-06, 
       7.3294539024553638e-06, 
       6.6607173307959310e-06, 
       7.3590952960829240e-06, 
       7.3059731209727630e-06, 
       7.8922593944444480e-06, 
       8.4318575308163327e-06, 
       9.4011807194441671e-06, 
       8.9339173219913714e-06, 
       8.0839945736551128e-06, 
       6.7467625964429040e-06, 
       6.7467625964429040e-06, 
       8.3588442700307246e-06, 
       6.8317009949242571e-06, 
       7.0130639700049206e-06, 
       8.0920101996331639e-06, 
       7.1808964872536828e-06, 
       7.0571409558812126e-06, 
       6.6548233743813297e-06, 
       7.7044994213222037e-06, 
       6.6342955198375408e-06, 
       7.8790143897317364e-06, 
       6.8280299424638658e-06, 
       6.8556152754124818e-06, 
       6.9791557535340669e-06, 
       6.6248946547636529e-06, 
       9.2147722391755671e-06, 
       6.6505806544854734e-06, 
       6.6505806544854734e-06, 
       7.4778303668220133e-06, 
       9.2231494546389956e-06, 
       7.3725528621304886e-06, 
       7.7840117742958585e-06, 
       9.3318422593327715e-06, 
       7.4034329758736306e-06, 
       7.2214648167241815e-06, 
       7.0699867118935984e-06, 
       7.6462040038293282e-06, 
       9.2291686206828167e-06, 
       7.9159964893499741e-06, 
       6.9113012173785423e-06, 
       7.1353400647428976e-06, 
       6.6103222641621750e-06, 
       9.3102462488111512e-06, 
       9.0746291707480987e-06, 
       6.6268259049878282e-06, 
       7.0367349677029740e-06, 
       9.0786076153189231e-06, 
       7.8584827224449888e-06, 
       9.2531473044081081e-06, 
       9.2986245220478116e-06, 
       8.9417254999352876e-06, 
       6.7897802955711695e-06, 
       8.7688459716966336e-06, 
       7.2569612885574776e-06, 
       7.7163313472312421e-06, 
       8.7220978972782175e-06, 
       7.0998373539109099e-06, 
       7.5627052324265657e-06, 
       8.4650258913157174e-06, 
       6.9828998804383820e-06, 
       7.9746856665137826e-06, 
       6.9297543682532431e-06, 
       6.5771804244100982e-06, 
       6.5771804244100982e-06, 
       7.4890132312797471e-06, 
       7.3979498735569698e-06, 
       9.2725911981186198e-06, 
       9.0817363724073848e-06, 
       9.1432624788577325e-06, 
       7.2457460287555945e-06, 
       7.1963679331424395e-06, 
       7.2502253066581621e-06, 
       6.8920983307010275e-06, 
       7.7893943370099836e-06, 
       8.6433265963005241e-06, 
       7.2805967397125414e-06, 
       8.5244315921790046e-06, 
       7.7550473013884136e-06, 
       7.0567694936574281e-06, 
       7.8521916916546849e-06, 
       6.9072254622430874e-06, 
       7.8477381502237174e-06, 
       9.1766855908569807e-06, 
       7.9645294428216512e-06, 
       8.8291190718911000e-06, 
       9.2336847247763000e-06, 
       6.6015429233127112e-06, 
       7.7042302017165378e-06, 
       9.2288231303598275e-06, 
       7.9055938434267945e-06, 
       7.4592961950107909e-06, 
       6.5491929133676266e-06, 
       7.6620451945574120e-06, 
       8.9221300377110478e-06, 
       8.3010984150470864e-06, 
       6.5474248716219182e-06, 
       8.0798256190126098e-06, 
       8.0798256190126098e-06, 
       7.6308146864574621e-06, 
       9.1052830972083158e-06, 
       6.6356742232704856e-06, 
       7.6478204372606517e-06, 
       6.7877071224781175e-06, 
       6.7043707098962448e-06, 
       6.9696284110407584e-06, 
       6.8430889404345125e-06, 
       7.8145467752953302e-06, 
       7.5895290214197811e-06, 
       8.2979264840318532e-06, 
       7.0225138182682791e-06, 
       6.5598947379226666e-06, 
       6.5263582528281802e-06, 
       8.0185240534889043e-06, 
       6.8591369464065336e-06, 
       6.6246946532254745e-06, 
       7.5831469362897161e-06, 
       6.5760772155722983e-06, 
       6.5860489774443317e-06, 
       9.1619977610123552e-06, 
       6.8396081617743118e-06, 
       9.1607080319894974e-06, 
       7.7069453503989028e-06, 
       7.5255181879462910e-06, 
       7.2423229291722075e-06, 
       6.5535409390625275e-06, 
       6.8339596578175687e-06, 
       6.5521607800905488e-06, 
       6.8199063245202792e-06, 
       7.6166767767009324e-06, 
       7.1642574103318959e-06, 
       9.1368235071684835e-06, 
       8.7001839197489778e-06, 
       6.6742214994262911e-06, 
       7.0055951277443041e-06, 
       8.5909450277774095e-06, 
       7.4923095572379223e-06, 
       6.8897996141610245e-06, 
       8.2064513474718100e-06, 
       7.0183627537525882e-06, 
       6.8217802487833250e-06, 
       8.7339399319581167e-06, 
       7.6249760639406681e-06, 
       8.2845651555819609e-06, 
       6.4672227601571984e-06, 
       7.0055068108393741e-06, 
       7.7161503679644622e-06, 
       8.8286353597846351e-06, 
       8.9510162436432471e-06, 
       6.5684851092576428e-06, 
       6.5684851092576428e-06, 
       6.5508866889942678e-06, 
       6.9180462463258925e-06, 
       8.5140284645219045e-06, 
       7.6621449265000683e-06, 
       9.0602526508623823e-06, 
       7.1518686419180610e-06, 
       6.5506460651062299e-06, 
       6.7914480886844448e-06, 
       7.7286393475152642e-06, 
       7.7286393475152642e-06, 
       7.7938798587020840e-06, 
       6.6237503088493044e-06, 
       8.2824168058431387e-06, 
       6.5322330079088950e-06, 
       8.2952146977195562e-06, 
       7.6354298398456278e-06, 
       6.7797129833401708e-06, 
       7.8277446877155383e-06, 
       8.8513599619117221e-06, 
       7.5720400061854970e-06, 
       8.5175754060970068e-06, 
       9.0636993764960856e-06, 
       6.5667610439140813e-06, 
       7.9052856087990585e-06, 
       7.8898233996348284e-06, 
       6.7711524854142965e-06, 
       9.0665994778222575e-06, 
       7.1946378227210689e-06, 
       6.7036901087952936e-06, 
       6.4384530719176966e-06, 
       6.5301888219157093e-06, 
       7.0906931992334687e-06, 
       9.0465546796408257e-06, 
       7.4465974784187845e-06, 
       6.8144679288521984e-06, 
       7.2643321294304105e-06, 
       8.6656111308997235e-06, 
       6.8648583168668612e-06, 
       8.3691712041021568e-06, 
       8.3691712041021568e-06, 
       6.7620815394756523e-06, 
       6.6778598689149639e-06, 
       7.6543380767563227e-06, 
       8.7542739742999081e-06, 
       9.0393374809911484e-06, 
       8.5187046038331045e-06, 
       8.7412468174090400e-06, 
       8.8855512930596958e-06, 
       7.5091204148449903e-06, 
       7.6307304300467335e-06, 
       6.4096719389816361e-06, 
       8.9099621029462012e-06, 
       8.3246379116954724e-06, 
       6.5962157263513901e-06, 
       6.3933113114614037e-06, 
       7.3519486084261897e-06, 
       6.4969275509580127e-06, 
       6.6857146335327857e-06, 
       7.8546493713742808e-06, 
       8.0013724890205717e-06, 
       7.6421612553330652e-06, 
       6.6374329641449255e-06, 
       8.0623647794336033e-06, 
       8.8039277632004543e-06, 
       8.9963744768525235e-06, 
       8.9871604762801666e-06, 
       7.2834477113909146e-06, 
       6.5228085492378918e-06, 
       7.3498006447304801e-06, 
       6.6019437100402593e-06, 
       8.7742980251340573e-06, 
       8.9849436974476711e-06, 
       6.5309578916674192e-06, 
       6.3867657018400496e-06, 
       8.6185752280280464e-06, 
       7.4675639816437450e-06, 
       6.9563312052050034e-06, 
       7.8859551537378748e-06, 
       8.9754900345756206e-06, 
       7.4500106276723023e-06, 
       8.9688963519355511e-06, 
       6.5582768655317222e-06, 
       6.4242157115110067e-06, 
       6.7375196424337785e-06, 
       6.6024831789930687e-06, 
       6.6024831789930687e-06, 
       6.7341810314687135e-06, 
       6.6591381745472444e-06, 
       7.2641374856137321e-06, 
       7.2641374856137321e-06, 
       6.9497242991630664e-06, 
       7.0029522369466259e-06, 
       7.5768103314666172e-06, 
       7.3107897533005535e-06, 
       6.5225792984745791e-06, 
       7.1356366163167647e-06, 
       8.0692365433367166e-06, 
       7.4444256159581871e-06, 
       7.1460107286311461e-06, 
       8.4275147813363165e-06, 
       7.0167331257169006e-06, 
       8.5905769790151593e-06, 
       8.9242152432771606e-06, 
       6.8886601593604030e-06, 
       6.3474672375825963e-06, 
       7.1929088275143092e-06, 
       7.1929088275143092e-06, 
       6.5654030041431047e-06, 
       6.9297082384390362e-06, 
       6.9297082384390362e-06, 
       8.9174526297043432e-06, 
       7.4515543233904749e-06, 
       7.3364888995001921e-06, 
       8.2753648078383072e-06, 
       8.2753648078383072e-06, 
       6.5516713789820937e-06, 
       7.1295750680138787e-06, 
       6.3078553517831054e-06, 
       7.4103657809244833e-06, 
       8.3629971692932712e-06, 
       7.3616465907945938e-06, 
       6.3372398729124551e-06, 
       6.5706063501504345e-06, 
       6.6456744548685912e-06, 
       6.7757057733577256e-06, 
       7.4086594382659704e-06, 
       6.7505533631938813e-06, 
       6.5360404508770609e-06, 
       7.3150528154931461e-06, 
       8.7285503443286675e-06, 
       7.1561224822003936e-06, 
       6.7909187309139897e-06, 
       6.4355689958022936e-06, 
       6.4355689958022936e-06, 
       7.2020014942128767e-06, 
       7.5183185827913282e-06, 
       6.6064049470720472e-06, 
       6.3002053911673322e-06, 
       8.7862137373627173e-06, 
       6.5812627415038502e-06, 
       8.0189033321025132e-06, 
       8.7775055137521579e-06, 
       7.3997968472898606e-06, 
       7.4804915528694705e-06, 
       6.2686442548773163e-06, 
       7.6286438487275157e-06, 
       7.4441561761663887e-06, 
       6.3560278006408548e-06, 
       7.2319543526141497e-06, 
       7.2991791113879763e-06, 
       6.3254200073059143e-06, 
       6.6913621685037686e-06, 
       7.3866593829998167e-06, 
       6.6826615682511985e-06, 
       8.8438123135625688e-06, 
       7.4505820885705501e-06, 
       6.2588206990894915e-06, 
       8.1456444943191182e-06, 
       7.5222728635699052e-06, 
       6.2803341098763689e-06, 
       6.2602441708171866e-06, 
       8.4328009011942075e-06, 
       8.8311432645383168e-06, 
       6.2515316972227607e-06, 
       7.2108069833971247e-06, 
       6.2912568150644549e-06, 
       6.2460126475488882e-06, 
       8.4601248272563673e-06, 
       6.7695777696724625e-06, 
       7.1336281004901149e-06, 
       6.5815994036313524e-06, 
       8.7347914051599332e-06, 
       7.1428506898094357e-06, 
       6.2684736108602153e-06, 
       6.5343859108454189e-06, 
       6.5343859108454189e-06, 
       6.2757831161466615e-06, 
       6.7284843171276053e-06, 
       8.3997266957885311e-06, 
       8.3997266957885311e-06, 
       7.4157811085411235e-06, 
       8.6480482353852202e-06, 
       7.1208558260262440e-06, 
       6.7626071642231572e-06, 
       7.8799488566105269e-06, 
       8.3291560350584627e-06, 
       7.2475475861736808e-06, 
       6.5543031976112280e-06, 
       6.8115595953373345e-06, 
       7.3002588688526544e-06, 
       7.3975234915129323e-06, 
       8.4977761668622579e-06, 
       8.6231294225766718e-06, 
       6.4197123049953335e-06, 
       6.4197123049953335e-06, 
       8.7629367907792299e-06, 
       8.5908154202243397e-06, 
       8.5908154202243397e-06, 
       6.2353605710632741e-06, 
       7.2645486231084375e-06, 
       7.0322620696110055e-06, 
       6.2036627329208141e-06, 
       8.1272754545177550e-06, 
       6.8028685586738674e-06, 
       6.3498724064878148e-06, 
       7.3787077898494652e-06, 
       6.3723951624183599e-06, 
       6.8025496843978953e-06, 
       7.2119864764766553e-06, 
       7.3143418473540846e-06, 
       6.4179889974482465e-06, 
       6.1835817505237845e-06, 
       8.6336856396131326e-06, 
       7.6405947450649844e-06, 
       8.7251851985558355e-06, 
       8.6241585815222008e-06, 
       6.8835241358467028e-06, 
       7.2576878081917892e-06, 
       6.4854865965629602e-06, 
       8.5181092998597026e-06, 
       8.7227818054858713e-06, 
       6.7431547226799023e-06, 
       6.2023087452962673e-06, 
       6.2190136898280959e-06, 
       8.0517529083978908e-06, 
       7.7314946770714672e-06, 
       7.2185100316606121e-06, 
       6.3029787688162320e-06, 
       7.4929647548053749e-06, 
       7.3862276380618472e-06, 
       7.1929809476813677e-06, 
       7.4210924521698175e-06, 
       6.9845835557839355e-06, 
       7.2339623599891363e-06, 
       7.2980766945396167e-06, 
       8.7026353556718936e-06, 
       6.4647316007578916e-06, 
       7.6734902627708431e-06, 
       8.3065186011947007e-06, 
       8.2566268674679823e-06, 
       7.2494077172805987e-06, 
       7.2357705684560732e-06, 
       7.9992740120157997e-06, 
       6.5257031967176887e-06, 
       8.0672066659405771e-06, 
       6.2988898677024704e-06, 
       6.2984922292067476e-06, 
       7.3162518795899722e-06, 
       7.4335463672576897e-06, 
       6.6686373188145086e-06, 
       6.9745201458353265e-06, 
       8.3800912899299453e-06, 
       6.1472356474906756e-06, 
       6.1472356474906756e-06, 
       7.1565923928930791e-06, 
       7.2037210253070906e-06, 
       6.2465978486557630e-06, 
       8.3845452518161466e-06, 
       6.1915228536191935e-06, 
       8.6711183139505477e-06, 
       6.2369806080215025e-06, 
       8.6578060372424294e-06, 
       6.6783136515748090e-06, 
       7.4141226085344546e-06, 
       8.5216927834975987e-06, 
       7.8268616905179159e-06, 
       7.6463396581371580e-06, 
       8.4123595512830509e-06, 
       6.1638949566598170e-06, 
       6.1990793319867436e-06, 
       6.3181482506115036e-06, 
       6.9939253436335080e-06, 
       6.5324409026322684e-06, 
       6.5619591124630627e-06, 
       6.1524682206353792e-06, 
       7.0905174121898160e-06, 
       7.1288151843027341e-06, 
       6.1446167360678263e-06, 
       6.7301497602913626e-06, 
       6.5560817518133754e-06, 
       7.4271775398430279e-06, 
       6.4636228271698370e-06, 
       8.3106498673420008e-06, 
       7.1778968243807378e-06, 
       6.1822753990940506e-06, 
       8.2189031357146174e-06, 
       6.2512574404560243e-06, 
       7.0812799642936359e-06, 
       8.3412112430763813e-06, 
       8.4041652315430232e-06, 
       7.1813933911322022e-06, 
       7.6650721813987826e-06, 
       7.0604328833151582e-06, 
       8.3189547259836080e-06, 
       6.2751464385722669e-06, 
       6.0934149408327473e-06, 
       8.5986050107562305e-06, 
       8.3887845878644261e-06, 
       8.2418809165492090e-06, 
       6.9625758579767307e-06, 
       7.4353825551699999e-06, 
       7.9585870182116215e-06, 
       7.9585870182116215e-06, 
       8.4369630626473101e-06, 
       6.1734556018040296e-06, 
       7.5056861541878824e-06, 
       6.0783668890218329e-06, 
       7.9509980734048850e-06, 
       7.2901613995684128e-06, 
       6.4270647401612042e-06, 
       6.3049974903736912e-06, 
       8.1674320639395898e-06, 
       8.1329460571515801e-06, 
       8.4914168310770236e-06, 
       6.0825632765300573e-06, 
       6.4446768907984583e-06, 
       8.2424512034177528e-06, 
       6.0656163413195227e-06, 
       7.8261093991125811e-06, 
       6.1731412291248245e-06, 
       6.0660816018400033e-06, 
       6.0940640239179070e-06, 
       6.2433837713859445e-06, 
       6.1375235328924300e-06, 
       6.6776705464370996e-06, 
       7.6128317508891224e-06, 
       6.2056603881787041e-06, 
       6.3160420616000400e-06, 
       6.5742987544099159e-06, 
       7.9092953110400073e-06, 
       8.5284909185904371e-06, 
       6.6184495095073507e-06, 
       6.0576522769766956e-06, 
       7.4315152197510592e-06, 
       8.4779703869546865e-06, 
       6.1128876309425450e-06, 
       6.3703879958432397e-06, 
       7.7811102823324509e-06, 
       7.8750202562067952e-06, 
       7.2238162828457384e-06, 
       6.8948415927883536e-06, 
       8.4819216693938803e-06, 
       6.7026834853040547e-06, 
       6.1836968654195716e-06, 
       7.2262296594756434e-06, 
       7.7562435236739028e-06, 
       7.0713108416821150e-06, 
       8.4945452015843609e-06, 
       6.2601246886244100e-06, 
       6.2601246886244100e-06, 
       6.1746067175039328e-06, 
       6.1698152756815594e-06, 
       8.2583553190279654e-06, 
       6.1193385783123349e-06, 
       7.0005149266175923e-06, 
       6.4476117150199198e-06, 
       7.5418946531039307e-06, 
       8.1958285386800848e-06, 
       6.2252740761213526e-06, 
       7.0374422054511995e-06, 
       7.1836958738174674e-06, 
       6.9547061605343956e-06, 
       6.8646116799650280e-06, 
       7.0234915560450410e-06, 
       6.8579550878500184e-06, 
       8.0088908788046566e-06, 
       6.3717790147651571e-06, 
       8.0602157610638725e-06, 
       8.0602157610638725e-06, 
       8.2788596034278513e-06, 
       6.0005315897409951e-06, 
       8.3971474609980947e-06, 
       6.7370093815358890e-06, 
       6.1833571940134427e-06, 
       7.2747414815575755e-06, 
       7.2747414815575755e-06, 
       7.9913846801651110e-06, 
       6.3526933546381042e-06, 
       7.0306330494084037e-06, 
       7.0083104475583571e-06, 
       8.2834279063047264e-06, 
       8.4292375148169116e-06, 
       5.9918877153763712e-06, 
       6.2461249565140326e-06, 
       7.8524633572922532e-06, 
       8.3925346810958888e-06, 
       6.4081097893016474e-06, 
       7.8489647709845187e-06, 
       6.6856768207824598e-06, 
       6.3866921119764571e-06, 
       5.9892405905555438e-06, 
       6.0701932028368224e-06, 
       6.9712321049450497e-06, 
       6.1109418140015768e-06, 
       6.6012669730450818e-06, 
       6.0506203602609542e-06, 
       6.6736511403254016e-06, 
       8.0494660034072037e-06, 
       6.1205292162906291e-06, 
       8.0978505534266053e-06, 
       7.3812817444794255e-06, 
       6.4110196892760487e-06, 
       6.4110196892760487e-06, 
       6.7490130214699421e-06, 
       5.9712615862261181e-06, 
       6.4292076745542406e-06, 
       6.3130719625325062e-06, 
       6.1295861106979835e-06, 
       6.2637025182357376e-06, 
       6.3972976294907306e-06, 
       6.8813159639501813e-06, 
       7.5723629563698862e-06, 
       6.8625717617732620e-06, 
       8.1284194695849764e-06, 
       6.6479316370298611e-06, 
       6.2263761972644871e-06, 
       6.9708152592193059e-06, 
       6.7983349303912446e-06, 
       6.8703284751379293e-06, 
       6.0976748192703715e-06, 
       6.3608475042346744e-06, 
       7.8743041072102623e-06, 
       7.0345031277281029e-06, 
       6.7810659927237325e-06, 
       6.9347736888875799e-06, 
       6.7583587870433055e-06, 
       6.6319777344867035e-06, 
       6.5406443811060676e-06, 
       5.9169725475530489e-06, 
       7.0424778804689125e-06, 
       6.9260243933788475e-06, 
       6.9661061540103577e-06, 
       6.9961003659945388e-06, 
       6.1926937743323093e-06, 
       6.4429823386278980e-06, 
       6.1368810661376337e-06, 
       6.9216023301264990e-06, 
       6.0459918914417011e-06, 
       8.3242173023020648e-06, 
       6.0888987622463703e-06, 
       6.1310715485463229e-06, 
       6.1310715485463229e-06, 
       6.8223957501686080e-06, 
       6.8780900439435869e-06, 
       6.9122752959014334e-06, 
       7.7558589804353135e-06, 
       7.7558589804353135e-06, 
       6.8121686787746056e-06, 
       8.2394357110695498e-06, 
       6.0103545451384626e-06, 
       6.9112676985500028e-06, 
       6.8490000809427573e-06, 
       5.9478858413916054e-06, 
       6.6918009694643482e-06, 
       8.3067856191332953e-06, 
       6.5134515604246363e-06, 
       8.0053412612514628e-06, 
       7.6486185600385615e-06, 
       8.2175187069414970e-06, 
       5.9804848335056000e-06, 
       8.2134113302511345e-06, 
       6.9245866844205943e-06, 
       6.3290084134759874e-06, 
       6.3290084134759874e-06, 
       7.6721996046676507e-06, 
       7.6721996046676507e-06, 
       5.8648483127511116e-06, 
       5.9458163922010010e-06, 
       6.3107790695953235e-06, 
       5.9068665958937301e-06, 
       6.9935313475040688e-06, 
       6.0117711779888827e-06, 
       5.8580256296403368e-06, 
       8.2464337603668212e-06, 
       5.9000330282288397e-06, 
       6.7239790210275806e-06, 
       5.8647339778188461e-06, 
       6.3518259329211351e-06, 
       8.2639639079514514e-06, 
       6.1648806215504545e-06, 
       6.3958209581944024e-06, 
       6.1554199347364266e-06, 
       6.3267592137151926e-06, 
       6.3001486701815570e-06, 
       5.8388459020743659e-06, 
       6.2813181533750810e-06, 
       6.6288000738691927e-06, 
       7.8375324620365539e-06, 
       7.0750487145454557e-06, 
       6.9593370203302197e-06, 
       5.9700674882967042e-06, 
       5.8805998299486068e-06, 
       6.0180050424936755e-06, 
       5.8878890538961769e-06, 
       6.4824211789942815e-06, 
       6.0765911321755052e-06, 
       5.9967601821535024e-06, 
       7.0520722870666350e-06, 
       6.3176149866831145e-06, 
       7.3748711806243560e-06, 
       5.8189616333131275e-06, 
       7.6014493443479603e-06, 
       5.8233736629191989e-06, 
       6.8373681992870242e-06, 
       6.8893867413851278e-06, 
       5.9318576361326868e-06, 
       7.3134476890165212e-06, 
       8.1555369446671217e-06, 
       7.0122464557963211e-06, 
       6.3881590923954351e-06, 
       5.8398447070955233e-06, 
       6.3207942216425624e-06, 
       5.8332751990217967e-06, 
       8.1928479770367015e-06, 
       5.8770604104701679e-06, 
       8.0645650882193115e-06, 
       6.3034333371821745e-06, 
       5.8791713453888670e-06, 
       7.4084705330522823e-06, 
       8.0896799952733750e-06, 
       7.4120442912588264e-06, 
       7.3094391173652882e-06, 
       5.7979373306140173e-06, 
       6.7079521650846923e-06, 
       6.4037527534294436e-06, 
       6.1053186671584738e-06, 
       7.9308164370539397e-06, 
       6.1621818815777966e-06, 
       6.1566434379916117e-06, 
       6.7845263322245182e-06, 
       7.7980221149136742e-06, 
       7.5501020554867114e-06, 
       8.1512373358501598e-06, 
       5.8407734058995067e-06, 
       5.8400656589179236e-06, 
       6.4927170252261992e-06, 
       5.8322236659581226e-06, 
       5.9449373704409834e-06, 
       7.5999379411496277e-06, 
       7.0397198435935514e-06, 
       6.7823182210344033e-06, 
       6.0874095657918735e-06, 
       6.2049603864947197e-06, 
       5.9302654726162630e-06, 
       7.5961201142158137e-06, 
       5.8083048572853333e-06, 
       6.1532008705229079e-06, 
       7.6981518595158221e-06, 
       8.0389131218149023e-06, 
       6.2072050912998679e-06, 
       6.7747902997990278e-06, 
       6.7733298474800394e-06, 
       6.2768221739696796e-06, 
       6.6769342024508076e-06, 
       6.0477432292033850e-06, 
       5.7714161697798590e-06, 
       7.5284719740124397e-06, 
       6.5316724809324856e-06, 
       7.9623069978104681e-06, 
       5.8755946358505967e-06, 
       5.9163091637401178e-06, 
       6.0512281175250681e-06, 
       8.1017484342337481e-06, 
       6.0119256384045336e-06, 
       7.9561526216720767e-06, 
       8.0685433275937159e-06, 
       6.8420766310820408e-06, 
       8.0716495037554573e-06, 
       5.7289473493859735e-06, 
       6.0573140302087647e-06, 
       5.7395235886413482e-06, 
       6.2484738484557058e-06, 
       7.7972646991467567e-06, 
       8.0811787268130118e-06, 
       7.3988564796031372e-06, 
       5.9469531554759990e-06, 
       6.6506772899210516e-06, 
       6.7373197816786533e-06, 
       6.3679053471746231e-06, 
       5.7281206425357208e-06, 
       6.6157581857603464e-06, 
       7.1895041992855789e-06, 
       7.7175147111385762e-06, 
       7.8335209297753050e-06, 
       6.4474738351189731e-06, 
       6.1473590376885219e-06, 
       6.4462811753832024e-06, 
       6.9315983704938046e-06, 
       6.5934960330411564e-06, 
       7.6909102929830486e-06, 
       7.7382084477576520e-06, 
       7.8958765981027960e-06, 
       6.7010444446199084e-06, 
       7.0363867459435089e-06, 
       7.9488449063163525e-06, 
       6.1513222449936413e-06, 
       5.7514353877765550e-06, 
       5.7514353877765550e-06, 
       5.9715857229413774e-06, 
       6.7019987093615118e-06, 
       7.8075888571475219e-06, 
       7.3330234284166976e-06, 
       7.3330234284166976e-06, 
       5.9194422721262626e-06, 
       5.9194422721262626e-06, 
       6.9360382308100745e-06, 
       8.0270695148698613e-06, 
       6.2752805484684084e-06, 
       6.5215071537263699e-06, 
       6.0057877098106507e-06, 
       7.6706556038852359e-06, 
       7.3077098656291973e-06, 
       5.8229767964387140e-06, 
       7.0372824891309952e-06, 
       7.0372824891309952e-06, 
       6.0839371205176060e-06, 
       7.9066086530323954e-06, 
       5.6856416551133155e-06, 
       6.9353637843832049e-06, 
       6.5572033527013221e-06, 
       6.4514001338088802e-06, 
       6.6767303036216666e-06, 
       6.1175214978259281e-06, 
       6.1175214978259281e-06, 
       6.6575831375598063e-06, 
       8.0110207126909263e-06, 
       5.9099378205266888e-06, 
       5.9098300791842611e-06, 
       6.3701150726510827e-06, 
       7.9946757366469358e-06, 
       5.7893825844294062e-06, 
       7.0296094625248600e-06, 
       7.7028325447432575e-06, 
       6.5241433513711065e-06, 
       6.6071763689915883e-06, 
       7.6060322278667517e-06, 
       6.1477708627416665e-06, 
       6.0542347017678760e-06, 
       6.4735352353305927e-06, 
       6.0946528049094158e-06, 
       5.6647376191841529e-06, 
       7.3042566503805005e-06, 
       6.7143271916453903e-06, 
       6.0139650461696895e-06, 
       5.9842169302488418e-06, 
       5.9842169302488418e-06, 
       6.4299785419398079e-06, 
       6.2317807024477101e-06, 
       6.2262177149300052e-06, 
       5.7864124709614297e-06, 
       6.0301924055209276e-06, 
       6.4609454024067838e-06, 
       7.6969107109906059e-06, 
       7.9219023381550292e-06, 
       7.5863884577044585e-06, 
       7.2666436111268533e-06, 
       6.0851308685511238e-06, 
       7.7478829669651739e-06, 
       7.2488045749190594e-06, 
       6.6338462833589685e-06, 
       5.8919124089728720e-06, 
       6.2678476077611997e-06, 
       7.5147541677235143e-06, 
       7.3874730463023694e-06, 
       7.3874730463023694e-06, 
       6.7227909984932959e-06, 
       6.5866295301633824e-06, 
       5.6653002417601156e-06, 
       5.8302446078644253e-06, 
       5.8717080253342111e-06, 
       6.9437227914826160e-06, 
       7.9569976055179080e-06, 
       7.8498790395516665e-06, 
       7.1313566831986382e-06, 
       6.1332973158058391e-06, 
       6.8225786586583919e-06, 
       7.3384879520962539e-06, 
       7.3351227683695434e-06, 
       6.4459088014364410e-06, 
       7.6360177266591665e-06, 
       6.6049477963966903e-06, 
       7.1570096894811623e-06, 
       7.1841774515509554e-06, 
       6.0723232771644539e-06, 
       6.0723232771644539e-06, 
       5.9321793651896593e-06, 
       6.8220138345120145e-06, 
       6.1917424327761135e-06, 
       7.3839755161443332e-06, 
       5.6133427323961324e-06, 
       6.7229970697189676e-06, 
       6.2936294218716995e-06, 
       5.8377923414176172e-06, 
       6.3092551123587698e-06, 
       6.3693271990030199e-06, 
       6.3663575982817277e-06, 
       5.9243213065365439e-06, 
       5.6068254434507447e-06, 
       6.5689183661893125e-06, 
       6.5657493265859072e-06, 
       5.6993735776600820e-06, 
       6.4776597852050243e-06, 
       6.4776597852050243e-06, 
       5.7257643624273743e-06, 
       6.3593939696084577e-06, 
       5.6460360586571671e-06, 
       6.6770364984684734e-06, 
       5.8506440103248970e-06, 
       7.7184389661012812e-06, 
       5.9421062779038503e-06, 
       5.9421062779038503e-06, 
       6.8546566850660382e-06, 
       5.6723282087207926e-06, 
       6.5818602233431880e-06, 
       7.8869489038372725e-06, 
       7.1811186110926320e-06, 
       7.4372955415553001e-06, 
       7.5977223209783669e-06, 
       5.6928202452816236e-06, 
       7.8850645551993581e-06, 
       5.6625581909836898e-06, 
       5.8306605132059918e-06, 
       7.8789798532756016e-06, 
       7.7566069046830254e-06, 
       7.4910838759822679e-06, 
       5.5828519665234011e-06, 
       6.9278586241127189e-06, 
       7.6938811413423828e-06, 
       6.4112766801815158e-06, 
       7.0884042681500066e-06, 
       5.8806492669706556e-06, 
       7.1714331081377097e-06, 
       5.5985294380252112e-06, 
       6.3366455192755263e-06, 
       5.8778240938400084e-06, 
       7.4062681867521400e-06, 
       5.8641135301341802e-06, 
       6.5375684984458203e-06, 
       5.9402176325205669e-06, 
       6.6468985395814031e-06, 
       5.5668211270054751e-06, 
       5.8690810990016784e-06, 
       7.4984984025530162e-06, 
       5.6351178191807373e-06, 
       6.0133744164299965e-06, 
       7.6969290813172705e-06, 
       5.8789567852999778e-06, 
       6.4872040006237233e-06, 
       7.4227540129962615e-06, 
       6.3602999352574965e-06, 
       5.6523251102634305e-06, 
       6.3577452605491751e-06, 
       5.5371040560367191e-06, 
       5.5705088695345787e-06, 
       5.5361041581552081e-06, 
       7.2554233267141175e-06, 
       7.4373738922535855e-06, 
       7.8088952722643637e-06, 
       7.2830173818326053e-06, 
       6.9568999828649608e-06, 
       5.9140118219653703e-06, 
       5.8240186323996355e-06, 
       6.0970259878867018e-06, 
       6.4092862479087905e-06, 
       7.7784793313186358e-06, 
       7.7619573181044552e-06, 
       5.8730017161119043e-06, 
       5.8401850622854228e-06, 
       5.8235763918139724e-06, 
       5.7669563939944181e-06, 
       6.6022766558331848e-06, 
       7.1321619948413291e-06, 
       6.0368389773663254e-06, 
       6.6673008964104412e-06, 
       6.5493117199386200e-06, 
       5.5716230856974613e-06, 
       7.6799908115435664e-06, 
       7.7290513817957976e-06, 
       5.6099146253383544e-06, 
       5.5485933268136754e-06, 
       7.4525522040293373e-06, 
       6.7449051623924324e-06, 
       6.4953756911488208e-06, 
       7.2346209776431906e-06, 
       7.1292065314911228e-06, 
       5.9273508629383414e-06, 
       5.9273508629383414e-06, 
       5.7839268094209508e-06, 
       5.5680000741447003e-06, 
       7.6492018435427706e-06, 
       6.2844759672116925e-06, 
       7.1165594983431288e-06, 
       7.6916502679306475e-06, 
       6.8815559859282941e-06, 
       6.8815559859282941e-06, 
       5.6108584914714207e-06, 
       7.2824102475582366e-06, 
       6.6606419430841566e-06, 
       5.4987685858258669e-06, 
       6.8216817942333458e-06, 
       7.7667959472634922e-06, 
       7.1488937072269070e-06, 
       5.6116216692315893e-06, 
       6.3315460759110286e-06, 
       5.8270755096995049e-06, 
       7.4499916002647527e-06, 
       5.5211507504764959e-06, 
       5.5990383611166498e-06, 
       5.7293898980322094e-06, 
       6.0248802367462348e-06, 
       5.4987039912939833e-06, 
       5.5202340728467789e-06, 
       7.4301426307197604e-06, 
       5.5904900780697039e-06, 
       5.5042720306013878e-06, 
       5.5141979517683643e-06, 
       5.5269235920617102e-06, 
       6.0214572820884799e-06, 
       6.4865826138278284e-06, 
       7.6216301996646295e-06, 
       6.0358238377978600e-06, 
       7.4894777729884937e-06, 
       5.7974342735583936e-06, 
       7.5712199588158649e-06, 
       7.3206924896966611e-06, 
       7.1087810180296741e-06, 
       5.4698617244385636e-06, 
       7.4780109657501206e-06, 
       6.6320813301216291e-06, 
       6.1979183696187411e-06, 
       5.4734043257187459e-06, 
       6.7264912384989215e-06, 
       5.5360451687354046e-06, 
       6.2969860873987095e-06, 
       6.9740899334537264e-06, 
       6.6151806310918619e-06, 
       5.8341809562237005e-06, 
       6.3867657018405756e-06, 
       5.8185899516587961e-06, 
       7.4039834423309210e-06, 
       7.7013523877199202e-06, 
       6.0183424099203077e-06, 
       5.4542435306703439e-06, 
       7.3305289119770143e-06, 
       5.5811844266112707e-06, 
       5.6434823862782500e-06, 
       6.4929668990443809e-06, 
       5.4372467239796982e-06, 
       5.9726579061218906e-06, 
       5.8312507766350068e-06, 
       7.2236510002249492e-06, 
       7.2236510002249492e-06, 
       5.7141670995395799e-06, 
       5.7141670995395799e-06, 
       6.5034786740290710e-06, 
       7.0126583371818916e-06, 
       5.9660078903655377e-06, 
       7.1068545367516976e-06, 
       7.6602298658043909e-06, 
       5.4354355768316042e-06, 
       5.4978848314097929e-06, 
       6.5490024985775831e-06, 
       7.1874704758251506e-06, 
       6.6328622534756901e-06, 
       6.3053843994556907e-06, 
       5.5735529363233826e-06, 
       7.5042120962215634e-06, 
       6.2332311584478082e-06, 
       5.4412318464924084e-06, 
       6.6556685438093709e-06, 
       7.4367528489456366e-06, 
       6.3334701581461318e-06, 
       5.6790505656733595e-06, 
       7.1737151705823430e-06, 
       5.8435749372809484e-06, 
       5.8195704108619180e-06, 
       5.6176878292815404e-06, 
       5.4850296311379635e-06, 
       5.8287010682684441e-06, 
       5.8287010682684441e-06, 
       5.4738037276891584e-06, 
       5.6055901277558427e-06, 
       5.4675266565406102e-06, 
       5.6953850928486172e-06, 
       6.1663904937912220e-06, 
       5.4560019388275427e-06, 
       6.2268138566635983e-06, 
       6.7962333816708824e-06, 
       5.6546447487018889e-06, 
       6.9389468873509928e-06, 
       5.4577517309906123e-06, 
       6.8453771303658551e-06, 
       5.3968428111083162e-06, 
       6.1411821759672176e-06, 
       5.4050708321308964e-06, 
       5.5910215998506130e-06, 
       5.6206609822563032e-06, 
       5.4178990030535030e-06, 
       5.9608602327330703e-06, 
       7.4459940824368707e-06, 
       6.3127771923031804e-06, 
       6.8012510034988692e-06, 
       7.2413663755627394e-06, 
       6.3218599194809736e-06, 
       7.3856445382819625e-06, 
       6.4747231373514488e-06, 
       7.2107875488546526e-06, 
       5.3964591284108049e-06, 
       5.6219306846195998e-06, 
       5.6649535137684141e-06, 
       5.5133894026622594e-06, 
       7.5416625089222462e-06, 
       7.3079470537212988e-06, 
       6.3446360574820194e-06, 
       5.3656301911468203e-06, 
       5.4710651506518317e-06, 
       5.4710651506518317e-06, 
       5.4840420268021623e-06, 
       6.2748978648343780e-06, 
       7.2901978480760592e-06, 
       5.3927708541901849e-06, 
       7.1159973601950606e-06, 
       5.7153026356728936e-06, 
       5.6090909472119815e-06, 
       5.5785916958863474e-06, 
       7.5045253671804486e-06, 
       6.2693712495954386e-06, 
       5.4492635435953270e-06, 
       5.4492635435953270e-06, 
       6.2328120330743371e-06, 
       5.5759594053786031e-06, 
       7.5576678516437239e-06, 
       5.5673370723316717e-06, 
       5.7580359593978296e-06, 
       5.4856844222718987e-06, 
       5.7382760900903607e-06, 
       6.6524389859029789e-06, 
       5.3574339895901584e-06, 
       5.6719069383675439e-06, 
       6.2730897331371592e-06, 
       5.3458904277265951e-06, 
       5.8802830805125336e-06, 
       5.5713564450323236e-06, 
       5.6306620204435878e-06, 
       5.7053699802332208e-06, 
       5.6267657299126743e-06, 
       5.6302117785843122e-06, 
       6.6411939948092841e-06, 
       5.5580604017678049e-06, 
       5.5919946275602895e-06, 
       6.3789967109831761e-06, 
       5.6988958066157142e-06, 
       7.2846802435576741e-06, 
       5.4636368134338368e-06, 
       5.7673413517829630e-06, 
       5.6188064782030739e-06, 
       5.6188064782030739e-06, 
       6.0196062491091521e-06, 
       7.0309581977162748e-06, 
       5.6129535813662744e-06, 
       5.4967947110936696e-06, 
       6.7630353971120394e-06, 
       6.0463083708660807e-06, 
       5.4349722481497518e-06, 
       5.3986288969722914e-06, 
       5.6071254763456087e-06, 
       5.6071254763456087e-06, 
       6.2383096400005441e-06, 
       6.0445671933963983e-06, 
       5.3256019043161995e-06, 
       6.2497837976480703e-06, 
       5.7948908437514988e-06, 
       5.7948908437514988e-06, 
       6.7156070078061986e-06, 
       6.7156070078061986e-06, 
       5.6418828638371840e-06, 
       7.2403598629070618e-06, 
       5.8417567807660990e-06, 
       6.2820589589913953e-06, 
       6.1233772335214195e-06, 
       6.6065835442861159e-06, 
       6.3128895552476755e-06, 
       5.4834543918027496e-06, 
       6.3648115902874342e-06, 
       5.4643121356496141e-06, 
       5.7249126991342857e-06, 
       5.4399498756473093e-06, 
       5.2999463145210319e-06, 
       6.2283724913761078e-06, 
       6.4658032042281087e-06, 
       7.1768090046384347e-06, 
       5.5894564580435195e-06, 
       6.8840015723682449e-06, 
       5.9261397066392665e-06, 
       6.9779388588296297e-06, 
       5.3191498843007388e-06, 
       5.3191498843007388e-06, 
       5.9874063918786388e-06, 
       7.2347945395158138e-06, 
       6.5705133871353495e-06, 
       6.3588287186123169e-06, 
       5.4733277142645209e-06, 
       5.4816491031014475e-06, 
       5.4852136140988793e-06, 
       5.3009848819773374e-06, 
       6.3823084746243504e-06, 
       5.2855854277772946e-06, 
       5.3234782223854016e-06, 
       7.3910706731593719e-06, 
       6.1971752137126430e-06, 
       6.1685728059759624e-06, 
       6.1963396464175342e-06, 
       6.0991906015090449e-06, 
       5.4282886626571874e-06, 
       7.3076513564598581e-06, 
       6.2509626386402173e-06, 
       6.8161365134358455e-06, 
       7.3284180558334787e-06, 
       6.9616228786228841e-06, 
       5.2882033478535780e-06, 
       5.8265221983726228e-06, 
       6.2459638192776110e-06, 
       7.3586269895958738e-06, 
       5.8240311390494092e-06, 
       5.2633040321757291e-06, 
       5.2623643485908661e-06, 
       6.1715095997981540e-06, 
       6.1711126851757028e-06, 
       7.3739283032986618e-06, 
       7.1182205718370804e-06, 
       5.5409420826400983e-06, 
       5.5409420826400983e-06, 
       5.2530462099487961e-06, 
       5.8151331651033968e-06, 
       6.1996736076808754e-06, 
       5.8136914912325525e-06, 
       5.7945581272735897e-06, 
       5.6720608441733446e-06, 
       5.6567068798802168e-06, 
       5.6567068798802168e-06, 
       5.3119579867566664e-06, 
       6.1446167360689486e-06, 
       6.2413637792225165e-06, 
       6.3736368740059631e-06, 
       5.6866138459997124e-06, 
       5.2597505066372498e-06, 
       5.5351059762663311e-06, 
       5.2449725505910261e-06, 
       5.8019847348223856e-06, 
       5.3881324039450042e-06, 
       6.1743409956739766e-06, 
       7.0960200525735063e-06, 
       6.1878075213491113e-06, 
       5.6639276859468927e-06, 
       6.7453826502003142e-06, 
       5.2801946144824463e-06, 
       5.6419414277686788e-06, 
       5.4953495705117457e-06, 
       5.3470216952694643e-06, 
       5.3316654200498077e-06, 
       5.3316654200498077e-06, 
       6.1592265135701084e-06, 
       6.3212243141321063e-06, 
       5.9038794404820273e-06, 
       7.1796222453561282e-06, 
       5.3586947602444381e-06, 
       6.1303323342424041e-06, 
       5.2142611407450598e-06, 
       5.7984582423616250e-06, 
       6.6732112277201568e-06, 
       6.4845793806532715e-06, 
       5.3803109637429919e-06, 
       6.0775451105302589e-06, 
       5.2869273923983769e-06, 
       6.1960396089228197e-06, 
       5.7942254924912351e-06, 
       5.2961199851837655e-06, 
       5.2099046119477297e-06, 
       5.2099046119477297e-06, 
       5.6195241637639676e-06, 
       6.5366961733991931e-06, 
       5.9939304118486463e-06, 
       5.5883114042347507e-06, 
       5.5604161751531724e-06, 
       5.7123056861289184e-06, 
       7.3033461951278232e-06, 
       5.1922576373620115e-06, 
       5.2103437310014724e-06, 
       5.2853177283297270e-06, 
       7.1841291337644549e-06, 
       6.0994001401383625e-06, 
       7.1315920389691240e-06, 
       5.1952394044418878e-06, 
       5.1839625594298388e-06, 
       5.3984776986229278e-06, 
       5.9828411333733380e-06, 
       5.2542327582255395e-06, 
       6.9195065994403507e-06, 
       5.7406816037181469e-06, 
       5.1815702179236271e-06, 
       6.1717121505326583e-06, 
       7.2375563353334706e-06, 
       6.0959330530099474e-06, 
       5.2654385040045693e-06, 
       5.2742141602359554e-06, 
       5.5099875120466912e-06, 
       6.1380621913181044e-06, 
       6.3587559695713922e-06, 
       6.3500495306671978e-06, 
       6.5504362344820278e-06, 
       7.3085476534325835e-06, 
       6.9369761671272424e-06, 
       5.2083034429496105e-06, 
       6.0943673243299101e-06, 
       5.5452162003177375e-06, 
       7.1812823493272101e-06, 
       5.2070903653600036e-06, 
       5.5450326961784913e-06, 
       6.0298629082148001e-06, 
       6.0759416025091884e-06, 
       5.4376153940847482e-06, 
       6.2216373638310003e-06, 
       5.9527824870020783e-06, 
       7.2761201307871922e-06, 
       7.2536247179587770e-06, 
       5.6946663319159855e-06, 
       5.9161843364260264e-06, 
       5.6036960515758617e-06, 
       6.9060878839117946e-06, 
       7.0621578828587832e-06, 
       5.9150176619157414e-06, 
       5.8707747590809960e-06, 
       5.2936260448345143e-06, 
       5.7895872312718065e-06, 
       5.2293502442081073e-06, 
       7.0978011155268866e-06, 
       5.2140242422292580e-06, 
       5.6370831467498159e-06, 
       6.2434331450144373e-06, 
       5.6761121138607354e-06, 
       5.7938011047819470e-06, 
       6.9798429801371771e-06, 
       5.1376124617099431e-06, 
       5.6177931049612327e-06, 
       5.3853481303807970e-06, 
       5.6489731767423241e-06, 
       7.1787845043765107e-06, 
       5.1355079062054452e-06, 
       5.2680750918229941e-06, 
       5.1952897824955046e-06, 
       5.2177128000369585e-06, 
       6.8740213496231926e-06, 
       5.1350352771996024e-06, 
       6.0330876273482068e-06, 
       5.3674535970162317e-06, 
       6.1665294127595418e-06, 
       7.1646367260284260e-06, 
       5.5463496105284731e-06, 
       5.1156604485734757e-06, 
       6.4097739868846593e-06, 
       7.0001168880692733e-06, 
       5.3309827991589180e-06, 
       5.9370042008787239e-06, 
       7.1483257138584613e-06, 
       5.2241201855979548e-06, 
       7.1381170983540448e-06, 
       5.9854629512188670e-06, 
       6.1984606602516078e-06, 
       5.1697216816634835e-06, 
       6.0421869211088783e-06, 
       5.9467558687254566e-06, 
       5.1963011923969957e-06, 
       5.7955574635328155e-06, 
       5.2335008206552384e-06, 
       5.9222083023801718e-06, 
       5.9222083023801718e-06, 
       5.1345183928977579e-06, 
       5.1779226813815735e-06, 
       5.2809291593338167e-06, 
       5.2500313721013064e-06, 
       6.1213134347757487e-06, 
       7.1994616371103522e-06, 
       5.3870815770524110e-06, 
       6.7989978936838905e-06, 
       5.1062573826758132e-06, 
       5.1062573826758132e-06, 
       5.2093764944275705e-06, 
       5.9983728259356309e-06, 
       5.5057082333435576e-06, 
       5.2956046853318113e-06, 
       5.9999787111417625e-06, 
       5.9947276188692832e-06, 
       5.1797367038177675e-06, 
       5.5646532733676450e-06, 
       5.0970033874482051e-06, 
       5.8449895608308438e-06, 
       5.5128873345893522e-06, 
       5.3354432389580698e-06, 
       5.9794584177036992e-06, 
       5.5269001653082006e-06, 
       5.4301204974799552e-06, 
       6.5113828350671211e-06, 
       6.0579157490319682e-06, 
       7.1818094599419126e-06, 
       5.6448908607957191e-06, 
       5.2886239188818242e-06, 
       6.1822240881396097e-06, 
       6.5761609005233121e-06, 
       6.1295876217032792e-06, 
       7.0609465838038365e-06, 
       5.3295903793182630e-06, 
       6.2210612366522984e-06, 
       6.4318915984687154e-06, 
       7.1416731953154181e-06, 
       7.1367795218627426e-06, 
       7.1367795218627426e-06, 
       5.4167911684879703e-06, 
       5.0812188668055641e-06, 
       5.5722174034112043e-06, 
       5.0809930755755908e-06, 
       5.2396561704941294e-06, 
       6.3116745772156059e-06, 
       5.0886273372111876e-06, 
       5.1370691443946227e-06, 
       5.4453595210934112e-06, 
       5.3515178448995128e-06, 
       6.8853121788173181e-06, 
       5.9588431157775341e-06, 
       5.4551879361301095e-06, 
       6.0230421128834749e-06, 
       5.4190001622071678e-06, 
       6.1885102346695336e-06, 
       5.9500114608948415e-06, 
       6.1723099000514734e-06, 
       7.1326684221226177e-06, 
       7.1103993029403522e-06, 
       5.1194619670739964e-06, 
       5.7030143347136473e-06, 
       7.1129987148367658e-06, 
       6.4073357510512288e-06, 
       5.2481544397776283e-06, 
       5.8553983504745477e-06, 
       6.8160774596724606e-06, 
       7.1142287488917522e-06, 
       6.8750335736659288e-06, 
       5.1592520815993292e-06, 
       5.5857631097333356e-06, 
       5.5747027211627644e-06, 
       5.1541793113280835e-06, 
       5.1674512553663078e-06, 
       7.0837631504613443e-06, 
       5.9171008319952491e-06, 
       7.0848727141699949e-06, 
       5.7461365044403820e-06, 
       5.4731974440827196e-06, 
       5.9480663561064358e-06, 
       5.0905241699708477e-06, 
       5.4591811203725754e-06, 
       5.1214759364607740e-06, 
       6.8891917435359078e-06, 
       5.0947041703477098e-06, 
       5.2406346594873073e-06, 
       7.0923674204867306e-06, 
       6.5089817294301258e-06, 
       5.8096522733005506e-06, 
       6.8506476457098806e-06, 
       6.4531293182565535e-06, 
       5.0547738214144203e-06, 
       5.3905457215428139e-06, 
       5.1566242860057006e-06, 
       5.1221577350799645e-06, 
       6.9806046736001449e-06, 
       5.7716041108821939e-06, 
       6.9851767682472947e-06, 
       6.5648383967170602e-06, 
       7.0106066233526353e-06, 
       6.8060965397364521e-06, 
       5.0842766940022541e-06, 
       5.9537133428881028e-06, 
       6.9528503282128028e-06, 
       6.1506390515912310e-06, 
       6.3961767867015086e-06, 
       6.8486980863331425e-06, 
       5.3921273203882736e-06, 
       5.1444193299806858e-06, 
       6.6849750460592125e-06, 
       7.0068683119154729e-06, 
       5.2577032268142826e-06, 
       6.2249357085535062e-06, 
       5.0833168941234252e-06, 
       5.0085014117636185e-06, 
       5.9013590922268348e-06, 
       6.6198000279741446e-06, 
       5.1614666727166710e-06, 
       5.0106631202157810e-06, 
       5.8445371863336206e-06, 
       5.2330392318547598e-06, 
       5.3486881901950089e-06, 
       5.2915355720685562e-06, 
       6.3139887568711686e-06, 
       7.0022259858289227e-06, 
       5.0318981072577734e-06, 
       5.5261245698180585e-06, 
       5.2440811592908714e-06, 
       5.0269721974645215e-06, 
       6.3632394457536943e-06, 
       5.2397438910679939e-06, 
       5.0826630699598868e-06, 
       5.1208378584291366e-06, 
       5.0435708758945564e-06, 
       6.1047026104769068e-06, 
       5.4953131563619164e-06, 
       5.0064830931092391e-06, 
       6.9703899192539470e-06, 
       5.8104779663789147e-06, 
       5.7509491820541933e-06, 
       5.9919504635408999e-06, 
       6.3819638508089249e-06, 
       5.8826005274842328e-06, 
       6.3032715192769789e-06, 
       5.8832286313925278e-06, 
       6.6032982901123253e-06, 
       5.3371778107890469e-06, 
       5.3068475987420778e-06, 
       5.3100447903422174e-06, 
       6.4535617853097211e-06, 
       5.2555838842759139e-06, 
       6.9874361639068328e-06, 
       6.7201838066916409e-06, 
       6.0803771249479365e-06, 
       6.6615922658869639e-06, 
       5.8938156826297979e-06, 
       6.9714315271608122e-06, 
       5.8035111196522279e-06, 
       6.9083745673522458e-06, 
       5.9291885976173082e-06, 
       5.3725088472169186e-06, 
       5.3313664439602887e-06, 
       5.0465607183655812e-06, 
       6.6726946095327684e-06, 
       5.7871767749112381e-06, 
       5.0314130846051858e-06, 
       6.9268197169265925e-06, 
       5.6534952389373843e-06, 
       6.6557575679690944e-06, 
       5.7290491855400866e-06, 
       5.8828977537553643e-06, 
       6.5172483037598923e-06, 
       6.5686845820461960e-06, 
       5.1086268120768148e-06, 
       5.1086268120768148e-06, 
       6.7095936786334403e-06, 
       5.2694113996491418e-06, 
       5.8870372093967251e-06, 
       6.4988046288068995e-06, 
       5.0630430177468774e-06, 
       5.1995986081133840e-06, 
       6.8690895681881170e-06, 
       5.5937738269508193e-06, 
       5.6658307962802522e-06, 
       5.4965421396530949e-06, 
       5.0326691764202182e-06, 
       5.1869476181368466e-06, 
       5.4543895889618590e-06, 
       5.0823147383892382e-06, 
       6.5588743433599956e-06, 
       5.0701327663356127e-06, 
       5.0701327663356127e-06, 
       6.5258547298671280e-06, 
       5.4158018427215725e-06, 
       5.5151859597019291e-06, 
       6.9224298685875303e-06, 
       5.3315887446736730e-06, 
       5.2779231048842430e-06, 
       5.5393283092395559e-06, 
       5.1682324120205102e-06, 
       6.7004027236315605e-06, 
       5.5934016400940161e-06, 
       5.0884830288265021e-06, 
       5.7174565362622801e-06, 
       5.2598475833986776e-06, 
       5.2442553462784150e-06, 
       5.5421848688388462e-06, 
       6.8116396911470511e-06, 
       5.8826332724209201e-06, 
       5.6662567728503056e-06, 
       5.9902828602964845e-06, 
       5.2466019332563461e-06, 
       5.0570042134846688e-06, 
       5.7111689516064055e-06, 
       5.2101380513567342e-06, 
       5.5602047657187201e-06, 
       6.8742956284347077e-06, 
       5.6540495539382093e-06, 
       5.0460292258312460e-06, 
       5.1549116944486901e-06, 
       5.5775265455652450e-06, 
       5.1640475731820762e-06, 
       5.0197965666856556e-06, 
       5.1737491968416198e-06, 
       5.7018284607207900e-06, 
       5.0050621522429372e-06, 
       5.7449206046869171e-06, 
       5.3824847455501841e-06, 
       6.8041304210485603e-06, 
       5.2615249107631482e-06, 
       5.7208001534270193e-06, 
       5.0212954559275695e-06, 
       5.1607434354432520e-06, 
       5.4611633917527863e-06, 
       5.9602933657748334e-06, 
       5.6528955640162581e-06, 
       6.8550672890222821e-06, 
       5.6488054282400489e-06, 
       5.3912334193558601e-06, 
       5.1163811574024321e-06, 
       6.7983717580557833e-06, 
       5.1066358624911472e-06, 
       5.0836626150728225e-06, 
       5.2332037482620057e-06, 
       5.6305180201971218e-06, 
       5.2045884493554040e-06, 
       5.6833752711610136e-06, 
       5.2190897181656702e-06, 
       6.3370363119653598e-06, 
       5.8718657000322704e-06, 
       5.4737350134082627e-06, 
       5.8965344129845294e-06, 
       6.4288473987285681e-06, 
       5.0768917666474741e-06, 
       5.7189481189440884e-06, 
       5.5049766052956606e-06, 
       6.6361527963698073e-06, 
       6.8345628765601126e-06, 
       5.7737045442855177e-06, 
       6.5305944359090663e-06, 
       5.1493852082100133e-06, 
       5.3961424670948650e-06, 
       6.2129269227920473e-06, 
       6.6022330015227166e-06, 
       5.0760636462681580e-06, 
       5.8278508831585285e-06, 
       5.0218524394220919e-06, 
       6.0800577367124848e-06, 
       5.6672786189137055e-06, 
       6.7925566062889826e-06, 
       5.0242959108969123e-06, 
       5.2477587134107276e-06, 
       5.2641498657831820e-06, 
       5.4199274228467586e-06, 
       5.4199274228467586e-06, 
       5.5966656683064003e-06, 
       5.0689655704583934e-06, 
       5.1101297893866222e-06, 
       5.7199182075604264e-06, 
       5.2176755710980267e-06, 
       5.3206573454895828e-06, 
       5.6922236530038079e-06, 
       5.0141864109769346e-06, 
       6.6514175209917794e-06, 
       5.9996529757580357e-06, 
       6.5176158995257485e-06, 
       6.5317630734928191e-06, 
       6.7655290325922168e-06, 
       5.6761972123955106e-06, 
       5.0286035151825328e-06, 
       6.2658662024399494e-06, 
       5.6147863389075215e-06, 
       5.7173672777150223e-06, 
       5.5904677456826809e-06, 
       5.5831621281208631e-06, 
       5.5831621281208631e-06, 
       6.6378375664960152e-06, 
       5.1557302974680786e-06, 
       6.3964295219645038e-06, 
       6.6262593234978287e-06, 
       6.5795957726478845e-06, 
       4.7992435814737550e-06, 
       5.2107465524985579e-06, 
       5.2520007262578167e-06, 
       5.2520007262578167e-06, 
       5.3025824794307352e-06, 
       5.0954540557249336e-06, 
       6.5281574882995468e-06, 
       6.6795097007978265e-06, 
       6.3784640823086232e-06, 
       5.4976469302318790e-06, 
       5.1452517256750275e-06, 
       5.6074495087505588e-06, 
       5.8963969250732190e-06, 
       5.4542435306721075e-06, 
       5.3913355520480451e-06, 
       6.0594296119022806e-06, 
       5.5451621006613134e-06, 
       5.4873575611822610e-06, 
       6.2507834835932795e-06, 
       6.7228326983438489e-06, 
       5.7812268800423634e-06, 
       5.1534234143823262e-06, 
       6.4625368067707079e-06, 
       5.0027598680037044e-06, 
       6.3045354836100039e-06, 
       6.0560481901183896e-06, 
       6.0357014312552698e-06, 
       5.7792162728220610e-06, 
       5.6826491846723122e-06, 
       6.0302854355404565e-06, 
       5.0210275337924473e-06, 
       6.6915531796553851e-06, 
       5.8450356906509580e-06, 
       5.3600249703567839e-06, 
       5.4185496293013407e-06, 
       5.4895535867999497e-06, 
       6.0222640448217595e-06, 
       5.2745603253994991e-06, 
       6.5324409026307853e-06, 
       5.1630943567168660e-06, 
       5.4936450107345400e-06, 
       5.5807566772513599e-06, 
       6.4179001922208611e-06, 
       6.6421778162269562e-06, 
       5.2990588006790259e-06, 
       5.8965122723314421e-06, 
       5.2612428625586008e-06, 
       6.6555890823717292e-06, 
       5.5029842709465513e-06, 
       6.6377495152204413e-06, 
       5.1935351190436556e-06, 
       6.5870771052481835e-06, 
       5.9855054114687412e-06, 
       5.9855054114687412e-06, 
       5.1598176631595898e-06, 
       5.6201479494211058e-06, 
       5.5064073446163104e-06, 
       5.6143782395154250e-06, 
       6.3054898553332670e-06, 
       6.4180314564261719e-06, 
       5.5874585180665623e-06, 
       5.3644805486260753e-06, 
       5.5279324878914114e-06, 
       5.5719098697993120e-06, 
       6.2405685159652341e-06, 
       6.5865834752201634e-06, 
       5.4625522499349752e-06, 
       6.5246776380086168e-06, 
       6.5545309827368852e-06, 
       6.3485976630824233e-06, 
       6.2999163408555939e-06, 
       5.4902305215171285e-06, 
       6.4717968138826564e-06, 
       6.4717968138826564e-06, 
       5.2730927349920946e-06, 
       5.9187959838327633e-06, 
       5.9936842264385107e-06, 
       6.1215251667772025e-06, 
       5.5381073499036053e-06, 
       5.4828488921237593e-06, 
       5.4828488921237593e-06, 
       5.9863829980264074e-06, 
       5.2180912813171309e-06, 
       6.5241052992798866e-06, 
       5.4803733004894887e-06, 
       5.4462180943788180e-06, 
       5.5151481148291688e-06, 
       5.9663070643777142e-06, 
       5.7995837240048130e-06, 
       6.5639073451497502e-06, 
       5.4895228082230622e-06, 
       5.1754656925323139e-06, 
       6.2175972770242291e-06, 
       5.4715922215058293e-06, 
       6.3368153853075715e-06, 
       6.4284846485669782e-06, 
       5.6751283956075012e-06, 
       6.3948153522664443e-06, 
       5.3198910560943191e-06, 
       5.1664982288260777e-06, 
       4.7521991519815740e-06, 
       5.4893169089131109e-06, 
       6.3420379067258069e-06, 
       5.4494624428744065e-06, 
       6.0510447410791965e-06, 
       6.4091495271231018e-06, 
       6.3459107681675649e-06, 
       5.2095264923620327e-06, 
       5.8773718763879571e-06, 
       5.1778553585803506e-06, 
       6.2898322719490820e-06, 
       5.4270082791666147e-06, 
       5.2037289754794092e-06, 
       5.4168279835365792e-06, 
       5.9054988439086120e-06, 
       5.7428663536124496e-06, 
       5.9120408095561665e-06, 
       5.0218212520073213e-06, 
       5.0340836941962763e-06, 
       5.3072479568843563e-06, 
       5.7825726904972110e-06, 
       5.4268528063671242e-06, 
       5.0250997454352976e-06, 
       5.1176939841153357e-06, 
       6.1685598719457118e-06, 
       5.2896231274812721e-06, 
       5.0879620037536710e-06, 
       5.0034395699456692e-06, 
       5.3718490042720656e-06, 
       5.4186745523148162e-06, 
       5.5263220172550726e-06, 
       5.4626803850014608e-06, 
       5.5229254447647654e-06, 
       5.4001416351328001e-06, 
       6.2356049745909060e-06, 
       6.3388839870347919e-06, 
       4.6664212737616913e-06, 
       5.2536632370275706e-06, 
       5.3737484900741266e-06, 
       5.2169472815965601e-06, 
       5.6109682864543693e-06, 
       5.4101379481269568e-06, 
       5.8314861251021297e-06, 
       6.4737380295128058e-06, 
       5.0482104983694529e-06, 
       6.0939237863137028e-06, 
       6.3943979041996382e-06, 
       5.3316653083492868e-06, 
       5.2310059277683736e-06, 
       5.0587546690044736e-06, 
       5.0583775456638594e-06, 
       6.2341629298008892e-06, 
       5.8702649819192267e-06, 
       6.1534053114077470e-06, 
       6.1534053114077470e-06, 
       5.3871528531629385e-06, 
       5.9986524682632356e-06, 
       5.8531159010470763e-06, 
       5.8531159010470763e-06, 
       5.2804664671475744e-06, 
       5.2931526854921598e-06, 
       5.3767447969360239e-06, 
       6.1263820624324322e-06, 
       5.5746415204192447e-06, 
       6.2151697911937897e-06, 
       5.3537378056026867e-06, 
       6.2149863772245279e-06, 
       5.2725993944539913e-06, 
       5.3248524742817636e-06, 
       5.3391753195553864e-06, 
       6.2035846243055683e-06, 
       5.4763623584831728e-06, 
       6.0714972440823765e-06, 
       5.3746466992960332e-06, 
       5.1804352527809584e-06, 
       5.0870310103445580e-06, 
       5.2831805709472684e-06, 
       6.1291588560095151e-06, 
       6.3037809378175243e-06, 
       5.0783389516416873e-06, 
       5.3310616767475379e-06, 
       5.2770438798567332e-06, 
       5.3097475355513358e-06, 
       5.3097623642754956e-06, 
       5.4105514130946883e-06, 
       6.1267866011622665e-06, 
       5.9282669734497838e-06, 
       5.2420689542674930e-06, 
       5.2446812980520261e-06, 
       6.3170739927257570e-06, 
       5.1777041650908182e-06, 
       5.1776797897796081e-06, 
       5.5055257950663088e-06, 
       6.2242965256978093e-06, 
       5.4289534128697224e-06, 
       6.3518850474802251e-06, 
       5.2425727551385661e-06, 
       5.2354607754403214e-06, 
       6.3550408154357236e-06, 
       6.1845650996092003e-06, 
       6.3616741339881534e-06, 
       5.0196847702697732e-06, 
       5.3614456434472771e-06, 
       5.3812634519404583e-06, 
       5.7308393630989936e-06, 
       6.1201332253082622e-06, 
       6.3107648868426264e-06, 
       6.1677310431917398e-06, 
       5.5444796246049634e-06, 
       5.1796768606720741e-06, 
       5.4747765538091168e-06, 
       5.8455324207581975e-06, 
       5.4277700209869721e-06, 
       5.0024295208455143e-06, 
       5.2643842900094456e-06, 
       6.3303785320649327e-06, 
       6.0114559276627244e-06, 
       6.3256015793503870e-06, 
       6.2808127364632513e-06, 
       5.2762368743937764e-06, 
       6.0972649739483321e-06, 
       5.8300872368097120e-06, 
       5.8300872368097120e-06, 
       5.1598370897904015e-06, 
       5.4626146644626077e-06, 
       5.0996278109875364e-06, 
       5.9463396058431414e-06, 
       5.7536138244090496e-06, 
       6.0858145226710934e-06, 
       5.0954068610092392e-06, 
       6.0350406989189679e-06, 
       5.3592951436384879e-06, 
       5.1155353431921269e-06, 
       5.5119125694427228e-06, 
       5.3365177727165231e-06, 
       5.0581859382161608e-06, 
       5.4369362561508835e-06, 
       6.0177691585081710e-06, 
       5.3897817444824042e-06, 
       5.4429047005213112e-06, 
       6.1548312689348900e-06, 
       6.2559393177394249e-06, 
       5.6238225810619494e-06, 
       5.3330220345457473e-06, 
       5.6200269433882879e-06, 
       5.3054326963884223e-06, 
       5.2155845987621009e-06, 
       6.0325271446213796e-06, 
       6.0325271446213796e-06, 
       6.1782126098662529e-06, 
       6.1782126098662529e-06, 
       5.1757901891024100e-06, 
       5.1324257605761870e-06, 
       5.3936376763456650e-06, 
       5.2861094044231784e-06, 
       5.3498189020532625e-06, 
       6.2560180924793723e-06, 
       5.2565053150963198e-06, 
       5.3484195326053427e-06, 
       5.8455289353692215e-06, 
       6.0588282668225581e-06, 
       5.0379413483211976e-06, 
       6.2057535478895710e-06, 
       6.0057155141293581e-06, 
       5.3310955905386799e-06, 
       5.3291115562465479e-06, 
       5.0127295060414864e-06, 
       5.9749777363796055e-06, 
       5.0081515349133331e-06, 
       5.0146144317898749e-06, 
       6.1883210328814205e-06, 
       6.2071692176588892e-06, 
       5.3805868871736665e-06, 
       5.9836572025951664e-06, 
       5.0333025461794840e-06, 
       6.1523342664345888e-06, 
       6.0427403501075467e-06, 
       6.0530270978632636e-06, 
       5.8799005963263358e-06, 
       6.1883890754386447e-06, 
       5.2780674936391770e-06, 
       5.4437892384562048e-06, 
       5.9337392507855844e-06, 
       5.8382447128135589e-06, 
       5.7209016592709124e-06, 
       5.0113330671737226e-06, 
       5.8087148737957375e-06, 
       5.1595070186797060e-06, 
       5.1399457319934617e-06, 
       6.0178969399976503e-06, 
       5.7444661140035553e-06, 
       5.1486621761844994e-06, 
       5.9558537417361689e-06, 
       5.3524918510568683e-06, 
       5.9528406430842774e-06, 
       5.0656448100715895e-06, 
       5.2044046269480460e-06, 
       5.6186530142220227e-06, 
       5.4562655704896460e-06, 
       6.1440587956218229e-06, 
       5.5986714627720149e-06, 
       5.5898780152126773e-06, 
       6.1184045316402038e-06, 
       5.1226469337313348e-06, 
       5.0023908414153552e-06, 
       5.0857870525654636e-06, 
       5.8383893306633182e-06, 
       5.1000824279336001e-06, 
       5.0914411998711972e-06, 
       6.0107621790383463e-06, 
       6.1133813652809107e-06, 
       5.8268113858044288e-06, 
       5.9841089652204617e-06, 
       5.5043874270889355e-06, 
       6.0098143645202329e-06, 
       6.0812296172472494e-06, 
       5.9215775460626108e-06, 
       5.3607888136203843e-06, 
       5.1473876953804448e-06, 
       5.9585940214251379e-06, 
       5.3136288670390372e-06, 
       5.3136288670390372e-06, 
       5.2884613512650158e-06, 
       5.0869469743453589e-06, 
       5.0716693689100437e-06, 
       5.5989902629521520e-06, 
       5.9592307068080828e-06, 
       5.2461339793188327e-06, 
       6.0459189947077105e-06, 
       5.3743566457657728e-06, 
       5.1868241822810742e-06, 
       5.0737919503516130e-06, 
       5.1148031161402039e-06, 
       5.9580740092558554e-06, 
       5.4047125899594316e-06, 
       5.0752825518101136e-06, 
       5.5675548668729676e-06, 
       5.9489761361053251e-06, 
       5.9639904532365475e-06, 
       6.0187906790448153e-06, 
       5.6717230633106364e-06, 
       5.0503370431109422e-06, 
       6.0166729205228162e-06, 
       6.0610055956872700e-06, 
       5.1668783178540350e-06, 
       5.0602226827531717e-06, 
       5.0030545453146077e-06, 
       5.9527703788844209e-06, 
       5.1546695371713818e-06, 
       5.9863421046551326e-06, 
       5.3621423238508812e-06, 
       5.0368919279212390e-06, 
       5.0052550777560114e-06, 
       5.9301269746991164e-06, 
       5.9301269746991164e-06, 
       6.0302269457810001e-06, 
       5.5134269004199718e-06, 
       5.8700772806571592e-06, 
       5.1336753862608268e-06, 
       5.1952897824953165e-06, 
       5.5635477614355360e-06, 
       5.1477465836711558e-06, 
       5.1850800173310276e-06, 
       5.7362718768636548e-06, 
       5.6947134222493196e-06, 
       5.7449798931401584e-06, 
       5.9955761872355403e-06, 
       5.1587237221590708e-06, 
       5.9877277740763452e-06, 
       5.6446954884715188e-06, 
       5.9007229710252235e-06, 
       5.6647491616665125e-06, 
       5.7264001663513060e-06, 
       5.9410527690328562e-06, 
       5.3217152354915820e-06, 
       5.3217152354915820e-06, 
       5.9527618272326271e-06, 
       5.9667441212585740e-06, 
       5.6021974795037798e-06, 
       5.2635331453849485e-06, 
       5.0374919295276705e-06, 
       5.0809930755746811e-06, 
       5.6366760787676291e-06, 
       5.5083637395121758e-06, 
       5.8341219367879098e-06, 
       5.9406452489022741e-06, 
       5.9110492104708608e-06, 
       5.0378812630258421e-06, 
       5.7002020148408301e-06, 
       5.1386929685965699e-06, 
       5.0971950924986852e-06, 
       5.1761038326235829e-06, 
       5.0706020531530833e-06, 
       5.5765089760325255e-06, 
       5.8397052964406561e-06, 
       5.1208030381570945e-06, 
       5.1017017515123913e-06, 
       5.5412966474801593e-06, 
       5.2927184156121335e-06, 
       5.0158445186118359e-06, 
       5.8128671961918113e-06, 
       5.8398416966148020e-06, 
       5.8818599718788383e-06, 
       5.7351418318056513e-06, 
       5.8767966991916143e-06, 
       5.5834309364316821e-06, 
       5.6496390270471529e-06, 
       5.1111775084856188e-06, 
       5.0905241699695831e-06, 
       5.4485705131333322e-06, 
       5.0141494433966451e-06, 
       5.7741083738719545e-06, 
       5.7580931033362738e-06, 
       5.5570545952195046e-06, 
       5.0139846842892758e-06, 
       5.6488399521117475e-06, 
       5.8332966795243346e-06, 
       5.5781167524448547e-06, 
       5.6826025096072567e-06, 
       5.1675695745309290e-06, 
       5.7683578268900917e-06, 
       5.8033858471278421e-06, 
       5.6047892526645588e-06, 
       5.0080991041142102e-06, 
       5.1711592226692452e-06, 
       5.4197206120298153e-06, 
       5.1681540059604073e-06, 
       5.7958033305428470e-06, 
       5.0850739827986408e-06, 
       5.2305982181848773e-06, 
       5.7222684747191234e-06, 
       5.0988049745223930e-06, 
       5.5230836394820703e-06, 
       5.2328760747914639e-06, 
       5.0557660700306887e-06, 
       5.3121492095264738e-06, 
       5.2687993902927303e-06, 
       5.2687993902927303e-06, 
       5.7706132811397440e-06, 
       5.5123040747650722e-06, 
       5.5769498830767172e-06, 
       5.2061781259568613e-06, 
       5.4354116781069014e-06, 
       5.7639818777898807e-06, 
       5.7188654748385452e-06, 
       5.4828044959411630e-06, 
       4.1764615318466408e-06, 
       5.4316315949652049e-06, 
       5.5107028634243977e-06, 
       5.7394686998754322e-06, 
       5.7359904372353817e-06, 
       5.7402263124701256e-06, 
       5.0914822169357031e-06, 
       5.6655685038791128e-06, 
       5.6798481359192832e-06, 
       5.7303422585022632e-06, 
       5.7255355375710926e-06, 
       5.1137142088643486e-06, 
       5.7150255980480998e-06, 
       5.2082277271798792e-06, 
       5.6365689227533783e-06, 
       5.5536958900480318e-06, 
       5.1958150120846636e-06, 
       5.7054117003563376e-06, 
       5.2484155199182350e-06, 
       5.2839575326039288e-06, 
       5.6431276911212468e-06, 
       4.8818541165884776e-06, 
       5.3660177641301225e-06, 
       5.5986888890180555e-06, 
       5.4016191175648048e-06, 
       5.0350877214284590e-06, 
       5.0883793820649863e-06, 
       5.1328643055958233e-06, 
       5.5122679323975302e-06, 
       5.2756103776186083e-06, 
       5.1016338865315503e-06, 
       5.0889083072643050e-06, 
       5.0255201896258539e-06, 
       5.0114019559471120e-06, 
       5.4163573445536790e-06, 
       5.5827640096120511e-06, 
       5.3924816868528117e-06, 
       4.0107417700803471e-06, 
       5.1198320135288938e-06, 
       5.4356376596428968e-06, 
       5.3785799184178310e-06, 
       5.3785799184178310e-06, 
       5.3666902486106625e-06, 
       5.5489520802024997e-06, 
       5.3430324147748822e-06, 
       5.2004128589288835e-06, 
       5.2344845002324264e-06, 
       5.5916786106987390e-06, 
       5.1042960463452030e-06, 
       5.0058057433437249e-06, 
       5.5461975267482173e-06, 
       5.4509189869747298e-06, 
       5.3139329183145442e-06, 
       5.0959858930645435e-06, 
       5.2369201014007488e-06, 
       5.5170066223313800e-06, 
       5.0599405267161582e-06, 
       5.3981951069656541e-06, 
       5.4835083589949598e-06, 
       5.2373372488400130e-06, 
       5.0098594388383370e-06, 
       5.5340366549082862e-06, 
       5.3581334658876339e-06, 
       5.5445668642847338e-06, 
       5.3390805778782203e-06, 
       5.4967517777583481e-06, 
       5.4570444438223800e-06, 
       5.4237702116104440e-06, 
       5.4321369776985504e-06, 
       5.2798992878887659e-06, 
       5.2752831942193078e-06, 
       5.3459980893226038e-06, 
       5.5036303581537377e-06, 
       5.4763647998242889e-06, 
       5.2914631796811970e-06, 
       5.0569303299710301e-06, 
       5.4644570077575748e-06, 
       5.4574851531646112e-06, 
       5.3506488461072221e-06, 
       5.4255230620318967e-06, 
       5.4635550371603619e-06, 
       5.2026455310589493e-06, 
       5.2741898402235437e-06, 
       5.4031742231514186e-06, 
       5.2288365347478204e-06, 
       5.2073524265684458e-06, 
       5.2014224215640365e-06, 
       5.3689955616577516e-06, 
       5.1154003794623961e-06, 
       5.2433728403881844e-06, 
       5.2882509971059874e-06, 
       5.4289621281983504e-06, 
       4.0188561026472077e-06, 
       5.2014913049325679e-06, 
       5.1689124892622495e-06, 
       5.3257829930331397e-06, 
       5.3688968592956671e-06, 
       5.3069212302261274e-06, 
       5.3069212302261274e-06, 
       5.4149365980804110e-06, 
       5.3724860372220292e-06, 
       5.1993932169724536e-06, 
       5.3289096220349370e-06, 
       4.6979732785818350e-06, 
       5.3128076504315207e-06, 
       5.3703345792560846e-06, 
       5.4007275206599324e-06, 
       5.0953348323435475e-06, 
       5.1994644592379628e-06, 
       5.1890259328273346e-06, 
       5.3117953752267757e-06, 
       5.3686594188764639e-06, 
       5.0041988766673299e-06, 
       5.3644787454305108e-06, 
       5.3374067489281747e-06, 
       5.1629011880157555e-06, 
       5.3404349588708749e-06, 
       5.1736240070327872e-06, 
       5.2176889157484829e-06, 
       5.2176889157484829e-06, 
       5.2016664016103271e-06, 
       5.1692510662168345e-06, 
       5.3350275550941942e-06, 
       5.2940469610276675e-06, 
       5.1294401935810315e-06, 
       5.3295786762989356e-06, 
       5.0274451289247776e-06, 
       5.2953681099277361e-06, 
       5.1682544786000140e-06, 
       5.1480288055990563e-06, 
       5.3174348734126608e-06, 
       5.2867214134870111e-06, 
       5.2951483965307036e-06, 
       5.2707784294424629e-06, 
       5.1925527684970269e-06, 
       5.2810458312463196e-06, 
       5.2843808019004501e-06, 
       5.2651312697631949e-06, 
       5.2732545685315601e-06, 
       5.2756383548186310e-06, 
       4.2553133236938207e-06, 
       5.0132351763416044e-06, 
       4.0653322648608337e-06, 
       5.0540832211679705e-06, 
       5.2325435948903670e-06, 
       5.0581468764195084e-06, 
       5.0290456866366287e-06, 
       5.0403817435733359e-06, 
       5.1817374429881562e-06, 
       5.0695608853696723e-06, 
       5.1826883761894547e-06, 
       5.0308771467482412e-06, 
       5.0866674452058403e-06, 
       5.0101393907263346e-06, 
       5.0377559537885877e-06, 
       5.2048477030060932e-06, 
       5.1144041116000440e-06, 
       5.1606391537171966e-06, 
       5.0782952286673656e-06, 
       5.1563038159900740e-06, 
       5.1092128326741333e-06, 
       3.6445288565889857e-06, 
       5.1044741941917574e-06, 
       4.6817843072271943e-06, 
       5.0919802979617073e-06, 
       5.0812516315489175e-06, 
       5.0217176709760366e-06, 
       5.0669850127083357e-06, 
       5.0623150247998982e-06, 
       5.0623563277233133e-06, 
       5.0415134111404351e-06, 
       5.0294090149971936e-06, 
       5.0138605174322225e-06, 
       3.6061083905788575e-06, 
       5.0087066005281490e-06, 
       4.7157887370096078e-06, 
       4.7897188816553370e-06, 
       4.0190420555389312e-06, 
       3.4280937214479663e-06, 
       3.2683039369825603e-06, 
       4.5439360425481115e-06, 
       4.3906793360816393e-06, 
       4.4577096361213861e-06, 
       3.7606238731965770e-06, 
       4.2553268880521830e-06, 
       4.3692138195555612e-06, 
       3.7645550140875171e-06, 
       3.7959686286191428e-06, 
       3.9406323286473640e-06, 
       4.0848370257426016e-06, 
       3.0724108836804624e-06, 
       4.0058863910256172e-06, 
       2.9453054766729291e-06, 
       3.6740512664573890e-06, 
       3.3596222534086296e-06, 
       3.5457440681216754e-06, 
       2.8769414880236166e-06, 
       3.0827636387375175e-06, 
       2.9248679763402892e-06, 
       3.2812002729520182e-06, 
       3.1952025050945725e-06, 
       3.4352154086302833e-06, 
       2.5863369360047297e-06, 
       2.8472900011959834e-06, 
       3.5270539384196333e-06, 
       3.1155017456187914e-06, 
       3.1469511900523859e-06, 
       3.1131355704337638e-06, 
       3.4632191032818884e-06, 
       2.4894192284453514e-06, 
       2.4894192284453514e-06, 
       2.6036523611461552e-06, 
       3.1718967289293811e-06, 
       2.4549565088739915e-06, 
       3.2447866746631603e-06, 
       3.1602908500623040e-06, 
       3.3460338124861593e-06, 
       2.9454910322133524e-06, 
       2.9939527757406180e-06, 
       2.9808893923106758e-06, 
       3.1173806684887831e-06, 
       3.1468941190235020e-06, 
       2.2173826843207871e-06, 
       2.3574293577752759e-06, 
       2.7053704866996431e-06, 
       2.4138602986550301e-06, 
       2.3778626024340618e-06, 
       2.6475442173597309e-06, 
       2.1115446236870956e-06, 
       1.9668109382580050e-06, 
       1.9809797216841081e-06, 
       2.0359554775028884e-06, 
       2.6138378238767261e-06, 
       2.0841133606305947e-06, 
       2.3131519585527844e-06, 
       1.7456838155087883e-06, 
       2.3652463792342635e-06, 
       1.7553329487995013e-06, 
       1.9877944423922688e-06, 
       2.3095649050887859e-06, 
       2.0773367622062853e-06, 
       2.0435816749251404e-06, 
       1.9592015644998100e-06, 
       1.3912497124621834e-06, 
       1.8775644539483207e-06, 
       1.9237477726393112e-06, 
       1.3601182142183656e-06, 
       1.7133237645418347e-06, 
       1.3696626727948160e-06, 
       1.8955436227967992e-06, 
       1.5277402074724151e-06, 
       1.7713908614740323e-06, 
       1.4484934338608677e-06, 
       1.6327268799973691e-06, 
       1.6092177938411747e-06, 
       1.2992445975798179e-06, 
       1.2791313356930112e-06, 
       1.0304074300189669e-06, 
       1.2924496282158034e-06, 
       1.2107750826153107e-06, 
       1.1662244481932350e-06, 
       1.1333784194240194e-06, 
       9.4877435005246230e-07, 
       9.1465683905706348e-07, 
       8.3270552020066231e-07, 
       6.8584670715570804e-07, 
       7.5071849672608790e-07, 
       5.6213213495874768e-07, 
       6.5168860764867182e-07, 
       6.8313173358539717e-07, 
       6.7822379339035126e-07, 
       4.5142205489306838e-07, 
       5.4018484783610423e-07, 
       3.3812713272444742e-07, 
       3.1968879635580148e-07, 
       3.8275975987512881e-07, 
       3.2556977434010945e-07, 
       3.3377133367819638e-07, 
       2.8887629371463892e-07, 
       2.1808147756675140e-07, 
       1.9865652723404625e-07, 
       2.0431345209440644e-07, 
       1.6725764829710641e-07, 
       1.0998523475471594e-07, 
       9.2489351323682330e-08, 
       5.1395000000000002e-01, 
       3.8245000000000001e-01, 
       3.2654000000000000e-01, 
       2.6395999999999997e-01, 
       1.2302000000000000e-01, 
       7.7539999999999998e-02, 
       6.0679999999999998e-02, 
       4.9700000000000001e-02, 
       4.1939999999999998e-02, 
       3.2219999999999999e-02, 
       2.5290000000000000e-02, 
       2.4899999999999999e-02, 
       1.7639999999999999e-02, 
       1.2568406893972081e-02, 
       1.1784003379098401e-02, 
       1.4489999999999999e-02, 
       1.3690897470438234e-02, 
       1.1860000000000001e-02, 
       1.0659999999999999e-02, 
       9.9299999999999996e-03, 
       6.5799999999999999e-03, 
       6.4753622005591753e-03, 
       5.8700000000000002e-03, 
       5.2279423322279840e-03, 
       5.2275372671141095e-03, 
       5.3600000000000002e-03, 
       4.7600000000000003e-03, 
       3.5343517042719782e-03, 
       3.5161394648211041e-03, 
       3.9399999999999999e-03, 
       3.3100000000000000e-03, 
       3.0999999999999999e-03, 
       2.3260935648086235e-03, 
       2.2062438129107935e-03, 
       2.1868024776725554e-03, 
       2.3500000000000001e-03, 
       2.3500000000000001e-03, 
       2.3300000000000000e-03, 
       2.1400000000000000e-03, 
       2.1299999999999999e-03, 
       1.4878968187292880e-03, 
       1.4872760675947584e-03, 
       1.7300000000000000e-03, 
       1.6500000000000000e-03, 
       1.4599999999999999e-03, 
       1.4000000000000000e-03, 
       1.2699402436660329e-03, 
       1.2688350196431452e-03, 
       1.2600000000000001e-03, 
       1.2500000000000000e-03, 
       1.1400000000000000e-03, 
       1.1199999999999999e-03, 
       8.7825392277994995e-04, 
       8.7166795623049573e-04, 
       1.0600000000000000e-03, 
       6.8723565472967240e-04, 
       7.0684341392724824e-04, 
       7.0440859794383606e-04, 
       8.8999999999999995e-04, 
       6.4461674483570384e-04, 
       8.4000000000000003e-04, 
       8.4000000000000003e-04, 
       8.0999999999999996e-04, 
       5.9271753236239406e-04, 
       7.3999999999999999e-04, 
       5.7902852888044225e-04, 
       5.5679248941917117e-04, 
       6.8999999999999997e-04, 
       5.6117488115408232e-04, 
       5.6057577514191095e-04, 
       4.8193391383684933e-04, 
       6.4000000000000005e-04, 
       6.4000000000000005e-04, 
       6.2000000000000000e-04, 
       6.0999999999999997e-04, 
       5.9999999999999995e-04, 
       4.6537040858705362e-04, 
       5.2592823124983834e-04, 
       5.2592823124983834e-04, 
       4.1531973246123569e-04, 
       4.5000000000003354e-04, 
       5.5000000000000003e-04, 
       5.4000000000000001e-04, 
       4.2908044958095469e-04, 
       5.2999999999999998e-04, 
       5.2999999999999998e-04, 
       5.2999999999999998e-04, 
       4.9768189447461128e-04, 
       3.7454492888425476e-04, 
       3.7071638480686291e-04, 
       3.9708750105345888e-04, 
       5.1000000000000004e-04, 
       4.6999999999999999e-04, 
       3.3080690776920719e-04, 
       4.2999999999999999e-04, 
       3.0532549555169122e-04, 
       3.9497173643677320e-04, 
       3.9113420847551659e-04, 
       3.0772601464211665e-04, 
       3.9675208999735158e-04, 
       4.0000000000000002e-04, 
       4.0000000000000002e-04, 
       3.9365143818023950e-04, 
       2.9994785474000198e-04, 
       2.8000000000000935e-04, 
       3.0900123578542367e-04, 
       3.8999999999999999e-04, 
       3.0270902761540380e-04, 
       3.8000000000000002e-04, 
       2.5991094753513762e-04, 
       2.6053178823735282e-04, 
       2.5922019768501338e-04, 
       2.5186415461473494e-04, 
       2.4454534469128033e-04, 
       2.4988031352489069e-04, 
       3.4000000000000002e-04, 
       2.4166799235374786e-04, 
       2.5101543941009126e-04, 
       2.4501256647819770e-04, 
       2.3000000000005058e-04, 
       2.2841802372000470e-04, 
       2.5294714920320313e-04, 
       2.2829490021872978e-04, 
       3.2000000000000003e-04, 
       2.2790008645504975e-04, 
       2.3263758280866889e-04, 
       2.8511287761438266e-04, 
       2.8507847399412414e-04, 
       2.5019068052258085e-04, 
       2.9325496945684521e-04, 
       2.9046198675541311e-04, 
       2.5125325925129052e-04, 
       2.4709836201773508e-04, 
       2.4695737727719485e-04, 
       2.5110045651422196e-04, 
       2.3767051727543534e-04, 
       2.3485512641444902e-04, 
       2.4522738186651729e-04, 
       2.9000000000000000e-04, 
       2.9000000000000000e-04, 
       2.7999999999999998e-04, 
       1.9354233554890547e-04, 
       2.7000000000000000e-04, 
       1.9797077374057048e-04, 
       1.8923655014563631e-04, 
       2.5999999999999998e-04, 
       1.9140286245590093e-04, 
       1.9049823101475376e-04, 
       1.8137838533852755e-04, 
       1.8390377530360578e-04, 
       1.8281122273027361e-04, 
       1.9430328733263945e-04, 
       1.9000000000004777e-04, 
       1.7667572091763153e-04, 
       1.7889787186768205e-04, 
       1.7056857781076406e-04, 
       2.3000000000000001e-04, 
       1.6999999999998966e-04, 
       1.6447454584617039e-04, 
       1.5977613888343729e-04, 
       2.2000000000000001e-04, 
       1.4964434856036944e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.9000000000000001e-04, 
       1.5017598457786838e-04, 
       1.4897594970445617e-04, 
       1.3723239055300529e-04, 
       1.3086431296222998e-04, 
       1.3557168198828488e-04, 
       1.3270684530295230e-04, 
       1.4359993940508937e-04, 
       1.2442503315285798e-04, 
       1.2367343471197728e-04, 
       1.2686173899535859e-04, 
       1.2884511326030930e-04, 
       1.7000000000000001e-04, 
       1.3513548092575971e-04, 
       1.3063050647139311e-04, 
       1.6000000000000001e-04, 
       1.6000000000000001e-04, 
       1.1642313303651290e-04, 
       1.5000000000004311e-04, 
       1.2749412219105222e-04, 
       1.1187488756414990e-04, 
       1.0755307430968702e-04, 
       1.4999999999999999e-04, 
       1.4999999999999999e-04, 
       1.4999999999999999e-04, 
       1.1299769972415393e-04, 
       1.1839141046751093e-04, 
       1.0999999999999572e-04, 
       1.0392399590770566e-04, 
       1.2307367730301260e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.3999999999999999e-04, 
       1.0905085975014206e-04, 
       1.0001913494704076e-04, 
       1.0559937521497350e-04, 
       1.0000000000000570e-04, 
       9.9405048484051031e-05, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.2999999999999999e-04, 
       1.0149101534921004e-04, 
       9.0000000000000792e-05, 
       9.0243828537369297e-05, 
       9.5165385561884030e-05, 
       8.9999999999997404e-05, 
       8.9999999999997404e-05, 
       1.1334847871294843e-04, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       1.2000000000000000e-04, 
       1.1000000000003241e-04, 
       8.7710285957550918e-05, 
       8.4460857744159920e-05, 
       8.4761128208903357e-05, 
       9.2252495680553986e-05, 
       8.2342999936493421e-05, 
       1.1550314987776648e-04, 
       1.1546447826408749e-04, 
       8.4547368504826441e-05, 
       8.3152647071812804e-05, 
       9.5960734913279127e-05, 
       8.0000000000001186e-05, 
       1.1033330856068844e-04, 
       7.8886000801812084e-05, 
       1.1000000000000000e-04, 
       8.3043091216989743e-05, 
       7.9816681021397759e-05, 
       7.9622816564665214e-05, 
       7.5891301312984365e-05, 
       8.1459149995481456e-05, 
       7.3365123912174588e-05, 
       8.1679035965688031e-05, 
       7.4075634107649023e-05, 
       7.9168261898614610e-05, 
       7.9088096306455838e-05, 
       8.1194627099614026e-05, 
       1.0000000000000000e-04, 
       1.0000000000000000e-04, 
       8.1146666386748705e-05, 
       8.1040148542723745e-05, 
       7.0116217323202353e-05, 
       6.8465088072203978e-05, 
       6.8241291959534998e-05, 
       6.9210065860619323e-05, 
       7.2614126026657202e-05, 
       6.6967527743241344e-05, 
       7.8203016676591175e-05, 
       6.7398905880599206e-05, 
       6.4303073408889709e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       9.0000000000000006e-05, 
       6.4979619015389175e-05, 
       6.3760429821466111e-05, 
       6.3069388092895171e-05, 
       6.2967914103979937e-05, 
       6.5929312836591415e-05, 
       8.2154476603235867e-05, 
       7.0000000000004344e-05, 
       6.2131835531902989e-05, 
       6.5187197546849373e-05, 
       6.3766869393550138e-05, 
       6.4914606257073944e-05, 
       6.0953568224508716e-05, 
       6.1452486986326326e-05, 
       6.3662204929734099e-05, 
       6.2125489845577463e-05, 
       6.0311746889320389e-05, 
       6.1890712814082030e-05, 
       6.8926078079690930e-05, 
       5.9120862622828267e-05, 
       5.8110994913895685e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       8.0000000000000007e-05, 
       6.1707893043326067e-05, 
       6.1692059474636917e-05, 
       5.8361048514878264e-05, 
       6.0000000000000889e-05, 
       6.1656465940055846e-05, 
       6.0574034811264697e-05, 
       5.4683645659718058e-05, 
       7.1003664702539888e-05, 
       5.4780535305623308e-05, 
       5.1782835873059048e-05, 
       6.8526730205726194e-05, 
       5.5359418872701745e-05, 
       5.1036428582262125e-05, 
       5.0531453149911530e-05, 
       5.2575609598701979e-05, 
       4.9980419730081616e-05, 
       5.5594112775369728e-05, 
       5.9328689179248411e-05, 
       5.0000000000000958e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       5.6519647400985846e-05, 
       4.9885129577079217e-05, 
       5.0000000000000484e-05, 
       5.0000000000000484e-05, 
       4.9599812436233439e-05, 
       4.8632420095004347e-05, 
       4.9398645239422328e-05, 
       4.8096626752022035e-05, 
       4.8959874911656805e-05, 
       5.0000000000000829e-05, 
       4.8485255662400824e-05, 
       4.6513486503327878e-05, 
       4.6157010319438953e-05, 
       4.5392787589185880e-05, 
       4.6177513316823789e-05, 
       4.5555108923361914e-05, 
       4.8899035332652549e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       6.0000000000000002e-05, 
       4.7418265235260363e-05, 
       4.3472723661292423e-05, 
       4.3400631161124156e-05, 
       4.7048147378972200e-05, 
       4.2184021451112264e-05, 
       3.9999999999998167e-05, 
       4.0000000000000057e-05, 
       4.2017656494641927e-05, 
       4.0000000000000844e-05, 
       4.1501398099675757e-05, 
       4.1978138299121989e-05, 
       4.0753305337013683e-05, 
       4.6191892594424316e-05, 
       3.9273783856895769e-05, 
       3.9802849971982503e-05, 
       5.2064486505768060e-05, 
       3.8800582781518947e-05, 
       3.8584568300673510e-05, 
       3.8284407720050025e-05, 
       4.6788648784923036e-05, 
       3.8172847651184046e-05, 
       4.0647446122550230e-05, 
       4.0805637271106639e-05, 
       3.8578230080372285e-05, 
       3.9378349820930966e-05, 
       3.7042812135612926e-05, 
       5.0482708280868738e-05, 
       3.7434534127795312e-05, 
       3.9999999999994840e-05, 
       3.7107575921643962e-05, 
       3.8491276509774090e-05, 
       3.6599474590053623e-05, 
       3.9999999999998025e-05, 
       3.9896459567427549e-05, 
       4.9561464705726884e-05, 
       3.9632905834956871e-05, 
       4.8957470931800587e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       5.0000000000000002e-05, 
       3.5307894505969600e-05, 
       3.6706748173119010e-05, 
       4.7036822723610919e-05, 
       3.4896201836217627e-05, 
       3.4896201836217627e-05, 
       4.0000000000001630e-05, 
       3.6452796315661451e-05, 
       3.8083150687420347e-05, 
       3.4928978498854827e-05, 
       3.3867620534448810e-05, 
       3.7520259983707021e-05, 
       3.5668559013682856e-05, 
       4.6506152394088104e-05, 
       4.4604463882644200e-05, 
       3.2699023886874744e-05, 
       3.4007252950701781e-05, 
       3.3034564518192191e-05, 
       3.2341141198643524e-05, 
       3.9407048502644538e-05, 
       3.2551398735100988e-05, 
       4.4138556343625333e-05, 
       3.3420174369559843e-05, 
       3.1538258279122524e-05, 
       3.1496886654673129e-05, 
       3.1493970367684139e-05, 
       3.6761965623058505e-05, 
       3.1659196010350884e-05, 
       3.2568230167760899e-05, 
       3.4521648219744358e-05, 
       3.0596838357052694e-05, 
       3.9916210866537443e-05, 
       2.9999999999995623e-05, 
       3.0490353142401590e-05, 
       3.0241412231445859e-05, 
       3.0350932444682084e-05, 
       3.2857246986154037e-05, 
       3.0000000000001004e-05, 
       3.2261315146808669e-05, 
       3.7959026900473236e-05, 
       3.1667618604007313e-05, 
       3.7933411674007313e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       2.8589603978043228e-05, 
       3.1524492131290423e-05, 
       2.8362946479193587e-05, 
       3.8626525785678050e-05, 
       3.8093536670983630e-05, 
       3.1222477793243399e-05, 
       3.1202989880283919e-05, 
       2.9999999999998578e-05, 
       2.9999999999998578e-05, 
       2.7681872941054794e-05, 
       2.7230973358653330e-05, 
       2.8182660385394178e-05, 
       2.8352723334744815e-05, 
       3.1260586080407508e-05, 
       3.0402734122248371e-05, 
       2.9999999999998259e-05, 
       2.7287745829678278e-05, 
       2.7644668719605025e-05, 
       2.9904463035609902e-05, 
       2.9999999999995478e-05, 
       2.8031009929185742e-05, 
       2.7711795395081385e-05, 
       3.0000000000003379e-05, 
       3.0000000000003379e-05, 
       3.0000000000003379e-05, 
       3.0000000000003379e-05, 
       2.8200418782756053e-05, 
       2.9201060919921262e-05, 
       2.7291010521720810e-05, 
       2.6700850744264351e-05, 
       2.8424040236260078e-05, 
       3.0000000000002017e-05, 
       3.0000000000001793e-05, 
       2.5387971797372546e-05, 
       2.6055731549064188e-05, 
       2.6869479518744932e-05, 
       2.5348528635847772e-05, 
       3.0137498916115293e-05, 
       3.0113508740696198e-05, 
       2.5751594850855288e-05, 
       2.5641570033012689e-05, 
       2.9293257112649028e-05, 
       3.3656473252729510e-05, 
       3.3103433646800270e-05, 
       2.5471544590581578e-05, 
       3.3535038549939455e-05, 
       3.3064468029852999e-05, 
       2.4319205301166105e-05, 
       2.5673926407981536e-05, 
       3.3412806132221019e-05, 
       2.4600570094929900e-05, 
       3.2758111197503761e-05, 
       3.2804861329707739e-05, 
       2.5119103852087525e-05, 
       2.5700355298132588e-05, 
       2.5700355298132588e-05, 
       2.4455687401803118e-05, 
       2.5280258792097136e-05, 
       3.2010309558555405e-05, 
       2.3371881420904367e-05, 
       2.3191029991928993e-05, 
       2.3054601462705134e-05, 
       2.5667700509819839e-05, 
       2.4255482066990104e-05, 
       2.4404707724799157e-05, 
       3.1077293641236036e-05, 
       2.4628521461384366e-05, 
       2.2842897000823129e-05, 
       2.6664807061112853e-05, 
       2.5782177582851115e-05, 
       2.3651192529226165e-05, 
       2.6103195788865071e-05, 
       2.9784774926826150e-05, 
       2.2025540017736664e-05, 
       2.4910720027409912e-05, 
       2.2513791214272149e-05, 
       2.4312406084897530e-05, 
       2.2717852425910860e-05, 
       2.1785674730739426e-05, 
       2.2370308360634774e-05, 
       2.2370308360634774e-05, 
       2.1652804158264882e-05, 
       2.3909890694788322e-05, 
       2.4152512197972920e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.3788430104401198e-05, 
       2.0978090127768883e-05, 
       2.3719965812685840e-05, 
       2.1494168635770795e-05, 
       2.6099462241861174e-05, 
       2.2217013440853598e-05, 
       2.7253304578086529e-05, 
       2.0481436127107325e-05, 
       2.4839153453821431e-05, 
       2.6987389394127891e-05, 
       2.6889520563811129e-05, 
       2.0574098169193254e-05, 
       2.5293651134129159e-05, 
       2.0793289668495495e-05, 
       2.6941697375168136e-05, 
       2.0373438942022411e-05, 
       2.2640866923612534e-05, 
       1.9823607667748466e-05, 
       2.0000000000005372e-05, 
       2.1795645248532292e-05, 
       2.4143358489140004e-05, 
       2.3112689627928407e-05, 
       1.9999999999997288e-05, 
       2.0099751201744037e-05, 
       2.3319758926953415e-05, 
       1.8854812707935437e-05, 
       2.1647722281125278e-05, 
       1.9297068739862420e-05, 
       2.1085360621631260e-05, 
       2.4495860202666620e-05, 
       2.1021480119838357e-05, 
       1.8288369937107696e-05, 
       1.8592980633971474e-05, 
       2.0454394181672909e-05, 
       2.4802079123496686e-05, 
       1.8290334782186212e-05, 
       2.4978113595954295e-05, 
       1.8544196614098385e-05, 
       1.9799893344402930e-05, 
       1.7936792018330110e-05, 
       2.3165049274500912e-05, 
       1.9566354639868191e-05, 
       1.9753377571509464e-05, 
       2.3266934548824592e-05, 
       1.7743990459484160e-05, 
       1.8002785822586596e-05, 
       2.3216764265444699e-05, 
       1.7710368627631337e-05, 
       2.0000000000000537e-05, 
       1.8865230683183364e-05, 
       1.8001063923225192e-05, 
       1.7842555882918417e-05, 
       1.7825460844297778e-05, 
       1.7712757114896970e-05, 
       2.1451828402676773e-05, 
       1.7528642660685291e-05, 
       2.1414732482260364e-05, 
       1.7452575761645021e-05, 
       1.9652980405081511e-05, 
       2.1376184080307644e-05, 
       1.7269222747465106e-05, 
       2.1323411353491352e-05, 
       1.7094557185043193e-05, 
       1.7039779640373359e-05, 
       2.0333544285585613e-05, 
       1.7484167385865677e-05, 
       1.7190010149894823e-05, 
       1.9999999999996559e-05, 
       1.9999999999996559e-05, 
       1.7011808470777423e-05, 
       1.9002429201614061e-05, 
       1.6830951520407571e-05, 
       2.0411089720788116e-05, 
       1.6670920597545245e-05, 
       1.6571842208823096e-05, 
       1.8685397456088012e-05, 
       1.9403934891001417e-05, 
       1.8355071144565587e-05, 
       1.6642354989153762e-05, 
       1.6808152851161150e-05, 
       1.8276185070206446e-05, 
       1.8115458148729392e-05, 
       2.1087620622527917e-05, 
       2.1087620622527917e-05, 
       1.6279188799200311e-05, 
       1.6750880293239179e-05, 
       1.6216072144592670e-05, 
       1.6489329929888867e-05, 
       1.7991949977368835e-05, 
       1.6335716544553203e-05, 
       1.6216155011468388e-05, 
       1.7573744286687250e-05, 
       1.7547748994287195e-05, 
       1.9999999999997061e-05, 
       2.1391474193466786e-05, 
       1.7291576990228942e-05, 
       1.8431090852052867e-05, 
       2.0000000000004813e-05, 
       1.5304570455558684e-05, 
       1.5358621616874319e-05, 
       1.5660277408606414e-05, 
       1.9999999999996698e-05, 
       1.9999999999996698e-05, 
       1.8190120812147348e-05, 
       1.6165886412933463e-05, 
       1.5606222406256737e-05, 
       1.6685554873310344e-05, 
       1.5899430431353373e-05, 
       1.4875289298916494e-05, 
       1.4850802965767678e-05, 
       2.0082540826426291e-05, 
       1.4882863623829293e-05, 
       1.6501039814091209e-05, 
       2.0385008116287263e-05, 
       1.9848939511585388e-05, 
       1.5650635876016359e-05, 
       1.5093514612430268e-05, 
       1.6640607382535858e-05, 
       1.6447002477705602e-05, 
       1.4917804831480329e-05, 
       1.5978200045072543e-05, 
       1.4585863928786180e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       2.0000000000000002e-05, 
       1.4570462038592828e-05, 
       1.4548663076605211e-05, 
       1.5685414700036578e-05, 
       1.4474125910208743e-05, 
       1.9365517980274570e-05, 
       1.8821586937772193e-05, 
       1.8807182851767827e-05, 
       1.4048284245040205e-05, 
       1.7385377344406373e-05, 
       1.7379015713368040e-05, 
       1.4088321779724864e-05, 
       1.3776952819239927e-05, 
       1.3990308702390204e-05, 
       1.4320532953379261e-05, 
       1.3501088620543262e-05, 
       1.7719856150355205e-05, 
       1.7709417175342579e-05, 
       1.7634153277693866e-05, 
       1.6390904418187451e-05, 
       1.3919229500910895e-05, 
       1.4338173253904613e-05, 
       1.3913927374749813e-05, 
       1.8469250275329990e-05, 
       1.3050630729572519e-05, 
       1.3613952334564977e-05, 
       1.3054152635182538e-05, 
       1.3405726778128933e-05, 
       1.3466072103855028e-05, 
       1.4373974059060835e-05, 
       1.2793964401976620e-05, 
       1.4159163904472759e-05, 
       1.3072426515244650e-05, 
       1.4595223222946117e-05, 
       1.3004172500496937e-05, 
       1.6750880293241520e-05, 
       1.5571871565364185e-05, 
       1.5571871565364185e-05, 
       1.3158900795072862e-05, 
       1.4328526581230689e-05, 
       1.2842855960269581e-05, 
       1.2576931002492702e-05, 
       1.2484543198423890e-05, 
       1.3399001468278735e-05, 
       1.4883740383401062e-05, 
       1.3155149403464286e-05, 
       1.2912947467300540e-05, 
       1.5592449403370351e-05, 
       1.2682396500829246e-05, 
       1.2236441354311879e-05, 
       1.4355160774092448e-05, 
       1.5841811442093344e-05, 
       1.2013877824227743e-05, 
       1.3620858279107597e-05, 
       1.2312265279682979e-05, 
       1.2216808093369565e-05, 
       1.2206843949530528e-05, 
       1.5770349181922546e-05, 
       1.2502644236347054e-05, 
       1.1844259935222618e-05, 
       1.1997676697147328e-05, 
       1.1837991875031265e-05, 
       1.2408266201573699e-05, 
       1.4022890600076445e-05, 
       1.4937707528793601e-05, 
       1.3039889740767648e-05, 
       1.2989125549696863e-05, 
       1.2204545186161065e-05, 
       1.1754400389805281e-05, 
       1.2099655400016914e-05, 
       1.1740206034148839e-05, 
       1.1738954996217235e-05, 
       1.1531923147655575e-05, 
       1.1584773043904730e-05, 
       1.2690199900280389e-05, 
       1.4235364985916107e-05, 
       1.1874281489299238e-05, 
       1.1329964783421689e-05, 
       1.2899869687755386e-05, 
       1.4391493753313829e-05, 
       1.1858175358032654e-05, 
       1.1759934567113864e-05, 
       1.1638428315735223e-05, 
       1.3401456710306259e-05, 
       1.4361919276825344e-05, 
       1.2170673549690623e-05, 
       1.2445876171938376e-05, 
       1.5181728277297751e-05, 
       1.1014838876694021e-05, 
       1.0903601780886999e-05, 
       1.3589561225417159e-05, 
       1.0982902513960771e-05, 
       1.2219076865613779e-05, 
       1.1462832579698723e-05, 
       1.0717430646007050e-05, 
       1.1190562103865615e-05, 
       1.1949004858055973e-05, 
       1.0717220743235374e-05, 
       1.2602434554336594e-05, 
       1.1193091379303580e-05, 
       1.0665946665475236e-05, 
       1.2189423617636098e-05, 
       1.1851867428322875e-05, 
       1.1492378436646006e-05, 
       1.2402437460254247e-05, 
       1.0501897353023273e-05, 
       1.0769361352568703e-05, 
       1.0660715299179784e-05, 
       1.0374449991422352e-05, 
       1.0369058316675662e-05, 
       1.3169266031338557e-05, 
       1.1319077525685141e-05, 
       1.0286753043817520e-05, 
       1.3634410117562983e-05, 
       1.1263356807985336e-05, 
       1.1689491022625012e-05, 
       1.0140840871030608e-05, 
       1.0370314950386196e-05, 
       1.0477076928911442e-05, 
       1.1373838594448410e-05, 
       9.8171802890085565e-06, 
       1.0929218512156398e-05, 
       9.9058879007358836e-06, 
       1.0907312010853471e-05, 
       9.9020587561811231e-06, 
       1.2329883850171766e-05, 
       1.2287846742225126e-05, 
       9.7601391118785833e-06, 
       1.0810989671155187e-05, 
       1.2826438668611414e-05, 
       1.0518105530445644e-05, 
       1.0467520347314968e-05, 
       9.4041025882412688e-06, 
       9.7838091661005748e-06, 
       9.6248840527469163e-06, 
       1.0375398206522494e-05, 
       9.5872968783310446e-06, 
       1.2608613236357157e-05, 
       9.3263932917629976e-06, 
       9.9310449206062467e-06, 
       1.2345438360638910e-05, 
       9.3433147664402195e-06, 
       9.6018533209568520e-06, 
       9.6180649223804651e-06, 
       1.0196186154948797e-05, 
       9.0548310137078983e-06, 
       1.0388469667100754e-05, 
       9.5980807834344346e-06, 
       9.3708193721244305e-06, 
       9.9371776464561289e-06, 
       9.5694966147049163e-06, 
       9.0966155162714480e-06, 
       9.1421652706040462e-06, 
       9.0602215610569848e-06, 
       9.7922551430125546e-06, 
       8.9260626300029436e-06, 
       9.5282931092297037e-06, 
       8.9590225639948553e-06, 
       9.3565070056060589e-06, 
       9.3186121677237623e-06, 
       8.9080038892701045e-06, 
       8.7543570600405882e-06, 
       1.0328187785966119e-05, 
       9.0325162377397451e-06, 
       8.7032329907912545e-06, 
       1.0833771441611283e-05, 
       1.0830820903337420e-05, 
       8.9132331306980121e-06, 
       8.8856451837987447e-06, 
       9.0957546797665316e-06, 
       9.7288650217690265e-06, 
       8.8403326850143654e-06, 
       1.1824711001772112e-05, 
       1.0165185739377282e-05, 
       8.5459251157449557e-06, 
       8.6725391624246413e-06, 
       8.4760960664675638e-06, 
       9.3669130719187560e-06, 
       1.1382089860841990e-05, 
       8.4361528444329976e-06, 
       9.4426513678278533e-06, 
       9.9735542821671082e-06, 
       8.6368010900088815e-06, 
       8.5837572488841785e-06, 
       8.3110585021256145e-06, 
       9.2525517275076938e-06, 
       8.8210997414247014e-06, 
       8.5344122617600387e-06, 
       8.5242264837865845e-06, 
       8.1833832546017160e-06, 
       8.1832145112202790e-06, 
       8.2112435963161936e-06, 
       8.2070696798167070e-06, 
       8.2349211126785980e-06, 
       8.1597053828882363e-06, 
       8.0185093665547183e-06, 
       9.1682244861165609e-06, 
       9.6635010610130547e-06, 
       8.9356673419142491e-06, 
       1.0868492926657315e-05, 
       9.3194228066974546e-06, 
       8.3189186031184482e-06, 
       1.0860785713890774e-05, 
       8.9192027120887603e-06, 
       8.7584731571415443e-06, 
       8.2663283767761240e-06, 
       7.9813698078336869e-06, 
       8.2293555424581714e-06, 
       7.8113145882220023e-06, 
       7.9973740233809380e-06, 
       7.6903816874505557e-06, 
       1.0539601170709950e-05, 
       8.0144730002726315e-06, 
       7.6995842152124265e-06, 
       8.4526566769240888e-06, 
       8.0062453251905186e-06, 
       7.8788314484962559e-06, 
       9.1080211200565109e-06, 
       8.3958281170095143e-06, 
       7.6104355949299831e-06, 
       8.6031187346043701e-06, 
       8.0517117893480499e-06, 
       9.9593032310541382e-06, 
       7.9321199367800675e-06, 
       9.9208134491744625e-06, 
       8.1791849230346208e-06, 
       7.6235860473217769e-06, 
       7.8072088469386917e-06, 
       9.0880906269530331e-06, 
       7.4961960723108302e-06, 
       8.3115242471540807e-06, 
       7.7296028991198358e-06, 
       8.2942225209006913e-06, 
       7.4391323999931147e-06, 
       8.3608347417200307e-06, 
       7.8020739612870757e-06, 
       7.8669021825709058e-06, 
       8.2369907808965320e-06, 
       7.5565087284292423e-06, 
       7.7018377912660353e-06, 
       7.6980867801185913e-06, 
       7.4395691839566759e-06, 
       7.4759465735550981e-06, 
       7.2540406206877551e-06, 
       8.0483136750243228e-06, 
       8.0483136750243228e-06, 
       7.5956879749604106e-06, 
       7.4097193479178969e-06, 
       7.0916411137885525e-06, 
       7.0693838551818292e-06, 
       8.4436596384628879e-06, 
       8.6389828507415969e-06, 
       9.6427639054843386e-06, 
       8.5553760380388220e-06, 
       7.1325489368607324e-06, 
       7.2553697159599648e-06, 
       7.0754050229257628e-06, 
       7.0695973231508207e-06, 
       9.6228969101457598e-06, 
       7.6531934404028063e-06, 
       7.3958434487168453e-06, 
       8.5692248872960298e-06, 
       7.3383706582692006e-06, 
       7.1854714019240426e-06, 
       7.5695005453400734e-06, 
       7.3534049644413207e-06, 
       8.9224062552253326e-06, 
       8.0440364611396338e-06, 
       6.9351579504160027e-06, 
       7.1731706648071492e-06, 
       7.2414232907511160e-06, 
       7.5914517616853656e-06, 
       7.1144438615740840e-06, 
       6.7881859574839110e-06, 
       8.2432295953023061e-06, 
       8.9369666098449463e-06, 
       8.5260400614014442e-06, 
       8.5306423584069898e-06, 
       8.1161924748027263e-06, 
       7.1830051066901237e-06, 
       7.8127449411731031e-06, 
       7.0870941545758944e-06, 
       6.9458209113742427e-06, 
       6.7636892687736263e-06, 
       6.7636892687736263e-06, 
       7.1394379159594234e-06, 
       6.8052713240863207e-06, 
       6.7966510516058743e-06, 
       6.7353906432321844e-06, 
       7.9178634525845773e-06, 
       6.6760335629827680e-06, 
       8.1368790951563903e-06, 
       9.2621059769026164e-06, 
       6.6455900426978548e-06, 
       7.5175700428122142e-06, 
       7.1621782109095060e-06, 
       7.1621782109095060e-06, 
       6.7472343586452060e-06, 
       7.5950482332312706e-06, 
       6.5338308507628183e-06, 
       6.5597495878337028e-06, 
       6.5213170430377275e-06, 
       6.9169630192050877e-06, 
       6.4902753335487729e-06, 
       8.4729920194510255e-06, 
       8.4729920194510255e-06, 
       7.7534866824578384e-06, 
       6.7395355866726167e-06, 
       6.8409174894260553e-06, 
       6.4999829569450186e-06, 
       6.4999829569450186e-06, 
       7.2034089576335261e-06, 
       7.1510840782560615e-06, 
       7.1241705664622865e-06, 
       7.1241705664622865e-06, 
       6.3222530748839867e-06, 
       6.4799563015104936e-06, 
       6.8080045890944089e-06, 
       8.7602520738025704e-06, 
       6.4578860935770902e-06, 
       6.8920567180715747e-06, 
       6.2384683098934866e-06, 
       6.2822798558249214e-06, 
       6.5780247373297760e-06, 
       6.7382527247518505e-06, 
       6.3827701255542561e-06, 
       6.1543963202956087e-06, 
       7.5183276575533373e-06, 
       8.3880151646012032e-06, 
       6.5045560304318092e-06, 
       7.1713681708993686e-06, 
       6.3437716282096654e-06, 
       8.0174566890029509e-06, 
       6.5054368741342887e-06, 
       7.5005437031529041e-06, 
       8.3163783593694593e-06, 
       6.3094168705288066e-06, 
       6.1356002839117095e-06, 
       6.3946997774153607e-06, 
       7.2399128418608813e-06, 
       6.3760734296361674e-06, 
       6.1230831531869733e-06, 
       8.4001409797736138e-06, 
       6.7835255895705884e-06, 
       6.9064471764360649e-06, 
       6.5596518342942182e-06, 
       8.2755937433931197e-06, 
       6.1492135127887412e-06, 
       6.1520176577604837e-06, 
       6.0226668186291452e-06, 
       6.7523292826438940e-06, 
       6.0666275178128038e-06, 
       6.3562982850042437e-06, 
       5.9967507767496810e-06, 
       7.0473615116974803e-06, 
       6.6714029894164123e-06, 
       5.8161734200203890e-06, 
       5.8100838021554864e-06, 
       5.8774046187868913e-06, 
       5.8691420639189546e-06, 
       6.1545441254885254e-06, 
       7.0068177970510037e-06, 
       6.4876817275913891e-06, 
       5.7958446823588512e-06, 
       5.7453321898965552e-06, 
       5.9448261711328567e-06, 
       5.9597742801824941e-06, 
       5.8838036299677197e-06, 
       6.4002514387297686e-06, 
       6.8963023034895994e-06, 
       5.9589844469564854e-06, 
       5.9455990434343156e-06, 
       5.9718716896313373e-06, 
       6.1740235380997552e-06, 
       5.5807457010752515e-06, 
       5.8022622185183938e-06, 
       6.1490838287004136e-06, 
       6.6931011957064057e-06, 
       6.4026776375959264e-06, 
       5.6086819124128343e-06, 
       6.2023096500389016e-06, 
       5.8924807938064745e-06, 
       6.0323439365576814e-06, 
       5.5333623990245080e-06, 
       6.1664880947978114e-06, 
       5.7188583147820323e-06, 
       5.7133969790055027e-06, 
       5.6544358741601256e-06, 
       7.2608850860695403e-06, 
       5.9179612057631273e-06, 
       5.5792238593155419e-06, 
       6.6024583872942040e-06, 
       5.9000159042981068e-06, 
       5.5628205001692157e-06, 
       6.2383764891975408e-06, 
       6.0065705098425825e-06, 
       7.2105422667616364e-06, 
       5.5297785453710923e-06, 
       5.8269152359354678e-06, 
       6.5034167644754156e-06, 
       5.7987387321087941e-06, 
       6.1244567299353366e-06, 
       5.4902248670982550e-06, 
       5.4381404142082752e-06, 
       5.3897061261285667e-06, 
       5.6537769330969066e-06, 
       5.8177954108434776e-06, 
       5.3511880178555835e-06, 
       5.9682279401161824e-06, 
       5.5531889501605182e-06, 
       6.1276104603660443e-06, 
       7.1829448073385632e-06, 
       5.8211208998089501e-06, 
       5.6075332458964771e-06, 
       5.5270128204197134e-06, 
       7.2438783991234791e-06, 
       6.5060578481571736e-06, 
       6.7949027083198405e-06, 
       5.4495255869135593e-06, 
       7.0780287595185447e-06, 
       5.3714182388177623e-06, 
       5.1993421118583157e-06, 
       5.1988368230429119e-06, 
       5.6522507644294218e-06, 
       6.1688520561651427e-06, 
       6.1753075974036657e-06, 
       5.3136884644573691e-06, 
       5.2736364496354909e-06, 
       6.0685638005720477e-06, 
       6.0640786267204043e-06, 
       5.1427248988323829e-06, 
       5.9984515111420417e-06, 
       5.0963400284239101e-06, 
       5.8708076861354811e-06, 
       5.8708076861354811e-06, 
       5.2297965961232114e-06, 
       5.7742602699776282e-06, 
       5.1430840907286563e-06, 
       5.3564923847103657e-06, 
       5.6764080388336965e-06, 
       5.3615112038371476e-06, 
       6.0518823783181513e-06, 
       5.5416494614751916e-06, 
       5.2440934785646912e-06, 
       6.9153676455002985e-06, 
       5.6200506928412139e-06, 
       6.5528189594751042e-06, 
       6.0570678577339580e-06, 
       5.2158300621543786e-06, 
       5.4881107063749253e-06, 
       5.4868925487152638e-06, 
       5.0736626595713733e-06, 
       5.4697639361438365e-06, 
       5.3659917032428393e-06, 
       5.0509042688983174e-06, 
       5.7040468038228717e-06, 
       5.3493546857912489e-06, 
       5.4123326868080220e-06, 
       5.1516637481318055e-06, 
       5.7817818478067093e-06, 
       5.2636369715464500e-06, 
       6.4671042069658231e-06, 
       5.0269987095309869e-06, 
       5.3747526771421389e-06, 
       5.5692530210595150e-06, 
       6.6520755264999482e-06, 
       6.3416798435693794e-06, 
       6.1520176577603210e-06, 
       6.3152716796667773e-06, 
       5.1169800781997018e-06, 
       5.1247027482866805e-06, 
       5.4547653230141280e-06, 
       6.2283629759942392e-06, 
       5.3575585972832832e-06, 
       5.3541602955320290e-06, 
       5.0417279061294094e-06, 
       6.0218225757066414e-06, 
       5.1189256413122862e-06, 
       5.1957810253843831e-06, 
       5.2362997084908111e-06, 
       5.6689435073143479e-06, 
       5.6523345053025765e-06, 
       5.9669516693967838e-06, 
       5.2184882583348902e-06, 
       5.5603111161946228e-06, 
       5.5560471620436252e-06, 
       5.3714182388160098e-06, 
       5.4395051384972718e-06, 
       5.2918898758861294e-06, 
       5.2771353456805135e-06, 
       5.4243350326745304e-06, 
       5.0401657310148476e-06, 
       4.0252578715627750e-06, 
       5.0064093343017481e-06, 
       1.7764796718998617e-06, 
       1.7612593025801785e-06, 
       1.4703465156921496e-06, 
       7.0102565801250459e-07, 
       5.4952125159578815e-07, 
       4.4413304680654396e-07, 
       2.6390632080274210e-07, 
       1.4900000000000000e-03, 
       1.1100000000000001e-03, 
       9.5000000000000000e-04, 
       7.6999999999999996e-04, 
       3.6000000000000002e-04, 
       2.3000000000000001e-04, 
       1.8000000000000001e-04, 
       1.1650112442953340e-04, 
       1.0925892459976593e-04, 
       1.3999999999999999e-04, 
       1.2000000000000000e-04, 
       9.0000000000000006e-05, 
       6.4565029040009129e-05, 
       6.4553285355187372e-05, 
       6.9999999999999994e-05, 
       6.9999999999999994e-05, 
       5.0000000000000002e-05, 
       3.2759437135047733e-05, 
       3.2589798205594919e-05, 
       4.0000000000000003e-05, 
       4.0000000000000003e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       3.0000000000000001e-05, 
       2.1554666176653731e-05, 
       2.0267972565679051e-05, 
       1.6991900081786556e-05, 
       1.6980756748035617e-05, 
       1.4494491681495742e-05, 
       2.0000000000000002e-05, 
       1.3078854957329289e-05, 
       1.2981060310400874e-05, 
       1.0289303103144626e-05, 
       7.5676592890847977e-06, 
       7.5606399659211027e-06, 
       7.3427300098279983e-06, 
       7.3050782509370220e-06, 
       6.3510187923679768e-06, 
       5.9583746048616614e-06, 
       5.5622497613625651e-06, 
       5.5105058225924277e-06, 
       7.4822714577540800e-06, 
       5.5252858812979813e-06, 
       7.4986039240655044e-06, 
       5.4640457060693577e-06, 
       5.3565054791010650e-06, 
       5.3566441843630026e-06, 
       5.1724642728830022e-06, 
       5.1144350418702510e-06, 
       5.1313811293051870e-06, 
       4.8637366012252047e-06, 
       4.6040648752585503e-06, 
       4.5460361819040874e-06, 
       3.4584927137592618e-06, 
       3.5165959372625102e-06, 
       3.5147168986542081e-06, 
       3.6447617045410646e-06, 
       3.6447617045410646e-06, 
       3.8286227323576215e-06, 
       3.7691215018687954e-06, 
       2.7124915472117445e-06, 
       2.8741174442898025e-06, 
       3.7108982032601902e-06, 
       2.8630801502195835e-06, 
       2.8430128048376965e-06, 
       2.7735041521540279e-06, 
       2.7757899616820560e-06, 
       2.6945036493993428e-06, 
       2.2579552085063673e-06, 
       3.0520449246461583e-06, 
       2.1595063039457711e-06, 
       2.2641155882160216e-06, 
       2.2468271062172162e-06, 
       2.0582972483827890e-06, 
       2.6724949636628015e-06, 
       2.2651113789799418e-06, 
       1.8552895547679796e-06, 
       1.8552895547679796e-06, 
       2.6030091245393579e-06, 
       1.9778872001593187e-06, 
       2.0056884112992946e-06, 
       2.3680265510958201e-06, 
       1.7441728282254390e-06, 
       2.2903953182433812e-06, 
       1.7478132208700874e-06, 
       1.7049263690199195e-06, 
       2.0212043860516087e-06, 
       1.5581705149181762e-06, 
       1.5189119937452621e-06, 
       2.0527234569691342e-06, 
       1.5094967688334891e-06, 
       1.9904754673432901e-06, 
       1.4925720522507429e-06, 
       1.4814521746804570e-06, 
       1.5397295960846138e-06, 
       1.7991579567505197e-06, 
       1.3179786614497433e-06, 
       1.6437568925932885e-06, 
       1.3054820767242094e-06, 
       1.3754579538485801e-06, 
       1.3282369114948043e-06, 
       1.3178417256854783e-06, 
       1.2434519789256758e-06, 
       1.2854979399775411e-06, 
       1.2268053247015361e-06, 
       1.1965666862810739e-06, 
       1.2735227371049395e-06, 
       1.2761994777433087e-06, 
       1.1895170929407851e-06, 
       1.1018287841462368e-06, 
       1.1029955308837029e-06, 
       1.0875540790367915e-06, 
       1.0573990085997407e-06, 
       1.0460658066656411e-06, 
       1.0820788598592123e-06, 
       1.0464621256188476e-06, 
       1.0351449870859697e-06, 
       1.0589664488342222e-06, 
       9.8224697385682902e-07, 
       9.8069220831819925e-07, 
       9.7717699178510151e-07, 
       1.0336806273220803e-06, 
       9.7075603751990794e-07, 
       9.6088863455750400e-07, 
       9.8216114996507725e-07, 
       9.7856094801760923e-07, 
       1.0419838346575139e-06, 
       1.0387856340644167e-06, 
       9.2905083182433053e-07, 
       9.6955203792036565e-07, 
       9.2097201459277882e-07, 
       1.1256355488727203e-06, 
       1.1559937754997115e-06, 
       1.2051196138163077e-06, 
       1.1962045108381221e-06, 
       9.8866879036363509e-07, 
       8.9389008999263979e-07, 
       1.1748993296901304e-06, 
       1.0111935221460964e-06, 
       1.0403815419576801e-06, 
       1.0406329054678555e-06, 
       8.3398073854650977e-07, 
       9.3849455520494792e-07, 
       9.3556437278789948e-07, 
       9.3601947727292253e-07, 
       7.6331718933777414e-07, 
       9.0856895471284970e-07, 
       7.8303506526691024e-07, 
       8.8648830007897938e-07, 
       7.5922953949112471e-07, 
       6.9639579629548862e-07, 
       6.9294747632699254e-07, 
       9.1789295390847046e-07, 
       9.1273442188211514e-07, 
       6.6513446246233394e-07, 
       6.5215159048423341e-07, 
       7.1044843807000765e-07, 
       7.1044843807000765e-07, 
       6.4171657279271983e-07, 
       6.0754186409324534e-07, 
       6.4909375384922481e-07, 
       6.8119223559722105e-07, 
       6.2712506514131937e-07, 
       6.1013519200641486e-07, 
       7.2228528133403244e-07, 
       6.0616971630424106e-07, 
       8.0834696347463327e-07, 
       5.9033419547507954e-07, 
       5.8481169371658993e-07, 
       5.8383454590388789e-07, 
       6.1545147735764275e-07, 
       7.8062345651834652e-07, 
       5.8377410752089955e-07, 
       6.0100892570784439e-07, 
       6.0100892570784439e-07, 
       5.8706206593572406e-07, 
       5.8801051909872250e-07, 
       7.0266029726937314e-07, 
       5.7427888929582681e-07, 
       5.4431464619825336e-07, 
       6.7666356977204396e-07, 
       5.5688692077565137e-07, 
       5.5763251202411191e-07, 
       5.2412338472350442e-07, 
       5.1433879723248250e-07, 
       5.1433879723248250e-07, 
       5.9192476042725944e-07, 
       5.3776828990228831e-07, 
       5.3590183341312974e-07, 
       5.4978479827318256e-07, 
       6.6105082718853496e-07, 
       6.4968859756962652e-07, 
       5.1018346374810569e-07, 
       5.1496785029614463e-07, 
       5.1240960144310893e-07, 
       5.1212729607826611e-07, 
       5.0162673907769992e-07, 
       6.1667826486585486e-07, 
       6.1622409788405526e-07, 
       6.1581886529524829e-07, 
       6.1320666737290239e-07, 
       6.0981240883340311e-07, 
       5.4739175782054580e-07, 
       5.7264073939333235e-07, 
       5.8325269687776216e-07, 
       5.4855481105700098e-07, 
       5.4631323714147601e-07, 
       5.1767330415775869e-07, 
       5.1578299686586698e-07, 
       5.1709689763904068e-07, 
       6.1356443173398706e-07, 
       5.7559501445016441e-07, 
    } ,  
  } ;  

  static const double fper[2][MAX2][5] = {  
    {  
      {   -2.4081036632027459e+01,    -2.3011998397341813e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.5913757193750548e-01,     3.3757047137445625e+01,    -3.0829301981755390e-05,    -3.6967043184602116e-08,     1.7385418604587960e-10, } , 
      {    1.3492887316547764e+01,     8.3647398411275371e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5829929133401038e+01,    -3.5231595807488247e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1725480483067283e+01,     8.3263902248853919e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9578197631664931e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5644384831743210e+00,     3.9302096969298748e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0601910820500013e+01,     3.2863247644877447e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1754041159951725e+00,     5.7533848783435178e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5450731193236688e+01,     8.3931257745377952e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9252790656709635e+00,     8.3117707439163460e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3964227599009260e+00,    -8.2949343775876805e+03,    -1.8314205262310364e-04,    -2.8737815761448853e-07,     1.3601932637209141e-09, } , 
      {    3.3146863846028740e+00,     8.3624484718625718e+03,     1.2148344865959285e-04,     2.1344407124528431e-07,    -1.0124848916291548e-09, } , 
      {   -5.1009433764673355e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4382615569241883e+00,     1.6833175268143528e+04,    -9.7213287932304592e-05,     2.7067147816345391e-08,    -1.3342072504134434e-10, } , 
      {    5.8034095343364145e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5691415857940321e+00,    -1.6920680808781185e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.7292215785537479e+00,     2.6298323161836379e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9779913603899413e+00,    -3.3781427502964334e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8488748628508307e+00,     7.8604193938597496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9951613666434609e+00,     3.6048416402410197e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.8082792557309908e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6039646883569464e+01,     7.2117616656190985e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.3493741188806553e+01,     8.3283391087670516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7629637245001106e+01,     8.3290437406832007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3684102027517266e+01,     1.5540453090344225e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7619564145153788e+01,     1.5545055490023693e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9986100572607679e+00,     6.6387243776244804e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5264010220231356e+01,     7.2163640652985669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.6209837135489504e+00,     5.8849268314763094e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7560362728742831e+00,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4570266766226636e+00,     1.1506769756687036e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1895568486895876e+00,     7.1780144490564217e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2442332187359026e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7767064219215762e+01,     7.2137105495007590e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3274628810276198e+01,     7.2144151814169072e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2353086438300076e+01,     1.4967415802349607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3322111006846455e+00,     6.4280209260728570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2158625283471221e+00,     7.9356704550321392e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3215747610659587e+00,     7.7533529368907739e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0886730333782616e+01,     8.0001048048448065e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0686820672420939e+01,     1.4756712350797985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6190106840381535e+00,     6.8210418957658458e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7542222205121639e+01,    -4.9310321559205477e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9200813906902283e+00,     8.7217123944181149e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6068378849137840e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2269792472525891e+00,     7.1496285156461636e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5288241355453773e+00,     8.9040299125594775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5420110158798597e+01,     1.5542401974225884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5638816861130849e+01,     1.5543106606142033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7530797754852205e+00,     7.2309835462676137e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3977700064609643e+01,    -1.8048943355373922e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6021274436567223e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0076212829829590e+01,    -6.5589555997275966e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.7747536715571872e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9745513554986598e+00,     1.5149733320490970e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2579388026630578e+01,     1.5508997243046513e+04,    -3.5358819241625563e-05,     1.0091396772294991e-07,    -4.8258353817643211e-10, } , 
      {    1.5988744188224674e+01,     8.3155735135256709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8270021248003712e+00,     8.2888765228272132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2001717207330858e+01,     6.0633858897901282e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6965955508771792e+01,     8.3685063266230391e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4355624097365704e+01,     1.4392077314515254e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9452643428319050e+00,     8.3294027701920022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1906079529818676e+00,     8.3279800792582500e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6873838726473025e+01,     8.7861467442307840e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0136409281434911e+02,    -5.0795853803393243e-01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8835833520015335e+01,     6.0334743949479162e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0267525833883944e+01,     1.6833150887778011e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.9545712375719724e+00,     6.4434349812668870e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.9369925950906410e+01,     1.6655081649610518e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.3631870558065108e+01,    -1.3117911199455193e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5848583239012823e+01,     1.6693431265852661e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5064281510513382e+00,     8.3467156329263307e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1933751649540181e+01,     1.6659684049289986e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.3402770408182842e+00,     8.2949099972221611e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.6290734454326499e+00,    -8.3624728522280911e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1356070167849946e+01,     1.5576511337321404e+04,    -9.7017423205136323e-05,     2.6979881353745674e-08,    -1.3487516608467297e-10, } , 
      {    1.2430699183029574e+01,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0223825076798271e+01,     7.1803058183213871e+03,    -1.8767156988297380e-04,    -1.4949714670693654e-07,     7.0375553949860234e-10, } , 
      {   -1.3792189295369884e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1570892326882445e+01,     6.6916934748356716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1634544908931503e+01,     7.0563285111184005e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9985083820159595e+00,     8.1709570703846939e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0102604384733048e+01,     6.6255701823112013e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0316988484325842e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7763961709837737e+01,     8.3512333298561825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0586341974251413e+01,     7.1610937682476415e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0427210534655451e+00,     7.2140628654588327e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1349612970368739e+00,     8.4864257790655574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6448477434786963e+00,     7.5426494853391505e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4857297961975709e+01,     7.2670319626700239e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7253353394913610e+01,     1.2619238322379260e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -9.3387769934275493e+00,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0894506025827155e+00,     8.3260615924089434e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.0462911421779992e+00,     8.3313212570413107e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4794882409188639e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0327004627079903e+00,     8.2757223275139349e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0811675714362128e+00,     8.3816605219363173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0698753098487485e+01,     7.2114330331426490e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4808538722007619e+01,     7.2166926977750172e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9726697128162458e+00,     7.8063220341019651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7944443895546286e+01,     1.5732484147591094e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.2051518544718576e+00,     7.7894013532931849e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2035257348220973e+00,     9.1147333641110999e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0827031449479336e+00,     8.5044838434184021e+03,    -2.4952603857365287e-04,    -2.2334396661354104e-07,     1.0529183526336901e-09, } , 
      {    3.0074678515220317e+00,     7.1742479635609207e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0005279874357349e+00,     7.2478199125962783e+03,    -2.4933017384648460e-04,    -2.2343123307614075e-07,     1.0514639115903615e-09, } , 
      {   -1.0607166078161251e+01,    -1.5419111039189374e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3926034269985848e+01,     1.5020384899560799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1002638671109715e+00,    -1.2904209779595694e+03,     3.1025166708923645e-05,     3.6879776722002399e-08,    -1.7530862708920817e-10, } , 
      {    2.2463350268492974e+01,     1.4954261607036327e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0228398603969424e+01,     6.6633075414254135e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.8335538944065526e+00,     1.5720838787719499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5788718124189888e+01,     8.3114184279582696e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2518603867086014e+01,     7.2538777673567447e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4654614300108996e+01,     2.0775427837006646e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3938813005594168e+00,     1.1790629090789623e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3992617239092333e+01,     1.5385019935843527e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4405514701536974e+01,     7.7401987415774947e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2382144589432251e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8246323151773179e+01,     8.3290200572015747e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5332134635975185e+00,     7.7779361006917070e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.1725292596457120e+00,     8.8510608153482863e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.8702530701566662e-01,     6.0697767706769139e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7938159546431631e+00,     2.5161866692868654e+04,     5.5099462709043649e-05,     2.7747826224623181e-07,    -1.3197598027163788e-09, } , 
      {    1.8297318496286593e+01,     9.1791677139237690e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7683633224464341e+01,     1.6864631115441240e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.3620032183394173e+01,     1.6869233515120708e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.2584902516496648e+01,     1.4991998966150539e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2281328084962011e+01,     1.6640462168641468e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9182880593114753e+00,     6.7514094274891249e+01,    -6.1658603963510780e-05,    -7.3934086369204231e-08,     3.4770837209175919e-10, } , 
      {    1.3024315726394651e+01,     1.5489785192972768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9525253592127545e+01,     1.6328796229569934e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3321723398161875e+00,     1.5502939388286046e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3863483065719407e+01,     7.3717972197992640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.6313416247758894e+01,     1.5542028288429719e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7039457486899419e+01,     1.5595723387395150e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.7519771576199012e+00,    -1.6623625802312807e+04,    -3.3545480326445187e-04,    -5.3778927204437500e-07,     2.5465323413959487e-09, } , 
      {   -1.4123687383441727e+01,     6.2133149972934916e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.8117596256064230e+00,     9.4793684003938306e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5995913250680509e+00,    -2.2365705603351671e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3067082257278219e+01,     1.5540124457867776e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0255754974709587e+01,     1.9651048484649373e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7150893259444139e+01,     1.5545384122500143e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6702407823218479e+00,     1.6691139896587698e+04,     2.7379619930094113e-04,     4.6385518567517073e-07,    -2.1988239693041891e-09, } , 
      {   -6.5430604447688836e+00,     8.9171841078727575e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4921515602076141e+01,     1.5582569192081872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7519063311648772e+01,     7.5251061172388290e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9475278617356386e+00,     1.5999508324485532e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1744416398200261e+00,     3.1546870868086353e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2709837962199249e+00,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1558462061905919e+01,     1.6118092778018310e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2422730281814882e+00,     1.9187669708227034e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3573264722131933e+01,     8.1578391591852396e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7805800881221746e+01,     1.6721817199262918e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6011783395199188e+00,     8.8794467487585462e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0787707178535175e+00,     8.0182093328307928e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8144181052175483e+01,    -1.7272996766856068e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4858973346818301e+01,     7.1365106044467084e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0507208378171821e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1328547363578149e+01,     2.3869144515069351e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.9975137657160019e+01,     2.3873746914748819e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0919996786496128e+01,     9.6640244767877448e-01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.3861796568412826e+01,     7.7364322560819937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4535166666302217e+01,     8.9400783289618885e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3749297499821983e+01,     8.2490616209293021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0089277473794800e+00,     1.8024208201205099e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8147165131802112e+01,     8.5508601288782265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0668808167864027e+01,     7.2916151264709570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7888776435797123e+01,     8.4083212285209502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6992274667834160e-01,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6979738060700484e+01,     2.2541905131056410e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1677859341315385e+01,     2.3581258181579246e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4961611561607352e+01,     1.5935775259876948e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6201912737354505e+01,     1.5700488644524392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4229371275849232e+00,     2.1329907977851814e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4794007027421497e+01,     1.4445046411726446e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9719509774444795e+01,     9.5721886836167578e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0131318958422186e+01,     4.6400665825343822e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2436869580723990e+01,     6.1163549870013203e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1485287798911372e+01,     7.2140561519081057e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5977746038187306e+01,     7.2140695790095597e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5838740117747747e+00,     8.3910482317540955e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1282960952827221e+01,     6.6653850842091142e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4949794007114384e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0929474397350020e+01,    -8.6364983834280338e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.0287541885951352e+00,     8.2511391637130018e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2942258512690572e+01,     8.4280182195942161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4161738998666626e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6309007175113397e+01,     8.4177089383387283e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1169892085280964e+00,     8.4062436857372504e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2300374513362447e+01,     1.4339108217304063e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.9703775214007422e+00,     7.1344330616630086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9300151880564975e-01,     7.2271807766582879e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7206823122856271e+01,     1.5465202029171833e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7224281455060432e+01,    -1.1944470569373675e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0002208879069851e+01,    -8.8601554394617779e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.4949374227618542e+00,     8.2588016467051548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9355964961702199e+00,     6.0104167925789370e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6112819556708340e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0735360665732102e+01,     1.4128404765752441e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7663299645309742e+01,     1.4916747904978150e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6241560790243748e+00,     1.2036460728798950e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0398810484795863e+01,     1.5542754290183959e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2209354118117552e+00,     1.4363691381104996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8686237813177948e+00,     6.0349999563798701e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3100388686737785e+01,     2.7511467878509129e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.5738581442230952e+00,     8.3200549263416979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.2055202359399942e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0162242217421550e+01,     8.8273902472144641e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3646386554591587e+01,     1.4994076508934240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1138140679777809e+01,     1.6657030533492180e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.9985154069433932e+01,     1.6657735165408325e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4707468704543638e+01,     2.3296107227074735e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.3565421113198806e+00,     1.6900689362418419e+04,    -1.5887189189581539e-04,    -4.6866938552858840e-08,     2.1428764705041486e-10, } , 
      {    7.9803270412310621e+00,     7.2133515199919557e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1243031092976466e+01,     7.2147742109257097e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6792896847214745e+01,     6.9596314257829290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3805713101952385e+01,    -1.1489987755645416e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.5450906897289061e+00,     1.5506705873781548e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.3806504606487096e+00,     5.7997133410273145e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3492731195404950e+01,     7.2936926692546567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3640976487894093e+01,     1.4912145505298682e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.8795903765069015e+00,    -1.3971962192534707e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.5580398999621249e+01,     7.7800136434754077e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6480530863117256e+01,    -8.3226579503301782e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.1191639551834825e+01,     8.3347248991200740e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.9882682108018965e+00,     5.8564776908983958e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3685037381979814e+00,    -2.1422382696247801e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.5083043301240657e-01,     7.8025555486064641e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3053666461345669e+01,     1.5620306551196083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4009524725869170e+01,     6.5854029644552147e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1142115814243446e+01,     9.9652096533097465e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1336562106886085e+01,     8.4292412282353416e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1446462517615482e+01,     4.4092271690706291e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2604505425825389e+01,     7.6070838351518205e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4934944693277583e+01,     2.3837688667771639e+04,     1.1695393139972269e-04,     3.5132508215283631e-07,    -1.6689226158514667e-09, } , 
      {    2.1534307542062709e+01,     1.5578802706586370e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.1659586869117707e+01,     1.4914094389180342e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0291643271617337e+01,     6.6070851883911409e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7630740100527742e+00,     8.1643981147849672e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.5797953855002561e+01,     6.2795526908746854e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6256820985279358e+01,     1.6065123680807119e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.6271879413910391e+00,     1.5463124486388133e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.5515981878881604e-01,     8.3598698282396108e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.2483808674326404e+00,     7.2206218210585603e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.6594233514444042e+01,    -2.0571522720911162e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.8433083623729924e+00,     2.8168728117789897e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1125890711076629e+01,     8.0742599850492225e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7180412374929990e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9854351518087494e+00,     8.5639780400776817e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4453987993466704e+01,     7.1915209603277763e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3041154229245624e+01,     2.3085403775523107e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.1672533073358927e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0166304127530239e+00,     1.4521425735445428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0424051062097236e+01,    -5.9000540864171001e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.0007427002014619e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3070166129412943e+01,     2.3871093398951012e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.7999990411167730e+01,     2.3871798030867158e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.2139125995076441e+00,    -8.3456121055339063e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.8763274941819472e+00,     8.0934048093725705e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1427646899761669e+01,     8.8773692059748446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.5502439936494596e+00,     3.5923954219002314e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.4346049862447980e+00,     2.9424634157381547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4522861438148949e+01,     3.1441677575438998e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.3622191290403393e+01,     1.5542761003734682e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3840836423290327e+01,     1.5542747576633234e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6620532866094049e+00,     6.1927343107203023e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1545395242425442e+00,     1.3763769729469712e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7964990649256830e+01,     1.5525833609375179e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.3889737289337244e+01,     2.3907494131311494e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.9926601525753576e+00,     8.5831228644010298e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3298812210333367e+01,     6.5861075963713638e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0606379083950593e+01,    -1.0816711359721012e+01,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    9.5714426100404157e+00,     1.6264361879757264e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9148390277785992e+01,     1.5288322850508055e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4656650423360320e+00,     6.8127668225673460e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5307274918711151e+01,     6.5880564802530243e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.1745843140816552e+00,     7.1960386572576272e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4249428016978261e+00,     1.7260154635030553e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2369671976041907e+00,     6.5497068640108791e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1097049872179591e+01,     8.2937465357151395e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.3520257437861893e+00,     8.3073615167472744e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7831401952939565e+00,     8.3500213327029778e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2646323486590452e+01,     1.4935776613116268e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6758923449520591e+00,     1.6082044361615901e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.5663194822169075e+01,     1.4914799021096491e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7510834130922319e+01,     1.7507859138648895e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1208174105782280e+01,     1.0958809679741535e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3912779710090160e+00,     9.4263993031826394e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5878444628727671e+01,     1.5622384093979783e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5822163842573271e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7564912444635443e+01,     1.3968541098505921e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1330086479591298e+01,     2.3478424745216096e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.2835866080804365e+01,     1.3498673398216670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6088895109916358e+01,     1.7114838168955906e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3711622471173822e+01,     2.3905202762046531e+04,     5.5295327436211918e-05,     2.7739099578363209e-07,    -1.3212142437597074e-09, } , 
      {    4.2464555197397019e+01,     7.2313358622256892e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.6442011596211108e+00,    -1.6899670471006452e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.5996519889699663e+01,     6.5834540805735560e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1072243627182735e+02,    -1.7309684784268029e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.2816622981182384e+00,     7.4684943051347364e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5117169105272268e+00,     1.6131246973331588e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3597416154407494e+01,     7.0382704467655567e+03,     1.8333791735027191e-04,     2.8729089115188881e-07,    -1.3616477047642427e-09, } , 
      {    3.5823190268198015e+00,     1.5478319940371290e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.2756626365713455e+00,     1.7050403819143241e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.6097937460137448e+00,     7.7217137476574344e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9846217108107655e-01,     6.7566075459531758e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6072275218558030e+00,     1.0447387812463210e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8006788451782825e+00,     5.4350783047445857e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0423209399545627e+01,     1.5542425657707512e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0204581538096459e+01,     1.5543082922660407e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8933583738331670e+01,     1.7900880108341884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5934178050855509e+01,     8.2353914201973639e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9550532307785083e+01,     8.9438810985712153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2564583541102330e+01,     1.0358230623002733e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.8598559704971009e+00,     8.5044594630528827e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2111208934507767e+01,     6.7446625720468637e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8067266306988969e+01,     7.2137342329823841e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3479640627782281e+01,     2.4047238133602361e+04,    -3.1571415979703382e-04,    -1.5939704207519326e-07,     7.4418900050313755e-10, } , 
      {   -1.2559847920211876e+01,     7.2143914979352812e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2623539211901912e+01,     2.5161842312503133e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    7.2233179461192352e+00,     1.3816738826680903e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9170253707857174e+01,     8.3266138819414264e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7022974610287250e+01,     8.3307689675088259e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3102004304400356e+00,    -1.2903965975940507e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.3364610738335863e+00,     7.2478442929617968e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.8833297244922971e+00,     1.7232721337284602e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1434908307455419e+01,     6.5864666258801662e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8157884648811930e+01,    -8.1204598547628848e-01,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    2.6060505670052780e+00,     6.2830195524848227e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.4898640127978135e+01,    -8.3160721864027473e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.9609782696343686e+01,     8.3413106630475049e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.3492005631344231e+01,     1.5607188639996630e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1188984607570241e+01,     9.9007753034970774e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7512112489511367e+00,     9.5077543338040887e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0448953056229682e+01,     8.3640284956606611e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5945358555505807e+01,     3.5371887272368867e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9418908604832614e-01,     7.3073628699865958e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2500131294085783e+01,     8.0310922092562214e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3458117363937790e+01,     7.2119853226751329e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2682546897237600e+01,     7.2161404082425333e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9641188245813147e+01,     8.3654511865944132e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9021940580558478e+00,     1.2139553541353828e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2785900004886748e+00,     6.5213209306006211e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0356140380763655e+01,     1.8293901078034873e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5186625320431993e+01,     8.3237603925692056e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8009105994537634e+01,    -1.9488838816592988e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7954648028927089e+01,     6.9994463276808419e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.3997269866739401e+01,     9.5171832084254220e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5621505421654085e+01,     6.9198165238850170e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9775359704987601e+00,    -1.9006704986522686e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.8438743391059148e+01,     7.2139648138849934e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0766309841592390e+02,     7.2141609170326719e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4207521753539153e+01,     8.8871092317506973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9622471317724099e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2036081663698992e+01,    -1.2451519525112520e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.8388400672834750e+00,     5.4880474019557778e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1836942856369475e+01,     8.3131493964366964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2780090692486837e+01,     8.3227913706387099e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.3984743144364899e+01,     8.3442334530135558e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9172591935976744e+00,     7.3783561753989925e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8345846748817280e+01,     1.6644264938250795e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2419650786474841e+00,     8.9356691017928169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7391155914085655e+01,     1.5508972862680992e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -2.4995071847653136e+01,    -1.3506511271354015e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6950274663185482e+01,    -3.6082627342699197e+01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.3937676306565500e+01,    -8.2483293168041200e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.7506713934098137e+00,     7.7627406467085511e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2622513882911381e+01,     1.4880695287798031e+04,    -3.2694937748717053e-05,     1.0029757560878723e-07,    -4.2814380992464229e-10, } , 
      {   -5.2920266220892174e+00,     8.0968018901802789e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6087833272548595e+00,     8.0132227160442617e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4742762210105091e+00,     7.8592911313131572e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5328078683142383e+01,     2.5193322540166366e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -2.1682331922233594e+01,     7.6980826398398494e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0791451008172588e+01,     6.5327861832020981e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3038951204778023e+01,     8.3336224568810467e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.6958786303768996e-01,     1.6617567947552339e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5975593376803971e+01,     2.5197924939845838e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.6332601533899414e+01,    -8.3467403680805000e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4466661303226477e+01,     1.5073353996771990e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9323250447075257e+01,     1.6697197751348165e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.2911303425853774e+01,     3.5902950880144147e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5938872827696473e+00,     7.1250453518452305e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3515690813157022e+00,     1.7789845607142468e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0171761277092992e+00,     8.2227532303027438e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2426926670740038e+01,     6.7051999861070262e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1056178988589060e+00,     8.4346296191475085e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3609478327431427e+01,     7.2353927734366844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5156674677376202e+01,     5.5356930186341742e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1891114727301021e+01,     7.1927329574809810e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5988602190719137e+00,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6328324701466901e+00,     8.7980917181370951e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0107747572691512e+01,    -1.1149808752243684e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.0904128876079852e+01,    -1.1142762433082185e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.5740963888452459e+01,     7.7690759452522452e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.5562684865935054e+01,     7.7736783449317118e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7016988592160303e+01,     8.3124820893832220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5847673133355517e+01,     2.3935880064721754e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1778664035621375e+01,     1.8686922047727858e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1300738958765852e+01,     1.6658094194917128e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.5459872231651861e+00,     1.6656671503983376e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4813115750387936e+01,    -3.2858661988031849e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.0826542932966836e+00,     2.8385933410258446e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8846922326168638e+00,    -1.4841705721395204e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.7468792119638323e+01,    -2.4272873609482303e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7054687592992739e+00,     1.2168002681931741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2297344078140718e+01,     8.1140748869471354e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3778977784224050e+01,     1.4915158050605292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6392239079940181e+00,    -8.2926430083227151e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1646254662708300e+01,     1.1035273924221087e+00,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    3.3016120275954698e+01,     8.2534403635527378e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1194826141922807e+00,     7.3849151309987192e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2489856346966683e+01,     1.4325954021990785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2610999268655588e+01,     1.0293796273190062e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.6427172818739599e+00,     2.2754515955803057e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0528217920802357e+01,     8.6441601334059906e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1757955324374443e+00,     7.2839526434788049e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3987021657620467e+01,     1.0751251592695720e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3195508473244127e+01,     1.4861477607927225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7461489320617247e+01,     7.1985208371704030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0657342890136976e+01,     8.3331006518941977e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6703691755943453e+01,     2.2720768739240382e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.8626923788377759e+01,     8.9546978099309345e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.1191405161224974e-01,     6.6026759612220712e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.9721246178217751e+00,     7.5226051188301062e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.5660943090183338e+01,     2.2759118355482529e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.7299577748309972e+01,     4.8701865665076184e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.4373333275060904e+00,     8.2485093313968173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6951816795870918e+01,     8.2716035260603567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3247856063940855e+00,     6.5857552804132893e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7367854487077647e+01,     3.9302096969298746e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7793798149085935e+00,     8.3985212686305003e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.8238341313181422e+01,     7.2296048937472624e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8874188051782577e+00,    -8.3095037550169000e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0320088777894089e+01,     2.3854525034100301e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.5985841844410107e+00,     8.3478790944333523e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.2635057188859244e+00,    -1.5076495289592810e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.8164866647599617e+01,     8.5433079625031169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5610361869028921e+01,     7.1971421846500516e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -1.0178596564603048e+01,     8.0569869882823677e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4132418796223210e+00,     8.4088735180534331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0130700439376170e+01,     5.9972625972656588e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7014371344673595e+01,     2.4983773074335644e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -3.8377375091809682e+01,    -1.5925960759164900e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8204011453540335e+01,     2.5022122690577788e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.2466125313678447e+00,     6.8985941567779691e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5500411735220858e+01,     7.7648181894922518e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0621472066353093e+00,     1.5797185729859861e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3411845398842193e+00,     7.7713771450919794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0064272901785834e+01,    -1.1997610547375871e+01,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.7546768224619310e+01,     6.3313238407373865e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1767675566143481e+01,     8.3908245746980610e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6109331740840727e-01,    -8.6359420039809891e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9710684145471561e+01,     7.7000315237215082e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6112879818143583e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6396202768830197e+01,     3.5460359810895170e-01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.7672146304474936e+01,     1.6093509614217375e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3693074795954850e+01,     8.4066883116163644e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1399196198663674e+01,     1.4948209382072921e+04,    -9.4353541712227819e-05,     2.6363489239583007e-08,    -8.0435437832883149e-11, } , 
      {    8.4314555842025598e+00,     3.8027696093255372e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8166139646947762e+01,     7.1802814379558667e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.5835195025501257e+01,     3.2375929558327487e+04,    -1.6340140915568555e-04,     9.1014072354693187e-08,    -4.4215007717189690e-10, } , 
      {    2.1934719102539955e+01,     7.4459524000036799e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.1403535939588814e+00,    -5.0435465820807576e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.4289308395458178e+01,     2.4988375474015109e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.4462220301638586e+01,     1.1769853662952619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9096140398379955e+00,     8.3110593984494699e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7495063143093560e+01,    -3.1480227663230835e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0672713471832873e+00,     9.5559973877790526e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7977186378268314e+01,     1.5248507948610144e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0310585493221289e+01,     1.5328137752405968e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2442515973339209e+01,     7.4286794032368234e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.8613843464992508e+00,     1.3970670411412008e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1861697551360129e+01,     1.6675407057651457e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1107397235404990e+01,     1.5540676747400259e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1696458930612554e+01,     1.6623601421947289e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    3.5043058697165378e+01,     1.5544831832967659e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0383521690867767e+01,     2.2181478667808442e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.1834364518711382e+00,     8.5382408905558477e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.6797956781194447e+01,     8.9930474261730797e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7387840439761426e+00,    -3.4944889009986269e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9892639578974762e+01,     1.4660015265462513e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3201126719701655e+01,     1.9079943017420846e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5975906322681572e+00,     7.5083092070326484e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4786780786536660e+01,     1.5033891410832151e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0996957863372955e+01,     9.3369020682642022e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4786032836500151e+01,     1.6650823893850524e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.7576178035236074e+00,     7.7010951851464597e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0371964864876821e+02,     8.3281834661870926e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9900853950426441e+02,    -8.3291993832631579e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8534123764891902e+01,     9.0214333595833377e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5055523168900544e+00,     5.6419789866868832e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2730344274237995e+00,    -1.6691164276953219e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.0499384003932807e+01,     6.3093741736172706e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6288457228425006e+01,     2.3349076324285925e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.2676040887507455e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6151368447222545e+00,     6.2702865717324257e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6036651070550050e+01,     7.2501112818612437e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    8.0647165080239702e+00,     7.1338807721305247e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6375864299123428e+00,     6.5055731870849813e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1351543022398497e+01,    -2.3029348371226774e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5934850881513940e+01,     1.6486530583910364e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0786475086981051e+00,     8.9562876643037926e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.3527648429668364e+00,     1.6499648495109821e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4824117342804463e+01,     2.3282953031761455e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0654978329561574e+01,     1.4914446705138416e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5929534661833811e+01,     1.5564084198161810e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5588805376327478e+01,     8.3444239088727172e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.8723874044860378e+00,     1.2566151700910864e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6351214892057666e+01,     2.3713711360568654e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.7144061826327404e+00,    -8.3352503803248546e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    5.5651886549332508e+00,    -7.6441243458667969e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6748333289656877e+01,     7.8198285453733197e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4276368342777333e+01,     1.5521424382206107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1514132820101596e+01,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3950387698082711e+01,     1.7268769753687015e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5974182955960693e+01,     2.4790585367493486e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.4580865130507501e+00,     6.9918941613057323e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6827449762827790e+01,     7.4857673019015920e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7609307541242856e+01,     7.7007361556376572e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6472133481217259e+00,    -2.2653055349349970e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0488833161906687e+01,     1.6815117203790684e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.4395468708524772e+00,    -7.6666324223391985e+03,    -1.8580593411601215e-04,    -2.8676176550032585e-07,     1.3057535354691242e-09, } , 
      {    3.8687368629626961e+00,     1.7579241869327598e+02,    -4.0183878921500109e-04,    -4.7375508104342746e-07,     2.2392574303087244e-09, } , 
      {    6.7993176979916292e+00,    -6.2567775272935887e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2470717522664895e+01,     6.6284150963689926e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7600689726956851e+01,    -2.8850395347046287e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4664977928098992e+01,    -1.6209335341904819e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0263257928438120e+01,     6.7470825087088679e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4033169607602689e+01,     1.0686817242883051e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8152828997345907e+00,     4.1643119620077772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5839734133038494e+01,    -1.1282625465931915e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0118883185734029e+01,     1.3367972634193943e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1604689250111690e+01,     1.5227285581503094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8790371150779137e+01,     4.3232306666228633e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1065319591602822e+00,     1.5559674970992739e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.2460174284028220e+01,     6.3711387426352985e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4449478635916879e+00,     1.6654753017134070e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1401806879337002e+01,     1.6660012681766435e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.9987256421008794e+00,     1.5871340910064278e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0426249399859804e+01,     1.6710351946661445e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.6546988817751345e+00,     1.2395292683746743e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.9633056645495124e+00,     1.4598977996457552e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3774115750146780e+00,     1.6604413752239059e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5409418909378044e+01,     1.1144272562388709e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6686701056922093e+00,     2.3831630813011172e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2562337523843286e+01,     6.0001075113234492e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0560916285054226e+01,    -1.1133368851508840e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1697312686388583e+00,     7.7003838396795827e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2231758066405341e+01,     8.6454796714299973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4641058246024848e+01,    -1.0903248867535331e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.5577948345637935e+00,     1.7443424788836230e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.3192335476816623e+00,     1.6496361397840737e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4917577690602275e+01,     1.4328798936048575e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5408337830684133e+01,     1.6679924754581309e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3392534140305283e+00,     1.6135013458827092e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0906726130384055e+01,     1.9472963987113835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4513782074691674e+01,     7.9225525438326831e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3694921765813461e+00,     1.6091432071433679e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9828017438935277e+01,     1.5527212261895529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0574127757825238e+00,    -7.9560465553304311e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7370339599882143e+01,     1.6879551553603353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3913175191204916e+01,     2.4446784202743434e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.1365147710166532e+00,    -6.9889778019972539e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8724649401561379e+01,     2.1970775216256814e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.6083187460411651e+01,     1.5542656238610120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1001865655682138e+02,     1.5542852341757798e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9390378245799198e+00,     8.0344450831513104e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0522513509599527e+01,     7.7376200979545338e+03,    -2.2647586299350867e-06,     6.8940505453776010e-08,    -3.2821886211115587e-10, } , 
      {    2.7301635192777191e+01,     2.3911260616806998e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.9885478535822187e+00,     1.4834264611810107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5599052726073829e+01,     6.6812248694345544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4847715499342097e+01,     6.5883851127294747e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5895871181735657e+01,     1.5558296318472387e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4940526277661306e+01,     2.3320690390875669e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.3432966633803474e+01,     1.6640109852683396e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1395007506367094e+01,     1.1452814815759980e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2870341344103698e+01,     6.1693240842125115e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4808789381091735e+01,     1.4699830167360424e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0149370352362139e+01,     3.3490558117593784e+04,     2.0741221335039190e-04,     5.2788937667611828e-07,    -2.5060988803914134e-09, } , 
      {   -2.4906857428799697e+01,     8.2092467190313891e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1010706451693851e+01,     1.1015106480668383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4061166602660421e+01,     1.4920438162112987e+01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.9256564653851360e+01,     2.3924414812120278e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9325687023994469e+01,     5.5228953248342805e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5455594100163461e+01,     2.3818476617697892e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1880248518386672e+01,     2.4657487654295062e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.7199860572265395e+00,     6.7434896347537206e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2590045956441571e+01,     8.8375543040769335e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7374956232596706e+01,     2.2756464839684719e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.3676384375980703e+01,     2.2757169471600864e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.4264862535327389e+01,     2.2291439747319600e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.1093469717044167e+00,    -1.1634615070219354e+00,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0939808890051534e+01,     8.3014614626608582e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5888522618534964e+00,     7.2490077544688183e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.3659559670877819e+01,     7.6329431902770966e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3352300462622559e+01,     7.1517060584298642e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5201247333279486e+00,     4.7320306302129484e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7776663161710911e+00,    -8.2822907588997823e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1157153593472787e+01,     7.2942449587871397e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3575783121531941e+00,     8.3455521714193092e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.6958299045804006e+00,     6.9143419002936080e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9047185387484873e+01,     8.4481361304188631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5229044374894288e+01,     1.4498015508937637e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5431463493750773e+01,     2.3868815882592899e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2488985228617784e+01,     8.3750920905504718e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.3038976356279662e+01,    -2.6535157978081543e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1873524133330307e+00,     1.4874631803240503e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9497278509532883e+01,     2.3874075547225271e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5794764080687630e+01,     9.5154168167962416e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1623478353593278e+01,     8.3106424813697522e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5254595648432918e+01,     7.2226993638422609e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0577336024596748e+00,    -3.9583408781527396e-01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.5857634356170713e+00,     6.5459403785153772e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7194769520082513e+01,     1.4917076537454601e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2945813092465022e+01,     2.1228392028053695e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9980397508797161e+00,     1.4425824531077931e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1410130268503064e+01,    -1.4511511601805273e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4826210718630374e+01,     1.4957106521094120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.4674940244872545e+01,    -9.8166181572923961e-01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0075175948195776e+01,     7.8423704505043761e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7909391326431878e+00,     7.7430436556352861e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0871070594363044e+00,     8.6229377662989409e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.3760318873965247e+01,     7.7710248291339039e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.7298608655610877e+01,     7.7717294610500539e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8028982190236139e+01,     1.3852787243083314e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7882974952989228e+00,    -8.9232363328361625e+03,    -1.8047817113019514e-04,    -2.8799454972865121e-07,     1.4146329919727039e-09, } , 
      {    7.4074740176280081e+00,    -1.6429330994015925e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.5541906159248953e+00,    -1.7632026275657509e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0428116828415408e+01,     1.5462572196855652e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0266950584013223e+01,     6.5520038630729050e+03,    -1.8500768839006530e-04,    -1.5011353882109919e-07,     7.5819526775039211e-10, } , 
      {    1.9436460835019748e+01,     8.3593443470348284e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.0299918634183378e+01,    -8.3129589405775332e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.7317248083959026e+01,     2.4264466684602074e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0064769295929626e+01,     1.5072181059478848e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3305316121329142e+01,     1.4430426930757400e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -3.6934620686463731e+01,     1.0973555624994374e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1601660257860740e-01,     6.6205900238592767e+02,    -3.3493183474663908e-05,    -3.6350651070439449e-08,     1.1941445779408981e-10, } , 
      {    6.8931038394037838e+00,     2.0426571094770613e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4737750976235269e+01,     1.6657054216973800e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6992528017203572e+01,     7.7229257448106400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6766239554686997e+01,     1.6068890166302621e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2420080742763069e+01,     9.7299230379571891e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5890873411876346e+01,     1.6657711481926701e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0525972982671178e+01,     1.7179752240073412e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0582505358952176e+01,     8.7085944832186578e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6503998318514945e+00,     8.9569990097706686e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2962353125400503e+01,    -8.3441105357643155e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1206352984071616e+01,    -1.7085226553988696e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.8860846786088183e+00,     1.6106627525416832e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.3662641578179979e+00,     2.8449140577913781e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8193020333101710e+00,     4.3770833509159729e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0212845236710574e+01,     4.7162516363158493e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2329072948699284e+01,     1.9865984956806824e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7016322948173826e+01,     2.2363796185949803e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.9113931487574497e+01,     7.0392612295787540e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.9729200533721958e+00,     3.2197835939794481e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    8.1431331025739500e+00,    -1.1675976564774849e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.2188542813829439e+00,    -1.6369979498894538e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.8655675383793771e+00,     1.0054706888228182e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4452833659446869e+00,     7.6474147424683915e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1952362162823928e+01,     8.4706780355499195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6341788515267446e+00,     6.9853352057060038e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0102078633115344e+01,     1.5757370827961951e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1076406923102180e+01,     8.6852945733279103e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0015716600832599e+01,     9.5323374976050218e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1122949592707982e+00,     1.6730463662918642e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7198295439528355e+00,    -8.9737562790106324e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.4251355727773447e+00,    -1.0616594620551020e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.7120940655779027e+00,     2.5229380787143542e+04,    -6.5591412544671441e-06,     2.0354417587702759e-07,    -9.7205143062461952e-10, } , 
      {   -3.2870949905407834e+00,     3.1083207380528183e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.6877799507548065e+01,     8.6284123898903508e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2330592140236760e+01,     3.2202438339473945e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.0668567904267028e-01,     8.3446909330496110e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5455572325600945e+01,     1.1079838212576040e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7150307418151062e+01,     2.2773737836451575e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1687267805233660e+01,     6.9787762501062771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8016427739141847e+01,     3.1087809780207655e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.0113985455517387e+01,    -8.9042248767769706e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.3391005198079348e+01,     8.6056647695320844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9113376491585544e+01,     7.0946181597650957e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2411536623643666e+01,     8.3129436812094864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5698305326836621e+01,     1.1371704643973499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8676198392916962e+01,    -1.3245303410550900e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.2383284429461128e+01,    -1.3238257091389401e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4637352489352740e+01,     2.4969153593366595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7742853744353518e+01,    -1.7031212424139244e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0236972165827829e+01,    -8.5021924741534367e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.3578092738239489e+00,     7.7341465166140897e+03,     1.2414733015250138e-04,     2.1282767913112163e-07,    -9.5804516337736496e-10, } , 
      {    9.7865348273800166e+00,    -1.0808715121288478e+03,    -4.0164292448783282e-04,    -4.7384234750602717e-07,     2.2378029892653959e-09, } , 
      {   -7.7050996801879791e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9441454568579999e+01,    -4.6446193678602299e-01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.2657865087530752e+01,     8.3286847111744009e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.4805167740064945e+01,     8.3286981382758513e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6831783347794662e+01,     1.1537293532081698e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0332698352681081e+01,    -3.1880355101566238e-01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1344218060495180e+01,     8.0171720863844548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7044059827059833e+01,     1.4948578398967882e-01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -6.1075315553388752e+00,    -2.4952317227037933e+04,    -4.8776755390580009e-04,    -7.8820038647426142e-07,     3.7328714190709830e-09, } , 
      {   -4.3727384332328256e-01,    -8.2611773304502349e+03,    -2.1397135460485901e-04,    -3.2434520079909064e-07,     1.5340474497667937e-09, } , 
      {   -2.4093775729468717e+01,    -5.5260821264806509e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0762303532579736e+01,     1.6799418221006083e+04,    -6.6383985950549209e-05,     6.4034191000947506e-08,    -3.0727491108722393e-10, } , 
      {    4.1438585252690547e+00,    -5.9454490811103642e+02,    -2.8165420488846872e-05,    -3.7583435298764782e-08,     2.2829391429766939e-10, } , 
      {    8.2485851019236858e+00,     2.4356780791700485e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0585986959674990e+01,     1.6168760675389767e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2738351266475938e+00,     8.3962055190000174e+03,     9.0654146677837468e-05,     1.7647702806068220e-07,    -8.3863070558327521e-10, } , 
      {    1.2818130396812403e+01,     1.5383024512689856e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -7.8095877046683633e+00,     9.2114304494465723e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1724064922362576e+01,     8.3265491864555006e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.0129575881015320e+00,    -8.3308336629947516e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.2767902040104211e+00,    -1.2723629136067254e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2277027466536174e+01,     1.6727212693355628e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.4815639349308114e+01,     7.2347941712979718e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5177366399675389e+01,     8.4683768357101853e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4011687773906061e+01,     1.4365768923888696e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5719738737122437e+01,     8.9504400541709419e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.8740395011449440e+00,    -1.6834175672870383e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0166068282380168e+01,     1.7808059825118955e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.7275346236745982e+01,     4.2923307555598194e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0965550104239981e+00,    -1.8995165129313668e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1736999644421674e+01,     1.5778040905536514e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.7160715251976796e+01,     6.7135507265447814e-03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1679853199160764e+01,     6.0370774991635708e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1276800992203482e+01,    -8.3418093359245813e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -4.1827790108888241e+00,     1.7245875532597882e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4370052876257814e+01,     2.5158601724983564e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9238804546940518e+00,     7.8210405425265244e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9725678712124207e+01,     1.4427773414959591e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1612430254075926e+01,     1.5542042944717083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1316016748803861e+01,     1.4428478046875740e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.4224356543679999e+00,     8.4308631336520066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0257951800408236e+00,     2.5019831321312828e+04,     4.2610894994228935e-04,     7.1426630010505710e-07,    -3.3851630469792241e-09, } , 
      {    1.3607132206670929e+01,     2.5231672156408509e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.1593676691798219e+01,     2.2692382805830126e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.3454634200539131e+00,     8.9143391938149653e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.0436533201177625e+00,     6.6195179573477963e+03,    -2.4666629235357607e-04,    -2.2404762519030341e-07,     1.1059036398421514e-09, } , 
      {   -1.7597878357486920e+01,    -2.8522768634084628e-01,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.3632330371883910e+01,     6.4109536445332105e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1589399239123164e+01,    -3.7387614536963997e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7445677174341387e+00,     7.7003894694766441e+03,     1.5497663213425675e-04,     2.4979472231572374e-07,    -1.1318993494232446e-09, } , 
      {    4.3465492346025492e+01,     8.1538897888450474e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8544424663755031e+01,     1.5543465635650835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2388516963436160e+01,     2.1606140179974089e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8548331853716977e+01,     2.4029180069249516e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7991004863139963e+01,     7.5255822037995040e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9512777151750269e+00,     1.5837000631757774e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0620764828560642e+01,     2.2723060108505346e+04,    -2.5385969110635476e-04,    -8.5550222168588733e-08,     3.9502618736804976e-10, } , 
      {    9.8104278234733595e+00,     3.3406124347172772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6269455940643802e+01,    -5.3813608571104010e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.8196324159585768e+00,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7327369857603753e+02,    -1.4014469809389993e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2106669919318506e+01,     6.2356807028968433e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5233025673321038e+01,     1.6872965789675924e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.8650204740503842e-01,     5.6431786111198778e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9541848576936136e+00,     9.3500199794636574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0006796956715682e+01,     2.3245439329703277e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.0086667256807793e+00,    -5.4297050329932972e+00,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.6370003653203135e+01,     1.3642083791531692e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4703235391899041e+01,     2.2669888994440441e+00,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -6.2605774965301705e+01,    -2.2994648423475041e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3494385423903411e+01,     1.5622936383512266e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3751399597523786e+01,     2.0259005926499809e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8983043751441659e+01,     9.1716155475486612e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1958474922577103e+01,     1.7208138173483672e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0161249904257435e+01,     2.5050508623988044e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.9333566006684755e+01,     8.5034930606052058e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4265161459618618e+01,     8.3283323952163246e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0129377865316826e+01,     8.3290504542339277e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1438404734693322e+00,     1.6860746694183149e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.0718030804516928e+01,     1.6173363075069235e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0446995320264030e+00,     8.5574190844779532e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3504775660250202e+01,    -9.3300004527762781e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.1367819917401420e+01,    -7.0463191614976495e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0472018956960110e+01,     1.5536195334584230e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.1635292983107163e+01,     5.1092726060088371e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0552778853523407e+02,     1.2695894585909855e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4028084581082787e+01,     1.5529636378984504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.1444770109284313e+01,     7.2321118088142066e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2960241698587032e+00,     9.2255392661935866e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8480059968759956e+00,    -4.2680239719595193e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.3143479501527011e+00,     1.3735383796059454e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1684574478827768e+01,     1.0677292205638742e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.6610124621023914e+00,     2.8286990488630363e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3244325967411577e+00,    -1.1172583915824780e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5981304872738448e+01,     1.4739645069258337e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2439952914701177e+01,     3.9324626244204808e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3603363746978975e+01,     7.3888645013389123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2738168769828992e+01,     3.0510170092533568e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2472489730066492e+01,    -1.1119987269501098e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0272445478159732e+01,     7.0790761314766660e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3309881565317010e+00,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8707162199426177e+00,    -1.6653838832997126e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.1100716757070490e+00,    -7.2160758957330597e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.2350589202275248e+00,     8.1889718027997787e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -7.5510058390353322e+01,     8.3279867928089770e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6528674119130393e+01,     1.6913332981106319e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.9645920596788017e+01,     8.3293960566412752e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8987564852116080e+01,    -2.5333555870792952e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0837755642702489e+02,     8.3269604562466993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3640272555003888e+01,     1.6886119984989204e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.0039102947600281e+01,     8.5359396907161135e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.2195175739656687e+01,    -8.4738065407431786e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.8113728371255036e+01,     8.2317886241624456e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6480329288810232e+01,    -8.2665582747521894e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4131509129838378e+00,     8.9907504271110556e+03,     1.1881956716668435e-04,     2.1406046335944699e-07,    -1.0669246198809447e-09, } , 
      {    8.1433897233917740e+00,    -1.9187229332080515e+03,     3.3689048201832156e-05,     3.6263384607839732e-08,    -1.2086889883741836e-10, } , 
      {    1.1385068336969692e+01,     9.7430409491566443e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4223491090982879e+01,     1.5307467674831403e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7690798345071386e+00,     6.3861686630103100e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5431794000536811e-01,     8.3284677676690917e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3843201508992191e+01,     1.0122944007650178e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5210820873550379e+01,     1.5214167670303641e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.0967662421390152e+00,     8.3289150817811606e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1285349129533458e+01,     2.3240836930023808e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.5239728991546091e+00,    -8.4684110466504735e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4516088115536014e+01,     4.5745531950565646e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5163268436221493e+01,     8.4039424858975144e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5454543420546250e+00,     7.2386755892333372e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2950522983025303e+00,     6.8800016219871050e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5929160906476127e+00,     6.3635865762601888e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1546115131774364e+01,     1.7346683848802240e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.5964103973368537e+01,     6.8366451523188516e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3025368508843025e+00,    -8.3126919164006413e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.2950844567169897e+00,     4.2659815955703628e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3826686413540157e+01,     2.6308390626558236e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4632062495264435e+01,     8.1937046907429594e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6877987105075938e+01,     1.1472859182269027e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1264405813228205e+01,     8.5405420903955819e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    5.1470798287941278e+01,     1.5367952654303879e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4165658164624579e+00,    -8.6750524714155702e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0050264591055210e+01,    -1.5747743515638831e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4660779789067879e+01,     1.4035104761224677e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4240735051945922e+01,     1.9313234209619732e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8794920688726569e+00,     1.0309281255487740e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6769539033663005e+01,     1.5423309584490224e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8307501634237006e+01,     2.4047213753236840e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.7565937217268775e+01,     2.3542859095028769e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5794128325401029e+01,     2.9088811421913433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4651664149500824e+01,     7.3335075711525697e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0504605328414796e+00,     8.9569933799736082e+03,     1.4964886914843974e-04,     2.5102750654404910e-07,    -1.2407788059268243e-09, } , 
      {    1.4130807460876257e+01,     7.4493494808113883e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3793581507975111e+01,     1.3869707923892094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7402660214944237e+00,     7.1147360705897427e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6675797121722979e+00,     8.3288398417823391e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.1485919879998292e-01,     8.3285430076679131e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.7918262328686474e+00,     7.0382948271310752e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.1949442985075187e+00,    -7.9694518825351033e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.0363732120776135e+00,    -6.6113995532934311e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.0499589334064943e+01,     8.4636781587072928e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4674577030534971e+01,     2.2667799642029193e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7412589323085502e+01,     1.5648692484606341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4463286315997273e+01,     7.1118911565319522e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0912546019519329e+00,     7.6202017463512748e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9507073488584243e+00,     3.5337070935533859e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1278918736200156e+01,     1.6149731967251650e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8254136195877582e+01,     1.1930314501774683e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.4062253451382887e+01,     6.2831429859626996e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4872326086476040e+00,    -8.0470041435472267e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.1391547442948157e+01,     1.6577753045654426e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.9501174387896619e+01,     7.2091318333029121e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4313199588862808e+00,    -2.1102199644829902e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.3025339717195301e+00,    -7.4883953726266327e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1082858359776548e+01,     3.2217200185560241e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.7845202509496261e+00,     1.1292505667315709e+00,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -8.2513415107289223e+00,     8.3132723136859368e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0246665639032933e+01,     1.6737012653246078e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.6525507655566081e+00,     1.1926254461575309e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9556541169663390e+01,     2.3793893453896959e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.4492235680489323e+00,    -1.7350104942831140e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.5173702539529039e+01,     2.0652026896192794e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4625933537117479e+00,    -1.9268871913653845e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.4538728268147141e+00,    -2.0764385369800257e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.5245218236525162e+01,     6.5857485668625623e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9652758081006976e+00,     5.4066923713343276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9240894296302944e+00,     9.0371811030989775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6290375302600655e+00,     6.5082030194011659e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0959643921064064e-01,    -1.1544434611642057e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4143362999696011e+01,     3.6759761869286535e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.2240837881585200e+00,     8.3783548221596702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.5203460193201170e+00,     7.6834631588708016e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3579215851375352e+01,     6.7450148880049383e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7187187317420314e+00,    -1.7171989947900026e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.0756774302181071e+02,     2.0355170170754775e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3184076973162897e-01,    -1.1020626722364071e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1424856692831137e+01,     5.5410164991669690e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8120738826548831e+01,     7.2075039098591033e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.1197971094340110e+01,     8.6103787059030255e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -3.4764301122633128e+01,     7.3490495994409994e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4153980632000271e+00,    -4.5779038050340205e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3057718141731531e+01,     5.5022935757018258e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0906037474489334e+01,     8.6879309669151498e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.1079029066134080e+01,     3.0299466640981940e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.7145309959167271e+01,     1.5537823258028038e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.2935822919141899e+01,     7.8400692506646410e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9023125529398026e+00,     9.0099681069818598e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.0091020336170864e+00,    -1.1139172137994171e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5046002105276788e+00,    -1.1153399047331698e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.6710642731256158e+00,     7.6136427907515472e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.7218098560370407e+00,     7.4781665690567305e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1080823775796169e+01,     1.5547685322339883e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0487784925882195e+01,     1.6867260947757422e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.0140998558530345e+01,     1.6866603682804522e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.6454507925470924e+01,    -9.5601204367496393e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0214589797068044e+01,     1.7665593492989326e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6792867664700172e+01,     8.9397260130038121e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5240495298928527e+01,     1.7148619596458873e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.9708032678583773e+00,     9.3734302059714482e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1009911852822292e+01,    -1.1483856064037391e+03,    -3.3998432052432208e-04,    -3.9990826113682296e-07,     1.8900946171736371e-09, } , 
      {    1.3089992535777423e+01,     2.2457096190477565e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0260885190852894e+01,     8.5662380094277087e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2355690452072352e-01,    -7.0585984605766726e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8812312710296155e+01,     1.7272572523296341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6245202628028732e+01,     3.2236185556036620e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.0523193477948114e+00,     3.1178403514484216e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.8998859287807877e+00,     8.0338862323477715e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.8725235983633702e+01,     7.2189938976147532e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3647103703319310e+01,     8.0776570658569317e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9397452729366545e+01,     2.2790574202780233e+04,    -3.1551829506986555e-04,    -1.5948430853779295e-07,     7.4273455945980900e-10, } , 
      {    1.5408559317756131e+01,     7.3200010598812150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0716255595543680e+01,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    6.8456980935722322e+00,     6.2162777500338962e-01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.6469283922336881e+00,     7.6605689377816707e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0352569641578384e+01,     1.4314168059711651e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9226744927655908e+01,     1.4712317078690776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0976738792387286e+01,     1.4836894444126290e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8587122813990369e+01,     2.4393815105532245e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.7327472375067096e+01,     3.8814179353585132e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5840601521181625e+00,    -2.7229962064268911e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3359921729891440e+00,     7.1081246710364503e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4851012537880695e+01,     8.3544305598720493e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1430279170959480e+01,     7.1188910333745789e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7995798672996781e+01,     7.2784972152715018e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.5186707883346093e+00,     8.9220541207606839e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1242342190429262e+01,     8.8783884527392942e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5388575467396628e+01,     8.9795409149017250e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1853802265942910e+01,     1.6587493071430279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.0224024592647698e+00,     7.2096536382897621e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4947768423600023e+01,     1.7226171857470996e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.5796573185726984e+01,     8.2335195926408724e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1637766275880601e+01,     1.8058614462682315e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5025170228428415e+01,    -1.1341123129878883e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4391453631812498e+01,     8.3162399052000128e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.4561599140005526e+01,     1.6051617169535766e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6010451545008586e+01,     2.3322767933659365e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.6421770471076620e+00,     2.4078693980900072e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.3790854804314975e+01,     8.8341401345395061e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6803446555488097e+00,    -8.3411429442502395e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.7521339309177378e+01,     8.9573513257287450e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3799617207348511e+01,     1.5560803233539333e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7700050493132888e+00,     5.2165804515628406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2943390968557789e+01,     1.6171061875229501e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2575227456616433e+00,     9.5426467788605096e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5663359615262072e+01,     2.3243443906549601e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5023166803280677e+01,     3.1085156264409841e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.6035786491442025e+01,     3.1085860896325990e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -4.9527222910601864e+01,    -1.9907450948956102e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5925195586518370e+00,    -7.3596329400876584e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1956447480240801e+01,     2.2234447765019631e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1392521366080617e+01,     1.6579830588438130e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.0078677638500162e+00,     6.6207001694232749e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7393875259709709e+01,     6.4092682336792222e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2504864507033329e+01,     3.6147409882490110e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1661339085116111e+01,     2.4083296380579541e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.7217903748421058e+01,     3.3019021118843320e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8941544892730747e+01,    -3.1776990852060044e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9968254765040303e+00,     1.3710800632258521e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.1663370079524526e+01,     8.4238632568093799e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1848096323353484e+01,     2.5260058089818765e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    6.6910297962420788e+00,     1.4205957026764565e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5266271772363893e+01,     1.0515964977343167e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6844046669118393e+00,     2.0954946583072160e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    6.2630541494664897e+01,     1.5407767556201792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2414373008629630e+01,     1.4945316177598261e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2028588340679285e+01,     2.2599082801302360e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.3976002919172046e+01,    -6.2830087149481687e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6593962218498085e+01,    -1.2205924819586471e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.0519880326772828e+01,     2.2168324472495162e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7290500312727033e+01,     3.2166380092496765e+04,     2.6926668204107091e-04,     6.0173619658272278e-07,    -2.8552616935265007e-09, } , 
      {   -2.2656152465406546e+01,     8.3151849134537715e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1538697324485820e+00,     1.2352852621132345e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1576978302926589e+01,     8.3996847301375219e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.3997740762175024e+00,     1.4911816872822232e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6792311519449967e+01,     8.3421979359964807e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3273824780241572e+00,     5.8772656020394388e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2630874368883248e+01,    -4.1369104726755222e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8367012107022759e+01,     1.9045585916046548e-01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8929231215252971e+00,    -1.1080696036665659e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8300364502819725e+01,     1.1865880151962010e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9308224203125572e+01,     2.3242785813905470e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1293339556493940e+01,     1.7844108241521368e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4034979704820419e+01,     2.3279186546265952e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.3622178277558888e+01,     7.2711507641236012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0034649354698860e+01,     1.3277448755248961e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.9096234322495782e+00,     2.0150518784959127e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1198730664419962e+01,    -6.6211902271108738e+02,     2.8361285216015134e-05,     3.7496168836165066e-08,    -2.2974835534099799e-10, } , 
      {    3.7649662473071984e+01,    -3.1132458252141078e+00,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    7.4809062490868996e+00,     1.6734935110462378e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5943160433026341e+01,     8.5047628919864383e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.3830602239183527e-01,     7.2302722008007368e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.6339261659333451e+00,     1.5208693046712231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5808417207403451e+00,     1.5524705346828585e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2754745396549367e+01,     2.3871445714909085e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.8611323806072360e+00,    -1.2128443365819267e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2678464239861119e+01,     1.4307469028070724e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.0147362729598676e+01,     9.1318006456507483e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5377177418391152e+01,     9.4624477195850486e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7600784358572255e+01,     1.4914439991587689e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2333078173323919e+00,     7.1787257945232986e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5431099371930026e+01,     2.3948997975921211e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7062270795951079e+01,     3.1624798651799862e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.4125969030762047e+00,     1.1499656302018273e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9369857172980502e+00,     7.1960139221034588e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.4551700502188666e+01,     8.5658498676341733e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7234123843061099e+01,     7.1828844619443480e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.3035475262458100e+01,    -1.0636614249512490e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6096488311507564e+01,     4.6827203086537702e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1603043501226720e+01,     1.1513883211355801e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9744901999569073e+01,     2.3837664287406118e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.2029921077783637e+01,     9.1016154529116466e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5934630407074664e+01,     6.5857619939640153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3104829219711545e+01,     1.4607501902568620e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8570268900572643e+01,    -8.3277250222774474e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.4772163738660158e+01,     7.5481241089305604e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6569301300987034e+00,     1.0107675985439373e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4303354504567441e+01,     6.5061254766174652e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4035852186647491e+00,    -1.1844584031716679e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8362412622980443e+01,     7.8446716503441094e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5462456631033049e+01,     8.4733078678661041e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2805865441195508e+00,     2.3791815911113259e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.1371038557916897e+00,     7.7030136719957682e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6595934946021771e+01,     2.1045047865885783e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0615022693834618e+01,    -7.9853460851858472e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5876937273921566e+01,     1.6012154583595928e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.9308581916422938e+01,    -5.7087898664768545e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1778050252552292e+00,     9.9176968403512956e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8994226410047799e+00,     7.2199629195452499e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9598019122376322e+00,     7.1773031035895456e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3060518897438879e+01,     1.8451635432375304e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2976321669740571e+01,     3.1051751533230472e+04,    -1.0154694046500651e-04,     1.6486089226129769e-07,    -7.9131289030698470e-10, } , 
      {   -1.1935926394221298e+01,     1.4834816901342590e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0038913767835098e+01,    -1.0748136573683812e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.7047171840198956e+01,     1.5662198995877696e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4480136459416059e+01,     5.8953145453948109e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9371860643112889e+01,     3.0692487610674929e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0322085598616850e+01,     5.9787776033455975e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5998636885259101e+00,    -8.2642570749124570e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0890170310286255e+01,     9.3434610238639289e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4545929169598235e+00,     1.3577649441719022e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5677854649406644e+01,     1.6137299198294995e+04,    -3.8022700734534074e-05,     1.0153035983711257e-07,    -5.3702326642822187e-10, } , 
      {    1.1367810797159651e+01,     2.2850117160170554e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.0830005829298109e+01,     1.8362316463774732e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.9676420151966809e+01,     1.2323335471467670e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5936472789115484e+01,     1.5073706312730064e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.5923939922310979e+01,     7.3092346975430864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1673040869929132e+01,     7.2009449542593775e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.1263915564268515e+01,     1.5717555926064038e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8394689337692828e+01,    -1.1169297591060276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6959697322214211e+01,     8.2733344945387835e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4914516922154171e+01,     1.6000209609689613e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -8.5143198376807533e+01,     8.3044185511156447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0731548332585485e+01,     7.7184080478807882e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.2996591068265758e+01,     8.3529642983346093e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6906256398079108e+01,     1.6251243968557810e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5667983775325531e+00,     1.5576535717686924e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2512981718040196e+01,    -1.1921808202784177e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4839045558168698e+00,     6.2082941847648647e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3352005791203567e+01,     8.9566466938125941e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.2825291060843426e+01,     8.3840483549114670e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.7859155326164839e+01,     8.8999111111059010e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9008414659899406e+00,     2.3835397298506676e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.4830611373313944e+00,     8.2757318033000556e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.1260718521823736e-01,     8.3003054913148680e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2592466006655931e+01,     8.8974804327458714e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.2105282086527875e+00,     6.5831254480971056e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.2866259240666871e+00,     3.2407385405625195e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    8.2587714889524150e+01,     7.1587059352724909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.2205400538504847e+00,     1.6719739656479222e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2775461071899560e+01,    -7.0793738884136769e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6322650212598667e+01,     2.5178703059197316e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    7.4400673052020805e+00,     8.3570773581353842e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7678167121208088e+00,    -2.4583163800932923e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0597653602275443e+00,     8.2058325494939636e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4016894288271871e+01,     3.2411987805304667e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.2340690850358087e+01,     2.4986426590133451e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.8782587174523666e+01,     2.4985721958217306e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.5127061298274949e+01,     2.2809786252853984e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.2878634203830195e+01,     3.8878927938312700e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3928597548036892e+01,     4.0077619579419988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2961966568394281e+01,     8.3178747133654051e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.2779217659679148e+01,     2.3871117082432636e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.8495119458142257e+00,     2.3871774347385533e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0019284920462709e+01,     7.2200963398537788e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -6.2190937235695074e+00,     1.6648746351066824e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0976930841265158e+01,     2.2703848058431598e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1550036186152838e+01,    -9.5689420492586169e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0665273169095805e+01,     3.2375905177961966e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    3.5169174432544859e+00,     8.7992646554302373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0346745348610192e+01,     7.2137038359500311e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1193478904257317e+00,     7.4650972243270280e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4840482316928224e+01,     7.2144218949676351e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8718219195361900e+00,     2.4459938398056715e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.6061941184682226e+01,     8.1694318171334780e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3455996033108439e+01,     1.7215800656475825e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4794640366006028e+01,     6.4507685464311235e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6064931303457310e+00,     8.1114384933598958e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.8078332928477067e+01,     6.5702132521248595e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7677209507957272e+00,     7.2316948917344898e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2589010535906464e+00,     6.5850439349464123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0320163005068852e+00,    -2.4694595066730130e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.2977088666144985e+00,     1.5536720815789011e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.0634121593450629e+01,     2.2206061831609371e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.2546754460767282e+00,     1.6362577657072898e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.6933226247919027e+01,     7.2005563541874781e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7085921908715392e+01,     2.7173653603479484e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8508913218347796e+01,     1.6746400363063851e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6723256506560048e+01,     6.0768924010614828e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8011812014830618e+01,    -2.6660706584633132e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.8333325677117429e+01,    -1.1146218457155669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.1421395740589624e+00,    -1.3241780250970151e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.9129713124397099e+01,    -1.1146352728170200e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.9809976444791149e+00,    -1.0808471317633291e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.9091599899500240e+00,     1.3500097180706898e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6421610366846870e+01,     8.3296578271728049e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1102567973835630e+01,     2.2754187323326609e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4483203943823391e+01,     1.8844656402068293e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.1184705778586988e+01,    -2.0350143195108103e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0960780561168413e+00,     5.2489580169938954e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5198208328486110e+01,     2.2759446987958974e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2697689606735292e+01,     8.9596288420868550e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8641503677911498e+01,     3.6949230815773199e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2132933468993672e+00,     1.3188431241635359e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1595522562431391e+01,     7.8604193938597491e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7864051463702637e+01,     7.8596434472712317e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3399188315447622e+01,     2.2717002253744879e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1551425210369453e+01,     4.3942163551833073e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6809023578631425e+01,     1.5520212385052902e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6067175996228169e+01,     3.2233894186771657e+04,     2.0760807807756017e-04,     5.2780211021351846e-07,    -2.5075533214347419e-09, } , 
      {    4.2441808169638442e+01,     7.2275693767301873e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8250594023303577e+01,     2.3951075518704911e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.8025266826003246e+01,     2.3243490445821615e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2884733189930193e+01,     8.8199281717337471e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0006870494431901e+01,     9.4144543292763265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7494282838304073e+01,     1.4288440319932606e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0693270405091649e+01,     8.2331314508473370e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3254123594868439e+00,    -3.4492772969840672e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1816136230048738e+01,     6.1138690423676201e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5803890134034795e+00,    -1.6058653062749145e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1962496125677166e+01,     1.6790491071822307e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4741227455195798e+01,     2.9934831604699215e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.0718556551255475e+01,     3.2199784823676138e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.0211947472677551e+01,     8.4879510323167742e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0359495899938167e+01,     3.2200489455592284e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.0005534215779566e+00,     7.7315622431940665e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.7082683739340553e+01,     7.2694197956451735e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6284119022888972e+00,     5.3821092075333945e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6142511627345641e+01,     5.3270638067804612e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9598547270558527e+01,     2.3332155643477141e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.1786121692804095e+00,     8.3853584036385128e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7646655735415063e+01,     8.3198312692856634e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.2357680183715669e+01,    -8.3375515801645888e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.3579063944517642e+01,     1.4914453418689140e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5902379286780022e+00,     1.4878398288736005e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1854303265468070e+01,    -3.4662030386764400e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2859699724806745e+01,     8.2817920860227059e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.6890175790680884e+00,    -1.1146285592662935e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3867510977547848e+01,     1.8875525874528130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7100507490366414e+00,     1.3893118150399885e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2658302322182024e+01,     3.9071976985325455e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6383137889934929e+01,     2.3858327803709632e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.8439556508181155e+01,     3.8941817365806946e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9884264094721598e+00,     1.5183514747993935e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1392244473365373e+01,     8.3220800251718338e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3362828278810674e+01,     8.8636990052429064e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2477321092771350e+01,     9.6654886881445200e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4124769132189408e+01,    -1.6651349375055306e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -3.0499024788721975e+01,    -8.3459644214919826e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.4435192255049580e+00,     8.4242513986029153e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.5714087512019518e+01,     8.5260349657362622e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.5117360059995146e+00,     7.2080293910638848e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3547034731037833e+01,     1.6663416323845202e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.1176102645745917e+02,     1.5542246331645923e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.8782556651764938e+02,     1.5543262248721994e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.0265399003086966e+00,     1.3241400338846550e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4978070549558385e+01,     2.3209386712523155e+04,     1.1961781289263120e-04,     3.5070869003867363e-07,    -1.6144828875996768e-09, } , 
      {    3.0170618845991598e+01,     7.4673984241667622e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9149051175958135e+01,     7.5653971056974169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7748203529887633e+01,     1.5525481293417102e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.2725215702980552e+00,    -1.0370762982541692e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.1504472139013590e+01,     2.3617014275233181e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.2954615780155414e+01,     2.2796632057540704e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0078757815657095e+01,    -2.2123364377238058e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5329165104015683e+00,    -2.6030239884801176e+00,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.2424850052558838e+01,     1.5677741024166125e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1705908598958793e+01,    -9.3115705540402757e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.0994472760754350e+00,     7.1856769320485746e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3790163298749732e+01,     1.5447582458099705e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7945358220185280e+01,     1.6108705068200534e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1979102443933961e+01,     1.6031376464244444e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2701229784154730e+01,     4.5515411966592092e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1688307213480616e+01,     1.4950495121540825e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6689143476807967e+01,     1.0908985947036155e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4138948954364682e+01,     9.1735285478644073e+00,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.5902937509792221e+01,     6.2883355150877996e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7270059219108340e+00,     7.7687473127757958e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.7142770928523085e+01,     1.4779459971156250e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8674034174965428e+01,     1.7321738850898208e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8265204147656359e+01,     1.4394368683780220e+04,    -4.0617244174770303e-04,    -3.3596133659847518e-07,     1.5813652650430841e-09, } , 
      {    1.2567494630516284e+01,     6.7976316692580549e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5395534528586850e+01,     3.1414095200248234e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.8018078047444348e+01,     2.1438068835578768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8111348209340719e+00,     7.7740069774081621e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1926311624120733e+02,    -3.4560240734833769e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0405913912624612e+01,     1.4898904676849985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.7494395743593447e+00,     7.8416591050374991e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0940547058533357e+02,     7.2135549069207973e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.9018111979002757e+02,     7.2145708239968681e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.9672464762264454e+01,     1.4270391376577234e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.8102003652682971e+01,     9.5914577475899023e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.9986954583612118e+00,     2.8351722469968081e+01,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.9722887656949947e+01,     1.2258901121654999e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5255810020386596e+01,     8.4493481275720678e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5297987478374679e+01,     1.6756709644319340e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.9392053456252711e+01,     7.1897899918493513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6149266538810760e+00,    -7.5764309128421155e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.9029086321245202e+01,     1.4405583825786609e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3768594786397195e+01,    -1.2712089278858236e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.8095863677261164e+01,     1.0575406606015249e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3601094519713577e+00,     8.3248886551158012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0775703249458559e+01,     8.3324941943344511e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4393378312965854e+00,     1.6832439542311135e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.8312100469508046e+00,     3.8132918131353413e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5813954523228528e+01,    -1.1484099867692596e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.5000971105905711e+01,     2.3264468037841394e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.3439254038095783e+00,     1.6892669464802806e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8943050127417237e+01,     8.3973835302977877e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.2433970674468604e-01,     9.5454916929183000e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0099678182422570e+01,     1.6170350529762625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5883841777563603e+00,    -2.2154707637809747e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0735007398040660e+01,    -1.4819123794374470e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3752865294234127e+01,     4.0183743840604041e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9606133462285477e+01,     1.3905756340294505e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.6993778573650893e+00,     8.3183821434696383e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4340681608863060e+00,     8.3390007059806139e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.3663932191933377e+00,     2.7997118871861858e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.9724661719013294e+01,     8.2802400244437849e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5352023513140606e+01,     4.4007829276349858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7504998490698640e+01,     2.1479393761841493e+01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    8.2356778977788867e+00,     6.5857609102103506e+03,    -2.1583699037182070e-04,    -1.8708058200570130e-07,     9.3204945379627173e-10, } , 
      {    8.3837209598614582e+00,     6.2901893051241950e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.1266637874882392e+01,     2.3871452428459808e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.6196390813569366e+01,     2.3871439001358362e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8025557966944472e+00,     1.6688561252964737e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3124615250567015e+01,     7.2424487988690908e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1116466999192596e+01,     9.2339723545776287e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5779839584619323e+01,     7.0045133996281111e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.3280947807711584e+01,    -8.3373279231085544e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    9.9006318103829205e+00,     7.1442330215534575e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.9232884538476618e+01,     1.5548787764578905e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.9866867124461137e+01,     2.5836550563374021e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5545920419933477e+01,    -3.3088781972005381e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8920833980670796e+01,    -1.6633063405160119e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.7809540610180548e+01,     7.6831108429127271e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9587268167547592e+01,     2.3872157060375961e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8444739675014151e+01,     2.5443529593681033e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.3754943676192582e+01,     8.7977394021790205e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.9788925723076165e+00,    -5.3929007143194167e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.9326730865804187e+01,     1.6026774064564976e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1985170968381151e+01,     1.5436816095761576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5905860265655921e+01,     1.9237677371761281e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1616163142943922e+01,     1.4285786804134796e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9494341194313147e+00,     7.9122602285243483e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3880568043937211e+01,     1.0058509657837509e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0180454065365101e+00,    -8.4707267962809565e+03,     2.1869673659189744e-04,     1.8637692342893893e-07,    -8.7906416658781052e-10, } , 
      {    9.8619515321510349e+01,     7.2383357390683150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6161368004106002e+01,    -8.3401814124807697e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.1450070970904740e+01,     8.3172014369694807e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8601221241286915e+01,     1.4375156633706474e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5335237704991297e+01,     2.3870734369442209e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3490552674819959e+01,     8.0753558660171966e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0240445130297360e+00,     7.2102600958495068e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8190749423220232e+01,     4.0704620983052613e+04,    -1.1088658514337291e-05,     3.4142518678457958e-07,    -1.6284891548469314e-09, } , 
      {    2.8159675213456680e+01,     1.3839633047770034e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.8199092442424307e+00,    -1.1326527674674985e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8073495526772554e+01,     7.2294819764980220e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6697531831189831e+00,    -8.5551615713646279e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.5316289474100184e+00,     1.5524730081982754e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2092583378786669e+01,     8.9017513613601565e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.2120604055512079e+01,     9.5056767910203889e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0511885726840907e+02,     8.3287894762989654e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.0982974541597201e+01,     8.3285933731512869e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6483230956491518e+01,     7.2178656350681586e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8568764390377950e+01,     1.5858222998864821e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3685482021164352e+01,     3.1807116169941222e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.7649681961885388e+00,    -1.3771471223082065e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.7137580429325755e+01,     8.0802868981731153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4210420846959586e+00,    -8.1982336688893952e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7145274064354240e+01,     4.1469633669423274e+01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.3385348471571636e+01,     1.6097767369977373e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.7363726687439453e+01,     1.2434972588916312e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.2968119969326420e+01,    -2.1955413120496451e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6540607517038200e+01,     7.8111920469898914e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.0520496935860519e+00,     8.3570249141818185e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.0380073789738402e+00,     8.1366167920781918e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5885304033710277e+00,     2.3791644766874015e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5535102359924196e+00,     6.4963749302339500e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6996633666163625e+01,     8.2196589360497746e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.2508611009937106e+01,     7.9773571844865419e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1100681262505692e+01,    -2.4748550007657186e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.1872476776967176e+00,     3.6978817021943087e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.4430532529161326e+01,    -3.0592045994253567e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2118686697944169e+01,     1.6493089539510092e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.6345743601562486e+00,    -3.1283887636467944e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4448653038347734e+00,     9.9326794869089298e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.1098662594874444e+01,     1.2716356441160659e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4524575720882654e+01,     1.8537711599498489e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1670755559722750e+01,    -1.2843631231991028e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.3479605964883788e+01,     1.6402951409774349e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.0107444562030068e+00,    -1.4226909337526763e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2235201103245625e+00,     1.3135462144424167e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9738238086796784e+01,     7.2130964630111539e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8508460780971674e+01,     3.1103863300409357e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.4207355469598593e+00,    -5.5111335480816152e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2878101170145662e+01,     1.4481094828128857e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0763379664113195e+00,    -1.3639929269949273e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.7918011320926666e+01,     1.6102369769656838e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5290911314728858e+01,     1.6854480795755862e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1814176312966151e+01,     6.0214374193013009e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7388703590532968e+00,     7.2412928275231016e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2991902402982127e+00,     7.8051341922294250e+03,    -6.3923362593445873e-05,    -4.9935809154282207e-09,     1.9489509980603329e-11, } , 
      {    1.1054254634565517e+01,     5.1697389653045479e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.6148720807480103e+01,     3.6027960349179011e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6906699421604760e+01,     8.1835763087070736e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.3046859763865545e+01,     7.7997106345486727e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8115425007452295e+00,    -1.3268078574131996e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -9.3338523333688798e+01,    -9.8051573838887407e-02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9049160568508547e+00,     7.8243462423031706e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6424015406688177e+01,     7.1185028915810426e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5685210438779480e+00,     1.5530135051861580e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -9.0648542117924897e+00,     1.7206060630699973e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3894827735721535e+01,    -1.3215481927808314e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.6271379949533809e+01,     1.7283389234656061e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.8661083505210939e+01,     8.4019859299772543e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -4.1510947548530247e+01,     1.3916019040732526e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8583997403359263e+00,    -1.6674303530259036e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.4169832185118560e+01,     1.4350128844026427e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4988006196884651e+00,     1.0001737791016991e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2972450815579318e+01,     3.3522013964891492e+04,     3.2889566200998476e-04,     7.4133344792140254e-07,    -3.5185837720205683e-09, } , 
      {    2.0064886876482056e+01,     4.0879440512703059e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5616557837654987e+00,    -9.5033524932613432e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.2607613488267056e+01,     1.5542395260675157e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2389938682952327e+01,     1.5543113319692760e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7521056522169118e+00,     8.3713512406808295e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1753003636427014e+01,     3.1119265627505360e+04,    -1.6320554442851728e-04,     9.0926805892093470e-08,    -4.4360451821522545e-10, } , 
      {    1.1835664700043161e+01,     3.3700107583424498e+04,    -2.2525587784636461e-04,     1.7167252448088666e-08,    -9.2987264036809076e-11, } , 
      {   -4.3713822029139795e+00,     6.1335125190742974e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4791303895003551e+01,     1.7256631475449803e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0259392474511344e+00,     7.6977540073633991e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8439876469335207e+01,     2.2756817155642791e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6084195233991590e+01,     1.4428125730917665e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.2738463184170685e+00,     8.2419503757970906e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1289216800858462e+01,     2.6229571533067010e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2681582269508535e+01,     3.2183216458825427e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2991218899680419e+01,     6.6394357230913574e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3883755825694593e+00,     8.2860316087694227e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4281226385027272e+01,     6.6435600144795342e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.6438917285835615e+00,    -2.3188953454484704e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.7514910241877928e+00,     2.2145430251406029e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.5947132387052505e+01,     6.4905834483290355e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4742213539645634e+00,     7.8581181940200140e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2219158405688511e+00,     1.5890729163357412e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6901400804041451e+00,    -2.8534893722171546e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8331075670590280e+01,     3.3526616364570960e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.7550473200192123e+01,     5.9503317656752188e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8740255581351860e+01,     1.4425495898601483e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.8203274360467557e+01,     3.2264571489446880e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1926954957472224e+01,     2.4593053304482390e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    9.6550532687267889e+00,     6.4901953065354992e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2849685635594078e+01,     1.4430755563233850e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.6383032767687880e+00,     1.6911814289126152e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.6512574289152830e-02,     6.1282999609076323e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5201617069619529e+00,    -2.2936914683452560e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3754748327454625e+01,     2.3276900806798047e+04,     5.7959208929120429e-05,     2.7677460366946941e-07,    -1.2667745155079175e-09, } , 
      {    4.1597679040606970e+01,    -1.7067281539646501e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4734703248406021e+01,     6.7848297899028512e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0372073347797052e+01,     2.1553171082762899e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.2826847187761505e+01,     2.3864886759309356e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.8819288401990626e+01,     1.0339477930609096e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.3044266375453191e+00,     5.5644267256747589e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4978152002016550e+01,     3.3490533737228259e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    7.1761183114093043e+00,     8.7451226209259039e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7326684738276491e+01,     6.6813564847807866e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.0260575145482523e+01,     7.0548032578671846e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6738784807927985e+01,    -2.9526287768931070e-01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    9.7484254213417820e+00,     1.7755090727907762e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.8738762883023369e+01,     1.6171414191187578e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1036416359981938e+01,     8.3290846709875677e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.9005454146259630e+00,     8.3282981784626845e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4950660475254622e+01,     1.5487397359997618e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5698671818704790e+00,     1.4790493778300948e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -4.5434462550635985e+01,     2.7608977078325552e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.6334717647828567e+00,     5.1714057559817711e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.6967409050932304e+01,     2.3878329751503742e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1335704682025714e+01,    -1.5252532512172365e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5953338264338067e+01,     7.6872659284801275e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3820995956246286e+01,     7.8423648207073147e+03,    -2.2116475335763771e-04,    -1.8584779777737599e-07,     8.2316999729269210e-10, } , 
      {    1.0662831612841565e+01,    -1.8465474157201754e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.3471079057418294e+01,     9.4946364226046353e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0623412863216064e-01,    -2.8624804763660013e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.0353129117241529e+01,     1.4445398727684522e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4764152619230995e+00,     7.6408652626547846e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6775027463974986e+01,     4.7938038973279736e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7248436992572000e+01,     1.5003464218752017e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2312706873998158e+01,     9.7364819935569176e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.2602599157973584e+00,     6.5677310722120837e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5307770325592394e+01,     2.2805735571458008e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3584753738323734e+01,     1.5637926122268213e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2246751246359404e+01,     9.6236882851727091e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2906756422434444e+00,     5.9336269603555695e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.8515074950948227e+00,     4.8597398169102344e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0512555019286323e+01,     2.2092461154194836e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.0299771492445942e+00,     2.4410735786341029e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.1911418397108537e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0138268537002505e+01,     1.6678712757428104e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7328455593997667e+01,     1.9630698341454263e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1497615047413092e+01,     7.8243367665170499e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7014478541351568e+01,     1.6835780720094193e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1124082913937727e+01,     1.5718546708877235e+04,    -4.6802691043838204e-04,    -4.0980815650507968e-07,     1.9305280781781724e-09, } , 
      {    1.1055926093603512e+01,     1.4388310829019752e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.3967544412511913e+01,     5.9570953794096722e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2329166972366988e+01,     9.0809519366081367e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.6625613253606555e+00,     4.9127089141214256e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2378929307578730e+00,     1.6422096234097698e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8330368643301313e+01,     1.6564082844922490e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0024362938031132e+01,     2.4422201038942501e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.5701826019452971e+00,    -2.3182746321461887e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.7070681792656170e+00,     1.6636052941472401e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5705073727915625e+01,     6.2915089388394736e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9256761189862566e+00,     1.6425493314905405e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6232759910520880e+00,     5.2308073609003679e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7542656940293199e+01,     1.5560778498385162e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.6670132815568849e+01,     7.6882061664600769e+00,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.0323262613758255e+01,     7.1972021187646496e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.4584743609493968e+01,     1.5529247778912606e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9440293167859238e+01,     2.1831089805271757e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9848646608968377e+00,    -5.5731427963314673e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5952970203060620e+01,     1.5366961871490683e+04,     3.3565066799162013e-04,     5.3770200558177518e-07,    -2.5479867824392772e-09, } , 
      {    1.7041907058293827e+01,     1.4461882778055107e+04,    -4.6783104571121383e-04,    -4.0989542296767940e-07,     1.9290736371348431e-09, } , 
      {    7.8587604151210959e+00,     5.1206010936767268e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3455952868204228e+01,     1.6622437960440267e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.8385105026904460e+00,     1.0984192239243887e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4992663986784578e+01,     1.5280332218451440e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7991326841769677e+01,     1.4897526024329634e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.3377901128070233e+00,     8.3659220532030158e+03,    -4.9286401228435976e-06,     6.9556897567938677e-08,    -3.8265859036294568e-10, } , 
      {    4.9413007733721443e+01,     1.4286491436050948e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9206509360309255e+01,    -1.9126707082446460e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0002099239792365e+01,     1.4286139120092872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3583258946818098e+01,     1.7654780494428928e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4919732416107083e+01,     1.5126323093983181e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0309949479619743e+01,    -1.1942583630621180e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5704318515802974e+01,     2.2130810770436987e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.4113200168281708e+02,     1.6219054166649585e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0675759612081883e+00,     4.4462587911343121e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8065358320861137e+01,     1.4620200363564600e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2043905653319893e+01,     8.3361695912941832e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0934018060509549e+00,     8.3212132581560691e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4053099552395487e+00,     8.3467251087124514e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.5655226589033253e+00,     6.9829843905374531e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.0161948695586386e+01,     1.4914775337614865e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8276713888958930e+01,     1.3877370406884245e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3694415216537820e+01,    -9.4410187841516854e+03,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    5.9375444749385116e+00,     2.3807011365096416e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.8112019263222749e+01,     1.1302006916729139e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.5267272693092252e+01,     6.0403471143606112e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9275416437326271e+01,     8.1919737222645344e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4805784464113671e+01,     1.5556260801455313e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1630906488940909e+01,     2.5379095243868367e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.0302996902870909e+01,     7.6052120075953289e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8155424526600044e+01,     7.9379716548718716e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0559618001648108e+00,     8.3989733846706458e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8505006907462693e+00,     6.6380130321576044e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2514801167691317e+01,     3.1053767652494389e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7846001690958762e+01,    -4.9867287707456853e-01,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    1.1269959937995834e+01,    -5.9225916864313297e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.2276859770815086e+01,     9.1941574526797176e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0474559494752164e+01,     7.2135105759263497e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8774154661543587e+00,    -1.7781919178480621e+01,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -1.1250531862767128e+01,     7.2146151549913156e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1145223488268080e+01,     1.2651922091347988e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0357416861636040e+00,     7.8554883617038313e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9718686291406753e+00,     1.3342362826366465e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5307145235110598e+01,     1.0451530627530496e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0564457672095067e+01,     1.4467940632815578e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.3786837572541820e+01,     8.8613978054031722e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9506209945540881e+01,    -1.3897602969063882e-01,     1.2370893738135807e-04,     1.4769363981320903e-07,    -6.9832562627017560e-10, } , 
      {    2.2711830204274762e+01,     2.6622592502759999e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.2507077186569205e+01,     7.6848418113911530e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.5474227356034262e+00,     1.5558486774331548e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -9.6978832742595955e-01,     5.2257751744315044e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2048732292833497e+01,     7.2320870736600364e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    7.8335890177244014e+00,     1.5969556687379909e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.8291743326138885e+01,     1.4819274873054163e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2569698991568163e-01,     7.3587659722166862e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5031769878442319e+01,     1.7287991634335529e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.6418906081745082e+00,     6.7408598024375378e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2013439209215207e+01,     1.4818886731260625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5664538256583773e+00,     7.1714030495031293e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8773689721618091e+01,     2.1342467631211272e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.2811239382128363e+00,     8.1670429858629632e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.9096130160130897e+00,     2.3905227142412048e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -6.4215702738174500e+01,     8.3284857094979179e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1499046993896691e+01,     9.8074752989693116e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7077599367143875e+01,     6.5303983502269475e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1087459443781533e+00,     6.1728695832317919e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9930934316440812e+01,     3.5034072997339226e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.2054430205057166e+01,    -2.6710628058917791e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1761629158992820e+01,     6.6497450043468443e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9336861852357803e+01,    -8.3061495195940697e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5045794561846140e+01,     6.0640972352570052e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0443459229113534e+00,     1.0873985972020246e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0251890645467627e+00,    -2.5289780836978116e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3661122249406361e+01,     6.5754459991578015e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9901923157395810e+01,     2.3889469923110286e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.8163284896038679e+01,     7.8450002828205606e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4013789677955490e+01,     9.2905014024388583e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.4438646872464126e+01,    -1.3690599989421967e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2945391088875095e+01,    -1.1664436707565833e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.0428261955114920e+01,     1.5558173401223148e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.4741746660034572e+01,     1.5598111220370300e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.3192597702347939e+00,    -8.2047384978876580e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3837276403011961e+01,     1.4286850465559748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6331354380539636e+00,     1.4850012355325747e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5869150202827576e+01,     2.3307572479676212e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.7765934902214802e+01,     1.5527021806036370e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8422777330761292e+01,     8.9968139116685816e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1488300331755905e+01,     4.4809650209632946e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4549210611960746e+00,     1.5514368356773701e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.7690840116834245e+01,    -1.7720310878923556e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.1316076140243197e+01,     2.5426438423554032e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1377484307696798e+02,     8.3283458223177768e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2791073018224118e+02,     8.3290370271324755e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8605666256398674e+00,     6.0626745443232521e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5345600417170891e+01,     1.4968127147816484e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0673438430392686e+01,     1.6755598626765885e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.8413755824060996e+00,     9.1922856251232261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6670140381143810e+01,    -1.6659450977989218e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7903422322837422e+01,     1.5475240195909068e+04,    -4.5295172598701735e-06,     1.3788101090755202e-07,    -6.5643772422231173e-10, } , 
      {    6.9648141116856532e+00,     1.6050405172382561e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.7726457323402698e+01,     6.8692105572661603e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4784472070798493e+01,     8.3281391351926432e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1486226434615944e+01,     5.9581590408346228e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6931834161056770e+01,     8.3292437142576091e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.9029800661325638e+00,     7.2297953496064220e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -1.3597778295711205e+01,     1.4283837920253138e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2039235091902562e+01,     8.4526443515625924e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.3668428164552582e+00,     7.6996724942127057e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4850166890438558e+01,     9.2567199749358933e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8880881002359811e+01,     7.0743774544737735e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0899143773195306e+00,     7.3096228393366227e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3831143554439844e+00,     7.6666080419736800e+03,     1.8590386647959628e-04,     2.8671813226902599e-07,    -1.3064807559907884e-09, } , 
      {    5.6419870890359309e+00,     6.5644253724354376e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6369870121807228e+01,     1.7619192827163984e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.2177235960782991e+00,     7.2298106089744706e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.7049159356662557e+01,     1.5518481416574477e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6216398372153741e+01,    -1.5322613319670108e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8250827873888049e+01,    -8.3395081360848471e+03,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -5.5995206190833180e+00,     2.2126208370757515e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.0309597604850937e+00,     1.0969965329906358e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7803779386819381e+00,    -8.8330460829332023e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.1877218309484547e+00,     2.2003914638174929e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2380478089611930e+00,     2.5561412762009728e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.6595262519838663e+00,     5.9574476953677458e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7967727475323343e+01,     6.1167073029593957e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0483002900204625e+01,    -7.1593706394196488e-01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.0952553764711901e-01,     1.5527335179144769e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4209220528430148e+01,     2.2914551509983223e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8198153608834360e+01,     5.1868248670209614e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.8105730572271703e+00,     1.6385678413007496e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8990948050973326e+00,    -7.6539831738542398e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7399697896509416e+01,     1.4392788659982130e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0342381322242193e+01,     8.7838455443910461e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0634706750823501e+01,     2.7314508362352967e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6325241823013337e+00,     6.8548233232688099e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0073742744153938e+01,     7.4313157968240630e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.0433912699420951e+00,    -9.0617981230515488e+03,     6.4119227320614115e-05,     4.9063144528285039e-09,    -2.0943951023931932e-11, } , 
      {    9.5389848741209740e+00,     1.6866932315280974e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.3666020026984693e+00,     4.7352075332125665e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.7490038561142480e+00,     5.1845236671812263e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0202079069254420e+01,     8.3309584136245703e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.8853209925627393e+01,     8.3297873056931003e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3544422999421897e+01,     1.8229466728222200e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1740700144339296e+01,     7.1507844869921537e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4132260972593622e+00,     8.3282955906373136e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.2978830472444365e+00,    -8.3290872588129423e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.0938569174326398e+00,     2.0618562510975480e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.8771633240723755e+00,     8.8018944877464201e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4307746163717049e+01,     3.0563139189744757e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.6815478753857054e+01,     1.6655305306666552e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6815890896075810e+01,     2.3402045421497118e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9379299117474986e+01,     1.6659460392233956e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0392107555458736e+01,     7.4061374981057670e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5651343129506028e+00,     7.1986437544196433e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.9696498599288947e+00,     1.2559038246242101e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9190390075961176e+01,     8.5021582632131485e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.6891269831731535e+01,     8.4862200638383492e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7183515065075650e+00,    -1.0988808157506548e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.7188432776151528e+00,     1.6200772690806727e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9882640130710092e+00,     7.1360659785675944e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8751138580454221e+01,     2.0023719311147252e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2520831092624476e+01,     1.3109377410853642e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2534879330715913e+01,    -2.7170443644275929e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.1307779935902566e+01,     9.0919857437528353e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2544036181346018e+01,    -1.6644763611127873e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.0261818021756529e+01,     7.2140733876626418e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9624962908379260e+00,     7.2140523432550235e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9920580830988502e+01,    -8.1890060137400669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2879639574454238e+01,     3.0497015897220288e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.2154622247137734e+00,     1.4391365969048378e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9206627501934497e+00,     8.2522501812664577e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.8651858582316549e+01,     1.6170709559271427e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2624261579654124e+00,     1.5636054294711721e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5373737815597487e+01,     8.3750491223830250e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1201954823145906e+01,     7.9018890275291742e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.6494789159790848e+00,     8.4875987163586997e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8786718868362499e+01,     1.6365182285846195e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.0178357636565742e+00,     1.4893116797160565e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2691990614913329e+00,     7.0035814788410278e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5480368207282545e+01,     1.5571140223594215e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4454535743428462e+01,     1.6204813292569888e+04,    -9.9681304698044827e-05,     2.7596273467908341e-08,    -1.8931489433646278e-10, } , 
      {    1.2098991817684997e+01,     7.8761218678447603e+03,    -2.5199405533939313e-04,    -2.2281484096197810e-07,     9.9702418333857162e-10, } , 
      {   -3.3563885442700055e+01,    -8.3275955437571520e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5943780876347692e+01,     2.2467921296428358e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5394987013108342e+01,     1.6128402059273798e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3369365458966385e+01,     9.4658618891224760e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7091140994829246e+01,     1.5113521214627977e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7269217369453905e+01,     2.4328901034414739e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0484175410585824e+01,     1.6173691707545688e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1966260950146705e+01,     1.6670002087772631e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    5.8711840491950440e+00,     4.8067707196990423e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0099080196610790e+02,     1.5567027163793440e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0222911802702015e+01,     3.0652922309702717e+01,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.2394447200287249e+01,     7.6846360961639439e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4183654488733396e+01,     1.4914118072661968e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2834393334160048e+01,     2.1659109277185282e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5285098777152903e+01,     2.2679264894630669e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3016530367862245e+01,    -8.0180863447385457e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.4383293554474545e+01,     3.0927774226027486e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.9953268175987517e+01,     9.2737872564755398e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3166667077180934e+00,     8.3275279632181046e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6937141462679442e+01,     1.0090755304630591e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.0228955921949080e+01,     6.2820953347170359e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2095519095049276e+00,    -8.4051326681837945e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.6782303799013354e+00,     5.6502920657886409e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9366759614849318e+00,    -1.7216062363339847e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1469024275330848e+00,     1.4397472808607961e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.7049573688788641e+01,     5.9578000113258213e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.1693037038929273e+00,     4.7915026974882394e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6048330924455003e+00,     6.3278378466679108e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4170477279046276e+01,     8.5846415563811934e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.4674096125344505e+01,     8.6129300055836393e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7651202003763636e+01,     1.5555373528506338e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -7.3663309356630151e+00,     2.3893987620040141e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5379992263898721e+01,    -5.0670719472693690e+00,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.7609222652022940e+01,    -9.4456211838311538e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4845851196952424e+01,     2.2654645446715909e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6339246808809143e+01,     1.6393230579382602e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.8702277846778683e+00,    -1.6900713742783941e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    5.0009931241215853e+00,     8.5719979376932934e+03,    -3.1118464253716361e-04,    -2.9727805298274525e-07,     1.4006267247254494e-09, } , 
      {    5.6364662858311803e+01,     7.8427227664624506e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3290519907403215e+00,    -1.8749333491304342e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5598964089786401e+01,     2.3190169032652349e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.3978258040180371e+00,     6.8401867200891929e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1568989175206328e+00,     1.5085298970678301e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4649928026520744e+01,    -6.2623004226184275e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.7202381233165696e+00,     1.5565296195315015e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.7579921326162406e+01,     8.6464613332457266e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1383579567115845e+01,     1.5538951520574634e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2208628825042355e+01,    -1.4308928027484455e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.3615526637367633e+01,     2.2128157254639176e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.8523469435786506e+00,     6.5988731916127444e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2908233178703670e+00,    -1.0018658471825771e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8774449738313201e+01,     1.5447194316306170e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2683410526826187e+01,     7.2567226814145361e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0082867213898631e+01,     5.9452052774551794e+02,     2.8263352852431007e-05,     3.7539802067464924e-08,    -2.2902113481933369e-10, } , 
      {    1.6187788847196643e+01,     6.9274193485821206e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4653243963190645e+00,     8.4617996470971793e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8834844968524628e+01,     1.5546557059793284e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.6009104810587589e+00,     7.3369217406899961e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9418020861756929e+01,     1.4859089774952074e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.9006078155708046e+01,     8.8600962092079899e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4134410884941332e+01,     2.7015613472452984e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.3079210141577775e+00,    -9.7771151436519773e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.4122798929775158e+00,     2.4125877154584989e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2109315605402180e+01,     1.7836445758529215e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6746374845138991e+01,     2.6735945268387886e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3762243297664309e+01,     7.2773412439255117e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5977506178575638e+00,     2.3013539513374071e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8968507696500936e+00,     6.2068100141927255e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0048801466355830e+01,     7.9978036050050723e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6334976628659632e+01,     7.2893139266312210e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2947580251729223e+00,     1.5523566620475733e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0770231307737058e+02,     6.5858533319871294e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6731276851833211e+01,     7.5449506851788865e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1056818026025518e+01,     1.5467503229011571e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7907770552805431e+01,     1.5383494682592311e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3640541138023671e+01,     6.4099684915170747e+03,     1.8600179884318042e-04,     2.8667449903772613e-07,    -1.3072079765124529e-09, } , 
      {    9.5625264471584330e+00,    -5.6387323523287432e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3930173300360579e+01,     8.3258465106673357e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.3155949004992511e+00,     7.7540642823576509e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.0777350931029570e+00,     8.3315363387829166e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2831965241850376e+01,     1.6652451817294332e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.2059988294440025e+00,     1.4966704456882731e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6943820242381022e+00,     1.5261067009006059e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2248138013473110e+01,     7.2037535842033449e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0487047044286783e+01,    -1.6279234438115964e+00,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.4377311145505779e+01,     6.5632133752822338e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5502626121118318e+01,    -3.9532216953272291e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7254032055095658e+00,     5.4039359882436554e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.7581388055319209e+00,     7.5360905297394238e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.3375527810221271e+01,     2.5195271424048027e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    4.7739856441304497e+01,     2.5195976055964173e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    4.0862391349842632e+01,     2.2224110774964745e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4611475556636986e+00,    -8.2849205912159669e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.6221821294628116e+00,     5.0136714016413398e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6011314790695454e+01,     1.4904137423882928e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7884995939781430e+00,    -8.3106672165239215e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.5307484233945132e+00,    -1.7947803827409822e+04,    -2.7360033457377286e-04,    -4.6394245213777045e-07,     2.1973695282608606e-09, } , 
      {    1.9478551645502858e+01,     7.2112179514010413e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9850567795329198e+01,     8.2972111970618971e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.8068791213029574e+00,     6.3989723933555439e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5144583312892284e+00,    -6.0911991533731612e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3781835017586754e+01,     1.4031707680416968e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2729373362833893e+00,     1.5707047600124117e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.0027811510942488e+00,     1.5069551227162665e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4780799221955661e+01,     7.3109656660215132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5531732520163342e-01,     7.2169077795166240e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6724475798088139e-01,     6.6790552849410524e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8738273624455655e+00,     1.2005565562947071e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4947375727444745e+01,     8.1551903752968155e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -3.5339225671394928e+01,     2.2775163581087554e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3337355147321173e+00,     5.5775446368742132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2171906137535510e+01,     8.3724622582342854e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0542455114489847e+01,     8.2407774385039484e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.9590492904933976e+00,     7.2243721467143205e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7443713106397727e+00,    -7.3568534194938411e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.5571550798323372e+01,     9.5282186961514453e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.1565898453429231e-01,    -3.2555014460589882e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.7951021412360788e+00,    -1.5995323847064325e+04,    -3.3811868475736040e-04,    -5.3717287993021232e-07,     2.4920926131441587e-09, } , 
      {    2.0702754590336504e+01,     2.4972956362975921e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9920870624533535e+01,     3.1871550519753895e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.2942920611046116e+00,     1.2398586532640060e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7804590686570698e+00,     7.5088680578361864e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.0186826041310855e+01,     7.6207540358837578e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6786529731313564e+00,     1.7259953284973963e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.3690429759861217e+01,    -4.5350594916546447e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0769436855270857e+00,     7.7643735636131378e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2951899919746079e+00,    -1.3705786909223589e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1353358317402140e+01,     3.8964282694269104e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2604911410232209e+01,     2.0413502016577513e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.4190386045550298e+01,     1.6641840821161823e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.5970896811088124e+00,     1.7264360526517943e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5135367760602383e+01,     1.6651482795363834e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.3154078584661448e-01,     3.9411898805253310e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {   -7.1627425645897574e+01,     1.6672924877738682e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.9621468565307207e+00,     1.6440129918085022e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.4453589524416177e+00,     7.3537824873841791e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.2713990796093114e+01,     6.5878328231969899e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4116894346392026e+01,     1.4978436429071971e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5392669158391207e+01,     2.3854172718142228e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.1670552814046591e+00,    -1.9988862759679018e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6731184917286086e+01,     1.0844551597223483e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3795395398141475e+00,     6.5923142360130169e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3000284009555045e+01,     2.3149838125335780e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.4771252804307281e+01,    -6.2631822850009485e+01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1750935460108547e+01,     8.9577103552375447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0015752442358689e+01,     4.5848376292860166e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.4720038182071260e+01,     9.2572735037103303e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3826188186028340e+01,     9.0349958966619070e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9621172471410844e+01,     5.5798458367139492e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.6706767873793531e+00,     1.7576803832775727e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.9305667039395820e+00,     1.5453736776570357e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2804624679224535e+01,     1.6692719920385785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1238366854841395e+00,    -8.3242821975560546e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4596681681098310e+01,     1.7791139144310175e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1446516529587910e+01,     7.6450269094932410e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6395800967360373e+01,     2.3505656692905457e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -5.5908382275166240e+00,     4.3984817277952516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0100721729652734e+02,    -2.2944862890076365e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.9257948612816804e+01,     1.0177257587036129e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6357256243954666e+01,    -1.7432884602699114e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0241881562743735e+01,     1.4269687627538979e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.0372033084637224e+01,     3.9416501204932778e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.7142038068113994e+01,     8.3297949521175487e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -3.4594564870723914e+01,     1.6600294950785483e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5550052515457926e+01,     1.6136116986219515e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6333056431727655e+01,     7.6265836262706825e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2186297725139230e+00,     9.5853065948162130e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5913162707097690e+01,     1.6858482456470683e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1753441898593566e+00,    -1.3433656948052421e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4494904809488936e+01,    -1.9643476706677241e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9534798625582599e+01,     1.1695027886422127e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6002451334829768e+00,     3.9218192554577058e-01,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    2.2911370872534498e+01,     4.8739859906562815e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.7552170323983454e+00,     2.3869368172125385e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4522730325911475e+00,     1.1612544593254084e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.0275313343252765e+00,    -6.6208338275144661e+02,     3.3591115838248028e-05,     3.6307017839139590e-08,    -1.2014167831575411e-10, } , 
      {    9.3438647080695461e+00,     2.5739135146922443e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.7231681793001163e+00,     1.3154195313279118e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8525297382088204e+00,     6.6556450584332624e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5981416673417385e+02,    -8.6027449611155760e-01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3305523953841746e+01,     6.2222931814237027e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8602966508106597e+01,     8.4460853959628657e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.7401974546769928e+01,     2.3873523257692785e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2587508904164743e+01,     9.0999486622344239e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7058781839952424e+01,     2.1735488600904260e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.0498587424042398e+00,     8.8690850235494927e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3384495409191006e+01,     9.4302020727919644e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.8374080062054219e+00,     5.4359143549118016e-01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2575884071380308e+01,     8.4111747178931673e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4309893308402284e+00,    -9.4095629368539740e+03,    -5.5395567512918104e-04,    -7.2425346193591365e-07,     3.4241420669404302e-09, } , 
      {    1.5397923451105999e+01,     2.4114752227877252e+04,    -3.7737276376054456e-04,    -2.3333112844439748e-07,     1.0918973725948968e-09, } , 
      {   -4.9714282628394123e+01,    -4.2281101218759432e-01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.5247962788784772e+01,     2.2642645991309291e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.4578855920475462e+01,     9.4260469872245631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2567559185197691e+01,     1.3044943061040976e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1996969748495072e+01,     1.6694142611319538e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0213106754993458e+01,     2.2299361836137137e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.8353021962360323e+00,     7.7526415914238978e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.7250457654504139e+00,     1.5604887440156894e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.7685916994090132e+00,     8.3356950062039668e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.9044791660614102e+01,     7.8994583491691446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2110381830382326e+00,     6.4618023535758221e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.0103099151892167e+01,     7.1967898686919762e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.3543073963698362e+01,     1.4978881054951085e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.9575019977703842e+00,     7.6228315786674593e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3735371132480276e+01,     1.0129502963249904e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4909299910266601e+01,     9.8876573922976240e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3250030418786114e+00,    -7.2609622041612520e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2971265902035055e+01,     8.3244260521131855e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6123337437192731e+01,     2.3243849475330418e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1680574152566415e+01,     2.5025889176073291e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    9.7787382464540382e+00,     1.6990885242118442e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8873494666271959e+00,     2.4946259372277465e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    9.0594136563532430e+00,    -1.2566639308221238e+03,     1.9586472716825521e-07,    -8.7266462599716787e-11,    -1.4544410433286028e-12, } , 
      {   -8.2302579148923343e+00,     2.4497452100114893e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.7360709336381262e+01,     1.6028722948446635e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.9663673981190737e+00,     1.6341914140769388e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5206669683197678e+00,     8.1697841330915517e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2174389527092515e+00,     1.4078727301228344e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0513439195921691e+01,     8.3278793787396498e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    6.5802330051417883e+01,    -8.3295034707106024e+03,     6.6570736180512717e-06,    -2.0358780910832745e-07,     9.7132421010295529e-10, } , 
      {    3.0173891210544156e+01,     2.0416740280840240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1459085842607900e+01,     1.6763321043872635e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.7757743403771524e+00,     8.1981795422879313e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.2595278769904823e+00,     1.8073704941245055e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.6825529507198791e+00,     1.3700491351003033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2666518376678539e+01,     2.3656829177131091e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2157188766022450e+01,     7.2266821037812115e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    9.3476976438279120e+00,     6.0372442124461054e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7042653269569090e+01,     1.5876815533655687e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7492264937188978e+01,     7.0405618160305221e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.2438454598340474e+01,     8.9232119524706432e+03,     1.8057610349377928e-04,     2.8795091649735135e-07,    -1.4153602124943682e-09, } , 
      {    3.5074543861107614e+01,     1.4916524247922114e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8263851492227086e+01,     2.3892775622886937e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2970029552156021e+01,     1.0200496268662300e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5395646184581246e+01,     1.6662313881606173e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2991125419910077e+01,     3.0534753256334498e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.9393696835571461e+01,    -6.1819776864531377e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -9.6436679513086858e+01,    -6.2840563661938234e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2623417823757643e+01,     9.0033996755960143e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.8535284104545537e+00,     1.6657382849450252e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.8272501861627060e+00,     1.6187982556038283e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2261935066423543e+01,     7.1618051137145176e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2878070874378228e+01,     1.6066311522679660e+04,     1.5004799201141317e-04,     3.1935161988366242e-07,    -1.5145579397861903e-09, } , 
      {    3.2241669676550345e+01,     2.2988706690187639e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.9188006629264677e+01,     2.0199094891187960e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7379247621255317e+01,     3.1138477677579111e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.2363966518793429e+01,     9.2544187750961592e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6361258218163634e+01,     9.6497409446288821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5808882312186775e+01,     7.3733224730504817e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9976529396870939e+00,     1.4233170022881681e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2855308472645131e+01,     1.0318038482644624e+01,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    6.9261467888446671e+00,    -3.3787057300025253e+01,     2.8263352852431007e-05,     3.7539802067464924e-08,    -2.2902113481933369e-10, } , 
      {    7.5803571540862116e+00,    -6.4522826585432438e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.8688076137418498e+01,    -1.6675431792805626e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8427189755989161e+01,     7.2332505351670579e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -5.0969257546968407e+00,     1.5168878144814318e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2010230926555341e-01,     1.4004451838914973e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1439428458098309e+01,     5.7030162556918431e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9372730858091529e+01,     1.6641173514108345e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0049437281926785e+00,     7.6900745584240949e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6225651895030162e+00,    -1.0229361923377393e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    9.9854076013641517e+00,     1.7126783142862219e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7124397122772510e+01,     6.3000663910267107e-01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0302633940808867e+00,     1.1219523623561096e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0332112651813141e+01,     2.3577199373335272e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3645087632559868e+01,     2.4986785619642254e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -3.4274786288614784e+01,     6.7423878976237575e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.9129575429229728e+00,     2.4985362928708502e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.3506674027890710e-01,     1.5534117791800531e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5665921235514119e+01,     1.4550984606148828e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6202485282634864e-01,    -1.4919017854436420e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2679115804656213e+01,     6.6030282771801458e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9727823466005603e+01,     2.5208242978328479e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.7146474498520313e+01,     2.3362582835557278e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1092258068943089e+01,     2.2834369416654914e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.1063688342461994e+01,    -1.0349987554704690e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.4158544347341371e+01,     7.6917473412961554e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1712808077382604e+01,     2.2208139374393071e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6651549706313912e+01,     2.3850115806931233e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3281338067433987e+01,     8.0868458537728438e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3416845384797682e+00,     3.6346209898958637e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1276987551444702e+00,    -5.2959621425070964e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.5706306193931582e+01,     2.9821482742594299e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4966407395756155e+01,     1.8622487697915189e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8748619619952589e+01,     3.1834348117630583e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -5.2467325179622321e+01,     9.0521708418549224e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3359978005840642e+01,     3.1032539483156725e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.9135228824114996e+01,     7.6432959410148142e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9619413124272267e+00,     7.2663206172031478e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4591294847222972e+01,     6.4291749117937588e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -8.8395808047570483e+01,     6.5856572288394509e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0390908393325679e+01,    -7.3964127356063182e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8752689815594074e+01,     6.8972746187539615e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4923963189184928e+00,    -8.7554938219210781e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6568618313854607e+01,    -5.1887086195407903e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5556781037690328e+01,     2.7408634442145973e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.5409417198152240e+00,     6.2449429323240781e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.0021242252600269e+00,     5.9213992789653357e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6661693290115146e+01,     1.7541640566151862e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.3708077571904070e+01,     2.2128861886555322e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5737819262055007e+01,     3.1045693678470005e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.2339202097307616e+01,     1.6154141194420721e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.0125614086387955e+00,     4.0054607581022628e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5638302121286543e+01,     1.6011802267637853e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.3671299015852881e+01,     7.9375422825886290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4082078702434480e+00,     1.5496353624358613e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7424495346164253e+01,     1.4880670907432510e+04,    -3.2597005385132925e-05,     1.0025394237748739e-07,    -4.2887103044630662e-10, } , 
      {    1.2954740504101807e+01,     2.3243138129863542e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.5770659696717573e-02,    -5.2613587611866251e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1861007518282193e+01,     1.4585860085258098e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.5371145589821857e+01,     1.6582131788277864e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5547859722070960e+01,     7.1465487711839414e+03,    -1.5684226790121844e-04,    -1.1253010352233441e-07,     5.2990135345272272e-10, } , 
      {    2.3463502139114038e+01,     3.1082878748051735e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    7.6536989065452463e+00,     9.9783275645091999e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5962890687579632e+01,     3.3560363581133635e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.2811525229750062e+01,     1.6780181790566818e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1005807145519330e+02,     1.4914544756712256e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0364413151411824e+01,     8.3267425408434665e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5583629014893252e+00,     3.1639189233339554e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3248264867793864e+00,    -2.1359571140048242e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.1016205692727635e+01,     7.8420181345463016e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3702552325063548e+00,     1.6551444655027870e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.7114028287785432e+00,    -7.1505810975969894e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.2517641265531967e+01,     6.5860839128897387e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5747659089666710e+00,     1.7442270803115323e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.0573391364614217e+01,    -1.5005817494529765e+00,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.7547833821893093e+01,     3.1088138412684100e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.8614774606479250e+00,     7.8489294061041037e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7661307734262593e+01,     1.5480621140211026e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6233757962207296e+01,     1.3697646436945241e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9408388095982012e+00,     8.9543691774544859e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2903477612008977e+01,     1.4350573469905543e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6918670408595542e+01,     1.4956333138113461e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9626943923897926e+00,     1.3107076211013909e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7973423180419523e+00,     1.5542530633127924e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5138117621767822e+01,     1.5542977947239990e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0327969678605605e+01,     2.3226217449054759e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3750811777683978e+01,     9.4739729063011237e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.5342284948051940e+01,     1.6867284631239047e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.1458774153776766e+01,     1.5638314264061748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2659593431789183e+01,    -2.0554756805361194e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5689041039613684e+01,     1.6866579999322901e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.3038782685383872e+00,     2.4044946764337397e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -8.9378665821933581e+00,     7.2366047705898891e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.6063052677941720e+01,     2.8398848101763542e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0559496660427055e+01,     3.3350814115302914e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.2171332014956143e+01,    -4.2577557599925058e+00,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.8125798083710538e+02,     2.5800432088522939e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.6731486287504339e+01,     9.4884037942535324e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2829512636109428e+01,     1.5542202000651476e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0060428824192051e+00,     2.4420123496158805e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.6725777508467878e-01,    -1.7251927757561287e+04,    -3.3279092177154333e-04,    -5.3840566415853768e-07,     2.6009720696477380e-09, } , 
      {    1.3522766134997234e+01,     2.3418936178353877e+04,    -3.1305027830412529e-04,    -1.6001343418935591e-07,     7.9862872875492742e-10, } , 
      {    8.4495925539599970e+00,     1.5558753798508444e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.4920616883626678e+01,     8.7579245002811076e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.8942737533250966e+00,     1.5543306579716442e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1046442921680342e+01,     5.9728668064069352e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7704751258268039e+00,     1.2714055241320923e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2531532012823302e+01,     8.9750232179718751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5336933064878515e+01,     3.1125323482265831e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.3151864313535370e+01,    -1.4200187019284203e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.6661903352245355e+00,     1.8319536579254382e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6748791119197683e+00,     1.0826502653868109e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9679290179683044e+00,     7.6048238658017926e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5287098816907985e+01,     5.9597488952074800e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6180032919600638e+01,     2.0977832594928543e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.0496893791902071e+00,     1.5555872201383414e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8187221736815410e+01,     3.0666154965317764e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2884476165448158e+01,     1.6972851558131115e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4650705165248006e+01,     1.6442766311672262e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.8927532653634628e+01,    -8.3288971399523343e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.3472515317803015e-01,     8.9033185670926014e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0212071964857287e+01,     8.3451998554612346e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.8701766726081893e+01,     7.1388118042864444e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4456833856755509e+00,     6.6736692666344670e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1276133215793021e+01,     8.4817508935890874e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.0001105806385134e+01,     3.1337310839808197e-01,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.7362567541553361e+01,     2.6398130993358245e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1587357128263989e+01,     1.5371902024644070e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.8230606472281611e+01,     3.1660847068202267e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.0805844595115950e+00,     7.2761960154317676e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.9956345795007074e+01,     8.2998410293780798e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.3943175390913723e+01,     1.3502398380546631e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7440088223492051e+01,     6.8574597168560485e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7095205544903695e+00,     1.2791607502333045e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6543312751645392e+00,    -9.4770770311288652e+03,    -4.9229707116567030e-04,    -6.5031937556670932e-07,     3.0764336948486714e-09, } , 
      {    5.7133652423392505e+00,     1.6062837941339214e+04,     2.7646008079384960e-04,     4.6323879356100805e-07,    -2.1443842410523998e-09, } , 
      {    1.7753474881765246e+01,     3.2443443652602378e+04,    -2.2506001311919634e-04,     1.7079985985488949e-08,    -9.4441705080137653e-11, } , 
      {    6.4418155845197909e+00,     7.2138392084027992e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0614530387796073e+01,    -3.1151595186780469e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4305986910806750e+01,     9.2129491414267359e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4601664576781699e+01,     6.6924048203025486e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2088490387516728e+01,     2.2617131744657730e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.4739123415408386e+01,     8.9725410380591002e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2780884515870493e+01,     7.2142865225148671e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.2281399754246358e+00,     8.3306403086067858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8872310193439130e+00,     8.3493549410286359e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8014265184268190e+01,     1.3224479658037770e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3951797401645420e+01,    -1.4017302861091393e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.4594738792396917e+01,     8.2926087973824269e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.2775816137420737e+01,     4.2894492391198983e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.0869281281186218e+00,    -1.7422247988449603e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0051384513706681e+01,     1.0630045376062535e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8057658177105246e+01,     3.0459502195162113e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9716719521118712e+01,     8.2288324645976354e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.0037803770910090e+00,     4.0676050866744572e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4658744319117918e+01,     3.3312464499060770e+04,     7.6156375320674127e-04,     1.2520555721494321e-06,    -5.9316953883751721e-09, } , 
      {   -2.7946948279140486e+01,    -3.0668181677752727e+01,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.3628957917865518e+00,     4.5909101215648133e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5484495232436011e+01,     1.5526754781859474e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.6310061474269503e+01,    -6.1276555675711370e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8376223196055790e+00,     8.7251094752258232e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5436068407053149e+01,     9.6835223721318453e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.4513577958929666e+01,     9.3284745159715385e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0941094297220481e+01,     9.6474397447891479e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7919394650186913e+00,     1.6577200756121943e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3012940140618737e+01,     1.6661792076619320e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2588252032564284e+00,     1.5654087978699048e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7755664828434168e+00,     8.8930092858371136e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5408742044350554e+01,     7.1983303813112434e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0209138598146986e+01,     2.0295391982776064e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.7731961152760460e+00,     6.1778546374826192e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.1582279411785848e+01,     8.2462081315570849e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.5967305068382931e+00,     8.1916205866882046e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3965023621048445e+01,     2.3555977006228222e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.0930092001090330e+01,    -1.7425838283537619e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.2555870170349365e+01,     3.7497202633761481e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3832646849876120e+00,     6.6195423377133147e+03,    -2.4676422471716020e-04,    -2.2400399195900355e-07,     1.1066308603638158e-09, } , 
      {    3.3414393269996081e+00,    -1.9186985528425330e+03,     3.3591115838248028e-05,     3.6307017839139590e-08,    -1.2014167831575411e-10, } , 
      {   -2.7652727668228167e+01,     2.4080642864781734e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.3388725235760546e+01,     2.4081347496697879e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.9384059687383495e+00,     4.0698951079149329e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3072337104245491e+01,     2.4502054499794362e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4486515211659823e+00,    -2.8936131199805400e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.0392056054446201e+01,    -1.0822044268987884e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.5505958224499409e+01,    -4.5744238736733678e-01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.0143265676643836e+01,     4.3561886590900474e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.5318036057434923e+00,    -1.6638195179742026e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.5615519442605599e+01,     8.3023830340985678e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4923168374777099e+01,     7.6823348963242097e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2835912335504345e+00,    -1.6621334433047843e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -2.9467901639579161e+01,     7.0041610836700374e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.6310908555358107e+01,     1.8083511027502664e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.1602907755473979e+01,     7.0048657155861856e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.5810941688735245e+01,     1.6871999387228243e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.0975047861470557e+00,     1.0433481684175122e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9102819746475479e+00,     1.8033683987325276e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.6328882199955402e+01,     2.5597461178412141e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8634089731195690e+01,     3.1677767749011051e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2195095672305795e+01,     8.3325047165382621e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9450603139570735e+01,     7.0689477494407802e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.9404727323759019e+00,     8.3248781329119902e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1480125889686411e+01,     8.3549998153516844e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.2011367353350053e+01,    -1.2569674860491614e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5180562602416092e+00,     3.6768741382219459e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6098473710224850e+00,     5.9044785981565547e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0810846225417997e+01,     7.5169457054160757e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.3523037258721645e+01,     6.5836777376295886e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5023977417007590e+01,     6.5932334469823463e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4159565695799960e+00,     1.2321034271627936e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9316839119221001e+00,     8.3301141156588783e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0789583328122472e+00,     8.3272687337913721e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0934429636580791e+00,     7.2300623737833184e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0695866252460696e+01,     1.6100068569817106e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.3503121009788515e+01,     1.4071522582314881e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4293252613598526e+01,     8.0181537482001822e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4245001690836771e+01,     2.4106732330261639e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.4108192852203914e+00,     1.6719515999423187e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4334322527008592e+01,     5.2670069603492684e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7046996303146834e+01,     8.1764316939761038e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4953745514889704e+01,     8.5378885745977732e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.6178967456372092e+01,     8.5385932065139223e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.8708614528192829e+01,     3.2042402785293780e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.1954106531282134e+01,     1.6676570519158482e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.2730893360829345e+01,     2.9881862507488026e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.7623433996788336e+00,    -1.5840288526782624e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0188222713253516e+02,     2.2756810442092064e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.5194795456682435e+00,     1.0040460714482133e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.5580811266856550e+01,     2.2756823869193518e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.7682279406258932e+00,     1.6737564942778561e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.9559410368818504e+01,     7.7003771261288557e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1534032884801009e+01,     1.4913735359671540e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0288494621181837e+02,     6.5550296898800298e-01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.0160966300322883e+01,     8.8202813073100770e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8996641829814669e+00,     7.3253965539739211e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2142893253053860e+01,     8.3434099331496855e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2191173424134291e+01,     2.3855903686620659e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.7516088191001217e+01,     1.4102020837711461e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1294311850692209e+01,     8.5574285602640739e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.1153076271023307e+01,     1.4912369162354715e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4180759663557236e+01,     1.3922677021103285e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.9188519157101425e+00,     9.1768665140840349e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.1248866177181910e+01,    -4.1188014535771771e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.6644731064550598e+01,     3.3317066898740231e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.0713903781474201e+01,     1.8136166723694438e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5367292268978048e+00,     1.7461477223392008e+04,    -9.9877169425213123e-05,     2.7683539930508054e-08,    -1.8786045329313413e-10, } , 
      {    1.8154345557553395e+01,     1.1237572566916468e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.4346780158214010e+00,     3.6124397884092768e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7181670760301721e+01,     3.1611644456486581e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.8039892012538115e+00,    -1.2466257640848908e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.1596378025151321e+01,     2.0809761250533225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8882685919395836e-01,    -5.3680442678067706e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9967282919187411e+01,     6.9370895206518735e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3966679127911256e+01,     1.6850011634472190e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    5.4765909862668209e+00,     1.5713606555723845e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.6874792633407143e+00,     8.9047412580263535e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1830152690896654e+00,     7.7184175236669080e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4051105497648610e+01,     2.4952292846672412e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    2.4310136604508443e+00,     2.2677187351846966e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.3285902218634902e+01,    -4.2937228324262833e-01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2655527507977880e+00,     1.6639750823174592e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7215895574238207e+01,     2.1577754246563829e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.3276331506484080e+00,     5.3717999262779067e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6201565473016366e+01,     1.5764922994337059e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0159601877178552e+01,    -2.4607898955102428e+01,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.6264645303194389e+01,     3.2775475627468564e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.9980601551333194e+01,     1.6029427580362784e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8690972780371823e+01,     3.0397872092521720e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4729509378910514e+01,     2.3294953241353829e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.0687434208493221e+01,     8.7151534388183863e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.3114736171331870e+01,     1.0723878087415005e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.3595841429551960e+00,     7.1824236762254932e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4217049813267964e+01,     2.5004098482376583e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    3.2329552043311320e+00,    -1.0796836702563071e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.3548737091166387e+01,     2.3886987743197511e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6726384469831723e+00,     7.2065846988897756e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3300718791602772e+00,     2.4802602770313111e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.0479807822720907e+01,     2.5934124335104807e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3641872278712098e+01,     7.1721143949700054e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9679517559343765e+01,     8.2380212525135485e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7704267799313367e+01,     1.0042761914321869e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8957088509439338e+01,     7.2150292679065096e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {   -1.3297576672022231e+01,     1.3187483200640218e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0989883792370298e+01,     5.4155706033138745e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.7803194391995207e+00,     1.9804827290144201e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2021699982622369e+01,    -3.2802776907677980e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.4344486423481424e+00,    -2.6906804285552005e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.1960151851715430e+00,     7.7996886891237494e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.4144973769587290e+00,    -1.3302114994919630e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2437771058233807e+01,     1.8419446729355717e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0993040304563104e-01,     6.0687813838828351e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3998019283649617e+01,     1.3030158359164298e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8677587153520903e+01,     4.0564020801536662e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0012993709023869e+01,     3.2160322237736302e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.1235770492230575e+01,     7.8374073954623964e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4289108962128250e+00,     1.7284979089028919e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3094518291338765e+01,    -3.1809920443429007e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8291056238869288e+01,     2.4815222008635490e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2171749902034023e+01,     1.8529187693387426e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6540466630436583e-02,     3.2838531685289581e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3365982164388939e+00,     2.3543230485485988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4681185656724836e+01,     8.3441212786165615e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1895063381564253e+00,     2.3203323227965629e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1828120055593516e+01,     6.5854266479368407e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6569533514700318e+01,     7.2215410320278897e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7381899607690006e+01,     1.5541787887736280e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -7.0157558721477198e-01,     1.3611430869221986e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -9.0794271293557856e+01,    -1.1147266108401323e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0530753380522972e+02,    -1.1145305076924546e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.2381604543090869e+00,    -3.6485224157519751e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7121934763954251e+01,     1.5618005351356347e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1155053776655286e+01,     7.9663233773418406e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.0957509411627235e+01,     1.2088048856115114e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2067647241566402e+01,     3.3558072211868668e+04,     1.4575360938688111e-04,     4.5395529030691401e-07,    -2.1583905082996538e-09, } , 
      {    1.5878320707249134e+01,     3.1747627603079003e+04,    -1.6073752766277704e-04,     9.0397680240530507e-08,    -3.8771034892010708e-10, } , 
      {   -2.2076682424250144e+01,    -4.8451400281340284e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0161966769702984e+00,     1.1928013301934949e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.8220028903854285e+01,     6.6012973087017181e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0581419000547978e+00,     6.0626633160081496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7774626634937896e+00,     8.4273068741273401e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4101000452281678e+01,     1.5479475911717280e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1074681213734195e+01,     5.7428311575897551e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3642864685266769e+00,    -9.2260670526261893e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.2564070571843075e+01,    -1.7315346537962250e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5285871120187764e+00,     1.5612644068203930e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0863083950736451e+01,     7.2457020546921722e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7410600530912657e+00,     7.6938248840798551e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.9558074308196023e+00,     6.8341598069653000e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1179244245341889e+02,    -2.2697202727468380e-01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.8041399998798564e+00,     1.6793335985880098e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.6439341178192173e+01,    -2.3079133904607261e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2900142970824128e+01,    -1.1059920608828652e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.7889834398489057e+00,    -8.3095132308030188e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.2467220772017818e+01,     6.3658877760999239e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.2439694924285654e+00,     3.0454899795482641e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.9673664246512537e+01,     3.2239952041532124e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -8.2219666934759461e-02,    -2.5019855701678345e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    4.6738311737461951e+00,     7.0934061626118910e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2550596764024391e+01,     7.2147674973749827e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9246600778139491e+01,     1.7743145754001449e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0649738702508049e+00,    -5.3954940927056612e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.6040018811285435e+01,     1.4914348653564577e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3989968829579610e+01,     1.3437964030733963e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0816392301766580e+01,     3.4910678069698406e+01,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1383323675635516e+01,     1.4624436658478407e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7141304286247646e+01,     2.4979515318575654e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.8715273964560744e+01,     1.4505677991929790e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0461907022885878e+01,    -6.7903889734561399e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.8045786653288626e+01,     2.0597243910167081e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9841827493110941e+01,     8.3284371603408908e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.1989117169731060e+01,     8.3289456891093614e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1664408920324647e+01,     1.8199897324468848e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2457654391273792e+01,    -8.6572780446054439e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2665639564659260e+01,     1.4252393332549549e+04,    -3.0031056255808542e-05,     9.9681183494624578e-08,    -3.7370408167285253e-10, } , 
      {    1.1667062020347416e+01,     2.2064075220784580e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4427379525162799e+01,     1.0593517238355289e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1866143727074890e+01,     7.2560113359476600e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1416467298224031e+00,     8.0066637604445350e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7041778532540913e+01,     1.6013327520889070e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.6988731449317989e+00,     1.4983138810711076e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.4282199089046820e+01,     1.5774643824728806e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.1548267775113370e+00,     1.3812936057071576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7083867974877354e+01,     3.1623644666078955e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    3.1277441074627230e+01,     7.1877544748322744e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6979261958497869e+01,     2.7801655411838961e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1813518729259449e+01,     5.5939855963781602e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1442692768468451e+01,     7.0022121997883769e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    6.5655636357519196e+00,     8.2764336729808110e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.3333297543379956e+01,     7.2133582335426827e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6885248054240815e+01,     8.8663288375590928e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6738024390213120e+00,     4.7783847862887842e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9672734180584541e+01,     3.2593158109327196e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.0341930992301940e+00,     2.5588644709699089e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    5.1473562351359092e+00,     9.4917567700018679e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9543708391384488e+01,     2.3245767962179729e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.4417238609024174e+00,     3.9609654580288402e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5857807255148888e+00,    -7.7341708969796082e+03,    -1.2404939778891725e-04,    -2.1287131236242152e-07,     9.5731794285570074e-10, } , 
      {    2.3725073813788192e+01,     8.7025675700947650e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7159255190232336e+01,     2.3028521592085548e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5360275043410308e+01,     1.5553063571439447e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9599647396933477e+00,     7.8070333795688421e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.8845076717406197e+00,     1.1087285051798765e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2870384967492434e+00,     8.3809491764694412e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3250324568464240e+01,     1.2543139702513527e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7630265282916393e+00,     4.1948470488827297e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8220443763422139e+01,     1.5702013897775607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0599645575790863e+01,    -4.0765757287260431e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1168708921721464e+01,     2.1986750431336623e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6419269375135926e+01,     1.3013776206486147e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.4814099002494743e+01,     1.5560027286950815e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5593593571785309e+00,     5.4369929860101229e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6537444021271313e+00,     6.2860343152480646e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0607520322102030e+02,     1.6656874890912219e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.9665298493201269e+02,    -1.6657890807988286e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -6.3622982778808698e+01,     9.0123559399570113e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0719224444405222e+01,     2.4620285252171750e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.1858230780284568e+01,     1.5539909376126168e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7585697805757277e+01,     2.2826706933662765e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.7760027166741153e+01,     1.6786251549075590e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -9.8569764358561187e+00,     3.2194188187162890e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5088083825444318e+01,     3.8838861517258694e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -6.5869670803912982e+01,     1.4929988733426844e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6411680344277691e+01,     7.2625142657401730e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.3254483196930984e+00,     2.8908321988359694e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0945417896538856e+01,    -5.6072867250284617e+01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.8221443995654010e+00,     5.4842446323464510e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4619779864010187e+00,    -2.8406440227693488e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0493276833141664e+01,     8.2484923654496670e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7517914778085739e+00,     1.5549313245783687e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.3603969149813949e+01,     1.4987741210390544e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.9528563741498530e+01,     1.5859207582252566e-01,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.3749143530773184e+01,     8.1306166872820022e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.9794707773581079e+00,    -1.6383415750240147e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4814937566206957e+01,     1.5532445008928471e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0752713330432746e+00,     1.5545599204241749e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.3927791357723915e+00,     7.9548152793554855e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9708513041186526e+00,     7.3424577946792590e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1128018426293849e+01,     2.9671159055936398e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0706885513460922e+01,     2.4828339919834947e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.6350289707464278e+00,     5.3339001734552448e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3539121668371568e+00,     4.4629160776079198e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1184589036789205e+00,     8.9673082910261564e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.3436618599965158e+01,     9.1543425507818047e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1308631180959060e+02,     8.3266559077080510e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3622841243500638e+01,     1.8922208663080415e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0523367982086884e+02,     8.3307269417422012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6413579344317363e+01,    -1.7578329769580705e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.9604221705269964e+01,     3.4960327176015744e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4128780180895994e+01,     8.4248296592570587e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9384083340609514e+01,    -7.7594658851739950e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.9671783215734990e-01,     1.1534992332241960e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4944045165101926e+01,    -8.2799895590600518e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    9.6518613233607233e+00,     7.9379304243821662e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0146684285830293e+01,     6.0768902253456770e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4198188013024151e+01,     4.6824702088128852e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.7687067069662135e+00,     1.7319441851836182e+04,     2.7113231780803260e-04,     4.6447157778933341e-07,    -2.2532636975559792e-09, } , 
      {    2.6345756517205011e+01,     1.0280678361990611e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8878188022321059e+00,     8.3260884007366458e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.3214030526916236e+00,     1.0371348534202187e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.1847071471525283e+01,     8.3283032829315907e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9699780041621963e+01,     8.3290795665186615e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3266629128048930e+01,    -1.4088749008401089e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.1414033101336766e+01,     1.6099716253859029e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5949760265527043e+01,     8.4287295650610922e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7186411825497650e+01,     2.2502385715966891e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.9861056161307630e+01,     7.0068145994678453e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.9663522584707678e+01,     1.6100420885775182e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.2055170341347770e+01,     7.2403712560853910e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8733055534360165e+02,    -5.3779714259599132e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1492750768432884e+01,     3.2253106236845404e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2418326982368395e+01,     2.3400872484203974e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.8951443653945759e+01,    -8.3408973495447135e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.2927105676285620e+01,     6.2184714602421354e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -5.4240296745643647e+01,     8.3164854999055406e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -2.5967277535622046e+01,     1.3799465829914046e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.3387225652571715e+00,     2.3162956036535234e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.8203302434205085e+01,     7.7007124721560322e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1875341165773744e+01,     1.7117692897570068e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0086014007112443e+00,     9.5213168708826579e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9037880359144426e+00,     9.0525589836484596e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.1706680293673742e+01,     8.9570057233213956e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.0472978236926089e+01,    -9.6902800562680568e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.9526678443883512e+01,     1.4428119017366940e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.5500778056793905e+01,     7.7713704315412524e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.7936351324444324e+01,     1.4428132444468392e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.1962256043618893e+01,     7.7713838586427064e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.8808382867629385e+00,    -1.6830883898878565e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    7.4546929570064666e+00,    -1.6883852996089758e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.3875108000855221e+01,     5.3689550122201153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0475975899285967e+01,     1.3330417852460152e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6953789178148988e+01,     1.5021096245027675e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2788909031784346e+01,     2.3791263621580780e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5183466396978091e+00,    -1.0012108992012165e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7792941484390724e+01,     1.8012213796856973e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7039118087741986e+01,     8.4073996570832405e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6388920687512872e+01,     1.9015508667608177e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7639029935977931e+01,     1.5561941959892183e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.8020442361869939e+00,    -5.2355723249570110e+02,    -2.1623611323479412e-04,    -2.5540469534531464e-07,     1.2058285876556379e-09, } , 
      {    1.4247588705612517e+00,     4.3853638165957964e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7295661982735247e+01,     3.1649381815600795e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.1572027398889286e+00,     7.3019768516800104e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9621152927347605e+01,     3.4596364662247634e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4235116355263596e+01,     3.2986179079020185e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.0202232922468912e+00,    -1.8888594952721269e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6048869646815412e+01,     9.0171499334503096e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.3233213385650906e+01,     1.6673115333597842e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -7.4942166570916946e+00,     9.5698874837770236e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5377577183559366e+00,    -1.5256832129466065e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.7475564001181851e+00,     2.4978112291429834e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.0027689146997226e+01,     9.8224650377252601e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7386215962158570e+01,     1.4921924871707473e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7283987956641880e+01,     2.4867037120125784e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -2.7978770889458158e+01,     9.7601082306962908e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5157987458649794e+01,     1.0522523932942891e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8313047667529993e+01,     8.5236376569749900e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.0673094984412700e+01,    -2.3888941138747350e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1973120838216706e+00,     4.6868894794509271e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3586788740582056e+00,    -1.6663941805049977e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.3942510591452361e+01,     2.3700593449369197e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2728590316395554e+01,    -4.8632097422075026e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.2420021507042243e+01,    -8.3406890352725022e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    7.5892121654141249e+01,     6.8246446918007632e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1457373159857623e+01,     8.4798668234658326e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.0336390156611472e+01,     1.9800945872208840e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3332511459106376e+01,     3.4886834119484905e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3016349506814128e+01,     2.3687974211046818e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -2.5305633720168291e+01,     3.2409334289506860e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    5.5752743945523321e+01,     3.2410038921423005e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.5756900824297297e+01,     5.6600279300422571e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7873931387160141e+01,     3.2147168042423018e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.5029734926583558e+00,    -1.3162864991988855e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1263243773591352e+01,     1.7430306877636773e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2931181714231659e+01,     1.4718508424558383e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.5600534483963301e+01,    -1.3292450970442842e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0368098443260685e+01,     1.8849227551366296e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1799556741880671e+01,     2.2779437248296172e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.2692223605544335e+01,     1.3517870021753406e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7272523952693355e+01,     2.2657490360773703e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.2898614416975235e+01,     7.6251327785071953e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7252883828002652e+00,     8.4557217024118054e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1755634947733732e+01,     8.2698222734275187e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.3727633606543634e+01,     2.3871347663335247e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1237421196598734e+02,     2.3871543766482922e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.1090650832037419e+01,     1.7268417437728938e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4040673018220065e+01,     8.3290460283232351e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2842577069761472e+01,     2.5143808628515810e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -9.9733739446031979e+00,     1.5110466097669896e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3917474681555504e+01,     2.2836446959438617e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.0452975160654063e+01,     2.1047902594499956e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4632545968966946e+01,     2.5055904118080751e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.1829290463056076e+01,     1.6473349399443396e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2187218198836796e+00,     5.9452615754257886e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5949838641403868e+01,     5.9551464955280117e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6108246146559232e+01,     1.5606032668650638e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1318051590020481e+01,     1.5542743767980148e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7794348255336260e+01,     3.2197507307318025e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9043331783544300e+01,     3.1049460163965508e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.8751783749238736e+01,    -8.3283368211270172e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.4982873622814779e+01,     2.1723767199718168e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2617298412745630e+01,     1.5542764812387770e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5227329007065986e+00,     8.3997091105030413e+03,    -3.3191992975274604e-05,     3.2017095500473753e-08,    -1.5363745554361197e-10, } , 
      {    1.1196847261803423e+01,     7.2670224868839032e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2608509125173825e+01,     1.3294369436057743e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6472725432520924e+01,     3.1607877970991078e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.1279266080921307e+01,     2.8263978490233003e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.8012006786721614e+01,     8.4464051619404363e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8970586105286266e+01,     1.6345863511109581e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9125144434462932e+01,     1.6148519970098445e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.5322019560422788e+01,     8.3850147573591457e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8780451176442359e+01,     3.0328340690288123e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1846279245618355e+01,     3.2202766971950397e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7111522305021914e+01,     6.1518967384608823e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.8148041959529504e+00,     1.1141971362548973e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1850665758964323e+00,     8.4608286444530918e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5304813718511614e+01,     7.7375957175890153e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.9266131719101214e+00,     2.5770962227840449e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.4798268236627401e+01,    -1.8658474968747760e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.5081881661846275e+00,    -8.3636363137351127e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -4.2438228372130897e+00,     5.4999923758620898e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5806085177678856e+01,    -1.1167061020499941e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.7215254895787751e+01,     8.1296169152355651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7181944936745392e+01,     2.3285797945819246e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1931130899203477e+01,     3.1121556996770323e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -7.1620438091745299e+01,     7.7003905532303088e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3772782821207269e+01,     1.6164502919629775e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    8.8540785355080871e+00,     1.5500094474228255e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5635991686311268e+00,     2.3345309838790421e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.1237695120048574e+00,     1.3218409899528999e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.5840394512965830e+01,     1.2562628541330114e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0000494489612496e+01,     2.6145729481331728e+00,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {   -2.9350981041186923e+00,     7.2014436271364539e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -3.6090551289006143e+00,    -8.6019711959039023e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.6214360605466354e+00,     6.3504686650607346e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1316361108930352e+01,     1.5543720692631636e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    3.3018950213212136e+01,     2.1202782220226214e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2904965815800082e+00,     1.4868046039313071e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.3348198829090769e+01,     1.3802119345711857e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.1445367962685111e+00,     1.9178193922106630e+01,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.0319403201960696e+01,    -1.1125510164825928e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4955019098053143e+01,     9.4094786223738574e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3321488188441499e+01,     9.5683859140074328e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5652659757545162e+01,     6.5688345996045082e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.5427845245263788e+01,     1.6960368056094171e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7470932038238214e+01,     8.1091372935201616e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.7588386015796779e+01,     2.5201657214401046e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.2701310097261668e+01,     2.4333768793303152e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0857650948667379e+00,     7.1948751957506056e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2770486073998983e+01,     2.5039043371386571e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -3.6196427380218935e+01,    -5.3321413169266265e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1978068153137382e+00,    -8.3345914788115424e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.8907423809689021e+00,     7.2054263670754026e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.4392938730268842e+01,     1.2254635749043055e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1468629857783220e+01,     7.6047826353120872e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1369186609352262e+00,     8.2654130462584471e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2315218560828818e+01,     1.5610268384458850e+04,    -1.2784672518689175e-04,    -9.9871618308564415e-09,     3.8979019961206658e-11, } , 
      {    7.8369414618581166e+00,     2.4533564737620174e+04,     5.7763344201952167e-05,     2.7686187013206913e-07,    -1.2653200744645889e-09, } , 
      {    1.2504924575548188e+01,     4.1819249542318910e+04,     3.5972496399174012e-04,     7.7830049110600470e-07,    -3.6924379580664477e-09, } , 
      {    5.7447150544456029e+00,    -9.6191124026846956e+03,    -1.2128758393242458e-04,    -2.1353133770788403e-07,     1.0110304505858264e-09, } , 
      {    6.0018772283229653e+00,     7.1430751898434974e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    3.5365268337325524e+01,     1.3895419350239619e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9577932681838000e+01,     1.1630593536609456e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0168815625445502e+00,    -1.7429361443118366e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.0703563834030358e+00,     9.9629084534700087e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.3412528198423232e+01,     9.4827049369287408e-01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.9348317467371146e+01,     1.3879447949667947e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1215239063419348e+01,     6.6114924299351242e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -6.6955739527709834e-03,     7.6790539317017310e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5139467176723642e+01,    -8.3601716523883588e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.7410103229527554e+00,    -2.3919072456608892e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.6260099825357868e+00,     2.4016560830927137e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -2.0406872551755530e+01,     2.0403559096373265e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.7539446984538323e+00,     2.4086062252687079e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.9124120098416562e+00,     2.5772116213561356e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.2701581051179858e+01,     1.7823327847329761e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8005708427953948e+00,     2.4983444441859196e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.3757612025993346e+01,     2.4988704106491561e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -4.6455093164992736e+00,     1.0355929423162997e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9121657223862901e+01,     1.0435782884014856e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3780823992799469e+01,     7.9219309160576804e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.6279320480820090e+00,     1.6753521083982185e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0586616700012183e+01,     1.0764369503895176e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4395219701058238e+01,     1.4111337484212794e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7387567243159969e+00,     9.2512453513444852e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1145800576322905e+01,     1.6862001283125053e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.7082199516150645e+01,     1.6871863347436894e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.3706346916652535e+01,     2.3010152714350719e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0503840584044468e+01,    -4.6899338702421119e+01,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -1.6852146086323394e+01,     9.7999231325942037e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7517733368554545e+01,     1.5472864512163987e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.4407367135396107e+01,     9.5680335980493564e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5068136032771527e+01,     1.9315229632773404e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3889415684356965e+01,     6.7589087457929118e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1968317992528753e+01,     2.2872765975282823e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.5357803783188547e+01,     3.1478529550060906e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.9318996598398090e+00,     1.6910703148790133e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.2781195972646913e+00,    -1.3433562190191219e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4402097831976498e+01,    -7.7342270106359592e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9733995710140743e+01,     7.5733024076488537e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.1087627319575148e+01,    -1.5898543728578431e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0683015785226843e+01,     2.2149839478575101e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0618866033816334e+00,     1.3521751439688769e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6092324682348291e+01,    -5.5766659559122161e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1037565635045056e+01,     1.4679160089785861e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5173924323345268e+01,     9.8669938759941124e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.4577676771261112e+00,     5.7352789912146454e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9960463580563941e+00,     8.3919698031918051e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.3521038731247383e+00,     2.4200032334789401e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0705353477412515e+01,     2.4463704883552218e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -3.2337412109631209e+00,     1.0748950392855986e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5190538449392687e+01,     2.9987800701910408e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.4919111224262480e+01,    -5.5696196367507184e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.0519362267047683e+00,    -8.4929847346652859e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.5223488632013389e+00,     1.6029075264404710e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0720777898602137e+01,     2.4933105176964185e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.2899277450246389e+00,     8.4514903658416915e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.1008628637270661e+00,     1.6029786609871586e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6859884405921811e+01,     8.2595129921720309e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4614933758111594e+01,     1.5490496538439644e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3401532847045026e+00,     7.4559084364111159e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2694283539442393e+01,    -3.8040173137060856e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.0068872570648701e+01,    -1.6234914386180999e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.9866852470983538e-01,     1.5320585586030858e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9174356319946025e+01,     1.5814458726626717e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3053461356150827e+01,     2.5008616179306435e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.5531704150645869e+00,     8.2491309591718218e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3117858453897487e+01,     2.5128109645731209e+04,     8.5928764690799039e-05,     3.1444530543083391e-07,    -1.4936139887622584e-09, } , 
      {    9.9596807437377315e+00,     7.2815769597337239e+03,    -2.8015947582823997e-04,    -2.6039827626074286e-07,     1.2253180976362411e-09, } , 
      {   -3.2110658173804637e+01,    -1.5339513482122129e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3967821878899791e+00,    -6.6209464234556845e+02,     2.8263352852431007e-05,     3.7539802067464924e-08,    -2.2902113481933369e-10, } , 
      {    1.0419858862689409e+01,     7.3008039143868673e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3956431969916672e+01,     3.1021074230555252e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.0585027058780426e+01,    -1.3241847386477416e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.6878006511365712e+01,    -1.3241713115462885e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.1631496135708510e+01,     2.4225601679705933e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.8797427183008377e+00,    -1.4038078288928396e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.3703766630104845e+01,    -3.1151933834067131e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.6173654691059873e+00,     4.0526527364519607e+04,     5.4306288134201204e-04,     1.0655913822578935e-06,    -5.0540856628306906e-09, } , 
      {    2.3391485900425444e+00,     6.9323755842809333e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4587680862285239e+00,     1.5605111097212928e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.0746923401127717e+01,    -1.3791187595379597e+01,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.3720759619987426e+00,     6.9765162807562501e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9509417815774235e-02,     3.9923428469028086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.8530675252182505e+00,     1.2411960590518975e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.9865175726334584e+01,     6.1390404581289658e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.5709456243013506e+01,     1.6508430736108494e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.5398795106380607e+01,     7.1934443029478580e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6048185267485071e+01,     1.6735379736341489e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2879439535384684e+01,     8.5442743649507956e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.1368546923206011e+01,     8.5277659342146871e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1885187607557096e+01,    -6.3973680037631530e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6526551799378630e+01,     1.0037786207419400e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3984355646047248e+01,     8.3264139083670179e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4141276229467380e+01,     8.7240135942578490e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0721477582572071e+01,     8.9407896744287646e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9848459969506454e+01,     8.3309689410832343e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9086510774735237e+00,    -9.8363409536844083e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.6344751896271807e+00,     5.6757604141898473e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2273515031737787e+00,     7.0694464223178556e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.1184698574047019e+01,     9.7826501358273472e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4554097871783007e+00,    -9.1887879159112344e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.2496505735654196e+01,     2.5195295107529648e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.0124067834906075e+01,     5.6648780818101113e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.3024891629220807e+00,     7.7078620062486398e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8132223745357390e+01,     2.5195952372482552e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.0298827030334040e+01,     2.2927669421182680e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2018408361792119e-01,     5.4298795313413598e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9550672466546715e+00,     1.3797516946032389e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9831988839659680e+01,     1.7037285907943784e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2684569630683447e+00,     2.6046491407085114e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.3642195618507614e+01,     2.3141558306926117e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5454129277870113e+01,     1.5429853891434718e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.8446598654402962e+01,     3.3036747383636111e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6704733363298494e+01,     1.6974171096155122e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1442322054944478e+01,     1.4319907426824440e+04,    -9.1689660219319288e-05,     2.5747097125420347e-08,    -2.5995709581093384e-11, } , 
      {    2.0663890684841444e+01,     2.2094758153256866e+04,    -2.5119580961344623e-04,    -8.6166614282751386e-08,     4.4946591561983952e-10, } , 
      {   -1.0251093975582975e+01,    -1.1045917293183756e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.2840234231969223e+00,     6.2058522219197566e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6140586595395462e+01,     7.7000552072031342e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8136212519419539e+00,    -8.9907748074765732e+03,    -1.1872163480310021e-04,    -2.1410409659074682e-07,     1.0661973993592805e-09, } , 
      {    1.8210536045496038e+01,     6.5519794827073847e+03,    -1.8490975602648117e-04,    -1.5015717205239902e-07,     7.5746804722872778e-10, } , 
      {    1.0112248938476416e+01,     7.2346814279698074e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0983683112529363e+01,     1.5642081085053049e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.0328566611025892e+01,     8.3276277633001773e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8181450022949328e+01,     8.3297550861500749e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4686049393442417e+01,     4.0531129764199068e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.4684856821279709e+01,     2.8194676381531950e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5371715620669287e+01,     3.2016665635771944e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.3707208111809912e+01,     8.3263884898880024e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.8521957440471635e+00,     1.8422629391809260e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7678557315657972e+00,     8.2419409000109699e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.1784140798305067e+00,     8.8503494698814102e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7806112666145566e-02,     1.4348495927121841e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3295523884930113e+01,     1.6630152887385982e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2673567963662133e-01,     7.3591779814768852e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.1262968089867339e+01,     3.0456848679364299e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.1322337959409083e+01,     7.2604635312841738e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -3.2431429646049565e+01,    -9.4436722999494941e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.8885993852132685e+01,    -1.0754655094424379e+00,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.3219298263310549e+01,    -5.0205345836834022e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.1076665014059152e+01,     2.4968801277408522e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8515166857459291e+00,     7.1883237303119104e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8516534086032394e+01,    -9.4429676680333450e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.3860698051522835e+00,    -1.7959052415230281e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1872051807416136e+02,     7.2120273484417576e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4040487080367766e+01,     8.0530644262698788e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.6203602677994562e+00,     5.1069714061691020e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6859829539606505e+01,     1.0076543341824834e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.4326089910561155e+01,     1.5631771803797561e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.8693957504458567e+01,     1.0120530647920214e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.9496449357311008e+01,     7.2160983824759078e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4497967617830199e-01,     1.1712955381796792e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3190224661516030e+00,     1.5595012041928274e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2627172443944048e+01,    -1.2445482213011906e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.1539600423847901e+01,     1.4485213629582435e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5622330270744669e+01,     1.7562663779302755e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5620529018725218e+01,     5.0754911785058730e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4702251887504522e+01,     1.3406797176179134e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6567594443089469e+01,     3.1243242934708349e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.9250291239757136e+00,     6.7500485903534491e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1227810593088439e+00,     6.1125522173919944e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.8149402670228874e+00,     1.3721821258980885e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2933810318810322e+00,    -1.0593996959660124e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -6.1052074951164478e+01,     2.1575502683377636e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.2380251457359762e+01,     1.2481069825808099e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3597836149632087e+01,     8.3375698131778645e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.8867286955667559e+00,    -8.3198130362723878e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.9310715832761467e+00,     4.0736076830350321e+04,     1.1039479014525555e-04,     5.5486925802986389e-07,    -2.6409740464760860e-09, } , 
      {    1.1654745161615963e+01,     1.6133825616954549e+04,     8.8389388047902374e-05,     2.4541753351445820e-07,    -1.1668495676944310e-09, } , 
      {    1.6440323691321609e+01,     6.4809561671324082e+03,    -2.0688939027668315e-06,     6.8853238991176307e-08,    -3.2967330315448447e-10, } , 
      {    4.0575293643022530e+00,    -8.4154419494392823e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.5876469918632182e+01,     1.0986538208048278e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6608070737159419e+00,    -4.0415433854449634e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.5170664875273872e+00,     8.3460042874594546e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7226430164769852e+01,     8.5502385011032238e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2009290891228098e+01,     5.7826460594876680e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4959140846423145e+01,     1.4592419040857822e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.0232830710161785e+01,     1.7166245728802060e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5713901075892135e+01,     1.5480397483154989e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2295344335578410e+01,     1.6363136507876437e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.8632602921811213e+01,    -6.4087775507394326e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3634718479747781e-01,     6.0273374733877190e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4138503852540726e+01,     1.8216348817022750e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0450015875392126e+01,     8.4706667759557968e+03,    -2.2382863485054624e-04,    -1.8523140566321331e-07,     7.6873026904090223e-10, } , 
      {    1.3818563952641027e+01,     1.6811820979800159e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9505223828908694e+01,     3.1102429261176701e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.2362514106361893e+01,     3.1574130754428403e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5412202220196360e+01,     1.3830985000426952e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6372440031557673e+01,     4.0740679230029797e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.5857547119802810e+01,     7.5879390108284724e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3891522031182239e+01,     1.4914452334935477e+04,    -6.3524239730472436e-05,     6.3330532424185123e-08,    -2.5428962387876272e-10, } , 
      {    2.8321892222703006e+01,     1.9053657519179978e+00,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7736846690768683e+01,     8.3236243527778570e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.0521661935104918e+00,     1.5243033325018734e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.3386152786995389e+00,     6.1156436415344433e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8545128349238901e+01,     7.7717057775684279e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.2083654645938694e+01,     7.7710485126155309e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1837310696913633e+01,     6.6007116135514025e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1347671012857077e+01,     1.3800170461830197e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.8543153529643011e+00,     8.3309943595622499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6670441242768007e+01,     1.4199398071164836e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.2548247217627047e+01,     1.6537938143756517e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0737003936542811e+01,    -8.7934194391373785e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4484762025962059e+00,     1.3095842673022778e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7572519953156224e+01,     7.8510069488878034e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1416743057610578e+01,     6.1266642682568081e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4219250567408235e+00,    -1.6610982183624910e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.4833291331787867e+01,     9.8232230424849513e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9129793667823827e+00,    -8.6741048928035525e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.5461751816479801e+00,     1.2825388929836010e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.3755803885763758e+00,     1.5538345063014887e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.8167369372989164e+00,     8.1360027055885867e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3086874218377801e+01,     2.2130107021398717e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.7421949505298215e+00,     1.8476324241350710e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0796572229524855e+01,     3.1085508580367918e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.6074991902350789e+01,     6.6218036968157003e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.7178501035235303e+01,     6.0530488997352422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0375115127325131e+01,    -1.6884981258636348e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.6141640669041035e+01,     5.9236418874992632e+03,    -1.8747570515580554e-04,    -1.4958441316953623e-07,     7.0230109845527369e-10, } , 
      {    1.3778321032068977e+01,     2.4604552413235506e+04,    -1.3030734854399507e-04,     5.9040610085519286e-08,    -2.8778540110662061e-10, } , 
      {    1.0892283101018002e+01,     2.5790168648117138e+04,     5.2435581216135132e-05,     2.7809465436039449e-07,    -1.3741995309681686e-09, } , 
      {    2.3433344846261910e+01,     3.8628158065707066e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.3863497128187490e+01,     1.6848883371925600e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    7.6936607483450992e+00,     1.4444335066259569e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7175264105973138e+00,    -1.2477367816383471e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.8791336865950186e+01,     1.6126640757574933e+00,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.8669639304447347e+00,     2.4097915861548590e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    4.1407569955054377e+01,     1.6776827555143987e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.5081304709775338e+01,     1.9997357440854557e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3159984642197546e+00,     8.3559138966283626e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.5756353520415288e+01,     8.9569922962199453e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9175328337459376e+01,     6.1565222048573078e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7665129016860082e+00,     9.4786570549269545e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3414012672721158e+00,     3.5993218772098217e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5543576293619648e+00,    -8.1271862368755355e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.5562739000897281e+00,     6.7020027560911603e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3542869810341176e+02,    -4.5036753191467938e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0312850310655556e+01,     7.9150003774575744e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5996008946677916e+01,     2.5214811409714326e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -3.7778849859018795e+00,     5.8930133455550776e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9413026515247648e+01,     3.0064180025629386e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.8884353442855755e+01,     2.5242024405831442e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -3.7075369727382309e+01,     8.3257092764508652e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6509151236177622e+01,     1.9708250602466389e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0845872477353094e+01,     8.6349713454900793e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3957510967219005e+01,     9.4800797458607067e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5897576224450169e+01,     7.1315795722907915e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1211262365371184e+01,     8.3316735729993870e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9750581739278950e+00,     1.6473911345587985e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.0648115322004061e+00,     8.3041082609241930e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6853991329304954e+01,     2.0890577753024054e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1152305881867708e+01,     2.2075540473386056e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3771350084934326e+01,     1.4455355692981933e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3218101795457834e+01,     2.6709646945226045e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6022016564218977e+00,    -7.2129593380664091e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.2288704234355997e+01,     1.4434684686517390e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -4.6135935238725921e+01,     7.2136747236652973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5360315349292350e+01,     7.2144510072523681e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1792103139627990e+01,     1.6688035771759955e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.0818773454495719e+01,     1.5536854236097543e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.9197477302411414e+01,     8.9566703772942201e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.0417838876610034e+01,     9.7791085680570086e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.2677702528243486e+00,     1.6639333906094878e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.7889444865079933e+00,     8.3532745885260592e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5012542077557015e+01,     2.9741976532549052e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0389580268880792e+01,    -1.8728558063467335e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4844720622138125e+01,     1.6703783515275598e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.9665681474046906e+01,     2.3977383909331467e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.3950156594903866e+00,     8.3323893064273216e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0539234865037066e+01,     2.1540016887449616e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1053836134015278e+01,     3.8526574359177503e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1824340586858430e+01,     1.7013985998053038e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1083754253377132e+00,    -8.5213801438616647e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.1727919307007175e+00,     4.7011163887884544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3473914020660562e+01,     8.3275631621785324e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0478486388140382e+00,    -9.5131498278967938e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8033411141520723e+01,     2.4465990623020123e+04,     1.1429004990681419e-04,     3.5194147426699899e-07,    -1.7233623441032565e-09, } , 
      {    2.5331877463722947e+01,     3.9380442957955602e+04,     5.0765810176341731e-05,     4.1527200669118408e-07,    -1.9776519679820193e-09, } , 
      {    1.4038881484479083e+01,     9.3964206452889994e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0801412271589397e+01,     1.5682473912109306e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.5245176876031842e+01,    -8.3575418200721724e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.2269315066965316e+01,     8.4023145624537065e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8066327637907675e+00,    -1.4481309519344827e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4148848460921505e+01,    -4.5424320840196799e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0842268912588233e+01,     9.2724677184515331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1744293715640390e+00,     5.8493642362296316e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5386733671956403e+01,     2.1030801692139736e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0862708262886084e+01,     7.1894797016578996e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8465095594511560e+01,     2.2756488523166339e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.4441511960484547e+01,     2.1595803189919199e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3459713462576154e+01,     6.5233984733843208e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1480908844628249e+01,     7.8953395477155673e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.4847674308238776e+00,    -1.1359584672441454e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5278607088764966e+01,     1.5959185486384737e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2670703705725789e+01,     3.9413847689134964e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.1637413998901396e+00,     2.2757145788119244e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.3053720448600252e+00,     2.4165692056482898e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2917352951104858e+01,     2.5508443664798539e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.5201622386054432e+01,    -1.9973675839877396e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.8394379151640649e+01,     3.9414552321051124e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.6865902235249450e-01,     1.6951629191024069e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4152112746239524e+01,     7.4336169966637990e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3135112817372685e+01,     7.1980633571343469e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5842513250767835e+01,     2.3951627808237390e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9190688185237299e+01,     1.5550232456753016e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.2613629136653959e+00,     8.5301966125747185e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.1495938036887745e+00,    -2.6867124380382438e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2008056969356101e+01,     1.1157390473588162e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1326321876426679e+01,     8.3298196872717199e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7811254462462323e+01,     1.9408529637301166e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7625532301536921e+01,     3.1007956319355795e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    7.4156769731788970e+00,     7.2139144484016188e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.4595984302980568e+01,     9.1145276488838917e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7439183613985243e+00,     1.8209330312030741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.0266016399895213e+00,     4.7139504364761151e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8093152344635467e+01,     7.2142112825160466e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4780321637587068e+00,     8.2580903012382787e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.6945303045126153e+01,    -4.3787426234357554e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6061418605471331e+00,     8.3805065362154164e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.8770616827991791e+01,     8.3363180083513962e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9381718711901392e+00,     2.1070345155162312e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0059556734590558e+00,    -1.1326622432536190e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.6345784651846689e+00,     8.3210648410988560e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5576888945770108e+01,     7.8266379663567877e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.5776201299619359e-01,     5.9743683761765278e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.8245830080379690e+01,     1.5153336116525890e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7770234324181647e+01,    -2.3314288274594674e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5453222335850841e+01,     6.1170663324681964e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3366514164370173e+01,     8.7811710373283131e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.4285493400371596e+00,     1.6342170287448835e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0542182833698604e+01,     1.0828803853707843e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6678927412525670e+00,     8.2445802081132751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4203731247454613e+01,     3.3588749514543895e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.2391007044520691e+01,     2.5182505828806643e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.7780542962688795e+01,     1.0673699331683596e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6842380116842236e+01,     1.5971987365739940e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7144160891793909e+00,     9.8754712723556634e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.0171254374927800e+02,    -4.9243186051921839e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1067239916756623e+01,     1.5535276123614902e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2635874370474687e+00,    -2.0584048529927004e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.2903700331034045e+01,     8.4065902600425234e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6021697115375122e+01,     1.4427414385450789e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7560097421763249e+01,     2.2738792947441587e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.5101242040016327e+00,     9.5639766868383613e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8906388318492375e+00,    -1.4300972679471574e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9124050524762879e+01,     2.4397581591027749e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.9417706967157020e+00,     2.3865412240514135e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    7.5931358428126732e+00,     7.1603824227807645e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6251535772372069e+01,     1.6957103814615479e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3440113392531394e-01,     6.2688489411179046e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9296024851091948e+02,    -1.9610314767777481e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.9297945019992824e+00,     3.1569528354748938e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    3.7854721704321168e+00,    -9.6190880223191762e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.7093305571647875e+01,     2.4985745641698926e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3636813750830058e+01,     1.5516445899557400e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6947547487022216e+00,     2.5155808838108187e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.1233133844977569e+01,     1.8383398312953308e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.5383489237600418e+01,     6.4663105747195532e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7061888513179522e+01,     2.5235438641904013e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -5.8526568610439782e+01,     7.2138571502316245e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.9283804818134080e+01,     1.4428837076384541e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1217562806708889e+01,     6.6659373737415963e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2285596297291129e+01,    -8.4377239134004813e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.0977837317051253e+00,    -1.0405129961705152e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.4833411599029503e+01,     1.4173694291241762e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7809427497602542e+01,     1.4445757757193322e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0988097325032033e+01,     1.5314026630431132e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.3535318816092474e+01,     2.4986402906651831e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    2.0198071275762203e+01,     3.4258550387217983e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5514108068196293e+01,     3.2689747906209732e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7527323034968934e+00,     3.2063009075168338e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8266069460142209e+01,     1.6083147889008324e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.5713567194505661e+01,     1.4985087694592736e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.2655493379880262e+01,    -1.6641650365302659e+04,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    5.5334950752840783e+01,     1.4985792326508885e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.8428731001771379e+01,     7.2081628113724155e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.5146712925637784e+00,    -2.5229405167509067e+04,     6.6570736180512717e-06,    -2.0358780910832745e-07,     9.7132421010295529e-10, } , 
      {    7.8701740095913975e+00,     1.5976106167193519e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6005467124672716e+01,     2.2739896474834004e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    4.3680959732276525e+00,    -1.1729372931423541e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0250879644158358e+01,     4.9656780113326167e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.8363556575808975e+00,     8.3195026368092149e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1511665344137908e+01,     8.9393669834950124e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0112624920228308e+01,     7.2936233310121370e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.6050951728913283e+00,     1.5533894134744496e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.9057639416746941e+00,     8.2691728477003289e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.2992073925602252e+00,    -8.4082518902784304e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5574204579892559e+01,     1.8609369786715739e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3540553242827229e+01,     1.5551614445623421e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6569478726582991e+00,     1.2949341856673480e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5671849864301358e+01,     8.3474269783932068e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4617239014980242e+01,     8.7423824719926779e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2744938758227882e+01,     2.2747620358265522e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3632727422867104e+01,     2.4478423391976776e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.6424013924321869e+01,    -2.4021764750847433e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0568347910802212e+00,     5.2827394444968595e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1964494226008648e+01,     1.6520312011413331e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.2390851773021261e+00,     2.4435318950141958e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.3902444813936157e+00,     6.6490336588799682e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.1386835418055455e+01,     7.0149883559692717e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8532280415956148e+01,     2.9306524019653669e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.7662675556792937e+00,     9.0345512707827929e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7711683491831668e+01,    -7.4775229073927994e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.5441948185543302e+00,     9.3204611087602570e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1538165438591037e+01,     1.4507755534713489e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.6042927213538958e+00,     1.0344294999908332e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0110366839228199e+01,     1.7308232339626851e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1422269432794856e+00,     1.6071735080360413e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3308682583552548e+01,    -6.0043470340701660e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0438378028382820e+01,     1.4443873474433305e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6770143063031089e+00,     2.0663516303508914e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    7.9775650299979288e+00,     1.5577699179193945e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.4622557785290304e+00,     8.6621938173933140e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1000813316998400e+01,     1.2023614506302445e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3035281968074646e+01,     1.4795001999444677e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4571526840849263e+00,     5.3931298342557584e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9472649495936412e+01,     7.2184720926279033e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.1325908816136019e+01,    -1.0932986512884415e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.1570195423020451e+00,     2.8132799378238465e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1104149304928136e+01,    -1.9304103509461311e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0723780720943472e+01,     1.5511115100950619e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0328108313597717e+00,     1.7285690434495795e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.3621080584963234e+00,     7.2386460292597658e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1144764428165956e+00,     8.4334361474779071e+03,    -6.6587244086354378e-05,    -4.3771888012655573e-09,    -3.4950218271186462e-11, } , 
      {    1.5997548536272406e+01,     1.5500805819695132e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5317509775360472e+01,    -1.6672801960489443e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.0893112205006521e+01,     3.2357871493974639e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5928129424385634e+01,     7.4771308035181637e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.8900681028822195e+00,    -1.3562537078568852e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2938030494434773e+00,     1.4246324218194961e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.0072639520985909e+01,     1.7856910120876571e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7851974678813143e+01,     7.2677433081369009e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2115894977019613e+01,     2.2930318205071104e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.7042657613408458e+01,     5.4685121481988608e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.9560867234974104e+00,     7.2162051037284573e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.0363386220817404e+01,    -7.1079087821358371e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2459058328709229e+00,     1.7652974254666951e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.8678442203932679e+01,     3.3507394483922450e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.0314939484897884e+01,     5.8538364239594066e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.5669206303551372e+01,     8.3020307181404933e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5726941829985753e+01,     9.6059701111197210e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6372090922759320e+01,     8.2911521814053704e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1147563379752285e+01,     6.9769044225497855e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.7890812746429148e+01,     9.4485888923556195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.0323820736121839e+01,     7.6034810391169012e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.9685221182362955e+00,     1.2432367960143023e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1960463591791640e+01,     7.2052027100193691e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.0203278095462501e+01,     8.7036634510627409e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.1318193215391787e+01,     8.7804664054121640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2867520491394853e+00,    -1.8231182376401448e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.9627886066376927e+00,     6.9012239890941537e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5088506279189730e+01,    -1.1301705875547232e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.6773606097851385e+01,     8.1758881025406117e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7606093367012093e+01,     1.6210876777127414e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1184811368009971e+01,     7.2229230208982945e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.9119946421595961e+02,     1.2646236775244503e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3522840950495098e+01,     8.3553521313097590e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5127617406372288e+01,     2.3403173684043708e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2163509981207255e+01,    -7.8712361052194692e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1781522881826907e+01,     5.5841780702557071e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5969199447138539e+01,     2.7401489332254982e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.4062667871581908e+01,     8.3667753581446123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4485530928318500e+01,     2.4482832619145847e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2506909249324096e+02,     7.2157938339372595e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3561791072624557e+01,     1.4994628798466721e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4221851154118145e+01,     1.5584702760672786e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1539162082405111e+01,    -2.4218859035545274e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.3960605021161197e+00,    -3.6986296615668834e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.0062711372134547e+00,     2.0272379984378713e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1952756095995408e+01,     1.4705368277206253e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.8131598229788608e+00,     2.3933802521938054e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.5352952635936008e+00,     6.4967377667996880e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3092940787940730e+01,     8.7927056998305106e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0912897170173032e+01,     1.7350124784308464e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6067871974665636e+01,     3.0457553311280448e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -7.7089209069320868e+00,     8.3166938141777500e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.4468792350920976e+01,     3.1259009629796230e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.5891577481469817e+01,    -1.5786094647729183e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.8487215225717462e+00,     8.9495208432016116e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2712359775754001e+00,     2.2483848541262159e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0677835798793893e+00,    -9.1485147916140650e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -5.5816972587452778e+00,     2.4202589681308591e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4790685379625863e+01,     2.3866514682753164e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.6580581918719474e+01,     1.0915544902635878e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.6998538254528333e+00,    -8.3298548862321477e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    9.6608483300034820e+01,     6.6100281540227716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4632606476072553e+01,    -4.9332085106337672e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.7047611083785750e+00,     1.5569062680810515e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6107162032722357e+01,     2.8587697351224931e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6250668714233356e+01,     2.2694460348613822e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9183873069819811e+01,     1.7179272518768579e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4864688052983958e+01,     1.3249062821838703e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.7535420205096930e+00,    -9.0502990142984308e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4268357352136555e+01,     7.2119206271892062e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.3197862556259134e+01,     7.6304940616596105e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1161533114840164e+01,     1.5403068879198900e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1910380353058464e+01,     8.3509828644724457e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.6620489712188068e+01,    -8.3063999849778065e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.0897775909263540e+01,     1.5015697695918107e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4128961351266291e+01,    -6.4635652840091643e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4574853749711512e+00,    -5.4812549171723549e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1588312224940751e+01,     2.3877479189304031e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.7947605418373563e+01,     2.0101271572159378e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1818276158890450e+01,     2.9864334032034676e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1705635893093827e+01,    -8.3137709865630131e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.6596417109684680e+01,     1.0260014845687099e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.6101970493352642e+00,     3.7590828829147540e+00,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -4.7774308726190018e-01,     7.3663225962078550e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6456554358387860e+00,    -9.4962890812026108e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.6366984913665295e+01,     8.2883510416224344e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.0745889696647161e+01,     1.5543315450069713e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5760139616315804e+01,     1.6560480048887570e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4192759243057786e+01,     1.4929403038276530e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9704872825308684e+01,     1.7035359266281102e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -8.4413786891531242e+00,     4.3209294667831273e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5934191773238211e+01,     7.3507805679194253e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8281611553113436e+01,     2.4820787449953923e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1981828965369186e+01,     9.3136021583734528e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7535009744325265e+00,     1.6522659721002574e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2441116051194099e+00,    -9.3903508867802284e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.2002083767344995e+00,     1.5016618414065297e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6992685907975503e+01,     3.3297845018091728e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.1741382755392323e+01,     2.5101909950014488e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8080338753865103e+01,     1.7279131478896066e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2326554445373924e+01,     6.5967759071356531e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1727301394043124e+01,     3.9021179035400055e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.3618656488612203e+01,    -8.0807855710501908e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.6060338004004183e+01,     8.2102131214790679e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.8360266859152599e+01,     3.1068587899559130e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    3.3898795329636795e+00,     8.3176707980027622e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5814646665342387e+01,     1.6184845445972933e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3206291781657946e+01,     1.5574393479417298e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7803866841825496e+01,     1.5378460980243799e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.3202208972963593e+01,    -2.8821286717886755e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.2307474944149980e+01,     1.7228602535831025e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0021069206714415e+01,     7.4957501466367303e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.4609643661219049e+00,     8.3397120514474900e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7628558508268259e+00,     1.6673382357774739e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3825983258325428e+01,     1.2649141170203418e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.8845348025292044e+00,     1.6168432042913319e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1370020105699847e+01,    -1.1901822862803542e+00,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.3908589177238667e+01,     1.3119721002774286e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4767421011533852e+01,     1.6641635105934613e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5147599437504851e+01,     8.9573276422471172e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3053897793242310e+01,     2.8155811376635811e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.9419311108244482e+01,     6.8003718181912809e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7020701104726623e+01,     2.4535801716357040e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.1644377411544902e+00,     1.0477473291536080e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -5.7062681284544013e+01,     1.6460333437491563e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9365295891891394e+00,     1.7024666669621405e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.7663389864014356e+01,    -1.1085950848713474e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    5.8073079127052951e+00,     7.0727876001009163e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2122054515277991e+00,     7.7886900078263079e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4983932948722533e+01,     7.2129992040338821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1095249842252329e+01,     7.0355864621793080e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.0683928823744765e+01,     8.3260379089273174e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.4210912767738208e+01,     7.2151265268837842e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9455554491674576e+01,     8.4123039684598989e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -7.0571524211225267e+01,    -1.0990865309778637e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5370890121615002e+00,     8.1355030621449828e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4002246642508123e+01,     8.4699666900830434e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8932405643093631e+01,     8.3235027161055859e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5483506116304138e+01,     7.7905553390140849e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.6987848067186661e+01,     2.5653079059511754e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.8600882808225819e+01,     6.4460698694282310e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7768005751644868e+01,     8.8951792329061373e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0308035030696015e+01,     6.2174862944581560e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0955105728587924e+01,     1.4288768952409057e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0562773172396733e+00,     8.5912005119809182e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2245816432429262e+00,     5.8062722966270421e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7319091971070876e+01,     1.1379559177741265e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.4790659206518093e+01,     2.2743409287946997e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0659726051332878e+00,     8.2925830742081180e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8254835270898564e+00,     2.5574566957323008e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.8323316317731468e+01,     2.3068336493983465e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.1284113034421335e+00,    -8.7913986221177652e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.6208116706268001e+00,    -7.7550024384729636e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.4009489292830082e+01,     7.7880893488572838e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2669224963497459e+01,     8.2152801934263371e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.9024294846752980e+01,    -1.8239399214534387e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2129701779315491e+01,     9.1081744085113751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.7030894336787995e+01,    -8.3296730865408554e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.3434742584025592e+01,     2.3876376747065009e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5963524689624387e+01,     7.5295315741396971e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0859594877334342e+01,     8.7523653167278171e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.3036934850846666e+01,     1.7162831559264632e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7928282410422574e+00,    -1.6589868755175361e+04,    -3.6628410524620729e-04,    -5.7475631522897706e-07,     2.7203865274418282e-09, } , 
      {   -8.4630861275907758e+00,    -3.3281008651763055e+04,    -6.4008030454714836e-04,    -1.0386115009041478e-06,     4.9192104967460172e-09, } , 
      {    1.9440578411114423e+01,     2.2162272247531750e+04,    -3.1285441357695702e-04,    -1.6010070065195560e-07,     7.9717428771159877e-10, } , 
      {    2.3876574270892274e+01,     3.9589992423786316e+04,    -3.8190228102041477e-04,    -9.5450117536845457e-08,     4.3545964837258507e-10, } , 
      {    9.4801080333961583e+00,     2.5371416158699376e+04,    -3.7756862848771283e-04,    -2.3324386198179776e-07,     1.0933518136382255e-09, } , 
      {    1.3322285416389025e+01,     7.8086077735698691e+03,    -1.9033545137588231e-04,    -1.4888075459277388e-07,     6.4931581124681258e-10, } , 
      {   -1.4421436181778148e+01,     2.3752001009215350e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3411838238466331e+01,     1.5531264302428312e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.5745841238500947e+01,     8.4246060022010242e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.3339141007492145e+01,     2.3970772509778173e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.4075368518045408e+01,     3.0536830799118197e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.7347277595060199e+01,     1.5554244277939606e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.3247907458551893e-01,     7.4624673920108435e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5341460692121647e+00,     1.6047108948392035e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4534443105520884e+01,     1.6906747217178887e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -6.2049370043919945e+00,    -7.0190391444641978e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.0271079909947575e+02,     7.2123318969804059e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.5558525782685836e+01,     6.8176448149581365e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3827863659879682e+01,     2.3696644079029007e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2456303264647026e+00,    -8.2038221769247798e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.4574051718361133e+01,     1.3102651831402909e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.8672411222005296e+00,    -1.1206620336612414e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    7.0216288907006842e+01,     7.9392732510670567e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0530283800988911e+01,     1.5589154956009301e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    8.9259821705551996e+00,    -1.6670500760649709e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    8.1051957989562009e+00,     6.1151820497081771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9285030304237971e+01,     8.3436477578632403e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6781569213930105e+01,     1.7522848877404846e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.1491718265132871e+00,     8.3137350915870120e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6839075496987292e+01,     1.4224005970119939e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3040107371917163e+01,     6.1480107377418915e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7875889262747728e+00,     8.2970522354917866e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6634045501770174e+01,     2.3636159099556531e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.3803039453924882e+01,     1.2874090795501088e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2883891443742730e+00,    -8.3275878973327035e+03,    -7.0853318440046442e-04,    -9.0572407091202402e-07,     4.2822622825043086e-09, } , 
      {   -4.4522288855822794e+01,     8.3053849535633217e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.1781358709405819e+01,    -4.4747247477051656e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9572403850484896e+01,     2.9906445671288955e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.2168038908282108e+00,     7.1158470881431995e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.2319787148360504e+01,     8.3277097629093969e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2521165532760092e+01,     2.6136751249844081e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.5013429251225915e+01,     1.6657376135899525e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.2449595678325480e+01,     1.6657389563000979e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.3982330333257181e+01,     2.4921272063502329e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.9407092651579969e+00,     8.6415303010898060e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1906279101093745e+01,     1.5030694180816286e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.0832662171859955e+00,    -4.4345643551379499e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5513620340673159e+01,     2.4499753299954627e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3006597062001637e+01,    -3.7737359114211813e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7580845656214919e+01,     1.3777276240741066e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7380155636101463e+01,    -8.4421026560239152e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -5.4655080209987048e+00,     1.7540121874171702e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.4483799849585871e+01,     1.0116711379299075e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0639157554932765e+00,    -2.2647014663950279e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.8082300551577390e+01,     1.7934661535844851e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1807944347154869e+01,     2.2129220916064125e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.2360880658928313e+00,     8.2550950953242500e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.0523165831147470e+00,     2.5196335085472976e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.2785390552441275e+01,     1.6797068260435313e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.3112034845179412e+00,     2.5194912394539224e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -3.2407507548995241e+01,     1.0054354572270106e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.2696917766491271e+00,    -8.2975130212106415e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    5.0642302555840839e+00,     8.3603306139584656e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4635261391451426e+01,     3.0639856642155919e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9130024780799495e+01,     9.7202933287983778e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2497895893456100e+01,     2.2287416862230824e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.7008101504272087e+01,     1.9002390756408720e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4450477708914988e+00,     2.6087903140599633e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0044351656572466e+01,     6.8390908391212179e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9384348694389422e+00,    -7.7353287286895684e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.6149797963145687e+01,     7.7153401728176959e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5767323265534337e+01,     7.2025728777031854e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3432055551375790e+01,     1.1550411443281149e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1574245398776890e+00,    -2.2041253444728142e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0115668309296833e+01,    -1.7002763283561330e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2722356283225778e+01,     9.0394823029387098e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7596028885498256e+01,     2.5477311021183996e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.6049835650365424e+01,    -1.1123273594265593e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2303025773678289e+01,     1.7068437503130564e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.4991672195021813e+01,     7.2255528532144799e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8211796406739040e+00,     2.3862809216525657e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.9871180944971446e+01,     2.3801555936889112e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.6731160399029935e+01,     2.3960463228522683e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.2486557831095876e+01,     4.2456784056107390e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4819952728065701e+01,     8.3313449405229348e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6996718589323434e+00,    -1.2285887523116389e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.6574087571384375e+00,    -1.6659525087719878e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.4079613751666695e+01,     1.6655240611180630e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.0495146757027958e+00,     4.8042092013433319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6285492589094769e+01,     1.0216244012177940e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8282536592195601e+01,     1.0469564311517821e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.4509353406392268e+01,     2.6060316718328595e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.0831569559499364e+01,    -6.4449609180719563e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.8055455092011627e+01,     1.3737461338843153e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8328746185810584e+01,     1.3821148053849975e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.9471267710848261e+00,    -8.4022877541260023e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.8005496943841441e+00,     3.3326578729832972e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2867910182510816e+00,     7.2206742650121250e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3309425607229372e+01,     1.0021837304151832e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6293890007677909e+00,     1.6724896943725144e+04,     2.4296689731918570e-04,     4.2688814249056862e-07,    -2.0249697832583097e-09, } , 
      {    1.0381349577759799e+01,     3.3348522746037954e+04,     5.7842170058363762e-04,     9.6467741453494352e-07,    -4.5715021246542584e-09, } , 
      {    1.4191219097762136e+01,     4.2028799008149625e+04,    -7.2943127205016360e-05,     2.6757836687797508e-07,    -1.2793263417118435e-09, } , 
      {    3.7631597370163785e+00,     8.6687527729930425e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.3137419120497249e+00,    -7.1504863397357872e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.9765343648720478e+01,     1.5540612051914331e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.9980908502407910e+00,     1.0217170892688068e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.4915413152679477e+01,    -1.9528379261006335e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.2501751847723682e+01,     2.3539707325905238e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9215647183941815e+01,     1.8405234766549958e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1289935543514503e+02,    -8.3304223932035529e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.6849332489720446e+01,     8.3267270770544601e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0453213259421450e+01,     1.5544896528453584e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.3908310754330611e+00,     1.2039346990450036e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.7718442650583182e+00,     1.6759554558377135e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3468445026741371e+01,     3.1163060841380040e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.7022650081979712e+01,     7.7024613824632834e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6374172926752678e+01,    -6.2227411065059528e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.9646821952661071e+00,     8.2867429542362988e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7866613032213262e+01,     1.6934356194257212e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3619089515893341e+00,     2.8920433022623548e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6154797902194368e+01,     2.3889494658264455e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0776149585967633e+02,     1.6866424356742940e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    2.0124987488061797e+02,    -1.6867440273819007e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.8658217611828746e+01,     2.2128509570597249e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.0766948015727280e+01,     3.0978836986224542e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5371437010882694e+01,     7.7901126987600610e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.8614521085791136e+00,     3.9279084970901399e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7052353795817297e+00,     1.7243030618540091e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.1963557756251937e+01,     1.1221824823400832e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3888995298481703e+01,     3.1149942930180583e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -4.4796278593967624e+00,     4.8897184748038726e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -7.0440929647243536e+01,     8.1521588203666215e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0111013322415157e+01,    -1.9575526820898276e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.9305393642390243e+00,     2.2065001217293889e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.4169381847615711e+01,     6.6428431790780578e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0207535488754292e+01,    -2.3285839134016760e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.0744670903610825e-02,     8.0907749770563860e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5843834843531283e+01,     2.1988824159612188e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.5637073198648693e+01,     1.0076896477401162e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9155391213398772e+01,     1.5507809401173970e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    6.1716932941600984e+00,     8.3706398952139534e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4149935601289858e+00,     6.6909821293687955e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.9069139315157699e+01,     8.3271661714739075e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6922045680810186e+01,     8.3302166779763429e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7562243661108976e+00,     8.5375295450889716e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.8464881496568569e+01,     5.8615331178918477e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6204734857845935e+01,     2.0643622000550749e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0727031412110335e+01,     1.4977725083605095e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2622163485900440e+01,     1.6864790077011352e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.8601258340715223e+01,     4.0564876980761743e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.5909815669827864e+01,     7.7544564642831965e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    7.9852482948353000e+01,    -1.2756074857376234e+00,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.7012827188401069e+01,     1.5129063242916405e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0098954930204684e+01,     7.9866117770835444e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1236044655814396e+01,    -2.5103435886819466e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.6900622481273757e+01,     1.2625412667720229e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.2252531045003195e+01,    -7.9587275331634828e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2414723772104422e+01,     1.7861028922330144e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.6411233262726288e+00,     7.6654389506695970e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4730535139048042e+00,     8.5389522360227238e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.4729949226904823e+01,    -5.6113260643771773e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.2063169358239385e-01,    -2.7849282153538770e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1363145567335195e+01,     6.5933818640395602e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5462137251099364e+00,    -1.8411519216274698e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0638110318344481e+01,     1.4821146700610654e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3154069732865805e+00,     1.4931367385947196e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7303284784266076e+01,     2.5554863282196122e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.2995158806203153e+02,     8.3282410571932087e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4408747516730472e+02,     8.3291417922570417e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8696476315324482e+01,     2.4721922004107728e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1448405020361893e+01,     8.4269072020407584e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.4033492582680040e+01,     7.2142685806860409e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8638070269317460e+01,     1.0082106435390760e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8546591555150105e+00,     1.4475025069620086e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2992699994145394e+01,     4.0704596602687096e+04,    -1.0990726150753163e-05,     3.4138155355327972e-07,    -1.6292163753685956e-09, } , 
      {    1.2108667534836587e+02,     1.5540718773166884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8325889659608713e+01,     9.2593498072520797e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6456856285617762e+00,     6.1992932663200299e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.9226949628229750e+00,     7.8166313153574538e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1977296057068862e+00,    -1.0697465854211118e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.1149140699402968e+00,    -5.8453675153638324e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.9312512825376387e+00,     7.4121376029019566e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6456555026739412e+00,    -2.8535837365010934e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.3586396305127533e+00,     8.4984259886579348e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1414213318286825e+01,    -2.5943543097703952e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -4.7717536323302383e+01,     1.8718275564915530e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.7151242630214867e+01,     1.5544789807201034e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9667785385636449e-01,     5.4132513269340543e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0746050163033001e+01,     1.7602478681200664e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0920463042544512e+01,     3.0457200995322375e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -9.9485659997379461e+00,     7.1874021588741998e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6973319921312271e+00,    -1.1948106525946014e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9383790003964275e+01,     2.0494292541852366e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0995591093934900e+00,     1.6029080894201770e+04,     3.0728938277560503e-04,     5.0020583674561016e-07,    -2.3182384270982792e-09, } , 
      {    1.8585846998078736e+01,     2.2756105810175915e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.1976746508567441e+01,     7.5567606073139887e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.2383850244391652e+01,     7.0167193244476975e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.6637090824561360e+00,    -7.2117958765593867e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.9662531861966144e+01,     7.0717239386759657e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2376389925806031e-01,     7.8056106886350890e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3019447351488449e+01,     3.0423449577981988e+04,    -9.8883058972098006e-05,     1.6424450014713503e-07,    -7.3687316205519493e-10, } , 
      {    2.1521019698640373e+01,     3.1261300999061190e+04,    -5.3421503166176305e-04,    -3.4586123196673190e-07,     1.6217987260476194e-09, } , 
      {    3.0015222700858244e+00,     8.2750109820470589e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4727119508968777e+01,     1.6207110291631912e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5746014365190369e+01,     1.8366477632144528e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5341321097874463e+00,     1.0147695991388032e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2737539860101668e+01,    -8.5513588017553029e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.5743643203605330e+01,     9.3862320853266501e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.6088706904234577e+01,     8.7181095983831947e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2517493922756842e+01,     3.3379200048713174e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -4.8621094971184711e+00,     1.6827117413383061e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.5360869453336711e+01,    -4.7550426286102993e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2394566007025265e+00,    -6.1028337684433814e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.6969217074157239e+00,     8.4646445611549716e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.5820406013143923e+01,     1.6482581213570171e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.3067087280783314e+00,     4.6206504319483520e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6073287824589695e+00,    -5.8190691922019960e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1112895669481276e+01,     8.3823718674031934e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2346779721561729e+01,     1.5229915413819279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7084177287270128e+00,     1.5377880549482597e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5224177016657077e+01,     1.5585414106139662e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4314710226888028e+01,     2.5536829598208798e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.0444240654647960e+00,     7.0720762546340393e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8604528513506970e+01,     1.3817091142638979e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0810943012836631e+01,     7.7357209106151167e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7702555810032290e+01,     2.2757528501109668e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.8611332665998424e+00,    -9.9656893746145797e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -4.1233337404873765e+01,     1.7817095218978659e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9707615279979485e+01,     2.8750997146883792e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4606432070060475e+01,     1.8764474308739980e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5828051422098905e+00,     7.9291209752185314e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.9667628478754757e+01,     1.0965796159109195e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4765700942941208e+01,    -9.0001202148830248e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6634488781035323e+01,     1.7500306972273786e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.1365413489308089e-01,     6.5393546145879463e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.7030155953804972e+01,     3.0996491066754319e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.0381383726824147e+00,    -2.5278240979769103e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    7.4965140530509462e+00,     1.5573932693698442e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    9.6519697025314493e+00,     2.3240508297547356e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.9131321730357431e+00,     1.1403676944132160e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1615404922221121e+01,     3.8188760084147871e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3681540575349974e+01,    -1.0785801428638833e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    9.1110058033969850e+00,     2.3011248595318692e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.6985908946300743e+00,     5.9176327934698338e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5399518306391016e+01,    -4.1106991304836265e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9003567174099549e+00,     7.5944456452572003e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5380263559421245e+01,     6.5943917787967175e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.9702240422311599e+00,    -2.5032409341759776e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.6553175390467327e+00,    -1.7079912553018503e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3183005764118249e+01,     1.8371453339046995e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.6172372357783345e+01,     1.5542548574956751e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.3242817157158928e+00,     1.2416588369529729e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5618081221917164e+00,     8.4852717933446565e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    7.4623591354266097e+00,     7.7783807265708210e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0649157896210504e+00,    -3.6465461162051724e+00,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4523836618854077e+01,     1.8151156462450388e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0915144573883993e+00,     9.9487686938057996e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9789127525327704e+01,    -1.1363488835482258e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.5222264122147529e+00,     7.5269017418235126e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0508838565490406e+01,    -2.1593673405126142e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.0735249173341312e+01,     4.7508145184884233e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.9089339637781038e+00,     7.1779802381161317e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -2.4806053692522951e+01,    -1.4119820071831782e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6853588553654744e+00,    -9.4426086385245435e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    9.9010205658925500e+01,     1.4938719578747898e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6059256254531107e+00,     5.9600775276839304e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6015610018302777e+01,     2.2462570814068975e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3506717853604373e+00,     1.4921005660738143e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.2423741580338117e+01,     1.2416635475995432e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1168888868908828e+00,    -9.4440313294582957e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.0336323601463114e+01,     1.5854273628524630e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.3953789174391758e+01,     8.3116241431854796e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8402048210030522e+00,    -1.6802497965468305e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.5988637785750981e+01,     7.8821853524022881e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1519686195616146e+00,     6.5514654386954234e+00,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.7914732318683990e-01,     6.4430709337002872e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4916730785425978e+00,     6.9661570117838974e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.9214043849667455e+01,     1.1066720301376583e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5877170359988469e+01,    -3.4431910402790709e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7566626303326053e+01,     1.6157943964030048e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6029981119806486e+01,     1.9157495278432969e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6637616667231654e+01,     3.3700083203058981e+04,    -2.2515794548278047e-04,     1.7123619216788808e-08,    -9.3714484558473377e-11, } , 
      {    1.6166945268925179e+01,     2.2198399348617218e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.9818319926184607e+01,     8.3457587062647726e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9233576990627689e+01,     1.9801550606994155e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.8096582486963108e+01,     2.6084021722664274e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9177424790596220e+01,     7.2407235720434655e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.0671255022960707e+01,     8.3250886286902096e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0748333404861620e+01,    -6.8679412344682642e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.8524065311378507e+01,     8.3322942207600445e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7616875701231951e+00,    -2.3117251523325813e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.3785823672835171e+01,     1.5542366148390423e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6979295606021740e+01,     1.6832184485330334e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4676745533584818e+01,     1.4925467331860780e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1282600561630728e+01,     3.5348875273971526e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6369182309811741e+01,     9.6078484999472676e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0197121070060064e+01,     2.0452869417932452e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7721291768144198e+01,     1.5543142431977494e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5384024168194404e+01,    -1.0031812667139051e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4820728777200372e+01,     6.9707563524906654e+03,     2.4499652131378265e-04,     3.6122497752109303e-07,    -1.7093560768560020e-09, } , 
      {    2.4108556637882590e+01,     3.9447957052230486e+04,    -1.0892793787169035e-05,     3.4133792032197986e-07,    -1.6299435958902598e-09, } , 
      {    8.2733865705896363e+00,     7.0720274939030023e+03,     1.5250861536851649e-04,     2.5032384796728670e-07,    -1.1877935187183631e-09, } , 
      {    5.6564374544898612e+00,     8.6622403271428266e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2545103598721930e+01,     2.5189064784406375e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.2525718686628720e+00,     5.3267867187864817e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2822239159515356e+00,     4.9073020138982088e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8993591628951796e+00,     1.2666061851012200e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8391823130384257e+01,     1.5697826789322144e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7758105947146113e+01,     8.2396491759573564e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.6641650433502662e+00,     5.9666839581220358e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7430834876075085e+00,     9.3880058682393815e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8073609056650177e+00,    -8.3985812027450975e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.8900272811282271e+01,     8.3245726232715479e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9229570744045965e+00,     2.3858826476586706e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -3.9032649325713393e+01,     1.5343679780694398e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1443168024438314e+01,     7.8887711163297172e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.7773293527943906e+01,     1.6657741878959056e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.6620293229580938e+01,     1.6657023819941449e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.8751353136938050e+01,     3.2187019228434750e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.6390532906865360e+01,     1.5542960005411169e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.7529504775555928e+01,     2.8980718320917920e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.8559307750054579e+01,     2.4093614419062185e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8178520251794485e+01,     2.4340846008321052e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.5017272087377584e+00,     7.8397406181881915e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.8132788917239377e+00,     1.1646326020757049e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.8209915703484853e+01,     1.4681461289625595e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0795385919746323e+01,     7.7246567132890650e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.8617038120435279e+00,     4.2276001699193148e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2533520745649064e+01,     1.4808508510716034e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4050583781674328e+01,     1.3834763034882106e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.0638840657996305e+00,     2.2754739612859088e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3531368045033076e-01,    -9.4124449719456607e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7217069675677763e-01,    -3.6784267097469206e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.9446796570952500e+00,     8.5666078723938663e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4652504171653860e+00,     8.9312618602488365e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.0371472925829917e+00,    -2.4565781634772981e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.0421735347757899e+00,     8.0158525483604471e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3485034417406887e+01,     1.7265479250254612e+02,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.1072525523323833e+00,     2.5037889385665669e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.2202906833441059e+01,     2.4380308594260892e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.3082190072057756e+01,     2.2758894698426495e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.7838813551785044e+00,     3.1005878776572092e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.2688249405606399e+01,    -8.3290102282761418e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1559481212118634e+01,     8.6126799057427634e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1122935911483215e+02,     7.2137172630514833e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3045370991013812e+02,     7.2144084678661811e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.4833417728129191e+01,     7.8977273806907178e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1945050445058090e+01,     4.3526654995092986e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.6715919968400899e+00,     6.2516940369415615e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0210918227185431e+01,     6.9875927188193300e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.3254971966571196e+01,    -1.1315492400750745e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    2.8440845689465441e+01,     1.9395411726101709e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2459582880455599e+01,     8.3263035917499747e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1797252193037380e+01,     8.3282269627883397e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.4607027159996818e+01,     8.3310792577002758e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8348445958561257e+01,     2.2542200617864801e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.2477529091814581e+01,    -8.4177336734928977e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.0908144564217668e+01,     8.2330902203576297e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7977148210397566e+01,     8.3283726211741105e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8573416577574609e+00,    -8.4995436902650126e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.1324831800579407e+00,     6.6103384442142224e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7330013071339783e+00,     7.3875639148871433e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.8179719188439574e+00,     5.5471125564466661e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.9930467167491299e+00,     1.7389301705904134e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.7086028909491347e+01,    -8.3291558866619162e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.4566605602296002e+01,     9.0136659886840553e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0958663429171018e+00,    -1.9187041826395939e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.1538155428574427e+01,     1.0801451991672366e+01,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.1606298902959908e+00,     2.9496950963714774e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.9048791686225741e+01,     9.4087739904577065e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3179657364123027e+01,     8.3347517074477764e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -2.9835464741998774e+01,    -6.1039694490597185e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1746135842359147e+01,     2.3923260826399375e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -1.9816303608918524e+01,     1.1767617092392284e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9399614405504856e+01,     8.3288409105091141e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.4688505261001048e+01,    -8.3285419389411345e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -7.9867236502824603e+00,     9.0193558167996380e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3324637744516616e+01,     2.3165585868851416e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1107528351421708e+01,     2.2769935066842248e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1681692181084568e+01,     2.3558625790116646e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    9.6680944992838889e+00,     1.0577076278851342e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2556003591046263e+00,     2.5353358094346531e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.1733336976103114e+01,     1.7978432369354006e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.0855813571662649e+00,     1.4580385461666689e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0808154563540016e+00,    -1.1539857209015310e+00,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.7741359676358709e+01,     1.5522135727672983e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8833420453540249e+01,    -1.6595249699477317e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3386276803626558e+01,     8.2801315546830065e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8757558749289281e+01,     1.1772580147434252e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7231401529093862e+01,     8.2500280233769809e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0602216967523394e+02,     1.6655651880971825e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1907683357785082e+01,     3.2122584878622085e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.3201911382852291e+01,     1.5518171126383024e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2894436916524756e+00,     7.2265143849839451e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.8857268331324377e+00,     1.6869562147597157e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    8.5055373247588797e+00,     7.1573958865454470e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5974990722955244e+01,     1.6406348490582059e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8021468293383220e+00,    -1.1314893059604765e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4958717202837024e+01,    -7.6877646013572039e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.7452297581165318e+01,     1.9550516248125958e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.3155807613868973e+01,     1.6656678217534103e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.2002776781622288e+01,     1.6658087481366401e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4335857381359324e+01,     1.4907887749538688e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.2092480030650677e+01,     3.2166355712131244e+04,     2.6936461440465505e-04,     6.0169256335142287e-07,    -2.8559889140481653e-09, } , 
      {    2.0159346314606676e+01,     2.3707152404968925e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.7667560642897158e+00,     1.6864302482964791e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.5133212189192555e+00,    -8.1528990060318501e+03,    -5.5415153985634931e-04,    -7.2416619547331383e-07,     3.4255965079837595e-09, } , 
      {    1.8563889613397887e+01,     1.4951682963413365e+04,    -2.2076563049466428e-04,    -1.1752368443776262e-07,     5.4939086343332605e-10, } , 
      {    1.8018998222686648e+01,     1.0653035815380086e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.2626757773621993e+00,     1.2132440086685065e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5853360676316548e+00,     2.3305494936892512e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.1738217853550687e+01,     8.2734306034603196e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3962424706730509e+01,     1.5449454285656197e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.3278062761807092e+00,     6.3478388327445500e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0477274449910343e+00,     2.2534648451489691e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.1524326572364494e+00,     1.5682439701169018e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5727304385221007e+01,     5.4087699141180274e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3039449402519653e+01,     8.8728877931588177e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5562050692749037e+00,    -1.7947779447044304e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.1243882069511947e+01,     1.0094169474168018e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0771697563555875e+00,     1.2613092753801007e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2703853398119280e+01,     3.1418665577041647e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4856155350673109e+01,     1.1943432412974136e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7866017133350397e+01,     2.1874078130921342e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5510104030201797e+01,     8.5780557924537607e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.6184396001902570e+01,     7.9176215116767635e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8809831992285160e+00,     7.6208233741262784e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9830969958799002e+01,     7.1653609997937565e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2249703709748445e+00,    -9.9345567310000406e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.2575250292921844e+01,     2.5994284917714453e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2860526972356430e+01,     7.0574824968393023e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.5069042990315268e+01,     6.6038042237686641e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2034099380381692e+01,    -4.0613888089244210e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3907127134503199e+00,    -1.9311557021647063e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.9021142766313961e+00,     4.4966975051108839e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0646756819299812e+00,     8.3249249392296242e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2780417944583256e+01,     2.2774090152409648e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.8876362219525411e+01,     9.8622799396231730e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6452931989807745e+01,    -8.3839522459899326e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.6213475065013210e+00,     2.5213647948207305e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.1562463154160133e+01,     2.4072921457597899e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9887102748955598e+01,     6.2545540875848346e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.5240695002726905e+01,     2.2131139402913432e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.6042476790323352e-01,     9.5979357886114087e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0028906233511487e+01,     1.5489073847505892e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3786612327624843e+01,     9.1945455944732530e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1380353724458576e+00,     5.1546406277438678e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1177406993827333e+01,     2.5601263948021468e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9575669171201536e+01,     2.2792865572045197e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.2484288552491503e+01,     1.5563372852694934e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6739142730178518e+00,    -3.6463503628272872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3686872681910511e+00,     2.2039492056983647e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.2807548917384594e+01,     1.6732633910622641e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.6796003868858207e+01,     1.5440582581257078e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.6993161142666846e+00,     1.6709197960940543e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.6582122940842591e+01,     1.6235496225042172e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.0262135951246094e+00,     1.5942334215476674e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.7091281296229099e+01,     3.8263523029424345e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.1098729928112370e+00,     1.6657159192394218e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.9512847964637725e+01,     1.6408978322898241e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4475928184603042e+01,     3.2086021073565698e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8257108260073540e+01,     1.4617026939812928e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0590692190654250e+01,     1.6517663227524907e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.0959469822872258e+01,     3.2722506530257375e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -5.3639364778809124e+00,    -8.3341211297581212e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    3.6269433531775306e+00,    -1.3679488586061748e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0217189322922117e+01,     2.3492502497592177e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -1.2253112814300025e+00,     2.3853396771553711e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.2642009418939866e-01,    -1.0798150931398139e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7236175017825484e+01,     2.0115055142902811e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.5175318439071326e-01,     8.0656234866657942e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.4523212995927928e+00,     1.6657606506506287e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3917761243733736e+01,     4.6386993753037259e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4286709399312919e+01,     3.9450248421495446e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -1.6969804351182681e+01,     1.9628036486252022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3982135661857615e+01,     2.2778147063620643e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.9738336352852404e+00,     2.2088694668699336e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.6197331505416948e+00,     5.0202303572410665e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.2407469485571596e+01,    -1.9521332941844835e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.3225256288436768e+01,     3.1085515293918645e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.3164302638878366e+01,     7.7692996023082778e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8171688592046005e+00,     7.7636622181462617e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5242323785231344e+01,     8.3284061970387847e+03,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.6039564594933044e+01,     3.4570066339085797e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1003340883095653e+01,     1.5531937578824238e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.1110782312128620e+01,     5.9414481870432610e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9953432929739201e+01,    -8.3289766524114675e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0423777323027754e+02,     3.1085501866817191e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.0817609450501784e+01,     2.0887313511545352e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.2343016959564920e+01,     9.7428352339294343e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8766190858330631e+01,     1.3746849048660932e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.5325668609396452e+01,     5.4485848160159403e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4712854837817952e+01,     6.5895685722264243e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7090314159365896e+00,    -7.8117175281946711e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1454115599619104e+01,     2.3888366395717869e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.6152572831926854e+01,    -5.2183050946841904e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.0919306758041216e+00,     1.5989260362506797e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.6652227645018467e+00,     7.7114044664019466e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2986080590798501e+01,     7.7734546878756801e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5087400835888083e+00,     1.7455424998428603e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.7998207391613931e+01,     8.6218534342906205e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    5.0892780668976599e+01,     8.3614825623844808e+00,    -2.7586509320370791e-04,    -3.9500194668399445e-07,     1.8691506661497048e-09, } , 
      {    2.5878552772468353e+01,     6.3038512782924363e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6581351671457831e+00,    -1.6641383341125766e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.0147892566561350e+00,     8.9420426766325545e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7285958583197733e+01,     2.2381845129305177e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2638468558194340e+00,    -6.6208901254850753e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5459254319040079e+01,     9.7894416149819881e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.8473625968137215e+00,     1.5294848436509023e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.2837072110163220e+01,     2.2247954276290984e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4125906529150987e+01,     2.7488455880111769e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1600527454123458e+01,     2.5981665679392077e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -9.2354612279908679e+00,     1.1253305051064061e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.1936272926589169e+00,     2.5192993907689917e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.2502706865531618e+01,    -1.7403063119956523e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.1705686529114416e+00,     2.5198253572322283e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -4.5685227518949468e+01,     1.8111341560552471e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3383507188501738e+01,     1.1614845793093820e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6578020392090096e+01,     1.6726074955022916e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    9.1973833750069947e+00,    -7.8892697892067954e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.8350282782706442e+00,     2.8333489456692838e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1133728371664109e+01,     3.2681206843813670e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.5056818438968183e+00,    -4.8275853248309377e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6977092655023167e+01,     2.4028026083528614e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.6389118368794025e+00,    -3.4892445056420662e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7732543982241239e+01,     1.0609264981870925e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9412736545862099e+00,     7.7500472371141277e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3718779974705399e+01,     1.4922073288764686e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.0880645120060471e+01,     8.2404251225458738e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2146676637487971e+01,    -8.3449007600670302e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.8011529219482663e+00,     6.2522528877451005e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8002992468514044e+01,     1.1308565872328865e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4988668183155539e+01,     2.3736458980926916e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0054114789756408e+01,     6.4798170859909069e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5225552591428411e+01,     1.3267111765194075e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5547796004091733e+01,     2.3558246183181909e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6346123602475785e+01,     9.2882002025991242e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.7885019774175515e+00,     7.8231827807961490e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.4098334307435062e+01,     1.4231524330124194e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6987395189080125e+01,     1.6522396115468087e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.5753159526128417e+00,     1.0683014473273726e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6810088046752334e+01,     2.4533504717295014e+04,     5.2631445943303407e-05,     2.7800738789779477e-07,    -1.3756539720114973e-09, } , 
      {    1.9646055234044780e+01,     4.0495071517221892e+04,     4.2157943268241913e-04,     8.5214731101260920e-07,    -4.0416007712015358e-09, } , 
      {    7.9860458566386612e+00,    -1.8157353293240543e+04,     1.5906775662298365e-04,     4.6779672090259123e-08,    -2.1574208809374349e-10, } , 
      {    1.4038315727191687e+01,     5.9715598985876295e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.6914443864022974e+00,     1.0454501267131973e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2618185613982718e+00,    -3.3056997766467155e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8079326328299878e+01,     2.9299529855370565e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5290835936226870e+00,     8.4059769661494865e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8702031877622161e+01,     9.5950877787399804e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.5931875221362226e+01,     2.9990526415258482e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.8168614364643119e+01,     3.1636263904401338e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.0032811728368743e+02,     1.5541023321705530e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.7761279865492465e+00,     1.4901827466816037e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.4860030205574830e+00,     1.8451078532387173e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0219907588122311e+01,     8.8407515340927985e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7562926628062769e+02,     8.3146769549157361e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.7091815713612391e+02,    -8.3427058945345143e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.4341813941828697e+01,    -1.6851199831133381e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5767884779152055e+01,     1.6894929434152837e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0739179848253041e+01,     1.5567337453984892e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9615412949038280e+01,     7.1969955839191853e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9248356948841383e+01,     8.3865668189380667e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.0044401607770901e+01,     1.5596434732862026e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.6272022878615537e-01,    -9.4367610283916911e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.5445619782522876e+01,     3.0785787615202691e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.6866258123203409e+01,     1.3660132734887064e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0737498062179345e+00,     5.0175082811218499e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1311478390119696e+00,    -1.6375695568272355e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    4.7171110801237237e+00,     1.1110175534561009e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4965518570459407e+01,     2.4133964277951000e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.5061782335288573e+01,     1.5526544954842055e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.9541986414569237e+01,     3.0831077140692018e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.8096771729027942e+01,     8.2162465958740158e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5194040762258059e+00,    -8.6753117285927692e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.2301944453427133e+01,     2.2735487247664940e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.0961157965019277e+00,     1.5443427495314869e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.2966184609655290e+01,    -6.9864054845662395e+01,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    7.7629486450196268e+00,     1.5019673554093923e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4739933922944095e+00,    -2.7385275495285332e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.4187131709438152e+01,     6.7605569162933680e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8253559164235220e+00,    -8.3353028242784185e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.1131322485186665e+01,     7.6799299603060172e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8941650677183738e+01,     2.4564187649767300e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.3997992871426902e+01,     2.6387305887407441e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.4044252912421884e+00,     1.7285684804698736e+04,     3.0196161978978796e-04,     5.0143862097393552e-07,    -2.4271178836018586e-09, } , 
      {    2.8873862433859465e+01,     1.9943537217818946e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.0254061416344129e+01,     8.3263919598827797e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.8980094030633445e+00,     1.2146666996022591e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.5799600380850158e+00,     2.3163508326067717e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.6877109351590725e+01,     4.4099465985608731e+01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.1802795968813211e+01,     3.2200848485101087e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.8975569646527568e+01,     6.9998768426266906e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.3950680601662953e+01,     7.7159258679680115e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7834714486698520e+00,     4.9757069024688008e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1770515341966359e+00,    -8.5447998461555762e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -3.8144266441784566e+01,     1.6792369583432417e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.1356993751924378e+01,     7.0724285705921147e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0606158794161004e+01,     1.7572293488461568e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.4209552360832545e+00,     7.7149269457173588e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.7104973234585991e+01,     1.4909515672982496e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6398569054637893e+00,     7.4896898639140800e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0788333311649532e+02,     8.3299610141837165e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0993258886166267e+02,     3.6574305952797204e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1024475963513567e+01,     1.6657531266507467e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5394420863269715e+00,     1.6657234432393037e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7835389710528244e+01,     3.2199425794167335e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.2063152645688966e+01,    -2.2036617719044261e+02,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.5908975427906004e+01,     2.1184733276870844e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.6191132706639948e+01,     5.4883997179138523e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4034158075717487e+01,     3.1887317214841773e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    4.8647758078771723e+01,     7.2032461540991117e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {   -9.6369691308856247e+01,     5.6421980953814818e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9532094582481463e+00,     6.8467790452876789e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -3.2548063706246417e+01,     1.5541690628759010e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3730557861611743e+01,     1.8071732373881772e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.0314633362327918e+01,     7.3230953541341860e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.0579734873071596e+00,     7.9946301812533984e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.4943621411045896e+01,     7.2104600694239161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4209234644324590e+01,     7.9820711208574830e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.4109725460686530e+01,     7.2311301469984792e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6484414760294207e+01,     1.5543817951608908e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8814186304322806e+00,     6.4821440090049500e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    6.1102773285713852e+01,     2.4500105615912700e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.6379159749490277e+00,     8.3368584677109884e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9268068304452259e+00,    -8.3205243817392638e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.9427064433417065e+01,     2.3990890420602820e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.1490909208469738e+01,     1.9402796853229720e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2432415442293244e+01,     1.5530302770658844e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.7312121555661033e+00,    -9.9615025602023525e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0726321071370045e+01,    -7.7173045204883647e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.6967895246446972e+01,    -7.2390878941303890e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9463245371050448e+01,     1.3800529491338999e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.6955862388460087e+01,     3.1571477238630596e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7789390869680886e+01,     1.5701661581817532e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8607851433793915e+01,     6.2870083130798514e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.8041811750190405e+01,     2.2756719104068950e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1806003213542417e+02,     2.2756915207216633e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.4168381276253356e+01,     7.2176656614937510e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3559654238180334e+01,     9.5011953782043602e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5035180286389361e+01,     2.2636160452795852e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1934561072318310e+01,     8.3297960164544438e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.7987486409924571e+00,     8.3275868329958084e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9073704282863648e+00,    -9.6010543383318509e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5603269451928684e+01,     2.4656333668574156e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    3.4115785499338529e+02,    -5.2836244883364998e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3189815996947075e+01,     5.9057022593133115e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4146251700794849e+01,    -4.2672315461151840e+00,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2447321800987336e+01,     2.4499041954487751e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9704871627664069e+01,     1.0862585281210808e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.0483381179800357e-01,    -9.5415357613070519e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.9647332090212281e+01,     1.8055984630366132e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0904291590562716e+01,     7.3445747478960275e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.1749904547338712e+00,     3.7760185865379663e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1038380432151127e+01,     6.0427673272791535e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4757449820567459e+01,     4.9513911650554292e+01,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.5205277989739656e+01,     8.3881947423818783e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1441217105485428e+01,     2.3836500825899097e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0812947085963126e+01,     2.4078365348423620e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.0317223397938315e+02,    -8.3274218352665357e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.3744732065287115e+00,    -1.5852017899714049e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2578261756914920e+01,     3.1821193922317303e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -1.4325412057910659e+01,     6.5342204455608197e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0262496155014791e+00,     7.0628874667181281e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6982071451656289e+01,     8.7059646509024751e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7067742653299298e+01,     1.5553571001543680e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.8700899620181744e+01,     1.0019088723747700e+00,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.9872976596257889e+01,     1.9788432695794698e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1411658145454129e+02,     2.3870937756371048e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -1.8547001237022769e+02,     2.3871953673447122e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.7151463117011328e+01,     5.3612831524162061e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0150832153491406e+01,     1.3805680969285538e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.5633422834807185e+00,     8.1305833176832439e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.6291954686283106e+01,     3.1165138384163743e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3909616049663549e+01,     8.7681130602434569e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.1776842418396649e-01,     1.2655675693335822e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8297472743721041e+01,     1.4410101522716459e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.4922823753362565e+01,     2.4083625013055993e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.4675852927581364e+01,     2.9972096516522470e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0657636882771921e+01,     1.0860284081371074e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.2286020068200205e+00,     6.6452738574380883e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.0917782256676812e+01,     7.0443283015260231e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -7.5752289867719014e+01,     9.0747127469859788e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7538307532172681e+01,     1.4932495648493790e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8063298922124634e+01,     2.2286243924937680e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7147924290398691e+01,     9.9989910808127097e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.3374116691525622e-01,     1.4082004099927097e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2669316720591056e+01,     1.7267268089699319e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6018171947624367e+01,     1.5542902707241172e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1669313611454442e+00,    -5.9107132677059235e+02,    -1.5457750927128333e-04,    -1.8147060897611042e-07,     8.5812021556387864e-10, } , 
      {    2.1638800810319193e+01,     1.4252333312224389e+04,    -3.5162954514457308e-05,     1.0082670126035020e-07,    -4.8403797921976076e-10, } , 
      {    3.9632453454551687e+01,     1.5074482259318582e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3070961217636112e+00,    -1.5055377096570101e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8386706372274513e+00,    -1.6298143307260230e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.0811269005058694e+01,     1.5543850171151933e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0914545349296866e+01,     2.4186914423589948e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.0367843089179672e+01,     6.1963371067552198e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9203704136750268e+00,     1.5542605873126746e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8106693216681563e+01,     8.3309908895674725e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3394099182019337e+01,     1.4371458752004279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6109558784495675e+01,     1.4427797098441217e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.5984083860912648e+00,     7.7927070530698311e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8371285494830289e+01,     3.1651459358384491e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.3987347763688263e+00,     2.9968578821261886e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.4746671749977168e+01,     1.5541658409215985e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0406767457016340e+01,     1.6886110509203081e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.8610751676445574e+00,     1.0557594079686851e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9731823548099801e+00,     5.6666978913385677e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.8953429734938840e+00,     1.6641963738411061e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.1550849029079373e-01,     6.2567032999802763e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6349089311932725e-01,    -8.3324579102206280e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.3031971655525702e+01,     4.2118969781077740e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1969483934564568e+01,     1.6266621849107292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.8148409817772553e+00,     8.5485589290384924e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8851908137060316e+00,    -1.2374275003828593e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.7091338627650661e+00,    -5.0765088219860206e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0200756810432603e+02,     7.7004818912534229e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5193878433857302e+00,     1.4428454363394114e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.1274886187722242e+00,     7.2030422387364688e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0492615735059070e+01,     1.4939029868939348e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5971311341396408e+00,     1.4948233762438442e+04,    -9.4451474075811953e-05,     2.6407122470882869e-08,    -7.9708217311218847e-11, } , 
      {    2.6290169432994777e+00,    -9.4095385564884546e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.5381518423163680e+01,     1.4461907158420629e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    9.0419354150267850e+00,     2.4409581800620123e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -3.0412523102720019e+00,     4.6272093875480796e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0149461881957713e+01,     1.3958725437505696e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.9245705102043217e+01,     2.5685872703685145e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0200682906225573e+01,     1.5418801363346493e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8425780755884350e+01,     6.4160207164804797e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1270390692120403e+01,     1.7365872527824104e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.8180628300689435e+01,     1.3211325462724491e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.7972517819476963e+01,     2.4189215623429685e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4391248177288711e+01,     1.6902980731683383e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    3.2524515807015362e+01,     1.5796089848891888e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2604222670681224e+00,     6.9646984977301991e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.4894319443184846e+01,     9.7717592624039230e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.3538995873658678e+01,     1.0014539670372194e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.2869987331154675e+01,     8.9327261361611872e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.3577509369686398e+00,     1.7417687639314394e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    8.0526860289447484e+00,     8.8517721608151624e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.9011373667453753e+01,    -6.2794855553674165e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3050088155732986e+00,     1.2592450024072705e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5677605028374479e+01,     7.7080104233058537e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.1177526790937158e+01,    -3.6283816188423179e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8917634441574883e+01,     2.4046247350789163e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.7150609497209857e+01,     8.3285524486954364e+03,     2.7602168802270629e-04,     3.9810475424309542e-07,    -1.8846647039452100e-09, } , 
      {   -2.1861718641713658e+01,    -8.3288304007548177e+03,    -2.8603813259990178e-05,    -1.0271747461667739e-07,     4.8801345140485886e-10, } , 
      {    1.6935075691148107e+01,     7.2016113459337184e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -7.3246794925246217e+00,     4.1036765354178970e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.5590332109511593e+01,     1.4151152386110707e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5679530937406378e+01,     5.5240958183581870e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    5.5130277764873883e+00,    -9.9940753080248396e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.7637341802860789e+00,     1.4682557170593569e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.1796995774433395e+01,    -8.9507442806132894e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.1030266942066767e+01,    -8.5156989854247913e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.0381421430497193e+00,     2.4673254349382940e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.3036690469619497e+01,     3.0478530903300227e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6016857116047833e+00,     1.2566039104969645e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    2.2726858695342176e+00,     8.9729985180951553e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.5282171752083684e-01,     8.3232617196921328e+03,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -6.8213811912091103e+01,     1.6062184418512443e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8119257163985878e+01,     7.2116750324836821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4363376218292363e+01,    -1.3262555678807157e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.7369533546305636e+01,     2.6172799666246494e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.1357609960081426e+01,     5.9831950133200735e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0917061462142577e+01,     8.3364063516708429e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1180608214070407e+01,     1.5366986251856202e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.3451410523182320e+01,     1.1867253920711139e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3613544286753813e+01,     1.4463070619927650e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.8262459337711476e+00,     5.3422580215216585e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.8155423799494361e+00,     2.1906340866444149e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4065031966181550e+01,     1.4901328793938959e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.9956117283747432e+01,     2.3243131416312815e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.8134877176787754e+00,     8.2173577362100368e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1066016864526043e+01,     7.2250834921811966e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9639109502209617e+01,     3.3195728544850906e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    4.6416615797134185e+01,     8.3436118628872391e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.3846448172872034e+01,     1.2809656445688421e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1787371086618293e+01,    -1.3221004823133144e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.8895785433934449e+01,     7.2164506984339823e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3419231777281748e+01,     8.8456653212555811e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.5197776506501427e+01,     1.7896725022774481e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8752741025979027e+00,     8.7397460784054401e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.3702249174573410e+01,     2.9973181220941351e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.2613999925890820e+00,     1.5948893171076401e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1579144382438125e+01,     1.6707046246884798e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2671763196480045e+01,    -1.3028481171191631e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0060788151503779e+01,     1.4012316132320630e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.3956193279072142e+00,     7.6911950517636715e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.8979894952770465e+01,     1.8445402804024201e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1118308459858023e+01,     1.4208586859080748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.2057883215205241e+00,     2.5248592837217293e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.1084297115956691e+01,    -1.1137671797569624e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.2027208345088933e+00,     8.3926811486586812e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7349340725766442e+00,     3.8264976869781339e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0007324303840711e+01,     2.3884064953231464e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {   -2.3166634737797562e+01,     7.2151587464268068e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.3356832305090535e+01,     7.0565342263456096e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6480202167430946e+01,     1.1175028323487672e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1809189222792931e+00,     7.2942619247342927e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9322396182561929e+00,     7.2156766919418402e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.1289816048104564e+00,     2.5142654642794907e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.1306395328312593e+01,     6.6475750572778234e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.4924049170981153e+01,     9.6812211722921111e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.6208083526610606e+00,    -5.5615081812612482e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7948478496047123e+01,     1.7306353950799305e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.5134750313088105e+01,     3.2199808507157763e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1923801916797780e+01,     3.1674001263515547e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.6757344321459485e+01,    -1.2894301951463722e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4075006735187399e+01,     1.6799369460275044e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8951694823806392e+01,     2.9373739290610909e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3156897895717788e+02,     4.4031195565530368e-01,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -1.6522287442968093e+01,     4.0875345384057800e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.7830141277279402e+01,     1.6882664799428563e+04,    -3.7747069612412869e-04,    -2.3328749521309762e-07,     1.0926245931165610e-09, } , 
      {    3.1214377105116018e+00,     2.4041180278841894e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -5.0457585117409849e+01,    -4.0167217855986905e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5383564659580369e+00,     6.9761464177900925e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.4939527914790007e+00,     3.2200465772110667e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.1611547933922139e+01,     2.4691269081798022e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    4.5436264869089174e+00,    -2.9480519237735416e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    9.0667764700456903e+00,     6.9515443024169199e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    5.5713228146748923e+00,     1.8017609290949677e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.9280278655233190e+01,     2.4579935393282936e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2947772284561456e+01,     1.6047099472605914e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -9.0411932469704084e+00,     2.4906444470379553e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.4105260451268169e+01,     3.2251952251124501e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.6422034471631786e+01,    -7.6943503652846357e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.2079833360396439e+01,     1.0467263111678087e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.2898721246343072e+01,     2.9868708312174742e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.9925605337961034e+00,     1.1933367916244072e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0353659581386719e+01,     1.7585411399661021e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0821512459558406e+01,     9.9020948415210842e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1675809407772897e+01,     8.3860697491053888e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3799942716698185e+01,     7.5798518874624633e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.3720785657174430e+01,     3.1178808584895680e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.2719421355539126e+01,     1.4525028531480346e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1818940637005014e+01,    -1.6003555418016731e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2002207306782184e+01,     8.3401566773266040e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1628086071301741e+01,     2.4383079114862339e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4066370464112175e+01,    -3.1285051932597980e+00,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    4.1016644744054489e+01,     1.6730677354702380e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1377386775349386e+01,     2.5218165645137156e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.9312608786483452e+00,     8.2647017007915711e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6281087670770326e+01,     1.2336453382667125e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2836671140478053e+00,     8.4169975928718522e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1935262467935170e+01,     1.6257812399943658e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.9682134773237728e+01,     1.5010047909505913e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5501778121945549e+00,     6.6806728481133086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8295134085788426e+01,     2.3402397737455191e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1534080289369196e+01,     2.2821251505455457e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    3.0295842200658299e+01,     2.0336558187511935e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5420705175634755e+01,     2.6780326857100430e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7803835674029010e+01,     3.2277689400646337e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.9706635790146514e+00,     2.3537384471437355e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8727884283667347e+01,    -1.6860898840886028e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    9.0352089594563854e+00,     7.0018835673119274e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    8.5643651939069230e+00,     1.6117381432551434e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7863117593960723e+01,     2.2741275127354362e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.2785751490725517e+00,     7.1960291814715065e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.2611048001596259e+01,     2.5065704077971204e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.2390503620130801e+01,     7.2129669844908585e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3553898993448426e+01,     1.6939070130628152e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.3584613947974232e+01,    -7.4782151355264823e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.2002514343509461e+00,     9.4271106486495155e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7501750613579592e+00,     1.3419915087378589e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7609560308026374e+01,     7.5246005419837766e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1344278780071669e+01,    -8.5820269834330556e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.6964906269703931e+01,     7.4131373749483937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3099556499672133e+01,     7.0071432319442956e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.7574995334545989e+01,    -4.8252841249912035e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.2559152084078619e+01,     3.6611416540743558e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.2250199400644959e+01,     2.1280334481238340e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0985876817612752e+00,    -3.1779491850468630e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0647277693833221e+01,     1.1459741271069572e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3450175733247107e+00,     1.7678711404188783e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.2112135023733913e+01,     8.2898429252748920e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.3901905565367414e+01,    -8.1552245862371019e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.3501957161731589e+01,     1.0074242141985098e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5897769421266617e+00,     1.0440274357794444e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7779363263414837e+01,     2.9359493116864862e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.2507662631939871e+00,    -2.1725293136523032e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.8145082425454959e+01,     1.6562211017365997e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.9799681411858812e+00,    -1.7827510462097489e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0903045393977891e+01,     6.7549718533023515e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4151911025424347e+01,     1.4190579146215083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5994841939730438e+01,     2.1813040861916386e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.5652311629323989e+01,    -4.1305824675200711e+00,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7333625819941986e+01,     3.1538078137248282e+04,     2.7193056353397944e-04,     6.0111980446856010e-07,    -2.8008219652747115e-09, } , 
      {    2.1796129667240741e+01,     3.0490963672256876e+04,    -1.6054166293560877e-04,     9.0310413777930817e-08,    -3.8916478996343569e-10, } , 
      {    2.0546303820939208e+01,     4.9033312407777739e+04,     1.4122409212701095e-04,     5.9183630121446606e-07,    -2.8148282325219654e-09, } , 
      {    5.4813823517501605e+00,     1.6204873312895046e+04,    -9.4549406439396088e-05,     2.6450755702182727e-08,    -7.8980996789554546e-11, } , 
      {    1.0310076091228174e+01,     5.9237019078244230e+03,    -1.8234380689715677e-04,    -1.5072993093526187e-07,     8.1263499600218198e-10, } , 
      {    1.1600697872488785e-01,    -7.1017685777915176e+01,     1.8550474361662286e-04,     2.1839401892941267e-07,    -1.0327016221314225e-09, } , 
      {    8.5803398874139170e+00,     7.9291114994324107e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.1531154388631151e+01,     1.5275584790397261e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6136155062979501e+01,     2.4493194344354899e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2169000877645695e+01,    -6.4749202647752772e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.3344410296140131e+01,     6.0658034231042279e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2623815286735315e+01,     8.6261111900506166e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9186476901593874e-01,     3.1085837212844370e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    5.0820647764193410e+01,     3.1085179947891465e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.0192576859406497e+01,     1.2165601117127240e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3003516517899349e+01,     1.2702110267414610e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.9186471367113999e+01,     3.1316750884112844e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1482591088731926e+01,     1.6031705096720894e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.4168309875116627e+00,     1.4426048188133966e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0492239372501212e+01,     1.6851513204241783e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -1.7003860351559574e+01,    -1.6882351426320165e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.1793979147041284e+01,     1.5089248341018496e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6404695830273866e+01,     6.6077965338580179e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5392462277701803e+01,     1.2706617266268040e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -6.0091762608015628e+01,     1.0371142818974979e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3415010587377232e+00,     6.9158596351222229e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0723919019946763e+01,     1.4430203273701369e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.6003471885264888e+00,     1.5516093583599324e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9632289667818895e+01,     2.2441348446961929e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.8430182329268053e+01,     9.0366288135664945e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.3220705703649642e-01,     3.2120507335838389e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.1965445099665217e+00,     1.4976803512167384e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9414636539382744e+01,     3.9953490076524991e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.0656258437073191e+01,     2.0194571576687136e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.8468261178486411e+00,     2.3902624118423566e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.3482048626883261e+00,     1.1609862569241914e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1975629037174659e+01,     1.9503719748212799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5348097952017330e+01,     5.0317203449967128e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0879198552685146e+01,     6.2546267098775184e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2298973653146437e+00,     3.2788629822781848e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0441375626544897e+01,    -5.6882183437559206e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7290064848646047e+01,     6.5545768768988055e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2511349881413869e+00,    -8.5202166823546431e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -5.9261325031467209e+01,    -1.1488252758260558e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.7848441044040278e+01,     2.2772359183931221e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.1359563739954641e+00,     7.7985996169952159e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.4430016791723412e+01,     6.5461597567210140e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3531818060409549e+01,     3.0404231898111182e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1020790202183875e+01,    -1.3455079330748670e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4318122053486643e+01,     2.5119182496547430e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    7.2255186262557682e+00,     8.5037481175860066e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0341521893322483e+01,     2.3530239856706390e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.7063859989564676e+01,     1.4918259996951552e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8068827055274994e+00,     1.5002360691359592e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -4.8941532747079442e+01,     1.0410957720872890e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5094972652754938e+01,     1.0336990054886343e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9572163671389280e+00,     1.4077905688317906e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -4.9301010413557400e+01,     1.6752554681534508e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.8765161850126866e+00,     1.7250745545485810e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1825435534485601e+01,     5.8618877214899567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8415871896846600e+01,     2.9395541533267271e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.4180491506977329e+01,     1.7738170047098985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2685616097826671e+00,     4.1237125021950959e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0157160035782944e-01,     6.1463336448949585e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1522039283450646e+01,     1.1916821474013414e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -9.5229210719618287e+00,     7.3537482764438919e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.5734498891578959e+01,     8.7821973738905908e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2743558591879390e+01,     2.2216870415310086e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6746639517707724e+01,     1.6644931329925141e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -4.2270299107100485e+01,    -2.3285593699252559e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1110416406210467e+01,     2.2221809575125011e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9315430149158189e+01,     1.7363687321387035e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.9028154919538792e+01,     1.6894541292359296e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7772854831565880e+01,     7.7558351168035497e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3661195075326653e+01,     1.4133879389343851e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.4203115036530138e+00,    -1.3051188243719480e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -9.3711212139807913e+01,    -2.5069150669460214e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2253613635582159e+01,     7.0589583434345841e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9441334783668324e+01,     1.1046056785073075e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.3170491826874802e+01,     5.8224609613855800e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.0138710698702127e+01,     1.8405587902126288e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.1012035636182027e+00,    -2.9683411442790408e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5639429469104648e+01,     8.3659033026345587e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2632246315245730e+01,     2.4120810082637719e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    8.5888192989019920e+00,     1.5507417219248424e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.3304593264589748e+00,     2.5717993130629260e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.0663897418414274e+01,     1.4676530257469676e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3064774063192907e+01,     1.6728370525065588e+04,     1.1655480853674925e-04,     2.8300096881322299e-07,    -1.3951434819921005e-09, } , 
      {   -4.1372520786003726e+00,     8.0258085847678831e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.7078429448197028e+00,     9.6013061031406978e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7391654028318083e+01,     8.3131934348500981e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5358478598191789e+00,     4.3373704262870733e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.8151516526232356e+00,     1.4896397761783042e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4045300644545958e+02,     7.2141048912254573e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0630270947238447e+00,    -1.3499151746188491e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.2122876622433348e+02,     7.2140208396922080e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4000912187853594e+00,     9.4160900219271516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.5866050119575483e+00,     1.4283509287776687e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7816187364119500e+00,    -2.3455065798355458e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3135183523939340e+01,     3.2356717508253740e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.5181867800743753e+01,     3.2193578184034483e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -3.2874201733696736e+01,     6.2866661455434496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0599844023157168e+00,     7.8827108336070678e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4805973639049427e+01,     1.2007866762786809e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8038560755322187e+01,     1.4887419890096953e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.0076488526458878e+00,    -8.3006943058532652e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.7728656540175418e+01,     1.6118804123485186e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8474354781830506e+01,     8.3470537411889018e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.7330580099437274e+01,    -7.4171881634433203e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.2740316376650725e+01,     1.6547317715121608e+00,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.5528323799990506e+01,     3.5693834235028953e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0776170019286377e+01,     5.4419636843369153e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.7948921619562661e+00,     2.5161130967036257e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    7.3244486910203683e+00,    -1.6669834368975367e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.5872061671862042e+01,     1.5718522328511714e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    4.5993711659519803e+01,     1.4213509193139675e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4949472024038155e+01,     1.5525122263908303e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.7883573011878848e+00,    -8.3118306780309431e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {   -1.2630448281954678e+01,     8.5459290967223078e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1924307201917342e+01,     1.4434025785004082e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0498253764916576e+01,     6.0736951710456169e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2159269182645883e+01,     1.3005864517673160e+00,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    7.4585777409492433e+00,     8.9459783830483047e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7507326227560123e+00,    -5.0434518242195537e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8071465359632363e+01,     1.0666093792464942e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9173092550133312e+01,    -6.6475608482545383e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1138016337801872e+01,     8.6748130557156928e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.1285190899913360e+01,    -1.9177377801919156e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9333892429181144e+01,    -2.2916139255615553e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9799874588216788e+01,    -1.1020093209439142e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    1.7159103340103250e+01,     3.2462655702676126e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    8.7594229794346390e+00,     7.2905041089175002e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.1439954318426487e+01,    -7.2322319956583669e+00,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.2261027616519042e+00,     2.1828788605432022e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.1809615390836058e+01,    -2.7870431328538598e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5754854065880295e+00,     2.3382823540848600e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3132213167195536e+01,     3.3794543728964559e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7238995792664120e+01,     1.6170733242753055e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6128509246721237e+00,     1.7871338203585634e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.7542860297862539e+00,     3.8297270245987012e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.7443898641267815e+01,     3.8500276036015671e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8392779608337211e+00,    -1.0344464659379855e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.8284235602948495e+00,    -4.9200538542155300e+01,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.1686111808168398e+00,    -1.6797102471375601e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    5.7583033419580136e+00,     8.1452199208628608e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.2276775701602030e+01,     9.0279923151830662e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.3442112317557058e+00,     2.3512206172719059e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    7.6854237908258609e+00,     1.5341249102334366e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.1459898749323749e+01,    -1.2340732649600302e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.3081259478595854e+01,     7.2125376122076141e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1945498857764592e+00,     7.7521989511698721e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.7268489990523246e+01,     6.9699969061820593e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5821075936827350e+01,     2.5544492081200951e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3074974147597368e+01,     1.6171056245432441e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {   -8.0397385882550154e+01,     1.4428027679343826e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1570448132665402e+02,     1.4428223782491505e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.2307549856766158e+01,     7.2155881187100495e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6750135291832301e+01,     8.2107987806103101e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.8139954196416539e+01,     7.7712790935181401e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4337510828814750e+01,     2.4360067888969566e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.4553917368320874e+00,     4.8858796312019615e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0796189333254320e+02,     7.7714751966658187e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2186382163527217e+01,    -7.1834099431491286e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8641432020240487e+01,     1.4932470913339619e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.9500814906970589e+01,    -8.2872217910557010e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7535794629643036e+00,     1.5310864755639112e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.4317862574770341e+00,     1.3679161443025181e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2123754410762938e+01,     7.1622477539685424e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.0249828552491714e+01,     1.9899015175103372e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.5862116097494923e+00,     7.0987675615335547e+01,    -1.8807069274594723e-04,    -2.1782126004654988e-07,     9.7753467335796839e-10, } , 
      {    8.0226201152054841e+00,    -7.1720034802932366e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.5721657965819080e+00,     6.1053343602789564e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9818472963300753e+01,     1.6713706182084272e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.8761621815802073e+01,    -1.3786525203510109e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.7467632395087289e+00,     3.7106555657249100e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.2985948571382970e+01,    -4.2641582942425885e+01,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.3500290956344344e+01,     9.7030203320315231e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1538173006212176e+01,     1.5569414996768592e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2957231647159190e+01,     2.1542861801507410e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5898405278414092e+01,     3.6016230770495558e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9061585154278937e+00,     7.2126401745250805e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9862144598908120e+00,     8.8937206313039896e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0393063198121386e+01,     7.7673708992829660e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8657738641830122e+01,     2.4045264839832689e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9761779312268576e+01,     9.6523707769450684e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2856568121191545e+01,     1.1482021389696513e+00,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.2671707294509833e+01,     3.3776014842818222e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.6515626607400449e+01,     8.3773932903902005e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.6057802896358908e+01,     3.8301872645666474e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.0638308274508923e+01,     1.8798255736242947e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3752391539994088e+01,     2.4908522013163256e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    9.8588517072546242e+00,     1.5383846998550385e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6282943960621818e+01,     1.3799818145872123e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.9368915608163917e+00,    -9.4459498163076041e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.1639805576628728e+01,     7.0697750547943051e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3260933235502542e+01,     1.7016622391640274e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    3.5564269900807972e+01,     9.1170345639508359e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9281440798020260e+00,    -5.2206062945239255e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.3692749508020530e+00,     7.9652293257355395e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1722083348776311e+01,     8.3566885435969871e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.7313616873607820e+01,     7.2154855563925848e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1304542120432259e+01,     2.0181453665487683e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.4292543624335394e+00,     8.3298206752918577e+03,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {   -9.1403635069373337e+00,    -8.3275621741583946e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {   -5.3346430436088930e+01,     1.6854726390461386e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.6322499412885467e-01,     7.0191071574228481e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.5338115334186320e-01,     1.6060253667919194e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1204853451919570e+01,    -1.7935783185089840e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2845347303987932e+01,     1.4402386595770742e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0776161180694347e+01,    -7.9915985058542992e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4519102232297598e+01,     2.4607131056858470e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.1390064832846925e+01,     3.2200137139634211e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.7155324155747728e+01,     8.1421066750376467e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7988524922951800e+01,     8.6310422222065372e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8296584147327941e+00,    -1.7091734673773299e+03,    -3.9897904299492429e-04,    -4.7445873962018985e-07,     2.2922427175171860e-09, } , 
      {    1.8422730044881295e+01,     4.0562585611496783e+04,     3.5992082871890839e-04,     7.7821322464340488e-07,    -3.6938923991097769e-09, } , 
      {    1.0412694502944248e+01,     7.6665724020140387e+03,     1.8067403585736341e-04,     2.8790728326605149e-07,    -1.4160874330160326e-09, } , 
      {    2.3994349622984561e+01,     2.2581024736949515e+04,     1.1714979612689093e-04,     3.5123781569023659e-07,    -1.6703770568947951e-09, } , 
      {    9.6343212912503198e+00,     7.1857293760021394e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7354246556553594e+01,     3.3920736112188342e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0148965210760046e+01,    -9.4406901516752368e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.6832111103707405e+01,     1.0348600913843922e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4328847262282687e+00,     7.7249670034805158e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1777875499798434e+01,     1.9754925623834401e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6603885228711775e+01,     6.2239302541083816e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4610489288637481e+01,     5.7202117817014368e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0433096111844371e+01,    -6.8875880761761437e+00,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.7053384382983332e+01,    -1.5387945628750058e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.3229244354488738e+00,     9.5845952493493369e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1338861975483699e+01,    -1.7769710208876812e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {    1.2461165967902872e+01,    -7.3139218255863225e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.1248960273446503e+01,     4.3679180320214709e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.7875409462988912e+01,     8.9569009581968294e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2249164985704144e+01,    -2.1508174160705806e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9422367214570031e+01,     7.4045775161118854e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3349406122134098e+00,     6.1799830379005539e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3872904588054737e+00,     9.4256879577157633e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6923794865421069e+01,    -7.8834313922570982e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.1363971934996790e+00,     7.2136696191963911e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4222977845171805e+00,     1.6769161163844456e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.1134255688778198e+00,     7.6136333149654274e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.8554818291921904e+01,     1.5541942244198479e+04,     9.2781703036018566e-05,     1.1077022985990678e-07,    -5.2374421970263164e-10, } , 
      {   -4.8336173044932551e+01,     1.5543566336169431e+04,    -2.2515794548278047e-04,     1.7123619216788808e-08,    -9.3714484558473377e-11, } , 
      {    9.3301740814588268e+00,     1.7702121630696573e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0190055603876573e+01,     1.6860373359681249e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.4564196396114474e+01,     8.3272605319223767e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.8530872516106740e+00,    -8.3301223175278756e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.7489877011599004e+00,    -1.1191171901091127e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.3997049605087945e+01,     8.8041956875861561e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.8010320994508241e+01,     1.5932172463842027e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1348544162594653e+01,     7.8751854755596614e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7759669853921167e+00,     8.2735800892443094e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9732898699209183e+01,     8.9166586266679769e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3668555306880602e+01,     1.4827705656210379e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.8754891027323520e+01,     1.7897077338732557e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9590958094501758e+00,     7.3691673874830813e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3359399470222531e+01,     7.2144561117212743e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3977626684186539e+00,     5.0912236626534632e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9688607412344965e+01,     7.2151663928512580e+03,    -7.7472130562384538e-04,    -8.4177714637367624e-07,     3.9735329303737567e-09, } , 
      {    4.5317884913463395e+01,     1.6646566138090529e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0506871075515944e+01,     1.6127291041720342e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2862063563377085e+01,     8.3963790303608766e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5020865572219995e+01,     2.2048327477268940e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.0385672021549880e+01,     3.1572181870546741e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.8223318353832685e+00,    -8.9378113400624425e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1173422755785186e+01,     1.7383906211811427e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -9.4082074386764859e+01,     7.7002857881057444e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1398161339309274e+01,     1.7338659531706988e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.7000503562953568e+00,     8.3029542752032921e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -5.6914765562137276e+01,     9.3464171834287390e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8163309774256824e+00,     1.6107072151295946e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7890706587934115e+00,     5.8798954343556225e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8319148139209943e+01,     8.4729792353896537e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4722018724040105e+01,    -1.6184243666708305e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2610335893469092e+01,     7.6600434565768910e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.0078282403231519e+01,     2.4006432448891253e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7170387675688827e+00,    -1.9207463264693479e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.7949790984902506e+01,     7.7869191733804091e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4951167901820657e+01,     1.7288320266811981e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.6270822077366836e+00,    -8.8679814961570682e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.0648626381815589e+00,    -8.3838027602059428e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.1717401241336745e+01,     2.0729579157204920e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1124280843932667e+01,     1.4919377737294337e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3950821044021140e+00,     5.8592319180521135e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.9098876054245277e+00,     7.2070592839799911e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.6292575420315924e+01,     2.4499400983996555e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6842838570468892e+01,     2.7173347826793419e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.4637771936455390e+00,     4.2341884178550927e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3929176432494032e+01,     1.6728376154862650e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.1039490911375410e+01,     6.5861143099220908e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6413356589687820e+02,     1.7163933705096497e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3039378376729154e+00,     1.5671081718694813e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1265177428748647e+01,     3.2233918567137174e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.9780051663745397e+01,     2.3209362332157634e+04,     1.1971574525621533e-04,     3.5066505680737377e-07,    -1.6152101081213410e-09, } , 
      {    2.0479824768019370e+01,     1.6137274817929474e+04,    -3.7924768370949947e-05,     1.0148672660581272e-07,    -5.3775048694988620e-10, } , 
      {    8.8385980720143529e+00,     6.4099928718825931e+03,     1.8590386647959628e-04,     2.8671813226902599e-07,    -1.3064807559907884e-09, } , 
      {    1.2699886343742879e+01,     3.1858931281431516e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.4642237000044396e+01,     9.2910602532423964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2116442846589646e+00,    -9.4114646141022286e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3540832771320275e+01,     1.5460270997015918e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9171335427523861e+01,     1.1002285951563914e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4291648202799649e+00,     1.6465308216803314e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1170651617244118e-01,     1.6866220969814098e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    9.6043755744098789e+00,     2.1517122666360487e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.8209652251643407e+00,     1.5508615116649019e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4409441717886899e+00,     1.6867643660747850e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    9.8421770301402294e+00,     2.5063626535187501e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.6850421629129508e+00,     1.7127956080155363e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0952895790031556e+01,    -2.4184557095228320e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.5681251058031243e+01,     2.5178350743239243e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    3.7856627522614623e+00,     6.4864284855442002e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2806632816897070e+02,    -4.4115689782829577e+00,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.6199263744932964e+01,     7.2132508194733527e+03,    -5.9531047605329682e-05,    -1.3964088456997964e-07,     6.6259485797240281e-10, } , 
      {   -5.0691727642447390e+01,     7.2148749114443053e+03,    -3.7747069612412869e-04,    -2.3328749521309762e-07,     1.0926245931165610e-09, } , 
      {    8.6501771345085903e+00,     2.3852258045200855e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.7647031817686866e+01,     2.5183172220480985e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.3002158159064859e+01,     6.8506007664692461e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4663967735366723e+01,     7.8430817959712531e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2798772707314200e+01,     2.1913893032819258e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.2052116241561102e+00,     7.7195715093878098e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.1060057782936322e+01,     2.5650430275623334e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.0131109963822338e+00,     6.1408648960366463e-01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.3397455327743302e+00,    -5.5994411194932945e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6094361560626417e+01,     9.0563258046397586e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4524767248383839e+01,     8.3356913015677528e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5136639586678514e+01,    -1.2881296086946049e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {   -8.6972233877471453e+01,    -2.3332291469364804e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2799086690344046e+00,     7.8696004125686450e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2179979656128204e+00,     1.3816027481214027e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.1909325858302694e+01,     2.6997346796433339e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8624705339013204e+01,     1.4914805734647218e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5219632541081358e+01,     5.6898281155827561e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5105206964694510e+01,     2.2125879738281063e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.4400236392006409e+00,     3.2369871703567023e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.8136641686999058e+00,    -1.4043096739222341e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.7047119857931321e+01,     6.8644595936986752e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.8891022514179134e+00,     7.0551555738252582e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.3149561670030252e+00,     5.1489635374675436e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.2866911151670219e+00,    -1.1272477975886727e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    7.3515920665165861e+00,     6.6063738429242640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6690533147462947e+00,     9.4973926085950352e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7279839510399182e+01,     1.3525297475669850e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.6149543069269399e+01,     2.3776273882824833e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.6971173219272629e+01,     2.4355465489290102e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    7.0850919474658056e+00,     7.3511337034957551e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.0345327144230900e+00,     7.7041971314927177e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0586055367032808e+01,    -8.3139729163005668e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.5470427207563482e+00,     6.4537580755946919e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.6320284970716042e+01,     2.3778145710381323e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.8674449811274734e-01,     1.3688983130234110e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.7800054735924391e+01,     1.7460492070375876e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.0622464342648897e+01,     3.2279766943430037e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2767713767422007e+01,     1.1465252601477502e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.9866574446999046e+00,     8.9783289177485203e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3985294479854264e+01,     4.0541626237673418e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.4886741516411242e+00,    -5.5468444731696309e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -5.1918473486286061e+00,     2.3887178199056674e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    1.8997371580584932e+01,    -4.4322631552982157e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.6879341075069092e+01,    -2.3306471161803984e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.7204749638047566e+01,     1.6683121984597172e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9263933920334658e+01,     2.2614478228859924e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.5338195985001704e+01,     1.8002579408774909e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.0301088445294049e+01,     1.6643876338178899e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0022723508907944e+01,     8.8391063656558545e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.6477712071413544e+00,     2.4644868415972680e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.7189101592489924e+00,     4.2844013290758821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4875398607315573e+01,    -1.1458069627807777e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1183479818440108e+02,    -2.5796902045055958e-01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3002525647782795e+01,     8.3757604678499010e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8959570768606419e+01,     7.7161163238271729e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.8063618926055760e+00,     2.3243466762339987e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -6.6793300460978145e+01,     1.0037081575503251e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2264254363270037e+01,     1.6101172097209530e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.8383987548806360e+00,     1.6604423228025182e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -5.7709271379845442e+00,    -8.3744704627754654e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.9148331339423265e+01,     1.6670889360721605e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8094496240681838e+01,    -5.5961547947288091e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.7439730349691533e+00,    -3.2202766428059113e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2823819055202510e+01,     1.0546720738442129e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3180984115394306e+01,     3.2407056773148746e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -1.3053970545241624e+01,     6.0587308533837302e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3624639203784117e+00,     1.6291593827446623e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.8985670878896197e+01,     1.6139883471715018e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.4447925479157956e+01,     4.6049179478007609e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.3526806239396914e+00,     1.2726146784155719e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6160434912995569e+00,     8.4895759721848208e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1067346929835333e+01,     1.6151874205521275e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8460689904026317e+01,     2.2676635062314483e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.7709266824267718e+01,    -8.8837045045185369e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    3.0559207274978281e+01,     4.0593262914172010e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    6.4283120955081028e+00,     5.9548178630515622e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3649318179332095e+00,     1.3264810565354341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7216158263373224e+01,     8.1529081270293191e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7265809887662797e+01,     3.2412316437781119e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    8.7643824959875303e+01,     1.4286132406542147e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4802583139764764e+01,     8.8878205772175716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2254981948492237e+01,     3.2218161347835412e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -2.5090414548600904e+01,     1.3093176045289326e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.1021549753426005e+01,     3.3523962848773146e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    5.0096987528146499e+01,     3.3524667480689299e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    8.5672939781064663e+00,     5.5471992532337629e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8795486765123641e+00,     4.6357697657897937e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3394452394492058e+00,    -2.7047461220255691e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3964715162086758e+01,     6.0602827226493901e+00,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {   -1.5415950649877388e+00,     8.9493724261443986e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5954544502134901e+01,     1.2801879355204449e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.0113911172665730e+01,     1.6306048609211668e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8230314875364954e+01,     7.9843723206972209e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.7482545803403156e+00,     9.0966844207557278e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.8704151299719143e+01,     7.7693460865759789e-01,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.1648319174124065e+01,     1.4449455638895517e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6663361935786224e+01,     3.8891830614469887e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.0155189031939154e+00,     1.7312862734328760e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1159889604337131e+01,    -8.4219914292528883e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.1029794357426763e+01,     2.4426458794702499e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {   -4.6433674832552043e+00,     8.3570431471950942e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -6.1223730739818066e+01,     2.3243144843414266e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7704609370898652e+01,     1.2729474352360114e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0980532140075887e+00,     1.6097438737500921e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2093676930628751e+01,    -1.3900827645847073e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.2738989136578169e+01,     2.3227596101575109e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.8708152687519508e+00,     1.3657831535047329e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3759288474493765e+01,     1.5625237583352000e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4016505140218715e+01,     5.3133923668065636e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -3.5059051996794054e+01,     6.7207420143954569e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0256383860795582e+01,     3.2164088723231802e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    4.0676978380323856e+00,     8.4630514519236494e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6461946608733093e+00,     8.5356110582396650e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.2164926417860878e+01,     1.0136061918849631e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7062115897807665e+01,     7.0743432435334844e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.5976399613618147e+01,     6.0814100979913319e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.7954176412131137e+01,     3.2985025093299286e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    7.5767259777708418e+00,     8.5408707228720323e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -8.9338996996775933e+01,    -1.3242760766708539e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0676284358520526e+02,    -1.3240799735231762e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.6781398298630306e+00,    -1.6873491270880699e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -1.8365852208793484e+01,     4.0528476248401261e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.6051064748685100e+01,     7.2121139815771730e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3681466635810345e+01,     2.1673355450931325e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2717850377691434e+01,     4.0529180880317421e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.7908011533574381e+01,     3.3050613428832861e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.9465130119450627e+01,     6.5808242482573696e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9425532245676511e+01,     1.1701586842021854e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3246271003810826e+01,     1.7672152448589055e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2426510781926763e+02,     1.5544485258662386e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8416565687088138e-01,     7.2832412980119279e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.8481331272554712e+01,     1.3738421114532321e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8490120231153941e+01,     1.5578091361119494e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1151645273136694e+01,     1.6102698402133288e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8611108890239798e+01,     7.1027291769437425e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -1.5522928752529129e+01,    -1.8700756069966519e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.8249697344558143e+01,     9.0353938597021681e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.9309662974928372e+01,     1.6602025919263913e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0169047396279185e+01,     3.3176345960319222e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.9783561460342929e+00,     6.0453616815889245e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8406061836663817e+01,     1.5540805406302299e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.9335506038955454e+00,     8.3532840643121781e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.3915831872107283e+01,     2.5612080659381187e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    3.6214714664332341e+01,    -1.6656416447002579e+04,    -7.0636635813411345e-04,    -9.7462094313450016e-07,     4.6112083651371291e-09, } , 
      {    1.9502568113896999e+01,     1.8327682505537836e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.9227714155444549e+00,     7.2729320167564410e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5243485857535518e+01,     3.8825707321945418e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.4367727340648216e+01,    -2.4839796528095530e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.9982817810574659e+01,    -1.1842303713721208e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.4633728000892710e-01,     2.5784116423153730e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -2.9987486988186838e+00,     7.3047330376704122e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4874241315811837e+01,    -3.5133424023858379e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8126279578768738e+00,     1.5544703174065618e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4440570478328656e+01,     2.4080995180739807e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.5042121932611870e+01,     3.2449501507362846e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -6.5533684122802711e+00,     1.5506671662841258e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {   -6.0438788111477493e-01,     1.5505994528314672e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6383069032750637e+01,     1.4985440010550810e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.2806317962111343e+01,     1.3522844801065163e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8694347878983354e+01,    -8.4242926290926225e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.6572516058813122e+01,    -1.3397200533854448e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.6110301852508972e+01,     3.1605592231523173e+04,     2.1027195957046868e-04,     5.2718571809935578e-07,    -2.4531135931829518e-09, } , 
      {    2.0109027708687893e+01,     4.0772135077327504e+04,    -7.2747262477848091e-05,     2.6749110041537536e-07,    -1.2807807827551721e-09, } , 
      {    4.1751943747413893e+00,    -2.6276495252134948e+04,    -4.2591308521512108e-04,    -7.1435356656765692e-07,     3.3837086059358957e-09, } , 
      {    1.8308329654871311e+01,     1.3766066728531738e+04,    -4.0350856025479450e-04,    -3.3657772871263786e-07,     1.6358049932948740e-09, } , 
      {    9.0867788273327168e+00,     5.9912160020993142e+03,    -2.4400241086066759e-04,    -2.2466401730446609e-07,     1.1603433680939410e-09, } , 
      {    2.3201168852174803e-01,    -1.4203537155583035e+02,     3.7100948723324572e-04,     4.3678803785882535e-07,    -2.0654032442628450e-09, } , 
      {    1.1285165230549158e+01,    -1.8830088428940428e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9330333151558193e+01,     3.3443634398525835e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.3824538133382426e+01,     6.1944742960608164e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.9153006841003481e+00,    -2.2627659553213221e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5269297044710655e+01,     1.3202677415381408e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6946450728037028e+01,     1.3858953001708426e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9975312055855108e+00,    -4.1408011129235014e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2867475762185908e+01,     2.2171169386552952e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.3575127090290678e+01,     1.8994785217190070e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1242223585577033e+01,     2.2612529344978262e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9930258512347088e+01,     1.4406795822939814e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.9005326634535784e+00,     1.5009364272838435e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8759329520824309e+01,     2.3387425940528072e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.4087590310078738e+01,     6.5853962509044877e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2106339833775079e+00,    -8.3463234510007824e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.3865131677311169e+01,     3.1945705699958307e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.0730327794651662e+00,     9.1135793783901991e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1127378487925299e+01,     1.1255606250903797e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.2012287831744821e+00,     8.3293130525001307e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9517684680530042e+01,    -6.0199919441898498e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.2614114052710121e+00,    -5.9712918153105898e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.6892101427527258e+01,     7.1975544347227242e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.0464273727311699e+01,     1.6712739779636591e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1399219608577916e+01,     1.6534523974219086e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3154461593844348e+02,    -1.4012734812003316e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7682687054087456e+01,     1.5579514052053246e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9759540310050658e+00,    -1.4486898027380212e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.9482039301075353e+01,     2.3108151395881374e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1979051998602614e+01,     2.1579831789347529e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.0480962908275593e+01,     1.3919262851565858e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.3681550807537013e+00,     8.3352428901638214e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.5036385299388147e+01,    -1.5497989875028509e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4426083616273482e+01,     1.6224030972440692e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0890070280235861e+01,    -9.1652045222441075e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3554888779802763e+00,     6.0803065705989102e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8757898953202361e+01,     1.4936487958583144e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6223967061578808e+01,     1.9669740514723344e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7638410942366578e+00,     8.6921535237147109e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1193819108646162e+01,     8.8863978862838212e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9964774055829000e+01,    -1.7774312608556280e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.3363850740620762e+01,    -1.1095614873190241e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.0273838793927823e+01,     2.4431061194381964e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.5226549107246949e+00,    -1.0888914097444595e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.6864068897672091e+00,    -8.4389140956867577e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -6.9151601092418787e+01,    -1.3086359968085853e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.3994597368638708e+01,     4.6072033348893456e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5871037327908851e+01,     7.7261229748265050e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5976414599142998e+00,    -3.1710705025588777e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6879055559970581e+01,     1.0182462584674973e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6737705318555722e+01,     3.9256465650752609e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.3270110420470019e+01,     2.1712078374396471e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.1403787584900762e+01,     7.2965461586268739e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5036783633056778e+01,     4.0511907883550557e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.3694590784552219e+00,     2.3933578864882020e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.1854040233710954e+00,     7.0993305412396467e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.6777914619098965e+01,     5.4247413146896997e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9492996623576623e+01,     1.4446174674273039e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4904704870032553e+01,     2.3871086685400282e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.0388900393054492e+01,    -1.4118085074443286e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.4901735922697386e+00,    -8.3280697969501234e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.2081915063404836e+00,     8.4404697391193276e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.0373621463262339e+01,     2.9766760260303894e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.9394147445339470e+01,     2.4582236593122674e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4062264734474450e+00,     5.5400857985649998e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9975641619072743e+01,     2.3871804744417888e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6060912393667341e+01,    -1.0954408895580682e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.3085080091782267e+01,    -8.3132615708336907e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.7908414237751398e+00,     7.1933993491553238e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.1102557076063022e+01,     4.0163039542355706e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -1.0151816314341141e+02,    -1.1483274204918871e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1789161780601152e+00,     8.2551037650029593e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5600140177389191e+01,     8.9776625260741785e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.4798877603825300e+00,     8.4526538273487131e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.7788183019802446e+01,    -8.3010824476468006e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.1976068228359509e+01,     1.5525687008644314e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.2296652364665661e+00,     1.4908413230743468e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.1571910000781941e+01,     9.3809595490778884e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5147774562510865e+01,     7.6932779342265758e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -7.1066018953338963e+00,     1.6171390507705950e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.7569342664050684e+01,     3.3731563430722214e+04,    -1.0377242918677173e-04,     2.3061132369337297e-07,    -1.1054721556659640e-09, } , 
      {    3.3734177275757204e+01,     3.3736165830401675e+04,    -4.0839793046946820e-04,    -2.7021090516639987e-07,     1.2672059996841049e-09, } , 
      {    1.8411646326615056e+01,     2.6046100029204743e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -5.4320095736690277e-01,     7.2160117493404932e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8418237619310961e+01,     1.6538904546204194e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3592004663092061e+01,    -6.4142549624499861e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1869556650589111e+01,     1.6674243596144435e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.5328994100422179e+01,     7.1519297154858978e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2074943884165027e+01,     6.0990819902344656e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -3.0073577937318792e+01,    -5.8884499910570582e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8015303807110080e+01,     6.6501896302259584e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    5.8954144409583591e+01,     6.0997866221506138e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.3078772541540529e+00,     7.1677051678009339e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.3107595088843226e+01,     7.2143171298430671e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4009848832316454e+00,     2.1570091763571676e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5899614170351799e+02,    -5.3829371707302926e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0644541891832041e+01,    -1.4719265880175317e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3057951018934340e+01,     2.8313288811792190e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4290110301218235e+01,     1.8052929513408053e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4198683661452092e+01,     2.4916184496155405e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4567380008544752e+01,     5.1205726359469850e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.0551742768879002e+00,     2.2971433693420782e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.0616896145652074e+01,     4.1850705389616618e+04,     4.8120841265133303e-04,     9.9174456235128896e-07,    -4.7049228496956025e-09, } , 
      {   -8.2473157286299212e+01,     8.7406515035142511e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2806662174532768e+01,    -2.4677660310820283e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.8614675544487199e+01,     7.2138086010745983e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6245145407288753e+01,     3.9478634354905713e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -6.4811881610502748e+00,     4.9780935827449866e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.6314328984662723e+00,    -8.8668275104361655e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.0764347885284359e+01,     3.7724232957992397e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.1323106089807810e+01,     1.0109573537433566e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7646149249055753e+01,     4.8319246902749046e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.1015137129161818e+00,     2.7222471903236510e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.7778298071538607e+01,     3.1051727152864954e+04,    -1.0144900810142239e-04,     1.6481725902999783e-07,    -7.9204011082864902e-10, } , 
      {    2.5422736343436764e+01,     2.2723035728139828e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {    1.4982672306189915e+01,     7.8085833932043488e+03,    -1.9023751901229817e-04,    -1.4892438782407371e-07,     6.4858859072514825e-10, } , 
      {    2.6726235895290102e+00,     8.4022790844472929e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8654850521476657e+01,     1.6764424571265059e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.3924121078549367e+00,     1.7253041180361790e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7873605299555066e+01,     1.4301095453230986e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3807698726515708e+01,    -6.1568834672316370e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.7295714705885803e+01,     7.2163403818169418e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5287721365225408e+01,     5.9574409818170188e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2028509007445592e+01,     1.7965314458154549e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1248184586096453e+01,     7.2260604760062088e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.5847706447494723e+00,    -2.9219990533907990e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5976099759959652e-01,    -1.2539853377749032e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8921485655486585e+01,     1.5485666391519189e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.4630461649160313e+00,    -9.4613441921926242e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    7.5674350424156724e+00,     8.5820117240650070e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.6427056318669873e+01,     3.3314413382942425e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -1.8733767091756199e+00,     2.5048431081204348e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2283184791473772e+01,     9.1453862864208022e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.4696340111041806e+01,     3.3315118014858585e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    2.1484097678919245e+01,     1.7942525829250499e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1783746373477296e+01,     8.3378649532729905e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5517026126924337e+01,     5.9724040285058591e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1624228333248894e+01,     1.2558622086820225e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9984587319849766e+01,     2.3890633384617311e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {   -4.3076613500270653e+01,     8.3563004018034517e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.2469362392155819e+01,    -8.5477773168504598e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3255436275491352e+01,     1.5083663278027478e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0034103926064525e+01,     1.6853814404081517e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -4.9036685467761863e+01,     8.3377268185848279e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9256059636486027e+00,     2.6346300715674803e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    5.1211634091961908e+00,    -9.7492788891318696e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.5945288186253151e+01,     2.2345260268967868e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7941417990491928e+01,     1.7483033975506929e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3585014608442290e+00,     3.8646201409325977e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1909310673797526e+01,     1.6597864272425453e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9499634731267403e+01,     8.3661625320612893e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9811900893877148e+01,     2.2992103770995345e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.8182939720167134e+01,    -4.6023996794683626e+00,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.2596487575401952e+01,     1.5539151494149039e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2890351857044958e+01,     1.3802447978188307e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4839024793569747e+01,     1.7994180112869646e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0994723053439998e+01,     1.6153037667028297e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.9972951765546156e+00,     1.3082493047212976e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.3474084098051129e+01,     1.7188787633933116e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.2544818060167302e+01,     2.9026485138402422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0833093677118868e+01,     1.4050852504740316e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3975797330308879e+01,     2.4238729535080238e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.1943165733382029e+01,     6.5791963248135598e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -5.4485738278539202e+00,     6.4411388372723131e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.6432218857282388e+00,     8.3507326781698539e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0852849515985472e-01,     8.3066501712803984e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.7327874115999862e+01,    -6.5258045865502572e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4060019717115189e+01,     3.1783680220259124e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.4984542473067640e+01,     9.8942163478973489e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6276025480519252e+00,     8.1528634522827715e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5121825420230071e+00,     6.4892614755651869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.8636257281300495e+01,     1.1309855641182480e+00,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -1.8617010232293147e+01,     1.2119971532045959e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8754671968888510e+00,     7.7953014073796021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1758225654711467e+01,     1.5559821571723607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.6951636508529084e+01,     2.6482170741643395e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0686631688327722e+01,     4.1855307789296086e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    7.6532123839286356e+01,     1.5546357086218877e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0856682771632970e+01,    -1.6854313076958595e+04,     1.8989705860473902e-04,     8.3746715274861239e-08,    -3.8959627413962306e-10, } , 
      {    7.3730680503491888e+00,    -1.9151578184339633e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.5523130522945920e+01,     1.3545386706196219e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1648616898301039e+01,     3.2445735021867342e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    2.6763913507435237e+01,     2.9184838081715643e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -2.3740173570907558e+01,     8.7969634555905031e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3929928267263485e+01,     7.3898461631546397e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3529271993751554e+01,    -8.9509655353757207e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.7915826973417076e+01,    -1.2496152932484601e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.8969793400619061e+01,     2.8480052700310989e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.0920322909497344e+01,     2.5075091787788977e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1369189047056743e+01,     1.1581788832155120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3176462227533730e+01,     2.3865545660822667e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8755750946148044e+00,    -1.7635996606153458e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.3435285782257878e+01,     1.8739649145598014e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3141362646680065e+01,     2.1122600126897909e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1210297440177312e+01,     5.6218429252107273e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5294314540215652e+01,    -8.3318046705503402e+03,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {   -1.4624409859876103e+01,     6.8962929569382322e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9654696529358979e+01,     1.3856906044536890e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6228209990983757e+01,     1.2400887732479796e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1048480495930349e+02,     1.0164056862467987e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6665664609230184e+00,    -6.4405532856118225e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2734777171133260e+01,     2.0574474635180668e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8327319179657994e+01,     7.2332410593809363e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    3.2078505577637991e+01,     1.1852762240762557e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8545443589189166e+00,     5.4873360564889008e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.0208247097213246e+01,     9.3689590885597954e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.4588897961888264e+01,     1.8699834243700105e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1591284913773023e+01,     2.9246136263389344e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.6772081616762115e+01,     1.5578836917526660e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0193236916472181e+02,     8.9570970613445097e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2733011978066635e+00,     1.2560123656589816e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5503321956374691e+01,     1.4412583702629236e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -8.2790421727761839e+01,     1.6633109975840773e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.7600819268344843e+00,    -7.6802720697089023e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.2879214124509375e+01,     7.8423771640551040e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2367366123547175e+01,     3.2750892463667627e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.4555293181885141e-01,     1.4692278000985316e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4144178830304689e+01,     9.4325032726316986e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5264043525352015e+01,     1.0196341183094901e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6381425657960506e+01,     1.4914087675629615e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1397842160427158e+00,    -8.5977594675806467e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0524022576895291e+01,    -1.9517742646756817e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3587577328917732e+01,    -6.0818992950822903e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.8265291852580742e+01,     2.7566368796486408e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9246072826049954e+01,     8.4085448855769846e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3670306538862512e+01,     7.7371436015488698e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5336693622268456e+00,    -3.9957992529271496e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.7453528710068237e+01,     1.7252268256255547e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3990495205180260e+01,     1.6189110818584875e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6646482330124464e+01,     8.3927734585797189e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5355092015113541e+01,     1.6681655723059732e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5124601620251097e+01,     6.7453739175137398e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9672407144425900e+01,     3.9467169102304237e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.3557685046593017e+01,    -9.4771014114943864e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {    1.6245539314082563e+01,     1.8387201082562635e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.5485000189704053e+01,     6.6826580809759698e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3858529569481327e+01,     8.1754652915284241e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.6455638304961555e+00,     8.4334661576404869e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0863773493444874e+01,     1.1439077754766062e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.4573983629568739e+00,     5.3131406019977176e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6668821887107910e+01,    -4.8728737666842881e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.6907213726221727e+01,     1.4190967288008618e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5857721966787274e+01,     1.7455507172599719e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8849936881163734e+00,    -1.7363771887121090e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.8941309369303632e+00,     1.5304837842515219e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8130138479629810e+01,     8.3827640493287399e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -6.4067892240127080e+01,    -1.3015241006864017e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.6938790113001918e+01,    -1.3281308227692898e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0723150583725449e+01,     7.7224250931243114e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7394206832539211e+01,     3.0027981596876339e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.6519962691662116e+01,     7.2117853491007236e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2992935637460306e+01,     1.5916630435553599e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2833807654522161e+01,     1.5049433439120583e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0419707700509246e+01,    -4.0392421856052279e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4315593244916666e+01,     5.8707066464397085e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -9.7098819280644868e+00,     1.1610139657235904e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3127046449868075e+01,     3.9545164805301738e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.0121628680893217e+01,     2.3855713230761496e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.2803988906066081e+02,    -1.4366785767463966e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2488917559217056e+00,    -1.5541650762791533e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -1.3342124982262177e+01,     8.1555379593455036e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6668228535653590e+01,     1.5038409107762001e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.3592607082565817e+01,    -8.8883069041980089e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.9241552088611877e+02,     6.5615333986435871e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1167310739607413e+02,    -1.4016204806776670e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.4408739576486624e+01,     8.3934780904958698e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1915516825599139e+01,     1.4206509316297048e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6954228957960185e+00,     8.6959752448962790e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.9213646618253524e+00,     5.7533126752019707e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.0998586775741217e+01,     1.4839195643966028e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.8462627918915189e+00,     8.7877824368816073e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9164706154845121e+01,     2.3848903809778029e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.8865061234008884e+00,     3.8411674481621048e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7668528036084403e+01,    -8.2828430484322653e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.2784044080186284e+01,     2.3886864825948272e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7630163214901483e+01,    -5.8245920875268303e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.1206812159448241e+00,    -2.3712437293573798e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9063070928241558e+01,     1.6550964933723037e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.9399495422606563e+00,     6.8276008513655725e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0048049201808817e+01,     1.8436367410164501e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7909994423413519e+01,     2.4464808410944639e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.5409134274012430e+01,     4.4471835934603296e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4642739199168950e+01,     2.2763376111242516e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.9317809270993003e+00,    -2.0054547073537497e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.4416563010974336e+01,     7.2501454928015337e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -2.4008912511653168e+01,    -4.2196866890442095e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.5994213023973094e+00,    -4.6796499913158868e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7844365946139007e+01,     5.8814036718955606e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6026714031300529e+00,    -3.2532558230811084e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.2622761610714576e+01,     1.9004647834250591e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8103239710959977e+00,    -7.7069427952793103e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -6.0197252881770865e+01,     1.4443667759206097e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0258978989484422e+01,     2.3803931620634194e+04,     1.4778323338147806e-04,     3.8829212533743842e-07,    -1.8427768018973463e-09, } , 
      {   -1.4826725526003557e+00,    -7.0383304670907164e+03,    -1.8846981560892068e-04,    -2.8614537338616322e-07,     1.2513138072173345e-09, } , 
      {    2.0325072915142641e+00,     1.6934446409555865e+04,    -1.8970119387757075e-04,    -8.3833981737460956e-08,     3.8814183309629446e-10, } , 
      {    9.4089898737987951e+00,     7.8761818881699201e+03,    -2.4686215708074434e-04,    -2.2396035872770372e-07,     1.1073580808854800e-09, } , 
      {    9.6933492229149785e+00,     1.6694613477928142e+04,     1.4738411051850466e-04,     3.1996801199782510e-07,    -1.5689976680379801e-09, } , 
      {    2.6232128668611249e+01,     4.7918683848511442e+04,    -2.2958953037906650e-04,     1.5496099689304096e-07,    -7.5087942930244938e-10, } , 
      {    2.1844313499196065e+01,     1.5547163517353030e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1852232298084612e+01,     1.5333204824353237e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -5.5237754360674835e+01,    -5.3313575296128874e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0920301929842301e+01,     7.3975343693210980e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.7682713191737447e+00,     1.6628996916039996e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.9397847385659162e+01,     1.5741828799673520e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4708175490375874e+01,     1.0143724401841784e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0480407674416796e+01,     1.5540366457208809e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.2389033541667395e+01,     1.7777280317080746e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6939041824247090e+00,    -1.0299316835231996e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1869037955802490e+01,     6.1796333654679993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8211438302900600e+00,    -8.3249935430229307e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4770887171871887e+01,    -2.4840452655201079e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0753379781232930e+01,     3.3836655368218824e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.5325663211577201e+01,     7.7828671328476248e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1280248149390495e+00,     1.5687869406202011e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.4522283197294843e+01,     2.3765507520486703e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5182937005316742e+01,    -1.1022267096163230e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4015519575241010e+01,    -3.7539243319756636e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3651762608211005e+01,     1.4411205050108882e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {   -9.3506488250456812e+00,    -5.6136272642169133e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7654688649900457e+01,     8.1866895545322841e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.6539182326578214e+01,     2.3242809497387094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7045041463330879e+00,    -1.7436474897787130e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.9519228524379262e+00,     2.3207089713461130e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.7040186809638920e+00,    -8.3444391682407640e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1771609676242228e+01,     7.8526797317598639e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.6545336242410556e+01,     1.5545142123159108e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.7245950835264630e+00,     1.6929065465253909e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -9.3320998675913778e+00,     2.9970527705143548e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    7.1716301338203621e+01,     2.9971232337059693e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.0358250273760007e+01,    -7.4186965585016833e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.0722665490602850e+01,     8.3288818805842857e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.6011556346099042e+01,    -8.3285009688659648e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.2853814004665114e+01,     6.8854762455785149e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.2078889922053445e+01,     2.0587592546380125e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.7957422875288174e+00,     1.6685768782860512e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4420658746356829e+00,     6.7025605553437049e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6126559483538951e+00,     1.3760003243974210e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4829243365785931e+00,    -1.7271884007961978e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5970444340769426e+01,     1.5599842188848728e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.0404547366731286e+01,     3.8788193619887243e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6342191888141173e+01,     3.2029284874094326e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.8842792989263344e+01,     4.2430485732945544e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7365084352772673e+00,    -1.1338162289745205e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.8958891474907302e+00,    -5.5885080353868943e-01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.5315687206714539e+01,    -1.2107667937982260e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7785464959693979e+01,     8.5379122580793992e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.0078766815488148e+01,     2.7975474536762558e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.2039142247562644e+00,     1.6459274742408372e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2510422349690014e+01,     2.2521530540290238e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.3437042473604102e+00,     9.4897053904487166e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4236198518897403e+01,    -1.6804799165308039e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.0015151834520731e+01,     1.5498983456674800e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.3013423228893203e+01,     2.5315926566459475e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.2381420407087418e+01,     8.3745398010179852e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.7823893648656757e+00,     9.0552147870863027e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.8776575763671964e+01,     3.7850945809118220e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.7354879916869134e+01,     3.1593159462566520e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.0845869732985111e+01,     2.3906782785844618e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.2158160473291552e+01,     8.4219761698848415e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.9383868312554611e+01,     8.5797105242252710e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.4387744000243998e+01,    -6.1257510089795323e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.1643075577343140e+01,    -1.3671770246059259e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.3372577336049652e+01,     1.8190971364348297e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0529039603746870e+00,     1.4048222672424134e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1466319860010277e+01,    -6.4181409631689723e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8567426082963429e+01,     1.0392012050505695e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.5175701106076254e+00,    -3.5709701547398508e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7007272996689586e+00,     5.7195312476990066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2843486711361177e+01,     8.5385695230322963e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.9974919217134897e+01,     2.5024771474466212e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -1.9632938421701347e+01,    -6.0576567991448701e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7653470267046174e+01,     2.5085401069044467e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7750691449115475e+01,     3.9742786624973363e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.3828752672980174e+00,     7.2272170607721118e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0742133863988830e+00,     5.3504700183000550e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1379921856360694e+01,     1.8008115385247779e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.8569425421916286e+01,    -8.4819175579218281e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.5004955527735087e+00,     4.2140690310324624e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6521342478665400e+00,     7.1500731415252776e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1603721919745604e+00,     1.6028363918937834e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.7127104601413706e+01,     8.2660596018751166e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.0596682950316207e+00,    -2.7876749255581572e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1338983121458170e+01,     1.3764481074936588e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7272283968851507e+00,     3.9559316112472516e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.9819215767652835e+01,     1.4286145833643597e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7321360147198384e+01,     3.9946440467425437e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.6916691883367534e+01,     1.6520665146989661e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4144568696820059e+00,     4.0612191494528997e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2807295113092859e+01,     2.4502383132270814e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.2894712831311622e+01,    -1.0359651579181475e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.6378810812122211e+01,     7.2951289152474510e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2945046689920861e+01,     1.5847031170943283e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9580212451033653e+01,     1.5511575886669474e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.4435592663872431e+01,     2.1385716869529597e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.0854701014468205e+01,     5.3536367279684782e+00,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -4.1237172314840429e+01,     2.3966617546993337e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.3556557444862783e+01,     5.4582429187750131e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4481881718913499e+01,     5.8622758632834921e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6236376370098604e+00,     6.7086141556444536e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6539577749658157e+01,     2.3236579174263814e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    6.6185001312455807e+00,     6.6703635668578199e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1583458651611576e+01,     8.9483625113872404e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.7865446773607943e+00,     3.3970808077087895e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1571422651093354e+01,     1.7735890666215412e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.9636311019268478e+00,     5.4662109483591257e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.4129146163074559e+01,     7.2009086701455535e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1810258046403463e-01,     6.5813460532442195e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5885624216425455e+01,     2.2528089495889959e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.4848116395773829e+01,     3.0616108286955950e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.4088343763721786e+01,     2.0924863497717353e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.9553244358182962e+00,     5.8635911455671589e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0004832428300489e+01,     8.3255781788999120e+03,     4.6359532554209602e-04,     5.4764553418133182e-07,    -2.5876930229221479e-09, } , 
      {    2.0949038999082987e+01,     7.7064173140745306e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.1246283578967535e-01,     6.2815916798832927e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5391238263320552e+00,     7.1993443570342733e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0960419799096181e+01,     6.6710749123246969e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0024921944888012e+01,     1.0160645082650564e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0299389785594361e+01,     2.4437396492925658e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2108578909676591e+01,     2.3878004670508813e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.0193731950774161e+01,     7.2846639889456810e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7282184642901935e+01,     1.4112657022236801e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9371597341664870e+01,     2.5164472144819320e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    6.6108874870189629e+00,     8.5912099877670389e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.1589543081821475e+01,     1.8025269626559748e+00,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {   -1.9526598841086177e+01,    -2.9710639221708880e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.0302311234125625e+01,     1.4270597091804442e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3992885098155504e+01,     2.3051063497216608e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.5163577824282788e+00,     1.5793788649052154e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.1774330626041163e+01,     2.2615182860776073e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1529500471989550e+01,    -1.8330409044488215e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.8250638089602660e+01,     1.4896005856039083e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7121520413157700e+00,     1.4935065267649392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2276788198194183e+01,     4.0200241944479021e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.5747803475060085e+01,     3.9361230907881851e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.0718015109108332e+01,     3.9397279324284260e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -5.7130751431340897e+01,     6.6411122105996310e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9233613985640945e+00,     2.3119185203026071e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.7311672163720289e+01,     2.3816088784722742e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6432387443630159e+00,     1.5025780393653504e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.3281897717112415e+01,     8.3174519023532157e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.4313388582556005e+01,     8.9630324841656166e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.4069657184453668e+00,     1.3295962160541123e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -4.6078016852496184e+01,     6.6809271124975439e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.1602153080100578e+01,     8.1123439184687086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0038119264567360e+01,     2.3908205476778370e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.8132853521361199e+00,     2.3242426784396666e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2323838672149055e+01,     4.0135807594666352e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -1.8778244084281963e+02,     1.4914954663676452e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.7978158445810628e+00,     1.6866708658224939e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.1138583095246677e+01,     1.6867155972337008e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.2459310221966870e+01,     8.4395264403631372e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.6192850308391186e+01,    -5.2548120477968041e-01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.3286804144997063e+02,     3.3361034688423388e+00,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.1874912689221189e+00,     8.9531857179575327e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.0056181472900079e+00,     9.8397380344921166e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7477766655807940e+01,     1.5494052424518881e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    7.7453818801019434e+00,     8.3007501171939821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7425244465296849e+01,    -1.8760288822250260e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1538185824006533e+00,    -6.2450481543621697e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1357755128425602e+01,     7.5809391339858466e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1711843068658629e+01,     8.9324158459697373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2006503460682353e+01,     1.5543857817576385e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    1.9128864360639660e+01,     1.3122495322053099e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2436706534635826e+00,    -1.6592948499637583e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    4.8896428634677331e+00,     1.4555207162948393e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4223313784255527e+01,     1.4508324790193124e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6744372231440314e+00,    -7.0786805059884773e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4641461415728195e+01,     8.6627526681968538e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5928399912547345e+01,    -9.3982692338621473e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.3201212510103932e+00,     8.4831295550750310e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.4326987307779895e+00,     5.7221610800151911e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5113874883712640e+01,     2.2066376420624314e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6208790656336703e+01,     1.4394344303414702e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    1.1738672243821448e+01,     1.5995299466698805e+04,     3.3821661712094453e-04,     5.3712924669891241e-07,    -2.4928198336658233e-09, } , 
      {    1.3338683986613116e+01,     1.7461452843026491e+04,    -9.9779237061628989e-05,     2.7639906699208196e-08,    -1.8858767381479843e-10, } , 
      {    2.0680272848970564e+01,     3.1747603222713482e+04,    -1.6063959529919291e-04,     9.0354047009230662e-08,    -3.8843756944177131e-10, } , 
      {    1.8324718276718663e+01,     2.3418911797988356e+04,    -3.1295234594054115e-04,    -1.6005706742065575e-07,     7.9790150823326309e-10, } , 
      {    1.7737117326778588e+01,     2.2790598583145755e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    8.5396004452121463e+00,    -4.8703231968762663e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.2889613395668434e+01,    -1.7429428578625634e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3237524943019125e+01,     1.6064412335340243e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6337478224934250e+00,     7.7210024021905574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2944550920942728e+00,     1.3763058384002836e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7584821747423817e+01,     6.5871779713470414e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5473551631687787e+01,     9.9017695955174986e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2678497563334243e+01,     1.5525893543489776e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    9.3085700858855773e+00,    -5.5660293416627042e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.6774024941713620e+01,    -9.4433266975421448e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.8565224768480277e+00,     7.4067515845953712e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.7386848037883439e+01,     2.5616683059060651e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6414015413251796e+01,     2.3230020218664085e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.7057822437945070e+01,     1.6791681254108586e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1769233718568145e+01,    -2.4980040799780432e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {   -1.8258445907567555e+01,     8.2488379638732677e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.0689066002094393e+01,    -9.4433132704406944e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.7847836673148958e+00,     1.5555205809709072e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.7015483397618087e+00,     2.5221360889527932e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.4521447267945096e+00,     1.6882931823605461e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -2.0642928439409509e+00,    -1.6850932806956487e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.5350702622557037e+01,     3.8863444681059620e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.0350510869177629e+01,     6.0971331063528050e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.7284233372973553e+01,     5.5282146198117644e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5718574061129303e+01,     1.3957552500212554e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8989558416750558e+01,    -8.3306557723957922e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.8073448271604697e+01,     1.4053153704580051e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.0283402065660709e+00,     3.2222679044765264e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5902655064847572e+01,     2.4992107748570324e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {   -1.9366096213063152e+01,    -3.7728848554539763e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4645349692764665e+01,     7.3380157922962971e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.8028624951693200e+00,    -5.5802562510002303e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6107459778157100e+01,     9.9848171102082088e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7477282810889143e+01,     6.5084949017659937e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9509734804549890e+00,    -2.4514258226856878e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5227328599048612e+01,     8.8767999504952095e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9237517564981012e+01,     1.5824441571361855e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -7.7951928727503017e+01,     9.9972666736053379e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.1321409417014664e+01,    -7.9982119753899383e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.0812865348184570e+01,     1.5541229036932742e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3522650546898218e+01,    -1.6880050226480431e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    4.2846205444475594e-01,     1.3072573442882340e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -3.1040089017603758e+01,     8.3993324141794474e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.7977942325126776e+01,     1.0308786011946011e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8773348981614753e+00,     2.3265680034994599e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.3749382939951829e+01,     7.2170450137330927e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5863402710919395e+01,     4.9979389174937496e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.0114189564299792e+01,     2.3167887068691154e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1705696003087844e+00,     1.5271049853741200e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0748860533615970e+01,     1.5531733663461595e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1544173276699667e+01,     2.2836999248971100e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.2186197643994213e+01,     6.5677063370579162e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.4748829770086957e+01,     1.5544279543435177e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3748504797821127e+01,     1.6650771449896958e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0221260323662094e+01,     1.6188334871996358e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.9076100796812021e+01,    -1.1496701306371961e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.9051601613830478e+01,    -8.5555138873227042e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1941184226491631e+01,     3.2373638189062523e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -1.4238980472034591e+01,    -1.5640079862269062e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6585962779740171e+01,     8.0109215162045257e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.3987815187887733e+00,    -5.8869151297608454e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.7613949185805482e-01,    -5.2353285213018239e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.1711116399163020e+00,    -1.1584192870784114e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8395667523512870e+01,     8.3192489104854030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7363450932989197e+00,     1.2871789595661356e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3526685798485047e+01,     3.1779077820579652e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -1.5291101163163251e+01,     1.6648522694010793e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -1.0053106438621160e+01,     6.6544473859859499e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0473350297311576e+01,     1.5527006546668321e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.1224020047859490e+01,    -1.5836765367201879e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5000447406115914e+01,     8.3285506374521137e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.6456026934728669e-01,     8.3288322119981385e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0229677807867848e+01,     1.3817450172147779e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1427411400719398e+00,     1.7964640193738484e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -5.0305666314168739e+01,     9.6804784269004667e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2234402825702821e+01,     2.3857939203637732e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.3345345436949842e+01,     1.0156526281196988e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4713067010954067e+01,    -1.6666243004889715e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -6.4547088693650352e+01,     1.0665389160548793e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1637541936475966e+01,     6.8886381203705332e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9005475572095335e+01,     2.2050957309585126e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1832332749313128e+01,     2.2599693500278590e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2132566397050205e+01,     8.5170787013752597e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.9439574586845545e+01,     3.9952336090804078e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -6.1499582254888566e+01,     7.5636661372189901e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7329387987389808e+01,     2.0949446661518283e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0312581830939092e+01,     2.1671054251091595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0617450808615038e+01,    -7.4962488195138076e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.8230735497824355e+00,     8.0596233818696064e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4684871488997803e+01,     1.5387649768159710e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3029111828956623e+01,     2.5084290051491011e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -8.2685265287733429e+01,    -2.9429656316304893e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5837056780859427e+01,     2.4731642834499478e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2017297297846788e+01,     6.5224769019466103e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4123717273455860e+01,     1.6181371156484989e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.2867102546253975e+00,     4.2407473734548203e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9889240636220833e+01,     2.4486635388755174e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3832596591788882e+00,    -1.8231751814136436e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2245825162682760e+00,    -4.7976201862102243e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.6212909398932229e+00,     1.0664953988925907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9219688045113685e+01,     1.5771481949936786e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    8.2164747184796152e+00,    -1.0438911389208115e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -7.2175320281771462e+01,     5.9574544089184728e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0608750802336679e+01,     1.1395306921256901e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7570733176856116e+01,     8.1240577316822737e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.2971374599929519e+00,    -2.5737149521833999e+01,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.8202044483577062e+01,     5.8177622843826866e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8061780653253344e+01,     6.6563962698676105e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.3185541665472257e+01,    -8.3103291082613505e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.1909738592859110e+01,    -1.1740174058236192e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3009747691882733e+01,     1.3347338533268934e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7748742574679248e+01,     2.4457093483998924e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.5174911591951137e+00,    -9.4433199839914196e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.8871504208196477e+00,    -1.5510452202424858e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4146760612709258e+01,     3.0457912340789251e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.0027370436155865e+00,     7.1986330115673973e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2972077813426182e+01,     6.6571009017837587e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.1121599663597273e+01,     1.6942183376453362e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    8.1135994335937802e+00,     1.0149624455928200e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1882496096217762e+01,     8.1054383885637071e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8169725996855647e+01,    -2.0678775671560288e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7846242043927569e+00,     7.4055786473022281e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8082085193741129e+01,     3.9374385103195134e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.4604252704898105e+01,    -1.1491722753032093e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4183350739525444e+00,     8.3286809025213151e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9534957125893769e+01,     7.6983062968958820e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5311901173579958e+00,     8.9881774132851533e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7176116910420696e+00,     8.3287019469289371e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.4769218126660952e+01,     1.4969803635324757e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8318628268696656e+01,     1.8875412163167253e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1841999382293338e+01,    -3.6462212159122410e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6525641463938463e+01,     2.2678820268751555e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3123458288828230e+01,     1.5553774916906323e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4363449443699480e+01,    -1.3218768252572809e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.5838707977131878e+02,     3.3528738950917614e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.1070282231712625e+01,     8.3110661120001951e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0227150630808346e-01,     3.0927843766463241e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3331291177765721e+01,     1.3443214997269339e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.8697079947235800e+00,     2.4977437775791954e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    6.9297235941517954e-01,    -2.2341728913204579e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5352503469909276e+01,     1.5179292191194374e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6832030090422641e+01,     7.5913513510042321e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.1704460644436601e+01,     2.3454988795533998e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    9.7148520558985254e+00,     2.5746379064039520e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    9.4753010542484475e+00,     7.8600024767800332e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0174578176647984e+01,     1.8735553668081423e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8726704996558809e+01,     2.5947884251889111e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.9400011306370104e+01,     1.5193151018423801e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7911500388738446e+00,     7.4299005189043186e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0411614442526691e-01,    -1.0327577700693029e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.6636307481380435e+00,     8.2309202732886060e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2339815425623998e+01,     7.1191452977588133e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4799193015932843e+01,     1.4329510281515451e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8551967440402279e+01,     3.2200130426083488e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -5.8911052083572962e+01,     3.2200143853184934e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.9581756899112737e+00,     4.9361191406292164e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.3689655884863491e+01,     2.8628336119423302e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.6906456312098328e+00,    -7.5729897991791631e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.9155657271299027e+01,     8.2560614977719270e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6225970759334757e+01,     1.8776564887513314e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.7165490055406508e+01,     2.3884952226180438e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.7624440626620270e+00,     2.3825045049083739e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.4240136459447292e+00,     4.7740590229978436e+04,     3.2456200947728281e-04,     8.7912719236635482e-07,    -4.1764759372862083e-09, } , 
      {    4.8380550469754567e+01,     4.1550855674013292e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9963049138350222e+00,     8.2930351902482635e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.9322989683618019e+01,     6.5614824068038070e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0905855515422836e+01,     1.7936539924672394e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7711257125331741e+01,     3.9454014906990953e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.1640689998255155e+01,     1.4722372072491480e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.9229150808151232e+00,     9.8827263601417017e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1333983226357125e+00,    -8.4400251132402154e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.3255988668612947e+01,     6.5722487691419356e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4509631549485558e+01,     8.1022212780856235e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.9153861126517260e+00,     7.9537041390194645e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9969956687870587e+00,     3.0914241424491706e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2256229788355251e+01,     1.5472924446278585e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -1.3035404003133763e+01,     3.2182864142867358e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.1528070629981352e+02,     1.5189712330354837e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8443091095486295e+01,     1.6015957353205253e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2444317787931364e+01,     6.0008188567903262e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3700442773651503e+01,     1.5009618537222670e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.6843292040657701e+01,     1.8071526658654562e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0579265894286319e+01,     3.9989538492927393e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.6545089708727254e+01,     2.3796194653736697e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6531872012340443e+01,     1.5543458922100108e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7636134704241524e+01,     7.3096640698263291e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2223295548182406e+01,     3.4165241988099151e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.0074162259000644e+01,     9.4275722404757798e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2356458211292612e+01,     6.6291264418358696e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5542659962890735e+00,     6.0950250790234686e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1046510615003058e+00,     1.2478768625968367e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4634028964607424e+00,     8.3080728622141505e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3737047575511385e+01,     6.0994343061925392e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.9202499790238710e+00,     6.8367896392814846e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8103548455116165e+00,     8.7062932833789237e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1735307888123401e+01,     1.5620751177075197e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.9933247864720460e+00,     7.2981740820706846e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.9951229279477289e+01,     7.1805901760288293e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2760442112667352e+01,     1.4982810178234628e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.3251485159485370e+01,    -9.2971372051313438e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6020512221271627e+01,    -3.7580948627777389e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.3402471015782620e+00,    -1.6396467337778790e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.2785841212880634e+01,     8.9696182480930474e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.8044412301459293e+01,    -1.7696627397299949e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6692055796921242e+01,     1.3595698385074396e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7564607029651100e+01,     4.0148512244796530e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.4562458636196695e+01,     2.1515621096590894e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2770914963509483e+01,     7.1978535301169286e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.0189275746408514e+01,     2.4298248112105037e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    9.3304462296209909e+00,     3.0417386093424462e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.4524713494258677e+01,     7.2110807171845727e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5824446819091204e+01,     3.9411570172776861e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -6.6363024615415156e+01,     1.5542049658267808e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1053033694845817e+01,    -1.7766875616522211e+03,    -3.3732043903141355e-04,    -4.0052465325098564e-07,     1.9445343454254264e-09, } , 
      {    1.2465120274274586e+01,    -1.3579350722344607e+03,     9.2683770672434432e-05,     1.1081386309120664e-07,    -5.2301699918096742e-10, } , 
      {    9.7434201412064283e+00,    -4.5256955688036578e+02,    -4.0430680598074135e-04,    -4.7322595539186449e-07,     2.1833632610136067e-09, } , 
      {    1.4464893148692441e+00,    -8.8522486572208272e+03,    -3.6854886387614234e-04,    -5.0581580977520112e-07,     2.3921676653306723e-09, } , 
      {    1.8803681730773203e+01,     2.4013481086464915e+04,    -2.8488485781527840e-04,    -1.2242999889059115e-07,     5.7033481445725793e-10, } , 
      {    8.1865154051396534e+00,    -2.5470248884565335e+03,     3.6352929694740667e-05,     3.5646992493677065e-08,    -6.6429170585628593e-11, } , 
      {    1.8233875279501035e+01,     4.0076319027804129e+04,    -8.4247770214287797e-06,     3.4080879467041690e-07,    -1.5740494265951415e-09, } , 
      {    1.6578107425091272e+01,     2.4675540088850845e+04,    -3.1837804128994235e-04,    -1.5878064996103061e-07,     6.8974927225134779e-10, } , 
      {    1.7340580481263519e+01,     1.5019197057688256e+04,    -2.8242423445817507e-04,    -1.9145777080696684e-07,     8.9709923552508530e-10, } , 
      {    1.7085032740041704e+01,     1.3833580822806623e+04,    -4.6516716421830535e-04,    -4.1051181508184208e-07,     1.9835133653866332e-09, } , 
      {    8.1238191184328059e+00,     3.7230628477889695e+01,    -1.5724139076419186e-04,    -1.8085421686194777e-07,     8.0368048731208877e-10, } , 
      {    8.9416349881353732e+00,     1.5752303756014680e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.1195676422022650e+01,     1.0045391746638052e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0390143517993504e+01,     2.3298541551282142e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8182484002324774e+01,     2.2650878961220409e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.1692635691461390e+00,    -1.4043601184253230e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4970368791438363e+01,     2.4003442919727680e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -6.4592904926335315e+00,     1.7642293583098581e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6086799140846423e+01,     8.3685331349507433e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.7096436766637417e+00,     3.8477264037618315e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3207457955507021e+01,    -8.4492233769422455e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.1494473580875507e+01,     1.5894088530422543e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.2727543815272057e+01,     4.7745192629657904e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.6320014517837365e+01,     3.2567420959805360e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -7.7148898859336867e+01,     7.8180975768948938e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8203589456911857e+01,     1.9920398800985640e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4926350543584924e+01,     1.7303714642696999e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6860512864608637e+01,     1.4988069842866995e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.5678923838395942e+01,     1.0388415815741833e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2912035792273935e+00,     1.6186809618745141e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9899014840476369e+01,     3.9416829837409226e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -7.3091723809326581e+00,     3.4481464784691161e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0801150934671973e+01,     3.3772221018406162e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.6832859073050980e+01,     4.8402045631533183e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.8181818844331897e+01,    -5.6423776227968858e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6780977695821910e+00,     8.3493099872361017e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.8841528193325203e-01,     3.8783591220207767e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {   -2.1294450342096333e+01,    -8.8506796533400092e+00,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.9687979432291932e+01,     2.7959389766179389e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.8698357146458555e+00,     1.7348047241524764e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1373996476143482e+00,     8.4772369911496462e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5075424113051547e+01,     3.2802707575157918e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    6.3604278985318544e+01,     2.4298587417755471e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.0524098070739385e+01,     1.6171068588780228e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6094144237015175e+00,     1.4896422496937210e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0378172411957543e+01,     2.2948891788289729e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3711853287526804e+01,     9.4210132848760531e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3043140681841212e+01,     1.6561822875572463e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.0627345659076290e+00,    -1.0522203809804065e-02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0962830592545458e+00,     2.3639556180364241e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.2831840676843065e+00,     7.2103649837566381e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3268206731384189e+00,     6.5994677427888280e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0375195978301150e+01,    -1.2880953977543159e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.6351313971855049e+01,    -7.7184327830349575e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6103980340698321e+01,     1.4347943637589358e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.4171811893546852e+01,     8.7374448785657041e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -9.7796216738384683e+00,     6.1288847400635404e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5295073596800233e+00,    -1.3117265055719026e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3603017489678567e+01,     1.5554751900731337e+04,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.3642214347320461e+01,     1.3484349437191258e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.7939277449154588e+01,    -3.3443319240394700e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2732988590544927e+01,     7.0405960269708121e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1460288621362853e+00,     2.3856026603869897e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.2663610988566312e+00,     1.4948228132641379e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.5340934077981345e+01,     7.2627647311239070e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.2512215159408907e+01,    -7.8056731579456573e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.8768254325882872e+00,    -1.0228207937656493e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.6533242977910831e+01,    -3.1029019144970960e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3180908588591054e+00,     8.3896940977654594e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2730062609703927e+01,     7.5890501511644943e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.9525210060973870e+00,    -8.4264625761616462e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2476418563746096e+01,     1.5622314755737261e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0224622282850437e+01,     6.5895580500226160e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1858747011721718e+01,     8.3904959422216125e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0654138591259725e+00,    -1.7824108539914505e+00,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    6.6805086489008509e+00,     8.8161159263383015e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3553826252240007e+01,     1.2856041852145714e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9891834872551843e+00,     7.2452412689733155e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.1539746616308083e+01,     6.6173944696466287e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7080457412518726e+01,     7.2074514659055403e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.2627689590591213e+00,    -1.8204884053239607e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4028839647818295e+01,    -4.0340383102691931e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.4164715417457941e+01,     2.0967495604873657e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0389763980297550e+01,     2.3221808221885691e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.0893566985495877e+01,     1.4900940193867060e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7935851104210172e+01,     5.9419056670793161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7591628560891952e+01,     3.0459830827638558e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.8282787371316349e+00,     1.2085747656275378e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4588135503495057e+00,     1.0027903107574936e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9116338389121843e+01,     3.7513529506440769e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    6.7081299185153984e+00,    -3.6108350517008205e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -4.5015307530028146e+00,     1.0275747329834690e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1753961794814913e+01,     1.3483029899167252e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2569374642148855e+01,     2.2614830544818000e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4487151589468940e+01,     3.2243498508722078e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0523079631578462e+01,     8.3325793175189610e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.5234188776082256e+01,    -8.3248035319312912e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -4.1289189566229432e+01,     1.0642847255417739e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1795776991953929e+01,     3.0159781229996883e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.6981923095864602e+01,     1.4890173831528931e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2175663415661880e+01,     3.4229676337911813e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.0209650310312220e+01,     3.0202769555646468e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.9004399467353110e+00,    -3.1031625858112616e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.8368837602136065e+01,     3.0791262238794101e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.9377931408391134e+01,    -4.9377457066452735e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2304723931329349e+01,     5.7064133364995523e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.7074168067461386e+01,     8.3021728081277342e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3505474301364515e+00,    -4.8737760641077799e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.1031724776783260e+00,     8.3533108726398823e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2817017240141055e+01,     2.3316432635115667e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -6.7062410394402789e+00,     2.5534752055425099e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.7155684468641397e+00,     1.6653580079840925e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.2770926758933072e+00,     2.5240505713851278e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.3131502215432953e+01,     1.6661185619059579e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4004117615143770e+01,     4.0774426446592464e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {   -1.6939870751245213e+01,    -1.3264792249367492e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1457833053585606e+01,     3.1339940020043818e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.2058352986337644e+00,    -9.2790266740512616e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -3.2863747221836306e+01,    -1.6674655846217109e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    2.7413275138676884e+01,     2.3855236379567177e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1180415213228881e+02,     1.4913938746600377e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -5.2392748308196765e+01,     2.3926802645095428e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.2955589834329949e+01,     4.0738025714231975e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    5.8113805589754421e+01,     4.0738730346148135e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -1.1133799237610425e+01,     1.6404047290742325e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7273685240894995e+01,     2.3455014518708307e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5646927962666635e+00,     7.8051585725949435e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.8082825966793827e+01,     1.3431284307855094e+01,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8979903293234656e+01,     1.5515524328119691e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3837023067732762e+01,     8.3338119973610737e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1598300657839982e+00,     2.5017252677689863e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -6.9491624298680378e+01,     1.2721571983795166e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4058152902333880e+00,     1.6685716338906946e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.2216657430788025e+01,    -7.7959850440470800e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.9711639091191611e+00,     8.8620814420706502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3135599670424433e+01,     7.5965755092119007e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.8216928352690926e+00,     7.4373159016202517e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.7966524391459648e+00,     8.1703982195811550e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2914296865903175e+01,     2.9589924398365770e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9081356558450224e-02,    -2.1202093704891854e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0333017205250620e+01,     3.0614935349662806e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.7879610258787395e+01,    -9.6722014249389758e+00,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    3.0321090263678840e-01,    -2.0082045411800209e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3719128308222764e+01,     1.6498123241858604e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.8744530597156484e+00,     3.2826143524840023e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    8.1358432955091722e+00,    -8.0199059275460257e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.4404393985435291e+01,     1.9264503241012972e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.7891995612036126e+01,     1.5511274062520726e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.8775429743436604e+01,     1.6658349251897933e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.5845016895899981e+01,     1.0900206603111928e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5070308966370121e+00,     6.9581127338027673e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.8078000698722176e+00,     1.6752942823328041e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.7791365467969484e+00,     1.6667692130705742e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.2596013497334368e+00,    -1.5905122337567243e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.5336487640143304e+01,     1.8230786266246207e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.3400455395073706e+01,     1.0705204062446706e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.8369293795711059e+01,     8.0898020133376522e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9617808170899664e-01,     3.1711514965573726e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.0068416277367966e+01,     1.6647073568194763e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0744084122293199e+01,     8.3537192144051733e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1249709567380997e+00,     1.5734270072027353e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.7650926263052170e+01,     1.2793908702172783e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0758600706517853e+00,     1.0668768299527677e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8441792763327133e+01,     2.4100173374661914e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.3245823195572033e-01,     1.5530756679636583e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -3.1394498579233385e+01,     1.7126430826904147e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1132419221154791e+01,     3.4547054340688446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1538407263484778e+01,    -4.8206321846075843e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.3592018267867957e+01,     6.1025864169016859e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3319961531194110e+01,     1.4436762229301092e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.1431206892605687e+01,     2.1335685909150470e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4644745411132128e+01,     1.5025254912448723e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -7.1245999241895731e+01,     1.0331327917077066e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3603272800785708e+00,     7.0125576776092421e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.5626960832269035e+01,    -1.1239522371909970e+01,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.5134458867415455e+01,    -2.2266737703017679e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.3055135952848545e+01,     1.2951643056513214e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2549969551984340e+01,     1.1648627220596783e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.7450892162288429e+00,    -1.2205667536886763e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.4297761668464879e+01,     3.5563674615510536e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4934855305706737e+01,     1.3986017809158793e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.1543416601564800e+01,     6.2361520086531318e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2030987549804358e+01,     3.0474121676131155e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.2487498004462793e+01,    -7.1089399043885351e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4914800623507322e+01,    -1.0419986323130954e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8129181576679496e+00,    -6.9587044005147186e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.7884761984307492e+01,    -7.9717530823748348e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.1247401068199196e+01,     6.6590497856654183e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {   -1.4271179719809918e+01,     5.8778178915719218e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4000828755495967e+00,     1.4642742268695656e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.9913114562708962e+00,     5.4357896502114627e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6528421518144292e+00,     1.1061789269220664e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.9992691257202075e+00,     1.5543108893782066e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.9934729065529989e+01,     1.5542399686585850e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0810246436167205e+01,     2.3843059781498829e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3871653499821265e+01,    -1.2505816956961385e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.6815990358297945e+00,    -7.7152588925544308e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.3333152350719040e+00,     7.2102495736456976e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8458754918708408e+01,     1.5056433315963208e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0251446364718010e+01,    -8.1245564045593510e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.2521602557399230e-01,    -3.7664854955020530e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.5062675538127181e+01,     8.5689090722336005e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8300625202483189e+01,     7.8423637369536491e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7696504280603818e+01,     1.5591456155849033e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.4359849093176798e+01,     1.4851168326671737e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6856648984203719e+00,     3.3131294195038237e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.2670057501062013e+01,     1.3719636052543814e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.8894464346881996e-01,     1.3513215091906352e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8606589763851300e+01,     9.0989856205954620e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9097730612809109e+01,     1.6645892861694607e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -2.8519948901816711e+01,    -1.6668872837205898e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.0230514111515641e+00,     7.2020652549114566e+03,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -9.6114358573099921e+01,    -8.0939704071521845e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9262466650858670e+01,     1.6512267733432196e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.3572753187727313e+00,    -9.0437495344848248e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.4077909140583355e+01,     1.4464174147320073e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {   -8.4855682651663535e+00,     2.3311830235436202e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    5.6459360117225241e+00,     7.7805659330078897e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4474258385832854e+01,     2.4821780964235222e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    5.0995044570993890e+00,    -2.5212544420814884e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    2.5093406531445950e+01,     3.8210553932213152e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.3264242804152634e+01,    -3.2532002462192531e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4900755591093464e+01,     9.2386862909485681e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.0258414869927659e+00,    -9.0683326982857579e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7130317651594410e+01,     1.2094607811714843e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1360579083976649e+01,     1.6875810703733714e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.1685769944241802e+00,     7.1676621996334889e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0175454442801810e+01,     7.7720884905588564e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.9006364198482051e+00,     6.5489955185440031e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5221712139960726e+01,     7.6519324393982424e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8267791019168296e+01,     2.9513424701595970e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.6275998138012419e+01,     1.5403034668258610e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.3582330152696318e+01,     1.7461458472823553e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2160505048571002e+01,     1.1168955481657392e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3341786360637707e+01,     7.7130030780019615e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.1172845467714012e+01,     8.0192829987669129e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -1.7748515160256712e+01,     8.0907684157853309e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -7.4703458612884987e+01,     2.3847172841299602e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.8828897478063041e+01,     8.3083412815300180e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.3540006622566835e+01,    -8.3490415679202342e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.1042815069379017e-01,     3.1083431037584214e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.3027104599226682e+01,    -1.0766362338128516e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3729062761798163e+00,     7.7706657996251024e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0701392429409395e+01,     3.0870892042589927e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3972159470159507e+01,     2.4603364571362967e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.2587596278147856e+01,     1.6816642457041900e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4102200257964119e+00,     8.9349577563259409e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2908014982025549e+01,     2.1872735526180386e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.2715812506078592e-01,     1.1454810238913653e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3074866750826745e+01,     3.2948976676896877e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.0858451709448008e+01,     8.3155285597331367e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.1473425649442071e+00,    -8.3418542897171155e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -8.2862956566348700e+00,     1.5540349354165755e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0860940745446197e+01,     1.8330429215742119e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.2794628655407916e+00,     1.6240951653249474e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.2786255442736296e+00,     7.7857034715909904e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.7495771590319311e+00,     6.3211035465486896e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8624708117256752e+00,     5.4887587474226539e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4236966778110121e+01,     1.7286401779962671e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5439961683653053e+01,     3.1087586123151621e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.1902463116358240e+00,     1.0770893159465368e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3645532841106068e+01,     3.4558262957792133e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.6819810872050638e+01,     1.8826215552968952e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5834506968503918e+01,     6.0164687846091010e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2286260821326159e+01,     1.1832098724459049e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1518243963630031e+01,     8.5070892953690673e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.0963689245804762e+01,     8.0155239158839977e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0773149995697512e+01,    -6.0066482339099002e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.3682735350139508e+00,     6.5504182094777561e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5640142679323450e+01,    -1.0501942094536246e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    7.0628294574180757e+01,     2.3147966297779290e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8947094414802304e+01,     1.5901993832373981e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.4794013359382568e+01,     1.7251903377195769e+04,     3.3288885413512747e-04,     5.3836203092723777e-07,    -2.6016992901694026e-09, } , 
      {    1.7680043785517753e+01,     1.6066287142314141e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.2139744348923543e+01,     8.3658976728374964e+03,    -4.8307077592594708e-06,     6.9513264336638819e-08,    -3.8338581088460996e-10, } , 
      {    1.2638893778112482e+01,     2.4533540357254649e+04,     5.7861276565536288e-05,     2.7681823690076927e-07,    -1.2660472949862531e-09, } , 
      {    1.7306875844605003e+01,     4.1819225161953385e+04,     3.5982289635532426e-04,     7.7825685787470479e-07,    -3.6931651785881123e-09, } , 
      {   -2.3428888937918524e+00,     6.5916553344997074e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6621237740479131e+01,     9.6213550112186240e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0262072190789894e+01,     8.2958763996698399e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8769554252981806e+01,     6.5906863125692071e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.7233124281472829e+00,    -1.0541027862448933e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1393658681010949e+01,    -1.9807473744312358e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.9044276121514358e+01,     1.8994080585273921e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5882236671236640e+01,     2.3450386395854530e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    9.5642252440716518e+01,     2.2128502857046522e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.4476520583435910e+00,    -1.9524856101425582e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0229955561085440e+00,     1.7782732152473704e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4448435927631458e+00,    -9.4831348858893325e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2679030912631955e+01,     8.2981277289513446e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.9102603651168764e+00,     8.1371756428817298e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.1321895756824844e+01,     6.5992617916846439e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4375872592554149e+01,     2.3147578155985753e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.1828689592013983e+00,     6.1217410053079057e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0238644972060568e+01,    -1.2439959317687071e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.0461656317934356e+01,     5.1941533763400366e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6322300329632494e+01,     3.1106838488345769e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.0949764471712086e+01,     6.1017355060322734e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {   -4.6157837313326890e+00,     1.0350680937129891e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4883460159202613e+01,     7.8420418180279266e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.9243415906798404e+01,    -6.2775529551305954e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7477798639497614e+01,     1.5074065342238866e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9352193784595663e+01,     1.6548350360774901e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2578967447190273e+01,     1.4918249474747741e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2700183768914766e+01,     2.0714160046165729e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.1420022093273374e+00,     1.4338396871837187e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5006349986915176e+01,    -5.6888783253893016e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6802996436010556e+01,     4.5480653561723182e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3745776556881019e+01,     6.2752622786348938e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8676592668482073e-01,     2.3599741278466325e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.8627443983091609e+01,     1.6657284797876411e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3023181378639044e+00,     5.9993961658565731e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9327362343389358e+00,    -3.0977670917185560e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4461995854090803e+01,     9.2659087628518064e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.4691651584562706e+00,     6.0207260738344248e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.3547966927787094e+00,     7.2144174690569398e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.0747443047766740e+02,     1.6657480901024093e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.3408961845635883e+00,     5.3602917055089838e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.7095766575899205e+01,     7.3905954698173373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0355535600598298e+00,     7.8363369761094273e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.2554818513789126e+01,     3.0616844643758577e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3149220331554337e+01,     6.9269611386159868e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.4966405147376474e+01,     8.5328264448909013e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -9.7937548529955620e+00,     1.7834368215745515e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0825067819113162e+01,     1.4349417498559551e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1744978137196620e+00,    -1.0622382893070382e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.4318012403588298e+01,     1.5894344246504602e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6561192008637668e+00,     1.4328087590581699e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4146377723485868e+00,    -8.2669128783503002e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.2098868336109252e+00,    -1.7609698282991624e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.2020740804725950e+01,    -2.3758781519228978e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3192568892704747e+01,     1.6833862233244887e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.3552272698298953e+01,     6.9044073501847743e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9408583671659763e+00,     5.4343669592777096e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4377619355130420e+01,     8.0006636556483463e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.6079422957704210e+00,     9.4547947123790182e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.4830545691220713e+01,     1.7550270478295715e+01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.1780866756924908e+00,     4.9492370518286698e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8231221358744598e-01,    -2.6271938610134448e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.0282504169192910e-01,    -2.5002994954984162e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {    1.0120930737605075e+01,     6.0530766085346404e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3508567148670110e+01,     1.2245783210455544e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7579148603013021e+01,     7.2137082618607237e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7948080551942311e-02,     6.5731360420909105e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -6.8641301023693906e+00,     4.6337683431478063e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0549087029404681e+01,     1.3515516291746086e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3849326806188900e+01,     7.2178761572719677e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9528667042753781e+01,     2.4342018945614196e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.8930646026321529e+00,     6.7266383638456582e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0198782916996098e+01,     4.1781155506048108e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.8261051888235187e+00,     1.2902941190848134e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0125468987012709e+01,     8.3904699710999521e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1077020305998541e+01,     6.7439512265799867e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7934611015292088e+00,     6.8056533678985829e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4820997180844985e+01,     1.5612584134089331e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    7.4265065926062492e+00,     6.5960645616687771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0406497260426383e+02,    -1.9475210194959800e+00,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.4502107124730397e+01,    -6.9178432553096229e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5634648412153389e+00,     6.5147619750008926e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3144167011336965e+01,     7.3089804331588521e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0303199346159431e+01,     8.7194111945783807e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.3112716741351207e+01,     1.5956445337451511e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0802535870614880e+01,     2.3887445223233568e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.6463082940794017e+01,    -7.2947436316642161e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.8953005624910492e+01,    -1.6811163674473046e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.1555805952196721e+01,     2.3560908038384143e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2766635995905089e+00,     1.4949391594148401e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.5072801983308803e+00,     1.7996031232252224e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5536483311643323e+01,     7.1825826377956000e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    8.1753092866913555e+00,     6.4835835714864088e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1499911576270712e+01,     3.1905684233692427e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.4567450679170229e+00,     9.6808665686940021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4852023706379018e+00,    -8.5671601409098912e+00,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    8.3883249671315028e+01,     1.4564843433378257e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2186875641105281e+01,    -9.9518385357255647e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5150716617736343e+01,     7.6954528075236631e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.9732807629460840e+00,     1.4999054991582947e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5399952282360058e+01,     2.2756425249898839e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5479041801728972e+01,     2.2862755350065174e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.2813899932829372e+01,     6.9284530222684452e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7907046030643997e+01,    -4.8862217406048512e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0411826288915350e+01,     8.4000370460955983e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.5419858585347882e+01,    -5.7295065485920122e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8975917115571789e+01,     1.7192390429968033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8569178027001105e+00,     7.9629024027214955e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.8298476659189358e+01,     1.0575483554367962e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1076992938029928e-01,     6.5886001944710806e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3977757837345656e+01,     2.6686634946828690e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9652568893851441e+01,    -1.7979259674707533e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    1.0846735053124034e+01,     7.1404752057366659e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6188333141608439e+01,     2.2615541890284876e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2183343479988680e+00,     1.0660686892241729e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1509907248338376e+00,     1.4945625108652897e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5385874200523793e+00,     5.0731899786661379e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7319465022627448e+01,    -9.0670172211579484e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.3975051966734760e+01,     2.1953707934717168e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.7366582502121926e+00,     5.8128312522267697e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2060694394714311e+01,     1.9191276705935936e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2475007147874443e+01,     1.4381768033259767e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5088423822703476e+01,    -9.5085538934818032e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3506648607473464e+01,    -1.2053195221712004e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.6595053350283692e+01,     7.1991424272967197e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    8.6459208403398726e+00,     1.6570641800522219e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.8535661216503883e+00,    -1.0294446713105998e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7836174320660252e+01,     2.3899831648319341e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.1277359924348458e+00,     5.3291401103222033e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7703218591676332e+01,     2.3296818572541608e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9634843220645415e+01,     6.2107832540756754e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7360103324292854e+01,    -9.9858960127492082e+01,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    9.7917872967250297e+00,     1.4275829838837384e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9509229720320739e+01,     2.2582015519762710e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8175162373617518e+01,     3.7990305849353263e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0335292056497318e+02,     2.3895718588518568e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7670610859364842e+01,     1.8780222052255624e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.2151459212432680e-01,     7.3325411687048909e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.2036912963790561e+01,    -4.0844008073217719e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9068972604819095e+01,    -1.2098003913505472e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1893909682452769e+01,    -4.8632660401781118e+02,    -3.7347750399898598e-04,    -4.3625891220726239e-07,     2.0095090749677269e-09, } , 
      {    5.0816446390976829e+01,     2.4975257562815663e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5726806976323013e+00,     6.3570276206604613e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1997721053296662e+01,     1.5750002556174946e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -2.6412211595328827e+01,     3.9462585939837368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0588984119467893e+01,     2.1464153569149294e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4223708316532093e+01,     1.6664861016019309e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1995128515761060e+00,     1.1847831208606640e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6514864310589434e+00,     8.7973803726702190e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3326859007080200e+01,     1.3743151166958736e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1210026980400560e+01,     1.1205383863896885e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0533552661026318e+01,     1.6714049828363637e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4263863629069213e+01,     8.3515856458142571e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3314654117193646e+01,     1.4999766337049823e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8743655718350727e+01,     4.3876650164355324e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.9793807780193724e+00,     3.8299219129868667e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    6.3145582086602250e+00,    -7.3411442974895599e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -8.7314962219084542e+00,     3.0551255087761301e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4074711270735108e+01,     3.8299923761784827e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.1407565989384754e+01,    -8.3598430199119066e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.6167844278426822e+00,     1.6649904682881195e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.9523501369519458e-01,     1.0522684895550640e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2587491399224700e+00,     1.7520977049848352e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.3918908712492497e+00,     2.7043502879377538e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2927774823777938e+00,     8.4482435358427501e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7790952595405241e+00,     6.7228261184502117e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1538398650634129e+01,     7.2780525893923877e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6539628625624715e+01,     2.2459397390317299e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1623063865682266e+01,     6.5697557720888035e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0793442176653269e+01,     7.7047930668486533e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6310142505309217e+00,     4.1714254166765403e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2186510555578739e+01,     1.4396947327403179e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.5198353525368745e+01,     3.0576645701016110e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5322932192910985e+01,     8.3283733255206898e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1870450876378058e+00,     8.3290095239295606e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7672868394527697e+01,     1.0613124695253751e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4610116685072228e+01,     1.3975646118314478e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0512991015135221e+01,     1.3842477961827826e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.6855538686625069e+00,     7.1430695600464360e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6118745007072452e+01,     8.2975398295383457e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.2511774602641857e+01,     8.1732582702244290e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6136282222134053e+00,     1.2556320886980491e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1780769162568829e+01,    -8.9443797714482898e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -6.1295731950268461e+01,     1.8365773000228379e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.4663993751789661e+01,     9.6632054301336102e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0836497259203384e+01,     2.3314483751234009e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -6.8290233281470151e+01,    -1.7429294307611103e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7095454967794169e+01,    -1.2597436752843462e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    8.5197885744330897e+00,    -1.0086903648439107e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0237875139772534e+01,     4.6360695429875423e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5055321296493485e+01,    -7.8735373050592052e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3898082706918338e+00,     3.3917336134424215e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    2.4686457932941522e+01,    -2.8601792765262653e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8847846242629004e+01,    -8.8689932809695733e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5009886421554990e+01,    -7.0851941658334945e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.5207006060488686e+00,    -1.8127659882172111e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7673410202079580e+01,     1.5574234517847188e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.4329121455958457e+01,     7.3560647356516765e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.6005575678399397e+00,    -1.0803546425490846e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.2523582493288473e+01,     8.5675808361126001e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1384414739321757e+01,     3.2739427211066155e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.0201247362687857e+01,    -9.5208722450035439e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1670907443918896e+01,     2.4100182850448033e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    4.0458056166023546e+01,     1.3585201608094130e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.5645980862765732e+01,    -1.6666019347833681e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -2.0220832975388465e+01,     2.3313779119317864e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.8256486082706985e+01,     5.2332255328463034e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4167742912171977e+00,     1.1692726686582393e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1286142939542529e+01,     2.4886215314047884e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2511682580848738e+01,     8.9363804472596930e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6410732727936661e+01,     1.0287237317590334e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3450665479712129e+01,     1.8358335427847538e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1158025606931137e+01,     9.3096795963609657e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4290413865768040e+01,     3.7724753425894432e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8569353310717300e+00,     1.6022516308804981e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1494543138391595e+01,     1.0529082888542620e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0466766825047209e+00,     1.6026445432088523e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.1386650501526418e+01,     7.9790881529649687e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.4607617604231287e-01,     7.7849921261241134e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.7156475578601516e-01,    -8.1191419588944045e+03,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {    4.9327720457713857e+00,    -1.7609603525130417e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.4367783693048239e+01,     1.9777240867873170e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.7781801818296241e+00,     6.0097054471120609e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6796290333260721e+01,     3.2409686605464933e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.3151345036774657e+01,     3.9414200005093044e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    8.4898173049190309e+00,     7.7441546731887429e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.7278506082516355e+01,     5.3909598871867365e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.0733088775900370e+01,     1.6186603903517933e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.2207561255389550e+01,    -8.9394564725845498e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.8416572967665701e+00,    -6.9658178551834817e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1910035648675199e+01,     7.3898552841521087e+03,    -6.2033966107973026e-04,    -6.6021927093496605e-07,     3.1168671558532067e-09, } , 
      {    6.2987737358529099e+00,    -1.7805768455853991e+04,    -6.4460982180701858e-04,    -9.0073048999659575e-07,     4.2627727725237056e-09, } , 
      {    5.0754368529834313e+00,    -1.7738254361579100e+04,    -7.0626842577052931e-04,    -9.7466457636580007e-07,     4.6104811446154644e-09, } , 
      {    3.6289747652505175e+00,    -8.8860057043582729e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -6.1506568880209036e+00,    -2.4324015271789453e+04,    -4.9043143539870856e-04,    -7.8758399436009874e-07,     3.6784316908191937e-09, } , 
      {    1.5021196231306265e+01,     2.2581084757274675e+04,     1.2228169438553973e-04,     3.5009229792451100e-07,    -1.5600431593478871e-09, } , 
      {    1.8933662346409022e+01,     3.3004231513575971e+04,    -1.6606529064859405e-04,     9.1630464468855841e-08,    -4.9658980542368677e-10, } , 
      {    8.9294283283284415e+00,    -1.0247414357933178e+04,    -1.1862370243951608e-04,    -2.1414772982204668e-07,     1.0654701788376160e-09, } , 
      {    1.8308521633719867e+01,     2.3695653296215809e+04,     4.8796341863296836e-04,     7.8811312001166160e-07,    -3.7343258601143123e-09, } , 
      {    1.3279164621563053e+01,     8.4369097288183511e+03,    -1.9299933286879084e-04,    -1.4826436247861120e-07,     5.9487608299502271e-10, } , 
      {    8.1573860495829802e+00,     7.1430451796809175e+03,    -3.2996128248106349e-05,     3.1929829037874036e-08,    -1.5509189658694057e-10, } , 
      {    9.4338159902515817e-01,     7.9099389353073657e+03,    -2.7769145906249971e-04,    -2.6092740191230583e-07,     1.2812122669313596e-09, } , 
      {    4.1633625586806531e+01,     3.8860007189904877e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9271981660408024e+01,     1.4920480179533362e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.6872965334332978e+01,     7.4427905252116580e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -7.0646869157697871e+00,     1.1432039387922978e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.6565193947504120e+01,     2.3814357816244315e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.2446432948670385e+01,     2.4081323813216255e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    3.8182404723034722e+01,     2.4080666548263358e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -1.6494157640105595e+01,    -8.3195178961772617e+03,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    3.8716109054188514e+01,     2.2774866098998165e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6184899867111390e+01,     8.4184202838056044e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4282458785449025e+01,     3.2921744729207516e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    6.5858498909471708e+00,     6.5574217909565959e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3491075084968077e+01,     7.2361041189035614e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4362676371444106e+01,     3.2392765924656149e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    1.5163776679527023e+01,     7.1441730874388604e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.4670056455637422e+01,     3.1064178672390066e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1322073389993218e+01,     9.8205932101687686e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1117211106632309e+01,    -7.6811961699713556e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.9051344496503617e+01,    -7.8756719263719219e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.8650694865839235e+00,    -8.4758840835268784e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.3747255160262998e+01,     8.0914269532057006e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6109810554168057e+01,     1.6248614136241626e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5594839965107052e+00,     1.7073814045651030e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.9708744926872892e+01,     1.0490587524668716e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6267991182717601e+01,     8.9619300419265892e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0100119964522683e+01,    -9.8231036683682760e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5986104038314188e+01,     4.1892444681612687e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4145554827163611e+01,    -1.2182398306746322e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0269009603330019e+01,     1.2933963976123996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6370363221977749e+02,    -8.7308453210534935e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9350851788510496e+01,     1.5664035399533313e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0601825990903780e+01,     1.3214170376782282e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.7929660196327024e-01,     5.4519989855533668e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5980968625668218e+01,     2.1908642066283883e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2711611007964972e+01,     1.7071067335446751e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.0734032241167553e-01,     7.2516021087785884e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5057638985244083e+01,     2.9577859051408632e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.9930184964069781e+01,     1.4915869396072170e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0646561561436911e+01,     2.4892774269647616e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.7592676468143154e+01,     7.3023291676380850e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.5529027019779349e+01,    -2.0858306338304283e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.7898239889661582e+01,     1.9033895487171834e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.9102467687171433e+01,     1.7443219073609020e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7483716503057067e+01,     6.7700945071062051e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.7239396537665455e+00,     7.1920216120141040e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3501565400950099e+01,     2.0980613516073114e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3482774204476630e+01,     3.7999850480661524e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -9.3541846868311573e+00,    -8.3003397022551562e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.0635117908846464e+01,     5.0186471085438079e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5037969605862720e+01,     1.5433721801508604e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.9392996131625651e+01,     7.2117599306217098e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0405621656305062e+01,     1.6727664809395774e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.8852734640011576e+01,     1.6250691679025327e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9040904787274954e+01,    -8.3210032185586660e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.2928912223978326e+00,    -4.5211604014559725e-01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.0856954985935445e+00,     6.9077829446938813e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3363060225455903e+00,     4.3677259666962855e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2613419258925926e+00,    -1.7597968910060199e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.5290020334176635e+00,     1.2043574183467715e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0268301031418012e+00,     3.1415379252277162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3375060985772489e+00,    -8.4111216611851887e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.9305785270015235e+01,     1.5754604955854415e+04,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {    1.7635412281222276e+01,     1.0857629045511997e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.1832715730050740e+00,    -8.5202072065685206e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.4963898643337998e+00,     6.1686127387456345e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7790520625904232e+01,    -8.1531514574520861e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.8262280803308144e+01,     2.5417792444159200e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.4265018604751427e+00,    -1.7161737507658330e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.8722149093547522e+01,     4.4172283535806491e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2603769435185798e+01,     6.2637276161326972e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.1650964106120671e+01,    -1.7983862074387001e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.7727351442085773e+01,     4.0345357060497066e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    2.1748476665508178e+01,     1.5540451355346835e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1869832743016016e+00,     1.5545057225021083e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0569277958276768e+01,     7.8348922839353190e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7530916921555416e+01,    -2.0741373371402915e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5333486376654873e+01,     9.1611340299364456e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.1207370626148110e+01,     6.5863075699457722e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3276707016446306e+01,    -3.6969987840120872e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6762239449273267e+01,     6.7224729828738818e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6635982783030666e+01,    -2.2069170498184612e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.0216289047416609e+00,     3.2530974074299925e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6111843468791719e+01,     1.4603953703360021e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2591973254251005e+01,     7.1964308391831755e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.6108639001196703e+01,    -2.4671583068332784e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.3245554429847218e+01,     6.2720299331622482e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8318979904316443e+01,     4.1889055005858754e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    1.4903872408750168e+01,     3.1311978727007423e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -1.0412160069238933e+01,     4.1790509985523272e-01,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    8.3797474161713801e+00,     1.0239583870547140e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3741803054192399e+01,     2.3867642945299758e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2276730987198659e+01,     3.4099653652565576e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    6.9449690767851905e+01,     2.3442212639353103e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1427365564076045e+01,     7.3265076943099430e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5675824939555365e+01,     8.2108082563964308e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0779684719687893e+01,     2.4158547441751933e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.9423356517093936e+01,     9.0148744039836092e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.7986966339863006e+01,     3.1518860457377476e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.6684723796388354e+00,     8.3426985876828094e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8713149334300780e+00,     2.1954157569150248e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1887333767051129e+02,     1.5542712264417334e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1114892489942378e+01,     3.1716117365253198e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.0458601189494740e+01,    -8.2327768472492280e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.9229266347295955e+00,     7.0966688942210931e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4280843481010012e+02,     1.5542796315950582e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1109821656450979e+01,     2.8352410735872378e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9780604594473758e+01,     1.8525032607820027e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3952164040092940e+00,     2.6367254876281447e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9689252625377591e+00,     1.2489885864648156e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2740359935855032e+01,     7.2163298543582805e+03,    -5.5838726002546712e-04,    -5.8641608425966146e-07,     2.7669771221964547e-09, } , 
      {    2.1124819887185655e+01,     2.3775885741031296e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2595854397256049e+01,     6.5334975286689742e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0954545729642220e+01,     5.1145235725442126e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6738634391159579e+01,     1.4891904800007360e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4722035943739771e+00,     1.4162186193255406e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.0016277042691961e+01,     2.3809312564936154e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.5796279192075406e-01,     5.6360651564511136e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2001867245920948e+01,     7.1577395328248122e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3869668670376159e+01,     1.4946085894371756e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6490796927002364e+01,     3.2657592459139869e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.3548389211155133e+01,     1.7737465415182833e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2910024983608487e+01,     8.3651330873899788e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9197228271598874e+01,     1.4508307824245972e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1187717174004725e+01,     2.3875248484518412e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0206696626052558e+01,     6.4633179324674825e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5119473093506498e+01,     1.4339819562770939e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7315665878476034e+00,     1.2669244513465742e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9626720554749426e+01,    -1.9381698168313051e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.7713468239712423e+01,     3.9807220974786032e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.2142885164386332e+01,     2.5319576666843568e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -8.7049026598526318e+00,     2.2637372449949056e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5746266812280693e+01,     2.4951129385165394e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.3606979317554043e+01,     3.2474120955277602e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    6.7834429231795905e+00,     4.2745182292314178e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6665802162314529e+00,    -1.7947512558021265e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.2821996640004757e+01,     8.9549214669869689e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9481493382640300e+00,    -3.0501934886000704e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.5124292152512169e-01,     8.3643465948650955e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7900095928474922e+01,    -8.6595792444451781e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3076454852586314e+01,    -3.3070082036087996e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2802444919719937e+01,     2.1938325082177662e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.8048693375084355e+01,     1.6173139418013201e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1677550003747523e+01,     6.5594468897867318e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4710621106476367e+01,     1.4957817866560996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3274028422890829e+01,     2.4087028655134753e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    4.5689650818899239e+01,     2.1502503185391441e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.2475275043579813e+01,     1.5860524198704559e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5475531226203074e-01,     3.9621448271084031e+04,    -2.6041883236082187e-04,     1.1799395370843885e-07,    -5.7702524325656986e-10, } , 
      {    2.1313190591733093e+00,    -1.3341674311032102e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5215766896166500e+01,     1.6610483510747832e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.0325597810740728e+01,     4.8700128879259807e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2776218715374178e+01,     1.3797188313555938e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.2635073298908370e+00,    -7.9260089498570244e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.1391142825301834e+00,     3.9565080200917146e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.2313599847058194e+00,     8.1071443483470284e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.0210989889174300e+01,    -8.2599993191524663e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.5550849514373333e-01,     6.6019646157551942e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0285083848744492e+00,     1.3906236061599338e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.4403092810035183e+00,     5.9471384141122580e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4598356906610590e+00,     7.1643994680242877e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4572443059297250e+01,     2.4127395846565149e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -2.5505337165351001e+01,    -1.1038118479065724e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.8637692037433666e+00,     2.2048879766801423e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.0508375214398971e+00,     2.3589758433731186e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    6.6735254283666272e+00,     7.7437550011021631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3620183358706877e+00,    -9.5764282063634728e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.8765389545413420e+00,    -9.3657677229792953e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1901998544042653e+00,     5.9567363499008688e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.4275269912714336e+01,    -1.1118678296202958e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5427110341070929e+01,     3.2830745924519491e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1954393182168115e+01,     8.3304166515507513e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6090280287441296e+01,     8.3269661978995009e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7999923435010331e+01,     2.5275253543801926e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    9.9816047976802054e+00,     3.2345252255652264e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -3.9357651726653700e+00,    -3.9639911244328396e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.3627746221552178e+01,     2.5115993936210274e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.7304155957050735e+01,     2.3853813688633425e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.6300529223324576e+00,     5.7287200356149178e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5593153877383294e+01,     2.1360516574566645e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2971422668467014e+01,     5.7985618476420314e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1478541495354177e+01,     1.2719030948223392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0605401590195598e+01,     1.6792810504675319e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -4.4525010195358670e+00,     8.3418456200384044e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1152616008583129e+01,     2.3871222057853054e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7493953075730449e+01,     2.3871669371965116e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1856821934319848e+01,     3.2809646224778923e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3441443127069581e+00,     1.5464757403292720e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8589952188074079e+01,     8.3155372294118479e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.2780352993572581e+00,     1.4508806497123049e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.2705792434160390e+01,     3.1554908873779885e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.9881010981792706e+01,     3.4622697307604802e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.2959000232851874e+01,    -2.8038349361568180e+01,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.6322682122504122e+00,     4.7538016224878511e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.1315409180817202e+00,     8.6541161698134256e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.0446685374498372e+00,     7.0820656606402354e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    5.2058273181291554e+01,     3.9626050670763492e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {   -6.8634051186921603e+00,     1.9470559051095643e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2042386667633135e+01,     1.1788327890949889e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2225703079060700e+01,     5.6469546935893513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2492092660107936e+01,     2.5007404182153230e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.4667741116252847e+01,     1.8065173418282044e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.2272140390746937e+00,     1.2029347274130187e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8116333346979587e+01,     1.3988719354767385e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8067101375052554e+00,     8.1179879731735027e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8113515422731989e+00,     1.3120194122213365e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.9897779415981702e+00,     2.3178703780050870e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1366570242677300e+01,     7.1868403935555962e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.7604532632707084e+00,     3.2454568579310115e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7412266719654443e+00,    -3.0797334077312307e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5068159191415639e+01,     3.4951283927485121e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.3701839907637225e+00,     7.2876505251809995e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9556891944288967e+01,     2.3261162338064743e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -8.6908770760771120e+01,     9.0348978450880677e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.2951388656427270e+00,     3.2135702789821542e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -1.6845608082049957e-01,     7.2163658002959555e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3376205620553410e+01,     1.6657776095712696e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.5290457330181848e+00,     1.6656989603187809e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8893664031320395e+01,    -5.4903009720779357e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9655027542834578e+00,    -2.3499138213795281e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0480995193810422e+01,     7.9159667799052522e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.7997630540368228e+01,     1.5513903894836909e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -4.5937706717632825e+00,    -1.1977213840477101e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.6835074488910621e+01,     1.6830849687938273e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.1136070992318256e+01,     7.0978153897809616e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.4708435817755145e+01,     7.0963491282435225e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1317711227000606e+01,     2.4768821342810144e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.1064088896441705e+01,     2.4964744366197527e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.8511305382974328e+00,     1.4497304163470762e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5597945215849261e+01,     2.3363794832710482e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.6669914497020404e+00,    -8.6567191938019059e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.0143188670483369e+01,     7.9514029391797285e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7311926268211863e+01,     3.2332134344452807e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -2.0744618192837549e+01,     6.2600638520580833e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9861076292078408e+01,     8.3501708184869658e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.5149967147574614e+01,    -8.3072120309632828e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    6.5071687594483025e+00,     1.1080171597130000e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9568954720965511e+01,     1.0635741349964728e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.4543790914004227e+00,     7.8426990829808246e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3970168626722966e+01,    -6.3060878488527669e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5781347645255556e+01,     1.7758893497517092e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6560000433026378e+01,    -9.9919977652411380e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -9.1548723701988379e+00,     1.4936988610269473e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.0655744619500851e+01,     1.6171055161678778e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5255843756303321e+01,     8.4586110867600237e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.8696208022128271e+00,     4.1500772012432408e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9561572046333229e-01,    -2.1539907979921495e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.4894889705194828e+01,     8.2204709820352473e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -8.2414927289268633e+01,     1.0291513015179155e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4744183564376927e+00,     4.5430981709362277e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4746796889163054e+00,     6.5797218060183413e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1383524691111587e+01,     8.3232959306324192e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.6724155466077857e+00,    -8.3340869188178331e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.0153470244303350e+01,     2.6621045390831423e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.1752597620821362e+01,     2.3187781199677200e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.4085602594556992e+00,     3.1049300177839496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.4637532905763777e+01,     1.7148972732035203e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9753679861680073e+01,     2.2721480084707258e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5061351343133314e+01,    -1.4246173746048385e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6068542148052412e+01,    -3.1972300158662605e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0513798168001429e+01,     1.6266663079596998e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.8724387491256238e+01,     3.6167022055005327e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.2429689226309499e+01,     1.0603032353519826e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1881957614657660e+01,     2.3853421506707884e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.1719948307032141e+01,     6.5726373692138341e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    8.3964032888404407e+00,     1.4886060771728156e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0423971705476031e+01,     2.8820203045432918e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.4336881463737390e+00,     1.4917291619196207e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.7259212495619636e+01,     8.6782946964852836e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.2485256372289326e+00,     3.1734237516739086e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4422047820633512e+01,     1.4830416300041798e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.4974239762323421e+01,     8.3425968314727525e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.9158901829987034e+00,     4.2314322318646909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4197413676186622e+01,     1.0327514268765484e+01,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.2086321249926080e+00,    -3.4123401757569809e+00,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.0366606655732920e-01,     6.6970889689283777e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.2709624380611444e+01,     2.5279236283740873e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.4418896352566314e+01,     8.1083252475346835e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    7.1167687742940409e+00,    -1.6847744645572748e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    3.5665895556394098e+01,     7.8444479932880768e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5537198825979399e+01,     2.0724826599098742e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -5.6518755226631079e-01,     1.4927564616337870e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2553379598162515e+01,     7.2128899281656895e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5010223407131527e+00,     2.2726164233333082e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -8.0196265116811531e+01,    -5.8054301112446410e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.4180725473692604e+01,     1.5859542536888830e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.0486822063289175e+01,     1.6943985903416022e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3986836242587811e+01,     3.3707786668593493e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -5.7009215076560125e+01,     3.1968948554140579e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9325540642388617e+01,     3.0756921960487598e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.0604987265992001e+01,     6.1583034574901476e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9529837390554312e+00,     6.5077583935220518e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.0982894496872273e+00,     7.1345023999055284e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9230989810165980e+01,    -2.5409782932901890e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2955028224861545e+01,     7.2152358027519749e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5916427869942609e+01,     3.6173708205651965e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5539576712363043e+00,    -1.3248893705638914e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.1056932100314839e+01,     8.3296396952188188e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -6.5768041244818633e+01,    -8.3277431542314334e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.7836294307312610e+01,     2.3855446206584602e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4529689506703322e+01,     9.3565095251626663e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4095617989810449e+01,     1.4897173708371560e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2735181708560448e+01,     6.0111281380458140e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5556521459852028e+01,     1.6946562119076800e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.0907294243833796e+01,     1.3383386949671343e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1943220631805683e+01,     1.3661973168243264e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0723309237695037e+01,     2.1521531893529558e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.8026919194683035e+00,     8.2337738570251076e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6676721971570359e+01,     4.8893554716888502e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.7354166644138203e+01,     8.7008366016163400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0165706103466690e+00,     6.5573693470030312e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7780549756420374e+00,    -4.9166327601865845e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1715812192853409e+01,     1.3991892778519057e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1413575452122529e+01,     1.4927065943460795e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -5.4461062195086534e+00,     5.0267893128407941e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1430815861738743e+01,     1.7190089230128298e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7681687843223290e+01,     4.9487573305238348e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -8.2448818713264149e+00,     7.9210338518525223e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.5073128557007713e+00,    -8.7025342004960075e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.9901072860331904e+01,     8.2016099926943971e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.6675084636535962e+00,     1.6944484576293100e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    9.1045508667472284e+00,     6.7336419453244998e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6568453942533594e+00,     2.3497569569539446e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.1925915026504917e+01,    -1.2795165804477676e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6774527483375380e+01,     2.3385477056646407e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0728490495648320e+00,     1.7874183117643428e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.0233426487878916e+01,     1.2888395899338502e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.9919706340166229e+01,    -8.2691881070683739e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3775434328114304e+01,     8.3260970704153551e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.6784439270228660e+01,    -8.3317506293245515e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.4149405060917003e+01,     2.0225316248533439e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7388933763936105e+00,    -2.0326677034708664e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.8847695093558812e+01,     8.4152267584259516e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.6040752776423631e+01,    -3.9114914213649627e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4090637256861854e+01,     2.3949442601800321e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.8633409911545421e+01,     1.7812526328403635e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0382812819924341e+01,     8.1398054751979134e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.5970743479766165e+00,    -2.8962619038689654e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.0942315959876527e+01,     1.1509277139800542e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.9091983578985332e+01,     1.5873642109904014e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6760476748355089e+01,    -2.1299517560523964e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.5323606866164791e+01,    -8.5482455559300888e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.9167640874552916e+01,     3.1730736846222244e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.7203227571048529e+01,     3.0983336871441035e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.8224475056596248e+00,     1.3562271561169538e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2729492961194325e+01,    -2.5213672683361474e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.2550006341534495e+01,     2.2983287302282297e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.0214739784008042e+01,     7.0809546430867786e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -7.3318493213390337e+00,     7.7653436706970315e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.3843095248895757e+01,     1.7588041231977208e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8361868428265719e+01,     2.3232828848608053e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0072983259999075e+00,     1.1299705716889404e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2320686001798556e+01,    -7.7894260884473551e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.4789518110910493e+01,     8.3701610583945494e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -1.2195810872952091e+01,    -6.3996692036028880e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.2829239101623742e+00,     1.2507154559378625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3704775734196765e+01,     9.5384072561137909e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.9254478963634817e+01,     7.0432105999189462e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.5627427899948657e+01,     3.1230623696385974e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.6118843185476084e+01,     2.2360399105142096e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0571999950155954e+01,     2.2720057393773506e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3854472944098998e+01,     1.6153028191242176e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    3.6979654236991152e+01,     9.5100555336438247e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7301951112814809e+01,     2.3576640522531728e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    5.6843982305258880e+00,    -8.5175773742523379e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.0904639830536720e+00,    -3.0202148307064322e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.3572495585189436e+01,     2.1499849669593630e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.7858647666458147e+01,     1.4898714220990827e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.1599505239662989e+01,     2.4080988467189083e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -6.5863528448100894e+01,     2.4081001894290530e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -8.8518629551797527e+00,     1.4914998994670899e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4712121351439423e+01,     7.2578336989679901e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.9074386248267199e+01,     1.3186929671865768e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7440138787939734e+00,     6.3578575161459992e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.6304338478946164e+01,     8.2646036492177318e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5607737320057410e+01,     3.2415720079859879e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    1.2301193453098406e+01,     2.9062513098751601e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4533949837619350e+01,     5.6323332634020517e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.1972905867469841e+01,     1.3908537261439073e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3855567155418335e+01,     1.4927953216409769e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3145215238058839e+01,    -5.5732099318387327e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.8091658137210338e+00,     8.4099940113930097e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.4317819073051012e+01,    -5.5730756608242018e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.9985899532371111e+01,     1.0785033020198684e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.2984681099659809e+01,     9.0057008754357448e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.2033751059822393e+01,     3.0493249411724781e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    8.9527441749214844e+00,     2.3276925187163564e+04,     5.7861276565536288e-05,     2.7681823690076927e-07,    -1.2660472949862531e-09, } , 
      {    2.2302153322216345e+00,    -1.6062862321704735e+04,    -2.7636214843026552e-04,    -4.6328242679230791e-07,     2.1436570205307352e-09, } , 
      {    1.0971028311936614e+01,     5.4144597422336110e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0276985935626453e+00,    -1.7091490870118114e+03,    -3.9907697535850842e-04,    -4.7441510638889000e-07,     2.2929699380388498e-09, } , 
      {    1.1339689749050088e+01,     5.9236662678647799e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.8580272999257485e+01,     2.4604528032869988e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5694235068206510e+01,     2.5790144267751617e+04,     5.2533513579719266e-05,     2.7805102112909463e-07,    -1.3749267514898329e-09, } , 
      {    1.4623773556294577e+01,     2.2422755912171066e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8345709705596899e+01,     3.0141732286641512e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6281111686644962e+00,     1.2240852178299627e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0128062242830396e+01,     1.6912780691573833e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4594067556016189e+01,     7.2598419035091729e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.4684652717465646e+01,     9.9747247684742833e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9833472793927335e+01,     2.6439120998897732e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -1.5442800489928505e+01,     1.5541626027637365e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.9378260429738276e+01,     1.5543882552730553e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2274349381178336e+01,    -1.6517697438465191e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.5613420630430568e+00,     2.3295395881607859e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.7992509299840677e+01,     1.8031711756756653e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9596278644428768e+01,     1.6578719448102107e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0668368794446996e+00,     8.5110184186526112e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.6195110731748308e+00,     1.4924755986393904e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9679448813025420e+01,     5.6262465025392921e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0612214276938488e+01,     3.2221467047612063e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.6910694188503150e+01,     2.1107181015858718e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.9572617737146585e-01,    -2.7998727168110236e+00,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.0949571459014367e+01,     5.3500758051778041e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3563879992419885e+00,     1.5697858812208207e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3186066018133573e-02,     2.1639887396536760e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -9.0871378748134113e+00,     1.5330903624513503e+04,     5.1879272061472372e-04,     8.2508016319626376e-07,    -3.9081800461601917e-09, } , 
      {    1.9020017447177381e+01,     2.5870331990876984e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.6018644421407878e+01,     2.4569617354800295e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.2451210623321934e+01,    -1.8779228782940027e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5546196210013761e+01,     2.3712186107317437e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.3218067066839019e+01,     3.0552802199689871e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.9738293238300173e+00,     8.1552093268690551e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1873385368582190e+01,     7.1088360165033264e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0329844962741067e+01,     9.4482357567792897e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.3936832755523241e+01,     1.3562659702963074e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.0395556390759836e+01,     2.5287723684706025e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4379208817988442e+01,    -2.2517990236636433e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.9466488276144752e+01,     2.0995392929146205e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2402682522747586e+01,     1.0124683992990676e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2413475662049560e+01,     1.7195020262284219e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5428678308544983e+00,     6.5430954644575859e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0416102889004912e+02,     8.3927792002325205e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.2451283326380000e+00,     1.2509920221155146e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.2417643398321552e+00,     5.3356990659219309e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8620379961982067e+01,    -1.4281391617831650e+02,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.5786403472707566e-01,     6.3482086957107067e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4926593304815190e+01,     1.4862188953394101e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.8073108830737397e+01,     9.3066022815308279e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0807822285645841e+01,     7.1179246309269001e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8589405276567570e+00,     4.1571985073390141e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9419404306642809e+00,     7.3008133901729880e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1485001850733253e+01,     1.7796959061811233e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4604393660519186e+00,     7.8585797858462802e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6892092580002867e+01,     1.8941816119978877e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1353699683293936e+01,     7.9220002543002001e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1204578464664301e+01,     1.5735443009320497e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.3225372462470339e+00,     8.4236089924251446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0943122720289960e+01,     2.5807010644242862e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.8510861933307083e+01,     2.2750258200043067e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    1.0588635976616349e+01,     3.3890104186734854e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    6.7151213056487435e-01,     8.3258379353529090e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6060785796410070e+01,     2.1066850108542145e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    8.0969581939732187e+00,     8.3471677489664762e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.3858490494694191e+00,    -8.3102151004837760e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.5642027918284050e+01,     7.1852124701117864e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    4.7495477906289139e+01,     9.5856589107742875e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4766461385632120e+01,     2.3946696776081473e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2413696879754374e+01,     3.1206040532585041e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -1.1576105802037578e+01,     2.2690835693901536e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.4396601700897200e+00,     2.3400768748025512e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5158019310104958e+01,     9.8140342545690401e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5077623873921915e+01,    -1.3234666796301387e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.7434665698749164e+01,     2.3245215672647239e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0612226712545144e+01,     8.3253857249484790e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8790202715539884e+01,     2.5190692707850179e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    3.5511281490382238e+01,    -2.6676684301578462e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.7808537301127917e+01,     7.2129346029122398e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4255534590438952e+01,     7.3303103411367038e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4481118732661109e+01,     9.2420682902920089e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.0214939526561430e+01,     6.4264956728216403e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6649868033111055e+01,    -1.8225659481076611e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1628629453328072e+01,     7.7029419587033613e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.3548651031298832e+01,     2.6365458669101827e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8666451684627219e+01,    -8.1478563144500995e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.9534520989650009e+01,     1.1890523150851577e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5428093084457650e+01,     9.9184131533965847e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4456925408336680e+01,    -7.3734007372089582e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2693368097810666e+01,    -4.8369114190456639e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.9465488228988264e+00,     1.8439016194052921e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.4160188808366208e+00,    -8.8548273008437882e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.7092352054777301e+01,    -9.7964182528412675e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.7168371240601648e+00,    -2.2679543188234220e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.1161778579520849e+00,    -7.0601123486848877e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2961056677423617e+01,     1.4486994882215273e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2682450554804021e+01,     9.5876077946559471e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4819507828254331e+01,     1.6013533236116278e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2553487115383653e+01,     1.6659649838349698e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.5743673245130685e+00,     8.8716793778592619e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6513585079579109e+01,     1.5529600094870681e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5694314640530465e+01,    -6.9300896438092468e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.2311061953716642e+00,     2.0334256987672197e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.4483691931073686e+01,     1.3344664026206201e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2363303891043298e+01,    -7.5583972288547902e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.5958165389398555e+00,     2.4750787658822825e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.2115938703478744e+00,     4.2276294622553651e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9019358594565421e+01,     3.2178807231656359e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.1146741802619164e+01,     5.6337456235579793e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.3847197798351871e+01,     1.6729479682255074e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1906545262451234e+01,     7.7018065306133349e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2194738110516449e+01,     8.4353409646143846e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3323925056491618e+01,     3.4227046505595630e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0690948189941832e+01,     1.5961863710045043e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0022630614776253e+02,     3.1069966552079488e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2962730925627614e+01,     2.1666100363145288e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.1678465043194368e+00,     2.5118071478993978e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.3674763919064578e+01,     1.7415610096530694e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.7096580457419663e+00,     1.5555908485497237e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2031323649994505e+01,     1.6700042665405956e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.7525655589126394e-01,     6.2829706284173358e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6301124566474883e-01,     8.3572668042511286e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.3237093078555169e+01,     8.9853324992273629e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.1918992505294284e+01,     6.9042744955965873e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3956368912509863e+01,     8.5095265350001519e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.2998947044474797e+01,     1.8760625996971685e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5312484826534632e+01,     2.9871553226232536e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -4.0811922263129787e+01,     1.0237592330985608e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4139106197720881e+01,     1.2228337425881220e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.9629329971456301e+01,     1.3251140364622403e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.3708650847483632e+01,     1.2225119694152036e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.3786844035826711e+00,     2.2561861447606807e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.7380060830686290e+01,     9.5102531490421825e+01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.8963150419494774e+01,    -1.0682278934409514e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -3.1574532486497393e+01,     1.3657479219089253e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4062155466682221e+01,     1.5571604685531001e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -5.4783490743499598e+01,    -3.2969381517490547e+01,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.6500486722391567e+00,     2.0727277957365186e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2669941423126890e+01,     1.7776927181504416e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.2522510531427358e+00,     8.3633171501937832e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1352973428493899e+01,     2.3398242651887791e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6906423322313763e+01,     1.3831979292447377e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3075915133078672e+01,     8.3129342054233675e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.7393419689562037e+01,     1.6608931449168911e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.4650388875486954e+01,     3.1678824670487165e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5631742264575015e+01,     2.1083770789350925e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1986500391221782e+01,     3.0770039871687055e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9368294189547179e+01,     1.7699374920492293e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.1394704876820853e+01,     2.3532317399490086e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -2.7836479860607618e+01,     1.8485217705922114e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7871650409562921e+01,     1.4755187097546768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.5769719090590428e+01,     1.6848692916066437e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.5537594200908195e+01,     2.2128516284147972e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.5802935182826431e+01,     1.2323422964816036e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3482378508193067e+00,     1.4895259035430188e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.2750209985363243e+00,    -8.3559213867893941e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3987872080080704e+01,    -5.1749937773523448e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4164353005233609e+01,     1.4286150379686995e+04,    -6.0860358237563932e-05,     6.2714140310022456e-08,    -1.9984989562697296e-10, } , 
      {    2.7885006800169229e+01,     6.6011743914524786e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0176427648489259e+00,     2.7330978444955381e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0516830060729378e+01,    -8.1867048139003327e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1120090440384738e+01,    -1.5594646404495707e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.1741575474742199e+00,     6.9984799252331632e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.5742396909268422e+01,     1.8660019341802195e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4684836064991778e+01,     1.7409643361333263e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    5.2237635697831330e+01,     2.5368278532508652e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7009543503290253e+01,     9.3706834957671672e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.7969665972001522e+01,     2.2852418360010288e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0106140643354603e+01,     1.8472415826566910e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1699430275709117e+01,     5.5060716101569797e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.1569929207599724e+01,    -1.9098257941868546e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.1620934350426253e+01,     4.5711365202977959e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4758755804354701e+01,     2.1420019892223398e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.7852189876402598e+00,     8.1179974489596225e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6472464449696798e+01,     2.3307127853797097e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.7601133813526246e+00,     4.9672707358159969e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.7033221181349546e+01,     7.2151911280054255e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1066645597790341e+00,    -3.9039113737680327e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6274324309710000e+00,     2.3964745719436847e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.4298269145368387e+01,     7.6363904111098127e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0090851159733926e+01,     3.2410015237941381e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    4.0537932078602068e+01,     3.2409357972988484e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.2629194659091301e+01,     1.0438412716331040e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9589208739978190e+00,    -1.6533429922612788e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    7.2983019265876861e+00,    -8.8925646599579995e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -5.1137501509952266e-01,     1.1919141006906548e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5443772621558729e+01,     3.1571829554588669e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.8830547660239811e+00,    -2.3269230140438708e+00,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    5.2064048456941912e+00,     2.5273176001018222e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.4329771236279456e+01,     8.3363796308915862e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.7476366210222558e-01,     7.2544032485615244e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9330657627308439e+00,     8.2220418848358677e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2045586770086176e+00,     1.0170954363906052e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5346544129572859e+00,     1.4540570559768776e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9132511474411075e+01,    -9.6697085335471684e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5652894887513675e+01,     1.4942832638548673e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.5686257460665743e+01,     3.1085410528794077e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5885189860003765e+01,     6.1700354296793876e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0923149575958291e+01,     1.8720703475230821e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7452503386341082e+01,     3.0454571163006189e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2041559265054815e+02,     3.1085606631941759e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.0505564901020552e+00,     1.4264809212115020e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5748952373899712e+01,    -1.7584781726002666e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.4010689032620903e+01,     1.4888138314511858e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3021469245583969e+01,     1.1337562948599225e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4132543192523983e+01,     2.5672257253433861e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.5032453134439780e+00,     7.0033594139072093e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8942787316280654e+01,     3.9571934359433479e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.6424271985464632e+01,    -5.2958573556963138e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.2976866513977463e+01,     2.3953929008077132e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2921357164018001e+01,     3.5835893930622315e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4331072504491832e+01,     2.5802945223110264e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3333061436113791e+01,     6.6141412138235482e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4397296375462538e+01,     8.2184687537634945e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.2101098705835074e+01,     2.6083782152632888e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.5917479308353002e+00,     3.1261188141885268e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3938575504851958e+01,    -1.9498557778263739e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.5946718394608059e+01,     7.9277487837749618e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1599954973238091e-01,     4.6801690089731510e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.2071152565774284e+01,     2.2743699244443334e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -7.2073258510736480e+01,     8.3256322201257008e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7720017359152401e+01,     1.5949445460608884e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.1862161902368030e+01,     1.6657177134223042e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1346695120043876e+01,     2.1644489796216229e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {   -1.9013799411519776e+02,     6.5862632389513255e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.7321169145532003e+00,     6.6033873066889473e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0428617663832760e+01,     1.4929865816177604e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.2488378830657965e+01,     6.9751734540713587e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5757494613059826e+01,     7.7490857053446580e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.5426458105779588e+00,    -9.5633535412232300e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0898913110069458e+01,     1.4423716503748594e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9437641869222695e+01,     2.5200554772162021e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.6657988529576393e+01,     3.2435423754986765e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.1453651615611689e+01,     1.5463193824630654e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.9869752848243710e+01,    -8.1876712163480115e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1684414355829395e+01,    -2.7562699845524776e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.5275374508555171e+01,     1.7981062201670196e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.0231442894097373e+00,     3.7322745552137349e+00,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.7645404407128251e+00,     1.3757738365572231e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1248515380570639e+01,     6.6037794886144929e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.1560508557148204e+00,    -1.0071627569036962e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.3794982391014976e+01,    -4.7393772291052016e-01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.3620610871954579e+00,    -5.9992165163528455e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4748872519500690e+01,     8.9923360807062036e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8664164443088296e+01,     1.5922657348677491e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.3195979205090964e+00,     2.4379096597107688e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.3972784807372790e+01,     1.2041648190289770e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.2682342181404046e+01,     7.1050303767834775e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    4.2216373313226271e+01,     1.5335506024192971e+04,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.7954655261152119e+00,     6.7096740100124680e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1439652558371378e+00,    -1.7455659766280214e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1995276284527861e+01,     1.6816290141083828e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.8088311641070316e+00,     6.2118791350436495e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2923850740171630e+01,     1.6163583708660444e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.9002746378422358e+01,     1.0959635502122610e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1305523992090020e+01,    -1.8050678352762418e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1918741788874918e+01,     7.7533282017366037e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.9519176917977532e+01,     2.2434499707170190e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0006070553427680e+01,     1.7583110199821283e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7538234296675732e+01,     1.4218576265086944e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.4670769991488061e+01,     2.3938959809183973e+04,     2.4466025454456515e-05,     2.4042395259902998e-07,    -1.1473600577138278e-09, } , 
      {    3.4009360027693791e+00,     7.1058445613656077e+03,     1.2681121164540989e-04,     2.1221128701695900e-07,    -9.0360543512557530e-10, } , 
      {    8.0689203381899262e+00,     2.4391529366064344e+04,     4.2877283143519788e-04,     7.1364990799089442e-07,    -3.3307233187274341e-09, } , 
      {   -2.9228126964024286e+00,    -2.5580619182286417e+04,    -4.8510367241289161e-04,    -7.8881677858842410e-07,     3.7873111473227731e-09, } , 
      {    8.4700285716075463e+00,     1.6762127572203033e+04,     8.5725506554993870e-05,     2.4603392562862088e-07,    -1.2212892959462209e-09, } , 
      {    1.5233626668360607e+01,     2.4395002947404784e+04,     3.0236074265276139e-04,     5.6976273431354889e-07,    -2.7008970174612251e-09, } , 
      {    1.3797874183735427e+01,     2.2648598851549566e+04,     6.0623090422028940e-05,     2.7615821155530678e-07,    -1.2123347872561279e-09, } , 
      {    1.7946548353651146e+01,     1.4846938240660586e+04,    -1.8656357669616559e-06,     1.3726461879338937e-07,    -6.0199799597052197e-10, } , 
      {    3.1017699693401113e+01,     3.8265814398689297e+04,    -3.2004781232973577e-04,    -2.1603297630240923e-08,     8.6296835237497271e-11, } , 
      {    1.6554539956464673e+01,     2.3347888482413382e+04,    -1.3011148381682680e-04,     5.8953343622919576e-08,    -2.8923984214994916e-10, } , 
      {    2.3719231800402557e+01,     2.3351362063753826e+04,    -2.5652357259926324e-04,    -8.4933830054426053e-08,     3.4058645911625999e-10, } , 
      {    2.8662142852221184e+01,     2.9937122973964171e+04,    -4.7236056297108399e-04,    -2.7201441206012735e-07,     1.2726359129125317e-09, } , 
      {    1.6133875429787953e+01,     3.2933243837960632e+04,     2.2005402097353173e-05,     3.0945172451540569e-07,    -1.4741244787816550e-09, } , 
      {    1.3247837324204053e+01,     3.4118860072842261e+04,     2.0474833185748340e-04,     5.2850576879028096e-07,    -2.5605386086432027e-09, } , 
      {    1.0192495685044166e+01,     3.2862256162345293e+04,     2.1007609484330041e-04,     5.2727298456195560e-07,    -2.4516591521396233e-09, } , 
      {    2.6454380282709105e+00,     2.4330651296816723e+02,    -4.6349739317851188e-04,    -5.4768916741263173e-07,     2.5869658024004840e-09, } , 
      {    7.7027477804268685e-01,    -4.5250953655520607e+02,    -3.9917490772209256e-04,    -4.7437147315759014e-07,     2.2936971585605145e-09, } , 
      {    1.5996095535742647e+01,     1.4738659916242199e+04,     3.3831454948452867e-04,     5.3708561346761250e-07,    -2.4935470541874871e-09, } , 
      {    3.0591553985418827e+00,     7.1768322369809430e+03,    -6.1259481100537342e-05,    -5.6099730295908842e-09,     7.3929238232393120e-11, } , 
      {    1.2661812191709100e+01,     8.3321650060655702e+03,     2.5900661858911786e-05,     1.0652394075254079e-07,    -5.5651277640882531e-10, } , 
      {    9.5398034075805906e+00,     6.3762114443796290e+03,     2.1683110082493579e-04,     3.2364154222232824e-07,    -1.4810621625583325e-09, } , 
      {    1.0196464443674849e+01,    -6.7574114600050507e+01,     5.6526705704862013e-05,     7.5079604134929848e-08,    -4.5804226963866738e-10, } , 
      {    3.7407115951294287e+01,     2.2301663035976871e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6959287715795345e+01,     2.6119830569035301e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.5498229848159767e+01,     3.1101050608656347e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    7.2324315538433890e+00,    -1.3707083803692240e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.8348484541660625e+01,    -1.7452373441515701e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0868278828409466e+01,     1.7492938700486782e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.0718371743044264e+01,     4.6401427510538394e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.9318641873291995e+01,     2.4205506958380814e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.5482240337127195e+01,    -6.9551812409339675e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -9.9554682425247734e+01,     3.8083933419486129e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8693624095123987e+01,     5.7839808568797253e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1710419966722609e+01,     6.6120636710398476e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0149106939061241e+00,     1.5558963625525863e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5901110374524151e+01,     2.6558158152947326e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.3015793309856907e+01,     1.5543147536446400e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.6205704531882441e-01,     6.6129852424775590e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6560186661149476e+01,     8.2838094508799441e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.2755134042829585e+01,     1.0213614179861755e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.9869851431393428e+00,     1.7112536969116172e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.3395686999812916e+01,    -5.7960571938046633e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -6.1585866905138609e-01,    -1.0720598670386014e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7652197525995717e+01,     2.2725638752128307e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.5161046413377438e+01,     8.4697116331022407e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.3497586529670937e+01,     3.5660314860278413e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.8152594173865594e+01,     3.1570799535161459e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.6340062828161788e+00,     6.1801150723979226e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.9351327142355110e-02,     6.0507476998955099e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.5190217363911671e+00,    -8.3662306680448819e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -4.1424144153314373e+01,     2.9424634157381552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3428492475094576e+01,     1.5072642651305116e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6490409774157484e+01,    -6.3694408342897123e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7556534322346202e+01,     1.6778039552297192e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.3140599801588415e+00,    -8.2577037491097908e+03,    -3.4038344338729550e-04,    -4.6823237447643627e-07,     2.1638737510330028e-09, } , 
      {    1.3699772412882428e+01,     9.9894060061951677e+00,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.0221914769700263e+01,    -1.1699854894526352e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.4326865283880563e+00,     6.9853257299198840e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.5394893174054083e+01,     8.4575601243504643e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.1936226641512739e+01,     9.5161281622631159e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1968518734597133e+01,     1.0834617047114643e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2887210560283521e+01,     2.3917846380734427e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    1.0304952452961983e+01,     6.5857658026170984e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3464885483468461e+01,     8.3315449140973433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9914509596702384e+01,     1.4192839115565110e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.1012543291168555e+01,     2.4109226078027859e+01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.2068048127947719e+01,    -8.7670555032649827e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.0710393621696941e+00,    -1.1910105941437159e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.9203153670791188e+01,     7.7664461129360589e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.9024865274670567e+01,     7.7763081772479018e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.9012743173177054e+00,     8.8723907233261380e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7798283829551416e+01,     3.2422305843787315e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -7.7085250514093431e+01,     2.1947111249988748e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1673830608322142e+01,    -8.3857793233898956e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.7775708967476461e+01,     1.6801999292591230e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1178767489833761e+00,    -6.8782616207034164e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.1141928850471609e+02,     1.6657037247042903e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3026628475449309e+02,     1.6657728451857602e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.4423477112101928e+00,    -1.9923005120404705e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.8269674058005529e+00,     1.6614723033494549e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0165840139553872e+01,     4.7806859861285175e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.9115585879285899e+01,     9.5830053949764788e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.9198754761709580e+00,     6.6813841935801838e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0765217913898306e+00,     2.2333143263640100e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.0845114798234597e+01,     3.0995337081033413e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    9.8943437119819500e+00,     8.0842160214566593e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1856188759598844e+01,     1.6020438766021281e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.1254593820136165e-01,     1.5542361043921517e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3069518858677267e+01,     8.6143012679155127e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4225277387606585e+01,     8.6376011778062621e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1692401956129181e+00,     3.5191397838815128e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7339313079786134e-01,     2.3872432376818651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0286325636968034e+01,     1.7302551181189978e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.4212629194122236e+01,     1.8819278949393840e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1142402621900978e+00,     7.8773153395143618e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.8142742895040286e+01,     6.3087819356063301e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.4021914302290845e+00,     3.2198059596850515e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.3770147574902850e+01,     6.5917887548082354e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.0801348635429804e+01,     1.2289656882753965e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1091588666800074e+01,     7.1763255063446213e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7926014509791441e+00,     1.5819594616669775e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0346773209783890e+01,     6.4404006919313360e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1053861767349435e+01,     2.3875854942078153e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4871693054154811e+01,     9.1857266695234975e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3999043449660569e+00,     1.6825524304151739e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6349734329673417e+01,     9.6317072606415568e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.4735053733452936e+00,     1.9941236017979209e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0369197165798235e+01,     1.8832037163745910e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -6.3501663917203146e+01,     1.9288326926847738e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4239460222002506e+01,     1.5527833852021846e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -4.0039453482306531e-01,     1.0906684747196417e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3426290458504626e+01,    -8.1065227205720257e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.9759857481640871e+01,     3.2202214682417911e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.3657998649136600e+01,     9.0460165233842708e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2534046639996035e+01,     2.8745431705565363e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9954009162326319e+01,     1.5684740901008752e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4706930204019757e+01,     1.5447836722483940e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0078678539114190e+00,     7.3898908379011873e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0518929927390573e+01,     2.5132303401821728e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1031980935631083e+01,    -4.8559870042120721e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.6296963024743012e-01,     7.1353546331007183e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5175422117651800e+01,     2.5108873215291940e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.4938646517476940e+01,     1.2638804180148532e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2764607211003614e+01,     9.5147054713293655e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8274982599851888e+00,     4.9426780962289431e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5791273193699165e+01,     2.3707450564803048e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -9.6975575228141224e+00,     7.0322613527361273e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9100437978273163e+01,     8.3311212834669022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.4964550872999993e+01,     8.3262615659833500e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.4495840724254325e+00,     1.4899027594099225e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.7919971354333306e+00,    -8.9209505933682594e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3176428725947734e+01,     1.5556434325677060e+00,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -2.4972240583798051e+01,     8.3643875251694608e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8270099396746307e+01,     1.6054814399551631e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.0896681845431413e+00,    -9.7490893734094641e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.5022804400194616e+00,     9.4999869629048062e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4861851771622030e+00,     8.2440831382805954e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9434358275313159e+01,     5.1543384744421246e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8512723784916076e+01,     2.5821630125211908e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    4.9094339380179889e+01,     8.3283961618474368e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    4.4383230235676081e+01,    -8.3289866876028136e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    5.3584460485262380e+00,     8.4132997111696568e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2358131360491971e+00,     1.7130585912471546e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.3645958592745302e+01,     7.2673842786280993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3824361671618796e+01,     2.3967005688786874e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.8866234784514333e+01,    -7.5608644720996381e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.4105133855070932e+01,     1.2566218836418134e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8799128723481946e+01,    -3.6236804738328465e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.3752465782279810e+01,     3.2992722795681475e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2842127498068107e+01,     7.9202513439929489e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.2992523066348074e+00,     8.9415798987314811e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9672689417947236e+01,     2.9278138086243409e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.1009300297849620e+01,     1.4774761294153357e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.9406663918309000e+01,    -3.5694264361586647e+00,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    1.5331513973853445e+01,     1.8313508534933335e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2690467472062313e+01,     3.1905890798060398e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.5420589975586640e+00,    -1.5982674598579368e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -5.3198817776234677e+01,    -1.8401259313449714e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2331311897090558e+00,     1.2727173152520378e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8910008823240975e+01,     3.8785540104089421e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.4400925686997617e+01,     1.6781335776287720e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    4.5137343580948070e+01,     3.0242584457544384e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.1701307207125708e+01,     8.5351276447306336e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8238171524515423e+01,     1.4498726854404513e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2887234243328094e+00,     7.2626227355009532e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5895273268954590e+01,    -1.6668199560809975e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {    1.8004417103892229e+02,    -2.7168665122668429e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6367947156273647e+01,    -2.5189590265611154e+04,     6.6570736180512717e-06,    -2.0358780910832745e-07,     9.7132421010295529e-10, } , 
      {    2.6484675872373462e+01,     8.3255629195318670e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    3.2773836474777895e+01,     3.7148894470158048e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.1444264103889026e+01,     9.2321273353488414e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.8578766621818836e+00,     2.4001365376943981e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.6662043837122091e-01,    -2.1566206303083336e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3432457636302662e+01,     1.4301558231132061e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5307627608644312e+00,    -5.2898079017645387e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.5704418933393526e+01,     1.0625574258650882e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.2445447571446692e+01,     1.8325958098330466e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.1050110079218811e+00,     2.2751886123486871e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.3143377000243794e-01,     1.4250090703690461e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6246557037701670e+01,     1.1739397123774006e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.9563521623639136e+01,     2.2163454459607234e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.4613312777711032e+00,     3.7570562315502530e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8540126211897707e+00,     1.4169793567477027e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9448353766925592e+01,     1.3658183851005406e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4337605772607830e+01,     1.6086950658617649e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    6.1409463674436935e+01,     1.6228149773894271e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.6204081910733992e+00,     1.1624747567787048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8592546112934620e+01,     2.2026337861670367e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.0627095996189766e+00,     7.2764196724878020e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.7908061166697429e+01,     2.0269093659614227e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0746397533451679e-01,     8.3422539618036953e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0651616080580411e+00,     3.1634186361617634e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.6224211269205941e+00,     1.5541590828676937e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.5026384044305416e+01,     3.1985520601856217e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -9.0539477944154115e+01,     7.1499750899514383e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4718241564835623e+01,     4.6986770028925093e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1568065155769389e+01,     2.4565014955120823e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -1.0862762741783083e+01,     9.0191321597436036e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4576226133650929e+01,     1.6657935138982735e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7139848330043232e+01,     1.6656830559917769e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.8446342112472767e+00,     7.2232516533747448e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4496366999731389e+00,     8.4606886295437234e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.1037527749880196e+00,     1.6723350208249876e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2452279160649038e+00,     8.3151288876465569e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8552572074365973e+01,     1.2487628781407831e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.7566808392701269e+01,     6.8969798962683308e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.6720379627052409e+01,     8.3508810138981080e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0188458817368181e+01,    -2.4973455035853000e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    1.9093525432102247e+01,     1.9173243021948609e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.4460285868293351e+01,     2.4983996731391675e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.3914077902723108e+01,     2.6231187654570931e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6490711743563132e+01,     3.5344304897178110e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.1735243595821075e+01,     2.4988151816959082e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.4323450826026772e+01,     2.9908523214072655e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.7695878156436997e+00,     9.0797979508872359e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.5847688158302759e+01,     1.6029403896881158e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.3880696730714833e+01,     1.3658542880514206e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.7010542921866065e+00,     1.4115785527430060e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.2292677208586520e+01,     8.3401661531127247e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.8347311062579227e+01,     2.1484202477227216e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9584701702587012e+01,     7.1361640301414345e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7547243978743388e+01,     3.8316492126635530e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.0226673805363978e+01,    -1.5310597554670712e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0751508922139992e+01,     5.9068055211705987e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.3695270794346342e+01,     3.7906550476133758e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.3056496458167846e+01,     2.5294950534875188e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -2.0156722834527848e+01,     8.6392291012500700e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.1002131000170827e+00,    -1.6550221580906591e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.6553821378665262e+01,     7.7007428691883842e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0978922059690925e+02,    -1.0390267740530726e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.5744493434067934e+01,     1.0937093596991554e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8846315180810365e+00,     1.5258327173432917e+00,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {    1.9002621893412993e+01,     2.4474016150432795e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.4709961589431479e+00,     5.2335801364444133e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4086806152569871e+01,     1.5520462850436639e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4699737653318641e+01,     9.6419735737295996e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4348416643057624e+02,     8.3303133301417911e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9407344693107564e+00,     4.6052725513988698e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2565885355964340e+01,     3.0628053260862263e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.7855376832717322e+00,    -5.9518577024798446e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.6463414691331295e+01,    -7.5787321126818497e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.1624126769410751e+01,     5.6803388186181110e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1330135285086183e+00,     1.0309174153739546e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0586702252231470e+01,     7.9563339713356490e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1625492110460094e+01,     5.0290905126805292e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1074272291697322e+02,     7.1759789320393465e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3867928892217130e+01,     1.5565045729931278e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.2650010176892881e+00,     5.8008673267482154e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.5005574575962115e+01,     7.7138903509509373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3061733237637242e+01,     7.7053148718355005e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2548015187335464e+00,    -7.5765600597571620e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3617767825215466e+01,    -6.7926901732958759e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.6497655227751860e+01,     1.6658478730418225e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7373408455317808e+01,     3.0377018901994066e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.1104042643281637e+01,     6.0192674722322809e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4321835921080677e+01,     2.4998693512497757e+04,     5.1879272061472372e-04,     8.2508016319626376e-07,    -3.9081800461601917e-09, } , 
      {   -1.0378377512243424e+00,     2.7674587943382107e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6459602705276620e+01,     2.9253554922442479e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.4997028695985179e+01,     7.2114093496610249e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6406794495669754e+01,     2.5284641253619699e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -1.6662679817034132e+01,    -8.6412416451586068e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.8108774779608382e+01,     7.1607414522895670e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0086624624013901e+01,     8.5405678135698909e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4566606565128623e+01,     3.2811524043870973e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3941871914270299e+01,     6.5438068099244620e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0892458429255123e+01,     1.7268058408220139e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7990418586285433e-02,     6.6260956635159819e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.2102522190029703e+01,     7.2289833036209457e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {   -6.1247570404968901e+00,     5.9705656065672010e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7802292942213711e+00,    -7.8930725588161213e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -9.1460504825296933e+00,     6.6123292524470486e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9655818355629034e+01,     7.7292402818651954e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.6258133174115400e+01,     8.0189030548628807e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2724453172592469e+01,     1.5734828922830893e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.4458094750821502e+01,     1.5557674728346072e+04,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    3.7024648172093116e+01,     2.1753537544259634e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2764249723747110e+00,     6.6813152542910793e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5378051492890386e+01,     1.3076680709749780e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6381874642024261e+00,     8.6157931515679720e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.5039876065993109e+01,    -7.1857111429888628e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.1303232806298951e+01,     3.5015718277297790e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.2641823226720433e+01,     8.4814947469096405e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1039132290984199e+01,     2.2029182775728157e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.7327521810952284e+01,     1.3228598459491346e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8301840852213417e+01,     8.9727314939182597e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.9121912197625107e+01,     9.9574517717074268e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5803182880240698e+01,     1.5564795543628687e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.7452163782634358e+00,     8.3134959707419712e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6197638655259812e+01,     1.7047767425555998e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5313248957312010e+02,    -2.2650968192816435e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1253579770903563e+00,     7.2064362818325617e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0163801131677843e+00,     1.6789380054268851e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.2571227690500213e+00,    -2.9465822171917316e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.7590929792098926e+00,     2.4011131125894142e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -8.5844771179056849e+00,     1.7976131169514272e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.1657939434779019e+00,     9.8146040277964175e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3750712992346649e+01,     5.7498617187627713e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1506732911508387e+01,     2.5177574796650726e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.5904209948961686e+01,     1.5143174364891243e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.3931870674428325e+01,     1.3403400095371426e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3862599115665136e+01,     2.5092012468597761e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.7360377783437414e+01,    -1.1142695297574919e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7041123846345105e+01,    -1.7176614055064761e+02,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.1550642241910840e+01,    -2.1671021479205492e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.8158874971138680e+01,    -1.1149875887750950e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.7055231836802182e+01,    -1.5292757364970953e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.5008806264178730e+01,     2.4357414373171763e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.4175259123734989e+00,     7.3192897144143390e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4026326501370413e+01,     3.0392766645509706e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.0664430820306222e+01,     2.2621830421660627e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.7153583312441285e+01,     1.4253324095037586e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.1697865184532020e+01,    -1.6634840944319196e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -2.6959813709637949e+01,    -1.8577270370200131e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.0242749702621573e+01,     9.4374199299050033e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1479841443946967e+01,     8.3259512757919001e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1628337072979912e+01,     2.4035739024849245e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -1.5900191148490842e+01,     1.6787302511485148e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.0049510946046945e+01,     2.2877349107859915e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -2.3627249929363590e+01,     8.3314315736583521e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2080786444363298e+01,     3.3536934403053601e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.9539140506446163e+01,     1.4381740324460368e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5415056624979981e+01,     8.2909540656109148e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4431933785940081e+00,    -1.6639358641249048e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    8.6187643727036125e+00,     6.5541160911799498e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.4234985775054767e+00,     9.0629372041930510e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1946710868361135e+01,    -7.6643354232771726e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.6245486179149044e+01,     2.0349525009883000e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.4618013982265552e+00,     2.2738768212287414e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    5.9196288147753002e+01,     6.3485968375042421e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4971129352434751e+01,     1.2620755236793160e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.1037277898988274e+00,     6.3115249910333455e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2120829013656586e+01,     2.3869303476639459e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    4.9123590591746783e+01,     2.2761748187798712e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.7414829047763281e+00,     7.7062282834678490e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2803250860473554e+01,     7.2166658894473130e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -5.2642042713169758e+01,     1.5540100774386150e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.2860687960159126e+01,     1.5545407805981768e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2457473184362211e+00,     2.1042746666046049e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1788539972520580e+00,     8.9136278483480892e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0916235714823635e+01,     1.5546567581997095e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5919440148327510e+01,    -1.6656286968482280e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.1488766022214314e+01,    -1.2302033741121704e+01,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    8.0178707608378907e-01,     2.4094113091939260e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.3756716622607428e+01,     3.1861008824215212e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    3.6349565399755353e+00,     1.6344562924657814e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.2253595474168996e+01,     1.5136615409291517e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.1577664337918300e+01,     5.2339682782379487e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.9992881852840334e+00,    -8.5605809592699734e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.5099148448568723e+01,     8.0177442353356673e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.1315353073734578e+00,     1.6526870967013059e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.3274325914348850e+00,     8.9738597564648535e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.5027824502752765e+01,     2.3870479312461408e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {   -4.2213305365914998e+01,     7.8353705736617503e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.8740631396548110e+01,    -1.4159773830261315e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3915892543055293e+00,     7.2735814424836317e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.5566925515665311e+01,     9.8255242423246873e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9959642045102392e+01,     5.3926533627777280e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2646768500150909e+01,     1.0922103858235605e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5826555621417246e+01,    -2.0037593728285719e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6655639964052568e+01,     1.3722265884860000e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.2740460503533185e+02,     7.2136124979269180e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4662895583063786e+02,     7.2145132329907474e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3546592756497109e+00,     5.6632013537939310e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9875508752632474e+00,     6.6234926395275015e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7583678404155478e+01,     2.3881754996164575e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4746742277339209e+01,     3.3511197253531769e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -8.7879425130869464e+00,     2.3241060587079843e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.8212583538214247e+01,    -1.4989796609770938e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.5555290752131569e+01,     2.6065278223126850e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2183357403108495e+01,     4.5892730430743832e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2429469944775191e+01,     1.5760007221549187e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.0240808712916074e+01,    -1.2896852521271744e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1965681466798060e+01,     7.7582592338925224e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.3877466680612935e+02,    -8.3270695193084612e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.1043175599711802e+01,    -2.5004123217530752e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    2.9523817019083083e+01,     7.6225029461910099e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5184558709486680e+01,     2.3870893425376602e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.6580575558425608e+00,     8.7988030636039693e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7020904476569747e+01,     1.7427677045320590e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3245884364718343e+01,     7.4121709725007149e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.6397661677495272e+01,     4.4130002434587777e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.8021233661120487e+01,     2.2879676030873958e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.6633594675947236e+00,     1.2116899251462160e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.1098389182231649e+00,     4.6625961670712146e+04,    -4.6251613028794612e-05,     4.4225188804492980e-07,    -2.1125271340666918e-09, } , 
      {    1.2112527845880368e+01,     1.5560386316459615e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2192553979074368e+01,     3.1320795195720471e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.5382060844415442e+00,     2.3871998004441568e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8452589113287878e+01,    -8.1174571930904995e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.6709899966412877e+01,     1.8374083171363178e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8875302820486395e+00,     1.6928710861655800e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.1369491872519033e+00,    -1.7084185246646208e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    8.1151143333965869e+00,     1.5520713036739231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0944859626016299e+02,     6.5852473218752511e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.5057630364730930e+00,    -3.4132358003994204e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.9274791462671722e+01,    -1.1281350705376474e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6223408490285941e+01,     7.2363543052061523e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.1730944592771500e+01,     7.1917714257115131e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -7.5599927156788835e+00,     8.3664287838393375e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7435125026641273e+01,     4.7167552941983820e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {   -2.7402153127815691e+01,     1.7007952523658088e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6033404023556205e+01,    -9.4425142397244546e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.6679463186675601e+01,    -2.2682246259757767e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0605600495618681e+00,     1.3769165223562417e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.9608597993384102e+01,     6.4558356183783926e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.5962531949762198e+01,     4.0802812380002732e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    8.0938324129100572e+00,     1.5569984252248227e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3557018053770731e+01,     1.6230580452254300e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -4.2011025268898152e+01,     8.2006102206479609e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6822744786085465e+01,     3.9587701054521356e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -1.5626686419809882e+01,     2.4636008260533221e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    3.5676833695568021e+01,     2.4640610660212686e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.9717657049481744e+01,     1.0968504524060951e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.6613945826748999e+00,    -7.7871343643937380e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.8925125064396937e+00,     7.3729701570924071e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3079175568191012e+01,     1.1474728387598020e+01,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    8.0304645618798354e+00,     9.8478156820720051e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.1174260573482728e+00,    -3.1166593907153731e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0547291857026398e+01,     7.2927262668069779e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3052682847770335e+01,     2.6464858148419564e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9937870841250813e+00,     1.5103332654665628e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.4830144010704815e+00,     8.9842214816739070e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.5218081661425713e+00,    -8.3464733439036063e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    5.1900867466604188e+00,    -3.4925013618382854e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.8072485515031982e+00,     1.0513663777503429e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7835202094407126e+01,     1.0680258287283323e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.1027620587827375e+01,     1.7794553313847602e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.2762681126972799e-01,    -3.4907880614115443e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3171703915679251e+01,     3.2487238866477059e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -4.8255272332639834e+01,    -1.0194567271820392e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4923605622305427e+01,     2.1373634485766102e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3066079284534062e+01,     3.3301647787701047e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7307795062801453e+01,     2.3861136433653595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.4958807188869301e+00,     2.5257980547035069e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.8413359411537655e+01,     4.6630564070391600e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.0257536037397923e+01,    -1.2264068736604952e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8092801336748352e+01,     6.2291521318105060e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2153174859341167e+00,     8.9555763188369147e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7345894650217037e+01,     4.0103917902581816e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3281497333641219e+01,     2.4362682461917702e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.5643737721305936e+01,     3.0040769799121597e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.2339777255847110e+01,     1.5232216613659015e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6716470150854601e+01,     7.1791179764488452e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    9.4176873486399337e-01,    -9.6992701156474868e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.8959672320084664e+00,     1.9548215048286220e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6394765626781236e+01,     8.5104929374478306e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.0132371393958604e+00,     9.3346008684244662e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9921948842841701e+01,     3.3530348639126169e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {   -1.3784488206560894e+01,     1.4930179189286004e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.9972576321605487e+01,     3.9336647744080925e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.1341933639778581e+01,     1.7325505336393708e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2000875561845122e+01,     1.5864781954464548e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.6613965429686992e+01,     8.6134919517282360e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.1084825253218826e+01,     1.7408489375612360e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.0925615106068065e+00,    -3.1166854554829115e+01,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.0148570561502325e+01,     2.6340905221582099e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.2808831260250969e+01,     2.3873587953178707e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.5755170321068697e+00,     4.9064768255075454e+04,     2.6270754078660380e-04,     8.0528037245975031e-07,    -3.8273131241511210e-09, } , 
      {   -3.8884166227004286e+01,     6.2678233179432573e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1279876889751073e+00,    -2.5224191280980840e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.4015419014292199e+01,     7.4619687191337680e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    9.6712773304603381e+00,     1.6082755707082777e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4428478237026354e+01,     3.1072390669168461e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -2.0023818298544850e+01,    -1.4436227307907518e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.5427921851803234e+01,     9.1972166572791430e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0504616704707345e+01,     7.2167163812566405e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6305632540884837e+01,     1.0992094482271041e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6429794640128435e+01,    -6.6921731961405067e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.3477272736263281e+01,     1.2181348860642876e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.4278006601762527e+01,     1.5541772628368230e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.8213466541572302e+01,     1.5543735951999688e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8383656496096705e+00,    -8.9132281762615094e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.0422014913956072e-01,     4.5496571265359553e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.6492818585919853e+00,     1.1377257977901529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7073856731748318e+01,    -7.4029640743540913e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2642998727456396e+00,     1.4456991385632758e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0043354484227038e+01,     6.8030082117785205e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -9.6745551421176614e+00,     6.9248835958322870e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    8.1337262339573009e+00,     2.9097278877134784e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5580139816484754e+01,     1.6158547122811096e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.2494234149044253e+01,     6.0145355940325144e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1617589503719044e+01,     1.5010006679016205e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0043157716920184e+01,     1.5478764566250404e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5474839990752582e+00,     1.6080890375895000e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.2988829635807473e+01,     2.4440234722929828e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.5182121624698951e+01,     8.3993560976610734e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1315344136332797e+01,     8.6038197503032970e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.0579126784701113e+01,     2.3384772424730261e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.9030506843679902e+01,     5.4832546529164358e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0476410629391914e+01,     2.4981143242019458e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    5.7460314229147002e+01,     2.1500554301509779e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1115583849496591e+01,     1.6100779915283982e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1162768307086932e+01,     9.9809573968253826e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5177366115278907e+01,     3.6186937450620644e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6718426418577991e+01,     1.7497541100166251e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    9.2931890122105543e+00,    -9.2855856296509883e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.2305962581682355e+00,     1.6282395563744591e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.3025381234079303e+00,    -1.5540487301284515e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.7525338936796842e+01,     2.6294005882879675e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.7736910462723174e+01,     1.3148968655695524e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3286645274651804e+01,     1.3799811432321396e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9479877713360473e+01,     1.6658486376842677e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    9.2234496913127924e+00,     1.5538940998370823e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7479651252467416e+00,     5.4874711949223865e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1241430317914242e+02,     2.3243236181437380e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3639927235266427e+00,     3.2748814920883931e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.1293272052595078e+01,     8.5663278190772417e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.1936068531331676e+00,    -1.0646966056871315e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4546670538972815e+01,     8.5482302965620438e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9431484724668209e+01,     9.0240631918995241e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2843574916980259e+01,     1.6244658204630379e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.8899100761010477e-01,    -2.1293981584050971e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.9474648128909234e+01,     7.8074255614943886e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.8748379007579974e+01,     7.8254497696955941e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.0783827083800190e+01,     1.3896720946434802e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2655675207579147e+01,     1.0133984376065931e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1757927468545589e+01,     1.6853769450288986e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7338276974695098e+01,     9.8228531795187955e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4872845583899306e+01,     1.8751356397540527e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.4384178621943531e+00,     7.8482705045907933e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3718738786005822e+01,     8.2480106585197427e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.3295759958342016e+01,     8.8702484850565143e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.0592126317165880e+00,     1.0229613688186240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5019943284854403e+01,     3.0850221965015364e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.4692129921998784e+00,     3.0627992076495207e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.0148559811330763e+01,     1.0682662157315650e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2560495639777507e+01,     9.0678340254086797e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -6.1392511996677300e+01,     9.6406635250025556e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6623862182894289e+00,     7.8274141173662629e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6513924248409571e-01,    -1.8347150386226588e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    4.9240202864840832e+00,     1.1823161649020431e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0348253688190017e+00,     6.7233326640690120e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8516299878502114e+00,     7.2518002245730440e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.3808517193306233e+00,     1.4940755095764973e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0652109183500809e+00,     2.1120298927058171e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.6340668963738691e+00,    -7.5234712914170441e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    4.1788569867567507e+00,     6.6015030239289281e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2445420066776691e+01,     8.7643102906341319e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.9512866214210565e+00,     2.5593051951243069e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.2546856063198121e+01,     6.9968251934616510e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2832531171323200e+00,     1.5951523003392587e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0114473879865407e+01,     1.6664386430929095e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.3604624739034097e+01,     1.6858072159841515e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -2.0116245718092461e+01,    -1.6875792470720437e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.8653097291991179e+00,    -1.2690666896161988e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.8728007414278991e+01,     4.9069370654754908e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    3.4569175931465374e+01,     3.1317398114912765e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.9959412852966715e+01,     7.3872163308384552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1059929492521377e+01,     8.7649243771237343e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.5463796309438877e+01,     9.0542510881277849e-01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.0164332975878327e+00,    -1.2370234810170090e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5201619980480482e+01,     3.6922932492611353e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4878060651539602e+01,     2.9042851470890855e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    7.1301724881263571e+00,     1.6099357224350229e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1058218759196617e+01,     1.4308681025223928e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.5028232462161650e+00,     6.2265157382232665e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.0074533757645384e+01,    -1.1011220479949393e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0497481038169639e+01,     1.3579950641558757e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2239157462321160e+01,     2.4928986375510609e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0818055869113749e+01,     3.1005326487039609e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5223102693674285e+01,     3.0499860811278078e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.4454483092061068e+01,    -8.2663346176961568e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5956711643369793e+01,     1.9494623224986390e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0543689550363638e+01,    -8.3381339389648492e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.5490228410683713e-01,     3.9121607535745006e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1857255122307031e+01,     7.0841432034239351e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.2878693013153653e+01,     3.0421152578919962e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.1157295835459173e+00,     1.6813963218069781e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.4409248543305388e+01,    -8.3313624875310179e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.9503645620612915e+01,     3.1691274260282404e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.4225384011775196e+01,     2.5868534779107531e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2043917243402369e+01,     1.6566712677238676e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6072154129371086e+01,    -4.2450119735621774e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.1354221764496927e+01,     7.2216894490851037e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8181858434665365e+01,     1.1033890368943853e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.1070064957597413e+01,    -1.4217856738230353e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2317244265183383e+01,     9.1899844252834901e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5521293895675150e+01,     7.1453707598861693e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.8268687554806636e+01,     8.3314228755613622e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4466438493443089e+01,     2.6165137183254341e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2839064646576318e+01,     1.6569468863229074e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1912610062450142e+01,    -1.1167707975359201e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    8.1461042185937718e+00,    -4.3335187700955890e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.5120654167224437e+01,     5.2616781253116471e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8372074828286362e+01,    -4.2910359703568611e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.9577270759532794e+01,     8.9590765525543702e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4591324764015297e+01,     2.4429863521934658e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.2210944643360389e+00,     1.7121363754956874e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.1594459167916945e+01,     2.6071837178726575e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.7823048272487199e+01,    -5.2710227856634083e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.8553390382018861e+01,     8.7190760008308735e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.9148278913654035e+00,    -7.7587579067696006e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -5.3532159919985247e+00,    -4.3570120941258274e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.6880024184732365e+00,     5.5901638751965929e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -5.5563950799401702e+00,     2.1497900785711972e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.4506276253607304e+01,     2.6857879118112553e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8115976191262256e+01,     7.1983151219431938e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.5857504516230321e+01,     7.2150445272745619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1992069667973535e+00,     5.9923925843777324e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0015109468300581e+01,     8.1878083412927563e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6594285548047503e+00,     1.4788666491121003e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5097973684719840e+01,     2.2876261861336527e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.1475661810570251e+01,    -2.0340999938798632e-01,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.2567690847867528e+01,     7.1342094046069742e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6995149829259919e+00,    -1.0966043510650902e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    4.1575249936198915e+01,     7.3222833081487115e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.6783986406652375e+01,    -8.6852523255401684e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0310419836209341e+00,     1.5899357438786743e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.4399471798526640e+01,     2.4346949977770120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0220474556808785e+00,     2.3782428201295486e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0331847339493811e+01,    -8.7090931560957342e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.6006117118153371e+00,    -1.1124863209966686e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.0276249433816162e+01,     1.6580941605991586e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.1358944787803466e+01,    -7.9268539863190863e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -5.9139001832477362e+01,     1.0268971110048098e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.1678028139117416e-01,     2.4754184739630531e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3801736557652553e+00,     5.6577267302025211e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2580617255031132e+01,     9.7027005660539508e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.5607921194770666e+00,     8.3922065635684648e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9011566245132819e+01,     8.4394996320354348e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.3413221538823716e+00,     1.1770278947594516e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4613701535581569e+00,    -2.3072540054238243e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.7236095409233059e+00,     9.4121531294365905e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3380164951197120e+00,     6.2330746938229931e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.7875974353635482e+00,     1.1720068836465555e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7322095812607358e+01,     2.6951179122640315e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.0522905409542460e+01,     1.4764826744044522e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4160396557826139e+01,     1.0784833866242530e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1054248429998044e+00,     4.3903395234982645e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0291410941457777e+01,     2.3385124740688334e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5163357627102808e+01,     2.5021411345110911e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.0994166510699088e+01,     8.3059438043668615e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.0344788242950980e+00,     1.0984237008208540e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2127683818639614e+00,     1.0120642807810442e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5088906948977218e+01,     3.3799660516241850e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6546286335179687e+01,     2.4970532245886949e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.0583596892859845e+01,     6.4762982289117645e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1684318186286955e+01,     2.1641836280418422e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    5.0506692416199623e+00,     6.9008613142036938e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0759500204167944e+02,     6.6498430559206845e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9357400071681738e+01,     2.1642540912334567e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    7.0772430942797113e+00,     8.3298024422785820e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3661339497759117e+00,    -8.3275804071716702e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4602155704168666e+01,    -1.5225973074902254e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2276584482164873e+01,     7.5268827902512721e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.9753124125349803e+01,    -3.8848499450233703e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -2.5331674832640836e+01,     1.5819727634990917e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.0088407393287156e+01,     7.3668661876433471e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1450795875530591e+01,     1.5062333370049626e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6784236307576297e+00,     5.9361177873898941e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4893994600291443e+00,     2.2666645656308287e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.5118116985416377e+00,     2.3413990395403431e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.2130000186285081e+01,     9.3884526157048640e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.3727614480572452e+00,     1.0059094436993400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2186023612464290e+00,     8.1889907543720201e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3019314156402846e+01,    -8.3337584966723953e+03,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {   -4.0235619395064830e+01,    -2.3164592077846464e+00,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.7458848983415798e+01,     1.4293617286661929e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0237303016077085e+01,    -1.0719687433105898e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.9271190631289898e+01,     2.5001616302463808e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1701745353314751e+01,     6.8494278291761029e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2112500655665315e+02,     2.1755943575404615e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7244030294222732e+00,     1.6660227763508043e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.8987598072907392e+00,     9.3772424513668939e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.7441123948655726e+00,    -1.1430144926765520e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0842191994043304e+00,    -8.5720223180588146e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    1.4125574533784601e+01,    -1.3579594525999812e+03,     9.2781703036018566e-05,     1.1077022985990678e-07,    -5.2374421970263164e-10, } , 
      {    5.4580598729059648e+00,    -1.7319466232201703e+04,    -2.7103438544444846e-04,    -4.6451521102063327e-07,     2.2525364770343145e-09, } , 
      {    2.1591165457789820e+00,    -3.3558096592234193e+04,    -1.4565567702329698e-04,    -4.5399892353821387e-07,     2.1576632877779900e-09, } , 
      {    1.5854983742172156e+01,    -1.7767119420177416e+03,    -3.3722250666782941e-04,    -4.0056828648228549e-07,     1.9438071249037621e-09, } , 
      {    2.8678526238080792e+01,     3.9589968043420806e+04,    -3.8180434865683064e-04,    -9.5493750768145316e-08,     4.3473242785092074e-10, } , 
      {    2.6322971665828891e+01,     3.1261276618695680e+04,    -5.3411709929817881e-04,    -3.4590486519803176e-07,     1.6210715055259552e-09, } , 
      {    1.8993171239483583e+01,     4.2028774627784107e+04,    -7.2845194841432225e-05,     2.6753473364667522e-07,    -1.2800535622335079e-09, } , 
      {    1.4282054764596921e+01,     2.5371391778333855e+04,    -3.7747069612412869e-04,    -2.3328749521309762e-07,     1.0926245931165610e-09, } , 
      {    1.6268835855729421e+01,     1.4461644314097703e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6287810549709508e+01,     1.6654537935392462e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7270550594995029e+01,     5.5770649155693782e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.4379348831902973e+01,     3.0945823169382857e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.1045867767150623e+00,    -1.6613612015941097e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -3.0997735010259795e+01,     5.4004128286629066e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.3043087213833388e+01,     4.9641574899907846e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.9695053752718103e+01,     6.5847888779656114e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.5130633413156396e+01,     8.3514390450833907e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1069406876860661e+00,     1.1293470676908516e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.1778034340756847e+00,     4.5742497661230082e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0048722195535067e+01,     1.7269481099153891e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.2557864947017769e+01,     1.7086615925006234e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.2638213626596986e+01,     1.5933903432320456e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6244782385719514e+01,     1.5087600170518039e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2364262882257400e-01,     1.6165028400834555e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6909039636306680e+01,     2.3826563741063903e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.3191648134458349e+00,     1.2491370035220295e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.3430520174171235e+01,    -2.7267083889043806e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.7862844097245166e+01,     1.0999450404020521e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3256325566235262e+01,    -1.4323984677868939e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6627590163665955e+00,    -1.0857669508422938e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.2206139503537273e+01,     1.6625902621787023e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.0011625890503865e+01,    -1.1139239273501437e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3913768480419689e+01,     1.0735983570484917e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.6130831149318343e+00,     9.7276218381174549e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4720445819270239e+01,    -2.0670077574564857e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.3737496947838572e+00,     9.0993142530719106e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6007958768335584e+00,     3.1292756846358905e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    1.0168377484510277e+01,     7.1967500027245042e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.2328258139980921e+00,     6.3932821176790719e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0111330591824476e+01,     6.1770018265727122e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.5335285544582227e+01,     8.3109711138462026e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.0046394689086021e+01,    -8.3464117356040497e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3217804283846110e+01,     2.2931618791522873e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4178835539302057e+01,     8.0465054706701494e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    7.0762167081930771e+00,     3.2370978991055063e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0307080376061813e+02,     2.3015856606642046e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9073985795254558e+01,     2.5826232524891380e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.0978424182229799e+02,     7.2781506409662270e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8328055239195983e+01,    -6.5255860009683602e+00,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.6633132784572950e+01,     7.2130812036431034e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1169405860703144e+01,     2.4516673980763411e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -9.3503209247160086e+01,     6.9055299050014582e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.0920854509855488e+01,     7.2262687902784191e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.6571263131881366e+01,     1.0207055224262029e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.3347673951902927e+01,     2.4340493692362979e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.8408915320427099e+01,     8.6384797945873725e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.6867791824800449e+01,     1.0585759356752969e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3206950552874911e+01,     1.5469166630461792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7172180033760839e+01,     6.6735549652405098e+01,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2935285178057173e+01,     7.2005003283802635e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5131056334075137e+01,     1.2618140663845024e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4529008911544652e+01,     1.6701153682959412e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    3.8946742165991992e+01,     2.6687026852572664e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.0197630082120739e+01,     1.8954265683376008e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9847643938030537e+01,     8.3114526447119097e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.6897426177600096e+01,     1.8620204439904279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3978220145965764e+01,     6.8053094116182565e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.0801009204959122e+00,     7.7320230289129222e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9563500833862850e+01,     1.5038399631975883e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.0588218726690144e+01,     6.5852029908808063e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0079845380302501e+01,     2.4539568201852540e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.9413597945111533e+01,    -1.0592716290799517e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.4133721346191432e-02,     3.3640352618572656e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2594363391142398e+00,    -1.1572883752219971e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.8592692696744095e+01,     9.0983545319977566e+01,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4025964579085837e+01,     3.2428874275173159e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -9.3571959901886984e+01,     1.0251698113281242e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3066379563616852e+00,     4.2871480392801623e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.0606949417038464e+01,     2.4030705322500733e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.0316024732058381e+01,     8.0315850325080373e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.5499985041017482e+01,    -1.1153331911824432e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5305973511910004e+01,     2.0402494107094193e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4818707443602605e+01,     1.4694579200825050e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.8272538203689344e+00,     9.6382756920274041e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5170575887955576e+02,     1.1130459310148581e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.3448815536569185e+01,     9.3275269373592707e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.2223573891958569e+01,    -7.8153371824224428e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.7351598248862047e+01,     2.6050522630237538e+01,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -9.0795501562724752e+00,     1.8695036440974400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1727391195038956e+01,    -7.0256755888086964e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2939168159633678e+00,    -3.4159825106037006e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7524318697168875e+00,     4.5490930642134636e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3214656345486699e+01,     1.7081056741452940e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -5.6387827296675574e-01,     3.2194103665239261e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    6.3120057959604488e+00,     1.5933515290526919e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3066601838701173e+00,     7.5420906345356125e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.4342274271177580e+01,     3.9766103627552184e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8485199255349450e+01,     1.8148728592984185e+00,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    1.1763919895607158e+01,     9.8412567264722784e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.3361555604627817e+01,     1.5231234951843287e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.3901368911430989e+00,    -2.7941307531145242e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4440649710938052e+01,     1.6678046365753762e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.8365051908864807e+01,     2.7938608133816342e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.2902421339092944e-01,    -9.4083750949814330e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.0500817544809387e+00,     8.5393948762767486e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4385458329210991e+01,     2.4080283835272930e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.9322798342158116e+01,    -1.1277464704657486e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    3.1296470329871688e+01,     7.3031051142266006e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.6385633942750530e+00,     1.1300584131577282e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0053446221338187e+00,     5.9394234871665412e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7842148958746080e+01,    -4.9028363860033278e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2237218116237059e+01,     7.2136670313710156e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.3270317986248502e+01,     7.2144586995466479e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.0174775587305160e+01,     8.3281927518480497e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.6462269036018103e+01,     2.3808167336442406e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1518060039160698e+01,     1.4374502803661130e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.8821972218431377e+01,     8.3283285865632424e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6674674016524968e+01,     8.3290542628870098e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6705548294771404e-01,    -6.2831810724935235e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9873231120605521e+01,     9.4920065902884508e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.2364217462019013e+01,     1.9328141828745647e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8299443835631708e+00,     2.0649725696353064e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.6553389572109509e+00,     1.2334152182827389e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5775568401389890e+01,    -1.1211875148660229e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    1.7291810677821708e+01,    -9.4035050820935066e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.3133414102969270e+00,     6.8805604727906430e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7632730225033868e+01,     2.4081706526206683e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.4409256227234916e+01,     2.0856146656990517e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.1688939079410471e+01,     6.5859036974705032e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5915546789568765e+01,     1.3737684995899186e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0903422404395240e+00,     1.6177620830829230e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.4352109251690230e+01,     2.5022834036044664e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.6662299703017851e+01,     8.3104520255105908e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -7.1373408847521659e+01,    -8.3469308239396596e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5181243296597355e+01,     1.4914670362194447e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7373071645363606e+01,     8.4215117079480533e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1182245049669532e+00,     1.8635928471587777e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.3641769550752088e+00,     1.4905810206754986e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.7087631388790836e+00,     7.0281959945520128e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.9417817101070085e+00,     1.6605125097705935e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5799915484429379e+00,     6.5782771138442322e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3603083980927870e+01,    -1.8381079763960906e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.6523336887371403e+00,    -3.6578902200363024e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.0578829529763713e+01,     7.7790920720376971e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3741886348013596e+01,     6.7872604682628789e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.9702718421239624e+01,     3.1297359246038370e+04,    -7.1735708428486659e-04,    -6.3323938958122043e-07,     2.9819919897685337e-09, } , 
      {   -5.4428795419123830e+00,     1.0591216038515553e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9270005104167716e+00,     1.6656219387943231e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6540329605502704e+00,     2.1940122293947112e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    6.7788366740672776e+00,     9.0606102811790079e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.6978061376476006e+01,     2.6458299192819839e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.1661869790321244e+01,    -2.5381333792323976e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.1026615424866231e+00,    -2.9400327373781247e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.9316936286213999e+00,    -1.8347349896550895e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -9.1378357560100710e+01,     9.3291441866618825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7885471524560757e+01,    -6.9017226619712283e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.1831212205105288e-01,     5.5049680827645579e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6288191981012524e+01,     1.4872498234649589e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9697767679571832e+01,     8.3260203619192343e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6512248217908674e+00,     2.4880136079752996e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.1587567052644747e+01,     5.1737069558215062e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.2979346499490322e+01,    -8.3259599738888901e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8497750450559295e+01,     1.6028746631928258e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0098836528182407e+01,     1.2165172699773211e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2541760429259231e+01,     7.5775249644484193e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7130349732912336e+01,     7.5521663751096639e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6402002831614475e+01,     1.4286467752569321e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.8428448853414459e+00,     1.4871786889182713e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.3182025833915461e+00,     1.9155194078593235e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2476670676797124e+00,     7.7960127528464773e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5900187745187043e+01,     9.6589297325447915e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.6889836153197727e+01,     1.0897278695093641e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0125788411372163e+00,     8.1168150358803605e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0634380873698145e+01,    -1.6892557016337689e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2829143448511374e+01,     2.3855697971393445e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2660184594383010e+01,     1.4914457227342227e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6489843155251673e+00,     1.5007746709666179e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9536380730689924e+01,     6.5829103663554979e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4086172952755767e+01,    -3.4551855606370765e+00,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.9820265520178665e+01,     2.5900984913186694e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.2890725985294118e+00,     7.2276254025374019e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3457577564313624e+01,     9.5849542788581366e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3956734379105541e+01,     2.9138452675258352e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0203434920477266e+00,     4.1566361568429684e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0852086608491490e+01,     8.2944578811820156e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.7621084284152833e+00,     7.3875449633149037e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1632742230327409e+01,     1.3625016509992043e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0979007065035177e+01,     1.4054893689920555e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0962176507493263e+01,     1.4882807515905077e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4569806937589402e+00,     7.8379612233353055e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3653679456099503e+01,     1.5631796538951727e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.8500448002090025e+01,     1.4230782189906529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.1244633748646979e+00,     1.3722764557737073e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.1102944526525796e+01,     1.5651786778859310e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.6969785254526894e+01,     2.2428230535762472e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    8.4327998694539108e+00,     1.6093064988338265e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8463998899751324e+01,     3.1572540900055545e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.2906336079483207e+00,    -1.8384961181896272e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.0217721325284558e+01,    -1.2566084565403598e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2112560595999797e+00,    -7.9164922611100319e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.6382365060214500e+01,     3.2932055996088093e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.6033801942495135e+01,     1.4322187536495283e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.1670257143232270e+00,     6.5853620341508476e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3633263164896052e+01,     3.7540510385881134e+01,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2235648746050147e+01,     8.6457984749810130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1062060480474649e+01,     7.2191299374061018e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -2.4523006274416247e+01,    -1.1493763892169363e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.3602810585722025e+01,     2.4160624984535632e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -6.5463490780279940e+01,     3.1582184425719333e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4608161505898507e+00,     7.9110967670173259e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3818403221130358e+01,     1.1315124827928594e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9494790186270592e-01,     3.1300125118145908e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.2721939377420872e+00,     8.2124439490472560e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9421939598622327e+01,     6.3758534986244931e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.4933768276089747e+00,    -3.4689516078148922e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.6615048535937291e+01,    -4.9375934269737627e+00,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.8187072129749549e+00,     7.4585382687273004e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0117542026590051e+01,     2.5114942973800717e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.2287463340650140e+00,     7.7702136835849578e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.5553565831387306e+01,     8.0404785575462583e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.5218797429378757e+01,     7.9859936828699701e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.8418698325442016e+00,    -1.0616689378412225e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0042259556885642e+02,     1.0543023588434880e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8622098740388413e+01,     4.1104167052193690e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -1.4106225236186951e+01,     7.4995931204327644e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.1976461569417065e-01,     1.6081333016149025e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4645829870662794e+01,     2.3465306834016643e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.6189533758851162e+01,     1.6100397202293556e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    3.4439169200054749e+01,     1.6099739937340657e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.8417123722003055e+01,     5.5680295217096764e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8127448327952720e+01,     8.3279754876611860e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -3.2838557472456529e+01,    -8.3294073617890663e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.4312406554270337e+01,     1.5543917751690980e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.1909881480421785e+01,    -6.9622275600954652e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.9350801384953179e+01,    -8.9750479531260426e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.4188039894045133e+01,     1.3905406747626330e+01,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {   -8.1721649041755128e+01,    -1.1389014328757758e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.1156038221208182e+00,     1.5548654344270375e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5101284856973400e+01,     5.9649258619368029e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7250656928905510e+00,     1.3072183765957489e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8143646736463968e+01,     1.8767104141056167e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.8351434570246568e+00,     7.6999905934171411e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5753315822265058e+01,     2.9733154920040124e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3011418901045971e+01,     8.1452532904616191e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5207959662431822e+01,    -8.3183733862424815e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.5405244905002853e-01,    -1.6733824092908922e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.7006567720556603e+01,     2.4771457736397388e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.6564826397262632e+01,    -1.3937203787665749e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3294014966510845e+01,    -6.6326546191157086e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.9433223023962213e+01,     2.8073983030246143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.2981640437753761e+00,     9.8051740991295774e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3457097898496480e+01,     1.5644925999110839e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7376465872997557e+01,    -7.5538901707627213e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -1.0547358290580444e-01,     6.7847492869157122e+00,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1865420022141961e+00,     1.4914053458875975e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1971769492418794e+01,    -2.3689168063433358e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.9878169973791174e+01,     3.4179375426598790e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5391633890966148e+00,    -7.7882978259007605e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.2510294860726745e+01,     6.4122884419252678e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7912207560266474e+01,     3.5737325866871099e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5334823874884480e+01,    -1.0087093164161515e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.1999904532548067e+00,    -3.3150200230837872e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2134973710944866e+01,     1.5837559482561313e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -8.0761765638456033e+00,     1.1723878281769172e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7025976051370780e+01,     2.0088756819740970e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.8652234923023485e+00,     1.0198195068822566e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4554685232891501e+01,     3.2184595111345785e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.3057975351535601e+01,     1.7043844863543512e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -9.7167875817528042e+00,     1.5453712041416187e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.9031117028955009e+00,     7.0747060869502238e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7659251245441752e+01,    -2.4858845489899437e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3983903173426008e+01,     2.2287769178188897e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6952776659768045e+01,     1.4757866336518886e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -1.4314917254758782e+01,     8.1620969149452339e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6639691820242518e+01,     4.0853142189541227e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2070436189382811e+01,     7.1250206166910612e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0972975383471329e+01,     2.0855894892181674e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3660628275308753e+01,     2.5761299502201637e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4508169437375262e+01,     1.3887732132093297e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.4055687900477306e+01,     8.5236471327611107e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9060971702395717e+00,    -8.0682265106542745e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.3919901581828544e+01,     3.9816636895299609e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3517589150914723e+01,     1.5426506814506089e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5701996513999639e+01,     2.4646945958756380e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.0396041667376110e+00,     9.1327857986668841e+03,    -2.5218992006656134e-04,    -2.2272757449937836e-07,     9.9847862438190027e-10, } , 
      {    2.6698284666667513e+00,    -8.9197627514957203e+03,    -3.0689025991263160e-04,    -4.3188172340599684e-07,     2.0444592932389127e-09, } , 
      {    1.1124261453751039e+01,     2.5648133276561312e+04,     4.2344506844938082e-04,     7.1488269221921978e-07,    -3.4396027752310134e-09, } , 
      {    2.1159236826623879e+01,     3.2342172511190041e+04,    -1.3257210717393018e-04,     1.2798111553929530e-07,    -6.1600426321777652e-10, } , 
      {    1.4010299908400789e+01,     2.4462517041679675e+04,     2.4070213868925065e-04,     4.9582864794434457e-07,    -2.3531886453694655e-09, } , 
      {    1.8314226534480245e+00,    -9.5515382880846446e+03,    -1.7781428963728661e-04,    -2.8861094184281384e-07,     1.4690727202244936e-09, } , 
      {    1.4423201115686599e+01,     4.1886763636593794e+04,     2.9806636002822933e-04,     7.0436640473680038e-07,    -3.3447295859746888e-09, } , 
      {    2.0388966760970174e+01,     3.2794682047745249e+04,     2.6660280054816238e-04,     6.0235258869688546e-07,    -2.9097014217782908e-09, } , 
      {    2.7687432734106551e+01,     4.7709134382680728e+04,     2.0307856081768996e-04,     6.6568312112107056e-07,    -3.1639910456570535e-09, } , 
      {    2.5375003669069603e+01,     3.8752141002707111e+04,     5.3429691669250249e-05,     4.1465561457702140e-07,    -1.9232122397302292e-09, } , 
      {    1.7796600563513124e+01,     3.1815141697353894e+04,    -2.2239613162628781e-04,     1.6463593871326283e-08,    -4.0001976828347862e-11, } , 
      {    2.6074788467049562e+01,     3.1680053488478956e+04,    -1.0421082195791504e-04,     1.6547728437546034e-07,    -8.4575261855877446e-10, } , 
      {    7.4707599472592676e+00,     3.3767621677699397e+04,    -2.8691448180987535e-04,    -5.6766833921115565e-08,     2.5472110805495014e-10, } , 
      {    1.0565638144149853e+01,     7.1093181427060517e+03,     3.9912286297342408e-07,     6.8324113339613344e-08,    -2.7377913385936605e-10, } , 
      {    1.8751686607489049e+00,     6.9581604952337511e+02,    -6.4322485456419284e-05,    -7.3317694255041564e-08,     2.9326864383996938e-10, } , 
      {    1.4084768595470933e+01,    -1.8477352575927180e+03,    -1.5438164454411506e-04,    -1.8155787543871011e-07,     8.5666577452054998e-10, } , 
      {    4.1869845560827841e+00,    -1.2228468633595185e+03,    -2.5501538995938368e-05,    -3.8199827412927442e-08,     2.8273364254945920e-10, } , 
      {    1.0593603394863006e+01,     2.5232854368483990e+04,    -1.3297123003690360e-04,     5.9657002199681953e-08,    -3.4222512935841037e-10, } , 
      {    8.7289039723195394e-01,     1.2903609576344097e+03,    -3.6157064967572412e-05,    -3.5734258956276789e-08,     6.4974729542300003e-11, } , 
      {    1.9877043596368328e+01,     4.0914170448883335e+04,    -4.4375674971109378e-04,    -1.6929693744344998e-07,     7.8462246150767287e-10, } , 
      {    2.7438833894619737e+01,     3.0004637068239070e+04,    -5.3401916693459478e-04,    -3.4594849842933157e-07,     1.6203442850042910e-09, } , 
      {    1.1167207897553903e+01,     1.5090244753628753e+04,    -4.6536302894547351e-04,    -4.1042454861924236e-07,     1.9849678064299617e-09, } , 
      {    3.8256153700050231e+00,     8.0409437394175802e+02,    -4.0450267070790962e-04,    -4.7313868892926478e-07,     2.1848177020569351e-09, } , 
      {   -4.4219862690843312e+00,    -8.3659520633655957e+03,     2.3626909935192144e-06,    -6.8984138685075869e-08,     3.2749164158949159e-10, } , 
      {    1.7302624162488705e+01,     1.6483881800021940e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    3.5592448998474158e+00,     3.1532014652690756e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.0056113001587832e+00,     6.5857447582094810e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6610596073561972e+00,    -1.1184313288756189e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.6254419584550508e+00,     1.4239738454267528e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.0958110463659317e+01,     1.6774776820687992e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -6.3517490280718476e+01,     2.3258680158151968e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.2725092554796760e+00,     5.2596646323672758e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1213856791648467e+01,     1.5225966043479088e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0869227142097898e+01,     3.1078949624768193e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {    6.2382649134863556e+01,    -1.5438616348222749e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2079837893423338e+01,    -9.6977514236673233e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.9896541382943477e+02,    -7.7008917982176154e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.9025995965263206e+01,    -8.4952859345050201e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.8959021697823196e+01,     1.7420677168477960e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.8960170678097632e+00,    -1.6848908107079769e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    4.3798023807587775e+01,    -1.2047333194032781e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.8010382882644462e+00,    -1.0267145730451157e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.4676324591278860e+00,     6.8198802772361091e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5461883059478637e+01,     1.5637671857883977e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7751403328917327e+01,     2.4991005306331299e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7477875673776332e+01,     1.5573407212493663e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    2.5793894622739970e+01,     5.5109416230449078e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -3.5328338918758270e+01,    -1.6756516827080486e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.6922335333883431e+00,     1.6326495029730198e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2969227781866946e+01,     1.6285025396060775e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.7213502094492220e+01,     2.7637660261732908e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.1910574288672475e+01,     1.7337006873109007e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.0635443122136721e+00,    -1.0278875103382582e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.6626826362285243e+01,    -8.2612675457488876e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.0767518686949522e+01,    -1.0862426258560349e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3921548117071040e+01,     3.2206170614029157e+04,     2.6936461440465505e-04,     6.0169256335142287e-07,    -2.8559889140481653e-09, } , 
      {    1.1401002203779207e+01,     3.3525026510198099e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -4.6778492189044300e+01,    -3.9813166900524891e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6403783061982054e+01,     1.5058734515802942e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.6744006177518980e+00,     3.3523603819264354e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    4.3995234801100011e+01,     6.9682679241663573e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.6919098916320827e-01,    -8.3816510461501966e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.8376994698414837e+01,     5.3161049971007174e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8378444935814393e-01,     3.3484500262833317e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -7.2561851503552539e+00,    -8.1080414125521875e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.9500130923481521e+01,     8.2508105312365533e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5382047573039802e+01,     1.6419459840510455e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.8277361143404818e-01,    -8.7278298831401071e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.1553091754489664e+01,     1.7285361802019346e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8222921093502080e+00,     1.6498475557816677e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.5145476359352266e+00,     3.2330056801669110e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    6.8426774168679643e+01,     3.8786244736005581e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.5955001768430428e+01,     2.7250900087805541e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.8097795928400098e+00,     8.4449389004029963e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.8740741984266407e+01,     6.4988308772892105e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.3173709128395164e+00,     1.6184179786428958e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.8404679928713801e+00,     1.4914223048082382e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1921807323594032e+02,     2.3058090260965969e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4135348718577823e+01,    -1.1108257896569680e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.4843841814739363e+01,     2.8230187100444182e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4280280847780801e+01,     1.5232965920321712e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4513523003906217e+01,    -6.2396336494227207e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.7411368131073701e+01,     1.6855442327525328e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -2.3922989110132058e+01,    -1.6878422303036619e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.5261990542531628e+01,     3.1781026704461317e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -2.8661320469573525e+01,     1.9113525290967656e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4185384299028616e+00,     8.3143650982261152e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5445802004921028e+00,     8.5493414368980648e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.6916640362476244e+01,     5.6183197656299785e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9460536034469975e+01,     3.3564130066629135e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.2373680524903172e+01,     2.5455474567587346e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2271421124506585e+01,     1.5010075618305311e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2528745979318996e+01,    -7.4730368764060984e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.8348079159434064e+01,    -1.1813641089186985e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.1691216034759435e+00,    -4.2417256487976738e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1356424572568343e+01,     3.0379648734310253e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.2933541386740409e+01,     1.4913894415605933e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3446129899035746e+01,     2.5200690811953369e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.7783489872963585e+01,     1.0024728230334544e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.4323052351543055e+01,     8.8176269718940111e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1732540695307716e+01,    -8.2919316628558390e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1778146980709927e+01,     3.8392871450354512e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.2270388195251009e+01,     3.3314054353433632e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    4.2850556212630920e+01,     8.3270635012813145e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.1197696560013824e+01,     3.2215679167922637e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5998869094115088e+01,     3.3315477044367377e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    5.2200183768958821e+00,    -9.8543746376717336e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.2657743805605701e+00,     1.3846717484574541e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.2912809993320238e+00,     1.5054132116123474e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.7822425607686782e+01,     1.0450772622770803e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1552002294596463e+01,     4.5141150762018611e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5806943504870468e+00,    -1.0050297661059110e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.2091086906562992e+01,     8.9520679776147517e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.0333924085897861e+00,     8.7987058046266993e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0994524112477361e+01,     4.0006459173736184e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.2521153757547323e+01,     7.9199227115164995e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2727069332883158e+01,     9.1304658482586929e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.2868993793315681e+01,     1.6656570803464776e+04,     4.6359532554209602e-04,     5.4764553418133182e-07,    -2.5876930229221479e-09, } , 
      {    6.3446775504308064e+01,    -1.6658194895435729e+04,    -1.4565567702329698e-04,    -4.5399892353821387e-07,     2.1576632877779900e-09, } , 
      {    1.3260105217531386e+01,     2.2901932271660848e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.8342832981571075e+01,     1.7762188989377264e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.9054643409212204e+00,     7.1154474160566197e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0294637025419632e+01,     6.3884117394021541e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.6455826072522939e+01,     8.9828105300987590e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1728230641082481e+01,     1.2596172072992227e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.1942306271147928e+01,    -1.0998759417582187e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4901475504983352e+00,     7.6371054612129037e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4955085848113470e+01,     4.8999070347662209e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.6510130959870637e+01,     8.4948565622217775e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4064188642927363e+01,     1.0831433686024029e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7116640751755870e+01,     1.5332852508395163e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.2917978323434955e+01,     4.1679505540028040e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    5.3962747452402624e+01,     1.5333557140311312e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    1.6135943509028742e+01,     1.6065835026273995e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.9280839272329597e+00,    -3.8062567700924064e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7579666214518817e+01,     2.2110485362396044e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -4.4942086116023461e-02,     8.2473363941036769e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7242215629080388e+01,    -6.9324060773528799e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.9373567432152132e+00,     2.1435767635739037e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.8532108675024164e+00,     1.4632759423960517e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.2403436687691574e+01,     1.5167782263846344e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3118967223190502e+01,     7.0052247450949881e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    8.6187962507171214e+00,     8.5891563387959777e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.7506708807096068e+01,     8.3900818293064167e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.2217817951599869e+01,    -8.2673010201438374e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.5268862994432148e+01,     7.1913152451005681e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4074946411730473e+01,    -1.2101885331440840e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2293724656055449e+01,     1.4094064487445938e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.3991096909939955e+01,     1.3578204765702885e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.2930529915294882e+00,     7.2128994039518111e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    4.2329091026192600e+00,     2.1116504337788916e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1238478494920791e+01,     2.4941192300330200e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.6991342792683008e+02,     1.5208449283131813e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5395146299803550e+01,     1.2434669159982757e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.5776635402183008e+01,     3.3524643797207675e+04,     3.2889566200998476e-04,     7.4133344792140254e-07,    -3.5185837720205683e-09, } , 
      {    3.4852067323838767e+01,     3.3523986532254778e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -2.1581291373745231e+01,     7.5583819694867398e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.2797940965957110e+01,    -4.9292971585320515e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6503032102744953e+01,    -1.1709518919003140e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.5270541943687865e+00,    -1.8556899362381617e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.4436020921485873e+01,     6.1524034034037304e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.7329524838052393e+01,     5.9416999518521070e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.8164571285640441e+00,    -9.6720571195303673e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.5262008421822443e+01,     1.5953824203232321e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7587299989749754e+00,     5.8400537241300062e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.3826676230834082e+01,     4.0581797661570534e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    4.2724146496809823e+01,     3.5408739246990779e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1535257342051707e+01,     5.3298514557890803e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2434185725725172e+01,     8.0957060092123047e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0910941834840452e+01,     3.3577284261942419e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -1.1050063686921781e+01,    -7.2194925704918242e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.6748081455504341e+01,     7.1376216220001634e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.6603009103171951e-01,    -2.4966886604467152e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    2.4230663086750088e+01,     2.3868600800851294e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2137632003747800e+01,    -7.3853025600717547e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.2612155497049534e+01,    -9.5229497877872436e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.4798669822136993e+01,     2.6748138154080843e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.1537641279146754e+01,     2.4889574665726905e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3557749407417383e+01,     6.0543504959304264e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1314104457972274e+01,     2.5744301521255820e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.0891116456407900e+01,     3.7635215444378802e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.0105749844521485e+01,     1.8307018989234326e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3977506480661699e+00,    -1.7378990439023557e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -2.2380069543443863e+01,     3.4758404868935600e-01,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.9970081050348742e+00,     1.5422097587337017e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5925548200803608e+01,    -7.2080025827361815e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.9483295957195121e+00,    -2.7642646990503672e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.4548115772719409e+01,     3.2055473220037368e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0106008204844432e+00,     7.0038020541612341e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.8807799745091735e+01,    -5.4264430685477055e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0321046743114465e+01,     1.6257802924157537e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.1404557962644049e+01,     1.1178053989891672e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.0116367760819443e+01,     1.5303864878796485e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8909496074129830e+00,     1.7232009991817729e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4299223882012681e+01,     2.5005261943883605e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    8.1310195890511192e+00,     3.3471346067520033e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    2.3483054528311065e+01,     1.9584297675628924e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6752568317017545e+01,     9.5541549996294307e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.4146657959750586e+00,     2.3874290628966875e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5127195423238021e+01,     9.9314282258067797e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    9.0222082693578418e+00,     1.7913998019541337e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1605632105037371e+01,     8.4710066680263681e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2481451122300298e+01,     1.5342720297662709e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6285523733262160e+01,     6.0464652089813480e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.7531714023670606e+00,     7.5930060827757425e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.2140044344924810e+01,     1.0032796113831573e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9601242401959347e+00,     1.3868996578425220e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2327182385786916e+01,    -8.3235708520891785e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.8983810201767337e+00,     8.4100464553465754e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.1644682909659458e+00,     9.1949154574394088e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8751163103374449e+00,     2.5450055179682000e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.1910125913640496e+01,     1.4911601791080624e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4369270458390808e+00,     3.7636151871499806e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.2181885402030304e+00,     9.4690591191383428e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7612097392044380e+01,     2.3452335279736191e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -3.3921192275494420e+01,     5.3287877943641288e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1114865903556776e+01,     1.5619195533642627e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.7944225861173244e+01,     8.7704142600831892e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7074686847971734e+01,     1.4989697766310805e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4158206459515368e+01,     1.0606635149554744e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7899609555482069e+01,     6.5236221304403552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1283095231911703e+01,     6.1343791952025231e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.3494805483689781e+01,     2.3007851514510985e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.0169937474060777e+00,     7.9640563884423964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6465255845542153e+00,    -3.5656234476862608e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.0121182068853926e+01,     7.5318327739794331e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.6552044205598762e+00,     7.1327078348373825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6947198218640359e+01,     3.2970431335504545e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.7052468561171807e+01,    -1.3544291459311353e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.2345336219324466e+01,     2.4358119005087909e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1020550704307444e+01,    -8.3338801333446663e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.5277192171109064e+00,     8.2234645757696198e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9838388211689960e+01,     1.5542289828247169e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.5902928271880182e+01,     1.5543218752120745e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.6548456332384582e+01,     2.2251368445828411e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.1847392317776535e+00,     3.5562595246677233e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6233643715167361e+01,    -1.0903556856568111e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8937735859120234e+01,     3.9936569395716200e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.1379669424497321e+00,     6.4437686695884968e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7726328169364386e+01,     2.4910823213002986e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0909396676217241e+01,     9.5798825653216522e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.7653555357442245e+00,     1.6656987015362436e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -3.6568627532633773e+00,    -1.6657778683538068e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    4.3943102373025695e+00,     8.8091123448594590e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4359711659878403e+01,     3.4713367479816385e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.5463872198426660e+00,    -8.3772512947672458e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.5989654316939088e+01,     2.2400214007040006e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.7792750083253637e+01,     6.3857903530227986e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.4720586142850713e-01,     2.4592639587052872e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.9837838544013017e+01,     1.3655530335207595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.9798435676099146e+01,     6.1103458827868690e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.8249336018410162e+01,     7.7000248101707812e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9759169305549953e+01,     8.6792610989329623e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.8126870159166595e+00,     5.9595705034532739e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4118358472763264e+00,     1.6919891936706044e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.3904397663663479e+00,    -1.8657350854284025e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6090341762842485e+01,     1.8458194387975029e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.3334623884038425e+01,    -2.5478375563397810e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1069677391641024e+01,    -4.9327671533090438e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3479913020356546e+00,     6.3790552083415457e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.5457145361366713e+01,     7.6762053130077256e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0154409450635193e+01,     1.5543073093734976e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6753077562153891e-01,     1.4952184064252628e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6524770414539809e+00,     6.5159254365079141e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.9935765216060208e+01,     1.5542435486632943e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.8351868767245421e+01,     2.2112967542308819e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.0369732082089897e+01,     2.7080047822265653e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.3916391639890051e+00,    -6.3486654064527102e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.7950911339736507e+01,     6.7622878847717948e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4931036041138704e+01,     1.6739866142618295e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.7408230030051683e+00,    -5.6595077801657476e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8678966076734067e+00,     1.3453696514881554e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.6116374962680347e+01,     3.8808993753706818e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4031500026503476e+00,    -8.9739196905794506e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -9.0570219633780322e+00,     5.7870941027049348e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.5510805709137623e+01,    -2.5070572639174634e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1868023734910125e+00,     1.1690096854266209e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3915509110979102e+01,     1.5598015111448765e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0845647472863416e+01,     5.0921519530988132e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6001798587048022e+00,     7.2501712159758408e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.7440993470169950e+01,     1.5542903775967947e+04,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -1.7222348223180578e+01,     1.5542604804399971e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    3.4057453970303504e+00,     2.0256704726660075e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1429269333234608e+01,     5.9650742789940168e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.7420797044714398e+00,     1.8762173108900250e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7116459319401152e+01,     7.2113227165256076e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0515567719878522e+01,     1.9566263991641594e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.3597942373991998e+01,     2.7174117373942863e+00,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.2391209538663418e+01,     9.2802855021076789e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1972522142067220e+01,     5.9747206921346024e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6963961577340175e+01,     1.5035103407985356e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.5788251396727169e+01,     4.6956849490432192e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    2.3169991993557360e+01,     1.5845637130141202e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.7665037018886061e+01,     8.4550416603238646e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.9895505151387409e+00,     8.2937989796687052e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.8104516891130515e+01,     8.9414569814822389e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5413614183966033e+01,     7.2018569406392471e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    5.5942299796120389e+00,     9.9248102950200587e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5340261955641008e+00,    -8.2933543537895912e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.0246252540409291e+00,     1.6156105542091387e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9034711844770840e+00,     8.4952447040153129e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4265272046042170e+01,     1.0983908375732091e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4592295109102714e+01,     8.3244336689651318e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.3900343850176469e+01,     1.6253545168397543e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4942585614271593e+01,     1.5247949097806604e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    2.3311598438333530e+00,     8.4339182736806324e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5448366712236052e+01,     8.4000133626139723e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.7892570806072431e+01,     7.2168030143920578e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3686015402911345e-01,     8.3635838697815470e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9143456060366152e+01,     8.7621343872047805e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.5844739485925075e+01,     9.9349098665763704e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7137740640939230e+00,    -1.7477206216339193e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.9538656388789043e+01,     3.9940335881211708e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.3784391213125097e+01,     1.4293765703719142e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7405117042221764e+01,     8.2637773536076220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6067866738237093e+00,     7.3549459488912016e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0358924050487246e+01,    -5.5974238302607198e+00,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.7807922878591302e+00,     1.2401216364956243e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0696831433553289e+00,    -4.2561664844940096e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3696842549462303e+01,     1.5487493468919152e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.3537318882034883e+00,     2.9710142921642782e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6399803408815977e+01,     1.3031825149841519e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6437325186181610e+01,     8.6178706943516718e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -8.3683834056643562e+01,     2.3243040078289705e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.7471784960980941e+00,     8.3150764436929912e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1960010625849876e+01,    -7.1207209183312017e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.0498616870315999e+01,     8.9434924984993158e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2322257281168874e+01,     3.4009471786667564e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4510189213548118e+01,     2.0791712307177855e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.0928353970183942e-01,    -3.1572856687433550e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5560533712413100e+00,     4.3776110417862952e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.8051517153707337e+01,     8.3313279705920359e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2187404258980521e+01,     8.3260548788582164e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.4660794943143891e+00,     2.5186987241622675e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    3.0062972007497507e+01,     2.9813145666761189e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.0273447008834893e+01,     4.0475859467148148e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    3.6646729192563058e+01,     1.6346845172925308e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4043751046245312e+00,     5.7199011106651633e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.1797096092483468e+00,     3.2720004311414141e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2254494190710348e+01,     2.4497123467638441e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8779864560634067e+01,     4.4034127599511703e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0357368548368839e+01,     1.6018765983149222e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1856012241564560e+01,     1.5332050838632336e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0997255159773054e+01,     2.3559926376568415e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.6335715999756104e+01,     8.4298967312043278e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.1046825144259905e+01,    -8.2274861182459244e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -6.5697410617713103e-01,    -2.5470117676851369e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6195415446356755e+01,     1.5152034520330704e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.7664043654193179e+01,     7.0045201131788381e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1792075385408085e+01,     1.9225058133438899e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -7.3388351141384518e+01,    -1.7625312724930495e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.7130301352582435e+00,     8.3978099231636243e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2789173215913655e+01,     7.6944837855931655e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9798989987445168e+01,     7.0045066860773841e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -2.1824281821991214e+01,     2.3225865133096686e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.5866448883722262e-01,     1.3376144253869430e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.1249329656218386e+01,     7.3078111812116913e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.6052400466090717e+01,     7.1588020441940262e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8463181703188550e+01,     2.3934724093375764e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.9713628222997954e+00,     1.4441243642117121e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5110445600289587e+01,     6.7500819599522074e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.0348112009104948e+01,     8.7423759107216229e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8692566483316732e+01,     7.7598871573363340e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3294717879313040e+00,     6.0983307788001157e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0843500276863509e+01,     9.4078953736765980e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    6.0777973870884324e+01,     7.2368104858170973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4196643073469536e+01,     1.5437527441228453e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6282705650506195e+01,     8.8817479485982803e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7377150097619442e+01,     6.2689670337084181e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5244469391921406e+01,     1.3180055746723765e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7749349603702175e+01,     3.1191446774790300e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -4.7321769787687359e+01,    -9.3136315067178288e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0876705024223369e+01,     8.8053249381528876e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    3.8828563948892629e+01,     2.2694684005669857e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5132995493764733e+01,     7.2152263269658542e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.7365392758262985e+01,     1.0166148709015033e+02,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.8917729272532512e+01,     3.6154074202975789e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.4697848664441807e+01,     1.6146789001620020e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3083548729041375e+01,    -3.9291232835421397e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3448472518598784e+00,     7.7249764792666356e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.6446154700979331e+01,    -5.8849094815024364e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0823660162045144e+01,     8.9389500664152947e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1893478737218782e+01,     3.1574459386904855e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -4.4251193248721485e+01,     5.8849441814501802e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8666404706917540e+01,     3.3926152603137270e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.8906379366538467e+01,     6.9166431001333422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1952218059419275e+02,     8.2871455543487882e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8982128874907126e+01,     8.5013462172276722e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1292965819181632e+01,     1.8100003264406896e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0012027061724629e+00,     8.3258378409886245e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.9009356166866240e-01,    -8.3315450084616277e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.9710111895489231e+00,    -1.5589653628886379e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.9062730914663252e+01,     2.4428407678584161e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.7482645093040233e+01,     7.2135984035220426e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -2.8258294297734249e+01,     7.2145273273956209e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8465937684115961e+01,     2.4018966516670844e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.7674040796440593e+00,     9.3319710361082834e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3004131012017083e+01,     5.4221114823735170e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5142185100748087e+00,    -8.2790280272905820e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    5.2023389013291577e+01,     2.4429112310500306e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.1559946297029370e+01,     7.2693236867236392e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.0291434972267801e+00,     8.3804970604292957e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.6482372772514484e+00,     2.3196737464038197e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.0720871725714128e+01,     1.5616341949906126e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3777307297250692e+01,     1.7342357413409183e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1451180034017820e+01,     1.5542358456096139e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.2484279905791954e+01,     1.5543150124271771e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.5085438897918230e+01,     7.2142123512428207e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {   -1.9577902795432667e+01,     7.2139133796748447e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {   -6.9381793467716335e-02,     7.3319555095736487e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4625223271542303e+01,     3.1729563908929096e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.5247999588461481e+00,    -2.5159575323603691e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    2.4237518884830340e+00,     1.4299257031292327e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.2032214981569966e+00,     1.5716255339612269e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.3460931904057098e+01,     9.0528092985250260e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.7965513816964602e+01,     1.9582573268421551e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8268416965676595e+00,    -6.1027390105821792e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.0086762749923928e+00,     7.1953273117907511e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1377069072845973e+01,     5.0544870981437930e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0498540197649969e+01,     2.7888882505770027e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5438448381705491e+01,     3.3367734796111698e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    4.3292582706079283e+00,     7.4556975189866250e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.4493048746632127e+00,     1.7197776448274617e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.3128318270687892e+01,     4.6052924382717523e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.3482745574751819e+01,     6.4749525475377027e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.6814739956339508e+00,     1.4976579855111351e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3655257384323903e+01,     1.6975152757970853e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7787224718354977e+01,     8.3820128378943919e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2026165492806502e+00,     3.3274950797002588e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -4.8462675794702230e+00,    -6.1292970449606059e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8272852460710340e+01,    -8.3459302047383426e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -7.1644484031557099e+00,     1.8369152139207261e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.6822569668623863e+01,    -8.0409088274134956e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6331484872165319e+01,     6.6610063415856785e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4955555265739831e+00,    -1.6906783925675215e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6494157096352822e+01,     1.5659001765861829e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.1341105347310567e+01,     1.1191525036667455e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1742809755277159e-01,     7.1140247251228666e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7798850837479407e+01,     7.2143816690098483e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4069383466610866e+01,    -1.0661771589849532e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8078701307787636e+01,     4.4783351886471100e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4509907599229255e+00,     1.2305656391078453e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4072193549520641e+01,     3.3354580600798421e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    5.9591187138534893e+00,     9.8215777315633204e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.1934145188160791e+01,     3.8235137096014078e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.6689643102862711e+01,     2.3474017503672116e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.9038534613828595e+01,     1.5613741965799292e+04,    -2.5425881396932824e-04,    -1.5387433550820208e-07,     6.6880532122741591e-10, } , 
      {    4.7010902428213399e+01,     2.1619646691245442e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6315058136891359e+01,     3.1884668430953348e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5787359295601291e+01,     7.1986849849093496e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2333291122065441e+01,     4.0489013662461424e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    7.9075401566401275e+00,    -1.1146180370624836e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4983435291228785e+01,     1.7332091100321140e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -6.1472252917076418e+01,     3.4911411969878736e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0943038915201328e+01,     6.9607273067509032e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.6058745003020462e+01,     5.9020907651814041e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3782412522162289e+01,     1.5973718334218367e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.2138307043568393e+01,    -9.0978563700287305e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    2.9165212361209822e+01,    -9.0932539703492585e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    7.6672562853380484e+00,     7.1257566973121075e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4569987356160663e+00,    -9.4601807306856026e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.9611084181706389e+00,     9.3631209247159604e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8400573506454251e+01,     2.6851320162512820e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    5.9376432631924473e+01,    -1.4591647590791817e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -8.8015064718259225e+01,     8.3355969546301276e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.0240427036975568e+00,     4.6385406457296995e+01,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -6.2179265598495448e+00,     1.3720188342076297e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3123249204566852e+01,     7.8313498237820122e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5423937342520304e-02,     3.1120053478965565e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1073184761455954e+01,    -6.7562855005928668e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.7603254399502557e+01,    -8.1022649648120332e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4543820083683894e+00,     1.3060675545188566e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7047186676369886e+01,     8.8981298584730612e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3641419924148991e+01,     8.2753700115558604e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2291322958954844e+01,     7.2137440619078170e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.5598354203191187e+00,     8.4323026961334654e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.0431762347514422e+01,     1.3571090486119292e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.0361044131755879e+00,    -1.1113619640079003e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.5028419852838066e+00,     1.5769224436823461e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    6.6069963707197097e+00,    -1.1146390814701033e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1413762013810292e+01,     2.3942433390524420e+04,    -1.0194606332797995e-04,     9.6536778921684358e-08,    -5.1753375644761854e-10, } , 
      {    1.4583397490598122e+01,     2.3558606838544405e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5430433288300165e+01,     2.4499747670157569e+04,     8.3460747925058777e-05,     3.1497443108239688e-07,    -1.5495081580573769e-09, } , 
      {   -2.3331317583235919e+01,     5.6811584368730337e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1149119504339104e+01,     1.5716221128671979e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.3033465027763857e+01,     1.6799358200680923e+04,    -7.1515884209197975e-05,     6.5179708766673123e-08,    -4.1760880863413212e-10, } , 
      {    3.0065663465607660e+01,     1.6652315777502983e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.2686879125642184e+01,     8.9761866794788966e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.4253314847943894e+00,    -8.9105983439453266e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.6852636480983790e+01,     8.3754118565280405e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.4878387734645759e+00,     1.4930446213462901e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0210341038194651e+01,    -8.8150123989458771e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.7755561121135635e+01,     7.0618847853411626e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.7893201621505789e+01,     1.3799824859422846e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.2719967614536664e+01,     1.6655433965568591e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.1066727319064395e+01,     4.0371094210018906e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    5.4282706844426968e+01,     1.5996260239349424e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0082067763634179e+00,    -1.7554758477351315e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -6.0781238098947945e+00,     1.1941131213134402e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3405437374136305e+01,    -1.5548183995216950e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.2054053519046420e+01,    -4.8341350903137936e-01,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    3.1925797983225198e+01,     1.3972971611251745e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.5959926142315233e+00,     4.1812287964300203e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.5774369689137856e+01,     5.1072534500530310e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6018688451683891e+00,    -2.6306793805933125e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.8122775367376974e+01,    -8.3303193481689341e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.9356305839966591e+01,     7.3274740967576217e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.2347587303129195e+01,     2.4013432325733876e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.8038075511239839e+01,     1.6849659318514117e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.9346799867962794e+01,     1.6505123118701231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2892161093207786e+01,     8.5551178846382190e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2687138057777354e+01,    -2.2645941894681209e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1255992897968916e+01,     2.6832141968590720e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.4837952461441439e+00,    -1.0551099822414951e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0376277432359785e+02,     7.6998758811415464e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.9833528341996605e+01,     2.3902098637218791e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {    6.8315353566114922e+01,     1.4293138996935499e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.2037099690993990e+01,     6.6252178663531276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.7569904355207395e+00,     1.2341806703839161e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8644206190608816e+00,     3.3705942174569932e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1496145778946358e+00,     7.4405330120983353e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    9.3955051870108321e+01,     3.2409679891914209e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -6.3507977049504007e+01,     3.2409693319015656e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -9.5569407715366204e+00,     1.7985103386850442e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.7036220042116263e+01,     7.6277539127263844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5870841257573812e+01,     2.3183619552838743e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    5.1744711988762493e+00,     2.5122027410605220e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.5154496778106186e+00,     8.3581883756888401e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.8043405333068172e+00,    -8.2991944737614122e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0276736585452412e+01,     4.1917440939269021e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    2.2621625691205370e+01,     3.1379754921941734e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5177195947836054e+01,     8.3685426107368621e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9985691337794783e+01,     2.9448510630478460e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    4.0523239391669588e+00,     1.2633873147992615e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2303189746492297e+01,     4.1641155923785896e+04,     9.1387650384808943e-04,     1.5024666865793185e-06,    -7.1180344660502071e-09, } , 
      {    1.2156917379710997e+01,     1.6278439632133342e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.1011705052972005e+02,     2.5195115781468066e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    1.9889432040615787e+02,    -2.5196131698544134e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    2.9583020620776871e+01,     1.9160125110749155e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5510460677262845e+00,    -3.3330537070711125e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0946991676946062e+01,     1.6675416533437579e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.2184033269985104e+00,     5.2647057605095342e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2029712327371286e+01,     6.9573146402178099e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.7227912561679560e+01,    -1.5540293291042133e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0089031461909983e+01,     1.5445851489621280e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.4856678197553057e+00,    -8.2565346578057088e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.4914405776007190e+01,     1.2574369830335865e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.1349268715468185e+01,     1.8914450781478095e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2659975296024943e+00,    -1.8558053348102523e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    5.9373763356263103e+00,     4.1784631346534989e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4916242460250565e+01,    -7.9537193983875113e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.4925748701763339e+00,     1.1330857312076185e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1166061867437030e+01,     1.6204843302732466e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1614195422897868e+01,     2.3982779403424171e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.4465091759314795e+01,     5.3571784596837715e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.7276196893546121e+01,     6.7457908345934557e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.5468621551410227e+00,     1.3329743588044086e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -9.5723939931526303e-01,    -2.5496416000013196e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.2831910915229479e+00,     1.7473909992348665e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.3716767715084217e+01,     7.1427172440883614e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9152555650739405e+01,     8.3957556068539816e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5376360862362596e+01,     4.8202772309703969e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5451175184577124e+01,     3.9889547627208799e-01,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    2.1598017842055324e+01,     5.0747086706462997e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6975686290347831e+01,     2.3819187963164768e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.8652312806761543e+01,     8.3263969384361171e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.3147368127536240e+01,     7.9333692551924032e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.9914486613379516e+01,    -1.1626472067100622e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -9.1780241318569324e+01,     8.9950829431901548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0984473248452682e+00,     5.8334947685302786e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.4058784539468157e+00,    -1.6789464203782811e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0875306380169727e+01,    -8.2396739111115239e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0196529681408290e+01,     8.4008481916445435e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.8573726846680373e+01,     2.9382387337953991e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -1.0751917163499161e+02,     5.2440490764023616e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5457035219144135e+00,     2.4669487863887436e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -2.7290231013106467e+00,     3.8941270695871754e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -7.2009447143768570e+00,     1.5542469062497617e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.7994812000596845e+01,    -1.5543039517870300e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6282369240004215e+01,     6.9913954884286559e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -2.5504511739946008e+00,    -2.9154400977910714e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2755265533484144e+01,     3.3261796601689312e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.4871426764232627e+01,     1.5541581352890818e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4380588011465349e+01,     2.2891803889624956e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.7802816560354344e+00,     2.5613670513754045e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.2027828567712305e+01,     4.0921849534052330e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.0553974641088619e+01,    -9.4346834856079913e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -4.8957388561886255e+00,     7.2129582737295141e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3808526084173138e-01,     1.2163299917287502e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1428082980202975e+01,     1.6621274498933246e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2790957749813252e+01,     7.6534845009771616e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.2993378199247312e+01,     8.3992326140973764e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.2054240629551430e+01,     4.0568643466257257e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    7.9303231963850500e+00,     2.5216953647983952e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    7.4734719133710072e+01,    -1.7982930744981786e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.7091586618250886e+01,     5.3294924262802779e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2043332827096069e+01,    -4.7287443054484629e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7400234430768151e+01,     2.7643921057498530e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.3776802558331553e+01,     1.8606498815271470e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2327008906208494e+00,     2.2141627481796702e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5348916962877482e+01,     1.5543927227477101e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.0382567955643331e+00,     1.6168984332445802e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1499363468913035e+00,     1.4984336483158388e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9975226671421851e+01,     1.2880649751100818e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7222015581333014e+00,     9.1267335737034791e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.3755113459596107e+00,    -9.3062491459544981e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.2599109111178535e+01,    -1.2738881668579425e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2939782620108078e+01,     8.5257893710307344e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -7.1279896647769533e+01,    -1.6657588564677462e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -1.7760557442498833e+01,    -6.2303231232788221e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6239185144203148e+01,     8.3063847256097579e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    8.1824732780646002e+01,     2.2661645323558536e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.8695941411581060e+01,     6.5105042192409019e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0071364965339775e+02,    -5.1280492861351377e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1695892837706678e+01,     3.1491647461260360e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.0679509536546163e+01,     2.0990178178879648e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.7914550266800788e+01,     9.1785460861487663e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1984168850222410e+01,     4.1555224226385249e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9266220004629460e+01,     3.2155255165789029e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -1.8251232334966623e+01,    -1.1971118524343347e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4326604054643504e+00,     2.5174293832028248e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -6.7588208803092149e+00,    -1.1677234346273667e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9545164298378221e+01,     8.3335614376130525e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8391667389045182e+01,     2.2146558513952623e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.3590654657722368e+01,     1.0563217451621913e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8440195854508431e+01,     1.3268413361389259e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1649449965764914e+01,     7.7474528828043567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1755289122173441e+01,     7.9483086449267585e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3309605527634094e+01,    -2.0371824858856521e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1730869399531706e+01,     1.5291719931315763e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -6.9713031400477750e+00,     1.3989304133923279e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6176414632060560e+01,     2.5140512404525281e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.9313117510942671e+01,     2.3349787669752801e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.9114151945454392e+00,     1.6283506704080612e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1009943219057030e+02,     1.4286237171666711e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5945114131229476e+01,    -5.8466126797297338e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6106369007299225e+01,     1.7802664331026252e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -7.2611176260590824e+01,     3.4513262950899607e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3758232174486920e+01,    -8.2385199253906230e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.3361838399234802e+00,     7.0912639243422664e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.6593782499099383e+00,     2.6484548605595683e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8643332288994930e+01,     2.5285795239340605e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4237299038616236e+01,     6.6081897801204595e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6178374992376625e+00,     1.1297075884573220e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.0448764398779289e+01,     2.0919714987264672e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    3.3008902771075967e+01,     5.5667279255144931e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.6444185271362677e+01,     3.8865522223843327e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.4115983144572372e+01,     7.2151674571881513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7219382972117288e+00,     7.2294407460083148e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5631452885480144e+00,     1.7505557938809161e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.5801011533133611e+00,     5.5403051537000920e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2311885273841817e+01,     1.6628532454103206e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -9.2002611393369662e+00,    -9.4493534583863675e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    2.5285494261026734e+01,    -9.4372865095964735e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {    1.2043179788740893e+01,     1.7233432682751478e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.3885842804707131e+00,     1.5537324585150962e+04,    -2.2283452439743112e-04,    -4.8670445446586314e-08,     2.1971756027884158e-10, } , 
      {    7.2685122235681749e+00,    -3.7578158167923021e+00,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -5.4497885904505852e+00,     7.7032287537373741e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2173449223918862e+00,     7.5633130016426612e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.5762974831785900e+01,     4.3696313324482053e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2045481041373648e+01,     3.1084797234901042e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5730364450374889e+01,     8.9412512662550307e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2948204780763945e+01,    -8.7363489975977300e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -9.9630288708147319e+00,    -5.4378592258891567e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.6883275369221117e+01,    -1.1161355279193231e+01,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.5046510079708053e+00,    -2.2660168804018731e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3159922189529247e-01,     2.1547101324254123e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.4112856859127005e+01,     1.8169948151197405e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1306126195020397e+01,    -1.7532454255673244e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6553470928842351e+01,     1.3011161633538009e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -7.6783658618930017e+00,     7.7381211987937950e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6347767948259012e+01,     2.1766655455459091e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.3269885982119099e+01,    -8.8266302179252307e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0045361577146192e+01,     8.3133135441756422e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0909229714747619e+00,     8.3440693052746083e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.5911623760446631e+01,     8.3288913982995327e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3764325558540222e+01,     8.3284914511507177e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6991358638540884e+01,     7.6841745043376786e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.4702927873060361e+01,     1.7697650513284923e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5514944401210222e+01,     2.6020788741605793e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4645374776024900e+01,     7.2178293509543346e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5881992880588793e+01,    -4.4054193617041005e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.1205910877538376e+00,     2.2128838203073698e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9259375561931162e+01,    -4.5098154163103391e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.3122786423642001e+00,     8.3847283969994078e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1312117687044854e+00,     1.7221700710562240e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.5516146718790587e+01,    -4.4828179095779342e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.8295422554387173e+01,     2.9831194610116567e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.8428859623198434e+01,     7.9041902273689102e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.7483447212251275e+00,     9.3242828299418215e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.6138091195911787e+01,     7.6740754490530253e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2908179897809067e+00,    -1.5878316222875883e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3887384293916725e+01,     3.1165690673696226e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.2674053001285824e+01,     3.2269966983539583e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -3.8725855248798204e+00,     1.6659331733331914e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.1342056977179560e+00,    -2.3705323838905038e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9308442522623501e+01,    -6.5087016922782414e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.5931656595349132e+01,     2.2734275250511735e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.2967194804278606e+01,     7.8568962342789985e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9060469393866742e+02,     8.3284953215774476e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9845739573676101e+02,     8.3288875278728046e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0317676149818464e+01,     4.7742539113860090e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    4.2415680698890185e+01,    -8.3913232475751356e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -2.0412922346408905e+01,    -9.4429913515149710e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.4813720212643195e+00,     8.3292350161606173e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2192481165768115e+01,    -8.3281478332896349e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.4389333042856709e+00,    -1.2763255779863077e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.0550024202839872e+00,     2.5048207424148313e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0861371555223236e+01,     1.3362449738869109e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.0737599104507538e+00,    -1.5195453983155630e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9141423726949018e+01,     1.3155511052502094e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1654663721652872e+02,     2.5798697091136262e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2239183979848470e+01,     1.6621300222107551e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {   -9.2753255031016465e+01,     8.0725290165707956e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9260092356359237e+01,     1.1073279256976311e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.1449781205701317e+00,     9.4444235113838440e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5932499320965604e+01,    -2.9973622453327334e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.1271538204299400e+01,     7.7802373005314412e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8558489203780604e+00,     7.8488795366507338e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.0156629072265478e+01,     1.2385468621440605e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.9262104894421057e-02,     1.2667654575495581e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7585194671115506e+01,     2.6767707618778048e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -3.0426373625383132e+00,    -5.4565133606974996e+00,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.0164085918669997e+01,     1.0919820600224697e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7600105640960928e+00,     2.5779933224712904e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8367063221914788e+02,     1.5528739820374567e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.5345198697215852e+02,     1.5556768759993351e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3746818148854874e+00,     1.6007745356426858e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0995888209177598e+02,     1.4978534480645811e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.3472036809981461e+01,     1.1253881843696425e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3101464033489840e+00,     1.7102218930633528e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.4204262546768906e+01,     2.3937856344884358e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6376317121272539e+01,     6.0968044738763556e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -7.0254500914593521e+01,     1.7403404171711103e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9815699456224500e+00,     1.9863683756967086e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6030012140502521e+00,     8.8613700966037741e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7994622670868694e+01,     1.0600076193955016e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.7842240319202034e+00,     1.7087582327453914e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.4384485516269931e+00,    -2.3472146247476594e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.8100982668459027e+00,     1.4906968538569359e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2181908369903260e+01,     1.5855593166548639e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5954190022919157e+00,    -8.0999637649722972e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    7.8501771085043508e+00,     1.3093513673935340e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6216972130642586e+02,     8.3278311502290144e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5745861216192208e+02,    -8.3295516992212360e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4793976038137064e+01,     8.8238308532840165e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5632082897573847e+01,     1.6415537278497970e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.9306861946837550e+00,     6.1059932617922650e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -9.5564992866283038e+00,     7.2137776377724922e+03,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.9470160237067919e+01,    -2.4414677079201829e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5846605381772974e+01,     3.9491752266105163e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -8.8197754740856666e+01,     1.5478666514676564e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9272915445670545e+01,     3.2864493141082159e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -9.2330804560614528e+00,    -8.4985466142364066e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9487685677651626e+01,     6.4406401643952368e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.4834295958178647e+01,     1.5906216389173544e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5639257428348628e+01,    -7.2143480931451732e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.0752642762303850e+01,     4.7743243745776250e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    2.1135190887246228e+01,     1.1361525493753939e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.0503205637407255e+01,     6.1020641385087238e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.0623550825433643e+00,    -1.4326326499010790e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.0630790719685663e+01,     5.8895325244037440e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.9299920158380448e+01,     1.7523237019198379e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2792346854372006e+01,     1.7286019066972243e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.8088697439637670e+01,     7.8639425534404984e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0420694453416644e+01,     5.8902371563198922e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.1098383609294899e+01,     1.1659449978795074e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0377799710712768e+01,     1.5052024088794138e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.1213214049226757e+01,     1.6266233707313755e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9753251792026166e+01,     2.3250616296432599e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.8118203708266961e+01,    -1.1151808487987769e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.3870481320148151e+00,    -1.3614981032134455e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.8771992402558126e+00,     2.5031330430065944e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -4.7358728056261299e+01,     8.3282686137129404e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -5.2069837200765100e+01,    -8.3291142357373137e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.5379511625389533e+01,     2.9531473644951340e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.3993219695201375e+00,     2.4714369837732622e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3606686358549508e+01,    -1.1140762697338100e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.9411211051432943e+01,     8.3313145434905855e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.5275323946159773e+01,     8.3260683059596668e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0774609980383521e+02,     5.9575457469415860e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9129840164701832e+01,     1.4787838742441872e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.2885021981294766e+00,     2.9775732477640058e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.8622181162550753e+00,    -1.8767602813933245e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.6784679764839979e+01,     3.9048410983089416e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    4.5822658762792571e+01,     2.4739970947463105e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4219567107912964e+01,     7.6316575231666329e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5274923111789896e+01,     1.4898447196813930e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2817013610966352e+00,     8.2949043674251006e+03,     1.8057610349377928e-04,     2.8795091649735135e-07,    -1.4153602124943682e-09, } , 
      {    4.5705922165928348e+00,    -8.3624784820251516e+03,    -1.2404939778891725e-04,    -2.1287131236242152e-07,     9.5731794285570074e-10, } , 
      {    5.2202000396473078e+00,    -1.0785202087492853e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.9227305931683745e+01,     1.6574899556282209e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.9900718338824628e+01,     2.7949176213600722e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.9000343614215353e+01,     4.1645758323465358e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.3681366978293884e+01,     7.1791704204024118e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1278787185345593e+02,     8.3309859110141351e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3482257423104000e+01,     5.7676117876810451e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1912528269959747e+01,     8.3961153037013628e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.5085312190246349e+01,     1.5369287451695938e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.7158914609785514e+00,     1.9009222634611151e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0408066973265983e+01,     6.7123120373466472e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7123114448806525e+01,     5.3145463525274654e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    3.3893503041829129e+00,    -1.6582392685687428e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.1812478674351983e+00,     5.4470327544370193e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.1627877673740699e+01,     6.3765581305406422e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.8432200123968297e+01,     9.7637044654601523e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1669825881867361e+01,     2.3923703466653402e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3074747264116283e+01,     8.2834372544596517e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3418721257530823e+01,     8.3462635168861852e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8722936121946454e+01,     2.5597108862454064e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8812239341201451e+01,     2.2127798225130373e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4552682786421203e+01,     3.0630702044750684e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    3.6794897419596682e+00,     6.8122254270527928e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -5.4560927464268559e+01,     7.2117634006164853e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.8351441625592848e-01,     1.4710311684972639e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.5010031767925554e+01,     7.6761109660701004e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0215398980778613e+01,    -9.4436486164678681e+03,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    5.6914421153368103e+01,     8.8220122757885038e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4035267434094026e+01,     1.5176965268180329e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.6361793425254135e+01,    -1.3347973920554068e+00,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3784664058178784e+00,     1.6655960158516500e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3737526308585384e+01,     7.4470482809716541e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1187283577748730e+01,     1.6658805540384004e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.8704330957060108e+01,     8.3502669274085038e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.3415440101563902e+01,    -8.3071159220417485e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -5.4580705446689237e+01,     2.5407912371782834e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0245455647848623e+00,     6.6277037509021166e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4120930509517709e+01,     1.5906764135628141e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9390317627410312e+00,     9.4923352227648993e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.1322291296783060e+00,     2.1915313083780302e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3446947814338127e+01,     1.5505016931069746e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3637428623561643e+01,     2.5758998302361899e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.5624569754832670e+01,    -2.2119841217657304e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8329478936408619e+00,     8.6933264610078550e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5994069345860062e+01,     3.2489316409260759e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.2864956030521570e+00,     1.6316176991247554e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.0806901563289086e+00,     8.2829123866747868e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5430680881826508e+01,     8.9297690477064007e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3331904034821196e+01,     1.4534063925340048e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.2296373865975170e+01,     1.6968137621760179e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1325943573389758e+02,     1.1128975907878666e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.4651883399257471e+01,     1.9248512024949821e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6218858181281091e+00,     1.3880017205147582e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.1238437943251309e+00,     1.6683691240076809e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5648356169831962e+01,     2.9798744476037400e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.2044770839918795e+02,     1.8085449898044317e+00,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.0188790850884487e+02,     7.7644716151869779e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6182352842249458e+01,    -4.2187136504003502e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.3143249224964428e+01,     3.0023849118312814e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.9420859537868147e+01,     1.5536142890630666e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8822528365283681e+01,     6.3434105944049111e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0970415228949398e+02,     7.7708691865539458e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8988244068561463e+02,     7.7718851036300130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.7971084503725777e+01,     2.3869714746430654e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1704880406654024e+01,     2.3878923881478142e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.2208841322191688e+01,     1.5540454825341611e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.6144301262001463e+01,     1.5545053755026303e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5140743075773742e+01,     2.1993522836615084e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7049250337581384e+01,     7.4155680533084223e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.7971357265013548e+01,     1.6631074458823696e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.8023384923957821e+01,     1.0545944454855056e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0496135913183995e+01,     8.3390094632077708e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -4.0342516759366923e+01,     5.7121968638077487e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.4755380798614780e+01,    -2.2979975511290831e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8963590716867344e+01,     1.6192391783207353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8131507764689553e+02,     7.2000483956494409e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.5580754154440996e+02,     7.2280773352682245e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.4510448715341102e+00,     1.0240836651764992e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4900618618973127e+01,     3.3517756209131505e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    5.2702833015893987e+01,     1.4547570436611402e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3191700154839163e+01,     1.0234277696165265e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2598475715731148e+01,     1.0373978366518370e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4582234291454675e+01,     1.6661196141263386e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6670194947771918e+01,     8.9937587716399557e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2645829325221545e+00,     1.6653569557637118e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.5563884069226624e+01,     1.4602927366797745e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3740436583071862e+01,    -1.0069326369197228e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.5310412628321556e+01,     7.8177778109173223e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.7112483452435594e+00,     2.5845522780710180e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.8057111210932373e+00,     6.5703361693740999e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.6371613341857667e+01,     1.3188783557593435e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4928871296110074e+01,     1.6538345695400658e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.4336501132227561e+01,     2.5596310060075166e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8159278827945229e+01,     7.9482896933545180e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -6.7694260319989787e+00,    -4.7500330638188134e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.2543778148867240e+01,    -1.8047208357985426e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6316668971408262e+01,    -2.3715960453154548e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2002268456437587e+01,     3.2306075334056593e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.1837187807251638e+01,    -8.5468773889392760e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    5.0330662764842344e+00,     7.2086331604258357e+03,    -3.7514727503877934e-04,    -2.9908155987647273e-07,     1.4060566379538760e-09, } , 
      {   -1.9948442887714219e+01,     8.3931275095351830e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6190336405664869e+00,    -1.1249378405217813e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.3992110509854768e+01,     8.3931240395404075e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3441622993316047e+00,     8.3448408259524331e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0054092469377062e+01,     4.6848330169521741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1103065249071840e+00,     4.7950139695809165e+04,    -1.0810608171947362e-04,     3.6840506813832525e-07,    -1.7633643209316043e-09, } , 
      {    5.4413826987360522e+01,     4.7954742095488618e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    2.9334843647748119e+01,     3.6130346836564087e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.2619689006296369e+01,     8.3362435911002358e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8087670775239655e+00,     1.4197535123067109e+02,    -3.7614138549189446e-04,    -4.3564252009309976e-07,     1.9550693467159368e-09, } , 
      {   -8.5350419713791979e+01,     7.8422723989305359e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3792793896674960e+00,     1.7127397229351824e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2722030640964027e+01,     1.2065670284694112e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1317684214658442e+01,     9.5234868179516798e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4311974481669262e+01,     2.4849003436138453e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    5.9037154285720831e+01,     6.6259224982692767e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8986331123232652e+01,     2.9917910923890428e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.8854849058058164e+01,     3.1086219925834794e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -7.2495396667910974e+00,     2.3862585559469622e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.5896108751104677e+01,     2.3880305870348548e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.4753994376498731e+00,     7.5606831693264767e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.1204441618204520e+01,     7.2373693366206371e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8340245460897130e+01,     3.3543502834439452e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.3258368394534003e+01,     2.3863967548340028e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9264064118612005e+01,     8.3538363538446629e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.5018199153451857e+00,    -2.5183004501683721e+04,     3.7584307963390776e-05,    -1.6666439915502520e-07,     7.9674280353541145e-10, } , 
      {    6.6178221926696583e+00,     1.5558502033699599e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.9156366239511783e+01,     1.7991896854858736e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9523671823024479e+01,     4.8713561583400970e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.3736909268450290e+00,    -7.0099431046611026e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.4242329077662561e+01,     7.5272303742999611e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6006181148247158e+01,     3.6257534337616093e-01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.8266170083259361e+00,     1.2090723363177849e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.0118008248142196e+01,     1.7380862266580050e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.9124671374154554e+00,    -1.1043192780108056e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.9107237465819207e+00,     3.2998179288612562e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.3256777940406302e+01,     2.5678816209033590e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0792025960189548e+02,     6.3034310206261887e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4299531394355782e+01,     8.5744863660176015e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.4726569301507254e+01,     1.5808667029850710e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6575440931284678e+01,     3.3332789907101709e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    1.7909133789066637e+01,     7.2876591948597088e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.1748172582855744e+01,    -8.3905112015896557e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    5.2101564099085858e+00,    -8.2588615808197519e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.2928187047428439e+01,     8.7282019401171219e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.2006212630837737e+01,     9.8394433120064878e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2342864987753675e+01,     4.9515382516684058e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0812436073297420e+01,     2.3638460299396262e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.7465921906889310e-01,    -3.8838090311045316e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.2398105741217805e+01,     1.5265780945377001e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.2434462493577740e+01,     1.7301232462784224e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5285101818391585e+01,     1.4547162884967263e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    1.5724751316510309e+01,     8.9510989556842524e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4341053734631799e+01,     2.5302862520880248e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.0503204488020046e+00,     1.0904054914880233e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9627789019124453e+01,     3.0986181785498829e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -8.3431628457788776e+00,     9.7249920058012722e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8409958689760309e+01,     9.9030765033368116e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0407717813562256e+01,     1.6884956523482178e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.0309015173672019e+01,     1.9467013015114558e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7790444061382541e+01,     1.4852313555165483e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.5751078356963319e+01,     2.1426422181991586e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0607428055425423e+01,     1.2692344084134656e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.5428819716108915e+01,     7.2529113649090668e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.8577901058866004e+00,     1.7589356526363222e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9493800304705729e+01,     3.1357213016810674e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.1980964061637117e+01,     7.1407683602067027e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.6315031428046922e+01,     3.0282399359442294e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1344335057692543e+01,     6.4990142314852519e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.2948045386924655e+01,     7.9497482074082163e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1977559452657999e+01,     3.3542339372932431e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    4.3786445083560022e+00,     5.0103948264530868e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3639047614688678e+01,     5.3445592213613927e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3373231280033114e+01,     3.6036963409828593e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0204729128153190e+01,     1.1231339938565368e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0261024025768188e+00,     1.5140916851777918e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.6062786315055721e+01,     2.1834263229023432e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7619006813521807e+01,     2.4287876911109863e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7062560271990064e+00,    -4.4598059111805833e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.8635923177586040e+01,     3.1567869958662795e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.9861226025331300e+00,    -1.6692327738460237e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    6.7831162659605226e+00,    -1.6652973622281184e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -4.4144677902690255e-01,     2.5226802143520585e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -4.1657854896152777e+00,     2.1849723527783035e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3782034858836880e+00,     1.2274633605802592e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.6308334775863088e+01,     1.5738203838235688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4629150887346974e+01,     1.0976908498889467e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6294151181304542e+01,     1.9144377367233516e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.5109083774511078e+01,     1.4288216662876570e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5680456286078112e+01,    -1.1891912911148484e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0679839571671190e+01,     1.6875099358266838e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.1215784431716081e+01,     2.9828541094318756e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.0976214567307085e+00,     1.5117539874070462e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4303872802784730e+01,     8.2904408359384943e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.2323623170034502e+01,     1.1293696745594338e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9130933582588497e+01,     3.1714168481371533e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.4226780163563014e+01,     1.4285810487616425e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.5683974027091363e+00,     1.7919513830853161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7259814143211543e+00,    -6.0779446853741668e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7175898967185784e+01,     3.3663364616970011e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.6564843513782508e+00,     7.2102963799633308e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.8367345215329358e+01,     1.3192840468804430e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.2462032329165647e+00,     7.7521894753837523e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5430499150955553e+01,     3.1388358050726401e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -6.3470929331725774e+00,    -4.1992777397853943e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.7329927558064142e+01,     1.4130705965592173e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4058119792877376e+01,     1.5178119253901230e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3571950629901517e+01,     7.2954178960802828e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7339342631599601e+00,    -1.0070916223570086e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.9157459215246604e+00,     7.0901004628352448e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9044007089814219e+01,     3.1802708928397242e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    6.7151355302653187e+01,     1.5639657090746639e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.7997718822499252e+01,     3.0457194281771648e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3919709121065937e+01,     7.4247568412243363e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1643467547544983e+01,     1.6711455474053866e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.7493650803140657e+01,     2.4980174220088960e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4003200666634648e+01,     1.0095721535746939e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3580422476703497e+01,     1.5508261517214116e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    7.7198240934318143e+00,    -6.9481233277965721e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5916455430980729e+01,     5.5486789821591201e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.1964797081228609e+01,     1.3815565889387763e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4356691957167946e+00,     8.8194216261149468e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6170762565525166e+01,     1.7031258994819895e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0908340324378759e+01,     1.0937836342383198e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -4.3470181901428653e+01,     1.6928322719862263e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.5089158483044951e+00,     7.3537635358119396e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.6063610340941494e+01,     3.1338844139075845e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1360879794881836e+01,     1.4914436182934605e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.5829161119991710e+01,     9.6233905282356991e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5777576924314971e+01,     3.1781731336377463e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    7.0743612434820848e+00,     7.6481260879352676e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1592472217834433e+02,     1.5544023879642549e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8570607693135494e+02,     1.5541484700725368e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5865043999759481e+01,     2.5125733961986127e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.9811730902479653e+01,     6.0682559026780527e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    7.3784747889930330e+01,     7.2163623303011800e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7851561098413548e+01,    -6.2991522264475198e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.7666718008646839e+01,    -8.3294127808839630e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.6392771907080768e+01,     4.0216008639566899e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    2.3478290074983899e+01,     1.6897585237590676e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    6.1976505394413168e+00,     2.3623539861234152e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    4.6702462916044965e+01,     1.5616048795436087e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.1522582083338815e+01,     1.8700039958927315e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.2306310356535377e+01,     1.8779464047495931e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.8513827578980710e+01,     8.6610216997184270e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.2982608425012671e+01,    -3.1851921518329800e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5150058649813722e+00,     3.0377571191526549e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.1286642763352710e+01,    -2.1446024163082257e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.5282796241517715e+00,    -3.0288283995724350e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.5485843766964663e+01,     8.8869637671650707e+00,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2762857756350572e+01,     9.5257880177914139e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.1065504718868766e+00,    -1.5119080398181268e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.9036699610742087e+01,     7.6868773284082290e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6160372415179715e+01,    -7.7092439951190445e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0005748784695157e+01,     1.5186151141581175e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2584327162325984e+01,     9.5639553204878575e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.1606429335026212e+00,     1.4007162957682056e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.7330648630523457e+00,     1.0241202187353454e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2721536790262121e+01,    -2.1272872460226372e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9925049189540648e+01,     3.1155398358387891e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -5.3999394517510240e+00,     3.2414753677412205e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.9889908731596417e+00,     6.1369735495122950e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1384803425411324e+01,    -5.8308801955821427e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5074052341177875e+01,     2.6152019272054888e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2019685841847592e+01,     6.6600542856023321e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0818928933848504e+01,     1.1152414766685693e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2697433162043170e+00,     1.7381250408373584e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9212150306071450e+01,     1.0417749200027531e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.6640369541507134e+00,    -1.8205161289070020e+00,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    9.3778754263180879e+00,    -1.1779069377329727e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.5164717811150314e+00,    -3.5219207050260834e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5544720103229199e+00,     1.0953481247109576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2337277137830839e+02,     2.6152748046596912e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1876218065550354e+01,     2.1500913331018583e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.4568641068947166e+01,     2.4058453275608750e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.5510748491555418e+01,     2.7473068791958642e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.2199810489620312e+01,     2.5365977332668910e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3315682600473728e+00,     4.2988325649586841e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3008024996421131e+01,     7.2493999363943667e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.6818835534713592e+01,     1.3870419269358970e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1977273179820607e+00,    -1.3220357868273902e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0215440045234072e+01,     6.5688945337191062e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6170803689969809e+01,     1.3799106800405247e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.5947087101699715e+00,     5.3053643360902345e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.9823087913565313e+01,     2.7244341132205809e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.9537388414565584e+01,     3.1614489370544372e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9586923605943934e+01,     1.6878929925828346e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0271347152814519e+01,     2.3274316533378023e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.6507561751969476e+01,     1.5762308421388925e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.3646436058872617e+01,     1.4359433625345002e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.4714931291487268e+01,     3.9902822179153532e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.9924811629882210e+01,     1.6179698373612930e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.8028353837851430e+01,     1.6628517194735155e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0155167814866896e+01,     4.0526198732043158e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -2.5738749063662105e+01,     1.5489432877014693e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.2160032953373104e+01,     1.6634076378288450e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.1435679894188064e+00,     1.1661349297708422e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9872778036553996e+01,     2.2539579483645612e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1340424226194976e+00,     8.8774385442173643e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.7718657633686137e+00,    -8.5731857795658343e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.6848343013536038e+01,     8.2892288387852896e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1165952738289104e+01,     1.7006617726266035e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    3.1681337967916988e+01,     3.9085613385212724e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.6639342222989022e-01,    -2.8444467923786751e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3463559803942829e+01,     5.9349057902366903e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4888252038217164e+01,     6.3975357225604193e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -7.5008272988697655e+00,     1.1548110243441415e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4196626161395180e+01,     4.0531458396675524e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    8.7059487779835809e+00,     6.0596194042946272e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.0534717530680462e+02,    -1.7428380927379972e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.6388983343307146e+01,     1.2734169435134390e-01,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    3.3517164849077702e+01,     2.3731760303924020e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.1320910171024337e+01,     1.0880005698326784e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0104342185222542e+01,    -1.6266538649635186e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.4825224112606428e+00,     1.0511033945187246e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9921042309145658e+01,     3.1067484372166713e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    7.3702907450551720e+00,     9.0617737426860294e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8744247846285489e+01,     2.7344200092333303e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.9658636048558293e+01,    -6.6285826471175824e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.0301776568041888e+01,     1.0229156208150185e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0681811562752867e+02,     8.9564910512326351e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5151506644965396e+01,     3.2119955046305906e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.7744380181027255e+00,     6.3040679992353907e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1980240552831745e+02,     2.2756309197104758e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {   -1.7978418744679149e+02,     2.2757325114180825e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.8093501620836275e+01,     3.3120119282845371e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.2960189444952874e+00,     4.8716847908165464e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8990652829971246e+00,     7.6827518134039256e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1165273688207985e+02,     1.5543763285817306e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4637096977676489e+01,     1.8247104578121711e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4768139396830222e+01,     7.0540273112786654e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7263341474417224e+01,     1.5592417687618501e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -6.9132685133307064e+00,     1.0118012975494259e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1278325566681495e+00,     9.2674920115490640e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8885734836387265e+00,     8.5169109825779960e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.4417477994264729e-01,     1.9470662787274097e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.6574308412369824e+01,     9.3657524636112485e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.6149023342173528e+01,     6.6014877645608776e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.4074383292808879e+00,     3.4006134826791651e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.1342793319811911e+01,     8.4383275008497039e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2199192636076106e+01,     6.5983745187356681e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    8.7845255402338527e+00,     1.6919901412492167e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.3210476961875180e+01,    -1.4968328175995612e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2773960655264620e+00,     1.4285427774625996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3413161604315619e+01,     2.3453039911652337e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.1334205793832350e+01,     2.3844215752844815e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2009591734049257e+01,    -7.1561874712458912e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.2445673361561109e+01,     2.6758563498918232e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4670899366914369e+01,     8.6813720113154232e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.0251665994794166e+01,     2.3087704975362845e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4735815959489905e+01,     1.5842475255349185e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4701967360046170e+01,     6.7097176830368753e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.4768077273078120e+01,     1.4759013550637717e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3077509472958859e+01,     2.9850223318254684e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.7181774583017340e+01,     3.1885603371565381e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8604076612476501e+01,    -4.4650607232283801e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5427125909868371e+00,     7.2489553105152536e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5015776972631794e+01,    -8.5342389927787372e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7545048152110175e+02,     2.4325289631394753e-01,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.2705917983196922e+01,     2.1249167626683506e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -2.1045883253535280e+01,    -8.0247127037999107e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.4932179574890853e+01,     4.0149885347042422e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -1.0302655084703936e+01,     8.1231438566715142e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2622643287346627e+01,     1.7109862462053443e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0207289281567050e+01,     4.7964337296441554e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3678900435345859e+01,     1.1947953932265536e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.2069385641454058e+01,     2.6084733570524264e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8728446958960660e+01,     3.1713816165413460e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.1147386887380293e+00,     3.7544263992340693e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5980578292627186e+01,     1.5596075703353225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4787970916833693e+01,     2.3858994195383970e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    9.6535623094504874e+00,    -1.3155257893948411e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.0968470542641878e+01,     1.2936722618351099e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.7373178517795189e+01,    -5.4535917100367469e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.1471084233194264e+02,    -1.3662153851316020e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4990554262711516e+01,     5.7375801910543805e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4146040045203875e+01,     3.5801760216683768e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.9194268903266341e+01,     1.5892357561944114e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8404563341873157e+00,     6.6593429401354560e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.2248777816926903e+00,     5.2620759281933497e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5821267471850700e+00,    -9.6951344324824072e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.7408956856733959e+00,     7.0901099386213646e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.2186312913371253e+00,     8.3271908429756731e+03,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {   -1.2929740435840934e+01,    -8.3301920064745791e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    3.7001782716021879e+01,     7.5111692576759215e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.2721282223455319e+01,     6.2759669105510438e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8762539047191549e+00,     2.3237104655468593e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.0509284660897672e+01,     1.3422544919694774e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8371310905851654e+00,     2.4042297980448973e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8864468477174412e+01,     1.6612500875605070e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.4549784926112281e+01,    -1.6884205312047830e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    3.1016561154981321e+01,     1.9553146080442137e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.2618108668095296e+00,     4.8855218789244740e+04,     6.9537563198336031e-04,     1.3160024966877799e-06,    -6.2404247405057249e-09, } , 
      {    4.1461987434214311e+00,     7.8347438668781051e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1460745737706141e+01,     1.4423194698761745e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.5804840368948334e+01,     1.7109157830137290e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9216826621974064e+00,    -9.6720476437442467e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.4695422815692350e+01,    -1.3410987059057961e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    1.7537717016873497e+01,     6.4122542309849787e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -3.7930196121363245e+01,     1.1995272714263183e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7677649454314714e+01,    -1.6727246904295913e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.6617050003951135e+01,     7.7160202149056377e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5873912340459825e+01,    -6.4537486658518992e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2391599695476041e+01,     1.3091328467498271e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.1754196334715434e+00,     7.4831309083143533e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.2988405034164245e+01,     7.7153042778416948e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.6581775886533903e+00,     5.7391579690059916e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6406661117098043e+01,     3.3280984271397538e+04,     6.4017823691073250e-04,     1.0385678676728479e-06,    -4.9199377172676819e-09, } , 
      {   -2.4377754609171598e+00,    -3.3348547126403471e+04,    -5.7832376822005349e-04,    -9.6472104776624343e-07,     4.5707749041325938e-09, } , 
      {    1.7467623122307216e+01,     1.4252368952184028e+04,    -2.9933123892224414e-05,     9.9637550263324720e-08,    -3.7443130219451685e-10, } , 
      {    3.0133855261784408e+01,     3.9380418577590077e+04,     5.0863742539925859e-05,     4.1522837345988422e-07,    -1.9783791885036835e-09, } , 
      {    2.2835385623456606e+01,     2.4465966242654602e+04,     1.1438798227039829e-04,     3.5189784103569913e-07,    -1.7240895646249208e-09, } , 
      {    4.9414589237729167e+00,    -4.5254517651484684e+02,    -4.0440473834432548e-04,    -4.7318232216056464e-07,     2.1840904815352705e-09, } , 
      {    3.1840944130489346e+00,    -1.8157328912875026e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    2.5348256311726487e+01,     4.9033288027412222e+04,     1.4132202449059509e-04,     5.9179266798316614e-07,    -2.8155554530436300e-09, } , 
      {    6.3151213424628612e+00,    -8.1529233863973686e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.9616383580933654e+01,     1.5613747595596353e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    7.4606690124659174e+00,     1.5471760984771563e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8361360587837586e+02,     8.3544918568136491e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.7890249673387206e+02,    -8.3028909926366014e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1331692389069595e+01,     3.1879385910530596e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8841101556283633e+01,     2.8036942027191515e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.4968450724976177e+02,    -2.0832844365007759e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.7417921236871763e+00,     1.6075460670862005e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.0132429686485409e+00,     3.7882078267370343e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.7954861974627718e+00,     1.1881612636109603e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4391683955555688e+01,     8.3284763429835202e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.5389890681144935e+00,     8.3289065064667338e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3456777777558749e+01,    -6.7102068801278328e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4114935122512579e+01,     8.3289882588395540e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.2622677189560889e+00,     8.3283945906106983e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4334494357051626e+00,    -1.6638204655528145e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -1.9694652830465209e+00,     9.5703502616780988e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4194492306848659e+01,    -4.4881973845181619e+01,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    8.8179632984093885e+00,     8.3047671624375034e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.2811063410361445e+00,     8.3846470581716349e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0490770197582929e+01,    -8.0442779811321543e+00,     4.6359532554209602e-04,     5.4764553418133182e-07,    -2.5876930229221479e-09, } , 
      {    2.2863799150526951e+01,     9.6251483050418283e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.9703086125661589e+00,     6.1479941762346598e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3551031568475365e+00,     8.3286914247251261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1637181634246444e+01,    -8.2610038190893756e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.8212631681190903e+00,     7.0021864766140670e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3609984289980638e+01,     5.3362343473633430e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0581700957499294e+00,     1.3604862437836138e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5942529376483025e+01,     7.4247663170104552e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3376630561039917e+00,     1.6970221725814932e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8341254361965568e+01,     8.2081594725080067e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.5477000083866933e+01,     1.2527203467456551e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7845552956089186e+01,    -6.3323861720146306e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5448583875140429e+01,     7.0497695555186710e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    7.1336817374774242e+00,     3.9334570201297218e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -3.8271930920534158e-02,     6.6518785729377596e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2023691399066141e+01,    -7.1032278498208207e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3523478010574582e+01,     1.5489794668758888e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.7861641451080160e+01,    -5.6880426738879214e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8918088964937191e+01,     1.0370610388861223e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.1981354609567333e+01,    -9.7262668907345142e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.1495871156617792e+00,     3.5011060998941884e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0502695817773864e+01,     5.7319624956472705e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3673604537984438e+01,     2.3871435192705278e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4175611340756925e+01,     2.0455463204305383e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4972885489757813e+01,     2.3871456237112892e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0295193553300264e+02,    -9.4432219324175803e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5914518946271956e+01,     9.2741060600265573e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2620761448969375e+01,     1.3976353784682005e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.3149584194363754e+01,    -9.4434180355652588e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.2764512705071695e+01,     8.2718092412875667e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1299440870315653e+01,     9.8850275599814358e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9569207150574748e+01,     9.0764437154644056e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1542062551880873e+01,     1.5892004426367788e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -4.9936325881549726e+00,    -3.7260746767640994e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1778867187582556e+01,     6.4809805474979285e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.3112715912296594e+01,     1.9407771632541473e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5574146364145776e+01,     1.0999656119247729e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5464967140147767e+01,     1.6160041248507137e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3542159241660528e+01,     1.4985433297000085e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.1406824649052794e+01,     8.3291360506042402e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6957155045489962e+00,    -8.3282467988460121e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -6.3920874386284240e+01,     1.4985446724101537e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.7949112444132563e+01,     8.3167464508188132e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3035688108104303e+01,     8.4557575973878065e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.7612207389144842e+01,     1.4933634374846641e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -7.7200315268294055e+00,     4.8690549585003628e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2008452685528184e+00,     2.9826592210437098e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.6381771506668990e+01,     6.1306011607473683e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5082797085949196e+01,     2.6154320471894625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3965559549394655e+01,     3.0521635345135041e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -1.2927213394920631e+00,     8.3237157178226571e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3760487540230672e+01,     8.9614082369397402e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1473354098284158e+01,     4.5842220931165895e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    4.2351568776102786e+00,     5.5513257804224568e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4966124440985343e+01,     2.1623060860782869e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.5123481869263529e+00,     2.6293331618463610e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    9.1455292232886141e+00,     8.3336671316275952e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1356917309431398e+02,     7.2153324549174240e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.8806163699182841e+02,     7.2127932760002413e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1458934629521153e+01,     6.4282266413000671e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4229240540488728e+01,    -6.6169221349681193e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2607952453299096e+00,     5.3990298883421756e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1271537938639071e+01,     5.9831848448895807e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -8.4617718192509372e+01,     1.1214066941798512e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.8601942473633763e+01,     1.1079080207816345e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7728762461848451e+01,     8.5840892668487068e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1149926978083233e+01,     2.5160678850996115e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -4.5915051078653676e+01,     3.2367097573119700e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6552704635725153e+00,    -1.5393158473110532e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1859930973582472e+01,     3.0910706944487836e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.3792220735819214e+00,     8.3113774976539044e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1556751090901995e+00,     1.6499089644306281e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.4957274415668707e+00,     8.3275184874319839e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.5458305662038399e+00,     2.3940137820481745e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.0843075779327214e+02,     2.4985566315637345e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    1.9429743035976074e+02,    -2.4986582232713412e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    4.4266083235569376e+01,     7.7710181155831779e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6158995399528058e+00,     1.6737577117587405e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8345258352347539e+01,     7.8358114949046467e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.7153385194272353e+01,     7.7774106194869273e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.6500637020240269e+01,    -5.8286487668803602e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3075870945128424e+01,     2.3059666626001275e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    7.6408008436804593e+00,     8.3298643620182684e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3384312141166600e+01,    -1.6844390410149917e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.8446138873479580e+01,     2.3387754573004520e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.2512210674932163e+01,    -8.3532993236802286e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {   -2.2739889009237832e+01,     8.3681540106649627e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0465748761491671e+00,     1.6818194554639897e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.5580201661333305e+01,     7.2181816669124100e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9773921570755292e+01,     2.1692398686307115e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.0729480030808773e+01,     7.7717361746007800e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.1391976625364002e+00,     1.7898578908502150e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.2199117744821010e+01,     1.6657128585066017e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5865639130367740e+01,     5.7501994293767584e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.5505901169399834e+01,    -8.4706932949179663e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.2200325092424036e+01,    -7.1304503217240572e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    9.6824390877854938e+01,     5.9817205689772290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8032125757396805e+01,     1.4269218439284092e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.9635257999697089e+01,     1.6657637113834488e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8571872410118870e+00,     7.7106931209350705e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.1914938578592682e+01,     1.6029068550853983e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3546228097577977e+00,     5.9110470295424029e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.2807121232889948e+01,    -6.2239011652751306e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.9322457510352002e+00,     1.5665613165415121e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9818693331769772e+01,    -8.3329491804851168e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.0955351089979565e+01,     5.2643771280330857e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0643345942860964e+00,     2.3642718055156256e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -3.1034909826427004e-01,     1.0173799277963844e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9736149403317455e+01,     4.0569112109957466e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0955707449293615e+00,    -1.1222304544705667e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.0558726964056369e+01,    -8.1294874367152679e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6784189520354040e+00,     4.3051817232674894e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.3075857286368251e+00,     1.1636437980397732e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.7509291340908431e-02,    -1.9716732798507853e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.2130938480185369e+00,     1.3827048107936391e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5025708671660540e+00,     9.8853561924578862e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5213788144551533e+01,     4.2060254855447340e+04,     4.8540321454576519e-05,     4.8102243812325939e-07,    -2.2918112333409983e-09, } , 
      {    3.6089731900761848e+01,     4.2064857255126801e+04,    -2.5608517982811998e-04,    -1.9799790736513449e-08,     8.0866922009070521e-11, } , 
      {    4.7838950218288545e+00,    -1.2934842093017298e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8337174306901129e+00,     7.1294545790143020e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9171404185026493e+00,     8.9253598205373291e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6054338284784404e+01,     2.2107179662619397e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0476903889072737e+01,     2.9828893410276833e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.2344979663967589e+01,     1.9113724444923810e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1027626282898602e+01,     1.5611446395756619e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -3.6026504887025858e+01,     7.5411242320879337e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2323441332048169e+01,     1.0544815372690209e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.9523296419410272e+00,     9.5628913166894989e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9854604676242964e-01,     1.5580491649298170e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8630488174962263e+01,     8.3855736081626856e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.9185951807535048e+01,     1.4310411993702355e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.8971230873876230e+00,     9.9165077876446667e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0047540919432407e+01,     2.2773026490984699e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.4992805841358976e+00,     8.3478696186472316e+03,     3.9957403058506603e-04,     5.4269558649720345e-07,    -2.5674762924198806e-09, } , 
      {    2.1719965199213675e+01,     2.4969864938833471e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.2788504218612182e+00,     8.3320885055328345e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8115862882128621e+01,     1.4645378662282894e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -3.7646546739703531e+01,     8.3290047978335242e+03,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -4.2357655884207340e+01,    -8.3283780516167280e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {    1.9694217593360688e+00,    -1.0464648538729951e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3082363264028039e+01,     3.4520309270061107e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.3610687805616948e+01,     1.8286143196432553e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9305818248278772e+00,     6.0879595778049415e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.2415634052662888e+01,     3.8141837091486312e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.7041631048119868e+01,     4.8859821188924194e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    5.3531650873059924e+00,    -1.1147769763235074e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.3447725478480042e+00,     1.4465664974237421e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5805144416571693e+01,     1.9208697123051912e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.6036893827495700e+00,     5.0540117847440306e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9750187277661627e+01,     7.6262620290739251e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    3.7793735022475083e+01,     1.3424846119534508e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0480147234456437e+01,     5.3940731330119479e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.0909213913475064e+01,     6.5700227962657009e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.5620731149043134e+00,     7.2114598414703523e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -3.6035564964659763e+01,    -1.0818135342110077e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6334406979940148e+01,     8.6326701456503433e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.8701417831143843e+01,    -1.0071979884995038e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.1832338483295622e+01,     2.4188233961613958e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5445523350934547e+01,    -1.1144801422090795e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.4293836631748977e-01,     1.2983123284176441e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.5990423517647571e+01,     2.7832038081465053e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5742417085971390e-01,     6.7746412299405019e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.2225060518939067e+01,     2.2128180938120797e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3707404987121244e+01,    -8.0447029437074925e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -6.7165435394685787e+01,    -5.2833481139945206e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2553502590176873e+00,     1.0524107181640151e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6600896107431325e+01,     3.9043881293744533e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2844440508554850e+01,     1.5062992271562935e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.9674172815919730e+00,     1.2037754265966658e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2858154126355091e+01,    -7.7158029507187721e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6275725086099992e+00,     6.6169336839277721e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0102102676842247e+01,    -1.6985969469330572e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    2.0645254206008715e+01,     3.3143913433360620e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.5001712298282239e+00,     6.5669351845580184e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3956502866308831e+01,     3.4310357104117204e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.5289963964576232e+01,     1.5529191753105388e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9657885958803806e+01,     3.9978073240325917e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.1571377614359132e+01,     2.6733926191275081e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.5496938489969022e+01,     1.1224454655717012e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2214088630913174e+01,    -5.2106880417139291e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.1021281595643508e+01,     2.4251348773402617e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.9232408878105343e-01,    -1.1015403862763371e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -8.6744125407404482e-01,     1.9077641817581112e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1448686673053437e+01,     2.4477211394823571e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.3267136532069749e+01,     2.5891355204027881e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.1980474991237369e+00,     7.1991065323207195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5968732318516841e+00,     1.2636502980308798e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6817819117277896e+01,     1.2827690129675746e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.5163373287988797e+00,    -1.5786428343716775e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4969182944428082e+01,     2.1447232888340513e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1939193355407667e+01,     1.9959284961334582e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.7455908740469656e+01,     1.0428230717639746e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.2159469601791386e+00,     1.1488591666416618e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0626430051278930e+01,     2.5430910355358654e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.1215334242205563e+02,     1.5606842065691353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1056283058830850e+01,     2.3859955727153436e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.9702852143144433e+01,     2.3882935702664734e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.2223011107816042e-01,    -7.7739801690804597e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.0759079418349891e+01,    -2.5011633913922769e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.8702093452450114e+01,     1.8177863307064399e+00,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {   -2.8863292454952409e+00,    -1.3361756356443909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4448837969156036e+01,     5.5417278446338451e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8644584497354639e+00,     1.5647981139139465e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7757739021069611e+01,     1.4750153395198255e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.8607804519376220e+01,     2.8678216434608126e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.1464430915621442e+00,     9.0889962145892678e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.3728953393417349e+00,     4.7434923412323633e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4706849976379102e+01,    -1.2543481811916424e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    7.4047441920591108e+00,    -7.9460416560273125e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.3969447276655995e+01,    -5.6617443507260850e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3521419124864446e+01,     6.2885987457802639e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4418571242862427e+01,     1.5056785631921284e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.6072655032657821e+01,     4.9397219366641330e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9141652576738441e+01,     2.9968250188785438e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.1730264880921702e+01,     7.1437809055133130e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.5132755264825200e+00,     7.7041866092889095e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4397635990189848e+00,     1.4956395175627244e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8935183592509102e+00,     2.4619447914756620e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.6423325869018344e+00,    -9.4613536679787449e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8954473853955662e+01,     2.3871594131966300e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6988185835914425e+01,     3.3384595542805873e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {   -2.5666166296250399e+01,     1.4356780109547193e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    9.6927137800231158e+00,     2.3871297297851870e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4157722965685512e+01,     1.5511602694997178e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    6.0690539703862953e+01,     2.2756458126133988e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.9353106567744135e+01,     1.7285697148046522e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1243423849408007e+01,     8.8172793878453231e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6514976616449463e+01,     2.6545040241747876e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.3656412664946558e-02,     6.0415589119795977e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.0653730668921559e+00,     8.6939310717113385e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9033583216640885e+01,     7.7936685848392990e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8746769242841854e+01,     2.2894221089131065e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5190006948930989e+01,     7.2309236121530139e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {    1.7428287588496584e+01,     5.8718606321606103e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.0675824453185321e+01,    -4.1167944466173522e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.0395470885923153e+01,     7.8402929077206754e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7842343786805962e+01,     8.0619245817093433e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.8117442498690757e+00,     9.5879364271323975e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4722658625837306e+01,     8.8348514800063804e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4390410904974971e+01,     2.2757176185151595e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.0710020773112312e+01,     1.3963005810761460e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1907829895215567e+00,     2.1154080354561134e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.6464628940402502e+01,     4.0580643675849620e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -2.7074549583611457e+01,    -7.7309048727256595e+00,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    6.5056824135535081e+00,    -1.0045890419515130e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.0641198408875283e+01,    -6.1338714688343134e+02,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.5537807954624689e+00,     3.3521685332415043e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.6959284678100573e+01,     1.6001845302340440e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4637097111527360e+01,     1.7474077711145928e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    7.7866228957520409e+00,     6.5320748377352211e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3242216487596075e+01,     2.9973509853417803e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    3.1096015020928753e+00,     6.5819525108039634e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3333503901453298e+01,     1.7730027842801996e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.3631316640861741e+01,    -7.4805163353662174e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1521533792758042e+01,     3.3526944997047409e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.8057296780470185e+01,     2.1500201985551706e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.6704489833310703e+01,     4.0721457349381279e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    1.6399021518719060e+01,     8.9503876102173763e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4593384937173703e+01,     1.7620638799054659e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.5475892906499507e-01,     8.7354519333925709e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0302888922030071e+00,     7.2140270396233709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0394736895610091e+01,     6.6541187535095005e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.6537863047399899e+01,     7.2140986912942944e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7923595651449276e+00,     7.2063479385131150e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5988988408956459e+01,     2.0556425691825301e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.7953058954086952e+01,     3.1692486257435608e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -7.2885850436876822e+00,     1.6193603780360558e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3570561518092500e+01,     2.4062970972538606e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -1.8541441759838456e+01,     1.5224984381663358e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.2593165519409149e+00,    -1.7642660522896886e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.3670915319965085e+00,     3.8814049924755354e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8753093703391400e+01,     7.3421440695359979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.1629420550281480e+01,     1.3959077753463771e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.9419664201351427e+01,     5.4845412419368495e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0790247852420695e+01,     1.8092395890185278e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.2341519030775565e+01,     3.7777202055203590e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.0394858114972449e+01,     5.2255288389138641e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    6.1808455176149792e+01,     1.2934352117917531e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1544436249628404e+01,     7.9248537436724164e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.5282876685915276e+00,     7.0560254324066314e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5514641409932004e+00,     8.8877249911639433e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.9686581834477568e+01,     1.4953909291078251e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.4621866166126949e+01,     8.6546602795459057e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.3080620638649428e+01,    -7.8536099728762838e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.2200220555166833e+01,     7.2508226273281198e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.9015393252452316e+01,     1.1275155693430747e+00,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.5637285779427636e+01,     8.3121639901787876e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7182899411320051e+01,     4.7626523021411940e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.7597267627322735e+00,     8.3289806290553515e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.3761510340799319e+00,     8.3284022203949007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2877811137990477e+01,     1.6107783496762822e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.4347526566818942e+00,     1.6122610474948160e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.4404444351917874e+01,     9.3987218451287317e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1080419848837742e+01,     9.5897339196156395e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5158024268061624e+01,     9.7587734333042354e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.9298302398772094e+01,     3.1172650516182330e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4665125410657396e+00,     1.7295999715751284e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5854901579273431e+01,     6.3527698649004687e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.9813520631692050e+01,     6.3785070144223027e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3239147883656061e+01,     1.2755079364625801e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0935191566067575e+00,     9.5419354333936335e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0730959836605820e+01,    -6.1156807225852544e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.2525351281297924e+01,     3.2545811468705881e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -8.1372121858732783e+01,     3.2200039088060366e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.1316948873162900e+01,     7.2986711519033634e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1472977052163981e+02,     3.2200235191208048e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.0302839157312832e+01,    -8.3593596064028789e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -2.1924117607254853e+01,     1.0722477059213561e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0210704622718286e+01,     2.4986074274175378e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8365544895762849e+00,    -1.1610292250916373e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.6371453079561782e+00,     1.6709640601194569e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0436476214996816e+01,     2.6263352960569973e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -9.5772607230870179e+01,     1.1174252039900599e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0775792970444682e-01,    -1.7841841252621922e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    7.4741141399234809e+00,    -1.0037864892102456e+04,    -5.5129179363627256e-04,    -7.2486985405007633e-07,     3.4785817951922203e-09, } , 
      {   -4.8039655801143244e-01,    -7.6328753752017528e+03,    -2.1663523609776755e-04,    -3.2372880868492796e-07,     1.4796077215150038e-09, } , 
      {    6.3700257548347334e+00,     9.6190523823595358e+03,     1.1615568567377584e-04,     2.1467685547360962e-07,    -1.1213643481327344e-09, } , 
      {    1.5473413375215239e+01,     3.3456801070456335e+04,     2.3824151533214727e-04,     5.6485641986072033e-07,    -2.6799530664372928e-09, } , 
      {   -3.5357368009079186e+00,    -8.8894792856987169e+03,    -2.1130747311195048e-04,    -3.2496159291325332e-07,     1.5884871780185836e-09, } , 
      {    8.3989348466197065e+00,    -7.3310669832642270e+02,     2.1643197796196239e-04,     2.5531742888271492e-07,    -1.2072830286989663e-09, } , 
      {    1.9165642095405460e+01,     3.2862196142020141e+04,     2.0494419658465167e-04,     5.2841850232768114e-07,    -2.5619930496865311e-09, } , 
      {    1.2358345987905945e+01,     1.4981966429210366e+04,    -1.2518284369398322e-04,    -1.0603553945019108e-08,     9.3418748212996449e-11, } , 
      {    4.6124736285853540e+00,    -1.8015317921684713e+04,    -2.1194173061026206e-04,    -3.9000836576856623e-07,     1.8496611561691016e-09, } , 
      {    1.7176284047583973e+01,     1.5299447777215792e+04,     3.9730927195513093e-04,     6.1163609195097950e-07,    -2.8956951545310360e-09, } , 
      {   -1.1801792615962894e+00,    -5.6078786097359080e+02,    -5.8994722470602276e-05,    -7.4550478483366898e-08,     4.0214810034354901e-10, } , 
      {    2.6464110337469865e+01,     4.7776648476955612e+04,     1.4141995685417922e-04,     5.9174903475186623e-07,    -2.8162826735652947e-09, } , 
      {    2.4151682668696317e+01,     3.8819655096982002e+04,    -8.2289122942605245e-06,     3.4072152820781718e-07,    -1.5755038676384699e-09, } , 
      {    9.5633626731597055e+00,     7.7038630508170882e+03,     2.8564543351820303e-05,     1.0590754863837813e-07,    -5.0207304815703544e-10, } , 
      {    9.7552192237270052e+00,     2.4601078831895058e+04,    -3.8952597615586333e-06,     2.0292778376286493e-07,    -9.1761170237282965e-10, } , 
      {    1.6184764779992587e+01,     5.2953399322507812e+03,    -1.8481182366289703e-04,    -1.5020080528369891e-07,     7.5674082670706345e-10, } , 
      {    9.1025398616998920e+00,    -1.8849658860706058e+03,     2.8597462200767728e-06,    -7.0365857676238356e-10,     5.2985287208461214e-11, } , 
      {    3.3373255312850560e+01,     4.6594505823414424e+04,    -1.6773506168838749e-04,     2.2880781679964549e-07,    -1.1000422424375372e-09, } , 
      {    2.9794384976213141e+01,     3.8333328492964196e+04,    -3.8170641629324650e-04,    -9.5537383999445148e-08,     4.3400520732925652e-10, } , 
      {    2.5794855348886738e+01,     3.9657506518061215e+04,    -4.4356088498392551e-04,    -1.6938420390604968e-07,     7.8316802046434421e-10, } , 
      {    2.2495919526675532e+01,     2.3418876158028717e+04,    -3.1818217656277408e-04,    -1.5886791642363030e-07,     6.8829483120801924e-10, } , 
      {    1.9240109037146350e+01,     6.5519438427477453e+03,    -1.9013958664871404e-04,    -1.4896802105537357e-07,     6.4786137020348393e-10, } , 
      {    2.0707016366589322e+01,     2.1466456198008382e+04,    -2.4853192812053770e-04,    -8.6783006396914066e-08,     5.0390564387162939e-10, } , 
      {    1.6546774542678662e+01,     5.0357490432874751e+04,     7.9369623436331888e-05,     5.1798948130786155e-07,    -2.4656654193868781e-09, } , 
      {    1.6483449547602412e+01,     5.8526542118839261e+03,     5.9498759014167929e-07,     6.8236846877013627e-08,    -2.7523357490269465e-10, } , 
      {    1.5441049132853877e+01,     2.3486450272628768e+04,    -3.7470888226763603e-04,    -2.3394752055856013e-07,     1.1463371008466867e-09, } , 
      {    2.3919699952640151e+01,     3.8961690468537832e+04,    -3.7923839952750624e-04,    -9.6066509651008111e-08,     4.8989937662437483e-10, } , 
      {    2.2232598168620228e+01,     4.9242861873608461e+04,    -2.9144399906974556e-04,     8.1114176986436463e-08,    -4.0171661616736159e-10, } , 
      {    6.5738741051423908e+00,    -1.8576105782658306e+04,    -2.7093645308086433e-04,    -4.6455884425193313e-07,     2.2518092565126507e-09, } , 
      {    1.4860478798734238e+01,     5.0147940967044036e+04,     5.1203771463308840e-04,     1.0287116055358911e-06,    -4.8787770357414819e-09, } , 
      {    3.6878373958218944e+00,    -1.7390489547776677e+04,    -8.8193523320734133e-05,    -2.4550479997705792e-07,     1.1653951266511026e-09, } , 
      {    1.5590984880220063e+01,     6.5182468159354594e+03,    -1.5417838640830990e-04,    -1.1314649563649707e-07,     5.8434108170451249e-10, } , 
      {    1.3683666470705699e+01,     5.7816665362685926e+03,     1.8866568033608895e-04,     2.8605810692356351e-07,    -1.2527682482606632e-09, } , 
      {    1.1422761572895173e+01,     1.6275860988510382e+04,    -2.8262009918534334e-04,    -1.9137050434436715e-07,     8.9855367656841395e-10, } , 
      {    1.2055868928463921e+01,     8.5044238230932424e+03,    -2.5465793683230161e-04,    -2.2219844884781542e-07,     9.4258445508678196e-10, } , 
      {    7.9271654237089102e+00,    -9.6528694498221412e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.7038751411304567e+00,    -9.5853553555472499e+03,    -1.5211688591418001e-04,    -2.5049838089248614e-07,     1.1848846366317058e-09, } , 
      {    1.8481438395715962e+01,     2.2128515200394308e+04,    -2.8202511159520165e-04,    -1.2313365746735350e-07,     6.2332010166571914e-10, } , 
      {    1.6125883823464061e+01,     1.3799823775669183e+04,    -4.3433786223654987e-04,    -3.7354477189723997e-07,     1.8096591793407536e-09, } , 
      {    1.5182502224438903e+01,     5.8898848403618176e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    8.8705428339438619e+00,    -1.7429305145147757e+03,    -3.6814974101316892e-04,    -4.3749169643558775e-07,     2.1183885314713062e-09, } , 
      {    7.1842488353286909e+00,    -1.9524799803454973e+03,     6.4518350183587553e-05,     7.3230427792441848e-08,    -2.9472308488329798e-10, } , 
      {    1.0878287472958499e+01,     1.6728400535228167e+04,     1.1912075766607364e-04,     2.8242820993036017e-07,    -1.3399765332186464e-09, } , 
      {    1.7913095303244468e+01,    -8.5322074161609016e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {   -2.6867903471110147e+01,    -8.2354066795654126e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.5260978809457393e+01,     6.1515338973226403e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.2985211289081676e+01,     2.2701460225456452e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5357645353094527e+00,    -9.5661189251079850e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.7541838578081368e+01,    -3.3502534561921493e-01,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.3876682185056517e+01,     8.1770173531073442e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.1710230422052668e+01,     1.4743594439598528e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.0946713448630014e+01,     3.7711078762679113e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.2753510808922144e+01,     5.9595252381514465e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.2210060256010308e+01,     8.3305632522816177e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3031663875962115e+01,     2.2397577613452766e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -7.2700329657844321e+01,     7.5238512353210790e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7003785449838979e+01,     1.6856115603921251e+04,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {   -6.4837249488921088e+01,     2.3870741082992932e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3939578235138939e+01,     1.4354831225665534e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.5055116802789456e+01,     1.4604658335276170e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1954662451568169e+01,     1.3936535848332715e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.5688024168227059e+01,     8.2655700516654106e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.6363815272157680e+00,     1.1095570696723629e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1485275218026619e+01,     6.8079409505135427e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6039340107683305e+01,     1.5525144140747443e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.9306778398580072e+01,    -8.3647740520678235e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8113867651282327e+01,     2.3384413395221458e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.8063582349643553e+01,     1.6480850245091748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5830088908723942e+01,     2.3973617423835967e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.7428335574734195e-01,     4.7970957466745688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.2747313809651132e+01,    -8.4369118674150050e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.3737516774685997e+01,     1.0151794402997224e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.8056685519718663e+00,     7.6267875102787057e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4515539027259599e+01,     6.6593784181418687e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -9.4706936203013008e+00,     2.5342677422778164e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.7157727958015041e+00,     1.8291599878195135e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.1419765663621355e+00,     1.2243482010615811e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5996805341178543e+01,     1.4286041068519031e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5974421633641640e+01,     1.7316966219399283e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.0601880020073310e+01,     2.4092492840454943e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3882323400083720e+00,     1.4910643935529091e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8678234806059461e+01,     2.9420124697068200e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.1324778489986031e+01,     1.6135724804293968e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.7318314507006875e+01,     7.1434218760045105e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8002066135283084e+01,     2.9412462214076051e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.7909356323981527e+00,     1.2476138793652182e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.7928503375540870e+01,     1.1548128185270240e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.7875466897556365e+01,     8.3775417204827390e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2729224058211518e+01,     2.2030212961336765e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5375317199389031e+01,     8.3290332184793915e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.1239430094115853e+01,     8.3283496309708607e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6453429857880355e+00,     1.5556316827262528e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9077191674542764e+00,     6.9945239936219159e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1782269410356605e+00,     3.2191500641250779e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    6.8665780568113952e+00,     8.3283215617589703e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2919328323265464e+00,     1.8684620847888124e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8228934648183014e+01,     7.8881494885547163e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.2162828104473064e+01,    -2.1496885373606585e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1382194036889297e+00,    -1.0513501807996145e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2225514452974480e-01,     1.2083117823959195e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5536074719691094e+01,     2.2851988987727047e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.6624102625748236e+00,     7.2497190999356953e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.2273723592397332e+01,     1.7754738411949689e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7340500280208204e+01,     1.0051950702237780e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1326131399722184e+01,     1.4476685600959785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2588491140760311e+01,    -1.6608680983785176e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {   -4.4897770489164472e+01,     2.4306783600300150e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.0428228568953870e+01,    -7.6846513555319907e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    8.1219156047325995e+00,     7.0720875142281620e+03,     1.5764051362716528e-04,     2.4917833020156111e-07,    -1.0774596211714549e-09, } , 
      {    6.6692516643169766e+00,     1.7157061822475678e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7132636334020944e+01,     2.9748711316948524e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.4270748225296657e+02,     2.5802167085909616e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.6051219922144703e+01,     1.3195380240076702e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5993692072960130e+01,     1.5780670737852699e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6340269584842048e+01,     6.4652233281961699e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -5.1114740928322266e+00,     3.0485665531764025e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0587394514588029e+00,     7.6584353691907554e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5316985267745391e+00,    -4.3371161422579974e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5206188858851835e+01,     1.1155089273748426e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.4759663891572146e+00,     1.0309528757337655e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.6510174578770211e-01,    -4.2587963168101942e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0877706690546233e+01,     2.5001806758322968e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.5805334187146943e+01,    -8.3441894146001541e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0174752339495910e+01,     2.2050512683706009e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.9822659963629050e+00,     2.1278033281398606e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.5368667335636452e+00,     2.6193101123747433e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.8230389298102082e+01,     2.1548962231009627e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6592596875692916e+01,     1.4936881204845586e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2048470457752515e+02,    -2.3992514135730687e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1217752258498052e+01,     4.5585172819754180e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9159667992688112e+00,    -1.1856162348816290e+03,    -1.8274292976013022e-04,    -2.1905404427487519e-07,     1.0864141298615481e-09, } , 
      {    2.7975780347410517e+01,     1.3404182603230998e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.4310524339041542e+01,     6.6759267797477933e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.1544229917559313e+02,     1.6655347332433175e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.7232894435762322e+01,     7.7266922303061401e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.8484021339686656e+00,     2.2625246137904269e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.1884772454701292e+01,     1.6645176924630665e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -2.5924772507924370e+01,     1.3171158244868504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0287898776133545e+02,     1.6659418366467329e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.6971487854178147e+00,    -2.0300378711546823e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.5742398553653771e+00,     3.9898219779474064e+04,     5.4306288134201204e-04,     1.0655913822578935e-06,    -5.0540856628306906e-09, } , 
      {    2.6350456480291946e+01,     1.2967390800028854e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7861512796751313e+01,     8.3282339823377588e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.3150403652247519e+01,    -8.3291488671124935e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.1667887604023171e+01,     7.1207628609310696e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -6.1307012952563355e+01,    -1.6669588774269840e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -3.0744711848476691e-01,     5.4404737988372917e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.5474564286248743e+01,     2.5211356224153689e+04,    -2.2515794548278047e-04,     1.7123619216788808e-08,    -9.3714484558473377e-11, } , 
      {   -6.3468351993716183e+00,     3.0966063874674179e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2448701181211639e+01,     1.8958736800787661e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0312936624497706e+01,     4.1313716518024405e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -2.5782512541016627e+00,    -3.2350534700215889e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5124882675073437e+01,     3.8837707531537788e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    1.4554161723392088e+01,     1.6731005987178833e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.7434696539286769e+00,     1.4221704770280203e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.7333215789145214e+01,    -1.4430307003257985e+02,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.7856333692175212e+01,     1.6100061856266380e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5571219124005458e+01,     4.0685408932978862e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -6.9606700865517624e+01,     1.6100075283367831e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5879795752365395e+01,     2.3323320223191848e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.1830484158989663e+00,     4.4760339888073759e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0945521666883700e+01,    -2.1063982433014230e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3475878342753632e+01,    -3.3452795026515560e+01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.4650192462877083e+00,     2.6555455834834147e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9765527598049886e+01,     2.3872150346825238e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.2995714444828977e+01,     1.6170963823655662e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0492132318410178e+01,    -1.6877749026640697e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {   -2.4451519453628254e+01,    -8.3771428250064673e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.1489441274341631e+01,     2.5027789680046126e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9334038830820699e+00,     5.5180260598494151e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6266537664043028e+01,     1.3069338851899698e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.8536999509238345e+00,     3.4415875228693894e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3199341271039749e+01,     6.5591656212386897e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.9424198380539089e+01,     7.2248795768185537e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.3358928121766986e+01,     2.3860629003549366e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7728189529651750e+01,     1.6703740547108147e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1364009043412224e+00,     3.2262493946663177e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.5310193264613957e+01,     3.1542963899873572e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.0665766884288022e+01,     3.2494383481208031e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.3504485254193909e+01,     7.3875297039468533e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.2824837554168010e+01,     6.5859789374693228e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0750357888011294e+02,    -6.2627206802846706e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6828633535181957e+01,     1.9622388170319464e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9497500176547135e+01,     1.9618079103131888e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.2513512708496952e+00,     1.3310822237205363e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.9591007252550432e+02,    -8.9575069683087077e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -9.4011209833383759e+01,     8.8929112342632743e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2845123307092972e+01,     1.6577183790174789e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.4712839875559585e+00,     4.5562160821356829e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.8350245759758366e+01,     5.9573496437939066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.0541770376623312e+00,     1.0702549727030640e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6549761857901096e+01,     1.0324449195499768e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.6128968210056026e+01,     2.2784047117707061e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.9725038890065516e+00,    -3.3084610674840596e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.4703657410033966e-01,     1.4614725739973192e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9408685668506028e+01,     4.0791347127401248e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    2.9651362473553977e+00,    -1.6869074553550599e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {   -1.0651348602416748e+00,     1.0278377162150873e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0524074360173142e+02,     8.3293469276941141e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.0052963445722764e+02,    -8.3280359217561399e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9522277962794611e+01,     2.9435872440583844e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.7704789391257663e+01,    -7.9628068798437369e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6144540411832075e+01,     7.2229412539115701e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -3.5538222690501158e+00,     3.3614054295410824e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2759976526579166e+01,     3.5809595607460469e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1132340932145031e+01,     2.3828785898953382e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.1006243185898761e+00,     5.7494524157190983e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.9491750609052332e+01,     1.6656994707656719e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.2055610551486410e+01,     1.6657770991243786e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.8126696501016241e+01,     7.4529522768463066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4684919720108283e+01,    -2.3708914133993044e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.4666757993342493e+01,     8.2776370004553028e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.1062573246976584e+00,     1.4410076787562288e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    7.8852307515122062e+00,    -9.6183766768523001e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.1780588123382589e+00,    -3.2892828735619523e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0237440009365692e+01,     8.6038292260894177e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8441500932835254e+01,     7.1571806820212732e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0687952040484220e+01,     2.3123693424169804e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.1523024339333574e+00,     1.5541331599250205e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7769737966082030e+01,     3.2960441929498353e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    2.4283010894562203e+01,     2.0911709302404073e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8369378782422935e+01,     8.8037728765739685e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2343652402552687e+01,    -8.3264244358256819e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {   -6.3411453795751749e-02,    -1.1021770397411810e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -7.0188923999749605e+01,     5.0851056339373156e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8800573644831442e+01,    -5.0605707403437591e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.1450238458546629e+01,     2.9931065119203708e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    3.6163208529449918e+01,     8.4549455514023302e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.2925966097502332e+01,     7.7578706338206248e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0662770268380999e+01,    -5.4399990013939814e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.2767492629632842e-01,    -2.4950025857772969e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {   -6.3372622037577273e+01,     7.8579124787928067e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.3761558580462214e-01,    -4.6518172865031802e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1142358476618268e+01,     9.3162385519606923e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.4370934883023835e+01,     5.8151324520665048e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.6437176531426694e+00,     5.6550968978863375e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6989877369372024e+01,     1.0218545212017674e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8752155852594736e+00,     7.9225695097798362e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1396481104373684e+01,     3.2738273225345249e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    4.6767620319417865e+01,     1.4433056763073586e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.0536031229253283e+01,    -1.4193498571812688e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.1895403485220672e+00,     8.8834026803697925e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.6468224477488427e+00,     1.6631645699928420e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.7010374341001429e+00,     2.6250908725579279e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.7286076260449402e+01,     8.3140214519467499e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9419003941151647e+01,     1.0993097163648004e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.3392678810369617e+01,    -1.7808449753553941e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1277588480038878e+01,     3.0740053688709457e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6857886552273030e+01,     5.7573173409679396e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9090177714866901e+00,     1.6577822383896950e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.6960020706109979e+00,     1.1850461040922824e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6856664174326838e+01,     2.2440028908937922e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.0473270552751750e+01,     7.4006476151463103e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.0177565312215515e+00,     7.9628929269353748e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.8686029812139839e+00,     1.7249582083978788e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6483108774170629e-01,    -1.8340800416737293e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    8.6725383876688209e+00,    -2.7943924385069414e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    7.4912486546228596e+00,     1.6689022038683594e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.3701372123279594e+00,     1.0510676020374744e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9715270029800633e+00,     3.0837602726692985e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    6.7006172238201698e+00,    -1.0190685853885029e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.7673000827789522e+00,    -9.5617467210442531e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5989970099968263e+01,     3.2807114816701906e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.9106022941147991e+00,     1.4807916080899506e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0961480377395478e+00,     2.2575015642920087e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2347315202359088e+02,    -6.2159012631855148e-01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.8883640336089346e+01,     9.1124321642713658e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.4513680615616629e+00,     5.6485379422866099e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6123772333705279e+01,     1.6250622340782804e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3685183157063818e+00,     2.3416639179291851e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.6953136077962593e+01,     6.9403451039537941e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.4810770962663323e+01,     1.6868028196248946e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -3.1322391941721680e+01,    -1.6865836434313002e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.7431065016074701e+01,     4.1131398999883044e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    3.8509303656843947e+01,     3.2218186082989581e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -4.9676887277765545e+01,     1.6122519162461922e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.1517150694385823e+00,     2.4905892180847070e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.3717737177952642e+00,     1.5107135424274953e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6952879652689134e+01,     1.5588533328234298e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.3345470408257611e+01,     1.4365992580944729e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5767857829688907e+00,     1.5533875901731220e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.8714070828114622e+01,     7.8908218507857164e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1245666032775347e+01,     2.7637362101898798e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.1007518110609830e+01,     3.0010694922999533e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -6.8046964580277375e+01,     1.0857463793195728e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2381286138082451e+01,     3.8153098193733967e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2118751070551054e+01,     8.7113411934229407e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.5100290792040017e+01,     1.6864854772497274e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.1036751488909907e+01,     1.6869009858064677e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    4.4010902711132260e+00,     7.2090871585563646e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9810366913681953e+01,    -1.5099507287990164e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2888698481803488e+01,     1.6155519846941073e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4036221939247522e+00,    -9.9127202774033885e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.0721001381919102e+00,     1.6625743660216911e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1694813369715789e+01,     6.0221487647681779e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6124062483443726e+00,     1.5105962486981811e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.0668698722924891e+01,     1.6852329081700118e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.3618984026034735e+01,     3.0683671141961877e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -6.3183427859706526e+01,     9.4412942930273675e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.2125537281253109e+00,     2.4251847446279695e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.5996096960709806e+01,     2.3845137324282528e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.0696903154023502e+01,     1.3318662312634751e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2540805986062384e+00,     6.7254749023386357e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.9833650844730724e+02,    -3.2585269574665290e+00,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -7.7646321380437762e+00,    -4.5922987094783821e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.2112183581701457e+00,     1.4712941517288822e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.6697287307308763e+01,    -2.5179891255858514e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.7090906575609058e+01,    -1.1174734733240848e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2198851068616387e+01,     2.2302010620025558e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -3.2545886190314235e+01,     2.6247371358567761e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.6790767747295050e+01,    -9.6627256388181166e+00,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.5537465975130242e+01,     8.9192616506564573e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0058994074552263e-01,    -1.1169335447494475e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.8038340003510690e+01,     1.4434159205312615e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    4.9058016655036724e+01,     2.3944740220161209e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3080887657853628e+01,     2.3839806525675744e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6800652483285049e+01,     2.9924494614644325e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2058845582326825e+01,     1.6622553781064526e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0122768849540277e+01,     2.5066256367503687e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1076124535222056e+02,     7.8424685020782154e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4711362369431866e+01,    -2.6518616597392594e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0307612635266760e+02,     6.9440893974944993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3333174084339063e+00,    -5.3274761116775289e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1611326695162667e+01,     7.9406014871880579e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9990481032377012e+00,     2.4960337124653546e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.6955923146487418e+01,     3.0751447336896192e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.5403450236078227e+00,     2.2743310644371439e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4775576289462631e+01,    -1.6669380459997625e+04,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {    2.0873121855028739e+01,     9.4170841615925092e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5398511107425659e+00,     1.6699331319939080e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.0930523403532831e+01,     8.3282620524418835e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.5641632548036625e+01,    -8.3291207970083688e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0618022212489780e+01,     2.4411839313733446e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2374462038327994e+01,     1.4966261816628707e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.8351888315755694e+01,     2.3829497244420258e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.4951599370283191e+01,     8.3159258294837473e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4039547359939437e+00,     3.3001945774108070e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.6104456390475558e+01,     1.6459308112007129e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.7362788881110358e+00,     7.8269405966129407e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4892505941626681e+01,     5.3314413101619375e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.4043044998618916e+01,    -8.8727294897475417e-01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4823771141152397e+01,     7.2190385723613017e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3868048963969542e+01,     2.2813905054307561e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2389849265749984e+01,     5.6573980977260726e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0976609836222277e+01,     1.5431420601668866e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -6.7702643327441248e-01,     6.9925157890807350e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0307445201077742e+01,     1.6615434378961425e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5095192330103384e+01,     1.5545021279083407e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {   -5.1524859410162449e+00,     8.1814987659233739e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.2822409492644649e+01,     1.1571074959584661e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.7547558845062341e+01,     6.7782149669609862e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.9662013219957910e+01,     3.1905331947256851e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    2.3426294195080072e+01,     2.3342464924732631e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.3032617543310572e+00,     2.4670605565494516e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.3471379098729770e+00,     8.5595707985336994e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.2944454896869950e+00,     7.2025976128573548e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4136234276404085e+01,     4.5176969298510841e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.8079187079731035e+01,     2.3866378642961812e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.2360882833110928e+01,     9.4617363741181744e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7408001669740756e+00,    -1.4301162195193979e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.7211390361264300e+01,     1.8151914467210077e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.6838551304566636e+00,     8.2941986517552850e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3786464059127216e+01,     1.8045995224359936e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.4394614038843157e+01,    -1.1789264411481599e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8550546846641609e+01,     3.4100807638286482e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -3.7729514121719106e+01,     2.3774542914346406e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4903267390527176e+01,    -8.6003958611678863e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.5751753866413454e+00,    -1.1117836452085021e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.4741611702093902e+00,     1.5630668276405137e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.5682085949754274e+00,     7.9794412885412985e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -3.8415476027476302e+01,     1.6520668181441042e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.9919423084619936e+00,     2.2382941010273149e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.4660726246779753e+01,     1.1525586280139183e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2480620921410406e+00,     1.1457440071229836e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9556067128944449e+01,     5.6078444236075884e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0430368539444355e+01,     3.3951868570440456e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.6613388673126712e-01,     1.0671398131843862e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5430259880682105e+00,     5.1134876390079637e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1734860796805862e+00,     2.4335426620415710e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.3998970372491293e+01,     4.8600921328683089e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6352055439923149e+01,     6.5684822836464336e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.6833995477577609e+01,     8.2622158162425803e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6132094487914792e+01,     4.5252428734973546e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5804064503559392e+00,     5.4868744646626346e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5822755856669845e+01,     9.4631590650519247e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5671708396365400e+01,     3.1821218486675207e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3003828659243339e+00,     1.2869159763345171e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9823945847551798e+01,     2.9829245726234905e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.7840705702550181e+01,    -2.6663736449549015e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -8.4489401499487471e-01,     1.4317290715279654e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.8427609607661871e+01,     2.8767233948221728e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.0126261155259844e+01,     1.5666707217021425e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -2.6728708847828862e+01,     1.4563112464899832e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.5775143503198379e+00,     1.3655201702731145e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.1290829221645566e+01,     1.7341047364682141e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.2120089802892309e+01,    -1.1155949617139720e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.2391000692427475e+01,     1.2543701648658111e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -5.3694000556687254e+00,    -1.9585190845375062e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.8613884455000100e+00,    -1.3156276749556064e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6374022829605327e+01,     2.4552722397165820e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5635869144877617e+01,     3.2691373886642828e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    1.4232261131974946e+02,     2.5448116130448966e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.5794353174305868e+01,     1.1485771378241270e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1737332076834997e+01,     9.2901055683510422e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0159490232460371e+01,     2.2774449181918451e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5415973790769918e+01,     3.8352540543037940e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.5593613299016612e+01,    -1.1944973906312839e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.3094707269172908e+01,     4.8630571485270048e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.1958637796061865e+01,     2.1493647215744568e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8147818931944869e+01,     6.1392492080904512e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.4997427061996831e+00,     1.6163435291603231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2921247856901608e+01,     1.5520006669825694e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.3169477666469529e+00,    -9.1982637020316389e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.9177076757941723e+01,    -3.2432886412032531e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8090398487555228e+00,     1.1064419101536849e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0804161275200300e+01,    -4.8596865826267515e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1685119235361963e+01,     1.6657737453048358e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.1107337524369566e+01,    -1.6657028245852143e+04,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.5995838634437728e+01,     1.0719062889676134e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3738558168157041e+01,    -8.1824470581403439e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.7723353346647002e+01,     7.7398464256194211e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.0751168947076508e+01,    -1.7430341958856757e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1909113696195348e+01,     3.9159768565417144e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.2943968525490355e+01,     1.9093060928620303e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.9824834274083809e-01,    -1.1731972189240395e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8550389419599682e+01,     7.7228183393867530e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4689459158587020e+01,     8.3951670332076719e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0057646258582125e+01,     3.5498079655592664e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.8782712463445712e+01,    -8.5044747224209332e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2210513921768680e+01,     2.1564191709485262e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.9818494161051756e+00,     9.4477292111604911e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7767894505885035e+01,     2.2159676425152080e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.3088520697207002e+00,     3.2968067148662203e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.5590474628680759e+00,     8.8702579608426349e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2684510659345218e+01,     1.1429802817362634e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6640976677136706e+01,     7.8637003584822278e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9716132876549413e+01,     8.3090145579259406e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.3635311779762421e+01,     8.3268084158822312e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.9883660705440192e+01,     3.2185531865214125e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.0529043954426438e+00,     9.3741415514383225e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2253153647739456e+01,     3.2130247361614238e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.7880759686834502e+01,     2.5494937153527186e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.1391541084915914e+01,     1.4954613922994404e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2764682881032405e+01,     1.5041003462071774e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8201460152679530e+01,     1.5780300874886541e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.4423517382493248e+01,     2.3457754667641530e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0775224403631793e+01,     8.8334287890726300e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0605368114222870e+01,     2.3482027541251016e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.7704167417593677e+00,     4.0698563128292139e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    1.4283946106009855e+01,     4.0002692688240670e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.9080964581150951e+01,    -4.3992030839540848e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.7877570559257620e+01,     1.6084800105665067e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.2769947169214385e+01,     2.5193546197222397e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.4147749991128162e+01,     2.5016727196485081e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    3.0758185799097220e+01,     3.6523367806257076e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.7244693853853633e+01,     2.5132991917156094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4857645907970003e+00,     6.5855316233572557e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7929411018579195e+01,    -8.2912203173889629e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.3376547506858735e+01,    -8.2134862014015653e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.9413258752432405e+01,     2.5120372678833715e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -7.6406669283112869e+00,     8.3313059976732584e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.8175544296228033e+01,     9.9471759693224194e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -9.2254456547632643e+01,     4.3025059165192943e+00,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    7.9473468440202183e+01,     1.4332953898833412e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.6198133586460379e+01,     1.5530548365364373e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.4704939861748521e+01,     1.9570123705024420e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7768500491755919e+01,    -1.0757800598160597e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.2150335982951781e+01,     1.2854722314121707e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4562114143874417e+01,    -3.0942942529705704e+00,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -5.6787696939520664e+00,     4.4786638211235586e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2435105746342629e+01,     3.2195206107478294e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    9.6984909119686868e+00,     9.4896776816493184e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8381430285710616e+00,     5.4983566832112656e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9482688738953840e+01,     1.5540014141250733e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1185858084973574e+00,     1.6997444197718167e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.2901131843368006e+01,     6.9630437659586896e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4050626857411572e+01,     1.4350840189493305e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5251551958340652e+01,     8.8195651686187375e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0941232094035392e+01,     1.6125213498936642e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.9187457338792413e+00,    -7.8918996215229781e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.0745481126522037e+01,     8.9085476094893293e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.5722040568744919e+00,     3.1080851302012006e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4619216200997192e+01,     1.0627298665858252e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -7.4910990845620162e+00,     7.0270553047591675e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -1.3334974989133524e+01,     7.7965220742115162e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3331771106748988e+01,     2.4068543661214691e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -2.5466532477200676e+01,    -8.2214526438509765e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.6933780727985202e+01,     2.7866089761651630e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.0662284072923825e+01,     3.3208347783173282e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6924172898196701e+01,     3.2456087271290275e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -4.0578252817233285e+00,     5.4198102825337828e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5488283277394892e+01,     7.3133896603279227e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.3775159081122048e+00,    -2.9426625696943083e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.9291440274556031e+00,     1.6627370850363764e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.5860281672653187e+00,     1.4370747406537403e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.6262417670223321e+00,     4.6627910554593793e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    7.6432650032592946e+01,     4.6628615186509953e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    5.2843069515835282e+01,     2.2770323666914144e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.2597182765896626e+00,    -5.0448382561961680e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1543790033704016e+02,    -1.6659113817928679e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    5.4874281846858608e+00,     4.7648222492102150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.2421648975248146e+01,     1.9876819609995364e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.2980645355231474e+01,     7.7557407698659244e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5344159493738285e+01,     3.2768926147654951e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    2.2637737694949138e+01,     1.4663412346270221e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3417721948871552e+01,     2.5197701282789803e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -6.9090580010811564e+00,     2.4968025330820004e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.8016480845229964e+01,    -8.5562898339112216e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.5168214923256773e+00,     1.2779450780689383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6149264783662289e-01,     7.0165136092204884e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2730837055417894e+01,    -3.5607702682931688e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1005815181510842e+01,     2.4588646062938409e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -2.9103758445107914e+01,    -6.4558058181239903e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.8793475488997423e+01,     8.3263835113346668e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.7801627509893542e+01,     1.5474062184611295e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5546664324857383e+01,     1.5545494439117187e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8658392550230701e+01,     3.1023151773338948e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9929891018354517e+00,     1.7053800899950947e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2897382667394542e+01,     9.2106479415869999e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2623901686559854e+01,     6.9515537782030397e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.7291796175419130e+00,     8.3650579403319716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9093109125563885e+01,    -2.6256751690332831e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.3201739591863504e+00,     9.7702634210598808e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8690541714984633e+01,     1.5544176981117713e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2331013010032255e+00,     1.6654779825461774e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.0285545538939338e+01,     2.8429962996884504e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2232080629926783e+01,     2.3611594887327839e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.7329642212083691e+01,    -6.2791944325200711e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0079804390237950e+00,     1.5461399259562510e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7291351188445056e+01,     6.2869572683907927e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.8062569568488001e+01,     1.8580389538006370e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6619176669002449e+01,     6.4468468040368889e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.4579709993017998e+01,    -1.1270800787914077e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    5.4440042387867607e+01,     6.4475514359530371e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -3.3752442673841188e-01,     6.6764254526248687e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6418470187397169e+01,     4.0112371644984247e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -1.6552986497193524e+02,     1.1411445820907829e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.2231773957082964e+01,     2.4031481269089250e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.2692985192552673e+00,    -9.7059805969643435e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3714705511947948e+01,     8.9100996710682484e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -5.3690107077213653e+00,     2.5257756889979035e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.0247679399694384e+01,     9.5433581243273857e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9697026482188740e+00,    -7.7025260779492073e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1794752301583014e+01,     8.9084391397285490e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6857005768986951e+01,     5.7202892524586987e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8599230215264654e+01,     3.7593574313899881e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4494812030085539e+01,     1.2964300719889986e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1630780165395377e+01,     7.8515592384202882e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1894211433232051e-01,     5.1250050901564273e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -9.6284938843564358e+00,    -1.4885047046359334e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5048699754655644e+01,     1.3602474604860987e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0345840381344267e+01,     1.0762068304055441e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0640337030896575e+01,     1.0810770169720518e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2968375676619596e+01,     2.2772564899158431e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9328218878296880e+01,     1.7248872009564620e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4480713044174415e+01,     2.4046502407769964e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -3.4559033936433128e+00,    -1.1506640609771988e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.0247643426720725e+02,     1.0840190796428871e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6891851886821851e+00,    -6.5289074884647584e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1562657510242539e+02,     3.9306943359875362e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.3870749003768598e+01,    -8.7577950217608104e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -7.4752875725751622e+01,     1.0523402549724000e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1633290167057581e+01,     5.1894546993371441e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3172547703500850e+01,    -8.2937807466554277e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.5843491748101584e+01,     2.7697334480695918e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3975044681225125e+01,     9.4608549951016666e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.2442111958218504e+01,     1.9946167050135125e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7080977098707883e+00,     3.2528723759514527e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    4.1006872355591266e+00,     3.9769649663533264e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2158204421028664e+01,     3.3312135866584322e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -1.1423150650852310e+02,     1.9034366416850844e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9472565247131733e+01,     9.9760263646694657e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.2823847117937053e+00,    -9.4541463530143074e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6110925168171953e+01,     3.3317395531216687e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    3.1339180989018754e+01,     3.8246602348615554e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    5.7623467123381467e-01,    -9.1655382182316987e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.9764436065472561e+00,    -5.7649247355525335e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.0362976905676646e+01,     1.7850008295607782e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.2679452248360597e+01,    -1.7932260025509095e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.5474193412675836e+00,     3.9414528637569492e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    5.3176202336445570e+01,     3.9413871372616595e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.7070976113902031e+01,     3.3728954172967278e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.8231159493744325e+01,     3.1713463849455384e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.3896390523818219e+01,     3.0546218508935970e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.5824810208546463e+01,     8.1976540610831526e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1689361132404663e+00,     8.3886641034151580e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.3976984881754406e+00,     7.7429912116817213e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8989063022213273e+01,    -3.3351765203623654e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.9106408735095414e+01,     1.9542758366523638e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1379725893540151e+01,     2.6211342191909353e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6037229639080770e+01,     1.4492560080730331e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {   -6.7493655478359003e+00,     3.1080577548211997e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.2596056847994145e+01,     8.3485904399002284e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.8849477034903437e+00,    -8.3087924095500239e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -9.0214849473718814e+00,     3.3546857069862279e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -4.4731956497567871e-01,    -1.8165032378409534e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.3797077403946252e+01,     1.6808558248190959e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -7.8500863235956615e+00,     1.7032354875787867e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4106648088129656e+01,     2.3620411356040891e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.0635209286565601e+01,     4.0830130191143871e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5162262044625706e+01,     6.4924552758855261e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.3658968860615225e+01,     1.4270139611768387e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4917278206093343e+01,    -8.5018448901047486e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.0261901339498909e+00,     4.6609908150510437e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1454755089323736e+01,     1.1520996666024562e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7952135080172177e+01,     2.6938061211440865e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3218650234983828e+01,     1.6310781497154849e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.2746861529753897e+00,     6.0420559818122765e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0267275262230143e+01,     8.5360986522862222e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.7343534593398921e+01,     6.6124159869979230e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.5054649143552467e+01,     5.3517126997836465e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8362235567745140e+00,     2.6403545199929872e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1634937475531455e+01,     3.5050000242173023e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3892258188051958e+00,     2.3230518891541164e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -9.6255549648109792e+01,     9.2893292847639714e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5563080890462517e+01,     2.3903084904142426e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.0351706246368522e+01,    -1.7408586015281362e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5270830181872778e+01,    -8.5078394813868954e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.3516344486663270e+00,     2.3897754105535641e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0178837900306158e+01,     7.2160272131295005e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9019176337140188e+01,     3.6417057434594244e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0659332487339036e+02,     3.9414193291542317e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -5.0869703159504802e+01,     3.9414206718643771e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.9641781605996051e+01,    -1.4669972778375950e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7003541190921769e+01,     2.3654192783543851e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.6463568055695696e+01,     7.7848836563633340e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7260678731244020e+01,     3.9623397154965685e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    1.7528782055649668e+01,     1.8195685300719237e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -2.1859494326066830e+01,    -1.1195595914222140e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4139534785939007e+00,     8.3275374390042252e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.2553432902283561e+01,     1.1754546463446923e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.4561597013264638e+01,    -5.3502283988582712e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.4582880440616215e+00,     8.3636021027948245e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.6740970533574879e+00,     1.5870782059260739e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.2783912609214944e+01,    -1.6688863077113485e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    6.3790534860955958e+01,     3.9624101786881845e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {   -5.3182129096390156e+01,     3.0457207708873102e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.3760118352716525e+01,     2.9344873635895809e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    3.0130718799476778e+01,     1.6315678318370479e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -8.1989023622236168e-01,     5.1815111490289837e+01,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.7426093417737732e+01,     4.0778192932087972e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    2.0225305037602276e+01,    -4.6675497706507704e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.2330541647105147e+00,     8.3109095055466460e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -6.4402828372869259e+00,    -4.1190956464570872e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.0001891571933497e+00,     1.5994130375394723e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5010921458333899e+01,     1.3964111455812279e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.6543711822847396e+00,    -6.4075910457065572e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.5292754182140946e+00,    -1.4726059428488283e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    4.2118048669020300e+00,     2.2803716494345208e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.3004966952501110e+01,     3.2792396308277348e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    6.4578549878938745e+00,     7.7172445863737657e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.1851239165714240e+00,    -5.1430540335118021e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3506788774842065e+01,     1.5687370733324937e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6591732320585368e+01,     4.1314870503745318e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -1.7360075048905131e+01,    -8.1114537527279444e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.3118081865925305e+01,     2.2130587113380949e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.1301423863203830e+01,     2.2932600453338600e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1183029094429413e+01,     1.4988034364860583e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1172791538502027e+01,     8.2168531762163275e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.9838409366682619e+00,     2.9683844598480937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3059656658515181e+01,     1.1373326549390162e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7691119515727154e+00,     6.1332756678101014e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7309663934084526e+01,     6.1432203990697471e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.3309880996990145e+01,     1.4691764298071212e+02,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.3166056986756942e+00,     6.2922360119646896e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4134523621756356e+01,     1.3206455449836563e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.1398221680565488e+01,     1.3273670720793807e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6082568449185636e+01,     7.3694960199595298e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8832333038892738e+01,     1.4731008570874907e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.1892015394054560e+02,     6.5837197633962141e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5736786174999274e+01,     3.5436159274475358e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3386621828504882e+01,     8.3644217419231009e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0197258295819742e+01,     1.4860766262460349e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.2790369487608544e+00,     1.7893215507691330e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4378662850528183e+01,     1.4988745710327459e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0683735453135565e+01,     7.2139220781858203e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8238009762165213e+00,     7.2142036527318451e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4034534869734646e+01,     1.0766999336211358e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5778000607060015e+01,     1.4852089898109447e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.5216584627087926e+00,    -1.4741631585778669e+02,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.0943688788116691e+02,     8.3284644526978500e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -1.1414799702567070e+02,    -8.3289183967523986e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    5.4601714943370041e+00,     1.7336332608692941e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.0968952246423896e+01,     6.1060457057458316e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5367226909928666e+01,     2.4990483501344446e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.9429927605150496e+01,    -8.2738209453510444e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8185580270314674e+01,     3.2280319232962516e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.6798547480425661e+01,     9.9569652974128431e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8653183180866428e+01,     3.9493829808888870e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.7680518683129108e+01,     2.8731185531819319e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.8302460776340929e+01,     2.8885117116550427e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.9872800361445403e+01,     1.5638355494551455e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5956351086069603e+01,     1.6666261237902992e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    6.5341327970620027e+00,    -1.6648504460997516e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.3330417476905074e+01,     1.7601159143176657e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.4713272991257753e+00,     3.3366580810390791e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    5.6099857539684130e+01,     3.7673565060620938e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -4.9096612797492213e+01,     1.1597123695284063e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1408187246051440e+01,    -1.0640265635916876e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3324749529735310e+01,     1.4434737130470958e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2118575697435316e+00,     1.5517015155037036e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0028008253445715e+02,     9.9176368698095139e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4356505357567837e+01,     6.7980760105785265e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.6328871425586158e+01,     7.6121175375003295e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7834495161259980e+01,     2.3453398941161140e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.0006477141132464e+01,     1.2658275915365084e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4187991076457621e+01,     3.9739805304390347e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0653735201994587e+00,     7.9874855665224286e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1594955046853038e+01,     1.6517682179097144e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.6294404220618340e+01,    -7.7556446609443883e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.3141458852466116e+01,     1.5565501910542225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1813918610108461e+01,     1.0937481738785089e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.9109856487184231e+01,     7.7471042714824962e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.2275828728193297e+01,     9.4509583175041462e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9612773180131963e+01,    -7.7193081741113747e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6760389809196202e+01,     2.3888306461603264e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -1.6277022113663396e+01,     4.6171307526565033e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.5628324444049237e+00,     2.1765958542539011e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0956461375117179e+00,     7.8203291970596474e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2266037345983129e+01,     9.8009163433695849e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    9.7419509434356428e+01,     8.0587179567607927e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9345848228461094e+01,    -1.3245370546058166e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.8049657878815129e+01,     1.0369047334362453e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4225590529618474e+01,     1.6939089082200393e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3840308918414044e+00,     8.3209240538258437e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5639625303394780e+01,    -1.3238189955882135e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5468832930474221e+01,     5.1038429009758420e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.4399948278616446e+01,     1.3210495199037553e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5834066929657798e+01,     2.5226276662315802e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -1.8954727956157726e+00,     1.3828683800587216e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3448460012746434e+01,     1.6676510585043881e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.3622159847467472e+00,    -9.2080333686388640e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.8071658501939385e+01,     2.0491511620707792e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -2.8736911512403776e+00,     5.2555169725936230e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7688822068057339e+00,     1.8612028775069066e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0468129774354262e+01,     3.2542474508829969e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -4.6838966084046973e+01,    -1.2290061930127608e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.6577977574067965e+01,     2.3913394185397912e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2558019131628637e+01,     1.5316284143544457e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.1610796798060283e+01,     2.4968442247899718e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.8862239040156538e+01,    -3.4469963266936247e+01,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.9913018563419826e+01,     2.3261187073218913e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0146282069894550e+01,     1.5496905913891100e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -4.6693402415960449e+01,     2.2812174085829130e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.7980480557800703e+01,     5.7601041543566116e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7558073143271438e+00,     7.0617669733785515e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2706862035044144e+02,     8.3135017123947709e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2751845422629280e+01,     8.3364587956244086e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1202255311087335e+01,     1.4284061577309172e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4059328607839927e+01,    -1.6023041790009764e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -3.4298863223275127e+01,    -7.8934248747741967e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7512961918919565e+01,     1.8851215357668018e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.8224473123450899e+01,     9.9843700759440253e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.0746026789172586e+01,     8.5662792399174159e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.5230056372867100e+00,    -7.7702736176995559e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1810080433366747e+02,     2.2128607622171090e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.2944218214712741e+01,     7.2709450488963921e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1316350143186106e+01,     3.2892879074492426e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.0679992621367699e+01,     1.1466300226669300e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3566372573479388e+01,    -8.5156062120696220e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5143937299985401e+01,     1.7854479442516538e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.7480788066912190e+01,     3.1731865108768834e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.9264869442209942e+01,     1.3817867089227497e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0196610259333909e+01,     1.7970664998454726e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3961095990276839e+01,    -2.1504401386205814e+00,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0009152833468558e+01,     7.1479395729343623e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8271692557938323e+01,     7.4304103717152502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8537442413648151e+01,     7.9068200596850911e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.9272170209898327e-01,     4.7319841204634404e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0593173027188982e+01,     3.3682049519071661e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    2.8997116960470901e+00,    -2.3410226507790242e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3584774052588493e+00,     7.7746828448686256e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5769552412712546e+01,     5.6611571806089869e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -5.1468407137041083e-01,     1.6153012931874127e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.3137761473254752e+00,     1.3926899577902848e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4426179475425780e+01,     3.0719719558364290e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    8.4957521813986006e+00,     1.6594104470983573e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9199812955419105e+01,     1.1241432280299294e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3591024339413714e+01,     1.4923306860577877e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2584082858922239e+01,     7.7577621640598454e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4110302247671555e+01,     1.6598513698152641e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6835391849962633e+01,     9.5902376269721317e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1708977447506523e+00,     9.9105833856825325e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2077722362150176e+01,     3.3601368752866270e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.1293339541167238e+02,     8.3438811370554795e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.3613348513821485e+01,     2.2777073309127007e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0800144753316117e+00,     7.2051844770060934e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.1475889679467549e+01,     3.1090439612523838e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.5716703765480105e+01,     7.2050274715991309e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0204543233877565e+00,    -1.8601232887856895e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4659538086889455e+01,     8.3309993381155855e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1044396493346337e+01,     3.7748816121793323e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.8784143904904347e+01,     1.5591205690465300e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.5418711369509900e+01,     8.3553884154235820e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2091796360523819e+01,    -3.7237734769243652e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8450445825969663e+01,     6.6049429501215145e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.3349555523917587e+01,     7.7405510575355702e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3042743778090312e+01,    -8.3029522895782029e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.8729370040593935e+01,     3.3297492702133648e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.5360136412714702e+00,     7.6731538776153138e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1245916554660464e+00,     5.0938534949696477e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9658932577389301e+01,     2.5810813413852189e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.2957157693588762e+01,     2.2340385959442014e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.2543319240951778e+00,     7.2192515740783729e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9731117354166798e+00,    -8.3631841976949672e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.8173554479897120e+00,     3.1067459637012540e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.0445343287132317e+01,     3.2669537433046182e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0786026945047361e+01,     3.7095925372946855e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.0013970213656991e+01,    -1.6820531649455632e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {   -2.6131331052498634e+00,     2.0761059384868146e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    4.4946986800914281e+00,     7.8385571586912411e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8935868036453414e+00,     1.5791158816735970e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9607968552202308e+01,     8.4017622729212235e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4275573978562326e+00,     8.9040393883455981e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0141334333443941e+01,     2.3883897234434196e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.5056290779408052e+01,     4.0187622706156631e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -1.1864359373544179e+01,     6.1967108666211516e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.7624527703693342e+00,    -2.5176436070297874e+04,     6.6570736180512717e-06,    -2.0358780910832745e-07,     9.7132421010295529e-10, } , 
      {   -3.4653998966202508e+01,     8.6394461970350494e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.6650113487866069e+01,     3.7676339846220429e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5545477896796445e+01,     4.0735748197873872e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    2.8387780438745658e+01,     3.0440280314513588e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -1.9433626713312243e+01,     7.8812189499546093e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -7.7712104487882883e+00,     2.5636923764351977e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2154514459891388e+00,     1.8826607458712919e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3117165441886307e+00,     8.8583148961274674e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5250358674810103e+01,     2.5301542982856241e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0875646268829513e+01,     1.0633857621457981e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.7996578303633559e+01,     2.3809088907880116e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.3848152478028013e+01,    -1.0073928768876696e+04,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.5522642258309283e-01,    -1.1221067258353503e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.0097737648897969e+01,     8.2668716478605947e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.9465307896085466e+00,     2.7703482817590167e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2309504120129915e+01,    -8.3847642919754089e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.0222656184814474e+01,     2.4304797591918643e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.7319527622217485e+00,     1.1616976023910679e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6496310650171633e+01,     2.2728431222232532e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0066082818383212e+01,     2.4030353006542660e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.2230502235192082e+00,     8.1311421684867819e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3528090180993100e+01,    -8.1088239204117599e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2533705184885523e+01,     1.5589659675724952e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2523563634059194e+00,     5.3265102780060197e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2307300799192582e+01,     1.2547818008868489e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.5520293335967867e+00,     1.4422092256522716e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -6.7072093615024819e+00,    -1.0216015996722763e+02,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.1978474302637871e+01,     2.2281963961196922e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9594550364916032e+01,     2.4499424667478183e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3859049730819589e+01,    -1.1071503926972366e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9442950822457359e+01,     4.6592214454149471e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    3.0365437737765905e+01,     4.7349870460125181e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    7.8666116253551612e+00,     7.5606642177542362e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.9010605022426830e+01,     8.4749357913099102e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.4523326114805926e+01,    -8.3302253760733383e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9612510154208174e+01,     4.0741007862506238e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {   -7.1779266616259719e+01,    -2.5281175351032289e+01,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.3814995428840959e+01,     2.3871239999681871e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.5115249251383961e+01,     1.6437828718245284e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.3016388196158459e+00,     1.5997908409849879e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.4969289972564823e-01,    -1.1764330767627789e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2923047996613562e+01,     4.9675500610605911e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.7173009169246590e+01,     8.3288500168009487e+03,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {   -2.1884118313750392e+01,    -8.3285328326493036e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -3.9900642826819187e+01,    -2.5563675854766188e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9587634113670070e+01,     8.7639579746760555e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.9086082521024572e+00,    -1.1668863110106083e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0445239300987279e+01,     1.7515395183815360e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2249971915758127e+01,    -1.6362866044225775e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.6389744430010360e+00,     2.5802601417073791e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.1749175846742016e+01,     2.1245770545875799e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.2170161336460961e+00,     9.3844508326938121e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3301345850970748e+01,     8.2726185574748433e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -9.9455463739121370e+01,     6.5877907974303653e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1652114104803765e+01,     1.4127526535791660e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2153351432668753e+01,    -5.7226597528922666e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -7.0609373097512353e+00,     2.4078917637956107e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.8557184668500611e+01,    -1.0071275253078888e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0689283544671998e+01,    -8.2789104888976763e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.5568017248011778e+01,     3.6194781186376757e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.4997516270408631e+00,     1.3851741408795084e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1543951198421919e+01,     7.8708232358113833e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.4164988632243052e+00,     2.5753634901551068e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4108345487512368e+00,     2.3571724749743858e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.3827095923930067e+01,     6.0504190674190595e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5075259306254205e+00,     5.6949386081119537e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.3498543609252785e+01,     8.4143547029158981e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.8787434464748983e+01,    -8.2430281465343542e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.6811360665405246e-01,    -1.5283130452627902e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.6259583847665757e+00,     7.1682838274084907e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    2.3490298392769759e+01,     1.7003504480440817e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.8961669230216607e+01,     2.7225162938283705e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.3146813787521667e+00,     8.0158714999326867e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    7.0259154891675912e-01,     4.9960335517418289e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2785692884169151e+01,     1.6728728470820723e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.9080165820964197e+01,     2.4083072723523506e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.8162249435762913e+01,     8.5946309623873840e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.5683951346218485e-01,    -8.3106577407377990e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {   -1.2257370891645611e+01,     1.5878163629195406e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.6964369417199663e+00,    -1.6821676159390408e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -8.2745769780631981e+00,     7.1184616610913363e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0369550886082955e+01,     1.6295014802066969e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    7.5341062180004066e+00,     1.7179040894606536e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.7493088516421860e+01,     1.6659660365808362e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.2079007549417966e+01,     3.6899920494214002e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9546920747241572e+00,     2.1187638884476547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7723738121495394e+01,     2.6185601545601698e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0210633442399413e+00,    -1.7313927071223205e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5425768783070133e+01,     1.0488742505439928e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.1582606352835434e+00,     7.1367773240344713e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6340101949124758e+01,     1.6655105333092142e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3283142065870202e+01,    -1.6527434546339123e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5939018296719716e+01,     3.8732923322836308e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.3362283378798412e+02,     5.0969849152006645e+00,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1688517236162369e+01,    -7.7315354348663632e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4288359540183119e+01,     1.4914595122195629e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3185989903220223e+00,     1.4980569997662886e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3306460853965071e+01,     2.2617460377134183e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.6349713502731699e+01,    -7.6382506897066469e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.7591368598619268e-01,     8.3270530831501019e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.9359394687833088e+01,    -2.5211042851045291e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.0422247858119604e+00,     8.9568505927134556e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.8466775200006431e+00,     2.1823425204621199e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1022427078249720e+01,     4.1836085908647568e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    5.6141682882692168e+00,     2.2508911301967859e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -6.2741511254146900e+00,     2.5082212508707311e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.6962236764132008e+00,     1.6843460169033500e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.0222537944333006e+00,     8.3936512557425740e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4986418185764402e+01,     2.3137199935441160e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.6068426785719112e+01,     8.3127970804786219e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8872985482676469e+01,     2.4098268177506663e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.2955649119556625e+01,     2.3214272475148442e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    3.2304500367535447e+01,     3.0128614375442052e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -2.6401188215730130e+01,     1.1016723400787378e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5637505433203856e+01,     1.5065069814346634e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.4881121306456397e+01,    -6.3171859104348450e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.2318369505710844e+00,     6.0317467005567887e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1134405576154720e+01,     6.1124030350589624e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0664849475773927e+01,     1.6217462541054843e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.8602952364289315e+01,     5.1556732718341809e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.0292222559992496e+01,     1.9502943464625725e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4170887447163132e+00,    -1.7195518935161293e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.8132742706355813e+01,     1.5527376409634475e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0988944684868081e+00,     4.8559370473009076e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6814405029148965e+01,    -2.6595479703030833e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.0409017362023128e+01,     7.6758171712141893e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.3712953046706012e+01,     8.3824117333706636e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.1846608406015761e+01,     3.0021658873510587e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.2507288491344831e+01,     7.2866927924120300e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1880061662429735e+01,     1.6048202999998339e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3714937367463191e+01,    -1.0241362132969773e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    8.3559071663831688e+00,     1.5471766614568623e+04,     1.2188257152256629e-04,     2.8176818458489763e-07,    -1.2862640254885208e-09, } , 
      {   -1.7505170960235468e+01,     1.4300820085791092e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.4345880481500288e+01,    -1.9524923236932850e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.8537348754075289e+01,     2.6880319760622515e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2791219447992354e+01,     1.5629495339111993e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.7206749507144970e+01,     3.1062966675236861e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.1191519473553035e-01,    -3.5503066384363419e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.2347275433621938e+00,     3.2426607286273716e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    3.1610327029918409e+01,    -1.5233820131068715e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3070125164426955e+01,     3.3157031344560070e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.0685540318279264e+01,     3.4046684555354383e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -2.9816337507372456e+01,     8.3271574733769139e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8778098352204373e+00,     7.1243340063783544e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5021065451046100e+01,     1.4444125239242147e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -1.6892554246025973e+01,     7.7009361292120666e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8593148550364322e+01,    -9.3636901801955955e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    9.6023052857953715e+01,     1.3784276117583691e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.1933450812366054e+01,     8.3445857689716322e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0712530068750674e+01,     8.3230015966095434e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.8387413817281328e+01,     7.7898835317398152e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.9264908636974511e+01,     1.6029081977955433e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.2541286140841962e+01,     7.2120985177881648e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -3.0728415485713860e+00,     2.2126432027813546e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0352227245759565e+01,     1.6720661227916931e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.0385823843185008e+01,     1.7081548853058965e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    9.1698640447549984e+00,     5.7914125744367755e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1150484822223937e+01,     1.7844819586988244e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2859998272940549e+01,     8.3343812528407088e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.9002413556576357e+01,     2.2732544282033312e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6898455441621948e+01,     8.2381830841941792e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7682752292414072e+01,     6.3739046147428344e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    4.5421699633993668e+00,    -1.0032948707512051e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.6811884071606151e+01,     2.8556530496670101e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    6.3689748068791330e+00,    -2.0690068194229670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1902747797568017e+01,     5.0010521633189610e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -3.2518964080886121e+00,     4.2313365701135730e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2019667545750202e+01,     2.3786748839165994e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.4404636285425191e+01,     1.5559615036878140e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -7.1583981185788659e+00,     8.7588909027287864e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.1314814397047082e+00,     1.1155948220227651e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5784864795983019e+01,     3.2205068171790135e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.7069458792773020e+01,     3.7455726366637773e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8914188828951346e+01,     8.0509535138874180e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.1618446841770087e+00,    -1.9551154424587430e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4560283648502104e+01,     7.5603377130462263e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6062762630537065e+01,     1.6655079914613132e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.0388721121081222e+01,     2.2597909864009216e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0082851461020443e+01,     2.2032949596719460e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.4680892351883401e+01,     6.0941653978283393e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.6915129205512983e+01,     1.7444931992200384e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8024782972540816e-01,     5.8959726221259757e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0618645418808775e+01,     1.8718048439371742e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9007789460533800e+01,     2.7160728588471036e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    7.2791720098837214e+00,     3.2187517901311832e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -1.9826968860722314e+01,     1.1238020878141579e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.6283239448888409e+01,     2.2033337738512993e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.4890569005094711e+01,     6.9300743844411973e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.5812395512400421e+01,     2.2708028073831059e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6985705249231209e+01,     1.1392677088940718e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0555536168917357e+01,     8.3551419053389254e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5248805798004312e+01,     3.8197399736899868e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.6641705064581735e+01,     4.7778939846220572e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    2.3792321155832266e+01,     3.2165192250624223e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.8516954973539876e+00,     1.3533878608209863e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3568327904383437e+01,     8.7654832279272741e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.4605011611547578e+01,     7.1321937888845241e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0539111170487708e+01,    -7.2253023878307413e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.9803658712127483e+01,     8.3299920111768934e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -2.4514767856631284e+01,    -8.3273908382733589e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.5604171949463392e+01,     8.3650921570856117e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8203347627560230e+01,     7.8217518879934014e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0287237523230793e+01,     3.7471107336162845e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4409948332238301e+01,     6.5868598721426079e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.7510611455037690e-01,     1.9907217044099156e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2178763391765266e+01,     6.6965634877235962e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.2331641837767634e+01,     3.3353462899191334e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.7242628262364509e+01,     1.5543375917958963e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.5922871035053646e+01,     3.7853875385616884e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6215296895549738e+00,     1.5162851231690427e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.1739637912861433e+01,     8.3279154781366087e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.6037508075882485e+00,     8.3294673713136435e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2672002060632504e+01,    -1.7354579777427798e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5364931279521784e+01,     1.4357484741463342e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.0442079244021762e+00,     6.6724963293413239e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.8088041666290389e+00,    -2.0246998767195663e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1609080246665421e+01,     2.3249171604258489e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.9719294736460540e+01,     1.4415007819718207e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {    2.0971378483327300e+01,     7.9935458492450780e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.3445464384035970e+01,     3.1098626491567375e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.3813096749556099e+01,     1.2150680678965124e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.7191987640798061e+00,    -8.1719353457275611e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2976010028377726e+01,     1.5542132662408954e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5372630492838681e+01,     1.7140704895967428e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.1098677964234852e+01,     7.1475872569762869e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6335810739945103e+01,     3.9349765655280375e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.4411446541396582e+01,     2.4833255692622817e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.9206045851630254e+01,     1.6588204416897155e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5514681512346733e+01,     2.5263047618982338e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3985241072611540e+01,     1.6076144307529483e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1285378814194289e+01,     7.2290191985969459e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0590434446560870e+01,     8.4003960756043980e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.9815169744265431e+00,     1.5496975252133616e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    8.2577894972838486e+00,     8.7348133396704143e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.7968380274577946e+01,     1.6656319188025303e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9398007884326248e+01,     1.3797203572923985e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.0533689183352855e+01,     1.6658446510875201e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5889176598350787e+01,     7.2135641925817545e+03,    -5.2746002568012762e-04,    -5.4949267430635921e-07,     2.5923957156289107e-09, } , 
      {    4.5082671766925877e+00,     6.0840044523011038e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3325811824183628e+01,     3.3507042167964370e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -2.0871287987042585e+01,     7.3554636591969302e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.9994678585179415e+01,     1.5751951440056606e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -1.9009008334948948e+01,    -8.4423263130799496e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.7184032128419409e+00,     7.7665071322040531e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1019536409424248e+01,     1.5752656071972755e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    3.1054672385462636e+00,     7.6687446504462432e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8574591400482188e+00,     7.3297495118427785e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1978656158004469e+01,     1.7843396896054492e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.8720990682919814e+00,     1.5566647677327601e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7835870827852229e+00,     6.3816202602475141e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.4519920001666492e+01,     1.6814911488563477e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6222463002303833e+00,     2.7905744886171487e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.6828618184805268e+00,     5.2761710131110121e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.9908430742641855e+01,     6.1006818583100903e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.9082822367911994e+01,     1.6153788878462645e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.2931584554602889e+01,     1.5056081000005133e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.3231847537156849e+00,     1.2573265155579627e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.4485229530291317e+00,     4.2679698453580568e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.6487609636332465e+01,     7.4898745214312498e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.8831662665823956e+01,    -7.0085749396166648e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.1612356764948437e+00,     1.5330877901339194e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5059854131386317e+01,     7.2621228856205562e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1414539508010490e+01,     8.3751015663365888e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5592475046998864e+01,     3.8302201278142929e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -3.6489002513670499e+01,     1.5779912733093008e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.1222317698391127e+00,     3.1079475105972968e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    5.6483381621444115e+00,     7.5698719572423888e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.6975030129169710e+01,     6.6342066806946295e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.9946384250236626e+00,     2.7150641605082128e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.6032988222338865e+01,    -1.2282367627073472e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3553366508100101e+01,     1.5595713911609031e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.1415456048155841e+00,     1.2590102314483454e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6033112680058117e+01,     2.2111588889788462e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -1.4230103337303440e+01,     4.0018977771096615e+00,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.3555025472211465e+01,     1.5282289376113107e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9732589223334340e+01,     3.1101508088692397e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.1298205383774818e+00,     1.6149401227306825e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.3572045362039056e+01,     6.8144829401661154e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.3357091419791288e+01,     1.3173811760666313e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.2386547710570934e+01,     2.3367100532487129e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -6.9521869281399447e+01,     8.0499871114397392e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4515631313010395e+01,     2.4300678790465066e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.5192006289886663e+01,     9.0065129214212247e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.3759647975182629e+01,     8.3303297663001504e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1204741111980791e+01,     2.5622608383237002e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0668227812710082e+01,     8.8015658552699715e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1501476986839716e+01,     3.8296941613510564e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.5364334253581253e+01,    -4.8560962875387389e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.9727537255745018e+00,    -2.9568914984472190e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.2117308443679931e+01,     1.4301681148381302e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6548050379868940e+01,     2.3258094463001657e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1115782392555985e+01,     4.0515710653159877e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    8.5100632413976374e-01,     6.5665676107050622e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6035613564613236e+01,     1.5507861845127536e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8727196314277048e+00,     1.6674695712184584e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9587784300958752e+01,     7.6005248795520911e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.1341242314615592e+01,     9.5220282163495340e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3057675956840473e+00,     4.3389631507704535e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1595521704667227e+01,     2.4153132996086981e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {   -2.0112950885055163e+00,     6.1349966896464366e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7717676439173466e+01,     1.9537398336926504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.8535470613422056e+01,     9.3228630597076153e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5407924719766186e+01,     2.5491522983989758e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.9429193712036813e+01,     4.6056606931924052e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2751667503689186e+00,    -4.2768300007975186e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5885180519335108e+01,     7.7871096292395705e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.1646366696048243e+01,    -4.2745288009577844e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0003955321759047e+01,     2.4004553937281136e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.8967782783390371e+02,     8.3233134532991662e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.9571259406562094e+00,     3.0155508227274086e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7259019735353899e+01,     1.0560155598042560e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1132242436052335e+00,    -3.2708793266084708e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2086163262263021e+01,     1.5558132170733443e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.7227190488302213e+00,     2.2739544158875931e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -5.6445000327616164e+01,     1.2781906727744645e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3242895603794274e+01,     4.0686562918699776e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.0811462777211045e+01,     6.5611721166123571e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4613111370513909e+01,     2.5397128927855691e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.5832369946331344e+00,     1.4360438125281915e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1273374494177013e+01,     7.1569749667940632e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -9.7621696349609863e+01,     8.5846615111196388e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8109714976784616e+01,     1.7285683720945071e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1974442692776659e+01,     1.6636764286939277e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4418604086992488e+00,     2.9340271236216344e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.3831057261987780e+01,     8.2760746434720095e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1772709031411996e+01,     8.4809873168054073e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9311227278478754e+01,     3.2184137631309728e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.8947074279599761e+01,    -7.7867962561311688e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.1172978645970293e+00,     6.6017547887377750e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0181960358387547e+01,     3.8012764073077904e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3039281749919454e+01,     1.5542613502910946e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.7578104006891891e+01,     2.5211165768294530e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -4.2330579859899458e+01,     1.0745018964344617e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0907839459009377e+01,    -9.7375663255652344e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9349766175629014e+01,     3.2815326813480293e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    7.1793635503000104e+00,     1.5542895077456969e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.9103604281794908e+00,     1.5447153085816462e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.3738662689266555e+01,     1.6082795573050247e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6396691564281909e+01,     1.6381755850995003e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.2694332755957475e+01,     5.2737831801358607e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8272102484751329e+01,     1.6866120269295498e+04,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.1760481505692965e+01,    -1.6867744361266450e+04,     2.8701241417345948e-04,     5.6723200689815707e-08,    -2.5544832857661442e-10, } , 
      {    1.2140564999550815e+01,     1.6885408639522328e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.5388863643097928e+01,     3.8807222328025353e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    7.5240805852717250e+00,     9.0202773882373476e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1311703319081820e+02,     1.6171159926803341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.1861153795927351e+00,    -4.9853196791713708e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.6538357284337550e+00,    -1.6736943315003558e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.2668909726673320e+01,     2.8030383071591787e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.5568032387467014e+01,     1.8088799655421419e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9853318819372241e+01,     1.0156725435153141e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9274934124501009e+01,    -5.9061312567545328e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.6879836013482379e+01,     3.2935822481583600e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    6.8739169600253174e+00,     1.6678001411961228e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.9570768486954762e+01,     8.5986648926894595e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3521754035544948e+01,     2.2785203089053051e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0192485677834895e+01,     3.7688184541589988e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.1171132178301008e+01,     4.0107769245304778e+04,     1.1039479014525555e-04,     5.5486925802986389e-07,    -2.6409740464760860e-09, } , 
      {    3.9553582496865225e+00,     9.3727188605045721e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3291875064784779e+01,    -2.2822482674961009e+00,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -3.6468197945290555e+01,     9.8997780559559669e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1582418885375525e+01,    -1.8788892807416814e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.2883248676036260e+01,     1.2974167019815444e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1623640901485515e+00,    -7.2318447846373110e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1262451256502480e+01,     1.5360012514292503e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.2667107275401737e+00,     7.7570508185929684e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6711568623460806e+00,     1.6335355185169661e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.3033005525861626e+01,     3.0950521846385753e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2919349395917012e+01,     7.0870325877721525e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6507469265756335e+01,     2.6547341441587614e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8632023360047668e+01,     2.3731726092983736e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.1423809900570692e+01,     2.3471865789616368e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3762965194946515e+01,     1.7685200949887792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8737881496731930e+01,     1.4523726935285162e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3058555204167156e+01,     2.9822783176360672e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0067423071535757e+01,     5.7875394212939682e+01,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    6.9485814734790834e+00,     1.3114843581913192e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3720617957101030e+01,     1.5606744014117514e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.6871199124027232e+00,     5.1277145483454910e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6143011703299479e+01,     2.6374686649085063e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    9.2305690682277319e+00,     1.5078740015078574e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.2410300733974893e+01,     2.4981007202228109e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.1230356177624071e+01,     2.2354979717236751e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.4416878211133639e+00,     3.7620319384527220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5547903898074098e+01,     1.8562969120890393e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.5208332039006152e+01,     2.3227390386347903e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4750351602121491e+01,     2.3238207097707622e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.1423092770654577e+01,     2.4879656358448163e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5614853490722552e+01,     4.3670015001320216e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.7745431709397508e+01,     8.3158895453699224e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7388239781353114e-01,     6.6698664970251411e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.6704667343382610e+01,     3.3534602717509188e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.2557080675322453e+01,     9.6008486231046427e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0218883983134845e+01,     7.5475805174950692e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2570787138616865e+01,     1.6267964675792184e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.5975286848768867e+01,     8.3813082059782428e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2312222597308642e+01,     2.5239394573515256e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.2991554106082898e+01,     7.9631538793211860e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8450339734822716e+00,     8.3920581465112518e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.2801741379519342e+01,    -1.7653260435850461e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.6364005450134712e+01,     8.4887269789052934e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0167144418308382e+01,     2.7737221062026292e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.9935368617668541e+00,    -3.8657753471172064e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.8199880714711220e+01,    -7.2386707644139351e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.7415731598798956e+01,     8.3114201629556574e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3520056295158252e+01,     3.2969710797284133e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0258398841318471e+01,    -4.8667329017882491e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4948671368226407e+01,     1.5397639174165908e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    7.6680544452932420e+00,     1.6684605321353487e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.0358076784065986e+00,     6.6478884303862242e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.5099307685881797e+01,     1.3171862876784655e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.5390687407887967e+01,     6.5861434222068256e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.3174627985075404e+01,     1.5380168189515302e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0365554151369917e+01,     8.5262406809634704e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.3164899934848622e+00,     7.6797652771686080e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0072983897084974e+01,    -1.6922434209607673e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.3610013767481085e+01,     8.3414933040803298e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.2598054749827323e+00,    -1.2598710250030740e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1652082926284452e+01,     1.6036553430973767e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.5376489046854314e+00,    -8.6123836969761669e-01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.4959202683058891e+01,    -3.6212105862994122e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9942555995755455e+01,     2.5557293960556151e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.9701969576665412e+00,     4.4878173090099364e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8979624304581762e+01,    -3.9795200184890564e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2180819874880896e+01,     6.5032719872452490e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7065435474476608e+01,     1.6918817200420162e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.2215949106250505e+01,     2.2189589172981528e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.7552982054638768e+01,     1.9702017974115290e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8462509027025021e+01,     2.2772236266681983e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4987608414204860e+00,     1.6659685784287376e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4824534399687927e+01,     1.2778489591133592e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -8.5496978913614630e+00,     7.7559580340527882e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.1642405376226684e+01,     1.5551632678636695e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.3378549369468850e+01,     2.2182190013275314e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.2978568990133589e+01,     2.4148419059716041e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.2940831163482340e+01,     9.0133223424046901e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.7347460815813331e+00,    -1.7068277937948285e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5030515051777265e+01,     1.0055418233695058e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4119463156154666e+00,     1.3140857638516874e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0902385210917525e+02,     2.2781090029252271e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.0444601221342609e+01,     3.0774501497203219e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2592175722781164e+01,    -1.2765595241380311e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.6011002278947837e+00,     6.4146178035882235e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4887045755485691e+01,     7.2146844932338372e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.1735449597584417e+01,     7.6385640628150504e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.5484177201765146e+00,    -8.9635579653703971e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    8.1696194760584305e+01,    -1.3795349552833568e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0008191004787850e+01,     8.1955232682384758e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.8984498041303191e+00,     5.8875836405220834e+03,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.6930888987688910e+01,     1.1617475625410001e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6312895761408029e+01,    -7.9230512167097595e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.1903602774910598e+02,    -1.0569478742247043e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.5314836789710881e+01,     7.6996792077634318e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2126643018352735e+02,     1.4912411188121339e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6495276189721558e+01,     1.7052705018982975e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2515335839664843e+01,     3.2035843829694051e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -1.5352852121629979e+01,     2.8560984319038744e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.3969893434811897e+00,     8.8304422528373107e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.1565974426471222e-01,    -2.2161392073331317e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5463464352210883e+00,     1.4303059800901652e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3359634178411813e+01,     2.0352305931027571e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0722177753259029e+01,     2.3867036487740017e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.3933852025042928e+00,     8.0830336083773955e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.1619118928404131e+00,     1.3183937164659126e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0436625640384953e+00,     7.7675743754826535e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.3354727943414872e+01,     2.6440032985277598e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.2041339736374567e+01,     2.4080897129165969e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -1.2356010439009875e+01,    -8.3260768517769939e+03,    -7.7255447935749440e-04,    -9.1067401859615238e-07,     4.3024790130065763e-09, } , 
      {    1.1406050033362600e+02,     2.4081093232313644e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.1302472861246127e+01,     2.4230685257099107e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.5612022342689425e+01,     9.3308751551403093e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9387888991638818e+00,    -1.8922688384385248e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -4.1276044548856028e-01,     9.0929521462005141e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.9009504561417199e+00,     1.1157320866587174e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8398373636381979e+00,    -8.6779415609089556e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.2971167901359024e+00,     4.8624960029006361e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.9164776360471514e+00,    -8.6039179247279378e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.7974081274758547e+00,     3.7668962660941470e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.0598578711289436e+01,    -6.9587606984853278e+02,     5.9190587197770524e-05,     7.4463212020767181e-08,    -4.0360254138687761e-10, } , 
      {    2.0695200729167077e+01,     8.7795877886310536e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.7395912174473064e+01,    -1.5514402567713989e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5273886565209482e+01,     3.3837135089523661e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    7.0656668700139171e+00,     6.0847157977679799e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8968787057392866e+00,     1.6861502610247982e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6721545493867328e+01,     2.0914554216461864e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.1675593343001358e+00,    -1.0175238840445301e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.0718410178651268e+01,    -8.0493053433869627e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1608644919512244e+01,     2.2004556643759777e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -4.3417488726465038e+01,     2.4889171473612696e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.0787807898906481e+00,     4.0856428514305717e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0219115495801265e+01,     1.5462900829332100e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.2671589365710956e+00,     1.3435662830894229e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2202885022664795e+01,    -8.3936244474148698e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -9.2517901806600207e+01,     1.8874635879580183e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7775404744346940e+01,     1.3360411769721841e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9383663678133725e+01,     2.7331082181133846e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.8266772676985667e+01,     2.1659461593143355e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.0714378424916553e+01,     3.3337307604031565e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.4652040558668027e+01,     2.4691827932601562e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.3498163523196098e+01,     3.3994389722559259e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0160458040518852e+01,     3.0208296430591105e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8240656858368880e+01,     1.3220711099368731e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.4455896813687659e+00,    -5.4695255442409314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4271224442909070e+01,     2.3359385605541414e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0724330016201293e+01,     8.3512315948587930e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5179073409251412e+01,     3.3472500053240939e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -9.2683296723550939e+01,     2.4491425646747775e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3368037176374941e+01,     3.8339386347724663e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.0808378524076787e+01,    -8.2553969194729943e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.2344906642646023e+02,     2.3869410197892012e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.4679690946969338e+01,     2.5507963943493705e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.3262228827623364e+01,     2.6091449176580718e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.6245465080154160e+00,     2.5280320615749195e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.4021493644730963e+01,     1.3494165017417736e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.3071998217617846e+01,     7.8267340752783221e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.4529929271901629e+01,     2.4266767884441808e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1480979670273264e+01,     1.5186919801617478e+00,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.4575960966577606e+01,     3.1940286312052966e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.6600263491645904e+01,     7.1868329033945638e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.5087893973560911e+01,     1.6791202417289183e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0620433103712287e+01,     7.2134412376838291e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3453023366390923e+01,    -8.5217324598197392e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -6.0950169951314495e+00,     5.7200382819625872e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.6097365853600408e+01,     6.5853671386197548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6347711828978762e-01,     6.7308703964313427e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.0307777792214290e+01,     1.5629504814898115e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -9.7105354443355765e+01,     1.4916482222155491e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8477479458252830e+00,     1.6167248583416365e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3571310025182177e+01,     1.4940183854660248e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    1.0258721864079874e+01,    -6.6864796814823512e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1767063142381623e+01,     7.6982416014099581e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.7365285015390342e+01,     5.6406594486628746e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.9655687855352681e+01,     2.5317807423447505e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3265860882106324e-01,    -1.2930264299102337e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.4802634705973531e+01,     2.3873481231926158e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.5689029499706599e+00,     2.4726223446594133e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.7840952678375601e+01,    -1.3376845363683690e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.9886031198373511e+00,     7.3307284217478355e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0615107626991467e+00,     1.6603702406772187e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3516049347765609e+01,     2.0552763477994408e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4162126409737660e+01,     2.5960422658266652e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8418056211224467e+01,     3.2106837135106449e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.2472379245863570e+00,     4.8766604977190154e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1481520936126056e+01,     3.9428289352522534e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0959525178242579e+01,     8.9573922560331102e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.5165303234809002e+00,    -9.0305953391715466e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.5822292166956032e+01,     8.5996312951371383e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9423242514724166e+01,     2.2753014386033465e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.1492653437105576e+00,    -1.8374581844240256e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.0420227841068598e+01,     6.5351131062161421e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    6.2307760750344514e+01,     6.7327639592622290e-01,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    8.8227594474851600e+01,     2.5180081711717670e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    7.9206178283091546e+00,     1.7572092161059516e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.5092154457336164e+01,    -8.0284116087563634e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.3817641624694716e+01,     3.8259756543928830e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    6.8746254231054920e+00,     1.4412695877026230e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1703111055526261e+01,     8.3267244506736533e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5008808156893210e+00,     8.3120888431207804e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2710461999402682e+01,     8.3339510893574952e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4254251058704952e+00,     8.3234317600927589e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2382088808980125e+00,     2.3649277010755985e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.0263982341153628e+01,     4.1379304853557987e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.2722836084847566e+00,     1.2242646353908706e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6714072919820048e+01,    -1.6876816148513080e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6397330055350295e+01,     1.7725112070014129e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    3.2445564746116396e+01,    -8.5152761744126055e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.9242616660426211e+01,     2.0132027333258342e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5397564917230300e+00,    -8.4154324736531598e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.4118853742886778e+01,    -8.7761638994956429e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0495053535061640e+01,     7.7751799147013053e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.2666651109812506e+00,     1.3924269745586666e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.3874782101561003e+01,     2.0339188019828114e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6881138480492401e+00,    -7.0468690876452479e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0413967125638781e+01,     2.5125759685160439e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.1785806401430854e+00,     3.8346084925623768e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3850385063208012e+01,     8.3306583987766007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4646017273659593e+01,     1.5325501358818727e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    2.1406119055044663e+01,     1.9047754314401260e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3590068577409166e+01,     2.3350179851678349e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {   -5.3947014721633284e+00,    -3.7014820371770456e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2623758148719798e+01,     3.1256360845907806e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.4367894723622868e+01,     1.9486081898313289e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2350005112668860e+01,     2.3316761267592119e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -3.5478542329014267e+01,     8.5346714641196886e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3743409975294847e+01,     1.6711063292128318e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.0303371577061540e+01,    -1.0629051038045763e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3749373635141691e+01,    -9.9011678162528733e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.3566951648735994e+01,     1.6660364997724511e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.1038463372727357e+01,     1.6757899826605622e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.6059934169123984e+01,     1.7867098680838917e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -3.4719985254531906e+01,     1.6654400701175993e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.9395524083568887e-01,     2.2772549639790388e+04,    -5.3411709929817881e-04,    -3.4590486519803176e-07,     1.6210715055259552e-09, } , 
      {    2.0841903528312336e+01,     1.1386118133340991e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4341040660232409e+01,     3.8821940836449903e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.6612583748206735e+01,     7.5489034419919517e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.0588815939596606e+02,     8.7205551005354209e-01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8415250135974389e+01,     8.4188629240596292e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -3.4853590062788840e+01,     5.5040300807534425e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1445870548591557e+01,     1.6715258243663193e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.8741634481616984e+01,     8.1356503896305130e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.2481660258164675e+00,    -1.9132237850215970e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.6877893544361434e+01,     2.2760619925252118e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7138903949171159e+01,     2.4970326530659739e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.0674464790351355e+00,     7.2062954945595493e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1665296612578004e+01,     2.3480725945055830e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.7833345496976367e+00,     2.5665024033418067e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.7819688733876951e+01,     2.1179081706494508e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3453935189114759e+01,     3.2828797040637830e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.6180354569310367e+01,     5.5110763999149401e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8500759612410089e+00,    -8.3378802126410374e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    9.3115507389106718e+00,     7.2075114000201374e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3060414586981985e+01,     5.5824756690301328e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5051969520883635e+00,    -1.4274674356309729e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1691664567783862e+01,     8.0174908899354705e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5648280031085315e-02,     1.2747836668823886e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.4086580343510853e+01,     1.6175992907385425e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6493483158316486e+00,     6.2713464775240084e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8043738514142795e+01,     2.5410240277784091e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.1951652026111645e+00,     7.7739714994017495e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6504262135722627e+01,    -1.7981913190505340e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -1.7061612659825003e+00,     5.0474528291443039e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.0630569765914586e+01,     1.4372768800731325e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3933262062149825e+00,     2.3388253245881591e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.8263771454438864e+01,     2.3311501602959754e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0425876222276107e+01,     7.9820900724297226e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.7866742818218961e+00,    -8.4166054109463039e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.3943099576379048e+00,     1.5523576096261855e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    4.4505854223929845e+01,    -1.7981208558589195e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    3.6258104080780335e+01,     3.8874909933661096e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.6590479300431948e+01,     2.4103335249453929e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.4929255808361717e+00,     7.7621883571760682e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4890041635484391e+01,     7.8953300719294457e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.2546328546730683e+00,    -7.9448781945202900e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.0295724831333429e+01,     6.4564068594831169e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.3157199239673588e+01,     2.8696223601709065e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8672105197621491e+01,     2.3035558319663687e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2183718431469948e+01,     6.0763750747097856e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0034557852128696e+01,     3.3411777830599760e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1549437388346163e+01,    -1.3106715138256609e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.5503973311214967e+00,     2.9865362353179705e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0757703428941079e+02,     8.3290571677846528e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2171292139468397e+02,     8.3283256816655976e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6407329381908312e+01,     8.9258474145838882e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.2765127317710085e+01,     1.0278048529674425e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6865083738383957e+00,     6.7926559623555868e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9078257936299440e+01,     3.1964850524281654e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9803739532158584e+01,     8.6289712406938888e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.2042835588269071e+01,     8.3634392546757699e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    9.0935845997410567e+00,    -8.3036636350450790e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -7.8723415035334874e+00,    -4.5121166161500751e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6139782020606322e+01,     8.8451607612618682e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4557417859020376e+01,     1.5521178787500581e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1384924750267952e+01,     1.6077761870701739e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.3718049709968540e+01,     7.9618151561981122e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.7887023806756439e+01,     1.4523297563001919e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.0279147465223364e-01,     1.9184868493072997e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3506953862851212e+01,    -3.3307525072313765e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4536312153795119e+01,     2.0163404722132313e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0618461622519078e+01,     3.7842116126321089e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    7.8059726486005658e+00,    -8.2816223816003512e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.9765568525875128e+01,     1.6146478711428566e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6445188800621239e+01,     6.9979544440283817e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.3423161977693507e+01,     7.7997630785022375e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.3210612952327931e+00,     2.5301561934428482e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    7.1247902172360078e+00,     1.7807348479652079e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.6117062101437902e+00,    -1.0623708075219783e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.9074009611388163e+01,     1.1508313283372327e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4453513189630236e+00,     3.5315281773409856e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0901282500388604e+01,     1.5495854951481542e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    1.4786326808520711e+01,     1.1708145797621582e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4952767308617958e+01,     2.9039501100354264e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1125492907716348e+01,     1.6570632324736096e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.1866172743331113e+01,     7.0258433076059628e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.9168184030639459e+01,     3.6885221921395227e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.3222770801380111e+01,     1.6963662770215204e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.8742639224001806e+01,     2.3871651430136299e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.0083169734131680e+01,     2.3850827152398109e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.9372060625101390e+00,     8.3213686797049577e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2963262425088782e+01,     2.3736415813625604e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4905132817507621e+01,     1.9977318645321910e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1731701372241448e+00,    -6.6325247405552943e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -8.4170739288486658e+00,     2.3220431908152259e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.0448197175455842e+00,    -1.0755758442516617e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -9.8403904233097599e+00,     1.9290222211222381e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2927770497371688e+01,     2.4455982466445468e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.7122081660541653e+01,     7.5137990899921024e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6628642673027798e+00,     2.4277584595801527e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.6579606208236253e+01,     1.0247353307562662e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.8414234228526247e-01,    -8.3900156081104569e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.0610493486266144e+01,     1.7342013767129818e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5929845280591108e+01,     1.5624109320805406e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1717829258486393e+01,     2.8822983966577493e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.2779902368515943e+00,    -1.7196672920882196e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    5.2235238781011004e+00,     4.3078115555836730e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6127806979183092e+01,     1.5361919179908933e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0741612469998669e+01,    -1.5757219301759235e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.2067507368780731e+01,     1.1203791139413504e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.0594900901079249e+00,     7.7423323101684100e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8906603458103604e+01,     1.0700304204036074e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.0042227895329390e-01,     2.4228048863511867e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.6388856949644833e+01,     2.5109291403424395e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0022608545408150e+02,     1.1468498381474414e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.9946978172346341e+00,    -1.6244578410657784e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.6091523656778104e+01,     1.8231029782720980e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8440075220179498e+01,     7.2218302363581161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.8107670529131266e+00,    -8.5070624870413631e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    9.4207769777455734e+00,     1.4413407222493106e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5599257990892447e+01,    -1.7278970396143507e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    7.4205921135586381e+00,     1.6632799685649323e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.2918963413909530e+01,     7.6895671283198626e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3499044384530476e-01,     3.0457529627798824e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    8.5079355433714401e+00,    -3.4233543543109931e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.2017004115008163e+00,     2.6247898223844495e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.7226346354895554e+00,     3.9459421810774656e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.8928905062197117e+01,     7.7956500187014617e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5783271767392222e+00,     1.4134963721352167e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0939469983088213e+01,    -5.4144456649482890e+00,     4.6359532554209602e-04,     5.4764553418133182e-07,    -2.5876930229221479e-09, } , 
      {   -1.8734992124878097e+01,     8.3271015703522680e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.1906818681424468e+01,     7.8665489771762332e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.4698220848135914e+00,    -2.7347058283469660e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8858061388600373e+01,     2.3889479398896408e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -2.1118359363282412e+01,     1.6656254586903658e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.8731552917035735e+01,     6.5362413687627341e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.8763945750132876e+01,     1.5542944746043117e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -3.8545300503143523e+01,     1.5542563834324801e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -7.3574966686316259e+00,     3.1068235583601061e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    8.6060414845349342e+00,    -4.0876871320862620e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5655052017161296e+01,     9.0306221474992490e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.6194344628677300e+01,     3.2025335503754133e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.3876427527950046e+00,    -1.0640416577057707e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.6296329102311145e+00,     9.0591707186975509e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3069800241191984e+01,    -3.8815078312647966e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.4055204395231975e+00,    -1.7421239489518774e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3760507466554570e+01,     5.4968638706688307e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.7933437149428784e+00,     1.8433586489019930e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.8087731076652673e+00,    -1.6199927529944594e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -5.4668234692385909e-01,     4.1474473689270562e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4677223956452835e+01,     2.2161239479650831e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.5132849393701377e+01,     2.2327584080086810e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.2405059516319454e+01,     5.8921860402015518e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    2.5432768828306347e+01,     2.1606851525440965e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.1472493295558438e+01,     9.6036689112799868e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.2889622327471379e+01,    -1.6686233244797299e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {   -5.7996450723444475e+01,    -1.2688210949106733e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1012865987542321e+01,     7.2320965494461561e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {    3.4755196418249469e+01,     1.8123048812494984e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6514702672674957e+01,     9.5787476392164845e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0896960923668031e+01,     2.3399718498483071e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.7958521495341664e+01,    -3.2815025055285787e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2796753588341211e+01,     1.3664831411889956e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6313299367104484e+01,     1.7808771170585831e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.9058136917649993e+00,     7.2691742009396485e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.4681951734707283e+01,     1.3290955266520315e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.2180686374628905e+01,     3.6916388775950058e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.4416850001867001e+00,     2.8004437673651275e+00,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.3501297875241747e+00,     6.5857639274313442e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8357518601930934e+01,     2.8259110731344619e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4471635467496634e+01,     8.3342735891873381e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2902118976582626e+01,    -8.3231092602629124e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.4996318986401125e+01,     3.1069509072043431e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1062029385504328e+01,     8.1442787279623406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0660828175397350e+01,     1.5359282786321692e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.6819281763600500e+01,     2.2661257181765002e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.7998699590641593e+01,     2.2128411519023408e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.1407026718042275e+00,     7.7006074967356162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0353600303416943e+00,     2.0368038415175160e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.0023287225979391e+01,     4.1523784010829750e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.2402486468614740e+01,     8.3782053363756804e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.1481280673687824e+01,     3.2726273015752875e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -6.3258733079297791e-01,     7.7002354226223688e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7443745987699856e+01,     8.6073957380105130e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.5488415090874188e+00,    -9.8411667682909865e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -4.4874914103684659e+01,     1.6458308339960691e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3235763354673786e+01,    -9.6579365217694103e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9795813352624126e+01,     1.4941107411723049e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3510911172675890e+01,     1.4854928128113615e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.1735080092200237e+01,     7.1894702258717789e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.8046596116181917e+01,     1.5539641044358745e+04,     2.4509445367736679e-04,     3.6118134428979317e-07,    -1.7100832973776660e-09, } , 
      {   -2.4111136176372128e+01,     1.5545867536009169e+04,    -3.7747069612412869e-04,    -2.3328749521309762e-07,     1.0926245931165610e-09, } , 
      {   -9.8708983244240738e+00,     3.8749488756650690e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.0821702722735694e+01,     1.6858295816897549e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {   -1.5572558134580394e+01,     2.9342220120098005e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -4.1433697759063030e+01,     2.3871057573115548e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5130660225249365e+01,     4.5477585894883174e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    7.0080284933687523e+01,     2.3871833856702622e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.8514549654982715e+00,     1.0866872517351480e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1488510935036951e+00,     1.6681966013251185e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5223684002176196e+01,     1.8464753343574761e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0632897248036475e+01,     2.4087554136339535e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.2210486157567329e+01,     3.2231315543148692e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.5272123027285041e+01,     6.4434400371120482e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4423189295914909e+01,     2.5851540302129972e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.5168217321526853e+01,     7.7756349008519719e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.0728174622757916e+00,     7.5864203188483107e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6818070209936252e+01,     1.2396944892823053e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9748144857125929e+01,     6.2829058100548045e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -1.1179704688628451e+01,     3.3315094331376953e+04,     7.6156375320674127e-04,     1.2520555721494321e-06,    -5.9316953883751721e-09, } , 
      {    3.9448919227413036e+01,     3.3314437066424056e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2467030195738658e+01,     1.5862958415777084e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.7722266608886898e+01,    -9.9521828633432269e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    9.8890641965110682e+00,     2.4375800373117159e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.5814727710189769e+00,    -9.7768223766596075e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4505039222419553e+01,     8.2625681322006567e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9230071545740152e+01,    -8.2845919587395183e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6503356858787040e+01,     1.5560787974171282e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    6.9903921492540428e+00,     8.9558355482636471e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.8474334028579229e+01,     1.6941356071099839e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6543390548173441e+01,     1.9312599800457217e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1832281403669640e+01,     2.6552169510069657e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2130226298108786e+00,    -2.4242128265685710e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.3294792728678146e+00,     4.8742450161317419e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.0835248117796880e+01,     1.9808373326125293e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.3753198919897741e+01,     3.4115113931920496e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4130615290998165e+01,     1.9798920774677968e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6395104600220328e+01,     7.2083730373432491e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7951872352941960e+00,    -3.7080409927767741e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.9239974159018772e+01,     1.6441035343193835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4492998274957561e+01,     1.6672101357874810e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0919980998478152e+01,     2.0011100072824876e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.8993140328316036e+00,     1.5577646735240380e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.2497935475929381e+01,     1.1151710134769542e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6222091923633300e+01,     8.2250202192021879e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.6099085957173205e+01,     8.3152211975675946e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5641177293567065e+01,     8.2998257700100330e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.1661292454401490e+01,     7.9557816818031642e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.2984510286935649e+00,     1.3026894117685602e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3895381316337867e+01,     1.2020833585157874e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.8810171906482793e+01,    -2.6413701069960371e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3061995606780219e+01,     2.3306416508330221e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0443260170486685e+01,     2.4358478034596712e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8374874430530589e+01,    -1.5448280372699537e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8188493730198971e+01,     3.0317515584337314e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0177917431512678e+02,     8.3294318824767379e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.7643287209853611e+01,     8.3279509669735144e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3036597029158965e+00,    -1.0036682680026977e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -7.9204249623866218e+01,     1.0817648891297815e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.2618975495577757e+01,     8.3294094837427292e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0471677293671348e+01,     8.3279733657075230e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7718222966800742e+01,     1.4880665277635449e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.4645955985960406e+01,     2.3072285864323654e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6340569947489481e+01,     8.3293214313642293e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4850271835371744e+01,     2.3892064277420061e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9199935869610414e+01,     2.2552697394845065e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.9827516511195732e+01,     1.6113973976564735e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2319168785195352e+01,     6.2326024626791905e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.1061681589331585e+01,     6.3069541802069375e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8199888856598470e+01,     5.2201707466508583e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0283856275944069e+01,    -1.3549297719249051e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.6317220477679829e+01,     8.1416838640254609e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.6042135154415490e+01,     1.1020319635551239e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.7510473230819494e+00,     2.4500081932431076e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.7205283626092385e+00,     7.7959697846790323e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4818701787217753e+01,     4.1654963122824302e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7996070997998373e+01,     2.1657936339892138e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4175546703502363e+01,     9.9188089874844009e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.8740836325705823e+00,     2.3469608276503044e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -4.0497892883850184e+01,     1.4213792167427528e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.8983682962399435e+01,    -5.6691718046425331e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.4712298926437761e+01,     2.3882262426268804e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -8.1031694139523092e-01,    -3.9433276081293302e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0329204817189083e+01,     1.4898698961622777e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3681876539121546e+01,     1.6658511111996846e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8283384884568513e+01,     8.6410316282127314e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    4.4859568405541808e+01,     1.5389950967999444e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2582197123378924e+00,    -1.8733821386430282e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.2152734443731372e+00,     3.4112807847878852e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -7.1716260034136425e+00,     7.2197526935744163e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8960048693715187e+01,    -8.5513004764312063e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.2122952773146101e-01,    -9.4112750983798207e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.7972234018253104e+01,     1.0717470165192753e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.3731044159660311e+00,     8.9466897285151808e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1181042752680526e+01,     6.6062273256480694e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4401144318248758e-01,    -1.7772722754183422e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    7.4241411323118180e+00,    -5.0318172091493341e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -5.3240599225423839e+00,     1.1909238973566187e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3470904498876157e+00,    -1.7771300063249670e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    6.2153908513538862e+00,    -8.6081306685758191e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -8.2693094840374943e+00,     3.7780344551551082e+00,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.9358062474485017e+01,     3.1982884208268977e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -1.6871908898350846e+01,     1.6889474220412030e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    7.1650495571638331e+00,     8.3563135687149424e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5217546400456044e+01,     6.6082971855443466e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -4.0290713530128478e+01,     1.6843624483613894e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5011027644569708e+01,    -2.4970341790027785e+04,    -7.0636635813411345e-04,    -9.7462094313450016e-07,     4.6112083651371291e-09, } , 
      {    8.0163446075315186e+00,     7.3118340168953519e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1315662405402875e+00,     3.2182088196278837e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    7.1309352228020568e+00,     1.3184628472026034e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8823329851091772e+00,     2.3260058810672323e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.3274621158440652e+01,     7.7010884715957327e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5412808855820142e+01,     1.7223870657631262e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    9.4177920656143090e+00,     2.7941340822824015e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0236674278226518e+01,     5.6878900802074475e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -2.6467072328254758e+01,     7.2171761112840431e+03,    -5.2978344676547697e-04,    -4.8369860964298404e-07,     2.2789636707915955e-09, } , 
      {    5.0584823042017142e+00,     7.1566845410785700e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9799648404387259e+01,     1.7285338118537722e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8605231066330603e+01,     3.8416081425352422e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0697807261078742e+01,     8.9345645100634993e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1827539224349174e+01,     1.5135993781516514e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1498441909070092e+01,     2.4400426505085539e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    9.7993296606883202e+00,    -8.7348303056175682e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1956424928226337e+01,     3.1569199722272482e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.1125601554443907e+01,     1.6665009433076524e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5694621656504960e+01,     3.2299463934503299e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.2787425196938162e+00,     1.6649756265823980e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1442105977438626e+01,     1.6149020621784774e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.2251841887975658e+01,     7.7955556717638374e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2473694537509569e+01,    -7.7622036165441132e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.5235400772547628e+01,     1.1757659709272137e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.8429724617785507e+01,     1.6728023838904573e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.9283578139910684e+01,     1.5398466479519431e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -3.6667749612153735e+01,     2.5223232717084422e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5571520984912041e+01,     8.9175364238308321e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3575758793072083e+01,    -5.3296387831992779e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.3145030100924220e+01,     8.9541540957128782e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3297358962007225e+01,     3.1728409923208197e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -7.2487326021049341e+01,    -1.1979702165181186e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5427352210570866e+01,     9.6206436657517479e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6862641511993651e+01,     2.9120403731902981e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.8664208024204491e+01,     2.5526012886849083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4939641681820977e+00,     1.3110601138975599e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.7936428948908109e+01,     3.1103532788569122e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {   -1.2070608697467371e+01,     3.2080692433940476e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -7.4472348232528887e+01,     5.9729897236561756e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9196371026465879e+00,     1.6121398477794957e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.8387955780539627e+01,     1.4427766701408864e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7923736595899484e+01,     6.6504563498137213e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5011864058219422e-01,     1.4825429191524814e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.6408289337645357e+01,     7.2142533213179913e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -4.0900753235159797e+01,     7.2138724095996749e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4670449700643630e+01,     5.9754966387231198e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0245112143109914e+01,     1.5458146003739428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1649063138470961e+00,    -8.3566327322562702e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0739015841272495e+01,    -6.9209008558933356e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.4874569118129976e+01,     4.1707891473438307e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.9380717908899641e+01,     7.3030803790724349e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.3166042077301796e+01,     1.4889863541337481e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.6710144210523508e+00,     8.3948147172495956e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0256875345317971e+01,     6.9831834916502594e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1084549444884084e+01,    -8.2565536093779501e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {   -2.2856546375107897e+01,     4.1531240944030469e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.0366434921445634e+01,     2.2066152763568280e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.5832382672446528e+00,     1.5289418731476029e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.6800898809891546e+01,     1.4428484760426467e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.1198839640800884e+00,     1.7494528554859640e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -9.0495582048394994e+01,    -5.5741233120698564e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.7469379343363343e+01,     3.1204953286061653e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.1688289728720022e+01,     4.1772325823250962e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.0560626395614820e+02,    -5.5721622805930781e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.3326581774327380e+01,     7.1499808316042381e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9998758906217216e+00,    -1.1462677484996329e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1379042824131371e+01,     5.8254504905491485e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    6.9496593478135473e+00,     8.9776175722816442e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0181825724132615e+01,     3.1782090365886266e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -4.0735093940283292e+01,     7.9063638790741461e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3824898541692832e+01,    -7.6359494898669145e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3723368540015015e+01,     2.4102325088717658e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.6739470906165856e+01,     1.7039587107783518e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5088947030259818e+00,     7.0191166332089688e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.3744243194763257e+01,     1.4914058563344881e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4513073500243529e+01,     2.2135068526196974e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.6253298380950106e+01,     2.1605428834507213e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0837139892294676e+01,    -8.6169042919039930e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.8517224689728661e+01,     6.5771886130044308e+00,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3540677444291106e+01,     1.4366321213421179e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8280336526060694e+01,     1.5805822228243291e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.1443958769395586e+00,     3.5529212113844769e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8455813709469776e+01,     2.3872541595877061e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.7102382793783363e+01,     2.3870349833941109e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3552411297985358e+01,    -7.5606984286945244e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.4927273722745387e+00,     8.9208906592536623e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.9135776447560843e+01,     1.6504579527265738e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3896937386039127e+01,     6.2562522256177544e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.8961474174558877e+00,    -1.6656279322057828e+04,    -8.6084593504181270e-04,    -1.1561351853419104e-06,     5.4686013601793429e-09, } , 
      {    1.3378834947188334e+01,     6.5651367179023146e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.7535700225013457e+01,     3.9266865373491248e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1138622261504070e+00,     7.9608487537504334e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3530793188863768e+01,     1.6090879781901196e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9237217901796260e+01,     2.3156397080935505e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.0622025542517441e+01,     2.4344648777930379e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0858819671273013e+01,     6.2729164040486103e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3782509285537792e+01,     2.5105518979869114e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.9930041997457941e+01,    -1.6766415338125607e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    1.0951808273019553e+01,    -1.4333038458795294e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.4924576623581022e+01,     1.5545031806542067e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5643732289981800e+01,     3.1494277293576542e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.3203105261965273e+00,     6.7772710622566847e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.5293586153486860e+00,     2.1947044114481482e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2004524781528316e+01,     2.4059665272761955e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    3.2727212940656123e+01,     7.6961260839195893e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.9454007857033968e+00,     4.7437600031443765e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2729802276364246e+01,     2.3325621423031589e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8493540181347363e+01,     3.7659163869897166e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.3519864870346851e+01,     3.9337328565106225e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5632394648191408e+00,     9.8615449402212960e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3541032493482440e+01,     1.6035108738799658e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.7540985696117968e+00,     1.4998557921750267e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2999446168413895e+00,     6.6659543396887502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4503622887009744e+00,     7.6929056731105256e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4645597832094897e-01,    -1.0169843346352596e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -6.9879209887634929e+01,     1.6882474343569404e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.7728674440140097e+01,     7.1492761996880899e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0204178471227522e+01,     2.4866629568481643e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.6091043954835314e+01,     8.3290217921989624e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.0226931060108498e+01,     8.3283610572512898e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4313671005031793e+01,     1.5456013241255921e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.0021110425806576e+01,     5.5486942415271697e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.3476662186516243e+01,     1.6949415608449017e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.0852016498039916e+00,     5.3019176384189668e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.7713941090354858e+01,     5.1331697981262550e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.4283387982256638e+01,     1.0018169274734815e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.7740800779809565e+00,     7.7948298378624759e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.3373234696749890e+01,     1.4902956717382769e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.9825047594817633e-01,     7.7001601826235492e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.8446493893743781e+00,     5.9640066509674743e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2920837411653769e+01,     5.5306452981717930e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    9.9908806268206156e+00,     7.3623137728580332e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0571621912601145e+01,    -1.3656476587664383e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.8600823953330149e+01,     4.7807325779630839e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.3884045787570342e+01,     2.3278083018873531e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.8320948588543654e+01,     1.5753457741735583e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6313353867120114e+01,     7.8274500123422640e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.4492842039246735e+01,    -3.4884868615718078e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3308160978494175e+01,     1.6922340360000912e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.9965905930217609e+00,     4.6917448933132619e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3157285992979672e+01,     2.4011165336834430e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.8027412554994282e+01,     1.6676118403118333e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.7031176798490235e+00,    -7.6843843313550969e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.9196908246619611e+00,     6.9930089045793966e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.0546266337657698e+01,     1.5781643701571433e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8024610162982913e+01,     6.6747727940268915e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.1536815264665332e+00,    -1.7004003153317899e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.4384053686103719e+00,    -1.4273882545445183e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4394079030619533e+01,     1.8128614557319328e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3594543192355143e+01,     1.2227734267100172e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.8262947072561602e-01,     4.5404683386200441e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7063054329297085e+00,     6.5147732345950153e+03,    -2.7766297625873455e-05,     3.0740678040848568e-08,    -4.5485219561696674e-11, } , 
      {    1.6497688483018528e+01,     1.8151544604243922e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9901947181282100e+00,     9.5852953352220902e+03,     1.4698498765553121e-04,     2.5164389865821173e-07,    -1.2952185341786140e-09, } , 
      {    1.0724587356489549e+01,     2.3020652968042837e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8113020601425610e+01,     2.6695169056869654e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.0178753204588130e+01,    -1.9164029827998602e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.7255577264987707e+00,     1.2210096417200662e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6550056865361363e+01,     1.6379644938612544e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3221681431355227e+01,     1.4601607828773735e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.3421810208578727e+01,     7.2012609861036281e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2828200497641951e+00,     7.3242855364204643e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.2299329585383894e+01,     9.1983459078458764e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.8503593245377015e+01,     4.2367389200269031e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -1.0473208106394199e+02,     1.0211883211383329e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0491594206890698e+00,    -9.6708936580233458e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.1192906103335343e+01,     6.6387149018383598e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.4257966018763426e+00,     5.2784979361250562e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    7.1484237121033729e+00,     1.6845602407303122e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    8.5814063258144188e+01,     1.6851390286992544e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.3497086089770740e+01,     8.5333098583999272e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.5774320394700929e+00,     1.5398137847042981e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.4656817263117112e+01,     1.6888507817964353e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.8375714386802819e+01,     2.5931170105925790e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.0820178073142220e+01,     1.4190224542616974e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -9.4112552226892099e+00,     2.3738583023055144e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.5193693258670486e+01,     1.6018258553044989e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1277911556175093e+01,     1.5535750708705118e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -9.6020836022011480e+00,    -5.5360750032047890e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6050686275045074e+01,     4.6235241900858884e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    2.8264338035208002e+01,     7.2447157877685386e+03,    -5.5838726002546712e-04,    -5.8641608425966146e-07,     2.7669771221964547e-09, } , 
      {    1.9511104320127792e+01,     3.3308206743300783e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    2.0535082246932305e+01,     6.0218820451804149e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.3825585933200708e+00,     5.5709856812744865e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.3325082441481246e+01,     4.6080311936259732e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -4.5818185605912376e+01,     5.4572285500835278e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.7205012722760635e+00,     4.4694750332076474e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.5178743018472645e+00,     2.1961271023819008e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3950712155547338e+01,     7.2067331159469904e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5013946029698413e+01,     2.3090878399114517e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9789175181539083e+01,     2.5128060885000170e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0025528126550359e+01,     1.1803335908533216e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.6626836087533164e+01,     2.7600694235928131e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.4495141784712523e+01,    -8.8368972371841664e+01,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -5.5788524863128089e+01,     8.2565112605667855e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.7776051718627905e+01,     2.4634740033936796e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.7706355851930393e+00,     6.8740015171909163e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.6725948800131851e+00,     1.3515844924222536e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3378618801956385e+01,     8.3612543734773681e+01,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {   -3.7989965945327597e+00,    -3.3356835393872962e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.1989783256752302e+00,    -1.3525639585072736e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.2834986251430792e+01,     2.6225416447499607e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.6754168755644088e-01,     7.7163833480040685e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3724863147635400e+00,    -1.1352920755698024e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0575748993589528e+01,     1.6150443312718526e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7403950989586914e+00,     8.7489348663213514e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6712681013140951e+01,     8.2065438949608397e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.0367845580833730e+01,     8.3257484590934964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.5607953892900284e+00,     3.1599002416914909e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.7436487610823264e+00,     8.9989474802594959e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0422889607128166e+01,     2.5607822903621196e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    9.4503867498744953e+01,     8.3316343903567576e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1191004558284844e+01,     1.1725039534792350e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.3719623182965194e+01,     8.3225371062544746e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5866921384871603e+01,    -8.3348457431957795e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.6283351499221517e+01,    -7.3204807811860501e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.0437770478639592e+01,     8.0289552001918546e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.3127178498096725e+00,     1.6078738661839254e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.9607165579440881e+01,     3.1091542054762867e+04,    -1.0144900810142239e-04,     1.6481725902999783e-07,    -7.9204011082864902e-10, } , 
      {   -1.4047812524065776e+01,     1.6166130843073584e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8775737816982460e+01,     1.0444971671930769e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.7501024629746311e+00,    -9.6589029242170891e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.5719187002373449e+01,    -1.1244482885111029e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.9478047978500683e+00,     8.3254381689020447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2373775832101375e+00,     3.6843780589205488e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4023357717858310e+01,     2.2118200289341759e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.6798623006389604e+01,     3.9401082093893587e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    6.1880746915044575e+00,     8.3319446805482075e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9951564264602681e+01,     3.3806002445909122e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    3.0863627173872089e+01,    -8.8307448830934682e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.3823797680346887e+01,    -8.5360479439190385e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2666546338552370e+01,     1.4181373862963632e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5194728253017615e+00,    -1.0829893700329540e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0438487033912141e+01,     1.7215089311008949e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5491406807229623e+01,     8.3914005477121700e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0373201590944959e+01,     2.8423404041284775e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.0865590298415910e+01,     3.1260310216247992e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.5670429554356886e+00,     4.2643614355924342e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3347712573271650e+01,    -1.6903014942623675e+04,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {   -1.8841328177091321e-01,     3.7394183816507430e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8654163821679429e+01,     1.8544935436903066e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.2132099903284894e+00,     1.7278212267926738e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5720401593336611e+01,     6.2081534217594926e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2144481132202678e+00,     6.8084226574434661e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.9491927141255957e+01,     1.7021538164428144e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.9888214678090392e+00,     2.7668792719985031e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3970663272262197e+01,     1.2147567433139911e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.2889544163046615e+01,    -6.2937124647049450e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6974906129666881e+01,     1.3564531530519567e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.4921388217555457e+01,    -2.9377315375383914e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.0012105295019431e+00,    -8.8852943588913968e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.3690149424572205e+01,     2.4424502238782239e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2599876083452756e+01,     3.1192886337271757e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -1.0607276067579150e+01,    -5.3783406488643568e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0305865193590082e+01,     5.8526824382385057e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6408108793119169e+01,    -3.2270381947252815e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9944456133111032e+01,     2.5597045159681898e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6302843402417388e+01,    -5.9241649407418572e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.1236790889093442e+01,     3.3570715830556568e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.2295628554621228e+01,     2.3845708565387253e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -5.7817969513807071e+01,     2.2144051598885679e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1552002485356294e+01,     7.2213926149706749e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5440514867151544e+01,     1.7121397124555635e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6916505991870551e+01,     8.2637584020353825e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.7042848774798043e+00,    -1.6744133374164412e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.3973671224067044e+00,    -8.8867170498251489e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -3.7016694752612963e+01,    -9.0959074861470708e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.9597369451661347e+01,     1.0340196939015408e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1728215344443234e+00,    -1.6455877661600662e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -6.1126058491420565e+00,    -4.4143900037556864e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9658550914225330e+01,     3.3883554706921248e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.2151691967779053e+01,     1.2463020882452729e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.4043531700494839e+01,    -9.0952028542309181e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.4669731570195339e+01,    -1.8254194374798772e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8762225657670903e+01,    -2.3094239524928471e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2553256304307922e+01,     1.6896429266244690e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.2810560363564292e+01,     6.5864599123294392e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.3524008326527508e+01,     6.5850506484971384e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5543796884006046e+01,     2.3846862551108152e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9611307799186552e+01,     3.1294705730240563e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    6.0382839375890711e+00,    -3.1315485192215206e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.1430059251000344e+01,     3.1295410362156712e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    3.2362990722563566e+01,     3.2212756377956593e+04,     3.0029184874999460e-04,     6.3861597330472512e-07,    -3.0305703206157094e-09, } , 
      {    1.1842816909891212e+01,     1.2772000045434583e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8951476924781570e+01,     1.8588706270412225e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.5242543160698288e+01,     1.7355520278401171e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.6959512059850340e+01,     1.2066948789895832e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.1293287402211163e+01,     3.4819279059720811e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.6031248932927458e+00,     8.3784723605525760e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.1137755635027959e+00,     8.3776434766927941e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0281462180546496e+01,     3.4674992140399925e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.5468288693261776e+01,     1.1160020305904345e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4238726440710181e+01,     1.1964095929277644e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6406079407918376e+01,     9.4521384383295608e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5805900273280280e+01,     2.2741084671495199e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.1679120464860059e+01,     8.3511259244322955e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3826418666766465e+01,     8.3062569250179567e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4914452471636615e+01,     3.4387410692252248e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.6217859307155855e+01,     8.5201919472004738e+03,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.0444243115991856e+01,     8.2042398250105798e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.9340191171803461e+01,     2.9948338115970568e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.9119387970855868e+01,     1.1271154840463281e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2054758094154181e+00,     5.9885992905545272e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3236930446989858e+01,     2.1026998922530409e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4142494450754306e+01,    -7.4075241389665348e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.6833968256781759e+01,    -1.9524788965918317e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0804415337016010e+01,     8.3448326240834376e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3332714054948127e-01,     3.3903258382048138e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -1.8404270576940348e+01,     1.5168230575071448e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5925291422411558e+01,     2.1899781910844416e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -8.3574450354309377e+01,     1.9837004708097455e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6343697180761126e+01,     3.0616460602914023e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2224626306445725e+01,    -1.2957920916867565e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.2963045661469721e+01,    -8.7508418815374716e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.1406517739410527e+00,     1.3613804272203267e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2210262564599454e+01,     2.3438093837899527e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5947503981498354e+01,     9.0460549055785577e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6338618119343931e+01,     2.4901377398432283e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.4224428146422554e+01,     5.9798821950749161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.9043785725327375e+01,     1.4483482661104006e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.2023243487687703e-02,     6.0257043552370919e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.4247018742665183e+01,     2.2180767322341566e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.7291683710302603e+00,    -1.6585245034103078e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    2.3413232525888390e+01,     3.9230728501230777e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {   -7.0572438434017528e+01,    -1.7273941160234072e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5682738629178431e+01,    -2.2031482658952939e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6005235316356359e+01,     3.2808794700324233e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9619142374886156e+01,     2.3921109112343627e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.5915027323462255e+01,    -3.0954658918788227e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.8171450467327539e+01,     2.9048539559783961e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2772557895377616e+00,     8.3213616752132821e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.5157545171716876e+01,     7.0080828260642702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0634619483936682e+01,     1.6111543298204702e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -1.7238608152221694e+01,    -8.0312716593996338e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3401569860804408e+00,     2.2909076886391813e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2994195407311782e+01,     2.7126947160968070e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.3706172779915239e+01,     1.3505571804298306e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.2107180317293903e+01,     6.6383720616664059e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.8390553939447145e+01,     8.3149002371297465e+03,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -5.3101663083950953e+01,    -8.3424826123205057e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1178074870964299e+00,     2.5002073782499865e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.6989828210231053e+01,     3.6587802156069745e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.3115772047167866e+01,     8.3512350648535703e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5724657933622872e+00,     8.3360211742369702e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6942301532641508e+01,     1.6801670660114782e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4496333281512593e+01,     1.7436865833236501e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.2811932755771953e+00,     1.4356028898112843e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.4491861598839471e+00,     2.3772118920039997e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.7323644192702098e+01,     1.4996929998306457e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3083678341193812e+00,     2.5489870081579917e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.4241089940800599e+01,     4.0451276303347215e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.1167075633981709e+01,    -2.5208741651205557e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {   -3.7916423403746414e+01,     7.2268647448140373e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5650240558383782e+01,     2.4958844312111109e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.6193899144104925e+01,     3.3038914684297924e+02,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.6813962282067330e+00,     1.5054166327063762e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.4481286659044862e+01,    -8.1810016772116796e+01,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.6579641885203475e+01,     1.4308573619800041e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.5619685642135330e+00,     1.4905586549698952e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.1855212349702173e+01,     1.3781793937670915e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5505830536952548e+01,    -8.3552100413225198e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.2603766909918455e+01,     1.6011026321049336e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.6657611102956590e+01,     1.1364638739579148e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.4423531257554885e+01,     1.4210888058920484e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1389999412329360e+01,     7.7270445462642165e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9559110543231768e+01,     7.9461787809720572e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3330559934378014e+01,     9.9024136450720998e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5330676785811775e+01,     3.4767812423622854e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -4.8003466014043809e+01,     7.2070629886162060e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1594988809995890e+00,     1.3741942751659180e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1491955703977779e+00,    -1.8668890711493043e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.0391123638101604e+01,     3.9964955329126467e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.0229137380455020e+00,    -2.4992633229775103e+04,    -6.4234506317708347e-04,    -9.6967099545037180e-07,     4.5909916346348614e-09, } , 
      {    1.3948072064515005e+01,     8.3413296146197463e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2378555573600998e+01,    -8.3160532348305060e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.8840429324529190e+01,     4.0986283883864999e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.4140909168031126e+01,     1.4399739797507407e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.8751605527145934e+00,    -1.1899661579464977e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7667881262403803e+01,     5.1041494093072743e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.7711783733424241e+01,     2.1484454242036063e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.2651395353928541e+01,     1.4785098284208243e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2104287490581129e+01,     1.1859321196362287e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.3204075030537190e+00,     1.6468481640554994e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.8072221782644302e+00,     2.5211623248330587e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -4.6435256447417842e+01,     8.3303040888008836e+03,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -5.1146365591921651e+01,    -8.3270787606493686e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.6941526460131506e+01,     7.1038401944972011e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.1851289846491373e+01,     3.9431120685901828e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -3.5384174894667044e+00,     1.2197081344790467e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.6562416508476733e+01,    -1.6876006062232932e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6539498617040309e+01,     1.3666691690486790e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7339156470745703e+01,    -2.7024449221858340e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4324684468170295e+01,     2.5754041547031673e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    9.0669585465606595e-01,    -1.0387096277717827e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5971649485098762e+01,    -1.0954503653441886e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.3709642111919891e+00,     1.8040565519326941e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.7959561623005778e+00,     1.8070578388160866e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.4375867056789883e+01,     3.2115440263891120e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    3.9350081298006955e+01,    -1.6642462411288139e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    2.1993008990884103e+01,     2.4064074499931023e+04,    -7.1735708428486659e-04,    -6.3323938958122043e-07,     2.9819919897685337e-09, } , 
      {    4.5343108725840764e+01,     1.4931719701905273e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.6883880020044479e+00,     1.3531248775893677e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.4257505424111283e+00,    -2.6609752885164094e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5660927103682013e+01,     2.9526519757005035e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.7751870793774138e+00,     6.5771187820298601e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    7.1943012815043819e+00,    -8.5029038196203128e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.2854371431384209e+01,    -1.0326059008712866e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0500770225831371e+01,     1.8637865578464673e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5806867882794531e+01,     8.4596574088275556e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0483829164060587e+00,     1.3262180733038158e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9397895656981142e+01,     3.9325182491479449e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.2482209804131536e+01,     7.0225470836154345e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    9.5754813872883595e+00,     2.1222367107048285e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5659541626326222e+01,     4.8957882333126436e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.5770239168848569e+01,     6.3166872375577677e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -5.4576091689582409e+00,     1.9975017445482172e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.3971447531746087e+01,     1.9519869372654830e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0932462196943979e+01,     2.4446072857276562e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.6958217322589233e+00,     1.6097679878251849e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.6846640362103820e-01,    -3.2374677620716629e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1711886974205497e+01,    -8.3870654918151431e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.0436634422319196e+01,    -8.4602440036646094e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.5113408596636173e+00,    -1.6828235114990137e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.1174439188072686e+01,     6.0376297886960529e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1692827511230314e+01,     6.2845600210275711e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5409257571919728e+00,     2.3006426058705310e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0096342506181513e+00,     3.2973559840697802e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.2706323685714336e+01,     8.3114166929608818e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5633926245326224e+00,     1.3413356131778859e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.0798696472291290e+01,     2.7724103150826832e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.2904140797000743e+01,     4.3358499049452421e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.1957205380873281e+00,     6.4875395030976561e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8721426705530245e+01,     1.6617042466347561e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.4848541920963800e+02,    -2.6973709261055774e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.1826707934819805e+01,     2.3378565785088609e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    3.5305184395762574e+01,     2.0732208989521103e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2036654759788469e+01,     8.5742893069582587e+03,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {    2.0796839614554568e+01,     3.2428864799387040e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.8420103742347390e+01,     3.0162954653748558e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.7032751751883357e+01,     6.1001456516594162e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.2767691397873239e+01,     3.0509016106812662e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    2.0301772660542611e+01,     2.5635045375524431e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.5757419993636640e+01,     3.3747216562676471e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4959879146271454e+01,     2.4482973174924282e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.2997123267582950e+01,     8.3027819295748413e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.4580588131843314e+01,     3.0456872362845923e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.8092771956623377e+00,     1.0053995542761306e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3566566058926128e+01,     8.8908757172461992e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4492218106778296e+01,     1.6618534350000016e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.3877609204226715e+01,     8.8325474100561241e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    6.0024817340244496e+00,    -7.9422294106318659e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.3563886499760176e+01,     6.0987229607256622e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.2811098323363730e+00,     1.0325173662064033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6385084498857413e+01,    -8.5014811286865606e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.6734357462773112e+01,     1.0003814033596564e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9847152001942181e+01,     1.5369253240755648e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.2089330719334882e+01,     2.2648258136301624e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1355938655063693e+00,     1.0476525712924040e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8466673972181230e+01,    -8.3226311420024758e+03,    -2.7586509320370791e-04,    -3.9500194668399445e-07,     1.8691506661497048e-09, } , 
      {    4.8174092126489008e+01,     2.4811272638295297e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -7.3042355455088313e+00,     8.2862734988387347e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.9359752017466803e+01,     8.3237671061199326e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2748485467404510e+01,     8.9968407199962858e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7955666193661575e+00,     1.5345205033945615e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9163042649715582e+01,     4.7377102407814542e+04,    -2.6041883236082187e-04,     1.1799395370843885e-07,    -5.7702524325656986e-10, } , 
      {    6.6169023793215320e+00,     2.4279107082707656e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.8328374289437235e+01,     1.6654729333652445e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.0425187834876795e+01,     3.5309560988857857e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5962151476053583e+01,     9.0743929810084082e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.8916263261369895e+01,    -1.1409369498928518e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5998040411504126e+00,     1.5001197229852571e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.1444268250022738e+00,     4.0764540635146595e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5868204118789659e+00,    -8.6965340956998189e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.8205448608813626e+01,    -1.2615462022470056e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3105284940808643e+01,     2.3896028878710018e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.0480873218544218e+01,     7.8268284222159464e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3823655404926733e+01,     1.6784449022112931e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    8.9541170551157752e+00,    -7.3177340709817699e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3774671095143445e+01,     7.5786979017415615e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1127172065901494e+00,     1.2279894111656590e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -2.6730869375173398e+01,     1.2393421733242303e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.9176006492590901e+01,     7.8466282062643677e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.6679739265241526e+01,    -1.3700264013898752e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.3762258501776600e+02,    -6.8279533936256485e+00,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.3569064801250470e+01,     1.5305595847274912e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3111994207980374e+01,     4.2165346792943950e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.3769553579819700e+01,     9.3995338911142117e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.3150047893826851e+01,     2.9216004936270474e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.5893836634657932e+00,     4.7861521571880676e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8663463462164533e+01,     4.0896112384530490e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    2.6753175099970065e+01,     3.5160833393315843e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.2809873387700009e+02,    -1.1142829568589443e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.9119416532209321e+00,     1.3169209360986844e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2176172305190141e+01,     1.0881099426689009e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2158233436948777e+01,     8.5700383228003320e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    7.4572816285970021e+00,     3.1968247605089360e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0147556924154712e+01,     1.4984728665083934e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0454687052606999e+01,     7.6666024121766186e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.1203726617795821e+01,     3.0666889882787700e+00,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.9971274684057285e+01,     1.0193937313062572e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.9240038337436869e+00,     6.6195367079162534e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.3890136712548731e+01,     1.6246312936401890e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5563507903814607e+01,     2.4631750504773223e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.0220904071333980e+01,     6.7102670580542954e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -2.4612002503719438e+01,     2.5160875910055460e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.9641876419072567e+01,     3.0710536554030303e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6726358297929309e+00,     8.4837959467493747e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9219471210383404e+01,    -1.0064081165764146e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.1818948190510461e+01,     1.6740995393185025e+04,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {   -3.5797760432342507e-02,     5.0120330600663156e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.8605676560192833e+01,    -8.3322608511612852e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.0033084652287072e+00,     7.3319460337875280e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7536664769667272e+01,     3.9178913389740490e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    8.6323140253988981e+00,     5.5186634158687525e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.8021123134868393e+00,     8.3958714383243860e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1343842765451441e+01,    -8.2922497620602735e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8717464877567785e+01,     2.3969661492224714e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.9203962726891561e+01,     1.3753432739414826e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5411683422317569e+01,     3.3386862531705323e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6608797254428261e+01,     2.1589219499165301e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.7568108561451879e+01,     7.1273123407446756e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.7021546155626437e+00,     2.3249697085463271e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    3.9993604505669921e+01,     6.9035251889338888e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6172351313031768e+01,     1.4421566775317940e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    4.3349273170245596e+01,    -4.5565279254299821e+02,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.4847591289314996e+00,    -2.5131189390193435e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    1.9526742406750870e+01,     6.2571686605329714e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.4998123947994284e+01,    -4.9366178135062919e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.8628676575831751e+01,     1.4378462333483119e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.1577854033364185e+01,    -9.5792731204212650e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.6202651393139682e+00,    -1.6626204445935764e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -1.0003160224311458e+00,     7.9072845216218793e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.9964231584911531e+01,     1.6656401187634525e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.9386449873919133e+01,    -1.6658364511265980e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.4130730229327419e+01,     8.1850681923595312e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.9127400636758509e+01,     6.8021027866697068e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.1814054847736330e+01,    -5.0124141238286211e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.4066476938423556e+00,     2.0962564572717740e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6828262223471455e+01,     8.8505810940434530e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -6.1846410751131842e-01,    -8.2930362545851567e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.9569668044248033e+01,     1.3308228263287172e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9638831193639970e+01,    -7.6316917341069220e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.7666455377867099e+01,     2.3198256156018360e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.5325862473818557e+01,     3.2081099985584617e+04,    -5.6504433364351842e-04,    -3.8282827515133401e-07,     1.7956529120934990e-09, } , 
      {    1.2207667697641137e+01,     1.6727924038822504e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4779616083520430e+01,     2.0745326900720560e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2506509738255104e+01,     3.8473072495255066e+04,    -4.1273158300217014e-04,    -1.3241716072144757e-07,     6.0931383441846459e-10, } , 
      {    1.0983403263544533e+01,     1.1283957973373766e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.7762906949608592e+01,     1.8129607768094713e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.1358766079421792e+02,    -1.1149741616736417e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8485519344945995e+01,     3.2821885769080029e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    1.0785646069505713e+01,     9.9674539093759790e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2523381380438794e+00,     1.6145322740082580e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0530539955318565e+01,    -2.9972243913888788e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.5880675556064432e+01,     1.6226418805415842e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8080371905029558e+01,     1.0971617769886163e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.8973565580252505e+01,     1.5515145313105633e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.8110074195853549e+01,     9.4147509388667095e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4641801293954977e+01,     3.8457305800167182e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.2670620847507537e+01,     1.4182527848684533e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -2.3943727273193833e+01,     3.2828092408721677e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.7560967967498771e+00,     1.6921657288906184e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8503487802265440e+01,     3.0527090773342345e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.7931594191141947e+01,     2.6940362411280596e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3506152774231508e+01,     1.6145324725707669e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    3.1444498470297397e+01,     4.5453993707759637e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9453073795187748e+01,     1.4986151356017686e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0182090892599325e+00,     5.1779647115814987e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8344647450719345e+00,    -1.0203624773855561e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.4874964429841308e+01,     3.4465442674569204e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    5.1498298869475496e+00,    -9.7755772122937151e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1860989696725984e+01,     1.6408608459932086e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1904287820637229e+01,     7.3723560706028029e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.7113569754138815e+01,     1.3253072756637484e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0964578485248200e+01,    -4.8125390227348089e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.6216471900969310e+01,     2.1053695913228865e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    7.1533663903028462e+00,     2.5195400082950066e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.3494141694028695e+01,     2.5195847397062134e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.2636448645911207e+01,     1.5655249231722228e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.1047131315178236e+00,     4.6544318594513161e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.8757971416018428e+01,     3.1558711643389208e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.1995630119420774e+01,     4.1524419969576033e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    3.7898316035957631e+00,     8.9286130763604106e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1589858730705842e+01,     2.8130242031719274e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.4554338274124852e+01,     1.5191420049945375e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0696323593384097e+01,    -5.5131102870615450e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.8071543763584330e-02,     4.6982026929604763e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.1446529337830782e-02,     1.7647544549633953e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4059133253713309e+01,     6.0541781503019756e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    7.4919162599218989e+00,    -8.1517788764625875e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.0253312473455821e+01,    -6.5420616736303884e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1468505883421596e+00,     1.5137114082168591e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.0430268034388583e+01,     2.7553749558164025e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    7.5764354167170280e+00,     1.5444538512868327e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.1497505421097433e+01,     7.0630447193678071e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.8173153833096674e+00,    -4.0945030068700335e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4941336649251959e+01,     1.6086453588784971e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0384070503674906e+01,     2.7618183907976694e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.1553384628942949e+00,     7.8761462482102788e+03,    -2.5209198770297726e-04,    -2.2277120773067824e-07,     9.9775140386023605e-10, } , 
      {    6.8527026118392786e+00,     1.6133849997320071e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9622731009871330e+01,     6.9707319721251451e+03,     2.4509445367736679e-04,     3.6118134428979317e-07,    -1.7100832973776660e-09, } , 
      {    2.4448035301034242e+01,     4.0495047136856367e+04,     4.2167736504600327e-04,     8.5210367778130929e-07,    -4.0423279917232004e-09, } , 
      {   -3.3554780112166744e+00,    -8.8522242768553078e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    3.1034080810332679e+01,     4.7918659468145932e+04,    -2.2949159801548236e-04,     1.5491736366174111e-07,    -7.5160664982411371e-10, } , 
      {    2.3035827421222464e+01,     4.0076294647438612e+04,    -8.3268446578446521e-06,     3.4076516143911704e-07,    -1.5747766471168057e-09, } , 
      {    2.1380059566812701e+01,     2.4675515708485324e+04,    -3.1828010892635822e-04,    -1.5882428319233044e-07,     6.8902205172968356e-10, } , 
      {    5.6107416630911127e+00,     7.6665967823795572e+03,     1.8057610349377928e-04,     2.8795091649735135e-07,    -1.4153602124943682e-09, } , 
      {   -6.2675829057881471e-01,    -2.6276470871769430e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    1.0283327512154589e+01,     1.6204848932529527e+04,    -9.4451474075811953e-05,     2.6407122470882869e-08,    -7.9708217311218847e-11, } , 
      {    1.9192407429639871e+01,     2.2581049117315033e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    7.9276949236851175e+00,     7.8761575078044016e+03,    -2.4676422471716020e-04,    -2.2400399195900355e-07,     1.1066308603638158e-09, } , 
      {    6.5111178335538593e+00,     1.6204837672935406e+04,    -9.9779237061628989e-05,     2.7639906699208196e-08,    -1.8858767381479843e-10, } , 
      {    1.2613980135002864e+01,     7.0797028382603103e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9950242290655940e+00,     2.7548319853131034e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.1440218498101672e+01,     5.3140478448286285e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6049578881129602e+01,     3.2431016513442784e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.4389731433551459e+01,     3.2388356697487081e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {   -1.4398999310700496e+01,    -1.0883201686397351e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1166677088887294e+01,     1.4456511664327925e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.1557068220805441e+01,     2.4099044124095184e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.8765811440775629e+01,     2.5270874801178488e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    3.3603622068281148e+01,     3.7309409745643046e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.4747142350471812e+01,     4.2150102239329844e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.8456435290559750e+01,    -1.3470227157530076e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4767544465813183e+00,    -1.3176190694972875e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -8.5713140911476486e+00,     3.6834330938216735e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2016362586528444e+01,     2.3137218887013401e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    6.3227174667764894e+00,    -8.7282713500178415e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.4477591452057258e+01,     1.7205491375220336e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1019371546630126e+01,     2.3464685206241640e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9139787397508144e+01,     1.9930419306619493e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1036361662615292e+01,    -1.7279656960100849e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.6125357589632159e+01,     1.0708424663890855e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -9.5698189388093482e+00,     6.1320283485021582e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1929550644051389e+01,     1.5929216304277215e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -6.0882927235064926e+01,     1.5724370143482793e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9095134352504811e+01,     8.0284907382550655e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7886160341738186e+00,     1.7810708609007379e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.8699512916952635e+01,    -1.0393774980939052e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4108468974970796e+00,    -2.5179624231681613e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -2.7739594357266675e+01,     7.4344290426492753e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.7445164262629042e+01,    -9.4307007456690408e+03,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {    2.7630257691809128e+01,     1.5557710623322073e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7486325534222079e+01,     4.7166398956262914e+04,    -2.6041883236082187e-04,     1.1799395370843885e-07,    -5.7702524325656986e-10, } , 
      {   -7.6951281807230018e+01,     1.1445956476343357e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1997566937734675e+01,     7.7080987666253004e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8414024901856720e+01,     1.0611566181710663e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9000879962653737e+01,    -9.4329875618913661e-01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.6690250635984096e+00,     5.7208523201127093e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6060309455121327e+01,     2.8652131701037604e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.9097294243368701e+01,     2.2681566094470407e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.2347006032668244e+01,     9.7925548608072004e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.5608871713597523e+01,    -5.7915843256366557e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.6339134143172473e+01,     1.7736586798401724e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    6.7607200994913805e+00,    -1.0831932358901104e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.5983641908113499e+01,     2.3243143759660601e+04,     8.8788510910875812e-05,     3.1374164685407152e-07,    -1.4406287015537972e-09, } , 
      {    2.1347706335349738e+01,     3.9378151588690642e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.6419454936329171e+01,     2.2752407928473724e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9502377162271241e+01,     1.0578596800193174e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.3908193936679076e+01,     3.4374791453929865e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    3.0612124533395193e+01,     2.2945718364538054e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.2304978943080034e+00,    -1.1396352083127204e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1611901086353239e+01,     3.2764010374867084e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.0954553502116049e+00,     2.4963734205461253e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.3336994413252613e+01,     1.9612649398098889e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0622255223404736e+01,     1.5524720606196632e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    6.9944115392543331e+00,    -1.7588539904854279e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.8935553846590381e+01,     1.9244236327361003e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.8533321932591802e+00,    -1.0657275338126803e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.4271524766150335e+01,    -6.5605553815355961e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.3489985777024714e+01,     2.3248069162019463e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0924684446654048e+01,     1.0185575830500187e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.8036550133387472e+01,     9.3626392177860362e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    6.8553263798482131e+00,     2.4357766689129836e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.6197711656453368e+01,     4.1589373566827026e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.3936112648195891e+01,     9.2426088529610570e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0329813223636847e+01,     3.2517907048154811e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.4779320183337603e+00,     1.7254523579940964e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.9188913043440671e+01,     1.8522628737787694e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3365663639897774e+00,    -2.5202182695605825e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    2.2126908079328280e+00,     6.5290883014999044e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1624526942391586e+01,     8.2439945489820311e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.3509002094850707e+01,    -1.9547171345464346e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2068200641539232e+01,     3.4075070488764650e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    6.1072658118181042e+00,     2.1748606512103717e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.9864922590057752e+00,     2.0569543603024751e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.2410715738154607e+00,     8.5328111855228544e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.1328052088478842e+00,    -1.7772011408716546e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    3.9184263285064546e+01,    -4.0383768105270974e+02,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.2342222539377644e+01,     2.0404121042517858e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    4.6100474718701161e+00,     7.5710354187494104e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1431296780739126e+01,     2.9448990351783297e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.1795345238136079e+01,     2.8816425010977764e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.2484983105719195e+01,     8.4756090677058382e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.9628494247085424e+00,    -1.0096248751991396e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3526261288425655e+01,     1.5004618204472918e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    3.2348586731099891e+00,     6.7615832528556448e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8837405678428645e+01,     3.8299571445826747e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.4307484436262508e+00,    -1.7981560874547267e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.4108273581310311e+01,    -5.2008365115525703e+02,    -3.4264820201723061e-04,    -3.9929186902266028e-07,     1.8356548889218472e-09, } , 
      {    4.4624994839756269e+00,    -1.0471144649914022e+03,    -4.3247222646958824e-04,    -5.1080939069062928e-07,     2.4116571753112758e-09, } , 
      {    2.2192795612675877e+01,     6.0656772590550936e+03,    -5.5848519238905125e-04,    -5.8637245102836155e-07,     2.7677043427181186e-09, } , 
      {    8.6974557351819399e+00,    -1.0105378986377349e+04,    -4.8963318967276172e-04,    -6.5093576768087200e-07,     3.1308734231004614e-09, } , 
      {    8.9849428748879916e+00,     2.5053588368450270e+04,     3.9527964796053398e-04,     6.7729925692045504e-07,    -3.2113088609333443e-09, } , 
      {   -5.1483824642283071e+00,    -2.4918560179900487e+04,    -5.1859685588755546e-04,    -8.2516742965886348e-07,     3.9067256051168632e-09, } , 
      {    6.9005066701728559e+00,     1.0474472275278094e+02,    -2.1889999472770265e-04,    -2.5478830323115201e-07,     1.1513888594038480e-09, } , 
      {    4.5449548904477588e+00,    -8.2239467019723452e+03,    -3.7121274536905087e-04,    -5.0519941766103844e-07,     2.3377279370788823e-09, } , 
      {    1.2736904150011700e+01,     4.1677214170763080e+04,     7.3073445122498590e-04,     1.2150885289648299e-06,    -5.7578412023292926e-09, } , 
      {    4.3880585476405862e+00,     2.5263137834280991e+04,    -3.7388443236222521e-05,     1.6657713269242548e-07,    -7.9819724457874010e-10, } , 
      {   -1.0818640699842677e+01,    -4.1609700076488181e+04,    -7.9239305518849664e-04,    -1.2890226153340343e-06,     6.1055495744210515e-09, } , 
      {    2.2001610329895460e+01,     4.8823762941947018e+04,     5.7389218332376741e-04,     1.1025584254424956e-06,    -5.2279398488765700e-09, } , 
      {    1.9689181264858512e+01,     3.9866769561973408e+04,     4.2424331417532766e-04,     8.5153091889844652e-07,    -3.9871610429497457e-09, } , 
      {    5.8746946228838688e+00,    -6.2836197557364176e+02,    -2.4680167657402624e-06,     5.2912565156294998e-10,    -5.5894169295118419e-11, } , 
      {    6.5273752066414570e+00,     2.5857682742392026e+04,    -9.2230227473756550e-06,     2.0416056799119024e-07,    -1.0264911588764094e-09, } , 
      {    2.2464581757341019e+01,     4.9100826502052631e+04,     7.9565488163500157e-05,     5.1790221484526173e-07,    -2.4671198604302066e-09, } , 
      {    2.4851470259203079e+01,     3.1747567582753843e+04,    -1.6586942592142578e-04,     9.1543198006256124e-08,    -4.9804424646701522e-10, } , 
      {    3.2149937454069928e+01,     4.6662019917689322e+04,    -2.2939366565189823e-04,     1.5487373043044127e-07,    -7.5233387034577794e-10, } , 
      {    1.1485447911225281e+01,     1.3691605471575956e+04,    -8.9025778726410784e-05,     2.5130705011257680e-08,     2.8444018670696433e-11, } , 
      {    4.1718222051858822e+00,     1.7528991317666903e+04,    -1.6153577338872389e-04,    -4.6250546438696180e-08,     1.5984791879862507e-10, } , 
      {    2.5944799299300410e+01,     2.2689303061367900e+04,    -2.2303038912459939e-04,    -4.8583178983986610e-08,     2.2117200132217024e-10, } , 
      {    1.2708765420940063e+01,     1.3624091377301065e+04,    -2.7367174762900028e-05,     9.9064791380461925e-08,    -3.1926435342106271e-10, } , 
      {    9.4134152320875337e+00,     2.4672066507510397e+04,    -1.9196595250750586e-04,    -1.4893476283684945e-08,     5.9922970985138590e-11, } , 
      {    2.3062573207769251e+01,     2.9795147622733508e+04,    -9.6219177479189502e-05,     1.6362810803297235e-07,    -6.8243343380340506e-10, } , 
      {    1.9483704092862300e+01,     2.1533970292283269e+04,    -3.1019053208404854e-04,    -1.6071709276611828e-07,     8.5161401596338864e-10, } , 
      {    1.5634727396862440e+01,     1.6765601153543477e+04,    -4.0686582227442585e-05,     1.0214675195127525e-07,    -5.9146299468001174e-10, } , 
      {    2.1289217442259698e+01,     4.1332922938301097e+04,    -1.3752540007245801e-05,     3.4204157889874226e-07,    -1.6829288830987212e-09, } , 
      {    1.9165465649987247e+01,     2.2932609574336064e+04,    -6.8652778230311122e-04,    -5.9627234639661827e-07,     2.8081378037226539e-09, } , 
      {    1.7734726373910895e+01,     2.3908676343386971e+04,    -7.1116761346224551e-05,     1.3350382210628648e-07,    -6.9138794249349817e-10, } , 
      {    1.5921447087128714e+01,     3.1119325647830523e+04,    -1.5807364616986851e-04,     8.9781288126367854e-08,    -3.3327062066831732e-10, } , 
      {    5.6304919825184605e+00,    -2.6486044717965669e+04,     6.7550059816353993e-06,    -2.0363144233962730e-07,     9.7059698958129107e-10, } , 
      {    1.3565891991278038e+01,     2.2790634223105397e+04,    -3.1038639681121681e-04,    -1.6062982630351859e-07,     8.5306845700671708e-10, } , 
      {    6.0865164280911976e+00,    -9.6901000783000309e+03,     6.6783108813522633e-05,     4.2899223386658372e-09,     3.3495777227857859e-11, } , 
      {    2.8587682891797300e+01,     5.6247375273236568e+04,    -7.7276779737718251e-05,     4.0537211132292736e-07,    -1.9372185069774839e-09, } , 
      {    2.1564144856789476e+01,     3.0632999043812706e+04,    -5.3155115016885457e-04,    -3.4647762408089458e-07,     1.6762384542994093e-09, } , 
      {    2.1819699404795369e+01,     3.1818615278694338e+04,    -3.4880822040872430e-04,    -1.2742357980601935e-07,     5.8982432443786136e-10, } , 
      {    1.5379169183665116e+01,     1.5579984918661847e+04,    -2.2342951198757279e-04,    -1.1690729232359997e-07,     4.9495113518153629e-10, } , 
      {    1.2768250995058324e+01,     1.5995263826739167e+04,     3.3298678649871160e-04,     5.3831839769593786e-07,    -2.6024265106910664e-09, } , 
      {    2.6349903671637687e+01,     3.0909716161674645e+04,     2.6946254676823918e-04,     6.0164893012012296e-07,    -2.8567161345698300e-09, } , 
      {    2.4037475828331214e+01,     2.1952722781701032e+04,     1.1981367761979946e-04,     3.5062142357607391e-07,    -1.6159373286430052e-09, } , 
      {    2.6306583567580301e+01,     2.1608431549239045e+04,    -6.2467331361243221e-04,    -5.2242552649001377e-07,     2.4589749905875666e-09, } , 
      {    2.0140371761207717e+01,     1.5090184733303591e+04,    -4.7049492720412230e-04,    -4.0927903085351672e-07,     1.8746339088830535e-09, } , 
      {    1.3042374463907706e+01,     1.5786060803152126e+04,    -5.2968551440189283e-04,    -4.8374224287428395e-07,     2.2782364502699313e-09, } , 
      {    1.0002805901886838e+01,     6.6532750044852401e+03,    -2.7749559433533144e-04,    -2.6101466837490554e-07,     1.2797578258880310e-09, } , 
      {    1.6208316017057140e+01,     6.6229915386882385e+03,    -3.7307838113601256e-04,    -3.6793479886764907e-07,     1.7357299411083605e-09, } , 
      {    8.8000580898688892e+00,    -8.3625084921877315e+03,    -1.2661534691824162e-04,    -2.1229855347955870e-07,     9.0215099408224665e-10, } , 
      {    1.9893611368942704e+01,     2.2547267689812070e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {    9.7276730694732674e+00,     7.4236123312895543e+03,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {    8.1276304343538488e+00,     5.9574589549618686e+03,    -2.1317310887891217e-04,    -1.8769697411986398e-07,     9.8648918204806139e-10, } , 
      {    2.8586846283104119e+00,    -8.4334961678030668e+03,     6.1455345827705611e-05,     5.5227065669911707e-09,    -7.5383679275721722e-11, } , 
      {    1.9426563320258161e+00,    -9.0955551701889945e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.9987228891191942e+00,     9.1665428458043298e+03,    -2.8301922204831677e-04,    -2.5969461768398047e-07,     1.1723328104277799e-09, } , 
      {    1.0751609567573617e+00,    -3.7260638640469551e+01,     1.5467544163486746e-04,     1.8142697574481057e-07,    -8.5884743608554286e-10, } , 
      {    1.1915483079841197e+01,     1.9219628428405904e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.0142384631263290e+00,    -7.7553776367674936e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.4017958018689871e+00,     7.7873766534164633e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.6572985256960202e+01,     2.4674554935834705e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.7263240929265300e+00,     1.1190377864353643e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5393933052233333e+00,     9.9559140521669678e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.6486669642866730e+01,     2.2772816663967271e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    7.2153081188188324e+00,     8.8768169164423616e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7767575297990632e+00,     4.9334893083130319e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2547608930442786e+00,     2.8766924391717594e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3400408253660963e+01,     3.8692592415519735e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.1034391179471623e+01,     3.0363900990794609e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    4.7009076547663433e+00,     6.4345798816725855e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1167837971362198e+01,     3.0235032291169275e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -9.0135908717494253e+00,    -2.4970074765850892e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.4750325755371096e+01,     2.0005670367791878e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9225204680993091e+01,     1.5115951892988005e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -4.4607949615484142e+00,     1.1306280675907790e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4463582634892088e+01,     1.3806377101471848e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.8516671287042401e+00,    -1.7870227186032182e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    6.1711803249220791e+00,    -1.9782227596643925e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3299288034478582e+00,     1.6344543973085572e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7372564708741034e+01,     7.8177872867034430e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1846606139626799e+00,     1.6323321605978525e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.5040906891609902e+00,    -8.9409995014461838e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1074359848502091e+01,     3.2298352916949840e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    1.2174552306452703e+01,     1.6546049160935163e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.3465437132227894e+01,     2.2740817647318305e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8419940269537143e+01,     3.1400977289048784e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.8953605157902800e+00,    -7.8341210706979109e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5441800174204886e+01,     2.4171166680074311e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1395018422253797e+01,     8.6585416974812665e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6677584045208267e+01,     1.6991444092921978e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.8146633187474439e+00,    -7.4227110587681545e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.3847969520833088e+00,    -3.9076100034295973e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.7440926995384158e+00,    -1.6768716537965342e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    7.7715893822454092e+00,     1.5889374594051602e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.9861562678231772e+00,     1.4527984691045156e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.1637763133338614e+00,    -1.6978266003796063e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    1.3158134759769785e+01,     3.2216136647958694e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    7.3706356163917057e+00,    -1.5846429391678662e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5384044649549644e+01,     3.3412981476216140e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.2761400946478499e+01,     2.5214801933928211e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    7.8617571899806027e+00,     2.0885012311705617e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3542939220228263e+01,     7.4519793131275728e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7088772892896046e+01,     2.8334361792517004e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.1727993130980341e+00,     6.5677215964259631e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2397868616011186e+01,     1.1676978943066752e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.6369874321811153e+00,     8.1735869027008775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0332928976205322e+01,    -3.0284947035848438e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4173369158332909e+01,     2.2921110465582948e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.8479635142321778e+00,     5.4324295208561616e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.4440546196108979e+01,     2.5424351399758933e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    8.3647866734842875e+00,     2.9296615363829505e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2949322098646824e+01,     7.5552687236615302e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.0172633448080358e+01,     2.1890962985894665e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.5695524562635388e+00,     1.6888262223258826e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.6257371289761146e+01,    -1.2259622477813837e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.8764330303105044e+01,     2.1893222955244692e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2962537300901371e+01,     6.5630470391740505e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0380655262551011e+01,     2.1513319896751160e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1682513045085788e+01,     2.5851928443923505e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.6118134171614535e+00,     2.1355585542410729e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9904932088648275e+00,     1.7161178656854790e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    6.2237835275243469e+00,    -9.2598484801291524e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.8667934350231551e+01,     2.2035209566069483e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.5700511258685024e+01,     2.5929850567901784e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.0662070665925594e+00,     2.8237233419605682e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0716345204448228e+00,    -3.4727543774242181e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5873092659039278e+01,     2.0398691337484866e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9448275925739111e+01,     1.5640970067499587e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    4.3844887208648835e+01,    -1.4651982334741297e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3583592044860767e+01,     1.2069999912759740e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.1536391962678081e+01,    -1.5050131353720417e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2788911615114982e+01,     2.9842011321476286e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.3206444748960632e+01,     2.3760112026393992e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    3.7381909966675950e+00,    -8.2304756474094938e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.6367802509721272e+01,     2.8727382762209993e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0324060172806826e+01,     2.5195623740006100e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -4.3125681747878231e-02,     6.2830195524848227e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.1226097406195763e+01,     6.5857609102103506e+03,    -2.1583699037182070e-04,    -1.8708058200570130e-07,     9.3204945379627173e-10, } , 
      {    1.3581651978447663e+01,     1.4914452334935477e+04,    -6.3524239730472436e-05,     6.3330532424185123e-08,    -2.5428962387876272e-10, } , 
      {    5.5402729075895722e+00,     7.7003894694766441e+03,     1.5497663213425675e-04,     2.4979472231572374e-07,    -1.1318993494232446e-09, } , 
      {   -2.5619821245892776e+01,    -2.3011998397341813e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3124288905040222e+00,     8.9569933799736082e+03,     1.4964886914843974e-04,     2.5102750654404910e-07,    -1.2407788059268243e-09, } , 
      {    7.9982533891102134e+00,     7.8423648207073147e+03,    -2.2116475335763771e-04,    -1.8584779777737599e-07,     8.2316999729269210e-10, } , 
      {    1.0353807961362115e+01,     1.6171056245432441e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {    8.2969337933969118e+00,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    1.3624777660195543e+01,     1.4286150379686995e+04,    -6.0860358237563932e-05,     6.2714140310022456e-08,    -1.9984989562697296e-10, } , 
      {   -8.6251363495756461e-02,     1.2566039104969645e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    1.1269223087943642e+01,     5.9574589549618686e+03,    -2.1317310887891217e-04,    -1.8769697411986398e-07,     9.8648918204806139e-10, } , 
      {    1.5937206550699564e+01,     2.3243143759660601e+04,     8.8788510910875812e-05,     3.1374164685407152e-07,    -1.4406287015537972e-09, } , 
      {    7.8958274798414720e+00,     1.6029080894201770e+04,     3.0728938277560503e-04,     5.0020583674561016e-07,    -2.3182384270982792e-09, } , 
      {    8.8705428339438619e+00,    -1.7429305145147757e+03,    -3.6814974101316892e-04,    -4.3749169643558775e-07,     2.1183885314713062e-09, } , 
      {   -2.2281722528813336e+01,     3.2863247644877447e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6679834627559220e+00,     1.7285684804698736e+04,     3.0196161978978796e-04,     5.0143862097393552e-07,    -2.4271178836018586e-09, } , 
      {   -2.3264249931354815e+01,     8.3263902248853919e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1116952198971685e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5833985893374507e+00,     7.0720875142281620e+03,     1.5764051362716528e-04,     2.4917833020156111e-07,    -1.0774596211714549e-09, } , 
      {    7.9551277073623350e+00,     8.4706667759557968e+03,    -2.2382863485054624e-04,    -1.8523140566321331e-07,     7.6873026904090223e-10, } , 
      {    2.1623031049305755e+01,     2.2128515200394308e+04,    -2.8202511159520165e-04,    -1.2313365746735350e-07,     6.2332010166571914e-10, } , 
      {    1.0398780774743523e+01,     1.5542754290183959e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0437708451329151e+00,     7.2140628654588327e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2709362533614016e+01,     2.4499747670157569e+04,     8.3460747925058777e-05,     3.1497443108239688e-07,    -1.5495081580573769e-09, } , 
      {    1.9267476477053854e+01,     1.3799823775669183e+04,    -4.3433786223654987e-04,    -3.7354477189723997e-07,     1.8096591793407536e-09, } , 
      {    8.7842914704481068e+00,    -4.8632660401781118e+02,    -3.7347750399898598e-04,    -4.3625891220726239e-07,     2.0095090749677269e-09, } , 
      {   -3.8338359893842949e+01,     8.3283391087670516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6191069687044674e+01,     8.3290437406832007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8338621497967855e+00,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2693032087561438e+00,     9.5852953352220902e+03,     1.4698498765553121e-04,     2.5164389865821173e-07,    -1.2952185341786140e-09, } , 
      {   -3.5349467081839535e+01,    -3.5231595807488247e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0325841488918483e+01,    -1.9524799803454973e+03,     6.4518350183587553e-05,     7.3230427792441848e-08,    -2.9472308488329798e-10, } , 
      {    2.3978585621557656e+01,     3.0457206625119434e+04,    -1.2971236095385340e-04,     1.2727745696253293e-07,    -5.6301897600931531e-10, } , 
      {   -9.7617405204576961e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7578578718115498e+01,     7.2117616656190985e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5980332232447443e+01,     2.2614841804412121e+04,     9.1452392403784316e-05,     3.1312525473990889e-07,    -1.3861889733020075e-09, } , 
      {   -1.5223036327914894e+01,     1.5540453090344225e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.9158499319998612e+01,     1.5545055490023693e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3600257064642083e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0979974718329339e+00,    -6.9587606984853278e+02,     5.9190587197770524e-05,     7.4463212020767181e-08,    -4.0360254138687761e-10, } , 
      {    3.6802936233708920e+01,     7.2163640652985669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.2314954066426580e-01,    -2.2365705603351671e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0812170851277632e+01,     8.3117707439163460e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0652488365648813e+01,     1.6728370525065588e+04,     1.1655480853674925e-04,     2.8300096881322299e-07,    -1.3951434819921005e-09, } , 
      {   -2.1824386755573189e+00,     3.3757047137445625e+01,    -3.0829301981755390e-05,    -3.6967043184602116e-08,     1.7385418604587960e-10, } , 
      {   -2.4023120420638513e+00,     1.5999508324485532e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8292761122951465e+01,     3.1571835184385727e+04,     2.4110126155222405e-04,     5.6415276128395794e-07,    -2.6269677792288316e-09, } , 
      {    4.2223540532269510e+00,     2.6298323161836379e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7222390695258945e+00,    -3.3781427502964334e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.9413792211450103e+00,     7.0987675615335547e+01,    -1.8807069274594723e-04,    -2.1782126004654988e-07,     9.7753467335796839e-10, } , 
      {    1.3667903341943420e+01,     1.3657848424438513e+04,    -5.8196476744655414e-05,     6.2097748195859803e-08,    -1.4541016737518314e-10, } , 
      {    8.4563043905538855e+00,    -1.6920680808781185e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.4816394177886394e+01,     1.5542028288429719e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0849840589882042e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3676311977898692e+00,     8.3294027701920022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8495061826727248e-01,     8.3279800792582500e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8302895044079408e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1312348769691519e+01,     5.3291569997133856e+03,    -2.1050922738600366e-04,    -1.8831336623402663e-07,     1.0409289102998513e-09, } , 
      {    1.2970221281043222e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0251382052093371e+01,     2.4357772318926895e+04,     4.5960213341695325e-04,     7.5061695117549658e-07,    -3.5045775047733139e-09, } , 
      {   -1.2212676811942453e+01,    -1.5419111039189374e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8395187032220207e+01,     2.3385119110891275e+04,    -2.8735287458101871e-04,    -1.2190087323902819e-07,     5.1444064516213961e-10, } , 
      {    2.1666156731053633e+01,     2.1500213245145827e+04,    -2.7936123010229317e-04,    -1.2375004958151618e-07,     6.7775982991750901e-10, } , 
      {    8.9136685156917412e+00,    -2.3712324697632580e+03,    -3.6548585952026044e-04,    -4.3810808854975037e-07,     2.1728282597230963e-09, } , 
      {    8.6246903929052543e+00,     8.3286914247251261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2415925033028747e+00,     7.1430751898434974e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    7.0235380350078218e+00,     2.5614376229423862e+04,     4.5427437043113619e-04,     7.5184973540382194e-07,    -3.6134569612768932e-09, } , 
      {   -1.5837317554495009e+01,    -1.8048943355373922e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2752488215361893e+01,     2.3871445714909085e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.6809311558916571e+00,    -6.5589555997275966e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.5995731351104361e+00,     1.4197535123067109e+02,    -3.7614138549189446e-04,    -4.3564252009309976e-07,     1.9550693467159368e-09, } , 
      {   -2.0908867472806183e+01,     1.6655081649610518e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.1649619478297146e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.5149882616919639e+00,    -1.0071621939239902e+04,    -5.2046249165451719e-04,    -6.8790281086547417e-07,     3.3047276091463408e-09, } , 
      {    2.7779338649578929e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9926629967106361e+01,     8.3290200572015747e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3472694029971066e+01,     1.6659684049289986e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6827391195277727e+01,     7.2137105495007590e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2336861829384148e+01,     7.2144151814169072e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4202498324665406e+01,    -4.9310321559205477e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4211929488556660e+01,     1.5542401974225884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4430557776330957e+01,     1.5543106606142033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1340315242135150e+01,     2.3452693225491323e+04,    -3.4387958028588066e-04,    -1.9698047737395802e-07,     9.7248291480080704e-10, } , 
      {    6.3542774613710948e+00,     1.7495234270529458e+04,    -1.3070647140696849e-04,    -9.2835032540940612e-09,    -1.4006267247254537e-11, } , 
      {    7.9389531615893496e+00,     1.5400778938953288e+04,     3.0995326426851350e-04,     4.9958944463144748e-07,    -2.2637986988464892e-09, } , 
      {    1.3695869814387050e+01,     3.1781384650216449e+04,    -1.9156682964453244e-04,     5.3430637055928398e-08,    -2.1385616287422749e-10, } , 
      {    1.0310682279614236e+01,     1.6799358200680923e+04,    -7.1515884209197975e-05,     6.5179708766673123e-08,    -4.1760880863413212e-10, } , 
      {    9.5398034075805906e+00,     6.3762114443796290e+03,     2.1683110082493579e-04,     3.2364154222232824e-07,    -1.4810621625583325e-09, } , 
      {    1.6039632459968306e+01,     1.5056427686166147e+04,    -4.3966562522236693e-04,    -3.7231198766891461e-07,     1.7007797228371739e-09, } , 
      {    3.5858246488931105e+00,    -8.2577037491097908e+03,    -3.4038344338729550e-04,    -4.6823237447643627e-07,     2.1638737510330028e-09, } , 
      {   -1.9696122434593839e+01,     1.5732484147591094e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.0945392324951921e+00,     8.1709570703846939e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0365469444722457e+00,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9310602158801732e+01,     1.3171521820420701e+04,    -4.3167398074364139e-04,    -3.7416116401140260e-07,     1.8640989075925435e-09, } , 
      {    2.4021711303305533e+01,     2.9828904669870954e+04,    -1.2704847946094487e-04,     1.2666106484837025e-07,    -5.0857924775752544e-10, } , 
      {    5.6544907435289593e+00,     2.4567321784757616e+04,     2.6934042220196770e-05,     2.3989482694746702e-07,    -1.0914658884187093e-09, } , 
      {   -1.2937704524363469e-01,     1.8849058657454468e+03,    -7.9916444787255359e-06,     1.8491763424879922e-09,    -1.6331918475536940e-10, } , 
      {    3.7557252368249143e+00,     8.4864257790655574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0750741604472108e+01,     3.1713810535616401e+04,    -1.3504012393967041e-04,     1.2851024119085823e-07,    -6.7189843251289484e-10, } , 
      {    1.5064917105865916e+01,     3.2828439094882691e+04,     2.3577349856640704e-04,     5.6538554551228330e-07,    -2.7358472357324109e-09, } , 
      {   -3.0564865329671711e+01,     8.3114184279582696e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6334140193809557e+01,     3.8785898049844560e+04,     2.2600389687494852e-05,     3.7768857139241929e-07,    -1.7493580536843496e-09, } , 
      {    1.1276751132458713e+00,     8.3260615924089434e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7251186169780652e+00,     8.3313212570413107e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8666039731953372e+00,     7.2309835462676137e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2008659186350521e+01,     1.6833150887778011e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6338312864255002e+01,     1.5613741965799292e+04,    -2.5425881396932824e-04,    -1.5387433550820208e-07,     6.6880532122741591e-10, } , 
      {    9.7827176234367119e+00,     7.0563285111184005e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2619343314744018e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7098320336229964e+00,     2.5823925695254584e+04,     2.1606279234379749e-05,     2.4112761117579238e-07,    -1.2003453449222890e-09, } , 
      {    2.0306594847030928e+02,    -5.0795853803393243e-01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9222566833117146e+01,     1.6864631115441240e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.5158965793693454e+01,     1.6869233515120708e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.0948730163011515e+01,     8.2949099972221611e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.2376210185077152e+00,    -8.3624728522280911e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.0922557277564624e+02,    -1.7309684784268029e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2819146567448884e+01,     1.2619238322379260e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.5665549970192629e+01,     6.2133149972934916e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4314118417102994e+00,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3282971574160966e+01,     1.4357138055302328e+04,    -2.4893105098351117e-04,    -1.5510711973652738e-07,     7.7768477773099544e-10, } , 
      {    1.3128590394796857e+01,     8.3155735135256709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.3587660843427471e+00,     8.2888765228272132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5212837518063814e+01,     8.3685063266230391e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.3165308095997048e+00,     6.6633075414254135e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8575073264155375e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9842634793674048e+00,     7.2114330331426490e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2239644248560900e+01,     7.2166926977750172e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8612055327404420e+00,     6.6916934748356716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5700442169893893e+01,    -8.8601554394617779e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2138529717268217e+01,     1.5385019935843527e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5533728390001467e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2867483978314727e+01,     2.3869144515069351e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.6249983980897980e+01,     8.3512333298561825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4287368981962070e+00,    -8.8150180287429375e+03,    -5.2579025464033415e-04,    -6.8667002663714881e-07,     3.1958481526427615e-09, } , 
      {    4.1514029166841254e+01,     2.3873746914748819e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0776942356459069e+01,    -1.3117911199455193e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6491000198553367e+00,     7.7779361006917070e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4105685182591362e+01,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5729651094468387e+01,    -1.1944470569373675e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.2920254904289912e+01,    -1.7272996766856068e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5278709808451268e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5982806296674582e+01,     1.6657030533492180e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.8546608341086170e+01,     1.6657735165408325e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1623924437851082e+01,     7.1610937682476415e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6913957264787332e+00,     7.1742479635609207e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8438312713968084e+01,     2.2756817155642791e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0668431106963329e+01,     1.4991998966150539e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5697025515343825e+01,     9.6640244767877448e-01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.0928921210600919e+01,     7.2538777673567447e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6616920801539166e+01,     8.3131493964366964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.2481485598295947e+01,     8.3442334530135558e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6612697803381824e+00,     7.2133515199919557e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7124568236679272e+00,     1.6657382849450252e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.6265242710853283e+00,     6.4437855589796791e+03,     1.6030439512007379e-04,     2.4856193808739843e-07,    -1.0230198929196651e-09, } , 
      {    1.5845762089524664e+01,     7.2147742109257097e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8693867436506903e+01,     2.3942433390524420e+04,    -1.0194606332797995e-04,     9.6536778921684358e-08,    -5.1753375644761854e-10, } , 
      {    5.9233059129747367e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2216690872809094e+01,     1.5020384899560799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1805202925508835e+00,     7.8063220341019651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8496945038337635e+01,     8.2490616209293021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.5977655621300116e+01,    -2.4272873609482303e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4068783411206063e+00,     7.3717972197992640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5444900158347069e+01,     8.1578391591852396e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6352316774123462e+01,     8.4083212285209502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4868542858093541e-01,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8860379310509741e+00,    -1.1856162348816290e+03,    -1.8274292976013022e-04,    -2.1905404427487519e-07,     1.0864141298615481e-09, } , 
      {    1.2156048284537622e+01,     8.3640284956606611e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1959603493224025e+00,     8.8794467487585462e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.3065587576443036e+00,     1.5540124457867776e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3501627208483655e+00,     2.1329907977851814e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6248577810080445e+00,     1.7913986759947216e+04,     2.9929773829687949e-04,     5.0205501308809820e-07,    -2.4815576118536486e-09, } , 
      {    1.4629617644640312e+01,     1.5545384122500143e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3814347076256430e+01,     7.2670319626700239e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8335886804699342e+01,     3.0943533229137247e+04,     2.4376514304513258e-04,     5.6353636916979526e-07,    -2.5725280509770416e-09, } , 
      {    1.0368967170666362e+01,    -2.5807819355939796e+03,     6.7182231676496057e-05,     7.2614035678279181e-08,    -2.4028335663150822e-10, } , 
      {    9.2252196673182681e+01,     7.1985208371704030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7610974486148230e+00,    -1.1146285592662935e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7940488580069655e+01,     8.3654511865944132e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.6745877112012067e+01,     7.2296048937472624e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9727907030055590e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5971470755547754e+00,     1.5471766614568623e+04,     1.2188257152256629e-04,     2.8176818458489763e-07,    -1.2862640254885208e-09, } , 
      {   -4.1329726166169102e+01,     1.3498673398216670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6704629471057810e+00,     8.8510608153482863e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9072151342157131e+00,     3.1546870868086353e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.1411231535808124e+00,    -1.3241780250970151e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.8735326867110045e+00,     8.2757223275139349e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9363374789082279e+01,     1.6799369460275044e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2765741494326615e+01,     7.1344330616630086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.2493976812916898e+01,     9.5171832084254220e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9200949166991343e+01,     1.4916747904978150e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6669824441903103e-01,     1.5502939388286046e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.0163641713531803e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.3656454532080062e+01,     5.5356930186341742e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2520271198779618e+00,     8.3816605219363173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3716529121127929e+01,     8.5508601288782265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3970836580817114e+01,     1.5489785192972768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6911921904801954e+01,     5.4711323509440572e+03,    -5.8665061287789815e-04,    -6.2395588632712639e-07,     2.9959982570157880e-09, } , 
      {    1.6023457914195323e+01,     2.1986539849163637e+04,     9.4116273896692834e-05,     3.1250886262574621e-07,    -1.3317492450502176e-09, } , 
      {    1.1774845305627721e+01,     1.5700488644524392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3279925116772695e+01,     1.5582569192081872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.0961627171319632e+00,     1.2566151700910864e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6639297533155272e+01,    -1.9488838816592988e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5820740575883605e+01,     1.5542425657707512e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1885279789027649e+01,     1.5543082922660407e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.6268306787695446e-02,     8.3446909330496110e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.0267515492376305e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5179910742378389e+01,     1.4912145505298682e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3167841831254496e+01,     1.6640462168641468e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3465215427617807e+01,     7.2137342329823841e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.8623784454187549e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4240857914279491e+01,     7.2143914979352812e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9664410120163847e+01,     2.9342578065853137e+04,    -5.0052598345993093e-04,    -3.0959784735889214e-07,     1.5009298272102012e-09, } , 
      {    2.0648315695203362e+01,     3.9900526609110857e+04,     3.9341401219357229e-04,     8.1456387571384436e-07,    -3.8133068569038663e-09, } , 
      {    1.3008042937900711e+01,     2.5057061949790717e+04,     2.6886755917809749e-04,     5.3341208324310941e-07,    -2.5814825596671350e-09, } , 
      {    3.2003433327201471e+01,     7.2936926692546567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0793867286219985e+01,     3.1085508580367918e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.4263108386405463e+00,     6.5147732345950153e+03,    -2.7766297625873455e-05,     3.0740678040848568e-08,    -4.5485219561696674e-11, } , 
      {   -4.0656783423869726e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5227112286963216e+00,     8.3110593984494699e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2230409228159429e+01,     7.1365106044467084e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0927417001909067e+01,     6.0284466305772021e+03,    -4.0124380162485939e-04,    -4.0551823416641386e-07,     1.9640238554060298e-09, } , 
      {    1.2595144697674629e+01,     7.6328153548765931e+03,     2.1150333783911878e-04,     3.2487432645065360e-07,    -1.5899416190619122e-09, } , 
      {    4.0849742526149511e+00,     7.9099389353073657e+03,    -2.7769145906249971e-04,    -2.6092740191230583e-07,     1.2812122669313596e-09, } , 
      {    1.7311589669458138e-01,     8.3624484718625718e+03,     1.2148344865959285e-04,     2.1344407124528431e-07,    -1.0124848916291548e-09, } , 
      {   -4.5379932478092186e+00,    -8.2949343775876805e+03,    -1.8314205262310364e-04,    -2.8737815761448853e-07,     1.3601932637209141e-09, } , 
      {    9.4616862226762862e+01,     1.5527212261895529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.6963743192937258e+01,    -7.9587275331634828e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5181864986668764e+01,     8.3124820893832220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1122154580592394e+00,     9.0279810555889435e+03,    -3.8421823597507498e-05,     3.3206246497499222e-08,    -2.6324413256885588e-10, } , 
      {    8.5713023552940171e-01,     9.1665428458043298e+03,    -2.8301922204831677e-04,    -2.5969461768398047e-07,     1.1723328104277799e-09, } , 
      {   -6.4401196265558312e+01,     1.5558296318472387e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7383130679804886e+00,     6.5864666258801662e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6187128842971141e+01,     1.5595723387395150e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6051424386638949e+01,     4.0110076074941571e+04,    -3.9254079003184183e-05,     3.0384175148581479e-07,    -1.4001952405492619e-09, } , 
      {    3.2989361712770586e+00,     1.6238630360032492e+04,    -1.2537870842115148e-04,    -1.0516287482419388e-08,     9.4873189256325051e-11, } , 
      {    3.2789763499509625e+00,     8.3284677676690917e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.0201629218120690e+01,     1.4914094389180342e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0856939994947144e+01,     8.3289150817811606e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3699648324173666e+01,     1.8684292215411675e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0415159449373457e+01,     1.5463124486388133e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0381664348604620e+01,     8.2588016467051548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2019964692415748e+01,     3.7671269490578263e+04,    -3.4821323281858260e-04,    -5.9186732929005705e-08,     3.1459074953516671e-10, } , 
      {    1.5996506778220427e+01,     1.5684729641414629e+04,    -4.4232950671527541e-04,    -3.7169559555475198e-07,     1.6463399945853842e-09, } , 
      {    1.6082758141716184e+01,     1.4428125730917665e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3652014355431341e+01,     1.4948578398967882e-01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.8616141766239682e+00,     8.0182093328307928e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1597835875327243e+01,     8.4706780355499195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6888293533777819e+01,    -1.5925960759164900e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3172411776849033e+01,     7.2916151264709570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6845975836811398e+01,     6.5880564802530243e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5457270842368501e+01,     9.7272932056410064e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2534920872232448e+01,     6.5854029644552147e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8780118800002658e+01,     2.2685829480027453e+04,    -9.6618300342162913e-05,     9.5303994693359024e-08,    -4.0865429994403896e-10, } , 
      {    1.4395656532229189e+01,     2.4709297135988290e+04,    -3.4920734327169772e-04,    -1.9574769314563272e-07,     8.6360345829722741e-10, } , 
      {   -4.7568552613303767e+00,    -8.3126919164006413e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.0322251419560683e+01,     1.0958809679741535e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1757137845090682e+01,     2.2541905131056410e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6751211104481090e+01,     3.3037988560713413e+04,    -1.9689459263034945e-04,     5.4663421284253732e-08,    -3.2273561937780709e-10, } , 
      {   -1.5707767054369187e+01,    -8.6364983834280338e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9794898168266926e+01,     6.6653850842091142e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0800821319490019e+00,     6.5857552804132893e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5003742989112789e+01,    -5.9000540864171001e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.4871486631236380e+01,    -2.0571522720911162e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7535397987998095e+01,     6.5834540805735560e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6095753891280857e+00,     8.2511391637130018e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5258880134899467e+01,     3.2200137139634211e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.8286942630767902e+00,    -1.0281171405070623e+04,    -8.7794400457760695e-05,    -1.7718068663744457e-07,     8.9161599279173642e-10, } , 
      {    8.5798418186762930e+00,     1.6833175268143528e+04,    -9.7213287932304592e-05,     2.7067147816345391e-08,    -1.3342072504134434e-10, } , 
      {    7.4893556143602087e+00,     8.4062436857372504e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4371608437805833e+01,     1.5622384093979783e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6433633272107109e+00,     7.2206218210585603e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4230828770941759e+00,     2.5132303401821728e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7239949058823413e+01,     7.2313358622256892e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2077954257178870e+01,     1.4915158050605292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4154735066642889e+00,     7.7003838396795827e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0614200302346624e+01,    -1.1489987755645416e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2261775270082653e+00,     1.0213597290470572e+04,     1.4432110616262271e-04,     2.5226029077237441e-07,    -1.3496582624304039e-09, } , 
      {   -3.8133131558328785e+01,     7.7000315237215082e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7330266485597168e-01,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7228529392193970e+00,     1.6658094194917128e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.8400769137536663e+00,     1.6656671503983376e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1536935906552777e+01,     6.1163549870013203e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2824277625849721e+01,    -1.6209335341904819e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0688419364365571e+02,     8.3269604562466993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1783676863716170e+01,     2.3871093398951012e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.6713220647632937e+01,     2.3871798030867158e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3106296176724008e+01,     4.0042501960341528e+04,     1.7272626701677830e-05,     3.7892135562074465e-07,    -1.8582375101879293e-09, } , 
      {    2.1709282412801514e+01,     2.0871911289897344e+04,    -2.7669734860938464e-04,    -1.2436644169567883e-07,     7.3219955816929878e-10, } , 
      {    1.3711029023691299e+01,     1.3029546469190031e+04,    -5.5532595251746910e-05,     6.1481356081697136e-08,    -9.0970439123393348e-11, } , 
      {    8.0100453157808591e+00,     3.2896013209482742e+04,     1.7924679286154501e-04,     4.9030594137735344e-07,    -2.2778049660937439e-09, } , 
      {    8.9319904212066419e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1670131819688638e+00,    -1.7632026275657509e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3889039398157738e+01,     1.4445046411726446e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3664860497812708e+00,     7.7364322560819937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5004511629532331e+00,     7.2478442929617968e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.5316861506054741e+01,     7.8423704505043761e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0599552535478558e+01,     6.0334743949479162e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.6871914924768490e+01,     7.2139648138849934e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0609629985518762e+02,     7.2141609170326719e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4564615595406051e+01,     1.5465202029171833e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3352177859324179e+01,     8.3200549263416979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.5389431999780712e+00,    -1.2903965975940507e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2640735866430409e+01,     1.6171061875229501e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2165497911375397e+01,     1.4994076508934240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1149950444220366e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6377265875557434e+01,     3.8157596094596076e+04,     2.5264271180403363e-05,     3.7707217927825667e-07,    -1.6949183254325599e-09, } , 
      {    7.0548717900850555e+00,    -6.7574114600050507e+01,     5.6526705704862013e-05,     7.5079604134929848e-08,    -4.5804226963866738e-10, } , 
      {    1.2606936624345272e+01,     3.2686463743652021e+04,     6.1191488405830147e-04,     1.0010280656053830e-06,    -4.6909165824483477e-09, } , 
      {    9.1368575718103330e+01,     1.5275584790397261e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4064672286612129e+01,     7.7800136434754077e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.7008209765505340e+00,     6.0104167925789370e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0014996435849403e+01,    -1.3971962192534707e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.5287553549852998e+01,     6.9596314257829290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7880541283281062e+00,     7.1927329574809810e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1434188230563851e+01,     7.2353927734366844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2748306610745466e+01,    -3.1480227663230835e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.0989580133621867e+01,     8.0310922092562214e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3898156956939053e+01,     8.2353914201973639e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0568382912713208e+00,     1.4339108217304063e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4606074571460560e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.5538168150677521e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5453525798141428e+01,     8.2716035260603567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4218941113977008e+00,     1.4128404765752441e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0506125797087265e+00,     7.7010951851464597e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5546144683021250e+00,     7.2271807766582879e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4541599862105684e+01,     6.2795526908746854e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4450087643596561e+01,     2.3243138129863542e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4732377653508571e+01,     1.4198661082479293e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5463214599038094e+01,     1.5620306551196083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8352061350472329e+01,     2.4013421066139756e+04,    -2.9001675607392719e-04,    -1.2128448112486551e-07,     4.6000091691034980e-10, } , 
      {    1.0841165638413312e+01,     7.2850505410741662e+03,    -4.0657156461067646e-04,    -4.0428544993808850e-07,     1.8551443989024503e-09, } , 
      {    5.1984668215549963e+00,     7.7713771450919794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1065386605874895e+01,     3.4152617119979710e+04,     1.7391902987572800e-04,     4.9153872560567880e-07,    -2.3866844225973233e-09, } , 
      {    4.0418485708670726e+00,     8.5382408905558477e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.7123889803846897e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6453556728861322e+01,     6.9994463276808419e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0689604971361932e+00,     5.7997133410273145e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1176606245056742e+01,    -2.1422382696247801e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.7731495151552750e+00,     8.9562876643037926e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6867014513667801e+01,     2.5193322540166366e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    1.0134346931419024e+01,     3.1415379252277162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9396489648976076e-01,     1.0309281255487740e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3221303292578177e+01,     7.6980826398398494e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.9563528097363392e+00,     8.5044594630528827e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.7514526437302308e+01,     2.5197924939845838e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    4.9381553262856848e+00,     6.8127668225673460e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2634429205202036e-01,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3608551709190086e+01,     2.8168728117789897e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.9810559337540234e+01,     8.4292412282353416e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0606417040599911e+02,     2.0355170170754775e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7308855547911950e+01,     2.1013886641128011e+04,    -6.5283873410127910e-04,    -5.6000896178877861e-07,     2.6872689048852357e-09, } , 
      {    9.3790926072597234e+00,     3.3943067654148988e+04,     6.0658712107248451e-04,     1.0022608498337084e-06,    -4.7997960389519279e-09, } , 
      {   -1.7279889125296254e+01,     7.7690759452522452e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.7101623049802583e+01,     7.7736783449317118e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1450940373155149e+01,     1.2395292683746743e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.5446202944097420e+00,     1.2036460728798950e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3414819806382710e+00,     1.8849227551366296e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5175596069883371e+01,     8.9438810985712153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6321541028383155e+00,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6190748235192547e+01,     8.8773692059748446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0928211834892068e+01,     6.7051999861070262e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.1816518534043787e+00,     2.2754515955803057e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.4567706265710621e+01,    -8.3441105357643155e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.0166674371337734e+01,     8.9546978099309345e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.7199873592196553e+01,     2.2759118355482529e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0673208405387806e+01,     6.7446625720468637e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2055595678364174e+01,    -9.5601204367496393e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0555792388904131e+00,    -8.6359420039809891e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9599614913530129e+01,     8.0742599850492225e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3738995496134930e+01,     3.1153082694967969e+04,    -1.8890294815162391e-04,     5.2814244941765738e-08,    -1.5941643462243769e-10, } , 
      {    8.4111516293362989e+00,     2.5266611415621439e+04,    -1.6380053201865897e-04,     2.2689959015079834e-08,    -1.6837094331253080e-10, } , 
      {    1.0868497076726705e+01,     8.3647398411275371e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3310759479677372e+01,     8.3908245746980610e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6152393730669402e+01,     2.0775427837006646e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4929170253215283e+00,     1.0447387812463210e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3083981958087030e+01,     1.5542042944717083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8553312902801114e+01,     2.4983773074335644e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -1.8190607524327490e+01,    -8.3467403680805000e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0777642339585316e+01,     8.1140748869471354e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9111396361413028e+01,     4.8701865665076184e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.2939500999094250e+01,     7.1915209603277763e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7083784882212427e+01,     1.5543465635650835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.3404511403408620e+01,    -8.1204598547628848e-01,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    1.0463517945008332e+01,    -8.3160721864027473e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1714513145205043e+01,     1.6065123680807119e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5174610902465217e+01,     8.3413106630475049e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.0091171765440198e+01,     3.5923954219002314e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.2493972720923701e+01,    -4.2577557599925058e+00,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.3386646676344625e+01,     8.2092467190313891e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7642894832246267e+01,     1.5288322850508055e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3215401391086150e+01,    -4.8632097422075026e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.5828273195294329e+01,     2.4988375474015109e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1746948970999481e+01,     8.5831228644010298e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8689694766061457e+01,     4.7114589474569686e+04,     1.7491314032884309e-04,     6.2809968582230571e-07,    -2.9356971313593844e-09, } , 
      {    1.1383440923883029e+01,     2.2824391270242842e+04,    -3.4121569879297218e-04,    -1.9759686948812070e-07,     1.0269226430525967e-09, } , 
      {    1.1355474451439399e+01,     4.7008550444649036e+03,    -2.0784534589309516e-04,    -1.8892975834818931e-07,     1.0953686385516411e-09, } , 
      {    6.5581139434398423e+00,    -1.0699923894488384e+04,    -5.1779861016160861e-04,    -6.8851920297963685e-07,     3.3591673373981309e-09, } , 
      {    9.5829290893284700e+00,     5.7479094891311470e+03,     2.1949498231784432e-04,     3.2302515010816561e-07,    -1.4266224343065428e-09, } , 
      {    1.0294507733841250e+01,     2.3729470363678414e+04,     4.6226601490986172e-04,     7.5000055906133390e-07,    -3.4501377765215238e-09, } , 
      {    3.0134893219791788e+01,     1.6657054216973800e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.5052756917682402e+00,     8.2937465357151395e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7571075396925760e+01,     1.6657711481926701e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.7527185852328138e+01,     8.4481361304188631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2926658130409088e+01,     8.5433079625031169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5775500830965463e+01,    -8.2483293168041200e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8007115214980081e+00,    -6.6208901254850753e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.5414276292880391e+00,     8.4066883116163644e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7358441114278737e+01,     8.3227913706387099e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.5768389247227255e+01,     4.2923307555598194e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8808978034158365e+01,     1.5328137752405968e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0146056860757051e+01,     1.3763769729469712e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5406305476886946e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9245359201533077e+01,     1.8055984630366132e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0234366487447248e+00,     7.8592911313131572e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.2154576793289927e+01,     7.6329431902770966e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4335526780985376e+01,     2.5161842312503133e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.4216089323590253e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6590617423854912e+00,     3.5337070935533859e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.7340305167502549e+01,     3.5902950880144147e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5225083266388431e+01,     7.8198285453733197e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.7071015638498990e+00,     8.9569990097706686e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0141942280336703e+01,     1.5521424382206107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3288963750819704e+01,     1.5033891410832151e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4467921578075867e+01,     3.7698455102732592e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4112097431209773e+02,     1.6219054166649585e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4064836985053415e+01,     2.9200602714622470e+04,    -1.2438459796803637e-04,     1.2604467273420759e-07,    -4.5413951950573568e-10, } , 
      {    1.6825670541306199e+01,     6.7277362614410213e+03,    -5.9197837586371510e-04,    -6.2272310209880103e-07,     2.8871188005122087e-09, } , 
      {    1.6381438546002880e+01,     1.4985440010550810e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.7420471678117817e+01,     4.1157130519607825e+04,     3.8808624920775523e-04,     8.1579665994216972e-07,    -3.9221863134074464e-09, } , 
      {    1.2666236851866136e+01,     2.5128049625406049e+04,     8.0796866432150273e-05,     3.1559082319655950e-07,    -1.6039478863091666e-09, } , 
      {    8.9567941974396188e+00,    -2.9995344250117405e+03,    -3.6282197802735191e-04,    -4.3872448066391305e-07,     2.2272679879748859e-09, } , 
      {    4.1594336894400632e+00,    -1.8400313363965030e+04,    -6.7277524229586547e-04,    -9.3831392529536059e-07,     4.4910666868213751e-09, } , 
      {    1.1895357979832490e+01,     1.4704902869104755e+04,     3.6914385146628409e-04,     5.7405265665221466e-07,    -2.6674012402333669e-09, } , 
      {    2.4731396908248904e+00,    -1.8609862829795751e+04,    -2.4010715109910896e-04,    -4.2759180106733102e-07,     2.0779550704667709e-09, } , 
      {   -1.4121729732267421e+00,    -4.1875248941776067e+02,    -4.3000420970384798e-04,    -5.1133851634219225e-07,     2.4675513446063939e-09, } , 
      {   -5.3307470300931385e+01,     7.2347941712979718e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3791795196331568e+01,     1.5564084198161810e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7340518865806708e+01,     8.3444239088727172e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.1060801098268360e+00,    -8.3456121055339063e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -8.3644311985984089e+01,     8.3044185511156447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0911840470326952e+00,     6.5327861832020981e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6902776609800849e+00,     6.1927343107203023e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5405327899192883e+01,     1.9170613189632426e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.4457608691759489e+01,     6.8366451523188516e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7780914262316116e+01,     8.3529642983346093e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3023788319228364e+01,     1.5073353996771990e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7617191385356115e+01,     5.5228953248342805e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3118390082668384e+01,     8.1937046907429594e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2443998129776967e+01,     3.6759761869286535e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.4386881961036508e+01,     1.4914799021096491e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7232055677342263e+01,     7.4286794032368234e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8853362459482653e+01,     1.5525833609375179e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -6.0664897399025079e+00,     7.4684943051347364e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7395235391192121e+00,     5.2165804515628406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2462924814481880e+01,     1.4920438162112987e+01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.7882549618012966e+00,     5.4350783047445857e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8775683221181787e+01,     7.0790761314766660e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1939849872658694e+00,     7.3849151309987192e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1949944949954578e+01,     8.1538897888450474e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1047432742222300e+01,     1.4521425735445428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0845190840719182e+02,     1.5542852341757798e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.4516589404048304e+01,     1.5542656238610120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7615525543339373e+01,     7.0392612295787540e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8732516104179919e+00,     7.3073628699865958e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.5588195052506073e+00,     6.7434896347537206e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.8984053926503520e+01,     8.4636781587072928e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.5359134224878179e-01,     5.8564776908983958e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0089925886017674e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9121095362314279e+01,     1.5508972862680992e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -2.1848186536893373e+01,     8.3237603925692056e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.1483545015595826e+00,     1.9187669708227034e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8158026117922246e+00,     8.7980917181370951e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4375519264667652e+01,     4.5999960915303389e+04,    -1.9590048217723436e-04,     1.9122438150088073e-07,    -8.7174832813986779e-10, } , 
      {    2.1737248885245144e+01,     3.8995447515675281e+04,    -4.1006770150926161e-04,    -1.3303355283561022e-07,     6.6375356267025435e-10, } , 
      {    1.9353727840549613e+01,     1.2543219865172219e+04,    -4.2901009925073286e-04,    -3.7477755612556528e-07,     1.9185386358443331e-09, } , 
      {    1.8367220559776577e+01,     5.2615828851133356e+03,    -1.5398252168114166e-04,    -1.1323376209909678e-07,     5.8288664066118393e-10, } , 
      {    2.1049422008758803e+01,     3.2271124815249546e+04,     5.0366687313368307e-05,     3.4694789335157077e-07,    -1.7038728341226531e-09, } , 
      {    1.3326097255908845e+01,     1.3728836100053846e+04,    -2.4626716949060270e-04,    -1.5572351185069006e-07,     8.3212450598278531e-10, } , 
      {    1.2302700766412109e+00,    -1.6586395173834913e+04,    -4.9269619402864367e-04,    -7.1864348890632269e-07,     3.3502128287080375e-09, } , 
      {    1.6431683168672946e+00,     8.3785142107920387e+02,    -4.3533197268966504e-04,    -5.1010573211386689e-07,     2.3586718881028146e-09, } , 
      {   -2.7815682357154881e+01,     8.5034930606052058e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9790080935006209e+00,     1.6496361397840737e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4478153436135511e+01,     6.6812248694345544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4285219328115971e+01,     8.2335195926408724e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2924553633404079e+00,     7.2839526434788049e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1983650417887981e-01,     4.1643119620077772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8506958339379040e+01,    -3.6082627342699197e+01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.7361845711489156e+01,     8.8375543040769335e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.6617311575658377e+01,     8.2317886241624456e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.3503225753735046e-01,     4.1948470488827297e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4083426492870306e+00,     1.4861477607927225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1217648581182260e+00,     1.1499656302018273e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7058792402384144e+00,    -1.7429361443118366e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3956028199431834e+01,     8.4177089383387283e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9537600407284060e+01,     7.1802814379558667e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -3.0103787155836548e-01,     7.0720762546340393e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9453081452761474e+00,     8.0132227160442617e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.0150791858926198e+01,     8.4238632568093799e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6571152930788056e+01,     9.0989856205954620e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0214427652916029e+00,    -1.2451519525112520e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.3912665531528454e+01,     7.0946181597650957e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.1248294216507475e+01,    -1.0636614249512490e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0407339884286401e+01,     1.5576535717686924e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.7900738543870460e+01,     7.1897899918493513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5451108503436842e+01,     8.2733344945387835e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2630187609463217e+00,    -1.4226909337526763e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9015862875517513e+00,     1.1513883211355801e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5070245742093498e+01,     1.9312599800457217e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9935121291190484e+01,     7.1188910333745789e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.1316378667365342e+01,     8.3840483549114670e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0392081387890457e+01,     1.6650823893850524e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.1129757259576056e+00,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.8565098683291357e+00,     8.3132723136859368e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5984496019616536e+01,     8.3336224568810467e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6666312273319992e+00,     1.7789845607142468e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1819526053783669e+00,    -1.9187041826395939e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0341112931642068e+02,     7.2383357390683150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5240307591763269e+00,    -9.4433199839914196e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1910058602078472e+01,     1.5072181059478848e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9806857215134098e+01,    -1.3506511271354015e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.5371966312859762e+00,     1.4425824531077931e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3286458878989587e+00,    -8.3352503803248546e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.1446436157410680e+01,     3.4227046505595630e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5139376542691025e+01,     6.5181867956102997e+03,    -1.5931028466695867e-04,    -1.1200097787077145e-07,     4.7400718415760441e-10, } , 
      {    1.2382715656883686e+01,     5.8188971647464805e+03,     3.1424289571897069e-05,     1.0520389006161575e-07,    -4.4908776094857428e-10, } , 
      {    8.9847606698832507e+00,     1.5124001800766198e+04,    -4.9619233092722899e-04,    -4.4739159180384447e-07,     2.1588219924758415e-09, } , 
      {    3.2440185628585358e+00,    -8.1867160734944555e+03,    -5.2845413613324273e-04,    -6.8605363452298613e-07,     3.1414084243909714e-09, } , 
      {    4.0731823259443063e+00,    -1.7143709453468062e+04,    -6.7810300528168242e-04,    -9.3708114106703523e-07,     4.3821872303177949e-09, } , 
      {    1.2638270379422506e+01,     7.0045133996281111e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    8.3860247414355982e+00,     2.0426571094770613e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0166869338274310e+01,     6.2702865717324257e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6562705606749837e+01,     9.3369020682642022e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0528114622822333e+01,     7.7627406467085511e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.2740822622858889e+00,     9.4263993031826394e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.1094018947107955e+01,     7.1587059352724909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3435270863708073e+01,     1.6623601421947289e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    4.0130525747004722e+00,    -1.6691164276953219e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    8.6557024627896453e+01,     6.5702132521248595e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2051660221961356e+01,    -8.3129589405775332e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.4844314667699081e+01,     1.4430426930757400e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -7.9983207796580700e+01,    -7.0463191614976495e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8215183478755137e+01,     1.6640109852683396e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7796871041366977e+01,     1.6486530583910364e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1918911038656983e+00,     8.6441601334059906e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5261188435148796e+00,     7.1787257945232986e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4831396267666648e+00,     1.6654753017134070e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.0804863873902235e+00,     1.6660012681766435e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0786778404426682e+00,     8.3073615167472744e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.7730084990478563e+00,     8.3500213327029778e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6406418388276940e+01,    -3.7387614536963997e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3183681915654539e+01,     1.6093509614217375e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.3272134391746576e+01,     7.3490495994409994e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0511930556626670e+01,     3.2197835939794481e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -1.2604508825845206e+00,     7.1773031035895456e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6181130177454591e+01,     1.8362316463774732e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.1560221516084574e+00,     7.2316948917344898e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6803869110743896e+00,    -5.3680442678067706e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5421717870981340e+01,     1.0973555624994374e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5862082144948157e+00,     1.4286139120092872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2526119911633824e+01,     1.4598977996457552e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.8261021089041627e+00,     3.1083207380528183e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    5.4101336997005864e+00,     4.3770833509159729e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.3869529495251214e+01,     3.2202438339473945e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.4545509986746037e+00,     1.6499648495109821e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.6930158811604414e+01,     3.8941817365806946e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9555386896263812e+01,     3.1087809780207655e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.3479884713285763e+01,     8.3106424813697522e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.4431600826351037e+01,     7.3092346975430864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9971264731143187e+01,     1.5367952654303879e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1133058655099937e+01,     1.5407767556201792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.5590017169578807e+01,     7.2694197956451735e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3118736516941233e+01,     2.2291439747319600e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -7.4254098777782218e+01,     8.3279867928089770e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.8389929960993868e+01,     8.3293960566412752e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4494551591322164e+01,     2.3713711360568654e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6459629278408741e+01,     1.7427677045320590e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9792513668024081e+00,     5.7533848783435178e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8882119886508789e+01,     1.4898904676849985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5463685868412353e+01,     1.6644264938250795e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2622602221394200e+01,     5.9452615754257886e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1816839018870095e+01,     6.5883851127294747e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3869771391018745e+01,     3.0943521969543126e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.4972890322418220e+00,     7.2302722008007368e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1513751487668500e+01,     7.5226051188301062e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5047592597276949e+00,     5.6431786111198778e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1728774579984304e+01,     7.7229257448106400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3886389339768179e+01,     1.0983908375732091e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2111208221373506e+01,     7.3888645013389123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3162881159469748e+01,     7.3335075711525697e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7489907101206128e+01,     3.1571829554588669e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.5884810612749636e+01,     1.6867260947757422e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.1821209552577375e+01,     1.6866603682804522e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -4.8049660944924156e+01,    -1.9907450948956102e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7998634730504310e+00,     2.4356780791700485e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0646357963812092e+01,     7.1081246710364503e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2123756944523969e+01,     1.6168760675389767e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.6151673931781678e+00,    -5.0435465820807576e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.1065392228093218e+00,     1.6815117203790684e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.6540509962789720e+00,     1.0213614179861755e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.5093591391867079e+01,    -1.2205924819586471e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.1715498294501465e+00,     5.9574476953677458e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2089647514019399e-01,     7.6474147424683915e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2174324199311471e+01,    -7.9853460851858472e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.5860171700661159e+00,     1.3367972634193943e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2161110227292138e+00,     4.6400665825343822e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8827296856487756e+01,     1.0355600790686549e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2063090374163629e+01,     3.7042967535329779e+04,    -3.4554935132567413e-04,    -5.9803125043168372e-08,     3.6903047778695648e-10, } , 
      {    2.9707535801911725e+01,     2.8714276110604656e+04,    -4.9786210196702235e-04,    -3.1021423947305477e-07,     1.5553695554619909e-09, } , 
      {    1.9381694312993243e+01,     3.0666756090950155e+04,    -5.6238045215060994e-04,    -3.8344466726549669e-07,     1.8500926403452889e-09, } , 
      {    2.3149421858471886e+01,     3.9414200005093044e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.8736993118254780e+01,     2.3314131435275936e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.9106765676732991e+01,     4.1366679985438539e+04,    -4.4581841989001205e-05,     3.0507453571414015e-07,    -1.5090746970528416e-09, } , 
      {    1.5720964972257104e+01,     1.5508997243046513e+04,    -3.5358819241625563e-05,     1.0091396772294991e-07,    -4.8258353817643211e-10, } , 
      {    9.9527016478066734e+00,     2.3800458039293750e+04,     2.7419532216391455e-04,     5.3217929901478405e-07,    -2.4726031031635556e-09, } , 
      {    7.4677700303111401e+00,     1.7356672480314071e+04,     1.1389092704384075e-04,     2.8361736092738562e-07,    -1.4495832102438902e-09, } , 
      {    2.8429122493030961e+00,    -5.5731427963314673e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.2232846975248446e+00,     6.7514094274891249e+01,    -6.1658603963510780e-05,    -7.3934086369204231e-08,     3.4770837209175919e-10, } , 
      {    7.0666637167557003e+00,     2.4986074274175378e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.4880368380239922e+01,     1.5757370827961951e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3087923163898983e+01,    -1.7085226553988696e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5093084806069159e+01,     8.8871092317506973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9667683258551030e+00,     6.9918941613057323e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0061560621399046e+00,    -6.2567775272935887e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1320083183774644e+01,     1.5542530633127924e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8898562063214722e+01,     1.5542977947239990e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4635802223965211e-01,     6.4437686695884968e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.7113875946814332e+00,     1.5797185729859861e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2534431213765023e+01,     1.1035273924221087e+00,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {   -6.5403920979117331e+01,     2.3878329751503742e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1878032345258951e+01,     8.6056647695320844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9642845290301647e+00,     7.2138392084027992e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4707999075430411e+00,     3.9302096969298748e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0672235736851814e+00,     8.4346296191475085e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.5268870904991960e+00,     2.1228392028053695e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0246469873659283e+00,    -6.9889778019972539e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1499922745299171e+00,    -3.4944889009986269e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.8500115090727736e+00,     6.5459403785153772e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6543284306367187e+01,     7.2142865225148671e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4411029057772769e+01,     3.9324626244204808e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1562767137985183e+01,     1.5423309584490224e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0915174861851197e+01,     8.3985212686305003e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.3774055443414667e+01,     6.0403471143606112e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9215433041216144e+01,     1.6879551553603353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2080536538143370e+01,     6.7450148880049383e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1523663623752386e+01,     1.4914446705138416e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6375657344394288e+00,     1.1015106480668383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5823031620572785e+01,     5.9972625972656588e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.5555930320530649e+01,     1.5518481416574477e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7532872130664883e+00,     6.2901893051241950e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3866298802786599e+01,     6.9787762501062771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1980610663643068e+01,     6.1693240842125115e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4256091698449591e+01,     1.4917076537454601e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1176349821704381e+00,     1.5558753798508444e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3018843219830892e+01,     2.3320690390875669e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0542150835030606e+02,     8.3281834661870926e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0071039920580225e+02,    -8.3291993832631579e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8173119605488381e+01,     1.4325954021990785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2075471147949877e+00,     9.5853065948162130e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2294566293031622e+01,     1.5447582458099705e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6200006252022023e+00,     6.9143419002936080e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1881097403102867e+01,     6.8985941567779691e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8104366142059050e+01,     8.3931257745377952e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1567496114079766e+01,     2.3245439329703277e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6811733572018682e+01,     1.9045585916046548e-01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.4958341534700832e+01,     8.0569869882823677e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8387251166907639e+01,     6.4109536445332105e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3882278547904718e+01,    -1.6756516827080486e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6014145552988637e+01,     1.1470229349952842e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6831796602397233e+00,    -1.4841705721395204e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6129023088661818e+01,     8.0171720863844548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3628126131085265e+01,    -1.1139172137994171e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.1693709237401375e+00,    -1.1153399047331698e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.1849905604180364e+01,    -1.1142762433082185e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.2256964758689193e+01,     1.6173363075069235e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5817793992825422e+01,     1.5526754781859474e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4336000158891443e+01,     1.4498015508937637e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2562769709429702e-01,     1.4874631803240503e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1044500886026146e+01,    -1.1149808752243684e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.3207471680478015e+01,     7.5251061172388290e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0578100765745873e+02,     1.5567027163793440e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.6262501695222591e+00,     1.2168002681931741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1216418069614940e+01,     2.8286990488630363e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8021924693034137e+01,    -8.2665582747521894e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.8002395617622398e+00,     6.5850439349464123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4571996536099313e+01,     2.3349076324285925e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.7612250529287419e+00,     7.2300623737833184e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.1624275229155877e+01,     8.3290504542339277e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5760162334429054e+01,     8.3283323952163246e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.0090469795597246e+00,     1.6617567947552339e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3977763339802474e+00,     7.8581181940200140e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3019343530631032e+01,    -8.1867048139003327e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1639484244181629e+01,     2.3868815882592899e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.1948588259252936e+01,    -1.6003555418016731e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7574312323693324e+01,     1.6697197751348165e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.5253167077167500e+01,    -1.2201333222521754e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7008031883029595e+01,     2.3874075547225271e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.1844312792974687e+00,    -8.9737562790106324e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.5335658328931572e+01,     8.9566466938125941e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8342941994277133e+00,    -7.6441243458667969e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.6047085464297595e+01,     7.7007361556376572e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1578079666041173e+01,     8.5359396907161135e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.6174173191484549e+01,    -1.0903248867535331e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.3455736473390161e+01,     1.5487397359997618e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0226349360770470e+00,     6.4280209260728570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9495117026112069e+01,     6.8692105572661603e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.6600956479533799e+00,     4.2659815955703628e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0396532757572103e+00,    -1.1133368851508840e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5036554972507444e+01,     6.7470825087088679e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1498246812792994e+01,     1.4756712350797985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1297561683800211e+01,     6.6394357230913574e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2824358441841895e+01,     2.3240836930023808e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.0729472070544738e+01,     1.4739645069258337e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0692323539970374e+00,     1.3029523950001785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7984659313729630e+01,     9.7299230379571891e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2803404250240618e+01,     8.5405420903955819e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    8.2531314803639315e+00,     1.6604413752239059e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.6326885146644301e+01,    -1.3791187595379597e+01,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.3627249835428955e+01,     2.4985721958217306e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.0902207344129039e+01,     2.4986426590133451e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.6642483781270073e+01,     1.9997357440854557e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8792120675330199e+01,     3.8927873401075231e+04,    -3.5354099580439967e-04,    -5.7953948700680372e-08,     2.0571129303158710e-10, } , 
      {    2.6436566103078299e+01,     3.0599181976350104e+04,    -5.0585374644574789e-04,    -3.0836506313056678e-07,     1.3920503707066215e-09, } , 
      {    2.4092803457497045e+01,     4.7324138940400408e+04,    -2.5775495086791334e-04,     1.1737756159427619e-07,    -5.2258551500478010e-10, } , 
      {    1.6955047586549831e+01,     4.8428303956955751e+03,    -5.8398673138498956e-04,    -6.2457227844128907e-07,     3.0504379852675781e-09, } , 
      {    1.8823244481750535e+01,     2.2057527524778972e+04,    -9.3954418849254408e-05,     9.4687602579196358e-08,    -3.5421457169224920e-10, } , 
      {    1.8406978958890850e+01,     4.8438767499666697e+04,     1.1305867163816407e-04,     5.5425286591570121e-07,    -2.5865343182242963e-09, } , 
      {    1.6794336786228968e+01,     3.2409686605464933e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.4352530850481310e+01,     2.5337599091236771e+04,    -3.5187122476460620e-04,    -1.9513130103147004e-07,     8.0916373004543754e-10, } , 
      {    8.2144949675526746e+00,     1.5576511337321404e+04,    -9.7017423205136323e-05,     2.6979881353745674e-08,    -1.3487516608467297e-10, } , 
      {    2.0691441376951239e+01,     3.9272224653862373e+04,     3.9607789368648077e-04,     8.1394748359968168e-07,    -3.7588671286520770e-09, } , 
      {    1.3365410400005203e+01,     7.1803058183213871e+03,    -1.8767156988297380e-04,    -1.4949714670693654e-07,     7.0375553949860234e-10, } , 
      {    4.6993172178331548e+00,    -8.3962655393251771e+03,    -9.5786044936486234e-05,    -1.7533151029495659e-07,     7.2829680803636702e-10, } , 
      {    1.0695614047396690e+01,     1.6100068569817106e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9857820137736943e+00,    -9.7238571254374765e+03,     9.7612410795278029e-05,     4.1256965523267953e-08,    -1.4035840881802174e-10, } , 
      {    6.3111517796232164e+00,     1.8123536225777938e+04,    -1.3337035289987702e-04,    -8.6671111399313961e-09,    -6.8445995499044334e-11, } , 
      {    7.9820788433372281e+00,     1.4772476983704806e+04,     3.1261714576142204e-04,     4.9897305251728480e-07,    -2.2093589705946995e-09, } , 
      {   -2.7494687029754846e+01,    -4.1369104726755222e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8176287410409387e+01,     2.3871117082432636e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.5297155698021356e+00,     2.3871774347385533e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.7183307734252303e+01,     6.3711387426352985e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0587277740438624e+01,     1.6710351946661445e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0022436396262165e+01,     1.4363691381104996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2551434517098016e+01,     7.7710248291339039e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.4203309292110504e+01,     8.5260349657362622e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6089897106542693e+01,     7.7717294610500539e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4652126838040218e+01,     3.6027960349179011e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6872359533421273e+00,    -8.6750524714155702e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.0926492087568136e+01,     1.5677741024166125e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4420954764988068e+01,     7.8427227664624506e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1737474898387124e+00,     7.8604193938597496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6385902821250831e+01,     7.8596434472712317e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1720705408380990e+01,     2.5128060885000170e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.8303535218528282e+00,     6.5831254480971056e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8055908285916839e+01,     5.6074473003961452e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.6586491115364385e+01,     7.8420181345463016e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5543531197267541e+01,     1.5662198995877696e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.2971886969783043e+01,    -3.0592045994253567e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4456549433723346e+01,     7.8400692506646410e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4438966076811468e+01,     1.5073706312730064e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1527319897033854e+01,     7.7029419587033613e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.1557706890573405e+00,     8.2227532303027438e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7961283072696759e+01,     1.9940907385502760e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5712802122122600e+01,    -7.6872181463180596e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.9169624188769703e+00,    -6.6113995532934311e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.7016870683201596e+01,     7.2321118088142066e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3759343405557427e+00,     8.4308631336520066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1952295016594339e-02,     7.1250453518452305e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4362536102695103e+01,     8.3129436812094864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2936658605069365e+01,     1.1452814815759980e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.5100823302923354e+00,     9.5559973877790526e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2634975006603565e+01,     3.2217200185560241e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9486244563953274e+00,     2.5158601724983564e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5459999408377190e+00,     6.6380130321576044e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0995053199052862e+01,     7.6848418113911530e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4557436838298024e+01,     8.1694318171334780e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0675297263810371e+01,     1.7285690434495795e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.1913961937331141e+00,     1.4911816872822232e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6250640218611011e+01,     2.2756464839684719e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.2086927082883264e+01,     1.5637926122268213e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2552271981802647e+01,     2.2757169471600864e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.5754372256747073e+00,     6.0349999563798701e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0711757219244312e+01,     8.6454796714299973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.3245361229028859e+01,     1.5598111220370300e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.3041734777702111e+01,     8.5658498676341733e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.8972693326078002e+01,     1.6641840821161823e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3345413040555439e+01,     8.3198312692856634e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.8056898808055887e+01,    -8.3375515801645888e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9679474962306237e+01,     7.2294819764980220e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4135410507477079e+01,     2.4029180069249516e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5078459426859300e+01,     8.6284123898903508e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.0125338874769710e+01,     1.6672924877738682e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.8667226553490535e+00,     1.5890729163357412e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4823150772765814e+00,     1.5400767679359165e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5365288122922358e+01,     8.0776570658569317e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7572545138160649e+00,     1.1790629090789623e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4247303819172927e+00,    -1.2723629136067254e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9588570317564280e+01,     3.2828444724679757e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0239072515685326e+00,     2.3831630813011172e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3042434647781430e+01,     1.5436816095761576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4152206838639268e+01,     7.3200010598812150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5039103721462728e+00,     1.1926254461575309e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6321774710981241e+01,     2.3818476617697892e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.1154699929901341e+01,    -1.3238257091389401e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.7447838258075169e+01,    -1.3245303410550900e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.3729028891357588e+01,     3.2809646224778923e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9519651053865346e+01,     1.4699830167360424e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1975470678576015e+01,     7.9225525438326831e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3334143660231710e+01,     6.0640972352570052e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5136115163173471e+00,     3.8027696093255372e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0170517794777941e+01,     7.2490077544688183e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9762720507152565e+01,     1.5717555926064038e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5052018186019183e+00,     6.5055731870849813e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.9785757382688857e+01,    -1.5252532512172365e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2478245329965279e+01,     3.4713367479816385e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.2337259966445401e+01,     6.5861075963713638e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8704732397916615e+01,     8.4879510323167742e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7750196474206028e+01,     2.8127612199403095e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.8308309593567351e+00,     1.3515844924222536e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4237994256500702e+01,     1.5227285581503094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3343065147347490e+01,    -6.3017820587637034e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.2668378243944629e+01,     6.5061254766174652e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3405057086552588e+01,     2.7012983640136801e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5688682267473450e+01,     1.4392788659982130e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3907604922337541e+01,     1.6679924754581309e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3993196424055974e+00,     1.1506769756687036e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2951569100677808e+01,     8.3347248991200740e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8240459956174007e+01,    -8.3226579503301782e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.5617887962308696e+00,     6.0626745443232521e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9990829269240509e+00,     1.6106627525416832e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.5360860062513693e+00,     1.6135013458827092e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.7811775270375421e+01,    -5.7087898664768545e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.1256882654276303e+00,     1.6091432071433679e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3650595823976463e+01,     1.4968127147816484e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8605285964786011e+00,     9.5323374976050218e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5631763798781122e+01,     2.3911260616806998e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2428483192270797e+01,     3.0457200995322375e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.2135210025338239e+01,     7.2711507641236012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2182120667483778e+00,     9.0099681069818598e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5682599106431745e+01,     3.5341675064861927e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.5565172648763351e+01,     6.5303983502269475e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9162849946192484e+00,     1.4391365969048378e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4433485513296873e+01,     6.4905834483290355e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0825248261780306e+01,     7.1338807721305247e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5279983739157537e+00,    -9.6190880223191762e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0978832904209575e+01,     8.3593443470348284e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    9.4604353674602013e+00,     7.8416591050374991e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4460459432606996e+00,     3.3406124347172772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1099360068293407e+01,     1.5859207582252566e-01,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -2.0963321404350289e+01,    -7.0793738884136769e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2493599416311431e+01,     7.4493494808113883e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0323927620447783e+01,     1.4834816901342590e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9560012241201100e+01,     6.4507685464311235e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3236010168497884e+01,     6.7848297899028512e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.7780442189532756e+01,     8.1919737222645344e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0192705515755128e+01,     3.9218192554577058e-01,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    1.6168283046096349e+01,     1.5307467674831403e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2246216226742028e+01,     8.7977394021790205e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2741568862846819e+01,     7.5255822037995040e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1207627133766394e+01,     2.3854525034100301e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.5585047298051350e+01,     8.9573513257287450e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.6506264918185209e+00,     1.1292505667315709e+00,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    5.5871198198323071e+01,     3.8814179353585132e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1181182294911718e+01,     2.4078693980900072e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    5.6225767599532244e+00,     7.7540642823576509e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9649043172219645e+01,     9.5914577475899023e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.6141361043040160e+01,     1.6577753045654426e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.6294769857218432e+00,     1.0969965329906358e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.4791637492835733e+01,    -6.1276555675711370e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4055665087200012e+01,     3.6147409882490110e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8767992982182687e+01,     7.0548032578671846e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3200343067144473e+01,     2.4083296380579541e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    3.0023295597938148e+01,     7.2226993638422609e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.5487483066113565e+00,    -8.1529233863973686e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.1799139923244475e+01,     1.6170350529762625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6491133007326887e+01,     7.1971421846500516e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.8709542709707712e+01,     1.6737012653246078e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.5645461738692177e+01,    -5.5260821264806509e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.9638768634350141e+00,    -1.6653838832997126e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5684425549991658e+00,     6.3093741736172706e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.5365463077268856e+01,     8.4862200638383492e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.2130713590996205e-01,    -2.2041253444728142e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9009143323106699e+00,     1.4966704456882731e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8574769022203430e+00,     1.4834264611810107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7859967419468397e+01,     2.3242785813905470e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3450612981946822e+01,     8.3301141156588783e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8626443844202178e-01,     8.3272687337913721e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7961978867737891e+01,     2.3872157060375961e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.2064132133706664e+01,    -6.1819776864531377e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.6592425952109913e+01,     1.3431284307855094e+01,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.9634868877555514e+00,     5.3291401103222033e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9141610457103404e+00,     5.2308073609003679e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8257941207906129e+01,    -4.6446193678602299e-01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.1828251556560900e+00,     8.2485093313968173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0259544861620586e+00,     1.7179752240073412e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8560615603831167e+01,     2.3924414812120278e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.2806396401308405e+01,    -2.0350143195108103e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0672309875578598e+01,     2.3870734369442209e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0699768558717782e+01,     3.6978817021943087e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.6678794607209056e+01,     6.6012973087017181e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.4552157835013162e+00,     9.7430409491566443e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.8522652836867355e-01,     6.6026759612220712e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9406136978568849e+01,     1.0122944007650178e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1902631552882710e+01,     1.4779459971156250e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5063736785633823e+00,    -1.6369979498894538e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.0532489831125712e+01,     2.3298541551282142e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3734553655878656e-01,     7.7526415914238978e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5482583997157180e+00,     1.7208138173483672e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.7904417715356686e+01,     1.4859089774952074e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0938223359042176e+00,     6.5497068640108791e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6979303470171928e+01,     2.3855903686620659e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.6776240737508104e+01,     1.4819274873054163e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0302616441075228e+01,     8.3853584036385128e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1370353530041665e+01,     2.9088811421913433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7351981229659827e+01,     2.0385584685879530e+04,    -6.5017485260837051e-04,    -5.6062535390294129e-07,     2.7417086331370257e-09, } , 
      {    2.1324350645019056e+01,     2.1571200920761163e+04,    -4.6743192284824035e-04,    -3.4157130962806608e-07,     1.6552945032754774e-09, } , 
      {    2.5461850748975909e+01,     4.8371193385066654e+04,     1.6958537734302609e-04,     6.2933247005063107e-07,    -3.0445765878629637e-09, } , 
      {    1.3640952205968528e+01,     7.3560382166895033e+03,    -5.9464225735662369e-04,    -6.2210670998463835e-07,     2.8326790722604187e-09, } , 
      {    1.9936737050690581e+01,     2.1918965734563586e+04,     1.5064297960155483e-04,     3.8758846676067602e-07,    -1.7897915146888849e-09, } , 
      {    2.0722775132028477e+01,     1.3590274309838462e+04,    -1.6697710397934066e-06,     1.3717735233078965e-07,    -6.0345243701385052e-10, } , 
      {    1.0970542683656946e+01,     5.4001446753287200e+03,    -3.9857992013195092e-04,    -4.0613462628057648e-07,     2.0184635836578199e-09, } , 
      {    8.4856110661614110e+00,    -1.0436408836509597e+03,    -5.5888431525202462e-04,    -6.5469656436797492e-07,     3.0414834765774851e-09, } , 
      {    1.5438056946977722e+01,     7.0755010752434446e+03,     2.6096526586080048e-05,     1.0643667428994108e-07,    -5.5796721745215386e-10, } , 
      {    1.6094550068386827e+01,     3.9481774119693087e+04,    -3.6590197510275672e-05,     3.0322535937165217e-07,    -1.3457555122974722e-09, } , 
      {    1.4438782213977067e+01,     2.4080995180739807e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.8589403953007739e+00,     7.2478199125962783e+03,    -2.4933017384648460e-04,    -2.2343123307614075e-07,     1.0514639115903615e-09, } , 
      {    2.3003870267455266e+01,     4.8229218033835983e+04,     5.4572676283492052e-04,     1.0649749901437308e-06,    -4.9996459345789005e-09, } , 
      {    1.8379012486447223e+01,     3.0315231273888763e+04,     2.4642902453804108e-04,     5.6291997705563258e-07,    -2.5180883227252519e-09, } , 
      {    4.9586844780234927e+00,    -1.2904209779595694e+03,     3.1025166708923645e-05,     3.6879776722002399e-08,    -1.7530862708920817e-10, } , 
      {    1.4950699269926528e+01,     1.5961506779601719e+04,     3.6381608848046703e-04,     5.7528544088054002e-07,    -2.7762806967369463e-09, } , 
      {    4.8718199448246686e+00,    -1.0909473360319105e+04,    -8.5130518964852191e-05,    -1.7779707875160725e-07,     9.4605572104352629e-10, } , 
      {    1.0766706201588200e+01,     3.3595302840346565e+04,    -1.1487781377310728e-05,     2.7310107344496626e-07,    -1.3547100209875654e-09, } , 
      {    1.5363597510152612e+01,     3.3385753374515843e+04,     4.2118030981944571e-04,     7.8382319767299583e-07,    -3.7678216373421700e-09, } , 
      {    8.4694365203884239e+00,     5.8864712793465324e+03,    -2.5102416132964941e-05,     3.0124285926685901e-08,     8.9545086900931297e-12, } , 
      {    2.7566608858073396e+00,     6.9928963086381782e+02,    -1.9073457423885576e-04,    -2.1720486793238720e-07,     9.2309494510617863e-10, } , 
      {    3.5426989671452325e+00,    -7.6294017938613088e+03,    -3.4304732488020398e-04,    -4.6761598236227365e-07,     2.1094340227812128e-09, } , 
      {    4.8735767705119581e-01,    -8.8860057043582729e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    5.3048335879907405e-01,    -9.5143076596067549e+03,    -3.3505568040147844e-04,    -4.6946515870476163e-07,     2.2727532075365822e-09, } , 
      {    1.0365599888032758e+01,     4.1224704634207868e+04,     3.3155954350289329e-04,     7.4071705580723986e-07,    -3.4641440437687786e-09, } , 
      {    5.6976164252768378e+00,     2.3939019829509136e+04,     2.9597923713105281e-05,     2.3927843483330439e-07,    -1.0370261601669196e-09, } , 
      {    6.3974031431189733e+00,     1.6866932315280974e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    6.4836545066147302e+00,     1.5610328404784010e+04,    -1.2271482692824295e-04,    -1.1132679596582053e-08,     1.4931291750811484e-10, } , 
      {    1.0935396390928192e+01,     2.5161866692868654e+04,     5.5099462709043649e-05,     2.7747826224623181e-07,    -1.3197598027163788e-09, } , 
      {    6.2242872464243924e+00,     8.5044838434184021e+03,    -2.4952603857365287e-04,    -2.2334396661354104e-07,     1.0529183526336901e-09, } , 
      {    6.9804123532599434e+00,     2.6242678184672342e+04,     4.5161048893822771e-04,     7.5246612751798462e-07,    -3.6678966895286833e-09, } , 
      {   -3.3334222617860636e+00,     5.8849268314763094e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4359325837018616e+01,     1.5383024512689856e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.9901388980907285e+01,     7.8446716503441094e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.4231364463490017e+00,     7.6834631588708016e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9569681837027364e+00,     8.4088735180534331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6842493576087715e+01,     2.2669888994440441e+00,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    6.7594541084130313e+00,    -2.6906804285552005e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.5733017259344273e+01,     3.1337310839808197e-01,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.7475514204961236e+01,     8.7085944832186578e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0283903150455718e+01,     1.0984192239243887e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2034394246322009e+01,     1.5558173401223148e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1555810782722244e+01,     7.4857673019015920e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4717293372203306e+01,     7.2942449587871397e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.2052305233800311e+01,     2.3886987743197511e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3532147231254672e+01,     8.3265491864555006e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.8210380867508711e+00,    -8.3308336629947516e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1399414051770432e+01,     1.4392077314515254e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2490589208294489e+00,     7.4650972243270280e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8930059277331885e+00,     7.7030136719957682e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3870637645164980e+00,     6.4963749302339500e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2000547439710012e+01,    -1.5747743515638831e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.7841804430852193e+00,     6.6387243776244804e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0123053986692980e+02,     6.5550296898800298e-01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.9933761859154824e+01,    -1.1169297591060276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.4333125841038822e+01,     1.4929988733426844e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3463392896709783e+01,     7.1980633571343469e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0081588319241721e+00,     1.6730463662918642e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8385035204288496e+00,     7.6202017463512748e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.7551369383589453e+01,     7.8994583491691446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6593941470375384e+01,    -3.1132458252141078e+00,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {   -1.8157548022100531e+00,     1.5527335179144769e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.2616000797755227e+00,     8.7992646554302373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1158536867164631e+02,    -8.3304223932035529e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9373176493963136e+01,     1.5560803233539333e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.6279037369454468e+00,     8.5639780400776817e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1300414816442350e+01,    -9.3300004527762781e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1842154537931922e+01,     1.4914775337614865e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.8256327259744900e+00,     3.2407385405625195e+04,    -4.1917960496092694e-05,     3.0445814359997747e-07,    -1.4546349688010517e-09, } , 
      {    1.1787565181899149e+02,     8.3266559077080510e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0373983842212779e+02,     8.3307269417422012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3353707783978694e+00,    -1.6834175672870383e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.1718444112001247e+00,     7.1986437544196433e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.5555900550336389e+01,     3.2411987805304667e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.6834527974653213e+00,    -1.1172583915824780e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9138045786912135e+00,    -1.1020626722364071e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5476630538037588e+01,     4.0077619579419988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.0627471677935745e+00,     2.3791815911113259e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0930734884505917e+01,     1.6012154583595928e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8999231202296478e+00,     7.4781665690567305e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1534391738365333e+00,     1.6579830588438130e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.8275345737662079e+00,    -1.1119987269501098e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.6944529821877516e+00,    -1.1997610547375871e+01,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -3.1013975080182469e+01,     7.9773571844865419e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7591724566385910e+01,     8.3266138819414264e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5444517734046116e+01,     8.3307689675088259e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2898043837424527e+01,     6.6924048203025486e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1549475721264134e+01,     1.8073704941245055e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0526936095286260e+01,     6.6284150963689926e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6583374517932636e+00,     1.6734935110462378e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.6079332904033006e+00,     2.8449140577913781e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.5592490486277242e+01,     1.5113521214627977e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5784065133047021e+00,     7.7533529368907739e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.4378596811934301e+00,     1.5524705346828585e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.0021408553016730e+01,     1.4285786804134796e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2252528378718610e+00,     8.9033185670926014e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.3428051867974418e+01,     2.5426438423554032e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1033359989343294e+00,    -4.2680239719595193e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.2650145454187509e+01,     8.3284857094979179e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6467834506411933e+01,     1.0677292205638742e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1543535277572397e+01,     6.5082030194011659e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.7931410685772491e+00,     7.1960139221034588e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.0221030501192159e+00,    -1.1634615070219354e+00,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.7575033450909334e+00,     8.8783884527392942e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0739241776936728e+01,     8.2691728477003289e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.4322228204588754e+00,     1.6673382357774739e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.1421812843868633e+00,     8.1114384933598958e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1650647800106004e+01,     8.0338862323477715e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.9306391354958254e+01,     1.6051617169535766e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1106753541977403e+01,     9.3500199794636574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.3245553297133768e+01,     8.9725410380591002e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0129262336625656e+01,     1.6727212693355628e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.9005432721577584e+00,     6.0633858897901282e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6809752934695631e+01,     1.7346683848802240e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8488631352776451e+01,     6.5342204455608197e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6357372848142873e+01,     8.8999111111059010e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.8504595340403540e-02,     7.6996724942127057e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8788575264121505e+01,     7.6052120075953289e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8091180374529557e+01,     3.4960327176015744e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7554011474246778e+00,     8.3989733846706458e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.4244942127642741e+01,     4.5515411966592092e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9150250227185595e+00,     1.3735383796059454e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4197600143291695e+01,     3.9071976985325455e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2756396471678126e+00,    -1.1675976564774849e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.7427612508683330e+00,     7.6605689377816707e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.7139069709415381e+01,    -6.4087775507394326e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0199799947694235e+00,     1.3710800632258521e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1421597602573126e-01,     5.2257751744315044e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4064073504709413e+01,     1.1371704643973499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9935680372072781e+01,     7.6450269094932410e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0704911694222970e+01,     1.4205957026764565e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2351228650568919e+02,     7.2120273484417576e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3508003989790694e+01,     1.6031376464244444e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9888505958058502e+00,     8.9047412580263535e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2881229310462036e+01,     5.1697389653045479e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.8005012997417253e+01,     7.2160983824759078e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3930058055245397e+01,     2.7608977078325552e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.2059112141149448e+01,     1.7665593492989326e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0566829392795370e+02,    -1.9475210194959800e+00,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.5009898639546693e+01,     8.8636990052429064e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3412930881222678e+01,     8.7579245002811076e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8302395858079397e+00,    -7.0585984605766726e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9098008252303407e+00,     6.8800016219871050e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1941709652512580e+01,     1.4967415802349607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7031432183147732e+00,    -3.1283887636467944e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0610876728825902e+00,     9.9176968403512956e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3185464140420745e+01,     1.5462572196855652e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9183432073847051e+00,     8.9577103552375447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0294946697285372e-01,     2.0618562510975480e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6415560526266649e+01,     1.5383494682592311e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.0497713284222669e+00,    -1.1544434611642057e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.3910874921676040e+01,     7.5653971056974169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.3584392989901364e+00,     5.8772656020394388e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8785394583251396e+01,     1.4427773414959591e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0609099420316909e+01,     1.7272572523296341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4055273148310565e+00,    -8.3418093359245813e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.0377384949159193e+01,     1.4428478046875740e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6788183353300816e+00,    -8.1982336688893952e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2792232493407795e+01,     3.3019021118843320e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1091032577583348e+01,     8.2764336729808110e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6309870814766292e+01,     1.5648692484606341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5781875751527831e+01,     1.9313234209619732e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.4321965720768389e+01,     7.3733224730504817e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0223120019883522e+01,     1.4945316177598261e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0448275550284691e+00,     8.3809491764694412e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7088029799940085e+01,    -3.3088781972005381e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8637305772217946e+01,     9.1318006456507483e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6732397631568588e+01,     2.3951075518704911e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0355173323191676e+02,     8.3287894762989654e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0369304761702463e+00,     9.6654886881445200e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0444176443430250e+01,    -1.4308928027484455e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.3227228612694995e+00,     4.6868894794509271e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.9416309277873864e+01,     8.3285933731512869e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4866706077692493e+01,    -5.1887086195407903e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.9095910596572114e+00,     8.1670429858629632e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.5417903725976236e+00,     1.3500097180706898e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6897940007413609e+01,     4.4007829276349858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5863865827400716e+01,     1.4914118072661968e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7800252830361494e+00,     6.5644253724354376e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3480917592701978e+01,     1.8058614462682315e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2504542853596128e+00,     7.8070333795688421e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2272378394056016e+01,     8.3220800251718338e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7464279036294803e+00,     6.0626633160081496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2370709488224756e+01,     8.1889718027997787e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.8503526113552713e+00,     1.4913735359671540e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3993003897014264e+01,     1.4836894444126290e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4340312677911484e+00,     1.7576803832775727e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    9.4936036975013440e+00,     8.9040299125594775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9052222600459388e+01,     2.1479393761841493e+01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.7178648916062169e+00,     8.0934048093725705e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0827676964412724e+01,     1.0515964977343167e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.1770856539873364e+01,    -9.8051573838887407e-02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0581800010876893e+01,     2.5756368470045716e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.6159742834619230e+01,     7.2091318333029121e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0865741798262697e+01,     1.6026774064564976e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.5677150692677152e+01,     3.2199784823676138e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2713424550277539e+00,    -3.6278588994484662e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.9035020254152130e+01,     3.2200489455592284e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.2245520956084697e+00,     1.5559674970992739e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6986595603204080e+01,     2.3793893453896959e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.0143744720408971e+00,     9.0371811030989775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.8481915783122833e+00,     9.3734302059714482e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4924359500040058e+01,     1.6097767369977373e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.3854009904989084e+00,    -1.2128443365819267e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.6596130604284189e+00,    -8.4684110466504735e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.4616414562637345e+00,     6.0768902253456770e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8632514390135590e+01,     8.3276277633001773e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6485316187029689e+01,     8.3297550861500749e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9586698246380472e+01,     2.0269093659614227e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0098834754275705e+01,     7.6136427907515472e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0179618129223513e+01,     2.2599082801302360e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.3196707296421263e+01,    -1.1282625465931915e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4509339401312889e+01,     1.6692719920385785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2540151854608593e+01,     1.7913998019541337e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.7353998803051702e+00,    -7.9694518825351033e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6062038878559942e+01,     1.5536195334584230e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    8.0295935005311669e+00,     1.6204843302732466e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.4600770096171596e+00,    -4.5253954671778592e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.4672150659042298e+01,     8.8341401345395061e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1160211190832811e+01,     1.5529636378984504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.3815459899635155e+01,     3.1085156264409841e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2263574686026200e+01,    -6.9300896438092468e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.4827752688648729e+01,     3.1085860896325990e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.2263438125008257e+01,     2.5898355080870508e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.8147130117316067e+01,     4.3670366489587061e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.3207300024253922e+01,     1.2098536934998385e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7317073594615195e+01,     2.4499753299954627e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1705708532834116e+01,    -6.9587044005147186e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.3916904798326669e+01,     8.4019859299772543e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7864976387370859e+01,     1.9798920774677968e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.0462677026876586e+00,    -9.5689420492586169e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0148676956753064e+01,     2.2128509570597249e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7793691850238634e+01,     1.8173599001307011e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.4470708985001046e+00,     6.0697767706769139e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6718534111750863e+01,     7.1618051137145176e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5667005965431002e+00,     6.2688489411179046e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1253035351445709e+01,    -8.0470041435472267e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0021351279591560e+01,     2.4047213753236840e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.9119884969759473e+00,    -2.0764385369800257e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.0152290851119705e+00,    -7.3596329400876584e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.9772251823382362e+01,     8.5378885745977732e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.4714416731024286e+01,     8.5385932065139223e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    8.7913639767479879e+00,     7.2663206172031478e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1927584886743755e+01,    -7.5583972288547902e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0280848123111532e+01,     7.0382948271310752e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -7.6528602779586848e+01,     8.4464051619404363e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7055550140599465e+01,     1.5622936383512266e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1964369261794662e+01,     2.5195271424048027e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    5.2611883784364963e+01,     2.5195976055964173e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.1219626372846482e+01,    -8.8150123989458771e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.9613864495272235e+00,     4.7320306302129484e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7501085129930438e+01,     8.8600962092079899e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.6545816983157557e+01,     1.0575406606015249e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6542113432752920e+01,     7.6831108429127271e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8892149103380325e+00,    -1.1328242739128049e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -5.0837569218852430e-01,     1.4772460094313623e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.6731073836919549e+01,     5.4328652340028515e+04,    -4.3587731535886102e-05,     4.4163549593076712e-07,    -2.0580874058149021e-09, } , 
      {    3.4418644946415526e+01,     4.5371658960054905e+04,    -1.9323660068432583e-04,     1.9060798938671805e-07,    -8.1730859988807793e-10, } , 
      {    2.8835246357078077e+01,     3.8299571445826747e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.6479691784826176e+01,     2.9970880021101621e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.4081011530826398e+01,     2.2270490551624978e+04,    -6.5816649708709605e-04,    -5.5877617756045325e-07,     2.5783894483816563e-09, } , 
      {    2.6735246507364995e+01,     3.1156496255983257e+04,    -3.2044693519270919e-04,    -8.9927410969854286e-08,     3.6007596909686327e-10, } , 
      {    2.8732820447809335e+01,     4.6486287519321202e+04,     1.7757702182175162e-04,     6.2748329370814303e-07,    -2.8812574031075943e-09, } , 
      {    2.6420391557305315e+01,     3.7529294139347599e+04,     2.7928152673311877e-05,     3.7645578716409399e-07,    -1.6404785971807700e-09, } , 
      {    2.0707615922724230e+01,     3.2342112490864882e+04,    -1.3770400543257894e-04,     1.2912663330502091e-07,    -7.2633816076468471e-10, } , 
      {    1.7463597359865695e+01,     4.0528828564359341e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.6066583595943200e+01,     2.1358237893915157e+04,     9.6780155389601338e-05,     3.1189247051158353e-07,    -1.2773095167984278e-09, } , 
      {    7.9120020256144574e+00,     9.0989687312042788e+03,    -2.2649251634345474e-04,    -1.8461501354905066e-07,     7.1429054078911247e-10, } , 
      {    1.0412092852414240e+01,    -3.2090838908424621e+03,     6.9846113169404574e-05,     7.1997643564116527e-08,    -1.8584362837971840e-10, } , 
      {    6.0707562663886456e+00,    -1.8139181901301113e+03,    -1.8007904826722169e-04,    -2.1967043638903787e-07,     1.1408538581133378e-09, } , 
      {    1.3420941178126796e+01,     4.2481308544704836e+04,     3.2623178051707622e-04,     7.4194984003556522e-07,    -3.5730235002723579e-09, } , 
      {    8.6667063518751171e+00,     2.6452227650503064e+04,     1.8942397741471238e-05,     2.4174400328995500e-07,    -1.2547850731740787e-09, } , 
      {    2.5286704689464816e+00,     1.6691139896587698e+04,     2.7379619930094113e-04,     4.6385518567517073e-07,    -2.1988239693041891e-09, } , 
      {   -6.8935478200611193e+00,    -1.6623625802312807e+04,    -3.3545480326445187e-04,    -5.3778927204437500e-07,     2.5465323413959487e-09, } , 
      {    5.6696499528332067e+00,     5.8154836037311970e+03,     1.6296827661298229e-04,     2.4794554597323575e-07,    -9.6858016466787519e-10, } , 
      {   -1.7250272699151292e-01,     2.5132078209939291e+03,    -1.0655525971634047e-05,     2.4655684566506563e-09,    -2.1775891300715919e-10, } , 
      {    6.8948358435506787e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5047118577258264e+01,     7.7648181894922518e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.9521807991523310e+01,     2.1575502683377636e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5438673164898354e+01,     1.6755598626765885e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.1208842759243591e+00,     9.2724677184515331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8960549719144097e+00,     8.4269072020407584e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4475220126309466e+01,     8.2173577362100368e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2504633853283790e+01,     7.5295315741396971e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6985079467993422e+00,    -8.4400251132402154e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.3125681747878231e-02,     6.2830195524848227e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {   -2.0887346525356048e+01,    -2.3011998397341813e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1226097406195763e+01,     6.5857609102103506e+03,    -2.1583699037182070e-04,    -1.8708058200570130e-07,     9.3204945379627173e-10, } , 
      {    1.3581651978447663e+01,     1.4914452334935477e+04,    -6.3524239730472436e-05,     6.3330532424185123e-08,    -2.5428962387876272e-10, } , 
      {    5.5402729075895722e+00,     7.7003894694766441e+03,     1.5497663213425675e-04,     2.4979472231572374e-07,    -1.1318993494232446e-09, } , 
      {    2.3124288905040222e+00,     8.9569933799736082e+03,     1.4964886914843974e-04,     2.5102750654404910e-07,    -1.2407788059268243e-09, } , 
      {   -2.3843179005341472e+01,     3.2863247644877447e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9982533891102134e+00,     7.8423648207073147e+03,    -2.2116475335763771e-04,    -1.8584779777737599e-07,     8.2316999729269210e-10, } , 
      {    1.0353807961362115e+01,     1.6171056245432441e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {   -1.8531870925950511e+01,     8.3263902248853919e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.2667748274991112e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2969337933969118e+00,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    1.3624777660195543e+01,     1.4286150379686995e+04,    -6.0860358237563932e-05,     6.2714140310022456e-08,    -1.9984989562697296e-10, } , 
      {    1.5937206550699564e+01,     2.3243143759660601e+04,     8.8788510910875812e-05,     3.1374164685407152e-07,    -1.4406287015537972e-09, } , 
      {    1.1269223087943642e+01,     5.9574589549618686e+03,    -2.1317310887891217e-04,    -1.8769697411986398e-07,     9.8648918204806139e-10, } , 
      {   -8.6251363495756461e-02,     1.2566039104969645e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {   -7.5183378010077462e-03,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7272239329825055e+01,     8.3283391087670516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5124859989250226e+01,     8.3290437406832007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6725289607955915e+01,    -3.5231595807488247e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8958274798414720e+00,     1.6029080894201770e+04,     3.0728938277560503e-04,     5.0020583674561016e-07,    -2.3182384270982792e-09, } , 
      {    8.8705428339438619e+00,    -1.7429305145147757e+03,    -3.6814974101316892e-04,    -4.3749169643558775e-07,     2.1183885314713062e-09, } , 
      {    4.6679834627559220e+00,     1.7285684804698736e+04,     3.0196161978978796e-04,     5.0143862097393552e-07,    -2.4271178836018586e-09, } , 
      {   -7.3304594446843780e+01,     1.5542028288429719e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6003052013880108e+00,     2.6298323161836379e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0633036388273506e+01,    -8.8601554394617779e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.9551277073623350e+00,     8.4706667759557968e+03,    -2.2382863485054624e-04,    -1.8523140566321331e-07,     7.6873026904090223e-10, } , 
      {    5.5833985893374507e+00,     7.0720875142281620e+03,     1.5764051362716528e-04,     2.4917833020156111e-07,    -1.0774596211714549e-09, } , 
      {   -1.2845890774370776e+01,     7.2117616656190985e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0490413801463124e+01,     1.5540453090344225e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.0709052485938940e+01,     1.5545055490023693e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8353423150710796e+01,     7.2163640652985669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1623031049305755e+01,     2.2128515200394308e+04,    -2.8202511159520165e-04,    -1.2313365746735350e-07,     6.2332010166571914e-10, } , 
      {    1.0396933643109993e+01,     1.5542754290183959e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3777449934184999e+01,    -1.5419111039189374e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7782407126128339e+00,     8.3294027701920022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0742153392851987e+00,     8.3279800792582500e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2079770335598713e+01,     8.3117707439163460e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2339010563664232e+01,     1.0958809679741535e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0772090648982459e+02,    -1.7309684784268029e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9267476477053854e+01,     1.3799823775669183e+04,    -4.3433786223654987e-04,    -3.7354477189723997e-07,     1.8096591793407536e-09, } , 
      {    1.2709362533614016e+01,     2.4499747670157569e+04,     8.3460747925058777e-05,     3.1497443108239688e-07,    -1.5495081580573769e-09, } , 
      {    8.0413790708580919e+00,     7.2140628654588327e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7842914704481068e+00,    -4.8632660401781118e+02,    -3.7347750399898598e-04,    -4.3625891220726239e-07,     2.0095090749677269e-09, } , 
      {    1.0325841488918483e+01,    -1.9524799803454973e+03,     6.4518350183587553e-05,     7.3230427792441848e-08,    -2.9472308488329798e-10, } , 
      {    2.2693032087561438e+00,     9.5852953352220902e+03,     1.4698498765553121e-04,     2.5164389865821173e-07,    -1.2952185341786140e-09, } , 
      {    3.0616615854986859e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9340097323944949e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1487508053129559e+01,     8.3290200572015747e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.1865934134705718e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9305333009146430e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7238813307774272e+00,    -1.6920680808781185e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4504454626108446e+02,     1.6219054166649585e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5036477271174910e+01,    -4.9310321559205477e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3136697609779334e+01,     8.3155735135256709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8331146950567199e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9982879647966740e+00,     8.3260615924089434e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8541483237646688e+00,     8.3313212570413107e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.4468921743503046e+01,    -2.4272873609482303e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6176316351107506e+01,     1.6655081649610518e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.5098899239488404e+01,     8.3131493964366964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.0963336593583264e+01,     8.3442334530135558e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5023303353603282e+01,     1.6659684049289986e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3978585621557656e+01,     3.0457206625119434e+04,    -1.2971236095385340e-04,     1.2727745696253293e-07,    -5.6301897600931531e-10, } , 
      {    1.5980332232447443e+01,     2.2614841804412121e+04,     9.1452392403784316e-05,     3.1312525473990889e-07,    -1.3861889733020075e-09, } , 
      {    7.0979974718329339e+00,    -6.9587606984853278e+02,     5.9190587197770524e-05,     7.4463212020767181e-08,    -4.0360254138687761e-10, } , 
      {    1.0780824816085845e+01,    -1.3117911199455193e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5445847309379683e+00,     7.2114330331426490e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1676785857687307e+01,     7.2166926977750172e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1255573623735209e+01,     1.5732484147591094e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.0743889082633075e+01,     7.1985208371704030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5024461828749631e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.5238176097063103e+01,     7.2296048937472624e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7872846219260019e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7587381161727322e+01,    -1.8048943355373922e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1585172682655003e+00,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5818781561176714e+00,     8.3640284956606611e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.2139979437293889e+01,     5.5356930186341742e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4869068272235396e+01,     7.2144151814169072e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6232979991073258e+01,     8.3654511865944132e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9367308390794868e+01,     7.2137105495007590e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.3583805415881500e+01,    -2.5069150669460214e+00,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3360914568395357e+00,    -6.5589555997275966e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.7480701755708552e+01,     1.6657735165408325e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.4916884436709836e+01,     1.6657030533492180e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.9262946039012601e+00,     1.5540124457867776e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2765871510760661e+01,     1.5543106606142033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.8835773917650293e+01,     1.5542401974225884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4005053413614057e+01,     1.5545384122500143e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.7256733353348714e+01,     9.5171832084254220e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0773130053229846e+01,     1.6864631115441240e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.0426343685877999e+01,     1.6869233515120708e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    9.3106416699676998e+01,     1.5527212261895529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.2891077752205462e+01,     1.5558296318472387e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0780734951353246e+00,     7.2133515199919557e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4145131428225724e+01,     7.2147742109257097e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.6088107368655869e+01,     1.3498673398216670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0948703408193062e+01,     6.2133149972934916e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5530275639472575e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9027154513696903e+01,     8.3114184279582696e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5972461609454385e+00,     7.2309835462676137e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0484668005487407e+01,    -1.1944470569373675e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.1348852067384847e+00,     2.3869144515069351e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.3064780440598383e+01,     2.3873746914748819e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3738565094474517e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0536335047781535e+02,     8.3269604562466993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6909180392401240e+00,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.2401282773139783e+00,     3.1546870868086353e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4916621969201294e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0126155950650435e+01,     8.3110593984494699e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3446245362231419e+01,     1.5543082922660407e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7381705302041198e+01,     1.5542425657707512e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9063710549729865e+02,    -5.3779714259599132e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5026268785518880e+01,     7.2137342329823841e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.4001991167166771e+01,     9.6640244767877448e-01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.5802035723866588e+01,     7.2143914979352812e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0762071547592740e+00,     7.0563285111184005e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0457280256411622e+02,     2.0355170170754775e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3845174060663914e+00,     8.1709570703846939e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1942899628883707e+01,     7.1365106044467084e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0117449632311974e+01,     1.5275584790397261e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.4653923920762049e+00,     8.4864257790655574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.1383673628409269e+01,    -1.7272996766856068e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4531460744672964e+01,     1.2619238322379260e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.7278886509186968e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.9493203338811071e+01,    -2.0571522720911162e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.8276963895746899e+01,     8.3198312692856634e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.2988073040250701e+01,    -8.3375515801645888e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.2585307233728118e+00,     8.2757223275139349e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0936677732354472e+00,     1.5999508324485532e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.4007791948778859e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1493057239361519e+01,     6.0334743949479162e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2942265855080612e+01,     7.1610937682476415e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2932835120198742e+00,     8.2511391637130018e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8739110125920417e+00,     8.3816605219363173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7698248612814691e+00,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7472653185558249e+00,     6.6633075414254135e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3238828540948493e+00,     7.2916151264709570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8188910141668263e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8505708149538531e+00,     8.4062436857372504e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0752661159073320e+01,     1.4916747904978150e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6941490372416588e+01,     8.2490616209293021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4296504242354588e+01,     6.5864666258801662e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5037746427139309e+01,     8.3647398411275371e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4796074308060632e+01,     8.4083212285209502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5460621385985647e+00,     7.2271807766582879e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0430580330776476e+01,     1.5385019935843527e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3877517116543494e+00,     6.6916934748356716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1951897312667867e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5725259765576380e+00,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1654152726542662e+01,    -1.5925960759164900e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1121406159011329e+01,     7.3717972197992640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.2129676617286762e+01,     8.3044185511156447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2543287285688439e+01,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2540160772075975e+01,     7.2670319626700239e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1231449117455398e+01,     7.1344330616630086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7023031708247089e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4296851347647154e+01,     1.5465202029171833e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6266578053039268e+01,     8.3529642983346093e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6730473134618698e+01,     1.4912145505298682e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0209354576918823e+01,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1097147640361413e+00,     1.4991998966150539e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.8351439785839485e+00,     8.2888765228272132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.9029993401688671e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3687643750257047e+01,     8.3685063266230391e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6112276670986239e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7141863877746083e+01,     8.1578391591852396e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.1396034451589578e-01,    -1.4226909337526763e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8546679075926125e+01,     1.4914094389180342e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.7536191693313956e+00,    -1.7632026275657509e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0271956664326014e+00,     7.7779361006917070e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0607958153106441e+01,     6.5854029644552147e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6683171241701178e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6631585675197435e+01,     1.4915158050605292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3310973860375523e+01,     8.3124820893832220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0467124877887940e+01,     7.2936926692546567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6164266445918916e+00,     5.2257751744315044e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0744033942512974e+01,     1.5020384899560799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.6394052015654623e+01,     7.1897899918493513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7099514770404913e+00,     7.8063220341019651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5296049507029890e+01,     1.5489785192972768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2059531116382693e-01,     7.1742479635609207e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6804391704240178e+00,     1.5620306551196083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0190629566051109e+02,     7.2383357390683150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6164120822341115e+00,     8.0182093328307928e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9359599543763501e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3731546468721350e+00,    -5.3680442678067706e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4691340045547690e+01,     8.3512333298561825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6140014211242775e+00,     2.1329907977851814e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.5470504853287892e+01,    -2.2979975511290831e+01,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0882591308776988e+01,    -1.1489987755645416e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6131682967045982e+01,    -8.3441105357643155e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9342014209551845e+01,     7.2538777673567447e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4284545722482250e+01,    -3.1480227663230835e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.5494181146583674e+00,     1.6656671503983376e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.0142508972724649e+00,     1.6658094194917128e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.8396624928610443e+01,     6.5880564802530243e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.5365674910311284e-02,     5.2165804515628406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1330432545161163e+02,    -3.4560240734833769e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6755196827968575e+01,     7.7000315237215082e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0961810121657308e+01,    -1.6209335341904819e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.5989437156845554e+00,     7.7010951851464597e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3577203555276859e+02,    -4.5036753191467938e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0529044875435915e+01,     4.2923307555598194e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.8786386675672944e+00,     1.5463124486388133e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8539927704248660e+00,     8.8794467487585462e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3491831987739191e+00,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9085964559393439e+01,     6.5834540805735560e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5423427125971530e+01,     8.5508601288782265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.0204144505812465e+01,     8.2716035260603567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3930467905397052e+01,     8.2733344945387835e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3483989786035735e+01,     1.5700488644524392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.9795779761655730e+01,     8.3840483549114670e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4951132609273952e+00,     8.9562876643037926e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4952628803005874e+01,     5.5228953248342805e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6695867705238513e+01,     2.3871093398951012e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.5340563392063089e+01,     2.3871798030867158e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.9983455684886888e+01,     8.3297873056931003e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9582035685833588e+01,     7.1587059352724909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2520724845284771e+01,    -2.3011998397341813e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8797258594149335e+00,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9519489657493914e+00,     8.3997091105030413e+03,    -3.3191992975274604e-05,     3.2017095500473753e-08,    -1.5363745554361197e-10, } , 
      {    1.1780914952228969e+01,     8.3996847301375219e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8850534222772854e+01,     8.4357673574802211e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.2453131171816981e+01,     8.4311649578007527e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6867902155503947e+00,    -7.1017685777915176e+01,     1.8550474361662286e-04,     2.1839401892941267e-07,    -1.0327016221314225e-09, } , 
      {    9.3075031889354403e+00,     1.6728400535228167e+04,     1.1912075766607364e-04,     2.8242820993036017e-07,    -1.3399765332186464e-09, } , 
      {    2.5870442129654974e+00,     8.4672232047779326e+03,    -9.4850596938785377e-05,    -4.1916990868730478e-08,     1.9407091654814723e-10, } , 
      {    9.7281698100072624e+00,     7.1430451796809175e+03,    -3.2996128248106349e-05,     3.1929829037874036e-08,    -1.5509189658694057e-10, } , 
      {    6.2832123464223244e+00,     7.0993305412396467e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.0097575922983641e+01,     1.6759856382525879e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.1206095677765202e+01,     1.6764458782205347e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4137402715096506e+01,     1.6728376154862650e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.4993328385673333e+01,     1.5613771975961874e+04,    -2.5169286484000379e-04,    -1.5444709439106490e-07,     7.2397227000086990e-10, } , 
      {   -2.8512053011868517e+00,    -8.3659520633655957e+03,     2.3626909935192144e-06,    -6.8984138685075869e-08,     3.2749164158949159e-10, } , 
      {    7.3726154122882868e+00,    -1.1856462450442086e+03,    -1.8530887888945459e-04,    -2.1848128539201239e-07,     1.0312471810880938e-09, } , 
      {    1.8881405644549965e+01,     8.4460853959628657e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.5074649751177303e+00,     8.2937465357151395e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1864942943583186e+01,    -6.8726316512950604e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5120795976607232e+01,     1.6798205998768022e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.9214575034271366e+00,     7.1430695600464360e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0463602994667053e+01,     8.4394996320354348e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7348882434326459e+01,     2.3942463400687000e+04,    -9.9380114198655565e-05,     9.5964020038821550e-08,    -4.6236680767416450e-10, } , 
      {    1.1759221987332728e+01,     8.4798668234658326e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.8706709368067038e+00,     8.4526538273487131e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3037835792708218e+01,     8.2883510416224344e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.9425947709601186e+00,     1.6795914629503059e+04,     5.7462153702562864e-05,     2.0849412356115595e-07,    -9.9226816112688717e-10, } , 
      {    1.0471060195576436e+01,     8.7151534388183863e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0178196213282353e+01,     8.7927056998305106e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8974079311266156e+01,     8.4337947901169355e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.1654668526170315e+01,     8.4331375251640384e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0725198485657222e+01,     6.5339496447091196e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3822902313759222e+01,    -4.0340383102691931e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.5312008287415562e+01,     7.1069869327037377e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.5991657413027497e+01,     7.1115893323832061e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.7048155496335834e+01,     7.1089358165853982e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.4011046633074670e+01,     7.1096404485015464e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7078628557417399e+01,     1.6826591932178282e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1663057237588566e+01,     2.5057091959953294e+04,     2.7143350830742188e-04,     5.3283932436024664e-07,    -2.5263156108936811e-09, } , 
      {    1.7601992537598946e+01,     8.6813720113154232e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1553157545263446e+01,     1.6745236901556829e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.0590171850446986e+00,     6.3232461931574962e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.2972040820872479e+00,    -1.2169541372414199e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1077540440113518e+01,     8.2599651082121763e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.1930974459924508e+00,     8.3570249141818185e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.3466497731309151e+00,     6.7162671628504850e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7897393259097864e+01,     7.8953300719294457e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.9529858244903533e+00,     8.3989733846706458e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.4411746625769357e+01,     1.5645227823259585e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.6891914316838751e+01,     1.5649830222939054e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.9821792401263437e+01,     1.5613747595596353e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.0423461840656962e+00,     8.2576737389472109e+03,     3.3781749425797111e-04,     4.6880513335929910e-07,    -2.2190406998064567e-09, } , 
      {    1.0521187142109419e+01,     8.4210146381153736e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0768606763852038e+01,     7.9291114994324107e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2083723858660388e+01,     1.5471736604406044e+04,     1.1931662239324191e-04,     2.8234094346776045e-07,    -1.3414309742619751e-09, } , 
      {   -1.2056196732073740e+01,     2.3973919247984712e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.1600323763973229e+01,     9.1857266695234975e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.9247469076666590e+01,     2.3978521647664184e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.5376268071634573e+00,    -1.1856218646786899e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2244970457140962e+01,     8.4023145624537065e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.9301641675239036e+00,     7.6846266203778241e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.7712954917551400e+00,     6.5147432244324355e+03,    -3.0332246755197832e-05,     3.1313436923711370e-08,    -1.0065216833515075e-10, } , 
      {    1.2728282588879766e+01,     5.9586111568747674e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2187135911104590e+01,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1891777790996530e+01,     9.1081744085113751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2178412257082233e+01,     2.3942439020321479e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -3.1863748682922907e+01,     1.6761805266407540e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.9255455878188258e+01,     1.6762509898323686e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2364416174087850e+01,     6.9515537782030397e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0951484876261091e+01,     7.0755310854060262e+03,     2.8662475715404431e-05,     1.0586391540707827e-07,    -5.0280026867869976e-10, } , 
      {    1.3023024304588173e+01,     9.5787476392164845e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1289623087957722e+01,     8.5574190844779532e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.3520466587259197e+00,    -8.4672475851434519e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.5407350716655953e+01,     7.1622572297546631e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1626513844739764e+01,     1.5072190535264968e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.4548630885284538e+01,     1.0512704887343568e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.6579158268890083e+01,     1.0442241695728592e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2296686607663586e+01,     6.5869187419203108e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.3476184775417011e+00,     7.5360905297394238e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1397088463761948e+01,     7.0563190353322807e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.5020043727598988e+00,     6.7500485903534491e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.7035037385440237e+00,     7.0694732306455599e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.0409006813117667e+00,     8.3467156329263307e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3295593215625487e+01,     7.1491030344413839e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0835121585409247e+01,     6.5207954493958405e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1439229044795493e+01,     7.1066583002272882e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.7742021124616546e+01,     2.5088547807251009e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    4.6856224781695843e+01,     8.4338184735985615e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.4175632286222410e+01,     8.4331138416824124e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5522920905108684e+01,     7.1119179648596564e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.0661668016887784e+00,     2.5266641425784015e+04,    -1.6123458288933460e-04,     2.2117200132217029e-08,    -1.1320399453907671e-10, } , 
      {    6.1922917077602389e+00,     8.8264871273334757e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.3561652720464593e+01,     2.5093150206930473e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3980966377501767e+01,     2.3400881959990096e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3333823915192054e+01,     7.5023091022364588e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9025203861981360e+01,     9.0743929810084082e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.9414163609370654e+00,    -8.4503868384492671e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    8.3707113456421904e+00,    -1.7947417800160060e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.8175104673440217e+00,     1.5611480606696909e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.0475690491405844e+01,     8.0066637604445350e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.6139095217163174e+01,     1.5647176707141245e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.4902470680009770e+01,     1.5647881339057394e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.8437825914266046e+00,     1.6369136612672624e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.9652266407579368e+00,     8.3598698282396108e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4883166376145720e+00,     8.0404451879475000e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.9600584745735610e+00,     1.4861487083713346e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0957023571851707e+01,     6.5585328085100527e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4446084094860154e+01,     9.9717686089094732e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0170617127010146e+00,    -9.5143376697693348e+03,    -3.3762162953080284e-04,    -4.6889239982189881e-07,     2.2175862587631283e-09, } , 
      {    5.9494073924247468e+00,     1.6186819094531262e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.8630518765911761e+00,     1.2554517085840644e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2314648573565158e+01,     2.3190178508438468e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4573645158434054e+01,     7.2670224868839032e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.6229999647599609e+00,     8.4695145740428979e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    8.5479797502349548e+00,     1.7155178552058602e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.9395038169868645e+01,    -1.2197556081397292e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.1646787378077207e+01,    -1.2190509762235793e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.0892635519515892e+00,     8.2407774385039484e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8043625352394019e+00,     9.0088046454748383e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2514617495774790e+01,     1.6433570962485297e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.0822902647841586e+00,     6.3570276206604613e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8368411887345868e+00,     4.9779570260834839e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.1970314991750683e+00,     8.4165454768317068e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.2826636956382103e+01,     7.7713471349293995e+03,    -3.5660009741014867e-05,     3.2546221152036703e-08,    -2.0953162483873039e-10, } , 
      {    6.7043574171491436e+00,    -2.0054452315676294e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.3397526929444766e+01,     8.3200549263416979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7476416938612033e+01,     2.5126897423493148e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.2468989590649695e+00,     1.5254508053406331e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1446270514487436e+01,     8.4772369911496462e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2311711967613142e+01,     8.4383971897964075e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3312436923204272e+01,     9.5011953782043602e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5914306779538158e+01,     8.4285351254845664e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.4775844240542251e-01,     6.8011322074971258e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.9704436482979585e+01,     3.2271154825412126e+04,     5.2932636442692690e-05,     3.4637513446870794e-07,    -1.6487058853491990e-09, } , 
      {    1.2983976634345016e+01,     2.4009977494961891e+04,    -1.6103871816216633e-04,     2.2029933669617312e-08,    -1.1465843558240531e-10, } , 
      {    4.1325738257188060e+00,    -2.8824623677762668e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0628426949015019e+01,     1.5681286070236765e+04,    -3.1335146880351458e-04,    -2.2838118076026912e-07,     1.0716806420926291e-09, } , 
      {   -2.3792074637310119e+01,     2.3975868131866373e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.7266586330028474e+01,     2.3976572763782518e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.1568874477075726e+00,     1.7337496070199963e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    9.0484267835653345e+00,     8.3783548221596702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8547955940445719e+00,     1.5583094673286651e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6493828453631643e+01,     2.5057067579587776e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.5372910321389135e+01,     8.8353035960465277e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2407777797208843e+00,    -4.7056375491899104e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0662521454496119e+01,     8.2069960110009852e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.2588222525745891e+00,    -4.3336135279567930e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.2349806384228621e+01,     8.4154172142851130e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.5605621271249497e+01,    -8.4515151009958608e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0602452025103949e+01,     2.3583199478131457e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5868703376320667e+01,     1.0364789578602460e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.3809166910913113e+00,     1.5664449703908100e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.9910964886753195e+00,    -1.6124242618746418e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4665203894680902e+01,     7.6316575231666329e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.5928689328965686e+00,     7.8581276698061347e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.5425674802547720e+00,     1.2838376419943231e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.6459414010560653e+00,    -3.7260638640469551e+01,     1.5467544163486746e-04,     1.8142697574481057e-07,    -8.5884743608554286e-10, } , 
      {    4.7106104841076260e+00,     1.6937950001058889e+04,    -3.1354733353068285e-04,    -2.2829391429766940e-07,     1.0731350831359578e-09, } , 
      {    4.4890940164546391e+00,     7.2206218210585603e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5578297532889495e+01,     7.0317358715313476e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.2067596989058273e+00,    -1.6694643488090722e+04,    -1.4995005964782903e-04,    -3.1939525311496228e-07,     1.5138307192645260e-09, } , 
      {   -1.1898329306411934e+00,     1.6722342680467700e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0447309405768351e+01,     9.4674139507013970e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8584723415579600e+01,     1.6801972484263526e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.7616584639989931e+01,     1.6749039671166160e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.2082507976674854e+00,     6.8276008513655725e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1237165370607748e+01,     1.6774776820687992e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.1614589785300389e-01,     7.6977808156911033e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.2995360199655988e+00,     1.2024826113728732e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.0315805678894252e+00,     7.2490077544688183e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5983099698168010e+01,     2.2825543472155743e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.6599378139312573e+00,     5.9640066509674743e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4360531218354707e+01,    -1.1789264411481599e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3629600230749469e+01,     1.4496852047430615e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.1192127988501683e+00,     1.0406338744848446e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.8503111322025747e+00,     1.0548607838223714e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3890055389453689e+01,     1.4458963481327305e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -5.9231883182810119e+00,     6.4959831017448550e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.2981481214815425e+00,     2.5124606054228185e+04,     2.0977490434391112e-04,     4.5890523799104237e-07,    -2.1786072388019214e-09, } , 
      {    7.5362034009916119e+00,     8.6349713454900793e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5036454067421211e+01,     1.4985470020713392e+04,    -2.4902898334709531e-04,    -1.5506348650522755e-07,     7.7841199825265977e-10, } , 
      {    1.8501714461522390e+01,     1.7219612901871267e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0585651614937735e+01,     1.6762868927832489e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.8059880347665356e+00,     1.6761446236898737e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1427480734109453e+01,     7.1868403935555962e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.4010170124180689e+01,     1.6761828949889165e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.6618532722948775e+01,     1.6762486214842062e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.6041433768490581e+01,     8.4732810595383999e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.4542363988892584e+00,     8.5912005119809182e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.3578788201964276e+00,     8.3936512557425740e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0299199796132183e+02,     8.4329581991024534e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9973618747179265e+02,    -8.4339741161785223e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0013442583214882e+00,    -1.6795939009868576e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.7291313662964050e+01,     1.0757810548295447e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.2247843281491644e+00,     8.2757318033000556e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4732635307465296e+01,     9.8942163478973489e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0373870896854809e+01,    -2.3700802678503578e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.4157410940361643e+00,    -1.8139482002926907e+03,    -1.8264499739654609e-04,    -2.1909767750617504e-07,     1.0856869093398837e-09, } , 
      {    1.3479590100952146e+01,     8.0868458537728438e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.8630115916964609e+00,     1.6622437960440267e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.2819183697596067e+01,     1.6768191056760559e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.7769549055398279e+01,     1.4357108045139750e+04,    -2.5149700011283557e-04,    -1.5453436085366459e-07,     7.2251782895754135e-10, } , 
      {    3.9011958187186941e+00,    -8.2576981193127303e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    7.8529541428945606e+00,     8.4313239193708614e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7392008116074336e+01,     2.3314161445438520e+04,    -9.6716232705747033e-05,     9.5347627924658870e-08,    -4.0792707942237469e-10, } , 
      {   -1.2956380260667078e+01,     1.5435678357428864e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    8.2935124144164138e+00,    -3.4944889009986269e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.8784577769989221e+00,    -8.6750524714155702e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8347130398400822e+01,     1.5440280757108332e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4058614700458374e+01,     1.6755598626765885e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.0492259974231857e+00,     7.6136427907515472e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1134296061201434e+01,     1.6780181790566818e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.3978730290566155e+00,     8.4360959899566715e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.2826620895523284e+00,     8.4308363253243042e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -7.2550294299600147e+00,     7.0296583287476478e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3166149088863747e+01,     6.0115802540859595e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0687321344578784e+01,     7.1768022268183631e+03,    -6.3825430229861726e-05,    -5.0372141467280791e-09,     1.8762289458939002e-11, } , 
      {    7.6233822675234695e+00,     1.6604423228025182e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3908790563135980e+01,     2.5073928326281955e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3179298228262725e+01,     8.8728877931588177e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9433937313715401e+01,     2.5155283356903408e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.2018089025177453e+01,     6.5606103512937534e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.7005694687955533e+00,     3.2302610672709838e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.3261505731060506e+01,     1.5489794668758888e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.5250100538860451e+01,     7.1092948460941989e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.2212932200795535e+01,     7.1092814189927449e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0802887282574964e+01,    -1.1133368851508840e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.1602946884537360e+01,     3.2307213072389310e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.3506286961503946e+00,     1.6100098579979685e+04,     1.2178463915898214e-04,     2.8181181781619749e-07,    -1.2855368049668566e-09, } , 
      {    1.3196716976960333e+01,     1.5125159632476159e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.7660593359783725e+00,     1.6919891936706044e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.4723584478134573e+00,    -9.7238871356000564e+03,     9.5046461665953646e-05,     4.1829724406130761e-08,    -1.9552535759147583e-10, } , 
      {   -1.9572240810732985e+01,    -8.4177336734928977e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.2799479097929467e+00,     1.5471760984771563e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.2339490805543241e+00,     5.5709856812744865e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.6704399173525850e+00,     1.5647529023099320e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.9391918915584654e+01,    -8.2232827784304391e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.4251626722542966e+01,     7.1889179363392959e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1730820236809919e+01,     1.5059036339951688e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.7123212212366994e+00,     1.6759527750049427e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.0679253698495934e+01,     1.6764787414681799e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.4144207720900326e+00,    -1.2194032921816543e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7299160718311981e+00,    -6.9931964102639722e+02,     1.8816862510953136e-04,     2.1777762681524999e-07,    -9.7826189387963262e-10, } , 
      {   -1.6135918786019950e+01,     1.6784699487496669e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1226395803234819e+01,     1.6781345252073843e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.2726789603562274e+00,     1.5976115642979639e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.4766353245687132e+00,     9.2195080970264607e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4297055822001372e+00,    -7.7714071552545593e+03,     3.0528111482366093e-05,    -3.1400703386311086e-08,     9.9197727291822176e-11, } , 
      {    1.4115137210565448e+01,     1.6808558248190959e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.4314611365399783e+01,     8.0530644262698788e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1869832649857024e+01,     9.8604349203943857e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2210655832140821e+01,     1.5594559925888128e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.0505442037111630e+00,     2.6250908725579279e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0669872023549965e+01,     5.9056420596635762e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.9693436990995394e+00,     6.5147676047979539e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    1.8713876255104594e+01,     1.1150831517988436e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.8313035627959096e+00,     1.7548199521751591e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6485942116562374e+01,     1.5700498120310511e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5245632724042707e+01,     3.5268058659029566e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.9211444325550406e+00,    -5.2959621425070964e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.9727806088123323e+00,     5.1805635704169433e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.0154909606072779e+00,     1.5774383012655215e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.3142349273494567e-01,     1.3853178005280643e+02,    -2.4716334758013363e-04,    -2.9232810529861689e-07,     1.3804099942231817e-09, } , 
      {   -4.3419013682399576e-01,    -6.8126720647061416e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.6010192113310220e+00,     1.6239788191742453e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.5549630288924261e+01,     2.3453851057201286e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.6560690561591906e+00,     1.6709188485154420e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2326645841492319e+01,     1.5644899190783137e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3388323988829478e+01,     1.1739397123774006e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.7105743371446653e+00,     1.6815126679576806e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.9358830751479896e+00,     8.3356950062039668e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6436475777402023e+01,     1.5650158855415504e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7214299621921974e+01,    -8.4274326832455390e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.2686798225637204e+01,     8.9504400541709419e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0833046870752755e+01,     1.6222867510933671e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.6316454141046485e+01,     8.4491986417880798e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    5.6007063098118879e+00,     7.2827891819717825e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.9417909745301341e+00,    -1.7417726828048146e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.4090660928819972e+01,     2.3387727764676816e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1582384860146892e+01,     8.4979005074531560e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -2.4588671384454706e+01,     8.2802400244437849e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0007112127522078e+01,    -1.3771376465220862e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.7852571873923093e+00,     5.5728426947056687e+02,     1.8284086212371435e-04,     2.1901041104357535e-07,    -1.0871413503832124e-09, } , 
      {    5.9574504381505733e+00,     1.0740456523154535e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.0121219192615207e+00,     1.0214490059917716e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.9229714811216798e+00,     7.0628874667181281e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5393324703384792e+01,     1.6617042466347561e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.7520432966450553e+01,     6.8548566928675682e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.9792580465534702e+00,     8.4180470466012976e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.2235069743417666e+01,    -8.4488852686796763e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.6365815220298661e+00,     1.5607714121201407e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.3287588565529390e+01,     6.4806282315398539e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8474506593222813e+01,    -1.8078959753292852e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.0088231337145945e+00,     1.6171086255595021e+04,    -6.6286053586965088e-05,     6.3990557769647648e-08,    -3.0800213160888826e-10, } , 
      {    1.2544843744294811e+01,     1.5117539874070462e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1245450900965870e+00,     8.9081667230133598e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -5.1823256925290657e+01,     8.4212602328209014e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5620555917868106e+01,     2.3818486093484014e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.2011999853548367e+00,     7.0966688942210931e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.1227850281306946e+00,     1.7356702490476651e+04,     1.1645687617316513e-04,     2.8304460204452285e-07,    -1.3944162614704363e-09, } , 
      {    8.9652747428654234e+00,    -1.2723723893928457e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.6712954675091343e+00,    -6.2833196541106190e+02,     9.7932363584127605e-08,    -4.3633231299858394e-11,    -7.2722052166430139e-13, } , 
      {    1.6152310686898218e+01,     1.0287237317590334e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.8043880397760494e+00,     7.0901004628352448e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7077073358084558e+00,    -1.3855616041832513e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -2.4160900457599837e+01,     1.6744884585598757e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.7672348192019247e+00,     9.0280110657515233e+03,    -3.5855874468183115e-05,     3.2633487614636420e-08,    -2.0807718379540176e-10, } , 
      {    9.6563295091104511e+00,     1.2396240262358783e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.6395907036226829e+00,     7.2544032485615244e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.1074642553299121e+00,     5.6949386081119537e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.4481032148908142e+00,    -8.4400251132402154e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1856340157439799e+01,     1.5096773699065900e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3231352151182966e+01,    -1.1664341949704628e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.5893191302297671e+00,     5.9302252234645093e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0304598226037626e+01,     8.8391063656558545e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.2416220016713595e+00,    -1.0459022427533428e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.7962153189709760e+00,     6.4345798816725855e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4322098537042711e+00,     8.9558355482636471e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1249535608201079e+01,     1.1080820731030872e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.0844944914493047e+00,     6.2714412353852128e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.4145137453876785e+01,     1.5687343924997233e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.1666711897312293e+01,     1.0247353307562662e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1153109866956658e+01,     2.4762262387210423e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.7416766969136965e+01,    -1.2204772968221687e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2826386964449311e+01,     1.7043844863543512e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6020096606458921e+01,     7.6600434565768910e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0993621091467061e+01,     8.4614632765123479e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6033813931823278e+01,     1.6173664899217982e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.7234219831613817e+00,     8.4327548121736108e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0956999281807853e+01,     8.4341775031073630e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.0710123679853716e+00,    -1.2220331244978388e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.7997571868642730e+00,     1.7284526972988773e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3180583999719339e+01,    -1.2167734598654706e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.4698236078144202e+01,     1.5437627241310523e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    5.6372240729018372e+01,     1.5438331873226673e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0527985412186144e+01,     8.3232434866788553e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0125103278584330e+01,     2.2685799469864876e+04,    -9.9184249471487296e-05,     9.5876753576221833e-08,    -4.6382124871749311e-10, } , 
      {    1.3079480385260217e+01,     1.4862641069434247e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -9.9351556216161363e-01,     9.0219588407881183e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.1612789368518532e+00,     1.6211402258332193e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2533774960180779e+01,     1.7121397124555635e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.1194623309002889e-01,     5.1779647115814987e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8793329187586663e+00,    -4.1759465770779968e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4212853128941639e+01,     2.3425465123791026e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1703726533829784e+01,     8.4269072020407584e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3776022210150121e+00,    -1.2592181940795665e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.9611471355600738e+00,     2.3936405545926533e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.0136392449788183e+01,     1.1543852487681423e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2648826542930339e+01,    -8.3974177412380759e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.4649879757864669e+01,     2.3923251350613253e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1018447648466553e+01,     6.5023104554757801e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.8724510155614169e+00,     8.9842214816739070e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1896844474662815e+01,     2.5266617045418498e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.4959020368149977e+01,     8.7823344988353365e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9925279139699768e+01,     1.7612633871564256e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.4035234743051859e+01,     6.4813328634560030e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8669382384999903e+01,     2.4029189545035639e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.6004645406585603e+00,    -1.7701586162150734e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5625176254418454e+01,    -8.4208469193181081e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    7.5774478851232887e+00,    -6.4406480434730247e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.1734447102838148e+00,     7.8243462423031706e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4694680476971623e+01,     2.3973590615508263e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.8779143046252962e+01,     2.3978850280140628e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.3895771096066181e+00,     6.0879595778049415e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2003810772776786e+01,    -8.4465840688399421e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -3.3524068648397218e+00,     6.9326741606299174e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.6547866571120210e+01,     2.4016035349722359e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3186619227363726e+01,     2.4551558935658795e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.7855292017475115e+00,     7.1032546581485240e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1257391994242232e+01,     3.8646201409325977e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.2646701015090144e+01,     8.5008900366167236e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -4.3805910564296838e+01,     8.4610751347188125e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.6725689009224922e-01,     7.3637195722193756e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1486805413314649e+01,     8.5407049385146365e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3292321424584046e+01,     1.0253455890087373e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0890879721275766e+01,    -1.1795883902837420e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.7741475998199867e+00,     7.1092881325434719e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -4.9149438975085999e+00,     1.7350650265513243e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.5491157510120654e+01,     1.5805263377439753e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.5461945258065413e+00,     8.5009802519153782e+03,    -1.2567989892054077e-04,    -7.8884034053332593e-08,     3.6792510259402680e-10, } , 
      {    6.5182679850444325e+00,     6.5553005694877993e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.2439031429313889e+00,     7.9110967670173259e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.9922501222813089e+00,     7.6579659137931903e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.4398688344111346e+00,     1.7912834558034316e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.4139350386254957e+01,    -8.2216154919700311e+03,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -1.5751579933424075e+01,    -8.3713330076675538e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1312807633148815e+01,    -4.8371795023227014e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.2495780662658326e+01,     8.4955993076134218e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.8317946226689372e-02,     8.4332425005844525e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.3690851968922892e+00,     8.4336898146965214e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.6035433216432061e+01,     6.4832817473376635e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.5754394757118941e+00,    -8.4174666493160021e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4018611111708765e+01,     3.3385783384678420e+04,     4.2374625894877011e-04,     7.8325043879013306e-07,    -3.7126546885687153e-09, } , 
      {    1.9204670517549993e+00,     7.6474242182545113e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.1958817509970872e-01,     8.4494656659649718e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1229240289548478e+01,     1.7312912906399029e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.5891156270672461e+01,     8.5087172188128752e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.2288558568552226e+01,     8.3582150964680986e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3675054658257316e+01,     7.8449734744928555e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.2261859292710930e+01,     1.5474027973671009e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -3.0831380993769528e+00,    -8.2239166918097653e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.8203455675291669e-01,    -9.5143132894038154e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.3455369339710828e+00,     7.7184080478807882e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0964230297754547e+01,     4.6401427510538394e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3307041193842243e+01,     1.5404222510131152e+04,     1.8097522635675267e-04,     3.5627502983696467e-07,    -1.6891393463537343e-09, } , 
      {    1.2018870940796623e+01,     7.6938248840798551e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7571565893129883e+01,     1.0680258287283323e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4516681560936684e+00,    -4.4598059111805833e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.8119193431710796e+00,     7.8827108336070678e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5200905209113840e+01,     1.6591305316825725e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.7164214673586240e+01,    -8.2262178916494995e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4207117480809607e+01,     9.3434610238639289e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.4217211994077523e+00,     3.3595332850509141e+04,    -8.9218322479863449e-06,     2.7252831456210344e-07,    -1.2995430722141111e-09, } , 
      {    1.9957473529895225e+01,     1.7010063436040546e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.4232477680647438e+01,     8.6376011778062621e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4600092292941200e+01,     9.2659087628518064e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5259230576563768e+01,     8.4203482464410317e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    9.7021569736200188e+00,     1.5044958587575609e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.4533945286598698e+01,     3.2271130445046605e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -6.1042856780904842e-01,     4.7849437418885118e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1558846796613835e+01,     1.1936873457374410e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6672220523810040e+01,     7.4570155628328303e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1413162286772437e+01,     1.4651937617882624e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.5268809873918130e+01,     6.4786793476581952e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.0822993870030526e+00,     8.1643981147849672e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.9509187993199195e+01,     2.5090496691132666e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    5.1612011703502830e+01,     2.5091201323048812e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.2236301532506335e+01,    -1.0616689378412225e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7648450857117691e+01,     8.5236376569749900e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3649811756430783e+01,     8.1054383885637071e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6536045595670945e+00,     9.5311740360980002e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.5485579815511059e+00,     1.6685716338906946e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.2735382050466127e+01,     7.1072105897597721e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6487935610402118e+01,     1.5569976762087197e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.3415208258550521e+01,     7.1113656753271725e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.5266316992549722e+00,     5.3303035718292249e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1943162560012246e+01,    -2.7562699845524776e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2835128619163584e+01,     6.6398878391315029e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.9790411071152496e+00,     2.8333489456692838e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4706253452030447e+01,     8.4003960756043980e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0252374626641483e+01,     1.6224021496654574e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.6335292765710964e+01,     3.1729573384715222e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.9290922029869513e+01,     2.3350214062618641e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.2699139964350339e+00,    -3.9957992529271496e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.5390261679717476e+01,     3.1156526266145833e+04,    -3.1788098606338479e-04,    -9.0500169852717095e-08,     4.1524291787031741e-10, } , 
      {    1.5484857098599976e+01,     1.6169898413722480e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.6589604368156667e+01,     2.4369241417517434e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.1832973879976308e+01,     7.1096167650199204e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.8795808637436494e+01,     7.1089595000670233e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.3686289180657916e+00,     6.0415589119795977e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2116716574491207e+01,     1.6682527778569787e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.4300972851860886e+01,     7.1306180405213236e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2870936465880103e+01,     8.4423445460932253e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4439278081846439e+01,     2.3800428029131170e+04,     2.7162937303459015e-04,     5.3275205789764682e-07,    -2.5277700519370095e-09, } , 
      {    1.7829844666538794e+01,     2.4133954802164877e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.9512568662854694e+01,     1.6841787386161435e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.7072628828416278e+01,     8.5130959614363110e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6153285009656827e+01,     1.7373544486602375e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -8.9726099050935471e+00,     8.0258085847678831e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.1731544089058810e+00,     2.3940172031422037e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.6737664059248672e+01,     8.2107987806103101e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.8682400611148076e+01,     6.8946715947654811e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6421959211462880e+01,     2.2878512569366936e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3433352577153252e+01,     1.6588656532937301e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    8.8390200471034550e+00,     9.0279923151830662e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.0232010999682579e+00,     8.2862734988387347e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4389479684478719e+01,     8.3538363538446629e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4558742261472011e+00,     7.0281959945520128e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.6452679913403880e+00,     8.2445802081132751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.1654356086386422e+00,     6.9077829446938813e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2648846737822590e+01,     7.0879582245656202e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0308580545070432e+01,     1.6727664809395774e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.3162187708238203e+01,     2.4012268864226851e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    8.0246862109240755e+00,     7.7713715152949180e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    2.4714755814807589e+01,     1.0646476859780360e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5339529286734738e+01,     3.2338668919687014e+04,    -8.7259675208180897e-06,     2.7244104809950373e-07,    -1.3009975132574398e-09, } , 
      {    1.9775235173741208e+01,     1.6984326286518713e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.8172477516955867e+00,    -1.0796836702563071e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2296354514028009e+01,     1.5725081284111444e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.6349344854129800e+01,     6.8150417909696562e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2152345418981504e+01,     6.4816918929648054e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.2012998451874665e+01,     2.3345611662939169e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.1757627427848172e+01,     8.8690850235494927e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1307324397825914e+01,     9.2323213390248284e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3229550134011630e+01,    -1.2969555531937785e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.0324710918816582e+01,     8.5805198404125495e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4807180363469357e+01,     1.8440453671118530e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -8.2134263063364479e+00,    -4.2417925886392368e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5630025863160421e+01,     9.7364819935569176e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1347476101062824e+01,     1.8005654841257245e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.6841328649193734e+01,     2.5144929119536300e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4581947462987104e+01,     8.3129342054233675e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1329961755495638e+01,     8.6038197503032970e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3034707631064350e+01,     2.2827834841420707e+04,    -4.7019373670473301e-04,    -3.4091128428260354e-07,     1.6015819955453518e-09, } , 
      {   -1.4683864211779778e+01,     8.6143012679155127e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3126206661564165e+01,     1.6257802924157537e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.1728569400131100e+01,     9.2321273353488414e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2600560288164804e+01,     8.9750232179718751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1568083986367133e+01,     6.4280114502867373e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.1644756376097284e+01,     8.4793145339333460e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1314327280116380e+01,    -5.4479380462714957e+00,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -7.0539354213365925e+01,     8.4152267584259516e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2981268846115114e+01,     1.2329894427067396e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5859768937985006e+00,    -1.6779078263174397e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    4.1699143674985333e+00,     8.5110184186526112e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2283320124411008e+01,     1.6013318045102949e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -4.9558833140071208e+00,     1.6824514389394582e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.5763196607199967e+00,     5.3832726690404170e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.6144927961052087e+00,    -8.4142784879322608e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    9.3113193274301210e+01,     1.5647522309548594e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.4349840673490505e+01,     1.5647535736650047e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8990219745071641e+01,     1.1073279256976311e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.3979012799163719e+00,     1.6586365163672337e+04,     4.9013024489931933e-04,     7.1921624778918546e-07,    -3.4053797774814918e-09, } , 
      {   -8.7711093060243272e+01,     7.1091900809696326e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0839075027627553e+02,     7.1093861841173111e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6494853795465037e+01,     1.0707593275509498e+02,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    8.8200192379116572e+01,     7.0937461042550422e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2026849261021454e+01,     2.3976220447824446e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4064965787243995e+01,     1.4549821144641806e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.0328255404027438e+00,     4.3919227721955231e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5888515363847034e+01,     8.5744863660176015e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5159044198640981e+00,     8.3559138966283626e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0670549188250453e+01,     1.6684605321353487e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3956962012195305e+01,     1.7514418094248624e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.4277733292907602e+00,     6.9853352057060038e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.7523916821528900e+01,     7.1248301608319016e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.3696196066353465e+00,     8.3920581465112518e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2876713145471145e+01,     1.6749706062840502e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.4669001177205566e+01,     3.1518869933163594e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.6337923140419587e+01,     7.0923674517346908e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    4.3157522126974776e+01,     6.6004252531916654e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4728106767069681e+01,    -1.5757219301759235e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.6055655016081676e+01,     2.5298097273081723e+04,    -3.9751134229741748e-05,     2.3556127137750133e-07,    -1.1256888861682315e-09, } , 
      {    3.5247864924211825e+01,     2.5302699672761195e+04,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {   -2.1442104082374115e+01,     3.2304559556591495e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    5.9627733213120294e+01,     3.2305264188507645e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    6.7312868307223059e+00,     1.6839709843377736e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2126849540408266e+01,     1.4843434649157560e+04,     1.2198050388615041e-04,     2.8172455135359777e-07,    -1.2869912460101852e-09, } , 
      {    1.2234442784628362e+01,    -6.9703390155849377e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.6934985178790505e+01,     1.5021522637893511e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.3927702747592472e+01,     2.2772574374944554e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.8835705043173277e+01,     2.3898668186812323e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.2911324017815254e+01,     1.5098851241849601e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8689421225625367e+01,     1.5421058876459818e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.5517642428513465e+01,     3.2340960288951977e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.1199298881917056e+01,     2.4697828037397750e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.4034177374700587e+01,     1.5580014928824428e+04,    -2.2086356285824842e-04,    -1.1748005120646278e-07,     5.5011808395499038e-10, } , 
      {    9.0406249608447542e+00,    -1.1418510311695300e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4570170404879264e+01,     1.6861484377234701e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7935565503891144e+01,     1.6851175095979212e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.1731386732190497e+01,     7.3239046703214626e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3030995042565547e+01,     7.3623137728580332e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.4011102144344640e+00,     8.4952447040153129e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.5125579134400766e+00,     1.6937925620693371e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.4600546449244465e+01,     1.6731005987178833e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2860471808126125e+01,     5.0661217132140121e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.9757475742389872e+00,     6.7938194238626083e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.3036979574855696e+00,     2.4515510519256390e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.1240755780467397e+00,    -8.1901596446723197e+03,    -3.9947609822148190e-04,    -5.4273921972850337e-07,     2.5667490718982164e-09, } , 
      {    1.0850838307031552e+01,     2.2797157538745483e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.6505980237344442e+00,     7.7162658096111636e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.3897631058839494e+01,     8.2464585969408217e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0406195425911555e+00,     1.8232699392748509e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -6.3427416248288253e+00,     1.5567899219303494e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.6369077096355289e+00,     8.4514903658416915e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -7.7469653027677121e-01,    -7.0732397161410609e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.6923987718889570e+01,     1.5660148261421698e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8091771016892878e+01,     1.1387225259762704e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.8503383845588872e+00,    -7.9260089498570244e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0808888379691751e+01,    -2.3171111706391666e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -6.7605650394625840e+01,    -1.2193965786309277e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.9857383325124147e+01,    -1.2194100057323808e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.9201965228228914e+00,     2.7222471903236510e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.4368547165332537e+01,     1.5016920238214043e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.9592680923931837e+01,     2.3959299767015658e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -4.9884891247256844e+00,    -1.0742881761636015e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2363357590166860e+01,     2.2561870923392926e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.8374129360808649e+01,     8.4246060022010242e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.1629939850337831e+01,    -8.4423263130799496e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0640477117038795e+01,     7.3008039143868673e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6021431995032284e+01,     9.6589297325447915e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0858859924288730e+01,     5.8924878643502980e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1122555757978416e+01,     1.6692267804345640e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.0261537270601515e+00,     6.4809805474979285e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.5386393903273170e+01,     3.3417239231976135e+04,     5.4522970760836301e-04,     9.9669451003541732e-07,    -4.7251395801978702e-09, } , 
      {    1.0113280144168824e+01,     7.3445747478960275e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0903535834268878e+01,     2.5483869976783728e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.7055250631181273e+01,     1.0129502963249904e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5273874789393403e+01,     2.3427542666574725e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.3212629112241121e+00,     1.5660646934298775e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.9477083487854692e+01,     8.2404251225458738e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2958000479777509e+01,     3.1911890902856583e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    6.9888635743131591e+00,     8.4203032926484975e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4853472716318979e+01,    -1.6756124107970663e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    3.4422065484843131e+01,    -9.5457935170670462e+03,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.9838405685543854e+01,     1.4985445640347871e+04,    -2.4893105098351117e-04,    -1.5510711973652738e-07,     7.7768477773099544e-10, } , 
      {    4.3232140876516452e+00,     7.4035344741172876e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.8264768543683516e+01,     6.2265491078220257e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9481986774295859e+00,     5.6485379422866099e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.8658447718252393e+00,    -1.7466451093533334e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1562986784189908e+01,     1.4443882950219424e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.9654814695507142e+01,    -1.7680810734313727e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.7998383310094770e+00,     7.0291060392151639e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.3317660741951833e+00,    -1.1518891979067630e+03,    -2.1613818087120999e-04,    -2.5544832857661450e-07,     1.2051013671339735e-09, } , 
      {    3.5917136020626657e+01,     3.3421841631655596e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.9696075130356505e+01,     1.5647200390622869e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -1.0932789769324039e+01,     1.5647857655575768e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.7958958916914307e+00,    -8.3636363137351127e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -6.9458822750181248e+00,     1.6753521083982185e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6137169272360655e+01,     1.1039497829473345e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.8951580768448144e+00,     8.6939310717113385e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.0171919782012324e+00,    -1.0123433204741132e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    5.2118532136947939e+00,     2.3911786098011780e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.4403644464167332e+01,     1.2722915396760385e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4543137089566443e+01,     1.6650823893850524e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.4829110106496980e+01,     8.1452532904616191e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.9771838858518507e+01,    -8.4507391544073434e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.1737546868609735e+01,    -1.2531603393190999e+03,    -1.2365027492594382e-04,    -1.4454719902280817e-07,     6.8353880899633469e-10, } , 
      {    1.7864080495053916e+01,     2.4498589838447606e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4680559193381582e+01,     2.4053772708836568e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.6129040131500378e+00,     1.5646817677632444e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.2180844131487198e+01,     8.4219761698848415e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.5436654621016366e+01,    -8.4449561453961342e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2866341007708703e+01,     1.5648240368566196e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.0201616015458328e+01,    -1.1341123129878883e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4123769823395051e+01,     7.5786884259554408e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.5112876893327032e+00,     2.5666187494925089e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    4.9789858222476155e+00,    -3.2202766428059113e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.4376778978244484e+00,     7.3019768516800104e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8420894001627932e+01,     1.5393097583423416e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8754952513532203e+01,     1.3748012510167953e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.4232626413610026e+01,     1.6040549992792308e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0359571586219131e+01,     1.2062853809821991e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6525424780835682e+01,     9.4106420838808808e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.1249434604797045e+01,     1.6136151197159805e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -9.5997981849310463e+00,    -1.2990330959774788e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.6881492002266913e+01,    -9.5503959167465146e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4724220585363511e+01,     1.5280245202928167e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.3559678306241842e+01,     1.6757226550209693e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -7.2705498890639468e+00,     1.4321049798162570e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    8.4943034739526020e+00,     1.4468466114020357e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.5120938542142568e+01,     1.5727158826895144e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5548759429899894e+01,     1.5507809401173970e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.5119918939424377e+01,     2.4040654797637111e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    6.6890086989909685e+00,    -6.5589555997275966e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2193958861532352e+01,     2.3314137065072995e+04,    -9.6618300342162913e-05,     9.5303994693359024e-08,    -4.0865429994403896e-10, } , 
      {    4.2051611172239554e+01,     2.3975891815347994e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -8.5771719938614623e+00,     2.3976549080300898e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.3722343560270995e+00,     8.3804970604292957e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5689462781296687e+01,     1.5831000526961587e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    6.5320678415490656e+00,     7.7713771450919794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.4550326678168073e+00,     3.9989018025025362e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7209047443564248e+01,     1.6213479801115895e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0408391022746255e+01,     1.1466300226669300e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5034649006642891e+00,    -8.3870654918151431e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -1.2006638856560152e+00,    -7.1262088133522529e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.4032964900998891e+01,     1.4325652197842039e+04,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    2.6615074895149640e+00,    -1.7843029094494461e+04,    -4.8993438017215106e-04,    -7.1930351425178529e-07,     3.4039253364381634e-09, } , 
      {    2.2770049950989943e+01,     1.8398675810950233e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0008484838757791e+01,     1.4233179498667801e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.0389066938106080e+00,     1.5398164655370685e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    8.3063990398580874e+00,    -2.4230493650615495e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.8933565756855046e+01,    -8.2242690077678399e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.2191929232857248e+01,    -8.2235643758516908e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    7.7732033975792625e-01,     1.6236021706246949e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.4250658127776058e+01,     1.7436865833236501e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.0367963734880510e+00,     8.4864352548516781e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.5674539755282211e+01,     1.5477794459166511e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.9990752276079142e+00,     2.3896590644028620e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0351951653227696e+01,     9.4210132848760531e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1659522261294741e+00,     6.5923142360130169e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2213468642478674e+01,     7.7352945177492802e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.0031595301906115e+01,     2.3347560546820830e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.0155961956030243e+01,     7.9859936828699701e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2165677046564689e+01,     8.9821439388902054e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.3614047904393853e+00,     6.4007984541696205e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7632468528988287e+01,     1.5490948654479789e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.4668085389587411e+01,     1.6767088614521537e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.3453453755485461e+00,     7.1857293760021394e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.5557386658645420e+00,    -1.2320225305040335e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.0921119053564228e+00,     9.5841431333091914e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.7455082792075576e+00,     7.8489294061041037e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1868146001680463e+01,     7.1894702258717789e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6137899995122718e+00,    -1.8139238199271722e+03,    -1.8274292976013022e-04,    -2.1905404427487519e-07,     1.0864141298615481e-09, } , 
      {   -2.5058935847042918e+01,     8.2066436950429088e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1331269763643810e+01,     7.7375957175890153e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.7066630235125075e+01,     8.5346714641196886e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.3372050809469158e+00,     6.1655118388170649e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8483658545524584e+01,     1.0522523932942891e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0460872875200245e+01,     1.0005550036412438e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9831907670091226e+01,     3.3455588848218271e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3653465906608563e+01,     1.6886110509203081e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.2337897239718751e+00,     5.5372042537715224e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.2381548592392392e+01,     1.5018869122095703e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -4.5704606608870755e+00,     7.7315622431940665e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.0147419621781246e+00,     3.1187982113443541e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    5.2408778377196645e+00,    -2.3984662012606168e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.3151247393437309e+01,     9.6251483050418283e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.0054380623284139e+01,     1.6131548797480336e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.4652277687634170e+01,     1.4915610166645438e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.5275486152438576e+01,     7.8177778109173223e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3833918355277904e+01,     1.5385010460057407e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.7288775236030958e+01,     3.1192584513123005e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    8.4341762214082294e+00,     6.6387149018383598e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.9002903564204441e+01,     1.4322998682044230e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    6.2048547092801897e+01,     1.4323703313960379e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    6.4006100716379475e+00,     6.1217410053079057e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3496563755344209e+00,    -1.0334136656292754e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3198314162560635e+01,     6.5236403634536318e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1808610224868588e+01,     1.6242073931210356e+04,    -2.5435674633291232e-04,    -1.5383070227690224e-07,     6.6953254174908013e-10, } , 
      {    1.5566799757752360e+01,     6.4836103798141139e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5379062736927228e+01,     1.7907439063941612e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6704080514212951e+00,    -9.7238627552345370e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -1.7960838437705398e+01,    -1.6780206525720987e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3804105577107659e+01,     1.5435349724952415e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.1702774445298914e+01,     1.5864781954464548e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0643970123892837e+01,     2.2954891893085914e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.6167630657040366e+00,     1.6997444197718167e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1390965359636285e+01,     1.6597864272425453e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0099303965730497e+01,     7.1085767870765949e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1832568195961297e+00,    -7.3596329400876584e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.2549847182629790e+01,     8.4344325600881675e+03,    -4.6576215180844699e-04,    -4.7874866195885568e-07,     2.2587469402893278e-09, } , 
      {    1.7869353622724923e+01,     1.5440609389584783e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.8158516041019102e+00,     6.4411656456000164e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0730913110099802e+00,     1.6793335985880098e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.6163642266328765e+01,     7.6181510118952792e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2753532938711380e+01,     1.6507726142689709e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.8573893614899752e+00,     1.5078740015078574e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.5165527864531320e+01,    -1.4738347318575575e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -4.8239658485988208e+01,     8.4948565622217775e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2059990357099782e+01,     4.0599846250137256e+04,     2.0524538708404093e-04,     5.9678624889859442e-07,    -2.8350449630242330e-09, } , 
      {    1.5747192900054399e+01,     7.0755067050405059e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.1919502518556138e+01,     1.5040551346031629e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.3536315124939922e+00,    -1.2201146376485306e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.0176089126107417e+00,     1.7859865460823123e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.4526381641927397e+01,     6.5345251416507949e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.7527280274236938e+00,    -2.6337528166131724e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2384469831743328e+01,     8.5703524479837142e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.5352564532774622e+01,     8.4560080627715433e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    9.6142432546986107e+00,    -1.2186919467147779e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.9973339332257993e+01,     2.5077731095891286e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.3178013512694690e+01,     6.2663640097199377e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2961174586356954e+01,     1.4357132425505268e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    9.5468738400528778e+01,     2.3976213734273719e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -6.1994289998154102e+01,     2.3976227161375173e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4152578743476733e+01,     1.6100074199614166e+04,     1.2188257152256629e-04,     2.8176818458489763e-07,    -1.2862640254885208e-09, } , 
      {   -9.7991130877276689e+00,     1.7310835363615333e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.2525034435133087e+00,     7.0655172990343126e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.1350779426422104e+00,    -2.6906804285552005e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3809143977603093e+00,    -8.4154419494392823e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.6999031213760134e+01,     1.7388163967571421e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.3194091454690678e+01,     1.4849486874120967e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -4.9150008031433110e+00,     2.2649741222887696e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    6.6610841211099157e+00,     8.1878083412927563e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6796948239397956e+01,     7.1099994780103489e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8082305943874509e+01,     7.8111920469898914e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.0892928809454734e+00,     8.2419503757970906e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.6388519842070195e+01,     2.2654343622567165e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    3.9090052330828428e+01,     7.7398969174287486e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.7667324675334331e+01,    -1.2217044920213884e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.8144209989700943e+00,     5.8864412691839534e+03,    -2.7668365262289327e-05,     3.0697044809548703e-08,    -4.6212440083360988e-11, } , 
      {    3.1863816750275323e+01,    -1.1397734883858298e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1632536756840217e+00,     2.5051034105192826e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.0946916389810745e+01,     2.5130663908988652e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.0776211058659477e+01,     2.3721789008148546e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.3982302342018231e+00,     8.0656234866657942e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.8761340885701356e+00,     1.5654087978699048e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3289757229779006e+01,     5.8953327784080884e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.6515991700158615e+01,     8.4161931608736304e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -1.4112997696051654e+01,     1.6202542102892730e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.7190521322549884e+01,     1.6207144502572202e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0396461703112264e+01,     1.6675407057651457e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9514316594339540e+01,     1.5317434956692587e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.8669745055693285e-01,    -1.3051188243719480e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.6082985976886022e+01,     1.5624987117968263e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.5013340411629610e+00,     1.6526870967013059e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.2480731075877515e+01,    -4.4390304833435789e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.6597016823460486e+00,    -9.9293314554215146e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.5825991998691876e+01,     1.3115936366453374e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0994610383476044e+01,     6.4472291301575442e+03,     3.1326357208312948e-05,     1.0524752329291559e-07,    -4.4836054042690995e-10, } , 
      {    1.4164164098988788e+01,     2.4570765355935480e+04,    -1.0204399569156407e-04,     9.6580412152984203e-08,    -5.1680653592595432e-10, } , 
      {    1.1969508989780730e+01,    -1.3951957108749302e+03,     2.4735921230730189e-04,     2.9224083883601718e-07,    -1.3818644352665104e-09, } , 
      {   -5.3634961238605889e+00,     1.7016589022041517e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.7443006339586844e+01,     1.0051950702237780e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.7228257196014933e+00,     8.3847283969994078e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5778563850149718e+01,     7.2152263269658542e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.8266302429344563e+00,     1.8024208201205099e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9280448283896540e+01,     2.3988839686146825e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    9.7825195991596257e+00,     2.2197235887110197e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0266317540438870e+01,     1.6762157582365613e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6389731423353712e+01,     2.3908706353549554e+04,    -6.8550812216900168e-05,     1.3293106322342366e-07,    -6.3622099372004408e-10, } , 
      {    1.9561960609839687e+01,    -9.8741258520412885e+01,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    5.7816945852915715e+00,     2.2872765975282823e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.2136285489470966e+00,     6.9929526066087874e+02,    -1.8807069274594723e-04,    -2.1782126004654988e-07,     9.7753467335796839e-10, } , 
      {    1.1797017364440736e+01,     7.7713827748890408e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    2.4218809358056451e+01,     1.5502413907081265e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.3837807018632372e+01,     1.6171061875229501e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8504741506590925e+01,     1.5653562497494266e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.7559503534896209e+01,     1.1432518799166333e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7475542602069691e+01,     3.2369346222362237e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -3.5637459989876859e+01,    -1.8480631931852722e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.7506229228643498e+01,     2.4055850251620272e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4274172705989772e+01,     2.3369242770756755e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.0074970492260837e+01,     3.5920617259126402e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.7291086301267185e+01,     2.3348265178736976e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.2844221506840583e-01,     5.2555169725936230e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.9092221423132258e+00,     8.4416332006263492e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1396796721420532e+00,     2.4564713130972075e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3601492510194911e+01,     6.0645493512971507e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9802272422174980e+01,     1.5693929688924662e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0044507214309353e+00,    -1.5786428343716775e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1884462455058943e+01,     1.0398571006105423e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2934848639153726e+01,     2.5091560352557615e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.2830522149229692e+01,     1.6335355185169661e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.1677950651084057e+00,     2.5090137661623863e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -7.6985452342786136e+00,     8.6878975973163906e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4846304885359455e+01,     7.8847883763907685e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.2898418286498323e+00,     1.4626200468360788e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7096918484810963e+01,     1.3537309058616331e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.6203503672041961e+00,    -1.2128443365819267e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.2279767832639568e+00,     7.8515592384202882e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9911149965657188e+01,     1.0915544902635878e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.1379544972002638e+00,    -1.2254367665766022e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8045811906257534e+01,    -1.4965670676054242e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.8978475720166394e+01,    -8.4442828690002079e+03,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    5.4926781906722283e+01,     1.5019573754011852e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.2234285996308195e+01,     8.4226494462807659e+03,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    8.9001812317158890e-01,     3.4006134826791651e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3271384810918667e+01,    -1.6749538344043234e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {   -4.8773973321782851e+00,     3.6058808328095492e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6637658897249512e+00,    -1.8263809692493455e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.1108476257140843e+01,     2.2076703934893081e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.1389049841014142e+01,     1.7403084405733534e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    8.8021470287470311e+00,     1.5629504814898115e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.5556560844354230e+00,    -7.1504863397357872e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6912024249532337e+01,     1.5666716692807544e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -4.1960531298669967e-01,    -1.9716638040646651e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5774844678845064e+01,     8.3659033026345587e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.3075019266129821e+00,     8.6541161698134256e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2702375318285539e+00,     8.3444824937508983e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2725248095363012e+01,     1.6727212693355628e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.3739198882164851e+00,    -8.4166054109463039e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.2673116895231020e+01,     7.7948298378624759e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.6999188870650466e+00,     1.2610464089321262e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.5852501027081978e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4304142405533753e+01,     1.7392766367250890e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2621101973313078e+01,     6.8740015171909163e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.9580267157153141e+01,     8.4345620386084611e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.2836077646682327e+01,    -8.4323702766725128e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3311226031388543e+01,     1.4887224233235178e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.4116148214423188e+00,     1.6783487490343465e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.9791569385832553e+00,     1.6740827674387761e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9335206884582405e+01,     8.3685331349507433e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1947896428098620e+01,     3.2287991191740784e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    9.4678633468810478e+00,    -1.3971962192534707e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1826009179374346e+01,     1.1859321196362287e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.1735485012373690e+01,    -9.5484470328648549e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.9276061217787046e+01,    -9.5477424009487058e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.1168045736932566e+00,    -1.8052578560325182e+04,    -5.7266288975394608e-05,    -2.0858139002375567e-07,     9.9081372008355873e-10, } , 
      {    2.7745815728370602e+01,     3.9485217690870959e+04,    -1.6556823542203652e-04,     1.5991094457716933e-07,    -7.7109615980471704e-10, } , 
      {    1.9747562164727462e+01,     3.1642852870163642e+04,     5.5596517935601208e-05,     3.4575874235454526e-07,    -1.5942661570974091e-09, } , 
      {    4.8657937752679192e+00,     1.4983173021651366e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.4801294094056967e+00,    -6.3091060903402331e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.9842386805126509e+01,     6.9344864966633932e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.4713886482437868e+00,     7.7368843721221383e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6921258841451944e+01,     1.5595713911609031e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.9193719248595666e+01,    -8.2937807466554277e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.8397612621646797e+01,     6.9898434268497349e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.5937889641666807e+01,     1.5109392937388278e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.9112662025203804e+01,     8.9085476094893293e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7598702326007292e+00,     6.5147619750008926e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5936744570195227e+01,     8.5731515686255461e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.4414736132302011e+00,    -5.5730864983608581e+02,    -1.8274292976013022e-04,    -2.1905404427487519e-07,     1.0864141298615481e-09, } , 
      {    1.8708787879598432e+01,     8.4641190799501892e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    6.1585911167898235e+00,     7.1217396520685843e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5182191179568154e+01,     1.6100038559654527e+04,     1.1665274090033337e-04,     2.8295733558192313e-07,    -1.3958707025137649e-09, } , 
      {    1.8331030097790784e+01,     3.1154234896880869e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -9.9946469664852700e+00,     1.6744108639010239e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.3285032548932207e+00,     8.6687527729930425e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.3402560283631900e+01,     1.2419451973127107e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.6266942388180809e+01,     8.4355437004241867e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.9869523205780006e+01,     8.4313886148567872e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -9.2307621657044603e-01,     6.8613822788685366e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.4542983508546192e-01,    -2.5124630434593702e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    8.5691939422403642e+00,     9.0279866853860040e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    1.0924735773588715e+01,     1.7356678110111130e+04,     1.1655480853674925e-04,     2.8300096881322299e-07,    -1.3951434819921005e-09, } , 
      {    2.0376745295934647e+01,     8.1790347179645833e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0249111731748071e+01,     8.1540269137897931e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.2376068994833442e+00,     1.6155179905297922e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.7343138656161141e+00,    -8.4356083959101124e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.9032964618021282e-01,     1.6688561252964737e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5388320261667586e+01,     1.3930330028309316e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.2516643671395062e+00,    -8.5731857795658343e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.4604127968979270e-01,     1.5901960462775221e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.4296233138431882e+00,     1.5190073703593662e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.4091642365385674e+01,     1.5664802019866176e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.5650477269906048e+01,     9.0306221474992490e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0534755321978534e+02,     1.6761649623827576e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.9738063358575235e+02,    -1.6762665540903650e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3027102316092893e+01,     2.3381675539713407e+04,    -1.5837483666925783e-04,     2.1413541555454645e-08,    -6.0218707330615518e-11, } , 
      {    2.4192570582371204e+01,     1.8791696780643222e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.4192193391584258e+00,     8.9881774132851533e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1801786908360690e+00,     3.1178403514484216e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.0175002998205347e+00,     7.3357582791829745e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3185895515048308e+01,    -8.7620527775208047e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.5642753363686431e+01,     1.5895434876774252e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.5067697403753895e+01,     1.4287302581599894e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    8.8428624022702884e+00,     2.2250204984321390e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.0298512666338677e-01,     7.4373159016202517e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.3819767629274935e+00,    -9.5818517640442260e+03,    -2.7596302556729204e-04,    -3.9495831345269459e-07,     1.8698778866713695e-09, } , 
      {    5.0344199482788010e+00,    -2.2407318469201850e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.7541450345782504e+01,     1.6760080039581913e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.8649967145646130e+01,     1.6764235125149313e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.1176759656763537e+01,     1.7377347256211702e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.2816870107608509e+01,     1.8334241461137561e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.2557014565515036e+01,    -8.4421026560239152e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7904280206091663e+01,     2.3836500825899097e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.5151153973243532e+00,     9.0404438347081777e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6352695059099487e+00,    -9.9411156865997691e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3489570367823406e+01,     2.5108873215291940e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.0209443634718916e+01,     8.4099940113930097e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7560971723980987e+01,     1.6668857577837851e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0735117278288683e+01,     2.3993141128633230e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    5.1670782577677073e+01,    -1.8473585612691227e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.5940276455590862e+01,     8.8349512800884513e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4573996977270998e+01,     1.0018169274734815e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0916083154634796e+01,     8.9220541207606839e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4378875155367323e+00,     1.4685719045385587e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.4293837768221334e+00,     1.3868544462385073e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.6413803631703281e+01,     2.5084290051491011e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.0318938690765023e+01,     8.3577362596486946e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8511341465175938e+01,     1.5929216304277215e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.0323945167848841e+01,     8.8687327075914163e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.0643194578253350e+01,     8.2006102206479609e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2010134443686045e+01,     3.0614944825448925e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    4.4865771321714636e+00,    -3.0010162579856114e-02,    -2.5659491293243832e-06,     5.7275888286280838e-10,    -5.5166948773454092e-11, } , 
      {    1.6801887330915843e+01,     1.8300460033634597e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.5460072750112346e+00,    -1.2374275003828593e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3738844858510738e+01,     1.5178128729687351e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.4416630940358584e+00,     8.2930351902482635e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7248279092160391e+01,     1.3508957336146363e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4300477865376932e+01,    -1.9807473744312358e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.2070348919338052e+00,     7.8773153395143618e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.3450149061393510e+00,     4.0631302097434964e+04,     3.2672883574363378e-04,     8.1023032014387868e-07,    -3.8475298546533887e-09, } , 
      {    1.4813318904361148e+00,    -7.8308957221167784e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    6.9785772387215289e+01,     8.4326541116550143e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.1338149067354756e+01,     1.1308565872328865e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4540467092959727e+01,     8.7293654016241453e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.8170337818951774e+00,     1.8837592936296076e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.9859381559247291e+00,     5.7287200356149178e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8388885067632263e+00,    -2.3646847737576522e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0860363898326639e+01,     2.3982253922219392e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.0632629424831710e+00,     8.1981795422879313e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.3702917624748956e+00,     2.2859290688718414e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    9.9792156787820581e+00,     2.4933114652750308e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.1780130955923367e+01,     9.8140342545690401e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6530001516807957e+01,    -8.4342782036259596e+03,     2.2299111921642950e-04,     5.1773253005687344e-08,    -2.3523159807434682e-10, } , 
      {    9.4435725981885508e+00,     1.6569468863229074e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.7769559328620750e+00,    -8.6169376615027541e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.3305924806500228e+01,     1.0791591975798412e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.8211055836196621e+00,    -6.9437661979826942e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.3958509024054379e+01,     4.0635904497114432e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3721618905152859e+01,     8.2944578811820156e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6651149514519460e+01,     2.2651690106769358e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    6.4407931516899296e+01,     2.2652394738685503e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.5979500256143897e-01,     7.1175240998305219e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.2154486822906296e+00,    -8.1517788764625875e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    5.7111156750810295e+01,     7.7379480335470898e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.9844473560844698e+00,     2.5334155520058906e+04,    -2.2289318685284537e-04,    -5.1816886236987203e-08,     2.3450437755268249e-10, } , 
      {    1.0471081860531427e+01,    -5.5734428979572658e+02,    -1.8797276038236309e-04,    -2.1786489327784974e-07,     9.7680745283630396e-10, } , 
      {    1.8219746954423286e+01,     1.6006768565289341e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.7905285137769564e+01,     3.1782542481926412e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3062782684454378e+01,     1.5019932783520653e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.7694569547243745e+01,     1.2021302954147982e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.6365701676837958e+01,     2.5090520374614291e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2076935540613189e+01,     1.6797494653301146e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.6763954153174758e+01,     7.1453365489458829e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -1.4262974779474000e+01,     2.5091177639567195e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    4.4933220742272596e+01,     2.2863893088397879e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {    1.8013277799626266e+01,    -6.8014971046074265e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.9255487493990209e+01,     8.9166586266679769e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.2893197006361632e+01,     7.2840897684235515e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3801137621253016e+01,     1.6805928415874772e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.5071342604303961e+00,     6.6698664970251411e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5218813418727115e+00,     1.5740829027627082e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8864469649493291e+01,     1.0444971671930769e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.4582132317109560e+00,    -1.2385909618898813e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.4961970423278323e+00,     7.2850805512367479e+03,    -4.0400561548135206e-04,    -4.0485820882095132e-07,     1.9103113476759048e-09, } , 
      {    9.6537021701346664e+00,     3.3453297478953311e+04,     3.6208765498525937e-04,     7.0931635242092874e-07,    -3.3649463164769565e-09, } , 
      {    1.5107545523481020e+01,     3.2480704291242848e+04,    -3.7973545475406380e-04,    -1.6434698975932162e-07,     7.6440573100540521e-10, } , 
      {    5.4966447257211648e+00,     8.6092585763337629e+03,    -4.6586008417203112e-04,    -4.7870502872755577e-07,     2.2594741608109920e-09, } , 
      {    4.8693362528544597e+00,     1.7369135259433304e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3942502837761900e+01,     1.0685227569906147e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.1041955375599830e+01,     1.6648746351066824e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.2256204526329345e+01,     1.0269719013166014e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.2386702469491645e+01,     5.9323027662482100e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.9677421291325556e+00,     1.6768586116873112e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.0936786345525519e+01,     1.9247644653622456e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.3088698985103875e+00,     8.3563135687149424e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4643879525919424e+01,     1.6686906521193225e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6450565741863315e+01,     3.1716419189401942e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -1.1830215055842343e+01,     1.5645451480315620e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.2912075527382740e+01,     8.8702579608426349e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4310445743017816e+01,     1.5649606565883023e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.1269110204031929e+01,     1.6798917344234898e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.8981835982855618e+01,     1.1825539768859322e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2117979920078952e+01,     2.5248583361431170e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.7978710446782475e+01,     3.2147177518209141e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.9451518867380496e+01,     2.1866000483341453e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    3.6268951022893980e+01,     8.3244336689651318e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8850633883775043e+01,     3.3385759004312902e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    8.3459810141011303e+01,     1.6746615554077183e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.5422026464516904e+01,     7.2287328382372089e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9138484510464551e+01,     2.5941325296289382e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.2107290629155285e+01,     1.6799388210843503e+04,    -6.8949935079873592e-05,     6.4606949883810315e-08,    -3.6244185986067803e-10, } , 
      {    1.1280219619600548e+01,     8.3019135787010036e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.1737646262023151e+00,    -2.9983878528959008e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -7.2351801583552785e+01,     1.6777699610654039e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0561500558487374e+01,     2.6334346265982371e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.5685789882029018e+01,     7.6917473412961554e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2611500927245920e+01,    -8.6364983834280338e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6996558019154644e+01,     1.5960348947891758e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.4351337119616645e+01,     6.3061789116178497e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.2997054115594420e+00,     3.2128598631165614e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.0109602780821882e+00,    -1.6726109165963204e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -9.3011416823372279e+00,     8.4248296592570587e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.9088808731237350e+00,    -9.6497185250575103e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.5646080855715532e+00,    -1.2658039580069981e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.8070968066486213e+00,     8.7489348663213514e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1854771568362823e+00,    -7.0448537827308028e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.5281046759384573e+00,    -5.0220294415233059e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.9352963679112257e+01,     9.0594725428462953e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8564629369119537e+01,     8.8882726932577189e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.9250170933540511e+00,     8.4864257790655574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0138459754108219e+01,     1.6824290732338548e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1377003219452455e+01,     1.7941220491444572e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.3960892152824364e+01,     1.6777890066513206e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.4407988000824988e+01,     1.6756257528279195e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1706182744803520e+01,     2.4428790004704810e+04,     2.7409738980033042e-04,     5.3222293224608396e-07,    -2.4718758826418910e-09, } , 
      {    1.9637010679527641e+01,     8.4315172737588273e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.9564222292290872e+00,     8.4354150415221466e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -5.9425337064858354e+01,     8.4550416603238646e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.0196601919592023e+00,    -1.0698771692575485e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -1.1140399670496315e+00,    -8.7992551796441167e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    7.1315345307790823e+00,    -1.1429620487229863e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.9223710766251383e+00,    -8.0742266154504641e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.5834426579533782e+01,     1.7320575389391186e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.1002137188684927e+01,     6.9743013985613052e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.9582364160833542e+01,     1.5432912485321329e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.9059521490525988e+00,    -7.6485782039754131e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -8.9090661333790067e-01,     4.8624960029006361e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5521821177285101e+01,     1.4340271678811085e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.0703735386654394e+00,     2.5088219174774553e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.3032425206160887e+01,     2.5093478839406926e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.5668057196180262e+01,     1.7829886802929486e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.0600825458965250e+01,     2.3764369782153990e+04,     4.5477142565769374e-04,     8.2013021551213540e-07,    -3.8879633156579240e-09, } , 
      {    1.7249781768594815e+01,     1.5353282681525505e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.9080832592705721e+00,     1.4954787088241108e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.6264775678614036e+00,     2.4304807067704762e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    5.2363519814914934e+00,     1.6446688873684750e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.4990354384104130e+01,     8.5159494508085281e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0484093448333820e+01,    -1.6888035855936232e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.1855851709111818e+00,     2.5876890946476717e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.6090883714867758e+01,     2.3506820154412479e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.5267228925105938e+00,     7.1960386572576272e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1461387590024270e+01,    -8.3690318078278178e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -1.3779377899440107e+01,     2.5113390912221796e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1403837216768846e+01,     1.6707046246884798e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -8.6810837454560840e+01,     1.5647430971525480e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0929100211653707e+02,     1.5647627074673159e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.5996638087758676e+01,     8.1850681923595312e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8610005183973364e+01,    -8.4109242525094305e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.2309769642882308e+01,     8.3509828644724457e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.0887598163131260e+00,    -1.6768716537965342e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    6.6820362969263707e+00,     7.6654389506695970e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.3716251416941496e+00,     8.4519424818818370e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5320114111828001e+01,     2.2685823850230394e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2053572267823666e+01,     2.5143818104301932e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.3006122723358395e+01,     1.6535687435726108e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -4.1235798279200822e+00,     1.6196206804349040e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.1022206178545897e+00,     1.6292757288953644e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    9.8342521251440012e+00,     4.9663397434544649e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3243061684222869e+01,     1.2252342166055276e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4611334551412520e+00,     1.3921513559596264e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7722489987373237e+01,     1.7163995020771654e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.1357025703088526e+01,     8.8161159263383015e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0975547757815646e+01,    -8.2638232248866677e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0026850214293960e+01,     7.7003594593140642e+03,     1.5241068300493238e-04,     2.5036748119858656e-07,    -1.1870662981966987e-09, } , 
      {    1.7812674737146157e+01,     1.3728806089891266e+04,    -2.4883311861992704e-04,    -1.5515075296782724e-07,     7.7695755720933111e-10, } , 
      {    1.5413994657679304e+01,     6.0284166204146240e+03,    -4.0380975075418379e-04,    -4.0494547528355104e-07,     1.9088569066325755e-09, } , 
      {    4.0702712302430953e+01,     2.3768972181833455e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.2158982679761088e+01,     2.4022621113649788e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.9662265579070812e+00,     2.4568479616467579e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.0001962918853437e+01,     2.5037879909879546e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -1.8578214475320777e+01,    -8.4695487849831879e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -7.8568427603107921e+01,     8.3754118565280405e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.2981669949162065e+00,     3.2265096970651659e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -7.7069357911213899e+01,     1.2031676120379052e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -7.4647070708242111e+01,     8.4490081859289166e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.8755768158317537e+01,     8.9232704626931081e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.5450503781142562e+00,     1.5811822333039478e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9267702823360366e+01,     2.3995408117532672e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -3.6644486880144385e+00,     1.6530268047820769e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.9736784487995713e+00,     1.4258916648189635e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.2961234647367821e+00,     1.7231557875777580e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.1044073662811371e+01,     8.4179241293520572e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.8072691811472517e+00,    -1.8469995317603211e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1152189901444720e+01,    -1.1067478700177790e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6539509307773749e+01,    -1.2531847196846204e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    3.2593480108893798e+00,    -1.6742969912657383e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    3.8822643277918938e+00,     7.8264884805727952e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5538766096009367e+01,     3.2880250360383921e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.7238350431034906e+01,     1.5630608342290536e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {    6.8927441229565964e+00,     8.9028664510524559e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.9501403776884505e+00,     1.4854556642452053e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.0601872199159637e+00,    -1.0142639625017817e+04,    -3.3495774803789431e-04,    -4.6950879193606149e-07,     2.2720259870149184e-09, } , 
      {    2.1984117254497150e+01,     2.6727367235675360e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.3593820545873292e+01,     2.5103468245413118e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    2.8745896749040902e+00,     8.1179974489596225e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.7285965083494741e+00,     8.5136482509687939e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.6591897594151170e+00,     3.9516673538168667e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {    1.6264162907174377e+01,     3.3402619751007085e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    8.6397387203765117e+00,     1.0263249464573602e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3507920142016712e+01,     3.3090953811935549e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.4239021465676812e+01,     1.8727262430830549e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.4247212003227775e+01,     3.3595308470143624e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {   -3.7097801735615541e+01,     8.7951363781905402e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1927807644236935e+01,    -1.6746425098218024e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.9451232288376412e+00,    -1.1634615070219354e+00,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.9644335144265156e+01,     3.9521275937848128e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.6952892601710303e+01,     7.4247568412243363e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2764334935726001e+01,     1.1701586842021854e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8725279007931763e+00,     1.1261032876538916e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6438350351597641e-01,     1.5425360318946219e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.4818079055531316e+01,     2.3294943765567710e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.9122967381713522e+01,     8.4019859299772543e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7562821580836065e+01,     2.4919996741550851e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5408141480813926e+01,     2.3348624208245779e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.6663709567175449e+00,     8.3532840643121781e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.0361772797328230e+00,     2.1071292733774354e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2417262225387136e+01,     1.4966252340842586e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.2521252667411620e+01,     7.5169457054160757e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9313046368547873e+01,     2.4526975771857866e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.2429719769873966e+00,     1.6026472240416228e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.0352436283958362e+01,     3.0404241373897297e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    3.3869786924499632e+00,    -3.6462556049660827e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7912195613545734e+01,     5.1041494093072743e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7743967773589276e+01,     2.2259021453034442e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.2772057715308573e+01,     8.9968407199962858e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.1949472481162957e+01,     9.0640749425257636e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7664515773860685e+01,     1.5683577439501727e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {    1.8844830259281899e+01,     7.1027291769437425e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -3.2615929077114183e+00,    -2.7577057434506405e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.5086102556700551e+01,    -1.2214808349653549e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.2030128893217205e+00,     1.5558511509485717e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.0216420773830123e+01,     3.1156501885780315e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {   -6.9210978471745506e+00,    -3.3666426746020784e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7983145538365171e+01,    -9.2155494592981540e+01,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {    3.1054686317497065e+01,    -1.2173257493979536e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0732346863365994e+01,     1.6868095776787995e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.5237470244313423e+01,     1.5668858931077171e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.8411205082997316e+01,     1.6969405848356600e+04,    -1.9206388487109000e-04,    -1.4849843052385087e-08,     6.0650191506802891e-11, } , 
      {    1.2156616532341866e+01,     1.7077626291046476e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.0083990696108378e+00,     6.7752268890717442e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0761193631923035e+01,     1.3172674022333600e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -4.5460765174626587e+00,     7.9920271572649181e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1804807970661038e+01,     2.5073576010323883e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.8644924440348991e+01,     1.6745948247023705e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3519004788068598e+01,     1.5626199115121468e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4115022312870657e+01,     2.2759420179631270e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.9680564653090808e+00,     1.5681310450602286e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.3074418824157290e+01,     8.4378753848095585e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    4.4055012498666017e+01,     1.5138666143747512e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.5720706882529409e+00,     5.7417502632732987e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.3924774442922192e+01,     1.6547541044587622e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1364377182367718e+01,     2.4499777680320149e+04,     8.6026697054383160e-05,     3.1440167219953406e-07,    -1.4943412092839226e-09, } , 
      {    2.8940705733807135e+01,     3.2344726774447485e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -3.0280864118333678e+01,     7.7372434016309408e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0884805789155816e+01,     3.2357880969760765e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    9.7747938931504521e+00,     1.9121452270398663e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.8670562563016027e+01,     1.3901978305839350e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6461642091796156e+00,     1.6656219387943231e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3994558360238639e+01,    -1.5136496337554697e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7080414379942201e+01,     3.2251942775338379e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.4728610843583313e+01,     1.1184612945491401e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1789346866755690e+01,     3.3483974781628538e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.7924819486527042e+01,     1.5360427296256476e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.5615036208669984e+01,     1.9184717750336204e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.7561859482066833e+00,     7.4585382687273004e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.5641422835964356e+00,     8.5056229245599043e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2042020472990709e+00,    -2.9454187556847096e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.2892310634306327e+01,     1.6974008248036069e+04,    -4.9668938615378649e-04,    -5.1567207191215793e-07,     2.4333283468568719e-09, } , 
      {    6.8443401846599690e+00,     1.5549313245783687e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.5746988118200267e+01,     3.1189930997325198e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    7.4448896501456527e+00,     4.2024344159622478e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.5304240197002727e+01,     3.1190635629241347e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.6570802594022407e+01,     3.1754156548516152e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -8.2807361627520635e+00,    -8.2637315937076783e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.9259690084097080e+01,     7.8510069488878034e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3807525016425558e+01,     1.2500562144913588e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -9.8626016433750152e-01,     9.2824237548589681e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.6327593059281320e+01,    -1.4340198299596450e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8224540579535631e+01,     1.8693481003327586e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.0296008453381535e+00,    -1.0036550574629017e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8855402449515747e+01,     7.7402255499051998e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.0630723452844162e+00,     1.6681975489037308e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -3.4054244904786927e+01,     7.2442748665256386e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7059179700559955e+01,     3.2302282040233389e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.1142931404785754e+00,     6.1992932663200299e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4142452889771753e+00,     1.0477473291536080e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -7.5623140966248021e+00,    -2.5023334912815848e+04,    -3.0226281028917725e-04,    -5.6980636754484870e-07,     2.7001697969395605e-09, } , 
      {    2.2480657327237456e+01,     3.1014490894590002e+04,     5.3128501169860945e-05,     3.4628786800610823e-07,    -1.6501603263925275e-09, } , 
      {    1.0671552630762896e+01,     1.5052984114988281e+04,    -3.1068758731060605e-04,    -2.2899757287443177e-07,     1.1261203703444190e-09, } , 
      {    2.5996595824174790e+01,     1.0411190244427802e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4773704352871810e+01,     8.8747661819863642e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.0066648272888870e+01,    -1.2195013437554931e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0603519084317350e+02,    -1.2193052406078154e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.1125607867825138e+01,     3.2307541704865755e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    6.0156822163441035e+00,     1.4292698075692600e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5388950900479799e+01,    -1.2098406825726562e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6110623428901334e+01,     9.3576729866696878e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7111352963533719e+01,     1.6198284347132740e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.6884863282901144e+00,     2.4230651887500346e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -6.2983829763010633e+01,     8.3814453309229884e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4589888506523385e+01,    -1.6777576693404801e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.1066339779557005e+01,     1.6792810504675319e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3766880130731014e+01,     7.6949359016333119e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.8197050264035141e+01,    -8.4336718728676951e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -6.4941198333600127e+01,     8.4332604424132787e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    4.7467362108765414e+00,     1.5818381288639206e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.0438456195634007e+00,     1.6842339675693918e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.8786301172748505e+00,    -8.7913986221177652e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.6535437812585183e+01,     1.6976774120143607e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.5042231123083820e+01,     7.9461787809720572e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.3510020463620762e-01,    -8.2246280372766414e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    3.4939002204891398e+00,    -8.2232053463428892e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.2803233009919815e+01,     1.7796105375426523e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.9377104138863039e+00,     1.5019221438053777e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1524449963811923e+01,    -8.1841017899118524e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    9.9681960196688983e+00,     1.6204843302732466e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.4783389022508935e+00,     2.5685393915201777e+04,     2.6876962681451335e-04,     5.3345571647440932e-07,    -2.5807553391454703e-09, } , 
      {    5.2892168876917092e+00,    -1.0551099822414951e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.8033604675034951e+00,     1.5717418801119291e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.7960366563763177e+00,    -8.4393662117269032e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    5.2440274542221665e-01,    -2.1069397576550273e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2192680623319015e+01,     2.5613218397713899e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    9.8186468951958812e+00,    -8.4290569304714154e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.3518962444344126e+01,     9.2027939510631440e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.9089330197392584e+01,     1.5631986994810890e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.5324305337206063e+01,     1.6705069683700844e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.3476098470800313e+00,     6.4034282864858051e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8392370518166587e+01,     2.4502356323943110e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    4.1938135406335228e+01,     2.3761603910046455e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.3847912284331825e+00,     1.5036142118862557e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.5637586985208021e+00,     1.5156292276090699e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.5174657267866891e+01,     2.5096882481485689e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.5234381589324762e+01,     1.5571087779640649e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.5671681135863651e+01,    -8.3847642919754089e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.2065242273896573e+01,    -2.6777846024123306e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.8895031647555840e+01,     7.8074597724346786e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -9.4794369479668301e+00,     2.3974142905040742e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.8944960423841724e+01,     3.2697932842242561e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    8.1180004749543535e+00,     7.1158470881431995e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3406645838793104e+01,     2.7120388205368348e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.6670854067736478e+01,     2.3978297990608149e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.0285755109804676e+01,     1.0837992641623756e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7881838965878538e+00,     8.4629631086042009e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9656741139468814e+01,     2.2286253400723799e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -1.8862036806494551e+01,     8.6480826954184777e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.7123962696059847e+00,    -6.6208901254850753e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.9857195800433693e+00,     1.6167612674254575e+04,     6.0126035195471375e-05,     2.0787773144699327e-07,    -9.3782843287509731e-10, } , 
      {    1.8408556399000595e+01,     1.7426513583813568e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.8082482982641054e+01,     1.6133497681361996e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9605434231820830e+01,     2.3681974106250629e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -6.6610450296339138e+01,     1.5663071051387751e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.1247173662009558e+01,     3.1939127069335177e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -7.7220851666942139e+00,     2.8198388934235736e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.4365515974546739e+00,     8.4214685470931108e+03,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {    1.2285048399993526e+01,     1.6766566809534681e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4247362354923695e+01,     1.4898689485836656e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -8.5514896218403962e+00,     1.6490453145922853e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0866779396760016e+01,     2.4257907729002342e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.4889509518903452e+00,     2.2404136569052498e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.1665612032333019e+00,    -1.9006799744383889e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -9.4234795220782139e+00,     7.9522122553670069e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.8579507754504398e+01,    -1.6186263158012935e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.4702846423658258e+00,     1.3084208057952558e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.4308252144493387e+01,     6.6402401550895775e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.5140921179957658e+00,     2.4540093683057323e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.2359192182881188e+01,     2.3177854350552325e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.0404116097124934e+01,     1.2218560738552309e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6943320232080843e+01,     7.2226993638422609e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.6414147018218486e+01,     2.3467357568472638e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.7865727393906685e+01,     2.2827810461055189e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    3.5771055854015330e+00,     5.3356990659219309e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9160656101995750e+00,    -8.7282713500178415e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -7.3836804245317076e+00,    -6.7072551093193556e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.1265123273085489e+01,     7.3975009997223387e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0163258640764973e+01,     1.7454899517223828e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3581781846366576e+00,     9.0610623972191534e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3803246456794488e+01,     1.5515531818280722e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0455296320156130e+01,     7.3188375983741935e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    5.7829821215016324e+00,     6.4783507151817448e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.9891487105684664e+00,    -2.4184557095228320e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.7327002401351876e+00,     8.9483625113872404e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8024116997544521e+01,    -8.4331115540423780e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0407887150562038e+01,    -8.4459176771656003e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    4.1640441769889911e+01,     3.1678905487343767e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.5972900935080503e+01,     6.4809738339472015e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4768307024761061e+01,     8.4338207612385977e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    6.3715406169184954e+00,    -1.1849105192118136e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.3173983711006578e+01,    -8.2599993191524663e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.2162160093503431e+01,     7.0141163004592181e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.7845449131905635e+00,     7.9640658642285180e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.0798501200924811e+01,    -1.7282661715334607e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.3876297978939807e+00,     1.5577699179193945e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.1756517373201632e+01,     1.4430728754906146e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5320980097963833e+01,     1.6840154469256850e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.3290748981081446e+00,     1.8495682624366873e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0870718714819734e+01,     1.5332060314418455e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.4072831432222177e+00,     1.6831987426270989e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.0264444924580614e+01,    -1.2407332001595062e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.9217745914500576e+01,    -1.0984180486263017e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.9228629445501205e+01,     8.4046157622934443e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1981635202894884e+01,    -1.1980733842038023e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.3562558189069662e+01,     1.5879418557644167e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.1515266075353193e+01,     2.3093481423102996e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.9289118906603420e+01,    -8.4324997551928082e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.2568300142157829e+01,     4.5364307411020985e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2052415731850155e+01,     8.4275661035540707e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -4.6847464748617460e+00,    -3.1507267131436274e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.8889009145018072e+00,     1.6963662770215204e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -6.4262258231206459e-01,     2.4592639587052872e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.5374818534431736e+00,     1.6744525556089953e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0575271330147707e+01,     2.4335459990014468e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.4190026204758702e+01,     1.2094607811714843e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9172761718567759e+01,     1.4764789998377873e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.0662379619911349e-01,     5.8062722966270421e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9015421719021788e+01,     8.7337459736092496e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8644150939582058e+01,     3.0797262343590286e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    2.0168228960332208e+01,     2.2057497514616392e+04,    -9.6520367978578792e-05,     9.5260361462059179e-08,    -4.0938152046570334e-10, } , 
      {    2.5281611620418700e+00,     1.6971707048196335e+04,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {    1.5118930537696626e+01,     5.5958126853259262e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.4751434397070838e+01,     9.1629790491652329e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5867163288830561e+01,    -8.4649463853037159e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -1.1808044899598258e+01,    -3.7647916935812009e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0747899172683820e+01,     9.3096795963609657e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.3095482883272425e+00,     4.4694750332076474e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0815388108397094e+01,     1.5862145560877312e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.2625885695453633e+01,     1.7770368225904691e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5994401143996157e+01,     1.5613036250129477e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    8.6987591922350997e+00,     8.3143650982261152e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1539710362917340e+01,     8.2188496198624962e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.6534820636392258e+00,    -1.6775275493565066e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    7.6868980469942443e+00,     5.3019176384189668e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2218572350782958e+01,     2.4951129385165394e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.7557909239860496e+01,     2.3997550355802297e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.7063467271654069e+00,     1.7389753821944279e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -4.1503954797837267e+01,     8.8289178056935052e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.3849581325286477e+00,     1.6134561342786947e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.4659495380324230e+01,     1.2645363135748263e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3115349590720669e+01,     2.1501365447058728e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.2160773310894916e+01,     5.4362417662516082e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.0183738498442656e+00,     9.0617737426860294e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.2815733080899303e+00,    -1.2240675919101831e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0397399301430767e+01,     1.4165616643661873e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8777624239843199e+01,     3.1207203994092062e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.0380210947247866e+01,     7.0033499381210895e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3258550643878271e+00,     7.8051585725949435e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.2409214731184662e+01,     7.8028573727552102e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2960040055587266e+01,     9.4444235113838440e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.5054356991314801e+00,    -9.6010638141179716e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -1.5840395337550923e+00,    -1.8603301155495765e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6151062122844813e+01,     1.1577633915184388e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.8772019408518630e+00,     1.1495135141616818e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4049951673570082e+01,     7.7038330406545083e+03,     2.5998594222495920e-05,     1.0648030752124093e-07,    -5.5723999693048953e-10, } , 
      {    5.0014999875652464e+00,     8.2914307860846566e+03,     3.0698819227621574e-04,     4.3183809017469699e-07,    -2.0451865137605773e-09, } , 
      {    7.9319659665784048e+00,     1.0485510266417677e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.5883481431307027e+00,    -1.0070464107529941e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.8562553183231099e+01,     1.5635077503574204e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.2251380578784419e+00,     1.5941775364673134e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.8774196916674690e+01,     7.1043571003875513e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.5903711076138295e+01,     2.3954890539846594e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.5563483297803806e+00,     2.5679341690238369e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -1.3564211415434052e+01,     6.4013507437021044e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.9453840446647341e+01,     7.1142191646993924e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2890259682158341e+01,    -7.9760108381348300e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3197396488293016e+01,     1.0207055224262029e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8520530776188355e+00,     6.6724963293413239e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2405694321261770e+01,     7.0961702213440167e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    2.0966556669005616e+00,     6.2768455273321533e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6086180435814672e+00,    -7.8646771496197425e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.6912238511473490e+00,    -5.2282150666330608e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.0351868334575745e+00,     1.6778157090690100e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.1831078198979341e+00,     8.3970548978213374e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.2707482086111481e+00,    -1.1080696036665659e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.0976781683975368e+01,     7.1089291030346703e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.6229932921727173e+00,     1.6746738471326425e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.4014827276617396e+01,     7.1096471620522743e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5661265336032480e+01,     1.9120283400523538e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.5834038559215756e+01,    -1.2349453204700840e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.3484959191709976e+01,     1.4376320095213496e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    5.8186061890341279e-01,    -8.3816510461501966e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    9.6926992282490669e+00,     1.5567346929771013e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.7161969602898928e+01,     8.2248830942574441e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6114955075472086e+01,    -1.0136587400054412e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -9.6574439525626516e+00,     3.1674303087664295e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.1404868163037815e+01,     1.5942334215476674e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.9017442111365384e+00,     1.6829671676640501e+04,     2.6632851720807460e-05,     1.7152708037655384e-07,    -8.1841397508100755e-10, } , 
      {    2.1348417297640605e+01,     2.2618285375589985e+04,    -3.7525645507976523e-05,     1.6981083994542605e-07,    -8.1152962080925225e-10, } , 
      {    1.8992859060197276e+01,     1.4289593950864859e+04,    -1.8983839614932478e-04,    -8.0600274484460370e-08,     3.7480945686578220e-10, } , 
      {    1.7463100270265848e+01,     4.0809395715967970e+04,    -2.2742270411271555e-04,     8.6064124670564825e-08,    -4.2193334666962924e-10, } , 
      {    4.0976002904211519e+00,     1.8307901964184161e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.1335441220663371e+00,     1.4075445144327368e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.4312035887485795e+01,     9.2632789305356218e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7100647335511038e+01,     9.4236431171922377e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.5212604854982885e+01,     7.2044599646277256e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.3700100287940199e+01,     3.3626788697806849e+04,     1.1256161641160650e-04,     4.8597238580738775e-07,    -2.3120279638432659e-09, } , 
      {    3.0092788523727901e+01,     1.4294999275532335e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8292725529552722e+01,     2.3438084362113408e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    4.7786889992516173e+00,    -9.4951256196955892e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    6.2688048502871592e+00,     1.4939039344725468e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.2979451912526214e+01,     8.4311632228033632e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.5820487694072458e+00,     8.4357690924776107e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.9707296863676591e+00,    -8.5587063207133767e+01,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -4.9096949564563843e+00,     1.5358349753472772e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.3159686035775110e+01,     8.2173577362100368e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.4829109709272409e+01,     2.7513409175061330e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.4480562191677919e+01,     7.3343463391897888e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8825544548674792e+01,     1.5143174364891243e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.7603479343600156e+01,     3.3631391097486325e+04,    -1.9206388487109000e-04,    -1.4849843052385087e-08,     6.0650191506802891e-11, } , 
      {   -8.9576568229225941e+00,     2.4602226833030254e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -6.9865867969724889e+01,    -1.2038612638932245e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.8376343494180901e+01,     1.6253294703013806e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.5322435000344967e+01,     8.3973835302977877e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8500343581275416e+01,    -2.4362035603748282e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.6373231475981509e+00,     2.3800452409496691e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    8.9017809955990810e+00,    -7.9422294106318659e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.7433062067331697e+00,     2.4524898229074166e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.6456133458226176e+00,    -9.7058290712472117e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.2237040251482693e+01,     2.3347912862778903e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.2428494727840622e+00,     1.4703752729372913e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0325366024652912e+01,     1.3800981607379144e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.8097765018306006e+01,     2.2861239572600076e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    6.2943896349379415e+01,     2.2861944204516221e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    4.9237439236927401e+01,     1.6134202313278145e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.0410608701374073e+00,     1.7424216584751543e+04,     5.4798272209654353e-05,     2.0911051567531863e-07,    -1.0467078893786770e-09, } , 
      {    8.7724378554111695e+00,     8.4526443515625924e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9647168048729959e+01,     1.9086501973020571e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9350656738217324e+01,     1.4270381900791113e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.5917606924446961e+01,    -2.1021423169031004e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.6158492586293534e+00,    -1.5348720008625182e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1794171945730055e+01,     2.6241525982759442e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.7087901001201807e+01,     1.8222907772622471e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5823689290221894e+01,     1.6415537278497970e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3221689186946136e+01,     3.1125848963470609e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -1.1235354788276220e+02,    -8.4351971261189137e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.0235403014645058e+01,     8.2419409000109699e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3792093793612318e+01,     9.5503617058062264e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0880093090591989e+01,     9.8751375763680721e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6906105045521297e+01,     5.8796720194285126e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.3320871435095995e+01,     8.9423290369922943e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.9812200311392019e+00,    -2.2533510852425638e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.1494311801912342e+00,    -1.6715756916540267e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    6.7248833695765171e+00,    -1.8477108772271974e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.1455913522592553e+01,     2.3960678419536016e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.8454254119794168e+01,     1.5919233459542078e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0176682345224364e+01,     3.2462646226890003e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -4.8219090782730945e-01,     5.4132513269340543e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5317556679553228e+01,     8.4111747178931673e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3651708716833166e-01,    -1.4673091458565891e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.0386742605977339e+01,     9.1291976216622679e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.8036302836369447e+00,    -1.2069517726565418e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.7419015741091751e+01,     1.0804211214120791e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.3322685845692959e+01,     7.0539312023571301e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.8891885116879656e+01,     1.3734858314854673e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.7998268260830272e+01,     2.2075549949172171e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.7718931004462256e+01,     2.5548304326596393e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -6.1077351356476672e+00,    -3.5437476828366143e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3225075481062902e+01,     2.3660751739143583e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3585704635959999e+01,     2.5641604331124156e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.7037467775032919e+01,     1.9577738720029192e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4016426817934672e+01,     3.0039606337614572e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    8.6217836610394851e+00,     2.3267730769450594e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.8105477584077036e+01,    -1.6700024432392682e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -1.1692462863580754e+01,    -8.4454637681878630e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.0909786469304294e+02,     8.4317351891620601e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -3.6217084916373693e+01,     8.1668287931449959e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.4455283371921041e+01,     2.3976122396250605e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1164656060514608e+02,     2.3976318499398287e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3581895588218487e+01,     2.5110036676798969e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.6545476422278799e+01,     2.2762265093689064e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.8003926252011073e+00,     1.6133849997320071e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.1025357799598211e+01,     3.1224040360420724e+04,    -3.7953959002689553e-04,    -1.6443425622192131e-07,     7.6295128996207656e-10, } , 
      {    1.8068229110619125e+01,     1.4914422324772895e+04,    -6.6090188859796819e-05,     6.3903291307047945e-08,    -3.0945657265221681e-10, } , 
      {    5.3157410697901586e+00,    -8.9570233901361880e+03,    -1.5221481827776411e-04,    -2.5045474766118628e-07,     1.1856118571533702e-09, } , 
      {    3.4379223943021515e+01,     1.4889301776018880e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.6623640285220731e+01,     1.5517609361064422e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -9.8801069977353517e-01,     7.3237097249402768e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4841484994355419e+01,     7.7375900877919539e+03,    -4.8307077592594708e-06,     6.9513264336638819e-08,    -3.8338581088460996e-10, } , 
      {   -6.5206947658887003e+01,     6.4809872610486545e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.5972787770880764e+01,    -8.4623165529875296e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    2.4622435234617495e+00,     7.3308231796090399e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -6.4267668160203101e+01,     2.3991762476112875e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.2746798792213056e+01,    -7.1273370758988458e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.6212981236040221e+01,     7.0957816212721173e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4069990400039032e+01,    -1.0047867544036633e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.1331534848253833e-01,    -8.2768857890209565e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.6981705239829434e+01,     8.1306166872820022e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9474805399457697e+01,     1.4320721165686122e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -5.7325410188877228e+01,     9.2568808095774330e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.5900085236909192e+00,     2.3032444154098041e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.3435052895485541e+01,     2.3831105331806390e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.1050503429273769e+01,     5.8740028704302367e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4363657635817518e+01,     3.8943636250174051e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    3.9908216767808824e+00,    -2.1631795859080612e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.3570251110018191e+01,     1.4325980830318489e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.5901134022446692e+00,     1.3899677105999612e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -8.5441838417459870e+00,     9.3162051823619331e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.4211854816497972e+01,     8.4323378950938932e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.0609257959616944e+01,     8.4345944201870807e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.8402945297941152e+01,     1.4811973172062790e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.0388384955148631e+00,     7.8058699180618205e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4098494246808222e+01,     1.5061881254009480e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.2485285002778269e+01,     1.4845720388625465e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.4291220236359647e+00,     6.2794753596483379e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.3278398897733510e+00,     3.0627992076495207e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.0662588160019997e+00,     5.7724908691240780e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9694160416139530e+00,    -9.2515695060783878e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.2086672507444618e+00,     7.0912391891880980e+03,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    8.1670779917038505e+00,     2.4010001875327413e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    4.3177244919761677e+01,     7.1227946438148265e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1706838172038271e+01,     1.1231013611316745e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6229437691344382e+00,    -8.6507190890057173e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.8775432418807698e+00,    -2.7914871709536046e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.5615210316603768e+01,     1.2487628781407831e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.8076584853407125e+00,     8.4346296191475085e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3830393211135963e+00,     1.6761933925309579e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.3535275757334176e+01,     1.6837408643538001e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.8246528230147909e+01,     1.8002579408774909e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.7246562228513618e+00,     1.6762381239421644e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -9.1443522147764718e+00,     2.4268179237305867e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5456417700656215e+00,    -1.6586389544037855e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.6471004167327649e+01,     2.5137249672916085e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.7601569180592904e+01,     7.1153216069384180e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.4758672973851930e+01,     1.5424861646069143e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    8.6091297815786394e+00,    -1.6935658631793925e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    2.0285974865279673e+01,     1.5616377427912539e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -5.2674082684527100e+01,     8.7891029037955923e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.9307148140474917e+00,    -1.6746158074041130e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3042875567764835e+01,     1.5505493651543489e+04,     8.8487320411486522e-05,     2.4537390028315834e-07,    -1.1675767882160955e-09, } , 
      {   -3.3421911001371527e+01,     8.3993324141794474e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1768885145369131e+01,     1.4283536096104392e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.6853841727281260e+00,     8.3539056920871826e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8369164689016415e+00,     8.3982620392037679e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3013189392293249e+01,    -8.5424986463158384e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    3.4087188524735623e+01,     1.4804604900275785e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -5.6369475974995694e+01,     7.1646450627298127e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3040697024912125e+01,     1.4234333484388702e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.0127855877889420e+00,    -1.9278929705555056e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.7043078004737247e-01,    -8.5130266231937912e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.2825171068985171e+00,     9.4547947123790182e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9369562732668015e+01,     9.0343591062371524e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.1331370827082800e+01,     1.4363165899900217e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -2.0962438176213944e+01,     1.7271020461717420e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.2345863477559817e+01,     2.4606829232709726e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.8225261653851366e+00,     8.0842160214566593e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4099065686195162e+01,     1.1725039534792350e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.4375797454544550e+00,    -1.2995853855099622e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.5691683936464976e+01,     6.1363761226716724e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.7018069231009827e+00,    -8.0661823374693331e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    3.8226478022496700e+01,     8.5529108633342239e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.6432129291536164e+01,     2.3133296325000912e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.3781716674744839e+01,     2.7108064047482203e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.1258711925859695e+00,     6.6207001694232749e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.8779321775786840e+01,     6.4654385192094987e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.1826380917905407e+01,     1.2611581708245298e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4190244798200382e+01,     1.4433573668963936e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5497601259942059e+01,     7.1828844619443480e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7873250568275356e+01,     4.0669651713677100e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -2.3273406692975406e+01,     1.6642712876671878e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.8267318635055396e+00,     7.1114303708130965e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0599010789447476e+01,     1.7078945829070486e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1566790478730063e+01,     1.6467911240791800e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.7322329688431893e+01,     7.5783361099973663e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5534599277314255e+01,     1.7201579217883940e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.2119736116739777e+01,     9.2426088529610570e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2711015285034337e+00,     6.7074506941374320e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8109024053358500e+01,     1.5404198129765631e+04,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {    7.3248963514892509e+00,     1.4843459029523081e+04,     1.2188257152256629e-04,     2.8176818458489763e-07,    -1.2862640254885208e-09, } , 
      {    2.7998462529701406e+01,     2.4224126301499378e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.0781067039614847e+00,    -1.1882516969948729e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.8681199404574844e+00,    -7.1224060437429271e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.5660704848932042e+01,     2.3963102536624989e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.1128283298226524e+01,     1.5437979557268598e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -5.4373193495622409e+00,     9.5766700964327847e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7573400243744665e+01,     1.1970654884877375e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.4492143155805230e-01,     2.3308097960880990e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.0854706440830979e+00,     7.6293717836987289e+03,     3.4048137575087964e-04,     4.6818874124513642e-07,    -2.1646009715546675e-09, } , 
      {    1.7435133797822218e+01,     2.2685859490190036e+04,    -9.4052351212838529e-05,     9.4731235810496216e-08,    -3.5348735117058492e-10, } , 
      {    1.1777275597126728e+01,     4.1924024275234267e+04,     1.4339091839336190e-04,     5.2293942899198992e-07,    -2.4858821498891458e-09, } , 
      {    1.8822854222944638e+01,     2.3350542695095086e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.0936146524552701e+01,     1.4809319656264981e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.1783796261747231e+01,    -1.6550306916695157e+04,    -6.7327229752242297e-04,    -1.0065944054036740e-06,     4.7655730412024055e-09, } , 
      {   -1.2913254593663211e+01,     1.4807370772383321e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.6856434803568710e+01,     2.2931481666578125e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.0672535836110377e+01,     1.6881602288059352e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.4161378624958550e+01,     8.7589242723275456e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.5542192466031821e+01,     8.3140214519467499e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5403269037660351e+01,     7.4433493760151996e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.7885503667711515e+00,    -9.0099586311957391e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    3.3643034280815279e+01,    -4.0408814643644564e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.8269080789256700e+00,     7.0809021991332138e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6143327819013095e+01,     1.6356018701473171e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.6475645120486828e+01,     1.5021851270369962e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7924136675326046e+01,     1.5438690902735474e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    5.5513176287278718e+01,     6.3459938135157627e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6251505655410771e+01,     2.7906430144754318e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.6838795238400600e+01,     2.3647633827944126e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    9.4568487168641596e+01,     1.5437972843717873e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -6.2894545431569483e+01,     1.5437986270819325e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.6556304119360362e+01,     7.3273000200282934e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.2433316552598257e+01,    -8.4185457194783739e+03,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    6.2424142023211990e+00,    -1.0098850040940200e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -3.4650800541274762e-01,    -8.2265465241259499e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.6453962079418055e+01,     2.3390572678734607e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.6044672827263788e+00,    -8.2212868594935826e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -2.2055148259008337e+01,     9.4769885573889496e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2567746292279878e+01,     1.6734927620301343e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.6676426456056884e+01,     8.4217259835822497e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2694188514315011e+01,     9.5349768057073252e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3744126232810839e+01,     1.5437268211801722e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -7.8677603241439471e+01,     7.0850152589339905e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4225679565346979e+01,     1.8189126345119512e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.2636963841847155e+01,     8.4557575973878065e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.7226600584548692e+01,     8.2606764536790524e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0315060065366669e+01,     3.7211877909432133e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.6824964365720462e+01,     9.0621260586441058e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.7073318720429889e+01,     1.6288455846467241e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.2517287014920173e+00,     1.5682473912109306e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.3496632322380187e+00,     1.5176955792394208e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.1977480850542346e+00,     6.4683613091755496e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9643007061664949e+01,     1.6399789534982330e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.0686186916916027e+01,    -2.6463720549355521e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.8911074888922279e+00,     5.0202303572410665e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.1696378938465521e+00,     3.9902107448917627e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.7268932119771847e+00,     4.0764540635146595e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4498331949986044e+01,     1.4602790241852997e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    7.8822356020859514e+00,    -5.0434518242195537e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.3394189759739085e+01,    -4.2641582942425885e+01,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.2346617147866436e+01,     2.3766318666035651e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    3.2066511803937175e+01,    -1.7874484941792172e+04,    -6.1141782883174396e-04,    -9.3274758549706954e-07,     4.4164102280673174e-09, } , 
      {    5.8731700529274349e+01,     2.3767023297951797e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    6.6963135432880776e+00,    -9.5507245492229649e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.7083363627780109e+01,     1.9513304370216527e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.5532811215659407e+01,     8.4334728711912121e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.1930215624544317e+01,     8.4334594440897617e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6075238417030452e+01,     1.3038384105441248e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3677929971978209e+00,     5.2773344746180337e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2784078229454371e+01,     1.5727711116427627e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.2270577555969222e+00,     1.4415719963347551e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0067652661949129e+01,     8.1901352643068012e+03,     3.9957403058506603e-04,     5.4269558649720345e-07,    -2.5674762924198806e-09, } , 
      {    8.0331885499865621e+00,     8.2592537627453003e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3852540992561824e+01,     7.1376740659537300e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.5300297586593370e+00,    -3.9367686525296026e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4346370344794394e+01,    -8.1855603039655516e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.0844714665460998e+01,    -9.5454648845905976e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    7.5120845557186664e+00,    -8.9378208158485631e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.1694813164936246e+01,     2.3505647217119335e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5662596115159996e+01,     2.3732913934856279e+04,     3.3328797699810095e-04,     6.0668614426685114e-07,    -2.8754784240287684e-09, } , 
      {    1.7695082811789089e+01,     4.0667360344412140e+04,     1.4358678312053017e-04,     5.2285216252939009e-07,    -2.4873365909324742e-09, } , 
      {    1.0730446851793737e+01,     6.5485002715698793e+03,    -6.1161548736953221e-05,    -5.6536062608907459e-09,     7.3202017710728818e-11, } , 
      {    7.1408108869795832e+00,     8.8859756941956948e+03,     3.3515361276506258e-04,     4.6942152547346178e-07,    -2.2734804280582468e-09, } , 
      {    1.5712675061966003e+01,     6.5857309000477690e+03,    -2.1840293950114509e-04,    -1.8650782312283848e-07,     8.7688250502281764e-10, } , 
      {    7.4315225487555914e+00,    -9.6901300884626107e+03,     6.4217159684198249e-05,     4.8626812215286456e-09,    -2.1671171545596207e-11, } , 
      {    1.2053501889378039e+01,     2.3896038354496137e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.6646921045444505e+00,    -8.1709475945985741e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.5806100360011875e+01,     9.6562999002286087e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9876830978080271e+01,     1.5183548117592694e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.9717171743176927e+01,     9.2763902804640220e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.3700450707609065e+01,     7.5567606073139887e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0842572770003994e+01,    -1.8050510612714938e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.9005848117158322e+01,     1.3772595673968885e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    3.7433622435215909e+01,     5.9721176681461220e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.3072276566889407e+01,     1.5665577966454694e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.1411060318235528e+01,     8.4308126418426782e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.4091648768534085e+01,     8.4361196734382956e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.3839260804377087e+00,    -1.6764299820635235e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    9.3119386979636616e+00,    -1.3200469497962970e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.9362335246391368e+01,     7.1335610061529542e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2221123808748015e+01,     9.7027005660539508e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.3446818688694577e+00,    -7.6812056457574763e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.6288728176366490e+01,     2.2468570918865160e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.0234279731245707e+01,     1.5536195334584230e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5431920640940215e+01,     2.3191332494159375e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.4111800564585902e+01,     5.1697389653045479e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4211843713481016e+01,     9.3898616896892727e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.1514973015644998e+01,     1.4688020245225322e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.9854535098356045e+00,     1.6779018329059796e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.0568689109931237e+00,    -1.0988713399645342e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.7941671114631944e+01,     9.4970402926369607e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.5852128286274230e-01,     1.7056403923939426e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.6000086052085443e+01,     7.9246032782886805e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1666651349050787e+00,     4.9426780962289431e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5965119687930908e+01,     2.2421943860909755e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.1173185574735587e+01,     7.4771308035181637e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2290803576336623e+01,     8.3790661676265463e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1709965295128974e+01,     1.6275836608144862e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7425822763913267e+01,     1.7948882974436729e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.9519761672951024e+01,    -1.6554909316374626e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.4336721443886979e+00,     7.1404397277302523e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2911950477914825e+01,     3.1976325252669252e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -2.8002642996044898e+01,    -1.4669972778375950e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3861805849766339e+01,     8.4280706635477800e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4984895788094747e+01,     1.5585172216070350e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.2099878424877680e+01,     2.3969661492224714e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -3.6208066593597380e+01,     7.9063638790741461e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.8214087967903758e+00,     6.6736692666344670e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.3047034959587094e+01,     8.4324844958247577e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -5.6302845449116276e+01,    -8.4344478194562162e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.2707854385868547e+01,     3.8732932798622423e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -4.1544478770228672e+01,     9.0893827197643568e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.9423999583946969e+00,     1.4979406536155864e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.3252686221316834e+00,     1.5347808057934095e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8360121536090734e+01,     1.5988734881302018e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.0831955828359686e+01,    -2.0623274150051884e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.8575322515673474e+00,    -1.8947160865845467e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.4145050195537259e+00,     1.6176509813275774e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -5.1806175328579386e+01,     8.1607953187500498e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2132056489074884e+01,     8.3865668189380667e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0934454904556251e+01,    -2.4333586463170368e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.7874079077709887e+01,     1.5642597990943399e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5065033098921990e+01,     1.7279131478896066e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.6597693729591001e+01,     2.3227380910561780e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.1312356590262930e+01,     1.6273535408305128e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -4.8257148867099154e+01,     8.7553214762926291e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0291218328906767e+01,     1.6664386430929095e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.7820870345974775e+00,    -8.5785812736585394e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.7875207599584257e+01,     8.5508601288782265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6293594075984746e+01,     3.1101265799669680e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    9.7557080170690469e+00,     7.1054853629341460e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7748866332235853e+01,     2.4945733891072687e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.4408935640919616e+01,    -1.1833548757792441e+03,    -5.2529319941377664e-04,    -6.1838954652883535e-07,     2.9213417982617307e-09, } , 
      {   -1.6390418926483726e+01,    -1.2282367627073472e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4298139584738376e+01,     7.2850561708712276e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.6881948131239437e+01,     1.5656165521482748e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4048894501456139e+01,     1.8163389195597676e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0927101419182259e+01,     1.6238600349869914e+04,    -1.2794465755047585e-04,    -9.9435285995565831e-09,     3.9706240482870959e-11, } , 
      {    1.5079579749169090e+01,     1.4357168065464910e+04,    -2.4636510185418678e-04,    -1.5567987861939023e-07,     8.3285172650444943e-10, } , 
      {    2.5433387361465353e+01,     3.0528224310897349e+04,    -3.1521710457047626e-04,    -9.1116561966879748e-08,     4.6968264612210718e-10, } , 
      {    1.0282218193971984e+01,    -1.0436108734883783e+03,    -5.5631836612270028e-04,    -6.5526932325083769e-07,     3.0966504253509390e-09, } , 
      {    1.7206898874730911e+01,     7.1130909021527978e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5748518543570039e+01,     2.5702235911327502e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.0315849848671421e+00,     1.6080880900108879e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.3081516567039850e-01,    -7.4378747524237897e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -3.2132156382610404e+01,     1.6204490986774390e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1069742723868600e+01,     1.9479522942713560e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8687992673166921e+01,     4.0058264809440348e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    2.2751591174506075e+01,     2.2494308068386999e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    4.8926265819285589e+01,     1.6205195618690543e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.8739224514792059e-01,    -9.3745936674784698e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    9.5838378414137200e+00,    -8.6043184231803734e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -8.4760236575288737e+00,     2.4190836985602436e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.1020183738571543e+00,     8.3886641034151580e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.3529296104360395e+01,     1.6901842993350670e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.0570438099127202e+01,     7.6519324393982424e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1882416299255993e+01,     8.4533651728155892e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9237044915402169e+01,    -1.7879087341471641e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.2212647717906854e+01,     1.7142060640859145e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0477166954757685e+01,     1.8488173045868669e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.2233053254526958e+01,     1.6608931449168911e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1229338401917175e+00,     8.3275279632181046e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4385796982581787e+00,     7.0035814788410278e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3290055271136280e+01,     6.6928569363426941e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7590003054324708e+01,     2.3899779204365776e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.3049653851779444e+00,    -1.8139294497242331e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.8239993987034300e+00,     5.1441832840785346e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7040484503245132e+01,     1.2880649751100818e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6656296699045825e+01,     1.6381755850995003e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.2298817945447986e+00,    -1.0267145730451157e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3136105917854497e+01,     2.3975509102357570e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.5365695267908947e+01,     1.6156391902451127e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.1034566735091712e+01,     2.3749750301184940e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.5477458303251002e+01,     7.5798613632485831e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.9184548286069820e+01,     9.4572253907390477e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8927421586766172e+01,     8.4821680233055649e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1186549825441681e+01,     3.2227359611537449e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    4.0354158651228452e+01,     1.5652460055255240e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.5182152630677335e+01,     2.5372536288268639e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.3993586926996450e+01,    -6.6864796814823512e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.6240907850667687e+00,     9.0191139267303261e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1637172185713352e+01,     1.7965803655245505e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.8841997205086940e+01,     1.1197232183813778e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6857790122031240e+00,    -7.7015473011866043e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    7.3343336351219923e+00,     1.7390465167411156e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1463925071912055e+01,     1.4023630032837080e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.5742952138422332e+01,     2.2705011519938624e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.2674743173728842e+01,     2.2129673032104271e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.7146954920348044e+01,     7.0137281586656818e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.1480609938931492e+00,    -6.1028337684433814e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.0281595296997935e+01,     2.3976931793291322e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4347599156660650e+01,     8.4071628967065790e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.7671261867142896e+00,     2.5011219203294913e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    4.1151292729791884e+01,     1.5437650924792148e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4713568420727015e+01,     3.0890562348118052e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    7.6323662466354207e+00,     8.5639780400776817e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1242174520889611e+01,    -2.2641420734279750e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.9965301355227050e+00,    -1.1844584031716679e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.4774105867550169e+00,     1.5438308189745047e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.7808674709201590e+01,    -7.4121810605658084e+01,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.3219707257813493e+01,    -1.8450810449110131e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8349957730061433e+01,     2.3941727674854603e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    6.2827698800171801e+00,    -1.1326527674674985e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1013312130976283e+02,     7.1087801740054365e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.8945347185948222e+02,     7.1097960910815036e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.7215001485702786e-01,     5.8838245576391664e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1675145749341942e+00,     1.3506656136306625e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1251844506358310e+01,     2.2063549739579801e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    7.4588667757840437e+00,    -2.4422501555411727e+03,    -1.7998111590363756e-04,    -2.1971406962033772e-07,     1.1401266375916734e-09, } , 
      {    1.2783517208753683e+01,     8.3996490901778816e+03,    -3.8323891233923371e-05,     3.3162613266199370e-08,    -2.6397135309052015e-10, } , 
      {   -2.3802381847360380e+01,    -4.8451400281340284e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.9786421596293424e+00,     7.3783561753989925e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.1521181716813693e+00,     1.4763271306397710e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.2784739121953281e+00,     8.5394043520628693e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.1879490022709540e+01,     2.5170478810886565e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.3982506843035262e+01,    -1.2363239729904353e+03,    -5.2529319941377664e-04,    -6.1838954652883535e-07,     2.9213417982617307e-09, } , 
      {    8.3600016658937424e+00,     1.5191242573468792e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3369875430928202e+01,     2.4211507063177000e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.7153963186375123e+01,     3.3419188115857789e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    5.3967883547612942e+01,     3.3419892747773949e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.3127893187580767e+01,     1.1624034581009731e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1230786476753650e+01,     7.5507271329190407e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8862401897919383e+01,    -7.7796429180608561e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3578374598014744e+01,     1.6904144193190405e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.1741649379138046e+01,    -8.5225084064082548e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.0741734255753869e+01,     1.5472727387219240e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.1826103830043522e+00,     1.7644896607087059e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.5736878320595480e+00,     6.4617928777897014e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8485838152880056e+01,     8.3444239088727172e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2789836846524677e+01,    -5.1091361380780290e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8534828384963028e+01,     9.8166640868852246e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0165678765502079e+01,     1.5418791887560372e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.2796534834130586e+00,     1.5391578891443252e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.0566661178017760e+01,     2.0337969540375989e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0187137776643366e+01,    -8.1775160259844215e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    9.2363750006641965e+00,     9.6371122305203826e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.6397405931439579e+01,     1.4810024288181130e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.7371017543605820e+00,     1.4416651002530067e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.8459856482622662e+01,     1.9970759689722181e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.0929333708509263e+00,    -1.6899765228867657e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.2661902126924858e+01,     1.7167797790380981e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    8.4285770237580920e+01,     1.6762150868814890e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.3177262140148343e+01,     1.6762164295916336e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3962833802030959e+01,     5.4419636843369153e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.2304625212801877e+01,     1.2817600992106227e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0567700432162290e+01,    -1.6907272698383666e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    2.3625941807611547e+01,    -1.2107667937982260e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4038491075793416e+01,     1.6746409838849977e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -8.9523628965332005e+00,    -4.3297896222225891e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -9.0670946344307701e-01,    -9.5879096188046933e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    5.5102533933404114e+00,    -8.5033559356604583e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -4.5938806550741106e+01,     9.1231641472673200e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4595484119513380e+01,     1.0600076193955016e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.4168910269514683e+01,     9.0614214267279549e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8498609388453154e+01,     1.7061878547530840e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.5307119734138914e+00,     8.4121362496626352e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0861410758611404e+01,    -5.8374067090016979e+01,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -3.3063445025010640e+00,     4.6272093875480796e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8995853448559668e+01,     1.2363675854570363e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.7363644213822411e+01,     1.7383906211811427e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.7497095449137752e-01,    -5.2353285213018239e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.3957041751168575e+01,     2.8299451114447307e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.1518361677325002e+00,     7.2195108035051035e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6023534042360559e+01,     7.1219073708658507e+03,     5.9687642424328090e-05,     1.4274369212908067e-07,    -6.7810889576790794e-10, } , 
      {   -4.0427010539869016e+01,     7.8048062566368690e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.5319859598890044e+00,    -1.1392211988533463e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.2659489672389617e+00,     6.9042250200520039e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.3395227473664118e+01,     3.9518622422050321e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    6.7663608799432509e+01,     3.9519327053966481e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -4.6573904793577636e-01,     5.4934334202623631e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.5290271197127634e+00,     3.2268863456147166e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    3.8663279623500721e+00,     8.4547960656183386e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.5637968230670204e+01,     2.2799235081529183e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.5388510754212192e+01,    -6.6244132850658241e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.5670949555576883e+01,    -6.6173669659043264e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3322929471342398e+01,     1.5412242407746764e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -9.4379032122454483e+00,     1.7118760730968395e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8507301885839247e+01,     1.8615928742315460e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.3606883389358700e+01,     2.4464842621884931e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9355651293962605e+01,     1.5771481949936786e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.5518383891831512e+01,     2.3971289415668525e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.0262897982287999e+01,     1.5418801363346493e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1189575109669534e+01,     2.3471865789616368e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.8364987054147557e-01,    -8.7462860824329255e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.1727106415659605e+01,     2.2599073325516238e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -5.2046500273918230e+00,    -1.8875257791251097e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3348430368128923e+01,     1.6790154701237476e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.5204124557908834e+01,     3.1038273744412090e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.1199073465848656e+01,     1.5431420601668866e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.4382634145722200e+01,     3.2304911872549572e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3273649945023910e+01,     8.4345696850329095e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {   -3.1758743806299684e+01,     1.7039130927172569e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0783575794473837e+01,    -1.2052247643099963e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.5154841295613348e+00,    -1.0352189090848538e+04,     9.7710343158862150e-05,     4.1213332291968094e-08,    -1.4108562933968601e-10, } , 
      {    7.5787854816786533e+00,     2.4270466789398262e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.6866447948779832e-01,     5.8864543899553501e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0917792814711731e+01,     2.3963768928299331e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.1816803079634095e+00,    -9.1655382182316987e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.7488512349793925e+01,     1.5528084317405584e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.8704793080470729e+01,     8.4331473540894713e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.1960603569999908e+01,    -8.4337849611915026e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0893969977360873e+00,     7.3307668816384307e+02,    -2.1899792709128676e-04,    -2.5474466999985210e-07,     1.1521160799255122e-09, } , 
      {    3.9990308474735059e+01,     1.6278137807984596e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -3.3244038402754413e+01,     5.8523206464943114e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.2709628582778684e+01,     2.3981151479980366e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1885607894320660e+01,     7.2070592839799911e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2235343747821439e+00,     9.4782049388868090e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.1069448037066181e+01,     8.4330016957036969e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.2215889862460090e+00,     3.2305240505026024e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    4.4407153867268363e+01,     3.2304583240073120e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.6610562715655870e+01,     1.6242049550844835e+04,    -2.5425881396932824e-04,    -1.5387433550820208e-07,     6.6880532122741591e-10, } , 
      {   -2.1404415106138153e+00,    -1.7843004714128940e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.6861938833629782e+01,     4.0599821869771731e+04,     2.0534331944762507e-04,     5.9674261566729451e-07,    -2.8357721835458977e-09, } , 
      {    3.3248567119748273e+01,     1.3004602677938285e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.8671965070901415e+01,     8.4336156434244785e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.5416154581372233e+01,    -8.4333166718564989e+03,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    6.8425826346687568e+00,     8.7854052106579616e+01,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    3.7813596480268515e+01,    -8.4339306195772733e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    8.9775479487477803e+00,     1.7347805351455452e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.3767852910228378e+00,    -9.5473833714399043e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.0017885432856014e+01,    -8.4323626302480643e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {    5.6283273670816527e+00,     7.8605141517209540e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.8809660474838896e+00,    -9.5488060623736565e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.2465859560544153e+01,     8.3782053363756804e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.9225405038420142e+01,     9.1690125235601790e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.8311320036578067e+00,     7.2048481064212619e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3027909058578695e+01,     2.5013296746078613e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    7.0796035993508983e+00,     5.4596519927593981e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0921354601240665e+01,     2.2168849953699937e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5124838485050079e+01,     2.4056402541152755e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.9322829420535967e+00,    -8.5447998461555762e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.4889021412872570e+01,     2.5450088549280765e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -2.0575543381977411e+01,     7.9123973534690940e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7064199251723721e+01,     3.2382464133561694e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -4.3087487145724239e+00,     1.5629480079743942e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -2.6283802820601703e+00,     6.6260956635159819e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.9531651769197090e+01,     6.9500285249518238e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4719000070793360e+00,     6.9051531123776967e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3718669827009199e+01,     6.5346609901759966e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0234751456629578e+01,     3.2827281263172728e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -1.2108557884139573e+01,    -3.0393930246285390e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4173701032934039e+01,     1.0819506591557531e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4945617867104191e+00,    -9.1212923197108285e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.4616769865349220e+00,    -9.4083750949814330e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -4.9469287817025055e+01,     1.6749951657546029e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.7635695055388240e+01,     3.1756234091299852e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.1374383065045436e+01,     1.4716371967695293e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.2541858262651925e+01,     5.9166626863859401e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.5462692323344612e+01,     6.6800550569874904e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.0260004180891542e+00,    -8.3095132308030188e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.7210766691253344e+01,     1.0049320869921597e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8726110717548451e+01,     8.5170787013752597e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.3373027667852261e+01,     1.4831001880200907e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.9371922245059441e+00,    -8.5099074010991553e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    4.5689038496464164e+01,     8.4483865958025999e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.6664185772341312e+01,     6.3858087154136747e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7669343016884632e+01,     9.9859805717152303e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.4168048719250663e+00,     1.2395292683746743e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9934755457458579e+01,     1.6322237273970204e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.8272242779478667e+01,     1.5456003765469801e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.8960809383039565e+00,    -1.6901877204290959e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    4.5092688034324986e+01,     1.6587355946485535e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -4.0319178888696676e+00,    -1.6212407305876945e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0386747527332080e+01,     6.3854205736201384e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.7032454473219900e+00,    -4.8087303617447878e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.1835867752108832e+01,     2.2649412590411244e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.0791842904549220e+01,     1.2964568803167022e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.2735254787980326e+00,     5.0666310230664103e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.3956292754148913e+00,     1.6133855627117131e+04,     9.0955337177226771e-05,     2.4484477463159538e-07,    -1.1116826189209769e-09, } , 
      {    8.6358648191169340e+00,    -7.7587579067696006e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.4549519190188583e+01,     8.9302211637465462e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1206005304225457e+01,     1.7356683739908192e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.1021757775884371e+00,     2.5168401268102865e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.7845034316671082e+00,     1.5692410996944500e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.5920158246782862e+01,     2.2654672255043617e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    3.3712622287800450e+01,     2.2691857324625344e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.7357577669344710e+01,     1.7605081705189150e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3753270395506732e+01,    -7.3814938990817336e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.9090508086963073e+01,     4.0633250981316618e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.4112876777784949e+01,     2.2612227520829518e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    6.1987231768767472e+01,     4.0633955613232778e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.0791409088336277e+01,     8.0092935927607195e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5908670179165600e+00,     1.7783486137104144e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.6111816387601998e+01,     7.5905420348169528e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.7388596272648552e+01,     1.4750153395198255e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.4315115822763760e+01,    -7.5829898684418431e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    3.8130836958323961e+01,     7.2625142657401730e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2580411050292696e+01,     1.4481085352342736e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.0376266103233700e+00,     1.6864329291292495e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.0428544330941861e+01,     7.8908218507857164e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2883595050040984e+01,     1.6651934911403980e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0030908395075297e+01,    -7.4716561799267547e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.3583662219110582e+01,     3.0667913922660115e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    3.2937087296749993e+01,     1.5081041214918307e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6811995020816141e+01,     1.4323350998002305e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.9574664958933781e+01,     1.0476801936463426e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -7.7888368665162730e+01,     1.0478144646608735e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.5232905134563953e+01,     1.6430771841843464e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2751989904031568e+01,     2.4152012866517722e+04,    -5.3204820539541208e-04,    -4.1475810418920804e-07,     1.9507448086804401e-09, } , 
      {    1.6804673058884141e+01,     8.7953355321466934e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.0304267533625893e+01,    -9.7579964986955765e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -1.7698822886035060e+01,     2.4460240222205466e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.3480238803146223e+01,     2.5020959229070766e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.0755515334102739e+01,    -9.7572918667794274e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -1.5969973112726338e+01,     8.4331809299541455e+03,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -1.9225783602255518e+01,    -8.4337513853268283e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -4.6362336373310775e+00,    -8.4388958626734784e+03,    -9.2625108217020158e-05,    -1.0766742230080575e-07,     5.0823018190712651e-10, } , 
      {    2.7490387774494010e+01,     1.3431405075134237e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1201453370679083e+01,     7.1087358430109889e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.5662372484157718e+01,     3.1712652703906435e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -1.0522046077520736e+01,     7.1098404220759548e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.3563663551753518e+00,     1.6100068569817106e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.9617211510991055e+01,     1.5959048361439991e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.5099518473386020e+00,    -6.3486654064527102e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.8632845917546369e+01,    -7.1054041282984781e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.1443500387754710e+00,     3.6834330938216735e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.3910689760161592e+00,     6.4596506395200768e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.6767793360135261e+00,     9.1235522890608554e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9726809639779020e+01,     1.6450638244024944e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.3682529249322037e+01,     1.0095721535746939e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2057590026683448e+01,     2.3373650012300735e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -1.5426152539258698e+01,    -9.7599453825772362e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {    2.8547635617272196e+00,     7.2298200847605913e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.5877367034707888e+01,    -9.7553429828977678e+03,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    3.5427267112113661e+01,     2.3994269391179820e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3654760806395640e+01,     3.1032548958942843e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -2.5960058559676753e+00,     2.5153205814119709e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.8154290930587123e+01,     8.4000370460955983e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3300649754023070e+01,     1.4470543656804057e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.2148610389149894e+01,     3.0601790630135645e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.8505976569285728e+01,     1.9906325339909512e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8966114844446821e+01,     2.4570740975569963e+04,    -1.0194606332797995e-04,     9.6536778921684358e-08,    -5.1753375644761854e-10, } , 
      {    5.4822639130415691e+00,    -9.1550737472137916e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    3.7811561814859811e+00,    -2.6030239884801176e+00,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3848933512767992e+01,     1.4985440010550810e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.8465166882846955e+01,     1.3273670720793807e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.7680210569760170e+01,     3.1676251971545953e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    4.7486802544517673e+01,    -1.3942049280617330e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0999342998205819e+01,     3.9125953768315412e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -3.6264618877548571e+00,     6.4749470731029805e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6294334332232374e+01,    -1.1458069627807777e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.7703217182355690e+01,     1.6936959218245691e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -3.0023116700451503e+01,     8.6082677935205666e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.3836507428007167e+01,     8.7492880018976794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0551961742429315e+01,     1.6254506700167010e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.1662532509018654e+01,     2.1710914912889446e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.7490160151676315e+02,     8.4194516878310969e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7164579102723400e+02,    -8.4474806274498787e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.6555162821373429e+01,     9.5368551945348718e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9623699841350234e+01,     1.5711963372911987e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {    4.5259409763146145e+01,     1.4844419802173697e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.1362821060340369e+01,     2.2101287098694014e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.3269325129561924e+01,     1.6049366461505360e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.6078800536933247e+01,     1.5560778498385162e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.0374215619398671e+01,    -4.7228105919155769e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.4840821041422629e+01,     9.4234439632360845e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.9542571229622556e+00,    -8.8124093749573967e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -3.4480845508783453e+00,     8.3985801384082042e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6393702096535446e+01,     6.1867342059241128e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.6251325550847078e+01,     1.6627170848383445e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.4691428244985962e+00,    -1.0332982670571853e+04,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -5.9416259778359915e+01,     8.7155065743947162e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1998643024317797e+01,     2.4728480959707456e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    5.8325143714668442e+01,     8.2646979961553552e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5955056850040542e+01,     7.7774106194869273e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.2138554307242657e+00,    -5.9109570713611106e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.1501635604032234e+00,     8.4308631336520066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4643481526075796e+01,     2.5716855392296551e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.2491907305630477e+01,     1.9872543912406549e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.0559618497067778e+00,     7.8478183885506469e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6542479851961680e+01,     2.4433675767330096e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.3504621732503574e+00,     7.4896898639140800e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.1757444024402293e+01,     2.4531233527617856e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    7.0551282070754899e+00,    -8.4684110466504735e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.7605021077253433e+01,     2.3173111226898822e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4541299611486897e+01,     1.3964111455812279e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.5913627293185179e+01,     1.6761798552856810e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.8522037969937017e+01,     1.6762516611874416e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.9411678450254009e+00,    -1.6429330994015925e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.0977871637136346e+01,     8.4311666927981405e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.7375274780255324e+01,     8.4357656224828315e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8413821875951871e+01,    -9.5082798150088674e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.7023519267867076e+01,     3.9847561357888721e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    5.5825346129671134e+00,     4.7549650839948736e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7315442659277505e+01,     1.7111025923560464e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.9169445998866974e+00,     5.1004124505693744e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5567826109780690e+01,     9.1519452420205325e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.5648475639366140e+01,     1.8582147314812501e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.6298730560137358e+00,     1.5986657338518318e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -5.7115479528473916e+00,    -7.6354240086621339e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    7.0072651755075519e+00,    -1.4481309519344827e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.6732528938449214e+00,     1.6990885242118442e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.9546080405241682e+01,     2.4535835927297328e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3036807787594235e+01,    -8.4496754929823910e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -3.8861734370147019e+01,     1.6897144316347782e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.1632873712736451e+00,     9.1147428398972206e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -5.7109739563171679e+01,     9.0833492453694071e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.8909168215908458e+00,     9.8709164380065995e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0010449680596802e+01,    -1.4546899075342099e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.9529518465432218e+00,     2.3958171504469068e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -1.6142355138962834e+01,     7.3576860978244276e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.7673650929414340e+00,    -7.2881846760644894e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -4.7241024211966387e+00,     4.2341884178550927e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5843034851330785e+01,     2.3740933832471892e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.9559396775620046e+00,     9.6125290667194495e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.0599141255889721e+00,    -9.5480947169067804e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.6851782642110265e+01,     7.8055108885530180e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.6527724707024589e+00,     3.2225282068753746e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.3380909928389155e+01,     3.9552731785145850e+04,    -2.2722683938554726e-04,     8.5976858207965108e-08,    -4.2338778771295790e-10, } , 
      {    2.3077833312812228e+01,     2.2199532886172223e+04,    -4.6752985521182448e-04,    -3.4152767639676622e-07,     1.6560217237971416e-09, } , 
      {    2.0868016201773120e+01,     1.4985410000388234e+04,    -2.5416088160574410e-04,    -1.5391796873950193e-07,     6.6807810070575158e-10, } , 
      {    6.6288960999581477e+00,     1.7005464095333780e+04,    -3.7520593749419359e-04,    -3.0222800066687362e-07,     1.4208434552277168e-09, } , 
      {    1.6519718147641910e+01,     3.2899456780660606e+04,     5.0268754949784173e-05,     3.4699152658287062e-07,    -1.7031456136009889e-09, } , 
      {    1.6374164985828966e+01,     4.1714474809403546e+04,     5.7605900959011838e-04,     1.0336615532200195e-06,    -4.8989937662437496e-09, } , 
      {    1.8669807939735289e+01,     2.2895348935695598e+04,    -5.3185234066824381e-04,    -4.1484537065180776e-07,     1.9492903676371108e-09, } , 
      {    4.5780914401680022e+00,     6.5332382992422436e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0097575146233110e+01,    -1.0691697118498558e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    2.3630325548257375e+01,     1.4245798736990178e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -3.2416586272869147e+01,     2.7957636879543952e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.3097908863037091e+01,     2.3801418811944364e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.0746270483739096e+01,    -1.3388479978753910e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0715569378000293e+02,     8.4347357470990773e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0389988329047375e+02,    -8.4321965681818965e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.5343231220916071e-01,    -1.3095747915161573e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.6516058742507465e-01,     4.5496571265359553e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.1451453473567369e+01,     1.6789766559443939e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.3006776320673502e+01,     1.8176507106797133e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.7723609383676038e+01,    -1.1059920608828652e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.3936204951282356e+00,     4.8079341812060647e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.0387226853484911e+01,    -7.0026902964717920e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.5594463993960304e+00,     3.0978432647612826e+04,     2.3627055379296457e-04,     6.3366602562059676e-07,    -3.0103535901134417e-09, } , 
      {    1.0541142542391340e+01,    -8.4388616517331939e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.3379459443371463e+01,     6.2174862944581560e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.0206082225986847e+01,     2.5179866520704338e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.3692260601893071e+00,     6.6360850695221770e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.6593368342094358e+01,     7.2685477401351209e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.0264394643206511e+01,     1.1590253153506766e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.2701652664895050e+01,     8.2586645217604091e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2994725300026600e+01,     3.1283583317811041e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -3.3190475870091689e+01,     1.0513376242416224e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.1160039809291156e+01,     1.0441570340655937e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.9051485803115987e+01,     2.4082158642246828e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.9831860187357382e+01,     4.0698037647087367e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -4.1178800512651897e+01,     8.5684528916226536e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.1698753138162497e+00,    -7.6731613677763462e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -1.3803534475594823e+00,     8.4280364526074936e+03,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.7359680698439540e+01,     8.1801305989325574e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    7.6580787308030480e+00,     1.0159481621143543e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8650606784259843e+01,     1.0442341839614583e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6569002628947146e+01,     1.0547936483151057e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2304130426079047e+00,    -2.5107769687899523e+04,    -6.1141782883174396e-04,    -9.3274758549706954e-07,     4.4164102280673174e-09, } , 
      {    4.4221574320425807e+00,    -8.4154324736531598e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -7.8599267084848123e+01,     1.0407010099921104e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.6514017149234537e-01,    -9.5415357613070519e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.1834699197843761e+01,     8.3841427018490940e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3767417208352310e+01,     2.2980629042607747e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    2.0418343019194310e+01,     1.2756696824263352e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.7449906838218237e+00,     1.3113635166613638e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -4.7375985366414511e+01,     8.1270138912470848e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.5379861525616818e+01,     2.8692472084140296e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.5471990720281166e+01,     1.8556410165290661e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.0292083439928696e+01,     1.6829581461341848e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    5.0124693808474383e+00,     5.8864656495494719e+03,    -2.7766297625873455e-05,     3.0740678040848568e-08,    -4.5485219561696674e-11, } , 
      {    3.2547765775696924e+01,     3.9485193310505441e+04,    -1.6547030305845238e-04,     1.5986731134586947e-07,    -7.7182338032638137e-10, } , 
      {    2.4549511688455016e+01,     3.1642828489798121e+04,     5.5694450299185329e-05,     3.4571510912324541e-07,    -1.5949933776190734e-09, } , 
      {    7.1675640039092290e+00,    -1.3951713305094115e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    4.8744101221333736e+01,     3.0983035047292291e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.9132057119444383e+01,     1.6869396363239764e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.7791804297801917e+01,     6.4256236173115867e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.9856909816000993e+01,     8.4239576037470051e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.7953317838830372e+00,    -1.1091806212200220e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.2987538576644361e+01,    -4.1629407125603211e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.0673945431937540e+02,     1.6760426613887183e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.7270637907746860e+00,     7.1437809055133130e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6755489166836025e+01,     3.1243252410494468e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -2.9432845118444009e+00,    -1.6742098277988860e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3741696684592654e+01,     7.1195974137989597e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.7785006407835823e+01,     5.8530252784104605e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.9399039167266272e+00,     1.5313467779627592e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.8988392574501276e+00,     8.3364063516708429e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4384694227448996e+01,     6.4706712662424407e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4548696467265845e+01,     1.2017055550702720e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7585501606737839e+01,     8.6284123898903508e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2643474487459375e+01,     2.3945068852637665e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2489109012606921e+01,     6.5449702714314835e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.2563370414635351e+00,     7.7349658852728307e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2473966841269114e+01,     7.0460097540767929e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.0615473080789656e+01,    -8.6868017163484164e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -6.7161462924528088e+00,     6.7354119871738321e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.6894147153149177e+00,     1.6153037667028297e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.1900250066347454e+01,     1.5512542289117153e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.4435009517456137e+01,    -2.2325303616141900e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.5721714267987323e-02,     1.4797052733900677e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2906121084245777e+01,     2.4583198124892137e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.3057344725073143e+00,    -8.2047290221015392e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -7.5056004421502109e-01,     5.4908035879461786e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.9320465932661843e-02,     7.4121376029019566e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.3576062594106148e+01,     2.3976146689752750e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.2364227912085376e+01,     9.7912866342107754e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2612675500465361e+01,     3.1137314216072082e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -1.1988881489701946e+01,     1.6407759843446111e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1453573952817631e+01,    -7.0786352102337678e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.5648951870953798e+01,     9.7828826593822596e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9747895990435939e+01,     1.5640970067499587e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.5313346764072756e+01,     4.0240582327581709e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.4383991851095425e+01,     1.6022516308804981e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.5333346391237967e+01,     9.5308217201399239e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6925758823532917e+01,     2.5190175801959827e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.0045376807905335e+01,    -2.3963886584769161e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6204488085019815e+01,     2.3314131435275936e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.5665594954300213e+00,    -1.6402918040175588e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -1.6018659806205793e+01,     1.6753297426926150e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.3183324595552211e+01,     7.0989788512879841e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.3985493969416321e+01,    -1.6771017737805076e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.7392163008273869e+00,     1.6047099472605914e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.6369731690744889e+01,     3.8368297762339702e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    2.2313143220107428e+01,     2.5338754860765672e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    2.3731601157271758e+01,     2.3045063392420419e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.5140403220186514e+01,     2.3856775742130711e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.3219725450754019e+01,     9.0788395384098294e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.3856825295006082e+01,     8.2984794236583202e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.1267731411496499e+01,     1.6762862214281766e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.3876846702858700e+01,     1.6761452950449460e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.4641688660175710e+01,    -8.9716022433515263e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    5.1546155832618339e+01,    -3.0212001066805897e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2650401862966772e+01,     2.0698795813802981e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6638523973293832e+01,     1.5459309465246450e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.6835346550801400e+01,     3.4910678069698406e+01,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2497439436842562e+01,    -2.5084815532695789e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {   -9.0556878448855027e-01,    -8.7746720158431854e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.5420324798050698e+01,     2.2303174081532579e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.3625699946483447e+01,     7.3165363985344593e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8511892399768392e+01,     1.5220726625903366e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    8.2246921937831754e+00,     9.1419558360143383e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5772283102224904e+01,     5.9593225023416444e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.8579133295341421e+00,     1.6063985942474406e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    2.8017040751931219e+01,     3.2291793961350115e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.3824968668418876e+01,     3.9841710225616225e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.2004588967834813e+01,    -2.0225125131072764e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8742983843367785e+01,     1.3119704925122409e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.2933590275421238e+01,     8.5191294358312589e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9918445368459018e+01,     8.6069329961285111e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    8.1891040412924134e+00,    -8.3896953241313277e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    5.4994746070473735e+00,     3.2893404555697198e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.2538016689137379e+01,     9.3914544141726510e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2591855954905103e+01,     2.3864886759309356e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3554771802214352e+01,     3.3026519462122880e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.3350166701057196e+01,     1.4775920554882670e+04,     1.8363910784966118e-04,     3.5565863772280199e-07,    -1.6346996181019445e-09, } , 
      {    1.5382654968482615e+01,     3.1710366964438530e+04,    -6.0620860279095789e-06,     2.7182465598534105e-07,    -1.2465577850056499e-09, } , 
      {    1.1339999833941269e+01,     3.3662846944784033e+04,    -7.0580436211497118e-05,     1.9859422819289922e-07,    -9.5183470012235207e-10, } , 
      {    1.6794832479565411e+01,     3.2129119453856300e+04,     4.2394212367593837e-04,     7.8316317232753324e-07,    -3.7141091296120446e-09, } , 
      {    1.6869292265456377e+01,     5.8188671545839024e+03,     2.8858340442572686e-05,     1.0577664894447856e-07,    -5.0425470972202832e-10, } , 
      {    1.7050202204572685e+01,     2.3385149121053852e+04,    -2.8478692545169432e-04,    -1.2247363212189098e-07,     5.6960759393559371e-10, } , 
      {    6.6101343276014815e+00,     8.4706967861183766e+03,    -2.2126268572122184e-04,    -1.8580416454607613e-07,     8.2389721781435643e-10, } , 
      {   -5.4387584719911102e+00,    -1.6552608116534891e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -1.1796509830131191e+01,    -5.1158315616085647e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3643210215302021e+01,     7.5257193287442497e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0657691535085723e+01,     3.2844201943981519e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.6626489184181541e+01,     3.4968483441341950e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -4.3617148876044467e+01,     9.7514717323128625e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4482181310718623e+01,     7.1725665110101509e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.9882392322964311e+01,     2.0363780659415170e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.8589178536858579e+00,    -1.6480470301187714e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    2.9567484242937056e+01,     2.4542171225841019e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    6.6612799506433804e+00,     1.7390459537614097e+04,     8.5627574191409736e-05,     2.4607755885992074e-07,    -1.2205620754245567e-09, } , 
      {    1.2209690928559507e+01,     5.8533843079192620e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.0881331296934441e+01,     9.2365753785661091e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.3531149157438795e+01,    -3.4324139943215259e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.3419890598726905e+01,     2.5121501929400445e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.9967672684354849e+01,     1.0209685056578213e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.0924427725919337e+00,    -9.3308417855415501e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.5229649531570313e+01,     1.6771035970818353e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -4.6488736343990197e+01,     1.6665254781802934e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.5882683045631598e+00,     5.9578998114078913e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2485585863496443e+01,     1.7185831474368300e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.6279495953362058e+01,     1.5766973728793057e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.2607962584242241e+01,     2.4552712921379702e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -2.4935061863179300e+00,     8.6203347423104624e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7629542652798424e+01,     8.7615541046437284e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.8682844533369177e+01,     6.0119325700440349e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1980586988484916e+01,     1.5822330658979399e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4306021560799650e+01,     4.0616682616465914e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    3.8624691817646656e+01,     1.6653125093690258e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0230734774361654e+01,     7.1064432184856805e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.8489242478087256e+01,     1.6681476816160230e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    3.6138169352967033e+01,     7.0935556483958790e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -4.5999299091394263e+01,     9.3836290613381716e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2730582210256834e+01,     3.1187653480967092e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.5023499866331372e+01,     1.5416649649290746e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.4771381507242136e-01,     7.1121330466012632e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.2927434930944770e+00,     5.8526729624523850e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2262004944276384e+01,     3.0639527989249855e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    6.8504810993288402e+01,     7.1214940573630583e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.6660047217929126e+01,     2.5721457791976016e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.1870604422454467e+00,    -9.5661189251079850e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.8489662615906198e+01,     7.5385212080994534e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.3339656978119123e+00,     9.8215777315633204e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.6826418141221104e+01,     3.1192913145599457e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.6501588687536419e+01,     1.7672152448589055e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.0383246249460049e+01,     1.5435902014484898e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.3307697061275469e+01,     2.3383961279181309e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    6.0761297771663347e+00,     1.5573932693698442e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0893111791782561e+01,    -9.6256469779189047e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    3.5774153707372335e+01,     1.5440057100052301e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.8221684146224701e+00,    -8.6069672070687975e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    2.1103337648485299e+02,     1.5647021064561284e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -1.8855321520001783e+02,     1.5648036981637351e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.3270553070505815e-01,    -5.2352722233312147e+02,    -2.1367016410546975e-04,    -2.5597745422817746e-07,     1.2609955364290920e-09, } , 
      {    1.4183954910054783e+01,    -9.5818761444097472e+03,    -2.7586509320370791e-04,    -3.9500194668399445e-07,     1.8691506661497048e-09, } , 
      {    3.4670716106474032e+01,     1.3397623647631273e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.8514721896231845e-01,    -1.8052554179959661e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    5.9653112260465605e+01,     3.1676956603462102e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    9.9175553734637916e+00,    -9.3903603625663491e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.6000263553345423e+01,     7.1284758022516971e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -4.4249761251865145e+00,     8.4646445611549716e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.8434676890917974e+00,     7.3549459488912016e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3446788733369246e+01,     6.4830580902816291e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.9729634662464253e+01,    -8.3375515801645888e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8942966867458431e+01,     2.7512832557817183e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.7314655103392283e+01,     1.6830849687938273e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    9.5449530106253508e+00,     8.3474269783932068e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3021288044578165e+01,     8.2353914201973639e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0552943669433068e+01,     1.7470647260739464e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.5023033076036961e+01,     3.1150468411385362e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    3.9879742144913479e+01,     3.2384541676345398e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.2578583317047604e+01,     2.3270360601766781e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.8374257520817334e+00,     7.0938690215042825e+03,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.9375482548772538e+01,    -7.1247072435826612e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.9889563557525307e+01,     1.3666691690486790e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0088725660821748e+01,     1.0835362809307571e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5107342536000775e+01,     2.4836417567414836e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    8.0521280791130696e+00,    -1.6774609101890725e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -3.3599715340842966e+00,     4.7073914808763875e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.9207670054213910e+00,     1.4310731759679926e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -8.1873805336036778e+00,    -3.0288283995724350e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.3133095132553123e+01,     3.2050480432873672e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.4359150217711168e+01,     1.5474062184611295e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0205466736312577e+01,     1.6760015344095991e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.7677881267092530e+01,     7.3211387982139277e+03,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    1.6065485643547198e+01,     1.5545357314172439e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.5053503186726260e-01,     1.6053667903991762e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.9961220315768216e+01,    -9.5480880033560552e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    8.7501816618084035e+01,    -9.5481014304575056e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.0534839364531468e+01,     9.8478156820720051e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.9745419523732728e+01,    -2.0167753347804137e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.2023881118908086e+00,     7.7927070530698311e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9927146717932093e+01,     1.9008949712008449e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.5958564887517834e+00,     4.7019959867836815e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.9825345869366238e+01,     1.6750667594609964e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.7792265503399214e+01,    -1.6773647570121262e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.5984712113488087e+00,     2.5082212508707311e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -5.1568494992039575e+00,     8.4345954082072185e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.0225742709728255e+01,     8.3893754488820341e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.3090332816775341e+00,     2.2639423184405052e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.2135851750356053e+01,    -1.6779430579132470e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    7.2323853988133378e+00,     7.0666283165877685e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1725584273391981e+01,     2.4100173374661914e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.5133501621582482e+01,     1.0488742505439928e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.7025798587426316e+01,     1.5420706560501741e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    3.4584937795270129e+01,     3.1230098215181188e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -6.1432309107745873e+00,     3.8411674481621048e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5186966298714900e+01,     1.4968579263856629e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.6901468685117891e+00,     7.1092776103396627e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0989446656296209e+01,     7.1092986547472810e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.0374286256261156e+00,     6.5677310722120837e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.4518730428768336e+00,    -1.0645802595364294e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.4589859034214676e+01,     1.6751340871005894e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.3259047502288654e+01,     3.3812561401508858e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.2215372994359580e+01,     1.6894778500096891e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.5667758269456247e+01,     7.2564807913452250e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0233657302275217e+01,     1.4941669177041655e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -8.4123654501755567e+00,    -8.4323369070737554e+03,     9.2781703036018566e-05,     1.1077022985990678e-07,    -5.2374421970263164e-10, } , 
      {    1.9469582245496021e+01,     1.4400903259014429e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.6883340245420504e+00,    -1.5534645356533820e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6005356648903863e+01,     1.6434129813288833e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2305156291231528e+01,    -9.8411667682909865e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5866390131504831e+01,     1.5882815638451875e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.2466438443931236e+00,    -1.0116883724927526e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -5.0378869670691088e+01,     9.4174104888411348e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9266596388827404e+00,    -8.4510981839161432e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.1169316177508929e+01,     1.6116929316511289e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    7.9721386701842878e+00,     2.5326135622443293e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    7.0264202708134160e+00,    -1.7910543188769352e+04,    -4.2827577620864032e-04,    -6.4536942788258096e-07,     3.0562169643464037e-09, } , 
      {   -4.4796282304364627e+00,    -1.6518851069397446e+04,    -5.5178884886283007e-04,    -7.9315033415838979e-07,     3.7530881495732506e-09, } , 
      {    5.9762127236737577e+00,    -9.4805806226318891e+03,    -3.6845093151255821e-04,    -5.0585944300650092e-07,     2.3914404448090081e-09, } , 
      {    9.5708248960565943e+00,    -9.0955851803515743e+03,     9.2382580173045129e-05,     4.2446116520293428e-08,    -2.4996508584326565e-10, } , 
      {    1.4482403763594316e+01,     2.3172126073882686e+04,     2.7429325452749863e-04,     5.3213566578348414e-07,    -2.4733303236852195e-09, } , 
      {    3.0101369427957877e+01,     4.7813909115596085e+04,    -1.3255484780688263e-05,     4.1032205900705572e-07,    -1.9574352374797516e-09, } , 
      {    1.6959546796800410e+01,     1.7594600187576929e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.2451877315472256e+01,     8.4395264403631372e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.4637328700563375e+01,     2.4342009469828074e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    2.9007871111534165e+00,    -1.0727157625985743e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    3.5812361266413504e+01,     5.8549741622921192e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0843027605451128e+02,     6.4810785990717686e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.8646144911690897e+01,    -1.3543900261638209e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5122659648248121e+01,     1.5454840303962779e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.8904763638732415e+01,     1.3824426044827220e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2947085322536356e+01,     2.4485083327176253e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.4942002960231751e+00,     7.1643994680242877e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1904177789218721e+01,     2.5014407763632073e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.9689268624736293e+01,     7.1102545349911488e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    8.2783596631776675e+00,     2.3642159204352720e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.7415889301138719e+01,     1.6666985750281361e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0479100179064652e+01,     2.5755205008538695e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.2606972068159747e+01,     1.5673266172621154e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    5.8039607759260896e+00,    -1.0543686122123476e+04,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -4.0091390663542104e+00,     1.0462631585814684e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.6954242626401896e+00,     1.0492314997257476e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.1884162795242061e+01,    -8.5267661621682491e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -4.1318536297457822e+00,    -6.0371494545849055e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6292601610267958e+01,     8.4172568222985828e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.0321717878029579e+01,    -7.5492084409388790e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -6.5611896828387870e+00,     3.2904121241286848e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.0069540064234985e+00,     2.5345280446766643e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.1300308342700532e+01,     2.2703857534217717e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.1840379047136157e+01,     1.3149717793956337e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5517766898604693e+01,     1.5655249231722228e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.7502535553352971e+01,     3.1765621801117624e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -1.9382941852476769e+01,     7.0920151357766154e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -2.3868178392204584e+01,     2.1201351378951085e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2991097440113716e+01,     7.0570303807991568e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9928472370282137e+01,     2.0299346309602501e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.1315555458813282e+00,    -2.2871325127455284e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.9740048434756279e+00,     8.9569990097706686e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0405286569400019e+01,     1.6762305999422828e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.5243333932477707e+01,     1.3694691096998686e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.0323507437143462e-01,     1.6762009165308398e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.6875205862708214e+00,     7.1615458842877870e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7896206384297383e+01,     7.9755121652577545e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.3685129524634814e+00,     1.5548202228230230e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3914365724030255e+01,     2.0265564882099534e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.7874725259020776e+00,     8.3876871195901476e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    3.8437643362023493e+01,     2.3953678542693389e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -1.3218789897140585e+01,    -5.5088525313015525e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.5675027103521399e+00,     3.2240477522736906e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.8113114697861530e+00,     8.4062436857372504e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1338893170222372e+02,     2.3975712489286412e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -1.8619766127288474e+02,     2.3976728406362479e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.1446332342905620e+01,     1.6068588342153875e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0852140394335938e+01,     1.6811820979800159e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7143050789761716e+01,     1.2909001473570786e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.5963995837391387e+00,    -5.5111335480816152e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.8633999933952730e+00,     3.3994878919650218e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    4.1040163075275714e+01,     1.5223363019490607e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -7.4391728887583959e+00,     2.5639526788340459e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.1100487197845489e+01,     2.1448396349847539e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.4313254412069742e+00,     1.5431946082873650e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.5272459807599148e+00,     1.6467352389988260e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {   -5.0025717693029065e+01,     1.6857329414449865e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.1751805518377115e+01,    -7.8669783494594776e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6996482826117163e+01,     9.5449662117135213e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.0288988126790343e+01,     1.6835452087617745e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0999447063631978e+02,    -1.0650570139378760e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.6873171827921340e+01,     1.5019214724503050e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.2758794685431685e+00,    -1.1642919567008380e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1234363795317240e+01,     2.2339222497934988e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    5.5207875482665891e+01,     1.5178481045645425e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.2001633071983742e+00,    -9.0437400586987042e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.6586987249486793e+01,     1.6966292602531386e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1635548153504907e+01,     3.2317531110871951e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    7.2131039636134533e+00,     7.7287173291362760e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.0069326508365606e+00,     1.5460521462399654e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.1221060039196836e+01,     8.9640025912495112e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8705195350111797e+01,     1.5143733215694783e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.6802207446154590e+01,     2.9085493053833285e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.6629026992652946e+01,     3.1697934195481881e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    4.4813877059312368e+00,     7.7521894753837523e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.7824295272319944e+01,     3.2304905158998845e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -5.9638733349758269e+01,     3.2304918586100299e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -9.1753704193335039e+00,     7.1250206166910612e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.7331883922349029e+01,     3.0564276928077463e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.2292388857861511e+01,     2.4650928698695330e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3049855284599809e+01,     1.3974482656807455e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1190137275958609e+01,    -6.8951637063715007e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.3008082553231681e+01,     8.8909214771461448e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    5.8882902810604500e+00,     4.8858796312019615e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.2094137349567866e+00,     7.1704650879272805e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1881202737544559e+01,     8.3680455409041824e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0537530370189362e+01,     3.2338693300052539e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    1.9909497839735373e+01,     3.2480679910877327e+04,    -3.7963752239047967e-04,    -1.6439062299062145e-07,     7.6367851048374099e-10, } , 
      {    1.6909242072745016e+01,     1.6799363830477985e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {    1.0380239561445498e+01,     1.6100062940020045e+04,     1.1655480853674925e-04,     2.8300096881322299e-07,    -1.3951434819921005e-09, } , 
      {    5.6691304169416954e+00,    -5.5731990943020764e+02,    -1.8807069274594723e-04,    -2.1782126004654988e-07,     9.7753467335796839e-10, } , 
      {   -3.1537448595119244e-01,    -5.6297970609193726e-03,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.6508133664794478e+01,     2.4428765624339292e+04,     2.7419532216391455e-04,     5.3217929901478405e-07,    -2.4726031031635556e-09, } , 
      {    3.0564965743604727e+01,     7.2962956932431371e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.3305964358621466e+01,     1.5634411111899861e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {   -5.2335714876415707e+01,     8.5286379897247407e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2415034250051216e+01,     6.7434991105398412e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.1191203592641866e+01,     3.2305623218016448e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -1.4703339316855253e+01,     8.3378649532729905e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.9173123542356976e+01,     2.2721906477573095e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3969388387124459e+01,     1.4756702875011864e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.7061807741445314e+01,     1.6729087500329526e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.6940760952161554e+01,    -1.6761191179917936e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.3746452391478174e+01,     2.4926555697150579e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.9932618935388797e-01,     7.1768509875494001e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.4975807566588619e+00,     9.6900813277315738e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.6071528654844096e+00,    -2.4449435484070787e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.2251742790952065e-01,     1.2720614196920649e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.3566851453820790e-01,     8.4323026961334654e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.7959130302419130e+01,    -8.5290673620079833e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0273714268826442e+01,     8.9853324992273629e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2664136756435948e+01,     2.5056356234120900e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    3.1687093931473214e+01,     1.1983274123199753e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.7242318725900745e+00,     1.4370250336704725e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0536124257501131e+01,     1.7443434264622349e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.4258579582550340e+01,     2.3841233713842281e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.8598811288221757e+00,     8.1326441128921215e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.1796126924094068e+00,     8.9040393883455981e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.2979953328566562e+00,     1.6047108948392035e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.2616067724314348e+00,     9.0999486622344239e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1374545845252023e+01,     2.6188556885548249e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.6987743161683039e+01,     3.2304200527082696e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -4.2752845688410105e+01,    -5.2388516808825727e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.0225608004804048e+01,     2.2808622791346956e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.1512588414487452e+00,    -5.7650194934137380e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.5167144092431414e+01,     1.7373192170644299e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.0784629261736329e+00,    -9.3996404833096017e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3122005478360276e+01,     1.3571090486119292e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    7.9725257254838908e+00,    -7.5000299876968518e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.4305262835997485e+00,     2.5122027410605220e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.6005872410400883e+00,     1.7806896363611933e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.4930878630354293e+01,     2.2809776777067862e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    5.5674071004273769e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0476275176956023e+01,     8.4958229646694563e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -5.3561913855859480e+00,     1.5534117791800531e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5971706301065410e+01,     1.0993097163648004e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1522931327935364e+01,     1.1228383779000560e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.7028901223960624e+01,     1.5631796538951727e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9088256804139164e+01,     1.6539490205335438e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -5.2714332486941771e+01,     9.0495678178664439e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.6705364883101907e+00,     2.3970186973429500e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.1577680020111821e+01,     1.5236474369419006e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2130592019705212e+01,     9.0066624072052145e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9788377258668611e+01,     2.2273099205410519e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.1804996641960633e+01,     1.6999704167068197e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.3020803285238649e+01,    -1.0999585864879173e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.0994242658231208e+01,     9.8311015361086884e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.5525113788826448e-01,     6.6544815969262399e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.1086253682468303e+00,    -3.3787057300025253e+01,     2.8263352852431007e-05,     3.7539802067464924e-08,    -2.2902113481933369e-10, } , 
      {    1.2364440886314895e+01,     1.6833145257980948e+04,    -9.9779237061628989e-05,     2.7639906699208196e-08,    -1.8858767381479843e-10, } , 
      {   -6.6168779721387914e+00,     1.6354500009493009e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9983528530090410e+01,     1.6852329081700118e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    5.1446788039968157e+00,    -1.0278875103382582e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.4736033848277330e+00,     8.4340877854154915e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.2177928952985519e+00,    -8.4328445298654842e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.9207650821518278e+01,    -8.3035464956055894e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    3.9428818247666960e+01,     7.0340370713710836e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.5234849825250063e+01,    -1.0446016563015755e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.5969440313049510e+01,     1.2410076520395705e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1448340101121328e+00,     9.2970603580385869e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0182152317810690e+01,     9.3222386567568810e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5944435714788092e+00,     1.3840158528974815e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -7.9411059117751881e+00,     1.6094035095422158e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.5014515752260266e+01,     8.3382943255562332e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.2964631306809746e+00,    -6.1242561511396286e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.0188227086958221e+00,     2.3345283030462717e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.2284616805377357e+00,     2.3311864446376490e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.7292577994574330e+01,    -1.0096772498156499e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.5756441599395455e+01,    -7.0867462274124155e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.4146794994760644e+01,     8.2414887839708244e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5274328915565025e+01,     1.6511123223497420e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3221986394025798e+01,    -7.9035597768438350e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7034475266501079e+01,     1.5753467217521702e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1043854726229474e+01,     1.5541590828676937e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3312887857776714e+01,     8.1578391591852396e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4036085713279654e+01,     6.1175184485083419e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.3140123661557880e+01,     1.5782515757081486e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    7.6262265248002699e+00,     2.2144266789899008e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.3846395030579082e+00,     1.3215750011085349e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.3426937457715740e+00,     4.9088966687259790e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.3030839281699588e+01,     4.1745930656701261e+04,     6.9754245824971129e-04,     1.2471056244653037e-06,    -5.9114786578729044e-09, } , 
      {   -6.9140294643253792e+00,    -9.5607139552291592e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.9148456496385862e+01,     1.7390817483369228e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -5.4741156079776957e+01,     1.9994656499961502e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.3710767091783964e+01,     8.7512360661610874e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.1637699937222315e+01,     3.2855667196582996e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.3460499056360675e+01,     3.0405395359618207e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {   -3.1748751857075597e+01,     7.6121175375003295e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5483464067418165e+01,     2.4586494348882665e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -1.6126488175263511e+01,     2.3958947451057586e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    4.6431645293500289e+01,     2.2026036037521622e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.9376722495743284e+00,     1.4911843681149936e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.2782322162796733e+01,     6.4789030047142278e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.9277898168917872e+01,     8.4303529118152765e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    7.6499597189884021e+00,     1.5643119795930248e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.4506004271561601e+01,     2.2914560985769342e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -1.5561454365124661e-01,     8.4333177405832739e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2836238086546743e+01,     8.4336145746976999e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.3121976760099452e+00,     1.3187300870507436e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.3411641119005562e+01,     1.2477550146516242e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2710854028055806e+01,     9.6029008311065809e+00,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.2703231381762389e+01,     2.5182841568742333e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.4191256778668233e+00,     7.7500472371141277e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.1729229884459542e+00,     5.0977826182531908e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6478432202017242e+01,     1.6787896717512533e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.3123252855555139e+00,     8.6430156234712085e+03,    -4.9668938615378649e-04,    -5.1567207191215793e-07,     2.4333283468568719e-09, } , 
      {    2.6894273114885301e+01,     1.8949431134983650e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0067490652909122e+01,     3.1190283313283275e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    6.6748819121950600e+00,    -9.1823056949031507e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.6082443257121078e+01,     2.4638279450210372e+04,    -1.6370259965507483e-04,     2.2646325783779979e-08,    -1.6909816383419513e-10, } , 
      {    1.8745285646539763e+01,     3.2237397778274681e+04,     8.3761938424448094e-05,     3.8334217765331005e-07,    -1.8225600713950786e-09, } , 
      {    7.5026033583069633e+00,     7.8051041820668470e+03,    -6.6489311722770230e-05,    -4.4208220325654124e-09,    -3.5677438792850815e-11, } , 
      {    2.3223570250426246e+01,     2.3314101425113360e+04,    -1.0184813096439580e-04,     9.6493145690384486e-08,    -5.1826097696928287e-10, } , 
      {    2.7788941410118479e+01,     3.8856915735622475e+04,    -1.6290435392912801e-04,     1.5929455246300667e-07,    -7.1665643155292728e-10, } , 
      {    1.0034639893392560e+01,     1.6904192953921443e+04,    -2.8271803154892748e-04,    -1.9132687111306729e-07,     8.9928089709007817e-10, } , 
      {    2.7070697947901245e+01,     1.8975168284505482e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.2457721856782925e+01,    -8.4151038411767113e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.9008814074621348e+01,     7.1083217300957931e+03,     3.9957403058506603e-04,     5.4269558649720345e-07,    -2.5674762924198806e-09, } , 
      {   -4.7990299689263560e+00,     4.3143705111833997e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1206034042115466e+01,     1.3225643119544791e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.9487806545358396e+01,     1.5691299856608477e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    7.3058573830646679e+01,     1.5552357191015066e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.9745103438778152e+01,     2.3078844819923383e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.4952989056688349e+01,    -3.8254349640145138e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.3684936127105555e+01,    -8.2235880593333168e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.9636442348030769e-01,     4.7845364962893800e+04,     1.0822796387890461e-04,     6.2376613025234003e-07,    -2.9699201291089064e-09, } , 
      {   -1.3758898501530439e+01,     1.7413007072542212e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.4015666631927370e+00,     1.6493089539510092e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0025431570322883e+02,     1.4323344284451579e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -5.7208719820366156e+01,     1.4323357711553028e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.3756782585080323e+01,    -8.4385332295877561e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.8527555880927302e+01,     2.4684710126198297e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.2530806603282137e+01,     1.4434684686517390e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.1141862778253691e+00,     9.0617681128889690e+03,    -6.6685176449938512e-05,    -4.3335555699656980e-09,    -3.4222997749522167e-11, } , 
      {    2.2831826476644181e+00,     7.8051642023920049e+03,    -6.1357413464121477e-05,    -5.5663397982910266e-09,     7.4656458754057421e-11, } , 
      {    3.5781040233316105e+01,     1.5867083154304282e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4455402945714443e+01,     3.2298352916949840e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    9.1449712183279743e+00,     1.7355520278401171e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.9228763333880778e+01,     2.3347906149228176e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.5503105091618377e+00,    -2.6801534824385162e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2941603698312720e+01,     8.3445733946567070e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.9700947506766497e+01,     1.5662948134138509e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0338483011452784e+01,     7.8331556914668054e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2894675889345795e+00,    -4.7976201862102243e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8048813878958079e+01,     3.8898693138271829e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.6943537540083110e+01,    -8.2242453242862139e+03,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    9.0564533114573706e+01,     1.5422437528980168e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.4306564400633730e+01,     2.2740935185711212e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.4640980598270971e+01,    -5.9018735009945394e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.9770049665088951e+00,    -1.9252631382393215e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.7266834421770795e+01,     7.6303569367148648e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.8393334784512803e+01,    -1.2133698177867082e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    2.2154261416699264e+01,     5.5982415227764823e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1993837750189909e+01,     1.6573271632838398e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9498207715194624e+01,     2.1237692898295911e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    5.1999886871828878e+01,     4.7849967362573254e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    3.8272691287312256e+01,     4.1750533056380722e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {   -2.1204102529324707e+01,     2.5085917974934822e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    8.0962898749414585e+00,     8.2304681572484606e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8044791431545956e+01,     2.4159691951686713e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -6.3361430252913653e-01,     4.1566361568429684e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.1026670003194059e+01,     1.6748651071094260e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.4076046270469682e+01,     1.3697344612796496e+04,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.2018052056507948e+01,    -3.5580909178355546e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.7313394427361303e+01,     8.1156962491198865e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.2291056488297066e+01,     1.5267127291728710e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    3.8418652635257700e+01,     1.6775664093636969e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.8856055399826381e+00,     1.7976112936500995e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    9.9622144201778173e+00,     6.4802692020310515e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.6637596648617614e+00,    -1.8452124629466256e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.5735750912192957e+01,    -7.1899688987488553e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    9.7902953143009555e+00,    -2.8924496584735180e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.1824628177938116e+01,     8.4017622729212235e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0421257351741593e+00,    -1.6988627729005118e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {   -6.5175982432189201e+01,     1.5453521585557030e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.9716164294655343e+00,     3.0559674528397998e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    1.2517436884196737e+01,     1.7776927181504416e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7558486726746310e+01,     7.1167662991125289e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6810260239312100e+01,     1.6641625630148490e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.6391894131203175e+00,     1.5054166327063762e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7075403917325147e+01,     1.6906774025506591e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.4249834454459904e+00,     1.2279894111656590e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    5.3364495776664604e+00,    -3.0267737863061598e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.1789848205146942e+01,     6.4813091799743779e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.1313490256633042e+01,     1.4059712660179779e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4846947083928058e+01,     7.9314126992721449e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.8676667414403404e+01,     2.4095665153518181e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.6403430582489911e+01,     5.4193210854428262e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    2.0368161721088367e+01,     2.2858962056241966e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.5788841703412364e+01,     2.4917347957662427e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    3.0688423338433628e+00,     8.1709570703846939e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.1556208778394918e+01,     9.3775955869432237e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3850893480519101e+01,     8.7735608755071626e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.6176395343496051e+00,    -8.5705369956774084e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.1156455547041459e+01,     2.3958196239623241e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.7279636107529974e+00,     3.0890975308113593e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.5021060990178954e+01,     7.0634397562506110e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5986183781886901e+01,     2.2574490161715305e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    9.3591484811442491e+00,     7.6162726230677308e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2451974167818506e+01,    -8.1442868880139413e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -8.7667697587916564e+01,     6.4808824959240901e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.6630356895097584e+00,     1.6614964923563861e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.4477750752127626e+01,     2.2864221720874331e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    5.1598187891754861e+01,     7.9306367526836284e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.0085236025378308e+01,     1.3774673216752586e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.0307557047945995e+01,     2.3268283058983077e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5692976531542591e+01,     9.3047038894584966e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5019107695060159e+01,     2.5055904118080751e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.4439240238793399e+01,     2.4604528032869988e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.8580028232881006e+01,     1.6706800652179270e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.1547757489345098e+01,    -1.6622472171380556e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3303210477846520e+01,     8.1048795377601673e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    7.2200594957980684e+00,    -8.8602685548364389e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    5.7564380266474245e+01,     2.3507172470370551e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.9994952144381145e+01,     8.4336566134996501e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    5.6739141654851963e+01,    -8.4332757017813292e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.3617376255346912e+01,    -1.8477175907779242e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.4361675919068304e+00,     7.1018099659744148e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.3619405760446424e+00,     8.0934048093725705e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7394475747644824e+00,    -1.4209084800312457e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.6259389883096658e+00,    -9.4705424558946561e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.4868730858468169e+01,     7.5965755092119007e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.5632459723075698e+00,     3.4481464784691161e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.0719354587045213e+00,     8.2400660930370723e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7746043076981621e+01,     8.2077073564678612e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7084735341056138e+01,     1.0175903629075246e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.0864214245131389e+00,     2.2011115599359509e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.2891152104973521e+01,    -7.0471549825705370e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.6021862875571301e+01,    -8.4365794034656974e+03,     3.7530386985777772e-04,     3.0218436743557376e-07,    -1.4215706757493813e-09, } , 
      {    5.1951774911935139e+01,    -1.6884512696355487e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -7.3462793971949134e+00,    -9.4029796008887279e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -7.0575320263313714e+01,     8.6756916724968032e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.0409461133155347e-01,     1.4243721194206481e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.4347348536309656e+01,     1.0165162779580416e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1304729136977564e+01,     2.0756801629108159e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.4119301786153486e+01,     8.6763963044129523e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.3780191617341959e+00,     1.1365312136810010e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7091011971027605e+01,     1.5592417687618501e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.3659818531360326e+01,     1.7535081610552133e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.9293699919790726e+01,     9.7176903048098993e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2596495000584786e+00,     7.5698719572423888e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.1189301268440985e+01,     1.6817514512551956e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8894958112900703e+01,    -8.2178832174148174e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    8.0349937076904787e+00,     1.3711964093765542e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.0317302864482510e+01,     8.4000779763999635e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7961535586655394e+01,     1.8065173418282044e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.1998726328382510e+01,     2.9986637240403383e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    5.3699861053603266e+00,     5.7997038652411939e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.2328239867919223e+01,     1.6942992692640641e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.0908736165767499e+00,    -1.0150665152430491e+04,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.0765490463392450e+01,     1.6136479829636255e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.0434917196943321e+00,    -8.1982336688893952e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.7793587141310994e+00,    -8.5977594675806467e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.3688663939417317e+01,     1.6932511175028423e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.1399544927680672e+01,     1.0602706026271200e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3201526755940876e+01,     1.6833150887778011e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.9457201061360259e+00,    -3.3781427502964334e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.4001862849351106e+01,     1.6808005958658476e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.4907566047812374e+01,     1.7303714642696999e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6166329673155008e+01,     2.4499753299954627e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5823647445793996e-01,    -1.0142615244652297e+04,    -3.3505568040147844e-04,    -4.6946515870476163e-07,     2.2727532075365822e-09, } , 
      {    3.6025509778543807e+01,     8.4469726689118415e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4089087705349279e+01,     1.7560818760073969e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.3262805284401061e+01,     1.3542738763649324e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.9637371197694762e+01,     2.4151022083704527e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1432601513876460e+01,     1.4809671972223055e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.8630293128648692e+00,     2.4855562391738185e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.3344672583313688e+01,     8.4199596463691323e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.7835899172190874e+00,     7.0812910136716109e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1260951464122986e+01,     3.0578896409046516e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.4086639820484368e+01,     1.7197776448274617e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.5940727851623478e+01,     8.4341707895566360e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6846687524721187e+00,    -2.5562005556010486e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -7.9543353040152965e+01,     8.4327615257243378e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.3215974336700761e+01,     3.2310945346944518e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -7.4628085217365303e-01,    -1.8941115430525410e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.5300064133901587e+00,     9.6474215117758704e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.3493137254867435e+01,     8.4888230878268278e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0246806401078771e+01,     8.7125236065022018e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9171703395375840e+01,     2.3994244656025647e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.7006350641543150e+01,     1.7895913877225536e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.7497678034027118e+01,     6.4965225757863573e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.1107104059874171e+00,     1.6709197960940543e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.5418167105616646e+01,     1.7267606292179993e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.6092848537574561e+01,     1.3790644617324258e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5326789074686602e+01,     6.5876300873871878e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2334662150048658e+01,     3.3406422520616405e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -2.4468379797793133e+01,     8.9017381885058639e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -3.4478426754574954e+01,     8.9025141350943813e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.2080227621281637e+01,     1.3695395728914837e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    3.5491615620077734e+00,    -8.3014689528218896e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    9.3268101196007844e+00,     1.8541142143079862e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1766053738639735e+01,     2.5112178915068591e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.0056786951014876e+01,    -7.3657369370766128e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.2005263491527232e+01,     1.6044957234336289e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.7023577805812273e+01,     2.5095780039246663e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -3.6788241445206751e+01,     1.0085532975784590e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2954269191657229e+01,     1.1621404748693547e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.2773733269928913e+00,     7.0475095861686459e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3910137055434969e+01,     1.7578852444061289e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -7.9702630110654091e+00,     1.6909211265137674e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0614796892703680e+01,    -7.8220792534037255e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -6.9753328029940604e+01,     8.6535333700826413e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5231876126177159e+01,     2.5078397487565628e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.5516074986139749e+01,     1.4929116677916791e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    6.4676806943681342e+00,     7.3887273763941657e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4494470046778195e+01,     1.4810383317689932e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0747092299739286e+01,     6.5327956589882178e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -5.3529695352008915e+00,     7.1714212825164068e+03,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.2896370408649211e+00,     1.5663738358441224e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.6579077307853076e+01,     1.0881763475132917e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.1667696486244424e+01,     8.5133196184923454e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0334729177513717e+01,     2.5069519099112887e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    4.4410385804257047e+01,    -5.2318053617210774e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6312993525202518e+01,     2.5843109518973753e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.4181642172712706e+01,     8.4331071281316854e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.1501156219613463e+01,     8.4338251871492885e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.8049380600229142e+01,     1.0045718073886679e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3209760847069322e+01,    -1.0722106333799011e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6893519207752615e+01,     2.2292802880537405e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.6171089571173297e+01,     8.3781092274541443e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.8718762739154826e+00,     2.2021433637842154e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    1.8418116611722972e+01,     9.9379871814065082e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.2890862860956055e+01,     2.2023382521723815e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.8717708592055899e+01,    -5.1749937773523448e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.0929992527879513e+00,     7.0776489433101324e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5235067583207252e+01,     3.3665138314048992e+04,    -4.1056475673581917e-04,    -2.0131403294392373e-07,     9.3825991705128483e-10, } , 
      {   -1.6063182274083399e+01,    -6.2948944706875272e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1842208869201755e+01,     2.6099059650629817e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.7371296037753918e+00,    -5.9712918153105898e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.1827224077845759e+00,     8.4158341313648307e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.2686309684741682e+01,     2.2821776986660236e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -1.2663529672807535e+01,     1.4939591634257951e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.2187521909595173e+01,     4.1784280272943397e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -9.3823389648098257e+00,     1.5652596095046591e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.8987115657413334e+01,     8.3536126967886285e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.7302717475589915e+01,     7.3178711959265147e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.2919922377870421e+01,     1.6999316025274660e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.5948967032839665e+00,     8.3260884007366458e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4368761952483558e+01,     7.0673396620546446e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.3711628440939080e+01,     1.1828124418671482e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.2776088114072319e+00,     2.5446480927466234e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -7.2274242870828260e+00,     1.3692742213117028e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.0624628940418340e+01,     4.0012030023422712e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.9202059223101010e+01,     3.6081820326492834e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.5742344227150994e+01,     9.1268221644006786e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4219682845875024e+01,     1.7330946590386360e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.4349878317514884e+01,     7.7289721985881624e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.0050515932527340e+00,     1.0083102297424561e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.8444770254026785e+00,     8.9145810838842772e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5643205220190179e+01,     8.3931257745377952e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.4306676913341036e+01,     1.5019228151604500e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.4195964554257330e+01,    -8.4169577269043784e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.1509624411734398e+00,     7.9628929269353748e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.4918825556419733e+01,    -1.6770794080749045e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.1324683131706536e+01,     1.0510336539180958e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2140347555315108e+01,     3.0525927311835323e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.3266019937879250e+01,     2.6492080620322802e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.3682249181384634e+01,     9.4632588651339956e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.9401880014373800e+00,    -1.0751740789786676e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    7.1694331408376080e+00,     7.1090644754874384e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.3510216309442999e+01,     7.1095117895995063e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.7768641656317456e+01,    -7.9007597769624408e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.3708785428328909e+00,     8.4618178801104550e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.1718439024560777e+01,     1.7390112851453083e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.3532832302565552e+01,     1.6903177790742724e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5256781637362327e+01,     2.5319586142629691e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    2.5006963216544861e+01,     2.3174411813350591e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.2371962613297621e+01,     1.6151874205521275e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.8626636590061478e+00,    -1.3253414866040371e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.9201922898276386e+01,     8.4518284741042626e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.8450426887835434e+01,     2.3889469923110286e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.9924484955371113e+01,     2.3819640079204917e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    8.7896703821728543e+00,     7.9302844367255539e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1432568652304198e+00,     1.7232730813070724e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.1494044386020157e+01,     3.4270016721014515e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.2009554125372560e+01,     7.0829797419169136e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1584114982237050e+01,     7.1409273217768114e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3637242980266969e+01,     2.0824988197026778e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -8.4942310366604836e+00,    -9.5541281913017283e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.4074376790737903e+01,     1.4463070619927650e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.9022712540787747e+00,    -8.4128026413369771e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -8.3514516963623294e+01,     1.6737884708756130e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.0488180548880898e+01,     7.0816791554651463e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3105379365116862e+01,     1.0163284390752870e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5757496574619406e+01,     6.4884587140669855e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0319485072714302e+01,     8.3985212686305003e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.4456641584916241e+00,    -8.4492139011561248e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.0930428256133791e+01,     8.7631468291271085e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9167976675010969e+01,     2.2652042422727431e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.7711930181433750e+01,     2.2861591888558149e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -3.1299340858262079e+00,    -1.5401931140866187e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -2.0329702561255857e+01,    -7.4739573797664889e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6275519696066709e+01,     7.0417740382685806e+03,     5.9491777697159834e-05,     1.4283095859168038e-07,    -6.7665445472457939e-10, } , 
      {    1.1765484543120708e+01,     1.6870375886458838e+04,    -2.5702062782582085e-04,    -1.5321431016273956e-07,     6.1509281349729026e-10, } , 
      {    2.8166482523975347e+01,     2.9899862335323705e+04,    -3.1768512133621652e-04,    -9.0587436315316785e-08,     4.1378847682698886e-10, } , 
      {    9.3937545524311972e+00,     1.5471796624731203e+04,     1.2444852065189067e-04,     2.8119542570203486e-07,    -1.2310970767150669e-09, } , 
      {    1.3726893746324006e+01,     1.6309588025485249e+04,    -3.1601535029642311e-04,    -2.2776478864610646e-07,     1.0172409138408393e-09, } , 
      {    8.7534563757670476e+00,     2.4915056588397463e+04,     6.4244299554066760e-04,     9.6962736221907188e-07,    -4.5917188551565261e-09, } , 
      {    3.0595361539476507e-01,    -2.6171720519219587e+04,    -6.4224713081349933e-04,    -9.6971462868167171e-07,     4.5902644141131968e-09, } , 
      {    2.4415544056687057e+01,     4.8928537674862382e+04,     3.5755813772538915e-04,     8.4719736332848084e-07,    -4.0213840406992681e-09, } , 
      {    7.6790760708956247e+00,     8.5755015291963173e+03,    -4.3503078219027570e-04,    -4.4173798554295371e-07,     2.0856199747651126e-09, } , 
      {   -1.3020416089721648e+01,    -1.2019384395455018e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.8184197741342132e+01,     1.6743918183151080e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.0316250440477326e+01,    -2.5108897950446109e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    9.4623545565354050e+01,     1.6786430455975096e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9938729096414185e+01,     2.4997549002562977e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.8896906430666057e+01,     3.2762367192055226e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -8.2750746533893036e+00,     1.5663261507246909e+04,    -5.3195027303182794e-04,    -4.1480173742050790e-07,     1.9500175881587754e-09, } , 
      {    3.8224504963656130e+01,     2.9478514023526273e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.4512909953766702e+01,     8.9661725383185330e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.7833663761690346e+00,     8.7277124992143017e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2876249644919113e+01,    -6.6856142405407791e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.6367857028664883e+01,    -1.1549689423689854e+03,    -5.2529319941377664e-04,    -6.1838954652883535e-07,     2.9213417982617307e-09, } , 
      {   -1.5517456706039511e+01,     8.4749357913099102e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.3208538497319235e+00,     5.4039359882436554e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.4142457885581763e+01,     8.3416304290250755e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.3670607234004812e-01,    -8.1463644307976410e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.0251319443470321e+01,     9.3772424513668939e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4328566921849696e+01,     7.9728823329415709e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6860855221051438e+01,     2.5306431947316411e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.9905250796482630e+01,     2.2310836564524729e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    3.6622921878118468e+01,    -1.6772974293725332e+04,    -6.1141782883174396e-04,    -9.3274758549706954e-07,     4.4164102280673174e-09, } , 
      {    1.2594297043162173e+01,     7.1512366030322992e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.8749813134466486e+01,     7.1845391937158602e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -6.5905710040183948e+01,     1.6013166310170254e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7259633671544911e+01,     9.9330114745040391e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1614509390590650e+01,     7.8941760862085457e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.2047137803665965e+01,     4.3942239720352591e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.6393751809293597e+01,    -1.0844165581994876e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.5146470594060204e+01,     8.4887269789052934e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9803959076232530e+01,    -1.3145751242659080e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4317056728399240e+01,     1.7393094999727342e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.5234020394915966e-01,    -8.7278298831401071e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.4298679561007585e+01,     3.0980381531494484e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {   -4.9696408904938298e+00,    -3.0731744521315022e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0865776157627913e+01,     7.9651941267751099e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.1517459643435828e+01,     8.8360149415134038e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.0318430379067692e+01,     1.4217447014520209e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6766548347654023e+01,     3.0981086163410630e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.5336784026400260e+01,     2.0658585851792523e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.9911656545573294e+01,     1.7074497785853218e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.2345803158360624e+01,     8.8345922505796516e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0865075716973159e+01,     2.3549418050628490e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -3.6921337918719509e+01,    -7.1265611293103284e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.0915934092479157e+01,    -2.5078229768768360e+04,    -1.7874973763498745e-04,    -4.2202546126903998e-07,     2.0032986117127136e-09, } , 
      {   -2.0180078786848433e+01,     2.8560984319038744e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.3875439156978530e+01,     4.9417802729019058e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.7627344513132268e+00,     8.3635763796205156e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    6.8201581818883650e+00,     7.1252876408679576e+03,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.6705109020556090e+00,     2.3989338359023903e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.1950058492612632e+01,     1.4412243760986084e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.2192807308508904e+01,     2.2901406790456062e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -6.8281482914299474e+01,     9.0435343434714960e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0406056154559771e+00,     1.6733771648955353e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.3503218780720014e+00,    -9.9073342590968041e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.0711996207584720e+01,    -7.0932886242189861e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8862707186032310e+01,     1.5616350619584835e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.9329737644450901e+01,     7.0418642535672343e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4819141616372695e+00,    -8.3003579352684337e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.9483874973462491e+01,     8.3045128980532681e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1345922726969768e+01,     1.9401970681701438e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1332135081234854e+01,     7.1355965231700302e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.5271976677995873e+00,     1.5653429077185736e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0398164120684374e+01,     2.9775933788851755e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -7.9779105784244493e+00,     2.8973911544356979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1350687611569182e+01,     2.0692367279295489e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3217651673058803e+01,     1.4221179289075424e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.2291081916869139e+01,     1.4402980801798129e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    9.5699755868967902e+00,     1.7691254826253817e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -5.0450823635055627e+01,     9.6778754029119864e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4642214908745233e+01,     8.4034874997468487e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.0678718972392538e+00,     1.6790543515775869e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.0999590662890650e+00,     1.2327593227227662e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2612787703565594e+01,    -6.5021427366785138e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    7.6416976558402592e+00,     8.2062846655341091e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.1482084409991167e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6231990378089634e+01,     1.7333377268746390e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7040274940917466e+00,     6.7604197913486232e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.2364491953746430e+00,     3.9213495414904123e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3805866571722161e+01,    -1.1134650977592714e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.5923995613812849e+00,     2.4483871330023048e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.7225013930931919e+01,     1.6763253463333589e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.9471625653142922e+01,     8.4357331465399311e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.7485395153003758e+01,    -6.6879154403805151e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.8596792004601141e-01,     1.6720749571236378e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.5612865207969136e+01,    -1.8343813426350675e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7779345889268740e+01,     3.2151610629562965e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -8.5355538022466391e+01,     1.5437881505694759e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.9948588555109574e+00,    -1.8662828081017884e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.1074630394279312e+02,     1.5438077608842437e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -9.3720050013720702e-01,     1.5632109912060127e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.3565010941222173e+01,     2.1554334544269921e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    3.2555749005086341e+01,     6.4806519150214799e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.5964665313526805e+00,     8.3509469694964446e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.3918280142343393e+00,    -1.0962444241338299e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.2917400887724355e+01,     3.4663037690707490e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -4.4298849114819014e+01,     1.5742700855183573e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.1528924812981941e+01,     3.2322936080750773e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -5.8354431280353403e-01,     6.3366721786125787e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.8254924044716638e+00,     1.5016591605737593e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5440765265823339e+01,     1.3302022443263773e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8240035208947620e+00,    -1.0509904694620513e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.0170502006157609e+01,     9.4636111810920720e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.5376270850810636e+00,     2.1933563338347383e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0687875929163397e+01,     1.6639298707134447e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.7436666156884693e+00,     1.5898563381967515e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -3.5191711844173028e+01,    -1.6761061701397637e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.1806857525394300e+01,     3.0194691908066579e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.1896729352131445e+01,     8.2299501662047132e+03,     3.9957403058506603e-04,     5.4269558649720345e-07,    -2.5674762924198806e-09, } , 
      {    8.4215962318786111e+01,     1.5592172092912979e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    7.8596125864108792e+00,     1.5612191952163785e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6243717544956912e+01,     2.3713701884782531e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.1930691626141551e+01,    -8.3444486440268847e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.6376796590605366e+01,     6.4640598666891474e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -3.6839201702702816e+01,     5.7382271036925522e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.7498092817135664e+01,     3.1741038637316695e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    6.7096077682863458e+00,     8.4618520910507450e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5506878189716209e+01,     1.8491975815477996e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.6147899207931967e+01,     1.5062310920822097e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.8512777084681083e+01,     1.7565266803291233e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.4229647018299758e+00,     1.0476123267117548e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.1353603243932817e+01,     5.9943072573191048e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -3.2124410745010238e+01,     1.7231205559819507e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.7821925183044804e+00,     1.5314026630431132e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0490731938686247e+01,     8.0596233818696064e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1756222433031706e+01,     1.6823936128740439e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.9688102120953147e+01,     5.9950118892352530e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    7.0548371501179101e+00,     5.2670251933625459e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1612085270805178e+01,     3.2324099542257798e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.7204484141774657e+01,     1.6260104123997271e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0445699495662401e+01,    -8.4111216611851887e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.2342733286858589e+00,     1.6544904651000383e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.2763201458526563e+01,    -1.1722625244047332e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -2.1250728959065547e+01,     8.4373540504343218e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.4506539448594729e+01,    -8.4295782648466520e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.1952538056097612e+00,     2.3963601209502067e+04,    -1.9206388487109000e-04,    -1.4849843052385087e-08,     6.0650191506802891e-11, } , 
      {    6.9390041153554538e+00,     7.6839152749109471e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4689279734433494e+01,     2.6885101590015787e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.3110431242786149e+01,     1.2376295092892742e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4573485967462913e+01,     1.0556305360445856e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5154107389532946e+01,     1.9494623224986390e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.1221051898266095e-01,     8.4050802242302289e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.6021024456984776e+00,     3.2930918257755380e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -5.1305691991947526e+00,     8.3772502304303525e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5764853443869827e+01,    -7.7925393342725647e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -4.7356701396484752e+01,     7.8665489771762332e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2919552674101983e+01,     2.6277574399161855e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.8902080842711285e+01,     7.1556887983688130e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    6.6785474828533635e+00,     1.5732225898842413e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.8869848248423672e+01,     2.3436930376392498e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -1.1934916818962190e+01,    -9.6277245207026044e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.0439120935051817e+02,     8.4335642092143262e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -9.1710621146480520e+01,     8.4333681060666477e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.6977544768957543e+01,     5.4607475726123857e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.9393417889950577e+01,     2.3960487963676853e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.7738527609086052e+00,     8.3349836607370889e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4909501257356567e+01,    -6.9833448801026134e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.2199824318242236e+01,     8.4577390312499701e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.8440187146563431e+00,     2.1621897399275847e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.0303184416069850e+00,     1.2238125193507251e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2326624194362029e+01,     1.5122510848587734e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.5551366541725997e+00,     1.1508401417084852e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -8.0142727177471507e+00,     3.2512160138540559e+04,    -2.5825200609447095e-04,     4.9097081485962703e-08,    -2.4807916062374962e-10, } , 
      {    2.3469587857686300e+01,     4.7872449417282460e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.1385293241292391e+01,     2.4579494129247541e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5894565241511403e+01,     1.5645386784829692e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.5935130093150576e+01,     3.8996605347385244e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    4.4148984972252139e+00,     9.4465451659873054e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.5116693060929883e+01,    -3.5811029162329032e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5029122581990096e+01,     1.7297165162883393e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1667213167901025e+01,     8.1966867297454974e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.4819281634815544e+00,     8.4444867843628508e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9154749604541377e+00,     8.4224455309181230e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.3185119929212377e+00,     2.4858959472545896e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -5.4817625584283768e+01,     9.7116568304149514e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.0681857949765430e+01,     2.2839049983427096e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.4617171485665391e+01,     2.3802719398396133e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.6375389181343756e+01,    -4.2184559337075025e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.8907561362316638e+01,    -7.0809364100735020e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6455024847145435e+01,     1.5907379850680567e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -8.5340380353919763e+00,     2.3348241495255352e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.9836033019381546e+01,    -1.6552255800576819e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -8.5801760741828062e+01,     8.4091932840310055e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.1288799409700374e+01,    -1.6552960432492964e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    2.7390048318717437e+01,     1.2803097490088692e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1078212183525000e+00,     1.2031939568397495e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4513142076878337e+01,     1.7253394329374234e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    8.4317008060367673e+00,     2.3902624118423566e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.3326978568531080e+00,     1.5610769261230032e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -5.9195169275059598e+00,     2.3991952931972035e+04,    -3.7963752239047967e-04,    -1.6439062299062145e-07,     7.6367851048374099e-10, } , 
      {    4.9390672249490493e+01,     7.3191899143322680e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -3.1621438580143494e+01,     7.3184852824161198e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -6.1951084895645906e+01,     2.3347919576329627e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1633075924508368e+01,     8.5600489167941378e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3826062616786663e+00,    -1.8127659882172111e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.6399368547432999e+01,     8.3708343347904774e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8735023726848841e+01,     8.5262674892911728e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5326955695381082e+01,     8.4434555636466830e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6957637665246796e+01,     2.5059697411903962e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    3.4509114557799208e+01,     3.8930482054860768e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -1.5186637623851585e+01,     2.5088771464307039e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -1.6539487129686563e+00,    -5.2616268444636626e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.8171348122448279e+01,     7.0020493516693223e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.1006282267513228e+01,     2.5092926549874439e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.5172741283151460e+01,     2.6662932885862687e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    4.3289244353818674e+01,     3.2516762538220024e+04,    -5.6287750737716745e-04,    -4.5172514737381015e-07,     2.1245989947263195e-09, } , 
      {    5.5264485914097747e+01,     1.5704616921764089e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3013869310571618e+01,     1.5629495339111993e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.1977139248129404e+01,     1.5647169993590518e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7652694601775149e+01,     2.4445620741236413e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.8060683288835989e+01,     2.3806485883891637e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -2.3215063196796301e+01,     1.5647888052608121e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1542693893260799e+01,     3.3418829086348996e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.1636339995679887e+01,     1.7847920486916810e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -8.9730633451827629e+01,     8.3355969546301276e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5271290764457364e+01,     3.3420251777282741e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    3.8004626447462604e+00,     7.1004097440907326e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.4788646960212063e+01,    -1.0380833046768203e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.3225370338287796e+01,     1.6293885551500236e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.6227338155463578e+00,    -8.4245877691877486e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    7.4290049336598285e+00,    -2.2763360814652469e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2104722132564145e+01,     9.0644035750022158e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.9491814820588473e+01,     1.6293109604911719e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8156915340851882e+01,     2.5346799138746806e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    8.9919888273333868e+01,     1.2904760652484310e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.3660265203902515e+00,     2.3726856080095811e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -6.3866255984577485e+01,     9.0097529159685309e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.1532938571840621e+00,     1.2135032380952373e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.5418833520879332e+01,     2.3881048615740190e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.7988161949733037e+00,    -8.5202166823546431e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.4383634656803821e+01,     1.2014425718386534e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1701328069080162e+01,     1.3178069516426305e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    7.7745018184786741e+00,     9.1611867923371946e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.5558908814301759e+01,     1.4068919558326401e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -8.8233875036324179e+01,     8.0501859305878497e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.1454116441419263e+01,    -8.6530202888454533e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.8014453865367219e-01,    -8.9137552310487445e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.7606816398788723e+01,     1.4183665587017245e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.8904420517797735e+00,     2.4069520452352208e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -1.5305080678053480e+01,     1.5634022511827967e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.2320553355890231e+00,    -3.6579849778975040e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.1578138839035832e+00,    -1.7266512240573687e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    2.9470558349478825e+01,     4.2828902835201716e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.2623890229368442e+00,     2.3283478512966234e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    9.8818328324120248e+00,     1.5018510092586901e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.9704747176839120e+01,     1.5003679409765346e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.9159142495309744e+00,     7.3045248564471967e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.5766931881477834e+01,    -8.4242926290926225e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7080565597119254e+01,     2.3608936627653293e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -2.8464849393729146e+01,     5.0625985536332610e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.5508445567290767e+00,    -4.8107754001975124e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.1273830803259077e+00,    -7.1505810975969894e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3683922608771345e+01,     2.3870282253402063e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.0797665005209240e+01,     2.4111207181806611e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5379332644027579e+01,     9.3721733812667935e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.9556782843775949e+01,     8.4131160144453788e+03,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -4.2812593333305131e+01,    -8.4538163008355950e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.2737385912736357e+01,     1.4452733629872764e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6076580172481503e+00,     8.4296633880311620e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.6422608313032704e+01,     1.4884234704071610e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0338310502903964e+01,     2.2326584308040372e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.7905312295986420e+00,     9.7518598741063997e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0072926356557758e+01,     8.4372689272498119e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1726003071761632e+01,     2.4603816687403112e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.4806453159518441e+01,    -3.6427324453853362e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.0602669115542128e+01,     1.4555240532547152e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.1395128280429343e+01,     1.5665161049374976e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.2056068609439620e+01,     2.3991639558863637e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.7322459692708119e+00,     1.3682424174634383e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4685278307464724e+01,     1.3935759733342313e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6009774849175503e+01,     3.9361240383667966e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -1.4833836680385311e+01,     8.5473404495863724e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4842415178855394e+01,     2.5514522899093427e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.5834712023565960e+01,     3.1348017667623706e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.3579596749340155e+01,     7.1519479484991753e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7678706232713576e+01,     3.1014515274955520e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    1.3010722769957653e+01,     1.3728830470256786e+04,    -2.4893105098351117e-04,    -1.5510711973652738e-07,     7.7768477773099544e-10, } , 
      {    1.0612044435820042e+01,     6.0284410007801407e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.3280289647708937e+01,     2.5685369534836260e+04,     2.6886755917809749e-04,     5.3341208324310941e-07,    -2.5814825596671350e-09, } , 
      {    1.0298594423981644e+01,     8.6092341959682453e+03,    -4.6576215180844699e-04,    -4.7874866195885568e-07,     2.2587469402893278e-09, } , 
      {   -7.4479096720027336e+00,     8.4002370196700067e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.9837160799438145e+00,     3.0551255087761301e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.6504583129422592e+00,     2.1674866496487037e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.8162671338136356e+00,     1.5019227067850838e+04,    -2.7985828532885068e-04,    -1.9203052968982966e-07,     9.5226618429853939e-10, } , 
      {    1.2883970872126840e+01,     7.7905648148002065e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1159048550141579e+01,     1.5357797463940291e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.3020315179932130e+01,     1.6755546182812319e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.5456747905391044e+01,     1.6102335558716552e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -3.5960330382603765e+00,     4.7047616485602030e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.7562472420252661e+01,    -1.8477041636764711e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.5922324609987850e+01,    -1.9273406810230219e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6100937563349756e+01,     7.6853379658447002e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2810750098995612e+01,    -9.8025261565826841e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    5.9890712467108980e+00,    -1.6009897070482601e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.8239411476406641e+01,     9.1883565018396821e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3343609921349202e+01,     3.3459355333713786e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.1847258694829279e+01,     2.2729594683739557e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -2.1669451238213178e+00,     7.1006516341600418e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.7218279713916584e+01,     3.1623635190292836e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    4.4069682486349791e+01,     1.5661035534370674e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.6883869809409994e+01,     3.1112731052271152e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    3.4799519792663531e+00,     3.3379725529917952e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.7967435351257095e+01,     2.4826696737023085e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    4.7158680168532129e+01,     7.9644181801865925e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.3571237994819212e+00,     1.3316066632561925e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.8350418008301709e+01,     1.4510358558701970e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.6707972973668541e+00,     7.6388799505102355e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.3035669366841205e+00,     1.3977229367011736e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1339846659022729e+01,    -1.1120255352778133e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.5562673250812318e+00,    -1.5454900238077378e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    9.8199966286446703e+00,     8.1010578165786019e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.2031501135134206e+01,     2.3332370834490473e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.4891915039901878e+01,     5.1802659114212338e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.8767002715699384e+01,     2.3212926128796738e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.4397627339605803e+00,     8.3922065635684648e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8442952791671118e+01,     3.1835511579137605e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.7848525912813011e+00,     2.3518765128318792e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.8316811764816666e+01,     1.9342452104676639e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8054919599368354e+01,     1.5543917751690980e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    8.5828453957099420e+00,     1.3865895678496647e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.8065249227554130e+00,     1.6818824561278998e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3636408623019888e+01,     1.5881969931580318e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4604794869181834e+01,     5.2641802793047545e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4055501599346009e+01,     1.4874106322035721e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8370189953619960e+01,     9.8098817163583885e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.6944299317037785e+00,    -2.4546885542948889e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4743176570349139e+01,     5.2086176338912028e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -6.1952257432403606e+00,     2.4819144570647979e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.7121385643333468e+01,     1.6569116547270998e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9840142267989584e+01,     1.0331008151099495e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0883839226365403e+01,     2.4796043814713386e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    2.3582806176847072e+01,     6.7807015126631541e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.3314146860826543e+01,     6.5545768768988055e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6897592159674343e+01,     2.4979515318575654e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.9399878413018747e+01,     1.8458194387975029e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1798016600884409e+01,     1.5900864581807249e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.3672304378692797e+00,     1.6165028400834555e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7011858079524782e+01,     2.3244301591370564e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    5.0776791139903125e+01,     1.5998863263337904e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.5437040488843223e+01,     1.0125347877682503e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.1060120406847673e+00,     2.3998762352955498e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    3.1914775430423958e+01,     5.6380564246743943e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.4999875605207336e+01,     8.7094730999997682e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.1030453936356821e+01,     8.8241859274937415e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.6619215858901256e+01,     6.7198699588854024e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3750984070814919e+01,     1.0289815619445617e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4183790355795924e+00,     2.3960801336785255e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.2316364201804937e+01,     1.6666597608487824e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8379587851263938e+01,     1.7987621157269918e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.5455671576499867e+01,     1.5702885953285660e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.8926699893108250e+00,     1.0869144236810536e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1231898571249260e+01,    -1.5167407161778192e+02,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.3071112491035256e+01,    -8.4311991687410427e+03,    -2.7586509320370791e-04,    -3.9500194668399445e-07,     1.8691506661497048e-09, } , 
      {    2.4339961713424103e+01,     3.5056058660400478e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.9044524714897397e+00,     1.6655327929018699e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2550698295787807e+01,     5.4892108634627994e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4326881300056794e+01,    -7.9783120379745660e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0684180147030963e+01,     1.3030158359164298e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3150541778676420e+01,     2.5196787201513122e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    6.4439889754718436e+01,     2.2024087153639961e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.7553084126109674e+01,     1.6120732086120614e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0397412827776048e+01,     6.6814929527115873e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.0748804216830070e+00,    -4.7713218630483880e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.0165398983606011e+01,     8.2545696141194676e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    7.9054470420903682e+00,     7.0912639243422664e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6401296733352137e+01,     1.5981590266571047e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -5.4079852326550615e+01,     1.6959501123376751e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.9865560160772020e+00,     1.6758354812756286e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2405097057163232e+01,     1.6765960351974940e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.6374295745671432e+00,     2.5072800063735365e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    9.3794802804503252e+00,     1.6275125262677986e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.8324110049975161e+01,     1.6053169231114687e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    5.0392204668242613e+00,     3.2559343312225472e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -1.2511054301131022e+01,     8.4426396861883513e+03,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.0646695553456365e+01,     1.4967406326563487e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.1969873580465258e+01,     2.2717304077893627e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -2.1089153452426629e+01,     9.5706366220378368e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.7746317064706520e+00,    -1.0300355228789791e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.5607329071939242e+01,    -1.2505816956961385e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8493227355459322e+01,     1.9368189254198471e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.1866079949438655e+00,     6.4633179324674825e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.2568455883558904e+00,     1.3815575365173881e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0260041574701244e+01,     3.2165192250624223e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    8.7010001357676217e+00,     1.5514420800727266e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0182548454207584e+01,     2.4291689156505308e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.3538930734179626e+00,     6.7266383638456582e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2621495587020494e+01,     2.5090849007090739e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -1.1806496988938637e+00,     7.0191166332089688e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.1539324493931829e+01,     1.6575150021665948e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.8834249141591908e-01,     7.6683373530128847e+02,    -2.4982722907304216e-04,    -2.9171171318445421e-07,     1.3259702659713919e-09, } , 
      {    1.0396614569724683e+01,     2.5752908009476669e+04,     2.0711102285100261e-04,     4.5952163010520505e-07,    -2.2330469670537115e-09, } , 
      {    7.3412732796306450e+00,     2.4496304098979701e+04,     2.1243878583681962e-04,     4.5828884587687969e-07,    -2.1241675105501322e-09, } , 
      {    2.3703973266155913e+01,     3.0946976800315111e+04,     1.1478710513337172e-04,     4.2022195437531245e-07,    -1.9978686984842869e-09, } , 
      {    1.4077302707382614e+01,     1.4951712973575946e+04,    -2.1819968136533988e-04,    -1.1809644332062543e-07,     6.0455781220678014e-10, } , 
      {    1.0427956004250623e+01,     7.0957665452755464e+01,    -1.9063664187527163e-04,    -2.1724850116368706e-07,     9.2236772458451420e-10, } , 
      {    1.7730417535790757e+00,    -1.3276215962748793e+03,     1.9083250660243989e-04,     2.1716123470108737e-07,    -9.2382216562784296e-10, } , 
      {    1.9818655017050670e+01,     4.9138087140693096e+04,    -7.5109953471367305e-05,     3.3647523910045122e-07,    -1.6082724243446637e-09, } , 
      {    1.0837261629801024e+01,    -9.7914012298749476e+03,     1.5670506562946441e-04,     1.1576381077533500e-07,    -5.4323372968323497e-10, } , 
      {    2.5810928300789296e+01,     2.1571170910598579e+04,    -4.6999787197756480e-04,    -3.4099855074520321e-07,     1.6001275545020233e-09, } , 
      {    1.4462844503275486e+01,     2.5128079635568629e+04,     8.3362815561474656e-05,     3.1501806431369674e-07,    -1.5487809375357125e-09, } , 
      {    4.4154855015787291e+00,    -1.7495264280692034e+04,     1.2814052227764412e-04,     9.8562621369568663e-09,    -4.1160681526199562e-11, } , 
      {    2.9601875447358106e+00,    -1.7285714814861312e+04,    -3.0452756891911236e-04,    -5.0086586209107275e-07,     2.3719509348284047e-09, } , 
      {    2.2103115864314734e+01,     3.9971544294888765e+04,     2.0790926857694944e-04,     5.9616985678443174e-07,    -2.7806052347724438e-09, } , 
      {   -2.8943233034860212e+00,    -7.7376501081171136e+03,    -3.0119049938930325e-07,    -6.8367746570913202e-08,     2.7305191333770177e-10, } , 
      {    6.4565995078413794e+00,    -1.8477052474301365e+03,    -1.5181569541479069e-04,    -1.8213063432157294e-07,     9.1183272329400408e-10, } , 
      {    5.5132179088162223e+00,    -9.7576441827375020e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -6.2588421786928805e+01,     1.6761951867138407e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.6474649522308731e+01,     3.1088111604356396e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.6166541112882989e+01,     2.3764041149677541e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    7.2245812433594558e+00,     2.5697826684158430e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.3358416910504260e+01,     8.6550132340185846e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -6.3884626545912973e+00,    -3.4661954218244900e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.6111207108120453e+01,     2.7278122559708776e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.4459520554348416e+01,     3.0092575434825765e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -3.4444566212854483e+01,     1.6602897974773965e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0218077311963718e+01,     2.3769300814309907e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    5.0763691254090846e+00,     1.6857717556243402e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.1071139503531217e+01,     8.8886202773064069e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.3439290851837136e+00,     1.6748994717373625e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.7965225149345986e+01,     2.1472979513648472e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    8.1891110694283888e+00,     7.6736059936554602e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4110060586009270e+01,     1.8098954845785007e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.3916870819189743e+01,     8.1025783379204331e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.9443905993415420e+01,     2.4617147271192367e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    7.7273746573406159e+00,     1.5619143089689062e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.0674122184656156e+01,     8.3443277999511811e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4514650862865366e+01,     3.2351312538374917e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.2088317106555437e+00,    -1.3433562190191219e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1945883865850440e+02,     7.1072526155263968e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5053443688053081e+01,     1.7863668230432449e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.8114414751476922e+00,     8.2834372544596517e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6110583965581146e+01,     7.0886695700324972e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1987521203433570e+01,     3.0561623412279656e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -9.8779378324563822e+01,     7.1113236495605470e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.3092141853597923e+00,    -1.0155072393974471e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.2768366390036704e+01,     1.7108279213356182e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.1134555015134198e+00,    -9.5672823866150065e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.4702663631706457e+01,     3.2935520657434856e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0298252136930605e+01,     2.3240535105875060e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.4622086408493429e+00,     1.2547403631171881e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -8.6266133100007885e+00,    -8.4051144351705207e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    2.6314254620081272e+01,     5.5732868811142216e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.7644310669483119e+00,    -1.1138613246865424e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.9988185358515963e+01,     2.2719252961775284e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.3546983636230507e+00,     1.6202213470416282e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.7185396187135769e+01,     1.6473944715186746e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.2830445289352664e+01,     1.0995726995964189e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6883364153425436e+01,     1.6882814285212553e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.9609662369552412e+00,     8.2119190812623892e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.6474553810747178e+01,     8.5293807351163887e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {   -4.9894603488097040e+00,     4.8959993522160090e+04,     4.7904158638498206e-04,     1.0606414345737651e-06,    -5.0338689323284230e-09, } , 
      {    1.4432886784429829e+01,     2.5126186078026272e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1050637188950082e+01,    -8.2112974534873865e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.9018834880551182e+01,     3.9571943835219594e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    1.0856643389067372e+01,     7.1299066950544466e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.8021263260173697e+01,     1.1274784444825902e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.1478518346736010e+01,     1.6730677354702380e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5225684411770867e+01,     5.8503717626126509e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -4.8867325578968263e+01,     1.1623153935168857e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.2726416775028341e+01,     2.1149822598801147e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -5.5523121483339111e-01,    -8.4338619917283031e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -1.1248398367389736e+01,     9.2088274254580920e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1700523251503594e+01,     8.4177184141248472e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    4.2446905045002090e+01,     2.2103364641477710e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    8.0961114166918946e+01,    -1.2747602223679960e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.3424915516268186e+01,     7.5383917295791562e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -4.3976663320153442e-01,     1.4769857070325143e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.9689655834984087e+01,     3.8892968352802593e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    2.8159141829430247e+01,    -1.0323957314819891e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.4462297174862777e+00,    -8.0999637649722972e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    8.5528011705827254e+00,     1.6345726386164835e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.0464363450187051e+00,     6.5121321426847089e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0969778432629610e+01,    -1.6739615677234557e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -5.7158026202205619e+01,     9.3438141594402587e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.5817387628149447e+01,     2.5075306978802313e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.0250605189045789e+01,     1.7022622496436466e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0440068328803676e+01,     1.6207473135048649e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4095773388306085e+01,     1.0019761999218194e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.5012291153188162e+00,     1.7625751782763713e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.8519256622788557e+01,     1.0568924598768235e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0900622396582660e+01,     2.4140513757764602e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.7427924237455535e+01,     8.0427463877872324e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4194832747317717e+01,     1.5231097826898542e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.5998415606166702e+01,     9.6718419285170385e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.7552327064537337e+01,    -1.1242314600974000e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.9360699151266072e-01,     8.4540847201514625e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -8.2178396017026998e+00,     7.1318300376745283e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -6.9997827407501745e+01,     2.5106391035379165e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.0486145697096568e+01,     2.5706038680936828e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.1724675216841831e+00,     8.1392198160666712e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.5552697439916507e+01,     2.3178178298846095e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    7.2141344465828311e+01,     1.6761345536380140e+04,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    6.4174424831795349e+01,    -1.6762969628351086e+04,     7.0678368575081252e-05,    -1.9863786142419908e-07,     9.5110747960068764e-10, } , 
      {    5.8037482398917355e+00,     8.4128475951295113e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4671849309694753e+00,    -3.1845081406465915e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -7.6690603832413844e+00,     3.5283285717974250e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6593997735142828e+01,     2.7055953855555676e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -5.7496733573933980e+00,     3.2327453777680628e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.5590275948192161e+01,     1.3144288088923342e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -7.7763551824368278e+01,     1.5623256149489838e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.1849282410675492e+01,     1.6921417189957261e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -8.9355060211050130e+01,     1.6762059530791776e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    9.3893280124188578e+00,     6.2456939321453729e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.8093385602599543e+00,     1.8018772752456698e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.0674677640195290e+02,     1.6762255633939450e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.7172628341213009e+01,    -8.7643539773605426e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.9937988550115250e+01,     1.5444013031663544e+04,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {    1.2047408099683539e+01,     8.2805836707231520e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7795728061402340e+01,     2.1630713867988899e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    5.4631560665846282e+00,    -6.4449321310955183e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.3413621170092238e+00,     1.7978957850558789e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.5811559814389909e+01,     2.3347584230302451e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.8490438906908111e+01,     5.2753906197862307e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.0302133326294598e+00,     3.7636151871499806e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0073896455158213e+01,     3.3876995751321527e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -3.0002222170734569e+00,     2.3862809216525657e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.7301720900665405e+00,     1.2561630540509409e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.0081138308385764e+00,     9.6502664258336608e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.5877160288230058e+01,    -8.4119867638786473e+03,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    7.1840616242336841e+00,     7.0371313656240536e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0438012855280125e+01,     2.4621448713678772e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    8.9591878159568274e+00,    -8.3341393627713969e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.5996859737241772e+01,     1.0949326330138843e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2845686598222779e+00,     1.2017712659059966e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.5332742511861845e+01,     1.7971873413754278e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.6314033192195353e+01,     4.8964595921839558e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -1.6660790716936109e+01,    -8.5468773889392760e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.1842497540534808e+01,     2.4019991281333601e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.8112976988003361e+01,    -8.3373279231085544e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.9487296737732997e+01,     1.7667438512218119e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.6002966589367986e+00,    -1.8503407095433822e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1762058263730369e+01,     4.3023284760241961e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.5550847718984455e+00,     1.4921005660738143e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0262626377954284e+01,     4.0111233906651541e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.1900221582935218e+01,     6.5611626408262355e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3624769681676664e+01,     2.5127608768960024e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.0337897918924117e+00,     6.9258166286812066e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.4552242529335052e+01,     1.2544366437288058e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.2156920434289994e+01,     1.7294863963043659e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.3338724324028313e+00,     1.6751848301110123e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.0567687736133244e+00,     1.6772466863621103e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.5105632948546599e+01,     8.8133757774050755e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4628897381103950e+01,     3.1583304282976263e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    5.4223671157401645e+00,    -7.6441243458667969e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.9646805955277188e+01,     2.9871534993219258e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.9305937996768343e+01,    -8.3771428250064673e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.9896132148760639e+01,     3.2326241780527424e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -8.2220022341418986e-01,     6.2330746938229931e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8769401951443090e+01,     1.6513753055813602e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4994553128634912e+01,     3.9466005640797208e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    5.4238446036353352e+01,     2.3975861418315642e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4214847104467745e+01,     1.5518161650596905e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.8427762037023736e+01,     6.6955970852759201e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.8715550249788649e+01,    -1.2197623216904558e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.7806488425435582e+01,     2.2291439747319600e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.0764907010552694e+01,     2.3976579477333249e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.6465129961540026e+01,    -1.2190442626728527e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -6.1786468210303951e+01,     8.4344144281341796e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7915807652259897e+01,     1.4812301804539240e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -7.8528812993341930e+00,     8.2152801934263371e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.0433216406727199e+00,    -8.4792451956908262e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.3168352531731948e+01,    -1.9826976112093635e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.0366503251356022e+01,     2.4604880348828061e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4752500156556533e+01,     1.5675914956509579e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.6782146084489185e+00,     1.6790491071822307e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7712550588637587e+01,     6.2603305353249898e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5811347623732551e+01,     1.2407446688079523e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6494682541582055e+01,    -1.2259622477813837e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -4.5414028070424457e+01,     7.1088999907499365e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.6453461477279745e+01,    -1.8632529055156274e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.6093697484891990e+01,     7.1096762743370073e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.9245475144377497e+00,     7.2232516533747448e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.6276422861318984e+01,     2.4876232469312745e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.9298146789682921e+01,     1.0444610043891203e+02,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.6015186493245505e+01,     2.5214801933928211e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    7.7176792237967735e+00,     1.4729489878894747e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.5057184459380721e+01,     8.4411543638069470e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.6171531001962343e+01,     1.6116577000553214e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.6357675088040253e+01,     2.8221400932633092e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1370182765432578e+01,     8.7675274011122146e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4680963554545896e+01,     8.3658920430404360e+03,    -7.4945892521679825e-06,     7.0129656450801486e-08,    -4.3782553913639972e-10, } , 
      {    3.8914497798123676e+01,    -1.0661771589849532e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1287403775571757e+01,     1.1262165206503523e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.8516714668395142e+00,     2.1568928302064654e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.7893404190696099e+01,     4.1208941785109047e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -2.5223690721870842e+00,     1.1934572257534675e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.0490539986763004e+01,     1.7939142948660876e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.5096763628616095e+01,    -2.3565737565790041e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.9359253743030231e-01,    -2.5071661337382509e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    2.5058803233878425e+01,     2.4755197893956656e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.8534842816138536e+01,     8.0871989893491736e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.1719833883802337e-01,     1.6982807594538550e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.9265113290977123e+00,     2.4984910812668357e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.6401742023059462e+00,     9.6836893331345971e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.3085254233452588e+01,     2.3360532101101282e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -2.1340064407479073e+01,     8.2139120264355242e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    3.7023397586467066e-01,     2.0033470679315133e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0510236857337263e+01,     1.6257812399943658e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.2553720970278768e+00,     9.0591439103698467e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.6308423463690813e+00,    -7.0190391444641978e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.6359697586019717e+00,    -1.8484222226940738e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.7736502860710541e+01,     5.9663078508072085e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.0770310804915047e+02,     2.5090341048552702e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.9502507927061370e+02,    -2.5091356965628776e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.8560962512914589e+01,    -1.2329098034530082e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4035006905102280e+01,    -6.1091217669855268e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -9.6853469201669817e-02,    -7.5881946748273799e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4298997196074691e+01,    -6.2925932708477922e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.6999392766676543e+01,     2.5015597945918351e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.5502871861366163e+00,     5.3794698994310902e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4620000150722849e+01,     3.8968219413974977e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -1.1860956276739600e+01,     1.6508822023657682e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.5994042931149712e+00,     9.2147590375702748e+00,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6607982993403233e+01,     2.5765557257961627e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -1.2218764635855897e+00,    -8.8775224590875268e+01,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.8927529950279709e+00,     1.3658994996554351e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.0809008588247096e+01,     1.5031840676376156e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -2.5132047394402200e+01,     9.9843700759440253e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6709349027465175e+00,     8.5290261315182761e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.5732407825235811e+01,     2.4462189106087124e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -5.8221383002031800e+01,    -8.4456720824600743e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    2.1960919212400142e+01,     3.2010665530975755e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    8.7772669456290018e+00,     8.4503269043346718e+03,    -4.6576215180844699e-04,    -4.7874866195885568e-07,     2.2587469402893278e-09, } , 
      {    1.8278061063703710e+01,     3.2283581964571720e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.0024636818826725e+02,     1.5671801896708801e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.9562128020130727e+00,    -8.4606961197047549e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.8808608694975057e+01,     4.0045110614127072e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.8713489520230333e+01,     1.4792751291414274e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.4936903106556088e+01,     1.0125700193640578e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5476194061360232e+01,     2.1899781910844416e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3736370789526251e+01,     2.6269911916169698e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.4251825922274829e+01,     1.4041663716824403e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.2789142100515405e+00,     4.2805890836804356e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3172898435277833e+01,     4.0686572394485891e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    2.6759158917877496e+01,     2.1051606821485511e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2142619807303641e+01,    -9.7046750855263108e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -7.1273034235673585e+00,     3.2302834329765872e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.5797732650213039e+01,     2.2783595001666916e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.7178706458944379e+01,     2.4569583143860000e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    7.5978937860497950e+00,     2.4775380298409877e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.0651914015067355e+01,     1.5643726253489995e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.9129252309195728e+01,     7.1179246309269001e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6108138973242909e+01,     1.4328780703035296e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.6979314152866166e+01,     6.6740215825925425e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8891488664227790e+01,     1.7739333508606007e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.5846827586749621e+01,     3.3437564640017066e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.2151819091435879e+01,     4.0067397689901850e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.9029757127735266e+01,     3.2306989415333275e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.3606477094716105e+00,     4.6736100533734234e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0005322450019665e+00,    -3.0513569501070924e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.7797951404312396e+01,    -4.6114769034004885e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.0599294124965589e+01,    -8.1486656306373770e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.0163905800047979e-01,    -1.7079912553018503e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.8773224075135957e+01,    -8.3919965239710655e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -1.5935010839977593e+01,     8.4929694752972391e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.9190821329506775e+01,    -8.3739628399837347e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0121153429375285e+01,     1.8435203948657479e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.6908296276574042e+00,     1.0857750252468634e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.7137692405246327e+01,     2.6403434050700980e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.7870124716106481e-01,     1.0097196330603526e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.2684923435347336e+01,     2.3900969386652061e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -7.2802627365698470e+01,     8.4304069530410616e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -7.6058437855227652e+01,    -8.4365253622399123e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.3770992260464013e+01,     3.2287638875782715e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.8111181301082233e+01,     1.5651331792708645e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.1608107219850119e+00,     2.2572412618931605e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -5.2481729137247108e+01,     1.0339964415460494e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.9147256662935126e-01,    -8.2452465997876170e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.4296536067782135e+01,     3.2090295334771581e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.2792351124645961e+01,     8.4123039684598989e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    3.1731020701619393e+01,     1.4610467984213195e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9055217422257527e+01,     1.3716373320934614e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8769824750136038e+01,     3.3412981476216140e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    8.7957373448388161e+00,     1.0147695991388032e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0829977754418742e+01,    -1.7090744202245933e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    4.0813895889851956e+01,    -1.2058967809103001e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.9298309224527713e+01,     7.3023291676380850e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4886753405187434e+01,     2.2763376111242516e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.1795284185924375e+00,    -2.6306793805933125e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6153654785878633e+01,     7.2681954241770463e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0333717116044799e+01,     4.0475868942934263e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.1649016853721175e+01,     2.5560249300502706e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    9.8681841645899340e+00,    -6.0912939112343656e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    8.6579064867219031e+01,     2.3920863517638107e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0071486152486509e+01,     1.6177620830829230e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.9176317846583913e+01,    -5.4484874676288200e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.9062286696692770e+00,     1.6675416533437579e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.7985144521178853e+01,    -1.0176754617910399e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.5267178711492480e+01,     4.0305016677394378e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    1.3678839104858804e+01,     1.5665553231300522e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.3156964766892774e+00,     8.0994049141687592e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.8918528501957965e-02,     7.3319555095736487e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3940060200419710e+01,     1.8169948151197405e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.4176630921094954e+01,     2.5305465544868734e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.1840650453152563e+01,     3.0732827993777624e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.0428281643714008e+01,     8.0710981102572014e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.1744328326662144e+01,    -7.1561874712458912e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.5798892085470390e+01,     1.7018107714021680e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    2.2773122233185582e+01,     2.1085388248988478e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.7157978833540554e+01,    -6.7969479290558675e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.2704198532640376e+01,     1.6535335119768035e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.1992440434919317e+01,     1.6760208698483952e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.6823661206426372e+01,     5.7304676378073782e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6905546160528559e+01,     2.0873057862013182e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.0011677782446259e+01,     8.3379061837626978e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9485642990222271e+01,     4.0580634200063505e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    3.9208029342297719e+00,     1.6754679415796556e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -4.1214488251907675e+01,     1.0379779317358407e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3470513191942992e+01,     1.6769635748934670e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.8940327070589440e+00,     6.8488232184726203e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3301009648327664e+01,    -8.1174571930904995e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4290666347720629e+01,    -7.4620022916035941e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -4.1956250385261598e+01,     2.4071392279908701e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.9158733342322094e+01,     6.3593288205001954e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -9.3934520955249248e+00,     2.5043701847427110e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6651760059953027e+00,     1.6131220165003884e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.8346936125028392e+00,     1.7178588778566391e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.7751292459013804e+01,     3.1677315632970905e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.4058567720883822e+01,     2.4193473379189680e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.9377047197499458e+01,     8.5927257652321350e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.0778874742414155e+00,     1.6291584351660502e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.6802614280625896e+00,    -8.4022877541260023e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7364546987965234e+01,     3.9479159836110484e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -4.0942761624630833e+01,    -8.4905540563052564e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.6493312688640049e+00,    -1.9151578184339633e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0078273060738388e+01,    -9.3646232130445132e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    3.0996038355381579e+00,    -6.6471884486469025e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.4532883009059411e+01,     1.2769316062585731e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.3885539349182761e+00,    -1.1216321407451344e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3685256381932054e+00,    -8.2025867838319136e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.8868120224064445e+01,     3.0011220404204312e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.1572738098764084e+01,     7.0847049687425388e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7533893938716705e+01,     2.7671143529401765e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.5091731899705156e+01,     3.9516344905692218e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    9.4739424523660443e+00,    -1.0908321158406206e+04,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    2.5762436498542016e+01,     3.5449079630093467e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    8.2817550233754833e+00,     2.3887202934210847e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -1.8859879681117665e+01,     2.5300727105397909e+04,    -3.9751134229741748e-05,     2.3556127137750133e-07,    -1.1256888861682315e-09, } , 
      {    3.1768903803295000e+01,     2.5300069840445012e+04,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {   -1.0889633416682159e+01,     1.5019550070530226e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.2658848490622425e+00,     1.6770324625351477e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.7569823177449688e+00,     1.4450432430033032e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.8806983086222417e+01,     1.0459409121154231e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1261333510969301e+01,     1.0724029120792482e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6935142522196358e+01,     1.8884996785170981e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.0580860359548677e+01,     6.7523497901931805e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -5.6587217173754958e+00,     4.0840851563265685e+04,    -1.0593925545312270e-04,     2.9950819591584911e-07,    -1.4344182382987841e-09, } , 
      {    4.5644798432952399e+01,     4.0845453962945146e+04,    -4.1056475673581917e-04,    -2.0131403294392373e-07,     9.3825991705128483e-10, } , 
      {    1.5553816637274466e+01,     1.4852331788178759e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.0405000678018510e+01,     2.6621045390831423e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4159918461152373e+01,     1.4845754599565751e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1219347137506546e+00,     5.2120592911613858e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.1213900153098528e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0222248029995336e+01,    -8.7800864615081300e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -4.6005566349657094e+00,     1.6764106466247275e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3765368209956964e+01,     8.4402023078236016e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.9176482091824106e+01,     3.9521604570324584e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    6.4779982333510748e+00,     6.8302306836817570e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.2964379517160495e+01,     8.1209804168521368e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.7829173718274731e+00,     2.5152982157063670e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.8810612044843019e+01,     1.3196118459781677e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0892790278220176e+01,     4.6759112532131567e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7734448036103117e+01,     2.6236130488666742e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -5.8709976745404738e+01,    -1.1640463619953125e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.9500257803890278e+01,     4.6644495346541407e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.1835601489408631e+00,    -6.5945918023232389e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1476749946842135e+01,    -6.9727159673836250e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -9.2733187155858197e+00,    -8.6653556921853342e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.8051352129536408e+01,     1.5551969049221530e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.6160656986177671e+00,     3.8888365953123124e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {   -5.4853162266751639e+01,     2.2652049136278154e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -9.6260872296384186e+00,     5.9923583734374442e+03,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.4070730644963799e+01,     8.4329138681080040e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.7673344802760354e+01,     8.4340184471729699e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4006400390799927e+00,     3.6523964698621194e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1300164811494657e+01,     4.1026624266967687e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    8.8096134388096168e+00,     1.1225289948441753e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0260986842920400e+02,     2.2652035709176707e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    7.3668841554879121e+00,     1.5674758985163588e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.6167536426883480e+00,     2.3616980905634424e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -6.9718761593475058e+00,     1.4878364077795715e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    4.4331644559917677e+01,     1.4882966477475184e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -4.4526950134140257e+00,     1.5709662173072251e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -8.4040393889985676e-01,     9.7296566346304076e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.1058679544357695e+00,    -9.9153785370779333e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -1.7963333285074278e+01,    -1.1441522310092660e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.8111750188812039e+01,     1.5646562620651641e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.0892465420535238e+01,     1.6410823342127027e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.6633306300938084e+00,     3.9651203749995721e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.0971491156572952e+00,     2.4038353597797377e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.0504157824448402e+01,    -8.4267300074573723e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    2.5128747430324472e-01,     6.1321122063030798e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.1372533625893286e+00,    -9.8106132799486932e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    3.8336198743381944e+00,    -8.6261548767770255e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.4531551773547045e+01,     7.8151479786011387e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7092265379699274e+01,     1.4827705656210379e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0203565162264697e+02,     1.0487278448919969e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.3221644579201715e+01,     2.3805368182284554e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    4.0111976774161695e-01,    -1.0902925664313500e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.1624124068376174e+01,     4.0593788395376789e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.5507628512341203e+01,    -3.5317925946737159e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3155286374504310e+01,    -8.8534204870992653e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.0727699881589743e+00,     8.5327929525095769e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    3.6964831495492710e+01,     3.9558789639906317e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    3.6425786956904012e+01,     8.2468814079530111e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.2041456326426712e+00,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    1.9618061090570727e+01,     2.3330992181970116e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -9.4066186995877956e+01,     1.0467668134152191e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.2957009320682625e+01,     2.3962713936553093e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.9445994058538201e+01,     2.3921109112343627e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3441946625714795e+01,     1.4454903576941788e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5796612267081168e+01,     6.4472047497920239e+03,     3.1424289571897069e-05,     1.0520389006161575e-07,    -4.4908776094857428e-10, } , 
      {   -1.7101252607651845e+00,    -3.3453321859318829e+04,    -3.6198972262167523e-04,    -7.0935998565222865e-07,     3.3642190959552919e-09, } , 
      {    3.0235339328653872e+01,     3.0528199930531831e+04,    -3.1511917220689213e-04,    -9.1160195198179620e-08,     4.6895542560044285e-10, } , 
      {    2.2265052411987295e+01,     4.0809371335602453e+04,    -2.2732477174913142e-04,     8.6020491439264966e-08,    -4.2266056719129352e-10, } , 
      {    1.5366277342209536e+01,     2.2057521894981910e+04,    -9.6618300342162913e-05,     9.5303994693359024e-08,    -4.0865429994403896e-10, } , 
      {    2.2237085939543658e+01,     2.2685835109824515e+04,    -9.3954418849254408e-05,     9.4687602579196358e-08,    -3.5421457169224920e-10, } , 
      {    1.3266277492496457e+01,     1.4914446705138416e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6579227913381100e+01,     4.1923999894868750e+04,     1.4348885075694603e-04,     5.2289579576069001e-07,    -2.4866093704108100e-09, } , 
      {    2.2764077590171699e+01,     1.9794991651394426e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.4710990037560705e+01,     1.6863362888844815e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.2227727632828582e+00,    -6.0911991533731612e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -7.8060685219514880e+00,    -3.8592163915174788e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5279031014983278e+01,    -2.4988984893341353e+02,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {   -1.5412742165378972e+01,     4.2793164776268532e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -4.3748083344208965e+01,     1.0011936646383714e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1050578062134091e+01,     9.0272809697161902e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9954291268102871e+01,     1.6679674289197574e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.5509556534611839e+01,     1.7953839729766954e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.2428735482605392e+01,    -7.9694852521338616e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    8.1736383033529658e-01,    -2.2069504194172205e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.4080633463442798e+00,     7.0556076898654037e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.8566121773754745e+00,     7.2981740820706846e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.5785125408766767e+00,     8.4688032285760219e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    4.4715102983543824e+00,    -1.2477892255919128e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.2564761919929083e+01,     6.4959368806360417e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8186478308193056e+01,     1.6244658204630379e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8414658762860739e+01,     7.1629685752215400e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.6411735904364679e+00,     6.7036479245281052e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.8017286807405406e-01,    -1.8269807111324895e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    2.2528997435577274e+01,     2.5783590941948954e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.9352663817504707e+00,     2.3957032778116220e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -2.4662752823402613e+01,     6.3615358418041924e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2706360092947858e+01,     1.1655186176196510e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.9474866184260506e+01,     1.6066639458272217e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.9426705979655875e+01,     9.0678072170809774e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0942668724420136e+01,    -1.6358344883824320e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0824248084340423e+01,     1.6065575796847264e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.8313155732923107e+01,    -8.4257779514740268e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {    7.1054829319428752e+00,     1.6074331420295273e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.0716522473226886e+01,     6.7964492561787929e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4286613920609495e+01,     8.6556348617935873e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    8.0701215066709828e+00,     1.5647305366043285e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4410039778164172e+01,     1.5647752680155354e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    4.0329679094489514e+01,     1.5572277961926931e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.2497690959985491e+01,     7.7479049988445031e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0607289967484505e+01,     6.4809861772949898e+03,     4.9705522655755253e-07,     6.8280480108313485e-08,    -2.7450635438103032e-10, } , 
      {   -6.2234644255386939e-02,     8.4306126682682698e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2742823094553938e+01,     8.4363196470127041e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.5533597944196053e+01,     1.5849034210948908e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3509641776053547e+01,     8.2180166377233490e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0123730068095639e+01,    -8.7725944730594838e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -2.8235904194936907e+00,    -1.6250435001970209e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.4465111990988866e+01,    -1.0174101102112589e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.3923959365899981e+01,     7.7026508285790278e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.1594848666499878e+01,     2.2719605277733361e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.9853866522575256e+01,     4.0094313225842750e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    2.6718828333051174e+01,     4.7272327674899178e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {    3.8482750502524175e+00,    -3.1165646328541692e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.6850338614108082e+00,    -1.6638204655528145e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    5.5480120326269500e-01,     7.8025287402787599e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.9449128441988368e+01,    -9.4684649131109563e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.9065562672157590e+01,     9.1545750743367153e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2722419934083376e+01,     3.1267835574295397e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    4.8774130058212663e+01,     3.1796048993197765e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.5098391391957449e+01,     1.7928102580245119e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0646607241203311e+01,     8.3225321412119793e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1974079277249842e+01,     1.0380167459151942e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.5159966701634922e+00,     8.8498973538412647e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0284348523044013e+01,     6.5972280231757995e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3863257706698253e+01,     1.6118102253804431e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0275822421656301e+01,     1.2785048546733316e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.1434786744667129e+01,     1.5300908719231673e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.1315662918347343e+01,     4.0673418199172607e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    2.1826667262544593e+01,     1.6764424571265059e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.4057847881172796e-01,    -1.6692327738460237e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.7193513298390577e+01,     3.3693524247459260e+04,    -4.1056475673581917e-04,    -2.0131403294392373e-07,     9.3825991705128483e-10, } , 
      {    5.9137475297738771e+00,    -8.4929847346652859e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.5519921071504371e+01,     1.4595055434445063e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.4620209072390585e+01,     1.5072901880731844e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0831276437835470e+01,     1.8851215357668018e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.1104336404176856e+00,     7.1338712963444050e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.0178371079914150e+00,     4.3117406788672160e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0043776655310218e+01,     1.3630609063144093e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.2634523433893097e+01,     8.8073423030101258e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.6432752326044536e+01,     2.3989726959095799e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.3858143824360539e+01,     3.1422172847828122e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -3.2974807564500885e+01,     2.5033761108425970e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.6886219319014021e+01,    -7.6752389105600469e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.5700797252612201e+01,     2.4378057886230486e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    4.8366690052692562e+01,     8.4331708947627976e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.5110879563163394e+01,    -8.4337614205181781e+03,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    2.4483420347457713e+01,     1.6977740522591284e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    2.5863136928316827e+01,     4.1419645236660675e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.7305779837168807e+01,     1.1386118133340991e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3343159047694044e+01,     3.0641605532033554e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -3.6688766755422364e+01,     8.2742065500488388e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.7575174468260286e+01,     1.4924049605969523e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.6546808997598575e+01,     2.3559789251623668e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -8.7688674703968097e+00,     3.2542474508829969e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0966753265728375e+01,     1.8073704941245055e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3032409231540386e+01,     5.6016386035841915e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    3.1333314691448805e+01,     1.7467518755546203e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.4976327776908825e+01,     2.2235585503352337e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {    4.7759458055907835e+00,    -9.4716534734481120e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.6847712286542986e+00,     8.3995363130803089e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.0660501841272563e+00,     6.0108689086190825e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7313748236821162e+01,     1.6130721492126808e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.1213486267178437e+01,     2.3407431439803702e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.1759107343921690e+01,     8.1889718027997787e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.6363571599294218e+01,     1.5218295947543338e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.7217437609859978e+01,    -8.9042248767769706e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.1019602326402548e+01,    -1.2436761657911366e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -4.2999526137506685e+01,     1.3238370999368635e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.6141842858634298e+01,    -8.3401814124807697e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.6526299405098641e+01,    -2.1299517560523964e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -7.2726227433192577e+01,     9.3377806850453107e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7209278546306610e+00,     1.5663528531423801e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    6.7127338036328785e+00,     8.0318420591628310e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2327231967342804e+00,    -9.6463104942224127e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.6961233523560779e+01,     1.4393215052847967e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.1282646702620475e+01,     1.4923661464175986e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5475189703650324e+01,    -1.0367120552293869e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    6.5336048438185429e+01,     3.0562328044195805e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.1672100619770246e+01,     1.3277185178722266e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8696339842898475e+01,     3.0168954758544744e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    2.0098856584376009e+01,     3.0587712877759568e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.3487234337690310e+01,     2.3019326242898580e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    5.9894380423892756e+01,     6.0517474719419470e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2456401604659160e+00,     3.5720994053065851e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0527720811657737e+01,    -1.2067840538592750e+03,    -9.2625108217020158e-05,    -1.0766742230080575e-07,     5.0823018190712651e-10, } , 
      {    4.0832319497830362e+01,     8.7174546386581205e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2144702201505694e+01,     6.0218895353414473e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.1040872263354842e+01,     2.2719957593691433e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.0809945651462964e+00,     6.4372097139887692e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4672944659373719e+01,     1.6233176792189159e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.6033432172714114e+01,     7.6363904111098127e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.5883760854101283e+01,     1.6823547986946905e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9268223601334238e+01,     2.3924405336334155e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -5.4903183935068007e+01,     9.9721217444858012e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.2310107764118321e+01,     7.7531377458774459e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6264977102536406e+01,     2.5907543868786415e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -5.9256046178243963e+01,     1.0005903171988766e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.9692695899295298e+00,     1.0438148665291875e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7938762834997274e+01,     1.0516797917780285e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.0620934586694265e+01,     1.5507843612114260e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2903894605221264e+01,     1.6990894717904564e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.2886463455026828e+01,     1.7250745545485810e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.3725101227174559e+00,    -9.9673591515147746e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.5442524114552558e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1955781544789374e+00,    -1.0693376198482778e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.1605304956111013e+01,     2.5106581491238332e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    1.5293183258860621e+01,     8.9088912557686945e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9008082455796696e+01,     9.0214333595833377e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4533039904255979e+01,     2.2784039627546026e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -1.6256044167698040e+01,     4.6171307526565033e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0393017630860065e+01,     1.2013543488262803e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.7714391240728755e+00,    -9.0257253262836202e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.7421205127756419e+01,     1.1342347299831832e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.1677343183394528e+00,     6.8090083165747083e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1584262911984887e+01,     1.6733307187018570e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.5946197374216347e+00,     2.1775828984006956e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0601373458178049e+01,    -7.4883006147654305e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.7144388532209813e+00,     9.6164849983306958e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.2786096451265536e+01,     5.6778713265723072e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6969794068395738e+00,    -1.0569423271645312e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    3.3521811709527540e+01,    -1.8084034407622894e+04,    -1.7874973763498745e-04,    -4.2202546126903998e-07,     2.0032986117127136e-09, } , 
      {    1.5771102986922523e+01,     8.4702259195097740e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -7.0552397794431670e+01,    -1.6762363297592823e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.2106857997191776e+01,     1.6960368056094171e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1453979402122712e+01,     1.6570641800522219e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.4663131060709659e+01,     2.2731672226523257e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    3.5828327059591572e+01,     6.6342066806946295e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7237876905898020e+01,     1.2800467657772508e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.1006841284922970e+00,     3.1353076021044371e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7494044904390154e+01,     2.3783531728687904e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    5.8845089725684918e+00,     1.5663276766614959e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1780673248489315e+01,    -1.8814622945675819e+03,    -1.2098639343303529e-04,    -1.4516359113697083e-07,     7.3797853724812445e-10, } , 
      {    1.2390226230636847e+01,     2.5232884378646570e+04,    -1.3040528090757920e-04,     5.9084243316819144e-08,    -2.8705818058495628e-10, } , 
      {    3.4161868575673191e+00,    -4.8977017519567562e+02,    -2.4449946608722509e-04,    -2.9294449741277957e-07,     1.4348497224749716e-09, } , 
      {    9.7948748286206531e+00,     7.8423948308698946e+03,    -2.1859880422831334e-04,    -1.8642055666023879e-07,     8.7833694606614629e-10, } , 
      {    1.2009256393320715e+01,     4.1781988903678437e+04,     5.1440040562660764e-04,     9.5972746685081516e-07,    -4.5512853941519907e-09, } , 
      {    1.3943126598120205e+01,     2.4043734542099337e+04,    -1.9186802014392173e-04,    -1.4937109514984804e-08,     5.9195750463474314e-11, } , 
      {    1.6405508340217082e+01,     1.6032524465379634e+04,     1.7831134486384417e-04,     3.5689142195112735e-07,    -1.7435790746055242e-09, } , 
      {    1.3282653722506129e+01,     2.4567291774595040e+04,     2.4368093090872387e-05,     2.4046758583032984e-07,    -1.1466328371921636e-09, } , 
      {    8.8946105337629628e+00,    -6.9584605968595315e+02,     6.1756536327094901e-05,     7.3890453137904373e-08,    -3.4843559261342347e-10, } , 
      {    1.6432857454167440e+01,     2.3280404398301071e+04,    -6.5886930723991650e-05,     1.3231467110926100e-07,    -5.8178126546825431e-10, } , 
      {    8.3748921050089109e+00,    -1.7801911531552450e+03,    -2.1347429937830146e-04,    -2.5606472069077718e-07,     1.2595410953857633e-09, } , 
      {    1.8728700240903951e+01,     1.4390865092277192e+04,    -2.8232630209459094e-04,    -1.9150140403826672e-07,     8.9637201500342097e-10, } , 
      {    2.6890681300014947e+00,    -6.6556259388895160e+02,     1.5733932312777599e-04,     1.8081058363064789e-07,    -8.0440770783375299e-10, } , 
      {   -5.2498846825220049e+00,    -1.6066341532842240e+04,    -1.5261394114073756e-04,    -3.1877886100079960e-07,     1.4593909910127362e-09, } , 
      {    2.4836211375890581e+01,     3.9343182319315129e+04,     2.0544125181120920e-04,     5.9669898243599460e-07,    -2.8364994040675623e-09, } , 
      {    4.2977152109412700e+00,    -4.8629659385523155e+02,    -3.7091155486966158e-04,    -4.3683167109012521e-07,     2.0646760237411803e-09, } , 
      {    1.1587578658119465e+01,     1.5715043117374211e+04,    -3.4418077078526995e-04,    -2.6534822394487122e-07,     1.2455348281385088e-09, } , 
      {    2.2523782834452408e+01,     3.0386188939341519e+04,     5.5792382662769456e-05,     3.4567147589194555e-07,    -1.5957205981407376e-09, } , 
      {    1.7537745228221279e+01,     2.4428729984379650e+04,     2.6896549154168162e-04,     5.3336845001180950e-07,    -2.5822097801887988e-09, } , 
      {    4.7421313311115050e+00,     1.1855862247190489e+03,     1.8017698063080582e-04,     2.1962680315773798e-07,    -1.1415810786350020e-09, } , 
      {    1.2169975047623220e+01,     1.4215132693909080e+04,     1.2464438537905891e-04,     2.8110815923943514e-07,    -1.2325515177583955e-09, } , 
      {    4.9739351582885085e+00,    -8.8860357145208527e+03,    -3.4028551102371137e-04,    -4.6827600770773613e-07,     2.1631465305113390e-09, } , 
      {    1.7612503505072068e+00,    -2.6381269985050309e+04,    -2.0957903961674285e-04,    -4.5899250445364209e-07,     2.1771527977585930e-09, } , 
      {    2.6301726872401785e+00,     7.8389212495294505e+03,    -9.2186715445876873e-05,    -4.2533382982893138e-08,     2.4851064479993704e-10, } , 
      {    1.4061736793456642e+01,     3.2757481429429936e+04,     4.2641014044167864e-04,     7.8263404667597038e-07,    -3.6582149603169261e-09, } , 
      {    1.2012634322462757e+01,    -2.0234976661234123e+03,     2.5002309380021037e-04,     2.9162444672185450e-07,    -1.3274247070147205e-09, } , 
      {    5.1312985727292464e+00,     7.3525946455116391e+03,    -4.6566421944486285e-04,    -4.7879229519015559e-07,     2.2580197197676636e-09, } , 
      {    7.8687724810681221e+00,    -1.4289527580123759e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {    4.8837157342937703e+00,     2.5300398472921461e+04,    -1.9206388487109000e-04,    -1.4849843052385087e-08,     6.0650191506802891e-11, } , 
      {    9.3907979776444996e+00,     8.3040558169706273e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3708870882223408e+01,     2.5043949668388319e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0688682984208320e+01,     8.4320352648377375e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4328724946791418e+00,    -8.4348970504432364e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -1.9355700632204261e+01,    -3.4269774185515575e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.4173325896896763e+01,     7.1082244711185213e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1496744636257759e+01,    -1.1910173587713957e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.4854428698268549e+01,     7.1103517939684234e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5673078806675377e+01,     2.3215915657960304e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    7.9335300961346302e+00,     1.3997892883315246e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.2649519771807562e+01,     1.6762550828628056e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -5.2578431120859008e+00,     1.6761764336103170e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.3997272266104837e+01,     4.0007596912068897e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3271824468138906e+01,     9.4973926085950352e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9649765462497580e+01,     2.3053666521205087e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.9738765721910070e+01,     1.9735473074369627e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.1365284524929127e+00,     6.1681606227054890e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -6.0683115623988595e+00,    -1.8537443516221454e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.4842277976318137e+00,     1.6056297736307944e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.2151801804464426e+01,     7.7376024311397432e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9320449912978795e+01,     1.5753448265949461e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.9817735536182191e+01,     8.3239548321457314e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.0303631394658790e+01,     2.2688090839129833e+04,    -4.3916856999580938e-04,    -3.0403150756060110e-07,     1.4262733684561437e-09, } , 
      {    4.3691052920542013e+01,     8.2130999804500443e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.1838669692118700e+00,    -9.7602740150536865e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.1876505699765612e+01,     1.0240836651764992e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0048767049699869e+01,    -9.7178292808395890e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    2.7531154635393992e+01,     1.4721801672728285e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1230905446901616e+01,     1.6086950658617649e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -8.6162154887511790e+01,     8.6696581981018553e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2266222520299081e+01,    -9.7550143504213193e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -5.3113278821314708e+01,     2.4031577378010785e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.6280571728545056e+01,     2.0598003939456294e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.1677412401686837e+01,     5.9969607731169126e+03,    -5.2529319941377664e-04,    -6.1838954652883535e-07,     2.9213417982617307e-09, } , 
      {    1.8216132115202897e+00,    -1.1327079277621027e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -3.4156890991820774e+01,    -4.5610897315394459e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2388941196215253e+01,     2.2003914638174929e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4359053303308887e+01,     3.8315328665128509e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -1.3807103246429136e+01,     1.5946842436620405e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.5257985763032394e+01,     2.3412347212591569e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -1.1180840441462756e+00,     1.3617989824821712e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.7781714302685362e+01,    -1.8088636807302362e+04,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    4.2497514239697205e+00,     1.3604871913622259e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.7325396028070790e+00,    -2.5097407962690468e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    4.4399516066687461e-01,    -9.7974590846354149e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    6.9919766131653731e+01,     2.3252741030694648e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.2122435995859639e+01,    -8.0316287192344471e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    7.0394872417957242e+00,     7.4559084364111159e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.3454481879189784e+00,     8.4345771751939428e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.0716154261992763e+00,     2.3416604968351563e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -3.1712429746818835e+01,     7.8725824515711811e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8569851294977976e+01,    -1.0250774401591676e+02,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.7806432208873119e+01,     2.9411298752569026e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    5.2826890588415694e+00,     1.3346175071761911e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7126182053477734e-01,     8.2550950953242500e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.9782241923825154e+01,     1.6371437812512359e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8140376975356130e+01,     7.1071458942738454e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.1540801787595445e+01,     3.4205582371201846e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    6.2738017704589790e+00,    -1.6087449331494725e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    9.6990115741614645e+01,    -1.1951304185721719e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1790342462096881e+01,     8.4636744540710770e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6430628974082822e+01,     2.2791762044652776e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.1426711166071630e+01,     3.3416910599499686e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.5387158818596539e+01,     3.3422170264132052e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {   -2.8626186746475074e+01,     1.5488269415507672e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.0054041827695269e+01,    -6.9696027215584127e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.5186740583426763e+01,     2.4664046609894787e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.0082753502062214e+01,     2.3947834514414186e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.8839389711174359e+00,     1.3584208397318749e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.7800024263656383e+01,     7.1090824173162637e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -7.5338081060347179e+01,    -7.1094938477706801e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    6.4766387971607644e+00,     1.3293205974550719e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.1907373715323605e+01,     3.3419869064292317e+04,     5.4522970760836301e-04,     9.9669451003541732e-07,    -4.7251395801978702e-09, } , 
      {    3.8721289599133570e+01,     3.3419211799339420e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.3643550737970170e+01,     2.3252352888901114e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.9421969372096164e+01,     4.0630973464958515e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.4402276564704616e+01,     3.3472509529027062e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    7.3961920325609150e+00,    -1.2232060617909797e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.4256669944926692e+00,     7.8077884049111290e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.8889634750881807e+00,     1.5001197229852571e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.2791884702972084e+01,     9.6104515239357497e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.1846768527553566e+00,     1.3447137559281826e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.6162243157957441e+00,     1.5569532136208081e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.4865548925202727e-01,     3.8875681139874478e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3474529919744121e+01,     4.0636233129590881e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    3.7910483412149247e+01,     2.2397610983051527e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.9523523072206739e+01,     1.3642074315745571e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -2.5400898351553458e+01,     9.6606773347372473e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5106021162497461e+01,     2.5649266814116312e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.7014756471133445e+00,     7.3571215027708877e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.9685400039865460e+01,    -9.5477660844303318e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5642293786923300e+01,     1.7723967560079349e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.9917313822667538e+01,     3.3248688166275984e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    4.5231900900535521e+01,     2.3421207368031031e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.6868316692114924e+01,     2.5582085754099360e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.4855264611134928e+01,    -1.2156005225723288e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.2865696518914760e-01,     3.2853589653799288e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.5951663446366885e+01,     1.4393240776022276e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.4275990692758171e+02,     8.4350880630571519e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.3950409643805250e+02,    -8.4318442522238220e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.8120745174078927e+01,     1.6713706182084272e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9643117653938123e+01,     2.1657945815678257e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -2.3365610395227790e+01,     8.5862694798250013e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.7562528503933656e+01,     1.6935624420853637e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    9.1016700971591824e+00,     6.5442589259646074e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3125084890570843e+01,     1.8240941456609798e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0425248894363284e+01,     2.3975996790768411e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.6766021534975682e+01,     2.3976444104880480e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.2339592191562673e+01,     1.4390561537050156e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -4.2562472468973294e+00,    -2.0180644698900087e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0076960101023447e+01,     2.3992219956148925e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    2.0131959053854576e+01,     8.8571400496431779e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.8035870401497199e+01,     8.3998331471947349e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.7022787609598641e+01,     2.4604175716911916e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.9288533893383551e+01,     2.3914087297851511e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3621831261383782e+01,     1.6066287142314141e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    8.9804660770731193e+00,     3.2633498492429891e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {   -7.5421789299983374e+01,     2.3951947574214966e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.9461359362511267e+01,     1.1667805414518887e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.0486945526563467e+00,    -8.9973393928733603e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.0670518093071724e+01,     3.9482926321605999e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -3.2730250934509186e+01,     8.0050619693714179e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.4260292921484016e+01,     1.1388747965657178e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6461830939184335e+01,     1.7690186132576382e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    7.4789108054777991e+00,     1.4577782437678208e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    3.0943067375082268e+01,    -9.5484233493832289e+03,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -1.3367944282009239e+00,    -1.7363771887121090e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.5674755098001725e+01,     9.5030737670319068e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.2463501907177879e+00,    -8.2319609697908963e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.0887036056565428e+01,     8.7440038341654308e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.9938730858748723e+00,     7.7706657996251024e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5465657359478667e+01,     2.2720316623200237e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4129969713340685e+01,     1.2048207145889497e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0186089956674298e+01,    -2.4120287383391851e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.3136181472286790e+00,     1.1921733301173854e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3583572088648608e+00,     6.2114916959657172e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.1374799539582870e+01,    -1.5322613319670108e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.0658240605330739e+01,     6.9102136230539109e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.5013253950758770e+01,     7.6643012123368844e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.4933601788086772e+01,    -6.2227411065059528e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.4286797339833726e+00,    -3.4492772969840672e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8928748545025908e+01,     4.0082847973241274e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    2.7830137907562765e+01,     2.4198389151977542e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    9.6843100911735061e+00,     1.6832439542311135e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -4.0660059637461281e+01,     1.7627623610320203e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.2825610019520219e+01,     1.5700488644524392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7621906935853584e+00,     6.0104073167928163e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.2494958227763338e+00,    -8.4512480768189671e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    4.2572497583083944e+01,     8.3773932903902005e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0050167130100093e+01,    -1.2325212033811094e+03,    -5.2529319941377664e-04,    -6.1838954652883535e-07,     2.9213417982617307e-09, } , 
      {    7.5507535794454332e+00,     1.6570082949718679e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.1951393408390036e+01,     3.1361135578823167e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    5.1608546168092275e+01,     2.4462893738003269e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.3456046051176152e+01,     1.5018892805577325e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.6180270981480575e+01,     6.0122915995528356e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.0345783290369521e+00,    -1.0280013573360662e+04,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.3905231478783426e+01,     2.5160678850996115e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.2762053970287754e+01,     2.5004098482376583e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    8.8830535763434835e+00,     8.3195026368092149e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2658630674124023e+00,     5.7534796362047223e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.6135124624016534e+01,     8.5021582632131485e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.9220262150302794e+01,    -5.0044978730934763e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -9.9585608774334560e+00,     8.0595900122708463e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.3271660807816232e+00,     2.2230983103672872e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.6974976048638695e+01,     2.3401593305456969e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.9447208084885741e+00,     1.1541551287841688e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.6046174880058700e+01,     8.2806628354559725e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6575159298892224e+01,     2.5659638015111486e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    7.4596055313320271e+00,    -1.0175690956485447e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.0033822160646693e+01,     1.6780191266352940e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    2.8957504939069985e+01,     2.8064164499094753e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5816778086181333e+00,    -1.6838598825824280e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.2342125636031692e+01,     3.3366571334604676e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.7022309003046836e+01,     2.4561868216914285e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.3002864771096800e+01,     1.5996397364294169e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.1069025297555712e+01,     3.0264555962912247e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.1044904013629555e+00,     2.4147072713364334e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.8937921606791733e+01,     1.4576686556710234e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5457607636944291e+01,    -5.7161007972925381e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.7092928608514178e+01,     9.1015886445839424e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -8.2452707012223776e+00,     3.2093061206879116e+04,     6.0708417629904202e-04,     1.0705413299420218e-06,    -5.0743023933329583e-09, } , 
      {    1.5644707657534042e+01,     1.4224024203133215e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -5.0442294036549136e+00,     1.9259968811508596e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.4125597288567533e+01,     1.6118307969031639e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    9.3818858436114709e+01,     3.2289369844261142e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    8.1516951746915094e+00,    -9.2459809980430032e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -1.1344545609290638e+01,     4.1955480122531975e+04,     2.6487436705295472e-04,     7.3638350023727418e-07,    -3.4983670415183006e-09, } , 
      {    9.0987624719234947e+00,     7.2497190999356953e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2102604691223767e+01,     1.5646976733566837e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.0848799467553256e+01,     2.3353946337173849e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.4794276643267825e+01,    -1.3149632660594448e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -9.6227647476281621e+00,     1.5648081312631803e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.3369482300871326e+01,    -8.1824470581403439e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0351013014533552e+02,     3.1190276599732548e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -5.3952905107421500e+01,     3.1190290026834002e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    6.0579394619088376e+00,     2.5718445246669406e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.9952833582712856e+01,     1.0961945568461222e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -4.7453859385363437e+01,     1.6616513749665046e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9915744909451405e+01,     1.9761210223891459e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -9.7229328475009709e+00,     2.0000155265346348e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.7359711813481273e+00,    -1.7659533210266673e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    4.9302699858660304e+01,    -1.0176049985994248e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.9882964744697603e+01,     1.0419594219256318e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5539792168681272e+01,     1.4879501816128428e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.6413313304997576e+01,     1.6761605292833130e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.5304858357918230e+01,     1.6762709871898096e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3743259706523636e+01,     1.7144807351063428e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.7185354782316317e+01,     3.5842100599786456e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -5.6150508204877614e-01,    -2.5999713891102083e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.4618259820722930e+00,     2.4403022845020401e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -4.2037687397587156e+01,     9.1967604766681961e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5160889318881328e+00,    -1.1120178595678732e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    8.9516880578662743e+00,    -8.2363682113348787e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.5998183501636971e+00,     1.4743454887106444e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.2322514130126123e-01,     3.1790784356135973e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.4783955914132463e+00,     1.5071479189798092e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.6463642530884943e+00,     2.3326582954801052e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.6657482923494262e+00,    -1.9027463117300609e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -6.8316592007872075e+01,     9.3039992575423457e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9027187788565875e+01,     2.2826254817622619e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.3124182143357558e+02,     8.4368022611093293e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.7146781178105428e+00,     1.7217311702031533e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1620952630110139e+01,     7.6309461776997559e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.6289927676984988e+01,     7.6689036120163546e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.6755301633478190e+01,     1.8364894383447267e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.1314291049245995e+01,     7.4831642779131116e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7759635821787554e+00,    -9.6179245608121546e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -8.7423490690205541e-01,    -1.6159587146841903e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    1.7293217131243537e+01,     1.4724975096479960e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.4660897760412634e+01,     3.1676604287504029e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    9.6546158145887180e+00,     1.5994130375394723e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2268423400761693e+01,     3.2897171041192712e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.6654470392198672e+01,     2.0360981538773340e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.3269249435895585e+00,     7.6911950517636715e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9296220432253875e+01,     8.9438810985712153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6480865872377692e+01,     2.8685407590886516e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.2610960854704825e+00,     1.5288289480909298e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    3.4148649668051519e+01,     2.1542843568494132e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -5.2055282397908496e+00,    -8.5070624870413631e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.7105549063430018e+01,     2.3960220939499959e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.3567598778349293e+01,     8.1229284811155430e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.5721569061058291e+01,    -5.8995723011548052e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.7108339955533818e+01,     2.4004606381234706e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.0426524957121806e+01,     8.4307950948345951e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.3682335446650988e+01,    -8.4361372204463787e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0445295660418417e+01,     8.1240577316822737e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.9958979545289438e+01,     4.1960082522211436e+04,    -3.9751134229741748e-05,     2.3556127137750133e-07,    -1.1256888861682315e-09, } , 
      {    2.2107524528952030e+01,     2.4607157865186175e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.0683479518636041e+01,     1.6332924506809632e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.0440329003251719e+01,     9.9660882700908551e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5584968191794399e+01,     3.9454540388195732e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    2.5467199173422621e+01,     2.9830056871783854e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.2684342195965399e+01,     1.1117050090485471e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3811477694256812e+01,     2.5934124335104807e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6458392273259534e+01,     2.3993493444591302e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    5.0778362852336532e+00,     4.3581413446925590e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.7852645187623764e-01,    -1.6945629086227880e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -1.1685754568841894e+02,     1.0442913050801246e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2482702499421369e+02,     1.0512033532270914e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.0585865240771959e+00,     3.1110653509487449e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.1365754558305742e+01,    -1.7940304345491297e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.7221588687896201e-01,     6.1191111729917220e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7611591939040103e+01,     7.3909754137213695e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.0265418152297836e+00,    -3.5207572435190614e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6451049472655807e+01,     1.5038409107762001e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.2966846328731826e+01,     1.6726074955022916e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.9801111945670492e+01,     1.8380642126962906e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.7000043847703772e+00,     5.1625868310320166e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5488709817480036e+01,     1.6275830978347802e+04,    -2.8518604831466774e-04,    -1.9079774546150435e-07,     8.4338672779495976e-10, } , 
      {    1.2232960414474569e+01,    -5.9110133693317198e+02,    -1.5714345840060770e-04,    -1.8089785009324763e-07,     8.0295326679042455e-10, } , 
      {    1.6554049412397698e+00,    -2.5054800590688330e+04,    -6.1141782883174396e-04,    -9.3274758549706954e-07,     4.4164102280673174e-09, } , 
      {    7.4451998544729552e+00,    -1.4088308343236901e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.1939991388213066e+01,     2.2878864885325009e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -6.1919460597340688e+01,     3.2320453900838002e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.0715750517956032e+01,     1.5743088996977109e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.9528727349058848e+00,    -1.0039331463915401e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.3223736033938550e+01,     3.2586599153727471e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -5.7648349093179675e+01,     1.6625439879905021e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -9.2242140753843032e+00,    -4.2522373612104666e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.4065292016416485e+01,     2.4587607352061204e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.3044758831521377e+00,    -9.3216245702672786e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.6765768100195292e+01,     2.5084948953004321e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.3251116558364537e+00,    -8.3106672165239215e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.5042438824082062e+01,     8.2668716478605947e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.8298249313611244e+01,    -8.6000606674203773e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0977075974867043e+01,     4.8609032784172559e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.2107529226614124e+00,     8.1348392440815651e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.1949570472972617e+00,     1.5604869207143616e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0261547165173049e+02,     2.4000493321433925e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.9099514981380612e+01,    -3.2432886412032531e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.7884664651307407e-01,    -3.2620604016587158e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.2754057797814113e+01,     3.8104625213576881e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    3.2182387827828451e+00,    -3.5737263407302530e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.2203254183982445e+01,     1.5263177921388518e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    6.8071524066444837e+00,     1.6151883681307398e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.6788979330787683e+01,    -1.2002156224734290e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    8.2709361271734192e+00,     1.5174325960078026e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.3097584989465901e+01,     8.4269405716395195e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.4321169422053899e+01,    -1.9428827093114514e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6023532957006900e+01,     8.3155735135256709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6735290497785300e-01,     1.6713419821724536e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.3058213782757846e+01,     3.2097663606558581e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -3.4057799740717186e+01,    -1.7877138457589979e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.2785101051322760e+01,     7.1273123407446756e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.3755607463357251e+01,     2.5731775830458660e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    9.9226162704956611e+00,    -7.9940445221221562e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.7202892533421945e+01,     7.8172255213848393e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.3018893764810997e+00,     4.0002994512389421e+04,     3.2672883574363378e-04,     8.1023032014387868e-07,    -3.8475298546533887e-09, } , 
      {    4.6887182596087513e+01,    -1.7876433825673834e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    6.1009067963819259e+00,     7.2482964090019423e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7577821335441520e+01,    -1.0346136865885133e+04,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    7.5227025756390500e+00,    -8.7015315599782616e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.7504777899955144e+01,     8.0891470536125780e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.6634217008539194e+00,     8.4363110716982774e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.9289175164167185e+00,    -3.4197947559991480e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.6991147998292103e+01,     6.9622344497714093e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.4677862163722568e+00,    -1.7127946604369241e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -1.0734180512672058e+01,     1.7687884932736648e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3060696975847929e+01,     8.4306212435826965e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.0260418486372096e+00,    -1.9432708511049880e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.6062934872069583e+01,     8.4157458467615634e+03,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -3.9318745361598765e+01,    -8.4511864685194105e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.8435057175003404e+00,     2.2824832126688867e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1891739788384967e+01,     7.7648181894922518e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0264249117251993e+01,     6.3399603391208148e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8270591148673230e+00,     6.0077774844766336e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2571370232728038e+01,     1.6482718338514918e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.4146043912947317e+01,     4.1812666206353664e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.2341813520516720e+01,     3.3261806077475434e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    4.6808882233983606e+00,    -2.8690394319657280e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.3316133368309778e+01,    -1.6713455716700533e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {   -4.2908486989299057e+01,     7.5723026356024184e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.3107227486668968e+01,    -1.6486363677376357e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.1305941582689485e+00,    -1.0017495010318749e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    9.7376706635327878e+01,     6.5052534211074108e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.7806594954873955e+01,     1.6314685107595096e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6964878314609521e+01,     1.4322639652535428e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.1826986234279264e+01,     2.2690835693901536e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6575376890780152e+01,     1.6134178629796519e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.3389438344664093e+01,     1.5083211161987332e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.9368751303420826e+00,     5.6036972274283002e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.2315097431213445e+01,     5.0689322229061454e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8130399774695714e+01,     1.6694594727359683e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.8706495047999894e+01,     1.4963864507867434e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.5368645074394344e+01,    -2.1380722159071411e+02,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.8718571893637957e+01,     2.3546987372268464e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.4943249595262699e+01,     8.4007893218668396e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.7960210535514571e+00,     1.2036555486660156e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7318611604776113e+01,     7.4685276747334929e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.8181552053434672e+01,     2.1444627791178500e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -4.2639539532635800e+00,    -1.5977269628700540e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    2.5063188829979929e+01,     4.7061624223347550e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {   -1.6753781087626422e+01,     1.7237239034214454e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.5585254234315798e+01,     1.5238775569258740e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1232999483807593e+01,     7.1004279771040083e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.1912648934088192e+01,     7.1181482879829337e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5840666336875968e+01,     1.4304163328294077e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.2886819631173614e+00,    -2.9684936556229559e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8016642507022212e+01,     2.7448974825248664e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.4106251761274056e-01,     1.6133138651853194e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.3636546897116304e+01,    -1.2171020923419201e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -3.5049895353094691e+01,     1.7219861189159838e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.1700838443449744e+00,    -1.5373545207455929e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -1.6052602294481197e+01,     1.5318534851574863e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.8221418864613650e+00,     3.1636789385606116e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    6.1863071681514405e+00,     5.2883551013403976e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2930357562787918e+01,     3.2664151414739594e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    9.4590555184139706e+00,     1.4336468909201760e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.5289680827529875e+01,     1.5008912156798289e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.2804667265918130e+00,     1.4317317523607355e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    6.1576326269232098e+00,     8.0170349614397091e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.9360040251700447e+01,    -8.3647740520678235e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3011389598329497e+01,     2.3972417678215119e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -4.4363851229354431e+01,     8.6806905194307046e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -9.2282250826683434e+00,     7.0826274259588390e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -4.4328396078534227e-01,    -8.3298548862321477e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.5582829736577096e+01,     3.1989443163868709e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3057045282417986e+01,     6.4920011742202923e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.6067013051165993e+01,     1.4324062343469181e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.3945383860157436e+00,    -8.4400775571937793e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.6674596183261666e+01,     1.4497563392897491e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.5395766103677015e+00,     1.3224968855128725e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9673796152216624e+01,     9.5813774715326690e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.5636371422450587e+01,     8.8626992331964684e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0462836067058923e+01,     2.3980023217433773e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.9156103188375582e+00,    -9.4367610283916911e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.1267500893352750e+01,     1.6804817398321316e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.8139718641514591e+01,     9.7575052067078122e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.0722348789003007e+01,     2.4581986127738932e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.0402186453563459e+01,     2.3880660473946657e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.8855097655440034e+01,     8.4327502205765431e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.2110908144969216e+01,    -8.4341820947044307e+03,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    5.1962057877335317e+00,     2.3223959935941435e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    8.5141126232105062e+00,     1.5450431076793711e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5425104947377344e+00,     1.4561766118548119e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.9006802064343280e+01,     8.4361976084767230e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.2262612553872462e+01,    -8.4307347068042509e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.9274058105276648e+00,     5.8128580605544730e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.4283367329276366e+01,    -2.5075116522943150e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.6471117839630029e+01,     8.8456653212555811e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.1434191186625043e+01,     3.3442082336946914e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    7.6435831732197137e+00,    -8.4054690387686260e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -1.6346085326250563e+01,    -7.5077388072694557e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.7994059953941299e+00,     1.7316868838010279e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1214748070537139e+02,     1.6761811979958264e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2953917830017377e+02,     1.6762503184772962e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.7191408936296831e+01,    -7.3995183645795769e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    8.3035481515927323e-01,    -1.1274110180409836e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.2977995850348853e+01,     8.5224836712540891e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.5790267958322872e+01,     3.2159796756531516e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.5574921752814213e+01,     3.2134059607009684e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    5.3148432904579552e+00,    -8.4672532149405124e+03,     9.2284647809460994e-05,     4.2489749751593286e-08,    -2.4923786532160132e-10, } , 
      {    1.8724514188856794e+01,     7.1737224823561410e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    2.1836226869873116e+01,     1.0611702649641466e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4053757357790570e+01,     2.1506760941151428e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.7920563361576203e+01,     1.6065934826356066e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.5955417806848459e+01,     1.3162337032278714e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.8953965698459761e+01,     1.5114822642421273e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -7.0164276153778982e+00,    -9.7636776571324499e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.9197695591450668e+00,     8.1063457897302806e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8845209218166438e+01,     1.1735368269524821e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3903364714919867e+01,     3.1834338641844461e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.9787980512283159e+01,     1.6278466440461048e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.4529929676390161e+01,     7.1094365496006858e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7110175217490848e+01,     2.3753553070794267e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.4754987185356455e+01,     1.5631529514774835e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4112019500196413e+00,    -5.4132558718213318e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.9457562419030673e+01,     3.3402267435049012e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    6.1530240759054635e+00,     7.1091397154862580e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4195552634759455e+01,     2.1478409218681463e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.5886208622586651e+01,     1.7396498641806105e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.4922997050921829e+00,    -1.6757748355196545e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    3.0230651018969574e+01,     1.3589139429474664e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2261043530379567e+01,     1.9244236327361003e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0039305301780837e+01,     4.6400665825343822e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0974149008939749e+01,     3.1596422194175721e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    6.1226857040572344e+00,    -5.9180705260298737e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.8663856429931805e+01,     1.3193488627465496e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4887277759603830e+00,     1.4496140701963739e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.1027600487029970e+01,    -8.4409391945168936e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -6.4667563661643719e+00,     1.8866947841815607e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.2690041099265112e+00,     1.4997891530075925e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.9857351336736507e+01,     1.3740091413288264e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3114875562303476e+01,     1.4391625198475109e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.9389453971222025e+01,     2.5219717706716074e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -1.3990115209769076e+01,     8.2095261745567223e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.4722911658871904e+00,    -1.2172610539120294e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8359754806878364e+01,     1.9278017754863969e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.1608554379956971e+01,     9.6380605010140735e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3163281654979793e+01,    -6.7631665015529034e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.5556160140054862e+01,     1.2441228115582486e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2910938583381609e+01,     9.1656821715905608e+01,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    2.1973812490426933e+01,     3.1659683606695242e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    8.9406469030305402e+00,     1.6187530439998138e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9345822517114655e+01,     2.4289040372616884e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -3.4236791942677165e+00,     3.3705942174569932e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8463145423464077e+01,     1.5025254912448723e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.6294763287646488e+00,    -1.8657350854284025e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7811940435114582e+00,     1.2974001790728917e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -4.0664335250382866e+00,     8.4320434667067329e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0463836668157260e+01,     8.4348888485742409e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.7993561855282096e+01,     1.6276188924102940e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.9911243242042225e+01,     7.1359488391281047e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1171847174540812e+01,     3.1679234119820212e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -7.5025672063650603e+01,     8.9699380140706180e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.2014324198284489e+00,     1.6760994120858591e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.0892514883049422e+00,     2.3967583949441017e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -3.1888220392997045e+01,    -1.3786628997733033e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.2412755859750906e+01,     1.6762512185963722e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -2.0379675493783882e+01,    -1.6761802978767504e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.4510081522090839e+01,     9.0897708615578922e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8044031894376030e+01,     2.1447242364126629e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    3.3082397397395887e+01,     1.5003488953906184e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.6475319188901523e+00,     8.3673341954373063e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8566155747360966e+01,     1.8524221462271078e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.5595355011789408e+01,    -2.4763707782308156e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -6.1875060766789436e-01,     6.7838300178564132e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1409230334530889e+01,     2.2045064745659740e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.3090437176446304e+01,     9.3591935080115200e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.1048237964941180e+01,     2.3292555932592557e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.1783233028994808e+01,     1.5362728496096210e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.3121541882247428e+01,    -1.0178703501792057e+04,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7912436961736290e+01,     2.3208193240853554e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.1316187933786022e+01,    -9.5999098283970707e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    2.8909842661576825e+01,     1.3106550729809131e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.3180826049799716e+01,     8.2325381729540709e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.1404784125806602e+00,     1.6719497766409910e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.5240267008910024e+01,     1.6919686221478834e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0367963061132180e+02,    -9.5479966653329411e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -9.2422187729187044e+01,    -9.5481927684806196e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3367945532381324e+01,     1.5584250644632641e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.0905429472728905e+01,    -2.4149243537595339e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.9816841691705136e-01,     2.7860574659206086e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6936699567546754e+01,     1.9585458928652097e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.4991323638505182e+01,     1.4880312961677375e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.7508038857419070e+01,     3.3425573906210819e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {   -1.1145342206311224e+01,     1.6069945568416460e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.1734532028018975e+01,     8.6358767705988903e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.6067898487987037e+01,     1.4881017593593524e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.0119115478088860e+01,     1.4843453399726019e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1689407653940478e+01,     9.1055445761951887e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0700136209374744e+01,     6.4748577896764982e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    1.9031997543835040e+01,     2.5313017711243840e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.9596537155328363e+01,     1.4550173460599883e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.0808133701891720e+01,     2.1113492150497223e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4405651209121579e+01,     8.4057182045324698e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.7916352109350528e+00,     1.4323679630478753e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    4.6837228110111425e+01,     1.4323022365525856e+04,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    2.5105961427958228e+00,     2.3754051743671345e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.3045638588953281e+01,     9.2947591581988509e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1280923474733937e+01,     1.6921064873999188e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -4.6917946093432157e+00,     1.4321273455218605e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.0534508210144564e+01,     2.7422866324114502e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.8337566892388715e+00,     2.3400170614523220e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4452535052595508e+01,     3.3577274786156304e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.0340248291435273e+01,     2.2939144149570271e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    6.4065672462086090e+00,     6.5574217909565959e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4558390228438967e+01,     1.2360650188166364e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0869118948668518e+01,     1.3853950704590336e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -2.1126125716789712e+00,     1.5032339349253230e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.8320306794192561e+01,     1.7707253414116029e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.3328439186763831e+01,     1.6971354732238262e+04,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {   -1.1675976888167252e+01,    -4.3566255263541461e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.7809464308393842e+01,     1.6972059364154407e+04,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {    4.1454334031566106e+01,     1.4325428540786004e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.3336219894547266e+01,     8.1670279471011509e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0240735856868556e+01,     1.0212450718354734e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2798677863058533e+01,     1.7749704709601177e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5548794911248919e+01,     1.6759312668307823e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.4229826198660147e+00,     8.4306125739039853e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1671721303368345e+00,    -8.4363197413769885e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -4.4405139593582836e+00,     1.6765002496423403e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.2502738140394847e+00,     6.7691934146767962e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5611090309016959e+01,     7.7372670851125658e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.0501231340835353e+01,     1.5330740776394448e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.4682338729580113e+00,     8.4088735180534331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.6914400538946552e+01,     1.0634210757034311e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8900825508400665e+01,     3.1090956518414190e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -1.5494051374040758e+01,     2.4051471508996834e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    9.3460644627489700e+00,     8.8794467487585462e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.7947068076528385e-01,     3.4945471442944608e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.6414678054035239e+00,    -8.4679589306103280e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -4.9654918034975971e+01,     7.4987063062015404e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4186655641078772e+01,     1.7501300183049167e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.4648893589516419e+01,     6.9695685106181236e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    3.6419204365222704e+01,     7.7396732603727160e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7639345367729035e+01,     2.3337603581523414e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -7.5428942798986611e+01,     9.0773157709744610e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.5819129011559077e+00,    -1.5682716333765034e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8998956806458864e+01,     2.3367100532487129e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.8226110332500031e+01,     3.9676709092348836e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    4.5601085960340937e+00,    -9.7768318524457281e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.7398651487942903e+01,     1.4094623338249476e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.5233995555171594e+01,     2.9864334032034676e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2915314223617909e+00,     3.2298878398154626e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    7.5053758031319848e+00,     8.4156842384620068e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.1889725345887260e+01,     1.5647539545303131e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0590401610180724e+01,     1.5647518500895509e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.0013900394700844e+01,     1.0532702244784423e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.1700244627373287e+01,     1.0422244338287737e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.6026664208083883e+00,    -2.4447540326846774e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0272181040574992e+01,     8.8287883271732098e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.6963960078426208e+00,     1.1294177590122376e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -5.6752022282883008e+00,    -2.4110854395829956e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4545144246100181e+01,     3.3941909822439025e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {   -1.1970458752755189e+01,     3.5037938675056012e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1325094823324655e+01,     9.0448530618772493e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.0907898121069278e+01,     6.3750423530755461e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1815892820351852e+01,     1.6834314349285032e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.1055075373991997e+01,     1.6164502919629775e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    6.9446636788106773e+00,     2.1540542368654395e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.0991061908010597e+01,     2.5074639671748831e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.2249892748115663e+00,     7.9264816671162280e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0215918205086760e+00,    -1.1295946634006488e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.1376305949531318e+00,     1.6060253667919194e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.2022518450291972e+00,     6.4934320670230409e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.3271542949227495e+00,     5.8553027947685696e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.6379190169236111e-02,    -8.3900156081104569e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -6.4393801251121232e+00,     3.9187197091742291e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9174145614155069e-01,     9.0080933000079622e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1690179213070905e+01,     1.6697224559675869e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.4059043436138010e+00,    -2.1293981584050971e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.3511585181718445e+01,    -1.6793637810028842e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    9.0879347418052934e+00,     2.8168728117789897e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2113730708524187e+01,     1.6674243596144435e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -4.3782859011310713e+01,     1.5635323098279734e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    6.4186616037271058e+00,     2.5010666913762434e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    8.9028045174942221e+01,     7.7375890040382883e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4891573564109247e+01,     1.5202692941916042e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.0379703054755790e+01,     2.8457185468787742e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.8532636151185571e+00,     1.2392027577040331e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -7.4405910327386415e-01,     7.7249764792666356e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.2226257442003465e+00,    -1.7954531254828823e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.3425223551278096e+00,     1.4057411460340045e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.6181794453453930e+00,    -1.1086397168175767e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    8.8588655415298661e+00,    -1.9268871913653845e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.1309574485731595e+00,    -9.7771151436519773e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.3116707668109200e+00,     1.8080905902429637e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.5411642223107087e+01,     2.4597969077270256e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.9459087915151859e+01,     8.4329674847634142e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -1.2567714722222163e+01,     1.2139735871486610e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.8892166955222898e+00,     1.8473926872122618e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4037393397403221e+01,    -2.1419572188010129e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.5408465025995191e+01,     2.2465173838057453e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.6945433114157346e+01,    -2.5106268118129927e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -7.7732255733012181e+01,     7.4862434624933394e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.7961858063435898e+01,     1.7197424132316544e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.1919654722239081e+01,     1.6038678165235817e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.9786008147713073e+01,     2.4533182411499518e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.0476837200321876e+01,     9.3500199794636574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9157114246811634e+01,     2.6629151458359724e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    8.2726020437954233e+00,    -1.2892331360870287e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    4.9262278835879005e+00,    -1.8433519846866511e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.6277813410194664e+01,     2.3816960840232798e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.0227452410562538e+01,     6.8886381203705332e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1291302905494732e+01,     2.4533887043415663e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.3384467424359528e+01,     4.9549450276861535e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.3289980785582834e+01,     1.7191390657921595e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8752746990979723e+01,     1.4514866779845699e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.4325421528650732e+01,     1.4388612653168499e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.6121343093230376e+00,     6.0441887442957814e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8149883923485390e+01,    -8.0247127037999107e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.0180490306637715e+01,     1.3695754758423638e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -4.8002179243685397e+01,     5.3400780847134274e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0527102812814190e+01,     1.6782821098669123e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {   -5.6914020020611815e+00,     1.5273652877729679e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.8813992097699575e+01,     3.0935851873607375e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.5744843994432568e+01,     2.3411902586712458e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.8665552324610911e+01,     2.4310281691296172e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.5013495139434045e+00,     2.0439031720587558e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.8878116833114973e+00,     4.8742450161317419e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8846514180863323e+01,    -1.2199555817141377e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.1224595270637725e+01,     8.4437841961231316e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -4.5566334256434438e+00,     1.1717319326169445e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.8780523148599343e+01,     3.1461987749726039e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3210852907019790e+01,     1.6913332981106319e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4181558035242332e+01,     2.0188012621087415e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.2878299451552889e+01,    -1.2188510026491708e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.8461153994983917e+01,     1.6054220193524245e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.5039742875847431e+01,     7.0977981447878246e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.5719392326128023e+01,     7.1207781202991155e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.0022149889779826e+01,     4.7848018478691607e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {   -1.1042715505139373e+01,     4.7847313846775447e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {   -2.0642709733992522e+01,    -5.3975188427864650e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.1366280777270692e+00,     6.5862073964534347e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0750831423284275e+01,     3.2831047748668236e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {   -1.3448367801256346e+01,     1.5754767803973469e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    8.2101222112701819e+00,     6.9503808409098974e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2100488882115794e+01,     2.1763884010100639e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.6770793134908724e+00,    -7.1048789053744013e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -2.4339240843329545e+01,     2.2232931987554533e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.0624926333753807e+01,     2.3503017384803152e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.6987341182009441e+01,     3.9911995707701390e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    1.0649646017175044e+01,     8.4268547580871946e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.9337793350505610e+00,    -9.3838014069666224e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5977355099801830e+01,     1.5549757871662798e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.7353511719530360e+01,     2.4779329668750070e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3529372686300915e+01,     2.0890577753024054e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.5643167513017261e+01,     1.0674025658932222e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.8677470798751642e+01,     1.4391266168966309e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.2157437634760150e+01,     5.9660841937511741e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.5851029698469778e+00,     1.6038290023442280e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.2132374312743792e+00,     1.6709899830621296e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.4309867340599801e+01,     1.5690188839055023e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.3961501979026121e+00,     2.5171031100419044e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.3373287654593533e+01,     1.7391176512878032e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.0535539144418319e+01,    -1.6304702262859955e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.0115457851138395e+02,     2.2861585175007422e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -5.6308452777481975e+01,     2.2861598602108876e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -4.7063372685202438e+01,    -8.4058571805621614e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    6.9208589761765591e+00,     2.0832052690280543e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5551085026262442e+01,     8.7830458443022108e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8408089179005575e+01,     4.8439825976084740e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.7983450910405480e+01,     3.4030927336052628e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    6.1255519679561647e+00,     7.3379366342778030e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -4.7380551368560702e+01,     1.7293562366848477e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.7845312440650876e+00,     1.5630668276405137e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.5304918274366521e+00,     1.5758862711614409e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    2.2090830853281709e+01,     6.4886071311241994e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.4994804585756665e+00,     1.3240236877339528e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.0159463971612919e+00,    -1.6926063552029491e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.5608252639377631e+00,     1.5089111216073748e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.4039575432637026e+00,     1.6631645699928420e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.6980676291185315e+00,    -1.5694428361801736e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    5.3996747273640899e+00,     7.2408407114829552e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.8951623674480089e+00,     1.6814415334109930e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9209239127159751e+01,     3.1260173091303252e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.7687533744890438e+01,     2.6300564838479404e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.5362662903853904e+01,     1.8562969120890393e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.5101139558840920e+01,    -1.5456028500623972e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -5.3673180412216404e+00,     1.1148530318148700e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0133674174826965e+01,    -1.7837211532936419e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.3874798525350535e+01,     8.0381439881077640e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.8482956770754811e+01,     1.5630256026332459e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {   -1.4047977059895132e+01,    -1.6774155192912989e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.5391249543229822e+01,     1.5710807401565999e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.0159881479618136e+00,    -9.5136019439369393e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.9578737354210562e+00,    -1.6167637054620092e+04,    -6.0028102831887247e-05,    -2.0792136467829315e-07,     9.3710121235343308e-10, } , 
      {    2.0464576356682340e+01,     2.3732889554490757e+04,     3.3338590936168508e-04,     6.0664251103555123e-07,    -2.8762056445504330e-09, } , 
      {    7.7552082380147747e+00,    -7.6685811566680741e+02,     2.4992516143662629e-04,     2.9166807995315435e-07,    -1.3266974864930561e-09, } , 
      {    1.7135328605044997e+00,    -1.0352164710483019e+04,     9.7612410795278029e-05,     4.1256965523267953e-08,    -1.4035840881802174e-10, } , 
      {    3.4903322791409778e+01,     4.7813884735230567e+04,    -1.3157552417104136e-05,     4.1027842577575586e-07,    -1.9581624580014158e-09, } , 
      {    2.7879784756401957e+01,     2.2199508505806705e+04,    -4.6743192284824035e-04,    -3.4157130962806608e-07,     1.6552945032754774e-09, } , 
      {    2.1852154171761192e+01,     2.3385124740688334e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1321670812962118e+01,     3.2899432400295089e+04,     5.0366687313368307e-05,     3.4694789335157077e-07,    -1.7038728341226531e-09, } , 
      {    1.9881531367291743e+01,     1.4357143685099389e+04,    -2.4626716949060270e-04,    -1.5572351185069006e-07,     8.3212450598278531e-10, } , 
      {    1.5084170859292188e+01,    -1.0436352538538988e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    1.0910722920244570e+01,     6.5857552804132893e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6569148085955376e+00,    -2.4422257751756542e+03,    -1.8007904826722169e-04,    -2.1967043638903787e-07,     1.1408538581133378e-09, } , 
      {    3.8580734530964254e+00,    -7.6293961640642474e+03,    -3.4038344338729550e-04,    -4.6823237447643627e-07,     2.1638737510330028e-09, } , 
      {    7.0859174701819754e+00,    -8.8860000745612124e+03,    -3.3505568040147844e-04,    -4.6946515870476163e-07,     2.2727532075365822e-09, } , 
      {    2.1176114858622373e+01,     4.1714450429038028e+04,     5.7615694195370252e-04,     1.0336179199887196e-06,    -4.8997209867654142e-09, } , 
      {    1.7553936809765201e+01,     2.4151988486152200e+04,    -5.3195027303182794e-04,    -4.1480173742050790e-07,     1.9500175881587754e-09, } , 
      {    7.9815807749955017e+00,     8.3996734705434010e+03,    -3.8421823597507498e-05,     3.3206246497499222e-08,    -2.6324413256885588e-10, } , 
      {    3.9132262761811788e+01,     8.4549455514023302e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1899487357183030e+01,     2.1970765740470695e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.0642140285033847e+01,    -4.6452583309034526e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2354670163023805e+01,     1.4344680905980156e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -4.3897020752450601e+01,     7.6069129503922977e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.7907796307120165e+00,     2.3014410470110714e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.8358618194081412e+01,     1.5978416842819372e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2821530979639283e+01,     8.2626138921006004e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.2818093501374330e+01,     7.2658685011630023e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -3.3696116345941931e+00,     2.3602344302454803e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.9169104622512121e+00,     6.3908090481634254e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4391591857901439e+01,     1.4029104656428490e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.7047266663804876e+00,     1.6197839721253626e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3089381898306440e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1162258253444374e+01,     2.0128494044062616e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -8.2099728068330819e+01,     3.2304813820975731e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -5.3578455847155926e-01,     3.1190611945759727e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    5.0092998436706289e+01,     3.1189954680806823e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.1400211041419311e+02,     3.2305009924123413e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -2.4610799014559419e+01,    -8.2259942345934651e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.3419744528850682e+01,    -2.3693689223834817e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5830028850196792e+01,     9.0978221590884405e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.5126771387159039e+01,     2.5436970638081308e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.8361532625352925e+00,    -2.5224191280980840e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1168152696500364e-01,    -9.5944953827321260e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.0605550306479294e+00,     1.4872028779252023e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    9.3868889654198036e+00,     1.6769161163844456e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.7253260246464599e+00,     1.6698879203898934e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8186960180702545e+01,     1.5368089779248627e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8580369709422783e+01,    -6.4039269593628806e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.0759391875444066e-01,     1.6186107749064386e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.5545771050586509e+01,     3.1269913117079101e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    7.5454545701231154e+01,    -1.9030678074135394e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1223894985852111e+01,     1.6625086744328692e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    2.1586234111880088e+01,    -8.2218391490260656e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.9874674578885426e+01,     1.7430280069309068e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8941376076671037e+00,     1.5112521442581539e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.1470976412091315e+02,    -1.6763888550844040e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    2.6249103177886322e+00,     1.4460803631028204e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.3654994377871681e+00,     7.7003838396795827e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2899256238157482e+01,    -6.5796949976906380e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -3.3952867158067768e+01,     1.7961684853791932e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.8600684019923939e+01,     2.5599711886442547e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.7770140528062079e+01,     1.6133521364843622e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1639761065942780e+01,    -9.8423410584805952e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.6719735118231952e+01,     1.4108734460224314e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.3285324358677777e+01,     7.5667434520491288e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.8936602776494752e+01,     1.5675526141971181e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8532075410006140e+01,     1.5273686247328438e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -7.0322741875902839e-01,    -5.2424419759705870e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -7.7170672461213002e+01,     9.6320270266191274e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4571895208188677e+00,    -1.1238433183038637e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3875523266018998e+01,     1.5637219741843832e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.5758505551324511e+00,     2.6368202454893492e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2035735343457716e+01,    -1.8580201584826852e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.7941674552177105e+00,    -9.2532895245294258e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -7.8117184663124171e+00,    -5.1496129891115288e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.9677587731273313e+01,     1.7525451901393324e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.2115014656524977e+01,     8.4318382341966790e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.8859204166995809e+01,    -8.4350940810842985e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    6.7768110994867357e+01,     7.3083626420330329e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.9338557756612458e+00,    -1.2620631081373579e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0706615225718707e+01,     9.2839424468391298e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.6334702736672156e+01,     7.1791179764488452e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.6418567385402888e+01,     1.5433048525112677e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    9.0656445871813514e+00,    -5.5731427963314673e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8607845099030254e+01,     3.6235121569479445e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.9207008184854672e+01,     1.7079927490886213e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4272952832116601e+01,     1.2751267119230353e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.2353088622318808e+01,     2.2587608072914762e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    9.3759048919830157e+00,     1.8411793722149687e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.0799515888594371e+00,     2.0046010750894573e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.0901112682319376e+01,     1.2060826384211876e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.3179660943650262e+00,     1.5670070928230372e+04,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {    1.6880544436386138e+01,    -5.3230798275995512e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    8.6576123548826587e+00,     1.4494203263542191e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.6932402681620186e+01,     1.8346860699459939e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.1106376792653254e+01,     1.5644684109041529e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8889639259202440e-01,     1.5429343058885170e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.4106204032304660e+01,     1.1510071060178459e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0261311219664108e+01,     2.3689118720981602e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.8809252149152858e+01,     3.1719264103459733e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -2.0200105895782094e+00,     2.3930364962276226e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.5745174519802791e+00,    -5.3157713011131273e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    4.7797276725886931e+01,     1.5557424262962335e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.6071344552844060e+01,    -6.5851958611234886e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.5970980394718779e+01,    -3.2445834264062114e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.6651319058346310e+00,     1.6825435960832292e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.2329926341838746e+00,    -2.3707916133172339e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4416102037409095e+01,     3.2224729779221263e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.6979507166409398e+01,     1.2834249085275474e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.2158879766298547e+00,     1.6320718581990046e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.1938827153978401e+01,     8.7816231533684604e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0933014986684161e+02,     1.5019319489627616e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3726894932387550e+00,     1.5650373937157110e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5178079716975576e+01,     8.3401661531127247e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.5893428849421802e+00,     1.6288954519344319e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.0195655265187381e+02,     7.1075571640650451e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.3752375892462990e+01,     1.2597335534499249e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8771397839996926e+01,     1.6459308112007129e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.9615162231084398e+00,     1.2831947885435738e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.1278028801129203e+00,     3.2519528410327563e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.5335060730154574e+01,     1.6759854647528489e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -5.3192506245676853e+01,     9.1569455747702832e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1001462812919391e+01,     8.2621073464818001e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2011498793766560e+01,     8.3150764436929912e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9494968592515164e+01,     1.3542765522506352e+02,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    1.5570216121897440e+01,    -7.8182764837943987e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    7.7535510917579309e+00,     7.1611032440337613e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -1.9571405065710421e+00,    -2.9929923588031952e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.8592414740719114e+00,    -2.4753071168058646e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.6012740028497507e+00,     2.4355163665141354e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -2.4607970361931294e+01,     1.5714952902075556e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.7624164626119949e+01,    -1.8500120770669309e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.6702499899966888e+01,     2.3349990405562603e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.5886742444920330e+00,     1.5657838304354807e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.4582289243360325e+01,     5.7985618476420314e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1766037943711067e+00,    -7.7545163983977964e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.7120859349284903e+00,     8.1302586667884214e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.3694016586508688e+01,     1.0300149513562583e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.8619903552477872e+00,     8.2517980652263122e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6656501881369493e+01,     3.3794543728964559e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7258560268056229e+00,     1.6780633906606967e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.8317986093118094e+01,     8.0042330820845054e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.2003216915080856e+01,     7.1453707598861693e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.0544377296512838e+01,     1.5612781193148678e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    4.7098337215378301e+00,    -1.6797102471375601e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    6.0982029270951781e+00,     1.6603250290732038e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.1334000760047850e+01,    -1.1559144427508181e+02,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    5.3779761538758306e+00,     6.1469193040262007e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0241090744195823e+01,     1.6879551553603353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5488313366123787e+00,     2.1147521398961409e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.1809136953453239e+00,     9.1573337165638204e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.7297279381201669e+00,     2.4463252767512073e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -6.0810392487964906e+00,     1.6750159971818237e+04,    -1.9206388487109000e-04,    -1.4849843052385087e-08,     6.0650191506802891e-11, } , 
      {    7.7024828091212489e+00,    -1.0308399506770922e+04,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    9.0125500950598383e+00,     1.6516499766017885e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0937920496496737e+01,     1.6217462541054843e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.5689393299915483e+01,     1.1781768935350161e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3033113732479737e+01,     6.9541836105192233e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.9607349913980006e+00,     9.0095159909417143e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0232884640950731e+01,     1.7693149983477920e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.4478608310884962e+00,     1.3917710789986937e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.1833078778670587e+00,    -7.9886300764572097e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.9713121279121602e-01,     1.4011010794514699e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.2522400364311270e+01,     4.0791337651615133e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.9448441560069920e+01,     2.3156397080935505e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2837965887093238e+01,     1.3648642747131416e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0119838235808345e+01,     1.4907887749538688e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.7904702359314189e+01,     3.2615617287816403e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6608381340306707e+01,     2.2370355141549531e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    6.2928038253063869e+00,    -1.8121597637402877e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    6.1707326026008484e+00,     8.9066692206617809e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1157355736746606e+01,     2.3051661967328546e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.1492458685161381e+01,     7.5229791798110236e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.9586620144372526e+00,    -1.5448614068687129e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.3143237263767460e+01,    -8.4960979804904964e+03,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    4.0146367315930807e+00,    -1.8690407852050494e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0008306029276230e+01,     6.4192020011231034e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1805503849709265e+00,    -8.9635579653703971e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -4.7105033661055327e+00,     6.9389345398806609e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.6341959128602532e+01,    -2.3167588546810912e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.3737255510385914e+01,     5.4619531925991323e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.0825481831992747e+01,     1.7390136534934711e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.2491210448253945e+01,     3.0657576932605236e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.8021473260127514e+01,     1.7359313572224375e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.8640996807522470e+00,    -1.7188959979561565e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    8.1334947640717630e+00,     2.1382808014313967e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.6537132775761929e+01,     1.8321123549938107e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0281095180393979e+01,     1.6815117203790684e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5877616492978426e-01,    -2.9154400977910714e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.4595497714871946e+01,     1.8633962426302787e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0215185993961814e+01,     7.1136973597125425e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -4.6141319921734954e+01,    -1.2808120407716478e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.1258722865523545e+00,    -1.7483765171938918e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.4650611761738546e+00,     1.3191187427625762e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.0683444682695860e+01,     7.7653436706970315e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.2264528542560615e+00,     1.6764460517202737e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.3972494003140035e+01,    -9.7627112546847711e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.7885183918934292e+01,     2.4177725635674036e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.0269052793761862e+01,     1.2128389239217810e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.5437467547289669e+00,     6.7940187528283627e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.9260865630367920e+00,    -1.6841718047918916e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3166365549774717e+01,     8.2081689482941256e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.0979788516563048e+01,     2.2312914107308428e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    8.4456897804306958e+00,    -1.0021503608164240e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.3557976537770188e+01,     3.9555023154410810e+04,    -5.6721115990986939e-04,    -3.1393140292885782e-07,     1.4667068294606790e-09, } , 
      {   -9.1707796191345849e+00,     1.0761964697315219e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.8878554779257783e+01,    -8.5416866003303658e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.6013231283913065e+01,    -9.5420612425118343e+03,    -2.7586509320370791e-04,    -3.9500194668399445e-07,     1.8691506661497048e-09, } , 
      {    2.0338480345586916e+01,     8.2365359301321441e+03,     4.3050126493040554e-04,     5.7961899645050571e-07,    -2.7420576989874243e-09, } , 
      {    1.0857228970022337e+01,     1.0192981885756942e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.1385940991077439e+01,    -9.2352747921143418e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -2.3581329688996895e+01,     1.7163642704813577e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0310531403142683e+01,     2.2133957508643518e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.8387575591406078e+00,     1.4895268511216311e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.2332112066104619e+01,     1.7148619596458873e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5050795104525207e+01,     9.3620535586547940e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6705673984667772e+01,     5.9766353650759711e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -1.1949477268249878e+02,    -7.1110191010218987e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -9.9379312245126641e+01,     8.4311716713514779e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1205990069542520e+02,     8.4357606439294959e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.2755676419770317e+01,     1.7860539725239192e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.1138946839679242e+00,     3.1821041020175962e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2565040752252887e+01,     1.6738685436118139e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.6854925097844173e+00,     9.0955251600264164e+03,    -9.7514478431693882e-05,    -4.1300598754567818e-08,     1.3963118829635744e-10, } , 
      {    9.4251017465436711e+00,    -1.0210153719292710e+04,    -2.7329914407438357e-04,    -3.9557470556685727e-07,     1.9243176149231587e-09, } , 
      {    8.2572973871250355e+00,     2.5158363101365627e+04,     1.7894560236215572e-04,     4.2193819480644026e-07,    -2.0047530527560420e-09, } , 
      {    9.5393178371538063e+00,     6.6567785959882658e+03,    -4.0134173398844353e-04,    -4.0547460093511400e-07,     1.9647510759276940e-09, } , 
      {    2.0050636860442207e+01,     4.8996051769137266e+04,     2.9589953376187841e-04,     7.7326327695927651e-07,    -3.6736756686075093e-09, } , 
      {    1.8437995909510800e+01,     3.2966970874935498e+04,    -1.1389849013726601e-05,     2.7305744021366641e-07,    -1.3554372415092296e-09, } , 
      {    1.1037736239756848e+01,     5.8189271749090622e+03,     3.3990238701221452e-05,     1.0463113117875294e-07,    -3.9392081217512018e-10, } , 
      {    2.5736463627976434e+01,     4.7881423209870976e+04,    -7.4914088744199009e-05,     3.3638797263785150e-07,    -1.6097268653879924e-09, } , 
      {    2.1391543328454333e+01,     2.1989983420341501e+04,    -3.4861764015068012e-05,     1.6919444783126340e-07,    -7.5708989255746248e-10, } , 
      {   -2.0220429343644826e+00,    -1.7322945443339206e+04,    -1.4728617815492056e-04,    -3.2001164522912496e-07,     1.5682704475163159e-09, } , 
      {    1.3070227823307842e+01,     2.2753373584464927e+04,    -1.5571095517634930e-04,     2.0797149441291985e-08,    -5.7789790788257277e-12, } , 
      {    7.7423839565092134e+00,     1.6866902305118398e+04,    -1.3060853904338436e-04,    -9.3271364853939196e-09,    -1.4733487768918831e-11, } , 
      {   -5.3877745788038123e-01,     5.9104131660801431e+02,     1.5201156014195893e-04,     1.8204336785897325e-07,    -9.1328716433733273e-10, } , 
      {    2.3472613458352178e+00,    -5.9457491827361628e+02,    -3.0731369618171262e-05,    -3.7010676415901974e-08,     1.7312696552421530e-10, } , 
      {    3.7241079157235699e+00,     9.6563130210000054e+03,    -3.8519755961091626e-05,     3.3249879728799080e-08,    -2.6251691204719155e-10, } , 
      {    1.4332857429921436e+01,     1.6137329208457575e+04,    -3.5456751605209691e-05,     1.0095760095424977e-07,    -4.8185631765476778e-10, } , 
      {    2.1084254987688777e+01,     2.2719556517002318e+04,    -1.3001355145324267e-04,     5.8909710391619711e-08,    -2.8996706267161348e-10, } , 
      {    2.1068483132280232e+01,     3.0595738405172240e+04,    -3.7687570853398700e-04,    -1.6505064833608397e-07,     8.1739101821386632e-10, } , 
      {   -9.9178676216791501e+00,    -3.3352026337540970e+04,    -4.5457556093052548e-04,    -8.2021748197473512e-07,     3.8865088746145956e-09, } , 
      {    1.2594649178871109e+01,     7.9133825064852299e+03,    -4.0666949697426059e-04,    -4.0424181670678864e-07,     1.8558716194241149e-09, } , 
      {    1.7025827997738890e+01,     3.2548218385517743e+04,    -4.4139405871757454e-04,    -2.3828107612852584e-07,     1.1121141030971645e-09, } , 
      {    3.0522036398095548e+01,     3.8228553760048831e+04,    -1.6537237069486825e-04,     1.5982367811456964e-07,    -7.7255060084804559e-10, } , 
      {    2.0423783333805176e+01,     2.3243113749498021e+04,     8.6222561781551429e-05,     3.1431440573693434e-07,    -1.4957956503272513e-09, } , 
      {    1.9035984916478082e+01,     1.3661291995616377e+04,    -1.8717451465641627e-04,    -8.1216666598623024e-08,     4.2924918511757197e-10, } , 
      {    1.4694648330452488e+01,     1.5056457696328725e+04,    -4.3709967609304254e-04,    -3.7288474655177743e-07,     1.7559466716106282e-09, } , 
      {    1.7855799895295259e+01,     1.3100504134642786e+04,    -2.4616923712701856e-04,    -1.5576714508198992e-07,     8.3139728546112098e-10, } , 
      {    1.4883510600748529e+01,     1.5542724280021379e+04,    -6.8754070352705350e-05,     6.4519683421210598e-08,    -3.6389630090400658e-10, } , 
      {    9.8575466807179719e+00,     5.2581393139354714e+03,    -2.5004483769380810e-05,     3.0080652695386049e-08,     8.2272881684288283e-12, } , 
      {    3.3431640925108496e+01,     3.8370589131604662e+04,    -5.3638185792811402e-04,    -2.7696435974425571e-07,     1.2928526434147994e-09, } , 
      {    1.2382404786545859e+01,     1.6029050884039192e+04,     3.0472343364628063e-04,     5.0077859562847293e-07,    -2.3734053758717331e-09, } , 
      {    2.2205543344379805e+01,     3.1784828221394317e+04,    -3.2054486755629332e-04,    -8.9883777738554441e-08,     3.6080318961852765e-10, } , 
      {    1.5457120339427181e+01,     5.4001146651661420e+03,    -4.0114586926127526e-04,    -4.0556186739771372e-07,     1.9632966348843656e-09, } , 
      {    6.0796591718498920e+00,     1.7985004445725132e+04,     1.1379299468025660e-04,     2.8366099415868547e-07,    -1.4488559897222259e-09, } , 
      {    1.3357120664247027e+01,    -1.7429605246773572e+03,    -3.7071569014249331e-04,    -4.3691893755272492e-07,     2.0632215826978519e-09, } , 
      {    1.5150670856163046e+01,     3.1852402335994364e+04,    -3.7707157326115527e-04,    -1.6496338187348428e-07,     8.1884545925719508e-10, } , 
      {    1.9790687671942415e+01,     3.1014550914915162e+04,     5.8260399428509712e-05,     3.4514235024038264e-07,    -1.5398264288456195e-09, } , 
      {    2.7046329967299174e+00,    -1.8471331049742941e+04,    -4.8727049867924258e-04,    -7.1991990636594797e-07,     3.4583650646899526e-09, } , 
      {    1.3719931056487919e+01,     3.2828469105045275e+04,     2.3833944769573141e-04,     5.6481278662942053e-07,    -2.6806802869589575e-09, } , 
      {    1.0833892776371094e+01,     3.4014085339926904e+04,     4.2108237745586157e-04,     7.8386683090429574e-07,    -3.7670944168205054e-09, } , 
      {    6.8421337988184661e+00,     8.3286614145625463e+03,     1.4974680151202385e-04,     2.5098387331274924e-07,    -1.2415060264484885e-09, } , 
      {    1.4325065307361879e+01,     6.9334957138501959e+03,     3.9967196294865017e-04,     5.4265195326590354e-07,    -2.5682035129415448e-09, } , 
      {    1.1934161817024240e+01,     8.4369397389809310e+03,    -1.9043338373946647e-04,    -1.4883712136147400e-07,     6.5004303176847680e-10, } , 
      {    3.1431085124106843e+01,     6.9284530222684452e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.2457235587761719e+00,    -3.6266954379414442e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.3270311114963469e+01,     1.0753655462728046e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.1065491223578405e+00,    -5.6334775402809464e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.8437561685300846e+01,    -8.9884792374339013e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.1418802903787022e+01,    -6.3375552309181467e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6805089418770311e+01,     1.3302383078196663e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.3250622579248812e+01,     1.8760625996971685e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1269930360495252e+01,     2.1435242154534255e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.9634986875447737e+01,     3.8890314837004778e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    4.1810867888089597e+01,     1.5442910589424519e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.8169391289717862e+00,     1.5703248465967584e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4523048685854883e+01,     2.5406317715771602e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.6815117941222884e+01,     3.8421266859550888e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    3.7781688179754575e+01,     3.2322960815904946e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.6086015811671601e+01,     3.1373754817145546e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.0672776760163995e+01,     1.4302021090024453e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.4182076655247471e+01,    -8.0584941313028721e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.0348009482383610e+01,     7.0922208510038245e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6144207197893860e+02,     8.4326058831443752e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5818626148940942e+02,    -8.4343264321366005e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.4341763966665646e+01,     1.5021298980837479e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.5032830135802189e+00,     5.2516981171045757e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.1396069717190578e+00,    -4.5137608521076402e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.4570448302087087e+01,     6.5943917787967175e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6219176450495549e+00,     2.2666636180522168e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.7144084725009421e+01,    -5.5065513314618165e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.1297924292500149e+01,     7.7301175510199582e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7998680236423908e+01,     1.8083207102269371e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.4360404757486771e+00,     2.2652371055203879e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    4.9192742519428847e+01,     2.2651713790250982e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.8986327426937404e+00,     5.7908944160775600e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.3901000779909271e+01,     8.7904044999907765e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -7.9446460138386044e+01,     9.0037194415735830e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8781374565814936e+01,     1.6210876777127414e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0361173294434671e+01,     1.4969707526403221e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.2095027614408167e+00,     8.4340097490759781e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.1465313250969999e+01,    -8.4329225662049957e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.4703244758591183e+01,     1.4260994190973335e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0112398570735134e+01,    -1.2826816706483335e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.0112182082370687e+01,     9.7973201086057234e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.1999524246014452e+00,     2.3878004670508813e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.6715460411370898e+01,     2.2635121741918640e+04,    -4.3916856999580938e-04,    -3.0403150756060110e-07,     1.4262733684561437e-09, } , 
      {   -2.0090231427723300e+00,     6.5948598856002741e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3595803939681673e+01,     2.4415642083342776e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0269420043165736e+01,     1.6180250663145414e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6974927512383125e+01,     1.2619238322379260e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.1513725712600884e+00,     1.5787229693452427e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.4968049672252704e+01,    -8.8266302179252307e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.1117010318153291e+00,     1.6280903680092133e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -7.9669714423727100e+01,     1.4323252946428465e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.1643212736684194e+02,     1.4323449049576144e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.6553232609212984e+00,    -8.4252991146546246e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -7.7177993825195357e-01,    -1.8668985469354245e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1990783024225671e+01,    -1.6514251728690677e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {   -8.4202901747176320e+00,     1.6431269762645559e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.0088425864480882e+01,     1.3586509597158485e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0151637790250499e+01,     1.4077905688317906e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9788492963656964e+01,     1.6446190200807672e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.8949266828798134e+01,     3.1556072335286906e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    8.6339084059642808e+00,    -1.2193927699778444e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.8623988978158525e+00,     6.5055637112988616e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.3345906073823492e+00,    -1.2194138143854641e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.6242318744694764e+00,     1.9652989781201584e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.2309756908375736e+01,     3.0583305636215584e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    7.2979802360819850e+00,    -2.5103966918290193e+04,    -6.1141782883174396e-04,    -9.3274758549706954e-07,     4.4164102280673174e-09, } , 
      {    5.7934174691882685e+00,    -8.4885774931213036e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.3083412418896710e+01,     1.7638371021086088e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.4639348077902651e+01,     2.5095258234259807e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.1382304082989247e+01,     1.1354966538154211e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7206685748166652e+01,     1.1070253590572311e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3533665826378520e+01,     3.1800557214341494e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    8.8003166378694981e+00,     8.3047671624375034e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.7345069012160046e+00,     7.7379243500654638e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -8.2655257698115236e-01,    -8.8308943688774561e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    6.6150914054125579e+01,    -1.2071973673620678e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.0942142072246188e+00,    -8.4695745081574951e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -4.8086377383882059e+01,     8.4330433466283012e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.1342187873411241e+01,    -8.4338889686526709e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -5.9356504323661312e+01,     1.0266368086059618e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.4187473396272079e+00,    -6.4165461976852603e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.6521689692216665e+01,     8.4298930265681138e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7292469928492007e+01,     2.2984450763789318e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.7434207048964126e+01,     1.5806788630690968e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.3086096201107544e+01,     2.4071780421702235e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.3366157072104116e+01,     1.5302992823286431e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.7433957088208153e+01,    -9.5501722596904801e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.6691996877164776e+01,     1.4634870336342976e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0490369360208747e+00,     8.9977840187524744e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.0038706238068240e+00,     3.1015261746014730e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.2230955436211683e+00,     1.5469157960783081e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -2.0640721680361676e+00,     5.6823193697895740e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5311566182318742e+01,     1.6572891469901051e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1194816977595176e+01,     3.1778775996430904e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    5.0377062082275588e+00,     1.6761761748277797e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.9292134258054237e+00,    -1.6762553416453426e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    2.5949085372692224e+01,     3.3432159670138244e+04,     4.5477142565769374e-04,     8.2013021551213540e-07,    -3.8879633156579240e-09, } , 
      {    2.1625306869623371e+00,     1.7336784724733090e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -3.3042509044762042e+01,     7.6662500962185431e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.7719617745805638e+00,     2.2830938966248446e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -2.3386462730056351e+01,     7.1263554140831184e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.4224522853100204e+01,     7.1056853365085553e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.8036265166667448e+01,     7.6669547281346931e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.9883484993704492e-01,     3.4008033114963502e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.2696850618006422e+01,     8.4430558915601014e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8691405680623120e+01,    -9.5460171741230806e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    7.7007045623344954e+00,     1.5394208723759439e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.1499960182625145e+01,     6.3797752410187268e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6531174316786910e+01,    -7.5113126131627723e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.6917223226053427e+01,     1.6205171935208913e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.3711560012282398e+01,     1.6204514670256018e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6518616404945014e+01,     2.1682528979479186e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -6.5783144899209134e-01,     6.0753403394825618e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.3162792071281977e+01,     3.2299980840393651e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    4.7370874644933343e+00,     1.6394314911390924e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.4904472304200169e+01,     7.1128909285783893e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.8779439531616653e+00,    -1.2631741256908140e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8023634066778243e+01,     2.6158578227654616e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.6924750389053198e+00,    -1.0305173426544416e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.6747377555764391e+00,     8.9426726832716595e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.6308239281867891e+00,     1.6123540716064583e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.4457509012398333e+01,     2.3975668158291963e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.6936215802153569e+01,     1.5585395873126383e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.7764963612964074e+01,     8.3716463807759574e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    2.1171610107935383e+01,     1.6068916974630327e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.8616830183911830e+00,     1.5641495548704374e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {   -1.4947351948932063e+01,    -6.3694408342897123e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8921639592096184e+01,     6.3001454372229018e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9191706532045281e+01,     7.1033880784570547e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -7.2664678503511873e+00,     2.3976772737356929e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.0926246086769403e+01,     8.8084381839781017e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7755855044985246e+01,     8.6011899179871143e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.8433870779735258e+01,     7.6701718386127777e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.0808134795793212e+00,    -3.3084610674840596e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.3913531275336322e+00,     2.5106848515415222e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.0354606924369605e+01,     3.2552817726224504e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.8177844787342686e+01,     1.8757915353140255e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.9443081067412813e+01,     7.5721731570821230e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6356660190553821e+01,     1.3852777767297193e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.1763099758968329e+02,     8.4331205552331376e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2402840073280227e+02,     8.4338117600478363e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.8438094535117429e+01,    -1.0056957596258588e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.5355073916352666e+01,     2.4864378860451234e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    2.9237994970178924e+01,     3.9441422476996275e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -3.5767179419065826e+01,     1.5675138000177643e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -4.8507625081406857e+01,    -8.4567726288022914e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.4929576078356973e+01,     1.4655759339064189e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.0673772623672466e+00,     2.5265905699951622e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.6486092205894680e+00,    -9.5131498278967938e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5455584096231352e+01,     1.7468462054302396e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    7.8923241061238878e+00,     1.5524730081982754e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.2867416704669779e+01,    -9.0460412585384420e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3688568850188702e+01,     1.9637257297053991e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.9710957587080504e+01,    -2.6203176366517298e+04,    -7.6373057947309224e-04,    -1.1831586999269560e-06,     5.6027493057423516e-09, } , 
      {    1.0769436943052112e+01,     7.1011210895576087e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1918670976014170e+00,     5.6511677746027945e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5570968999574973e+01,     2.1935864538187121e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.5421478538919303e+01,    -2.4756661463146652e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.7386109639454217e+00,     8.4320265951590281e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0868063328736604e+01,     1.6206006764239488e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -7.4881897621318743e+01,     1.0260334611664670e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0880521104423588e+01,     2.5073216980815079e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.9487395857290544e+00,     1.6805311138203990e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.1168595702626160e+02,     2.3348010914352744e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.4479990345544014e+01,    -8.4231481191578423e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {   -5.9790240472093394e+00,     1.1324298356476456e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -5.0687334274834583e+00,    -1.7025957612091447e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.9603923679125263e+01,     2.1837648760871489e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.1439843806744250e+00,    -1.7338650055920869e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -1.3620009954359201e+01,     8.6598926175535798e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.6875820443888383e+01,    -8.2070396977273940e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6899253808127828e+01,     1.7058056826349271e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.2673961617820048e+01,     1.6686427684373823e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.8978346959078394e+00,     8.4732723898596887e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6119251788678845e+01,     2.0926026959224375e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -9.1880402003538144e+00,     8.4683768357101853e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1109766552652935e+00,    -5.2090301981399716e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.0215364842670560e+00,     4.1496354448681693e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.5934509936042994e+01,     1.7646415299067223e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.5651728068683559e+01,     3.2540198487902126e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.4160295422285463e+01,     1.7408489375612360e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6918304228743828e+01,     2.5287946953396349e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    3.3617252542604369e+01,    -1.6791007977712656e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {   -4.8203162613816247e+01,     1.0306182987957529e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8806053212112666e+01,     1.0561319059549582e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7090244242316146e+00,     2.2781962084762323e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    7.6397789633758720e+01,    -8.2394587200981950e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -1.4171213946691296e+01,    -8.7051705940832471e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.7341528152186410e+01,     6.5516215369522497e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.5517594672231226e+01,     3.9583409087821077e+04,    -5.6721115990986939e-04,    -3.1393140292885782e-07,     1.4667068294606790e-09, } , 
      {    1.3289999871081453e+00,    -8.4349057201219457e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    5.3707022056018545e+01,     6.5523261688683979e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -5.5517974620127262e+01,     8.2825415004515799e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.3013669513425805e+00,     5.9977880784704384e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1276286285998147e+01,    -9.5394582185233521e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.9424588998097974e+01,    -8.2083746635213356e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -8.9431547297044602e+00,    -1.5932794375512942e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.1962713876216240e+01,     2.3492656627184629e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6630514767778688e+01,     2.1079958543955483e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -2.1559518184297218e+01,     8.4032109883505382e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.9900186722669746e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4864560350570706e+01,     8.1271212175672849e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2419680733025082e+01,     8.2496558269566885e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1309551111843653e+01,     3.2410850066971954e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3889820433821136e+01,     1.5595271271355005e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.4748320886337218e+00,    -7.9164922611100319e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.9386724158635953e+01,     1.6275484292186788e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.8054420020738888e+00,     5.7260902032987342e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3590493579739849e+01,     2.5223957229462794e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.7065026343676948e+01,     1.8116988529772338e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.4628921757674064e+01,     8.3026249241678797e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.1689761448340441e+01,     1.5370555678292361e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.1962203859993599e+00,    -7.6169390147420736e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.8742369033431373e+01,     7.8615486444264798e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    7.3719729000859351e+00,     1.5103918313796870e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.6914124358508289e+00,     7.1224509975354613e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.1851059165163864e+01,     1.6761029319819023e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.2959571452786633e+01,     1.6763285844912203e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.1338265871626792e+01,     2.0154231193584448e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    7.3065809005100419e+01,     1.6078493067133728e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.7604303013634475e+00,     2.5017252677689863e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    3.1801853041871261e+01,     2.8850206438480727e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1913704212011513e+00,    -2.9492215252940359e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8402978235339923e+01,     1.3227270054968463e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0430790161067481e+01,     9.4144543292763265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1051902575892917e+01,    -1.1767434762259506e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.6810139594328630e+00,     2.3676499482659219e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -7.6247243997449861e+00,     1.6388281436995974e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.3074979119371279e+00,     6.6909259402734858e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7862496266515716e+01,     2.2131112594585727e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.2346011186438268e+00,    -4.6985240945211473e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -7.0992569388039364e+00,    -2.8041064092759825e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.0747302025907508e+01,     2.3140664596787912e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -5.7069735880453706e+01,     1.6783733085048989e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.1132824553132558e+01,    -8.4336051336701821e+03,     1.8773023233838805e-04,     1.5264358749733739e-07,    -7.1854235677244325e-10, } , 
      {    1.3319262092895228e+01,    -2.7827578234367218e+02,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {    9.0576656240318449e+00,     6.2430640998291892e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3921474458866410e+01,     2.4796602665516926e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.4086874113816139e+01,     2.4629005510732854e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.5063465856801990e+01,     3.2309842904705492e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.7609043188686950e-01,     1.4719500472888552e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1104441561044906e+01,     2.9044834923569169e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0073366941236857e+01,     1.5787248645024669e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.3182876425000497e+01,    -8.4370392887128601e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    7.8852516813758356e+01,     7.1069002995683213e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.5107243927265362e+01,     3.0822999493112125e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.0500996517117198e+01,     9.5475960440297040e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.0954880613188696e-01,     8.2864229183869452e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.3613275877367563e+00,     6.8805604727906430e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.1121799061800663e+00,     2.0754500429268424e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.0841984236497488e+01,     2.5010168240885356e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    6.5687393549183781e+00,     1.7387835335094969e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -4.3457057005973443e+00,    -1.9378823765617005e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.0203002894218747e+00,     4.5158756990329912e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.4692291217670164e+00,     4.3561886590900474e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.8267845498941488e+00,     1.5025780393653504e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.4326652658313623e+00,     2.4409572324834004e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -5.8173451977464104e+01,     7.1116759655186215e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.5814489795611777e+01,     5.4004128286629066e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.3253187767895628e+01,     6.6557821833780072e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5043378782289768e+01,     5.7659319135243477e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.1366934336253394e+00,    -1.6551896771068015e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -1.5784529711719912e+01,    -5.7905398124794501e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.5894727529313339e+00,    -1.6553319462001768e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -9.2329760992010712e+00,    -5.5426339588045175e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8078876610980132e+01,     3.7724753425894432e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6646029363284107e-01,    -1.6979410513730843e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    5.1352032789950686e+00,    -6.4979012283067104e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.6230177249723077e+01,     6.7303449152265621e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    3.1650458837831472e+01,     1.3982160399167653e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5873371882071453e+01,     1.4996679532922721e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9783256403977088e+01,     1.9671038724556958e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    6.6049435628714024e+00,     6.8082456621658889e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.7385389502815656e+01,     1.3555358001971703e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.4137157897881121e+00,     7.6124698534584049e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8609748497208081e+01,     4.1731311175732211e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.2135923832870933e+01,    -1.1561249137149750e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.7629452567332544e+00,    -1.0229371399163514e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.1222663938480114e+01,     1.8773663096655895e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.6897679760212547e+00,    -1.2362640388758373e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    7.6749674930082969e+00,     6.7964682077510324e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.6429317042719667e+01,     5.3039951812026666e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7271691490296575e+01,     2.3913863640795476e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.6179163118854781e+01,     2.3844223243005847e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -1.2539661337852776e+01,     1.6125213498936642e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.3112497978003816e+01,     1.7406007195699589e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.9570914021212249e+00,    -1.7929755238843100e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    4.8295269378667678e+00,     2.3364833543587683e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -1.1149952062748465e+01,     2.5230534418075797e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -3.4332117641304990e+01,     1.0713840560830135e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5192295737909639e+01,     1.7198949385567761e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.9637073280674457e+01,     1.5693377399392179e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0607429655344067e+02,    -1.8476128256533580e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.9346070018883204e+00,     3.2599158214123388e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -4.2058853492902934e+01,     1.6371008440229116e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.9432289736143137e+01,     2.3811927137884286e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.8019078683647638e+01,     3.1033702944663750e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.7147341346347571e+00,    -1.7823817044420714e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.2966086037606937e+01,    -2.0042912458467279e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {    6.5855705909135001e+00,     7.8588390152730117e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0149001905394918e+01,     8.9100996710682484e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.3541975359684344e+01,     2.2416755807374877e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    8.8345230651710835e+00,    -1.5223855131096625e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.1669356771950318e+01,     1.4732119711210933e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.7673139374508132e+01,     1.6739866142618295e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3745602087251651e+01,    -5.8417035257462864e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.5619431643225454e+01,     2.1871430188374448e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.1204422795789242e+01,     1.3814488118650497e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.3760148173865010e+01,     2.5035737671609924e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.5972927427837750e+01,     8.7720252175798487e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.3959455226135891e+00,     9.0720830239415172e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.5898564938225221e+00,    -1.5766566177148918e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -7.2766622447906656e+01,     9.5982455991161623e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.5050913782126099e+00,    -3.7014820371770456e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7822815751347715e+01,     9.1262080924986985e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    3.8043045148534436e+00,     5.7797779593665564e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.1823839788679877e+01,     7.7896165443065138e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.1263772078816494e+00,     2.9863804088346522e+04,    -1.3454306871311288e-04,     1.9679072129917174e-07,    -9.4640478689392522e-10, } , 
      {    5.4429897670335123e+01,     2.9868406488025987e+04,    -4.3916856999580938e-04,    -3.0403150756060110e-07,     1.4262733684561437e-09, } , 
      {   -6.7896907316024624e+00,     1.0755509348455713e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2305444419321340e+01,     7.7740069774081621e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0657663189139761e+01,    -1.6871190071040965e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {   -8.5300839636109131e+01,     8.3018155271271626e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5438709459187152e+01,     6.4847938393110635e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0628041635344751e+01,    -1.2297125734371421e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.5134640585342805e+00,     1.7235360645386907e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.2059377117388824e+01,    -5.0382793005964413e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2118820683680983e+01,     9.3384853169614598e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1738767155942345e-01,     7.3073628699865958e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6236841025823210e+01,     1.4307808969713875e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.0787188884557061e+01,     1.8804814691842676e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0179362066767602e+01,     2.3767382327460600e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.5664874274899356e+01,    -8.4783481314856690e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    5.6118425979204822e-01,    -1.7056962774742966e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.2631207471557754e+01,     2.1113739971458446e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.2659706378546360e-01,     1.0437889882754553e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7818957187600839e+00,    -1.0517056700317607e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    2.3612716424249594e+01,     1.3697673245272947e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.0893050145211904e+01,     5.4155706033138745e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.9133429744943324e+00,    -8.0224115039601747e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.4554503265641188e+01,     1.5396509923599173e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1431921758598840e+01,     1.7062437398334376e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1316077466206412e+01,     1.7338207415666839e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6206976821458397e+01,     2.3765959636526848e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.4148515980079788e+01,     3.4570217407484568e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    6.0460258872395585e+01,     7.3421440695359979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5899106337820008e+00,     1.5699786774843635e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.2975432915359676e+01,     2.2233636619470679e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    3.7501809047227583e+00,    -8.1179784973873830e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    5.8169220207459077e+01,     6.7138364844904545e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.1592588252760077e+01,    -2.6207778766196763e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    2.1129665325664963e+01,     3.9535895418817185e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -1.1667907096398913e+01,     9.7522144777045087e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7937978037445486e+01,     3.3612169216837807e+04,    -4.1056475673581917e-04,    -2.0131403294392373e-07,     9.3825991705128483e-10, } , 
      {    3.4036305931344946e+00,     1.7326475443477600e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -4.5111695234077544e+01,     1.7921869951894019e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.3605406732914531e+00,     7.8140369610476828e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.0342570383833802e+00,     1.4807042139906871e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.3070657604446499e+01,     1.4136482413332329e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -5.3844604126097124e+00,     1.2438926915742750e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -8.9454861198495692e+00,     8.4319655758910340e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.2201296609378751e+01,    -8.4349667393899399e+03,     9.2781703036018566e-05,     1.1077022985990678e-07,    -5.2374421970263164e-10, } , 
      {    1.7850788417674824e+01,    -1.2222482062394456e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.1523551366924892e+01,     8.2914642874476467e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    6.3233998039057351e+00,     2.3694533166646550e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    2.5869038925644123e+01,     2.4208109982369293e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    7.9508510244898559e+01,    -5.3907488041861234e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -7.0407269343281280e+00,    -1.1479418137868754e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    8.3354836205777598e+00,     2.1225073659973532e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -8.7943263994638450e+00,     7.2489735435285311e+03,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    7.6783492942044003e+00,     1.0106512523932352e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6414687389436292e+01,     2.5075101263575099e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.6424217592449509e+01,     1.5744431823662000e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.7251417756173240e+00,     1.7302551181189978e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.4287608212333499e+01,    -8.6390137256940980e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.0589394000467998e+00,     2.4038577254853411e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.9580739947322758e+00,     8.5070538173626537e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.4393291363863181e+00,     8.3598784979183201e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5359371211495287e+01,     8.4733078678661041e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9439364165647397e+01,     2.7841995794941653e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.8621523883167868e+01,     8.4455331064303828e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.6452991002518225e+00,     7.1742211552332165e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.0656890008427812e+01,     9.1076221189788903e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.2116740297100907e+00,     9.1393260036981537e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8047563093720679e+01,     5.8369252189367462e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -1.8833646828963397e+00,    -1.2165583781238629e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -8.3884794187012446e-02,    -8.8084534433461486e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.3689215100462757e+00,     8.1206272812758079e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -9.0522741526667283e-01,     6.9143419002936080e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8288588608187018e+02,     8.4592665897290099e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7963007559234100e+02,    -8.4076657255519658e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.3340401724433963e+00,    -1.2090940109261664e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.7815465928368099e+01,     1.5361915388865259e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.8308659273197740e+00,     2.2906473862403334e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.4372330126394132e+01,     8.4338594039029285e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -7.9749269832751590e+00,     8.4330729113780453e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.1031797278959370e+01,     8.2279185895868759e+03,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.6224988672971424e+01,     2.6224605301950662e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3685367789185083e+01,     9.9667929020070023e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6481887748889434e+00,     7.7779361006917070e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1333007863770618e+01,     1.7670301110218134e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0072955838209928e+01,     2.5492686445496780e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.5729874116467617e+01,     2.3912942069357767e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.4754805806512124e+01,     1.2162747869883945e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.9476654245659198e+01,     1.4338893026290732e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.6537144357829487e+00,    -7.1151881866298891e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    6.2679483192882337e+00,     1.5526872320252378e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.4035549576604636e+01,     9.9279977754003121e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0884830882421191e+01,     3.2868785107782445e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.1193499286911340e+01,     7.3911579235585191e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.5055327212365004e+00,     1.6759554558377135e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3389650456686706e+00,     1.3655228511058849e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -4.1845345624287873e+00,     1.4949357383208113e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -8.5306794616472530e+00,     8.4088582586853845e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -1.1786489951176435e+01,    -8.4580740565955894e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.3635922532291644e+01,     1.6630605003426128e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.0532690701163073e+01,     8.4347667440922542e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.9301180829002224e+01,     1.5290925874496535e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3138431540323616e+01,    -8.1337451924752622e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.9362786704344597e+00,     2.4382359328716888e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -3.7210814739325777e+01,     1.0947525385109575e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.2866087561655750e+01,    -8.9930816371133660e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7242075489698966e+01,     1.6735849191739053e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.8434937990939652e+01,     1.6024817508644715e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.3740357851368170e+00,    -9.1691514995898706e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -1.3520035586849453e+01,     1.7390793799887604e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.5847610849582169e+01,     1.6065123680807119e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0471840488975577e+00,     6.9787762501062771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.6767913690008641e+01,     1.5019123386479938e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8959923402749022e+00,     2.5385095348664552e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.4885663100466232e+01,    -5.6072867250284617e+01,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.5031469826333257e+01,     3.0458364456829397e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.0221327381424121e+00,     8.2399162001342502e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    6.6236409832544894e+00,    -9.5033524932613432e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8728676924069749e+01,     1.6763261109758037e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -7.8616465297662437e+00,     3.5256987394812404e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6121143373063592e+01,     1.6023627326358437e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.6648798013993193e+00,     1.6685004993440070e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.2313716053828925e+00,    -1.8399155532255067e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -9.5258687477027646e+00,     2.3345835319995200e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1088384773599161e+01,     7.3718066955853847e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -1.0669345940888126e+01,    -1.6592423018432804e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -6.2503489996544232e+01,     1.5629289623884784e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.3799228874161777e+01,     8.4292084018804962e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.0543418384632595e+01,    -8.4377239134004813e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.1248087848239186e+01,     1.5969556687379909e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.4827031219651108e+00,     5.2892984000965871e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.2777442711543429e+01,     3.0509006631026539e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -2.0161749918197658e+00,    -1.6605577213746084e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    1.0422154213115741e+01,    -1.2013790839804510e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    3.3723838430811483e+01,     8.0957060092123047e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.4124926160167163e+01,     8.0811655149542239e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2222530812737784e+01,     1.7183282453567372e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7001162684287152e+01,     1.6808515280023508e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.2524886673922380e+00,    -2.5264350056519052e+04,    -1.7874973763498745e-04,    -4.2202546126903998e-07,     2.0032986117127136e-09, } , 
      {    9.6022854755591851e+00,     2.4011165336834430e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -3.8373200561620557e+01,     8.4337795307488850e+03,    -5.3195027303182794e-04,    -4.1480173742050790e-07,     1.9500175881587754e-09, } , 
      {   -4.1629011051149739e+01,    -8.4331527845320888e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -9.8507990898062356e+00,     1.6788465972992170e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6176582308123720e+01,     6.9449948226033102e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.0389409392050624e-01,     3.2286862929194194e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    3.4435130835277597e+01,    -8.1494776766228497e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -1.1661909900143549e+01,     9.6138234531932767e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.5311868940135209e+00,     5.6147565147836458e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.2358361200870274e+00,    -1.6933009847905501e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -3.2998478542849462e+01,     1.7859513144865050e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.4343625673499631e+01,     9.1171306728723721e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.2441830880524542e+01,     7.1077628792922533e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.2261671931921697e+01,     2.2716476772540092e+04,    -4.3916856999580938e-04,    -3.0403150756060110e-07,     1.4262733684561437e-09, } , 
      {    1.7312811325038023e+01,     1.4121845810152712e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.9945557321131268e+01,     3.2228470629090902e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.5862521324907148e+00,    -8.6741048928035525e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.4004116071245996e+00,     1.0931277386783469e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.3122586041311365e+01,     7.1108133857946887e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.1560082435771015e+00,     1.2325105715671151e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.4123303245855094e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0433723871439220e+01,     8.1332465195981858e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.4307797252359418e+00,     1.1999006607347343e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.7986238900698595e+01,     5.4722901826540174e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    2.5528161202273381e+01,     1.1903092029871445e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.8165348842162210e+00,     2.3278048807933239e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    3.6094190295312586e+01,     5.3438100831005795e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.4273730898960171e+00,     2.7085052049084857e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3110686729510679e+01,     9.4985655458881774e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8379390212828916e+01,     1.5450598795590977e+04,     2.1200039306567634e-04,     3.9315480655896706e-07,    -1.8644479734429424e-09, } , 
      {    9.1332121284259795e+00,     7.0982675058211080e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1344094863301045e+01,     1.6662298622238119e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    7.7267824380327905e+00,     8.5547892521617705e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6341179875635465e+01,     2.1840263333819617e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    3.6236864907732226e+01,     1.3440611973280858e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    9.1848053660405604e+00,     1.6412907446181784e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8302996089282750e+01,    -4.9300588579065643e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.3158950669226822e+01,     2.3347201517312027e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.1692733669385284e+01,     1.2521410208910793e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.1085794456519011e+01,    -1.8456333344434970e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.2557211827266244e-01,     7.9602630946191921e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.7470197996266563e+01,     3.2385093965877881e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.0742608441328192e+01,     2.3906390603919073e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.1070582578869931e+01,     1.0923218601509270e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2618074195321142e+01,     2.3747492788071613e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.5921445154881042e+01,     1.6451619905840671e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.5529609007407984e+01,    -1.6588690743877592e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.7967323222881788e+01,     3.1127926506254309e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.3616286746873911e+01,     3.0392241164304927e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    4.4657983141738924e+01,     6.0457139975469991e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1799655132248112e+01,     2.4898160287954201e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.1106703852966401e+01,     9.9743984412256577e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1235034869609237e+01,     2.4820787449953923e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.9938891488105259e+00,    -4.5954312819662698e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.6319010445633168e+01,     8.4027979759627342e+03,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {   -4.9186664003090410e+00,    -4.7127510038586735e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.4443160373149437e+00,    -1.1688967603699475e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.7004401644652546e+01,     2.3986529729079935e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5686679946145562e+01,     1.4487644307942461e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.0332743566530265e+01,     1.5638668867659857e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.2812904851365417e+01,     1.5656389178538782e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.0841521468572878e+01,     1.7485636999495408e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.2903561094849202e+01,    -7.5852910682815782e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.1943764352518009e+01,    -1.1103708035063009e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3342511767603796e+01,     3.1050597902298221e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.7966047710719231e+00,     1.6621726614973391e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.0030264293758698e+01,     3.6628142539172433e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.0142910282014267e+01,     1.6791654533329329e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.2168214886101623e+01,     2.9973483045090099e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.2611150293702954e+01,     5.7571129885745959e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8121725069317973e+00,     2.7622143989816493e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4160330594439532e+01,     8.4107053568598858e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.1701959769945177e+01,     1.5572966291250765e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.0933770504204034e+01,    -1.6798240209708314e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.9227656894708289e+00,     8.3253857249484790e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0557699689520412e+01,     2.3136062197108447e+04,     4.5477142565769374e-04,     8.2013021551213540e-07,    -3.8879633156579240e-09, } , 
      {    1.1894413086667715e+01,     9.0552147870863027e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.5157148846542901e+01,     9.0809614123942574e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.2434786521885670e+01,     9.0617804562367564e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.2674439655490470e+00,     2.5075429896051552e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.3487200440351588e+00,     2.2437917996555465e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.4912885326822432e+00,    -1.7877497487098783e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -2.8764042548010295e+01,     8.3252457149506081e+03,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {    1.1671917126579480e+01,    -2.2111729762167838e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.0551588139161909e+00,    -1.7876074796165030e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -2.0525471373103883e+01,    -7.1734983381331404e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5912041891684057e+00,     9.3560200842598460e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.1618480899756962e+00,     5.8400537241300062e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1499593418981979e+01,     7.1203087592658358e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.1730726237041381e+01,     7.3361105951410500e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4912556472689319e+01,     1.6520312011413331e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3754545104316122e+01,    -1.8465278824779536e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.8583590193345383e+01,     3.2288912364225085e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    8.9152248171638648e-01,    -1.8445575149652650e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -2.0926925216593439e+01,     2.4971404301397004e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.1403458564320330e+01,     1.5307467674831403e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8188231780661560e+01,     4.0041344128631557e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    7.9094409659217417e+00,     1.6763321043872635e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3275138963063117e+01,     1.4541912346571344e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.7236827597068853e+01,    -6.1870468421443638e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.1095941880690045e+01,    -1.4293050739704508e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {    4.3654716324144019e+01,     8.4131912414088765e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.0398905834614837e+01,    -8.4537410738720973e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    4.9975190904487405e+01,    -1.4286004420543009e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {   -1.0655520268995950e+01,    -5.9356549284975035e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6233655464767093e+01,     1.5125870977943036e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.8518958253835365e+00,     1.5419930613913224e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.6804834067092598e+00,     2.0361479459575436e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.1960749207418925e+01,     8.8851547246160044e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.2790847308197666e+00,     9.6260916037980187e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6435751246893673e+01,     2.3965911166568956e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.9837901439858602e+01,     1.6083147889008324e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -7.2908958841782976e+00,     1.7610332671724522e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.1218145238002634e+01,     3.8339911828929435e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    4.4408581698611968e+01,     1.5036494434820634e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.1155497251297479e+00,    -3.6842833010593404e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.3573915267585491e+01,    -1.0430833741628227e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.2885416905207812e+01,     8.5267509028002041e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    5.5355344297328886e+00,     1.7486025141288945e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9825641695460099e+01,     1.3620291024661450e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.9135789835931050e+01,     2.9290766800351907e+04,    -2.8685581935446115e-04,    -5.3620393130714676e-08,     2.3993429078110924e-10, } , 
      {    1.9340391948995730e+01,     1.8476228071962356e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.4957774379087487e+00,    -1.6768057636452031e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.0768353924550505e+01,     9.2724677184515331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5807619743491447e+01,     2.2356143178743772e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.2098403996325970e+00,    -1.6512793214636979e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    1.6521944559054909e+00,     8.3582767190082886e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.3500879695323786e+01,    -6.7216968678834801e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4246061023264826e+01,     1.6093509614217375e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.5321422938110871e+01,    -4.9592387505185707e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -4.0633198384762226e+00,     1.0335204198160812e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.6893205571066883e+00,    -1.6561950953838016e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.7496440623318161e+00,     1.0619742384911348e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.7133863469264043e+01,     8.4330087152531196e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.3878052979734861e+01,    -8.4339236000278543e+03,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    4.0077746275710751e+01,    -1.6747237144203500e+04,    -1.7874973763498745e-04,    -4.2202546126903998e-07,     2.0032986117127136e-09, } , 
      {    1.0316931868415761e+01,     8.2431043615179933e+03,     3.9957403058506603e-04,     5.4269558649720345e-07,    -2.5674762924198806e-09, } , 
      {    3.4405854710676902e+00,    -1.7581980949254550e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.2960650790484234e+01,     8.2702743894676642e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.2336943825343312e+01,     1.2453847353904865e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.2483789086308015e+00,    -9.2936632772308767e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -3.7562590528881135e-01,     8.8402266663079317e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3521644524805936e+01,     4.0556051036262579e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -1.8467596262858383e+01,     8.4597745482670452e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0770504802771931e+01,     2.4505201238000900e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.7019084844423123e+01,     2.9464267849780219e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -2.0281442632707559e+01,     1.6784449022112931e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    8.7544454946214412e+00,     7.0563285111184005e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3045239661962469e+01,     2.5210293712784474e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    4.9375416188485055e+01,     7.0984714211837509e+03,    -5.2529319941377664e-04,    -6.1838954652883535e-07,     2.9213417982617307e-09, } , 
      {   -2.3600330207019613e+01,    -8.0093001540317746e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.0160506881577042e+01,     3.7739990177294159e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    1.6141511327216698e+01,    -1.0764894985099954e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -3.9298794384485181e+00,    -4.0945030068700335e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -9.6479979409460235e+00,     9.2816412469993957e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.3223404245129458e+01,     1.4300809092871248e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.0713353404251158e+01,     7.9935458492450780e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.8614199972256024e+01,    -7.0064973948865891e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.6993987883896722e+01,     5.9416904760659872e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.6661063229250260e+01,     7.6213482419111442e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.3098593212391574e+00,     7.0908118083021209e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4864690624944213e+01,     8.4071577670139286e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4959750223456936e+01,     1.7136033727735252e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.4819214309132406e+01,     1.1779139103033978e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0461038791533980e+01,    -2.5028715857117808e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    6.2688975125227664e+00,     6.8713716848747317e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.6246126084199854e+00,    -1.6761054054973189e+04,    -6.4451188944343444e-04,    -9.0077412322789566e-07,     4.2620455520020410e-09, } , 
      {    1.3546877774434368e+01,     1.4082314931777100e+02,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    2.1051013737284926e+01,     3.8497646183269870e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    1.1603025644428504e+01,    -7.7844950562914346e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.3683831004761451e+01,     1.6950544859015747e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.1837279400564803e+01,    -7.1201048439031929e+03,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -6.6059416290513553e+01,     9.9323068425878901e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4481367799319433e+00,    -9.5356431973816671e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -5.8638263875052836e+01,     1.2635023559873844e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7650750040855154e+01,     2.2860880543091273e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    3.0475527654957464e+00,     7.4339159921639180e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6076878430499150e+01,     1.6623149305907144e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.3539742425542554e+00,    -3.3860133284961835e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.1757885460812876e+00,     1.0096144110222546e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.1453759889691071e+00,     1.0858802472849614e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.6193223651063189e-01,    -6.1909282600797292e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8739256627491212e+01,     2.4446793678529557e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0469509080467656e+01,     7.3717972197992640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9652159197444554e+01,    -7.5035531472775983e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.7324068593315410e+01,    -8.8499230770155737e+03,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    4.5973161449208071e+00,     1.3211850943929270e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0468360379240908e+01,     1.6763580273299365e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.5263242726666695e+01,     3.2911889549617263e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -6.1347815137936280e+01,     1.0928457098608125e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3549139743957159e+01,     1.0441946005526768e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6369331207129862e+00,    -1.0491871010633187e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    6.4009972250486946e-01,     1.6760734891431861e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1491977454733670e+01,     1.5334690146734640e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4109726643455957e+01,     3.0927764750241364e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    5.8697159562854431e+00,     1.4998557921750267e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.3219037329162148e+01,     2.2182642129315464e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    2.1132821626137442e+01,     2.2843558204570825e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.3486488253274425e+00,     1.4354865436605822e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.7530758874651905e+01,     2.9618199434511327e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    3.2585884857823913e+01,     2.0521515013755601e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0910946723853378e+01,     2.2862303234025025e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.1655918296248339e+01,    -3.4662030386764400e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.4869421232360693e+01,    -1.2011638929671199e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.1108529443330708e+01,     2.9775732477640058e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.3487136441534975e+01,    -6.1835607821724388e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.0579760759844284e+01,     2.7022172428052712e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -5.7649862487985963e+00,    -2.3309033462546877e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.7754043252868051e+01,    -7.3626236912514014e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.0323830408774491e+01,     3.3155388161748211e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {   -2.5633799887253472e+00,     7.8574163243392586e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.2270963024582819e+00,    -3.5656234476862608e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -5.8530396696348944e+01,     1.7253747464950564e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.9501256250112419e+01,     4.1654963122824302e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3075725257459556e+00,     1.6127291041720342e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.8399059275582643e+00,    -7.1081846051510483e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    8.7402020271417182e+00,     1.6449318706000933e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.0288243448941961e+00,    -2.5092991245360361e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    2.0705512739977049e+01,     3.2270419099579729e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    1.0222564457385124e+01,    -7.8750483506149167e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    8.0932647037378125e+00,     1.6895265804737668e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    8.7868505994134534e+00,    -9.1731074312011187e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.7695553522416375e+01,     3.8891019468920931e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    1.4583110405159479e+01,    -6.3701455318599164e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.8211936988047935e+01,     1.4558637613354860e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.9327655794820146e+01,     3.6545826984746273e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.9637434064845849e+01,     4.7454645193040538e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {    3.6463456454143454e+01,     2.2607160448882249e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.6821516353374948e+00,     8.4330703235526744e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.7323412160713303e+01,     8.6298432962039442e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5193139469432474e+01,     2.2504988739955370e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.4109729116573275e-02,    -1.6745296835671430e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    9.6038499292827595e+00,     7.5226998766913107e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0187026531019292e+01,     3.2527080576702672e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    5.9379493577616014e+00,     1.3664390490647056e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3863217787086564e+01,     1.6765970874178747e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.9110046924252735e+01,     2.6693585808172393e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    8.1187309769402034e+01,     1.6133843283769344e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2319518579715767e+01,    -9.8351964437496263e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    5.4170015114321274e+01,    -1.0601436845900053e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.5282852274326100e+00,     1.6758344290552479e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.6360884229470969e+00,     1.5828364133374345e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.1765845405217219e+01,     5.9379925943637927e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.9055771278990747e+01,     1.6759504066567806e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.0451285145658272e+02,     8.4341216606094749e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0125704096705351e+02,    -8.4328106546714971e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    2.1312736847258115e+01,     1.6671487410154034e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.2077896743636842e+01,    -6.3286758981904914e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.8923302855986872e+00,     7.0797911815797570e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.2040879762103565e+00,     1.5770387898330482e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.1512787217363766e+01,     1.3979530566851472e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1278160182799127e+01,     6.0476286704883696e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    8.5482472069614648e+00,    -7.9978472917314821e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.8724399580620748e+01,     4.6696989187064828e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {    2.6527927338569683e+01,     5.4061668901295470e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    7.1086252518651563e+01,     6.0915623738398590e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.4067090729992017e+01,     2.4445268425278340e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.3744468315438196e-01,    -1.6499639019323698e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {   -8.8230616114229541e+00,     1.0538256417090483e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.8080426928304512e+00,     7.3162077660580098e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.5385739015567149e+01,     3.3341988170803743e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {   -5.8523678597199492e+01,     7.8267340752783221e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.3681483614265346e+00,     2.2621389500417725e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    5.9845335605160727e+01,     2.4135480055416094e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.5785526006122421e+01,    -7.4171881634433203e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0738569396356493e+01,     2.4317426306027144e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.0634184371112088e+01,    -1.2716610439259691e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.4998108233323791e+01,     7.1099927644596219e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.1614770226878534e+01,     9.9262733681929421e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.4672546500072489e+01,     1.4830416300041798e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8659023318584580e+01,     3.9429957224394806e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -1.0245368550062413e+01,     1.0145235447397494e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.5917905418369688e+01,     7.1261488792376531e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0956846514296878e+01,     7.3214674306903780e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.9391447745581971e+01,     1.5817882615762130e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    6.6424514797566543e+00,     6.2205156334270778e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3618993525009445e+01,     9.0493222231609179e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0272795917007944e+00,    -8.6933169852217343e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.4158393033753529e+00,     1.4502247541523324e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {   -2.3235308587866225e+01,     1.8702527823582536e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.5121814845452199e+01,     1.6847820860556385e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0378111782204343e+01,     7.1103916599358972e+03,    -5.5838726002546712e-04,    -5.8641608425966146e-07,     2.7669771221964547e-09, } , 
      {   -6.6663095151585154e+01,     7.8843924814724596e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.5598369139550012e+01,     2.0581033590780404e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.1179699685898804e+01,     2.5166100068263131e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.4310379974233090e+01,     1.7409652837119385e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.7117983796916569e+01,     1.2174789905043150e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.1097608253809454e+00,    -6.3749637296145465e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.0250990055729986e+01,     6.4813395770067300e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -8.4412125713212859e+01,     2.3347814811205062e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.6456809588597139e+01,     1.6146469235642449e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.1133218523642991e+01,     2.4046110225844419e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -9.3274181286635347e+00,     6.5496726530705891e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.9817352576290123e+01,    -2.7304499019486439e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.3485798298419770e+01,     2.3973375533766655e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -8.6610065357459931e+01,     8.5960618687009792e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.3167297426175992e+01,     2.2941221692353975e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.6645796967203474e+01,     1.5026699604622834e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8005561669276528e+01,     1.4007882289321440e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -6.8080435371976762e+01,     7.1085835006273219e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.0811101106265191e+01,     2.5612738676409062e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.0994642069574434e+01,     5.6993351795552599e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    1.5242708272557307e+01,     6.6812248694345544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1898053738718621e+01,     1.5017143895270076e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.0377915848616666e+01,     4.0620485386075241e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -1.0546795948164547e+01,     7.4340035644354793e+02,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5688941941034944e+00,     7.9073302815218249e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.1899338650092570e+01,     9.7430409491566443e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6933748473370819e+01,     5.3836249849984915e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.9209466248104636e+00,     9.0268288536760447e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.2077047333302620e+00,    -9.7833813322593360e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -7.1657996973870269e+01,     8.4330367853572443e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -7.4913807463399451e+01,    -8.4338955299237296e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.4563971293253960e+01,     1.3108312354191904e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.2241855483029553e+00,    -3.4233543543109931e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.7394230642003485e+00,     2.5090625350034708e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.1080198367926226e+01,     2.5091072664146770e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.6051767625488068e+01,     1.9026983395995776e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.4804198650487201e+01,     8.2576828599446799e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.8060009140016383e+01,    -8.6092494553362940e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.3701917408736740e+01,     2.3629600143956799e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.4367256240508897e+01,     3.1766775786838531e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    3.7042249830896656e+00,     2.3979065361882236e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -2.7741808538907264e-01,    -9.5169431217199999e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.9857113276858541e+01,     1.6417838478337704e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1057560372679173e+01,     1.6702639005340818e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.2070017960053331e+01,    -8.5519444608865451e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.2825274064398897e+01,     2.3836535036839385e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.0209720507443055e+01,     8.4062361955762190e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    3.6086755518222702e+01,     3.1020548749350470e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {   -5.1349864896475232e+01,     1.5669104525782697e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.3542974875287534e+01,     2.4493194344354899e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -4.9029268887782376e+00,     4.8962774304036002e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3272807432378748e+01,     3.0497541378425063e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    7.6447525709850879e+01,     1.6764811098163420e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.7674461768693712e+00,    -9.3565789350633841e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.4910811083257343e+01,    -6.5765817518654258e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.2776719271939427e+00,     6.5758981151979478e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.0691682763574043e+01,     1.6761175920549882e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -5.8658602397607027e+01,    -1.6763139244181344e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.2533909077848477e+01,     7.8466342288802565e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8566635277638056e+01,    -5.1135303617688296e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.5140196440358533e+01,     1.7325532144721412e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.7947861543689651e+01,    -8.2526310473654594e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.7644260932030296e+00,     2.2246402214712063e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.2531584482562028e+01,     1.5778040905536514e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5376956431089113e+01,     1.6564082844922490e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9594006070614810e+01,     7.7099867405106897e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.2628145654893439e+00,    -9.8635634255876448e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.5604025750924873e+01,     1.7894321152742152e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0411013250112553e+01,     8.7795877886310536e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.7737527919840169e+01,     2.4039498826291125e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.5319515985540821e+00,     6.0791431090918886e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6452819785096793e+01,     3.0940918945554644e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    1.3056825758265070e+01,     8.0267056489730412e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.5647603218469062e+00,     1.2503361265555421e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.2363964428086698e+00,     1.4337944755797042e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.0218061471861226e+01,     1.4341399941357678e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.8285510349512968e+01,     1.6529301645373089e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1978181883891432e+00,     7.7095985987171534e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.6221073067834162e+00,     1.7074996458730293e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8088152566522368e+01,    -2.0463369304285288e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -8.2830384867202618e+00,    -2.5074849498766256e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -6.8065081000429224e+00,     1.2045905946049763e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.3480795088409110e+00,     7.8691482965284986e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.2121804907047640e+00,    -2.5235964123108792e+04,    -1.7874973763498745e-04,    -4.2202546126903998e-07,     2.0032986117127136e-09, } , 
      {   -1.7841299647601005e+01,     1.5788549231476432e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -8.2120854920067821e+00,     1.0362488378762726e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.7742637566487414e+00,     8.8683736780826166e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3106878996726090e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9475495793173806e+01,     9.2641843556444346e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1448788279414142e+01,     3.3497092692827995e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.1629486243377116e+01,     6.6501971203869907e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.0578996530471017e+01,    -1.6774363507185197e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    8.7365740123611335e+00,     7.7118585680671813e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.0544028816050439e+01,     8.7101777319159173e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.7690869435077730e+01,     8.3763782589757175e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.7867819332226318e+01,     2.2668963103536214e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -6.0027288547023119e+01,     1.1225004916189737e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0589174451747837e+01,     2.4873596075725509e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -1.3327225712118377e+01,     1.6115801053964697e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -8.5226889406198403e+00,    -3.1971273789689712e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -6.4241355112644598e+01,     2.3363454891067333e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.7443005687777571e+01,     2.4916047371210661e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.5158532725268905e+01,     2.6124796800151649e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -1.3475809158113613e+01,    -5.4313002702894282e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7644548351761252e+01,     1.5458643073572104e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.5115824630374984e+01,     2.0030278266746980e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -4.8507141000157947e+00,     2.3154842352154983e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4205018622023506e+01,     1.8142725679294166e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.2817067274529510e+00,    -1.8034520495972338e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    4.6576749417905461e-02,     1.2798166457932773e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -9.9755701296215804e-02,    -8.4360691816289673e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.9870922037829750e+00,     1.5615889833865980e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.3381058477423657e+00,     2.2649964879943731e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -1.4950208604910280e+01,     2.5159541112663399e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.2550442540091407e+00,     1.9968458489882447e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.7112373086131239e+01,     5.9920297409609948e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.9706177723585782e+01,     2.5712597636536553e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.2123061731543388e+01,     1.0405067486606372e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    1.7674772729440011e+01,     2.5168845893981976e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.8355039052760475e+01,     1.8739881669152928e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.1936114890901965e+00,     8.5032960015458630e+03,    -4.6576215180844699e-04,    -4.7874866195885568e-07,     2.2587469402893278e-09, } , 
      {    1.1139566632893866e+01,     1.1732342603120820e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5269825948582257e+01,     1.5605580552610492e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.3811766787824581e+01,     2.2654119965511130e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    4.3043026700708600e+00,     1.5640050856530263e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4420804721105185e+01,     1.5344679552740836e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.7823059642507808e+01,    -1.3645184081997070e+03,    -9.2625108217020158e-05,    -1.0766742230080575e-07,     5.0823018190712651e-10, } , 
      {    3.8880335860657689e+00,     8.4404697391193276e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9005524833430609e+01,     7.6780771405642145e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.4924616071837931e+01,    -7.1147178375764670e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.6861381052396318e+00,     1.4537967535780295e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.1222025176098320e+01,     5.9972894055933630e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    7.9923495130556699e+00,    -8.9737562790106324e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4315908576691353e+01,    -6.1866740279976511e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -7.9771889942784817e+00,     2.3967360292384983e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    3.5168459423617435e+01,     2.3985080603263908e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.5709851928379173e+01,     3.8970296956758684e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -1.0935597133476932e+01,     8.2603126922608644e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.4191407623006114e+01,    -8.6066196230201094e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.2439145707436960e+01,     7.1103840135114460e+03,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -3.9977202504127774e+01,    -7.1081922515754977e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.7131431871598679e+01,     1.4953959782887581e+04,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {   -6.1762154327182186e+00,    -1.1677234346273667e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4196689291441640e+01,     3.0978104015136370e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {   -1.9607997364779205e+01,     9.2172068971867266e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6785041227238146e+01,     1.8955990090583382e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.9150195030558017e+01,    -8.2924459492633723e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.1186210886114178e+00,     5.9923925843777324e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4934850978402741e+01,     2.4291191235703209e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.8457189285170621e+01,     7.2780562940286018e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1610604182037938e+00,    -1.6742979388443509e+04,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    6.0641178959250244e+00,     8.2511391637130018e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1676919909745777e+01,     1.7899328046762963e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -5.3530553572090014e+00,    -4.4875239765630213e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -5.0646630835997001e+01,     4.2663005156750205e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.3222129610811479e+01,     2.9243227408173716e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.2189814270446888e+01,     1.6557523889322758e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3008811657128501e+01,     1.4329909953602029e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.0446437082074343e+01,     8.4263454367221548e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7190626592545161e+01,    -8.4405868785588173e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    9.3564419961388179e+00,     5.7751207014402617e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.5077853070778204e+01,     1.0502899729959634e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.6764147069393374e+01,     1.0452046853112526e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.1545227003509240e+00,     1.5500336364297567e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.3389675390013592e+01,     1.9562446930254746e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0042548004067493e+01,     2.4061286842527679e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7132175113737798e+01,     6.5030218009426571e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.5309391444965836e+00,    -4.2680239719595193e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5361706957615517e+00,    -1.0432352433608390e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.8270772053984707e+01,     3.0983363679768743e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    1.0489255618702455e+01,     2.3933560631868742e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.0862293821519788e+00,    -9.3128081015542248e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    8.8015494152804141e+00,     1.6440129918085022e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.0074159635711929e+01,     1.0968272000506038e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.5917204637382731e-01,    -1.0256584395280632e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.5944737376656242e+01,     3.3970295755849293e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    3.2657641834972672e+01,     3.1016782263854959e+04,    -2.8685581935446115e-04,    -5.3620393130714676e-08,     2.3993429078110924e-10, } , 
      {   -9.0023798130156109e+01,    -1.8478089288010365e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.7482770732090680e+00,     1.7400774448666645e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.4695675967935891e+01,     2.2673372330705282e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.6540798517960848e+01,    -5.8846587481992719e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.3340886444255062e+00,    -1.1671455404373394e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.5377847910171063e+00,    -8.8330366071470817e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.4129324814979487e+00,     1.3400599950285953e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.4139486958570082e+01,     1.5636039035343672e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.6619648243405244e+01,     1.5659019010854963e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.4169772982073255e+01,     1.5648233655015469e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.0145325208943246e+00,     1.7778210992072250e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.1976091560612247e+01,     6.5542750527500575e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    6.0033175888461852e+00,    -1.7300293668076654e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    3.9393850589546979e+01,    -9.9258082958407658e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.9486985578237125e+01,     2.3282651207612707e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.8189079982184683e+01,     1.5655007189668377e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.6546309855369241e+01,     1.5494302889902618e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.4740991150292572e+01,     1.7123827802915668e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -5.3586055761169078e+00,     1.7277053936112367e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -7.9652483459400827e+01,     1.2939004299949374e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.1447717708554475e+00,     1.2110340295862432e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.8055503543450989e+01,    -1.9544490512694028e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.7198814979008112e+01,     1.6224030972440692e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0497852576110059e+01,     1.9178193922106630e+01,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    5.9736489748820700e+00,    -1.6697723232552944e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {   -1.1016453720865441e+02,     8.4332391856132144e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1342034769818359e+02,    -8.4336931296677631e+03,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    6.3536296441089872e+00,     6.7680822743407743e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.5667951173487621e+01,     1.6486530583910364e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2520140506411391e+01,     2.2024446183148764e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.7070129877073132e+01,     3.9219253772843178e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -1.7062698110249784e+01,     8.7374448785657041e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.0318508599778966e+01,    -8.1294874367152679e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0707290180643334e+01,     2.4381860655839813e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    5.3830427318436040e+00,     2.1853381245019074e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.2437282386395335e+01,     1.5950411863056563e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2223187088848805e+01,     1.9197835661535661e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.8042596833760058e+01,     8.9942108876801012e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1645193690641085e+01,    -7.8727214276008726e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    2.4552291906062614e+01,     2.3595818716453836e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -6.8132058053335840e+01,     1.0594395855136398e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6747286253346832e+01,     2.2357796081153618e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    7.9555752535580151e+00,     1.3805266083918394e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.4744509300214972e+00,     1.7544108726994348e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.5959118514701061e+00,    -1.8411519216274698e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1254284938245924e+01,     1.4013311994354437e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.3224214741633489e+01,     3.1878109475353616e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.2814179823283091e+01,     1.1747987507847200e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.5177851769199417e+00,    -8.7025342004960075e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    8.5587790236358998e+00,     1.4521425735445428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6581746236112380e+01,     6.2479951319851079e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.8010783028980031e+00,     8.4242678939384550e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5452678133688234e+00,    -8.4426644213425188e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    4.0266355259278264e+00,     3.8402044978902377e+04,    -4.1489840926852112e-04,    -6.3520288498971401e-08,     2.8036775178564455e-10, } , 
      {    5.5330151186843509e+01,     3.8406647378581838e+04,    -7.1952391055121756e-04,    -5.6434251735874430e-07,     2.6530459071357137e-09, } , 
      {    3.4237669325436570e+00,    -1.6926450892305773e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.2869530183786715e+00,     1.5794582705871382e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.0171430288856818e+01,     7.0193072277412375e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.5652065540215077e+00,     2.3687041178197898e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    5.8067246235196144e+01,     8.4311582442500276e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -4.5386657784897579e+01,     8.4357740710309463e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.0480956007888782e+01,    -9.3119042500278670e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1347081455224405e+01,     1.6139883471715018e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9470458645168456e+01,     3.5306297716371591e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.4287870528167254e+01,     6.6159672814800961e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9527770698457285e+01,    -4.6714278029864545e+02,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.8472932300412833e+01,     1.7686364411394814e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -7.1696485501633816e+01,     1.5646824391183172e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.3089612186612065e+01,    -2.9054743661736211e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.7390632828198228e+01,     1.2815364421545892e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5496862878946285e+01,     1.0321018745093301e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0888458745012830e+01,     3.9236507413301479e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4778050042314918e+01,     1.3729988301966745e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.2582149594096101e+01,     4.1155972687897854e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -1.2444565667427486e+01,    -8.3985554795707885e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.0798206944936304e+00,     1.5107135424274953e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.9600310878027635e+01,     1.9150936322833237e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.7019453834436653e+01,    -8.3063999849778065e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6774484685165845e+01,     6.5170289639003395e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    4.8254640272426435e+01,     8.6061209501430330e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -4.9775527773982127e+00,     5.6047671087774506e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8516047543869096e+01,     1.7299794995199576e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.2993123737768677e+00,     5.6173863470998294e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.4736868807749389e+01,    -5.9074339117803595e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3671555761691868e+01,     7.2166658894473130e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.2611286756641221e+01,     2.4075547242693530e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.9933580421030396e+01,     7.7206750367802333e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    7.2627397510051379e+00,    -1.6753279193912873e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    2.5099075048745867e+01,     3.9596526999020527e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    3.1613468729493084e+00,    -2.0492160650767887e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2491145045775639e+01,     1.5679168212332659e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4730746707601842e+01,     8.6475905838124563e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.6088785572880411e+01,     1.6092957566813820e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0287335005583316e+01,     4.7865087828500077e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.7987444586026612e+01,     1.8304378804603411e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.3375199006158752e+01,     8.6507038296376686e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.6631009495687934e+01,    -8.2162284856433052e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6833268487606922e+01,     1.4243168904673998e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.3581869162628848e+01,     4.0505454888412419e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8299230509915951e+01,     2.2632864228805320e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -6.3234148847155751e+00,     4.5032564607106124e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.5710192714076969e+00,     1.4527984691045156e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3349503155073312e+00,    -3.6550813713517027e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.5748043452723657e+00,     1.3427168387907557e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5926448554294797e+01,     3.3438728101524095e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    3.5964312235840652e+01,     1.5789530893292160e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1300966905494747e+01,    -2.2149946973983510e+03,     2.7818851428905726e-04,     3.2920788202061928e-07,    -1.5557186213123900e-09, } , 
      {   -7.0821699014105320e+01,     2.5090855720641463e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.6246854562033626e+01,     2.3923962696080129e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    7.7215682900708282e+00,     2.3876893652955358e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    7.8472435911395468e+00,    -9.5694246248846321e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.3115699236198790e+01,     1.2914431178603782e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4661309923547911e+01,     3.3517756209131505e+04,    -8.8238998844022173e-06,     2.7248468133080358e-07,    -1.3002702927357755e-09, } , 
      {    4.9352547078864735e+00,    -5.3384213338567019e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.8660544848257787e+00,     4.0633931929751147e+04,     3.2672883574363378e-04,     8.1023032014387868e-07,    -3.8475298546533887e-09, } , 
      {    4.6762728510351927e+01,     4.0633274664798249e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.3854448455962780e+01,     2.3663381571459766e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.6641328998346722e+01,     2.5090842293540016e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.2494906810781714e+01,     2.3236579174263814e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    5.6664614807441666e+01,     1.4968931579814704e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.5341828260521897e+01,     2.5265650642970817e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    4.8029323506001962e+00,     1.4546018375032481e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.1145313847815842e+01,     1.7531485375788274e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1038850181428138e+01,     8.2860498417827002e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.4793479377206918e+01,    -8.5010290126464151e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    2.2268347929649646e+01,     2.3254612858251137e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.7974221427040256e+01,     1.8714144519631096e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.6592206699830257e+01,     1.5034763466342205e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.5676613902559613e+01,     8.4303376524472278e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.9015674282516809e+01,     2.3355391029347960e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.9755147592661743e+01,     8.5797105242252710e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.3010958082190925e+01,    -8.2872217910557010e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0227671104866246e+01,     4.8998343138402226e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {    4.3961248194324138e+00,     7.7226393844509021e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.0002758734900979e+01,    -6.0109059896698936e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -1.4998379432994420e+02,     8.4307687867143795e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0923745291096957e+01,     4.5585172819754180e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1554485410552548e+01,     9.2501610193361630e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.1026233663750268e+01,     2.2230669730564478e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2591065985521377e+01,     1.6570632324736096e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.2200424187457497e+01,     6.0152973902141991e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1639669794389217e+01,     1.5839603655746254e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.9350518449531098e+00,     4.1761360928004024e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.3460694093610357e+01,    -9.5636367451952101e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.0257063839719699e+01,    -1.4830834161403640e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.2870229355284037e+01,     2.5799338685464591e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    5.4300151677111401e+00,     2.3200720203977151e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    3.2905367181364397e+01,     1.5621220632472761e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.2945956964190394e+01,     2.3976209925620638e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.0439382520746654e+00,    -1.0878507132421701e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8170770963836631e+01,     2.4744228703223096e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.4245228038796881e+01,     2.3976230970028253e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.8004121771223872e+01,     6.6410161016780949e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.3307087924472075e+01,     1.6020438766021281e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.8308909375952656e+00,     8.4092826659261336e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.2491955193391462e-01,    -8.4576496493548402e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.8706774644498537e+01,     2.2181469192022319e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.8644039281176386e+01,     2.5269246877734680e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -9.5408412745239382e+00,     8.6533336619538532e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.2796651764053120e+01,    -8.2135986533271207e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8248624896416668e+01,     2.3974078209554820e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -7.5299898905523532e+00,    -7.9296703502359496e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.4602063005306260e+01,     1.5526997070882200e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    9.2635768666298457e-01,     2.2044897026862476e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    5.5458626972115368e+01,     3.2130110236669490e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.2299305098437365e+00,     9.9107313399045124e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4244970943421293e-01,    -8.8515129313884318e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    3.6993165863877060e+01,     2.2328885507880110e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.3721278947968869e+01,     3.0429978523419140e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {   -3.2447577733367829e+00,     5.3330692336057455e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2302293340145287e+01,     2.5189064784406375e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    5.3941782348828390e+01,     5.7176862284702192e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5863359529099968e+01,    -1.3929043416099657e+03,    -9.2625108217020158e-05,    -1.0766742230080575e-07,     5.0823018190712651e-10, } , 
      {    1.9947260321526034e+01,     1.6957103814615479e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5454964124647388e+01,     2.1082607327843900e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {   -3.8145117056192657e-01,    -1.1863332101455662e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.3273697392896963e+01,     1.5437620527759796e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.8292103928910386e+01,     3.2069625257197018e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    4.3411411011537373e+00,     1.6730518393132272e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.6081786922663987e+01,     7.6961260839195893e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.1600437542789255e+01,     1.5438338586777400e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.3800641002802658e+01,     3.2489424904592615e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8184093592086558e+01,     2.2632854753019201e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -8.4631213069240427e+00,    -1.1872439107561740e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    8.1719763186135044e+00,    -9.6800919217253777e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    4.0522606301571802e+00,    -2.0806414344931818e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2718337183821829e+01,     3.1203401224482732e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.9317496130345276e+01,    -8.4895390248907734e+03,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    3.0774897965072444e+01,     2.4247924884267202e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.1759151989417095e+01,     8.4391473160773585e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5014962478946273e+01,    -8.4277849992036154e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -7.7443874532760475e+00,     4.1102354910176236e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.7667355350289409e+00,     1.6793796771598954e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9725285304363553e+01,    -4.5370378882135765e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.4921427932535650e+01,     2.5063619045026469e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.0352746809501507e+00,    -1.2903909677969898e+03,     3.3591115838248028e-05,     3.6307017839139590e-08,    -1.2014167831575411e-10, } , 
      {    7.2910825524854861e+00,     1.5576541347483984e+04,    -9.4451474075811953e-05,     2.6407122470882869e-08,    -7.9708217311218847e-11, } , 
      {    2.2304457786302194e+01,     4.3166717110231348e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2305815176677628e+01,     1.6904132933596284e+04,    -2.8784992980757622e-04,    -1.9018135334734167e-07,     7.8894699954316989e-10, } , 
      {    9.0500100976691851e+00,     3.7200618315310294e+01,    -1.5980733989351623e-04,    -1.8028145797908495e-07,     7.4851353853863468e-10, } , 
      {    2.6546735332735189e+01,     2.5829991607774293e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.6221087601052298e+00,     1.6112520089342219e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.4177929890851416e+01,     1.8253560694932181e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.2131819636245304e+01,     7.1917714257115131e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.9970556770211829e+01,     6.9384358670035854e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    6.6506492911187713e+00,     8.3286914247251261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1204059117445063e+01,     2.0064059694249947e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.2168113550226661e+01,     8.3976071873538222e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6773573981701713e+00,    -8.9596020337591490e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.1727368076239028e+01,     8.8697963690163670e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4065356059022342e+01,     2.6852912886996204e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.2290086989831199e+01,     1.5851664043265093e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    7.2809990838155754e+00,     7.7948492913394557e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0632586857573990e+01,     2.3784695190194925e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.4435539862299237e+01,     3.0954996697930721e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    2.3793907164413220e+01,     2.6962653851027913e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -6.7751613350133475e+00,    -4.8805449462560100e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.5157486638631519e-01,     8.4318278160654627e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3032163316684874e+01,     8.4351044992155112e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.3065625281070695e+01,     1.4375181368860642e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9111262967619485e+01,     2.3906330669804469e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.0715887283384411e+01,     9.3369020682642022e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2532893348879300e+01,     1.7705933876092018e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.6728710601524416e+01,    -7.6316917341069220e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2365617271928709e+01,     1.4617597339576119e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    6.5825755865726174e+00,    -1.2374369761689798e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.3332573820849021e+01,     6.0058990956490870e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.1671816472449166e+00,     3.7172145213246376e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.2854013701244629e+00,     1.2216702810811000e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.5963568749566452e+01,     2.2026364669998075e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {   -5.1820545321092965e+01,     1.7587808708422293e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.5855495717167827e+01,    -1.0498044814256991e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.2337292806341669e+01,     2.6569632881334925e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.7042074720950776e+01,     2.2264451158067437e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1332782553592864e+01,     2.5676496776180578e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.2470822061295461e+00,     8.4231568763849991e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.0389210142008185e+00,     2.3929819781999104e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.6842670170822174e+01,     9.8800423772928480e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7619110042740200e+01,     3.1173362632474495e+04,    -7.1952391055121756e-04,    -5.6434251735874430e-07,     2.6530459071357137e-09, } , 
      {    1.0475616385350405e+01,    -8.8396050385329290e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    3.0379409940850515e+01,    -7.5933079069244886e+03,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    7.4615564725427861e+00,    -8.3352503803248546e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.0300183321501152e+01,     6.1523939276176106e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.3431213371366514e+01,     1.4027451754018646e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.2349241295147433e+00,    -1.5322421685463337e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2969174417425760e+01,     1.4276950332176959e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.9269631717886226e+00,    -9.4613441921926242e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.2894076906509650e+01,     1.5760064407844384e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3036973440987955e+01,     1.5332041362846216e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.8505839868992460e+01,    -8.3876177813476261e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.3039167866205350e+01,     2.2630712514749579e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    3.8798775221227139e+01,     1.3948378971664690e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.2254576189610901e+01,     9.1104756083511093e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.4085673300169457e+01,     2.3418553852233224e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.1011988669698376e+01,     8.4326902110519695e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -8.3314002193998178e+00,     8.4342421042290043e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.6962716915366840e+01,     2.3419258484149370e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    6.0044817258904173e+00,    -1.7301447653797557e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    2.0671407395076667e+01,     1.6189086083430706e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.1042229890196527e+01,     4.8386956234165475e+04,     3.2672883574363378e-04,     8.1023032014387868e-07,    -3.8475298546533887e-09, } , 
      {   -7.6342799596354666e+01,     7.7133228439795330e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1254174012551074e+01,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0585877819268426e+01,     3.3241935516316498e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3331260278136304e+01,     6.1589528832173364e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.1675746174801649e+00,     8.4437754388959747e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.4003352079631650e+00,     2.5193020716017621e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0682836369871211e+01,     8.4339107835196010e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    7.4270258803420326e+00,    -8.4330215317613729e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.4352999955472928e+01,     3.3057172384432582e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    4.4751531399199749e+01,     6.4806215179891269e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.4145348734166170e+00,     3.2367268679578534e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    8.7775954823997665e+01,     6.7596848607833144e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4053012355812980e+01,     2.6294495079970631e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.1452620797591191e+01,     3.7021163508865422e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.2657689527746506e+01,     1.6479971628310635e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.8052657758401981e+01,     3.1376088019441731e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3388045693130316e+01,    -1.8402327106581406e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2767953090887879e+01,    -9.2068888587040819e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -1.2007171119147113e+01,     2.9311725819386611e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.2458635714065505e+00,    -8.9727562290724291e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -9.6344198114132737e+00,     9.9694674090697372e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1042309981670506e-01,    -4.7565920194185410e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.6454194584736392e+01,     6.8144829401661154e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.2644544863932481e+01,     2.5164143512342871e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2791814942045576e+01,     2.1600510807148016e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3067537022287521e+00,    -4.3635710497255550e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.0513762761211190e+01,    -3.8251264375306801e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2747660678229632e+01,     1.5614458941063229e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5408470242504334e+01,     2.2563024909113832e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {   -1.0427216778771951e+01,     1.5673837413725876e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.8061005150987658e+01,     7.1791521873891343e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -9.6648974873366011e+00,     7.7355181748053146e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0544973221874388e+01,    -7.3964127356063182e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.2760852216077971e+01,     2.0547252163277437e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.1409531214000216e+01,     1.3255621777438428e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.2789882390445616e+01,     3.2185467166855833e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -1.5343268297838804e+01,     3.2788931646930592e+04,     5.4522970760836301e-04,     9.9669451003541732e-07,    -4.7251395801978702e-09, } , 
      {   -1.6130305997899164e+01,     1.6673115333597842e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.1255847300217035e+00,    -1.6744123898378286e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.9973232067373250e+01,     3.9502054057199624e+04,    -5.6721115990986939e-04,    -3.1393140292885782e-07,     1.4667068294606790e-09, } , 
      {   -8.2289664679080907e+00,     1.1652884976356776e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.0943441615411253e+01,     2.4028478199568763e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.1783932386318178e+01,     2.3964730460068800e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    3.8975202815657134e+01,     2.3987710435580091e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.7065912962602159e+00,    -9.5267648089289287e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {   -2.3417071325950030e+00,     1.5267625964605788e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.5794094484756016e+00,    -1.0762050071042164e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    4.4724940755890774e+00,    -8.2408373726185455e+03,    -7.3946041874580393e-04,    -9.4264748086532627e-07,     4.4568436890718527e-09, } , 
      {    2.1902739261551343e+00,     7.7276063115828201e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.8024585407273882e+01,     3.1939122850545944e+04,    -4.1056475673581917e-04,    -2.0131403294392373e-07,     9.3825991705128483e-10, } , 
      {   -1.4268888698568780e+01,     1.5565045729931278e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.8756929190963291e+01,     7.7871096292395705e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.9824246388630744e+00,     1.2896382235248406e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7836759878057357e+01,     1.6721817199262918e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4235675730591034e+01,     2.1727835593698230e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.2478972563543003e+01,     9.1834254696837634e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.7915779344005944e+00,    -2.5044448341265394e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    4.6997823628109092e+00,    -3.0616662313625798e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.3196667136536044e-01,    -8.4264625761616462e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4368905697710215e+01,     1.3379574704275898e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    2.1853499226920000e+01,     9.8074752989693116e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3243234974189271e+01,     4.1531240944030469e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5222084339629413e+01,     3.6419634601522475e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.8487684349355529e+01,     1.8510009499465319e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.2648165298971637e+01,     2.3112626247426346e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.9949945702290151e+01,     1.5666707217021425e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.4535708022582881e+01,     1.6058598936147682e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.2419198179497506e+01,     1.7203457606711490e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.9955711106641065e+01,     2.2254614211490458e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.0964444373973535e+01,     8.4990544931740569e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.9670819943316147e+00,     1.5365841741921422e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.9366689582765892e+00,    -1.5646425495706897e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.4891643888552334e+01,    -5.8243212399824151e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6685212459455862e+01,     9.0171499334503096e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.2657316378471236e+01,     2.8750997146883792e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.4880122260078572e+00,    -1.0133742485996620e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    6.0082386437570303e+00,     1.1605985637654354e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6472429096163623e+01,     1.8256689200125438e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.5028243574154232e+01,     9.0617670291353061e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.9144040264501477e+01,     1.7157609230418631e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -8.3892431812051228e+01,     9.2979657831473996e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.2826414672449076e+00,     3.1326777697882535e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.6366196626974290e+00,     1.7954783028523143e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.2644567740063856e+01,     1.9166684066348884e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.3678134936506003e+00,    -1.1518404371757256e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.2653837000980079e+00,     1.8345929660277425e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -8.3722312419588967e+00,     8.4360807305886192e+03,    -6.8426302367317611e-04,    -6.6521285185039432e-07,     3.1363566658338101e-09, } , 
      {   -1.1628041731488079e+01,    -8.4308515846923547e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    3.8588550560204723e+01,     2.1769303398005984e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.2120969547492241e+01,    -3.0284947035848438e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5311820847113138e+01,     1.0038949668926423e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.3429685873613593e+01,     2.5381516122456760e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.8876001987672888e+00,    -1.8427541465665327e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    3.0752884271347046e+01,     4.0476036681676142e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.9106731442697139e+01,     7.0605862668783957e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.3806529942339871e+00,    -7.8919090973090988e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.5243153226414570e+01,     2.7355674820720902e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.2384542020808288e+01,     2.8559215207662737e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.4105280516345822e+01,    -1.6851199831133381e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6160296557949209e+01,     6.8920352011782397e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.1213262883110211e+01,     1.6839211170500661e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -1.4852103045974918e+01,     2.1451306425526873e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.4124820381459697e+00,    -7.0202706189298697e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    5.5162935248867448e+00,    -8.4142879637183814e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    9.1969528905550497e+00,     2.5875736960755810e+04,    -1.9206388487109000e-04,    -1.4849843052385087e-08,     6.0650191506802891e-11, } , 
      {    2.7489260958994130e+01,     2.9080063348800279e+04,    -2.8685581935446115e-04,    -5.3620393130714676e-08,     2.3993429078110924e-10, } , 
      {    4.2272248943982177e+01,     7.8200790107570556e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1343814699239408e+01,    -7.6010235524291684e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -3.9539799545289807e+00,    -1.0189021540274702e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.1208880088559475e+01,     2.0720719001765458e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.3769692253678912e+01,     1.2846868323597853e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4927980610038629e+01,     3.1599052026491907e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    8.8306371191122484e+00,     2.3450084571705778e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    5.8127111670988512e+01,    -1.2470122692599798e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.0861976336214909e+01,     2.8235016764634638e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.7317033466485448e+01,     3.1202902551605654e+04,    -2.8685581935446115e-04,    -5.3620393130714676e-08,     2.3993429078110924e-10, } , 
      {    1.6646042577642810e+01,     5.4439042492437593e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    8.0827448357422789e+00,     6.5015991100089032e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.5189813443133851e+00,     2.4115905858809499e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    9.9882894939114450e+00,     7.4765719527146248e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    5.1494645917647297e+00,    -6.1592835131478409e+00,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.7314255536961163e+00,    -1.2510424814149937e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.6795237887785358e+01,     2.0962075375626780e+04,    -4.3916856999580938e-04,    -3.0403150756060110e-07,     1.4262733684561437e-09, } , 
      {    1.3394915219600405e+01,     1.3614861319628453e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.5318109561584285e+01,     1.7773114936108970e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -9.9449869365494727e+00,    -3.5901483486619591e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.5488750316026536e+00,    -2.3209139402484925e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.4082673456656689e+01,     1.6872992598003624e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0262975265030299e+01,     1.4783934822701221e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.9992538586409935e+01,     1.6133856710870794e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    9.4555235230673773e-02,    -1.1513199565371717e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.0209853192576865e+01,     3.1189571967816399e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    1.0381628247789925e+00,    -1.2268814587507111e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5408836122216961e+01,     1.6355397073698168e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.4556826453667732e+01,     2.4886215314047884e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.0848079722058657e+01,     7.4270580410640723e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0503568691412153e+00,     1.3641392214870035e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.1468638035797234e+01,     1.7216964117982847e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.1738024558871587e+01,     7.1273218165307953e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.7049956762642950e+01,    -8.0278259496251203e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -3.6362173041070589e-01,     7.3135543682021250e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.2599374186858881e+01,     1.0051751548281627e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1470211367395290e+01,     9.6877801278918396e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.0509565774627088e+01,    -1.4362323236559450e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.5027217919705524e+00,     1.4949391594148401e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.2957129730380089e+01,    -7.7556446609443883e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.9081700223563351e+00,    -8.4592033071623209e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.7006545845770965e+01,    -7.4252555141014109e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.4935665429530587e+01,    -1.2119251256125974e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.1131733279394076e+01,     1.6845770126100386e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.3162032066174234e+01,     3.1254717663095948e+04,    -7.1952391055121756e-04,    -5.6434251735874430e-07,     2.6530459071357137e-09, } , 
      {    7.4214459655309941e+00,     1.4101182293849204e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.5534336742809003e+01,     8.4034980219506579e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2278526253279823e+01,    -8.4634342933303160e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.4681089986833165e+00,     1.5206090022723751e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -2.2780012759602620e+01,     1.8001499755689845e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -4.7483314028814130e+00,    -3.7790342981891708e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.4588885052029198e+01,     1.6624704949339713e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.2663902198811691e+01,     6.9682679241663573e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0949621785657062e+01,    -7.5234712914170441e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.4337206053573328e+00,    -1.6848908107079769e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -7.6399642681302453e+01,     1.6879871319580925e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.8921019748940138e+00,    -7.7069427952793103e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.8370279525619670e+00,     1.8122384023865037e+04,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {    6.5505149897606145e+00,     3.2292292634227189e+04,    -3.9751134229741748e-05,     2.3556127137750133e-07,    -1.1256888861682315e-09, } , 
      {    1.0271374113350992e+01,     7.3181262529073165e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    2.0924912055108884e+01,     1.4088064382649749e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    3.4644400664657120e+01,     2.9636248377866705e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.3759604036064669e+01,     2.4271025640201802e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -1.7339620894412988e-01,    -6.0108511314230645e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.0477307629564070e+00,     2.5928706057967003e+04,    -1.9206388487109000e-04,    -1.4849843052385087e-08,     6.0650191506802891e-11, } , 
      {    2.3972224796348684e+01,     1.4930328675069995e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {   -8.5732995076931928e+00,     1.7137129608703228e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.1227093261984482e+00,     2.4007398851338927e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.4008801748777699e+01,     2.0914535983448590e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.4541492415940368e+01,     1.3307452148296768e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -7.1526417159494891e+00,    -8.3836852218130371e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.5117696777591654e+00,     2.5719156592136282e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.1830352443580551e+01,    -9.1293365976919595e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -2.2703484587203771e+01,     1.6274870205697183e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0993094814936963e+00,     2.2255544771318672e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.2351979296290381e+00,    -1.1877641029483148e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4008627819828474e+00,     1.5792644139117372e+04,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {    3.2187468832034916e+01,     4.9026729071812493e+04,    -4.4084786762443640e-05,     3.7335501582245361e-07,    -1.7835810514338718e-09, } , 
      {    1.9690467899884272e+01,     2.2450537234877836e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    6.2964613866029921e+00,     2.4462541422045197e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    2.4715122516936795e+01,     1.8472122043501222e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -6.9693497992259324e+01,     8.0473840874512607e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.6512785054876211e+00,     1.1259864006663787e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.5891073949472148e+00,     8.4336069449134993e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4269695845245774e+01,     8.4333253703674745e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.1160112175831731e+00,     4.9864489297381024e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4818671189346540e+01,     8.4706780355499195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1562860699817360e+01,    -8.3962542797310543e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -3.0482752929040092e+01,     8.4319322062922747e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.3738563418569271e+01,    -8.4350001089886991e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.5332849403779676e+01,     3.1443618871991210e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.7500146018419144e+01,     1.9420004365688765e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0935171485950676e+01,     1.6141328163889128e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.0200835486420289e+01,     1.0080801097584826e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.1188587619844332e+00,    -1.8661674095296978e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {    8.5686873070133203e+01,     1.5382622627082257e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.3256608486699532e+01,    -9.5792731204212650e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.9000452777028247e+00,     6.5159254365079141e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.9119018156363140e+01,     8.4196749700451037e+03,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -5.2374828645892322e+01,    -8.4472573452358702e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0458252394256885e+01,     8.6621938173933140e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.9918153923029536e+00,    -1.6767587287398605e+04,    -2.4493785885836841e-04,    -3.5807853673069220e-07,     1.6945692595821610e-09, } , 
      {    4.8039964652449868e+01,     1.6777078020527726e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.2969275778480473e+01,    -1.7015493141073544e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.1095165439417816e+01,     1.8001852071647918e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.9028194204711522e+01,     1.4305326789801098e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.9649559944371951e+01,    -1.2457116828082126e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.7645310174579869e+01,     3.0896036971709458e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    2.0036744145434206e+01,    -6.6134764251936022e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    7.8039439180701322e+00,    -2.2381188696495769e+02,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    2.9196405571213134e+01,     2.4591410121670531e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    1.7792242268094899e+01,     3.0631865506257705e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.4538598310654221e+01,     1.3764933190976733e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    5.9695187088204458e+00,     8.3908063416847835e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.6274936422111615e+01,     1.7521096728597004e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6686539931823035e+01,     2.7748695790413891e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.7394875031948631e+00,     8.4761259735961903e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6644957071014627e+01,     4.0349844298452354e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7662791659494360e+01,     1.5962997731780182e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.2250821065306354e+00,     2.3381699920078929e+04,    -1.5847276903284196e-04,     2.1457174786754507e-08,    -5.9491486808951217e-11, } , 
      {    6.1857181015724612e+00,    -1.7424240965117060e+04,    -5.4700339846070225e-05,    -2.0915414890661846e-07,     1.0459806688570128e-09, } , 
      {    5.2423248088413050e+00,    -2.5334179900424424e+04,     2.2299111921642950e-04,     5.1773253005687344e-08,    -2.3523159807434682e-10, } , 
      {    1.2423204965272989e+01,     1.6518826689031928e+04,     5.5188678122641420e-04,     7.9310670092708988e-07,    -3.7538153700949153e-09, } , 
      {    1.1828400163269956e+01,    -1.7910567569134873e+04,    -4.2817784384505619e-04,    -6.4541306111388087e-07,     3.0554897438247395e-09, } , 
      {    1.6582633244192383e+01,    -1.8814866749331024e+03,    -1.2088846106945117e-04,    -1.4520722436827066e-07,     7.3725131672646013e-10, } , 
      {    1.0810551098905121e+01,     4.8974579483015714e+02,     2.4459739845080923e-04,     2.9290086418147971e-07,    -1.4355769429966358e-09, } , 
      {    9.0111819725240512e+00,     1.5053008495353804e+04,    -3.1078551967419018e-04,    -2.2895393964313191e-07,     1.1268475908660832e-09, } , 
      {    4.7688738015327248e+00,    -9.0955607999860549e+03,     9.2284647809460994e-05,     4.2489749751593286e-08,    -2.4923786532160132e-10, } , 
      {    3.2590893900905755e+01,     3.8856891355256957e+04,    -1.6280642156554388e-04,     1.5925091923170681e-07,    -7.1738365207459160e-10, } , 
      {    2.9217498292803587e+01,     4.8928513294496857e+04,     3.5765607008897329e-04,     8.4715373009718093e-07,    -4.0221112612209328e-09, } , 
      {    1.6066064060051691e+01,     1.4985434380753750e+04,    -2.5425881396932824e-04,    -1.5387433550820208e-07,     6.6880532122741591e-10, } , 
      {    1.4596813182240993e+01,     7.8423704505043761e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1992881734107382e+01,     3.2129143834221817e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {   -4.4960002716559195e+00,    -2.6171696138854066e+04,    -6.4234506317708347e-04,    -9.6967099545037180e-07,     4.5909916346348614e-09, } , 
      {    1.2067341694531271e+01,     5.8188915349494191e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    5.3996070654718329e+00,    -9.0955495403919340e+03,     9.7612410795278029e-05,     4.1256965523267953e-08,    -1.4035840881802174e-10, } , 
      {    2.6145816957212391e+00,     7.8669371189697695e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2851284329734632e+01,     1.5410370580190272e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3151745108671371e+01,     8.2817920860227059e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.1715157243079227e+01,     1.9063511533703022e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.2484083083143760e+01,     2.5845522780710180e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.2434664681179797e+00,     8.8172793878453231e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1617292023652066e+01,     9.2386862909485681e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3357636291482828e+01,    -1.6121230758997692e+04,    -2.6436986993639205e-05,    -1.7161434683915356e-07,     8.1695953403767900e-10, } , 
      {   -1.1618259352465889e+01,    -1.0582042509967690e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.2368221237162917e+01,     1.6920908272802967e+02,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    9.4425672109408616e+00,     2.2155287416621370e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1957165984929473e+01,    -1.5911681293166970e+04,    -4.5910507819039569e-04,    -6.8233647106718312e-07,     3.2300711503922835e-09, } , 
      {    1.1127812792700496e+01,     1.6781335776287720e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -8.7719999680323504e+00,    -9.7960005705817166e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -3.8284597771503357e+00,     2.5973588031812185e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -2.7951065721666410e+01,    -6.0601614529822291e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3907569780487552e+01,     2.2868150844157881e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.7792577471369373e+01,     3.1520023918884501e+04,     4.5477142565769374e-04,     8.2013021551213540e-07,    -3.8879633156579240e-09, } , 
      {    3.1202139342789760e+01,     3.1226331729685688e+04,    -7.1952391055121756e-04,    -5.6434251735874430e-07,     2.6530459071357137e-09, } , 
      {    7.8548179501896644e+00,     4.1222095980422324e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {   -1.1073594995629250e+01,     1.0866843036970802e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.8777965945733616e+01,     1.0473822067277813e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.7059626602236300e+01,    -8.2419656351651392e+03,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    2.1355685802066638e+01,     3.3641709135968966e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -7.0511275640676047e+01,     1.0226553184161705e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2112926038917418e+01,     7.7310367619892859e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.5044254030425313e+00,    -8.5181630333835801e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.1436446158492718e+01,     7.8695929224076126e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -7.8941349223050956e+00,     1.5787214434084375e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.1329731799997734e+01,     1.3109395643866921e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.0003453552430093e+01,     2.3971811220655378e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.0459958762515150e+01,     1.5444538512868327e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -3.6410399503599020e+00,     1.3595313642984502e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.9379150146502038e+01,     4.8176252782613847e+04,     3.2672883574363378e-04,     8.1023032014387868e-07,    -3.8475298546533887e-09, } , 
      {    8.3484141771460052e-01,    -1.7894810349833111e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.5168637925346097e+01,     7.0268048393754307e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.2647755839919341e+01,    -5.9144802309418571e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.3683094949932636e+01,    -1.1930949015550959e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.5678593807508641e+00,     1.7468017428423282e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.4938772059166265e+01,     3.2193578184034483e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.0925015196232888e+01,     2.3865997776862812e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.6245270432129189e+01,     5.8676292955904983e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.7350603612035105e+00,     1.3079853739110673e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    9.2727841700669416e+00,     1.4390913853008233e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.0169736805377596e+00,    -2.4760184622727406e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1765041618117372e+01,    -7.8117175281946711e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7310858321500934e+01,     3.1138468201792984e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    3.0191511412881109e+00,     1.3524689820293950e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.8470254070748453e+01,     2.2657472127760422e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.1114601344336403e+00,    -1.0098538263509326e+03,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    1.4975860935527313e+01,     3.3419540431815869e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    2.2423634797296454e+01,     3.9518974738008401e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    7.6131012980412684e+00,     7.8761518780073402e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.3572908085120883e+00,    -8.9907804372736337e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1250169992936771e+01,     7.6328453650391730e+03,     2.1406928696844315e-04,     3.2430156756779078e-07,    -1.5347746702884579e-09, } , 
      {    8.5528079383868914e+00,    -6.2485838407061783e+02,    -1.2631415641885233e-04,    -1.4393080690864552e-07,     6.2909908074454492e-10, } , 
      {    1.8018151385543597e+01,     3.2061605359581405e+04,     4.8560072763944911e-04,     8.5709725869673756e-07,    -4.0618175017038034e-09, } , 
      {    1.7738208668069888e+01,     4.0039058389163656e+04,     1.4625066461343867e-04,     5.2223577041522741e-07,    -2.4328968626806841e-09, } , 
      {    8.4817129916685801e+00,    -1.8120092654600074e+04,     4.3923149881161714e-06,    -1.3464730365455145e-07,     6.4310534799179948e-10, } , 
      {    1.3695553358995618e+01,     4.1991538369509159e+04,     8.1732314429851130e-05,     4.4900534262278565e-07,    -2.1381737777973861e-09, } , 
      {    2.3424035784669961e+01,     3.8924429829897366e+04,    -2.2456295789263873e-04,     8.5360466093802428e-08,    -3.6894805946116813e-10, } , 
      {    1.9381380301062762e+01,     4.0876909810242869e+04,    -2.8908130807622632e-04,     1.2130038301360587e-08,    -7.4224974577870048e-11, } , 
      {    1.5398429965483810e+01,     2.3834185076268615e+04,     2.4080007105283478e-04,     4.9578501471304476e-07,    -2.3539158658911297e-09, } , 
      {    2.4431109796080104e+01,     3.1122769219008387e+04,    -2.8705168408162937e-04,    -5.3533126668114973e-08,     2.4138873182443779e-10, } , 
      {    3.1076086876455374e+01,     3.0041897706879536e+04,    -6.8869460856946219e-04,    -5.2737547417414213e-07,     2.4791917210898342e-09, } , 
      {    2.0211354293014235e+01,     2.1429195559367912e+04,    -9.3856486485670261e-05,     9.4643969347896499e-08,    -3.5494179221391347e-10, } , 
      {    1.8111354617834078e+01,     1.4286120369524411e+04,    -6.3426307366888315e-05,     6.3286899192885265e-08,    -2.5501684440042700e-10, } , 
      {    1.9849989295726679e+01,     2.3456136796669191e+04,    -4.7285761819764155e-04,    -3.4029489216844086e-07,     1.5471422672935619e-09, } , 
      {    1.7506226475612500e+01,     4.0181093760719494e+04,    -2.2475882261980705e-04,     8.5447732556402145e-08,    -3.6749361841783948e-10, } , 
      {    9.4963666809619607e+00,     1.7214667118920821e+04,     4.8746636340641080e-04,     7.1983263990334814e-07,    -3.4598195057332811e-09, } , 
      {    1.4812418795622872e+01,    -1.9525099905080788e+03,     6.1952401054263170e-05,     7.3803186675304669e-08,    -3.4989003365675212e-10, } , 
      {    1.3463570119340677e+01,     4.2133573741064989e+04,    -2.8927717280339459e-04,     1.2217304763960304e-08,    -7.2770533534541446e-11, } , 
      {    1.0164633948063617e+01,     2.5894943381032499e+04,    -1.6389846438224310e-04,     2.2733592246379692e-08,    -1.6764372279086650e-10, } , 
      {    4.1599306045069850e+00,    -1.8680880515573663e+04,    -5.4602407482486091e-05,    -2.0919778213791832e-07,     1.0452534483353484e-09, } , 
      {    1.4132830518444479e+01,     5.0252715699959394e+04,     2.9570366903471014e-04,     7.7335054342187634e-07,    -3.6722212275641800e-09, } , 
      {    6.4410257399337727e+00,     1.5958063208423853e+04,     4.9279412639222780e-04,     7.1859985567502278e-07,    -3.3509400492297017e-09, } , 
      {    3.5787194450162843e+01,     4.6699280556329788e+04,    -3.8406910728676575e-04,    -2.6553245314369285e-08,     1.0651356573976493e-10, } , 
      {    3.0144495633304530e+01,     4.7185607160347601e+04,    -1.0591603287779752e-05,     4.0970566689289304e-07,    -1.9029955092279615e-09, } , 
      {    7.3570550834159203e+00,     1.6620122210809786e+04,     4.5930094291756396e-04,     6.8224920460458341e-07,    -3.2315255914356120e-09, } , 
      {    1.8875271847229186e+01,     4.1228148205385733e+04,     2.0258150559113243e-04,     5.9740264101275710e-07,    -2.8894846912760231e-09, } , 
      {    7.9266625745225312e+00,    -9.3723022982135044e+03,    -7.0863111676404856e-04,    -9.0568043768072411e-07,     4.2829895030259733e-09, } , 
      {    1.0069977990436939e+01,     7.0720575040655822e+03,     1.5507456449784089e-04,     2.4975108908442388e-07,    -1.1326265699449088e-09, } , 
      {    2.0679149917219799e+01,     1.4499143416695581e+04,    -6.2250648734608123e-04,    -5.9132239871248991e-07,     2.7879210732203862e-09, } , 
      {    7.3961776449271301e+00,     1.4200536139325050e+02,    -3.7357543636257011e-04,    -4.3621527897596253e-07,     2.0102362954893911e-09, } , 
      {    2.2366445425417524e+01,     1.4147558579309029e+04,     1.8117109108392094e-04,     3.5618776337436495e-07,    -1.6905937873970628e-09, } , 
      {    5.0759660566415121e+00,    -1.8018821513187737e+04,    -8.8095590957149985e-05,    -2.4554843320835778e-07,     1.1646679061294381e-09, } , 
      {    5.7098796320546770e+00,    -6.7544104437471105e+01,     5.9092654834186390e-05,     7.4506845252067040e-08,    -4.0287532086521329e-10, } , 
      {    6.6454837692859989e+00,    -1.3614386637374846e+03,     2.1652991032554652e-04,     2.5527379565141507e-07,    -1.2080102492206308e-09, } , 
      {    1.1718813227906277e+00,    -8.3624784820251516e+03,    -1.2404939778891725e-04,    -2.1287131236242152e-07,     9.5731794285570074e-10, } , 
      {    1.0150638145471188e+01,     1.6833175268143528e+04,    -9.7213287932304592e-05,     2.7067147816345391e-08,    -1.3342072504134434e-10, } , 
      {    2.5299503048273708e+00,     3.3757047137445625e+01,    -3.0829301981755390e-05,    -3.6967043184602116e-08,     1.7385418604587960e-10, } , 
      {    8.7210508844427636e+01,     7.1275275317580063e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.1819010737054061e+00,     1.3472874708803662e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.7237950214778994e+00,    -3.9705500800325672e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -5.7446387327179934e+01,     7.7929526477753570e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.0151430587329342e+00,    -2.0829974925797533e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2034585572109293e+01,     2.5319576666843568e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.0445019363410472e+01,     3.0339807024084635e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    1.0524678915787488e+01,     2.2874709799757606e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.1725053433126789e+00,    -2.1610842143044920e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -7.7211555535434995e+00,     8.4259931207640802e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.5352056052282377e+00,     8.4514998416278140e+03,    -4.6576215180844699e-04,    -4.7874866195885568e-07,     2.2587469402893278e-09, } , 
      {    2.6047925130067838e+01,     2.3202797746760847e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.9332174323460261e+01,     6.9979544440283817e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    5.8136104626801428e+00,     4.3883597638708852e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.5189006704672590e+00,     4.7511623143855468e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.5956874089817896e+00,     8.7463050340051668e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.1719475311800363e+00,     1.5453979065593083e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.1702044738780268e+01,     9.7702634210598808e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.9913073443111777e+00,     8.4259879910714299e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.3869061361168171e+00,     8.4409443242095440e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.2826318159972885e+01,     7.0877126298600942e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7265647919863969e+01,     2.2615994006325021e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    3.5911455905837988e+00,    -1.3176190694972875e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -5.2261869911214696e+00,     1.2354028005061398e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5408159105526170e+01,     1.7628011752113740e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7133996359731892e+01,     1.5456013241255921e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.8349019465018781e+00,    -1.5010905733595537e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.2893594325095407e+01,    -8.4118906549571093e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.2432295615813539e+01,     3.2320911380874051e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {   -1.3072702765392275e+01,    -1.5421980048944113e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.2117395453198904e+01,     7.6107943683276585e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0084108750599516e+01,     1.6362577657072898e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.3877369298433507e+01,     7.0080828260642702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.4845163478214879e+01,     8.5589443377291718e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.4030327483942493e+00,     1.0413948886654907e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.1884888956314210e+00,     5.1288054035290525e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.9937969679630196e+00,     1.5784599861136241e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2974530065408873e+01,     7.5824911955647667e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.9552320463657384e+00,     4.1228547293400043e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -4.2654016646709110e+01,     7.8327675496732681e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.7266629514525840e+00,    -2.2928651055636692e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0055534989126425e+01,     8.3812084058961718e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.5598383117702692e+00,     8.4857239093848020e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0342280901897805e+01,    -8.2047384978876580e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2486604435238759e+01,     1.0734820108977896e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3068512576761293e+01,     1.6860373359681249e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {   -2.8061109043632776e+01,     7.0236248543526999e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.5214966075521632e+00,    -1.6873491270880699e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.8222035759805820e+01,     2.5390569972255966e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.7463111852339299e-01,     1.6428096338893884e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.9492661630299487e+01,     2.3784704665981048e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.9252402623985599e+01,     3.3308206743300783e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    1.5679695050350489e+01,     1.7096218825837343e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -4.5101887542357684e+00,     1.4404031764207688e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.9017650462954903e+00,     8.5044538332558222e+03,    -2.5209198770297726e-04,    -2.2277120773067824e-07,     9.9775140386023605e-10, } , 
      {    6.8882713243640303e-01,     4.5934279600451155e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2655873882416126e+01,    -9.0393458350079072e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.0625132486652646e+01,     3.1078949624768193e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {    7.6646871612325258e+00,    -5.0435465820807576e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.4152141537818936e+00,     8.4018269684071474e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.9464223783873238e+00,     8.4651053468738264e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.6918682437114541e+00,    -8.6249819394838814e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.4741058322932307e+01,     7.2075039098591033e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.4086051310294962e+00,     4.2004069903521286e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4900760442205899e+01,     2.6034625300817144e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.2639305935159495e+00,    -3.5775291103395789e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.5476263471653184e+01,     7.0478977279621822e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.9492219084112907e+00,     1.3289403204941395e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.1981080723267299e+00,     8.4334661576404869e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.0990105337024589e+00,    -1.6663941805049977e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    8.0872855914813027e+00,     2.1618094629666521e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.3022495941212942e+00,     2.9946786054391647e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    5.5568159710711615e+00,     8.4967445361071659e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.0236196295036839e+00,     8.3701877791738079e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2670410803399799e+00,    -2.5202182695605825e+04,    -2.0967697198032698e-04,    -4.5894887122234223e-07,     2.1778800182802572e-09, } , 
      {    1.7653119858196675e+01,     6.3876805429701653e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.1953746733683072e+01,     6.4809861772949898e+03,     4.9705522655755253e-07,     6.8280480108313485e-08,    -2.7450635438103032e-10, } , 
      {    8.7259027165975240e+00,     7.7375900877919539e+03,    -4.8307077592594708e-06,     6.9513264336638819e-08,    -3.8338581088460996e-10, } , 
      {    1.2854002650960354e+01,     1.5019227067850838e+04,    -2.7985828532885068e-04,    -1.9203052968982966e-07,     9.5226618429853939e-10, } , 
      {    1.5209557223212254e+01,     2.3347918492575962e+04,    -1.2754553468750243e-04,     5.8380584740056761e-08,    -2.3407289337649512e-10, } , 
      {    9.5981921614311716e+00,    -1.8477052474301365e+03,    -1.5181569541479069e-04,    -1.8213063432157294e-07,     9.1183272329400408e-10, } , 
      {    7.1681781523541614e+00,     1.6133855627117131e+04,     9.0955337177226771e-05,     2.4484477463159538e-07,    -1.1116826189209769e-09, } , 
      {    1.5847795630167119e+00,     9.0617681128889690e+03,    -6.6685176449938512e-05,    -4.3335555699656980e-09,    -3.4222997749522167e-11, } , 
      {    7.2039825286842678e+00,     8.3996847301375219e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0389318572758828e+01,     8.4357673574802211e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.3991915440882220e+01,     8.4311649578007527e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.7077500923518838e-01,     7.3307668816384307e+02,    -2.1899792709128676e-04,    -2.5474466999985210e-07,     1.1521160799255122e-09, } , 
      {    3.9403341352686123e+00,     1.7390459537614097e+04,     8.5627574191409736e-05,     2.4607755885992074e-07,    -1.2205620754245567e-09, } , 
      {    2.4570690078503610e+00,    -5.2352722233312147e+02,    -2.1367016410546975e-04,    -2.5597745422817746e-07,     1.2609955364290920e-09, } , 
      {    4.8126235801022617e+00,     7.8051642023920049e+03,    -6.1357413464121477e-05,    -5.5663397982910266e-09,     7.4656458754057421e-11, } , 
      {    1.1996872415430952e+01,     5.8526842220465078e+03,     3.1609367194660642e-06,     6.7664087994150818e-08,    -2.2006662612924056e-10, } , 
      {    1.4309301305934973e+01,     1.4809677602020116e+04,     1.5280980586790581e-04,     3.1869159453819988e-07,    -1.4608454320560648e-09, } , 
      {    3.8103439203219089e+00,     8.3997091105030413e+03,    -3.3191992975274604e-05,     3.2017095500473753e-08,    -1.5363745554361197e-10, } , 
      {    9.6261586338748053e+00,     1.6275830978347802e+04,    -2.8518604831466774e-04,    -1.9079774546150435e-07,     8.4338672779495976e-10, } , 
      {    1.1981713206126706e+01,     2.4604522403072930e+04,    -1.3287329767331944e-04,     5.9613368968382094e-08,    -3.4295234988007475e-10, } , 
      {    1.5035431969718889e+01,     7.1092881325434719e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7565111795464155e+01,     3.1676609917301088e+04,     2.4767215953845825e-05,     3.0879169916994315e-07,    -1.4204119710515295e-09, } , 
      {    1.1081457288849425e+01,     1.6066281512517080e+04,     1.4748204288208880e-04,     3.1992437876652524e-07,    -1.5697248885596446e-09, } , 
      {    9.9248390381615010e+00,     1.6833145257980948e+04,    -9.9779237061628989e-05,     2.7639906699208196e-08,    -1.8858767381479843e-10, } , 
      {    6.6690285486323209e+00,    -3.3787057300025253e+01,     2.8263352852431007e-05,     3.7539802067464924e-08,    -2.2902113481933369e-10, } , 
      {    9.5237327246060612e+00,     2.4462547051842255e+04,     2.4326808781857502e-04,     4.9525588906148180e-07,    -2.2980216965960116e-09, } , 
      {    6.3703481443456225e+00,    -5.9110133693317198e+02,    -1.5714345840060770e-04,    -1.8089785009324763e-07,     8.0295326679042455e-10, } , 
      {    1.5252682904960134e+01,     2.2719616537327482e+04,    -1.2488165319459390e-04,     5.7764192625894100e-08,    -1.7963316512470535e-10, } , 
      {    7.2426375891792727e+00,    -1.0176396672155262e+04,    -3.0412844605613894e-04,    -4.3254174875145938e-07,     2.0981718009690385e-09, } , 
      {    6.2958887075205121e+00,     2.5719150962339223e+04,     2.3794032483275801e-04,     4.9648867328980716e-07,    -2.4069011530995909e-09, } , 
      {   -2.1636513536639473e+01,     1.6759856382525879e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.2897128332708233e+01,     1.4390925112602356e+04,    -2.7719440383594220e-04,    -1.9264692180399234e-07,     1.0067059125503291e-09, } , 
      {    3.2745031738464021e+01,     1.6764458782205347e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9995125804541164e+01,     1.3695049042753823e+04,    -2.1800381663817164e-04,    -1.1818370978322515e-07,     6.0310337116345149e-10, } , 
      {    9.6413178431790509e+00,    -2.4760072026786188e+03,    -1.4915181392188216e-04,    -1.8274702643573562e-07,     9.6627245154579394e-10, } , 
      {    2.2350680376793065e+01,     2.2023740467478947e+04,    -6.5691065996823395e-05,     1.3222740464666129e-07,    -5.8323570651158297e-10, } , 
      {   -2.0654308532471322e+01,     8.4337947901169355e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.7051711632854516e+01,     8.4331375251640384e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    7.9877554806777820e+00,     7.0993305412396467e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3250936294070346e+01,     3.0561981358034791e+04,    -3.4604640655223163e-04,    -1.2808360515148186e-07,     6.4353683216798680e-10, } , 
      {    9.5567650733332918e+00,     1.6728376154862650e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0151443559846096e-01,    -8.8522186470582474e+03,    -3.6598291474681795e-04,    -5.0638856865806388e-07,     2.4473346141041262e-09, } , 
      {   -3.1263295814870888e+00,    -7.5956147365612833e+03,    -3.7131067773263501e-04,    -5.0515578442973852e-07,     2.3384551576005469e-09, } , 
      {    1.4447221603765358e+01,     8.4460853959628657e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.6827770348496447e+00,     8.3658920430404360e+03,    -7.4945892521679825e-06,     7.0129656450801486e-08,    -4.3782553913639972e-10, } , 
      {    1.4337267778378607e+01,     3.2933213827798056e+04,     1.9439452968028803e-05,     3.1002448339826851e-07,    -1.5292914275551093e-09, } , 
      {    9.5830329521269260e+00,     1.6904132933596284e+04,    -2.8784992980757622e-04,    -1.9018135334734167e-07,     7.8894699954316989e-10, } , 
      {    7.2113038341020399e+00,     1.5505553671868649e+04,     9.3619218670135275e-05,     2.4422838251743275e-07,    -1.0572428906691873e-09, } , 
      {    2.0895381721818445e+01,     2.2233289933309665e+04,    -4.9835915719357996e-04,    -3.7849471958136833e-07,     1.8298759098430213e-09, } , 
      {    1.2024838887874584e+01,     2.3976220447824446e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    6.3272224625977440e+00,     3.7200618315310294e+01,    -1.5980733989351623e-04,    -1.8028145797908495e-07,     7.4851353853863468e-10, } , 
      {    9.3495231182509322e+00,     7.1430695600464360e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6850942707996300e+01,     7.1069869327037377e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.7530592444263917e+01,     7.1115893323832061e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1643000556937185e+01,     8.2937465357151395e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.1390069098306661e-01,     1.3613786434123253e+03,    -2.2166180858419529e-04,    -2.5412827788568948e-07,     1.0976763516737226e-09, } , 
      {    1.6767281787455616e+01,     1.4951652953250787e+04,    -2.2333157962398865e-04,    -1.1695092555489981e-07,     4.9422391465987196e-10, } , 
      {    2.5606490866322247e+01,     3.8890672782759917e+04,    -1.9373365591088338e-04,     1.2232750927840456e-07,    -5.4280224550704765e-10, } , 
      {    9.6692843156226829e+00,     1.5647529023099320e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.4352426987682852e+01,     1.4181375646771634e+04,     1.5547368736081434e-04,     3.1807520242403725e-07,    -1.4064057038042752e-09, } , 
      {   -1.5950678305415369e+01,     1.5645227823259585e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.6664855878186874e+01,     2.3138369026745240e+04,     3.0512255650925405e-04,     5.6910270896808630e-07,    -2.6471845097310993e-09, } , 
      {    3.8430844294499366e+01,     1.5649830222939054e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.9920666367716056e+01,     4.0005301342026214e+04,     1.7707996659519407e-04,     5.5920281359982958e-07,    -2.6067510487265640e-09, } , 
      {    2.5001946895982394e+00,    -1.1518291775816037e+03,    -2.1100628261256122e-04,    -2.5659384634234009e-07,     1.3154352646808816e-09, } , 
      {    1.5236542802785861e+01,     8.7151534388183863e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3097188839878200e+01,     1.0512704887343568e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -4.1410892752129499e+01,     1.0442241695728592e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.5829672777301813e+01,     7.1089358165853982e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.2792445007138340e+01,     7.1096404485015464e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7608237477212032e+01,     3.1048307962052608e+04,     2.7431097446754335e-05,     3.0817530705578053e-07,    -1.3659722427997399e-09, } , 
      {    1.5416538812688336e+00,     9.6900700681374510e+03,    -6.9349057942847016e-05,    -3.7171634558030342e-09,    -8.8662726001311964e-11, } , 
      {    1.1719179315265237e+01,     8.7927056998305106e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9122836359707517e+01,     2.3280344377975915e+04,    -7.1018828982640430e-05,     1.3346018887498660e-07,    -6.9211516301516249e-10, } , 
      {    1.2039998097178829e+01,     5.2243822667980248e+03,     5.8248182123745767e-06,     6.7047695879988165e-08,    -1.6562689787745074e-10, } , 
      {    3.8972084535207339e+00,     1.8018761492862577e+04,     8.2963692698501232e-05,     2.4669395097408337e-07,    -1.2750018036763463e-09, } , 
      {   -1.3595130743887498e+01,     2.3973919247984712e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    4.0786402744630834e+01,     2.3978521647664184e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -3.6721903319605104e+01,     1.6761805266407540e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.7830419629811658e+01,     1.6762509898323686e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3076343835111615e+01,     8.5574190844779532e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4134738260935009e+00,    -1.2194032921816543e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2280393610413402e+01,     2.5161836682706074e+04,     5.2533513579719266e-05,     2.7805102112909463e-07,    -1.3749267514898329e-09, } , 
      {    4.8303619112951253e+01,     8.4338184735985615e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.5623025990552648e+01,     8.4331138416824124e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1124582970597302e+01,     1.5437979557268598e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    8.6979362441538939e+00,    -1.0385946137985984e+04,     1.2853964514061755e-04,     7.8180375476570210e-08,    -3.1493981538556564e-10, } , 
      {    7.1563862256835158e+00,    -8.9197927616582983e+03,    -3.0945620904195600e-04,    -4.3130896452313402e-07,     1.9892923444654592e-09, } , 
      {    1.1879287296857960e+01,     3.2791238476567385e+04,     3.9558083845992327e-04,     7.4566700349136822e-07,    -3.4843607742710462e-09, } , 
      {   -1.4547893480700087e+00,    -7.1017685777915176e+01,     1.8550474361662286e-04,     2.1839401892941267e-07,    -1.0327016221314225e-09, } , 
      {    5.4621024347110030e+00,     8.3570249141818185e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.1658984925738096e+00,     1.6728400535228167e+04,     1.1912075766607364e-04,     2.8242820993036017e-07,    -1.3399765332186464e-09, } , 
      {    2.0038251486289042e+01,     1.3066747087505340e+04,    -2.1533993514526311e-04,    -1.1880010189738781e-07,     6.5754309941524136e-10, } , 
      {    9.3217883630890430e+00,     8.3467156329263307e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.3568554251186615e+00,     6.6905356431257114e+03,    -4.3217103597019890e-04,    -4.4244164411971611e-07,     2.1386052619735738e-09, } , 
      {    1.3437011861101325e+01,     2.4394972937242208e+04,     2.9979479352343699e-04,     5.7033549319641166e-07,    -2.7560639662346786e-09, } , 
      {    8.6514432797724119e+00,     3.4047842387064353e+04,     3.9025307547410620e-04,     7.4689978771969358e-07,    -3.5932402307746256e-09, } , 
      {    1.5251853381237089e+01,     1.5613747595596353e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.5510529859357511e+00,     8.4332425005844525e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.0129535464362807e+01,     8.4336898146965214e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.2393806058540942e+01,     2.1395438512230467e+04,    -6.3027184503914891e-05,     1.3161101253249861e-07,    -5.2879597825979310e-10, } , 
      {   -1.9280960049556089e+01,     2.5088547807251009e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.2440043715160698e+01,     1.6745236901556829e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.2418631276836258e+00,     7.9291114994324107e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3789746828710856e+01,     8.4210146381153736e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5100589377707948e+01,     2.5093150206930473e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    9.1842193071711193e+00,    -1.2169541372414199e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.4706234949044966e+01,     3.0352431892204073e+04,     8.6621684644524853e-05,     3.8263851907654766e-07,    -1.7695747841866172e-09, } , 
      {    1.0412196715212698e+01,     7.9471395536226755e+03,    -4.3749879895601596e-04,    -4.4120885989139075e-07,     2.0297258054699945e-09, } , 
      {    4.8870830169273729e+00,    -1.8505088096880390e+04,    -4.5644119669748716e-04,    -6.8295286318134580e-07,     3.2845108786440732e-09, } , 
      {    5.4700922270683439e+00,    -9.1293422274890199e+03,     1.2321188215480054e-04,     7.9413159704895543e-08,    -4.2381927188914527e-10, } , 
      {    4.4276918123198081e+00,     8.5044538332558222e+03,    -2.5209198770297726e-04,    -2.2277120773067824e-07,     9.9775140386023605e-10, } , 
      {   -5.5453343079272877e-01,     8.4672232047779326e+03,    -9.4850596938785377e-05,    -4.1916990868730478e-08,     1.9407091654814723e-10, } , 
      {   -4.0302152911879698e+00,    -8.4174666493160021e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -7.7440363794446143e-01,     8.4494656659649718e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.8598514512205675e+00,     8.4023145624537065e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0023092276984798e+01,     3.1818585268531759e+04,    -3.5137416953804869e-04,    -1.2685082092315655e-07,     5.3465737566440727e-10, } , 
      {    1.7639571232289263e+01,     5.3663576180286964e+03,    -3.7031656727951989e-04,    -3.6859482421311161e-07,     1.7894424488384859e-09, } , 
      {    6.5865897227880836e+00,     7.1430451796809175e+03,    -3.2996128248106349e-05,     3.1929829037874036e-08,    -1.5509189658694057e-10, } , 
      {    1.3141942701863073e+01,     9.1857266695234975e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4801629713251785e+00,     7.7713771450919794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3294061975818224e+01,     2.9933679402786311e+04,    -3.4338252505932315e-04,    -1.2869999726564454e-07,     6.9797656041977667e-10, } , 
      {    1.5295808586708011e+01,     2.2091314582078998e+04,    -1.2221777170168539e-04,     5.7147800511731434e-08,    -1.2519343687291553e-10, } , 
      {    7.2857632709271503e+00,    -1.0804698627403744e+04,    -3.0146456456323046e-04,    -4.3315814086562201e-07,     2.1526115292208286e-09, } , 
      {    1.0668350803654128e+01,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0510165692210425e+01,     6.9515537782030397e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.9407254902039721e+01,     1.6761828949889165e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.8298739185169328e+01,     1.6762486214842062e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.4380393460126484e+01,     3.2304911872549572e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    7.0962564744396825e+00,    -8.4672475851434519e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.6665189600625730e+01,     9.1081744085113751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.0397319871905477e+00,     8.4772369911496462e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5345727390029715e+00,     1.0406338744848446e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.2209303554247839e+00,     1.0548607838223714e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7065962191742312e+01,     1.5508967232883932e+04,    -3.7924768370949947e-05,     1.0148672660581272e-07,    -5.3775048694988620e-10, } , 
      {    9.0245831208842215e+00,     8.2949043674251006e+03,     1.8057610349377928e-04,     2.8795091649735135e-07,    -1.4153602124943682e-09, } , 
      {    3.9834598170164903e+00,     1.6762157582365613e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.8557492618501392e+00,     7.1768622471435228e+03,    -5.8693531971212973e-05,    -6.1827319124536909e-09,     1.2909618700584724e-10, } , 
      {    2.4139433261024830e+00,     1.0477473291536080e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.7077013084348742e-01,     8.3598698282396108e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.0291567532071504e+00,     1.6762868927832489e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.0792205587499630e+00,     1.6761446236898737e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.6769297730643373e+00,     7.1066583002272882e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.6192402888761315e+00,     8.4327548121736108e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.0612274126468328e+00,     8.4341775031073630e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3003344123165725e+01,     7.1119179648596564e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7607407953489055e+01,     2.3942439020321479e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    8.8462627299103662e+00,     8.4798668234658326e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4564658280642558e+01,     9.5787476392164845e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3959686045425546e+01,     9.0279923151830662e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.4897194890024670e+00,     9.0617737426860294e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.0768365411650653e-01,     7.8051585725949435e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.7962045438574147e+01,     4.7219364207485043e+04,    -4.1420905269535129e-05,     3.7273862370829093e-07,    -1.7291413231820819e-09, } , 
      {    1.6810407469203493e+01,     1.4323350998002305e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    1.6692822350630507e+01,     4.1261905252523182e+04,     1.7175220360937706e-04,     5.6043559782815494e-07,    -2.7156305052301433e-09, } , 
      {    1.2940254014456110e+01,     1.3762623157353873e+04,    -2.7453052234303367e-04,    -1.9326331391815499e-07,     1.0611456408021189e-09, } , 
      {    9.5668584063539406e+00,     2.3834245096593775e+04,     2.4593196931148352e-04,     4.9463949694731912e-07,    -2.2435819683442215e-09, } , 
      {   -5.9927825958792278e+00,    -8.3659520633655957e+03,     2.3626909935192144e-06,    -6.8984138685075869e-08,     3.2749164158949159e-10, } , 
      {    1.0150097686629287e+01,     7.2670224868839032e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.8124362230020701e+00,     7.5360905297394238e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2221594410244982e+01,     8.4394996320354348e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.0407347497449209e+00,     6.5585328085100527e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8261213155033914e+01,     8.3659033026345587e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.0584771487239623e+01,     8.4180470466012976e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.3840582627775460e+01,    -8.4488852686796763e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0588663615812065e+01,     6.5869187419203108e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5432359811284083e+01,     7.7375957175890153e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9304616750954040e+01,     1.0250826057771188e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.1889552480154677e+00,    -1.1856218646786899e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0801365193853487e+01,     8.4526538273487131e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.0323343842805755e+01,     8.4212602328209014e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7667537704732897e+01,     2.3489893843806633e+04,    -5.0368692017939691e-04,    -3.7726193535304297e-07,     1.7209964533394415e-09, } , 
      {    1.9165962041455394e+01,     2.2652042422727431e+04,    -6.8354947489731913e-05,     1.3284379676082394e-07,    -6.3767543476337273e-10, } , 
      {    1.1938587524378827e+01,     2.5232824358321410e+04,    -1.3553717916622797e-04,     6.0229761082544761e-08,    -3.9739207813186451e-10, } , 
      {    9.6844435249269285e+00,    -3.1043091579271013e+03,    -1.4648793242897365e-04,    -1.8336341854989827e-07,     1.0207121797975836e-09, } , 
      {    1.2340990055728762e+01,     7.0563190353322807e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4206892295479555e+01,     8.4154172142851130e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.7462702909659747e+01,    -8.4515151009958608e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.9628645117806052e+00,     6.7500485903534491e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.6037992520208526e-01,     7.0694732306455599e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.7306783095950156e+00,     8.0066637604445350e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3143394637242690e+01,     9.6562999002286087e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1650764593050013e+01,     7.1491030344413839e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.0892969369596219e+00,     8.3989733846706458e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2378646849236699e+01,     4.0147276693256885e+04,    -1.9906141889670039e-04,     1.2356029350672987e-07,    -6.5168170201062718e-10, } , 
      {    2.5649616548070124e+01,     3.8262370827511433e+04,    -1.9106977441797485e-04,     1.2171111716424188e-07,    -4.8836251725525778e-10, } , 
      {    2.0938507403566323e+01,     2.1604987978061185e+04,    -4.9569527570067138e-04,    -3.7911111169553096e-07,     1.8843156380948111e-09, } , 
      {    1.4010620901648277e+01,     1.4252363322386967e+04,    -3.2597005385132925e-05,     1.0025394237748739e-07,    -4.2887103044630662e-10, } , 
      {    3.9716678903458447e+00,    -8.2914908064098163e+03,    -3.1212009053486448e-04,    -4.3069257240897140e-07,     1.9348526162136691e-09, } , 
      {   -5.4818841537389886e+00,    -1.5924306161286409e+04,    -5.2362342837398317e-04,    -7.5556689885962494e-07,     3.5247942352755816e-09, } , 
      {   -2.2540401366534386e+00,    -1.7180910071783375e+04,    -5.1829566538816622e-04,    -7.5679968308795030e-07,     3.6336736917791609e-09, } , 
      {    1.4567765340923136e+01,     7.1622572297546631e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5987312590406432e+01,     9.9717686089094732e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0444623053129538e+00,    -8.2576981193127303e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.6389267332936345e+01,     9.6225184727256456e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.2594637392049602e+01,     2.3975868131866373e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    5.6069014157825393e+01,     2.3976572763782518e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -1.4495390852343132e+01,     1.5435678357428864e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    9.2989217796877313e+01,     7.0937461042550422e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2697065354077544e+01,     8.4003960756043980e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.6029185649444628e+01,     7.1248301608319016e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.9886182831491695e+01,     1.5440280757108332e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7966218109019593e+01,     2.4047208123439781e+04,    -3.1828010892635822e-04,    -1.5882428319233044e-07,     6.8902205172968356e-10, } , 
      {    1.1655066329396377e+01,     5.9236718976618413e+03,    -1.8490975602648117e-04,    -1.5015717205239902e-07,     7.5746804722872778e-10, } , 
      {    4.8008316534316160e+00,    -1.7248484186383423e+04,    -4.6176895968330422e-04,    -6.8172007895302044e-07,     3.1756314221404939e-09, } , 
      {    6.8694977480674648e+00,     1.5576541347483984e+04,    -9.4451474075811953e-05,     2.6407122470882869e-08,    -7.9708217311218847e-11, } , 
      {    5.9692418307901844e+00,     7.0383004569281366e+03,     1.8590386647959628e-04,     2.8671813226902599e-07,    -1.3064807559907884e-09, } , 
      {    5.8388753850582731e-02,    -8.2239166918097653e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    6.8283956451677525e+00,    -8.4503868384492671e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.8095573536787619e+01,     9.5011953782043602e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1921583454882871e+01,     2.5057067579587776e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -2.2574652207744556e+01,     8.4285351254845664e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.5255240658043114e+01,     8.4383971897964075e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4282905075821443e+01,     8.9942108876801012e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.3482184125353989e+00,     8.1643981147849672e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1239578092080098e+01,     3.2302610672709838e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.4488785855862524e+01,     8.9504400541709419e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5234350339105085e+01,     1.5647176707141245e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    5.3999548939307303e+01,     1.5647881339057394e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.3141936500633733e+01,     3.2307213072389310e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.5405097404086368e+00,     6.3570276206604613e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2042847625657767e+01,     7.0296583287476478e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1324016804228584e+01,    -8.4177336734928977e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6802758628514063e+01,     1.6774776820687992e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.0852767836816357e+01,     8.3783548221596702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.0132309223305702e+00,     7.2206218210585603e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3244194714620445e+01,     6.4809805474979285e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.8422873284187975e+01,    -1.2197556081397292e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    5.0676340725459710e+01,    -1.2190509762235793e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    3.2743700089980130e+01,     7.1889179363392959e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7409844689435289e+01,     1.0364789578602460e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1710678837602252e+01,    -8.5010290126464151e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    3.0392059447651157e+01,     2.9237803332937772e+04,    -2.8419193786155262e-04,    -5.4236785244877343e-08,     2.9437401903289905e-10, } , 
      {    2.1478390931959417e+01,     3.1609035802701041e+04,     8.1293921658707818e-05,     3.8387130330487302e-07,    -1.8784542406901967e-09, } , 
      {    1.1851722991180001e+01,     1.5613771975961874e+04,    -2.5169286484000379e-04,    -1.5444709439106490e-07,     7.2397227000086990e-10, } , 
      {    7.9118981628160006e+00,    -2.0572547132608579e+03,     2.8085239578196579e-04,     3.2859148990645660e-07,    -1.5012788930606001e-09, } , 
      {    5.2137298936576997e+00,     1.7576240853069635e+02,    -4.0440473834432548e-04,    -4.7318232216056464e-07,     2.1840904815352705e-09, } , 
      {    1.9579194041285206e+00,    -1.6691169906750278e+04,    -2.7636214843026552e-04,    -4.6328242679230791e-07,     2.1436570205307352e-09, } , 
      {    4.7694978983543832e+00,     8.4334661576404869e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.8068205846271034e+01,     8.4491986417880798e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    9.9594309754375878e+00,    -7.2106324150523651e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    8.1789206969637203e+00,     7.2490077544688183e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.7407019021877286e+00,    -1.6795939009868576e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    9.6645415649041766e+00,     1.6755598626765885e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -4.2302814938009327e+01,     8.4610751347188125e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0086118919978860e+01,     7.6316575231666329e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2981302408373724e+01,     7.0317358715313476e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.1826186907368641e+01,     6.4806282315398539e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.3205606962089039e+01,     1.0247353307562662e+02,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.4085953923579924e+00,    -1.1133368851508840e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.4360555688114800e+01,     2.5090496691132666e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    5.0180194953788543e+01,     2.5091201323048812e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.1575349349324892e+01,     7.6600434565768910e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1538466825897082e+01,     8.2069960110009852e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.7694291683431942e+00,     1.6759527750049427e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.3388470949146551e+00,     1.6764787414681799e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.3513185555183252e+01,     7.1096167650199204e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.4192835005463856e+01,     7.1089595000670233e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.1150003212388540e+00,     5.9640066509674743e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.9021020769353754e+01,     8.4152267584259516e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3834751097186823e+01,     7.1868403935555962e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4564793108005592e+01,     8.4732810595383999e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.7302979898851483e+00,     1.6604423228025182e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.8036504875399256e+01,     2.0909111908212646e+04,    -4.3650468850290084e-04,    -3.0464789967476378e-07,     1.4807130967079336e-09, } , 
      {    2.0066217958732675e+01,     3.1190283313283275e+04,    -3.4871028804514016e-04,    -1.2746721303731920e-07,     5.8909710391619704e-10, } , 
      {    2.0321772681271494e+01,     3.2375899548164907e+04,    -1.6596735828500991e-04,     9.1586831237555996e-08,    -4.9731702594535089e-10, } , 
      {    1.4910876818925555e+01,     2.2790604212942813e+04,    -3.1295234594054115e-04,    -1.6005706742065575e-07,     7.9790150823326309e-10, } , 
      {    1.4710407619490411e+01,     7.1802758081588054e+03,    -1.9023751901229817e-04,    -1.4892438782407371e-07,     6.4858859072514825e-10, } , 
      {    1.2968220486899739e+01,     3.1886159383131810e+04,    -4.0790087524291064e-04,    -2.0193042505808638e-07,     9.9269964530307470e-10, } , 
      {    2.2276220939967953e+01,     4.8333992766751340e+04,     3.2939271723654231e-04,     8.0961392802971600e-07,    -3.7930901264015986e-09, } , 
      {    1.9963792049463933e+01,     3.9376999386777730e+04,     1.7974384808810260e-04,     5.5858642148566690e-07,    -2.5523113204747739e-09, } , 
      {    7.2274783798750271e+00,     8.5754415088711576e+03,    -4.4016268044892444e-04,    -4.4059246777722812e-07,     1.9752860772182044e-09, } , 
      {    4.2310351505361830e+00,    -1.1856462450442086e+03,    -1.8530887888945459e-04,    -2.1848128539201239e-07,     1.0312471810880938e-09, } , 
      {    1.6707981559934751e+01,     2.2510067071496760e+04,     3.0778643800216253e-04,     5.6848631685392362e-07,    -2.5927447814793092e-09, } , 
      {    1.1038331607101545e+01,     1.6694583467765562e+04,     1.4481816138918027e-04,     3.2054077088068787e-07,    -1.6241646168114342e-09, } , 
      {    6.3423816719019941e+00,    -1.8714637562711112e+04,    -2.3773105500730694e-05,    -1.7223073895331621e-07,     8.7139926228946876e-10, } , 
      {    1.4635948182665302e+01,     3.3490528107431201e+04,     2.0484626422106753e-04,     5.2846213555898105e-07,    -2.5612658291648673e-09, } , 
      {    3.6136872585382847e+00,    -1.2903909677969898e+03,     3.3591115838248028e-05,     3.6307017839139590e-08,    -1.2014167831575411e-10, } , 
      {    4.3997253398761780e+00,    -9.6190823925221157e+03,    -1.1872163480310021e-04,    -2.1410409659074682e-07,     1.0661973993592805e-09, } , 
      {    7.9821827061356840e+00,     2.5928700428169945e+04,    -1.9472776636399850e-04,    -1.4233450938222424e-08,     6.2104632550130938e-12, } , 
      {    6.2527630257726337e+00,     2.6347452917587703e+04,     2.3527644333984948e-04,     4.9710506540396984e-07,    -2.4613408813513810e-09, } , 
      {    6.3390143892683906e+00,     2.5090849007090739e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -3.1694552632349668e+00,    -6.9673127813128012e+03,    -3.7397455922554348e-04,    -5.0453939231557584e-07,     2.2840154293487568e-09, } , 
      {   -8.1672681634584450e+00,     8.3936512557425740e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1411190496918934e+01,     1.5489794668758888e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.6572705844011821e+00,     1.0740456523154535e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.9707810690883463e+00,     1.0214490059917716e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.8943503670265912e+01,     1.6744884585598757e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.9527230938753746e+01,     9.8942163478973489e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8832223558896146e+01,     1.0757810548295447e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0469385372239080e+02,     8.4329581991024534e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.0143804323286165e+02,    -8.4339741161785223e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.3749018484235345e+00,     1.6919891936706044e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -3.2199412507212948e+01,    -1.2204772968221687e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5955364663096386e+01,     9.3434610238639289e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4494729769831975e+00,     6.8276008513655725e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.8421513589297396e+00,    -9.1293365976919595e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {   -1.7294541760936731e+01,    -8.3713330076675538e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.4038732528543527e+01,     8.4955993076134218e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.7477848168676058e-01,     2.6250908725579279e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4742588883432200e+01,     1.6749039671166160e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.4270199215208041e+00,     9.6900813277315738e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.2535685287777549e+00,     7.1768509875494001e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0143613164853518e+00,    -5.2959621425070964e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.5678371716952260e+01,    -8.2216154919700311e+03,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    4.3155261786613091e+00,     7.6136427907515472e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6030288013355861e+01,    -7.2444138425553292e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.6167722613914812e+01,     8.0530644262698788e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.6806240630244318e+00,     8.4360959899566715e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.1688650545176102e-01,     8.4308363253243042e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0449704171559993e+01,     6.4816918929648054e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1488045067096008e+01,    -1.1789264411481599e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.0082664388706238e+01,     8.4165454768317068e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    3.7573873937982604e+01,     6.4832817473376635e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5232273560725949e+01,     1.1739397123774006e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -3.1138917312196085e+01,     8.5008900366167236e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.1538327084127999e+00,    -1.3771376465220862e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.1825199212162392e+01,     1.5471760984771563e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.5127703216583690e+00,     6.5147676047979539e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    9.4518110466915992e+00,    -1.3855616041832513e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    8.0651102965531951e+00,     2.4010001875327413e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -5.7418435598832778e+00,     1.6722342680467700e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.9410257665347830e+00,     8.5912005119809182e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.2327537655994565e+00,     7.0966688942210931e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6851831421149054e+01,     1.6801972484263526e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    5.5079735614748245e+01,     1.5438331873226673e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.3998491340507279e+01,     1.6622437960440267e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.9689078651847673e+01,     1.5437627241310523e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -2.8703244888373003e+01,    -8.2262178916494995e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.0535513936617189e+01,     6.5606103512937534e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3767091774564193e+01,     2.3818486093484014e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    9.9445363284302211e+00,     1.5096773699065900e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.9295251314578810e-01,     5.5709856812744865e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5868832807873632e+01,     8.8391063656558545e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.6137815247924792e+00,     1.5644899190783137e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    6.1457839128089242e+00,    -3.4944889009986269e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3865826931297292e+01,     1.5650158855415504e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.4571240735492026e+00,     8.2757318033000556e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.4072507764858788e+01,     8.4246060022010242e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.7328318254387970e+01,    -8.4423263130799496e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1489409191306427e+01,     1.5125159632476159e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.6807816980396424e+01,     6.4786793476581952e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    2.0254474870840916e+01,     1.1150831517988436e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5093105701532721e+01,     1.5647200390622869e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -1.2612941053931358e+01,     1.5647857655575768e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.3647869937180346e+01,     4.6104735648218753e+04,    -4.1223452777561258e-04,    -6.4136680613134068e-08,     3.3480748003743437e-10, } , 
      {    3.1292315364928438e+01,     3.7776044223493627e+04,    -5.6454727841696091e-04,    -3.1454779504302050e-07,     1.5211465577124689e-09, } , 
      {    3.2747614019903061e+01,     3.7566494757662898e+04,    -1.3187918722020438e-04,     1.9617432918500909e-07,    -8.9196505864213545e-10, } , 
      {    2.2421772530984576e+01,     3.9518974738008401e+04,    -1.9639753740379189e-04,     1.2294390139256722e-07,    -5.9724197375883742e-10, } , 
      {    2.7061789521296866e+01,     3.8681123316929203e+04,     2.3893443528587307e-04,     6.3304963350643408e-07,    -2.9559138618616521e-09, } , 
      {    2.4749360630792843e+01,     2.9724129936955593e+04,     8.9285566137433357e-05,     3.8202212696238503e-07,    -1.7151350559348275e-09, } , 
      {    1.6724156105707738e+01,     1.5579954908499269e+04,    -2.2599546111689718e-04,    -1.1633453344073715e-07,     4.3978418640808214e-10, } , 
      {    1.5323775059151640e+01,     4.0214850807856936e+04,    -2.5558812460156242e-04,     4.8480689371800036e-08,    -1.9363943237195983e-10, } , 
      {    1.7651363158959914e+01,     3.0420006006804124e+04,     3.0094978939662850e-05,     3.0755891494161785e-07,    -1.3115325145479500e-09, } , 
      {    8.1428935064565540e+00,    -1.6381557815994149e+03,    -5.8448378661154717e-04,    -6.9285275854960253e-07,     3.3249443396486081e-09, } , 
      {    1.4207277563431902e+01,     2.3942463400687000e+04,    -9.9380114198655565e-05,     9.5964020038821550e-08,    -4.6236680767416450e-10, } , 
      {    1.9020410450438771e+01,     3.1467060451470366e+04,     4.5743530715060228e-04,     8.1951382339797272e-07,    -3.8335235874061339e-09, } , 
      {    1.3480137542849203e+01,     2.3766670981993724e+04,     3.0245867501634552e-04,     5.6971910108224898e-07,    -2.7016242379828889e-09, } , 
      {    1.4395552669430730e+01,     1.3553073691523152e+04,     1.5813756885372284e-04,     3.1745881030987457e-07,    -1.3519659755524853e-09, } , 
      {    9.2250523203193637e+00,     2.3905232772209107e+04,     5.7861276565536288e-05,     2.7681823690076927e-07,    -1.2660472949862531e-09, } , 
      {    4.3134739763804211e+00,    -8.3624784820251516e+03,    -1.2404939778891725e-04,    -2.1287131236242152e-07,     9.5731794285570074e-10, } , 
      {    1.0337737278387586e+01,     3.4257391852895067e+04,    -4.2415015722650259e-05,     2.3617766349166399e-07,    -1.1801286144200214e-09, } , 
      {    7.2823959882935485e+00,     3.3000787942398099e+04,    -3.7087252736833238e-05,     2.3494487926333868e-07,    -1.0712491579164416e-09, } , 
      {    2.4850354802939920e+00,     1.7600009003444819e+04,    -3.4704051700534675e-04,    -2.6464456536810883e-07,     1.1925495409300476e-09, } , 
      {    5.7128794973795403e+00,     1.6343405092947853e+04,    -3.4171275401952969e-04,    -2.6587734959643419e-07,     1.3014289974336273e-09, } , 
      {    1.8010211414591719e+00,     1.6795914629503059e+04,     5.7462153702562864e-05,     2.0849412356115595e-07,    -9.9226816112688717e-10, } , 
      {    1.4234841869109861e+01,     4.1119929901292511e+04,     5.4789358910127149e-04,     9.9607811792125464e-07,    -4.6706998519460801e-09, } , 
      {    7.2544295158499175e+00,     1.4877251716620167e+04,     9.6283100163043793e-05,     2.4361199040327007e-07,    -1.0028031624173974e-09, } , 
      {    5.1728699108024925e+00,     1.5774383012655215e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.0829200534951703e+01,     2.3350214062618641e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    7.5439977704084749e+00,     1.6709188485154420e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2127914911036347e+01,     1.4458963481327305e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.0443173554518250e+01,     6.4959831017448550e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.2296120027355990e+01,     2.3425465123791026e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    8.8154767360510569e+00,     8.4203032926484975e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8016807687349568e+00,     5.1805635704169433e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.7404349637024644e+01,     9.7364819935569176e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1043356067085124e+01,     1.5805263377439753e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.8401003410738976e+00,     1.6815126679576806e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -1.7889406775317859e+01,    -8.2232827784304391e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    6.6890969175454975e+00,    -1.7701586162150734e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.4973017393369510e+00,     1.5664449703908100e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -6.4560031241585492e-02,     1.5607714121201407e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.0959985940588645e+01,     1.0287237317590334e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.8242597963887945e+01,     1.6746615554077183e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.1191229722445092e+01,    -8.4208469193181081e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    1.3256047433818912e+01,     1.5594559925888128e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.7044110318037917e+00,     1.6778157090690100e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -7.0850815406695077e+01,     1.6777699610654039e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.2262172413442013e+01,     6.0115802540859595e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2164341234763656e+01,     7.1306180405213236e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4633844816440181e+01,     1.6784699487496669e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    8.5136155331646304e+00,     7.0879582245656202e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3844537756824566e+01,     2.3453851057201286e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    2.6015065530485515e+01,     6.8548566928675682e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -4.3771649839460466e+01,    -8.2242690077678399e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.0746889916012293e+01,    -8.2235643758516908e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.3231483807637128e+00,     8.4864352548516781e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.3545058027047823e+00,    -1.2220331244978388e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -4.6669754467226554e+01,     8.4948565622217775e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0257374934927320e+01,     2.3976549080300898e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    3.7448645364266284e+01,     2.3975891815347994e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0613301369894977e+01,    -1.2167734598654706e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.9371363510019286e+00,     8.9558355482636471e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.2335172300586716e+00,    -1.7417726828048146e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0743465515554012e+00,     8.3804970604292957e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.2557703811881083e+01,     1.5687343924997233e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -7.3157984849423727e+01,     8.4490081859289166e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.8085062643704708e+00,     1.6239788191742453e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.2749315270861645e+00,     1.6211402258332193e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.5838223295812398e+01,     8.4179241293520572e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.7070100703539474e+01,     1.6591305316825725e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.7181677569754669e+01,     6.8946715947654811e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1283955562811956e+01,     1.5646817677632444e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.3552005283501131e+01,     2.3345611662939169e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {   -8.6144367607764480e+01,     7.1091900809696326e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0682392447026935e+02,     7.1093861841173111e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7478991313474033e+01,     1.5648240368566196e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.9067459504084887e+00,     1.6761933925309579e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    1.2484966766068030e+01,     1.6762381239421644e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0935998583785711e+01,     2.3973590615508263e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.1676650397078145e+01,     7.1085767870765949e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.5567612381082810e+01,     1.2031676120379052e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.3537058038738365e+01,     8.9232704626931081e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.6255779617676943e+01,     2.3978850280140628e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.4496764316376796e+00,     8.9842214816739070e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.4285126959401451e+00,     5.9056420596635762e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.1832842138333630e+00,     1.2024826113728732e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    3.7710288558701452e+00,     6.9853352057060038e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.9926366256799586e+00,    -1.2723723893928457e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.5218865006520533e+01,     7.1099994780103489e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.3814487655343333e+01,    -2.7562699845524776e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.6868622961726722e+01,     1.6682527778569787e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.8034255808852205e+01,     1.0707593275509498e+02,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.6880902352000149e+00,     8.3356950062039668e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7890317185092011e+00,    -1.0616689378412225e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -7.3097668388672776e-01,     5.1779647115814987e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5601595774343689e+01,     8.5130959614363110e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -6.2623246743344190e+00,    -1.6746158074041130e+04,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.1676607042849398e+01,     1.1543852487681423e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7979779681659100e+01,     1.6841787386161435e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.3650719649711421e+01,     8.2445802081132751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.3182036371358468e+00,    -1.2592181940795665e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.7168575375027260e-02,     6.4345798816725855e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7591162062435963e+01,     1.7043844863543512e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.7594661598726287e+01,     2.5298097273081723e+04,    -3.9751134229741748e-05,     2.3556127137750133e-07,    -1.1256888861682315e-09, } , 
      {    3.6786871028083198e+01,     2.5302699672761195e+04,    -3.4437663551243822e-04,    -2.6526095748227151e-07,     1.2469892691818374e-09, } , 
      {   -1.9202931409722268e+01,     8.3538363538446629e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.8473955214665516e+01,     1.5021522637893511e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    4.7950597413910296e+00,     5.6949386081119537e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.1246346772576219e+00,     6.5553005694877993e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.8930884421012486e-01,     6.2714412353852128e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    5.2985380117961407e+00,     1.7284526972988773e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.0396520668450204e+00,    -1.2201146376485306e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4212082132728911e+01,    -1.2186919467147779e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.4074129976022718e+01,     1.7121397124555635e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5436991633559101e+01,     1.5700498120310511e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.9299980443087996e+01,    -1.1795883902837420e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.8846372912637349e+01,     1.0129502963249904e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.3880535106985988e+01,     1.5631986994810890e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    1.0374047599725504e+01,     8.4423445460932253e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.8214389764953767e+00,     1.7312912906399029e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.3268655057704059e+01,     1.8440453671118530e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.1660450267580586e+01,     6.6004252531916654e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.5118009902377111e+01,     1.5663071051387751e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -2.1584463923769686e+01,     2.5144929119536300e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.3404146878259745e+01,     2.4133954802164877e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.0026573599830176e+01,    -3.9957992529271496e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.0375738465054622e+00,    -8.4400251132402154e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.2624245395360116e-01,    -4.4598059111805833e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -9.2352051825778787e+00,     8.3200549263416979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.6428958754293781e+00,     8.8690850235494927e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5907551913697148e+01,     1.5016920238214043e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    1.9476160278815096e+00,     7.8827108336070678e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1885985070687747e+01,     8.2944578811820156e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.7385360818314037e+00,     3.8646201409325977e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2393775484778924e+01,     1.0680258287283323e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1085746323999558e+02,    -8.4351971261189137e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3607877453498057e+01,     2.5266617045418498e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    5.3798649907459746e+00,     7.3783561753989925e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.2747688940946006e+01,     6.4813328634560030e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.0760175124223255e+02,     8.4317351891620601e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    8.4974587781809934e+00,     8.3577362596486946e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.1322326354559831e+00,    -8.4465840688399421e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    2.1879589592194315e-01,    -9.7238627552345370e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.0161406817620716e+01,     3.2304559556591495e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    5.8346915094259657e+01,     3.2305264188507645e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    2.8936760792676541e+01,     2.9447352798768501e+04,    -7.1686002905830908e-04,    -5.6495890947290698e-07,     2.7074856353875033e-09, } , 
      {    3.0317600010826048e+01,     5.5548055632210169e+04,     1.1089184537181313e-04,     6.2314973813817735e-07,    -2.9154804008571168e-09, } , 
      {    2.4734201421488599e+01,     4.8475968117982011e+04,    -4.6748668255352150e-05,     3.7397140793661629e-07,    -1.8380207796856616e-09, } , 
      {    2.8005171120322025e+01,     4.6591062252236559e+04,    -3.8757023776626618e-05,     3.7212223159412830e-07,    -1.6747015949302922e-09, } , 
      {    2.3337187657566105e+01,     2.9305377447537827e+04,    -3.4071864356641462e-04,    -1.2931638937980719e-07,     7.5241628867156643e-10, } , 
      {    1.7624412022985020e+01,     2.4118195799055113e+04,    -5.0635080167230550e-04,    -3.7664554323888034e-07,     1.6665567250876519e-09, } , 
      {    1.7710663386480775e+01,     2.2861591888558149e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    2.1521516613707295e+01,     3.0980733847452557e+04,     8.3957803151616335e-05,     3.8325491119071034e-07,    -1.8240145124384071e-09, } , 
      {    2.2436931740288824e+01,     2.0767136556981983e+04,    -6.0363303011006373e-05,     1.3099462041833595e-07,    -4.7435625000800333e-10, } , 
      {    2.0081377168036923e+01,     1.2438445132256858e+04,    -2.1267605365235461e-04,    -1.1941649401155047e-07,     7.1198282766703112e-10, } , 
      {    1.7682696914037141e+01,     4.7380556627802143e+03,    -3.6765268578661141e-04,    -3.6921121632727423e-07,     1.8438821770902756e-09, } , 
      {    1.9094869887263886e+01,     5.1568081521979748e+03,     6.2351523917236607e-05,     1.4212730001491801e-07,    -6.2366916751611817e-10, } , 
      {    1.9421516763994212e+01,     2.3837658657609059e+04,     1.1438798227039829e-04,     3.5189784103569913e-07,    -1.7240895646249208e-09, } , 
      {    1.6735948032378385e+01,     4.0633603297274698e+04,     1.7441608510228556e-04,     5.5981920571399226e-07,    -2.6611907769783536e-09, } , 
      {    1.5792566433353224e+01,     3.2723664361967334e+04,     4.5210754416478521e-04,     8.2074660762629808e-07,    -3.9424030439097141e-09, } , 
      {    1.2083123778926709e+01,     4.5960803115495428e+03,     8.4886997052830875e-06,     6.6431303765825498e-08,    -1.1118716962566095e-10, } , 
      {    2.5315284446754722e+00,    -2.6833779521605516e+04,    -6.0875394733883549e-04,    -9.3336397761123222e-07,     4.4708499563191074e-09, } , 
      {    1.0967239452910039e+01,    -8.0065080276389358e+02,     2.7552463279614873e-04,     3.2982427413478196e-07,    -1.6101583495641799e-09, } , 
      {    5.5132179088162223e+00,    -9.7576441827375020e+03,     1.2587576364770904e-04,     7.8796767590732877e-08,    -3.6937954363735545e-10, } , 
      {    8.7410619259017732e+00,    -1.1014248093234466e+04,     1.3120352663352608e-04,     7.7563983362407543e-08,    -2.6050008713377587e-10, } , 
      {    6.7401207028238304e+00,     1.7461447213229432e+04,    -1.0244311855453749e-04,     2.8256298813370859e-08,    -2.4302740206658822e-10, } , 
      {    1.0054216083405136e+01,     1.4948239392235502e+04,    -9.1787592582903449e-05,     2.5790730356720206e-08,    -2.5268489059429044e-11, } , 
      {    6.7984055938759553e+00,    -1.9186929230454721e+03,     3.6254997331156539e-05,     3.5690625724976930e-08,    -6.5701950063964304e-11, } , 
      {    1.1006997852024313e+01,     4.2376533811789479e+04,     5.4256582611545453e-04,     9.9731090214958000e-07,    -4.7795793084496602e-09, } , 
      {   -8.5702637273094484e-01,     1.9896805986608076e+03,    -2.2432569007710380e-04,    -2.5351188577152680e-07,     1.0432366234219327e-09, } , 
      {    1.4464011734633919e-01,    -9.4805206023067294e+03,    -3.6331903325390947e-04,    -5.0700496077222656e-07,     2.5017743423559163e-09, } , 
      {    8.5214530648257103e+00,     2.5057091959953294e+04,     2.7143350830742188e-04,     5.3283932436024664e-07,    -2.5263156108936811e-09, } , 
      {   -5.7891270221589181e+01,     8.4550416603238646e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3022474083620779e+01,     1.6650823893850524e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1453958629215940e+01,    -6.5589555997275966e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1953746733683072e+01,     6.4809861772949898e+03,     4.9705522655755253e-07,     6.8280480108313485e-08,    -2.7450635438103032e-10, } , 
      {    3.1940031137670214e+01,     8.4357673574802211e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.9259442601420847e+01,     8.4311649578007527e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    8.7259027165975240e+00,     7.7375900877919539e+03,    -4.8307077592594708e-06,     6.9513264336638819e-08,    -3.8338581088460996e-10, } , 
      {    1.5209557223212254e+01,     2.3347918492575962e+04,    -1.2754553468750243e-04,     5.8380584740056761e-08,    -2.3407289337649512e-10, } , 
      {    1.2854002650960354e+01,     1.5019227067850838e+04,    -2.7985828532885068e-04,    -1.9203052968982966e-07,     9.5226618429853939e-10, } , 
      {    9.5981921614311716e+00,    -1.8477052474301365e+03,    -1.5181569541479069e-04,    -1.8213063432157294e-07,     9.1183272329400408e-10, } , 
      {    7.1681781523541614e+00,     1.6133855627117131e+04,     9.0955337177226771e-05,     2.4484477463159538e-07,    -1.1116826189209769e-09, } , 
      {    1.5847795630167119e+00,     9.0617681128889690e+03,    -6.6685176449938512e-05,    -4.3335555699656980e-09,    -3.4222997749522167e-11, } , 
      {    3.9403341352686123e+00,     1.7390459537614097e+04,     8.5627574191409736e-05,     2.4607755885992074e-07,    -1.2205620754245567e-09, } , 
      {    4.8126235801022617e+00,     7.8051642023920049e+03,    -6.1357413464121477e-05,    -5.5663397982910266e-09,     7.4656458754057421e-11, } , 
      {   -7.7077500923518838e-01,     7.3307668816384307e+02,    -2.1899792709128676e-04,    -2.5474466999985210e-07,     1.1521160799255122e-09, } , 
      {    2.4570690078503610e+00,    -5.2352722233312147e+02,    -2.1367016410546975e-04,    -2.5597745422817746e-07,     1.2609955364290920e-09, } , 
      {    1.1996872415430952e+01,     5.8526842220465078e+03,     3.1609367194660642e-06,     6.7664087994150818e-08,    -2.2006662612924056e-10, } , 
      {   -1.6903927239959462e+01,     1.6759856382525879e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.4295619112938994e+01,     1.6764458782205347e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.2215272739433704e+01,     8.4337947901169355e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.8612675882552676e+01,     8.4331375251640384e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.4309301305934973e+01,     1.4809677602020116e+04,     1.5280980586790581e-04,     3.1869159453819988e-07,    -1.4608454320560648e-09, } , 
      {   -1.2118317012823919e+01,     7.1069869327037377e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.9081151333012684e+01,     7.1115893323832061e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.2034509575338404e+01,     1.0512704887343568e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -4.0348183040529136e+01,     1.0442241695728592e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -3.5652204618309128e+01,     1.6761805266407540e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    4.6760656216271471e+01,     1.6762509898323686e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6158972752064280e+01,     8.4460853959628657e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.7384748773471799e+01,     8.4338184735985615e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -3.4704133496870284e+01,     8.4331138416824124e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.1218030581467151e+01,     1.5645227823259585e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    3.9981369284567506e+01,     1.5649830222939054e+04,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -8.8625090194899592e+00,     2.3973919247984712e+04,     2.2103334460937314e-05,     3.0940809128410583e-07,    -1.4748516993033194e-09, } , 
      {    4.2336962886164599e+01,     2.3978521647664184e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    9.0353062780418139e+00,     8.3989733846706458e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9005212659199753e+01,     8.4246060022010242e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.2261023148728935e+01,    -8.4423263130799496e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.5956471116472706e+00,     8.2937465357151395e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.4994841849266898e-01,     8.4327548121736108e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.3473515616116432e+00,     8.4341775031073630e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5406170627760961e+01,     8.4023145624537065e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1151613455417007e+01,     7.1096404485015464e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.0476961645023611e+01,     7.1089358165853982e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.3131138040378225e+01,     8.4394996320354348e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.3009930187774685e+00,     7.1066583002272882e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2375722955268650e+01,     7.1119179648596564e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.0968115027820229e+01,     1.6761828949889165e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.9859659904898894e+01,     1.6762486214842062e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    9.1479187871265509e+01,     7.0937461042550422e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.4518690410343183e+01,     7.1248301608319016e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.4548385574616081e+01,     2.5088547807251009e+04,     3.9291695696701474e-04,     7.4628339560553090e-07,    -3.5388005025228359e-09, } , 
      {    1.2504927350811654e+01,     8.4526538273487131e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2123549498839647e+00,     8.3783548221596702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.8961634365784477e+00,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1922090366831494e+01,     8.3647398411275371e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3296276779431896e+01,     8.3263902248853919e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4290586701908992e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3879936122484008e+01,     8.3931257745377952e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4637661641230405e+01,     8.3117707439163460e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7437374142689555e-01,    -8.2949343775876805e+03,    -1.8314205262310364e-04,    -2.8737815761448853e-07,     1.3601932637209141e-09, } , 
      {    1.7438898832750525e+00,     8.3624484718625718e+03,     1.2148344865959285e-04,     2.1344407124528431e-07,    -1.0124848916291548e-09, } , 
      {   -1.5254898726344738e+01,     1.5540453090344225e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2331952619947117e+01,     1.5545055490023693e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0782296054485398e+01,     1.4967415802349607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5064537766352814e+01,     8.3283391087670516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6058841327148798e+01,     8.3290437406832007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7610443061898014e+01,     7.2117616656190985e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9976399007024867e+01,     7.2163640652985669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.1160341424859723e+00,     1.4756712350797985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4277976747917318e+00,     6.6387243776244804e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6188677209461668e+00,     7.1780144490564217e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6990915314778267e+01,     1.5542401974225884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4068029646506005e+01,     1.5543106606142033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6450670475105742e+00,     7.9356704550321392e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9337861482370094e+01,     7.2137105495007590e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1703831587221785e+01,     7.2144151814169072e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7507598876409189e+00,     7.7533529368907739e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3686940218740032e+01,     1.5149733320490970e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5599118708254686e+01,     8.0001048048448065e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3492863566786202e+00,     8.7217123944181149e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7613997153999410e+00,     6.4280209260728570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.2411922908830633e+00,     8.9040299125594775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7291777356081120e+01,     1.5508997243046513e+04,    -3.5358819241625563e-05,     1.0091396772294991e-07,    -4.8258353817643211e-10, } , 
      {    1.2785067135100222e+01,     1.4392077314515254e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1331398123408510e+01,     6.8210418957658458e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4390746054890098e-01,     7.1496285156461636e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7852745391867515e+00,     1.5576511337321404e+04,    -9.7017423205136323e-05,     2.6979881353745674e-08,    -1.3487516608467297e-10, } , 
      {    5.1822650120227589e+00,     7.2309835462676137e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7950940451948014e+00,     8.5044838434184021e+03,    -2.4952603857365287e-04,    -2.2334396661354104e-07,     1.0529183526336901e-09, } , 
      {    1.4417941931821423e+01,     8.3155735135256709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3976489895040256e+00,     8.2888765228272132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5395314257468259e+01,     8.3685063266230391e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5303048280172916e+01,     8.7861467442307840e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3744726687218707e+00,     8.3294027701920022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6198032432473424e+00,     8.3279800792582500e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9356211562059267e+00,     8.3467156329263307e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0430917597404855e+01,     6.0633858897901282e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7695867457555927e+00,     8.2949099972221611e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.2708093758470172e+01,     3.6048416402410197e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.1997242655589506e+00,    -8.3624728522280911e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0859893751661428e+01,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2354868497410342e+01,     1.5020384899560799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0892554260575764e+01,     1.4954261607036327e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.4275067363617993e+00,     8.1709570703846939e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2421699458235048e+01,     1.5385019935843527e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9334777426095243e+01,     8.3512333298561825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4936214231715883e+01,     7.1803058183213871e+03,    -1.8767156988297380e-04,    -1.4949714670693654e-07,     7.0375553949860234e-10, } , 
      {   -2.0940722266455996e+01,     1.6655081649610518e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.5639482116764514e+00,     8.4864257790655574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5186368008717253e+00,     8.3260615924089434e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4755018082584179e+00,     8.3313212570413107e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0063847267703824e+01,     7.0563285111184005e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6646141141847139e+01,     1.6659684049289986e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.5105310619052208e+00,     8.3816605219363173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4620823251324824e+00,     8.2757223275139349e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0741237055927439e+00,     7.5426494853391505e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.9159430476777253e+00,     9.1147333641110999e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2754871398974387e+01,     7.2140628654588327e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0000471190539171e+01,     6.6916934748356716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0159067719189103e+00,     7.1610937682476415e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1014145751429247e+01,     1.4991998966150539e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.4015993487229741e+00,     7.8063220341019651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3287072702710230e+01,     7.2670319626700239e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8531804765315847e+01,     6.6255701823112013e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1452808830977421e+01,     1.5489785192972768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3917562105952118e+01,     7.7894013532931849e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7615556533124970e+00,     1.5502939388286046e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.8877721035793931e+00,     5.7533848783435178e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4277992849385731e+01,     1.6693431265852661e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1279547936982510e+00,     7.2114330331426490e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5467539724367711e+01,     1.5595723387395150e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3237752431770367e+01,     7.2166926977750172e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7954456322476453e+01,     1.6328796229569934e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7359516811568607e+01,     8.3114184279582696e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1496279303602735e+01,     1.5540124457867776e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5580094604168679e+01,     1.5545384122500143e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3350923927711484e+01,     1.5582569192081872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4365462061200347e+00,     7.1742479635609207e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6270682254496307e+01,     1.6118092778018310e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4297318351737651e+00,     7.2478199125962783e+03,    -2.4933017384648460e-04,    -2.2343123307614075e-07,     1.0514639115903615e-09, } , 
      {    3.0811348476251037e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9817120274518960e+01,     8.3290200572015747e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.6020433141613308e+00,     8.8510608153482863e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6726516491065858e+01,     9.1791677139237690e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9936452388274049e+00,     3.9302096969298748e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2834733399667810e+01,     7.7401987415774947e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0947683224735940e+01,     7.2538777673567447e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9626100799899238e+00,     7.7779361006917070e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6576154337902729e+00,     6.6633075414254135e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4666061358195904e+01,     6.4434349812668870e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2410492188626625e+00,     9.4793684003938306e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8306544056611465e+00,     8.9171841078727575e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5707963267948966e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2292684399107621e+01,     7.3717972197992640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6994506775495708e+01,     1.6640462168641468e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0303623692733517e+00,     8.8794467487585462e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.1164252329534730e-01,     3.3757047137445625e+01,    -3.0829301981755390e-05,    -3.6967043184602116e-08,     1.7385418604587960e-10, } , 
      {    1.3390854516955899e+01,     1.5935775259876948e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2002398639710028e+01,     8.1578391591852396e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1408161065108544e+00,    -1.6920680808781185e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4631166465327730e+01,     1.5700488644524392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5615341074686029e+00,     7.8604193938597496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3223207668554716e+01,     1.4445046411726446e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1811808308250047e+00,    -1.6623625802312807e+04,    -3.3545480326445187e-04,    -5.3778927204437500e-07,     2.5465323413959487e-09, } , 
      {    4.0994446300598781e+00,     1.6691139896587698e+04,     2.7379619930094113e-04,     4.6385518567517073e-07,    -2.1988239693041891e-09, } , 
      {   -1.2899343765531373e+01,     2.3869144515069351e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2967021432895443e+01,     8.9400783289618885e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4687534659874302e+01,     2.3873746914748819e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8148699888004955e+01,     9.5721886836167578e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3288117547077507e+01,     7.1365106044467084e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0728914152997600e+01,     1.4339108217304063e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6576296706514448e+01,     8.5508601288782265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5319955634146247e+01,     8.2490616209293021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2288121138080193e+01,     7.7364322560819937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5636027679021806e+01,     1.5465202029171833e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6318122731930508e+01,     8.4083212285209502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6092563465563181e+01,     1.4916747904978150e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1646118066532711e+00,     1.4128404765752441e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.1547233575846541e+00,     8.3910482317540955e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0979363055637172e+00,     7.2916151264709570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2075551782197977e+01,     1.4994076508934240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5110888736621156e+01,     1.5542754290183959e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.6475383848085974e+00,     1.4363691381104996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1371462822137353e+01,     8.4280182195942161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4583610200279775e+00,     8.2511391637130018e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4576947136565801e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5211772964015637e+01,     1.4912145505298682e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4739003003894426e+01,     8.4177089383387283e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.7548541053063801e+01,     7.2140695790095597e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9914492523701810e+01,     7.2140561519081057e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5466754990993961e+00,     8.4062436857372504e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1483234040026842e+01,     1.5620306551196083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.3136306438390140e+00,     8.5044594630528827e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4207231300689765e+01,     8.2588016467051548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6947197549341084e+01,     1.4914094389180342e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.1978601673482387e+00,     1.5463124486388133e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9712131022162126e+01,     6.6653850842091142e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0800994788447447e+01,     1.7114838168955906e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4701458912121710e+01,     1.6065123680807119e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0866042098919968e+01,     6.1163549870013203e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2223287211908094e+01,     1.7507859138648895e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.5413025224287757e+00,     7.1344330616630086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0144735296299825e+01,     8.3200549263416979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.7761826163938217e-01,     7.2271807766582879e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9571292143467993e+01,     9.9652096533097465e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4460518453773279e+00,     1.4521425735445428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2372561543567198e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1917452020497450e+00,     5.8849268314763094e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3136676301433813e+01,     2.3296107227074735e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.5192992165620169e+01,     1.5542761003734682e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2270037591854717e+01,     1.5542747576633234e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5835404472872643e+00,     1.3763769729469712e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3645980261368614e+01,     1.7900880108341884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8051323729615927e+01,    -8.3226579503301782e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7577536825100736e+01,     1.5288322850508055e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9620849302731614e+01,     8.3347248991200740e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.4009587964328599e+01,     7.7800136434754077e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1075899713986509e+01,     1.4935776613116268e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4094890283314587e+00,     7.2133515199919557e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4092395924968031e+01,     1.4914799021096491e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5955468138634382e+01,     7.2147742109257097e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4307937411967551e+01,     1.5622384093979783e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9740719548151491e+00,     1.5506705873781548e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.6735530757142163e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1921849668718238e+01,     7.2936926692546567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6233595599593418e+01,     1.6721817199262918e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5222183528504587e+01,     6.9596314257829290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8615435802062894e+00,     7.8025555486064641e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3649748561891641e+00,     6.0104167925789370e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1259314520306525e+00,     8.3598698282396108e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2242669135091564e+00,     1.6131246973331588e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2708937697181540e+01,     1.6657030533492180e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.8414357575054353e+01,     1.6657735165408325e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.1922710846286666e+00,     8.1643981147849672e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6973700524284201e+00,     8.5639780400776817e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8852440080765774e+01,     1.5542425657707512e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1775397359533287e+01,     1.5543082922660407e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.2150386830694782e+00,    -1.6899670471006452e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9555033650113074e+01,     8.0742599850492225e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5068531083325610e+01,     1.8293901078034873e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7847489940736185e+00,    -8.3456121055339063e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.2968603418446021e+00,     6.0349999563798701e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2998496259201477e+01,     8.8773692059748446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1033600969975289e+01,     7.6070838351518205e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2510001364978809e+01,    -2.3011998397341813e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1470343296975825e+01,     2.3085403775523107e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9647334197261049e+01,     2.3837688667771639e+04,     1.1695393139972269e-04,     3.5132508215283631e-07,    -1.6689226158514667e-09, } , 
      {    2.3051201984629270e+00,     8.0934048093725705e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1928364611051883e+01,     1.3816738826680903e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9607580002428939e+00,     7.2206218210585603e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.5636521920343505e+00,     8.5831228644010298e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4640963683635881e+01,     2.3871093398951012e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.6429193653637938e+01,     2.3871798030867158e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.9297138380213578e+01,     6.5854029644552147e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8206430434951919e+00,     9.4263993031826394e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8100329638633132e+00,     5.7997133410273145e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2677594039301290e+01,     1.5525833609375179e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2140827017043549e+01,     2.3905202762046531e+04,     5.5295327436211918e-05,     2.7739099578363209e-07,    -1.3212142437597074e-09, } , 
      {    2.0993758226048946e+01,     1.0358230623002733e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2883682858830824e+01,     7.1915209603277763e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.7208443543456315e+00,     6.6070851883911409e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2123393562849998e+00,     8.3500213327029778e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.5262421970020998e+00,     8.2937465357151395e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.7812269822745002e+00,     8.3073615167472744e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2318943590492356e+01,     2.3907494131311494e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.9271555892719725e+00,     8.3286914247251261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6710596703070912e+00,    -1.2904209779595694e+03,     3.1025166708923645e-05,     3.6879776722002399e-08,    -1.7530862708920817e-10, } , 
      {    2.6491019182828069e+01,     1.8686922047727858e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5484756439027159e+00,    -1.9006704986522686e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6042483515164726e+01,     2.3478424745216096e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7980224766438560e+01,     8.9438810985712153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6038572663627653e+00,     7.1960386572576272e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.3155066688515191e+00,    -9.6191124026846956e+03,    -1.2128758393242458e-04,    -2.1353133770788403e-07,     1.0110304505858264e-09, } , 
      {    3.0389663109722940e+00,     7.7217137476574344e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1728014783789760e+01,     6.5861075963713638e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4363122573271093e+01,     8.2353914201973639e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2895999485782518e+01,     1.5073353996771990e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6181701544939493e+00,     9.9007753034970774e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0006306005854189e+00,     1.6264361879757264e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7334903037828997e+01,     1.4880695287798031e+04,    -3.2694937748717053e-05,     1.0029757560878723e-07,    -4.2814380992464229e-10, } , 
      {   -1.4457867180876598e+01,     8.3266138819414264e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.8522754770565353e+00,     7.4684943051347364e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5452178069370042e+01,     8.3307689675088259e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7176703317635265e+01,     7.2313358622256892e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0909816373028374e+00,     6.1927343107203023e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1798850459510568e+00,     9.5077543338040887e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3736416565361203e+01,     6.5880564802530243e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.9935020992113337e+00,     1.1790629090789623e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0186211625490097e+01,    -8.3160721864027473e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.9963643789975457e+01,     1.5578802706586370e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6656067965953112e+00,     6.5497068640108791e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8038989196484451e+01,     8.3413106630475049e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    8.8781602638064356e+00,     8.3640284956606611e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1051252429941956e+00,     1.6082044361615901e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2208208304138871e+01,     1.4915158050605292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4820117929475359e+01,    -1.1149808752243684e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.2474936581027023e+01,    -1.1142762433082185e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8070395734977097e+01,     8.3654511865944132e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0918883302784842e+01,     1.4325954021990785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1624598932714784e+01,     1.4861477607927225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7644059026032402e+00,     7.2478442929617968e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.7567323290577235e+01,     6.5834540805735560e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7913509672959435e+01,     1.9079943017420846e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6496502041235701e+01,     7.2137342329823841e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2637071078784087e+01,     8.8871092317506973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0116157198404787e+00,     1.5478319940371290e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.7048608277303270e+00,     1.7050403819143241e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.4130677161960175e+01,     7.2143914979352812e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8284002209346291e+00,     1.4948209382072921e+04,    -9.4353541712227819e-05,     2.6363489239583007e-08,    -8.0435437832883149e-11, } , 
      {   -2.6757422315523321e+01,     8.3237603925692056e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0101876082968122e+01,    -3.2858661988031849e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.6856451561488104e+00,     1.9651048484649373e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2623941373634411e+00,     1.5720838787719499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9543480224890271e+00,     8.9356691017928169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2314298690196077e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0920872061088573e-01,     1.5797185729859861e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0542389377530565e+01,     6.7446625720468637e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.7632294175902663e-01,     7.3073628699865958e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2026620525744299e+01,     7.0382704467655567e+03,     1.8333791735027191e-04,     2.8729089115188881e-07,    -1.3616477047642427e-09, } , 
      {    8.6549295463796867e+01,     8.3131493964366964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7492475067694766e+01,     8.3227913706387099e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2416177579468531e+01,     1.0751251592695720e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.5555558580194301e+01,     8.3442334530135558e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8051661686739955e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.7457364448959503e+00,     7.2119853226751329e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7696310212301762e+00,     6.7566075459531758e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1111749025520886e+01,     7.2161404082425333e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0595719882377976e+01,     1.7232721337284602e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6090990937988789e+01,     1.6093509614217375e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6406304219972135e+01,     1.5248507948610144e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8739722806904808e+01,     1.5328137752405968e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8617398966624105e+01,     1.7268769753687015e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.7965574409569598e-01,     8.0968018901802789e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.3950136072400277e+00,     1.5540676747400259e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0378266742298319e+00,     8.0132227160442617e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8641242398963342e+00,     6.5864666258801662e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3472238698198460e+01,     1.5544831832967659e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.8342992330072843e+00,    -1.5076495289592810e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.9048183170486190e+00,     7.8592911313131572e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7751343649568817e+01,     8.3336224568810467e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4761808013370661e+01,    -8.3467403680805000e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.3215908211347504e+01,     1.5033891410832151e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8321697027184964e+01,     1.4660015265462513e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5350919128367275e+00,     8.4346296191475085e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.3222446551825513e+00,     7.7627406467085511e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2910319776072763e+00,     1.3970670411412008e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1040259070379923e+01,     1.0293796273190062e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0622160445701914e+00,     8.7980917181370951e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.3726354445950761e+01,     7.2139648138849934e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0609230211270213e+02,     7.2141609170326719e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3464671661426193e+00,     7.3783561753989925e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.6383901270206543e+01,     6.9994463276808419e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4462700114393900e+00,     8.2227532303027438e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5810337222087174e+01,     1.5508972862680992e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -2.3253127561156475e+01,     7.6980826398398494e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0107140858116503e+01,     2.3581258181579246e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4050721291611421e+01,     6.9198165238850170e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5446192891515285e+01,     8.3124820893832220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2297038374181994e+00,     5.4350783047445857e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9097342923842229e-01,     6.5213209306006211e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6776774855824115e+01,     1.6644264938250795e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9335937257861811e+01,     1.9472963987113835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4358630818003091e+01,     1.5564084198161810e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0197672948669897e+01,     8.9546978099309345e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.9573367540504183e+00,     8.6441601334059906e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0726391262506716e+01,     8.1140748869471354e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.0804110672965166e+00,     1.4914446705138416e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7311758919694430e+01,     7.7690759452522452e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1445330241159617e+01,     8.2534403635527378e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3991886889634628e+01,     7.7736783449317118e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2705533555691686e+01,     1.5521424382206107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0045108554902837e+00,     7.1250453518452305e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5369728417960070e+01,     8.3331006518941977e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1921089109396299e+01,     1.5607188639996630e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.2135145688590860e+00,     2.2754515955803057e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.4276856474102363e+01,     2.3935880064721754e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.2979471684189576e+00,     1.7579241869327598e+02,    -4.0183878921500109e-04,    -4.7375508104342746e-07,     2.2392574303087244e-09, } , 
      {    8.6646259734459896e-01,     8.2485093313968173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.8522587100808451e+01,     8.2716035260603567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5204119906456741e+01,    -1.3117911199455193e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.0373331754193615e+01,     2.2759118355482529e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2038728830395733e+01,     7.2353927734366844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4582245175396062e+00,    -8.3095037550169000e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.9736012496627449e+01,     8.5433079625031169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4310990418321945e+01,     8.3478790944333523e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0320377929762170e+01,     7.1927329574809810e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0125528937957975e+01,     8.4088735180534331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5132826176050495e+01,     2.2720768739240382e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0033683772954634e+01,     1.5227285581503094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9350895770629863e+00,    -2.2653055349349970e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3338442749140228e+01,     8.3908245746980610e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1749357456804372e+01,     8.0569869882823677e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4367116569707777e+01,     1.6486530583910364e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2085740823734605e+00,     8.3985212686305003e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.2208746968212498e+00,     6.5327861832020981e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0401357818991919e+00,     1.6617567947552339e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7752669204201219e+01,     1.6697197751348165e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2122269747782720e+01,     8.4066883116163644e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9426082344214016e+01,     9.3369020682642022e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0855769682666477e+01,     6.7051999861070262e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5495051161064470e+00,     7.3849151309987192e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2676159763525696e+01,     1.4598977996457552e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1807815544267532e+01,    -8.5021924741534367e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.6049712331824972e+00,     7.2839526434788049e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5032617395566930e+01,     2.3854525034100301e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.1914839929057894e+00,     1.6799418221006083e+04,    -6.6383985950549209e-05,     6.4034191000947506e-08,    -3.0727491108722393e-10, } , 
      {    1.3346830637271662e+01,     1.4328798936048575e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1529780916787738e+01,     2.7511467878509129e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.9375966257322119e+00,     1.6091432071433679e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4540356396350461e+01,     1.5527212261895529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5233744947101661e+01,     8.9930474261730797e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4998297315083974e+01,     7.7000315237215082e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2173867672645812e+01,     7.1985208371704030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1621996369676134e+01,     8.3110593984494699e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2098501014717806e+00,    -8.2926430083227151e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3936847110141521e+01,     7.7648181894922518e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.1370796393233192e+01,     1.5542656238610120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0844785889231049e+02,     1.5542852341757798e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6963276862305001e+01,     9.0214333595833377e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7298919156384915e+00,     1.6658094194917128e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.9751977401792100e+00,     1.6656671503983376e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1701033113276759e+01,     1.4834264611810107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6126434198949848e+00,     8.5382408905558477e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.7702429415851624e+00,     7.7713771450919794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3838577023009801e+01,     1.1144272562388709e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6089671502681099e+01,     4.5745531950565646e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.9809133304289773e+01,     7.2296048937472624e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.7466643035341775e+01,     1.5558296318472387e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8812659596808608e+01,     2.2181478667808442e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.5515821248279096e+00,     5.4880474019557778e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1105179756809470e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9520991038777282e+01,     1.4699830167360424e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0214885232197335e+02,     8.3281834661870926e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0057933583105935e+02,    -8.3291993832631579e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.0363738881446142e+01,     7.4459524000036799e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5363034252062334e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7897769541535755e+00,     8.9562876643037926e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0322788200292695e+01,     7.1971421846500516e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.0758302338599449e+01,     1.9865984956806824e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7408129401199730e+00,     6.5857552804132893e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2462337182649032e+01,     1.0686817242883051e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1856068466098586e+00,     7.7010951851464597e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.6759535870557567e+00,     6.8985941567779691e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0090543930605804e+00,     1.6833175268143528e+04,    -9.7213287932304592e-05,     2.7067147816345391e-08,    -1.3342072504134434e-10, } , 
      {    3.5251055228764994e+00,     1.4874631803240503e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4013305376900277e+01,     7.4286794032368234e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.9266072443038347e-01,     6.6026759612220712e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3658451723597114e+01,     1.4498015508937637e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4717593574026205e+01,     2.3349076324285925e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5624022217865591e+01,     1.4917076537454601e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7159668383395633e+01,     8.3444239088727172e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.3098838037759108e+00,     7.5083092070326484e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1609972739440831e+01,     8.5359396907161135e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.0020983556179290e+00,    -8.3352503803248546e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.3255469411358245e+01,     1.4957106521094120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0749052796505854e+01,     7.2501112818612437e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.3253290267206115e+01,     2.3282953031761455e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1269695568247869e+01,     1.5576535717686924e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4780347635733037e+01,     2.3713711360568654e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.8571999005700963e+00,     1.5462572196855652e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5177117869021536e+01,     7.8198285453733197e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6593591506789377e+01,     7.1802814379558667e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.9693682283906778e+01,     8.5405420903955819e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.4776671130781814e+01,     1.5072181059478848e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5976002127609927e+01,     6.3313238407373865e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7218865599186852e+00,     1.8024208201205099e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2952141357824003e+01,     3.1441677575438998e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0290878116593143e+01,     1.6675407057651457e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8560579006177246e+01,     5.9972625972656588e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3124945594241932e-01,    -7.6666324223391985e+03,    -1.8580593411601215e-04,    -2.8676176550032585e-07,     1.3057535354691242e-09, } , 
      {    6.4939946058701175e+00,     7.1338807721305247e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6038505837899820e+01,     7.7007361556376572e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3802662911149525e+01,     8.6454796714299973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0126925594990675e+01,     1.6623601421947289e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    5.3649600098555998e+00,    -6.5589555997275966e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.8462676303723899e+00,    -1.6691164276953219e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.1539420634643225e+01,     7.4857673019015920e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1673534175532016e+01,     1.5757370827961951e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3215284213004772e+01,     1.6650823893850524e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.8585169621613691e+01,     2.4983773074335644e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.4223686915428347e+01,     9.5154168167962416e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6923134923180623e+00,    -1.1675976564774849e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0975690135949403e+00,     2.3831630813011172e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7153773590018297e+01,     2.1970775216256814e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8549171355613183e+01,     2.2541905131056410e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3499380670947421e+00,     6.5055731870849813e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4161112711962527e+01,     8.8375543040769335e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8625293822112539e+01,     2.4446784202743434e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.8584907594617235e+00,     7.7003838396795827e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0781524897184829e+01,    -1.6369979498894538e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2941935118516861e+01,     7.9225525438326831e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7817856002661276e+00,     1.6499648495109821e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.3683164071793215e+00,     8.0344450831513104e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2180635745863867e+01,     2.0259005926499809e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9001696514090256e+01,     2.4988375474015109e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5731052132213993e+01,     2.3911260616806998e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.2546295502640259e+00,     6.9918941613057323e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6477387197385230e+01,     8.2092467190313891e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0327304816851743e+01,     6.2702865717324257e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2156718931688268e+01,     1.6168760675389767e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.9517171828046287e+00,     7.7376200979545338e+03,    -2.2647586299350867e-06,     6.8940505453776010e-08,    -3.2821886211115587e-10, } , 
      {    1.3369666592499332e+01,     2.3320690390875669e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5652193209522718e+01,     8.3014614626608582e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3765852957400650e+01,    -8.4738065407431786e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.6633310020142762e+01,     2.5022122690577788e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9312880200212817e+01,    -1.7031212424139244e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.9662454241730760e+00,    -1.0616594620551020e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3484493671572526e+00,    -8.2822907588997823e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.7476687016102012e+01,     8.4481361304188631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6456002459170524e+01,     1.5778040905536514e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8945764065752826e+01,     2.2756464839684719e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7685189674118199e+01,     2.3924414812120278e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0849171131647672e+01,     9.7299230379571891e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2105587116863234e+01,     2.2757169471600864e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3885028077090345e+01,     2.3818476617697892e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.2447494355138495e+01,     1.4365768923888696e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5260913023562765e+01,     1.1537293532081698e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0918187532199676e+01,     8.3750920905504718e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.7867350286165813e+00,     8.3455521714193092e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0309379099402847e+01,     2.4657487654295062e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.9178437135021582e+00,     1.6815117203790684e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3194262384706668e+01,     8.3106424813697522e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0041318607741937e+01,     1.5542042944717083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0899932138501473e+01,     6.6284150963689926e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3860993612730372e+01,     2.3868815882592899e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.0741444507319251e+00,    -1.0808715121288478e+03,    -4.0164292448783282e-04,    -4.7384234750602717e-07,     2.2378029892653959e-09, } , 
      {    2.6252497705100941e+00,     8.6229377662989409e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2956873885228894e+00,     1.0054706888228182e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9240914850857678e+00,    -8.9232363328361625e+03,    -1.8047817113019514e-04,    -2.8799454972865121e-07,     1.4146329919727039e-09, } , 
      {    1.7926251527128549e+01,     2.3874075547225271e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.5994191925688330e+00,     6.0697767706769139e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4374609571711430e+01,     3.5371887272368867e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0842831421615941e+01,    -1.2723629136067254e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6972760194947522e+01,     1.5543465635650835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.8742347724919455e+00,     1.6654753017134070e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.8310703488278861e+00,     1.6660012681766435e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3884687802582922e+01,     1.1079838212576040e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.8559674083967597e+00,     1.6710351946661445e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0663643089867850e+01,     1.5837000631757774e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2270176216963769e+01,     1.7443424788836230e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0366942728592678e+01,     8.9569990097706686e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4280052305465749e+00,     1.5871340910064278e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.7172521882897918e+01,     6.3711387426352985e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5012406047599356e+01,    -8.3129589405775332e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.8063952162429446e+00,     1.6604413752239059e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9011815794957379e+01,     8.7085944832186578e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7865720503542875e+01,     8.3593443470348284e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.9354586282270470e+00,     5.6419789866868832e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4818721673722361e+01,     7.8423704505043761e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5003351121074388e+01,     1.6640109852683396e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2201234243828063e+00,     7.7430436556352861e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4430521482789516e+00,     9.5323374976050218e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0301027505462111e+01,     7.2490077544688183e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8206738627425409e+01,     1.5622936383512266e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7714019435853023e+00,     1.6135013458827092e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5869536916431789e+01,     7.2942449587871397e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1289279899852616e+00,     6.9143419002936080e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2577316684281543e+00,    -1.1146285592662935e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5746497916159299e+01,     2.4264466684602074e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.9147274220794401e+01,     1.6173363075069235e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5331132572426796e+01,     7.7710248291339039e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.2011017234175696e+01,     7.7717294610500539e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7674744978166594e+01,    -8.3441105357643155e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6665223429304252e+01,     8.4706780355499195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1786330572203301e+01,     7.1517060584298642e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2647181340960469e+01,     8.6852945733279103e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3276877601854398e+01,     6.5883851127294747e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9967009227736803e+01,     7.2226993638422609e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2433595274955561e+01,     6.7434896347537206e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5308312448759580e+01,     8.6284123898903508e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1728729370147114e+01,     2.2363796185949803e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0991529149248507e+01,     6.0001075113234492e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0693555290998404e+01,     1.4739645069258337e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4961933024941800e+01,     8.6056647695320844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2856402310531237e+01,    -1.7272996766856068e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.8565684400600372e+01,     7.7229257448106400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1194507637018329e+00,     8.3446909330496110e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2662739581437812e+01,     1.5307467674831403e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0840664706012596e+01,     8.3129436812094864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8733058376939815e+00,     7.6474147424683915e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6979250462431665e+01,     1.6679924754581309e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3602931094048294e+01,     2.0652026896192794e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.3800433013582527e+00,     9.2114304494465723e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1299646464493502e+01,     6.1693240842125115e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7436235625656309e+00,     1.3735383796059454e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4149605365586059e+01,     8.9504400541709419e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.6112356817340832e+00,    -1.1172583915824780e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.9464600084151682e+00,     1.1506769756687036e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1262737612213094e+00,     6.5459403785153772e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5579368304693649e+01,     2.2773737836451575e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.1087065162269084e+01,     8.3286847111744009e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.0092782536422206e+01,     8.3286981382758513e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4043689135305421e+01,    -1.1119987269501098e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.4424036351405078e+00,    -1.6653838832997126e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.1496626491750472e+00,    -8.2611773304502349e+03,    -2.1397135460485901e-04,    -3.2434520079909064e-07,     1.5340474497667937e-09, } , 
      {    4.9899779826583782e+01,     1.5367952654303879e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9734978402214525e+00,     1.2036460728798950e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7030386253197705e+00,     8.3962055190000174e+03,     9.0654146677837468e-05,     1.7647702806068220e-07,    -8.3863070558327521e-10, } , 
      {    1.7230705757221962e+01,     1.7272572523296341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0153268596986361e+01,     8.3265491864555006e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.5837539134760554e+00,    -8.3308336629947516e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2847621623691674e+01,    -8.3418093359245813e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4979339738930836e+01,     6.5520038630729050e+03,    -1.8500768839006530e-04,    -1.5011353882109919e-07,     7.5819526775039211e-10, } , 
      {   -1.0464872870813021e+01,     8.4683768357101853e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5797103102894315e+01,     3.9302096969298746e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7870127724961273e+00,     7.7341465166140897e+03,     1.2414733015250138e-04,     2.1282767913112163e-07,    -9.5804516337736496e-10, } , 
      {   -1.8340448227310915e+01,     1.5423309584490224e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.8579638095655504e+00,     3.1083207380528183e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.2915144374389243e+01,     8.0171720863844548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7746493462562256e+00,     8.9143391938149653e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8644231723305310e+01,     1.7665593492989326e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2728891489682312e+01,     3.1087809780207655e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.8200552628124917e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8514054599411356e+00,     8.4308631336520066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7543005217710167e+01,     7.0392612295787540e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3304157899364064e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.5687883420912989e+00,     1.4425824531077931e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.5040394201101943e+00,     1.1015106480668383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1664351119949297e+01,     9.3500199794636574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2221212480716817e+01,     1.3869707923892094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5839133581636483e+01,     1.5648692484606341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9554889315389516e+00,     1.7179752240073412e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5991190276765389e+01,     1.6149731967251650e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3166954706559295e+01,     1.6657054216973800e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0633652997785230e+00,     6.9853352057060038e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5452276286588607e+00,    -3.3781427502964334e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7461669680636934e+01,     1.6657711481926701e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8901506007555248e+01,     1.5536195334584230e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6458132159613815e+01,     1.3852787243083314e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.0684214839541056e+01,     7.0946181597650957e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6683211218317428e+01,     1.1930314501774683e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5195380291177170e+01,     1.6068890166302621e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.1894719927748149e+01,     8.1538897888450474e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2272302868936496e+01,     1.0122944007650178e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0553565699227327e+01,     9.1716155475486612e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0817765386091075e+01,     2.1606140179974089e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.8017778635626286e+01,     1.4430426930757400e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.0098062322498036e+01,     6.9787762501062771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3146477167080954e+00,     1.6106627525416832e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0022753265489282e+01,     2.2692382805830126e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.4781251030221378e+00,     7.7003894694766441e+03,     1.5497663213425675e-04,     2.4979472231572374e-07,    -1.1318993494232446e-09, } , 
      {    1.3636280519817520e+01,     7.8210405425265244e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6976249791893053e+01,     2.4029180069249516e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0543788787281169e+01,     3.2197835939794481e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.0067123187800512e+01,     1.8058614462682315e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2757249636509751e+01,     8.5574190844779532e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7632246976506174e+01,     1.6171061875229501e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5307041963360811e+01,     1.1472859182269027e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0904548396466190e+01,     8.5034930606052058e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8172325808320018e+00,     9.7430409491566443e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2694447300061007e+01,     8.3283323952163246e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.1700223436947695e+01,     8.3290504542339277e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2702896227839858e+01,     7.5255822037995040e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8427928350134714e+01,     2.3245439329703277e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7043050754975468e+01,     3.2202438339473945e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7511597187241783e+01,     8.5047628919864383e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.8716098062000135e+01,     1.5537823258028038e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2457349940936965e+01,     1.5529636378984504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.5793251232166661e+01,     1.5547685322339883e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8423545861889394e+00,     8.9907504271110556e+03,     1.1881956716668435e-04,     2.1406046335944699e-07,    -1.0669246198809447e-09, } , 
      {    1.9722834910012235e+01,     1.7844108241521368e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5498940677154209e+00,     1.7260154635030553e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5953779533554755e+00,     1.7808059825118955e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.7080772534842680e+01,     8.3279867928089770e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.8075045353750227e+01,     8.3293960566412752e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2580704514425758e+01,    -1.1483856064037391e+03,    -3.3998432052432208e-04,    -3.9990826113682296e-07,     1.8900946171736371e-09, } , 
      {    3.7002147469349396e+00,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6642560816059975e+00,     8.1889718027997787e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.5333153983478258e+01,     2.2723060108505346e+04,    -2.5385969110635476e-04,    -8.5550222168588733e-08,     3.9502618736804976e-10, } , 
      {    3.5025089815084030e+01,     2.1045047865885783e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0994834172698107e+02,     8.3269604562466993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4909513633103252e+01,    -8.2665582747521894e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.2966535269249881e-01,     1.7245875532597882e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.9904281693734287e+00,    -1.1544434611642057e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4237613674987760e+01,     1.6012154583595928e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7219598767764566e+01,     4.3232306666228633e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.1647712444308596e-01,     8.3284677676690917e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.9684677657303610e+01,     8.2317886241624456e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0753843356802744e+00,    -1.1153399047331698e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0579455339343003e+01,    -1.1139172137994171e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.4058969391462117e+00,     1.4836894444126290e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4362639257785386e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1809156497073534e+01,     8.3289150817811606e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0947655852381333e+00,    -8.4684110466504735e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.1489964717277783e+01,     1.8451635432375304e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.2404254963650239e+00,    -8.5551615713646279e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.0450881753125095e+01,     8.4039424858975144e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9350684435683394e+01,     2.4969153593366595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.4428715733098159e+00,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7317527811870317e+00,    -8.3126919164006413e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1734049757972297e+01,     8.8273902472144641e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2789965198629167e+01,    -1.7085226553988696e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1167258946558732e+01,     3.0510170092533568e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.3061153066100914e+01,     8.1937046907429594e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6120742812790599e+01,     1.6051617169535766e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0910387649279420e+01,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2228738406161902e+01,     1.5560803233539333e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8344538153945798e+01,     6.4109536445332105e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2856217989756999e+01,     2.3240836930023808e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.1296480195023072e+01,     1.4427773414959591e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.5174101720254232e+01,     7.3888645013389123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9745219353522970e+01,     1.4428478046875740e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.4728573423887177e+00,     6.6195179573477963e+03,    -2.4666629235357607e-04,    -2.2404762519030341e-07,     1.1059036398421514e-09, } , 
      {    3.4758289996178471e+00,     8.9569933799736082e+03,     1.4964886914843974e-04,     2.5102750654404910e-07,    -1.2407788059268243e-09, } , 
      {    2.9873887803120816e+01,     7.2321118088142066e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8166983455789321e+01,     1.7215800656475825e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1059602282849070e+01,     1.5407767556201792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.2070687861660915e+01,     8.4636781587072928e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0109655963284972e+01,     6.0370774991635708e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5802345778067499e+01,     1.6879551553603353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3380152305190435e+01,     8.3288398417823391e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3858389337433827e+00,     8.3285430076679131e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0709078888363939e+01,     1.3710800632258521e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2110184798319850e+01,     1.4911816872822232e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.7656827090929230e+00,    -7.9694518825351033e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0387540495214679e+01,     1.7208138173483672e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.1516006349892578e+00,     1.5524705346828585e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5995174736630009e+01,     2.3542859095028769e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.8701515159805780e+01,     7.0790761314766660e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1221931907884066e+00,     1.4205957026764565e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0635254277731923e+00,     1.5208693046712231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4549992023117788e+00,     2.5019831321312828e+04,     4.2610894994228935e-04,     7.1426630010505710e-07,    -3.3851630469792241e-09, } , 
      {   -4.5367355776098295e+00,    -2.4952317227037933e+04,    -4.8776755390580009e-04,    -7.8820038647426142e-07,     3.7328714190709830e-09, } , 
      {    9.2313171624509479e+01,     1.4914439991587689e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0516228536145680e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1107675280270408e+01,     1.4307469028070724e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0580045897014028e+00,    -8.0470041435472267e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7826656577104572e+01,     2.2790574202780233e+04,    -3.1551829506986555e-04,    -1.5948430853779295e-07,     7.4273455945980900e-10, } , 
      {    7.3529900617835953e+00,     9.0371811030989775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3103758908662039e+01,     2.2667799642029193e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.8223420851501420e+00,     8.3132723136859368e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6207057750970151e+00,     7.6202017463512748e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4799181915331175e+01,     1.3642083791531692e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2537213625826588e+01,     7.4493494808113883e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8611186140812057e+00,    -1.2903965975940507e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.5476822745680522e+01,     1.5662198995877696e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2912736334520375e+01,     1.8844656402068293e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.2352592900728041e+00,     1.4834816901342590e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.8360228013652911e+01,     8.9397260130038121e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3695342080102336e+01,     1.0515964977343167e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3301251405897885e+00,     9.0099681069818598e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8105496841798363e+01,     1.2323335471467670e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6532857391219800e+00,     8.3783548221596702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.4003377106565598e+00,     9.3734302059714482e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4366015501418943e+01,     1.5073706312730064e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3080601995951746e+01,     7.3335075711525697e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7241802357166991e+00,     6.8800016219871050e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5910353109084317e+01,     8.6103787059030255e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.3351523163269388e+00,     8.6879309669151498e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.2835140979845278e+01,     1.5717555926064038e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7985309717541668e+01,     2.3793893453896959e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1628308949881813e+01,    -1.0748136573683812e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2895570637862257e+01,     7.1118911565319522e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8525550810264597e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8642084779574891e+01,     4.7162516363158493e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4638579323560768e+00,    -1.1080696036665659e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0344182797057386e+00,     1.0447387812463210e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6729402823097104e+01,     1.1865880151962010e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1612175227655968e+01,     8.0338862323477715e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4260832216685156e+01,     7.6834631588708016e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4973301711346998e+01,     8.5662380094277087e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2908483720469953e+01,     8.4292412282353416e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5341726930824725e+01,     1.6251243968557810e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3640027478598991e+01,     1.5214167670303641e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.6962527209185311e+01,     8.9795409149017250e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5194981142961250e+00,    -9.6190880223191762e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.6447281049284186e+01,     2.1438068835578768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2230568346027928e+01,     8.9220541207606839e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.8847464027762797e+00,     1.3577649441719022e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6867524400401619e+00,     9.5426467788605096e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1071971821581762e+01,     7.2091318333029121e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1500750119886298e+00,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    6.8277978764884972e+00,    -9.5689420492586169e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1696457761975365e+00,     7.1147360705897427e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2219947931266443e+01,     8.8341401345395061e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.8039224067766479e+00,    -1.1921808202784177e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2077124045200417e+01,     8.0776570658569317e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4506703214836186e+01,     7.8400692506646410e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5950571139283483e+01,     8.9573513257287450e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3280246736349504e+01,     8.3544305598720493e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3806323911831363e+01,     9.4624477195850486e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6824085704388740e+01,    -1.1169297591060276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9508273666757752e+01,     3.0299466640981940e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0383283740524440e+01,     7.6136427907515472e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1519088053859516e+01,     2.2457096190477565e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2367818584974509e+01,     1.0107675985439373e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6334828816992484e+01,     7.3490495994409994e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2708069105462890e+01,     7.2784972152715018e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.9373060754405138e+01,     1.4035104761224677e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.4226418008992027e+01,     8.2335195926408724e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2820886195517978e+01,     8.3162399052000128e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.3882181829144917e+00,    -1.1326527674674985e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1077973478303464e+01,     2.2216870415310086e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.9681885552380738e+00,    -8.3411429442502395e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.9403539941504953e+00,     7.6605689377816707e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5902346788872606e+01,     1.4288440319932606e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.8531102596559975e+01,     1.0973555624994374e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6550075780958508e+01,     7.2075039098591033e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    6.6050444808266295e+00,     3.1546870868086353e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4335319087524486e+01,     1.9237677371761281e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0002824835315526e+01,     8.3996847301375219e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2961944443036543e+01,     1.6577753045654426e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.5553706348730607e+00,    -1.0808471317633291e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    9.6219714566931351e+00,     1.3500097180706898e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.3234323907487322e+01,     8.4238632568093799e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7027041010113305e+01,     2.4393815105532245e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1021603843122177e+01,     8.8974804327458714e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0459191600050129e+01,     9.1016154529116466e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1717852454441612e+01,     9.1318006456507483e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0385633901756464e+01,     2.2234447765019631e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.8476614834660801e-01,     1.5536720815789011e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.8676459625296911e+01,     1.6737012653246078e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4439119477194055e+01,     2.3322767933659365e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.0783197166304369e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.5149862140834273e+01,     1.4914453418689140e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3931356507908077e+01,     6.1138690423676201e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.2240248250775387e+00,     7.0382948271310752e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.4226989861169656e+01,     8.3151849134537715e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2008419084245077e+01,     6.7450148880049383e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5221533848078742e+01,     8.3421979359964807e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6455227552757874e+00,     1.3188431241635359e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4417334023490668e+01,    -1.1146352728170200e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.6762518956857846e+01,    -1.1146218457155669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6571309422685367e+01,     1.6587493071430279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5602575399979379e+01,     9.3434610238639289e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4107059020743449e+01,     1.6137299198294995e+04,    -3.8022700734534074e-05,     1.0153035983711257e-07,    -5.3702326642822187e-10, } , 
      {    2.0457616118203529e+01,     2.2599082801302360e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.8949152900929747e+01,     2.2168324472495162e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.3437622115242185e+01,     7.2189938976147532e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.9957639230862412e+01,     6.5857485668625623e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3840517479277233e+01,     7.3200010598812150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6521005999718859e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1019023013066416e+02,     1.5542246331645923e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.8311317753726459e+02,     1.5543262248721994e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3762505130885096e+01,     1.4405583825786609e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1380344125850836e+00,     1.3893118150399885e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.6122838052711778e+01,     8.5658498676341733e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0585267528531954e+00,     6.5082030194011659e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3995377512681259e+01,     1.5677741024166125e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.0484438565066313e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0175164818665507e+01,     8.4733078678661041e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2219858720576212e+01,     1.5447582458099705e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7661819789957065e+01,     1.5548787764578905e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -7.3704726706007548e+00,     6.2356807028968433e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.4209454240423796e+00,    -5.3929007143194167e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0064612058653104e+01,     5.1092726060088371e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6593995617256756e+01,     3.1085156264409841e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.4464962707806009e+01,     3.1085860896325990e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.0141066110807117e+01,    -8.3277250222774474e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    6.1854866450112880e+01,     1.4779459971156250e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7467092071162437e+01,     2.3871445714909085e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.4595834607364491e+01,     2.3242785813905470e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0738323727671590e+01,     1.3241400338846550e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9859593110603768e+01,     7.1188910333745789e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4301997643864190e+00,     8.5719979376932934e+03,    -3.1118464253716361e-04,    -2.9727805298274525e-07,     1.4006267247254494e-09, } , 
      {    1.1314151653287437e+01,     8.8199281717337471e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.8835406733173812e+01,     1.4898904676849985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.7650254043254305e+00,     7.1081246710364503e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2468487688520547e+01,     2.3279186546265952e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.4515984387677783e+00,     7.2096536382897621e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3860563618560098e+01,     2.3948997975921211e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4811610812983112e+01,     1.6170350529762625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3676968373606098e+00,    -2.2123364377238058e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.9429447904744443e+01,     8.8999111111059010e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8517226508299967e+00,    -1.0370762982541692e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.8511461350724083e+00,     2.3791815911113259e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.4922979649281572e+01,     8.9566466938125941e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5663451130157746e+00,     7.7030136719957682e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6806873707844154e+00,    -7.2160758957330597e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5118301847551880e+01,     1.0908985947036155e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5388967204068095e+01,     8.2733344945387835e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.6713941706646764e+01,     8.3044185511156447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0418608625951844e+01,     1.5436816095761576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7708979348846171e+01,     8.3529642983346093e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7003616981237570e+01,     1.5858222998864821e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.8334273359113574e+01,     7.2711507641236012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8934494847900261e+01,     1.2434972588916312e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.6791612897417920e+01,     7.8446716503441094e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.4396199120164653e+01,     8.3840483549114670e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9484878537980251e+01,     7.5481241089305604e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3186942215662427e+01,     1.4285786804134796e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9091235166062908e+00,     7.2302722008007368e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9527788423447628e+01,     1.2716356441160659e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4674529398039507e+01,     3.2236185556036620e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.7969065845516354e+00,     2.2850117160170554e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.9124224125068228e+00,     8.2757318033000556e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.1249969727789422e+00,     8.3003054913148680e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1099470999901754e+01,    -1.1484099867692596e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    4.4373063603930447e+00,     6.6207001694232749e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.7869481473798913e+01,     2.1831089805271757e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.8692723275401617e+00,     8.3570773581353842e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5757941264573084e+01,     1.9630698341454263e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2157424082669230e+01,     9.9326794869089298e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8437301024893682e+01,     9.4144543292763265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.5077759164540581e+00,     7.1960139221034588e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.2271896582497011e+00,     8.7992646554302373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3771982515301122e+01,     8.2058325494939636e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1498575890560812e+00,     7.7184080478807882e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.8221476454815022e+00,     1.6579830588438130e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2883741511028667e+01,     1.6204813292569888e+04,    -9.9681304698044827e-05,     2.7596273467908341e-08,    -1.8931489433646278e-10, } , 
      {    4.1391177268556227e+01,     8.3178747133654051e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.6625774142438763e+00,     7.1787257945232986e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4084134235870756e+01,     3.0692487610674929e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.0491622327790122e+01,    -1.6633063405160119e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0965288078679327e+00,    -2.2936914683452560e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.0355870415214348e+00,     8.1114384933598958e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.5663167510913823e+01,     7.1828844619443480e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3555528118666896e+01,     2.2809786252853984e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8151751473969991e+01,     1.2258901121654999e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3286050603246777e+00,     7.2199629195452499e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8544103988974499e+00,     5.5410164991669690e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2802519329365314e+00,     8.9017513613601565e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1127944261750169e+01,     8.9596288420868550e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9405233026763955e+01,     2.2703848058431598e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.7631977397966843e+01,     8.1694318171334780e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1208422625625118e+01,     2.3871117082432636e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1036929524802169e+01,     1.5542395260675157e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3955755467129691e+00,     5.4066923713343276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6521129402696069e+00,     1.3135462144424167e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3960681299203273e+01,     1.5543113319692760e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.4203069716604872e+00,     2.3871774347385533e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.4185213548493358e+00,     5.8564776908983958e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7128107795795877e+00,    -6.9889778019972539e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7526437525499015e+00,     7.1773031035895456e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.7494779013621091e+01,     7.3092346975430864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2599022158017878e+01,     1.4350128844026427e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9102014149173865e+00,     1.6734935110462378e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.5845122697086627e+00,     2.4459938398056715e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0502843103765684e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4850796154354050e+01,     8.3296578271728049e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9063411730949227e+01,     2.2206061831609371e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2998278198285194e+01,     6.5857609102103506e+03,    -2.1583699037182070e-04,    -1.8708058200570130e-07,     9.3204945379627173e-10, } , 
      {    1.8075561609780138e+01,     8.8636990052429064e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9531757138896609e+01,     2.2754187323326609e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.8642144284666117e+01,     8.4879510323167742e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1487071260270849e+01,     5.5022935757018258e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7368744681183031e+01,     6.2795526908746854e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5464431091970150e+00,     7.4650972243270280e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3627518366266038e+01,     2.2759446987958974e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0102308085018997e+01,     7.2009449542593775e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    5.7168049932118919e+01,     1.6171414191187578e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1829007076941052e+01,     2.2717002253744879e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.1229435892491715e+00,     8.2331314508473370e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.7505394769016576e+01,     6.5857619939640153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5211414420821939e+01,    -8.3459644214919826e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -5.1001186655574493e+01,     8.5260349657362622e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5491004903636371e+01,     3.1624798651799862e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2890957537251600e+01,     8.3853584036385128e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6294857593024631e+01,     7.8596434472712317e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2934267172493595e+01,     8.3198312692856634e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.0786882891784746e+01,    -8.3375515801645888e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.3379806602400791e+01,     1.5487397359997618e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.5925933596215547e+00,     6.5061254766174652e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1288954037971067e+01,     8.2817920860227059e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    8.1017164885080291e+01,     7.1587059352724909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6680103519977372e+01,     2.3951075518704911e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.1764880338229489e+00,     1.5549313245783687e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0907634386112983e+01,     9.6654886881445200e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8214580514635017e+00,     8.3220800251718338e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.5154657959091566e+01,     1.5637926122268213e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8733104159080414e+00,     8.4242513986029153e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4310835627209514e+01,     2.3332155643477141e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.0922682067127534e+00,     1.7576803832775727e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    5.6454483248014512e+01,     2.3243490445821615e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.0024747284685169e+01,     7.8604193938597491e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8463729450318944e+01,     1.3277448755248961e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.8448387833834609e+01,     7.2200963398537788e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -1.8912732231837914e+00,     9.5454916929183000e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2687381331799493e+01,     9.2339723545776287e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2002889642177575e+01,     3.2166380092496765e+04,     2.6926668204107091e-04,     6.0173619658272278e-07,    -2.8552616935265007e-09, } , 
      {    2.7688710999191112e+01,     3.1051751533230472e+04,    -1.0154694046500651e-04,     1.6486089226129769e-07,    -7.9131289030698470e-10, } , 
      {    9.9770814741165381e-01,     1.5530135051861580e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.1381357248034149e+01,     2.2796632057540704e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2309856424226558e+01,     1.0058509657837509e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8166135763855010e+01,     2.3837664287406118e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    4.8776034408171448e+01,     7.2137038359500311e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1967944058059707e+00,     7.2316948917344898e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5236899571669447e+01,     1.5520212385052902e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.6411405125870711e+01,     7.2144218949676351e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8589666376544020e+01,     2.5050508623988044e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.4281941779251838e+01,     1.1371704643973499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7579755148318128e+00,     5.8772656020394388e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9318804971561271e+01,     1.5525481293417102e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.6155500169388091e+01,     1.5529247778912606e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3170559265114850e+01,     2.9934831604699215e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.4813158540535497e+01,     2.3858327803709632e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.5704258794569679e+00,     7.7315622431940665e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3684583840374080e+01,     8.4493481275720678e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8503993800344713e+01,     7.2005563541874781e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3343719544053982e+01,     1.6000209609689613e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.9933441440782410e+01,     2.3617014275233181e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4578989053415825e+01,     2.5836550563374021e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.8903935197111572e+00,     6.8127668225673460e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7886886526408876e+00,     8.3248886551158012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2055423003913575e+00,     8.3324941943344511e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7180434084317259e+01,     2.0023719311147252e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3348101239315787e+01,     1.5126323093983181e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9226041827590628e+00,     6.5831254480971056e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4496380716630824e+01,     3.2233894186771657e+04,     2.0760807807756017e-04,     5.2780211021351846e-07,    -2.5075533214347419e-09, } , 
      {    5.3860882636407837e+01,     7.5653971056974169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5641723513075812e+01,    -8.6364983834280338e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.4230552198574820e+01,     8.3973835302977877e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.3688841934990968e+01,     9.5056767910203889e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7842209648992025e+01,     1.4286491436050948e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1732739526059309e+01,     1.4914775337614865e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3157028092156793e+01,     2.5443529593681033e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.5326325266299449e+01,     8.7977394021790205e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6495018770482158e+01,     1.4620200363564600e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0870988712993650e+01,     7.2275693767301873e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5836473814184593e+01,     1.5718546708877235e+04,    -4.6802691043838204e-04,    -4.0980815650507968e-07,     1.9305280781781724e-09, } , 
      {    4.8633484413660675e+00,     8.3390007059806139e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3234698788987473e+01,     1.5556260801455313e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1855998277893534e+00,    -7.5764309128421155e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.1292705978763600e+00,     8.3183821434696383e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0183965435220443e+00,     1.4878398288736005e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.9278831122759517e+00,     1.0001737791016991e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7993472850110329e+01,    -8.3373279231085544e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.1295506390716767e+01,     8.2802400244437849e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.5051935025921949e+01,     1.4445398727684522e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.8653463335904831e+01,     7.2694197956451735e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7512558280799251e+00,     5.9787776033455975e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.7899517393953204e+00,     1.6648746351066824e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1562201718360754e+00,     7.7687473127757958e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8544378470667624e+00,    -2.3182746321461887e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.2767792906456510e+00,     1.2168002681931741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9690459704475998e+01,     2.3209386712523155e+04,     1.1961781289263120e-04,     3.5070869003867363e-07,    -1.6144828875996768e-09, } , 
      {    8.9782271628932779e+00,    -1.6429330994015925e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.1454806081770457e+00,     6.3635865762601888e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3523515226339704e+01,     7.7740069774081621e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6720734042533778e+01,     1.4819274873054163e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.1182215249124070e+00,     1.5558486774331548e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    6.6878181262188683e+00,     6.5850439349464123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8599945591771124e+01,     7.4673984241667622e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0182208356829669e+01,     3.1119265627505360e+04,    -1.6320554442851728e-04,     9.0926805892093470e-08,    -4.4360451821522545e-10, } , 
      {   -3.6943514278298126e+00,    -8.4707267962809565e+03,     2.1869673659189744e-04,     1.8637692342893893e-07,    -8.7906416658781052e-10, } , 
      {    8.6507632708026108e+01,     6.5702132521248595e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3430208469457844e+01,     2.3264468037841394e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.4353107125588167e+00,     1.4286139120092872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6737684045016721e+01,     8.3172014369694807e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -6.2837424889627783e+01,     2.3871452428459808e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.4625604916447273e+01,     2.3871439001358362e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.3300128664792537e+00,     2.3835397298506676e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.4590568701832801e+01,    -8.3401814124807697e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.6541161010843581e+01,     1.1302006916729139e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2461468076057916e+01,     7.8416591050374991e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6079594081395683e+01,     1.5555373528506338e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    5.9507177694966437e+01,     6.4507685464311235e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0137842281102944e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0412606280944425e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6195188492502361e+01,     1.5527021806036370e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6728100129505648e+01,     1.4818886731260625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4071597947487298e+01,     8.0310922092562214e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0354806195028063e+02,     8.3287894762989654e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.2553772064503519e+01,     8.3285933731512869e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6833840068985513e+01,     1.7321738850898208e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2909277326340494e+01,     5.8953145453948109e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9291523125096340e+01,     2.2224110774964745e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.0824910871559741e+00,     7.2080293910638848e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8015027254112155e+01,     2.3872157060375961e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3310019404915545e+00,    -9.0617981230515488e+03,     6.4119227320614115e-05,     4.9063144528285039e-09,    -2.0943951023931932e-11, } , 
      {   -5.6312026481569262e+01,     1.5598111220370300e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4812158305546177e+00,     8.3570249141818185e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.3788777058787467e+00,     7.9122602285243483e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4020161315391642e+00,     1.3342362826366465e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8857448776368642e+01,     1.5558173401223148e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3167270788574214e+01,     1.5514368356773701e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0811834132511699e+01,     7.1856769320485746e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5425778208643884e+01,     8.2196589360497746e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5061361752856669e+01,     8.0753558660171966e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3622019690863056e+01,     2.3905227142412048e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.3766266992320652e+01,     2.3870734369442209e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2266619862538114e+01,     1.4286850465559748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7025198020677472e+01,     9.7364819935569176e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6001714331630410e+01,     2.6229571533067010e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5013992948182601e+01,    -1.1942583630621180e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0750941831298977e+01,     8.1366167920781918e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5566458568703933e+01,     8.0802868981731153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3774473858873980e+01,     1.4968127147816484e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7803200852718160e+00,     1.5527335179144769e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.6524530965166974e+01,     8.5378885745977732e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.4608199036469827e+01,     8.5385932065139223e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1900106138289594e+01,     9.4946364226046353e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0783467425853885e+02,     7.2135549069207973e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.8546873080964306e+02,     7.2145708239968681e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0949992880218147e+01,     1.3109377410853642e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9371342053303746e+01,     7.6831108429127271e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.8619927793203672e+01,     1.5518481416574477e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3123283474999672e+00,    -2.5289780836978116e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.7090287244543653e+01,     8.4019859299772543e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.4079119223732015e+01,     7.9773571844865419e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9606718824589313e+00,     1.5524730081982754e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.5168574835682090e+01,     1.4283837920253138e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.2289352862798875e+01,     3.2199784823676138e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.8788718010879393e+01,     3.2200489455592284e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5335828448145218e+01,     8.1835763087070736e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.1439202785758795e+01,     6.0768924010614828e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.0962784546827066e+01,     7.1897899918493513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2183952698791426e+01,     2.3276900806798047e+04,     5.7959208929120429e-05,     2.7677460366946941e-07,    -1.2667745155079175e-09, } , 
      {    1.5829341714020941e+01,     1.4392788659982130e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0667380921510377e+01,    -1.8995165129313668e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.2078982220309229e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5265211565981460e+01,    -9.4410187841516854e+03,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    8.1815202425640230e+00,     8.3713512406808295e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6007115906622936e+01,    -1.3690599989421967e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1254110789067141e+01,     7.8111920469898914e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5695557984707220e+01,    -1.6651349375055306e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.1976236671401935e+01,     1.6663416323845202e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.4209040854812173e+01,     7.0045133996281111e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.2600409564510867e+01,     8.5846415563811934e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.0769915451603552e+01,     2.4986426590133451e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -3.0353417871141783e+01,     2.4985721958217306e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.9574024804546234e+01,     1.2651922091347988e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9309678014752365e+01,     1.7791139144310175e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3939429171769287e+01,     1.6170709559271427e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1553830305946583e+01,     7.2424487988690908e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8174197803141965e+00,     8.2860316087694227e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0996777389115193e+01,     6.7976316692580549e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7944511795449607e+00,     1.8073704941245055e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0987286658414643e+01,     8.2419503757970906e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.6055436573016504e+00,     8.7451226209259039e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.6447766930737657e+00,     1.4391365969048378e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3823994986007300e+01,     1.6128402059273798e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8913408063476812e+01,     1.6173691707545688e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2823450163981249e+00,     1.6657382849450252e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.7009102933927540e+00,     1.5183514747993935e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3909547439323656e+01,     1.5571140223594215e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2730430749916939e+01,     1.4893116797160565e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2443311218640762e+01,     9.2905014024388583e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.3322100922572480e+00,     1.2139553541353828e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3283068278254166e+00,     7.8243462423031706e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0333194709508886e+02,     7.2383357390683150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7903515311532168e+00,     1.6425493314905405e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.8969284731270406e+00,     5.5775446368742132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0570315091353108e+02,     1.5567027163793440e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9072823929127756e+00,     7.8581181940200140e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6374827798255243e+01,     1.6108705068200534e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.8896043469118069e+01,     1.4914118072661968e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1475871396818725e+01,     7.7997106345486727e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4532560178431559e+00,     7.2102600958495068e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0021055677521321e+01,     1.6496361397840737e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0409328709621548e+01,     1.6031376464244444e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.3298287618168203e+00,     7.1442330215534575e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.5520872278548282e+01,     1.5113521214627977e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.4538168004342680e-01,     7.6977540073633991e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6502605340349348e+01,     7.2294819764980220e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9019431161337916e+00,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9466036903567176e+01,     8.3290846709875677e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.4717611633496563e+00,     8.3282981784626845e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3736700395885132e+01,     1.0451530627530496e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8800764013970877e+01,     2.1553171082762899e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.7283933899045394e+00,     7.8051341922294250e+03,    -6.3923362593445873e-05,    -4.9935809154282207e-09,     1.9489509980603329e-11, } , 
      {    1.4912419675047865e+01,     7.2178656350681586e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3848717913695737e+01,     9.1941574526797176e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9926475728925688e+01,     9.8074752989693116e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8602961564080495e+01,     2.0416740280840240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2989443831015182e+01,     2.4356780791700485e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1056441674457263e+01,     1.6892669464802806e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1604625676693452e+01,     2.0426571094770613e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3824597756022019e+01,     3.1414095200248234e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.2977593128041050e+01,     1.4394368683780220e+04,    -4.0617244174770303e-04,    -3.3596133659847518e-07,     1.5813652650430841e-09, } , 
      {    9.8127078871654678e+00,     1.5538951520574634e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4467214483193050e+01,     2.2145430251406029e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0188937991075564e+01,     7.6408652626547846e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.7203694925730520e+01,     1.5447194316306170e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1376529236885601e+01,     2.1228392028053695e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3153103462143537e+01,     2.2756817155642791e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0758399733501967e+01,     9.0809519366081367e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3147593568625879e-01,     1.6688561252964737e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.1680663352103338e+00,     7.2412928275231016e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7264155203018177e+01,     1.5546557059793284e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1309040237114949e+01,     7.2130964630111539e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.7847423581425858e+01,     1.4859089774952074e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1255621414007351e+01,     2.3864886759309356e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2245859470199168e+01,     7.8423648207073147e+03,    -2.2116475335763771e-04,    -1.8584779777737599e-07,     8.2316999729269210e-10, } , 
      {    1.4376410635493425e+01,     2.8286990488630363e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9475834896835941e+01,     1.5383494682592311e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3417814926612106e+00,     3.1178403514484216e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6424985394655586e+01,     1.4897526024329634e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8102255787383204e+01,     1.4270391376577234e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.4332027924472879e+01,     6.2883355150877996e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4383438581669919e+01,     7.6872659284801275e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5078181448178594e+00,    -1.7216062363339847e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0918418106711595e+01,     1.4966704456882731e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9269052333109293e+00,     7.8243367665170499e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4853258745142718e+01,     7.1185028915810426e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7424323120221544e+01,     2.6622592502759999e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.3167035532308606e+00,     5.1845236671812263e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.1718640553116195e+00,    -8.2642570749124570e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4441002377594764e+01,     1.4904137423882928e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4307622835664962e+00,     1.5069551227162665e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9820464108521234e+00,    -3.4944889009986269e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.0897524041232046e+01,     1.6026774064564976e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.4956145339918752e+01,     1.6097767369977373e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    3.9486157864988847e+01,     1.5467503229011571e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0356967609055957e+01,     1.6365182285846195e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8192028037370644e+01,     2.4047238133602361e+04,    -3.1571415979703382e-04,    -1.5939704207519326e-07,     7.4418900050313755e-10, } , 
      {    1.2050179093191712e+01,     8.3659220532030158e+03,    -4.9286401228435976e-06,     6.9556897567938677e-08,    -3.8265859036294568e-10, } , 
      {   -9.0745085782073929e+00,     8.8613978054031722e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8036235739670573e+01,     1.3905756340294505e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.5196985183651854e+00,     1.5559674970992739e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0473285590305007e+01,     8.3361695912941832e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2217258668796784e-01,     8.3212132581560691e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8346960221985498e+00,     8.3467251087124514e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.4852567924357167e+00,     8.3989733846706458e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2800292663817975e+01,     2.5158601724983564e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4133595529306668e+01,     2.2130810770436987e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0547972871936881e+01,     1.6493089539510092e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5364253365262606e+01,     1.0090755304630591e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2197952994367760e+01,     1.4031707680416968e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9417382905476899e+00,     2.2092461154194836e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.3415145351225171e+00,     5.3821092075333945e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6347276806818343e+01,     1.6102369769656838e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8397473024515914e+01,     3.1807116169941222e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.8447956632615995e+01,     2.4422201038942501e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2546137720580388e+01,     1.4978436429071971e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6332626868707152e+01,     1.5475240195909068e+04,    -4.5295172598701735e-06,     1.3788101090755202e-07,    -6.5643772422231173e-10, } , 
      {   -6.0846378799006388e+01,     8.1919737222645344e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1909087318691189e+01,     1.6402951409774349e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0635859169881435e+01,     1.7206060630699973e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.7842181129045507e+01,     1.7283389234656061e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.4292267168697617e+00,    -1.6674303530259036e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.5796473977483661e+01,     3.2217200185560241e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9562556920487900e+01,     9.2567199749358933e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6789077112822149e+01,     1.6564082844922490e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.3744322627174519e+00,     1.6385678413007496e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7023582769061903e+01,     1.4375156633706474e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.7237476410317196e+00,     1.2398586532640060e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.5804163996226393e+01,     8.3284857094979179e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7141858756537438e+00,    -1.9187229332080515e+03,     3.3689048201832156e-05,     3.6263384607839732e-08,    -1.2086889883741836e-10, } , 
      {   -2.2321597981009095e+01,    -9.4456211838311538e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.5896713352908058e+00,     1.2566151700910864e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6588907867021540e+01,     1.3839633047770034e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.3032590159172450e+00,     1.2005565562947071e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0606931970707265e+01,     2.2617131744657730e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.3812549291397538e-01,     7.8554883617038313e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2806651306384431e+01,     7.5251061172388290e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.1830956369379720e+01,     7.0548032578671846e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2867052450559434e+01,     7.9379716548718716e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6841237453807111e+01,     8.9968139116685816e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8730875710926725e+01,     7.6052120075953289e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1420302222683453e+01,     6.6394357230913574e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5506222872840851e+00,     9.1922856251232261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7963870354228614e+01,     1.1695027886422127e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0018854645418198e+00,    -2.1102199644829902e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.1774410734820666e+00,     1.7755090727907762e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.3164050016693743e+01,     6.7848297899028512e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7203141449566012e+01,     2.1342467631211272e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1310986072958453e+01,     1.4481094828128857e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.8901920548421387e+00,    -8.2047384978876580e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.0936772192430297e+01,     7.6848418113911530e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1534563940376289e+02,     8.3283458223177768e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2633993385544626e+02,     8.3290370271324755e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4786152017545575e+01,    -1.7432884602699114e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.2830521508238633e+00,     1.4790493778300948e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.2084670340729389e+00,     1.6911814289126152e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7394792236929202e+01,     3.2183216458825427e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.4376324904457988e+01,     6.4905834483290355e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8611443811732089e+00,    -1.0018658471825771e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.1524154241283583e+01,     9.2737872564755398e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3503703228846867e+01,     1.4916524247922114e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0468812187589883e+01,     8.4526443515625924e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3213689454446502e+01,     8.3281391351926432e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2219439120841651e+01,     8.3292437142576091e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.7525180197709265e+00,     7.1972021187646496e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.9695908945939591e+00,    -5.9225916864313297e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.4013959587395721e+00,     7.5226051188301062e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4366300854528159e+01,     6.4901953065354992e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1751803621030337e+01,     1.5876815533655687e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9821624723600671e+01,    -8.3395081360848471e+03,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -4.2879135302298579e+01,     9.0919857437528353e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7217468222673048e+01,     3.6147409882490110e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0243361387858823e+01,     6.0214374193013009e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1823683873447131e+01,     1.6833150887778011e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1589979981257214e+01,     8.8018944877464201e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1118363038143340e+00,     1.3700491351003033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1914138722054911e+01,     8.7838455443910461e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5755681943122468e+01,     6.6813564847807866e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8537947927127547e-01,     1.5168878144814318e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3510042086179226e+00,    -8.8330460829332023e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.1324145081171402e+00,    -9.5033524932613432e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2355235812859187e+00,    -8.6359420039809891e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8180272057359232e-01,     1.1612544593254084e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4224098546984983e+01,     2.9088811421913433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4936412946403571e+01,     9.4658618891224760e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.7755875677946840e+00,     1.6719739656479222e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9794461950824171e+00,     3.5923954219002314e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.2178201111394671e+01,     8.4862200638383492e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7198279415084894e+00,     8.5574285602640739e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -7.1703170692395304e+00,     2.2126208370757515e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.2974917202087006e+00,     1.5604887440156894e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6653203994957746e+01,     1.6746400363063851e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4049167424743509e+01,    -8.3061495195940697e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.6761062025773935e+01,     7.2320870736600364e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.7251906822031060e+00,     1.5523566620475733e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8630819419586185e+01,     8.3309584136245703e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.4295485261307846e+01,     2.3307572479676212e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.4700873757527706e+00,    -7.6539831738542398e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7169491298534833e+01,     1.4425495898601483e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.0424005922423330e+01,     8.3297873056931003e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0025454686768054e+01,     2.0809761250533225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2638924979803601e+01,     2.2914551509983223e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4477993078762090e+01,     8.5021582632131485e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6592176804730070e+01,     7.8450002828205606e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8424297704640979e+00,     8.3282955906373136e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.7291327955009059e-01,    -8.3290872588129423e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4093410826669061e+01,     1.4550984606148828e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8903773387344067e+01,     7.2135105759263497e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5381427965569223e+00,     7.2146151549913156e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1279038281376749e+01,     1.4430755563233850e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.5423308839599940e+01,     1.8136166723694438e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8349849707227062e+01,    -8.1890060137400669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.8123176795832459e+00,     7.6666080419736800e+03,     1.8590386647959628e-04,     2.8671813226902599e-07,    -1.3064807559907884e-09, } , 
      {   -5.7380534825319582e+00,     4.7915026974882394e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0786131152973191e+00,     8.4875987163586997e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6910813239403337e+00,     6.5677310722120837e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2372383724888579e+01,     1.3502398380546631e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3283716205776459e+00,     7.6996724942127057e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0400402509336200e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1713667811124088e+01,     1.4233170022881681e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5674642178597189e+00,     1.6678712757428104e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3802836584881703e+01,     8.3750491223830250e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2651626673082454e+01,    -1.0349987554704690e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0796630694841525e+01,     1.4428125730917665e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.4447149782004534e+00,    -1.6834175672870383e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6009067990508044e+01,     8.6464613332457266e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0150492198900725e+01,     1.4950495121540825e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5133998004654641e+01,    -9.3300004527762781e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1547038739690615e+01,     2.5128109645731209e+04,     8.5928764690799039e-05,     3.1444530543083391e-07,    -1.4936139887622584e-09, } , 
      {    6.3600404269785784e+00,    -8.3106672165239215e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.3498659692583628e+00,     8.2522501812664577e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -3.7143835810746069e+01,     9.5282186961514453e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1021256933474497e+01,    -1.7350104942831140e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.1993089466258130e+01,    -8.3275955437571520e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8846870093246203e+01,     2.7015613472452984e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8330670034479567e+01,     2.3889469923110286e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.5160129942752382e+01,     1.0844551597223483e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1362221003490749e+00,     1.6636052941472401e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0996336909049031e+01,     1.3044943061040976e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.0577008234035901e+01,     8.8600962092079899e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6669664105212529e+00,     1.6422096234097698e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.2374932731895989e+01,     1.5480621140211026e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7234926356887286e+01,     1.6011802267637853e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1278783228725541e+01,     7.1714030495031293e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0848727767625914e+02,     1.4914544756712256e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.0289649078117522e+00,     8.3275279632181046e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.5029311880122620e+00,     7.4313157968240630e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3953524190168569e+01,     1.7743145754001449e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2265459912470611e+00,     1.5542530633127924e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9850506602152510e+01,     1.5542977947239990e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7803478385296367e+00,    -8.4051326681837945e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1884792555122834e+01,     1.6622437960440267e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.3388593781956331e+01,    -1.3245303410550900e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0867011442236262e+01,     1.8419446729355717e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6311538717749396e+00,     7.9018890275291742e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.6571763444224494e+00,     6.1282999609076323e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3954085758558634e+01,    -1.3238257091389401e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.6882095102793400e+01,     1.8529187693387426e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.3450023322321538e+00,     6.8548233232688099e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.1258651750479260e+01,     1.5542202000651476e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0916524491592467e+01,     9.5853065948162130e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6632384173932827e+01,     3.2264571489446880e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3712418634747742e+01,     2.2679264894630669e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.1818826860685938e+00,     1.5543306579716442e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4654699115877587e+00,     5.9705656065672010e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7186000013896869e+00,    -9.4095629368539740e+03,    -5.5395567512918104e-04,    -7.2425346193591365e-07,     3.4241420669404302e-09, } , 
      {    1.0744424677494870e+01,     1.5610268384458850e+04,    -1.2784672518689175e-04,    -9.9871618308564415e-09,     3.8979019961206658e-11, } , 
      {    6.7236195680921442e+00,     5.9336269603555695e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1961330140898763e+01,     1.5003464218752017e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.5308406332871627e+00,     5.2489580169938954e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3162056681803639e+01,     1.5558753798508444e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.8889320401959893e+00,     1.5638314264061748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1261887396433004e+01,     2.1659109277185282e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5245745672970067e+01,     2.3402045421497118e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4136334655675423e+01,     6.2915089388394736e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5628737714638685e+00,     6.6380130321576044e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8447952861555606e+01,     1.0129502963249904e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0736553077025892e+00,     6.7408598024375378e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.4847388868691294e+00,     1.4388310829019752e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0946090552045682e+01,     1.3697646436945241e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8220678775118468e+00,     7.4061374981057670e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3338872219270986e+01,     9.8876573922976240e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5194823504289756e+00,     7.3096228393366227e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1247373238935609e+01,     1.5383024512689856e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8641528992759540e+01,     4.0077619579419988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9684192068670114e+00,     1.6866932315280974e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.1401026904420441e+01,     1.0200496268662300e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4100434069654817e+00,     1.3107076211013909e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1397249246335033e+00,     1.2791607502333045e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.4021645735821977e-01,     1.1219523623561096e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6449568924439095e+00,     7.2298106089744706e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0811577673262949e+01,     7.6846360961639439e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3914897416313769e+01,     1.5526754781859474e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1473776483364773e+01,     7.2297953496064220e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.0356015627432265e+01,     2.4593053304482390e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.2737156635913848e+01,     3.0563139189744757e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.8903130645293832e+01,     2.2128157254639176e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3461350687482160e+01,     1.7287991634335529e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2255400850357507e+01,     9.0349958966619070e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1817688301555156e+01,     9.4302020727919644e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5471110906031857e+01,     1.4461882778055107e+04,    -4.6783104571121383e-04,    -4.0989542296767940e-07,     1.9290736371348431e-09, } , 
      {    3.1309700723266637e+01,     3.0497015897220288e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.7299506812686580e+00,     5.5644267256747589e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4029294899440812e+01,     2.3190169032652349e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5698474469142255e+01,     2.4328901034414739e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.2050856758809849e+00,     1.2714055241320923e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3275158497739248e+01,     2.2654645446715909e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2358565523701969e+01,     8.3258465106673357e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0190824886718785e+01,     6.6497450043468443e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7317638288016571e+01,     7.0743774544737735e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3645186086677592e+00,     8.3315363387829166e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8812290911293825e+00,    -2.1359571140048242e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.5538406164106249e+01,     5.9570953794096722e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.9379787651687934e+00,     2.3893987620040141e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.7488629269713254e+00,    -8.2849205912159669e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.8279772853999020e+01,     8.2972111970618971e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8414455848659941e+00,     2.4125877154584989e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.6149685505979249e+01,     9.4260469872245631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0528195490890102e+01,     7.8761218678447603e+03,    -2.5199405533939313e-04,    -2.2281484096197810e-07,     9.9702418333857162e-10, } , 
      {    1.4382174574397425e+01,     1.5366961871490683e+04,     3.3565066799162013e-04,     5.3770200558177518e-07,    -2.5479867824392772e-09, } , 
      {    1.3376739547032694e+01,     8.1551903752968155e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.5647441021509763e+01,     3.3019021118843320e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6706204903263988e+01,     1.3877370406884245e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3555559226500732e+01,     2.8168728117789897e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2812046958818172e+01,     3.0927774226027486e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -4.1359310891854246e+00,     7.1986437544196433e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.5507821126912944e+01,     6.5303983502269475e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0601155510774221e+01,     8.3724622582342854e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.4793993524073798e+01,     7.8427227664624506e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0622816366073553e+00,     1.4850012355325747e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2090555388516027e+01,     6.5754459991578015e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9713600019891313e+00,     8.2407774385039484e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.7444265063694004e+00,     7.7540642823576509e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1620323320506753e+01,     7.9978036050050723e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4974192101864043e+01,     7.2140733876626418e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3916997709358938e+00,     7.2140523432550235e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8992238130505299e+01,     1.4467940632815578e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0169719552018915e+01,     7.1507844869921537e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1938532559817059e+00,    -1.0229361923377393e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0186597738236781e+01,     8.9577103552375447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8243674961112557e+00,     1.1452814815759980e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1761318206518705e+01,     8.8690850235494927e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8202678980417517e+01,     1.4071522582314881e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8332761131036129e+01,     1.8922208663080415e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9642791881644399e+00,     1.4913735359671540e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1307717484894717e+01,     8.5442743649507956e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.8593648952515132e+00,     1.4397472808607961e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0658812680543530e+01,    -1.7422247988449603e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.4593723910331695e+00,     2.4410735786341029e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.2723833020150829e+01,     1.4912369162354715e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5095981894773152e+00,     7.7643735636131378e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6218805749300088e+01,    -1.7425838283537619e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.2409684624351733e+01,     1.5618005351356347e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2734963074784206e+01,     9.2129491414267359e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0225127078360659e+01,    -9.4770770311288652e+03,    -4.9229707116567030e-04,    -6.5031937556670932e-07,     3.0764336948486714e-09, } , 
      {    4.9599524456654684e+00,    -1.7947803827409822e+04,    -2.7360033457377286e-04,    -4.6394245213777045e-07,     2.1973695282608606e-09, } , 
      {    1.1974274689810954e+01,     1.8229466728222200e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2669986868609488e+01,     1.9313234209619732e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1429681564265220e+01,     2.3149838125335780e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1068798252957176e+01,     9.6497409446288821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0186993819164716e+00,    -2.8936131199805400e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.1005510681547584e+01,     8.4111747178931673e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5571287140427559e+01,     8.3297949521175487e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.2351604024024720e+01,     1.5561941959892183e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.3865636960568725e+01,     9.6835223721318453e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.0295295210900548e+00,     7.3369217406899961e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0694972383284899e+01,    -8.3242821975560546e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0354200393960378e+01,     6.5644253724354376e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9691766736728358e+01,     1.7430306877636773e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9622700805401188e+00,     1.0826502653868109e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2609502509380821e+01,     1.3922677021103285e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.7006969279731550e+00,     7.1360659785675944e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7032174223037629e+01,     8.4460853959628657e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.6516349746332004e+00,     9.2544187750961592e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3940250729598436e+00,     1.6050405172382561e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1443838834253359e+01,     7.5449506851788865e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1855386563089017e+00,     7.5360905297394238e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3666981603014685e+00,     2.3807011365096416e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.0066672120856552e+01,     4.4007829276349858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4809677475570791e+00,     1.0630045376062535e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0245193796919461e+00,    -2.0764385369800257e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5482879557902642e+01,     7.6207540358837578e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1054071187167551e+00,     1.5534117791800531e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.2697497463932606e+01,     1.7268417437728938e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1112277846983385e+01,     7.2567226814145361e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1317712269460951e+00,     6.6790552849410524e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4573566363208474e-01,     1.2321034271627936e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0060271815304013e+01,     2.5379095243868367e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.7905807535115152e+01,     7.2893139266312210e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8745634374373212e+00,     7.3537824873841791e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.6636269744533507e+00,     3.5337070935533859e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.1977914406638455e+00,     8.3356950062039668e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2191276455643564e+01,     7.2773412439255117e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3475048100778764e+01,     6.0640972352570052e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5488114369135600e+01,     2.1735488600904260e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0867658620611405e+01,     8.9232119524706432e+03,     1.8057610349377928e-04,     2.8795091649735135e-07,    -1.4153602124943682e-09, } , 
      {   -1.2103086037701585e+01,     1.6655305306666552e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7808500976175523e+01,     1.6659460392233956e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.6720512196177324e+01,    -9.6505682499824070e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.6926399438096816e+01,     1.4252333312224389e+04,    -3.5162954514457308e-05,     1.0082670126035020e-07,    -4.8403797921976076e-10, } , 
      {    1.2518998604424814e+01,     6.1518967384608823e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.3626186232618744e+00,     5.1714057559817711e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6162378584625053e+01,     1.6600294950785483e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1356053117728532e+01,     2.3243138129863542e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.2096165370799135e+00,     7.5088680578361864e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.6962668660999306e+01,     2.3854172718142228e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.9386662369355491e+01,     1.2088048856115114e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5489833779913308e+01,     1.6013327520889070e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9874590444596834e+01,     7.6450269094932410e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1448152997017296e+01,     2.1202782220226214e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.1600683795229010e+00,     4.3984817277952516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2895548589270156e+00,    -1.7171989947900026e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.2135543962986432e+01,     2.3010152714350719e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1327325297089679e+01,    -1.5840288526782624e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.8942642869126743e+01,     8.3278793787396498e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    6.1089941071033152e+01,    -8.3295034707106024e+03,     6.6570736180512717e-06,    -2.0358780910832745e-07,     9.7132421010295529e-10, } , 
      {   -4.0428158409788502e+00,     2.3869368172125385e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.8302747183351173e+01,     9.4884037942535324e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2530402976253555e+01,     1.5479475911717280e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.3209421808411001e+01,     7.3109656660215132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8255418990597214e+01,     1.7037285907943784e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7335981106469028e+01,     9.0033996755960143e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.8307256388749433e+00,    -1.6644763611127873e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    3.5831206454086377e+01,     2.3873523257692785e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.8940599199233734e+00,     8.9312618602488365e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.2682556096267049e+01,     6.1167073029593957e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0678218823056465e+01,     7.2037535842033449e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2049919086621816e+00,     8.1981795422879313e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.7475208655128519e+01,     7.8994583491691446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0269343729326771e+01,     2.7408634442145973e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2897288112288745e+00,     6.0626745443232521e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9498466113718278e+00,     8.1697841330915517e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2572802426705332e+01,     3.6759761869286535e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.6881223433238439e+00,     6.8401867200891929e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8952516037651591e+01,     1.5541787887736280e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -8.7610798521301859e+01,     1.4914348653564577e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7907745213578298e+01,     7.2112179514010413e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8770823282050006e+00,     7.7526415914238978e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6648752982376614e+01,     1.5702013897775607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0895804694542479e+01,    -7.2609622041612520e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0394666021654125e+01,     1.6670002087772631e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.8349728284796907e+01,     3.1871550519753895e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -9.8007577922188531e+00,     2.4497452100114893e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -6.0769518003299652e+00,    -2.3188953454484704e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.4570518075248247e+00,     7.2169077795166240e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1671783410404988e+01,     7.2243721467143205e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.7753977016065456e+01,     9.0521708418549224e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.9153363462206983e+00,     5.9581590408346228e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4552314673973056e+01,     2.3243849475330418e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.3847155157997868e+00,     1.0433481684175122e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5853708095081458e+01,     1.4880670907432510e+04,    -3.2597005385132925e-05,     1.0025394237748739e-07,    -4.2887103044630662e-10, } , 
      {    1.3883079622725578e+00,     7.6228315786674593e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1238957529444500e+00,     1.5261067009006059e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1130931529616880e+01,     1.7823327847329761e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1096461981164431e+01,     2.3656829177131091e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4092570648775764e+01,     2.3243443906549601e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0538254114266113e+01,     1.7836445758529215e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.6424569982551063e+00,     2.2299361836137137e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2359191900246618e+01,     9.9783275645091999e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7615847921749026e-01,     2.9424634157381547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4768449470052261e+01,     1.6393230579382602e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.0866205601722260e+01,    -9.6551706496618754e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -8.9223480456995233e+01,    -1.1147266108401323e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3790472010128713e+01,     1.5553063571439447e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0687833459274084e+02,    -1.1145305076924546e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7071179259951638e+01,     3.6949230815773199e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6692985763099045e+01,     2.3892775622886937e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.1205045759209344e+00,     1.5496353624358613e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0947165854929157e+01,     8.9750232179718751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9778370977139829e+01,     1.9315229632773404e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0287560920288993e+01,     1.5539909376126168e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8278858215067380e+01,     1.6974171096155122e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3700976267558254e+00,     8.9543691774544859e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.7440390984675233e+01,     1.4929988733426844e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7880284683797560e+00,     1.5545599204241749e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0670715997723470e+01,     2.2988706690187639e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3244000529046495e+01,     1.5532445008928471e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8785487431659167e+00,     1.3812936057071576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.2291784005932271e+00,     9.6474397447891479e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6583406610697583e+01,     1.1237572566916468e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8945185182065396e+00,     8.4617996470971793e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6946485212851212e+00,     1.5636054294711721e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9520706860155506e+01,     2.2834369416654914e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.0142034337161988e+01,     2.2208139374393071e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.2813031048965486e+00,     4.8597398169102344e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2951214508353315e+01,     2.5561412762009728e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0613143852522140e+02,     6.5858533319871294e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1710893757093539e+01,     8.0868458537728438e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2832951691403508e+01,     6.5632133752822338e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.8068215449824443e+00,     1.0040460714482133e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8761223758047610e+01,     2.3577199373335272e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.5260619638474839e-01,     1.1928013301934949e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.6491541166147400e+01,     8.7579245002811076e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.0026696512154587e+01,     8.9725410380591002e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2180109882500119e+01,     9.4739729063011237e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.7455712036089679e+00,    -8.6572780446054439e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.4347954685936699e+00,     7.6900745584240949e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7800112142354867e+00,    -8.7554938219210781e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.4402761469484140e+01,     1.6652451817294332e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.1487782492800179e+01,     4.7938038973279736e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7378028719576875e+01,     1.4252393332549549e+04,    -3.0031056255808542e-05,     9.9681183494624578e-08,    -3.7370408167285253e-10, } , 
      {    8.3766653081617957e+00,     4.9127089141214256e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5574651053163386e+01,     2.3362582835557278e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.2468172153196111e+00,     8.9033185670926014e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8793616537949525e+01,     8.3267425408434665e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7901852127766098e+00,     6.5923142360130169e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.4324054275193321e+00,     1.4004451838914973e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5080927458116072e+01,     2.3850115806931233e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.2137243872883403e+01,     2.2128861886555322e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1779514060478732e+01,     2.0295391982776064e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1421075392815840e+01,     3.0534753256334498e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.0586368207429892e+01,     7.2266821037812115e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.2418563875530388e+01,     1.3437964030733963e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9233746973344994e+00,     6.4618023535758221e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.3794534534839151e+01,     1.3895419350239619e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4868338161351780e+00,     1.6730463662918642e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.7388637764551774e+01,    -8.3288971399523343e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.5806155049019260e+01,     3.1138477677579111e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0309622061852375e+01,     1.7264360526517943e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5383314486964050e+01,     1.5021096245027675e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8724437854616305e+00,     8.7251094752258232e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9848331448845972e+01,     1.6651482795363834e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3146956505004663e+00,    -1.0228207937656493e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.5242124015675678e+01,     7.9375422825886290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2858796097611283e+01,     1.0593517238355289e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6028997482016024e+01,     1.5543720692631636e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.1990620130663752e+01,    -5.9000540864171001e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5478671409080249e+01,     5.9578000113258213e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.3384763284279373e+00,    -8.6019711959039023e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3442360000033007e+01,     5.5841780702557071e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4836905372061960e+01,    -1.4088749008401089e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5815942538188818e+01,     1.4921924871707473e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4879749086790870e+00,     8.8930092858371136e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.1782948076177576e+01,     8.3451998554612346e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.8385534433582642e+01,     8.2998410293780798e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5761060537670184e+01,     7.6917473412961554e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2418429850976388e+01,     1.0042761914321869e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0266594959341706e+00,     8.1916205866882046e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.7188846975394370e+01,     1.7562663779302755e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4239268186709516e+01,     7.3733224730504817e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0397274384125890e+00,    -2.8406440227693488e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.1673882579077258e+01,     7.1967898686919762e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.1143164554809069e+01,     6.5878328231969899e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.5448606174394811e+01,     8.8202813073100770e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4608771791299651e+01,     2.0977832594928543e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6856347011510390e+01,     7.2332505351670579e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.2243058144411822e+00,    -1.5995323847064325e+04,    -3.3811868475736040e-04,    -5.3717287993021232e-07,     2.4920926131441587e-09, } , 
      {    7.9202659603623724e+00,     9.6889178662245522e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.0262643807948347e+01,     1.6136116986219515e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -7.7989366219170613e+00,     8.3306403086067858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6165534842999193e+01,     8.2926087973824269e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.4176352358265378e+01,     3.1045693678470005e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.3881965561013798e+00,    -2.4748550007657186e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.8905003528089082e+01,     1.6641840821161823e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.5742183189444603e+00,     2.4420123496158805e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0691116361878228e+01,     7.1618051137145176e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.0703187016006282e+01,     7.6432959410148142e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4468947040516952e+01,    -1.1059920608828652e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.3199924203771587e+01,     1.6672924877738682e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4537725287642573e+01,     1.5606032668650638e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7472558141526839e+00,     1.5542743767980148e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7329806838058420e+01,     1.5542764812387770e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3599157807680724e+01,     8.3493549410286359e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.5743307055304747e+00,     7.8489294061041037e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5515100388235615e+01,     2.7173653603479484e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.7887557715889804e+00,     9.1135793783901991e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1786875244416478e+01,     3.1032539483156725e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.3153075844529262e+01,     8.2462081315570849e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.5023372872170242e+00,     3.9411898805253310e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    1.2482054151001909e+01,     6.6435600144795342e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.7019410737617218e+00,     1.5707047600124117e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.1006749262377532e+01,     2.5597461178412141e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2567763250079953e+01,     1.8216348817022750e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3567025707803566e+01,     1.5500094474228255e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.1214614814989012e+00,     1.7013985998053038e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6795990738172293e+00,     7.6048238658017926e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8755411098430034e+01,     2.3226217449054759e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.6303696322957006e+01,     7.8420181345463016e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2217688836296107e+01,     1.4914452334935477e+04,    -6.3524239730472436e-05,     6.3330532424185123e-08,    -2.5428962387876272e-10, } , 
      {    7.2744394856748755e+00,     1.6187982556038283e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5969212595405471e+01,     1.5560778498385162e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.4975333495736296e+01,     6.7470825087088679e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8145915589991475e+01,     8.2288324645976354e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    5.5084399195525975e+01,     3.9416501204932778e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.9647375787754182e+00,     1.1534992332241960e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1892898863801737e+01,     3.1082878748051735e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0624198506636558e+01,     8.3325047165382621e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7026067802771934e-01,     8.3248781329119902e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7168326448888386e+01,    -1.3292450970442842e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.3690585044224077e+00,     1.5453736776570357e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3759648605804607e+01,     3.1125323482265831e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3645645997873693e+01,     8.3301141156588783e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6511267472795499e+00,     8.3272687337913721e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0491436892591793e+01,     9.1768665140840349e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5977119699729144e+01,     3.1088138412684100e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1674317822574682e+01,     7.2663206172031478e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1167264719427141e+00,     8.9047412580263535e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2016904746961075e+00,    -8.8668275104361655e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.0508565759530129e+01,     7.2366047705898891e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.1501334386582016e+01,     2.4502054499794362e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1770863812974612e+00,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.6886795859828432e+00,     6.2174862944581560e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1284851668485864e+01,    -5.6387323523287432e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.9294650893651735e+00,     6.3093741736172706e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8715259026825084e+00,     1.4319907426824440e+04,    -9.1689660219319288e-05,     2.5747097125420347e-08,    -2.5995709581093384e-11, } , 
      {    1.3977063744341914e+01,     7.1465487711839414e+03,    -1.5684226790121844e-04,    -1.1253010352233441e-07,     5.2990135345272272e-10, } , 
      {    2.1760108676669262e+01,     8.1764316939761038e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7169271021611729e+00,    -2.6867124380382438e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1039933313150524e+01,     1.3294369436057743e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4045175004366325e+01,     8.3023830340985678e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4775475374478056e+01,     1.0280678361990611e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2896673809028751e+00,    -8.9737562790106324e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8494301433116185e+01,     4.0879440512703059e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8893160508960225e+00,     1.6763321043872635e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7801468536124183e+01,     1.6641173514108345e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2910618791943314e+01,     5.1868248670209614e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5399337918200777e+01,     8.7151534388183863e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.5735393109211728e+01,    -9.6532217657802157e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.1589723200357867e+01,    -9.6525171338640666e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.0572097177561156e+01,     8.3434099331496855e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1220103194026777e+01,     1.9708250602466389e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2394604953810649e+01,     2.3555977006228222e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3051297362426753e+01,     8.3549998153516844e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6024510937254841e+00,    -2.4694595066730130e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.9094724388867967e+01,     1.4111337484212794e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9134446233822317e+01,     2.4972956362975921e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.3947597000269791e+00,     1.6341914140769388e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.6549737859164537e-01,     6.5988731916127444e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2980102465717042e+01,    -1.4511511601805273e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.7151581727493898e+01,     2.2756823869193518e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0031145033084874e+02,     2.2756810442092064e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5189024365633788e+01,    -8.0807855710501908e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.0108111945948174e+01,     1.6662313881606173e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8755788529780215e+00,     4.4092271690706291e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9870282909022009e+01,     1.0522523932942891e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6657556512467821e+01,     3.1660847068202267e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -9.2695253226649790e+00,     1.4983138810711076e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3395816748575525e+01,     1.8622487697915189e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3044600312292204e+01,     1.5490496538439644e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5822174483898266e+00,     4.0054607581022628e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2648234810463606e+01,     1.6028722948446635e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8959959600510295e+01,     2.4106732330261639e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.8556395474891598e-01,     1.5085298970678301e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0710176876918162e+01,    -5.0435465820807576e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.4655222113297626e+00,     1.4679160089785861e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1733844502283624e+01,     6.2222931814237027e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4235199715303906e+01,    -1.1167061020499941e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5554577463111450e+00,    -5.5731427963314673e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1307274722116258e+01,     1.6066311522679660e+04,     1.5004799201141317e-04,     3.1935161988366242e-07,    -1.5145579397861903e-09, } , 
      {    3.2870452307402829e+01,     2.1595803189919199e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1802159554215033e+00,     8.5375295450889716e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.9038251381357956e+00,     8.5389522360227238e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.6124105075163495e+00,     7.7184175236669080e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.4155765194320828e+00,     1.7126783142862219e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.7245484412886640e+01,    -2.7170443644275929e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.1890195878605979e+01,    -1.1125510164825928e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7117261483821835e+01,    -1.6675431792805626e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.6486575075262486e+01,     3.0459502195162113e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.1691915341664473e+01,     2.7801655411838961e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6234568848642606e+01,    -1.6209335341904819e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.6028162291501076e-01,     2.2677187351846966e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.4271773027196332e+01,     7.7003771261288557e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6494402466543882e+01,     7.6823348963242097e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0691076819834883e+01,     1.6154141194420721e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.6903726124462835e+01,     2.3855903686620659e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.8901470329387617e+00,     8.4984259886579348e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.5190746001472476e+01,     1.6869233515120708e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.2033198708737402e+01,     1.4987741210390544e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.2722086700760855e+01,     8.0181537482001822e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3290603356419828e+00,     7.3253965539739211e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.0107218624065224e+00,     1.3611430869221986e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8108725172398046e+01,     8.2380212525135485e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2069745683893402e+01,     6.4099684915170747e+03,     1.8600179884318042e-04,     2.8667449903772613e-07,    -1.3072079765124529e-09, } , 
      {    1.3110396179618725e+01,     8.3441212786165615e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.5229807054425081e+01,     7.6329431902770966e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3800114508906795e+01,     1.6582131788277864e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.6513817898179219e+00,     7.2761960154317676e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4303766137301862e+01,    -8.5513588017553029e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.8987712334519857e-01,     6.6556450584332624e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1087896016026953e+01,     6.6030282771801458e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8710214289982012e+00,     7.2138392084027992e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1597803608512411e+01,     8.8663288375590928e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.1599543990252570e+01,     9.8224650377252601e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1489807801017735e+01,     8.4273068741273401e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7493275688818603e+01,     7.2142865225148671e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.3845252595406112e+00,     1.1141971362548973e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7131381736089416e+01,     7.1388118042864444e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.8033113684806947e+00,    -1.0796836702563071e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.3597581603250468e+00,    -8.3095132308030188e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.5641057504705330e+01,     2.1577754246563829e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5523444923265416e+01,     7.5169457054160757e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.5007845974889840e+01,     9.1543425507818047e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9072971003248558e+00,     2.3203323227965629e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.5119557944961670e+01,     2.3886987743197511e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.9966668061274603e+01,     6.5856572288394509e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6937555301972679e+01,     3.1103863300409357e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0809376938883528e+01,     1.2481069825808099e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1161297124012933e+01,     2.9881862507488026e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9584608205175609e+01,     7.9663233773418406e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -1.1628233768121348e+01,     1.7308232339626851e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3837917029213038e+01,     7.1983303813112434e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.7169139978249916e+01,     6.8972746187539615e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.3586321760664717e+00,    -1.0247414357933178e+04,    -1.1862370243951608e-04,    -2.1414772982204668e-07,     1.0654701788376160e-09, } , 
      {   -6.5477954968336327e-01,     6.6205900238592767e+02,    -3.3493183474663908e-05,    -3.6350651070439449e-08,     1.1941445779408981e-10, } , 
      {    4.1638731368696334e+01,    -1.6234914386180999e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.3557126134557784e+01,     2.3403173684043708e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1233898638338646e+01,     1.6692719920385785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.8909667522626627e+01,     9.0123559399570113e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4031604874098203e+01,     1.5595012041928274e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0891294970953371e+00,    -1.0012108992012165e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.4553823233467639e+01,     8.3284371603408908e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.3559523547224615e+01,     8.3289456891093614e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2032889430926474e+01,    -6.7903889734561399e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0922344622057057e+01,     1.6957103814615479e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4088437448584870e+01,     6.5860839128897387e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2204667175229716e+01,     7.0405618160305221e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -4.4018678353236098e+01,    -5.7087898664768545e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2159799021498234e+01,     8.7025675700947650e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8012536380514694e+00,     1.6551444655027870e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3967539526879918e+01,     1.6362577657072898e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.6656704902578547e+00,    -1.7959052415230281e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6396645930717824e+01,     1.0677292205638742e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.9346919239303109e+00,    -9.2260670526261893e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1422747350075438e+01,     7.6938248840798551e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4003328264381885e+01,     1.8609369786715739e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1890573142462948e+00,     8.1452199208628608e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.3384598903422720e+01,     9.4094786223738574e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0805811760984968e+01,     7.2300623737833184e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5288815920423939e+01,     1.0076543341824834e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.8743773636832444e+00,     6.6736692666344670e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.3005675820040228e+00,     4.8067707196990423e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4334646506639565e+01,     5.5798458367139492e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5485609479334252e+00,     8.9673082910261564e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9947315993132682e+00,     8.2764336729808110e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0865083340474087e+00,    -6.2567775272935887e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1313018508786422e+01,     1.6972851558131115e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3833217655384740e+01,     1.0435782884014856e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0528934402970112e+01,     1.6345863511109581e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.2796469300022935e+00,    -1.7251927757561287e+04,    -3.3279092177154333e-04,    -5.3840566415853768e-07,     2.6009720696477380e-09, } , 
      {    5.9995546462470504e+00,     8.3809491764694412e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4375708281658284e+00,     9.5213168708826579e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8959388837357480e+01,     1.5589154956009301e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.3031375315417323e+01,     6.6924048203025486e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9550652868083652e+01,     9.7601082306962908e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2527543070215486e+01,     1.3721821258980885e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0178394458487503e+00,     8.3805065362154164e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.2395958410910882e+01,     1.6864631115441240e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.8045492917904475e+01,     9.5683859140074328e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4055749475402056e+01,     1.0371348534202187e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0110325883599511e+01,     2.5025889176073291e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.6300100484968894e+01,     1.5371902024644070e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3080590096675415e+01,     1.6442766311672262e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1654624158986699e-01,     2.4946259372277465e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8006199523780090e+01,     1.1630593536609456e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0426330041064290e+01,     1.6694142611319538e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9917734739377178e+01,     4.4809650209632946e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7901443190280748e+00,     8.0182093328307928e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1639177445772134e+01,     1.7856910120876571e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.6245941238159816e+01,     1.4485213629582435e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0289361244264985e+01,     1.4585860085258098e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.5706877163871100e+00,     8.0066637604445350e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.8035868442379854e+00,     1.0748950392855986e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1239596105650023e+00,     1.4444335066259569e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3949298585225298e+01,     8.5780557924537607e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.5838952660660357e+01,     6.8574597168560485e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3315885425018923e+00,     9.0525589836484596e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5205560985917309e+01,     8.2484923654496670e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.3277503521324363e+01,     8.9570057233213956e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1565768777768790e+00,    -7.7341708969796082e+03,    -1.2404939778891725e-04,    -2.1287131236242152e-07,     9.5731794285570074e-10, } , 
      {    5.8124687072585788e+00,     6.6195423377133147e+03,    -2.4676422471716020e-04,    -2.2400399195900355e-07,     1.1066308603638158e-09, } , 
      {   -9.0696677552925387e+00,     9.5698874837770236e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6723268313937439e+01,     2.4815222008635490e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1779877145839903e+02,     8.3266559077080510e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0680452241836824e+02,     8.3307269417422012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1425690900772798e+00,     1.6062837941339214e+04,     2.7646008079384960e-04,     4.6323879356100805e-07,    -2.1443842410523998e-09, } , 
      {    1.6742272830892993e+01,     8.5236376569749900e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.4751145473712697e-01,     6.7514094274891249e+01,    -6.1658603963510780e-05,    -7.3934086369204231e-08,     3.4770837209175919e-10, } , 
      {    2.1333016555146479e+01,     1.4350573469905543e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.4344124976522457e+00,     7.8070333795688421e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0096155033351693e+01,     2.2064075220784580e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6514842215790097e+01,    -8.2799895590600518e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.2178086801204357e+01,     8.1306166872820022e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4379028141116898e+01,     8.4287295650610922e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0958979298668517e+01,     7.9548152793554855e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7972919554017505e+01,     2.3245767962179729e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.3884277999698810e+01,     1.7228602535831025e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3150574279778287e+00,     8.3260884007366458e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.7681434671201053e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2208012173356408e+00,     1.6577200756121943e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.7134676439463419e+01,     8.3283032829315907e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8128974997813422e+01,     8.3290795665186615e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3071572908018945e+01,     7.1824236762254932e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1872234859757569e+01,     2.3028521592085548e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.7380766456174442e+01,    -8.3408973495447135e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.3852854621357578e+00,     7.2065846988897756e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3669494279123427e+01,     7.2150292679065096e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {   -2.5976842646875212e+01,     9.5680335980493564e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1002404071860274e+00,     7.0934061626118910e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.8147660863591071e+00,     3.0454899795482641e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.4955790760964991e+01,     1.0037786207419400e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0700648550072636e+00,     7.9379304243821662e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2657851847636749e+01,     2.0101271572159378e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9868494920430795e+01,     5.7030162556918431e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2071105204013991e+01,     7.1721143949700054e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4307701338104330e+01,     1.0986538208048278e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9946606897456114e+00,     6.9829843905374531e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -4.9527910510486031e+01,     8.3164854999055406e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.7725586329266253e+01,     1.6661792076619320e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2072397448428575e+01,     2.4986785619642254e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.4014964976278685e+01,     6.6812248694345544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3431937436597039e+00,     2.4985362928708502e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.7745098873252005e+01,     1.7162831559264632e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5468199354800877e+01,     8.4073996570832405e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1025488666294381e+00,    -1.6638195179742026e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.6013272291399094e+01,     2.2826706933662765e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2200696278021059e+01,     1.4455355692981933e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5617188894144675e+01,     2.2502385715966891e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.7380476362320387e+01,     1.6871999387228243e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3978063918880583e+01,     1.6639750823174592e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1977486394200998e+01,     1.4978881054951085e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4997440859359141e+01,     7.2215410320278897e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1721301730137892e+00,     6.2860343152480646e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3453723441228828e+01,     6.5932334469823463e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8897136491415676e+00,     8.4798668234658326e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7519729013029774e+00,     9.1327857986668841e+03,    -2.5218992006656134e-04,    -2.2272757449937836e-07,     9.9847862438190027e-10, } , 
      {    1.4570845563976615e+01,     5.9236418874992632e+03,    -1.8747570515580554e-04,    -1.4958441316953623e-07,     7.0230109845527369e-10, } , 
      {    1.1549767182027626e-01,     2.0954946583072160e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.6666442903157201e+01,     1.6676570519158482e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -6.2253062088995055e+00,     1.0355929423162997e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0569620938011637e+00,     5.0136714016413398e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0981616748486573e+01,     1.6719515999423187e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7101462495259538e+00,     5.9213992789653357e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3747400900939017e+01,     7.7375957175890153e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.5094286043476584e+01,     6.5836777376295886e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3727873772771016e+01,     1.6756709644319340e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.6472173454738215e+00,     9.9629084534700087e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.2756204784392196e+01,     9.7826501358273472e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2480583509602157e+01,     1.6737564942778561e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8426188870070710e+01,     9.7999231325942037e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1544700494176752e+00,     8.4557217024118054e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2922875270979493e+00,     7.2457020546921722e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3840813407987884e+01,     1.3830985000426952e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.7143646548543813e+01,    -9.4436722999494941e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0849225486914158e+01,    -8.3406890352725022e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    6.6249427718903195e+00,     7.7996886891237494e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.0087160867903535e+01,    -9.4429676680333450e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.8389053305654393e+01,     6.9370895206518735e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8360009684859520e+01,     3.5034072997339226e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7062389720262640e+01,     3.1677767749011051e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7897707364784148e+00,    -8.3636363137351127e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.7618416415836762e+01,     1.5550232456753016e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0980186292268229e+01,     7.2147674973749827e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7137926291464240e+01,     3.2042402785293780e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.4793852406675576e+00,    -9.8363409536844083e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1656986481763067e+01,     1.7619192827163984e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.5611469346246960e+01,     8.3290460283232351e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6443692293275255e+01,     1.3224479658037770e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7132056468893357e+01,     2.3400872484203974e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3710061954760034e+01,     1.5959185486384737e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5887902584215521e+00,     1.3218409899528999e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.5759115325639282e+01,     5.9728668064069352e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6468015129669386e+01,     8.2698222734275187e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0295353721240048e+01,     7.2560113359476600e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1250691527342447e+00,     1.6100068569817106e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.9774098684116076e+01,     7.7007124721560322e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.3646046019893507e+00,     2.5161866692868654e+04,     5.5099462709043649e-05,     2.7747826224623181e-07,    -1.3197598027163788e-09, } , 
      {    6.9519365739117021e+00,     8.3997091105030413e+03,    -3.3191992975274604e-05,     3.2017095500473753e-08,    -1.5363745554361197e-10, } , 
      {    1.4050046898563851e+01,     2.3162956036535234e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.1246426924929871e+00,    -1.7632026275657509e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.6674983225348385e+01,     1.5153336116525890e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5984451860472751e+01,     2.2738792947441587e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.7180987421187627e+01,    -8.3283368211270172e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -7.3299808811385532e+01,     8.4464051619404363e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6436563580376280e+00,    -1.7429361443118366e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1218610731130040e+01,     2.3791263621580780e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.4954983845966474e+00,     1.5535276123614902e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9558105128149723e+01,     2.9671159055936398e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.5025013963038560e+00,     6.0687813838828351e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.8345568857797844e+01,     1.7522848877404846e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.8415944725962710e+01,     1.5971987365739940e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1769572795387626e+00,    -2.8624804763660013e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4163009743842071e+01,     7.0689477494407802e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.9706642865495393e+01,     7.1877544748322744e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2346217079277167e+01,     2.2836446959438617e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.8621600844816555e+01,     7.2133582335426827e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.4168211238256134e+00,     8.4248296592570587e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1730836273669345e+00,     9.2512453513444852e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7663055348288434e+00,    -8.3345914788115424e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.2066327319936384e+01,     1.5516445899557400e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8656860237273321e+01,     2.3700593449369197e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.4522165678187875e+00,    -1.6830883898878565e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    3.3751509524064019e+01,     5.9597488952074800e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0004512037444510e+01,     3.6124397884092768e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.1411004221707639e+00,     1.7284979089028919e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0975244413323253e+01,     3.2775475627468564e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.6475215518486035e+00,    -1.0438911389208115e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.5611306423734362e+01,     3.1611644456486581e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.8409752219060323e+01,     1.6029427580362784e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.8927052586583422e+00,     1.5533894134744496e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.1969757852057914e+01,     1.5551614445623421e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2546491816318294e+01,     1.5969556687379909e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.3929981674166427e+01,     7.7713704315412524e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.7249866980722715e+01,     7.7713838586427064e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.8786447017317151e+01,     8.1296169152355651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.9015243265295709e+01,     2.3871347663335247e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1125829803842734e+01,     7.2625142657401730e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0491933434683666e+01,    -1.9973675839877396e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6061485642899846e+01,     1.6210876777127414e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1080341801810252e+02,     2.3871543766482922e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.6225081345184567e+00,    -8.4929847346652859e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.6693288357501910e+00,     6.2830195524848227e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {   -5.6853028058270993e-01,    -5.9454490811103642e+02,    -2.8165420488846872e-05,    -3.7583435298764782e-08,     2.2829391429766939e-10, } , 
      {    1.5324414880508915e+01,     1.0764369503895176e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8390672191591086e+00,    -2.0584048529927004e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4276054487272686e+01,     2.3345309838790421e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9152373499730903e+01,     8.9407896744287646e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6540509177390334e+01,     6.5854266479368407e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5661168590500898e+00,     8.2654130462584471e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5701613195069914e+01,     2.2657490360773703e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1309571252671422e+01,     1.4948233762438442e+04,    -9.4451474075811953e-05,     2.6407122470882869e-08,    -7.9708217311218847e-11, } , 
      {    2.5437020985869108e+01,     1.9002390756408720e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3603003882305028e+01,     9.8669938759941124e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6238995155285174e+01,     1.4445757757193322e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0609901454464111e+01,     8.3850147573591457e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.0427078162150877e+01,    -8.3601716523883588e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.2570392309373709e+01,     8.7240135942578490e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4503160425988950e+00,    -1.3971962192534707e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0062989433771881e+00,     8.4852717933446565e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -3.1940023802097471e+01,    -1.1944470569373675e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4281883070537802e+01,     2.1988824159612188e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5437045320619356e+01,     1.5511115100950619e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8542949845012133e+00,    -1.6621334433047843e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.1051390927625720e+01,    -1.1359584672441454e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3114173169074554e+01,     2.8194676381531950e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9041942230727582e+01,     8.1091372935201616e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.4403960721163624e+00,     3.2160322237736302e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -6.9791064990420523e+01,     6.6012973087017181e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4426976405518209e+01,     1.5500805819695132e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8581590521448792e+00,     1.4443873474433305e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3079723670373991e+01,     1.8012213796856973e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.8047014086369355e+00,     1.5538345063014887e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4630857210913554e+01,     1.5764922994337059e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.3191260968564293e+01,     7.7003905532303088e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3058555680189270e+01,    -2.3314288274594674e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4245471993221779e+00,     8.3919698031918051e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5610536292006032e+01,     2.3285797945819246e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0234913171881130e+01,     1.7226171857470996e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.8811831853777115e+00,     9.7173037996348103e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.5616312062672030e+01,     1.1769853662952619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3231885691246372e+00,     5.9044785981565547e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3734351089638647e+01,    -7.8712361052194692e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.0488386798053835e+00,     5.9574476953677458e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1340928226108545e+01,     4.8739859906562815e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.3428217785180570e+00,     7.2403712560853910e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1038677395824639e+01,     7.0041610836700374e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    5.6315279926825724e+01,     7.0048657155861856e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -3.3244190113323732e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5288965581411023e+01,     8.2595129921720309e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9111919929943657e+01,     2.2149839478575101e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8355374783083910e+01,     1.4795001999444677e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8102426062110844e+01,     3.2239952041532124e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.8653210302135630e+00,     1.4246324218194961e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.3247850167962980e+00,     2.4086062252687079e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1012329696889527e+00,    -8.9907748074765732e+03,    -1.1872163480310021e-04,    -2.1410409659074682e-07,     1.0661973993592805e-09, } , 
      {    5.9793977956695585e+01,     8.5277659342146871e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2963601456291038e+00,     7.9629024027214955e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1991014036310487e+01,     1.4994628798466721e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2162336798563800e+01,     8.6621938173933140e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.4423945492106824e-01,     6.8341598069653000e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5808027271321237e+00,    -1.0593996959660124e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.7060099801237936e+00,     7.6790539317017310e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6210474883687642e+01,     1.0673699331683596e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6065274202531050e+00,    -1.0932986512884415e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.9822587672837191e+00,     8.2491309591718218e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.9918810248609722e+00,     1.5569062680810515e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2413492459111865e+01,     8.3264139083670179e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.1419133010143224e+01,     8.3309689410832343e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8162815189467832e+01,     7.5733024076488537e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.5253045052813853e+01,     1.0828803853707843e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5963990183864327e+01,     1.6902980731683383e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.3021013051027840e+01,     6.4291749117937588e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.5099376466807477e+01,     1.4199398071164836e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.1328657913098539e+01,     7.6251327785071953e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9026739453543726e+00,     8.4706667759557968e+03,    -2.2382863485054624e-04,    -1.8523140566321331e-07,     7.6873026904090223e-10, } , 
      {    1.4818055063870352e+01,     1.9015508667608177e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.9068300491690344e+00,     1.5713606555723845e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7180164892752416e+01,     6.3658877760999239e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3040029932744389e+01,     7.6047826353120872e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6189270161423458e+01,     1.6786251549075590e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.4637167315942188e+00,    -8.5447998461555762e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    7.8214565053535869e+00,    -8.5202166823546431e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.0450440689422530e+02,     1.6656874890912219e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.9822378125880752e+02,    -1.6657890807988286e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.6034134221991408e+01,     1.7602478681200664e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4596242461475139e+01,     1.6168432042913319e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7321031195728107e-01,     9.2255392661935866e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7810398720358759e+01,     2.0494292541852366e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5690599465313273e+00,     8.8503494698814102e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9448819290268492e+01,    -8.5156989854247913e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.9436589894775747e+01,     1.6934356194257212e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5909218718012735e+01,     7.2670224868839032e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8752907897550536e+01,     8.0530644262698788e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7142977115924964e+01,     1.4505677991929790e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.5616471753385419e+01,     8.3276277633001773e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6610881479139429e+01,     8.3297550861500749e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1979105547042419e+00,     1.7319441851836182e+04,     2.7113231780803260e-04,     4.6447157778933341e-07,    -2.2532636975559792e-09, } , 
      {    8.7119190678433398e+00,    -1.3241780250970151e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.9338944863645509e+00,    -1.0045890419515130e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.2622205083652275e+01,     1.4929403038276530e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3264830980226362e+01,     9.8232230424849513e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3035219194050146e+01,     1.8764474308739980e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0935480643186883e+00,     1.2949341856673480e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5531856694913152e+01,     1.5536854236097543e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8419596951588979e+01,     8.3263884898880024e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9782421151084492e+01,     3.8964282694269104e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2209005183488017e+01,     7.9219309160576804e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1636297873209458e+01,     1.5574393479417298e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7249997420761023e+00,     1.4868046039313071e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.2843433421952515e+00,     9.2724677184515331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3202011164456898e+01,     2.4225601679705933e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.6820944463577590e-01,     6.9323755842809333e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3388196884456844e+01,     1.4592419040857822e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.2709857970484325e+01,     1.5774643824728806e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0848079162925238e+01,     7.3019768516800104e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4156751901613317e+01,     9.6059701111197210e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0459296690504615e+01,     1.8366477632144528e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1965747612879882e+00,     8.2419409000109699e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.8592326442556715e+00,    -1.8231182376401448e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1613198907834175e+01,     1.8371453339046995e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3924335094450063e+01,     1.0082106435390760e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8101791401852129e+01,     3.2593158109327196e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.6656997615491761e+01,    -1.1301705875547232e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2231562764105988e+01,     1.2874090795501088e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3787484417831404e+01,     3.1478529550060906e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.3580234807342704e+00,     9.6359487690133610e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1664110662232634e+00,     1.2825388929836010e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1288695811677727e+01,     1.5565296195315015e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3456298015670676e+01,    -6.3973680037631530e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.1959952370458709e+00,     9.4786570549269545e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.2795680263882076e+01,     1.7817095218978659e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7817607955648349e+01,     1.4607501902568620e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.3518939180111570e+01,     3.8838861517258694e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    5.6283730558410854e+00,    -8.4154419494392823e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.4547922879655601e+00,    -1.4300972679471574e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2027039824161999e+01,     8.3375698131778645e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0457525021035682e+01,    -8.3198130362723878e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.9566132830824262e+01,     1.5403068879198900e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7316052233220089e+00,     7.7078620062486398e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2646826702898919e+01,     2.5004098482376583e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.9429121538229939e+01,     1.2023614506302445e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2387080475821492e+01,     9.4800797458607067e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2650235956444542e+01,     1.5584702760672786e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.6994494416488735e+00,     1.5531264302428312e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.4320901389695919e+01,     6.8246446918007632e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5010018172040219e+01,     2.2927669421182680e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5776597350891201e+01,     1.5554244277939606e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.9462836405283888e+00,     8.3460042874594546e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1292895319156209e+01,     1.0915544902635878e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3691657936469088e+00,     4.0698951079149329e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4457915584195405e+01,     1.9157495278432969e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2985061296081028e+01,     1.6099716253859029e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.7872978255779337e+00,     4.4629160776079198e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5652699747613390e+01,     8.5502385011032238e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0711415309460543e+01,     7.1430751898434974e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    5.4376155101390026e+01,     1.6100420885775182e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.6878354414145944e+00,     1.5654087978699048e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.0853219882792274e+01,     7.7000552072031342e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6229540649512170e+01,     3.3312464499060770e+04,     7.6156375320674127e-04,     1.2520555721494321e-06,    -5.9316953883751721e-09, } , 
      {   -1.6469006527486751e+00,    -5.2613587611866251e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8988740410080336e+01,     3.3350814115302914e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.4185530291466591e+01,     8.9569922962199453e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5683168743637119e+00,    -1.3433562190191219e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3618118795710746e+01,     2.9987800701910408e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5667496744170990e+01,     1.4288768952409057e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0510816025123795e+00,     7.4559084364111159e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.6095978255766866e+01,     1.8151156462450388e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.4613416299132851e+00,     7.2054263670754026e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.4803995380382581e+01,     1.6673115333597842e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.9921549924767099e+01,     3.2253106236845404e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.4634249209578609e-01,    -1.6663941805049977e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.4920006791232542e+01,     3.1607877970991078e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.5778168727848325e+01,     5.7428311575897551e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2480310218051244e+01,     2.4952292846672412e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    1.4885754381296354e+00,    -2.5019855701678345e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {   -7.4251118392255169e+00,     8.3309943595622499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.8156994816604843e+00,     5.4999923758620898e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0337457116051901e+01,     1.5030694180816286e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6811046230644884e+01,     1.7148619596458873e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.8137617007557578e+00,    -8.2038221769247798e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4848223001691466e+01,     1.3013776206486147e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4503072360994015e+01,     1.8405234766549958e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.0104355674558640e+01,     7.2347941712979718e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5750440806913332e+01,     1.1379559177741265e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.8474236434926858e+00,     7.3008039143868673e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.9542002655807149e+00,     1.5612644068203930e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3909866396808127e+01,     8.9566703772942201e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9883708765217605e+01,     9.1145276488838917e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2951001751681083e+01,    -1.1085950848713474e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -6.9001239925699537e+01,    -1.0990865309778637e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2663885607235862e+01,     3.2986179079020185e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.2459889843688448e+00,     8.1360027055885867e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.9685855546472141e+01,     1.0575406606015249e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7567948908065780e+00,     5.3717999262779067e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0178249522584789e+01,     1.7978432369354006e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8727913684309900e+00,     1.7789845607142468e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5724542616766044e+01,     3.1649381815600795e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.7451827026764217e+00,     8.3559138966283626e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.2231488549688127e+01,     1.5472864512163987e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7766693871497594e+00,     7.2014436271364539e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.1255207910272125e+00,    -8.1271862368755355e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5571169690738571e+01,     2.4979515318575654e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -3.8646096717130206e+01,     8.3257092764508652e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.5161780999181689e+00,     7.1948751957506056e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.5376279665226136e+01,     2.2094758153256866e+04,    -2.5119580961344623e-04,    -8.6166614282751386e-08,     4.4946591561983952e-10, } , 
      {    7.9383347364333758e+00,     9.5639766868383613e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9640395001917710e+01,     8.3316735729993870e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6720525297832179e+01,     1.1157390473588162e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7530057921754818e+01,     2.3316432635115667e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    6.1209834570829260e+00,    -8.7934194391373785e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6302536610282207e+01,     3.2147168042423018e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9751402102147722e+01,     7.2604635312841738e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.1254813175716198e+01,     1.3799465829914046e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.7955878986296412e+01,     1.4428119017366940e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.9507150364102593e+01,     1.4428132444468392e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.2450711589737480e+01,     8.3236243527778570e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.3013489093768978e+01,     7.0022121997883769e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.6870052955634790e+01,     1.9395411726101709e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.0047283513424773e+01,     1.4985792326508885e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.7284335971178479e+01,     1.4985087694592736e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.0056360793968654e+01,     2.3311830235436202e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    1.3827809461283287e+01,     7.1934443029478580e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6791878446634385e+00,    -8.5213801438616647e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.6550585400600532e+01,     3.0456848679364299e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.1882957524073912e+01,     7.9150003774575744e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1357083102932449e+01,     3.3317066898740231e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8751883710564591e+01,     9.3964206452889994e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.2763134868385446e+01,     5.2670069603492684e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7559943661875934e+01,     1.6148519970098445e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.2963813395866142e+00,    -1.1206620336612414e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.2337133712967812e+01,     1.3777276240741066e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6223678243790207e+01,     3.2197507307318025e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.3832739368854330e+01,     7.7717057775684279e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.0512858319143906e+01,     7.7710485126155309e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.4322302454529723e+00,    -1.2128443365819267e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.0627563453326561e+00,     6.3504686650607346e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2232371572194001e+01,    -1.8465474157201754e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9577189923212977e+01,     2.2075540473386056e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3107362744429951e+01,     8.3323893064273216e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7146273297658423e+00,     8.4514903658416915e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.5879832187534610e+01,     1.9550516248125958e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0698859450804516e+01,     8.4023145624537065e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1796101378167723e+01,     8.7811710373283131e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8905211340237763e+01,     1.3330417852460152e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.8761524816658447e+01,     8.3275631621785324e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0275261879706250e+01,     3.2202766971950397e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.6711404912906151e+01,     1.0469564311517821e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.0573224168597026e+01,     7.5879390108284724e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8194614359514421e+01,     1.5540612051914331e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.0239939157352213e+01,     5.5939855963781602e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2125950529691556e+00,    -9.4962890812026108e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.0221198966726330e+01,     1.6688035771759955e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.6683870741975229e+00,    -1.0405129961705152e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.6484418006713987e+01,     1.3737461338843153e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2723454866015160e+00,     8.6349713454900793e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4996659559423634e+01,     3.1243242934708349e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.6815973040499678e+01,    -8.3575418200721724e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.1427200927966041e+01,     3.2194188187162890e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.5414262093881810e+00,     7.2346814279698074e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.9462287134584251e+01,     9.4485888923556195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8080802826793567e+00,    -9.3903508867802284e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.7981752950335030e+00,     7.3591779814768852e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    9.9435195850691773e+00,     8.9393669834950124e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2974392763611739e+01,     8.5301966125747185e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4082997616788754e+01,     6.5688345996045082e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.2289586783940308e+01,     7.8510069488878034e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.8969490714156024e+01,     2.1540016887449616e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.8602874344967493e+01,     6.7589087457929118e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4632045761698294e-01,     7.0694464223178556e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2299318260609759e+01,     1.5129063242916405e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1550953091986642e+00,     1.4977725083605095e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1680416483090244e+01,    -9.6866264969595868e+03,    -5.9628979968913809e-05,    -1.3959725133867981e-07,     6.6332207849406703e-10, } , 
      {    8.3888851150745403e+00,     7.2815769597337239e+03,    -2.8015947582823997e-04,    -2.6039827626074286e-07,     1.2253180976362411e-09, } , 
      {    2.0778066431563712e+01,    -1.9126707082446460e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.6563407424629133e+01,    -7.6877646013572039e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.3735081754003893e+01,    -2.2647014663950279e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.7986858983215566e+00,     6.9765162807562501e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7620603373656294e+01,    -1.1123273594265593e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.8823842237376347e+00,     1.5544896528453584e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.1359598980817864e+00,     2.4828339919834947e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    9.7701544976977708e+00,     8.3041082609241930e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0548952164577486e+01,     1.0984192239243887e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2343290889323704e+02,     7.2120273484417576e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3242067243436146e+00,    -9.0502990142984308e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2579895945506601e+02,     1.5540718773166884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2193277288932062e+01,     1.6164502919629775e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.5268299894481110e+01,     1.4224005970119939e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1204773998202928e+01,     4.2894492391198983e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.9740437350537059e+00,     9.3204611087602570e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9755525787208089e+01,     8.3298196872717199e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8093180640846477e+01,     2.3977383909331467e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.5457921135444206e-01,    -3.2555014460589882e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0551406492439007e+01,     2.3951627808237390e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0106720462409687e+02,     7.2160983824759078e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8007990419570547e+01,     1.6630152887385982e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7470618164307194e+01,     3.1049460163965508e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.6638865330164194e+01,     1.4681461289625595e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9495860951852666e+01,    -9.0001202148830248e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.8722251615801795e+01,     1.5544789807201034e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1685992688415336e+01,    -1.7002763283561330e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7199955396107235e+01,     8.3363180083513962e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.4821884490030008e+00,     9.0345512707827929e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0101321657364846e+01,     2.1030801692139736e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.0360407965953588e+01,     3.1121556996770323e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.2254081689782796e+00,     8.3532745885260592e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.2056628202740924e+00,     8.3210648410988560e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8217009307950969e-01,     7.1883237303119104e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4951746650318913e+00,    -2.3285839134016760e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.7345130893784857e+00,     2.2757145788119244e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.6894323824494244e+01,     2.2756488523166339e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.5715579052920340e+01,     2.2381845129305177e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8290151099297773e+01,     7.0068145994678453e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.1688957931226316e+00,     5.8930133455550776e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1190194402547382e+01,     8.2580903012382787e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.4474528123141965e+01,     8.4065902600425234e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3010424665853456e+01,     3.1678824670487165e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3172325921368078e-01,    -1.6058653062749145e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.3229502466220193e+00,     8.2691728477003289e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.2828017555328440e+01,    -5.6595077801657476e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.2335525824070004e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5622444453681759e-01,     1.2432367960143023e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.1686166571080367e+01,    -1.9575526820898276e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1777903533599449e+01,     1.3802119345711857e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.2305023315662821e+01,    -1.4819123794374470e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4403158927644405e+01,     2.4790585367493486e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.9243990494859645e+01,     2.0887313511545352e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4380334999184660e+01,     8.2445802081132751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2474506366924163e+00,     4.1643119620077772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0088740920717932e+01,     2.4499753299954627e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7059937251448993e+01,     1.5229915413819279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7841861191724213e+01,     3.0064180025629386e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5252242704717764e+01,    -1.0785801428638833e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    4.6143723458230586e+00,     1.5555872201383414e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6178886232348901e+01,     7.8953395477155673e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5305494573858267e+00,     1.6029786609871586e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1499549884609049e+01,     8.7927056998305106e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3050006043805183e+01,     8.7423824719926779e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4536500093314402e+01,     2.8587697351224931e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.7519919723643891e+01,     5.9551464955280117e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3656439287376259e+01,     1.5585414106139662e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.2681051514789843e+00,     9.3136021583734528e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3856393985739892e+01,    -8.4377239134004813e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -5.1734409256846803e-02,     3.6768741382219459e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6453409361274083e+01,    -1.1458069627807777e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.6605898537032118e+01,     8.7804664054121640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.6055925122952587e+01,     1.0116711379299075e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5508629041722759e+01,     3.1085508580367918e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.2337622717106580e+00,     4.7320306302129484e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0083949183886919e+01,     1.6133825616954549e+04,     8.8389388047902374e-05,     2.4541753351445820e-07,    -1.1668495676944310e-09, } , 
      {    1.4869527364526714e+01,     6.4809561671324082e+03,    -2.0688939027668315e-06,     6.8853238991176307e-08,    -3.2967330315448447e-10, } , 
      {    3.7779012148291962e+01,     1.3879447949667947e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6675530797773334e+01,     1.1221824823400832e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2655520875467854e+00,     8.3195026368092149e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.3511074680773749e+00,     6.7500485903534491e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4101058688262302e+01,     8.3474269783932068e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2385369115108386e+01,     3.1021074230555252e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9237313537711657e+01,     1.8537711599498489e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.0953075258501848e+00,     1.4921005660738143e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.3886293387862374e+01,     1.5814458726626717e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7158669130030866e-01,    -8.4082518902784304e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.1823991118262294e+00,     5.1069714061691020e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3018512389749457e+01,     2.4165692056482898e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.4875404159485051e+00,    -4.8632097422075026e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.4005570522945774e+01,     7.8266379663567877e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.4489908869540429e+00,     5.4039359882436554e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5925753546323865e+01,     1.8798255736242947e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1773582951300909e+01,     8.7036634510627409e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4510234079131628e+01,     6.6218036968157003e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    5.1840763095842588e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0372295607655830e+02,     1.4938719578747898e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3297428234417474e+01,     1.3817091142638979e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1271832356691927e+01,     2.5143808628515810e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.7301745786452450e+01,     1.9943537217818946e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.7248848359051337e+01,     1.8111341560552471e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.9073421299613621e+01,     1.5542366148390423e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2799007831521871e+00,     8.9495208432016116e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.6150511026791747e+01,     1.5543142431977494e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5727219415303439e+00,     1.6860746694183149e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    7.2654736315980788e+00,    -1.1948106525946014e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.7767940118804653e+01,     1.5542548574956751e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.0861851142603818e+01,     1.4712317078690776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3108012421662320e+01,     1.4925467331860780e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7240067927647583e+01,     8.3020307181404933e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6113704547306167e+00,     1.5320585586030858e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0155070976930311e+01,    -2.7229962064268911e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0781802741445903e+00,     5.6757604141898473e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.1728521616305638e+00,    -7.2129593380664091e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.4138319072008080e+01,     1.5480397483154989e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4801321931874776e+01,     8.2911521814053704e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7468569638663816e+01,     7.1315795722907915e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5360644577788616e+00,     5.4842446323464510e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.0603470495968416e+01,     1.5343679780694398e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8345555719379533e+01,     2.4478423391976776e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.1423539709881908e+01,     7.2136747236652973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2581290858548357e+01,     7.4336169966637990e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8235101600985985e+01,     8.3553521313097590e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3789508232434841e+01,     7.2144510072523681e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7000365492324867e+01,     1.4434684686517390e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -9.5633482948674242e+01,     8.3667753581446123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7710509198706124e+00,     2.3865412240514135e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    9.2382042420134134e+00,     1.6029075264404710e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4784094826624962e+01,     8.2883510416224344e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.1804843072260109e+01,     1.7166245728802060e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1194062239503244e+00,     4.7783847862887842e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8009401056290457e+00,    -2.1593673405126142e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5436799414887759e+00,     8.4334361474779071e+03,    -6.6587244086354378e-05,    -4.3771888012655573e-09,    -3.4950218271186462e-11, } , 
      {    1.7869782258852450e+01,     2.2162272247531750e+04,    -3.1285441357695702e-04,    -1.6010070065195560e-07,     7.9717428771159877e-10, } , 
      {   -3.3981673316361906e+01,     1.0054354572270106e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4845308627316228e+01,     1.5542960005411169e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8587581322837256e+01,     5.3689550122201153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.7319841714510325e+01,     1.6560480048887570e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6053594528787229e+01,     3.1007956319355795e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.7212551620256377e+01,     1.0076896477401162e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5202088048814133e+01,     8.1758881025406117e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6746105177561272e+01,     1.4410101522716459e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5025454495955337e+01,     1.0260014845687099e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.9966385931922668e+00,     8.3166938141777500e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.6983500592540587e+01,     2.4093614419062185e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2280530974992164e+01,     1.8875525874528130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1283890273357216e+00,    -8.3298548862321477e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    5.1114068709246796e+00,     8.6959752448962790e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1201320124269072e+01,     2.5039043371386571e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.6718410194050758e+01,     8.9573276422471172e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.5258636733681215e+00,     1.3797516946032389e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0723576172689844e+01,     1.6363136507876437e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4185821038885578e+01,     5.6600279300422571e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4489691843678937e+01,     8.2102131214790679e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3732358190972620e+01,    -4.0613888089244210e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6841249529805054e+01,     9.1081744085113751e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0546287407620207e+00,     8.5912099877670389e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.3481182439332322e+01,     8.3509828644724457e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -3.1332878692572514e+01,    -8.3063999849778065e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.0266530668655918e+01,     6.6007116135514025e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4624937789446751e+01,     2.5772116213561356e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.4931335020949916e+00,     1.4348495927121841e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.5764748915648283e+00,     1.2352852621132345e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5574865087665312e+01,     7.1894797016578996e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1565473316497947e+01,     7.1980633571343469e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7434362528778934e+01,     9.0394823029387098e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6993246852713185e+01,    -8.3137709865630131e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    3.0017786213592771e+01,     2.3877479189304031e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.7642397765066789e+01,     1.1066720301376583e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0786145211265868e+01,     3.1574130754428403e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2469731277399479e+01,     8.4699666900830434e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.2300655387007193e+00,     2.4983444441859196e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2186538275778894e+01,     2.4988704106491561e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.3055800072323049e+01,     8.8951792329061373e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.7619238139938371e-01,     7.7521989511698721e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.6170932045945353e+01,     1.5522135727672983e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4118175362942594e+01,     1.6537938143756517e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4878491407445990e+01,    -6.0043470340701660e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.9412911655690497e+01,     1.5642081085053049e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.6401861525524509e+01,     2.4189215623429685e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.0705591237749502e+01,     9.7202933287983778e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8504124373513662e+00,     7.2139144484016188e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9856206868177528e+00,     6.7020027560911603e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0759891944758984e+00,     8.5356110582396650e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.0065903912130096e+00,     8.5408707228720323e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.4718306486444998e+01,     1.0216244012177940e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8301850386253722e+01,     1.9788432695794698e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2721255991399708e+01,     1.5631771803797561e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6516771559438595e+01,     7.2142112825160466e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1410359499299034e+01,    -1.6802497965468305e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.8181876303171447e+00,     8.3176707980027622e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6639740765898637e+01,     6.5519794827073847e+03,    -1.8490975602648117e-04,    -1.5015717205239902e-07,     7.5746804722872778e-10, } , 
      {    9.1738819393154358e+00,     8.3397120514474900e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7914094430609644e+01,     1.5518171126383024e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6147797227727914e+00,     1.2666061851012200e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0787406944657008e-01,     1.5605111097212928e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.9836669723078174e+01,     1.6776827555143987e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8231010188185600e+01,     1.7541640566151862e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.6361479637835579e+01,     2.3866514682753164e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.2627056901015372e+01,     4.6824702088128852e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6533569239146275e+00,     1.6291593827446623e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.3132032071938317e+01,     1.3406797176179134e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.1382793314998469e+00,     2.4463704883552218e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.7190323277570876e+01,     1.1772580147434252e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4167944620460410e+01,     8.4123039684598989e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    5.5114285136215022e+00,     1.4580385461666689e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1476440174255451e+00,     2.4933105176964185e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.7686519322604219e+00,     8.5912005119809182e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.9930223357469057e+00,    -1.6610982183624910e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -6.2254725250217120e+01,     8.3260379089273174e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.7814512378279375e+00,     2.4200032334789401e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.4952984975959946e+00,     8.2925830742081180e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6462615338324600e+01,     2.4465990623020123e+04,     1.1429004990681419e-04,     3.5194147426699899e-07,    -1.7233623441032565e-09, } , 
      {    7.0521842635986642e+00,     1.6204873312895046e+04,    -9.4549406439396088e-05,     2.6450755702182727e-08,    -7.8980996789554546e-11, } , 
      {    1.7361598466814424e+01,     8.3235027161055859e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6990256452473336e+00,    -8.2975130212106415e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.3654347254324236e+01,     1.3267111765194075e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6540102922105637e+00,     1.5716255339612269e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.6785082075207267e+00,     1.2039346990450036e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2440704867008421e+00,     2.3933802521938054e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.8344746011086936e+00,     6.1125522173919944e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4508095236199363e-01,     1.0969965329906358e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8144194182008160e+01,     1.1550411443281149e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4853489408305238e+00,     1.5243033325018734e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.0230699719470149e+00,     7.1603824227807645e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5497481200011105e-01,    -9.1485147916140650e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4705457363556745e+01,     1.5403034668258610e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.0851827770420599e+01,     1.2416635475995432e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4676285764401420e+01,     2.2694460348613822e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3273679105858522e+01,     1.6703783515275598e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.8776019585906035e+01,     1.7585411399661021e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3037247526193894e+01,     2.3068336493983465e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8450060298437840e+01,     7.4957501466367303e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.4356643449293571e+01,     7.4771308035181637e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.3102145759749755e+00,     1.1087285051798765e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6629171345299323e+00,     1.8383398312953308e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0955099056232434e+01,     1.4808508510716034e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2647154352299005e+01,     2.4968801277408522e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.6690686019660774e+01,     5.7826460594876680e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4824787242294361e+01,     7.2936233310121370e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8695293813908780e+00,    -7.4883953726266327e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.5460098011486238e+01,    -8.3296730865408554e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0838366002287067e+01,     1.6639333906094878e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9288702149830367e+01,     8.7523653167278171e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.0912959787635014e+01,     1.5563372852694934e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.9568171202602720e+00,     8.2152801934263371e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6240598996207837e+01,     1.9408529637301166e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.3718557398681686e+00,     8.6415303010898060e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7934284091663887e+01,     3.1102429261176701e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -8.1881618478709264e+00,     4.0526527364519607e+04,     5.4306288134201204e-04,     1.0655913822578935e-06,    -5.0540856628306906e-09, } , 
      {   -5.1893158839460895e+01,     7.6034810391169012e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1862778377095960e+01,     3.8628158065707066e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.7316655342865804e+01,     8.4246060022010242e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.6444437268640648e+00,     6.6114924299351242e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.3140858977919585e+01,     7.2081628113724155e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.3724858448009151e+01,     2.2805735571458008e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0450047655573913e+00,     7.3663225962078550e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9776904603985372e+01,     1.3800170461830197e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.5274029062515638e+01,     1.3660132734887064e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6140754702595622e+01,     2.9972096516522470e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1335472280916093e+01,    -1.3771471223082065e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.6092972167601467e+01,     8.3053849535633217e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6962233713551726e+01,     2.9306524019653669e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.7901941938009795e+01,     7.2184720926279033e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    8.2561318668881931e+00,    -9.4426086385245435e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.6876995646314530e+00,    -9.4440313294582957e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.7711291122113440e+01,     8.3436477578632403e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1356665833525327e+01,     6.2184714602421354e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.7682140409539464e+00,     6.1156436415344433e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2884560652328030e+01,     2.4482832619145847e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.6075959977284242e+00,     9.9176968403512956e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3038407797025236e+01,     9.2593498072520797e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5692855811858102e+01,     6.2133149972934916e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.7184839256930493e+00,     8.3137350915870120e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.2954531643369009e+01,     7.0149883559692717e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7912754342618511e+00,     7.2386460292597658e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9798060383196932e+00,     1.6951629191024069e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.5809395698370126e+01,    -8.4421026560239152e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.4992642176735114e+00,     8.2970522354917866e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.7607398166400145e+01,     8.3277097629093969e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0859177550180076e+01,    -8.3275878973327035e+03,    -7.0853318440046442e-04,    -9.0572407091202402e-07,     4.2822622825043086e-09, } , 
      {    1.0791232035108520e+01,     1.4945316177598261e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4497083355904479e+01,     3.0457553311280448e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.0121057390280598e+01,     7.2138571502316245e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.4871771624581225e+00,    -9.9345567310000406e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.8459750863501156e+00,     6.1266642682568081e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2849325983875342e+01,     2.8263978490233003e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8726955769968640e+01,     2.0336558187511935e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6283295394116539e+01,     7.2677433081369009e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0488267728289500e+01,     1.7896725022774481e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5446035273742268e+01,     2.4535801716357040e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3852998565500467e+00,     7.2162051037284573e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.0063741759224003e+00,     5.7352789912146454e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4936057125483804e+00,     8.3603306139584656e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4355234540074029e+01,     1.5718522328511714e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -7.2008153766089208e+01,     8.1521588203666215e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9432534996489075e+01,     1.5531937578824238e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    9.0559538699814706e+00,    -1.2712089278858236e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.8067013590613321e+00,     8.2550950953242500e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4825458154622133e+01,    -1.1315492400750745e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    6.8646391087523284e+01,     7.9392732510670567e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4313190323941356e+00,    -9.9656893746145797e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.9152187331597251e+01,     1.6799369460275044e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8146397245363609e+01,     2.3876376747065009e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9398380309839155e+01,     4.0531129764199068e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -4.0550443437164830e+01,     1.8445402804024201e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.8407455725416103e+01,    -3.1151933834067131e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.3889950964184855e+01,     6.1565222048573078e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1751489787725831e+01,     7.8086077735698691e+03,    -1.9033545137588231e-04,    -1.4888075459277388e-07,     6.4931581124681258e-10, } , 
      {   -7.2480731041657585e+00,     7.2052027100193691e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.9614016553061528e+01,     7.2229230208982945e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1920800674446461e+00,     8.6687527729930425e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.7241020275623029e-02,     7.2386755892333372e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0235073399619949e+01,     2.2129220916064125e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.0718157626082526e+01,     2.2739896474834004e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    4.5064263147367232e+00,     1.2613092753801007e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5994227142356024e+01,     2.3752001009215350e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0924167078509230e+01,     7.7886900078263079e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2147978811367224e+01,     1.4922073288764686e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0675650342489046e+01,     2.1280334481238340e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3951412582785352e+01,     1.5824441571361855e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    7.3249156285831887e+01,     8.3313449405229348e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6187357628762227e+00,    -9.5131498278967938e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4997372713594148e+01,     1.1867253920711139e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.3763577067853099e+00,    -8.4022877541260023e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.3756769813357202e+01,    -8.5555138873227042e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.8061153502636024e+01,     1.5074482259318582e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2257178418607893e+01,     2.3696644079029007e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.7314360965726664e+01,     9.3862320853266501e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1893579716876332e+01,     6.5233984733843208e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1076067859958551e+01,    -1.3499151746188491e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1642472069980739e+01,     7.6304940616596105e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4621788305062822e+01,     2.5008616179306435e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0926843983214916e+01,     2.2287416862230824e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.1133051652631906e+02,    -8.3304223932035529e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2349826613082516e+02,     7.2157938339372595e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4591142822900274e+00,     1.5489073847505892e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1982620195829895e+01,     1.4314168059711651e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6773809897872560e-01,    -3.6485224157519751e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.6458222213669540e+00,     6.6659373737415963e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4457869467905731e+00,     1.5682439701169018e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.1685294713593350e+00,     1.5567337453984892e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5219753044537070e+00,     9.9487686938057996e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4365969249734761e+01,     7.3507805679194253e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0968718754810673e+00,     1.0577076278851342e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.4189307746165403e-01,     6.0273374733877190e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7348433879324773e+01,     1.9191276705935936e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1657273470117840e+01,     9.6078484999472676e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0899765801374048e+01,    -1.8749333491304342e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.4241495575289296e+01,     3.9413847689134964e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    6.6823582833410910e+01,     3.9414552321051124e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.1214209269849977e+01,     1.4198661082479293e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5278539728044663e+01,     8.3267270770544601e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.5944160655883088e+00,     4.7139504364761151e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6459178439579114e+00,     7.1158470881431995e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.0655224976033690e+01,     2.2467921296428358e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.4977573128738207e+01,     2.3796194653736697e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6695433613814672e+01,     1.6083147889008324e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0430086907947898e+01,    -8.4995436902650126e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.4735761845949611e+00,     3.2838531685289581e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.1254500248129176e+01,     6.8366451523188516e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5056023096416382e+01,     2.3636159099556531e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7944303279841417e+01,     1.6408978322898241e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6457565836986170e+00,    -1.3639929269949273e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3514600430931457e+01,     3.7990305849353263e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7112381936798077e+01,     2.2128509570597249e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.0189919513303185e+02,     1.5541023321705530e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0404883948295552e+01,     1.9503719748212799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8935142133348544e+00,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3938407618906830e+00,     8.2867429542362988e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8980756648637513e+01,     7.2119206271892062e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.2481181533567602e+01,     1.3834763034882106e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.2476209892207066e+01,     1.5019673554093923e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2356731652315283e+00,     7.0727876001009163e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7367862028565320e+01,    -1.6641650365302659e+04,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    2.2715042829540636e+01,     1.1308565872328865e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6953713748936128e+01,    -1.0031812667139051e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1768382271210999e+01,     2.3970772509778173e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.4405894199129762e+01,     1.3498673398216670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.8202318148280048e+00,     6.6490336588799682e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5607239882381375e+01,     6.0530488997352422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4911200203047201e-01,     2.3862809216525657e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.4579019173410554e+01,     7.7153401728176959e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9323897504331207e+01,     3.2357871493974639e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.7917122279721518e+00,    -1.7403063119956523e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5973436007868550e+01,    -4.9310321559205477e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0026108916866789e+00,     1.7442270803115323e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    4.6008220077962285e+00,     8.3706398952139534e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8097254234360555e+01,     1.1614845793093820e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3525882654527566e-01,     4.3853638165957964e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0500590892475543e+01,     3.1569528354748938e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.4674098013477893e+01,     6.9012239890941537e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8674831145001811e+01,     1.4909515672982496e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.4356648358081415e+01,     8.3271661714739075e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5351065068599709e+01,     8.3302166779763429e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7835085398684072e+01,     1.5541690628759010e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7746492456634087e+01,     1.7363687321387035e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6214535463256489e+01,     1.5701661581817532e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8651843505585305e+01,     3.0652922309702717e+01,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -3.2537577138910230e+00,     9.0193558167996380e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2355376087944002e+01,     1.5553571001543680e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.4914327544600361e+01,     1.5543817951608908e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9571609448156764e+01,     6.9769044225497855e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2732958421436832e+01,     1.7285690434495795e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8894150586962418e+01,     1.7738170047098985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4953918042827787e+01,     1.7654780494428928e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6685726946084234e+01,     1.4617026939812928e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2507462884293545e+01,     3.0536830799118197e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.0992323697724160e+00,    -1.3268078574131996e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.2952377928209016e+00,     1.5976106167193519e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3152238438882645e+02,     8.3282410571932087e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4251667884050983e+02,     8.3291417922570417e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6213988541205815e+00,     8.0907749770563860e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4582971925656118e+01,     2.3801555936889112e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5465639704391151e+01,    -1.3215481927808314e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.5454160948451467e+01,     7.7024613824632834e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0022699589988200e+00,     7.4624673920108435e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0271815679326718e+01,     7.2129992040338821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5714616006256827e+00,     1.6071735080360413e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.0029898772606025e+01,    -1.6672801960489443e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    9.3297343109369457e-01,     1.4901827466816037e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.4049343977289464e+01,     5.0754911785058730e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.3728410865806673e+00,    -5.2355723249570110e+02,    -2.1623611323479412e-04,    -2.5540469534531464e-07,     1.2058285876556379e-09, } , 
      {    1.5239293639819614e+01,     2.4533504717295014e+04,     5.2631445943303407e-05,     2.7800738789779477e-07,    -1.3756539720114973e-09, } , 
      {    3.0044266793173488e+01,     3.9380442957955602e+04,     5.0765810176341731e-05,     4.1527200669118408e-07,    -1.9776519679820193e-09, } , 
      {    6.9659318529053564e+00,     1.7461477223392008e+04,    -9.9877169425213123e-05,     2.7683539930508054e-08,    -1.8786045329313413e-10, } , 
      {    1.4450177025703287e+01,     1.4427414385450789e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.2640155769481183e+01,     7.2151265268837842e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1432340658172264e+00,     6.4967377667996880e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3767006013203940e+00,    -8.3985812027450975e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.4583958530452470e+01,     2.3889494658264455e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.1007373922149188e+00,     1.1646326020757049e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.7658814453069382e+01,     8.7181095983831947e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9044205907630548e+01,     1.4907887749538688e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.3997153852931461e+01,     1.4428837076384541e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6214668573917415e+01,     8.5379122580793992e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    3.5959378286045130e+01,     2.8980718320917920e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.2125337896020483e+00,     1.3657831535047329e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5152585394307238e+01,     2.3960463228522683e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3262789821136060e+01,     1.4173694291241762e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8133985895061301e+01,     1.0862585281210808e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4275514393047599e+00,     8.2750109820470589e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3883376189045991e+01,     6.1170663324681964e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9292341319661750e+00,    -1.5852017899714049e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1329453741278890e+00,     5.6431786111198778e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5822992261056045e+01,     6.4092682336792222e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -5.1702515601849420e+01,     1.8405587902126288e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0066980254804623e+00,     7.9291209752185314e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8131097790953884e+01,     8.5385695230322963e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    9.5446948509788303e+00,     8.3823718674031934e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.4338707293279015e+01,     9.4087739904577065e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8469627976169512e+01,     1.5596434732862026e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7612856620960272e+01,     1.7179272518768579e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.4172729363145216e+00,     1.5314026630431132e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.2382036789467005e+01,     1.5543850171151933e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.9459089382866580e+01,     1.5541658409215985e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5699677618212000e+00,     2.3011248595318692e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5168551473236118e+01,     2.6735945268387886e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7015912270432082e+01,     2.2756105810175915e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.6684566835757913e+01,     7.5567606073139887e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6233010947722562e+01,     1.5378460980243799e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.0444726957806946e+01,     9.8622799396231730e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5450842234966107e+00,     1.6473911345587985e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.0734539178006017e+01,     1.5542902707241172e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3250882990316807e+01,    -1.2843631231991028e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.0706010110599827e+01,     6.4663105747195532e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8542474304566845e-01,     1.4475025069620086e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.5609337819162992e+00,     4.6400665825343822e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.2251806869311963e+01,     1.4932495648493790e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3452886436692495e+00,     1.5542605873126746e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1846190140192309e+01,     3.5660314860278413e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.9967005099578316e+01,     1.4507755534713489e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0428160620566348e+02,     7.2123318969804059e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.0131629296443337e+02,     6.6100281540227716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1969475253735826e+00,     6.3861686630103100e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6571870887361353e+01,     2.4340846008321052e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7997369391596902e+01,     2.9906445671288955e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3912297911848462e+01,     7.7905553390140849e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1625553507948336e+01,    -1.5747743515638831e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3497936529511758e+00,     7.8166313153574538e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4714806787521670e+00,     1.0873985972020246e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9871425906287293e+01,     7.8887711163297172e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.4624640465802440e+01,     2.8155811376635811e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3800789723227044e+01,     7.7901126987600610e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2820957295386002e+01,     1.0094169474168018e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0393974810480184e+01,     1.6266621849107292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5884000423497380e+00,     7.8056106886350890e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3281528518899265e+01,     1.0683014473273726e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1823288444111363e+01,     1.4371458752004279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1417914249381766e+00,     1.6674243596144435e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.9206705335122667e+00,     1.4939029868939348e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.5522892358610783e-01,     5.9743683761765278e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3974650451670954e+01,     1.4151152386110707e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6132714686476930e+01,     2.2757528501109668e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4706115151258359e+01,     6.8003718181912809e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7155562648466681e+01,     1.7897077338732557e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3198172337675203e+01,     1.6641635105934613e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9734308481931627e+01,     2.0181453665487683e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1150845945148944e+01,     1.4525028531480346e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1345682900274726e+01,     2.5508443664798539e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2995775216359009e+01,     9.0136659886840553e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3980695837154384e+01,     6.8176448149581365e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3072905999929063e+01,     3.1068587899559130e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -2.1383571796920883e+01,     4.6827203086537702e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7376539337180350e+01,     1.5901993832373981e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.1900002446336565e+01,     3.1163060841380040e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5576816479796088e+01,     9.9989910808127097e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.2383034601361310e+01,     8.3116241431854796e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6552284659515184e+00,     6.6452738574380883e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0754836286695257e+01,     6.5967759071356531e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8705488975214912e+00,     3.9609654580288402e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7585944512051846e+01,     1.5507809401173970e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.1054933354074855e+01,     7.2025728777031854e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.3420876802970618e+01,     7.2255528532144799e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.0148792122126352e+01,     2.0729579157204920e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6508685635066524e+01,     1.7279131478896066e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.5088835562641600e+00,    -7.7353287286895684e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -6.3914868710783615e+01,     9.7428352339294343e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4339516717287619e+01,     7.7544564642831965e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -3.1389016954804408e+01,     8.3457587062647726e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4745549322080276e+00,     1.6673382357774739e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.5958836359767631e+01,     8.3250886286902096e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9783135226120234e+00,     1.6080890375895000e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.2762846382330517e+00,     1.5294848436509023e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    6.6953199463642591e+01,     8.3322942207600445e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9989258933512922e+01,     8.6126799057427634e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.7612530357999407e+00,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9329052010674907e+01,     8.2396491759573564e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.7158775253267731e+00,     7.2206742650121250e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4171826669441572e+01,     2.7697334480695918e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5347637541914532e+01,     1.4012316132320630e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2460932297379759e+00,    -1.0798150931398139e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.2403769425492413e+01,    -1.8048943355373922e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0278950638043955e+01,     1.8017609290949677e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.0273080063306683e+01,     1.5547163517353030e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4390335064248273e+01,     1.6406348490582059e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1082723613747127e+00,    -9.4367610283916911e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -4.0471069127757090e+01,     8.3245726232715479e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6548844456564202e+00,     5.2827394444968595e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3604565241692712e+00,     7.4121376029019566e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.3734112533579221e+00,     8.5666078723938663e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2319734795473044e+01,    -6.8679412344682642e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.3604642680616426e+00,     8.7397460784054401e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.5484637390483353e+00,     1.4208586859080748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4366779252148124e+01,     2.3240508297547356e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4334576259485496e+01,     7.5295315741396971e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6439532799180935e+01,     3.9021179035400055e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.5522509286340359e+01,     2.4985745641698926e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.2375080582637796e+00,     7.7357209106151167e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3276522411357849e+01,     1.6117381432551434e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4271477003138507e+01,     2.4333768793303152e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2216056854961481e+01,     9.1945455944732530e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9569363994195470e+01,     1.1943432412974136e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6301029894239427e+01,    -5.6113260643771773e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7921800240912610e+01,     1.6931366665093639e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.7553056497181988e+01,     2.4397581591027749e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.5106115100734929e+01,     2.4986402906651831e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -3.1117453084990736e+01,    -8.3290102282761418e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.3908851650037359e+00,     7.6654389506695970e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7171970331202857e+01,     8.3263035917499747e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.6177770423611292e+01,     8.3310792577002758e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4644358942626354e+00,     9.3242828299418215e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.0226455777874918e+01,     8.3282269627883397e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8625977604422641e+01,     1.2165601117127240e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9652329789748684e+01,     1.7934661535844851e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5155543953519368e+01,    -7.4782151355264823e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.9547944530904342e+01,     8.3283726211741105e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.0325538113450001e+00,     1.7540121874171702e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.1005265973193275e+01,     2.5994284917714453e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1557744457940469e+01,    -5.3813608571104010e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.6484914084270450e+00,     2.2754739612859088e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.0693922445440300e+00,     1.6797068260435313e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.5229068865310929e+00,     1.1253305051064061e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.1386575370856082e+00,     1.6759554558377135e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2174947355035268e+01,     7.7783807265708210e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9598617514269058e+01,     2.5601263948021468e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.4568170695397527e-01,     5.8062722966270421e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8656825326934708e+01,    -8.3291558866619162e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.5660913836993437e+01,     8.2500280233769809e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.1233545288627610e+01,     7.0717239386759657e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1511393976487241e+01,     2.2758894698426495e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.4679472965564511e+00,     8.0158525483604471e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0906834611605809e+01,    -8.4177336734928977e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3293733987680156e+01,     1.3249062821838703e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.0343822651022645e+00,    -1.0803546425490846e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.2366814084095772e+01,    -8.2483293168041200e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6779973252519099e+01,     2.2542200617864801e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.3361643207010779e+01,     1.5089248341018496e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2477240737757267e+01,     8.2330902203576297e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1608860906711495e+01,     8.3347517074477764e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    3.7828818078710093e+01,     8.3288409105091141e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.6259301587796095e+01,    -8.3285419389411345e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -5.1081919327764993e+00,     1.4896397761783042e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8588301389175303e+01,     2.3707152404968925e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.3917370304668228e+00,     1.7259953284973963e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.6680055256324726e+00,     2.4435318950141958e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.4443498468376447e+01,     2.2462570814068975e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.1706770833118050e+00,     1.5516093583599324e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3442633794121079e+01,     1.6657376135899525e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6448032103607460e+01,     1.0653035815380086e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.7737205572695686e+01,     1.6657389563000979e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4909518254593795e+01,     1.1175028323487672e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8473852245765961e+01,     6.8390908391212179e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5471628008503501e+01,     5.4685121481988608e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -7.8850446667419583e-02,     6.9661570117838974e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -7.5633019376100805e+01,     6.2831429859626996e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1871531619106319e+00,     9.0606102811790079e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.0012428491160916e+01,     4.3209294667831273e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6612057054333363e+01,     3.0666154965317764e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2073986844505860e+01,     2.6398130993358245e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.0823539783546998e+00,     1.1926254461575309e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1519402805755075e+01,     7.1874021588741998e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4485913489637937e+01,     7.8821853524022881e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1815424462226749e+01,     8.2801315546830065e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9074874640831627e+01,     1.1459741271069572e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2105369187186973e+01,     2.1673355450931325e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.5623779570053209e-01,     6.5393546145879463e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.3309014065130626e+01,     8.2734306034603196e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    7.9478949929754048e+00,     7.5269017418235126e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.2345174018849807e+00,    -7.2117958765593867e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1911095095528845e+01,     1.5016618414065297e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2156232999378130e+01,    -7.7173045204883647e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2255676545733516e+01,     1.2649141170203418e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8771009089252800e+01,     1.9584297675628924e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.4769926807214109e+00,    -9.6010543383318509e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.0585931975716711e+00,     1.6724896943725144e+04,     2.4296689731918570e-04,     4.2688814249056862e-07,    -2.0249697832583097e-09, } , 
      {   -7.5052172214269470e+00,    -1.6589868755175361e+04,    -3.6628410524620729e-04,    -5.7475631522897706e-07,     2.7203865274418282e-09, } , 
      {    2.2537761532818170e+01,     3.9447957052230486e+04,    -1.0892793787169035e-05,     3.4133792032197986e-07,    -1.6299435958902598e-09, } , 
      {    6.4863416840352706e+00,    -6.6211902271108738e+02,     2.8361285216015134e-05,     3.7496168836165066e-08,    -2.2974835534099799e-10, } , 
      {    1.6993093112070067e+01,     1.4951682963413365e+04,    -2.2076563049466428e-04,    -1.1752368443776262e-07,     5.4939086343332605e-10, } , 
      {    1.0736873655326438e+00,     6.1992932663200299e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7030634579802189e+01,     6.4460698694282310e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6403907248919964e+01,     7.8977273806907178e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2579640297623614e+01,     1.4190579146215083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3760322283995229e+01,     2.4380308594260892e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1460362087333424e+01,     8.8728877931588177e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5494255294540753e+01,     1.4918259996951552e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2508824072593818e+01,     1.6655240611180630e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    6.2281984371760100e+00,    -1.6659525087719878e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.0492103194140119e+01,    -1.6670500760649709e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    4.8325820622238552e+01,    -1.2894301951463722e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0839255421695404e+01,     1.7861028922330144e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0767697370082741e+01,    -7.8892697892067954e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.2274515071861035e+01,     1.2809656445688421e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6710788414194237e+01,     2.4047213753236840e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.6157610774512577e+01,     1.0609264981870925e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9177481376688952e+01,     1.5551390788567387e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.4566313823202695e+00,     2.2534648451489691e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5683892946827813e+01,     1.2619238322379260e+01,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.3577110489413675e+00,     7.5944456452572003e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7824400263204964e+01,     2.4582236593122674e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2214300389868693e+01,     7.8397406181881915e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0354845070924744e+01,     1.4434025785004082e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2850897186329338e+00,    -7.0585984605766726e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9396770931821332e-01,     8.3249249392296242e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5852990642778604e+00,     8.9420426766325545e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5637515666222804e+01,     9.6640244767877448e-01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.2330879634426024e+00,     6.2522528877451005e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.1951384773048964e+01,     1.6170733242753055e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0941401647357285e+00,     1.4676530257469676e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3276274052856550e+01,    -9.3115705540402757e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.4882172993870064e+01,    -8.3839522459899326e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -1.1633631395724564e+01,     9.2882002025991242e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.1346604217560730e+00,    -4.2680239719595193e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.4053383659274647e+00,    -1.7827510462097489e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.6294850428776876e+01,     2.1874078130921342e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.0902189537006901e+00,    -4.0415433854449634e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6158041550427289e+01,     1.6118804123485186e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4521952581213768e+01,    -5.5766659559122161e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.1343320222479321e+00,     1.7243030618540091e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.6490485048026713e+01,    -5.5696196367507184e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4624699180186662e+01,     2.2198399348617218e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1987834912030417e+01,     9.5011953782043602e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2710670086786223e+01,     8.6218534342906205e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0127156998635160e+01,     6.6909821293687955e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.7931401648018337e+00,    -8.3341211297581212e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    2.8110650139503434e+01,     1.5010047909505913e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.8956306588890328e+00,     2.8908321988359694e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8035218735518917e+01,     2.3165585868851416e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.2492426118564140e+01,     7.2142685806860409e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3669881405362528e+01,     2.2131139402913432e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.4642717681323090e+00,     8.0656234866657942e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.5354019167914821e+00,     2.2769935066842248e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.8843203442253433e+01,     4.2923307555598194e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3722763132380631e+01,    -8.3449007600670302e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.2004677037859420e+01,     1.8994785217190070e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4560148295749766e+01,     1.5383846998550385e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5005547928614300e+00,     3.9923428469028086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0169647719021551e+01,     5.3931298342557584e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4643839792867517e+01,     6.9274193485821206e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6813120112026127e+01,     8.3284061970387847e+03,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.0171664683555562e+01,     9.7894416149819881e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.0201806440114858e+00,     2.3305494936892512e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.4628151706096997e+00,     1.3419915087378589e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8382636602944217e+01,    -8.3289766524114675e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7663206648294739e+01,     1.9801550606994155e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0479713744005409e+01,     7.1779802381161317e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    4.1864999705185308e+00,     1.1190377864353643e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2712025466809960e+01,     1.5577699179193945e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.2427281494761196e+01,     2.6387305887407441e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0404365653689736e+01,    -1.0344464659379855e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7248383830607459e+01,     1.0339477930609096e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.2101889084918724e+00,     3.1005878776572092e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.5515868687678406e+01,     7.7246567132890650e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1472821629602919e+01,     7.9176215116767635e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3889581316882357e+01,     7.2407235720434655e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.8491468736702128e+00,     6.2058522219197566e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1686972328540572e+01,     2.2088694668699336e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.9489088234429897e+00,    -9.8106038041625725e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.9520065337265692e+01,     1.2007866762786809e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3714439549503581e+00,    -1.0988808157506548e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0861837494353853e+01,     1.5530302770658844e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    6.5442113565538680e+00,     6.1151820497081771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9792375557163496e+01,     6.5933818640395602e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2411061313350547e+01,     2.2778147063620643e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.5451041621463428e+00,    -1.9988862759679018e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6252141698180338e+01,     1.5569414996768592e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7962962703533094e+00,     2.3853396771553711e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.9450885962631865e+00,     1.0860284081371074e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.6494758834085665e+00,     8.8407515340927985e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9961165666256647e+01,     8.6129300055836393e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5046344426396303e+01,     1.7188787633933116e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0761168647945558e+00,    -9.5415357613070519e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0280388898532983e+01,    -7.8117175281946711e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.2189134733862666e+01,     1.5625237583352000e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2897705124923505e+02,     1.5544485258662386e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.7493979146652478e+01,     2.2774090152409648e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.6984021965127013e+01,     1.5541942244198479e+04,     9.2781703036018566e-05,     1.1077022985990678e-07,    -5.2374421970263164e-10, } , 
      {   -4.9906969371727442e+01,     1.5543566336169431e+04,    -2.2515794548278047e-04,     1.7123619216788808e-08,    -9.3714484558473377e-11, } , 
      {    1.6757688919552258e+01,     1.3821148053849975e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.2156975281462232e+00,     7.8231827807961490e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2341098122025000e+01,     8.7681130602434569e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.6106367352844408e+01,     9.9020948415210842e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1271355165650608e+01,     1.4402386595770742e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2451724088793725e+01,     8.2404251225458738e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2318210104264026e+01,     3.1149942930180583e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    5.1268912841009296e+01,     2.2247954276290984e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.7140384200761510e+01,     2.2774866098998165e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9582733890935291e+00,     8.4059769661494865e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2490635309124829e+00,     7.7636622181462617e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.1667031587065892e+01,     1.0371142818974979e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1571319215881424e+01,     2.1122600126897909e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.9503858037015561e+01,     1.7483033975506929e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8856797116213873e+00,     2.5574566957323008e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -7.1041599749502168e+01,     9.0747127469859788e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9451740043593357e+00,     1.4283509287776687e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.5115562823082932e+01,     1.0014539670372194e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1144591580333804e+01,     1.3679161443025181e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2858686115837507e+01,    -1.9177377801919156e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.1172687362164027e+00,    -1.7091734673773299e+03,    -3.9897904299492429e-04,    -4.7445873962018985e-07,     2.2922427175171860e-09, } , 
      {    2.7731836855471911e+01,     3.0423449577981988e+04,    -9.8883058972098006e-05,     1.6424450014713503e-07,    -7.3687316205519493e-10, } , 
      {    1.7232863936428508e+01,     2.4013481086464915e+04,    -2.8488485781527840e-04,    -1.2242999889059115e-07,     5.7033481445725793e-10, } , 
      {    4.4251605451333816e+01,     1.6482581213570171e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7405846995383280e+02,     8.3146769549157361e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6620576815573918e+02,    -8.3427058945345143e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.0651697013444746e+02,     7.2137172630514833e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2888291358334322e+02,     7.2144084678661811e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3915864511612162e+00,     1.6440129918085022e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4627442034823318e+01,    -2.2916139255615553e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.0833690923111421e+00,    -9.9940753080248396e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1370789587279340e+01,    -1.1020093209439142e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    1.7677077473415657e+01,     8.3865668189380667e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -4.9581212579760276e+01,     1.5932172463842027e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8208818310674243e+01,     1.7500306972273786e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1363531662553301e+01,     8.5257893710307344e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4528994805101100e+01,     1.6107072151295946e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0958737880804721e+00,     7.7114044664019466e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3081812526066864e+01,     2.2039492056983647e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.7128144838603525e+01,     2.4721922004107728e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.9553667562355130e+01,     1.4919377737294337e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6549916151884769e+00,    -8.5720223180588146e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    4.1998125719626724e+00,    -9.4095385564884546e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    4.9122347811298663e+00,    -1.9186985528425330e+03,     3.3591115838248028e-05,     3.6307017839139590e-08,    -1.2014167831575411e-10, } , 
      {    2.5978765354367638e+01,     1.6157943964030048e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.9144071099960289e+00,    -9.7058385470333324e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.4238713093863904e+00,     8.8517721608151624e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.0513994486242979e+01,     1.0410957720872890e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3517759828454992e+00,     2.3858826476586706e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.0950633192515884e+01,     2.6136751249844081e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.2807770195767347e+01,     8.2162465958740158e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2969729856748220e+00,    -3.0797334077312307e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.1824857691579851e+01,     8.3263919598827797e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1164701481687242e+01,     2.0574474635180668e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8081098469953805e+00,    -8.6753117285927692e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.3810717122888377e+01,     6.5943917787967175e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1257216045534587e+01,    -8.4389140956867577e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.1129738636507192e+00,    -8.3353028242784185e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.6797933923147603e+00,     4.9656780113326167e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0725273419075180e+01,     7.0167193244476975e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5410871794254891e+01,     8.7059646509024751e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.2701427931421545e+00,     8.4646445611549716e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.7097429912134441e+00,    -8.1982336688893952e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4476324258485388e+01,     1.6735379736341489e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.3418625457628181e+01,     2.3736458980926916e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.8412216706887214e+01,     1.0348600913843922e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0631253679728562e+02,     8.3299610141837165e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3490973876830083e+01,     1.5526544954842055e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6500123357874511e+01,     1.0666093792464942e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0730868924276049e+01,     2.2735487247664940e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.8927758918406932e+00,    -9.5999003526109500e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.0706452533567319e+01,     9.0279923151830662e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6893862893899762e+01,     5.8615331178918477e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.1338476115715554e+00,     8.1305833176832439e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4312893324228355e+01,     2.2528089495889959e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.4735130736614442e+01,     7.7692996023082778e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6707556842855379e+01,    -1.2881296086946049e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {   -2.6163321181745872e+01,    -1.2340732649600302e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.3912375164585978e+01,     1.4914805734647218e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2565438797324298e+01,     6.6428431790780578e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0360154184017823e+01,     8.3297960164544438e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7381612972814082e+01,     2.9373739290610909e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.3658918486090030e+00,     8.3275868329958084e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1415310849408506e+01,     7.7734546878756801e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.5006734536684831e+01,     1.8739649145598014e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.6156899303253297e+01,     1.8699834243700105e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.8548947060131439e+01,     1.6832184485330334e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.3672304326480926e+00,     1.0467263111678087e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5460062726612952e+01,     3.0996491066754319e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2638252730082693e+01,     7.9820711208574830e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.4796053433636999e+01,     3.1085515293918645e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1287574769952794e+01,     1.6171056245432441e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {    1.0000732854013664e+01,     1.4441243642117121e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8816617416335575e+00,    -1.0175238840445301e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -8.4445085029815445e+01,     8.9327261361611872e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0233560529331696e-01,     8.9729985180951553e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5693985688124744e+01,     2.0403559096373265e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0211489493028704e+01,     9.6812211722921111e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4194932593005046e+01,     1.6894929434152837e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0266697598394559e+02,     3.1085501866817191e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.2385436126829845e+01,     1.5449454285656197e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1705729250433677e+01,     3.3297845018091728e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.5111590242371328e+01,     1.5460270997015918e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.9531853239397904e+01,     2.4500105615912700e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9845984499088044e+02,    -8.3274218352665357e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.2856001741255096e+01,     1.3968541098505921e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2082162157208018e+01,     2.6172799666246494e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.7459772402596538e+00,     5.8538364239594066e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.6254111099710773e+00,     1.0217170892688068e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.1403108026464807e+01,     7.1653609997937565e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.8551201531521301e+01,     2.3167887068691154e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6469406350980069e+00,     1.3816027481214027e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4338277587239265e+01,     2.1184733276870844e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.3329421180699498e+01,     2.2756719104068950e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1648923430697445e+02,     2.2756915207216633e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.5519975447870195e+01,     7.7159258679680115e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7675038303092556e+00,     5.4132513269340543e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.9343067067246153e+01,     1.6657741878959056e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5048482140079066e+01,     1.6657023819941449e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.8436455228999911e+01,     2.3884064953231464e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {    3.0952891699624587e+01,     1.5796089848891888e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2556278853968976e+00,     2.2872765975282823e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.5047894744725561e+01,     1.5854273628524630e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8046734141306569e+01,     3.4596364662247634e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.8680331855560723e+00,     2.3163508326067717e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8326896807237912e+01,     1.4463070619927650e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    7.5197599444809624e+00,    -9.0775309619877899e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.3133706717900475e+00,     7.6208233741262784e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2951956845247050e+00,     9.1267335737034791e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -4.8896321674874976e+01,    -9.6902800562680568e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0385481751258190e+01,     8.5485589290384924e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2819082255497591e+01,     8.3309908895674725e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.7893604550999882e+00,     1.0074242141985098e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.7183612799423578e+01,     3.2187019228434750e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2911850036135224e+01,     9.4271106486495155e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7162277021017026e+01,     2.4499041954487751e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.7855385305409811e+01,     2.3990890420602820e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0994335416417272e+01,     1.2336453382667125e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.3952782011868436e+01,     1.7777280317080746e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.3490184174280300e+00,    -8.3324579102206280e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.0010726211563341e+00,     7.2265143849839451e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2889221953079495e+01,     1.6224030972440692e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6073205959132707e+01,     1.4286132406542147e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6835243310852874e+01,     1.5540805406302299e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4180928254670064e-01,     1.5544703174065618e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5620556034862005e+00,     6.6103384442142224e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9819259430339919e+01,     3.2200137139634211e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.0308516200150208e+01,     2.3314131435275936e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.3849625836543918e+01,     2.6780326857100430e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.8105536000307545e+00,     3.3348522746037954e+04,     5.7842170058363762e-04,     9.6467741453494352e-07,    -4.5715021246542584e-09, } , 
      {   -6.8922899753288052e+00,    -3.3281008651763055e+04,    -6.4008030454714836e-04,    -1.0386115009041478e-06,     4.9192104967460172e-09, } , 
      {    1.3249931228674999e+01,     6.9707563524906654e+03,     2.4499652131378265e-04,     3.6122497752109303e-07,    -1.7093560768560020e-09, } , 
      {    2.0225333864044622e+01,     3.0490963672256876e+04,    -1.6054166293560877e-04,     9.0310413777930817e-08,    -3.8916478996343569e-10, } , 
      {    2.6441540035998168e+00,     1.6900689362418419e+04,    -1.5887189189581539e-04,    -4.6866938552858840e-08,     2.1428764705041486e-10, } , 
      {    6.7025907673935148e+00,     7.0720274939030023e+03,     1.5250861536851649e-04,     2.5032384796728670e-07,    -1.1877935187183631e-09, } , 
      {   -1.1434742764303051e+01,     3.9279084970901399e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4138071419525343e+01,     1.1701586842021854e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3492371108307953e+01,     3.0328340690288123e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.8721405824004741e+01,     8.3285524486954364e+03,     2.7602168802270629e-04,     3.9810475424309542e-07,    -1.8846647039452100e-09, } , 
      {   -2.6574107622098339e+01,    -8.3288304007548177e+03,    -2.8603813259990178e-05,    -1.0271747461667739e-07,     4.8801345140485886e-10, } , 
      {    5.9266396958768524e+01,     2.3314483751234009e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    6.7907617463761341e+00,     1.3264810565354341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1071820732685367e+01,     1.8327682505537836e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.2150256787152198e+00,     1.5555205809709072e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5414539608770297e+00,     1.5548654344270375e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2243372282631952e+00,    -9.4459498163076041e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.7309506585731746e+00,    -9.6554992821383257e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.1792403344387054e+01,     2.3313779119317864e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.1254578512774648e+02,     2.3870937756371048e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -1.8075762338984310e+02,     2.3871953673447122e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.9266196387168062e+00,     1.5573932693698442e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.2236213703308252e+00,     8.3232617196921328e+03,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.3464496137773066e+01,     2.2636160452795852e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2775706083497813e+01,     2.2286243924937680e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.7973076083793870e+01,     3.0831077140692018e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3123416816929208e+01,    -9.6502396175059584e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1847927422359312e+01,     8.8456653212555811e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9556489365285344e+01,     1.1255606250903797e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.1353774517523618e+00,     1.4082004099927097e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7723531188025476e+01,    -5.2183050946841904e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7392296307553003e+01,     1.7192390429968033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9064913048393937e+01,     1.4821146700610654e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.3473269381519497e+00,     8.3364063516708429e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0691353131056500e+00,     7.4896898639140800e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.4845819641484269e+01,     8.3436118628872391e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.7161209670836470e+00,    -1.1272477975886727e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.7262466885062587e+01,    -1.6595249699477317e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -8.6773982221288080e+00,     1.6171390507705950e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1719934085628189e+01,    -9.4406901516752368e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.4352582356138072e+01,     1.3856906044536890e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0948884427771945e-01,     7.0720762546340393e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.7439155549852199e+00,    -9.4771014114943864e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {   -2.4046794120263478e+01,    -6.0576567991448701e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.4944617777264678e+01,     1.8190971364348297e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5631678147577857e+00,     1.6727212693355628e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.6766557381551522e+01,     8.3881947423818783e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9344021773161138e+01,     2.4186914423589948e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.3311984650837125e+00,     1.5009364272838435e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.1241653691967954e+00,     1.5506671662841258e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    7.6308680465974170e+00,     8.3926811486586812e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8854108029171126e+00,    -1.6883852996089758e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.6842863666489709e+01,     9.0366288135664945e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8799784633307937e+01,     9.7030203320315231e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9924340366175318e+01,     2.3836500825899097e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    6.9348581350949541e+00,     7.1573958865454470e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.4725089212071850e+01,     1.6656678217534103e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0759330875936625e+02,     1.6655651880971825e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0430466236803255e+01,     1.6658087481366401e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3536610566844644e+01,     2.2066376420624314e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4566833866930054e+01,     2.4493194344354899e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.0192767344714525e+01,     1.9977318645321910e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0043909986661777e+02,     7.7004818912534229e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8415788610587644e+01,    -1.0241362132969773e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.9984887692925017e+00,     8.5820117240650070e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.7187851722692997e+01,     1.4936487958583144e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4474178496044754e+01,     9.6523707769450684e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0058942613832883e+01,     1.2558622086820225e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1672050782738763e+00,     6.2516940369415615e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6742337509806049e+00,     1.7652974254666951e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.2745036625385410e+01,     2.5536829598208798e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.2728962519884863e+00,     5.9176327934698338e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8044706698294931e+01,     7.1969955839191853e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3272076527702801e-01,     4.8897184748038726e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.1393437103094959e+00,     9.6013061031406978e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8670969734584475e+01,     1.4269687627538979e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2997260700245320e+01,     8.4169975928718522e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.0199150299730650e+00,     1.6517663227524907e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.4106907497394635e+01,     7.7080104233058537e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2396476822597656e+01,     1.6850011634472190e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.6387431759791617e+01,     8.3860697491053888e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0404740008372002e+01,     1.5525687008644314e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4025928691915150e+01,     8.9776625260741785e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.9785630779610557e+01,     7.0724285705921147e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3709810885266425e+00,     7.7500472371141277e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0488554149038620e+01,     2.4046247350789163e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3318089267712814e+01,     5.9600775276839304e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1886578608354789e+01,     7.3230953541341860e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6714599621508011e+01,     8.3401566773266040e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2496046587451389e+01,     1.4901328793938959e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.6174269993137340e+00,     7.9946301812533984e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6314305748992794e+01,    -3.7387614536963997e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2331104699195549e+01,    -8.1552245862371019e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.3610144133202646e+00,     8.2647017007915711e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0337299792897213e+01,     3.2122584878622085e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.1665893569868278e+00,     1.5531733663461595e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.6681433813109038e+01,     1.7256631475449803e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1349110582815529e+01,     8.3368584677109884e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.4964087847219329e+00,    -8.3205243817392638e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.1625939526056897e+00,     2.6298323161836379e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6824807543216217e+01,     8.2898429252748920e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.8297542113111964e+00,     9.4160900219271516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9782252998760285e+01,     6.6038042237686641e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6811982402863414e+01,    -1.5925960759164900e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7374016798405666e+01,     2.4564187649767300e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.7030767709262978e+01,     4.0564876980761743e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.8095164641650413e+01,     1.6732633910622641e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -7.9115446645217133e+00,     8.6261111900506166e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.8688417689955756e+01,    -6.2830087149481687e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7536539710084295e+01,     1.0546720738442129e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8316094513757740e+01,     6.2545540875848346e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.9702049130470831e+00,     2.2003914638174929e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0917680196638379e+02,    -1.7309684784268029e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5398830465757705e+00,     1.6657159192394218e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.4165520549611418e+01,     1.6657606506506287e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.5761511562532773e+00,     9.0889962145892678e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.4526459167564742e+01,     8.2173577362100368e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.7370516314897159e+01,    -3.1776990852060044e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.5126611195540125e+00,    -4.4345643551379499e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7392441122952613e+00,    -1.6797102471375601e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.3994604044383010e+01,     9.1170345639508359e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5201271220151249e+01,     1.5578836917526660e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.2729856149606569e+00,    -8.4758840835268784e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.2586285578043046e+01,     1.4301095453230986e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3336022951419348e+00,     7.3445747478960275e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.8874733038405545e+00,     1.6029080894201770e+04,     3.0728938277560503e-04,     5.0020583674561016e-07,    -2.3182384270982792e-09, } , 
      {    4.7076955794859209e+01,     7.2032461540991117e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    9.4668456035512136e+01,     2.3243131416312815e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.5680413217680652e+01,     7.2311301469984792e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0257659842041095e+02,     2.2741275127354362e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.7531341456479524e+01,     1.3522844801065163e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.0231239462604265e+01,     7.2104600694239161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1739967609452169e+00,     4.8559370473009076e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4715271041381584e+01,     3.1165138384163743e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.5655086359049637e+00,     7.7985996169952159e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2739374555142989e+01,     1.5942334215476674e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1684695916287172e+01,     1.5083663278027478e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1288607173253382e+01,     1.5916630435553599e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9421658043028117e+01,     1.4839195643966028e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7858550985443429e+01,     1.5553774916906323e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3142001870261526e+01,     6.5895685722264243e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.7884098834725862e+01,     1.5539151494149039e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2597550753247759e+01,     7.2176656614937510e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9088400536391630e+00,     3.5993218772098217e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2995994655139015e+01,     2.1515621096590894e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3174521307167989e+01,     1.6207110291631912e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.9656530453789749e+00,    -9.6926843517200541e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.3189157326251930e+00,     2.4978112291429834e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5223509813856523e+01,     1.5440582581257078e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.0860451744171797e+00,    -1.6641383341125766e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.2341971720994671e+00,    -1.1664436707565833e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.5427950029226691e+00,    -1.4486898027380212e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6747181680260983e+01,     1.8875412163167253e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7991412245199431e+01,    -7.6943503652846357e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.4166656103351322e+01,     8.7821973738905908e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1182866312887992e+01,    -7.0793738884136769e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.2562075183037216e+00,     1.4050852504740316e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5519774117469304e+01,     3.8263523029424345e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.7600361437240597e+01,     1.1002285951563914e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7703875436327678e+01,     2.4579935393282936e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.3329762681948136e+01,     1.5559821571723607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2676180997525810e-01,    -2.9219990533907990e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.4961631034864808e+01,     1.5546357086218877e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9773324499446751e+01,    -8.5820269834330556e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9390879745368952e+01,     3.2722506530257375e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.4043416090716319e+00,     2.3537384471437355e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3696976540612312e+01,     1.3202677415381408e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8371378848766341e+01,     1.0037081575503251e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3583212867238608e+00,    -9.6528694498221412e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.2378187023841036e+00,     1.5002360691359592e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2166087511400923e+01,     1.8071732373881772e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0940783233374383e+01,     1.2400887732479796e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1865647475520994e+01,     1.6726074955022916e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.6966483663697600e+00,     7.8761818881699201e+03,    -2.4686215708074434e-04,    -2.2396035872770372e-07,     1.1073580808854800e-09, } , 
      {   -6.8168347242993503e+00,     1.4908413230743468e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.6725009927662633e+01,     2.3402397737455191e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.5782476571579789e+00,    -8.3006943058532652e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.7516241199511899e+00,     9.5845952493493369e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7870557382401586e+01,     1.1046056785073075e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.0507568688407368e+01,     1.1852762240762557e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8909430296355350e+00,     8.9459783830483047e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1465910942788550e+01,     3.0478530903300227e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3148141756373928e+01,     1.6465308216803314e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5789243531411053e+01,     6.7605569162933680e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8065869816014736e+01,     2.2441348446961929e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3381283371056195e+01,     9.4973926085950352e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.3820338204435156e+00,     1.5989260362506797e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.6288988650488143e+00,     1.5418801363346493e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.5418656337528986e+01,     1.6522396115468087e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.4505380740975085e+00,    -8.4492233769422455e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.1093837468412985e+01,     1.4914087675629615e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9508900509537629e+01,     8.8878205772175716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3740034846261791e+01,    -6.4749202647752772e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.4335532654072480e+01,     1.5049433439120583e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4546412161052498e+00,     7.0628874667181281e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.2462566307660943e+00,     2.1906340866444149e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.8247724689668754e+00,     7.6911950517636715e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.7047859262805645e+00,     8.0258085847678831e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9035768962130945e+01,     1.7572293488461568e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.9419033086712766e+01,     2.2772359183931221e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.8155231305300079e+00,     9.4256879577157633e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2426554192162477e+01,     8.8041956875861561e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6211816088739376e+01,    -1.0903248867535331e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0529832844166926e+01,     2.5544492081200951e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2606610214422528e+01,     1.5304837842515219e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6500381505453554e+01,     1.4053153704580051e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2104050419080600e+01,     8.3131934348500981e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7922876038578146e+01,     1.4446174674273039e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6639240193649350e+01,     3.1674001263515547e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5011271339406044e+01,     1.6235496225042172e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.4827734011728690e+00,     6.4798170859909069e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8710016193854884e+00,     8.4269072020407584e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.6586141545899650e+01,     3.1636263904401338e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.4263848894014473e+00,     1.3082493047212976e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5558222387176848e+00,     7.2030422387364688e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.8405672117038677e+01,     1.8071526658654562e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1326299539642903e+01,     2.9868708312174742e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.3761965842018713e+01,     8.3470537411889018e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.6977054106356224e+00,     8.8937206313039896e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2480633364663358e+01,     1.7936539924672394e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2309646370071894e+01,     7.7927070530698311e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.6100649713966666e+00,     7.2942619247342927e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8407637490476876e+01,     1.4252357692589907e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.4199370684152640e+01,     8.5459290967223078e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5850066108088384e+01,     8.6748130557156928e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.8034305538128471e+01,    -7.2947436316642161e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -5.4567835607374001e+01,     1.0177257587036129e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2592125971619907e+01,     2.0967495604873657e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5335030563696762e+01,     1.4190967288008618e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2716061773333557e+01,     3.9450248421495446e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.7198919399735384e+01,     1.3746849048660932e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.7830383868572053e-01,    -1.1191171901091127e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8162731898317645e+01,     8.9166586266679769e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.5118750992711568e+01,     1.6886110509203081e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.1226035188350508e+01,     8.3773932903902005e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.3486078086088167e+01,     1.4206509316297048e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8386879619967900e+01,     1.4827705656210379e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    8.2831695135939725e+01,     7.2116750324836821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1955061637887875e+01,     1.9187669708227034e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9480041714861024e+01,     3.8526574359177503e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.0351906335203083e+00,     1.7642293583098581e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5192549844579389e+01,     1.5540366457208809e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8640975580427870e+01,     6.9875927188193300e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.5777668197438119e+01,     7.2250834921811966e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6418277922986871e+01,     8.6310422222065372e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7822245524297841e+01,     1.5741828799673520e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.0466558264730303e+01,     7.2164506984339823e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5735238626353919e+01,     1.6657531266507467e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.9870283623591121e-02,     1.6657234432393037e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5272963798722991e+01,     2.7173347826793419e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.8116354452127013e+01,     1.5545142123159108e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5252824088602122e+00,     1.5443427495314869e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.4213221312595124e+01,    -8.2872217910557010e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2843961980504975e+01,    -2.5103435886819466e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.9443685423043760e+01,     8.9569009581968294e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.9714989320787012e+01,     1.6792369583432417e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.7471183272847597e+00,     6.3478388327445500e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2502073878395613e+01,     2.9359493116864862e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.0356834906181582e+00,    -9.4613441921926242e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.1303694976595262e+01,     7.0574824968393023e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.4918208961728672e+00,     7.8827108336070678e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.0228706252762514e+01,     3.2200848485101087e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.0652937278134608e+01,     1.0136061918849631e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4100122290129669e+00,    -1.6298143307260230e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.7115954672229684e+01,     1.6473349399443396e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.1474546455891144e+00,     2.3382823540848600e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.8199073521243290e+00,    -1.5541650762791533e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -2.1233086836314975e+01,     1.0965796159109195e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4737430447383463e+01,     7.2151587464268068e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.2775278632969798e+00,     2.3902624118423566e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3506344804146533e+01,     9.0552147870863027e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.1212972629566391e+01,     2.4078693980900072e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    1.8167388676494426e+01,     2.5208242978328479e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.5973526676206715e-02,    -8.3118306780309431e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    2.0829882311909916e+01,    -9.6130545479242282e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.9662239318683483e+01,     1.5525122263908303e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5179101428217075e+01,     8.2107987806103101e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    9.6229816344115076e+00,     8.8863978862838212e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6778118051251418e+00,     7.1430695600464360e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.6999827342588265e+00,     1.2560123656589816e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6434486799978131e+01,     8.3566885435969871e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.5364303231238440e+01,     7.2016113459337184e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.3876545895435822e+01,     3.0785787615202691e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.6267974606010064e+01,     3.2199425794167335e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.4068627731789046e+01,     8.3659033026345587e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.3789315577792189e+00,     3.1085837212844370e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.9249851437398775e+01,     3.1085179947891465e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2729932900081298e+01,    -8.4219914292528883e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -6.0000506892284378e+00,     8.3298206752918577e+03,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {   -1.3852752487322029e+01,    -8.3275621741583946e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    4.3416714499338322e+00,    -2.7876749255581572e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.9246432989104257e+01,     2.4975257562815663e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3194700267892308e+01,    -1.2496152932484601e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.5372094538371570e+00,     2.2221809575125011e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.2201755671470721e+01,     9.3464171834287390e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1143304965158700e+00,    -2.9480519237735416e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.3746886948269768e+01,     1.6646566138090529e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2672087066719540e+00,     5.2165804515628406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1964323132461910e+01,     3.0404231898111182e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1890976007752455e+01,    -8.7090931560957342e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.5397164498804386e+01,     7.4131373749483937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4525010370899793e+01,     9.0563258046397586e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2853756891859030e+01,     3.5348875273971526e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.1668421185765183e+01,     1.6306048609211668e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4228787476510249e+01,     3.9071976985325455e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6996660116227297e+01,     1.0392012050505695e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0695524216646593e+01,     2.1813040861916386e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.1389836253755462e+01,     1.4286145833643597e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1386310406184712e+01,     2.1542861801507410e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2993400069319380e+01,     8.3272605319223767e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1423883578405373e+01,    -8.3301223175278756e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.7017808629587545e+00,    -1.6375695568272355e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    9.9646908433385075e+00,     1.4949391594148401e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.9280108303582605e+01,    -8.8837045045185369e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    4.0090543539958333e+01,     2.4083296380579541e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.3085687345320562e+01,     9.2910602532423964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8224411612605222e+00,     5.8592319180521135e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6047099480728562e+01,     3.2086021073565698e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.3031762676778527e+01,     8.4729792353896537e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.7673520863079766e+00,     1.3764481074936588e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.2051690371433761e+00,     8.2735800892443094e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.9984498588387765e+01,     2.3560908038384143e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7102893257927889e+01,     7.2129669844908585e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6994864807263479e+00,     1.7365872527824104e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8935323716961480e+02,     1.4914954663676452e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1865766113831519e+01,     8.1421066750376467e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.0590516743437995e+01,     1.6894541292359296e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0320259963836165e+01,     9.0966844207557278e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6770634655226676e+01,     1.7942525829250499e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.1777426083941052e+01,     7.0565342263456096e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5391918038993495e+01,     2.0990178178879648e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -4.2019351789747589e+01,     2.0919714987264672e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    5.7988756165023547e+00,     7.9652293257355395e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3806484138892188e+00,    -8.9378113400624425e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.4910133143495536e+01,     3.4258550387217983e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.8030235119791680e+00,     1.1581788832155120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1289657629599528e+01,     8.3963790303608766e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2173590561334136e+00,    -1.7079912553018503e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.1409689957911002e+00,     1.4935065267649392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7087615556018996e+00,     1.4948228132641379e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.5399301338309819e+00,     5.9414481870432610e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9982377327390225e+01,     5.4485848160159403e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6203149759987781e+01,     7.7558351168035497e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.4661477794516653e+00,     1.6641963738411061e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9738410066320004e+01,     6.6475750572778234e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0093471675145622e+01,     1.8199897324468848e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0250476046539646e+01,     6.9761464177900925e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1935615576799677e+01,     2.1828788605432022e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.2793835926106141e-01,     2.9968578821261886e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.8881562853860636e+00,    -7.1504863397357872e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.0937307570411479e+00,     7.9291114994324107e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8451874272519554e+01,     8.8391063656558545e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.6356513070858778e+00,    -8.3838027602059428e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.2915004220492747e+01,     7.8751854755596614e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9085990336070367e+01,     2.0194571576687136e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.2243624984911072e+01,     3.1571477238630596e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.8731474326669201e+01,     1.4270597091804442e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4185626541516818e+00,     8.9783289177485203e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6912101773597410e+01,     1.8230786266246207e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2415718297932415e+01,     1.2729474352360114e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0547153780931744e+01,     1.0196341183094901e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0655556118841446e+01,    -9.3982692338621473e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1670898223963050e+01,     1.6064412335340243e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1886179625870961e+00,     7.2905041089175002e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2623828276311460e+01,    -1.7766875616522211e+03,    -3.3732043903141355e-04,    -4.0052465325098564e-07,     1.9445343454254264e-09, } , 
      {    1.4983742058873450e+01,     2.3347888482413382e+04,    -1.3011148381682680e-04,     5.8953343622919576e-08,    -2.8923984214994916e-10, } , 
      {    1.2215293958784443e+01,    -2.4050495372258629e+03,    -3.3978845579715381e-04,    -3.9999552759942267e-07,     1.8886401761303078e-09, } , 
      {    1.6128162833560737e+01,     1.6534523974219086e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8658508036507424e+00,    -1.3705786909223589e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.1907483753617267e+00,     5.0202303572410665e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6275838839394792e-01,     7.7249670034805158e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9048602254861891e+01,     1.5494052424518881e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -5.0668994320461685e+01,     8.3377268185848279e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2898735241080381e+01,     7.5246005419837766e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2653081947171987e+01,     1.4508324790193124e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1271841881487425e-01,     6.2082941847648647e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9732837598645517e+01,     2.2048327477268940e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0675498663036731e+01,     9.6236882851727091e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7390492682830221e+01,     6.1103458827868690e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.8181849358845064e+00,     1.7285684804698736e+04,     3.0196161978978796e-04,     5.0143862097393552e-07,    -2.4271178836018586e-09, } , 
      {    1.7892457337204931e+01,     1.3800529491338999e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.2401565905589745e+01,     1.7350124784308464e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8804393838941593e+01,     2.9766760260303894e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.6099936199636115e+01,     1.5541229036932742e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2953930121105323e+01,     8.3356913015677528e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2557920550842589e+01,     1.6181371156484989e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3178924380262494e+01,     1.5544279543435177e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1122986961278212e+00,     8.9493724261443986e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7095652837807593e+00,     7.1960291814715065e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.7621673325995726e+00,     1.4048222672424134e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7611643910691548e-01,    -1.7436474897787130e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5554092138928386e+00,     1.3688983130234110e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.6800400874367291e+01,     3.1651459358384491e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.8855982236097496e+01,     9.4325032726316986e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7641607171838487e+01,    -1.0954408895580682e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    6.7235766105210306e+00,     1.3763058384002836e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3411479691965335e+01,     9.8942163478973489e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3121872145992722e+01,     1.5387649768159710e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0329753544741571e+00,     2.5934124335104807e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1580123884425060e+01,     2.4499400983996555e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.5650823267873889e+01,     7.7002857881057444e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8414638269617953e+01,     2.9973181220941351e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -3.5291204671934970e+00,     1.7348047241524764e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9852944329767714e+00,     1.4931367385947196e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0008320488287294e+01,     1.6707046246884798e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.4159391471993811e+01,     5.4087699141180274e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4053629368920129e+01,     1.7678711404188783e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.6812182758102061e+01,     8.4586110867600237e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    8.2426021577578084e+00,     3.3406124347172772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4630986833433162e+00,    -1.3433656948052421e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7713773758973140e+01,     8.3757604678499010e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.9710744067542720e+01,     7.7712790935181401e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.5374702260036672e+00,     7.6136333149654274e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.9753526291512618e+00,     1.5948893171076401e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0639110354107534e+02,     7.7714751966658187e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.0617464567391806e+01,     1.8994080585273921e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.9027318031570992e+00,    -1.0327577700693029e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.7527794790945350e+00,     1.7702121630696573e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.7508571550298981e+01,     2.1913893032819258e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9818468703733295e+01,     2.2125879738281063e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.4010297025857206e+00,     1.2559038246242101e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0172523726983016e+00,     1.5507417219248424e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0535299229131656e+01,     2.3878004670508813e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -5.5570974186323411e+01,     9.5171832084254220e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1649133422236645e+01,     2.4006432448891253e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.7120595701127996e+01,    -1.0393774980939052e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2267205373906531e+00,    -1.4017302861091393e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.0745126347969162e+01,     2.0199094891187960e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9428017832218639e+01,     1.0143724401841784e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2491623137546178e+01,     1.8720703475230821e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9145983126505101e+01,    -4.8252841249912035e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4946844439230750e+01,     1.3817450172147779e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.8116514843189826e+00,     7.3537482764438919e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.2001340303548789e+00,    -8.3744704627754654e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    8.4460431273578394e+01,    -1.7429428578625634e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.0901222665979349e+00,     1.4428454363394114e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.4538823857209891e+01,     1.4427797098441217e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.0134457633141096e+00,     1.8319536579254382e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9721320417836967e+00,    -2.3919072456608892e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.3025522411113837e+00,     3.2063009075168338e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6346947394179780e+00,     7.7195715093878098e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.9801022795593273e+00,     6.6806728481133086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9737033003325228e+00,     8.0158714999326867e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    7.7488360585309701e+00,    -7.3139218255863225e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.7808908611974381e+00,    -8.3463234510007824e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2891357489641315e+01,    -7.9268539863190863e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1690422059436523e+01,     1.7016622391640274e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.8266355236471973e+01,     1.6939070130628152e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    6.8796457511327390e+01,     6.1963371067552198e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1615149143017724e+01,     2.0370339615014898e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4516536655899006e+02,     7.2141048912254573e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2279959124175224e+02,     7.2140208396922080e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3324687508099160e+00,     6.7549718533023515e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7322631474824600e+01,     7.6600434565768910e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.6340011004226959e+01,     1.4969803635324757e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4676115610261053e+00,     6.0427673272791535e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0521691732370318e+01,     3.2166355712131244e+04,     2.6936461440465505e-04,     6.0169256335142287e-07,    -2.8559889140481653e-09, } , 
      {    1.5977579150036856e+01,     3.2233918567137174e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    1.3810722445434642e+01,     1.4461907158420629e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.5799995487818230e-01,    -1.7947779447044304e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    9.5802332537173509e+00,     1.5366986251856202e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    3.9445848407669786e+01,     1.6730677354702380e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.2094610519384119e+01,     1.6171068588780228e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5176604058678880e+00,    -2.4514258226856878e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8579782809765220e+01,     1.3958725437505696e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3781017109085457e+01,     1.5179292191194374e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4961258726178343e+01,     2.2459397390317299e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4969807912161492e+00,     8.4630514519236494e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1572528099315351e+01,     1.6728370525065588e+04,     1.1655480853674925e-04,     2.8300096881322299e-07,    -1.3951434819921005e-09, } , 
      {    4.1772160876469924e+00,     1.2871789595661356e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.2141625695521299e+00,     8.3570431471950942e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.0678803268716390e+01,     7.0589583434345841e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6186079720844901e+00,     1.1205383863896885e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.7830580194128430e+01,     1.5193151018423801e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.5507977857134051e+01,     9.3689590885597954e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.0647160382493341e+00,     3.2200465772110667e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    4.3563986920656511e+01,     3.2199808507157763e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.1923314587180274e+00,     8.6921535237147109e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0728654322505253e+00,     6.1463336448949585e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.4418206372835876e+01,     2.1908642066283883e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3626908170384766e+00,     8.3532840643121781e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.6610578389288591e+01,     1.3211325462724491e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.4614865457999855e+00,     7.7041971314927177e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1958723433797225e+00,     1.4682557170593569e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.9563256045721111e+01,     1.0308786011946011e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.4956082492555595e+01,     1.5543458922100108e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3209877635206890e+01,     7.0697750547943051e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.4580318636718673e+01,     2.3776273882824833e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.4766636577456804e+00,     2.3754658470258346e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.5424843587833763e+00,    -1.0203624773855561e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1433581439220990e+01,     1.3347338533268934e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2619096765780689e+01,     3.3050613428832861e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.7626443489753072e+00,     2.3887178199056674e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -1.3568510939865337e+01,     1.5750002556174946e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -4.5270640279847832e+01,     1.5009618537222670e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.8368833291138664e+01,     7.2125376122076141e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3156985090374942e+01,     2.4100173374661914e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3228597750836347e+01,     1.4329510281515451e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0736484927243616e+01,     7.2155881187100495e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1990816992260420e+00,    -8.8679814961570682e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.6891058105020178e+01,     2.2676635062314483e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1386655268047734e+01,     1.6047099472605914e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -6.9520511267681769e+01,     7.7869191733804091e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.0904194284405605e+01,     1.5860524198704559e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4400410019343521e+00,     4.9663397434544649e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.7928393097168453e+01,     1.5542049658267808e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5103494142290774e+00,    -9.6992701156474868e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.9350677541160568e+00,    -4.8275853248309377e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8517050874144885e+00,     7.0191071574228481e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7650631463684562e+01,     1.5771481949936786e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -9.3771582194003535e+00,     2.3243466762339987e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.1762902011036637e+01,     1.3443214997269339e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1371281960980685e+01,     7.9497482074082163e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.4834181148695361e+01,     6.6077965338580179e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2149374071423198e+01,     3.1178808584895680e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.8244089053559343e+00,    -2.4218859035545274e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.2909658309884975e+01,    -1.7769710208876812e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {   -4.6217871340136810e+01,    -8.3010824476468006e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.2790964878607468e+01,    -1.5836765367201879e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3094646379585381e+01,     7.8430817959712531e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1490212913770851e+01,     1.2951643056513214e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4060551985865057e+01,     2.5739135146922443e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.0354615416793997e+00,     2.1517122666360487e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1084857674962283e+00,     1.5505994528314672e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.6154497537524399e+00,     7.2126401745250805e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.7414670895376005e+00,     2.1047902594499956e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7123008097308567e+01,    -8.4242926290926225e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.6304324979758524e+00,     8.3293130525001307e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6755778533221431e+01,     1.5543857817576385e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    3.6659262376638651e+01,     7.9843723206972209e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4753018629462428e+01,     2.3778145710381323e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.0834750156782949e+01,     2.2614478228859924e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0912343249064353e+01,     4.2456784056107390e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5631191496585721e+01,     7.0443283015260231e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.7283667111776843e+01,     1.4932470913339619e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -6.2858519169430572e+01,     1.8365773000228379e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1591299139961059e+01,     1.0182462584674973e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.2032191846921332e+01,     1.5554751900731337e+04,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.6845859733890711e-01,    -1.7609603525130417e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.7973566247581489e+00,     8.3352428901638214e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1671954176993719e+01,     7.3691673874830813e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5745743103241688e+01,     7.2154855563925848e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7188539062539249e+01,     2.3387425940528072e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6240899319183590e+01,     3.2277689400646337e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.7244270405588686e-01,     8.9881774132851533e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.6985970042235934e-01,     7.2136696191963911e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1898284931587373e+00,     4.4966975051108839e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.0532459933386960e+01,     7.7161163238271729e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7263399525322484e+01,     6.0334743949479162e+00,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0945749213357407e+01,     3.2788629822781848e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.1165478108362265e+01,     2.3227596101575109e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6695875395405917e+01,     2.7566368796486408e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6842052591928510e+01,     2.9395541533267271e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -2.2221543114511544e-01,    -8.3280697969501234e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.7271070966438451e+01,     1.0388415815741833e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6125710795485844e+01,     1.5591456155849033e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.4042635817900909e+01,     3.3443634398525835e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.9119270107061617e+00,     1.6031705096720894e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.1296924308794864e+01,     2.2171169386552952e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1794969017863700e+01,     7.2144561117212743e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0686713304234594e+00,     8.4334661576404869e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5718415175647074e+01,     6.5545768768988055e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.3726615259421191e+00,    -8.3132615708336907e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1023335580549372e+02,     1.4913938746600377e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.4400997159515882e+01,     7.2151663928512580e+03,    -7.7472130562384538e-04,    -8.4177714637367624e-07,     3.9735329303737567e-09, } , 
      {    1.3174940637255848e+01,     8.4526538273487131e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1279107440091831e+00,     1.6200772690806727e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6130189151187039e+01,     1.0665389160548793e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4588635990286631e+01,     1.7250745545485810e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.6180031227041564e+01,     1.4900940193867060e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.7766000564372206e+01,     8.7406515035142511e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4473776246989054e+00,     1.0160645082650564e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.2794541037771339e+01,     2.3243144843414266e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5346640976885926e+00,     1.2478768625968367e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4668282850405465e+01,     6.8467790452876789e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -6.0662283572757666e+01,     1.7443219073609020e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0448677851684414e+00,     1.5530756679636583e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.1020044411741345e+00,     8.4022790844472929e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5043955484838918e+00,     3.2120507335838389e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.8916599858158456e+00,     8.2551037650029593e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.8552670482671374e+01,     1.4890173831528931e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0420796404735420e+00,     1.7871338203585634e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.2051481153502621e+00,     1.9804827290144201e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0409463153118224e+01,     2.1579831789347529e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5175853359146387e+01,     1.6644931329925141e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    3.2361419487820967e+01,     1.2793908702172783e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5103499067217495e+01,    -8.9509655353757207e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.7734988941849238e+01,     1.5754604955854415e+04,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {   -2.2022738435408204e+01,    -8.8883069041980089e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -4.9464842903292251e+01,     1.9033895487171834e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.3251896639412024e+01,     9.9972666736053379e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3391221027593918e+00,     7.2070592839799911e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.0893936602885582e+01,     5.4883997179138523e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0979040189282323e+01,     1.1648627220596783e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.6120491787437992e+00,    -1.6669834368975367e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.6506604058674402e+01,     1.3986017809158793e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.9159657361115627e+00,     9.4897053904487166e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1696044357327459e+01,     2.1712078374396471e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4024777005359312e-02,    -1.1338162289745205e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    8.1496558635154894e+00,    -1.2205667536886763e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4811243485135297e+01,     1.4985440010550810e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6851934765283946e+01,     4.0562585611496783e+04,     3.5992082871890839e-04,     7.7821322464340488e-07,    -3.6938923991097769e-09, } , 
      {    2.4358444738028250e+01,     4.0495071517221892e+04,     4.2157943268241913e-04,     8.5214731101260920e-07,    -4.0416007712015358e-09, } , 
      {    2.2046015149392527e+01,     3.1538078137248282e+04,     2.7193056353397944e-04,     6.0111980446856010e-07,    -2.8008219652747115e-09, } , 
      {    2.2490244352169039e+01,     2.3280374388138491e+04,    -6.8452879853316033e-05,     1.3288742999212380e-07,    -6.3694821424170840e-10, } , 
      {    1.5769783979935696e+01,     1.5019197057688256e+04,    -2.8242423445817507e-04,    -1.9145777080696684e-07,     8.9709923552508530e-10, } , 
      {    1.5022464897079940e+01,     5.9237019078244230e+03,    -1.8234380689715677e-04,    -1.5072993093526187e-07,     8.1263499600218198e-10, } , 
      {    2.0711567981297396e+01,     9.1453862864208022e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -7.0443763676489723e-01,     1.4426048188133966e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5151311867651751e+01,     1.5038409107762001e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.4018340922873854e+01,     2.1360516574566645e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.5700654525151059e+00,     1.5543108893782066e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.4647118045914681e+01,     1.5542399686585850e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.7726413863024737e+00,     7.8696004125686450e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4628467392875095e+01,     7.2132508194733527e+03,    -5.9531047605329682e-05,    -1.3964088456997964e-07,     6.6259485797240281e-10, } , 
      {   -5.2262523943979332e+01,     7.2148749114443053e+03,    -3.7747069612412869e-04,    -2.3328749521309762e-07,     1.0926245931165610e-09, } , 
      {   -3.0951210245910765e-01,     6.9646984977301991e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.6321199695408026e+01,     1.5511274062520726e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.9153118232181470e+01,     1.4430203273701369e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4536881791804140e+01,     9.9848171102082088e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2813020006600699e+01,     2.2612529344978262e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.4191211046707650e+01,     2.3108151395881374e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.6269134119285660e+00,     2.2971433693420782e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4274713867301193e+01,     3.1418665577041647e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5560465148337187e+01,     1.2094607811714843e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.5150574840238171e+00,     7.3511337034957551e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.4647409827065573e+01,     8.3563004018034517e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.3354537983165091e+01,     8.3378649532729905e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    2.7645909127595987e+00,     4.6206504319483520e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7928843102186853e+01,     8.3661625320612893e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0664720401302544e+02,     8.9570970613445097e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5491343432040546e+01,     7.0743432435334844e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -3.4912670611104055e+01,     1.0156526281196988e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.8476423053816326e+01,     1.6520665146989661e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4637350005203480e+00,     1.5310864755639112e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.8359563492073345e+00,    -1.8204884053239607e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.1350195217382279e+01,     1.8525032607820027e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9402549056944771e+00,     2.3933578864882020e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.7725055638735146e+01,     1.2702110267414610e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.8488520838236155e+00,    -6.4522826585432438e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2788326618035935e+01,     1.4851168326671737e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1515929542301926e+01,     2.2130107021398717e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.9118237661310800e+01,    -9.4433132704406944e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.2061659403186198e+01,    -9.4433266975421448e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3644819179309529e+01,     8.8767999504952095e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5119148429693464e+01,     1.3595698385074396e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3957858977405138e+01,     8.4085448855769846e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2490801678850886e+01,     8.1529081270293191e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8665704213427695e+01,    -1.2597436752843462e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    5.3333227836985394e+01,     2.3871086685400282e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.0803018658544632e+00,     1.5341249102334366e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0234029226100876e+01,     2.2779437248296172e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7985064373139142e+01,     2.2163454459607234e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.0801217589304875e+00,     2.3852258045200855e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.2355294559020713e+01,     8.3507326781698539e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3616958593774759e+00,     8.3066501712803984e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1545715483033174e+01,     2.3871804744417888e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.8743827193705602e+00,    -8.3139729163005668e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4299794800272803e+01,     7.7261229748265050e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5115619849579005e+01,     1.7737465415182833e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.6207678448235541e+01,     1.7148972732035203e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2739344881060513e+00,     8.7877824368816073e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2226570082043345e+01,     7.5798518874624633e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    5.9745108875870798e+00,    -1.1844584031716679e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.1834895807166994e+00,     3.8297270245987012e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    4.7168597673347383e+00,    -8.5977594675806467e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.3613152008776929e+01,     3.2193578184034483e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    1.4661554383307362e+01,     1.4461644314097703e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.6865110242820464e+01,    -8.3318046705503402e+03,     1.5896982425939952e-04,     4.6823305321558982e-08,    -2.1501486757207916e-10, } , 
      {    2.2089716048767716e+01,     1.4133879389343851e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4597132847070828e+01,     6.4821440090049500e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.0837354826516343e+01,     1.7195020262284219e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0677998249386896e+01,     1.6101172097209530e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.7552968296589890e+00,     7.2832412980119279e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3492873841750068e+00,     7.0551555738252582e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8217278518590966e+01,     8.3927734585797189e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6327692694615667e+01,     3.8188760084147871e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9039502051233804e+01,     1.1395306921256901e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1545046523255401e+01,     1.5956445337451511e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.8310482057876492e+00,     1.7338659531706988e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0389398858896692e+00,    -1.4919017854436420e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1938719114671091e+01,     1.2245783210455544e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3339822079112128e+01,     8.1528634522827715e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0770256574357944e+01,     3.8301872645666474e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    3.3517064725579666e+00,     7.2729320167564410e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3442787449250204e-01,     1.4976803512167384e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6580121001358698e+01,    -7.0851941658334945e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.2988636000677438e+00,     9.8397380344921166e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9293113382695374e+01,     1.1439077754766062e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7210672316816751e+01,    -1.0501942094536246e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {   -1.9027412841418922e+01,     8.7969634555905031e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3779688699948014e+01,     6.9515443024169199e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    6.6582466640195193e+01,     1.6562211017365997e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.4643703126941592e+00,     7.0018835673119274e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.6478252678508795e+01,     1.3483029899167252e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1432093252984526e+01,     6.8506007664692461e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6548226074527747e+01,     8.3827640493287399e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.7812081743134165e+01,     7.0071432319442956e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    5.9121127760528310e+01,     8.3934780904958698e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2283015801422206e+00,     1.6060253667919194e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8705992377892677e+01,     2.3051063497216608e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7292006221375683e+01,     1.4918249474747741e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5906774151089659e+01,     1.5074065342238866e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5431971806240199e+00,     1.0149624455928200e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0612557493325493e+01,     2.4906444470379553e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.7132234695291615e+00,     7.3047330376704122e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0557227320373933e+01,    -8.3306557723957922e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.5872222566861254e+00,     7.0987675615335547e+01,    -1.8807069274594723e-04,    -2.1782126004654988e-07,     9.7753467335796839e-10, } , 
      {    1.0835311749799686e+01,     2.3543230485485988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9370915719219585e+01,     1.7460492070375876e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.4424354315071341e+01,     1.4213509193139675e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5069178507871721e+01,     9.6213550112186240e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4480268010955001e+01,     7.2121139815771730e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0599169147798690e+00,     7.1857293760021394e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0549887194768344e-01,     7.7953014073796021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3384239355599693e+01,     1.7288320266811981e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.8978671433198047e+00,    -9.4951350954817099e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.1228044239672109e+01,     8.9696182480930474e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.9146106260250971e+01,     8.1754652915284241e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4553637153090824e+01,     1.6248614136241626e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5711445135169608e+00,     1.4338396871837187e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5476523125225725e+01,     6.8644595936986752e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3115687807682919e+01,     2.1570091763571676e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2358379407567416e+01,     1.6728376154862650e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.2271676667804094e+01,     1.7588041231977208e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0780564452650125e+00,    -8.7025342004960075e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.8209260572938252e+01,     2.3209362332157634e+04,     1.1971574525621533e-04,     3.5066505680737377e-07,    -1.6152101081213410e-09, } , 
      {    1.8909046243582843e+01,     1.6137274817929474e+04,    -3.7924768370949947e-05,     1.0148672660581272e-07,    -5.3775048694988620e-10, } , 
      {    1.0790246264618540e+01,     7.0993305412396467e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.1038698931498168e+01,     2.5065704077971204e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.9244148685692544e+01,     1.5574234517847188e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.9239720930239805e+01,    -8.2828430484322653e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.9416224148642858e+01,     3.0457200995322375e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -8.6296496454962544e+00,     8.1555379593455036e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9132844976486414e+00,     8.4606886295437234e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.5909559583671928e+01,     1.0045391746638052e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2556788270429351e+00,     1.4349417498559551e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.8784856976113922e+00,     8.9483625113872404e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0369202083312743e+01,     1.4328087590581699e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.9311948147557263e-01,    -9.6720571195303673e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.3838097084557532e+01,     1.3122495322053099e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.4974829341626148e+01,     1.0705204062446706e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.5684994666810383e+01,     1.4428027679343826e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1413368404768578e+02,     1.4428223782491505e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.0878276567411561e+01,     1.6752554681534508e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.4677216833518646e+01,    -1.7774312608556280e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.6998737490445443e+01,    -8.4819175579218281e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.0039592667799440e+01,     7.1519297154858978e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3140683863241332e+01,     8.6627526681968538e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4035601335789293e+01,    -7.8056731579456573e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6833316670988072e+01,     7.1622477539685424e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    8.3920209046173309e+00,    -8.3249935430229307e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3686654575096975e+01,     1.4999054991582947e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6596564642969252e+01,     8.1054383885637071e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5999308961924342e+01,     2.4886215314047884e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.5850359635323068e+00,    -5.2959621425070964e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -4.6117087371561087e+00,     4.6272093875480796e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0393497804384142e+01,     1.7127956080155363e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.3159478005277677e+01,     1.3916019040732526e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.7887189674198865e+01,     1.5542028288429719e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0956868588449311e+01,     8.5797105242252710e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.9151869163807945e+01,     8.3288818805842857e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.7582352672893933e+01,    -8.3285009688659648e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.7689023004574004e+01,     1.4920480179533362e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    8.9256435658168662e+00,     6.0736951710456169e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.7498306794148597e-02,     7.8051585725949435e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.8550832062255971e+01,     2.3855713230761496e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -3.9745405503137157e+00,     1.3295962160541123e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.0953719596320060e+01,     2.2521530540290238e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.2316508375642059e+01,     1.4564843433378257e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.2307493020534636e+01,     1.6186603903517933e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6327566558738923e+01,     6.5861143099220908e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3273328442805292e+00,    -2.5278240979769103e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.2742578198275460e+01,     8.9630324841656166e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.0810334295929881e+01,     8.3745398010179852e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7038770066121096e+01,     7.1027291769437425e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.7888993238328400e+01,     2.3865545660822667e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.7445957784792064e+01,     8.4219761698848415e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.5254734928300824e+00,     8.9531857179575327e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1604424953672279e+01,     7.1975544347227242e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.9225473756841382e+01,     8.1866895545322841e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0904378641477422e+01,     1.4381768033259767e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1212325313606556e+01,     2.3236579174263814e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0105160404197768e+01,     1.0147695991388032e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.4954965481542857e+00,     9.8827263601417017e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2802046341414570e+01,     7.8374073954623964e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6373715726701707e+00,     8.4404697391193276e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.1514552029240939e+00,     7.7224250931243114e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4707562187766214e+01,     2.3890633384617311e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.4872334054348983e+01,     8.6178706943516718e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    8.0014979146593976e+00,     6.1053343602789564e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7285300064236822e+01,     1.6250691679025327e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.4450009195946748e+01,     7.8423771640551040e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1405626507291394e+01,     2.3953929008077132e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8028385423119204e+01,     1.4999766337049823e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0668065854725192e+01,     2.5368278532508652e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7807284518989484e+00,     6.6063738429242640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5158424221685284e+01,    -6.0818992950822903e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.5597010523756445e+01,     9.6804784269004667e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9833029212496292e+01,     7.2965461586268739e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0257074765981532e+01,     5.8618877214899567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0142085119269252e+01,     8.9324158459697373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1213261494058955e+01,     2.3886864825948272e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1652283825454980e+01,     2.6709646945226045e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7906601899354190e+01,     1.3643237777252592e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    5.8814911334308093e+01,     3.1572181870546741e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2095536034046651e+01,     7.7371436015488698e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.2829990941173591e+01,     1.0331327917077066e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5556300984306063e+01,     8.2660596018751166e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.2865475854707093e+01,     1.0642847255417739e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4225952870716378e+00,     1.3602264414994252e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    8.2209511273802516e+00,     1.4275829838837384e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1600733406149985e+01,     5.8224609613855800e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8180371054038487e+00,     6.2901893051241950e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6889896958079291e+01,     1.4396947327403179e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6577043692877536e-01,     7.3875639148871433e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1989420640093176e+00,     1.2085747656275378e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1139977407045727e+00,     7.2160117493404932e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2198421960711450e+00,     7.1933993491553238e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.5465305804823572e+01,     3.3836655368218824e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.4717221913298246e+01,     8.3255781788999120e+03,     4.6359532554209602e-04,     5.4764553418133182e-07,    -2.5876930229221479e-09, } , 
      {    2.3139387419791134e+01,     6.4160207164804797e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1756104263867366e+01,     1.3743151166958736e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1872083847729240e+01,     1.6643876338178899e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.4676860752159925e+01,     7.2143171298430671e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6192405495422619e+01,     6.8962929569382322e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1605133152398125e+01,     1.8941816119978877e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3029889545258253e+01,     1.3975646118314478e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6452074067154712e+01,     6.6501896302259584e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.1764861367723096e+00,     8.3029542752032921e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.7966538967837451e+00,     1.1454810238913653e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9813760943376795e+00,     6.4963749302339500e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.9568721138155894e+01,     1.8031711756756653e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2342641054867975e+01,     4.6386993753037259e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7577602069579513e+01,     1.6670889360721605e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.2233069547399609e+00,     1.1061789269220664e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4407188073588408e+01,     2.2992103770995345e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.6004345952264076e+00,    -1.1584192870784114e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.5333022466446018e+01,     2.3715844290904715e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.7042373250585058e+01,     7.2138086010745983e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3004741716076857e+01,     5.0544870981437930e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0201444254170475e+00,     7.1677051678009339e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3914655676266046e+00,     5.4662109483591257e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.4693693321808361e+00,    -9.5208722450035439e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.1251571306962987e+01,     2.3242809497387094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1293664307733046e+01,     8.8161159263383015e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7845179796276650e+01,     3.9953490076524991e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -5.6687025459989826e-01,     1.4508806497123049e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -6.0734423599931482e+00,     1.0275747329834690e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.7328707842832873e+00,     2.8132799378238465e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2976532159976021e+01,     2.3765507520486703e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1956466980442180e+01,    -1.8728558063467335e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9677392715774229e+01,     7.2260604760062088e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1189566537074754e+01,     1.4982810178234628e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    8.2811350051721710e+00,    -1.2285887523116389e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.4852627905267582e+01,     8.3174519023532157e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.5840355425739205e+01,     2.2301663035976871e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0568388100068496e+01,    -4.4322631552982157e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.3698819409777009e+01,     1.6139883471715018e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.8866608691221114e+01,     7.2163403818169418e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1920515383424728e+01,    -7.4186965585016833e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8802324263899102e+01,     2.0924863497717353e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.2675993199030877e+00,     1.6604423228025182e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.6460865645892930e+01,     1.5540451355346835e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.1618694750663927e-01,     1.5545057225021083e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.6492385698965926e+00,     1.0668768299527677e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.8218245034585712e+00,    -2.1725293136523032e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -3.2597013696456742e+01,     8.3993324141794474e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2457775078309806e+01,     8.3007501171939821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0905393787020401e+00,     1.2395292683746743e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.6132742653762769e+01,     1.4927065943460795e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.7421993394413853e+01,     2.5279236283740873e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.9045518057506271e+01,     3.2279766943430037e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.6746791856667556e+01,     6.7207420143954569e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1234806562292722e+01,     2.4502383132270814e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8361312231328391e+01,     1.4915869396072170e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5532388207459817e+01,    -9.7964182528412675e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0888337294878273e+01,     8.4395264403631372e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -7.1636558103248937e+00,     1.7117692897570068e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1572863747829402e+01,     1.4988069842866995e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0944699050398725e+01,     3.3379200048713174e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.6386988732378754e+01,     2.2852418360010288e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2143312927331673e+01,     9.4210132848760531e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2441016863942615e+01,     1.6189110818584875e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9829244254198350e+01,     8.2488379638732677e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4957342011943293e+01,     7.7292402818651954e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0842451292824729e+00,    -8.1528990060318501e+03,    -5.5415153985634931e-04,    -7.2416619547331383e-07,     3.4255965079837595e-09, } , 
      {    1.0339266199950783e+01,     7.3898552841521087e+03,    -6.2033966107973026e-04,    -6.6021927093496605e-07,     3.1168671558532067e-09, } , 
      {    3.0172854671312175e+00,    -8.8522486572208272e+03,    -3.6854886387614234e-04,    -5.0581580977520112e-07,     2.3921676653306723e-09, } , 
      {    1.4539506049312852e+01,     3.1605592231523173e+04,     2.1027195957046868e-04,     5.2718571809935578e-07,    -2.4531135931829518e-09, } , 
      {    8.8124297793315876e-02,    -7.0383304670907164e+03,    -1.8846981560892068e-04,    -2.8614537338616322e-07,     1.2513138072173345e-09, } , 
      {    6.8281502135308010e+00,    -7.3310669832642270e+02,     2.1643197796196239e-04,     2.5531742888271492e-07,    -1.2072830286989663e-09, } , 
      {    8.8418988742810498e+00,     7.6665724020140387e+03,     1.8067403585736341e-04,     2.8790728326605149e-07,    -1.4160874330160326e-09, } , 
      {    1.8028001613350106e+00,    -1.4203537155583035e+02,     3.7100948723324572e-04,     4.3678803785882535e-07,    -2.0654032442628450e-09, } , 
      {    8.6374582109548825e+00,     2.4986074274175378e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.6756516035562587e+01,     7.2332410593809363e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    3.3751857499493234e+01,     7.7828671328476248e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1033380511980305e+01,     6.5808242482573696e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7767173587936584e+00,    -9.2790266740512616e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.6482339982753736e+01,     1.6173139418013201e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2357497688528461e+01,     7.3898461631546397e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5050715279935787e+00,     9.4275722404757798e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.7099706264886834e-01,     1.1847831208606640e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7785511537637340e+01,    -1.6656416447002579e+04,    -7.0636635813411345e-04,    -9.7462094313450016e-07,     4.6112083651371291e-09, } , 
      {   -7.0083337497455247e+00,     5.0267893128407941e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9597465048795225e+01,     2.0597243910167081e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8280630717408464e+00,     1.2933963976123996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.3002602072675785e+01,    -1.7429294307611103e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5277601058939965e+00,     1.6097438737500921e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -6.3575433110814021e+00,    -5.2206062945239255e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.2808272587607597e+01,     2.3966617546993337e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7957543752828492e+01,     1.7672152448589055e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.8455325305729762e+00,    -1.3302114994919630e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2945945029614185e+01,     8.1022212780856235e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    7.7740691559638620e+01,     1.6602025919263913e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.8118574010111146e+01,     2.7959389766179389e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.0492189811018726e+01,     1.5485666391519189e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4536848932674957e+01,     1.4603953703360021e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8528227115102641e+01,     1.8472415826566910e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2295107529012341e+01,     1.4946085894371756e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5096115596607259e+01,     3.8891830614469887e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.0204080623863817e+01,     7.8526797317598639e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2044390679800257e+02,     1.5542712264417334e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4752105102339772e+02,     1.5542796315950582e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8644332117249164e+01,     1.3562659702963074e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3037437888109409e+01,     2.0763360584707883e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5863560117651055e+01,     1.6102698402133288e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9372406269274791e+01,     7.7064173140745306e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6824869486208303e+01,     8.3192489104854030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0226684660292136e+01,     3.0159781229996883e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.3165713604106628e+01,     8.1123439184687086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8505957442362186e+00,     2.1329907977851814e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3707551946225774e+01,     1.7981062201670196e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9404698597310393e+01,     1.8485217705922114e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3337270012758591e+00,     1.4555207162948393e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3429655051092430e+01,     8.3285506374521137e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4353605845321464e+00,     8.3288322119981385e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9274821221484597e+01,     2.3906782785844618e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7189694500871703e+01,     1.5622314755737261e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2123473782587197e+00,     6.4864284855442002e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0458356987469820e+01,     1.7965314458154549e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.6931023489308585e+00,    -6.1028337684433814e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.8816636636136614e+00,     1.4927564616337870e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.5368415537693053e+01,     1.6171055161678778e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5728860719161361e+01,     4.8701865665076184e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -6.7312031170558015e+01,     1.8660019341802195e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1380481937033270e+01,    -1.2466257640848908e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.3910152081730711e+01,     1.9113724444923810e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.5696547066147359e+01,     2.7488455880111769e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7350124096047892e+01,    -4.1369104726755222e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7636567483331662e+00,    -2.7385275495285332e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.2035209719269162e+01,     1.6712739779636591e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.5740671637523164e+01,     2.3816088784722742e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.3332579113732752e+00,     1.4423716503748594e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0562975654366710e+01,     8.5170787013752597e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.2766751235289547e+01,     2.4360067888969566e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6257079491765538e+01,     2.2836999248971100e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3538169160564895e+01,     2.3242426784396666e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9424239742712111e+01,     1.4957817866560996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8801254534724848e+01,     6.5853962509044877e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2176787401951009e+01,     2.4908522013163256e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.6278102517223783e+00,     7.9537041390194645e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.4413082819636625e+00,     1.5498983456674800e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.1232075420186902e+01,     7.2117853491007236e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4536594564615488e+01,     8.2975398295383457e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.2280208763865012e+01,     8.1240577316822737e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.1614745247103315e+01,    -8.3103291082613505e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    9.8245233862792709e+00,     2.3639556180364241e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1827026678321129e+01,    -8.1245564045593510e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7572794800752746e+01,     2.3261162338064743e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.1107468302037850e+01,     1.5525893543489776e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    5.0203667795013004e+01,     2.2615182860776073e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.4349377511246459e+01,    -1.2098003913505472e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8919190000313016e+01,     2.4916184496155405e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2938393430940263e+01,     2.6060316718328595e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.3669400583932621e+01,     3.8825707321945418e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    7.2138205967996978e+00,     5.8798954343556225e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4584036306011473e+01,     1.1337562948599225e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6101929878702450e+01,     1.0613124695253751e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8475266256236331e+00,     8.3286809025213151e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1468473854793544e+00,     8.3287019469289371e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0076728814806739e+01,     1.4449455638895517e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3545657786686808e+01,     1.4339819562770939e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2855244000327648e+00,     7.4067515845953712e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.6838136443394234e+00,    -8.3598430199119066e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.3467187501226270e+01,     1.5433721801508604e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.0687028826721324e+01,     3.2218161347835412e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -2.6650514794534985e+01,     7.5809391339858466e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2262691184214491e+01,    -7.9915985058542992e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7999814484041888e+01,     1.5038399631975883e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    6.2143697683020527e-02,     7.7210024021905574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.5123135806168193e+00,     3.3326578729832972e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3717662129272682e+01,     2.2050957309585126e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.6357860209409139e+01,     8.3110661120001951e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2296474667632083e+01,     3.1945705699958307e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.0509399096516727e+01,     2.0587592546380125e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.4303474512081031e+00,     4.6337683431478063e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3785143504843205e+01,    -9.9518385357255647e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2845569707233501e+01,     7.2501454928015337e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.2162773151195829e+01,     4.0183743840604041e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.3061304168567698e+01,     2.1484202477227216e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2532999545428515e+01,     8.0596233818696064e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.4528987783703924e-01,     1.1692726686582393e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2100704241258503e+01,     4.9123674700862057e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.4177303816076572e+00,     9.0617737426860294e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5637605433090059e+01,     1.6683121984597172e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7212018907676754e+00,     1.3367972634193943e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7573714570640391e+01,     1.6858482456470683e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3279000195087281e+01,     3.0616108286955950e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.8467345009434524e+01,     2.3908205476778370e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.3858226851630082e+00,     8.7062932833789237e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4150287442802583e+00,     6.0453616815889245e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.3607251259542146e+00,     5.3422580215216585e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4841243661561904e+01,     2.3230020218664085e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.7032661184665967e+00,    -8.4400251132402154e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.3367658801314278e+01,     1.6764424571265059e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.7592778819907814e+01,     8.2560614977719270e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.4253886936364690e+00,     8.2930351902482635e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2188831131954062e+01,    -7.4962488195138076e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.3707001961883538e+00,     2.3207089713461130e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -3.8606824874267609e+01,     5.7121968638077487e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.6426831547074705e+01,     1.5513903894836909e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    6.2805634897976930e+00,     1.4497304163470762e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.0928360809886186e+00,     8.2309202732886060e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.9348164895327876e+01,     1.5025254912448723e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.4992457691609346e+01,     1.0490587524668716e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5469893106176560e+00,    -1.0086903648439107e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2923487031725671e+01,     1.3344664026206201e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8075301990169272e+01,     1.8055984630366132e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3805242269212981e+01,     2.3857939203637732e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0799102581055834e+01,     8.3685331349507433e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -6.5073135163732573e+01,     1.9288326926847738e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3398460383092210e+01,     8.5328264448909013e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.8845148547698209e+01,     2.5672257253433861e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    8.6225302687954457e+00,     7.2846639889456810e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.4423513877545666e+00,     2.3265680034994599e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9358490895311760e+01,     7.3380157922962971e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.5281451019717339e+01,     2.0890577753024054e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.5167148998906701e+01,     3.9256465650752609e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.5045773946560308e+00,     7.9874855665224286e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.3784392773073705e+01,     1.3186929671865768e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5674273274737702e+00,     8.4772369911496462e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4775245961422016e+01,     3.2029284874094326e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.8359453239046488e+01,     1.4406795822939814e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7035272805340590e+01,     9.0989856205954620e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1875186078622832e+01,     8.0109215162045257e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.6122639539055736e+00,     4.1036765354178970e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.5743358371384517e+01,     8.7374448785657041e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.6382955803783812e-01,     1.4917291619196207e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2266569658939259e+00,     8.1703982195811550e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.8902088282268776e+00,     8.3080728622141505e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2684054470666837e+00,     2.5063626535187501e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.6178873443077823e+01,     2.4457093483998924e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5448465811351248e+01,     2.1753537544259634e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0043745935315542e+01,     1.8313508534933335e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2129661376136278e+00,     7.4055786473022281e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.8252003337755331e+00,     1.4886060771728156e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.8537070079875306e+00,     4.2407473734548203e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6088300133494563e+01,     2.2725638752128307e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.9034631254460432e+00,     1.2719030948223392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5387469254629433e+00,     1.4945625108652897e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.9514795473646586e+01,     1.5732484147591094e+01,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.7744176846336060e+01,     4.2118969781077740e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2699929358351245e+01,     8.3515856458142571e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5792903048559985e+01,     1.8819278949393840e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0903118494800772e+00,     8.6157931515679720e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.1102044969716204e+01,     7.6983062968958820e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.1991814288655771e+00,    -9.6589029242170891e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.8908818944905661e+01,     1.3919262851565858e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.6273726160793295e+01,     1.4898714220990827e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4837377161180369e+01,     1.0287237317590334e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9350047007672828e+01,     2.5650430275623334e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.9046880598951290e+00,     7.8600024767800332e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.5233179219534883e+00,    -8.4264625761616462e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5757942871180290e+01,     2.0949446661518283e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.3197622954302819e+00,    -8.0180863447385457e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -7.9959296529396326e+01,     9.6632054301336102e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2546753950986158e+00,    -8.3444391682407640e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0684159330544308e+01,     8.8620814420706502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8668931618853506e+00,     5.9548178630515622e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9192098358516489e+01,     3.7724232957992397e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.2533450601747731e+01,     1.3562271561169538e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5192778561280537e+01,     2.9184838081715643e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.4762445744077096e+01,     1.8436367410164501e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0607444311084198e+01,    -6.6113995532934311e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.1078903674687117e+00,     8.3493099872361017e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.3666918717427770e+00,    -7.7069427952793103e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.6710136536335312e+01,     2.4820787449953923e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3861158973487077e+01,     1.6960368056094171e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6653409706623250e+01,     6.5791963248135598e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.3081583890442152e+01,     3.0639856642155919e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6927679120827996e+01,    -7.7959850440470800e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5573903300225648e+01,     1.7268058408220139e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.4021432943914050e+01,     1.8325958098330466e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2896126466962953e+01,     9.2659087628518064e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0286569709847665e+01,     8.3904959422216125e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3665144150880469e+01,     2.3276925187163564e+04,     5.7861276565536288e-05,     2.7681823690076927e-07,    -1.2660472949862531e-09, } , 
      {    1.3411876328460878e+01,     7.8085833932043488e+03,    -1.9023751901229817e-04,    -1.4892438782407371e-07,     6.4858859072514825e-10, } , 
      {    2.5566802088779577e+00,    -1.0247389977567658e+04,    -1.1872163480310021e-04,    -2.1410409659074682e-07,     1.0661973993592805e-09, } , 
      {    7.2678043632133260e+00,     6.4099928718825931e+03,     1.8590386647959628e-04,     2.8671813226902599e-07,    -1.3064807559907884e-09, } , 
      {   -3.7256973203866330e+00,     1.5271049853741200e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3553601969341004e+01,     6.7453739175137398e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3716655353190514e+00,     6.8276008513655725e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.5301950359886627e+00,    -2.5032409341759776e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.8011909023350078e+01,     1.5511575886669474e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.8756768956595182e-01,     7.2272170607721118e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8005646831304890e+01,     2.2792865572045197e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -1.0730411509543803e+01,     1.5541626027637365e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7807464102943371e+01,     1.5543882552730553e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5594841630489888e+01,     2.3884952226180438e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1019057397768659e+00,     1.2556320886980491e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5239366164148631e+00,     5.6878900802074475e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.9318871241395229e+01,     8.0907684157853309e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.3072409455425841e+01,     7.5636661372189901e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0135417322199349e+00,    -1.0541027862448933e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2935851762084312e+00,     7.8488795366507338e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.6542302441834213e+01,    -8.5468773889392760e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.2093875958373069e+01,     8.3325793175189610e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.9946577756466660e+01,    -8.3248035319312912e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1330281547427489e+01,    -6.4405532856118225e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8415509765347881e+01,     1.0785033020198684e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.4361620718175715e+01,     1.6633109975840773e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5503366798564109e+01,     8.3021728081277342e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9683664331035300e+00,     7.1993443570342733e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8541969549845760e+01,     2.4355465489290102e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    3.3490850854958552e+01,     8.5689090722336005e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2558258271927464e+01,     7.2009086701455535e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2283116997320903e+01,     1.4927953216409769e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6428622927358365e+01,     1.3991892778519057e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8800723080211924e+01,     1.6295014802066969e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.2531202419247531e+00,    -2.3499138213795281e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0819590062630340e+00,     7.1500731415252776e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0422233495962809e+01,     1.4914998994670899e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0006770872401509e+02,     1.6681655723059732e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.2600590694262419e+01,     2.4426458794702499e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    1.4406107639897892e+01,     6.0814100979913319e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    5.2422134051983615e+00,     1.6344562924657814e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.0797391695469976e+01,     3.2750892463667627e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -4.3924637264894191e+00,    -2.7849282153538770e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.7433576538734066e+00,     2.1671054251091595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.3395130435222207e+00,     1.3184628472026034e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0463547839126939e+00,     1.4924755986393904e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7478524156171611e+00,    -1.0622382893070382e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3353537137324077e+01,     1.4862188953394101e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8549399541506219e+01,     8.3338119973610737e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7924650048714792e+00,    -4.8632660401781118e+02,    -3.7347750399898598e-04,    -4.3625891220726239e-07,     2.0095090749677269e-09, } , 
      {    2.6545187279295369e+01,     1.3988719354767385e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0365952206071665e+00,     9.4547947123790182e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.4067622520170875e+01,     1.5697858812208207e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9349769645452348e+01,     1.5571604685531001e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    9.8505585285161583e+00,     8.3232959306324192e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.2810420376021501e+00,    -8.3340869188178331e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8193838345035516e+01,     2.1872735526180386e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.9871785812583269e+00,     6.5147619750008926e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2578724141239672e+01,     3.0457912340789251e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3728601506540464e+01,     1.6278439632133342e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.8398731006550086e+00,     8.9349577563259409e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.9419355237829885e+00,     6.1335125190742974e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3773638965448026e+01,     5.0317203449967128e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0554574359024684e+00,     1.2240852178299627e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5260785002029003e+01,     7.5913513510042321e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.7523330489278202e-01,     2.3856026603869897e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.4496376040150167e+00,     1.6153037667028297e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.5379487981561162e+01,    -3.6082627342699197e+01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.6798693352513645e+00,     6.0257043552370919e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.5486628203800272e+01,     2.5612080659381187e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -5.3935006998441715e+00,     1.2902941190848134e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.5403217845363333e+00,     8.3533108726398823e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1160188451926247e+01,     7.5890501511644943e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.1732764830874505e+00,     8.3537192144051733e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4674301741569771e+01,     3.9478634354905713e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.2423511916889195e+01,     1.3030158359164298e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3289768037387082e+01,    -8.9443797714482898e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.2178418529459620e+01,     7.2170450137330927e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1496374781723572e+01,     1.5110466097669896e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5261274661291182e+01,     1.3515516291746086e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5318603060457319e+01,    -1.5419111039189374e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7154974672956360e+00,    -8.1065227205720257e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8137111513547396e+01,     2.8750997146883792e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3374754525939148e+00,     6.8367896392814846e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7489860450053683e+01,     1.6550964933723037e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.2916639393511886e+01,     2.2650878961220409e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9456793128711965e+01,    -6.9017226619712283e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -3.4932762661897971e+00,     5.7533126752019707e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.9935732427698042e+01,     8.0898020133376522e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4942088333518726e+01,     1.5529600094870681e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.8716151904051188e+01,     7.8180975768948938e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1389757062075880e+01,     1.4486994882215273e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6887521032165694e+01,     3.4229676337911813e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    7.8128294724001091e+00,    -8.6081306685758191e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.6417652497605431e+01,     6.5084949017659937e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0902922948394856e+01,     2.9970527705143548e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    5.8738336223822483e+00,     2.3013539513374071e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0145527094243832e+01,     2.9971232337059693e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.1319594890821801e+01,     1.3802447978188307e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.4824118319767514e+00,    -5.0434518242195537e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2075163290457979e+01,    -4.6899338702421119e+01,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    9.9797633821980849e+00,     4.3942163551833073e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.2021982609269841e+01,     1.3228598459491346e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6194185441135776e+01,     7.3096640698263291e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8086463952687243e+01,     3.9467169102304237e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -5.9404041050357748e+00,     1.4540570559768776e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4807033940943173e+00,     1.6628996916039996e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.1379376068492459e+00,     1.5555908485497237e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.5868876559352323e+00,     5.4198102825337828e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3325253684759197e+01,     9.2386862909485681e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.8305602639153371e+01,     2.4486635388755174e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3948922626171665e+01,     1.3545386706196219e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8455494504691288e+01,     1.7269481099153891e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7519808027135078e+00,     1.8849227551366296e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0437673355783488e+01,    -1.2569674860491614e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5139160879613001e+01,     1.8374083171363178e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8807236335743877e+01,     2.2948891788289729e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8886759715036053e+01,     2.2921110465582948e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.0267213022361531e+01,     2.6065278223126850e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    8.2249966608931935e+00,     1.6685768782860512e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.4106564370052901e+00,     5.6666978913385677e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.1186840917250560e-01,     6.0803065705989102e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4045365450695884e+01,     3.0417386093424462e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.9441887320943184e+01,     1.4755187097546768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5705154343567902e+01,     2.3455014518708307e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.8702938191089629e+01,     2.4431061194381964e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8516128973066792e+01,     1.5666707217021425e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    6.7130914973803142e+00,     1.2507154559378625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0021592533422793e+02,     1.4412583702629236e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.9568481017791669e+01,    -3.5133424023858379e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.3496682619140357e+00,     1.5025780393653504e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.7704641568115079e+01,     1.0291513015179155e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3716917376693317e+01,     5.9574409818170188e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5102321354678326e+01,     2.3221808221885691e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.5867796340629425e+00,     1.3187483200640218e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7594269096665116e+01,     2.3848903809778029e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.8535682130583627e+01,     1.2543139702513527e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.0399693804857833e+01,     8.3083412815300180e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.1969210295771845e+01,    -8.3490415679202342e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2436033663231825e+01,     1.4888138314511858e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.5203596309964666e+01,     2.3141558306926117e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3271608431437157e+01,     1.7994180112869646e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.8860405245994105e+00,    -9.7304217108342655e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.8688089729884547e+00,    -1.5256832129466065e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1359510351087248e+01,     1.6163583708660444e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2876553826530888e+00,     8.3155285597331367e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4349535845594943e+00,    -8.3418542897171155e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.9936880712470856e+01,     4.0528476248401261e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    9.1820178152366054e+00,     9.6808665686940021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2401716230408226e+01,     1.2041648190289770e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.1147291720782718e+01,     4.0529180880317421e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    5.1422949902463078e+00,     1.3072573442882340e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -5.3963811094038377e+01,     2.3926802645095428e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.4851592622854071e+00,     5.1697389653045479e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1329257884206291e+01,     6.6703635668578199e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2827374722327157e+01,     2.4346949977770120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0229527216089668e+01,     5.9724040285058591e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9588100802242405e+01,     9.3096795963609657e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.4019013702648692e+01,    -1.8779228782940027e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1008840914928069e+01,     9.0999486622344239e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.8684192392860666e+01,     8.2958763996698399e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4497061987977684e+00,    -9.3657677229792953e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.6313680833698189e+01,    -7.9717530823748348e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4079561139000294e+01,     1.4942832638548673e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7888674348546592e+00,     6.5813460532442195e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2157745712293693e+01,     3.2863247644877447e-01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9751388607846369e+01,     4.0511907883550557e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.5672527107718592e+01,     6.6710749123246969e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3600843456965777e+01,    -1.4038078288928396e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.6525113034958228e+00,     7.7849921261241134e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5842364399022124e+01,     2.3855236379567177e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.5959882341008601e+01,    -2.2517990236636433e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0478265671154336e+01,     1.1788327890949889e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8970582267915823e+00,     1.0529082888542620e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5319055097349938e+00,     6.5731360420909105e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.3470149021247408e+00,    -1.2566639308221238e+03,     1.9586472716825521e-07,    -8.7266462599716787e-11,    -1.4544410433286028e-12, } , 
      {    1.8688184233485853e+01,     2.3803931620634194e+04,     1.4778323338147806e-04,     3.8829212533743842e-07,    -1.8427768018973463e-09, } , 
      {    1.4405738203299670e+01,     1.6694613477928142e+04,     1.4738411051850466e-04,     3.1996801199782510e-07,    -1.5689976680379801e-09, } , 
      {    1.6969525468910543e+01,     1.3624031356975906e+04,    -3.2499073021548791e-05,     1.0021030914618754e-07,    -4.2959825096797100e-10, } , 
      {    2.2548225338686301e+00,     8.0409437394175802e+02,    -4.0450267070790962e-04,    -4.7313868892926478e-07,     2.1848177020569351e-09, } , 
      {    9.7377313531317675e+00,    -5.9107132677059235e+02,    -1.5457750927128333e-04,    -1.8147060897611042e-07,     8.5812021556387864e-10, } , 
      {    2.3020718809788928e+01,     1.3766066728531738e+04,    -4.0350856025479450e-04,    -3.3657772871263786e-07,     1.6358049932948740e-09, } , 
      {    1.1708368469301083e+01,     8.4369097288183511e+03,    -1.9299933286879084e-04,    -1.4826436247861120e-07,     5.9487608299502271e-10, } , 
      {    7.5159828496036720e+00,     5.9912160020993142e+03,    -2.4400241086066759e-04,    -2.2466401730446609e-07,     1.1603433680939410e-09, } , 
      {    2.2423556961381095e+01,     2.2581024736949515e+04,     1.1714979612689093e-04,     3.5123781569023659e-07,    -1.6703770568947951e-09, } , 
      {    1.9588417810548705e+01,     3.2342172511190041e+04,    -1.3257210717393018e-04,     1.2798111553929530e-07,    -6.1600426321777652e-10, } , 
      {    1.6564621278469303e+01,     1.5333204824353237e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    5.6557705794098476e+00,     7.9099389353073657e+03,    -2.7769145906249971e-04,    -2.6092740191230583e-07,     1.2812122669313596e-09, } , 
      {    6.7030410483137377e+00,     7.2452412689733155e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -3.6110055417280673e+00,     7.8363369761094273e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.7118735053798137e+01,     2.3558246183181909e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0715567961092077e+01,     1.1832098724459049e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.3841624425608892e-01,     1.3906236061599338e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.0334022294688570e+00,     1.4896422496937210e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.3191329425174985e-01,     7.1986330115673973e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8843392618106805e+01,     8.4000370460955983e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3370252063386715e+01,     1.2638804180148532e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.1537473922067019e+00,     2.4202589681308591e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8861685331387584e+01,     1.4929865816177604e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6020862170542284e+01,     3.0459830827638558e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.7729377453281383e+01,     1.5543147536446400e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.9313330418326284e-01,     9.5559973877790526e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1161817901005186e+01,     7.0405960269708121e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.5275208654144244e+01,     1.6498123241858604e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.3144065346418188e+01,     1.3657479219089253e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1430495057902142e+01,     5.3270638067804612e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1667113559110199e+01,     8.9853324992273629e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9602049377660851e+01,     8.0192829987669129e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    5.9034386305543718e+00,    -9.1305000591989810e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.4798103496968444e+00,     1.4264809212115020e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0776240368852168e+00,     7.7805659330078897e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4015370396183924e+01,     1.5541590828676937e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.0818842468858003e+00,     8.7973803726702190e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9561798107142774e+01,     3.2681206843813670e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.9273201938422164e+00,    -9.0437495344848248e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.3892194679397292e+01,     6.6826580809759698e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8671185290563184e+00,     1.6028363918937834e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9962938757343334e+01,     2.2821251505455457e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    2.7469273191852057e+01,     3.6016230770495558e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.8840033515336252e+01,     3.8788193619887243e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0035463074841655e+02,     2.2128502857046522e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5784645531667246e+01,     3.1593159462566520e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.0221586487952958e-01,    -8.2669128783503002e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.0524598393168414e+01,    -1.1155949617139720e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8633586113856722e+01,     3.0202769555646468e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.0994169361601145e+01,     1.6816642457041900e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5522637297700898e+01,     2.3843059781498829e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9902856894087044e+00,     1.4115785527430060e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.0065690042478099e+01,     6.8886381203705332e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0943331448011747e+01,     8.9363804472596930e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4360687355843964e+01,     2.9990526415258482e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3260198239989172e+01,     6.4537580755946919e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -6.1767944767814676e+01,     1.4443667759206097e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -6.5685992782440650e-01,     1.5542361043921517e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7857724716030843e+00,     7.0125576776092421e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1857682161846842e+01,     1.5072642651305116e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6855981921590715e+00,     3.2164088723231802e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -5.6414488642236105e-01,     1.1299705716889404e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2806897262023643e+01,     8.0006636556483463e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4674253111537999e+01,     1.8387201082562635e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5049801179469412e+01,     9.4482357567792897e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.8702695842671254e+01,     6.6411122105996310e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3692810101169847e+01,     1.5515524328119691e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.5545393296329877e+00,     8.3904699710999521e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.6252793728345321e+01,     1.3432534325700970e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.6800484448056253e+01,     3.0791262238794101e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.4705087271648615e+01,     8.9619300419265892e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4246659733887448e+00,     7.2981740820706846e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.9812248480807739e+01,     7.2110807171845727e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9538809501029267e+01,     2.8352410735872378e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.7652478899769839e+01,     6.6809271124975439e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6982914910551447e+01,     1.2487628781407831e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4952962173763702e+01,     2.2678820268751555e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.8816933569143295e+00,     3.1339940020043818e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.3365164047019622e+00,     8.1371756428817298e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.1990758181118917e+01,    -4.0392421856052279e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -5.4013227949338258e+01,     1.0603032353519826e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.7785412634725830e+00,    -5.6136272642169133e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5076242924501372e+01,    -1.8225659481076611e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9392774092107263e+01,     8.0155239158839977e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0769904372104008e+01,     7.1191452977588133e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0298256772072985e+01,     6.1796333654679993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3484541518165830e+01,    -7.8735373050592052e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.2285789519713923e+01,     7.3023291676380850e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.2435031781709389e+00,     7.4373159016202517e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.6274343625037559e+01,     2.3847172841299602e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.9906644444411086e+00,     7.7857034715909904e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7371942150345863e+00,     1.4899027594099225e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7411171270271918e+01,     2.0714160046165729e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.3345180741587271e+01,    -1.9528379261006335e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.6710635815953502e+01,     1.6054814399551631e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3235887849188703e+00,     1.3760003243974210e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.7206648074865441e+01,     1.6658349251897933e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6013965736500150e+01,     6.5871779713470414e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7963760615734873e+01,     2.4342018945614196e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8883616004027608e+01,     1.5527006546668321e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.9849894365895555e+01,     1.4218223949128869e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -2.5810254299020301e+01,     1.5527833852021846e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.8060745055955380e+01,     7.7130030780019615e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.3010573791114165e+01,     1.0575483554367962e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.1209252926288912e+01,     1.4218928581045018e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.4188026178375981e+01,     3.9361230907881851e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.3136590129405862e+01,     1.5447836722483940e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8035988134385988e+01,     1.4436762229301092e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -8.1137036898793617e+00,     1.4216275065247210e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {    4.9473002590440558e+01,     1.4220877464926678e+04,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    3.9441431986597211e+00,     1.5793788649052154e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0705234219832207e+01,     4.0200241944479021e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.8364047502183890e+01,     1.4411205050108882e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {   -1.7594259662465994e+00,     3.1711514965573726e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -6.0341505367987605e+00,     4.2341884178550927e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.8682197239616009e+01,     2.1953707934717168e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.3012742266950340e+01,     7.8423637369536491e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.4743509947794635e+00,     6.5677063370579162e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.4135901713157267e+01,     9.0148744039836092e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.3907385173398833e+01,     2.2862755350065174e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.7847531810616822e+01,     7.5965755092119007e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.7123621838792165e+00,     7.2103649837566381e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1819086579911806e+01,     1.6913332981106319e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.8678313333968637e+00,     2.6087903140599633e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6493128806153713e+01,     1.9734354101113513e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1201037193562442e+01,     7.1978535301169286e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.0959363907283063e+01,     8.5675808361126001e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9060429030484215e+01,     2.3147966297779290e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.7796202806653625e-01,     8.4482435358427501e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5439152727240943e+01,     9.3706834957671672e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.4080934509398766e+00,     1.7389301705904134e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.0578712182778538e+01,     1.6648522694010793e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -2.3142523125375934e+01,    -1.6666243004889715e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -5.1887845354710407e+01,     1.9800945872208840e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6667814315606364e+01,     1.4498726854404513e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4477209284622148e+01,     3.4570066339085797e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1874035390242405e+01,     8.7194111945783807e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.3770139429153659e+01,     7.2627647311239070e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    4.6946990961000914e+00,    -1.7455659766280214e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2178122278890921e+01,     1.6650771449896958e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0339082498670722e+01,     2.9864334032034676e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.8487851648367098e+01,     1.8620204439904279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4250439851741865e+01,    -1.0682278934409514e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    9.8830942510293536e+00,     1.5463193824630654e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.0769959878092862e+01,     1.5232216613659015e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3752135866115742e+01,     8.3283733255206898e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7578414144323542e+00,     8.3290095239295606e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5509680429074951e+01,     7.2074514659055403e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -5.3926125801715081e+01,     1.9328141828745647e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9091362066342278e+01,     2.3147578155985753e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.1765469486771508e+01,     1.8954265683376008e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.0588401526798094e+01,     1.0959635502122610e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7941137581930576e+01,     2.2582015519762710e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.7324119603333273e+01,    -1.7584781726002666e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8135775804485995e+01,    -9.9919977652411380e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.6361627460183410e+01,    -2.4840452655201079e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0393703958319039e+01,     1.6520312011413331e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.5417258280977791e+01,     9.5100555336438247e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5514127344480547e+01,     2.3887445223233568e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.9060642431268811e+00,    -8.8925646599579995e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.8669711899953874e+01,     1.6188334871996358e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.2887300859401535e+01,     1.5557674728346072e+04,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    8.5872633610241152e+01,    -1.3241847386477416e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.5307210197008104e+01,    -1.3241713115462885e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.5429829653090422e+01,     3.9397279324284260e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.9132976561620843e+01,     3.0870892042589927e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.6919844942424703e+01,     1.5578091361119494e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1401277814681224e+01,     8.3244260521131855e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5438725219012220e+01,     1.5565045729931278e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {    4.2515777812411791e+01,     1.5520462850436639e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4613764197704493e+01,     8.4184202838056044e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5711982914064226e+01,     1.4112657022236801e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.0511198699688180e+01,     7.6519324393982424e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.6724091558525537e+01,     7.6954528075236631e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0008486165619086e+01,     9.2572735037103303e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.4335087670291777e+00,     2.4977437775791954e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -1.1275817223459637e+01,     3.8477264037618315e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0939878119901827e+01,     8.1732582702244290e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0685414658942817e+01,     1.5472924446278585e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    4.3841355183869668e+00,     1.3886123749096839e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.6780970838276858e+01,    -1.7452373441515701e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3762200073076068e+01,     9.1611340299364456e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0447253431131593e+01,     6.5224769019466103e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4923362130463286e+01,    -8.3195178961772617e+03,    -5.8714766810445565e-04,    -6.9223636643543985e-07,     3.2705046113968184e-09, } , 
      {    4.0019659948940109e+00,     3.1083431037584214e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.6271870446504778e+01,     9.9747247684742833e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6828787379433752e+01,    -4.8451400281340284e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0473839134527251e+01,     2.3825045049083739e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.4109227686979878e+01,     5.5240958183581870e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.2056641717673779e+01,     2.3512206172719059e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.6112144477667641e+01,     1.5579514052053246e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.1201421027880173e+01,     3.2657592459139869e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.2089718385227926e+01,     2.5761299502201637e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7344874644518807e+01,     1.0438412716331040e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1129595419393059e+00,     7.7706657996251024e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.9595911765112227e+01,     7.8420418180279266e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.0534779188886446e+01,    -3.5231595807488247e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3869169266087553e+01,     3.1087586123151621e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6934727422412280e+01,     3.4165241988099151e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.9160036280574094e+01,     4.6049179478007609e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.5724173424254282e+00,     7.7720884905588564e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2959172728505541e+01,     9.3565095251626663e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6728044597543068e+00,     9.4374199299050033e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9823953222207990e+01,     7.9790881529649687e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3776716670555228e+01,     3.8863444681059620e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.2231268132243699e+01,     1.8045995224359936e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.0496080410574500e-02,     5.0731899786661379e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.7153960857462103e-01,    -7.3568534194938411e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.4205124051907596e+00,     8.6541161698134256e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.4084345481508862e+01,     3.1053767652494389e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2491746305426181e+00,     4.3373704262870733e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0987965014195069e+01,     3.6611416540743558e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.5524400500987809e+01,     7.3905954698173373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6265341091632560e+01,     2.3899831648319341e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.7878973366975984e+01,     1.3658183851005406e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0516924719122951e+01,     1.5564795543628687e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.7835215019523041e+00,    -8.3003397022551562e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    6.7526463980394205e+00,    -8.5202072065685206e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6699050502301297e+01,     2.9513424701595970e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.9439986255863772e+01,     1.4774761294153357e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7470108444532435e+01,    -8.3210032185586660e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.4459871209617990e+01,     2.1156381554400872e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0806455794003732e+02,     2.3895718588518568e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6577361737724715e+01,    -5.6888783253893016e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.1689857827765671e+01,     2.6458299192819839e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -6.1158403762769815e+00,    -9.4831348858893325e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.7782443802363561e+00,     1.4895259035430188e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9401814605795394e+01,     2.0855894892181674e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5198531956738982e+01,     5.4247413146896997e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8892082248323561e+01,     7.9159667799052522e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.7068586662285938e+01,     6.6291264418358696e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2234802519379020e+00,     7.7047930668486533e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5475076857672114e+00,    -1.1314893059604765e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1623803310149643e+01,     6.6544473859859499e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3857252261076891e+01,     9.9184131533965847e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2854458001426430e+00,     5.4873360564889008e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.4212839039964635e+01,     1.5540100774386150e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.1289891633364206e+01,     1.5545407805981768e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3360946561003574e+01,     1.7303714642696999e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9348879134264713e+01,     1.5546567581997095e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4387822980184028e+01,     8.9549214669869689e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7512364644663005e+00,     7.2102495736456976e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8300877348317091e+00,     8.4236089924251446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8887785886287080e+01,    -8.2327768472492280e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.6893373756294308e+00,     7.2020652549114566e+03,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    7.8849865474999650e+00,    -7.3411442974895599e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -8.6834430362645548e+01,     6.0403471143606112e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8543450109309088e+01,     6.5722487691419356e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4618180814962907e+01,     2.2615541890284876e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6135584525432389e+01,     2.3296818572541608e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8000784996101729e+01,    -6.6921731961405067e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.7252530063221023e+01,     2.2694684005669857e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2311963043564784e+01,     1.3658542880514206e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5503627592208971e+01,    -1.1699854894526352e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4753529761059376e+01,     3.1106838488345769e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.2602085782759085e+00,    -1.9207463264693479e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.0773279673322609e+00,     1.7455424998428603e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -8.0895161355129773e+01,     6.5614824068038070e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0018093349738271e+00,     6.7086141556444536e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9603246717998299e+00,     1.3513215091906352e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9352044094601162e+01,     2.2763376111242516e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.5233473278206873e+01,     1.4891904800007360e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.0976460285736032e+00,     8.3426985876828094e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3736420875058570e+00,     2.3599741278466325e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.8792414617006266e+00,     1.5542358456096139e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.7197811131698941e+01,     1.5543150124271771e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.5876936206717716e+00,     3.2222679044765264e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3584147861054152e+01,     9.8140342545690401e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6511790031829598e+01,     3.9374385103195134e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.4105047261426723e+01,     8.2108082563964308e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0431702376153165e+01,     1.3957552500212554e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.6926355457181810e+00,    -6.8782616207034164e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.2900966514322477e+01,     5.8622758632834921e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1648008711884422e+01,     3.0552802199689871e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5191099364829919e+00,     8.1071443483470284e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.5447188035446837e+00,     1.5520713036739231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.0229281185160364e+00,    -1.1977213840477101e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7622410765058302e+01,     8.3651330873899788e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6604842440122889e+00,    -9.7490893734094641e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.2125737080467786e+01,    -1.1133368851508840e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.0790820068459368e+00,     1.4930179189286004e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    9.6981172921958205e+01,     3.2200130426083488e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -6.0481844532435375e+01,     3.2200143853184934e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.6571969644918237e+01,     1.8767104141056167e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5332237544155589e+01,     1.9506745414616798e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.8642669998607580e+01,    -8.2599993191524663e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.9162483147585228e+00,     7.7441546731887429e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.9632482764718823e+01,     6.6563962698676105e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -6.1955128852169139e-01,     8.3643465948650955e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7684323631991077e+01,     6.6571009017837587e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.4364958096945685e+01,    -1.7979259674707533e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {   -1.4605905719564035e+01,     3.2182864142867358e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.7575950336332333e+01,    -1.6674655846217109e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.1237711881591483e+01,    -8.6567191938019059e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.3371856927930050e+00,     8.8723907233261380e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1470556411599496e+01,     1.6561822875572463e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.4814714908496995e+00,    -8.6741048928035525e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.4047062715807286e+00,    -7.6811961699713556e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.2329057916771603e+01,     1.6253545168397543e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9675090721631143e+00,     6.6173944696466287e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.2203046181886265e+01,     9.0348978450880677e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9397800286324653e+01,     1.5431420601668866e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.9015457111908223e+01,     3.9989538492927393e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.5206418795557582e+01,     1.3579950641558757e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5128884487696325e+01,     3.0614935349662806e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.8120823647582185e+00,     1.2509920221155146e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6207516580042345e+01,     3.1051727152864954e+04,    -1.0144900810142239e-04,     1.6481725902999783e-07,    -7.9204011082864902e-10, } , 
      {    2.4637995376739347e+01,     1.4394344303414702e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    1.1142592667392357e+01,    -1.7767119420177416e+03,    -3.3722250666782941e-04,    -4.0056828648228549e-07,     1.9438071249037621e-09, } , 
      {    6.5984930004953668e+00,    -1.7091490870118114e+03,    -3.9907697535850842e-04,    -4.7441510638889000e-07,     2.2929699380388498e-09, } , 
      {    1.3223217730719384e+01,     1.7251903377195769e+04,     3.3288885413512747e-04,     5.3836203092723777e-07,    -2.6016992901694026e-09, } , 
      {    1.0167876266092417e+01,     1.5995299466698805e+04,     3.3821661712094453e-04,     5.3712924669891241e-07,    -2.4928198336658233e-09, } , 
      {    6.2707502078203197e+00,     9.1327614183013648e+03,    -2.5209198770297726e-04,    -2.2277120773067824e-07,     9.9775140386023605e-10, } , 
      {    1.6052076809572586e+01,     5.9236662678647799e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.0568948022128676e+01,     8.3658976728374964e+03,    -4.8307077592594708e-06,     6.9513264336638819e-08,    -3.8338581088460996e-10, } , 
      {    8.5120735050975913e+00,     5.9452052774551794e+02,     2.8263352852431007e-05,     3.7539802067464924e-08,    -2.2902113481933369e-10, } , 
      {    8.5983248685933482e+00,    -6.6208338275144661e+02,     3.3591115838248028e-05,     3.6307017839139590e-08,    -1.2014167831575411e-10, } , 
      {    2.2364492956577219e+01,     1.3403400095371426e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2912426354458582e+00,     1.8330429215742119e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.4581808520276169e-01,     2.2048879766801423e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.3525189508962971e+01,     8.3304166515507513e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4519483960646358e+01,     8.3269661978995009e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6134078646872851e+01,     3.9454014906990953e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.0274944851067303e+01,     2.2637372449949056e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.3341564690463015e+00,     7.3325411687048909e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.7998069111851130e+01,     3.3314413382942425e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    5.3125685243902510e+01,     3.3315118014858585e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.3094670325693093e+01,     1.5543917751690980e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    6.4535044549914904e+00,     7.3898908379011873e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2745104660518901e+01,     5.8707066464397085e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -3.0238977518815371e+01,     1.9113525290967656e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6390237150410549e+01,     1.5651786778859310e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    4.4109794688175654e+01,     2.1502503185391441e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.7711485984218207e+00,     9.8205932101687686e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6465806203131095e+01,     8.2204709820352473e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.2212652572000721e+01,     9.5856589107742875e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0135922032682476e+00,     2.4044946764337397e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    4.2349667638526691e+00,     1.6752942823328041e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2641706697967502e+01,     5.3500758051778041e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2654666957402521e+00,     2.8333489456692838e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.9565617621377910e+01,     1.5541772628368230e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.6642670214777482e+01,     1.5543735951999688e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4352292529804089e+01,     1.3737684995899186e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3624799880155166e+01,     3.0576645701016110e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.6207628808371979e+01,     1.5894088530422543e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7018969365015536e+01,     8.3155372294118479e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.1715982853916458e+01,     1.5303864878796485e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.9255935742331225e+00,     7.2144174690569398e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    3.2291537023128896e+01,     7.2137082618607237e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.1413823279518141e+01,     9.0057008754357448e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.0188645777468341e+00,     2.1464153569149294e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.4728611252766974e+01,     2.3809312564936154e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.0245900551561071e+00,     8.3418456200384044e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8668531150940177e+00,     5.7221610800151911e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.7284421300861112e+01,     1.0625574258650882e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4778510071056033e+01,    -7.7184327830349575e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0019337380884512e+01,     1.6186809618745141e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4936687585193493e+01,     6.5895580500226160e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.8830297236127819e+01,     8.6782946964852836e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4253677386390024e+01,     3.9411570172776861e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -5.3283700189536116e+00,    -1.0720598670386014e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.0954783778653921e+01,     1.4940755095764973e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.0186787006996640e+00,    -2.2154707637809747e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1644059371273752e+01,     1.7081056741452940e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    7.6524156457030035e+00,     1.5538940998370823e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2969077342004596e+01,     1.7126430826904147e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.8925011790144168e+01,     8.7008366016163400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2380565013956475e+00,     8.1179879731735027e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.0346442077015237e+00,     3.7106555657249100e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8544677268974667e+01,     1.9628036486252022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8728115622201276e+01,     2.4437396492925658e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.4592117156211764e+00,     3.8783591220207767e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    1.0045016697810217e+01,     1.5010006679016205e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4849803326527269e+00,     1.6647073568194763e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6350086103003822e+01,     1.4722372072491480e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.2225067955745539e+00,     1.6667692130705742e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.3930119668525109e+01,    -3.9532216953272291e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.6793219612593120e+01,     9.6406635250025556e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1560610488303237e+01,     1.6661185619059579e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8573508562990199e+01,     7.2178761572719677e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1449167436778378e+00,     1.6653580079840925e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0751398852567082e+01,     2.2614830544818000e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.4736416174512517e+00,     1.5478764566250404e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9718408518953998e+00,     1.0906684747196417e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8328135492120396e+01,     3.9416829837409226e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.6685074386349839e+01,     1.3908537261439073e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7545693778026724e+01,     3.7513529506440769e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.7398178130588008e+01,     9.5876077946559471e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0596489654902097e+01,     9.5830053949764788e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.9355235132328012e+01,     1.4897173708371560e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.3965686933811540e+01,     7.1825826377956000e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.2171005406791121e+01,     2.3867642945299758e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.7465295845467459e+01,    -1.5786094647729183e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.8290279965283801e+01,     8.3501708184869658e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.6720763474369804e+01,    -8.3072120309632828e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.9554151364978559e+01,     2.3775885741031296e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.6290219802927965e+01,     1.6146789001620020e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8350120222404920e+00,     1.6685716338906946e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.7877525432322443e+01,     2.3442212639353103e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5815687541783191e+01,     2.5616683059060651e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5512126100180552e+01,     3.3772221018406162e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.6329145742201550e+01,    -8.6595792444451781e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8202039745808176e+00,    -2.9465822171917316e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1916667722803293e+01,     1.0170954363906052e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.8135925897123748e+01,     2.3814357816244315e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5556759149716584e+01,     2.3385124740688334e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.3199653154292587e+01,     1.5056433315963208e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1572303889350984e+01,     7.3089804331588521e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3941434964019578e+01,    -7.5583972288547902e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.9041090344399095e+01,     7.3560647356516765e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8834627458239368e+01,     9.7272932056410064e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0981925660145777e+01,    -6.9300896438092468e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3339874114576205e+01,    -2.4980040799780432e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    2.3100273769726996e+01,     3.1064178672390066e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9616918811278417e+01,     2.3875248484518412e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6181870504309497e+01,     3.9742786624973363e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    8.9938899951281677e+00,     7.8348922839353190e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.3746126258673442e+01,     5.9574544089184728e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3076724314612844e+00,    -1.8347150386226588e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -6.0017639438384435e+01,     3.8941817365806946e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5301981385497719e+01,     7.4427905252116580e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.2169356121548226e+01,     5.3131406019977176e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7326580254133368e+01,     1.0937093596991554e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.8165849613622896e+01,     7.1991424272967197e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -7.4452494763061097e+00,     3.2826143524840023e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -3.6545031649895449e+01,     8.3425968314727525e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.2413802246843421e+01,     4.0541626237673418e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4476994786580876e+01,     1.2065670284694112e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.5325921842902908e-01,     1.4299257031292327e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.9826749703419715e+00,     6.0950250790234686e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8309375235501484e+01,     3.4622697307604802e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1385946083177679e+01,     7.7437550011021631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2176838253417110e+01,    -1.0816711359721012e+01,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.9213110406645768e+01,     1.3896720946434802e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9045572924991037e+00,     1.4162186193255406e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.5821005634989778e+00,     2.3871222057853054e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.2206061174404400e+01,     2.3871669371965116e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1779458789270192e+01,    -1.5310597554670712e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3009044905481378e+01,     7.3265076943099430e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.9801136342611212e+00,    -9.5753171888100169e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.3414307050480952e+01,     2.5221360889527932e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.7970559074266621e+01,     1.4381740324460368e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4331873146225867e+01,     2.4992107748570324e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    2.8790297660164928e+01,     1.4464174147320073e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {   -6.3359212544438073e+01,     9.3066022815308279e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7718443375165965e+01,     1.7523237019198379e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8357289022253113e+01,     3.4558262957792133e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.3455864692482152e+01,    -1.1149875887750950e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5800336416184788e+01,    -1.1142695297574919e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.0417410113758407e+01,     8.4152267584259516e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.5777879142782627e+00,     2.0272379984378713e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0752904290024023e+01,     4.0135807594666352e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -6.2627728427109766e+01,     8.3296396952188188e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -6.4197244918023770e+01,    -8.3277431542314334e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.4534261987744111e+01,     1.4347943637589358e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.0633518382183853e+00,    -8.5731857795658343e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -2.1711146182488918e+01,     7.9514029391797285e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5633031941967690e+01,     7.3975343693210980e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.4385341632424421e+01,     1.6645892861694607e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -2.6949152575021806e+01,    -1.6668872837205898e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.5559197188059118e+01,     7.1404752057366659e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4029169228178780e+01,     2.3363794832710482e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.3221760179500880e+01,    -1.7983862074387001e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.5783122313598991e+01,     3.3920736112188342e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.1931423713465406e+01,     2.0980613516073114e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2890074875169944e+01,     6.4835835714864088e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.3812832069034169e+00,     1.3120194122213365e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0873625755095862e+01,     6.0008188567903262e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2669853479212383e+01,     1.7286401779962671e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.9544094496206824e+01,     3.1716117365253198e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.6162116066393310e+01,     1.3148968655695524e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9152722002131231e+01,     2.1521531893529558e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8940863248089233e+01,     1.6608931449168911e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.1737512168136405e+01,     1.6854726390461386e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3161887255927542e+00,    -9.0683326982857579e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.2848085385609728e+01,     8.1083252475346835e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0365206447229852e+01,     9.5161281622631159e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0684412957071643e+01,     1.5599842188848728e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6098948784911606e+01,     1.8780222052255624e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.8843562808651200e+00,     1.5062333370049626e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5392692998101771e+00,     6.1728695832317919e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6729193692997939e+01,     5.5356930186341742e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.4410750083549601e+01,     9.9574517717074268e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6163984116688965e+01,     1.7427677045320590e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5206798765991820e+01,     2.3385477056646407e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.9120091205578440e+00,    -5.5994411194932945e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.5213642891849474e+01,    -8.3317506293245515e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5352211355480208e+01,     2.1107181015858718e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.7874769876605328e+01,     8.2646036492177318e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0523879688694944e+01,     8.4099940113930097e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2204637744718479e+01,     8.3260970704153551e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2016642851374982e+01,     2.3853813688633425e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.7861390042696375e+00,     5.3504700183000550e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2635356190117082e+00,     7.2516021087785884e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6252359392186669e+01,     7.2780525893923877e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5886266442242452e+01,     1.4293617286661929e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2620980425491990e+01,     2.4464808410944639e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.4200906994541477e+01,    -1.2445482213011906e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.7792800823907786e+00,     8.5110184186526112e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.4712159862905008e+00,     6.5489955185440031e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.6687526963685414e+01,    -1.0194567271820392e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2133963514915798e+01,     9.5384072561137909e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.2348472347053399e+00,     8.2337738570251076e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0164610675045093e+01,     1.5620751177075197e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.6843114461540793e+01,     2.2146558513952623e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2306741717912750e+00,     1.2289656882753965e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5380049867252247e+01,     1.6217462541054843e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.6360314759367142e+01,     8.3701610583945494e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.8330814548914404e+01,     8.2016099926943971e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -5.8113675245912706e+00,    -3.1779491850468630e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.1145387528148785e+00,     5.0912236626534632e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6956180685901749e+01,     1.6015957353205253e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1315625326269554e+01,     2.3917846380734427e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    7.1428120184075166e+00,    -3.4662030386764400e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.7781414897150476e+01,     1.6548350360774901e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8354310089548044e+01,     1.3484349437191258e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8201308136723267e+01,     7.2361041189035614e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7918356242062110e+01,     1.5469166630461792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0278619152782134e+01,     9.8255242423246873e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8430749442320016e+00,    -8.3559213867893941e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.1367336629777094e+01,     1.7834368215745515e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.8364640513558122e+01,     1.5631796538951727e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5145645440040074e+01,     2.1499849669593630e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.5770855620942408e+00,    -9.2855856296509883e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2207526774234756e+01,     1.7342357413409183e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0034964365689589e+00,     8.8716793778592619e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3524474760316494e+01,     3.8210553932213152e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.7987111481320008e+00,     6.5504182094777561e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.2008301382925453e+00,    -6.0066482339099002e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.9865246923387421e-01,    -8.1191419588944045e+03,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {    2.8986489531422247e+01,     4.0593262914172010e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.5953198717865959e+01,     2.6294005882879675e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4391289113561051e+01,     1.5466313046725289e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -3.4127520052781026e+00,     1.3722764557737073e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.9736887146358960e+01,     6.5992617916846439e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4699213197252678e+00,     1.5400767679359165e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.7198614907373369e+01,     6.5906863125692071e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.9529797760248073e+01,     1.9582573268421551e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1997948285232409e+00,    -8.8860057043582729e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.0746048750287978e+00,     2.4330651296816723e+02,    -4.6349739317851188e-04,    -5.4768916741263173e-07,     2.5869658024004840e-09, } , 
      {    7.8695691899831797e+00,    -1.7805768455853991e+04,    -6.4460982180701858e-04,    -9.0073048999659575e-07,     4.2627727725237056e-09, } , 
      {    3.6304734899996483e-01,    -1.7738254361579100e+04,    -7.0626842577052931e-04,    -9.7466457636580007e-07,     4.6104811446154644e-09, } , 
      {    1.8301393269086326e+00,     7.1058445613656077e+03,     1.2681121164540989e-04,     2.1221128701695900e-07,    -9.0360543512557530e-10, } , 
      {    1.3099976457219970e+01,     2.3938959809183973e+04,     2.4466025454456515e-05,     2.4042395259902998e-07,    -1.1473600577138278e-09, } , 
      {    1.6375752550455026e+01,     1.4846938240660586e+04,    -1.8656357669616559e-06,     1.3726461879338937e-07,    -6.0199799597052197e-10, } , 
      {    1.9733585560756808e+01,     2.2581084757274675e+04,     1.2228169438553973e-04,     3.5009229792451100e-07,    -1.5600431593478871e-09, } , 
      {    4.1307879844479976e+00,    -8.9908104474362135e+03,    -1.2395146542533309e-04,    -2.1291494559372138e-07,     9.5659072233403651e-10, } , 
      {   -1.0245526317603457e+00,    -1.7390489547776677e+04,    -8.8193523320734133e-05,    -2.4550479997705792e-07,     1.1653951266511026e-09, } , 
      {    5.6292140664997472e+00,    -9.8286618685154172e+03,     3.1138050726433187e-04,     2.9719078652014554e-07,    -1.4020811657687780e-09, } , 
      {    1.8235152846453893e+01,     2.3418936178353877e+04,    -3.1305027830412529e-04,    -1.6001343418935591e-07,     7.9862872875492742e-10, } , 
      {    4.4294720539261228e+00,    -8.4334961678030668e+03,     6.1455345827705611e-05,     5.5227065669911707e-09,    -7.5383679275721722e-11, } , 
      {    1.5514236587779735e+01,     1.3833580822806623e+04,    -4.6516716421830535e-04,    -4.1051181508184208e-07,     1.9835133653866332e-09, } , 
      {    1.1091015864914205e+01,     8.3321650060655702e+03,     2.5900661858911786e-05,     1.0652394075254079e-07,    -5.5651277640882531e-10, } , 
      {    6.5865897227880836e+00,     7.1430451796809175e+03,    -3.2996128248106349e-05,     3.1929829037874036e-08,    -1.5509189658694057e-10, } , 
      {    1.3902593311942514e+01,     3.3456801070456335e+04,     2.3824151533214727e-04,     5.6485641986072033e-07,    -2.6799530664372928e-09, } , 
      {    3.1252428229914088e+01,     6.9284530222684452e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1193449709020824e+01,     9.5147054713293655e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3987128816509408e+01,    -1.1281350705376474e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.3994241552885942e+00,     5.7195312476990066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5322427700330472e+00,     7.9210338518525223e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.3517092767518051e+01,    -9.6528761633728664e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -6.7662755298910511e+01,    -9.6528627362714160e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.4266856218517027e+01,     2.4731642834499478e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    9.5079307153128063e+00,     6.7439512265799867e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7106462661904782e+00,     1.6240951653249474e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.8178264460866814e+01,     8.3508810138981080e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6265504317346029e+01,     2.3855446206584602e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.0198256371204138e+01,     1.6657284797876411e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.3752830081794617e+01,    -8.5482455559300888e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.0590363803450370e+02,     1.6657480901024093e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.2000756825053571e+01,     5.5282146198117644e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.7236271765559010e+00,    -5.9712918153105898e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.4529624675487888e+01,     2.5900984913186694e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0078312238050697e+00,     6.3570276206604613e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.3184770504516532e+00,     6.7266383638456582e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.1799323715547871e+01,     6.2820953347170359e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0584128591313231e+00,    -5.5468444731696309e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.0180309013568674e+01,     2.3187781199677200e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3689274055703327e+00,     6.5916553344997074e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3053179654847703e+01,     2.2422755912171066e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.8784197603956043e+00,    -1.1856162348816290e+03,    -1.8274292976013022e-04,    -2.1905404427487519e-07,     1.0864141298615481e-09, } , 
      {    1.2473990454569813e+01,     1.4286150379686995e+04,    -6.0860358237563932e-05,     6.2714140310022456e-08,    -1.9984989562697296e-10, } , 
      {    2.4105384876801523e+01,     7.2117599306217098e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4715135392879683e+01,     1.4872498234649589e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0887298194529731e+02,     8.3927792002325205e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5104129495159606e+00,     6.9077829446938813e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1880953347855399e+01,     1.8358335427847538e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6012519462205621e+00,    -8.2577037491097908e+03,    -3.4038344338729550e-04,    -4.6823237447643627e-07,     2.1638737510330028e-09, } , 
      {   -4.6385703089072820e+01,    -8.3857793233898956e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.9678757956666274e+01,     6.6590497856654183e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {   -3.1922684547670137e+01,     9.0528092985250260e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5882975619297184e+01,     3.8500276036015671e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4735918320107348e+01,     1.3799818145872123e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.9703688170597360e+00,     1.4642742268695656e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.3868381452577907e+00,     1.4882807515905077e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2821564344542089e+01,     1.2254635749043055e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1466799080553085e+00,     4.2844013290758821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3798504797515164e+01,     1.5873642109904014e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9973527072790858e+01,     6.2361520086531318e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.9928456704413051e-01,     8.3258379353529090e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6736721971674832e+01,     3.0474121676131155e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.1031334772436278e+01,    -2.3072540054238243e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.5261618671783008e+00,     8.3471677489664762e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.9566453762642944e+00,    -8.3102151004837760e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.1533507057400021e+00,     7.1920216120141040e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3302408446822373e+01,     9.1857266695234975e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1983169329589330e+01,     1.2856041852145714e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3198900752765464e+01,     8.9923360807062036e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6117866975095865e+01,     7.9220002543002001e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0961936790900545e+01,    -1.9311557021647063e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3128735341682065e+01,     9.6419735737295996e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6923689132684796e+00,     8.9738597564648535e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.1024540546512053e+01,     7.8426990829808246e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0414345267445775e+00,     8.3253857249484790e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.4678275845708910e-01,     4.7740590229978436e+04,     3.2456200947728281e-04,     8.7912719236635482e-07,    -4.1764759372862083e-09, } , 
      {   -1.5527127761248217e+01,     8.5095265350001519e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7331710605198609e+01,     1.0922103858235605e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0054050534936287e+01,     2.3946696776081473e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0960928480574722e+01,     2.8745431705565363e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.6365467488162167e+01,     5.9419056670793161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2972147193450798e+00,     5.1206010936767268e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3610446969802378e+01,     1.4914670362194447e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9430820234973631e+01,     1.0784833866242530e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6261175398043367e+01,     1.0680258287283323e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.2735106315298266e+00,     1.4871786889182713e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1265647843404828e+01,    -1.0966043510650902e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    2.6618454188162069e+01,     5.8177622843826866e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0809457502576905e+00,    -6.0911991533731612e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0613034545756692e+01,     8.4353409646143846e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3413087516376528e+01,    -3.6462212159122410e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7452755752840467e+01,     7.2163298543582805e+03,    -5.5838726002546712e-04,    -5.8641608425966146e-07,     2.7669771221964547e-09, } , 
      {    4.6078464377788091e+00,     6.1217410053079057e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3483406389244887e+01,    -1.1167707975359201e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -1.7096649031430019e+01,    -8.1478563144500995e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4824443769033186e+01,     2.3505656692905457e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    4.9480946342931837e+01,     1.5560027286950815e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3112943617202681e+01,     2.6851320162512820e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0557219330946637e+01,     1.6700042665405956e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.0368785416498056e-01,     6.4411388372723131e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2698502615474041e+01,     3.4547054340688446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4554397444547618e+01,     8.3114526447119097e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4642861170583963e+01,     8.6143012679155127e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.2941007965761642e+01,     1.8914450781478095e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7691685869965379e+01,     1.6512267733432196e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -3.1126639890486789e+01,     1.2562628541330114e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8731837885270526e+00,    -1.5540487301284515e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    4.0430874215095706e+01,     7.1577395328248122e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.8580431249835350e+00,     6.5960645616687771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1670881100025479e+01,     2.6119830569035301e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.8319959789230644e+00,    -1.5905122337567243e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -6.0778172043299250e-01,     8.3572668042511286e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.8861771094247430e+00,     7.8585797858462802e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1714121667891115e+00,    -1.1124863209966686e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.9872695799892441e+01,     9.2321273353488414e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.9645433372341863e+00,     8.3633171501937832e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.9439628360033936e+01,     1.0999450404020521e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7556669693355520e+00,     8.5393948762767486e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3358283407004844e+01,     1.6570641800522219e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.4586532827968677e+00,     6.5886001944710806e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1089502316595793e+01,     1.4914457227342227e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6219553990306601e+01,    -6.2623004226184275e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.7439819213983981e+01,     4.7745192629657904e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.4095156255059756e+01,     7.8444479932880768e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0673771086708701e+00,     1.1377257977901529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4221341482616410e+00,     2.5048431081204348e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6786229660108578e+01,     2.3232828848608053e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7380714067148681e+01,     1.3719636052543814e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.1689613851229904e+01,     1.4286467752569321e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3189025795331483e+01,     2.1420019892223398e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7788301971765069e+01,     8.3129342054233675e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.1644327464131500e+01,    -1.1011220479949393e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7303396213295109e+01,     7.1964308391831755e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.2212907964535002e+00,     6.9581127338027673e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -8.4273610823021805e+00,    -8.5605809592699734e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.7514234239524509e+00,     8.4831295550750310e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1954129974973302e+01,     1.5426506814506089e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3957060768722105e+00,     1.1770278947594516e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1570826664811079e+01,    -1.1139239273501437e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.7694241198979853e+01,    -1.9521332941844835e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.0575219149314840e+01,     4.9979389174937496e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.7998866581424195e-01,     1.4914053458875975e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5074977369244758e+01,    -6.4142549624499861e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.0756777281213544e+01,     1.5892357561944114e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.7108402628756444e+01,     2.2128516284147972e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.3843724035805359e+01,     4.4471835934603296e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2175622006474178e+01,     6.2752622786348938e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0358560804481520e+01,    -1.6666019347833681e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.9778430107053631e+00,     8.4323026961334654e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.1287584716003572e+01,     1.5453712041416187e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -3.8478390175403838e+00,     2.5240505713851278e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.6920228994985322e+00,    -8.9739196905794506e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -9.6173822788611290e+00,     8.3363796308915862e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.4547337682263475e+01,     2.2360399105142096e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.3784014425342725e+00,     1.0513663777503429e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3273765982827904e+01,     2.3295395881607859e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2519921769264258e+01,     2.3949442601800321e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.2137970879464980e+01,     1.2225119694152036e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.5834219398207487e+00,    -2.3712437293573798e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.7115963395760652e+01,     2.3712186107317437e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.6927192982177928e+01,     1.4230782189906529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3275798752019212e+00,     8.5169109825779960e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1911193799782453e+01,     1.2181348860642876e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.2157346329979845e+01,     1.0585759356752969e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1729864575257615e+01,     1.0682662157315650e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3249781463561705e+01,     1.5612584134089331e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    7.2145514044027621e+00,     9.4999869629048062e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4250037826486022e+01,     2.4739970947463105e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0341724949935340e+01,     2.1083770789350925e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.3644054898663882e+01,     8.3256322201257008e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0056720940022206e+01,     6.5697557720888035e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2418498851228794e+01,     9.0191321597436036e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.1818876263778350e+01,     1.7403404171711103e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.3922084577145597e+01,    -1.1153331911824432e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.4046807616115888e+00,    -1.1610292250916373e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.4559331996891228e+01,     1.5369253240755648e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.5605482770636895e+01,     2.3941275558814457e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    5.7578916427167668e+00,     7.7702136835849578e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2120675146752138e+00,     8.1179974489596225e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -8.2760795749097618e+00,     2.5534752055425099e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.6964423705747222e+00,     8.5360986522862222e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.8005279786891499e+01,     1.9160125110749155e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8153064089541974e+01,     1.4301558231132061e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4936583697566732e+00,     7.0966688942210931e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.3782912433352799e+00,    -1.4481309519344827e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.1158683814011194e+00,    -7.7882978259007605e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.2748583291966824e+00,    -7.1505810975969894e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3895223159267371e+01,     1.5637671857883977e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6729905729080883e+01,     8.4697116331022407e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.8299422219994888e+01,    -8.1876712163480115e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.7392526736721070e+00,     7.2163658002959555e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1032885037710960e+01,     6.2637276161326972e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.2826499746150265e+01,     8.2184687537634945e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.6950799483326335e-01,     8.2220418848358677e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.3639495102347148e+00,     5.6502920657886409e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.5501754395174494e+00,     3.8299219129868667e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.8135706990573912e+01,     9.0460165233842708e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2503927989692755e+01,     3.8299923761784827e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.0879268661188405e+01,    -6.5087016922782414e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.8991431141139543e+01,     1.5232965920321712e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7956987815071859e+00,     7.1868403935555962e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2653620105000780e+01,     8.6376011778062621e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6986197478601280e+01,     1.5010075618305311e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9769953410582925e+01,     2.9577859051408632e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    5.6740583543074070e+00,    -7.3964127356063182e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.7989210417882280e+00,     7.2876505251809995e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.9124849400607484e+00,     5.3602917055089838e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.4080398913188290e+01,     6.9751734540713587e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0828208091087820e+01,     1.3825555295393955e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    5.8875314051893266e+00,     7.1643994680242877e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0332109218867688e+01,     7.7018065306133349e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5889628111210339e+00,     2.5017252677689863e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.6479526567759237e+01,     9.8412567264722784e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.5863876720616894e+01,     2.3245215672647239e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5842684642507592e+01,     5.8778178915719218e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.6272118624125483e+01,     1.7697650513284923e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.0335670384534890e+00,    -1.9268871913653845e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.8704811744120899e+00,     8.9415798987314811e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8512065589384030e+00,    -3.4159825106037006e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3063287473317626e+01,     3.0141732286641512e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.8649570992264486e+01,     1.4989697766310805e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1496617422914719e+00,     5.2308073609003679e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9876891383445273e+01,     7.1441730874388604e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.3481924182023803e+00,    -2.3455065798355458e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.8615166517373956e-02,    -2.3117251523325813e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.4269278730244611e+01,     4.5515411966592092e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3148788744014297e+00,    -1.1430144926765520e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.2696716660764942e+00,     1.4914223048082382e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.8593872223618657e+00,     1.6316176991247554e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    8.8349312557744106e+00,     1.6727664809395774e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3565389099103834e+01,     1.6022516308804981e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0421540039423716e+01,     9.6397515386226860e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -7.4555677805807541e+00,     3.8411674481621048e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.5608057305246916e+00,     1.0309174153739546e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0791056189792005e+00,     2.0727277957365186e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7377213462179853e+01,     2.6687026852572664e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.3698444585564671e+00,     7.3008133901729880e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.8476126733817644e-01,     1.0984237008208540e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1362987042439826e+01,     1.4913894415605933e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0089895289283472e+01,    -8.3662306680448819e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.5274512713705573e+00,     7.1345023999055284e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.8110580819467419e+00,     2.3274316533378023e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.1894088724525258e+01,     8.3315449140973433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4939756135154312e+00,     1.6026445432088523e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4828749014339273e+01,     1.3031825149841519e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0895818436004134e+00,    -3.6578902200363024e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    7.9888611276794617e+00,     1.7073814045651030e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.2295737452172553e+00,     1.2116899251462160e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.3137714133766821e+01,     7.0963491282435225e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8396426786736164e+00,    -1.1572883752219971e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2345624186972200e+01,     1.0735983570484917e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4245265722289524e+01,     1.6184845445972933e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8675050698294484e+01,     2.3087704975362845e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1844577006603076e+00,     1.6614723033494549e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8743427821016017e+00,    -3.4727543774242181e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.1560801976437425e+00,     1.3072183765957489e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3141489713470433e+01,     7.2578336989679901e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    6.8982518622790012e+00,     6.0207260738344248e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0567510716773466e+01,    -9.0978563700287305e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.0736008688004684e+01,    -9.0932539703492585e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    7.1027130209706524e+00,     2.5595013165606542e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.5227660712570852e+01,     1.6266663079596998e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1911892571471821e+01,     3.7999850480661524e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    8.5498714230121351e+00,     6.0530766085346404e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1902062734737505e+01,     8.6134919517282360e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    4.0130473250180060e+01,     8.5351276447306336e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.3855805903952984e+01,     9.1972166572791430e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5995336933170707e+01,     2.2110485362396044e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.7802424414053437e+00,     2.3398242651887791e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.4704126818465024e+00,    -3.1031625858112616e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.0144788608980857e+00,     5.9993961658565731e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4089925615449529e+01,     2.2561861447606807e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3824159759011565e+01,     1.2434669159982757e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4901760175021442e+01,     2.3307127853797097e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1064275446378023e+01,    -1.1211875148660229e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {   -1.7439322084590101e+00,     6.6019646157551942e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0156034466969235e+00,     6.5574217909565959e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8645357704346772e+01,     1.5586525123693116e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.4776200104252203e+01,     9.6317072606415568e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.4691650564859629e+00,     6.2068100141927255e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.7843949141183721e+01,    -1.0592716290799517e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0020234018885072e+01,     8.7926723302317514e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    7.8449798752239017e+00,     9.0629372041930510e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.5618214765940017e+00,     7.0978153897809616e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.6228096271161874e+01,     1.9248512024949821e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6577455107541095e+00,     2.0334256987672197e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0983059494251162e+01,     7.2128899281656895e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9240040500704680e+01,     2.3638460299396262e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.5721038350891831e+01,     9.7791085680570086e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0018583078701715e+00,     6.6970889689283777e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9080689310172449e+01,     2.4892774269647616e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0893533256281025e+01,    -1.1277464704657486e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    1.1383993238027788e+01,     7.2152358027519749e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3142227161821680e+01,     2.3465306834016643e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.6519125969863424e+01,     1.5397639174165908e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    7.1797270987976880e+00,     7.8379612233353055e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6543776206685116e+01,     8.3643875251694608e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7529641651478457e+01,     8.3311212834669022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.6535347199795083e+01,     8.3262615659833500e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1230738324286227e+00,    -1.8411519216274698e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.5260471875677204e+01,     8.8220122757885038e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5328745377720523e+00,     6.3482086957107067e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3806728360564577e+01,     8.3283961618474368e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    4.5954026562470986e+01,    -8.3289866876028136e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.9834430381162520e+01,     1.1178053989891672e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -7.9385898523499456e+00,     1.0133984376065931e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2401962757862179e+00,    -8.5070624870413631e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -1.1384301819266312e+01,    -8.7363489975977300e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.4911666214037599e+01,     8.3255629195318670e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    6.4590261423349560e+00,     9.8478156820720051e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.2778159513044171e+01,     6.5863075699457722e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1957673295985041e+01,     8.3151288876465569e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0001267576471443e+01,     1.6342170287448835e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3216798897645035e+00,     8.0842160214566593e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9967322927384490e+01,     1.5697826789322144e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4524575343905415e+00,     1.6156105542091387e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8941014604389931e+01,     1.3842477961827826e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.5171074904527813e+01,    -1.8381079763960906e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.0469995206533135e+01,     7.7490857053446580e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.0248458466661798e+00,     7.9563339713356490e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9857526035604323e+01,     1.7325505336393708e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7756593213660050e+00,     1.5999508324485532e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.6336708157642148e-01,     8.3422539618036953e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4572775229921213e+01,     1.6065835026273995e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1810280369001887e+01,    -1.0719687433105898e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.1999847338536345e+00,     8.2440831382805954e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9778215552763754e+01,     3.4951283927485121e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.9306509247151251e+01,     7.2598419035091729e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.2016128150698435e+00,     1.4952184064252628e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2155343373634125e+01,     3.0454571163006189e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5389548811169693e+01,     1.6755598626765885e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.0338717535301452e+01,     1.4911601791080624e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.7196159406732576e+00,     1.7190089230128298e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.8267591884895197e+01,     1.5542289828247169e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.1190539291495622e+01,     1.5543218752120745e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.3591336052595501e+01,     8.9727314939182597e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.5895758933218378e+00,     9.3346008684244662e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5874645435729466e+01,     9.9809573968253826e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1039446927989815e+01,     1.3188783557593435e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5164961773016699e+01,     1.3585201608094130e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.3505347560684644e+00,     4.9361191406292164e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4471834738881835e+01,     9.0869186718055662e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.6939674044322260e+01,     2.2750258200043067e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    1.2209439901912649e+01,    -1.6892557016337689e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4800391014818928e+01,     1.5790660143858891e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -2.1506561036366183e+01,     1.5542435486632943e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.8583613629765750e+01,     1.5543073093734976e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9126965968578595e+00,     2.3497569569539446e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.8468251322809323e+01,     1.0897278695093641e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6413626476614169e+01,     3.1518860457377476e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.5188058801304663e+01,     6.7224729828738818e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2903527877090859e+01,     2.4821780964235222e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.5882453774224494e+01,     2.1549402524093857e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.0105013524127013e+01,     6.5594468897867318e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2344707245180004e+01,     1.5598015111448765e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0493792047591798e+02,     3.1069966552079488e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.5870197143375030e+01,     1.5542903775967947e+04,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -1.8793144549975448e+01,     1.5542604804399971e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.1025930708116661e+01,     6.5334975286689742e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0635067045370841e+00,    -3.4689516078148922e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.5644051087527870e+00,     1.8422629391809260e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5244956740554754e+00,     7.8773153395143618e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8540836110460226e+01,    -8.9042248767769706e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4996301766521409e+01,     1.0207055224262029e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0736473348295994e+00,    -2.5002994954984162e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {    4.3858277257216514e+01,     3.2830745924519491e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3742131913564460e+01,     2.9871553226232536e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.4927884762830601e-01,     7.2764196724878020e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    8.8648039783166368e+00,    -1.8384961181896272e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.7778283958688790e+00,     8.4132997111696568e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7205343436347391e+01,     3.7850945809118220e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.7837618554884482e+00,     1.0120642807810442e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8322610664452498e+01,    -8.2691881070683739e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1527853268500349e+01,     1.4077905688317906e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2445220096064332e+01,     5.3133923668065636e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    5.2731023939862581e+01,     7.6363904111098127e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.9965459918152888e-01,     1.7874183117643428e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.3803394359068690e+00,     1.3868996578425220e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8303441437955172e+01,     2.2539579483645612e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4409055929475739e+01,     7.9202513439929489e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7652919432461665e+00,     4.3677259666962855e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9616731256666124e+01,     2.5802945223110264e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0285892256815265e+01,     1.5332050838632336e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7330772805148069e+01,     1.7109862462053443e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5267158353789046e+01,     4.8402045631533183e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5267638876257209e+01,     1.5487493468919152e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -8.0724969648785034e+01,     1.7991896854858736e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3863410297602382e+01,    -8.3235708520891785e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.8054227077266795e+01,     6.2107832540756754e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4034626983458622e+01,     2.3539707325905238e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7613678366592769e+00,     1.6184179786428958e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2213775658577902e+01,     1.4293765703719142e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8405453899128226e+01,     1.2880649751100818e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0418711436271050e+01,     3.0770039871687055e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.0693438763666103e+00,    -7.8930725588161213e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1577718615300428e+01,     6.9269611386159868e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.4071231731098301e+01,     7.1852124701117864e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -8.0151441598541709e+01,    -1.1389014328757758e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.7755258647668413e+01,     2.6464858148419564e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2681830479778146e+01,     7.3303103411367038e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0675805426221991e+01,     2.2751886123486871e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.9233418752881732e+00,     6.1801150723979226e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.1727462210914062e+01,     8.6392291012500700e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.8851023997014886e+00,     1.3095842673022778e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9179213308528382e-01,     8.9136278483480892e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0623139089323953e+01,    -6.3996692036028880e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.5746653839520244e+00,     4.0676050866744572e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7862857646190470e+01,     1.5536142890630666e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3522283700410442e+01,    -2.2266737703017679e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.7861724859092412e+01,     9.0240631918995241e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6143208580911313e+01,     3.9807220974786032e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.0721868723949795e+01,     8.3401661531127247e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.3843915479385378e+01,     8.2909540656109148e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0773949997586122e+01,     7.7664461129360589e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.7454068947875690e+01,     7.7763081772479018e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.6631026008449538e+01,     1.3972971611251745e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1804447548782004e+01,     1.6657776095712696e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.0988778046080316e+00,     1.6656989603187809e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3096148747970187e+01,     7.2129346029122398e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0958457084434887e+01,     1.5087600170518039e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.0973869027725527e+01,     3.1085410528794077e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2623191890118024e+01,     1.5437527441228453e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.6199957327279479e+00,     1.8033683987325276e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.1884479684707384e+02,     3.1085606631941759e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.6374707600599677e+01,     2.2743409287946997e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0529753401744419e+01,     2.6083782152632888e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.7578960077322021e+00,     8.5595707985336994e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -6.7363626596090029e+01,     1.7109157830137290e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.9399685193618595e+01,     1.0450772622770803e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5316547246525605e+01,     1.6792810504675319e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    6.4539564772896263e-01,     8.9555763188369147e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1010898198993623e+01,     9.7027005660539508e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2253839803514275e+01,     2.3967005688786874e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.1408634870807688e+01,     1.3655530335207595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.4529111725027715e+00,     4.6801690089731510e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.5114695617498484e+00,    -3.3150200230837872e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.3645980629500787e+01,     6.0990819902344656e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.9254725775321493e+00,     6.1686127387456345e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9537372426396686e+00,     1.6047108948392035e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1135431785183933e+01,     3.1554908873779885e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.8853308934921312e+01,     2.8820203045432918e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.4984955360362435e+00,    -9.4083750949814330e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.3912280127885346e+01,     1.4508307824245972e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.4212630789138345e+01,     8.4814947469096405e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.3231833515217932e+00,     2.0883812036384523e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -4.8025890210190099e+00,     1.2727173152520378e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.6957932507765623e+00,     2.1026081129759794e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.2578553763870840e+01,     5.4887587474226539e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1531904111110265e+00,     1.5616341949906126e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9031225709366606e+01,     1.3214170376782282e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.7383473715264088e+01,     6.0997866221506138e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.4819734196990350e+02,     8.3303133301417911e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4034464017180991e+02,    -8.3270695193084612e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9117905507138595e+01,     2.5075091787788977e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.0872096069319319e+00,     8.7988030636039693e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0304613993930680e+01,     7.1088360165033264e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0665068617282298e+01,     7.2136670313710156e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.1700874831613467e+01,     7.2144586995466479e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9413052891174182e+01,     2.2669888994440441e+00,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -5.2856120739217864e+00,     8.3134959707419712e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8105185331806425e+01,     2.9278138086243409e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2743884422903196e+01,     8.2904408359384943e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5280818939637779e+00,     8.5405678135698909e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2317132343361905e+00,    -1.1184313288756189e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.4244406691370708e+00,     5.4357896502114627e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3026925508389144e+00,     7.2544032485615244e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0636062975979080e+01,     2.3119185203026071e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.2059686839858745e+00,     6.0097054471120609e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8912106086381520e+01,     1.6943985903416022e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.1842791450037652e+01,     7.7007428691883842e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5329078783638312e+00,     6.7336419453244998e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9380235749712028e+01,     1.8065173418282044e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -7.2825250871331448e-01,     1.0591216038515553e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6355706689878716e+01,     1.1035273924221087e+00,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    5.3511752781844564e+01,     2.6154320471894625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.4569772101115053e+00,     3.3176345960319222e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.7069100058634767e+01,     3.1101050608656347e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5766152680665179e+01,     2.3875854942078153e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.2892048223246860e+01,     3.0945823169382857e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3284939791915789e+01,     7.8254497696955941e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.4535402206617164e+01,     2.7244341132205809e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    5.2716015135705646e+01,     1.5996260239349424e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8779179744430955e+01,     1.0831433686024029e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8475579700607518e+00,     8.3664287838393375e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9098217346195440e+01,     2.2621830421660627e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4674440879112504e+01,     2.0349525009883000e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.1876936472393975e+00,     9.7276218381174549e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.6763777726755684e+00,     1.0524107181640151e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0234006666525788e+01,    -7.7871343643937380e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.2782652018249962e+01,    -3.1480227663230835e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4928553663705706e+01,     1.5659001765861829e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5754931137852488e+01,     1.5859542536888830e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.6192782930021181e+01,     8.3259512757919001e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.3871615066144244e+01,     1.8779464047495931e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6750809481243312e+01,     1.9899766384309787e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.5462424400838472e+01,     7.2151911280054255e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.0403283908233814e+00,     6.5797218060183413e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.5198537616008629e+01,     8.3314315736583521e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.7666792934330033e+01,    -1.5438616348222749e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0309133768182633e+01,     2.3853421506707884e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.2338444744062805e+01,    -1.0862426258560349e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0828317966603841e+01,     1.5167782263846344e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7805441664627069e-01,    -1.0646966056871315e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.2705676468196117e+01,     1.6404047290742325e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1271229556811534e+01,     9.4121531294365905e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4854516955165309e+00,     1.2633873147992615e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.8351813853964547e+00,     1.3329743588044086e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.6298814246747551e+01,     2.6071837178726575e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5502292262737217e+01,    -7.4029640743540913e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.4229987909879149e+00,    -1.1108257896569680e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9236735490021587e+01,     7.1179246309269001e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0630992472083545e+01,     2.0150518784959127e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.1952103763488147e+00,     3.2454568579310115e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.7839025606718764e-01,     1.7053800899950947e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4337297806751618e+01,     9.6589297325447915e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3158773825046701e+01,     2.3294953241353829e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.7995276455226151e+01,    -5.2958573556963138e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.2364600060357169e+00,     1.9941236017979209e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.3431620491763510e+01,     7.7138903509509373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.4865257928882954e+01,    -6.2840563661938234e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4849985238075064e+01,     8.5104929374478306e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4037837869246488e+01,     3.0756921960487598e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -1.5944210211360994e+01,    -9.7324992536179652e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.9731543655530160e+01,     3.5015718277297790e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.0151010386639712e+01,     6.5726373692138341e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.1240961073024467e+00,    -8.9209505933682594e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.1111546039459220e+01,     7.1050303767834775e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.2277055727417164e+01,     1.6153028191242176e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    5.3835547559217865e+01,     2.2761748187798712e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4188970043094413e+01,     8.9510989556842524e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4429610870498919e+01,     1.0968504524060951e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.3581755353830061e+01,     8.2006102206479609e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.6065380276363239e+00,     1.6168984332445802e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4863121580848393e+00,     8.7649243771237343e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0817174617420484e+00,     1.6204837672935406e+04,    -9.9779237061628989e-05,     2.7639906699208196e-08,    -1.8858767381479843e-10, } , 
      {    2.1663380435764008e+01,     3.1119290007870881e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    3.8010109608848408e+00,    -1.6062862321704735e+04,    -2.7636214843026552e-04,    -4.6328242679230791e-07,     2.1436570205307352e-09, } , 
      {    7.4566949625787826e-01,    -1.7319466232201703e+04,    -2.7103438544444846e-04,    -4.6451521102063327e-07,     2.2525364770343145e-09, } , 
      {    1.6109248505920409e+01,     1.6066287142314141e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    5.7087969461417898e-01,    -5.2353285213018239e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    1.0067575920936971e+01,     6.4809805474979285e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.1854135532655484e+01,     1.6204848932529527e+04,    -9.4451474075811953e-05,     2.6407122470882869e-08,    -7.9708217311218847e-11, } , 
      {    1.7485079344296281e+01,     1.3797188313555938e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1161376063670549e+01,     2.9253554922442479e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.9484539594282295e-01,     1.3769165223562417e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0195648871846480e+01,     8.9842214816739070e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.7277171448542379e+01,     4.2430485732945544e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5504543438078420e+00,     6.0507476998955099e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9159193117037976e+00,     1.4930446213462901e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1611255512378643e+01,     9.3772424513668939e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6399707597179791e+01,     1.1659449978795074e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9742857692674683e+01,     8.6038197503032970e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.5883355174043157e+00,     7.1676621996334889e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.4682103515445668e+00,     2.2029182775728157e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2742987713740568e+01,     2.9908523214072655e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.0482915689357718e+01,     1.6566712677238676e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.2510917881038139e+00,     1.4976579855111351e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.2494646049995071e+00,     9.6382756920274041e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0926322320754078e+00,    -8.3464733439036063e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -1.4122270599521780e+01,     3.0616844643758577e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8304224999864857e+01,     9.4920065902884508e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3066247766961524e+01,     1.8247104578121711e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9210766803915341e+01,     2.6857879118112553e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1493881365819348e+01,     7.7053148718355005e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4674479007573765e+01,     1.9407771632541473e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1688002154336724e+01,     1.6177620830829230e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -8.7717410411717349e+00,     1.5542469062497617e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.6424015673801925e+01,    -1.5543039517870300e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3300229673380880e+01,     1.5541581352890818e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7855453030463435e+00,     1.7417687639314394e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.0331656604689231e+01,     1.6257812399943658e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2473060124345805e+01,     1.5176965268180329e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.9218985576871098e+01,     3.2811524043870973e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2935805984376316e+01,     1.3887732132093297e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.4891159511013534e+01,    -7.5787321126818497e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.0910539836436598e+01,     1.5152034520330704e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7570098125092052e+01,     1.3155511052502094e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2385326315190724e+01,     2.9138452675258352e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.7589713412783077e+00,     1.5541331599250205e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7601186561075309e+01,     3.1730736846222244e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.1257558843892751e+01,     7.0322613527361273e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0504947998729037e+01,     2.2743699244443334e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.8725295230435401e+00,    -3.1572856687433550e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.0751776381289446e+01,    -7.7894260884473551e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8005721244417622e+01,     8.9590765525543702e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3777164017844656e+01,     1.5543927227477101e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4677899466092477e+01,     8.1552093268690551e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3611325297904088e+01,     8.3993560976610734e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.6534541676011898e+00,    -1.0071627569036962e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.5778134131814724e+01,     1.4130705965592173e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0585981558790563e+00,     5.7287200356149178e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9589349372685895e+01,    -1.0201547231071860e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.7820187913500414e+00,    -7.9164922611100319e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.7177788604689881e+00,     7.2626227355009532e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2976091258455911e+01,     8.5482302965620438e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9853220006131025e+01,     3.0242584457544384e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.5058332673394702e+01,    -8.2612675457488876e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.5891635573486084e+01,     2.1500554301509779e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.8431131339929713e+01,     8.2480106585197427e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    3.1989719992121618e+01,     5.3909598871867365e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5862742582782847e+01,     2.3384772424730261e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.6439528761681217e+00,     7.0809546430867786e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.8147536230416010e+00,    -1.6592948499637583e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -7.9409306253845422e+00,     1.4905810206754986e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.8177887805051363e+00,     1.5537324585150962e+04,    -2.2283452439743112e-04,    -4.8670445446586314e-08,     2.1971756027884158e-10, } , 
      {   -1.1834485852125056e+01,    -1.5548183995216950e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.5666549734571785e+01,     2.0115055142902811e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.0060540067491928e+01,    -1.3671770246059259e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.1359201741489686e+00,    -7.6441243458667969e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7453748962120681e+00,     5.4519989855533668e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2093801489973977e+01,     3.0377018901994066e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.1422524688692835e+01,     1.2411960590518975e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6632004318828223e+01,     6.9042744955965873e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1726584475953324e+01,     8.8702484850565143e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -8.9499528814096664e+01,     8.3355969546301276e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4835362062532692e+01,    -3.2532002462192531e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.3613727309459421e+01,     2.3870893425376602e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3303769143406825e+01,     2.2026337861670367e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0982794174759348e+01,     1.6659649838349698e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.2412711526739288e+00,     1.4906968538569359e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5166978312229666e+01,     5.1941533763400366e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3871809397232102e+00,     1.2163299917287502e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9441753203891022e+01,     1.9553146080442137e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3709176729705383e+00,     5.4343669592777096e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1457799636258557e+00,     1.0198195068822566e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.4453923351233531e+00,    -1.2370234810170090e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5057974580381750e+01,     8.7423759107216229e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8258169443249805e+00,     2.3871998004441568e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1391544810639985e+01,     1.6973456393280935e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    7.9325108131242645e+00,     7.0033594139072093e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0550078788384745e+01,     2.3869303476639459e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    1.1238082949518628e+01,     2.3873587953178707e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.3738098718407386e+01,     1.4898447196813930e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3112243560979021e+01,     1.7409643361333263e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.3736501385529831e+01,     8.4394996320354348e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.9487556700635071e+00,     9.0993142530719106e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7269143847422598e+01,     9.0678340254086797e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.0852861076364060e+02,     1.4286237171666711e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6598564873501115e+01,     2.3870479312461408e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    4.4433749871937707e+00,     6.5573693470030312e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8799238580504088e+01,     2.7080047822265653e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.9540675491967335e+01,     1.4694579200825050e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0770907897013107e+00,     1.4250090703690461e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.8511253398900713e+00,     9.8051740991295774e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6691319505906542e+01,     5.2332255328463034e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.7808721750448228e+01,    -1.0903556856568111e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2277877389621334e+01,     1.6729479682255074e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.8142248400600733e+00,    -1.7363771887121090e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.5632208583460553e+01,     3.0983336871441035e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.8739766388870709e+01,     9.5849542788581366e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9730628733742455e+00,     6.5430954644575859e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4826196752621605e+00,     3.1634186361617634e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.2839333929026772e+01,    -8.3313624875310179e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.4935604218331093e+01,    -6.4558058181239903e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1466951575820651e+02,     1.4978534480645811e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0872311420963447e+01,     8.7643102906341319e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.0497855624061458e+00,    -1.0857669508422938e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -7.2285542775863121e+00,    -3.5709701547398508e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.7394607234368920e+01,     1.9208697123051912e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9040133700092557e+01,     3.2221467047612063e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5096830133090963e+01,     1.5052024088794138e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4357269097724906e+01,     1.5143174364891243e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.8697938301421129e+00,     5.9471384141122580e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3150102380562828e+01,     1.3268413361389259e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0491837644085438e+01,     2.2873713553894862e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    9.0343124625408961e+00,     6.1583034574901476e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2114484696478058e+01,    -8.3381339389648492e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -8.5178492984693236e+01,     1.8286143196432553e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1988606691711681e+00,     1.6675416533437579e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.9770924977812697e+01,     1.5478666514676564e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6014045689345515e+01,     2.3881754996164575e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8209983589235293e+02,     1.5528739820374567e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.5502278329895336e+02,     1.5556768759993351e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3647663594790622e+01,    -9.6977514236673233e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3766358035531477e+01,     7.0820656606402354e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.6455283263512705e+01,     2.2879676030873958e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0353784535051664e+01,     2.3241060587079843e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2856919905824050e+01,     1.6664861016019309e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.0714764809458401e+01,     1.0268971110048098e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4041674153821386e+00,    -8.9132281762615094e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.2846133445950841e+01,     8.4817508935890874e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9839483552676004e+01,     8.3314228755613622e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.6467096637610759e+00,     1.1723878281769172e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.3720521969396415e+00,    -1.2101885331440840e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.3762638199754775e+01,     7.8353705736617503e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2048392990260979e+00,    -7.5234712914170441e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.8427891056217804e+00,     1.6649904682881195e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9773146877687613e+00,    -9.7768223766596075e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.3822175508188117e+01,     8.4575601243504643e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -8.4049692504415674e-01,    -8.8548273008437882e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -4.3074531643102645e+01,     8.6546602795459057e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6447358281114973e+01,    -8.5562898339112216e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.8849071098024126e+00,     5.9567363499008688e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4175555595155492e+01,     2.4951129385165394e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -8.8862585038086792e+01,     1.0251698113281242e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3262930330267221e+01,     1.5906216389173544e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0465941106266058e+01,     3.0493249411724781e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.5997362799454974e+01,     1.4933634374846641e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.9335803869132501e+01,     1.3383386949671343e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0393856453011637e+01,     7.7582592338925224e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0466285101059205e+01,     8.5742893069582587e+03,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {   -1.7604232984328149e+01,     9.1899844252834901e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3468612692172844e+01,    -7.1857111429888628e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.6176874277580961e+01,     1.4852313555165483e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2075973771573267e+01,     7.2673842786280993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.5580073814548037e+00,     1.7112536969116172e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9520274800495212e+01,     7.1763255063446213e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4830529893484350e+00,    -2.7642646990503672e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1878220507168297e+01,     1.5505016931069746e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.4194403884456559e+00,     2.3178703780050870e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8677923135834313e+00,     7.8482705045907933e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7392300053121264e+01,     1.6192391783207353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7791704763040165e+01,    -8.9750479531260426e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7257926321521936e+01,     1.5544176981117713e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7522475719424527e+01,     1.9173243021948609e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4590092989198197e+01,    -8.3337584966723953e+03,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {   -1.4741827768239415e+00,     6.6260956635159819e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0429613616953947e+02,     7.1361640301414345e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.9329971036216644e+00,     1.6127291041720342e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5734345436388903e+01,     2.3861136433653595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1595194378621089e+01,    -7.9230512167097595e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.2565060690740637e+01,     8.3059438043668615e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6682327468849589e+00,     2.1042746666046049e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.6463200879144999e+00,     8.1889907543720201e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.1283751824843304e+01,     6.8854762455785149e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.7448891855841786e+01,     3.2178807231656359e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.7957589851440595e+01,     7.6225029461910099e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9882387346180872e+00,     8.3922065635684648e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6320189931012052e+01,     6.6011743914524786e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0664402027393640e+00,    -1.6906783925675215e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.1738515862455273e+01,     1.0919820600224697e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2514041678669638e+00,     5.4419636843369153e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2607188926727545e+01,     8.0465054706701494e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.2711788674113455e+01,     3.2921744729207516e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -6.3432956855554735e+01,     1.6657177134223042e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8608255254590649e+00,     1.3571090486119292e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3465520299040753e+01,     9.0797979508872359e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.2664632877984751e+01,     7.0355864621793080e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7907651311514719e+01,     7.8074255614943886e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.2406240953299470e+00,    -8.9197627514957203e+03,    -3.0689025991263160e-04,    -4.3188172340599684e-07,     2.0444592932389127e-09, } , 
      {    6.4981240113950296e+00,     2.4391529366064344e+04,     4.2877283143519788e-04,     7.1364990799089442e-07,    -3.3307233187274341e-09, } , 
      {   -4.5798607357589329e+00,    -2.4324015271789453e+04,    -4.9043143539870856e-04,    -7.8758399436009874e-07,     3.6784316908191937e-09, } , 
      {    6.8992322448126480e+00,     1.6762127572203033e+04,     8.5725506554993870e-05,     2.4603392562862088e-07,    -1.2212892959462209e-09, } , 
      {   -2.8809665014695902e+00,    -9.5515382880846446e+03,    -1.7781428963728661e-04,    -2.8861094184281384e-07,     1.4690727202244936e-09, } , 
      {    1.0787550184709826e+01,     1.4981966429210366e+04,    -1.2518284369398322e-04,    -1.0603553945019108e-08,     9.3418748212996449e-11, } , 
      {    1.3662830167032784e+01,     2.4395002947404784e+04,     3.0236074265276139e-04,     5.6976273431354889e-07,    -2.7008970174612251e-09, } , 
      {    1.2227078206006379e+01,     2.2648598851549566e+04,     6.0623090422028940e-05,     2.7615821155530678e-07,    -1.2123347872561279e-09, } , 
      {    3.5730088499252872e+01,     3.8265814398689297e+04,    -3.2004781232973577e-04,    -2.1603297630240923e-08,     8.6296835237497271e-11, } , 
      {    1.3827125728047701e+01,     2.4114752227877252e+04,    -3.7737276376054456e-04,    -2.3333112844439748e-07,     1.0918973725948968e-09, } , 
      {    2.0925123548946488e+01,     2.3418876158028717e+04,    -3.1818217656277408e-04,    -1.5886791642363030e-07,     6.8829483120801924e-10, } , 
      {    2.2148435648140588e+01,     2.3351362063753826e+04,    -2.5652357259926324e-04,    -8.4933830054426053e-08,     3.4058645911625999e-10, } , 
      {    9.7573119064674732e+00,    -2.5470248884565335e+03,     3.6352929694740667e-05,     3.5646992493677065e-08,    -6.6429170585628593e-11, } , 
      {    1.6737724608793268e+01,     2.3695653296215809e+04,     4.8796341863296836e-04,     7.8811312001166160e-07,    -3.7343258601143123e-09, } , 
      {    3.3374531658072954e+01,     2.9937122973964171e+04,    -4.7236056297108399e-04,    -2.7201441206012735e-07,     1.2726359129125317e-09, } , 
      {    1.5879598797800771e+01,     1.5090244753628753e+04,    -4.6536302894547351e-04,    -4.1042454861924236e-07,     1.9849678064299617e-09, } , 
      {    1.4425300081612377e+01,     1.4738659916242199e+04,     3.3831454948452867e-04,     5.3708561346761250e-07,    -2.4935470541874871e-09, } , 
      {    7.7715442043936456e+00,     7.1768322369809430e+03,    -6.1259481100537342e-05,    -5.6099730295908842e-09,     7.3929238232393120e-11, } , 
      {    1.6009578540771962e+01,     2.4080995180739807e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.3654023968520061e+01,     1.5752303756014680e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.4252192387965280e+01,     6.3762114443796290e+03,     2.1683110082493579e-04,     3.2364154222232824e-07,    -1.4810621625583325e-09, } , 
      {   -2.8511899422894347e+00,    -8.3659520633655957e+03,     2.3626909935192144e-06,    -6.8984138685075869e-08,     3.2749164158949159e-10, } , 
      {    7.1541949161323473e+00,     7.0720875142281620e+03,     1.5764051362716528e-04,     2.4917833020156111e-07,    -1.0774596211714549e-09, } , 
      {   -1.9170879044199322e+02,     6.5862632389513255e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -5.3779637648986593e+01,     1.5540454825341611e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.0856690242386165e+01,     1.5545053755026303e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2323233689027886e+01,     7.2501712159758408e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8183961717254171e+01,     2.2721480084707258e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.1786942277030498e+01,     6.4264956728216403e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3254771115490371e+01,     1.6013533236116278e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.2713103609887479e+00,     7.2232516533747448e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3639080860641663e+01,     8.9297690477064007e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8192086226122797e+00,     8.3113774976539044e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9785233819076122e+00,    -2.2645941894681209e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3514273656600103e+01,     1.5369287451695938e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0423809960377607e+01,     1.6816290141083828e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0922977023227306e+01,     2.5007404182153230e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.6567825086776338e+01,     7.2114093496610249e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0664208685967646e+01,     8.6457984749810130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4457715355486975e+00,     8.1878083412927563e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0407682732725276e+00,     6.6129852424775590e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9422814051540016e+01,    -6.2991522264475198e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3457877444499484e+01,     1.6522659721002574e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3515799265379826e+01,     2.2876261861336527e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.0531725987785478e+01,     7.2289833036209457e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    1.9914232486076113e+01,     1.3067899289418239e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -3.6906081871377218e+01,     8.3109711138462026e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.4758783669470809e+01,    -8.3464117356040497e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3872358530516986e+00,     6.5077583935220518e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1517695089114817e+00,    -1.0050297661059110e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.5864988110832847e+00,     7.2064362818325617e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2045646871539461e+01,    -6.7926901732958759e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1341063291299065e+01,     1.3880017205147582e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5802276674407988e+01,     8.4215117079480533e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3740086223867454e+01,     2.0402494107094193e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.8478881101216288e+01,     6.5917887548082354e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.3560248030683482e+01,     8.3514390450833907e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2794673200560247e+00,     1.3093513673935340e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3066823709494720e+01,     1.7474077711145928e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -7.5048346810321760e+01,     1.1253881843696425e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.4367685427356562e+00,     1.5716221128671979e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.4762449305395933e+01,     2.4936436772863385e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -7.1271941638841518e+00,     2.1497900785711972e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3353377799037116e+01,     2.1373634485766102e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.9207618624931619e+00,     7.7653436706970315e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.4906216187985475e+00,     2.1120298927058171e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.6667441900363315e+01,     9.3291441866618825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6548359035594345e+01,     1.4645378662282894e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.5986599339614495e+01,     1.6778039552297192e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1158045376472767e+01,     6.0111281380458140e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2765385526024495e-01,     7.7249764792666356e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.0864333044706616e+01,    -5.5660293416627042e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5196364602439358e+01,     1.6886119984989204e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2508005706990748e+01,     8.5236471327611107e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1675204122095877e+01,     7.4121709725007149e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7689570264253483e+01,     1.1073279256976311e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.4945523826489335e+00,     2.4964744366197527e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.7644671747480185e+00,    -8.5029038196203128e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.6902967314219310e+01,     3.1320795195720471e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1545518853989843e+02,     7.1759789320393465e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3956585807115395e+01,     7.1453707598861693e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.0448931122376806e-01,    -9.8543746376717336e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.3283309673526725e+01,     6.6120636710398476e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8793288139911652e+01,    -7.1079087821358371e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2767410700231935e+01,     1.6086950658617649e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5257212904514910e+01,     1.6714049828363637e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.2109759033493361e+01,     7.1499750899514383e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1086975864809760e+00,    -5.5802562510002303e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.5316532741968203e+00,    -1.6533429922612788e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.0661877492733325e+01,     5.6469546935893513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2834126123997105e+01,     1.6781335776287720e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.0484545789123553e+01,    -7.7587579067696006e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    5.8554355081892329e+00,    -1.5878316222875883e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3279988721238512e+01,    -7.0256755888086964e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.7901465591343175e+00,     1.4914436182934605e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1200544646283298e+01,     3.5344304897178110e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.4780625630785558e+00,    -1.1146180370624836e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.8346109482514130e+00,     7.2927262668069779e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9582215274782211e+01,     2.9042851470890855e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -7.2084142547254189e+00,     7.7533282017366037e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6862391262196365e+01,     9.7637044654601523e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.0392768545226254e+01,     8.3283285865632424e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1387062996909641e+01,     8.3290542628870098e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8051356548453718e+01,     1.3251140364622403e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5864107764644915e+01,     9.8228531795187955e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1084341648581737e+02,     2.3243236181437380e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5549957933739313e+01,     5.3145463525274654e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.4070036320832415e+01,     3.0040769799121597e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.8603968835875929e+01,     8.3281927518480497e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.6026721010325982e+01,     6.6037794886144929e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.3012381995659672e+01,     1.5949445460608884e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.8391510639574861e+01,     1.9622388170319464e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8154998668521429e+00,     1.9548215048286220e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2131393730729915e+01,     7.3192897144143390e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2487065341056448e+01,     1.5178119253901230e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1777125138644635e+00,    -1.1146390814701033e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.7068784483745869e+00,     1.5103332654665628e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.9600555953472391e+01,    -8.6003958611678863e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.1299008483151098e+02,     1.6657037247042903e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2869548842769819e+02,     1.6657728451857602e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.5603618424634455e+01,     8.0315850325080373e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.2700860603017908e+01,     1.0428230717639746e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0756548618430006e+00,    -2.2660168804018731e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3347088342368547e+01,     1.2574369830335865e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8180681084530281e+00,     7.2735814424836317e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4631691689065981e+01,     6.6813841935801838e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.8233096029812685e+01,     8.3104520255105908e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.9802612520726697e+01,    -8.3469308239396596e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.1398329728084655e+00,     6.6033873066889473e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0921162606465629e+00,     7.8274141173662629e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4787729123217346e+01,     2.2931618791522873e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4644754697381764e+01,     7.3729701570924071e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7319925272642269e+01,    -9.5229497877872436e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.4986330126105869e+01,     8.3260203619192343e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1766691012117768e+01,     6.6141412138235482e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9681170094350929e+01,     7.1607414522895670e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.7653212336063618e+01,     1.5752863911322311e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1982650987772139e+01,     1.7520977049848352e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1761513500930656e+00,     6.2449429323240781e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4663778043996878e+00,    -1.0750577328279654e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.0887506212896614e+01,     2.3367100532487129e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.3539759714727275e+01,     1.4288216662876570e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7369940529693142e+01,     3.9571934359433479e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.6693780010427723e+01,     2.3311501602959754e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.3644551239492539e-01,     7.5420906345356125e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -9.8638281596190200e+00,     1.6657935138982735e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5569053807514379e+01,     1.6656830559917769e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.0002961931374834e+00,     2.2720057393773506e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.2599620689020643e+00,    -8.5175773742523379e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0412756765651828e+01,    -1.0616689378412225e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -1.2897540136212677e+02,     7.2136124979269180e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5134134481102254e+02,     7.2145132329907474e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7340863299499585e+00,     6.5857658026170984e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3560327432130972e+01,     1.2618140663845024e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.9465003594178629e+00,     9.6180997771373222e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.4080923002602173e+01,     8.3121639901787876e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9502138558281079e+01,    -8.5322074161609016e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    2.8627673214534827e+01,     4.1781155506048108e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.1408550551570602e+01,    -8.3259599738888901e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0778228087439189e+01,     2.3316761267592119e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.8472476084087056e+01,     8.9614082369397402e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2666092775199465e+01,    -7.5538901707627213e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -1.2122490643579532e+01,     1.6830849687938273e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.0663125875001903e+01,     2.7250900087805541e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4188344215778143e+01,     1.7758893497517092e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.7801270947938246e+00,     6.0372442124461054e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0771925568991261e+00,    -8.9105983439453266e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4314162961187085e+01,     6.1700354296793876e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6304390574435729e+01,    -1.7982930744981786e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.3338517921888400e+00,     1.6709197960940543e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.5096260424955291e+01,     8.1398054751979134e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.1435392585154929e+02,     1.5544023879642549e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8727687325814972e+02,     1.5541484700725368e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2811992298401247e+00,     8.2944578811820156e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7794204817080676e+01,     7.2363543052061523e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    4.0160148265976439e+01,     7.1917714257115131e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.0108675270886396e+00,    -1.6639358641249048e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    6.0553864381915057e+00,     2.4016560830927137e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.1672976782396978e+01,    -8.5014811286865606e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.8505189695451726e+01,     2.4539568201852540e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.7097419841681649e+01,     1.5922657348677491e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.2304601770770898e+00,     8.7735275059084033e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.8113508768195548e+01,     5.6262465025392921e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.4913894072898017e+01,     2.4340493692362979e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4894849085446140e+01,     2.0452869417932452e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1232313597655095e+01,     7.2166658894473130e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.7905184456732307e+01,     1.0234277696165265e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.2655983836768179e+01,     1.4285810487616425e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5555563088829288e+01,     9.4078953736765980e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -6.7486250662048821e+00,    -1.1113619640079003e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.1381264842764498e+01,     3.2435423754986765e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.0056300583633785e+01,     1.5464757403292720e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.7648009709701942e+00,     8.4383275008497039e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.9431172625698361e+00,    -8.3816510461501966e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    5.4804872555571755e+00,     2.3589758433731186e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.7466071427153331e+01,    -1.6668199560809975e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {    9.8161194659589892e+00,     3.2739427211066155e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -1.3736251617093069e+00,    -2.7047461220255691e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.9866200359235275e-01,     1.4456991385632758e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5673757705915705e+01,     2.5132991917156094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3105024843811485e+01,     2.5507963943493705e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.9640466929440706e+01,     2.6084021722664274e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3163593160970795e+00,    -3.2202766428059113e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.3028004225664205e+00,     4.6357697657897937e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2586559280540502e+01,     1.0606635149554744e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9698244654747612e+01,     8.3279754876611860e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -3.1267761145661609e+01,    -8.3294073617890663e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.0976504152900933e+01,    -1.6789464203782811e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.7304115365705911e+01,     1.5489432877014693e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1924752780407797e+01,     6.0971331063528050e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3200354818883637e+01,     7.7029419587033613e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3637162893341635e+01,    -8.3183733862424815e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.5131836978621180e+00,     7.7320230289129222e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2871110854571196e+01,     9.1949154574394088e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4299994966678877e+01,     9.1304658482586929e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.4933217337771136e+00,     1.5140916851777918e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.8891822304942046e+01,     3.9946440467425437e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.2428903352946712e-01,     7.6267875102787057e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -8.8783021741350794e+00,     3.4481464784691161e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.3660553523635059e+01,     8.9520679776147517e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.7896875847671687e+00,     2.3888366395717869e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.4443023929199437e+00,     8.1168150358803605e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.5203939002025164e+00,    -3.0501934886000704e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.5957883024702653e+01,     2.7637362101898798e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.4016067705065204e+01,     2.4205506958380814e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.0191416739551347e+00,     2.4750787658822825e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.3645306291063868e+01,     3.0499860811278078e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.5994772060995599e+00,    -1.7532454255673244e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.4789888096274439e+01,    -5.0205345836834022e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.2291841303891191e+00,     8.4449389004029963e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.8933820304357042e+01,     7.2167163812566405e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.5885711837794986e+01,     8.1620969149452339e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9249184383742822e+01,     3.1005326487039609e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -8.0528370853953746e-01,    -8.0682265106542745e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4891491014293724e+01,     2.3808167336442406e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.7195295945784174e+01,    -2.2119841217657304e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.4197565147990947e+01,     3.8785540104089421e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.5239878007888299e+01,     1.3870419269358970e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.0008303383933717e+00,    -1.7161737507658330e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    5.9022508935864948e+01,     2.4030705322500733e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3476316087169273e+01,     1.0055418233695058e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9422207378117346e+01,    -1.3410987059057961e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    4.7113554231194090e+00,     8.2124439490472560e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.9601964087197654e+00,     7.7960127528464773e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6225725214159546e+01,     1.3424846119534508e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1794460732376937e+00,     1.8209330312030741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6784931788962199e+00,     5.3356990659219309e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5964786657673393e+00,    -2.0371824858856521e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2412711836820783e+01,     2.2287769178188897e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.6899517983973062e+00,     3.2198059596850515e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.3448428736472266e+01,     3.0850221965015364e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3228664617698968e+01,     2.6748138154080843e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.6535235095622278e+00,     9.4690591191383428e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2043098702921764e+00,    -3.1283887636467944e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9807964515895257e+01,     1.5545021279083407e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    1.3944907305543920e+01,     1.7130585912471546e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.3821284058216694e+00,     6.2118791350436495e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5893882629456058e+01,     8.8176269718940111e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0705497559696990e+01,    -2.3705323838905038e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2123956612719123e+01,    -9.4346834856079913e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.3738384945647717e+01,    -8.4952859345050201e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2115354258818163e+01,    -8.1867048139003327e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.4875509574938226e+00,     1.0229613688186240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9911048733861154e+00,    -1.3051188243719480e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    9.2466839392108842e+00,     1.1152414766685693e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2323234565356760e+00,    -9.4885761398819832e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.6444073255003396e+01,     1.4269218439284092e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.8189198287399648e+01,     3.2202214682417911e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.7866256107692259e+01,     1.0992094482271041e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.4058334627860276e+00,     7.6999905934171411e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6049014393724288e+01,     2.3844215752844815e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.3430822769107916e+01,     1.6020438766021281e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.7667718803016470e+01,     2.3156397080935505e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.3012536396140547e+01,     2.4429863521934658e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.8068451350513900e+01,     1.6658478730418225e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.4238961183509005e+01,     7.5238512353210790e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2793281227847668e+00,     7.0281959945520128e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.0653662817144856e+00,     5.0186471085438079e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8382667253547048e+01,     7.3872163308384552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1204804353175486e+01,     3.7148894470158048e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    5.4408135244530918e+01,     1.5596075703353225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5071391002299954e+01,     8.5333098583999272e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.3558851306289794e+01,    -7.6643354232771726e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.2301686987326391e+01,    -6.1156807225852544e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.0665891793929106e+01,     2.4031481269089250e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5375549805656760e+01,     7.3668661876433471e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2444608916162323e+01,     7.4619687191337680e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -3.1019304497829221e+01,    -1.2738881668579425e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7087451624258634e+01,     2.4045264839832689e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -7.4742074544571594e+01,    -6.1276555675711370e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6422254178296164e+01,     1.0600076193955016e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7723706930696775e+01,     8.1452532904616191e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5274378066777690e+00,     1.2090723363177849e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8895598290471085e+00,     2.2738768212287414e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.1258603353708196e+01,     2.3855697971393445e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.8093621285651373e+00,    -8.2293646298560361e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.4748893710192050e+01,    -2.3715960453154548e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.9741409556690908e+00,     7.0035814788410278e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6208919559823592e+01,     1.0024728230334544e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.0004543853356296e+01,     7.3222833081487115e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -6.2259087428764630e+00,     1.2334152182827389e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.7062315316770391e+00,     6.6813152542910793e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5622638962546143e+00,    -9.6463104942224127e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    6.2964836991457087e+00,    -9.6535807952890173e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.9376645842145265e+01,     5.1145235725442126e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.2569991988859108e+00,     1.2936722618351099e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    9.5579395156295242e+00,    -9.6521581043552651e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.2656359714034501e+01,     8.7704142600831892e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0190465854121719e+01,     1.2527203467456551e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9893112093355096e+00,     1.4285427774625996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7418484485097778e+01,     1.6029403896881158e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -8.6611214315089711e+01,     1.0880005698326784e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8477433378652357e+00,     8.3143650982261152e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2495022641699258e+01,    -8.2396739111115239e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -8.9647332561204877e+01,     1.8580389538006370e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4968156959383689e+00,     5.1815111490289837e+01,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.4235467225701097e+01,    -8.6315408950836099e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.5543836456064007e+01,     2.6686634946828690e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8323062317798065e+01,     1.4192839115565110e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.9733378280477076e+01,     2.2932600453338600e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1119490270125667e+01,     3.1905890798060398e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.1483639668900946e+02,     1.1128975907878666e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3303645220111285e+01,     1.8751356397540527e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.1732079894208391e+00,     7.9110967670173259e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5302289144000802e+01,     7.7790920720376971e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6677275423279063e+00,     8.2473363941036769e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4702622567785593e+00,    -7.1834099431491286e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4120543911701443e+01,     1.7197776448274617e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.0704853564927152e+01,    -1.6517697438465191e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.5967650192761216e+00,     7.1967500027245042e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5276102755366558e+00,     2.1639887396536760e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -1.0657934241347302e+01,     1.5330903624513503e+04,     5.1879272061472372e-04,     8.2508016319626376e-07,    -3.9081800461601917e-09, } , 
      {    2.3983162973362081e+01,     8.0404785575462583e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.1337137272562209e+01,     9.3319710361082834e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2125163368351473e+01,     3.7906550476133758e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.7298482975795725e+01,     3.9936569395716200e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.1279748167509652e+01,     8.3270635012813145e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.0476382969968094e+00,     8.5891563387959777e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.1072739060673022e+00,     8.8091123448594590e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3403425635375541e+01,     5.7839808568797253e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5183539720284987e+01,     9.5541549996294307e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.8842296078917329e+01,    -9.4429913515149710e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.1786021238805500e+01,    -9.4436486164678681e+03,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    6.7036980162177855e+00,     8.3846470581716349e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8505688727735283e+01,     2.3353946337173849e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.3503512822613888e-01,     6.2265157382232665e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3902368057267207e+01,     1.6160041248507137e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.1782258358261529e+01,     1.1231339938565368e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8605702593432554e+00,     7.2518002245730440e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4951557188785394e+00,     1.3604862437836138e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9779037533309271e+01,     7.2216894490851037e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7929486977526352e+01,     8.2508105312365533e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3457304670692409e+01,     3.1985520601856217e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.4827635654442743e+01,    -6.6326546191157086e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.9631575109116945e+01,     7.2373693366206371e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.7243379345061713e+00,     3.2135702789821542e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    2.5976028078088746e+01,     3.8316492126635530e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -3.4348644038146503e+01,    -1.6656286968482280e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.2201617054770054e+01,     5.8895325244037440e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.9441530291208146e+01,     1.6801999292591230e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9689671223453246e+01,    -1.1151808487987769e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4329798992982784e+01,     2.6558158152947326e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.8849909292750041e+01,     5.8902371563198922e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.2369687680465296e+01,     6.5438068099244620e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2036207859860438e+01,    -1.1140762697338100e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.7304694947904483e+01,     2.1942423493786846e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6845377249603448e+00,     1.3846717484574541e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.5088048653847508e+01,     2.3258680158151968e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.9077505133891904e+01,     8.3900818293064167e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.0647021624805909e+01,    -8.2673010201438374e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0519087698625572e+01,     8.6622403271428266e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5148161905725109e+01,     7.1791179764488452e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -7.4007334101115362e+01,     1.9876819609995364e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3134122637811934e+01,     1.9570123705024420e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1063340164446558e+01,     1.1330857312076185e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.8135804518163408e+00,    -1.3707083803692240e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.3982719239309107e+00,    -6.1027390105821792e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.2858285110358292e+00,     1.3453696514881554e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.3681770968325679e+01,     1.5953824203232321e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0098020317505799e+01,     2.3480725945055830e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9486435514087887e+01,     1.4759013550637717e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3886112735641611e+01,     1.1079080207816345e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0701845366054670e+01,     1.5136615409291517e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.2593839360198768e+01,     1.7794553313847602e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.7988365988290695e-02,     2.4011131125894142e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.2491552170767148e+00,     2.0649725696353064e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.5478881514585483e+00,     1.5734270072027353e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.7737141663123861e+00,     1.4465664974237421e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1141826621639783e+01,    -1.1249378405217813e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3556373392347346e+01,     9.9314282258067797e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -6.1136143630117424e+01,     9.9349098665763704e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.3315479256351459e+00,     4.9672707358159969e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.1920754419311535e+01,     7.2130812036431034e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.4286697868207298e+01,     7.2150445272745619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3813021105533167e+01,     1.7754738411949689e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2852837776099195e+01,     9.4444235113838440e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.9580035740618937e+00,     5.6577267302025211e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0851282713757140e+00,    -8.2790280272905820e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.9350032489823590e+01,     7.2262687902784191e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.0053621015622966e+02,    -7.7008917982176154e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.1799120121458820e+01,    -9.5732396460263171e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.3367683364339797e+01,    -8.9507442806132894e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.1128283609741434e+00,    -1.5982674598579368e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.6342312056041681e+01,     9.1785460861487663e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4003465091604212e+00,     5.4144597422336110e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4495395383369253e+01,     2.1066850108542145e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    2.0787779993336821e+02,     6.5852473218752511e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.4719301143321450e+00,     7.3424577946792590e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4463996135033721e+00,     7.9640563884423964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.9366323516214603e+00,     1.3422544919694774e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6836529198318750e+01,     9.9030765033368116e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0873825199207516e+01,     1.9946167050135125e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8057953812550871e+01,     1.4365992580944729e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.8792744885277557e+00,     2.9246136263389344e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.2101966219969938e+01,     8.9434924984993158e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2858118825182395e+01,     3.1072390669168461e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    7.0462107185953773e+00,     6.5541160911799498e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2394976497768646e+01,     8.9389500664152947e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6535795489495968e+01,     8.9414569814822389e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1161483970676425e+01,     1.5734828922830893e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1931853237198734e+01,     1.3827048107936391e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7615900623318311e+01,     1.4310411993702355e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3237610053832439e+01,     1.7306353950799305e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5672513489830921e+01,     1.8169948151197405e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8673130312969100e+01,     4.6360695429875423e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6545103355297556e+01,     7.1983151219431938e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.2150298494206202e+01,     1.4956395175627244e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9430149078955523e-01,     1.9155194078593235e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9534547004242881e+01,     6.0192674722322809e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1186113998519884e+01,     9.5257880177914139e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0543255616770622e+01,     1.4936988610269473e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1844377151545114e+01,     2.3342464924732631e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5922065688705498e+00,     6.6015030239289281e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2455113692490603e+01,     3.0392766645509706e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.1533128223192135e+01,     7.4585382687273004e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.8921481061028498e+00,     8.4710066680263681e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5110782675407094e+01,    -9.9073008894980449e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.3277259257121781e+00,     8.4100464553465754e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.0694724359378512e+01,     1.9542758366523638e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5477038685143757e+01,     8.8981298584730612e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6773465434045022e+01,     2.2112967542308819e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4139024237008968e+01,     7.1342094046069742e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0223951403982614e+00,     1.4239738454267528e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.3900504725941438e+00,     9.3631209247159604e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0994235589904534e+01,     3.0628053260862263e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.5591774834889289e+00,     6.6234926395275015e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6257723654476791e+01,    -2.8601792765262653e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0355219101265778e+00,    -2.8444467923786751e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0002418739410505e+01,     1.5529191753105388e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1172360683937121e+00,    -8.3772512947672458e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -9.0801869800304269e-01,     1.3376144253869430e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0972446804821164e+01,     7.5775249644484193e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0429511603331601e+01,     1.5864781954464548e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.9227274009015076e+01,     8.4550416603238646e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4572411795690559e+01,     1.5647981139139465e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7411608933843219e+01,     8.5013462172276722e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.6009020281360158e+01,     7.3031051142266006e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.4660782022243875e+01,     1.5684740901008752e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.9776521680753049e+01,     5.3140478448286285e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7610395649251789e+01,     1.6107783496762822e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1986707208770833e+01,     1.6230580452254300e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.5246625080037365e+01,     1.2827690129675746e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.5876999712798327e+01,     8.2168531762163275e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4716013682459469e+01,     8.8817479485982803e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6082835653037865e+01,     2.5085401069044467e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1686311438792345e+02,     1.5606842065691353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7019152964284398e+01,     8.4000133626139723e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.0242186055024030e+01,     8.2234645757696198e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.5223970464736007e+00,    -1.1043192780108056e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6066860432991589e+01,     3.0379648734310253e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.6410097958921263e+01,    -1.6634840944319196e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.7717122827427382e+01,     7.2493999363943667e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.9722998758858932e+00,     2.3884563626108538e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.5607591158879419e+00,     1.6526870967013059e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.6327827478702400e+01,     2.4383079114862339e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.3351956044889874e+01,     8.9950829431901548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.6011094457599491e+00,     1.5899357438786743e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.2150473004971314e+00,     1.5580491649298170e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5376475654851909e+01,     1.6001845302340440e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7574286318807633e+01,     2.2791762044652776e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.2754815675524718e+00,     3.1300125118145908e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1449689820882590e+02,     7.2781506409662270e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5094559850126888e+01,     1.5004618204472918e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    4.0220130762433655e+01,     2.7473068791958642e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7105659480828564e+01,     1.8562969120890393e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1116075865520759e+01,     8.9761866794788966e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.9301335970879101e+01,     8.3244336689651318e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1310451152364696e+01,     1.6990885242118442e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2416109844226293e+01,     3.3707786668593493e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -2.8751764608359007e+00,     4.2871480392801623e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.3808004516550930e+01,     2.9531473644951340e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5916942115861756e+01,     8.5361633477721480e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.0969862747996189e+01,     6.9968251934616510e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1763821449068743e+00,     8.3150764436929912e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.4187039810774804e+00,     8.2937989796687052e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7495738554918034e+00,     2.4379096597107688e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -6.9622313480502172e+01,     8.3313279705920359e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0616607932185559e+01,     8.3260548788582164e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6059084083747322e+01,     2.1644489796216229e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {    1.0858108981675301e+01,     1.5760007221549187e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -2.1258316140724670e+01,     1.4953909291078251e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1924710571598563e+01,     2.3007851514510985e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1505471480192774e+01,     8.5201919472004738e+03,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -2.9070475645783995e+00,     6.2330746938229931e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.2414442835036240e+01,    -8.6169042919039930e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.7906512326551052e+01,     8.4298967312043278e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -3.9476028817465064e+01,    -8.2274861182459244e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.7054539766683625e+00,     1.6580941605991586e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.5839101654647033e+01,     1.6100779915283982e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.5589103977814197e+01,     8.8053249381528876e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.4066643493487209e+01,     1.6505123118701231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5583518085652400e+01,     1.4253324095037586e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5834377309104251e+01,     8.2637773536076220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9328405633291696e+00,    -9.5764282063634728e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.0365220911358177e+00,    -1.9152525762951649e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    7.7246766717113058e-01,     8.4339182736806324e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1153501707684200e+01,     3.5408739246990779e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.3344495132111660e+01,    -7.9537193983875113e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0107399300470423e+01,     1.6711455474053866e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.7647678300748503e+01,     7.2005003283802635e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0098384406516955e+01,     1.4910643935529091e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6218066682870884e+01,     8.3820128378943919e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2407634857690681e+01,     6.8053094116182565e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.9605813111967052e+00,    -2.1610842143044920e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.2102474664619876e+01,     2.7643921057498530e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7308676815653122e+01,     1.0373978366518370e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.5499023895001365e+00,     1.6099357224350229e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.2632268495231717e+01,     7.2191299374061018e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.5756087791934705e+00,     8.3635838697815470e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0918428788490772e+01,     7.9199227115164995e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2238379514429091e+01,    -1.1489987755645416e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7910605879018799e+01,     1.6658486376842677e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5302291358712113e+01,     3.3890104186734854e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    4.9788795691662891e+01,     1.4915413107586091e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -3.5216636685187218e+01,     8.2753700115558604e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.5436775641393492e+00,     1.6664386430929095e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1800245601848935e-01,     1.3060675545188566e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4265044045988311e+01,     1.7776927181504416e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.3851386934621249e+01,     8.7190760008308735e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.0555075760392185e+00,     2.3964745719436847e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    9.7548418381012993e+00,     1.4476685600959785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2537112437616749e+01,     7.9895631093061293e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1366923439400225e+01,     1.3722265884860000e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.8636032139368481e+01,     2.2813905054307561e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3416176721104231e+01,     1.3964111455812279e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.1480979217377565e+02,     8.2871455543487882e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7135916865574821e+00,     8.3258378409886245e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.8608898884638876e+00,    -8.3315450084616277e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6081269806967036e+01,    -6.2396336494227207e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -8.7567617106742631e+01,     1.4286041068519031e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2808773740238994e+01,    -5.5961547947288091e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.2975525407042774e+00,     1.7583110199821283e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7016641813330153e+01,     5.7064133364995523e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    8.3948202929816063e+00,     6.5688945337191062e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4429373561516037e+01,     1.5780670737852699e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8168886231831905e+01,     2.0292787354002772e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.1009866985185983e+00,     1.6082755707082777e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7959701241587695e+01,     3.3794543728964559e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3656910309523902e+01,     1.8439016194052921e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.4001599342616252e+01,     8.5403184333395475e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    7.6191940307063106e+00,     7.0747060869502238e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3399321563665591e+01,     1.2620755236793160e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.2076863178564345e+01,     1.4359433625345002e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.0163779889940443e+01,     2.4988151816959082e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.6966344671282457e+01,     2.4497123467638441e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.0889929890277958e+00,    -3.5219207050260834e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2660428608256556e+01,     2.3868600800851294e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.9270990490386364e+00,     7.6371054612129037e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8626031631789580e+01,     1.7970664998454726e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8397949566883135e+01,     3.9336647744080925e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.5333977907547142e+01,     8.2381830841941792e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6511355673302461e+01,     1.2934352117917531e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.0746899954414726e+00,     1.5556316827262528e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0645574392546919e+01,     1.5335506024192971e+04,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -3.2127411102109910e+01,    -4.5350594916546447e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9838105047401307e+00,     1.0953481247109576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9027698318733812e+01,    -1.0069326369197228e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    3.0831561338619824e+01,     8.0957060092123047e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2962189547567938e+01,     7.7000248101707812e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.9048651092945672e-01,     1.6660227763508043e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.4566429481839762e+01,     1.6170963823655662e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1632009074116709e+00,     8.3650579403319716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1305106772991724e+01,     1.4936881204845586e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9171629978078705e+01,    -2.8850395347046287e+01,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.3491125989535220e+01,     1.4604658335276170e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.7474024411254980e+00,     2.4983996731391675e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.5029997553917047e+00,    -1.8347349896550895e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -3.8731520309049330e+00,     1.5289418731476029e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.7091631553732274e+00,     4.9492370518286698e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9843643814230212e+01,    -8.3459302047383426e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.4906242644721148e+01,     7.7965220742115162e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.3733206632409090e+00,     1.6151874205521275e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1269786764116930e+01,     1.6569468863229074e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.3149176390590810e+01,     7.5583819694867398e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.1265512944937583e+00,     2.3874290628966875e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.8833962111827578e+00,     1.0240836651764992e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2744891986008978e+01,     6.7233326640690120e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4358357618992001e+01,     1.1258352961108078e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4717823489321688e+01,     7.9333692551924032e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5177289756234661e+01,     7.1376216220001634e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    8.0370240772405566e+01,     1.3936535848332715e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.4336346467831760e-01,     1.4692278000985316e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.5073041359713812e-01,    -1.1147769763235074e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4715264391317849e+01,     1.6654537935392462e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4695684988325242e+01,     1.3069338851899698e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2433068515943596e+01,     1.0095721535746939e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.6902399402764551e+01,     1.5819727634990917e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.4165913307791143e+01,     8.9412512662550307e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7182416009206776e+00,     7.2276254025374019e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0691754270428461e+02,    -1.7428380927379972e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.2315644286972191e+01,    -8.1022649648120332e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.1318960776117251e+01,     1.6155519846941073e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.8909547714942224e+01,     1.4913480302690738e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -2.0769655609829634e+01,     9.4509583175041462e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0803589119658017e+01,     1.4966261816628707e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.7005673667441883e+01,    -1.1144801422090795e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.7702223970669912e+01,     3.2864493141082159e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.0210467888459188e+01,     1.1224454655717012e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4248499654013971e+00,    -1.6733824092908922e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.2771126612584519e+01,    -1.7935783185089840e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.1422159910570038e+00,     8.3978099231636243e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0948268623365735e+01,    -1.1779069377329727e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3643050891050287e+01,     7.1250206166910612e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.6833732346858064e+01,     1.7346683848802240e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.3155982639229101e+01,     8.5431719227117683e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.8440970550152471e+01,     1.1315124827928594e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6687880286852312e+00,    -7.7342270106359592e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4962929729164053e+01,     2.2251368445828411e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.1720478126221217e+01,     2.1619646691245442e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.4761512120502243e+00,    -2.4973455035853000e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    1.6142465679581768e+01,     1.7620638799054659e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.7990756182631522e+01,    -1.9907450948956102e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0131765406008652e+01,     6.8494278291761029e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4932782266042118e+01,     2.6547341441587614e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3410400969591336e+01,    -8.3303193481689341e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -5.5146993005893634e+00,     9.4923352227648993e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9776896227101457e+00,     4.5430981709362277e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9827737030631251e+01,     1.3273670720793807e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.9145910395760275e-01,    -1.6613612015941097e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.4462957420002841e+01,     8.5551178846382190e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9413063507946504e+01,    -4.9028363860033278e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.3977656525099222e+00,     1.2037754265966658e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.2925070512786597e+01,     1.1191525036667455e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7926307847572524e+01,     3.1691274260282404e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0049812297677434e+01,     4.5711365202977959e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.0281891529624927e+00,     1.2636502980308798e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.8429442683755184e+01,     8.3754118565280405e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4078200914676671e+01,     9.2802855021076789e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3437055032193443e+01,     9.8754712723556634e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.0281865753056607e+00,     1.8762173108900250e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1820392431867969e+01,     8.3644217419231009e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6893651154813750e+01,     2.3934724093375764e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.4157619841088938e+00,     6.8030082117785205e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6518816640345950e+01,     6.2291521318105060e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2957666442976237e+01,     1.6701153682959412e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.3227838658195157e+01,     8.3581883756888401e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.3751368601015628e+00,    -8.2991944737614122e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.4492099486340173e+01,     2.8398848101763542e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.1265847387983811e+01,     6.5847888779656114e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9889585545655351e+01,     8.3685426107368621e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2079824883972773e+01,     2.5758998302361899e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2943798496165313e+01,     2.0791712307177855e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.0811100752062828e-01,     3.2748814920883931e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.5364415552324893e+00,    -9.0643767666745098e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.0732746654553956e+01,     2.2400214007040006e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.4133073645599943e+01,     1.7086615925006234e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1214630226955117e+01,     7.6944837855931655e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5328911310468261e+00,     1.2667654575495581e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3222842340812360e+01,     8.8238308532840165e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7578540755662917e+01,     8.3957556068539816e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3978886622814876e+01,     7.7598871573363340e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5390406981032321e+00,     4.6625961670712146e+04,    -4.6251613028794612e-05,     4.4225188804492980e-07,    -2.1125271340666918e-09, } , 
      {    6.3125746467736306e+01,     4.6630564070391600e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -2.0245598182005811e+01,    -1.8700756069966519e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.0635343174478933e+01,     1.6625902621787023e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7107241005691947e+01,     4.0564020801536662e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.2751045965791739e+01,     2.4998693512497757e+04,     5.1879272061472372e-04,     8.2508016319626376e-07,    -3.9081800461601917e-09, } , 
      {    1.4743850677446648e+01,     8.1955232682384758e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4492945644174441e+01,     3.1338844139075845e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.2241449326371363e+00,     1.8451078532387173e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0948790270686029e+01,     8.3063847256097579e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.2172139928303944e+01,     6.7872604682628789e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.2055391238171964e+00,     2.4754184739630531e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4088038104004948e+01,    -3.6236804738328465e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -9.1848933015159417e+01,     1.9502943464625725e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.0502072261243178e+00,     1.0511033945187246e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0022310889026916e+02,     8.3263969384361171e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3357912113630569e+01,     1.6538345695400658e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    9.0991208531168823e+01,     8.0189030548628807e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2461938405962911e-01,     1.2476138793652182e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7940034856152800e+00,     1.5658036861953025e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.8536987684830542e+01,     1.8955990090583382e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.3060687531961737e+00,     2.1435767635739037e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.3058912804560023e+00,    -1.0070916223570086e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    9.7998997811046763e+01,     1.3799811432321396e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9239298583291465e+01,    -8.3294127808839630e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.8509115679702969e+01,     2.7975474536762558e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.9095076700502744e+00,     9.7249920058012722e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1230441101488785e+02,     6.6498430559206845e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2997962847233019e+01,     3.1317398114912765e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.5740274744593483e-01,     4.9426780962289431e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5075330604915138e+01,     1.6346845172925308e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1856273484307557e+01,     1.1661349297708422e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2284203669352561e+01,     2.5092012468597761e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.1252569168185257e+01,     1.1571074959584661e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.6466083935980222e+00,    -3.9639911244328396e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.8723047559663537e+00,     6.9853257299198840e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.3827272885907377e+01,     1.5224984381663358e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0327551909181945e+00,     8.8613700966037741e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.3888853775065222e+00,     5.6550968978863375e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4660262136496806e+01,     1.7699374920492293e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9576901518664226e+01,     1.7844819586988244e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9299909533549851e+01,     3.1078949624768193e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {    2.0613567717721657e+00,     4.7538016224878511e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2448315316590097e+00,     1.2083117823959195e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2622763535265584e+01,     3.5737325866871099e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -8.5254659034334864e+01,     2.3243040078289705e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.9275147739986771e+00,     1.1080171597130000e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7974367854600480e+01,     8.3335614376130525e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2883187315548916e+00,    -1.2477367816383471e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.2479116374693678e+01,     1.0937836342383198e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.2631921256510442e+01,     1.4308681025223928e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7675661339130230e+01,     7.0432105999189462e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.3899717866884604e+01,    -8.2214526438509765e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -5.1235380168957789e-01,     6.0626633160081496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3294410416339566e+01,    -8.2919316628558390e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -6.5168717445546108e+01,     1.0563217451621913e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5024668295093793e+01,     7.6841745043376786e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1218119124128890e+01,     7.6534845009771616e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -1.2890182372168972e+01,     9.5234868179516798e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.5662152631664327e+01,     6.1017355060322734e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    6.3889276059496275e+01,     7.6762053130077256e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.0185425308637868e+00,     1.0344294999908332e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0176405619819789e+01,     5.2335801364444133e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4828442221864712e+01,     6.1770018265727122e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.0258363665610929e+01,     2.2661645323558536e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1072456843257381e+01,     1.7267268089699319e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1067871317046723e+01,     1.5933903432320456e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0452273914325076e+02,    -9.4432219324175803e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -9.1578831851194863e+01,    -9.4434180355652588e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0848819438945112e+01,     1.8958736800787661e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.4811643751696835e+00,     1.0118012975494259e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6393874707098774e+00,     1.6656219387943231e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.5947600598126421e+01,     2.2891803889624956e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.8807571328483359e+00,    -4.7976201862102243e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -3.4491635428890206e+01,     1.5520006669825694e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2005515480223123e+01,     7.2128994039518111e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    1.1882467276136131e+01,     1.5644925999110839e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.3411208890078822e+01,     1.4793914752921295e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.8262742908589672e+01,     2.3902098637218791e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {   -1.8238171188552421e+01,     1.1767617092392284e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8181391994199201e+00,     2.0256704726660075e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2719178555062793e+00,     1.0053995542761306e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.7482420087241394e+01,     8.3288913982995327e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8476714538924782e+01,     8.3284914511507177e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7020450917289320e+00,     1.3815565889387763e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9299008474650226e+00,     1.6163435291603231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2335868660157034e+00,    -2.0054547073537497e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.0527810067028938e+01,     1.7420677168477960e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.9976307125023194e+00,     1.5951523003392587e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8944986680000380e+01,     1.9566263991641594e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.9072296156028727e+00,    -1.0697465854211118e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1194890309425549e+01,     1.5041003462071774e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0219197799680293e+02,     7.6998758811415464e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1212486087472286e+01,    -7.2080025827361815e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    3.7703292948921458e+01,    -8.3913232475751356e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -1.9217549026546251e+02,     8.3284953215774476e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0316978471714589e+02,     8.3288875278728046e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6762295283155542e+01,     7.8358114949046467e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.5866372357189430e+01,     8.5744863660176015e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.0521683480592330e+00,     8.3292350161606173e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6904870146152831e+01,    -8.3281478332896349e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.6105642814175582e+01,     1.4954613922994404e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5743420977037736e+00,     1.2243482010615811e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.4309036371107574e+01,     2.2734275250511735e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0128937833595666e+01,     5.5060716101569797e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.1028743221316130e+01,     1.5933515290526919e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.2687095711956289e+00,     7.0901099386213646e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.8830604323904110e+00,     8.3847283969994078e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0090598096248065e+01,     7.9557816818031642e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -9.3314594587347131e+01,     1.0545944454855056e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7218580070773424e+00,     7.9820900724297226e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.9502887441880361e+00,     1.2274633605802592e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.6477252914367213e+00,     1.1941131213134402e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6687353609161914e+01,     1.7843396896054492e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7380400336559816e+01,     2.8030383071591787e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.1387084783665259e+01,     8.9937587716399557e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9018793987841718e+01,     6.5852029908808063e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5847653887772740e+01,     1.1361525493753939e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -9.2493341275566650e+00,     7.7381211987937950e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3373317344632989e+00,    -1.0278875103382582e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.2554819237364462e+01,     2.3559926376568415e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.3452303139684343e+00,     5.7870941027049348e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.9472407761393072e+01,    -2.5004123217530752e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.8762975029088459e+01,     1.4350840189493305e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6403420494454735e+01,     6.5859036974705032e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7186384951406630e+00,    -8.4602440036646094e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.0416170653654646e+00,     1.5543883540750694e+04,    -3.7427713144392395e-05,     1.6976720671412619e-07,    -8.1225684133091668e-10, } , 
      {   -2.1893842874444310e+01,    -1.5541625039617227e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.1422568142446943e+01,     8.3992326140973764e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.4738090896071299e+00,     8.3133135441756422e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4084254994281544e+01,    -7.0099431046611026e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.5291837071093411e+01,     7.1427172440883614e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3030793670515028e+01,     7.0630447193678071e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7817705054173224e+01,     1.1739397123774006e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    9.4059314915566503e+00,     5.4155706033138745e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.4194594020009013e+01,     1.5540014141250733e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9726744108530337e+01,     9.8850275599814358e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0941632669626106e+01,     6.4122884419252678e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.6913604552011248e+00,     2.2128838203073698e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -9.4099764593506762e+01,     1.8874635879580183e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1477184796602380e+01,     1.6100397202293556e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    3.2868412891395764e+01,     1.6099739937340657e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.9944817607920040e+00,    -1.2543481811916424e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    5.1581784539678326e+00,     1.4632759423960517e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1549145935485299e+01,     7.8313498237820122e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.6793495528485920e+00,     1.2543701648658111e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.8731160449244904e+00,     1.7976131169514272e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6179910465651592e+01,     3.1191446774790300e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.7184996695528113e+01,     1.4523726935285162e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1665893022898761e+01,     1.5588533328234298e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -9.4323651640158261e+01,     8.0725290165707956e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8613836749983825e+00,     1.6093064988338265e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.1304070693607834e+01,     2.2688125050070124e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5273648340808201e+01,    -8.2385199253906230e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -3.0296813634293404e+01,     2.4357414373171763e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -4.8929524383056325e+01,     8.3282686137129404e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -5.6782226181149916e+01,    -8.3291142357373137e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5404989789639568e+01,     2.3819187963164768e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.3397095124715719e+00,    -1.6652973622281184e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    4.2427919083612068e+01,     6.9682679241663573e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5044066651094981e+00,    -1.1021770397411810e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.0524458287421445e+00,     1.5007746709666179e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.7840414724637967e+01,     8.3313145434905855e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.6846120272954579e+01,     8.3260683059596668e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3372048455380515e+01,     7.6217204383314365e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.5457473320326871e+00,     8.6933264610078550e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2720517609503810e+01,     1.5461399259562510e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7759050726977819e+01,     2.8313288811792190e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1701932054721368e+01,     1.0218545212017674e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8475330075945564e+01,     1.7685200949887792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6562747091888909e+01,     7.0841432034239351e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1358855184655374e+01,     8.7282019401171219e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1121707579866019e+02,     8.3309859110141351e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5212687385717709e+00,     8.3440693052746083e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2099086541759178e+01,    -1.9517742646756817e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.3483324596754535e+01,     8.3961153037013628e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.7118497374813760e+01,     1.5545494439117187e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.6839523104134685e+01,     7.1913152451005681e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.8678245118210715e+00,    -1.2763255779863077e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.3210139430943833e+01,     1.6028746631928258e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.7209034263339440e+00,     7.1953273117907511e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6962708520122565e+01,     2.2756425249898839e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2115537659902795e+00,    -1.3679488586061748e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6059892497963091e+02,     8.3278311502290144e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.5902940848871688e+02,    -8.3295516992212360e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0126567030019293e+00,     6.5782771138442322e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7238844725254939e+01,     1.4356780109547193e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.1847924553568134e+01,     8.3462635168861852e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9334630489955003e+01,     1.0627298665858252e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3512175047921247e+01,     2.6152019272054888e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7242977613498677e+01,     2.2127798225130373e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.0275127283856136e+01,     8.3502669274085038e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.1844643774770141e+01,    -8.3071159220417485e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0057563788351860e+01,     2.4035739024849245e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.4982633833248158e+01,     1.3011161633538009e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4343622294809007e+01,     9.2741060600265573e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1485025030705369e+00,     3.2194103665239261e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    8.1391647851330884e+00,    -5.0765088219860206e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2350747672234789e+01,     3.2206170614029157e+04,     2.6936461440465505e-04,     6.0169256335142287e-07,    -2.8559889140481653e-09, } , 
      {   -2.4508579279614899e+00,     1.8369152139207261e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0659944784428583e+02,     7.7644716151869779e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0332540208088115e+01,     1.7337006873109007e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.8209982412619041e+01,     1.7003504480440817e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -7.8802367756147120e+00,     5.2620759281933497e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1292998991091984e+01,     9.6251483050418283e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.8432034455997080e+01,    -8.8601554394617779e+00,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.4148785687016534e+01,     5.1543384744421246e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8824275676446405e+00,    -8.0999637649722972e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    7.6446517487312615e+00,     7.5633130016426612e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.4933732146167436e+01,     1.5231234951843287e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.2688178873441753e+00,     3.1532014652690756e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.2405698348293459e+00,    -2.1063982433014230e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.7679766143181567e+01,     1.3817867089227497e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6265480734874451e+01,     1.7285361802019346e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.9999672115337574e+01,     8.6384797945873725e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6403094721151579e+00,     7.3319555095736487e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.2540354022811631e+00,     1.6498475557816677e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7967518676727469e+01,     6.5829103663554979e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0346817498192273e+01,     2.5597108862454064e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7383661122205929e+00,     1.2869159763345171e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2113155662924697e+00,     2.2726164233333082e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -3.6210919435712881e+01,     1.0976908498889467e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.3816507209347378e+01,     2.4928986375510609e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.8983773222509592e+00,     3.6728382017115359e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -5.7837476426799093e+00,     8.3390094632077708e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.4770359497435621e+01,     1.7736586798401724e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.6858168963166463e+01,     7.9041902273689102e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.1540140911709624e+00,    -8.9635579653703971e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.9001979365234767e+01,     8.3551419053389254e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0770915881746390e+00,     1.6153012931874127e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.9223297402511307e+01,     2.4080642864781734e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -6.3924406971128178e+01,     1.1293696745594338e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8100921678354467e+01,     2.4081347496697879e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.5750971394363217e+01,     2.6951179122640315e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.7382502260437285e+00,     6.5853620341508476e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1504020903154029e+01,     8.2834372544596517e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1927373437188669e+01,     2.2141627481796702e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.8256274714053307e+00,     7.2192515740783729e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.9130709441853426e+01,     1.9702017974115290e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9848366910097965e+01,     2.1993522836615084e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.8169082068532084e+01,    -8.5217324598197392e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2824528915077092e+01,     1.7383906211811427e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.1519241515994846e+01,     8.3931275095351830e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5874561555751068e+01,     4.7167552941983820e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    5.2421312278994705e+01,     8.3931240395404075e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1248672804952564e+01,     6.9248835958322870e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    7.7889354654506855e+00,     2.2333143263640100e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.0303657028451347e+00,     1.6075460670862005e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.6565179859129103e+01,     1.5280332218451440e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2047206884160111e+01,     2.4981143242019458e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    2.7785479870682646e+01,     7.3274740967576217e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.7740697676098851e+01,     2.3349787669752801e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7998460064717484e+01,     9.0764437154644056e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0264701037123523e+00,     1.4370747406537403e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.4789681888828028e+00,    -7.2194925704918242e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.1491107188668330e+01,    -1.1303610434138845e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.6984446150537416e+01,     7.2018569406392471e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.0206805606823028e+01,     3.8392871450354512e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -8.0708035868823274e+00,    -1.4885047046359334e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9664844244033022e+00,    -1.0856515522702037e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    8.2778351421106642e+00,    -1.0190685853885029e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4781068562463815e+01,     1.2967390800028854e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9387229132693157e+01,     1.6774776820687992e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.1564546734539757e+01,     1.5751951440056606e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.4489412313924696e+01,     1.7867098680838917e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    5.5731023572432974e+01,     1.5752656071972755e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    4.5912037312643207e+01,     7.2135984035220426e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -2.3546093863747426e+01,     7.2145273273956209e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.2320466870818443e+00,     1.6081333016149025e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7733655040165113e+00,     8.3448408259524331e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.4450402168773131e+01,    -3.9114914213649627e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2664606440833385e+01,     2.5593051951243069e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.1011946219653268e+00,     9.8957995965946066e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.2308148793800342e+00,     6.9607273067509032e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0367659461485795e+01,     7.1327078348373825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1048887948828693e+01,     8.3362435911002358e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2020700591659498e+01,     1.4923306860577877e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.1118651710040695e+01,     9.6233905282356991e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0951247396022605e+00,    -8.2933543537895912e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1037659931589438e+01,     2.9733154920040124e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3562285384685707e+01,     7.2152263269658542e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    4.1828736133461739e+01,     7.2113227165256076e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3836306026527758e+00,    -1.0267145730451157e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2894338140520549e+01,     2.6165137183254341e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2806615760594227e+01,     1.4988745710327459e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.6367334983808760e+01,     1.1995272714263183e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7623197108045716e+01,     7.1588020441940262e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.9989140556941493e+01,     7.2693236867236392e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    8.5095115622086894e+01,     8.5226988622674180e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1159476348474202e+00,     6.8122254270527928e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.1726885543778662e+00,    -2.8962619038689654e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.0585432545463554e+01,     1.2854722314121707e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6363180081126835e+01,     7.7474528828043567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1237469997840570e+00,     1.1850461040922824e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1835307251030169e+00,    -2.4184557095228320e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.8777962481991963e+00,     1.0173799277963844e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.0146470071444478e+00,     7.7032287537373741e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4572179064110543e+01,     7.6740754490530253e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.7747567356854255e+00,     6.4633179324674825e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.6813085954426725e+00,     1.1690096854266209e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4551674671595146e+01,     8.3538363538446629e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.6856401439326461e+01,     3.8786244736005581e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.3514642571122998e+01,     7.2142123512428207e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {   -2.1148699122227217e+01,     7.2139133796748447e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    3.2306183116584840e+01,     2.0339188019828114e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9346784953019345e+00,     4.5909101215648133e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0648128854796404e+01,     2.5365977332668910e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9463012324400321e+01,     7.2168030143920578e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6894025773178328e+01,     3.1572540900055545e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.4986085366057438e+00,     7.8761575078044016e+03,    -2.4676422471716020e-04,    -2.2400399195900355e-07,     1.1066308603638158e-09, } , 
      {    1.1767884169159714e+01,     1.7461452843026491e+04,    -9.9779237061628989e-05,     2.7639906699208196e-08,    -1.8858767381479843e-10, } , 
      {    7.9675760712239025e+00,    -6.6209464234556845e+02,     2.8263352852431007e-05,     3.7539802067464924e-08,    -2.2902113481933369e-10, } , 
      {    2.3851919247223758e+01,     2.2723035728139828e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {    1.6166298834302200e+01,     2.2790598583145755e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.6382189520026337e+01,     2.3923703466653402e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.3347372537562414e+00,     7.1154474160566197e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1052135622044002e+01,     3.1379754921941734e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.7679978187618115e+01,    -2.4671583068332784e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.3403203043862888e+00,     5.3339001734552448e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3693797806760415e+00,     1.1881612636109603e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7525412568714991e+01,     1.5062992271562935e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -9.2271930163242502e+00,     1.4354831225665534e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.9114146839695298e+01,     1.1466300226669300e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.6295962819144538e+00,     1.4860766262460349e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6165316096793862e+00,     1.4988034364860583e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5080322138656451e+01,     1.8832037163745910e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.1320764380004846e+01,     7.3549459488912016e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0177354789648810e+01,    -8.3905112015896557e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    5.1000617969587392e-01,     8.2829123866747868e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.8665720370135142e+00,     8.8194216261149468e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7846629001707054e+01,     1.0993097163648004e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.9267724614329993e+01,     3.2184595111345785e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.3046729338640368e+00,     4.1500772012432408e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.7019717700656159e+01,     8.5537829188442774e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.5622608407404002e+00,     8.8774385442173643e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.1570123085575979e+01,     1.5565501910542225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9760513941898424e+01,     1.3602474604860987e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5764449565080589e+00,     6.0768902253456770e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6228054521457324e+01,     7.2143816690098483e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0053559360689759e+00,     1.5533875901731220e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1314156325938741e+01,     3.0421152578919962e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.0137859518871970e+01,     2.3878923881478142e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.0016381350292569e+00,     2.3791644766874015e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4776650750564514e+01,     2.1766655455459091e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2536973111983071e+01,    -7.9798230835302775e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.6489328276906296e+01,     7.9483086449267585e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4491814459284971e+01,     2.1834263229023432e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.6713597462671750e+00,    -1.6550221580906591e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.9207122178925836e+01,     7.2368104858170973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1298197466520762e+01,     1.6656570803464776e+04,     4.6359532554209602e-04,     5.4764553418133182e-07,    -2.5876930229221479e-09, } , 
      {    5.8734386523923362e+01,    -1.6658194895435729e+04,    -1.4565567702329698e-04,    -4.5399892353821387e-07,     2.1576632877779900e-09, } , 
      {    9.6133251260145620e+00,     2.4516673980763411e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.4069938288714676e+01,     5.6632013537939310e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0008477403470417e+02,     8.6610216997184270e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4002771505424917e+01,     1.0999656119247729e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0301807236425869e+01,     3.0551255087761301e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8551092058185105e+01,     7.5318327739794331e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.5410366757603100e+00,     9.5703502616780988e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4936678976456022e+01,     1.5762308421388925e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9961729035657832e+01,     7.1207628609310696e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.4822975896223122e+01,     1.8307018989234326e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8135258804397154e+01,     9.3657524636112485e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.0316015267855512e+01,     8.1356503896305130e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.2951202655468855e+01,     2.4266767884441808e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5731824936983019e+01,     1.6483881800021940e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    9.1971362571265431e+01,     1.4985433297000085e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -5.9208485961799816e+01,     1.4985446724101537e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.3862119274780600e+01,     7.2137440619078170e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7641376023475416e+01,     1.0417749200027531e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7660809257297711e+01,     1.5115951892988005e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -4.0791803769408927e+00,     9.8853561924578862e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4347532282068514e+00,     6.5857447582094810e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5543899029729165e+00,     9.2674920115490640e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5326847431353919e+01,    -7.9853460851858472e+01,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7096310142278604e+01,     2.5526012886849083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7184299808270396e+00,     5.9394234871665412e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9965083597517136e+01,     1.9349011060276367e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.3554321051703511e+01,     2.8036942027191515e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3894230398639827e+01,     1.0544815372690209e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6192770394995136e+01,     1.8129607768094713e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0712915581694965e+01,     1.0719062889676134e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5810069249913930e+01,     2.1558294022566952e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    7.1884715403846258e+00,     7.5606831693264767e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8172609410959357e+01,     2.3250616296432599e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.5168893357204389e+01,     1.7039587107783518e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9829940394152722e+01,     1.9047754314401260e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.6029169234032548e+00,     1.7232009991817729e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.6291362128840973e+01,     5.2339682782379487e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7787588947333006e+00,    -3.0977670917185560e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.9631409663067814e+01,     1.6480850245091748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9728520882959355e+00,    -8.5338316633867762e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.5642374507537255e+01,    -1.0661771589849532e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7522451374131073e+01,    -5.8466126797297338e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1752502232500230e+01,     1.4434737130470958e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3615533790331158e+00,    -1.1015403862763371e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.3348342431936178e+01,     2.5285795239340605e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1683245555034130e+01,     2.3863967548340028e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.7043486665346430e+00,    -1.0513501807996145e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9693912031808470e+01,     7.7802373005314412e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9228995593860807e-01,    -9.6720476437442467e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.5167835762665876e+00,     8.6939310717113385e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7321953174660649e+01,     6.9166431001333422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1073466911439134e+00,     5.9361177873898941e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6589405845234140e+01,     7.9482896933545180e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.3896864020162674e+01,     5.1038429009758420e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    8.1861597895682348e-01,     5.7199011106651633e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.9562503853531110e+00,    -9.0305953391715466e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.9384763651712085e+01,     8.6813720113154232e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7720744219970754e+01,     2.9435872440583844e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    7.0939123019713968e-01,     1.3515844924222536e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3426495875076039e+01,     3.1415379252277162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.9541910915767829e+01,     2.3869714746430654e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3860181094355740e+01,     1.5330877901339194e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6349540492463341e+01,     1.3625016509992043e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6813225199115571e+00,     1.1457440071229836e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5371506864064912e+00,     2.3862585559469622e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.4325312424309857e+01,     2.3880305870348548e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.6043290640292117e+01,     2.4287876911109863e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8716780239236428e+01,    -9.4035050820935066e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.0845892219309697e+01,    -5.9061312567545328e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -4.3295192182417592e+00,     5.8400537241300062e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.6253022416436878e+01,    -2.3708914133993044e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0062756821388284e+01,     5.0290905126805292e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0857111857224346e+00,     7.1257566973121075e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5764506037823736e+01,     1.0051950702237780e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0381326573679967e+01,     1.3781793937670915e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -6.5589575345318266e+01,     1.0229156208150185e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0524731803856636e+02,     8.9564910512326351e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.3421375832643356e+00,     7.1353546331007183e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.6580406184320694e+01,     7.6761109660701004e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0813335596269926e+02,     7.7708691865539458e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9145323701240969e+02,     7.7718851036300130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8602207938223074e+01,     3.1165690673696226e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3345770003978622e+01,     1.7343565828708735e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.4216585367321542e+01,     7.1986849849093496e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0486237825338087e+01,     3.1084797234901042e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.8756775174564854e+01,     1.7216512001942701e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6150515253233966e+01,    -1.1270800787914077e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -1.7268575081252173e+01,     7.1407683602067027e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.1471443148870836e+01,     9.9165077876446667e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.3399798594539476e+01,     2.4188233961613958e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.3015232298544674e+00,     2.5315926566459475e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -6.4619004656230885e+00,     7.2129582737295141e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1945592089494399e+00,     1.6656987015362436e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -2.0860664264683670e+00,    -1.6657778683538068e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.6878027905273729e-01,    -1.0032948707512051e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.1028663262376064e-01,     1.9863683756967086e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0563117434132529e+01,     1.5837559482561313e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    6.4983067854245160e+00,     9.1452185676235385e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -5.7837834317850962e+00,     1.1488591666416618e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.2768818662519323e+01,     3.2215679167922637e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.1489443389602329e+01,     3.3301647787701047e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.0071674646418046e+01,     1.5551632678636695e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.9106842524507996e+00,     8.3120888431207804e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5307054709446579e+01,     7.4470482809716541e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.1299940522507832e+00,     1.1297075884573220e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4681017408461270e-01,     1.4980569997662886e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9256153310490660e+00,     1.3655201702731145e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4107349106736384e+00,     1.5496975252133616e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    5.2984659132067389e+00,     1.4710311684972639e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5476933163482208e+01,     7.4155680533084223e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.4517588132079428e+01,    -2.5409782932901890e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.8218767162709728e+00,    -1.7642660522896886e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.5902394443228802e+00,     8.1231438566715142e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.2755570664914632e+01,     1.4284061577309172e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9694211140895455e+00,    -3.9957992529271496e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.2794195788381867e+01,     2.9828541094318756e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9317633712086703e+01,     3.7635215444378802e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.6292795093804546e+01,    -1.6184243666708305e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.4313013133583155e+01,     6.4558356183783926e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.1160255807213282e+01,     2.3325621423031589e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2833649356478787e+01,     9.3987218451287317e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.6695471112371095e+01,     6.3087819356063301e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7908785201549357e+01,     6.6610063415856785e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.1903180812209140e+00,     1.0350680937129891e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.4865056169524102e+00,     1.5447153085816462e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.8370191564268985e-01,     2.6251856304191296e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.3255115923126265e+01,     2.1641836280418422e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    2.8823926344144986e+01,     7.2151674571881513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6563073271465569e+01,     1.5527376409634475e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7786603387975958e+01,     2.1642540912334567e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -2.5056627199465368e+01,     8.6792610989329623e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3737352496486112e+01,     7.8177778109173223e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.0726048603357899e+01,     7.5268827902512721e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.8981598741666986e+01,     1.4914595122195629e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.7129317267571054e+01,    -1.0071979884995038e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.8490830411731700e+01,     2.9813145666761189e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -8.6194997847990820e+01,     1.1214066941798512e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5694583281689219e+01,    -1.0281529350825760e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.9929883761075288e+01,     3.1905684233692427e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4233813057297460e-01,     1.1064419101536849e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5807061823031670e+01,     7.5272303742999611e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1088989431483574e+01,    -7.7702736176995559e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8530621398392171e+01,     7.9866117770835444e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.6294086471236673e+00,     1.0278377162150873e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5165105685614167e+01,     8.2065438949608397e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4455036695562518e+00,     8.7987058046266993e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.9820691689691920e+01,     1.6228149773894271e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.3750390894139919e+00,     8.3298643620182684e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.7279544745470474e+01,     1.6508430736108494e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.6200562524282249e+00,     8.3275184874319839e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3101910455761869e+00,     1.2983123284176441e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2648770847720192e+01,     7.6316575231666329e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -1.1910593130741227e+00,     1.0671398131843862e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6379274844728428e+01,     6.7622878847717948e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8350559065619066e+01,     3.1067484372166713e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.0302126568801150e+01,     2.1500913331018583e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.6921166665496855e+01,     7.8422723989305359e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1471957773431971e+01,     4.9641574899907846e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.6268457118931732e+00,    -7.9560465553304311e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8930973054255869e+01,    -9.9858960127492082e+01,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    9.1260583826595138e+01,     2.4970532245886949e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.2115929702670321e+01,     2.4424502238782239e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4270814368355520e+00,     1.5422097587337017e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.7476231637989095e+00,     3.5191397838815128e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1642070911270700e+01,     2.7866089761651630e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.2923955229242843e+00,    -6.9481233277965721e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0595441496518767e+01,    -9.4372865095964735e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {   -8.5487063193879087e-01,     7.1140247251228666e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3724180232477654e+01,    -5.7226597528922666e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -7.6520090713505198e+00,    -9.4493534583863675e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.4518799637657501e+01,     9.5879364271323975e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6443784182559408e+01,     1.2596172072992227e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8657311191470686e+01,    -1.1174734733240848e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3596490786945033e+01,    -7.1032278498208207e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.8727047502652290e+01,     2.2335444463479835e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6678080788430130e+00,    -1.1222304544705667e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.1034445195444018e+00,     2.0663516303508914e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -6.0813694433078467e+01,     2.5685872703685145e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9676826034933274e+01,     2.1447232888340513e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9653103523021663e+01,     1.5247949097806604e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -7.0843812107920087e+01,     2.5001616302463808e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -9.7894276181319739e+00,     8.3271908429756731e+03,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {   -1.1358944109045986e+01,    -8.3301920064745791e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.7626980084041811e+01,     1.1241432280299294e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5437165838241874e+01,     2.4771457736397388e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -7.1954444034440677e+00,     1.1095570696723629e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9696962131569663e+01,     1.7332091100321140e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    7.4502296969941861e+00,     2.3782428201295486e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.2383956738739855e+01,     1.6460333437491563e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5412664750133840e+00,    -1.0464648538729951e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -9.2625676423397785e+00,     1.6326495029730198e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.0873288044440287e+01,    -7.7092439951190445e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8350205617599600e+00,    -1.8558053348102523e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.6323981245404777e+01,     2.4691269081798022e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.1234966696851167e+01,    -6.2239011652751306e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3074499791020374e+01,     7.2178293509543346e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2657331696139824e+01,    -6.6169221349681193e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.6986056228552489e+00,     5.2647057605095342e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1823160920152219e+02,     2.2756309197104758e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {   -1.7507179846640645e+02,     2.2757325114180825e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.7144792454469888e+00,     1.5687869406202011e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.6844399205557849e+01,     7.9859936828699701e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8628166885429209e+01,    -1.5387945628750058e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.3454802855535597e+00,     4.2314322318646909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0522022147976685e+01,     1.6872965789675924e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3530357587958120e+01,     8.3047671624375034e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9103888484944566e+01,     8.3284763429835202e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.1096000481671311e+00,     8.3289065064667338e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8204280955158075e+02,     8.3544918568136491e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.8047329306066675e+02,    -8.3028909926366014e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.6760655780302307e+00,     7.7521894753837523e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.4547771891176396e+01,     1.5638355494551455e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5308324545541989e+01,     1.0151794402997224e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.1927857129290471e+00,    -7.6328753752017528e+03,    -2.1663523609776755e-04,    -3.2372880868492796e-07,     1.4796077215150038e-09, } , 
      {    4.7992296025727619e+00,     9.6190523823595358e+03,     1.1615568567377584e-04,     2.1467685547360962e-07,    -1.1213643481327344e-09, } , 
      {    9.5534654760219926e+00,     2.5648133276561312e+04,     4.2344506844938082e-04,     7.1488269221921978e-07,    -3.4396027752310134e-09, } , 
      {    1.2439503930671744e+01,     2.4462517041679675e+04,     2.4070213868925065e-04,     4.9582864794434457e-07,    -2.3531886453694655e-09, } , 
      {    9.3248631325688187e+00,    -1.8015317921684713e+04,    -2.1194173061026206e-04,    -3.9000836576856623e-07,     1.8496611561691016e-09, } , 
      {   -7.6352018513200441e+00,    -2.5580619182286417e+04,    -4.8510367241289161e-04,    -7.8881677858842410e-07,     3.7873111473227731e-09, } , 
      {    1.4275751304478545e+01,     7.7038630508170882e+03,     2.8564543351820303e-05,     1.0590754863837813e-07,    -5.0207304815703544e-10, } , 
      {    3.0087393173053066e+01,     3.8752141002707111e+04,     5.3429691669250249e-05,     4.1465561457702140e-07,    -1.9232122397302292e-09, } , 
      {    2.4432900858727784e+01,     1.4184819217949498e+04,     2.6495649449053485e-05,     1.7476078762955441e-07,    -8.3174635131151991e-10, } , 
      {    5.4826702161456096e+00,    -4.5250953655520607e+02,    -3.9917490772209256e-04,    -4.7437147315759014e-07,     2.2936971585605145e-09, } , 
      {    2.8223589173017015e+01,     3.8333328492964196e+04,    -3.8170641629324650e-04,    -9.5537383999445148e-08,     4.3400520732925652e-10, } , 
      {    2.0547586449346898e+01,     3.2375929558327487e+04,    -1.6340140915568555e-04,     9.1014072354693187e-08,    -4.4215007717189690e-10, } , 
      {    2.6043954299526160e+00,    -2.6276495252134948e+04,    -4.2591308521512108e-04,    -7.1435356656765692e-07,     3.3837086059358957e-09, } , 
      {    8.9948414682891062e+00,     7.1093181427060517e+03,     3.9912286297342408e-07,     6.8324113339613344e-08,    -2.7377913385936605e-10, } , 
      {    1.4613971769323268e+01,     5.2953399322507812e+03,    -1.8481182366289703e-04,    -1.5020080528369891e-07,     7.5674082670706345e-10, } , 
      {    5.0310439017252762e+00,    -4.5256955688036578e+02,    -4.0430680598074135e-04,    -4.7322595539186449e-07,     2.1833632610136067e-09, } , 
      {    2.5868037393291917e+01,     3.0004637068239070e+04,    -5.3401916693459478e-04,    -3.4594849842933157e-07,     1.6203442850042910e-09, } , 
      {    9.8519685622258528e+00,     1.6275860988510382e+04,    -2.8262009918534334e-04,    -1.9137050434436715e-07,     8.9855367656841395e-10, } , 
      {    1.1471579707909136e+01,     1.5786060803152126e+04,    -5.2968551440189283e-04,    -4.8374224287428395e-07,     2.2782364502699313e-09, } , 
      {   -1.0000885672817921e+00,    -1.1117836452085021e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2250445777075491e+01,     1.4218576265086944e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    4.9010662531491889e+00,    -9.4433199839914196e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8825628714840967e+01,     8.3289882588395540e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5955494777993399e+01,     1.6678046365753762e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.8313641203485744e+00,     8.3283945906106983e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4831232665492717e+01,     8.9503876102173763e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0735763769823805e+00,     2.4880136079752996e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.3155055700260121e+00,     3.9121607535745006e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1154641657129079e+02,     1.6171159926803341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2062028327032724e+01,    -7.4805163353662174e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3445082532028248e+01,    -8.5342389927787372e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9340463904988422e+01,     3.1729563908929096e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.1547896113920979e+00,     4.5496571265359553e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.4716021250583580e+01,    -5.5732099318387327e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.2747012374942756e+01,    -5.5730756608242018e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.0376908829271471e+01,     3.8235137096014078e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.8704975723651216e+01,     1.6076144307529483e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1922398960358485e+01,    -1.7408586015281362e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6721960801741616e+01,     9.7587734333042354e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5671835409887368e+01,     1.5543375917958963e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1405217183512203e+01,     1.5542132662408954e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.1508223098176558e+00,     7.2294407460083148e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8012928186727990e+00,     2.1217929814690615e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    6.3275834752826352e+00,     8.9253598205373291e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4291462251709426e+00,     2.0691963351453796e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.1127295613423202e+01,     7.2137776377724922e+03,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.1927822896635757e+01,     2.3731760303924020e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0322837524805788e+01,     3.1574459386904855e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.9247974830139913e+00,    -8.2610038190893756e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.4860405101542682e+01,     2.5891355204027881e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.6015180221240755e+01,     8.3694727290707160e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0050593235302578e+01,     1.0309528757337655e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0271010896704333e+02,     3.0457194281771648e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.4325657605346205e+01,     7.6868773284082290e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4068461101553822e+01,    -7.2143480931451732e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.2522949419933063e+01,    -5.5260821264806509e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.9716878684095729e+00,    -9.9127202774033885e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.4942892204935990e+01,     2.6545040241747876e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.1816256585676905e+00,    -1.0831932358901104e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -8.6177806688707612e+00,     1.0702549727030640e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.7405238189661020e+01,     1.4301681148381302e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.0713495017685268e+00,     1.1548110243441415e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9434914072510630e+01,     8.8348514800063804e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5228716311090793e+01,     2.9924494614644325e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.1343145921598357e+00,     1.7505557938809161e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7837651549366715e+01,     1.5589659675724952e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.8360283222579241e+00,     8.3291360506042402e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.2665118313439194e+00,    -8.3282467988460121e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.2575405531089629e+01,     3.5801760216683768e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.7838932280771651e+00,    -7.9260089498570244e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.0013234091732187e+01,     2.2683522650390660e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    1.6769602383525982e+01,     8.2081594725080067e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -4.8173745513036891e+01,     1.3517870021753406e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7061263359855200e+01,     2.3731726092983736e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    3.5278920701650577e+01,     8.2892288387852896e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1468524797020523e+00,     8.6441790849782301e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.0723695463250802e+01,     1.4094064487445938e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.4832986725916882e+01,     1.5058734515802942e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -1.7213997193617999e+01,     1.0722477059213561e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7285508982017305e+01,     3.1086219925834794e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.3521513878729698e+01,     8.8425771670160839e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.6081152297237292e+01,     1.8092395890185278e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4865367657778290e+01,     1.3805680969285538e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0958798710402831e+01,     7.2120985177881648e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.3613965734554192e+01,     1.6018258553044989e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5092230380211236e+00,     7.6481260879352676e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.7667993895474234e-01,    -1.7084185246646208e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.2562982382922398e+01,     1.3206455449836563e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.1193850835308247e+01,     8.2718092412875667e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5141614322791124e+01,     1.7215089311008949e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0769807570884719e+01,     2.2107179662619397e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5381269500728392e+01,     1.4757866336518886e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    8.5643058952909428e+00,     4.3561886590900474e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.8062954495623629e+00,     9.5419354333936335e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4446043436663842e+00,     3.2530974074299925e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4196145172719365e+00,     8.3237157178226571e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.6131723131634686e+01,     7.2117634006164853e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5961753949239579e+01,     2.5302862520880248e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.7526786643362406e+00,     1.6257802924157537e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3533758531292435e+01,     5.9649258619368029e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5747574945366249e+00,     8.3336671316275952e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6723861759978877e+00,     8.8172793878453231e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3291293084447434e+01,     1.4360438125281915e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4465129997975541e+01,     1.5507861845127536e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9304248267375545e+01,     9.4170841615925092e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9474609773299683e+01,    -8.0247127037999107e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3359752673882799e+01,     3.3926152603137270e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.5973368420525702e-01,    -9.9615025602023525e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1149171110498763e+01,     1.6655433965568591e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.4211679912750814e+01,     8.3478696186472316e+03,     3.9957403058506603e-04,     5.4269558649720345e-07,    -2.5674762924198806e-09, } , 
      {    1.2217378968682970e+01,     7.3537635358119396e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.2011255625454080e+01,     2.3576640522531728e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -1.0941414517055001e+01,    -8.3532993236802286e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    3.5278197474154389e+01,    -8.0447029437074925e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -9.1065488221963605e+01,     1.1174252039900599e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.1015725253479580e+00,     1.2150680678965124e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.4302872299756515e+01,     8.3681540106649627e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2454785689107091e+01,     2.4991005306331299e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9307736994709712e+00,    -9.4613536679787449e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8437406671012393e+01,     1.5606744014117514e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0802920861554398e+01,     1.8458194387975029e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0671505764869352e+01,    -2.6256751690332831e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.0289193028705306e+01,    -1.1195595914222140e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0544405887220087e+01,     2.2930318205071104e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.7615254884477196e+01,     1.8522628737787694e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.1389492877572955e+01,    -8.3329491804851168e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -9.3452995346265872e+00,     4.8690549585003628e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6304992543354690e+01,     6.5236221304403552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1468485955355121e+01,     1.5542613502910946e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0516447466714816e+01,     1.5558132170733443e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1682119808340069e+01,     2.2428230535762472e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    5.6857362143433745e+00,     2.2483848541262159e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6085677079043066e+00,     1.5542895077456969e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6999389900144905e+01,     7.2529113649090668e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.3649325695948438e+00,     4.6052725513988698e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7908219851741940e+00,     1.6018765983149222e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1955048526046319e+00,     3.0377571191526549e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.8893788602763912e+01,     2.0455463204305383e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8990191370880076e+01,    -8.1294874367152679e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.9180612256576723e+01,    -1.7430341958856757e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2836246789625651e+01,     2.0856146656990517e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -1.2792637223331107e+01,    -1.3262555678807157e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9175939637803321e+01,     3.2243498508722078e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7974428132010081e+02,     7.2000483956494409e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.5737833787120508e+02,     7.2280773352682245e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -5.5698854090275347e+00,     1.0904054914880233e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8415562868946459e+01,     2.9448510630478460e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.6338509516770817e+01,     7.2876591948597088e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0024263833886327e+01,    -8.1174571930904995e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0789775280653700e+01,     9.4617363741181744e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1445617552884199e+01,     2.3687974211046818e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -4.2660363285199161e+01,     1.6266233707313755e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1969369496783548e+02,     1.4912411188121339e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4356236410034548e+01,     2.1899781910844416e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -4.1647266188438486e+01,     1.5445851489621280e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.9643962904190282e+01,     1.5225966043479088e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9704340555542026e+00,     9.9559140521669678e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.1510234324168707e+01,     2.9850223318254684e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.2934157759318843e+01,     8.3290047978335242e+03,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -4.0786859557412434e+01,    -8.3283780516167280e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {   -2.8496415376743442e+00,     8.3320885055328345e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5030516434381994e+01,    -6.7102068801278328e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.6074041871700562e+00,    -1.0259847126889832e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.1485858472274828e+01,     1.5530548365364373e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -2.3344288740172125e+01,     9.6754113549531976e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.3602253054095247e+01,     2.5021411345110911e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4242506646982495e+01,    -1.3028481171191631e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0533396852271920e+01,     9.4631590650519247e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3358214962805228e+01,    -1.3221004823133144e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.2740408877363215e+01,     6.6759267797477933e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.8284842772000857e+01,     1.0156725435153141e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.7882512838195723e+01,     1.5845637130141202e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.5013163672728878e+00,     8.3298024422785820e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.9317998763588813e+00,    -8.3275804071716702e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.9508923867341252e+01,     1.1548128185270240e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.2974734350375297e+00,     3.8941270695871754e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.3935091849988268e+01,    -8.4706932949179663e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.1042955149938578e+01,     3.0282399359442294e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.7905208506406979e+01,     8.6326701456503433e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -7.7497603281289429e+00,     4.5742497661230082e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.6806340518254501e+00,     1.3310822237205363e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.4622699602457843e+00,     7.2086331604258357e+03,    -3.7514727503877934e-04,    -2.9908155987647273e-07,     1.4060566379538760e-09, } , 
      {    8.3591521246609108e+00,     6.5159254365079141e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0472255667810419e+01,     1.7233432682751478e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2110546616531163e+01,     7.1791704204024118e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7567318632237672e+01,     3.1714168481371533e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.2349714828424554e+01,     7.4247568412243363e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2122442713561956e+01,     1.4270139611768387e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0201225974913360e+01,     8.3855736081626856e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4535516993015928e+01,     1.7802664331026252e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.9547536293597773e+01,     1.4060841910746512e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.3109125657447505e+01,     1.2396944892823053e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.6847409772311565e+01,     2.9831194610116567e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.4111117072152119e+01,     2.9026485138402422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6876398027759226e+01,     2.3387754573004520e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1830726464309578e+00,     7.5930060827757425e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.4747294891467405e+01,     3.1884668430953348e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5852987531116399e+01,     9.3162385519606923e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.7672412566911921e+00,     7.0912639243422664e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.7601057340366921e+01,     1.2974167019815444e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9760490004505439e+01,     2.3471865789616368e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.0256752557682756e+00,    -9.4601807306856026e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5189065930197856e+01,     2.3888306461603264e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.7004993678920201e+01,     2.9382387337953991e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -6.9709103764503965e+01,    -1.6657588564677462e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -8.4803312759570471e+00,     1.6283506704080612e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.1683524600664956e+00,    -1.1506640609771988e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.9585254930798161e+01,     2.0685808323695761e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.2878580590994932e+01,    -8.2663346176961568e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -4.6925900097565698e-01,     5.8334947685302786e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1659878985891924e+01,     1.3114843581913192e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1417492562449027e+01,     7.7153042778416948e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5726015216189721e+01,     2.0227587307136131e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.8552749711200093e+01,     1.9168882221153995e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0376591315372696e+00,     7.3875449633149037e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4546100835486378e+01,     5.7985618476420314e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.3105594166190713e+00,    -1.3455079330748670e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.3098723569844886e+01,    -1.8330409044488215e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.6937422539976389e+01,     2.2128180938120797e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.7258431691555572e+01,     8.2655700516654106e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0152028273930398e+00,     1.9470662787274097e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6106658704700347e+01,     4.9487573305238348e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -1.3453603961983703e+01,    -1.1341123129878883e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.4097851294380643e+01,     9.3308751551403093e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2212727763847198e+01,     1.5973718334218367e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.7301776783390221e+00,     6.0983307788001157e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6642199450668729e+00,    -1.1169335447494475e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1394909832897760e+01,     1.9742032029969352e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4064902471382892e+01,     1.6415537278497970e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.3614080070323258e+01,     6.6252178663531276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4137568580249130e+01,     7.6715181849644896e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.0166842182213854e+01,     9.2901055683510422e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6801740357616017e+00,    -1.0169843346352596e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6147153306998948e+01,     1.6703740547108147e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3343834160307740e+01,     4.1104167052193690e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    6.6850235977751780e+01,     5.5680295217096764e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.3043421116008339e+01,     1.9402796853229720e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5753129800690022e+00,     7.8347438668781051e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0741088934844718e+01,     1.6628532454103206e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.0517891722881764e+01,     1.6557523889322758e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -5.1152901734778098e+01,     1.6226418805415842e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0154440903852304e+01,     1.5291719931315763e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.6208175045565930e+01,     1.5474062184611295e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2547206738884995e+00,    -1.1221067258353503e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1585449906387929e+01,     1.4412695877026230e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7600050621899133e+00,     2.4670605565494516e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.5266573834351036e+01,     9.5902376269721317e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8123836127624688e+00,     1.2305656391078453e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0808775488253417e+01,     2.5455474567587346e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -4.4470530774664319e+00,     2.3237104655468593e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.8424209315451385e+00,     1.7221700710562240e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.5151406374034664e+01,     1.9837004708097455e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.8687109599479921e+01,     1.5332852508395163e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.1676082375845123e+01,     1.5035103407985356e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.4724954254796529e+01,     1.9144377367233516e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.0855964450203262e+00,     7.2102963799633308e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    5.8674811215062803e+01,     1.5333557140311312e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -9.8676909142767812e+01,     1.4916482222155491e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5016939749179198e+01,     2.5200690811953369e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.1665560262530299e+00,    -3.5503066384363419e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1592094497777408e+01,     8.3283215617589703e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4045236108582571e+01,     3.6130346836564087e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.9819838508528967e+01,     1.4896005856039083e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6155351944002469e+01,     2.4910823213002986e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2618180777734615e+01,    -9.0670172211579484e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.0604228300628343e+01,     7.7936685848392990e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.5431141409830495e+01,     7.5111692576759215e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1194017863684994e+01,     1.8888847022767688e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2102808457415058e+01,     2.3871435192705278e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9685337823210304e+01,     2.3871456237112892e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.8732042099648240e+01,    -8.3593596064028789e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -9.5575098705993582e+01,     8.8929112342632743e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5297111626983643e+01,    -8.2354066795654126e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -2.9164112248198791e+01,    -8.3771428250064673e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.4363143181167040e+01,     1.7286019066972243e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.6677589387093199e+01,    -1.1971118524343347e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.4433820493336746e+00,     1.6659331733331914e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6978682254285388e+01,     9.8009163433695849e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    7.8497137615597111e+01,     7.2163623303011800e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3652511984381547e+01,     2.3871594131966300e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1563657522197705e+01,     1.6167248583416365e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1278570733802768e+01,    -7.7158029507187721e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.1362811906239010e+00,     2.3871297297851870e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.6304501634954903e+01,     1.9797189806199538e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5028902576829921e+00,     1.3720188342076297e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.2158051375295189e+00,     8.7354519333925709e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3809980313497569e+01,     1.6621300222107551e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    1.7560133157194578e+01,     1.6577183790174789e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8802617990503954e+01,     2.8423404041284775e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.7521226072243607e+01,     2.3225865133096686e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.3780845127799694e+01,     8.3305632522816177e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9436887186616364e+01,     4.0006459173736184e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.7873063779809083e+01,     7.0052247450949881e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.0870392554607221e+01,     8.4549455514023302e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.9331920833043732e+00,    -9.2080333686388640e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -9.7826571902819268e+01,     9.2893292847639714e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6844977163078209e+01,     1.6538904546204194e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.1190485740510915e+01,     8.7528574283338385e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.5129812068052173e+00,     2.8385933410258446e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6930434821222038e+00,     5.8008673267482154e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    5.9120205894045625e+01,     2.2756458126133988e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.0642836448388908e-01,    -1.1067218974253659e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.4826677447765508e+00,     6.9984799252331632e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9011754159887605e+01,     2.5005261943883605e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    6.3664720796195828e+00,     2.1940122293947112e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.9748086759013194e+02,    -8.9575069683087077e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.8743457312657267e+01,     1.0766999336211358e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0249171117045179e+01,     8.9192616506564573e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7368265521153172e+00,    -2.4966886604467152e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    4.0908508366701524e-01,    -5.7649247355525335e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    3.3220597957493809e+00,     7.0901004628352448e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.1176520287872648e+01,    -8.4369118674150050e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.2001209477208464e+01,     7.2954178960802828e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5418656646650813e+01,    -1.1071503926972366e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.3438940784462190e+00,     2.3859955727153436e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.8132055816349599e+01,     2.3882935702664734e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.5961556899839447e+01,     2.2757176185151595e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.1894133174152595e+00,     8.3289806290553515e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8048761617102445e+00,     8.3284022203949007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3804520872594232e+01,     8.3290332184793915e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2810226420910844e+01,     8.3283496309708607e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4995505847911502e+01,     2.8429962996884504e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.9777492830426651e-01,    -8.2588615808197519e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    2.6669236433981105e+01,     1.3220711099368731e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.9446263207634118e+01,     8.3775417204827390e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1788835554825706e+01,     1.5462900829332100e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.9464034802845042e+01,     1.3799824859422846e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.4869959253992512e+01,     3.8865522223843327e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.8666666402998572e+01,     8.6038292260894177e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7106672216394930e+01,     2.0724826599098742e+02,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.5908686360274707e+01,     3.1731865108768834e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.3736425356283128e+01,     2.0732208989521103e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4769175048475027e+00,     2.2773026490984699e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0064079732488839e+01,    -4.3570120941258274e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8939362312004349e+01,    -5.4535917100367469e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.5582591839654910e+01,     7.7774106194869273e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.0117074301718745e+00,     2.3400768748025512e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1818994942121641e+00,     5.9068055211705987e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.5906992406151918e+01,     1.5573407212493663e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    1.5564457399591864e+00,     7.9608487537504334e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.3549793031466905e+01,     1.4731008570874907e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.9101876429957144e+01,    -2.0741373371402915e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4134074238701146e+01,     1.4413407222493106e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8753239256746696e+01,     6.3884117394021541e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2781793904735149e+01,     2.5022834036044664e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2904525314709449e+01,     2.2135068526196974e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.2837003998047834e+01,     3.8821940836449903e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -1.6354791231689006e+01,     7.0540273112786654e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5730592404916216e+01,     2.3464685206241640e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8193512753626102e+01,     1.6146478711428566e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1954464363990118e+01,     1.6090879781901196e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1564059949415483e+01,     4.6052924382717523e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.3403706499741173e+01,     2.1623060860782869e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.6290716469956507e+01,     8.3282339823377588e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.4721199979042495e+01,    -8.3291488671124935e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    9.0901223455272149e+00,     1.5359282786321692e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -1.8371958930613516e+01,     1.8826215552968952e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1646316346967481e+01,     1.1617475625410001e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7376037027495027e+01,    -8.3441894146001541e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.4978945424094185e+01,     1.0324449195499768e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.1309350652941667e+01,     1.0110324769327799e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.0101622710743570e-01,     7.7106931209350705e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9950012161968921e+01,     1.1916821474013414e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.8949719883363453e+01,     6.6081897801204595e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.1680840516871918e+00,     3.1261188141885268e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8179390097259390e+01,     7.6262620290739251e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    6.6745784224781062e+01,     1.4293138996935499e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.3635906090880283e+01,     1.6610483510747832e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.0830581792553854e+01,    -4.5098154163103391e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.3956868537005640e+00,     7.0038020541612341e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.4426079238276774e+01,     2.0556425691825301e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8837714317972853e-01,     1.6655960158516500e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5131662761919898e+01,     1.5616048795436087e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.5896354271615078e+01,     1.6658805540384004e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.1420643964094523e+01,     2.2701460225456452e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.6307760216502402e+01,     2.4153132996086981e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {   -1.8970221331639738e+00,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.2951653310840214e+01,     7.0045201131788381e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.1920071673814061e+01,    -2.2069170498184612e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -4.7716415953476847e+00,     2.9826592210437098e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    8.5423988945624973e+00,    -1.5589653628886379e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.3032467025976819e+01,     9.7702634210598808e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.8228196562372105e+01,     7.0045066860773841e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.8994151404756359e+01,     2.0911709302404073e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.7134448113132350e+01,     6.5105042192409019e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.8950696134262309e+01,     1.6240925930075166e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.4857224279427687e+01,    -6.3694408342897123e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.6113124741834977e+00,     5.9923925843777324e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4358681339282013e+01,     1.5624109320805406e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1951746515180139e+01,    -1.6880050226480431e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.0366994727493625e+02,     8.3293469276941141e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.0210043078402225e+02,    -8.3280359217561399e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.0869793782648589e+00,     1.6007745356426858e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4263189880137638e+01,     3.9491752266105163e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.2307877798573394e+01,     8.1770173531073442e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -1.7415777561792815e+01,    -2.5333555870792952e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9940272179983332e+01,     8.8037728765739685e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.1509788670897343e+01,    -7.8536099728762838e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.5785565381039817e-01,     2.2575015642920087e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.0797539680253585e+01,     1.6574899556282209e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.9954359154414938e+01,     8.2776370004553028e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    8.8370373876891399e+00,     6.7123120373466472e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5595062650886966e+00,    -8.7278298831401071e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9417502984414368e+00,     2.3413990395403431e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.7021596227104624e+01,     2.8678216434608126e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.6409522199253395e+01,     3.1491647461260360e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.3855712976741678e+01,     1.0488742505439928e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9858272808527488e+01,     5.9650742789940168e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7193149423337871e+01,     1.5542944746043117e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -4.0116096829938300e+01,     1.5542563834324801e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.6270121303613642e+01,     8.0619245817093433e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.2049931505059917e+00,     8.8334287890726300e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1372053937996391e+01,     1.4718508424558383e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.6964764824524295e+01,    -7.9587275331634828e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6184924860334409e+00,     8.8834026803697925e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.2342093120601536e+00,    -7.7739801690804597e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.1279187798671977e+00,     9.4896776816493184e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4491041263460659e+01,     1.4941107411723049e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8515784518446088e+00,     3.6346209898958637e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2836905729256674e+01,     2.3384413395221458e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0500369858943932e+01,     1.6597864272425453e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.0292950453581490e+01,     3.0910706944487836e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.7106087158929206e+01,     2.2851988987727047e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3810344418332310e+01,    -1.6362866044225775e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3007225481396974e+01,     1.6661196141263386e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.7526732052084839e+01,     1.1597123695284063e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6980148239297410e+00,     1.6653569557637118e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.5445877012520739e+01,    -9.0959074861470708e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.9094301633250581e+01,     3.3208347783173282e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9731720141396167e+01,     2.4230685257099107e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.4219713532377707e+01,     4.6956849490432192e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    1.0615324981545706e+01,     4.5892730430743832e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1478313090921343e+01,    -1.1668863110106083e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.0464103720743417e+01,     8.3634392546757699e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.9345151005582984e+01,     1.7324387634786628e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.5689074613848813e+01,     1.0560155598042560e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9887771162153447e+00,     8.8702579608426349e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.5614253015911530e+01,    -9.0952028542309181e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -4.0933685780063502e+00,     6.5703361693740999e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.5364297455868805e+01,    -7.4075241389665348e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5745882627258169e+01,     7.1434218760045105e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.8856871974419029e+01,     8.3140214519467499e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2764721630558970e+01,     9.3741415514383225e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3543079948394325e+00,     6.7746412299405019e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.5413279963267506e+01,     1.7380862266580050e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0555968387661038e+01,    -7.7135017508790379e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0493915996616142e+01,     1.1155089273748426e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7404428957559084e+01,     2.9917910923890428e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    7.5046865408164535e+01,     7.7160202149056377e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2833708698313451e+01,    -1.6409476995775316e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.2060897098195305e+01,     2.2836377621196098e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2279559646543667e+00,     7.7041866092889095e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3796318177440597e+01,     1.4357484741463342e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.1966360636659861e+01,     7.8402929077206754e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5329497166714152e+01,     1.3210495199037553e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5251812139196195e+01,     1.5456003765469801e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    5.2758985096566661e+01,     1.5539641044358745e+04,     2.4509445367736679e-04,     3.6118134428979317e-07,    -1.7100832973776660e-09, } , 
      {   -2.5681932503167090e+01,     1.5545867536009169e+04,    -3.7747069612412869e-04,    -2.3328749521309762e-07,     1.0926245931165610e-09, } , 
      {    2.2515510397735092e+01,     1.4271545362298135e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.8464635232406714e+01,     1.6853814404081517e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -6.6412916397656033e+01,     2.3870741082992932e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.4606287434234169e+01,     8.4557575973878065e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.1250948295021331e+01,     1.3666691690486790e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.7442699488956190e+00,     6.6277037509021166e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1891878852755413e+00,     6.3040679992353907e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.5486393266787255e+01,    -5.1749937773523448e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6507386686734691e+01,     8.9084391397285490e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1631159325260107e+01,     9.1642472757616561e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.9491937002604637e+01,     1.1708145797621582e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6404912456233166e+01,     1.3404182603230998e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.0200421097636760e+01,     7.3133896603279227e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    5.6355494048782553e+01,     2.6940362411280596e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1952755323175893e+01,     1.5489794668758888e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -6.7724221764093215e+01,     1.7301232462784224e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.7731308261631071e+01,     6.3485968375042421e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2612059066256364e+01,     1.5577646735240380e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2264884046782935e+01,     1.5136632375238671e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6768230899974963e+00,     9.5433581243273857e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1351135463606131e+01,     3.5835893930622315e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4310441542425975e+01,     2.3323320223191848e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6658302400613152e+01,     7.8881494885547163e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.1568730805027512e-01,     7.3191885645388209e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.5997925271506672e+01,     6.1343791952025231e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    5.0774985275386356e+01,     2.4358119005087909e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.1614111091764677e+01,     3.0023849118312814e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    5.1788190669050046e+01,     2.3860629003549366e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.1606486748166631e+00,     8.5493414368980648e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.2013350472910256e+01,    -8.1080414125521875e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0762783927969615e+01,     1.1020319635551239e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.3047121387769618e+00,     7.9225695097798362e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.9355876007026183e+00,     2.4714369837732622e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.9623050943314567e+01,     1.0857463793195728e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1059124895150063e+01,     6.4652233281961699e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -3.8482872597323841e+01,     2.6482170741643395e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0628444774788207e+01,     7.2508226273281198e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.0255019958831223e+01,     7.2489553105152536e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2115895487611716e-02,    -8.6965340956998189e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.3288067585411810e+01,     1.5389950967999444e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.2720663930298235e-01,    -1.6692327738460237e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.3914406695127058e+01,    -8.3264244358256819e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {   -7.8470513685771252e+01,     2.0171065951569180e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3360571269590132e+01,     1.6739866142618295e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2277447574991093e+01,    -1.0073928768876696e+04,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.2501319730327808e+01,     8.3282620524418835e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.4070836221241819e+01,    -8.3291207970083688e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.7488751903203585e+00,     2.5342677422778164e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.6181492598215726e+01,     1.4914834846931952e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1163803575718068e+01,     1.6011443238129052e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4468237648475093e+01,     9.2106479415869999e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.8102279094609681e-01,    -2.6271938610134448e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.8863284615285146e+01,    -7.6846513555319907e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.5979491359092346e+01,     2.3123693424169804e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.9663988527384475e+01,     8.3159258294837473e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2894100522660274e+01,     5.3571784596837715e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4870441976050678e+01,     1.2385468621440605e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.8196604167759318e+01,     2.3872150346825238e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.7586879363757816e+01,     7.5411242320879337e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8979317937989578e+01,     7.7710181155831779e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6604331072459566e+01,     9.9471759693224194e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.3696175672497688e+00,     1.3548309474067196e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.0368109353200758e+01,     1.9959284961334582e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2643316808232434e+00,    -2.2341728913204579e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.9427823029779567e+01,     2.4251348773402617e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.6959314789817634e+00,     1.7381250408373584e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.7232822678111432e+00,     8.1814987659233739e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -3.2301141669165119e+01,     7.7717361746007800e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.4389925979796843e+00,     1.9077641817581112e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.2390483052213156e+01,    -7.4775229073927994e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0429975146115716e+01,     7.5022757326376995e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.8798319631407281e+01,     7.7266922303061401e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7719345637371759e+00,     7.6584353691907554e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6232321816200944e+01,     1.8700039958927315e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6379265485173001e+01,     2.6938061211440865e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6460590314307037e+01,     1.6628517194735155e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -6.0944234371876775e+01,     4.9158988258495083e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.8176793248450025e+01,     8.4948565622217775e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.2862344441653924e+01,     1.9916634511893280e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1199837676751881e+02,     7.2153324549174240e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.8963243331862301e+02,     7.2127932760002413e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2460960824176048e+01,     1.6175992907385425e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3001900739502563e+00,    -7.1561874712458912e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.2261230200920224e+01,     4.0581797661570534e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.3217295035878138e+01,     2.3858994195383970e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    5.9937885138296352e+00,     7.9794412885412985e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4961028361627431e+01,     1.5458146003739428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2984537347813490e+00,     1.0128294547950349e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.7768100333043918e+01,     1.0840190796428871e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.6247525783295345e+01,     1.1525586280139183e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5691788414890878e+01,     1.5592417687618501e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.9068864900551148e+01,     1.0810770169720518e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9269873735570712e+01,     1.1386118133340991e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.8344273658016299e+01,     1.1373326549390162e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0523882669284511e+01,     1.1757659709272137e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.6846521699770740e+00,     1.5078740015078574e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1653747231378659e+00,     7.8269405966129407e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9293124447018862e+01,     7.7398464256194211e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.7110405208221060e+00,     1.7898578908502150e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1320581964936849e+01,     7.1868329033945638e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.3749075636665395e+01,     6.6259224982692767e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9933562990171110e+00,    -8.6750524714155702e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.4425257107520252e+01,     2.3845137324282528e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2158258810367117e+00,     7.2063479385131150e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4558607137414185e+01,     2.2784047117707061e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9835533030213153e+00,     9.0929521462005141e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.5550105530816609e+01,     8.0645391546574756e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.2167761841362157e+01,    -8.1824470581403439e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.8405684704900555e+01,     8.2622158162425803e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.3040652522184750e+00,     1.0633857621457981e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -9.7237263643869198e+00,     2.1385716869529597e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3619221069513353e+01,     7.2309236121530139e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {   -3.8726064914333982e+00,     1.5997908409849879e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.1724873225539618e+01,     2.6621045390831423e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.4009404999220664e+01,     7.2181816669124100e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3397340993400807e+01,     2.2112510062272762e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0523052620019158e-01,     4.3051817232674894e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2064515110680532e+01,     8.3268084158822312e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8725668269243496e+01,     4.0475859467148148e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.4252310518894573e+01,     2.3973617423835967e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4230376098714260e+01,    -1.7354579777427798e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9477076426536286e+01,     1.3696206874463784e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.6401708692348102e+01,     1.6631074458823696e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.0450740288105255e+01,     9.1124321642713658e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4428493695703654e+01,     8.3090145579259406e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -8.1110678575186608e+00,     2.2743310644371439e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.3263129641481626e+01,     7.7228183393867530e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6560413265937370e+00,     9.0076411839678167e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.6385442766740159e+01,     3.1692486257435608e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    6.6744954747902874e+00,    -1.1642919567008380e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -5.7520352341178711e+00,     5.4470327544370193e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5843753660666852e+01,     2.3828785898953382e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0973362114933058e+01,     1.5855593166548639e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9749550004175269e+00,     6.8805604727906430e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.0415156035834119e+01,     3.2306075334056593e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -6.7384886512766471e+01,     1.1485771378241270e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2886840717880350e+00,    -2.2679543188234220e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0702046208902185e+01,     3.3314054353433632e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    5.4453197304863332e+00,     7.9628929269353748e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1594299548897627e+01,     1.4889863541337481e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3030616815796899e+01,    -7.5300302470167717e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.4425618745623945e+01,     3.3315477044367377e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    9.8732477112301744e+00,     1.6149020621784774e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5707066590763805e+00,     2.3883897234434196e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.7966447803092933e+01,     3.9940335881211708e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.0251167501110569e+01,     1.0937481738785089e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.2114724309271487e+00,     8.3313059976732584e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.3217015620281268e+01,    -8.2912203173889629e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.9126415023096101e+01,     1.4210888058920484e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2478992689093964e+01,    -9.7375663255652344e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0659255476874419e+01,     1.1157320866587174e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.4473444468313357e+00,     1.2132440086685065e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7006080170080899e+00,     2.2382941010273149e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.5315576111844464e+01,     1.4914058563344881e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7793383652762245e+01,     2.3839806525675744e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.7735204964527362e+01,    -8.3647740520678235e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.0737908226750889e+01,     1.6177095349624447e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -2.2329919009695573e+01,     8.9085476094893293e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.9974236307692252e+01,     7.9248537436724164e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3181902718443396e+01,     1.5842475255349185e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7505106797972445e+01,     1.8485416859878267e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.0258798810645416e+01,     1.5135993781516514e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1628590499181140e+01,     1.0021837304151832e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0695872102430613e+01,     1.6683691240076809e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6781081623733900e+01,     2.3829497244420258e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1491147198477067e+01,     2.5613670513754045e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.2030429006847790e+01,     1.4996929998306457e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7212509381775369e+01,    -8.5044747224209332e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.9912750778842128e+00,     7.2114598414703523e+03,    -3.7298044877242842e-04,    -3.6797843209894893e-07,     1.7350027205866961e-09, } , 
      {    1.1966795970776751e+01,     1.4366321213421179e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.2514543411495360e+00,     4.3770833509159729e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1530794098475768e+01,     7.0021864766140670e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4373658224844561e+01,     2.4659788854134793e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2300637519739265e+01,    -8.7577950217608104e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1069625533826095e+01,     2.4552722397165820e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2598988845502413e+01,    -8.3338801333446663e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9037924730578194e+01,     1.8915649707480683e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.6210476940980040e+01,     1.3360411769721841e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9822870015171596e+01,     6.7500819599522074e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1612072457824356e+01,     1.4601607828773735e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4955883871921305e+00,     8.8537095992367122e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.1760731345877337e+01,     3.4886834119484905e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.6495450760663065e+01,     1.5545031806542067e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.5570017753027173e+01,     9.9176368698095139e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0401529568578775e+01,     1.0454501267131973e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5436782279205605e+00,    -8.3631841976949672e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.0991044885405532e+01,     6.3758534986244931e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.6320282589897609e+01,     7.9406014871880579e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8060946024467214e+01,     1.3630083581939312e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    7.4475008959829026e+01,     5.9020907651814041e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5332343512468309e+00,     7.1991065323207195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8563030586959982e+01,     1.5554960215003544e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.1129119957494922e+01,     1.4547570436611402e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.5807298760220307e+00,     4.1566361568429684e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.0043023404625814e+01,     1.0523402549724000e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3893591633326317e+00,    -1.6985969469330572e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    4.2313011721317451e-01,    -1.8668890711493043e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.3880279516411672e+01,     1.5429853891434718e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7139310281941750e+01,     1.4602927366797745e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.9171089633248357e+00,     3.0966063874674179e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.4207176258028433e+01,     3.1357213016810674e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.1580163216179542e+01,     2.2440028908937922e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0114491863726425e+00,    -9.6951344324824072e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.0639979095723390e+01,     2.1564191709485262e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0625564956741115e+01,     6.5983745187356681e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    5.7223132622399298e+01,     8.3263835113346668e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.7884936683885581e+01,    -8.2937807466554277e+03,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    2.0603670609659524e+01,     3.6173708205651965e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5790309487520432e+01,    -8.1114537527279444e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.2902415593095196e+00,    -1.3156276749556064e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.4101916561468810e+00,     2.1278033281398606e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6751975651520141e+01,     1.5753457741735583e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2553583811351922e+01,     7.1294545790143020e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0449293920279535e+01,     6.6600542856023321e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1437318157026382e+01,     6.6014877645608776e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    2.2745054444869069e+01,     9.3626392177860362e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -9.6896754959761466e+00,     2.3457754667641530e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.5435746783069444e+00,    -2.9568914984472190e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5982399586337603e+00,     8.3886641034151580e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.8245617759703080e+01,     2.9829245726234905e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1856903658209788e+01,     1.9178193922106630e+01,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -2.4712011974882849e+01,     7.2248795768185537e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2690142089852792e+01,     2.9412462214076051e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    7.6461435116423750e+00,     9.3844508326938121e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4939508283048106e+01,     1.4902956717382769e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0171081456209247e+01,     7.1437809055133130e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5618608850860724e+01,     1.6166130843073584e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7308445828379206e+01,     8.3485904399002284e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.4557440302856115e+00,    -8.3087924095500239e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2964099360678846e+01,    -1.9807473744312358e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.3099540049564030e+01,     1.4561293887404318e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5708577043518837e+01,     6.7457908345934557e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.6619935756622493e+00,     8.3109095055466460e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -5.3383772669588403e-01,    -3.0202148307064322e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5553529644516528e+01,     2.9748711316948524e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6789370136548868e+01,     1.3109153753797611e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8990225913560266e+01,     1.5781643701571433e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4555053843954873e+01,     1.4166694414399135e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.0921958092020475e+01,     1.7285697148046522e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.0634643942551779e+01,     2.4428407678584161e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.1310721505939103e+01,     8.2838094508799441e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -6.0741422908796068e+00,     1.1306280675907790e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8232303812986842e+01,     9.0133223424046901e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4884819189902750e+01,     2.2050512683706009e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5911961052529918e+01,     1.8260119650531906e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.0510934942654480e+01,     3.3601368752866270e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.6736895041001674e+01,     2.4429112310500306e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.3524840382228227e+01,     2.4817523208475228e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1423255829340606e+00,     8.9040393883455981e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.3772075455660001e+01,    -8.3936244474148698e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.1272284514812185e+01,     2.2770323666914144e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.0480052005074711e+01,     9.0065129214212247e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.1447425597954265e+01,     5.4221114823735170e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7465989485870903e+00,     7.2986711519033634e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2750248436671001e+01,     1.9517154607813231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8351900321638524e+01,     3.1155398358387891e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.8630421081490685e+00,     1.8684620847888124e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5414125156801372e+01,     1.1392677088940718e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3842313395228857e+01,    -1.0071275253078888e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1788408427732310e+01,     1.4534063925340048e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6435815973809010e+01,     6.6747727940268915e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.9037949871440077e+01,     1.4960847370963756e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.1888329621575057e+01,     4.7742539113860090e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    3.9403093598322080e+01,     8.3951670332076719e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0029065908102734e+01,     8.3636021027948245e+03,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.8431592415056879e+00,     8.3275374390042252e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.0315648245819709e+01,     1.5065069814346634e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.2820841057002720e+01,     2.0135052999662341e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0919042289031759e+02,     7.8424685020782154e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.7206832058033488e+00,     1.5561116606647731e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.2797735799202925e+01,     1.5524391973720183e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9496064452759146e+01,     4.0371094210018906e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.7265879456631261e+01,     1.1754546463446923e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.8565744027631226e+00,    -3.9076100034295973e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.8518127519448818e+01,     2.5919033856542072e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8084611768981770e+01,    -1.1095614873190241e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0758864270574707e+01,     4.0489013662461424e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.1729251151571436e+00,     9.3880058682393815e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.5915826235882840e+00,     2.7330978444955381e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0496034616294403e-01,     3.7570562315502530e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.2904379347785735e+00,     2.5636923764351977e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3374598139497500e+01,     7.2497190999356953e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.0469245925317278e+01,     5.8875836405220834e+03,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.4703691155219175e+01,     6.9913954884286559e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.3326021124176920e+01,     9.7717592624039230e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1014833682170764e+02,     1.5272780822219625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5454034142097186e+01,     8.5662792399174159e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9035134450850563e+01,     2.3482027541251016e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.9181493802810493e+01,     4.7743243745776250e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {   -2.2044104059859258e+01,     7.4006476151463103e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1653010483300920e+02,     2.2128607622171090e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.4599222332945899e+01,     2.4148419059716041e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9180385363943497e+01,     1.7205491375220336e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.8590449716788076e+00,     4.8716847908165464e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1374766869023320e+01,     2.2772564899158431e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4032758842472948e+00,    -2.2161392073331317e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8472675155443874e+01,     5.4968638706688307e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.4819680899320758e+01,     9.5787476392164845e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4598437574892998e+01,     1.7031258994819895e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.6272848113409566e+00,     3.3274950797002588e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.7853941341634805e+01,     1.6171773220696377e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0042104421516598e+00,    -1.6638204655528145e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -8.6257003885129038e+00,     2.5048207424148313e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.8686684129426641e+01,     9.4608549951016666e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -8.9261109890357115e-01,     2.3872432376818651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5077710997605639e+01,     7.8637003584822278e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1133079000651396e+00,     8.2941986517552850e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.1188812615191427e+01,     1.0278048529674425e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.7649238769465384e+01,     9.6355964530552847e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.4993809657926320e+00,     8.3804970604292957e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    7.1507266677634123e+00,    -8.2565346578057088e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.6772672177318277e+00,    -1.5119080398181268e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.2669524086180338e+01,     1.1964095929277644e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.9426757095519818e+01,     1.6673891280186363e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.1691918905712406e+01,     1.1016723400787378e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.6010920645909366e+00,     7.2140270396233709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3346250552198484e+01,    -8.5018448901047486e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2514935720094485e+01,     2.3620411356040891e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.5654228897033072e+01,     6.5700227962657009e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1342107715937754e+01,    -7.7550024384729636e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.1100768420796197e+02,     8.3284644526978500e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {   -1.1257720069887597e+02,    -8.3289183967523986e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    2.4967074644910571e+01,     7.2140986912942944e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6054511941181943e+01,     6.4990142314852519e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -8.5913044747628220e+01,    -2.4272873609482303e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7751209134204906e+01,     1.7043844863543512e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.2561129033686960e+01,     5.4004128286629066e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.1748982233081662e+01,     2.2685471534272321e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.1988137081038897e+00,     6.7228261184502117e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.3321142033374365e-01,     4.6868894794509271e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.9673537082127837e+00,     3.2414753677412205e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -4.1841321815407024e+00,    -3.4907880614115443e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.8057750157490180e+01,     3.0986181785498829e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -3.7094835561812731e+00,     1.2590102314483454e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4896478792191150e+01,     1.6318254534031255e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.2603104291925643e+01,     9.9188089874844009e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.7965840271503094e+01,     3.1614489370544372e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.6393888107738960e+01,     3.9085613385212724e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -1.0769800435104024e+01,     4.4760339888073759e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.2871638879492817e+00,     1.8291599878195135e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8492051280864757e+01,     1.4165607167875753e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    3.8687067232684633e+00,     1.4317290715279654e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1004616284707403e+01,     1.6122610474948160e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -7.4518105808098554e+01,     9.2667873796329150e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.0568718234446500e+01,     1.4715372195648855e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8865276547197816e+01,     1.5622607751035815e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1381020375848944e+01,     2.2340385959442014e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.5375431725513291e+00,     4.3903395234982645e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2951449897839423e+01,     1.8008115385247779e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.3788998272010311e+01,    -4.7550426286102993e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7321775758996409e-01,     8.9568505927134556e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7497032654557083e+00,    -2.7347058283469660e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1957065045823452e+01,    -8.1088239204117599e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.5213922390151238e+00,     1.5569984252248227e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.7776094144137793e+00,    -2.4242128265685710e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1916303432640085e+01,     1.0762068304055441e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.2084467891213055e+00,    -7.8918996215229781e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.3503762870458362e+00,     5.4298795313413598e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0856947418236960e+01,     7.2229412539115701e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -5.7677057280338051e+00,     5.3940731330119479e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.1819866082348902e+00,    -1.0623708075219783e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.7864977992448070e+00,     7.7746828448686256e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1642728468852747e+01,     6.0879595778049415e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6796766840315446e+01,     1.3192840468804430e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -6.4536742472419917e+01,     7.7557407698659244e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2834103367360310e+00,     7.2295608553338607e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3066567452174496e+01,     2.8259110731344619e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.8367463610457023e+01,     1.9073710389069744e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9007157674130980e+01,     1.6150443312718526e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1548980957667482e+01,     2.2130587113380949e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.5404989750136178e+00,    -7.7025260779492073e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.2714586550135554e+00,     6.6593429401354560e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5330114654440067e+01,    -1.0280824718909609e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.9687997432608384e+00,     5.3298514557890803e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8589509975445878e+01,     2.2774449181918451e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5014878948440623e+01,     8.4008481916445435e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6197848671889833e+01,     2.2159676425152080e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.9584503935226270e+00,    -1.8556899362381617e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    7.4366636136428843e-01,    -3.9433276081293302e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0141799523286643e+01,    -9.3636901801955955e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.7912415687051620e+01,     6.4406401643952368e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.3992396620189696e+01,     2.3903084904142426e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5931310342975877e+00,     7.9312632134881560e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.2497396756920946e+01,     3.3354580600798421e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    2.4254032452926968e+01,     8.1976540610831526e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8255239657001745e+01,     1.0296097473029797e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3426872042251290e+01,     7.8908218507857164e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.6391348940008532e+00,     2.3897754105535641e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4598541299970094e+01,     1.3799106800405247e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.4496969026420825e+01,     7.7578706338206248e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1365965369750672e+01,     1.5847031170943283e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.7182382623149408e+01,     7.3421440695359979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6911454540221484e+01,     1.6657128585066017e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1865016766495184e+01,     5.9349057902366903e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4000778294256257e+01,     3.6194781186376757e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3847914189193531e+01,     8.3553884154235820e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6551838629355032e+01,     7.4529522768463066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1683398747526468e+01,    -1.4323984677868939e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.1205989942309571e+01,     1.6657637113834488e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.7669110686426173e+00,     1.6970221725814932e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5934424043882643e+01,    -1.3218768252572809e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.7717032364583432e+00,     9.5220282163495340e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7329939898286852e+00,     1.5671081718694813e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5489851073113911e+01,     5.3287877943641288e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4320746198671905e+01,     8.4017622729212235e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6088088682859911e+00,     6.6518785729377596e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9227649372697854e+01,     1.5485872106746401e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0537821583827167e+01,     1.1859321196362287e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.8933541828058580e+00,     7.7172445863737657e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.2169611085607322e+01,     8.9100996710682484e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.9893397474515105e+01,     8.5765972784000624e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1962443421543572e+01,     1.5627362576628489e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7570980381234254e+01,     2.9968250188785438e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.3769995132066645e+01,    -7.1304503217240572e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.1341580212003308e+01,     8.2771450184476871e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3580049761248608e+01,     3.2119955046305906e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.7387149303756228e+01,    -9.6525408173456926e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.3241471773593638e+01,    -9.6531980822985897e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    3.0480410295762979e+01,    -1.0278875835027950e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    5.2923711445221446e+00,     1.0557594079686851e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.0372693220941578e+00,     6.9066439207451185e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8826150631889252e+01,     1.6598513698152641e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4708569794396738e-01,     8.3802378310025651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9315183976976577e+01,     2.6185601545601698e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.6230835526027036e+01,     8.3309993381155855e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0247950554430435e+01,    -2.5858602671353710e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0617651206057216e+02,     5.9575457469415860e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4547787452262192e+01,     1.6250622340782804e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.2875482570647932e+01,     1.6814911488563477e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.8061587495355212e+01,     7.7405510575355702e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9440514958991795e+00,     2.1547101324254123e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    5.5679211025346831e+00,     3.9334570201297218e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.9292133346775167e+01,    -9.9521828633432269e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.1754285153352306e+01,     1.7730027842801996e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.3119101708304814e+00,     2.3230518891541164e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.0213255705544165e+02,     8.0587179567607927e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5350309811420782e+01,    -7.8834313922570982e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.9150901503393492e+00,    -1.6396467337778790e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.7173800334794173e+01,     2.7344200092333303e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1778856279270874e+01,     1.7601159143176657e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4806933801372473e+01,    -9.6579365217694103e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -9.2524804378455894e+00,    -9.3984380101462375e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.7859056699093170e+00,     5.9110470295424029e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.6817894752226792e+00,     8.7348133396704143e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1685558491717686e+01,     7.6277539127263844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3874569870917171e+01,     3.3367734796111698e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    3.1671232366750253e+01,     2.9973509853417803e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -8.4379830883426656e+00,     1.9470559051095643e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4298110575434833e+01,     8.4749357913099102e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3687391151140487e+01,     1.3859336722896918e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -8.7921038810254046e+00,     7.3875297039468533e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -1.8743805496041347e+01,     8.3288500168009487e+03,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {   -2.6596507294134938e+01,    -8.3285328326493036e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -4.8264224152257434e+01,     2.2812174085829130e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.0477383986772118e+01,     4.0568643466257257e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -3.1158430440464905e+01,     8.7639579746760555e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -3.2727946931378909e+01,    -7.8934248747741967e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.2352823805545135e+01,     1.6236323530395697e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -1.0300618049281258e+02,     1.9463128562727812e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1730549524175856e+01,     8.2726185574748433e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.3877847726082265e+01,    -8.3847642919754089e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {   -4.4924678232357408e+01,    -8.5953287892206172e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1080449533500067e+01,     1.4785098284208243e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9256475547387168e+00,     1.4916588943408040e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.9668026134441071e+01,     4.8999070347662209e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.7970373819910872e+00,     8.1311421684867819e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5009226960890878e+01,     1.4308573619800041e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0839312777065597e+00,     2.3196737464038197e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.0383514505842140e+02,     8.5377329320178160e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.9889304057027707e+02,    -8.5387488490938795e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    2.0112217476893385e+01,     1.4743594439598528e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.2873854473702362e+01,     8.5946309623873840e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.7741667210305650e+01,     3.2815326813480293e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.3031541911647956e+01,     1.4423194698761745e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.9760318092648448e+00,     5.4369929860101229e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1132764891008499e+01,     1.4905586549698952e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.5391926330713488e+01,     8.8015658552699715e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1092413435876741e+01,     1.4181373862963632e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5054719461284382e+01,     1.8151544604243922e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0149476065718563e+01,     2.2728465433172823e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.4758561501095379e+01,     9.9760263646694657e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.3251540435014881e+00,     1.2146666996022591e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2549907434481189e+02,     8.3135017123947709e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9011575677849502e+00,     1.7295999715751284e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5007508362702591e+01,     2.3913394185397912e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3838111290324067e+00,     9.3727188605045721e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9689314807055069e+00,    -1.7378990439023557e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -7.2400665848251791e+00,     1.5054132116123474e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.6155889848024486e+01,     1.7492938700486782e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.2392656801166879e+01,     3.0521635345135041e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.0598812516797331e+01,     1.7024666669621405e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.6100437173826347e+01,     5.3763741283396394e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3509364290422756e+01,     3.1713463849455384e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.3100532191420839e+00,    -2.0326677034708664e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.0524138007548760e+01,    -1.6811163674473046e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.1450513935130189e+02,     8.3438811370554795e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4587687488009479e+01,     1.6179698373612930e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3113037603796977e+01,     6.7097176830368753e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0060867282688482e+01,     1.1513883211355801e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1385042599758211e+01,     8.3271574733769139e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2954559090672205e+01,    -8.3302253760733383e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.6592710767612161e+01,     1.7432608077476511e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1329074462417905e+00,     1.2210096417200662e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7117449677261618e+01,     5.8921860402015518e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {    3.4180907505936058e+00,     7.2025976128573548e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5146576679312588e+01,    -5.3296387831992779e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    8.3218640486508235e+00,    -8.3029522895782029e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.6634467997884883e+01,     3.0317515584337314e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0359608379893512e+01,    -5.0605707403437591e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.9083095125194134e+01,     3.3143913433360620e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.1948823968717497e+00,     8.3270530831501019e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5186035337814857e+01,     1.4915551296867734e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9489432897197538e+00,     1.4221704770280203e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.4752924683200547e+01,     3.0457207708873102e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9098695554941671e+01,     3.7724753425894432e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4903881033829069e+00,     1.3262180733038158e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2771599858461492e+01,     3.9766103627552184e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.3077300357997714e+00,    -1.0640416577057707e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.5385005208131197e+01,     8.2668716478605947e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.6870973181064549e+01,     7.1571806820212732e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3477388317073746e+00,     1.6709640601194569e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8019158964505465e+01,     2.2617460377134183e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.3426985392290520e+00,     1.6605125097705935e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0779074094135272e+01,     3.7777202055203590e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.6967768333914847e+01,     7.9068200596850911e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.5285310475924675e+01,     8.3824117333706636e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -3.8377278370300658e+00,     2.3940137820481745e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.7546300496276729e+01,     2.5197924939845838e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.0457096620475863e+01,     4.0921849534052330e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.1208696237508423e+01,     2.2728431222232532e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.4723669962862985e+01,    -7.7556446609443883e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    8.3613523248494950e+00,     1.5665613165415121e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3335032913766536e+01,     1.0369047334362453e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5421684007247610e+01,     3.0751447336896192e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.2206282831594571e+01,     2.4980174220088960e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9416787390847805e+00,    -3.4132358003994204e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3262526618663109e+01,    -7.7315354348663632e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.1234712596159140e+00,     5.5901638751965929e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.8391343564218946e+01,     2.2216423556818427e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.5824867942453853e+01,    -1.5514402567713989e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6614081004691577e+01,     2.3391683696288059e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.4306939345300819e+01,     2.4499424667478183e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7437231687316118e+01,    -8.4423263130799496e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2695633069585936e+01,     1.7408489375612360e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.1919137575206037e+01,     1.5852542660046205e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.3589065890686967e+01,     1.9996264315689106e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5172484236002433e+01,     2.1125229959214092e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2340617164696148e+00,     2.4228048863511867e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.7768201114608033e+01,    -1.3242760766708539e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.1348452487580349e+01,     4.1679505540028040e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.0833363967232384e+02,    -1.3240799735231762e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.2267554342201127e+01,    -5.5131102870615450e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.1736364927038228e+00,    -1.7271884007961978e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3714092207424638e+01,     1.4547162884967263e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    5.9354490935322435e+00,     9.0202773882373476e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5487289344656432e+00,     1.7253041180361790e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7123000942146550e+01,     2.9420124697068200e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.6122265283193986e+01,     2.4092992791287183e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5610106424543808e-01,     1.4614725739973192e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.4052496887515098e+01,     1.5546642182977790e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.1129549480914683e+01,     1.5538866397390124e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1841252056403103e+01,     1.7302551181189978e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9865272343283543e+01,     2.9931065119203708e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    4.5791112368222375e+00,    -1.7554758477351315e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    7.9361398734243527e-01,    -1.1634615070219354e+00,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.7919670378084220e+01,     7.9618151561981122e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9154640757303172e+00,    -3.7080409927767741e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6984238025251610e+00,     6.2688489411179046e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3226251638880324e+00,    -1.2451519525112520e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6430812432335031e+01,     2.8822983966577493e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.4182441046594318e+00,     6.1369735495122950e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.7994067854609863e+00,     7.9804214897835254e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -3.0484086837951132e+01,     8.0509535138874180e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0441137005457605e+01,     2.8766924391717594e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1379543658399767e+01,    -8.7348303056175682e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    9.1134720523437736e+00,     7.2090871585563646e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4863124124631426e+01,     6.9979544440283817e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.2556400021039913e-01,     1.5345205033945615e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0019870785848470e+01,    -4.3335187700955890e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.8143864244301824e+01,    -1.3397200533854448e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1467572629392055e+01,     1.5537778583281490e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6627342118215765e+01,     1.6029068550853983e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5377519596689844e+00,     1.3240688993379676e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7640292792552046e+01,     6.9630437659586896e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1373437760413815e+01,     7.3694960199595298e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2006655287708941e+01,     8.7654832279272741e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.0183278999312527e+01,     2.1245770545875799e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.4301893592583454e+01,     5.7501994293767584e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.8658868679557372e-01,     8.3209240538258437e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.3919406224687364e+01,     1.0745018964344617e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3878270083063805e+01,     7.0497695555186710e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.6106753766380134e+01,     5.6323332634020517e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.4633742084167693e+01,     2.3261187073218913e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.4135127817526740e+00,    -8.6779415609089556e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1180885025409584e+01,     8.3364587956244086e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5695116471682748e+00,    -1.1462677484996329e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.4033385918338951e+01,     8.3650921570856117e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0273422975785072e+01,     1.2772000045434583e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1986200391775537e-01,    -1.5786428343716775e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.0119007256040319e+01,     2.5491522983989758e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2577002215693422e+01,     2.5016727196485081e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    3.9625337697365390e+00,     8.6000194369306737e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.7551086675080558e+00,     1.6135724804293968e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0107498798167436e+00,     5.5403051537000920e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.1374455038922370e+01,     8.3299920111768934e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -2.9227156837015965e+01,    -8.3273908382733589e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -3.9298322989807687e+01,     2.3774542914346406e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.1946086068697301e+01,     1.1364638739579148e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    6.8454331707593159e+00,    -1.7195518935161293e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.2793453242100810e+01,     1.9777240867873170e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3252966690861149e+01,     7.2190385723613017e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0546673110259679e+00,     6.6169336839277721e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.2605557607743179e+01,     2.3718642392724574e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.5406691805222081e+01,     2.3871239999681871e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4291459910252023e+01,     7.6731538776153138e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2858630194835944e+01,    -1.1565770297551207e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1278368639882377e+00,    -8.3106577407377990e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    2.0168841586066428e+01,     8.3279154781366087e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.1745471343830403e+00,     8.3294673713136435e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0210386293536930e+01,     9.5595694452943790e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.9411408984712551e+01,     8.9735074405067771e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.0850904893055915e+01,     3.8141837091486312e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -1.6478481866772512e+01,    -1.1142999267898447e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5579800905006579e+01,     1.5639657090746639e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7632525440449658e+01,     1.6588204416897155e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1203752538669189e+01,     3.3261796601689312e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    6.9674700277537100e+00,     6.7254749023386357e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.1097712074892762e+01,    -8.5441409446422640e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -6.5266932602696158e+01,     2.8628336119423302e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5609828609243307e+01,     1.5547729997086428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1152993099876838e+01,     6.2759669105510438e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9122439944548404e+00,     6.8079409505135427e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4703968342992020e+01,     7.8812189499546093e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.5251043652183780e+00,     7.8203291970596474e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0075994250630745e+02,     2.0091436147773358e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8176884939577747e+01,     6.2829058100548045e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -8.3201421865107719e+00,     3.1080577548211997e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    5.6191062066130335e+01,     3.1090439612523838e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4768737718526971e+01,     6.1306011607473683e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0582817545286840e+01,     1.5764301366562058e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4497729321775154e+01,     8.3127970804786219e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0038280857849315e+01,     2.3249171604258489e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    1.6732150842841179e+01,     2.8885117116550427e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.4159287431018868e+01,    -1.6608680983785176e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    2.5610714124886900e+00,     1.4984336483158388e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6099574768043059e+01,     2.8731185531819319e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.9906323633675921e+01,     4.5842220931165895e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {   -8.9188426127856744e+00,     1.6193603780360558e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.1560635375676654e+01,     2.5557293960556151e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3872871370251303e+01,     9.6206436657517479e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9372914344759081e+01,     3.7711078762679113e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.7811377087460098e+01,     2.7331082181133846e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3610643934126662e+01,     3.6922932492611353e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4754419041980864e+01,    -7.6382506897066469e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2944917293474856e+00,     1.3533878608209863e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.9389840883872637e+01,     8.9573922560331102e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5275093375027181e+01,     8.4003960756043980e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.6206579174251484e+01,    -1.5225973074902254e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.3503975631299724e+01,     8.3445857689716322e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6485246177727621e+01,     8.4809873168054073e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8637311312603320e+01,     1.4215946432770761e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -1.6814214847625880e+01,     2.2642645991309291e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.0573111936394525e+01,     2.2732544282033312e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5642229262656608e+01,     1.8151914467210077e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.8437429731076147e+00,     8.3751015663365888e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.2082788728673730e+00,     7.8385571586912411e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8496175134644832e+00,    -1.4738681014563167e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.0314425484024138e+01,     4.3876650164355324e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2039813568354216e+01,     4.0103917902581816e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2506466963155859e+01,     8.5840892668487068e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4902091657744535e+01,     2.4298248112105037e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    5.0843830783732365e+01,     1.5265780945377001e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1254892397550417e+01,     6.5859789374693228e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.5467658054389538e+01,     3.6523367806257076e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.2132184507880748e+00,     6.5320748377352211e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1643488233675798e+01,    -8.5256216522334689e+03,     3.4220980924608724e-04,     3.3415782970474765e-07,    -1.5759353518146575e-09, } , 
      {    1.8609730181261721e+00,     4.2276294622553651e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2188851915355796e+01,     8.3303297663001504e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7495852162642340e+01,     1.3171158244868504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.2367138715628112e+00,     1.5682473912109306e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.8980647823923480e+00,     1.2416588369529729e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7458217880134796e+00,     5.8493642362296316e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0919684383877257e+00,    -2.6030239884801176e+00,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    7.4713316524621135e+01,     2.2033337738512993e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4643332317545337e+01,     6.1059932617922650e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0814477088936842e+01,     1.3091328467498271e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.2912237259819781e+00,     7.5606642177542362e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.4524497479783385e+01,     6.0682559026780527e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -7.2876250684767809e+00,     7.7665071322040531e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2721183764742555e+01,     1.4221206097403128e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -2.1383514074275759e+01,    -1.6659450977989218e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9482295496644539e+01,     1.5013773500007504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.4791217641277125e+01,     2.2032949596719460e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.0348832645199229e+01,     1.5540151266195478e+04,    -2.2066769813108015e-04,    -1.1756731766906248e-07,     5.4866364291166172e-10, } , 
      {    4.8269113381437139e+00,     7.7429912116817213e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0508937833679951e+01,     1.4258391166984857e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.1013077693108480e+01,     7.7577621640598454e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8810704101424017e+02,     8.3233134532991662e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.9687682296303937e+00,     1.4310722283893805e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6502776908089075e+01,     1.9225058133438899e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -4.6451522148019215e+00,     2.2126432027813546e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9403551580915256e+01,     7.9935458492450780e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.7581740684772194e+00,     1.3926899577902848e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.4130791180165893e+01,     3.7497202633761481e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7359674013098565e+00,    -1.8340800416737293e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.0460613685306171e+01,     1.9308670677173672e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.0264188439119152e+01,     3.0397872092521720e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.8240493984693380e+00,     6.6541187535095005e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    2.6428286911746630e+00,     1.5517015155037036e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.1160577545450593e+01,     3.4960327176015744e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1928976909370501e+01,     3.3994389722559259e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -4.4779338784259870e+01,     1.6656994707656719e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.0484789711432910e+01,     1.6657770991243786e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.7172355938234048e+01,     1.6645176924630665e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -5.9736166880831448e+01,    -1.6669588774269840e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -7.6659744827674430e+01,     3.2200039088060366e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.1315897813074230e+02,     3.2200235191208048e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.8543064367940282e+01,     8.2760746434720095e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8504460477898643e+01,     2.4030353006542660e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0919496741587944e+01,     1.4178761363189031e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    7.8266940019504858e+00,     8.8304422528373107e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3569196005242176e+01,     1.5540476773825851e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5857069033685805e+01,     5.8718606321606103e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    6.6516217228588337e-01,    -9.8815971095749683e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -5.7197217483622023e+00,     3.5011060998941884e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -6.7581114316513634e+01,    -1.3086359968085853e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.8203294061561488e+00,     2.7043502879377538e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0224864355764197e+01,     2.8816425010977764e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    5.2465577727396457e+00,     3.1067459637012540e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.4590797297582835e+01,     1.4421566775317940e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    1.6450859900140433e+01,    -6.3171859104348450e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.5053724799886659e+01,     4.0853142189541227e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9227715046401187e+01,     6.6593784181418687e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0778963018529673e+02,     6.9440893974944993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5706210400168546e+01,     5.5770649155693782e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.8466004476904478e+00,     1.0317225451270095e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.1175960786536649e+01,     7.7848836563633340e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.3997722345541384e+00,     6.1479941762346598e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1621232935596282e+01,     7.6121175375003295e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.9496350983743609e+00,    -2.5381333792323976e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.5425136114030842e+01,     8.3230015966095434e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4430910790357720e+01,     8.3343812528407088e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8370455729251859e+01,     1.6975152757970853e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8180676760398452e+01,     1.3667820941053524e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.2463681397769495e+01,     3.3663364616970011e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1565909586218487e+01,    -7.3204807811860501e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2425391071805961e+01,     8.6026752403685168e+03,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {    3.0749624877211428e+01,     1.4492560080730331e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {    4.5522279041938091e+01,     5.3294924262802779e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2030823370399261e+01,     6.0543504959304264e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6536433830725299e+00,     1.4712941517288822e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.6486886753257934e-01,     1.3140857638516874e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.7040679146969623e+01,     8.5346714641196886e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.1681008965108326e+00,     4.1812287964300203e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6645302728651281e+01,     7.8217518879934014e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.6581704977440239e+01,     1.7341047364682141e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.8696166181451277e+00,    -1.0036682680026977e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -5.8444629786217028e-01,     8.4580587972275389e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.2379174852702576e+01,    -8.2553969194729943e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.0149874243506893e+01,     2.4969864938833471e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.4664531838020185e+00,     7.2160272131295005e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2556594319092468e+01,    -8.7761638994956429e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.2247030521559985e+01,    -4.1167944466173522e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.7546083175564036e+01,     8.3813082059782428e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2986164371033670e+01,     1.6731005987178833e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.4569055853035611e+01,     8.8451607612618682e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2015608025760889e+02,     1.6655347332433175e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.8438327758603034e+01,     1.5186151141581175e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0445116103259380e+02,     1.6659418366467329e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1481572361213004e+01,    -4.7500330638188134e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.8336686697129352e+00,     6.5819525108039634e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4158301879297266e+00,     8.3920581465112518e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.0704901531965412e+01,     5.9747206921346024e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2358568462271890e+01,     1.5522404146988851e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.9435621055671461e+01,     1.5563104433379067e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.2703342258888703e+01,     8.3114201629556574e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2182531912230743e+01,     3.2992722795681475e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1123553302092084e+00,    -8.4154324736531598e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.0227621564485545e+02,     9.5835756263377862e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2712500875590347e+01,     1.0018169274734815e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.9671754123941225e-01,    -2.2599100408422914e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.9153540189041966e+01,     8.3512315948587930e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6263244716504204e+00,     1.5735443009320497e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.4890167191045748e+01,     1.4915510366563365e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6700642363260911e+01,     7.4304103717152502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8460727422414173e+01,     7.7009361292120666e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4115427170053643e+01,     7.2866927924120300e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1037716628590058e+01,     1.4322187536495283e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7654438691346559e+01,     1.9093060928620303e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3423980428290514e+01,     2.3137199935441160e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.7827250819902282e+01,     1.3797203572923985e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4079432368770405e+01,     9.0306221474992490e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9120598669180900e+01,     8.7795877886310536e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0735847824567490e+01,     2.9062513098751601e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.8835804121949487e+00,     1.0239583870547140e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1001957906347556e+01,     2.1078829293388746e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7846582203519858e+00,     6.9945239936219159e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9869944359860376e+01,     2.4477211394823571e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.5232981349753981e+01,     1.7415610096530694e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.3079819404935613e+01,     1.2010496595102988e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1053137956237570e+01,     6.9515537782030397e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.1438571002659899e-01,     7.2151921160255679e+03,    -1.8974046378574064e-04,    -8.0643907715760229e-08,     3.7408223634411777e-10, } , 
      {   -1.9538736505332579e+01,    -7.2129336148921011e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    2.6070357881764856e+00,     3.2191500641250779e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -8.9609529575227754e+01,     2.0131251049671268e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0090895082802152e+01,    -1.0829893700329540e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4977412473154015e+01,     2.3258094463001657e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.6026662953771186e+01,     2.5851540302129972e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4286552863200201e+01,     1.4893828142627441e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3204779963652953e+01,    -1.6669380459997625e+04,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {   -1.1145878303715911e+01,     1.7455507172599719e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -5.4235123356757924e+01,     8.2565112605667855e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4533660092109281e+01,     1.6459308112007129e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.9423085835116240e+01,     3.0010694922999533e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.6134604874763490e+00,     7.7739714994017495e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0532364643721284e+01,     1.4611597234779932e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.0330809413465758e-03,     1.0440274357794444e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2596471353741130e+00,     5.6485379422866099e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.3147952191592065e+01,     2.2690835693901536e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1950907427725316e+01,     2.2785203089053051e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.1141490898601774e+01,     8.5986648926894595e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8817519579178690e+01,     2.2597909864009216e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.8190027064792098e+01,     6.4468468040368889e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    7.3390024231122180e+00,     1.6577822383896950e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.3163641250789219e+01,     3.9902822179153532e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.3043323077479320e+01,     9.6036689112799868e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.9152479829119130e+01,     6.4475514359530371e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    2.6974392323766121e+01,     3.2130247361614238e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.6070915866943578e+00,     1.2747836668823886e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.4123613408122264e+01,     3.2299463934503299e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0437277891429458e+01,    -1.0122269743234110e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.0785210087651823e+01,    -8.3260768517769939e+03,    -7.7255447935749440e-04,    -9.1067401859615238e-07,     4.3024790130065763e-09, } , 
      {   -8.0681235274498022e+01,     7.7471042714824962e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.5518904504769338e+01,     1.1468498381474414e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.9875805831120630e+00,    -8.7701304251006950e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.0059087933086651e+01,     1.3959077753463771e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.3873986083526976e+01,     4.1641155923785896e+04,     9.1387650384808943e-04,     1.5024666865793185e-06,    -7.1180344660502071e-09, } , 
      {   -9.0651605822649458e+01,     1.1508313283372327e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.7331677547481501e-01,     4.9960335517418289e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4252826322373714e+01,     2.0528073969355330e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.5127085673094584e+01,     7.8579124787928067e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9112938129642153e+01,     7.2139220781858203e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2530054118217535e+00,     7.2142036527318451e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3741417551283121e+01,     3.1542963899873572e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.5361537972742738e+00,     7.6687446504462432e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0040406219098504e+01,     2.5193322540166366e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {    3.6625837909660497e-01,     5.5180260598494151e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8792280150874724e+01,     8.5262406809634704e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8043807738932756e+01,     9.9024136450720998e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4049361739600869e+01,     2.2105967665466193e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0183684626231219e+01,     1.1160020305904345e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.0033150185148330e+00,     3.7636151871499806e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.1097926602314203e+01,     8.0499871114397392e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.7102954586090835e-01,    -4.3363485778223176e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6499181134396252e+01,     4.4783351886471100e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9873108870840387e+01,     6.4924552758855261e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.9910832087349704e+01,     6.4282266413000671e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1913071147540897e+01,     1.6950544859015747e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.1343367058377503e+00,     7.0165136092204884e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2212152100592224e+01,     4.0830130191143871e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7999202896270063e+01,     1.3308228263287172e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4301027568700729e+01,     1.5350065571047418e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    4.0205177098689537e+00,     6.9925157890807350e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6938752797712830e+01,     3.2218186082989581e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8462549028703634e-01,     7.0617669733785515e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.8170485132297962e+00,    -5.8308801955821427e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7198227724068072e+01,     1.0444971671930769e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.5143274717415064e+01,    -9.6549469926058409e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -8.7452429374050453e+01,     1.7657835611387010e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.7128655730120617e-01,     8.3382893605137378e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4859053977891577e+01,     8.4887269789052934e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3255915562156869e+01,     1.6657737453048358e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.9536541197574675e+01,    -1.6657028245852143e+04,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.9097906510421922e+01,     1.6852329081700118e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.1463918212106458e+01,     8.3190934889365144e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4145907371035520e+01,     7.2050274715991309e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9637424358258606e+01,     1.7396525450133806e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.4016345883892278e+01,    -8.5152761744126055e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.7458283467259925e+01,     7.7871096292395705e+03,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {    3.0997578837120805e+01,    -9.6507919070384414e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.9141170938246042e+00,     6.5855316233572557e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3256591225217155e+01,    -2.7562699845524776e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.4014188865286656e+01,     2.4111663362417563e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3546088927421920e-01,     1.4410076787562288e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.8232467275045263e+01,     8.6289712406938888e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0379765476951672e+01,    -8.0284116087563634e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1139665672607592e+01,     8.3339510893574952e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4537122092420418e-01,     8.3234317600927589e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.8812182683255344e+01,     1.1771373721531018e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5661025752368801e+01,     1.2894537216019618e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.8561061219349355e+00,     1.5540349354165755e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2668388332055676e-01,     5.3990298883421756e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1373639161673334e+02,     2.2781090029252271e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.4234349564920574e+00,    -8.3378802126410374e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.0027721553185536e+01,     5.0747086706462997e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.1744557354584249e+00,     5.0540117847440306e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    6.2971213944816050e+01,     2.1659461593143355e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.5247825634127736e+01,     5.9817205689772290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2966953020379498e+02,    -1.1142829568589443e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.4514695172449294e+01,     7.2709450488963921e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9650483817837909e+01,     9.1688496754411244e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.9147882687829146e+01,    -8.0493053433869627e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0965993873272147e+01,     1.4663412346270221e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8925075007427193e+01,     1.8878437829571256e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.7376311038484850e+01,    -7.7867962561311688e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.7853992942480680e+01,     2.2753014386033465e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1006765496949420e+01,     2.4650928698695330e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.8585551130106673e+01,     4.0526198732043158e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -8.3001436163101459e+01,     1.7363589269813194e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7487220328241719e+01,     2.3944740220161209e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0704885644634299e+01,     3.2807114816701906e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.5030450295834299e+01,     1.6411279522737976e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2851097878987929e+01,     1.5056785631921284e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.6014800117996273e+01,    -1.0920866541352370e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    5.1480093641889646e+01,     1.4433056763073586e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.1634094901926375e+01,     6.5684822836464336e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -9.4199000014853667e+00,     1.7032354875787867e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3077505799322717e+01,    -8.5536600015950371e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.8267374510536484e+01,    -2.2682246259757767e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.0390723691839558e+00,     1.2354815699130899e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0766709032196893e+00,    -8.4166054109463039e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.6576955015007275e+01,    -9.4453975267751193e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5092423235149584e+00,     7.2051844770060934e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3380198346738322e+01,     5.7375801910543805e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1345002379477512e+01,     5.5306452981717930e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.1824591550988174e+01,     2.2182190013275314e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.0981068393803042e+01,     1.2640105776343717e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5378969826448813e+01,     3.2494383481208031e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.4116558709757793e+01,     9.9318480129970176e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5204698078697190e+00,     1.5321207213805861e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.6087577899688256e+01,     2.4300678790465066e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2624628518941471e+01,     4.0531458396675524e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -1.8731586791767152e+00,     5.4404737988372917e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8130911476415069e+01,    -1.0326059008712866e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.8226105888863344e+00,     8.0830336083773955e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3316047490211279e+01,     1.7321027505431332e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5151602770454947e+01,     2.0914554216461864e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9235365073383342e+01,     6.1524034034037304e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    8.2574138699939184e+00,     8.4952447040153129e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4830175221276242e+01,     8.9258474145838882e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.3970903676120983e+01,     8.3782053363756804e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.6175095034068931e+01,     8.3158895453699224e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7414104296400172e+01,     2.1249167626683506e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    2.5305563929544412e+01,     2.2760619925252118e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.0551603756081951e+00,     7.1682838274084907e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    8.2139182110726594e+01,     8.6073957380105130e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.3785467122287010e+00,    -8.2816223816003512e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.0685996020430989e+02,     2.4985566315637345e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    1.9586822542438827e+02,    -2.4986582232713412e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -6.2080016267574090e-02,     7.0191166332089688e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.0914783061620570e+02,     8.3290571677846528e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2014212506788908e+02,     8.3283256816655976e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.5181016362394715e+01,     8.3414933040803298e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9052772651212129e+00,    -8.5190532208476234e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    4.5366589829554904e+01,     2.4504684332110552e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7058027274969412e+01,     1.4378462333483119e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.5251578475098377e+01,    -7.2253023878307413e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -9.2757918246182065e+00,     1.5638668867659857e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.4469490669579983e+00,     1.3924269745586666e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -9.8461666987119720e+00,     7.1184616610913363e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.3125499415189367e+01,     1.0611566181710663e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8022537124305586e+01,     7.6005248795520911e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.5003335231050389e+01,     3.3332789907101709e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {   -1.4325209877763781e+01,    -1.7450136870955371e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9522454253456644e+01,    -9.4412424412077198e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.3812009810725918e+00,    -8.3036636350450790e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.3128080832695467e+01,    -2.1671021479205492e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.7950818480530199e+01,     1.2755079364625801e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8446875935541929e+01,     7.1479395729343623e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6623053896678694e+00,     1.6654779825461774e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -1.2268589499498102e+01,     8.9345645100634993e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7030225810041554e+01,     7.6385640628150504e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.7919349303525980e+00,     1.7850008295607782e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.6467541144549884e+01,     1.4434159205312615e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    2.0693325532207432e+01,     1.1779139103033978e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.8467176131311724e+00,    -1.1429620487229863e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2701096627679448e+01,     2.3359385605541414e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1972517815131667e+01,    -7.5606984286945244e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.0393463771539846e+00,     9.0591707186975509e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2199654490241031e+01,     1.1413340605244226e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.7491578661962315e+00,    -3.2892828735619523e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1245179467907864e+01,     1.9494623224986390e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    1.2816137869155526e+02,     2.3869410197892012e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.0284328018044405e+01,     8.9175364238308321e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0753122751125881e+00,     7.5698719572423888e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -4.7036852860182066e+00,    -2.1202093704891854e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6321329944693556e+01,     1.7735890666215412e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.7778892937692106e+01,     2.7333383380973581e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6842779225772460e+01,     8.4188629240596292e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.0505567895263113e+01,     1.1203791139413504e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6860681968082311e+01,     2.8767233948221728e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -2.2489408504224691e+01,     6.2600638520580833e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2159892757993742e+01,     1.5599636473621520e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.3589761733235175e+00,    -3.8062567700924064e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.3963919652917021e+00,    -5.4378592258891567e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.4639981832970879e+00,     1.4998557921750267e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3373674016647533e+01,     1.6086453588784971e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0334570525814371e+01,     3.9159768565417144e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.4005900959697918e+01,     3.2195206107478294e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.0754127308822326e+01,     1.1780722351731236e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9123578581924356e+01,     7.4362315696119331e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -6.8365416497601160e+01,     9.6008486231046427e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0098379062227574e+01,     7.8515592384202882e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.4070448212785230e+01,     1.4913383043713467e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0908917193581093e+01,    -7.7622036165441132e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.0575721289611728e-01,    -3.2374677620716629e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.6373494578482365e+01,     2.3873481231926158e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.4180051811781800e+00,     8.9558355482636471e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.5538219635504113e+01,     1.4289952411906008e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3369688388567496e+01,     1.0971617769886163e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.5869676303949651e+00,     7.1367773240344713e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8091419456405678e+01,     1.3613732069061720e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.1201640421319483e+02,    -1.1149741616736417e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7937388953673182e+01,     4.8713561583400970e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0768420271768377e+00,     2.2772549639790388e+04,    -5.3411709929817881e-04,    -3.4590486519803176e-07,     1.6210715055259552e-09, } , 
      {    5.0055650084980698e+01,     6.3765581305406422e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.8987814576662075e+01,     4.0002692688240670e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.8043369617852267e+00,    -1.0726800887774662e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8708789048420179e+01,     1.4979147774538849e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7370487335429248e+00,     7.6797652771686080e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.2363482121026426e+00,     5.8128312522267697e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5278319059051114e+01,     1.0881763475132917e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.1903613044617330e+01,     1.7952081952960827e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1335068897382003e+01,     7.7948298378624759e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.5216707198916541e+00,     2.1154080354561134e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.8320676613222702e+00,     7.6758171712141893e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0080576394273628e+01,     8.9466897285151808e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0002213905355299e+02,     1.0211883211383329e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4551027680685507e+00,     7.1243340063783544e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7752479789426232e+00,    -8.8330366071470817e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.3651646517600689e+01,     2.5931170105925790e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.6318316199597511e+01,     2.3238207097707622e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2517988363335060e+00,    -1.0124596666248153e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0120377846525406e+01,     7.7559580340527882e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.3883283506565050e+01,     3.8352540543037940e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.5147630091782993e+01,     1.7497541100166251e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.8299104954085486e+01,     1.4563112464899832e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -4.4704887157130750e+00,     1.1961794729437912e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2900839140701736e+01,     8.3342735891873381e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1897299961979382e+00,    -8.3231092602629124e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.4192108726507332e+01,     4.3696313324482053e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.0013623985593654e+00,    -1.0674198004560672e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.3625206706812827e+01,     1.6157555363958149e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.2412069880493917e+00,    -1.7947512558021265e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.4188208358642932e+01,     1.4852089898109447e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0063659039672059e+01,     1.4993724192976164e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3772759278391105e+01,     8.9833074003972270e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7213910163537331e+00,    -9.7492788891318696e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.2174577690882366e+01,     2.9344873635895809e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    8.4883435622022176e+01,     1.4914652420365626e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8512839483089948e+01,     2.3850827152398109e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.0471546659202655e+00,     1.5105962486981811e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1094449748943351e+01,     1.4180200925670490e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0196266815586771e+01,     7.6982416014099581e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.0479535253674577e+00,     1.6085742243318095e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1703780987255948e+01,     1.5994130375394723e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -7.2292674060053216e+01,     9.8950949646784575e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0828991319810939e+01,     1.5476630997659489e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3781262504389176e+00,     5.5049680827645579e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9051922002636299e+01,     2.4411839313733446e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3092434500666801e+01,    -2.9597364125050108e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.1588347763768141e-01,     1.5137114082168591e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.6053232376927149e+01,     1.5557710623322073e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.5876170143363773e+00,    -1.1899661579464977e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.8071523636528291e+01,    -1.6688863077113485e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {   -2.0321715431494663e+01,     1.5476278681701415e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4833845964941620e+01,     9.4521384383295608e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0120876936703203e+01,     8.0174908899354705e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3807981484785643e+01,     4.8202772309703969e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1754534853948117e+01,     1.5121347387080712e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0746067690775922e+02,     1.9757374904301629e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1649717551121354e+01,     8.3213686797049577e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5322198275746270e+01,    -8.5544502258977518e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.5002807330564401e+00,     7.7423323101684100e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.2511883249226727e+00,     4.1474473689270562e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6147258096991472e+01,     1.9537398336926504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.9569495800085036e+01,     2.2128411519023408e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.7659275418628802e+01,    -8.2845919587395183e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.6285537513974020e+01,     1.6100061856266380e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -6.4894311927062745e+01,     1.6100075283367831e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2936966081120067e+01,     8.2625681322006567e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3712732582965359e+01,     4.1645758323465358e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    6.6963063189458278e+00,     7.7570508185929684e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6414809660197150e+01,     8.3267244506736533e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.0123758123594428e+01,    -9.6529675013959804e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.0597808092286287e+02,    -9.6527713982483019e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.4651981816076139e+01,     8.2250202192021879e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.5967190580476755e+01,     6.4122542309849787e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    9.7073695374408899e+00,     1.5535750708705118e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -7.4497084896241219e+01,     1.0817648891297815e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.4802803003802469e+00,     1.3828683800587216e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0653477346669138e+01,    -7.0874953656732287e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.7362718767524878e+00,     1.6615434378961425e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.1088753898774355e+00,    -1.6873491270880699e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.9703678668947195e+01,     1.0549746404846130e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.5420486389054801e+01,     8.3306583987766007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1786078678116059e+01,     2.0352305931027571e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0624405241040225e+01,     8.2661243917253232e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5795045194119851e+01,     2.5052809823827782e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4217403344391723e+01,     2.3707450564803048e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.4800066550057725e+01,     6.0464652089813480e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.7669881119465085e+01,     8.3152211975675946e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9237704495600605e+00,     1.6594104470983573e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.9993613200397746e+01,     1.5309768874671137e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4219809085467411e+01,     5.5109416230449078e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.6424360442780205e+01,     2.1657936339892138e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.1654879474551496e+00,     8.7489348663213514e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -2.8704090160203215e+00,    -1.9923005120404705e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.9219107583888658e+01,     3.7095925372946855e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.2769654408334027e+01,     1.6095810814057113e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2913627367849024e+01,     1.5780300874886541e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.4521014717857014e+01,     1.5545736438458220e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5738949925320057e+01,     6.1432203990697471e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0502252414541915e+02,     3.9414193291542317e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -5.2440503723795295e+01,     3.9414206718643771e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    6.2612378669811246e+00,     2.4042297980448973e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0020837798833199e+02,     8.3294318824767379e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.9214083536648602e+01,     8.3279509669735144e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8083932240211507e+01,     3.9978073240325917e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.7821282895449016e+01,     7.3030803790724349e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -6.7906586515193084e+01,     8.3294094837427292e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.8900880966876471e+01,     8.3279733657075230e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6914761427550147e-01,     8.3912584577249290e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.7195964934768924e+01,     2.3871651430136299e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.3141406142753436e+01,    -1.9098257941868546e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2631099579424541e+01,    -8.1583378320623151e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.7621326106003547e+01,     8.7305288631311669e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.9725183582578296e+01,     7.2290191985969459e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7638455093455299e+01,     1.3753432739414826e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.4615015675330554e+00,     1.5611446395756619e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.0302869074118007e+01,     8.3271015703522680e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1297163525801395e+01,     8.3302812790979842e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8626062811148685e+01,    -9.5536895167464445e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1663825419817201e+01,     1.3259424547047754e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.4769774185162662e+01,     8.3293214313642293e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7888016804474695e+01,     8.1416838640254609e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.7925419858395152e+01,     3.3308206743300783e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    4.5287226304297175e+00,     2.7222471903236510e+01,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.2416902106259826e+01,     2.5043949668388319e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.3616534138670691e+00,     7.2420041729899767e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.3558736697586269e+01,     2.2327584080086810e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4385554759274626e+01,     1.6666261237902992e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    8.1049291238568184e+00,    -1.6648504460997516e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.0741345169312240e+01,     9.8101051312854979e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2271677184830558e+00,    -1.9498557778263739e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.1783901890351700e+01,    -4.6675497706507704e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.5248032536031722e+01,     2.2661257181765002e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.7061163130074661e+01,     3.9493829808888870e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -7.4077493125584013e+01,     1.1151710134769542e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3317920798501671e+01,     6.3975357225604193e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.6966111661070760e+00,     1.5514757171312098e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.1616685942003462e+01,     9.9966898809729737e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6883536128514930e+01,     6.6049429501215145e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.4318393092898788e+01,     7.2135641925817545e+03,    -5.2746002568012762e-04,    -5.4949267430635921e-07,     2.5923957156289107e-09, } , 
      {    1.3022365937309758e+01,     1.4759342183114168e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1377643975321437e+01,    -1.6199927529944594e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    2.0658095398526726e+01,    -4.3992030839540848e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.8884396743081162e+01,     1.5649403830073217e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7662301447284516e-01,     3.9414528637569492e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    5.1605406009650515e+01,     3.9413871372616595e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0056023466812967e+01,     8.2439945489820311e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.6888617252136740e+01,     2.2772236266681983e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8658490371144172e+00,     1.1712955381796792e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6514560798155429e+01,     1.1898895882411007e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2820927730581381e+01,     2.6695169056869654e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1875669224527122e+01,     1.6676510585043881e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5957883403935835e+01,     1.8195685300719237e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -7.6919982274285019e+01,     7.6996792077634318e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1456812711701204e+01,     3.0950521846385753e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6712562567293404e+01,     8.6410316282127314e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.3251608119517563e+01,     8.9548567715010449e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.4494047506419411e+01,    -8.9724181208098598e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.3128116380496415e+00,     1.5551927818731820e+04,    -5.0102303868648844e-04,    -3.7787832746720565e-07,     1.7754361815912314e-09, } , 
      {    6.9692800349843136e+00,     1.6699331319939080e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.7863258806850943e+00,     1.6720661227916931e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.2739202350475853e+01,    -8.1816377419530636e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    5.5942531124454860e+00,     8.3563135687149424e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1581266569421793e+01,     2.9216004936270474e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.2441143617692951e+01,    -6.3060878488527669e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.8546402198659266e+01,     3.1968948554140579e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8918690634767596e+00,     9.2698378861353485e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3863793341419044e+01,     2.1606851525440965e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1978705919496743e+01,     1.6884956523482178e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.9582763122218886e+01,     2.4257907729002342e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.0222170490851639e+02,     1.1134437138002686e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5124255594154463e+00,     3.3640352618572656e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4165358689096497e+00,     1.1933367916244072e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4948860997052620e+01,     2.5210544178168217e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0516395949246522e+01,     1.4603909028613472e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7903426717467099e+01,     1.4332953898833412e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6818243937076712e+01,     3.0440280314513588e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    4.5635485176072095e+00,     3.2262493946663177e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.8237656388319600e+01,     1.4912304466868789e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    4.1527702705315832e+01,     8.9281486144236223e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7334390229875254e+01,     1.8653140620224895e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    6.2002294773445961e+00,     8.8827000045816258e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.5707200866977900e+00,    -8.8852943588913968e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.7980330926130556e+01,     8.4187961848621107e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.2448438086533109e+01,     2.5001806758322968e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {    5.4530295716518076e+01,     3.7673565060620938e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.2014355448240841e-01,    -1.8922688384385248e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    6.8624177171127139e+00,    -1.7196672920882196e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.4139015611165267e+01,     1.0807871976487311e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7831805207328863e+01,     8.9541540957128782e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2071942341775248e+01,     6.6124159869979230e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0973993544716333e+01,     1.5744259478033548e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    6.9944068309866818e+00,     1.3026894117685602e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8819967506078214e+01,    -1.5040467329243629e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.5029221671374877e+00,     1.8433586489019930e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -6.9000115643196594e+01,    -1.7273941160234072e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6914806033438023e+01,    -8.3985554795707885e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0083007077817740e+01,    -1.0888914097444595e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.1477597905675459e+01,     2.3306416508330221e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.9127686066142582e-01,    -7.2318447846373110e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.8302411467169609e+01,     2.3350179851678349e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    2.0151189772069500e+01,     8.5164570736002588e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.6383206472817804e+00,     3.3350239266818903e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.3136486660694509e+00,    -8.8867170498251489e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.9372810582144037e+01,     5.2643771280330857e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5435345679789510e+01,     2.6340905221582099e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    5.3926711717793321e+00,     8.9776175722816442e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4493543908047911e+01,     2.8652131701037604e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.2576968860085227e+00,    -2.3689168063433358e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.3763355009801534e+00,    -2.1539907979921495e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.7133997004990761e+01,     1.8088799655421419e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.6142964624776457e+01,     1.1310969742361194e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5819626918927113e+01,     1.4925936692894062e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4812471723102156e+00,    -1.0387096277717827e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.2965082197839809e+01,    -1.5533580761636094e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.6493720954833492e+01,     9.1353968804146098e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3331200724268339e+01,     7.8953300719294457e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -9.0617993082310377e+00,     7.0270553047591675e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    3.2338038058932135e+01,     3.0546218508935970e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -5.1072377918999354e+00,     1.2197081344790467e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.9244801227443755e+00,     6.3816202602475141e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3121753319107235e+01,     2.3072285864323654e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.7810432842413748e+00,     1.2726146784155719e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5411958155835809e+01,     5.6648780818101113e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.2000965245643243e+01,     5.3445592213613927e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0822003870835786e+01,     1.4937736130001340e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0926197518001839e+00,    -1.3117265055719026e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.8666979530568746e+00,     9.6476634018451805e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4927718537406108e+01,     7.5475805174950692e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1183958980167141e+01,     5.9595252381514465e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0823079744203085e+01,    -1.0283478234707418e+04,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -9.6748291544075951e+00,     8.3380791345429043e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7356893141651604e+00,    -8.3566327322562702e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9580813798514627e+00,     1.5769224436823461e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.9296981071934056e+01,     2.2686176166188467e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.5220235766005633e+01,     1.6012865929062802e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4012910594914320e+01,     2.5947884251889111e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.0732033563876278e+01,     1.6153788878462645e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3123084621111224e+02,     1.5543099892591308e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0571583839472835e+01,     1.5117539874070462e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8549433717185249e+01,     1.6315678318370479e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.0485907132944575e+01,    -1.1409369498928518e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.5636220203438150e+01,     3.1062966675236861e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7129758337525391e+00,     1.0325173662064033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0216040597795139e+01,    -1.7741145478263206e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8401627499439300e+01,     1.0193937313062572e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    8.9421713204614193e+00,     5.5513257804224568e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.9016577094216105e+00,     1.1568773759744923e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.3218436498768975e+00,     2.4500081932431076e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.0606800867155459e+00,    -7.6843843313550969e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.2492081670019193e+00,     4.4786638211235586e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9779082020475688e+01,     2.3090878399114517e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.9999910404715941e+01,     2.3882262426268804e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    3.3352004662482102e+01,     3.4387410692252248e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3464939117231864e+01,     4.4034127599511703e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9939989685317798e+01,     2.6312054826235049e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4722506881958237e+00,     1.5663286242401078e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.4101521729065990e+01,     1.8052929513408053e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1173540734272763e+01,    -2.5460453652374581e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2785419746101345e+00,     7.6929056731105256e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9795281664220202e+01,     3.8246602348615554e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -2.2670087576854872e+01,     7.1475872569762869e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0991765349934131e+00,    -1.0092291085340472e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8274417196877053e+00,     1.5571106012653931e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {    4.2519729060460101e+00,    -8.5743492410728559e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7011733890990484e+01,     9.1983459078458764e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    9.7356225237245255e+01,     8.3283811345336762e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7245718391102141e+01,     1.1188463183088103e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.6361944795897060e+01,     8.3290017149165760e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3279535765942681e+01,     2.3892064277420061e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9057999222788766e+01,     9.3995338911142117e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.2567987388351025e+01,     1.3525297475669850e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.6721348827591520e+01,     2.3871057573115548e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4178244251229232e+01,    -1.6016456026082331e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    4.1529211287290000e+00,     8.9989474802594959e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.8509542324553479e+01,     2.3871833856702622e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.4603859254945483e+01,     6.1480107377418915e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6401384529068096e+01,     1.0717470165192753e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.7097249415183446e+00,     2.6046491407085114e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1734747344869091e+02,     6.5837197633962141e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5630124934634882e+01,    -1.6023041790009764e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -8.7126500374249627e+00,    -1.3549297719249051e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.5105447992418028e+01,     1.2560475972547891e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1506432663953614e+00,     2.3867036487740017e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.5690678989443683e+00,     7.7006074967356162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2212047694833222e+01,     1.6808558248190959e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.8077648678715143e+01,     3.0710536554030303e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1832438222091994e+01,     1.3400307630480125e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7884689966235257e+01,     7.8274500123422640e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -5.8326082991088590e+00,     4.8855218789244740e+04,     6.9537563198336031e-04,     1.3160024966877799e-06,    -6.2404247405057249e-09, } , 
      {   -8.9919924587838892e+01,     5.9573496437939066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1328869710942442e+02,     1.6170553916691466e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    6.2610401395085145e+00,     2.2741398044603600e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.2908957627388141e+01,     3.2280319232962516e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.2818437013120928e+01,     5.8151324520665048e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9246819346875043e+01,     2.7724103150826832e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.4549960831052431e-01,     1.5542718464348498e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6345430790369516e+01,     1.4750153395198255e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0119334204426190e+01,    -9.8411667682909865e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {    4.1353387391869752e+01,     7.6895671283198626e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.1901919152131395e+00,     1.4601626780345976e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -1.7712685601974769e+02,     8.3340693961510860e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1100020029782584e+01,    -1.4726059428488283e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.7322608805269315e+01,     1.5542790116019420e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.2672931403914490e+01,     1.4059712660179779e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.9621535195439730e+00,     1.7179040894606536e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.6562817647793409e+01,     8.6902321667548858e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3313902244640676e+01,     1.3806377101471848e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.1419928799530732e+00,     7.6546048016292425e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.6723423918237135e+00,    -3.1281992479243888e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    7.4861879729479156e+01,     4.6628615186509953e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    4.9956594270999251e+00,    -5.4812549171723549e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.1970332321661949e+00,     4.6627910554593793e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    4.2670102486628885e+00,    -1.6821676159390408e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    2.4060707187227475e+01,     8.2998257700100330e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.9475864087091658e+01,     1.4903629993778695e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.1183356174342056e+00,    -9.5661189251079850e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.8216200576278041e+01,     1.5687370733324937e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.5485980943647242e+00,     1.0264256354058904e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.6814186129491159e+01,     1.1785698058633707e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2727329725202061e+00,     8.3970548978213374e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.9925919108295407e+00,     9.8615449402212960e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4558444112086960e+01,     1.4787838742441872e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.6629924104936002e+01,    -7.2386707644139351e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.8405828322043760e+01,     1.1587943087168152e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5842710748925029e+01,     2.2073152640410903e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4464725860514726e+01,     2.2111588889788462e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -1.0187668774420521e+01,     1.0389710850665961e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.0987057893091983e+00,    -9.7587886926722840e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4190535826786274e+01,     6.3166872375577677e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.6844703165328944e+01,     3.2106837135106449e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.7966074552417368e+01,     8.4710303515079941e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9063882040232468e+01,     1.6659660365808362e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.6721935518908623e+01,     7.7756349008519719e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.6541079236208247e+01,     1.7285683720945071e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.4769295567782137e+01,     1.6655105333092142e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.0617187007950939e+00,    -1.1150218055287357e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3664806471808056e+01,     3.8197399736899868e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.5418854277790530e+01,     3.6587802156069745e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.7661840408945935e+01,     8.3290217921989624e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.8656134860629322e+01,     8.3283610572512898e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.3312526461661829e+00,     1.1175752790051938e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.3316249503005277e+01,     7.2146844932338372e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2625576550395605e+01,     8.9608017793799936e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2000108995541757e+01,     1.4539963979426466e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4828793572997379e+00,     1.5508615116649019e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2712984027379903e+01,    -1.1142353130038514e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.7008051943100035e+00,     3.1640521660161328e+04,     2.0534331944762507e-04,     5.9674261566729451e-07,    -2.8357721835458977e-09, } , 
      {    5.6433835755392314e-01,     1.4910037477969345e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.1551145149738300e+00,     7.7959697846790323e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3938010673215370e+01,    -9.9471157913959560e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.5550938345221411e+01,     7.5137990899921024e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6273651780261467e+01,    -9.8674859876001319e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.8448398135485064e+01,     1.4500755657870863e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.6354124279870973e+01,     1.8246328294534640e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.7710537926495160e+00,     2.6555455834834147e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9269453394152052e+01,     1.9519869372654830e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4483166841564014e+01,     6.8740015171909163e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -2.8595524893289323e+01,     1.4925263416498137e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.1984016655120939e-01,     1.5791158816735970e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.0860364890442762e+00,    -4.7293695475153045e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7874890859799887e+01,     1.8083511027502664e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6407121322254220e+01,     6.0221487647681779e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3772669398258785e+01,     2.6479847887175514e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.7724383434422233e+00,    -4.1992777397853943e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.6595135249598243e+01,     7.9122345053500394e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    3.6590280045955652e+01,     2.1518250928907080e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.9756240488484060e+00,    -5.0448382561961680e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.8036378002152297e+01,     3.1091542054762867e+04,    -1.0144900810142239e-04,     1.6481725902999783e-07,    -7.9204011082864902e-10, } , 
      {    1.0507580477010631e+01,     8.3363539077172791e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.1089733475240351e-01,    -2.9400327373781247e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.7598561375522497e+00,     1.0782731820358949e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1513996677977214e+01,    -1.7489696187067846e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -8.1450408825244693e+00,     3.9898219779474064e+04,     5.4306288134201204e-04,     1.0655913822578935e-06,    -5.0540856628306906e-09, } , 
      {    5.1753990435193771e+01,     4.8859821188924194e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -1.4792007017908308e+01,     1.0578596800193174e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.0415540133509093e+02,     1.5542408687776609e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1882504394351177e+01,     1.9701691646866660e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.9886777037333701e+01,     3.1645124059840793e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -4.0478258766650882e+01,     2.6145786643703785e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9440136531839386e+01,     2.3558606838544405e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.0502429418383620e+01,     3.2205068171790135e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.7561452613708308e+00,     7.7675743754826535e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.9462963036852138e+01,     3.7748816121793323e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -5.8358971933216679e+01,     7.8267340752783221e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.6331135807013695e+01,     1.5564233683945798e+04,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -1.9254083213614120e+01,     1.5521274896422119e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    1.8023653033641221e+01,     1.0340196939015408e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.7084758313202244e+01,     3.1023151773338948e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -4.7066332341741457e+01,     8.9168317919146812e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3358058829494550e+00,     3.0457529627798824e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    8.4829589685150619e+00,     4.6848330169521741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3925813261403590e+01,     1.6036075141247335e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2882512540213202e+01,     5.5417278446338451e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7629948382609683e+01,     2.2552697394845065e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.7599861282991984e+00,     6.6724963293413239e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4838435801782671e+01,     2.5066256367503687e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.6353527753128375e+01,     1.5446839712708061e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.7436851740757024e+00,    -1.4043601184253230e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.0806632081510038e+01,     5.6573980977260726e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6829749828661615e+00,    -5.8453675153638324e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.0496368539561374e+00,     7.2134412376838291e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2922088332586128e+00,    -1.8767602813933245e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    3.1571654557610415e+01,     2.3121486402865808e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.5072181729718359e+01,     2.2066152763568280e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.8550490891093304e+01,     1.7069342928239377e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4353660869757460e+01,     7.4247663170104552e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.9329644192362494e+00,     3.9641805050069615e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5242179813807240e+01,     2.8556530496670101e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    5.7322475003643909e+00,     1.7312862734328760e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.4689511381820253e+01,     2.1605428834507213e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.4494875435676580e+00,    -1.7635996606153458e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.2148826856170345e+01,     8.3225371062544746e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7437717711666551e+01,    -8.3348457431957795e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.2502482634983281e+00,     7.7002354226223688e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0273353360569697e+00,     5.1250050901564273e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.3970732968390948e+01,     1.1842374526844054e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6236811518806363e+01,     8.6394461970350494e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.7751317185564837e-01,    -9.4114646141022286e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.5953937404769533e+01,     8.3276159592156837e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    8.8101235606675942e+01,    -8.3297668902345686e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    2.9330453405574701e+01,     1.5495854951481542e+04,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    3.8209590674405824e+01,     6.3785070144223027e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.2209000675524251e+00,    -3.7664854955020530e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -8.7367273535765548e+00,     1.5602257607840711e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0393297366448763e+01,     1.5982175925702288e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    9.8346842350878578e+00,     3.1079475105972968e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    1.9811312914219545e+00,     1.4422092256522716e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -2.6613009763903140e+01,     9.2027939510631440e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3920610489676392e+01,     8.3914005477121700e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5810644309911172e+01,     8.3754879246382861e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2558702233294563e+01,     8.3292980050674360e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3552964438478185e+01,     8.3280848443828163e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7407592462821393e+00,     7.2075114000201374e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.0026482217215285e+01,     2.3872541595877061e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.1814643955118655e+01,     2.3870349833941109e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6879245504356987e+01,     1.7140704895967428e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0916872443543198e+01,    -9.2598484801291524e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.1284197453845032e+01,     2.2739544158875931e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.2653414442736445e+01,     2.2292802880537405e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.1165505797412802e+01,     8.9968407199962858e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.7315979487678739e+00,     1.4321084009102859e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.3851802671649232e+01,     2.5436970638081308e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    7.3018368965911602e+00,     1.6674695712184584e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.9242076063173492e+00,     7.7751799147013053e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9451263670129386e+01,     2.5494937153527186e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.3539444826051032e+01,     1.6586781725963403e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0726967951974189e+01,     1.1026878591150966e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -8.1252703496377698e+00,     9.9419097434189953e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1221095900408784e+01,     1.6728728470820723e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.8905282472230596e-01,     1.6468481640554994e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5042374025893039e+00,    -1.0151809662365271e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.4583968309883101e+01,    -1.7342996459284086e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1766478225033962e+01,     2.1535146874561688e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.4199351172088388e+00,    -2.6529310105352797e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    8.6636210418212531e+00,     8.6184229838841547e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.9648254919508126e+00,     8.3948147172495956e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.2937033223716853e+01,     2.0465312293142997e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0305061968670362e+01,     1.6048202999998339e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.8549271584561016e+01,     2.3078844819923383e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    8.0932678220185075e+00,     1.6335355185169661e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.2481186210249451e+00,     8.3349836607370889e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0659158050419773e+01,     4.9515382516684058e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5396261192784436e+01,     1.5501284656514537e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0706544407067870e+01,    -1.0660120252184593e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.7178319449781235e+01,    -1.1319015560331495e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    1.9377336147065272e-01,     6.1332756678101014e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5374907371398113e+01,     2.7093165733465106e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.3788722548249524e+01,     8.3417940765565309e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.0982185966515985e+00,     1.6684605321353487e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -7.6881563323791909e+01,     9.3809595490778884e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8073972674925656e+01,    -1.7981913190505340e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.8326687449332947e+01,     1.0530721230117644e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9355726548542709e+00,     7.6827518134039256e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2664878649226623e+01,     8.2964286892023229e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0775041147286849e+01,     1.6655079914613132e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -5.0696375510996550e+00,     1.6659685784287376e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9217172747834951e+01,    -1.7981208558589195e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.8869288711988283e+01,     3.2669537433046182e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -5.7913602979961043e+00,     1.4985768643027259e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.8554237390001873e+01,     1.4985111378074360e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.2670260225113786e+01,     1.4550273260681954e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5823979712081041e+01,     1.5483250972527207e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0102686889285093e+02,     6.5877907974303653e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3822226527536539e+01,     3.8807222328025353e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.1089038620845887e+01,    -2.0554756805361194e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1586959560087067e-01,     1.1499656302018273e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2523785138990476e+01,     1.0236156084992812e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1786950210283425e+01,     3.2370978991055063e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5579722600554096e+00,     6.6017547887377750e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3657827764937778e+01,     1.8464753343574761e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.5841163004949035e+00,     2.4905892180847070e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.4198699334168960e+01,     5.6611571806089869e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -3.1323243774241025e+01,     1.4967063486391533e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2933027141097327e+01,     8.3316343903567576e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.1938669846952124e+01,     8.3257484590934964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.5849886188885529e+00,    -3.8838090311045316e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.5585174140116074e+00,    -2.2130477831906819e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0195648882886822e+01,     1.6059728186714410e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.6685921780375875e+01,     1.6437828718245284e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.4857868987828841e+01,     1.4984728665083934e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    7.8261646427529703e+00,     8.3776434766927941e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.0178719979205873e+01,     6.9300743844411973e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.4448441098736353e+01,     1.3784276117583691e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0125080208087507e+00,     1.4134963721352167e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.1629543901041593e+01,     8.2637584020353825e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.0906663288654901e+00,     7.7163833480040685e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0108324138065015e+01,     8.3511259244322955e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.1140296863816292e+00,     8.3062569250179567e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6325420802095628e+01,     2.4968442247899718e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0674199140354169e+01,     9.0743929810084082e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.8683066032522053e+01,     1.2147567433139911e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.5415223826987472e+01,     6.6342066806946295e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.7170740478149824e-01,     7.7001601826235492e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0590324879715677e+01,     2.3183619552838743e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -2.5228497590025761e+01,     1.2393421733242303e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.8165539022202060e+00,     8.2818949248119661e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7535820616742548e+00,    -1.0280465689400808e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.3176656815280751e+01,     1.5630668276405137e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.5238780951368263e+00,     7.5864203188483107e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.2434407440958282e+01,    -8.8307448830934682e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.6568939989106585e+01,    -4.9366178135062919e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.7192726860374570e+01,     7.8665489771762332e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.3841686869318465e+01,     7.7955556717638374e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8279389670013924e+01,     8.8908757172461992e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7430256463499710e+01,     7.6961260839195893e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.1109242528402476e+01,     1.6381755850995003e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    6.3772220483719160e+00,     8.3254381689020447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0319561871291111e+01,     8.3739809502144471e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1646862916402085e+01,     2.9209445980670749e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.6170736646477017e+00,     8.3319446805482075e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.1765294782153557e+00,     8.3784723605525760e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.9572806113947330e+00,     6.6764254526248687e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4408607945737231e+01,     3.8732923322836308e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.6325694729750770e+00,     2.3623539861234152e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.0467530142005504e+02,     1.4644473237174083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9233625592392599e+01,     8.3448326240834376e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1852331651579792e+01,     7.7997630785022375e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9333637852147866e+01,     8.4713893810167956e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5996553486910818e+01,     1.8718048439371742e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0045560733017279e+01,     2.3982779403424171e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -6.5010155763198668e+01,    -2.0571522720911162e-01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2460513266824773e+01,     2.2118200289341759e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6448324363893654e+01,     7.1894702258717789e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.8526205695068469e+00,     1.5534986919284676e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.6434444593155373e+00,     2.4304797591918643e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -8.4795074058971309e+00,     2.4968025330820004e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -4.0727329138666761e+01,     1.1271154840463281e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.7276415033336185e+00,     3.1594371459719721e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.4073061210439839e+01,     9.1634352297761779e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.6801755311229890e+01,     2.7486186703158095e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8523354240849290e+00,     5.2555169725936230e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.0848756214262245e+00,    -9.6786065993439752e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.6062082918274569e+00,     3.6978817021943087e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9249391559939088e+01,     1.4190224542616974e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.9752531398724919e+01,     3.2892879074492426e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5641363881088047e+01,     2.7637660261732908e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.4984469853478277e+00,    -2.4950025857772969e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {   -1.8153021844321980e+01,    -8.8099453269986079e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.5771304753459546e+01,     5.9416999518521070e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.4455779843570902e+00,     7.3118340168953519e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.3250704467372159e+01,     1.6656319188025303e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.2844061822286356e+01,     7.1569749667940632e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.8957567177413225e+01,     1.6658446510875201e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3188110441446156e+01,     2.3227390386347903e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.4966766658165973e+00,     7.2062954945595493e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1661551859331185e+01,     2.1026998922530409e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.6028373141780996e+00,     1.1984785168755465e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6686821354655571e+01,     1.7854479442516538e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2841759678644102e+01,     6.5868598721426079e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0098939495338415e+01,     1.6036553430973767e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.3353636427921600e+00,     7.2691742009396485e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.6185158628270990e+00,     1.6860373359681249e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    1.9015940288311533e+01,    -9.4307007456690408e+03,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {    2.1960800188893199e+01,     1.2252342166055276e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.6310279668983663e+00,     8.9750326937579957e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.0414049373564875e+00,    -5.9518577024798446e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.5275928881486891e+01,     2.5829991607774293e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0250922629440424e+00,     1.3531248775893677e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.3989984983971866e+01,     9.5950877787399804e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.1862710343545144e+00,    -3.3630134133925094e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1322830780924455e+00,    -1.2539853377749032e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3678164959062485e+01,     8.3149002371297465e+03,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -5.1530866757156076e+01,    -8.3424826123205057e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.9095192699399739e+00,    -8.3003579352684337e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.6857939738751444e+01,     2.6733926191275081e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    4.3495046370295887e+00,    -2.2020271564683180e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.2516291226904883e+00,     9.3526498117798383e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.4686568353707706e+01,     8.3512350648535703e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0720997324856496e+00,     7.6030478986414073e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.8404999295607020e+01,     8.4329406764357082e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.7728319812186282e+00,     1.6459274742408372e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.2004980296789736e+00,    -5.8190691922019960e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.2111672663619210e+01,     1.2887208706700541e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4235694702840735e+01,     8.4596574088275556e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7357890251393613e+01,     7.7956500187014617e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0587286329642815e+01,     8.5700383228003320e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    9.9122954604158693e+01,     3.0441658967033945e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.4525493419011212e+01,     9.1669007915594666e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.8755284674576345e+01,     3.2517907048154811e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6890004193803833e+01,     3.6916388775950058e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.0830616407038640e+00,    -1.6736943315003558e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.5153844207797338e+01,     8.2042398250105798e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -4.8473252408291865e+00,     1.3435662830894229e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5301494842422150e+00,     6.4875395030976561e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.5530637757906023e+00,    -7.4988786518299921e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.7928084871560621e+00,     1.7478519170997072e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -1.1278684011534875e+01,     1.1610139657235904e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3659198773876874e+01,     3.6899920494214002e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.8059336016293010e+01,     1.5779912733093008e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.3090188269411641e+01,     2.0775427837006646e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6886313992696884e+01,     6.6965634877235962e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    7.0346359935132483e+00,    -2.6906804285552005e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7679299618291935e+01,     2.1666100363145288e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9075223849888395e+01,     1.9486081898313289e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.1844993269269779e+01,     1.1780017719815085e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7594358511823017e+01,     3.6885221921395227e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    5.0736230360360643e+01,     1.4897379423598770e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9575850930319871e+00,    -3.3330537070711125e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.9428909910946532e+00,     1.6259954638213283e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.5888096110409904e+01,     3.1204953286061653e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.2008674949414626e+01,     8.0289552001918546e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.2058227132711892e+01,     1.2679920678241630e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4666850259668722e+00,    -1.6848908107079769e+04,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    1.0793453818556756e+01,    -8.3552100413225198e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9930572151401901e+01,     3.8296941613510564e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.5612459054492781e+01,     4.7626523021411940e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4105730766088692e+01,    -7.3515154984847341e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.3130592578645386e+00,    -1.7772722754183422e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.2001525740359494e+01,     6.8144829401661154e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    5.9196223090038389e+00,    -1.7771300063249670e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.8959536563480452e+01,     2.2180767322341566e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.4263192246905039e+00,     3.3705942174569932e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7611656363657024e+00,     2.2909076886391813e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.7091010574764702e+00,     6.0420559818122765e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6869301474745122e+01,     7.2218302363581161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3105549815285883e+01,     1.4931513986678065e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8660461044899705e+01,     8.3413296146197463e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3949351900395909e+01,    -8.3160532348305060e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.3181139219678172e+01,     1.8123048812494984e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.1412875597086177e+01,     1.6113973976564735e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.2220391799277980e+01,     3.2165192250624223e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -1.8916988354469833e+01,     1.6867260947757422e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.1711794640707776e+01,     1.6866603682804522e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.6586898672282192e+01,     7.0258433076059628e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    9.3988175839582659e+00,     6.1060457057458316e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.8006052774212677e+01,     8.3303040888008836e+03,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -4.9575569265126703e+01,    -8.3270787606493686e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {    4.3752314555901883e+01,     1.5938076459716682e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2694179539154304e+01,     8.3169512506668889e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1124663048240295e+01,    -8.3404315987833634e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -5.5758254732725314e+00,    -1.3625344129412283e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.2225512021242622e+01,     1.8851215357668018e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.9392183856875882e+01,     2.2144051598885679e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5675576321742998e+01,     2.0921094939048315e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.7808550914749048e+00,     8.7588909027287864e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0057846594748035e+01,    -8.3280614180860230e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.2827715599005144e+01,    -1.0883201686397351e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0219523565927499e+01,     5.6949386081119537e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.6984131001311233e+01,     3.4100807638286482e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.9224288311815450e+01,     1.5550521661083239e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7066170657171853e+01,     2.5263047618982338e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2243336165489097e+01,     6.0504190674190595e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5715906590619966e+01,     1.7090947826942745e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6426062655392332e+01,     2.3809088907880116e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.5098831608890828e+00,     1.3952077876621144e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5310638070953821e+01,     1.6082795573050247e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.4022617037358728e+01,     3.8302201278142929e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -2.1814929982321935e+01,    -1.6844390410149917e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.3967484025113144e+01,    -8.0289704595599014e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.6129848428938068e+01,     7.7270445462642165e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2070571461982036e+00,    -1.1326622432536190e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.6693511718704936e+01,     7.2447157877685386e+03,    -5.5838726002546712e-04,    -5.8641608425966146e-07,     2.7669771221964547e-09, } , 
      {    2.2554916497699029e+01,     1.0671069499367413e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4255492032848458e+01,     9.7917428148217205e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.7606725352207206e+00,     4.7011163887884544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2210656230495772e+01,     3.8259756543928830e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.0334646230528900e+01,     2.3367091056701007e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.0993724778191005e+00,     2.0761059384868146e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    5.6935113097983194e+00,     1.3187719896168484e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3364137217072125e+01,    -8.3332338568091454e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.8041037427307714e+01,     7.7010884715957327e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2898917136201863e+01,    -5.1430540335118021e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.9106867817977637e+01,     8.3305967904770441e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7537351327063632e+01,    -8.3267860589732099e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    8.7105722842886379e-01,     2.9340271236216344e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.2927725050025831e+01,     1.4297159746815236e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0507787795117070e+01,     1.3805718199958539e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.1902890248694202e+00,     5.0010521633189610e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.7161135934240995e+01,     6.5362413687627341e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.1101449972297445e+01,    -1.0055960705909401e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.7903366132148697e+01,     1.3014930192207048e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -6.5604395212846072e+01,     1.5539695085584533e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.2681447806245643e+01,     1.5545813494783384e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7340297681466166e+00,     6.7308703964313427e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    8.4323566211897841e+01,     1.3642289506758900e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5163813837260859e+01,     1.0003814033596564e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1037684297327104e+01,     2.0413502016577513e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.0478052834808473e+01,     1.5537687218236690e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.1473404250356691e+00,    -9.1655382182316987e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    2.3208900289687836e+01,    -4.2745288009577844e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.4837594851085981e+01,     7.2142533213179913e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -4.2471651402190204e+01,     7.2138724095996749e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.5633674498718380e+00,     6.5665676107050622e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.4976451549360597e+01,     4.1379304853557987e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.7888557517033902e+01,     1.4986151356017686e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.7767687548934248e+00,     6.8084226574434661e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.9055253525426341e+00,     1.3618016633149417e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3529819078352574e+01,     1.3171862876784655e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0672802827624056e+02,     9.8778219679116010e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9062604998567981e+01,     1.6111543298204702e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -1.4569480008109657e+01,     1.8760625996971685e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0255943126391323e+01,     2.3846862551108152e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.6498184173448770e+01,     7.9997524888867329e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1495452227137664e+01,     3.4520309270061107e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4556870381408302e+01,     8.0004571208028819e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.6020344415395378e+01,     7.9063638790741461e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.2490581162303641e+01,     1.4920438162112987e+01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.0956152874343656e+00,    -2.5159575323603691e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {   -1.6030768823540001e+01,     8.8149971395778266e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.3965748323487457e+01,     2.6700144763772121e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.5581963898342295e+00,    -8.3367357027062590e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    3.4919685580259054e+00,     1.6603702406772187e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.5128377396501733e+01,     1.4831963411970375e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6688138763669520e+00,     9.6350875306436628e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.6084119133521924e+00,     6.6478884303862242e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.6135880371738963e+01,    -8.0312716593996338e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2823079183838630e+01,     1.4507686196470971e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.1420362403441988e+00,     3.1080851302012006e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.2482494864345615e+00,     1.5469157960783081e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.4086631351784302e+01,     1.2357116898970633e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.4559971836564518e+00,     8.3103480529825920e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8037867900248500e+01,     7.2171761112840431e+03,    -5.2978344676547697e-04,    -4.8369860964298404e-07,     2.2789636707915955e-09, } , 
      {    1.7874520954205096e+01,    -5.9241649407418572e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6391443603912069e+01,     2.3558625790116646e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.4929115183500123e+01,     2.3492502497592177e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.8000219385603465e+01,     1.3783818637547636e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7313816216597750e+00,    -1.7609698282991624e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.1366658509507523e+00,    -1.1216321407451344e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.1107584848933229e+01,     7.2083730373432491e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2655549150795574e+01,    -8.2565536093779501e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {    3.3463607716106907e+01,     1.5537064462068376e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3392856796477005e+01,     1.7352754616624650e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9980753810218079e+00,     5.4868744646626346e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.6968382448501593e+00,     5.9831848448895807e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    8.7458111353462051e+00,     8.4631998689808624e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7996852596344752e+01,     5.6078444236075884e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0351440237670104e+01,     1.5929216304277215e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0037486715079222e+01,    -8.3226311420024758e+03,    -2.7586509320370791e-04,    -3.9500194668399445e-07,     1.8691506661497048e-09, } , 
      {    2.8148744749203182e+01,     1.4415007819718207e+04,    -8.0781536623553585e-04,    -8.0980368410450235e-07,     3.8191682543084803e-09, } , 
      {    2.0143650175142289e+00,     1.5452582790849456e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.1627255687855566e+01,     6.9924464508382152e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3717295780441141e+01,     8.5471192790085861e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.1398041478191068e+01,     2.2597557548051143e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.9247691731987095e+00,    -9.6741993577999929e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.2387400936932034e+01,     7.2067331159469904e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.0718011866909496e+00,    -9.4559392223137984e+03,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {    5.4816249035050966e+00,    -1.2374275003828593e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.0553072487700526e+00,    -2.5131189390193435e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    2.8794804672201256e+01,     4.7349870460125181e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {   -1.2699887320021745e+01,     7.6799299603060172e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7557301014772776e+01,     4.7392078646931132e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    4.4236690235383854e+01,     2.2741084671495199e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.2072635682169476e+01,     1.4976266482002949e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.2307597336296510e+01,     8.8325474100561241e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -7.0835696521273249e+01,     1.6029081977955433e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.9025543084350200e+00,    -8.3080279084216163e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    3.4881772751993831e+00,     7.1566845410785700e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2426351827633724e+00,     6.5611721166123571e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6213422603094813e+01,     1.0185575830500187e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -7.1283795230770835e+01,    -8.3463167374500572e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.5712517345045836e+00,    -7.9422294106318659e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    5.4061032338154774e+01,     2.9948338115970568e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.6367934412505157e+01,    -1.8658474968747760e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.4894837848752381e+01,     4.7964337296441554e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.1979545200663281e+01,     1.4539587111022211e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.8016935283864058e+01,     1.7825629047169499e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0079875471894571e+01,     2.4990483501344446e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.2125317355221181e+00,    -6.6944743959802427e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3475454243360470e+01,     8.3130513448628571e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7267773924453007e+01,     9.7092595216536793e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.4568877385507250e+01,     8.3027819295748413e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.8596842048356649e+01,     2.7737221062026292e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.4766324385948067e+01,     3.0616460602914023e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.5611445847180214e+01,     9.0471037699076551e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1110375074650349e+01,     1.7725112070014129e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.0507957251631101e+01,     1.4903426078416052e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.7423493286345444e+00,     7.2197526935744163e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1810746005230456e+01,     3.8339386347724663e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.6959328625927075e+01,     2.3438093837899527e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1762831181237580e+01,     1.4376310619427375e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -1.3971017085990052e+01,    -3.3088781972005381e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0080661639824767e+02,     1.8540574636108453e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0587726588090714e+01,     2.2192499294530804e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.8750304486134901e+00,     8.2862734988387347e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -3.5088035863686482e+00,    -5.1223905172082923e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.7447391767287741e+01,    -3.4431910402790709e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.3751849851980966e+01,     1.3242111684313426e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5549059529160356e+00,     1.6570632324736096e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.4469739006689334e+01,     8.3443315045873951e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0346178861163558e+01,     1.9271458799264241e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.3662878720425912e+01,    -3.7237734769243652e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6300084647028726e+01,     1.7790427798843299e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4452926582967462e+01,     1.0163284390752870e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.8410652227555062e+01,    -2.1721692198678184e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5470172673295751e+01,     2.2283614092621498e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.6895530143822626e-01,     9.3534170602713657e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.5904842987894092e+00,    -7.0190391444641978e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.6946890859764974e+00,    -8.0199059275460257e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0521680678847540e+02,     2.0385682489347171e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.3191290631452404e+00,     1.5584223923853380e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    6.3685433061293715e+01,     1.5515145313105633e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.5193299789354207e+00,     5.9367841790642369e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3077190191049061e+01,     1.5325501358818727e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.0889072052013757e+01,     2.3817765272231019e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.8493000011554157e+00,     7.2801861579833030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2266118006808995e+00,     3.7668962660941470e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    2.8224493539367977e+01,     2.7879207672851084e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3874962250371823e+01,     7.8784188669067862e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    5.2008946599526968e+00,     6.6698664970251411e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1003588466339778e+01,     1.2494187737007554e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1996159673311112e+01,     1.6660364997724511e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0613317177341089e+02,     2.3267411003473026e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.3874203988344771e+00,     1.5548444118299540e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1427143376155204e+01,     2.4517777508155828e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0320061348176292e+01,     1.3136173489891044e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.6290793177297360e+01,     1.6654400701175993e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7540190523079051e+01,     3.1964850524281654e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.7017317674504486e+00,     9.4477292111604911e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2435306990428060e+01,     3.1260310216247992e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.1057636685701155e+01,    -1.7279798111737232e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1928918078358645e+01,    -9.6315395418442895e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.0099510495545196e+00,     8.3213616752132821e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5048405904831419e+01,     7.2805384739413785e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8238081058668314e+00,     1.0345611384680353e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.6493359886661739e+01,     2.0491511620707792e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    3.4034124115565163e+00,     8.0688405971438788e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.0537999430181035e+01,    -8.5797257835933215e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.9829440966930090e+00,     8.3360211742369702e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6092297692852338e+01,     1.6077761870701739e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1258429450719669e+01,     1.4304163328294077e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.0092761336867885e+02,     8.3237671061199326e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8629789586568410e+02,     1.6171569833767537e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9747280265829602e+01,     1.7791850489777051e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3252240374179559e+01,     1.2778489591133592e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.7467190336579654e+00,    -9.6404179302970279e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.5544125086963252e+00,     1.7807348479652079e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.6942452608302673e+01,     1.0530402426566628e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7744240467518580e+01,     2.2397577613452766e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    3.9333519767684875e+00,    -9.6179245608121546e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -1.7371207263482358e+01,     8.6598926175535798e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2101862406877391e+01,     1.5125611748516307e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9737647655516639e+00,     7.2213926149706749e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4986625797492451e+01,     2.4889171473612696e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.2804046668314754e+01,     1.1667572890963977e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4712691938595547e+01,     3.8874909933661096e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {   -3.1414588696567836e+01,     1.7285338118537722e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.0229260559325883e+00,    -4.0765757287260431e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.2751636187450930e+00,    -1.6744133374164412e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.3620305686087328e+01,     8.8385207065246104e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3403826527566903e+01,     1.3280229676393530e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8432425662600664e-01,     1.3199720481859731e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    4.1755782067138398e+01,     7.1499808316042381e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6697707318127268e+01,     2.6223012577467278e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.1045467867279708e+00,     8.4837959467493747e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1135567216753495e+01,     8.3114166929608818e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2134208493025696e+01,     1.6632799685649323e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6272895687916225e+01,     2.2707153758208246e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.1501142371278327e+01,     1.8859093851291756e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.7720131678963838e+01,     1.4914240989911208e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.7426885170162674e+00,     6.7772710622566847e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.3820639813626421e+01,     6.5861434222068256e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3622536499119029e+00,    -1.1352471217772691e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3301091276503574e+00,     8.0404451879475000e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -8.9283278817436837e+00,     3.1068235583601061e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -4.3318063401095436e+01,    -8.3322608511612852e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -1.5361583702403927e+01,     8.7812157120748616e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1201681943704553e-01,     7.3242855364204643e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.2313159608932889e+00,     8.3958714383243860e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.0800000694675347e+01,     1.6441035343193835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2146019332382721e+01,    -3.7894321731690934e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -7.6745471125394772e+01,     1.0189341306252272e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7878322118927787e+01,     4.6592214454149471e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    1.0648942294643813e+01,     8.3790661676265463e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.2914642544335592e+01,    -8.2922497620602735e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3906644252503845e+01,     8.3247623014415840e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4805594104470476e+01,     6.0968044738763556e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -4.7103098896066427e+01,     8.3906959157960191e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9060427904189723e+01,     1.4372768800731325e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0585043061024743e+01,     3.3312135866584322e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    3.2418122868922609e+01,     2.1471850263081735e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.3633685665425226e+00,     8.8834113500485018e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9027327566270301e+01,     1.5456013241255921e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.0492263392443419e+01,     6.6082971855443466e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.4542454104767003e+01,     3.3317395531216687e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -2.8883180628905400e+01,     8.4703257195918432e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5835689811779126e+01,     2.2291439747319600e+01,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.6841556530070498e+01,    -7.8454989556976361e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -6.9721051724422153e+01,     3.1570799535161459e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8771406565313814e+01,     1.4898698961622777e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -8.9971552684622552e+01,     7.7898835317398152e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.1903588300778367e+01,     3.2828797040637830e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.9725859217123336e+00,     7.8708232358113833e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0945843269139267e+01,     1.5115302467260819e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.9919802154210934e+01,    -6.8539548371031151e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.5974198271100505e+01,     1.8128614557319328e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.2901088270379351e+00,     1.4887785998553782e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3092917653028497e+01,     8.5070892953690673e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0585588141305500e+01,     3.7882078267370343e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.4240992258417844e+00,    -8.3312944487136065e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.4742562899743721e+01,     1.7808771170585831e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.1268866665107808e+01,    -8.4255942252878067e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.1000967011727440e+01,     1.6267964675792184e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.4734588832250015e+00,     8.6617979833054997e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5621925848672973e+00,     5.4983566832112656e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8853483783056319e+01,     1.4516039717138841e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    9.3986140303736541e+00,     6.7926559623555868e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0891117998452517e+01,     6.1067555876988558e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.9299480072676062e+01,     7.1492761996880899e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8933006204820636e+01,     6.1020641385087238e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.0944906671274840e+01,     6.0317467005567887e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.9457814591216774e+01,     1.4523297563001919e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.2436959670085486e+00,    -7.3177340709817699e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.2118814053863360e+00,    -8.5558729168315040e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.8793301432300119e+01,     1.5588602666476818e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    6.9550700562309231e-01,    -1.1352920755698024e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.0783543941597657e+01,     4.9397219366641330e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3675243117154707e+01,     6.4434400371120482e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3409332049390642e+01,     2.4245321860278724e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.4460418692721042e+01,    -1.6686233244797299e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    1.2164794212755790e+01,     1.6711063292128318e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.4851576710828738e+00,    -8.9591412480402942e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.5716607108081860e+01,     7.0065909424118117e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -8.5968996753187010e+00,     4.6609908150510437e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    9.0616885528345765e+00,    -8.1517788764625875e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -6.0716083694068907e+01,     1.1819832621712998e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6601213905120439e+01,     1.5754630679028724e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1336229249093266e+01,     7.0870325877721525e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.7512893182704630e+01,     2.2345260268967868e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2418561532384444e+00,     9.0092567615149837e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.8275725153811422e+00,     8.9150505392818413e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0748580457899244e+01,     7.8466282062643677e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.2369110076087701e+01,     9.0342845511950291e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4766243683990702e+01,     3.9349765655280375e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.1190468976126887e+01,     1.5629495339111993e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.7415022194023152e+01,     1.5609229898666505e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3685659515384680e+01,     2.5301542982856241e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -6.2359811642907417e+00,     2.4522295205085684e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.9487298314164960e+01,     7.2268647448140373e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7924998225014100e+01,     2.2852377129520581e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.3219029750516640e+01,     1.4913350824170442e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4417236526583386e-01,     6.3642082040351925e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.3308527887355623e+00,    -8.2930362545851567e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9424558484454106e+00,     1.2703027768667947e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.7439671242994926e+01,     1.5539772141909698e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2837583692704491e+00,     5.9640066509674743e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.4003575358574141e+00,    -3.7260746767640994e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.7575117803936337e+01,     1.6504579527265738e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4848773165289870e+01,    -2.4021764750847433e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1616432173885944e+01,     1.1553041275597334e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1542115476881403e+01,     2.3880082213292513e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.0264252518571826e+01,     5.9428539375730088e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    3.9299325653120249e+01,     2.2756264866110309e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.6401320237016623e+00,     2.2803716494345208e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    9.1147541329091837e+00,     1.2592450024072705e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9095545034100049e+01,     1.4146453709107813e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1407139137134830e+00,    -1.0657275338126803e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.6621665862618570e+01,     8.3167464508188132e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0634722377539097e+01,     3.2231315543148692e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    1.0747269018446740e+01,    -8.5539886340714856e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.5761165269763922e+00,     1.5550442496350419e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {    2.8653169120375964e+01,     1.5535066084017499e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    9.9219510957635855e+00,     5.1376243284788052e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -1.8218086661309183e+01,     1.5056081000005133e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.7815555923331019e+01,     1.6031152807188411e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4758546166014714e+01,     1.6798658114808168e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4509411103036747e+01,     1.2403517564795977e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3639410746536278e+00,     1.4914298288081203e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9680177342716295e+01,    -1.3506511271354015e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2317833581510513e+01,     8.4359302055992757e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.4858578189523634e+01,     3.7688184541589988e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.8602623431817555e+00,     2.2757369445175274e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.4708372065384665e+01,     7.9076321056705710e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.6416236008150005e+01,     1.6656254586903658e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.2121639604337190e+01,     1.6658511111996846e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6003254048651335e+01,     7.1273123407446756e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    4.3496292693925305e+01,     1.5591205690465300e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -4.7667671685149017e+01,     6.5853671386197548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6762587692847308e+00,    -8.3210289417329750e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5285771705666948e+01,     2.9120403731902981e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5474616146729968e+01,     2.3254158756585904e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.7697448768892201e+00,    -1.6266886686339660e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8043921170445806e+01,     2.3921109112343627e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.4528891515414516e+01,     1.0072731096429387e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.6399725586475931e+01,     4.1772325823250962e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.7751546720254748e+01,    -8.5075879682461455e+03,    -5.9531047605329682e-05,    -1.3964088456997964e-07,     6.6259485797240281e-10, } , 
      {    4.5387220485446932e+01,     1.6801670660114782e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4719227129018087e+01,     1.5718537587879766e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5684837323960863e+01,     8.1858775085468096e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.1850971858408109e+01,     7.2012609861036281e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1359972359109456e+00,     1.5539900800811742e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9941055357488633e+01,     1.5545607779556176e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7085112676092319e+01,     1.8544935436903066e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.5926155223804138e+01,     1.9153340192865569e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1534310145871292e+01,     2.3896028878710018e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2826259052848249e+01,     7.6879323201544703e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.2209153926593039e+00,     6.0840044523011038e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4005029584876709e+00,    -2.2116250922569297e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9626057645354241e+01,     1.8260539437722146e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.2112010196185707e+00,     1.6246312936401890e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6097620146160745e+00,    -8.7282713500178415e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.0263230129627214e+01,     7.5869726083807946e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.1930518921696002e+01,     1.1596812109106493e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.1981251271756207e+01,     1.5282289376113107e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.4426495723649175e+00,     1.3660461367363514e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0966146056791672e+01,    -5.8869151297608454e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.4922912856843816e+01,     1.7052705018982975e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0671346277869660e+01,     1.4583245512309961e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.7456234244901353e+01,     6.5032719872452490e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0676348727004470e+01,     2.8117124120519820e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.0119371005346242e+01,     3.3386862531705323e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1876622430640934e+01,     3.1098626491567375e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.5303638272827264e+01,     1.6889474220412030e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.6478170586047334e-01,     5.5471125564466661e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    8.6819421752648314e+00,     1.3189142587102235e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5913959842514274e+01,     2.9045152670730593e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9503973508819836e+01,     8.3478191603187552e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -4.8052980904777584e+01,     2.0290510657262919e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.8912162520030051e+01,     7.8268284222159464e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9422391105299415e+00,    -9.1278512753105551e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.3616851963005953e+01,     1.1806361574937215e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    8.5668132073023742e+00,     1.5279081741421145e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.0809314696306522e+00,     1.8040565519326941e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.8025554420395011e+01,     7.0720695410833123e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.1427603368662993e+00,     1.4306757682603848e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4665330675115165e+00,    -8.5459443560903565e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {   -3.8256034513995431e+01,     2.3672371205419524e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2204460750435663e+01,     7.5786979017415615e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2114869474165140e+01,     1.2172160072726965e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.4827991623461767e+01,     1.7745775586317635e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5257415734603921e+00,     1.2279894111656590e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.8005352850750782e+01,     1.6878929925828346e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.1513883104493019e+01,     1.3976350750230627e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2909578503331398e+01,     1.5699334658803489e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -5.5562848187892389e+00,    -4.6518172865031802e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2118811758558223e+01,     7.2120958914073599e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0721879534508223e+00,     7.1669938223340587e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4772693269564130e+01,     3.4780431661945237e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.4557047887196497e+01,     1.5361919179908933e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -7.0865400445790740e+00,     1.8564847509717936e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3690520001758884e+01,    -5.1200893173685563e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4720296053082931e+01,     1.4774727083213067e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.6511473737554660e+01,     7.2939163263106911e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4208634907178165e+01,     1.5529952410828755e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2941161648383641e+01,     1.6672101357874810e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3080772827837599e+01,     2.4691827932601562e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.6794706151388997e+01,     8.7857944282727076e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6713372185068192e+00,    -2.9355615904693673e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0525893143708021e+02,     1.8834820977682270e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.6456911833894623e+01,     1.6458308339960691e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1384910125994928e+01,     2.3214272475148442e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.5645294872853320e+01,     2.4446072857276562e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6271888143379115e+01,     7.2287813738833920e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    7.9749503295195172e+00,    -1.0381873158076255e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    9.3038663435338407e+00,    -1.0940099967553181e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.4931449136271240e+01,     2.6440032985277598e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.0543741593445375e+01,     8.7864990601888585e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0944498874904330e+01,     3.2035843829694051e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -2.7666256565052947e+01,     7.7541041483251211e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -5.3005505587765778e-01,     1.5087947754566727e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.7191680558847683e+01,     5.7321504860213854e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    8.6368138743431100e-01,     7.1672663655456727e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3608573520989866e+00,     4.1784631346534989e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1766396895467983e+01,     1.9612649398098889e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.5812483945278397e+00,     1.6681966013251185e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5052343223642623e+01,     1.7291723908890744e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -1.0997335702673216e+02,     7.6986528712011568e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6549424666923578e+00,     1.6127690713806922e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3304838355841191e+01,     2.0094712616559649e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    8.6665942793657145e+00,     6.7784439995498278e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.1191609895368796e+02,     2.0051621245875445e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.8544053760879649e+01,    -8.3807122514426246e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.5429595069702359e+01,     2.3870382053484136e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.7504925488706832e+01,     1.3664831411889956e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9554826328553911e+01,     1.6665009433076524e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.8496036138045273e+00,     1.6649756265823980e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.2535750010273459e+00,     1.3042641861201242e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.0199141112148222e+00,    -8.3870654918151431e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -1.0440156509766174e+01,     7.0024358568444113e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    7.0916117093753162e+00,     6.0596194042946272e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -9.9975254174327574e+01,     1.1762744723048232e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9325967988228140e+01,    -1.0286244106814955e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -4.9351098974506673e+00,    -5.5154114869012810e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.6811772415966914e+01,     1.4427766701408864e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.9045142791739615e+01,     3.7842116126321089e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    5.7219138175226185e+01,     2.3872509376334034e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.8580908085118066e+00,     2.8710172684284589e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.0735095974156312e+01,     2.3925126157587150e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1856855338267238e+01,     1.3173811760666313e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.2000539605429568e+01,     1.4940183854660248e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    8.6039098186306120e+00,     1.8735553668081423e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.7249758342744084e+00,     6.4564068594831169e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3318496881809665e+00,    -1.0028967753081259e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0447436305769725e+02,     1.0506129552957143e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2630595267307189e+01,     2.3382494908372151e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.6189837356967836e+00,     6.6387149018383598e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.8859551172260197e+01,     8.1850681923595312e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.4481258315727548e+00,     4.7319841204634404e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.2855327081060295e+01,     7.4519858743986279e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.3289511881044021e+01,     7.2070629886162060e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2647850145620026e+01,     9.8491073364832710e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2412996294385490e+00,     7.4831309083143533e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.6164690754801072e+01,     8.3243126821016904e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.0403186844466333e+01,     7.2109496196336186e+03,     9.2781703036018566e-05,     1.1077022985990678e-07,    -5.2374421970263164e-10, } , 
      {    1.2590061164740494e+01,     2.5554863282196122e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.7586845361099552e+00,     1.2143692460389040e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6539570547934157e+00,    -7.9460416560273125e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1436434177656723e+01,     3.2792396308277348e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.6331313069645706e+01,     1.3672787868185444e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3755597445926085e+01,    -7.7488352399609230e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -1.8365979045696612e+01,     1.4428484760426467e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -8.5363267489037113e+00,     8.8435877784718796e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.6389028546997302e+01,    -7.8137950709783718e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.3639302400298586e+01,     7.7625169896525176e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9513574249392768e+01,     1.4006753038754707e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6300495565177052e+01,     1.2804226740655426e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    6.0084495849850839e+01,     7.2542300833148192e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1683103561541426e+00,     7.9115488830574714e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6892734522575878e+00,     3.7544263992340693e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4354940624834427e-01,    -2.5224191280980840e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.2534360221581863e+02,     8.3286493989585015e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3633817028839985e+02,     8.3287334504917508e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7622972433855157e+01,     9.7925548608072004e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.9359034567599039e+01,     1.6033991037192576e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -9.7530470019538171e+00,     7.2160298395103036e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3923088288816103e-01,     3.2182088196278837e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.1440935108686064e+01,     1.6285025396060775e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.7141461387753583e+00,    -1.1033437546750694e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2333594115919617e+01,     7.6975389256217913e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.2225102338786193e+00,     8.3256000005826754e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1848107431422662e+01,     1.5542459027306266e+04,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -3.4771054838023090e+01,     1.5543049553061648e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    3.6698612919434815e+01,     9.9832585966651204e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7717440199778034e+00,     8.3317828488675768e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6817171183891659e+00,     6.9831834916502594e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.0987718778104307e+00,     5.9666839581220358e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9994522789270903e+01,     2.3314138148826663e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.7468513209446414e+01,     2.3314124721725209e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.0069371916192637e+01,     8.4486367821051917e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0000638223741699e+01,     1.6728023838904573e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.1677921954624139e+01,     2.2714157339687088e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.0116245568710482e+00,     1.6158442636907123e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.0792145714657835e+01,     3.2212756377956593e+04,     3.0029184874999460e-04,     6.3861597330472512e-07,    -3.0305703206157094e-09, } , 
      {   -1.3208322914511717e+02,     8.3320275281939685e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.3365274563603117e+02,    -8.3253553212562838e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -7.8952028438795082e+01,     1.1111895232871630e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3005155539862496e+00,     7.7989992890817957e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7882367146307416e+00,    -8.3452940063294718e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.2836338991491587e+00,     8.9567753527146360e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6456613970915527e+01,     1.6676118403118333e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.3386744665346679e+00,     9.1658371301345142e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -6.7392499548583430e+01,     2.0210880853467093e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.7905832973754574e+00,    -6.3014534262872548e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9818050822276922e+00,     7.9012396018019845e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9292977112228158e+01,     3.0456872362845923e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -8.0762571760051056e+00,     7.2324251819226056e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    3.9664045672413081e+01,     1.0689118442722785e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5507621485939101e+01,     9.2426088529610570e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5509242702158655e+01,     3.2428864799387040e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    3.1284894436573452e-01,    -1.8733821386430282e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.7950446362412240e-01,    -1.7870227186032182e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.6378079601701764e+01,     3.1103532788569122e+04,    -3.1778305369980066e-04,    -9.0543803084016953e-08,     4.1451569734865308e-10, } , 
      {    9.0668389351521128e+00,     1.5539056408481763e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.7081444290437187e+00,     3.2187517901311832e+04,     1.7658291136863651e-04,     4.9092233349151612e-07,    -2.3322446943455336e-09, } , 
      {   -9.6832756395021971e+00,    -7.4797338275066440e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.2331798664077738e+00,     1.5549365689737251e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8230154645412888e+01,     1.4854928128113615e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -2.1756487971810429e+01,     8.5539733747034370e+03,    -5.2978344676547697e-04,    -4.8369860964298404e-07,     2.2789636707915955e-09, } , 
      {    4.6373988585335226e+01,     2.5603565147861205e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6298866313653328e+01,     8.3290183222041869e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.3045718619699418e+00,     8.3283645272460653e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1382602371211874e+01,     1.5702789844364126e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9200785290578153e+01,     2.7726404350666569e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1500838100613144e+01,    -1.6766415338125607e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    9.7511619331204447e+00,     1.0109573537433566e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.5808073405209853e+01,     7.9728823329415709e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -7.2104228274533114e+01,     4.5177498068703790e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.4669457290093408e+00,    -1.6656279322057828e+04,    -8.6084593504181270e-04,    -1.1561351853419104e-06,     5.4686013601793429e-09, } , 
      {    2.0028222307284601e+01,     2.3444643317713129e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {   -9.5301666809133636e+01,     8.3278820276844108e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0629596126081701e+02,     8.3295008217658415e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9384034891003296e+01,     1.3290955266520315e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6036412528103000e+01,    -2.5011633913922769e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.8564787371577431e+01,     2.5114942973800717e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4081180902283133e+01,    -9.4744983875059042e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -6.5017087444591599e+00,     1.1804060375097481e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    5.4631061795618386e+01,     3.2025335503754133e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.7923603971750957e+01,     1.7021538164428144e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.6871397097786538e+00,     6.0415589119795977e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.8903855879841558e+00,     7.3975009997223387e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.7122767223141553e+00,     7.9072845216218793e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.5784659784259446e+01,     5.6406594486628746e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7200791523328164e+01,     8.4756090677058382e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4933553465885941e+01,     8.3241489926411068e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5069698393079769e+01,     4.7778939846220572e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.8536635048254744e+00,     1.4292698075692600e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.7032654928687926e+01,     5.1556732718341809e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.6700875715367516e+01,     1.1274784444825902e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.4345356599714702e+01,     5.5486789821591201e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -6.1913594308530904e+00,    -4.5922987094783821e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.3012593354964224e+01,     2.9609025905963463e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.3660011467984834e+00,    -5.9084324565942670e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8280495898604254e+01,     2.7126947160968070e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.2420968713073259e+01,     6.6650327682510397e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0185640785137485e+02,     1.4912715736659988e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0463415745656784e+01,     1.4414619219646311e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.3079147677000034e+00,     2.4097915861548590e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.0990935051519695e+00,     6.4650985775663485e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8992391096221120e-01,     1.3741942751659180e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1080493321568046e+00,     5.5437632093712491e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.5777672249579780e+01,    -9.6684114781105709e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.8310193862186019e+00,    -1.7719042311505353e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    8.8734499046934978e+00,     2.4358478034596712e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.7451037092471577e+01,     1.4879501816128428e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.6952076230235591e+01,    -5.0670719472693690e+00,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -3.7132911497175805e+01,     1.5555556169539163e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.5395994646078492e+01,     7.8583622415876562e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4708163234590021e+01,    -7.3385144651733763e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.7923942357340188e+01,     1.9618079103131888e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.4378874262553666e+01,     1.5570751409055820e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    3.0255859250859668e+01,     2.3378565785088609e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    8.9470915877281580e+00,    -7.8738848891078942e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.4011036237218878e+01,     1.0311829957177390e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6249216340386095e+01,     1.4722930923295018e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.6331526289421205e+01,     5.1894546993371441e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0113037728760270e+02,     3.8083933419486129e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.7063652660178715e+01,     3.1567869958662795e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.9622742126352506e-01,     1.3138227806200690e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.6204524952458090e+00,     1.6648194061534341e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1674630114259919e+01,     1.6963662770215204e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.4632321962236190e+01,    -3.8815078312647966e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.8461244694694985e+00,     2.3571724749743858e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -9.7948272859335361e-01,    -2.9154400977910714e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.6296739423713277e+01,     8.5503078393457436e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8032531819306932e+01,     2.1911271898600069e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9200912074186640e+01,     7.7708248555594964e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.4969317955998598e+01,     8.9303383031860358e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9866633656617747e+00,     9.4820171842822547e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.0395170966253707e+01,    -1.6627540509835289e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0700365035662106e+01,     1.7223870657631262e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    4.7235777059799453e+01,     2.5445830793520770e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0717983372931652e+00,     1.5162851231690427e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1763082645228399e+01,     2.2537278283805874e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3948189232857004e+01,     8.2464585969408217e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    6.2181341697242935e+01,     1.6856457358939813e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -5.8626001866580971e+01,     8.2979119166869168e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2245685423172930e-01,     4.8624960029006361e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8397618840118092e+01,     2.2208070036150555e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2548444590477413e+01,     2.6307123794079132e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.4068610474639684e+01,     2.0425497391245088e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9152828277474999e+01,     1.4923083203521845e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1777765256563875e+01,     6.0847157977679799e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2170016075020673e+01,     3.1968247605089360e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.5239600601770462e+00,     1.5890729163357412e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8422588585776554e+00,    -3.0459519802282666e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -2.4656636575714543e+01,     1.6448035220036465e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.3134971721109622e+01,     6.0941653978283393e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8666038024153572e+01,     8.3612543734773681e+01,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {   -1.0667626566395678e+02,     1.1428683479576504e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0781792466230662e+01,    -8.6402107630657974e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.2522094170384440e+01,     7.7719294346244642e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7427926596163992e+01,     2.2648258136301624e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9646098773602418e+01,     2.8272228642544072e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5064781498408017e+01,     7.9951737726888896e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6652594689236679e+01,     3.1569199722272482e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0106345222383574e+01,     9.8538759647946572e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    5.1362317438282702e+01,     2.6705075795928038e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6098169620599098e+01,     5.8369157431506264e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    4.0740534293573397e+01,    -1.0161732329173949e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.8756833884326326e+01,     1.9046161589917880e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.2517556152062292e+01,     1.4518634071448614e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2522549835079898e+01,    -1.4333038458795294e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.7149386667752751e+01,     7.7014884314089013e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.2271322455620819e+01,     8.3179250623869975e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9528932911387546e+01,     8.9306906191441121e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0815133124841214e+00,    -1.6828235114990137e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -1.6734562881000858e-01,    -1.0562864316045583e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.5104563570444416e+01,     3.2969710797284133e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.4735963102292668e+00,     6.0169662723925449e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.1911898186288310e+00,     1.5608877582708428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7824147093935583e+00,    -2.2135093750169481e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.9339576537378818e+01,     2.3870633668923605e+04,     2.4509445367736679e-04,     3.6118134428979317e-07,    -1.7100832973776660e-09, } , 
      {   -4.7551414799475438e+01,     2.3872257760894558e+04,    -7.2845194841432225e-05,     2.6753473364667522e-07,    -1.2800535622335079e-09, } , 
      {    1.6352996853124612e+01,     6.6504563498137213e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3265958859649032e+01,     8.3394577870632547e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4445175148198127e+01,    -7.1370092773237848e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.1969527308232905e+01,     1.6035108738799658e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    8.1386827262100976e+00,     3.2528723759514527e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    6.7708615224608160e-01,     2.0368038415175160e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.6375387273281802e+01,     1.5220726625903366e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.3068416406447085e+01,     2.9602466950363738e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.1338521907500741e+01,     1.0824648768140443e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.2967793331921555e+01,     2.9424634157381552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2200508306671258e+00,     1.5495957790270799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9391630027651228e+01,     1.5550921333169819e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    7.6854225657483379e+00,     1.5534587247198097e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2515339440151498e+00,     8.9286130763604106e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9204826403880668e+00,     2.9710142921642782e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5660773507012982e+00,    -1.7824980505927739e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.7351761138415643e+01,     3.1256360845907806e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.6353744971992715e+01,     1.6142675941819243e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2295576697968151e+00,     7.2134328588197304e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.5822765149782846e+01,     2.3762413226233730e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.8105500167430684e+00,     1.6861502610247982e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.3203671149774564e+01,     2.0745326900720560e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2056332008358000e+01,     7.2108096096357513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5567882718343299e+01,     2.4970326530659739e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.6500023492394888e+01,     9.1661961596433157e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    7.9749875887702473e+00,    -1.7597968910060199e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.6521078432607247e+01,     6.2689670337084181e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2326335762134891e+01,     1.2020833585157874e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.3885675974171026e-01,     5.5709856812744865e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.8465444274917626e+00,     2.9775732477640058e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1758961388370903e+01,     2.2379543929465442e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1245617131860615e+00,    -1.8165032378409534e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    3.2397133739350778e+01,     2.6451740237220110e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.9531556201629328e+01,     1.3119721002774286e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5076439418091887e+00,     5.9885992905545272e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.9021576498500341e+00,    -9.5235020773197266e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.2977944789090277e+00,     7.0713649091671623e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.6760176711418053e-01,     7.3319460337875280e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.6631656143208176e+00,     1.1687011838699086e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9526240019015241e+01,     4.0515710653159877e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.0170578107411703e+01,     6.4448979201552283e+03,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {    4.1132943364862911e+01,     6.4495003198346967e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    3.4471001345340547e+01,     1.5540787316132488e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5138958793752153e+01,     1.6075890043145248e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9860287344727084e+01,     8.2319943393896538e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4045168423482302e+01,     5.9577763278441953e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1279667445976941e+01,     6.5864599123294392e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3371973392758667e+00,     2.5280320615749195e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0309656741816534e+01,     7.2173161212819141e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3309489791799862e+00,    -8.2940656992564691e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8637734758380070e+01,     1.0837992641623756e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5013494893543456e+01,     7.2210664469376725e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -3.2431231772308593e+01,     8.9573580392794702e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4942034306464931e+01,     1.2772786863945958e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    5.2743264546718134e+00,     2.1460387083653794e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.5432875209847959e+01,     2.3654192783543851e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    9.3880204334089967e-01,    -1.1408424832456356e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.5023185951582160e+01,     4.1314870503745318e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -1.0859837093043115e+01,     2.9342220120098005e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.5895122620461223e+01,     1.4304188063448246e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.3165049248681555e+00,     7.0754733354417485e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4130535100904837e+01,     1.0411190244427802e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.6970518399351100e+00,     2.5125759685160439e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.1505681490999109e+01,     1.0091466650097467e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4803396935298956e+01,     1.6652315777502983e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.8766004757185989e+01,     7.2294927193502681e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -9.5110303934966083e+01,     1.1802559624946141e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1810870346910187e+01,     6.5651367179023146e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.9922901857819141e+01,    -9.6373274215337115e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    4.8912602871166044e+01,     1.4441632242189020e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.8121369762661502e+01,     2.6616033547160267e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.4781977353250287e+01,     1.5535424540672115e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8306572133149395e+00,     9.1776490219436073e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.2971818607735258e+01,    -1.7369026699168908e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -7.3941272385873731e+00,     1.5544721264235430e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.8390697600522978e+00,    -4.1190956464570872e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.2680136266663213e+00,    -1.0299316835231996e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.1101361555426292e+00,    -7.2136706835332852e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.1954645046591803e+01,     1.6959733646931665e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0298263012058193e+01,     1.7892668111563486e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0977096771338964e+01,     2.6058719267527122e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.7068570227765818e+00,     1.7473909992348665e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.1024371356550049e+01,     1.4994428824892317e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0427712096865051e+01,     7.7080987666253004e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.9025395302817365e+01,     8.1060240476949493e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    8.3661152637317038e-01,     1.5527797957045845e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1488083226447792e+00,    -8.2720244458117395e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8513500696081358e+01,     2.4447495548210311e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7543451149970274e+01,     1.7820698015013575e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5453742521590643e+01,     1.6672812703341686e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1192227643863167e+02,     8.3336157433303197e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5635552313219463e+01,     8.0884737772166518e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.5991151169970554e+01,     1.4830416300041798e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.5784057908331812e+00,    -1.8374581844240256e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    2.5017702277665435e+01,     2.4103335249453929e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.4780394127081070e+01,    -9.5601204367496393e+01,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0464220154449659e+00,    -2.1566206303083336e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4310257396659942e+01,     2.3818486093484014e+04,    -1.3020941618041093e-04,     5.8996976854219427e-08,    -2.8851262162828493e-10, } , 
      {   -9.0813720618430693e+00,    -1.4719265880175317e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -2.0359034842764657e+01,     1.9447586534439382e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.2906834830812699e+01,     6.3866807709237291e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0751660677961041e+00,    -9.1887879159112344e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.8708622062884409e+01,     4.8600921328683089e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3015201582779351e+01,     7.2088741568392925e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    3.4470829640774356e+00,    -1.0488058765237742e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.0508976941124084e+00,     7.3472310219454839e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3358608614319721e+01,     9.3031206407612372e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.7210314301159329e+01,     1.6023175210318292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6941035530893672e+01,     1.5238775569258740e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.6418240429639370e+00,     1.7278212267926738e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.2207652389422691e+01,     3.0322214261340210e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9198422246141675e+02,     8.3413376615003708e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.9041470597050275e+02,    -8.3160451879498796e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.3756881608381899e+01,     6.1209824970363843e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.9046761279859680e+01,     1.5885452032039115e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.2062428392094899e+00,     6.9503903166960181e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4907594510054704e+01,     8.9572226668267031e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6618636679101471e+00,     1.5338619270018184e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.9079427117501460e+01,     1.4102020837711461e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.4331898607482856e+00,     9.3871984877743289e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.5384999735074835e+01,    -7.6359494898669145e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.4418646422391005e+01,     8.5228217795166584e+03,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.2864442114964547e+01,     1.6845770126100386e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    8.9150640722325704e+01,     1.3348043165185085e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1628997161585453e+01,     8.4032541565736810e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.6925154003588215e+01,    -1.7212395929243663e+04,    -6.4234506317708347e-04,    -9.6967099545037180e-07,     4.5909916346348614e-09, } , 
      {   -2.9485859077054628e+00,    -1.5560536209362437e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2290501333719476e+01,     1.5550084039695803e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6408051459767215e+01,     7.2179507582526639e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.8773994908662978e+01,     7.2101749726649978e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.7963790923297589e+00,     3.8814049924755354e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5685370677225315e+01,    -6.4635652840091643e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.6268196893085118e+00,     4.0856428514305717e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7120994582048617e+01,     2.1314115908741303e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.7414719916478198e+01,     1.0900206603111928e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4721410188777774e+01,     7.2758826423233659e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.1690874863340142e+00,     6.0481178675793253e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.9775278641828589e+00,     1.5892004426367788e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.8702797743874733e+00,     8.4253885100605967e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0854405938828091e+01,     1.5398466479519431e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.6048229386066645e+01,    -6.9178432553096229e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.9053404255053273e+01,     6.8187406959261098e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3989062728540105e+01,     8.4672809547422112e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4500162809918054e+01,    -8.3406363986314391e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9042482446813640e+01,     2.2945718364538054e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.3566924153843850e+01,     4.5477585894883174e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.6727387112935546e+01,     1.5545357314172439e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.4750742544614106e+01,     1.3673250646086519e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1670178665899520e+01,     4.0686562918699776e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.1222291109431017e+01,     2.2376914097149260e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6709752453569614e+01,     1.5805822228243291e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.7990947110788873e+01,    -4.5435968438133041e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.6617528315440104e+01,     7.3723560706028029e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.9196009819507950e+01,     1.4476577131199003e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.3588130471221618e+01,     7.0080828260642702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8219090568373620e+01,     1.2178719028326692e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9168225375754481e+01,     2.5477311021183996e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.3643169321128195e+01,     3.2080692433940476e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.8195869026889518e+01,     1.4357843770972144e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.8479571753533222e+01,     2.3255757368185921e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {    3.3513766778471528e+00,     9.5826767625000284e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.9348428390604147e+00,    -1.6658546310957274e+04,    -5.2095954688107470e-04,    -7.5618329097378762e-07,     3.5792339635273712e-09, } , 
      {    1.5239760959773790e+01,     9.0357072421287830e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2028877671287674e+01,     8.3382943255562332e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -5.8283147898327190e+00,     8.3324505076080404e+03,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {   -7.3978312807467272e+00,    -8.3249323418422118e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    5.3036451434715364e+00,     7.7287173291362760e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2000497778708684e+01,     2.0994753275737326e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.7529197445944313e+01,     2.2681566094470407e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.8534053930497535e+01,     1.1445956476343357e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0572159036461130e+01,     2.2693538777176112e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5271925304715744e+01,     2.4108992299611669e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2473125774301870e+01,     5.9715598985876295e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.0752465731093171e+01,     6.2326024626791905e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8973566510980149e+01,     1.8213718984706564e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1834855935447841e+01,     8.1512802035855102e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    6.7639859416659824e+00,     4.8042092013433319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6257386804917431e+01,     1.8138796556010624e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4621747956528177e+01,     1.6121398477794957e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.7204195520481960e+01,     7.0225470836154345e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -4.9589639782958308e+00,     1.5404164760166876e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.6248133052553314e+01,     7.2111198998272030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8015628428714088e+00,     4.7434923412323633e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.8614589266236038e+01,     7.2170058310904633e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9289343610659806e+00,    -8.8363423069237288e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -6.2783373712375308e+01,     1.5264049976898572e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0351298524646737e+01,     3.1494277293576542e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.0347042524312483e+01,     1.9219628428405904e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.5831450459248630e+00,     1.5401479024826041e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.8588558772570032e+00,     1.5323200158978992e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    2.4167628988847262e+01,     9.7556601874790249e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -6.4107196029409597e-01,     1.6645385238902876e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    3.0563106282597900e+01,     2.4016062158050063e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1791576793442117e+02,    -1.1166640762833690e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2961707710039503e+01,    -1.5414309564622472e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.4848696752231101e+01,     2.2752407928473724e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.1401024672402098e+00,    -1.4274674356309729e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0998346060565547e+01,     7.1894434175440765e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7881093606619530e+01,     1.5543198916063075e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.1959589867800400e+00,     1.5542309664304845e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.4913020509000418e+01,     1.5469459784931831e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.2390957878447473e+01,     1.4691193668976995e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -5.5295912567535723e+01,     1.7046801023108321e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8901148366074157e+01,     2.2974070087008022e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {   -3.1185852599799610e+01,     8.2378220985573917e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    7.3325139137166184e+00,     1.7649152533485380e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.9345697240700801e+00,    -8.2584094647796064e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    7.6266645777945650e+00,    -1.6796577658451574e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9294668343288020e+01,    -5.5311439710488703e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2023823022985141e+01,     1.2227734267100172e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.3713562449909162e+01,     2.1502831817867889e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.1317188658151558e+00,     1.0106964639972497e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.2040402058141083e+01,    -8.3328102261787044e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0026607053314271e+02,    -1.1125930422492179e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0024288654247417e+01,     8.5213202097470676e+03,    -6.5116896306148563e-04,    -6.9718631411956821e-07,     3.2907213418990861e-09, } , 
      {    3.8549156121652828e+01,     6.8233430956055781e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.1975104863268182e+00,     8.3263645017110830e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.1058222995311151e+01,     1.7789546419826795e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.4044418247330698e+01,     1.5229934365391520e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    8.2607530434657320e+01,     3.0990336748283662e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.4570002546714354e+01,     1.5907389326466688e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8997139640365965e+00,    -9.6966402833313005e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.4285859182799811e+01,     6.3527698649004687e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6286189754171339e+01,     2.3914105530864788e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.7603394303521629e+00,     6.7036479245281052e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.1812482961076460e+01,    -1.2439959317687071e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5367262009697406e+01,     8.3265578561342118e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3031269795298364e+01,     8.1429187210231248e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -4.3731736053303205e+00,     8.3308249933160405e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2376701747259187e+00,     8.0181984229467307e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9752506559125820e+00,    -9.3062491459544981e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.4269746906034584e+01,    -9.4588620122798493e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.6585698541936473e+00,     1.7647544549633953e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.2461840414278996e+01,     1.3013981921713355e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.8229599556827708e+01,     2.6774094228749327e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.2993501745770532e+01,     8.1901018947080429e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3933746689436500e+01,    -8.3291900976022007e+03,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {    1.3879975509303346e+01,     1.7068437503130564e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.5761955242610526e+01,     1.5401734081806844e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.3314460296420833e+01,     1.5325516618186777e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2785493074013758e+00,     7.1463752598230822e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0503775947991482e+01,     1.9879102868006274e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3088261840049711e+01,     1.7949335090476874e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7403912645690993e+00,    -1.6455877661600662e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.0886915510922679e+01,     1.6282395563744591e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.4116933180310120e+01,     1.4910843909103496e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8416411070094657e+01,     7.7462322159724426e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.9964151276385287e+01,     8.4841245792258233e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7524308124322825e+01,     8.0284907382550655e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8825924022278194e+01,     3.9964955329126467e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.0113017663806449e+01,     8.3301929944947169e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1107312115489830e+01,     8.3271898549555353e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2609960666662374e-02,     1.5429343058885170e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.4238750346577305e+01,     6.3038512782924363e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5488313539498240e+00,     6.4430709337002872e+02,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9288052819745065e+00,     1.3110601138975599e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -2.9310402084458481e+01,     7.4344290426492753e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.2526764171341121e+01,    -8.0776723252249794e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -5.5609771076111613e+01,    -4.3096404403440829e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5192847585099198e+01,     3.1713104819946584e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5624132430631187e+02,     1.5544376195600624e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.2916427171291232e+02,     1.5541132384767294e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5582346074798167e+01,    -4.0340383102691931e+01,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -2.0304508326111595e+01,     3.3297492702133648e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.1217632237238467e+01,     1.5805272853225872e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    6.5450432105051739e+00,     5.0938534949696477e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.0614612470698361e+01,     1.4483482661104006e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.6849327366540592e+01,     3.0162954653748558e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.7620359686161144e+00,     6.7969203237911779e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3149783748877397e+01,     9.0531372443026012e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.3627904156666126e+00,     1.4574394832656619e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5646827054199761e+00,     2.7150641605082128e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.7503474142536906e+00,     1.6078738661839254e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.7823375909376828e+01,     4.2165346792943950e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.1499595732716131e+01,     7.2608593653719927e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6091112450476029e+01,     7.8288639392330224e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.1963299034378032e+00,     2.2625246137904269e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4887180748308719e+01,     1.6630722142865619e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.7569755032436504e+01,     1.9930419306619493e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.5679495765612288e+01,     1.7071073896717804e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3687120727189885e+01,     8.2808429001498826e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1581686540133299e+01,     8.0779856983333793e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9446317479263300e+01,     2.3393761239071762e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.4487230126413628e+01,     1.3641832026722845e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2699573537644488e+01,     2.7548319853131034e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.1006425590180790e+01,    -1.2292682549157966e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.0668022747652326e+01,     7.7650849090800148e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9416727553052581e+01,     4.5585172819754180e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.8686710387717810e+01,     1.1198974676304933e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.8973392619423585e+01,     7.3672889986555338e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0215601470162875e+02,     8.3191828708316443e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.4303312903535158e+01,    -8.3381999786186079e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7455346177180715e+00,     2.2818950305615723e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.2558140572090618e+01,     4.5247961260318789e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.5138306646993897e+01,     6.5850506484971384e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8100933149241058e+01,     2.6225416447499607e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4350986135987007e+01,    -9.6168210334197302e+03,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {    2.4562288354138211e+01,     7.4116121216971760e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1506263786904515e+01,     3.3157031344560070e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -1.8069697600365711e+01,    -7.6316917341069220e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.7823810422217740e+01,     1.1110491134885742e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    6.8807763576332643e+00,    -7.7844950562914346e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.1923125670242566e+01,     1.6033484491573779e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.0424588336165534e+00,    -1.6626204445935764e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    3.4637692280523552e+01,    -1.6642462411288139e+04,    -3.9508378323336571e-04,    -6.7738652338305476e-07,     3.2098544198900159e-09, } , 
      {    6.4004287283264478e+00,     8.4191398311414760e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.1467765269627485e+01,     7.5328186714087769e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.6839457885339719e+00,     1.6169639184295747e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.5778067498697965e+01,    -1.7186632707023546e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.6658301375788422e+00,     1.2649620891508253e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.3744336117108640e+00,    -7.6823596314783772e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.8128792850420625e+00,     9.7418869634357434e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1124480402136506e+01,     1.0942767374539117e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.8331986068034183e+01,     8.3329939306416454e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.0636547349308447e+00,     3.4415875228693894e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2634714098397717e+01,    -7.1419660326540125e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3870744558112497e+01,     1.7121397124555635e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.2319921791828250e+00,     1.7254523579940964e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.1708342211768237e-01,     5.3267867187864817e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.9973811190331091e+01,     7.7782826749969809e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4503452898214476e+01,     2.3250764713489814e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2127611996665404e+01,     2.1765047471607664e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.6626916324724714e+00,     1.4381056687792890e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4097033373534110e+01,     1.4452733629872764e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.2296785164607943e+01,     4.7508145184884233e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    5.9612860600560680e-01,     4.0040206390298852e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    1.6930587856204141e+01,     2.4577305560966754e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7380591453994683e+01,     1.8588706270412225e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    5.1680161905071365e+01,     1.4392429630473331e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.5136932829594961e+01,     5.4582429187750131e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4479762584062279e+01,     4.6235241900858884e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.1124423625683463e+01,     1.6919891936706044e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    1.7432706438581135e+01,     2.4474016150432795e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    3.2798857891906636e+01,     2.3059699995600036e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.6896979388526638e+00,     1.8270601168144123e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6965329100115678e+00,     1.6165028400834555e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.2058716377377434e+02,     3.1085000621829884e+04,    -1.6547030305845238e-04,     1.5986731134586947e-07,    -7.7182338032638137e-10, } , 
      {   -1.7271624389415476e+02,     3.1086016538905951e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    8.0043011928318535e+00,     8.7561432476482660e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7921820276688281e+00,     7.9106446509771813e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -4.6980599626454911e+00,     7.2157880922844588e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7064003411541258e+01,     7.2123376386332066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8148756005093079e+00,     8.7458339663927782e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9285277613017374e+01,     1.4908546651051998e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.3775989788575469e+00,     2.4277584595801527e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.9121544934157737e+00,     4.7648222492102150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3157988093663082e+00,     6.2729164040486103e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4797855565741909e+00,     2.2750783681247842e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.2327830309995388e+01,     1.0090043959163717e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1444001996185296e+01,     3.2499858104799438e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.7163220669339040e+01,     8.3330701673485619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6608251691217944e+01,     1.5570363267262286e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.1583656404043477e+01,     7.3623137728580332e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.1967960202068070e+01,     2.5287723684706025e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8312061584257002e+01,     3.7309409745643046e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.8510311937663815e+00,     7.0795544212030964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.1834419335292754e+00,     8.4895759721848208e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.5976056061134383e+00,     8.1882604573329018e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.0280891151994318e+00,    -8.4691223921173496e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.6868134410087052e+01,     1.5614099911554429e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.1124549717618272e+01,     1.6679674289197574e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {   -3.3688360660215672e+01,    -1.6635091409702931e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    1.9195242512537241e+01,     1.7436865833236501e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1813765737907405e+00,     1.4685719045385587e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.1168549939234271e+00,    -1.1431854010536388e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.6672139007273842e+01,     1.5542255617306879e+04,    -3.7514727503877934e-04,    -2.9908155987647273e-07,     1.4060566379538760e-09, } , 
      {    1.9019864374643650e+01,    -1.5543252963061033e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -8.5072079979769093e+00,     3.7670911544823124e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.3399693693045052e+01,     1.5503291704244122e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4937270040160406e+00,     1.6827117413383061e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.6296162204771917e+01,    -8.2980385024154239e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {    9.3680930174953936e+00,     1.3721109913514008e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9296472399453421e+01,     2.6225768763457683e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9195126149497415e+01,     3.1716445997729643e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1365212672191689e+01,     1.2073802682369065e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.0592453526398238e+01,     1.6684043556034885e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1117887139329372e+02,     1.0172068309485416e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0135312050743043e+01,    -1.3942318688942039e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    8.5640374903222263e+00,     1.5522090773880449e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -1.9805526284728955e+01,     1.3589916364083325e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {   -3.3677189243355080e+01,     6.6383720616664059e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8271486236393741e+00,     1.4744093112475604e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.5078067966127506e+01,    -3.3307525072313765e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.8303193899729275e+00,    -1.0334146132078875e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.0585261860243133e+01,     8.3275012424388460e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -2.2154778351157137e+01,    -8.3298816070114044e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.7211605585243701e+00,     7.7467415373374815e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8237195414510694e+00,     1.2745206836507703e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.3596748503474394e+01,     3.1220495314350082e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5134425849740442e+01,     6.5351131062161421e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.5955399342930811e+01,     8.7108956830583938e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4833306109604660e+01,     2.3348364978819049e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.2681965398282586e+00,     5.0474528291443039e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.6531020313517651e+01,     1.6740995393185025e+04,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {    3.4212016591028338e+01,     2.3424939642586247e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3274534509747418e+00,    -3.1315485192215206e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    5.7208338405207444e+00,     1.5923137069982327e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.8192236559417623e+00,     4.8766604977190154e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2135388147162569e+01,     1.3505571804298306e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.0614486305820272e+01,     1.1605216083935009e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5275808637133114e+01,    -9.4241323142831952e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {   -2.9723416595466354e+01,    -2.4970341790027785e+04,    -7.0636635813411345e-04,    -9.7462094313450016e-07,     4.6112083651371291e-09, } , 
      {    1.7957652910267001e+01,     1.3172221906293455e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -5.9725333397138343e+00,     1.6186794359377091e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -2.1396594889723108e+01,    -2.1955413120496451e+02,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.1059936292962448e+01,     2.8665249612237061e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.1653915730749027e+01,     7.1038401944972011e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.4413110257687785e+01,     1.5539501034360876e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5246267455107478e+01,     7.2468778905141189e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8041707002982683e+01,     8.4057450128601740e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.1766537748059658e+01,     1.9664479768957230e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.1709253224990945e+00,     1.1033041712662878e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.8883362625612179e+01,     1.5400415363401089e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4084044331631858e+01,     1.5594569401674249e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    4.6846075542945357e+01,     3.6980823125762734e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.2580468622234704e+01,     6.5868189418382408e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3321689717045255e+00,    -9.4625076536996457e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    5.5728949635755747e+01,     2.2145782567364105e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5142021487303076e+01,     5.9553701525840461e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9155858520158011e+00,    -8.6507190890057173e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -4.2172190244571951e+01,     1.5543008554568194e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6768945678458191e+00,     8.8768169164423616e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4922493983507692e+01,     1.5709348799963849e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0828456420803434e+01,     2.4500464645421504e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9870657256310377e+01,     3.0979570385945535e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.2079378888443472e+01,     2.9775924313065640e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.4714123034523794e+01,     7.1937127222637246e+03,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.7655288522650181e+01,    -7.2344130086539408e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6932294300396606e+01,     8.4065723182136990e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.0754721188434024e+01,    -8.3193037149073480e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.1696718232818419e+00,     7.7711534880359459e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.1510447127007740e+01,     7.7716008021480120e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.5289055832805104e+01,     1.6858888037299843e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.6470540493874214e+01,     2.3923260826399375e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.0093489744415614e+00,     1.5398137847042981e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.6054848527993375e+00,    -1.8888594952721269e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0468204094186373e+01,     7.7310367619892859e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    3.3814404409180618e+01,     2.6844761206913099e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.7047710216868399e+01,     6.2870083130798514e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.4478163454459647e-01,     4.5404683386200441e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5998348673644678e+00,     1.6464694130313714e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    3.7606860192232512e+00,    -8.6234966171024807e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    4.4504129784986128e+00,    -9.9138932146965308e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.6452667263415591e+01,     1.6804300492430968e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8853266566857901e+01,     1.4399739797507407e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.4363037509889292e+01,     9.0980192181477832e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.5932554000803300e+01,    -7.5593636313024690e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.9824492047448640e+01,     1.0586958282755557e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -1.0680072629225783e+01,     1.5542615314154265e+04,     5.7520816157977117e-05,     2.1164056435155683e-07,    -1.0070549784007282e-09, } , 
      {   -3.4615532569035565e+01,    -1.5542893266213649e+04,     1.8989705860473902e-04,     8.3746715274861239e-08,    -3.8959627413962306e-10, } , 
      {   -5.8075225706435930e+01,     1.4916834538113566e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2970370751821491e+01,     6.3739046147428344e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    1.3880917518680352e+01,     7.2109714413163838e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4703555767924469e+01,     1.5524972371005482e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {    1.0436053910711664e+01,     2.3249697085463271e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.3093480363286982e+00,    -1.2658039580069981e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6276051313258243e+01,     5.8872550080456340e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.2663836978161727e+01,     1.5546007546007040e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1193754855175122e+01,    -7.7193081741113747e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.5932841066077046e+00,     6.7866110425356910e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8136590640775413e+01,     1.8247500412209527e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.6901857488483529e+01,    -1.1633304249313696e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -3.1586061769128563e+01,     1.8383045996995232e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.5235141409575261e+00,    -1.9782227596643925e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6918679975184897e+01,     3.2821885769080029e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -1.1124217546461216e+02,     8.6212067978205159e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.4152584395214749e+00,     1.1283957973373766e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.2567267692717998e+01,     1.5163811072867049e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    1.0504944085356335e+01,     8.7113411934229407e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.1132010978465896e+00,    -1.5512101367874257e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    4.3123306841468381e+00,    -9.3036193136383117e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.1777680802716983e+01,     2.4866629568481643e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2488312860490911e+01,     6.0541781503019756e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -9.8087243548705061e-01,     7.1742211552332165e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.4961226246247239e+01,     1.6753297426926150e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.5055586602688247e+00,    -9.3248416807453614e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.9388790367863308e+01,     5.9754966387231198e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9789337959599106e+01,    -1.9643476706677241e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.0058115116844482e+00,     1.5444538512868327e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    4.4707329009644823e+01,     1.4770823472772820e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0622652466214937e+01,     7.7556199257902190e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.2659967753212218e+00,    -8.2265197157982457e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.1929222493992615e+00,     6.3106269548351174e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.9445564329307395e+01,     1.0733217908708395e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.4848756812766943e+00,     7.2171542896012816e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6116285751155587e+01,     2.2097331167082761e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.7065353012095787e+00,     7.6719979062693246e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3552007377146630e+01,     4.0986283883864999e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    6.8586027180412952e+00,     2.0941784178526133e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5006122114412470e+01,     2.2443978279278112e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.8421777055928942e+01,     6.9035251889338888e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5354288545280488e+01,     2.4028026083528614e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    3.2993755158762113e+01,     2.3514507372558797e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.0007727242503028e+00,    -1.7477206216339193e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.0971178476664622e+02,     1.9129067319256086e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9528733131828076e+01,     3.2827733379212885e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.0510017679987136e+01,     1.4539474678800801e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0371120717873275e+01,     5.8925146726780022e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -6.6907243245900183e+00,     1.4983362467767111e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.8452546466711588e+01,     5.5486942415271697e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7151756290416667e+01,     1.6617042466347561e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {   -3.7439909601629111e+01,    -6.2794855553674165e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7760254682026059e+01,     5.5824756690301328e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2756026943079796e+01,     2.3788962421741046e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.9190797883104516e+01,     2.3869496831027423e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.5973638547988465e+00,     2.3873394598790746e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.6131590763247168e+00,    -4.9853196791713708e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.0066995655580380e+00,    -4.4598059111805833e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -8.1198432172320096e+01,     1.0483587647826087e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3885952036271171e+01,     3.7593574313899881e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2684335318018213e+01,     2.9829604755743709e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.8079650498054205e+00,    -1.1224953328594092e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    6.8106752018549784e+01,     1.5481363885602668e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    8.9221913315659975e+00,     1.8826607458712919e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1269106375004807e+01,     2.8784154629030509e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.5151109734953152e+01,     6.9353585521734458e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.2733714204679316e-01,     8.5328111855228544e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7039723831738627e+01,     7.1625164591813937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1203484326450209e+01,     7.7610678638364916e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7404904613236212e+01,     5.2737831801358607e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9761175824123750e+01,     3.1879385910530596e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.7413140795394867e+01,     8.5996312951371383e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.2686621682962908e+01,     9.3075686839785049e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.4322481383129357e+00,     7.8243215071490013e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.7027147639956425e+01,     1.4913634659152936e+04,     9.2781703036018566e-05,     1.1077022985990678e-07,    -5.2374421970263164e-10, } , 
      {    3.4730052943594387e+01,     7.2147257237235426e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1189775837610071e+01,     3.5809595607460469e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.1777006939773905e+01,     1.5496905913891100e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -3.8668072897084915e+00,     4.3389631507704535e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -9.5021246328179885e+00,     2.5054766379748042e+04,     4.2384419131235424e-04,     7.8320680555883315e-07,    -3.7133819090903800e-09, } , 
      {    2.2160919534628160e+01,     1.0203126100978485e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2364185777994479e+01,     7.2134000071941191e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0076087938112899e+00,    -7.3596329400876584e+01,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.5358615323353376e+01,    -8.3155887728937232e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    2.8486491895934478e+01,     3.5498079655592664e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.4091430990226641e+00,     9.9525904149873659e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    8.3671458136084453e+00,     7.9129715739912244e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9542638995154260e+01,     2.7009054516853255e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.9453751911235507e+01,     1.4987517553334514e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.3132031832804930e+01,     3.0435871087344523e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.2675401720376726e+01,     8.0825880980128495e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.2195633222084091e+01,     2.5105518979869114e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.1947379334754352e+01,    -1.2880953977543159e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.0563970243984819e+00,     1.5524720606196632e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -7.7581668113033615e+00,     7.1984227855965637e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9204931227550770e+01,     3.2828092408721677e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -2.0241045253576431e+01,     8.3332091216549761e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.8105891489171952e+01,     7.7796613275173322e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0393838935608670e+00,     7.5710354187494104e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2823084875359276e+00,     8.7500983278283729e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9789142348951337e+01,    -1.4989796609770938e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.4314508663981982e+01,    -5.5360750032047890e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -7.9437658483658907e+01,     7.0720829681847663e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2878067877238381e+01,     8.1896831482666548e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.0253660791447867e+00,    -8.4676997011835974e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -6.9758572343708209e+01,     5.9729897236561756e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5437496025400058e+01,     3.1397027918708591e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.1334712295056706e+00,    -1.8127659882172111e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.9406948890224722e+00,     1.3596052988672503e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.0104713664971797e+01,    -1.1392117230672263e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.3893074382600034e+01,     1.5539811324552329e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1348778965116296e+01,     2.9240400727129199e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.5371046766131727e+01,    -8.0119031780202549e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.9836967252174411e+01,     2.4879656358448163e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0340994908785973e+01,     1.5006769918528662e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.8649519147724330e+01,     6.7102670580542954e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.1540916770985907e+01,    -1.7319155175894725e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8313829705627839e+01,     2.1070914592897307e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0361586847639970e+01,     2.4958844312111109e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.2385453347081182e+01,    -2.5001493385214570e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -2.4274343308854629e+00,    -7.7553776367674936e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.8327482203641789e+01,     2.2753972241585001e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.1524588550514725e+00,     7.1911686443697017e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8330041179005832e+01,     2.7949176213600722e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0764006999918383e+00,     3.8346084925623768e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.4912557918497125e+01,     9.6138234531932767e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -3.6475595809386064e+01,     6.5846916189883377e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5786296818537483e+01,     8.2867989800435134e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6118637339927460e+01,    -1.6884205312047830e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -7.4323269147606386e+00,     4.2276001699193148e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9241855276367701e+01,     1.5542500025799724e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6196047004566118e+01,     3.5173452631638225e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.1638134832582390e+00,     8.8324872321296625e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.3186894154845472e+00,    -1.6850071568586794e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    4.7270173146275285e+00,     1.6919901412492167e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -2.3703005540445808e+01,     9.3884526157048640e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.8574028083882190e+00,    -8.5156062120696220e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -6.2798377867158557e+00,    -4.5121166161500751e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -4.2953995388346392e+01,     1.5740097831195095e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.6200525296385493e+00,    -7.1599902408552180e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    2.4136883463600562e+01,     6.2081534217594926e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7215287456030541e+00,    -8.4360691816289673e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.5530636819158673e+01,     7.1173657801233603e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2759164293956434e+01,     2.7832411172937926e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.3932411297360936e+00,     7.1765917581226695e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0417411073422223e+01,    -9.4382529120441504e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.0097084724015039e+02,     1.5545697255815589e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8248002865339732e+00,     7.7873766534164633e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.0019450523085899e+01,     4.2679698453580568e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2313004011137304e+01,     2.3278083018873531e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -6.9451075815622971e+00,     1.5540083227378067e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.4022160174961861e+01,     1.5545425352989851e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2717913472687384e+01,     2.2762717209729210e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.8295058908232697e+01,     1.8048984753523502e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.3758501982701091e+01,     2.3520111474670499e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4831086942215133e+01,     2.2759662069700582e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6551295901914791e+01,     1.5146336239683262e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0048849915486965e+01,     3.0627992076495207e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.6781051927276174e+01,     8.3705838694067388e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8855530089622476e+01,     2.5607822903621196e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.1617815842483648e+01,     8.1140683256760803e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.5202755443717948e+01,     3.9401082093893587e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.9584990957626438e+01,     7.2599112417516908e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    8.8283663561290027e+00,     5.2255288389138641e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.0099509225980256e+01,     1.2540588402832898e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.0590364970885183e+01,     2.5870331990876984e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    9.9278281666869646e+00,     2.4400426505085539e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.8033700988881407e+01,    -8.9412665256230775e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -2.4744962700492728e+01,     1.9741832876013199e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5267925062105155e+01,     2.2756593498586757e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.7891885673010595e+01,     2.2757040812698826e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.3383191320049050e+01,     1.2645363135748263e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.1684276054181097e+01,     1.2721571983795166e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1937335131055178e+01,     2.3248069162019463e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.5341906429333436e+00,     3.0155508227274086e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.3005557716236858e+01,     3.4911411969878736e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9899236466671709e+01,     1.6654729333652445e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.5604640062858905e+01,     1.6660036365248059e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0364742878732642e+01,     6.5771187820298601e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -6.6962347409964664e+00,     1.5211866470463905e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.3206193406391886e+00,     1.7316868838010279e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.7974451407567997e+00,     5.3019176384189668e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.2583131364511178e+01,     2.2820095534109470e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.4547763568002573e+01,     1.1004587151403652e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2618414094180302e+01,     3.9739805304390347e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.2281425392065707e+01,     1.2991966203489710e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {   -1.7932097674476609e+01,     7.8374394183484583e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8842214314637499e+01,     6.9699969061820593e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0404577546837325e+01,     1.9928118106779755e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.9109552629273239e+01,     1.4912058872163265e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0073379691512741e+01,     1.6672339182588366e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1354113506440171e+01,     1.4268090176737498e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.2922179837255214e+01,     1.4403852857308184e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.9854351095636495e+00,     8.2691109279606426e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4282838012261969e+00,    -8.3900156081104569e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    4.0984295662081564e+01,     6.7898968618501203e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.5438476013036073e+00,     5.3265102780060197e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3714111904411446e+01,     3.3837135089523661e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -4.3677447632598589e+00,     1.6642426516312138e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3610831035094296e+01,     3.3472500053240939e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    2.3765154213036052e+01,     7.1938375492102996e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.9521263112290814e+00,     2.3649277010755985e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.3133279098466247e+02,     2.3873176683387512e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.3197572087800111e+01,     8.3536126967886285e+03,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {   -2.4767088578714123e+01,    -8.3037701526616238e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -2.6581737241730892e+00,     2.3225089186508168e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.6730539153744139e+00,     6.9158596351222229e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6722901125739433e+01,     1.6573271632838398e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -6.8200071137062261e+00,     2.3844785008324452e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.9385969682913540e+01,     2.6734279326851414e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0479166032019229e+00,     3.1928432703191451e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.8671299000563693e+01,     6.0326987565401341e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6295480391405218e+02,     8.3199605794040726e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3283385525657856e+00,     7.2656092717362717e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0123152561732010e+01,     7.2297029453211017e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7840690153651444e+00,     1.3606035375129281e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.0754875664537380e+00,     1.4825429191524814e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.3638583615443487e+00,     7.8335102950649125e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -5.7686077062115828e+00,     2.3835363087566384e+04,     2.6936461440465505e-04,     6.0169256335142287e-07,    -2.8559889140481653e-09, } , 
      {    2.1734726626165404e+01,     3.2515605848315074e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7394910611584169e+02,     8.3374222700461796e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.2340748395968980e+01,     1.4912921451887200e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0751354253171692e+02,     1.8206513392636727e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3376160833113708e+01,     3.4322976342439579e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.6055946277654126e+01,    -3.4884868615718078e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.9312989067966299e+01,     1.4391724998557180e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5478497121305654e+00,     1.6997444197718167e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    2.7885856963295645e+01,     1.5446057204848488e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6145456986233029e+01,    -1.8623808500055734e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.1321992365581877e+01,    -9.4277779557029899e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.9011942058423465e+01,     1.4011694534716888e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.7625161451674265e+00,     3.7394183816507430e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8266203462328022e+01,     1.5595713911609031e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    1.8436005141578931e+01,     8.8118838937526161e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.2795388906882756e+01,     1.4141024004074821e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    9.4382336744827935e+00,    -1.2182398306746322e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.7659649039116871e-01,     1.3104446378697725e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.8584908652479548e+00,     2.2343126108375240e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1154368537898563e+01,     2.4623706226792096e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {   -1.2385498427180792e+01,     1.6097991027033404e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.4027833912769196e+00,    -3.3084610674840596e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.6719117879033973e+01,     7.6932779342265758e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1587605462927050e+02,     1.4427617772379634e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {   -1.7742735303865854e+02,     1.4428633689455697e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.1451248452781552e+01,     2.3005221682194802e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2946215272973681e+01,     8.6928619990710686e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.4159300108031060e+01,     8.2626138921006004e+03,    -4.3483491746310743e-04,    -4.4182525200555343e-07,     2.0841655337217842e-09, } , 
      {   -4.2339627386635264e+01,     8.7241430727781444e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.3909143877549276e+01,    -7.9332397766721087e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0310395029394716e+01,     1.4918855932307486e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.7890226244380010e+01,     5.9057022593133115e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3854652141768305e+01,     1.5589550790097119e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9703030648825923e+01,     2.7519968130661058e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.3049186635247153e+01,     1.1946062245290323e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7137913337341789e+01,     5.3675154497386566e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    8.9490381271713879e+00,     9.1235522890608554e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4727550123085791e+01,     2.0487733586252638e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    6.7977352353902027e+00,     1.4390654623581502e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5873984542501450e+01,    -6.7439883076307979e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.7538229740456078e+01,     2.0132027333258342e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4988315852168807e+01,     2.2519270570940207e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2006140146505107e+01,     7.9974749725286238e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5055579287701995e+01,     2.3530239856706390e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.6438140565595458e+00,    -8.4592033071623209e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {   -2.8864702800519275e+01,     1.5525033979305035e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.5941755393918847e+01,     1.5560474601062882e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.5251442119984034e+01,     1.6656401187634525e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.7815253062581419e+01,    -1.6658364511265980e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.8157068084626378e+00,     9.4593194923159044e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.4529988600257106e+01,    -9.5394582185233521e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -9.8612797333281776e+01,     1.7618020709489097e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5426266509664345e+01,     7.4840363334231661e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.5972850688509315e+00,     9.1419558360143383e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.3776882961462256e+01,     1.6102146112600807e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1933497512911789e+01,     8.3204072422997724e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.1780945495210887e+01,     2.3642718055156256e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -1.0898543070339750e+02,     9.2495143828660584e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.8004484939702195e+01,     1.4913742073222265e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.8124124357098545e+01,     5.9296997422597296e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    1.2259732199599172e+01,     9.4057807406716638e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0767827516683568e+01,     1.7717408604479617e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -6.4430940378711963e+00,     1.5011143790473887e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.1882148271460462e+00,     3.8264976869781339e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.4380465948145865e+01,    -1.7216998328923135e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.0210985857464562e+01,    -4.0876871320862620e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.8353289551433374e+00,    -8.3282992427995796e+03,    -7.0853318440046442e-04,    -9.0572407091202402e-07,     4.2822622825043086e-09, } , 
      {    9.5086838834416323e+00,     1.6187061383553988e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.8114280118132370e+01,    -9.0704102410694577e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.7899457229537880e+01,    -8.3931410339058457e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.2101666903340920e+01,     2.8510145090212809e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4726287975251855e+01,     2.4011165336834430e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.3355110247025515e+01,     1.1989833078799482e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.8792017941753141e+00,     9.3680536634509826e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.6897815141656555e+01,     2.4018966516670844e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    9.8595206405961644e+00,     1.6621274498933246e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.1048375892731976e+00,     5.2761710131110121e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5184819279262385e+00,    -1.0955885285738572e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.0353129138918483e+01,    -7.2651172897286533e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.3984080467177655e+00,     7.2342881817073658e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4795425561099428e+01,     1.4901292509825136e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4236950849456118e+00,     1.3580279274035203e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4974630101573512e+00,     7.7816864263474672e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.3127853964730782e+00,     1.1615570259658081e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.9869680568134074e+00,    -4.2768300007975186e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.5329322432245320e+01,     8.4086735444790265e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.4517587919845063e-01,     1.9975017445482172e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.3271728790663087e+01,    -1.6189832174743710e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {   -1.1637130578328282e+02,     2.0345867587449262e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6344102264309306e+01,     9.3549510116195779e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6866155681731932e+01,     2.4897705301803540e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3509939333017552e+01,     7.6496817313678357e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    2.0538930629975177e+01,     8.8047392790216472e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7494576062709140e+00,    -8.3290612876912819e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.1259274985025915e+01,     2.6066861471824112e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1374114092884454e+01,    -1.7406349444721018e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -8.6124294702919677e+00,     4.6080311936259732e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -4.1647503046245433e+01,     8.0111386119895069e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.2958630785304628e+00,    -3.4120585820269753e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    9.9049896092753507e+00,     1.0770893159465368e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.0689217153032693e+01,     8.2766705980076276e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    3.9463282719412739e+01,     2.2304292868293058e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4813066665386323e+00,     8.5562650987570541e+03,    -4.6576215180844699e-04,    -4.7874866195885568e-07,     2.2587469402893278e-09, } , 
      {    2.1197507900556214e+01,     2.2772816663967271e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.3350694567797055e+02,     1.5527564577853604e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.4332722776531270e+01,     8.2487093049712257e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6716741043032506e+01,     2.1285585447773716e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.6909807976447254e+01,     1.6199447808639758e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.8440478468504438e+00,     1.5376159780404065e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.2989503345760317e+01,    -6.9274598114930632e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.6755732575856058e+01,    -8.8863580203163492e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.4011810611236953e+01,    -8.8856533884001965e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    5.8128115278188908e+00,     8.9208906592536623e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.6761255340925860e+00,    -1.1683090019443612e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.1975923818830736e+01,     3.3929955372746590e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.3310873328161748e+01,     2.4124781273617009e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.6910661880284309e+01,    -1.1150167010598293e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4566066781095955e+01,    -1.1142404174727581e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.4721249007492617e+01,     2.4816860350210514e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.4299987990266544e+01,    -2.1272872460226372e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    9.6258222347782070e+00,     8.4242773697245757e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2273652648725377e+02,     1.4916177673616841e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.0764650724273359e-01,     2.0098970372319640e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6321508806597162e+01,     2.2004556643759777e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -9.0742555808776153e+00,     6.8831750457387789e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.0306319836029736e+02,     1.7912267051062910e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0967365109979760e+01,     8.7631468291271085e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.9265923963508023e+00,    -8.3360141697452946e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1402123810186939e+01,     1.6737577117587405e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4330724168330988e+00,     6.5856068633560753e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8826404658600111e+01,     8.5906750307761395e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9905697740472490e+01,     3.2726273015752875e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.2164019847489312e+01,     7.5023091022364588e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.7788634021504429e+01,     2.2756806633438984e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.5371074288649179e+01,     2.2756827677846599e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.3846031469272262e+00,     6.7778299130602245e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5666953882608254e+00,    -9.9683381585030056e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.9058662345486788e+00,     1.7796959061811233e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4044608733584120e+01,     4.3670015001320216e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0874859756982058e+01,     8.9876519320803727e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.6885606093652452e+01,     2.4435763576021072e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6022656046224711e+01,     8.6576855962495847e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.2521547869974381e+01,    -1.5594646404495707e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.0636414646301535e+00,    -7.8401034616049292e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.7337998786365784e+01,     2.1185066567045578e+02,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {    2.0101717004605348e+01,     2.5692378746112161e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.4675823419105768e+01,     2.2916076763234440e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.6893301975190642e+00,    -2.4992633229775103e+04,    -6.4234506317708347e-04,    -9.6967099545037180e-07,     4.5909916346348614e-09, } , 
      {    1.0825152621812181e+01,     2.1748606512103717e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.9534423102569445e+01,     8.3439594469359308e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8583418523881413e+01,     6.9292623384557228e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.4998278142688708e+01,     1.4412126222593181e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.7414998107765300e+01,     2.0759558634716814e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.3513720269786688e+01,     8.3175727464289230e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5510112074722546e+01,     9.4236431171922377e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0163686332395500e+01,     1.2101166767314571e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2489308110686952e+01,     8.2917214368850055e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0154249864188834e+01,    -1.3248893705638914e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5656317918307424e+01,     1.7618796993076168e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7575074817243841e+01,    -7.7364664670222792e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.7099683492225420e+01,     3.5693834235028953e+02,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.1368606908716714e+02,    -1.6659113817928679e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.0450817658311887e+02,     8.3398101030213293e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5910730235040472e+01,     3.1556081811073029e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.1809005661173160e+00,     8.9213427752938078e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.2606466038998855e+01,    -1.7279656960100849e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -3.5740721562964957e+01,     2.4108604157818132e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8702291460974667e+01,     4.0528117218892468e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.9666765623962490e+01,     6.5946154358527510e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.2508011420797210e+01,     4.0529539909826213e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.0726166246586743e+01,     1.8420158074822593e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.4074072354524283e+00,     1.5527114210321688e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0590746743852053e+01,     1.2463020882452729e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.1200968147270068e+01,     2.2746507874387302e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.8921325042983437e+00,     1.1411039405404492e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.2149437959059872e+01,     1.1419899560843955e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    3.5935628425416553e+01,     1.2796538534488966e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0058239725439506e+00,     4.4694750332076474e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.0118858622632750e+01,     1.7258460472431529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.2097666959964926e+01,     8.3312232054674714e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0309196141164831e+02,     8.3261596439827827e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7455572959721284e+00,     8.6391844265035197e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.5694632593251310e+00,     1.4815119910269326e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.6184401316340555e+01,    -8.7397613377734888e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.8385850964687247e+00,     7.2145363862121549e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.3527358352427047e+01,     7.2135893447055105e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1490133927516823e+02,     3.2199629181096174e+04,     2.0534331944762507e-04,     5.9674261566729451e-07,    -2.8357721835458977e-09, } , 
      {   -1.7840206839276104e+02,     3.2200645098172248e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    7.5080599538837820e+00,     9.5491982442992048e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.3484043181343360e+01,     8.3656614125652468e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1321257620063712e+01,     7.2144550473843819e+03,    -7.7472130562384538e-04,    -8.4177714637367624e-07,     3.9735329303737567e-09, } , 
      {   -2.2739613320361681e+01,     8.3225874552760670e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3535801743487738e+01,     1.2565181042419954e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0962691197136344e+01,     2.7402075486546244e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3362272602749835e+01,     2.4133964277951000e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    3.2101306196587442e+00,     6.3842500925636987e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.5527149158313051e+01,     7.2145074913379467e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.8387942905827206e+00,     7.2136182395797186e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2685613779677482e+01,     1.5619195533642627e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.4439078450263068e+01,     3.0002046875656451e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.9320484749383716e+01,    -9.1245641780414735e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8209166497462824e+01,     7.4356099418369304e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4272711784357330e+01,     1.4904848769349805e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0661932848255667e+01,     6.4306507583890416e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6521753340872309e+01,     2.9039501100354264e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2750580554991618e+01,     3.3315094331376953e+04,     7.6156375320674127e-04,     1.2520555721494321e-06,    -5.9316953883751721e-09, } , 
      {    3.7878202440186413e+01,     3.3314437066424056e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.2714232225336319e+00,     2.6250908725579279e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -8.3271540154597012e-01,     1.3348921781966194e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    5.8888596876045554e+01,     8.3269101720922845e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.7894314393897936e+01,     8.3304726773579678e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0495661573662066e+01,     1.6174864644838826e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0464622574461238e+01,     2.3338739334231035e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -6.1686629184768549e+00,     8.9558944180413509e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6461823350013788e+01,     2.2479843810835831e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6895623594812541e+01,     1.5740303546422303e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0326121010832022e+01,     2.4737669747623368e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6060193308520731e+01,     4.8319246902749046e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.1376007720923536e+00,     8.9640025912495112e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9406046089761695e+01,     6.8692105572661603e+01,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.5782873075654521e+01,     1.1118895109714258e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1066347230326656e+01,     7.1746002795189952e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6954728544076040e+01,     1.6419459840510455e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    9.1284370893584708e+01,     8.9577036416868195e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7588459153128696e+01,     6.8021027866697068e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.3717364827970879e+00,    -5.3009180772991022e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    7.4075031469230037e+00,    -1.6663282903536670e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -4.8633513061211858e+01,     7.2142628390332402e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.0999456510107635e+01,     7.2138628918844233e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6241197412533591e+01,     1.5539662866041508e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1403171935065558e+01,     1.5684756160376803e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.3333008825907193e+01,    -1.0419986323130954e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6351907686780073e+01,     1.5127034439450057e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5218906581598276e+01,     1.4950529332481115e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0547374632745735e+02,     8.9552680412922437e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4134309301502199e+01,     7.8730386321821279e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.0200003247642213e+00,     2.4620285252171750e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.1947956120175114e+01,     2.2767126436898281e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.2653632671477993e+01,     5.9798821950749161e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.4160629578590743e+01,     1.7323774367915285e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2338911114637693e+01,     1.4915542586106390e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.8294632679399481e+00,     2.5002073782499865e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.3945437082062044e+01,    -1.3940017489102305e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    4.7774002869723475e+01,     1.7286042750453871e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7704982910130802e+00,    -1.6663994249003546e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    3.5242810084335105e+01,     2.7237782176606088e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.1656865583889541e+01,     1.5542038353120015e+04,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    4.8733918177289112e+01,     1.5543470227247903e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {    2.5041583257997466e+01,     1.2199382544630203e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    5.7967889698956823e+00,    -8.8848422428512513e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.5126985573318272e+01,     6.8364610068050360e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3720922622702020e+01,    -9.1041916685724227e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.0967421977409654e+02,     2.0679928830920988e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2087759516168957e+01,     8.9227654662275600e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.7030041594750102e+01,     4.7807325779630839e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {   -2.8281792165593345e+01,     8.4173566223806520e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.9967894123623978e+01,     6.9738452179503583e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7730371636323703e+00,    -8.0178968290161447e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2631530534800670e+00,    -1.7841841252621922e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.1416641954179610e+02,     1.9423313660829903e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.0664847109744343e+01,     8.9336925386088660e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.5503112401180815e+01,     1.2750137868663620e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.5560006047896238e+01,     1.4845754599565751e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8398633045794934e+01,     1.3778488237894271e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3113980396766945e+02,     7.1988731531284775e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9392732314915065e+01,     1.5491207883906520e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4864250285260951e+01,     2.4983932035905753e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    3.8725155053393423e+00,    -2.4988216512445004e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    3.0482566569067604e+00,     7.7003943618833919e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8843569905761751e+01,     2.4957208619460282e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3906844862681265e+01,     9.9017695955174986e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1909196787643070e+01,     1.7363242695507921e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.8615473903108111e+00,     7.6124698534584049e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.1199268440295668e+01,     8.4793145339333460e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7698336515983403e+01,     1.8531817525703613e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1224078950327247e+01,     1.4962898105419756e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.3832592367499412e+00,    -9.7330515431504482e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -3.8349447708692935e+01,     8.3245363391577248e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9343179912168324e+01,     8.3328465102925275e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.2759027128925435e+00,     2.5845522780710180e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.8208198354524425e+00,     9.8795529363900278e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2845050519631762e+00,     9.3947601139492999e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2275867132563249e+01,     1.3466315753203931e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.3656757222626666e+01,     8.0293780112040413e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.9077038599293736e+00,     8.3326473563363725e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -3.4772203508433819e+00,    -8.3247354931138798e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.2526970418434395e+01,     1.5475330411207722e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.0835547660210405e+01,     1.5545845714326410e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5971774625100021e+00,     5.8500699384639056e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.5764628113127593e+01,     3.1793998258741773e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5650534059935808e+01,     2.8917926962654610e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    2.5983869985395625e+01,     3.9178913389740490e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -1.1808569434508387e+01,     1.5545485741020195e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.8885622027907957e+01,     1.5540022839347726e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3800922074008639e+00,     1.5889374594051602e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1842418032182891e+01,     1.4740945655710104e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.0337600313041561e+01,     8.6103976574752651e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0019743674945033e+01,     2.8130242031719274e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.6839545102185873e+00,     1.6145322740082580e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9926959510968270e+01,     2.3871838961171528e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.0394264936116588e+00,     2.2744197917320413e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.8230652470367097e+01,     7.7685322310341880e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.8784041507584872e+00,     2.3772118920039997e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.6462702318444364e+01,     7.2044649296702210e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0414202700425392e+01,     1.8606739954399549e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6310664909826773e+01,    -9.5884351000094721e+03,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {   -8.1678234541445072e-01,    -3.6304887317646508e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2009920557683884e+01,     1.5305595847274912e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4547723948852564e+01,     1.4443544841956855e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6631923186150370e+01,     9.0256911153433321e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.5713099325105127e-01,     3.0485665531764025e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2002845627487627e+01,     3.3951868570440456e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4599291530168154e+01,     9.5330488430718979e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4490506720584975e+00,     1.6390929379542868e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -7.1220324447546233e+01,    -1.7653260435850461e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7136570506714044e+00,     1.7976354826570307e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4756842706530136e+00,    -8.3216878432462854e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.3494828650601931e+01,     1.5316327830270866e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.8606037397596538e+00,     2.3871052468646642e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7828088061342715e+01,    -6.2227411065059528e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.7171945984394648e+00,     1.5740829027627082e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.2482097167895304e+01,     2.9058270581930046e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.2447824037392365e+00,     1.2352185866814716e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.6112361111370843e+00,     8.8640276377193550e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6771170568830101e+01,    -1.1813641089186985e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1421948957143094e+01,     1.3774079010725200e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4764374053846311e+01,     1.4918049501173333e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5665801537913993e+00,     1.4518795903129245e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7352604200652539e+00,     5.8526824382385057e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.4092937974938664e+00,     1.3921965675636411e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4696965823376456e+00,     1.3848378015914242e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4599907959356706e+01,     1.8516069782187973e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -6.5381629985019785e+01,     2.4986080987726105e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -7.4788708402215924e+00,     1.5723589400458985e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    8.5798196480004421e+01,     2.4986067560624651e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.0392797434561913e+00,     8.3720625861477056e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.0996786778494965e+00,     2.2612200712501814e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3435212446847331e+01,     2.2235159110486507e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.5489748958208969e+00,    -1.0900453954653606e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0365776159641978e+01,    -1.3234666796301387e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.4491084082009841e+01,     2.9995487920056719e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -2.4742197876466449e+01,    -1.4436227307907518e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.7831609341070688e+01,     3.9325182491479449e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.3502103005292847e+01,     2.2007852867750305e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.1378124612613121e+00,    -9.3373817895690372e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.4554157979443243e+00,    -8.1775160259844215e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    6.8665833064727167e+01,     6.6657374001671888e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2307946586206549e+01,    -6.9209008558933356e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    9.5983714430534768e+00,    -7.1720034802932366e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.2616909067219080e-01,     8.8922979403702357e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9068561200102096e+01,    -8.3216915478824994e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.6676663191388919e+01,     1.5147432120651236e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4991583490633623e+01,     1.4318489654793088e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1651256965047017e+00,     8.4864352548516781e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    8.2152672555811392e+01,     8.3377688443514526e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9201768347904995e+00,     6.9117120679774234e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9777641829662977e+00,    -9.2311277742289735e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.1737921799056643e+01,     2.9893327760089498e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.5253746522303295e+00,     3.9769649663533264e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9830817057511929e+00,     2.0176522633331766e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.9438229225059040e+01,     1.8218650016862488e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1715346292030617e+01,     8.7835169119145994e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2534764881007121e+01,     1.5582921508039946e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3251089862739171e+01,     7.6179005465115424e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6247432208297980e+01,     1.5487642954703140e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -7.6687267427595565e+00,     7.2232363940066934e+03,    -6.5333578932783660e-04,    -6.2828944189709207e-07,     2.9617752592662660e-09, } , 
      {    2.1089543739746663e+01,     2.2704559403898475e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.7706907516056461e+01,     1.6658908102701469e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.0637241193809771e+01,     8.3274158172393891e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.9067724702895774e+01,    -8.3299670322108595e+03,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    7.6175040104658356e+00,     6.3232367173713765e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.6169840752780020e+00,     8.7444112754590278e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9595790593851813e+01,     1.4456511664327925e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0084438592004869e+01,     1.6075432962062607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2000369509410028e+01,     1.6655857596199035e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.4827448196864417e+00,     1.3169209360986844e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.7992631760317206e+00,     8.3306099115744346e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1950312756516643e+00,     8.3267729378758195e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.6118978138548641e+01,    -8.2024372980479257e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.0769928216151353e+01,     2.3417939765743620e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0811285927737618e+01,     9.5956158760717008e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5987876953886419e+01,     6.5461597567210140e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9055199244612897e+01,     2.5596310060075166e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.0385049051882962e+01,    -1.6527646449773335e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {   -1.0892684195680800e+02,     1.0800375894530958e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3673877828733410e+01,     5.3314413101619375e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.1115627977413283e+00,     3.3614054295410824e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7294135205532289e+01,     6.4257197262331229e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8225792243980159e-01,    -3.8657753471172064e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3591333027256105e+01,    -3.2802776907677980e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.5505653108954798e+00,     7.7742220591497698e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.5837343861902267e-01,     8.0650188759623115e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1087173985389093e+01,     1.6668403476172614e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.8511523079277609e+01,     7.1340807457049332e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5918676970732912e+01,     8.9445474902455571e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.2080089483288035e+00,     1.8070578388160866e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.8814442071878634e+00,     7.2236608012474444e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5470130109639165e+01,     4.7600224698250095e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1137268471550009e+02,     1.4452398604527147e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -8.3080249494933966e+01,     1.5812727758148292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6156864124665677e+00,     1.6646362222727890e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.1412556158319820e+00,    -2.4999192185374835e+04,    -8.2775187443012222e-04,    -1.1881086476110843e-06,     5.6229660362446193e-09, } , 
      {    4.6602954546101493e+01,     2.4811272638295297e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0004621221960800e+01,     1.3806429545425415e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2643320798413011e+00,     1.4351072142782617e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -3.3517055984295169e+01,     4.3526654995092986e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -3.7193361438473644e+00,     1.5917726316521574e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.1267414550782208e+01,     2.3267721293664476e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3666903304056732e+00,    -8.9389748015694640e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    5.9792655320162424e+00,     9.5316261521381457e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2733146061652375e+01,     1.1725039534792350e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.9490208796475077e+00,     2.6247898223844495e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5451698909430720e+01,     3.0368183481708773e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -4.5007839848177369e+00,     5.2930916939197923e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0301641205857980e+01,     1.4929824585687898e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.7963975327762910e+01,     5.6421980953814818e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2485705857035214e+01,     1.5558394370046228e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4432855157607676e+01,     6.2866661455434496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4631574683250184e+01,     9.6551364387215872e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -2.9184988665235764e+01,     2.0036079217587016e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.1899649068458899e+01,     4.0044808789978328e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    5.1014442437135159e+00,     1.6239336075702307e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    9.5638679956354267e-01,    -1.1160512502000461e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3068300397565473e+01,     3.8457305800167182e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    7.3028087435039186e+00,     1.5305207705481376e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.8699768259736280e+02,     2.3243646088401580e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.5703870998306378e+01,     5.3013921572141862e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    9.1558930507237655e+00,     8.0092935927607195e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1236487187648919e+01,     6.0145355940325144e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1572317175932056e-01,    -2.5496416000013196e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.5467666971974239e+01,     1.2345859434769900e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6622842584567472e+00,     1.5037305580369579e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.0202215170293897e+01,    -4.4747247477051656e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.2916682981657453e+02,     1.6656932481918335e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4487223341276172e+02,     1.6657833216982166e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.5592588850999697e+01,     9.0582043162498721e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -3.7162105341913701e+01,    -7.5991785332003810e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1150702135323618e+01,    -7.9487883662315935e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.8403465748032268e+01,     6.5686879988736418e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4971006635866022e-01,    -1.5542362108258409e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.1303907711624145e+01,    -6.5846517530208657e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    4.7335429872687705e+01,     2.9438173640423582e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.3602304562350671e-01,     7.6467033970015145e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1138456624112873e+01,     8.3158962589206476e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0970891804090437e+00,     5.6239453026995570e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.2000843628683199e+01,     8.2222735089979087e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.6303534440177319e+01,     7.1514958324590298e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.0369938172902167e+00,    -5.3783406488643568e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.3278986522219274e+01,     1.5494302889902618e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.3171464401739252e+01,     2.0005670367791878e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.8711759640903409e+00,     8.3040558169706273e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1630354162079730e+01,     7.3099774429347299e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {   -2.0090145417634432e+01,    -1.5283196065338457e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9884447794151477e+01,     8.4007882575299482e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.0624987945251796e+01,     1.3768639742357636e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.5913964549276548e+01,     1.4844556927118443e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.9557580634044317e+01,     2.5665698297834133e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.5416904333538440e+01,    -8.1222552047196186e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -1.1196455976980127e+02,     1.8500759734210544e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8304706917188039e+01,     2.3173248351843569e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.2132934678963778e+01,     8.3414865905296047e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.5773983793340500e+00,     2.3314460067752389e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    4.4051384615843617e+01,     2.3313802802799484e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    7.4030968131572399e+00,     9.0268288536760447e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2243374264275772e+01,     3.9668631444768944e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -7.7142625512839595e+00,    -4.9051375858430620e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.2071839007719252e+00,    -6.3486654064527102e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9806689853332003e+01,     5.9417152112201584e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.2231276403391300e+00,     1.7579936776069611e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4509892320648913e+01,    -8.3125502253668146e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0643301993766475e+02,     1.5557944002514312e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.3405571086351955e+01,     1.1406141574445448e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0004354572516242e+01,     2.2208691663925554e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.7730964177948835e+01,     9.1629790491652329e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7828979296954802e+01,     1.8418735383888841e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.0895942991081668e+01,     7.2940449852127322e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3833626683421848e+01,     2.1864871232774723e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.5629156667135131e+01,     3.8944799711681073e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.5446090076009064e+01,     6.6028225619529367e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1868005849660044e+00,     7.1706917040362523e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6291830482962411e+01,    -7.2632634040009862e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.6086507128969348e+01,     1.4066271615779508e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.9284851708279476e+00,     1.7996031232252224e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1688172474518058e+01,     1.4329909953602029e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.0694350505873228e+01,     2.3450079069584699e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.5496147351849778e+00,     1.5923156021554567e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.8123371779927290e+01,     1.1667805414518887e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.5677275089353886e+00,    -9.0437400586987042e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.6114059464936894e+01,     7.7567071723136032e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    6.6899821702856928e+00,     2.3260058810672323e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.8676739182911632e+01,     2.3839965487245852e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    4.6493923122599284e+01,     7.1345621370471576e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.8248384272573553e+01,    -1.5099507287990164e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2425438113982263e+00,     2.4078365348423620e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    5.8284361570029057e+00,    -7.9448781945202900e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    2.9909107147201052e+01,     3.9428289352522534e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.6026801670179530e+01,    -7.3734007372089582e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.2541497506995043e+01,     1.5526859945937453e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.8571716840025942e+01,     8.5375362586396986e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    8.6655413981760091e+01,     8.5389455224719968e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -4.6943305388124529e+01,    -7.2001574587112045e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    9.9662781833976855e+00,    -1.5196632659193392e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -2.6127415457795493e+01,     1.5191420049945375e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.7728520108150967e+00,    -8.5477773168504598e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0959465184632707e+01,    -1.0609481165882257e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.9578967970825012e+00,     1.9535097137086770e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -9.0925135673655273e+00,     1.2256599921815265e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.0753132794591806e-01,     3.1108050485498974e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    5.0176085095962151e+01,     3.2910928017847800e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -7.3165373703744962e+01,     1.6044617292693138e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.1476531823718283e+01,     8.0958354877326001e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -4.9329233621811881e+01,    -8.5615473617176522e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -3.1151427714734403e-01,     5.1507422396782622e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    6.1208105063471709e+00,     8.5136482509687939e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    2.8434315154462865e+01,     9.5913668775388633e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    7.8041219843993916e+00,     7.8669371189697695e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.4139449814404337e+01,    -1.1149571917427422e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.9607493664028034e+01,     1.6849659318514117e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.9692001425144962e+01,     7.3526523954759177e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1433180890591146e+01,     3.1085149550859114e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5366070303778945e+00,     4.9073020138982088e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2287353045857008e+01,     1.0108387330906249e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1125514067312643e+01,     2.7912989100354047e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -4.2156117826307195e+01,    -1.3137030687558545e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0646743900019970e+00,     1.2711425409004740e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7342037766293394e+00,     1.3858687297169730e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4267889239758453e+00,     8.4552481816584841e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.1512828246155713e+01,     2.3243497159372346e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.8654468509884090e+01,     2.5838851763213759e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.3917583840920074e+01,     9.2677537820805937e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.3563386384624710e+01,     3.1085867609876717e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -7.6305930903790866e+01,     1.2074264061388903e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.8440243977649629e+01,     1.2136620868417869e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2176099355049281e-01,    -1.0485312055033461e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1718732075363953e+01,     8.9996588257263720e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4628307350693756e+01,     2.1138347870413549e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9634549482307094e+01,     2.4083625013055993e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    2.1695628115198986e+01,     1.5419895414952796e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.3975581234761762e+01,     7.2355422592206723e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {   -2.1609637785865981e+01,     7.1925834716969930e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    2.3496818247205212e+01,     7.6998315501470997e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6013818917409957e+00,     6.0376297886960529e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5648749073040698e+00,     8.2021346677917682e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.4179564119407544e+01,     3.4513262950899607e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0877595636498306e+02,     7.2292525777891869e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6835785586692360e+01,     7.3207124053480920e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4284398631509944e+01,     6.1339802997262514e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.0243767238872977e+01,    -1.1143742948820577e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.7876550146816029e+01,     1.6678958352133632e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -6.4157175782233836e+01,    -1.6635807346766876e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.9781970027949369e+00,     3.4006134826791651e+02,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {   -4.3160308970431895e+00,    -4.1010619624697611e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.8051900645030528e+01,     2.4674554935834705e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.2810651079302318e-01,     7.2424145879288044e+03,    -4.0607450938411890e-04,    -3.3600496982977504e-07,     1.5806380445214199e-09, } , 
      {    9.4753417461517877e+00,     8.2529615267333338e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.9058252552377812e+00,    -8.4044213227169184e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.9873919939841503e+01,     8.3295960302156836e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8879625488158116e+01,     8.3277868192345686e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8612100420503602e+01,     2.3898106421493718e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.7173535758602149e+01,     3.1558711643389208e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3683706876324401e+01,     1.5543146472109509e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    1.5073481260643550e+01,     1.7514418094248624e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.4754173568898736e+01,     3.8850326769860170e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    5.5685530863690573e+00,     1.7283060602179612e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.7479620318995757e+01,    -3.0954658918788227e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.7457219633883085e+01,     8.9812718833801518e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0715657998789275e+01,    -5.1381230013558834e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.8685111889355959e+00,     1.5749673923698498e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    8.6407822408174127e+01,     1.6115704945043160e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0182733318080317e+01,     7.8473014826602957e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5431370943574212e+01,     1.4946797239838632e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0178847905207384e+01,     1.9439182559610868e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.6292538475444221e+01,     9.0143773341509313e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8864664776612507e+01,     2.6578467833231498e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3451709268839366e+01,    -8.4761077405829110e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.3232656417707595e+01,     8.9566399802618689e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.6221071339353799e+01,     8.8618271776864149e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0179998330079670e+01,     2.9342924752014151e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -1.1756820385549917e+01,     8.3212183878487194e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.3326336876463925e+01,    -8.3361644616015328e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.4084771901760831e+01,     8.9562943778545214e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6706933616507563e+01,    -1.1132058683325408e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2165585672793028e+01,     9.4674139507013970e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.8525964014330910e+01,     1.8937956406596051e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.8224915835615416e+00,     8.1206272812758079e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.5230196168530092e+01,     1.4980922313620960e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7508366003073469e+01,     1.4762186974389393e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.9645528507590981e+01,     2.4579383103750453e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.0007079372421446e+00,     1.4666540851463480e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.0003691599741845e+01,    -1.6045995945213490e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.5463902125994831e+01,     2.0088756819740970e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.9148431958599739e+00,    -9.3586231082483264e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -5.2640551563894551e+00,    -3.4389729499212535e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -2.7075665578264214e+00,    -1.2320225305040335e+03,    -2.1633404559837826e-04,    -2.5536106211401478e-07,     1.2065558081773021e-09, } , 
      {    6.9260036274742411e+00,     1.0660686892241729e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6036278770405531e+01,     8.7266434265740318e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7298180342112651e+01,     1.5754933588330865e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -2.6182985508765412e+01,     2.5160875910055460e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.5485449459212790e+01,     8.7589242723275456e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    8.3051995326311747e+00,     7.2593170357243071e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2045520842957851e+00,     7.8346555235586584e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.7788641351205037e+01,     2.5996586117554194e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2721186089565226e+01,     8.2150565363703026e+03,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -8.8924690949623226e+01,    -5.5741233120698564e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0729909435454269e+01,     1.6465867067606858e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -9.8533468393568256e+01,     2.0719743732818901e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0089396824649027e+02,    -5.5721622805930781e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.9364212507908952e+01,     8.3293542829898397e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8369993252680146e+01,     8.3280285664604162e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3536664525642863e+01,     2.8949551466363089e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    8.4988815712013981e+00,     4.7446033587858183e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -2.1012452349291834e+01,     1.7248872009564620e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1653799594515117e+01,     1.3932986302358775e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4768875105287481e+02,     8.3259940537990224e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.4925826754378883e+02,    -8.3313887956512317e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.9623527852263454e+00,     8.2853202633025467e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.1669460262721799e+01,     1.4985341958976971e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.5914546436104473e+01,     6.7700945071062051e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.1443238327684662e+02,     1.4985538062124650e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.8715739885921046e+01,     3.2758554946659780e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.4451192383482157e+01,     6.3769171600494446e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -9.5345284645764039e+00,     7.0048420321045596e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    3.4811069223421420e+01,     7.0041847671516625e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.3532251225850551e+01,     9.1585041976202610e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.1801395829635425e+01,     2.5059368779427507e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    1.3195942318806251e+01,     7.7725406065990010e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.8789806212638904e+00,     1.6625743660216911e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.2565954401930369e+01,     7.2099077798914313e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0760018710903935e+01,     1.4967768118307682e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1923654325372265e+01,     2.0272123837699262e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.8428641675246325e+01,     2.2614823831267273e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.8716240579758306e+01,     4.1469633669423274e+01,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6119117645231775e+01,     1.5522528973935425e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0214402786394583e+01,     6.9945087342538664e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.6914499742454122e+01,     3.0527090773342345e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7576778949332596e+01,     2.3769274005982203e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.4927611968837965e+01,     7.2182179510262340e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6746712313251798e+01,     8.3121230598744187e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1713988298623530e+00,     1.0857921273231785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3956224575125770e+01,     1.1064090469060398e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6653799330385493e+00,    -7.0601123486848877e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0662252423902771e+01,    -4.4650607232283801e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.8012576266972289e-01,     8.3250448786089219e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -8.1328275607633174e+00,    -8.3323379708413304e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -9.6936682330638391e+00,    -1.1507111866089945e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.2117405621885860e+01,     9.3396487784684832e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.6492567912562283e+01,    -2.9377315375383914e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.8873664765310401e+01,     9.4252710406360457e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2672261376228415e+00,     4.1948470488827297e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8544024084284768e+01,     2.1707136878434292e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.8136703248605336e+01,    -5.1887086195407903e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.5808601258938367e+01,     8.3327483359361213e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.6870247644264573e+00,     1.5594300696461398e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6068068360472294e+00,     5.0120330600663156e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1429595844010322e+01,     3.2709000018986018e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.8779862768028607e+01,     2.6073475520301599e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7898096081395195e+01,    -1.1148828236505287e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2032878787106254e+01,     2.5694563952549230e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0231575989031337e+02,     6.3157818124489568e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.1230205781264742e+00,     8.4979079976141875e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.6357201774252999e+01,     7.2164927242006079e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.3991258325357236e+01,     7.2116330067170575e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0916138232021225e+01,     2.0643622000550749e+02,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.4178722048243872e+01,     2.2700150176729407e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.3314013373484652e+00,     7.8409477595706239e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0707901867671046e+00,     1.4374053106314050e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    8.8569591537884506e+01,     1.6084526541528676e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    6.6730019636784910e+00,    -1.2735358508998675e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.3846852532085947e+00,     7.7467692461368788e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.5471424942223241e+01,     1.5558648634430465e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6895175971708468e+01,     5.1530434395179973e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.1094359598723187e+01,     7.2119292968679174e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5408557684743087e-01,     1.9783501663638781e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5890534838967305e+01,     3.7278242891088215e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.6145003322666327e+01,     2.1484454242036063e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -2.5169203316388113e+00,     3.1492199750792846e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.9100309873081205e+01,     8.8777215219329210e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4693209472341394e+01,     1.0723878087415005e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.7052727328678859e+01,     3.2219324809342437e+04,     2.6936461440465505e-04,     6.0169256335142287e-07,    -2.8559889140481653e-09, } , 
      {    3.1955837943540260e+01,     1.6186480986268693e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7271542463429008e+00,     1.4356028898112843e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -7.7482348128431298e+01,     8.2712512101022821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3275403714662474e+01,     1.5622736409937859e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4779180559733934e+01,     1.2382854048492467e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.9392008960913314e+00,    -8.9409995014461838e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -9.9870182998122310e+00,     2.3220431908152259e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    2.3442127602560070e+01,     1.5540620721593044e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2698700445436102e+00,     7.2161964340497479e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.8205611655634710e-01,     7.6474242182545113e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    4.2669183846391912e-01,    -2.3870342187516657e+04,    -6.4234506317708347e-04,    -9.6967099545037180e-07,     4.5909916346348614e-09, } , 
      {    2.3247615051404569e+01,     1.6768217865088263e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.4471802498847506e+00,    -1.7004003153317899e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.6937025927788596e+01,     1.6044031597542824e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.5257057794940273e+01,     8.8505810940434530e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    2.2995163195529134e+01,     2.9892883134210388e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5634202238724644e+01,     8.7337459736092496e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1040666742976195e+00,    -1.9126842356123270e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.9017069106503421e+01,     1.5533156354395347e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1098585739889602e+01,     1.4182527848684533e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    8.2471488250611813e-01,     6.2084820542359412e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.1241385720572747e+00,     1.3699780005536157e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5679353159754452e+00,     8.2719558420184330e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0752040094415072e+01,     1.6399789534982330e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0486115863118229e+01,     1.5055721970496332e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    9.3940607326930259e+00,     1.3795408918703051e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0577722069494211e+01,    -1.7353095606855659e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9413478650883743e+01,    -4.1505758741203153e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    3.3223260305640743e+00,    -3.6823038432549397e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.6265360099569666e+01,     9.3188683842768769e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.1041027806213592e+01,     8.3931190609870700e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6047613080385666e+01,     9.0033567074285675e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8282948786932458e+01,     1.3235500284760134e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0544144792063129e+01,     9.1726087583240405e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -6.0138772967797337e+01,     8.3931324880885204e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0278629140115907e+01,     2.2009064864903510e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.7872254484680870e+00,    -8.1874161593672097e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0031775896472972e+01,     3.6926218817375839e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0149145335896385e+01,     1.4286498149601672e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2884255250977446e+00,     7.2235053796985567e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.3738130328275290e+01,     1.6634076378288450e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1640838737385673e+01,     6.5590945738286564e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0285682065500414e+01,     3.9431120685901828e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    7.3514129787715177e+01,     2.2853719956205474e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -6.5470825582028844e+00,     1.6094035095422158e+04,     8.8291455684318254e-05,     2.4546116674575806e-07,    -1.1661223471727668e-09, } , 
      {    3.3536731724031469e+01,     2.8903166059905794e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.0186830971294313e+01,     7.1981685212123284e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7929085675806210e+01,     9.5284178501075985e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.6218043189219653e+00,    -1.6377092953563304e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9780370796642522e+01,     1.5300908719231673e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.6788908753795200e+00,    -1.7790035616917747e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.9929685311857117e+01,     2.3208193240853554e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.9480263096196655e+00,    -9.3838014069666224e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -6.0469150274534087e+00,     1.3800146778348571e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.6325448112426844e+00,     1.5544887858774873e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4792486553200114e+01,     3.4715997312132567e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.6140419668343597e+01,     7.2316881781837628e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.5896627623137469e+00,     1.9188975506096198e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    1.7878657679357804e+01,     7.6097136674680060e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3659306158263817e+01,     2.3897184850056005e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.9820759274844679e+00,    -9.2532895245294258e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.7154898912780361e+01,     1.6619843606130493e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7175005924174517e+01,     2.2632854753019201e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.2739404706622022e+00,     1.5486087311270572e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.2466429691178060e-01,     6.5290883014999044e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2272536314173699e+01,     3.3337307604031565e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.0334575579446636e+01,     1.5784599861136241e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.0429583939192213e+01,     1.8422860898893145e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.6454384814282008e+00,     6.2112725547013388e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0622257080271133e+01,     2.8119425320359558e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5222086060905276e+01,    -8.8127111991061429e+03,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.0036830722636793e+01,     3.0711632434998275e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.6081430030292843e+01,     2.1119819205753334e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    3.5092072439596237e+01,    -8.6730105287530350e+03,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.1685527960713870e+01,     2.3824546376206668e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {    1.0066017022015451e+01,     3.5050000242173023e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3281799189201786e+00,     2.5779933224712904e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6513616821328352e+01,     6.1392492080904512e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -2.4021295447065853e+01,     8.3145716046532962e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.5590811937979865e+01,    -8.3428112447969579e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1207801898730112e+01,    -7.7128353592046951e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.6412974005443289e+01,     3.8746077518149585e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -4.5279010662929984e+01,     1.7880199545739488e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.5985409034336300e-01,     1.5562979606432493e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5528386329237094e-02,     5.5955783208615394e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.5112886630624917e+01,    -4.7270683476755703e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -6.2276252028096195e+00,     7.6692169851247536e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.5479560491055125e+01,     3.9392870097115192e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.1034766141357565e+01,     1.5541323397381208e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    6.0422864520420028e+00,     1.5544185182986708e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.4902115189136726e+01,     8.2081689482941256e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    7.6901262437441709e+00,     7.7713666228881702e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.1089172520754047e+01,     6.4680415431979791e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0202226561543860e+01,     7.0875061085254747e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.9163987451916675e+00,     7.5710012078091204e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    8.9897891718019398e+00,     7.7713876672957886e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.3446866207920381e+01,     1.4028615215525670e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5908981444180579e+01,     2.3623041188357074e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.6173575119128841e+01,     1.5462772170430058e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3142572065377173e+01,     7.6888938519239382e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0485813986277456e+01,     7.1765236221390769e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.1209554867376937e+01,     8.5378818610470462e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.0936549482589893e+01,     1.4399774008447695e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.9101605514789433e+00,     7.6893632129572188e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0017357335088185e+01,     1.5542091431919245e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4369791960204905e+01,     8.4460948717489864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5580905666471772e+01,     6.2546267098775184e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6973405699563081e+01,     1.6568340600682484e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    8.8312247107885682e+00,     1.1099445669474502e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1128585157935436e+01,    -9.5572126763271933e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0363663879587452e+01,     1.3120868386629430e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.9995426027658922e+01,     1.6708699288063464e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0138723988521750e+01,     8.5443011732784980e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.4604235318224442e+01,    -1.8360504091864175e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -3.3126167641309038e+01,     8.5385999200646493e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.8660206520375894e+01,     1.5480187257184156e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.9724640927635406e+01,    -8.3196560308654243e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -3.1644415087148964e+01,    -1.4968328175995612e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6580492276405145e+01,     8.2304603880414434e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1291601420908940e+01,    -8.4269224614088089e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.7094229862757985e+01,     1.5543417148448669e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2609270001960233e+00,     8.9886381990040081e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.6497910745632737e+01,     5.7601041543566116e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8630824479961788e+01,     1.4281208087936953e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0984721008184366e+01,     7.7005322567367966e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6167544779785402e+01,     1.5695557612368473e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.4146565888355852e+01,     4.6056606931924052e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4737978540140958e+01,     1.6640315567910606e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1562939836700420e+02,     1.0466314651059231e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5717948317133525e+00,     7.0507463466561876e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2529446751185418e+00,    -8.5654708526201157e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    7.4282029274809513e+00,     1.1677348806032909e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3868931318752910e+01,     3.0157480030157149e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    8.4322688475145711e+00,     1.2669244513465742e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0506550755501848e+01,     6.9889713713396814e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    5.8390053042582100e+00,     1.6689022038683594e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.2781899389373237e+00,     1.7616891627324250e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.3389621956777527e+00,    -1.3361756356443909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3362293444167257e+00,     2.4409581800620123e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -3.1869042884649783e+00,     9.5776800111899429e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9034200868011741e+01,     1.6674450130989899e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.6154690538249898e+00,     5.9820308591686789e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4375182453754759e+01,     2.1938325082177662e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    2.3857416677670731e+01,     2.2135987737166306e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.2524770122916358e+01,     2.0584756998836732e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4362346920564892e+00,     3.2180949469925981e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {   -5.1619853637445047e+01,     2.6105971741805872e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1280887742520921e+01,     2.7155298883438045e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.2622174291618812e+01,     2.6519662789073423e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.3603199924184949e+01,     1.6653780053415336e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.5343221114681882e+01,     1.3831979292447377e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    7.2385603389408345e+01,     1.3704646313787869e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.9309102486617206e+01,     1.6660985645485169e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.7138438785015488e+01,     1.2791570752221423e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3245259838946311e+00,     2.4404152095587131e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.1673476142042034e+00,     1.9581996475789187e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    3.9246482319418696e+01,     4.2558367762636939e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -4.3020200377698409e+01,     1.9367956730643564e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3544892798991388e+01,     1.5294349763631948e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.0904952089605594e+01,     1.4901995185613301e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.4285277010661450e+00,     6.5878975186829139e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.3894631521663761e+01,     7.2222299084446940e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {   -1.1113006364089847e+02,     1.1722929821150319e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7851240708620594e+01,     2.3246951421676677e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.6990947551025130e+01,     1.5656165521482748e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.0443138608079555e+02,     1.2056991064622047e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2368877530081051e+01,     8.7130425264359274e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.3291074515420192e+01,     1.3701202696469909e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5177092605582345e+00,     1.6723350208249876e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7877029096886936e+01,     1.7197321462123957e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.5090963490164881e+01,     1.4915151337054564e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.9813782507574356e+00,     4.5562160821356829e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5495433733468619e+01,     1.4424322961308340e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1031624230023162e+01,    -8.1859934684334557e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4002756804932709e+01,     6.7983430147249310e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.4163203297106310e-01,     5.1779647115814987e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.4756719802900093e+00,    -9.4646498919692713e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3363262242397324e+01,     1.4807068948234575e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.2069519992722750e+01,     2.2809074907387108e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.3386750188034924e+00,    -1.0246232145857699e+04,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {   -3.2462541432975875e+00,     8.3750768311824231e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.4755693694219801e+01,     3.1180680412452173e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.9461094052714721e+01,     1.3853498588550190e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    4.7325276722775179e+01,     3.2124886078461823e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2144761768137604e+01,    -6.1620813884105974e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.2007240607708248e+01,     1.5003488953906184e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.0280691666497745e+01,     5.9571190628912973e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.4457757447713617e+00,     1.7301689942820281e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.5451853156541802e+01,     6.4956505202763046e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.0329820679433961e+01,     1.7780829863054689e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9164476256578276e+01,     3.9119953663519227e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -8.7972496253228307e+00,     6.6824523657487607e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3666911151185062e+00,     1.4054249585548025e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.9553679280202698e+00,     7.3259011139676322e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2581518324448933e+00,     1.4926898224663528e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.5382481074916281e+00,    -1.9716638040646651e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    3.5910815398919851e+01,     3.0395067845349440e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2891046703530861e+01,     1.4332565757039876e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5500203512676478e+00,     2.4041180278841894e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.2248051937199959e+01,     1.5412242407746764e+04,    -2.8252216682175921e-04,    -1.9141413757566701e-07,     8.9782645604674952e-10, } , 
      {   -1.5860702187722730e+01,    -8.8173135987856112e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2952885293671017e+01,     1.4431928500526990e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.6217958547698004e+01,     2.4143150151351845e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.6269170295578466e+00,     2.5088245983102261e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.9940855939636553e+00,     1.0890937003680778e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.7601678750183556e+01,     1.4954263342033717e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0314820377625162e+01,     6.2215818359568257e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.4914186989687703e+00,     9.2632789305356218e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.6790698694363901e+00,     1.4927600900451695e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3768989197370672e+01,     3.1030151650181579e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.0681929488062647e+01,     9.7456707814728270e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1417470280958096e+02,     1.3799916197445962e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.7811007402844670e+00,     8.3936512557425740e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -8.5651010607090711e+01,     1.0777833989399902e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.0540934203712915e+00,     7.2766298984586356e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3040839133503354e+01,     7.1725932317894112e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.0621426038809600e+00,    -1.8294380799339706e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -1.2179878255589745e+01,     8.2588273698794637e+03,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    2.2012353023825149e+01,     2.6263352960569973e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.8025333607978201e+01,    -8.8948658597977337e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.5627381021641074e+00,     6.8651945242639840e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.0317892751444322e+01,     7.6159003038648725e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    3.6050937786055982e+01,     8.3318886547409929e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9588787237985468e+01,    -1.0535608474543587e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -2.5056263615376984e+01,     8.3254941947092593e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7148317036151916e+01,     4.0357976298819456e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.0820585833341209e+01,     7.0878087387816304e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -6.5229024674161124e+01,     7.0200554279165408e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    5.5457993585110028e+00,     1.8635928471587777e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7621986968108189e+01,     3.5566473601331214e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.1077553556719380e+01,     8.3954269743775312e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.8260896130706410e+00,    -3.7014820371770456e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.8097037673939784e+00,    -2.9972243913888788e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.7998362005424973e+01,     1.4102700985829364e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.0086664542783836e+01,     8.9968501957824064e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4407586214915762e+00,    -1.6848898631293650e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    1.7542749593251095e+01,    -1.0954503653441886e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    1.4592836792967367e+01,     1.3197247710348413e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    4.8278789016743730e+01,     3.7373844095455715e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3188152279229136e+01,     2.0057500738650215e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -1.1356252594066881e+01,     7.2827549710314925e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -4.6556995065417553e+01,     7.2032965031207041e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9216544253693158e+01,     8.0027346371609929e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9112716943425343e+00,     8.3778940364408154e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3709355215466227e+01,     1.2439591704372697e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    6.2520585551829484e+01,     1.4954259872038940e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5814806124729316e+01,     5.3391295163283985e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -7.1548068887565194e+01,     8.5382476041065729e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.2383355800311932e+01,     2.7795096456239233e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    9.5412939395601004e+00,     8.2795250971232599e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5822063857752385e+01,     3.0964851877520978e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    7.2539544212510904e+01,     3.7671616176739277e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    4.4752334943962452e+01,     1.5782971937692433e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2007192590738640e-01,     8.3192789797531805e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -6.4944456500661829e-01,    -8.3381038696970736e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.3597416667417789e+01,     1.5480122467333949e+04,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    9.6801476455078017e+00,    -1.1910698027249632e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    1.4974458271285757e+01,     1.6310762545582609e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    6.1240034355995476e+01,     1.3590620995999476e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.1021833566548210e+01,     1.7226821283198187e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0653222954423747e+01,    -9.1888885443155159e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.3311708638499155e+01,     3.4465442674569204e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -1.5730459866557990e+01,    -5.9290959728977759e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7531296914541976e+01,     9.3065838252695175e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6920019249816985e+01,     2.3712538423275510e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.5815227571027499e+01,     1.6408220318138548e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -9.6991736782621469e+00,     2.1915313083780302e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.7175274956427792e+01,     3.2289154653247810e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.6555082626512274e+01,     8.2514914796710782e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4192598448897147e+01,     1.5613395279638278e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.9313184360602008e+01,     1.5541126366740151e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -2.2236131767202437e+01,     1.5544382213627770e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    4.3849255382897852e+01,     7.2447310471365854e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    8.8665775728413152e+00,     2.1593501990079469e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5679768430052157e+01,     2.2762850630037741e+04,    -2.5376175874277062e-04,    -8.5593855399888591e-08,     3.9429896684638543e-10, } , 
      {    1.0470949892090598e+01,    -1.1022267096163230e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.0292595066738588e+01,     2.3941335492929058e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -1.1313762350333491e+01,    -1.0359651579181475e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.9054751714825517e+00,    -1.1348906255431559e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.0490211948079089e+01,     1.4915550232530843e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    5.2778216550306745e+01,     2.7098096765621027e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3730255067159689e+01,     8.3347953941741853e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2938877265326857e+01,    -7.9059314077331946e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.2815837792559783e+00,     3.9459421810774656e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.0274947195627266e+02,     8.2247887473198189e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2045812078226234e+01,     1.5681343820201044e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2645845882932363e+01,     4.0451276303347215e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    1.2153550467391387e+01,     1.9910480256880244e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2586847863194720e+01,     9.3898616896892727e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    5.9573963623189474e+00,     6.5360918829787443e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.6399442378068745e+00,     1.9705949402626655e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.5759771454905975e+01,    -7.4171881634433203e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.6911104577518671e+01,     5.8380029896740098e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.8422363552943981e+00,    -1.8263248155725167e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -6.7178508691754615e+01,     1.6115610598105535e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.1515282861847187e+01,     1.2933609372525887e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.1829708520715339e+01,     3.8692592415519735e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.4550138658577800e+00,     8.3778577523269923e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.0655132609319935e-01,     1.5239871450226714e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.3473007657236330e+01,    -1.6657701653001266e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.9631780183920185e+01,     8.5382341770051225e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    7.9527051217367585e+01,     1.6654995016475106e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.3933388166126726e+01,     2.4285136762176640e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.1785479696084156e+01,     2.3378124863845711e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1215417622264930e+01,     2.1408728867926948e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.3494936401480173e+01,     7.7158136935710181e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.7953832401115704e+01,     2.5109291403424395e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2894545014526750e+00,     9.4367085844381272e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2700274913040310e+01,    -8.1560366322225800e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    8.0030223588520801e+00,     6.1799830379005539e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1213874351317683e+00,    -1.1378884913325197e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.7268863951322210e+01,     1.1961819232537793e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.3822403460829740e+01,     1.6659770682425398e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.4719907057655650e+00,    -9.5726873564938323e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.6068473047348668e+00,     2.3331052116084724e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {   -4.6124271437949194e+00,     5.9176059851421305e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    5.1524785550224133e+00,    -1.6064020153414693e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -1.0458389961402148e+01,    -5.7713616185573446e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.2796437591457021e+01,    -7.8942008213627123e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.0155993853906622e+01,     5.6396777868471472e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.8834535287305528e+01,     8.8943671869206592e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -3.7951689895312391e+01,     7.2951289152474510e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.5791285751351225e+01,     3.2065150405652043e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.8925469370547901e+01,     7.2248292277969613e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0723277813956896e+01,     9.0066624072052145e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.1641674917597675e+02,     1.8795006075784357e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1835673210652683e+01,    -6.6864796814823512e+02,    -1.5447957690769919e-04,    -1.8151424220741028e-07,     8.5739299504221431e-10, } , 
      {   -8.9709002335022049e+01,     1.7029528026341468e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9975976532900419e+00,    -9.1485242674001856e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    9.9195773134074088e+00,     2.5055904118080751e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.9136353825703161e+01,     1.8924838495396594e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0427361680870698e+01,     1.1727753254520045e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8138995844536385e+00,    -8.9701169209701256e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0505918860161833e+01,     1.1863578952122280e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.2379596488435185e+00,     4.2659815955703628e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1091050908157825e+01,     7.2735661831155849e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.0179287183568555e+01,     3.2319581845327946e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    2.9472949033634986e+01,     3.0363900990794609e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -4.6653774888915898e+00,     1.1959164897121727e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.8995965417113281e+00,    -1.8657350854284025e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.3909552985709475e+01,     2.9451291551623035e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6501496808084514e+01,     1.2126771779580256e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -3.6492736965076688e+01,    -8.3121829939890176e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    4.0909109784227766e+01,     3.1496578493416280e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.9843374700650145e+00,    -1.1184418510794287e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0698712489877867e+01,     2.1136739886562118e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.2942302771109212e+01,     1.5471408668813487e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -5.1192345776696140e+00,    -1.1426458516443683e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.8802881152217594e+01,     1.8981727240105214e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -1.4211076128983084e+01,    -7.9720882761223420e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0430514421250033e+01,     8.5923639734879398e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2362726213687676e+00,     7.2482964090019423e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.0227096154443984e+01,     1.5538599204616556e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2505335514223983e+01,     8.3052192784776489e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.6526337161303921e+00,    -8.3521635709726033e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.9715575337834139e+01,    -8.3755907634275463e+03,    -4.9219913880208617e-04,    -6.5036300879800924e-07,     3.0757064743270071e-09, } , 
      {    7.1206385914398979e+01,     1.3998892655361684e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.7192389958658989e+01,     1.6657064045899235e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0195427013102520e+00,    -2.6091601770261195e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5585228706445497e+01,    -3.7539243319756636e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.3905700965332059e+01,     3.0459278538106075e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.6693117935074440e+01,     2.3822743849244005e+04,     2.0751014571397604e-04,     5.2784574344481837e-07,    -2.5068261009130776e-09, } , 
      {   -3.1141618717555289e+01,     1.0635741349964728e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2211579854264535e+01,     8.1824223229861727e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2534644272903954e+01,     2.9574461970600925e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.8331729366599472e+00,    -2.4668037032351704e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.4635453468836282e+01,     2.1053695913228865e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    3.9734138803271861e+01,     3.7702430715336035e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.4607660872841677e+01,     9.2080181092708153e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.1156677843034199e+01,     2.5914102824386147e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.9397250016742483e+01,    -7.7176568364464392e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.0889810632874582e+01,    -8.2070396977273940e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2758039409741597e+01,     1.2917964467799509e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2076602591962036e+01,     6.7615385781090972e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0753397183117343e+01,     8.3394928767167985e+03,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -5.2322913674031341e+01,    -8.3178899727334538e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    9.2695573578688251e+00,    -2.0300378711546823e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.5439418991142819e+01,     2.3716341192884836e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.9831352408050087e+00,    -2.6609752885164094e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    2.4367122860380320e+01,     2.7323476641915197e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5066827692374261e+01,     2.1589219499165301e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.0183291446549170e+00,     4.9334893083130319e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7297557910733488e+01,     1.5546909375751360e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2581767386288874e+01,     1.6656918387513466e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.6301141750871068e+01,    -1.6657847311387042e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    4.6227822729834696e+00,     7.3362103952231191e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6370575892656714e+01,     8.3340527078775431e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2950241009635324e+00,     9.8750381539752416e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    5.2443972436747703e+00,     3.3903258382048138e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -9.2779828067521937e+00,     1.1018018435711505e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.8692138260694255e+00,     5.2784979361250562e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -4.4206290837284186e+01,     1.5531987927845830e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1557534740663417e+01,    -1.4335339658635028e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.4406674509930006e+01,     8.8289178056935052e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.1258891037774580e+02,     2.3242630171325505e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {    2.6007658185130779e+01,     1.6715258243663193e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.3356274791703878e+01,     1.4460940755972950e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3775152771248058e+01,     2.4375790897331040e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    2.0069319294763428e+01,     1.1231013611316745e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0991046975584137e+01,     7.7191193933476643e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.6150684485536544e+01,     2.0880754555945623e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.8996490013186932e+01,     1.5978373156092963e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    2.6847935694905470e+01,    -8.9111506334778096e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -7.6421572275474972e+01,     2.8230187100444182e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3327277564115185e+01,     8.9289606604091023e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.6668133334055433e+01,     2.7630803146299073e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.1481781076700566e+00,    -8.9378208158485631e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    1.4880361365876762e+01,     1.5542465085853733e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3964415479098950e+01,     1.6663443132172903e+04,     1.8107315872033681e-04,     3.5623139660566481e-07,    -1.6898665668753985e-09, } , 
      {    5.8952559664603621e+00,    -1.0123442680527252e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.2196559254907708e+01,     1.5543043494514182e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4574266703368171e+00,     7.0169649191532235e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.8234629964380176e+00,     1.5523576096261855e+04,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    7.9488196390518508e+00,     6.6011331609627714e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9044374335223413e+00,     1.3521751439688769e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3228791747414084e+00,     9.1238886596456869e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -3.9361305424382117e+01,     7.7530006209326993e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.0085029064386738e+01,     7.1721704207772200e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4859977273610959e+01,     1.4285780090584072e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.9863843696897945e+01,     1.4915258751123889e+04,    -2.2515794548278047e-04,     1.7123619216788808e-08,    -9.3714484558473377e-11, } , 
      {    2.4589627161822378e+01,     8.3619741793510148e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.0663877167082898e+01,     9.8215777315633204e+01,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    4.5677287703124044e+01,     1.0184763784514709e+04,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.1331969777229212e+01,     4.3358499049452421e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.9345567089464730e+01,     2.0011100072824876e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.4501899153716096e+00,    -1.0068997736720779e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -1.0643145365255428e+01,     1.8695036440974400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7472347498171398e+01,     1.6470782840394728e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -2.2553029725561604e+00,     1.2955900812273205e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -3.1991323914021947e+00,     7.3107599507943032e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2369415705621659e+01,     1.4964110102572959e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.8372030441679531e+01,     3.8788522252363684e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.1486687486719823e+01,     2.1742047556503989e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.6319087189940603e+01,     8.3343725831619977e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -2.4171788988034198e+01,    -8.3230102662882546e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    5.4144065877455475e+01,     1.6170702845720702e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9569813707321764e+00,    -1.0361359128195993e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.3911143241098845e+01,    -9.9067563242882784e+02,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    2.4980792129101861e+01,     3.6520586885112505e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    3.9341258798010887e+00,    -1.0596645743548548e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.6892008381650534e+00,     1.3958051173089631e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    5.8645019258735278e+00,     9.2467305689296991e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    6.3787658261056315e+00,    -9.7755772122937151e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.0953463374863496e+01,     1.6602122028185448e+04,     2.4060420632566652e-04,     4.9587228117564448e-07,    -2.3524614248478012e-09, } , 
      {    3.2719400464771788e+01,     6.7277571506061340e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    2.7101146963899456e+01,     3.9435529913070895e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {   -2.3258658526707421e+01,    -1.6746425098218024e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.5872877453182548e+01,     1.0556305360445856e+04,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -8.2991130907158350e+01,     1.1740202817917176e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4031325638108463e+01,     2.4656333668574156e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {   -6.9701856555508229e+01,     8.5200014913413124e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.3353325795741853e+00,    -1.7183529805109040e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1264782805217433e+00,     1.5033539094874077e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6177097772329759e+01,     7.2139238894291393e+03,    -9.4791934483371131e-05,    -3.8770550078329589e-08,     1.7928409927430627e-10, } , 
      {   -2.9118263260456146e+01,    -7.2142018414885233e+03,     3.4220980924608724e-04,     3.3415782970474765e-07,    -1.5759353518146575e-09, } , 
      {    3.9394508490863966e+00,    -8.5705369956774084e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    3.8883867920201807e+00,    -2.0769973877835646e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.8995711001432014e+01,     9.5610337212067298e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0686117461913589e+01,     2.0951524204301986e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.4091853061816529e+01,     3.8222019184814621e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    1.0401779016555223e+02,     8.3314193086151499e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.3023495713868840e+01,     8.3259635408351041e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0605347238204875e+01,     3.0128614375442052e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.8400496918630971e+01,     2.4406453295426865e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2699891399023276e+01,     1.5201548431981260e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    2.9246181888946591e+01,     1.1503512104578731e+04,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.6977025698566997e+01,     1.4308180373537600e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5989197478377715e+01,     1.4669971301869946e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.4984554592238446e+01,    -2.9199215106070988e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9293915456724228e+01,     1.4803113016623323e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -6.2297972315697479e+00,     8.3040835257700237e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    4.5499264408828779e+01,     9.5564561994691649e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.3369467450097764e+01,    -6.5551023581035852e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.8415336920426341e+01,     2.5445167935256057e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1441773920840602e+01,     3.8749488756650690e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {    1.2757840803942715e+00,     7.2089422928228851e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.4945757885435071e+00,     1.7929997527865824e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.9309610239138582e+01,     2.8873172142644111e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {   -1.6669039869905923e+01,     1.4624436658478407e+02,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.5876280622596962e+01,     2.3899779204365776e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    2.0441791798467733e+01,     2.4022621113649788e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -8.5198603162865147e+00,    -8.2789104888976763e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.4365128454501427e+01,     7.5798613632485831e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    3.8115846280496051e+01,     1.0376279566358109e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4898895050660151e+01,     2.1611535674066792e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -1.1861818211228896e+02,     1.9717560002403719e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1580889619691117e+01,     5.2965221443262581e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.6051770927125979e+01,     1.6696080049741086e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -8.1425137105970720e+01,     8.3230110859065080e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7732459809829557e+01,     1.5500861845502346e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9612016595094708e+01,     1.2571739998019681e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8613180232631508e+00,     6.7025605553437049e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8606130497102086e+01,     2.3051622348020144e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    9.2419421408682723e+01,     8.3343717635437442e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9688559040752281e+01,     1.5988734881302018e+04,     1.5014592437499730e-04,     3.1930798665236256e-07,    -1.5152851603078547e-09, } , 
      {    3.4264328012481116e+00,    -1.3341674311032102e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5143706421519916e+01,     1.6544938020599140e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4724094070181977e+00,     1.6627370850363764e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9332675138803509e+01,    -1.1148342744935026e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    6.8967836954762394e+00,     1.6134302113360216e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8680544833914215e+01,     1.0102280491346668e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.5973687711340848e+01,     7.2398000149806667e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.2647594192135415e+01,     2.8191895460387372e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.1037875163842958e+01,     2.0321139076472740e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4901710401369542e+01,     3.8786603765514374e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.4060227349709503e+01,     7.8139845170941171e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2581240560182927e+00,    -1.9808525919805761e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.4924911453512060e+00,     1.2318404439311751e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6227251530189417e+01,     7.9755121652577545e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.2193202234513016e+00,     2.2410695524652219e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -1.9487616917769060e+01,    -7.3691826468511281e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    3.3109690599034877e+01,     1.6703231225743111e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.8904377450730099e+01,    -2.7024449221858340e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.6829051821048282e+01,    -1.6611534473157393e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    8.0155161669822359e+00,     1.4973965282163213e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    7.4926454959637869e+00,     6.8497456395892891e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.2595387223063966e+01,     1.5944591728590000e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    1.6609414668421576e+00,     6.7615832528556448e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0490298379303518e+01,     2.5571722043265218e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    2.2532707116750885e+01,     1.8118133039707114e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -2.3191438699115615e+01,     8.2642418155444084e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.5441766182889618e+01,    -1.2505816956961385e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    3.1285468570272513e+01,     6.3347361809131453e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2243155122585975e+01,     1.2043574183467715e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3648465935785531e+01,     1.9244236327361003e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -1.8307632410152404e+01,    -7.6778419345485272e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5804303229788109e+01,     7.8177872867034430e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0457466093953894e+01,     3.0941220769703395e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -4.4070806694542668e+00,     3.2215869623781804e+04,    -1.0990726150753163e-05,     3.4138155355327972e-07,    -1.6292163753685956e-09, } , 
      {    1.0785923966190822e+01,    -1.0238517218911980e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1068919276225188e+02,     7.0046114512019503e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -8.5412651006300067e+01,     7.0044153480542718e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.3460449541988723e+01,     1.6487850121934374e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.9408320925847143e+01,     1.7414559134121133e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    8.7146091900413936e-01,     8.5380172334998133e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.2122362227300094e+00,     8.5384645476118822e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    4.2344702440379983e+01,     1.6941371330467889e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.3208692478256175e+01,     1.0857629045511997e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1187811366373401e+01,     1.5543227810937280e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5889241227026167e+01,     1.5542280769430638e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.1921178366305682e+01,     6.5761573446246775e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2977344762546466e+01,     7.7803659594334822e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5294398997946248e+01,    -1.7068877279094265e+03,    -7.4162724501215490e-04,    -8.7375060864285013e-07,     4.1278976064390330e-09, } , 
      {    1.3189886007726168e+01,    -3.0141450721976607e+03,    -3.0895915381539839e-04,    -3.6302848441482056e-07,     1.7147859900844286e-09, } , 
      {   -6.4659893954630832e+00,     2.2065001217293889e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.2889385991574251e+01,     8.0627366276948178e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    6.0788472956483730e+00,    -8.1790013483658267e+03,     3.1128257490074774e-04,     2.9723441975144540e-07,    -1.4013539452471136e-09, } , 
      {   -2.5211430598749061e+01,    -8.6852523255401684e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.5549159701946525e+01,     1.0804211214120791e+04,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    1.9553260469761884e+01,     1.8640521381902516e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -2.7241557673394787e+01,    -1.6712643670715057e+04,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    4.0059337142120995e+01,    -6.2631822850009485e+01,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.3182300126503280e+01,     5.9612609871808809e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6218567924289172e+01,     3.5109018281825556e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.6227392060187842e+01,     3.9114479039927821e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -2.8085391032336592e+01,    -1.1123510429081848e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.2959406287541748e+00,     1.4273519881770495e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.2478509597299578e+01,     1.8182567389519783e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.9835259780866821e-01,     1.6162425376846073e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    1.0033771171096355e+01,     3.2764010374867084e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    3.0584138811525555e+01,     6.3685176084161085e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9759900070809213e+01,     1.1033890368943853e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.6224218878129602e+01,     3.1571118209121792e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5368038091230142e+01,    -1.3546996519409317e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    2.0484761552886848e+01,     1.6504564267897691e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    2.2545660091619336e+01,     1.1335788344232102e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.4300119797896398e+01,     8.3390284147800121e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4761454808159726e+01,     1.3039513356007981e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.8694024420068104e+01,     8.3493615022996910e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7698864526151382e+01,     8.3080213471505613e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.7956703596385983e+00,     7.1770928776187120e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.9615810353889870e+01,     6.6567418722749571e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {   -6.1564037796420180e+01,     6.6567552993764111e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.0472020465840597e+01,     1.8905767703576468e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.3084602211403279e+01,    -1.8254194374798772e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3721359151396404e+01,     1.1168955481657392e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.5912778653973177e+01,     3.0388508889749704e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    4.8542333612263249e+01,     9.1162586173623185e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6152893627331814e+01,    -7.3434454973292923e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.2798321135119009e+01,    -1.5806906560251609e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -1.8131116350020918e+01,     3.2374938775514289e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.8475368873684243e+01,     6.8402200896879494e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.4403590404673640e+01,     1.5152363152807156e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7410404296048362e+01,     3.8943626774387936e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.7305149409325136e-01,     9.5206055254157800e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3562300064819716e+01,     1.7298309672818174e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {   -7.1157944886600177e+01,     8.3416304290250755e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {   -5.5393699200265196e+00,     1.2489885864648156e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5393455448250654e+01,     1.4456067038448809e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0548217423077662e+02,     8.0327141146728845e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.0615806609644967e+00,    -9.3631378906631107e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.2395081797177024e+01,     6.0602827226493901e+00,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {    5.6938536186641329e+01,     1.5441548983704757e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    2.6297733926871146e+01,     7.2063746592923726e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    1.6775262445362475e+01,     1.7762188989377264e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    6.5916766161271996e+00,     1.4580944312470227e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.8135149120302636e+01,     9.0340525979057184e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    8.1141878359158923e+00,     1.1403676944132160e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8993104464400972e+01,     2.0323440276312478e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -6.1486947370394986e+00,    -1.6662812554483247e+04,    -4.6127190445674666e-04,    -6.1343959884470698e-07,     2.9011250677594635e-09, } , 
      {   -2.3721068513167587e+01,    -9.1508159914537991e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.7296742822303791e+01,    -6.3509673379378091e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -1.0112706898903019e+00,     1.5543297881619450e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.8088323283289867e+01,     1.5542210698748468e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.2505267480674256e+00,     2.1954157569150248e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3289142593331702e+01,     2.6189720347055270e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.8631252330295219e+01,     1.4296128526099068e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    2.9371020773935804e+01,     2.1695157693587687e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.1022269151608775e+01,     1.7169442958817923e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.2711990050245483e+01,     7.4800176624891410e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.5740166357824961e+01,    -1.1169060756244030e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.3402401059382079e+01,     2.3242779100354739e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0863967905674212e+01,     1.1126238878401380e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -1.5724682779163805e+02,     7.2053320201377792e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.4766686574835610e+00,     6.6214115148901519e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.7577193906355241e+01,     9.3473835858764178e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.9304613671768607e+00,     5.5617968933585753e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0276652937826061e+00,     7.1404665360579547e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.2397518753001023e+01,     1.4041663716824403e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.4387755218354464e+01,     2.3883443325456461e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    3.3529123422335338e+00,    -8.3113785619907976e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {   -2.4356462136229347e+00,     4.0764540635146595e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.7489468803786572e+00,     7.7003733174757745e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2143974969974607e+01,     1.5401119995317242e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1224803910736272e+00,    -3.7498728570566163e+02,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    1.7090152171392980e+01,     8.3302470681576942e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    7.1829668045480830e+01,     7.1918732762858508e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.0038122200796920e+01,     2.4515295328243061e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6477309777308932e+01,     3.1098127818690296e+04,    -7.0521773756082872e-05,     2.0174066898330011e-07,    -9.6662151739619287e-10, } , 
      {    1.1393610102310623e+01,     3.1072889342045539e+04,    -1.9423071113744094e-04,     5.4047029170091065e-08,    -2.6829589112601728e-10, } , 
      {    5.9492552859170772e+01,     7.2137676025811415e+03,    -1.2355234256235969e-04,    -1.4459083225410801e-07,     6.8281158847467036e-10, } , 
      {   -3.7126609410274995e+01,     7.2143581283365238e+03,    -3.1344940116709871e-04,    -2.2833754752896926e-07,     1.0724078626142934e-09, } , 
      {    2.4474240983214695e+01,     1.2339083214983310e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2423433946315090e+01,    -1.2685703518850954e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -4.3973699490623908e-01,     1.6583786520049376e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -8.0657025849720654e-01,    -2.9426625696943083e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.0898671045865591e+01,     5.1489635374675436e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -9.9569153137024117e+01,     1.2096805966519960e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3764183522894506e+01,     5.3268389104824691e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.4135057149605448e+00,    -1.6217961213931923e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -4.0026844464215081e+00,    -8.3298454104460270e+03,    -5.8498084183810468e-04,    -7.6113323865791599e-07,     3.5994506940296389e-09, } , 
      {    9.5968613739358926e+00,     3.0235032291169275e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.3338922811103416e+01,    -6.5574035579433184e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -2.2124214845598260e+01,    -1.6910718408158180e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.3186199118561689e+01,     8.8487785670807916e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.5626798855588119e+01,     2.5032474940000720e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {    4.6350533725839949e+00,    -2.4939673608350036e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    5.6275183864258224e+01,     7.6074361511098950e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0076319864449875e+01,     1.5910038110355114e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -3.1063732540905949e+01,     7.6067315191937460e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0537230795410887e+01,     8.9571474268278816e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0670345677492051e+01,     4.2951388732329928e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    3.1246312932664004e+01,     1.0979979252448546e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    4.9687932790312477e+00,     8.3010168367817441e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.2012576818806462e+00,     6.4154016877504782e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.7961243205405736e+02,     7.2227937107798862e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4411744372234976e+00,     1.5505784302343836e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3929939496454519e+01,    -1.7391228524987014e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0069540873036647e+01,     1.6561781645082756e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6835055316501560e+01,     2.5064071161066615e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4926794385838637e+01,     2.4510062581210117e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5280999278049508e+01,     7.1648965378569665e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1280523255278997e+01,     1.8592947778784030e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -5.6072081233047037e+01,     2.6400218083379688e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.7441791150410268e+01,     3.2367097573119700e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1338251272208416e+02,     1.1094622236104777e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.9162822310908581e+01,     8.3197026103836233e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.8152681836235427e+00,     3.0863339876214817e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.7812340032311802e+01,     1.5258765809166322e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    2.2547906601680808e+01,     2.8306010070047036e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    7.0081205675762570e+00,     6.2457034079314935e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6894864362096058e+00,     7.8236348968362945e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -7.8560282978433627e+00,     1.8009912597017235e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.6520307620811664e+01,     1.5605386113033968e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    6.0255869948810563e+00,     8.3563660126685081e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8005667608496591e+01,     7.0781097290289872e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.5320972510558349e+01,     2.8492096146857439e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.0862922318148307e+00,     1.1566143927428740e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.0897367195319561e+01,     2.2697313837986043e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0759337747280870e+00,     8.2995941458479920e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8292932747136035e+01,     2.5290726271496525e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8286621018447491e+00,    -2.3686138970411967e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0782434259245946e+01,     1.5715907755563581e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2359862766480532e+01,     1.5650974732873838e+04,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    4.0985295765888580e+01,     3.8531460980371601e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    2.7554754015515527e+01,     3.0048432282113743e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.5464031933051983e+01,    -8.2112974534873865e+03,    -3.0679232754904747e-04,    -4.3192535663729670e-07,     2.0437320727172489e-09, } , 
      {    1.3996644062166094e+01,     8.3369034215035226e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.4613794291643121e+00,     1.5054166327063762e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.3749340946050461e+00,     4.9088966687259790e+03,    -2.1668262663509591e-06,     6.8896872222476152e-08,    -3.2894608263282014e-10, } , 
      {    2.4804759575896487e+01,     1.3038384105441248e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.9549947967931081e+00,     9.0863474307008437e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.8922527472544362e+00,    -1.6140408952919799e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {   -4.2743166222482934e+01,     2.5517479058658242e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2453688937969716e+01,     7.2559553101404454e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.9413958089535015e+00,     1.7425375845480856e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.4442594863098382e+01,     3.4501491090971620e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    5.9076117061570748e+01,     3.1731089162180317e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    3.7363692472366893e+01,     1.3189559504181952e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6466666813069114e+01,     1.2592403514323189e+04,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -4.7190814560877058e+01,     2.5811725400232055e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2575452239417118e+00,     1.5553072328666603e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {    2.4819507369457856e+01,     1.5532436251701314e+04,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.1586352952035014e+01,     2.2799476971598495e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.2160596977239493e+01,    -1.0869669718015315e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.4986246897262774e+00,     7.9640221775021073e+03,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    5.3453459049361540e-01,     8.5542403988803326e+03,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {   -2.9676152549272437e+01,     1.5335885148980460e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.7202158075058822e+00,    -1.6730979178851128e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    4.3824797193032893e+01,    -8.3203299421627416e+03,    -4.2817784384505619e-04,    -6.4541306111388087e-07,     3.0554897438247395e-09, } , 
      {   -1.0614164773141267e+01,     1.0624997466018518e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    3.1341849792459922e+00,     1.5765421667214136e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -5.0117499486307828e+00,     1.8795954536403209e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {   -2.2310540040552091e+01,     1.5543315450069713e+02,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.6861441726832922e-01,     1.9390480693945792e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8256925217885666e+01,     2.1655306507575951e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.3226695784281574e+01,     7.7007770859420243e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.3721964901039712e+01,    -8.8159788013935540e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -2.4560191324559803e+01,     7.8531871618640962e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.8472505473691150e+01,     5.7875394212939682e+01,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    9.8917641619622660e+00,    -8.1124058087112899e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    5.4825348189437628e-02,     6.7592563298416007e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.2489277257339522e-01,    -7.2025881370712350e+03,    -3.6864679623972648e-04,    -5.0577217654390120e-07,     2.3928948858523366e-09, } , 
      {    2.1046902593327275e+00,     7.0668702066570804e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {   -1.0034690709764437e+01,     4.2150102239329844e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    4.9105752244191780e+00,     1.5193885948989109e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -4.0811391926936409e+00,     2.4044912553397106e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {   -2.8915910008136834e+00,    -8.3460053517963479e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.8116070933288082e+01,     2.2740817647318305e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.6035345231463403e+00,     2.9683844598480937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9145484756200801e+01,     2.0508432301516576e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.0897252094243640e+01,     7.0612595432743183e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.4337837623227117e+00,     1.8402933566710220e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    4.8782423854538514e+01,     2.3842595319562035e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -5.2145108802631874e+00,     2.3871800318507194e+04,    -1.6330347679210141e-04,     9.0970439123393329e-08,    -4.4287729769356112e-10, } , 
      {    3.7002672618166557e+01,     2.3871091111310980e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    2.3304633789238110e+01,     4.1707891473438307e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {   -5.5368868765046049e+00,     7.1032546581485240e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.7179292514108440e-01,     2.3234501631480111e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.9403138482514095e+01,     2.1179081706494508e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4581867967724399e+01,     1.3799489513395674e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -4.6645929635347061e+00,     4.6544318594513161e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.8117105937018110e+01,     1.7035359266281102e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.8862272058297380e+00,     8.0170254856535885e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.1714214538536126e+01,     1.6954802614775741e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -5.9151573620685198e+01,    -2.3888941138747350e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3804820706553926e+01,     2.8188117425932218e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    8.4395293929744408e+00,     2.9055969382090312e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2906924886171467e+01,     9.2420682902920089e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    7.2919925866025217e+00,    -3.7384277577088085e+02,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    2.7643559774408629e+01,     3.5502039251518545e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.6785667133642501e+00,    -1.9132237850215970e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    6.1081494607673879e+00,    -1.7929745763056977e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    2.7477689385249302e+01,     1.7894321152742152e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.2430239318265190e+00,    -3.6463503628272872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5140248955365927e+01,     8.0802934594441704e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.3570732464451922e+01,    -8.5770893900060819e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    3.1734420649001031e+01,     4.7806859861285175e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3131655512434470e+01,     8.4629898293834594e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    4.0247297452327500e+00,    -2.2833297431362021e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1441382552454540e+00,     7.7638989785229223e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.3013704226295228e+00,     8.1993430037949529e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9597194465500095e+01,     1.4291070152248794e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.3820494548141387e+00,    -4.6698509704905064e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.4319154963632322e+01,     1.6488831783750102e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5375339776092778e+01,     2.6308390626558236e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6777426207231567e+00,     1.2562879842580220e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.9164942446654390e+01,     7.2510328532989533e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7628092752805536e+01,     1.5493587962582093e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.3139084989189811e+01,     1.4141438092043090e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.8652792767469279e+01,    -1.2566084565403598e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4036251948976176e+01,     7.2838927093642060e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {    8.4086881569111362e+00,     1.5605321323183762e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4458747144570992e+01,     7.0640989454321812e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.6763454926207601e+01,     6.5833674474381387e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7750117773862378e+01,     8.5449770407389624e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1178578104191726e+02,     7.6983483226625076e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1661227721380055e+01,     3.7201863567369233e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    2.2252645986364246e+00,     5.4700231937545723e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.2291454298039781e+00,     1.7270734101357681e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.4430108092958669e+00,     9.6628588558283373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3851808475299112e+00,    -8.7913986221177652e+01,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    5.0858815370197924e+01,     1.8611670986555469e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6673898456316941e+01,     3.0856780920615085e+04,     5.3030568806276818e-05,     3.4633150123740809e-07,    -1.6494331058708632e-09, } , 
      {   -3.2242208154424326e+00,     1.3748501707258909e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.1442300185551709e+01,     2.8523263001412262e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.6019719730029493e+01,     1.8909090751880962e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    3.2754659568072555e+01,     4.7288708746382272e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.3803466517936727e+00,    -1.8034529971758460e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.2740481790309016e+01,     6.5836130421436628e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.3511929636117301e+01,     6.6049334743353938e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {    1.5814509943913139e+01,     6.6574599312925611e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    1.5085688235942499e+01,     2.4271025640201802e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    8.3189330451290608e+00,     2.5770962227840449e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    2.1402642223608165e+01,    -7.2145615383359072e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.2246083900467127e+01,     6.7687605635749748e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.4965636287149991e+01,     2.9296187029598783e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7215316811099640e+01,     2.9316200956549528e+02,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    2.3474245206799029e+01,     2.2725177966409450e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.6851702511212316e+00,     2.5868813298896828e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1328179088329405e+02,     8.0985328586587057e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0107739374746536e+01,     2.2730508765016235e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -4.6630423511987544e+00,     2.3618110156201154e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -6.1146371605982530e+00,     1.3565030203396642e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.2926120402034574e+01,     1.5231088351112421e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4283605162058759e+01,     6.1217743749066649e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    2.4264832753927120e+01,     2.0145462192858773e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.6743994911421414e+01,    -8.9344571046396122e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.2069980127355580e+00,     1.4303059800901652e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.3601908665592513e+01,     1.5917741575889620e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {   -8.1925134340885521e+01,     1.3799720094298284e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.6503782313968479e+01,     2.0633977952837420e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9533654691810112e+00,     6.0945185334046691e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    3.3592478884902199e-01,     6.5600161452663679e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0937067952334555e+01,     2.3747492788071613e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    4.5887170944779623e+00,     6.6199888239563988e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -4.5913660256953222e+01,     3.1140865510554264e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.9465462147592817e+01,     1.6655083384607908e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.5170865743779999e+01,     1.6659682314292600e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8983724622764456e+01,    -1.5567362189139061e+04,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.2013471845165320e+01,     7.5819055364335245e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.6051923882813924e+01,     2.1531368840106534e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.1504128992804485e+00,     1.4408997995324036e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.5961020709296385e+01,    -9.6181216198714992e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -5.2487923379277301e+00,    -4.0945030068700335e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.0230914608803285e+00,    -1.4913343177745988e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    7.0953091455990993e+00,     7.7397379558586399e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7232761716122962e+00,     1.6344543973085572e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.4628657941091564e+01,     6.0587308533837302e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2343035149429369e+01,     3.1889599463109269e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.0835659722391796e+01,    -3.7575549044273303e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.8594981324683637e+00,     2.6762277913745056e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -1.5739606466463755e+01,     2.5061325335347767e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    8.9187316576221125e+00,     8.3577887036022603e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.5517103000279171e+00,     1.7818396815173845e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.1597532053316336e+01,     3.3303948987540789e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9475697344802519e+01,    -9.3680689228190313e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.2069180798986947e+00,     1.5652976961145590e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    1.0256654889273216e+01,    -2.2859240974459726e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.5717980608201962e+00,     1.0497916033987791e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -7.9571647678623414e-01,    -7.0560767463095544e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.3045556946038488e+01,     4.0546852772560538e+04,     2.0534331944762507e-04,     5.9674261566729451e-07,    -2.8357721835458977e-09, } , 
      {    7.8179507360130271e+00,     6.6725058051274445e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -8.3448985058018081e+01,     9.8552800627805445e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7987829303319337e+01,     1.5726225794046226e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    1.2409218024089972e+01,     1.3722532604447762e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9040737080354333e+01,     3.5959494571024203e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.2150363409230707e+00,     7.0655172990343126e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0066471900543906e+01,     2.8237233419605682e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2338349136633715e+00,     3.1639189233339554e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7088470937652751e+00,     1.8604438754559815e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3488319463260339e+01,     6.5768951249738257e+03,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -1.9621653065811913e+00,     7.2405814820562246e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.7898825486325709e+00,    -1.9551154424587430e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.4986444634748077e+01,     8.4042024270673737e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4412217820109220e+01,    -6.5344388418000763e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4200821134213896e+01,     1.7279079034942504e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2579089726995961e+02,     1.5018349382543722e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.4812132254158435e+01,     1.5584646734865570e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.3178992209473069e+01,     1.5322104278011020e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -5.1631284672497330e+01,     8.2847577213736367e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0288534717868146e+01,     3.0508677530161367e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9402805548594582e+01,     6.0476191947022498e+03,    -1.5664640317405017e-04,    -1.1261736998493412e-07,     5.2844691240939417e-10, } , 
      {    2.6059031289819096e+01,     1.7501300183049167e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -9.6100984106774812e-01,     6.6659543396887502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8714885877773128e+01,     1.6290506580923238e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4119784177903192e+01,     2.2769436393965167e+04,    -2.2283452439743112e-04,    -4.8670445446586314e-08,     2.1971756027884158e-10, } , 
      {    3.8962760648927635e+00,     2.4985850617119344e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {    1.6520236675798220e+01,     2.4986297931231413e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9683117866403592e+01,     2.2788456344876133e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4806465981969826e+01,     1.4446149939118872e+04,    -6.2240855498249710e-04,    -5.9136603194378982e-07,     2.7871938526987224e-09, } , 
      {    3.1940503163915910e+00,     7.5944645968294408e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.2404303218596880e+01,     1.4549811668855687e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9211804547209887e+01,     2.6389607087247183e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2754742812588535e+00,     1.2627314192392885e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -2.5724664351372923e+01,     3.2143049240969445e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -7.2205855339715406e+00,     1.7589356526363222e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3634133753575348e+01,    -2.2036617719044261e+02,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    2.2511128085418420e+01,     7.2046203512191087e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -1.0828388302556792e+01,     9.4209799152772939e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    2.0103773520611746e+01,     7.0945107543412096e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.8089853225314243e+01,     2.8023824115992062e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.3529456624637294e+01,     1.4705368277206253e+02,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -7.4427929722105048e+00,    -2.4970074765850892e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    1.2460910043635996e+01,     7.7467845055049274e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.4590463886940499e+01,     1.5908543312187589e+04,     3.6647996997337556e-04,     5.7466904876637734e-07,    -2.7218409684851566e-09, } , 
      {    1.6555074179635966e+01,     1.5125150156690039e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7340059876065030e+00,    -1.0141457412942338e+04,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.5336637099810506e-01,    -9.1490736424176048e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.9299782937297074e+01,     1.7590342431816945e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1727271654405227e+00,     6.5664107450223605e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1905324745923457e+01,    -7.3657369370766128e+03,     9.4948529302369512e-05,     4.1873357637430620e-08,    -1.9479813706981153e-10, } , 
      {    4.7372250350464933e+00,    -1.8322766732749962e+04,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {    7.3691698020421992e+00,    -8.2506945378338878e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.3564205497872667e+01,     3.1057122646957658e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -5.1311319567383578e+00,     1.8211417784866826e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5227457454388485e+01,     2.8581138395625207e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.3569297774195547e+01,     8.1971388457856428e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.3417593351094748e+01,     6.1967108666211516e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.2006236336717997e+00,    -1.0834501557518111e+03,    -5.5622043375911615e-04,    -6.5531295648213760e-07,     3.0959232048292748e-09, } , 
      {    3.1521486314184134e+01,     1.5579724278024079e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5530934255439494e+01,     2.2764295322211849e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.1321114104874596e+01,     1.6452749156407397e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {   -1.8541616039755880e+01,     8.3228998403994883e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1278498092559325e+01,    -8.5490576019155687e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.7642042639478660e+01,     2.2749338989073734e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    6.4101266592824917e+00,     6.6879269893401706e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.2635499312556307e+01,     2.6346300715674803e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {   -2.0203839099966387e+01,     1.5547012045943953e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.3218753667714662e+01,    -9.5792731204212650e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.2176731718183962e+01,     9.7152262568511105e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -1.6578144807997248e+01,     1.5582216876123797e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5209947702384286e+00,     8.1781376537594251e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    4.7274381997105245e+01,     1.5538496534423964e+04,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    3.5990325110353552e+01,     4.7154398746670544e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    9.3025548954587975e+00,     4.7861521571880676e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8395044322877506e+01,     2.1799922950716926e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {   -2.2505716665309023e+00,    -4.0235097014576368e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -6.6711975122097833e+00,    -4.4875239765630213e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.7852735178568446e+00,    -9.1021141257887211e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.6460404856675790e+01,     1.6539499681121557e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0243224532183024e+01,     2.1669418558440768e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.4306628361079721e+01,     3.1113894513778177e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3413378855666487e+01,     1.7304818170089424e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.6365851251752851e+01,     1.6657243873420564e+04,     4.2833443866405457e-04,     6.4851586867298184e-07,    -3.0710037816202447e-09, } , 
      {   -2.8929662194350250e+01,    -1.6657521825479940e+04,    -1.8091656390133843e-04,    -3.5312858904656384e-07,     1.6743525290798933e-09, } , 
      {    4.2594626685319930e+01,     3.1103557523723292e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.1914404075030482e+01,     1.7265479250254612e+02,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    8.1784789385667239e+01,     7.2323022646733671e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    4.5287378563098208e+01,     8.2403224523532845e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8401456570599514e+01,     1.1803335908533216e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2597407362711666e+00,    -1.7765452453116817e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {    1.3191047103213904e-01,     1.6651953144417261e+04,     1.4797909810864633e-04,     3.8820485887483870e-07,    -1.8442312429406747e-09, } , 
      {   -8.0810406855212484e+00,     1.4878364077795715e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    2.0575788275663189e+01,     2.0935884124439715e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.1702089645020251e+01,     4.6039770187404240e+04,    -1.9856436367014286e-04,     1.9184077361504338e-07,    -9.2618805639165756e-10, } , 
      {    1.7221334853488234e+01,     8.8145326776410402e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1362303997309031e+01,     1.3999056344822267e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9940222597957931e+01,     8.3619473710233106e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.7586057997902614e+01,     5.7999190562545236e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4749181291975763e+01,     2.1091433272343074e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    3.0498273905217319e+00,     2.2783125546269348e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.0504209325295715e+01,    -7.1676774590015357e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.7334416402395107e+01,     3.0781529859442693e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -3.0779147534786278e+00,    -8.4792451956908262e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -2.4891783182890457e+01,     1.5543067663292364e+04,    -5.3411709929817881e-04,    -3.4590486519803176e-07,     1.6210715055259552e-09, } , 
      {   -4.8827243122700231e+01,    -1.5542440917075557e+04,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {   -4.6488710432812539e+00,     1.3355480737565920e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.4250624633833709e+01,    -1.7932260025509095e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {   -1.3320561541510649e+00,     1.5518146391228860e+04,    -4.7009580434114888e-04,    -3.4095491751390340e-07,     1.6008547750236875e-09, } , 
      {    2.2847057470381870e+01,     1.4378923119201976e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.1277533479470449e+01,     1.5984193290559528e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    7.1790649000829951e+00,     2.3852267520986978e+04,    -1.6113665052575047e-04,     2.2073566900917170e-08,    -1.1393121506074101e-10, } , 
      {   -3.4496137988945605e+01,     8.2507868477549273e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3052815099433149e+01,     8.6585416974812665e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3351232823630383e+01,     1.5396509923599173e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.2350182643929955e+01,    -2.5062304633395461e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.9856787652492450e+01,     9.9843878472318502e+03,     2.8760408078988558e-05,     1.0582028217577842e-07,    -5.0352748920036409e-10, } , 
      {   -2.3592542589610964e+01,     5.7172300478592715e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0023507390895638e+01,     2.5981665679392077e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.2397696689191058e+01,     2.4238729535080238e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    3.7291356088745133e+01,     3.0788088815042425e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.3047826338369216e+01,     1.3771484656415427e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.7572618407214605e+01,     2.1273775525638612e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    3.6521620384630651e+01,     1.5222199557983586e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5971326929970537e+01,     1.1397608121096637e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5713959589687271e+01,     2.4867037120125784e+04,    -3.4654346177878919e-04,    -1.9636408525979537e-07,     9.1804318654901718e-10, } , 
      {    1.3940864994132747e+01,     7.7621883571760682e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5033708478528915e+01,     3.1307677284521018e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    1.3085191518422537e+01,     2.0163404722132313e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.5345713454960951e+01,    -1.2047333194032781e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    4.9692622211989708e+01,     3.7766865065148711e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.1659496576240080e+00,    -8.2823337270672273e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {   -2.0823095326732521e+01,     1.6171420904738303e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3837479766665190e+01,     8.6590805715681054e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    3.0010511760796636e+01,     7.7450687544654211e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.5391902195720863e+00,    -1.7037784580820859e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -1.9424069722537070e+00,     4.7849437418885118e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.5837867719194705e+00,    -1.8139295228887699e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {   -3.8623639816815007e+00,     3.6834330938216735e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.0525312859092139e+01,     2.6694464424953505e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.6665311560340555e+01,     3.0914656314828029e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    7.3717048109943599e+00,     9.7521962446912312e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {   -2.7583465256939270e+01,     7.0414080729562866e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    2.9379860895489912e+01,     4.8893568405486869e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    2.7535829218057454e+01,     4.7585157075477735e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    3.5163785510479151e+01,     3.1935984869566561e+04,    -5.0318986495283941e-04,    -3.0898145524472946e-07,     1.4464900989584113e-09, } , 
      {    2.6535294481195798e+01,    -4.3340473779825825e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.6776741665365464e+01,     3.1400977289048784e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.5590195795241705e+01,     1.0148681157172012e+04,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.9775468186969530e+01,     3.9378151588690642e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    8.9009826739121767e+00,     2.4802602770313111e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    1.4591420849785163e+01,     2.6374686649085063e+04,     2.4270160727288259e-05,     2.4051121906162970e-07,    -1.1459056166704991e-09, } , 
      {    9.0003060027046455e+00,    -7.5492084409388790e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    1.5919733998900591e+01,     5.6218429252107273e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.1025029174008054e+00,     1.2169858872887231e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.8498630374689512e+01,     4.8135912399511159e+04,     1.7224925883593459e-04,     6.2871607793646839e-07,    -2.9901368596111741e-09, } , 
      {    7.0142739680690394e+00,     2.8662948412397323e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.2895373148081664e+01,    -9.4072715675890086e+03,    -8.9393999565350317e-04,    -1.1241617230727366e-06,     5.3142366841140673e-09, } , 
      {    8.4375545540928414e+00,     2.6369256944052067e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.3995339840258485e+01,     2.4171166680074311e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.5944435895456834e+00,    -1.0866824803957525e+04,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {   -4.0890809930735461e+00,    -4.2587963168101942e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7456064719631218e+01,     2.5214801933928211e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    5.5681072040066955e+00,     1.6813963218069781e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -4.0208515044309845e-01,     1.6323321605978525e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.6634673834302465e+00,     2.9296615363829505e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4110549412667156e+01,     2.7941340822824015e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    3.6619138826064059e+00,     5.2257751744315044e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4147568664124437e+00,     1.4527984691045156e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.7481367589980312e+00,     2.4768821342810144e+04,     5.5197395072627777e-05,     2.7743462901493195e-07,    -1.3204870232380430e-09, } , 
      {    8.8710908274863254e+00,     2.1513319896751160e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7985478921427664e+01,     6.2571686605329714e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.5414548212224457e+01,     1.6991444092921978e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.6895803101763800e+01,     1.6546049160935163e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    1.7225531950234025e+01,     2.3889479398896408e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.7053124381153747e+01,     1.6155062366905016e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9550304531843036e+01,     1.2964300719889986e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5506865163593021e+01,     2.8334361792517004e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.8517085514133136e+00,    -9.8881560651746968e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.7867719517465279e+01,     3.2216136647958694e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.7837564806344805e+01,    -1.2259622477813837e+03,    -3.3988638816073795e-04,    -3.9995189436812281e-07,     1.8893673966519725e-09, } , 
      {    4.0276537235705963e+00,     6.3278378466679108e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2001717465085434e+01,     7.4519793131275728e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1100499182779515e+00,     8.1735869027008775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8424797579828427e+00,    -1.0551099822414951e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    4.3177692383566635e+00,    -1.6768716537965342e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    3.2083411498620418e+00,     6.4345798816725855e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    2.3553810421192441e+01,     3.1101508088692397e+04,    -3.5087711431149114e-04,    -5.8570340814843039e-08,     2.6015102128337689e-10, } , 
      {    6.3420987665536490e+00,     2.0885012311705617e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.2190630326591403e+01,     7.6675688146242974e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0831862315627822e+01,     1.1676978943066752e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.4345043550689841e+01,     3.1069509072043431e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.2720360683917615e+00,     2.3259137638188025e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.5657678711893727e+01,     7.2320965494461561e+03,    -6.2024172871614613e-04,    -6.6026290416626596e-07,     3.1161399353315424e-09, } , 
      {    3.4169609137858483e+00,     2.0569543603024751e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2189501275893456e+01,     8.4143547029158981e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.0619984784979444e+01,    -8.2430281465343542e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -5.8603709491582975e+01,     8.5598163932392254e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.4883030438174719e+01,     2.1890962985894665e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0852052954094813e+01,     1.5107135424274953e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    6.0318880578209182e+00,     1.3890488318083702e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2326452978496679e+01,     1.3413356131778859e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.4243970537036450e+00,    -1.7588539904854279e+04,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    1.8273639695694468e+00,    -8.5222413822313629e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    1.8839953741755817e+01,     1.4444125239242147e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.8625131908063423e+01,     3.2184137631309728e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.2572540109742429e+01,     9.7518598741063997e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.8702252589988682e+00,     1.1609862569241914e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2945283837163508e+00,     1.7782732152473704e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.2226145952714496e+01,     1.5564329792867333e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.6146887688835381e+01,     2.3227138621539059e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.0143762882308600e-01,     6.6549337129663854e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9646500831858713e+00,     6.5677215964259631e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    5.4736678933021832e+00,     1.6678001411961228e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.9313437720769400e+01,     1.5521178787500581e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.7200640751648191e+01,     2.1893222955244692e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9786174602005353e+01,     2.4344648777930379e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.1222881931784139e+00,     6.6563564039001385e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7781488483950407e-01,    -9.4112750983798207e+02,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    7.7187814485461539e+00,     2.4072921457597899e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4247547420219160e+00,     4.6982026929604763e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6134994724882539e+01,     7.5552687236615302e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -1.4231963096249654e+01,     2.0995392929146205e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5059819334895739e+01,     1.3564531530519567e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.2602691066150221e+01,     9.6251214967141241e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8894444906993897e+00,     1.5156292276090699e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0070997910877249e+01,     2.5851928443923505e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.8802941188696067e+00,     2.0962564572717740e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.8550228690593880e+00,     2.9448990351783297e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    6.6870302760062454e+00,     2.0491991342012629e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.8352310064169499e+00,     8.5110089428664905e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0147670016687513e+01,     1.6636764286939277e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.3101865582080521e+00,     1.3497467348390714e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.3564706473318644e+00,    -1.5510452202424858e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9322649698685918e+01,     3.0221914379969818e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    8.9286366922419163e+00,     1.8637865578464673e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.5211359541673076e+01,     7.9461787809720572e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    4.1600982159240933e+00,     8.1463739065837617e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.9185521104776171e+01,     2.4634740033936796e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.2202982075739888e+01,     1.2069999912759740e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.6278511640382138e+00,     1.3020335162085874e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    2.1720552015902737e+01,     2.3760112026393992e+04,     1.1705186376330681e-04,     3.5128144892153645e-07,    -1.6696498363731309e-09, } , 
      {    1.3711193218206009e+01,     6.7692267842755555e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    9.0678311196681740e+00,     5.8254504905491485e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    6.5637136623589356e+00,     1.7964640193738484e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.1509552065984654e+01,    -8.8946422027417011e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.9347911342818875e+00,    -1.9151578184339633e+02,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {    1.1939010102187099e+01,     1.6145324725707669e+04,    -4.9885621242013746e-04,    -4.4677519968968179e-07,     2.1043822642240518e-09, } , 
      {    9.2276617355592752e+00,     2.1355585542410729e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4067309083834401e+01,     2.5929850567901784e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.7800701009669444e+00,    -3.0951112882807138e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0150597838778470e+01,     1.6408608459932086e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.5411546038625379e+00,    -1.6428655189697423e+04,    -4.9003231253573519e-04,    -7.1925988102048537e-07,     3.4046525569598272e-09, } , 
      {   -8.1774969171477618e-01,    -2.1293981584050971e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.1141409633321125e+01,     7.0797028382603103e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7084983625817660e+01,     2.2035209566069483e+04,    -6.5550261559418758e-04,    -5.5939256967461593e-07,     2.6328291766334460e-09, } , 
      {    1.1304170100156517e+01,     2.9842011321476286e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.3551461182482639e+01,     1.9770383752439324e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.2436729040462380e+00,    -8.2304756474094938e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    1.6407893025745750e+01,     2.8727382762209993e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.7999107753897565e+00,     1.5001197229852571e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    7.2160454805117613e-01,     7.4405330120983353e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.7992868980103999e+01,     1.5640970067499587e+04,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    1.6332997177269966e+01,     1.5351238508340561e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    1.3176653910077368e+01,     2.0398691337484866e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.1255119680537149e+01,     8.8687327075914163e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.0921310682522112e+00,     5.4324295208561616e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.9644412692946647e+00,    -2.5097407962690468e+04,    -4.2601101757870521e-04,    -7.1430993333635701e-07,     3.3844358264575595e-09, } , 
      {    1.9353727343130121e+01,     2.4874740585660289e+04,     4.8786548626938422e-04,     7.8815675324296151e-07,    -3.7335986395926476e-09, } , 
      {   -5.1647872472579287e+01,     7.2356383681422085e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.7426340446557990e+01,     2.3969661492224714e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {    1.2010855651652767e+01,     1.4914452334935477e+04,    -6.3524239730472436e-05,     6.3330532424185123e-08,    -2.5428962387876272e-10, } , 
      {    9.6553010794008660e+00,     6.5857609102103506e+03,    -2.1583699037182070e-04,    -1.8708058200570130e-07,     9.3204945379627173e-10, } , 
      {    3.9694765807946748e+00,     7.7003894694766441e+03,     1.5497663213425675e-04,     2.4979472231572374e-07,    -1.1318993494232446e-09, } , 
      {    7.0248178708887119e+00,     8.9569933799736082e+03,     1.4964886914843974e-04,     2.5102750654404910e-07,    -1.2407788059268243e-09, } , 
      {    4.6692632986368112e+00,     6.2830195524848227e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.5066196941746805e+01,     1.6171056245432441e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {    1.2710642369494904e+01,     7.8423648207073147e+03,    -2.2116475335763771e-04,    -1.8584779777737599e-07,     8.2316999729269210e-10, } , 
      {    1.2053981333400646e+01,     1.4286150379686995e+04,    -6.0860358237563932e-05,     6.2714140310022456e-08,    -1.9984989562697296e-10, } , 
      {    6.7261374666020144e+00,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    1.4366410223904667e+01,     2.3243143759660601e+04,     8.8788510910875812e-05,     3.1374164685407152e-07,    -1.4406287015537972e-09, } , 
      {    1.0441339160738760e+01,    -1.7429305145147757e+03,    -3.6814974101316892e-04,    -4.3749169643558775e-07,     2.1183885314713062e-09, } , 
      {    9.6984267611487454e+00,     5.9574589549618686e+03,    -2.1317310887891217e-04,    -1.8769697411986398e-07,     9.8648918204806139e-10, } , 
      {    6.3250311530465755e+00,     1.6029080894201770e+04,     3.0728938277560503e-04,     5.0020583674561016e-07,    -2.3182384270982792e-09, } , 
      {   -1.8551860951847868e+01,     8.3263902248853919e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9546155599817300e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3803724431406117e+00,     1.7285684804698736e+04,     3.0196161978978796e-04,     5.0143862097393552e-07,    -2.4271178836018586e-09, } , 
      {    1.5140732604919886e+01,     1.5542754290183959e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2667516687747025e+01,     8.4706667759557968e+03,    -2.2382863485054624e-04,    -1.8523140566321331e-07,     7.6873026904090223e-10, } , 
      {    4.0126022625425533e+00,     7.0720875142281620e+03,     1.5764051362716528e-04,     2.4917833020156111e-07,    -1.0774596211714549e-09, } , 
      {    2.0052234722510860e+01,     2.2128515200394308e+04,    -2.8202511159520165e-04,    -1.2313365746735350e-07,     6.2332010166571914e-10, } , 
      {    1.7421751513998704e+01,     2.4499747670157569e+04,     8.3460747925058777e-05,     3.1497443108239688e-07,    -1.5495081580573769e-09, } , 
      {    1.2756849312935840e+01,     7.2140628654588327e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9816921891408334e+00,     9.5852953352220902e+03,     1.4698498765553121e-04,     2.5164389865821173e-07,    -1.2952185341786140e-09, } , 
      {   -3.3625969613438613e+01,     8.3283391087670516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4620264876860517e+01,     8.3290437406832007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6261376168889328e+00,     1.2566039104969645e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    1.7696680150258960e+01,     1.3799823775669183e+04,    -4.3433786223654987e-04,    -3.7354477189723997e-07,     1.8096591793407536e-09, } , 
      {   -1.0510498597748654e+01,     1.5540453090344225e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7587553282816771e+01,     1.5545055490023693e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2407789294762761e+01,     3.0457206625119434e+04,    -1.2971236095385340e-04,     1.2727745696253293e-07,    -5.6301897600931531e-10, } , 
      {    1.4409535905652547e+01,     2.2614841804412121e+04,     9.1452392403784316e-05,     3.1312525473990889e-07,    -1.3861889733020075e-09, } , 
      {    1.2097107015148524e+01,     1.3657848424438513e+04,    -5.8196476744655414e-05,     6.2097748195859803e-08,    -1.4541016737518314e-10, } , 
      {   -1.2866068952918287e+01,     7.2117616656190985e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5524561715421850e+01,     8.3117707439163460e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5232013626789737e+01,     7.2163640652985669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.0816920388539160e+00,     1.6728370525065588e+04,     1.1655480853674925e-04,     2.8300096881322299e-07,    -1.3951434819921005e-09, } , 
      {    7.0679435526365904e+00,     8.3286914247251261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5707963267948966e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5121755479399068e+00,     7.0987675615335547e+01,    -1.8807069274594723e-04,    -2.1782126004654988e-07,     9.7753467335796839e-10, } , 
      {    1.6721964796156566e+01,     3.1571835184385727e+04,     2.4110126155222405e-04,     5.6415276128395794e-07,    -2.6269677792288316e-09, } , 
      {    5.7968679017395193e+00,     8.3294027701920022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1973122194822245e+00,     8.3279800792582500e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0484464842486640e+01,    -2.3712324697632580e+03,    -3.6548585952026044e-04,    -4.3810808854975037e-07,     2.1728282597230963e-09, } , 
      {    1.1896637815713380e+01,    -1.9524799803454973e+03,     6.4518350183587553e-05,     7.3230427792441848e-08,    -2.9472308488329798e-10, } , 
      {    2.3107576012604895e+01,     2.3385119110891275e+04,    -2.8735287458101871e-04,    -1.2190087323902819e-07,     5.1444064516213961e-10, } , 
      {    2.0095360404258738e+01,     2.1500213245145827e+04,    -2.7936123010229317e-04,    -1.2375004958151618e-07,     6.7775982991750901e-10, } , 
      {   -2.5782739495401408e+01,     1.5542401974225884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2859760089318684e+01,     1.5543106606142033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.9539814836875635e+00,     7.1430751898434974e+03,    -3.0430179118781966e-05,     3.1357070155011228e-08,    -9.9924947813486478e-11, } , 
      {    8.0857845884868595e+00,    -1.0071621939239902e+04,    -5.2046249165451719e-04,    -6.8790281086547417e-07,     3.3047276091463408e-09, } , 
      {    1.7464877195746581e+01,     2.3871445714909085e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.7415524428966229e+00,     5.3291569997133856e+03,    -2.1050922738600366e-04,    -1.8831336623402663e-07,     1.0409289102998513e-09, } , 
      {    3.2489987486013668e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1495255660661709e+01,     8.3290200572015747e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.6805857252984744e+00,     2.4357772318926895e+04,     4.5960213341695325e-04,     7.5061695117549658e-07,    -3.5045775047733139e-09, } , 
      {   -1.6196478482373255e+01,     1.6655081649610518e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {   -2.8398241196013689e+01,     7.2137105495007590e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.0766097469616284e+01,     7.2144151814169072e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1735927015392512e+01,     2.5614376229423862e+04,     4.5427437043113619e-04,     7.5184973540382194e-07,    -3.6134569612768932e-09, } , 
      {    3.1901896575137027e+01,     1.6659684049289986e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1566209756880070e+00,    -8.2577037491097908e+03,    -3.4038344338729550e-04,    -4.6823237447643627e-07,     2.1638737510330028e-09, } , 
      {    5.5695192159140907e+00,     9.1665428458043298e+03,    -2.8301922204831677e-04,    -2.5969461768398047e-07,     1.1723328104277799e-09, } , 
      {    9.5410832434614790e+00,    -1.0281171405070623e+04,    -8.7794400457760695e-05,    -1.7718068663744457e-07,     8.9161599279173642e-10, } , 
      {    2.5240291575327030e+00,     8.1709570703846939e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5023071259998925e+01,     1.6799358200680923e+04,    -7.1515884209197975e-05,     6.5179708766673123e-08,    -4.1760880863413212e-10, } , 
      {    1.2748929666618711e+01,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.4684077548362318e+00,     8.4864257790655574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3681568347944530e+00,     1.5400778938953288e+04,     3.0995326426851350e-04,     4.9958944463144748e-07,    -2.2637986988464892e-09, } , 
      {    5.8387617786074486e+00,     8.3260615924089434e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1556417306838433e+00,     8.3313212570413107e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5852470458148574e+01,     8.3114184279582696e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2450914976510639e+01,     2.9828904669870954e+04,    -1.2704847946094487e-04,     1.2666106484837025e-07,    -5.0857924775752544e-10, } , 
      {    1.7995360554545655e+01,     1.4357138055302328e+04,    -2.4893105098351117e-04,    -1.5510711973652738e-07,     7.7768477773099544e-10, } , 
      {    2.5141779258200545e+00,     7.9099389353073657e+03,    -2.7769145906249971e-04,    -2.6092740191230583e-07,     1.2812122669313596e-09, } , 
      {    2.5463130584856795e+01,     3.1713810535616401e+04,    -1.3504012393967041e-04,     1.2851024119085823e-07,    -6.7189843251289484e-10, } , 
      {    2.0752021440352994e+01,     1.5056427686166147e+04,    -4.3966562522236693e-04,    -3.7231198766891461e-07,     1.7007797228371739e-09, } , 
      {    1.0567710782002996e+01,     1.5385019935843527e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3679998545591587e+00,     8.2949099972221611e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.7984833636451505e+00,    -8.3624728522280911e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.2957101276567116e+00,     7.2309835462676137e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.9425167340036332e+01,     1.8684292215411675e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1703694619053326e+00,     1.4197535123067109e+02,    -3.7614138549189446e-04,    -4.3564252009309976e-07,     1.9550693467159368e-09, } , 
      {    1.7739805832006837e+01,     1.3171521820420701e+04,    -4.3167398074364139e-04,    -3.7416116401140260e-07,     1.8640989075925435e-09, } , 
      {    8.2119561772688012e+00,     7.0563285111184005e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4767516537460107e+01,     1.5613741965799292e+04,    -2.5425881396932824e-04,    -1.5387433550820208e-07,     6.6880532122741591e-10, } , 
      {    1.9777306086250604e+01,     3.2828439094882691e+04,     2.3577349856640704e-04,     5.6538554551228330e-07,    -2.7358472357324109e-09, } , 
      {    1.4252192387965280e+01,     6.3762114443796290e+03,     2.1683110082493579e-04,     3.2364154222232824e-07,    -1.4810621625583325e-09, } , 
      {    1.7841057251434634e+01,     8.3155735135256709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.6465195048798602e+00,     8.2888765228272132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3641915629414884e+01,     8.3685063266230391e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.0992997539063722e+00,     1.4991998966150539e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.4763343867014662e+01,     3.8785898049844560e+04,     2.2600389687494852e-05,     3.7768857139241929e-07,    -1.7493580536843496e-09, } , 
      {    4.8697324980719552e+00,     1.6238630360032492e+04,    -1.2537870842115148e-04,    -1.0516287482419388e-08,     9.4873189256325051e-11, } , 
      {    1.0645835407520213e+01,     1.5020384899560799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9995332249911035e+00,    -8.8150180287429375e+03,    -5.2579025464033415e-04,    -6.8667002663714881e-07,     3.1958481526427615e-09, } , 
      {   -1.7820660680426592e+01,     8.3512333298561825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4018906019144403e+01,     1.5540124457867776e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3058744946147485e+01,     1.5545384122500143e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.1550951079881848e+00,     2.3869144515069351e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.1696640794659992e+01,     7.2114330331426490e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7209322473207763e+01,     2.2685829480027453e+04,    -9.6618300342162913e-05,     9.5303994693359024e-08,    -4.0865429994403896e-10, } , 
      {    4.7834811345761983e+00,     1.7495234270529458e+04,    -1.3070647140696849e-04,    -9.2835032540940612e-09,    -1.4006267247254537e-11, } , 
      {    1.0668828346675099e+01,     7.2166926977750172e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9943214722179007e+01,     2.3873746914748819e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9668670879073520e+01,     1.7427677045320590e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0786180092130477e+00,     7.7779361006917070e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0355087797243005e+01,    -4.8632660401781118e+02,    -3.7347750399898598e-04,    -4.3625891220726239e-07,     2.0095090749677269e-09, } , 
      {    6.7458926500196474e+00,     6.6633075414254135e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3150701694352772e+01,     2.2756817155642791e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    8.2904469424382565e+00,     6.6916934748356716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5047278536022731e+01,     8.3131493964366964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.4053457193039250e+01,     8.3442334530135558e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.3806933367257717e+00,     1.5502939388286046e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7629994373497986e+01,     1.4916747904978150e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0053368391985652e+01,     7.1610937682476415e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.6687937986278314e+00,    -6.9587606984853278e+02,     5.9190587197770524e-05,     7.4463212020767181e-08,    -4.0360254138687761e-10, } , 
      {    3.0209384691613952e+00,     7.1742479635609207e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3784430817170962e+01,     8.2490616209293021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4781553186409276e+01,     8.4083212285209502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7794365853974480e+01,     1.6799369460275044e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6199074799129876e+00,     7.8063220341019651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3874116401182059e+01,     8.1578391591852396e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1707232926553939e+01,     1.5582569192081872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.4234981248884893e+00,     1.6657382849450252e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.9357984741407616e+01,     7.2538777673567447e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2400264870881772e+01,     1.5489785192972768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1270445620640576e+01,     1.6657030533492180e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.6975844798185904e+01,     1.6657735165408325e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.9087070015644514e+00,     8.8794467487585462e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0903820850221759e+00,     7.2133515199919557e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4275083033080742e+01,     7.2147742109257097e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6487428736981862e+01,     1.5700488644524392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0528598439237101e+01,     1.5542425657707512e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3451546025093865e+01,     1.5543082922660407e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0585239194949743e+01,     8.3640284956606611e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4119372869198624e+01,     7.3717972197992640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6369692780392771e+01,     8.3654511865944132e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6750707276625334e+01,     1.4912145505298682e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.5984269114356637e+00,    -1.1856162348816290e+03,    -1.8274292976013022e-04,    -2.1905404427487519e-07,     1.0864141298615481e-09, } , 
      {    4.0557279442904317e+00,     6.4437855589796791e+03,     1.6030439512007379e-04,     2.4856193808739843e-07,    -1.0230198929196651e-09, } , 
      {    1.2309536055939464e+01,     1.5471766614568623e+04,     1.2188257152256629e-04,     2.8176818458489763e-07,    -1.2862640254885208e-09, } , 
      {    8.3734825521218781e+00,     8.8510608153482863e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.7123071109712008e+01,     2.3942433390524420e+04,    -1.0194606332797995e-04,     9.6536778921684358e-08,    -5.1753375644761854e-10, } , 
      {    1.2243678879459635e+01,     7.2670319626700239e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3037358801015779e+00,     8.2757223275139349e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.3372467613927324e+00,     1.7913986759947216e+04,     2.9929773829687949e-04,     5.0205501308809820e-07,    -2.4815576118536486e-09, } , 
      {    9.3046169506600378e+01,     1.5527212261895529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0681474192465259e+01,     7.1985208371704030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6819127346472778e+00,     8.3816605219363173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8316660062006406e+01,     7.2296048937472624e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.5971863690224666e+01,     1.5558296318472387e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8428920219397924e+01,     8.5508601288782265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2819475421906816e+01,     1.9312599800457217e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4616676358490770e+01,     1.5595723387395150e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1772548177216041e+01,     1.4914094389180342e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.7026600418751414e+00,     1.5463124486388133e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6765090477904444e+01,     3.0943533229137247e+04,     2.4376514304513258e-04,     5.6353636916979526e-07,    -2.5725280509770416e-09, } , 
      {    1.7251662477006271e+01,     1.0355600790686549e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.0532912940751444e+00,     7.1344330616630086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.6660998996609013e+00,     8.3446909330496110e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4452661587400424e+01,     2.1986539849163637e+04,     9.4116273896692834e-05,     3.1250886262574621e-07,    -1.3317492450502176e-09, } , 
      {    6.9519088545274048e+00,     8.3110593984494699e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5506256266604673e+01,     3.1085508580367918e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    9.8246044384439291e+00,     9.0279810555889435e+03,    -3.8421823597507498e-05,     3.3206246497499222e-08,    -2.6324413256885588e-10, } , 
      {    4.8855048770792706e+00,     8.3624484718625718e+03,     1.2148344865959285e-04,     2.1344407124528431e-07,    -1.0124848916291548e-09, } , 
      {   -2.9671969210143225e+00,    -8.2949343775876805e+03,    -1.8314205262310364e-04,    -2.8737815761448853e-07,     1.3601932637209141e-09, } , 
      {    3.8177604358130786e+01,     7.2137342329823841e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.5811654291760153e+01,     7.2143914979352812e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0432789459889499e+01,     7.2936926692546567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3611055957433049e+01,     8.3124820893832220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2801003270316237e+01,     1.5622384093979783e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7880340851391775e+01,     1.6640462168641468e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6790446046812747e+01,     1.4915158050605292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7081800003667347e+00,     8.3284677676690917e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.2861436450850867e+00,     8.3289150817811606e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0027379066314122e+01,    -1.6920680808781185e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.4048995987283927e+01,    -2.3011998397341813e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8093613793368949e+01,     2.9342578065853137e+04,    -5.0052598345993093e-04,    -3.0959784735889214e-07,     1.5009298272102012e-09, } , 
      {    1.0660307953515355e+01,     7.1365106044467084e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5094041162028597e+01,     8.2588016467051548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2140232696896401e+01,     1.3029546469190031e+04,    -5.5532595251746910e-05,     6.1481356081697136e-08,    -9.0970439123393348e-11, } , 
      {    4.5830119351410552e+00,     1.8849058657454468e+03,    -7.9916444787255359e-06,     1.8491763424879922e-09,    -1.6331918475536940e-10, } , 
      {    1.2318353142923591e+01,     1.4445046411726446e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6310207751485265e+01,     8.4706780355499195e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2993508361655158e+01,     1.5465202029171833e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1860614220409555e+00,    -8.3126919164006413e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0594627716275888e+01,     1.4994076508934240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9077519368408467e+01,     3.9900526609110857e+04,     3.9341401219357229e-04,     8.1456387571384436e-07,    -3.8133068569038663e-09, } , 
      {    1.1437246611105817e+01,     2.5057061949790717e+04,     2.6886755917809749e-04,     5.3341208324310941e-07,    -2.5814825596671350e-09, } , 
      {    6.8555145118456497e+00,     6.5147732345950153e+03,    -2.7766297625873455e-05,     3.0740678040848568e-08,    -4.5485219561696674e-11, } , 
      {    1.1582223903583735e+01,     7.2916151264709570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4450662792734713e+01,     6.5864666258801662e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3220709496643064e+00,     1.4286139120092872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2768914539275693e+01,     1.4339108217304063e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.6400270505524297e+00,     8.5044594630528827e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9151978805233579e+01,     1.3590274309838462e+04,    -1.6697710397934066e-06,     1.3717735233078965e-07,    -6.0345243701385052e-10, } , 
      {    1.1024348370879732e+01,     7.6328153548765931e+03,     2.1150333783911878e-04,     3.2487432645065360e-07,    -1.5899416190619122e-09, } , 
      {    1.1939763497461259e+01,    -2.5807819355939796e+03,     6.7182231676496057e-05,     7.2614035678279181e-08,    -2.4028335663150822e-10, } , 
      {    6.9385665073929559e+00,     1.0213597290470572e+04,     1.4432110616262271e-04,     2.5226029077237441e-07,    -1.3496582624304039e-09, } , 
      {    5.0387241149143573e+00,     8.2511391637130018e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1134826113427220e+01,     1.4128404765752441e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9189207185738013e+00,     8.4062436857372504e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.2564558603974270e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3916531850875696e+01,     1.5620306551196083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0449168365620849e+01,     3.7671269490578263e+04,    -3.4821323281858260e-04,    -5.9186732929005705e-08,     3.1459074953516671e-10, } , 
      {    2.0708895758605117e+01,     1.5684729641414629e+04,    -4.4232950671527541e-04,    -3.7169559555475198e-07,     1.6463399945853842e-09, } , 
      {    1.5639805982293755e+01,     6.0284466305772021e+03,    -4.0124380162485939e-04,    -4.0551823416641386e-07,     1.9640238554060298e-09, } , 
      {    7.2371990072398029e-02,     8.3785142107920387e+02,    -4.3533197268966504e-04,    -5.1010573211386689e-07,     2.3586718881028146e-09, } , 
      {    7.5446240707431143e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1878249158147339e+01,     6.5857552804132893e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5275203064666066e+01,     6.5880564802530243e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.4106486665517522e+01,     6.5854029644552147e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5983274003795147e+00,     8.9569990097706686e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3959670513038720e+00,     7.7003838396795827e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0845497683539946e+02,     8.3269604562466993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0138486086006619e+01,     2.0871911289897344e+04,    -2.7669734860938464e-04,    -1.2436644169567883e-07,     7.3219955816929878e-10, } , 
      {    2.0795147122100872e+01,     1.4428125730917665e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.3132392263759298e+00,    -9.0245674945736591e+03,    -9.3122163443577730e-05,    -1.7594790240911927e-07,     7.8273653628815689e-10, } , 
      {    1.8890110522259121e+01,     1.8055984630366132e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.9707311635525116e+01,     7.7000315237215082e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8224165916366001e+01,     6.6653850842091142e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7244234891537822e+00,     7.2478442929617968e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    7.0726248804429117e+00,     7.2206218210585603e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.4098124800220001e+01,     7.7364322560819937e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5142365443417233e+01,     2.3871798030867158e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.3354439241847746e+01,     2.3871093398951012e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.5668650652741704e+01,     7.2313358622256892e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.6397914045227147e+00,     8.3200549263416979e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -1.9106121264331790e+01,     6.5834540805735560e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.4242629829224782e+00,     1.2566151700910864e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7019649485236826e+01,     8.2716035260603567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0250162812261232e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5341125578007061e+01,     5.4711323509440572e+03,    -5.8665061287789815e-04,    -6.2395588632712639e-07,     2.9959982570157880e-09, } , 
      {    1.1513201951638687e+01,     1.5542042944717083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.3460213738049820e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2327363944019451e+01,     8.2353914201973639e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.9579664793087758e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5512117827162390e+01,     1.5543465635650835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6426949623238485e+01,     1.6065123680807119e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2025699481179322e+00,     8.9562876643037926e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6072270927037483e+01,     1.5288322850508055e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.8442712933865295e+01,     7.2139648138849934e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1080865240413380e+02,     7.2141609170326719e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7628178732394666e+00,     7.7010951851464597e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5553554618798000e+01,     7.2850505410741662e+03,    -4.0657156461067646e-04,    -4.0428544993808850e-07,     1.8551443989024503e-09, } , 
      {    2.4710522440721765e+00,     8.5382408905558477e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    8.1521864147618253e+00,     1.6658094194917128e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.5523106838914114e+00,     1.6656671503983376e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.2496051896093753e+01,     7.7800136434754077e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8633572402694707e+00,     7.2353927734366844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2500485617585204e+01,     7.1927329574809810e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7238648135826118e+01,     1.5328137752405968e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9887258601343341e+01,     8.9438810985712153e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.3716728609869996e+01,     6.9596314257829290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4792078254849791e+01,     7.6980826398398494e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.9661066648315035e+00,     6.1163549870013203e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5750476584399120e+00,     1.3763769729469712e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7818685157108696e+01,     4.0042501960341528e+04,     1.7272626701677830e-05,     3.7892135562074465e-07,    -1.8582375101879293e-09, } , 
      {    2.4806469548762539e+01,     3.8157596094596076e+04,     2.5264271180403363e-05,     3.7707217927825667e-07,    -1.6949183254325599e-09, } , 
      {    2.3064450330857017e+01,     2.4013421066139756e+04,    -2.9001675607392719e-04,    -1.2128448112486551e-07,     4.6000091691034980e-10, } , 
      {    1.6052704222519839e+01,     2.3452693225491323e+04,    -3.4387958028588066e-04,    -1.9698047737395802e-07,     9.7248291480080704e-10, } , 
      {    1.9820431767998482e+01,     3.2200137139634211e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {   -1.1477784919484787e+01,     8.8773692059748446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4854218989858159e+01,     1.5521424382206107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1737431122859213e+01,     8.9546978099309345e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.4420579495401196e+00,     1.4521425735445428e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1718555509237966e+01,     1.5033891410832151e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.9654610148457579e+00,     7.2271807766582879e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0073875804337742e+01,     9.7272932056410064e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2221203789708577e+01,     1.5564084198161810e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.8850705832240426e+01,     7.7690759452522452e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.4882924545945208e+01,     6.9994463276808419e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.5530826769471531e+01,     7.7736783449317118e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.2996909836828594e+01,    -8.3441105357643155e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.1452487923491992e+01,     1.5073353996771990e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.4692629844403413e+00,     2.2754515955803057e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.5629078115688884e+01,     2.2759118355482529e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5579774579861775e+01,     8.3647398411275371e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -8.5983788938611312e+00,     8.3908245746980610e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2816231633951674e+01,     1.4914799021096491e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8028597567093126e+01,     8.0742599850492225e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0413005500069664e+01,     6.0104167925789370e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3697149650267939e+01,     1.5124001800766198e+04,    -4.9619233092722899e-04,    -4.4739159180384447e-07,     2.1588219924758415e-09, } , 
      {    1.0527590524234517e+01,    -2.9995344250117405e+03,    -3.6282197802735191e-04,    -4.3872448066391305e-07,     2.2272679879748859e-09, } , 
      {    8.1289102702347389e+00,    -1.0699923894488384e+04,    -5.1779861016160861e-04,    -6.8851920297963685e-07,     3.3591673373981309e-09, } , 
      {    9.9108558019396860e+00,     7.7713771450919794e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.6119196306626273e+00,    -8.4672532149405124e+03,     9.2284647809460994e-05,     4.2489749751593286e-08,    -2.4923786532160132e-10, } , 
      {    3.3002160071579478e+00,    -4.1875248941776067e+02,    -4.3000420970384798e-04,    -5.1133851634219225e-07,     2.4675513446063939e-09, } , 
      {   -8.6087316434317017e+01,     1.5542656238610120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1316445979911724e+02,     1.5542852341757798e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9207275019378550e+01,     8.1140748869471354e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0355894228466189e+01,     1.8849227551366296e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2034330695473992e+01,    -8.3160721864027473e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.9886999909661526e+01,     8.3413106630475049e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {   -7.0345915402500676e+00,     8.5831228644010298e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4957179947321844e+01,     8.2092467190313891e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8214190366431147e+01,     8.5433079625031169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1217646538879073e+01,     8.2937465357151395e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.5957244003193217e+01,     8.4481361304188631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3121177108068771e+01,     1.4861477607927225e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4253800206491345e+01,     8.4066883116163644e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.7815142278135294e+00,     5.7997133410273145e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9357456579986241e+01,     6.7051999861070262e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5787644897222409e+01,     8.3227913706387099e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6621659063579848e+01,    -8.3467403680805000e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2645288632361968e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2645098686178507e+01,     1.6833150887778011e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.6796424232981678e+01,     5.2615828851133356e+03,    -1.5398252168114166e-04,    -1.1323376209909678e-07,     5.8288664066118393e-10, } , 
      {    1.8038486236293533e+01,     1.3728836100053846e+04,    -2.4626716949060270e-04,    -1.5572351185069006e-07,     8.3212450598278531e-10, } , 
      {    1.1036140297550375e+01,     3.2686463743652021e+04,     6.1191488405830147e-04,     1.0010280656053830e-06,    -4.6909165824483477e-09, } , 
      {    6.0784077884676320e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1019028705524896e+00,     6.7446625720468637e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3834531572397989e+01,     1.9170613189632426e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.1234224278941873e+00,     2.5132303401821728e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1368913428145607e+01,     7.1915209603277763e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.6767049064410440e+00,    -8.3456121055339063e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.0174947670444315e+01,     1.5072181059478848e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4523933758618641e+00,     7.8592911313131572e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.3148889342558991e+01,     8.5359396907161135e+03,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {    3.0161202596759056e+00,    -1.2903965975940507e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -1.8911304721707900e+01,     8.3444239088727172e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {   -8.5215255350585281e+01,     8.3044185511156447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6210095501819779e+01,     8.3529642983346093e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.2517848711890426e+01,     7.4286794032368234e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1232607929018489e+01,     8.5405420903955819e+03,    -4.3266809119675651e-04,    -5.1072212422802957e-07,     2.4131116163546042e-09, } , 
      {    7.7105972452440747e+00,    -1.1146285592662935e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7898045716005655e+01,     1.6093509614217375e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.5738059221117052e+01,     2.1013886641128011e+04,    -6.5283873410127910e-04,    -5.6000896178877861e-07,     2.6872689048852357e-09, } , 
      {    2.2494040658258520e+01,     2.9200602714622470e+04,    -1.2438459796803637e-04,     1.2604467273420759e-07,    -4.5413951950573568e-10, } , 
      {    1.5254874214511306e+01,     6.7277362614410213e+03,    -5.9197837586371510e-04,    -6.2272310209880103e-07,     2.8871188005122087e-09, } , 
      {    1.4810642219207985e+01,     1.4985440010550810e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {    5.7302300162349598e+00,    -1.8400313363965030e+04,    -6.7277524229586547e-04,    -9.3831392529536059e-07,     4.4910666868213751e-09, } , 
      {    1.4091481587644413e+01,     3.3943067654148988e+04,     6.0658712107248451e-04,     1.0022608498337084e-06,    -4.7997960389519279e-09, } , 
      {    3.4847282196865130e+01,     1.6657054216973800e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7020104204976203e+00,     9.4263993031826394e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.3565695479557842e+01,     1.5525833609375179e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {   -1.9141871727084627e+01,     1.6657711481926701e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    7.5269912199684992e+00,     8.7980917181370951e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6038717904870801e+01,     3.7698455102732592e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3737511686494894e+00,     1.4914446705138416e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3418981914430098e+01,     8.3237603925692056e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.1678291554300460e+00,    -1.7429361443118366e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.3660362247979805e+01,     7.8198285453733197e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3840923926441089e+01,     2.4983773074335644e+04,     6.0925100256539299e-04,     1.0016444577195457e-06,    -4.7453563107001378e-09, } , 
      {   -2.2650114534373021e+01,     8.8375543040769335e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0955836317314478e+01,     1.4598977996457552e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.3540742404749722e+00,     7.4684943051347364e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2714195898473591e+01,     8.2335195926408724e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.8401215846257784e+01,     1.5367952654303879e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9562606150864781e+01,     1.5407767556201792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.8188005240520141e+01,     8.2317886241624456e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3007652559182543e+00,     7.3073628699865958e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2385563116879341e+01,     8.4177089383387283e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1549827348675521e+01,     8.1937046907429594e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.7732467966524119e-01,     7.3849151309987192e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.3594696832843681e+01,     1.4898904676849985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.1721703687755820e+01,     8.4238632568093799e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4257488669580070e+01,     2.4988375474015109e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.3695013338526699e+01,     1.6168760675389767e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.3880160980774335e+01,     8.2733344945387835e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3729589560592927e+00,     8.0132227160442617e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1576367106415272e+01,     3.1415379252277162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.7204932969742870e+01,     7.0790761314766660e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0804062565132142e+01,     6.5327861832020981e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.0557683848448633e+01,     8.4636781587072928e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9875639830615789e+01,     3.4227046505595630e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    2.1275270403499455e+01,     9.3369020682642022e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.2887261605311224e+01,     8.3840483549114670e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6225760347502217e+01,     1.6486530583910364e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.6044848853558555e+01,     7.0392612295787540e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7118898439266562e+01,     4.7114589474569686e+04,     1.7491314032884309e-04,     6.2809968582230571e-07,    -2.9356971313593844e-09, } , 
      {    2.2207320095327614e+01,     1.4846878220335426e+04,    -6.9975340256104300e-06,     1.3841013655911496e-07,    -7.1233189351743005e-10, } , 
      {    1.7252448154955641e+01,     2.2057527524778972e+04,    -9.3954418849254408e-05,     9.4687602579196358e-08,    -3.5421457169224920e-10, } , 
      {    1.7378625832250826e+01,     2.5128049625406049e+04,     8.0796866432150273e-05,     3.1559082319655950e-07,    -1.6039478863091666e-09, } , 
      {    4.8148148896534320e+00,    -8.1867160734944555e+03,    -5.2845413613324273e-04,    -6.8605363452298613e-07,     3.1414084243909714e-09, } , 
      {    1.4150168645462209e+01,     1.5508997243046513e+04,    -3.5358819241625563e-05,     1.0091396772294991e-07,    -4.8258353817643211e-10, } , 
      {    1.6607746960217181e+01,     1.4704902869104755e+04,     3.6914385146628409e-04,     5.7405265665221466e-07,    -2.6674012402333669e-09, } , 
      {    8.7119194803757090e+00,    -1.3241780250970151e+03,     6.1854468690679035e-05,     7.3846819906604514e-08,    -3.4916281313508780e-10, } , 
      {    6.2701135446280514e+00,    -8.3962655393251771e+03,    -9.5786044936486234e-05,    -1.7533151029495659e-07,     7.2829680803636702e-10, } , 
      {    9.5842089252093583e+00,    -1.0909473360319105e+04,    -8.5130518964852191e-05,    -1.7779707875160725e-07,     9.4605572104352629e-10, } , 
      {    8.7237114070463537e+00,     2.3729470363678414e+04,     4.6226601490986172e-04,     7.5000055906133390e-07,    -3.4501377765215238e-09, } , 
      {   -1.1427312809154239e+01,     8.3132723136859368e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0168374008811131e+01,     1.5757370827961951e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.0380094624374202e+01,     8.1538897888450474e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7492868569795892e+00,     1.5542530633127924e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.7327765736419980e+01,     1.5542977947239990e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7550299035519370e+01,     1.5508972862680992e+04,    -3.5260886878041436e-05,     1.0087033449165005e-07,    -4.8331075869809644e-10, } , 
      {    3.4413624337578206e+01,     8.3336224568810467e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7216956839803812e+00,     7.2839526434788049e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0011461436829343e+00,     1.5797185729859861e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.1195674826282893e+00,     6.1927343107203023e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8994037173970861e+00,    -8.3352503803248546e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    7.8212367448468294e+00,     6.7434896347537206e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6849670281842464e+01,     1.5423309584490224e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9387490412835891e+01,     8.5034930606052058e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.7126519116082932e+01,     1.5518481416574477e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0480863908571365e+01,    -8.3129589405775332e+03,    -4.0174085685141695e-04,    -4.7379871427472732e-07,     2.2385302097870602e-09, } , 
      {    6.6195531186657863e+00,     8.6441601334059906e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.9471791049794547e+01,     7.1897899918493513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2684836177270316e+01,     1.4917076537454601e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.0723751862277396e+01,     1.5447582458099705e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9199974566590367e+01,     7.0946181597650957e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2830031362953518e+01,     1.5558753798508444e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.2885462304725010e+01,     1.4325954021990785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.2022700528615502e+00,     8.3500213327029778e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7912876576301002e+00,     8.3073615167472744e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0686168112413668e+01,     1.6173363075069235e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.8364361346708776e+01,     7.1188910333745789e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4247005660040529e+01,     1.5526754781859474e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3957652483576677e-01,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0184042365074501e+02,     7.2383357390683150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1073962230216416e+00,    -9.4433199839914196e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.2298975064223850e+01,     3.0943521969543126e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.0420974156058057e+02,     1.5567027163793440e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0452095670662391e+00,     1.4874631803240503e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.8146756268643109e+00,     7.7627406467085511e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2765392443543202e+01,     1.4498015508937637e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.2315593012973284e+01,     1.0983908375732091e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.8796353860559298e+00,     1.6499648495109821e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    8.7393498685849504e+00,    -1.1153399047331698e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.2515452806885152e+00,    -6.5589555997275966e+00,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    8.9152505716654087e+00,    -1.1139172137994171e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.2804722937872754e+01,     4.5999960915303389e+04,    -1.9590048217723436e-04,     1.9122438150088073e-07,    -8.7174832813986779e-10, } , 
      {    1.7782931513754718e+01,     1.2543219865172219e+04,    -4.2901009925073286e-04,    -3.7477755612556528e-07,     1.9185386358443331e-09, } , 
      {    1.9851765523075713e+01,     6.5181867956102997e+03,    -1.5931028466695867e-04,    -1.1200097787077145e-07,     4.7400718415760441e-10, } , 
      {    1.9478625681963909e+01,     3.2271124815249546e+04,     5.0366687313368307e-05,     3.4694789335157077e-07,    -1.7038728341226531e-09, } , 
      {    1.7095104637268374e+01,     5.8188971647464805e+03,     3.1424289571897069e-05,     1.0520389006161575e-07,    -4.4908776094857428e-10, } , 
      {    1.2926883947937363e+01,     1.5576511337321404e+04,    -9.7017423205136323e-05,     2.6979881353745674e-08,    -1.3487516608467297e-10, } , 
      {    2.2132860658502505e+01,     4.1157130519607825e+04,     3.8808624920775523e-04,     8.1579665994216972e-07,    -3.9221863134074464e-09, } , 
      {    9.7846781246445005e+00,     4.7008550444649036e+03,    -2.0784534589309516e-04,    -1.8892975834818931e-07,     1.0953686385516411e-09, } , 
      {    5.6439786527392029e+00,    -1.7143709453468062e+04,    -6.7810300528168242e-04,    -9.3708114106703523e-07,     4.3821872303177949e-09, } , 
      {    1.4295318069713160e+01,     5.7479094891311470e+03,     2.1949498231784432e-04,     3.2302515010816561e-07,    -1.4266224343065428e-09, } , 
      {    7.1855286712095801e+00,    -1.8609862829795751e+04,    -2.4010715109910896e-04,    -4.2759180106733102e-07,     2.0779550704667709e-09, } , 
      {    4.4137085760979922e+00,    -5.5731427963314673e+02,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    7.6415525931835422e+00,    -1.8139181901301113e+03,    -1.8007904826722169e-04,    -2.1967043638903787e-07,     1.1408538581133378e-09, } , 
      {    2.8010664034361072e+00,    -1.6586395173834913e+04,    -4.9269619402864367e-04,    -7.1864348890632269e-07,     3.3502128287080375e-09, } , 
      {    4.6534909196294958e+00,     8.5044838434184021e+03,    -2.4952603857365287e-04,    -2.2334396661354104e-07,     1.0529183526336901e-09, } , 
      {    4.7134896402967371e+01,    -1.1142762433082185e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5104477629557426e+01,     1.6650823893850524e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.9472952494106721e+01,    -1.1149808752243684e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.1884948531540985e+01,     1.5487397359997618e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9523244593898681e+01,     7.1587059352724909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.5825097027050901e+01,     8.3279867928089770e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6819340652374365e+01,     8.3293960566412752e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2711448800717935e+01,     1.0213614179861755e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7171543490345487e+00,     6.4437686695884968e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2177198261502244e+00,     5.4350783047445857e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9266145532197854e+00,     1.4756712350797985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3522751032080750e+01,     8.8871092317506973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.3503030525268386e+01,     1.6640109852683396e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2385648642237133e+00,     7.1787257945232986e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0824182263099189e+01,     1.6815117203790684e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4906843330687103e+01,    -6.3017820587637034e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    4.9158099371500484e+01,     1.4739645069258337e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4842918460597858e+01,     7.3490495994409994e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2923985410876426e+01,     2.3713711360568654e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4216484121615718e+01,     7.8423704505043761e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1371334320707760e-01,     3.1083207380528183e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    4.7984635265340316e+01,     3.1087809780207655e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.8336337116857209e+01,     7.1802814379558667e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    9.5267535578802551e+00,     1.5576535717686924e+04,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    1.1864474536913125e+01,     1.6623601421947289e+04,     3.3555273562803600e-04,     5.3774563881307509e-07,    -2.5472595619176130e-09, } , 
      {    5.5838489014953163e+00,    -1.6691164276953219e+04,    -2.7369826693735699e-04,    -4.6389881890647059e-07,     2.1980967487825249e-09, } , 
      {    6.5852357302721840e+00,     7.2316948917344898e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.8312709711052420e+00,     7.1773031035895456e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.8248072113628275e+00,     1.4425824531077931e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -3.3449970227983457e+01,     8.6056647695320844e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1269710582097460e+01,     6.5702132521248595e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1919936095179645e+01,     9.5853065948162130e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.2564164349629063e-01,     7.0720762546340393e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.4484895270385181e+00,     1.4363691381104996e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2497069692703526e+01,     1.5677741024166125e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7019143435807653e+01,     7.7229257448106400e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3860703188831739e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1956519501572949e+00,     1.6654753017134070e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.5097205386614121e+00,     1.6660012681766435e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.2318720632916174e+00,     7.6474147424683915e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.6002381274829453e+01,     7.3092346975430864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5960452164125609e+00,     6.2702865717324257e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.7160840008041298e+01,     7.2694197956451735e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.3972079517990657e+01,     1.5662198995877696e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.1356165949597159e+01,    -1.7272996766856068e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.3681926020077999e+01,     7.3888645013389123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2867923990603302e+01,     1.5073706312730064e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.3593994750456915e-02,     7.2302722008007368e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3273518144708731e+01,     1.4430426930757400e+04,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    4.1591918463234407e+01,     7.3335075711525697e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.3657472454568278e+01,     1.5637926122268213e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.6203550441166445e+00,     1.4911816872822232e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.4815993340274275e+01,     1.5598111220370300e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.7995416405496094e+00,     3.2197835939794481e+04,     3.9075013070066376e-04,     8.1518026782800704e-07,    -3.8677465851556564e-09, } , 
      {    2.0385071202351094e+02,     8.3281834661870926e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    2.0228119553259694e+02,    -8.3291993832631579e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.6530586516951569e+01,     8.3931257745377952e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.3443758543379420e+00,     8.3985212686305003e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.0170539331817693e+01,     1.6644264938250795e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.0785470388010854e+00,     7.1081246710364503e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1471638233686223e+01,     1.5436816095761576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3467100251130599e+01,     8.6284123898903508e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2697245693754994e+01,     9.7299230379571891e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2298770953160002e+01,     3.2202438339473945e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.7949050138676917e+01,     1.4699830167360424e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1333751807550403e+01,     1.5717555926064038e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1448699433384302e+01,     2.3320690390875669e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.0245856853899518e+01,     8.0569869882823677e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0627349108942806e+01,     8.9566466938125941e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2667961518031802e+01,     1.5227285581503094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.6451162191940533e+01,    -8.2665582747521894e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0247387213173131e+01,     6.5883851127294747e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2329884850159397e+01,    -1.3117911199455193e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.0006114967506839e+01,     2.3245439329703277e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4117922800077492e+01,     1.4392788659982130e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.4116269840572748e+00,     1.6091432071433679e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5055019869849602e+01,     8.3106424813697522e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1416484477461911e+01,     8.0171720863844548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3934881592329127e+00,     7.2138392084027992e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.3195071555950747e+01,     8.3290504542339277e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4189366007634128e+01,     8.3283323952163246e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4972487936039695e+01,     7.2142865225148671e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.8364056557706476e+01,     1.4968127147816484e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9967044587422720e+00,    -1.2723629136067254e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0492294047368727e+01,     3.7042967535329779e+04,    -3.4554935132567413e-04,    -5.9803125043168372e-08,     3.6903047778695648e-10, } , 
      {    2.8136739475116826e+01,     2.8714276110604656e+04,    -4.9786210196702235e-04,    -3.1021423947305477e-07,     1.5553695554619909e-09, } , 
      {    2.7861810838856574e+01,     3.9414200005093044e+04,     1.9936508194586341e-05,     3.7830496350658197e-07,    -1.8037977819361394e-09, } , 
      {    1.7166196791459885e+01,     2.3314131435275936e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    1.1067474052627610e+01,     7.0045133996281111e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.4665090628191365e+01,     2.3800458039293750e+04,     2.7419532216391455e-04,     5.3217929901478405e-07,    -2.4726031031635556e-09, } , 
      {    1.2180159010695830e+01,     1.7356672480314071e+04,     1.1389092704384075e-04,     2.8361736092738562e-07,    -1.4495832102438902e-09, } , 
      {    3.5565783405685911e+00,    -9.7238571254374765e+03,     9.7612410795278029e-05,     4.1256965523267953e-08,    -1.4035840881802174e-10, } , 
      {    4.9128581798198336e+00,     1.5610328404784010e+04,    -1.2271482692824295e-04,    -1.1132679596582053e-08,     1.4931291750811484e-10, } , 
      {    1.1779052697140390e+01,     2.4986074274175378e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.3958686050017057e+00,     6.9918941613057323e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3001931933913172e+01,     2.3349076324285925e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.3456988032289043e+00,     1.4391365969048378e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1195985643988635e+01,     7.5255822037995040e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8229605136364890e+00,     7.8581181940200140e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.2968364392157126e+00,     8.4346296191475085e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6351565040825687e+01,     2.3868815882592899e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.1894618795887860e+01,     1.4834816901342590e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4596023533407312e+01,     1.5307467674831403e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5437507976545767e+01,     2.3874075547225271e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.4487843610742118e+01,     7.7007361556376572e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0929384384455298e+00,     6.9143419002936080e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2292766796983665e+01,     6.9787762501062771e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5921411680502036e+01,     2.3854525034100301e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    5.0510049539901424e+01,     6.7450148880049383e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.5775827938854206e+01,     8.5260349657362622e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0309427269705809e+01,     6.8985941567779691e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.4232045494692009e+00,     6.5459403785153772e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5919110774411184e+01,     3.1571829554588669e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.0226718935715972e+01,     1.6170350529762625e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4971941562691296e+00,     8.2485093313968173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4395154858977918e+01,     2.3240836930023808e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    9.1075130262384558e+00,     1.7285690434495795e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0473613600115820e+01,     7.2300623737833184e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    7.5031655827962940e+00,     8.4088735180534331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.7371847416201955e+00,     8.3853584036385128e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.3303167404069747e+00,     1.4966704456882731e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0018404427844317e+01,     7.4857673019015920e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.2888676317591205e+01,     2.3871117082432636e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.2864447802299175e+00,     1.4834264611810107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1100511970257809e+01,     2.3871774347385533e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.8051457536450517e+00,     8.4308631336520066e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2282660101122630e+01,     8.6454796714299973e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5911168485332370e+00,     8.2227532303027438e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.4815085266196483e+01,     7.8596434472712317e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.6817645660958760e+01,     6.4109536445332105e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.4611657792070631e+01,     8.5658498676341733e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1207672454489245e+01,     7.1971421846500516e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    1.0409514970116673e+01,     6.1693240842125115e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2851860306983625e+01,     7.8427227664624506e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0331705240694234e+01,     1.4779459971156250e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5341666933435780e+00,    -1.6653838832997126e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.0790145365308256e+01,     1.5536195334584230e+04,    -2.5159493247641971e-04,    -1.5449072762236473e-07,     7.2324504947920567e-10, } , 
      {   -3.4122593242139004e+01,     7.7710248291339039e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.2616243801960223e+01,     1.4859089774952074e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.6128202782062147e+01,     8.1694318171334780e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0802715667534187e+01,     7.7717294610500539e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.5123897532726680e+00,     6.5850439349464123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.9468689013454714e+00,    -8.8150123989458771e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    7.1488182639922002e+01,     1.4819274873054163e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2890039702003611e+00,     9.5323374976050218e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4916209367350394e+01,     8.3198312692856634e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.2769289962221226e+01,    -8.3375515801645888e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    9.6564442689116881e+00,     9.3500199794636574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7820986155207720e+01,     2.2756464839684719e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    2.0534680305214820e+01,     5.9972625972656588e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0981167111838062e+01,     2.2757169471600864e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -2.9684404942374343e-01,     1.6617567947552339e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -2.8158878774905961e+01,     7.8420181345463016e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.5060691429097091e+01,     8.5378885745977732e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.6003334880201098e+01,     1.6697197751348165e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.3144414511502703e+01,     8.5385932065139223e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.0463586456356161e+01,     1.5558173401223148e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.5392854276998783e+01,    -7.9587275331634828e+02,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5706968402738426e+01,     7.6848418113911530e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3794227158370330e+01,     8.0776570658569317e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.7133131441336019e+01,     8.4879510323167742e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6683111916246602e+00,     7.0382948271310752e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.1560939353106554e-01,    -9.6190880223191762e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.6026537799628620e+01,     7.8400692506646410e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8847929363592961e+01,     2.4029180069249516e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.6466781302866718e+00,     9.0099681069818598e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8239422274175148e+00,     1.4392077314515254e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.3865452714337518e+00,     1.5527335179144769e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.0771089313756850e+00,    -1.6369979498894538e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.9053979100055571e+01,     1.7272572523296341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.5903989884995227e+01,     1.4929988733426844e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6009527310737688e+01,     6.6394357230913574e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7363406408188755e+00,     2.3831630813011172e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4751545050553018e+01,     2.3818476617697892e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.0488784924113133e+01,     1.7665593492989326e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6691252014533994e+01,     7.9225525438326831e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1380772773882985e+01,     8.3347248991200740e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.9811256282968976e+01,    -8.3226579503301782e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    7.4529990983152121e+00,     6.4280209260728570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.3412950864727019e+01,     1.4914775337614865e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.0298203203971198e+01,     8.9573513257287450e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7228309988770727e+00,     7.1250453518452305e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.4085493520614811e+01,     1.5560803233539333e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7046530981230681e+01,     2.2541905131056410e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5626644618144759e+01,     1.6012154583595928e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1729298191113763e+01,     7.2321118088142066e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4118780577787224e+01,     1.0122944007650178e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.1177870868494946e+00,    -8.1529233863973686e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {   -3.3816005473422067e+01,     8.7977394021790205e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.0086549687822668e+00,     1.5524705346828585e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.9408037883277526e+01,     8.3593443470348284e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    8.3212831798064872e+00,    -8.1867048139003327e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.8108603395237353e+01,     7.2294819764980220e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4060943106003137e+01,     2.3911260616806998e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.4021850918046681e+01,     1.5113521214627977e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6785921637849651e+01,     1.6171061875229501e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9332198397560578e+00,    -4.8632097422075026e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2584319953332711e+01,     7.3200010598812150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.8363031747928961e+01,    -1.1169297591060276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.9350953353635617e+01,     8.1919737222645344e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4657462503832525e+01,    -1.7085226553988696e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.5613062212238582e+01,     6.3711387426352985e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.1593111793331591e+01,     1.4285786804134796e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5996360545183972e+00,     7.2490077544688183e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    6.2545820009959456e+00,    -1.1172583915824780e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.8832099795930670e+00,     9.7430409491566443e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1910476377915963e+01,     1.8058614462682315e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.2602673646954159e+00,     6.5831254480971056e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1399090914958848e+01,    -1.1119987269501098e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.4424533071366161e+00,     1.3500097180706898e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4598718068619927e+01,     1.6051617169535766e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.9751495474087761e+00,     6.6380130321576044e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2295032625572684e+01,     8.3129436812094864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.5016158395717966e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8880372825578693e+00,     7.8416591050374991e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.6938205397877255e+01,     8.4862200638383492e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3504509655714891e+01,     3.8927873401075231e+04,    -3.5354099580439967e-04,    -5.7953948700680372e-08,     2.0571129303158710e-10, } , 
      {    3.1148955083462990e+01,     3.0599181976350104e+04,    -5.0585374644574789e-04,    -3.0836506313056678e-07,     1.3920503707066215e-09, } , 
      {    1.9195104486981457e+01,     1.2961972354589980e+04,     9.9411045311510506e-07,     1.3656096021662697e-07,    -5.4901270876206065e-10, } , 
      {    1.9564877045459664e+01,     3.1014520904752579e+04,     5.5694450299185329e-05,     3.4571510912324541e-07,    -1.5949933776190734e-09, } , 
      {    1.3867260620182828e+01,     7.0755010752434446e+03,     2.6096526586080048e-05,     1.0643667428994108e-07,    -5.5796721745215386e-10, } , 
      {    1.1794614073210308e+01,     7.1803058183213871e+03,    -1.8767156988297380e-04,    -1.4949714670693654e-07,     7.0375553949860234e-10, } , 
      {    9.1248177206017935e+00,     1.6100068569817106e+04,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    7.4690498661920284e+00,     6.9928963086381782e+02,    -1.9073457423885576e-04,    -2.1720486793238720e-07,     9.2309494510617863e-10, } , 
      {    2.0581540038460924e+00,    -8.8860057043582729e+03,    -3.3771956189438697e-04,    -4.6884876659059895e-07,     2.2183134792847925e-09, } , 
      {    7.2252870703238559e+00,     2.4567321784757616e+04,     2.6934042220196770e-05,     2.3989482694746702e-07,    -1.0914658884187093e-09, } , 
      {    5.5263935341662123e+00,     9.7948448010528118e+03,    -2.8568310354122524e-04,    -2.5907822556981784e-07,     1.1178930821759902e-09, } , 
      {    6.4112825165423315e+00,     1.4772476983704806e+04,     3.1261714576142204e-04,     4.9897305251728480e-07,    -2.2093589705946995e-09, } , 
      {    5.2077608380319722e+00,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7402115039840695e+01,     1.6641840821161823e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.0335583759248191e+01,     7.7540642823576509e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.1696425495499568e+01,     1.6672924877738682e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.6846567201923008e+01,     7.2711507641236012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1882061811424702e+01,     8.3301141156588783e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.8782994052563602e-01,     8.3272687337913721e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0912764965481328e+01,     7.4493494808113883e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.8914554701846239e+01,     2.4985721958217306e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    4.9331105298205401e+01,     2.4986426590133451e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    5.0764646486605017e+01,     6.5861075963713638e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1328096789350326e+00,     1.4205957026764565e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.6252710437454390e+00,     1.3735383796059454e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0371728068789205e+01,     1.4967415802349607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.4484838123924124e+00,     1.3710800632258521e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.2545544843861851e+00,     7.1338807721305247e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1613972257186674e+01,     1.5462572196855652e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.9340555119311755e+00,     6.5055731870849813e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5902121379942399e+01,     8.7085944832186578e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.7987321432395046e+01,     1.5383494682592311e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.9430605305318519e+01,     2.3242785813905470e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.6388508531837804e+01,     2.3872157060375961e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.4734996812977117e+01,     1.5648692484606341e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4239811617165619e+01,     6.5061254766174652e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5388502723361007e+01,     2.3870734369442209e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    6.0113449196636628e+00,     6.0349999563798701e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0847728799197364e+01,    -1.1675976564774849e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6990419596335560e+01,     2.3924414812120278e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.3331942690349404e+01,     1.8451635432375304e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9647755191680014e+00,     1.6135013458827092e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.4286984873280248e+00,     1.6106627525416832e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    7.3594137447425627e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7081281381558311e+00,     7.7526415914238978e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.1664661627489885e+01,     6.7848297899028512e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.4293069500605796e+01,     1.4914118072661968e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1961350904459632e+01,     8.3265491864555006e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    4.1086491063660393e+00,    -8.3308336629947516e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.5408002889381791e+01,     2.3855903686620659e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1763247494309823e+01,     6.0640972352570052e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.8453177397158068e+01,     7.2226993638422609e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.0278393805626749e+01,     6.5303983502269475e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.4794285876975888e+00,    -1.1544434611642057e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.9040821652293204e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1564756563313390e+01,     1.4913735359671540e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.9146974607784145e+01,     6.4905834483290355e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0339076436385092e+01,     7.0548032578671846e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2423268313054914e+01,     1.4836894444126290e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.3622720129551432e+01,     2.3886987743197511e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.1311888871948891e+01,     1.4957106521094120e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8330607017149504e+01,     7.8446716503441094e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.7990316662973726e+01,     6.4507685464311235e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8630659337795990e+01,    -8.3277250222774474e+03,    -5.5405360749276517e-04,    -7.2420982870461374e-07,     3.4248692874620948e-09, } , 
      {    5.9911249092098124e+00,     6.0626745443232521e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5979942957577961e+01,     7.8994583491691446e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1630481444216396e+02,     8.3266559077080510e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0531056969629540e+02,     8.3307269417422012e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.0789886875318286e+00,     8.7992646554302373e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3369138798963895e+00,     7.7030136719957682e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.7665603759671313e+01,     1.4288440319932606e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.9627859301890060e+00,     7.4650972243270280e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3680493460478974e+01,     1.5021096245027675e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.1429036841782544e+01,     1.6577753045654426e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.0317818841842286e+00,     8.5639780400776817e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4166185223351633e+01,     7.6834631588708016e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5752580719236908e+00,    -1.1080696036665659e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    6.5554957782331957e-01,     8.9033185670926014e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9278895676077170e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5478046371391521e+01,     1.6679924754581309e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.7137412372292211e+01,     1.6737012653246078e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.0156358117835858e+01,     8.3307689675088259e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.9161970140501765e+01,     8.3266138819414264e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.5292767815907951e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.3987826794413909e+00,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7643973150430355e+01,     1.6879551553603353e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.7976155616276166e+01,     7.6831108429127271e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.7927582252841383e+01,     8.8999111111059010e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.0153389632209866e+01,    -1.0748136573683812e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    8.6214831629757995e+00,    -9.5689420492586169e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.3146462354639043e+01,     7.2942449587871397e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5376350483137905e+01,     1.5537823258028038e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    4.2452826128087501e+01,     1.5547685322339883e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.3510295456384398e+00,     2.3791815911113259e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.0080478720321274e+01,     8.0338862323477715e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -7.4816360207020793e+01,     8.9725410380591002e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.4220940196893466e+01,     8.3284857094979179e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.5541018997872030e+01,     1.0515964977343167e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2585149439295961e+01,     7.9773571844865419e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3219255489333971e+01,     1.5778040905536514e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0739717355617605e+01,     1.7179752240073412e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    6.7451438634855183e+00,     1.1015106480668383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2378150536852375e-01,     6.5497068640108791e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.4753512180791432e+01,     1.8844656402068293e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1892572981072128e+01,     7.1980633571343469e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5669799893767546e+00,     1.4893116797160565e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.7047100928084305e+01,     1.0973555624994374e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2262458020289952e+01,     1.7208138173483672e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    4.3269189042751952e+00,     6.6026759612220712e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.8249698141370970e+01,     6.6012973087017181e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3438945483614338e+01,     8.8636990052429064e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1330584292045026e+01,     1.3869707923892094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2701270142863626e+01,     8.9047412580263535e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.1846803875246934e+00,     8.3989733846706458e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    1.0429039770422495e+01,     1.4286850465559748e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9922849563646898e+00,     7.7533529368907739e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0453958703816729e+01,     1.4350128844026427e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3476079003165715e+00,     8.9577103552375447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.4984163956102897e+01,     8.7579245002811076e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.2029251522271824e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.7426391725574302e+00,     7.1986437544196433e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.5484755892528982e+01,     1.5622936383512266e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2936624227508053e+01,     2.2216870415310086e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.6604431058451201e+00,     1.6604413752239059e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.9762343989570166e+00,    -8.3418093359245813e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.8432812177763758e+01,     7.2075039098591033e+03,    -4.0390768311776793e-04,    -4.0490184205225118e-07,     1.9095841271542401e-09, } , 
      {    8.2086626115839856e+00,     6.6387243776244804e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6417767469682856e+00,     7.6996724942127057e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0207333862293105e+01,     9.1318006456507483e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3501915382300425e+01,     7.6052120075953289e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.1146892932354817e+01,     1.4954261607036327e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.3271606883287603e+00,     7.6605689377816707e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2750697342531852e+01,     9.6654886881445200e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4649223491211771e+01,     7.6450269094932410e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.9077324655610424e+00,     8.9040299125594775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.0401313107793673e+00,     1.6710351946661445e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    9.5200519827087913e+00,     8.2764336729808110e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4738790179755261e+00,     8.3809491764694412e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.6946449266517165e+00,     1.5208693046712231e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -9.3639377636829479e+00,     7.1960139221034588e+03,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    5.1137750588159054e+00,     8.0934048093725705e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0198107108992343e+02,     8.3287894762989654e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -9.0987082807579441e+01,     8.3285933731512869e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1327923860432927e+01,     6.6924048203025486e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0569803112286500e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0701641503992480e+01,     8.3220800251718338e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.2793641561942923e+00,     9.3734302059714482e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.5815161848689652e+00,     1.6579830588438130e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.2341108392269192e+01,     7.5653971056974169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.9563546278404793e+00,     6.6284150963689926e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0799915580392877e+01,     8.1889718027997787e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0726597944651315e+01,     9.0371811030989775e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0854835180283988e+00,     1.6734935110462378e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.5162109193476240e+01,     2.3951075518704911e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.5168066141123361e+01,    -1.1301705875547232e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    2.6174924625959964e+01,     1.9237677371761281e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6802474595911647e+00,     7.8070333795688421e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0892839939975119e+00,     3.5337070935533859e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.0869985017519788e+01,     1.5558486774331548e+04,    -3.1561622743344969e-04,    -1.5944067530649309e-07,     7.4346177998147333e-10, } , 
      {   -3.3711562202088814e+01,    -8.3459644214919826e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.4492551959636893e+01,     1.5858222998864821e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.7069242800613491e+00,    -1.0616594620551020e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2194149165502849e+02,     7.2120273484417576e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7425028068995267e+01,     8.3267425408434665e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -9.9575775932570508e+01,     7.2160983824759078e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3101460177867366e+01,     8.8341401345395061e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.0943527717947905e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9472235045662343e+00,    -8.4684110466504735e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2430640948490664e+02,     1.5540718773166884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.7947037815038293e+01,     1.5527021806036370e+04,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    7.1446129030398158e+00,     1.6673382357774739e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -9.7229692393360153e+01,     1.5544789807201034e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5778343223297441e+01,     8.3973835302977877e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    9.9751486517942123e+00,     6.5082030194011659e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9670028602962959e+00,     1.2036460728798950e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9895793078454318e+01,     7.5328186714087769e+03,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -4.0203385988085778e+01,     8.3276277633001773e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.1197726426774807e+01,     8.3297550861500749e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2063398192199344e+01,     1.5126323093983181e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5142931665323900e+01,     1.5702013897775607e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.7514013113125941e+01,    -1.0990865309778637e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.7156785647259412e+01,     3.6759761869286535e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.2749201993055742e+01,     7.3733224730504817e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.6707581959245989e+01,     1.4283837920253138e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.3060898050859659e+00,    -7.9694518825351033e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    4.3392683069355362e+00,     6.8800016219871050e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.4315543465320921e+00,     8.3306403086067858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1073312013354425e+01,    -8.3345914788115424e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.5417782213233407e+01,     2.3793893453896959e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.5872135073268076e+01,     1.5529636378984504e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.3908199703313716e+01,     1.4679160089785861e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8608729184449015e+01,     2.2599082801302360e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.8629293405920137e+01,     8.4019859299772543e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -1.9029712761231469e+01,     9.1716155475486612e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.3753026813664526e+01,     7.9375422825886290e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.1369203525493852e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9240844863961839e+00,     1.0447387812463210e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0521054570556355e+01,    -9.4410187841516854e+03,    -6.7543912378877395e-04,    -9.3769753318119791e-07,     4.4366269585695850e-09, } , 
      {   -1.3540705210375817e+01,    -1.1942583630621180e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7818123919536546e+01,     1.5525481293417102e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.2873004093590673e+01,     8.3290795665186615e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.1878718995667128e+01,     8.3283032829315907e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.2663649720925720e+00,     7.7430436556352861e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.6367486387307686e+00,     1.3135462144424167e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3075130520184013e+01,     1.4307469028070724e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.1932607329449546e+01,     1.6031376464244444e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.6743444263721258e+01,     1.4978436429071971e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.0981606772818026e+02,    -8.3304223932035529e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    6.5406934560439858e+00,    -8.0470041435472267e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    2.6961932990828107e+01,     1.0908985947036155e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.3378329620451739e+00,     6.0633858897901282e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0410818568328500e+01,     1.3893118150399885e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7730539298341718e+01,     7.2091318333029121e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.9074259150942332e+01,     8.8600962092079899e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.5284079882019412e+00,     7.6136427907515472e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.5782567050487728e+01,     1.6171414191187578e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.8135488478343440e+00,     2.0426571094770613e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.4137482276290129e+01,     8.3667753581446123e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.2289405194903722e+00,     2.4356780791700485e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9816334792292182e+01,     8.2972111970618971e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -6.0231744378991763e+01,     8.8202813073100770e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3458272911734447e+01,     1.4445398727684522e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.8099369007564022e+01,     8.4464051619404363e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4670876253901902e+01,     2.2773737836451575e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    8.0953588662931413e+00,     1.7789845607142468e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6337398012408322e-01,     7.6202017463512748e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -8.2977156737006126e+01,     8.4065902600425234e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9632656307771086e+00,     9.1922856251232261e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0875554300234032e+01,     8.6879309669151498e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.7446056800810208e+01,     1.6587493071430279e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.3160394197180612e+00,     8.8503494698814102e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.9666332199655834e+00,    -1.7422247988449603e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.5209303979172772e+00,     7.5426494853391505e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.5386087779691717e+01,     3.1085156264409841e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    4.0969112997484993e+00,     7.9122602285243483e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.3257313030531520e+01,     3.1085860896325990e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {    1.4622320452856073e+01,     1.4445757757193322e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4494399287991033e+01,     7.8953395477155673e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.2538006170526387e+01,     9.0521708418549224e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4393534425161375e+01,     8.6103787059030255e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -3.7748507504582456e+01,    -9.1245641780414735e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0355476915417377e+01,     1.4427773414959591e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.8009827529444507e+01,     7.3109656660215132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.8805092981059317e+01,     1.4428478046875740e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    6.2092515660066656e+00,     6.5644253724354376e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7596227735333521e+01,     1.9630698341454263e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4062267474366354e+00,     1.6496361397840737e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6454174285387229e+00,     2.8286990488630363e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4177087963964473e+01,     1.0058509657837509e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.4077075549931983e+00,     8.2419503757970906e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    3.3320805094194547e+01,     7.2135105759263497e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0955187938658314e+01,     7.2146151549913156e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.3695756390613539e+01,     9.0123559399570113e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.7131520378046812e+00,     6.7976316692580549e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0461559344770828e+01,     1.0107675985439373e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.8054613604064649e+00,     8.3162399052000128e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.2359448694924566e+00,    -8.3411429442502395e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {   -1.6495155839822626e+01,     1.6097767369977373e+04,     2.7153144067100602e-04,     5.3279569112894673e-07,    -2.5270428314153453e-09, } , 
      {    1.6207462106522968e+01,     2.3948997975921211e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -2.6957972501241411e+01,     8.9397260130038121e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0224956880524856e+02,     1.4938719578747898e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2436538130980757e+01,     1.6026774064564976e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.7512346533519434e+01,     1.5490496538439644e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0964399449520009e+01,     3.2199784823676138e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.5147744360942699e+01,     7.1618051137145176e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.7869270977426766e+00,     5.8772656020394388e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.5176699504858817e+01,     1.5153336116525890e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.5672116069029649e+01,     2.2130810770436987e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.2161829438139439e+01,     8.9400783289618885e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7463868547065999e+01,     3.2200489455592284e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    1.7124883577714545e+01,     2.2206061831609371e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    1.4847675861603072e+01,     9.2567199749358933e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4145753014650477e+01,     7.8450002828205606e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.0096605750043800e+01,     7.2189938976147532e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -4.2331184149776597e+01,     9.1941574526797176e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.2070397982594400e+01,    -8.3288971399523343e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    3.7886389907050500e+01,     1.6102369769656838e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    9.1320064814932653e+00,     8.2691728477003289e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.9124032347306269e+01,     2.3243849475330418e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.9381241161403317e+01,     1.7283389234656061e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    8.4306002715493378e+00,     1.3107076211013909e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1063615929243610e+01,     3.2217200185560241e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7156325878758803e+01,     9.1016154529116466e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9048909621605681e+01,     1.7215800656475825e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9958445375413497e+01,     7.6207540358837578e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.2205060783107946e+00,     7.2663206172031478e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0040312970631169e+02,     1.5541023321705530e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.1715045721417967e+01,     7.6846360961639439e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2119854094918850e+01,     1.4405583825786609e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.4424902888578846e+00,     8.2757318033000556e+03,    -6.4021294957029994e-05,    -4.9499476841283623e-09,     2.0216730502267630e-11, } , 
      {   -1.7577030881198009e+01,    -9.4456211838311538e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.3138693035374462e+00,    -8.2047384978876580e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.8898744253486388e+00,     8.7451226209259039e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1285286097387042e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0637200199548040e+00,     1.6657159192394218e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.1641683576232385e+01,     1.6657606506506287e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.8122086490142053e+01,     7.9379716548718716e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2724621477153580e+01,     8.3151849134537715e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.3719069591363414e+01,     8.3421979359964807e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.7282076372911819e+00,     8.9143391938149653e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4627780274117306e+01,     8.2058325494939636e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2966407904421608e+01,     2.3322767933659365e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    9.5607729360261509e+00,     1.5595012041928274e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.1374548314732309e+01,     9.0919857437528353e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1449338837564618e+00,     8.9543691774544859e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9041217530680505e+00,     8.4273068741273401e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.1189208578662436e+02,     1.5542246331645923e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {   -1.8481503319322482e+02,     1.5543262248721994e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    5.1381962846934126e-01,    -8.6359420039809891e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2165002257176663e+01,     8.4526443515625924e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.2759070575513238e+00,     8.3356950062039668e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    8.8020602923986090e+01,     1.4914439991587689e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.9700820924774014e+00,     1.4444335066259569e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.1762351911844995e+00,     8.7838455443910461e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8686685089864728e+01,     2.2234447765019631e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.9364985151141560e+00,     8.3478790944333523e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.3669820242001460e+00,    -8.3095037550169000e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2559090744394661e+01,     8.8974804327458714e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {   -1.9814367706936235e+01,     8.2802400244437849e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -5.5729385574289481e-01,     1.5604887440156894e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.0719957908068004e+01,     8.3288409105091141e+03,     2.4726127994371776e-04,     2.9228447206731703e-07,    -1.3811372147448460e-09, } , 
      {    3.2867256109974413e+01,    -8.3285419389411345e+03,    -5.7364221338978736e-05,    -2.0853775679245581e-07,     9.9154094060522295e-10, } , 
      {    6.6352538234453924e+00,     8.3460042874594546e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -8.5103995084701012e-01,     1.5530135051861580e+04,    -1.2804258991405999e-04,    -9.8998953682567247e-09,     4.0433461004535261e-11, } , 
      {   -1.1409456626600810e+02,     8.3283458223177768e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2508886071769150e+02,     8.3290370271324755e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.3907693775257727e+01,     5.9570953794096722e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.3676006109711394e+00,     6.8127668225673460e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1880814322962701e+01,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4662797619638113e+01,     1.6600294950785483e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2555558015037697e+01,     8.0310922092562214e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7468089121594829e+01,     8.9407896744287646e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4386323696963313e+01,     6.6497450043468443e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.9533385418848486e+01,     8.3297873056931003e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2483714889800002e+01,     3.6147409882490110e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.2980950167537078e+00,     1.6639750823174592e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.5878927812291110e+01,    -8.3408973495447135e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    9.9154182632145389e+00,     7.7894013532931849e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.5124483725313567e+01,     2.3243490445821615e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    2.2075089417191407e+01,     2.2754187323326609e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -5.4187648631108321e+00,    -1.6641383341125766e+04,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    1.1394086202366791e+01,    -1.1921808202784177e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.4309341956179438e+01,     8.3164854999055406e+03,     2.1416721933202728e-04,     3.2425793433649092e-07,    -1.5355018908101224e-09, } , 
      {    1.2197772735214852e+02,     7.2157938339372595e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.7634766574075897e+01,     1.5480621140211026e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.7109943851014947e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1084554294926036e+01,     2.2759446987958974e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    3.1199513635467309e+01,    -1.0349987554704690e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.1107650468532665e+01,    -8.3275955437571520e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.0221934113961932e+00,     1.2714055241320923e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.7093227267932534e+00,     2.2126208370757515e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -3.7392797398769090e+01,     1.6028722948446635e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    5.5719367164819449e+00,     7.1360659785675944e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5766294012659802e+01,     1.6011802267637853e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9017475854064049e+01,     2.0023719311147252e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.9057652863121586e+00,     9.4786570549269545e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -6.9442445967428014e+01,     1.4914453418689140e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5941262593248794e+01,     1.4914348653564577e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.2822836795374599e+01,     6.8246446918007632e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.5768531938940910e+01,     6.5860839128897387e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6855724768661551e+00,     8.2750109820470589e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8967185557196736e+01,     8.4287295650610922e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5125911847849386e+01,     2.2667799642029193e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.5228814745128055e+01,     9.0214333595833377e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.2861274379844474e+00,     8.3823718674031934e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.5709477646661494e+00,     8.1114384933598958e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.0687037480692631e+01,     9.4800797458607067e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.8779264493960873e+01,     2.4393815105532245e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.0930188568096915e+00,     1.4233170022881681e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.0981962758485260e+00,     6.0214374193013009e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3487021646455652e+01,     2.2457096190477565e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.4407860822096293e+01,     8.2330902203576297e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.7938825307979354e+00,     8.8199281717337471e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    2.8383237906058504e+01,     1.1302006916729139e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5961448266081888e+01,     1.6641173514108345e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.7006846321582536e+00,     7.7643735636131378e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.7143833212462795e+01,     7.9392732510670567e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0234281496701136e+01,     2.4969153593366595e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    7.2825976462645201e+00,     5.4066923713343276e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.7928457981278839e+01,     1.5555373528506338e+04,    -4.3336525327019183e-06,     1.3779374444495230e-07,    -6.5789216526564028e-10, } , 
      {    8.6613745870841097e+00,     7.6408652626547846e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    3.5651296588043458e+01,     1.5542202000651476e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.5744511235483252e+00,     1.5543306579716442e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2213828337092190e+01,     1.4935776613116268e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.2363114877761660e+01,     1.5541690628759010e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5920348189884903e+00,     1.3188431241635359e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1363969508005106e+01,     8.4292412282353416e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9441778845617144e+01,     1.5543817951608908e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.7107945059347060e+01,     8.4733078678661041e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2400366987862840e+01,     8.3474269783932068e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -3.8638474690946456e+00,     9.5454916929183000e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.6393595740786679e-01,     7.6977540073633991e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3904586162115629e+01,     4.0077619579419988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0784520802934061e+00,     5.9581590408346228e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9839542425061040e+01,     6.6100281540227716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3241014953195263e+01,     2.1228392028053695e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0364905517770444e+01,     7.2119853226751329e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.2729773400891290e+01,     7.2161404082425333e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.2726803510156770e+01,     1.1371704643973499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0275887916153354e+02,     7.2123318969804059e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {   -3.4783706128709788e+01,     7.2129992040338821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1619844645368822e+01,     6.3504686650607346e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1671561952670821e+01,     8.9393669834950124e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.7151164471121248e+01,     7.2151265268837842e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3081554006724240e+01,     8.0001048048448065e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6771188480999626e+01,     9.4144543292763265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.7256702889784155e+00,    -1.7632026275657509e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.2748082700752477e+02,     1.5544485258662386e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.9001565497548626e+01,    -1.6633063405160119e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.3225557031584543e+01,     7.2297953496064220e+03,    -4.6792897807479791e-04,    -4.0985178973637954e-07,     1.9298008576565078e-09, } , 
      {    3.5000361284240661e+01,     1.7287991634335529e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.3161385229363670e+01,     7.9978036050050723e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.0271264439309721e+01,     2.4422201038942501e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    1.8851279035486082e+00,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4854593847049699e+01,     7.1118911565319522e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.0886286367064706e+00,     5.3821092075333945e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.6063428268309705e+00,     1.2321034271627936e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.0734377659613255e+01,     8.0957060092123047e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2226114892864352e+01,     8.3544305598720493e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5594473949008213e+01,     1.0451530627530496e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.4703520299594420e+01,     9.4624477195850486e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2940993453500468e+01,     1.6692719920385785e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.6785814549381044e+00,     7.7687473127757958e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {    7.9644020283553676e+00,     1.3812936057071576e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6267440528820817e+01,     9.6497409446288821e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -4.7291892522464401e+01,     8.1296169152355651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.6176031111487340e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9809637905146964e+01,     8.2817920860227059e+03,    -1.8757363751938967e-04,    -1.4954077993823637e-07,     7.0302831897693802e-10, } , 
      {    1.8611475199615874e+01,     6.6255701823112013e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.1256833592987974e+00,    -8.5551615713646279e+03,    -9.0458281950669213e-05,    -1.7656429452328192e-07,     8.3717626453994666e-10, } , 
      {    1.1001227103877220e+01,     7.7740069774081621e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -2.1454304074768956e+01,     7.7007124721560322e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.4386205544259301e+01,    -1.6209335341904819e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6144591496868699e+01,     8.9930474261730797e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.1428184555874576e+00,     7.6228315786674593e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1168836344981949e+01,     7.9018890275291742e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    8.3909078156203307e+00,     7.8243367665170499e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {   -5.6149336623683851e+01,     8.7181095983831947e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.0446224664395384e+01,     2.2703848058431598e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    4.2862803749351981e+00,     8.5389522360227238e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    3.7972766925590857e+00,     8.5375295450889716e+03,    -2.8035534055540824e-04,    -2.6031100979814315e-07,     1.2267725386795698e-09, } , 
      {    1.3419153277388508e+01,     2.2717002253744879e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    7.6415547128642576e+00,     7.1603824227807645e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.4148058080082393e+00,     1.3342362826366465e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.9829102009989931e+01,     8.3172014369694807e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.1398618500903929e+01,    -8.3401814124807697e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {   -7.1387000748032037e+01,     8.7804664054121640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4528485879138831e+01,     2.2914551509983223e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {    9.4009626586704400e+01,     7.7246567132890650e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.6698050065495668e+01,     7.5449506851788865e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0066588990084290e+01,     1.5489073847505892e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3292565949777732e+01,     7.7648181894922518e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3207750088556420e+01,     1.4550984606148828e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.3236759842541840e+01,    -5.6387323523287432e+02,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.8310213400665766e+01,     8.5277659342146871e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.8753153725246765e+00,     1.5168878144814318e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8724369286637703e+01,     1.6694142611319538e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.3844764145785344e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6083081876795816e+01,     6.2795526908746854e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5325640864174337e+01,     4.4007829276349858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6977750249885570e+00,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4650103407896379e+01,     7.2677433081369009e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.4728730567114638e+00,    -2.2123364377238058e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.0439558676453615e+01,     2.0416740280840240e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.7656501298452326e+01,    -1.1147266108401323e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    7.2954984535179257e-01,     7.1496285156461636e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.3939962357516180e+01,     7.7159258679680115e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0531329362491975e+02,    -1.1145305076924546e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.4662240130959225e+01,     2.5158601724983564e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1976340508212690e+01,    -9.6505682499824070e+03,    -2.4277103259201746e-04,    -4.2697540895316834e-07,     2.0235153422149812e-09, } , 
      {    6.9687708945028142e+00,     7.4313157968240630e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    9.5841037076800060e+00,    -6.9889778019972539e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -3.1736659003310713e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1888948711242966e+00,     1.1928013301934949e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.3027617295491162e+01,     1.4795001999444677e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8252859015808902e+01,     9.8074752989693116e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.3498774176755454e+01,     9.1543425507818047e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.2056060521234890e+01,     7.8821853524022881e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.1937541430316195e+01,     6.5854266479368407e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -7.0509474387724865e+01,     8.1521588203666215e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.6252269558055929e+01,     1.4270391376577234e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    1.0022726880545768e+01,     7.8397406181881915e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.6958915202560028e+01,     8.2196589360497746e+03,     3.0927234345339518e-05,     3.6923409953302257e-08,    -1.7458140656754390e-10, } , 
      {   -8.8297612771028383e+01,     6.5856572288394509e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9815049670349136e+00,     2.4985362928708502e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    1.0435106013095556e+01,     2.4986785619642254e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    6.1993312972853714e+00,    -1.0370762982541692e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.6289406858101518e+01,     1.6173691707545688e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.0344378673635235e+01,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0727394503860346e+01,    -7.5764309128421155e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    6.8312702156769172e+00,    -8.8330460829332023e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    9.5875083279171093e+01,     7.2147674973749827e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0608644263274471e+02,     1.6655651880971825e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -4.2520734189874645e+01,    -5.7087898664768545e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.4699601893995236e+01,     1.4485213629582435e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.3512089896193913e+01,     7.2133582335426827e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.1987686765571048e+00,     6.8401867200891929e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.6251664568931709e+01,     7.7000552072031342e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3515353246830344e+01,     7.2009449542593775e+03,    -5.8931449437080662e-04,    -6.2333949421296371e-07,     2.9415585287639984e-09, } , 
      {    6.2778034323659808e+00,     8.3080728622141505e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7176362332591797e+00,     8.3493099872361017e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.3717503069585042e+01,     7.6329431902770966e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0765578957220018e+02,    -1.7309684784268029e+00,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    9.0848103114716618e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8226195992343818e+01,     2.3858327803709632e+04,    -2.8468899308811018e-04,    -1.2251726535319084e-07,     5.6888037341392938e-10, } , 
      {   -1.4674206043237303e+00,     7.9946301812533984e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1793366622163111e+01,     9.2724677184515331e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2433936103651153e+01,     8.6627526681968538e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.3037998762829766e+00,     8.4269072020407584e+03,    -2.4942810621006873e-04,    -2.2338759984484089e-07,     1.0521911321120259e-09, } , 
      {    1.2897964942050216e+01,     8.2173577362100368e+03,     1.8323998498668778e-04,     2.8733452438318867e-07,    -1.3609204842425783e-09, } , 
      {    1.0634198914020745e+01,     7.5295315741396971e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.9678070413629056e+00,    -8.4400251132402154e+03,    -1.2138551629600873e-04,    -2.1348770447658417e-07,     1.0117576711074906e-09, } , 
      {    1.2010855651652767e+01,     1.4914452334935477e+04,    -6.3524239730472436e-05,     6.3330532424185123e-08,    -2.5428962387876272e-10, } , 
      {    9.6553010794008660e+00,     6.5857609102103506e+03,    -2.1583699037182070e-04,    -1.8708058200570130e-07,     9.3204945379627173e-10, } , 
      {    3.9694765807946748e+00,     7.7003894694766441e+03,     1.5497663213425675e-04,     2.4979472231572374e-07,    -1.1318993494232446e-09, } , 
      {    7.0248178708887119e+00,     8.9569933799736082e+03,     1.4964886914843974e-04,     2.5102750654404910e-07,    -1.2407788059268243e-09, } , 
      {    4.6692632986368112e+00,     6.2830195524848227e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.5066196941746805e+01,     1.6171056245432441e+04,    -6.8852002716289471e-05,     6.4563316652510456e-08,    -3.6316908038234235e-10, } , 
      {    1.2710642369494904e+01,     7.8423648207073147e+03,    -2.2116475335763771e-04,    -1.8584779777737599e-07,     8.2316999729269210e-10, } , 
      {   -2.0102667270567796e+01,     8.3263902248853919e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.1096952431353994e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2053981333400646e+01,     1.4286150379686995e+04,    -6.0860358237563932e-05,     6.2714140310022456e-08,    -1.9984989562697296e-10, } , 
      {    6.7261374666020144e+00,     8.3996791003404614e+03,    -3.5757942104598988e-05,     3.2589854383336562e-08,    -2.0880440431706606e-10, } , 
      {    1.4366410223904667e+01,     2.3243143759660601e+04,     8.8788510910875812e-05,     3.1374164685407152e-07,    -1.4406287015537972e-09, } , 
      {   -3.2559810597216710e+01,     8.3283391087670516e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.3554037726366133e+01,     8.3290437406832007e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.6984267611487454e+00,     5.9574589549618686e+03,    -2.1317310887891217e-04,    -1.8769697411986398e-07,     9.8648918204806139e-10, } , 
      {    1.0441339160738760e+01,    -1.7429305145147757e+03,    -3.6814974101316892e-04,    -4.3749169643558775e-07,     2.1183885314713062e-09, } , 
      {    6.3250311530465755e+00,     1.6029080894201770e+04,     3.0728938277560503e-04,     5.0020583674561016e-07,    -2.3182384270982792e-09, } , 
      {   -1.2061209776475696e+01,     1.5540453090344225e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.9138261232051704e+01,     1.5545055490023693e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5109322623494682e+01,     1.5542754290183959e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.3803724431406117e+00,     1.7285684804698736e+04,     3.0196161978978796e-04,     5.0143862097393552e-07,    -2.4271178836018586e-09, } , 
      {    2.0052234722510860e+01,     2.2128515200394308e+04,    -2.8202511159520165e-04,    -1.2313365746735350e-07,     6.2332010166571914e-10, } , 
      {    1.2667516687747025e+01,     8.4706667759557968e+03,    -2.2382863485054624e-04,    -1.8523140566321331e-07,     7.6873026904090223e-10, } , 
      {    4.0126022625425533e+00,     7.0720875142281620e+03,     1.5764051362716528e-04,     2.4917833020156111e-07,    -1.0774596211714549e-09, } , 
      {   -1.4416685078564054e+01,     7.2117616656190985e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.6782626784972507e+01,     7.2163640652985669e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    1.0490530175405421e+01,     8.3294027701920022e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.0350342068974285e-01,     8.3279800792582500e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0508975181965335e+01,     8.3117707439163460e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.7421751513998704e+01,     2.4499747670157569e+04,     8.3460747925058777e-05,     3.1497443108239688e-07,    -1.5495081580573769e-09, } , 
      {    2.7769301084726568e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6775118983842695e+01,     8.3290200572015747e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.7845419673229948e+01,     8.3155735135256709e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.2745927906877412e-01,     8.3260615924089434e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0566343350107111e+01,     8.3313212570413107e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.3528013759876956e+01,     8.3131493964366964e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.2534055780514166e+01,     8.3442334530135558e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.7747112673693859e+01,     1.6655081649610518e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.3452507031590301e+01,     1.6659684049289986e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.9737565826255619e+00,     7.2114330331426490e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.3557361761175564e+00,     1.5540124457867776e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7478476611595198e+01,     1.5543106606142033e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.6389136155575848e+01,     7.2166926977750172e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0406443185015807e+01,     1.5542401974225884e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.8717856688440143e+01,     1.5545384122500143e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    9.1535859264325765e+01,     1.5527212261895529e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.9173011335898096e+01,     7.1985208371704030e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.6808878718746456e+01,     7.2296048937472624e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -6.4461842091051182e+01,     1.5558296318472387e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2294293180759379e+01,     8.3640284956606611e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.4662165695525649e+01,     8.3654511865944132e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.3296219484201700e+01,     7.2144151814169072e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0936488792497055e+01,     7.2137105495007590e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.0597926996159003e+01,     8.3114184279582696e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    4.5910020423674268e+01,     1.6657735165408325e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -3.0204799823555806e+01,     1.6657030533492180e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -8.7338563818469197e+00,     1.5543082922660407e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    3.5810908975246491e+01,     1.5542425657707512e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.7905782812869102e+00,     7.2133515199919557e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2574215935856811e+01,     7.2147742109257097e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.0264754532067575e+00,     7.2309835462676137e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.1093161673359369e+00,     8.1709570703846939e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0693414704254978e+02,     8.3269604562466993e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    8.1203069540111201e+00,     8.3221324691253976e+03,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -9.7056814045675033e+00,     2.3869144515069351e+04,     2.3843738005931554e-04,     5.6476915339812062e-07,    -2.6814075074806213e-09, } , 
      {    4.1493905123120783e+01,     2.3873746914748819e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    8.5553868533872901e+00,     8.3110593984494699e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.8828957241878941e+00,     8.4864257790655574e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.9181387129980983e+01,     1.4916747904978150e+04,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3472396642603446e+01,     8.3647398411275371e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.5143545574780713e+01,     1.5385019935843527e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3455472613154853e+01,     7.2137342329823841e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.1089646586662145e+01,     7.2143914979352812e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2788563363454617e+01,     7.0563285111184005e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2725510444281499e+01,     1.5465202029171833e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2018083880403040e+01,     1.4912145505298682e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -1.3564574915361934e+01,     8.3198312692856634e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {   -2.1417276713455525e+01,    -8.3375515801645888e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    1.0372067525243656e+01,     7.1365106044467084e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8040309711134363e+01,     8.3124820893832220e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3822557551449695e+01,     1.4991998966150539e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7223001967002656e+00,     8.2511391637130018e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.0117666284630548e+01,     1.4914094389180342e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.2792811189082229e+00,     8.4062436857372504e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.8511436473527752e+01,     8.2490616209293021e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    2.9508184990532808e+01,     8.4083212285209502e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.5060647954877227e+01,     1.4915158050605292e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5456071744635457e+01,     1.5020384899560799e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1371611517644480e+01,     7.1610937682476415e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2035842282551572e+01,     7.2916151264709570e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3726003736518239e+01,     1.5489785192972768e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.4392913656889592e+01,     1.5620306551196083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -8.3700526451428786e+01,     8.3044185511156447e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.4695717630357734e+01,     8.3529642983346093e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.1459716264462621e+01,     6.6633075414254135e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.9752562419222635e+00,     7.2271807766582879e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -7.4059037117505335e+00,     8.2888765228272132e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2726047646946943e+01,     6.5864666258801662e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.8399937047545468e+01,     8.3685063266230391e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.5505113621410231e+00,     7.3717972197992640e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.0450252871010852e+01,     1.5463124486388133e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.5571070022099113e+01,     8.1578391591852396e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0968834329379821e+01,     7.2670319626700239e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.2802427726031747e+01,     7.1344330616630086e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3100225128071136e+01,     6.6916934748356716e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.3328960225648876e+01,     1.5595723387395150e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1914123291328032e+01,     1.5700488644524392e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.7390274901161176e+00,     7.7779361006917070e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.5599954665844376e+01,    -2.3011998397341813e+00,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    5.6908852692498391e+00,     8.2757223275139349e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.4225959289598764e+00,     7.8063220341019651e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6262121680041844e+01,     8.3512333298561825e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.2987037053694062e+00,     8.3816605219363173e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4560886875046942e+01,    -8.3441105357643155e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    3.5179669878945965e+01,     7.2936926692546567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -7.5619384746158175e+01,     1.5518481416574477e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -7.7964829570098175e+01,     7.1897899918493513e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.7541078302625749e+01,     1.5622384093979783e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    6.5872045904819210e-01,     1.5502939388286046e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.7130588319555116e+01,    -1.1142762433082185e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.9476608691654299e+01,    -1.1149808752243684e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    5.2835390349686859e+00,     8.8794467487585462e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.6913660738065062e+00,     7.1742479635609207e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.2175164093097919e+01,     6.5854029644552147e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0033558265379831e+02,     7.2383357390683150e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.0269999779421052e+02,     1.5567027163793440e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.8330382440084961e+01,     7.7000315237215082e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3193011619767180e+01,     1.5542042944717083e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.5680382796777090e+00,     8.8510608153482863e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3853608135238447e+01,     8.5508601288782265e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    2.4054197976413988e+01,     7.2538777673567447e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    3.9242125906952321e+00,     8.9562876643037926e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.0287627287173704e+00,     7.7010951851464597e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -3.5491710757514397e+01,     8.2716035260603567e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    7.2359592265109853e+01,     8.2733344945387835e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    9.2049799333565900e+01,     1.4898904676849985e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.6416965968137596e+01,     1.5582569192081872e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -6.1366403028952476e+01,     8.3840483549114670e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.3816239248271158e+01,     1.5543465635650835e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    2.9758573538391593e+00,     1.6656671503983376e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    1.2729024038488497e+01,     1.6658094194917128e+04,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.6825818244525642e+01,     6.5880564802530243e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -2.1985039672625803e+01,     2.3871093398951012e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {    5.3771680882208472e+01,     2.3871798030867158e+04,     8.6124629417967295e-05,     3.1435803896823420e-07,    -1.4950684298055870e-09, } , 
      {   -3.1554252012155541e+01,     8.3297873056931003e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.1035838248015953e+01,     1.4445046411726446e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -3.3123769124419894e+01,    -8.3275955437571520e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    6.1189925219292370e+01,     8.2335195926408724e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.4373575559690266e+01,     6.5834540805735560e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -5.0197136239459162e+01,     8.4238632568093799e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    3.1145226431118196e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0017271680828500e+01,     8.1937046907429594e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.0370779784756849e+01,     1.5487397359997618e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.8011360723994912e+01,     7.1587059352724909e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -3.9024620220079782e+01,     8.4636781587072928e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2859143156460673e+01,     1.6640462168641468e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.4419508025382488e+01,     1.6657711481926701e+04,     4.5693825192404472e-04,     7.5123334328965926e-07,    -3.5590172330251036e-09, } , 
      {    3.0124911621569677e+01,     1.6657054216973800e+04,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    6.9205578613702954e+01,     1.5447582458099705e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.2991885307238386e+01,     8.3132723136859368e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    7.5289271646394234e+00,     7.1787257945232986e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    8.9731227437486211e+01,     6.5702132521248595e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    6.6847957227076776e+01,     7.1188910333745789e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    5.8042136907825231e+01,     1.5407767556201792e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0059541578806744e+01,     7.6980826398398494e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -4.6662267531269642e+01,     8.2317886241624456e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -5.5651333396059911e+01,     7.2694197956451735e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.9632010200221632e+00,     8.2588016467051548e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.1290452990630250e+01,    -1.6920680808781185e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -1.7004774915814131e+01,     8.9546978099309345e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {   -1.1217693672709954e+00,     7.1773031035895456e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -5.3301575878028970e+01,     1.5598111220370300e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    5.5682501291287146e+01,     7.0790761314766660e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -2.0401268684185386e+01,     7.7690759452522452e+03,     1.1921869002965777e-04,     2.8238457669906031e-07,    -1.3407037537403106e-09, } , 
      {    3.7081387634474964e+01,     7.7736783449317118e+03,    -1.8540681125303872e-04,    -2.1843765216071253e-07,     1.0319744016097583e-09, } , 
      {    4.4137075173898381e+01,     7.2313358622256892e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.0198089036543543e+00,     2.2754515955803057e+04,    -1.3237624244676191e-04,     1.2789384907669558e-07,    -6.1745870426110507e-10, } , 
      {   -2.3414583301104091e+01,     8.2092467190313891e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    4.7179613890746197e+01,     2.2759118355482529e+04,    -4.3700174372945840e-04,    -3.7292837978307729e-07,     1.7552194510889637e-09, } , 
      {    2.4583731131079318e+02,     8.3303133301417911e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4412582724152301e+01,     8.4481361304188631e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -1.0165885687303948e+01,     8.3908245746980610e+03,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8491847111226623e+00,    -8.3463234510007824e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    2.4425294302317613e+02,    -8.3270695193084612e+03,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {   -5.5237137647785453e+00,     1.5533894134744496e+04,    -9.9282181835071430e-05,     9.5920386807521691e-08,    -4.6309402819582878e-10, } , 
      {    3.2600766358178120e+01,     1.5551614445623421e+04,    -3.3094060611690477e-05,     3.1973462269173895e-08,    -1.5436467606527627e-10, } , 
      {   -6.4352956589167448e+01,     1.4929988733426844e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    4.1979654828896379e+01,     1.5328137752405968e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.3942233525363918e+01,     8.1140748869471354e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.4255748644031904e+01,     8.3237603925692056e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    2.0206568810342496e+01,     1.6644264938250795e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.8842158187964628e+01,     8.1538897888450474e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -4.4490202094043660e+01,     7.3092346975430864e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    1.2405218011975144e+01,     1.4392788659982130e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.0470441704044603e+01,     8.3444239088727172e+03,    -9.7115355568720471e-05,     2.7023514585045533e-08,    -1.3414794556300864e-10, } , 
      {    4.6471937760193974e+01,     1.5033891410832151e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.2948732252078138e+01,     8.5433079625031169e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {   -2.7849362430323996e+01,     8.5034930606052058e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.0204504014414075e+01,     8.3301141156588783e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    7.8969474937436956e-01,     8.3272687337913721e+03,     1.5231275064134825e-04,     2.5041111442988642e-07,    -1.1863390776750345e-09, } , 
      {    1.2351941355019600e+01,    -1.3117911199455193e+01,    -3.7081362250607745e-04,    -4.3687530432142507e-07,     2.0639488032195165e-09, } , 
      {    4.6872302456046846e+01,     1.5367952654303879e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    3.4195419378112796e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4514869526594659e+01,     7.0392612295787540e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {   -1.1478877770841667e+02,    -8.3304223932035529e+03,    -5.2312637314742567e-04,    -6.8728641875131149e-07,     3.2502878808945512e-09, } , 
      {    8.2816019777608734e+00,     1.4834264611810107e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.5327813898394748e+01,     1.5423309584490224e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -1.7678545229150011e+01,     7.0946181597650957e+03,    -2.1850087186472920e-04,    -1.8646418989153864e-07,     8.7760972554448186e-10, } , 
      {    9.2068852653874540e+00,     1.4911816872822232e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {    1.2917081444514125e+01,     1.5227285581503094e+04,    -6.6188121223380953e-05,     6.3946924538347790e-08,    -3.0872935213055254e-10, } , 
      {   -2.1736365912639020e+01,     8.3263902248853919e+03,     3.0462550128269650e-04,     5.0082222885977284e-07,    -2.3726781553500689e-09, } , 
      {    3.2730503217768330e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
    } ,  
    {  
      {   -2.4081036653892557e+01,    -2.3011999414102320e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.5913740007105552e-01,     3.3757047124063320e+01,    -3.0816644950982666e-05,    -3.6447710769397583e-08,     1.7385418604587960e-10, } , 
      {    1.3492887293616077e+01,     8.3647398410270489e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5829929133401038e+01,    -3.5231595807488247e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1725480526797480e+01,     8.3263902246820398e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9578197631664931e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5644384831743210e+00,     3.9302096969298748e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0601910842365111e+01,     3.2863237477272378e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1754041159951725e+00,     5.7533848783435178e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5450731170305001e+01,     8.3931257744373070e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9252790427392767e+00,     8.3117707438158577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3964229099022782e+00,    -8.2949343774993868e+03,    -1.8310905271330885e-04,    -2.8716658492405232e-07,     1.4098381846665302e-09, } , 
      {    3.3146861908713268e+00,     8.3624484717475134e+03,     1.2147576281134351e-04,     2.1427116338525712e-07,    -1.0621298125747714e-09, } , 
      {   -5.1009433764673355e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4382617260756811e+00,     1.6833175267859137e+04,    -9.7195685801984544e-05,     2.5757278212723649e-08,    -2.3271056693257730e-10, } , 
      {    5.8034095343364145e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5691415847274435e+00,    -1.6920680807593385e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.7292215785537479e+00,     2.6298323161836379e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9779913393490274e+00,    -3.3781427455160383e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8488748628508307e+00,     7.8604193938597496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9951613655768723e+00,     3.6048416403597997e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.8082792557309908e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6039646928366249e+01,     7.2117616654169342e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.3493741210671644e+01,     8.3283391086653737e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7629637223136015e+01,     8.3290437405815264e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3684102094179149e+01,     1.5540453090040384e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7619564122222101e+01,     1.5545055489923205e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9986100343290811e+00,     6.6387243775239922e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5264010219164760e+01,     7.2163640652997528e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.6209837135489504e+00,     5.8849268314763094e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7560362728742831e+00,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4570266766226636e+00,     1.1506769756687036e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1895568268244903e+00,     7.1780144489547456e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2442332187359026e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7767064242147448e+01,     7.2137105494002708e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3274628787344511e+01,     7.2144151813164190e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2353086393503292e+01,     1.4967415802147443e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3322110777529588e+00,     6.4280209259723688e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2158625064820239e+00,     7.9356704549304632e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3215747392008614e+00,     7.7533529367890978e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0886730310850929e+01,     8.0001048047443182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0686820627624154e+01,     1.4756712350595821e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6190106611064667e+00,     6.8210418956653575e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7542222226986745e+01,    -4.9310322575947794e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9200813688251319e+00,     8.7217123943164388e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6068378849137840e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2269792253874918e+00,     7.1496285155444875e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5288241136802792e+00,     8.9040299124578014e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5420110203595382e+01,     1.5542401974023718e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5638816816334057e+01,     1.5543106605939871e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7530797536201241e+00,     7.2309835461659377e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3977700087541329e+01,    -1.8048943455862172e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6021274436567223e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0076212829296296e+01,    -6.5589555991336965e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.7747536715571872e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9745513107018748e+00,     1.5149733320288806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2579388153700243e+01,     1.5508997242857731e+04,    -3.5366531172850963e-05,     9.8565433288919105e-08,    -5.8187338006766507e-10, } , 
      {    1.5988744165292987e+01,     8.3155735134251827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8270021466654693e+00,     8.2888765227255371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2001717184399171e+01,     6.0633858896896400e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6965955486906694e+01,     8.3685063265213612e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4355624052568919e+01,     1.4392077314313090e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9452643209668068e+00,     8.3294027700903262e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1906079311167694e+00,     8.3279800791565740e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6873838703541338e+01,     8.7861467441302957e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0136409279195070e+02,    -5.0795863911699257e-01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8835833498974420e+01,     6.0334744427518672e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0267525810128072e+01,     1.6833150887528041e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.9545712365053838e+00,     6.4434349813856670e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.9369926016501697e+01,     1.6655081649305488e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.3631870556998519e+01,    -1.3117911198267393e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5848583194216038e+01,     1.6693431265650499e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5064281286529457e+00,     8.3467156328252495e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1933751627675083e+01,     1.6659684049188309e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.3402769979122713e+00,     8.2949099971682899e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.6290734462568341e+00,    -8.3624728520786120e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1356069951186711e+01,     1.5576511337105858e+04,    -9.6999821074816303e-05,     2.5670011750123932e-08,    -2.3416500797590593e-10, } , 
      {    1.2430699160631178e+01,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0223825225733034e+01,     7.1803058182342811e+03,    -1.8765893893517714e-04,    -1.5215344086573563e-07,     6.5411061855298576e-10, } , 
      {   -1.3792189295369884e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1570892303950759e+01,     6.6916934747351834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1634544885999816e+01,     7.0563285110179122e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9985083601508622e+00,     8.1709570702830179e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0102604361801362e+01,     6.6255701822107130e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0316988484325842e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7763961732769424e+01,     8.3512333297556943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0586341951319726e+01,     7.1610937681471532e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0427210305338583e+00,     7.2140628653583444e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1349612751717757e+00,     8.4864257789638814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6448477216135990e+00,     7.5426494852374744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4857297939044022e+01,     7.2670319625695356e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7253353352831780e+01,     1.2619238417987162e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -9.3387769934275493e+00,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0894505807176174e+00,     8.3260615923072655e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.0462911203129011e+00,     8.3313212569396328e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4794882409188639e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0327004408428921e+00,     8.2757223274122589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0811675495711146e+00,     8.3816605218346413e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0698753075555798e+01,     7.2114330330421608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4808538699075932e+01,     7.2166926976745290e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9726696909511476e+00,     7.8063220340002890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7944443897437061e+01,     1.5732483999299802e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.2051518315401708e+00,     7.7894013531926967e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2035257129569992e+00,     9.1147333640094239e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0827033359645242e+00,     8.5044838432356883e+03,    -2.4948809356431073e-04,    -2.2496159594193106e-07,     1.0032734316880736e-09, } , 
      {    3.0074678285903449e+00,     7.1742479634604324e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0005277926375982e+00,     7.2478199124824077e+03,    -2.4929222883714246e-04,    -2.2504886240453078e-07,     1.0018189906447449e-09, } , 
      {   -1.0607166101092938e+01,    -1.5419111139677625e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3926034225189063e+01,     1.5020384899358634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1002636531626955e+00,    -1.2904209778773438e+03,     3.1012509678150921e-05,     3.6360444306797873e-08,    -1.7530862708920817e-10, } , 
      {    2.2463350223696189e+01,     1.4954261606834163e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0228398581037737e+01,     6.6633075413249253e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.8335538944065526e+00,     1.5720838787719499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5788718147121575e+01,     8.3114184278577814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2518603844154327e+01,     7.2538777672562564e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4654614300108996e+01,     2.0775427837006646e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3938813005594168e+00,     1.1790629090789623e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3992617194295548e+01,     1.5385019935641363e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4405514679671878e+01,     7.7401987414758187e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2382144589432251e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8246323195503376e+01,     8.3290200569982226e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5332134417324204e+00,     7.7779361005900309e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.1725292377806138e+00,     8.8510608152466102e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.8702530701566662e-01,     6.0697767706769139e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7938161019295578e+00,     2.5161866692482588e+04,     5.5096721960341630e-05,     2.7647615236737837e-07,    -1.4686945655532281e-09, } , 
      {    1.8297318473354906e+01,     9.1791677138232808e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7683633249044394e+01,     1.6864631115041790e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.3620032202544323e+01,     1.6869233514924610e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.2584902471699863e+01,     1.4991998965948376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2281328040165226e+01,     1.6640462168439306e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9182877155785771e+00,     6.7514094248126639e+01,    -6.1633289901965333e-05,    -7.2895421538795167e-08,     3.4770837209175919e-10, } , 
      {    1.3024315681597866e+01,     1.5489785192770603e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9525253547330760e+01,     1.6328796229367768e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3321722950194026e+00,     1.5502939388083882e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3863483042787720e+01,     7.3717972196987757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.6313416247758894e+01,     1.5542028288429719e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7039457442102634e+01,     1.5595723387192986e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.7519772857561557e+00,    -1.6623625802122835e+04,    -3.3540146047563498e-04,    -5.3788545907870699e-07,     2.6458221832871811e-09, } , 
      {   -1.4123687406373413e+01,     6.2133149872446666e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.8117596037413248e+00,     9.4793684002921545e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5995913469331491e+00,    -2.2365715770956740e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3067082212481434e+01,     1.5540124457665610e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0255754974709587e+01,     1.9651048484649373e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7150893214647354e+01,     1.5545384122297977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6702405667252043e+00,     1.6691139896370962e+04,     2.7376817057366969e-04,     4.6499003753991189e-07,    -2.2981138111954222e-09, } , 
      {   -6.5430604666339818e+00,     8.9171841077710815e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4921515557279356e+01,     1.5582569191879707e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7519063333513877e+01,     7.5251061070712240e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9475278846673254e+00,     1.5999508223997282e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1744416398200261e+00,     3.1546870868086353e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2709837962199249e+00,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1558462017109134e+01,     1.6118092777816146e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2422730071405743e+00,     1.9187669756030985e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3573264699200246e+01,     8.1578391590847514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7805800836424961e+01,     1.6721817199060755e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6011783176548207e+00,     8.8794467486568683e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0787707178535175e+00,     8.0182093328307928e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8144181053242072e+01,    -1.7272996765668267e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4858973323886614e+01,     7.1365106043462201e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0507208378171821e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1328547452105131e+01,     2.3869144514663836e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.9975137612363241e+01,     2.3873746914546657e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0919996806470451e+01,     9.6640240106353303e-01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.3861796545481139e+01,     7.7364322559815055e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4535166643370530e+01,     8.9400783288614002e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3749297521687081e+01,     8.2490616208276260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0089277468461857e+00,     1.8024208201798999e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8147165108870425e+01,     8.5508601287777383e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0668808144932340e+01,     7.2916151263704687e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7888776413932025e+01,     8.4083212284192741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6992274667834160e-01,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6979738061767073e+01,     2.2541905132244210e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1677859341315385e+01,     2.3581258181579246e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4961611516810567e+01,     1.5935775259674781e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6201912692557720e+01,     1.5700488644322226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4229371275849232e+00,     2.1329907977851814e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4794006982624712e+01,     1.4445046411524281e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9719509751513108e+01,     9.5721886835162695e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0131318937381272e+01,     4.6400665873147773e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2436869557792303e+01,     6.1163549869008320e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1485287775979685e+01,     7.2140561518076174e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5977746061118992e+01,     7.2140695789090714e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5838740347064615e+00,     8.3910482316536072e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1282960929895534e+01,     6.6653850841086260e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4949794007114384e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0929474397883315e+01,    -8.6364983828341337e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.0287541667300371e+00,     8.2511391636113258e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2942258489758885e+01,     8.4280182194937279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4161738998666626e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6309007152181710e+01,     8.4177089382382401e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1169891866629982e+00,     8.4062436856355744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2300374468565662e+01,     1.4339108217101899e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.9703775443324290e+00,     7.1344330615625204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9300149694055158e-01,     7.2271807765566118e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7206823078059486e+01,     1.5465202028969670e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7224281455060432e+01,    -1.1944470569373675e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0002208901468247e+01,    -8.8601555405439285e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.4949373998301674e+00,     8.2588016466046665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9355964732385331e+00,     6.0104167924784488e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6112819556708340e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0735360620935317e+01,     1.4128404765550276e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7663299622378055e+01,     1.4916747904877662e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6241560790243748e+00,     1.2036460728798950e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0398810439999078e+01,     1.5542754289981794e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2209353670149703e+00,     1.4363691380902832e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8686237583861081e+00,     6.0349999562793819e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3100388686737785e+01,     2.7511467878509129e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.5738581666214877e+00,     8.3200549262406166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.2055202359399942e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0162242217421550e+01,     8.8273902472144641e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3646386509794795e+01,     1.4994076508732076e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1138140723508005e+01,     1.6657030533288827e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.9985154025703736e+01,     1.6657735165204973e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4707468637881755e+01,     2.3296107226770895e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.3565419367384743e+00,     1.6900689362107263e+04,    -1.5882897570394989e-04,    -4.7138143326071518e-08,     1.1499780515918190e-10, } , 
      {    7.9803270182993753e+00,     7.2133515198914674e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1243031070044779e+01,     7.2147742108252214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6792896824283059e+01,     6.9596314256824407e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3805713124350781e+01,    -1.1489987856728476e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.5450906459987115e+00,     1.5506705873578196e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.3806504377170228e+00,     5.7997133409268263e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3492731172473263e+01,     7.2936926691541685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3640976554555976e+01,     1.4912145504994842e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.8795903554659876e+00,    -1.3971962187754312e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.5580398977756150e+01,     7.7800136433737316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6480530863941439e+01,    -8.3226579501806991e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.1191639508928816e+01,     8.3347248990662010e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.9882682108018965e+00,     5.8564776908983958e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3685037171570684e+00,    -2.1422382218208291e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.5083045594409334e-01,     7.8025555485059758e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3053666416548884e+01,     1.5620306550993919e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4009524748800857e+01,     6.5854029643547265e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1142115791311760e+01,     9.9652096532092582e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1336562106886085e+01,     8.4292412282353416e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1446462516548893e+01,     4.4092271702584291e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2604505402893702e+01,     7.6070838350513322e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4934944798482150e+01,     2.3837688667481183e+04,     1.1692587658947522e-04,     3.4928430744357381e-07,    -1.8178573786883158e-09, } , 
      {    2.1534307496199336e+01,     1.5578802706385393e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.1659586913914485e+01,     1.4914094388978176e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0291643248685650e+01,     6.6070851882906527e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7630739876543817e+00,     8.1643981146838842e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.5797953855002561e+01,     6.2795526908746854e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6256820940482573e+01,     1.6065123680604955e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.6271879861878240e+00,     1.5463124486185970e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.5515984118720851e-01,     8.3598698281385277e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.2483808450342480e+00,     7.2206218209574781e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.6594233515510638e+01,    -2.0571522602131154e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.8433083413320777e+00,     2.8168728122570292e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1125890689211531e+01,     8.0742599849475464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7180412374929990e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9854351299436517e+00,     8.5639780399760057e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4453987971601606e+01,     7.1915209602261002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3041154162583741e+01,     2.3085403775219267e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.1672533073358927e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0166303679562390e+00,     1.4521425735243263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0424051061030648e+01,    -5.9000540852293000e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.0007427002014619e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3070166196074826e+01,     2.3871093398647172e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.7999990344505846e+01,     2.3871798030563317e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.2139126203061519e+00,    -8.3456121054310406e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.8763274723168490e+00,     8.0934048092708945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1427646921626767e+01,     8.8773692058731685e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.5502439726085449e+00,     3.5923954223782710e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.4346049862447980e+00,     2.9424634157381547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4522861438148949e+01,     3.1441677575438998e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.3622191335200185e+01,     1.5542761003532520e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3840836378493549e+01,     1.5542747576431069e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6620532636777181e+00,     6.1927343106198141e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1545394794457593e+00,     1.3763769729267548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7964990603393456e+01,     1.5525833609174202e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.3889737221608772e+01,     2.3907494131008843e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.9926601744404557e+00,     8.5831228642993538e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3298812187401680e+01,     6.5861075962708755e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0606379104749102e+01,    -1.0816711256855342e+01,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    9.5714425663102212e+00,     1.6264361879553911e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9148390232989208e+01,     1.5288322850305891e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4656650423360320e+00,     6.8127668225673460e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5307274917644556e+01,     6.5880564802542103e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.1745842916832627e+00,     7.1960386571565450e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4249428016978261e+00,     1.7260154635030553e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2369671757390925e+00,     6.5497068639092031e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1097049849781198e+01,     8.2937465356140583e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.3520257219210912e+00,     8.3073615166455984e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7831401734288583e+00,     8.3500213326013018e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2646323441793667e+01,     1.4935776612914104e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6758923012218636e+00,     1.6082044361412549e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.5663194777372297e+01,     1.4914799020894327e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7510834086125534e+01,     1.7507859138446733e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1208174128713967e+01,     1.0958808674859029e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3912779491439178e+00,     9.4263993030809634e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5878444583930886e+01,     1.5622384093777619e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5822163842573271e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7564912467567130e+01,     1.3968541088457096e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1330086412929415e+01,     2.3478424744912256e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.2835866080804365e+01,     1.3498673398216670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6088895065119573e+01,     1.7114838168753744e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3711622232645489e+01,     2.3905202761729310e+04,     5.5292586687509885e-05,     2.7638888590477866e-07,    -1.4701490065965570e-09, } , 
      {    4.2464555175531920e+01,     7.2313358621240131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.6442011585545204e+00,    -1.6899670470994574e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.5996519934496447e+01,     6.5834540803713917e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1072243627289393e+02,    -1.7309684772390028e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.2816623210499252e+00,     7.4684943050342481e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5117168657304418e+00,     1.6131246973129426e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3597415918594116e+01,     7.0382704467461062e+03,     1.8330491744047710e-04,     2.8707931846145255e-07,    -1.4112926257098591e-09, } , 
      {    3.5823189830896061e+00,     1.5478319940167938e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.2756625928411509e+00,     1.7050403818939889e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.6097937241486466e+00,     7.7217137475557583e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9846219294617473e-01,     6.7566075458514997e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6072275218558030e+00,     1.0447387812463210e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8006788222465957e+00,     5.4350783046440974e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0423209376613940e+01,     1.5542425657607024e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0204581604758342e+01,     1.5543082922356567e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8933583693534885e+01,     1.7900880108139721e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5934178027923823e+01,     8.2353914200968757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9550532284853396e+01,     8.9438810984707270e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2564583518170643e+01,     1.0358230622902245e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.8598559686063290e+00,     8.5044594629045914e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2111208911576080e+01,     6.7446625719463755e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8067266305922381e+01,     7.2137342329835719e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3479640774002085e+01,     2.4047238133217485e+04,    -3.1567126968814438e-04,    -1.6284387342240958e-07,     5.9525423766628801e-10, } , 
      {   -1.2559847965008661e+01,     7.2143914977331169e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2623539166280942e+01,     2.5161842312151493e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    7.2233179013224502e+00,     1.3816738826478739e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9170253729722273e+01,     8.3266138818397485e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7022974588422151e+01,     8.3307689674071516e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3102004093991217e+00,    -1.2903965975462468e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.3364610719428125e+00,     7.2478442928135055e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.8833296807621016e+00,     1.7232721337081250e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1434908284523733e+01,     6.5864666257796780e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8157884630485981e+01,    -8.1204563989740564e-01,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    2.6060505608966258e+00,     6.2830195521406040e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.4898640107761404e+01,    -8.3160721862054634e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.9609782632396758e+01,     8.3413106630414368e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.3492005585480857e+01,     1.5607188639795653e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1188984585705143e+01,     9.9007753033953995e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7512112270860385e+00,     9.5077543337024126e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0448953033297995e+01,     8.3640284955601728e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5945358555505807e+01,     3.5371887272368867e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9418906418322885e-01,     7.3073628698849197e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2500131294085783e+01,     8.0310922092562214e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3458117386869477e+01,     7.2119853225746447e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2682546874305913e+01,     7.2161404081420451e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9641188222881460e+01,     8.3654511864939250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9021940580558478e+00,     1.2139553541353828e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2785899786235766e+00,     6.5213209304989450e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0356140335966870e+01,     1.8293901077832710e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5186625364162190e+01,     8.3237603923658535e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8009105972672536e+01,    -1.9488839833353495e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7954648005995402e+01,     6.9994463275803537e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.3997269866739401e+01,     9.5171832084254220e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5621505398722398e+01,     6.9198165237845287e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9775359694321715e+00,    -1.9006704986510808e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.8438743413990835e+01,     7.2139648137845052e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0766309839299221e+02,     7.2141609169321837e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4207521730607466e+01,     8.8871092316502090e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9622471317724099e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2036081642658079e+01,    -1.2451519477308569e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.8388400443517883e+00,     5.4880474018552895e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1836942834504384e+01,     8.3131493963350204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2780090669555150e+01,     8.3227913705382216e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.3984743166229990e+01,     8.3442334529118798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9172591711992819e+00,     7.3783561752979103e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8345846704020495e+01,     1.6644264938048633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2419650567823868e+00,     8.9356691016911409e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7391155848247955e+01,     1.5508972862526634e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -2.4995071847653136e+01,    -1.3506511271354015e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6950274706091491e+01,    -3.6082627396572207e+01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.3937676328430591e+01,    -8.2483293269717251e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.7506714163415005e+00,     7.7627406466080629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2622514016089699e+01,     1.4880695287643670e+04,    -3.2702649679942452e-05,     9.7949041174756439e-08,    -5.2743365181587530e-10, } , 
      {   -5.2920266450209041e+00,     8.0968018900797906e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6087833053897622e+00,     8.0132227159425856e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4742761991454119e+00,     7.8592911312114811e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5328078729587535e+01,     2.5193322539665241e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -2.1682331965963790e+01,     7.6980826396364973e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0791450985240902e+01,     6.5327861831016098e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3038951204778023e+01,     8.3336224568810467e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.6958790676788631e-01,     1.6617567947348987e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5975593374089016e+01,     2.5197924939548062e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.6332601534966003e+01,    -8.3467403680793141e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4466661258429692e+01,     1.5073353996569826e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9323250403345064e+01,     1.6697197751144813e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.2911303425853774e+01,     3.5902950880144147e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5938872609045500e+00,     7.1250453517435544e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3515690813157022e+00,     1.7789845607142468e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0171761058442010e+00,     8.2227532302010677e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2426926647808351e+01,     6.7051999860065380e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1056178769938079e+00,     8.4346296190458324e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3609478304499740e+01,     7.2353927733361961e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5156674677376202e+01,     5.5356930186341742e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1891114704369334e+01,     7.1927329573804927e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5988602190719137e+00,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6328324482815919e+00,     8.7980917180354190e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0107747573758100e+01,    -1.1149808752231806e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.0904128875013257e+01,    -1.1142762433070307e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.5740963932715946e+01,     7.7690759450494879e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.5562684865401764e+01,     7.7736783449323084e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7016988569228616e+01,     8.3124820892827338e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5847673065627045e+01,     2.3935880064419103e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1778663990824590e+01,     1.8686922047525695e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1300738915035655e+01,     1.6658094194713776e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.5459871794349898e+00,     1.6656671503780024e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4813115749321348e+01,    -3.2858661987913069e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.0826542932966836e+00,     2.8385933410258446e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8846922326168638e+00,    -1.4841705721395204e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.7468792119638323e+01,    -2.4272873609482303e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7054687592992739e+00,     1.2168002681931741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2297344056275620e+01,     8.1140748868454593e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3778977739427265e+01,     1.4915158050403128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6392239287925259e+00,    -8.2926430082198494e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1646254661108413e+01,     1.1035273942083563e+00,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    3.3016120275954698e+01,     8.2534403635527378e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1194825923271825e+00,     7.3849151308970431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2489856302169898e+01,     1.4325954021788621e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2610999246790493e+01,     1.0293796273088388e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.6427173714675298e+00,     2.2754515955398729e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0528217898937259e+01,     8.6441601333043145e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1757955105723479e+00,     7.2839526433771289e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3987021634688780e+01,     1.0751251592595232e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3195508428447342e+01,     1.4861477607725061e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7461489297685560e+01,     7.1985208370699147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0657342867205289e+01,     8.3331006517937094e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6703691689281570e+01,     2.2720768738936542e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.8626923832107956e+01,     8.9546978097275824e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.1191402974715157e-01,     6.6026759611203952e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.9721245967808620e+00,     7.5226051193081457e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.5660943044319964e+01,     2.2759118355281553e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.7299577749134158e+01,     4.8701865814555276e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.4373333056409940e+00,     8.2485093312951431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6951816818802605e+01,     8.2716035259598684e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3247855834623987e+00,     6.5857552803128010e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7367854487077647e+01,     3.9302096969298746e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7793798130178198e+00,     8.3985212684822091e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.8238341336113109e+01,     7.2296048936467741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8874188060024419e+00,    -8.3095037548674209e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0320088710165617e+01,     2.3854525033797650e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.5985841415349977e+00,     8.3478790943794811e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.2635057178193350e+00,    -1.5076495289580932e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.8164866669464715e+01,     8.5433079624014408e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5610361845030646e+01,     7.1971421845507530e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -1.0178596587534734e+01,     8.0569869881818795e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4132418577572228e+00,     8.4088735179517589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0130700416444483e+01,     5.9972625971651705e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7014371432133988e+01,     2.4983773073928940e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -3.8377375091809682e+01,    -1.5925960759164900e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8204011386878452e+01,     2.5022122690273947e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.2466125084361579e+00,     6.8985941566774809e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5500411712289171e+01,     7.7648181893917636e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0621471618385243e+00,     1.5797185729657698e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3411845174858268e+00,     7.7713771449908972e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0064272882635692e+01,    -1.1997610743472023e+01,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.7546768201687623e+01,     6.3313238406368982e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1767675610940266e+01,     8.3908245744958967e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6109331740840727e-01,    -8.6359420039809891e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9710684167336666e+01,     7.7000315236198321e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6112879818143583e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6396202770720965e+01,     3.5460344981765957e-01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.7672146259678151e+01,     1.6093509614015213e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3693074773023163e+01,     8.4066883115158762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1399195988109092e+01,     1.4948209381891797e+04,    -9.4335939581907799e-05,     2.5053619635961265e-08,    -1.7972527972411611e-10, } , 
      {    8.4314555842025598e+00,     3.8027696093255372e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8166139602975164e+01,     7.1802814379031852e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.5835195149855963e+01,     3.2375929557840933e+04,    -1.6337886192581817e-04,     8.7875000732245171e-08,    -6.4072976095436282e-10, } , 
      {    2.1934719080674856e+01,     7.4459523999020039e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.1403535729179666e+00,    -5.0435465816027181e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.4289308351727982e+01,     2.4988375473811760e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.4462220301638586e+01,     1.1769853662952619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9096140169063087e+00,     8.3110593983489816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7495063143917747e+01,    -3.1480227513751743e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0672713471832873e+00,     9.5559973877790526e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7977186333471529e+01,     1.5248507948407980e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0310585448424504e+01,     1.5328137752203804e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2442515996270895e+01,     7.4286794031363352e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.8613843017024658e+00,     1.3970670411209845e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1861697507096638e+01,     1.6675407057448698e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1107397280201775e+01,     1.5540676747198093e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1696458865841445e+01,     1.6623601421791740e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    3.5043058652368593e+01,     1.5544831832765496e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0383521623139295e+01,     2.2181478667505791e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.1834364710212792e+00,     8.5382408903597516e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.6797956758262760e+01,     8.9930474260725914e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7387840434428483e+00,    -3.4944889009392369e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9892639534177977e+01,     1.4660015265260348e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3201126674904870e+01,     1.9079943017218680e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5975906093364705e+00,     7.5083092069321601e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4786780741739875e+01,     1.5033891410629989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0996957840441269e+01,     9.3369020681637139e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4786032792236661e+01,     1.6650823893647768e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.7576177816585092e+00,     7.7010951850447836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0371964860450473e+02,     8.3281834659843353e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9900853950373113e+02,    -8.3291993832625649e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8534123741960215e+01,     9.0214333594828495e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5055522939583676e+00,     5.6419789865863950e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2730344501130810e+00,    -1.6691164276702060e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.0499384003932807e+01,     6.3093741736172706e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6288457161763123e+01,     2.3349076323982084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.2676040887507455e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6151368217905677e+00,     6.2702865716319375e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6036651046551775e+01,     7.2501112817619451e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    8.0647164850922834e+00,     7.1338807720300365e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6375864069806561e+00,     6.5055731869844931e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1351543000533391e+01,    -2.3029349387987281e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5934850836717155e+01,     1.6486530583708201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0786474868330069e+00,     8.9562876642021165e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.3527647992366418e+00,     1.6499648494906469e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4824117276142580e+01,     2.3282953031457615e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0654978284764789e+01,     1.4914446704936252e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5929534617037026e+01,     1.5564084197959646e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5588805400083345e+01,     8.3444239086227499e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.8723874044860378e+00,     1.2566151700910864e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6351214825395786e+01,     2.3713711360264813e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.7144062039645442e+00,    -8.3352503802225838e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    5.5651886338923369e+00,    -7.6441243410864018e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6748333267791779e+01,     7.8198285452716436e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4276368297980548e+01,     1.5521424382003943e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1514132820101596e+01,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3950387653285926e+01,     1.7268769753484848e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5974182913878867e+01,     2.4790585377054276e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.4580865349158483e+00,     6.9918941612040562e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6827449740962692e+01,     7.4857673017999159e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7609307519377765e+01,     7.7007361555359812e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6472133470551373e+00,    -2.2653055349338092e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0488833118176494e+01,     1.6815117203587331e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.4395470269624822e+00,    -7.6666324222853264e+03,    -1.8577293420621736e-04,    -2.8655019280988964e-07,     1.3553984564147406e-09, } , 
      {    3.8687370758443844e+00,     1.7579241861223818e+02,    -4.0178050132663691e-04,    -4.7568047009658573e-07,     2.2392574303087244e-09, } , 
      {    6.7993176979916292e+00,    -6.2567775272935887e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2470717499733208e+01,     6.6284150962685044e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7600689727781024e+01,    -2.8850395197565376e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4664977927032400e+01,    -1.6209335340717018e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0263257907397206e+01,     6.7470825091869074e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4033169585737591e+01,     1.0686817242781375e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8152828997345907e+00,     4.1643119620077772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5839734154903592e+01,    -1.1282626482692422e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0118883185734029e+01,     1.3367972634193943e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1604689205314905e+01,     1.5227285581300930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8790371150779137e+01,     4.3232306666228633e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1065319154300859e+00,     1.5559674970789387e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.2460174261096533e+01,     6.3711387425348103e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4449478198614933e+00,     1.6654753016930714e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1401806835606806e+01,     1.6660012681563087e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.9987255983706831e+00,     1.5871340909860926e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0426249356129608e+01,     1.6710351946458093e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.6546988607342215e+00,     1.2395292688527138e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.9633056197527274e+00,     1.4598977996255388e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3774115312844817e+00,     1.6604413752035707e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5409418886446357e+01,     1.1144272562288221e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6686700390303262e+00,     2.3831630812707332e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2562337500911600e+01,     6.0001075112229610e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0560916264013315e+01,    -1.1133368846728445e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1697312467737602e+00,     7.7003838395779067e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2231758089337028e+01,     8.6454796713295091e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4641058246849035e+01,    -1.0903248852587421e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.5577947908335990e+00,     1.7443424788632878e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.3192335476816623e+00,     1.6496361397840737e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4917577645805491e+01,     1.4328798935846411e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5408337875480917e+01,     1.6679924754379146e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3392533703003320e+00,     1.6135013458623740e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0906726085587263e+01,     1.9472963986911669e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4513782051759987e+01,     7.9225525437321949e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3694922213781311e+00,     1.6091432071231513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9828017394138485e+01,     1.5527212261693365e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0574127528508370e+00,    -7.9560465653792562e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7370339555085359e+01,     1.6879551553401190e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3913175124543033e+01,     2.4446784202439594e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.1365147699500646e+00,    -6.9889778018784739e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8724649333832907e+01,     2.1970775215954163e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.6083187505208429e+01,     1.5542656238407953e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1001865651202459e+02,     1.5542852341555634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9390378027148216e+00,     8.0344450830496344e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0522513659067585e+01,     7.7376200978668339e+03,    -2.2749431109341485e-06,     6.7506572029158348e-08,    -3.7786378305677235e-10, } , 
      {    2.7301635126115315e+01,     2.3911260616503158e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.9885478087854338e+00,     1.4834264611607941e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5599052726073829e+01,     6.6812248694345544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4847715476410411e+01,     6.5883851126289865e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5895871226532435e+01,     1.5558296318270224e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4940526210999423e+01,     2.3320690390571825e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.3432966678600259e+01,     1.6640109852481233e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1395007485326179e+01,     1.1452814816238019e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2870341321172011e+01,     6.1693240841120232e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4808789336294950e+01,     1.4699830167158261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0149370477783435e+01,     3.3490558117106033e+04,     2.0738912972266782e-04,     5.2719502652203315e-07,    -2.7046785641738789e-09, } , 
      {   -2.4906857450664795e+01,     8.2092467189297131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1010706451693851e+01,     1.1015106480668383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4061166582443690e+01,     1.4920438359396940e+01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.9256564587189477e+01,     2.3924414811816438e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9325687023994469e+01,     5.5228953248342805e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5455594033501578e+01,     2.3818476617394052e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1880248451724789e+01,     2.4657487653991222e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.7199860342948528e+00,     6.7434896346532323e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2590045978306669e+01,     8.8375543039752556e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7374956300325177e+01,     2.2756464839382068e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.3676384308252238e+01,     2.2757169471298213e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.4264862536684866e+01,     2.2291439598433499e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.1093469511967975e+00,    -1.1634615542319864e+00,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0939808867119847e+01,     8.3014614625603699e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5888522394551039e+00,     7.2490077543677362e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.3659559670877819e+01,     7.6329431902770966e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3352300440757460e+01,     7.1517060583281882e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5201247333279486e+00,     4.7320306302129484e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7776663169952780e+00,    -8.2822907587503032e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1157153570541100e+01,     7.2942449586866514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3575783102624221e+00,     8.3455521712710179e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.6958298827153033e+00,     6.9143419001919319e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9047185365619775e+01,     8.4481361303171871e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5229044330097503e+01,     1.4498015508735472e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5431463427088890e+01,     2.3868815882289062e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2488985185711774e+01,     8.3750920904965969e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.3038976378144753e+01,    -2.6535158994842050e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1873524581298156e+00,     1.4874631803038339e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9497278442871000e+01,     2.3874075546921427e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5794764057755943e+01,     9.5154168166957534e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1623478398390070e+01,     8.3106424811675879e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5254595626034522e+01,     7.2226993637411788e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0577335814187601e+00,    -3.9583404001132294e-01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.5857634585487581e+00,     6.5459403784148890e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7194769475285735e+01,     1.4917076537252435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2945813092465022e+01,     2.1228392028053695e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9980398186081878e+00,     1.4425824530775280e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1410130226421238e+01,    -1.4511511592244483e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4826210673833589e+01,     1.4957106520891955e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.4674940267804232e+01,    -9.8166191621749022e-01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0075175925264089e+01,     7.8423704504038878e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7909391107780905e+00,     7.7430436555336100e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0871070813014017e+00,     8.6229377661972649e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.3760318896363643e+01,     7.7710248290328227e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.7298608633212496e+01,     7.7717294609489718e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8028982144372765e+01,     1.3852787242882338e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7882973514062224e+00,    -8.9232363327134481e+03,    -1.8044517122040035e-04,    -2.8778297703821500e-07,     1.4642779129183203e-09, } , 
      {    7.4074740170947138e+00,    -1.6429330993956535e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.5541906148583067e+00,    -1.7632026274469709e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0428116783618623e+01,     1.5462572196653486e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0266950739056639e+01,     6.5520038630202207e+03,    -1.8499505744226863e-04,    -1.5276983297989825e-07,     7.0855034680477553e-10, } , 
      {    1.9436460791047146e+01,     8.3593443469821468e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.0299918614209055e+01,    -8.3129589406241503e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.7317248017297143e+01,     2.4264466684298233e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0064769251132841e+01,     1.5072181059276681e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3305316097330859e+01,     1.4430426930658101e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -3.6934620686463731e+01,     1.0973555624994374e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1601642460350519e-01,     6.6205900233812281e+02,    -3.3480526443891184e-05,    -3.5831318655234917e-08,     1.1941445779408981e-10, } , 
      {    6.8931038394037838e+00,     2.0426571094770613e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4737750954370171e+01,     1.6657054216872126e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6992528040135259e+01,     7.7229257447101518e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6766239510956801e+01,     1.6068890166099269e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2420080719831383e+01,     9.7299230378567008e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5890873477471640e+01,     1.6657711481621671e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0525972938940983e+01,     1.7179752239870060e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0582505336020489e+01,     8.7085944831181696e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6503998099863963e+00,     8.9569990096689926e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2962353126467091e+01,    -8.3441105357631277e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1206352983005027e+01,    -1.7085226553869916e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.8860846348786238e+00,     1.6106627525213480e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.3662641578179979e+00,     2.8449140577913781e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8193020122692580e+00,     4.3770833556963680e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0212845236710574e+01,     4.7162516363158493e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2329072903902500e+01,     1.9865984956604658e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7016322880445355e+01,     2.2363796185647152e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.9113931464642810e+01,     7.0392612294782657e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.9729201637642682e+00,     3.2197835939287281e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    8.1431331015073614e+00,    -1.1675976564762971e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.2188542803163536e+00,    -1.6369979498882660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.8655675165142807e+00,     1.0054706888126506e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4452833440795887e+00,     7.6474147423667155e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1952362139892241e+01,     8.4706780354494313e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6341788291283521e+00,     6.9853352056049225e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0102078677912129e+01,     1.5757370827759785e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1076406946033867e+01,     8.6852945732274220e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0015716578967501e+01,     9.5323374975033457e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1122949592707982e+00,     1.6730463662918642e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7198295229119207e+00,    -8.9737562785325929e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.4251355717107552e+00,    -1.0616594620539142e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.7120938691313974e+00,     2.5229380786730715e+04,    -6.5365679416237024e-06,     2.0358073082858322e-07,    -1.1209861934614689e-09, } , 
      {   -3.2870951019994514e+00,     3.1083207380022177e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.6877799484616379e+01,     8.6284123897898626e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2330592073574877e+01,     3.2202438339170101e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.0668563424588533e-01,     8.3446909328474467e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5455572303735847e+01,     1.1079838212474362e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7150307351489179e+01,     2.2773737836147735e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1687267782301973e+01,     6.9787762500057888e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8016427671413382e+01,     3.1087809779905005e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.0113985477915783e+01,    -8.9042248868852766e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.3391005221011035e+01,     8.6056647694315961e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9113376514517231e+01,     7.0946181596646074e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2411536600711980e+01,     8.3129436811089981e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5698305326836621e+01,     1.1371704643973499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8676198434998792e+01,    -1.3245303409594821e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.2383284387379305e+01,    -1.3238257090433322e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4637352422690856e+01,     2.4969153593062754e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7742853743286929e+01,    -1.7031212424127366e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0236972145611100e+01,    -8.5021924739561528e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.3578090862010539e+00,     7.7341465165334521e+03,     1.2413964430425204e-04,     2.1365477127109444e-07,    -1.0076900843229813e-09, } , 
      {    9.7865346544469745e+00,    -1.0808715121410423e+03,    -4.0158463659946864e-04,    -4.7576773655918550e-07,     2.2378029892653955e-09, } , 
      {   -7.7050996801879791e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9441454569404186e+01,    -4.6446178730874976e-01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.2657865065665661e+01,     8.3286847110727249e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.4805167761930036e+01,     8.3286981381741753e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6831783324862975e+01,     1.1537293531981210e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0332698375612768e+01,    -3.1880365150391299e-01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1344218083426867e+01,     8.0171720862839666e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7044059807376392e+01,     1.4948598067882202e-01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -6.1075316616100332e+00,    -2.4952317226746283e+04,    -4.8769386823796122e-04,    -7.8860433323336171e-07,     3.8818061819078319e-09, } , 
      {   -4.3727416519108608e-01,    -8.2611773303753234e+03,    -2.1392569766429153e-04,    -3.2361429569344993e-07,     1.5836923707124101e-09, } , 
      {   -2.4093775730826195e+01,    -5.5260821413692611e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0762303873597681e+01,     1.6799418220735075e+04,    -6.6379040851001885e-05,     6.2204988982121232e-08,    -4.0656475297845690e-10, } , 
      {    4.1438583595112570e+00,    -5.9454490808999708e+02,    -2.8152763458074149e-05,    -3.7064102883560250e-08,     2.2829391429766939e-10, } , 
      {    8.2485851019236858e+00,     2.4356780791700485e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0585987026336873e+01,     1.6168760675085927e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2738347610495939e+00,     8.3962055188715749e+03,     9.0659117860360849e-05,     1.7782345261585954e-07,    -8.8827562652889158e-10, } , 
      {    1.2818130375771489e+01,     1.5383024513167895e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -7.8095877265334614e+00,     9.2114304493448963e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1724064879456565e+01,     8.3265491864016294e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.0129575889257154e+00,    -8.3308336628452726e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.2767902029438325e+00,    -1.2723629136055376e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2277027442780303e+01,     1.6727212693105659e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.4815639349308114e+01,     7.2347941712979718e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5177366444472181e+01,     8.4683768355080210e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4011687729109269e+01,     1.4365768923686534e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5719738714724048e+01,     8.9504400540698607e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.8740394801040292e+00,    -1.6834175672392344e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0166068238649972e+01,     1.7808059824915603e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.7275346236745982e+01,     4.2923307555598194e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0965549683421703e+00,    -1.8995165128357589e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1736999599624889e+01,     1.5778040905334350e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.7160715251976796e+01,     6.7135507265447814e-03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1679853176229077e+01,     6.0370774990630825e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1276801013001988e+01,    -8.3418093358217156e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -4.1827790546190204e+00,     1.7245875532394530e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4370052876257814e+01,     2.5158601724983564e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9238804317623650e+00,     7.8210405424260362e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9725678757987581e+01,     1.4427773414758614e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1612430209279141e+01,     1.5542042944514918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1316016702940487e+01,     1.4428478046674763e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.4224356314363131e+00,     8.4308631335515183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0257949425790827e+00,     2.5019831320994414e+04,     4.2606057833599587e-04,     7.1570891169456661e-07,    -3.5340978098160731e-09, } , 
      {    1.3607132181024285e+01,     2.5231672156010249e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.1593676625136336e+01,     2.2692382805526282e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.3454633981888149e+00,     8.9143391937132892e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.0436531314282789e+00,     6.6195179572683483e+03,    -2.4662834734423393e-04,    -2.2566525451869346e-07,     1.0562587188965348e-09, } , 
      {   -1.7597878424148803e+01,    -2.8522799018119827e-01,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.3632330348952223e+01,     6.4109536444327223e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1589399239123164e+01,    -3.7387614536963997e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7445677016776937e+00,     7.7003894694093897e+03,     1.5495628925523469e-04,     2.5010248204049204e-07,    -1.1815442703688610e-09, } , 
      {    4.3465492324160394e+01,     8.1538897887433714e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8544424618958246e+01,     1.5543465635448671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2388516895707689e+01,     2.1606140179671438e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8548331787055098e+01,     2.4029180068945676e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7991004841274865e+01,     7.5255822036978279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9512776703782420e+00,     1.5837000631555609e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0620764932698624e+01,     2.2723060108216079e+04,    -2.5384211505901077e-04,    -9.0035718346214099e-08,     2.4609142453120021e-10, } , 
      {    9.8104278234733595e+00,     3.3406124347172772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6269455898561976e+01,    -5.3813608561543219e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.8196324159585768e+00,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7327369855363912e+02,    -1.4014469910473053e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2106669920385094e+01,     6.2356807030156233e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5233025649565167e+01,     1.6872965789425954e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.8650204740503842e-01,     5.6431786111198778e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9541848358285154e+00,     9.3500199793619813e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0006796911918904e+01,     2.3245439329501114e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.0086667906942957e+00,    -5.4297050378736458e+00,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.6370003607339761e+01,     1.3642083791330715e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4703235369791537e+01,     2.2669889484385237e+00,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -6.2605774987166811e+01,    -2.2994649440235548e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3494385379106626e+01,     1.5622936383310103e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3751399552727008e+01,     2.0259005926297643e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8983043773306758e+01,     9.1716155474469833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1958474878846911e+01,     1.7208138173280317e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0161249837595552e+01,     2.5050508623684207e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.9333566028549853e+01,     8.5034930605035279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4265161437753520e+01,     8.3283323951146504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0129377887181924e+01,     8.3290504541322498e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1438404934436583e+00,     1.6860746647567908e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.0718030781585242e+01,     1.6173363074968747e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0446995096280105e+00,     8.5574190843768720e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3504775659183611e+01,    -9.3300004526574980e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.1367819917401420e+01,    -7.0463191614976495e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0472018911630030e+01,     1.5536195334382661e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.1635292983107163e+01,     5.1092726060088371e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0552778851230235e+02,     1.2695893581027349e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4028084535219413e+01,     1.5529636378783527e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.1444770109284313e+01,     7.2321118088142066e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2960241698587032e+00,     9.2255392661935866e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8480059758350826e+00,    -4.2680239714814797e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.3143479053559162e+00,     1.3735383795857289e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1684574457786857e+01,     1.0677292206116781e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.6610124621023914e+00,     2.8286990488630363e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3244325956745673e+00,    -1.1172583915812884e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5981304827941663e+01,     1.4739645069056172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2439952914701177e+01,     3.9324626244204808e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3603363769910661e+01,     7.3888645012384241e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2738168680235422e+01,     3.0510170092129239e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2472489728999896e+01,    -1.1119987269489220e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0272445455228045e+01,     7.0790761313761777e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3309881565317010e+00,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8707162188760291e+00,    -1.6653838832985248e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.1100716746404604e+00,    -7.2160758957211817e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.2350588773215136e+00,     8.1889718027459076e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -7.5510058412218427e+01,     8.3279867927073010e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6528674095374527e+01,     1.6913332980856350e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.9645920574922911e+01,     8.3293960565395992e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8987564875047767e+01,    -2.5333555880841777e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0837755644995657e+02,     8.3269604561462111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3640272531248016e+01,     1.6886119984739234e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.0039102950315236e+01,     8.5359396904183413e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.2195175719439955e+01,    -8.4738065405458947e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.8113728394186722e+01,     8.2317886240619573e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6480329289876821e+01,    -8.2665582747510034e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4131507131436374e+00,     8.9907504269615747e+03,     1.1881188131843500e-04,     2.1488755549941980e-07,    -1.1165695408265611e-09, } , 
      {    8.1433895155521512e+00,    -1.9187229330914042e+03,     3.3676391171059432e-05,     3.5744052192635206e-08,    -1.2086889883741836e-10, } , 
      {    1.1385068315104593e+01,     9.7430409490549682e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4223491046186094e+01,     1.5307467674629237e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7690798345071386e+00,     6.3861686630103100e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5431789627517176e-01,     8.3284677674657396e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3843201486060504e+01,     1.0122944007549690e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5210820827687005e+01,     1.5214167670102664e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.0967662421390152e+00,     8.3289150817811606e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1285349218060432e+01,     2.3240836929618294e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.5239728999787934e+00,    -8.4684110465009944e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4516088114469426e+01,     4.5745531950684426e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5163268479951689e+01,     8.4039424856941623e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5454543420546250e+00,     7.2386755892333372e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2950522753708436e+00,     6.8800016218866167e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5929161125127109e+00,     6.3635865761585128e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1546115133665133e+01,     1.7346683833973111e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.5964103973368537e+01,     6.8366451523188516e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3025368519508920e+00,    -8.3126919163994535e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.2950844567169897e+00,     4.2659815955703628e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3826686413540157e+01,     2.6308390626558236e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4632062473399344e+01,     8.1937046906412834e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6877987083210840e+01,     1.1472859182167349e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1264405854243446e+01,     8.5405420903011618e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    5.1470798243144493e+01,     1.5367952654101717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4165657954215440e+00,    -8.6750524666351751e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0050264589988622e+01,    -1.5747743514451031e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4660779743204506e+01,     1.4035104761023700e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4240735030905007e+01,     1.9313234210097771e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8794920688726569e+00,     1.0309281255487740e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6769539078459790e+01,     1.5423309584288058e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8307501587549446e+01,     2.4047213752886382e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.7565937149540304e+01,     2.3542859094726118e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5794128325401029e+01,     2.9088811421913433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4651664126569138e+01,     7.3335075710520814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0504605048677291e+00,     8.9569933798375096e+03,     1.4962852626941768e-04,     2.5133526626881740e-07,    -1.2904237268724407e-09, } , 
      {    1.4130807437944570e+01,     7.4493494807109000e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3793581463178326e+01,     1.3869707923689930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7402659996293264e+00,     7.1147360704880666e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6675796903071998e+00,     8.3288398416806631e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.1485922066508110e-01,     8.3285430075662371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.7918261899626344e+00,     7.0382948270772031e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.1949442993317021e+00,    -7.9694518823856242e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.0363732110110249e+00,    -6.6113995521056310e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.0499589355930041e+01,     8.4636781586056168e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4674576963873092e+01,     2.2667799641725353e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7412589278288717e+01,     1.5648692484404177e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4463286294132175e+01,     7.1118911564302762e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0912546238170311e+00,     7.6202017462495987e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9507073477918357e+00,     3.5337070936721659e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1278918691403371e+01,     1.6149731967049485e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8254136172945895e+01,     1.1930314501674195e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.4062253451382887e+01,     6.2831429859626996e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4872326094717874e+00,    -8.0470041433977476e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.1391547486678352e+01,     1.6577753045451074e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.9501174432693396e+01,     7.2091318331007478e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4313199168044530e+00,    -2.1102199643873823e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.3025339296377023e+00,    -7.4883953716705537e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1082858359776548e+01,     3.2217200185560241e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.7845203154298446e+00,     1.1292505612600507e+00,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -8.2513415555257072e+00,     8.3132723134837725e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0246665595302737e+01,     1.6737012653042722e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.6525507655566081e+00,     1.1926254461575309e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9556541103001507e+01,     2.3793893453593122e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.4492235259671045e+00,    -1.7350104933270350e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.5173702494732247e+01,     2.0652026895990632e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4625933326708340e+00,    -1.9268871908873450e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.4538728057737993e+00,    -2.0764385369322217e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.5245218213593475e+01,     6.5857485667620740e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9652757851690108e+00,     5.4066923712338394e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9240894077651962e+00,     9.0371811029973032e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6290375073283787e+00,     6.5082030193006776e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0959643814405204e-01,    -1.1544434611630179e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4143362998629422e+01,     3.6759761870474335e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.2240837657601276e+00,     8.3783548220585890e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.5203459963884303e+00,     7.6834631587703134e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3579215828443665e+01,     6.7450148879044500e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7187186896602036e+00,    -1.7171989946943947e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.0756774302181071e+02,     2.0355170170754775e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3184076973162897e-01,    -1.1020626722364071e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1424856669899450e+01,     5.5410164990664807e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8120738803083849e+01,     7.2075039097592116e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.1197971051434097e+01,     8.6103787058491525e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -3.4764301145564815e+01,     7.3490495993405111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4153980650907991e+00,    -4.5779038198631497e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3057718141731531e+01,     5.5022935757018258e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0906037431583323e+01,     8.6879309668612768e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.1079028976540510e+01,     3.0299466640577612e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.7145310025829154e+01,     1.5537823257724198e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.2935822963938683e+01,     7.8400692504624767e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9023125310747044e+00,     9.0099681068801838e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.0091020325504978e+00,    -1.1139172137982293e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5046002094610902e+00,    -1.1153399047319820e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.6710642507272233e+00,     7.6136427906504650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.7218098560370407e+00,     7.4781665690567305e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1080823752864482e+01,     1.5547685322239387e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0487784950462249e+01,     1.6867260947357972e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.0140998577680488e+01,     1.6866603682608427e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.6454507948402611e+01,    -9.5601204467984644e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0214589752271266e+01,     1.7665593492787164e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6792867687631858e+01,     8.9397260129033239e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5240495275172654e+01,     1.7148619596208904e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.9708032459932792e+00,     9.3734302058697722e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1009912023622153e+01,    -1.1483856063891690e+03,    -3.3995134669750335e-04,    -4.0287231502039035e-07,     1.8900946171736367e-09, } , 
      {    1.3089992469115540e+01,     2.2457096190173725e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0260885167921208e+01,     8.5662380093272204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2355690452072352e-01,    -7.0585984605766726e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8812312665499370e+01,     1.7272572523094179e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6245202538435162e+01,     3.2236185555632292e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.0523193483281066e+00,     3.1178403515078116e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.8998859268900121e+00,     8.0338862321994802e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.8725235982567121e+01,     7.2189938976159428e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3647103680387623e+01,     8.0776570657564434e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9397452489771620e+01,     2.2790574202464199e+04,    -3.1547540496097606e-04,    -1.6293113988500929e-07,     5.9379979662295946e-10, } , 
      {    1.5408559294824444e+01,     7.3200010597807268e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0716255567036635e+01,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    6.8456980706405455e+00,     6.2162767451513901e-01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.6469284140987845e+00,     7.6605689376799946e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0352569641578384e+01,     1.4314168059711651e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9226744927655908e+01,     1.4712317078690776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0976738747590501e+01,     1.4836894443924128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8587122747328486e+01,     2.4393815105228405e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.7327472375067096e+01,     3.8814179353585132e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5840601510515739e+00,    -2.7229962063081111e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3359921500574572e+00,     7.1081246709359621e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4851012514949009e+01,     8.3544305597715611e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1430279148027793e+01,     7.1188910332740907e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7995798648998505e+01,     7.2784972151722031e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.5186707659362169e+00,     8.9220541206596008e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1242342169388351e+01,     8.8783885005432452e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5388575490328314e+01,     8.9795409148012368e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1853802221146125e+01,     1.6587493071228117e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.0224024373996716e+00,     7.2096536381880860e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4947768399844151e+01,     1.7226171857221030e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.5796573163861879e+01,     8.2335195925391963e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1637766231083809e+01,     1.8058614462480149e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5025170249469328e+01,    -1.1341123125098488e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4391453588906485e+01,     8.3162399051461416e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.4561599184802311e+01,     1.6051617169333600e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6010451478346702e+01,     2.3322767933355524e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.6421770946194023e+00,     2.4078693980400138e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.3790854781383288e+01,     8.8341401344390179e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6803446563729931e+00,    -8.3411429441007604e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.7521339287312273e+01,     8.9573513256270689e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3799617185483420e+01,     1.5560803233437655e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7700050493132888e+00,     5.2165804515628406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2943390923761005e+01,     1.6171061875027337e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2575227237965469e+00,     9.5426467787588335e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5663359594221157e+01,     2.3243443907027640e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5023166892874247e+01,     3.1085156264005516e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.6035786401848455e+01,     3.1085860895921662e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -4.9527222910601864e+01,    -1.9907450948956102e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5925195796927500e+00,    -7.3596329353072633e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1956447412512329e+01,     2.2234447764716981e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1392521322350420e+01,     1.6579830588234774e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.0078677414516237e+00,     6.6207001693221937e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7393875217627880e+01,     6.4092682346353013e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2504864507033329e+01,     3.6147409882490110e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1661339081334560e+01,     2.4083296380282958e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.7217903748421058e+01,     3.3019021118843320e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8941544914595838e+01,    -3.1776990862227649e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9968254317072454e+00,     1.3710800632056356e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.1663370101389631e+01,     8.4238632567077038e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1848096297706842e+01,     2.5260058089420509e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    6.6910297514452939e+00,     1.4205957026562399e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5266271749432207e+01,     1.0515964977242678e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6844047079270776e+00,     2.0954946573630150e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    6.2630541449868112e+01,     1.5407767555999628e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2414373008629630e+01,     1.4945316177598261e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2028588272950813e+01,     2.2599082800999709e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.3976002919172046e+01,    -6.2830087149481687e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6593962260579914e+01,    -1.2205924723978569e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.0519880259044356e+01,     2.2168324472192511e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7290500396066502e+01,     3.2166380092104631e+04,     2.6921828435180138e-04,     6.0000318159822853e-07,    -3.0538413773089666e-09, } , 
      {   -2.2656152487271640e+01,     8.3151849133520955e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1538697324485820e+00,     1.2352852621132345e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1576978280528197e+01,     8.3996847300364407e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.3997740314207174e+00,     1.4911816872620067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6792311497584869e+01,     8.3421979358948047e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3273824550924704e+00,     5.8772656019389506e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2630874368883248e+01,    -4.1369104726755222e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8367012105131991e+01,     1.9045571086917334e-01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8929231209920028e+00,    -1.1080696036659720e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8300364480954627e+01,     1.1865880151860336e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9308224269787456e+01,     2.3242785813601629e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1293339511697155e+01,     1.7844108241319202e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4034979637091947e+01,     2.3279186545963301e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.3622178255693790e+01,     7.2711507640219252e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0034649308835487e+01,     1.3277448755047984e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.9096234112086652e+00,     2.0150518789739522e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1198730444363033e+01,    -6.6211902266328343e+02,     2.8348628185242411e-05,     3.6976836420960540e-08,    -2.2974835534099799e-10, } , 
      {    3.7649662432880930e+01,    -3.1132455813112756e+00,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    7.4809062053567050e+00,     1.6734935110259023e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5943160431959752e+01,     8.5047628919983163e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.3830604425693345e-01,     7.2302722006990607e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.6339261211365601e+00,     1.5208693046510067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5808417884688168e+00,     1.5524705346525934e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2754745329887484e+01,     2.3871445714605245e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.8611323595663221e+00,    -1.2128443365341227e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2678464195064334e+01,     1.4307469027868559e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.0147362751463774e+01,     9.1318006455490722e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5377177395459466e+01,     9.4624477194845604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7600784313775463e+01,     1.4914439991385527e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2333077954672937e+00,     7.1787257944216226e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5431099305268143e+01,     2.3948997975617367e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7062270707424098e+01,     3.1624798651394347e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.4125969030762047e+00,     1.1499656302018273e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9369857631614238e+00,     7.1960139219024823e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.4551700525120353e+01,     8.5658498675336850e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7234123820662710e+01,     7.1828844618432668e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.3035475262458100e+01,    -1.0636614249512490e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6096488333372655e+01,     4.6827203076370097e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1603043501226720e+01,     1.1513883211355801e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9744901911866275e+01,     2.3837664287150084e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.2029921054851950e+01,     9.1016154528111583e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5934630430006351e+01,     6.5857619938635271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3104829198670631e+01,     1.4607501903046659e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8570268942412063e+01,    -8.3277250222223847e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.4772163715728471e+01,     7.5481241088300721e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6569301082336052e+00,     1.0107675985337697e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4303354527499128e+01,     6.5061254765169770e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4035851976238352e+00,    -1.1844584031238639e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8362412621913862e+01,     7.8446716503452953e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5462456608101363e+01,     8.4733078677656158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2805866107814339e+00,     2.3791815910809419e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.1371038339265915e+00,     7.7030136718940903e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6595934901224979e+01,     2.1045047865683620e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0615022715699716e+01,    -7.9853460953534523e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5876937229124781e+01,     1.6012154583393763e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.9308581917489526e+01,    -5.7087898663580745e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1778050252552292e+00,     9.9176968403512956e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8994226191396817e+00,     7.2199629194435738e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9598019341027300e+00,     7.1773031034878695e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3060518852642094e+01,     1.8451635432173138e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2976321752013455e+01,     3.1051751532839527e+04,    -1.0154970729668459e-04,     1.6068315580844063e-07,    -9.8989257408945052e-10, } , 
      {   -1.1935926439018083e+01,     1.4834816901140426e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0038913766768509e+01,    -1.0748136573671934e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.7047171795402164e+01,     1.5662198995675531e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4480136459416059e+01,     5.8953145453948109e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9371860553519319e+01,     3.0692487610270600e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0322085575685163e+01,     5.9787776032451093e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5998637093244179e+00,    -8.2642570748095914e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0890170287887862e+01,     9.3434610237628476e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4545928721630386e+00,     1.3577649441516856e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5677854770367656e+01,     1.6137299198071792e+04,    -3.8030412665759474e-05,     9.9181825403081772e-08,    -6.3631310831945483e-10, } , 
      {    1.1367810730497768e+01,     2.2850117159866713e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.0830005831188885e+01,     1.8362316315483440e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.9676420129035122e+01,     1.2323335471367182e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5936472744318699e+01,     1.5073706312527900e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.5923939945242665e+01,     7.3092346974425982e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1673040845930856e+01,     7.2009449541600789e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.1263915609065300e+01,     1.5717555925861872e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8394689360624515e+01,    -1.1169297592065159e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6959697300349120e+01,     8.2733344944371092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4914516876290797e+01,     1.6000209609488636e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -8.5143198398672624e+01,     8.3044185510139669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0731548310187092e+01,     7.7184080477797061e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.2996591046400653e+01,     8.3529642982329315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6906256353282323e+01,     1.6251243968355648e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5667983537766812e+00,     1.5576535717436956e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2512981716973608e+01,    -1.1921808202772299e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4839045558168698e+00,     6.2082941847648647e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3352005813068658e+01,     8.9566466937109180e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.2825291082708524e+01,     8.3840483548097927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.7859155349096525e+01,     8.8999111110054127e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9008414003946479e+00,     2.3835397298201646e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.4830611359739159e+00,     8.2757318031511695e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.1260716335313918e-01,     8.3003054912131920e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2592466004765157e+01,     8.8974804325975801e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.2105281857211008e+00,     6.5831254479966174e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.2866259934435256e+00,     3.2407385405023590e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    8.2587714866592464e+01,     7.1587059351720027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.2205400986472696e+00,     1.6719739656277055e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2775461071899560e+01,    -7.0793738884136769e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6322650186952021e+01,     2.5178703058799056e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    7.4400672833369823e+00,     8.3570773580337082e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7678167121208088e+00,    -2.4583163800932923e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0597653372958575e+00,     8.2058325493934753e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4016894262625229e+01,     3.2411987804906403e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.2340690784762799e+01,     2.4986426589828421e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.8782587240118961e+01,     2.4985721957912276e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.5127061230546477e+01,     2.2809786252551334e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.2878634226228577e+01,     3.8878926927536668e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3928597548036892e+01,     4.0077619579419988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2961966567327686e+01,     8.3178747133665947e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.2779217614882370e+01,     2.3871117082230470e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.8495120343412070e+00,     2.3871774346980023e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0019284876490111e+01,     7.2200963398010972e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -6.2190937678329981e+00,     1.6648746350864069e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0976930773536687e+01,     2.2703848058128948e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1550036185086249e+01,    -9.5689420492467389e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0665273100543150e+01,     3.2375905177509834e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    3.5169174213893877e+00,     8.7992646553285613e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0346745325678505e+01,     7.2137038358495429e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1193478685606335e+00,     7.4650972242253520e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4840482339859911e+01,     7.2144218948671469e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8718218528743069e+00,     2.4459938397752874e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.6061941206547331e+01,     8.1694318170318002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3455995988311654e+01,     1.7215800656273659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4794640343074342e+01,     6.4507685463306352e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6064931284549573e+00,     8.1114384932116045e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.8078332905545381e+01,     6.5702132520243713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7677209289306290e+00,     7.2316948916328138e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2589010306589596e+00,     6.5850439348459240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0320162794659722e+00,    -2.4694595066252091e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.2977088903703695e+00,     1.5536720815539044e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.0634121525722158e+01,     2.2206061831306721e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.2546754223208580e+00,     1.6362577656822932e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.6933226270850714e+01,     7.2005563540869898e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7085921887674481e+01,     2.7173653603957523e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8508913173551012e+01,     1.6746400362861688e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6723256483628361e+01,     6.0768924009609946e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8011812014830618e+01,    -2.6660706584633132e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.8333325678184025e+01,    -1.1146218457143791e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.1421395319771346e+00,    -1.3241780250014071e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.9129713123330504e+01,    -1.1146352728158322e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.9809976644534402e+00,    -1.0808471318099444e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.9091599451532391e+00,     1.3500097180504734e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6421610364956102e+01,     8.3296578270245136e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1102567906107158e+01,     2.2754187323023958e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4483203899026606e+01,     1.8844656401866127e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.1184705823383766e+01,    -2.0350143397274223e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0960780331851545e+00,     5.2489580168934071e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5198208260757646e+01,     2.2759446987656323e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2697689584870194e+01,     8.9596288419851771e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8641503677911498e+01,     3.6949230815773199e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2132933021025822e+00,     1.3188431241433196e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1595522562431391e+01,     7.8604193938597491e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7864051441837539e+01,     7.8596434471695557e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3399188247719151e+01,     2.2717002253442228e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1551425189328542e+01,     4.3942163556613468e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6809023534901229e+01,     1.5520212384849550e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6067175735834738e+01,     3.2233894186352754e+04,     2.0758499444983606e-04,     5.2710776005943332e-07,    -2.7061330052172078e-09, } , 
      {    4.2441808146706755e+01,     7.2275693766296990e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8250593956641694e+01,     2.3951075518401070e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.8025266759341363e+01,     2.3243490445517775e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2884733188039418e+01,     8.8199281715854559e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0006870471500214e+01,     9.4144543291758382e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7494282815372387e+01,     1.4288440319832118e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0693270383226551e+01,     8.2331314507456591e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3254123384459309e+00,    -3.4492772922036721e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1816136228982145e+01,     6.1138690423794981e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5803889923625665e+00,    -1.6058653062271096e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1962496101921293e+01,     1.6790491071572338e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4741227365602228e+01,     2.9934831604294886e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.0718556639782456e+01,     3.2199784823270620e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.0211947450812453e+01,     8.4879510322151000e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0359495811411179e+01,     3.2200489455186769e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.0005534439763490e+00,     7.7315622430929852e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.7082683762272239e+01,     7.2694197955446853e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6284118793572104e+00,     5.3821092074329063e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6142511650277328e+01,     5.3270638057755787e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9598547202830055e+01,     2.3332155643174490e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.1786121463487227e+00,     8.3853584035380245e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7646655779678550e+01,     8.3198312690829080e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.2357680184248967e+01,    -8.3375515801639958e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.3579063989314434e+01,     1.4914453418486977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5902378849478058e+00,     1.4878398288532653e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1854303244427152e+01,    -3.4662030381984005e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2859699680834140e+01,     8.2817920859700243e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.6890175780014989e+00,    -1.1146285592651057e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3867510977547848e+01,     1.8875525874528130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7100507042398565e+00,     1.3893118150197721e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2658302344047122e+01,     3.9071976975157850e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6383137822206450e+01,     2.3858327803406981e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.8439556508181155e+01,     3.8941817365806946e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9884263857162878e+00,     1.5183514747743968e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1392244450433687e+01,     8.3220800250713455e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3362828255878988e+01,     8.8636990051424182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2477321070906251e+01,     9.6654886880428458e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4124769154878690e+01,    -1.6651349374804147e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -3.0499024767923466e+01,    -8.3459644213891170e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.4435192036398599e+00,     8.4242513985012411e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.5714087534951204e+01,     8.5260349656357739e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.5117360078902884e+00,     7.2080293909155935e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3547034666266725e+01,     1.6663416323689653e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.1176102639026399e+02,     1.5542246331342680e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.8782556654004779e+02,     1.5543262248620909e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.0265398555119116e+00,     1.3241400338644387e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4978070660871605e+01,     2.3209386712267122e+04,     1.1958975808238373e-04,     3.4866791532941113e-07,    -1.7634176504365261e-09, } , 
      {    3.0170618845991598e+01,     7.4673984241667622e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9149051154093044e+01,     7.5653971055957409e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7748203575751006e+01,     1.5525481293216126e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.2725215692314649e+00,    -1.0370762982529814e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.1504472072351707e+01,     2.3617014274929345e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.2954615712426943e+01,     2.2796632057238054e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0078757814590507e+01,    -2.2123364377226180e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5329164888273592e+00,    -2.6030239400822666e+00,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.2424850097355630e+01,     1.5677741023963963e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1705908556876965e+01,    -9.3115705530841967e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.0994472531437482e+00,     7.1856769319480863e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3790163253952940e+01,     1.5447582457897541e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7945358176455084e+01,     1.6108705067997182e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1979102422068863e+01,     1.6031376464142766e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2701229807086417e+01,     4.5515411956543267e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1688307167617243e+01,     1.4950495121339849e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6689143453876280e+01,     1.0908985946935667e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4138948956788749e+01,     9.1735284001661057e+00,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.5902937509792221e+01,     6.2883355150877996e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7270058995124415e+00,     7.7687473126747136e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.7142770883726300e+01,     1.4779459970954085e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8674034130168643e+01,     1.7321738850696041e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8265204273659435e+01,     1.4394368683592627e+04,    -4.0613452282133695e-04,    -3.4075459250086880e-07,     1.4820754231518511e-09, } , 
      {    1.2567494607584598e+01,     6.7976316691575666e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5395534440059869e+01,     3.1414095199842719e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.8018078002647556e+01,     2.1438068835376605e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8111347985356758e+00,     7.7740069773070809e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1926311624120733e+02,    -3.4560240734833769e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0405913867827820e+01,     1.4898904676647822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.7494395514276579e+00,     7.8416591049370109e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0940547054000348e+02,     7.2135549067192296e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.9018111979056087e+02,     7.2145708239974610e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.9672464716401080e+01,     1.4270391376376258e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.8102003654573746e+01,     9.5914577327607731e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.9986955023338098e+00,     2.8351722417286510e+01,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.9722887635084849e+01,     1.2258901121553325e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5255809997454909e+01,     8.4493481274715796e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5297987433577894e+01,     1.6756709644117178e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.9392053479184398e+01,     7.1897899917488630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6149266547052612e+00,    -7.5764309126926364e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.9029086276448417e+01,     1.4405583825584445e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3768594744315367e+01,    -1.2712089277902157e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.8095863677261164e+01,     1.0575406606015249e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3601094301062595e+00,     8.3248886550141251e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0775703227593461e+01,     8.3324941942327750e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4393378075407153e+00,     1.6832439542061165e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.8312100469508046e+00,     3.8132918131353413e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5813954501121028e+01,    -1.1484099867202651e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.5000971039243828e+01,     2.3264468037537554e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.3439253600793837e+00,     1.6892669464599454e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8943050171680731e+01,     8.3973835300950304e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.2433972860978422e-01,     9.5454916928166240e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0099678137625785e+01,     1.6170350529560461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5883841996214585e+00,    -2.2154707647977352e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0735007355958832e+01,    -1.4819123793418391e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3752865252152301e+01,     4.0183743850164831e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9606133416422104e+01,     1.3905756340093529e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.6993778354999911e+00,     8.3183821433679623e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4340681390212078e+00,     8.3390007058789379e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.3663931771115099e+00,     2.7997118967469760e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.9724661741411687e+01,     8.2802400243427037e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5352023513140606e+01,     4.4007829276349858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7504998471015199e+01,     2.1479393958530636e+01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    8.2356778809558531e+00,     6.5857609101442840e+03,    -2.1581170239325131e-04,    -1.8921754374929586e-07,     8.8240453285065505e-10, } , 
      {    8.3837209598614582e+00,     6.2901893051241950e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.1266637941544261e+01,     2.3871452428155972e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.6196390746907483e+01,     2.3871439001054518e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8025557524309566e+00,     1.6688561252761978e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3124615227635328e+01,     7.2424487987686025e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1116467022124283e+01,     9.2339723544771405e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5779839520672397e+01,     7.0045133996220429e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.3280947786379780e+01,    -8.3373279230062853e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    9.9006317674769093e+00,     7.1442330214995854e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.9232884472638922e+01,     1.5548787764424547e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.9866867057799254e+01,     2.5836550563070181e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5545920442865164e+01,    -3.3088781982054206e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8920833979604208e+01,    -1.6633063405148241e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.7809540633112235e+01,     7.6831108428122388e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9587268100885709e+01,     2.3872157060072121e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8444739608352268e+01,     2.5443529593377192e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.3754943698057673e+01,     8.7977394020773445e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.9788925712410261e+00,    -5.3929007143075387e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.9326730931399482e+01,     1.6026774064259946e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1985170923584366e+01,     1.5436816095559412e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5905860220859136e+01,     1.9237677371559115e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1616163187740714e+01,     1.4285786803932633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9494340975662166e+00,     7.9122602284226723e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3880568022072113e+01,     1.0058509657735831e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0180450436534709e+00,    -8.4707267961116249e+03,     2.1867144861332805e-04,     1.8851388517253349e-07,    -8.2941924564219393e-10, } , 
      {    9.8619515298578662e+01,     7.2383357389678267e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6161368004639293e+01,    -8.3401814124801767e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.1450071015168227e+01,     8.3172014367667234e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8601221195423541e+01,     1.4375156633505498e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5335237638329414e+01,     2.3870734369138368e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3490552719616744e+01,     8.0753558658150323e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0240444900980492e+00,     7.2102600957490185e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8190749525709840e+01,     4.0704620982464388e+04,    -1.1086454163491999e-05,     3.3859387488689992e-07,    -1.8767137595750137e-09, } , 
      {    2.8159675167593306e+01,     1.3839633047569057e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.8199092437091364e+00,    -1.1326527674669046e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8073495526772554e+01,     7.2294819764980220e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6697531629022535e+00,    -8.5551615711673439e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.5316289031465278e+00,     1.5524730081779997e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2092583377720079e+01,     8.9017513614789365e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.2120604077377177e+01,     9.5056767909187110e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0511885724654398e+02,     8.3287894761972893e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.0982974563462292e+01,     8.3285933730496108e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6483230933559831e+01,     7.2178656349676703e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8568764345581165e+01,     1.5858222998662659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3685481932637373e+01,     3.1807116169535708e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.7649681541067110e+00,    -1.3771471222125986e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.7137580406394068e+01,     8.0802868980726271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4210420636550438e+00,    -8.1982336684113557e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7145274107260249e+01,     4.1469633615550265e+01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.3385348537700221e+01,     1.6097767369672936e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.7363726686372864e+01,     1.2434972588928190e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.2968119971217195e+01,    -2.1955413135325580e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6540607494639808e+01,     7.8111920468888093e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.0520496711876595e+00,     8.3570249140807373e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.0380073560421534e+00,     8.1366167919777035e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5885304033710277e+00,     2.3791644766874015e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5535102359924196e+00,     6.4963749302339500e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6996633645122706e+01,     8.2196589360975777e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.2508611032868792e+01,     7.9773571843860536e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1100681220423864e+01,    -2.4748550006701107e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.1872476766301272e+00,     3.6978817033821088e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.4430532552093013e+01,    -3.0592046999136073e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2118686653680678e+01,     1.6493089539307337e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.6345743601562486e+00,    -3.1283887636467944e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4448653027681848e+00,     9.9326794870277098e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.1098662571942757e+01,     1.2716356441060170e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4524575699841739e+01,     1.8537711599976528e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1670755517640920e+01,    -1.2843631231034949e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.3479605921153592e+01,     1.6402951409570996e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.0107444562030068e+00,    -1.4226909337526763e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2235200655277776e+00,     1.3135462144222005e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9738238129702793e+01,     7.2130964629572827e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8508460759930763e+01,     3.1103863300887397e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.4207355259189463e+00,    -5.5111335433012201e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2878101124282288e+01,     1.4481094827927880e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0763379243294917e+00,    -1.3639929268993194e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.7918011298528285e+01,     1.6102369769555757e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5290911290972987e+01,     1.6854480795505893e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1814176290034464e+01,     6.0214374192008127e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7388703371881986e+00,     7.2412928274214255e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2991900460333703e+00,     7.8051341921149615e+03,    -6.3908233012899474e-05,    -5.3888495096368257e-09,    -3.0155410965013152e-11, } , 
      {    1.1054254613524602e+01,     5.1697389657825875e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.6148720807480103e+01,     3.6027960349179011e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6906699357657835e+01,     8.1835763087010055e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.3046859740933858e+01,     7.7997106344481845e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8115424586634017e+00,    -1.3268078573175899e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -9.3338523333688798e+01,    -9.8051573838887407e-02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9049160344524623e+00,     7.8243462422020884e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6424015383756490e+01,     7.1185028914805544e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5685210411629908e+00,     1.5530135051563808e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -9.0648542555226861e+00,     1.7206060630496620e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3894827693639705e+01,    -1.3215481926852235e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.6271380015129104e+01,     1.7283389234351031e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.8661083504677649e+01,     8.4019859299778509e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -4.1510947548530247e+01,     1.3916019040732526e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8583997829995322e+00,    -1.6674303530054498e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.4169832140321775e+01,     1.4350128843824263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4988005978233669e+00,     1.0001737790915315e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2972450883889568e+01,     3.3522013964288686e+04,     3.2886489253401133e-04,     7.4146618990729021e-07,    -3.7668083767486503e-09, } , 
      {    2.0064886876482056e+01,     4.0879440512703059e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5616557832322044e+00,    -9.5033524932554042e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.2607613443470271e+01,     1.5542395260472995e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2389938727749112e+01,     1.5543113319490594e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7521056303518137e+00,     8.3713512405791535e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1753003374966994e+01,     3.1119265627087651e+04,    -1.6318299719864993e-04,     8.7787734269645441e-08,    -6.4218420199769137e-10, } , 
      {    1.1835664866479696e+01,     3.3700107582842335e+04,    -2.2520801655495176e-04,     1.5066845656049714e-08,    -2.9156694781927502e-10, } , 
      {   -4.3713822029139795e+00,     6.1335125190742974e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4791303895003551e+01,     1.7256631475449803e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0259392255860398e+00,     7.6977540072617230e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8439876401606735e+01,     2.2756817155340141e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6084195188128216e+01,     1.4428125730716689e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.2738462960186760e+00,     8.2419503756960075e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1289216734196579e+01,     2.6229571532763170e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2681582179914965e+01,     3.2183216458421099e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2991218876748732e+01,     6.6394357229908692e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3883755607043620e+00,     8.2860316086677467e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4281226383960684e+01,     6.6435600144914122e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.6438917296501501e+00,    -2.3188953454365924e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.7514909575259097e+00,     2.2145430251102189e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.5947132364120819e+01,     6.4905834482285472e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4742213320994653e+00,     7.8581181939183380e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2219158405688511e+00,     1.5890729163357412e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6901401022692362e+00,    -2.8534894738932053e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8331075646010234e+01,     3.3526616364171510e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.7550473223123809e+01,     5.9503317556263937e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8740255535488487e+01,     1.4425495898400506e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.8203274270873987e+01,     3.2264571489042548e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1926954891876930e+01,     2.4593053304177360e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    9.6550532457951022e+00,     6.4901953064350109e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2849685589730704e+01,     1.4430755563032873e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.6383033204989825e+00,     1.6911814288922804e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.6512596154251007e-02,     6.1282999608059563e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5201617058953643e+00,    -2.2936914683440682e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3754748095034941e+01,     2.3276900806515248e+04,     5.7956468180418396e-05,     2.7577249379061598e-07,    -1.4157092783447669e-09, } , 
      {    4.1597679040606970e+01,    -1.7067281539646501e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4734703225474334e+01,     6.7848297898023629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0372073280068580e+01,     2.1553171082460249e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.2826847120566327e+01,     2.3864886759006109e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.8819288359908796e+01,     1.0339477931565175e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.3044266146136323e+00,     5.5644267255742707e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4978151934530484e+01,     3.3490533736774938e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    7.1761182895442062e+00,     8.7451226208242279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7326684738276491e+01,     6.6813564847807866e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.0260575168414210e+01,     7.0548032577666963e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6738784788244558e+01,    -2.9526268100016750e-01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    9.7484253776115857e+00,     1.7755090727704410e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.8738762838226577e+01,     1.6171414190985413e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1036416338116844e+01,     8.3290846708858917e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.9005454364910612e+00,     8.3282981783610085e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4950660430457845e+01,     1.5487397359795452e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5698671581146089e+00,     1.4790493778050981e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -4.5434462592717807e+01,     2.7608977173933454e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.6334717418511699e+00,     5.1714057558812829e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.6967409050932304e+01,     2.3878329751503742e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1335704682025714e+01,    -1.5252532512172365e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5953338241406380e+01,     7.6872659283796393e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3820995927205946e+01,     7.8423648205724048e+03,    -2.2113946537906831e-04,    -1.8798475952097055e-07,     7.7352507634707552e-10, } , 
      {    1.0662831570759737e+01,    -1.8465474156245675e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.3471079034486607e+01,     9.4946364225041470e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0623410759124763e-01,    -2.8624804763181965e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.0353129072444744e+01,     1.4445398727482358e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4764152600323275e+00,     7.6408652625064933e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6775027463974986e+01,     4.7938038973279736e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7248436946708626e+01,     1.5003464218551040e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2312706851599765e+01,     9.7364819934558345e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.2602598933989659e+00,     6.5677310721110025e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5307770325592394e+01,     2.2805735571458008e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3584753783120519e+01,     1.5637926122066050e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2246751246359404e+01,     9.6236882851727091e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2906756417101501e+00,     5.9336269603615085e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.8515074721631359e+00,     4.8597398168097461e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0512554952624440e+01,     2.2092461153890996e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.0299770836493014e+00,     2.4410735786035999e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.1911418397108537e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0138268493272308e+01,     1.6678712757224752e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7328455549200882e+01,     1.9630698341252100e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1497615004507079e+01,     7.8243367664631787e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7014478517595695e+01,     1.6835780719844228e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1124083082022629e+01,     1.5718546708594033e+04,    -4.6796367745047056e-04,    -4.1356274757706424e-07,     1.8312382362869388e-09, } , 
      {    1.1055926047740138e+01,     1.4388310828818776e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.3967544435443600e+01,     5.9570953793091840e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2329166929460975e+01,     9.0809519365542656e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.6625613024289687e+00,     4.9127089140209373e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2378928870276784e+00,     1.6422096233894346e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8330368598504528e+01,     1.6564082844720324e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0024362871369249e+01,     2.4422201038638665e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.5701826008787085e+00,    -2.3182746321450009e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.7070681355354207e+00,     1.6636052941269048e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5705073704983938e+01,     6.2915089387389853e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9256761637830415e+00,     1.6425493314703243e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6232759910520880e+00,     5.2308073609003679e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7542656894963120e+01,     1.5560778498183594e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.6670132879515776e+01,     7.6882061603919283e+00,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.0323262570852243e+01,     7.1972021187107775e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.4584743654290753e+01,     1.5529247778710440e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9440293123062446e+01,     2.1831089805069594e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9848646603635434e+00,    -5.5731427963255283e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5952969945382144e+01,     1.5366961871369556e+04,     3.3559732520280330e-04,     5.3779819261610727e-07,    -2.6472766243305099e-09, } , 
      {    1.7041906840564003e+01,     1.4461882777840750e+04,    -4.6776781272330229e-04,    -4.1365001403966401e-07,     1.8297837952436104e-09, } , 
      {    7.8587604151210959e+00,     5.1206010936767268e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3455952823940738e+01,     1.6622437960237508e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.8385105026904460e+00,     1.0984192239243887e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4992664008649683e+01,     1.5280332208283835e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7991326795906303e+01,     1.4897526024128658e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.3377902561664285e+00,     8.3659220530808943e+03,    -4.9388246038426593e-06,     6.8122964143321001e-08,    -4.3230351130856216e-10, } , 
      {    4.9413007688924665e+01,     1.4286491435848784e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9206509318227425e+01,    -1.9126707081490381e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0002099194995580e+01,     1.4286139119890708e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3583258946818098e+01,     1.7654780494428928e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4919732371310298e+01,     1.5126323093781017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0309949480686331e+01,    -1.1942583630609302e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5704318469939601e+01,     2.2130810770236010e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.4113200165988539e+02,     1.6219053161767079e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0675759601415997e+00,     4.4462588030123129e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8065358276064352e+01,     1.4620200363362435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2043905631454795e+01,     8.3361695911925071e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0934017841858577e+00,     8.3212132580543930e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4053099533487750e+00,     8.3467251085641601e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.5655226788776515e+00,     6.9829843858759290e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.0161948762248269e+01,     1.4914775337311024e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8276713843095557e+01,     1.3877370406683269e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3694415259201421e+01,    -9.4410187839471473e+03,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    5.9375444093432188e+00,     2.3807011364791386e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.8112019240291062e+01,     1.1302006916628650e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.5267272693092252e+01,     6.0403471143606112e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9275416460257958e+01,     8.1919737221640462e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4805784419316893e+01,     1.5556260801253149e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1630906423345616e+01,     2.5379095243563337e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.0302996881005811e+01,     7.6052120074936529e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8155424526600044e+01,     7.9379716548718716e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0559617777664183e+00,     8.3989733845695646e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8505006678145826e+00,     6.6380130320571161e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2514801189556408e+01,     3.1053767642326784e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7846001647810350e+01,    -4.9867278027886641e-01,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    1.1269959936929245e+01,    -5.9225916864194517e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.2276859793746773e+01,     9.1941574525792294e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0474559471820477e+01,     7.2135105758258615e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8774154221817572e+00,    -1.7781919231162192e+01,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -1.1250531885698814e+01,     7.2146151548908274e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1145223466402982e+01,     1.2651922091246313e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0357416642985076e+00,     7.8554883616021552e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9718685843438903e+00,     1.3342362826164301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5307145213245503e+01,     1.0451530627428820e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0564457626231693e+01,     1.4467940632614602e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.3786837617338605e+01,     8.8613978052010079e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9506210029704540e+01,    -1.3897583847483475e-01,     1.2365830925826718e-04,     1.4561631015239090e-07,    -6.9832562627017560e-10, } , 
      {    2.2711830137612878e+01,     2.6622592502456158e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.2507077164704100e+01,     7.6848418112894769e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.5474227822909725e+00,     1.5558486773981091e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -9.6978832742595955e-01,     5.2257751744315044e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2048732269368516e+01,     7.2320870735601447e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    7.8335889939685313e+00,     1.5969556687129943e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.8291743281342093e+01,     1.4819274872851996e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2569698991568163e-01,     7.3587659722166862e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5031769856577220e+01,     1.7287991634233851e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.6418905852428214e+00,     6.7408598023370496e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2013439164418422e+01,     1.4818886731058461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5664538027266905e+00,     7.1714030494026410e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8773689653889619e+01,     2.1342467630908621e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.2811239171719233e+00,     8.1670430336669142e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.9096129703921214e+00,     2.3905227142060408e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -6.4215702761106186e+01,     8.3284857093974297e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1499046970965004e+01,     9.8074752988688233e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7077599344212189e+01,     6.5303983501264593e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1087459443781533e+00,     6.1728695832317919e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9930934295399901e+01,     3.5034072997817257e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.2054430227988853e+01,    -2.6710629063800297e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1761629136061133e+01,     6.6497450042463561e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9336861831559297e+01,    -8.3061495194912040e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5045794538914453e+01,     6.0640972351565169e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0443459229113534e+00,     1.0873985972020246e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0251890634801741e+00,    -2.5289780836966238e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3661122226474674e+01,     6.5754459990573132e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9901923090200636e+01,     2.3889469922807042e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.8163284873106992e+01,     7.8450002827200724e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4013789676064714e+01,     9.2905014022905671e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.4438646871397538e+01,    -1.3690599989410089e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2945391046793269e+01,    -1.1664436706609754e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.0428261933249829e+01,     1.5558173401121474e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.4741746704831357e+01,     1.5598111220168137e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.3192597710589773e+00,    -8.2047384977381789e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3837276358215176e+01,     1.4286850465357584e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6331353943237690e+00,     1.4850012355122395e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5869150135099105e+01,     2.3307572479373561e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.7765934859308793e+01,     1.5527021805982495e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8422777308896194e+01,     8.9968139115669055e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1488300331755905e+01,     4.4809650209632946e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4549210163992896e+00,     1.5514368356571536e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.7690840161631037e+01,    -1.7720311081087857e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.1316076140243197e+01,     2.5426438423554032e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1377484309883309e+02,     8.3283458222161025e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2791073016037608e+02,     8.3290370270307976e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8605666027081806e+00,     6.0626745442227639e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5345600372374106e+01,     1.4968127147614319e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0673438406636812e+01,     1.6755598626515919e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.8413755605410023e+00,     9.1922856250215500e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6670140403542206e+01,    -1.6659450988097524e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7903422621773537e+01,     1.5475240195733668e+04,    -4.5498862218682970e-06,     1.3501314405831670e-07,    -7.5572756611354469e-10, } , 
      {    6.9648140679554569e+00,     1.6050405172179209e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.7726457345801087e+01,     6.8692105471578543e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4784472048933395e+01,     8.3281391350909671e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1486226411684257e+01,     5.9581590407341346e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6931834182921868e+01,     8.3292437141559349e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.9029800909550261e+00,     7.2297953493576424e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -1.3597778362373088e+01,     1.4283837919949297e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2039235048996549e+01,     8.4526443515087212e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.3668428383203559e+00,     7.6996724941110297e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4850166867506871e+01,     9.2567199748354051e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8880881002359811e+01,     7.0743774544737735e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0899143543878438e+00,     7.3096228392361345e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3831143186466264e+00,     7.6666080419542295e+03,     1.8587086656980149e-04,     2.8650655957858978e-07,    -1.3561256769364048e-09, } , 
      {    5.6419870661042442e+00,     6.5644253723349493e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6369870098051358e+01,     1.7619192826914019e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.2177235742132009e+00,     7.2298106088727945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.7049159401459349e+01,     1.5518481416372311e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6216398393194652e+01,    -1.5322613314889713e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8250827916551650e+01,    -8.3395081358803091e+03,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -5.5995207086768879e+00,     2.2126208370353186e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.0309597604850937e+00,     1.0969965329906358e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7803779395061214e+00,    -8.8330460827837233e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.1877218309484547e+00,     2.2003914638174929e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2380477433659003e+00,     2.5561412761704698e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.6595262290521795e+00,     5.9574476952672576e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7967727452391657e+01,     6.1167073028589075e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0483002919888051e+01,    -7.1593686725282168e-01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.0952560537559073e-01,     1.5527335178842119e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4209220460701676e+01,     2.2914551509680572e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8198153608834360e+01,     5.1868248670209614e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.8105731020239553e+00,     1.6385678412805326e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8990948059215160e+00,    -7.6539831737047607e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7399697851712631e+01,     1.4392788659779966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0342381367038978e+01,     8.7838455441888837e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0634706773755187e+01,     2.7314507357470461e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6325241804105599e+00,     6.8548233231205186e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0073742701247927e+01,     7.4313157967701900e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.0433910783922107e+00,    -9.0617981228682402e+03,     6.4104097740067743e-05,     5.3015830470371089e-09,     2.8700969921684549e-11, } , 
      {    9.5389848714060186e+00,     1.6866932314983202e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.3666020016318807e+00,     4.7352075450905673e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.7490038779793462e+00,     5.1845236670795503e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0202079025281822e+01,     8.3309584135718887e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.8853209970424178e+01,     8.3297873054909360e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3544422955691704e+01,     1.8229466728018848e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1740700121407610e+01,     7.1507844868916654e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4132260543533492e+00,     8.3282955905834388e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.2978830464202531e+00,    -8.3290872586634614e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.0938569174326398e+00,     2.0618562510975480e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.8771633022072773e+00,     8.8018944876447458e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4307746074123479e+01,     3.0563139189340429e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.6815478797587247e+01,     1.6655305306463200e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6815890829413931e+01,     2.3402045421193277e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9379299073744789e+01,     1.6659460392030600e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0392107533593638e+01,     7.4061374980040910e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5651343588139763e+00,     7.1986437542186668e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.9696498599288947e+00,     1.2559038246242101e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9190390099717042e+01,     8.5021582629631812e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.6891269854663221e+01,     8.4862200637378610e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7183515065075650e+00,    -1.0988808157506548e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.7188432776151528e+00,     1.6200772690806727e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9882639912059119e+00,     7.1360659784659183e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8751138535657436e+01,     2.0023719310945089e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2520831069692790e+01,     1.3109377410753154e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2534879331782502e+01,    -2.7170443644157149e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.1307779957767664e+01,     9.0919857436511593e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2544036182994384e+01,    -1.6644763610828915e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.0261817998824842e+01,     7.2140733875621536e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9624962679062392e+00,     7.2140523431545353e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9920580832055091e+01,    -8.1890060137388809e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2879639484860668e+01,     3.0497015896815959e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.2154621799169885e+00,     1.4391365968846214e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9206627072874385e+00,     8.2522501812125865e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.8651858627113334e+01,     1.6170709559069264e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2624261142352169e+00,     1.5636054294508369e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5373737792665800e+01,     8.3750491222825367e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1201954780239895e+01,     7.9018890274753030e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.6494788941139866e+00,     8.4875987162570236e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8786718867295910e+01,     1.6365182285858073e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.0178357188597893e+00,     1.4893116796958400e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2691990609580386e+00,     7.0035814794349278e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5480368162485760e+01,     1.5571140223392053e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4454535520656570e+01,     1.6204813292319919e+04,    -9.9663702567724807e-05,     2.6286403864286599e-08,    -2.8860473622769569e-10, } , 
      {    1.2098991616778203e+01,     7.8761218676964672e+03,    -2.5195611033005099e-04,    -2.2443247029036815e-07,     9.4737926239295504e-10, } , 
      {   -3.3563885443766651e+01,    -8.3275955437559660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5943780855306780e+01,     2.2467921296906388e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5394986968311557e+01,     1.6128402059071634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3369365480831483e+01,     9.4658618890207999e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7091140950032468e+01,     1.5113521214425813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7269217301725433e+01,     2.4328901034112088e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0484175365789032e+01,     1.6173691707343522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1966260864334679e+01,     1.6670002087664889e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    5.8711840262633572e+00,     4.8067707195985540e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0099080192131110e+02,     1.5567027163591278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0222911780594512e+01,     3.0652922358697197e+01,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.2394447177355563e+01,     7.6846360960634556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4183654465801709e+01,     1.4914118072561479e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2834393266431576e+01,     2.1659109276882631e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5285098709424432e+01,     2.2679264894328018e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3016530325780417e+01,    -8.0180863437824667e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.4383293464880975e+01,     3.0927774225623158e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.9953268198919204e+01,     9.2737872563750516e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3166667063606148e+00,     8.3275279630692185e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6937141439747755e+01,     1.0090755304530103e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.0228955921949080e+01,     6.2820953347170359e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2095519103291119e+00,    -8.4051326680343154e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.6782303799013354e+00,     5.6502920657886409e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9366759604183432e+00,    -1.7216062363327969e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1469024037772129e+00,     1.4397472808357994e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.7049573665856954e+01,     5.9578000112253330e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.1693037257580254e+00,     4.7915026973865633e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6048330924455003e+00,     6.3278378466679108e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4170477277155504e+01,     8.5846415562329021e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.4674096147209603e+01,     8.6129300045668788e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7651201916885022e+01,     1.5555373528399781e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -7.3663310033914868e+00,     2.3893987619737491e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5379992304913959e+01,    -5.0670720416894710e+00,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.7609222653089528e+01,    -9.4456211838299678e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4845851130290541e+01,     2.2654645446412069e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6339246762945770e+01,     1.6393230579181625e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.8702277663519116e+00,    -1.6900713742438362e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    5.0009929714052772e+00,     8.5719979374838113e+03,    -3.1112138346627602e-04,    -2.9785701748072627e-07,     1.3509818037798328e-09, } , 
      {    5.6364662835380116e+01,     7.8427227663619624e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3290519486584955e+00,    -1.8749333490348263e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5598964023124518e+01,     2.3190169032348509e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.3978258269497239e+00,     6.8401867199887047e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1568988737904382e+00,     1.5085298970474949e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4649928026520744e+01,    -6.2623004226184275e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.7202381691799431e+00,     1.5565296195114039e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.7579921326162406e+01,     8.6464613332457266e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1383579522319060e+01,     1.5538951520372470e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2208628804001442e+01,    -1.4308927549444945e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.3615526705096105e+01,     2.2128157254336526e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.8523469654437488e+00,     6.5988731915110684e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2908233386688748e+00,    -1.0018658471722905e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8774449693516416e+01,     1.5447194316104004e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2683410503894500e+01,     7.2567226813140478e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0082867186749064e+01,     5.9452052775890002e+02,     2.8250695821658283e-05,     3.7020469652260392e-08,    -2.2902113481933369e-10, } , 
      {    1.6187788846130054e+01,     6.9274193485939986e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4653243944282925e+00,     8.4617996469488880e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8834844923727843e+01,     1.5546557059591121e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.6009104591936607e+00,     7.3369217405883201e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9418020816960137e+01,     1.4859089774749909e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.9006078178639733e+01,     8.8600962091075016e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4134410818279449e+01,     2.7015613472149147e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.3079209931168645e+00,    -9.7771151388715822e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.4122798263156326e+00,     2.4125877154281145e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2109315561671988e+01,     1.7836445758325859e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6746374845138991e+01,     2.6735945268387886e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3762243274732622e+01,     7.2773412438250234e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5977506178575638e+00,     2.3013539513374071e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8968507696500936e+00,     6.2068100141927255e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0048801511152615e+01,     7.9978036048029080e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6334976673456417e+01,     7.2893139264290567e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2947580014170503e+00,     1.5523566620225765e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0770231305443889e+02,     6.5858533318866412e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6731276851833211e+01,     7.5449506851788865e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1056818003093831e+01,     1.5467503228911082e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7907770597602216e+01,     1.5383494682390145e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3640540908318947e+01,     6.4099684915320468e+03,     1.8596879893338560e-04,     2.8646292634728987e-07,    -1.3568528974580690e-09, } , 
      {    9.5625264460918444e+00,    -5.6387323523168652e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3930173278495481e+01,     8.3258465105656578e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.3155948786341529e+00,     7.7540642822559748e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.0777351149680543e+00,     8.3315363386812405e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2831965307445664e+01,     1.6652451816989302e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.2059987846472175e+00,     1.4966704456680567e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6943820004822321e+00,     1.5261067008756092e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2248137990541423e+01,     7.2037535841028566e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0487047065618597e+01,    -1.6279233415407361e+00,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.4377311123640681e+01,     6.5632133751805577e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5502626142983416e+01,    -3.9532216963439896e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7254032049762706e+00,     5.4039359882495944e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.7581387831335284e+00,     7.5360905296383407e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.3375527834801332e+01,     2.5195271423648577e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    4.7739856416724450e+01,     2.5195976055564723e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    4.0862391305045840e+01,     2.2224110774762583e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4611475564878837e+00,    -8.2849205910664878e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.6221821065311248e+00,     5.0136714015408515e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6011314745898670e+01,     1.4904137423680764e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7884996153099459e+00,    -8.3106672164216525e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.5307482531764327e+00,    -1.7947803827124244e+04,    -2.7357230584650142e-04,    -4.6507730400251161e-07,     2.2966593701520934e-09, } , 
      {    1.9478551622571171e+01,     7.2112179513005531e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9850567774288280e+01,     8.2972111971097002e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.8068791213029574e+00,     6.3989723933555439e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5144583102483145e+00,    -6.0911991528951216e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3781834972789969e+01,     1.4031707680214806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2729372920198987e+00,     1.5707047599921360e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.0027811062974639e+00,     1.5069551226960499e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4780799200090570e+01,     7.3109656659198372e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5531734813332019e-01,     7.2169077794161358e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6724473611578321e-01,     6.6790552848393763e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8738273176487805e+00,     1.2005565562744909e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4947375663497818e+01,     8.1551903752907474e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -3.5339225671394928e+01,     2.2775163581087554e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3337355365972154e+00,     5.5775446367725372e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2171906094629499e+01,     8.3724622581804142e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0542455092091455e+01,     8.2407774384028671e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.9590492675617108e+00,     7.2243721466138322e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7443713527216005e+00,    -7.3568534185377621e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.5571550821255059e+01,     9.5282186960509571e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.1565900557520532e-01,    -3.2555014460111843e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.7951022754809856e+00,    -1.5995323846908777e+04,    -3.3806534196854351e-04,    -5.3726906696454431e-07,     2.5913824550353914e-09, } , 
      {    2.0702754523674621e+01,     2.4972956362672085e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9920870534939965e+01,     3.1871550519349563e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.2942920163078266e+00,     1.2398586532437894e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7804590257510569e+00,     7.5088680577823143e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.0186826063175953e+01,     7.6207540357820817e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6786529704163993e+00,     1.7259953284676190e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.3690429759861217e+01,    -4.5350594916546447e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0769436636619893e+00,     7.7643735635114617e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2951899709336949e+00,    -1.3705786908745549e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1353358296361229e+01,     3.8964282694747135e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2604911389191297e+01,     2.0413502021357908e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.4190386001820102e+01,     1.6641840820958470e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.5970896373786161e+00,     1.7264360526314591e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5135367715805598e+01,     1.6651482795161672e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.3154091917038073e-01,     3.9411898804645629e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {   -7.1627425689627785e+01,     1.6672924877535326e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.9621468327748506e+00,     1.6440129917835053e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.4453589505508440e+00,     7.3537824872358879e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.2713990773161427e+01,     6.5878328230965017e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4116894301595242e+01,     1.4978436428869807e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5392669226119679e+01,     2.3854172717839578e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.1670552603637461e+00,    -1.9988862759200979e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6731184895420988e+01,     1.0844551597121805e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3795395174157550e+00,     6.5923142359119347e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3000283941826574e+01,     2.3149838125033129e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.4771252784090549e+01,    -6.2631822652725532e+01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1750935438243449e+01,     8.9577103551358687e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0015752421317778e+01,     4.5848376340664117e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.4720038205002947e+01,     9.2572735027054478e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3826188163096653e+01,     9.0349958965614187e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9621172471410844e+01,     5.5798458367139492e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.6706768073536775e+00,     1.7576803828114203e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.9305666602093856e+00,     1.5453736776367004e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2804624634427750e+01,     1.6692719920183623e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1238367062826473e+00,    -8.3242821974531889e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4596681636301525e+01,     1.7791139144108009e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1446516507722805e+01,     7.6450269093915649e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6395800941713727e+01,     2.3505656692507197e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -5.5908382493817221e+00,     4.3984817276935755e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0100721731839245e+02,    -2.2944863906836872e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.9257948612816804e+01,     1.0177257587036129e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6357256245021262e+01,    -1.7432884602687236e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0241881520661906e+01,     1.4269687628495058e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.0372032995043654e+01,     3.9416501204528453e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.7142038044649013e+01,     8.3297949520176571e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -3.4594564915520699e+01,     1.6600294950583320e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5550052470127847e+01,     1.6136116986017947e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6333056431727655e+01,     7.6265836262706825e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2186297506488248e+00,     9.5853065947145369e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5913162707097690e+01,     1.6858482456470683e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1753441688184436e+00,    -1.3433656947574382e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4494904808422348e+01,    -1.9643476694799240e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9534798602650913e+01,     1.1695027886321639e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6002451318830939e+00,     3.9218192733201818e-01,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    2.2911370872534498e+01,     4.8739859906562815e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.7552170990602285e+00,     2.3869368171821541e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4522729877943625e+00,     1.1612544593051920e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.0275313193930149e+00,    -6.6208338266922078e+02,     3.3578458807475305e-05,     3.5787685423935058e-08,    -1.2014167831575411e-10, } , 
      {    9.3438647070029575e+00,     2.5739135148110243e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.7231681793001163e+00,     1.3154195313279118e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8525297163437231e+00,     6.6556450583315864e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5981416671177547e+02,    -8.6027459719461774e-01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3305523930910059e+01,     6.2222931813232144e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8602966464667290e+01,     8.4460853959095875e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.7401974480108045e+01,     2.3873523257388948e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2587508883123828e+01,     9.0999486627124634e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7058781772223952e+01,     2.1735488600601610e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.0498587200058473e+00,     8.8690850234484096e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3384495387325908e+01,     9.4302020726902883e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.8374080291371087e+00,     5.4359133500292955e-01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2575884071380308e+01,     8.4111747178931673e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4309891797722871e+00,    -9.4095629367644906e+03,    -5.5387704436179476e-04,    -7.2648661071384027e-07,     3.4737869878860464e-09, } , 
      {    1.5397923253592904e+01,     2.4114752227465604e+04,    -3.7730455959010967e-04,    -2.3573929496120473e-07,     9.4296260975804725e-10, } , 
      {   -4.9714282671300133e+01,    -4.2281106605878449e-01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.5247962811183168e+01,     2.2642645981201167e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.4578855942340560e+01,     9.4260469871228870e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2567559163332593e+01,     1.3044943060939302e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1996969703698287e+01,     1.6694142611117375e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0213106688331575e+01,     2.2299361835833297e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.8353022181011305e+00,     7.7526415913222218e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.7250458331788856e+00,     1.5604887439854243e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.7685916770106207e+00,     8.3356950061028856e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.9044791638749004e+01,     7.8994583490674686e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2110381811474589e+00,     6.4618023534275308e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.0103099175890449e+01,     7.1967898685926775e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.3543073917834988e+01,     1.4978881054750109e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.9575019759052861e+00,     7.6228315785657833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3735371110081880e+01,     1.0129502963148821e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4909299887334914e+01,     9.8876573921971358e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3250030427027948e+00,    -7.2609622040117729e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2971265880994141e+01,     8.3244260525912250e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6123337370530848e+01,     2.3243849475026578e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1680574086971124e+01,     2.5025889175768261e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    9.7787382226981663e+00,     1.6990885241868476e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8873494010319032e+00,     2.4946259371972435e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    9.0594132705385153e+00,    -1.2566639307532805e+03,     1.9586472716825521e-07,    -8.7266462599716787e-11,    -1.4544410433286028e-12, } , 
      {   -8.2302580034193156e+00,     2.4497452099709379e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.7360709380111459e+01,     1.6028722948243283e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.9663673543888791e+00,     1.6341914140566036e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5206669464546705e+00,     8.1697841329898756e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2174389527092515e+00,     1.4078727301228344e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0513439155730637e+01,     8.3278793789835545e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    6.5802330054957039e+01,    -8.3295034702633493e+03,     6.6345003052078436e-06,    -2.0362436405988305e-07,     1.1202589729398046e-09, } , 
      {    3.0173891165747364e+01,     2.0416740280638074e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1459085798877705e+01,     1.6763321043669283e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.7757743390196747e+00,     8.1981795421390452e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.2595278769904823e+00,     1.8073704941245055e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.6825529059230941e+00,     1.3700491350800869e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2666518310016656e+01,     2.3656829176827254e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2157188701008934e+01,     7.2266821037763330e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    9.3476976438279120e+00,     6.0372442124461054e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7042653224772305e+01,     1.5876815533453522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7492264872175465e+01,     7.0405618160256417e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.2438454549325810e+01,     8.9232119523823494e+03,     1.8054310358398448e-04,     2.8773934380691514e-07,    -1.4650051334399845e-09, } , 
      {    3.5074543816310829e+01,     1.4916524247719952e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8263851425565203e+01,     2.3892775622583096e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2970029529224334e+01,     1.0200496268561812e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5395646162716140e+01,     1.6662313881504499e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2991125330316507e+01,     3.0534753255930173e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.9393696837462237e+01,    -6.1819777012822669e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -9.6436679513086858e+01,    -6.2840563661938234e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2623417780851634e+01,     9.0033996755421413e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.8535283667243583e+00,     1.6657382849246900e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.8272501424325114e+00,     1.6187982555834931e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2261935043491857e+01,     7.1618051136140293e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2878071001981187e+01,     1.6066311522490283e+04,     1.5001746465139205e-04,     3.1822544618381309e-07,    -1.6138477816774232e-09, } , 
      {    3.2241669609888461e+01,     2.2988706689883795e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.9188006629264677e+01,     2.0199094891187960e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7379247531661747e+01,     3.1138477677174782e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.2363966563590214e+01,     9.2544187748939949e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6361258195231947e+01,     9.6497409445283938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5808882289255088e+01,     7.3733224729499934e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9976528948903089e+00,     1.4233170022679516e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2855308536592055e+01,     1.0318038476576476e+01,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    6.9261467616951009e+00,    -3.3787057286643176e+01,     2.8250695821658283e-05,     3.7020469652260392e-08,    -2.2902113481933369e-10, } , 
      {    7.5803571530196230e+00,    -6.4522826585313658e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.8688076116619996e+01,    -1.6675431792702759e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8427189712016556e+01,     7.2332505351143764e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -5.0969257994936257e+00,     1.5168878144612154e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2010228550968325e-01,     1.4004451838665005e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1439428435166622e+01,     5.7030162555913548e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9372730813294744e+01,     1.6641173513906182e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0049437063275821e+00,     7.6900745583224189e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6225652103015240e+00,    -1.0229361923274528e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    9.9854075576339572e+00,     1.7126783142658867e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7124397122239216e+01,     6.3000663969657111e-01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0302633492841018e+00,     1.1219523623358931e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0332112585151258e+01,     2.3577199373031428e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3645087566964575e+01,     2.4986785619337224e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -3.4274786330696614e+01,     6.7423879071845477e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.9129574773276818e+00,     2.4985362928403472e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.3506678560898635e-01,     1.5534117791598961e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5665921190717334e+01,     1.4550984605946665e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6202485282634864e-01,    -1.4919017854436420e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2679115782791115e+01,     6.6030282770784697e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9727823399343720e+01,     2.5208242978024638e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.7146474431858429e+01,     2.3362582835253437e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1092258001214617e+01,     2.2834369416352263e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.1063688341395405e+01,    -1.0349987554692811e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.4158544369739763e+01,     7.6917473411950723e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1712808009654140e+01,     2.2208139374090420e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6651549639652028e+01,     2.3850115806627393e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3281338045035591e+01,     8.0868458536717608e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3416845384797682e+00,     3.6346209898958637e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1276987756520898e+00,    -5.2959621472281015e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.5706306193931582e+01,     2.9821482742594299e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4966407352025959e+01,     1.8622487697711837e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8748619572440845e+01,     3.1834348117130648e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -5.2467325201487419e+01,     9.0521708417532482e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3359977916247072e+01,     3.1032539482752396e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.9135228847046683e+01,     7.6432959409143259e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9619412894955399e+00,     7.2663206171026595e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4591294783276046e+01,     6.4291749117876907e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -8.8395808070502170e+01,     6.5856572287389627e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0390908372284768e+01,    -7.3964127351282787e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8752689793728976e+01,     6.8972746186522854e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4923963197426779e+00,    -8.7554938217715990e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6568618312788018e+01,    -5.1887086183529902e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5556780971028445e+01,     2.7408634441842129e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.5409417198152240e+00,     6.2449429323240781e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.0021242471251250e+00,     5.9213992788636597e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6661693266359279e+01,     1.7541640565901893e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.3708077504175591e+01,     2.2128861886252671e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5737819172461437e+01,     3.1045693678065676e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.2339202051444243e+01,     1.6154141194219745e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.0125614305038937e+00,     4.0054607580005868e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5638302166083328e+01,     1.6011802267435691e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.3671299038784568e+01,     7.9375422824881407e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4082078464875760e+00,     1.5496353624108648e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7424495286435207e+01,     1.4880670907312575e+04,    -3.2604717316358325e-05,     9.7905407943456567e-08,    -5.2816087233753963e-10, } , 
      {    1.2954740437439924e+01,     2.3243138129559702e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.5770660763307944e-02,    -5.2613587611747471e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1861007472418819e+01,     1.4585860085057122e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.5371145567956766e+01,     1.6582131788176186e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5547860042872173e+01,     7.1465487711102178e+03,    -1.5684229398419449e-04,    -1.1570573009633805e-07,     4.8025643250710614e-10, } , 
      {    2.3463502049520468e+01,     3.1082878747647403e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    7.6536988846801481e+00,     9.9783275644075238e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5962890640067892e+01,     3.3560363580633697e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.2811525205994192e+01,     1.6780181790316848e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1005807141039652e+02,     1.4914544756510091e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0364413107681628e+01,     8.3267425406401144e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5583629014893252e+00,     3.1639189233339554e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3248264857127960e+00,    -2.1359571140036364e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.1016205715659321e+01,     7.8420181344458133e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3702551887761585e+00,     1.6551444654824518e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.7114028077376293e+00,    -7.1505810971189499e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.2517641310328752e+01,     6.5860839126875744e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5747659062517139e+00,     1.7442270802817551e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.0573391429094450e+01,    -1.5005817549244966e+00,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.7547833732299523e+01,     3.1088138412279775e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.8614774382495325e+00,     7.8489294060030215e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7661307712397495e+01,     1.5480621140109348e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6233757917410511e+01,     1.3697646436743078e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9408387877331030e+00,     8.9543691773528080e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2903477566145604e+01,     1.4350573469704566e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6918670408595542e+01,     1.4956333138113461e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9626943475930076e+00,     1.3107076210811745e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7973422513800692e+00,     1.5542530632824084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5138117598836136e+01,     1.5542977947139501e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0327969610877133e+01,     2.3226217448752108e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3750811734777965e+01,     9.4739729062472525e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.5342284945336985e+01,     1.6867284630941274e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.1458774108979981e+01,     1.5638314263859585e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2659593476585968e+01,    -2.0554756825577624e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5689041042328640e+01,     1.6866579999025129e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.3038782439583327e+00,     2.4044946763937947e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -8.9378666061916334e+00,     7.2366047704905905e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.6063052678765892e+01,     2.8398848116711451e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0559496571900073e+01,     3.3350814114897395e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.2171332036287943e+01,    -4.2577556577234645e+00,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.8125798081470697e+02,     2.5800431987439879e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.6731486310436026e+01,     9.4884037941530441e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2829512591312636e+01,     1.5542202000449310e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0060429490810883e+00,     2.4420123495854965e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.6725789711228103e-01,    -1.7251927757336896e+04,    -3.3273757898272645e-04,    -5.3850185119286967e-07,     2.7002619115389707e-09, } , 
      {    1.3522766287325691e+01,     2.3418936178003423e+04,    -3.1300738819523585e-04,    -1.6346026553657220e-07,     6.4969396591807777e-10, } , 
      {    8.4495924862315253e+00,     1.5558753798205793e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.4920616905491777e+01,     8.7579245001794316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.8942737981218816e+00,     1.5543306579514279e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1046442921680342e+01,     5.9728668064069352e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7704750810300189e+00,     1.2714055241118758e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2531531990424909e+01,     8.9750232178707920e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5336932975284945e+01,     3.1125323481861502e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.3151864315426138e+01,    -1.4200187034113333e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.6661903352245355e+00,     1.8319536579254382e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6748791567165533e+00,     1.0826502653665946e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9679289961032067e+00,     7.6048238657001166e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5287098815841404e+01,     5.9597488952086660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6180032851872166e+01,     2.0977832594625892e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.0496893354600125e+00,     1.5555872201180062e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8187221736815410e+01,     3.0666154965317764e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2884476121717961e+01,     1.6972851557927763e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4650705121517809e+01,     1.6442766311468909e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.8927532632836119e+01,    -8.3288971398494687e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.3472513131293287e-01,     8.9033185669909253e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0212071966748056e+01,     8.3451998553129433e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.8701766725015297e+01,     7.1388118042876304e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4456833632771584e+00,     6.6736692665333858e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1276133238724707e+01,     8.4817508925842048e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.0001105831207603e+01,     3.1337285961853922e-01,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.7362567520512449e+01,     2.6398130993836276e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1587357082400615e+01,     1.5371902024443094e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.8230606382688041e+01,     3.1660847067797939e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.0805845053749685e+00,     7.2761960152307911e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.9956345773966156e+01,     8.2998410294258865e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.3943175367982036e+01,     1.3502398380446142e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7440088201626956e+01,     6.8574597167543725e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7095205096935846e+00,     1.2791607502130882e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6543314678294969e+00,    -9.4770770310126172e+03,    -4.9224375445982953e-04,    -6.5359118917504507e-07,     3.1260786157942875e-09, } , 
      {    5.7133650328512582e+00,     1.6062837941156900e+04,     2.7643205206657822e-04,     4.6437364542574921e-07,    -2.2436740829436326e-09, } , 
      {    1.7753474662387056e+01,     3.2443443652089056e+04,    -2.2501215182778352e-04,     1.4979579193449997e-08,    -2.9302138886260358e-10, } , 
      {    6.4418155397230059e+00,     7.2138392082006349e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0614530366755158e+01,    -3.1151595138976518e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4305986908915978e+01,     9.2129491412784446e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4601664553850012e+01,     6.6924048202020604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2088490342719936e+01,     2.2617131744455568e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.4739123437273491e+01,     8.9725410379574223e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2780884514803905e+01,     7.2142865225160531e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.2281399754246358e+00,     8.3306403086067858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8872310174531410e+00,     8.3493549408803447e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8014265138404816e+01,     1.3224479657836793e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3951797359563594e+01,    -1.4017302860135314e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.4594738857168032e+01,     8.2926087972268797e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.2775816116379819e+01,     4.2894492391677013e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.0869281270520332e+00,    -1.7422247988437725e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0051384491841583e+01,     1.0630045375960857e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8057658109376781e+01,     3.0459502194859462e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9716719497653735e+01,     8.2288324644977438e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.0037803989561072e+00,     4.0676050856576967e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4658744428443409e+01,     3.3312464498552392e+04,     7.6146203881163085e-04,     1.2535943707732736e-06,    -6.1799199931032542e-09, } , 
      {   -2.7946948259990336e+01,    -3.0668181873847061e+01,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.3628957917865518e+00,     4.5909101215648133e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5484495210570913e+01,     1.5526754781757798e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.6310061474269503e+01,    -6.1276555675711370e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8376223425372658e+00,     8.7251094751253349e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5436068405162374e+01,     9.6835223719835540e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.4513577960820434e+01,     9.3284745011424093e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0941094342017266e+01,     9.6474397445869836e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7919394212884967e+00,     1.6577200755918595e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3012940095821952e+01,     1.6661792076417158e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2588251795005583e+00,     1.5654087978449079e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7755664609783186e+00,     8.8930092857354375e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5408742023309649e+01,     7.1983303813590464e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0209138597080397e+01,     2.0295391982787942e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.7731960942351312e+00,     6.1778546422630143e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.1582279455516044e+01,     8.2462081313537328e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.5967305049475176e+00,     8.1916205865399133e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3965023554386562e+01,     2.3555977005924382e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.0930092000023734e+01,    -1.7425838283525741e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.2555870193281052e+01,     3.7497202623712656e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3832646892054932e+00,     6.6195423375994451e+03,    -2.4672627970781806e-04,    -2.2562162128739358e-07,     1.0569859394181992e-09, } , 
      {    3.3414393120673465e+00,    -1.9186985527603072e+03,     3.3578458807475305e-05,     3.5787685423935058e-08,    -1.2014167831575411e-10, } , 
      {   -2.7652727693874812e+01,     2.4080642864383473e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.3388725210113904e+01,     2.4081347496299619e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.9384059458066627e+00,     4.0698951078144446e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3072337059448714e+01,     2.4502054499592199e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4486515200993928e+00,    -2.8936131199793522e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.0392056035296065e+01,    -1.0822044288597499e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.5505958204282678e+01,    -4.5744219008338405e-01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.0143265655602923e+01,     4.3561886595680869e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.5318036284327738e+00,    -1.6638195179490867e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.5615519420740501e+01,     8.3023830339968918e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4923168419573884e+01,     7.6823348961220454e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2835912762140405e+00,    -1.6621334432843305e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -2.9467901703526088e+01,     7.0041610836639693e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.6310908576399022e+01,     1.8083511032283059e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.1602907691527051e+01,     7.0048657155801175e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.5810941732465441e+01,     1.6871999387024895e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.0975048309438407e+00,     1.0433481683972957e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9102819946218723e+00,     1.8033683940710034e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.6328882133293519e+01,     2.5597461178108300e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8634089642668709e+01,     3.1677767748605536e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2195095650440697e+01,     8.3325047164365860e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9450603074557222e+01,     7.0689477494358998e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.9404727105108046e+00,     8.3248781328103141e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1480125911551509e+01,     8.3549998152500084e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.2011367353350053e+01,    -1.2569674860491614e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5180562373099225e+00,     3.6768741381214577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6098473480907982e+00,     5.9044785980560664e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0810846203019604e+01,     7.5169457053149945e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.3523037281653332e+01,     6.5836777375291003e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5023977394075903e+01,     6.5932334468818581e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4159565247832111e+00,     1.2321034271425769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9316838900570037e+00,     8.3301141155572022e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0789583546773454e+00,     8.3272687336896961e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0934429177947056e+00,     7.2300623735823419e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0695866208197206e+01,     1.6100068569614348e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.3503120964991723e+01,     1.4071522582112717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4293252613598526e+01,     8.0181537482001822e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4245001624174888e+01,     2.4106732329957798e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.4108193518822745e+00,     1.6719515999119347e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4334322527008592e+01,     5.2670069603492684e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7046996280215147e+01,     8.1764316938756156e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4953745495739554e+01,     8.5378885744016752e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.6178967475522242e+01,     8.5385932063178279e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.8708614439665848e+01,     3.2042402784888265e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.1954106466511023e+01,     1.6676570519002933e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.2730893271235775e+01,     2.9881862507083697e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.7623433986122450e+00,    -1.5840288526770746e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0188222706480668e+02,     2.2756810441789414e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.5194795904650356e+00,     1.0040460714279969e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.5580811334585022e+01,     2.2756823868890868e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.7682278968956968e+00,     1.6737564942575205e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.9559410346953413e+01,     7.7003771260271797e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1534032840004224e+01,     1.4913735359469376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0288494616891234e+02,     6.5550291511499381e-01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.0160966323254570e+01,     8.8202813072095887e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8996641810906931e+00,     7.3253965538256298e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2142893230122173e+01,     8.3434099330491972e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2191173357472408e+01,     2.3855903686316815e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.7516088212042131e+01,     1.4102020842491856e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1294311848801435e+01,     8.5574285601157826e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.1153076315820092e+01,     1.4912369162152550e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4180759618760451e+01,     1.3922677020901123e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.9188519605069274e+00,     9.1768665138818706e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.1248866178248498e+01,    -4.1188014523893770e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.6644730998955310e+01,     3.3317066898435209e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.0713903736677416e+01,     1.8136166723492275e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5367293899406445e+00,     1.7461477223073198e+04,    -9.9859567294893075e-05,     2.6373670326886312e-08,    -2.8715029518436709e-10, } , 
      {    1.8154345535688297e+01,     1.1237572566814793e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.4346780376864992e+00,     3.6124397883076008e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7181670671774739e+01,     3.1611644456081067e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.8039891591719837e+00,    -1.2466257639892829e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.1596377980354536e+01,     2.0809761250331063e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8882685919395836e-01,    -5.3680442678067706e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9967282897322306e+01,     6.9370895205501974e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3966679124129708e+01,     1.6850011634175607e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    5.4765909425366246e+00,     1.5713606555520493e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.6874792414756161e+00,     8.9047412579246775e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1830152671988934e+00,     7.7184175235186167e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4051105411012403e+01,     2.4952292846415188e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    2.4310135927223726e+00,     2.2677187351544315e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.3285902241566589e+01,    -4.2937238373087894e-01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2655527060010030e+00,     1.6639750822972430e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7215895506509735e+01,     2.1577754246261178e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.3276331277167213e+00,     5.3717999261774185e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6201565427152993e+01,     1.5764922994136083e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0159601900643530e+01,    -2.4607899054994050e+01,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.6264645214667407e+01,     3.2775475627063046e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.9980601507602998e+01,     1.6029427580159432e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8690972802770212e+01,     3.0397872082413414e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4729509353263872e+01,     2.3294953240955569e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.0687434186094825e+01,     8.7151534387173033e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.3114736213413700e+01,     1.0723878096975795e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.3595841200235093e+00,     7.1824236761250049e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4217049747139376e+01,     2.5004098482072146e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    3.2329552037978369e+00,    -1.0796836702557132e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.3548737157828270e+01,     2.3886987742893674e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6726384240514856e+00,     7.2065846987892874e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3300718545802219e+00,     2.4802602769913661e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.0479807822720907e+01,     2.5934124335104807e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3641872255780411e+01,     7.1721143948695171e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9679517536412078e+01,     8.2380212524130602e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7704267776381680e+01,     1.0042761914221381e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8957088506481973e+01,     7.2150292677594080e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {   -1.3297576694953918e+01,     1.3187483199635335e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0989883771329383e+01,     5.4155706037919140e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.7803194391995207e+00,     1.9804827290144201e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2021699961581456e+01,    -3.2802776902897585e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.4344486213072276e+00,    -2.6906804280771610e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.1960151841049544e+00,     7.7996886892425294e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.4144973979996429e+00,    -1.3302114994441590e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2437771013437022e+01,     1.8419446729153555e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0993040493640480e-01,     6.0687813837345439e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3998019262608706e+01,     1.3030158359642337e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8677587111439074e+01,     4.0564020811097453e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0012993620496887e+01,     3.2160322237330784e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.1235770514095673e+01,     7.8374073944456359e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4289108524826286e+00,     1.7284979088825567e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3094518291338765e+01,    -3.1809920443429007e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8291056172207405e+01,     2.4815222008331650e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2171749857237238e+01,     1.8529187693185264e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6540443698749812e-02,     3.2838531684284699e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3365982164388939e+00,     2.3543230485485988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4681185633793149e+01,     8.3441212785160733e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1895062714945421e+00,     2.3203323227661789e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1828120054526920e+01,     6.5854266479380285e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6569533491768631e+01,     7.2215410319274015e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7381899672461113e+01,     1.5541787887580735e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -7.0157561097064391e-01,     1.3611430868972020e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -9.0794271294624451e+01,    -1.1147266108389445e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0530753380416314e+02,    -1.1145305076912668e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.2381604753500000e+00,    -3.6485224157041712e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7121934830616134e+01,     1.5618005351052507e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1155053732682685e+01,     7.9663233772891590e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.0957509388695549e+01,     1.2088048856014626e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2067647023254800e+01,     3.3558072211354171e+04,     1.4575583982070247e-04,     4.5429960498323794e-07,    -2.3569701920821197e-09, } , 
      {    1.5878320837712494e+01,     3.1747627602626872e+04,    -1.6071498043290967e-04,     8.7258608618082491e-08,    -5.8629003270257295e-10, } , 
      {   -2.2076682424783439e+01,    -4.8451400280746384e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0161966321735134e+00,     1.1928013301732783e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.8220028926785972e+01,     6.6012973086012298e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0581419000547978e+00,     6.0626633160081496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7774626405621028e+00,     8.4273068740268518e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4101000407484893e+01,     1.5479475911515116e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1074681190802508e+01,     5.7428311574892668e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3642864693508603e+00,    -9.2260670524767102e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.2564070594774762e+01,    -1.7315346548011075e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5285870682885818e+00,     1.5612644068000578e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0863083927804764e+01,     7.2457020545916839e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7410600306928732e+00,     7.6938248839787739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.9558074526847005e+00,     6.8341598068636240e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1179244247310231e+02,    -2.2697183058554060e-01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.8041399761239845e+00,     1.6793335985630129e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.6439341156327075e+01,    -2.3079134921367768e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2900142970290830e+01,    -1.1059920608822713e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.7889834201654722e+00,    -8.3095132306063297e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.2467220772017818e+01,     6.3658877760999239e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.2439696038872334e+00,     3.0454899794976634e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.9673664157985556e+01,     3.2239952041126609e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -8.2219622380380608e-02,    -2.5019855701325509e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    4.6738311518810978e+00,     7.0934061625102149e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2550596741092704e+01,     7.2147674972744944e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9246600733342706e+01,     1.7743145753799286e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0649738492098919e+00,    -5.3954940449017101e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.6040018856082213e+01,     1.4914348653362411e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3989968807714511e+01,     1.3437964030632289e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0816392345205887e+01,     3.4910678016420206e+01,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1383323698033912e+01,     1.4624436648370101e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7141304220119057e+01,     2.4979515318271213e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.8715273918697370e+01,     1.4505677991728813e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0461907023710062e+01,    -6.7903889733066608e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.8045786653288626e+01,     2.0597243910167081e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9841827471245836e+01,     8.3284371602392148e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.1989117191596165e+01,     8.3289456890076854e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1664408878242824e+01,     1.8199897325424927e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2457654412072301e+01,    -8.6572780445025783e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2665639703946230e+01,     1.4252393332429610e+04,    -3.0038768187033942e-05,     9.7332649060593772e-08,    -4.7299392356408544e-10, } , 
      {    1.1667061953685536e+01,     2.2064075220480736e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4427379502231112e+01,     1.0593517238254801e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1866143704143203e+01,     7.2560113358471717e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1416467074240106e+00,     8.0066637603434538e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7041778487744129e+01,     1.6013327520686908e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.6988732121269727e+00,     1.4983138810407832e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.4282199045316627e+01,     1.5774643824525454e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.1548267327145521e+00,     1.3812936056869414e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7083867927365610e+01,     3.1623644665579021e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    3.1277441051695543e+01,     7.1877544747317861e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6979261891835986e+01,     2.7801655411535117e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1813518706327763e+01,     5.5939855962776719e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1442692854280470e+01,     7.0022121996806327e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    6.5655636138868214e+00,     8.2764336728791350e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.3333297566311643e+01,     7.2133582334421944e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6885248031309128e+01,     8.8663288374586045e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6738024160896252e+00,     4.7783847861882959e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9672734092057560e+01,     3.2593158108921685e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.0341930746501387e+00,     2.5588644709299639e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    5.1473562351359092e+00,     9.4917567700018679e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9543708324722605e+01,     2.3245767961875885e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.4417238609024174e+00,     3.9609654580288402e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5857807202304199e+00,    -7.7341708968645498e+03,    -1.2404171194066790e-04,    -2.1369840450239435e-07,     1.0069628638013171e-09, } , 
      {    2.3725073791923094e+01,     8.7025675699930907e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7159255123570453e+01,     2.3028521591781711e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5360274998613523e+01,     1.5553063571237282e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9599647178282495e+00,     7.8070333794671660e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.8845076717406197e+00,     1.1087285051798765e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2870384748841461e+00,     8.3809491763677652e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3250324590329338e+01,     1.2543139701496766e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7630265282916393e+00,     4.1948470488827297e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8220443718625354e+01,     1.5702013897573444e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0599645596831778e+01,    -4.0765757282480035e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1168708944653151e+01,     2.1986750421287798e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6419269329272552e+01,     1.3013776206285171e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.4814098958764546e+01,     1.5560027286747463e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5593593571785309e+00,     5.4369929860101229e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6537444239922285e+00,     6.2860343151463885e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0607520315489171e+02,     1.6656874890607785e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.9665298495334449e+02,    -1.6657890807886015e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -6.3622982800673796e+01,     9.0123559398553352e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0719224419825165e+01,     2.4620285251772300e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.1858230735487783e+01,     1.5539909375924004e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7585697739095394e+01,     2.2826706933358924e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.7760027120877780e+01,     1.6786251548874614e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -9.8569764577212169e+00,     3.2194188186146130e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5088083713985650e+01,     3.8838861516752688e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -6.5869670848709760e+01,     1.4929988733224682e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6411680321879302e+01,     7.2625142656390908e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.3254483426247852e+00,     2.8908321987354811e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0945417876855416e+01,    -5.6072867053595473e+01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.8221443766337142e+00,     5.4842446322459627e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4619779853344292e+00,    -2.8406440227681610e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0493276810209977e+01,     8.2484923653491787e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7517914335450833e+00,     1.5549313245580928e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.3603969126348971e+01,     1.4987741210290653e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.9528563828377145e+01,     1.5859196926612640e-01,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.3749143487333876e+01,     8.1306166872287240e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.9794707992232061e+00,    -1.6383416767000654e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4814937521410172e+01,     1.5532445008726307e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0752712882464897e+00,     1.5545599204039585e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.3927791576374897e+00,     7.9548152792538094e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9708513041186526e+00,     7.3424577946792590e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1128018336700279e+01,     2.9671159055532069e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0706885447865629e+01,     2.4828339919529917e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.6350289707464278e+00,     5.3339001734552448e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3539121439054700e+00,     4.4629160775074315e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1184588818138232e+00,     8.9673082909244804e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.3436618622896844e+01,     9.1543425506813164e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1308631178772551e+02,     8.3266559076063750e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3622841198703860e+01,     1.8922208662878253e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0523367984273395e+02,     8.3307269416405252e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6413579367249049e+01,    -1.7578329779629530e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.9604221705269964e+01,     3.4960327176015744e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4128780182786768e+01,     8.4248296591087674e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9384083318744416e+01,    -7.7594669019345019e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.9671787695413485e-01,     1.1534992332039796e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4944045187791204e+01,    -8.2799895588088930e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    9.6518613004290366e+00,     7.9379304242816779e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0146684285830293e+01,     6.0768902253456770e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4198187991983232e+01,     4.6824702088606900e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.7687064852609140e+00,     1.7319441851585023e+04,     2.7110428908076121e-04,     4.6560642965407457e-07,    -2.3525535394472119e-09, } , 
      {    2.6345756494273324e+01,     1.0280678361890123e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8878187587927986e+00,     8.3260884006833676e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.3214030308265254e+00,     1.0371348534100511e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.1847071493390388e+01,     8.3283032828299147e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9699780019756865e+01,     8.3290795664169855e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3266629126982341e+01,    -1.4088749008389211e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.1414033145600261e+01,     1.6099716253656274e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5949760242595357e+01,     8.4287295649606040e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7186411757769179e+01,     2.2502385715664241e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.9861056119225807e+01,     7.0068145995634513e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.9663522540444191e+01,     1.6100420885572423e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.2055170364279457e+01,     7.2403712559849027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8733055532173654e+02,    -5.3779715276359639e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1492750679905903e+01,     3.2253106236439889e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2418326915706512e+01,     2.3400872483900133e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.8951443674162483e+01,    -8.3408973493474296e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.2927105612338693e+01,     6.2184714602360673e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -5.4240296809590568e+01,     8.3164854998994724e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -2.5967277581485419e+01,     1.3799465829713070e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.3387224985952884e+00,     2.3162956036231393e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.8203302477935281e+01,     7.7007124719526801e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1875341188705431e+01,     1.7117692896565186e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0086013788461461e+00,     9.5213168707809818e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9037880140493462e+00,     9.0525589835467836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.1706680315538847e+01,     8.9570057232197178e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.0472978237992677e+01,    -9.6902800561492768e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.9526678398020138e+01,     1.4428119017165964e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.5500778034395509e+01,     7.7713704314401703e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.7936351370307698e+01,     1.4428132444267416e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.1962256066017275e+01,     7.7713838585416252e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.8808382884113071e+00,    -1.6830883898579606e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    7.4546929586548352e+00,    -1.6883852995790799e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.3875107977923534e+01,     5.3689550121196271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0475975853422593e+01,     1.3330417852259176e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6953789133352203e+01,     1.5021096244825510e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2788908965122463e+01,     2.3791263621276936e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5183466405219943e+00,    -1.0012108991862686e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7792941460634850e+01,     1.8012213796607008e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7039118064810300e+01,     8.4073996569827523e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6388920643782676e+01,     1.9015508667404825e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7639029870140231e+01,     1.5561941959737826e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.8020440637872506e+00,    -5.2355723250848860e+02,    -2.1620064077522566e-04,    -2.5610772366429155e-07,     1.2058285876556377e-09, } , 
      {    1.4247588476295650e+00,     4.3853638164953081e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7295661894208266e+01,     3.1649381815195276e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.1572027174905362e+00,     7.3019768515789283e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9621152927347605e+01,     3.4596364662247634e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4235116266736615e+01,     3.2986179078614674e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.0202232712059764e+00,    -1.8888594947940874e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6048869604733586e+01,     9.0171499430110998e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.3233213431271871e+01,     1.6673115333246202e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -7.4942167018884795e+00,     9.5698874835748593e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5377576973150235e+00,    -1.5256832128988026e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.7475564230498719e+00,     2.4978112290424951e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.0027689169928912e+01,     9.8224650376247719e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7386215917361785e+01,     1.4921924871505309e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7283987930995234e+01,     2.4867037119727524e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -2.7978770911323256e+01,     9.7601082305946147e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5157987436251402e+01,     1.0522523932841810e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8313047624090686e+01,     8.5236376569217118e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.0673094984412700e+01,    -2.3888941138747350e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1973120838216706e+00,     4.6868894794509271e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3586789172551059e+00,    -1.6663941804846032e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.3942510523723890e+01,     2.3700593449066546e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2728590315328965e+01,    -4.8632097421956246e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.2420021466027006e+01,    -8.3406890353669223e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    7.5892121631209562e+01,     6.8246446917002750e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1457373137459234e+01,     8.4798668233647477e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.0336390156611472e+01,     1.9800945872208840e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3332511417024547e+01,     3.4886834129045695e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3016349481167483e+01,     2.3687974210648557e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -2.5305633767680032e+01,     3.2409334289006922e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    5.5752743898011573e+01,     3.2410038920923071e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.5756900824297297e+01,     5.6600279300422571e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7873931298633160e+01,     3.2147168042017503e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.5029734716174428e+00,    -1.3162864944184903e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1263243728794567e+01,     1.7430306877434607e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2931181713165071e+01,     1.4718508425746183e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.5600534482896713e+01,    -1.3292450970430964e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0368098443260685e+01,     1.8849227551366296e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1799556741880671e+01,     2.2779437248296172e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.2692223605544335e+01,     1.3517870021753406e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7272523886031472e+01,     2.2657490360469863e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.2898614416975235e+01,     7.6251327785071953e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7252883598685784e+00,     8.4557217023113171e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1755634924802045e+01,     8.2698222733270304e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.3727633673205503e+01,     2.3871347663031403e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1237421189932546e+02,     2.3871543766179086e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.1090650876834204e+01,     1.7268417437526776e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4040673041975939e+01,     8.3290460280732677e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2842577004166181e+01,     2.5143808628210780e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -9.9733739446031979e+00,     1.5110466097669896e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3917474613827039e+01,     2.2836446959135967e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.0452975183585750e+01,     2.1047902593495073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4632545923345976e+01,     2.5055904117729111e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.1829290484921174e+01,     1.6473349398426635e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2187217988427665e+00,     5.9452615759038281e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5949838686200646e+01,     5.9551464953258474e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6108246101762447e+01,     1.5606032668448473e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1318051545223703e+01,     1.5542743767777985e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7794348166809279e+01,     3.2197507306912514e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9043331695017319e+01,     3.1049460163559994e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.8751783729264414e+01,    -8.3283368211736342e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.4982873645746466e+01,     2.1723767189669343e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2617298367948845e+01,     1.5542764812185605e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5227330712155691e+00,     8.3997091103675375e+03,    -3.3189520425500942e-05,     3.1102494491060616e-08,    -2.0328237648922845e-10, } , 
      {    1.1196847218364116e+01,     7.2670224868306250e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2608509080377040e+01,     1.3294369435855580e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6472725342927355e+01,     3.1607877970586749e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.1279266102786405e+01,     2.8263978489216242e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.8012006809653300e+01,     8.4464051618399481e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8970586150083051e+01,     1.6345863510907418e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9125144390732736e+01,     1.6148519969895093e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.5322019562313560e+01,     8.3850147572108544e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8780451155401440e+01,     3.0328340690766154e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1846279157091367e+01,     3.2202766971544879e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7111522303955326e+01,     6.1518967384727603e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.8148042407497353e+00,     1.1141971362346809e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1850665758964323e+00,     8.4608286444530918e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5304813675072308e+01,     7.7375957175357371e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.9266131473300661e+00,     2.5770962227440999e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.4798268237451587e+01,    -1.8658474953799850e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.5081881875164305e+00,    -8.3636363136328437e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -4.2438228590781861e+00,     5.4999923757604138e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5806085178745446e+01,    -1.1167061020488063e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.7215254917652850e+01,     8.1296169151338891e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7181944870083509e+01,     2.3285797945515405e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1931130808543319e+01,     3.1121556996367188e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -7.1620438113610405e+01,     7.7003905531286346e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3772782775877189e+01,     1.6164502919428205e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    8.8540784907113022e+00,     1.5500094474026091e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5635991009026551e+00,     2.3345309838487770e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.1237695357607276e+00,     1.3218409899279031e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.5840394512965830e+01,     1.2562628541330114e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0000494492569857e+01,     2.6145728010314997e+00,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {   -2.9350980849685513e+00,     7.2014436269403577e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -3.6090551299672029e+00,    -8.6019711958920243e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.6214360376149486e+00,     6.3504686649602463e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1316361084107889e+01,     1.5543720692382856e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    3.3018950168415351e+01,     2.1202782220024052e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2904965578241363e+00,     1.4868046039063105e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.3348198805092487e+01,     1.3802119345612558e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.1445367547199758e+00,     1.9178194017119722e+01,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.0319403200894108e+01,    -1.1125510164814050e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4955019075121456e+01,     9.4094786222733692e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3321488165509813e+01,     9.5683859139069446e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5652659733546887e+01,     6.5688345995052096e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.5427845224222873e+01,     1.6960368056572211e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7470932061994080e+01,     8.1091372932701943e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.7588385970175811e+01,     2.5201657214049406e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.2701310119126767e+01,     2.4333768792286392e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0857650929759641e+00,     7.1948751956023143e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2770486008403692e+01,     2.5039043371081541e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -3.6196427380218935e+01,    -5.3321413169266265e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1978068361122460e+00,    -8.3345914787086767e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.8907424044338867e+00,     7.2054263669755110e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.4392938709227927e+01,     1.2254635749521094e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1468629902580005e+01,     7.6047826351099229e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1369186390701280e+00,     8.2654130461567711e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2315218172299131e+01,     1.5610268384229923e+04,    -1.2781646602579895e-04,    -1.0777699019273651e-08,    -6.0310821930026303e-11, } , 
      {    7.8369416152531635e+00,     2.4533564737268527e+04,     5.7760603453250148e-05,     2.7585976025321569e-07,    -1.4142548373014384e-09, } , 
      {    1.2504924679104388e+01,     4.1819249541729485e+04,     3.5968153748499395e-04,     7.7791390067668787e-07,    -3.9406625627945298e-09, } , 
      {    5.7447148623624225e+00,    -9.6191124025007939e+03,    -1.2127989808417527e-04,    -2.1435842984785683e-07,     1.0606753715314426e-09, } , 
      {    6.0018772120332251e+00,     7.1430751897768369e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    3.5365268314393838e+01,     1.3895419350139131e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9577932659972902e+01,     1.1630593536507782e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0168815614779607e+00,    -1.7429361443106488e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.0703564281998208e+00,     9.9629084532678462e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.3412528240505068e+01,     9.4827058930077612e-01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.9348317421507772e+01,     1.3879447949466970e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1215238999472422e+01,     6.6114924299290560e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -6.6955958178687158e-03,     7.6790539316000550e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5139467199412927e+01,    -8.3601716521372000e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.7410103019118424e+00,    -2.3919072456130853e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.6260099579557297e+00,     2.4016560830527687e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -2.0406872573620628e+01,     2.0403559095356504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.7539447651157154e+00,     2.4086062252383235e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.9124119442463634e+00,     2.5772116213256326e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.2701581006383066e+01,     1.7823327847127595e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8005707772001003e+00,     2.4983444441554166e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.3757611960398052e+01,     2.4988704106186531e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -4.6455093612960585e+00,     1.0355929422960833e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9121657200931214e+01,     1.0435782883914368e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3780823992799469e+01,     7.9219309160576804e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.6279320718378791e+00,     1.6753521083732219e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0586616678147086e+01,     1.0764369503793499e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4395219656261453e+01,     1.4111337484010630e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7387567024508988e+00,     9.2512453512428092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1145800600902952e+01,     1.6862001282725603e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.7082199535300781e+01,     1.6871863347240800e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.3706346871855757e+01,     2.3010152714148557e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0503840561936961e+01,    -4.6899338653426639e+01,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -1.6852146108188492e+01,     9.7999231324925277e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7517733322691171e+01,     1.5472864511963011e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.4407367158327794e+01,     9.5680335979488682e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5068135987974742e+01,     1.9315229632571238e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3889415684356965e+01,     6.7589087457929118e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1968317991995459e+01,     2.2872765975342213e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.5357803693594978e+01,     3.1478529549656578e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.9318996360839371e+00,     1.6910703148540168e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.2781195967313970e+00,    -1.3433562190185280e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4402097789894668e+01,    -7.7342270096798802e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9733995666168138e+01,     7.5733024075961721e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.1087627341440246e+01,    -1.5898544745338938e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0683015717498371e+01,     2.2149839478272450e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0618866033816334e+00,     1.3521751439688769e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6092324682881596e+01,    -5.5766659559062771e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1037565590248271e+01,     1.4679160089583695e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5173924280439255e+01,     9.8669938759402394e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.4577676989912094e+00,     5.7352789911129694e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9960463361912959e+00,     8.3919698030901291e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.3521038075294456e+00,     2.4200032334484371e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0705353411817223e+01,     2.4463704883247188e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -3.2337412557599059e+00,     1.0748950392653820e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5190538359799117e+01,     2.9987800701506079e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.4919111223729189e+01,    -5.5696196367447794e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.0519362480365722e+00,    -8.4929847345630169e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.5223488194711434e+00,     1.6029075264201358e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0720777833006844e+01,     2.4933105176659155e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.2899277431338669e+00,     8.4514903656934002e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.1008628199968697e+00,     1.6029786609668234e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6859884382990124e+01,     8.2595129920715426e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4614933713314809e+01,     1.5490496538237479e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3401532623061101e+00,     7.4559084363100337e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2694283497360567e+01,    -3.8040173127500066e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.0068872569582112e+01,    -1.6234914386169121e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.9866848097964080e-01,     1.5320585585827506e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9174356276215832e+01,     1.5814458726423365e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3053461422812710e+01,     2.5008616179002594e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.5531703702678019e+00,     8.2491309589696575e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3117858773050333e+01,     2.5128109645358527e+04,     8.5913366911324296e-05,     3.1292386313677598e-07,    -1.6425487515991077e-09, } , 
      {    9.9596803770731466e+00,     7.2815769596064711e+03,    -2.8010887378812518e-04,    -2.6149657317392838e-07,     1.1756731766906245e-09, } , 
      {   -3.2110658196736324e+01,    -1.5339514487004635e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3967821607404112e+00,    -6.6209464233218637e+02,     2.8250695821658283e-05,     3.7020469652260392e-08,    -2.2902113481933369e-10, } , 
      {    1.0419858840291017e+01,     7.3008039142857860e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3956431881389690e+01,     3.1021074230149734e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.0585027016698604e+01,    -1.3241847385521337e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.6878006553447548e+01,    -1.3241713114506806e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.1631496134641914e+01,     2.4225601679717811e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.8797427603826655e+00,    -1.4038078287972317e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.3703766631171433e+01,    -3.1151933833948351e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.6173656013631650e+00,     4.0526527363910733e+04,     5.4298645492547107e-04,     1.0649932191381403e-06,    -5.3519551885043885e-09, } , 
      {    2.3391485881517706e+00,     6.9323755841326420e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4587680403651504e+00,     1.5605111097011952e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.0746923445391204e+01,    -1.3791187798138708e+01,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.3720759401336444e+00,     6.9765162806545741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9509394884087463e-02,     3.9923428468023203e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.8530675481499372e+00,     1.2411960589514092e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.9865175747375496e+01,     6.1390404629093609e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.5709456223039183e+01,     1.6508430689493252e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.5398795083448920e+01,     7.1934443028473697e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6048185222688286e+01,     1.6735379736139326e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2879439533493908e+01,     8.5442743648025044e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.1368546901340913e+01,     8.5277659341130111e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1885187608381280e+01,    -6.3973680036136739e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6526551776446944e+01,     1.0037786207318912e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3984355624182143e+01,     8.3264139082653419e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4141276229467380e+01,     8.7240135942578490e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0721477559640384e+01,     8.9407896743282763e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9848459991371559e+01,     8.3309689409815564e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9086510982720315e+00,    -9.8363409535815426e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.6344751915179527e+00,     5.6757604140415560e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2273514813086841e+00,     7.0694464222161796e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.1184698596978706e+01,     9.7826501357268589e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4554097866450064e+00,    -9.1887879153173344e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.2496505732939241e+01,     2.5195295107231879e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.0124067836796844e+01,     5.6648780803271984e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.3024891410569843e+00,     7.7078620061469637e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8132223791802545e+01,     2.5195952371981428e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.0298826963672157e+01,     2.2927669420878839e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2018408361792119e-01,     5.4298795313413598e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9550673143831432e+00,     1.3797516945729738e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9831988794862895e+01,     1.7037285907741621e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2684569420274316e+00,     2.6046491411865509e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.3642195618507614e+01,     2.3141558306926117e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5454129276803524e+01,     1.5429853892622518e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.8446598676268067e+01,     3.3036737216031042e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6704733408095279e+01,     1.6974171095952959e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1442321850498548e+01,     1.4319907426677735e+04,    -9.1672058088999267e-05,     2.4437227521798605e-08,    -1.2528555147232634e-10, } , 
      {    2.0663890795088076e+01,     2.2094758153002018e+04,    -2.5117823356610229e-04,    -9.0652110460376752e-08,     3.0053115278298998e-10, } , 
      {   -1.0251093975582975e+01,    -1.1045917293183756e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.2840234450620205e+00,     6.2058522218180806e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6140586595395462e+01,     7.7000552072031342e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8136212588747895e+00,    -8.9907748072926734e+03,    -1.1871394895485087e-04,    -2.1493118873071965e-07,     1.1158423203048969e-09, } , 
      {    1.8210536007632093e+01,     6.5519794826891248e+03,    -1.8489712507868453e-04,    -1.5281346621119814e-07,     7.0782312628311120e-10, } , 
      {    1.0112248915544729e+01,     7.2346814278693191e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0983683066665989e+01,     1.5642081084852072e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.0328566632890990e+01,     8.3276277631984995e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8181450001084229e+01,     8.3297550860484007e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4686049304915429e+01,     4.0531129763793550e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.4684856754617826e+01,     2.8194676381228106e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5371715573157546e+01,     3.2016665635272009e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.3707208068079716e+01,     8.3263884896846503e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.8521957440471635e+00,     1.8422629391809260e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7678556886597843e+00,     8.2419408999570987e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.1784140579654085e+00,     8.8503494697797341e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7806066802772023e-02,     1.4348495926920865e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3295523840133328e+01,     1.6630152887183820e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2673566048648031e-01,     7.3591779812807890e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.1262968179460909e+01,     3.0456848678959974e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.1322337915436478e+01,     7.2604635312314922e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -3.2431429625251056e+01,    -9.4436722998466303e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.8885993808160094e+01,    -1.0754655621240090e+00,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.3219298264134736e+01,    -5.0205345821886112e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.1076665080721035e+01,     2.4968801277104682e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8515166638808309e+00,     7.1883237302102343e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8516534106830903e+01,    -9.4429676679304775e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.3860698040856949e+00,    -1.7959052415218403e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1872051805122967e+02,     7.2120273483412693e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4040487036928463e+01,     8.0530644262166006e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.6203602896645544e+00,     5.1069714060674260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6859829537715733e+01,     1.0076543341676543e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.4326089864697781e+01,     1.5631771803596585e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.8693957525499485e+01,     1.0120530652700609e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.9496449380242694e+01,     7.2160983823754195e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4497967617830199e-01,     1.1712955381796792e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3190224213548181e+00,     1.5595012041726110e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2627172401862218e+01,    -1.2445482212055826e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.1539600379051116e+01,     1.4485213629380269e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5622330315541454e+01,     1.7562663779100592e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5620528997684307e+01,     5.0754911785536779e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4702251841641148e+01,     1.3406797175978158e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6567594353495899e+01,     3.1243242934304017e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.9250291015773211e+00,     6.7500485902523669e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1227810363771571e+00,     6.1125522172915062e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.8149402222261024e+00,     1.3721821258778720e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2933810526795400e+00,    -1.0593996959557258e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -6.1052074972205389e+01,     2.1575502731181587e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.2380251434428075e+01,     1.2481069825707611e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3597836106726078e+01,     8.3375698131239933e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.8867286963909393e+00,    -8.3198130361229087e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.9310716745180834e+00,     4.0736076829647041e+04,     1.1038930864785152e-04,     5.5286503827215703e-07,    -2.9388435721497847e-09, } , 
      {    1.1654744945486023e+01,     1.6133825616738413e+04,     8.8384174749426707e-05,     2.4533002464501789e-07,    -1.2661394095856640e-09, } , 
      {    1.6440323454974937e+01,     6.4809561671135543e+03,    -2.0790783837658933e-06,     6.7419305566558618e-08,    -3.7931822410010095e-10, } , 
      {    4.0575293651264372e+00,    -8.4154419492898032e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.5876469895700495e+01,     1.0986538207947789e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6608070947568567e+00,    -4.0415433853971595e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.5170664651289947e+00,     8.3460042873583734e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7226430164769852e+01,     8.5502385011032238e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2009290868296411e+01,     5.7826460593871798e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4959140801093062e+01,     1.4592419040656254e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.0232830753891982e+01,     1.7166245728598708e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5713901032161942e+01,     1.5480397482951637e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2295344291848213e+01,     1.6363136507673085e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.8632602921811213e+01,    -6.4087775507394326e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3634716293238052e-01,     6.0273374732860430e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4138503807743948e+01,     1.8216348816820584e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0450015840243134e+01,     8.4706667757864652e+03,    -2.2380334687197685e-04,    -1.8736836740680787e-07,     7.1908534809528565e-10, } , 
      {    1.3818563928885156e+01,     1.6811820979550190e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9505223740381712e+01,     3.1102429260771187e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.2362514039700010e+01,     3.1574130754124559e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5412202198331261e+01,     1.3830985000325278e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6372439984045940e+01,     4.0740679229529851e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.5857547096871123e+01,     7.5879390107279842e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3891521992494107e+01,     1.4914452334767735e+04,    -6.3519294630925112e-05,     6.1501330405358849e-08,    -3.5357946576999568e-10, } , 
      {    2.8321892223527193e+01,     1.9053659013989090e+00,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7736846709918822e+01,     8.3236243525817608e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.0521661497802954e+00,     1.5243033324815382e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.3386152557678521e+00,     6.1156436414339551e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8545128393502392e+01,     7.7717057773656707e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.2083654645405396e+01,     7.7710485126161238e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1837310673981946e+01,     6.6007116134509142e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1347670966993704e+01,     1.3800170461629221e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.8543153529643011e+00,     8.3309943595622499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6670441197437928e+01,     1.4199398070963269e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.2548247261357243e+01,     1.6537938143553165e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0737003935476222e+01,    -8.7934194391255005e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4484762025962059e+00,     1.3095842673022778e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7572519930757831e+01,     7.8510069487867222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1416743034678891e+01,     6.1266642681563198e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4219250794301015e+00,    -1.6610982183373751e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.4833291309922769e+01,     9.8232230423832771e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9129793662490879e+00,    -8.6741048927441625e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.5461752054038502e+00,     1.2825388929586043e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.3755803448461794e+00,     1.5538345062811535e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.8167368943929034e+00,     8.1360027055347155e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3086874176295971e+01,     2.2130107022354796e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.7421949294889085e+00,     1.8476324289154661e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0796572139931286e+01,     3.1085508579963589e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.6074991878352513e+01,     6.6218036967164016e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.7178501035235303e+01,     6.0530488997352422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0375115147541855e+01,    -1.6884981258439064e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.6141640432161072e+01,     5.9236418874810024e+03,    -1.8746307420800890e-04,    -1.5224070732833534e-07,     6.5265617750965711e-10, } , 
      {    1.3778321178822079e+01,     2.4604552412850036e+04,    -1.3028727386390137e-04,     5.6816139472484406e-08,    -4.3672016394347005e-10, } , 
      {    1.0892283242195745e+01,     2.5790168647696650e+04,     5.2432840467433113e-05,     2.7709254448154105e-07,    -1.5231342938050182e-09, } , 
      {    2.3433344734803242e+01,     3.8628158065201060e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.3863497153834139e+01,     1.6848883371527339e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    7.6936607035483142e+00,     1.4444335066057407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7175263895563999e+00,    -1.2477367815905432e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.8791336890239350e+01,     1.6126638263813220e+00,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.8669639058646794e+00,     2.4097915861149140e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    4.1407569911324181e+01,     1.6776827554940639e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.5081304709775338e+01,     1.9997357440854557e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3159984628622752e+00,     8.3559138964794765e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.5756353498550197e+01,     8.9569922961182674e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9175328314527690e+01,     6.1565222047568195e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7665128798209100e+00,     9.4786570548252785e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3414012902038026e+00,     3.5993218771093334e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5543576301861486e+00,    -8.1271862367260564e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.5562738771580413e+00,     6.7020027559906721e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3542869810341176e+02,    -4.5036753191467938e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0312850332520654e+01,     7.9150003773558983e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5996008901056946e+01,     2.5214811409362686e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -3.7778850077669777e+00,     5.8930133454534016e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9413026425654078e+01,     3.0064180025225058e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.8884353397234783e+01,     2.5242024405479802e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -3.7075369749247415e+01,     8.3257092763491910e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6509151191380838e+01,     1.9708250602264226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0845872454954701e+01,     8.6349713453889963e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3957510945353908e+01,     9.4800797457590306e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5897576269246947e+01,     7.1315795720886272e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1211262343506078e+01,     8.3316735728977092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9750581712129378e+00,     1.6473911345290213e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.0648115103353080e+00,     8.3041082608225170e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6853991308264042e+01,     2.0890577753502093e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1152305814139236e+01,     2.2075540473083405e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3771350040137541e+01,     1.4455355692779769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3218101795457834e+01,     2.6709646945226045e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6022016777537007e+00,    -7.2129593379641401e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.2288704189025918e+01,     1.4434684686315824e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -4.6135935261657607e+01,     7.2136747235648090e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5360315326360663e+01,     7.2144510071518798e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1792103073790294e+01,     1.6688035771605599e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.0818773408632346e+01,     1.5536854235896566e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.9197477302411414e+01,     8.9566703772942201e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.0417838876610034e+01,     9.7791085680570086e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.2677703194862318e+00,     1.6639333905791038e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.7889444646428951e+00,     8.3532745884243832e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5012542100488702e+01,     2.9741976522500227e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0389580226798962e+01,    -1.8728558062511256e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4844720557367014e+01,     1.6703783515120049e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.9665681407385023e+01,     2.3977383909027627e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.3950156365586999e+00,     8.3323893063268333e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0539234797308595e+01,     2.1540016887146965e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1053836134015278e+01,     3.8526574359177503e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1824340631655215e+01,     1.7013985997850876e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1083754261618974e+00,    -8.5213801437121856e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.1727919307007175e+00,     4.7011163887884544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3473914064390758e+01,     8.3275631619751803e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0478486396382221e+00,    -9.5131498277473147e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8033411240616637e+01,     2.4465990622695241e+04,     1.1426199509656672e-04,     3.4990069955773649e-07,    -1.8722971069401059e-09, } , 
      {    2.5331877524130729e+01,     3.9380442957462976e+04,     5.0742700465641588e-05,     4.1140202996309535e-07,    -2.2258765727101013e-09, } , 
      {    1.4038881441573070e+01,     9.3964206452351282e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0801412247833525e+01,     1.5682473911859339e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.5245176898721134e+01,    -8.3575418198210136e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.2269315044566927e+01,     8.4023145623526252e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8066327427498536e+00,    -1.4481309518866788e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4148848437989818e+01,    -4.5424321845079305e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0842268890723135e+01,     9.2724677183498570e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1744293715640390e+00,     5.8493642362296316e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5386733604227931e+01,     2.1030801691837085e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0862708239954397e+01,     7.1894797015574113e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8465095548648186e+01,     2.2756488522965363e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.4441511915687769e+01,     2.1595803189717037e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3459713440711056e+01,     6.5233984732826448e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1480908821696563e+01,     7.8953395476150790e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.4847674297572873e+00,    -1.1359584672429576e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5278607043968181e+01,     1.5959185486182572e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2670703817184457e+01,     3.9413847688628957e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.1637414894837095e+00,     2.2757145787714911e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.3053719781981421e+00,     2.4165692056179061e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2917352885509564e+01,     2.5508443664493509e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.5201622384987843e+01,    -1.9973675839865518e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.8394379040181988e+01,     3.9414552320545117e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.6865897862229904e-01,     1.6951629190820713e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4152112725198606e+01,     7.4336169967116020e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3135112817372685e+01,     7.1980633571343469e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5842513184105952e+01,     2.3951627807933553e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9190688140440514e+01,     1.5550232456550852e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.2613628707593829e+00,     8.5301966125208455e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.1495938026221841e+00,    -2.6867124380370560e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2008056947491003e+01,     1.1157390473486486e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1326321876426679e+01,     8.3298196872717199e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7811254418732126e+01,     1.9408529637097814e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7625532211943352e+01,     3.1007956318951463e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    7.4156769502472102e+00,     7.2139144483011305e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.4595984325912255e+01,     9.1145276487834035e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7439183613985243e+00,     1.8209330312030741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.0266016618546194e+00,     4.7139504363744391e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8093152321703780e+01,     7.2142112824155583e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4780321408270201e+00,     8.2580903011377904e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.6945303066991244e+01,    -4.3787427251118061e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6061418796972742e+00,     8.3805065360193203e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.8770616806126693e+01,     8.3363180082497202e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9381718711901392e+00,     2.1070345155162312e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0059556524181410e+00,    -1.1326622432058150e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.6345784870497662e+00,     8.3210648409971800e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5576888924729190e+01,     7.8266379664045908e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.5776199113109630e-01,     5.9743683760748518e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.8245830035582898e+01,     1.5153336116323724e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7770234323115059e+01,    -2.3314288274582796e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5453222312919154e+01,     6.1170663323677081e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3366514141438486e+01,     8.7811710372278249e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.4285493629688464e+00,     1.6342170286443952e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0542182810766917e+01,     1.0828803853607355e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6678927188541746e+00,     8.2445802080121903e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4203731199942872e+01,     3.3588749514043964e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.2391007018874042e+01,     2.5182505828408386e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.7780542939757108e+01,     1.0673699331583108e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6842380161639014e+01,     1.5971987365537778e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7144160681384779e+00,     9.8754712728337029e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.0171254377114309e+02,    -4.9243187068664156e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1067239871959838e+01,     1.5535276123412737e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2635874359808792e+00,    -2.0584048529915126e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.2903700353965732e+01,     8.4065902599420351e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6021697069511749e+01,     1.4427414385249813e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7560097354568072e+01,     2.2738792947138343e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.5101241821365345e+00,     9.5639766867366852e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8906388307826489e+00,    -1.4300972679459696e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9124050459167591e+01,     2.4397581590722719e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.9417707423366704e+00,     2.3865412240162495e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    7.5931358198809864e+00,     7.1603824226802762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6251535727575284e+01,     1.6957103814413313e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3440113392531394e-01,     6.2688489411179046e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9296024851091948e+02,    -1.9610314767777481e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.9297946123913619e+00,     3.1569528354241738e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    3.7854721712563002e+00,    -9.6190880221696971e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.7093305527917678e+01,     2.4985745641495578e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3636813706033273e+01,     1.5516445899355236e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6947547732822770e+00,     2.5155808837708737e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.1233133801247375e+01,     1.8383398312749956e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.5383489260532105e+01,     6.4663105746190649e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7061888488599465e+01,     2.5235438641504563e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -5.8526568634438050e+01,     7.2138571501323258e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.9283804772270706e+01,     1.4428837076183565e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1217562783777202e+01,     6.6659373736411080e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2285596319980421e+01,    -8.4377239131493225e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.0977837325293087e+00,    -1.0405129961555673e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.4833411553166130e+01,     1.4173694291040785e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7809427452805757e+01,     1.4445757756991157e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0988097280768542e+01,     1.5314026630228374e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.3535318903552866e+01,     2.4986402906245126e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    2.0198071254721292e+01,     3.4258550387696032e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5514108046331195e+01,     3.2689737738604663e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7527323264285801e+00,     3.2063009074163456e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8266069414812129e+01,     1.6083147888806756e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.5713567239835740e+01,     1.4985087694391166e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.2655493338040841e+01,    -1.6641650365247599e+04,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    5.5334950707510700e+01,     1.4985792326307319e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.8428730977773103e+01,     7.2081628112731169e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.5146712961029198e+00,    -2.5229405167061814e+04,     6.6345003052078436e-06,    -2.0362436405988305e-07,     1.1202589729398046e-09, } , 
      {    7.8701739658612029e+00,     1.5976106166990166e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6005467055877656e+01,     2.2739896474532547e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    4.3680959732276525e+00,    -1.1729372931423541e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0250879621226671e+01,     4.9656780112321285e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.8363556351825050e+00,     8.3195026367081337e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1511665321206221e+01,     8.9393669833945241e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0112624920228308e+01,     7.2936233310121370e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.6050952400865057e+00,     1.5533894134441252e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.9057639397839203e+00,     8.2691728475520376e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.2992073936268138e+00,    -8.4082518902772426e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5574204535095781e+01,     1.8609369786513569e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3540553220428833e+01,     1.5551614445522340e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6569478278615142e+00,     1.2949341856471314e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5671849841902965e+01,     8.3474269782921256e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4617238993115144e+01,     8.7423824718910018e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2744938758227882e+01,     2.2747620358265522e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3632727356205221e+01,     2.4478423391672935e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.6424013946186975e+01,    -2.4021764761014856e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0568347929709931e+00,     5.2827394443485682e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1964494202252778e+01,     1.6520312011163362e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.2390851117068316e+00,     2.4435318949836928e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.3902444584619289e+00,     6.6490336587794800e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.1386835440987142e+01,     7.0149883558687834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8532280326362578e+01,     2.9306524019249340e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.7662675338141964e+00,     9.0345512706811169e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7711683491831668e+01,    -7.4775229073927994e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.5441947966892320e+00,     9.3204611086585792e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1538165392727663e+01,     1.4507755534512513e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.6042927213538958e+00,     1.0344294999908332e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0110366884024984e+01,     1.7308232339424689e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1422268995492892e+00,     1.6071735080157061e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3308682584376731e+01,    -6.0043470339206870e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0438377983586035e+01,     1.4443873474231139e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6770143262774351e+00,     2.0663516256893672e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    7.9775649857344382e+00,     1.5577699178991188e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.4622557766382567e+00,     8.6621938172450227e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1000813295133302e+01,     1.2023614506200771e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3035282012871431e+01,     1.4795001999242515e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4571526611532395e+00,     5.3931298341552701e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9472649471938137e+01,     7.2184720925286047e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.1325908815069431e+01,    -1.0932986512872537e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.1570195652337318e+00,     2.8132799377233582e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1104149325969047e+01,    -1.9304103504680916e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0723780676146687e+01,     1.5511115100748455e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0328107876295753e+00,     1.7285690434292443e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.3621080355646367e+00,     7.2386460291592775e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1144762424431010e+00,     8.4334361473290228e+03,    -6.6572114505808005e-05,    -4.7724573954741622e-09,    -8.4595139216802968e-11, } , 
      {    1.5997548491475621e+01,     1.5500805819492967e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5317509754561963e+01,    -1.6672801960386576e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.0893112116479539e+01,     3.2357871493569128e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5928129401987242e+01,     7.4771308034170816e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.8900681028822195e+00,    -1.3562537078568852e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2938030942402623e+00,     1.4246324217992797e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.0072639565782694e+01,     1.7856910120674405e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7851974655881456e+01,     7.2677433080364126e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2115894951372967e+01,     2.2930318204672843e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.7042657592367540e+01,     5.4685121482466657e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.9560867216066384e+00,     7.2162051035801660e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.0363386242682509e+01,    -7.1079087831525976e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2459058301559658e+00,     1.7652974254369179e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.8678442156420939e+01,     3.3507394483422511e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.0314939420950958e+01,     5.8538364239533385e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.5669206325416466e+01,     8.3020307180388172e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5726941828094978e+01,     9.6059701109714297e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6372090899827633e+01,     8.2911521813048821e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1147563357887194e+01,     6.9769044224481095e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.7890812769360835e+01,     9.4485888922551312e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.0323820759053525e+01,     7.6034810390164130e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.9685221419921657e+00,     1.2432367959893056e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1960463637121716e+01,     7.2052027098178014e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.0203278140259279e+01,     8.7036634508605766e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.1318193238323474e+01,     8.7804664053116758e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2867520480728949e+00,    -1.8231182376389570e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.9627885837060060e+00,     6.9012239889936654e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5088506278123148e+01,    -1.1301705875535354e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.6773606097851385e+01,     8.1758881025406117e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7606093322215308e+01,     1.6210876776925250e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1184811367476673e+01,     7.2229230208988874e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.9119946419356123e+02,     1.2646236674161443e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3522840928630000e+01,     8.3553521312080829e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5127617361575510e+01,     2.3403173683841545e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2163510002005765e+01,    -7.8712361051166035e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1781522880760319e+01,     5.5841780702675851e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5969199447138539e+01,     2.7401489332254982e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.4062667894513595e+01,     8.3667753580441240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4485530860590028e+01,     2.4482832618843197e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2506909247137585e+02,     7.2157938338355834e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3561791027827772e+01,     1.4994628798264559e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4221851109321360e+01,     1.5584702760470622e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1539162040323284e+01,    -2.4218859034589195e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.3960605010495311e+00,    -3.6986296496888826e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.0062711601451415e+00,     2.0272379983373830e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1952756138077238e+01,     1.4705368286767043e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.8131597552503891e+00,     2.3933802521635404e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.5352952854586985e+00,     6.4967377666980119e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3092940765542338e+01,     8.7927056997294276e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0912897125376247e+01,     1.7350124784106301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6067871885072080e+01,     3.0457553310876119e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -7.7089209096470439e+00,     8.3166938138799778e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.4468792303409236e+01,     3.1259009629296295e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.5891577439387987e+01,    -1.5786094646773104e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.8487215007066480e+00,     8.9495208430999355e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2712359775754001e+00,     2.2483848541262159e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0677835807035745e+00,    -9.1485147914645859e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -5.5816972816769646e+00,     2.4202589680303709e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4790685468152837e+01,     2.3866514682347650e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.6580581896321078e+01,     1.0915544902534797e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.6998537830801164e+00,    -8.3298548861776817e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    9.6608483277103133e+01,     6.6100281539222833e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4632606476072553e+01,    -4.9332085106337672e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.7047611531753599e+00,     1.5569062680608353e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6107161966060474e+01,     2.8587697350921095e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6250668647571473e+01,     2.2694460348309982e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9183873023956437e+01,     1.7179272518567603e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4864688007120584e+01,     1.3249062821637726e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.7535420413082008e+00,    -9.0502990141955652e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4268357308163955e+01,     7.2119206271365247e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.3197862533327447e+01,     7.6304940615591222e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1161533092975066e+01,     1.5403068879097222e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1910380376281040e+01,     8.3509828642218836e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.6620489691680454e+01,    -8.3063999850250166e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.0897775931128638e+01,     1.5015696679157600e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4128961374197978e+01,    -6.4635652850140468e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4574853749711512e+00,    -5.4812549171723549e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1588312137237956e+01,     2.3877479189047997e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.7947605373576778e+01,     2.0101271571957212e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1818276158890450e+01,     2.9864334032034676e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1705635894742187e+01,    -8.3137709862640550e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.6596417066778670e+01,     1.0260014845633228e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.6101971138154862e+00,     3.7590828774432339e+00,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -4.7774310912699836e-01,     7.3663225961061789e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6456554566372938e+00,    -9.4962890810997451e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.6366984870225988e+01,     8.2883510415691562e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.0745889718512259e+01,     1.5543315439902108e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5760139661112589e+01,     1.6560480048685407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4192759198261001e+01,     1.4929403038074366e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9704872783226854e+01,     1.7035359275841893e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -8.4413787110182223e+00,     4.3209294666814512e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5934191751373106e+01,     7.3507805678177492e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8281611532072517e+01,     2.4820787450431962e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1981828988300872e+01,     9.3136021582729645e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7535009744325265e+00,     1.6522659721002574e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2441116259179177e+00,    -9.3903508866773627e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.2002083308711260e+00,     1.5016618413864320e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6992685819448521e+01,     3.3297845017686210e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.1741382757283091e+01,     2.5101909935185358e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8080338709601612e+01,     1.7279131478693311e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2326554422442237e+01,     6.5967759070351649e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1727301282584456e+01,     3.9021179034894049e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.3618656468395475e+01,    -8.0807855708529069e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.6060338002113411e+01,     8.2102131213307766e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.8360266768492441e+01,     3.1068587899155995e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    3.3898795110985822e+00,     8.3176707979010862e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5814646644301476e+01,     1.6184845446450963e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3206291736861161e+01,     1.5574393479215134e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7803866796495416e+01,     1.5378460980042231e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.3202208951922678e+01,    -2.8821286713106360e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.2307474988946765e+01,     1.7228602535628863e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0021069162741817e+01,     7.4957501465840487e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.4609643442568077e+00,     8.3397120513458140e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7628557841649428e+00,     1.6673382357470899e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3825983212462054e+01,     1.2649141170002442e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.8845347577324194e+00,     1.6168432042711152e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1370020148605857e+01,    -1.1901823401533633e+00,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.3908589177238667e+01,     1.3119721002774286e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4767420966737067e+01,     1.6641635105732450e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5147599481235048e+01,     8.9573276420437651e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3053897792175722e+01,     2.8155811376647689e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.9419311131176169e+01,     6.8003718180907927e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7020701036998151e+01,     2.4535801716054390e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.1644377616621089e+00,     1.0477473286815075e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -5.7062681284544013e+01,     1.6460333437491563e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9365295864741805e+00,     1.7024666669323633e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.7663389841906856e+01,    -1.1085950848223529e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    5.8073078908401969e+00,     7.0727875999992402e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2122054285961124e+00,     7.7886900077258197e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4983932971654220e+01,     7.2129992039333938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1095249864117427e+01,     7.0355864611625475e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.0683928867474947e+01,     8.3260379087239653e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.4210912744806521e+01,     7.2151265267832960e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9455554427194350e+01,     8.4123039684544274e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -7.0571524212291848e+01,    -1.0990865309766759e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5370890121615002e+00,     8.1355030621449828e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4002246619576436e+01,     8.4699666899825552e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8932405620161944e+01,     8.3235027160050977e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5483506052357214e+01,     7.7905553390080167e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.6987848041540019e+01,     2.5653079059113494e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.8600882808225819e+01,     6.4460698694282310e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7768005775400741e+01,     8.8951792326561699e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0308035030162721e+01,     6.2174862944640950e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0955105683791139e+01,     1.4288768952206890e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0562773158821956e+00,     8.5912005118320321e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2245816208445337e+00,     5.8062722965259600e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7319091948139189e+01,     1.1379559177640776e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.4790659206518093e+01,     2.2743409287946997e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0659726032425141e+00,     8.2925830740598267e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8254835926851491e+00,     2.5574566957017978e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.8323316251069585e+01,     2.3068336493679621e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.1284113029088392e+00,    -8.7913986220583752e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.6208117127086261e+00,    -7.7550024375168846e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.4009489314695173e+01,     7.7880893386896787e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2669225006403472e+01,     8.2152801933724659e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.9024294867793898e+01,    -1.8239399166730436e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2129701756917099e+01,     9.1081744084102920e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.7030894337854583e+01,    -8.3296730865396694e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.3434742539228814e+01,     2.3876376746862839e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5963524666692701e+01,     7.5295315740392089e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0859594833361740e+01,     8.7523653166751355e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.3036934806049882e+01,     1.7162831559062466e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7928285410449636e+00,    -1.6589868754998774e+04,    -3.6621810542661771e-04,    -5.7433316984810465e-07,     2.8196763693330605e-09, } , 
      {   -8.4630862119968349e+00,    -3.3281008651369732e+04,    -6.3998627600028734e-04,    -1.0393232073880164e-06,     5.1177901805284827e-09, } , 
      {    1.9440578177628151e+01,     2.2162272247250137e+04,    -3.1281152346806753e-04,    -1.6354753199917192e-07,     6.4823952487474922e-10, } , 
      {    2.3876574372315293e+01,     3.9589992423199277e+04,    -3.8185444581197798e-04,    -1.0072615090288805e-07,     1.8723504364450256e-10, } , 
      {    9.4801082216977939e+00,     2.5371416158218886e+04,    -3.7750042431727794e-04,    -2.3565202849860501e-07,     9.4441705080137570e-10, } , 
      {    1.3322285559215135e+01,     7.8086077734483415e+03,    -1.9032282042808564e-04,    -1.5153704875157295e-07,     5.9967089030119600e-10, } , 
      {   -1.4421436248440028e+01,     2.3752001008911509e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3411838305661512e+01,     1.5531264302125068e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.5745841262256821e+01,     8.4246060019510587e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.3339140939763674e+01,     2.3970772509475522e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.4075368428451839e+01,     3.0536830798713869e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.7347277572661802e+01,     1.5554244277838521e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.3247909645061711e-01,     7.4624673919091674e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5341460454562927e+00,     1.6047108948142068e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4534443102805927e+01,     1.6906747216881115e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -6.2049370254329084e+00,    -7.0190391439861583e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.0271079912347403e+02,     7.2123318968811072e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.5558525760820737e+01,     6.8176448148564605e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3827863593217799e+01,     2.3696644078725163e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2456303253981131e+00,    -8.2038221769129018e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.4574051720251902e+01,     1.3102651683111617e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.8672411022262061e+00,    -1.1206620337078566e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    7.0216288885141751e+01,     7.9392732509653806e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0530283735151215e+01,     1.5589154955854945e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    8.9259822132188038e+00,    -1.6670500760445168e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    8.1051957760245141e+00,     6.1151820496076889e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9285030282372873e+01,     8.3436477577615642e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6781569258726890e+01,     1.7522848877202676e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.1491718483783853e+00,     8.3137350914853359e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6839075475122193e+01,     1.4224005970018261e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3040107371917163e+01,     6.1480107377418915e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7875889044096755e+00,     8.2970522353901106e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6634045435108291e+01,     2.3636159099252691e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.3803039430993195e+01,     1.2874090795400600e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2883891646394883e+00,    -8.3275878972292430e+03,    -7.0844439523505521e-04,    -9.0969891283933662e-07,     4.3319072034499252e-09, } , 
      {   -4.4522288857713562e+01,     8.3053849534150304e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.1781358730446733e+01,    -4.4747247472271260e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9572403760891326e+01,     2.9906445670884630e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.2168038479221988e+00,     7.1158470880893274e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.2319787193157289e+01,     8.3277097627072326e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2521165467164799e+01,     2.6136751249539051e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.5013429207495719e+01,     1.6657376135696177e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.2449595722055676e+01,     1.6657389562797624e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.3982330334614659e+01,     2.4921271914616227e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.9407092432928987e+00,     8.6415303009881281e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1906279056296960e+01,     1.5030694180614122e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.0832662382269103e+00,    -4.4345643550901468e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5513620274011275e+01,     2.4499753299650787e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3006597062001637e+01,    -3.7737359114211813e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7580845611418141e+01,     1.3777276240538902e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7380155635277280e+01,    -8.4421026558744361e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -5.4655080647288994e+00,     1.7540121873968346e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.4483799872517558e+01,     1.0116711379198587e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0639157134114487e+00,    -2.2647014654389488e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.8082300527821523e+01,     1.7934661535594882e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1807944279426398e+01,     2.2129220915761474e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.2360881087988425e+00,     8.2550950952703770e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.0523165585346916e+00,     2.5196335085073526e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.2785390619103158e+01,     1.6797068260131469e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.3112034599378859e+00,     2.5194912394139774e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -3.2407507570860346e+01,     1.0054354572168431e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.2696917553173233e+00,    -8.2975130211083724e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    5.0642302337189857e+00,     8.3603306138567896e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4635261391451426e+01,     3.0639856642155919e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9130024802664593e+01,     9.7202933286967036e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2497895825727628e+01,     2.2287416861928174e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.7008101459475309e+01,     1.9002390756206558e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4450477708914988e+00,     2.6087903140599633e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0044351656572466e+01,     6.8390908391212179e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9384348907707460e+00,    -7.7353287285872993e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.6149797941280589e+01,     7.7153401727160199e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5767323310864420e+01,     7.2025728775016178e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3432055529510695e+01,     1.1550411443179475e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1574245398776890e+00,    -2.2041253444728142e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0115668308230244e+01,    -1.7002763283549452e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2722356283225778e+01,     9.0394823029387098e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7596028839877285e+01,     2.5477311020832356e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.6049835671163933e+01,    -1.1123273593236936e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2303025749922419e+01,     1.7068437502880599e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.4991672194488523e+01,     7.2255528532150729e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8211795734787266e+00,     2.3862809216222413e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.9871180877242974e+01,     2.3801555936586461e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.6731160331301464e+01,     2.3960463228220033e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.2486557831095876e+01,     4.2456784056107390e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4819952728065701e+01,     8.3313449405229348e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6996718578657548e+00,    -1.2285887522997609e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.6574087798277191e+00,    -1.6659525087468719e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.4079613686895586e+01,     1.6655240611025081e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.0495146757027958e+00,     4.8042092013433319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6285492567229664e+01,     1.0216244012076262e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8282536590304829e+01,     1.0469564311369530e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.4509353364310439e+01,     2.6060316719284674e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.0831569561390133e+01,    -6.4449609329010855e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.8055455047214842e+01,     1.3737461338640989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8328746139947210e+01,     1.3821148053648998e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.9471267702606418e+00,    -8.4022877539765232e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.8005496943841441e+00,     3.3326578729832972e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2867909963859843e+00,     7.2206742649104490e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3309425607229372e+01,     1.0021837304151832e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6293886133046964e+00,     1.6724896943495027e+04,     2.4295152562268702e-04,     4.2854232677051423e-07,    -2.1242596251495428e-09, } , 
      {    1.0381349318432957e+01,     3.3348522745617869e+04,     5.7835298609832211e-04,     9.6642778584922133e-07,    -4.7700818084367239e-09, } , 
      {    1.4191219242333574e+01,     4.2028799007465786e+04,    -7.2915608792625580e-05,     2.6578571981070443e-07,    -1.5275509464399260e-09, } , 
      {    3.7631597356588999e+00,     8.6687527728441564e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.3137419115164297e+00,    -7.1504863397298482e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.9765343582882782e+01,     1.5540612051759974e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.9980908491742024e+00,     1.0217170892806848e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.4915413194761300e+01,    -1.9528379260050256e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.2501751847723682e+01,     2.3539707325905238e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9215647160185942e+01,     1.8405234766299989e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1289935541434653e+02,    -8.3304223931006873e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.6849332466788759e+01,     8.3267270769539718e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0453213235665579e+01,     1.5544896528203617e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.3908310991889312e+00,     1.2039346990200069e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.7718442202615332e+00,     1.6759554558174968e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3468444937147801e+01,     3.1163060840975715e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.7022650060114614e+01,     7.7024613823616073e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6374172905711767e+01,    -6.2227411060279132e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.9646821734010107e+00,     8.2867429541346228e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7866613077010047e+01,     1.6934356194055050e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3619089515893341e+00,     2.8920433022623548e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6154797858464171e+01,     2.3889494658061107e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0776149583456296e+02,     1.6866424356344087e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    2.0124987486093451e+02,    -1.6867440273622316e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.8658217544100275e+01,     2.2128509570294598e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.0766948015727280e+01,     3.0978836986224542e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5371436987951007e+01,     7.7901126986595727e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.8614521304442082e+00,     3.9279084969884639e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7052353358515369e+00,     1.7243030618336739e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.1963557733320251e+01,     1.1221824823300343e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3888995207821537e+01,     3.1149942929777444e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -4.4796278612875344e+00,     4.8897184746555813e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -7.0440929670175223e+01,     8.1521588202661333e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0111013321348569e+01,    -1.9575526820886398e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.9305393852799391e+00,     2.2065001222074284e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.4169381825750612e+01,     6.6428431789763818e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0207535487687704e+01,    -2.3285839134004882e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.0744692768709001e-02,     8.0907749769547099e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5843834798734505e+01,     2.1988824159410025e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.5637073221580380e+01,     1.0076896477300674e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9155391168068693e+01,     1.5507809400972403e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    6.1716932722950002e+00,     8.3706398951122774e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4149935371972990e+00,     6.6909821292683073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.9069139337022797e+01,     8.3271661713722315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6922045658945095e+01,     8.3302166778746669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7562243852610404e+00,     8.5375295448928755e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.8464881475527651e+01,     5.8615331179396508e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6204734858670122e+01,     2.0643622015498840e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0727031367313550e+01,     1.4977725083402931e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2622163462144570e+01,     1.6864790076761383e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.8601258229256555e+01,     4.0564876980255736e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.5909815646362883e+01,     7.7544564641833049e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    7.9852482969684829e+01,    -1.2756073834667632e+00,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.7012827233197854e+01,     1.5129063242714243e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0098954888122854e+01,     7.9866117780396235e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1236044654747808e+01,    -2.5103435886700686e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.6900622482097944e+01,     1.2625412682668139e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.2252531089799973e+01,    -7.9587275351851258e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2414723728374227e+01,     1.7861028922126792e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.6411233038742363e+00,     7.6654389505685149e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4730535330549444e+00,     8.5389522358266277e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.4729949227729010e+01,    -5.6113260642276982e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.2063167254148084e-01,    -2.7849282153060731e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1363145544403508e+01,     6.5933818639390720e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5462137040690225e+00,    -1.8411519215796659e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0638110272481107e+01,     1.4821146700409678e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3154069295563851e+00,     1.4931367385743844e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7303284738645107e+01,     2.5554863281844482e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.2995158808389664e+02,     8.3282410570915345e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4408747514543961e+02,     8.3291417921553657e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8696476247596010e+01,     2.4721922003805077e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1448405018471121e+01,     8.4269072018924671e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.4033492560814949e+01,     7.2142685805843648e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8638070268250871e+01,     1.0082106435402638e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8546591317591403e+00,     1.4475025069370116e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2992699903727640e+01,     4.0704596602133286e+04,    -1.0988521799907885e-05,     3.3855024165560006e-07,    -1.8774409800966779e-09, } , 
      {    1.2108667530356909e+02,     1.5540718772964719e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8325889636677026e+01,     9.2593498071515914e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6456856061633838e+00,     6.1992932662189469e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.9226949409578777e+00,     7.8166313152557777e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1977296477887123e+00,    -1.0697465853255039e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.1149140488993829e+00,    -5.8453675148857928e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.9312512811801601e+00,     7.4121376027530705e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6456554816330273e+00,    -2.8535836886971424e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.3586396113626105e+00,     8.4984259884618386e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1414213317753530e+01,    -2.5943543091764951e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -4.7717536324368979e+01,     1.8718275576793530e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.7151242675011645e+01,     1.5544789806998868e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9667787625475697e-01,     5.4132513268329731e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0746050207829786e+01,     1.7602478680998502e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0920462952950942e+01,     3.0457200994918046e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -9.9485660226696329e+00,     7.1874021587737116e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6973319910646367e+00,    -1.1948106525934136e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9383789959167498e+01,     2.0494292541650200e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0995590717719468e+00,     1.6029080894032839e+04,     3.0724869701756089e-04,     5.0082135619514676e-07,    -2.4175282689895120e-09, } , 
      {    1.8585846930350264e+01,     2.2756105809873265e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.1976746486169048e+01,     7.5567606072129065e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.2383850222526547e+01,     7.0167193243460215e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.6637090832803203e+00,    -7.2117958764099076e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.9662531883831235e+01,     7.0717239385742896e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2376392112315848e-01,     7.8056106885334129e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3019447439869985e+01,     3.0423449577625466e+04,    -9.8885825803776082e-05,     1.6006676369427795e-07,    -9.3545284583766085e-10, } , 
      {    2.1521019821928491e+01,     3.1261300998575825e+04,    -5.3414685357430422e-04,    -3.5144502505754275e-07,     1.4232190422651533e-09, } , 
      {    3.0015222482207262e+00,     8.2750109819453828e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4727119463105403e+01,     1.6207110291430936e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5746014320393584e+01,     1.8366477631942365e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5341321092541520e+00,     1.0147695991393971e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2737539839884940e+01,    -8.5513588015580190e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.5743643225470429e+01,     9.3862320852249759e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.6088706926099675e+01,     8.7181095982815204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2517493834229860e+01,     3.3379200048307663e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -4.8621094998334282e+00,     1.6827117413085289e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.5360869475201802e+01,    -4.7550426296270598e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2394566001692322e+00,    -6.1028337684374424e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.6969217093064959e+00,     8.4646445610066803e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.5820405969413727e+01,     1.6482581213366822e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.3067087051466446e+00,     4.6206504318478637e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6073287614180547e+00,    -5.8190691917239565e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1112895647616179e+01,     8.3823718673015173e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2346779676764944e+01,     1.5229915413617116e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7084177287270128e+00,     1.5377880549482597e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5224176971860292e+01,     1.5585414105937498e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4314710161292737e+01,     2.5536829597903768e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.0444240435996988e+00,     7.0720762545323632e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8604528468710186e+01,     1.3817091142436813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0810942989904945e+01,     7.7357209105146285e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7702555742303819e+01,     2.2757528500807017e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.8611332873983502e+00,    -9.9656893745117140e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -4.1233337449670543e+01,     1.7817095218776492e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9707615258938567e+01,     2.8750997147361822e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4606432025263690e+01,     1.8764474308537818e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5828051403191168e+00,     7.9291209750702401e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.9667628500619855e+01,     1.0965796158092435e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4765700944007801e+01,    -9.0001202148711468e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6634488824765519e+01,     1.7500306972070437e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.1365413300230891e-01,     6.5393546144396551e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.7030155865277990e+01,     3.0996491066348804e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.0381383306005869e+00,    -2.5278240978813024e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    7.4965140077208670e+00,     1.5573932693496874e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    9.6519696358695661e+00,     2.3240508297243516e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.9131321730357431e+00,     1.1403676944132160e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1615404901180209e+01,     3.8188760084625919e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3681540573216795e+01,    -1.0785801428615077e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    9.1110057356685132e+00,     2.3011248595016041e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.6985909175617611e+00,     5.9176327933693456e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5399518329322703e+01,    -4.1106991314885090e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9003566955448568e+00,     7.5944456451555243e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5380263537022856e+01,     6.5943917786956354e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.9702240001493321e+00,    -2.5032409340803697e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.6553175385134384e+00,    -1.7079912553012564e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3183005719321464e+01,     1.8371453338844833e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.6172372403646719e+01,     1.5542548574755774e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.3242817157158928e+00,     1.2416588369529729e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5618081413418574e+00,     8.4852717931485604e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    7.4623591124949229e+00,     7.7783807264703328e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0649158344178353e+00,    -3.6465463183694737e+00,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4523836663650869e+01,     1.8151156462248218e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0915144355233011e+00,     9.9487686937041217e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9789127568233713e+01,    -1.1363488840869468e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.5222263892830661e+00,     7.5269017417230243e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0508838564423817e+01,    -2.1593673405114264e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.0735249194382227e+01,     4.7508145189664629e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.9089340296157999e+00,     7.1779802379617740e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -2.4806053714921347e+01,    -1.4119820172913023e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6853588761639822e+00,    -9.4426086384216778e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    9.9010205614128722e+01,     1.4938719578545733e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6059256025214239e+00,     5.9600775275834421e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6015609950574305e+01,     2.2462570813766324e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3506717410969467e+00,     1.4921005660535386e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.2423741558473019e+01,     1.2416635475893758e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1168889076893906e+00,    -9.4440313293554300e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.0336323557732918e+01,     1.5854273628321278e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.3953789152526660e+01,     8.3116241430838018e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8402048226514189e+00,    -1.6802497965169347e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.5988637762819295e+01,     7.8821853523017999e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1519686395359372e+00,     6.5514653920801820e+00,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.7914734611852667e-01,     6.4430709326954047e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4916730976927370e+00,     6.9661570115878012e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.9214043826735768e+01,     1.1066720301276095e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5877170360812656e+01,    -3.4431910387842800e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7566626257462680e+01,     1.6157943963829071e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6029981075009701e+01,     1.9157495278230806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6637616640760825e+01,     3.3700083202511240e+04,    -2.2511008419136763e-04,     1.5023212424749856e-08,    -2.9229416834093930e-10, } , 
      {    1.6166945202263296e+01,     2.2198399348313378e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.9818319948049705e+01,     8.3457587061630966e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9233576946897493e+01,     1.9801550606790799e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.8096582486963108e+01,     2.6084021722664274e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9177424767664533e+01,     7.2407235719429773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.0671255044825813e+01,     8.3250886285885317e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0748333405685804e+01,    -6.8679412343187851e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.8524065289513416e+01,     8.3322942206583684e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7616875490822821e+00,    -2.3117251522847773e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.3785823717631956e+01,     1.5542366148188261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6979295649751936e+01,     1.6832184485126978e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4676745488788033e+01,     1.4925467331658616e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1282600583495826e+01,     3.5348875272954765e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6369182332743428e+01,     9.6078484998467793e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0197121070060064e+01,     2.0452869417932452e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7721291723347406e+01,     1.5543142431775332e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5384024188992912e+01,    -1.0031812667036185e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4820728885119895e+01,     6.9707563524979814e+03,     2.4493820734244241e-04,     3.5997474000024776e-07,    -1.7590009978016184e-09, } , 
      {    2.4108556354557468e+01,     3.9447957051711099e+04,    -1.0890589436323744e-05,     3.3850660842430015e-07,    -1.8781682006183425e-09, } , 
      {    8.2733861629098104e+00,     7.0720274938701696e+03,     1.5248827248949445e-04,     2.5063160769205500e-07,    -1.2374384396639795e-09, } , 
      {    5.6564374544898612e+00,     8.6622403271428266e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2545103573608582e+01,     2.5189064784007514e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.2525718686628720e+00,     5.3267867187864817e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2822239159515356e+00,     4.9073020138982088e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8993591180983946e+00,     1.2666061850810034e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8391823152249362e+01,     1.5697826788305383e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7758105991409600e+01,     8.2396491757545991e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.6641650433502662e+00,     5.9666839581220358e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7430834876075085e+00,     9.3880058682393815e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8073609264635255e+00,    -8.3985812026422318e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.8900272834213958e+01,     8.3245726231710596e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9229570498245412e+00,     2.3858826476187256e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -3.9032649370510178e+01,     1.5343679780492235e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1443167980465716e+01,     7.8887711162770356e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.7773293571674103e+01,     1.6657741878755704e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.6620293185850741e+01,     1.6657023819738097e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.8751353047344480e+01,     3.2187019228030425e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.6390532863135164e+01,     1.5542960005207817e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.7529504708894045e+01,     2.8980718320614083e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.8559307682326107e+01,     2.4093614418759535e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8178520185132605e+01,     2.4340846008017212e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.5017271858060717e+00,     7.8397406180877033e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.8132789154798079e+00,     1.1646326020507080e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.8209915680553166e+01,     1.4681461289525107e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0795385897881232e+01,     7.7246567131873890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.8617038568403128e+00,     4.2276001678976718e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2533520700852279e+01,     1.4808508510513870e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4050583736344249e+01,     1.3834763034680538e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.0638841335281022e+00,     2.2754739612556437e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3531365805194184e-01,    -9.4124450730287208e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7217067571586373e-01,    -3.6784267092688810e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.9446796352301519e+00,     8.5666078722921902e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4652504363155270e+00,     8.9312618600527403e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.0371472715420769e+00,    -2.4565781629992586e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.0421735129106917e+00,     8.0158525482587711e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3485034397190155e+01,     1.7265479269983007e+02,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.1072525277523280e+00,     2.5037889385266219e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.2202906900102942e+01,     2.4380308593957052e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.3082190004329291e+01,     2.2758894698123844e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.7838812655849345e+00,     3.1005878776167763e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.2688249406672981e+01,    -8.3290102282749540e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1559481191077715e+01,     8.6126799057905664e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1122935913776384e+02,     7.2137172629509951e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3045370988720646e+02,     7.2144084677656929e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.4833417751060878e+01,     7.8977273805902296e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1945050466099005e+01,     4.3526654999873381e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.6715919739084031e+00,     6.2516940368410733e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0210918162171915e+01,     6.9875927188144497e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.3254971964438017e+01,    -1.1315492400726989e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    2.8440845644668656e+01,     1.9395411725899543e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2459582858590508e+01,     8.3263035916483004e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1797252171996476e+01,     8.3282269628361428e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.4607027181861909e+01,     8.3310792575986015e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8348445890832792e+01,     2.2542200617562150e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.2477529092347872e+01,    -8.4177336734923010e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.0908144609014460e+01,     8.2330902201554654e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7977148255194351e+01,     8.3283726209719462e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8573416785559669e+00,    -8.4995436901621470e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.1324831571262539e+00,     6.6103384441137341e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7330013262841193e+00,     7.3875639146910471e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.8179718978030426e+00,     5.5471125569247056e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.9930467140341728e+00,     1.7389301705606362e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.7086028932180625e+01,    -8.3291558864107574e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.4566605579364316e+01,     9.0136659885835670e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0958663218761888e+00,    -1.9187041825917900e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.1538155452330315e+01,     1.0801451741706842e+01,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.1606298692550769e+00,     2.9496951011518725e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.9048791709157427e+01,     9.4087739903572183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3179657299642802e+01,     8.3347517074423049e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -2.9835464763863879e+01,    -6.1039695507357692e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1746135816712503e+01,     2.3923260826001115e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -1.9816303630783622e+01,     1.1767617091375523e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9399614363956324e+01,     8.3288409106041308e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.4688505263182712e+01,    -8.3285419386427693e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -7.9867236732141471e+00,     9.0193558166991497e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3324637677854733e+01,     2.3165585868547580e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1107528284759828e+01,     2.2769935066538404e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1681692155437924e+01,     2.3558625789718386e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    9.6680944774187907e+00,     1.0577076278749664e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2556003345245692e+00,     2.5353358093947081e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.1733336931306329e+01,     1.7978432369151844e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.0855813123694800e+00,     1.4580385461464524e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0808154973692399e+00,    -1.1539858153216329e+00,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.7741359631561924e+01,     1.5522135727470819e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8833420432741740e+01,    -1.6595249699374450e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3386276780694871e+01,     8.2801315545825182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8757558726357594e+01,     1.1772580147333763e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7231401527203090e+01,     8.2500280232286896e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0602216972003073e+02,     1.6655651880769663e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1907683269258101e+01,     3.2122584878216574e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.3201911338055506e+01,     1.5518171126180861e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2894436897617028e+00,     7.2265143848356538e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.8857268304174788e+00,     1.6869562147299388e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    8.5055373028937815e+00,     7.1573958864437709e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5974990678158459e+01,     1.6406348490379893e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8021468082974081e+00,    -1.1314893059126725e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4958717182620296e+01,    -7.6877646011599199e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.7452297536368533e+01,     1.9550516247923795e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.3155807657599155e+01,     1.6656678217330747e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.2002776737892077e+01,     1.6658087481163053e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4335857336029248e+01,     1.4907887749337118e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.2092479921082781e+01,     3.2166355711773533e+04,     2.6931621671538551e-04,     5.9995954836692873e-07,    -3.0545685978306309e-09, } , 
      {    2.0159346247411499e+01,     2.3707152404665681e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.7667560615747568e+00,     1.6864302482667015e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.5133214536660402e+00,    -8.1528990060112083e+03,    -5.5407290908896314e-04,    -7.2639934425124045e-07,     3.4752414289293752e-09, } , 
      {    1.8563889739934258e+01,     1.4951682963225179e+04,    -2.2075052699709397e-04,    -1.2109457960597486e-07,     4.5010102154209299e-10, } , 
      {    1.8018998179780638e+01,     1.0653035815326213e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.2626757773621993e+00,     1.2132440086685065e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5853359999031831e+00,     2.3305494936589861e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.1738217876773263e+01,     8.2734306032097575e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3962424660867136e+01,     1.5449454285455220e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.3278062532490225e+00,     6.3478388326440618e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0477273783291512e+00,     2.2534648451185851e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.1524327249649211e+00,     1.5682439700866367e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5727304362289320e+01,     5.4087699140175391e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3039449380121260e+01,     8.8728877930577346e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5562050919641861e+00,    -1.7947779446793145e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.1243882091377046e+01,     1.0094169474066342e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0771697115588026e+00,     1.2613092753598843e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2703853419984378e+01,     3.1418665576024887e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4856155328808011e+01,     1.1943432412872462e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7866017065621925e+01,     2.1874078130618691e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5510104049351938e+01,     8.5780557922576627e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.6184396046699355e+01,     7.9176215114745992e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8809831544317310e+00,     7.6208233739241141e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9830969982554876e+01,     7.1653609995437891e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2249703717990279e+00,    -9.9345567308505615e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.2575250226259961e+01,     2.5994284917410616e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2860526908409504e+01,     7.0574824968332341e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.5069042990315268e+01,     6.6038042237686641e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2034099379315100e+01,    -4.0613888089125430e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3907126713684921e+00,    -1.9311557020690984e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.9021142785221681e+00,     4.4966975049625926e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0646756589982944e+00,     8.3249249391291360e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2780417877921373e+01,     2.2774090152105808e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.8876362242457098e+01,     9.8622799395226848e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6452931969300124e+01,    -8.3839522460371427e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.6213474813879731e+00,     2.5213647947808447e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.1562463154160133e+01,     2.4072921457597899e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9887102727914687e+01,     6.2545540876326377e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.5240694934998434e+01,     2.2131139402610781e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.6042476790323352e-01,     9.5979357886114087e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0028906188714702e+01,     1.5489073847303727e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3786612304693156e+01,     9.1945455943727648e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1380353724458576e+00,     5.1546406277438678e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1177406927165450e+01,     2.5601263947717627e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9575669102406472e+01,     2.2792865571743740e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.2484288507694718e+01,     1.5563372852492770e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6739142730178518e+00,    -3.6463503628272872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3686872015291680e+00,     2.2039492056679806e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.2807548982979888e+01,     1.6732633910317611e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.6796003825128011e+01,     1.5440582581053726e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.6993161115517275e+00,     1.6709197960642770e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.6582122894979218e+01,     1.6235496224841196e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.0262135508611188e+00,     1.5942334215273917e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.7091281184770430e+01,     3.8263523028918338e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.1098729272159424e+00,     1.6657159192089188e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.9512847919840947e+01,     1.6408978322696079e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4475928183536453e+01,     3.2086021073577576e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8257108238208442e+01,     1.4617026939711250e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0590692125883141e+01,     1.6517663227369358e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.0959469734345276e+01,     3.2722506529851857e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -5.3639365210293306e+00,    -8.3341211296613219e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    3.6269433321366176e+00,    -1.3679488585583708e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0217189297275471e+01,     2.3492502497193916e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -1.2253113710235723e+00,     2.3853396771149382e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.2642009336521349e-01,    -1.0798150931248660e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7236174996784573e+01,     2.0115055143380841e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.5175316199232078e-01,     8.0656234865647129e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.4523212777276946e+00,     1.6657606506404609e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3917761243733736e+01,     4.6386993753037259e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4286709286787655e+01,     3.9450248420990632e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -1.6969804373047779e+01,     1.9628036485235261e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3982135594129144e+01,     2.2778147063317992e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.9738335675567686e+00,     2.2088694668396685e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.6197331729400872e+00,     5.0202303571399843e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.2407469443489774e+01,    -1.9521332940888756e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.3225256378030338e+01,     3.1085515293514312e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.3164302661276759e+01,     7.7692996022071966e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8171688373395032e+00,     7.7636622180445856e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5242323873758325e+01,     8.3284061966332702e+03,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.6039564594933044e+01,     3.4570066339085797e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1003340859097371e+01,     1.5531937578724939e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.1110782312128620e+01,     5.9414481870432610e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9953432974535986e+01,    -8.3289766526136300e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0423777314068397e+02,     3.1085501866412866e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.0817609405704999e+01,     2.0887313511343185e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.2343016982496607e+01,     9.7428352338289460e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8766190812467258e+01,     1.3746849048459955e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.5325668586464765e+01,     5.4485848159154521e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4712854814886265e+01,     6.5895685721259360e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7090314167607730e+00,    -7.8117175280451920e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1454115534023810e+01,     2.3888366395412839e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.6152572832751034e+01,    -5.2183050945347113e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.0919307195343171e+00,     1.5989260362303445e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.6652227426367485e+00,     7.7114044663002705e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2986080568400105e+01,     7.7734546877745988e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5087400863037654e+00,     1.7455424998130831e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.7998207368148954e+01,     8.6218534341907289e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    5.0892780648226577e+01,     8.3614827602650621e+00,    -2.7584724895743558e-04,    -3.9757073197335323e-07,     1.9187955870953214e-09, } , 
      {    2.5878552772468353e+01,     6.3038512782924363e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6581351463472753e+00,    -1.6641383341022902e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.0147892785212331e+00,     8.9420426765308785e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7285958538400941e+01,     2.2381845129103014e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2638468347785201e+00,    -6.6208901250070357e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5459254276134066e+01,     9.7894416149281151e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.8473625940987626e+00,     1.5294848436211250e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.2837072042434741e+01,     2.2247954275988333e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4125906551016085e+01,     2.7488455879095009e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1600527429543405e+01,     2.5981665678992627e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -9.2354612517467380e+00,     1.1253305050814093e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.1936272680788633e+00,     2.5192993907290467e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.2502706864465036e+01,    -1.7403063119944645e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.1705686283313881e+00,     2.5198253571922833e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -4.5685227563746253e+01,     1.8111341560350309e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3383507165570052e+01,     1.1614845792993332e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6578020458218681e+01,     1.6726074954718479e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    9.1973833758311780e+00,    -7.8892697890573163e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.8350282777373499e+00,     2.8333489457286738e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1133728350623198e+01,     3.2681206844291701e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.5056818649377295e+00,    -4.8275853247831346e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6977092629376521e+01,     2.4028026083130353e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.6389118368794025e+00,    -3.4892445056420662e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7732543960376141e+01,     1.0609264981769251e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9412736321878175e+00,     7.7500472370130456e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3718779929908614e+01,     1.4922073288562522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.0880645142458864e+01,     8.2404251224447908e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2146676658286477e+01,    -8.3449007599641664e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.8011528790422551e+00,     6.2522528876912284e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8002992446115655e+01,     1.1308565872227784e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4988668116493656e+01,     2.3736458980623080e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0054114766824721e+01,     6.4798170858904186e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5225552568496724e+01,     1.3267111765093587e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5547796025956831e+01,     2.3558246182165149e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6346123626231659e+01,     9.2882002023491568e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.7885019755267777e+00,     7.8231827806478577e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.4098334330366749e+01,     1.4231524320075368e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6987395145349929e+01,     1.6522396115264732e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.5753159307477436e+00,     1.0683014473172048e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6810087802115344e+01,     2.4533504716943371e+04,     5.2628705194601375e-05,     2.7700527801894134e-07,    -1.5245887348483466e-09, } , 
      {    1.9646055295519155e+01,     4.0495071516728080e+04,     4.2151069211412756e-04,     8.5072205575288325e-07,    -4.2898253759296179e-09, } , 
      {    7.9860456454053397e+00,    -1.8157353292860542e+04,     1.5902484043111813e-04,     4.7050876863471795e-08,    -1.1645224620251053e-10, } , 
      {    1.4038315706150776e+01,     5.9715598990656690e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.6914443864022974e+00,     1.0454501267131973e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2618185608649775e+00,    -3.3056997760528155e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8079326327233289e+01,     2.9299529974150573e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5290835706910002e+00,     8.4059769660489983e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8702031879512937e+01,     9.5950877772570675e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.5931875179280397e+01,     2.9990526416214561e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.8168614275049549e+01,     3.1636263903997005e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.0032811732955081e+02,     1.5541023321504554e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.7761279892642037e+00,     1.4901827466518265e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.4860030205574830e+00,     1.8451078532387173e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0219907566257213e+01,     8.8407515339911224e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7562926623636420e+02,     8.3146769547129788e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.7091815713559060e+02,    -8.3427058945339213e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.4341813941004517e+01,    -1.6851199830983904e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5767884734355270e+01,     1.6894929433950674e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0739179803456256e+01,     1.5567337453782728e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9615412926106593e+01,     7.1969955838186970e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9248356925376406e+01,     8.3865668188381751e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.0044401562974116e+01,     1.5596434732659862e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.6272020745435238e-01,    -9.4367610282894238e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.5445619693995894e+01,     3.0785787614797177e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.6866258100271722e+01,     1.3660132734786575e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0737498062179345e+00,     5.0175082811218499e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1311478617012511e+00,    -1.6375695568021196e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    4.7171110590828098e+00,     1.1110176012600519e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4965518544812761e+01,     2.4133964277552739e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.5061782289425199e+01,     1.5526544954641078e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.9541986324975667e+01,     3.0831077140287685e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.8096771706096256e+01,     8.2162465957735276e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5194040770499928e+00,    -8.6753117284432901e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.2301944385698661e+01,     2.2735487247362289e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.0961157527717331e+00,     1.5443427495111517e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.2966184673602218e+01,    -6.9864054851730543e+01,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    7.7629486002228418e+00,     1.5019673553891758e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4739933502125817e+00,    -2.7385275494329253e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.4187131732369838e+01,     6.7605569161928797e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8253559372220280e+00,    -8.3353028241755528e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.1131322508118352e+01,     7.6799299593011347e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8941650609455266e+01,     2.4564187649464649e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.3997992804765019e+01,     2.6387305887103601e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.4044252414033407e+00,     1.7285684804460961e+04,     3.0192093403174383e-04,     5.0205414042347212e-07,    -2.5264077254930913e-09, } , 
      {    2.8873862389062687e+01,     1.9943537217616780e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.0254061460074325e+01,     8.3263919596794276e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.8980094030633445e+00,     1.2146666996022591e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.5799601047468990e+00,     2.3163508325763876e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.6877109394496742e+01,     4.4099465931736631e+01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.1802795880286229e+01,     3.2200848484695573e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.8975569645460979e+01,     6.9998768438144907e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.3950680623528058e+01,     7.7159258678663373e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7834714486698520e+00,     4.9757069024688008e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1770515145132023e+00,    -8.5447998459588853e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -3.8144266485514756e+01,     1.6792369583229069e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.1356993730059273e+01,     7.0724285704904387e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0606158748297631e+01,     1.7572293488260591e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.4209552355499602e+00,     7.7149269463112589e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.7104973301247874e+01,     1.4909515672678655e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6398569041063107e+00,     7.4896898637651939e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0788333307169853e+02,     8.3299610139815522e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0993258886166267e+02,     3.6574305952797204e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1024475919783370e+01,     1.6657531266304115e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5394420425967752e+00,     1.6657234432189685e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7835389622001262e+01,     3.2199425793761820e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.2063152625472249e+01,    -2.2036617699315866e+02,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.5908975360177532e+01,     2.1184733276568193e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.6191132683708261e+01,     5.4883997178133641e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4034158028205745e+01,     3.1887317214341838e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    4.8647758076638546e+01,     7.2032461541014873e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {   -9.6369691308856247e+01,     5.6421980953814818e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9532093943012203e+00,     6.8467790452816107e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -3.2548063751043202e+01,     1.5541690628556844e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3730557817881547e+01,     1.8071732373678420e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.0314633386083791e+01,     7.3230953538842186e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.0579735091722577e+00,     7.9946301811517224e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.4943621433977583e+01,     7.2104600693234279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4209234601418581e+01,     7.9820711208036100e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.4109725483618217e+01,     7.2311301468979909e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6484414715497422e+01,     1.5543817951406745e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8814185664853547e+00,     6.4821440089988819e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    6.1102773219051969e+01,     2.4500105615608860e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.6379159320430148e+00,     8.3368584676571172e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9268068312694102e+00,    -8.3205243815897848e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.9427064366755189e+01,     2.3990890420298980e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.1490909208469738e+01,     1.9402796853229720e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2432415376455548e+01,     1.5530302770504486e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.7312121766070172e+00,    -9.9615025597243130e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0726321092168551e+01,    -7.7173045203854990e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.6967895269378658e+01,    -7.2390878951352715e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9463245325187074e+01,     1.3800529491138022e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.6955862476987068e+01,     3.1571477238225078e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7789390824884102e+01,     1.5701661581615368e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8607851433793915e+01,     6.2870083130798514e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.8041811817918870e+01,     2.2756719103766300e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1806003206769569e+02,     2.2756915206913982e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.4168381253321670e+01,     7.2176656613932628e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3559654215781942e+01,     9.5011953781032789e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5035180219727478e+01,     2.2636160452492011e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1934561050453212e+01,     8.3297960163527678e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.7987486628575535e+00,     8.3275868328941324e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9073704490848726e+00,    -9.6010543382289852e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5603269426282038e+01,     2.4656333668175896e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    3.4115785497098688e+02,    -5.2836245894177409e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3189815996947075e+01,     5.9057022593133115e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4146251701619036e+01,    -4.2672313966360917e+00,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2447321734325453e+01,     2.4499041954183911e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9704871625773301e+01,     1.0862585281062517e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.0483381262218785e-01,    -9.5415357611575746e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.9647332045415496e+01,     1.8055984630163966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0904291547123410e+01,     7.3445747478427493e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.1749904776655580e+00,     3.7760185855330838e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1038380409219441e+01,     6.0427673271786653e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4757449801417316e+01,     4.9513911454458139e+01,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.5205278034536441e+01,     8.3881947421797140e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1441217038290251e+01,     2.3836500825595853e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0812947060316482e+01,     2.4078365348025363e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.0317223397831654e+02,    -8.3274218352653461e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.3744732054621212e+00,    -1.5852017899702171e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2578261709403179e+01,     3.1821193921817368e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -1.4325412080842346e+01,     6.5342203450725691e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0262495931030866e+00,     7.0628874666170459e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6982071428724602e+01,     8.7059646508019869e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7067742718894586e+01,     1.5553571001238650e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.8700899641513544e+01,     1.0019089746419922e+00,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.9872976551461097e+01,     1.9788432695592532e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1411658136548101e+02,     2.3870937755966126e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -1.8547001241449121e+02,     2.3871953673244363e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.7151463139943015e+01,     5.3612830519279555e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0150832132450493e+01,     1.3805680969763578e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.5633422853714904e+00,     8.1305833175349526e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.6291954596689536e+01,     3.1165138383759415e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3909616006757540e+01,     8.7681130601895838e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.1776842418396649e-01,     1.2655675693335822e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8297472698390962e+01,     1.4410101522514891e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.4922823727715915e+01,     2.4083625012657729e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.4675852926514771e+01,     2.9972096516641250e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0657636906527792e+01,     1.0860284081121106e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.2286019639140093e+00,     6.6452738573842162e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.0917782192729888e+01,     7.0443283015199549e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -7.5752289890650701e+01,     9.0747127468854906e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7538307510307590e+01,     1.4932495648392112e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8063298854396162e+01,     2.2286243924635030e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7147924288507916e+01,     9.9989910806644184e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.3374114315938606e-01,     1.4082004099677129e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2669316720591056e+01,     1.7267268089699319e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6018171902827582e+01,     1.5542902707039008e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1669315324785998e+00,    -5.9107132675661705e+02,    -1.5456735087326034e-04,    -1.8321230212549640e-07,     8.5812021556387854e-10, } , 
      {    2.1638800551574128e+01,     1.4252333312104452e+04,    -3.5170666445682708e-05,     9.8478166826319389e-08,    -5.8332782111099372e-10, } , 
      {    3.9632453431620000e+01,     1.5074482259218094e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3070961441620037e+00,    -1.5055377106678316e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8386706599167328e+00,    -1.6298143307009072e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.0811269072787166e+01,     1.5543850170849282e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0914545282634982e+01,     2.4186914423286107e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.0367843066247985e+01,     6.1963371066547315e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9203703688782419e+00,     1.5542605872924581e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8106693216681563e+01,     8.3309908895674725e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3394099137222552e+01,     1.4371458751802114e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6109558760497400e+01,     1.4427797098341918e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.5984083636928723e+00,     7.7927070529687489e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8371285406303315e+01,     3.1651459357978976e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.3987346638435696e+00,     2.9968578820757073e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.4746671728112062e+01,     1.5541658409114307e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0406767412752851e+01,     1.6886110509000326e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.8610751676445574e+00,     1.0557594079686851e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9731823537433915e+00,     5.6666978914573477e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.8953430401557672e+00,     1.6641963738107224e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.1550849029079373e-01,     6.2567032999802763e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6349091391783328e-01,    -8.3324579101177624e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.3031971634484790e+01,     4.2118969781555788e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1969483889767783e+01,     1.6266621848905130e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.8148410265740402e+00,     8.5485589288363281e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8851907926651177e+00,    -1.2374275003350554e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.7091338616984775e+00,    -5.0765088219741426e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0200756808246092e+02,     7.7004818911517468e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5193879111142019e+00,     1.4428454363091463e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.1274885958405374e+00,     7.2030422386359805e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0492615690262285e+01,     1.4939029868737185e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5971311164924238e+00,     1.4948233762222895e+04,    -9.4433871945491906e-05,     2.5097252867261127e-08,    -1.7899805920245181e-10, } , 
      {    2.6290169851389003e+00,    -9.4095385564333919e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.5381518398341225e+01,     1.4461907158171849e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    9.0419353904467297e+00,     2.4409581800220672e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -3.0412523326703909e+00,     4.6272093874469974e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0149461836094339e+01,     1.3958725437304720e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.9245705102043217e+01,     2.5685872703685145e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0200682882469701e+01,     1.5418801363096525e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8425780691937426e+01,     6.4160207164744115e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1270390648390210e+01,     1.7365872527620751e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.8180628254826061e+01,     1.3211325462523515e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.7972517774680185e+01,     2.4189215623227523e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4391248173507167e+01,     1.6902980731386800e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    3.2524515785150257e+01,     1.5796089848790210e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2604222031211965e+00,     6.9646984977241309e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.4894319401103020e+01,     9.7717592633600020e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.3538995895523776e+01,     1.0014539670270518e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.2869987353019766e+01,     8.9327261360595112e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.3577509342536827e+00,     1.7417687639016618e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    8.0526860070796502e+00,     8.8517721607134863e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.9011373667453753e+01,    -6.2794855553674165e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3050088155732986e+00,     1.2592450024072705e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5677605006509381e+01,     7.7080104232041776e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.1177526790937158e+01,    -3.6283816188423179e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8917634508236766e+01,     2.4046247350485326e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.7150609603238614e+01,     8.3285524487849743e+03,     2.7595071702059336e-04,     3.9633518430704559e-07,    -1.9343096248908262e-09, } , 
      {   -2.1861718704012219e+01,    -8.3288304004619240e+03,    -2.8634098504059006e-05,    -1.0510256400226382e-07,     5.3765837235047523e-10, } , 
      {    1.6935075647175509e+01,     7.2016113458810369e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -7.3246794944153972e+00,     4.1036765352696057e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.5590332064714808e+01,     1.4151152385908541e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5679530913408103e+01,     5.5240958182588884e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    5.5130277972858961e+00,    -9.9940753079219721e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.7637341344227053e+00,     1.4682557170392593e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.1796995775257582e+01,    -8.9507442791184985e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.1030266962283498e+01,    -8.5156989852275074e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.0381421184696649e+00,     2.4673254348983490e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.3036690380025927e+01,     3.0478530902895898e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6016856993874788e+00,     1.2566039104281208e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    2.2726858247374326e+00,     8.9729985178929910e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.5282180439945137e-01,     8.3232617195855764e+03,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -6.8213811912091103e+01,     1.6062184418512443e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8119257141054192e+01,     7.2116750323831939e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4363376260374190e+01,    -1.3262555677851078e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.7369533479643753e+01,     2.6172799665942654e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.1357610004878204e+01,     5.9831949931034615e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0917061439744185e+01,     8.3364063515697617e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1180608149299298e+01,     1.5366986251700655e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.3451410522115728e+01,     1.1867253920723017e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3613544241423734e+01,     1.4463070619726082e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.8262459556362458e+00,     5.3422580214199825e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.8155423132875530e+00,     2.1906340866140308e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4065031920318177e+01,     1.4901328793737983e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.9956117217085549e+01,     2.3243131416008975e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.8134876747727660e+00,     8.2173577361561656e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1066016841594356e+01,     7.2250834920807083e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9639109454697877e+01,     3.3195728544350975e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    4.6416615755052348e+01,     8.3436118629828488e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.3846448151006935e+01,     1.2809656445586746e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1787371044536464e+01,    -1.3221004822177065e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.8895785456866136e+01,     7.2164506983334941e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3419231734375739e+01,     8.8456653212017100e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.5197776551298212e+01,     1.7896725022572318e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8752741007071307e+00,     8.7397460782571488e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.3702249105778350e+01,     2.9973181220639894e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.2613999488588874e+00,     1.5948893170873049e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1579144338174634e+01,     1.6707046246682039e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2671763154398217e+01,    -1.3028481170235552e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0060788150437190e+01,     1.4012316132332508e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.3956193055088217e+00,     7.6911950516625893e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.8979894997567243e+01,     1.8445402803822035e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1118308415061238e+01,     1.4208586858878585e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.2057882969404705e+00,     2.5248592836817843e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.1084297093558302e+01,    -1.1137671898652684e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.2027208126437952e+00,     8.3926811485570051e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7349340515357312e+00,     3.8264976874561734e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0007324195096999e+01,     2.3884064953023233e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {   -2.3166634783660935e+01,     7.2151587462258303e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.3356832283225444e+01,     7.0565342262439335e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6480202144499259e+01,     1.1175028323387183e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1809189004141949e+00,     7.2942619246326167e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9322395972152790e+00,     7.2156766967222353e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.1289815802304020e+00,     2.5142654642395457e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.1306395307271675e+01,     6.6475750573256264e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.4924049194737020e+01,     9.6812211720421437e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.6208083316201467e+00,    -5.5615081807832087e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7948478472291249e+01,     1.7306353950549339e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.5134750246426222e+01,     3.2199808506853919e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1923801827204210e+01,     3.1674001263111219e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.6757344320392896e+01,    -1.2894301951451844e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4075006690390614e+01,     1.6799369460072881e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8951694757144509e+01,     2.9373739290307072e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3156897889269766e+02,     4.4031195018378355e-01,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -1.6522287444858861e+01,     4.0875345369228671e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.7830141281885126e+01,     1.6882664798982496e+04,    -3.7740249195369381e-04,    -2.3569566172990487e-07,     9.4368983027971148e-10, } , 
      {    3.1214376848649561e+00,     2.4041180278443633e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -5.0457585117409849e+01,    -4.0167217855986905e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5383564430263501e+00,     6.9761464176896043e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.4939529018710802e+00,     3.2200465771603467e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.1611547888301168e+01,     2.4691269081446382e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    4.5436265068832400e+00,    -2.9480519242396940e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    9.0667764060987643e+00,     6.9515443024108517e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    5.5713228119599352e+00,     1.8017609290651904e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.9280278588571306e+01,     2.4579935392979096e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2947772240297965e+01,     1.6047099472403157e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -9.0411933125656994e+00,     2.4906444470074523e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.4105260403756425e+01,     3.2251952250624567e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.6422034472455969e+01,    -7.6943503651351566e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.2079833384152312e+01,     1.0467263111428118e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.2898721156749502e+01,     2.9868708311770413e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.9925605337961034e+00,     1.1933367916244072e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0353659536589934e+01,     1.7585411399458855e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0821512482490093e+01,     9.9020948414205959e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1675809384841211e+01,     8.3860697490049006e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3799942652751259e+01,     7.5798518874563952e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.3720785568647448e+01,     3.1178808584490165e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.2719421310742348e+01,     1.4525028531278182e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1818940658870119e+01,    -1.6003555428184336e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2002207283850497e+01,     8.3401566772261158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1628086071301741e+01,     2.4383079114862339e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4066370420963761e+01,    -3.1285050964640959e+00,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    4.1016644721656093e+01,     1.6730677354601299e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1377386800996028e+01,     2.5218165644738896e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.9312608567832470e+00,     8.2647017006898950e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6281087648905228e+01,     1.2336453382565449e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2836670911161185e+00,     8.4169975927713640e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1935262444179298e+01,     1.6257812399693690e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.9682134751372629e+01,     1.5010047909404235e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5501777892628681e+00,     6.6806728480128204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8295134019126543e+01,     2.3402397737151350e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1534080220574136e+01,     2.2821251505153999e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    3.0295842155861514e+01,     2.0336558187309769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5420705108972871e+01,     2.6780326856796586e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7803835585502028e+01,     3.2277689400240819e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.9706635123527683e+00,     2.3537384471133519e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8727884265341391e+01,    -1.6860898840540449e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    9.0352088955094594e+00,     7.0018835673058593e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    8.5643651491101380e+00,     1.6117381432349270e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7863117526232259e+01,     2.2741275127051711e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.2785751061665387e+00,     7.1960291814176353e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.2611047936000972e+01,     2.5065704077666174e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.2390503620130801e+01,     7.2129669844908585e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3553898928677318e+01,     1.6939070130472603e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.3584613968772741e+01,    -7.4782151354236166e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.2002514124858479e+00,     9.4271106485478376e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7501750165611742e+00,     1.3419915087176425e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7609560352823159e+01,     7.5246005417816123e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1344278781138257e+01,    -8.5820269834318697e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.6964906246772244e+01,     7.4131373748479054e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3099556435725212e+01,     7.0071432319382275e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.7574995335370168e+01,    -4.8252841248417244e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.2559152063037700e+01,     3.6611416541221588e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.2250199355848167e+01,     2.1280334481036174e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0985877028021900e+00,    -3.1779491849990600e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0647277670901534e+01,     1.1459741270969083e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3450175295945161e+00,     1.7678711403985431e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.2112135021843137e+01,     8.2898429251266007e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.3901905545393092e+01,    -8.1552245862837190e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.3501957185487463e+01,     1.0074242141735131e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5897769421266617e+00,     1.0440274357794444e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7779363173821267e+01,     2.9359493116460533e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.2507662831683124e+00,    -2.1725293141184557e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.8145082381724762e+01,     1.6562211017162648e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.9799681422524706e+00,    -1.7827510462085611e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0903045371046204e+01,     6.7549718532018633e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4151910980627562e+01,     1.4190579146012918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5994841872001967e+01,     2.1813040861613736e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.5652311628499803e+01,    -4.1305823180409789e+00,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7333625909390108e+01,     3.1538078136890570e+04,     2.7188216584470991e-04,     5.9938678948406585e-07,    -2.9994016490571770e-09, } , 
      {    2.1796129411889375e+01,     3.0490963671873589e+04,    -1.6051911570574143e-04,     8.7171342155482774e-08,    -5.8774447374590161e-10, } , 
      {    2.0546303901563718e+01,     4.9033312407087826e+04,     1.4120595359883419e-04,     5.8931274904155458e-07,    -3.1126977581956645e-09, } , 
      {    5.4813825270103056e+00,     1.6204873312645079e+04,    -9.4531804309076040e-05,     2.5140886098560985e-08,    -1.7827083868078751e-10, } , 
      {    1.0310076252380242e+01,     5.9237019078061603e+03,    -1.8233117594936010e-04,    -1.5338622509406093e-07,     7.6299007505656529e-10, } , 
      {    1.1600678635081874e-01,    -7.1017685744087430e+01,     1.8548192818782712e-04,     2.1961637966359409e-07,    -1.0327016221314225e-09, } , 
      {    8.5803398650155245e+00,     7.9291114993313295e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.1531154366232755e+01,     1.5275583779575754e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6136154995784324e+01,     2.4493194344051652e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2169000878469879e+01,    -6.4749202646257982e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.3344410296140131e+01,     6.0658034231042279e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2623815331532100e+01,     8.6261111898484523e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9186465755727067e-01,     3.1085837212338363e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    5.0820647696464945e+01,     3.1085179947588815e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.0192576836474810e+01,     1.2165601117026752e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3003516472035976e+01,     1.2702110267213633e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.9186471388979097e+01,     3.1316750873945239e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1482591045001730e+01,     1.6031705096517542e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.4168310333750362e+00,     1.4426048187932989e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0492239398147854e+01,     1.6851513203843526e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -1.7003860371776298e+01,    -1.6882351426122877e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.1793979191838069e+01,     1.5089248340816330e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6404695807342179e+01,     6.6077965337575297e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5392462258551653e+01,     1.2706617246658425e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -6.0091762630947315e+01,     1.0371142818874490e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3415010587377232e+00,     6.9158596351222229e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0723918974083389e+01,     1.4430203273500392e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.6003472333232738e+00,     1.5516093583397162e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9632289600090424e+01,     2.2441348446659278e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.8430182307402955e+01,     9.0366288134648166e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.3220714556347772e-01,     3.2120507335432871e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.1965444641031482e+00,     1.4976803511966407e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9414636428990669e+01,     3.9953490076017792e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.0656258393342995e+01,     2.0194571576483788e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.8468260506534637e+00,     2.3902624118120322e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.3482048626883261e+00,     1.1609862569241914e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1975628992377874e+01,     1.9503719748010637e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5348097952017330e+01,     5.0317203449967128e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0879198552685146e+01,     6.2546267098775184e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2298972767876624e+00,     3.2788629822376330e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0441375626544897e+01,    -5.6882183437559206e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7290064826247654e+01,     6.5545768767977233e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2511349684579516e+00,    -8.5202166821579540e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -5.9261325053865598e+01,    -1.1488252859343618e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.7848441111768750e+01,     2.2772359183628570e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.1359563721046921e+00,     7.7985996168469246e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.4430016791723412e+01,     6.5461597567210140e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3531817970815979e+01,     3.0404231897706853e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1020790160102047e+01,    -1.3455079329792591e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4318122007865675e+01,     2.5119182496195790e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    7.2255186243649945e+00,     8.5037481174377153e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0341521867675841e+01,     2.3530239856308126e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.7063859944767891e+01,     1.4918259996749388e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8068826612640088e+00,     1.5002360691156835e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -4.8941532770011129e+01,     1.0410957720772401e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5094972675686625e+01,     1.0336989954398092e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9572163666056337e+00,     1.4077905688323845e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -4.9301010457287596e+01,     1.6752554681331152e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.8765161407491959e+00,     1.7250745545283051e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1825435511553914e+01,     5.8618877213894684e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8415871806186441e+01,     2.9395541532864136e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.4180491462180544e+01,     1.7738170046896819e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2685616097826671e+00,     4.1237125021950959e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0157160224860320e-01,     6.1463336447466672e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1522039241368816e+01,     1.1916821474969493e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -9.5229211178252022e+00,     7.3537482762429154e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.5734498869713860e+01,     8.7821973737889148e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2743558590812802e+01,     2.2216870415428866e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6746639452936613e+01,     1.6644931329769592e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -4.2270299128141396e+01,    -2.3285593694472163e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1110416339548584e+01,     2.2221809574821171e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9315430104361404e+01,     1.7363687321184869e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.9028154964335577e+01,     1.6894541292157133e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7772854809167484e+01,     7.7558351167024675e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3661195029463279e+01,     1.4133879389142874e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.4203115241606334e+00,    -1.3051188248440485e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -9.3711212162739599e+01,    -2.5069151674342720e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2253613612650472e+01,     7.0589583433340958e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9441334740762315e+01,     1.1046056785019202e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.3170491803943115e+01,     5.8224609612850918e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.0138710743498912e+01,     1.8405587901924126e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.1012035636182027e+00,    -2.9683411442790408e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5639429425665341e+01,     8.3659033025812805e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2632246289599085e+01,     2.4120810082239459e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    8.5888192551717957e+00,     1.5507417219045072e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.3304593018789195e+00,     2.5717993130229810e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.0663897373617489e+01,     1.4676530257267512e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3064774012820763e+01,     1.6728370524828410e+04,     1.1653693820750085e-04,     2.8239412752857817e-07,    -1.4944333238833335e-09, } , 
      {   -4.1372521009987651e+00,     8.0258085846668000e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.7078429000229178e+00,     9.6013061029385335e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7391654005386396e+01,     8.3131934347496099e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5358478368874922e+00,     4.3373704261865851e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.8151517203517145e+00,     1.4896397761480392e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4045300642252789e+02,     7.2141048911249691e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0630270936572543e+00,    -1.3499151746176613e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.2122876624726517e+02,     7.2140208395917198e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4000911969202612e+00,     9.4160900218254756e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.5866049671607634e+00,     1.4283509287574525e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7816186943301222e+00,    -2.3455065797399379e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3135183476427597e+01,     3.2356717507753805e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.5181867711683481e+01,     3.2193578183629561e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -3.2874201733696736e+01,     6.2866661455434496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0599844009582382e+00,     7.8827108334581817e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4805973616117740e+01,     1.2007866762686321e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8038560777720583e+01,     1.4887419879988738e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.0076488324291581e+00,    -8.3006943056559812e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.7728656495378633e+01,     1.6118804123283022e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8474354805586373e+01,     8.3470537409389326e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.7330580120478189e+01,    -7.4171881629652808e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.2740316419556734e+01,     1.6547317176391516e+00,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.5528323842896516e+01,     3.5693834229641652e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0776170018753080e+01,     5.4419636843428543e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.7948921163352960e+00,     2.5161130966684617e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    7.3244487137096499e+00,    -1.6669834368724209e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.5872061647039583e+01,     1.5718522328262934e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    4.5993711613656430e+01,     1.4213509192938698e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4949471978174781e+01,     1.5525122263707326e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.7883573430273056e+00,    -8.3118306779758805e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {   -1.2630448326751463e+01,     8.5459290965201435e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1924307157120557e+01,     1.4434025784801917e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0498253741984890e+01,     6.0736951709451287e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2159269224727709e+01,     1.3005865473752181e+00,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    7.4585777190841451e+00,     8.9459783829466287e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7507326222227180e+00,    -5.0434518242136147e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8071465336700676e+01,     1.0666093792364454e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9173092550133312e+01,    -6.6475608482545383e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1138016293829274e+01,     8.6748130556630113e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.1285190898846771e+01,    -1.9177377801907278e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9333892428114556e+01,    -2.2916139255603675e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9799874545068370e+01,    -1.1020093208471185e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    1.7159103292591510e+01,     3.2462655702176191e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    8.7594229775438670e+00,     7.2905041087692089e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.1439954362156698e+01,    -7.2322321990104683e+00,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.2261026949900211e+00,     2.1828788605128182e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.1809615390836058e+01,    -2.7870431328538598e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5754854961815994e+00,     2.3382823540444271e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3132213167195536e+01,     3.3794543728964559e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7238995769732433e+01,     1.6170733242652566e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6128508809419273e+00,     1.7871338203382282e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.7542858953958991e+00,     3.8297270245380525e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.7443898641267815e+01,     3.8500276036015671e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8392779597671325e+00,    -1.0344464659367977e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.8284236042674475e+00,    -4.9200538594836871e+01,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.1686112035061214e+00,    -1.6797102471124443e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    5.7583033611081529e+00,     8.1452199206667647e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.2276775679203638e+01,     9.0279923150819832e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.3442111861347357e+00,     2.3512206172367420e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    7.6854237670699890e+00,     1.5341249102084401e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.1459898750390337e+01,    -1.2340732649588424e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.3081259501527541e+01,     7.2125376121071258e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1945499049266002e+00,     7.7521989509737759e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.7268490012921642e+01,     6.9699969051712287e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5821075870165467e+01,     2.5544492080897111e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3074974096691930e+01,     1.6171056245195854e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {   -8.0397385928413527e+01,     1.4428027679142850e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1570448128079065e+02,     1.4428223782290528e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.2307549833834472e+01,     7.2155881186095612e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6750135248392994e+01,     8.2107987805570319e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.8139954218814935e+01,     7.7712790934170571e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4337510785084540e+01,     2.4360067888766218e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.4553917157911744e+00,     4.8858796316800010e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0796189331014480e+02,     7.7714751965647365e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2186382164351402e+01,    -7.1834099429996495e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8641431974910407e+01,     1.4932470913138051e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.9500814928011508e+01,    -8.2872217910078980e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7535794171009300e+00,     1.5310864755438135e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.4317862126802492e+00,     1.3679161442823017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2123754346816010e+01,     7.1622477539624742e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.0249828531450801e+01,     1.9899015222907323e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.5862116031075448e+00,     7.0987675581507574e+01,    -1.8804787731715151e-04,    -2.1904362078073129e-07,     9.7753467335796819e-10, } , 
      {    8.0226200731236563e+00,    -7.1720034793371576e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.5721657736502213e+00,     6.1053343601784682e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9818472987056627e+01,     1.6713706181834306e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.8761621816868654e+01,    -1.3786525191632109e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.7467632413995009e+00,     3.7106555655766188e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.2985948614822270e+01,    -4.2641582995704084e+01,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.3500290979276031e+01,     9.7030203319310349e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1538172961415391e+01,     1.5569414996566427e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2957231579430719e+01,     2.1542861801204759e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5898405277347511e+01,     3.6016230770507436e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9061584924962069e+00,     7.2126401744245923e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9862144380257147e+00,     8.8937206312023136e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0393063198121386e+01,     7.7673708992829660e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8657738620789210e+01,     2.4045264840310720e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9761779289336889e+01,     9.6523707768445802e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2856568144656523e+01,     1.1482020390780292e+00,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.2671707295867311e+01,     3.3776014827929612e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.6515626586359531e+01,     8.3773932904380072e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.6057802805698756e+01,     3.8301872645263342e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.0638308250753052e+01,     1.8798255735992978e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3752391474398797e+01,     2.4908522012858226e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    9.8588516624578393e+00,     1.5383846998348221e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6282943914758444e+01,     1.3799818145671146e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.9368915816149013e+00,    -9.4459498162047366e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.1639805620358924e+01,     7.0697750545909530e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3260933170731432e+01,     1.7016622391484729e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    3.5564269900807972e+01,     9.1170345639508359e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9281441008429407e+00,    -5.2206062944761225e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.3692749289369557e+00,     7.9652293256338635e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1722083284829385e+01,     8.3566885435909189e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.7313616850676134e+01,     7.2154855562920966e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1304542075635474e+01,     2.0181453665285520e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.4292544487788490e+00,     8.3298206751847101e+03,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {   -9.1403635495524540e+00,    -8.3275621740621882e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {   -5.3346430480885715e+01,     1.6854726390259224e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.6322501599395285e-01,     7.0191071573211721e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.5338119760535385e-01,     1.6060253667716435e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1204853409837742e+01,    -1.7935783184133761e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2845347259191147e+01,     1.4402386595568578e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0776161179627758e+01,    -7.9915985058424212e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4519102207717545e+01,     2.4607131056459020e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.1390064744319943e+01,     3.2200137139228696e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.7155324134706810e+01,     8.1421066750854534e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7988524900020114e+01,     8.6310422221060489e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8296582479084051e+00,    -1.7091734673551027e+03,    -3.9892075510656011e-04,    -4.7638412867334818e-07,     2.2922427175171852e-09, } , 
      {    1.8422729762622765e+01,     4.0562585610976210e+04,     3.5987740221216227e-04,     7.7782663421408804e-07,    -3.9421170038378582e-09, } , 
      {    1.0412694261022217e+01,     7.6665724019601657e+03,     1.8064103594756859e-04,     2.8769571057561523e-07,    -1.4657323539616488e-09, } , 
      {    2.3994349342374402e+01,     2.2581024736727904e+04,     1.1712174131664347e-04,     3.4919704098097409e-07,    -1.8193118197316447e-09, } , 
      {    9.6343212688519273e+00,     7.1857293759010572e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7354246514471765e+01,     3.3920736113144421e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0148965231558556e+01,    -9.4406901515723694e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.6832111125572503e+01,     1.0348600913742246e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4328847043631714e+00,     7.7249670033788398e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1777875499798434e+01,     1.9754925623834401e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6603885205780088e+01,     6.2239301536201310e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4610489333434259e+01,     5.7202117614848248e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0433096133176178e+01,    -6.8875879739071024e+00,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.7053384340901502e+01,    -1.5387945627793979e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.3229244135837757e+00,     9.5845952492476608e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1338862040012405e+01,    -1.7769710208570599e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {    1.2461165989234672e+01,    -7.3139218254840534e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.1248960252405588e+01,     4.3679180324995104e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.7875409484854003e+01,     8.9569009580951533e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2249164985704144e+01,    -2.1508174160705806e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9422367214570031e+01,     7.4045775161118854e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3349406122134098e+00,     6.1799830379005539e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3872904369403756e+00,     9.4256879576140873e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6923794887286167e+01,    -7.8834313932738587e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.1363972164313658e+00,     7.2136696190959028e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4222977607613085e+00,     1.6769161163594486e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.1134255259718087e+00,     7.6136333149115553e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.8554818228799164e+01,     1.5541942244341903e+04,     9.2743731943700382e-05,     1.0921223261429318e-07,    -5.2374421970263164e-10, } , 
      {   -4.8336173071403380e+01,     1.5543566335621690e+04,    -2.2511008419136763e-04,     1.5023212424749856e-08,    -2.9229416834093930e-10, } , 
      {    9.3301740377286304e+00,     1.7702121630493220e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0190055600628323e+01,     1.6860373359384066e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.4564196353208462e+01,     8.3272605318685055e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.8530872524348574e+00,    -8.3301223173783965e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.7489877003357153e+00,    -1.1191171900941648e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.3997049605087945e+01,     8.8041956875861561e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.8010321039305026e+01,     1.5932172463639865e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1348544184459751e+01,     7.8751854754579854e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7759669424861055e+00,     8.2735800891904382e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9732898655769880e+01,     8.9166586266146987e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3668555261550523e+01,     1.4827705656008811e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.8754890982526735e+01,     1.7897077338530391e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9590957865184890e+00,     7.3691673873825930e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3359399447290844e+01,     7.2144561116207860e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3977626454869672e+00,     5.0912236625529749e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9688607387813395e+01,     7.2151663927525524e+03,    -7.7462757135888876e-04,    -8.4758119031981508e-07,     3.9238880094281401e-09, } , 
      {    4.5317884890531708e+01,     1.6646566137990041e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0506871051760070e+01,     1.6127291041470377e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2862063540445398e+01,     8.3963790302603884e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5020865504491523e+01,     2.2048327476966289e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.0385671933022905e+01,     3.1572181870141227e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.8223318362074523e+00,    -8.9378113399129634e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1173422732029312e+01,     1.7383906211561462e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -9.4082074408629950e+01,     7.7002857880040665e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1398161295579078e+01,     1.7338659531503636e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.7000503754454979e+00,     8.3029542750071960e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -5.6914765584002382e+01,     9.3464171833270630e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8163309326288974e+00,     1.6107072151093782e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7890706358617248e+00,     5.8798954342551342e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8319148138143348e+01,     8.4729792353908397e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4722018681958275e+01,    -1.6184243665752226e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2610335850029784e+01,     7.6600434565236128e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.0078282469893402e+01,     2.4006432448587409e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7170387665022941e+00,    -1.9207463264574699e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.7949791007300902e+01,     7.7869191732793279e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4951167858090461e+01,     1.7288320266608629e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.6270822285351914e+00,    -8.8679814960542026e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.0648626390057423e+00,    -8.3838027600564637e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.1717401196539953e+01,     2.0729579157002758e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1124280821000980e+01,     1.4919377737193841e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3950820614961010e+00,     5.8592319179982405e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.9098875830261353e+00,     7.2070592838789089e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.6292575486977807e+01,     2.4499400983692714e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6842838503807009e+01,     2.7173347826489575e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.4637772160439351e+00,     4.2341884177540105e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3929176388230541e+01,     1.6728376154659891e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.1039490934307096e+01,     6.5861143098216025e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6413356587394651e+02,     1.7163933604608246e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3039378376729154e+00,     1.5671081718694813e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1265177361262580e+01,     3.2233918566683853e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.9780051582151255e+01,     2.3209362331936023e+04,     1.1968769044596786e-04,     3.4862428209811127e-07,    -1.7641448709581904e-09, } , 
      {    2.0479824696073020e+01,     1.6137274817740694e+04,    -3.7932480302175347e-05,     9.9138192171781901e-08,    -6.3704032884111916e-10, } , 
      {    8.8385980352169948e+00,     6.4099928718631427e+03,     1.8587086656980149e-04,     2.8650655957858978e-07,    -1.3561256769364048e-09, } , 
      {    1.2699886296231139e+01,     3.1858931280931578e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.4642236978179298e+01,     9.2910602531407203e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2116442846589646e+00,    -9.4114646141022286e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3540832837982158e+01,     1.5460270996712077e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9171335405658763e+01,     1.1002285951462240e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4291647754831800e+00,     1.6465308216601152e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1170651345748404e-01,     1.6866220969516326e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    9.6043755077479958e+00,     2.1517122666056646e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.8209652251643407e+00,     1.5508615116649019e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4409441690737346e+00,     1.6867643660450078e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    9.8421769645449366e+00,     2.5063626534882471e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.6850421191827554e+00,     1.7127956079952011e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0952895789498266e+01,    -2.4184557095168930e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.5681251083677886e+01,     2.5178350742840983e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    3.7856627303963641e+00,     6.4864284854425241e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2806632816707992e+02,    -4.4115691265742498e+00,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.6199263703675300e+01,     7.2132508197184507e+03,    -5.9548675818625800e-05,    -1.4150664154036154e-07,     7.1223977891801919e-10, } , 
      {   -5.0691727647053128e+01,     7.2148749109982382e+03,    -3.7740249195369381e-04,    -2.3569566172990487e-07,     9.4368983027971148e-10, } , 
      {    8.6501770888876219e+00,     2.3852258044849215e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.7647031772065894e+01,     2.5183172220129345e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.3002158136133172e+01,     6.8506007663687578e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4663967712435037e+01,     7.8430817958707648e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2798772639585735e+01,     2.1913893032516607e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.2052115812500990e+00,     7.7195715093339377e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.1060057716274443e+01,     2.5650430275319493e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.0131110163565582e+00,     6.1408644298842319e-01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.3397455322410394e+00,    -5.5994411194873646e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6094361537694731e+01,     9.0563258045392704e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4524767225452152e+01,     8.3356913014672646e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5136639543530096e+01,    -1.2881296085978092e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {   -8.6972233922268245e+01,    -2.3332291671530925e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2799086471693064e+00,     7.8696004124669689e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2179979208160354e+00,     1.3816027481011864e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.1909325858302694e+01,     2.6997346796433339e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8624705383809996e+01,     1.4914805734445052e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5219632541081358e+01,     5.6898281155827561e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5105206896966038e+01,     2.2125879737978412e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.4400235916888970e+00,     3.2369871703067085e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.8136641476589919e+00,    -1.4043096734441946e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.7047119834999634e+01,     6.8644595935981870e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.8891022284862267e+00,     7.0551555737247700e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.3149561880439400e+00,     5.1489635379455831e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.2866910741517827e+00,    -1.1272477976830928e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    7.3515920435848994e+00,     6.6063738428237757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6690532923479022e+00,     9.4973926084939540e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7279839512289957e+01,     1.3525297474186937e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.6149543002607516e+01,     2.3776273882520989e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.6971173306733021e+01,     2.4355465488883397e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    7.0850919045597944e+00,     7.3511337034418830e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.0345326925579919e+00,     7.7041971313910417e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0586055387831315e+01,    -8.3139729161977011e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.5470426568094240e+00,     6.4537580755886238e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.6320284902987570e+01,     2.3778145710078672e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.8674452186862105e-01,     1.3688983129984143e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.7800054779654587e+01,     1.7460492070172520e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.0622464254121908e+01,     3.2279766943024522e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2767713766355419e+01,     1.1465252602665302e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.9866574228348082e+00,     8.9783289176468443e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3985294458813353e+01,     4.0541626238151466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.4886741511078299e+00,    -5.5468444731636919e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -5.1918474171812576e+00,     2.3887178198604543e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    1.8997371581409119e+01,    -4.4322631551487366e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.6879341055094770e+01,    -2.3306471208419225e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.7204749593250781e+01,     1.6683121984395009e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9263933986996541e+01,     2.2614478228556083e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.5338195986359182e+01,     1.8002579393886299e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.0301088489024245e+01,     1.6643876337975547e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0022723465468637e+01,     8.8391063656025763e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.6477711825613000e+00,     2.4644868415573230e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.7189101363173056e+00,     4.2844013289753939e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4875398606782278e+01,    -1.1458069627801838e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1183479818386779e+02,    -2.5796901985665954e-01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3002525624851108e+01,     8.3757604677494128e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8959570792362292e+01,     7.7161163235772055e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.8063619811325573e+00,     2.3243466761934476e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -6.6793300483909832e+01,     1.0037081575402763e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2264254317406664e+01,     1.6101172097008553e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.8383987316580601e+00,     1.6604423227774620e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -5.7709271180102180e+00,    -8.3744704628220825e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.9148331295693069e+01,     1.6670889360518257e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8094496263080234e+01,    -5.5961547957396397e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.7439730344358582e+00,    -3.2202766427999723e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2823819032270823e+01,     1.0546720738341641e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3180984067882564e+01,     3.2407056772648815e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -1.3053970545241624e+01,     6.0587308533837302e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3624639176634545e+00,     1.6291593827148850e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.8985670834632707e+01,     1.6139883471512261e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.4447925458117044e+01,     4.6049179478485639e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.3526806010080046e+00,     1.2726146783150837e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6160435123404717e+00,     8.4895759726628603e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1067346906079461e+01,     1.6151874205271308e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8460689836297846e+01,     2.2676635062011832e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.7709266867464617e+01,    -8.8837045043145918e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    3.0559207163519613e+01,     4.0593262913666003e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    6.4283120725764160e+00,     5.9548178629510739e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3649317731364246e+00,     1.3264810565152176e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7216158308170009e+01,     8.1529081268271548e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7265809840151057e+01,     3.2412316437281181e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    8.7643824915078511e+01,     1.4286132406339982e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4802583116833077e+01,     8.8878205771170833e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2254981859431961e+01,     3.2218161347430490e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -2.5090414570999286e+01,     1.3093175944204447e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.1021549799871156e+01,     3.3523962848272022e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    5.0096987481701348e+01,     3.3524667480188175e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    8.5672939781064663e+00,     5.5471992532337629e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8795486754457755e+00,     4.6357697659085737e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3394452184082928e+00,    -2.7047461219777651e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3964715119471638e+01,     6.0602828188539206e+00,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {   -1.5415950868528370e+00,     8.9493724260427225e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5954544502134901e+01,     1.2801879355204449e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.0113911217462515e+01,     1.6306048609009506e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8230314854324035e+01,     7.9843723207450239e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.7482546251371005e+00,     9.0966844205535635e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.8704151364199362e+01,     7.7693460318607777e-01,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.1648319128260692e+01,     1.4449455638694541e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6663361824327556e+01,     3.8891830613963881e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.0155189026606211e+00,     1.7312862734922660e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1159889625135639e+01,    -8.4219914291500227e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.1029794445420453e+01,     2.4426458794296384e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {   -4.6433675490929041e+00,     8.3570431470407366e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -6.1223730806479963e+01,     2.3243144843110425e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7704609349033554e+01,     1.2729474352258436e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0980531697440981e+00,     1.6097438737298166e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2093676909587838e+01,    -1.3900827641066678e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.2738989069916286e+01,     2.3227596101271272e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.8708152239551659e+00,     1.3657831534845165e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3759288451562078e+01,     1.5625237583251512e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4016505116220436e+01,     5.3133923667072650e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -3.5059052019725740e+01,     6.7207420142949686e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0256383773335191e+01,     3.2164088722825098e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    4.0676978151006988e+00,     8.4630514518231612e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6461946800234522e+00,     8.5356110580435670e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.2164926395995780e+01,     1.0136061918747955e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7062115853835063e+01,     7.0743432434808028e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.5976399590153166e+01,     6.0814100978914403e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.7954176364619396e+01,     3.2985025092799347e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    7.5767259969209846e+00,     8.5408707226759343e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -8.9338997038857769e+01,    -1.3242760765752460e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0676284354312344e+02,    -1.3240799734275683e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.6781398320446934e+00,    -1.6873491270582334e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -1.8365852319185560e+01,     4.0528476247894061e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.6051064703888308e+01,     7.2121139813750087e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3681466591013567e+01,     2.1673355450729163e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2717850267299355e+01,     4.0529180879810221e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.7908011443980811e+01,     3.3050613428428522e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.9465130164247405e+01,     6.5808242480552053e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9425532223278115e+01,     1.1701586841920773e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3246270959547335e+01,     1.7672152448386296e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2426510777553742e+02,     1.5544485258459034e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8416567873597778e-01,     7.2832412979102519e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.8481331317351490e+01,     1.3738421094315709e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8490120185290568e+01,     1.5578091360918517e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1151645228873207e+01,     1.6102698401930533e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8611108846267200e+01,     7.1027291768910609e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -1.5522928794610955e+01,    -1.8700756060405729e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.8249697345624732e+01,     9.0353938608899682e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.9309662931198176e+01,     1.6602025919060557e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0169047398169957e+01,     3.3176345958836309e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.9783561236359004e+00,     6.0453616814878424e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8406061770001934e+01,     1.5540805405998459e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.9335506025380660e+00,     8.3532840641632938e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.3915831959567676e+01,     2.5612080658974483e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    3.6214714728036860e+01,    -1.6656416446845840e+04,    -7.0626738448770513e-04,    -9.7716185163719508e-07,     4.7104982070283614e-09, } , 
      {    1.9502568090141125e+01,     1.8327682505287870e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.9227713936793567e+00,     7.2729320166547650e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5243485746076850e+01,     3.8825707321439404e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.4367727319607305e+01,    -2.4839796523315135e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.9982817832973055e+01,    -1.1842303814804268e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.4633730458898064e-01,     2.5784116422754279e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -2.9987487206837820e+00,     7.3047330375687361e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4874241316878425e+01,    -3.5133424023739599e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8126279349451870e+00,     1.5544703173965130e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4440570452682014e+01,     2.4080995180341546e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.5042121885100130e+01,     3.2449501506862911e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -6.5533684999830584e+00,     1.5506671662585224e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {   -6.0438792484497039e-01,     1.5505994528111320e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6383068987420558e+01,     1.4985440010349243e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.2806317917314558e+01,     1.3522844800862998e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8694347880049950e+01,    -8.4242926290914365e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.6572516016731285e+01,    -1.3397200532898369e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.6110301598224193e+01,     3.1605592231138693e+04,     2.1024887594274456e-04,     5.2649136794527064e-07,    -2.6516932769654181e-09, } , 
      {    2.0109027467444601e+01,     4.0772135076712511e+04,    -7.2719744065457339e-05,     2.6569845334810471e-07,    -1.5290053874832544e-09, } , 
      {    4.1751942263884052e+00,    -2.6276495251747692e+04,    -4.2586471360882760e-04,    -7.1579617815716632e-07,     3.5326433687727442e-09, } , 
      {    1.8308329786983045e+01,     1.3766066728378566e+04,    -4.0347064132842842e-04,    -3.4137098461503148e-07,     1.5365151514036408e-09, } , 
      {    9.0867786447518846e+00,     5.9912160020542869e+03,    -2.4396446585132548e-04,    -2.2628164663285614e-07,     1.1106984471483244e-09, } , 
      {    2.3201130377360979e-01,    -1.4203537148817486e+02,     3.7096385637565423e-04,     4.3923275932718818e-07,    -2.0654032442628450e-09, } , 
      {    1.1285165230015863e+01,    -1.8830088423001428e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9330333061964623e+01,     3.3443634398121510e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.3824538155780814e+01,     6.1944742950499949e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.9153007288971118e+00,    -2.2627659755379341e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5269297022845549e+01,     1.3202677415279733e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6946450728037028e+01,     1.3858953001708426e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9975312266264247e+00,    -4.1408011081431063e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2867475694457436e+01,     2.2171169386250302e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.3575127045493893e+01,     1.8994785216987908e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1242223674104014e+01,     2.2612529344572751e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9930258466483714e+01,     1.4406795822738837e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.9005326186567935e+00,     1.5009364272636270e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8759329474960936e+01,     2.3387425940327095e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.4087590287147052e+01,     6.5853962508039995e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2106340041760157e+00,    -8.3463234508979167e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.3865131588784187e+01,     3.1945705699552796e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.0730327986153072e+00,     9.1135793781941029e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1127378486034523e+01,     1.1255606250755505e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.2012287383776972e+00,     8.3293130522979645e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9517684680530042e+01,    -6.0199919441898498e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.2614114058043064e+00,    -5.9712918153046508e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.6892101384621249e+01,     7.1975544346688521e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.0464273771041896e+01,     1.6712739779433243e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1399219563781131e+01,     1.6534523974016924e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3154461591604507e+02,    -1.4012734913084557e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7682687008224082e+01,     1.5579514051852269e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9759540299384772e+00,    -1.4486898027368334e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.9482039234413470e+01,     2.3108151395577537e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1979051930874149e+01,     2.1579831789044878e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.0480962862412220e+01,     1.3919262851364881e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.3681550788629293e+00,     8.3352428900155301e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.5036385298321559e+01,    -1.5497989873840709e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4426083571476697e+01,     1.6224030972238528e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0890070259194950e+01,    -9.1652045217660680e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3554888561151781e+00,     6.0803065704972341e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8757898908405576e+01,     1.4936487958380980e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6223967061578808e+01,     1.9669740514723344e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7638410723715596e+00,     8.6921535236130367e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1193819085714475e+01,     8.8863978861833330e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9964774035030491e+01,    -1.7774312608453412e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.3363850782702588e+01,    -1.1095614872234162e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.0273838749664336e+01,     2.4431061194179205e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.5226548686428671e+00,    -1.0888914096488515e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.6864068695504812e+00,    -8.4389140954894738e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -6.9151601134500623e+01,    -1.3086359967129774e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.3994597369462895e+01,     4.6072033498372548e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5871037304977165e+01,     7.7261229747260168e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5976414817793980e+00,    -3.1710705035756382e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6879055517064565e+01,     1.0182462584621102e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6737705207097054e+01,     3.9256465650246602e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.3270110352741547e+01,     2.1712078374093820e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.1403787583834180e+01,     7.2965461586280635e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5036783521598110e+01,     4.0511907883044551e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.3694591680487918e+00,     2.3933578864477691e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.1854040228378011e+00,     7.0993305412990367e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.6777914619098965e+01,     5.4247413146896997e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9492996600644936e+01,     1.4446174674172551e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4904704803370670e+01,     2.3871086685096441e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.0388900415452888e+01,    -1.4118085175524527e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.4901735912031500e+00,    -8.3280697969489356e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.2081915044497098e+00,     8.4404697389710363e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.0373621396600456e+01,     2.9766760260000057e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.9394147400542693e+01,     2.4582236592920504e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4062264734474450e+00,     5.5400857985649998e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9975641685734626e+01,     2.3871804744114048e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6060912371559841e+01,    -1.0954408895090737e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.3085080112580775e+01,    -8.3132615707308250e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.7908413808691286e+00,     7.1933993491014517e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.1102557006686180e+01,     4.0163039541754100e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -1.0151816316580981e+02,    -1.1483274306001931e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1789161561950179e+00,     8.2551037649012833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5600140175498419e+01,     8.9776625259258872e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.4798877379841375e+00,     8.4526538272476319e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.7788183040019177e+01,    -8.3010824474495166e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.1976068183562724e+01,     1.5525687008442148e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.2296652602224381e+00,     1.4908413230493501e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.1571910000781941e+01,     9.3809595490778884e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5147774583551779e+01,     7.6932779347046153e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -7.1066019619957794e+00,     1.6171390507402110e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.7569342691345700e+01,     3.3731563430024995e+04,    -1.0373225374360825e-04,     2.2933800904130685e-07,    -1.3536967603940461e-09, } , 
      {    3.3734177292192385e+01,     3.3736165829907812e+04,    -4.0831706926826062e-04,    -2.7209973926800261e-07,     1.1182712368472555e-09, } , 
      {    1.8411646300968414e+01,     2.6046100028806482e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -5.4320095843349847e-01,     7.2160117493416792e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8418237595555091e+01,     1.6538904545954225e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3592004662025472e+01,    -6.4142549624381081e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1869556626833241e+01,     1.6674243595894466e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.5328994100422179e+01,     7.1519297154858978e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2074943908163306e+01,     6.0990819901351670e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -3.0073577937318792e+01,    -5.8884499910570582e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8015303783111804e+01,     6.6501896301266597e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    5.8954144385585309e+01,     6.0997866220513151e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.3078772322889551e+00,     7.1677051676992578e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.3107595111774913e+01,     7.2143171297425788e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4009848165697623e+00,     2.1570091763267836e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5899614168111958e+02,    -5.3829371808385986e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0644541933913867e+01,    -1.4719265870614527e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3057951018934340e+01,     2.8313288811792190e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4290110301218235e+01,     1.8052929513408053e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4198683594790209e+01,     2.4916184495851565e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4567380007478164e+01,     5.1205726371347851e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.0551743446163719e+00,     2.2971433693118131e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.0616896235827422e+01,     4.1850705388912138e+04,     4.8115730029633751e-04,     9.9218506406194504e-07,    -5.0027923753693012e-09, } , 
      {   -8.2473157309230899e+01,     8.7406515034137628e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2806662196931157e+01,    -2.4677660320928589e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.8614675521555512e+01,     7.2138086009741100e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6245145294763496e+01,     3.9478634354400900e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -6.4811881629410468e+00,     4.9780935679158574e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.6314328782495409e+00,    -8.8668275102388816e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.0764347772759102e+01,     3.7724232957487584e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.1323106068766895e+01,     1.0109573537911606e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7646149206973924e+01,     4.8319246912309836e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.1015137334238005e+00,     2.7222471856026459e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.7778297960904130e+01,     3.1051727152508429e+04,    -1.0145177493310048e-04,     1.6063952257714078e-07,    -9.9061979461111474e-10, } , 
      {    2.5422736254667385e+01,     2.2723035727884977e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {    1.4982672256108662e+01,     7.8085833931172456e+03,    -1.9022488806450153e-04,    -1.5158068198287283e-07,     5.9894366977953167e-10, } , 
      {    2.6726235676639130e+00,     8.4022790843456169e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8654850476146578e+01,     1.6764424571063490e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.3924121078549367e+00,     1.7253041180361790e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7873605254758282e+01,     1.4301095453028822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3807698684433879e+01,    -6.1568834662755580e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.7295714728817490e+01,     7.2163403817164535e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5287721342293722e+01,     5.9574409817165306e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2028508961582219e+01,     1.7965314457953573e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1248184544014627e+01,     7.2260604761018167e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.5847706436828837e+00,    -2.9219990533896112e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5976099759959652e-01,    -1.2539853377749032e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8921485701349958e+01,     1.5485666391318213e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.4630461862478343e+00,    -9.4613441920903551e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    7.5674350405249005e+00,     8.5820117239167157e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.6427056406130262e+01,     3.3314413382535720e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -1.8733767758375031e+00,     2.5048431080900507e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2283184747501174e+01,     9.1453862863681206e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.4696340023581413e+01,     3.3315118014451873e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    2.1484097677852656e+01,     1.7942525829262377e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1783746397766468e+01,     8.3378649530236162e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5517026103992650e+01,     5.9724040284053708e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1624228310317207e+01,     1.2558622086719737e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9984587232146968e+01,     2.3890633384361277e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {   -4.3076613564217581e+01,     8.3563004017973835e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.2469362350073991e+01,    -8.5477773158943808e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3255436230694567e+01,     1.5083663277825313e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0034103922282981e+01,     1.6853814403784934e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -4.9036685490693550e+01,     8.3377268184843397e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9256059390685474e+00,     2.6346300715275353e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    5.1211633881552796e+00,    -9.7492788886538301e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.5945288186253151e+01,     2.2345260268967868e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7941418035288713e+01,     1.7483033975304766e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3585014603109347e+00,     3.8646201409385367e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1909310650041654e+01,     1.6597864272175488e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9499634708335716e+01,     8.3661625319608011e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9811900826148676e+01,     2.2992103770692695e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.8182939763897316e+01,    -4.6023998828204640e+00,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.2596487620198744e+01,     1.5539151493946876e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2890351811181585e+01,     1.3802447977987331e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4839024749839554e+01,     1.7994180112666294e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0994723009176507e+01,     1.6153037666825539e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.9972951317578307e+00,     1.3082493047010814e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.3474084142847914e+01,     1.7188787633730950e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.2544818060167302e+01,     2.9026485138402422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0833093632322083e+01,     1.4050852504538150e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3975797304662237e+01,     2.4238729534681977e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.1943165709917048e+01,     6.5791963247136682e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -5.4485738497190184e+00,     6.4411388371706371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.6432218638631406e+00,     8.3507326780681778e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0852847329475743e-01,     8.3066501711787223e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.7327874115999862e+01,    -6.5258045865502572e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4060019691468533e+01,     3.1783680219860860e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.4984542450669247e+01,     9.8942163477962658e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6276025251202384e+00,     8.1528634521822833e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5121825420230071e+00,     6.4892614755651869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.8636257345780713e+01,     1.1309855586467279e+00,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -1.8617010233359739e+01,     1.2119971543923960e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8754671750237528e+00,     7.7953014072779260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1758225699508252e+01,     1.5559821571521441e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.6951636508529084e+01,     2.6482170741643395e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0686631641882570e+01,     4.1855307788794962e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    7.6532123794489578e+01,     1.5546357086016711e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0856682732266103e+01,    -1.6854313076565217e+04,     1.8984148538210081e-04,     8.3498587632869385e-08,    -2.9030643224839010e-10, } , 
      {    7.3730680293082749e+00,    -1.9151578179559237e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.5523130477082546e+01,     1.3545386705995243e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1648616849722707e+01,     3.2445735021368593e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    2.6763913416775079e+01,     2.9184838081312508e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -2.3740173614637754e+01,     8.7969634553871510e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3929928267263485e+01,     7.3898461631546397e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3529271994575739e+01,    -8.9509655352262416e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.7915826972350487e+01,    -1.2496152932472723e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.8969793381468925e+01,     2.8480052680701374e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.0920322842835461e+01,     2.5075091787485137e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1369189002259958e+01,     1.1581788831952954e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3176462159805258e+01,     2.3865545660520016e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8755750735738914e+00,    -1.7635996605675418e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.3435285827054670e+01,     1.8739649145395852e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3141362601883280e+01,     2.1122600126695743e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1210297440177312e+01,     5.6218429252107273e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5294314521889689e+01,    -8.3318046702047614e+03,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {   -1.4624409904672888e+01,     6.8962929567360679e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9654696484562194e+01,     1.3856906044334726e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6228209968052070e+01,     1.2400887732379308e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1048480493743840e+02,     1.0164056760791937e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6665664598564298e+00,    -6.4405532855999445e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2734777126336468e+01,     2.0574474634978505e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8327319115177776e+01,     7.2332410593754648e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    3.2078505554706304e+01,     1.1852762240662069e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8545443359872298e+00,     5.4873360563884125e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.0208247120144932e+01,     9.3689590884593072e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.4588898006685056e+01,     1.8699834243497942e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1591284915663795e+01,     2.9246136261906431e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.6772081614871333e+01,     1.5578836917378369e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0193236914285671e+02,     8.9570970612428318e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2733011530098786e+00,     1.2560123656387652e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5503321910511318e+01,     1.4412583702428259e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -8.2790421771492049e+01,     1.6633109975637417e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.7600819057935704e+00,    -7.6802720692308628e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.2879214147441061e+01,     7.8423771639546158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2367366035020193e+01,     3.2750892463262117e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.4555288808865505e-01,     1.4692278000781964e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4144178830304689e+01,     9.4325032726316986e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5264043548283702e+01,     1.0196341182994413e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6381425613163714e+01,     1.4914087675427450e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1397842168669001e+00,    -8.5977594674311676e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0524022534813465e+01,    -1.9517742645800738e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3587577329741915e+01,    -6.0818992949328112e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.8265291785918858e+01,     2.7566368796182564e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9246072826049954e+01,     8.4085448855769846e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3670306515930825e+01,     7.7371436014483816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5336693616935513e+00,    -3.9957992529212106e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.7453528710068237e+01,     1.7252268256255547e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3990495183315169e+01,     1.6189110818483197e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6646482353056150e+01,     8.3927734584792306e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5355091971383345e+01,     1.6681655722856383e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5124601597319410e+01,     6.7453739174132515e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9672407032967232e+01,     3.9467169101798230e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.3557685046350606e+01,    -9.4771014113437141e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {    1.6245539270352367e+01,     1.8387201082359283e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.5485000167838962e+01,     6.6826580808742938e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3858529612387340e+01,     8.1754652914745529e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.6455638291386769e+00,     8.4334661574916008e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0863773450538865e+01,     1.1439077754712191e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.4573983629568739e+00,     5.3131406019977176e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6668821908148825e+01,    -4.8728737662062485e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.6907213681424935e+01,     1.4190967287806454e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5857721968678050e+01,     1.7455507171116806e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8849936875830791e+00,    -1.7363771887115151e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.8941308921335782e+00,     1.5304837842313054e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8130138456164829e+01,     8.3827640492288483e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -6.4067892262525476e+01,    -1.3015241107947077e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.6938790114359396e+01,    -1.3281308242581508e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0723150561860351e+01,     7.7224250930226353e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7394206811498300e+01,     3.0027981601656734e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.6519962668730429e+01,     7.2117853490002353e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2992935592663528e+01,     1.5916630435351435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2833807699318953e+01,     1.5049433438918417e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0419707701333426e+01,    -4.0392421854557488e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4315593221451682e+01,     5.8707066463398169e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -9.7098819509961736e+00,     1.1610139656231022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3127046470908986e+01,     3.9545164810082133e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.0121628616122102e+01,     2.3855713230605950e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.2803988903826240e+02,    -1.4366785868547026e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2488917991186059e+00,    -1.5541650762587587e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -1.3342125027058970e+01,     8.1555379591433393e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6668228490323511e+01,     1.5038409107560434e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.3592607083099107e+01,    -8.8883069041974122e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.9241552086372036e+02,     6.5615333885352811e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1167310737367572e+02,    -1.4016204907859731e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.4408739553554938e+01,     8.3934780903953815e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1915516870395924e+01,     1.4206509316094884e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6954229149461577e+00,     8.6959752447001829e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.9213646637161244e+00,     5.7533126750536794e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.0998586752809530e+01,     1.4839195643865540e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.8462627700264207e+00,     8.7877824367799312e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9164706089249826e+01,     2.3848903809472999e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.8865061457992809e+00,     3.8411674480610227e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7668528036908590e+01,    -8.2828430482827862e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.2784044036456088e+01,     2.3886864825744924e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7630163193860568e+01,    -5.8245920870487907e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.1206812148782337e+00,    -2.3712437293561920e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9063070882378184e+01,     1.6550964933522060e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.9399495198622638e+00,     6.8276008512644912e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0048049158078621e+01,     1.8436367409961149e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7909994356218341e+01,     2.4464808410641395e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.5409134252971519e+01,     4.4471835935081344e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4642739131973773e+01,     2.2763376110939276e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.9317808850174725e+00,    -2.0054547072581418e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.4416563030948659e+01,     7.2501454927549166e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -2.4008912533518259e+01,    -4.2196867907184412e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.5994213023973094e+00,    -4.6796499913158868e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7844365946139007e+01,     5.8814036718955606e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6026714031300529e+00,    -3.2532558230811084e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.2622761610714576e+01,     1.9004647834250591e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8103239924278016e+00,    -7.7069427951770413e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -6.0197252927634239e+01,     1.4443667759005120e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0258979266555439e+01,     2.3803931620357118e+04,     1.4774252154045787e-04,     3.8573201821297136e-07,    -1.9917115647341956e-09, } , 
      {   -1.4826727148190124e+00,    -7.0383304670712660e+03,    -1.8843681569912589e-04,    -2.8593380069572702e-07,     1.3009587281629509e-09, } , 
      {    2.0325069450664071e+00,     1.6934446409231328e+04,    -1.8964562065493256e-04,    -8.3585854095469101e-08,     2.8885199120506149e-10, } , 
      {    9.4089900709240375e+00,     7.8761818880216279e+03,    -2.4682421207140225e-04,    -2.2557798805609374e-07,     1.0577131599398632e-09, } , 
      {    9.6933493444092864e+00,     1.6694613477704344e+04,     1.4735358315848355e-04,     3.1884183829797577e-07,    -1.6682875099292129e-09, } , 
      {    2.6232128748169174e+01,     4.7918683847822729e+04,    -2.2956203804965180e-04,     1.4999272325176669e-07,    -1.0487489549761484e-09, } , 
      {    2.1844313453332692e+01,     1.5547163517152054e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1852232212272590e+01,     1.5333204824245493e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -5.5237754360674835e+01,    -5.3313575296128874e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0920301865895375e+01,     7.3975343693150298e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.7682712754435492e+00,     1.6628996915836640e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.9397847340862384e+01,     1.5741828799471356e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4708175467444187e+01,     1.0143724401741296e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0480407629620018e+01,     1.5540366457006643e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.2389033586464180e+01,     1.7777280316878579e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6939041403428812e+00,    -1.0299316834275917e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1869037932870803e+01,     6.1796333653675110e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8211438510885678e+00,    -8.3249935429200650e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4770887170805299e+01,    -2.4840452655082299e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0753379691639360e+01,     3.3836655367814499e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.5325663211577201e+01,     7.7828671328476248e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1280248122240923e+00,     1.5687869405904239e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.4522283130632960e+01,     2.3765507520182862e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5182936963234916e+01,    -1.1022267086602440e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4015519574174418e+01,    -3.7539243318568836e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3651762561281039e+01,     1.4411205049909098e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {   -9.3506488460865960e+00,    -5.6136272641691103e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7654688673656331e+01,     8.1866895542823167e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.6539182281781429e+01,     2.3242809497184928e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7045041452664975e+00,    -1.7436474897775252e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.9519227868426334e+00,     2.3207089713156100e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.7040187017623998e+00,    -8.3444391681378984e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1771609653310541e+01,     7.8526797316593756e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.6545336287207341e+01,     1.5545142122956946e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.7245951091731051e+00,     1.6929065464855648e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -9.3320999582515363e+00,     2.9970527704740412e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    7.1716301247543456e+01,     2.9971232336656558e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.0358250274584190e+01,    -7.4186965583522042e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.0722665466846976e+01,     8.3288818803343202e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.6011556366073364e+01,    -8.3285009689125818e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.2853813980666835e+01,     6.8854762454792162e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.2078889878323249e+01,     2.0587592546176777e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.7957422437986246e+00,     1.6685768782657156e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4420658746356829e+00,     6.7025605553437049e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6126559024905216e+00,     1.3760003243773233e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4829243365785931e+00,    -1.7271884007961978e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5970444297039236e+01,     1.5599842188645376e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.0404547277137716e+01,     3.8788193619482903e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6342191798547603e+01,     3.2029284873689994e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.8842792989263344e+01,     4.2430485732945544e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7365084552515908e+00,    -1.1338162290211358e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.8958891674650564e+00,    -5.5885085015393088e-01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.5315687185673625e+01,    -1.2107667937504220e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7785465000709220e+01,     8.5379122579849791e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.0078766794447237e+01,     2.7975474537240607e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.2039142485121346e+00,     1.6459274742158399e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2510422281961542e+01,     2.2521530539987587e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.3437042921571951e+00,     9.4897053902465523e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4236198539114135e+01,    -1.6804799165110759e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.0015151810764861e+01,     1.5498983456424834e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.3013423230783971e+01,     2.5315926564976562e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.2381420364181402e+01,     8.3745398009641140e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.7823893629749037e+00,     9.0552147869380115e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.8776575721590142e+01,     3.7850945810074300e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.7354879828342153e+01,     3.1593159462161006e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.0845869665256640e+01,     2.3906782785541967e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.2158160497047426e+01,     8.4219761696348742e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.9383868336310485e+01,     8.5797105239753037e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.4387744002134774e+01,    -6.1257510104624453e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.1643075578409736e+01,    -1.3671770245940479e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.3372577380846437e+01,     1.8190971364146135e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0529039155779021e+00,     1.4048222672221968e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1466319860010277e+01,    -6.4181409631689723e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8567426081072654e+01,     1.0392012050357403e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.5175701316485402e+00,    -3.5709701546920478e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7007272767372719e+00,     5.7195312475985183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2843486714076136e+01,     8.5385695227345241e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.9974919191488254e+01,     2.5024771474067951e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -1.9632938422767936e+01,    -6.0576567991329921e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7653470200384291e+01,     2.5085401068740626e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7750691338723399e+01,     3.9742786624466164e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.3828752443663306e+00,     7.2272170606716236e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0742133634671962e+00,     5.3504700181995668e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1379921855827400e+01,     1.8008115385253718e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.8569425421092102e+01,    -8.4819175577723472e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.5004955527735087e+00,     4.2140690310324624e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6521342249348532e+00,     7.1500731414247894e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1603721482443650e+00,     1.6028363918734482e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.7127104559331869e+01,     8.2660596019707264e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.0596682939650321e+00,    -2.7876749255569694e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1338983076661385e+01,     1.3764481074734424e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7272284179260637e+00,     3.9559316590512026e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.9819215812449627e+01,     1.4286145833441435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7321360146131802e+01,     3.9946440467437315e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.6916691928164319e+01,     1.6520665146787494e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4144568696820059e+00,     4.0612191494528997e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2807295046430976e+01,     2.4502383131966973e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.2894712852352537e+01,    -1.0359651578703435e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.6378810833163122e+01,     7.2951289157254905e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2945046647839032e+01,     1.5847031171899362e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9580212406770158e+01,     1.5511575886466717e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.4435592665763206e+01,     2.1385716868046684e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.0854701058731692e+01,     5.3536365252093674e+00,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -4.1237172381502312e+01,     2.3966617546689500e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.3556557466727874e+01,     5.4582429177582526e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4481881695981812e+01,     5.8622758631830038e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6236376151447622e+00,     6.7086141555427776e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6539577682462976e+01,     2.3236579173960567e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    6.6185001083138939e+00,     6.6703635667573317e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1583458674009968e+01,     8.9483625112861591e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.7865446784273846e+00,     3.3970808088965896e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1571422630052439e+01,     1.7735890666693451e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.9636310590208348e+00,     5.4662109483052536e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.4129146140142872e+01,     7.2009086700450653e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1810260232913281e-01,     6.5813460531425435e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5885624149230278e+01,     2.2528089495586715e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.4848116306180259e+01,     3.0616108286551622e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.4088343695993315e+01,     2.0924863497414703e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.9553244358182962e+00,     5.8635911455671589e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0004832366244329e+01,     8.3255781790421388e+03,     4.6351172359218639e-04,     5.4853225840408113e-07,    -2.5876930229221479e-09, } , 
      {    2.0949038956176977e+01,     7.7064173140206585e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.1246283578967535e-01,     6.2815916798832927e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5391238044669588e+00,     7.1993443569325973e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0960419776164494e+01,     6.6710749122242087e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0024921923022914e+01,     1.0160645082548888e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0299389719999070e+01,     2.4437396492620628e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2108578843548003e+01,     2.3878004670204380e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.0193731928909063e+01,     7.2846639888440050e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7282184597038562e+01,     1.4112657022035824e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9371597296043905e+01,     2.5164472144467680e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    6.6108875061691039e+00,     8.5912099875709428e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.1589543060538148e+01,     1.8025271611295466e+00,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {   -1.9526598842976952e+01,    -2.9710639236538009e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.0302311189328847e+01,     1.4270597091602278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3992885030427033e+01,     2.3051063496913957e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.5163577386980842e+00,     1.5793788648848802e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.1774330559379280e+01,     2.2615182860472232e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1529500429907728e+01,    -1.8330409043532136e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.8250638111467758e+01,     1.4896005855022322e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7121519965189851e+00,     1.4935065267447228e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2276788086735515e+01,     4.0200241943973015e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.5747803363601417e+01,     3.9361230907375844e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.0718014996583076e+01,     3.9397279323779447e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -5.7130751454272584e+01,     6.6411122104991427e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9233613529431262e+00,     2.3119185202674431e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.7311672097058405e+01,     2.3816088784418906e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6432387206071448e+00,     1.5025780393403536e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.3281897761085013e+01,     8.3174519023005341e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.4313388539649992e+01,     8.9630324841117454e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.4069657422012405e+00,     1.3295962160291156e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -4.6078016875427871e+01,     6.6809271123970557e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.1602153103032265e+01,     8.1123439183682203e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0038119196838888e+01,     2.3908205476475719e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.8132852854742367e+00,     2.3242426784092826e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2323838561756975e+01,     4.0135807594159152e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -1.8778244086521801e+02,     1.4914954663575367e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.7978158200010022e+00,     1.6866708657825489e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.1138583114396820e+01,     1.6867155972140910e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.2459310177994272e+01,     8.4395264403104557e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.6192850286816977e+01,    -5.2548115638182935e-01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.3286804149205247e+02,     3.3361035644502408e+00,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.1874912907872162e+00,     8.9531857178558566e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.0056181691551060e+00,     9.8397380343904388e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7477766701428912e+01,     1.5494052424167241e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    7.7453818571702566e+00,     8.3007501170934938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7425244488228536e+01,    -1.8760288922738511e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1538185824006533e+00,    -6.2450481543621697e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1357755150290700e+01,     7.5809391338841706e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1711843046793531e+01,     8.9324158458680595e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2006503414285685e+01,     1.5543857817376005e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    1.9128864338774562e+01,     1.3122495321951425e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2436706961271886e+00,    -1.6592948499433045e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    4.8896428397118630e+00,     1.4555207162698425e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4223313739458742e+01,     1.4508324789990962e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6744372002123447e+00,    -7.0786805069933598e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4641461393863096e+01,     8.6627526680951778e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5928399913613934e+01,    -9.3982692338502693e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.3201212491196195e+00,     8.4831295549267397e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.4326987078463027e+00,     5.7221610799147029e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5113874838915862e+01,     2.2066376420422152e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6208790589432422e+01,     1.4394344303261529e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    1.1738672185158991e+01,     1.5995299466577679e+04,     3.3816327433212770e-04,     5.3722543373324450e-07,    -2.5921096755570561e-09, } , 
      {    1.3338683956748593e+01,     1.7461452842742103e+04,    -9.9761634931308941e-05,     2.6330037095586454e-08,    -2.8787751570603142e-10, } , 
      {    2.0680272786526562e+01,     3.1747603222295773e+04,    -1.6061704806932556e-04,     8.7214975386782646e-08,    -5.8701725322423728e-10, } , 
      {    1.8324718236139759e+01,     2.3418911797672321e+04,    -3.1290945583165172e-04,    -1.6350389876787206e-07,     6.4896674539641355e-10, } , 
      {    1.7737117280091027e+01,     2.2790598582795297e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    8.5396004441455560e+00,    -4.8703231968643883e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.2889613394601852e+01,    -1.7429428578613756e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3237524898222340e+01,     1.6064412335138079e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6337478006283268e+00,     7.7210024020888814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2944550472974878e+00,     1.3763058383800671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7584821724492130e+01,     6.5871779712465532e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5473551610646869e+01,     9.9017695959955290e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2678497498563134e+01,     1.5525893543334229e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    9.3085700853522830e+00,    -5.5660293416567652e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.6774024962512129e+01,    -9.4433266974392791e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.8565224749572558e+00,     7.4067515844470799e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.7386847994153243e+01,     2.5616683058857303e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6414015345523325e+01,     2.3230020218361435e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.7057822457095220e+01,     1.6791681253912488e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1769233763122525e+01,    -2.4980040799427596e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {   -1.8258445951297752e+01,     8.2488379636699156e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.0689065981295883e+01,    -9.4433132703378287e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.7847836435590239e+00,     1.5555205809459105e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.7015482741665160e+00,     2.5221360889222902e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.4521447011478639e+00,     1.6882931823207200e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -2.0642928641576805e+00,    -1.6850932806759207e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.5350702511098369e+01,     3.8863444680553614e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.0350510915041003e+01,     6.0971331061518285e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.7284233350041866e+01,     5.5282146197112761e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5718574015265929e+01,     1.3957552500011578e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8989558437549064e+01,    -8.3306557722929265e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.8073448248673010e+01,     1.4053153704479562e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.0283402961596408e+00,     3.2222679044360939e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5902654978211366e+01,     2.4992107748313101e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {   -1.9366096214953920e+01,    -3.7728848569368893e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4645349648792063e+01,     7.3380157922436156e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.8028624946360257e+00,    -5.5802562509942913e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6107459755225413e+01,     9.9848171101077205e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7477282811955732e+01,     6.5084949017778717e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9509734793884004e+00,    -2.4514258226845000e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5227328576116925e+01,     8.8767999503947212e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9237517499143308e+01,     1.5824441571207499e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -7.7951928750434703e+01,     9.9972666735048497e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.1321409417014664e+01,    -7.9982119753899383e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.0812865392981355e+01,     1.5541229036730576e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3522650548546585e+01,    -1.6880050226181469e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    4.2846209545999425e-01,     1.3072573441938139e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -3.1040089040002147e+01,     8.3993324140783643e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.7977942346991874e+01,     1.0308786011844333e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8773349658899470e+00,     2.3265680034691948e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.3749382917020142e+01,     7.2170450136326044e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5863402689878484e+01,     4.9979389175415527e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.0114189519503014e+01,     2.3167887068488992e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1705696461721580e+00,     1.5271049853540224e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0748860488819185e+01,     1.5531733663259431e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1544173208971202e+01,     2.2836999248668450e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.2186197689857586e+01,     6.5677063368569397e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.4748829725290172e+01,     1.5544279543233013e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3748504753024342e+01,     1.6650771449694796e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0221260279931897e+01,     1.6188334871793006e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.9076100774413639e+01,    -1.1496701407455021e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.9051601634047206e+01,    -8.5555138871254203e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1941184180046475e+01,     3.2373638188561399e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -1.4238980472034591e+01,    -1.5640079862269062e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6585962823470368e+01,     8.0109215160011736e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.3987815177221847e+00,    -5.8869151296420654e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.7613947135043566e-01,    -5.2353285217739244e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.1711116390921168e+00,    -1.1584192870634633e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8395667500581183e+01,     8.3192489103849148e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7363450485021348e+00,     1.2871789595459191e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3526685867861886e+01,     3.1779077819978047e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -1.5291101229291836e+01,     1.6648522693706356e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -1.0053106483951236e+01,     6.6544473857843823e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0473350251448203e+01,     1.5527006546467344e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.1224020046792901e+01,    -1.5836765367190001e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5000447384250815e+01,     8.3285506373504377e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.6456029121238487e-01,     8.3288322118964625e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0229677763071063e+01,     1.3817450171945615e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1427411373569827e+00,     1.7964640193440711e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -5.0305666336033838e+01,     9.6804784267987907e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2234402892364704e+01,     2.3857939203333892e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.3345345459881528e+01,     1.0156526281096500e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4713066989622259e+01,    -1.6666243004787444e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -6.4547088716582039e+01,     1.0665389160448305e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1637541914077573e+01,     6.8886381202694511e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9005475504366864e+01,     2.2050957309282476e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1832332748246539e+01,     2.2599693512156591e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2132566353077600e+01,     8.5170787013225781e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.9439574517468706e+01,     3.9952336090202472e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -6.1499582277820252e+01,     7.5636661371185019e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7329387919661336e+01,     2.0949446661215632e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0312581764277208e+01,     2.1671054250787754e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0617450809439218e+01,    -7.4962488193643285e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.8230735068764243e+00,     8.0596233818157334e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4684871444201018e+01,     1.5387649767957548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3029111783335654e+01,     2.5084290051139371e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -8.2685265287733429e+01,    -2.9429656316304893e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5837056715264133e+01,     2.4731642834194448e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2017297274915101e+01,     6.5224769018461220e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4123717228659075e+01,     1.6181371156282825e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.2867102764904921e+00,     4.2407473733531442e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9889240568492355e+01,     2.4486635388452523e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3832596591788882e+00,    -1.8231751814136436e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2245825157349808e+00,    -4.7976201862042853e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.6212909398932229e+00,     1.0664953988925907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9219687999783606e+01,     1.5771481949735216e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    8.2164746982628856e+00,    -1.0438911389010831e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -7.2175320304703149e+01,     5.9574544088179846e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0608750780471581e+01,     1.1395306921155227e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7570733132883518e+01,     8.1240577316295921e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.2971375010081911e+00,    -2.5737149616254101e+01,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.8202044483577062e+01,     5.8177622843826866e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8061780676718325e+01,     6.6563962697677189e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.3185541645497928e+01,    -8.3103291083079675e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.1909738571818197e+01,    -1.1740174010432241e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3009747647085948e+01,     1.3347338533066772e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7748742508017365e+01,     2.4457093483695084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.5174911799936215e+00,    -9.4433199838885539e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.8871504208196477e+00,    -1.5510452202424858e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4146760523115688e+01,     3.0457912340384923e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.0027370217504883e+00,     7.1986330114657212e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2972077789961205e+01,     6.6571009016838671e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.1121599688177326e+01,     1.6942183376053912e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    8.1135994117286820e+00,     1.0149624455826524e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1882496073819370e+01,     8.1054383884626250e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8169726019787333e+01,    -2.0678775772048539e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7846242025019849e+00,     7.4055786471539368e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8082085082282461e+01,     3.9374385102689121e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.4604252682499709e+01,    -1.1491722854116972e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4183350520874463e+00,     8.3286809024196391e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9534957147758867e+01,     7.6983062967942060e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5311900949596033e+00,     8.9881774131840721e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7176116691769714e+00,     8.3287019468272611e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.4769218171457737e+01,     1.4969803635122595e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8318628223899871e+01,     1.8875412162965091e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1841999383117525e+01,    -3.6462212157627619e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6525641397276580e+01,     2.2678820268447715e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3123458244031445e+01,     1.5553774916704158e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4363449423482749e+01,    -1.3218768250599969e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.5838707974945368e+02,     3.3528737934175297e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.1070282254644312e+01,     8.3110661118997068e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0227150630808346e-01,     3.0927843766463241e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3331291132968936e+01,     1.3443214997067174e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.8697080608521688e+00,     2.4977437775487517e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    6.9297233837426564e-01,    -2.2341728912726530e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5352503425112491e+01,     1.5179292190992208e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6832030069381723e+01,     7.5913513510520352e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.1704460640655050e+01,     2.3454988795237416e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    9.7148520313184719e+00,     2.5746379063640070e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    9.4753010323833511e+00,     7.8600024766783572e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0174578176114689e+01,     1.8735553668675323e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8726704950937840e+01,     2.5947884251537471e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.9400011261573326e+01,     1.5193151018221637e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7911500388738446e+00,     7.4299005189043186e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0411614524945207e-01,    -1.0327577700543548e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.6636307052320323e+00,     8.2309202732347349e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2339815402692311e+01,     7.1191452976583250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4799192971136058e+01,     1.4329510281313287e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8551967351875305e+01,     3.2200130425677969e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -5.8911052172099936e+01,     3.2200143852779423e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.9581756669795869e+00,     4.9361191405287282e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.3689655884863491e+01,     2.8628336119423302e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.6906456101689180e+00,    -7.5729897943987680e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.9155657248367341e+01,     8.2560614976714387e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6225970760158944e+01,     1.8776564902461223e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.7165489988744625e+01,     2.3884952225876597e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.7624440170410569e+00,     2.3825045048732100e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.4240134907558684e+00,     4.7740590229269088e+04,     3.2451087103931128e-04,     8.7639206750300724e-07,    -4.5239903839055236e-09, } , 
      {    4.8380550469754567e+01,     4.1550855674013292e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9963048914366297e+00,     8.2930351901471822e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.9322989706549706e+01,     6.5614824067033187e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0905855560219621e+01,     1.7936539924470231e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7711257013873073e+01,     3.9454014906484954e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.1640689952391782e+01,     1.4722372072290504e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.9229151256119152e+00,     9.8827263599395374e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1333983234598959e+00,    -8.4400251130907345e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.3255988691544633e+01,     6.5722487690414473e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4509631485538634e+01,     8.1022212780795553e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.9153861107609531e+00,     7.9537041388711732e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9969956687870587e+00,     3.0914241424491706e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2256229723584141e+01,     1.5472924446123037e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -1.3035404092727333e+01,     3.2182864142463026e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.1528070629981352e+02,     1.5189712330354837e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8443091050689510e+01,     1.6015957353003090e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2444317764999678e+01,     6.0008188566898380e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3700442818448288e+01,     1.5009618537020508e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.6843292085454486e+01,     1.8071526658452400e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0579265782827651e+01,     3.9989538492421387e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.6545089663930476e+01,     2.3796194653534534e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6531871967543651e+01,     1.5543458921897944e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7636134704241524e+01,     7.3096640698263291e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2223295459655425e+01,     3.4165241987693633e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.0074162237135546e+01,     9.4275722403741056e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2356458188360925e+01,     6.6291264417353814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5542659733573867e+00,     6.0950250789229804e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1046510167035208e+00,     1.2478768625766203e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4634028745956442e+00,     8.3080728621124745e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3737047551513108e+01,     6.0994343060932406e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.9202499571587728e+00,     6.8367896391798085e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8103548903084015e+00,     8.7062932831767575e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1735307842260028e+01,     1.5620751176874221e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.9933247640736536e+00,     7.2981740819696024e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.9951229279477289e+01,     7.1805901760288293e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2760442067337273e+01,     1.4982810178033058e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.3251485182417056e+01,    -9.2971372151801688e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6020512243136718e+01,    -3.7580949644519706e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.3402470594964342e+00,    -1.6396467336822711e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.2785841148933706e+01,     8.9696182480869793e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.8044412278527606e+01,    -1.7696627497788199e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6692055775056144e+01,     1.3595698384972718e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7564607071732929e+01,     4.0148512254357320e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.4562458591399917e+01,     2.1515621096388728e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2770914939511208e+01,     7.1978535300176300e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.0189275700787546e+01,     2.4298248111753397e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    9.3304461400274210e+00,     3.0417386093020134e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.4524713517190364e+01,     7.2110807170840844e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5824446707632536e+01,     3.9411570172270855e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -6.6363024660211948e+01,     1.5542049658065645e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1053033871754330e+01,    -1.7766875616032303e+03,    -3.3728746520459481e-04,    -4.0348870713455303e-07,     1.9445343454254264e-09, } , 
      {    1.2465120404059206e+01,    -1.3579350721254687e+03,     9.2645799580116261e-05,     1.0925586584559303e-07,    -5.2301699918096742e-10, } , 
      {    9.7434199621647366e+00,    -4.5256955692698193e+02,    -4.0424851809237717e-04,    -4.7515134444502282e-07,     2.1833632610136059e-09, } , 
      {    1.4464891643345981e+00,    -8.8522486571319387e+03,    -3.6849304853755181e-04,    -5.0682659781894627e-07,     2.4418125862762885e-09, } , 
      {    1.8803682048859457e+01,     2.4013481086093419e+04,    -2.8485462473716171e-04,    -1.2639616265301197e-07,     4.2140005162040839e-10, } , 
      {    8.1865152034086819e+00,    -2.5470248883054646e+03,     3.6340272663967943e-05,     3.5127660078472540e-08,    -6.6429170585628593e-11, } , 
      {    1.8233875388099296e+01,     4.0076319027250327e+04,    -8.4225726705834886e-06,     3.3797748277273724e-07,    -1.8222740313232240e-09, } , 
      {    1.6578107565202423e+01,     2.4675540088431542e+04,    -3.1833515118105291e-04,    -1.6222748130824690e-07,     5.4081450941449824e-10, } , 
      {    1.7340580264066990e+01,     1.5019197057473302e+04,    -2.8238381689905929e-04,    -1.9399000114477001e-07,     7.9780939363385223e-10, } , 
      {    1.7085032528420534e+01,     1.3833580822626689e+04,    -4.6510393123039382e-04,    -4.1426640615382669e-07,     1.8842235234954001e-09, } , 
      {    8.1238192836573084e+00,     3.7230628457444254e+01,    -1.5723123236616884e-04,    -1.8259591001133372e-07,     8.0368048731208867e-10, } , 
      {    8.9416349843538256e+00,     1.5752303755718098e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.1195676399090964e+01,     1.0045391746537563e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0390143539858602e+01,     2.3298541541114537e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8182483934596302e+01,     2.2650878960917758e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.1692635270643112e+00,    -1.4043601183297151e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4970368765791717e+01,     2.4003442919329424e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -6.4592905374303164e+00,     1.7642293582896418e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6086799097407116e+01,     8.3685331348974651e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.7096436985288399e+00,     3.8477264036601555e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3207457935290291e+01,    -8.4492233767449616e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.1494473537145311e+01,     1.5894088530219191e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.2727543703813396e+01,     4.7745192629151890e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.6320014470325624e+01,     3.2567420959305429e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -7.7148898882268554e+01,     7.8180975767944055e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8203589499817866e+01,     1.9920398795598339e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4926350499321433e+01,     1.7303714642494240e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6860512819278561e+01,     1.4988069842665425e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.5678923860261040e+01,     1.0388415815640159e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2912035354971971e+00,     1.6186809618541789e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9899014729017701e+01,     3.9416829836903220e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -7.3091724033310541e+00,     3.4481464783680340e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0801150846144996e+01,     3.3772221018000651e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.6832859052010061e+01,     4.8402045632011232e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.8181818866197002e+01,    -5.6423777244711175e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6780977477170929e+00,     8.3493099871344257e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.8841541525701828e-01,     3.8783591219600086e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {   -2.1294450343987108e+01,    -8.8506798016313013e+00,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.9687979365630049e+01,     2.7959389765875552e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.8698357594426405e+00,     1.7348047241322602e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1373996252159557e+00,     8.4772369910485650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5075424065539806e+01,     3.2802707574657987e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    6.3604278985318544e+01,     2.4298587417755471e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.0524098115536177e+01,     1.6171068588578062e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6094143794380269e+00,     1.4896422496734453e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0378172345295660e+01,     2.2948891787985885e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3711853265128411e+01,     9.4210132847749701e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3043140638111016e+01,     1.6561822875369111e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.0627345659076290e+00,    -1.0522203809804065e-02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0962829915260741e+00,     2.3639556180061591e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.2831840458192119e+00,     7.2103649836549621e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3268206731384189e+00,     6.5994677427888280e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0375195979125337e+01,    -1.2880953976048368e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.6351313972921631e+01,    -7.7184327830337716e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6103980294834948e+01,     1.4347943637388382e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.4171811917302719e+01,     8.7374448783157368e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -9.7796216967701550e+00,     6.1288847390586579e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5295073386391103e+00,    -1.3117265055240987e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3603017425731640e+01,     1.5554751900725269e+04,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.3642214301457088e+01,     1.3484349436990282e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.7939277471552984e+01,    -3.3443319341476851e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2732988569504009e+01,     7.0405960270186151e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1460287725427154e+00,     2.3856026603465565e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.2663610751007610e+00,     1.4948228132391412e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.5340934055873845e+01,     7.2627647311728997e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.2512215158342318e+01,    -7.8056731579337793e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.8768254123715575e+00,    -1.0228207937459209e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.6533242977910831e+01,    -3.1029019144970960e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3180908588591054e+00,     8.3896940977654594e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2730062566797917e+01,     7.5890501511106231e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.9525210069215704e+00,    -8.4264625760121671e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2476418541881001e+01,     1.5622314755635587e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0224622259918750e+01,     6.5895580499221278e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1858746988790031e+01,     8.3904959421211242e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0654139260302538e+00,    -1.7824110071649102e+00,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    6.6805086265024585e+00,     8.8161159262372184e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3553826206376634e+01,     1.2856041851944738e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9891834637902015e+00,     7.2452412688734239e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.1539746593376396e+01,     6.6173944695461405e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7080457388520450e+01,     7.2074514658062417e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.2627689579925327e+00,    -1.8204884053227729e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4028839626244089e+01,    -4.0340383054294080e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.4164715372661163e+01,     2.0967495604671494e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0389763913635667e+01,     2.3221808221581850e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.0893567030292662e+01,     1.4900940193664897e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7935851081278486e+01,     5.9419056669788279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7591628471298382e+01,     3.0459830827234233e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.8282786923348500e+00,     1.2085747656073214e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4588135503495057e+00,     1.0027903107574936e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9116338276596586e+01,     3.7513529505935956e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    6.7081299384897228e+00,    -3.6108350563623446e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -4.5015307977995995e+00,     1.0275747329632526e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1753961750018135e+01,     1.3483029898965086e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2569374575486975e+01,     2.2614830544514160e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4487151589468940e+01,     3.2243498508722078e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0523079675841949e+01,     8.3325793173162019e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.5234188776615547e+01,    -8.3248035319306982e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -4.1289189588094530e+01,     1.0642847255316061e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1795776925292046e+01,     3.0159781229693046e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.6981923140661394e+01,     1.4890173831326769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2175663326068310e+01,     3.4229676337507488e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.0209650220718650e+01,     3.0202769555242143e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.9004399046534832e+00,    -3.1031625857156537e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.8368837512542495e+01,     3.0791262238389776e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.9377931386526043e+01,    -4.9377458083195052e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2304723907331072e+01,     5.7064133364002537e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.7074168044529699e+01,     8.3021728080272460e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3505474511773654e+00,    -4.8737760593273848e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.1031724547466393e+00,     8.3533108725393940e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2817017194810973e+01,     2.3316432634914101e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -6.7062411050355735e+00,     2.5534752055120069e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.7155684031339433e+00,     1.6653580079637573e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.2770927414886017e+00,     2.5240505713546248e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.3131502171702756e+01,     1.6661185618856227e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4004117544700343e+01,     4.0774426445992038e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {   -1.6939870815192140e+01,    -1.3264792249428174e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1457832963992036e+01,     3.1339940019639493e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.2058353199655674e+00,    -9.2790266739489925e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -3.2863747179172698e+01,    -1.6674655846012571e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    2.7413275070948412e+01,     2.3855236379264526e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1180415206509363e+02,     1.4913938746297134e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -5.2392748374858648e+01,     2.3926802644791584e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.2955589903706787e+01,     4.0738025713630370e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    5.8113805520377582e+01,     4.0738730345546530e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -1.1133799304272308e+01,     1.6404047290438481e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7273685174233112e+01,     2.3455014518404467e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5646927949091849e+00,     7.8051585724460574e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.8082825990549701e+01,     1.3431284057889570e+01,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8979903247371283e+01,     1.5515524327918714e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3837023044801075e+01,     8.3338119972605855e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1598299996554093e+00,     2.5017252677385426e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -6.9491624298680378e+01,     1.2721571983795166e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4058152459698974e+00,     1.6685716338704187e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.2216657431854621e+01,    -7.7959850440458940e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.9711638872540647e+00,     8.8620814419689741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3135599648026037e+01,     7.5965755091108185e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.8216928128707002e+00,     7.4373159015191695e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.7966524372551911e+00,     8.1703982194328637e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2914296823821346e+01,     2.9589924407926560e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9081356558450224e-02,    -2.1202093704891854e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0333017115657050e+01,     3.0614935349258474e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.7879610218063036e+01,    -9.6722011804449721e+00,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    3.0321087970510163e-01,    -2.0082046416682715e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3719128351952961e+01,     1.6498123241655252e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.8744531701077207e+00,     3.2826143524332823e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    8.1358432944425836e+00,    -8.0199059274272457e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.4404394028874599e+01,     1.9264503235685152e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.7891995568063521e+01,     1.5511274062468045e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.8775429719680737e+01,     1.6658349251647964e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.5845016918298377e+01,     1.0900206602101098e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5070308537310009e+00,     6.9581127337488952e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.8078000261420213e+00,     1.6752942823124689e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.7791365030667521e+00,     1.6667692130502390e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.2596013724227184e+00,    -1.5905122337316085e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.5336487684940089e+01,     1.8230786266044044e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.3400455418005393e+01,     1.0705204062346218e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.8369293817576157e+01,     8.0898020132359761e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9617819316766472e-01,     3.1711514965067719e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.0068416233637770e+01,     1.6647073567991411e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0744084099361512e+01,     8.3537192143046850e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1249709329822259e+00,     1.5734270071777388e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.7650926240120484e+01,     1.2793908702072295e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0758601154485703e+00,     1.0668768299325513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8441792696131955e+01,     2.4100173374358670e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.3245820630907641e-01,     1.5530756679238322e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -3.1394498622963582e+01,     1.7126430826700795e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1132419243019896e+01,     3.4547054339671686e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1538407285883167e+01,    -4.8206321856184150e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.3592018290799643e+01,     6.1025864158968034e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3319961485864027e+01,     1.4436762229099524e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.1431206892605687e+01,     2.1335685909150470e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4644745365802052e+01,     1.5025254912247156e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -7.1245999264827418e+01,     1.0331327916976577e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3603272781877971e+00,     7.0125576774609508e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.5626960854376541e+01,    -1.1239522322915491e+01,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.5134458825333626e+01,    -2.2266737693456889e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.3055135929916858e+01,     1.2951643056412726e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2549969550093564e+01,     1.1648627220448492e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.7450892151622535e+00,    -1.2205667536874885e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.4297761689505791e+01,     3.5563674620290931e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4934855304640148e+01,     1.3986017809170671e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.1543416578633114e+01,     6.2361520085526436e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2030987461277377e+01,     3.0474121675725641e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.2487497981531106e+01,    -7.1089400048767857e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4914800623507322e+01,    -1.0419986323130954e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8129181155861218e+00,    -6.9587043995586396e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.7884762005348403e+01,    -7.9717530823270317e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.1247401066599309e+01,     6.6590497856672046e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {   -1.4271179742741605e+01,     5.8778178914714335e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4000829214129702e+00,     1.4642742268494680e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.9913114333392095e+00,     5.4357896501109744e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6528421966112141e+00,     1.1061789269018500e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.9992691724077680e+00,     1.5543108893431609e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.9934729022623973e+01,     1.5542399686531979e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0810246369505322e+01,     2.3843059781194985e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3871653478780351e+01,    -1.2505816956483345e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.6815990147888815e+00,    -7.7152588877740357e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.3333152121402172e+00,     7.2102495735452094e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8458754872845034e+01,     1.5056433315762231e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0251446365542190e+01,    -8.1245564044098719e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.2521602450740015e-01,    -3.7664854943142529e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.5062675538127181e+01,     8.5689090722336005e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8300625179551503e+01,     7.8423637368531608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7696504236631228e+01,     1.5591456155796352e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.4359849048380013e+01,     1.4851168326469573e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6856648519752202e+00,     3.3131294194537113e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.2670057455198640e+01,     1.3719636052342838e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.8894468719901631e-01,     1.3513215091702999e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8606589740919613e+01,     9.0989856204949738e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9097730678937701e+01,     1.6645892861390173e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -2.8519948880484911e+01,    -1.6668872837103627e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.0230514149331116e+00,     7.2020652546148740e+03,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -9.6114358573099921e+01,    -8.0939704071521845e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9262466565046648e+01,     1.6512267733324454e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.3572753195969152e+00,    -9.0437495343353457e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.4077909093653389e+01,     1.4464174147120289e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {   -8.4855683542266327e+00,     2.3311830235031281e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    5.6459359898574260e+00,     7.7805659329062155e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4474258319704266e+01,     2.4821780963930785e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    5.0995044806128540e+00,    -2.5212544420414244e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    2.5093406419987282e+01,     3.8210553931707145e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.3264242804976821e+01,    -3.2532002460697740e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4900755548187455e+01,     9.2386862908946969e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.0258414878169528e+00,    -9.0683326981362789e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7130317629196014e+01,     1.2094607811613761e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1360579060220779e+01,     1.6875810703483745e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.1685769925334064e+00,     7.1676621994851976e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0175454420403417e+01,     7.7720884904577742e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.9006364417133033e+00,     6.5489955184423270e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5221712162359118e+01,     7.6519324392971612e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8267790929574726e+01,     2.9513424701191641e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.6275998072174719e+01,     1.5403034668104252e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.3582330128940448e+01,     1.7461458472573584e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2160505027530087e+01,     1.1168955482135432e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3341786296690783e+01,     7.7130030779958934e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.1172845403233794e+01,     8.0192829987614414e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -1.7748515162147484e+01,     8.0907684156370397e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -7.4703458679546870e+01,     2.3847172840995765e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.8828897544724924e+01,     8.3083412812261777e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.3540006645498522e+01,    -8.3490415680207225e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.1042824028736007e-01,     3.1083431037179889e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.3027104599226682e+01,    -1.0766362338128516e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3729062537814238e+00,     7.7706657995240203e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0701392339815825e+01,     3.0870892042185595e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3972159444512863e+01,     2.4603364570964706e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.2587596234417660e+01,     1.6816642456838548e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4102200039313137e+00,     8.9349577562242648e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2908014980958953e+01,     2.1872735526192264e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.2715816985757087e-01,     1.1454810238711489e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3074866704381591e+01,     3.2948976676395752e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.0858451666541997e+01,     8.3155285596792655e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.1473425657683922e+00,    -8.3418542895676364e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -8.2862956795665568e+00,     1.5540349353160873e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0860940701716002e+01,     1.8330429215538767e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.2794628218105970e+00,     1.6240951653046122e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.2786255224085323e+00,     7.7857034714893143e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.7495771590319311e+00,     6.3211035465486896e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8624707887939884e+00,     5.4887587473221656e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4236966734379925e+01,     1.7286401779759319e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5439961594059483e+01,     3.1087586122747289e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.1902463116358240e+00,     1.0770893159465368e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3645532752579086e+01,     3.4558262957386622e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.6819810893915736e+01,     1.8826215551952191e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5834506968503918e+01,     6.0164687846091010e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2286260778420150e+01,     1.1832098724405178e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1518243961739255e+01,     8.5070892952207760e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.0963689245804762e+01,     8.0155239158839977e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0773150016738427e+01,    -6.0066482338620954e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.3682735131488535e+00,     6.5504182093760801e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5640142677190269e+01,    -1.0501942094512490e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    7.0628294507518874e+01,     2.3147966297475446e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8947094348964608e+01,     1.5901993832219623e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.4794013288502805e+01,     1.7251903377005801e+04,     3.3283551134631064e-04,     5.3845821796156986e-07,    -2.7009891320606354e-09, } , 
      {    1.7680043720213348e+01,     1.6066287142159186e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.2139744299375584e+01,     8.3658976727497975e+03,    -4.8408922402585326e-06,     6.8079330912021156e-08,    -4.3303073183022644e-10, } , 
      {    1.2638893738600165e+01,     2.4533540356937432e+04,     5.7858535816834269e-05,     2.7581612702191584e-07,    -1.4149820578231025e-09, } , 
      {    1.7306875755253841e+01,     4.1819225161398390e+04,     3.5977946984857813e-04,     7.7787026744538796e-07,    -3.9413897833161944e-09, } , 
      {   -2.3428889156569506e+00,     6.5916553343980313e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6621237717547444e+01,     9.6213550111181357e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0262072167858207e+01,     8.2958763995693516e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8769554251915210e+01,     6.5906863125703967e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.7233124489457907e+00,    -1.0541027862346067e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1393658659436742e+01,    -1.9807473739472573e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.9044276166311136e+01,     1.8994080585071755e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5882236718748381e+01,     2.3450386395354595e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    9.5642252372988054e+01,     2.2128502856743871e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.4476520162617632e+00,    -1.9524856100469503e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0229955561085440e+00,     1.7782732152473704e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4448435719646380e+00,    -9.4831348857864668e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2679030891591040e+01,     8.2981277294293841e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.9102603222108652e+00,     8.1371756428278577e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.1321895733893157e+01,     6.5992617915841556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4375872525892266e+01,     2.3147578155681913e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.1828689368030059e+00,     6.1217410052068235e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0238644929978742e+01,    -1.2439959316730992e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.0461656295002669e+01,     5.1941533762395484e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6322300240038924e+01,     3.1106838487941441e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.0949764469578909e+01,     6.1017355060346490e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {   -4.6157837531977872e+00,     1.0350680936113131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4883460158136025e+01,     7.8420418180291144e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.9243415906798404e+01,    -6.2775529551305954e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7477798594700829e+01,     1.5074065342036702e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9352193741689653e+01,     1.6548350360721026e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2578967402393488e+01,     1.4918249474545579e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2700183701186294e+01,     2.0714160045863078e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.1420021645305525e+00,     1.4338396871635023e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5006349987739362e+01,    -5.6888783252398225e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6802996393928730e+01,     4.5480653571283972e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3745776533949332e+01,     6.2752622785344056e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8676585895634901e-01,     2.3599741278163674e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.8627444026821792e+01,     1.6657284797673063e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3023181149322176e+00,     5.9993961657560849e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9327362132980228e+00,    -3.0977670916707521e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4461995831692411e+01,     9.2659087627507215e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.4691651355245838e+00,     6.0207260737339366e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.3547967176011717e+00,     7.2144174688081603e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.0747443043393720e+02,     1.6657480900820738e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.3408961864543603e+00,     5.3602917053606925e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.7095766554034100e+01,     7.3905954697156612e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0355535619506036e+00,     7.8363369759611360e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.2554818535654224e+01,     3.0616844642741817e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3149220267607413e+01,     6.9269611386099186e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.4966405104470457e+01,     8.5328264448370282e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -9.7937548967257584e+00,     1.7834368215542163e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0825067774316377e+01,     1.4349417498357387e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1744978345181698e+00,    -1.0622382892967516e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.4318012403588298e+01,     1.5894344246504602e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6561191560669819e+00,     1.4328087590379535e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4146377731727702e+00,    -8.2669128782008211e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.2098868125700122e+00,    -1.7609698282513584e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.2020740826591048e+01,    -2.3758781529396583e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3192568868948875e+01,     1.6833862232994918e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.3552272721230640e+01,     6.9044073491798918e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9408583442342895e+00,     5.4343669591772214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4377619312224407e+01,     8.0006636555944742e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.6079422944129416e+00,     9.4547947122301321e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.4830545671003982e+01,     1.7550270675579668e+01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.1780866975575890e+00,     4.9492370517269937e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8231223462836077e-01,    -2.6271938609656409e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.0282510622063326e-01,    -2.5002994954677950e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {    1.0120930714673388e+01,     6.0530766084341521e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3508567125738423e+01,     1.2245783210355055e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7579148581972110e+01,     7.2137082619085268e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7948099702081564e-02,     6.5731360418948143e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -6.8641301242344888e+00,     4.6337683430461302e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0549087007539583e+01,     1.3515516291644411e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3849326783257213e+01,     7.2178761571714795e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9528666976091898e+01,     2.4342018945310356e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.8930645802337605e+00,     6.7266383637445770e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0198782874914269e+01,     4.1781155507004187e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.8261052125793888e+00,     1.2902941190598167e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0125468944106697e+01,     8.3904699710460809e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1077020283066854e+01,     6.7439512264794985e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7934611015292088e+00,     6.8056533678985829e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4820997156022527e+01,     1.5612584133840552e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    7.4265065696745625e+00,     6.5960645615682888e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0406497253978358e+02,    -1.9475210249675001e+00,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.4502107123663809e+01,    -6.9178432551908429e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5634648188169464e+00,     6.5147619748998113e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3144166988405278e+01,     7.3089804330583638e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0303199389889627e+01,     8.7194111943750286e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.3112716696554422e+01,     1.5956445337249346e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0802535781021311e+01,     2.3887445222829243e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.6463082920577286e+01,    -7.2947436314669321e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.8953005625734672e+01,    -1.6811163672978255e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.1555805907399929e+01,     2.3560908038181980e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2766635553270200e+00,     1.4949391593945644e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.5072801983308803e+00,     1.7996031232252224e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5536483289535823e+01,     7.1825826378445963e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    8.1753092648262573e+00,     6.4835835713847327e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1499911555229801e+01,     3.1905684234170476e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.4567450460519247e+00,     9.6808665685923261e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4852023471729208e+00,    -8.5671602408015133e+00,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    8.3883249626518250e+01,     1.4564843433176095e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2186875640038693e+01,    -9.9518385357136867e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5150716661466525e+01,     7.6954528073203110e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.9732807181492991e+00,     1.4999054991380783e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5399952304225156e+01,     2.2756425248882078e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5479041734000500e+01,     2.2862755349762523e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.2813899910430976e+01,     6.9284530221673631e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7907046053575684e+01,    -4.8862217416097337e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0411826266516954e+01,     8.4000370459945170e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.5419858585347882e+01,    -5.7295065485920122e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8975917070775004e+01,     1.7192390429765866e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8569178218502516e+00,     7.9629024025253993e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.8298476616283342e+01,     1.0575483554314091e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1076990644861251e-01,     6.5886001943705924e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3977757859210755e+01,     2.6686634945811929e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9652568917364906e+01,    -1.7979259674306893e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    1.0846735030192347e+01,     7.1404752056361776e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6188333074946556e+01,     2.2615541889981036e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2183343479988680e+00,     1.0660686892241729e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1509906795037583e+00,     1.4945625108451330e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5385873771463663e+00,     5.0731899786122658e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7319465021560852e+01,    -9.0670172210391684e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.3975051899006289e+01,     2.1953707934414517e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.7366582720772907e+00,     5.8128312521250937e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2060694370958437e+01,     1.9191276705685967e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2475007103077658e+01,     1.4381768033057602e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5088423801662557e+01,    -9.5085538456778522e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3506648565391636e+01,    -1.2053195212151213e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.6595053352998647e+01,     7.1991424269989475e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    8.6459207960763820e+00,     1.6570641800319459e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.8535661216503883e+00,    -1.0294446713105998e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7836174253998369e+01,     2.3899831648015504e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.1277359695031590e+00,     5.3291401102217151e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7703218525014449e+01,     2.3296818572237768e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9634843220645415e+01,     6.2107832540756754e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7360103322692964e+01,    -9.9858960125705835e+01,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    9.7917872519282447e+00,     1.4275829838635220e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9509229652592268e+01,     2.2582015519460059e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8175162372550929e+01,     3.7990305849472043e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0335292049831129e+02,     2.3895718588214724e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7670610815634646e+01,     1.8780222052052268e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.2151454921831473e-01,     7.3325411686510188e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.2036912986722248e+01,    -4.0844008083266544e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9068972603752499e+01,    -1.2098003913493594e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1893909675277527e+01,    -4.8632660405104525e+02,    -3.7343187314139455e-04,    -4.3870363367562522e-07,     2.0095090749677264e-09, } , 
      {    5.0816446346180037e+01,     2.4975257562613493e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5726806752339089e+00,     6.3570276205593791e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1997721078943304e+01,     1.5750002555776686e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -2.6412211595328827e+01,     3.9462585939837368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0588984052806010e+01,     2.1464153568845453e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4223708272801897e+01,     1.6664861015815957e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1995128067793210e+00,     1.1847831208404474e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6514864091938453e+00,     8.7973803725685430e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3326858962283415e+01,     1.3743151166756572e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1210026958535462e+01,     1.1205383863795210e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0533552616229533e+01,     1.6714049828161475e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4263863606137527e+01,     8.3515856457137688e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3314654072396861e+01,     1.4999766336847659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8743655717284138e+01,     4.3876650164367202e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.9793808905446291e+00,     3.8299219129363853e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    6.3145582094844084e+00,    -7.3411442973400808e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -8.7314962443068431e+00,     3.0551255086750480e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4074711158209865e+01,     3.8299923761280013e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.1407565990208933e+01,    -8.3598430197624275e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.6167843841124876e+00,     1.6649904682677843e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.9523501369519458e-01,     1.0522684895550640e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2587490961922754e+00,     1.7520977049645000e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.3918908712492497e+00,     2.7043502879377538e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2927774594461070e+00,     8.4482435357422619e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7790952166345129e+00,     6.7228261183963405e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1538398627702442e+01,     7.2780525892918995e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6539628580827937e+01,     2.2459397390115137e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1623063865682266e+01,     6.5697557720888035e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0793442153721582e+01,     7.7047930667481651e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6310142505309217e+00,     4.1714254166765403e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2186510510248660e+01,     1.4396947327201611e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.5198353435775175e+01,     3.0576645700611782e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5322932171045888e+01,     8.3283733254190138e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1870451095029022e+00,     8.3290095238278845e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7672868371596010e+01,     1.0613124695153263e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4610116640275443e+01,     1.3975646118112314e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0512990969271847e+01,     1.3842477961626850e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.6855538462641144e+00,     7.1430695599453538e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6118744964166435e+01,     8.2975398294844745e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.2511774602641857e+01,     8.1732582702244290e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6136281774166203e+00,     1.2556320886778327e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1780769142352101e+01,    -8.9443797712510059e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -6.1295731995065239e+01,     1.8365773000026213e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.4663993774721348e+01,     9.6632054300331220e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0836497192008210e+01,     2.3314483750930765e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -6.8290233282536732e+01,    -1.7429294307599225e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7095454924645754e+01,    -1.2597436751875505e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    8.5197885733665011e+00,    -1.0086903648427229e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0237875139772534e+01,     4.6360695429875423e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5055321297560074e+01,    -7.8735373050580174e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3898082242466820e+00,     3.3917336133923091e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    2.4686457933765709e+01,    -2.8601792763767862e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8847846220230608e+01,    -8.8689932910778793e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5009886442353499e+01,    -7.0851941657306288e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.5207005850079547e+00,    -1.8127659881694071e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7673410247700552e+01,     1.5574234517495548e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.4329121434917539e+01,     7.3560647356994796e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.6005575476232101e+00,    -1.0803546425293562e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.2523582471423367e+01,     8.5675808360109240e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1384414651861364e+01,     3.2739427210659451e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.0201247383486367e+01,    -9.5208722449006782e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1670907397231339e+01,     2.4100182850097575e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    4.0458056120160172e+01,     1.3585201607893154e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.5645980819568834e+01,    -1.6666019347629735e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -2.0220833042583639e+01,     2.3313779119014620e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.8256486061666067e+01,     5.2332255328941064e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4167742464204127e+00,     1.1692726686380227e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1286142872347348e+01,     2.4886215313744640e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2511682558983640e+01,     8.9363804471580170e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6410732705538265e+01,     1.0287237317489253e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3450665433848755e+01,     1.8358335427646562e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1158025563491833e+01,     9.3096795963076875e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4290413865768040e+01,     3.7724753425894432e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8569352868082394e+00,     1.6022516308602226e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1494543116526497e+01,     1.0529082888440944e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0466766387745245e+00,     1.6026445431885171e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.1386650479661313e+01,     7.9790881528632926e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.4607615417721513e-01,     7.7849921260224373e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.7156469290567955e-01,    -8.1191419588871468e+03,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {    4.9327720452380914e+00,    -1.7609603525124478e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.4367783650966416e+01,     1.9777240868829249e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.7781801588979373e+00,     6.0097054470115727e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6796290285748981e+01,     3.2409686604964998e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.3151344925315989e+01,     3.9414200004587037e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    8.4898172620130197e+00,     7.7441546731348708e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.7278506061475436e+01,     5.3909598872345396e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.0733088820697155e+01,     1.6186603903315767e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.2207561277787946e+01,    -8.9394564826928558e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.8416572546847423e+00,    -6.9658178542274027e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1910035838625198e+01,     7.3898552839705844e+03,    -6.2025608521279669e-04,    -6.6428162173171901e-07,     3.0672222349075897e-09, } , 
      {    6.2987739503829649e+00,    -1.7805768455636069e+04,    -6.4453616222215565e-04,    -9.0431006332969979e-07,     4.3620626144149388e-09, } , 
      {    5.0754367237805846e+00,    -1.7738254361387942e+04,    -7.0616945212412100e-04,    -9.7720548486849499e-07,     4.7097709865066976e-09, } , 
      {    3.6289747865823205e+00,    -8.8860057042560038e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -6.1506570004007139e+00,    -2.4324015271532222e+04,    -4.9035774973086969e-04,    -7.8798794111919903e-07,     3.8273664536560427e-09, } , 
      {    1.5021196348728136e+01,     2.2581084757053060e+04,     1.2225363957529226e-04,     3.4805152321524850e-07,    -1.7089779221847365e-09, } , 
      {    1.8933662464655075e+01,     3.3004231513054998e+04,    -1.6604274341872668e-04,     8.8491392846407824e-08,    -6.9516948920615259e-10, } , 
      {    8.9294281423539168e+00,    -1.0247414357714853e+04,    -1.1861601659126676e-04,    -2.1497482196201951e-07,     1.1151150997832324e-09, } , 
      {    1.8308521354176293e+01,     2.3695653295993005e+04,     4.8788973296512949e-04,     7.8851706677076199e-07,    -3.8832606229511612e-09, } , 
      {    1.3279164758280512e+01,     8.4369097286624019e+03,    -1.9298670192099417e-04,    -1.5092065663741027e-07,     5.4523116204940613e-10, } , 
      {    8.1573858342772247e+00,     7.1430451796142570e+03,    -3.2993655698332687e-05,     3.1015228028460899e-08,    -2.0473681753255705e-10, } , 
      {    9.4338162428395100e-01,     7.9099389351456912e+03,    -2.7764085702238487e-04,    -2.6202569882549135e-07,     1.2315673459857430e-09, } , 
      {    4.1633625585739942e+01,     3.8860007308684885e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9271981594570327e+01,     1.4920480179379005e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.6872965310867993e+01,     7.4427905251117663e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -7.0646869368107019e+00,     1.1432039388401017e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.6565194015232592e+01,     2.3814357815941665e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.2446432996182129e+01,     2.4081323812716324e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    3.8182404719253178e+01,     2.4080666547966775e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -1.6494157620664570e+01,    -8.3195178962232822e+03,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    3.8716109009391737e+01,     2.2774866098796003e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6184899844179704e+01,     8.4184202837051162e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4282458697988632e+01,     3.2921744728800812e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    6.5858498685487783e+00,     6.5574217908555147e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3491075062036391e+01,     7.2361041188030731e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4362676322865774e+01,     3.2392765924157400e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    1.5163776655528748e+01,     7.1441730873395618e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.4670056366043852e+01,     3.1064178671985737e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1322073368128120e+01,     9.8205932100670925e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1117211107456493e+01,    -7.6811961698218765e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.9051344496503617e+01,    -7.8756719263719219e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.8650695068006549e+00,    -8.4758840833295944e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.3747255181303910e+01,     8.0914269536837401e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6109810509371272e+01,     1.6248614136039461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5594839527805107e+00,     1.7073814045447678e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.9708744949804579e+01,     1.0490587524568227e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6267991182717601e+01,     8.9619300419265892e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0100119987454370e+01,    -9.8231036784171010e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5986104038314188e+01,     4.1892444681612687e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4145554785081783e+01,    -1.2182398305790243e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0269009558533234e+01,     1.2933963975921832e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6370363221977749e+02,    -8.7308453210534935e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9350851788510496e+01,     1.5664035399533313e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0601825945040407e+01,     1.3214170376581305e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.7929662382836842e-01,     5.4519989854516907e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5980968580871433e+01,     2.1908642066081717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2711610984209099e+01,     1.7071067335196782e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.0734030054657824e-01,     7.2516021086769124e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5057638894583924e+01,     2.9577859051005496e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.9930184919272996e+01,     1.4915869395870006e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0646561494775028e+01,     2.4892774269343776e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.7592676445744765e+01,     7.3023291675370028e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.5529026998738438e+01,    -2.0858306333523888e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.7898239934458367e+01,     1.9033895486969668e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.9102467731968218e+01,     1.7443219073406850e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7483716503881254e+01,     6.7700945086009961e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.7239396308348587e+00,     7.1920216119136157e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3501565357219903e+01,     2.0980613515869765e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3482774093017962e+01,     3.7999850480155517e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -9.3541847089386572e+00,    -8.3003397022061636e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.0635117885914777e+01,     5.0186471084433197e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5037969561890115e+01,     1.5433721801455922e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.9392996086828866e+01,     7.2117599304195455e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0405621612041571e+01,     1.6727664809193015e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.8852734595214791e+01,     1.6250691678823163e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9040904829356776e+01,    -8.3210032184630600e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.2928912429054518e+00,    -4.5211608735564823e-01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.0856954761951521e+00,     6.9077829445927982e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3363060006804939e+00,     4.3677259665946094e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2613419048516779e+00,    -1.7597968909582160e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.5290020334176635e+00,     1.2043574183467715e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0268301031418012e+00,     3.1415379252277162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3375060980439510e+00,    -8.4111216611257987e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.9305785288098789e+01,     1.5754604955659510e+04,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {    1.7635412280155688e+01,     1.0857629045523875e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.1832715738292574e+00,    -8.5202072064190434e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.4963898414021131e+00,     6.1686127386451462e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7790520667986058e+01,    -8.1531514564960071e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.8262280777661502e+01,     2.5417792443760940e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.4265018831644243e+00,    -1.7161737507407172e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.8722149094371709e+01,     4.4172283550754400e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2603769411720817e+01,     6.2637276160328056e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.1650964126337403e+01,    -1.7983862074189718e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.7727351372708931e+01,     4.0345357059895461e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    2.1748476598846295e+01,     1.5540451355042995e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1869832513699148e+00,     1.5545057224920594e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0569277935345081e+01,     7.8348922838348308e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7530916922379596e+01,    -2.0741373369908124e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5333486333748864e+01,     9.1611340298825708e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.1207370649079797e+01,     6.5863075698452840e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3276707016446306e+01,    -3.6969987840120872e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6762239427408161e+01,     6.7224729827722058e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6635982740948837e+01,    -2.2069170497228533e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.0216288818099741e+00,     3.2530974073295042e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6111843423994934e+01,     1.4603953703157857e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2591973230252730e+01,     7.1964308390838769e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.6108639002020890e+01,    -2.4671583066837993e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.3245554429847218e+01,     6.2720299331622482e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8318979834939604e+01,     4.1889055005257149e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    1.4903872361238424e+01,     3.1311978726507488e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -1.0412160133719144e+01,     4.1790509438371259e-01,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    8.3797474161713801e+00,     1.0239583870547140e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3741802987530519e+01,     2.3867642944995918e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2276730940753508e+01,     3.4099653652064451e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    6.9449690701190022e+01,     2.3442212639049263e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1427365585941143e+01,     7.3265076942082669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5675824916623679e+01,     8.2108082562959426e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0779684694041249e+01,     2.4158547441353669e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.9423356493628951e+01,     9.0148744038837176e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.7986966251336025e+01,     3.1518860456971961e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.6684723567071487e+00,     8.3426985875823211e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8713149334300780e+00,     2.1954157569150248e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1887333771530808e+02,     1.5542712264215168e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1114892422213899e+01,     3.1716117364950547e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.0458601169520417e+01,    -8.2327768472958414e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.9229266360870749e+00,     7.0966688940722070e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4280843476530333e+02,     1.5542796315748419e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1109821589789096e+01,     2.8352410735568541e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9780604639270543e+01,     1.8525032607617861e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3952164040092940e+00,     2.6367254876281447e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9689252625377591e+00,     1.2489885864648156e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2740359890815842e+01,     7.2163298543067849e+03,    -5.5832899822007905e-04,    -5.9151709988682344e-07,     2.7173322012508378e-09, } , 
      {    2.1124819820523772e+01,     2.3775885740727455e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2595854374324363e+01,     6.5334975285684859e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0954545706710533e+01,     5.1145235724437243e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6738634347429382e+01,     1.4891904799804008e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4722035706181051e+00,     1.4162186193005438e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.0016276998961771e+01,     2.3809312564732798e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.5796279192075406e-01,     5.6360651564511136e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2001867203014932e+01,     7.1577395327709401e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3869668625579374e+01,     1.4946085894169591e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6490796837408794e+01,     3.2657592458735537e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.3548389255951918e+01,     1.7737465414980670e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2910024960676800e+01,     8.3651330872894905e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9197228225735500e+01,     1.4508307824044996e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1187717107342841e+01,     2.3875248484214571e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0206696625519264e+01,     6.4633179324734215e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5119473048709713e+01,     1.4339819562568775e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7315665878476034e+00,     1.2669244513465742e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9626720597655435e+01,    -1.9381698173700352e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.7713468128253755e+01,     3.9807220974280026e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.2142885118765363e+01,     2.5319576666491928e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -8.7049027275811035e+00,     2.2637372449646406e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5746266746152104e+01,     2.4951129384860957e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.3606979268975714e+01,     3.2474120954778849e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    6.7834429231795905e+00,     4.2745182292314178e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6665801741496269e+00,    -1.7947512557065186e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.2821996661869855e+01,     8.9549214668852928e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9481492961822022e+00,    -3.0501934885044625e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.5124289859343492e-01,     8.3643465947646073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7900095929541511e+01,    -8.6595792444439921e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3076454831545401e+01,    -3.3070081988284045e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2802444919186643e+01,     2.1938325082183601e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.8048693330287570e+01,     1.6173139417811039e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1677549980815837e+01,     6.5594468896862436e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4710621061679582e+01,     1.4957817866358831e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3274028376203276e+01,     2.4087028654784295e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    4.5689650773035865e+01,     2.1502503185190464e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.2475275020648127e+01,     1.5860524198604071e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5475521995350192e-01,     3.9621448270381930e+04,    -2.6037868300063448e-04,     1.1354501248236910e-07,    -8.7489476893026875e-10, } , 
      {    2.1313190591733093e+00,    -1.3341674311032102e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5215766830328803e+01,     1.6610483510593473e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.0325597810207437e+01,     4.8700128885198808e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2776218669510804e+01,     1.3797188313354962e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.2635073293575427e+00,    -7.9260089498510854e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.1391142825301834e+00,     3.9565080200917146e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.2313600284360158e+00,     8.1071443481436763e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.0210989889707591e+01,    -8.2599993191518697e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.5550851700883062e-01,     6.6019646156535182e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0285083411442546e+00,     1.3906236061395986e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.4403092580718315e+00,     5.9471384140117698e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4598356682626665e+00,     7.1643994679232055e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4572443012609696e+01,     2.4127395846214691e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -2.5505337187216092e+01,    -1.1038118480082485e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.8637692714718384e+00,     2.2048879766498772e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.0508374758189269e+00,     2.3589758433379546e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    6.6735254065015299e+00,     7.7437550010004870e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3620183366948710e+00,    -9.5764282062139937e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.8765389753398498e+00,    -9.3657677228764296e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1901998314725786e+00,     5.9567363498003806e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.4275269912714336e+01,    -1.1118678296202958e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5427110274409046e+01,     3.2830745924215647e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1954393204033213e+01,     8.3304166514490753e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6090280265576197e+01,     8.3269661977978249e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7999923410430277e+01,     2.5275253543402476e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    9.9816047512350501e+00,     3.2345252255151139e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -3.9357651937062812e+00,    -3.9639911243850365e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.3627746246132233e+01,     2.5115993935810824e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.7304155889322264e+01,     2.3853813688330774e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.6300528999340651e+00,     5.7287200355138357e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5593153832586516e+01,     2.1360516574364483e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2971422669000304e+01,     5.7985618476479704e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1478541450557392e+01,     1.2719030948021227e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0605401544865519e+01,     1.6792810504473749e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -4.4525010414009651e+00,     8.3418456199367283e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1152615920056148e+01,     2.3871222057447536e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7493953030933671e+01,     2.3871669371762953e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1856821957251535e+01,     3.2809645219896417e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3441442689767626e+00,     1.5464757403089368e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8589952166208981e+01,     8.3155372293101700e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.2780353020722153e+00,     1.4508806496825277e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.2705792344566820e+01,     3.1554908873375556e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.9881010892199136e+01,     3.4622697307200477e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.2959000213701739e+01,    -2.8038349557664333e+01,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.6322681893187254e+00,     4.7538016223873628e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.1315409404801127e+00,     8.6541161697123443e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.0446684735029113e+00,     7.0820656606341672e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    5.2058273132713225e+01,     3.9626050670264754e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {   -6.8634051416238471e+00,     1.9470559050090760e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2042386645768044e+01,     1.1788327890848215e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2225703056129014e+01,     5.6469546934888631e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2492092594512643e+01,     2.5007404181848200e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.4667741071989356e+01,     1.8065173418079285e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.2272140390746937e+00,     1.2029347274130187e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8116333325114489e+01,     1.3988719354665707e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8067101156401582e+00,     8.1179879730718267e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8113515860033953e+00,     1.3120194122010013e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.9897778760028793e+00,     2.3178703779745840e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1366570199237994e+01,     7.1868403935023180e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.7604531747437271e+00,     3.2454568578904597e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7412266708988540e+00,    -3.0797334077300429e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5068159102888657e+01,     3.4951283927079610e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.3701839678320358e+00,     7.2876505250805112e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9556891877093790e+01,     2.3261162337761500e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -8.6908770783702806e+01,     9.0348978449875794e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.2951387781823342e+00,     3.2135702789414838e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -1.6845608188708106e-01,     7.2163658002971415e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3376205576823214e+01,     1.6657776095509344e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.5290457767483812e+00,     1.6656989602984457e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8893664031320395e+01,    -5.4903009720779357e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9655027532168674e+00,    -2.3499138213783403e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0480995191919646e+01,     7.9159667797569609e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.7997630496395622e+01,     1.5513903894784231e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -4.5937706709390973e+00,    -1.1977213840327622e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.6835074534531593e+01,     1.6830849687586633e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.1136070969386569e+01,     7.0978153896804733e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.4708435795890054e+01,     7.0963491281418464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1317711181379638e+01,     2.4768821342458505e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.1064088830846414e+01,     2.4964744365892497e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.8511304935006478e+00,     1.4497304163268598e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5597945148120790e+01,     2.3363794832407832e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.6669914505262255e+00,    -8.6567191936524268e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.0143188694239242e+01,     7.9514029389297611e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7311926220700123e+01,     3.2332134343952875e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -2.0744618214702648e+01,     6.2600638510413228e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9861076250529877e+01,     8.3501708185819825e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.5149967149756279e+01,    -8.3072120306649176e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    6.5071687594483025e+00,     1.1080171597130000e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9568954742006426e+01,     1.0635741350442768e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.4543791361972076e+00,     7.8426990827786603e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3970168648588064e+01,    -6.3060878498695274e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5781347601525363e+01,     1.7758893497313737e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6560000453824888e+01,    -9.9919977651382724e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -9.1548724160622115e+00,     1.4936988610068496e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.0655744574704059e+01,     1.6171055161476612e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5255843737153182e+01,     8.4586110865639275e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.8696207792811403e+00,     4.1500772011427525e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9561569942241928e-01,    -2.1539907979443456e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.4894889707909783e+01,     8.2204709817374751e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -8.2414927312200319e+01,     1.0291513015078666e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4744183335060059e+00,     4.5430981708357394e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4746796908070792e+00,     6.5797218058700500e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1383524648205574e+01,     8.3232959305785480e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.6724155474319691e+00,    -8.3340869186683522e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.0153470266701738e+01,     2.6621045389820601e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.1752597554159479e+01,     2.3187781199373356e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.4085602594556992e+00,     3.1049300177839496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.4637532950560555e+01,     1.7148972731833037e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9753679795018190e+01,     2.2721480084403418e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5061351366065001e+01,    -1.4246173846536635e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6068542148052412e+01,    -3.1972300158662605e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0513798146136331e+01,     1.6266663079495320e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.8724387533338060e+01,     3.6167022064566117e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.2429689248174597e+01,     1.0603032353418150e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1881957548529071e+01,     2.3853421506403447e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.1719948283033865e+01,     6.5726373691145354e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    8.3964032440436558e+00,     1.4886060771525994e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0423971614815866e+01,     2.8820203045029783e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.4336881015769540e+00,     1.4917291618994042e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.7259212517484741e+01,     8.6782946963836075e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.2485256601606194e+00,     3.1734236511856579e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4422047843031908e+01,     1.4830416299030985e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.4974239785788406e+01,     8.3425968313728608e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.9158901600670166e+00,     4.2314322317642027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4197413719625922e+01,     1.0327514215487284e+01,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.2086321268833764e+00,    -3.4123403240482730e+00,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.0366606466655544e-01,     6.6970889687800864e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.2709624313416271e+01,     2.5279236283437629e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.4418896310484492e+01,     8.1083252476302932e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    7.1167687559680841e+00,    -1.6847744645227169e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    3.5665895533462411e+01,     7.8444479931875885e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5537198806829256e+01,     2.0724826579489127e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -5.6518759599650714e-01,     1.4927564616134518e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2553379575230828e+01,     7.2128899280652013e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5010222950921843e+00,     2.2726164232981442e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -8.0196265137852436e+01,    -5.8054301064642459e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.4180725519555978e+01,     1.5859542536687853e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.0486822017425801e+01,     1.6943985903215045e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3986836155127421e+01,     3.3707786668186789e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -5.7009215076560125e+01,     3.1968948554140579e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9325540551728459e+01,     3.0756921960084463e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.0604987243060314e+01,     6.1583034573896593e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9529837171903339e+00,     6.5077583934203758e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.0982894038238538e+00,     7.1345023997045519e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9230989768084154e+01,    -2.5409782931945811e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2955028201929858e+01,     7.2152358026514867e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5916427869942609e+01,     3.6173708205651965e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5539576291544765e+00,    -1.3248893704682835e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.1056932164261760e+01,     8.3296396952127507e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -6.5768041265035365e+01,    -8.3277431540341495e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.7836294263582413e+01,     2.3855446206381246e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4529689483771635e+01,     9.3565095250621780e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4095618035673823e+01,     1.4897173708170583e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2735181685628762e+01,     6.0111281379453258e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5556521457137070e+01,     1.6946562118779024e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.0907294197970423e+01,     1.3383386949470367e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1943220631805683e+01,     1.3661973168243264e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0723309169966566e+01,     2.1521531893226907e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.8026918976032054e+00,     8.2337738569234316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6676721973461127e+01,     4.8893554702059373e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.7354166667069890e+01,     8.7008366015158517e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0165705874149822e+00,     6.5573693469025429e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7780549756420374e+00,    -4.9166327601865845e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1715812148056624e+01,     1.3991892778316893e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1413575365243915e+01,     1.4927065943354239e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -5.4461062413737515e+00,     5.0267893127391180e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1430815928400627e+01,     1.7190089229824458e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7681687819225012e+01,     4.9487573304245361e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -8.2448819142324279e+00,     7.9210338517986493e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.5073128360173360e+00,    -8.7025342002993184e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.9901072816359303e+01,     8.2016099926417155e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.6675084609386390e+00,     1.6944484575995324e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    9.1045508438155416e+00,     6.7336419452240116e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6568454609152425e+00,     2.3497569569235602e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.1925915026504917e+01,    -1.2795165804477676e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6774527415646915e+01,     2.3385477056343756e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0728490058346356e+00,     1.7874183117440076e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.0233426488703103e+01,     1.2888395914286593e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.9919706341232825e+01,    -8.2691881070671843e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3775434305715912e+01,     8.3260970703142739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.6784439271295241e+01,    -8.3317506293233619e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.4149405060917003e+01,     2.0225316248533439e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7388933343117827e+00,    -2.0326677033752585e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.8847695115957208e+01,     8.4152267583248704e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.6040752777490219e+01,    -3.9114914213530847e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4090637189133382e+01,     2.3949442601497671e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.8633409911545421e+01,     1.7812526328403635e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0382812777018328e+01,     8.1398054751440404e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.5970743058947887e+00,    -2.8962619037733575e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.0942316003315835e+01,     1.1509277134472723e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.9091983557120226e+01,     1.5873642109802337e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6760476749712566e+01,    -2.1299517575412574e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.5323606846190469e+01,    -8.5482455559767059e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.9167640786025935e+01,     3.1730736845816729e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.7203227482521548e+01,     3.0983336871035521e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.8224474608628398e+00,     1.3562271560967374e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2729492959545958e+01,    -2.5213672683062519e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.2550006315887849e+01,     2.2983287301884036e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.0214739741102033e+01,     7.0809546430329065e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -7.3318493226965122e+00,     7.7653436705481454e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.3843095204098965e+01,     1.7588041231775038e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8361868361603836e+01,     2.3232828848304212e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0072982812031226e+00,     1.1299705716687240e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2320686002331847e+01,    -7.7894260884467585e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.4789518175681600e+01,     8.3701610582390022e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -1.2195810893993002e+01,    -6.3996692035550832e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.2829238653655892e+00,     1.2507154559176461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3704775690224160e+01,     9.5384072560611094e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.9254478939636542e+01,     7.0432105998196475e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.5627427852436917e+01,     3.1230623695886035e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.6118843118814201e+01,     2.2360399104838252e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0571999883494071e+01,     2.2720057393469666e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3854472879327886e+01,     1.6153028191086631e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    3.6979654236991152e+01,     9.5100555336438247e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7301951066127248e+01,     2.3576640522181271e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    5.6843982313500714e+00,    -8.5175773741028606e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.0904640040945850e+00,    -3.0202148306586282e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.3572495652917908e+01,     2.1499849669290979e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.7858647623552123e+01,     1.4898714220936952e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.1599505214016347e+01,     2.4080988466790819e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -6.5863528473747536e+01,     2.4081001893892273e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -8.8518629999765377e+00,     1.4914998994468737e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4712121307466822e+01,     7.2578336989153086e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.9074386225335513e+01,     1.3186929671765280e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7440138787939734e+00,     6.3578575161459992e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.6304338500811269e+01,     8.2646036491160557e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5607737251504751e+01,     3.2415720079407747e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    1.2301193453098406e+01,     2.9062513098751601e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4533949817645027e+01,     5.6323332587405275e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.1972905845604743e+01,     1.3908537261337398e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3855567110621550e+01,     1.4927953216207607e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3145215237525534e+01,    -5.5732099318327937e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.8091657913226413e+00,     8.4099940112919285e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.4317819073584303e+01,    -5.5730756608182628e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.9985899530480335e+01,     1.0785033020050392e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.2984681078618905e+01,     9.0057008754835515e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.2033750969162234e+01,     3.0493249411321645e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    8.9527441354091675e+00,     2.3276925186846347e+04,     5.7858535816834269e-05,     2.7581612702191584e-07,    -1.4149820578231025e-09, } , 
      {    2.2302153488022638e+00,    -1.6062862321487999e+04,    -2.7633411970299409e-04,    -4.6441727865704901e-07,     2.2429468624219679e-09, } , 
      {    1.0971028289004927e+01,     5.4144597421331227e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0276986196456228e+00,    -1.7091490870240048e+03,    -3.9901868747014424e-04,    -4.7634049544204832e-07,     2.2929699380388498e-09, } , 
      {    1.1339689705077488e+01,     5.9236662678120983e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.8580272953103222e+01,     2.4604528032518938e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5694235016476888e+01,     2.5790144267365555e+04,     5.2530772831017247e-05,     2.7704891125024120e-07,    -1.5238615143266822e-09, } , 
      {    1.4623773488566105e+01,     2.2422755911868415e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8345709616003330e+01,     3.0141732286237184e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6281111238677113e+00,     1.2240852178097462e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0128062219074522e+01,     1.6912780691323867e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4594067534975274e+01,     7.2598419035569759e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.4684652739330744e+01,     9.9747247683726055e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9833472768280693e+01,     2.6439120998499471e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -1.5442800556590388e+01,     1.5541626027333525e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.9378260406806589e+01,     1.5543882552630064e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2274349360379833e+01,    -1.6517697438362324e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.5613419963811737e+00,     2.3295395881304019e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.7992509344637455e+01,     1.8031711756554483e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9596278620672898e+01,     1.6578719447852141e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0668368780872211e+00,     8.5110184185037251e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.6195110283780458e+00,     1.4924755986191740e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9679448791984509e+01,     5.6262465025870952e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0612214188411507e+01,     3.2221467047206548e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.6910694120774679e+01,     2.1107181015556067e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.9572611235795305e-01,    -2.7998727216913721e+00,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.0949571457947778e+01,     5.3500758051896821e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3563879544452035e+00,     1.5697858812006043e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3185975357975082e-02,     2.1639887396133625e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -9.0871379824905354e+00,     1.5330903624304083e+04,     5.1870637791611210e-04,     8.2496477754015965e-07,    -4.0571148089970406e-09, } , 
      {    1.9020017401556416e+01,     2.5870331990525345e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.6018644418692922e+01,     2.4569617354502519e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.2451210622255346e+01,    -1.8779228782928149e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5546196276675644e+01,     2.3712186107013596e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.3218067000177136e+01,     3.0552802199386027e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.9738293008983305e+00,     8.1552093267685668e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1873385345650503e+01,     7.1088360164028381e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0329844960850295e+01,     9.4482357566309984e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.3936832710726456e+01,     1.3562659702760911e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.0395556390759836e+01,     2.5287723684706025e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4379208816921846e+01,    -2.2517990236624555e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.9466488276144752e+01,     2.0995392929146205e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2402682545145982e+01,     1.0124683991979855e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2413475617252775e+01,     1.7195020262082053e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5428678079228115e+00,     6.5430954643570976e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0416102886818403e+02,     8.3927792001308444e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.2451283563938702e+00,     1.2509920220905180e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.2417643174337627e+00,     5.3356990658208488e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8620379941765350e+01,    -1.4281391598103255e+02,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.5786405659217340e-01,     6.3482086956090307e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4926593260018405e+01,     1.4862188953191937e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.8073108852602488e+01,     9.3066022814291500e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0807822242739832e+01,     7.1179246308730280e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8589405276567570e+00,     4.1571985073390141e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9419404287735071e+00,     7.3008133900246967e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1485001850733253e+01,     1.7796959061811233e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4604393441868204e+00,     7.8585797857446041e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6892092535206082e+01,     1.8941816119776715e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1353699660362249e+01,     7.9220002541997119e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1204578440908429e+01,     1.5735443009070530e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.3225372243819358e+00,     8.4236089923234686e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0943122694643318e+01,     2.5807010643844602e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.8510861865045328e+01,     2.2750258199741005e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    1.0588635889155958e+01,     3.3890104186328150e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    6.7151208683467800e-01,     8.3258379351495569e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6060785727615006e+01,     2.1066850108240687e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    8.0969581510672075e+00,     8.3471677489126050e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.3858490502936025e+00,    -8.3102151003342969e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.5642027896176550e+01,     7.1852124701607790e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    4.7495477884424034e+01,     9.5856589106726115e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4766461474159094e+01,     2.3946696775675959e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2413696832242632e+01,     3.1206040532085106e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -1.1576105824435967e+01,     2.2690835692890714e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.4396601930214068e+00,     2.3400768747020629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5158019287706566e+01,     9.8140342544679588e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5077623831840087e+01,    -1.3234666795345308e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.7434665632087281e+01,     2.3245215672343402e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0612226690146752e+01,     8.3253857248473978e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8790202761985022e+01,     2.5190692707349055e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    3.5511281491206411e+01,    -2.6676684286630552e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.7808537345924709e+01,     7.2129346027100755e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4255534567507265e+01,     7.3303103410362155e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4481118690579283e+01,     9.2420682912480879e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.0214939549493117e+01,     6.4264956727211520e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6649868034177643e+01,    -1.8225659481064733e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1628629452261482e+01,     7.7029419587152393e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.3548651031298832e+01,     2.6365458669101827e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8666451683803036e+01,    -8.1478563143006204e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.9534520947568183e+01,     1.1890523151807656e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5428093061525963e+01,     9.9184131532960964e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4456925409160867e+01,    -7.3734007357141672e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2693368096744070e+01,    -4.8369114190337859e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.9465488201838710e+00,     1.8439016193755149e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.4160188600381129e+00,    -8.8548273007409225e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.7092352055843890e+01,    -9.7964182528293895e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.7168370819783370e+00,    -2.2679543187278141e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.1161778568854963e+00,    -7.0601123485661077e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2961056632626832e+01,     1.4486994882013110e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2682450554804021e+01,     9.5876077946559471e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4819507784524134e+01,     1.6013533235912926e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2553487049545957e+01,     1.6659649838195339e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.5743673026479712e+00,     8.8716793777575858e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6513585034782324e+01,     1.5529600094668516e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5694314661328974e+01,    -6.9300896437063811e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.2311061287097811e+00,     2.0334256987368357e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.4483691908141999e+01,     1.3344664026105713e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2363303911841808e+01,    -7.5583972287519246e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.5958164733445628e+00,     2.4750787658517795e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.2115938474161876e+00,     4.2276294621548768e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9019358506038440e+01,     3.2178807231250845e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.1146741760537335e+01,     5.6337456245140584e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.3847197752488498e+01,     1.6729479682054098e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1906545240586135e+01,     7.7018065305116588e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2194738088651350e+01,     8.4353409645127085e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3323924966898048e+01,     3.4227046505191305e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0690948189941832e+01,     1.5961863710045043e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0022630605816896e+02,     3.1069966551675163e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2962730904586699e+01,     2.1666100363623327e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.1678464797393797e+00,     2.5118071478594527e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.3674763921779533e+01,     1.7415610096232922e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.7096580009451814e+00,     1.5555908485295073e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2031323606264309e+01,     1.6700042665202604e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.7525655589126394e-01,     6.2829706284173358e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6301120169215082e-01,     8.3572668041984471e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.3237093056156777e+01,     8.9853324991262798e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.1918992482362597e+01,     6.9042744954960990e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3956368955415876e+01,     8.5095265349462807e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.2998947066873193e+01,     1.8760625995960863e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5312484736941062e+01,     2.9871553225828207e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -4.0811922284170706e+01,     1.0237592331463647e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4139106176679967e+01,     1.2228337426359260e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.9629329925592927e+01,     1.3251140364421426e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.3708650804577616e+01,     1.2225119694098164e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.3786843369207880e+00,     2.2561861447302967e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.7380060810469558e+01,     9.5102531687705778e+01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.8963150397387267e+01,    -1.0682278933919570e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -3.1574532531294178e+01,     1.3657479218887089e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4062155512303185e+01,     1.5571604685179362e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -5.4783490746214554e+01,    -3.2969381815262750e+01,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.6500486055772736e+00,     2.0727277957061342e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2669941399371019e+01,     1.7776927181254447e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.2522510302110490e+00,     8.3633171500932949e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1352973361832015e+01,     2.3398242651583951e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6906423301272845e+01,     1.3831979292925407e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3075915089639366e+01,     8.3129342053700893e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.7393419733825528e+01,     1.6608931448966156e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.4650388876553542e+01,     3.1678824670605945e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5631742196846544e+01,     2.1083770789048274e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1986500301628212e+01,     3.0770039871282726e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9368294165791312e+01,     1.7699374920242324e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.1394704851174211e+01,     2.3532317399091826e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -2.7836479905404396e+01,     1.8485217705719948e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7871650454359706e+01,     1.4755187097344602e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.5769719114346316e+01,     1.6848692915816471e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.5537594268636667e+01,     2.2128516283845322e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.5802935182826431e+01,     1.2323422964816036e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3482378270634348e+00,     1.4895259035180221e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.2750210193348321e+00,    -8.3559213866865284e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3987872079547410e+01,    -5.1749937773464057e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4164352972654129e+01,     1.4286150379553674e+04,    -6.0855413138016608e-05,     6.0884938291196182e-08,    -2.9913973751820592e-10, } , 
      {    2.7885006800169229e+01,     6.6011743914524786e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0176427877806127e+00,     2.7330978443950498e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0516830081527885e+01,    -8.1867048137974671e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1120090398302910e+01,    -1.5594646403539627e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.1741575903802310e+00,     6.9984799251792911e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.5742396954065200e+01,     1.8660019341600026e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4684836000220667e+01,     1.7409643361177717e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    5.2237635653034538e+01,     2.5368278532306482e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7009543501399477e+01,     9.3706834956188759e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.7969665927204730e+01,     2.2852418359808125e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0106140598557818e+01,     1.8472415826364748e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1699430252244136e+01,     5.5060716100570880e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.1569929165517895e+01,    -1.9098257940912467e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.1620934308344431e+01,     4.5711365203934038e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4758755736626229e+01,     2.1420019891920747e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.7852189862827803e+00,     8.1179974488107364e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6472464383034914e+01,     2.3307127853493257e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.7601133832433966e+00,     4.9672707356677056e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.7033221180282965e+01,     7.2151911280066115e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1066645597790341e+00,    -3.9039113737680327e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6274323653757072e+00,     2.3964745719131817e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.4298269122969998e+01,     7.6363904110087306e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0090851229110765e+01,     3.2410015237339776e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    4.0537932052955426e+01,     3.2409357972590220e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.2629194636159614e+01,     1.0438412716230552e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9589208966871006e+00,    -1.6533429922361629e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    7.2983019473861939e+00,    -8.8925646598551357e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -5.1137501509952266e-01,     1.1919141006906548e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5443772533031748e+01,     3.1571829554183154e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.8830547250087428e+00,    -2.3269231084639728e+00,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    5.2064048211141358e+00,     2.5273176000618772e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.4329771322091474e+01,     8.3363796307838420e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.7476366074474701e-01,     7.2544032484126383e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9330657408657466e+00,     8.2220418847341916e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2045586551435212e+00,     1.0170954363804376e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5346544577540708e+00,     1.4540570559566611e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9132511474411075e+01,    -9.6697085335471684e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5652894842716890e+01,     1.4942832638346510e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.5686257550259313e+01,     3.1085410528389752e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5885189837072078e+01,     6.1700354295788993e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0923149552202418e+01,     1.8720703474980855e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7452503296747512e+01,     3.0454571162601860e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2041559256095458e+02,     3.1085606631537426e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.0505564453052703e+00,     1.4264809211912856e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5748952372833116e+01,    -1.7584781725990788e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.4010688987824125e+01,     1.4888138314309694e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3021469244517380e+01,     1.1337562948611103e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4132543125328802e+01,     2.5672257253130618e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.5032452905122913e+00,     7.0033594138067210e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8942787204821986e+01,     3.9571934358927472e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.6424271986288819e+01,    -5.2958573555468347e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.2976866469180685e+01,     2.3953929007874962e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2921357142977083e+01,     3.5835893931100363e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4331072503425244e+01,     2.5802945223122142e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3333061413182104e+01,     6.6141412137230600e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4397296311515614e+01,     8.2184687537574264e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.2101098640239780e+01,     2.6083782152327858e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.5917479537669870e+00,     3.1261188140880386e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3938575462770128e+01,    -1.9498557777307660e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.5946718394608059e+01,     7.9277487837749618e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1599950682636795e-01,     4.6801690089192789e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.2071152496979224e+01,     2.2743699244141877e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -7.2073258555533243e+01,     8.3256322199235365e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7720017402882597e+01,     1.5949445460405532e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.1862161947164822e+01,     1.6657177134020880e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1346695073113906e+01,     2.1644489796016445e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {   -1.9013799411573103e+02,     6.5862632389519185e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.7321168926881021e+00,     6.6033873065872713e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0428617641967662e+01,     1.4929865816075930e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.2488378853589651e+01,     6.9751734539708705e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5757494592018922e+01,     7.7490857053924647e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.5426457895370440e+00,    -9.5633535407451905e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0898913065272673e+01,     1.4423716503546430e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9437641866507740e+01,     2.5200554771864252e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.6657988441049412e+01,     3.2435423754581250e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.1453651547883217e+01,     1.5463193824328004e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.9869752847419527e+01,    -8.1876712161985324e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1684414334255186e+01,    -2.7562699840684991e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.5275374463758393e+01,     1.7981062201468027e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.0231443323157468e+00,     3.7322745013407257e+00,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.7645404407128251e+00,     1.3757738365572231e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1248515357105658e+01,     6.6037794885146013e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.1560508765133282e+00,    -1.0071627568934096e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.3794982371331535e+01,    -4.7393752621955798e-01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.3620610871954579e+00,    -5.9992165163528455e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4748872496569003e+01,     8.9923360806057153e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8664164397224923e+01,     1.5922657348476514e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.3195978549138037e+00,     2.4379096596802658e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.3972784805482014e+01,     1.2041648190141479e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.2682342159296546e+01,     7.1050303768324738e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    4.2216373249279343e+01,     1.5335506024186903e+04,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.7954655261152119e+00,     6.7096740100124680e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1439652547705492e+00,    -1.7455659766268318e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1995276240797667e+01,     1.6816290140880476e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.8088311870387184e+00,     6.2118791349431613e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2923850695374846e+01,     1.6163583708458280e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.9002746401354045e+01,     1.0959635502022122e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1305523969158333e+01,    -1.8050678453250669e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1918741834204994e+01,     7.7533282015350360e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.9519176917977532e+01,     2.2434499707170190e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0006070620089563e+01,     1.7583110199517447e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7538234209797121e+01,     1.4218576264980387e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.4670769581093280e+01,     2.3938959808853375e+04,     2.4475941736527219e-05,     2.3994117513538105e-07,    -1.2962948205506771e-09, } , 
      {    3.4009358212551364e+00,     7.1058445613193926e+03,     1.2680352579716054e-04,     2.1303837915693181e-07,    -9.5325035607119168e-10, } , 
      {    8.0689201068368348e+00,     2.4391529365780352e+04,     4.2872445982890440e-04,     7.1509251958040393e-07,    -3.4796580815642834e-09, } , 
      {   -2.9228127965649335e+00,    -2.5580619181960345e+04,    -4.8502998674505274e-04,    -7.8922072534752439e-07,     3.9362459101596220e-09, } , 
      {    8.4700283493689525e+00,     1.6762127571952475e+04,     8.5720293256518189e-05,     2.4594641675918057e-07,    -1.3205791378374536e-09, } , 
      {    1.5233626774098468e+01,     2.4395002947113735e+04,     3.0230987241371821e-04,     5.6894432033846781e-07,    -2.8498317802980741e-09, } , 
      {    1.3797873957424400e+01,     2.2648598851301187e+04,     6.0620349673326907e-05,     2.7515610167645335e-07,    -1.3612695500929772e-09, } , 
      {    1.7946548658695917e+01,     1.4846938240519608e+04,    -1.8860047289597794e-06,     1.3439675194415402e-07,    -7.0128783786175493e-10, } , 
      {    3.1017699752742306e+01,     3.8265814398197865e+04,    -3.2002529118284442e-04,    -2.7917995826692583e-08,    -1.6192776949058513e-10, } , 
      {    1.6554539717403046e+01,     2.3347888482096754e+04,    -1.3009140913673313e-04,     5.6728873009884696e-08,    -4.3817460498679860e-10, } , 
      {    2.3719231898431882e+01,     2.3351362063430137e+04,    -2.5650599655191930e-04,    -8.9419326232051445e-08,     1.9165169627941039e-10, } , 
      {    2.8662142933427475e+01,     2.9937122973574420e+04,    -4.7231769894517061e-04,    -2.7863686998134726e-07,     1.0740562291300656e-09, } , 
      {    1.6133875554675956e+01,     3.2933243837473492e+04,     2.2005133898424815e-05,     3.0753501362713915e-07,    -1.6727041625641207e-09, } , 
      {    1.3247837443516698e+01,     3.4118860072320094e+04,     2.0472524822975931e-04,     5.2781141863619583e-07,    -2.7591182924256686e-09, } , 
      {    1.0192495816574116e+01,     3.2862256161891972e+04,     2.1005301121557632e-04,     5.2657863440787047e-07,    -2.6502388359220893e-09, } , 
      {    2.6454378974196979e+00,     2.4330651286036300e+02,    -4.6341379122860225e-04,    -5.4857589163538094e-07,     2.5869658024004836e-09, } , 
      {    7.7027499703302738e-01,    -4.5250953660182131e+02,    -3.9911661983372838e-04,    -4.7629686221074841e-07,     2.2936971585605140e-09, } , 
      {    1.5996095284172824e+01,     1.4738659916155495e+04,     3.3826120669571184e-04,     5.3718180050194459e-07,    -2.5928368960787203e-09, } , 
      {    3.0591552103856925e+00,     7.1768322369009002e+03,    -6.1244351519990970e-05,    -6.0052416237994891e-09,     2.4284317286776639e-11, } , 
      {    1.2661812506934956e+01,     8.3321650059568310e+03,     2.5877820347140000e-05,     1.0457067491271859e-07,    -6.0615769735444168e-10, } , 
      {    9.5398033497423178e+00,     6.3762114444079825e+03,     2.1678544388436827e-04,     3.2291063711668748e-07,    -1.5307070835039487e-09, } , 
      {    1.0196464389375716e+01,    -6.7574114573286352e+01,     5.6501391643316566e-05,     7.4040939304520783e-08,    -4.5804226963866738e-10, } , 
      {    3.7407115906497495e+01,     2.2301663035774705e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6959287649133461e+01,     2.6119830568731461e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.5498229937753337e+01,     3.1101050608252015e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    7.2324315738177152e+00,    -1.3707083808353764e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.8348484564592312e+01,    -1.7452373442520584e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0868278852989526e+01,     1.7492938700087332e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.0718371742510969e+01,     4.6401427510597784e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.9318641806630112e+01,     2.4205506958076970e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.5482240295045372e+01,    -6.9551812399778885e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -9.9554682426314315e+01,     3.8083933420673930e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8693624074083075e+01,     5.7839808569275283e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1710419989654296e+01,     6.6120636709393593e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0149106501759277e+00,     1.5558963625322511e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5901110308928857e+01,     2.6558158152642296e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.3015793265060122e+01,     1.5543147536244236e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.6205706718392170e-01,     6.6129852423758830e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6560186680299616e+01,     8.2838094506838479e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.2755134020964487e+01,     1.0213614179760079e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.9869852098012259e+00,     1.7112536968812332e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.3395687000637103e+01,    -5.7960571923098723e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -6.1585866822720092e-01,    -1.0720598670236537e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7652197458800543e+01,     2.2725638751825063e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.5161046456283451e+01,     8.4697116330483677e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.3497586530737525e+01,     3.5660314860397193e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.8152594173865594e+01,     3.1570799535161459e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.6340062636660395e+00,     6.1801150722018265e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.9351305277256934e-02,     6.0507476997938338e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.5190217571896767e+00,    -8.3662306679420180e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -4.1424144153314373e+01,     2.9424634157381552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3428492430297791e+01,     1.5072642651102949e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6490409774690782e+01,    -6.3694408342837733e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7556534277549417e+01,     1.6778039552095030e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.3140599953819923e+00,    -8.2577037490419425e+03,    -3.4034028507947772e-04,    -4.6976249493538599e-07,     2.2135186719786194e-09, } , 
      {    1.3699772370800600e+01,     9.9894061018030698e+00,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.0221914768633667e+01,    -1.1699854894514474e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.4326864854820442e+00,     6.9853257298660119e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.5394893150055800e+01,     8.4575601242511657e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.1936226618581053e+01,     9.5161281621626276e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1968518757528820e+01,     1.0834617046109761e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2887210472580719e+01,     2.3917846380478393e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    1.0304952430030296e+01,     6.5857658025166102e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3464885483468461e+01,     8.3315449140973433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9914509550839011e+01,     1.4192839115364133e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.1012543271485114e+01,     2.4109226274715184e+01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.2068048106906801e+01,    -8.7670555027869432e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.0710393411287793e+00,    -1.1910105936656764e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.9203153715054690e+01,     7.7664461127333016e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.9024865274137277e+01,     7.7763081772484948e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.9012742954526090e+00,     8.8723907232244619e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7798283739957846e+01,     3.2422305843382983e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -7.7085250514093431e+01,     2.1947111249988748e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1673830587523632e+01,    -8.3857793232870281e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.7775708922679677e+01,     1.6801999292389064e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1178767689577005e+00,    -6.8782616211695688e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.1141928854844629e+02,     1.6657037246839551e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3026628471076287e+02,     1.6657728451654250e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.4423477532920206e+00,    -1.9923005119448626e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.8269673620703566e+00,     1.6614723033291197e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0165840138487283e+01,     4.7806859861297053e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.9115585923016095e+01,     9.5830053947731267e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.9198754532392712e+00,     6.6813841934796956e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0765217457688605e+00,     2.2333143263288461e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.0845114750722853e+01,     3.0995337080533478e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    9.8943436895835575e+00,     8.0842160213555762e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1856188803862334e+01,     1.6020438765818522e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.1254589340457670e-01,     1.5542361043719353e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3069518881075659e+01,     8.6143012678144314e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4225277365208189e+01,     8.6376011777051790e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1692402185446049e+00,     3.5191397837810246e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7339310893276316e-01,     2.3872432375801891e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0286325613212163e+01,     1.7302551180940009e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.4212629238919021e+01,     1.8819278949191677e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1142402397917053e+00,     7.8773153394132796e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.8142742873175180e+01,     6.3087819355046540e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.4021915187560623e+00,     3.2198059596444997e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.3770147530930252e+01,     6.5917887547555538e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.0801348613564706e+01,     1.2289656882652291e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1091588643868388e+01,     7.1763255062441331e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7926014509791441e+00,     1.5819594616669775e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0346773211674666e+01,     6.4404006904484231e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1053861699620963e+01,     2.3875854941775502e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4871693031756418e+01,     9.1857266694224163e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3999043212101849e+00,     1.6825524303901770e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6349734286767401e+01,     9.6317072605876838e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.4735054400071768e+00,     1.9941236017675372e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0369197163083276e+01,     1.8832037163448138e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -6.3501663961999931e+01,     1.9288326926645575e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4239460246582560e+01,     1.5527833851622396e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -4.0039457961985025e-01,     1.0906684746994251e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3426290479303132e+01,    -8.1065227204691600e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.9759857393113890e+01,     3.2202214682012400e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.3657998626204913e+01,     9.0460165232837826e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2534046573334152e+01,     2.8745431705261526e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9954009116462945e+01,     1.5684740900807776e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4706930159222972e+01,     1.5447836722281776e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0078678320463226e+00,     7.3898908377995112e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0518929927390573e+01,     2.5132303401821728e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1031980934564494e+01,    -4.8559870040932921e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.6296965211252830e-01,     7.1353546329990422e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5175422072030830e+01,     2.5108873214940300e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.4938646494545253e+01,     1.2638804180048044e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2764607188071928e+01,     9.5147054712288773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8274982375867963e+00,     4.9426780961278610e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5791273151617336e+01,     2.3707450565759127e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -9.6975575446792206e+00,     7.0322613526344512e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9100437956408072e+01,     8.3311212833652262e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.4964550894865084e+01,     8.3262615658816740e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.4495841401539042e+00,     1.4899027593796574e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.7919971562318384e+00,    -8.9209505932653938e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3176428703840234e+01,     1.5556434815621856e+00,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -2.4972240606729738e+01,     8.3643875250689725e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8270099351949522e+01,     1.6054814399349467e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.0896682045174648e+00,    -9.7490893738756165e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.5022804181543643e+00,     9.4999869628031302e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4861851552971048e+00,     8.2440831381789194e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9434358252381472e+01,     5.1543384743416364e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8512723831361228e+01,     2.5821630124710784e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    4.9094339338631357e+01,     8.3283961619424499e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    4.4383230237857745e+01,    -8.3289866873044484e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    5.3584460266611398e+00,     8.4132997110679808e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2358130923190007e+00,     1.7130585912268194e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.3645958569813615e+01,     7.2673842785276111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3824361604956913e+01,     2.3967005688483034e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.8866234806379424e+01,    -7.5608645737738698e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.4105133855070932e+01,     1.2566218836418134e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8799128701374443e+01,    -3.6236804733429017e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.3752465782279810e+01,     3.2992722795681475e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2842127542864892e+01,     7.9202513437907846e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.2992523514315923e+00,     8.9415798985293150e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9672689328353666e+01,     2.9278138085839084e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.1009300275984529e+01,     1.4774761294051679e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.9406663959857532e+01,    -3.5694263411455722e+00,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    1.5331513929056660e+01,     1.8313508534731172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2690467383535331e+01,     3.1905890797654880e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.5420590202479456e+00,    -1.5982674598328209e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -5.3198817799166363e+01,    -1.8401259413937964e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2331312334392521e+00,     1.2727173152317026e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8910008934699643e+01,     3.8785540103583415e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.4400925622226506e+01,     1.6781335776132175e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    4.5137343491354500e+01,     3.0242584457140052e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.1701307186084790e+01,     8.5351276447784403e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8238171479718638e+01,     1.4498726854202348e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2887234024677121e+00,     7.2626227353992772e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5895273333483296e+01,    -1.6668199560503759e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {    1.8004417101652388e+02,    -2.7168665223751489e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6367947159812790e+01,    -2.5189590265163901e+04,     6.6345003052078436e-06,    -2.0362436405988305e-07,     1.1202589729398046e-09, } , 
      {    2.6484675807359945e+01,     8.3255629195269867e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    3.2773836362252631e+01,     3.7148894469653234e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.1444264060449719e+01,     9.2321273352955632e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.8578766878285293e+00,     2.4001365376545720e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.6662041733030790e-01,    -2.1566206302605297e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3432457614437563e+01,     1.4301558231030387e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5307627597978426e+00,    -5.2898079017526607e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.5704418956325213e+01,     1.0625574258550394e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.2445447616243470e+01,     1.8325958098128300e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.1050110975154510e+00,     2.2751886123082542e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.3143381373263341e-01,     1.4250090703487109e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6246557016127461e+01,     1.1739397128613791e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.9563521556443956e+01,     2.2163454459303990e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.4613312548394164e+00,     3.7570562314497647e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8540126211897707e+00,     1.4169793567477027e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9448353722128815e+01,     1.3658183850803240e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4337605727277747e+01,     1.6086950658416079e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    6.1409463630706739e+01,     1.6228149773690919e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.6204081910733992e+00,     1.1624747567787048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8592546046272737e+01,     2.2026337861366526e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.0627096236172520e+00,     7.2764196723885034e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.7908061165630841e+01,     2.0269093659626105e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0746395346941862e-01,     8.3422539617020193e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0651615184644712e+00,     3.1634186361213309e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.6224211026314297e+00,     1.5541590828427563e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.5026383955778442e+01,     3.1985520601450706e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -9.0539477967085801e+01,     7.1499750898509501e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4718241541903936e+01,     4.6986769024042587e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1568065202214548e+01,     2.4565014954619699e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -1.0862762786579868e+01,     9.0191321595414393e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4576226177381125e+01,     1.6657935138779383e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7139848286313040e+01,     1.6656830559714417e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.8446341888488842e+00,     7.2232516532736618e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4496367191232817e+00,     8.4606886293476273e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.1037527749880196e+00,     1.6723350208249876e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2452278941998056e+00,     8.3151288875448809e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8552572051967584e+01,     1.2487628781306747e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.7566808413742180e+01,     6.8969798967463703e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.6720379649984096e+01,     8.3508810137976197e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0188458840881646e+01,    -2.4973455035452360e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    1.9093525388372051e+01,     1.9173243021745257e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.4460285933888644e+01,     2.4983996731086645e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.3914077902723108e+01,     2.6231187654570931e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6490711655036151e+01,     3.5344304896772592e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.1735243530225780e+01,     2.4988151816654052e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.4323450736433202e+01,     2.9908523213668326e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.7695878137529260e+00,     9.0797979507389446e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.5847688223898054e+01,     1.6029403896576128e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.3880696685918048e+01,     1.3658542880312041e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.7010542949015619e+00,     1.4115785527132288e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.2292677206162454e+01,     8.3401661529650264e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.8347311017782442e+01,     2.1484202477025054e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9584701680721921e+01,     7.1361640300397585e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7547243867284720e+01,     3.8316492126129524e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.0226673804297388e+01,    -1.5310597554658834e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0751508858193066e+01,     5.9068055211645305e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.3695270681821079e+01,     3.7906550475628945e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.3056496432521200e+01,     2.5294950534476928e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -2.0156722878258044e+01,     8.6392291010467179e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.1002130994837884e+00,    -1.6550221580900652e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.6553821400530360e+01,     7.7007428690867082e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0978922059690925e+02,    -1.0390267740530726e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.5744493455933039e+01,     1.0937093596889878e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8846316082078971e+00,     1.5258323136076797e+00,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {    1.9002621867766351e+01,     2.4474016150034535e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.4709961360114612e+00,     5.2335801363439250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4086806109130556e+01,     1.5520462850383361e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4699737630386954e+01,     9.6419735736291113e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4348416638577945e+02,     8.3303133299396268e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9407344463790697e+00,     4.6052725512983816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2565885267437359e+01,     3.0628053260456749e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.7855377032460567e+00,    -5.9518577071413688e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.6463414712372206e+01,    -7.5787321126340466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.1624126769410751e+01,     5.6803388186181110e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1330135066435201e+00,     1.0309174153637870e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0586702250340698e+01,     7.9563339711873577e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1625492110460094e+01,     5.0290905126805292e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1074272289510813e+02,     7.1759789319376705e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3867928938371392e+01,     1.5565045729580228e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.2650009537423621e+00,     5.8008673267421473e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.5005574554097016e+01,     7.7138903508492613e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3061733237637242e+01,     7.7053148718355005e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2548015187335464e+00,    -7.5765600597571620e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3617767846256378e+01,    -6.7926901732480710e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.6497655294413743e+01,     1.6658478730114388e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7373408365724238e+01,     3.0377018901589738e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.1104042622240726e+01,     6.0192674722800839e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4321835813403553e+01,     2.4998693512288337e+04,     5.1870637791611210e-04,     8.2496477754015965e-07,    -4.0571148089970406e-09, } , 
      {   -1.0378377512243424e+00,     2.7674587943382107e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6459602615683050e+01,     2.9253554922038151e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.4997028740781957e+01,     7.2114093494588606e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6406794470023112e+01,     2.5284641253221438e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -1.6662679839432521e+01,    -8.6412416552668219e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.8108774802540069e+01,     7.1607414521890787e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0086624602148802e+01,     8.5405678134682148e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4566606475535053e+01,     3.2811524043466648e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3941871891338613e+01,     6.5438068098239737e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0892458384458338e+01,     1.7268058408017972e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7990417228806859e-02,     6.6260956633670958e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.2102522146881284e+01,     7.2289833037177414e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {   -6.1247570623619882e+00,     5.9705656064655250e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7802292950455545e+00,    -7.8930725586666422e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -9.1460504825296933e+00,     6.6123292524470486e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9655818354562445e+01,     7.7292402818770734e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.6258133152250309e+01,     8.0189030547612047e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2724453128862272e+01,     1.5734828922627541e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.4458094685807993e+01,     1.5557674728341190e+04,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    3.7024648127296338e+01,     2.1753537544057472e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2764249494430242e+00,     6.6813152541905911e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5378051514755484e+01,     1.3076680608073730e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6381874833525663e+00,     8.6157931513718759e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.5039876087034028e+01,    -7.1857111429410597e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.1303232716705381e+01,     3.5015718276893465e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.2641823270450629e+01,     8.4814947467062884e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1039132224322316e+01,     2.2029182775424317e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.7327521766155499e+01,     1.3228598459289184e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8301840875969283e+01,     8.9727314936682924e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.9121912220556794e+01,     9.9574517716069386e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5803182835443913e+01,     1.5564795543426522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.7452164011951226e+00,     8.3134959706414829e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6197638679015682e+01,     1.7047767425306032e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5313248955018838e+02,    -2.2650968293304686e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1253580000220431e+00,     7.2064362817320734e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0163801104528272e+00,     1.6789380053971076e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.2571227679834318e+00,    -2.9465822171905438e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.7590930688034625e+00,     2.4011131125489817e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -8.5844771845675680e+00,     1.7976131169210435e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.1657939434779019e+00,     9.8146040277964175e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3750712992346649e+01,     5.7498617187627713e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1506732959020127e+01,     2.5177574796150791e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.5904209903631607e+01,     1.5143174364689674e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.3931870629631540e+01,     1.3403400095169261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3862599050069843e+01,     2.5092012468292731e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.7360377784504010e+01,    -1.1142695297563041e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7041123870100979e+01,    -1.7176614080061495e+02,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.1550642199829010e+01,    -2.1671021478249413e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.8158874970072091e+01,    -1.1149875887739072e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.7055231859200578e+01,    -1.5292757466052194e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.5008806329774018e+01,     2.4357414372866733e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.4175258894418121e+00,     7.3192897143138507e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4026326412843432e+01,     3.0392766645104188e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.0664430752577744e+01,     2.2621830421357976e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.7153583266577911e+01,     1.4253324094836609e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.1697865141868412e+01,    -1.6634840944114658e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -2.6959813690487813e+01,    -1.8577270389809746e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.0242749680756475e+01,     9.4374199298033272e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1479841422081869e+01,     8.3259512756902241e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1628337006851325e+01,     2.4035739024544808e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -1.5900191151205799e+01,     1.6787302511187376e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.0049510920400301e+01,     2.2877349107461654e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -2.3627249951228688e+01,     8.3314315735566761e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2080786355836317e+01,     3.3536934402648090e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.9539140484581065e+01,     1.4381740324358690e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5415056603114882e+01,     8.2909540655092387e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4431934217909084e+00,    -1.6639358641045103e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    8.6187643497719257e+00,     6.5541160910794615e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.4234985556403785e+00,     9.0629372040913750e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1946710889159645e+01,    -7.6643354231743069e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.6245486111420572e+01,     2.0349525009580349e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.4618013075663967e+00,     2.2738768211884279e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    5.9196288125887904e+01,     6.3485968374025661e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4971129306571378e+01,     1.2620755236592184e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.1037277898988274e+00,     6.3115249910333455e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2120828925953788e+01,     2.3869303476383426e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    4.9123590545883410e+01,     2.2761748187597736e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.7414829277080148e+00,     7.7062282824629665e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2803250859116076e+01,     7.2166658892984269e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -5.2642042779831641e+01,     1.5540100774082310e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.2860687937227439e+01,     1.5545407805881279e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2457472517743380e+00,     2.1042746665742208e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1788539753869607e+00,     8.9136278482464131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0916235670026850e+01,     1.5546567581794930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5919440127529001e+01,    -1.6656286968379412e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.1488765981489969e+01,    -1.2302033496625882e+01,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    8.0178705150373375e-01,     2.4094113091539810e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.3756716575095680e+01,     3.1861008823715278e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    3.6349565372605781e+00,     1.6344562924360042e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.2253595428305623e+01,     1.5136615409090540e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.1577664314986613e+01,     5.2339682781374604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.9992881644855238e+00,    -8.5605809591671077e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.5099148450459499e+01,     8.0177442338527544e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.1315352841508801e+00,     1.6526870966762497e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.3274325895441130e+00,     8.9738597563165622e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.5027824589388970e+01,     2.3870479312204181e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {   -4.2213305387780096e+01,     7.8353705735600724e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.8740631396548110e+01,    -1.4159773830261315e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3915892113995181e+00,     7.2735814424297596e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.5566925515665311e+01,     9.8255242423246873e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9959642067500781e+01,     5.3926533617668974e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2646768478285814e+01,     1.0922103858133931e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5826555664856553e+01,    -2.0037593733613539e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6655639918189195e+01,     1.3722265884659024e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.2740460505826357e+02,     7.2136124978264297e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4662895580770615e+02,     7.2145132328902591e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3546592527180241e+00,     5.6632013536934428e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9875508981949341e+00,     6.6234926394270133e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7583678337493595e+01,     2.3881754995860734e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4746742229827461e+01,     3.3511197253031838e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -8.7879425797488295e+00,     2.3241060586775999e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.8212583496132417e+01,    -1.4989796608814859e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.5555290684936388e+01,     2.6065278222823606e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2183357403108495e+01,     4.5892730430743832e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2429469880004079e+01,     1.5760007221393642e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.0240808691875159e+01,    -1.2896852520793705e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1965681443333075e+01,     7.7582592337926308e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.3877466680506276e+02,    -8.3270695193072752e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.1043175557048194e+01,    -2.5004123217326218e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    2.9523817019083083e+01,     7.6225029461910099e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5184558642824797e+01,     2.3870893425072762e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.6580575339774626e+00,     8.7988030635022951e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7020904431772962e+01,     1.7427677045118424e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3245884321812333e+01,     7.4121709724468437e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.6397661719577101e+01,     4.4130002444148568e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.8021233594458604e+01,     2.2879676030570117e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.6633594913505938e+00,     1.2116899251212193e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.1098387619677119e+00,     4.6625961670003977e+04,    -4.6257120609174708e-05,     4.3707204171321924e-07,    -2.4600415806860071e-09, } , 
      {    1.2112527802150172e+01,     1.5560386316256263e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2192553889480799e+01,     3.1320795195316143e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.5382061511034273e+00,     2.3871998004137728e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8452589112754584e+01,    -8.1174571930845605e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.6709899921616092e+01,     1.8374083171161012e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8875302582927675e+00,     1.6928710861405831e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.1369492099411858e+00,    -1.7084185246395049e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    8.1151142885998020e+00,     1.5520713036537067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0944859621483292e+02,     6.5852473216736835e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.5057630575140060e+00,    -3.4132358003516165e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.9274791463738310e+01,    -1.1281350705364596e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6223408514575105e+01,     7.2363543049567779e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.1730944571197291e+01,     7.1917714257599127e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -7.5599927375439808e+00,     8.3664287837376614e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7435124893317507e+01,     4.7167552941376140e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {   -2.7402153151571561e+01,     1.7007952523408123e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6033404022489616e+01,    -9.4425142385366545e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.6679463228757427e+01,    -2.2682246250196977e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0605600258059962e+00,     1.3769165223312450e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.9608597929437174e+01,     6.4558356183723245e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.5962531879318767e+01,     4.0802812379402305e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    8.0938323691798608e+00,     1.5569984252044875e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3557017988999620e+01,     1.6230580452098753e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -4.2011025291296541e+01,     8.2006102205468796e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6822744716708623e+01,     3.9587701053919751e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -1.5626686466788332e+01,     2.4636008260032686e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    3.5676833692319775e+01,     2.4640610659915506e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.9717657006575735e+01,     1.0968504524007079e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.6613945834990833e+00,    -7.7871343642442589e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.8925124835080069e+00,     7.3729701569919189e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3079175587632042e+01,     1.1474728341577588e+01,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    8.0304645605223577e+00,     9.8478156819231190e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.1174260573482728e+00,    -3.1166593907153731e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0547291899932409e+01,     7.2927262667531068e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3052682781108452e+01,     2.6464858148115723e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9937870603692094e+00,     1.5103332654415661e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.4830143997130021e+00,     8.9842214815250190e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.5218081440350679e+00,    -8.3464733438546118e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    5.1900867256195049e+00,    -3.4925013613602368e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.8072485962999831e+00,     1.0513663777301266e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7835202072008730e+01,     1.0680258287182241e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.1027620631557571e+01,     1.7794553313644250e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.2762679022881498e-01,    -3.4907880613637394e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3171703868167508e+01,     3.2487238865977120e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -4.8255272333706422e+01,    -1.0194567271808514e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4923605578575231e+01,     2.1373634485562750e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3066079196007081e+01,     3.3301647787295537e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7307794996139570e+01,     2.3861136433349755e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.4958807445335722e+00,     2.5257980546636809e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.8413359299012399e+01,     4.6630564069886794e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.0257536016357008e+01,    -1.2264068736126912e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8092801314883257e+01,     6.2291521317088300e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2153174640690185e+00,     8.9555763187352386e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7345894650217037e+01,     4.0103917902581816e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3281497286953659e+01,     2.4362682461567245e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.5643737631712366e+01,     3.0040769798717269e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.2339777232915424e+01,     1.5232216613558527e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6716470127389620e+01,     7.1791179763489536e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    9.4176873568817676e-01,    -9.6992701154980059e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.8959672986703495e+00,     1.9548215047982383e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6394765603849550e+01,     8.5104929373473424e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.0132371841926453e+00,     9.3346008682223019e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9921948775355631e+01,     3.3530348638672847e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {   -1.3784488253248455e+01,     1.4930179188935546e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.9972576210146819e+01,     3.9336647743574911e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.1341933596048385e+01,     1.7325505336190356e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2000875517581632e+01,     1.5864781954261793e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.6613965432401947e+01,     8.6134919514304638e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.1084825229462954e+01,     1.7408489375362391e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.0925615346050819e+00,    -3.1166854654127746e+01,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.0148570515881353e+01,     2.6340905221230460e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.2808831214629999e+01,     2.3873587952827067e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.5755171452139010e+00,     4.9064768254270486e+04,     2.6268171641017767e-04,     8.0358391242681175e-07,    -4.1748275707704355e-09, } , 
      {   -3.8884166227004286e+01,     6.2678233179432573e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1279876895084016e+00,    -2.5224191280974901e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.4015418949278683e+01,     7.4619687191288876e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    9.6712772867301418e+00,     1.6082755706879425e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4428478146366196e+01,     3.1072390668765322e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -2.0023818340626676e+01,    -1.4436227306951439e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.5427921851803234e+01,     9.1972166572791430e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0504616703640764e+01,     7.2167163812578265e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6305632562749935e+01,     1.0992094481254280e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6429794660926945e+01,    -6.6921731960376410e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.3477272714398183e+01,     1.2181348860541202e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.4278006669490999e+01,     1.5541772628065579e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.8213466519707211e+01,     1.5543735951898010e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8383656504338539e+00,    -8.9132281761120303e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.0422012674116825e-01,     4.5496571264348731e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.6492819033887702e+00,     1.1377257977699364e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7073856732814907e+01,    -7.4029640743529053e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2642998290154432e+00,     1.4456991385429406e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0043354441321025e+01,     6.8030082117246475e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -9.6745552060645874e+00,     6.9248835958262189e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    8.1337262339573009e+00,     2.9097278877134784e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5580139795443841e+01,     1.6158547123289136e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.2494234127179155e+01,     6.0145355939308383e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1617589458922259e+01,     1.5010006678814043e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0043157672123399e+01,     1.5478764566048239e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5474839963603011e+00,     1.6080890375597228e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.2988829589119913e+01,     2.4440234722579371e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.5182121624165660e+01,     8.3993560976616664e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1315344092893490e+01,     8.6038197502500188e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.0579126852429585e+01,     2.3384772424427611e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.9030506843679902e+01,     5.4832546529164358e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0476410716852307e+01,     2.4981143241612754e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    5.7460314161418523e+01,     2.1500554301207128e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1115583805233101e+01,     1.6100779915081224e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1162768285221834e+01,     9.9809573967237084e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5177366094237996e+01,     3.6186937455400948e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6718426437728127e+01,     1.7497541099970153e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    9.2931890330090603e+00,    -9.2855856295481226e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.2305962344123644e+00,     1.6282395563494623e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.3025381460972119e+00,    -1.5540487301033356e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.7525338869068371e+01,     2.6294005882577025e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.7736910417926389e+01,     1.3148968655493358e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3286645228788430e+01,     1.3799811432120419e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9479877668030394e+01,     1.6658486376641107e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    9.2234496465160074e+00,     1.5538940998168659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7479651023150549e+00,     5.4874711939175040e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1241430311248052e+02,     2.3243236181133543e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3639926349996614e+00,     3.2748814920478413e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.1293272031554167e+01,     8.5663278238576368e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.1936068739316754e+00,    -1.0646966056768450e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4546670516041129e+01,     8.5482302964615556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9431484701736522e+01,     9.0240631917990358e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2843574893224389e+01,     1.6244658204380412e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.8899100707681047e-01,    -2.1293981584045023e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.9474648105444253e+01,     7.8074255613944970e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.8748378983581699e+01,     7.8254497695962955e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.0783827039003413e+01,     1.3896720946232639e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2655675229444244e+01,     1.0133984375964255e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1757927444789717e+01,     1.6853769450039017e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7338276951763412e+01,     9.8228531794183073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4872845538035932e+01,     1.8751356397339550e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.4384178403292549e+00,     7.8482705044891172e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3718738720992313e+01,     8.2480106585148624e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.3295759915436005e+01,     8.8702484850026412e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.0592125869198030e+00,     1.0229613687984078e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5019943195260833e+01,     3.0850221964611035e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.4692129916665841e+00,     3.0627992076554597e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.0148559833195861e+01,     1.0682662157213974e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2560495595804902e+01,     9.0678340253559982e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -6.1392512018542398e+01,     9.6406635249008777e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6623861953577421e+00,     7.8274141172657746e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6513928349933402e-01,    -1.8347150395668598e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    4.9240202864840832e+00,     1.1823161649020431e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0348253458873149e+00,     6.7233326639685238e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8516300107818982e+00,     7.2518002244725558e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.3808517641274083e+00,     1.4940755095562808e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0652108516881977e+00,     2.1120298926754331e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.6340668766904338e+00,    -7.5234712912203549e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    4.1788569648916525e+00,     6.6015030238272520e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2445420023870680e+01,     8.7643102905802589e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.9512865558257637e+00,     2.5593051950938039e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.2546856063198121e+01,     6.9968251934616510e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2832530734021237e+00,     1.5951523003189232e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0114473835601917e+01,     1.6664386430726336e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.3604624764147449e+01,     1.6858072159442658e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -2.0116245737775895e+01,    -1.6875792470523746e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.8653097081582040e+00,    -1.2690666895683948e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.8728007344902153e+01,     4.9069370654153303e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    3.4569175842938392e+01,     3.1317398114507247e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.9959412852966715e+01,     7.3872163308384552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1059929469589690e+01,     8.7649243770232461e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.5463796330479795e+01,     9.0542515661672951e-01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.0164332967636458e+00,    -1.2370234810020611e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5201619980480482e+01,     3.6922932492611353e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4878060561946032e+01,     2.9042851470486527e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    7.1301724438628664e+00,     1.6099357224147472e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1058218805059990e+01,     1.4308681025022952e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.5028232443253930e+00,     6.2265157380749752e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.0074533756578788e+01,    -1.1011220479937515e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0497481015237952e+01,     1.3579950641458268e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2239157528983043e+01,     2.4928986375206769e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0818055779520179e+01,     3.1005326486635284e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5223102604080715e+01,     3.0499860810873750e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.4454483071262558e+01,    -8.2663346175932929e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5956711621795584e+01,     1.9494623229826175e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0543689571162144e+01,    -8.3381339388619836e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.5490226117515036e-01,     3.9121607534740124e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1857255058360103e+01,     7.0841432034178670e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.2878692924626671e+01,     3.0421152578514448e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.1157295808309602e+00,     1.6813963217772009e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.4409248544371984e+01,    -8.3313624875298283e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.9503645532085933e+01,     3.1691274259876889e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.4225384011241902e+01,     2.5868534779113470e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2043917198605577e+01,     1.6566712677036507e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6072154130195273e+01,    -4.2450119720673865e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.1354221741565240e+01,     7.2216894489846154e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8181858455706280e+01,     1.1033890369421893e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.1070064957597413e+01,    -1.4217856738230353e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2317244308913580e+01,     9.1899844250801380e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5521293895141859e+01,     7.1453707598867659e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.8268687599603414e+01,     8.3314228753591979e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4466438427847795e+01,     2.6165137182949311e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2839064602312828e+01,     1.6569468863026315e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1912610040342640e+01,    -1.1167707974869256e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    8.1461042175271814e+00,    -4.3335187700837110e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.5120654167224437e+01,     5.2616781253116471e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8372074871192371e+01,    -4.2910359708955912e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.9577270737667696e+01,     8.9590765524526942e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4591324696286826e+01,     2.4429863521632007e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.2210944670509960e+00,     1.7121363754659105e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.1594459101255062e+01,     2.6071837178422735e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.7823048293528117e+01,    -5.2710227851853688e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.8553390383909637e+01,     8.7190760006825822e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.9148278716819664e+00,    -7.7587579065729115e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -5.3532160130394395e+00,    -4.3570120940780243e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.6880023545263105e+00,     5.5901638751905248e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -5.5563951695337401e+00,     2.1497900785307640e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.4506276186945421e+01,     2.6857879117808712e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8115976167263980e+01,     7.1983151218438952e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.5857504516230321e+01,     7.2150445272745619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1992069443989610e+00,     5.9923925842766503e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0015109445902189e+01,     8.1878083411916750e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6594285548047503e+00,     1.4788666491121003e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5097973616991368e+01,     2.2876261861033876e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.1475661834035236e+01,    -2.0341009927960840e-01,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.2567690892664313e+01,     7.1342094044048099e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6995149813261072e+00,    -1.0966043510633040e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    4.1575249894117093e+01,     7.3222833082443176e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.6783986428517466e+01,    -8.6852523265569107e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0310420294843077e+00,     1.5899357438585766e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.4399471753729848e+01,     2.4346949977567951e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0220473900855875e+00,     2.3782428200990456e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0331847319277079e+01,    -8.7090931558984503e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.6006117317896624e+00,    -1.1124863210432839e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.0276249369045050e+01,     1.6580941605836037e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.1358944767586735e+01,    -7.9268539861218023e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -5.9139001854342467e+01,     1.0268971109946422e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.1678034805305728e-01,     2.4754184739326694e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3801736776303535e+00,     5.6577267301008451e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2580617211591829e+01,     9.7027005660006726e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.5607920970786742e+00,     8.3922065634673836e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9011566222734427e+01,     8.4394996319343518e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.3413221986791566e+00,     1.1770278947392351e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4613701524915683e+00,    -2.3072540054226365e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.7236095857200908e+00,     9.4121531292344262e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3380164964771906e+00,     6.2330746936741070e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.7875974353635482e+00,     1.1720068836465555e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7322095747012064e+01,     2.6951179122335285e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.0522905432474147e+01,     1.4764826743039639e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4160396580757826e+01,     1.0784833866142042e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1054248200681176e+00,     4.3903395233977762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0291410873729305e+01,     2.3385124740385683e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5163357560440925e+01,     2.5021411344807071e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.0994166532564194e+01,     8.3059438042651836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.0344788690918829e+00,     1.0984237008006377e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2127684266607464e+00,     1.0120642807608278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5088906971375614e+01,     3.3799660415159700e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6546286269584400e+01,     2.4970532245581919e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.0583596915791532e+01,     6.4762982188629394e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1684318255082015e+01,     2.1641836280116964e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    5.0506692205790493e+00,     6.9008613146817333e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0759500201874775e+02,     6.6498430558201962e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9357400002886664e+01,     2.1642540912033110e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    7.0772430513737001e+00,     8.3298024422247108e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3661339506000951e+00,    -8.3275804070221911e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4602155703102078e+01,    -1.5225973074783474e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2276584418217947e+01,     7.5268827902452040e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.9753124145324129e+01,    -3.8848499496848945e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -2.5331674878504209e+01,     1.5819727634789941e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.0088407438083934e+01,     7.3668661874411828e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1450795830733806e+01,     1.5062333369847462e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6784236078259429e+00,     5.9361177872894059e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4893994343825003e+00,     2.2666645655910026e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.5118116329463449e+00,     2.3413990395098401e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.2130000208150186e+01,     9.3884526146881035e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.3727614709889320e+00,     1.0059094435988518e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2186023593556570e+00,     8.1889907542237288e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3019314219283181e+01,    -8.3337584966651375e+03,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {   -4.0235619419887300e+01,    -2.3164594565641892e+00,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.7458848938619013e+01,     1.4293617286459765e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0237303015010497e+01,    -1.0719687433094020e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.9271190696885185e+01,     2.5001616302158778e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1701745330383064e+01,     6.8494278290756147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2112500655665315e+02,     2.1755943575404615e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7244030731524687e+00,     1.6660227763304691e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.8987598059332615e+00,     9.3772424512180078e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.7441123937989831e+00,    -1.1430144926753642e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0842191592132755e+00,    -8.5720223178149117e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    1.4125574470661858e+01,    -1.3579594524565648e+03,     9.2743731943700382e-05,     1.0921223261429318e-07,    -5.2374421970263164e-10, } , 
      {    5.4580599017038995e+00,    -1.7319466231916122e+04,    -2.7100635671717702e-04,    -4.6565006288537437e-07,     2.3518263189255473e-09, } , 
      {    2.1591165711832208e+00,    -3.3558096591685266e+04,    -1.4565790745711834e-04,    -4.5434323821453780e-07,     2.3562429715604555e-09, } , 
      {    1.5854983726173305e+01,    -1.7767119419343255e+03,    -3.3718953284101068e-04,    -4.0353234036585289e-07,     1.9438071249037621e-09, } , 
      {    2.8678526146596454e+01,     3.9589968042868182e+04,    -3.8175651344839385e-04,    -1.0076978413418789e-07,     1.8650782312283823e-10, } , 
      {    2.6322971596209651e+01,     3.1261276618244734e+04,    -5.3404892121072009e-04,    -3.5148865828884261e-07,     1.4224918217434891e-09, } , 
      {    1.8993171191147656e+01,     4.2028774627134691e+04,    -7.2817676429041446e-05,     2.6574208657940457e-07,    -1.5282781669615902e-09, } , 
      {    1.4282054759991192e+01,     2.5371391777887788e+04,    -3.7740249195369381e-04,    -2.3569566172990487e-07,     9.4368983027971148e-10, } , 
      {    1.6268835854662832e+01,     1.4461644314216483e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6287810505979316e+01,     1.6654537935189110e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7270550570996750e+01,     5.5770649154700795e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.4379348765241090e+01,     3.0945823169079013e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.1045867994043448e+00,    -1.6613612015689938e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -3.0997735010793093e+01,     5.4004128286688456e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.3043087171751566e+01,     4.9641574900863925e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.9695053795624112e+01,     6.5847888779117384e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.5130633391291298e+01,     8.3514390449817147e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1069406876860661e+00,     1.1293470676908516e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.1778034359664566e+00,     4.5742497659747169e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0048722150738282e+01,     1.7269481098951725e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.2557864990747959e+01,     1.7086615924802882e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.2638213582866790e+01,     1.5933903432117104e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6244782363854416e+01,     1.5087600170416365e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2364260506670384e-01,     1.6165028400584586e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6909039610660038e+01,     2.3826563740665646e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.3191648134458349e+00,     1.2491370035220295e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.3430520195212146e+01,    -2.7267083884263411e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.7862844120176852e+01,     1.0999450403920033e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3256325589166948e+01,    -1.4323984678873821e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6627590371651033e+00,    -1.0857669508320072e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.2206139460631263e+01,     1.6625902621733148e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.0011625889437283e+01,    -1.1139239273489559e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3913768458554591e+01,     1.0735983570383240e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.6130831597286193e+00,     9.7276218379152888e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4720445819270239e+01,    -2.0670077574564857e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.3737497395806422e+00,     9.0993142528697463e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6007959472769855e+00,     3.1292756845758478e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    1.0168377462111884e+01,     7.1967500026234220e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.2328258139980921e+00,     6.3932821176790719e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0111330570783558e+01,     6.1770018266205152e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.5335285611244110e+01,     8.3109711135423622e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.0046394712017708e+01,    -8.3464117357045379e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3217804351574582e+01,     2.2931618791220222e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4178835495329459e+01,     8.0465054706174678e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    7.0762167081930771e+00,     3.2370978991055063e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0307080376061813e+02,     2.3015856606642046e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9073985792539602e+01,     2.5826232524593604e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.0978424179936630e+02,     7.2781506408657388e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8328055197114153e+01,    -6.5255859053604581e+00,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.6633132830436324e+01,     7.2130812034421269e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1169405795107853e+01,     2.4516673980458381e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -9.3503209247693377e+01,     6.9055299055953583e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.0920854529005638e+01,     7.2262687900823230e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.6571263109482974e+01,     1.0207055224160948e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.3347674018564810e+01,     2.4340493692059139e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.8408915342292190e+01,     8.6384797944856946e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.6867791847732136e+01,     1.0585759356652481e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3206950508078126e+01,     1.5469166630259628e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7172180054559348e+01,     6.6735549755270767e+01,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2935285155125486e+01,     7.2005003282797752e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5131056291169127e+01,     1.2618140663791153e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4529008846773543e+01,     1.6701153682803866e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    3.8946742098263513e+01,     2.6687026852270013e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.0197630126917517e+01,     1.8954265683173846e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9847643915098850e+01,     8.3114526446114214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.6897426222396874e+01,     1.8620204439702116e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3978220124924846e+01,     6.8053094116660595e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.0801008986308140e+00,     7.7320230288112461e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9563500768025150e+01,     1.5038399631821523e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.0588218703758457e+01,     6.5852029907803180e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0079845313640618e+01,     2.4539568201548700e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.9413597946178122e+01,    -1.0592716290787639e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.4133698414504661e-02,     3.3640352617567773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2594363380476503e+00,    -1.1572883752208093e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.8592692720499969e+01,     9.0983545070012042e+01,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4025964510533182e+01,     3.2428874274721027e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -9.3571959924818671e+01,     1.0251698113180753e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3066379992676982e+00,     4.2871480392262911e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.0606949350376588e+01,     2.4030705322196893e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.0316024755814254e+01,     8.0315850322580700e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.5499985042084063e+01,    -1.1153331911812554e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5305973444181532e+01,     2.0402494106791542e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4818707421737507e+01,     1.4694579200723376e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.8272537985038362e+00,     9.6382756919257281e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5170575887766498e+02,     1.1130459161857289e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.3448815558967581e+01,     9.3275269272509647e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.2223573912999484e+01,    -7.8153371819444033e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.7351598314699743e+01,     2.6050522475881735e+01,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -9.0795501792041620e+00,     1.8695036439969517e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1727391195863140e+01,    -7.0256755886592173e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2939168148967783e+00,    -3.4159825106025128e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7524318697168875e+00,     4.5490930642134636e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3214656279648999e+01,     1.7081056741298584e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -5.6387834045282226e-01,     3.2194103664785940e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    6.3120057522302533e+00,     1.5933515290323567e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3066601819793453e+00,     7.5420906343873212e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.4342274250136668e+01,     3.9766103628030214e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8485199253749567e+01,     1.8148728610846661e+00,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    1.1763919893716386e+01,     9.8412567263239871e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.3361555650491191e+01,     1.5231234951642311e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.3901368900765103e+00,    -2.7941307529957442e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4440649687182178e+01,     1.6678046365503793e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.8365051951770823e+01,     2.7938608128429132e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.2902423472273243e-01,    -9.4083750948791640e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.0500817326158405e+00,     8.5393948761750726e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4385458303564349e+01,     2.4080283834874670e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.9322798340024939e+01,    -1.1277464704633730e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    3.1296470329338398e+01,     7.3031051142271972e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.6385633942750530e+00,     1.1300584131577282e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0053445997354262e+00,     5.9394234870654591e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7842148959570260e+01,    -4.9028363858538487e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2237218072264458e+01,     7.2136670313183340e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.3270317984357728e+01,     7.2144586993983567e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.0174775522291643e+01,     8.3281927518431712e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.6462268969356220e+01,     2.3808167336138566e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1518060060201609e+01,     1.4374502804139170e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.8821972240296475e+01,     8.3283285864615664e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6674673994659869e+01,     8.3290542627853338e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6705548294771404e-01,    -6.2831810724935235e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9873231097673834e+01,     9.4920065901879625e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.2364217506815791e+01,     1.9328141828543485e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8299443169012877e+00,     2.0649725696049223e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.6553390009411473e+00,     1.2334152182624037e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5775568399790004e+01,    -1.1211875148642366e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    1.7291810698620218e+01,    -9.4035050819906410e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.3133413673909149e+00,     6.8805604727367718e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7632730199387218e+01,     2.4081706525808422e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.4409256158439852e+01,     2.0856146656689059e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.1688939056478784e+01,     6.5859036973700149e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5915546766637078e+01,     1.3737684995798698e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0903421961760333e+00,     1.6177620830626473e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.4352109185028347e+01,     2.5022834035740823e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.6662299745923860e+01,     8.3104520254567196e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -7.1373408846697473e+01,    -8.3469308237901823e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5181243273665668e+01,     1.4914670362093959e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7373071622431919e+01,     8.4215117078475650e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1182245049669532e+00,     1.8635928471587777e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.3641770004052880e+00,     1.4905810206553419e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.7087631383457893e+00,     7.0281959946114029e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.9417816663768122e+00,     1.6605125097502583e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5799915255112511e+00,     6.5782771137437439e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3603083979861282e+01,    -1.8381079763949028e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.6523337297523786e+00,    -3.6578902209805034e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.0578829506832026e+01,     7.7790920719372089e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3741886304040996e+01,     6.7872604682101974e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.9702718394526386e+01,     3.1297359245641306e+04,    -7.1725590628761307e-04,    -6.3861160998159502e-07,     2.8330572269316835e-09, } , 
      {   -5.4428795867091679e+00,     1.0591216038313389e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9270004871941948e+00,     1.6656219387692669e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6540329149293020e+00,     2.1940122293595472e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    6.7788366932174169e+00,     9.0606102809829117e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.6978061309280825e+01,     2.6458299192516592e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.1661869748239415e+01,    -2.5381333791367897e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.1026615635275370e+00,    -2.9400327373303207e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.9316936712850059e+00,    -1.8347349896346357e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -9.1378357583032397e+01,     9.3291441865613942e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7885471504344025e+01,    -6.9017226617739443e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.1831210018595471e-01,     5.5049680826628819e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6288191936215739e+01,     1.4872498234447425e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9697767724368610e+01,     8.3260203617170719e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6512247561955746e+00,     2.4880136079447966e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.1587567051578166e+01,     5.1737069558226940e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.2979346500556904e+01,    -8.3259599738877041e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8497750428694197e+01,     1.6028746631826583e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0098836571088416e+01,     1.2165172694386092e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2541760406327544e+01,     7.5775249643479310e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7130349731845747e+01,     7.5521663762974640e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6402002898276358e+01,     1.4286467752265480e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.8428448405446609e+00,     1.4871786888980549e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.3182026500534292e+00,     1.9155194078289394e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2476670458146151e+00,     7.7960127527448012e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5900187722788655e+01,     9.6589297324437102e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.6889836175062825e+01,     1.0897278694991965e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0125788192721181e+00,     8.1168150357786844e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0634380872631557e+01,    -1.6892557016325811e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2829143380782902e+01,     2.3855697971090794e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2660184549586225e+01,     1.4914457227140061e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6489842717949710e+00,     1.5007746709462826e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9536380707758237e+01,     6.5829103662550096e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4086172933605624e+01,    -3.4551857567332291e+00,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.9820265452450187e+01,     2.5900984912884043e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.2890725755977250e+00,     7.2276254024369136e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3457577586178729e+01,     9.5849542787564606e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3956734312443658e+01,     2.9138452674954515e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0203435144461190e+00,     4.1566361567418862e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0852086586093098e+01,     8.2944578810809344e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.7621084065501851e+00,     7.3875449632132277e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1632742184464036e+01,     1.3625016509791067e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0979007087433569e+01,     1.4054893688909733e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0962176462696478e+01,     1.4882807515702912e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4569806718938429e+00,     7.8379612232336294e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3653679433701107e+01,     1.5631796538850645e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.8500447957293247e+01,     1.4230782189704367e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.1244633775796569e+00,     1.3722764557439301e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.1102944572146761e+01,     1.5651786778507671e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.6969785185731830e+01,     2.2428230535461014e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    8.4327998257237144e+00,     1.6093064988134913e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8463998811224343e+01,     3.1572540899650030e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.2906336068817312e+00,    -1.8384961181884394e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.0217721325284558e+01,    -1.2566084565403598e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2112560399165426e+00,    -7.9164922609133428e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.6382365012702760e+01,     3.2932055995588154e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.6033801896631761e+01,     1.4322187536294306e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.1670257372549138e+00,     6.5853620340503594e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3633263208335359e+01,     3.7540510332602935e+01,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2235648746050147e+01,     8.6457984749810130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1062060544421573e+01,     7.2191299374000337e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -2.4523006316498073e+01,    -1.1493763891213284e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.3602810560075376e+01,     2.4160624984137372e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -6.5463490801320859e+01,     3.1582184430499728e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4608161492323721e+00,     7.9110967668684398e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3818403199265260e+01,     1.1315124827826918e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9494781226913602e-01,     3.1300125117741583e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.2721939158769890e+00,     8.2124439489455799e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9421939662569255e+01,     6.3758534986184250e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.4933768265423852e+00,    -3.4689516078137044e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.6615048537828059e+01,    -4.9375935752650548e+00,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.8187071905765624e+00,     7.4585382686262174e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0117541958861572e+01,     2.5114942973498066e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.2287463321742420e+00,     7.7702136834366665e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.5553565788481293e+01,     8.0404785574923862e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.5218797451777149e+01,     7.9859936827688889e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.8418698109699925e+00,    -1.0616689377928246e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0042259554645804e+02,     1.0543023578326574e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8622098629996337e+01,     4.1104167051686491e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -1.4106225279092961e+01,     7.4995931198940525e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.1976457196397519e-01,     1.6081333015945673e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4645829802934315e+01,     2.3465306833713992e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.6189533824979751e+01,     1.6100397201989119e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    3.4439169177656360e+01,     1.6099739937239576e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.8417123699071368e+01,     5.5680295216091881e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8127448369501266e+01,     8.3279754877561954e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -3.2838557470274864e+01,    -8.3294073614907011e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.4312406488965934e+01,     1.5543917751536026e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.1909881522503614e+01,    -6.9622275591393861e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.9350801386019761e+01,    -8.9750479531248566e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.4188039895645026e+01,     1.3905406749412577e+01,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {   -8.1721649042821724e+01,    -1.1389014328745880e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.1156037783906227e+00,     1.5548654344067023e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5101284834041714e+01,     5.9649258618363147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7250656480937661e+00,     1.3072183765755326e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8143646691667190e+01,     1.8767104140854000e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.8351434788897549e+00,     7.6999905933154650e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5753315821198470e+01,     2.9733154920052002e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3011418878647579e+01,     8.1452532903605370e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5207959704513652e+01,    -8.3183733861468736e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.5405247173931009e-01,    -1.6733824092657764e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.7006567654961309e+01,     2.4771457736092358e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.6564826399153407e+01,    -1.3937203802494878e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3294014967335032e+01,    -6.6326546189662295e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.9433223023962213e+01,     2.8073983030246143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.2981640885721610e+00,     9.8051740989274149e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3457097852633106e+01,     1.5644925998909863e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7376465850890057e+01,    -7.5538901702727765e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -1.0547362634510993e-01,     6.7847492336375126e+00,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1865419574174112e+00,     1.4914053458673810e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1971769450336966e+01,    -2.3689168062477279e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.9878169973791174e+01,     3.4179375426598790e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5391634104284186e+00,    -7.7882978257984914e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.2510294839685841e+01,     6.4122884419730708e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7912207471739485e+01,     3.5737325866465580e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5334823832802650e+01,    -1.0087093163205436e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.1999904521882181e+00,    -3.3150200230825994e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2134973646173755e+01,     1.5837559482405766e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -8.0761765876014771e+00,     1.1723878281519204e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7025976030329865e+01,     2.0088756820219010e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.8652235370991335e+00,     1.0198195068620402e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4554685144364527e+01,     3.2184595110940267e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.3057975307272114e+01,     1.7043844863340753e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -9.7167876489479852e+00,     1.5453712041112944e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.9031116810304063e+00,     7.0747060868485460e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7659251245441752e+01,    -2.4858845489899437e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3983903105697536e+01,     2.2287769177886246e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6952776573956022e+01,     1.4757866336411142e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -1.4314917299022270e+01,     8.1620969147424767e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6639691820242518e+01,     4.0853142189541227e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2070436234712894e+01,     7.1250206164894935e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0972975338674544e+01,     2.0855894891979511e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3660628230511975e+01,     2.5761299501999467e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4508169392045183e+01,     1.3887732131891729e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.4055687877545619e+01,     8.5236471326606224e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9060971710637569e+00,    -8.0682265105047954e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.3919901581828544e+01,     3.9816636895299609e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3517589106117939e+01,     1.5426506814303924e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5701996489419582e+01,     2.4646945958356930e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.0396043516455489e+00,     9.1327857984497496e+03,    -2.5215197505721926e-04,    -2.2434520382776838e-07,     9.4883370343628348e-10, } , 
      {    2.6698286598650043e+00,    -8.9197627513800653e+03,    -3.0685975863558652e-04,    -4.3393117628015106e-07,     2.0941042141845293e-09, } , 
      {    1.1124261210180642e+01,     2.5648133276208475e+04,     4.2339669684308734e-04,     7.1632530380872929e-07,    -3.5885375380678631e-09, } , 
      {    2.1159237122845035e+01,     3.2342172510716868e+04,    -1.3256221697483553e-04,     1.2432271150164275e-07,    -8.1458394700024244e-10, } , 
      {    1.4010299670405750e+01,     2.4462517041361858e+04,     2.4067658251175289e-04,     4.9604889879967261e-07,    -2.5021234082063148e-09, } , 
      {    1.8314225156639763e+00,    -9.5515382879275076e+03,    -1.7778128972749182e-04,    -2.8839936915237763e-07,     1.5187176411701100e-09, } , 
      {    1.4423200875509899e+01,     4.1886763635977608e+04,     2.9804824758302871e-04,     7.0501847913789266e-07,    -3.5929541907027709e-09, } , 
      {    2.0388966838200993e+01,     3.2794682047318696e+04,     2.6655440285889285e-04,     6.0061957371239121e-07,    -3.1082811055607563e-09, } , 
      {    2.7687432772649238e+01,     4.7709134382086428e+04,     2.0303510822796778e-04,     6.6212090411775007e-07,    -3.4618605713307522e-09, } , 
      {    2.5375003735586038e+01,     3.8752141002248914e+04,     5.3406581958550106e-05,     4.1078563784893267e-07,    -2.1714368444583117e-09, } , 
      {    1.7796600350243587e+01,     3.1815141696874995e+04,    -2.2234827033487499e-04,     1.4363187079287331e-08,    -2.3858166061081376e-10, } , 
      {    2.6074788543213792e+01,     3.1680053488053589e+04,    -1.0421358878959312e-04,     1.6129954792260326e-07,    -1.0443323023412404e-09, } , 
      {    7.4707597699629043e+00,     3.3767621677090472e+04,    -2.8684130645691705e-04,    -5.7828575882745452e-08,     5.6141424272484197e-11, } , 
      {    1.0565638299726562e+01,     7.1093181426527735e+03,     3.8893838197436232e-07,     6.6890179914995668e-08,    -3.2342405480498253e-10, } , 
      {    1.8751683109073536e+00,     6.9581604946218795e+02,    -6.4297171394873837e-05,    -7.2279029424632500e-08,     2.9326864383996938e-10, } , 
      {    1.4084768380989363e+01,    -1.8477352575098957e+03,    -1.5437148614609209e-04,    -1.8329956858809611e-07,     8.5666577452054988e-10, } , 
      {    4.1869843964336377e+00,    -1.2228468633040575e+03,    -2.5488881965165645e-05,    -3.7680494997722910e-08,     2.8273364254945920e-10, } , 
      {    1.0593603535507455e+01,     2.5232854368064098e+04,    -1.3295115535680988e-04,     5.7432531586647067e-08,    -4.9115989219525992e-10, } , 
      {    8.7289021314819859e-01,     1.2903609575521841e+03,    -3.6144407936799688e-05,    -3.5214926541072256e-08,     6.4974729542299990e-11, } , 
      {    1.9877043739873177e+01,     4.0914170448200675e+04,    -4.4368360044111160e-04,    -1.7353430597908347e-07,     5.3639785677959036e-10, } , 
      {    2.7438833632093129e+01,     3.0004637067822547e+04,    -5.3395098884713595e-04,    -3.5153229152014247e-07,     1.4217646012218248e-09, } , 
      {    1.1167208071747460e+01,     1.5090244753379971e+04,    -4.6529979595756203e-04,    -4.1417913969122692e-07,     1.8856779645387285e-09, } , 
      {    3.8256155767780591e+00,     8.0409437382629858e+02,    -4.0444438281954544e-04,    -4.7506407798242305e-07,     2.1848177020569347e-09, } , 
      {   -4.4219866114597526e+00,    -8.3659520632434742e+03,     2.3728754745182761e-06,    -6.7550205260458206e-08,     3.7713656253510807e-10, } , 
      {    1.7302624076676683e+01,     1.6483881799914197e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    3.5592448113204362e+00,     3.1532014652285241e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.0056112772270964e+00,     6.5857447581089928e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6610596062896086e+00,    -1.1184313288744311e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.6254419346991789e+00,     1.4239738454017563e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.0958110398354911e+01,     1.6774776820533039e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -6.3517490347380360e+01,     2.3258680157848132e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.2725092554796760e+00,     5.2596646323672758e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1213856747918271e+01,     1.5225966043275736e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0869227051971038e+01,     3.1078949624364457e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {    6.2382649133796967e+01,    -1.5438616348210871e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2079837914221844e+01,    -9.6977514235644576e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.9896541382890149e+02,    -7.7008917982170224e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.9025995965796497e+01,    -8.4952859345044235e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.8959021741553393e+01,     1.7420677168274611e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.8960170699914261e+00,    -1.6848908106781404e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    4.3798023765505953e+01,    -1.2047333193076702e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.8010382877311510e+00,    -1.0267145730445218e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.4676324591278860e+00,     6.8198802772361091e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5461883014681852e+01,     1.5637671857681813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7751403285187131e+01,     2.4991005306127950e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7477875606872050e+01,     1.5573407212340489e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    2.5793894598741694e+01,     5.5109416229456092e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -3.5328338963555055e+01,    -1.6756516847296916e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.6922336000502263e+00,     1.6326495029426358e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2969227758111076e+01,     1.6285025395810806e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.7213502052410398e+01,     2.7637660262688987e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.1910574242809101e+01,     1.7337006872908030e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.0635443116803778e+00,    -1.0278875103376643e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.6626826382501974e+01,    -8.2612675455516037e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.0767518685882933e+01,    -1.0862426258548471e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3921548007503148e+01,     3.2206170613671446e+04,     2.6931621671538551e-04,     5.9995954836692873e-07,    -3.0545685978306309e-09, } , 
      {    1.1401002157334052e+01,     3.3525026509696974e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -4.6778492189044300e+01,    -3.9813166900524891e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6403783037983771e+01,     1.5058734515703643e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.6744005713067462e+00,     3.3523603818763229e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    4.3995234778701622e+01,     6.9682679240652751e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.6919103153592516e-01,    -8.3816510460957306e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.8376994698414837e+01,     5.3161049971007174e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8378449580329566e-01,     3.3484500262332193e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -7.2561851495310705e+00,    -8.1080414124027084e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.9500130923481521e+01,     8.2508105312365533e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5382047596795676e+01,     1.6419459840260490e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.8277361090075210e-01,    -8.7278298831341590e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.1553091732624566e+01,     1.7285361801917672e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8222920656200117e+00,     1.6498475557613325e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.5145476834469669e+00,     3.2330056801169172e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    6.8426774057220982e+01,     3.8786244735499567e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.5955001701768545e+01,     2.7250900087501701e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.8097795709749134e+00,     8.4449389003013202e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.8740742005307325e+01,     6.4988308777672501e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.3173708691093200e+00,     1.6184179786225606e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.8404679262094970e+00,     1.4914223047778542e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1921807325780541e+02,     2.3058080093360900e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4135348717511235e+01,    -1.1108257896557802e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.4843841814739363e+01,     2.8230187100444182e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4280280802984024e+01,     1.5232965920119550e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4513523004730400e+01,    -6.2396336492732416e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.7411368156187052e+01,     1.6855442327126475e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -2.3922989129815498e+01,    -1.6878422302839928e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.5261990590043371e+01,     3.1781026703961379e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -2.8661320514370310e+01,     1.9113525290765490e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4185384075044691e+00,     8.3143650981250339e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5445802433981157e+00,     8.5493414368441936e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.6916640362476244e+01,     5.6183197656299785e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9460535988024823e+01,     3.3564130066128011e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.2373680459307879e+01,     2.5455474567282316e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2271421079709800e+01,     1.5010075618103147e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2528746000359911e+01,    -7.4730368286021474e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.8348079181299163e+01,    -1.1813641090203746e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.1691215824350287e+00,    -4.2417256483196343e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1356424482974774e+01,     3.0379648733905920e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.2933541341943624e+01,     1.4913894415403767e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3446129964631041e+01,     2.5200690811648339e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.7783489830057576e+01,     1.0024728230280671e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.4323052375298928e+01,     8.8176269716440438e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1732540716106223e+01,    -8.2919316627529734e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1778146869251259e+01,     3.8392871449848506e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.2270388107790616e+01,     3.3314054353026928e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    4.2850556212097629e+01,     8.3270635012819112e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.1197696648540806e+01,     3.2215679167517126e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5998869006654695e+01,     3.3315477043960673e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    5.2200183777200655e+00,    -9.8543746375222545e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.2657743568047000e+00,     1.3846717484324574e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.2912810670604884e+00,     1.5054132115820823e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.7822425630618469e+01,     1.0450772622670314e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1552002294596463e+01,     4.5141150762018611e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5806943712855546e+00,    -1.0050297660956245e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.2091086950293189e+01,     8.9520679774113996e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.0333924066990123e+00,     8.7987058044784080e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0994524002085285e+01,     4.0006459173228985e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.2521153734615636e+01,     7.9199227114160112e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2727069356639031e+01,     9.1304658480087255e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.2868993731259522e+01,     1.6656570803606999e+04,     4.6351172359218639e-04,     5.4853225840408113e-07,    -2.5876930229221479e-09, } , 
      {    6.3446775529712298e+01,    -1.6658194894886801e+04,    -1.4565790745711834e-04,    -4.5434323821453780e-07,     2.3562429715604555e-09, } , 
      {    1.3260105191884740e+01,     2.2901932271262584e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.8342832960530156e+01,     1.7762188989855294e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.9054643190561222e+00,     7.1154474159549436e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0294637003554541e+01,     6.3884117393004781e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.6455826072522939e+01,     8.9828105300987590e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1728230595219108e+01,     1.2596172072791251e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.1942306315944712e+01,    -1.0998759437798617e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4901475286332380e+00,     7.6371054611112277e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4955085825181783e+01,     4.8999070346657327e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.6510130982269025e+01,     8.4948565621206944e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4064188619995676e+01,     1.0831433685923541e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7116640837567893e+01,     1.5332852508287417e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.2917978213042876e+01,     4.1679505539520840e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    5.3962747366590591e+01,     1.5333557140203569e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    1.6135943464231957e+01,     1.6065835026071831e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.9280839482738745e+00,    -3.8062567700446034e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7579666147323643e+01,     2.2110485362092801e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -4.4942107981121637e-02,     8.2473363940020008e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7242215586998558e+01,    -6.9324060763968009e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.9373566765533301e+00,     2.1435767635435193e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.8532108437465444e+00,     1.4632759423710548e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.2403436665826469e+01,     1.5167782263744666e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3118967159243576e+01,     7.0052247450889199e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    8.6187962078111084e+00,     8.5891563387421047e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.7506708850002077e+01,     8.3900818292525437e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.2217817950775682e+01,    -8.2673010199943565e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.5268863017363834e+01,     7.1913152450000798e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4074946410663884e+01,    -1.2101885331428962e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2293724610192076e+01,     1.4094064487244961e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.3991096930980873e+01,     1.3578204766180925e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.2930529891054201e+00,     7.2128994038041128e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    4.2329091026192600e+00,     2.1116504337788916e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1238478470340734e+01,     2.4941192299930750e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.6991342790443167e+02,     1.5208448272301212e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5395146297912774e+01,     1.2434669159834466e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.5776635470493257e+01,     3.3524643796604869e+04,     3.2886489253401133e-04,     7.4146618990729021e-07,    -3.7668083767486503e-09, } , 
      {    3.4852067299258714e+01,     3.3523986531855327e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -2.1581291397501104e+01,     7.5583819692367724e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.2797940987822216e+01,    -4.9292972602081022e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6503032081704042e+01,    -1.1709518918525100e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.5270541960171533e+00,    -1.8556899362082659e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.4436020897487596e+01,     6.1524034033044318e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.7329524814054114e+01,     5.9416999517528084e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.8164571293882292e+00,    -9.6720571193808883e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.5262008399957352e+01,     1.5953824203130644e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7587300003324549e+00,     5.8400537239811201e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.3826676120442006e+01,     4.0581797661063334e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    4.2724146407216253e+01,     3.5408739246586454e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1535257319120021e+01,     5.3298514556885921e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2434185725725172e+01,     8.0957060092123047e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0910941788395300e+01,     3.3577284261441295e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -1.1050063729003611e+01,    -7.2194925703962181e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.6748081411531739e+01,     7.1376216219474818e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.6603013558609836e-01,    -2.4966886604114316e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    2.4230663020088205e+01,     2.3868600800547454e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2137631961665974e+01,    -7.3853025591156756e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.2612155476251026e+01,    -9.5229497876843780e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.4798669755475110e+01,     2.6748138153777003e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.1537641279146754e+01,     2.4889574665726905e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3557749385552285e+01,     6.0543504958287504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1314104482552327e+01,     2.5744301520856370e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.0891116344949232e+01,     3.7635215443872796e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.0105749800791289e+01,     1.8307018989030974e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3977506907297759e+00,    -1.7378990438819019e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -2.2380069524293720e+01,     3.4758385259320335e-01,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.9970080613046779e+00,     1.5422097587133665e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5925548181120174e+01,    -7.2080025825394923e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.9483295946529218e+00,    -2.7642646990491794e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.4548115772719409e+01,     3.2055473220037368e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0106007565375172e+00,     7.0038020541551659e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.8807799724050817e+01,    -5.4264430680696660e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0321046698850974e+01,     1.6257802923954781e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.1404557960753273e+01,     1.1178053989743381e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.0116367805616228e+01,     1.5303864878594322e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8909495636827875e+00,     1.7232009991614377e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4299223795376474e+01,     2.5005261943626381e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    8.1310195426059657e+00,     3.3471346067018909e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    2.3483054504555199e+01,     1.9584297675378955e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6752568274111535e+01,     9.5541549995755595e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.4146657293131755e+00,     2.3874290628663035e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5127195379265423e+01,     9.9314282257540981e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    9.0222082256276472e+00,     1.7913998019337985e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1605632149834156e+01,     8.4710066678242038e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2481451099901905e+01,     1.5342719286841202e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6285523709263881e+01,     6.0464652088820494e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.7531713384201328e+00,     7.5930060827696743e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.2140044367856497e+01,     1.0032796112826691e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9601241953991497e+00,     1.3868996578223056e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2327182406585424e+01,    -8.3235708519863147e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.8983809983116355e+00,     8.4100464552448993e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.1644682691008477e+00,     9.1949154573377327e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8751163349175002e+00,     2.5450055179282550e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.1910125868843711e+01,     1.4911601790878460e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4369270682374733e+00,     3.7636151870488984e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.2181885183379322e+00,     9.4690591190366667e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7612097417691025e+01,     2.3452335279337931e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -3.3921192298426107e+01,     5.3287877942636405e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1114865879800906e+01,     1.5619195533392660e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.7944225840132340e+01,     8.7704142601309959e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7074686914633617e+01,     1.4989697766006964e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4158206437650270e+01,     1.0606635149453068e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7899609555482069e+01,     6.5236221304403552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1283095208446722e+01,     6.1343791951026315e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.3494805417027898e+01,     2.3007851514207145e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.0169937255409796e+00,     7.9640563883407203e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6465255840209210e+00,    -3.5656234476268708e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.0121182067787345e+01,     7.5318327739806191e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.6552043976281894e+00,     7.1327078347368943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6947198129046789e+01,     3.2970431335100220e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.7052468562529285e+01,    -1.3544291474199781e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.2345336153729178e+01,     2.4358119004782879e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1020550725105952e+01,    -8.3338801332418006e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.5277191952458082e+00,     8.2234645756679438e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9838388167717355e+01,     1.5542289828194491e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.5902928317501146e+01,     1.5543218751769105e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.6548456265722699e+01,     2.2251368445524571e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.1847392317776535e+00,     3.5562595246677233e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6233643714100765e+01,    -1.0903556856556233e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8937735747661566e+01,     3.9936569395210194e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.1379669205846339e+00,     6.4437686694868207e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7726328125634190e+01,     2.4910823212799638e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0909396655176327e+01,     9.5798825657996917e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.7653554709731134e+00,     1.6656987015206887e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -3.6568627305740957e+00,    -1.6657778683286910e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    4.3943102154374714e+00,     8.8091123447577829e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4359711571351422e+01,     3.4713367479410867e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.5463872406411738e+00,    -8.3772512946643801e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.5989654250277205e+01,     2.2400214006736169e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.7792750062212733e+01,     6.3857904008267496e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.4720584092088842e-01,     2.4592639539842821e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.9837838610674901e+01,     1.3655530334903755e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.9798435677165735e+01,     6.1103458827987470e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.8249335996545064e+01,     7.7000248100691051e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9759169307440725e+01,     8.6792610987846710e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.8126870159166595e+00,     5.9595705034532739e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4118358240537505e+00,     1.6919891936455482e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.3904397453254349e+00,    -1.8657350853805985e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6090341718578998e+01,     1.8458194387772273e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.3334623884038425e+01,    -2.5478375563397810e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1069677369775933e+01,    -4.9327672549832755e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3479913020356546e+00,     6.3790552083415457e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.5457145338968317e+01,     7.6762053129066426e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0154409428770094e+01,     1.5543073093633298e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6753073082475396e-01,     1.4952184064050463e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6524769985479697e+00,     6.5159254364540429e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.9935765283788680e+01,     1.5542435486330292e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.8351868699516942e+01,     2.2112967542006169e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.0369732014361418e+01,     2.7080047821963002e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.3916391634557108e+00,    -6.3486654064467712e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.7950911317871402e+01,     6.7622878846701187e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4931036019273598e+01,     1.6739866142516621e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.7408230040717569e+00,    -5.6595077801538696e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8678965839175348e+00,     1.3453696514631585e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.6116374984545452e+01,     3.8808993743539395e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4031500234488554e+00,    -8.9739196904765849e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -9.0570219442278930e+00,     5.7870941025088387e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.5510805732069310e+01,    -2.5070572649223459e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1868024182877974e+00,     1.1690096854064044e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3915509067539801e+01,     1.5598015111395487e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0845647472863416e+01,     5.0921519530988132e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6001798157987910e+00,     7.2501712159219687e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.7440993405689731e+01,     1.5542903775962475e+04,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -1.7222348248293930e+01,     1.5542604804001114e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    3.4057453303684673e+00,     2.0256704726356234e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1429269310302921e+01,     5.9650742788935286e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.7420797711333229e+00,     1.8762173108596406e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7116459296469465e+01,     7.2113227164251193e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0515567676148326e+01,     1.9566263991438245e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.3597942354841862e+01,     2.7174115412999527e+00,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.2391209537596829e+01,     9.2802855021195569e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1972522120202129e+01,     5.9747206920329263e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6963961531476802e+01,     1.5035103407784380e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.5788251263403403e+01,     4.6956849489824512e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    2.3169991949293870e+01,     1.5845637129938445e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.7665037041284450e+01,     8.4550416602227833e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.9895504932736428e+00,     8.2937989795670292e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.8104516869265424e+01,     8.9414569813805647e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5413614248979549e+01,     7.2018569406343649e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    5.5942299796120389e+00,     9.9248102950200587e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5340262163626086e+00,    -8.2933543536867255e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.0246252092441441e+00,     1.6156105541889223e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9034711620786915e+00,     8.4952447039142316e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4265272023110484e+01,     1.0983908375631603e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4592295108569424e+01,     8.3244336689657284e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.3900343827244782e+01,     1.6253545168297054e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4942585589449131e+01,     1.5247949097557825e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    2.3311598219682548e+00,     8.4339182735789564e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5448366756499542e+01,     8.4000133624112132e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.7892570829004118e+01,     7.2168030142915695e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3686017589421073e-01,     8.3635838696798710e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9143456018284322e+01,     8.7621343881608595e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.5844739507790166e+01,     9.9349098664746944e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7137740867832054e+00,    -1.7477206216088034e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.9538656278396964e+01,     3.9940335880704515e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.3784391168328312e+01,     1.4293765703516978e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7405117019290078e+01,     8.2637773535071337e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6067866514253168e+00,     7.3549459487901195e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0358924052378022e+01,    -5.5974239785520119e+00,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.7807922430623453e+00,     1.2401216364754078e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0696831422887403e+00,    -4.2561664844928218e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3696842595616566e+01,     1.5487493468568102e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.3537319111351751e+00,     2.9710142920637900e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6399803385884290e+01,     1.3031825149741031e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6437325205331753e+01,     8.6178706941555756e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -8.3683834123305445e+01,     2.3243040077985861e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.7471784736997016e+00,     8.3150764435919100e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1960010604808957e+01,    -7.1207208705272507e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.0498616892181094e+01,     8.9434924983976380e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2322257260127959e+01,     3.4009471791447959e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4510189145819647e+01,     2.0791712306875204e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.0928353863524993e-01,    -3.1572856687421672e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5560533712413100e+00,     4.3776110417862952e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.8051517175572442e+01,     8.3313279704903580e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2187404237115430e+01,     8.3260548787565422e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.4660795194277370e+00,     2.5186987241223818e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    3.0062971916837348e+01,     2.9813145666358054e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.0273446898442813e+01,     4.0475859466640948e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    3.6646729170697952e+01,     1.6346845172823630e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4043750827594339e+00,     5.7199011105634872e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.1797096540451317e+00,     3.2720004291197711e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2254494124048465e+01,     2.4497123467334604e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8779864560634067e+01,     4.4034127599511703e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0357368504638643e+01,     1.6018765982945870e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1856012196767775e+01,     1.5332050838430172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0997255227501526e+01,     2.3559926376265765e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.6335716042662114e+01,     8.4298967311504566e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.1046825143435726e+01,    -8.2274861180964454e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -6.5697412721804493e-01,    -2.5470117676373329e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6195415423425068e+01,     1.5152034520230216e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.7664043718140107e+01,     7.0045201131727699e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1792075382693129e+01,     1.9225058133141127e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -7.3388351142451114e+01,    -1.7625312723742695e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.7130301333674716e+00,     8.3978099230153330e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2789173192981968e+01,     7.6944837854926773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9798989923498240e+01,     7.0045066860713159e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -2.1824281889719686e+01,     2.3225865132794035e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.5866446508135246e-01,     1.3376144253619463e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.1249329657575863e+01,     7.3078111797228303e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.6052400490379881e+01,     7.1588020439446518e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8463181636526667e+01,     2.3934724093071924e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.9713627775030105e+00,     1.4441243641914956e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5110445536342663e+01,     6.7500819599461392e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.0348112007214173e+01,     8.7423759105733316e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8692566528113517e+01,     7.7598871571341697e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3294717655329116e+00,     6.0983307786990335e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0843500253398531e+01,     9.4078953735767063e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    6.0777973847952637e+01,     7.2368104857166090e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4196643028672751e+01,     1.5437527441026288e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6282705650506195e+01,     8.8817479485982803e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7377150075754344e+01,     6.2689670336067420e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5244469412962317e+01,     1.3180055747201804e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7749349514108605e+01,     3.1191446774385971e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -4.7321769787687359e+01,    -9.3136315067178288e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0876704959743151e+01,     8.8053249381474161e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    3.8828563904095837e+01,     2.2694684005467694e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5132995450325428e+01,     7.2152263269125760e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.7365392738579558e+01,     1.0166148728683766e+02,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.8917729273356699e+01,     3.6154074217923699e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.4697848709238599e+01,     1.6146789001417854e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3083548728508084e+01,    -3.9291232829491491e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3448472505023989e+00,     7.7249764791177495e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.6446154700979331e+01,    -5.8849094815024364e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0823660206841922e+01,     8.9389500662131304e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1893478648691808e+01,     3.1574459386499337e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -4.4251193248721485e+01,     5.8849441814501802e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8666404618390558e+01,     3.3926152602731752e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.8906379366538467e+01,     6.9166431001333422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1952218061712443e+02,     8.2871455542483000e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8982128832825289e+01,     8.5013462173232820e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1292965819181632e+01,     1.8100003264406896e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0012026632664517e+00,     8.3258378409347533e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.9009356249284668e-01,    -8.3315450083121486e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.9710112122382046e+00,    -1.5589653628635220e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.9062730980791837e+01,     2.4428407678279724e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.7482645070932733e+01,     7.2135984035710389e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -2.8258294321490123e+01,     7.2145273271456535e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8465937663075046e+01,     2.4018966517148883e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.7674041244408372e+00,     9.3319710359061191e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3004131012017083e+01,     5.4221114823735170e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5142185314066126e+00,    -8.2790280271883130e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    5.2023388947162999e+01,     2.4429112310195869e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.1559946275455161e+01,     7.2693236867720352e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.0291434958693015e+00,     8.3804970602804096e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.6482372316304783e+00,     2.3196737463686557e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.0720871680917343e+01,     1.5616341949703961e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3777307252453907e+01,     1.7342357413207017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1451179968180121e+01,     1.5542358455941783e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.2484279882036084e+01,     1.5543150124021806e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.5085438855303117e+01,     7.2142123513390270e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {   -1.9577902798680920e+01,     7.2139133793776637e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {   -6.9381794825195797e-02,     7.3319555094247626e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4625223183015322e+01,     3.1729563908523585e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.5247999823596130e+00,    -2.5159575323203051e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    2.4237518447528394e+00,     1.4299257031088975e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.2032214954420413e+00,     1.5716255339314497e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.3460931905123687e+01,     9.0528092985369040e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.7965513861761380e+01,     1.9582573268219388e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8268417165419848e+00,    -6.1027390110483316e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.0086762525940003e+00,     7.1953273116896689e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1377069071779381e+01,     5.0544870981556710e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0498540197649969e+01,     2.7888882505770027e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5438448294245100e+01,     3.3367734795704993e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    4.3292582495670153e+00,     7.4556975667905760e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.4493048303997220e+00,     1.7197776448071858e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.3128318136297537e+01,     4.6052924382111036e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.3482745553710906e+01,     6.4749525480157422e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.6814740633624226e+00,     1.4976579854808700e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3655257362458798e+01,     1.6975152757869175e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7787224696489879e+01,     8.3820128377927176e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2026164618202575e+00,     3.3274950796595891e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -4.8462675794702230e+00,    -6.1292970449606059e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8272852481508846e+01,    -8.3459302046354769e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -7.1644484698175930e+00,     1.8369152138903417e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.6822569691555550e+01,    -8.0409088284183781e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6331484916962104e+01,     6.6610063413835142e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4955555255073936e+00,    -1.6906783925663337e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6494157051556037e+01,     1.5659001765659665e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.1341105369175665e+01,     1.1191525036565781e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1742807568767430e-01,     7.1140247250211905e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7798850837479407e+01,     7.2143816690098483e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4069383466077575e+01,    -1.0661771589843593e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8078701307787636e+01,     4.4783351886471100e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4509907151261405e+00,     1.2305656390876289e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4072193462060248e+01,     3.3354580600391710e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    5.9591187338278120e+00,     9.8215777269017963e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.1934145076702123e+01,     3.8235137095508071e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.6689643077216065e+01,     2.3474017503273855e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.9038534562389863e+01,     1.5613741965563302e+04,    -2.5423105344098514e-04,    -1.5692589826120981e-07,     5.6951547933618284e-10, } , 
      {    4.7010902360484927e+01,     2.1619646690942791e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6315058048364378e+01,     3.1884668430547834e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5787359272669605e+01,     7.1986849848088614e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2333291011673365e+01,     4.0489013661954232e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    7.9075401555735390e+00,    -1.1146180370612958e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4983435226457674e+01,     1.7332091100165591e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -6.1472252917076418e+01,     3.4911411969878736e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0943038961064701e+01,     6.9607273065499267e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.6058744980088775e+01,     5.9020907650809158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3782412478432093e+01,     1.5973718334015015e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.2138307085116931e+01,    -9.0978563699337137e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    2.9165212363391486e+01,    -9.0932539700508933e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    7.6672562634729511e+00,     7.1257566972104314e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4569987364402497e+00,    -9.4601807305361235e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.9611083963055425e+00,     9.3631209246142826e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8400573439259070e+01,     2.6851320162209580e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    5.9376432589842651e+01,    -1.4591647589835738e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -8.8015064740657607e+01,     8.3355969545290445e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.0240427276958393e+00,     4.6385406357998363e+01,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -6.2179266057129183e+00,     1.3720188341875320e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3123249181635165e+01,     7.8313498236815240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5423915477423904e-02,     3.1120053468797960e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1073184719374126e+01,    -6.7562854910320766e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.7603254400035848e+01,    -8.1022649648114402e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4543819846125174e+00,     1.3060675544938600e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7047186653438199e+01,     8.8981298583725729e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3641419946014082e+01,     8.2753700114541825e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2291323004818217e+01,     7.2137440617068405e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.5598354394692597e+00,     8.4323026959373692e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.0431762302184342e+01,     1.3571090485917724e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.0361044123514009e+00,    -1.1113619639929524e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.5028419825688495e+00,     1.5769224436525688e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    6.6069963696531175e+00,    -1.1146390814689155e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1413761940506461e+01,     2.3942433390186754e+04,    -1.0193864567865896e-04,     9.3792975893444940e-08,    -6.6646851928446799e-10, } , 
      {    1.4583397423936239e+01,     2.3558606838240565e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5430433215529629e+01,     2.4499747669819306e+04,     8.3445350145584034e-05,     3.1345298878833894e-07,    -1.6984429208942262e-09, } , 
      {   -2.3331317604276830e+01,     5.6811584846769847e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1149119551026665e+01,     1.5716221128321522e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.3033464970749767e+01,     1.6799358200409915e+04,    -7.1510939109650651e-05,     6.3350506747846849e-08,    -5.1689865052536508e-10, } , 
      {    3.0065663462892701e+01,     1.6652315777205211e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.2686879082736173e+01,     8.9761866794250236e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.4253314856185728e+00,    -8.9105983437958457e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.6852636503382186e+01,     8.3754118564269575e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.4878387057361042e+00,     1.4930446213160250e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0210341058993158e+01,    -8.8150123988430114e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.7755561143000726e+01,     7.0618847843244021e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.7893201667369162e+01,     1.3799824859221870e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.2719967548941369e+01,     1.6655433965263561e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.1066727208672319e+01,     4.0371094209511706e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    5.4282706799630176e+01,     1.5996260239147259e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0082067990527013e+00,    -1.7554758477100157e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -6.0781238536249909e+00,     1.1941131212931050e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3405437394353036e+01,    -1.5548183995019670e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.2054053478855366e+01,    -4.8341326512672822e-01,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    3.1925797960293512e+01,     1.3972971611151257e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.5959926161222953e+00,     4.1812287962817290e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.5774369666739460e+01,     5.1072534399447250e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6018688457016834e+00,    -2.6306793805873735e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.8122775410573880e+01,    -8.3303193479649890e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.9356305838075819e+01,     7.3274740966093304e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.2347587235400724e+01,     2.4013432325431226e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.8038075515021383e+01,     1.6849659318217535e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.9346799823166009e+01,     1.6505123118499065e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2892161137471280e+01,     8.5551178844354617e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2687138056710765e+01,    -2.2645941894669331e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1255992872322267e+01,     2.6832141968192460e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.4837952251032300e+00,    -1.0551099821936912e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0376277427933437e+02,     7.6998758809387891e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.9833528253227222e+01,     2.3902098636963943e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {    6.8315353520251549e+01,     1.4293138996734522e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.2037099713925677e+01,     6.6252178662526394e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.7569904355207395e+00,     1.2341806703839161e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8644206414592741e+00,     3.3705942173559110e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1496145970447764e+00,     7.4405330119022392e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    9.3955051822596587e+01,     3.2409679891414271e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -6.3507977097015740e+01,     3.2409693318515725e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -9.5569407939350093e+00,     1.7985103385839620e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.7036220019184576e+01,     7.6277539126258962e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5870841210886258e+01,     2.3183619552488286e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    5.1744711532552810e+00,     2.5122027410253580e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.5154496349046056e+00,     8.3581883756349689e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.8043405341310015e+00,    -8.2991944736119331e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0276736516075573e+01,     4.1917440938667416e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    2.2621625601611800e+01,     3.1379754921537402e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5177195924904368e+01,     8.3685426106363739e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9985691247134625e+01,     2.9448510630075325e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    4.0523238943701738e+00,     1.2633873147790451e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2303189877682886e+01,     4.1641155923175837e+04,     9.1375444657395709e-04,     1.5043132449279283e-06,    -7.4159039917239050e-09, } , 
      {    1.2156917376996041e+01,     1.6278439631835570e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.1011705048274160e+02,     2.5195115780967531e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    1.9889432040833955e+02,    -2.5196131698245768e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    2.9583020575980086e+01,     1.9160125110546989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5510460887671984e+00,    -3.3330537070233086e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0946991653190191e+01,     1.6675416533187610e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.2184033488636086e+00,     5.2647057604078582e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2029712369453115e+01,     6.9573146411738890e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.7227912561679560e+01,    -1.5540293291042133e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0089031507773356e+01,     1.5445851489420304e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.4856678205794900e+00,    -8.2565346576562297e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.4914405754142091e+01,     1.2574369830234191e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.1349268760264962e+01,     1.8914450781275933e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2659975722661003e+00,    -1.8558053347897985e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    5.9373763126946235e+00,     4.1784631345530106e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4916242461317161e+01,    -7.9537193983863253e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.4925748939322006e+00,     1.1330857311826217e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1166061843681160e+01,     1.6204843302482499e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1614195377276896e+01,     2.3982779403072531e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.4465091717232973e+01,     5.3571784597793794e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.7276196892479526e+01,     6.7457908345946416e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.5468621578559780e+00,     1.3329743587746314e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -9.5723942035617782e-01,    -2.5496415999535166e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.2831910888079907e+00,     1.7473909992050892e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.3716767737482613e+01,     7.1427172439872802e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9152555627807718e+01,     8.3957556067534933e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5376360839430909e+01,     4.8202772308699086e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5451175186176997e+01,     3.9889547805833558e-01,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    2.1598017819123637e+01,     5.0747086705458114e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6975686223685948e+01,     2.3819187962860928e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.8652312850491725e+01,     8.3263969382327650e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.3147368171266436e+01,     7.9333692549890511e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.9914486656285526e+01,    -1.1626472072487923e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -9.1780241341501011e+01,     8.9950829430896665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0984473229544962e+00,     5.8334947683819873e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.4058784528802271e+00,    -1.6789464203770933e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0875306400968235e+01,    -8.2396739110086583e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0196529638502279e+01,     8.4008481915906723e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.8573726756020214e+01,     2.9382387337550856e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -1.0751917163499161e+02,     5.2440490764023616e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5457034962677714e+00,     2.4669487863489176e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -2.7290231442166579e+00,     3.8941270695333033e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -7.2009448258355251e+00,     1.5542469061991611e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.7994812022461943e+01,    -1.5543039517971974e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6282369174990691e+01,     6.9913954884237755e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -2.5504511745278950e+00,    -2.9154400977904775e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2755265446023754e+01,     3.3261796601282607e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.4871426719435842e+01,     1.5541581352688652e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4380588079193821e+01,     2.2891803889322306e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.7802815904401417e+00,     2.5613670513449015e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.2027828457320229e+01,     4.0921849533545130e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.0553974662420423e+01,    -9.4346834855057223e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -4.8957388791203122e+00,     7.2129582736290258e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3808530563851633e-01,     1.2163299917085338e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1428082956447104e+01,     1.6621274498683277e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2790957705840654e+01,     7.6534845009244800e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.2993378197356538e+01,     8.3992326139490851e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.2054240519159357e+01,     4.0568643465750050e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    7.9303231718049947e+00,     2.5216953647584502e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    7.4734719132643491e+01,    -1.7982930744969908e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.7091586595319200e+01,     5.3294924261797896e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2043332848961168e+01,    -4.7287443064652234e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7400234364106268e+01,     2.7643921057194690e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.3776802558331553e+01,     1.8606498815271470e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2327008239589663e+00,     2.2141627481492862e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5348916918080697e+01,     1.5543927227274937e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.0382568403611181e+00,     1.6168984332243635e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1499363031611072e+00,     1.4984336482955036e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9975226649023462e+01,     1.2880649750999733e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7222015389831586e+00,     9.1267335735073830e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.3755113467837941e+00,    -9.3062491458050190e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.2599109112245117e+01,    -1.2738881668567547e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2939782618217306e+01,     8.5257893708824431e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -7.1279896605105932e+01,    -1.6657588564472924e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -1.7760557464897222e+01,    -6.2303232243609727e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6239185120738167e+01,     8.3063847255098663e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    8.1824732712917537e+01,     2.2661645323255885e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.8695941410514465e+01,     6.5105042192420879e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0071364965339775e+02,    -5.1280492861351377e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1695892749179698e+01,     3.1491647460854845e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.0679509577561404e+01,     2.0990178169437638e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.7914550266800788e+01,     9.1785460861487663e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1984168850222410e+01,     4.1555224226385249e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9266219957117720e+01,     3.2155255165289098e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -1.8251232357898310e+01,    -1.1971118525348229e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4326603808842950e+00,     2.5174293831628798e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -6.7588208816666926e+00,    -1.1677234361162277e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9545164275979825e+01,     8.3335614375119694e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8391667344248404e+01,     2.2146558513750460e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.3590654679587459e+01,     1.0563217451520237e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8440195809711639e+01,     1.3268413361187095e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1649449942833227e+01,     7.7474528827038685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1755289099241754e+01,     7.9483086448262702e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3309605526567506e+01,    -2.0371824858844643e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1730869353668332e+01,     1.5291719931114787e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -6.9713031629794617e+00,     1.3989304132918396e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6176414586439588e+01,     2.5140512404173642e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.9313117444280788e+01,     2.3349787669448961e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.9114152382756338e+00,     1.6283506703877259e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1009943214577352e+02,     1.4286237171464549e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5945114132053659e+01,    -5.8466126795802547e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6106368942528114e+01,     1.7802664330870706e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -7.2611176260590824e+01,     3.4513262950899607e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3758232154270193e+01,    -8.2385199251933391e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.3361837970174690e+00,     7.0912639242883943e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.6593782499099383e+00,     2.6484548605595683e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8643332222333047e+01,     2.5285795239036765e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4237299015684549e+01,     6.6081897800199713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6178375440344475e+00,     1.1297075884371056e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.0448764357764063e+01,     2.0919714977822662e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    3.3008902770009378e+01,     5.5667279255156809e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.6444185159904009e+01,     3.8865522223337321e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.4115983121640685e+01,     7.2151674570876630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7219382742800420e+00,     7.2294407459078266e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5631453552098975e+00,     1.7505557938505321e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.5801011303816743e+00,     5.5403051535996037e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2311885230935793e+01,     1.6628532454049335e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -9.2002610974975436e+00,    -9.4493534583313049e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    2.5285494260784326e+01,    -9.4372865094458011e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {    1.2043179745010697e+01,     1.7233432682548126e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.3885841706604154e+00,     1.5537324584943924e+04,    -2.2282960538085985e-04,    -5.3675274039416252e-08,     7.0782797441992036e-11, } , 
      {    7.2685122435424985e+00,    -3.7578158634075436e+00,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -5.4497886123156825e+00,     7.7032287536356980e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2173449205011106e+00,     7.5633130014943699e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.5762974810744989e+01,     4.3696313324960083e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2045480951780078e+01,     3.1084797234496713e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5730364427443202e+01,     8.9412512661545425e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2948204779939761e+01,    -8.7363489974482509e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -9.9630288718813205e+00,    -5.4378592258879689e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.6883275413484618e+01,    -1.1161355481952342e+01,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.5046510069042167e+00,    -2.2660168804006853e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3159917627432414e-01,     2.1547101323902483e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.4112856835371135e+01,     1.8169948150947435e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1306126193953808e+01,    -1.7532454255661366e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6553470885936342e+01,     1.3011161633484138e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -7.6783658837580990e+00,     7.7381211986921189e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6347767904528823e+01,     2.1766655455255735e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.3269885982652390e+01,    -8.8266302178658407e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0045361555281094e+01,     8.3133135440739679e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0909229496096637e+00,     8.3440693051729322e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.5911623782311722e+01,     8.3288913981978585e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3764325536675116e+01,     8.3284914510490416e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6991358615609197e+01,     7.6841745042371904e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.4702927917857153e+01,     1.7697650513082757e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5514944402567707e+01,     2.6020788726717183e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4645374754159803e+01,     7.2178293508526585e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5881992881412977e+01,    -4.4054193602093096e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.1205911773474075e+00,     2.2128838202669365e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9259375562755348e+01,    -4.5098154161608600e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.3122786647625926e+00,     8.3847283968983265e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1312117249742908e+00,     1.7221700710358888e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.5516146720148065e+01,    -4.4828179110667952e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.8295422486658694e+01,     2.9831194609813916e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.8428859602157516e+01,     7.9041902274167132e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.7483447403752685e+00,     9.3242828297457254e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.6138091174046689e+01,     7.6740754489513492e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2908179887143163e+00,    -1.5878316222864005e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3887384204323155e+01,     3.1165690673291894e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.2674052932733169e+01,     3.2269966983087452e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -3.8725855467449257e+00,     1.6659331733230236e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.1342056966513638e+00,    -2.3705323838893160e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9308442502406777e+01,    -6.5087016920809574e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.5931656528687249e+01,     2.2734275250207895e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.2967194804278606e+01,     7.8568962342789985e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9060469396053253e+02,     8.3284953214757734e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9845739571489591e+02,     8.3288875277711286e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0317676283142230e+01,     4.7742539113252409e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    4.2415680700538559e+01,    -8.3913232472761774e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -2.0412922347475494e+01,    -9.4429913515137832e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.4813720660610974e+00,     8.3292350159584530e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2192481166834710e+01,    -8.3281478332884453e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.4389333034614875e+00,    -1.2763255779713600e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.0550025088109685e+00,     2.5048207423742795e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0861371555223236e+01,     1.3362449738869109e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.0737599115173424e+00,    -1.5195453981967830e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9141423704017331e+01,     1.3155511052401605e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1654663719413034e+02,     2.5798696990053202e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2239184066484675e+01,     1.6621300221850328e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {   -9.2753255053948152e+01,     8.0725290164703074e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9260092333960849e+01,     1.1073279256875230e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.1449780981717392e+00,     9.4444235112827628e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5932499322856387e+01,    -2.9973622468156464e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.1271538204299400e+01,     7.7802373005314412e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8558489613932987e+00,     7.8488795357065328e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.0156629026402104e+01,     1.2385468621239628e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.9262081138549121e-02,     1.2667654575245611e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7585194646535449e+01,     2.6767707618378598e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -3.0426374059776204e+00,    -5.4565134139756992e+00,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.0164085941601684e+01,     1.0919820600124209e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7600105870277796e+00,     2.5779933223708022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8367063215195270e+02,     1.5528739820071323e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.5345198699455690e+02,     1.5556768759892266e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3746817711552919e+00,     1.6007745356223506e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0995888204697920e+02,     1.4978534480443646e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.3472036832913147e+01,     1.1253881843595937e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3101464006340269e+00,     1.7102218930335755e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.4204262546768906e+01,     2.3937856344884358e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6376317097274260e+01,     6.0968044737770570e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -7.0254500959390313e+01,     1.7403404171508941e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9815698789605669e+00,     1.9863683756663246e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6030011921851548e+00,     8.8613700965020980e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7994622648470298e+01,     1.0600076193853933e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.7842240556760736e+00,     1.7087582327203949e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.4384485305860792e+00,    -2.3472146199672643e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.8100982220491177e+00,     1.4906968538367195e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2181908325106475e+01,     1.5855593166346473e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5954190236237196e+00,    -8.0999637648700282e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    7.8501770637075658e+00,     1.3093513673733178e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6216972126216237e+02,     8.3278311500262571e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5745861216138877e+02,    -8.3295516992206431e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4793976015205377e+01,     8.8238308531835282e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5632082853310360e+01,     1.6415537278295211e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.9306861712187739e+00,     6.1059932616923734e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -9.5564993762218737e+00,     7.2137776373681636e+03,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.9470160259466315e+01,    -2.4414677089310044e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5846605270314306e+01,     3.9491752265599163e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -8.8197754785653444e+01,     1.5478666514474398e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9272915356076975e+01,     3.2864493140677834e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -9.2330804779265510e+00,    -8.4985466152531671e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9487685612638113e+01,     6.4406401643903564e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.4834295913381862e+01,     1.5906216388971381e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5639257472078825e+01,    -7.2143480933485253e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.0752642628980084e+01,     4.7743243745168569e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    2.1135190844340219e+01,     1.1361525493700068e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.0503205613408973e+01,     6.1020641384094251e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.0623550820100691e+00,    -1.4326326498416890e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.0630790784699180e+01,     5.8895325243988618e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.9299920113583664e+01,     1.7523237018996217e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2792346919967301e+01,     1.7286019066667213e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.8088697439637670e+01,     7.8639425534404984e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0420694388403135e+01,     5.8902371563150118e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.1098383608228318e+01,     1.1659449978806952e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0377799665915983e+01,     1.5052024088591974e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.1213214094023542e+01,     1.6266233707111593e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9753251725364283e+01,     2.3250616296128759e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.8118203707200372e+01,    -1.1151808487975891e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.3870481314815208e+00,    -1.3614981031540555e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.8771992151424630e+00,     2.5031330429667087e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -4.7358728121032406e+01,     8.3282686135573931e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -5.2069837221806019e+01,    -8.3291142356895070e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.5379511558727650e+01,     2.9531473644647504e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.3993220361820207e+00,     2.4714369837428778e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3606686359616100e+01,    -1.1140762697326222e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.9411211029567852e+01,     8.3313145433889076e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.5275323968024864e+01,     8.3260683058579925e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0774609978090352e+02,     5.9575457468410978e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9129840163635244e+01,     1.4787838742453750e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.2885022205278691e+00,     2.9775732476629237e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.8622181179034420e+00,    -1.8767602813634287e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.6784679694396548e+01,     3.9048410982488989e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    4.5822658717995793e+01,     2.4739970947260936e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4219567064473656e+01,     7.6316575231133547e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5274923089924798e+01,     1.4898447196712254e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2817013120819709e+00,     8.2949043673368069e+03,     1.8054310358398448e-04,     2.8773934380691514e-07,    -1.4650051334399845e-09, } , 
      {    4.5705922113083659e+00,    -8.3624784819100933e+03,    -1.2404171194066790e-04,    -2.1369840450239435e-07,     1.0069628638013171e-09, } , 
      {    5.2202000186063948e+00,    -1.0785202087014814e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.9227305997279039e+01,     1.6574899555977179e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.9900718317783717e+01,     2.7949176214078761e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.9000343526754961e+01,     4.1645758323058661e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.3681366955362197e+01,     7.1791704203019235e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1278787185345593e+02,     8.3309859110141351e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3482257423104000e+01,     5.7676117876810451e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1912528333906671e+01,     8.3961153036952946e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.5085312147340339e+01,     1.5369287451642063e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.7158914380468646e+00,     1.9009222633606269e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0408066950334296e+01,     6.7123120372461590e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7123114383793009e+01,     5.3145463525225850e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    3.3893502621010851e+00,    -1.6582392684731349e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.1812478687926760e+00,     5.4470327542881332e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.1627877609793785e+01,     6.3765581305345740e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.8432200122077528e+01,     9.7637044653118610e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1669825815205478e+01,     2.3923703466349561e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3074747241717890e+01,     8.2834372543585705e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3418721255640047e+01,     8.3462635167378940e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8722936188608337e+01,     2.5597108862150224e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8812239273472979e+01,     2.2127798224827722e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4552682738909462e+01,     3.0630702044250749e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    3.6794897611098074e+00,     6.8122254268566967e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -5.4560927509065351e+01,     7.2117634004143210e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.8351439250005832e-01,     1.4710311684722674e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.5010031789790659e+01,     7.6761109659684244e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0215399023442220e+01,    -9.4436486162633300e+03,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    5.6914421131503012e+01,     8.8220122756868259e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4035267430312478e+01,     1.5176965267883746e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.6361793423363359e+01,    -1.3347975403466990e+00,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3784663620876820e+00,     1.6655960158313148e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3737526353382169e+01,     7.4470482807694898e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1187283534018535e+01,     1.6658805540180652e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.8704330999966118e+01,     8.3502669273546326e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.3415440100739723e+01,    -8.3071159218922694e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -5.4580705446689237e+01,     2.5407912371782834e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0245455418531755e+00,     6.6277037508016283e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4120930509517709e+01,     1.5906764135628141e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9390318075378161e+00,     9.4923352225627350e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.1322291520766985e+00,     2.1915313082769480e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3446947769541342e+01,     1.5505016930867583e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3637428556899760e+01,     2.5758998302058058e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.5624569753766082e+01,    -2.2119841217645426e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8329478717757641e+00,     8.6933264609061789e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5994069298348322e+01,     3.2489316408760824e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.2864956003371999e+00,     1.6316176990949782e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.0806901325730367e+00,     8.2829123864248177e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5430680858894821e+01,     8.9297690476059124e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3331903988957823e+01,     1.4534063925139071e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.2296373889731044e+01,     1.6968137621510206e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1325943573389758e+02,     1.1128975907878666e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.4651883444054249e+01,     1.9248512024747659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6218857733313241e+00,     1.3880017204945420e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.1238438380553273e+00,     1.6683691239873457e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5648356169298667e+01,     2.9798744476043330e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.2044770841887140e+02,     1.8085451864935749e+00,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.0188790848697977e+02,     7.7644716150853019e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6182352843073645e+01,    -4.2187136489055410e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.3143249134304277e+01,     3.0023849117909678e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.9420859492004773e+01,     1.5536142890429690e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8822528344242766e+01,     6.3434105948829506e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0970415224469721e+02,     7.7708691863517815e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8988244068561463e+02,     7.7718851036300130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.7971084571454242e+01,     2.3869714746128004e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1704880339992140e+01,     2.3878923881174302e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.2208841388853571e+01,     1.5540454825037774e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.6144301239069776e+01,     1.5545053754925815e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5140743008045277e+01,     2.1993522836312433e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7049250293608782e+01,     7.4155680532557408e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.7971357221283352e+01,     1.6631074458620340e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.8023384946889507e+01,     1.0545944454754568e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0496135998996017e+01,     8.3390094631000266e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -4.0342516760433512e+01,     5.7121968638196267e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.4755380843411572e+01,    -2.2979975713456952e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8963590672070559e+01,     1.6192391783005189e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8131507760156543e+02,     7.2000483954478732e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.5580754154494326e+02,     7.2280773352688175e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.4510448701766308e+00,     1.0240836651616106e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4900618571994677e+01,     3.3517756208630970e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    5.2702832970030613e+01,     1.4547570436410426e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3191700152948387e+01,     1.0234277696016974e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2598475693866050e+01,     1.0373978366416695e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4582234247724479e+01,     1.6661196141060034e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6670194924840231e+01,     8.9937587715394675e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2645828887919599e+00,     1.6653569557433766e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.5563884115089998e+01,     1.4602927366596768e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3740436625735462e+01,    -1.0069326368992690e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.5310412584882249e+01,     7.8177778108640441e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.7112483676419519e+00,     2.5845522779699359e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.8057111669566108e+00,     6.5703361691731234e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.6371613297060883e+01,     1.3188783557391271e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4928871292328527e+01,     1.6538345695104075e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.4336501111186646e+01,     2.5596310060553205e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8159278763998309e+01,     7.9482896933484499e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -6.7694260530398935e+00,    -4.7500330637710085e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.2543778171798927e+01,    -1.8047208458473676e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6316668972474858e+01,    -2.3715960453142670e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2002268367910606e+01,     3.2306075333651079e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.1837187826935075e+01,    -8.5468773887425868e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    5.0330661885390349e+00,     7.2086331603204726e+03,    -3.7512201314318606e-04,    -3.0439414819407097e-07,     1.3067667960626429e-09, } , 
      {   -1.9948442910645905e+01,     8.3931275094346947e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6190336394998965e+00,    -1.1249378405205935e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.3992110486923082e+01,     8.3931240394399192e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3441622974408327e+00,     8.3448408258041418e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0054092469377062e+01,     4.6848330169521741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1103064107335605e+00,     4.7950139695005390e+04,    -1.0808627523830830e-04,     3.6426388663702380e-07,    -2.1108787675509198e-09, } , 
      {    5.4413826916917088e+01,     4.7954742094888192e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    2.9334843559221138e+01,     3.6130346836158569e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.2619688983364682e+01,     8.3362435909997475e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8087670642400706e+00,     1.4197535116301515e+02,    -3.7609575463430303e-04,    -4.3808724156146259e-07,     1.9550693467159364e-09, } , 
      {   -8.5350419736723666e+01,     7.8422723988300477e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3792793659116240e+00,     1.7127397229101858e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2722030639897437e+01,     1.2065670284812892e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1317684238414309e+01,     9.5234868177017124e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4311974436048297e+01,     2.4849003435786813e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    5.9037154262789144e+01,     6.6259224981687885e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8986331032572494e+01,     2.9917910923487292e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.8854848968464594e+01,     3.1086219925430465e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -7.2495397558513730e+00,     2.3862585559064701e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.5896108706841190e+01,     2.3880305870145792e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.4753994357590994e+00,     7.5606831691781854e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.1204441575298510e+01,     7.2373693365667641e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8340245393411060e+01,     3.3543502833986131e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.3258368327872120e+01,     2.3863967548036188e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9264064119969486e+01,     8.3538363536957768e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.5018198978434132e+00,    -2.5183004501188669e+04,     3.7549077619774624e-05,    -1.6722028652178533e-07,     9.4567756637226078e-10, } , 
      {    6.6178221489394620e+00,     1.5558502033496246e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.9156366284308561e+01,     1.7991896854656574e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9523671823024479e+01,     4.8713561583400970e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.3736909279116105e+00,    -7.0099431046599166e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.4242329122459346e+01,     7.5272303740977968e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6006181106165332e+01,     3.6257543898406297e-01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.8266169635291511e+00,     1.2090723362975685e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.0118008291872385e+01,     1.7380862266376695e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.9124671363488659e+00,    -1.1043192780096178e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.9107236990701804e+00,     3.2998179288112631e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.3256777873744422e+01,     2.5678816208729753e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0792025960189548e+02,     6.3034310206261887e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4299531416754178e+01,     8.5744863659165185e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.4726569301507254e+01,     1.5808667029850710e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6575440843290991e+01,     3.3332789906695594e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    1.7909133787175865e+01,     7.2876591947114175e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.1748172562881422e+01,    -8.3905112016362727e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    5.2101564517480066e+00,    -8.2588615807646893e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.2928187024496752e+01,     8.7282019400166337e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.2006212651878656e+01,     9.8394433124845273e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2342864987753675e+01,     4.9515382516684058e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0812436028500642e+01,     2.3638460299194099e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.7465924010980789e-01,    -3.8838090310567277e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.2398105697487608e+01,     1.5265780945173648e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.2434462537307923e+01,     1.7301232462580876e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5285101732579562e+01,     1.4547162884859519e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    1.5724751293578622e+01,     8.9510989555837641e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4341053801293683e+01,     2.5302862520576411e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.0503204935987966e+00,     1.0904054914678069e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9627788930597472e+01,     3.0986181785093315e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -8.3431628905756625e+00,     9.7249920055991079e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8409958689760309e+01,     9.9030765033368116e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0407717810314004e+01,     1.6884956523184999e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.0309015152631106e+01,     1.9467013015592597e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7790444039517443e+01,     1.4852313555063805e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.5751078356963319e+01,     2.1426422181991586e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0607428013343597e+01,     1.2692344085090735e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.5428819759014926e+01,     7.2529113648551938e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.8577901277516986e+00,     1.7589356525346461e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9493800216178748e+01,     3.1357213016405163e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.1980964105900611e+01,     7.1407683600039454e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.6315031338453352e+01,     3.0282399359037969e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1344335034227562e+01,     6.4990142313853603e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.2948045406074796e+01,     7.9497482072121202e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1977559405679552e+01,     3.3542339372431896e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    4.3786445083560022e+00,     5.0103948264530868e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3639047614688678e+01,     5.3445592213613927e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3373231280033114e+01,     3.6036963409828593e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0204729150018281e+01,     1.1231339938463692e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0261023998618617e+00,     1.5140916851480146e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.6062786247327249e+01,     2.1834263228720782e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7619006746859924e+01,     2.4287876910806022e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7062560477066260e+00,    -4.4598059116526838e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.8635923135504211e+01,     3.1567869959618874e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.9861226457300303e+00,    -1.6692327738256292e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    6.7831163086241286e+00,    -1.6652973622076646e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -4.4144680414025217e-01,     2.5226802143121728e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -4.1657855125469645e+00,     2.1849723526778153e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3782035096395582e+00,     1.2274633605552624e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.6308334775863088e+01,     1.5738203838235688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4629150909212072e+01,     1.0976908498787789e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6294151135441169e+01,     1.9144377367032539e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.5109083729714293e+01,     1.4288216662674407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5680456328159941e+01,    -1.1891912910192405e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0679839547915318e+01,     1.6875099358016869e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.1215784521309651e+01,     2.9828541093914428e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.0976214561974142e+00,     1.5117539874129852e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4303872779853043e+01,     8.2904408358380060e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.2323623192966188e+01,     1.1293696745493849e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9130933492994927e+01,     3.1714168480967204e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.4226780140631327e+01,     1.4285810487515937e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.5683974256408231e+00,     1.7919513829848279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7259814353620673e+00,    -6.0779446375702157e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7175898966119203e+01,     3.3663364616981889e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.6564843273799719e+00,     7.2102963798640321e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.8367345169465985e+01,     1.3192840468603454e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.2462032315590861e+00,     7.7521894752348662e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5430499103443809e+01,     3.1388358050226467e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -6.3470929542134922e+00,    -4.1992777397375912e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.7329927535132455e+01,     1.4130705965491685e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4058119748080591e+01,     1.5178119253699064e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3571950606969830e+01,     7.2954178959797946e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7339342839584679e+00,    -1.0070916223467220e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.9157458991262679e+00,     7.0901004627341626e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9044007042302475e+01,     3.1802708927897307e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    6.7151355258922990e+01,     1.5639657090543287e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.7997718732905682e+01,     3.0457194281367319e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3919709077626630e+01,     7.4247568411710581e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1643467502214907e+01,     1.6711455473852297e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.7493650736478774e+01,     2.4980174219785120e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4003200623195340e+01,     1.0095721535693661e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3580422410865793e+01,     1.5508261517059758e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    7.7198240942559977e+00,    -6.9481233276470930e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5916455406982454e+01,     5.5486789820598215e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.1964797036431824e+01,     1.3815565889185596e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4356691738516965e+00,     8.8194216260132707e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6170762521794970e+01,     1.7031258994616539e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0908340348134633e+01,     1.0937836342133231e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -4.3470181925184527e+01,     1.6928322719612297e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.5089158053984821e+00,     7.3537635357580666e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.6063610274279611e+01,     3.1338844138772001e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1360879750085044e+01,     1.4914436182732443e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.5829161142923397e+01,     9.6233905281352108e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5777576876803238e+01,     3.1781731335877528e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    7.0743612216169875e+00,     7.6481260878335916e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1592472211061585e+02,     1.5544023879339898e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8570607695322005e+02,     1.5541484700623694e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5865043974112838e+01,     2.5125733961587866e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.9811730879014675e+01,     6.0682559025781611e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    7.3784747888863734e+01,     7.2163623303023678e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7851561119212057e+01,    -6.2991522263446541e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.7666717986539339e+01,    -8.3294127803940182e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.6392771837703926e+01,     4.0216008638965293e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    2.3478290050161437e+01,     1.6897585237341897e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    6.1976505148612615e+00,     2.3623539860834702e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    4.6702462892579987e+01,     1.5616048795336195e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.1522582039608618e+01,     1.8700039958723963e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.2306310401332162e+01,     1.8779464047293761e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.8513827601912396e+01,     8.6610216996179388e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.2982608425012671e+01,    -3.1851921518329800e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5150059545749421e+00,     3.0377571191122221e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.1286642785217808e+01,    -2.1446024264758307e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.5282796246850658e+00,    -3.0288283995664960e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.5485843767788850e+01,     8.8869639166441630e+00,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2762857754459798e+01,     9.5257880176431227e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.1065504945761582e+00,    -1.5119080397930109e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.9036699632607185e+01,     7.6868773283065530e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6160372415713013e+01,    -7.7092439951184515e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0005748740964961e+01,     1.5186151141377823e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2584327120244154e+01,     9.5639553214439366e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.1606429324360326e+00,     1.4007162958869856e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.7330648630523457e+00,     1.0241202187353454e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2721536748180291e+01,    -2.1272872459270293e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9925049101013666e+01,     3.1155398357982376e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -5.3999395402780053e+00,     3.2414753677006687e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.9889908502279550e+00,     6.1369735494118067e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1384803426477919e+01,    -5.8308801955809549e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5074052274515992e+01,     2.6152019271751044e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2019685818915905e+01,     6.6600542855018439e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0818928911983406e+01,     1.1152414766584017e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2697432724741224e+00,     1.7381250408170232e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9212150263165441e+01,     1.0417749199973659e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.6640369344672798e+00,    -1.8205159322178588e+00,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    9.3778754252514975e+00,    -1.1779069377317849e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.5164717800484411e+00,    -3.5219207050248956e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5544719655261350e+00,     1.0953481246907410e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2337277135590998e+02,     2.6152747945513852e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1876217997821882e+01,     2.1500913330715932e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.4568641044367112e+01,     2.4058453275209300e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.5510748423826954e+01,     2.7473068791655991e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.2199810422958429e+01,     2.5365977332365070e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3315682829790596e+00,     4.2988325549098590e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3008024972422856e+01,     7.2493999362950681e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.6818835489916808e+01,     1.3870419269156806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1977272969411477e+00,    -1.3220357867795863e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0215440002328060e+01,     6.5688945336652350e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6170803644106435e+01,     1.3799106800204270e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.5947086891290585e+00,     5.3053643365682740e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.9823087846370139e+01,     2.7244341131902569e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.9537388326038602e+01,     3.1614489370138857e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9586923584078836e+01,     1.6878929925726672e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0271347085619341e+01,     2.3274316533074780e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.6507561709063467e+01,     1.5762308421335050e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.3646436035407625e+01,     1.4359433625245110e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.4714931202960280e+01,     3.9902822178748014e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.9924811585618720e+01,     1.6179698373410172e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.8028353791988057e+01,     1.6628517194534179e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0155167704474820e+01,     4.0526198731535958e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -2.5738749108458890e+01,     1.5489432876812527e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.2160032977128971e+01,     1.6634076378038480e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.1435679675537083e+00,     1.1661349297606748e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9872777991757204e+01,     2.2539579483443449e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1340423778227127e+00,     8.8774385440152000e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.7718657436851784e+00,    -8.5731857793691452e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.6848342991670947e+01,     8.2892288386836135e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1165952763935746e+01,     1.7006617725867771e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    3.1681337855391732e+01,     3.9085613384707911e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.6639342116330162e-01,    -2.8444467923774873e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3463559782077731e+01,     5.9349057901350143e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4888251974270238e+01,     6.3975357225543512e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -7.5008273425999619e+00,     1.1548110243238063e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4196626051003108e+01,     4.0531458396168324e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    8.7059487539853055e+00,     6.0596194041953286e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.0534717530573803e+02,    -1.7428380927368094e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.6388983429119165e+01,     1.2734158660896355e-01,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    3.3517164805347505e+01,     2.3731760303720672e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.1320910193956024e+01,     1.0880005698226296e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0104342207087633e+01,    -1.6266538659802609e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.4825224560574277e+00,     1.0511033944985082e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9921042220085383e+01,     3.1067484371761791e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    7.3702907436976943e+00,     9.0617737425371433e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8744247780690202e+01,     2.7344200092028274e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.9658636048558293e+01,    -6.6285826471175824e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.0301776589906979e+01,     1.0229156208048509e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0681811558326518e+02,     8.9564910510298778e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5151506556438418e+01,     3.2119955045900388e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.7744380162119517e+00,     6.3040679990870995e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1980240543819062e+02,     2.2756309196701022e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {   -1.7978418749212159e+02,     2.2757325113979259e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.8093501622727050e+01,     3.3120119268016242e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.2960189663603856e+00,     4.8716847907148704e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8990652600654379e+00,     7.6827518133034373e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1165273688207985e+02,     1.5543763285817306e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4637096932879704e+01,     1.8247104577919548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4768139441627000e+01,     7.0540273110765011e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7263341429087145e+01,     1.5592417687416933e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -6.9132685581274842e+00,     1.0118012975292095e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1278325348030513e+00,     9.2674920114473880e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8885735027888675e+00,     8.5169109823818999e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.4417471328076417e-01,     1.9470662786970257e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.6574308436125690e+01,     9.3657524633612811e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.6149023366995991e+01,     6.6014877643120981e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.4074383497885070e+00,     3.4006134822070646e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.1342793296880224e+01,     8.4383275007492157e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2199192571062582e+01,     6.5983745187307895e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    8.7845255375188955e+00,     1.6919901412194395e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.3210476983740271e+01,    -1.4968328177012354e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2773960207296771e+00,     1.4285427774423832e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3413161578668976e+01,     2.3453039911254076e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.1334205726103878e+01,     2.3844215752542164e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2009591755381058e+01,    -7.1561874711436221e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.2445673361561109e+01,     2.6758563498918232e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4670899323475062e+01,     8.6813720112621450e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.0251665949997374e+01,     2.3087704975160683e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4735815913626531e+01,     1.5842475255148209e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4701967336581189e+01,     6.7097176829369837e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.4768077250146433e+01,     1.4759013550537229e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3077509383365289e+01,     2.9850223317850356e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.7181774561976425e+01,     3.1885603849604891e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8604076611409909e+01,    -4.4650607232165021e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5427125680551503e+00,     7.2489553104147653e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5015776995563481e+01,    -8.5342389928792254e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7545048149710348e+02,     2.4325279701531599e-01,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.2705917914401859e+01,     2.1249167626382048e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -2.1045883254068578e+01,    -8.0247127037993141e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.4932179505514007e+01,     4.0149885346440817e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -1.0302655151365819e+01,     8.1231438563676738e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2622643242549842e+01,     1.7109862461851277e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0207289281567050e+01,     4.7964337296441554e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3678900437236628e+01,     1.1947953930782624e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.2069385663319157e+01,     2.6084733468848214e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8728446869367090e+01,     3.1713816165009132e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.1147387116697161e+00,     3.7544263991335811e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5980578247830401e+01,     1.5596075703151062e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4787970829130899e+01,     2.3858994195127936e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    9.6535622673686596e+00,    -1.3155257884387620e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.0968470545356833e+01,     1.2936722618053327e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.7373178518619369e+01,    -5.4535917098872678e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.1471084230954423e+02,    -1.3662153952399080e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4990554262711516e+01,     5.7375801910543805e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4146039955610306e+01,     3.5801760216279443e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.9194268948063126e+01,     1.5892357561741952e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8404563112556289e+00,     6.6593429400349678e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.2248778035577885e+00,     5.2620759280916736e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5821267061698316e+00,    -9.6951344334266082e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.7408956837826230e+00,     7.0901099384730733e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.2186313776824491e+00,     8.3271908428685256e+03,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {   -1.2929740478456047e+01,    -8.3301920063783764e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    3.7001782694980967e+01,     7.5111692577237245e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.2721282200523632e+01,     6.2759669104505556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8762539503401250e+00,     2.3237104655116953e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.0509284616100887e+01,     1.3422544919492608e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8371310249898727e+00,     2.4042297980143943e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8864468474459454e+01,     1.6612500875307298e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.4549784924463914e+01,    -1.6884205311748872e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    3.1016561110184536e+01,     1.9553146080239974e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.2618110209318054e+00,     4.8855218788534185e+04,     6.9527886268779720e-04,     1.3157120932927950e-06,    -6.5879391871250393e-09, } , 
      {    4.1461987204897444e+00,     7.8347438667776169e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1460745805434605e+01,     1.4423194698459094e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.5804840413745112e+01,     1.7109157829935124e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9216826835292093e+00,    -9.6720476436419776e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.4695422772543935e+01,    -1.3410987058090004e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    1.7537716951859981e+01,     6.4122542309800983e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -3.7930196122429834e+01,     1.1995272714275061e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7677649474531446e+01,    -1.6727246904098633e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.6617049981552753e+01,     7.7160202148045555e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5873912340459825e+01,    -6.4537486658518992e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2391599649612667e+01,     1.3091328467297295e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.1754196315807715e+00,     7.4831309081660620e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.2988404992082422e+01,     7.7153042779373045e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.6581775886533903e+00,     5.7391579690059916e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6406661008596735e+01,     3.3280984271038644e+04,     6.4008420836387148e-04,     1.0392795741567165e-06,    -5.1185174010501474e-09, } , 
      {   -2.4377753944976828e+00,    -3.3348547125948964e+04,    -5.7825505373473797e-04,    -9.6647141908052124e-07,     4.7693545879150593e-09, } , 
      {    1.7467623068686823e+01,     1.4252368952098514e+04,    -2.9940835823449814e-05,     9.7289015829293900e-08,    -4.7372114408574976e-10, } , 
      {    3.0133855129284832e+01,     3.9380418577131881e+04,     5.0840632829225702e-05,     4.1135839673179549e-07,    -2.2266037932317656e-09, } , 
      {    2.2835385529645151e+01,     2.4465966242364146e+04,     1.1435992746015083e-04,     3.4985706632643663e-07,    -1.8730243274617701e-09, } , 
      {    4.9414589376385898e+00,    -4.5254517659588396e+02,    -4.0434645045596130e-04,    -4.7510771121372296e-07,     2.1840904815352705e-09, } , 
      {    3.1840943947229778e+00,    -1.8157328912529447e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    2.5348256199443636e+01,     4.9033288026756731e+04,     1.4130388596241830e-04,     5.8926911581025478e-07,    -3.1134249787173287e-09, } , 
      {    6.3151213843022820e+00,    -8.1529233863423060e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.9616383535603575e+01,     1.5613747595394785e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    7.4606689682024268e+00,     1.5471760984568806e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8361360583411238e+02,     8.3544918566108918e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.7890249673333878e+02,    -8.3028909926360084e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1331692368028680e+01,     3.1879385911008635e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8841101489621749e+01,     2.8036942026887671e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.4968450725082837e+02,    -2.0832844353129758e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.7417921914156480e+00,     1.6075460670559354e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.0132429705393164e+00,     3.7882078265887430e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.7954862212186455e+00,     1.1881612635859636e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4391683933690590e+01,     8.3284763428818424e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.5389890899795917e+00,     8.3289065063650560e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3456777778382936e+01,    -6.7102068799783538e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4114935100647481e+01,     8.3289882587378779e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.2622677408211871e+00,     8.3283945905090222e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4334494378868246e+00,    -1.6638204655229780e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -1.9694653049116191e+00,     9.5703502615764228e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4194492347863898e+01,    -4.4881973939601721e+01,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    8.8179632760109961e+00,     8.3047671623364222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.2811063181044577e+00,     8.3846470580711466e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0490770135526777e+01,    -8.0442778389053728e+00,     4.6351172359218639e-04,     5.4853225840408113e-07,    -2.5876930229221479e-09, } , 
      {    2.2863799107087644e+01,     9.6251483049885501e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.9703085896344721e+00,     6.1479941761341715e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3551031349824392e+00,     8.3286914246234501e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1637181655044952e+01,    -8.2610038189865099e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.8212631451874035e+00,     7.0021864765135788e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3609984289980638e+01,     5.3362343473633430e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0581700509531444e+00,     1.3604862437633974e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5942529353551338e+01,     7.4247663169099669e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3376630123737954e+00,     1.6970221725611580e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8341254298018644e+01,     8.2081594725019386e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.5477000060935246e+01,     1.2527203467356063e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7845552977954277e+01,    -6.3323861730313729e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5448583851675448e+01,     7.0497695554187794e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    7.1336816260187561e+00,     3.9334570200791211e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -3.8271953852220930e-02,     6.6518785728372713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2023691399890328e+01,    -7.1032278496713416e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3523477986285418e+01,     1.5489794668509514e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.7861641474011847e+01,    -5.6880426748928039e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8918088966827959e+01,     1.0370610387378310e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.1981354610391520e+01,    -9.7262668892397232e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.1495871585677921e+00,     3.5011060998403163e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0502695796732951e+01,     5.7319624961253101e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3673604471322555e+01,     2.3871435192401437e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4175611273028453e+01,     2.0455463204002732e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4972885423095930e+01,     2.3871456236809052e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0295193555380115e+02,    -9.4432219323147146e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5914518946271956e+01,     9.2741060600265573e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2620761470010290e+01,     1.3976353785160045e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.3149584173565245e+01,    -9.4434180354623932e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.2764512683206597e+01,     8.2718092411858906e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1299440847383966e+01,     9.8850275598809476e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9569207128709650e+01,     9.0764437153627296e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1542062528124999e+01,     1.5892004426117819e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -4.9936326091958865e+00,    -3.7260746767162955e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1778867144143250e+01,     6.4809805474446503e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.3112715957093371e+01,     1.9407771632339311e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5574146342280677e+01,     1.0999656119146053e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5464967095350982e+01,     1.6160041248304973e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3542159196330445e+01,     1.4985433296798517e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.1406824626121107e+01,     8.3291360505037519e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6957155253475023e+00,    -8.3282467987431464e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -6.3920874431614322e+01,     1.4985446723899968e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.7949112421200876e+01,     8.3167464507183249e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3035688110819258e+01,     8.4557575970900343e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.7612207323307146e+01,     1.4933634374692283e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -7.7200315486945037e+00,     4.8690549583986867e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2008453800114864e+00,     2.9826592209931092e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.6381771506668990e+01,     6.1306011607473683e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5082797041152403e+01,     2.6154320471692456e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3965559458734504e+01,     3.0521635344731902e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -1.2927213613571613e+00,     8.3237157177209792e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3760487517298985e+01,     8.9614082368392519e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1473353963893803e+01,     4.5842220930559408e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    4.2351568546785918e+00,     5.5513257803219685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4966124374323460e+01,     2.1623060860479029e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.5123481623462993e+00,     2.6293331618064160e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    9.1455292014235159e+00,     8.3336671315259191e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1356917304845061e+02,     7.2153324547164475e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.8806163699182841e+02,     7.2127932760002413e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1458934607656062e+01,     6.4282266411983910e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4229240541555317e+01,    -6.6169221349669315e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2607952234648114e+00,     5.3990298882404995e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1271537874692145e+01,     5.9831848448835126e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -8.4617718215441059e+01,     1.1214066941698024e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.8601942496565449e+01,     1.1079080207715857e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7728762459957679e+01,     8.5840892667004155e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1149926952969883e+01,     2.5160678850597258e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -4.5915051078653676e+01,     3.2367097573119700e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6552704635725153e+00,    -1.5393158473110532e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1859930883988902e+01,     3.0910706944083511e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.3792220506502346e+00,     8.3113774975534161e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1556750853343276e+00,     1.6499089644056316e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.4957274197017725e+00,     8.3275184873303078e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.5458306552641119e+00,     2.3940137820076823e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.0843075770527844e+02,     2.4985566315231230e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    1.9429743040295762e+02,    -2.4986582232509467e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    4.4266083213170987e+01,     7.7710181154820957e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6158995399528058e+00,     1.6737577117587405e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8345258328882558e+01,     7.8358114948047551e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.7153385150833046e+01,     7.7774106194336491e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.6500637042638651e+01,    -5.8286487769884843e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3075870919481778e+01,     2.3059666625603015e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    7.6408008218153611e+00,     8.3298643619165923e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3384312139518233e+01,    -1.6844390409850959e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.8446138805751108e+01,     2.3387754572701870e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.2512210676532050e+01,    -8.3532993236784423e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {   -2.2739889031102930e+01,     8.3681540105632866e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0465748523932970e+00,     1.6818194554389927e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.5580201639468207e+01,     7.2181816668107340e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9773921549714373e+01,     2.1692398686785164e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.0729480053207169e+01,     7.7717361744996988e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.1391977291982833e+00,     1.7898578908198309e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.2199117701090813e+01,     1.6657128584862669e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5865639088285917e+01,     5.7501994294723645e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.5505901149425512e+01,    -8.4706932949645834e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.2200325072740602e+01,    -7.1304503215273680e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    9.6824390854923251e+01,     5.9817205688767408e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8032125711533432e+01,     1.4269218439083115e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.9635258043427271e+01,     1.6657637113631132e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8571872191467889e+00,     7.7106931208333945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.1914938534862486e+01,     1.6029068550650631e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3546228078670257e+00,     5.9110470293941116e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.2807121233956543e+01,    -6.2239011652739428e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.9322457073050074e+00,     1.5665613165211769e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9818693374966678e+01,    -8.3329491802811717e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.0955351089979565e+01,     5.2643771280330857e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0643345281575094e+00,     2.3642718054851819e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -3.1034912012936822e-01,     1.0173799277862166e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9736149404384044e+01,     4.0569112121835467e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0955707657278693e+00,    -1.1222304544602801e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.0558726985097280e+01,    -8.1294874366674649e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6784189291037173e+00,     4.3051817231670011e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.3075857515685119e+00,     1.1636437979392849e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.7509312381822331e-02,    -1.9716732798029814e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.2130938032217520e+00,     1.3827048107734227e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5025709119628390e+00,     9.8853561922557219e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5213788193711640e+01,     4.2060254854648440e+04,     4.8560154018717928e-05,     4.8005688319596154e-07,    -2.5896807590146970e-09, } , 
      {    3.6089731895331923e+01,     4.2064857254531264e+04,    -2.5602466150593443e-04,    -2.1380865113347869e-08,    -1.1771276177339540e-10, } , 
      {    4.7838950218288545e+00,    -1.2934842093017298e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8337174077584262e+00,     7.1294545789138137e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9171403966375511e+00,     8.9253598204356531e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6054338217055932e+01,     2.2107179662316747e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0476903799479167e+01,     2.9828893409872504e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.2344979640211715e+01,     1.9113724444673840e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1027626350093776e+01,     1.5611446395453375e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -3.6026504908890956e+01,     7.5411242319862577e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2323441355804036e+01,     1.0544815372440242e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.9523296419410272e+00,     9.5628913166894989e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9854600196564469e-01,     1.5580491649096006e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8630488196827361e+01,     8.3855736080610095e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.9185951762738270e+01,     1.4310411993500191e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.8971231111434967e+00,     9.9165077873946993e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0047540852770524e+01,     2.2773026490680859e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.4992805207222659e+00,     8.3478696186405687e+03,     3.9950555821570275e-04,     5.4318704212574410e-07,    -2.6171212133654968e-09, } , 
      {    2.1719965132551792e+01,     2.4969864938529630e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.2788504447929050e+00,     8.3320885054323462e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8115862816290921e+01,     1.4645378662128536e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -3.7646546786391092e+01,     8.3290047974830668e+03,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -4.2357655887164690e+01,    -8.3283780517638297e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {    1.9694217801345766e+00,    -1.0464648538627085e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3082363264028039e+01,     3.4520309270061107e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.3610687850413740e+01,     1.8286143196230390e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9305817813885717e+00,     6.0879595777516633e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.2415633940137639e+01,     3.8141837090981498e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.7041630937727788e+01,     4.8859821188417001e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    5.3531650862394038e+00,    -1.1147769763223196e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.3447725030512192e+00,     1.4465664974035259e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5805144461368485e+01,     1.9208697122849750e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.6036893841070476e+00,     5.0540117845951445e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9750187213181405e+01,     7.6262620290684536e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    3.7793734999543396e+01,     1.3424846119434020e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0480147215306298e+01,     5.3940731328158517e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.0909213892434146e+01,     6.5700227963135039e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.5620730703984211e+00,     7.2114598414182637e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -3.6035564964659763e+01,    -1.0818135342110077e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6334407024203635e+01,     8.6326701454475860e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.8701417810345333e+01,    -1.0071979884892173e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.1832338551024094e+01,     2.4188233961311307e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5445523349867958e+01,    -1.1144801422078917e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.4293839007335993e-01,     1.2983123283926476e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.5990423517647571e+01,     2.7832038081465053e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5742417275048588e-01,     6.7746412297922107e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.2225060473075693e+01,     2.2128180937919820e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3707405009810522e+01,    -8.0447029434563337e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -6.7165435416550892e+01,    -5.2833482156687523e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2553502371525909e+00,     1.0524107181538473e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6600896130363012e+01,     3.9043880288862027e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2844440463224771e+01,     1.5062992271361367e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.9674172367951881e+00,     1.2037754265764492e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2858154127421679e+01,    -7.7158029507175843e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6275724851450182e+00,     6.6169336838278805e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0102102719505854e+01,    -1.6985969469126034e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    2.0645254117481741e+01,     3.3143913432955102e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.5001712298282239e+00,     6.5669351845580184e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3956502819863680e+01,     3.4310357103616079e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.5289963919779447e+01,     1.5529191752903225e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9657885848411727e+01,     3.9978073239818717e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.1571377568738164e+01,     2.6733926190923441e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.5496938467037335e+01,     1.1224454655616524e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2214088672995004e+01,    -5.2106880407578501e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.1021281527915043e+01,     2.4251348773099966e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.9232410957956123e-01,    -1.1015403862660503e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -8.6744132073592795e-01,     1.9077641817277272e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1448686607458143e+01,     2.4477211394518541e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.3267136598731632e+01,     2.5891355203724044e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.1980475220554236e+00,     7.1991065322202312e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5968731870548991e+00,     1.2636502980106634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6817819115387120e+01,     1.2827690129527455e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.5163373282655854e+00,    -1.5786428343710836e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4969182876699611e+01,     2.1447232888037863e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1939193311677471e+01,     1.9959284961131234e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.7455908762334754e+01,     1.0428230717538070e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.2159469839350088e+00,     1.1488591666166649e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0626430026698875e+01,     2.5430910354959204e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.1215334237725884e+02,     1.5606842065489189e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1056283147891129e+01,     2.3859955726748514e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.9702852098880946e+01,     2.3882935702461975e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.2223011025397703e-01,    -7.7739801689309807e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.0759079376268062e+01,    -2.5011633912966690e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.8702093434124166e+01,     1.8177866762853228e+00,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {   -2.8863292684269277e+00,    -1.3361756357448792e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4448837946224350e+01,     5.5417278445333568e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8644584049386790e+00,     1.5647981138937301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7757738975739532e+01,     1.4750153394996687e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.8607804429782650e+01,     2.8678216434203798e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.1464431107122834e+00,     9.0889962143931698e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.3728953164100481e+00,     4.7434923411318750e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4706849954271595e+01,    -1.2543481811426479e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    7.4047441928832960e+00,    -7.9460416558778334e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.3969447299587681e+01,    -5.6617443517309675e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3521419124864446e+01,     6.2885987457802639e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4418571196999054e+01,     1.5056785631720308e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.6072655009726134e+01,     4.9397219365636447e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9141652486078282e+01,     2.9968250188382299e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.1730264858523309e+01,     7.1437809054122308e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.5132755046174218e+00,     7.7041866091872334e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4397635542221998e+00,     1.4956395175425079e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8935183581843216e+00,     2.4619447915944420e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.6423325877260178e+00,    -9.4613536678292658e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8954473787293779e+01,     2.3871594131662459e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6988185768428359e+01,     3.3384595542352552e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {   -2.5666166341580478e+01,     1.4356780109345624e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    9.6927137133612327e+00,     2.3871297297548030e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4157722899847816e+01,     1.5511602694842819e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    6.0690539636134488e+01,     2.2756458125831337e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.9353106611474331e+01,     1.7285697147843170e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1243423806501996e+01,     8.8172793877914501e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6514976549787580e+01,     2.6545040241444032e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.3656435063339032e-02,     6.0415589118785156e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.0653730444937635e+00,     8.6939310716102555e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9033583240396759e+01,     7.7936685845893317e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8746769241775265e+01,     2.2894221090318865e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5190006945973625e+01,     7.2309236120059122e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {    1.7428287524016365e+01,     5.8718606321551388e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.0675824454009501e+01,    -4.1167944464678731e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.0395470908854840e+01,     7.8402929076201872e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7842343765765044e+01,     8.0619245817571464e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.8117442280039775e+00,     9.5879364270307196e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4722658602905620e+01,     8.8348514799058921e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4390410972703442e+01,     2.2757176184848944e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.0710020796043999e+01,     1.3963005809756578e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1907830351425268e+00,     2.1154080354209495e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.6464628871025663e+01,     4.0580643675248015e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -2.7074549670490072e+01,    -7.7309049792820588e+00,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    6.5056823933367784e+00,    -1.0045890419317846e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.0641198388658552e+01,    -6.1338714668614739e+02,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.5537807490173172e+00,     3.3521685331913919e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.6959284633303788e+01,     1.6001845302138276e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4637097045689664e+01,     1.7474077710991573e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    7.7866228728203541e+00,     6.5320748376347328e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3242216396935923e+01,     2.9973509853014672e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    3.1096014791611886e+00,     6.5819525107034751e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3333503855589925e+01,     1.7730027842601019e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.3631316641928322e+01,    -7.4805163353650296e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1521533746312890e+01,     3.3526944996546285e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.8057296712741714e+01,     2.1500201985249056e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.6704489762867276e+01,     4.0721457348780852e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    1.6399021495787373e+01,     8.9503876101168880e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4593384936107114e+01,     1.7620638799066537e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.5475888533479871e-01,     8.7354519331892188e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0302889151346939e+00,     7.2140270395228827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0394736872145110e+01,     6.6541187534096089e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.6537863024468212e+01,     7.2140986911938062e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7923595427465351e+00,     7.2063479384120328e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5988988341227987e+01,     2.0556425691522651e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.7953058864493382e+01,     3.1692486257031280e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -7.2885850895510558e+00,     1.6193603780159581e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3570561492979150e+01,     2.4062970972139748e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -1.8541441826500339e+01,     1.5224984381359518e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.2593165508743254e+00,    -1.7642660522885008e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.3670915090648217e+00,     3.8814049923750472e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8753093680993004e+01,     7.3421440694349149e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.1629420504418107e+01,     1.3959077753262794e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.9419664223216532e+01,     5.4845412409200890e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0790247828664821e+01,     1.8092395889935313e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.2341518918250301e+01,     3.7777202054698777e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.0394858051025521e+01,     5.2255288389077959e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    6.1808455131353007e+01,     1.2934352117715367e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1544436248561823e+01,     7.9248537436736060e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.5282876685915276e+00,     7.0560254324066314e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5514641199522856e+00,     8.8877249916419828e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.9686581879274353e+01,     1.4953909290876089e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.4621866167193538e+01,     8.6546602795577837e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.3080620637825245e+01,    -7.8536099727268047e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.2200220531168554e+01,     7.2508226272288211e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.9015393187972091e+01,     1.1275155638715546e+00,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.5637285756495949e+01,     8.3121639900782993e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7182899390279140e+01,     4.7626523021889971e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.7597267408671753e+00,     8.3289806289536755e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.3761510122148337e+00,     8.3284022202932247e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2877811093193692e+01,     1.6107783496560658e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.4347527020119699e+00,     1.6122610474746591e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.4404444330876956e+01,     9.3987218451765384e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1080419827796828e+01,     9.5897339674195905e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5158024291817497e+01,     9.7587734330542680e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.9298302398772094e+01,     3.1172650516182330e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4665124973355432e+00,     1.7295999715547932e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5854901578206849e+01,     6.3527698649016547e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.9813520589610214e+01,     6.3785070145179088e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3239147837792688e+01,     1.2755079364424824e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0935191347416593e+00,     9.5419354332919575e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0730959816389095e+01,    -6.1156807223879705e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.2525351239216096e+01,     3.2545811478266671e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -8.1372121947259757e+01,     3.2200039087654855e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.1316948850231213e+01,     7.2986711518028751e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1472977043311283e+02,     3.2200235190802538e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.0302839116297591e+01,    -8.3593596064972990e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -2.1924117629119952e+01,     1.0722477059111887e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0210704557122993e+01,     2.4986074273870348e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8365545095506093e+00,    -1.1610292251382525e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.6371452642259818e+00,     1.6709640600991217e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0436476169375851e+01,     2.6263352960218333e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -9.5772607253801866e+01,     1.1174252039800111e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0775790701516526e-01,    -1.7841841252370763e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    7.4741139949641893e+00,    -1.0037864891978552e+04,    -5.5121316286888629e-04,    -7.2710300282800295e-07,     3.5282267161378360e-09, } , 
      {   -4.8039688598788821e-01,    -7.6328753751612639e+03,    -2.1658957915720006e-04,    -3.2299790357928725e-07,     1.5292526424606202e-09, } , 
      {    6.3700255488858817e+00,     9.6190523821756324e+03,     1.1614799982552650e-04,     2.1550394761358242e-07,    -1.1710092690783507e-09, } , 
      {    1.5473413672502986e+01,     3.3456801069981972e+04,     2.3820577467365049e-04,     5.6364273729143070e-07,    -2.8785327502197583e-09, } , 
      {   -3.5357371166670681e+00,    -8.8894792855893829e+03,    -2.1126181617138300e-04,    -3.2423068780761261e-07,     1.6381320989641999e-09, } , 
      {    8.3989346332047248e+00,    -7.3310669824479100e+02,     2.1639650550239392e-04,     2.5602045720169183e-07,    -1.2072830286989661e-09, } , 
      {    1.9165641828903375e+01,     3.2862196141566819e+04,     2.0492111295692756e-04,     5.2772415217359600e-07,    -2.7605727334689975e-09, } , 
      {    1.2358345605484912e+01,     1.4981966429015862e+04,    -1.2515258453289047e-04,    -1.1394091133436318e-08,    -5.8710936782364866e-12, } , 
      {    4.6124738021001717e+00,    -1.8015317921372367e+04,    -2.1193901594453610e-04,    -3.9218188246371640e-07,     1.9489509980603341e-09, } , 
      {    1.7176284133638401e+01,     1.5299447777121431e+04,     3.9723061510476860e-04,     6.1069361415490248e-07,    -2.9949849964222692e-09, } , 
      {   -1.1801795992205371e+00,    -5.6078786096593467e+02,    -5.8969408409056829e-05,    -7.3511813652957834e-08,     4.0214810034354901e-10, } , 
      {    2.6464110032279649e+01,     4.7776648476334551e+04,     1.4140181832600243e-04,     5.8922548257895487e-07,    -3.1141521992389933e-09, } , 
      {    2.4151682391479849e+01,     3.8819655096497037e+04,    -8.2267079434152334e-06,     3.3789021631013747e-07,    -1.8237284723665529e-09, } , 
      {    9.5633629944942129e+00,     7.7038630507427697e+03,     2.8541701840048518e-05,     1.0395428279855592e-07,    -5.5171796910265192e-10, } , 
      {    9.7552190333891531e+00,     2.4601078831516654e+04,    -3.8726864487151916e-06,     2.0296433871442054e-07,    -1.0665464652096792e-09, } , 
      {    1.6184764549221274e+01,     5.2953399322669429e+03,    -1.8479919271510039e-04,    -1.5285709944249797e-07,     7.0709590576144687e-10, } , 
      {    9.1025394819938175e+00,    -1.8849658859673400e+03,     2.8597462200767728e-06,    -7.0365857676238356e-10,     5.2985287208461214e-11, } , 
      {    3.3373255350326659e+01,     4.6594505822821317e+04,    -1.6773288342051822e-04,     2.2280087832796212e-07,    -1.3979117681112361e-09, } , 
      {    2.9794384691821435e+01,     3.8333328492446002e+04,    -3.8165858108480972e-04,    -1.0081341736548776e-07,     1.8578060260117391e-10, } , 
      {    2.5794855106576858e+01,     3.9657506517447400e+04,    -4.4348773571394328e-04,    -1.7362157244168319e-07,     5.3494341573626171e-10, } , 
      {    2.2495919280971957e+01,     2.3418876157678260e+04,    -3.1813928645388459e-04,    -1.6231474777084661e-07,     5.3936006837116969e-10, } , 
      {    1.9240108794157731e+01,     6.5519438426950619e+03,    -1.9012695570091740e-04,    -1.5162431521417266e-07,     5.9821644925786734e-10, } , 
      {    2.0707016482944606e+01,     2.1466456197787957e+04,    -2.4851435207319376e-04,    -9.1268502574539432e-08,     3.5497088103477984e-10, } , 
      {    1.6546774665385005e+01,     5.0357490432089238e+04,     7.9376798969700601e-05,     5.1650459396535920e-07,    -2.7635349450605764e-09, } , 
      {    1.6483449317364396e+01,     5.8526542118994948e+03,     5.8480310914261753e-07,     6.6802913452395951e-08,    -3.2487849584831118e-10, } , 
      {    1.5441048941449434e+01,     2.3486450272251543e+04,    -3.7464067809720114e-04,    -2.3635568707536736e-07,     9.9740233800983692e-10, } , 
      {    2.3919700060171824e+01,     3.8961690467985216e+04,    -3.7919056431906945e-04,    -1.0134254301705070e-07,     2.4167477189629238e-10, } , 
      {    2.2232598290259979e+01,     4.9242861872824127e+04,    -2.9139119267878542e-04,     7.7184568175571223e-08,    -6.9958614184106047e-10, } , 
      {    6.5738739410329634e+00,    -1.8576105782338305e+04,    -2.7090842435359289e-04,    -4.6569369611667429e-07,     2.3510990984038830e-09, } , 
      {    1.4860478880425340e+01,     5.0147940966352937e+04,     5.1197394524732018e-04,     1.0286327748313424e-06,    -5.1766465614151806e-09, } , 
      {    3.6878372261371082e+00,    -1.7390489547491692e+04,    -8.8188310022258438e-05,    -2.4541729110761760e-07,     1.2646849685423355e-09, } , 
      {    1.5590985207129927e+01,     6.5182468158961565e+03,    -1.5417841249128596e-04,    -1.1632212221050070e-07,     5.3469616075889590e-10, } , 
      {    1.3683666247109628e+01,     5.7816665363179854e+03,     1.8863268042629413e-04,     2.8584653423312725e-07,    -1.3024131692062794e-09, } , 
      {    1.1422761741513369e+01,     1.6275860988226586e+04,    -2.8257968162622755e-04,    -1.9390273468217029e-07,     7.9926383467718089e-10, } , 
      {    1.2055868721448480e+01,     8.5044238229105285e+03,    -2.5461999182295946e-04,    -2.2381607817620547e-07,     8.9293953414116527e-10, } , 
      {    7.9271654034921806e+00,    -9.6528694496248572e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.7038747771808271e+00,    -9.5853553553767306e+03,    -1.5209654303515791e-04,    -2.5080614061725444e-07,     1.2345295575773222e-09, } , 
      {    1.8481438334096143e+01,     2.2128515200126079e+04,    -2.8199487851708491e-04,    -1.2709982122977434e-07,     4.7438533882886960e-10, } , 
      {    1.6125883783709341e+01,     1.3799823775502628e+04,    -4.3428728627941109e-04,    -3.7781869538442909e-07,     1.7103693374495204e-09, } , 
      {    1.5182502159425390e+01,     5.8898848403569373e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    8.8705428389859264e+00,    -1.7429305144791660e+03,    -3.6810411015557749e-04,    -4.3993641790395058e-07,     2.1183885314713058e-09, } , 
      {    7.1842487993555153e+00,    -1.9524799802154675e+03,     6.4493036122042105e-05,     7.2191762962032783e-08,    -2.9472308488329798e-10, } , 
      {    1.0878287621602372e+01,     1.6728400534990986e+04,     1.1910288733682525e-04,     2.8182136864571535e-07,    -1.4392663751098792e-09, } , 
      {    1.7913095263053414e+01,    -8.5322074159169988e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {   -2.6867903451135824e+01,    -8.2354066796120260e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.5260978809457393e+01,     6.1515338973226403e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.2985211221353211e+01,     2.2701460225153802e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5357645361336365e+00,    -9.5661189249585059e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.7541838554616390e+01,    -3.3502544551083702e-01,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.3876682120576291e+01,     8.1770173531018727e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.1710230376189294e+01,     1.4743594439397551e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.0946713336104757e+01,     3.7711078762174307e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.2753510785990457e+01,     5.9595252380509583e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.2210060278941995e+01,     8.3305632521811294e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3031663829274558e+01,     2.2397577613102309e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -7.2700329680776008e+01,     7.5238512352205908e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7003785467922533e+01,     1.6856115603726350e+04,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {   -6.4837249555582986e+01,     2.3870741082689095e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3939578302334112e+01,     1.4354831225362290e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.5055116757992664e+01,     1.4604658335074006e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1954662406771391e+01,     1.3936535848130550e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.5688024170117828e+01,     8.2655700515171193e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.6363815509716417e+00,     1.1095570696473662e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1485275195094932e+01,     6.8079409504130545e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6039340065601479e+01,     1.5525144141703522e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.9306778419620983e+01,    -8.3647740520200205e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8113867583553855e+01,     2.3384413394918807e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.8063582394440346e+01,     1.6480850244889582e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5830088840995471e+01,     2.3973617423533316e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.7428335574734195e-01,     4.7970957466745688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.2747313768635891e+01,    -8.4369118675094251e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.3737516798441863e+01,     1.0151794402747257e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.8056685948778775e+00,     7.6267875102248345e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4515539003794622e+01,     6.6593784180419771e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -9.4706936869631839e+00,     2.5342677422474320e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.7157728624633872e+00,     1.8291599877891294e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.1419765215653506e+00,     1.2243482010413647e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5996805385975335e+01,     1.4286041068316868e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5974421654682551e+01,     1.7316966219877322e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.0601880020073310e+01,     2.4092492840454943e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3882322952115871e+00,     1.4910643935326925e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8678234715399302e+01,     2.9420124696665069e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.1324778446255836e+01,     1.6135724804090616e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.7318314484608479e+01,     7.1434218759034284e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8002066045689514e+01,     2.9412462213671723e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.7909355876013677e+00,     1.2476138793450018e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.7928503398472557e+01,     1.1548128185169751e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.7875466920488051e+01,     8.3775417203822508e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2729224058211518e+01,     2.2030212961336765e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5375317177523939e+01,     8.3290332183777155e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.1239430115980952e+01,     8.3283496308691847e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6453429409912506e+00,     1.5556316827060364e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9077191455891782e+00,     6.9945239935202399e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1782268519753849e+00,     3.2191500640845858e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    6.8665780338797084e+00,     8.3283215616584821e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2919328989884296e+00,     1.8684620847584283e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8228934627142099e+01,     7.8881494886025193e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.2162828104473064e+01,    -2.1496885373606585e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1382194026223402e+00,    -1.0513501807984267e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2225509973295985e-01,     1.2083117823757031e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5536074787419565e+01,     2.2851988987424396e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.6624102401764311e+00,     7.2497190998346132e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.2273723636127521e+01,     1.7754738411746337e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7340500257809808e+01,     1.0051950702136699e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1326131354925399e+01,     1.4476685600757621e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2588491185314695e+01,    -1.6608680983432343e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {   -4.4897770510205383e+01,     2.4306784078339660e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.0428228548979540e+01,    -7.6846513555786078e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    8.1219155950848076e+00,     7.0720875141953293e+03,     1.5762017074814322e-04,     2.4948608992632941e-07,    -1.1271045421170713e-09, } , 
      {    6.6692516643169766e+00,     1.7157061822475678e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7132636244427374e+01,     2.9748711316544195e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.4270748223056816e+02,     2.5802166984828375e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.6051219923211292e+01,     1.3195380251954703e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5993692028163345e+01,     1.5780670737650536e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6340269519828531e+01,     6.4652233281912895e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -5.1114741157639134e+00,     3.0485665530759143e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0587394295937052e+00,     7.6584353690890794e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5316985267745391e+00,    -4.3371161422579974e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5206188902582024e+01,     1.1155089273545074e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.4759664129130883e+00,     1.0309528757087688e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.6510174472111352e-01,    -4.2587963168090046e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0877706758032303e+01,     2.5001806757869646e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.5805334207945451e+01,    -8.3441894144972885e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0174752272834027e+01,     2.2050512683402168e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.9822659297010219e+00,     2.1278033281094766e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.5368667335636452e+00,     2.6193101123747433e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.8230389298102082e+01,     2.1548962231009627e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6592596830896131e+01,     1.4936881204643421e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2048470459939026e+02,    -2.3992515152491194e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1217752258498052e+01,     4.5585172819754180e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9159668048441691e+00,    -1.1856162348466132e+03,    -1.8272011433133451e-04,    -2.2027640500905660e-07,     1.0864141298615479e-09, } , 
      {    2.7975780304504507e+01,     1.3404182603177127e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.4310524274028030e+01,     6.6759267797429129e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.1544229913186294e+02,     1.6655347332229827e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.7232894457627420e+01,     7.7266922302044641e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.8484021569003524e+00,     2.2625246136899386e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.1884772540513318e+01,     1.6645176924522922e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -2.5924772553787744e+01,     1.3171158244667527e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0287898780506565e+02,     1.6659418366263973e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.6971487433359886e+00,    -2.0300378710590744e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.5742399876225548e+00,     3.9898219778865190e+04,     5.4298645492547107e-04,     1.0649932191381403e-06,    -5.3519551885043885e-09, } , 
      {    2.6350456458426848e+01,     1.2967390799927176e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7861512754669491e+01,     8.3282339824333649e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.3150403653895893e+01,    -8.3291488668135353e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.1667887580024896e+01,     7.1207628608317709e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -6.1307012950914988e+01,    -1.6669588773970878e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -3.0744712037554067e-01,     5.4404737986890004e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.5474564312719572e+01,     2.5211356223605948e+04,    -2.2511008419136763e-04,     1.5023212424749856e-08,    -2.9229416834093930e-10, } , 
      {   -6.3468352889651882e+00,     3.0966063874269854e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2448701137481445e+01,     1.8958736800584309e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0312936555120864e+01,     4.1313716517422799e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -2.5782512541016627e+00,    -3.2350534700215889e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5124882604630006e+01,     3.8837707530937361e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    1.4554161679128601e+01,     1.6731005986976077e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.7434696976588724e+00,     1.4221704770076851e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.7333215812901088e+01,    -1.4430307028254538e+02,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.7856333647911725e+01,     1.6100061856063621e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5571219054628619e+01,     4.0685408932377257e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -6.9606700909781111e+01,     1.6100075283165075e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5879795685703511e+01,     2.3323320222888007e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.1830484377640644e+00,     4.4760339887056998e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0945521665817111e+01,    -2.1063982433002352e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3475878385659641e+01,    -3.3452795080387659e+01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.4650192692193951e+00,     2.6555455833829265e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9765527531387988e+01,     2.3872150346521394e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.2995714489625755e+01,     1.6170963823453496e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0492132341923657e+01,    -1.6877749026240053e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {   -2.4451519432296450e+01,    -8.3771428249041965e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.1489441273275043e+01,     2.5027789681233926e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9334038811912961e+00,     5.5180260597011238e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6266537619246243e+01,     1.3069338851697534e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.8536999738555213e+00,     3.4415875227689012e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3199341313121579e+01,     6.5591656221947687e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.9424198424269285e+01,     7.2248795766152016e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.3358928075903613e+01,     2.3860629003348389e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7728189484854965e+01,     1.6703740546905985e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1364008147476525e+00,     3.2262493946258852e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.5310193173953799e+01,     3.1542963899470433e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.0665766795761041e+01,     3.2494383480802513e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.3504485279016372e+01,     7.3875297036980737e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.2824837553101414e+01,     6.5859789374705106e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0750357888011294e+02,    -6.2627206802846706e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6828633579978742e+01,     1.9622388170117301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9497500173832179e+01,     1.9618079102834115e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.2513512908240196e+00,     1.3310822232543839e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.9591007252497101e+02,    -8.9575069683081110e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -9.4011209855248865e+01,     8.8929112341615983e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2845123262296188e+01,     1.6577183789972627e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.4712840094210566e+00,     4.5562160820340068e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.8350245782690052e+01,     5.9573496436934183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.0541770614182013e+00,     1.0702549726780673e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6549761813928498e+01,     1.0324449195447087e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.6128968143394143e+01,     2.2784047117403221e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.9725038895398459e+00,    -3.3084610674834657e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.4703653037014508e-01,     1.4614725739769839e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9408685599129186e+01,     4.0791347126799643e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    2.9651362290294410e+00,    -1.6869074553205020e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {   -1.0651349050384598e+00,     1.0278377161948709e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0524074353696034e+02,     8.3293469275385669e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.0052963443618671e+02,    -8.3280359217083333e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9522277873201041e+01,     2.9435872440179515e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.7704789391257663e+01,    -7.9628068798437369e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6144540367859477e+01,     7.2229412538588886e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -3.5538222919818026e+00,     3.3614054294405942e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2759976505538251e+01,     3.5809595607938509e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1132340865483147e+01,     2.3828785898649541e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.1006243185898761e+00,     5.7494524157190983e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.9491750652782528e+01,     1.6656994707453363e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.2055610507756221e+01,     1.6657770991040437e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.8126696478084554e+01,     7.4529522767458184e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4684919719041687e+01,    -2.3708914133981166e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.4666758037315091e+01,     8.2776370004026212e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.1062572559025980e+00,     1.4410076787260830e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    7.8852307523363914e+00,    -9.6183766767028210e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.1780587702564311e+00,    -3.2892828734663444e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0237439986434005e+01,     8.6038292259889295e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8441500909903567e+01,     7.1571806819207850e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0687952107146103e+01,     2.3123693423865963e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.1523023891365725e+00,     1.5541331599048041e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7769737918570286e+01,     3.2960441928998422e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    2.4283010826833731e+01,     2.0911709302101423e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8369378825328944e+01,     8.8037728765200955e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2343652402310282e+01,    -8.3264244356750096e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {   -6.3411433821427332e-02,    -1.1021770397877963e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -7.0188923999749605e+01,     5.0851056339373156e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8800573645655628e+01,    -5.0605707401942800e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.1450238367886470e+01,     2.9931065118800572e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    3.6163208508408999e+01,     8.4549455514501333e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.2925966119900725e+01,     7.7578706337195435e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0662770247340088e+01,    -5.4399989966135863e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.2767499082503257e-01,    -2.4950025857466757e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {   -6.3372622060508959e+01,     7.8579124786923185e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.3761558687121074e-01,    -4.6518172865019924e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1142358433712255e+01,     9.3162385519068193e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.4370934883023835e+01,     5.8151324520665048e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.6437176750077676e+00,     5.6550968977846615e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6989877369372024e+01,     1.0218545212017674e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8752155633943755e+00,     7.9225695096781601e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1396481057928533e+01,     3.2738273224844124e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    4.6767620295419583e+01,     1.4433056762974287e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.0536031187171460e+01,    -1.4193498570856609e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.1895403056160543e+00,     8.8834026803159213e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.6468224450338855e+00,     1.6631645699630648e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.7010374546077616e+00,     2.6250908720858274e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.7286076261806883e+01,     8.3140214517978638e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9419003918753255e+01,     1.0993097163546921e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.3392678855166395e+01,    -1.7808449773770553e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1277588480038878e+01,     3.0740053688709457e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6857886552273030e+01,     5.7573173409679396e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9090177048248069e+00,     1.6577822383593106e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.6960020258142130e+00,     1.1850461040720660e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6856664107664955e+01,     2.2440028908634078e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.0473270576507623e+01,     7.4006476148963429e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.0177565298640738e+00,     7.9628929267864887e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.8686029574581138e+00,     1.7249582083728819e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6483111043098875e-01,    -1.8340800416486134e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    8.6725383455869931e+00,    -2.7943924375508624e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    7.4912486108926633e+00,     1.6689022038480241e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.3701372571247443e+00,     1.0510676018353101e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9715269554683212e+00,     3.0837602726193050e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    6.7006172227535794e+00,    -1.0190685853873151e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.7673000827789522e+00,    -9.5617467210442531e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5989970011441281e+01,     3.2807114816296387e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.9106022941147991e+00,     1.4807916080899506e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0961479710776647e+00,     2.2575015642616247e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2347315200337415e+02,    -6.2158992903459875e-01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.8883640379819536e+01,     9.1124321640680137e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.4513680839600553e+00,     5.6485379421855278e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6123772311840181e+01,     1.6250622340681130e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3685182911263274e+00,     2.3416639178892401e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.6953136100360982e+01,     6.9403450938454881e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.4810770988309965e+01,     1.6868028195850689e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -3.1322391961938404e+01,    -1.6865836434115714e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.7431064946697862e+01,     4.1131398999281439e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    3.8509303591248646e+01,     3.2218186082684559e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -4.9676887298806456e+01,     1.6122519162939961e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.1517150038432895e+00,     2.4905892180542040e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.3717736940393923e+00,     1.5107135424024986e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6952879609783121e+01,     1.5588533328180427e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.3345470385325925e+01,     1.4365992580844240e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5767857592130170e+00,     1.5533875901481253e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.8714070805716233e+01,     7.8908218506846342e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1245665965580173e+01,     2.7637362101595558e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.1007518019949671e+01,     3.0010694922596398e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -6.8046964602142467e+01,     1.0857463793094053e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2381286160480848e+01,     3.8153098183625661e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2118751027645041e+01,     8.7113411933690677e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.5100290794754972e+01,     1.6864854772199502e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.1036751486194945e+01,     1.6869009857766901e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    4.4010902481815393e+00,     7.2090871584558763e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9810366936613640e+01,    -1.5099507288995046e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2888698437006695e+01,     1.6155519846738907e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4036222147232600e+00,    -9.9127202773005229e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.0721000944617138e+00,     1.6625743660013559e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1694813346784102e+01,     6.0221487646676896e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6124062245885025e+00,     1.5105962486731845e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.0668698657620485e+01,     1.6852329081545162e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.3618983978522991e+01,     3.0683671141461942e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -6.3183427880747438e+01,     9.4412942935054070e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.2125537035452565e+00,     2.4251847445880245e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.5996096894047923e+01,     2.3845137323978688e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.0696903175888600e+01,     1.3318662311617991e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2540805967154647e+00,     6.7254749021903444e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.9833650842597544e+02,    -3.2585268551974877e+00,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -7.7646321590846874e+00,    -4.5922987094305790e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.2112183344142737e+00,     1.4712941517038857e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.6697287284619485e+01,    -2.5179891255607348e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.7090906574542469e+01,    -1.1174734733228970e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2198851042969741e+01,     2.2302010619627297e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -3.2545886212712631e+01,     2.6247371257482882e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.6790767727078332e+01,    -9.6627254415341639e+00,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.5537465953265144e+01,     8.9192616505547812e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0058996154403044e-01,    -1.1169335447391610e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.8038339936606405e+01,     1.4434159205159442e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    4.9058016609706641e+01,     2.3944740219959644e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3080887591191745e+01,     2.3839806525371903e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6800652416623166e+01,     2.9924494614340481e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2058845540244995e+01,     1.6622553782020605e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0122768783944986e+01,     2.5066256367198657e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1076124532928887e+02,     7.8424685019777271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4711362368365275e+01,    -2.6518616596204794e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0307612632973591e+02,     6.9440893973940110e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3333174084339063e+00,    -5.3274761116775289e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1611326695162667e+01,     7.9406014871880579e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9990480786576459e+00,     2.4960337124254096e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.6955923056893848e+01,     3.0751447336491859e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.5403450913362944e+00,     2.2743310644068788e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4775576226582292e+01,    -1.6669380459990371e+04,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {    2.0873121832097052e+01,     9.4170841614920209e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5398510670123695e+00,     1.6699331319735727e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.0930523448329609e+01,     8.3282620522397192e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.5641632549103221e+01,    -8.3291207970071828e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0618022145294606e+01,     2.4411839313430202e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2374462034546447e+01,     1.4966261816332124e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.8351888249093811e+01,     2.3829497244116417e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.4951599347351504e+01,     8.3159258293832590e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4039546895487920e+00,     3.3001945773606945e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.6104456325171153e+01,     1.6459308111852173e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.7362788662459394e+00,     7.8269405965112646e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4892505940560099e+01,     5.3314413101631235e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.4043045040700742e+01,    -8.8727285336685213e-01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4823771118220710e+01,     7.2190385722608135e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3868048897307659e+01,     2.2813905054003721e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2389849265749984e+01,     5.6573980977260726e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0976609770384581e+01,     1.5431420601514510e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -6.7702647807119742e-01,     6.9925157888785707e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0307445157347546e+01,     1.6615434378758073e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5095192263199102e+01,     1.5545021278930233e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {   -5.1524860049631691e+00,     8.1814987659173057e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.2822409490753873e+01,     1.1571074959436370e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.7547558825912205e+01,     6.7782149650000247e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.9662013151405255e+01,     3.1905331946804723e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    2.3426294127351600e+01,     2.3342464924429980e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.3032616887357644e+00,     2.4670605565189486e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.3471379290231180e+00,     8.5595707983376033e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.2944455115520932e+00,     7.2025976127556787e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4136234276937380e+01,     4.5176969304449841e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.8079187054084393e+01,     2.3866378642563555e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.2360882810179241e+01,     9.4617363740176861e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7408001248922478e+00,    -1.4301162194237900e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.7211390317534104e+01,     1.8151914467006725e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.6838550875506524e+00,     8.2941986517014138e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3786464056412260e+01,     1.8045995224062161e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.4394614017268948e+01,    -1.1789264406641814e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8550546759181216e+01,     3.4100807637879778e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -3.7729514189447578e+01,     2.3774542914043755e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4903267369728667e+01,    -8.6003958610650188e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.5751753877079340e+00,    -1.1117836452073143e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.4741611259458995e+00,     1.5630668276202379e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.5682085520694162e+00,     7.9794412884874255e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -3.8415476048517220e+01,     1.6520668181919082e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.9919422407335219e+00,     2.2382941009970498e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.4660726268644851e+01,     1.1525586280037509e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2480620473442556e+00,     1.1457440071027671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9556067106012762e+01,     5.6078444235071001e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0430368541335124e+01,     3.3951868568957543e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.6613384193448217e-01,     1.0671398131641698e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5430259880682105e+00,     5.1134876390079637e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1734861053272283e+00,     2.4335426620017446e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.3998970349559606e+01,     4.8600921327678207e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6352055463921424e+01,     6.5684822835471350e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.6833995499442707e+01,     8.2622158161409043e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6132094510846478e+01,     4.5252428724924721e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5804064274242524e+00,     5.4868744645621464e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5822755833738158e+01,     9.4631590649514364e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5671708396365400e+01,     3.1821218486675207e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3003828211275490e+00,     1.2869159763143005e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9823945757958228e+01,     2.9829245725830577e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.7840705744632011e+01,    -2.6663736439988224e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -8.4489403875074842e-01,     1.4317290715029685e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.8427609517001713e+01,     2.8767233947818593e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.0126261089422140e+01,     1.5666707216867066e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -2.6728708893692236e+01,     1.4563112464698856e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.5775143055230529e+00,     1.3655201702528982e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.1290829265375763e+01,     1.7341047364478785e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.2120089823933228e+01,    -1.1155949616661680e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.2391000695142434e+01,     1.2543701648360338e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -5.3694000767096401e+00,    -1.9585190844897022e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.8613884446758266e+00,    -1.3156276749406587e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6374022762943444e+01,     2.4552722396861980e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5635869076324958e+01,     3.2691373886190700e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    1.4232261129735107e+02,     2.5448116029365906e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.5794353196170960e+01,     1.1485771378139596e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1737332053903311e+01,     9.2901055682505539e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0159490165798488e+01,     2.2774449181614611e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5415973678244654e+01,     3.8352540542533126e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.5593613297950023e+01,    -1.1944973905125039e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.3094707271063676e+01,     4.8630571470440918e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.1958637840858643e+01,     2.1493647195527956e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8147818907946586e+01,     6.1392492079911526e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.4997426614028981e+00,     1.6163435291401067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2921247901698393e+01,     1.5520006669623528e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.3169477456060390e+00,    -9.1982636542276879e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.9177076780340112e+01,    -3.2432886422140837e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8090398039587379e+00,     1.1064419101334683e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0804161274133705e+01,    -4.8596865826148735e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1685119280982935e+01,     1.6657737452696718e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.1107337482530145e+01,    -1.6657028245797079e+04,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.5995838611506041e+01,     1.0719062889575645e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3738558168690339e+01,    -8.1824470581397472e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.7723353368512100e+01,     7.7398464255177450e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.0751168948143089e+01,    -1.7430341958844879e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1909113584736673e+01,     3.9159768564911137e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.2943968481760159e+01,     1.9093060928416948e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.9824834274083809e-01,    -1.1731972189240395e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8550389397734584e+01,     7.7228183392850769e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4689459136721922e+01,     8.3951670331059959e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0057646216500295e+01,     3.5498079656548744e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.8782712464512301e+01,    -8.5044747224197472e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2210513854040208e+01,     2.1564191709182611e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.9818493942400774e+00,     9.4477292110588151e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7767894462154839e+01,     2.2159676424948724e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.3088520486797854e+00,     3.2968067153442598e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.5590474404696835e+00,     8.8702579607415519e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2684510702251227e+01,     1.1429802816823913e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6640976654205019e+01,     7.8637003583817395e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9716132853084435e+01,     8.3090145578260490e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.3635311757364029e+01,     8.3268084157811500e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.9883660747522015e+01,     3.2185531874774915e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.0529043735775474e+00,     9.3741415513366464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2253153558145886e+01,     3.2130247361209909e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.7880759752429793e+01,     2.5494937153222156e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.1391541040119122e+01,     1.4954613922792239e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2764682836235620e+01,     1.5041003461869610e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8201460106816157e+01,     1.5780300874685565e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.4423517449155131e+01,     2.3457754667337693e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0775224380700106e+01,     8.8334287889721418e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0605368047560987e+01,     2.3482027540947176e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.7704166723825256e+00,     4.0698563127690533e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    1.4283945994551187e+01,     4.0002692687734670e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.9080964559043451e+01,    -4.3992030834641400e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.7877570557366845e+01,     1.6084799957373775e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.2769947193794440e+01,     2.5193546196822947e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.4147749903425368e+01,     2.5016727196229047e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    3.0758185710570231e+01,     3.6523367805851558e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.7244693809056841e+01,     2.5132991916953924e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4857645460002153e+00,     6.5855316231550914e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7929411039377701e+01,    -8.2912203172860973e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.3376547527899646e+01,    -8.2134862009235258e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.9413258749717450e+01,     2.5120372678535939e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -7.6406669531337492e+00,     8.3313059974244788e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.8175544253322016e+01,     9.9471759692685482e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -9.2254456569740142e+01,     4.3025059655137738e+00,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    7.9473468394338809e+01,     1.4332953898632435e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.6198133673338987e+01,     1.5530548365257817e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.4704939816951736e+01,     1.9570123704822257e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7768500512796834e+01,    -1.0757800597682558e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.2150335938154996e+01,     1.2854722313919543e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4562114144698590e+01,    -3.0942941034914782e+00,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -5.6787697158171646e+00,     4.4786638210218825e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2435105856734701e+01,     3.2195206106971094e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    9.6984908901035887e+00,     9.4896776815476424e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8381430056393748e+00,     5.4983566831107773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9482688694157055e+01,     1.5540014141048568e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1185857852747816e+00,     1.6997444197467605e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.2901131822327088e+01,     6.9630437660064927e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4050626812614787e+01,     1.4350840189291141e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5251551960231428e+01,     8.8195651671358246e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0941232117791264e+01,     1.6125213498686673e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.9187457347034247e+00,    -7.8918996213734990e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.0745481148920433e+01,     8.9085476093882462e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.5722040997805049e+00,     3.1080851301473285e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4619216199106416e+01,     1.0627298665709961e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -7.4910991495755326e+00,     7.0270553047542871e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -1.3334974991024296e+01,     7.7965220740632249e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3331771060061428e+01,     2.4068543660864234e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -2.5466532497417404e+01,    -8.2214526436536926e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.6933780660256737e+01,     2.7866089761348980e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.0662283983330255e+01,     3.3208347782768957e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6924172829644043e+01,     3.2456087270838147e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -4.0578253035884231e+00,     5.4198102824321068e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5488283253396617e+01,     7.3133896602286241e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.3775159291531196e+00,    -2.9426625696465053e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.9291440274556031e+00,     1.6627370850363764e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.5860281224685338e+00,     1.4370747406335238e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.6262419014126870e+00,     4.6627910553987305e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    7.6432649898202584e+01,     4.6628615185903465e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    5.2843069448106803e+01,     2.2770323666611494e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.2597182776562512e+00,    -5.0448382561949802e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1543790029437656e+02,    -1.6659113817724141e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    5.4874281617541740e+00,     4.7648222491097267e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.2421649020044924e+01,     1.9876819609793201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.2980645377096572e+01,     7.7557407697642484e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5344159425185630e+01,     3.2768926147202823e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    2.2637737649085764e+01,     1.4663412346069244e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3417721924291499e+01,     2.5197701282390353e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -6.9090580896081306e+00,     2.4968025330414490e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.8016480887311793e+01,    -8.5562898338156156e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.5168214923256773e+00,     1.2779450780689383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6149267076830967e-01,     7.0165136091200002e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2730837055417894e+01,    -3.5607702682931688e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1005815156930787e+01,     2.4588646062538959e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -2.9103758446174503e+01,    -6.4558058181121123e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.8793475445267227e+01,     8.3263835111313147e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.7801627487495146e+01,     1.5474062184510214e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5546664369654168e+01,     1.5545494438915021e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8658392461703713e+01,     3.1023151772933434e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9929890570386668e+00,     1.7053800899748785e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2897382691150412e+01,     9.2106479413370325e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2623901643120549e+01,     6.9515537781497615e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.7291795946102262e+00,     8.3650579402314834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9093109124497296e+01,    -2.6256751690320953e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.3201739578288727e+00,     9.7702634209109965e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8690541670187848e+01,     1.5544176980915548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2331012356988200e+00,     1.6654779825306818e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.0285545472277455e+01,     2.8429962996580660e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2232080604280139e+01,     2.3611594886929579e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.7329642212083691e+01,    -6.2791944325200711e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0079803942270100e+00,     1.5461399259360345e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7291351188445056e+01,     6.2869572683907927e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.8062569613284779e+01,     1.8580389537804207e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6619176733482668e+01,     6.4468468040314174e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.4579709970910494e+01,    -1.1270800787424132e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    5.4440042323387388e+01,     6.4475514359475656e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -3.3752444860351005e-01,     6.6764254525231927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6418470139885436e+01,     4.0112371644484316e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -1.6552986497111107e+02,     1.1411445970386922e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.2231773912286172e+01,     2.4031481268887088e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.2692984982143543e+00,    -9.7059805921839484e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3714705467975357e+01,     8.9100996710155669e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -5.3690107552331057e+00,     2.5257756889479097e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.0247679377829288e+01,     9.5433581242257096e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9697026490430574e+00,    -7.7025260777997282e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1794752278651327e+01,     8.9084391396280607e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6857005747121853e+01,     5.7202892523570226e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8599230214198073e+01,     3.7593574313911759e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4494812030085539e+01,     1.2964300719889986e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1630780142996985e+01,     7.8515592383192052e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1894211244154675e-01,     5.1250050900081360e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -9.6284938854230244e+00,    -1.4885047046347456e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5048699709858852e+01,     1.3602474604658822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0345840425074456e+01,     1.0762068303852089e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0640336987990565e+01,     1.0810770169666646e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2968375609957713e+01,     2.2772564898854591e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9328218900161978e+01,     1.7248872008547860e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4480712997486858e+01,     2.4046502407419506e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -3.4559033928191294e+00,    -1.1506640609622511e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.0247643429013894e+02,     1.0840190796328383e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6891851886821851e+00,    -6.5289074884647584e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1562657508002698e+02,     3.9306943258792302e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.3870749024809509e+01,    -8.7577950217130074e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -7.4752875747616713e+01,     1.0523402549622326e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1633290167057581e+01,     5.1894546993371441e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3172547725075059e+01,    -8.2937807466070317e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.5843491749168173e+01,     2.7697334480814698e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3975044637252527e+01,     9.4608549950489851e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.2442111913421712e+01,     1.9946167049932963e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7080976224103974e+00,     3.2528723759107823e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    4.1006872126274398e+00,     3.9769649662528382e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2158204333568275e+01,     3.3312135866177618e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -1.1423150650852310e+02,     1.9034366416850844e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9472565290861930e+01,     9.9760263644661136e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.2823847336588035e+00,    -9.4541463540310679e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6110925080711560e+01,     3.3317395530809983e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    3.1339180876493497e+01,     3.8246602348110740e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    5.7623469174143382e-01,    -9.1655382187037992e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.9764436475624945e+00,    -5.7649247364967346e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.0362976861946452e+01,     1.7850008295404430e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.2679452206278768e+01,    -1.7932260024553016e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.5474192079438174e+00,     3.9414528636961812e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    5.3176202246852000e+01,     3.9413871372212270e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.7070976113368733e+01,     3.3728954172973208e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.8231159583337895e+01,     3.1713463849051059e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.3896390433158061e+01,     3.0546218508532835e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.5824810185614776e+01,     8.1976540609826643e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1689360908420738e+00,     8.3886641033140768e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.3976984657770481e+00,     7.7429912115806392e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8989063045144960e+01,    -3.3351765213672479e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.9106408779892192e+01,     1.9542758366321475e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1379725870608464e+01,     2.6211341187026846e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6037229592150805e+01,     1.4492560080530548e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {   -6.7493656592945754e+00,     3.1080577547705991e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.2596056805088134e+01,     8.3485904398463572e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.8849477043145288e+00,    -8.3087924094005448e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -9.0214849948836218e+00,     3.3546857069362348e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -4.4731952231207273e-01,    -1.8165032378204996e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.3797077359682765e+01,     1.6808558247988200e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -7.8500863902575446e+00,     1.7032354875484027e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4106648020401185e+01,     2.3620411355738241e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.0635209308430692e+01,     4.0830130190127111e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5162262020627427e+01,     6.4924552757862275e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.3658968838750127e+01,     1.4270139611666711e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4917278207159924e+01,    -8.5018448901035608e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.0261901353073721e+00,     4.6609908149021576e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1454755089323736e+01,     1.1520996666024562e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7952135013510294e+01,     2.6938061211137021e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3218650211227958e+01,     1.6310781496904881e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.2746861300437029e+00,     6.0420559817117883e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0267275260339369e+01,     8.5360986521379309e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.7343534570467234e+01,     6.6124159868974348e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.5054649143552467e+01,     5.3517126997836465e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8362235567745140e+00,     2.6403545199929872e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1634937475531455e+01,     3.5050000242173023e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3892258433852511e+00,     2.3230518891141714e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -9.6255549671041479e+01,     9.2893292846634831e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5563080823800634e+01,     2.3903084903838586e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.0351706245301934e+01,    -1.7408586015269484e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5270830159474389e+01,    -8.5078395824690460e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.3516345153282101e+00,     2.3897754105231801e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0178837922171255e+01,     7.2160272130278245e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9019176337964375e+01,     3.6417057449542335e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0659332476193168e+02,     3.9414193291036310e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -5.0869703270963470e+01,     3.9414206718137764e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.9641781585488435e+01,    -1.4669972825586001e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7003541144234212e+01,     2.3654192783193394e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.6463568033297300e+01,     7.7848836562622519e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7260678801687448e+01,     3.9623397154365259e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    1.7528781990878560e+01,     1.8195685300563688e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -2.1859494348998517e+01,    -1.1195595915227022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4139534977440400e+00,     8.3275374388081291e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.2553432859377551e+01,     1.1754546463393051e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.4561597015155407e+01,    -5.3502284003411842e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.4582881093660269e+00,     8.3636021026398703e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.6740970296016160e+00,     1.5870782059010771e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.2783912653769320e+01,    -1.6688863076760648e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    6.3790534790512524e+01,     3.9624101786281419e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {   -5.3182129185983726e+01,     3.0457207708468770e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.3760118283921464e+01,     2.9344873635594351e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    3.0130718753613404e+01,     1.6315678318169503e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -8.1989019520712336e-01,     5.1815111395869735e+01,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.7426093348360894e+01,     4.0778192931486366e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    2.0225305038426463e+01,    -4.6675497705012913e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.2330540988728185e+00,     8.3109095053922883e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -6.4402828583278389e+00,    -4.1190956464092824e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.0001891129298590e+00,     1.5994130375191966e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5010921413003819e+01,     1.3964111455610711e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.6543711612438248e+00,    -6.4075910452285177e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.5292754409033762e+00,    -1.4726059428237124e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    4.2118048212810617e+00,     2.2803716493993568e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.3004966865040720e+01,     3.2792396307870644e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    6.4578549860031007e+00,     7.7172445862254744e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.1851239376123424e+00,    -5.1430540334639973e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3506788728978691e+01,     1.5687370733123960e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6591732210193289e+01,     4.1314870503238119e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -1.7360075049971712e+01,    -8.1114537527267585e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.3118081798196840e+01,     2.2130587113078298e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.1301423818407052e+01,     2.2932600453136438e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1183029049632628e+01,     1.4988034364658419e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1172791515570340e+01,     8.2168531761158392e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.9838409595999487e+00,     2.9683844597476054e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3059656681446867e+01,     1.1373326549289673e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7691119297076172e+00,     6.1332756677084253e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7309663892002703e+01,     6.1432203991653532e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.3309881017788655e+01,     1.4691764308357597e+02,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.3166056981423999e+00,     6.2922360125585897e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4134523576426277e+01,     1.3206455449634996e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.1398221658167092e+01,     1.3273670720692722e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6082568493982421e+01,     7.3694960197573655e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8832332993562659e+01,     1.4731008570673339e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.1892015391761392e+02,     6.5837197632957259e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5736786196864372e+01,     3.5436159172799307e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3386621805573196e+01,     8.3644217418226126e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0197258251022957e+01,     1.4860766262258185e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.2790369716925412e+00,     1.7893215506686447e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4378662805731398e+01,     1.4988745710125295e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0683735430203878e+01,     7.2139220780853320e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8238009532848345e+00,     7.2142036526313568e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4034534847869548e+01,     1.0766999336109684e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5778000563329819e+01,     1.4852089897906095e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.5216585266557203e+00,    -1.4741631586385483e+02,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.0943688792271544e+02,     8.3284644527928667e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -1.1414799702348904e+02,    -8.3289183964540334e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    5.4601714916220470e+00,     1.7336332608395169e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.0968952223492209e+01,     6.1060457056453433e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5367226843266783e+01,     2.4990483501040606e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.9429927605150496e+01,    -8.2738209453510444e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8185580181787692e+01,     3.2280319232557005e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.6798547480425661e+01,     9.9569652974128431e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8653183069407760e+01,     3.9493829808382863e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.7680518593535538e+01,     2.8731185531414991e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.8302460686747359e+01,     2.8885117116146099e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.9872800339580305e+01,     1.5638355494449777e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5956351021298495e+01,     1.6666261237747443e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    6.5341328197512842e+00,    -1.6648504460746357e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.3330417433174878e+01,     1.7601159142973305e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.4713272526806236e+00,     3.3366580809889667e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    5.6099857449023979e+01,     3.7673565060217807e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -4.9096612798558795e+01,     1.1597123695295941e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1408187246875613e+01,    -1.0640265620969149e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3324749484938526e+01,     1.4434737130268793e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2118575260133353e+00,     1.5517015154833683e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0028008255738884e+02,     9.9176368697090256e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4356505379966226e+01,     6.7980760004702205e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.6328871447984554e+01,     7.6121175373992482e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7834495135613338e+01,     2.3453398940762880e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.0006477120091546e+01,     1.2658275920145388e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4187991055416710e+01,     3.9739805304868387e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0653735393495998e+00,     7.9874855663263324e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1594955023097166e+01,     1.6517682178847179e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.6294404200110719e+01,    -7.7556446609915984e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.3141458807669331e+01,     1.5565501910340059e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1813918588243363e+01,     1.0937481738683413e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.9109856509582627e+01,     7.7471042713814149e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.2275828729259885e+01,     9.4509583175160242e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9612773179598669e+01,    -7.7193081741054357e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6760389762508641e+01,     2.3888306461252807e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -1.6277022136061785e+01,     4.6171307516456727e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.5628324673366105e+00,     2.1765958442050760e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0956461145800311e+00,     7.8203291969591592e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2266037322518144e+01,     9.8009163432696932e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    9.7419509412491323e+01,     8.0587179566591167e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9345848186379264e+01,    -1.3245370545102087e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.8049657922545325e+01,     1.0369047334159101e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4225590505862602e+01,     1.6939089081950424e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3840308699763071e+00,     8.3209240537241676e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5639625345476606e+01,    -1.3238189954926056e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5468832865460708e+01,     5.1038429009709616e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.4399948277549857e+01,     1.3210495199049431e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5834066882970241e+01,     2.5226276661965348e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -1.8954728393459690e+00,     1.3828683800383864e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3448459966883060e+01,     1.6676510584842905e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.3622160055452550e+00,    -9.2080333685359983e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.8071658433144322e+01,     2.0491511620406334e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -2.8736911736387700e+00,     5.2555169724925408e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7688822068057339e+00,     1.8612028775069066e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0468129774887558e+01,     3.2542474508889359e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -4.6838966126128803e+01,    -1.2290061929171529e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.6577977507406082e+01,     2.3913394185094072e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2558019044750026e+01,     1.5316284143437901e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.1610796731398400e+01,     2.4968442247595878e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.8862239107351712e+01,    -3.4469963570183609e+01,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.9913018519689629e+01,     2.3261187073015564e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0146282093650422e+01,     1.5496905913641131e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -4.6693402483688921e+01,     2.2812174085526480e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.7980480535935598e+01,     5.7601041542549356e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7558072924620456e+00,     7.0617669732768754e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2706862032857633e+02,     8.3135017122930967e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2751845400764186e+01,     8.3364587955227325e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1202255355884120e+01,     1.4284061577107008e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4059328630529208e+01,    -1.6023041789758605e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -3.4298863222450947e+01,    -7.8934248746247176e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7512961874656071e+01,     1.8851215357465262e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.8224473124808377e+01,     9.9843700610555970e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.0746026789172586e+01,     8.5662792399174159e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.5230056580852196e+00,    -7.7702736175966902e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1810080426593899e+02,     2.2128607621868439e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.2944218237644428e+01,     7.2709450487959039e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1316350053592537e+01,     3.2892879074088094e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.0679992598969310e+01,     1.1466300226568219e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3566372572946094e+01,    -8.5156062120636830e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5143937276229531e+01,     1.7854479442266573e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.7480788000250293e+01,     3.1731865108464990e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.9264869419278256e+01,     1.3817867089127009e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0196610214537124e+01,     1.7970664998252560e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3961095948195009e+01,    -2.1504400430126793e+00,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0009152810536872e+01,     7.1479395728338741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8271692536073232e+01,     7.4304103716135742e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8537442392607232e+01,     7.9068200597328978e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.9272170398975526e-01,     4.7319841203151491e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0593172980743827e+01,     3.3682049518570537e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    2.8997116960470901e+00,    -2.3410226507790242e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3584773833937511e+00,     7.7746828447669495e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5769552348232324e+01,     5.6611571806035154e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -5.1468413909887545e-01,     1.6153012931571477e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.3137761235696050e+00,     1.3926899577652881e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4426179426847451e+01,     3.0719719557865541e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    8.4957521376684042e+00,     1.6594104470780221e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9199812932487418e+01,     1.1241432280198806e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3591024317015325e+01,     1.4923306860476796e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2584082835990552e+01,     7.7577621639593572e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4110302202874770e+01,     1.6598513697950479e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6835391849962633e+01,     9.5902376269721317e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1708977447506523e+00,     9.9105833856825325e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2077722272556606e+01,     3.3601368752461945e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.1293339543353748e+02,     8.3438811369538053e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.3613348513821485e+01,     2.2777073309127007e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0800144734408379e+00,     7.2051844768578021e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.1475889611739085e+01,     3.1090439612221187e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.5716703743615014e+01,     7.2050274714974548e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0204543004560698e+00,    -1.8601232988345146e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4659538086889455e+01,     8.3309993381155855e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1044396380821080e+01,     3.7748816121288510e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.8784143860640860e+01,     1.5591205690262541e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.5418711346578213e+01,     8.3553884153230938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2091796361347999e+01,    -3.7237734767748861e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8450445781997065e+01,     6.6049429500688329e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.3349555502052496e+01,     7.7405510574338941e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3042743798888818e+01,    -8.3029522894753372e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.8729370129120909e+01,     3.3297492701728137e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.5360136183397834e+00,     7.6731538775148256e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1245916325343597e+00,     5.0938534948691595e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9658932551742659e+01,     2.5810813413453929e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.2957157625860290e+01,     2.2340385959139363e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.2543319459602760e+00,     7.2192515739766968e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9731117362408632e+00,    -8.3631841975454881e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.8173553354644483e+00,     3.1067459636507727e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.0445343198605336e+01,     3.2669537432640664e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0786026832522104e+01,     3.7095925372442041e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.0013970195331032e+01,    -1.6820531649110053e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {   -2.6131331508708318e+00,     2.0761059384516506e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    4.4946986571597414e+00,     7.8385571585907528e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8935867599151450e+00,     1.5791158816532617e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9607968529803919e+01,     8.4017622728201422e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4275573964987540e+00,     8.9040393881967120e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0141334287822973e+01,     2.3883897234082557e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.5056290710031210e+01,     4.0187622705555026e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -1.1864359374077473e+01,     6.1967108666270906e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.7624527739084757e+00,    -2.5176436069850621e+04,     6.6345003052078436e-06,    -2.0362436405988305e-07,     1.1202589729398046e-09, } , 
      {   -3.4653998968093276e+01,     8.6394461968867581e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.6650113510264465e+01,     3.7676339745139188e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5545477827419607e+01,     4.0735748197272267e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    2.8387780348085499e+01,     3.0440280314110452e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -1.9433626756218256e+01,     7.8812189499007382e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -7.7712105154501714e+00,     2.5636923764048137e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2154513782606671e+00,     1.8826607458410268e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3117165441886307e+00,     8.8583148961274674e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5250358609214809e+01,     2.5301542982551211e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0875646267472035e+01,     1.0633857621309095e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.7996578238038268e+01,     2.3809088907575086e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.3848152479094608e+01,    -1.0073928768875510e+04,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.5522642151650334e-01,    -1.1221067258341625e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.0097737672653835e+01,     8.2668716476106274e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.9465307896085466e+00,     2.7703482817590167e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2309504122311580e+01,    -8.3847642916770437e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.0222656119219179e+01,     2.4304797591613613e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.7319527622217485e+00,     1.1616976023910679e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6496310582443162e+01,     2.2728431221929881e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0066082751721328e+01,     2.4030353006238820e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.2230502453843055e+00,     8.1311421683851058e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3528090182059689e+01,    -8.1088239204105721e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2533705183818942e+01,     1.5589659675736830e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2523563404742326e+00,     5.3265102779055314e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2307300778151664e+01,     1.2547818013648885e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.5520293087743280e+00,     1.4422092256273936e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -6.7072093849674665e+00,    -1.0216016006711925e+02,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.1978474324502969e+01,     2.2281963859520872e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9594550320119247e+01,     2.4499424667276016e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3859049729753000e+01,    -1.1071503926960488e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9442950689133593e+01,     4.6592214453541790e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    3.0365437604442139e+01,     4.7349870459517500e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    7.8666115824491500e+00,     7.5606642177003641e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.9010605066690324e+01,     8.4749357911071529e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.4523326115872521e+01,    -8.3302253760721487e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9612510084831335e+01,     4.0741007861904633e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {   -7.1779266682921616e+01,    -2.5281175654869003e+01,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.3814995496569431e+01,     2.3871239999379221e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.5115249297004933e+01,     1.6437828717893644e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.3016388873443177e+00,     1.5997908409547228e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.4969289972564823e-01,    -1.1764330767627789e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2923047954531736e+01,     4.9675500620166702e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.7173009277990296e+01,     8.3288500165927180e+03,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {   -2.1884118378763901e+01,    -8.3285328326541839e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -3.9900642871615965e+01,    -2.5563675874982800e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9587634156576080e+01,     8.7639579746221843e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.9086082510358668e+00,    -1.1668863110094205e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0445239300987279e+01,     1.7515395183815360e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2249971914691537e+01,    -1.6362866044213897e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.6389744184209807e+00,     2.5802601416674341e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.1749175779013544e+01,     2.1245770545573148e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.2170161117809979e+00,     9.3844508325921342e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3301345809422216e+01,     8.2726185575698528e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -9.9455463762053057e+01,     6.5877907973298770e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1652114103737176e+01,     1.4127526535910440e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2153351412452022e+01,    -5.7226597526949827e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -7.0609373353978775e+00,     2.4078917637557843e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.8557184689299120e+01,    -1.0071275252976022e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0689283524697673e+01,    -8.2789104889442915e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.5568017158418222e+01,     3.6194781185972432e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.4997516270408631e+00,     1.3851741408795084e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1543951197355330e+01,     7.8708232359301633e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.4164988861559920e+00,     2.5753634900546185e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4108344831559441e+00,     2.3571724749438828e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.3827095923930067e+01,     6.0504190674190595e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5075258871861150e+00,     5.6949386080586755e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.3498543566346775e+01,     8.4143547028620269e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.8787434465573170e+01,    -8.2430281463848751e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.6811358561313767e-01,    -1.5283130452149862e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.6259583599441134e+00,     7.1682838271597111e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    2.3490298326932056e+01,     1.7003504480286458e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.8961669204569958e+01,     2.7225162937885449e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.3146813979023069e+00,     8.0158714997365905e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    7.0259152705166183e-01,     4.9960335516401528e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2785692839905664e+01,     1.6728728470617967e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.9080165795317555e+01,     2.4083072723125249e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.8162249391790311e+01,     8.5946309623347024e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.5683955530160656e-01,    -8.3106577406827382e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {   -1.2257370893536386e+01,     1.5878163627712493e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.6964369849168666e+00,    -1.6821676159186463e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -8.2745770239265717e+00,     7.1184616608903598e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0369550820245255e+01,     1.6295014801912610e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    7.5341061742702102e+00,     1.7179040894403184e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.7493088560152056e+01,     1.6659660365605010e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.2079007571283064e+01,     3.6899920493197242e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9546920747241572e+00,     2.1187638884476547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7723738188157277e+01,     2.6185601545297857e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0210633431733527e+00,    -1.7313927070035405e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5425768739630826e+01,     1.0488742505386650e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.1582606134184452e+00,     7.1367773239327953e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6340101905394562e+01,     1.6655105332888790e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3283142064803609e+01,    -1.6527434545151323e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5939018185261048e+01,     3.8732923322330302e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.3362283380713427e+02,     5.0969847191045119e+00,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1688517236986556e+01,    -7.7315354347168841e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4288359495386334e+01,     1.4914595121993465e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3185989455252374e+00,     1.4980569997460721e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3306460787303187e+01,     2.2617460376830342e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.6349713503798288e+01,    -7.6382506897054609e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.7591364225599810e-01,     8.3270530829467498e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.9359394686184721e+01,    -2.5211042850746329e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.0422247639468623e+00,     8.9568505926117796e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.8466775429323299e+00,     2.1823425203616316e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1022427008872882e+01,     4.1836085908045963e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    5.6141682626225728e+00,     2.2508911301569598e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -6.2741511710356619e+00,     2.5082212508355671e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.6962236526573307e+00,     1.6843460168783531e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.0222537957907800e+00,     8.3936512555936879e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4986418119102519e+01,     2.3137199935137320e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.6068426763854006e+01,     8.3127970803769458e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8872985458096416e+01,     2.4098268177107213e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.2955649050761558e+01,     2.3214272474846985e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    3.2304500276875295e+01,     3.0128614375038916e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -2.6401188237595228e+01,     1.1016723400685702e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5637505387873780e+01,     1.5065069814145067e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.4881121307280576e+01,    -6.3171859102853659e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.2318369276393977e+00,     6.0317467004563005e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1134405576154720e+01,     6.1124030350589624e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0664849409936224e+01,     1.6217462540900488e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.8602952343248404e+01,     5.1556732718819840e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.0292222604789288e+01,     1.9502943464423563e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4170887463646800e+00,    -1.7195518934862335e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.8132742661559028e+01,     1.5527376409432312e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0988944455551213e+00,     4.8559370472004193e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6814405009998829e+01,    -2.6595479722640448e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.0409017339624736e+01,     7.6758171711131063e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.3712953070170997e+01,     8.3824117332707719e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.1846608407906530e+01,     3.0021658872027674e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.2507288513209929e+01,     7.2866927923103540e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1880061616566362e+01,     1.6048202999797362e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3714937347246462e+01,    -1.0241362132772489e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    8.3559071282283313e+00,     1.5471766614400287e+04,     1.2186470119331788e-04,     2.8116134330025281e-07,    -1.3855538673797537e-09, } , 
      {   -1.7505170962126243e+01,     1.4300820084308180e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.4345880439418465e+01,    -1.9524923235976771e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.8537348754075289e+01,     2.6880319760622515e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2791219403728864e+01,     1.5629495338909237e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.7206749418617989e+01,     3.1062966674831347e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.1191519366894176e-01,    -3.5503066384351541e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.2347274969170421e+00,     3.2426607285772592e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    3.1610327030742596e+01,    -1.5233820129573924e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3070125076966562e+01,     3.3157031344153365e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.0685540271834110e+01,     3.4046684554853258e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -2.9816337552169234e+01,     8.3271574731747496e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8778098570855346e+00,     7.1243340062766783e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5021065382251036e+01,     1.4444125238940689e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -1.6892554267891072e+01,     7.7009361291103924e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8593148571162828e+01,    -9.3636901800927299e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    9.6023052812090341e+01,     1.3784276117382715e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.1933450834231152e+01,     8.3445857688699543e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0712530046885576e+01,     8.3230015965078674e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.8387413818347909e+01,     7.7898835318585952e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.9264908680704721e+01,     1.6029081977752081e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.2541286116843686e+01,     7.2120985176888662e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -3.0728416162998577e+00,     2.2126432027510895e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0352227202029368e+01,     1.6720661227713579e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.0385823845899967e+01,     1.7081548852761196e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    9.1698640447549984e+00,     5.7914125744367755e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1150484777427152e+01,     1.7844819586786078e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2859998294805647e+01,     8.3343812527390328e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.9002413624304836e+01,     2.2732544281730661e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6898455418690261e+01,     8.2381830840936909e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7682752378226098e+01,     6.3739046146350902e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    4.5421699833736904e+00,    -1.0032948707978203e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.6811883980945993e+01,     2.8556530496266965e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    6.3689748068791330e+00,    -2.0690068194229670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1902747778417877e+01,     5.0010521631228648e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -3.2518964291295260e+00,     4.2313366179175240e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2019667520103557e+01,     2.3786748838767729e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.4404636260602729e+01,     1.5559615036629360e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -7.1583981204696379e+00,     8.7588909025804951e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.1314814626363949e+00,     1.1155948210178826e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5784864729321136e+01,     3.2205068171486291e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.7069458792773020e+01,     3.7455726366637773e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8914188830842122e+01,     8.0509535137391267e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.1618446420951827e+00,    -1.9551154423631333e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4560283626637005e+01,     7.5603377028786213e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6062762564941778e+01,     1.6655079914308102e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.0388721053352750e+01,     2.2597909863706565e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0082851393291971e+01,     2.2032949596416809e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.4680892330018295e+01,     6.0941653977266633e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.6915129205512983e+01,     1.7444931992200384e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8024782972540816e-01,     5.8959726221259757e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0618645417742187e+01,     1.8718048439383620e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9007789435953740e+01,     2.7160728588071586e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    7.2791719634385679e+00,     3.2187517900810708e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -1.9826968862079791e+01,     1.1238020876652718e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.6283239381159945e+01,     2.2033337738210343e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.4890569028850585e+01,     6.9300743841912299e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.5812395512400421e+01,     2.2708028073831059e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6985705227366111e+01,     1.1392677088839042e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0555536145985670e+01,     8.3551419052384372e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5248805686545644e+01,     3.8197399736393869e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.6641704930191381e+01,     4.7778939845614084e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    2.3792321066771990e+01,     3.2165192250219301e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.8516954735981157e+00,     1.3533878607959896e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3568327861477425e+01,     8.7654832278734011e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.4605011633945971e+01,     7.1321937787763090e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0539111169663521e+01,    -7.2253023876812622e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.9803658776074410e+01,     8.3299920111708252e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -2.4514767876848016e+01,    -8.3273908380760749e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.5604171926531706e+01,     8.3650921569851234e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8203347604628544e+01,     7.8217518878929131e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0287237522164205e+01,     3.7471107337350645e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4409948309306614e+01,     6.5868598720421196e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.7510611455037690e-01,     1.9907217044099156e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2178763368300288e+01,     6.6965634876237045e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.2331641790255894e+01,     3.3353462898691403e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.7242628194636037e+01,     1.5543375917656313e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.5922871035053646e+01,     3.7853875385616884e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6215297332851701e+00,     1.5162851231487075e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.1739637869131236e+01,     8.3279154779332566e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.6037508075882485e+00,     8.3294673713136435e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2672002059565916e+01,    -1.7354579777415920e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5364931234191715e+01,     1.4357484741261775e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.0442079020037838e+00,     6.6724963292402426e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.8088041890274313e+00,    -2.0246999778017170e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1609080158962620e+01,     2.3249171604002455e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.9719294689530578e+01,     1.4415007819518423e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {    2.0971378459862315e+01,     7.9935458491451864e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.3445464295508989e+01,     3.1098626491161856e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.3813096752271054e+01,     1.2150680678667351e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.7191987430388913e+00,    -8.1719353452495216e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2976010006512627e+01,     1.5542132662307280e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5372630491772092e+01,     1.7140704895979306e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.1098677987166539e+01,     7.1475872568757986e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6335810629553023e+01,     3.9349765654773175e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.4411446494709022e+01,     2.4833255692272360e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.9206045806833469e+01,     1.6588204416694993e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5514681579008617e+01,     2.5263047618678494e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3985241027814755e+01,     1.6076144307327319e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1285378791262602e+01,     7.2290191984964576e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0590434424162478e+01,     8.4003960755033168e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.9815169277389826e+00,     1.5496975251783158e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    8.2577894754187504e+00,     8.7348133395687400e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.7968380318308142e+01,     1.6656319187821951e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9398007841420231e+01,     1.3797203572870114e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.0533689139622659e+01,     1.6658446510671849e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5889176532270682e+01,     7.2135641925780692e+03,    -5.2741442090551219e-04,    -5.5511302234872569e-07,     2.5427507946832937e-09, } , 
      {    4.5082671537609009e+00,     6.0840044522006156e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3325811871695372e+01,     3.3507042167464439e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -2.0871288029124411e+01,     7.3554636601530092e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.9994678588960962e+01,     1.5751951439760023e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -1.9009008355989867e+01,    -8.4423263130321466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.7184032347070390e+00,     7.7665071321023770e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1019536405642697e+01,     1.5752656071676172e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    3.1054672166811663e+00,     7.6687446503445672e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8574591400482188e+00,     7.3297495118427785e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1978656113207684e+01,     1.7843396895852326e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.8720990912236681e+00,     1.5566647676322718e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7835870846759949e+00,     6.3816202600992228e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.4519920046463270e+01,     1.6814911488361307e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6222462791894703e+00,     2.7905744890951974e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.6828617955488401e+00,     5.2761710130105239e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.9908430763682759e+01,     6.1006818587881298e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.9082822413775368e+01,     1.6153788878261668e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.2931584600466262e+01,     1.5056080999804157e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.3231847537156849e+00,     1.2573265155579627e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.4485229543866094e+00,     4.2679698452091707e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.6487609658197556e+01,     7.4898745112636448e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.8831662687689047e+01,    -7.0085750412945345e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.1612356316980588e+00,     1.5330877901137030e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5059854153251408e+01,     7.2621228754527692e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1414539485612094e+01,     8.3751015662355076e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5592474934473621e+01,     3.8302201277638116e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -3.6489002559533873e+01,     1.5779912732892031e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.1222317012864576e+00,     3.1079475105520836e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    5.6483381607869338e+00,     7.5698719570935027e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.6975030106771314e+01,     6.6342066805935474e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.9946384679296720e+00,     2.7150641604543407e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.6032988265778165e+01,    -1.2282367632401292e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3553366442795696e+01,     1.5595713911454075e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.1415456285714560e+00,     1.2590102314233489e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6033112611263054e+01,     2.2111588889487004e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -1.4230103341084988e+01,     4.0018974805270773e+00,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.3555025448455595e+01,     1.5282289375863140e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9732589110809084e+01,     3.1101508088187584e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.1298205583518062e+00,     1.6149401180691584e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.3572045338574075e+01,     6.8144829400662238e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.3357091395793006e+01,     1.3173811760567014e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.2386547643375756e+01,     2.3367100532183886e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -6.9521869303264538e+01,     8.0499871113380650e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4515631379672278e+01,     2.4300678790161226e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.5192006292601626e+01,     9.0065129211234525e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.3759647975182629e+01,     8.3303297663001504e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1204741090939873e+01,     2.5622608383715051e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0668227812710082e+01,     8.8015658552699715e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1501476874314459e+01,     3.8296941613005751e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.5364334252514663e+01,    -4.8560962875268609e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.9727537245079123e+00,    -2.9568914984460312e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.2117308488476723e+01,     1.4301681148179139e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6548050313207057e+01,     2.3258094462697816e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1115782281097317e+01,     4.0515710652653877e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    8.5100632224898998e-01,     6.5665676105567709e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6035613519816451e+01,     1.5507861844925374e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8727195871642142e+00,     1.6674695711981825e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9587784277493775e+01,     7.6005248794521995e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.1341242292750493e+01,     9.5220282162478579e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3057675975748193e+00,     4.3389631506221622e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1595521616964433e+01,     2.4153132995830947e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {   -2.0112950885055163e+00,     6.1349966896464366e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7717676393310093e+01,     1.9537398336725528e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.8535470635820452e+01,     9.3228630586967847e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5407924653104303e+01,     2.5491522983685918e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.9429193689105126e+01,     4.6056606930919170e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2751667714098334e+00,    -4.2768300007497146e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5885180543624273e+01,     7.7871096289901961e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.1646366696872423e+01,    -4.2745288008083053e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0003955275071494e+01,     2.4004553936930679e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.8967782779017352e+02,     8.3233134530958141e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.9571259625213058e+00,     3.0155508226257325e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7259019712422212e+01,     1.0560155597942072e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1132242225643196e+00,    -3.2708793265606669e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2086163217466236e+01,     1.5558132170531280e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.7227191176252887e+00,     2.2739544158574474e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -5.6445000348657075e+01,     1.2781906728222684e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3242895493402195e+01,     4.0686562918192576e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.0811462754279358e+01,     6.5611721165118688e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4613111324892941e+01,     2.5397128927504051e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.5832369498363494e+00,     1.4360438125079751e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1273374518175288e+01,     7.1569749666947646e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -9.7621696349609863e+01,     8.5846615111196388e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8109714933054420e+01,     1.7285683720741719e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1974442649046463e+01,     1.6636764286735925e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4418602961739921e+00,     2.9340271235711531e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.3831057240122682e+01,     8.2760746433703353e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1772709008480309e+01,     8.4809873167049191e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9311227212883463e+01,     3.2184137631004698e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.8947074280133059e+01,    -7.7867962561305740e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.1172978187336557e+00,     6.6017547885367985e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0181960358387547e+01,     3.8012764073077904e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3039281705122669e+01,     1.5542613502708782e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.7578104031471952e+01,     2.5211165767895080e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -4.2330579882831145e+01,     1.0745018964244129e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0907839479807887e+01,    -9.7375663254623687e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9349766086035444e+01,     3.2815326813075968e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    7.1793635055032254e+00,     1.5542895077254807e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.9103604959079625e+00,     1.5447153085513812e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.3738662734596637e+01,     1.6082795572848681e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6396691498977503e+01,     1.6381755850840051e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.2694332733025789e+01,     5.2737831800353724e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8272102463710425e+01,     1.6866120269343301e+04,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.1760481490081972e+01,    -1.6867744360623103e+04,     2.8693923882050119e-04,     5.7784942651445594e-08,    -5.6868644794148498e-11, } , 
      {    1.2140564975794945e+01,     1.6885408639272358e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.5388863531639259e+01,     3.8807222327519346e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    7.5240805634066268e+00,     9.0202773881356716e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1311703314602141e+02,     1.6171159926601178e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.1861154006336463e+00,    -4.9853196791235678e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.6538357076352472e+00,    -1.6736943314900691e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.2668909659478146e+01,     2.8030383071288546e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.5568032431197210e+01,     1.8088799655218063e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9853318817481465e+01,     1.0156725435004850e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9274934145299518e+01,    -5.9061312566516672e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.6879835967037224e+01,     3.2935822481082476e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    6.8739169162951228e+00,     1.6678001411757876e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.9570768508819853e+01,     8.5986648925877835e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3521753967816476e+01,     2.2785203088750401e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0192485566376227e+01,     3.7688184541083981e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.1171132269542937e+01,     4.0107769244601499e+04,     1.1038930864785152e-04,     5.5286503827215703e-07,    -2.9388435721497847e-09, } , 
      {    3.9553582278214243e+00,     9.3727188604028961e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3291875045634637e+01,    -2.2822484635922535e+00,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -3.6468197987372378e+01,     9.8997780569120459e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1582418864334610e+01,    -1.8788892806938775e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.2883248631239468e+01,     1.2974167019613280e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1623641111894654e+00,    -7.2318447845895080e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1262451299408490e+01,     1.5360012513753782e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.2667107046084869e+00,     7.7570508184924802e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6711568180825900e+00,     1.6335355184966904e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.3033005436268070e+01,     3.0950521845981420e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2919349374051913e+01,     7.0870325876704765e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6507469220959557e+01,     2.6547341441385444e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8632023272344874e+01,     2.3731726092727702e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.1423809833375515e+01,     2.3471865789313124e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3762965150149730e+01,     1.7685200949685626e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8737881473800243e+01,     1.4523726935184673e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3058555204167156e+01,     2.9822783176360672e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0067423069935870e+01,     5.7875394214725929e+01,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    6.9485814286822984e+00,     1.3114843581711029e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3720617912304245e+01,     1.5606744013915350e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.6871199124027232e+00,     5.1277145483454910e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6143011678719425e+01,     2.6374686648685612e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    9.2305690239642413e+00,     1.5078740014875817e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.2410300709394832e+01,     2.4981007201828659e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.1230356151977428e+01,     2.2354979716838490e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.4416877981816771e+00,     3.7620319383522337e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5547903874318225e+01,     1.8562969120640424e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.5208331971277680e+01,     2.3227390386045252e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4750351690648472e+01,     2.3238207097302107e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.1423092702926098e+01,     2.4879656358145512e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5614853469681641e+01,     4.3670015001798256e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.7745431687532410e+01,     8.3158895452682464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7388237541513867e-01,     6.6698664969240590e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.6704667365247715e+01,     3.3534592549904119e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.2557080697187558e+01,     9.6008486230029666e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0218883983134845e+01,     7.5475805174950692e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2570787094886668e+01,     1.6267964675588832e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.5975286870633965e+01,     8.3813082058765649e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2312222551687672e+01,     2.5239394573163616e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.2991554106082898e+01,     7.9631538793211860e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8450339958806641e+00,     8.3920581464101706e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.2801741380585923e+01,    -1.7653260435731681e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.6364005450134712e+01,     8.4887269789052934e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0167144352713095e+01,     2.7737221061721262e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.9935368607002655e+00,    -3.8657753471160186e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.8199880715244511e+01,    -7.2386707644133385e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.7415731621730643e+01,     8.3114201628551691e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3520056317023350e+01,     3.2969710796267373e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0258398842385066e+01,    -4.8667329017763711e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4948671281347792e+01,     1.5397639174059352e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    7.6680544220706661e+00,     1.6684605321102925e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.0358077242699721e+00,     6.6478884301852477e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.5099307640018424e+01,     1.3171862876583678e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.5390687384956280e+01,     6.5861434221063373e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.3174627986966172e+01,     1.5380168041224010e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0365554129504819e+01,     8.5262406808617925e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.3164899716197649e+00,     7.6797652770669320e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0072983919483370e+01,    -1.6922434219716160e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.3610013789346183e+01,     8.3414933039786538e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.2598054968478305e+00,    -1.2598710260198345e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1652082882554257e+01,     1.6036553430770415e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.5376489475914425e+00,    -8.6123842356971636e-01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.4959202683058891e+01,    -3.6212105862994122e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9942556062417339e+01,     2.5557293960252315e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.9701969576665412e+00,     4.4878173090099364e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8979624326446867e+01,    -3.9795200195057987e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2180819919677681e+01,     6.5032719870430847e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7065435495517519e+01,     1.6918817200898202e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.2215949107317087e+01,     2.2189589174169328e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.7552982099435553e+01,     1.9702017973913124e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8462508982228243e+01,     2.2772236266479813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4987608632855913e+00,     1.6659685784185698e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4824534353824554e+01,     1.2778489590932615e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -8.5496979366915422e+00,     7.7559580338512205e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.1642405310388988e+01,     1.5551632678482338e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.3378549301740378e+01,     2.2182190012972664e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.2978569057862060e+01,     2.4148419059413391e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.2940831165373112e+01,     9.0133223422563988e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.7347460605404184e+00,    -1.7068277937470245e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5030515029912166e+01,     1.0055418233593380e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4119462918595964e+00,     1.3140857638266907e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0902385204144677e+02,     2.2781090028949620e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.0444601221342609e+01,     3.0774501497203219e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2592175721714568e+01,    -1.2765595240192511e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.6011002278947837e+00,     6.4146178035882235e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4887045709622317e+01,     7.2146844930328607e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.1735449621340290e+01,     7.6385640625650831e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.5484177415083185e+00,    -8.9635579652681281e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    8.1696194718502468e+01,    -1.3795349551877489e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0008190981856163e+01,     8.1955232681379875e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.8984498910089265e+00,     5.8875836404155270e+03,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.6930888964757223e+01,     1.1617475625309513e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6312895741191305e+01,    -7.9230512165124755e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.1903602774910598e+02,    -1.0569478742247043e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.5314836811575958e+01,     7.6996792076617576e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2126643013873057e+02,     1.4912411187919177e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6495276167856467e+01,     1.7052705018881297e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2515335750604567e+01,     3.2035843829289130e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -1.5352852122163274e+01,     2.8560984319098134e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.3969893216160916e+00,     8.8304422527356346e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.1565974319812362e-01,    -2.2161392073319439e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5463463914908937e+00,     1.4303059800698300e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3359634134681617e+01,     2.0352305930824223e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0722177687663736e+01,     2.3867036487434987e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.3933851595982816e+00,     8.0830336083235243e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.1619118717995001e+00,     1.3183937165137165e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0436625416401029e+00,     7.7675743753815714e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.3354728010076755e+01,     2.6440032984973761e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.2041339762021209e+01,     2.4080897128767705e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -1.2356010420102141e+01,    -8.3260768518224213e+03,    -7.7245056061153879e-04,    -9.1504412911767354e-07,     4.3024790130065763e-09, } , 
      {    1.1406050030797935e+02,     2.4081093231915387e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.1302472773543329e+01,     2.4230685256843073e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.5612022320824327e+01,     9.3308751550386332e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9387889418274877e+00,    -1.8922688384180710e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -4.1276044737933226e-01,     9.0929521460522228e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.9009504556084256e+00,     1.1157320866593113e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8398373644623813e+00,    -8.6779415607594747e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.2971168125342949e+00,     4.8624960027995539e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.9164776150062375e+00,    -8.6039179199475427e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.7974079930854998e+00,     3.7668962660334983e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.0598578663098955e+01,    -6.9587606978734675e+02,     5.9165273136225077e-05,     7.3424547190358117e-08,    -4.0360254138687761e-10, } , 
      {    2.0695200705702092e+01,     8.7795877885311620e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.7395912173648878e+01,    -1.5514402567564508e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5273886477749093e+01,     3.3837135089116957e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    7.0656668470822304e+00,     6.0847157976674916e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8968787734677584e+00,     1.6861502609945332e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6721545426138857e+01,     2.0914554216159213e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.1675593140834062e+00,    -1.0175238840248017e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.0718410199692187e+01,    -8.0493053433391578e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1608644872824687e+01,     2.2004556643409320e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -4.3417488793126921e+01,     2.4889171473308859e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.0787808117557462e+00,     4.0856428513288956e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0219115562463145e+01,     1.5462900829028260e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.2671589803012919e+00,     1.3435662830690877e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2202885002448067e+01,    -8.3936244472175858e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -9.2517901851396985e+01,     1.8874635879378020e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7775404722481849e+01,     1.3360411769620163e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9383663611471842e+01,     2.7331082180830010e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.8266772609257195e+01,     2.1659461592840704e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.0714378513443535e+01,     3.3337307603626046e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.4652040493072736e+01,     2.4691827932296532e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.3498163433602528e+01,     3.3994389722154934e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0160458019477938e+01,     3.0208296478395056e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8240656856478111e+01,     1.3220711099220440e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.4455896813687659e+00,    -5.4695255442409314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4271224376247186e+01,     2.3359385605237574e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0724329993269606e+01,     8.3512315947583047e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5179073321791019e+01,     3.3472500052834235e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -9.2683296723550939e+01,     2.4491425646747775e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3368037063849677e+01,     3.8339386347219843e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.0808378567273692e+01,    -8.2553969192690492e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.2344906635979835e+02,     2.3869410197588168e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.4679690879240866e+01,     2.5507963943191055e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.3262228829514132e+01,     2.6091449175097805e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.6245464424201232e+00,     2.5280320615444165e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.4021493602649134e+01,     1.3494165018373815e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.3071998240016242e+01,     7.8267340751772390e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.4529929227104851e+01,     2.4266767884239645e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1480979690247587e+01,     1.5186919335465063e+00,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.4575960919065864e+01,     3.1940286311553031e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.6600263467647629e+01,     7.1868329032952652e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.5087893949805041e+01,     1.6791202417039214e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0620433103712287e+01,     7.2134412376838291e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3453023365566736e+01,    -8.5217324596702601e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -6.0950170399282415e+00,     5.7200382617459752e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.6097365876532095e+01,     6.5853671385192665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6347709913964481e-01,     6.7308703962352465e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.0307777768458420e+01,     1.5629504814648148e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -9.7105354488152543e+01,     1.4916482221953325e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8477479010284981e+00,     1.6167248583214201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3571309939370154e+01,     1.4940183854552504e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    1.0258721842505665e+01,    -6.6864796809983727e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1767063099475610e+01,     7.6982416013560860e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.7365284993525243e+01,     5.6406594485611986e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.9655687855352681e+01,     2.5317807423447505e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3265862986197803e-01,    -1.2930264298624306e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.4802634772635429e+01,     2.3873481231622322e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.5689029253906046e+00,     2.4726223446194683e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.7840952720457427e+01,    -1.3376845362727611e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.9886031198373511e+00,     7.3307284217478355e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0615107189689521e+00,     1.6603702406568835e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3516049305683786e+01,     2.0552763478950487e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4162126409737660e+01,     2.5960422658266652e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8418056121630897e+01,     3.2106837134702120e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.2472379464514542e+00,     4.8766604976173394e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1481520894044227e+01,     3.9428289353478613e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0959525156377481e+01,     8.9573922559314342e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.5165303226567168e+00,    -9.0305953390220675e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.5822292168846801e+01,     8.5996312949888470e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9423242446995694e+01,     2.2753014385730814e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.1492653453589252e+00,    -1.8374581843941298e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.0420227777121672e+01,     6.5351131062100740e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    6.2307760793541419e+01,     6.7327659987131483e-01,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    8.8227594450271553e+01,     2.5180081711318220e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    7.9206178482834781e+00,     1.7572092114444274e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.5092154458160348e+01,    -8.0284116086068843e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.3817641512169452e+01,     3.8259756543424024e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    6.8746253783087070e+00,     1.4412695876824066e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1703111033661163e+01,     8.3267244505719773e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5008807927576342e+00,     8.3120888430202922e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2710461977537584e+01,     8.3339510892558174e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4254250840053970e+00,     8.3234317599910828e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2382088153027198e+00,     2.3649277010450955e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.0263982229694953e+01,     4.1379304853051974e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.2722836084847566e+00,     1.2242646353908706e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6714072898779136e+01,    -1.6876816143732685e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6397329990579188e+01,     1.7725112069858580e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    3.2445564768805689e+01,    -8.5152761741614468e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.9242616615629426e+01,     2.0132027333056179e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5397565130548330e+00,    -8.4154324735508926e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.4118853742062591e+01,    -8.7761638993461638e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0495053512663247e+01,     7.7751799146002222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.2666651347371207e+00,     1.3924269745336698e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.3874782056764211e+01,     2.0339188019625952e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6881138480492401e+00,    -7.0468690876452479e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0413967214165762e+01,     2.5125759684754921e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.1785806630747722e+00,     3.8346084924618885e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3850385085073111e+01,     8.3306583986749229e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4646017248837138e+01,     1.5325501358569947e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    2.1406119010247878e+01,     1.9047754314199097e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3590068509147404e+01,     2.3350179851376291e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {   -5.3947014726966263e+00,    -3.7014820371764526e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2623758060192817e+01,     3.1256360845502291e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.4367894679892672e+01,     1.9486081898109936e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2350005045473686e+01,     2.3316761267288879e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -3.5478542351412656e+01,     8.5346714640186074e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3743409931564651e+01,     1.6711063291924966e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.0303371577061540e+01,    -1.0629051038045763e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3749373593059865e+01,    -9.9011678152967943e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.3566951605005798e+01,     1.6660364997521159e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.1038463370836574e+01,     1.6757899826457331e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.6059934103286281e+01,     1.7867098680684558e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -3.4719985298262102e+01,     1.6654400700972641e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.9395517121644872e-01,     2.2772549639339442e+04,    -5.3404892121072009e-04,    -3.5148865828884261e-07,     1.4224918217434891e-09, } , 
      {    2.0841903505913947e+01,     1.1386118133239906e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4341040547707145e+01,     3.8821940835945090e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.6612583727165823e+01,     7.5489034424699821e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.0588815937545844e+02,     8.7205546284349111e-01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8415250072027465e+01,     8.4188629240535611e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -3.4853590062788840e+01,     5.5040300807534425e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1445870503261474e+01,     1.6715258243461623e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.8741634524522993e+01,     8.1356503895766400e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.2481660274648352e+00,    -1.9132237849917012e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.6877893476632956e+01,     2.2760619924949468e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7138903882509275e+01,     2.4970326530355898e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.0674464561034487e+00,     7.2062954944590610e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1665296567781226e+01,     2.3480725944853668e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.7833345251175814e+00,     2.5665024033018617e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.7819688755742057e+01,     2.1179081705477747e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3453935100587771e+01,     3.2828797040232312e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.6180354569310367e+01,     5.5110763999149401e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8500759825728128e+00,    -8.3378802125387665e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    9.3115506960046623e+00,     7.2075113999662653e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3060414586981985e+01,     5.5824756690301328e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5051969510217713e+00,    -1.4274674356297851e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1691664567783862e+01,     8.0174908899354705e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5648303786957252e-02,     1.2747836668573918e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.4086580320579166e+01,     1.6175992907284930e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6493483158316486e+00,     6.2713464775240084e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8043738538722849e+01,     2.5410240277384641e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.1951651807460664e+00,     7.7739714993000734e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6504262134074267e+01,    -1.7981913190206382e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -1.7061612678732740e+00,     5.0474528289960126e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.0630569720051213e+01,     1.4372768800530348e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3933261816349281e+00,     2.3388253245482141e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.8263771387243690e+01,     2.3311501602656506e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0425876220385335e+01,     7.9820900722814313e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.7866743031536991e+00,    -8.4166054108440349e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.3943099543896533e+00,     1.5523576095964674e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    4.4505854225578219e+01,    -1.7981208558290236e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    3.6258103968255078e+01,     3.8874909933156283e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.6590479234836653e+01,     2.4103335249148899e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.4929255579044849e+00,     7.7621883570755799e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4890041592045083e+01,     7.8953300718761675e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.2546328349896330e+00,    -7.9448781943236008e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.0295724808401742e+01,     6.4564068593826287e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.3157199239673588e+01,     2.8696223601709065e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8672105219486582e+01,     2.3035558309496264e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2183718431469948e+01,     6.0763750747097856e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0034557852128696e+01,     3.3411777830599760e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1549437346264341e+01,    -1.3106715137300530e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.5503973300549081e+00,     2.9865362365057706e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0757703431127588e+02,     8.3290571676829786e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2171292137281887e+02,     8.3283256815639197e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6407329339002303e+01,     8.9258474145300170e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.2765127294778399e+01,     1.0278048529573936e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6865083509067089e+00,     6.7926559622550985e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9078257847772463e+01,     3.1964850523876143e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9803739489252575e+01,     8.6289712406400176e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.2042835607419214e+01,     8.3634392544796719e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    9.0935846205395645e+00,    -8.3036636349422133e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -7.8723415245744004e+00,    -4.5121166161022702e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6139781997674636e+01,     8.8451607611613799e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4557417835264516e+01,     1.5521178787250614e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1384924704404579e+01,     1.6077761870500763e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.3718049687036853e+01,     7.9618151560976239e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.7887023852619812e+01,     1.4523297562800943e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.0279147465223364e-01,     1.9184868493072997e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3506953863675392e+01,    -3.3307525070818974e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4536312086066648e+01,     2.0163404721829662e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0618461511060410e+01,     3.7842116125815082e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    7.8059726693990736e+00,    -8.2816223814974855e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.9765568481078343e+01,     1.6146478711226404e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6445188736141017e+01,     6.9979544440229101e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.3423161955295114e+01,     7.7997630784011562e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.3210612706527396e+00,     2.5301561934029032e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    7.1247901735058115e+00,     1.7807348479448727e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.6117062090772016e+00,    -1.0623708075207905e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.9074009634319850e+01,     1.1508313283271838e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4453513189630236e+00,     3.5315281773409856e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0901282433484319e+01,     1.5495854951328369e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    1.4786326786655613e+01,     1.1708145797519906e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4952767330483056e+01,     2.9039501099337504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1125492842945238e+01,     1.6570632324580547e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.1866172679384187e+01,     7.0258433075998946e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.9168183918114202e+01,     3.6885221920890413e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.3222770757116621e+01,     1.6963662770012448e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.8742639158406519e+01,     2.3871651429831269e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.0083169667469797e+01,     2.3850827152094269e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.9372060395784523e+00,     8.3213686796044694e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2963262425088782e+01,     2.3736415813625604e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4905132793751747e+01,     1.9977318645071944e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1731701366908505e+00,    -6.6325247405493553e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -8.4170739717546752e+00,     2.3220431907613538e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.0448196983954414e+00,    -1.0755758462126232e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -9.8403904662157728e+00,     1.9290222210683660e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2927770451750719e+01,     2.4455982466093828e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.7122081639500735e+01,     7.5137990900399091e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6628642017074871e+00,     2.4277584595496497e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.6579606209593731e+01,     1.0247353292674052e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.8414236279288296e-01,    -8.3900156085825574e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.0610493510022017e+01,     1.7342013766879849e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5929845235794323e+01,     1.5624109320603244e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1717829191824510e+01,     2.8822983966273649e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.2779902795152003e+00,    -1.7196672920677658e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    5.2235238551694136e+00,     4.3078115554831847e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6127806955427218e+01,     1.5361919179658964e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0741612448424460e+01,    -1.5757219253361384e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.2067507325874722e+01,     1.1203791139359633e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.0594900682428285e+00,     7.7423323100667340e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8906603458927790e+01,     1.0700304205530865e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.0042234668176562e-01,     2.4228048863209217e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.6388856971509931e+01,     2.5109291402407634e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0022608547701319e+02,     1.1468498381373925e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.9946978382755480e+00,    -1.6244578410179745e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.6091523658668880e+01,     1.8231029781238067e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8440075197247811e+01,     7.2218302362576278e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.8107670725965619e+00,    -8.5070624868446739e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    9.4207769329487885e+00,     1.4413407222290942e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5599257948810619e+01,    -1.7278970386582716e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    7.4205920698284435e+00,     1.6632799685445967e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.2918963412842949e+01,     7.6895671283210522e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3499033238663669e-01,     3.0457529627292817e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    8.5079355428381440e+00,    -3.4233543542516031e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.2017003885691295e+00,     2.6247898222839613e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.7226346373803274e+00,     3.9459421809291744e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.8928905039798735e+01,     7.7956500186003796e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5783271324757315e+00,     1.4134963721149412e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0939469921032060e+01,    -5.4144455227215076e+00,     4.6351172359218639e-04,     5.4853225840408113e-07,    -2.5876930229221479e-09, } , 
      {   -1.8734992168608294e+01,     8.3271015701489159e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.1906818703822864e+01,     7.8665489770751519e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.4698220837470028e+00,    -2.7347058283457782e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8858061341912816e+01,     2.3889479398545951e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -2.1118359428877699e+01,     1.6656254586598629e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.8731552874953913e+01,     6.5362413688583401e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.8763945703445330e+01,     1.5542944745692659e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -3.8545300546049532e+01,     1.5542563834270926e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -7.3574967592917844e+00,     3.1068235583197922e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    8.6060414834683439e+00,    -4.0876871320743840e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5655051994762907e+01,     9.0306221473981677e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.6194344540150311e+01,     3.2025335503348615e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.3876427536191898e+00,    -1.0640416576908228e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.6296328872994277e+00,     9.0591707185970627e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3069800242016164e+01,    -3.8815078311153175e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.4055203974413697e+00,    -1.7421239479957984e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3760507401541053e+01,     5.4968638706639504e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.7933436472144066e+00,     1.8433586488717279e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.8087731503288751e+00,    -1.6199927529740056e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -5.4668236985554586e-01,     4.1474473688265680e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4677223958343610e+01,     2.2161239478167918e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.5132849325972913e+01,     2.2327584079784159e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.2405059473171036e+01,     5.8921860402983475e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    2.5432768760577876e+01,     2.1606851525138314e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.1472493319314308e+01,     9.6036689110300213e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.2889622372025748e+01,    -1.6686233244444462e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {   -5.7996450765526305e+01,    -1.2688210948150654e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1012865984584955e+01,     7.2320965492990545e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {    3.4755196372386095e+01,     1.8123048812294008e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6514702650276568e+01,     9.5787476391154032e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0896960898021387e+01,     2.3399718498084811e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.7958521494275075e+01,    -3.2815025054097987e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2796753542477838e+01,     1.3664831411688980e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6313299323374288e+01,     1.7808771170382479e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.9058136898742273e+00,     7.2691742007913572e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.4681951688843910e+01,     1.3290955266319339e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.2180686286101931e+01,     3.6916388775544547e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.4416849581048705e+00,     2.8004438629730295e+00,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.3501298085650895e+00,     6.5857639752352952e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8357518534202470e+01,     2.8259110731041968e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4471635444564948e+01,     8.3342735890868498e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2902118997381136e+01,    -8.3231092601600467e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.4996318919739242e+01,     3.1069509071739594e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1062029385504328e+01,     8.1442787279623406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0660828171615806e+01,     1.5359282786025109e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.6819281695872029e+01,     2.2661257181462352e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.7998699658370072e+01,     2.2128411518720757e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.1407026718042275e+00,     7.7006074967356162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0353600759626662e+00,     2.0368038414823521e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.0023287224912796e+01,     4.1523784010841628e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.2402486471329695e+01,     8.3782053360779082e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.1481280586227435e+01,     3.2726273015346171e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -6.3258735265807431e-01,     7.7002354225206927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7443745965834751e+01,     8.6073957379088351e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.5488414875132115e+00,    -9.8411667678070080e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -4.4874914147414856e+01,     1.6458308339757339e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3235763375472295e+01,    -9.6579365216665446e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9795813307827341e+01,     1.4941107411520885e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3510911147853427e+01,     1.4854928127864836e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.1735080048760929e+01,     7.1894702258185007e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.8046596031194071e+01,     1.5539641044400483e+04,     2.4503613970602655e-04,     3.5993110676894790e-07,    -1.7597282183232826e-09, } , 
      {   -2.4111136180977866e+01,     1.5545867535563102e+04,    -3.7740249195369381e-04,    -2.3569566172990487e-07,     9.4368983027971148e-10, } , 
      {   -9.8708983257815515e+00,     3.8749488755161829e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.0821702725983943e+01,     1.6858295816600366e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {   -1.5572558225240552e+01,     2.9342220119694870e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -4.1433697825724913e+01,     2.3871057572811711e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5130660090859010e+01,     4.5477585894276686e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    7.0080284867025640e+01,     2.3871833856398778e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.8514549654982715e+00,     1.0866872517351480e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1488510497735005e+00,     1.6681966013047830e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5223683958446003e+01,     1.8464753343371405e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0632897222923127e+01,     2.4087554135940682e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.2210486068507054e+01,     3.2231315542743770e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.5272123027285041e+01,     6.4434400371120482e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4423189362576792e+01,     2.5851540301826128e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.5168217365257050e+01,     7.7756349006486198e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.0728174193697795e+00,     7.5864203187944386e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6818070208869663e+01,     1.2396944892834931e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9748144792112416e+01,     6.2829058100499242e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -1.1179704797953939e+01,     3.3315094330868575e+04,     7.6146203881163085e-04,     1.2535943707732736e-06,    -6.1799199931032542e-09, } , 
      {    3.9448919161817741e+01,     3.3314437066119026e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2467030174697744e+01,     1.5862958416255124e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.7722266629685407e+01,    -9.9521828632403594e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    9.8890641508900980e+00,     2.4375800372765520e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.5814727718431603e+00,    -9.7768223765101284e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4505039200554455e+01,     8.2625681320989788e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9230071566781071e+01,    -8.2845919586917116e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6503356833964578e+01,     1.5560787973922503e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    6.9903921478965652e+00,     8.9558355481147610e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.8474333982715855e+01,     1.6941356070898863e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6543390503376656e+01,     1.9312599800255055e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1832281402603051e+01,     2.6552169510081535e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2130226287442891e+00,    -2.4242128265673832e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.3294792723345203e+00,     4.8742450161376809e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.0835248119687648e+01,     1.9808373324642380e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.3753198919897741e+01,     3.4115113931920496e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4130615247267970e+01,     1.9798920774474616e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6395104577288642e+01,     7.2083730372427608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7951872342276074e+00,    -3.7080409927755863e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.9239974203815564e+01,     1.6441035342991669e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4492998230160776e+01,     1.6672101357672647e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0919980995763197e+01,     2.0011100072527101e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.8993139880348187e+00,     1.5577646735038215e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.2497935497794487e+01,     1.1151710134667868e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6222091879660699e+01,     8.2250202191495064e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.6099085980104892e+01,     8.3152211974671063e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5641177269568789e+01,     8.2998257699107344e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.1661292452510715e+01,     7.9557816816548730e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.2984509838967799e+00,     1.3026894117483438e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3895381270474493e+01,     1.2020833584956898e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.8810171907306980e+01,    -2.6413701055012280e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3061995540118335e+01,     2.3306416508026381e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0443260104891390e+01,     2.4358478034291682e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8374874409489674e+01,    -1.5448280372221498e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8188493663537088e+01,     3.0317515584033470e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0177917429326169e+02,     8.3294318823750618e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.7643287231718702e+01,     8.3279509668718383e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3036597242476995e+00,    -1.0036682679924708e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -7.9204249645731323e+01,     1.0817648891196141e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.2618975517442863e+01,     8.3294094836410532e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0471677271806257e+01,     8.3279733656058470e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7718222900963042e+01,     1.4880665277481092e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.4645955918231934e+01,     2.3072285864021003e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6340569925091089e+01,     8.3293214312631480e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4850271768709861e+01,     2.3892064277116220e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9199935825880218e+01,     2.2552697394641709e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.9827516557059106e+01,     1.6113973976363759e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2319168762263665e+01,     6.2326024625787022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.1061681589331585e+01,     6.3069541802069375e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8199888856598470e+01,     5.2201707466508583e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0283856275119886e+01,    -1.3549297719099573e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.6317220541626753e+01,     8.1416838640193928e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.6042135152524715e+01,     1.1020319635402948e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.7510474116089307e+00,     2.4500081932025565e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.7205283607184647e+00,     7.7959697845307410e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4818701787217753e+01,     4.1654963122824302e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7996070930269902e+01,     2.1657936339589487e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4175546701611591e+01,     9.9188089873361096e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.8740836079905270e+00,     2.3469608276103594e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -4.0497892905715290e+01,     1.4213792065753296e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.8983682983440353e+01,    -5.6691718041644936e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.4712299013898154e+01,     2.3882262425862100e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -8.1031694246181951e-01,    -3.9433276081281424e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0329204771325710e+01,     1.4898698961421800e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3681876517256441e+01,     1.6658511111895165e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8283384819554996e+01,     8.6410316282078493e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    4.4859568382610121e+01,     1.5389950967898956e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2582196896486100e+00,    -1.8733821386179123e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.2152733979279855e+00,     3.4112807847377728e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -7.1716260263453293e+00,     7.2197526934739280e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8960048735797013e+01,    -8.5513004754751273e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.2122948671622269e-01,    -9.4112750993240218e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.7972233974280506e+01,     1.0717470165140072e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.3731043941009329e+00,     8.9466897284135048e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1181042775078922e+01,     6.6062273155395815e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4401148584609356e-01,    -1.7772722753978884e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    7.4241411112709041e+00,    -5.0318172086712946e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -5.3240599225423839e+00,     1.1909238973566187e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3470904925512217e+00,    -1.7771300063045132e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    6.2153908311371566e+00,    -8.6081306683785351e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -8.2693095075024754e+00,     3.7780343552634861e+00,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.9358062405932362e+01,     3.1982884207816845e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -1.6871908902132390e+01,     1.6889474220115448e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    7.1650495347654406e+00,     8.3563135686138612e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5217546424454319e+01,     6.6082971854450479e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -4.0290713551993576e+01,     1.6843624381937843e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5011027580865189e+01,    -2.4970341789871047e+04,    -7.0626738448770513e-04,    -9.7716185163719508e-07,     4.7104982070283614e-09, } , 
      {    8.0163446056407466e+00,     7.3118340167470606e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1315661290816266e+00,     3.2182088195772831e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    7.1309351780052719e+00,     1.3184628471823869e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8823329195138863e+00,     2.3260058810367293e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.3274621136575576e+01,     7.7010884714940566e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5412808901441114e+01,     1.7223870657279622e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    9.4177919760207391e+00,     2.7941340822419690e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0236674298200843e+01,     5.6878900797412950e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -2.6467072310995391e+01,     7.2171761109396539e+03,    -5.2969489971602004e-04,    -4.8641453588455959e-07,     2.1796738289003623e-09, } , 
      {    5.0584822823366160e+00,     7.1566845409768939e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9799648448117456e+01,     1.7285338118334370e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8605231088728992e+01,     3.8416081415244207e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0697807282943840e+01,     8.9345645099618232e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1827539201417487e+01,     1.5135993781416026e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1498441843474801e+01,     2.4400426504780509e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    9.7993296814868298e+00,    -8.7348303055147026e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1956424839699359e+01,     3.1569199721866968e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.1125601510713711e+01,     1.6665009432873172e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5694621566911390e+01,     3.2299463934098971e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.2787425634240126e+00,     1.6649756265620628e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1442105932641841e+01,     1.6149020621582609e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.2251841909840763e+01,     7.7955556716621613e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2473694517535247e+01,    -7.7622036165907302e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.5235400775262583e+01,     1.1757659708974365e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.8429724662048994e+01,     1.6728023838701818e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.9283578205505986e+01,     1.5398466479214401e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -3.6667749678815618e+01,     2.5223232716780585e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5571520963046950e+01,     8.9175364237291560e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3575758772855352e+01,    -5.3296387830019939e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.3145030079059122e+01,     8.9541540956112021e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3297358914495483e+01,     3.1728409922708262e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -7.2487326021049341e+01,    -1.1979702165181186e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5427352187639180e+01,     9.6206436656512597e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6862641422400081e+01,     2.9120403731498653e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.8664207979407699e+01,     2.5526012886646913e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4939642091973351e+00,     1.3110601138031398e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.7936428858781248e+01,     3.1103532788165387e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {   -1.2070608785994352e+01,     3.2080692433534958e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -7.4472348255460574e+01,     5.9729897235556873e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9196370583830973e+00,     1.6121398477592200e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.8387955734676254e+01,     1.4427766701207887e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7923736572967798e+01,     6.6504563497132331e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5011859685199875e-01,     1.4825429191321462e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.6408289312822887e+01,     7.2142533210692118e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -4.0900753256200701e+01,     7.2138724096474780e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4670449700643630e+01,     5.9754966387231198e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0245112098313129e+01,     1.5458146003537264e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1649063346456039e+00,    -8.3566327321534045e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0739015862604296e+01,    -6.9209008557910665e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.4874569007737900e+01,     4.1707891472931107e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.9380717884901362e+01,     7.3030803789731362e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.3166042032505011e+01,     1.4889863541135319e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.6710144429174489e+00,     8.3948147171479195e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0256875281371045e+01,     6.9831834916441912e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1084549404693030e+01,    -8.2565536091340473e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {   -2.2856546376465374e+01,     4.1531240929141859e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.0366434854783751e+01,     2.2066152763264439e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.5832383349731174e+00,     1.5289418731173379e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.6800898855754919e+01,     1.4428484760225490e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.1198839613651312e+00,     1.7494528554561868e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -9.0495582048928284e+01,    -5.5741233120639174e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.7469379253769773e+01,     3.1204953285657324e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.1688289617261354e+01,     4.1772325822744962e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.0560626395561491e+02,    -5.5721622805871391e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.3326581752462289e+01,     7.1499808315025621e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9998758895551330e+00,    -1.1462677484984451e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1379042760184447e+01,     5.8254504905430804e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    6.9496593259484492e+00,     8.9776175721799682e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0181825676620875e+01,     3.1782090365386332e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -4.0735093962681688e+01,     7.9063638789730630e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3824898562491338e+01,    -7.6359494897640488e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3723368514368371e+01,     2.4102325088319398e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.6739470883234169e+01,     1.7039587107683030e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5088947016685033e+00,     7.0191166330600827e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.3744243239560042e+01,     1.4914058563142715e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4513073433048351e+01,     2.2135068525893734e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.6253298313221634e+01,     2.1605428834204562e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0837139893118859e+01,    -8.6169042917545139e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.8517224732634677e+01,     6.5771885591314216e+00,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3540677399494321e+01,     1.4366321213219016e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8280336481797203e+01,     1.5805822228040533e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.1443958788303306e+00,     3.5529212112361856e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8455813799063346e+01,     2.3872541595472729e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.7102382750053167e+01,     2.3870349833737760e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3552411299051947e+01,    -7.5606984286933384e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.4927273703837667e+00,     8.9208906591053710e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.9135776425695738e+01,     1.6504579527164064e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3896937386039127e+01,     6.2562522256177544e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.8961474595861976e+00,    -1.6656279321852693e+04,    -8.6073680299738134e-04,    -1.1604177869939913e-06,     5.5678912020705760e-09, } , 
      {    1.3378834924256648e+01,     6.5651367178018263e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.7535700225013457e+01,     3.9266865373491248e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1138621832443958e+00,     7.9608487536965622e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3530793144066983e+01,     1.6090879781699030e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9237217834601083e+01,     2.3156397080632261e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.0622025475855565e+01,     2.4344648777626539e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0858819648341326e+01,     6.2729164039481220e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3782509219942504e+01,     2.5105518979564084e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.9930042042012325e+01,    -1.6766415337772771e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    1.0951808295708835e+01,    -1.4333038458544135e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.4924576668377806e+01,     1.5545031806339901e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5643732201454823e+01,     3.1494277293171031e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.3203105243057554e+00,     6.7772710621083934e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.5293586153486860e+00,     2.1947044114481482e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2004524755881672e+01,     2.4059665272363694e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    3.2727212940122833e+01,     7.6961260839201859e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.9454007857033968e+00,     4.7437600031443765e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2729802231567469e+01,     2.3325621422829419e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8493540180814065e+01,     3.7659163869903095e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.3519864870346851e+01,     3.9337328565106225e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5632394637525522e+00,     9.8615449403400760e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3541032428711326e+01,     1.6035108738644110e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.7540985253483061e+00,     1.4998557921547510e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2999445949762913e+00,     6.6659543395870742e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4503623105660726e+00,     7.6929056730088496e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4645597914513235e-01,    -1.0169843346203117e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -6.9879209890349884e+01,     1.6882474343271628e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.7728674462005202e+01,     7.1492761995864121e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0204178536822813e+01,     2.4866629568176613e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.6091043998565510e+01,     8.3290217919956103e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.0226931060108498e+01,     8.3283610572512898e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4313670959701714e+01,     1.5456013241054356e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.0021110404765658e+01,     5.5486942415749727e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.3476662205666379e+01,     1.6949415608252923e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.0852016063646843e+00,     5.3019176383656886e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.7713941091179031e+01,     5.1331698130739824e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.4283387938817331e+01,     1.0018169274681537e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.7740800774476622e+00,     7.7948298378684149e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.3373234763945071e+01,     1.4902956717079525e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.9825043221797998e-01,     7.7001601824201971e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.8446493669759856e+00,     5.9640066508663913e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2920837367681168e+01,     5.5306452981191114e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    9.9908806262873213e+00,     7.3623137729174232e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0571621913425332e+01,    -1.3656476586169592e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.8600823818939787e+01,     4.7807325779024352e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.3884045721441753e+01,     2.3278083018569094e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.8320948543746869e+01,     1.5753457741533417e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6313353869835069e+01,     7.8274500120444918e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.4492842040070922e+01,    -3.4884868614223287e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3308160957453261e+01,     1.6922340360478952e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.9965906154201427e+00,     4.6917448832047739e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3157285947358702e+01,     2.4011165336482791e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.8027412510730791e+01,     1.6676118402915574e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.7031176809156130e+00,    -7.6843843313539091e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.9196908241286668e+00,     6.9930089045853356e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.0546266292860921e+01,     1.5781643701369268e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8024610138984638e+01,     6.6747727939275928e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.1536815491558148e+00,    -1.7004003153066740e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.4384053686103719e+00,    -1.4273882545445183e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4394079074349726e+01,     1.8128614557115980e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3594543146491770e+01,     1.2227734266899195e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.8262944779392924e-01,     4.5404683385195558e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7063054227486205e+00,     6.5147732345627765e+03,    -2.7763825076099793e-05,     2.9826077031435431e-08,    -9.5130140507313155e-11, } , 
      {    1.6497688438221743e+01,     1.8151544604041759e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9901946840458073e+00,     9.5852953350515709e+03,     1.4696464477650915e-04,     2.5195165838298003e-07,    -1.3448634551242304e-09, } , 
      {    1.0724587356489549e+01,     2.3020652968042837e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8113020534763727e+01,     2.6695169056565814e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.0178753205412317e+01,    -1.9164029826503811e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.7255576817019858e+00,     1.2210096416998496e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6550056889117236e+01,     1.6379644938362579e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3221681386558442e+01,     1.4601607828571572e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.3421810185647040e+01,     7.2012609860031398e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2828200689143361e+00,     7.3242855362243681e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.2299329520903676e+01,     9.1983459078404048e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.8503593223269515e+01,     4.2367389205168479e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -1.0473208108687368e+02,     1.0211883211282840e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0491594010056344e+00,    -9.6708936578266566e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.1192906059896035e+01,     6.6387149017850816e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.4257965379294184e+00,     5.2784979361189880e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    7.1484237093884158e+00,     1.6845602407005350e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    8.5814063255429232e+01,     1.6851390286694772e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.3497086092485695e+01,     8.5333098581021550e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.5774319957398983e+00,     1.5398137846839629e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.4656817286872979e+01,     1.6888507817714388e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.8375714453464703e+01,     2.5931170105621954e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.0820178030236210e+01,     1.4190224542563103e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -9.4112552245799854e+00,     2.3738583021572231e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.5193693235738799e+01,     1.6018258552944500e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1277911511911602e+01,     1.5535750708502361e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -9.6020836232420592e+00,    -5.5360750031569842e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6050686140654719e+01,     4.6235241900252397e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    2.8264337990168812e+01,     7.2447157877170430e+03,    -5.5832899822007905e-04,    -5.9151709988682344e-07,     2.7173322012508378e-09, } , 
      {    1.9511104232134105e+01,     3.3308206742894668e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    2.0535082222934030e+01,     6.0218820450811163e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.3825585709216783e+00,     5.5709856811734044e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.3325082422331104e+01,     4.6080311934298770e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -4.5818185650709168e+01,     5.4572285298665520e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.7205012946744560e+00,     4.4694750331065652e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.5178743018472645e+00,     2.1961271023819008e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3950712132615651e+01,     7.2067331158465022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5013945961969942e+01,     2.3090878398811867e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9789175114877199e+01,     2.5128060884696330e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0025528103618672e+01,     1.1803335908432728e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.6626836108574082e+01,     2.7600694240708526e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.4495141805511018e+01,    -8.8368972268977814e+01,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -5.5788524864194677e+01,     8.2565112605786635e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.7776051785289788e+01,     2.4634740033632952e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.7706355417537321e+00,     6.8740015171376381e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.6725949237433806e+00,     1.3515844924019184e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3378618759341265e+01,     8.3612543830978211e+01,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {   -3.7989966155736745e+00,    -3.3356835393394931e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.1989782835934024e+00,    -1.3525639584116657e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.2834986318092675e+01,     2.6225416447195770e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.6754164275965593e-01,     7.7163833478019042e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3724862937226270e+00,    -1.1352920755219984e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0575748948792743e+01,     1.6150443312516361e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7403950976012101e+00,     8.7489348661724653e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6712680990209265e+01,     8.2065438948603514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.0367845602698836e+01,     8.3257484589918186e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.5607953911808004e+00,     3.1599002415431996e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.7436487392172282e+00,     8.9989474801578199e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0422889540999577e+01,     2.5607822903316755e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    9.4503867476879861e+01,     8.3316343902550798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1191004557751549e+01,     1.1725039534798289e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.3719623160033507e+01,     8.3225371061539863e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5866921405670112e+01,    -8.3348457430929120e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.6283351520020020e+01,    -7.3204807810831844e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.0437770502395452e+01,     8.0289551999418873e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.3127178055461837e+00,     1.6078738661636497e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.9607165468806400e+01,     3.1091542054406342e+04,    -1.0145177493310048e-04,     1.6063952257714078e-07,    -9.9061979461111474e-10, } , 
      {   -1.4047812590727659e+01,     1.6166130842769744e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8775737794584071e+01,     1.0444971671829688e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.7501024621504477e+00,    -9.6589029240676100e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.5719187001306857e+01,    -1.1244482884992249e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.9478047759849701e+00,     8.3254381688003687e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2373775832101375e+00,     3.6843780589205488e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4023357650129839e+01,     2.2118200289039109e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.6798622893864348e+01,     3.9401082093388766e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    6.1880746696393594e+00,     8.3319446804465315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9951564197116610e+01,     3.3806002445455801e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    3.0863627196561367e+01,    -8.8307448828423094e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.3823797638265059e+01,    -8.5360479429629595e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2666546293755585e+01,     1.4181373862761468e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5194728242351712e+00,    -1.0829893700317662e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0438486989115356e+01,     1.7215089310806783e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5491406784297936e+01,     8.3914005476116818e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0373201523749778e+01,     2.8423404040981535e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.0865590388009480e+01,     3.1260310215843667e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.5670429554356886e+00,     4.2643614355924342e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3347712613462704e+01,    -1.6903014942379774e+04,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {   -1.8841330363601139e-01,     3.7394183815490669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8654163777949233e+01,     1.8544935436699714e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.2132099465982948e+00,     1.7278212267723386e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5720401593336611e+01,     6.2081534217594926e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2144480940701268e+00,     6.8084226572473699e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.9491927095392583e+01,     1.7021538164227168e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.9888214696998148e+00,     2.7668792718502118e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3970663229356187e+01,     1.2147567433086040e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.2889544163046615e+01,    -6.2937124647049450e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6974906083803507e+01,     1.3564531530318591e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.4921388218379644e+01,    -2.9377315373889123e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.0012105508337461e+00,    -8.8852943587891277e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.3690149379775413e+01,     2.4424502238580077e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2599876035941016e+01,     3.1192886336771822e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -1.0607276088620065e+01,    -5.3783406488165519e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0305865170658395e+01,     5.8526824381380175e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6408108815517565e+01,    -3.2270382048334966e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9944456133111032e+01,     2.5597045159681898e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6302843403241575e+01,    -5.9241649405923781e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.1236790821607372e+01,     3.3570715830103247e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.2295628528974584e+01,     2.3845708564988992e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -5.7817969581535543e+01,     2.2144051598583028e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1552002462424607e+01,     7.2213926148701867e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5440514822888053e+01,     1.7121397124352879e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6916505927923623e+01,     8.2637584020293143e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.7042849001690858e+00,    -1.6744133373913253e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.3973671437385073e+00,    -8.8867170497228799e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -3.7016694772296404e+01,    -9.0959074859503799e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.9597369408755338e+01,     1.0340196938961537e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1728215571336049e+00,    -1.6455877661349503e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -6.1126058912238861e+00,    -4.4143900027996074e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9658550846739260e+01,     3.3883554706467927e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.2151691921915679e+01,     1.2463020882251752e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.4043531680811398e+01,    -9.0952028540342308e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.4669731593127025e+01,    -1.8254194375803654e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8762225658495090e+01,    -2.3094239523433680e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2553256280552052e+01,     1.6896429265994720e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.2810560340632605e+01,     6.5864599122289510e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.3524008349459194e+01,     6.5850506483966501e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5543796817344163e+01,     2.3846862550804315e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9611307847764881e+01,     3.1294705729741818e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    6.0382838955072433e+00,    -3.1315485191259127e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.1430059202422015e+01,     3.1295410361657963e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    3.2362990591954762e+01,     3.2212756377646681e+04,     3.0023079402995232e-04,     6.3636362590502636e-07,    -3.2291500043981749e-09, } , 
      {    1.1842816865094427e+01,     1.2772000045232418e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8951476860010462e+01,     1.8588706270256676e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.5242543136942414e+01,     1.7355520278151202e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.6959512017768510e+01,     1.2066948885503734e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.1293287403568645e+01,     3.4819279058231950e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.6031249170486159e+00,     8.3784723603026086e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.1137755411044035e+00,     8.3776434765917129e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0281462134101343e+01,     3.4674992139898801e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.5468288671396678e+01,     1.1160020305802671e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4238726438819405e+01,     1.1964095929129353e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6406079384986690e+01,     9.4521384382290726e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5805900207442583e+01,     2.2741084671340839e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.1679120442994961e+01,     8.3511259243306195e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3826418688631563e+01,     8.3062569249162807e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4914452382043045e+01,     3.4387410691847923e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.6217859310937406e+01,     8.5201919469038912e+03,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.0444243072019251e+01,     8.2042398249578982e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.9340191082209891e+01,     2.9948338115566239e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.9119387992720959e+01,     1.1271154840361605e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2054758113061901e+00,     5.9885992904062359e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3236930379261386e+01,     2.1026998922227758e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4142494449687717e+01,    -7.4075241389546568e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.6833968298863581e+01,    -1.9524788964962238e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0804415314084324e+01,     8.3448326239829494e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3332705308909034e-01,     3.3903258381641426e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -1.8404270578297826e+01,     1.5168230573582587e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5925291355216380e+01,     2.1899781910541173e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -8.3574450399106169e+01,     1.9837004707895292e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6343697091167556e+01,     3.0616460602509695e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2224626264363899e+01,    -1.2957920915911486e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.2963045683868110e+01,    -8.7508418825483022e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.1406517739410527e+00,     1.3613804272203267e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2210262496870982e+01,     2.3438093837596876e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5947503981498354e+01,     9.0460549055785577e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6338618094763877e+01,     2.4901377398032833e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.4224428123490867e+01,     5.9798821949744279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.9043785771190748e+01,     1.4483482660903030e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.2023202472449386e-02,     6.0257043542928909e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.4247018674936712e+01,     2.2180767322038915e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.7291683527043045e+00,    -1.6585245033757499e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    2.3413232455444962e+01,     3.9230728500630350e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {   -7.0572438435084123e+01,    -1.7273941160222194e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5682738607313325e+01,    -2.2031483675713446e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6005235274274529e+01,     3.2808794709885024e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9619142307690975e+01,     2.3921109112040383e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.5915027324286434e+01,    -3.0954658917293436e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.8171450467327539e+01,     2.9048539559783961e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2772557676726635e+00,     8.3213616751116060e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.5157545149318480e+01,     7.0080828259631871e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0634619459114219e+01,     1.6111543297955923e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -1.7238608153288283e+01,    -8.0312716593984478e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3401569194185576e+00,     2.2909076886087976e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2994195361690814e+01,     2.7126947160616430e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.3706172734051865e+01,     1.3505571804097330e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.2107180340225590e+01,     6.6383720615659176e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.8390554005575737e+01,     8.3149002368253095e+03,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -5.3101663106349349e+01,    -8.3424826124215906e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1178073985694486e+00,     2.5002073782094347e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.6989828120637483e+01,     3.6587802155665420e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.3115772070099553e+01,     8.3512350647530820e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5724657714971890e+00,     8.3360211741352941e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6942301509709822e+01,     1.6801670660014293e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4496333237249099e+01,     1.7436865833033742e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.2811932318469998e+00,     1.4356028897909491e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.4491860942886525e+00,     2.3772118919734967e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.7323644169770411e+01,     1.4996929998205969e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3083678095393259e+00,     2.5489870081180467e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.4241089830408519e+01,     4.0451276302840022e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.1167075657495174e+01,    -2.5208741650804914e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {   -3.7916423426678101e+01,     7.2268647447135490e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5650240491721899e+01,     2.4958844311807268e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.6193899123888194e+01,     3.3038914704026320e+02,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.6813962044508619e+00,     1.5054166326813795e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.4481286680376662e+01,    -8.1810016669847755e+01,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.6579641840406691e+01,     1.4308573619597877e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.5619686314087105e+00,     1.4905586549395708e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.1855212304372094e+01,     1.3781793937469347e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5505830557751056e+01,    -8.3552100412196542e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.2603766976580339e+01,     1.6011026320745495e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.6657611105671545e+01,     1.1364638739281376e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.4423531234623198e+01,     1.4210888058819995e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1389999390464261e+01,     7.7270445461625404e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9559110565630164e+01,     7.9461787808709760e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3330559934378014e+01,     9.9024136450720998e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5330676738300035e+01,     3.4767812423122923e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -4.8003466035908907e+01,     7.2070629885145299e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1594988367360983e+00,     1.3741942751456423e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1491955504234543e+00,    -1.8668890711959195e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.0391123526642936e+01,     3.9964955328620461e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.0229138031075005e+00,    -2.4992633229469484e+04,    -6.4226121911122155e-04,    -9.7181663535885805e-07,     4.7399263974717103e-09, } , 
      {    1.3948072041583318e+01,     8.3413296145192580e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2378555594399506e+01,    -8.3160532347276403e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.8840429213070522e+01,     4.0986283883358992e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.4140909122167752e+01,     1.4399739797306431e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.8751605518904064e+00,    -1.1899661579315500e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7667881240829594e+01,     5.1041494097912528e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.7711783664629177e+01,     2.1484454241734606e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.2651395309131756e+01,     1.4785098284006077e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2104287468182733e+01,     1.1859321196261202e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.3204075707821978e+00,     1.6468481640252343e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.8072221307526863e+00,     2.5211623247830648e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -4.6435256493572112e+01,     8.3303040884498332e+03,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -5.1146365594345710e+01,    -8.3270787607970633e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.6941526395117993e+01,     7.1038401944923207e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.1851289736099293e+01,     3.9431120685394628e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -3.5384175132225764e+00,     1.2197081344540500e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.6562416531408420e+01,    -1.6876006162721183e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6539498594641920e+01,     1.3666691690385709e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7339156471569883e+01,    -2.7024449220363549e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4324684442523649e+01,     2.5754041546633413e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    9.0669587545457375e-01,    -1.0387096277614961e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5971649442483642e+01,    -1.0954503652479841e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.3709641434635174e+00,     1.8040565519024291e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.7959561595856206e+00,     1.8070578387863094e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.4375867009278139e+01,     3.2115440263391181e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    3.9350081321520420e+01,    -1.6642462410887500e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    2.1993008964170869e+01,     2.4064074499533952e+04,    -7.1725590628761307e-04,    -6.3861160998159502e-07,     2.8330572269316835e-09, } , 
      {    4.5343108682110568e+01,     1.4931719701701921e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.6883880257603217e+00,     1.3531248775643711e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.4257505003293005e+00,    -2.6609752884208015e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5660927082641098e+01,     2.9526519757483074e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.7751871028423949e+00,     6.5771187819299685e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    7.1943012612876522e+00,    -8.5029038194230288e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.2854371452182715e+01,    -1.0326059008610000e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0500770182101174e+01,     1.8637865578261320e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5806867859862844e+01,     8.4596574087270674e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0483828716092738e+00,     1.3262180732835994e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9397895546589066e+01,     3.9325182490972249e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.2482209760158934e+01,     7.0225470835627530e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    9.5754813872883595e+00,     2.1222367107048285e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5659541602327943e+01,     4.8957882332133449e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.5770239124875967e+01,     6.3166872375050862e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -5.4576092145792092e+00,     1.9975017445130532e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.3971447530679498e+01,     1.9519869372666708e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0932462130282095e+01,     2.4446072856972722e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.6958217322589233e+00,     1.6097679878251849e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.6846640468762679e-01,    -3.2374677620704751e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1711886954522063e+01,    -8.3870654916184540e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.0436634402102467e+01,    -8.4602440034673255e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.5113409023272233e+00,    -1.6828235114785602e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.1174439165140999e+01,     6.0376297885955646e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1692827511230314e+01,     6.2845600210275711e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5409257571919728e+00,     2.3006426058705310e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0096342041729978e+00,     3.2973559840196678e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.2706323662782649e+01,     8.3114166928603936e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5633925792025432e+00,     1.3413356131577293e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.0798696405629407e+01,     2.7724103150522995e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.2904140754918913e+01,     4.3358499050408500e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.1957204951813170e+00,     6.4875395030437840e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8721426640225840e+01,     1.6617042466192604e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.4848541923067890e+02,    -2.6973708783007169e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.1826707866558035e+01,     2.3378565784786548e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    3.5305184350965796e+01,     2.0732208989318940e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2036654777872021e+01,     8.5742893067633540e+03,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {    2.0796839525494295e+01,     3.2428864798982118e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.8420103652753820e+01,     3.0162954653344234e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.7032751727885078e+01,     6.1001456515601176e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.2767691349294907e+01,     3.0509016106313913e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    2.0301772614921646e+01,     2.5635045375172791e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.5757420016568327e+01,     3.3747216462188220e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4959879104189625e+01,     2.4482973175880361e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.2997123291047934e+01,     8.3027819294749497e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.4580588064114842e+01,     3.0456872362543272e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.8092771737972395e+00,     1.0053995542659630e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3566566037061030e+01,     8.8908757171445232e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4492218083022429e+01,     1.6618534349750051e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.3877609160254110e+01,     8.8325474100034426e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    6.0024817553562517e+00,    -7.9422294105295969e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.3563886475761898e+01,     6.0987229606263636e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.2811097875395880e+00,     1.0325173661861867e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6385084478640685e+01,    -8.5014811284892767e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.6734357461706523e+01,     1.0003814033715344e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9847151915063570e+01,     1.5369253240649092e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.2089330718268293e+01,     2.2648258136313502e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1355938655063693e+00,     1.0476525712924040e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8466673951431201e+01,    -8.3226311418045952e+03,    -2.7584724895743558e-04,    -3.9757073197335323e-07,     1.9187955870953214e-09, } , 
      {    4.8174092060893713e+01,     2.4811272637990267e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -7.3042355889481385e+00,     8.2862734987854565e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.9359752061197000e+01,     8.3237671059165805e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2748485423965203e+01,     8.9968407199430076e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7955665745693725e+00,     1.5345205033743452e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9163042557407056e+01,     4.7377102407112441e+04,    -2.6037868300063448e-04,     1.1354501248236910e-07,    -8.7489476893026875e-10, } , 
      {    6.6169023793215320e+00,     2.4279107082707656e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.8328374355032523e+01,     1.6654729333347415e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.0425187834876795e+01,     3.5309560988857857e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5962151432614275e+01,     9.0743929809551300e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.8916263260303307e+01,    -1.1409369498916640e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5998040173945407e+00,     1.5001197229602603e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.1444268474006662e+00,     4.0764540634135774e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5868204110547817e+00,    -8.6965340955503398e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.8205448630678717e+01,    -1.2615462023486798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3105284874146760e+01,     2.3896028878406174e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.0480873195612531e+01,     7.8268284221154581e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3823655429506793e+01,     1.6784449021713481e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    8.9541170559399585e+00,    -7.3177340708322909e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3774671072211758e+01,     7.5786979016410733e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1127172265644756e+00,     1.2279894106995066e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -2.6730869376239987e+01,     1.2393421733254181e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.9176006536854388e+01,     7.8466282060616104e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.6679739286282441e+01,    -1.3700264013420713e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.3762258503798273e+02,    -6.8279531963416957e+00,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.3569064757520273e+01,     1.5305595847071560e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3111994096521705e+01,     4.2165346792437951e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.3769553582534655e+01,     9.3995338908164395e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.3150047827164968e+01,     2.9216004935966637e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.5893836405341064e+00,     4.7861521570875793e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8663463392787691e+01,     4.0896112383928885e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    2.6753175052458332e+01,     3.5160833392815912e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.2809873387593348e+02,    -1.1142829568577583e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.9119417209494038e+00,     1.3169209360684194e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2176172305190141e+01,     1.0881099426689009e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2158233372468558e+01,     8.5700383227948605e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    7.4572815390034322e+00,     3.1968247604685035e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0147556878824632e+01,     1.4984728664882367e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0454687009700988e+01,     7.6666024121227465e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.1203726641260802e+01,     3.0666888883871479e+00,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.9971274660592307e+01,     1.0193937312962680e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.9240038318529131e+00,     6.6195367077679621e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.3890136779210614e+01,     1.6246312936098049e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5563507878167961e+01,     2.4631750504374966e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.0220904007387052e+01,     6.7102670580482272e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -2.4612002569314733e+01,     2.5160875909750430e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.9641876352410684e+01,     3.0710536553726459e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6726358079278327e+00,     8.4837959466476987e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9219471190166686e+01,    -1.0064081163791307e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.1818948104165145e+01,     1.6740995393077879e+04,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {   -3.5797805229127455e-02,     5.0120330598641513e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.8605676579876267e+01,    -8.3322608509645961e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.0033084433636090e+00,     7.3319460336858519e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7536664658208604e+01,     3.9178913389234483e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    8.6323140043579869e+00,     5.5186634163467920e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.8021122910884468e+00,     8.3958714382233047e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1343842786249947e+01,    -8.2922497619574078e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8717464830880225e+01,     2.3969661491874260e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.9203962705026463e+01,     1.3753432739313148e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5411683332723999e+01,     3.3386862531300998e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6608797185633197e+01,     2.1589219498863844e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.7568108517479274e+01,     7.1273123406919940e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.7021545494340549e+00,     2.3249697085158838e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    3.9993604504603340e+01,     6.9035251889350748e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6172351266635104e+01,     1.4421566775117561e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    4.3349273150028864e+01,    -4.5565279234571426e+02,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.4847591524449646e+00,    -2.5131189389792791e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    1.9526742382752591e+01,     6.2571686604336728e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.4998123927777559e+01,    -4.9366178133090079e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.8628676530501671e+01,     1.4378462333281552e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.1577854034188368e+01,    -9.5792731202717860e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.6202650961170679e+00,    -1.6626204445731819e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -1.0003160442962422e+00,     7.9072845215202033e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.9964231651573414e+01,     1.6656401187330681e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.9386449853120624e+01,    -1.6658364511163112e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.4130730206929023e+01,     8.1850681922584499e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.9127400614893403e+01,     6.8021027865680308e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.1814054828586180e+01,    -5.0124141257895826e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.4066476271804724e+00,     2.0962564572413903e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6828262199473187e+01,     8.8505810939441544e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -6.1846408671281239e-01,    -8.2930362544822910e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.9569667999451255e+01,     1.3308228263085008e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9638831194173267e+01,    -7.6316917341063254e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.7666455352220453e+01,     2.3198256155620096e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.5325862425240228e+01,     3.2081099985085864e+04,    -5.6496349852528684e-04,    -3.8789273582694036e-07,     1.5970732283110329e-09, } , 
      {    1.2207667673885267e+01,     1.6727924038572535e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4779616039790234e+01,     2.0745326900517204e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2506509667811674e+01,     3.8473072494654640e+04,    -4.1267109076296066e-04,    -1.3717386167228564e-07,     3.6108922969038208e-10, } , 
      {    1.0983403217681159e+01,     1.1283957973172790e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.7762906904278509e+01,     1.8129607767893140e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.1358766079528452e+02,    -1.1149741616724539e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8485519255885716e+01,     3.2821885768675107e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    1.0785646069505713e+01,     9.9674539093759790e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2523380943136848e+00,     1.6145322739879228e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0530539913236735e+01,    -2.9972243912932709e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.5880675600861217e+01,     1.6226418805213680e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8080371907744514e+01,     1.0971617769588391e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.8973565577537549e+01,     1.5515145312807861e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.8110074217718648e+01,     9.4147509378499490e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4641801181429713e+01,     3.8457305799662368e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.2670620761695513e+01,     1.4182527848576789e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -2.3943727361720814e+01,     3.2828092408316166e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.7560967967498771e+00,     1.6921657288906184e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8503487713738458e+01,     3.0527090772936830e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.7931594146345155e+01,     2.6940362411078433e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3506152770449964e+01,     1.6145324725411087e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    3.1444498469230801e+01,     4.5453993707771515e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9453073749857673e+01,     1.4986151355816119e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0182090668615400e+00,     5.1779647114804166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8344647248552048e+00,    -1.0203624773658277e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.4874964342380915e+01,     3.4465442674162499e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    5.1498298659066357e+00,    -9.7755772118156665e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1860989650862610e+01,     1.6408608459731109e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1904287777731216e+01,     7.3723560705489308e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.7113569776537204e+01,     1.3253072755626654e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0964578527330026e+01,    -4.8125390217787299e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.6216471832174250e+01,     2.1053695912927407e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    7.1533663438576944e+00,     2.5195400082448941e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.3494141691313740e+01,     2.5195847396764359e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.2636448645377913e+01,     1.5655249231728167e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.1047131334085973e+00,     4.6544318593030248e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.8757971326424858e+01,     3.1558711642984883e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.1995630050043932e+01,     4.1524419968974427e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    3.7898315817306658e+00,     8.9286130762587345e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1589858665110548e+01,     2.8130242031414244e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.4554338319988226e+01,     1.5191420049744398e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0696323614182607e+01,    -5.5131102869586794e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.8071566695271102e-02,     4.6982026928599880e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.1446597066302502e-02,     1.7647544549331302e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4059133189233089e+01,     6.0541781502965041e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    7.4919162402384654e+00,    -8.1517788762658984e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.0253312452414908e+01,    -6.5420616688499933e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1468505856272007e+00,     1.5137114081870819e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.0430268009808529e+01,     2.7553749557764575e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    7.5764353519459187e+00,     1.5444538512712777e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.1497505420030841e+01,     7.0630447193796851e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.8173153838429652e+00,    -4.0945030068694405e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4941336604455174e+01,     1.6086453588582806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0384070478028264e+01,     2.7618183907578430e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.1553384548948689e+00,     7.8761462480275650e+03,    -2.5205404269363512e-04,    -2.2438883705906827e-07,     9.4810648291461947e-10, } , 
      {    6.8527025886167028e+00,     1.6133849997069508e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9622730924883488e+01,     6.9707319721668855e+03,     2.4503613970602655e-04,     3.5993110676894790e-07,    -1.7597282183232826e-09, } , 
      {    2.4448035169601251e+01,     4.0495047136396985e+04,     4.2160862447771169e-04,     8.5067842252158344e-07,    -4.2905525964512817e-09, } , 
      {   -3.3554779688439575e+00,    -8.8522242768008418e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    3.1034080696983242e+01,     4.7918659467491634e+04,    -2.2946410568606770e-04,     1.4994909002046683e-07,    -1.0494761754978126e-09, } , 
      {    2.3035827336913364e+01,     4.0076294646919225e+04,    -8.3246403069993746e-06,     3.3793384954143738e-07,    -1.8230012518448880e-09, } , 
      {    2.1380059514016494e+01,     2.4675515708100444e+04,    -3.1823721881746878e-04,    -1.6227111453954675e-07,     5.4008728889283402e-10, } , 
      {    5.6107416140764492e+00,     7.6665967822912635e+03,     1.8054310358398448e-04,     2.8773934380691514e-07,    -1.4650051334399845e-09, } , 
      {   -6.2675824602443675e-01,    -2.6276470871416594e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    1.0283327494507372e+01,     1.6204848932313980e+04,    -9.4433871945491906e-05,     2.5097252867261127e-08,    -1.7899805920245181e-10, } , 
      {    1.9192407341937077e+01,     2.2581049117058999e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    7.9276949279029978e+00,     7.8761575076905319e+03,    -2.4672627970781806e-04,    -2.2562162128739358e-07,     1.0569859394181992e-09, } , 
      {    6.5111178036893342e+00,     1.6204837672651018e+04,    -9.9761634931308941e-05,     2.6330037095586454e-08,    -2.8787751570603142e-10, } , 
      {    1.2613980113137766e+01,     7.0797028381586342e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9950241394720241e+00,     2.7548319852726701e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.1440218499168253e+01,     5.3140478448405065e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6049578833617858e+01,     3.2431016512942850e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.4389731386039717e+01,     3.2388356696987146e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {   -1.4398999311767085e+01,    -1.0883201686385473e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1166677043023920e+01,     1.4456511664126949e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.1557068218090485e+01,     2.4099044123797408e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.8765811487220780e+01,     2.5270874800677364e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    3.3603621979754159e+01,     3.7309409745237535e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.4747142331321676e+01,     4.2150102237368883e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.8456435291626338e+01,    -1.3470227156342276e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4767544050327848e+00,    -1.3176190694022744e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -8.5713141135460447e+00,     3.6834330937205914e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2016362560881802e+01,     2.3137218886615141e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    6.3227174881082933e+00,    -8.7282713499155725e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.4477591407260473e+01,     1.7205491375018169e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1019371501833341e+01,     2.3464685206039474e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9139787351644770e+01,     1.9930419306418516e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1036361685304573e+01,    -1.7279656959849690e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.6125357570482016e+01,     1.0708424661929894e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -9.5698189388093482e+00,     6.1320283485021582e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1929550598721306e+01,     1.5929216304075650e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -6.0882927256105837e+01,     1.5724370143960832e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9095134329573124e+01,     8.0284907381545772e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7886160314588633e+00,     1.7810708608709607e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.8699512939884322e+01,    -1.0393774981943934e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4108468958487119e+00,    -2.5179624231382659e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -2.7739594359981631e+01,     7.4344290423515031e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.7445164241345722e+01,    -9.4307007454705672e+03,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {    2.7630257647012343e+01,     1.5557710623119909e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7486325441913550e+01,     4.7166398955560813e+04,    -2.6037868300063448e-04,     1.1354501248236910e-07,    -8.7489476893026875e-10, } , 
      {   -7.6951281829095109e+01,     1.1445956476241683e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1997566915336282e+01,     7.7080987665242183e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8414024901856720e+01,     1.0611566181710663e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9000879920571908e+01,    -9.4329866058123457e-01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.6690250635984096e+00,     5.7208523201127093e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6060309387392863e+01,     2.8652131700734953e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.9097294197505327e+01,     2.2681566094269430e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.2347006035383206e+01,     9.7925548605094282e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.5608871714130817e+01,    -5.7915843250427557e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.6339134097842390e+01,     1.7736586798200155e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    6.7607200792746509e+00,    -1.0831932358703820e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.5983641847560270e+01,     2.3243143759391183e+04,     8.8773113131401069e-05,     3.1222020456001358e-07,    -1.5895634643906465e-09, } , 
      {    2.1347706224957658e+01,     3.9378151588183442e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.6419454869667288e+01,     2.2752407928169883e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9502377164986196e+01,     1.0578596799895402e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.3908193889167336e+01,     3.4374791453429934e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    3.0612124489664996e+01,     2.2945718364334698e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.2304978732670886e+00,    -1.1396352078346808e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1611900998892850e+01,     3.2764010374460380e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.0954553245649628e+00,     2.4963734205062996e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.3336994345524142e+01,     1.9612649397796238e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0622255158633628e+01,     1.5524720606041086e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    6.9944115409027017e+00,    -1.7588539904555320e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.8935553802326893e+01,     1.9244236327158251e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.8533322140576880e+00,    -1.0657275338023937e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.4271524724068506e+01,    -6.5605553805795171e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.3489985732227936e+01,     2.3248069161817293e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0924684449369011e+01,     1.0185575830202415e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.8036550069440544e+01,     9.3626392177799680e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    6.8553263142529204e+00,     2.4357766688824806e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.6197711655920077e+01,     4.1589373566832974e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.3936112670594280e+01,     9.2426088528599739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0329813156974964e+01,     3.2517907047850968e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.4779320860622320e+00,     1.7254523579638313e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.9188912998110588e+01,     1.8522628737586128e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3365663880365384e+00,    -2.5202182695205785e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    2.2126907860677303e+00,     6.5290883013982284e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1624526961541729e+01,     8.2439945487859368e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.3509002094850707e+01,    -1.9547171345464346e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2068200595094076e+01,     3.4075070488263525e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    6.1072657451562211e+00,     2.1748606511799873e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.9864921923438921e+00,     2.0569543602720914e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.2410716196788343e+00,     8.5328111853218779e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.1328052515114901e+00,    -1.7772011408512008e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    3.9184263305863055e+01,    -4.0383768094984407e+02,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.2342222536662689e+01,     2.0404121042220089e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    4.6100474500050197e+00,     7.5710354186477343e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1431296692212145e+01,     2.9448990351377783e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.1795345170940891e+01,     2.8816425010674524e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.2484983104652613e+01,     8.4756090677070279e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.9628494247085424e+00,    -1.0096248751991396e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3526261201547044e+01,     1.5004618204366361e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    3.2348586512448918e+00,     6.7615832527539687e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8837405565903389e+01,     3.8299571445321933e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.4307484452746184e+00,    -1.7981560874248309e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.4108273746001519e+01,    -5.2008365117510766e+02,    -3.4261522819041188e-04,    -4.0225592290622767e-07,     1.8356548889218470e-09, } , 
      {    4.4624991391761366e+00,    -1.0471144650169772e+03,    -4.3240128155045131e-04,    -5.1221544732858310e-07,     2.4116571753112754e-09, } , 
      {    2.2192795760544055e+01,     6.0656772589691791e+03,    -5.5842693058366318e-04,    -5.9147346665552352e-07,     2.7180594217725020e-09, } , 
      {    8.6974559339555491e+00,    -1.0105378986226679e+04,    -4.8957987296692094e-04,    -6.5420758128920775e-07,     3.1805183440460772e-09, } , 
      {    8.9849424655597989e+00,     2.5053588368118471e+04,     3.9524393338501320e-04,     6.7926120092516895e-07,    -3.3602436237701937e-09, } , 
      {   -5.1483827423659143e+00,    -2.4918560179622222e+04,    -5.1851051318894384e-04,    -8.2505204400275936e-07,     4.0556603679537113e-09, } , 
      {    6.9005064916644576e+00,     1.0474472270557089e+02,    -2.1886452226813419e-04,    -2.5549133155012887e-07,     1.1513888594038480e-09, } , 
      {    4.5449547338044587e+00,    -8.2239467019178774e+03,    -3.7115693003046034e-04,    -5.0621020570478359e-07,     2.3873728580244988e-09, } , 
      {    1.2736903868819759e+01,     4.1677214170241321e+04,     7.3064539386064823e-04,     1.2171466600038760e-06,    -6.0060658070573747e-09, } , 
      {    4.3880581793276310e+00,     2.5263137833854780e+04,    -3.7353212892606369e-05,     1.6713302005918562e-07,    -9.4713200741558923e-10, } , 
      {   -1.0818640762383637e+01,    -4.1609700075993183e+04,    -7.9227868376261347e-04,    -1.2900420815426711e-06,     6.3537741791491336e-09, } , 
      {    2.2001610369504736e+01,     4.8823762941351524e+04,     5.7380309987645380e-04,     1.1014409299075379e-06,    -5.5258093745502679e-09, } , 
      {    1.9689181332441535e+01,     3.9866769561514018e+04,     4.2417457360703609e-04,     8.5010566363872057e-07,    -4.2353856476778278e-09, } , 
      {    5.8746942309604879e+00,    -6.2836197553922102e+02,    -2.4680167657402624e-06,     5.2912565156294998e-10,    -5.5894169295118419e-11, } , 
      {    6.5273750040862994e+00,     2.5857682741944776e+04,    -9.2004494345322132e-06,     2.0419712294274585e-07,    -1.1754259217132589e-09, } , 
      {    2.2464581494232633e+01,     4.9100826501335963e+04,     7.9572663696868842e-05,     5.1641732750275938e-07,    -2.7649893861039052e-09, } , 
      {    2.4851469991634403e+01,     3.1747567582301712e+04,    -1.6584687869155844e-04,     8.8404126383808094e-08,    -6.9662393024948114e-10, } , 
      {    3.2149937147813127e+01,     4.6662019917069454e+04,    -2.2936617332248356e-04,     1.4990545678916697e-07,    -1.0502033960194768e-09, } , 
      {    1.1485447712888003e+01,     1.3691605471463678e+04,    -8.9008176596090763e-05,     2.3820835407635938e-08,    -7.0845823220536528e-11, } , 
      {    4.1718220244958220e+00,     1.7528991317321324e+04,    -1.6149285719685839e-04,    -4.6521751211908858e-08,     6.0558076907392106e-11, } , 
      {    2.5944799575304835e+01,     2.2689303061092010e+04,    -2.2302547010802809e-04,    -5.3588007576816509e-08,     7.2237238485320691e-11, } , 
      {    1.2708765566335686e+01,     1.3624091377215551e+04,    -2.7374886694125427e-05,     9.6716256946431118e-08,    -4.1855419531229567e-10, } , 
      {    9.4134150351077324e+00,     2.4672066507098159e+04,    -1.9192056376586669e-04,    -1.6079282066310760e-08,    -8.9011791851710852e-11, } , 
      {    2.3062573302259437e+01,     2.9795147622411405e+04,    -9.6221944310867578e-05,     1.5945037158011527e-07,    -8.8101311758587099e-10, } , 
      {    1.9483703865484681e+01,     2.1533970292036076e+04,    -3.1014764197515905e-04,    -1.6416392411333460e-07,     7.0267925312653909e-10, } , 
      {    1.5634727511714800e+01,     1.6765601153285854e+04,    -4.0694294158667985e-05,     9.9798217517244439e-08,    -6.9075283657124470e-10, } , 
      {    2.1289217538640656e+01,     4.1332922937678450e+04,    -1.3750335656400510e-05,     3.3921026700106260e-07,    -1.9311534878268037e-09, } , 
      {    1.9165465795140467e+01,     2.2932609573952377e+04,    -6.8643926133663035e-04,    -6.0216389921219747e-07,     2.6592030408858041e-09, } , 
      {    1.7734726472473515e+01,     2.3908676343062689e+04,    -7.1122000727676289e-05,     1.3024068666284252e-07,    -8.4032270533034761e-10, } , 
      {    1.5921447223700724e+01,     3.1119325647412810e+04,    -1.5805109894000114e-04,     8.6642216503919837e-08,    -5.3185030445078329e-10, } , 
      {    5.6304917931502372e+00,    -2.6486044717483994e+04,     6.7324326687919576e-06,    -2.0366799729118294e-07,     1.1195317524181404e-09, } , 
      {    1.3565892149715147e+01,     2.2790634222789362e+04,    -3.1034350670232737e-04,    -1.6407665765073488e-07,     7.0413369416986764e-10, } , 
      {    6.0865162426499646e+00,    -9.6901000780823015e+03,     6.6767979232976247e-05,     4.6851909328744421e-09,     8.3140698173474340e-11, } , 
      {    2.8587682949490127e+01,     5.6247375272446181e+04,    -7.7269630287325622e-05,     4.0071159740642141e-07,    -2.2847329535967992e-09, } , 
      {    2.1564144986186250e+01,     3.0632999043361764e+04,    -5.3148297208139574e-04,    -3.5206141717170543e-07,     1.4776587705169434e-09, } , 
      {    2.1819699528616784e+01,     3.1818615278208377e+04,    -3.4876285775006121e-04,    -1.3178501216264880e-07,     3.9124464065539534e-10, } , 
      {    1.5379169304092834e+01,     1.5579984918439241e+04,    -2.2341440849000247e-04,    -1.2047818749181223e-07,     3.9566129329030322e-10, } , 
      {    1.2768250731271195e+01,     1.5995263826583618e+04,     3.3293344370989477e-04,     5.3841458473026995e-07,    -2.7017163525822996e-09, } , 
      {    2.6349903369162426e+01,     3.0909716161351356e+04,     2.6941414907896965e-04,     5.9991591513562881e-07,    -3.0552958183522955e-09, } , 
      {    2.4037475553829704e+01,     2.1952722781513843e+04,     1.1978562280955200e-04,     3.4858064886681141e-07,    -1.7648720914798550e-09, } , 
      {    2.6306583670651694e+01,     2.1608431548950972e+04,    -6.2461010670749684e-04,    -5.2935574413600198e-07,     2.3100402277507168e-09, } , 
      {    2.0140371537369241e+01,     1.5090184733054812e+04,    -4.7043169421621077e-04,    -4.1303362192550133e-07,     1.7753440669918203e-09, } , 
      {    1.3042374288259708e+01,     1.5786060802842156e+04,    -5.2959696735243580e-04,    -4.8645816911585939e-07,     2.1789466083786981e-09, } , 
      {    1.0002805541330902e+01,     6.6532750043924098e+03,    -2.7744499229521665e-04,    -2.6211296528809101e-07,     1.2301129049424142e-09, } , 
      {    1.6208316165458612e+01,     6.6229915386017292e+03,    -3.7304293475942017e-04,    -3.7181345376062958e-07,     1.6860850201627441e-09, } , 
      {    8.8000578855684068e+00,    -8.3625084920726731e+03,    -1.2660766106999230e-04,    -2.1312564561953153e-07,     9.5179591502786302e-10, } , 
      {    1.9893611260198995e+01,     2.2547267689603839e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {    9.7276730875568163e+00,     7.4236123310946496e+03,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {    8.1276304236394665e+00,     5.9574589549302236e+03,    -2.1314782090034278e-04,    -1.8983393586345854e-07,     9.3684426110244481e-10, } , 
      {    2.8586844306518744e+00,    -8.4334961676541807e+03,     6.1440216247159225e-05,     5.9179751611997757e-09,    -2.5738758330105242e-11, } , 
      {    1.9426563123423808e+00,    -9.0955551699923053e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.9987229021606829e+00,     9.1665428455738111e+03,    -2.8296862000820193e-04,    -2.6079291459716599e-07,     1.1226878894821633e-09, } , 
      {    1.0751605925168430e+00,    -3.7260638620024110e+01,     1.5466528323684447e-04,     1.8316866889419651e-07,    -8.5884743608554286e-10, } , 
      {    1.1915483012112725e+01,     1.9219628428103253e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.0142384636596233e+00,    -7.7553776367669007e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.4017957565389079e+00,     7.7873766532148957e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.6572985323622085e+01,     2.4674554935530869e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.7263240929265300e+00,     1.1190377864353643e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5393933462385725e+00,     9.9559140512227668e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.6486669552206571e+01,     2.2772816663564139e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    7.2153080969537342e+00,     8.8768169163406856e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7767575068673764e+00,     4.9334893082125436e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2547608930442786e+00,     2.8766924391717594e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3400408141135706e+01,     3.8692592415014922e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.1034391088811464e+01,     3.0363900990391474e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    4.7009076323679508e+00,     6.4345798815715025e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1167837882835217e+01,     3.0235032290763760e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -9.0135908290858193e+00,    -2.4970074765646357e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.4750325687642624e+01,     2.0005670367489227e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9225204615155395e+01,     1.5115951892833647e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -4.4607949834135114e+00,     1.1306280674891029e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4463582589562009e+01,     1.3806377101270280e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.8516671060149585e+00,    -1.7870227185781023e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    6.1711803238554905e+00,    -1.9782227596632047e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3299287597176654e+00,     1.6344543972882220e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7372564685809348e+01,     7.8177872866029547e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1846605702324853e+00,     1.6323321605775172e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.5040906902275797e+00,    -8.9409995014449960e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1074359779949436e+01,     3.2298352916497712e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    1.2174552241681596e+01,     1.6546049160779618e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.3465437087431109e+01,     2.2740817647116142e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8419940179943573e+01,     3.1400977288644452e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.8953605157902800e+00,    -7.8341210706979109e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5441800106476414e+01,     2.4171166679771661e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1395018421187206e+01,     8.6585416974931445e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6677584001478071e+01,     1.6991444092718630e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.8146632977065291e+00,    -7.4227110582901150e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.3847969510167202e+00,    -3.9076100034177193e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.7440927222276974e+00,    -1.6768716537714183e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    7.7715893584895372e+00,     1.5889374593801635e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.9861562235596866e+00,     1.4527984690842397e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.1637762950079038e+00,    -1.6978266003450484e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    1.3158134648311117e+01,     3.2216136647452688e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    7.3706355953507909e+00,    -1.5846429391200622e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5384044582063574e+01,     3.3412981475762819e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.2761400880349910e+01,     2.5214801933623774e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    7.8617571233187196e+00,     2.0885012311401777e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3542939197296576e+01,     7.4519793130270846e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7088772803302476e+01,     2.8334361792112675e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.1727992701920211e+00,     6.5677215963720919e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2397868570147812e+01,     1.1676978942865775e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.6369874103160171e+00,     8.1735869025992015e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0332928934123494e+01,    -3.0284947026287648e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4173369091137731e+01,     2.2921110465279704e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.8479634502852518e+00,     5.4324295208500935e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.4440546170995631e+01,     2.5424351399360075e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    8.3647866734842875e+00,     2.9296615363829505e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2949322034699900e+01,     7.5552687236554621e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.0172633381418475e+01,     2.1890962985590824e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.5695524535485816e+00,     1.6888262222961053e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.6257371267653642e+01,    -1.2259622477323892e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.8764330235376573e+01,     2.1893222954942041e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2962537258819543e+01,     6.5630470401301295e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0380655195889128e+01,     2.1513319896447319e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1682512978423905e+01,     2.5851928443619665e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.6118133504995704e+00,     2.1355585542106888e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9904932061498704e+00,     1.7161178656557015e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    6.2237835073076173e+00,    -9.2598484799318685e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.8667934281436487e+01,     2.2035209565768026e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.5700511193089730e+01,     2.5929850567596754e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.0662070665925594e+00,     2.8237233419605682e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0716345193782342e+00,    -3.4727543774230303e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5873092591310806e+01,     2.0398691337182216e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9448275900916652e+01,     1.5640970067250808e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    4.3844887187607924e+01,    -1.4651982334263257e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3583591998997393e+01,     1.2069999912558764e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.1536391941637170e+01,    -1.5050131353242377e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2788911526588000e+01,     2.9842011321070771e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.3206444661257834e+01,     2.3760112026137958e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    3.7381910385070158e+00,    -8.2304756473544294e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.6367802420127703e+01,     2.8727382761805664e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0324060148226771e+01,     2.5195623739606650e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -4.3125687856530481e-02,     6.2830195521406040e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.1226097389372729e+01,     6.5857609101442840e+03,    -2.1581170239325131e-04,    -1.8921754374929586e-07,     8.8240453285065505e-10, } , 
      {    1.3581651939759531e+01,     1.4914452334767735e+04,    -6.3519294630925112e-05,     6.1501330405358849e-08,    -3.5357946576999568e-10, } , 
      {    5.5402728918331263e+00,     7.7003894694093897e+03,     1.5495628925523469e-04,     2.5010248204049204e-07,    -1.1815442703688610e-09, } , 
      {   -2.5619821267757874e+01,    -2.3011999414102320e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3124288625302718e+00,     8.9569933798375096e+03,     1.4962852626941768e-04,     2.5133526626881740e-07,    -1.2904237268724407e-09, } , 
      {    7.9982533600698744e+00,     7.8423648205724048e+03,    -2.2113946537906831e-04,    -1.8798475952097055e-07,     7.7352507634707552e-10, } , 
      {    1.0353807910456677e+01,     1.6171056245195854e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {    8.2969337648898662e+00,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    1.3624777627616062e+01,     1.4286150379553674e+04,    -6.0855413138016608e-05,     6.0884938291196182e-08,    -2.9913973751820592e-10, } , 
      {   -8.6251375713060963e-02,     1.2566039104281208e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    1.1269223077229260e+01,     5.9574589549302236e+03,    -2.1314782090034278e-04,    -1.8983393586345854e-07,     9.3684426110244481e-10, } , 
      {    1.5937206490146334e+01,     2.3243143759391183e+04,     8.8773113131401069e-05,     3.1222020456001358e-07,    -1.5895634643906465e-09, } , 
      {    7.8958274422199288e+00,     1.6029080894032839e+04,     3.0724869701756089e-04,     5.0082135619514676e-07,    -2.4175282689895120e-09, } , 
      {    8.8705428389859264e+00,    -1.7429305144791660e+03,    -3.6810411015557749e-04,    -4.3993641790395058e-07,     2.1183885314713058e-09, } , 
      {   -2.2281722550678435e+01,     3.2863237477272378e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6679834129170743e+00,     1.7285684804460961e+04,     3.0192093403174383e-04,     5.0205414042347212e-07,    -2.5264077254930913e-09, } , 
      {   -2.3264249975085011e+01,     8.3263902246820398e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1116952198971685e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5833985796896570e+00,     7.0720875141953293e+03,     1.5762017074814322e-04,     2.4948608992632941e-07,    -1.1271045421170713e-09, } , 
      {    7.9551276722133437e+00,     8.4706667757864652e+03,    -2.2380334687197685e-04,    -1.8736836740680787e-07,     7.1908534809528565e-10, } , 
      {    2.1623030987685937e+01,     2.2128515200126079e+04,    -2.8199487851708491e-04,    -1.2709982122977434e-07,     4.7438533882886960e-10, } , 
      {    1.0398780729946738e+01,     1.5542754289981794e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0437708222012283e+00,     7.2140628653583444e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2709362460843479e+01,     2.4499747669819306e+04,     8.3445350145584034e-05,     3.1345298878833894e-07,    -1.6984429208942262e-09, } , 
      {    1.9267476437299134e+01,     1.3799823775502628e+04,    -4.3428728627941109e-04,    -3.7781869538442909e-07,     1.7103693374495204e-09, } , 
      {    8.7842914632728650e+00,    -4.8632660405104525e+02,    -3.7343187314139455e-04,    -4.3870363367562522e-07,     2.0095090749677264e-09, } , 
      {   -3.8338359915708054e+01,     8.3283391086653737e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6191069665179583e+01,     8.3290437405815264e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8338621497967855e+00,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2693031746737415e+00,     9.5852953350515709e+03,     1.4696464477650915e-04,     2.5195165838298003e-07,    -1.3448634551242304e-09, } , 
      {   -3.5349467081839535e+01,    -3.5231595807488247e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0325841452945308e+01,    -1.9524799802154675e+03,     6.4493036122042105e-05,     7.2191762962032783e-08,    -2.9472308488329798e-10, } , 
      {    2.3978585538072739e+01,     3.0457206624749528e+04,    -1.2970247075475876e-04,     1.2361905292488038e-07,    -7.6159865979178123e-10, } , 
      {   -9.7617405204576961e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7578578762912290e+01,     7.2117616654169342e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5980332178002865e+01,     2.2614841804177122e+04,     9.1436994624309573e-05,     3.1160381244585095e-07,    -1.5351237361388568e-09, } , 
      {   -1.5223036394576777e+01,     1.5540453090040384e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.9158499297066925e+01,     1.5545055489923205e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3600257064642083e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0979974236424539e+00,    -6.9587606978734675e+02,     5.9165273136225077e-05,     7.3424547190358117e-08,    -4.0360254138687761e-10, } , 
      {    3.6802936232642338e+01,     7.2163640652997528e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.2314951879916762e-01,    -2.2365715770956740e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0812170828345945e+01,     8.3117707438158577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0652488315276669e+01,     1.6728370524828410e+04,     1.1653693820750085e-04,     2.8239412752857817e-07,    -1.4944333238833335e-09, } , 
      {   -2.1824388474237688e+00,     3.3757047124063320e+01,    -3.0816644950982666e-05,    -3.6447710769397583e-08,     1.7385418604587960e-10, } , 
      {   -2.4023120649955381e+00,     1.5999508223997282e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8292761040533136e+01,     3.1571835184014635e+04,     2.4106552089372724e-04,     5.6293907871466830e-07,    -2.8255474630112975e-09, } , 
      {    4.2223540532269510e+00,     2.6298323161836379e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7222390484849814e+00,    -3.3781427455160383e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.9413792145030637e+00,     7.0987675581507574e+01,    -1.8804787731715151e-04,    -2.1904362078073129e-07,     9.7753467335796819e-10, } , 
      {    1.3667903315472593e+01,     1.3657848424339612e+04,    -5.8191531645108090e-05,     6.0268546177033528e-08,    -2.4470000926641610e-10, } , 
      {    8.4563043894872969e+00,    -1.6920680807593385e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.4816394177886394e+01,     1.5542028288429719e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0849840589882042e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3676311759247710e+00,     8.3294027700903262e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8495059640217431e-01,     8.3279800791565740e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8302895044079408e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1312348765085790e+01,     5.3291569997161632e+03,    -2.1048393940743427e-04,    -1.9045032797762119e-07,     9.9128398935423468e-10, } , 
      {    1.2970221281043222e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0251381992606731e+01,     2.4357772318656287e+04,     4.5954110477988707e-04,     7.5154023034980148e-07,    -3.6535122676101628e-09, } , 
      {   -1.2212676834874140e+01,    -1.5419111139677625e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8395186958383082e+01,     2.3385119110554202e+04,    -2.8732264150290197e-04,    -1.2586703700144904e-07,     3.6550588232529002e-10, } , 
      {    2.1666156675542467e+01,     2.1500213244912018e+04,    -2.7933099702417643e-04,    -1.2771621334393702e-07,     5.2882506708065947e-10, } , 
      {    8.9136685268424571e+00,    -2.3712324696932264e+03,    -3.6544022866266901e-04,    -4.4055281001811320e-07,     2.1728282597230958e-09, } , 
      {    8.6246903710401561e+00,     8.3286914246234501e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2415924870131345e+00,     7.1430751897768369e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    7.0235379633038768e+00,     2.5614376229084410e+04,     4.5421334179407001e-04,     7.5277301457812684e-07,    -3.7623917241137426e-09, } , 
      {   -1.5837317577426695e+01,    -1.8048943455862172e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2752488148700010e+01,     2.3871445714605245e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.6809311553583628e+00,    -6.5589555991336965e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.5995731218265412e+00,     1.4197535116301515e+02,    -3.7609575463430303e-04,    -4.3808724156146259e-07,     1.9550693467159364e-09, } , 
      {   -2.0908867538401470e+01,     1.6655081649305488e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.1649619478297146e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.5149882885991239e+00,    -1.0071621939102613e+04,    -5.2039651791790367e-04,    -6.9065529205860530e-07,     3.3543725300919566e-09, } , 
      {    2.7779338649578929e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9926630010836558e+01,     8.3290200569982226e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3472694008105961e+01,     1.6659684049188309e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6827391218209414e+01,     7.2137105494002708e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2336861806452461e+01,     7.2144151813164190e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4202498346530497e+01,    -4.9310322575947794e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4211929533353445e+01,     1.5542401974023718e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4430557731534179e+01,     1.5543106605939871e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1340315222597159e+01,     2.3452693225127485e+04,    -3.4382403314621852e-04,    -1.9990797630596978e-07,     8.2354815196395740e-10, } , 
      {    6.3542774525474854e+00,     1.7495234270197263e+04,    -1.3067621224587572e-04,    -1.0074040442511271e-08,    -1.1329610913848749e-10, } , 
      {    7.9389531300764595e+00,     1.5400778938818779e+04,     3.0991257851046937e-04,     5.0020496408098408e-07,    -2.3630885407377219e-09, } , 
      {    1.3695869772983961e+01,     3.1781384649750937e+04,    -1.9153162538389234e-04,     5.0810897848684914e-08,    -4.1243584665669338e-10, } , 
      {    1.0310682222600146e+01,     1.6799358200409915e+04,    -7.1510939109650651e-05,     6.3350506747846849e-08,    -5.1689865052536508e-10, } , 
      {    9.5398033497423178e+00,     6.3762114444079825e+03,     2.1678544388436827e-04,     3.2291063711668748e-07,    -1.5307070835039487e-09, } , 
      {    1.6039632407996280e+01,     1.5056427685930750e+04,    -4.3961504926522815e-04,    -3.7658591115610373e-07,     1.6014898809459407e-09, } , 
      {    3.5858246641162612e+00,    -8.2577037490419425e+03,    -3.4034028507947772e-04,    -4.6976249493538599e-07,     2.2135186719786194e-09, } , 
      {   -1.9696122436484615e+01,     1.5732483999299802e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.0945392106300940e+00,     8.1709570702830179e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0365469220738532e+00,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9310602125155665e+01,     1.3171521820288568e+04,    -4.3162340478650261e-04,    -3.7843508749859172e-07,     1.7648090657013101e-09, } , 
      {    2.4021711225929270e+01,     2.9828904669535470e+04,    -1.2703858926185022e-04,     1.2300266081071770e-07,    -7.0715893153999136e-10, } , 
      {    5.6544907250575562e+00,     2.4567321784392589e+04,     2.6943958502267475e-05,     2.3941204948381809e-07,    -1.2404006512555586e-09, } , 
      {   -1.2937706356959144e-01,     1.8849058656421812e+03,    -7.9916444787255359e-06,     1.8491763424879922e-09,    -1.6331918475536940e-10, } , 
      {    3.7557252149598161e+00,     8.4864257789638814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0750741508769885e+01,     3.1713810535177650e+04,    -1.3503023374057576e-04,     1.2485183715320568e-07,    -8.7047811629536076e-10, } , 
      {    1.5064917011230282e+01,     3.2828439094442758e+04,     2.3573775790791023e-04,     5.6417186294299366e-07,    -2.9344269195148769e-09, } , 
      {   -3.0564865352603398e+01,     8.3114184278577814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6334140088459542e+01,     3.8785898049372976e+04,     2.2589937007567433e-05,     3.7433792707953507e-07,    -1.9975826584124323e-09, } , 
      {    1.1276750913807732e+00,     8.3260615923072655e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7251185951129671e+00,     8.3313212569396328e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8666039513302390e+00,     7.2309835461659377e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2008659162594650e+01,     1.6833150887528041e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6338312812816273e+01,     1.5613741965563302e+04,    -2.5423105344098514e-04,    -1.5692589826120981e-07,     5.6951547933618284e-10, } , 
      {    9.7827176005050251e+00,     7.0563285110179122e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2619343314744018e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7098320029342879e+00,     2.5823925694820711e+04,     2.1616195516450453e-05,     2.4064483371214345e-07,    -1.3492801077591383e-09, } , 
      {    2.0306594844791090e+02,    -5.0795863911699257e-01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9222566857697199e+01,     1.6864631115041790e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.5158965812843590e+01,     1.6869233514924610e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.0948730120105504e+01,     8.2949099971682899e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.2376210193318986e+00,    -8.3624728520786120e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.0922557277671284e+02,    -1.7309684772390028e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2819146525367058e+01,     1.2619238417987162e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.5665549993124316e+01,     6.2133149872446666e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4314118417102994e+00,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3282971534939540e+01,     1.4357138055135183e+04,    -2.4890329045516808e-04,    -1.5815868248953511e-07,     6.7839493583976237e-10, } , 
      {    1.3128590371865171e+01,     8.3155735134251827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.3587661062078444e+00,     8.2888765227255371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5212837496198716e+01,     8.3685063265213612e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.3165307866680180e+00,     6.6633075413249253e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8575073264155375e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9842634564357180e+00,     7.2114330330421608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2239644225629213e+01,     7.2166926976745290e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8612055098087552e+00,     6.6916934747351834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5700442192292289e+01,    -8.8601555405439285e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2138529672471432e+01,     1.5385019935641363e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5533728390001467e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2867484066841701e+01,     2.3869144514663836e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.6249984003829667e+01,     8.3512333297556943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4287369128860625e+00,    -8.8150180286744944e+03,    -5.2572428090372062e-04,    -6.8942250783027994e-07,     3.2454930735883773e-09, } , 
      {    4.1514029122044477e+01,     2.3873746914546657e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0776942355392480e+01,    -1.3117911198267393e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6490999979902385e+00,     7.7779361005900309e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4105685182591362e+01,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5729651094468387e+01,    -1.1944470569373675e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.2920254905356501e+01,    -1.7272996765668267e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5278709808451268e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5982806340404778e+01,     1.6657030533288827e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.8546608297355974e+01,     1.6657735165204973e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1623924414919395e+01,     7.1610937681471532e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6913957494104199e+00,     7.1742479634604324e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8438312646239613e+01,     2.2756817155340141e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0668431062166544e+01,     1.4991998965948376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5697025535318147e+01,     9.6640240106353303e-01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.0928921187669232e+01,     7.2538777672562564e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6616920779674061e+01,     8.3131493963350204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.2481485620161038e+01,     8.3442334529118798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6612697574064956e+00,     7.2133515198914674e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7124567799377317e+00,     1.6657382849246900e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.6265242675461868e+00,     6.4437855589812689e+03,     1.6028405224105172e-04,     2.4886969781216673e-07,    -1.0726648138652814e-09, } , 
      {    1.5845762066592977e+01,     7.2147742108252214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8693867363203076e+01,     2.3942433390186754e+04,    -1.0193864567865896e-04,     9.3792975893444940e-08,    -6.6646851928446799e-10, } , 
      {    5.9233059129747367e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2216690828012309e+01,     1.5020384899358634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1805202706857854e+00,     7.8063220340002890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8496945060202734e+01,     8.2490616208276260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.5977655621300116e+01,    -2.4272873609482303e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4068783181889195e+00,     7.3717972196987757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5444900135415383e+01,     8.1578391590847514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6352316752258364e+01,     8.4083212284192741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4868542858093541e-01,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8860379366263320e+00,    -1.1856162348466132e+03,    -1.8272011433133451e-04,    -2.2027640500905660e-07,     1.0864141298615479e-09, } , 
      {    1.2156048261605935e+01,     8.3640284955601728e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1959603274573043e+00,     8.8794467486568683e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.3065587128475187e+00,     1.5540124457665610e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3501627208483655e+00,     2.1329907977851814e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6248577250605436e+00,     1.7913986759675019e+04,     2.9925705253883535e-04,     5.0267053253763480e-07,    -2.5808474537448814e-09, } , 
      {    1.4629617599843527e+01,     1.5545384122297977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3814347053324743e+01,     7.2670319625695356e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8335886728389667e+01,     3.0943533228800574e+04,     2.4372940238663577e-04,     5.6232268660050562e-07,    -2.7711077347595075e-09, } , 
      {    1.0368967140801839e+01,    -2.5807819354295279e+03,     6.7156917614950609e-05,     7.1575370847870116e-08,    -2.4028335663150822e-10, } , 
      {    9.2252196650250994e+01,     7.1985208370699147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7610974475482344e+00,    -1.1146285592651057e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7940488557137968e+01,     8.3654511864939250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.6745877134943754e+01,     7.2296048936467741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9727907030055590e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5971470373999370e+00,     1.5471766614400287e+04,     1.2186470119331788e-04,     2.8116134330025281e-07,    -1.3855538673797537e-09, } , 
      {   -4.1329726166169102e+01,     1.3498673398216670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6704629252406828e+00,     8.8510608152466102e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9072151342157131e+00,     3.1546870868086353e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.1411231114989846e+00,    -1.3241780250014071e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.8735326648459072e+00,     8.2757223274122589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9363374744285494e+01,     1.6799369460072881e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2765741517258302e+01,     7.1344330615625204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.2493976812916898e+01,     9.5171832084254220e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9200949144059656e+01,     1.4916747904877662e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6669819962224608e-01,     1.5502939388083882e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.0163641713531803e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.3656454532080062e+01,     5.5356930186341742e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2520270980128636e+00,     8.3816605218346413e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3716529098196242e+01,     8.5508601287777383e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3970836536020329e+01,     1.5489785192770603e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6911921886912332e+01,     5.4711323508791802e+03,    -5.8657969404173733e-04,    -6.2853756953908375e-07,     2.9463533360701711e-09, } , 
      {    1.6023457865859395e+01,     2.1986539848963064e+04,     9.4100876117218091e-05,     3.1098742033168827e-07,    -1.4806840078870670e-09, } , 
      {    1.1774845260830936e+01,     1.5700488644322226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3279925071975910e+01,     1.5582569191879707e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.0961627171319632e+00,     1.2566151700910864e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6639297511290174e+01,    -1.9488839833353495e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5820740552951918e+01,     1.5542425657607024e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1885279855689532e+01,     1.5543082922356567e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.6268351584480394e-02,     8.3446909328474467e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.0267515492376305e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5179910809040273e+01,     1.4912145504994842e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3167841786457711e+01,     1.6640462168439306e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3465215426551218e+01,     7.2137342329835719e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.8623784454187549e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4240857959076276e+01,     7.2143914977331169e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9664410035612342e+01,     2.9342578065484424e+04,    -5.0047046240324475e-04,    -3.1570097286490760e-07,     1.3023501434277351e-09, } , 
      {    2.0648315590919939e+01,     3.9900526608638080e+04,     3.9335792865605347e-04,     8.1365795286932302e-07,    -4.0615314616319476e-09, } , 
      {    1.3008042865663471e+01,     2.5057061949451858e+04,     2.6882934596982700e-04,     5.3311300168323284e-07,    -2.7304173225039839e-09, } , 
      {    3.2003433304269784e+01,     7.2936926691541685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0793867196626415e+01,     3.1085508579963589e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.4263108284594583e+00,     6.5147732345627765e+03,    -2.7763825076099793e-05,     2.9826077031435431e-08,    -9.5130140507313155e-11, } , 
      {   -4.0656783423869726e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5227112057646348e+00,     8.3110593983489816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2230409205227742e+01,     7.1365106043462201e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0927416984552737e+01,     6.0284466305117321e+03,    -4.0119569821749426e-04,    -4.0887755664418986e-07,     1.9143789344604132e-09, } , 
      {    1.2595144627619050e+01,     7.6328153548361033e+03,     2.1145768089855126e-04,     3.2414342134501284e-07,    -1.6395865400075284e-09, } , 
      {    4.0849742778737443e+00,     7.9099389351456912e+03,    -2.7764085702238487e-04,    -2.6202569882549135e-07,     1.2315673459857430e-09, } , 
      {    1.7311570296303369e-01,     8.3624484717475134e+03,     1.2147576281134351e-04,     2.1427116338525712e-07,    -1.0621298125747714e-09, } , 
      {   -4.5379933978105713e+00,    -8.2949343774993868e+03,    -1.8310905271330885e-04,    -2.8716658492405232e-07,     1.4098381846665302e-09, } , 
      {    9.4616862181966070e+01,     1.5527212261693365e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.6963743237734050e+01,    -7.9587275351851258e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5181864963737077e+01,     8.3124820892827338e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1122154234435424e+00,     9.0279810554190190e+03,    -3.8419351047733836e-05,     3.2291645488086085e-08,    -3.1288905351447236e-10, } , 
      {    8.5713024857089004e-01,     9.1665428455738111e+03,    -2.8296862000820193e-04,    -2.6079291459716599e-07,     1.1226878894821633e-09, } , 
      {   -6.4401196310355090e+01,     1.5558296318270224e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7383130450488018e+00,     6.5864666257796780e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6187128798174356e+01,     1.5595723387192986e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6051424323370764e+01,     4.0110076074374389e+04,    -3.9239217621566155e-05,     3.0152977200333963e-07,    -1.6484198452773442e-09, } , 
      {    3.2989361746707537e+00,     1.6238630359769140e+04,    -1.2534844926005871e-04,    -1.1306824670836598e-08,    -4.4166526349079032e-12, } , 
      {    3.2789763062207662e+00,     8.3284677674657396e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.0201629262917468e+01,     1.4914094388978176e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0856939994947144e+01,     8.3289150817811606e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3699648279376881e+01,     1.8684292215209513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0415159494170242e+01,     1.5463124486185970e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0381664325672933e+01,     8.2588016466046665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2019964585999141e+01,     3.7671269490107872e+04,    -3.4817805464091857e-04,    -6.4982098710252840e-08,     6.6366144807084178e-11, } , 
      {    1.5996506720139749e+01,     1.5684729641144810e+04,    -4.4227893075813663e-04,    -3.7596951904194110e-07,     1.5470501526941510e-09, } , 
      {    1.6082758095852810e+01,     1.4428125730716689e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3652014335747900e+01,     1.4948598067882202e-01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.8616141766239682e+00,     8.0182093328307928e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1597835852395557e+01,     8.4706780354494313e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6888293533777819e+01,    -1.5925960759164900e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3172411753917347e+01,     7.2916151263704687e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6845975835744802e+01,     6.5880564802542103e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5457270819436815e+01,     9.7272932055405181e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2534920895164134e+01,     6.5854029643547265e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8780118738916137e+01,     2.2685829479758631e+04,    -9.6610882692841927e-05,     9.2560191665119606e-08,    -5.5758906278088846e-10, } , 
      {    1.4395656500473891e+01,     2.4709297135555607e+04,    -3.4915179613203559e-04,    -1.9867519207764447e-07,     7.1466869546037787e-10, } , 
      {   -4.7568552623969662e+00,    -8.3126919163994535e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.0322251442492369e+01,     1.0958808674859029e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1757137846157271e+01,     2.2541905132244210e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6751211050860693e+01,     3.3037988560179059e+04,    -1.9685938836970935e-04,     5.2043682077010248e-08,    -5.2131530316027296e-10, } , 
      {   -1.5707767054902483e+01,    -8.6364983828341337e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9794898145335239e+01,     6.6653850841086260e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0800821090173152e+00,     6.5857552803128010e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5003742988046200e+01,    -5.9000540852293000e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.4871486632302961e+01,    -2.0571522602131154e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7535398032794880e+01,     6.5834540803713917e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6095753672629876e+00,     8.2511391636113258e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5258880046372486e+01,     3.2200137139228696e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.8286942489687128e+00,    -1.0281171404838919e+04,    -8.7799371640284076e-05,    -1.7852711119262194e-07,     9.4126091373735280e-10, } , 
      {    8.5798419878277841e+00,     1.6833175267859137e+04,    -9.7195685801984544e-05,     2.5757278212723649e-08,    -2.3271056693257730e-10, } , 
      {    7.4893555924951105e+00,     8.4062436856355744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4371608393009041e+01,     1.5622384093777619e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6433633048123184e+00,     7.2206218209574781e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4230828770941759e+00,     2.5132303401821728e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7239949036958315e+01,     7.2313358621240131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2077954212382085e+01,     1.4915158050403128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4154734847991917e+00,     7.7003838395779067e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0614200324745021e+01,    -1.1489987856728476e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2261774868172113e+00,     1.0213597290265632e+04,     1.4430076328360064e-04,     2.5256805049714271e-07,    -1.3993031833760202e-09, } , 
      {   -3.8133131580193890e+01,     7.7000315236198321e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7330266485597168e-01,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7228528954892042e+00,     1.6658094194713776e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.8400768700234709e+00,     1.6656671503780024e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1536935883621091e+01,     6.1163549869008320e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2824277624783132e+01,    -1.6209335340717018e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0688419366658739e+02,     8.3269604561462111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1783676930378054e+01,     2.3871093398647172e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.6713220580971054e+01,     2.3871798030563317e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3106296059156687e+01,     4.0042501959801099e+04,     1.7262174021750411e-05,     3.7557071130786043e-07,    -2.1064621149160116e-09, } , 
      {    2.1709282363398998e+01,     2.0871911289697960e+04,    -2.7666711553126790e-04,    -1.2833260545809968e-07,     5.8326479533244923e-10, } , 
      {    1.3711029003329124e+01,     1.3029546469125553e+04,    -5.5527650152199586e-05,     5.9652154062870862e-08,    -1.9026028101462631e-10, } , 
      {    8.0100452754443587e+00,     3.2896013209016040e+04,     1.7923636626459365e-04,     4.9013092363847281e-07,    -2.4763846498762099e-09, } , 
      {    8.9319904212066419e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1670131809022752e+00,    -1.7632026274469709e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3889039353360953e+01,     1.4445046411524281e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3664860268495840e+00,     7.7364322559815055e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5004511610624611e+00,     7.2478442928135055e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.5316861483123054e+01,     7.8423704504038878e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0599552514437640e+01,     6.0334744427518672e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.6871914947700176e+01,     7.2139648137845052e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0609629983225594e+02,     7.2141609169321837e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4564615550609266e+01,     1.5465202028969670e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3352177881722572e+01,     8.3200549262406166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.5389431789371582e+00,    -1.2903965975462468e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2640735821633625e+01,     1.6171061875027337e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2165497866578612e+01,     1.4994076508732076e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1149950444220366e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6377265776316072e+01,     3.8157596094158915e+04,     2.5253818500475944e-05,     3.7372153496537244e-07,    -1.9431429301606422e-09, } , 
      {    7.0548717357859232e+00,    -6.7574114573286352e+01,     5.6501391643316566e-05,     7.4040939304520783e-08,    -4.5804226963866738e-10, } , 
      {    1.2606936542993534e+01,     3.2686463743279739e+04,     6.1183351254221320e-04,     1.0022591045044562e-06,    -4.8894962662308132e-09, } , 
      {    9.1368575695704919e+01,     1.5275583779575754e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4064672264747031e+01,     7.7800136433737316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.7008209536188472e+00,     6.0104167924784488e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0014996414808490e+01,    -1.3971962187754312e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.5287553526921311e+01,     6.9596314256824407e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7880541053964194e+00,     7.1927329573804927e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1434188207632165e+01,     7.2353927733361961e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2748306611569646e+01,    -3.1480227513751743e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.0989580133621867e+01,     8.0310922092562214e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3898156934007366e+01,     8.2353914200968757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0568382464745358e+00,     1.4339108217101899e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4606074571460560e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.5538168150677521e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5453525821073114e+01,     8.2716035259598684e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4218940666009159e+00,     1.4128404765550276e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0506125578436283e+00,     7.7010951850447836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5546144464370268e+00,     7.2271807765566118e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4541599862105684e+01,     6.2795526908746854e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4450087576934678e+01,     2.3243138129559702e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4732377652441983e+01,     1.4198661082598073e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5463214554241310e+01,     1.5620306550993919e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8352061270526551e+01,     2.4013421065768263e+04,    -2.8998652299581045e-04,    -1.2525064488728636e-07,     3.1106615407350025e-10, } , 
      {    1.0841165608839676e+01,     7.2850505409398529e+03,    -4.0652346120331132e-04,    -4.0764477241586450e-07,     1.8054994779568337e-09, } , 
      {    5.1984667991566038e+00,     7.7713771449908972e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1065386553321090e+01,     3.4152617119444163e+04,     1.7390860327877664e-04,     4.9136370786679817e-07,    -2.5852641063797892e-09, } , 
      {    4.0418485900172136e+00,     8.5382408903597516e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.7123889803846897e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6453556705929635e+01,     6.9994463275803537e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0689604742045065e+00,     5.7997133409268263e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1176606224015829e+01,    -2.1422382218208291e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.7731494932901768e+00,     8.9562876642021165e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6867014560112953e+01,     2.5193322539665241e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    1.0134346931419024e+01,     3.1415379252277162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9396489648976076e-01,     1.0309281255487740e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3221303336308374e+01,     7.6980826396364973e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.9563528078455654e+00,     8.5044594629045914e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.7514526434587353e+01,     2.5197924939548062e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    4.9381553262856848e+00,     6.8127668225673460e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2634429205202036e-01,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3608551688149172e+01,     2.8168728122570292e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.9810559337540234e+01,     8.4292412282353416e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0606417040599911e+02,     2.0355170170754775e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7308855485225539e+01,     2.1013886640860976e+04,    -6.5276287016557098e-04,    -5.6641984701956232e-07,     2.5383341420483859e-09, } , 
      {    9.3790925136906793e+00,     3.3943067653707862e+04,     6.0650574955639625e-04,     1.0034918887327816e-06,    -4.9983757227343934e-09, } , 
      {   -1.7279889169559741e+01,     7.7690759450494879e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.7101623049269293e+01,     7.7736783449323084e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1450940352114234e+01,     1.2395292688527138e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.5446202944097420e+00,     1.2036460728798950e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3414819806382710e+00,     1.8849227551366296e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5175596046951684e+01,     8.9438810984707270e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6321541028383155e+00,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6190748257057646e+01,     8.8773692058731685e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0928211811960381e+01,     6.7051999860065380e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.1816519429979486e+00,     2.2754515955398729e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.4567706266777209e+01,    -8.3441105357631277e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.0166674415067931e+01,     8.9546978097275824e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.7199873546333180e+01,     2.2759118355281553e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0673208382456119e+01,     6.7446625719463755e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2055595701295861e+01,    -9.5601204467984644e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0555792388904131e+00,    -8.6359420039809891e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9599614891665031e+01,     8.0742599849475464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3738995460840492e+01,     3.1153082694536875e+04,    -1.8886774389098381e-04,     5.0194505734522254e-08,    -3.5799611840490362e-10, } , 
      {    8.4111515981142961e+00,     2.5266611415188159e+04,    -1.6376780030779255e-04,     2.0984820817249487e-08,    -3.1730570614938024e-10, } , 
      {    1.0868497053795018e+01,     8.3647398410270489e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3310759524474157e+01,     8.3908245744958967e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6152393730669402e+01,     2.0775427837006646e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4929170253215283e+00,     1.0447387812463210e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3083981913290245e+01,     1.5542042944514918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8553312990261499e+01,     2.4983773073928940e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -1.8190607525394078e+01,    -8.3467403680793141e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0777642317720222e+01,     8.1140748868454593e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9111396362237215e+01,     4.8701865814555276e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.2939500977229152e+01,     7.1915209602261002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7083784837415642e+01,     1.5543465635448671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.3404511385082643e+01,    -8.1204563989740564e-01,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    1.0463517924791605e+01,    -8.3160721862054634e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1714513100408258e+01,     1.6065123680604955e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5174610838518293e+01,     8.3413106630414368e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.0091171744399285e+01,     3.5923954223782710e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.2493972742255501e+01,    -4.2577556577234645e+00,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.3386646698209724e+01,     8.2092467189297131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7642894787449482e+01,     1.5288322850305891e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3215401390019561e+01,    -4.8632097421956246e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.5828273151564133e+01,     2.4988375473811760e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1746948992864578e+01,     8.5831228642993538e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8689694638846344e+01,     4.7114589473996428e+04,     1.7488234476989361e-04,     6.2505680123418984e-07,    -3.2335666570330831e-09, } , 
      {    1.1383440910453690e+01,     2.2824391269913423e+04,    -3.4116015165331005e-04,    -2.0052436842013246e-07,     8.7798788021574726e-10, } , 
      {    1.1355474452942321e+01,     4.7008550445021028e+03,    -2.0782005791452577e-04,    -1.9106672009178387e-07,     1.0457237176060245e-09, } , 
      {    6.5581139764556546e+00,    -1.0699923894316675e+04,    -5.1773263642499508e-04,    -6.9127168417276798e-07,     3.4088122583437467e-09, } , 
      {    9.5829290375988485e+00,     5.7479094891939221e+03,     2.1944932537727680e-04,     3.2229424500252485e-07,    -1.4762673552521590e-09, } , 
      {    1.0294507680463262e+01,     2.3729470363442226e+04,     4.6220498627279555e-04,     7.5092383823563880e-07,    -3.5990725393583728e-09, } , 
      {    3.0134893197926690e+01,     1.6657054216872126e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.5052756693698477e+00,     8.2937465356140583e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7571075462521055e+01,     1.6657711481621671e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.7527185830463040e+01,     8.4481361303171871e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2926658152274186e+01,     8.5433079624014408e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5775500852830554e+01,    -8.2483293269717251e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8007115004570942e+00,    -6.6208901250070357e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.5414276063563523e+00,     8.4066883115158762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7358441091347050e+01,     8.3227913705382216e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.5768389247227255e+01,     4.2923307555598194e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8808977989361580e+01,     1.5328137752203804e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0146056815960266e+01,     1.3763769729267548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5406305476886946e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9245359156736292e+01,     1.8055984630163966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0234366268796276e+00,     7.8592911312114811e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.2154576793289927e+01,     7.6329431902770966e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4335526735364406e+01,     2.5161842312151493e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.4216089323590253e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6590617413189026e+00,     3.5337070936721659e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.7340305167502549e+01,     3.5902950880144147e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5225083244523333e+01,     7.8198285452716436e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.7071015419848017e+00,     8.9569990096689926e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0141942235539918e+01,     1.5521424382003943e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3288963706022919e+01,     1.5033891410629989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4467921578075867e+01,     3.7698455102732592e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4112097428916604e+02,     1.6219053161767079e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4064836913785800e+01,     2.9200602714321409e+04,    -1.2437470776894172e-04,     1.2238626869655504e-07,    -6.5271920328820160e-10, } , 
      {    1.6825670511199270e+01,     6.7277362613073010e+03,    -5.9190745702755428e-04,    -6.2730478531075839e-07,     2.8374738795665917e-09, } , 
      {    1.6381438500672804e+01,     1.4985440010349243e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.7420471561617084e+01,     4.1157130519066202e+04,     3.8803016567023641e-04,     8.1489073709764838e-07,    -4.1704109181355277e-09, } , 
      {    1.2666236772986949e+01,     2.5128049625033367e+04,     8.0781468652675530e-05,     3.1406938090250157e-07,    -1.7528826491460159e-09, } , 
      {    8.9567942146989878e+00,    -2.9995344249072868e+03,    -3.6277634716976048e-04,    -4.4116920213227588e-07,     2.2272679879748855e-09, } , 
      {    4.1594337382123214e+00,    -1.8400313363726069e+04,    -6.7268892568022990e-04,    -9.4137416621326002e-07,     4.5903565287126083e-09, } , 
      {    1.1895357900129120e+01,     1.4704902869031434e+04,     3.6907785164669451e-04,     5.7362951127134225e-07,    -2.7666910821245997e-09, } , 
      {    2.4731396985819103e+00,    -1.8609862829462370e+04,    -2.4009177940261027e-04,    -4.2924598534727668e-07,     2.1772449123580036e-09, } , 
      {   -1.4121729261028515e+00,    -4.1875248947775890e+02,    -4.2993326478471105e-04,    -5.1274457298014607e-07,     2.4675513446063935e-09, } , 
      {   -5.3307470300931385e+01,     7.2347941712979718e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3791795151534783e+01,     1.5564084197959646e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7340518889562574e+01,     8.3444239086227499e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.1060801306253438e+00,    -8.3456121054310406e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -8.3644312007849180e+01,     8.3044185510139669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0911840241010085e+00,     6.5327861831016098e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6902776380483981e+00,     6.1927343106198141e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5405327855462687e+01,     1.9170613189429074e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.4457608691759489e+01,     6.8366451523188516e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7780914240451025e+01,     8.3529642982329315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3023788274431579e+01,     1.5073353996569826e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7617191385356115e+01,     5.5228953248342805e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3118390060803286e+01,     8.1937046906412834e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2443998128710378e+01,     3.6759761870474335e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.4386881916239716e+01,     1.4914799020894327e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7232055700273950e+01,     7.4286794031363352e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8853362413619280e+01,     1.5525833609174202e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -6.0664897628341947e+00,     7.4684943050342481e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7395235391192121e+00,     5.2165804515628406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2462924794265149e+01,     1.4920438359396940e+01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.7882549388696098e+00,     5.4350783046440974e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8775683198250100e+01,     7.0790761313761777e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1939849654007721e+00,     7.3849151308970431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1949944928089479e+01,     8.1538897887433714e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1047432697425515e+01,     1.4521425735243263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0845190836239503e+02,     1.5542852341555634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.4516589448845082e+01,     1.5542656238407953e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7615525520407687e+01,     7.0392612294782657e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8732515885528946e+00,     7.3073628698849197e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.5588194823189205e+00,     6.7434896346532323e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.8984053948368619e+01,     8.4636781586056168e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.5359134224878179e-01,     5.8564776908983958e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0089925886017674e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9121095296476579e+01,     1.5508972862526634e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -2.1848186580623569e+01,     8.3237603923658535e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.1483544805186696e+00,     1.9187669756030985e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8158025899271264e+00,     8.7980917180354190e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4375519136385947e+01,     4.5999960914731324e+04,    -1.9588564687859238e-04,     1.8573677544440189e-07,    -1.1696178538135666e-09, } , 
      {    2.1737248820910366e+01,     3.8995447515109277e+04,    -4.1000720927005213e-04,    -1.3779025378644829e-07,     4.1552895794217190e-10, } , 
      {    1.9353727813012195e+01,     1.2543219865074509e+04,    -4.2895952329359408e-04,    -3.7905147961275440e-07,     1.8192487939531000e-09, } , 
      {    1.8367220500871714e+01,     5.2615828851428769e+03,    -1.5398254776411772e-04,    -1.1640938867310041e-07,     5.3324171971556735e-10, } , 
      {    2.1049421913589878e+01,     3.2271124814810202e+04,     5.0353762083667226e-05,     3.4451185004809966e-07,    -1.9024525179051186e-09, } , 
      {    1.3326097222796070e+01,     1.3728836099921122e+04,    -2.4623940896225960e-04,    -1.5877507460369779e-07,     7.3283466409155224e-10, } , 
      {    1.2302701137294589e+00,    -1.6586395173665391e+04,    -4.9263269284180379e-04,    -7.2048136909004076e-07,     3.4495026705992703e-09, } , 
      {    1.6431683517738807e+00,     8.3785142095036190e+02,    -4.3526102777052811e-04,    -5.1151178875182071e-07,     2.3586718881028141e-09, } , 
      {   -2.7815682379019979e+01,     8.5034930605035279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9790080935006209e+00,     1.6496361397840737e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4478153436135511e+01,     6.6812248694345544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4285219306250866e+01,     8.2335195925391963e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2924553414753106e+00,     7.2839526433771289e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1983650417887981e-01,     4.1643119620077772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8506958382285049e+01,    -3.6082627396572207e+01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.7361845733354251e+01,     8.8375543039752556e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.6617311598590064e+01,     8.2317886240619573e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.3503225753735046e-01,     4.1948470488827297e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4083426044902456e+00,     1.4861477607725061e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1217648581182260e+00,     1.1499656302018273e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7058792391718249e+00,    -1.7429361443106488e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3956028176500148e+01,     8.4177089382382401e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9537600363311462e+01,     7.1802814379031852e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -3.0103789342346321e-01,     7.0720762545323632e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9453081234110492e+00,     8.0132227159425856e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.0150791880791289e+01,     8.4238632567077038e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6571152907856369e+01,     9.0989856204949738e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0214427442506899e+00,    -1.2451519477308569e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.3912665554460141e+01,     7.0946181596646074e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.1248294216507475e+01,    -1.0636614249512490e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0407339860530529e+01,     1.5576535717436956e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.7900738566802147e+01,     7.1897899917488630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5451108481571737e+01,     8.2733344944371092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2630187609463217e+00,    -1.4226909337526763e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9015862875517513e+00,     1.1513883211355801e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5070245697296713e+01,     1.9312599800255055e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9935121268258797e+01,     7.1188910332740907e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.1316378689230433e+01,     8.3840483548097927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0392081343626970e+01,     1.6650823893647768e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.1129757259576056e+00,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.8565099131259206e+00,     8.3132723134837725e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5984496019616536e+01,     8.3336224568810467e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6666312273319992e+00,     1.7789845607142468e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1819525843374521e+00,    -1.9187041825917900e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0341112929348900e+02,     7.2383357389678267e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5240307799748347e+00,    -9.4433199838885539e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1910058557281687e+01,     1.5072181059276681e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9806857215134098e+01,    -1.3506511271354015e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.5371966990144479e+00,     1.4425824530775280e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3286459092307608e+00,    -8.3352503802225838e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.1446436067817110e+01,     3.4227046505191305e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5139376471568859e+01,     6.5181867955709977e+03,    -1.5931031074993473e-04,    -1.1517660444477507e-07,     4.2436226321198777e-10, } , 
      {    1.2382715598512119e+01,     5.8188971647754306e+03,     3.1401448060125284e-05,     1.0325062422179354e-07,    -4.9873268189419070e-10, } , 
      {    8.9847606722103563e+00,     1.5124001800504037e+04,    -4.9611644090854471e-04,    -4.5062685046062452e-07,     2.0595321505846083e-09, } , 
      {    3.2440185714397392e+00,    -8.1867160734604331e+03,    -5.2838816239662921e-04,    -6.8880611571611726e-07,     3.1910533453365872e-09, } , 
      {    4.0731823624992600e+00,    -1.7143709453297946e+04,    -6.7801668866604686e-04,    -9.4014138198493466e-07,     4.4814770722090281e-09, } , 
      {    1.2638270315475580e+01,     7.0045133996220429e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    8.3860247414355982e+00,     2.0426571094770613e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0166869315342623e+01,     6.2702865716319375e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6562705583818151e+01,     9.3369020681637139e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0528114645754020e+01,     7.7627406466080629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.2740822404207908e+00,     9.4263993030809634e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.1094018924176268e+01,     7.1587059351720027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3435270798936964e+01,     1.6623601421791740e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    4.0130525973897537e+00,    -1.6691164276702060e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    8.6557024604964766e+01,     6.5702132520243713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2051660201987033e+01,    -8.3129589406241503e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.4844314643700798e+01,     1.4430426930658101e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -7.9983207796580700e+01,    -7.0463191614976495e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8215183523551921e+01,     1.6640109852481233e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7796870996570192e+01,     1.6486530583708201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1918910820006001e+00,     8.6441601333043145e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5261188216497814e+00,     7.1787257944216226e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4831395830364684e+00,     1.6654753016930714e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.0804863436600272e+00,     1.6660012681563087e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0786778185775705e+00,     8.3073615166455984e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.7730084771827581e+00,     8.3500213326013018e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6406418388276940e+01,    -3.7387614536963997e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3183681870857754e+01,     1.6093509614015213e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.3272134414678263e+01,     7.3490495993405111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0511930667018749e+01,     3.2197835939287281e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -1.2604509044496188e+00,     7.1773031034878695e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6181130179345367e+01,     1.8362316315483440e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.1560221297433593e+00,     7.2316948916328138e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6803869110743896e+00,    -5.3680442678067706e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5421717870981340e+01,     1.0973555624994374e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5862081696980308e+00,     1.4286139119890708e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2526119866837039e+01,     1.4598977996255388e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.8261022203628308e+00,     3.1083207380022177e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    5.4101336786596725e+00,     4.3770833556963680e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.3869529428589331e+01,     3.2202438339170101e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.4545509549444082e+00,     1.6499648494906469e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.6930158811604414e+01,     3.8941817365806946e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9555386828535333e+01,     3.1087809779905005e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.3479884758082541e+01,     8.3106424811675879e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.4431600849282724e+01,     7.3092346974425982e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9971264686346402e+01,     1.5367952654101717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1133058610303152e+01,     1.5407767555999628e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.5590017192510494e+01,     7.2694197955446853e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3118736518298711e+01,     2.2291439598433499e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -7.4254098799647323e+01,     8.3279867927073010e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.8389929939128763e+01,     8.3293960565395992e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4494551524660281e+01,     2.3713711360264813e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6459629233611956e+01,     1.7427677045118424e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9792513668024081e+00,     5.7533848783435178e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8882119841711997e+01,     1.4898904676647822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5463685823615569e+01,     1.6644264938048633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2622602200353287e+01,     5.9452615759038281e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1816838995938408e+01,     6.5883851126289865e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3869771302491763e+01,     3.0943521969137611e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.4972890103767238e+00,     7.2302722006990607e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1513751466627586e+01,     7.5226051193081457e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5047592597276949e+00,     5.6431786111198778e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1728774602915990e+01,     7.7229257447101518e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3886389316836492e+01,     1.0983908375631603e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2111208244305192e+01,     7.3888645012384241e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3162881136538061e+01,     7.3335075710520814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7489907012679147e+01,     3.1571829554183154e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.5884810637329689e+01,     1.6867260947357972e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.1821209571727518e+01,     1.6866603682608427e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -4.8049660944924156e+01,    -1.9907450948956102e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7998634730504310e+00,     2.4356780791700485e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0646357940880405e+01,     7.1081246709359621e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2123757011185852e+01,     1.6168760675085927e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.6151673721372548e+00,    -5.0435465816027181e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.1065391790791255e+00,     1.6815117203587331e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.6540509744138738e+00,     1.0213614179760079e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.5093591433948902e+01,    -1.2205924723978569e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.1715498065184597e+00,     5.9574476952672576e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2089645327509626e-01,     7.6474147423667155e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2174324221176562e+01,    -7.9853460953534523e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.5860171700661159e+00,     1.3367972634193943e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2161110016883026e+00,     4.6400665873147773e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8827296833556069e+01,     1.0355600790586061e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2063090273855671e+01,     3.7042967534893811e+04,    -3.4551417314801009e-04,    -6.5598490824415507e-08,     1.2080587305887397e-10, } , 
      {    2.9707535723468872e+01,     2.8714276110270366e+04,    -4.9780658091033616e-04,    -3.1631736497907023e-07,     1.3567898716795247e-09, } , 
      {    1.9381694270523564e+01,     3.0666756090485829e+04,    -5.6229961703237836e-04,    -3.8850912794110304e-07,     1.6515129565628228e-09, } , 
      {    2.3149421747013218e+01,     3.9414200004587037e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.8736993051059606e+01,     2.3314131434972693e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.9106765601247496e+01,     4.1366679984802511e+04,    -4.4566980607383177e-05,     3.0276255623166499e-07,    -1.7572993017809239e-09, } , 
      {    1.5720965099326769e+01,     1.5508997242857731e+04,    -3.5366531172850963e-05,     9.8565433288919105e-08,    -5.8187338006766507e-10, } , 
      {    9.9527015877867395e+00,     2.3800458039023735e+04,     2.7415710895564407e-04,     5.3188021745490748e-07,    -2.6215378660004046e-09, } , 
      {    7.4677699738303449e+00,     1.7356672480042471e+04,     1.1387305671459234e-04,     2.8301051964274080e-07,    -1.5488730521351231e-09, } , 
      {    2.8429122487698013e+00,    -5.5731427963255283e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.2232850412577445e+00,     6.7514094248126639e+01,    -6.1633289901965333e-05,    -7.2895421538795167e-08,     3.4770837209175919e-10, } , 
      {    7.0666636511604075e+00,     2.4986074273870348e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.4880368425036707e+01,     1.5757370827759785e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3087923162832395e+01,    -1.7085226553869916e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5093084783137472e+01,     8.8871092316502090e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9667683039900066e+00,     6.9918941612040562e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0061560621399046e+00,    -6.2567775272935887e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1320083117112761e+01,     1.5542530632824084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8898562040283036e+01,     1.5542977947139501e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4635804410474940e-01,     6.4437686694868207e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.7113876394782181e+00,     1.5797185729657698e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2534431212165140e+01,     1.1035273942083563e+00,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {   -6.5403920979117331e+01,     2.3878329751503742e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1878032368190638e+01,     8.6056647694315961e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9642844842333798e+00,     7.2138392082006349e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4707999075430411e+00,     3.9302096969298748e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0672235518200832e+00,     8.4346296190458324e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.5268870904991960e+00,     2.1228392028053695e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0246469862993397e+00,    -6.9889778018784739e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1499922739966228e+00,    -3.4944889009392369e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.8500115320044603e+00,     6.5459403784148890e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6543284305300599e+01,     7.2142865225160531e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4411029057772769e+01,     3.9324626244204808e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1562767182781968e+01,     1.5423309584288058e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0915174859960425e+01,     8.3985212684822091e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.3774055443414667e+01,     6.0403471143606112e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9215432996419359e+01,     1.6879551553401190e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2080536515211683e+01,     6.7450148879044500e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1523663578955601e+01,     1.4914446704936252e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6375657344394288e+00,     1.1015106480668383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5823031597641098e+01,     5.9972625971651705e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.5555930365327441e+01,     1.5518481416372311e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7532872130664883e+00,     6.2901893051241950e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3866298779854912e+01,     6.9787762500057888e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1980610640711381e+01,     6.1693240841120232e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4256091653652813e+01,     1.4917076537252435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1176349144419664e+00,     1.5558753798205793e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3018843153169009e+01,     2.3320690390571825e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0542150830604257e+02,     8.3281834659843353e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0071039920526894e+02,    -8.3291993832625649e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8173119560691596e+01,     1.4325954021788621e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2075470929298895e+00,     9.5853065947145369e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2294566248234830e+01,     1.5447582457897541e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6200006470673001e+00,     6.9143419001919319e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1881097380171180e+01,     6.8985941566774809e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8104366119127363e+01,     8.3931257744373070e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1567496069282988e+01,     2.3245439329501114e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6811733570127913e+01,     1.9045571086917334e-01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.4958341557632519e+01,     8.0569869881818795e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8387251143975952e+01,     6.4109536444327223e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3882278592701503e+01,    -1.6756516847296916e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6014145531123539e+01,     1.1470229349851166e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6831796602397233e+00,    -1.4841705721395204e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6129023111593504e+01,     8.0171720862839666e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3628126130018677e+01,    -1.1139172137982293e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.1693709226735471e+00,    -1.1153399047319820e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.1849905603113783e+01,    -1.1142762433070307e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.2256964735757506e+01,     1.6173363074968747e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5817793970960324e+01,     1.5526754781757798e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4336000114094658e+01,     1.4498015508735472e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2562765229751207e-01,     1.4874631803038339e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1044500887092742e+01,    -1.1149808752231806e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.3207471702343106e+01,     7.5251061070712240e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0578100761266194e+02,     1.5567027163591278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.6262501695222591e+00,     1.2168002681931741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1216418069614940e+01,     2.8286990488630363e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8021924694100726e+01,    -8.2665582747510034e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.8002395388305530e+00,     6.5850439348459240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4571996469437430e+01,     2.3349076323982084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.7612250070653683e+00,     7.2300623735823419e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.1624275251020975e+01,     8.3290504541322498e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5760162312563963e+01,     8.3283323951146504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.0090470232899200e+00,     1.6617567947348987e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3977763121151492e+00,     7.8581181939183380e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3019343551429539e+01,    -8.1867048137974671e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1639484177519746e+01,     2.3868815882289062e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.1948588281118035e+01,    -1.6003555428184336e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7574312279963127e+01,     1.6697197751144813e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.5253167099565896e+01,    -1.2201333323604814e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7008031816367712e+01,     2.3874075546921427e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.1844312582565557e+00,    -8.9737562785325929e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.5335658350796663e+01,     8.9566466937109180e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8342941783867994e+00,    -7.6441243410864018e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.6047085442432490e+01,     7.7007361555359812e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1578079668756128e+01,     8.5359396904183413e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.6174173192308736e+01,    -1.0903248852587421e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.3455736428593383e+01,     1.5487397359795452e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0226349131453603e+00,     6.4280209259723688e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9495117048510458e+01,     6.8692105471578543e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.6600956479533799e+00,     4.2659815955703628e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0396532547162991e+00,    -1.1133368846728445e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5036554951466529e+01,     6.7470825091869074e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1498246767996209e+01,     1.4756712350595821e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1297561660868524e+01,     6.6394357229908692e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2824358530368869e+01,     2.3240836929618294e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.0729472025747953e+01,     1.4739645069056172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0692323092002525e+00,     1.3029523949799623e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7984659290797943e+01,     9.7299230378567008e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2803404291255859e+01,     8.5405420903011618e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    8.2531314366337369e+00,     1.6604413752035707e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.6326885190907788e+01,    -1.3791187798138708e+01,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.3627249901024243e+01,     2.4985721957912276e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.0902207278533751e+01,     2.4986426589828421e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.6642483781270073e+01,     1.9997357440854557e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8792120556696290e+01,     3.8927873400535995e+04,    -3.5350581762673563e-04,    -6.3749314481927506e-08,    -4.2513311696495410e-11, } , 
      {    2.6436566006309487e+01,     3.0599181975912546e+04,    -5.0579822538906170e-04,    -3.1446818863658224e-07,     1.1934706869241553e-09, } , 
      {    2.4092803371297169e+01,     4.7324138939732729e+04,    -2.5771480150772595e-04,     1.1292862036820643e-07,    -8.2045504067847898e-10, } , 
      {    1.6955047574768862e+01,     4.8428303956651198e+03,    -5.8391581254882874e-04,    -6.2915396165324643e-07,     3.0007930643219611e-09, } , 
      {    1.8823244426772668e+01,     2.2057527524544574e+04,    -9.3947001199933423e-05,     9.1943799550956940e-08,    -5.0314933452909859e-10, } , 
      {    1.8406978873757566e+01,     4.8438767498997840e+04,     1.1305319014076003e-04,     5.5224864615799435e-07,    -2.8844038438979950e-09, } , 
      {    1.6794336738717224e+01,     3.2409686604964998e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.4352530812617360e+01,     2.5337599090769669e+04,    -3.5181567762494406e-04,    -1.9805879996348179e-07,     6.6022896720858800e-10, } , 
      {    8.2144947508894397e+00,     1.5576511337105858e+04,    -9.6999821074816303e-05,     2.5670011750123932e-08,    -2.3416500797590593e-10, } , 
      {    2.0691441278776470e+01,     3.9272224653424018e+04,     3.9602181014896195e-04,     8.1304156075516034e-07,    -4.0070917333801583e-09, } , 
      {    1.3365410548939966e+01,     7.1803058182342811e+03,    -1.8765893893517714e-04,    -1.5215344086573563e-07,     6.5411061855298576e-10, } , 
      {    4.6993171853991207e+00,    -8.3962655391967364e+03,    -9.5791016119009615e-05,    -1.7667793485013395e-07,     7.7794172898198340e-10, } , 
      {    1.0695614003133199e+01,     1.6100068569614348e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9857820001989113e+00,    -9.7238571252063666e+03,     9.7584624183958920e-05,     4.1132901702272025e-08,    -9.0713487872405258e-11, } , 
      {    6.3111517646909547e+00,     1.8123536225411321e+04,    -1.3334009373878425e-04,    -9.4576483283486060e-09,    -1.6773583739027728e-10, } , 
      {    7.9820788179329893e+00,     1.4772476983604720e+04,     3.1257646000337790e-04,     4.9958857196682140e-07,    -2.3086488124859323e-09, } , 
      {   -2.7494687029754846e+01,    -4.1369104726755222e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8176287365612602e+01,     2.3871117082230470e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.5297156583291169e+00,     2.3871774346980023e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.7183307711320616e+01,     6.3711387425348103e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0587277696708428e+01,     1.6710351946458093e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0022436351465380e+01,     1.4363691380902832e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2551434539496412e+01,     7.7710248290328227e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.4203309315042191e+01,     8.5260349656357739e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6089897084144297e+01,     7.7717294609489718e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4652126838040218e+01,     3.6027960349179011e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6872359323012143e+00,    -8.6750524666351751e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.0926492132364920e+01,     1.5677741023963963e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4420954742056381e+01,     7.8427227663619624e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1737474898387124e+00,     7.8604193938597496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6385902799385732e+01,     7.8596434471695557e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1720705341719107e+01,     2.5128060884696330e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.8303534989211414e+00,     6.5831254479966174e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8055908243835013e+01,     5.6074473013522243e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.6586491138296072e+01,     7.8420181344458133e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5543531152470763e+01,     1.5662198995675531e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.2971886992714730e+01,    -3.0592046999136073e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4456549478520124e+01,     7.8400692504624767e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4438966032014690e+01,     1.5073706312527900e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1527319895967263e+01,     7.7029419587152393e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.1557706671922432e+00,     8.2227532302010677e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7961283027899974e+01,     1.9940907385300598e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5712802144520989e+01,    -7.6872182474011197e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.9169624178103817e+00,    -6.6113995521056310e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.7016870683201596e+01,     7.2321118088142066e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3759343176240559e+00,     8.4308631335515183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1952273151496606e-02,     7.1250453517435544e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4362536079763416e+01,     8.3129436811089981e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2936658584028450e+01,     1.1452814816238019e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.5100823302923354e+00,     9.5559973877790526e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2634975006603565e+01,     3.2217200185560241e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9486244563953274e+00,     2.5158601724983564e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5459999179060322e+00,     6.6380130320571161e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0995053177187771e+01,     7.6848418112894769e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4557436860163122e+01,     8.1694318170318002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0675297220080175e+01,     1.7285690434292443e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.1913961489363292e+00,     1.4911816872620067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6250640286339483e+01,     2.2756464839382068e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.2086927127680042e+01,     1.5637926122066050e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2552271914074183e+01,     2.2757169471298213e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.5754372027430206e+00,     6.0349999562793819e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0711757242175999e+01,     8.6454796713295091e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.3245361273825644e+01,     1.5598111220168137e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.3041734800633797e+01,     8.5658498675336850e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.8972693282347805e+01,     1.6641840820958470e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3345413084818933e+01,     8.3198312690829080e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.8056898808589185e+01,    -8.3375515801639958e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9679474962306237e+01,     7.2294819764980220e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4135410440815196e+01,     2.4029180068945676e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5078459403927614e+01,     8.6284123897898626e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.0125338918499892e+01,     1.6672924877535326e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.8667226553490535e+00,     1.5890729163357412e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4823150335463851e+00,     1.5400767679155813e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5365288099990671e+01,     8.0776570657564434e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7572545138160649e+00,     1.1790629090789623e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4247303808507041e+00,    -1.2723629136055376e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9588570229037298e+01,     3.2828444724274239e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0239071849066494e+00,     2.3831630812707332e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3042434602984645e+01,     1.5436816095559412e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4152206815707581e+01,     7.3200010597807268e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5039103721462728e+00,     1.1926254461575309e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6321774644319358e+01,     2.3818476617394052e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.1154699887819518e+01,    -1.3238257090433322e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.7447838300156995e+01,    -1.3245303409594821e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.3729028914289275e+01,     3.2809645219896417e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9519651009068561e+01,     1.4699830167158261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1975470655644328e+01,     7.9225525437321949e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3334143637300023e+01,     6.0640972351565169e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5136115163173471e+00,     3.8027696093255372e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0170517772379549e+01,     7.2490077543677362e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9762720551949350e+01,     1.5717555925861872e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5052017956702315e+00,     6.5055731869844931e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.9785757382688857e+01,    -1.5252532512172365e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2478245241438298e+01,     3.4713367479410867e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.2337259943513715e+01,     6.5861075962708755e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8704732376051524e+01,     8.4879510322151000e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7750196408610737e+01,     2.8127612199098065e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.8308310030869306e+00,     1.3515844924019184e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4237994211703917e+01,     1.5227285581300930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3343065168145998e+01,    -6.3017820586608377e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.2668378266876315e+01,     6.5061254765169770e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3405057019890705e+01,     2.7012983639832961e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5688682222676665e+01,     1.4392788659779966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3907604967134326e+01,     1.6679924754379146e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3993196424055974e+00,     1.1506769756687036e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2951569057771795e+01,     8.3347248990662010e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8240459956998190e+01,    -8.3226579501806991e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.5617887732991829e+00,     6.0626745442227639e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9990828831938545e+00,     1.6106627525213480e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.5360859625211729e+00,     1.6135013458623740e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.7811775271442009e+01,    -5.7087898663580745e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.1256883102244153e+00,     1.6091432071231513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3650595779179678e+01,     1.4968127147614319e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8605285746135030e+00,     9.5323374975033457e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5631763732119239e+01,     2.3911260616503158e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2428483102677227e+01,     3.0457200994918046e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.2135210003473141e+01,     7.2711507640219252e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2182120448832796e+00,     9.0099681068801838e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5682599017904764e+01,     3.5341675064456416e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.5565172625831664e+01,     6.5303983501264593e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9162849498224634e+00,     1.4391365968846214e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4433485490365186e+01,     6.4905834482285472e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0825248238848619e+01,     7.1338807720300365e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5279983747399388e+00,    -9.6190880221696971e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0978832860236977e+01,     8.3593443469821468e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    9.4604353445285145e+00,     7.8416591049370109e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4460459432606996e+00,     3.3406124347172772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1099360155172022e+01,     1.5859196926612640e-01,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -2.0963321404350289e+01,    -7.0793738884136769e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2493599393379744e+01,     7.4493494807109000e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0323927665244568e+01,     1.4834816901140426e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9560012218269414e+01,     6.4507685463306352e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3236010145566198e+01,     6.7848297898023629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.7780442212464443e+01,     8.1919737221640462e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0192705514155245e+01,     3.9218192733201818e-01,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    1.6168283001299564e+01,     1.5307467674629237e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2246216248607126e+01,     8.7977394020773445e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2741568840981721e+01,     7.5255822036978279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1207627066037922e+01,     2.3854525033797650e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.5585047276186245e+01,     8.9573513256270689e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.6506265562987466e+00,     1.1292505612600507e+00,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    5.5871198198323071e+01,     3.8814179353585132e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1181182342423458e+01,     2.4078693980400138e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    5.6225767380881271e+00,     7.7540642822559748e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9649043174110421e+01,     9.5914577327607731e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.6141361086770353e+01,     1.6577753045451074e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.6294769857218432e+00,     1.0969965329906358e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.4791637492835733e+01,    -6.1276555675711370e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4055665087200012e+01,     3.6147409882490110e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8767993005114374e+01,     7.0548032577666963e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3200343063362922e+01,     2.4083296380282958e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    3.0023295575539755e+01,     7.2226993637411788e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.5487483484507774e+00,    -8.1529233863423060e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.1799139878447690e+01,     1.6170350529560461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6491132983328612e+01,     7.1971421845507530e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.8709542665977516e+01,     1.6737012653042722e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.5645461740049655e+01,    -5.5260821413692611e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.9638768623684255e+00,    -1.6653838832985248e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5684425549991658e+00,     6.3093741736172706e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.5365463100200543e+01,     8.4862200637378610e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.2130713590996205e-01,    -2.2041253444728142e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9009142875138849e+00,     1.4966704456680567e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8574768574235581e+00,     1.4834264611607941e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7859967486130280e+01,     2.3242785813601629e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3450612960081724e+01,     8.3301141155572022e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8626441657692538e-01,     8.3272687336896961e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7961978801076008e+01,     2.3872157060072121e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.2064132135597433e+01,    -6.1819777012822669e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.6592425975865787e+01,     1.3431284057889570e+01,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.9634868648238646e+00,     5.3291401102217151e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9141610457103404e+00,     5.2308073609003679e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8257941208730315e+01,    -4.6446178730874976e-01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.1828251337909919e+00,     8.2485093312951431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0259544424318632e+00,     1.7179752239870060e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8560615537169284e+01,     2.3924414811816438e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.2806396446105182e+01,    -2.0350143397274223e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0672309808916715e+01,     2.3870734369138368e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0699768557651193e+01,     3.6978817033821088e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.6678794630140743e+01,     6.6012973086012298e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.4552157616362198e+00,     9.7430409490549682e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.8522655023377128e-01,     6.6026759611203952e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9406136955637162e+01,     1.0122944007549690e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1902631508085925e+01,     1.4779459970954085e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5063736774967937e+00,    -1.6369979498882660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.0532489852990803e+01,     2.3298541541114537e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3734555842388474e-01,     7.7526415913222218e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5482583559855216e+00,     1.7208138173280317e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.7904417670559894e+01,     1.4859089774749909e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0938223140391203e+00,     6.5497068639092031e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6979303403510045e+01,     2.3855903686316815e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.6776240692711312e+01,     1.4819274872851996e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0302616418143542e+01,     8.3853584035380245e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1370353530041665e+01,     2.9088811421913433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7351981173082070e+01,     2.0385584685646914e+04,    -6.5009898867266240e-04,    -5.6703623913372500e-07,     2.5927738703001760e-09, } , 
      {    2.1324350582865947e+01,     2.1571200920493528e+04,    -4.6737887434132792e-04,    -3.4675983412466832e-07,     1.5063597404386279e-09, } , 
      {    2.5461850609543490e+01,     4.8371193384424550e+04,     1.6955458178407660e-04,     6.2628958546251520e-07,    -3.3424461135366624e-09, } , 
      {    1.3640952169752946e+01,     7.3560382165213614e+03,    -5.9457133852046287e-04,    -6.2668839319659571e-07,     2.7830341513148017e-09, } , 
      {    1.9936736948055525e+01,     2.1918965734389778e+04,     1.5060226776053464e-04,     3.8502835963620902e-07,    -1.9387262775257344e-09, } , 
      {    2.0722775051258516e+01,     1.3590274309766326e+04,    -1.6901400017915301e-06,     1.3430948548155433e-07,    -7.0274227890508348e-10, } , 
      {    1.0970542672409268e+01,     5.4001446752976717e+03,    -3.9853181672458579e-04,    -4.0949394875835249e-07,     1.9688186627122033e-09, } , 
      {    8.4856110584528732e+00,    -1.0436408836835953e+03,    -5.5881586896563745e-04,    -6.5836364657051922e-07,     3.0414834765774847e-09, } , 
      {    1.5438056876388853e+01,     7.0755010752035514e+03,     2.6073685074308262e-05,     1.0448340845011887e-07,    -6.0761213839777023e-10, } , 
      {    1.6094550011227295e+01,     3.9481774119160327e+04,    -3.6575336128657644e-05,     3.0091337988917701e-07,    -1.5939801170255545e-09, } , 
      {    1.4438782188330421e+01,     2.4080995180341546e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.8589402005026363e+00,     7.2478199124824077e+03,    -2.4929222883714246e-04,    -2.2504886240453078e-07,     1.0018189906447449e-09, } , 
      {    2.3003870141306741e+01,     4.8229218033261532e+04,     5.4565033641837955e-04,     1.0643768270239776e-06,    -5.2975154602525984e-09, } , 
      {    1.8379012416246198e+01,     3.0315231273586516e+04,     2.4639328387954424e-04,     5.6170629448634294e-07,    -2.7166680065077178e-09, } , 
      {    4.9586842640752158e+00,    -1.2904209778773438e+03,     3.1012509678150921e-05,     3.6360444306797873e-08,    -1.7530862708920817e-10, } , 
      {    1.4950699178005852e+01,     1.5961506779459552e+04,     3.6375008866087745e-04,     5.7486229549966761e-07,    -2.8755705386281790e-09, } , 
      {    4.8718199368252435e+00,    -1.0909473360052978e+04,    -8.5135490147375572e-05,    -1.7914350330678462e-07,     9.9570064198914267e-10, } , 
      {    1.0766706148501099e+01,     3.3595302839811608e+04,    -1.1475392545466375e-05,     2.7170369497190422e-07,    -1.5532897047700311e-09, } , 
      {    1.5363597416050274e+01,     3.3385753374075306e+04,     4.2112175373215318e-04,     7.8383187583788756e-07,    -3.9664013211246355e-09, } , 
      {    8.4694365163159890e+00,     5.8864712793487161e+03,    -2.5099943583191279e-05,     2.9209684917272764e-08,    -4.0690412255523351e-11, } , 
      {    2.7566608730567403e+00,     6.9928963079556797e+02,    -1.9071175881006004e-04,    -2.1842722866656861e-07,     9.2309494510617842e-10, } , 
      {    3.5426989762597310e+00,    -7.6294017938278830e+03,    -3.4300416657238620e-04,    -4.6914610282122336e-07,     2.1590789437268294e-09, } , 
      {    4.8735769838299881e-01,    -8.8860057042560038e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    5.3048338623952929e-01,    -9.5143076594700651e+03,    -3.3501252209366066e-04,    -4.7099527916371135e-07,     2.3223981284821988e-09, } , 
      {    1.0365599825831161e+01,     4.1224704633639485e+04,     3.3152877402691986e-04,     7.4084979779312753e-07,    -3.7123686484968607e-09, } , 
      {    5.6976164129140869e+00,     2.3939019829178527e+04,     2.9607839995175986e-05,     2.3879565736965546e-07,    -1.1859609230037689e-09, } , 
      {    6.3974031404040161e+00,     1.6866932314983202e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    6.4836545161170775e+00,     1.5610328404555081e+04,    -1.2268456776715018e-04,    -1.1923216784999263e-08,     5.0023075616881894e-11, } , 
      {    1.0935396538214587e+01,     2.5161866692482588e+04,     5.5096721960341630e-05,     2.7647615236737837e-07,    -1.4686945655532281e-09, } , 
      {    6.2242874374409825e+00,     8.5044838432356883e+03,    -2.4948809356431073e-04,    -2.2496159594193106e-07,     1.0032734316880736e-09, } , 
      {    6.9804122754473461e+00,     2.6242678184298471e+04,     4.5154946030116154e-04,     7.5338940669228952e-07,    -3.8168314523655318e-09, } , 
      {   -3.3334222617860636e+00,     5.8849268314763094e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4359325815977702e+01,     1.5383024513167895e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.9901388979840689e+01,     7.8446716503452953e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.4231364234173149e+00,     7.6834631587703134e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9569681618376382e+00,     8.4088735179517589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6842493553980212e+01,     2.2669889484385237e+00,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    6.7594540873721183e+00,    -2.6906804280771610e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.5733017284166742e+01,     3.1337285961853922e-01,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.7475514182029549e+01,     8.7085944831181696e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0283903150455718e+01,     1.0984192239243887e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2034394224456911e+01,     1.5558173401121474e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1555810760857145e+01,     7.4857673017999159e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4717293349271619e+01,     7.2942449586866514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.2052305300462194e+01,     2.3886987742893674e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3532147188348661e+01,     8.3265491864016294e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.8210380875750563e+00,    -8.3308336628452726e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1399414006973647e+01,     1.4392077314313090e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2490588989643525e+00,     7.4650972242253520e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8930059058680904e+00,     7.7030136718940903e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3870637645164980e+00,     6.4963749302339500e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2000547438643423e+01,    -1.5747743514451031e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.7841804201535325e+00,     6.6387243775239922e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0123053982402377e+02,     6.5550291511499381e-01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.9933761882086511e+01,    -1.1169297592065159e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.4333125885835599e+01,     1.4929988733224682e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3463392896709783e+01,     7.1980633571343469e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0081588319241721e+00,     1.6730463662918642e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8385034985637514e+00,     7.6202017462495987e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.7551369361724355e+01,     7.8994583490674686e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6593941430184330e+01,    -3.1132455813112756e+00,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {   -1.8157548699385249e+00,     1.5527335178842119e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.2616000579104245e+00,     8.7992646553285613e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1158536865084781e+02,    -8.3304223931006873e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9373176472098031e+01,     1.5560803233437655e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.6279037150803504e+00,     8.5639780399760057e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1300414815375760e+01,    -9.3300004526574980e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1842154604593805e+01,     1.4914775337311024e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.8256327953513285e+00,     3.2407385405023590e+04,    -4.1903099114474666e-05,     3.0214616411750231e-07,    -1.7028595735291340e-09, } , 
      {    1.1787565179712638e+02,     8.3266559076063750e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0373983844399288e+02,     8.3307269416405252e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3353707573569555e+00,    -1.6834175672392344e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.1718444570634983e+00,     7.1986437542186668e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.5555900524689747e+01,     3.2411987804906403e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.6834527963987327e+00,    -1.1172583915812884e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9138045786912135e+00,    -1.1020626722364071e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5476630538037588e+01,     4.0077619579419988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.0627472344554576e+00,     2.3791815910809419e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0930734839709132e+01,     1.6012154583393763e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8999231202296478e+00,     7.4781665690567305e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1534391301063369e+00,     1.6579830588234774e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.8275345726996193e+00,    -1.1119987269489220e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.6944529630376088e+00,    -1.1997610743472023e+01,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -3.1013975103114156e+01,     7.9773571843860536e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7591724588251008e+01,     8.3266138818397485e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5444517712181018e+01,     8.3307689674071516e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2898043814492841e+01,     6.6924048202020604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1549475721264134e+01,     1.8073704941245055e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0526936072354573e+01,     6.6284150962685044e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6583374080630691e+00,     1.6734935110259023e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.6079332904033006e+00,     2.8449140577913781e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.5592490441480450e+01,     1.5113521214425813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5784064914396048e+00,     7.7533529367890978e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.4378597489219018e+00,     1.5524705346525934e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.0021408597813515e+01,     1.4285786803932633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2252528160067637e+00,     8.9033185669909253e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.3428051867974418e+01,     2.5426438423554032e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1033359778934155e+00,    -4.2680239714814797e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.2650145477119196e+01,     8.3284857093974297e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6467834485371021e+01,     1.0677292206116781e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1543535254640711e+01,     6.5082030193006776e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.7931411144406226e+00,     7.1960139219024823e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.0221030706268346e+00,    -1.1634615542319864e+00,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.7575033240500204e+00,     8.8783885005432452e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0739241775045954e+01,     8.2691728475520376e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.4322227537969923e+00,     1.6673382357470899e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.1421812824960895e+00,     8.1114384932116045e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1650647798215228e+01,     8.0338862321994802e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.9306391399755039e+01,     1.6051617169333600e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1106753520112305e+01,     9.3500199793619813e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.3245553318998873e+01,     8.9725410379574223e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0129262312869784e+01,     1.6727212693105659e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.9005432492260717e+00,     6.0633858896896400e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6809752936586406e+01,     1.7346683833973111e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8488631375708138e+01,     6.5342203450725691e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6357372871074560e+01,     8.8999111110054127e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.8504617205500828e-02,     7.6996724941110297e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8788575242256414e+01,     7.6052120074936529e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8091180374529557e+01,     3.4960327176015744e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7554011250262853e+00,     8.3989733845695646e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.4244942150574428e+01,     4.5515411956543267e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9150249779217745e+00,     1.3735383795857289e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4197600165156793e+01,     3.9071976975157850e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2756396461012240e+00,    -1.1675976564762971e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.7427612727334303e+00,     7.6605689376799946e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.7139069709415381e+01,    -6.4087775507394326e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0199799499726385e+00,     1.3710800632056356e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1421597602573126e-01,     5.2257751744315044e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4064073504709413e+01,     1.1371704643973499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9935680350207690e+01,     7.6450269093915649e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0704911649426185e+01,     1.4205957026562399e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2351228648275750e+02,     7.2120273483412693e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3508003967925596e+01,     1.6031376464142766e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9888505739407520e+00,     8.9047412579246775e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2881229289421121e+01,     5.1697389657825875e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.8005013020348940e+01,     7.2160983823754195e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3930058097327226e+01,     2.7608977173933454e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.2059112096352663e+01,     1.7665593492787164e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0566829386347348e+02,    -1.9475210249675001e+00,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.5009898616615006e+01,     8.8636990051424182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3412930903087776e+01,     8.7579245001794316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8302395858079397e+00,    -7.0585984605766726e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9098008022986539e+00,     6.8800016218866167e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1941709607715795e+01,     1.4967415802147443e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7031432183147732e+00,    -3.1283887636467944e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0610876728825902e+00,     9.9176968403512956e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3185464095623960e+01,     1.5462572196653486e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9183431855196069e+00,     8.9577103551358687e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0294946697285372e-01,     2.0618562510975480e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6415560571063434e+01,     1.5383494682390145e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.0497713294888555e+00,    -1.1544434611630179e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.3910874899810942e+01,     7.5653971055957409e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.3584392760584496e+00,     5.8772656019389506e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8785394629114769e+01,     1.4427773414758614e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0609099375520117e+01,     1.7272572523094179e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4055273356295661e+00,    -8.3418093358217156e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.0377384903295820e+01,     1.4428478046674763e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6788183142891677e+00,    -8.1982336684113557e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2792232493407795e+01,     3.3019021118843320e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1091032555718250e+01,     8.2764336728791350e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6309870769969507e+01,     1.5648692484404177e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5781875730486917e+01,     1.9313234210097771e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.4321965697836703e+01,     7.3733224729499934e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0223120019883522e+01,     1.4945316177598261e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0448275331633710e+00,     8.3809491763677652e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7088029822871771e+01,    -3.3088781982054206e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8637305794083041e+01,     9.1318006455490722e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6732397564906705e+01,     2.3951075518401070e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0355173321005165e+02,     8.3287894761972893e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0369304543051481e+00,     9.6654886880428458e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0444176422389337e+01,    -1.4308927549444945e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.3227228612694995e+00,     4.6868894794509271e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.9416309299738955e+01,     8.3285933730496108e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4866706076625904e+01,    -5.1887086183529902e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.9095910386162966e+00,     8.1670430336669142e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.5417903278008387e+00,     1.3500097180504734e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6897940007413609e+01,     4.4007829276349858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5863865804469029e+01,     1.4914118072561479e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7800252601044626e+00,     6.5644253723349493e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3480917547905193e+01,     1.8058614462480149e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2504542634945155e+00,     7.8070333794671660e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2272378371124329e+01,     8.3220800250713455e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7464279036294803e+00,     6.0626633160081496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2370709445318745e+01,     8.1889718027459076e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.8503525665584863e+00,     1.4913735359469376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3993003852217479e+01,     1.4836894443924128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4340312877654746e+00,     1.7576803828114203e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    9.4936036756362459e+00,     8.9040299124578014e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9052222580775947e+01,     2.1479393958530636e+01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.7178648697411187e+00,     8.0934048092708945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0827676941481037e+01,     1.0515964977242678e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.1770856539873364e+01,    -9.8051573838887407e-02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0581799944215010e+01,     2.5756368469741876e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.6159742879416022e+01,     7.2091318331007478e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0865741863857991e+01,     1.6026774064259946e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.5677150781204134e+01,     3.2199784823270620e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2713424539611635e+00,    -3.6278588994472784e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.9035020165625141e+01,     3.2200489455186769e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.2245520518782733e+00,     1.5559674970789387e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6986595536542197e+01,     2.3793893453593122e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.0143744501757990e+00,     9.0371811029973032e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.8481915564471851e+00,     9.3734302058697722e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4924359566168651e+01,     1.6097767369672936e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.3854009694579945e+00,    -1.2128443365341227e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.6596130612526041e+00,    -8.4684110465009944e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.4616414562637345e+00,     6.0768902253456770e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8632514412000695e+01,     8.3276277631984995e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6485316165164591e+01,     8.3297550860484007e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9586698245313883e+01,     2.0269093659626105e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0098834731877313e+01,     7.6136427906504650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0179618061495042e+01,     2.2599082800999709e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.3196707318286361e+01,    -1.1282626482692422e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4509339356516104e+01,     1.6692719920183623e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2540151810878399e+01,     1.7913998019337985e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.7353998811293536e+00,    -7.9694518823856242e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6062038833229863e+01,     1.5536195334382661e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    8.0295934767752950e+00,     1.6204843302482499e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.4600770295914831e+00,    -4.5253954676440117e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.4672150636110612e+01,     8.8341401344390179e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1160211144969438e+01,     1.5529636378783527e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.3815459989228724e+01,     3.1085156264005516e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2263574706824709e+01,    -6.9300896437063811e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.4827752599055160e+01,     3.1085860895921662e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.2263438057279785e+01,     2.5898355080567857e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.8147130006923987e+01,     4.3670366489079854e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.3207300002388825e+01,     1.2098536934896709e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7317073527953312e+01,     2.4499753299650787e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1705708490752288e+01,    -6.9587043995586396e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.3916904797793372e+01,     8.4019859299778509e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7864976343640663e+01,     1.9798920774474616e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.0462677016210700e+00,    -9.5689420492467389e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0148676889024593e+01,     2.2128509570294598e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7793691808156808e+01,     1.8173599002263090e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.4470708985001046e+00,     6.0697767706769139e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6718534088819176e+01,     7.1618051136140293e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5667005965431002e+00,     6.2688489411179046e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1253035352269896e+01,    -8.0470041433977476e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0021351232904003e+01,     2.4047213752886382e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.9119884759350334e+00,    -2.0764385369322217e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.0152291061528835e+00,    -7.3596329353072633e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.9772251804232226e+01,     8.5378885744016752e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.4714416750174422e+01,     8.5385932063178279e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    8.7913639538163011e+00,     7.2663206171026595e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1927584907542265e+01,    -7.5583972287519246e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0280848080205519e+01,     7.0382948270772031e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -7.6528602802518535e+01,     8.4464051618399481e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7055550095802680e+01,     1.5622936383310103e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1964369286374723e+01,     2.5195271423648577e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    5.2611883759784916e+01,     2.5195976055564723e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.1219626393644990e+01,    -8.8150123988430114e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.9613864495272235e+00,     4.7320306302129484e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7501085152862125e+01,     8.8600962091075016e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.6545816983157557e+01,     1.0575406606015249e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6542113455684607e+01,     7.6831108428122388e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8892148895395247e+00,    -1.1328242739025183e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -5.0837573591871976e-01,     1.4772460094110271e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.6731073686772746e+01,     5.4328652339354776e+04,    -4.3593239116266197e-05,     4.3645564959905656e-07,    -2.4056018524342174e-09, } , 
      {    3.4418644824242477e+01,     4.5371658959517263e+04,    -1.9322176538568385e-04,     1.8512038333023921e-07,    -1.1151781255617769e-09, } , 
      {    2.8835246244552820e+01,     3.8299571445321933e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.6479691694166018e+01,     2.9970880020698485e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.4081011455922685e+01,     2.2270490551289098e+04,    -6.5809063315138794e-04,    -5.6518706279123696e-07,     2.4294546855448066e-09, } , 
      {    2.6735246411129481e+01,     3.1156496255545098e+04,    -3.2041422956481880e-04,    -9.4808175741688251e-08,     1.6149628531439737e-10, } , 
      {    2.8732820326702875e+01,     4.6486287518782367e+04,     1.7754622626280214e-04,     6.2444040912002716e-07,    -3.1791269287812930e-09, } , 
      {    2.6420391464172603e+01,     3.7529294138944853e+04,     2.7917699993384458e-05,     3.7310514285120976e-07,    -1.8887032019088525e-09, } , 
      {    2.0707615820913354e+01,     3.2342112490391708e+04,    -1.3769411523348429e-04,     1.2546822926736836e-07,    -9.2491784454715063e-10, } , 
      {    1.7463597249473615e+01,     4.0528828563852141e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.6066583553715926e+01,     2.1358237893749003e+04,     9.6764757610126595e-05,     3.1037102821752559e-07,    -1.4262442796352771e-09, } , 
      {    7.9120019843568139e+00,     9.0989687310005247e+03,    -2.2646722836488535e-04,    -1.8675197529264522e-07,     6.6464561984349588e-10, } , 
      {    1.0412092828658370e+01,    -3.2090838906435883e+03,     6.9820799107859127e-05,     7.0958978733707463e-08,    -1.8584362837971840e-10, } , 
      {    6.0707562780726558e+00,    -1.8139181900606736e+03,    -1.8005623283842597e-04,    -2.2089279712321928e-07,     1.1408538581133376e-09, } , 
      {    1.3420941103707893e+01,     4.2481308544067608e+04,     3.2620101104110280e-04,     7.4208258202145289e-07,    -3.8212481050004404e-09, } , 
      {    8.6667063150777572e+00,     2.6452227650034773e+04,     1.8952314023541942e-05,     2.4126122582630608e-07,    -1.4037198360109280e-09, } , 
      {    2.5286702533498362e+00,     1.6691139896370962e+04,     2.7376817057366969e-04,     4.6499003753991189e-07,    -2.2981138111954222e-09, } , 
      {   -6.8935479481973738e+00,    -1.6623625802122835e+04,    -3.3540146047563498e-04,    -5.3788545907870699e-07,     2.6458221832871811e-09, } , 
      {    5.6696499554027175e+00,     5.8154836037672085e+03,     1.6294793373396022e-04,     2.4825330569800405e-07,    -1.0182250856134916e-09, } , 
      {   -1.7250275142612193e-01,     2.5132078208562416e+03,    -1.0655525971634047e-05,     2.4655684566506563e-09,    -2.1775891300715919e-10, } , 
      {    6.8948358435506787e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5047118554326577e+01,     7.7648181893917636e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.9521808012564222e+01,     2.1575502731181587e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5438673141142484e+01,     1.6755598626515919e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.1208842540592610e+00,     9.2724677183498570e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8960549700236342e+00,     8.4269072018924671e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4475220083403457e+01,     8.2173577361561656e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2504633830352104e+01,     7.5295315740392089e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6985079476235256e+00,    -8.4400251130907345e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.3125687856530481e-02,     6.2830195521406040e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {   -2.0887346547221146e+01,    -2.3011999414102320e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1226097389372729e+01,     6.5857609101442840e+03,    -2.1581170239325131e-04,    -1.8921754374929586e-07,     8.8240453285065505e-10, } , 
      {    1.3581651939759531e+01,     1.4914452334767735e+04,    -6.3519294630925112e-05,     6.1501330405358849e-08,    -3.5357946576999568e-10, } , 
      {    5.5402728918331263e+00,     7.7003894694093897e+03,     1.5495628925523469e-04,     2.5010248204049204e-07,    -1.1815442703688610e-09, } , 
      {    2.3124288625302718e+00,     8.9569933798375096e+03,     1.4962852626941768e-04,     2.5133526626881740e-07,    -1.2904237268724407e-09, } , 
      {   -2.3843179027206570e+01,     3.2863237477272378e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9982533600698744e+00,     7.8423648205724048e+03,    -2.2113946537906831e-04,    -1.8798475952097055e-07,     7.7352507634707552e-10, } , 
      {    1.0353807910456677e+01,     1.6171056245195854e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {   -1.8531870969680707e+01,     8.3263902246820398e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.2667748274991112e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2969337648898662e+00,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    1.3624777627616062e+01,     1.4286150379553674e+04,    -6.0855413138016608e-05,     6.0884938291196182e-08,    -2.9913973751820592e-10, } , 
      {    1.5937206490146334e+01,     2.3243143759391183e+04,     8.8773113131401069e-05,     3.1222020456001358e-07,    -1.5895634643906465e-09, } , 
      {    1.1269223077229260e+01,     5.9574589549302236e+03,    -2.1314782090034278e-04,    -1.8983393586345854e-07,     9.3684426110244481e-10, } , 
      {   -8.6251375713060963e-02,     1.2566039104281208e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {   -7.5183378010077462e-03,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7272239351690146e+01,     8.3283391086653737e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5124859967385134e+01,     8.3290437405815264e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6725289607955915e+01,    -3.5231595807488247e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8958274422199288e+00,     1.6029080894032839e+04,     3.0724869701756089e-04,     5.0082135619514676e-07,    -2.4175282689895120e-09, } , 
      {    8.8705428389859264e+00,    -1.7429305144791660e+03,    -3.6810411015557749e-04,    -4.3993641790395058e-07,     2.1183885314713058e-09, } , 
      {    4.6679834129170743e+00,     1.7285684804460961e+04,     3.0192093403174383e-04,     5.0205414042347212e-07,    -2.5264077254930913e-09, } , 
      {   -7.3304594446843780e+01,     1.5542028288429719e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6003052013880108e+00,     2.6298323161836379e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0633036410671902e+01,    -8.8601555405439285e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.9551276722133437e+00,     8.4706667757864652e+03,    -2.2380334687197685e-04,    -1.8736836740680787e-07,     7.1908534809528565e-10, } , 
      {    5.5833985796896570e+00,     7.0720875141953293e+03,     1.5762017074814322e-04,     2.4948608992632941e-07,    -1.1271045421170713e-09, } , 
      {   -1.2845890819167568e+01,     7.2117616654169342e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0490413868125007e+01,     1.5540453090040384e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.0709052463007254e+01,     1.5545055489923205e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8353423149644215e+01,     7.2163640652997528e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1623030987685937e+01,     2.2128515200126079e+04,    -2.8199487851708491e-04,    -1.2709982122977434e-07,     4.7438533882886960e-10, } , 
      {    1.0396933598313208e+01,     1.5542754289981794e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3777449957116685e+01,    -1.5419111139677625e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7782406907477357e+00,     8.3294027700903262e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0742153174201015e+00,     8.3279800791565740e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2079770312667026e+01,     8.3117707438158577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2339010586595919e+01,     1.0958808674859029e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0772090649089117e+02,    -1.7309684772390028e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9267476437299134e+01,     1.3799823775502628e+04,    -4.3428728627941109e-04,    -3.7781869538442909e-07,     1.7103693374495204e-09, } , 
      {    1.2709362460843479e+01,     2.4499747669819306e+04,     8.3445350145584034e-05,     3.1345298878833894e-07,    -1.6984429208942262e-09, } , 
      {    8.0413790479264051e+00,     7.2140628653583444e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7842914632728650e+00,    -4.8632660405104525e+02,    -3.7343187314139455e-04,    -4.3870363367562522e-07,     2.0095090749677264e-09, } , 
      {    1.0325841452945308e+01,    -1.9524799802154675e+03,     6.4493036122042105e-05,     7.2191762962032783e-08,    -2.9472308488329798e-10, } , 
      {    2.2693031746737415e+00,     9.5852953350515709e+03,     1.4696464477650915e-04,     2.5195165838298003e-07,    -1.3448634551242304e-09, } , 
      {    3.0616615854986859e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9340097323944949e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1487508096859752e+01,     8.3290200569982226e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.1865934134705718e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9305333009146430e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7238813297108386e+00,    -1.6920680807593385e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4504454623815278e+02,     1.6219053161767079e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5036477293040015e+01,    -4.9310322575947794e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3136697586847648e+01,     8.3155735134251827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8331146950567199e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9982879429315759e+00,     8.3260615923072655e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8541483018995706e+00,     8.3313212569396328e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.4468921743503046e+01,    -2.4272873609482303e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6176316416702793e+01,     1.6655081649305488e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.5098899217623313e+01,     8.3131493963350204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.0963336615448355e+01,     8.3442334529118798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5023303331738191e+01,     1.6659684049188309e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3978585538072739e+01,     3.0457206624749528e+04,    -1.2970247075475876e-04,     1.2361905292488038e-07,    -7.6159865979178123e-10, } , 
      {    1.5980332178002865e+01,     2.2614841804177122e+04,     9.1436994624309573e-05,     3.1160381244585095e-07,    -1.5351237361388568e-09, } , 
      {    7.0979974236424539e+00,    -6.9587606978734675e+02,     5.9165273136225077e-05,     7.3424547190358117e-08,    -4.0360254138687761e-10, } , 
      {    1.0780824815019256e+01,    -1.3117911198267393e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5445847080062816e+00,     7.2114330330421608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1676785834755620e+01,     7.2166926976745290e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1255573625625985e+01,     1.5732483999299802e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.0743889059701388e+01,     7.1985208370699147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5024461828749631e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.5238176119994790e+01,     7.2296048936467741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7872846219260019e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7587381184659009e+01,    -1.8048943455862172e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1585172682655003e+00,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5818781331859846e+00,     8.3640284955601728e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.2139979437293889e+01,     5.5356930186341742e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4869068249303709e+01,     7.2144151813164190e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6232979968141571e+01,     8.3654511864939250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9367308413726555e+01,     7.2137105494002708e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.3583805438813187e+01,    -2.5069151674342720e+00,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3360914563062414e+00,    -6.5589555991336965e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.7480701711978355e+01,     1.6657735165204973e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.4916884480440032e+01,     1.6657030533288827e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.9262945591044751e+00,     1.5540124457665610e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2765871465963869e+01,     1.5543106605939871e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.8835773962447078e+01,     1.5542401974023718e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4005053368817272e+01,     1.5545384122297977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.7256733353348714e+01,     9.5171832084254220e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0773130077809899e+01,     1.6864631115041790e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.0426343705028142e+01,     1.6869233514924610e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    9.3106416654880206e+01,     1.5527212261693365e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.2891077797002247e+01,     1.5558296318270224e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0780734722036378e+00,     7.2133515198914674e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4145131405294038e+01,     7.2147742108252214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.6088107368655869e+01,     1.3498673398216670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0948703431124748e+01,     6.2133149872446666e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5530275639472575e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9027154536628590e+01,     8.3114184278577814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5972461390803421e+00,     7.2309835461659377e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0484668005487407e+01,    -1.1944470569373675e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.1348852952654660e+00,     2.3869144514663836e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.3064780395801591e+01,     2.3873746914546657e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3738565094474517e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0536335050074703e+02,     8.3269604561462111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6909180168417315e+00,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.2401282773139783e+00,     3.1546870868086353e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4916621969201294e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0126155927718749e+01,     8.3110593983489816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3446245428893302e+01,     1.5543082922356567e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7381705279109511e+01,     1.5542425657607024e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9063710547543354e+02,    -5.3779715276359639e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5026268784452284e+01,     7.2137342329835719e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.4001991187141094e+01,     9.6640240106353303e-01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.5802035768663373e+01,     7.2143914977331169e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0762071318275872e+00,     7.0563285110179122e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0457280256411622e+02,     2.0355170170754775e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3845173842012937e+00,     8.1709570702830179e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1942899605952020e+01,     7.1365106043462201e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0117449609913592e+01,     1.5275583779575754e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.4653923702111067e+00,     8.4864257789638814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.1383673629475858e+01,    -1.7272996765668267e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4531460702591135e+01,     1.2619238417987162e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.7278886509186968e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.9493203339877653e+01,    -2.0571522602131154e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.8276963940010386e+01,     8.3198312690829080e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.2988073040783991e+01,    -8.3375515801639958e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.2585307015077145e+00,     8.2757223274122589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0936677503037604e+00,     1.5999508223997282e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.4007791948778859e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1493057218320608e+01,     6.0334744427518672e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2942265832148925e+01,     7.1610937681471532e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2932834901547761e+00,     8.2511391636113258e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8739109907269444e+00,     8.3816605218346413e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7698248612814691e+00,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7472652956241381e+00,     6.6633075413249253e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3238828311631625e+00,     7.2916151263704687e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8188910141668263e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8505707930887549e+00,     8.4062436856355744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0752661136141633e+01,     1.4916747904877662e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6941490394281686e+01,     8.2490616208276260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4296504219422902e+01,     6.5864666257796780e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5037746404207622e+01,     8.3647398410270489e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4796074286195534e+01,     8.4083212284192741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5460621167334683e+00,     7.2271807765566118e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0430580285979691e+01,     1.5385019935641363e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3877516887226626e+00,     6.6916934747351834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1951897312667867e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5725259765576380e+00,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1654152726542662e+01,    -1.5925960759164900e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1121406136079642e+01,     7.3717972196987757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.2129676639151853e+01,     8.3044185510139669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2543287285688439e+01,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2540160749144288e+01,     7.2670319625695356e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1231449140387085e+01,     7.1344330615625204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7023031708247089e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4296851302850369e+01,     1.5465202028969670e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6266578031174163e+01,     8.3529642982329315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6730473201280581e+01,     1.4912145504994842e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0209354576918823e+01,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1097147192393564e+00,     1.4991998965948376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.8351440004490467e+00,     8.2888765227255371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.9029993401688671e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3687643728391949e+01,     8.3685063265213612e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6112276670986239e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7141863854814396e+01,     8.1578391590847514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.1396034451589578e-01,    -1.4226909337526763e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.8546679120722910e+01,     1.4914094388978176e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.7536191682648070e+00,    -1.7632026274469709e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0271956445675041e+00,     7.7779361005900309e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0607958176038128e+01,     6.5854029643547265e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6683171241701178e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6631585630400650e+01,     1.4915158050403128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3310973837443836e+01,     8.3124820892827338e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0467124854956253e+01,     7.2936926691541685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6164266445918916e+00,     5.2257751744315044e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0744033897716189e+01,     1.5020384899358634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.6394052038586310e+01,     7.1897899917488630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7099514551753936e+00,     7.8063220340002890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5296049462233105e+01,     1.5489785192770603e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2059533409551371e-01,     7.1742479634604324e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6804391256272329e+00,     1.5620306550993919e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0190629563757940e+02,     7.2383357389678267e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6164120822341115e+00,     8.0182093328307928e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9359599543763501e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3731546468721350e+00,    -5.3680442678067706e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4691340068479377e+01,     8.3512333297556943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6140014211242775e+00,     2.1329907977851814e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.5470504898084656e+01,    -2.2979975713456952e+01,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0882591331175384e+01,    -1.1489987856728476e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6131682968112571e+01,    -8.3441105357631277e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9342014186620158e+01,     7.2538777672562564e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4284545723306429e+01,    -3.1480227513751743e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.5494180709281711e+00,     1.6656671503780024e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.0142508535422703e+00,     1.6658094194713776e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.8396624927543861e+01,     6.5880564802542103e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.5365674910311284e-02,     5.2165804515628406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1330432545161163e+02,    -3.4560240734833769e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6755196849833666e+01,     7.7000315236198321e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0961810120590719e+01,    -1.6209335340717018e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.5989436938194590e+00,     7.7010951850447836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3577203555276859e+02,    -4.5036753191467938e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0529044875435915e+01,     4.2923307555598194e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.8786387123640793e+00,     1.5463124486185970e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8539927485597678e+00,     8.8794467486568683e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3491831987739191e+00,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9085964604190224e+01,     6.5834540803713917e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5423427103039844e+01,     8.5508601287777383e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.0204144528744152e+01,     8.2716035259598684e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3930467883531961e+01,     8.2733344944371092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3483989741238950e+01,     1.5700488644322226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.9795779783520828e+01,     8.3840483548097927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4951132390622970e+00,     8.9562876642021165e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4952628803005874e+01,     5.5228953248342805e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.6695867771900396e+01,     2.3871093398647172e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.5340563325401206e+01,     2.3871798030563317e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.9983455729683673e+01,     8.3297873054909360e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9582035662901902e+01,     7.1587059351720027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2520724867149870e+01,    -2.3011999414102320e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8797258594149335e+00,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9519491362583619e+00,     8.3997091103675375e+03,    -3.3189520425500942e-05,     3.1102494491060616e-08,    -2.0328237648922845e-10, } , 
      {    1.1780914929830576e+01,     8.3996847300364407e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8850534243280467e+01,     8.4357673574330111e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.2453131195039564e+01,     8.4311649575501906e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6867900231763260e+00,    -7.1017685744087430e+01,     1.8548192818782712e-04,     2.1961637966359409e-07,    -1.0327016221314225e-09, } , 
      {    9.3075033375793144e+00,     1.6728400534990986e+04,     1.1910288733682525e-04,     2.8182136864571535e-07,    -1.4392663751098792e-09, } , 
      {    2.5870440397415688e+00,     8.4672232046156641e+03,    -9.4822810327466282e-05,    -4.1792927047734551e-08,     1.4442599560253075e-10, } , 
      {    9.7281695947015052e+00,     7.1430451796142570e+03,    -3.2993655698332687e-05,     3.1015228028460899e-08,    -2.0473681753255705e-10, } , 
      {    6.2832123458890301e+00,     7.0993305412990367e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.0097575968071315e+01,     1.6759856382173639e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.1206095676407724e+01,     1.6764458782056459e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4137402670833016e+01,     1.6728376154659891e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.4993328533250619e+01,     1.5613771975725882e+04,    -2.5166510431166075e-04,    -1.5749865714407260e-07,     6.2468242810963683e-10, } , 
      {   -2.8512056435622739e+00,    -8.3659520632434742e+03,     2.3728754745182761e-06,    -6.7550205260458206e-08,     3.7713656253510807e-10, } , 
      {    7.3726152188476286e+00,    -1.1856462450091931e+03,    -1.8528606346065888e-04,    -2.1970364612619381e-07,     1.0312471810880936e-09, } , 
      {    1.8881405601110657e+01,     8.4460853959095875e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.5074649527193378e+00,     8.2937465356140583e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1864942922008977e+01,    -6.8726316464552752e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5120795952318064e+01,     1.6798205998518650e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.9214574810287441e+00,     7.1430695599453538e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0463602972268660e+01,     8.4394996319343518e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7348882560038646e+01,     2.3942463400349334e+04,    -9.9372696549334565e-05,     9.3220217010582131e-08,    -6.1130157051101389e-10, } , 
      {    1.1759221964934339e+01,     8.4798668233647477e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.8706709144083113e+00,     8.4526538272476319e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3037835749268911e+01,     8.2883510415691562e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.9425945758710919e+00,     1.6795914629239112e+04,     5.7469597434859906e-05,     2.0892594710692014e-07,    -1.0915580030181199e-09, } , 
      {    1.0471060173178044e+01,     8.7151534387173033e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0178196190883961e+01,     8.7927056997294276e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8974079334488732e+01,     8.4337947898663733e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.1654668546677936e+01,     8.4331375251168283e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0725198442217915e+01,     6.5339496446558414e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3822902292185013e+01,    -4.0340383054294080e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.5312008352719968e+01,     7.1069869325487834e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.5991657391453288e+01,     7.1115893324316021e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.7048155539775141e+01,     7.1089358165321200e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.4011046589635370e+01,     7.1096404484482682e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7078628533128235e+01,     1.6826591931928906e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1663057364367342e+01,     2.5057091959614438e+04,     2.7139529509915140e-04,     5.3254024280037007e-07,    -2.6752503737305300e-09, } , 
      {    1.7601992494159639e+01,     8.6813720112621450e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1553157520974279e+01,     1.6745236901307457e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.0590171416053931e+00,     6.3232461931042180e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.2972040605130406e+00,    -1.2169541367574413e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1077540396674213e+01,     8.2599651081588981e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.1930974235940583e+00,     8.3570249140807373e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.3466497296916078e+00,     6.7162671627972068e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7897393215658557e+01,     7.8953300718761675e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.9529858020919608e+00,     8.3989733845695646e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.4411746671923627e+01,     1.5645227822908535e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.6891914314414677e+01,     1.5649830222791355e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.9821792355933354e+01,     1.5613747595394785e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.0423459698265294e+00,     8.2576737388793626e+03,     3.3777433595015333e-04,     4.7033525381824881e-07,    -2.2686856207520733e-09, } , 
      {    1.0521187119711026e+01,     8.4210146380142924e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0768606741453645e+01,     7.9291114993313295e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2083723621489533e+01,     1.5471736604237707e+04,     1.1929875206399349e-04,     2.8173410218311563e-07,    -1.4407208161532080e-09, } , 
      {   -1.2056196800093101e+01,     2.3973919247531987e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.1600323741574837e+01,     9.1857266694224163e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.9247469052377426e+01,     2.3978521647414807e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.5376268066301630e+00,    -1.1856218646780960e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2244970434742573e+01,     8.4023145623526252e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.9301641240845981e+00,     7.6846266203245459e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.7712952825580359e+00,     6.5147432244001975e+03,    -3.0329774205424170e-05,     3.0398835914298233e-08,    -1.5029708928076723e-10, } , 
      {    1.2728282545440461e+01,     5.9586111568214892e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2187135888706198e+01,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1891777768598141e+01,     9.1081744084102920e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2178412189887059e+01,     2.3942439020018235e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -3.1863748706145483e+01,     1.6761805266156978e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.9255455854965689e+01,     1.6762509898073124e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2364416130648545e+01,     6.9515537781497615e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0951485004688234e+01,     7.0755310853661304e+03,     2.8639634203632645e-05,     1.0391064956725607e-07,    -5.5244518962431625e-10, } , 
      {    1.3023024282189780e+01,     9.5787476391154032e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1289623065559329e+01,     8.5574190843768720e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.3520466390424843e+00,    -8.4672475849467628e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.5407350673216648e+01,     7.1622572297013849e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1626513820450597e+01,     1.5072190535015594e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.4548630905792152e+01,     1.0512704882622563e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.6579158248382470e+01,     1.0442241691007587e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2296686564224279e+01,     6.5869187418670326e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.3476184551433086e+00,     7.5360905296383407e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1397088420322643e+01,     7.0563190352790025e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.5020043503615064e+00,     6.7500485902523669e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.7035036951047164e+00,     7.0694732305922817e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.0409006589133742e+00,     8.3467156328252495e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3295593172186184e+01,     7.1491030343881057e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0835121541969944e+01,     6.5207954493425623e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1439229001356187e+01,     7.1066583001740100e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.7742021191569325e+01,     2.5088547806797091e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    4.6856224780338366e+01,     8.4338184734496772e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.4175632287579887e+01,     8.4331138415335245e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5522920861669377e+01,     7.1119179648063782e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.0661669694827927e+00,     2.5266641425350739e+04,    -1.6120185117846815e-04,     2.0412061934386682e-08,    -2.6213875737592615e-10, } , 
      {    6.1922917064027594e+00,     8.8264871271845896e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.3561652697242025e+01,     2.5093150206679911e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3980966331347503e+01,     2.3400881959639046e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3333823871752749e+01,     7.5023091021831806e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9025203818542053e+01,     9.0743929809551300e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.9414163612279562e+00,    -8.4503868382991932e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    8.3707113240679831e+00,    -1.7947417799676082e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.8175104441214458e+00,     1.5611480606446346e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.0475690469007452e+01,     8.0066637603434538e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.6139095241452338e+01,     1.5647176706891869e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.4902470655720606e+01,     1.5647881338808022e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.8437825682040287e+00,     1.6369136612422062e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.9652266631563293e+00,     8.3598698281385277e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4883166362570943e+00,     8.0404451877986139e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.9600584502843965e+00,     1.4861487083463971e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0957023528412400e+01,     6.5585328084567745e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4446084072461762e+01,     9.9717686088083901e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0170615411254529e+00,    -9.5143376696326432e+03,    -3.3757847122298506e-04,    -4.7042252028084853e-07,     2.2672311797087445e-09, } , 
      {    5.9494073692021709e+00,     1.6186819094280700e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.8630518970987948e+00,     1.2554517085368543e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2314648527410895e+01,     2.3190178508087418e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4573645114994747e+01,     7.2670224868306250e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.6229999623358928e+00,     8.4695145738951996e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    8.5479797270123790e+00,     1.7155178551808040e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.9395038191442854e+01,    -1.2197556080913314e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.1646787356502998e+01,    -1.2190509761751814e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.0892635295531967e+00,     8.2407774384028671e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8043625338819234e+00,     9.0088046453259522e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2514617471485622e+01,     1.6433570962235921e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.0822902423857661e+00,     6.3570276205593791e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8368412092422055e+00,     4.9779570256113834e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.1970314967510003e+00,     8.4165454766840085e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.2826636734967693e+01,     7.7713471348283165e+03,    -3.5657537191241205e-05,     3.1631620142623566e-08,    -2.5917654578434687e-10, } , 
      {    6.7043573955749363e+00,    -2.0054452315192316e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.3397526951843158e+01,     8.3200549262406166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7476416892457770e+01,     2.5126897423142098e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.2468989347758050e+00,     1.5254508053156957e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1446270492089043e+01,     8.4772369910485650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2311711988120756e+01,     8.4383971897491974e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3312436900805880e+01,     9.5011953781032789e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5914306802760741e+01,     8.4285351252340042e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.4775846291304167e-01,     6.8011322070250253e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.9704436586826674e+01,     3.2271154824972782e+04,     5.2919711212991609e-05,     3.4393909116523689e-07,    -1.8472855691316647e-09, } , 
      {    1.2983976416324300e+01,     2.4009977494597460e+04,    -1.6100598645129991e-04,     2.0324795471786965e-08,    -2.6359319841925475e-10, } , 
      {    4.1325738462264257e+00,    -2.8824623682483673e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0628426752859401e+01,     1.5681286069974009e+04,    -3.1329839421362609e-04,    -2.3039407868286775e-07,     9.7239080020139618e-10, } , 
      {   -2.3792074683464381e+01,     2.3975868131515323e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.7266586283874204e+01,     2.3976572763431468e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.1568874244849967e+00,     1.7337496069949400e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    9.0484267611669420e+00,     8.3783548220585890e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8547955708219961e+00,     1.5583094673036088e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6493828387503054e+01,     2.5057067579283339e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.5372910277949828e+01,     8.8353035959932495e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2407778002285044e+00,    -4.7056375496620109e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0662521411056812e+01,     8.2069960109477070e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.2588222310003800e+00,    -4.3336135274728144e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.2349806408517786e+01,     8.4154172140357387e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.5605621292823699e+01,    -8.4515151009474648e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0602451978949686e+01,     2.3583199477780407e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5868703353922275e+01,     1.0364789578501379e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.3809166678687355e+00,     1.5664449703657538e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.9910964671011122e+00,    -1.6124242618262440e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4665203851241595e+01,     7.6316575231133547e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.5928689315390900e+00,     7.8581276696572486e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.5425675007623907e+00,     1.2838376419471131e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.6459410368155467e+00,    -3.7260638620024110e+01,     1.5466528323684447e-04,     1.8316866889419651e-07,    -8.5884743608554286e-10, } , 
      {    4.7106106737667375e+00,     1.6937950000727287e+04,    -3.1349425894079436e-04,    -2.3030681222026804e-07,     9.7384524124472463e-10, } , 
      {    4.4890939940562467e+00,     7.2206218209574781e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5578297489450192e+01,     7.0317358714780694e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.2067600194161514e+00,    -1.6694643487866924e+04,    -1.4991953228780789e-04,    -3.1826907941511290e-07,     1.6131205611557588e-09, } , 
      {   -1.1898329538637693e+00,     1.6722342680217138e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0447309362329044e+01,     9.4674139506481188e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8584723392357020e+01,     1.6801972484012964e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.7616584615700759e+01,     1.6749039670916784e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.2082507752690930e+00,     6.8276008512644912e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1237165305303343e+01,     1.6774776820533039e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.1614594129231115e-01,     7.6977808156378251e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.2995360404732175e+00,     1.2024826113256631e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.0315805454910327e+00,     7.2490077543677362e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5983099652013747e+01,     2.2825543471804693e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.6599377915328648e+00,     5.9640066508663913e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4360531196780498e+01,    -1.1789264406641814e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3629600206460305e+01,     1.4496852047181241e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.1192128193577870e+00,     1.0406338740127441e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.8503111527101934e+00,     1.0548607833502709e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3890055409961310e+01,     1.4458963476606300e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -5.9231882977733923e+00,     6.4959830970238500e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.2981479045274176e+00,     2.5124606053862564e+04,     2.0976200519718608e-04,     4.5964482126157486e-07,    -2.3275420016387712e-09, } , 
      {    7.5362033785932194e+00,     8.6349713453889963e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5036454221107149e+01,     1.4985470020511821e+04,    -2.4900122281875227e-04,    -1.5811504925823523e-07,     6.7912215636142660e-10, } , 
      {    1.8501714437233222e+01,     1.7219612901621895e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0585651591715159e+01,     1.6762868927581927e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.8059880115439597e+00,     1.6761446236648175e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1427480690670148e+01,     7.1868403935023180e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.4010170122823212e+01,     1.6761828949740277e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.6618532768036449e+01,     1.6762486214489822e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.6041433767133103e+01,     8.4732810593895119e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.4542363975317807e+00,     8.5912005118320321e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.3578788215539053e+00,     8.3936512555936879e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0299199793756597e+02,     8.4329581988524860e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9973618745075171e+02,    -8.4339741161307156e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0013442605031511e+00,    -1.6795939009570211e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.7291313640565662e+01,     1.0757810548194364e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.2247843267916858e+00,     8.2757318031511695e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4732635285066907e+01,     9.8942163477962658e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0373870875280602e+01,    -2.3700802678019600e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.4157409067041593e+00,    -1.8139482002232535e+03,    -1.8262218196775037e-04,    -2.2032003824035649e-07,     1.0856869093398835e-09, } , 
      {    1.3479590078553750e+01,     8.0868458536717608e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.8630115474329703e+00,     1.6622437960237508e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.2819183653332580e+01,     1.6768191056557804e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.7769548817160835e+01,     1.4357108044972603e+04,    -2.5146923958449248e-04,    -1.5758592360667232e-07,     6.2322798706630828e-10, } , 
      {    3.9011958400504971e+00,    -8.2576981192104613e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    7.8529541204961681e+00,     8.4313239192697802e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7392008247895177e+01,     2.3314161445135273e+04,    -9.6708815056426061e-05,     9.2603824896419465e-08,    -5.5686184225922413e-10, } , 
      {   -1.2956380347836582e+01,     1.5435678357172234e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    8.2935124138831195e+00,    -3.4944889009392369e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.8784577559580091e+00,    -8.6750524666351751e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8347130354961507e+01,     1.5440280757055054e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4058614676702504e+01,     1.6755598626515919e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.0492259750247932e+00,     7.6136427906504650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1134296037445564e+01,     1.6780181790316848e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.3978730276991378e+00,     8.4360959898077836e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.2826620881948507e+00,     8.4308363251754163e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -7.2550294733993219e+00,     7.0296583286943696e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3166149045424442e+01,     6.0115802540326813e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0687320957406579e+01,     7.1768022267383203e+03,    -6.3810300649315353e-05,    -5.4324827409366841e-09,    -3.0882631486677479e-11, } , 
      {    7.6233822443008918e+00,     1.6604423227774620e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3908790516981718e+01,     2.5073928325930905e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3179298205864329e+01,     8.8728877930577346e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9433937267561138e+01,     2.5155283356552358e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.2018088981738146e+01,     6.5606103512404752e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.7005695586800122e+00,     3.2302610672155432e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.3261505706771342e+01,     1.5489794668509514e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.5250100582299751e+01,     7.1092948460409207e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.2212932157356235e+01,     7.1092814189394667e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0802887261534050e+01,    -1.1133368846728445e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.1602946838383104e+01,     3.2307213072038259e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.3506288509029201e+00,     1.6100098579776928e+04,     1.2176676882973375e-04,     2.8120497653155267e-07,    -1.3848266468580895e-09, } , 
      {    1.3196716952671165e+01,     1.5125159632226785e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.7660593127557966e+00,     1.6919891936455482e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.4723582352226581e+00,    -9.7238871353689447e+03,     9.5018675054634523e-05,     4.1705660585134834e-08,    -1.4588043664585935e-10, } , 
      {   -1.9572240811266283e+01,    -8.4177336734923010e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.2799478655294561e+00,     1.5471760984568806e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.2339490581559316e+00,     5.5709856811734044e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.6704398930634206e+00,     1.5647529022849945e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.9391918937158859e+01,    -8.2232827735906540e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.4251626679103659e+01,     7.1889179362860177e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1730820212520754e+01,     1.5059036339702314e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.7123211980141235e+00,     1.6759527749798865e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.0679253675273365e+01,     1.6764787414431237e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.4144207505158235e+00,    -1.2194032921332564e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7299158855657817e+00,    -6.9931964095814783e+02,     1.8814580968073562e-04,     2.1899998754943141e-07,    -9.7826189387963241e-10, } , 
      {   -1.6135918810309118e+01,     1.6784699487247297e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1226395758971330e+01,     1.6781345251871084e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.2726789371336515e+00,     1.5976115642729077e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.4766353232112355e+00,     9.2195080968775746e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4297054055825136e+00,    -7.7714071551534771e+03,     3.0525638932592432e-05,    -3.0486102376897949e-08,     1.4884264823743866e-10, } , 
      {    1.4115137166301958e+01,     1.6808558247988200e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.4314611321960475e+01,     8.0530644262166006e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1869832606417717e+01,     9.8604349203411075e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2210655807851657e+01,     1.5594559925638754e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.0505442242187817e+00,     2.6250908720858274e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0669871980110660e+01,     5.9056420596102980e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.9693436828097992e+00,     6.5147676047312934e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    1.8713876232706198e+01,     1.1150831517887353e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.8313035395733337e+00,     1.7548199521501028e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6485942092273209e+01,     1.5700498120061136e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5245632702468498e+01,     3.5268058663869351e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.9211444530626594e+00,    -5.2959621472281015e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.9727806293199515e+00,     5.1805635656959382e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.0154909811148967e+00,     1.5774383007934210e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.3142334137611531e-01,     1.3853177999221407e+02,    -2.4711521808979244e-04,    -2.9251180120238930e-07,     1.3804099942231817e-09, } , 
      {   -4.3419011631637661e-01,    -6.8126720651782421e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.6010191881084461e+00,     1.6239788191491891e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.5549630242769998e+01,     2.3453851056850235e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.6560690329366148e+00,     1.6709188484903858e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2326645817203151e+01,     1.5644899190533761e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3388323967255269e+01,     1.1739397128613791e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.7105743139220895e+00,     1.6815126679326244e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.9358830527495972e+00,     8.3356950061028856e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6436475753112859e+01,     1.5650158855166128e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7214299602238537e+01,    -8.4274326830488499e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.2686798203238812e+01,     8.9504400540698607e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0833046846463590e+01,     1.6222867510684297e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.6316454144294738e+01,     8.4491986414908988e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    5.6007063084544084e+00,     7.2827891818228964e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.9417909529559267e+00,    -1.7417726827564168e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.4090660882665706e+01,     2.3387727764325766e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1582384857722825e+01,     8.4979005073054577e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -2.4588671406853095e+01,     8.2802400243427037e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0007112105947870e+01,    -1.3771376464736884e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.7852569889095884e+00,     5.5728426946997206e+02,     1.8281804669491861e-04,     2.2023277177775677e-07,    -1.0871413503832121e-09, } , 
      {    5.9574504586581902e+00,     1.0740456518433530e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.0121219397691412e+00,     1.0214490055196711e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.9229714587232873e+00,     7.0628874666170459e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5393324638080387e+01,     1.6617042466192604e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.7520432923011249e+01,     6.8548566928142900e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.9792580708426382e+00,     8.4180470463519232e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.2235069764991872e+01,    -8.4488852686312784e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.6365814977406981e+00,     1.5607714120952032e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.3287588608968690e+01,     6.4806282314865757e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8474506571648604e+01,    -1.8078959752808873e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.0088232818251726e+00,     1.6171086255358434e+04,    -6.6281108487417764e-05,     6.2161355750821374e-08,    -4.0729197350012122e-10, } , 
      {    1.2544843743761517e+01,     1.5117539874129852e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1245451106042066e+00,     8.9081667225412593e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -5.1823256968729964e+01,     8.4212602327676232e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5620555871713840e+01,     2.3818486093132964e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.2011999867123162e+00,     7.0966688940722070e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.1227851706659164e+00,     1.7356702490205047e+04,     1.1643900584391674e-04,     2.8243776075987803e-07,    -1.4937061033616692e-09, } , 
      {    8.9652747212912161e+00,    -1.2723723893444478e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.6712952746017704e+00,    -6.2833196537664026e+02,     9.7932363584127605e-08,    -4.3633231299858394e-11,    -7.2722052166430139e-13, } , 
      {    1.6152310664499822e+01,     1.0287237317489253e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.8043880173776570e+00,     7.0901004627341626e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7077072942599223e+00,    -1.3855616032331204e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -2.4160900481889001e+01,     1.6744884585349384e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.7672349836022438e+00,     9.0280110655815988e+03,    -3.5853401918409453e-05,     3.1718886605223283e-08,    -2.5772210474101826e-10, } , 
      {    9.6563295085771568e+00,     1.2396240262418173e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.6395907049801615e+00,     7.2544032484126383e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.1074642118906066e+00,     5.6949386080586755e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.4481032157150011e+00,    -8.4400251130907345e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1856340133150631e+01,     1.5096773698816527e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3231352129608759e+01,    -1.1664341949220650e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.5893190867904616e+00,     5.9302252234112311e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0304598182598326e+01,     8.8391063656025763e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.2416220221789782e+00,    -1.0459022428005528e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.7962152965725835e+00,     6.4345798815715025e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4322098523467934e+00,     8.9558355481147610e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1249535607667788e+01,     1.1080820731090262e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.0844945119569243e+00,     6.2714412349131123e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.4145137429587617e+01,     1.5687343924747858e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.1666711898669778e+01,     1.0247353292674052e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1153109820802396e+01,     2.4762262386859373e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.7416766990711174e+01,    -1.2204772963381902e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2826386920185817e+01,     1.7043844863340753e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6020096563019614e+01,     7.6600434565236128e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0993621048027753e+01,     8.4614632764590697e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6033813908600703e+01,     1.6173664898967420e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.7234219818039032e+00,     8.4327548120247247e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0956999280450376e+01,     8.4341775029584769e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.0710123464111643e+00,    -1.2220331244494391e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.7997571636416954e+00,     1.7284526972738211e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3180583978145130e+01,    -1.2167734598170728e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.4698236143448607e+01,     1.5437627241155567e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    5.6372240663713967e+01,     1.5438331873071720e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0527985368746837e+01,     8.3232434866255771e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0125103018481788e+01,     2.2685799469596051e+04,    -9.9176831822166324e-05,     9.3132950547982415e-08,    -6.1275601155434255e-10, } , 
      {    1.3079480319955813e+01,     1.4862641069279292e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -9.9351556351909309e-01,     9.0219588406392322e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.1612789136292774e+00,     1.6211402258081631e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2533774915917290e+01,     1.7121397124352879e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.1194621069163642e-01,     5.1779647114804166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8793329392662854e+00,    -4.1759465775500973e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4212853082787372e+01,     2.3425465123439975e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1703726531939012e+01,     8.4269072018924671e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3776021994408048e+00,    -1.2592181940311687e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.9611470894058112e+00,     2.3936405545575482e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.0136392427389787e+01,     1.1543852487580341e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2648826543221229e+01,    -8.3974177410880020e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.4649879711710406e+01,     2.3923251350262202e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1018447605027248e+01,     6.5023104554225019e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.8724510142039357e+00,     8.9842214815250190e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1896844449549466e+01,     2.5266617045019640e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.4959020324710670e+01,     8.7823344987820583e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9925279115410603e+01,     1.7612633871314883e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.4035234699612545e+01,     6.4813328634027248e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8669382338845637e+01,     2.4029189544684588e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.6004645190843512e+00,    -1.7701586161666755e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5625176213694106e+01,    -8.4208469190736123e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    7.5774478635490796e+00,    -6.4406480429890462e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.1734446878854223e+00,     7.8243462422020884e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4694680430817360e+01,     2.3973590615157213e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.8779143000098692e+01,     2.3978850279789578e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.3895770661673108e+00,     6.0879595777516633e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2003810773067675e+01,    -8.4465840686898682e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -3.3524068443321031e+00,     6.9326741601578169e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.6547866524965944e+01,     2.4016035349371308e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3186619181209462e+01,     2.4551558935307745e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.7855292241459040e+00,     7.1032546580474418e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1257391993708941e+01,     3.8646201409385367e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.2646701058529452e+01,     8.5008900365634454e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -4.3805910607736138e+01,     8.4610751346655343e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.6725693353155648e-01,     7.3637195721660974e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1486805456753956e+01,     8.5407049384613583e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3292321381144745e+01,     1.0253455890034094e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0890879699701561e+01,    -1.1795883902353441e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.7741475563806812e+00,     7.1092881324901937e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -4.9149439207311776e+00,     1.7350650265262680e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.5491157485831490e+01,     1.5805263377190377e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.5461941807161637e+00,     8.5009802517397256e+03,    -1.2563945527844893e-04,    -7.8240637817132127e-08,     3.1828018164841032e-10, } , 
      {    6.5182680055520530e+00,     6.5553005690156988e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.2439031415739104e+00,     7.9110967668684398e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.9922501657206162e+00,     7.6579659137399121e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.4398688111885587e+00,     1.7912834557783754e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.4139350450492774e+01,    -8.2216154918138855e+03,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -1.5751579954998284e+01,    -8.3713330076191542e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1312807653656433e+01,    -4.8371795027948019e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.2495780686947491e+01,     8.4955993073640475e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.8317923004113510e-02,     8.4332425003338903e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.3690852173999026e+00,     8.4336898146493113e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.6035433194857852e+01,     6.4832817473860596e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.5754394972861014e+00,    -8.4174666492676042e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4018611216622443e+01,     3.3385783384237890e+04,     4.2368770286147758e-04,     7.8325911695502479e-07,    -3.9112343723511808e-09, } , 
      {    1.9204670503975199e+00,     7.6474242181056252e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.1958819938887495e-01,     8.4494656657155974e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1229240266325903e+01,     1.7312912906148467e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.5891156293895037e+01,     8.5087172185623131e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.2288558589059853e+01,     8.3582150964208886e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3675054656899837e+01,     7.8449734743439694e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.2261859226339936e+01,     1.5474027973517244e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -3.0831380570042359e+00,    -8.2239166917552993e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.8203457808471968e-01,    -9.5143132893015463e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.3455369115726903e+00,     7.7184080477797061e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0964230297221253e+01,     4.6401427510597784e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3307041300404290e+01,     1.5404222509989580e+04,     1.8093204196595883e-04,     3.5462952372191079e-07,    -1.7884291882449669e-09, } , 
      {    1.2018870918398230e+01,     7.6938248839787739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7571565870731487e+01,     1.0680258287182241e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4516681766012871e+00,    -4.4598059116526838e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.8119193418136010e+00,     7.8827108334581817e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5200905184824675e+01,     1.6591305316576352e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.7164214653078620e+01,    -8.2262178916967096e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4207117458411219e+01,     9.3434610237628476e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.4217213453366710e+00,     3.3595332849974191e+04,    -8.9094434161419785e-06,     2.7113093608904140e-07,    -1.4981227559965770e-09, } , 
      {    1.9957473464590819e+01,     1.7010063435885593e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.4232477658249046e+01,     8.6376011777051790e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4600092270542808e+01,     9.2659087627507215e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5259230574139702e+01,     8.4203482462933334e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    9.7021569083156152e+00,     1.5044958587420655e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.4533945197538422e+01,     3.2271130444641683e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -6.1042859020744089e-01,     4.7849437417874296e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1558846774215446e+01,     1.1936873457273328e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6672220502235838e+01,     7.4570155633168088e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1413162221468033e+01,     1.4651937617727670e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.5268809939222528e+01,     6.4786793475032409e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.0822993646046601e+00,     8.1643981146838842e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.9509188038286869e+01,     2.5090496690780426e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    5.1612011658415156e+01,     2.5091201322696572e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.2236301510932126e+01,    -1.0616689377928246e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7648450813678384e+01,     8.5236376569217118e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3649811734032390e+01,     8.1054383884626250e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6536045582096168e+00,     9.5311740359491141e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.5485579372876153e+00,     1.6685716338704187e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.2735382093905432e+01,     7.1072105897064939e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6487935586112954e+01,     1.5569976761837821e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.3415208215111214e+01,     7.1113656752738943e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.5266316558156650e+00,     5.3303035717759467e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1943162538438036e+01,    -2.7562699840684991e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2835128575724276e+01,     6.6398878390782247e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.9790411065819553e+00,     2.8333489457286738e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4706253429632055e+01,     8.4003960755033168e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0252374561337081e+01,     1.6224021496499618e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.6335292697691603e+01,     3.1729573384262498e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.9290922005580335e+01,     2.3350214062369265e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.2699139959017396e+00,    -3.9957992529212106e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.5390261782497976e+01,     3.1156526265707678e+04,    -3.1784828043549440e-04,    -9.5380934624551073e-08,     2.1666323408785139e-10, } , 
      {    1.5484857074310812e+01,     1.6169898413473105e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.6589604322002405e+01,     2.4369241417166384e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.1832973945280713e+01,     7.1096167648649662e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.8795808615862292e+01,     7.1089595001154212e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.3686288956673991e+00,     6.0415589118785156e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2116716597713783e+01,     1.6682527778319225e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.4300972808421580e+01,     7.1306180404680454e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2870936443481710e+01,     8.4423445459921440e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4439277822810485e+01,     2.3800428028861159e+04,     2.7159115982631967e-04,     5.3245297633777035e-07,    -2.6767048147738589e-09, } , 
      {    1.7829844620384531e+01,     2.4133954801813827e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.9512568639632118e+01,     1.6841787385910873e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.7072628827058800e+01,     8.5130959612874249e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6153284985367662e+01,     1.7373544486352999e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -8.9726099274919395e+00,     8.0258085846668000e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.1731543638182051e+00,     2.3940172031069797e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.6737664015809365e+01,     8.2107987805570319e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.8682400567708768e+01,     6.8946715947122029e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6421959165308618e+01,     2.2878512569015886e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3433352511848847e+01,     1.6588656532782348e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    8.8390200247050625e+00,     9.0279923150819832e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.0232011434075652e+00,     8.2862734987854565e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4389479685836198e+01,     8.3538363536957768e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4558742256139068e+00,     7.0281959946114029e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.6452679689419920e+00,     8.2445802080121903e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.1654355862402497e+00,     6.9077829445927982e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2648846694383284e+01,     7.0879582245123420e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0308580500806942e+01,     1.6727664809193015e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.3162187661017352e+01,     2.4012268863876991e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    8.0246861824170299e+00,     7.7713715151594142e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    2.4714755771368282e+01,     1.0646476859727081e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5339529046848924e+01,     3.2338668919220912e+04,    -8.7135786889737233e-06,     2.7104366962644168e-07,    -1.4995771970399055e-09, } , 
      {    1.9775235149452044e+01,     1.6984326286269341e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.8172477511622924e+00,    -1.0796836702557132e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2296354489738842e+01,     1.5725081283862070e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.6349344810690496e+01,     6.8150417909163780e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2152345375542197e+01,     6.4816918929115272e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.2012998519894026e+01,     2.3345611662486444e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.1757627405449780e+01,     8.8690850234484096e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1307324397292620e+01,     9.2323213390842184e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3229550112437421e+01,    -1.2969555531453807e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.0324710962255885e+01,     8.5805198403592713e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4807180383976974e+01,     1.8440453666397525e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -8.2134263279106570e+00,    -4.2417925837994517e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5630025840762029e+01,     9.7364819934558345e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1347476076773660e+01,     1.8005654841007872e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.6841328628686114e+01,     2.5144929072326249e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4581947419547799e+01,     8.3129342053700893e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1329961712056338e+01,     8.6038197502500188e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3034707755709949e+01,     2.2827834841084226e+04,    -4.7014068819782058e-04,    -3.4609980877920578e-07,     1.4526472327085022e-09, } , 
      {   -1.4683864234178170e+01,     8.6143012678144314e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3126206617300674e+01,     1.6257802923954781e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.1728569356691793e+01,     9.2321273352955632e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2600560265766411e+01,     8.9750232178707920e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1568083942927828e+01,     6.4280114502334591e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.1644756353698888e+01,     8.4793145338322647e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1314327258542171e+01,    -5.4479379978736446e+00,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -7.0539354235764321e+01,     8.4152267583248704e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2981268823716725e+01,     1.2329894426966315e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5859769159544861e+00,    -1.6779078262922645e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    4.1699143661410556e+00,     8.5110184185037251e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2283320059106604e+01,     1.6013318044947995e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -4.9558833382962888e+00,     1.6824514389145206e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.5763196172806913e+00,     5.3832726689871388e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.6144927764217734e+00,    -8.4142784877355716e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    9.3113193250012046e+01,     1.5647522309299220e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.4349840697779683e+01,     1.5647535736400670e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8990219722673253e+01,     1.1073279256875230e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.3979010438121078e+00,     1.6586365163502811e+04,     4.9006674371247945e-04,     7.2105412797290353e-07,    -3.5046696193727241e-09, } , 
      {   -8.7711093103682572e+01,     7.1091900809163544e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0839075023283621e+02,     7.1093861840640329e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6494853837837756e+01,     1.0707593280956098e+02,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    8.8200192335677272e+01,     7.0937461042017640e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2026849214867191e+01,     2.3976220447473395e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4064965762954827e+01,     1.4549821144392432e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.0328255628011362e+00,     4.3919227720944418e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5888515386245427e+01,     8.5744863659165185e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5159044185066204e+00,     8.3559138964794765e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0670549165027877e+01,     1.6684605321102925e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3956961967931818e+01,     1.7514418094045868e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.4277733068923677e+00,     6.9853352056049225e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.7523916864968214e+01,     7.1248301607786234e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.3696196290337390e+00,     8.3920581464101706e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2876713101207654e+01,     1.6749706062637742e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.4669001109186201e+01,     3.1518869932710870e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.6337923095913691e+01,     7.0923674516826022e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    4.3157522083535468e+01,     6.6004252531383872e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4728106745495474e+01,    -1.5757219253361384e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.6055655042019215e+01,     2.5298097272533392e+04,    -3.9726088367124645e-05,     2.3468322531964382e-07,    -1.3242685699506974e-09, } , 
      {    3.5247864942004497e+01,     2.5302699672416205e+04,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {   -2.1442104150393476e+01,     3.2304559556138771e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    5.9627733145100933e+01,     3.2305264188054920e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    6.7312868074997301e+00,     1.6839709843127173e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2126849309346063e+01,     1.4843434649023646e+04,     1.2196263355690202e-04,     2.8111771006895295e-07,    -1.3862810879014180e-09, } , 
      {    1.2234442763054155e+01,    -6.9703390151009592e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.6934985176366432e+01,     1.5021522637745813e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.3927702701438209e+01,     2.2772574374593503e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.8835704997019008e+01,     2.3898668186461273e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.2911323993526089e+01,     1.5098851241600227e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8689421159254373e+01,     1.5421058876306053e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.5517642359427512e+01,     3.2340960288500439e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.1199298836829382e+01,     2.4697828037045510e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.4034177694144322e+01,     1.5580014928601819e+04,    -2.2084845936067807e-04,    -1.2105094637467500e-07,     4.5082824206375731e-10, } , 
      {    9.0406249392705451e+00,    -1.1418510311211321e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4570170380590096e+01,     1.6861484376985329e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7935565479601980e+01,     1.6851175095729839e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.1731386775629804e+01,     7.3239046702681844e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3030995042032249e+01,     7.3623137729174232e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.4011101920360716e+00,     8.4952447039142316e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.5125579101918252e+00,     1.6937925620396192e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.4600546404980978e+01,     1.6731005986976077e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2860471786551916e+01,     5.0661217136979906e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.9757475307996817e+00,     6.7938194238093301e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.3036979123978956e+00,     2.4515510518904150e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.1240757075404728e+00,    -8.1901596446312360e+03,    -3.9940762585211862e-04,    -5.4323067535704402e-07,     2.6163939928438321e-09, } , 
      {    1.0850838260877286e+01,     2.2797157538394433e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.6505979802951369e+00,     7.7162658095578854e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.3897631102278797e+01,     8.2464585968875435e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0406195630987751e+00,     1.8232699388027504e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -6.3427416491179898e+00,     1.5567899219054121e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.6369077077447560e+00,     8.4514903656934002e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -7.7469648790405432e-01,    -7.0732397160865949e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.6923987652518569e+01,     1.5660148261267936e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8091770995318669e+01,     1.1387225260246682e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.8503383840255929e+00,    -7.9260089498510854e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0808888358117544e+01,    -2.3171111705907688e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -6.7605650416200049e+01,    -1.2193965785825299e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.9857383303549952e+01,    -1.2194100056839829e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.9201965433305102e+00,     2.7222471856026459e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.4368547211486799e+01,     1.5016920237862993e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.9592680876710983e+01,     2.3959299766665798e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -4.9884891042180657e+00,    -1.0742881762108116e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2363357544012594e+01,     2.2561870923041875e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.8374129384564515e+01,     8.4246060019510587e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.1629939871378742e+01,    -8.4423263130321466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0640477094640403e+01,     7.3008039142857860e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6021431972633888e+01,     9.6589297324437102e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0858859880849423e+01,     5.8924878642970198e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1122555733689250e+01,     1.6692267804096267e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.0261536836208442e+00,     6.4809805474446503e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.5386393992091033e+01,     3.3417239231420543e+04,     5.4516346567282114e-04,     9.9753028034028185e-07,    -4.9733641849259522e-09, } , 
      {    1.0113280100729519e+01,     7.3445747478427493e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0903535789181204e+01,     2.5483869976431488e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.7055250608782877e+01,     1.0129502963148821e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5273874743239134e+01,     2.3427542666223675e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.3212628880015362e+00,     1.5660646934048213e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.9477083510253081e+01,     8.2404251224447908e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2958000411758146e+01,     3.1911890902403858e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    6.9888635519147666e+00,     8.4203032925474163e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4853472718500642e+01,    -1.6756124107672298e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    3.4422065506999118e+01,    -9.5457935168152944e+03,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.9838405646322428e+01,     1.4985445640180726e+04,    -2.4890329045516808e-04,    -1.5815868248953511e-07,     6.7839493583976237e-10, } , 
      {    4.3232140442123379e+00,     7.4035344740640094e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.8264768500244212e+01,     6.2265491077687475e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9481986550311934e+00,     5.6485379421855278e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.8658447502510302e+00,    -1.7466451088693549e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1562986759900740e+01,     1.4443882949970050e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.9654814673932933e+01,    -1.7680810733829749e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.7998382875701715e+00,     7.0291060391618857e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.3317657088880761e+00,    -1.1518891978851298e+03,    -2.1610270841164155e-04,    -2.5615135689559141e-07,     1.2051013671339735e-09, } , 
      {    3.5917135975538983e+01,     3.3421841631303352e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.9696075127932438e+01,     1.5647200390475175e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -1.0932789815478305e+01,     1.5647857655224718e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.7958959130232337e+00,    -8.3636363136328437e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -6.9458822987739985e+00,     1.6753521083732219e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6137169228921348e+01,     1.1039497829420070e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.8951580544464219e+00,     8.6939310716102555e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.0171919784921206e+00,    -1.0123433204591058e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    5.2118531686071217e+00,     2.3911786097659537e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.4403644441768936e+01,     1.2722915396659304e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4543137045302952e+01,     1.6650823893647768e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.4829110084098588e+01,     8.1452532903605370e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.9771838858227614e+01,    -8.4507391542572695e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.1737547018901978e+01,    -1.2531603392573197e+03,    -1.2365277355869353e-04,    -1.4680822458739865e-07,     6.8353880899633458e-10, } , 
      {    1.7864080448899649e+01,     2.4498589838096555e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4680559147227317e+01,     2.4053772708485518e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.6129039888608734e+00,     1.5646817677383069e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.2180844155243058e+01,     8.4219761696348742e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.5436654642057285e+01,    -8.4449561453483275e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2866340983419537e+01,     1.5648240368316821e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.0201616036499242e+01,    -1.1341123125098488e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4123769779955744e+01,     7.5786884259021626e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.5112876442450291e+00,     2.5666187494572849e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    4.9789858217143212e+00,    -3.2202766427999723e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.4376778754260560e+00,     7.3019768515789283e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8420893977338768e+01,     1.5393097583174042e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8754952447161209e+01,     1.3748012510014189e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.4232626389320860e+01,     1.6040549992542932e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0359571606726748e+01,     1.2062853809349890e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6525424737396374e+01,     9.4106420838276026e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.1249434603439560e+01,     1.6136151197010917e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -9.5997982065052554e+00,    -1.2990330959290809e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.6881492023841115e+01,    -9.5503959166981185e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4724220520059106e+01,     1.5280245202773212e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.3559678351329516e+01,     1.6757226549857452e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -7.2705499773000319e+00,     1.4321049797907130e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    8.4943034496634375e+00,     1.4468466113770983e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.5120938517853396e+01,     1.5727158826645769e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5548759384569815e+01,     1.5507809400972403e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.5119918892203522e+01,     2.4040654797287247e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    6.6890086984576742e+00,    -6.5589555991336965e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2193958800445827e+01,     2.3314137064804174e+04,    -9.6610882692841927e-05,     9.2560191665119606e-08,    -5.5758906278088846e-10, } , 
      {    4.2051611147950382e+01,     2.3975891815098621e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -8.5771720618808267e+00,     2.3976549079848173e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.3722343546696210e+00,     8.3804970602804096e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5689462715992285e+01,     1.5831000526806631e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    6.5320678191506731e+00,     7.7713771449908972e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.4550326902151998e+00,     3.9989018024014540e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7209047420341673e+01,     1.6213479800865332e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0408391000347866e+01,     1.1466300226568219e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5034648809808520e+00,    -8.3870654916184540e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -1.2006638432832983e+00,    -7.1262088132977869e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.4032964856492995e+01,     1.4325652197789950e+04,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    2.6615073398045004e+00,    -1.7843029094256090e+04,    -4.8987087898531118e-04,    -7.2114139443550325e-07,     3.5032151783293953e-09, } , 
      {    2.2770049926700779e+01,     1.8398675810700861e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0008484814468627e+01,     1.4233179498418427e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.0389066285062061e+00,     1.5398164655215731e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    8.3063990182838801e+00,    -2.4230493650131516e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.8933565714482327e+01,    -8.2242690077133757e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.2191929275229967e+01,    -8.2235643757972230e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    7.7732031546876001e-01,     1.6236021705997577e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.4250658083512567e+01,     1.7436865833033742e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.0367963721305724e+00,     8.4864352547027920e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.5674539689977809e+01,     1.5477794459011557e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.9990752737621769e+00,     2.3896590643677569e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0351951630829303e+01,     9.4210132847749701e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1659522037310817e+00,     6.5923142359119347e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2213468707783079e+01,     7.7352945175943260e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.0031595348060382e+01,     2.3347560546469780e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.0155961978428635e+01,     7.9859936827688889e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2165677047922166e+01,     8.9821439387413193e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.3614047470000799e+00,     6.4007984541163423e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7632468463683885e+01,     1.5490948654324835e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.4668085388229933e+01,     1.6767088614372649e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.3453453531501536e+00,     7.1857293759010572e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.5557386453569233e+00,    -1.2320225305512436e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.0921119039989451e+00,     9.5841431331603053e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.7455082568091651e+00,     7.8489294060030215e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1868145958241156e+01,     7.1894702258185007e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6137900050876297e+00,    -1.8139238198921564e+03,    -1.8272011433133451e-04,    -2.2027640500905660e-07,     1.0864141298615479e-09, } , 
      {   -2.5058935890482225e+01,     8.2066436949896306e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1331269720204503e+01,     7.7375957175357371e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.7066630257523464e+01,     8.5346714640186074e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.3372050375076103e+00,     6.1655118387637867e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8483658523126195e+01,     1.0522523932841810e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0460872873842767e+01,     1.0005550036263550e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9831907602071862e+01,     3.3455588847765546e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3653465862345072e+01,     1.6886110509000326e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.2337896805325697e+00,     5.5372042537182442e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.2381548616681556e+01,     1.5018869121846326e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -4.5704606832854680e+00,     7.7315622430929852e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.0147420531291722e+00,     3.1187982112890328e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    5.2408778161454572e+00,    -2.3984662012122189e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.3151247349998009e+01,     9.6251483049885501e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.0054380668371813e+01,     1.6131548797128096e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.4652277622329764e+01,     1.4915610166490484e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.5275486108999269e+01,     7.8177778108640441e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3833918289973500e+01,     1.5385010459902453e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.7288775188810106e+01,     3.1192584512773148e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    8.4341761779689222e+00,     6.6387149017850816e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.9002903630575432e+01,     1.4322998681890464e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    6.2048547026430896e+01,     1.4323703313806616e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    6.4006100492395550e+00,     6.1217410052068235e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3496563758253100e+00,    -1.0334136656142680e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3198314119121331e+01,     6.5236403634003536e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1808610366337220e+01,     1.6242073930939943e+04,    -2.5432898580456928e-04,    -1.5688226502990992e-07,     5.7024269985784707e-10, } , 
      {    1.5566799714313053e+01,     6.4836103797608357e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5379062692663737e+01,     1.7907439063738857e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6704080317378598e+00,    -9.7238627550378478e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -1.7960838437414509e+01,    -1.6780206525570913e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3804105511803254e+01,     1.5435349724797459e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.1702774401035423e+01,     1.5864781954261793e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0643970077738574e+01,     2.2954891892734864e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.6167630424814607e+00,     1.6997444197467605e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1390965335880415e+01,     1.6597864272175488e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0099303922291192e+01,     7.1085767870233167e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1832568406370427e+00,    -7.3596329353072633e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.2549847201246635e+01,     8.4344325598926625e+03,    -4.6568873433953641e-04,    -4.8106931960622267e-07,     2.2091020193437112e-09, } , 
      {    1.7869353557420517e+01,     1.5440609389429826e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.8158516475412174e+00,     6.4411656455467382e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0730912872541101e+00,     1.6793335985630129e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.6163642309768072e+01,     7.6181510118420010e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2753532915488805e+01,     1.6507726142439147e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.8573893172264846e+00,     1.5078740014875817e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.5165527842957111e+01,    -1.4738347318091596e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -4.8239658508386604e+01,     8.4948565621206944e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2059990439081773e+01,     4.0599846249596230e+04,     2.0521211897531780e-04,     5.9465796531989161e-07,    -3.0832695677523156e-09, } , 
      {    1.5747192835574182e+01,     7.0755067050350344e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.1919502494266972e+01,     1.5040551345782254e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.3536314909197849e+00,    -1.2201146376001327e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.0176088893881641e+00,     1.7859865460572561e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.4526381621419780e+01,     6.5345251411786944e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.7527280058494847e+00,    -2.6337528165647745e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2384469831210033e+01,     8.5703524479896532e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.5352564535198688e+01,     8.4560080626238450e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    9.6142432331244017e+00,    -1.2186919466663801e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.9973339286103730e+01,     2.5077731095540235e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.3178013469255383e+01,     6.2663640096666595e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2961174541026875e+01,     1.4357132425303700e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    9.5468738354374509e+01,     2.3976213733922668e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -6.1994290044308357e+01,     2.3976227161024122e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4152578705321895e+01,     1.6100074199445829e+04,     1.2186470119331788e-04,     2.8116134330025281e-07,    -1.3855538673797537e-09, } , 
      {   -9.7991131109502447e+00,     1.7310835363364771e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.2525034211149162e+00,     7.0655172989332314e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.1350779216012974e+00,    -2.6906804280771610e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3809143985844936e+00,    -8.4154419492898032e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.6999031258847808e+01,     1.7388163967219181e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.3194091389386276e+01,     1.4849486873966012e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -4.9150009132445014e+00,     2.2649741222530582e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    6.6610840987115232e+00,     8.1878083411916750e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6796948195958652e+01,     7.1099994779570707e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8082305921476120e+01,     7.8111920468888093e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.0892928585470809e+00,     8.2419503756960075e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.6388519775699194e+01,     2.2654343622413402e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    3.9090052309254219e+01,     7.7398969174771446e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.7667324718773639e+01,    -1.2217044920746666e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.8144207958816416e+00,     5.8864412691861362e+03,    -2.7665892712515666e-05,     2.9782443800135566e-08,    -9.5857361028977469e-11, } , 
      {    3.1863816728701117e+01,    -1.1397734883374319e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1632536305963477e+00,     2.5051034104840586e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.0946916344723071e+01,     2.5130663908636412e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.0776211012505215e+01,     2.3721789007797495e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.3982302118034307e+00,     8.0656234865647129e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.8761340648142637e+00,     1.5654087978449079e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3289757186339703e+01,     5.8953327783548102e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.6515991702582681e+01,     8.4161931607259321e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -1.4112997741672618e+01,     1.6202542102541091e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.7190521320659116e+01,     1.6207144502423911e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0396461658848773e+01,     1.6675407057448698e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9514316572765331e+01,     1.5317434957176565e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.8669747106455157e-01,    -1.3051188248440485e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.6082985953663446e+01,     1.5624987117717701e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.5013340179403851e+00,     1.6526870966762497e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.2480731096385131e+01,    -4.4390304838156794e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.6597017028536674e+00,    -9.9293314558936152e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.5825991976293487e+01,     1.3115936366352289e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0994610518011839e+01,     6.4472291301520700e+03,     3.1303515696541163e-05,     1.0329425745309340e-07,    -4.9800546137252638e-10, } , 
      {    1.4164164218592322e+01,     2.4570765355563391e+04,    -1.0203657804224310e-04,     9.3836609124744798e-08,    -6.6574129876280376e-10, } , 
      {    1.1969508755324831e+01,    -1.3951957107454946e+03,     2.4731108281696070e-04,     2.9242453473978953e-07,    -1.3818644352665102e-09, } , 
      {   -5.3634961470831648e+00,     1.7016589021790955e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.7443006317188448e+01,     1.0051950702136699e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.7228257419998858e+00,     8.3847283968983265e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5778563806710412e+01,     7.2152263269125760e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.8266302424011620e+00,     1.8024208201798999e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9280448195660455e+01,     2.3988839685891384e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    9.7825195530053630e+00,     2.2197235886759147e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0266317517216294e+01,     1.6762157582115051e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6389731720932350e+01,     2.3908706353225269e+04,    -6.8556051598351906e-05,     1.2966792777997972e-07,    -7.8515575655689352e-10, } , 
      {    1.9561960568291152e+01,    -9.8741258425399792e+01,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    5.7816945847582772e+00,     2.2872765975342213e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.2136285423051500e+00,     6.9929526062705077e+02,    -1.8804787731715151e-04,    -2.1904362078073129e-07,     9.7753467335796819e-10, } , 
      {    1.1797017348150996e+01,     7.7713827748223803e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    2.4218809291685456e+01,     1.5502413906927502e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.3837806973835587e+01,     1.6171061875027337e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8504741461260849e+01,     1.5653562497292698e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.7559503491456894e+01,     1.1432518799113059e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7475542532983741e+01,     3.2369346221910699e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -3.5637460011451068e+01,    -1.8480631931368744e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.7506229182489236e+01,     2.4055850251269221e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4274172659835509e+01,     2.3369242770405705e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.0074970471753218e+01,     3.5920617254405397e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.7291086255112916e+01,     2.3348265178385926e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.2844219267001336e-01,     5.2555169724925408e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.9092221199148334e+00,     8.4416332005252680e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1396796259877888e+00,     2.4564713130621025e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3601492466755605e+01,     6.0645493512438725e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9802272376844897e+01,     1.5693929688723096e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0044507208976410e+00,    -1.5786428343710836e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1884462453701461e+01,     1.0398571005956539e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2934848594066054e+01,     2.5091560352205375e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.2830522104966201e+01,     1.6335355184966904e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.1677950200207317e+00,     2.5090137661271623e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -7.6985452356360913e+00,     8.6878975971675045e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4846304884001977e+01,     7.8847883762418824e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.2898418043606661e+00,     1.4626200468111414e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7096918418439969e+01,     1.3537309058462566e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.6203503461632822e+00,    -1.2128443365341227e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.2279767608655643e+00,     7.8515592383192052e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9911149943258792e+01,     1.0915544902534797e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.1379544977335581e+00,    -1.2254367665760083e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8045811926765154e+01,    -1.4965670680775247e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.8978475742322381e+01,    -8.4442828687484562e+03,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    5.4926781882433104e+01,     1.5019573753762477e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.2234286015749213e+01,     8.4226494462347455e+03,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    8.9001814367920806e-01,     3.4006134822070646e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3271384792059417e+01,    -1.6749538343697062e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {   -4.8773973545766811e+00,     3.6058808327084671e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6637658681507439e+00,    -1.8263809692009477e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.1108476168904755e+01,     2.2076703934637640e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.1389049775709736e+01,     1.7403084405578582e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    8.8021470049911592e+00,     1.5629504814648148e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.5556560839021278e+00,    -7.1504863397298482e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6912024204202257e+01,     1.5666716692605976e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -4.1960531351999397e-01,    -1.9716638040640712e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5774844635405756e+01,     8.3659033025812805e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.3075019490113746e+00,     8.6541161697123443e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2702375523361726e+00,     8.3444824890298932e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2725248071607140e+01,     1.6727212693105659e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.3739199095482881e+00,    -8.4166054108440349e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.2673116894697726e+01,     7.7948298378684149e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.6999189075726662e+00,     1.2610464084600257e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.5852501027081978e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4304142404176268e+01,     1.7392766367102002e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2621101929873774e+01,     6.8740015171376381e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.9580267181442306e+01,     8.4345620383590867e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.2836077668256522e+01,    -8.4323702766241167e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3311225966084137e+01,     1.4887224233080226e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.4116147982197429e+00,     1.6783487490092903e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.9791569153606794e+00,     1.6740827674137199e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9335206841143098e+01,     8.3685331348974651e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1947896359012667e+01,     3.2287991191289246e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    9.4678633258401348e+00,    -1.3971962187754312e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1826009156975957e+01,     1.1859321196261202e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.1735485012082808e+01,    -9.5484470327147810e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.9276061218077942e+01,    -9.5477424007986319e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.1168043829675556e+00,    -1.8052578559992395e+04,    -5.7273732707691651e-05,    -2.0901321356951991e-07,     1.0901035619747915e-09, } , 
      {    2.7745815809286004e+01,     3.9485217690331127e+04,    -1.6555587267316819e-04,     1.5533793953010366e-07,    -1.0193207645327994e-09, } , 
      {    1.9747562274683204e+01,     3.1642852869758721e+04,     5.5583592705900127e-05,     3.4332269905107421e-07,    -1.7928458408798751e-09, } , 
      {    4.8657937520453434e+00,     1.4983173021400804e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.4801294309799040e+00,    -6.3091060898562546e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.9842386761687202e+01,     6.9344864966101150e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.4713886048044813e+00,     7.7368843720688601e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6921258776147539e+01,     1.5595713911454075e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.9193719270169868e+01,    -8.2937807466070317e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.8397612665086104e+01,     6.9898434267964567e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.5937889575295813e+01,     1.5109392937234514e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.9112662047602196e+01,     8.9085476093882462e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7598702102023367e+00,     6.5147619748998113e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5936744594484392e+01,     8.5731515683761718e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.4414736188055599e+00,    -5.5730864980107003e+02,    -1.8272011433133451e-04,    -2.2027640500905660e-07,     1.0864141298615479e-09, } , 
      {    1.8708787856133451e+01,     8.4641190798502976e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    6.1585910943914310e+00,     7.1217396519675021e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5182190936288645e+01,     1.6100038559451768e+04,     1.1663487057108498e-04,     2.8235049429727831e-07,    -1.4951605444049977e-09, } , 
      {    1.8331030029771419e+01,     3.1154234896428145e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -9.9946470126395326e+00,     1.6744108638659189e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.3285032535357422e+00,     8.6687527728441564e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.3402560263124279e+01,     1.2419451972655006e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.6266942386823331e+01,     8.4355437002753024e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.9869523207137483e+01,     8.4313886147078993e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -9.2307621792792549e-01,     6.8613822787196505e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.4542985913222206e-01,    -2.5124630434193663e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    8.5691939137333186e+00,     9.0279866852505002e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    1.0924735723216571e+01,     1.7356678109873952e+04,     1.1653693820750085e-04,     2.8239412752857817e-07,    -1.4944333238833335e-09, } , 
      {    2.0376745294577169e+01,     8.1790347178156971e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0249111688308764e+01,     8.1540269137365149e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.2376068762607666e+00,     1.6155179905047360e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.7343138459326788e+00,    -8.4356083957134233e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.9032960191672217e-01,     1.6688561252761978e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5388320195296592e+01,     1.3930330028155551e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.2516643474560709e+00,    -8.5731857793691452e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.4604125540062824e-01,     1.5901960462525849e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.4296232906206123e+00,     1.5190073703343100e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.4091642342163098e+01,     1.5664802019615614e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.5650477247507652e+01,     9.0306221473981677e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0534755317416437e+02,     1.6761649623475936e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.9738063358657655e+02,    -1.6762665540754166e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3027102104180830e+01,     2.3381675539383399e+04,    -1.5834210495839141e-04,     1.9708403357624298e-08,    -2.0915347016746493e-10, } , 
      {    2.4192570558082032e+01,     1.8791696780393846e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.4192193167600333e+00,     8.9881774131840721e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1801786913693633e+00,     3.1178403515078116e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.0175002984630570e+00,     7.3357582790340884e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3185895515339197e+01,    -8.7620527773707308e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.5642753297315437e+01,     1.5895434876620489e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.5067697338449490e+01,     1.4287302581444939e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    8.8428623561160258e+00,     2.2250204983970340e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.0298514906177925e-01,     7.4373159015191695e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.3819769350848325e+00,    -9.5818517638807716e+03,    -2.7594518132101971e-04,    -3.9752709874205337e-07,     1.9195228076169856e-09, } , 
      {    5.0344199267045937e+00,    -2.2407318468717872e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.7541450369005080e+01,     1.6760080039331351e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.8649967122423547e+01,     1.6764235124898751e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.1176759632474372e+01,     1.7377347255962330e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.2816870084385929e+01,     1.8334241460886999e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.2557014564690853e+01,    -8.4421026558744361e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7904280138896482e+01,     2.3836500825595853e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.5151153959668746e+00,     9.0404438345592916e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6352695062008387e+00,    -9.9411156864496952e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3489570322202436e+01,     2.5108873214940300e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.0209443612320523e+01,     8.4099940112919285e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7560971699691823e+01,     1.6668857577588475e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0735117233201009e+01,     2.3993141128280990e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    5.1670782556102864e+01,    -1.8473585612207248e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.5940276499030169e+01,     8.8349512800351731e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4573996933831690e+01,     1.0018169274681537e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0916083132236404e+01,     8.9220541206596008e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4378874712732417e+00,     1.4685719045182830e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.4293837525329689e+00,     1.3868544462135698e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.6413803586082313e+01,     2.5084290051139371e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.0318938668366631e+01,     8.3577362595476134e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8511341419845863e+01,     1.5929216304075650e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.0323945190247230e+01,     8.8687327074903351e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.0643194600651739e+01,     8.2006102205468796e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2010134374600092e+01,     3.0614944824997387e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    4.4865769331554466e+00,    -3.0010162579856114e-02,    -2.5659491293243832e-06,     5.7275888286280838e-10,    -5.5166948773454092e-11, } , 
      {    1.6801887286652349e+01,     1.8300460033431838e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.5460072539703207e+00,    -1.2374275003350554e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3738844834221570e+01,     1.5178128729437976e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.4416630716374659e+00,     8.2930351901471822e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7248279069761995e+01,     1.3508957336045278e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4300477843802724e+01,    -1.9807473739472573e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.2070348695354127e+00,     7.8773153394132796e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.3450150178889082e+00,     4.0631302096778883e+04,     3.2668788178666131e-04,     8.0892912870514867e-07,    -4.1453993803270866e-09, } , 
      {    1.4813319328088317e+00,    -7.8308957220623124e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    6.9785772367531862e+01,     8.4326541118517052e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.1338149044956360e+01,     1.1308565872227784e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4540467049520419e+01,     8.7293654015708671e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.8170338024027961e+00,     1.8837592935823975e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.9859381335263366e+00,     5.7287200355138357e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8388885072965215e+00,    -2.3646847737570583e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0860363831131458e+01,     2.3982253921916148e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.0632629411256933e+00,     8.1981795421390452e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.3702918315608521e+00,     2.2859290688266876e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    9.9792156336943840e+00,     2.4933114652398068e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.1780130933524974e+01,     9.8140342544679588e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6530001499839486e+01,    -8.4342782031315000e+03,     2.2293307344401761e-04,     5.2439726373108627e-08,    -8.6296835237497348e-11, } , 
      {    9.4435725539250619e+00,     1.6569468863026315e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.7769559752347919e+00,    -8.6169376614482881e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.3305924805142750e+01,     1.0791591975649526e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.8211055620454530e+00,    -6.9437661931429091e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.3958508956035018e+01,     4.0635904496661708e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3721618882754466e+01,     8.2944578810809344e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6651149602755549e+01,     2.2651690106513917e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    6.4407931428663218e+01,     2.2652394738430063e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.5979502306905813e-01,     7.1175240993584214e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.2154486626071961e+00,    -8.1517788762658984e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    5.7111156707370995e+01,     7.7379480334938116e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.9844471801455832e+00,     2.5334155519598866e+04,    -2.2283514108043350e-04,    -5.2483359604408485e-08,     8.5569614715833047e-11, } , 
      {    1.0471081660982115e+01,    -5.5734428979513268e+02,    -1.8794994495356738e-04,    -2.1908725401203118e-07,     9.7680745283630396e-10, } , 
      {    1.8219746909093207e+01,     1.6006768565087774e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.7905285069750203e+01,     3.1782542481473687e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3062782660165213e+01,     1.5019932783271279e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.7694569526736132e+01,     1.2021302953675881e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.6365701653615375e+01,     2.5090520374363728e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2076935516324021e+01,     1.6797494653051774e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.6763954108668866e+01,     7.1453365488937943e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -1.4262974846426768e+01,     2.5091177639113277e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    4.4933220716916836e+01,     2.2863893088149696e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {    1.8013277778052057e+01,    -6.8014970997676414e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.9255487450550902e+01,     8.9166586266146987e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.2893197049800939e+01,     7.2840897683702733e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3801137576989525e+01,     1.6805928415672017e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.5071342380320036e+00,     6.6698664969240590e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5218813186501348e+00,     1.5740829027376520e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8864469627094902e+01,     1.0444971671829688e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.4582132311776617e+00,    -1.2385909618892874e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.4961972117702160e+00,     7.2850805511024319e+03,    -4.0395751207398698e-04,    -4.0821753129872727e-07,     1.8606664267302878e-09, } , 
      {    9.6537019313154460e+00,     3.3453297478486013e+04,     3.6205441295951229e-04,     7.1036369541622958e-07,    -3.5635260002594220e-09, } , 
      {    1.5107545668343349e+01,     3.2480704290709084e+04,    -3.7967743506462802e-04,    -1.6818908970074650e-07,     5.6582604722293918e-10, } , 
      {    5.4966449372453745e+00,     8.6092585761038390e+03,    -4.6578666670312054e-04,    -4.8102568637492276e-07,     2.2098292398653755e-09, } , 
      {    4.8693362296318838e+00,     1.7369135259182742e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3942502858269517e+01,     1.0685227565185141e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.1041955419863321e+01,     1.6648746350864069e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.2256204505821728e+01,     1.0269719008445009e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.2386702426052338e+01,     5.9323027661949318e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.9677421496401744e+00,     1.6768586116401011e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.0936786323951310e+01,     1.9247644654106434e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.3088698761119950e+00,     8.3563135686138612e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4643879524561946e+01,     1.6686906521044337e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6450565673843954e+01,     3.1716419188949218e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -1.1830215080131509e+01,     1.5645451480066244e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.2912075504984347e+01,     8.8702579607415519e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4310445718728651e+01,     1.5649606565633647e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.1269110179742761e+01,     1.6798917343985526e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.8981835939416310e+01,     1.1825539768806044e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2117979874991278e+01,     2.5248583361078930e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.7978710378763115e+01,     3.2147177517756416e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.9451518779144404e+01,     2.1866000483086013e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    3.6268951022360675e+01,     8.3244336689657284e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8850633795781359e+01,     3.3385759003906787e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    8.3459810117788734e+01,     1.6746615553826621e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.5422026421077597e+01,     7.2287328381839307e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9138484464310288e+01,     2.5941325295938332e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.2107290771157210e+01,     1.6799388210572495e+04,    -6.8944989980326268e-05,     6.2777747864984041e-08,    -4.6173170175191099e-10, } , 
      {    1.1280219576161242e+01,     8.3019135786477254e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.1737646046281078e+00,    -2.9983878528475029e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -7.2351801606775368e+01,     1.6777699610403477e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0561500512333104e+01,     2.6334346265631320e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.5685789904427411e+01,     7.6917473411950723e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2611500927779215e+01,    -8.6364983828341337e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6996557953850242e+01,     1.5960348947736804e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.4351337076177337e+01,     6.3061789115645715e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.2997054339578380e+00,     3.2128598630154793e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.0109603002381755e+00,    -1.6726109165711452e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -9.3011416842280035e+00,     8.4248296591087674e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.9088808946979423e+00,    -9.6497185245735318e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.5646080850382589e+00,    -1.2658039580064042e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.8070968052911400e+00,     8.7489348661724653e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1854771992089992e+00,    -7.0448537826763368e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.5281046964460765e+00,    -5.0220294419954064e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.9352963702334833e+01,     9.0594725425957331e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8564629325680229e+01,     8.8882726932044407e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.9250170714889530e+00,     8.4864257789638814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0138459800262481e+01,     1.6824290731987498e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1377003196229879e+01,     1.7941220491194010e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.3960892177937716e+01,     1.6777890066114349e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.4407987976535821e+01,     1.6756257528029822e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1706182877690946e+01,     2.4428790004400376e+04,     2.7405917659205993e-04,     5.3192385068620739e-07,    -2.6208106454787403e-09, } , 
      {    1.9637010656305065e+01,     8.4315172735082651e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.9564222087214596e+00,     8.4354150414749365e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -5.9425337087256750e+01,     8.4550416602227833e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.0196601922500932e+00,    -1.0698771692425411e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -1.1140399246769146e+00,    -8.7992551795896507e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    7.1315345302457880e+00,    -1.1429620487223924e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.9223710569417030e+00,    -8.0742266152537750e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.5834426555244617e+01,     1.7320575389141810e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.1002137145245619e+01,     6.9743013985080270e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.9582364136544378e+01,     1.5432912485071955e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.9059521066798819e+00,    -7.6485782039209471e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -8.9090663573629314e-01,     4.8624960027995539e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5521821111980696e+01,     1.4340271678656130e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.0703734935777653e+00,     2.5088219174422316e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.3032425161073213e+01,     2.5093478839054682e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.5668057151916768e+01,     1.7829886802726731e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.0600825567999845e+01,     2.3764369781795682e+04,     4.5470021253962852e-04,     8.1961956126182262e-07,    -4.0865429994403895e-09, } , 
      {    1.7249781744305650e+01,     1.5353282681276130e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.9080832360479962e+00,     1.4954787087990546e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.6264775227737314e+00,     2.4304807067352522e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    5.2363519582689158e+00,     1.6446688873434188e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.4990354404611757e+01,     8.5159494507613181e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0484093426759612e+01,    -1.6888035855452254e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.1855851258235077e+00,     2.5876890946124477e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.6090883668713495e+01,     2.3506820154061428e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.5267228701122013e+00,     7.1960386571565450e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1461387590315159e+01,    -8.3690318076777439e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -1.3779377945594369e+01,     2.5113390911870745e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1403837172505355e+01,     1.6707046246682039e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -8.6810837478850004e+01,     1.5647430971276104e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0929100209224791e+02,     1.5647627074423785e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.5996638065360287e+01,     8.1850681922584499e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8610005183682471e+01,    -8.4109242523593566e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.2309769666104877e+01,     8.3509828642218836e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.0887598390024076e+00,    -1.6768716537714183e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    6.6820362745279782e+00,     7.6654389505685149e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.3716251192957571e+00,     8.4519424817807558e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5320114044632827e+01,     2.2685823849927150e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2053572222735992e+01,     2.5143818103949692e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.3006122658053991e+01,     1.6535687435571155e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -4.1235798522092502e+00,     1.6196206804099664e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.1022205946320138e+00,     1.6292757288703082e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    9.8342521246107069e+00,     4.9663397435138549e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3243061661824481e+01,     1.2252342165954191e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4611334308520876e+00,     1.3921513559346889e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7722489922068831e+01,     1.7163995020616698e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.1357025680690134e+01,     8.8161159262372184e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0975547736241436e+01,    -8.2638232244026892e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0026849999521499e+01,     7.7003594592468107e+03,     1.5239034012591032e-04,     2.5067524092335486e-07,    -1.2367112191423151e-09, } , 
      {    1.7812674505017366e+01,     1.3728806089758542e+04,    -2.4880535809158395e-04,    -1.5820231572083495e-07,     6.7766771531809805e-10, } , 
      {    1.5413994441306958e+01,     6.0284166203491513e+03,    -4.0376164734681866e-04,    -4.0830479776132704e-07,     1.8592119856869589e-09, } , 
      {    4.0702712237126548e+01,     2.3768972181678506e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.2158982612565914e+01,     2.4022621113346544e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.9662265128194072e+00,     2.4568479616115339e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.0001962873765763e+01,     2.5037879909527306e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -1.8578214516869309e+01,    -8.4695487848881712e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -7.8568427625506317e+01,     8.3754118564269575e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.2981669268968457e+00,     3.2265096970198934e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -7.7069357890706286e+01,     1.2031676115658047e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -7.4647070709599589e+01,     8.4490081857800305e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.8755768178825150e+01,     8.9232704579721030e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.5450503543583860e+00,     1.5811822332789510e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9267702756165189e+01,     2.3995408117229428e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -3.6644487123036065e+00,     1.6530268047571393e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.9736783834951659e+00,     1.4258916648034681e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.2961234415142062e+00,     1.7231557875527018e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.1044073661453893e+01,     8.4179241292031711e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.8072691595730426e+00,    -1.8469995317119233e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1152189879870512e+01,    -1.1067478695338005e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6539509265158628e+01,    -1.2531847195884159e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    3.2593480130710426e+00,    -1.6742969912359018e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    3.8822643264344152e+00,     7.8264884804239091e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5538766027990002e+01,     3.2880250359931197e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.7238350405679149e+01,     1.5630608342042353e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {    6.8927441215991170e+00,     8.9028664509035698e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.9501403771551562e+00,     1.4854556642511443e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.0601870544490568e+00,    -1.0142639624846703e+04,    -3.3491458973007653e-04,    -4.7103891239501121e-07,     2.3216709079605341e-09, } , 
      {    2.1984117208342880e+01,     2.6727367235324309e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.3593820458703796e+01,     2.5103468245156488e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    2.8745896735466117e+00,     8.1179974488107364e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.7285965069919964e+00,     8.5136482508199097e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.6591898722312663e+00,     3.9516673537513780e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {    1.6264162839155013e+01,     3.3402619750554361e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    8.6397387198432174e+00,     1.0263249464632992e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3507920073997347e+01,     3.3090953811482825e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.4239021442454236e+01,     1.8727262430579987e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.4247211956249327e+01,     3.3595308469643089e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {   -3.7097801779054848e+01,     8.7951363781372620e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1927807622905128e+01,    -1.6746425098115753e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.9451232493452604e+00,    -1.1634615542319864e+00,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.9644335075179200e+01,     3.9521275937396596e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.6952892558270996e+01,     7.4247568411710581e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2764334913327612e+01,     1.1701586841920773e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8725279213007955e+00,     1.1261032876066815e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6438348029340055e-01,     1.5425360318695657e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.4818079009377053e+01,     2.3294943765216660e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.9122967381180217e+01,     8.4019859299778509e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7562821534681802e+01,     2.4919996741199800e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5408141434659662e+01,     2.3348624207894729e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.6663709553600654e+00,     8.3532840641632938e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.0361773002404417e+00,     2.1071292729053349e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2417262201097971e+01,     1.4966252340593212e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.2521252645013227e+01,     7.5169457053149945e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9313046322393607e+01,     2.4526975771506815e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.2429719116829911e+00,     1.6026472240261275e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.0352436214872412e+01,     3.0404241373445759e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    3.3869787129575819e+00,    -3.6462556054381832e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7912195591971525e+01,     5.1041494097912528e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7743967685353184e+01,     2.2259021452779001e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.2772057671869273e+01,     8.9968407199430076e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.1949472501670577e+01,     9.0640749424785536e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7664515748504932e+01,     1.5683577439253544e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {    1.8844830215309301e+01,     7.1027291768910609e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -3.2615929082447126e+00,    -2.7577057434500457e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.5086102578274760e+01,    -1.2214808349169571e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.2030128660991446e+00,     1.5558511509235155e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.0216420683703262e+01,     3.1156501885376580e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {   -6.9210978687487597e+00,    -3.3666426741180999e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7983145475775729e+01,    -9.2155494450161768e+01,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {    3.1054686295922856e+01,    -1.2173257493495557e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0732346840143418e+01,     1.6868095776537433e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.5237470220024258e+01,     1.5668858930827797e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.8411205087069757e+01,     1.6969405847909940e+04,    -1.9201849612945083e-04,    -1.6035648835010902e-08,    -8.8284571330046551e-11, } , 
      {    1.2156616509119290e+01,     1.7077626290795914e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.0083990261715323e+00,     6.7752268890184660e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0761193565552041e+01,     1.3172674022179835e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -4.5460765609019660e+00,     7.9920271572116399e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1804808016815301e+01,     2.5073576009972832e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.8644924416059826e+01,     1.6745948246774333e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3519004763779433e+01,     1.5626199114872094e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4115022266716394e+01,     2.2759420179280220e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.9680564620608294e+00,     1.5681310450305105e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.3074418821733222e+01,     8.4378753846618602e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    4.4055012474376852e+01,     1.5138666143498140e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.5720707087605597e+00,     5.7417502628011982e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.3924774419699617e+01,     1.6547541044337060e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1364377308613200e+01,     2.4499777679981886e+04,     8.6011299274908417e-05,     3.1288022990547612e-07,    -1.6432759721207722e-09, } , 
      {    2.8940705665787775e+01,     3.2344726773994760e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -3.0280864161772985e+01,     7.7372434015776626e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0884805721136456e+01,     3.2357880969308040e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    9.7747939136580726e+00,     1.9121452269926563e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.8670562540617631e+01,     1.3901978305738265e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6461641859570397e+00,     1.6656219387692669e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3994558338664433e+01,    -1.5136496337070719e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7080414311922837e+01,     3.2251942774885654e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.4728610842225835e+01,     1.1184612945342513e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1789346798736329e+01,     3.3483974781175813e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.7924819421222637e+01,     1.5360427296101519e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.5615036184380813e+01,     1.9184717750086831e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.7561859258082908e+00,     7.4585382686262174e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.5641422611980431e+00,     8.5056229244588230e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2042020257248636e+00,    -2.9454187556363117e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.2892310673964090e+01,     1.6974008247792761e+04,    -4.9660331165410322e-04,    -5.1747339714432031e-07,     2.3836834259112553e-09, } , 
      {    6.8443401403964783e+00,     1.5549313245580928e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.5746988187286224e+01,     3.1189930996873660e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    7.4448896706532697e+00,     4.2024344154901473e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.5304240127916771e+01,     3.1190635628789809e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.6570802526003042e+01,     3.1754156548063427e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -8.2807361203793466e+00,    -8.2637315936532104e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.9259690061698688e+01,     7.8510069487867222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3807525015892267e+01,     1.2500562144919527e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -9.8626018673589400e-01,     9.2824237547578869e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.6327593037707111e+01,    -1.4340198299112471e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8224540535272141e+01,     1.8693481003124827e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.0296008237639445e+00,    -1.0036550569789232e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8855402406076440e+01,     7.7402255498519216e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.0630723220618403e+00,     1.6681975488786746e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -3.4054244948226227e+01,     7.2442748664723604e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7059179632540591e+01,     3.2302282039780664e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.1142931180801829e+00,     6.1992932662189469e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4142453094847944e+00,     1.0477473286815075e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -7.5623143952700280e+00,    -2.5023334912490376e+04,    -3.0221194005013407e-04,    -5.6898795356976762e-07,     2.8491045597764094e-09, } , 
      {    2.2480657045269815e+01,     3.1014490894219500e+04,     5.3115575940159864e-05,     3.4385182470263712e-07,    -1.8487400101749936e-09, } , 
      {    1.0671552440715931e+01,     1.5052984114759947e+04,    -3.1063451272071756e-04,    -2.3101047079703038e-07,     1.0268305284531858e-09, } , 
      {    2.5996595780735483e+01,     1.0411190244374528e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4773704396311118e+01,     8.8747661819330860e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.0066648294463079e+01,    -1.2195013437070952e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0603519082159929e+02,    -1.2193052405594176e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.1125607799805778e+01,     3.2307541704413030e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    6.0156821720806128e+00,     1.4292698075489843e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5388950878905590e+01,    -1.2098406820886777e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6110623385462027e+01,     9.3576729866164096e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7111352939244554e+01,     1.6198284346883363e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.6884862821358517e+00,     2.4230651887149295e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -6.2983829806449940e+01,     8.3814453308697102e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4589888506232496e+01,    -1.6777576693254727e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.1066339734226926e+01,     1.6792810504473749e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3766880087291707e+01,     7.6949359015800337e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.8197050263744245e+01,    -8.4336718727176212e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -6.4941198336024200e+01,     8.4332604422655804e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    4.7467361876539655e+00,     1.5818381288388644e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.0438455963408249e+00,     1.6842339675443356e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.8786301167415562e+00,    -8.7913986220583752e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.6535437835807759e+01,     1.6976774119893045e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.5042231145482212e+01,     7.9461787808709760e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.3510016226349073e-01,    -8.2246280372221754e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    3.4939002628618558e+00,    -8.2232053462884232e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.2803232944615409e+01,     1.7796105375271571e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.9377103895971377e+00,     1.5019221437804403e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1524450006184640e+01,    -8.1841017898573864e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    9.9681959959130282e+00,     1.6204843302482499e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.4783390229210180e+00,     2.5685393914828499e+04,     2.6873141360624287e-04,     5.3315663491453275e-07,    -2.7296901019823197e-09, } , 
      {    5.2892168666507953e+00,    -1.0551099821936912e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.8033604442809175e+00,     1.5717418800868729e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.7960366566672068e+00,    -8.4393662115768293e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    5.2440276592983714e-01,    -2.1069397581271278e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2192680578231341e+01,     2.5613218397361659e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    9.8186468954867703e+00,    -8.4290569303213415e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.3518962466742515e+01,     9.2027939509620628e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.9089330173103420e+01,     1.5631986994561516e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.5324305361495227e+01,     1.6705069683451471e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.3476098036407240e+00,     6.4034282864325269e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8392370473078913e+01,     2.4502356323590870e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    4.1938135360180965e+01,     2.3761603909695405e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.3847912052106066e+00,     1.5036142118611995e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.5637586542573114e+00,     1.5156292275887941e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.5174657201738306e+01,     2.5096882481181252e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.5234381543994683e+01,     1.5571087779439082e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.5671681138045315e+01,    -8.3847642916770437e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.2065242252322365e+01,    -2.6777845975725455e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.8895031668063460e+01,     7.8074597723874685e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -9.4794369941210928e+00,     2.3974142904689692e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.8944960355822364e+01,     3.2697932841789836e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    8.1180004320483405e+00,     7.1158470880893274e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3406645792638841e+01,     2.7120388205017298e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.6670854021582215e+01,     2.3978297990257099e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.0285755087406280e+01,     1.0837992641522675e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7881838741894613e+00,     8.4629631085031197e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9656741092247959e+01,     2.2286253400373938e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -1.8862036807852029e+01,     8.6480826952695916e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.7123962485650708e+00,    -6.6208901250070357e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.9857193910629958e+00,     1.6167612674025051e+04,     6.0133478927768417e-05,     2.0830955499275746e-07,    -1.0371182747663303e-09, } , 
      {    1.8408556374711427e+01,     1.7426513583564192e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.8082483005863637e+01,     1.6133497681111434e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9605434185666567e+01,     2.3681974105899579e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -6.6610450320628303e+01,     1.5663071051138375e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.1247173641501938e+01,     3.1939127064614172e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -7.7220851890926063e+00,     2.8198388933224915e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.4365515796620087e+00,     8.4214685467481249e+03,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {    1.2285048375704360e+01,     1.6766566809285308e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4247362288552701e+01,     1.4898689485682891e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -8.5514896461295606e+00,     1.6490453145673477e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0866779329564842e+01,     2.4257907728699098e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.4889509057360826e+00,     2.2404136568701448e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.1665611816590946e+00,    -1.9006799743899910e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -9.4234795655175212e+00,     7.9522122553137287e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.8579507776078600e+01,    -1.6186263153173149e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.4702846628734454e+00,     1.3084208057480457e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.4308252101054080e+01,     6.6402401550362993e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.5140920729080918e+00,     2.4540093682705079e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.2359192161306979e+01,     2.3177854351036303e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.0404116053685634e+01,     1.2218560738499031e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6943320209682451e+01,     7.2226993637411788e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.6414146972064223e+01,     2.3467357568121588e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.7865727325644922e+01,     2.2827810460753131e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    3.5771055630031405e+00,     5.3356990658208488e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9160656315313789e+00,    -8.7282713499155725e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -7.3836804461059167e+00,    -6.7072551088353771e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.1265123295483882e+01,     7.3975009996212575e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0163258616475808e+01,     1.7454899516974452e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3581781832791791e+00,     9.0610623970702673e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3803246391490083e+01,     1.5515531818125768e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0455296317732062e+01,     7.3188375982264952e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    5.7829820780623251e+00,     6.4783507151284666e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.9891487100351686e+00,    -2.4184557095168930e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.7327002625335801e+00,     8.9483625112861591e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8024116998077819e+01,    -8.4331115540417850e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0407887130878603e+01,    -8.4459176769689111e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    4.1640441723735655e+01,     3.1678905486992717e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.5972900891641189e+01,     6.4809738338939233e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4768307028009307e+01,     8.4338207609414167e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    6.3715406163852011e+00,    -1.1849105192112197e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.3173983711539876e+01,    -8.2599993191518697e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.2162160050064116e+01,     7.0141163004059399e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.7845449118330849e+00,     7.9640658640796319e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.0798501179350602e+01,    -1.7282661714850628e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.3876297536304900e+00,     1.5577699178991188e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.1756517348912460e+01,     1.4430728754656771e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5320980073674669e+01,     1.6840154469007477e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.3290749186157615e+00,     1.8495682619645868e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0870718690530568e+01,     1.5332060314169081e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.4072831399739645e+00,     1.6831987425973810e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.0264444903006407e+01,    -1.2407332001111083e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.9217745935008196e+01,    -1.0984180490984022e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.9228629444967900e+01,     8.4046157622940373e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1981635181320677e+01,    -1.1980733841554045e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.3562558165847086e+01,     1.5879418557393605e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.1515266029198926e+01,     2.3093481422751946e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.9289118927935228e+01,    -8.4324997550905391e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.2568300120583620e+01,     4.5364307415860770e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2052415729426089e+01,     8.4275661034063723e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -4.6847464753950403e+00,    -3.1507267131430335e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.8889008702383201e+00,     1.6963662770012448e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -6.4262256180444588e-01,     2.4592639539842821e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.5374818291540073e+00,     1.6744525555840581e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0575271262952530e+01,     2.4335459989711224e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.4190026182360313e+01,     1.2094607811613761e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9172761694278595e+01,     1.4764789998128499e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.0662377380072101e-01,     5.8062722965259600e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9015421696623395e+01,     8.7337459735081684e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8644150870496105e+01,     3.0797262343138747e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    2.0168228706338319e+01,     2.2057497514381990e+04,    -9.6512950329257819e-05,     9.2516558433819761e-08,    -5.5831628330255278e-10, } , 
      {    2.5281611798345320e+00,     1.6971707047851349e+04,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {    1.5118930516122417e+01,     5.5958126858099047e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.4751434419469227e+01,     9.1629790490641499e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5867163291012226e+01,    -8.4649463850053507e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -1.1808044921172467e+01,    -3.7647916930972224e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0747899129244512e+01,     9.3096795963076875e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.3095483107256349e+00,     4.4694750331065652e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0815388132686259e+01,     1.5862145560627936e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.2625885671164468e+01,     1.7770368225655315e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5994401098666078e+01,     1.5613036249927909e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    8.6987591698367073e+00,     8.3143650981250339e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1539710361559862e+01,     8.2188496197136101e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.6534820857952131e+00,    -1.6775275493313318e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    7.6868980035549388e+00,     5.3019176383656886e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2218572284654369e+01,     2.4951129384860957e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.7557909193706234e+01,     2.3997550355451247e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.7063467039428311e+00,     1.7389753821693717e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -4.1503954820235656e+01,     8.8289178055924222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.3849581093060719e+00,     1.6134561342536384e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.4659495357925842e+01,     1.2645363135647178e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3115349502484580e+01,     2.1501365446803287e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.2160773267455612e+01,     5.4362417661983300e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.0183738484867870e+00,     9.0617737425371433e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.2815732865157212e+00,    -1.2240675914262046e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0397399235059776e+01,     1.4165616643508109e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8777624171823838e+01,     3.1207203993639338e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.0380210903808560e+01,     7.0033499380678113e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3258550630303485e+00,     7.8051585724460574e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.2409214754407245e+01,     7.8028573725046481e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2960040033188873e+01,     9.4444235112827628e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.5054356994223683e+00,    -9.6010638139678977e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -1.5840395132474736e+00,    -1.8603301155967865e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6151062121487335e+01,     1.1577633915035500e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.8772019613594817e+00,     1.1495135141144717e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4049951795888571e+01,     7.7038330405801908e+03,     2.5975752710724135e-05,     1.0452704168141873e-07,    -6.0688491787610611e-10, } , 
      {    5.0014996014596296e+00,     8.2914307860034241e+03,     3.0695769099917065e-04,     4.3388754304885126e-07,    -2.0948314347061939e-09, } , 
      {    7.9319659870860253e+00,     1.0485510265945577e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.5883481434215918e+00,    -1.0070464107379867e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.8562553137901020e+01,     1.5635077503372637e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.2251380335892756e+00,     1.5941775364423760e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.8774196981979095e+01,     7.1043571002325971e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.5903711029984033e+01,     2.3954890539495544e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.5563483748680547e+00,     2.5679341689886129e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -1.3564211458873357e+01,     6.4013507436488262e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.9453840425073132e+01,     7.1142191647477921e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2890259682449226e+01,    -7.9760108379847561e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3197396465894624e+01,     1.0207055224160948e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8520530552204431e+00,     6.6724963292402426e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2405694276755874e+01,     7.0961702212919281e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    2.0966556445021691e+00,     6.2768455272310712e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6086180649132711e+00,    -7.8646771495174735e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.6912238716549681e+00,    -5.2282150671051613e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.0351867873033118e+00,     1.6778157090339049e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.1831077974995416e+00,     8.3970548977202561e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.2707482080778538e+00,    -1.1080696036659720e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.0976781640536061e+01,     7.1089291029813921e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.6229933383269799e+00,     1.6746738470975375e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.4014827320056703e+01,     7.1096471619989961e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5661265312809904e+01,     1.9120283400272976e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.5834038537641547e+01,    -1.2349453204216861e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.3484959125338985e+01,     1.4376320095059731e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    5.8186066127612968e-01,    -8.3816510460957306e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    9.6926992039599007e+00,     1.5567346929521636e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.7161969580500539e+01,     8.2248830941563610e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6114955075762975e+01,    -1.0136587399904338e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -9.6574440424471106e+00,     3.1674303087109889e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.1404868118774324e+01,     1.5942334215273917e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.9017438441810608e+00,     1.6829671676363178e+04,     2.6652952483877240e-05,     1.7247823633752259e-07,    -9.1770381697224051e-10, } , 
      {    2.1348417381270966e+01,     2.2618285375347925e+04,    -3.7543541920200984e-05,     1.6602837208677761e-07,    -9.6046438364610159e-10, } , 
      {    1.8992859165692735e+01,     1.4289593950724475e+04,    -1.8983594968252716e-04,    -8.4690502067877125e-08,     2.7551961497454914e-10, } , 
      {    1.7463100393263076e+01,     4.0809395715332532e+04,    -2.2738502730230178e-04,     8.2529784453908197e-08,    -6.7015795139771165e-10, } , 
      {    4.0976003109287706e+00,     1.8307901963712061e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.1335440977771709e+00,     1.4075445144077996e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.4312035865087402e+01,     9.2632789304345388e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7100647313112649e+01,     9.4236431170911565e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.5212604898422185e+01,     7.2044599645744474e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.3700100335742835e+01,     3.3626788697156844e+04,     1.1256631939520154e-04,     4.8540209947429857e-07,    -2.5602525685713480e-09, } , 
      {    3.0092788501329505e+01,     1.4294999275431253e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8292725441316630e+01,     2.3438084361857967e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    4.7786889995425055e+00,    -9.4951256195455153e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    6.2688048259979929e+00,     1.4939039344476092e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.2979451889303638e+01,     8.4311632225528010e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.5820487488996324e+00,     8.4357690924304006e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.9707296448191256e+00,    -8.5587063112120674e+01,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -4.9096950217607862e+00,     1.5358349753317820e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.3159685992869097e+01,     8.2173577361561656e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.4829109663118146e+01,     2.7513409174710279e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.4480562212185532e+01,     7.3343463387176882e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8825544503344712e+01,     1.5143174364689674e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.7603479339527723e+01,     3.3631391097039654e+04,    -1.9201849612945083e-04,    -1.6035648835010902e-08,    -8.8284571330046551e-11, } , 
      {   -8.9576568909419549e+00,     2.4602226832577529e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -6.9865867991299083e+01,    -1.2038612638448267e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.8376343470958325e+01,     1.6253294702763244e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.5322435044608454e+01,     8.3973835300950304e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8500343559701207e+01,    -2.4362035603264303e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.6373230814695638e+00,     2.3800452409192254e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    8.9017810169308813e+00,    -7.9422294105295969e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.7433062528874359e+00,     2.4524898228723116e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.6456133461135085e+00,    -9.7058290710971378e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.2237040205328428e+01,     2.3347912862427853e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.2428494484948978e+00,     1.4703752729123538e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0325365958281918e+01,     1.3800981607225380e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.8097765065526861e+01,     2.2861239572250215e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    6.2943896302158564e+01,     2.2861944204166361e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    4.9237439213704832e+01,     1.6134202313027583e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.0410606689397284e+00,     1.7424216584453174e+04,     5.4805715941951395e-05,     2.0954233922108282e-07,    -1.1459977312699100e-09, } , 
      {    8.7724378125051601e+00,     8.4526443515087212e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9647168004466465e+01,     1.9086501972817816e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9350656671846334e+01,     1.4270381900637349e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.5917606902872755e+01,    -2.1021423168547026e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.6158492370551460e+00,    -1.5348720008141204e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1794171900642381e+01,     2.6241525982407202e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.7087900956938320e+01,     1.8222907772419720e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5823689245958400e+01,     1.6415537278295211e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3221689118926776e+01,     3.1125848963017885e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -1.1235354788247130e+02,    -8.4351971259688398e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.0235402971739047e+01,     8.2419408999570987e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3792093771213926e+01,     9.5503617057051451e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0880093070084371e+01,     9.8751375758959716e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6906105023947092e+01,     5.8796720199124911e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.3320871436453473e+01,     8.9423290368434064e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.9812200106315832e+00,    -2.2533510852897739e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.1494311823728953e+00,    -1.6715756916241902e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    6.7248833480023080e+00,    -1.8477108771787996e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.1455913476438283e+01,     2.3960678419184966e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.8454254096571592e+01,     1.5919233459291516e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0176682277205000e+01,     3.2462646226437279e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -4.8219093022570192e-01,     5.4132513268329731e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5317556679553228e+01,     8.4111747178931673e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3651706666071295e-01,    -1.4673091459037992e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.0386742649416647e+01,     9.1291976216089897e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.8036302831036504e+00,    -1.2069517726559479e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.7419015697652444e+01,     1.0804211214067516e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.3322685802253659e+01,     7.0539312023038519e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.8891885050508670e+01,     1.3734858314700909e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.7998268213609418e+01,     2.2075549948822310e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.7718930958307990e+01,     2.5548304326245343e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -6.1077351361809615e+00,    -3.5437476828360204e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3225075434908637e+01,     2.3660751738792533e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3585704590872325e+01,     2.5641604330771919e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.7037467750743755e+01,     1.9577738719779820e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4016426748848723e+01,     3.0039606337163033e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    8.6217836148852207e+00,     2.3267730769099544e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.8105477583786147e+01,    -1.6700024432242608e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -1.1692462843073137e+01,    -8.4454637682350731e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.0909786469546701e+02,     8.4317351890143618e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -3.6217084959813000e+01,     8.1668287930917177e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.4455283418075311e+01,     2.3976122395899554e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1164656055899181e+02,     2.3976318499047236e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3581895522089900e+01,     2.5110036676494532e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.6545476376124537e+01,     2.2762265093338014e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.8003926019785306e+00,     1.6133849997069508e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.1025357558645808e+01,     3.1224040359955801e+04,    -3.7948157033745969e-04,    -1.6827635616334627e-07,     5.6437160617961053e-10, } , 
      {    1.8068228872914979e+01,     1.4914422324605155e+04,    -6.6085243760249495e-05,     6.2074089288221657e-08,    -4.0874641454344977e-10, } , 
      {    5.3157408987478929e+00,    -8.9570233900000894e+03,    -1.5219447539874207e-04,    -2.5076250738595458e-07,     1.2352567780989866e-09, } , 
      {    3.4379223877717116e+01,     1.4889301775863925e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.6623640219916325e+01,     1.5517609360909468e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -9.8801067926591557e-01,     7.3237097244681763e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4841484944807460e+01,     7.7375900877042541e+03,    -4.8408922402585326e-06,     6.8079330912021156e-08,    -4.3303073183022644e-10, } , 
      {   -6.5206947702326303e+01,     6.4809872609953763e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.5972787773062429e+01,    -8.4623165526891644e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    2.4622435439693691e+00,     7.3308231791369394e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -6.4267668206357357e+01,     2.3991762475761825e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.2746798771705436e+01,    -7.1273370759460558e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.6212981279479528e+01,     7.0957816212188391e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4069990421613241e+01,    -1.0047867543552654e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.1331539085525522e-01,    -8.2768857889664905e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.6981705196390131e+01,     8.1306166872287240e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9474805333086707e+01,     1.4320721165532355e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -5.7325410210451437e+01,     9.2568808144172181e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.5900084775366565e+00,     2.3032444153746990e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.3435052807249448e+01,     2.3831105331550953e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.1050503385834462e+01,     5.8740028703769585e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4363657544866467e+01,     3.8943636249620838e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    3.9908216552066751e+00,    -2.1631795858596633e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.3570251043647197e+01,     1.4325980830164723e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.5901133579811786e+00,     1.3899677105796854e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -8.5441838431034682e+00,     9.3162051822130470e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.4211854839720548e+01,     8.4323378948433310e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.0609257980124571e+01,     8.4345944201398706e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.8402945254501851e+01,     1.4811973172009511e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.0388384941573845e+00,     7.8058699179129344e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4098494222519054e+01,     1.5061881253760106e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.2485284936407279e+01,     1.4845720388471700e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.4291220012375723e+00,     6.2794753595472548e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.3278398892400567e+00,     3.0627992076554597e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.0662587725626924e+00,     5.7724908690707998e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9694160200397439e+00,    -9.2515695055944093e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.2086673171154629e+00,     7.0912391890343333e+03,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    8.1670779665905009e+00,     2.4010001874928555e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    4.3177244876322369e+01,     7.1227946437615483e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1706838149639882e+01,     1.1231013611215660e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6229437904662412e+00,    -8.6507190889034482e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.8775432203065625e+00,    -2.7914871709052068e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.5615210294205379e+01,     1.2487628781306747e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.8076584634756152e+00,     8.4346296190458324e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3830392760259222e+00,     1.6761933924957339e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.3535275802421850e+01,     1.6837408643185761e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.8246528231505387e+01,     1.8002579393886299e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.7246562214938841e+00,     1.6762381239272760e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -9.1443522371748642e+00,     2.4268179236295045e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5456418132625218e+00,    -1.6586389543833910e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.6471004101199064e+01,     2.5137249672611648e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.7601569116112685e+01,     7.1153216069329465e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.4758672907480943e+01,     1.5424861645915380e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    8.6091297627193875e+00,    -1.6935658631447754e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    2.0285974819949590e+01,     1.5616377427710970e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -5.2674082706925489e+01,     8.7891029036945110e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.9307148137566017e+00,    -1.6746158073891056e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3042875158727531e+01,     1.5505493651361772e+04,     8.8482107113010828e-05,     2.4528639141371808e-07,    -1.2668666301073284e-09, } , 
      {   -3.3421911023769923e+01,     8.3993324140783643e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1768885078998137e+01,     1.4283536095950627e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.6853841484389598e+00,     8.3539056918378083e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8369164465032490e+00,     8.3982620391026867e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3013189394474914e+01,    -8.5424986460174732e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    3.4087188500446459e+01,     1.4804604900026412e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -5.6369476018434995e+01,     7.1646450626765345e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3040696959607722e+01,     1.4234333484233748e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.0127855662147329e+00,    -1.9278929705071077e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.7043080162158155e-01,    -8.5130266231453934e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.2825171055410376e+00,     9.4547947122301321e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9369562711093806e+01,     9.0343591067211310e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.1331370760711806e+01,     1.4363165899746453e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -2.0962438199436519e+01,     1.7271020461466858e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.2345863453270653e+01,     2.4606829232460350e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.8225261429867441e+00,     8.0842160213555762e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4099065685661868e+01,     1.1725039534798289e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.4375797238802477e+00,    -1.2995853854615643e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.5691683915957359e+01,     6.1363761221995719e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.7018069654736996e+00,    -8.0661823374148671e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    3.8226478021139222e+01,     8.5529108631853378e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.6432129245381894e+01,     2.3133296324649862e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.3781716653170637e+01,     2.7108064047966182e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.1258711701875770e+00,     6.6207001693221937e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.8779321732347540e+01,     6.4654385191562205e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.1826380874466100e+01,     1.2611581708192020e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4190244773911218e+01,     1.4433573668714562e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5497601237543666e+01,     7.1828844618432668e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7873250477324309e+01,     4.0669651713123887e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -2.3273406716197982e+01,     1.6642712876421316e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.8267318411071471e+00,     7.1114303707120143e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0599010813736641e+01,     1.7078945828821110e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1566790455507483e+01,     1.6467911240541238e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.7322329731871196e+01,     7.5783361099440881e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5534599233050765e+01,     1.7201579217681188e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.2119736139138165e+01,     9.2426088528599739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2711015271459543e+00,     6.7074506939885459e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8109023967013183e+01,     1.5404198129658484e+04,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {    7.3248963133344134e+00,     1.4843459029354744e+04,     1.2186470119331788e-04,     2.8116134330025281e-07,    -1.3855538673797537e-09, } , 
      {    2.7998462441465314e+01,     2.4224126301243938e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.0781067034281904e+00,    -1.1882516969942799e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.8681199828302031e+00,    -7.1224060436884611e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.5660704801711184e+01,     2.3963102536275124e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.1128283232922120e+01,     1.5437979557113644e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -5.4373193719606370e+00,     9.5766700963317016e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7573400242387187e+01,     1.1970654884728487e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.4492138540378789e-01,     2.3308097960529940e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.0854704359525842e+00,     7.6293717836653013e+03,     3.4043821744306186e-04,     4.6971886170408613e-07,    -2.2142458925002837e-09, } , 
      {    1.7435133935751708e+01,     2.2685859489921211e+04,    -9.4044933563517557e-05,     9.1987432782256798e-08,    -5.0242211400743436e-10, } , 
      {    1.1777275721190549e+01,     4.1924024274597643e+04,     1.4338296434618421e-04,     5.2184981024369612e-07,    -2.7341067546172279e-09, } , 
      {    1.8822854176790369e+01,     2.3350542694744036e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.0936146589857106e+01,     1.4809319656110025e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.1783796347850139e+01,    -1.6550306916437337e+04,    -6.7317579642578831e-04,    -1.0082207128969559e-06,     4.9145078040392544e-09, } , 
      {   -1.2913254680832715e+01,     1.4807370772126691e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.6856434757414444e+01,     2.2931481666227075e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.0672535812887801e+01,     1.6881602287808790e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.4161378581519243e+01,     8.7589242722742674e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.5542192467389299e+01,     8.3140214517978638e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5403268994221047e+01,     7.4433493759619214e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.7885503243984346e+00,    -9.0099586311412731e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    3.3643034301322899e+01,    -4.0408814648365569e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.8269080354863645e+00,     7.0809021990799356e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6143327794723930e+01,     1.6356018701223798e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.6475645096197663e+01,     1.5021851270120585e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7924136610021641e+01,     1.5438690902580520e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    5.5513176243839411e+01,     6.3459938134624845e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6251505609256508e+01,     2.7906430144403268e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.6838795191179745e+01,     2.3647633827594265e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    9.4568487103337191e+01,     1.5437972843562919e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -6.2894545496873874e+01,     1.5437986270664369e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.6556304098852749e+01,     7.3273000195561929e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.2433316533739017e+01,    -8.4185457191322021e+03,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    6.2424142026120899e+00,    -1.0098850040790127e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -3.4650796304003251e-01,    -8.2265465240714821e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.6453962033263792e+01,     2.3390572678383556e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.6044673250990940e+00,    -8.2212868594391148e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -2.2055148238500717e+01,     9.4769885569168491e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2567746267990714e+01,     1.6734927620051971e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.6676426433658492e+01,     8.4217259834811684e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2694188512957533e+01,     9.5349768055584391e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3744126167506433e+01,     1.5437268211646768e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -7.8677603284878785e+01,     7.0850152588807123e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4225679500042574e+01,     1.8189126344964559e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.2636963844562125e+01,     8.4557575970900343e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.7226600541109384e+01,     8.2606764536257742e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0315060043792460e+01,     3.7211877957829984e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.6824964364362984e+01,     9.0621260584952197e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.7073318654058887e+01,     1.6288455846313478e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.2517286777361463e+00,     1.5682473911859339e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.3496632079488506e+00,     1.5176955792144832e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.1977480864117123e+00,     6.4683613090266635e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9643007016334870e+01,     1.6399789534780761e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.0686186896408406e+01,    -2.6463720549827622e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.8911075112906204e+00,     5.0202303571399843e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.1696378733389325e+00,     3.9902107444196622e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.7268932343755772e+00,     4.0764540634135774e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4498331925696879e+01,     1.4602790241603623e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    7.8822356015526571e+00,    -5.0434518242136147e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.3394189803178385e+01,    -4.2641582995704084e+01,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.2346617235035939e+01,     2.3766318665779021e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    3.2066511847958267e+01,    -1.7874484941438746e+04,    -6.1134664179665469e-04,    -9.3541255782076041e-07,     4.5653449909041671e-09, } , 
      {    5.8731700442104852e+01,     2.3767023297695167e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    6.6963135435789667e+00,    -9.5507245490728910e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.7083363604557533e+01,     1.9513304369965965e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.5532811217016899e+01,     8.4334728710423260e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.1930215623186839e+01,     8.4334594439408756e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6075238394632063e+01,     1.3038384105340167e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3677929537585154e+00,     5.2773344745647555e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2784078205165207e+01,     1.5727711116178254e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.2270577761045391e+00,     1.4415719962875451e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0067652598535499e+01,     8.1901352643001383e+03,     3.9950555821570275e-04,     5.4318704212574410e-07,    -2.6171212133654968e-09, } , 
      {    8.0331885065472548e+00,     8.2592537626920221e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3852540949122517e+01,     7.1376740659004518e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.5300297591926313e+00,    -3.9367686525290087e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4346370304070046e+01,    -8.1855603037210558e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.0844714665751887e+01,    -9.5454648844405237e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    7.5120845360352293e+00,    -8.9378208156518740e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.1694813118781983e+01,     2.3505647216768284e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5662596199856942e+01,     2.3732913934613032e+04,     3.3322444972828501e-04,     6.0534839787656556e-07,    -3.0244131868656177e-09, } , 
      {    1.7695082550038176e+01,     4.0667360343844368e+04,     1.4357882907335245e-04,     5.2176254378109640e-07,    -2.7355611956605563e-09, } , 
      {    1.0730446470730181e+01,     6.5485002715242590e+03,    -6.1146419156406849e-05,    -6.0488748550993508e-09,     2.3557096765112338e-11, } , 
      {    7.1408106666317641e+00,     8.8859756940934240e+03,     3.3511045445724479e-04,     4.7095164593241149e-07,    -2.3231253490038630e-09, } , 
      {    1.5712674846126948e+01,     6.5857308999817051e+03,    -2.1837765152257568e-04,    -1.8864478486643309e-07,     8.2723758407720106e-10, } , 
      {    7.4315221642983431e+00,    -9.6901300882448813e+03,     6.4202030103651864e-05,     5.2579498157372571e-09,     2.7973749400020273e-11, } , 
      {    1.2053501843223776e+01,     2.3896038354145086e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.6646921469171674e+00,    -8.1709475945441081e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.5806100337613483e+01,     9.6562999001275275e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9876830912775866e+01,     1.5183548117437740e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.9717171744534404e+01,     9.2763902803151341e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.3700450685210669e+01,     7.5567606072129065e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0842572748429788e+01,    -1.8050510612230960e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.9005848050787328e+01,     1.3772595673815122e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    3.7433622391776602e+01,     5.9721176680928438e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.3072276565531929e+01,     1.5665577966305806e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.1411060341458111e+01,     8.4308126415921161e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.4091648789041713e+01,     8.4361196733910856e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.3839260826193716e+00,    -1.6764299820336870e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    9.3119386763894525e+00,    -1.3200469493123185e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.9362335202952067e+01,     7.1335610060996760e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2221123765308707e+01,     9.7027005660006726e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.3446818491860242e+00,    -7.6812056455607872e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.6288728129145639e+01,     2.2468570918515299e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.0234279685915627e+01,     1.5536195334382661e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5431920553770711e+01,     2.3191332493902744e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.4111800543544991e+01,     5.1697389657825875e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4211843670041709e+01,     9.3898616896359945e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.1514972993246609e+01,     1.4688020245124240e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.9854535065873531e+00,     1.6779018328762617e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.0568689315007429e+00,    -1.0988713400117442e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.7941671137030333e+01,     9.4970402925358776e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.5852130608531816e-01,     1.7056403923688864e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.6000086050727965e+01,     7.9246032781397944e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1666651125066863e+00,     4.9426780961278610e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5965119708438529e+01,     2.2421943856188750e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.1173185552337195e+01,     7.4771308034170816e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2290803553938231e+01,     8.3790661675254651e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1709965270839808e+01,     1.6275836607895488e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7425822739624099e+01,     1.7948882974187352e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.9519761630578305e+01,    -1.6554909316320158e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.4336721219903090e+00,     7.1404397276291702e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2911950408828872e+01,     3.1976325252217714e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -2.8002642975537277e+01,    -1.4669972825586001e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3861805827367943e+01,     8.4280706634466987e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4984895764872171e+01,     1.5585172215819788e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.2099878378190120e+01,     2.3969661491874260e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -3.6208066615995776e+01,     7.9063638789730630e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.8214087743919833e+00,     6.6736692665333858e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.3047034983876259e+01,     8.4324844955753833e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -5.6302845470690485e+01,    -8.4344478194078201e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.2707854294917500e+01,     3.8732932798069211e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -4.1544478813667979e+01,     9.0893827197110786e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.9423999826838632e+00,     1.4979406535906490e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.3252685989091075e+00,     1.5347808057683533e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8360121470786329e+01,     1.5988734881147062e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.0831955806785480e+01,    -2.0623274149567906e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.8575322720749652e+00,    -1.8947160870566472e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.4145049963311500e+00,     1.6176509813025212e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -5.1806175350977774e+01,     8.1607953186489667e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2132056465609899e+01,     8.3865668188381751e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0934454882982045e+01,    -2.4333586462686389e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.7874079123864156e+01,     1.5642597990592349e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5065033054658500e+01,     1.7279131478693311e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.6597693641354912e+01,     2.3227380910306340e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.1312356636417199e+01,     1.6273535407954078e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -4.8257148910538461e+01,     8.7553214762393509e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0291218284643277e+01,     1.6664386430726336e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.7820869938731292e+00,    -8.5785812734140436e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.7875207576652571e+01,     8.5508601287777383e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6293594007965385e+01,     3.1101265799216955e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    9.7557079736297396e+00,     7.1054853628808678e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7748866245066349e+01,     2.4945733890816056e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.4408935618278816e+01,    -1.1833548757296549e+03,    -5.2523741015816222e-04,    -6.2257596114658415e-07,     2.9213417982617303e-09, } , 
      {   -1.6390418969923026e+01,    -1.2282367632401292e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4298139561273391e+01,     7.2850561707713359e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.6881948107483566e+01,     1.5656165521232781e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4048894477166975e+01,     1.8163389195348300e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0927101223559937e+01,     1.6238600349606560e+04,    -1.2791439838938311e-04,    -1.0734065787973793e-08,    -5.9583601408361976e-11, } , 
      {    1.5079579908963680e+01,     1.4357168065297761e+04,    -2.4633734132584374e-04,    -1.5873144137239791e-07,     7.3356188461321636e-10, } , 
      {    2.5433387470354507e+01,     3.0528224310493617e+04,    -3.1518439894258587e-04,    -9.5997326738713726e-08,     2.7110296233964115e-10, } , 
      {    1.0282218385279464e+01,    -1.0436108735210164e+03,    -5.5624991983631311e-04,    -6.5893640545338199e-07,     3.0966504253509386e-09, } , 
      {    1.7206898831291603e+01,     7.1130909020995196e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5748518497415773e+01,     2.5702235910976451e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.0315849616445663e+00,     1.6080880899858317e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.3081512329768161e-01,    -7.4378747523693237e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -3.2132156406366278e+01,     1.6204490986524424e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1069742679605113e+01,     1.9479522942510805e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8687992583282462e+01,     4.0058264808885942e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    2.2751591086269983e+01,     2.2494308068131559e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    4.8926265795529716e+01,     1.6205195618440574e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.8739228752063748e-01,    -9.3745936674240038e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    9.5838378417046091e+00,    -8.6043184230302995e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -8.4760237036831398e+00,     2.4190836985251386e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.1020183514587618e+00,     8.3886641033140768e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.3529296150514664e+01,     1.6901842992999620e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.0570438121525594e+01,     7.6519324392971612e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1882416276857601e+01,     8.4533651727145079e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9237044915111273e+01,    -1.7879087341321567e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.2212647693617683e+01,     1.7142060640609772e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0477166931535109e+01,     1.8488173045618107e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.2233053298790445e+01,     1.6608931448966156e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1229338388342389e+00,     8.3275279630692185e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4385796977248844e+00,     7.0035814794349278e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3290055227696975e+01,     6.6928569362894159e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7590002987129530e+01,     2.3899779204062532e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.3049653846446501e+00,    -1.8139294497236392e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.8239993552641227e+00,     5.1441832840252564e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7040484480846736e+01,     1.2880649750999733e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6656296633741420e+01,     1.6381755850840051e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.2298817940115043e+00,    -1.0267145730445218e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3136105871700234e+01,     2.3975509102006519e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.5365695292198115e+01,     1.6156391902201751e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.1034566646855620e+01,     2.3749750300929500e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.5477458259811698e+01,     7.5798613631953049e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.9184548308468216e+01,     9.4572253906379665e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8927421586232882e+01,     8.4821680233061579e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1186549757422320e+01,     3.2227359611084725e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    4.0354158648804393e+01,     1.5652460055107545e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.5182152564548746e+01,     2.5372536287964202e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.3993586905422241e+01,    -6.6864796809983727e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.6240907837092911e+00,     9.0191139265814400e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1637172162490776e+01,     1.7965803654994943e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.8841997161647633e+01,     1.1197232183760500e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6857790545758409e+00,    -7.7015473011321383e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    7.3343336118994156e+00,     1.7390465167160593e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1463925006607653e+01,     1.4023630032682126e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.5742952050186240e+01,     2.2705011519683183e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.2674743085492750e+01,     2.2129673031848830e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.7146954876908740e+01,     7.0137281586124036e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.1480609933598549e+00,    -6.1028337684374424e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.0281595250843672e+01,     2.3976931792940271e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4347599134262257e+01,     8.4071628966054977e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.7671262318019636e+00,     2.5011219202942673e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    4.1151292686352576e+01,     1.5437650924738873e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4713568352707654e+01,     3.0890562347665327e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    7.6323662247703226e+00,     8.5639780399760057e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1242174499315404e+01,    -2.2641420733795771e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.9965301144817911e+00,    -1.1844584031238639e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.4774106739245205e+00,     1.5438308189488416e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.7808674666586469e+01,    -7.4121810509453553e+01,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.3219707236239284e+01,    -1.8450810448626153e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8349957662866256e+01,     2.3941727674551359e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    6.2827698794838858e+00,    -1.1326527674669046e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1013312124392513e+02,     7.1087801738510789e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.8945347188052315e+02,     7.1097960911293103e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.7214999245863538e-01,     5.8838245575380843e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1675145306707035e+00,     1.3506656136103869e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1251844418122218e+01,     2.2063549739324360e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    7.4588665945606900e+00,    -2.4422501554373139e+03,    -1.7995830047484184e-04,    -2.2093643035451917e-07,     1.1401266375916734e-09, } , 
      {    1.2783516981230619e+01,     8.3996490900423778e+03,    -3.8321418684149709e-05,     3.2248012256786233e-08,    -3.1361627403613663e-10, } , 
      {   -2.3802381847893674e+01,    -4.8451400280746384e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.9786421372309499e+00,     7.3783561752979103e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.1521181274178787e+00,     1.4763271306194954e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.2784739108378487e+00,     8.5394043519139832e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.1879489977621866e+01,     2.5170478810534325e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.3982506820394462e+01,    -1.2363239729408460e+03,    -5.2523741015816222e-04,    -6.2257596114658415e-07,     2.9213417982617303e-09, } , 
      {    8.3600016864013611e+00,     1.5191242572996691e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3369875384773939e+01,     2.4211507062825949e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.7153963253327895e+01,     3.3419188115403871e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    5.3967883480660177e+01,     3.3419892747320031e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.3127893165182371e+01,     1.1624034580908647e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1230786475396172e+01,     7.5507271327701546e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8862401918427004e+01,    -7.7796429185329566e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3578374573725577e+01,     1.6904144192941032e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.1741649400178964e+01,    -8.5225084063604518e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.0741734231464704e+01,     1.5472727386969867e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.1826104062269280e+00,     1.7644896606836497e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.5736878096611555e+00,     6.4617928776886201e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8485838176635930e+01,     8.3444239086227499e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2789836824950468e+01,    -5.1091361375940505e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8534828362564632e+01,     9.8166640867841434e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0165678721238589e+01,     1.5418791887357615e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.2796534391495680e+00,     1.5391578891240497e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.0566661200416153e+01,     2.0337969539365167e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0187137797975170e+01,    -8.1775160258821525e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    9.2363749993067188e+00,     9.6371122303714965e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.6397405866135173e+01,     1.4810024288026176e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.7371016890561766e+00,     1.4416651002375113e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.8459856458333498e+01,     1.9970759689472809e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.0929333492767173e+00,    -1.6899765228383678e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.2661902061620452e+01,     1.7167797790226028e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    8.4285770214358351e+01,     1.6762150868564328e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.3177262163370912e+01,     1.6762164295665774e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3962833801497665e+01,     5.4419636843428543e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.2304625192294257e+01,     1.2817600991634126e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0567700413303040e+01,    -1.6907272698037494e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    2.3625941786570635e+01,    -1.2107667937504220e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4038491051504248e+01,     1.6746409838600601e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -8.9523628970664948e+00,    -4.3297896222219961e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -9.0670946315218615e-01,    -9.5879096186546194e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    5.5102533936313005e+00,    -8.5033559355103844e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -4.5938806573139495e+01,     9.1231641471662388e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4595484097114987e+01,     1.0600076193853933e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.4168910270872161e+01,     9.0614214265790688e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8498609364163990e+01,     1.7061878547281463e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.5307119720564129e+00,     8.4121362495137491e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0861410717062869e+01,    -5.8374066995003886e+01,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -3.3063445248994565e+00,     4.6272093874469974e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8995853447202190e+01,     1.2363675854421475e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.7363644190066541e+01,     1.7383906211561462e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.7497093398375836e-01,    -5.2353285217739244e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.3957041705014312e+01,     2.8299451114096257e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.1518361663750225e+00,     7.2195108033562174e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6023533956839422e+01,     7.1219073709081822e+03,     5.9652143881783567e-05,     1.4027109387405393e-07,    -7.2775381671352442e-10, } , 
      {   -4.0427010541226494e+01,     7.8048062564879829e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.5319859383147989e+00,    -1.1392211988049485e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.2659489877465804e+00,     6.9042250195799033e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.3395227564615166e+01,     3.9518622421497108e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    6.7663608708481476e+01,     3.9519327053413268e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -4.6573907033416884e-01,     5.4934334201612801e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.5290270527599912e+00,     3.2268863455693248e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    3.8663279609925936e+00,     8.4547960654694525e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.5637968184515941e+01,     2.2799235081178133e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.5388510775253110e+01,    -6.6244132845877846e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.5670949534535964e+01,    -6.6173669654262869e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3322929447053230e+01,     1.5412242407497388e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -9.4379032365346163e+00,     1.7118760730719026e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8507301841575760e+01,     1.8615928742112705e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.3606883366136124e+01,     2.4464842621634369e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9355651248632526e+01,     1.5771481949735216e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.5518383959850887e+01,     2.3971289415215801e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.0262897958532127e+01,     1.5418801363096525e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1189575042474360e+01,     2.3471865789313124e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.8364991291419246e-01,    -8.7462860823784595e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.1727106327423513e+01,     2.2599073325260797e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -5.2046500489660303e+00,    -1.8875257790767118e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3348430302824521e+01,     1.6790154701082523e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.5204124536334625e+01,     3.1038273744896069e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.1199073400010956e+01,     1.5431420601514510e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.4382634077702839e+01,     3.2304911872096847e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3273649942066550e+01,     8.4345696848858079e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {   -3.1758743830588848e+01,     1.7039130926923201e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0783575772899628e+01,    -1.2052247638260178e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.5154839230791888e+00,    -1.0352189090583004e+04,     9.7682556547543055e-05,     4.1089268470972167e-08,    -9.1440708394069533e-11, } , 
      {    7.5787854355243907e+00,     2.4270466789047212e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.6866445708940585e-01,     5.8864543898542679e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0917792747516554e+01,     2.3963768927996087e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.1816802874557903e+00,    -9.1655382187037992e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.7488512374083093e+01,     1.5528084317156208e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.8704793104759894e+01,     8.4331473538400969e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.1960603591574117e+01,    -8.4337849611431047e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0893970121350538e+00,     7.3307668808221115e+02,    -2.1896245463171829e-04,    -2.5544769831882901e-07,     1.1521160799255120e-09, } , 
      {    3.9990308472310986e+01,     1.6278137807836898e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -3.3244038446193713e+01,     5.8523206464410332e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.2709628558489520e+01,     2.3981151479730990e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1885607871922268e+01,     7.2070592838789089e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2235343734246662e+00,     9.4782049387379229e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.1069448036532890e+01,     8.4330016957042935e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.2215890761304680e+00,     3.2305240504471618e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    4.4407153821114093e+01,     3.2304583239722069e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.6610562664217134e+01,     1.6242049550608845e+04,    -2.5423105344098514e-04,    -1.5692589826120981e-07,     5.6951547933618284e-10, } , 
      {   -2.1404414674169150e+00,    -1.7843004713924995e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.6861938722704409e+01,     4.0599821869265135e+04,     2.0531005133890193e-04,     5.9461433208859170e-07,    -3.0839967882739798e-09, } , 
      {    3.3248567076308959e+01,     1.3004602677885003e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.8671965049860511e+01,     8.4336156434722816e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.5416154563046284e+01,    -8.4333166715109201e+03,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    6.8425826541097869e+00,     8.7854052060559184e+01,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    3.7813596482450180e+01,    -8.4339306192789081e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    8.9775479255252044e+00,     1.7347805351204890e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.3767852913137260e+00,    -9.5473833712898304e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.0017885432613609e+01,    -8.4323626300973920e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {    5.6283273875892723e+00,     7.8605141512488535e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.8809660477747805e+00,    -9.5488060622235826e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.2465859563259109e+01,     8.3782053360779082e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.9225405081859449e+01,     9.1690125235069008e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.8311319602184994e+00,     7.2048481063679837e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3027909013491021e+01,     2.5013296745726377e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    7.0796035559115911e+00,     5.4596519927061199e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0921354555086399e+01,     2.2168849953348887e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5124838438895816e+01,     2.4056402540801704e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.9322829223701596e+00,    -8.5447998459588853e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.4889021346743982e+01,     2.5450088548976328e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -2.0575543425416718e+01,     7.9123973534158158e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7064199183704361e+01,     3.2382464133108970e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -4.3087487617932752e+00,     1.5629480079394081e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -2.6283802834176488e+00,     6.6260956633670958e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.9531651812636397e+01,     6.9500285248985456e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4718999846809435e+00,     6.9051531122766146e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3718669783569894e+01,     6.5346609901227184e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0234751388610217e+01,     3.2827281262720004e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -1.2108557863631953e+01,    -3.0393930246757491e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4173701011359830e+01,     1.0819506592041510e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4945618080422212e+00,    -9.1212923196085594e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.4616770078667249e+00,    -9.4083750948791640e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -4.9469287882329461e+01,     1.6749951657391073e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.7635694987368879e+01,     3.1756234090847127e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.1374382998674442e+01,     1.4716371967541529e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.2541858219212617e+01,     5.9166626863326619e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.5462692279905312e+01,     6.6800550569342122e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.0260003984057171e+00,    -8.3095132306063297e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.7210766668854951e+01,     1.0049320869820514e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8726110673575853e+01,     8.5170787013225781e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.3373027602547856e+01,     1.4831001880045953e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.9371922048225088e+00,    -8.5099074009024662e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    4.5689038474889955e+01,     8.4483865958509996e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.6664185728901998e+01,     6.3858087153603965e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7669342973445325e+01,     9.9859805716619521e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.4168048508841515e+00,     1.2395292688527138e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9934755412128496e+01,     1.6322237273768638e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.8272242713640967e+01,     1.5456003765315443e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.8960809404856194e+00,    -1.6901877203992593e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    4.5092688011102410e+01,     1.6587355946234973e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -4.0319178683620489e+00,    -1.6212407306349046e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0386747483892776e+01,     6.3854205735668602e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.7032454678296087e+00,    -4.8087303622168884e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.1835867663872744e+01,     2.2649412590155807e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.0791842882975011e+01,     1.2964568803651000e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.2735254353587253e+00,     5.0666310230131321e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.3956292583009668e+00,     1.6133855626900990e+04,     9.0950123878751090e-05,     2.4475726576215507e-07,    -1.2109724608122099e-09, } , 
      {    8.6358647994334969e+00,    -7.7587579065729115e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.4549519146749276e+01,     8.9302211636932680e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1206005259961966e+01,     1.7356683739705433e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.1021757325007631e+00,     2.5168401267750622e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.7845033663627028e+00,     1.5692410996789547e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.5920158158546776e+01,     2.2654672254788173e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    3.3712622199564358e+01,     2.2691857324369903e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.7357577692567286e+01,     1.7605081704938588e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3753270416014352e+01,    -7.3814938995538341e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.9090508176847532e+01,     4.0633250980762212e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.4112876689548857e+01,     2.2612227520574077e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    6.1987231678883020e+01,     4.0633955612678372e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.0791409065937884e+01,     8.0092935926596365e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5908669946939842e+00,     1.7783486136853582e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.6111816386244520e+01,     7.5905420346680667e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.7388596227318477e+01,     1.4750153394996687e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.4315115823054649e+01,    -7.5829898682917692e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    3.8130836935925572e+01,     7.2625142656390908e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2580410983921706e+01,     1.4481085352188971e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.0376265860342055e+00,     1.6864329291043119e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.0428544308543465e+01,     7.8908218506846342e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2883594984736579e+01,     1.6651934911249027e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0030908416407101e+01,    -7.4716561798244857e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.3583662150024633e+01,     3.0667913922208576e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    3.2937087274351597e+01,     1.5081041214817225e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6811994954445151e+01,     1.4323350997848540e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.9574664979441394e+01,     1.0476801931742420e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -7.7888368644655117e+01,     1.0478144641887729e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.5232905134030656e+01,     1.6430771841849403e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2751990070758993e+01,     2.4152012866085635e+04,    -5.3196984282695414e-04,    -4.1890796385540122e-07,     1.8018100458435903e-09, } , 
      {    1.6804673036485752e+01,     8.7953355320456121e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.0304267574350238e+01,    -9.7579964984510825e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -1.7698822952987825e+01,     2.4460240221751548e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.3480238756991961e+01,     2.5020959228719716e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.0755515293378394e+01,    -9.7572918665349298e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -1.5969973180745699e+01,     8.4331809295014209e+03,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -1.9225783667559917e+01,    -8.4337513854817807e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -4.6362337009871162e+00,    -8.4388958625294727e+03,    -9.2640263880542614e-05,    -1.1044778028060079e-07,     5.0823018190712641e-10, } , 
      {    2.7490387752095614e+01,     1.3431405075033155e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1201453327239776e+01,     7.1087358429577107e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.5662372415071768e+01,     3.1712652703454896e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -1.0522046120960043e+01,     7.1098404220226766e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.3563663109118629e+00,     1.6100068569614348e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.9617211487768472e+01,     1.5959048361189429e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.5099518468053077e+00,    -6.3486654064467712e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.8632845939120578e+01,    -7.1054041278144996e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.1443500611738671e+00,     3.6834330937205914e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.3910689325768519e+00,     6.4596506394667986e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.6767793136151337e+00,     9.1235522889597742e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9726809664068185e+01,     1.6450638243775567e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.3682529205882737e+01,     1.0095721535693661e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2057589939513944e+01,     2.3373650012044105e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -1.5426152601848148e+01,    -9.7599453824344164e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {    2.8547635603697410e+00,     7.2298200846117052e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.5877367015848634e+01,    -9.7553429825515959e+03,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    3.5427267088891078e+01,     2.3994269390929257e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3654760737309690e+01,     3.1032548958491305e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -2.5960059021219415e+00,     2.5153205813768658e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.8154290908188727e+01,     8.4000370459945170e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3300649729733898e+01,     1.4470543656554684e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.2148610320063945e+01,     3.0601790629684107e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.8505976546063152e+01,     1.9906325339658950e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8966114771142990e+01,     2.4570740975232296e+04,    -1.0193864567865896e-04,     9.3792975893444940e-08,    -6.6646851928446799e-10, } , 
      {    5.4822639133324582e+00,    -9.1550737470637177e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    3.7811561599117720e+00,    -2.6030239400822666e+00,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3848933467437913e+01,     1.4985440010349243e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.8465166860448566e+01,     1.3273670720692722e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.7680210637779535e+01,     3.1676251971093228e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    4.7486802522943464e+01,    -1.3942049280133351e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0999342907254768e+01,     3.9125953767762199e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -3.6264619101532496e+00,     6.4749470730018984e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6294334331699080e+01,    -1.1458069627801838e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.7703217205578266e+01,     1.6936959217995129e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -3.0023116701808981e+01,     8.6082677933716786e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.3836507450405556e+01,     8.7492880017965981e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0551961718140150e+01,     1.6254506699917636e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.1662532461797799e+01,     2.1710914912539585e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.7490160149300729e+02,     8.4194516875811296e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7164579100619306e+02,    -8.4474806274020721e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.6555162843771825e+01,     9.5368551944337905e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9623699815994481e+01,     1.5711963372663804e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {    4.5259409738856981e+01,     1.4844419801924323e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.1362820972104277e+01,     2.2101287098438570e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.3269325063190930e+01,     1.6049366461351596e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.6078800491603168e+01,     1.5560778498183594e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.0374215619931961e+01,    -4.7228105919149839e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.4840821084861929e+01,     9.4234439631828063e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.9542571653349725e+00,    -8.8124093749029307e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -3.4480845732767378e+00,     8.3985801383071230e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6393702053096142e+01,     6.1867342058708346e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.6251325527624502e+01,     1.6627170848132882e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.4691427837742461e+00,    -1.0332982670327357e+04,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -5.9416259821799223e+01,     8.7155065743414380e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1998642957122620e+01,     2.4728480959404213e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    5.8325143692270046e+01,     8.2646979960542740e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5955056806601238e+01,     7.7774106194336491e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.2138554091500584e+00,    -5.9109570708771321e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.1501635374715367e+00,     8.4308631335515183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4643481593028561e+01,     2.5716855391842633e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.2491907261366983e+01,     1.9872543912203793e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.0559618483492992e+00,     7.8478183884017608e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6542479804740829e+01,     2.4433675766980235e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.3504621718928789e+00,     7.4896898637651939e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.1757444091888363e+01,     2.4531233527164535e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    7.0551282078996751e+00,    -8.4684110465009944e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.7605021031099170e+01,     2.3173111226547771e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4541299566156818e+01,     1.3964111455610711e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.5913627269962603e+01,     1.6761798552606248e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.8522037993159600e+01,     1.6762516611623854e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.9411678444921066e+00,    -1.6429330993956535e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.0977871660358929e+01,     8.4311666925475784e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.7375274800762938e+01,     8.4357656224356251e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8413821876242761e+01,    -9.5082798148587935e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.7023519177982617e+01,     3.9847561357334314e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    5.5825345695278079e+00,     4.7549650839415954e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7315442593973099e+01,     1.7111025923405508e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.9169446222850899e+00,     5.1004124504682923e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5567826066341382e+01,     9.1519452419672543e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.5648475574061742e+01,     1.8582147314657544e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.6298731213181412e+00,     1.5986657338363362e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -5.7115479104746747e+00,    -7.6354240086076679e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    7.0072651544666389e+00,    -1.4481309518866788e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.6732528700890494e+00,     1.6990885241868476e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.9546080381485808e+01,     2.4535835927047363e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3036807787885124e+01,    -8.4496754928323171e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -3.8861734393369602e+01,     1.6897144316097219e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.1632873699161674e+00,     9.1147428397483345e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -5.7109739585570075e+01,     9.0833492452683258e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.8909168439892419e+00,     9.8709164379055183e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0010449659022594e+01,    -1.4546899074858120e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.9529519156291784e+00,     2.3958171504017530e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -1.6142355161361227e+01,     7.3576860977233446e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.7673650732579986e+00,    -7.2881846758678003e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -4.7241024435950347e+00,     4.2341884177540105e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5843034805176522e+01,     2.3740933832120842e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.9559396762045269e+00,     9.6125290665705634e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.0599141258798621e+00,    -9.5480947167567065e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.6851782640752788e+01,     7.8055108884041319e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.6527725387218197e+00,     3.2225282068301021e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.3380909665571657e+01,     3.9552731784579257e+04,    -2.2718916257513354e-04,     8.2442517991308480e-08,    -6.7161239244104030e-10, } , 
      {    2.3077833443566480e+01,     2.2199532885870165e+04,    -4.6747680670491205e-04,    -3.4671620089336846e-07,     1.5070869609602919e-09, } , 
      {    2.0868015957427023e+01,     1.4985410000186665e+04,    -2.5413312107740101e-04,    -1.5696953149250964e-07,     5.6878825881451841e-10, } , 
      {    6.6288959458843602e+00,     1.7005464094975414e+04,    -3.7512754884275965e-04,    -3.0320223375906324e-07,     1.3215536133364839e-09, } , 
      {    1.6519718245380346e+01,     3.2899456780186840e+04,     5.0255829720083092e-05,     3.4455548327939957e-07,    -1.9017252973834548e-09, } , 
      {    1.6374165068877545e+01,     4.1714474808861341e+04,     5.7598011062380376e-04,     1.0339779911096794e-06,    -5.1472183709718317e-09, } , 
      {    1.8669807720647984e+01,     2.2895348935332353e+04,    -5.3177397809978588e-04,    -4.1899523031800099e-07,     1.8003556048002615e-09, } , 
      {    4.5780913967286949e+00,     6.5332382991889654e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0097575104684577e+01,    -1.0691697108997249e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    2.3630325481886381e+01,     1.4245798736836416e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -3.2416586252361526e+01,     2.7957636874822947e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.3097908816882828e+01,     2.3801418811593314e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.0746270505313305e+01,    -1.3388479978269932e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0715569375571377e+02,     8.4347357468497030e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0389988326889954e+02,    -8.4321965681334968e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.5343229170154200e-01,    -1.3095747915633674e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.6516056502668217e-01,     4.5496571264348731e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.1451453538871775e+01,     1.6789766559288983e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.3006776297450926e+01,     1.8176507106546571e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.7723609383142744e+01,    -1.1059920608822713e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.3936204516889283e+00,     4.8079341811527866e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.0387226873992532e+01,    -7.0026903011927971e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.5594465313623154e+00,     3.0978432647154033e+04,     2.3622462865346871e-04,     6.3101840962668944e-07,    -3.2585781948415238e-09, } , 
      {    1.0541142522707906e+01,    -8.4388616515365029e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.3379459442838169e+01,     6.2174862944640950e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.0206082179832585e+01,     2.5179866520353287e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.3692260167499999e+00,     6.6360850694688988e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.6593368298655051e+01,     7.2685477400818427e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.0264394599767204e+01,     1.1590253153453488e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.2701652663537573e+01,     8.2586645216115212e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2994725232007239e+01,     3.1283583317358316e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -3.3190475849584075e+01,     1.0513376237695219e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.1160039829798777e+01,     1.0441570335934932e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.9051485756961721e+01,     2.4082158641895778e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.9831860096406327e+01,     4.0698037646534154e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -4.1178800514009374e+01,     8.5684528914737675e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.1698753561889683e+00,    -7.6731613677218802e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -1.3803535139304763e+00,     8.4280364524537290e+03,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.7359680722728704e+01,     8.1801305986831831e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    7.6580787294455703e+00,     1.0159481620994657e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8650606761861450e+01,     1.0442341839513501e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6569002649454774e+01,     1.0547936478430051e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2304130866289902e+00,    -2.5107769687546097e+04,    -6.1134664179665469e-04,    -9.3541255782076041e-07,     4.5653449909041671e-09, } , 
      {    4.4221574533743837e+00,    -8.4154324735508926e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -7.8599267064340495e+01,     1.0407010095200098e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.6514017231652875e-01,    -9.5415357611575746e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.1834699175445365e+01,     8.3841427017480109e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3767417121182806e+01,     2.2980629042351116e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    2.0418343017836833e+01,     1.2756696824114464e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.7449906395583330e+00,     1.3113635166410881e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -4.7375985409853818e+01,     8.1270138911938066e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.5379861479462562e+01,     2.8692472083789246e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.5471990695992002e+01,     1.8556410165041289e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.0292083505233101e+01,     1.6829581461186899e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    5.0124693706663503e+00,     5.8864656495172330e+03,    -2.7763825076099793e-05,     2.9826077031435431e-08,    -9.5130140507313155e-11, } , 
      {    3.2547765663704965e+01,     3.9485193310000031e+04,    -1.6545794030958409e-04,     1.5529430629880380e-07,    -1.0200479850544637e-09, } , 
      {    2.4549511605503394e+01,     3.1642828489427622e+04,     5.5681525069484248e-05,     3.4327906581977430e-07,    -1.7935730614015391e-09, } , 
      {    7.1675639623606955e+00,    -1.3951713304143984e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    4.8744101133097644e+01,     3.0983035047036847e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.9132057184748788e+01,     1.6869396363084808e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.7791804254362603e+01,     6.4256236172583085e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.9856909793602597e+01,     8.4239576036459221e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.7953318043906559e+00,    -1.1091806212672320e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.2987538598218570e+01,    -4.1629407120763426e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.0673945434366456e+02,     1.6760426613637814e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.7270637683762935e+00,     7.1437809054122308e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6755489097750075e+01,     3.1243252410042929e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -2.9432844913367813e+00,    -1.6742098278460960e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3741696641153350e+01,     7.1195974137456815e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.7785006364396509e+01,     5.8530252783571823e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.9399038924374628e+00,     1.5313467779378218e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.8988392350517351e+00,     8.3364063515697617e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4384694184009689e+01,     6.4706712661891625e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4548696444867449e+01,     1.2017055550601635e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7585501583806153e+01,     8.6284123897898626e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2643474420264194e+01,     2.3945068852334422e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2489108969167615e+01,     6.5449702713782053e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.2563369980242278e+00,     7.7349658852195525e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2473966797829807e+01,     7.0460097540235147e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.0615473102363865e+01,    -8.6868017163000204e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -6.7161463358921143e+00,     6.7354119871205539e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.6894146710514288e+00,     1.6153037666825539e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.1900250042058282e+01,     1.5512542288867779e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.4435009516922843e+01,    -2.2325303616082510e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.5721691045411461e-02,     1.4797052733650115e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2906121038091515e+01,     2.4583198124541086e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.3057344938391173e+00,    -8.2047290219992701e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -7.5056006661341712e-01,     5.4908035878450964e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.9320464575183269e-02,     7.4121376027530705e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.3576062573598534e+01,     2.3976146685031745e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.2364227934483765e+01,     9.7912866341096942e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2612675431379412e+01,     3.1137314215620543e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -1.1988881512100342e+01,     1.6407759842435289e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1453573974149435e+01,    -7.0786352101314988e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.5648951827514498e+01,     9.7828826593289850e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9747895965613477e+01,     1.5640970067250808e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.5313346674188297e+01,     4.0240582327027303e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.4383991806831935e+01,     1.6022516308602226e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.5333346392595445e+01,     9.5308217199910378e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6925758777378650e+01,     2.5190175801608777e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.0045376786331129e+01,    -2.3963886584285183e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6204488017824637e+01,     2.3314131434972693e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.5665594976116832e+00,    -1.6402918039877222e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -1.6018659851826765e+01,     1.6753297426574511e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.3183324552112904e+01,     7.0989788512347059e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.3985493968592142e+01,    -1.6771017737655595e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.7392162565638962e+00,     1.6047099472403157e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.6369731599793841e+01,     3.8368297761786489e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    2.2313143132937931e+01,     2.5338754860509041e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    2.3731601069035666e+01,     2.3045063392164979e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.5140403266340780e+01,     2.3856775741779660e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.3219725430246399e+01,     9.0788395379377289e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.3856825293648598e+01,     8.2984794235094341e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.1267731388273916e+01,     1.6762862214031204e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.3876846726081254e+01,     1.6761452950198898e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.4641688619451358e+01,    -8.9716022431070305e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    5.1546155853125960e+01,    -3.0212001114015948e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2650401883474389e+01,     2.0698795813330880e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6638523907989427e+01,     1.5459309465091495e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.6835346594240701e+01,     3.4910678016420206e+01,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2497439460889321e+01,    -2.5084815532295750e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {   -9.0556874211583338e-01,    -8.7746720157887175e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.5420324751896436e+01,     2.2303174081181529e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.3625699970772619e+01,     7.3165363982850849e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8511892354438309e+01,     1.5220726625701798e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    8.2246921924256942e+00,     9.1419558358654540e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5772283058785598e+01,     5.9593225022883662e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.8579134167036457e+00,     1.6063985942217776e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    2.8017040682845263e+01,     3.2291793960898569e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.3824968646844667e+01,     3.9841710274014076e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.2004588946260611e+01,    -2.0225125130588785e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8742983776996791e+01,     1.3119704924968644e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.2933590253022842e+01,     8.5191294357301776e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9918445371707271e+01,     8.6069329958313301e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    8.1891040216089781e+00,    -8.3896953239346385e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    5.4994745390280091e+00,     3.2893404555244473e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.2538016666738987e+01,     9.3914544140715698e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2591855887709926e+01,     2.3864886759006109e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3554771735261580e+01,     3.3026519461668962e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.3350166813727895e+01,     1.4775920554775519e+04,     1.8359592345886734e-04,     3.5401313160774811e-07,    -1.7339894599931772e-09, } , 
      {    1.5382654734705454e+01,     3.1710366964006851e+04,    -6.0496971960652125e-06,     2.7042727751227900e-07,    -1.4451374687881158e-09, } , 
      {    1.1339999636137286e+01,     3.3662846944222314e+04,    -7.0542733318107318e-05,     1.9823551455024625e-07,    -1.1504143839048178e-09, } , 
      {    1.6794832198664359e+01,     3.2129119453484604e+04,     4.2388356758864590e-04,     7.8317185049242507e-07,    -3.9126888133945093e-09, } , 
      {    1.6869292008068790e+01,     5.8188671546128498e+03,     2.8835498930800901e-05,     1.0382338310465636e-07,    -5.5389963066764480e-10, } , 
      {    1.7050202329751578e+01,     2.3385149120716778e+04,    -2.8475669237357757e-04,    -1.2643979588431183e-07,     4.2067283109874416e-10, } , 
      {    6.6101344914685054e+00,     8.4706967859490451e+03,    -2.2123739774265245e-04,    -1.8794112628967069e-07,     7.7425229686873974e-10, } , 
      {   -5.4387584077532960e+00,    -1.6552608116378746e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -1.1796509830664485e+01,    -5.1158315616079717e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3643210171862714e+01,     7.5257193286909715e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0657691468132951e+01,     3.2844201943527602e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.6626489162607331e+01,     3.4968483441825929e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -4.3617148898442863e+01,     9.7514717322117813e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4482181267279319e+01,     7.1725665109568727e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.9882392298675146e+01,     2.0363780659165794e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.8589178558675208e+00,    -1.6480470300889348e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    2.9567484197849382e+01,     2.4542171225488779e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    6.6612799213121523e+00,     1.7390459537329112e+04,     8.5622360892934055e-05,     2.4599004999048043e-07,    -1.3198519173157896e-09, } , 
      {    1.2209690885120200e+01,     5.8533843078659838e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.0881331298291919e+01,     9.2365753784172230e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.3531149136931175e+01,    -3.4324139943687360e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.3419890531531728e+01,     2.5121501929097198e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.9967672661956453e+01,     1.0209685056477132e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.0924427529084983e+00,    -9.3308417853448609e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.5229649487306823e+01,     1.6771035970615594e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -4.6488736368279362e+01,     1.6665254781553558e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.5882682611238526e+00,     5.9578998113546131e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2485585818166360e+01,     1.7185831474166735e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.6279495929072894e+01,     1.5766973728543682e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.2607962497072737e+01,     2.4552712921123071e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -2.4935062297572372e+00,     8.6203347422571842e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7629542609359117e+01,     8.7615541045904502e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.8682844489929870e+01,     6.0119325699907567e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1980587012774080e+01,     1.5822330658730023e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4306021469848599e+01,     4.0616682615912701e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    3.8624691795248260e+01,     1.6653125093589177e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0230734730922350e+01,     7.1064432184324023e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.8489242411716269e+01,     1.6681476816006467e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    3.6138169311418501e+01,     7.0935556484908957e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -4.5999299134833564e+01,     9.3836290612848934e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2730582141170885e+01,     3.1187653480515553e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.5023499801026968e+01,     1.5416649649135792e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.4771377163311588e-01,     7.1121330465479850e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.2927434496551697e+00,     5.8526729623991068e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2262004875190435e+01,     3.0639527988798316e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    6.8504810991930924e+01,     7.1214940572141722e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.6660047194706550e+01,     2.5721457791725454e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.1870604430696301e+00,    -9.5661189249585059e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.8489662659345505e+01,     7.5385212080461752e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.3339657177862350e+00,     9.8215777269017963e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.6826418072135148e+01,     3.1192913145147919e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.6501588643272925e+01,     1.7672152448386296e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.0383246314764454e+01,     1.5435902014329942e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.3307697014054618e+01,     2.3383961278831448e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    6.0761297318362555e+00,     1.5573932693496874e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0893111792073453e+01,    -9.6256469777688308e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    3.5774153642067930e+01,     1.5440057099897345e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.8221683738981218e+00,    -8.6069672068243017e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    2.1103337643816545e+02,     1.5647021064210827e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -1.8855321520190861e+02,     1.5648036981489060e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.3270550408878674e-01,    -5.2352722234590965e+02,    -2.1363469164590129e-04,    -2.5668048254715437e-07,     1.2609955364290918e-09, } , 
      {    1.4183954889304761e+01,    -9.5818761442118666e+03,    -2.7584724895743558e-04,    -3.9757073197335323e-07,     1.9187955870953214e-09, } , 
      {    3.4670716063034718e+01,     1.3397623647577992e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.8514721678065604e-01,    -1.8052554179661296e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    5.9653112192446244e+01,     3.1676956603009377e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    9.9175553737546807e+00,    -9.3903603624162752e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.6000263488865205e+01,     7.1284758022462256e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -4.4249761270772883e+00,     8.4646445610066803e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.8434676666934049e+00,     7.3549459487901195e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3446788689929946e+01,     6.4830580902283509e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.9729634662997551e+01,    -8.3375515801639958e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8942966845884229e+01,     2.7512832562656968e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.7314655149013255e+01,     1.6830849687586633e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    9.5449529882269584e+00,     8.3474269782921256e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3021288021646479e+01,     8.2353914200968757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0552943646210492e+01,     1.7470647260488902e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.5023033006951009e+01,     3.1150468410933823e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    3.9879742076894118e+01,     3.2384541675892673e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.2578583270893342e+01,     2.3270360601415730e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.8374258184527275e+00,     7.0938690213505179e+03,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.9375482528264918e+01,    -7.1247072436298713e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.9889563535126911e+01,     1.3666691690385709e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0088725638423355e+01,     1.0835362809206490e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5107342490913098e+01,     2.4836417567062599e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    8.0521280812947325e+00,    -1.6774609101592359e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -3.3599715564826891e+00,     4.7073914807753054e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.9207669811322248e+00,     1.4310731759430550e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -8.1873805341369721e+00,    -3.0288283995664960e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.3133095064533762e+01,     3.2050480432420947e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.4359150195312772e+01,     1.5474062184510214e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0205466692049086e+01,     1.6760015343893232e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.7677881286533562e+01,     7.3211387981679072e+03,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    1.6065485620324623e+01,     1.5545357313921877e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.5053500864468674e-01,     1.6053667903741200e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.9961220315477320e+01,    -9.5480880032059813e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    8.7501816618374932e+01,    -9.5481014303074317e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.0534839363173990e+01,     9.8478156819231190e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.9745419545306937e+01,    -2.0167753342964352e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.2023880894924162e+00,     7.7927070529687489e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9927146673668599e+01,     1.9008949711805690e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.5958564453124779e+00,     4.7019959867304033e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.9825345914987203e+01,     1.6750667594258324e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.7792265502575027e+01,    -1.6773647569971778e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.5984712569697805e+00,     2.5082212508355671e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -5.1568495650416608e+00,     8.4345954080528609e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.0225742687329863e+01,     8.3893754487809529e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.3090332355232714e+00,     2.2639423184054001e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.2135851728200066e+01,    -1.6779430578880718e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    7.2323853553740305e+00,     7.0666283165344903e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1725584206196803e+01,     2.4100173374358670e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.5133501578143175e+01,     1.0488742505386650e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.7025798653797310e+01,     1.5420706560347979e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    3.4584937726184179e+01,     3.1230098214729649e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -6.1432309331729833e+00,     3.8411674480610227e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5186966233410496e+01,     1.4968579263701675e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.6901468250724818e+00,     7.1092776102863845e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0989446612856902e+01,     7.1092986546940028e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.0374286032277231e+00,     6.5677310721110025e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.4518730431677245e+00,    -1.0645802595214220e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.4589859031790617e+01,     1.6751340870858192e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.3259047435335882e+01,     3.3812561401054940e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.2215372972785378e+01,     1.6894778500580869e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.5667758268098769e+01,     7.2564807911963389e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0233657277986053e+01,     1.4941669176792278e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -8.4123655132982975e+00,    -8.4323369069303390e+03,     9.2743731943700382e-05,     1.0921223261429318e-07,    -5.2374421970263164e-10, } , 
      {    1.9469582179125027e+01,     1.4400903258860664e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.6883340029678431e+00,    -1.5534645356049841e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6005356604640372e+01,     1.6434129813086074e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2305156269657321e+01,    -9.8411667678070080e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5866390107215667e+01,     1.5882815638202501e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.2466438247096883e+00,    -1.0116883724730837e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -5.0378869693089477e+01,     9.4174104887400536e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9266596391736313e+00,    -8.4510981837660693e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.1169316153219762e+01,     1.6116929316261914e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    7.9721386250966138e+00,     2.5326135622091053e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    7.0264204648358524e+00,    -1.7910543188504216e+04,    -4.2823758908334595e-04,    -6.4824597289670804e-07,     3.1555068062376365e-09, } , 
      {   -4.4796283380650985e+00,    -1.6518851069254684e+04,    -5.5170003361444480e-04,    -7.9394954951169874e-07,     3.8523779914644830e-09, } , 
      {    5.9762123802317451e+00,    -9.4805806225085798e+03,    -3.6839511617396773e-04,    -5.0687023105024608e-07,     2.4410853657546243e-09, } , 
      {    9.5708246773571410e+00,    -9.0955851801548852e+03,     9.2354793561726019e-05,     4.2322052699297501e-08,    -2.0032016489764917e-10, } , 
      {    1.4482403510667016e+01,     2.3172126073647098e+04,     2.7425504131922820e-04,     5.3183658422360767e-07,    -2.6222650865220692e-09, } , 
      {    3.0101369487008181e+01,     4.7813909114954571e+04,    -1.3263464910842020e-05,     4.0605681368475838e-07,    -2.2553047631534503e-09, } , 
      {    1.6959546752536916e+01,     1.7594600187374173e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.2451877271499651e+01,     8.4395264403104557e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.4637328613393873e+01,     2.4342009469571443e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    2.9007871114443065e+00,    -1.0727157625835669e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    3.5812361244839295e+01,     5.8549741623405152e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0843027601107198e+02,     6.4810785990184904e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.8646144890116688e+01,    -1.3543900261154231e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5122659602918041e+01,     1.5454840303761212e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.8904763616334019e+01,     1.3824426044726139e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2947085368690622e+01,     2.4485083326825203e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.4942002736247826e+00,     7.1643994679232055e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1904177723090132e+01,     2.5014407763327636e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.9689268601271309e+01,     7.1102545348912572e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    8.2783596170234013e+00,     2.3642159204001669e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.7415889277916150e+01,     1.6666985750030799e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0479100132910389e+01,     2.5755205008187644e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.2606972002855343e+01,     1.5673266172466199e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    5.8039607352017395e+00,    -1.0543686121878980e+04,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -4.0091390458465916e+00,     1.0462631581093679e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.6954242831478084e+00,     1.0492314992536471e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.1884162795532951e+01,    -8.5267661620181752e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -4.1318536092381617e+00,    -6.0371494550570060e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6292601607843892e+01,     8.4172568221508845e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.0321717899361383e+01,    -7.5492084408366100e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -6.5611897052371795e+00,     3.2904121240276027e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.0069540515111726e+00,     2.5345280446414399e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.1300308295479681e+01,     2.2703857533867857e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.1840379045778679e+01,     1.3149717793807453e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5517766898071399e+01,     1.5655249231728167e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.7502535484267021e+01,     3.1765621800666086e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -1.9382941896982665e+01,     7.0920151357245268e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -2.3868178413778793e+01,     2.1201351383790870e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2991097396674411e+01,     7.0570303807458786e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9928472347059561e+01,     2.0299346309351939e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.1315555464146225e+00,    -2.2871325127449345e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.9740048216105297e+00,     8.9569990096689926e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0405286546177443e+01,     1.6762305999172266e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.5243333998848701e+01,     1.3694691096844921e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.0323505114885876e-01,     1.6762009165057836e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.6875205428315141e+00,     7.1615458842345088e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7896206340858075e+01,     7.9755121652044763e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.3685129292409055e+00,     1.5548202227979667e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3914365679766767e+01,     2.0265564881896775e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.7874725291503291e+00,     8.3876871192929684e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    3.8437643316935819e+01,     2.3953678542341149e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -1.3218789897673883e+01,    -5.5088525313009595e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.5675026433993695e+00,     3.2240477522282989e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.8113114479210548e+00,     8.4062436856355744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1338893163367106e+02,     2.3975712488834277e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -1.8619766129664063e+02,     2.3976728406112514e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.1446332299466320e+01,     1.6068588342100596e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0852140370580068e+01,     1.6811820979550190e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7143050723390726e+01,     1.2909001473417022e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.5963995626982257e+00,    -5.5111335433012201e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.8633999264425025e+00,     3.3994878919196301e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    4.1040163009971316e+01,     1.5223363019335653e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -7.4391729338460699e+00,     2.5639526787988219e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.1100487109609396e+01,     2.1448396349592098e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.4313254854704649e+00,     1.5431946082670893e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.5272459775116616e+00,     1.6467352389691077e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {   -5.0025717716251641e+01,     1.6857329414199303e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.1751805518910412e+01,    -7.8669783494588810e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6996482782677855e+01,     9.5449662116602431e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.0288988124899561e+01,     1.6835452087469454e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0999447065789398e+02,    -1.0650570134538975e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.6873171803632175e+01,     1.5019214724253678e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.2758794680098742e+00,    -1.1642919567002441e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1234363748096385e+01,     2.2339222497585128e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    5.5207875458376726e+01,     1.5178481045396051e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.2001633285301772e+00,    -9.0437400585964351e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.6586987205223306e+01,     1.6966292602328631e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1635548043403723e+01,     3.2317531110514839e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    7.2131039412150608e+00,     7.7287173290351939e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.0069327172075546e+00,     1.5460521462245892e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.1221060016798443e+01,     8.9640025911484281e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8705195284807395e+01,     1.5143733215539829e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.6802207400000327e+01,     2.9085493053482234e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.6629026924633582e+01,     3.1697934195029156e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    4.4813877045737582e+00,     7.7521894752348662e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.7824295204300583e+01,     3.2304905158546120e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -5.9638733417777630e+01,     3.2304918585647574e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -9.1753704646635796e+00,     7.1250206164894935e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.7331883875128177e+01,     3.0564276927727606e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.2292388790666330e+01,     2.4650928698392087e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3049855260310645e+01,     1.3974482656558081e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1190137256275172e+01,    -6.8951637061748115e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.3008082550807611e+01,     8.8909214769984465e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    5.8882902600195353e+00,     4.8858796316800010e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.2094137344234923e+00,     7.1704650879866705e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1881202715146166e+01,     8.3680455408031012e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0537530323210914e+01,     3.2338693299552004e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    1.9909497791690338e+01,     3.2480679910377985e+04,    -3.7957950270104383e-04,    -1.6823272293204638e-07,     5.6509882670127486e-10, } , 
      {    1.6909242021839578e+01,     1.6799363830241396e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {    1.0380239511073354e+01,     1.6100062939782867e+04,     1.1653693820750085e-04,     2.8239412752857817e-07,    -1.4944333238833335e-09, } , 
      {    5.6691304102997488e+00,    -5.5731990946403562e+02,    -1.8804787731715151e-04,    -2.1904362078073129e-07,     9.7753467335796819e-10, } , 
      {   -3.1537449205984469e-01,    -5.6298314827927243e-03,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.6508133604774542e+01,     2.4428765624069278e+04,     2.7415710895564407e-04,     5.3188021745490748e-07,    -2.6215378660004046e-09, } , 
      {    3.0564965742247246e+01,     7.2962956930942510e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.3305964333265706e+01,     1.5634411111651678e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {   -5.2335714877773185e+01,     8.5286379895758546e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2415034247627149e+01,     6.7434991103921429e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.1191203524622505e+01,     3.2305623217563723e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -1.4703339341144417e+01,     8.3378649530236162e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.9173123518067811e+01,     2.2721906477323719e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3969388321820054e+01,     1.4756702874856910e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.7061807697181823e+01,     1.6729087500126767e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.6940760995358453e+01,    -1.6761191179713991e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.3746452345857204e+01,     2.4926555696798940e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.9932618799641029e-01,     7.1768509874005140e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.4975807553013833e+00,     9.6900813275826877e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.6071528239358761e+00,    -2.4449435474569478e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.2251738364603000e-01,     1.2720614196717892e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.3566853368834848e-01,     8.4323026959373692e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.7959130323993339e+01,    -8.5290673619595873e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0273714246428050e+01,     8.9853324991262798e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2664136690307362e+01,     2.5056356233816463e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    3.1687093888033914e+01,     1.1983274123146475e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.7242318283265838e+00,     1.4370250336501966e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0536124234278555e+01,     1.7443434264371786e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.4258579536396070e+01,     2.3841233713491230e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.8598811493297962e+00,     8.1326441124200210e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.1796126910519282e+00,     8.9040393881967120e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.2979953091007843e+00,     1.6047108948142068e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.2616067513905200e+00,     9.0999486627124634e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1374545800164348e+01,     2.6188556885196009e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.6987743093663674e+01,     3.2304200526629971e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -4.2752845667902491e+01,    -5.2388516813546732e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.0225607957583193e+01,     2.2808622790997095e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.1512588619563648e+00,    -5.7650194938858385e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.5167144116720582e+01,     1.7373192170394927e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.0784629466812516e+00,    -9.3996404837817022e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3122005433030193e+01,     1.3571090485917724e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    7.9725257039096817e+00,    -7.5000299872128733e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.4305262379787802e+00,     2.5122027410253580e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.6005872178175107e+00,     1.7806896363361371e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.4930878542118201e+01,     2.2809776776812421e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    5.5674071004273769e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0476275179380092e+01,     8.4958229645217580e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -5.3561914309160272e+00,     1.5534117791598961e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5971706278667018e+01,     1.0993097163546921e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1522931305536975e+01,     1.1228383778899477e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.7028901201562228e+01,     1.5631796538850645e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9088256738834758e+01,     1.6539490205180482e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -5.2714332530381071e+01,     9.0495678178131657e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.6705365134235404e+00,     2.3970186973030643e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.1577679975848330e+01,     1.5236474369216250e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2130591997306819e+01,     9.0066624071041315e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9788377211447759e+01,     2.2273099205060658e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.1804996617671469e+01,     1.6999704166818825e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.3020803263664440e+01,    -1.0999585864395194e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.0994242680629597e+01,     9.8311015360076053e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.5525113924574305e-01,     6.6544815967773538e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.1086253410972642e+00,    -3.3787057286643176e+01,     2.8250695821658283e-05,     3.7020469652260392e-08,    -2.2902113481933369e-10, } , 
      {    1.2364440856450372e+01,     1.6833145257696560e+04,    -9.9761634931308941e-05,     2.6330037095586454e-08,    -2.8787751570603142e-10, } , 
      {   -6.6168780164022820e+00,     1.6354500009290250e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9983528464786005e+01,     1.6852329081545162e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    5.1446788034635214e+00,    -1.0278875103376643e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.4736033605385686e+00,     8.4340877851661153e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.2177928737243446e+00,    -8.4328445298170864e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.9207650779145563e+01,    -8.3035464955511234e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    3.9428818226092751e+01,     7.0340370714194796e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.5234849846824272e+01,    -1.0446016562531777e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.5969440290651121e+01,     1.2410076520294624e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1448340087546542e+00,     9.2970603578897008e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0182152295412298e+01,     9.3222386566557998e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5944435471896412e+00,     1.3840158528725440e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -7.9411059349977648e+00,     1.6094035095171595e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.5014515750902774e+01,     8.3382943254073471e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.2964631091067655e+00,    -6.1242561506556501e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.0188226625415577e+00,     2.3345283030111666e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.2284616354500617e+00,     2.3311864446024250e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.7292577994865216e+01,    -1.0096772498006425e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.5756441557022740e+01,    -7.0867462273579495e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.4146794972362251e+01,     8.2414887838697432e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5274328891275861e+01,     1.6511123223248043e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3221986372451592e+01,    -7.9035597720040499e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7034475242211911e+01,     1.5753467217272328e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1043854701940308e+01,     1.5541590828427563e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3312887834845027e+01,     8.1578391590847514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4036085669840350e+01,     6.1175184484550637e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.3140123685847044e+01,     1.5782515756832114e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    7.6262264786460072e+00,     2.2144266789547957e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.3846394825502895e+00,     1.3215750010613249e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.3426937023322685e+00,     4.9088966686727008e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.3030839392382557e+01,     4.1745930656043995e+04,     6.9745587343514727e-04,     1.2482491544949366e-06,    -6.2093481835466031e-09, } , 
      {   -6.9140294219526641e+00,    -9.5607139551746950e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.9148456473163279e+01,     1.7390817483118666e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -5.4741156059269343e+01,     1.9994656495240497e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.3710767112291578e+01,     8.7512360661138773e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.1637699869202955e+01,     3.2855667196130271e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.3460498946259484e+01,     3.0405395359261092e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {   -3.1748751879473989e+01,     7.6121175373992482e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5483464001289576e+01,     2.4586494348578228e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -1.6126488222484362e+01,     2.3958947450707725e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    4.6431645227129302e+01,     2.2026036037367860e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.9376721832033343e+00,     1.4911843680996171e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.2782322206236039e+01,     6.4789030046609496e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.9277898127369326e+01,     8.4303529119102859e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    7.6499596957658262e+00,     1.5643119795679686e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.4506004224340749e+01,     2.2914560985419481e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -1.5561454500872429e-01,     8.4333177404343878e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2836238085189265e+01,     8.4336145745488138e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.3121976965175639e+00,     1.3187300870035335e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.3411641141403958e+01,     1.2477550145505420e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2710854048563419e+01,     9.6029007838965299e+00,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.2703231403336599e+01,     2.5182841573582118e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.4191256554684308e+00,     7.7500472370130456e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.1729230108443467e+00,     5.0977826181521086e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6478432177728077e+01,     1.6787896717263160e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.3123253252132745e+00,     8.6430156232279005e+03,    -4.9660331165410322e-04,    -5.1747339714432031e-07,     2.3836834259112553e-09, } , 
      {    2.6894273090596137e+01,     1.8949431134734277e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0067490583823169e+01,     3.1190283312831736e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    6.6748818714707108e+00,    -9.1823056946586548e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.6082443032991712e+01,     2.4638279449811522e+04,    -1.6366986794420842e-04,     2.0941187585949632e-08,    -3.1803292667104452e-10, } , 
      {    1.8745285922253302e+01,     3.2237397777848717e+04,     8.3736356163974276e-05,     3.8038680193463444e-07,    -2.0211397551775446e-09, } , 
      {    7.5026029650261048e+00,     7.8051041819523816e+03,    -6.6474182142223858e-05,    -4.8160906267740173e-09,    -8.5322359738467244e-11, } , 
      {    2.3223569984215050e+01,     2.3314101424810113e+04,    -1.0184071331507483e-04,     9.3749342662145068e-08,    -6.6719573980613231e-10, } , 
      {    2.7788941497142535e+01,     3.8856915735117065e+04,    -1.6289199118025969e-04,     1.5472154741594098e-07,    -9.6488103628100978e-10, } , 
      {    1.0034640254918124e+01,     1.6904192953603226e+04,    -2.8267761398981169e-04,    -1.9385910145087046e-07,     7.9999105519884501e-10, } , 
      {    2.7070697882596839e+01,     1.8975168284350530e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.2457721857316223e+01,    -8.4151038411761183e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.9008814138034978e+01,     7.1083217300891301e+03,     3.9950555821570275e-04,     5.4318704212574410e-07,    -2.6171212133654968e-09, } , 
      {   -4.7990299913247485e+00,     4.3143705110823175e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1206033975744475e+01,     1.3225643119391027e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.9487806500028320e+01,     1.5691299856406909e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    7.3058573806357515e+01,     1.5552357190765691e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.9745103371582978e+01,     2.3078844819620139e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.4952989036180728e+01,    -3.8254349640617238e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.3684936106597934e+01,    -8.2235880593805268e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.9636428879906376e-01,     4.7845364962137231e+04,     1.0821229790050149e-04,     6.2032797707001550e-07,    -3.3174345757282217e-09, } , 
      {   -1.3758898525819603e+01,     1.7413007072292839e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.4015666189292464e+00,     1.6493089539307337e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0025431563685783e+02,     1.4323344284297815e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -5.7208719886737150e+01,     1.4323357711399265e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.3756782627453042e+01,    -8.4385332295332901e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.8527555834773040e+01,     2.4684710125847247e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.2530806557952058e+01,     1.4434684686315824e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.1141862703592391e+00,     9.0617681127056603e+03,    -6.6670046869392126e-05,    -4.7288241641743030e-09,    -8.3867918695138641e-11, } , 
      {    2.2831826524155918e+00,     7.8051642022775404e+03,    -6.1342283883575091e-05,    -5.9616083924996316e-09,     2.5011537808440947e-11, } , 
      {    3.5781040210917709e+01,     1.5867083154203201e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4455402877161792e+01,     3.2298352916497712e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    9.1449711945721024e+00,     1.7355520278151202e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.9228763287726508e+01,     2.3347906148877126e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.5503105096951320e+00,    -2.6801534824379223e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2941603654873415e+01,     8.3445733946034288e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.9700947505409019e+01,     1.5662948133989625e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0338482968013476e+01,     7.8331556914135272e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2894675884012852e+00,    -4.7976201862042853e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8048813857383870e+01,     3.8898693138755807e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.6943537604320920e+01,    -8.2242453241300682e+03,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    9.0564533049269301e+01,     1.5422437528825214e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.4306564354479468e+01,     2.2740935185360162e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.4640980598804269e+01,    -5.9018735009939446e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.9770049449346860e+00,    -1.9252631381909237e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.7266834420413318e+01,     7.6303569365659787e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.8393334741897675e+01,    -1.2133698176905036e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    2.2154261373259960e+01,     5.5982415227232041e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1993837705926419e+01,     1.6573271632635642e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9498207626958532e+01,     2.1237692898040470e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    5.1999886780877830e+01,     4.7849967362020048e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    3.8272691220359476e+01,     4.1750533055926804e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {   -2.1204102596277473e+01,     2.5085917974480904e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    8.0962898525430660e+00,     8.2304681571473793e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8044791344376453e+01,     2.4159691951430083e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -6.3361432492752900e-01,     4.1566361567418862e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.1026670026416635e+01,     1.6748651070843698e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.4076046225963786e+01,     1.3697344612744408e+04,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.2018052034933742e+01,    -3.5580909173515761e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.7313394450583885e+01,     8.1156962488693243e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.2291056421926072e+01,     1.5267127291574947e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    3.8418652612035125e+01,     1.6775664093386407e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.8856055167600623e+00,     1.7976112936250433e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    9.9622143767385118e+00,     6.4802692019777733e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.6637596870177487e+00,    -1.8452124629214504e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.5735750912483846e+01,    -7.1899688985987814e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    9.7902952927267481e+00,    -2.8924496584251201e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.1824628155539720e+01,     8.4017622728201422e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0421257163149082e+00,    -1.6988627728658947e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {   -6.5175982497493607e+01,     1.5453521585402073e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.9716165204165819e+00,     3.0559674527844785e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    1.2517436860440867e+01,     1.7776927181254447e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7558486683307002e+01,     7.1167662990592507e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6810260174007695e+01,     1.6641625629993538e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.6391893893644456e+00,     1.5054166326813795e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7075403893035975e+01,     1.6906774025257215e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.4249834654203140e+00,     1.2279894106995066e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    5.3364495560922514e+00,    -3.0267737862577619e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.1789848270451344e+01,     6.4813091798194237e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.1313490234234653e+01,     1.4059712660078698e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4846947083394767e+01,     7.9314126992727415e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.8676667368249142e+01,     2.4095665153167130e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.6403430537984018e+01,     5.4193210853907376e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    2.0368161673867515e+01,     2.2858962055892101e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.5788841616242861e+01,     2.4917347957405796e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    3.0688423119782646e+00,     8.1709570702830179e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.1556208800793314e+01,     9.3775955868421406e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3850893458120709e+01,     8.7735608754060813e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.6176395556814089e+00,    -8.5705369955751394e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.1156455501420488e+01,     2.3958196239271601e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.7279636102197031e+00,     3.0890975308172983e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.5021061012577347e+01,     7.0634397561495298e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5986183693650808e+01,     2.2574490161459868e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    9.3591484587458531e+00,     7.6162726229666478e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2451974210191224e+01,    -8.1442868879594753e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -8.7667697631355864e+01,     6.4808824958708119e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.6630357548141639e+00,     1.6614964923408905e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.4477750704906775e+01,     2.2864221720524474e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    5.1598187869356465e+01,     7.9306367525825463e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.0085235959007321e+01,     1.3774673216598821e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.0307557094100259e+01,     2.3268283058632027e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5692976488103284e+01,     9.3047038894052184e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5019107649439187e+01,     2.5055904117729111e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.4439240192639136e+01,     2.4604528032518938e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.8580028209658423e+01,     1.6706800651928708e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.1547757489054209e+01,    -1.6622472171230482e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3303210475422450e+01,     8.1048795376124690e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    7.2200594761146331e+00,    -8.8602685546397497e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    5.7564380220319983e+01,     2.3507172470019501e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.9994952141132899e+01,     8.4336566132024673e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    5.6739141654318672e+01,    -8.4332757017807326e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.3617376233772703e+01,    -1.8477175907295264e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.4361675484675231e+00,     7.1018099659211366e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.3619405541795442e+00,     8.0934048092708945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7394475742311881e+00,    -1.4209084800306518e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.6259389886005540e+00,    -9.4705424557445822e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.4868730836069780e+01,     7.5965755091108185e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.5632459947059658e+00,     3.4481464783680340e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.0719354363061289e+00,     8.2400660929359910e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7746043033542314e+01,     8.2077073564145830e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7084735297616831e+01,     1.0175903629021972e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.0864213783588763e+00,     2.2011115599008459e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.2891152084465908e+01,    -7.0471549826177470e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.6021862836737725e+01,    -8.4365794030729121e+03,     3.7522548120634379e-04,     3.0315860052776333e-07,    -1.3222808338581481e-09, } , 
      {    5.1951774890360930e+01,    -1.6884512695871508e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -7.3462793548221947e+00,    -9.4029796008342619e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -7.0575320306753014e+01,     8.6756916724435250e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.0409454496056298e-01,     1.4243721194052716e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.4347348558708049e+01,     1.0165162779479333e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1304729112688399e+01,     2.0756801628858782e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.4119301742714178e+01,     8.6763963043596741e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.3780191612009016e+00,     1.1365312136869400e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7091011925697526e+01,     1.5592417687416933e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.3659818507071162e+01,     1.7535081610302757e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.9293699963230033e+01,     9.7176903047566211e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2596494987010010e+00,     7.5698719570935027e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.1189301291663561e+01,     1.6817514512301394e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8894958134232503e+01,    -8.2178832173125484e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    8.0349936423860733e+00,     1.3711964093610588e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.0317302842084118e+01,     8.4000779762988823e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7961535542391900e+01,     1.8065173418079285e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.1998726259296557e+01,     2.9986637239951844e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    5.3699860619210211e+00,     5.7997038651879157e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.2328239912182713e+01,     1.6942992692437881e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.0908735758524006e+00,    -1.0150665152185995e+04,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.0765490440169874e+01,     1.6136479829385693e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.0434916986534182e+00,    -8.1982336684113557e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.7793587149552827e+00,    -8.5977594674311676e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.3688663874112912e+01,     1.6932511174873471e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.1399544905282283e+01,     1.0602706026170115e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3201526732185004e+01,     1.6833150887528041e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.9457200850951129e+00,    -3.3781427455160383e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.4001862805087619e+01,     1.6808005958455717e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.4907566003548883e+01,     1.7303714642494240e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6166329606493125e+01,     2.4499753299650787e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5823650189839520e-01,    -1.0142615244515608e+04,    -3.3501252209366066e-04,    -4.7099527916371135e-07,     2.3223981284821988e-09, } , 
      {    3.6025509777186329e+01,     8.4469726687629554e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4089087640044873e+01,     1.7560818759919013e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.3262805283043583e+01,     1.3542738763500440e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.9637371243849024e+01,     2.4151022083353477e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1432601448572054e+01,     1.4809671972068101e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.8630292677771951e+00,     2.4855562391385945e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.3344672584671166e+01,     8.4199596462202462e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.7835899158616080e+00,     7.0812910135227248e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1260951396103625e+01,     3.0578896408593791e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.4086639776220878e+01,     1.7197776448071858e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.5940727850266001e+01,     8.4341707894077499e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6846687308979114e+00,    -2.5562005555526507e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -7.9543353041510471e+01,     8.4327615255754517e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.3215974247640482e+01,     3.2310945346539596e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -7.4628085270694733e-01,    -1.8941115430519471e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.5300064120326793e+00,     9.6474215116269843e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.3493137256224912e+01,     8.4888230876779435e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0246806378680379e+01,     8.7125236064011206e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9171703348688283e+01,     2.3994244655675189e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.7006350617253986e+01,     1.7895913876976159e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.7497678077466418e+01,     6.4965225757330791e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.1107104032724600e+00,     1.6709197960642770e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.5418167081327482e+01,     1.7267606291930617e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.6092848494135261e+01,     1.3790644617270980e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5326789031247294e+01,     6.5876300873339096e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2334662082029290e+01,     3.3406422520163680e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -2.4468379821015709e+01,     8.9017381882553018e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -3.4478426755932432e+01,     8.9025141349454952e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.2080227554910635e+01,     1.3695395728761072e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    3.5491616043804921e+00,    -8.3014689527674236e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    9.3268100963782086e+00,     1.8541142142829300e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1766053693552061e+01,     2.5112178914716351e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.0056786931331439e+01,    -7.3657369368799236e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.2005263426222827e+01,     1.6044957234181335e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.7023577782589690e+01,     2.5095780038996094e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -3.6788241467605140e+01,     1.0085532975683509e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2954269169258836e+01,     1.1621404748592464e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.2773733045944988e+00,     7.0475095860675629e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3910137010104890e+01,     1.7578852443859723e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -7.9702630763698146e+00,     1.6909211264982725e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0614796892994569e+01,    -7.8220792532536516e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -6.9753328030473909e+01,     8.6535333701420313e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5231876060048572e+01,     2.5078397487261191e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.5516074920835351e+01,     1.4929116677761838e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    6.4676806930106565e+00,     7.3887273762452796e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4494469981473792e+01,     1.4810383317534977e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0747092297315216e+01,     6.5327956588405195e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -5.3529696015718820e+00,     7.1714212823626422e+03,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.2896370176423453e+00,     1.5663738358190662e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.6579077264413769e+01,     1.0881763475079639e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.1667696506752037e+01,     8.5133196184451353e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0334729132426043e+01,     2.5069519098760647e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    4.4410385824764660e+01,    -5.2318053621931779e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6312993459073933e+01,     2.5843109518669316e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.4181642171355229e+01,     8.4331071279828011e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.1501156220970941e+01,     8.4338251870004005e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.8049380622627531e+01,     1.0045718073785596e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3209760867576943e+01,    -1.0722106334271111e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6893519140557437e+01,     2.2292802880234161e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.6171089569815820e+01,     8.3781092273052600e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.8718763840166694e+00,     2.2021433637485039e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    1.8418116568283665e+01,     9.9379871813532300e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.2890862949192147e+01,     2.2023382521468375e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.8717708591522602e+01,    -5.1749937773464057e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.0929992093486458e+00,     7.0776489432568542e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5235067556203129e+01,     3.3665138313501840e+04,    -4.1049408001561064e-04,    -2.0463680047014410e-07,     7.3968023326881881e-10, } , 
      {   -1.6063182274616697e+01,    -6.2948944706869324e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1842208823047493e+01,     2.6099059650278767e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.7371296043086861e+00,    -5.9712918153046508e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.1827224053605097e+00,     8.4158341312171324e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.2686309637520830e+01,     2.2821776986310375e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -1.2663529697096699e+01,     1.4939591634008577e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.2187521819710714e+01,     4.1784280272388991e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -9.3823390301142311e+00,     1.5652596094891634e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.8987115680635910e+01,     8.3536126965380663e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.7302717497988308e+01,     7.3178711958254335e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.2919922402159585e+01,     1.6999316025025284e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.5948966598446592e+00,     8.3260884006833676e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4368761909044252e+01,     7.0673396620013664e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.3711628461446701e+01,     1.1828124413950476e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.2776087698586984e+00,     2.5446480936967544e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -7.2274243753189182e+00,     1.3692742212861587e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.0624628939885042e+01,     4.0012030023428642e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.9202059222567712e+01,     3.6081820326498764e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.5742344206643374e+01,     9.1268221596796735e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4219682842626773e+01,     1.7330946590089177e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.4349878338022505e+01,     7.7289721981160619e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.0050515918952545e+00,     1.0083102297275675e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.8444770688419858e+00,     8.9145810838309990e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5643205197258492e+01,     8.3931257744373070e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.4306676937630201e+01,     1.5019228151355128e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.4195964532925530e+01,    -8.4169577268021094e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.1509624398159621e+00,     7.9628929267864887e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.4918825533730452e+01,    -1.6770794080497886e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.1324683133064013e+01,     1.0510336524292347e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2140347487295747e+01,     3.0525927311382598e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.3266019891724987e+01,     2.6492080619971752e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.3682249224823938e+01,     9.4632588650807174e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.9401880017282709e+00,    -1.0751740789636602e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    7.1694330755332025e+00,     7.1090644753324841e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.3510216287868790e+01,     7.1095117896479023e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.7768641677891665e+01,    -7.9007597769140448e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.3708785881629701e+00,     8.4618178799088873e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.1718439047783352e+01,     1.7390112851202521e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.3532832346829043e+01,     1.6903177790539972e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5256781612248977e+01,     2.5319586142230834e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    2.5006963128308769e+01,     2.3174411813095150e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.2371962589541750e+01,     1.6151874205271308e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.8626636374319405e+00,    -1.3253414865556392e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.9201922901524640e+01,     8.4518284738070834e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.8450426820640256e+01,     2.3889469922807042e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.9924484868201610e+01,     2.3819640078948287e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    8.7896703597744619e+00,     7.9302844366244717e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1432568420078439e+00,     1.7232730812820162e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.1494044318000796e+01,     3.4270016720561791e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.2009554081933253e+01,     7.0829797418636354e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1584114938797743e+01,     7.1409273217235332e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3637242958692759e+01,     2.0824988197510756e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -8.4942310153286815e+00,    -9.5541281911994593e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.4074376745407823e+01,     1.4463070619726082e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.9022712754105751e+00,    -8.4128026412347099e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -8.3514516986845877e+01,     1.6737884708505568e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.0488180547523420e+01,     7.0816791553162602e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3105379363759381e+01,     1.0163284390603982e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5757496531180099e+01,     6.4884587140137073e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0319485070823529e+01,     8.3985212684822091e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.4456641587825132e+00,    -8.4492139010060509e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.0930428233735398e+01,     8.7631468290260273e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9167976586774877e+01,     2.2652042422471990e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.7711930134212899e+01,     2.2861591888208288e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -3.1299340215883937e+00,    -1.5401931140710041e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -2.0329702561789155e+01,    -7.4739573797658959e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6275519996360302e+01,     7.0417740382420689e+03,     5.9456279154615312e-05,     1.4035836033665364e-07,    -7.2629937567019587e-10, } , 
      {    1.1765484678480689e+01,     1.6870375886154005e+04,    -2.5699286729747781e-04,    -1.5626587291574724e-07,     5.1580297160605720e-10, } , 
      {    2.8166482240941118e+01,     2.9899862334954396e+04,    -3.1765241570832613e-04,    -9.5468201087150789e-08,     2.1520879304452284e-10, } , 
      {    9.3937547132923740e+00,     1.5471796624562867e+04,     1.2443065032264228e-04,     2.8058858441739004e-07,    -1.3303869186062999e-09, } , 
      {    1.3726893544059738e+01,     1.6309588025188070e+04,    -3.1596227570653462e-04,    -2.2977768656870507e-07,     9.1795107194960631e-10, } , 
      {    8.7534561177976862e+00,     2.4915056588126263e+04,     6.4235915147480569e-04,     9.7177300212755835e-07,    -4.7406536179933750e-09, } , 
      {    3.0595348754939966e-01,    -2.6171720518879545e+04,    -6.4216328674763742e-04,    -9.7186026859015797e-07,     4.7391991769500465e-09, } , 
      {    2.4415544116803950e+01,     4.8928537674219682e+04,     3.5750452673764398e-04,     8.4537683947454633e-07,    -4.3192535663729660e-09, } , 
      {    7.6790764542862853e+00,     8.5755015289797757e+03,    -4.3497002175213782e-04,    -4.4457797560552510e-07,     2.0359750538194960e-09, } , 
      {   -1.3020416133160955e+01,    -1.2019384400782837e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.8184197785605619e+01,     1.6743918182948320e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.0316250418321339e+01,    -2.5108897950194361e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    9.4623545542131467e+01,     1.6786430455724534e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9938729050259923e+01,     2.4997549002211927e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.8896906361580101e+01,     3.2762367191603687e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -8.2750746795692436e+00,     1.5663261506849245e+04,    -5.3187191046337001e-04,    -4.1895159708670113e-07,     1.8010828253219257e-09, } , 
      {    3.8224504917501875e+01,     2.9478514023175223e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.4512909978055873e+01,     8.9661725380691587e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.7833663775265132e+00,     8.7277124990654156e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2876249666250917e+01,    -6.6856142404385100e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.6367857006024085e+01,    -1.1549689423193961e+03,    -5.2523741015816222e-04,    -6.2257596114658415e-07,     2.9213417982617303e-09, } , 
      {   -1.5517456750302998e+01,     8.4749357911071529e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.3208538491986292e+00,     5.4039359882495944e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.4142457929021077e+01,     8.3416304289717973e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.3670602996733301e-01,    -8.1463644307431750e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.0251319442112843e+01,     9.3772424512180078e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4328566878410392e+01,     7.9728823328882927e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6860855195938090e+01,     2.5306431946917553e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.9905250749261779e+01,     2.2310836564174871e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    3.6622921922139547e+01,    -1.6772974293371910e+04,    -6.1134664179665469e-04,    -9.3541255782076041e-07,     4.5653449909041671e-09, } , 
      {    1.2594296999722868e+01,     7.1512366029790210e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.8749813199770891e+01,     7.1845391935609059e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -6.5905710019676334e+01,     1.6013166305449249e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7259633628105608e+01,     9.9330114744507609e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1614509388166582e+01,     7.8941760860608474e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.2047137803132674e+01,     4.3942239720358521e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.6393751830867807e+01,    -1.0844165581510897e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.5146470594060204e+01,     8.4887269789052934e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.9803959054658321e+01,    -1.3145751242175102e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4317056705176658e+01,     1.7393094999476780e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.5234020341586536e-01,    -8.7278298831341590e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.4298679671108776e+01,     3.0980381531137366e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {   -4.9696408910271241e+00,    -3.0731744521309092e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0865776178135540e+01,     7.9651941267278999e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.1517459599996521e+01,     8.8360149414601256e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.0318430356669303e+01,     1.4217447014419127e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6766548237552840e+01,     3.0981086163053515e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.5336783982136772e+01,     2.0658585851589763e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.9911656479202300e+01,     1.7074497785699456e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.2345803114921317e+01,     8.8345922505263734e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0865075649777982e+01,     2.3549418050325246e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -3.6921337876346797e+01,    -7.1265611292558624e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.0915934095485001e+01,    -2.5078229768320511e+04,    -1.7874949551903517e-04,    -4.2328437695477703e-07,     2.1522333745495625e-09, } , 
      {   -2.0180078787381724e+01,     2.8560984319098134e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.3875439177486157e+01,     4.9417802681809007e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.7627344488891605e+00,     8.3635763794728173e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    6.8201581155173709e+00,     7.1252876407141930e+03,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.6705108569679350e+00,     2.3989338358671659e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.1950058468323464e+01,     1.4412243760736710e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.2192807261288053e+01,     2.2901406790106201e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -6.8281482936697870e+01,     9.0435343433704129e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0406055922334003e+00,     1.6733771648704791e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.3503218567401984e+00,    -9.9073342589945350e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.0711996187077101e+01,    -7.0932886242661962e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8862707162276436e+01,     1.5616350619334868e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.9329737643093424e+01,     7.0418642534183482e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4819141829690725e+00,    -8.3003579351661647e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.9483874951064095e+01,     8.3045128979521869e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1345922682706281e+01,     1.9401970681498678e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1332135124674162e+01,     7.1355965231167520e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.5271976445770115e+00,     1.5653429076935174e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0398164051598425e+01,     2.9775933788400216e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -7.9779106008228382e+00,     2.8973911543346157e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1350687588346606e+01,     2.0692367279044927e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3217651607754398e+01,     1.4221179288920470e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.2291081850498145e+01,     1.4402980801644364e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    9.5699755453482584e+00,     1.7691254835755126e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -5.0450823678494942e+01,     9.6778754028587082e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4642214886346840e+01,     8.4034874996457638e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.0678718740166744e+00,     1.6790543515525307e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.0999591105525557e+00,     1.2327593227024905e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2612787683882157e+01,    -6.5021427364818246e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    7.6416976124009537e+00,     8.2062846654808309e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.1482084409991167e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6231990402378798e+01,     1.7333377268497014e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7040274927342680e+00,     6.7604197911997371e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.2364492177730355e+00,     3.9213495413893302e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3805866550147954e+01,    -1.1134650977108736e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.5923995162936109e+00,     2.4483871329670808e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.7225013977086181e+01,     1.6763253462982539e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.9471625629677945e+01,     8.4357331464400395e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.7485395153537056e+01,    -6.6879154403799203e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.8596787578252076e-01,     1.6720749571033619e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.5612865207435838e+01,    -1.8343813426291285e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7779345888735442e+01,     3.2151610629568904e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -8.5355538087770782e+01,     1.5437881505539803e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.9948588776669447e+00,    -1.8662828080766132e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.1074630387748871e+02,     1.5438077608687483e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -9.3720054735806180e-01,     1.5632109911710266e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.3565010852986081e+01,     2.1554334544014480e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    3.2555748983512132e+01,     6.4806519150698778e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.5964664879133732e+00,     8.3509469694431664e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.3918280145252284e+00,    -1.0962444241188225e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.2917400819704994e+01,     3.4663037690254765e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -4.4298849139108185e+01,     1.5742700854934201e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.1528924744429286e+01,     3.2322936080298641e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -5.8354431333682832e-01,     6.3366721786719687e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.8254923801824994e+00,     1.5016591605488218e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5440765199452345e+01,     1.3302022443110009e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8240035012113267e+00,    -1.0509904694423823e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.0170501962718301e+01,     9.4636111810387938e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.5376270389268010e+00,     2.1933563337996333e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0687875904874232e+01,     1.6639298706885074e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.7436665924658916e+00,     1.5898563381716953e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -3.5191711843882132e+01,    -1.6761061701247563e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.1806857415293109e+01,     3.0194691907709464e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.1896729288717815e+01,     8.2299501661980503e+03,     3.9950555821570275e-04,     5.4318704212574410e-07,    -2.6171212133654968e-09, } , 
      {    8.4215962294496947e+01,     1.5592172092663603e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    7.8596125631883034e+00,     1.5612191951913223e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6243717457787412e+01,     2.3713701884525901e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.1930691626432440e+01,    -8.3444486438768108e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.6376796546099470e+01,     6.4640598666370588e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -3.6839201682195196e+01,     5.7382271032204517e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.7498092748049714e+01,     3.1741038636865156e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    6.7096077669288663e+00,     8.4618520909018589e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5506878166493633e+01,     1.8491975815227433e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.6147899207398673e+01,     1.5062310920881487e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.8512777107903659e+01,     1.7565266803040671e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.4229647004724963e+00,     1.0476123266968661e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.1353603288438709e+01,     5.9943072572670162e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -3.2124410768232821e+01,     1.7231205559568945e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.7821924740409898e+00,     1.5314026630228374e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0490731895780236e+01,     8.0596233818157334e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1756222388768212e+01,     1.6823936128537680e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.9688102076447251e+01,     5.9950118891831644e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    7.0548371066786046e+00,     5.2670251933092677e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1612085181744902e+01,     3.2324099541852876e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.7204484119376261e+01,     1.6260104123896190e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0445699495129103e+01,    -8.4111216611257987e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.2342733254376075e+00,     1.6544904650703204e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.2763201416978029e+01,    -1.1722625234546022e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -2.1250728982821421e+01,     8.4373540501843527e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.4506539469635634e+01,    -8.4295782647988490e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.1952538015373264e+00,     2.3963601209055407e+04,    -1.9201849612945083e-04,    -1.6035648835010902e-08,    -8.8284571330046551e-11, } , 
      {    6.9390040719161483e+00,     7.6839152748576689e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4689279688279232e+01,     2.6885101589664737e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.3110431199346849e+01,     1.2376295092839460e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4573485966105435e+01,     1.0556305360296970e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5154107367958737e+01,     1.9494623229826175e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.1221052034014041e-01,     8.4050802240813427e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.6021025355829366e+00,     3.2930918257200974e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -5.1305692215931451e+00,     8.3772502303292713e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5764853403145478e+01,    -7.7925393340280689e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -4.7356701418883141e+01,     7.8665489770751519e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2919552627947720e+01,     2.6277574398810804e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.8902080778231067e+01,     7.1556887983633414e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    6.6785474175489599e+00,     1.5732225898687459e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.8869848201202817e+01,     2.3436930376042637e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -1.1934916818671299e+01,    -9.6277245205525305e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.0439120934916069e+02,     8.4335642090654401e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -9.1710621147837998e+01,     8.4333681059177616e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.6977544790531752e+01,     5.4607475730963642e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.9393417845687075e+01,     2.3960487963474094e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.7738527385102127e+00,     8.3349836606360077e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4909501277864187e+01,    -6.9833448805747139e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.2199824316884758e+01,     8.4577390311010822e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.8440186685020805e+00,     2.1621897398924797e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.0303184621146038e+00,     1.2238125193035150e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2326624129057626e+01,     1.5122510848432779e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.5551366746802193e+00,     1.1508401412363847e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -8.0142727666163687e+00,     3.2512160137891733e+04,    -2.5820167225328446e-04,     4.6082073684510614e-08,    -4.9630376535183202e-10, } , 
      {    2.3469587857153002e+01,     4.7872449417288408e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.1385293241825686e+01,     2.4579494129306931e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5894565196181324e+01,     1.5645386784628125e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.5935130002199529e+01,     3.8996605346832032e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    4.4148985177328335e+00,     9.4465451612663003e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.5116693104369190e+01,    -3.5811029167656852e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5029122557700932e+01,     1.7297165162634017e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1667213124461718e+01,     8.1966867296922192e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.4819281621240759e+00,     8.4444867842139647e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9154749590966591e+00,     8.4224455307692369e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.3185120390755003e+00,     2.4858959472194845e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -5.4817625606682157e+01,     9.7116568303138683e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.0681857903611167e+01,     2.2839049983076045e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.4617171397429299e+01,     2.3802719398140693e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.6375389160836136e+01,    -4.2184559337547125e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.8907561362849936e+01,    -7.0809364100729090e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6455024781841033e+01,     1.5907379850525613e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -8.5340381034113371e+00,     2.3348241494802627e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.9836033083619355e+01,    -1.6552255800420673e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -8.5801760743185540e+01,     8.4091932838821176e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.1288799345462564e+01,    -1.6552960432336818e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    2.7390048296319048e+01,     1.2803097489987611e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1078212388601196e+00,     1.2031939567925394e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4513142073630084e+01,     1.7253394329077055e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    8.4317007388415899e+00,     2.3902624118120322e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.3326978800756839e+00,     1.5610769260979470e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -5.9195169755509909e+00,     2.3991952931472693e+04,    -3.7957950270104383e-04,    -1.6823272293204638e-07,     5.6509882670127486e-10, } , 
      {    4.9390672247066433e+01,     7.3191899141845697e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -3.1621438582567553e+01,     7.3184852822684215e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -6.1951084941800175e+01,     2.3347919575978576e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1633075902109972e+01,     8.5600489166930565e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3826062406377524e+00,    -1.8127659881694071e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.6399368525858790e+01,     8.3708343348388771e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8735023683409533e+01,     8.5262674892378982e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5326955651941775e+01,     8.4434555635934048e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6957637599118211e+01,     2.5059697411599525e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    3.4509114466848160e+01,     3.8930482054307562e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -1.5186637668939259e+01,     2.5088771463954796e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -1.6539486924610394e+00,    -5.2616268449357631e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.8171348121090801e+01,     7.0020493515204362e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.1006282222425554e+01,     2.5092926549522203e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.5172741238063786e+01,     2.6662932885510447e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    4.3289244348679645e+01,     3.2516762537774553e+04,    -5.6278648777793676e-04,    -4.5535567462479882e-07,     1.9756642318894697e-09, } , 
      {    5.5264485890875164e+01,     1.5704616921513527e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3013869266308127e+01,     1.5629495338909237e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.1977139223840240e+01,     1.5647169993341146e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7652694555620883e+01,     2.4445620740885362e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.8060683201666485e+01,     2.3806485883635007e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -2.3215063221085469e+01,     1.5647888052358745e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1542693826308028e+01,     3.3418829085895079e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.1636339971390722e+01,     1.7847920486667437e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -8.9730633474226025e+01,     8.3355969545290445e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5271290697504591e+01,     3.3420251776828823e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    3.8004626223478679e+00,     7.1004097439896505e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.4788646959678772e+01,    -1.0380833046708813e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.3225370336930318e+01,     1.6293885551351348e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.6227337958629207e+00,    -8.4245877689910594e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    7.4290049120856194e+00,    -2.2763360809812684e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2104722131206660e+01,     9.0644035748533279e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.9491814797365898e+01,     1.6293109604661156e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8156915315738530e+01,     2.5346799138347949e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    8.9919888293841495e+01,     1.2904760647763305e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.3660264332207479e+00,     2.3726856079839181e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -6.3866256028016792e+01,     9.0097529159152527e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.1532938776916808e+00,     1.2135032380480272e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.5418833474725062e+01,     2.3881048615389140e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.7988161752898684e+00,    -8.5202166821579540e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.4383634634405428e+01,     1.2014425718285451e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1701328023750083e+01,     1.3178069516224738e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    7.7745018179453798e+00,     9.1611867923965846e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.5558908747930765e+01,     1.4068919558172636e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -8.8233875015816565e+01,     8.0501859258668446e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.1454116441952561e+01,    -8.6530202888448566e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.8014453812037790e-01,    -8.9137552310428055e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.7606816355349409e+01,     1.4183665586963967e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.8904420066920995e+00,     2.4069520451999968e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -1.5305080702342645e+01,     1.5634022511578591e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.2320553560966419e+00,    -3.6579849783696045e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.1578138860852478e+00,    -1.7266512240275322e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    2.9470558327904616e+01,     4.2828902835685694e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.2623889778491701e+00,     2.3283478512613990e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    9.8818328081228568e+00,     1.5018510092337527e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.9704747152549956e+01,     1.5003679409515973e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.9159142700385914e+00,     7.3045248559750962e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.5766931882544423e+01,    -8.4242926290914365e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7080565509949750e+01,     2.3608936627396663e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -2.8464849415303355e+01,     5.0625985541172395e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.5508445351548694e+00,    -4.8107753953577273e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.1273830592849947e+00,    -7.1505810971189499e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3683922562617081e+01,     2.3870282253051013e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.0797665051363502e+01,     2.4111207181455560e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5379332643494285e+01,     9.3721733812727325e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.9556782889930204e+01,     8.4131160140943284e+03,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -4.2812593376744431e+01,    -8.4538163008888732e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.2737385890337968e+01,     1.4452733629771683e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6076580158906708e+00,     8.4296633878822759e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.6422608288743540e+01,     1.4884234703822236e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0338310456749699e+01,     2.2326584307689322e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.7905312072002495e+00,     9.7518598740053185e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0072926355200281e+01,     8.4372689271009258e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1726003025607369e+01,     2.4603816687052062e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.4806453180026061e+01,    -3.6427324458574367e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.0602669050237722e+01,     1.4555240532392198e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.1395128257206768e+01,     1.5665161049124414e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.2056068586217044e+01,     2.3991639558613075e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.7322459449816456e+00,     1.3682424174385007e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4685278306107246e+01,     1.3935759733193428e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6009774758224452e+01,     3.9361240383114753e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -1.4833836702783707e+01,     8.5473404485755600e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4842415111660213e+01,     2.5514522898790183e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.5834711954480010e+01,     3.1348017667172167e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.3579596705900851e+01,     7.1519479484458971e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7678706143653301e+01,     3.1014515274550598e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    1.3010722730736227e+01,     1.3728830470089641e+04,    -2.4890329045516808e-04,    -1.5815868248953511e-07,     6.7839493583976237e-10, } , 
      {    1.0612044412355061e+01,     6.0284410006802491e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.3280289575471695e+01,     2.5685369534497400e+04,     2.6882934596982700e-04,     5.3311300168323284e-07,    -2.7304173225039839e-09, } , 
      {    1.0298594442598489e+01,     8.6092341957727403e+03,    -4.6568873433953641e-04,    -4.8106931960622267e-07,     2.2091020193437112e-09, } , 
      {   -7.4479096944011296e+00,     8.4002370195689236e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.9837161023422070e+00,     3.0551255086750480e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.6504582667879948e+00,     2.1674866496135986e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.8162671156331243e+00,     1.5019227067635886e+04,    -2.7981786776973489e-04,    -1.9456276002763283e-07,     8.5297634240730622e-10, } , 
      {    1.2883970828687533e+01,     7.7905648147469283e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1159048484837173e+01,     1.5357797463785335e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.3020315155642965e+01,     1.6755546182562946e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.5456747839020050e+01,     1.6102335558562787e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -3.5960330606587689e+00,     4.7047616484591208e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.7562472441826870e+01,    -1.8477041636280733e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.5922324631562059e+01,    -1.9273406809746241e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6100937519910449e+01,     7.6853379657914220e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2810750099286501e+01,    -9.8025261564326102e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    5.9890712488925608e+00,    -1.6009897070184235e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.8239411454008252e+01,     9.1883565017386009e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3343609854396430e+01,     3.3459355333259868e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.1847258606593186e+01,     2.2729594683484112e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -2.1669451677939158e+00,     7.1006516341073602e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.7218279645897219e+01,     3.1623635189840112e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    4.4069682462060626e+01,     1.5661035534121300e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.6883869740324045e+01,     3.1112731051819614e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    3.4799519123135827e+00,     3.3379725529464034e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.7967435304036243e+01,     2.4826696736673224e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    4.7158680167174651e+01,     7.9644181800377064e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.3571238199895399e+00,     1.3316066627840920e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.8350417984012545e+01,     1.4510358558452595e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.6707972768592354e+00,     7.6388799457892304e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.3035668924206298e+00,     1.3977229366808979e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1339846679530350e+01,    -1.1120255353250234e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.5562672608434176e+00,    -1.5454900237921232e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    9.8199965852053630e+00,     8.1010578165253237e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.2031501088979937e+01,     2.3332370834139423e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.4891915039368588e+01,     5.1802659114218286e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.8767002669545121e+01,     2.3212926128445688e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.4397627115621878e+00,     8.3922065634673836e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8442952723651754e+01,     3.1835511578684880e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.7848525461936271e+00,     2.3518765127966552e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.8316811740527502e+01,     1.9342452104427266e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8054919534063952e+01,     1.5543917751536026e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    8.5828453304055401e+00,     1.3865895678341694e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.8065248984662485e+00,     1.6818824561029625e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3636408601445680e+01,     1.5881969979978170e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4604794825742527e+01,     5.2641802792514763e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4055501532975015e+01,     1.4874106321881958e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8370189932045754e+01,     9.8098817168423670e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.6944299101295712e+00,    -2.4546885542464911e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4743176525843243e+01,     5.2086176338391142e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -6.1952257893946232e+00,     2.4819144570296929e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.7121385687596955e+01,     1.6569116547068243e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9840142224550277e+01,     1.0331008151046217e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0883839201252052e+01,     2.4796043814314529e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    2.3582806132341180e+01,     6.7807015126110655e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.3314146838428151e+01,     6.5545768767977233e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6897592093545754e+01,     2.4979515318271213e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.9399878368755260e+01,     1.8458194387772273e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1798016599526932e+01,     1.5900864581658361e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.3672304616251516e+00,     1.6165028400584586e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7011857992355282e+01,     2.3244301591113934e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    5.0776791116680549e+01,     1.5998863263087342e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.5437040511241619e+01,     1.0125347877581422e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.1060120879056257e+00,     2.3998762352605638e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    3.1914775386984651e+01,     5.6380564246211161e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.4999875627605718e+01,     8.7094730998986852e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.1030453959579397e+01,     8.8241859272431793e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.6619215815461956e+01,     6.7198699588321242e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3750984049240710e+01,     1.0289815619929595e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4183789664936430e+00,     2.3960801336333716e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.2316364178582361e+01,     1.6666597608237262e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8379587807000444e+01,     1.7987621157067162e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.5455671600789032e+01,     1.5702885953036288e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.8926699879533455e+00,     1.0869144236661650e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1231898528634133e+01,    -1.5167407152157739e+02,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.3071112470285231e+01,    -8.4311991685431622e+03,    -2.7584724895743558e-04,    -3.9757073197335323e-07,     1.9187955870953214e-09, } , 
      {    2.4339961645404735e+01,     3.5056058659947754e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.9044524709564454e+00,     1.6655327929078089e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2550698252348504e+01,     5.4892108634095212e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4326881321631003e+01,    -7.9783120379261682e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0684180125990052e+01,     1.3030158359642337e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3150541733588746e+01,     2.5196787201160882e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    6.4439889666482358e+01,     2.2024087153384520e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.7553084101820509e+01,     1.6120732085871241e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0397412806201839e+01,     6.6814929531955659e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.0748804211497127e+00,    -4.7713218630424490e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.0165398919125792e+01,     8.2545696141139961e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    7.9054469991843570e+00,     7.0912639242883943e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6401296709062972e+01,     1.5981590266321673e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -5.4079852350839779e+01,     1.6959501123127375e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.9865559928546261e+00,     1.6758354812505724e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2405097033940656e+01,     1.6765960351724378e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.6374296425865040e+00,     2.5072800063282641e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    9.3794802561611590e+00,     1.6275125262428612e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.8324109983604174e+01,     1.6053169230960924e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    5.0392203988049005e+00,     3.2559343311772747e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -1.2511054304912566e+01,     8.4426396858917687e+03,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.0646695488151959e+01,     1.4967406326408531e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.1969873648484619e+01,     2.2717304077440902e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -2.1089153453784107e+01,     9.5706366218889507e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.7746316851388491e+00,    -1.0300355228687522e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.5607329050898327e+01,    -1.2505816956483345e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8493227290154916e+01,     1.9368189254043518e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.1866079944105712e+00,     6.4633179324734215e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.2568455640667242e+00,     1.3815575364924507e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0260041485640969e+01,     3.2165192250219301e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    8.7010000915041310e+00,     1.5514420800524509e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0182548408053322e+01,     2.4291689156154258e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.3538930510195701e+00,     6.7266383637445770e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2621495541932820e+01,     2.5090849006738499e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -1.1806497002513423e+00,     7.0191166330600827e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.1539324559236235e+01,     1.6575150021510995e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.8834233394843647e-01,     7.6683373520627356e+02,    -2.4977909958270097e-04,    -2.9189540908822662e-07,     1.3259702659713917e-09, } , 
      {    1.0396614346661902e+01,     2.5752908009076626e+04,     2.0709812370427758e-04,     4.6026121337573754e-07,    -2.3819817298905608e-09, } , 
      {    7.3412730687851733e+00,     2.4496304098648503e+04,     2.1242588669009458e-04,     4.5902842914741218e-07,    -2.2731022733869815e-09, } , 
      {    2.3703973327921172e+01,     3.0946976799971377e+04,     1.1474886584212520e-04,     4.1674724624143233e-07,    -2.1964483822667524e-09, } , 
      {    1.4077303032935003e+01,     1.4951712973387759e+04,    -2.1818457786776960e-04,    -1.2166733848883765e-07,     5.0526797031554708e-10, } , 
      {    1.0427955798592659e+01,     7.0957665418927718e+01,    -1.9061382644647591e-04,    -2.1847086189786850e-07,     9.2236772458451409e-10, } , 
      {    1.7730415734223124e+00,    -1.3276215961722082e+03,     1.9080969117364415e-04,     2.1838359543526878e-07,    -9.2382216562784285e-10, } , 
      {    1.9818655118182804e+01,     4.9138087139955984e+04,    -7.5092619539975601e-05,     3.3324865860856294e-07,    -1.9061419500183626e-09, } , 
      {    1.0837261760943125e+01,    -9.7914012296170731e+03,     1.5665196495659987e-04,     1.1460108212393000e-07,    -4.9358880873761860e-10, } , 
      {    2.5810928039620165e+01,     2.1571170910330948e+04,    -4.6994482347065226e-04,    -3.4618707524180555e-07,     1.4511927916651738e-09, } , 
      {    1.4462844623412312e+01,     2.5128079635195947e+04,     8.3347417781999913e-05,     3.1349662201963880e-07,    -1.6977157003725618e-09, } , 
      {    4.4154853113863224e+00,    -1.7495264280359843e+04,     1.2811026311655135e-04,     1.0646799325374076e-08,     5.8129160365033399e-11, } , 
      {    2.9601873955586422e+00,    -1.7285714814623541e+04,    -3.0448688316106823e-04,    -5.0148138154060924e-07,     2.4712407767196374e-09, } , 
      {    2.2103115952405382e+01,     3.9971544294382169e+04,     2.0787600046822630e-04,     5.9404157320572893e-07,    -3.0288298395005259e-09, } , 
      {   -2.8943236519700957e+00,    -7.7376501080294147e+03,    -2.9100601839024149e-07,    -6.6933813146295539e-08,     3.2269683428331825e-10, } , 
      {    6.4565994923758243e+00,    -1.8477052473473168e+03,    -1.5180553701676770e-04,    -1.8387232747095888e-07,     9.1183272329400397e-10, } , 
      {    5.5132178680918731e+00,    -9.7576441824930062e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -6.2588421811217970e+01,     1.6761951866889030e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.6474649454289370e+01,     3.1088111603903671e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.6166541025713489e+01,     2.3764041149420911e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    7.2245811982717818e+00,     2.5697826683806190e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.3358416931011881e+01,     8.6550132339713746e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -6.3884626551245880e+00,    -3.4661954218238970e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.6111207061966191e+01,     2.7278122559357726e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.4459520485262466e+01,     3.0092575434374226e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -3.4444566236077058e+01,     1.6602897974523403e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0218077224794222e+01,     2.3769300814053277e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    5.0763691021865087e+00,     1.6857717555992840e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.1071139527820382e+01,     8.8886202770570344e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.3439290409202229e+00,     1.6748994717170866e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.7965225061109898e+01,     2.1472979513393027e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    8.1891110259890816e+00,     7.6736059936021820e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4110060562786694e+01,     1.8098954845534445e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.3916870843478907e+01,     8.1025783376710588e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.9443905905179335e+01,     2.4617147270936926e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    7.7273746330514497e+00,     1.5619143089439687e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.0674122162257760e+01,     8.3443277998500980e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4514650773805094e+01,     3.2351312537969996e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.2088317101222494e+00,    -1.3433562190185280e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1945883861506509e+02,     7.1072526154731186e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5053443664830501e+01,     1.7863668230181887e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.8114414527492997e+00,     8.2834372543585705e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6110583922141839e+01,     7.0886695699792190e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1987521272519519e+01,     3.0561623411828117e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -9.8779378368003137e+01,     7.1113236495072688e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.3092141856506796e+00,    -1.0155072393824397e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.2768366344706628e+01,     1.7108279213154608e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.1134555228452228e+00,    -9.5672823865127375e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.4702663585552187e+01,     3.2935520657083805e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0298252048694513e+01,     2.3240535105619620e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.4622086203417237e+00,     1.2547403630699780e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -8.6266133526159088e+00,    -8.4051144350743143e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    2.6314254619547974e+01,     5.5732868811148164e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.7644310453741028e+00,    -1.1138613242025639e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.9988185404670229e+01,     2.2719252961424234e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.3546983398671806e+00,     1.6202213470166316e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.7185396142872278e+01,     1.6473944714983991e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.2830445266954275e+01,     1.0995726995863104e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6883364129136272e+01,     1.6882814284963180e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.9609662601778188e+00,     8.2119190810118271e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.6474553813995431e+01,     8.5293807348192058e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {   -4.9894604824243594e+00,     4.8959993521402343e+04,     4.7898028954898749e-04,     1.0596480028598035e-06,    -5.3813833789477374e-09, } , 
      {    1.4432886738275567e+01,     2.5126186077675222e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1050637189240971e+01,    -8.2112974533373126e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.9018834789600135e+01,     3.9571943834666381e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    1.0856643345628067e+01,     7.1299066950011684e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.8021263216734397e+01,     1.1274784444772624e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.1478518324337614e+01,     1.6730677354601299e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5225684477075266e+01,     5.8503717624576966e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -4.8867325558460642e+01,     1.1623153934696757e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.2726416750739176e+01,     2.1149822598551771e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -5.5523123451682643e-01,    -8.4338619915316121e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -1.1248398410829044e+01,     9.2088274254048138e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1700523249079524e+01,     8.4177184139771489e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    4.2446904956765998e+01,     2.2103364641222270e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    8.0961114145344737e+01,    -1.2747602223195981e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.3424915515734895e+01,     7.5383917295797528e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -4.3976669850593986e-01,     1.4769857070170188e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.9689655765898131e+01,     3.8892968352351061e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    2.8159141849937868e+01,    -1.0323957315291991e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.4462297388180807e+00,    -8.0999637648700282e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    8.5528011473601495e+00,     1.6345726385914273e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.0464363226203126e+00,     6.5121321425836268e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0969778410473623e+01,    -1.6739615676982805e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -5.7158026245644926e+01,     9.3438141593869805e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.5817387583061773e+01,     2.5075306978450069e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.0250605144782298e+01,     1.7022622496233707e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0440068305047802e+01,     1.6207473134798684e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4095773365907693e+01,     1.0019761999117112e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.5012290920962403e+00,     1.7625751782513151e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.8519256579349257e+01,     1.0568924598714957e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0900622350961690e+01,     2.4140513757412962e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.7427924257963156e+01,     8.0427463877400223e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4194832723028551e+01,     1.5231097826649167e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.5998415628565084e+01,     9.6718419284159554e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.7552327086111546e+01,    -1.1242314600490022e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.9360699015518215e-01,     8.4540847200025764e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -8.2178396462085956e+00,     7.1318300376224397e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -6.9997827452589419e+01,     2.5106391035026929e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.0486145650942305e+01,     2.5706038680585778e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.1724675203267054e+00,     8.1392198159177851e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.5552697352747003e+01,     2.3178178298589464e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    7.2141344424279779e+01,     1.6761345536475150e+04,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    6.4174424836691955e+01,    -1.6762969627754952e+04,     7.0640665681691425e-05,    -1.9827914778154611e-07,     1.1496871633831536e-09, } , 
      {    5.8037482385342569e+00,     8.4128475949806252e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4671849093952680e+00,    -3.1845081405981937e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -7.6690604056397769e+00,     3.5283285716963428e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6593997690055154e+01,     2.7055953855203436e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -5.7496734264793474e+00,     3.2327453777229082e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.5590275881821167e+01,     1.3144288088769577e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -7.7763551848657443e+01,     1.5623256149240462e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.1849282387452917e+01,     1.6921417189706699e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -8.9355060234272699e+01,     1.6762059530541213e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    9.3893279689795506e+00,     6.2456939320920947e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.8093385370373785e+00,     1.8018772752206136e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.0674677637873033e+02,     1.6762255633688888e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.7172628362787218e+01,    -8.7643539773121429e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.9937988463769933e+01,     1.5444013031556397e+04,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {    1.2047408056244231e+01,     8.2805836706698738e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7795727973166251e+01,     2.1630713867733459e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    5.4631561089573450e+00,    -6.4449321310410523e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.3413620937866479e+00,     1.7978957850308227e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.5811559790100745e+01,     2.3347584230053078e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.8490438928482320e+01,     5.2753906246260158e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.0302133550278523e+00,     3.7636151870488984e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0073896387138852e+01,     3.3876995750868802e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -3.0002222842686344e+00,     2.3862809216222413e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.7301721105741592e+00,     1.2561630540037308e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.0081138321960541e+00,     9.6502664256847747e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.5877160269904110e+01,    -8.4119867635330684e+03,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    7.1840616018352916e+00,     7.0371313655229715e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0438012810192451e+01,     2.4621448713326532e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    8.9591878162477165e+00,    -8.3341393626213230e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.5996859735884291e+01,     1.0949326329989955e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2845686803298983e+00,     1.2017712658587866e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.5332742466531762e+01,     1.7971873413552708e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.6314033102310901e+01,     4.8964595921285145e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -1.6660790736619546e+01,    -8.5468773887425868e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.1842497473339627e+01,     2.4019991281030361e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.8112976966671560e+01,    -8.3373279230062853e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.9487296762022162e+01,     1.7667438511968743e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.6002966373625895e+00,    -1.8503407094949825e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1762058242156161e+01,     4.3023284765081746e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.5550847276349549e+00,     1.4921005660535386e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0262626288069825e+01,     4.0111233906097135e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.1900221539495911e+01,     6.5611626407729573e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3624769635522402e+01,     2.5127608768608974e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.0337897894683454e+00,     6.9258166285335083e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.4552242462964058e+01,     1.2544366437134293e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.2156920480444256e+01,     1.7294863962692609e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.3338724091802554e+00,     1.6751848300859561e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.0567687503907486e+00,     1.6772466863370541e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.5105632926148203e+01,     8.8133757773039924e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4628897312017997e+01,     3.1583304282524725e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    5.4223670946992506e+00,    -7.6441243410864018e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.9646805909122932e+01,     2.9871534992868208e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.9305937975436542e+01,    -8.3771428249041965e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.9896132080741278e+01,     3.2326241780074699e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -8.2220022477166843e-01,     6.2330746936741070e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8769401927153925e+01,     1.6513753055564230e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4994553037683865e+01,     3.9466005640243995e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    5.4238445990199097e+01,     2.3975861417964592e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4214847039163340e+01,     1.5518161650441953e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.8427762080463040e+01,     6.6955970852226419e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.8715550228214440e+01,    -1.2197623216420579e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.7806488426793059e+01,     2.2291439598433499e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.0764907056706956e+01,     2.3976579476982199e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.6465129983114235e+01,    -1.2190442626244549e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -6.1786468253743259e+01,     8.4344144280809014e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7915807586955491e+01,     1.4812301804384284e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -7.8528813422402060e+00,     8.2152801933724659e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.0433216412060142e+00,    -8.4792451956902332e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.3168352510157739e+01,    -1.9826976111609656e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.0366503205201766e+01,     2.4604880348477011e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4752500132267368e+01,     1.5675914956260203e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.6782145846930483e+00,     1.6790491071572338e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7712550566239194e+01,     6.2603305352239076e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5811347601334163e+01,     1.2407446687978438e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6494682519474555e+01,    -1.2259622477323892e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -4.5414028113863765e+01,     7.1088999906966583e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.6453461455705536e+01,    -1.8632529054672295e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.6093697441452690e+01,     7.1096762742837291e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.9245474920393573e+00,     7.2232516532736618e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.6276422816231310e+01,     2.4876232468960508e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.9298146832055640e+01,     1.0444610049337803e+02,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.6015186427116916e+01,     2.5214801933623774e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    7.7176791584923681e+00,     1.4729489878739791e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.5057184524685127e+01,     8.4411543636519928e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.6171531026251508e+01,     1.6116577000303841e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.6357675087506955e+01,     2.8221400932639031e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1370182764075100e+01,     8.7675274009633285e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4680963498889286e+01,     8.3658920429183145e+03,    -7.5047737331670442e-06,     6.8695723026183823e-08,    -4.8747046008201626e-10, } , 
      {    3.8914497797590386e+01,    -1.0661771589843593e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1287403774214278e+01,     1.1262165206354637e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.8516714206852516e+00,     2.1568928301713604e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.7893404100811640e+01,     4.1208941784554641e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -2.5223691164505784e+00,     1.1934572257331918e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.0490540009985581e+01,     1.7939142948410314e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.5096763607041886e+01,    -2.3565737565306063e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.9359256147706245e-01,    -2.5071661336982470e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    2.5058803212304223e+01,     2.4755197894440635e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.8534842859577836e+01,     8.0871989892958954e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.1719838310151403e-01,     1.6982807594335798e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.9265112840100382e+00,     2.4984910812316117e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.6401742228135650e+00,     9.6836893326624966e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.3085254145216503e+01,     2.3360532100845841e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -2.1340064410727319e+01,     8.2139120261383432e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    3.7023399637229115e-01,     2.0033470674594128e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0510236833581393e+01,     1.6257812399693690e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.2553720956703991e+00,     9.0591439102209588e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.6308423674099952e+00,    -7.0190391439861583e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.6359697370277626e+00,    -1.8484222226456759e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.7736502860177250e+01,     5.9663078508078015e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.0770310798166443e+02,     2.5090341048099381e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.9502507929330298e+02,    -2.5091356965377618e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.8560962534488798e+01,    -1.2329098034046103e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4035006885418843e+01,    -6.1091217667888377e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -9.6853448694051103e-02,    -7.5881946752994804e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4298997217406495e+01,    -6.2925932707455231e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.6999392743453974e+01,     2.5015597945667789e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.5502871426973091e+00,     5.3794698993778120e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4620000059771801e+01,     3.8968219413421764e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -1.1860956322893863e+01,     1.6508822023306631e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.5994043136225899e+00,     9.2147589903602238e+00,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6607982927274641e+01,     2.5765557257657190e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -1.2218765070248969e+00,    -8.8775224644153468e+01,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.8927529297235655e+00,     1.3658994996399397e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.0809008521876109e+01,     1.5031840676222393e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -2.5132047395759677e+01,     9.9843700610555970e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6709349013890380e+00,     8.5290261313693918e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.5732407870323485e+01,     2.4462189105734884e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -5.8221383042756152e+01,    -8.4456720822155767e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    2.1960919144380782e+01,     3.2010665530523031e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    8.7772669642458467e+00,     8.4503269041391668e+03,    -4.6568873433953641e-04,    -4.8106931960622267e-07,     2.2091020193437112e-09, } , 
      {    1.8278060995684346e+01,     3.2283581964118996e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.0024636816397809e+02,     1.5671801896459428e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.9562128023039635e+00,    -8.4606961195546810e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.8808608605090598e+01,     4.0045110613572659e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.8713489453859342e+01,     1.4792751291260509e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.4936903084157695e+01,     1.0125700193539496e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5476193994165055e+01,     2.1899781910541173e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3736370744438577e+01,     2.6269911915817462e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.4251825876944750e+01,     1.4041663716622836e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.2789141666122350e+00,     4.2805890836271574e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3172898345393371e+01,     4.0686572393931485e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    2.6759158873614009e+01,     2.1051606821282752e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2142619766579291e+01,    -9.7046750852818150e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -7.1273034915867228e+00,     3.2302834329313147e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.5797732604058776e+01,     2.2783595001315865e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.7178706412256822e+01,     2.4569583143509542e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    7.5978937409621228e+00,     2.4775380298057637e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.0651913990778191e+01,     1.5643726253240620e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.9129252266289715e+01,     7.1179246308730280e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6108138971885431e+01,     1.4328780702886412e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.6979314130467770e+01,     6.6740215824914594e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8891488598923385e+01,     1.7739333508451051e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.5846827519263554e+01,     3.3437564639563745e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.2151819069861670e+01,     4.0067397694741635e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.9029757059715905e+01,     3.2306989414880551e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.3606476660323032e+00,     4.6736100533201452e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0005322234277592e+00,    -3.0513569500586946e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.7797951383804776e+01,    -4.6114769034476985e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.0599294104457975e+01,    -8.1486656306845871e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.0163905746718505e-01,    -1.7079912553012564e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.8773224116684489e+01,    -8.3919965238760487e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -1.5935010864266758e+01,     8.4929694750478648e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.9190821351080984e+01,    -8.3739628399353351e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0121153406152708e+01,     1.8435203948406917e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.6908296481650211e+00,     1.0857750247747629e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.7137692425753947e+01,     2.6403434045979975e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.7870126766868353e-01,     1.0097196325882521e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.2684923411058158e+01,     2.3900969386402685e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -7.2802627389987634e+01,     8.4304069527916872e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -7.6058437876801861e+01,    -8.4365253621915126e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.3770992329549969e+01,     3.2287638875331177e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.8111181276793069e+01,     1.5651331792459272e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.1608106337489197e+00,     2.2572412618676164e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -5.2481729159645496e+01,     1.0339964415359411e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.9147260900206815e-01,    -8.2452465997331510e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.4296535999762774e+01,     3.2090295334318856e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.2792351060165743e+01,     8.4123039684544274e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    3.1731020679220997e+01,     1.4610467984112114e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9055217355886533e+01,     1.3716373320780849e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8769824682649972e+01,     3.3412981475762819e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    8.7957373443055218e+00,     1.0147695991393971e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0829977776574729e+01,    -1.7090744201994181e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    4.0813895868277747e+01,    -1.2058967808619022e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.9298309202129325e+01,     7.3023291675370028e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4886753337992257e+01,     2.2763376110939276e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.1795284180591432e+00,    -2.6306793805873735e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6153654742439326e+01,     7.2681954241237681e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0333717026160336e+01,     4.0475868942379857e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.1649016808633501e+01,     2.5560249300150466e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    9.8681841230414022e+00,    -6.0912939102842347e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    8.6579064821064776e+01,     2.3920863517287056e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0071486108223018e+01,     1.6177620830626473e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.9176317826076293e+01,    -5.4484874723498251e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.9062286459134059e+00,     1.6675416533187610e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.7985144520887971e+01,    -1.0176754617760325e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.5267178620541433e+01,     4.0305016676841165e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    1.3678839080036346e+01,     1.5665553231051743e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.3156964753317988e+00,     8.0994049140198731e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.8918529859435651e-02,     7.3319555094247626e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3940060176663840e+01,     1.8169948150947435e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.4176630966182628e+01,     2.5305465544516490e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.1840650385133202e+01,     3.0732827993324900e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.0428281620249024e+01,     8.0710981101573097e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.1744328347993948e+01,    -7.1561874711436221e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.5798892082222137e+01,     1.7018107713724501e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    2.2773122209963006e+01,     2.1085388248737916e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.7157978833831450e+01,    -6.7969479289057936e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.2704198597944782e+01,     1.6535335119613082e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.1992440389831643e+01,     1.6760208698131712e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.6823661207783850e+01,     5.7304676363185172e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6905546072292466e+01,     2.0873057861757741e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.0011677781088780e+01,     8.3379061836138098e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9485642900337812e+01,     4.0580634199509099e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    3.9208029110071960e+00,     1.6754679415545994e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -4.1214488274306071e+01,     1.0379779317257324e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3470513168720416e+01,     1.6769635748684108e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.8940326846605515e+00,     6.8488232183715390e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3301009647794370e+01,    -8.1174571930845605e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4290666326146420e+01,    -7.4620022911196156e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -4.1956250431415860e+01,     2.4071392279557651e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.9158733341788796e+01,     6.3593288205007884e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -9.3934521179233137e+00,     2.5043701846416288e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6651759827727268e+00,     1.6131220164753322e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.8346935892802634e+00,     1.7178588778315829e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.7751292390994443e+01,     3.1677315632518181e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.4058567654755237e+01,     2.4193473378885243e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.9377047196141980e+01,     8.5927257650832507e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.0778874510188396e+00,     1.6291584351409940e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.6802614272384062e+00,    -8.4022877539765232e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7364546897014179e+01,     3.9479159835557279e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -4.0942761624339937e+01,    -8.4905540561551825e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.6493312478230910e+00,    -1.9151578179559237e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0078273020014038e+01,    -9.3646232128000174e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    3.0996038144972431e+00,    -6.6471884481688721e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.4532882965620111e+01,     1.2769316062532449e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.3885539343849818e+00,    -1.1216321407445405e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3685256805659223e+00,    -8.2025867837774476e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.8868120154978495e+01,     3.0011220403752774e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.1572738055324781e+01,     7.0847049686892606e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7533893892562435e+01,     2.7671143529050714e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.5091731808754108e+01,     3.9516344905139005e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    9.4739424116416942e+00,    -1.0908321158161711e+04,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    2.5762436430522655e+01,     3.5449079629640742e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    8.2817549782878093e+00,     2.3887202933858607e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -1.8859879707055196e+01,     2.5300727104849579e+04,    -3.9726088367124645e-05,     2.3468322531964382e-07,    -1.3242685699506974e-09, } , 
      {    3.1768903821087658e+01,     2.5300069840100023e+04,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {   -1.0889633462836422e+01,     1.5019550070179175e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.2658848047987519e+00,     1.6770324625148718e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.7569822734814782e+00,     1.4450432429830274e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.8806983108620805e+01,     1.0459409121053150e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1261333467529994e+01,     1.0724029120739204e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6935142498973782e+01,     1.8884996784920419e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.0580860359015386e+01,     6.7523497901937772e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -5.6587217881098191e+00,     4.0840851562515185e+04,    -1.0590926449095827e-04,     2.9680094783916534e-07,    -1.7322877639724827e-09, } , 
      {    4.5644798405948279e+01,     4.0845453962397994e+04,    -4.1049408001561064e-04,    -2.0463680047014410e-07,     7.3968023326881881e-10, } , 
      {    1.5553816571970064e+01,     1.4852331788023805e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.0405000700416906e+01,     2.6621045389820601e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4159918438753976e+01,     1.4845754599464670e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1219347342582733e+00,     5.2120592906892853e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.1213900153098528e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0222248010311903e+01,    -8.7800864613114409e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -4.6005566363231871e+00,     1.6764106466098387e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3765368166517657e+01,     8.4402023077703234e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.9176482000873058e+01,     3.9521604569771371e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    6.4779982109526788e+00,     6.8302306835806739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.2964379539558884e+01,     8.1209804167510556e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.7829174398468339e+00,     2.5152982156610946e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.8810612022444623e+01,     1.3196118459680596e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0892790256645966e+01,     4.6759112532615545e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7734447969974532e+01,     2.6236130488362302e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -5.8709976766978947e+01,    -1.1640463619469147e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.9500257825464487e+01,     4.6644495351381192e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.1835601278999484e+00,    -6.5945918018451994e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1476749927158702e+01,    -6.9727159671869358e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -9.2733187152949306e+00,    -8.6653556920352603e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.8051352105247243e+01,     1.5551969048972154e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.6160658114339164e+00,     3.8888365952468237e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {   -5.4853162354987731e+01,     2.2652049136022717e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -9.6260872960094055e+00,     5.9923583732836796e+03,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.4070730643606321e+01,     8.4329138679591179e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.7673344804117832e+01,     8.4340184470240856e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4006400385466984e+00,     3.6523964698680584e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1300164721610198e+01,     4.1026624266413281e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    8.8096134593172373e+00,     1.1225289943720747e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0260986834096791e+02,     2.2652035708921263e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    7.3668841322653362e+00,     1.5674758984913025e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.6167536175749984e+00,     2.3616980905235567e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -6.9718762470503073e+00,     1.4878364077539682e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    4.4331644515945086e+01,     1.4882966477422502e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -4.4526950606348805e+00,     1.5709662172722390e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -8.4040391839223716e-01,     9.7296566299094025e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.1058679137114193e+00,    -9.9153785368334375e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -1.7963333328513585e+01,    -1.1441522310625442e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.8111750233075526e+01,     1.5646562620448885e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.0892465444824403e+01,     1.6410823341877658e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.6633305866545030e+00,     3.9651203749462939e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.0971491847432482e+00,     2.4038353597345838e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.0504157783724054e+01,    -8.4267300072128764e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    2.5128747294576703e-01,     6.1321122061541937e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.1372533218649785e+00,    -9.8106132797041973e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    3.8336198546547591e+00,    -8.6261548765803363e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.4531551730107740e+01,     7.8151479785478605e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7092265334369195e+01,     1.4827705656008811e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0203565164315459e+02,     1.0487278444198964e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.3221644531980864e+01,     2.3805368181934693e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    4.0111974805818162e-01,    -1.0902925664116810e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.1624123978491712e+01,     4.0593788394822375e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.5507628490766994e+01,    -3.5317925941897374e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3155286352930105e+01,    -8.8534204866152868e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.0727699857349080e+00,     8.5327929523618786e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    3.6964831404541663e+01,     3.9558789639353097e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    3.6425786956370722e+01,     8.2468814079536041e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.2041456041356255e+00,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    1.9618061043349876e+01,     2.3330992181620255e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -9.4066186975370329e+01,     1.0467668129431186e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.2957009366836891e+01,     2.3962713936202043e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.9445993991343023e+01,     2.3921109112040383e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3441946601425627e+01,     1.4454903576692413e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5796612208709600e+01,     6.4472047498209740e+03,     3.1401448060125284e-05,     1.0325062422179354e-07,    -4.9873268189419070e-10, } , 
      {   -1.7101252148533270e+00,    -3.3453321858817108e+04,    -3.6195648059592815e-04,    -7.1040732864752949e-07,     3.5627987797377574e-09, } , 
      {    3.0235339244635664e+01,     3.0528199930162518e+04,    -3.1508646657900174e-04,    -9.6040959970013584e-08,     2.7037574181797698e-10, } , 
      {    2.2265052342077158e+01,     4.0809371335001437e+04,    -2.2728709493871765e-04,     8.2486151222608339e-08,    -6.7088517191937597e-10, } , 
      {    1.5366277281123011e+01,     2.2057521894713089e+04,    -9.6610882692841927e-05,     9.2560191665119606e-08,    -5.5758906278088846e-10, } , 
      {    2.2237085884565790e+01,     2.2685835109590116e+04,    -9.3947001199933423e-05,     9.1943799550956940e-08,    -5.0314933452909859e-10, } , 
      {    1.3266277447699672e+01,     1.4914446704936252e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6579227844537552e+01,     4.1923999894266541e+04,     1.4348089670976832e-04,     5.2180617701239621e-07,    -2.7348339751388921e-09, } , 
      {    2.2764077545908204e+01,     1.9794991651191667e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.4710990081824193e+01,     1.6863362888642056e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.2227727422419443e+00,    -6.0911991528951216e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -7.8060685224847823e+00,    -3.8592163915168858e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5279030952393832e+01,    -2.4988984879059376e+02,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {   -1.5412742208818280e+01,     4.2793164770940712e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -4.3748083387648272e+01,     1.0011936646330436e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1050578039735699e+01,     9.0272809696151071e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9954291313190545e+01,     1.6679674288845330e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.5509556469307434e+01,     1.7953839729611998e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.2428735440232675e+01,    -7.9694852520793957e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    8.1736382980200228e-01,    -2.2069504194166266e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.4080633029049725e+00,     7.0556076898121255e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.8566121549770820e+00,     7.2981740819696024e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.5785125384526104e+00,     8.4688032284283236e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    4.4715102767801751e+00,    -1.2477892255435149e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.2564761876489779e+01,     6.4959368805827635e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8186478284437186e+01,     1.6244658204380412e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8414658719421436e+01,     7.1629685751682619e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.6411735917939456e+00,     6.7036479243792191e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.8017289023003955e-01,    -1.8269807111073143e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    2.2528997389423012e+01,     2.5783590941597904e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.9352663566371193e+00,     2.3957032777717362e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -2.4662752866841920e+01,     6.3615358417509142e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2706360091590380e+01,     1.1655186176047626e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.9474866118956101e+01,     1.6066639458117263e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.9426705957257482e+01,     9.0678072169798961e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0942668702845927e+01,    -1.6358344883340342e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0824248019036020e+01,     1.6065575796692310e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.8313155795512557e+01,    -8.4257779513312071e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {    7.1054829286946219e+00,     1.6074331419998090e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.0716522429787579e+01,     6.7964492561255147e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4286613918185425e+01,     8.6556348616458890e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    8.0701214605167202e+00,     1.5647305365692235e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4410039775740106e+01,     1.5647752680007652e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    4.0329679092065440e+01,     1.5572277961779233e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.2497690916546187e+01,     7.7479049987912249e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0607289930153852e+01,     6.4809861772761333e+03,     4.8687074555849077e-07,     6.6846546683695823e-08,    -3.2415127532664681e-10, } , 
      {   -6.2234667477962802e-02,     8.4306126680177076e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2742823115061565e+01,     8.4363196469654940e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.5533597898865974e+01,     1.5849034210747341e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3509641732614240e+01,     8.2180166376700708e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0123730110468358e+01,    -8.7725944730050178e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -2.8235903989860720e+00,    -1.6250435002442309e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.4465112013144854e+01,    -1.0174101101860837e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.3923959321927381e+01,     7.7026508285263462e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.1594848620345616e+01,     2.2719605277382310e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.9853866431624208e+01,     4.0094313225289538e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    2.6718828220235025e+01,     4.7272327674244290e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {    3.8482750707600362e+00,    -3.1165646333262697e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.6850338635924693e+00,    -1.6638204655229780e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    5.5480120190521731e-01,     7.8025287401298738e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.9449128442279260e+01,    -9.4684649129608824e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.9065562628718276e+01,     9.1545750742834407e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2722419864997420e+01,     3.1267835573843859e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    4.8774129990193302e+01,     3.1796048992745040e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.5098391367668285e+01,     1.7928102579995746e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0646607197764007e+01,     8.3225321411587011e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1974079254851450e+01,     1.0380167459050861e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.5159966688060145e+00,     8.8498973536923786e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0284348479604706e+01,     6.5972280231225213e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3863257682409088e+01,     1.6118102253555058e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0275822376326222e+01,     1.2785048546531749e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.1434786699337046e+01,     1.5300908719030107e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.1315662828462884e+01,     4.0673418198618208e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    2.1826667217214510e+01,     1.6764424571063490e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.4057852200862824e-01,    -1.6692327738256292e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.7193513271386454e+01,     3.3693524246912108e+04,    -4.1049408001561064e-04,    -2.0463680047014410e-07,     7.3968023326881881e-10, } , 
      {    5.9137475511056810e+00,    -8.4929847345630169e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.5519921006199965e+01,     1.4595055434290110e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.4620209048101421e+01,     1.5072901880482470e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0831276393571983e+01,     1.8851215357465262e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.1104335969783783e+00,     7.1338712962911268e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.0178371303898039e+00,     4.3117406787661339e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0043776590005812e+01,     1.3630609062989139e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.2634523432535619e+01,     8.8073423028612415e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.6432752279890273e+01,     2.3989726958744748e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.3858143756341178e+01,     3.1422172847375397e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -3.2974807610655148e+01,     2.5033761108074919e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.6886219276641306e+01,    -7.6752389105055809e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.5700797164376112e+01,     2.4378057885975046e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    4.8366690031651657e+01,     8.4331708948106007e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.5110879544837431e+01,    -8.4337614201725992e+03,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    2.4483420344209463e+01,     1.6977740522294102e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    2.5863136838432364e+01,     4.1419645236106269e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.7305779814770414e+01,     1.1386118133239906e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3343158978608088e+01,     3.0641605531582016e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -3.6688766756779842e+01,     8.2742065498999509e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.7575174443971122e+01,     1.4924049605720147e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.6546808951444309e+01,     2.3559789251272618e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -8.7688674709301040e+00,     3.2542474508889359e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0966753265728375e+01,     1.8073704941245055e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3032409187034490e+01,     5.6016386035321029e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    3.1333314625077819e+01,     1.7467518755392441e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.4976327751553065e+01,     2.2235585503104154e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {    4.7759458269225865e+00,    -9.4716534733458429e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.6847712062559062e+00,     8.3995363129792277e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.0660501406879526e+00,     6.0108689085658043e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7313748170450168e+01,     1.6130721491973045e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.1213486201049848e+01,     2.3407431439499265e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.1759107301015678e+01,     8.1889718027459076e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.6363571575005054e+01,     1.5218295947293964e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.7217437632258374e+01,    -8.9042248868852766e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.1019602347976758e+01,    -1.2436761657427387e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -4.2999526159080894e+01,     1.3238371004208420e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.6141842859167589e+01,    -8.3401814124801767e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.6526299406456118e+01,    -2.1299517575412574e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -7.2726227455590973e+01,     9.3377806849442277e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7209278074098098e+00,     1.5663528531073940e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    6.7127337601935730e+00,     8.0318420591095528e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2327231770508451e+00,    -9.6463104940257253e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.6961233521136705e+01,     1.4393215052700269e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.1282646678331311e+01,     1.4923661463926612e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5475189681542824e+01,    -1.0367120547394421e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    6.5336048369099473e+01,     3.0562328043744266e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.1672100598196039e+01,     1.3277185183562051e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8696339773812522e+01,     3.0168954758093205e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    2.0098856474274818e+01,     3.0587712877402453e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.3487234290469459e+01,     2.3019326242548719e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    5.9894380380453448e+01,     6.0517474718886688e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2456401170266105e+00,     3.5720994052533069e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0527720748001698e+01,    -1.2067840537152674e+03,    -9.2640263880542614e-05,    -1.1044778028060079e-07,     5.0823018190712641e-10, } , 
      {    4.0832319497297071e+01,     8.7174546386587172e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2144702158066391e+01,     6.0218895352881691e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.1040872217200572e+01,     2.2719957593340383e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.0809945427479075e+00,     6.4372097138876879e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4672944635084551e+01,     1.6233176791939784e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.6033432150315718e+01,     7.6363904110087306e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.5883760898364777e+01,     1.6823547986744146e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9268223514164735e+01,     2.3924405336077525e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -5.4903183978507315e+01,     9.9721217444325230e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.2310107807557628e+01,     7.7531377458241677e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6264977035341232e+01,     2.5907543868483172e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -5.9256046200642352e+01,     1.0005903171887683e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.9692695694219093e+00,     1.0438148660570870e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7938762855504894e+01,     1.0516797913059280e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.0620934585336787e+01,     1.5507843611965372e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2903894601973013e+01,     1.6990894717607385e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.2886463410763337e+01,     1.7250745545283051e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.3725101022098354e+00,    -9.9673591519868751e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.5442524114552558e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1955781331471353e+00,    -1.0693376198380509e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.1605305003089462e+01,     2.5106581490737797e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    1.5293183215421314e+01,     8.9088912557154163e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9008082432865010e+01,     9.0214333594828495e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4533039857035128e+01,     2.2784039627196162e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -1.6256044190096429e+01,     4.6171307516456727e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0393017632217543e+01,     1.2013543486773942e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.7714391243637664e+00,    -9.0257253261335463e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.7421205126398934e+01,     1.1342347299682944e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.1677343407378453e+00,     6.8090083164736261e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1584262889586505e+01,     1.6733307186917486e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.5946196912673720e+00,     2.1775828983655905e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0601373436603842e+01,    -7.4883006142814520e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.7144388308225889e+00,     9.6164849982296146e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.2786096407826236e+01,     5.6778713265190290e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6969794071304620e+00,    -1.0569423271495238e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    3.3521811712533392e+01,    -1.8084034407175044e+04,    -1.7874949551903517e-04,    -4.2328437695477703e-07,     2.1522333745495625e-09, } , 
      {    1.5771102984498455e+01,     8.4702259193620757e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -7.0552397772275697e+01,    -1.6762363297341071e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.2106857976150865e+01,     1.6960368056572211e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1453979357859222e+01,     1.6570641800319459e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.4663130972473567e+01,     2.2731672226267816e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    3.5828327037193183e+01,     6.6342066805935474e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7237876883499631e+01,     1.2800467657671426e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.1006841508906895e+00,     3.1353076020033550e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7494044837194977e+01,     2.3783531728384660e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    5.8845089493459160e+00,     1.5663276766364397e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1780673404890209e+01,    -1.8814622944713810e+03,    -1.2098889206578503e-04,    -1.4742461670156133e-07,     7.3797853724812435e-10, } , 
      {    1.2390226570297312e+01,     2.5232884378226678e+04,    -1.3038520622748551e-04,     5.6859772703784265e-08,    -4.3599294342180572e-10, } , 
      {    3.4161867123171410e+00,    -4.8977017522184633e+02,    -2.4445133659688390e-04,    -2.9312819331655198e-07,     1.4348497224749714e-09, } , 
      {    9.7948749985963310e+00,     7.8423948307349847e+03,    -2.1857351624974395e-04,    -1.8855751840383332e-07,     8.2869202512052961e-10, } , 
      {    1.2009256132636395e+01,     4.1781988903109464e+04,     5.1434682072183842e-04,     9.6108256957088451e-07,    -4.7995099988800728e-09, } , 
      {    1.3943126208233041e+01,     2.4043734541721518e+04,    -1.9182263140228258e-04,    -1.6122915297610619e-08,    -8.9739012373375128e-11, } , 
      {    1.6405508440670477e+01,     1.6032524465203642e+04,     1.7826816047305033e-04,     3.5524591583607347e-07,    -1.8428689164967569e-09, } , 
      {    1.3282653505018711e+01,     2.4567291774230009e+04,     2.4378009372943078e-05,     2.3998480836668091e-07,    -1.2955676000290129e-09, } , 
      {    8.8946106845884998e+00,    -6.9584605962476780e+02,     6.1731222265549453e-05,     7.2851788307495308e-08,    -3.4843559261342347e-10, } , 
      {    1.6432857757854730e+01,     2.3280404398011207e+04,    -6.5892170105443388e-05,     1.2905153566581704e-07,    -7.3071602830510365e-10, } , 
      {    8.3748917458104568e+00,    -1.7801911530991893e+03,    -2.1343882691873302e-04,    -2.5676774900975404e-07,     1.2595410953857631e-09, } , 
      {    1.8728699830800061e+01,     1.4390865092096661e+04,    -2.8228588453547515e-04,    -1.9403363437606989e-07,     7.9708217311218790e-10, } , 
      {    2.6890677718696292e+00,    -6.6556259383408542e+02,     1.5732916472975297e-04,     1.8255227678003383e-07,    -8.0440770783375299e-10, } , 
      {   -5.2498850091409803e+00,    -1.6066341532652863e+04,    -1.5258341378071642e-04,    -3.1765268730095022e-07,     1.5586808329039689e-09, } , 
      {    2.4836211072057843e+01,     3.9343182318842955e+04,     2.0540798370248604e-04,     5.9457069885729189e-07,    -3.0847240087956444e-09, } , 
      {    4.2977154027820443e+00,    -4.8629659388846630e+02,    -3.7086592401207015e-04,    -4.3927639255848804e-07,     2.0646760237411807e-09, } , 
      {    1.1587578290097403e+01,     1.5715043117098066e+04,    -3.4411503916460877e-04,    -2.6684178945226530e-07,     1.1462449862472756e-09, } , 
      {    2.2523782558593421e+01,     3.0386188939005438e+04,     5.5779457433068375e-05,     3.4323543258847444e-07,    -1.7943002819232039e-09, } , 
      {    1.7537744963076673e+01,     2.4428729984075220e+04,     2.6892727833341119e-04,     5.3306936845193303e-07,    -2.7311445430256485e-09, } , 
      {    4.7421311265201327e+00,     1.1855862246840334e+03,     1.8015416520201008e-04,     2.2084916389191940e-07,    -1.1415810786350018e-09, } , 
      {    1.2169974822669669e+01,     1.4215132693809584e+04,     1.2462651504981052e-04,     2.8050131795479032e-07,    -1.3318413596496283e-09, } , 
      {    4.9739349806042954e+00,    -8.8860357144185837e+03,    -3.4024235271589359e-04,    -4.6980612816668585e-07,     2.2127914514569548e-09, } , 
      {    1.7612501816466031e+00,    -2.6381269984615847e+04,    -2.0956614047001784e-04,    -4.5973208772417463e-07,     2.3260875605954423e-09, } , 
      {    2.6301725201249022e+00,     7.8389212494016028e+03,    -9.2158928834557778e-05,    -4.2409319161897217e-08,     1.9886572385432054e-10, } , 
      {    1.4061736904478973e+01,     3.2757481429023825e+04,     4.2635158435438611e-04,     7.8264272484086211e-07,    -3.8567946440993916e-09, } , 
      {    1.2012634094115512e+01,    -2.0234976659595550e+03,     2.4997496430986924e-04,     2.9180814262562685e-07,    -1.3274247070147201e-09, } , 
      {    5.1312983984387293e+00,     7.3525946453505603e+03,    -4.6559080197595222e-04,    -4.8111295283752247e-07,     2.2083747988220470e-09, } , 
      {    7.8687724184786756e+00,    -1.4289527578695561e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {    4.8837157302213345e+00,     2.5300398472474801e+04,    -1.9201849612945083e-04,    -1.6035648835010902e-08,    -8.8284571330046551e-11, } , 
      {    9.3907979552461072e+00,     8.3040558168695461e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3708870815028227e+01,     2.5043949668085075e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0688682961809928e+01,     8.4320352647366562e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4328724749957065e+00,    -8.4348970502465472e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -1.9355700632737555e+01,    -3.4269774185456185e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.4173325940336063e+01,     7.1082244710652431e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1496744614683550e+01,    -1.1910173587229979e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.4854428654829242e+01,     7.1103517939151452e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5673078719505877e+01,     2.3215915657703674e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    7.9335300718454640e+00,     1.3997892883065872e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.2649519748584986e+01,     1.6762550828377494e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -5.2578431353084767e+00,     1.6761764335852607e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.3997272198085476e+01,     4.0007596911616172e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3271824445740513e+01,     9.4973926084939540e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9649765416343314e+01,     2.3053666520854036e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.9738765697620899e+01,     1.9735473074120251e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.1365284500688446e+00,     6.1681606225577907e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -6.0683115629321538e+00,    -1.8537443516215515e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.4842277744092378e+00,     1.6056297736057382e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.2151801847903741e+01,     7.7376024310864650e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9320449847674389e+01,     1.5753448265794508e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.9817735492742884e+01,     8.3239548320924532e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.0303631305356109e+01,     2.2688090838875585e+04,    -4.3912817851966964e-04,    -3.0973936447240789e-07,     1.2773386056192941e-09, } , 
      {    4.3691052898967804e+01,     8.2130999804984440e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.1838669284875198e+00,    -9.7602740148091889e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.1876505698408133e+01,     1.0240836651616106e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0048767008975524e+01,    -9.7178292805950932e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    2.7531154634036515e+01,     1.4721801672579400e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1230905401571540e+01,     1.6086950658416079e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -8.6162154909910186e+01,     8.6696581980007722e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2266222479574729e+01,    -9.7550143501768216e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -5.3113278867468971e+01,     2.4031577377659734e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.6280571729078346e+01,     2.0598003939515684e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.1677412379046046e+01,     5.9969607731665019e+03,    -5.2523741015816222e-04,    -6.2257596114658415e-07,     2.9213417982617303e-09, } , 
      {    1.8216132118111796e+00,    -1.1327079277470953e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -3.4156891013394983e+01,    -4.5610897310554674e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2388941196215253e+01,     2.2003914638174929e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4359053212357836e+01,     3.8315328664575296e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -1.3807103311733542e+01,     1.5946842436465449e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.5257985715811543e+01,     2.3412347212241704e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -1.1180840673688515e+00,     1.3617989824571150e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.7781714343409714e+01,    -1.8088636807057865e+04,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    4.2497513996805560e+00,     1.3604871913372885e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.7325396473614587e+00,    -2.5097407962337635e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    4.4399511994252450e-01,    -9.7974590843909191e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    6.9919766085499461e+01,     2.3252741030343597e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.2122435955135288e+01,    -8.0316287189899513e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    7.0394872193973317e+00,     7.4559084363100337e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.3454481655205859e+00,     8.4345771750928616e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.0716154947519350e+00,     2.3416604967899431e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -3.1712429790258138e+01,     7.8725824515179029e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8569851252362852e+01,    -1.0250774391971223e+02,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.7806432139787166e+01,     2.9411298752117487e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    5.2826890345524049e+00,     1.3346175071512538e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7126177762876615e-01,     8.2550950952703770e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.9782241922467676e+01,     1.6371437812363471e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8140376910875908e+01,     7.1071458942683739e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.1540801720642673e+01,     3.4205582370747929e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    6.2738017726406401e+00,    -1.6087449331196360e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    9.6990115720040436e+01,    -1.1951304185237741e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1790342439698488e+01,     8.4636744539699957e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6430628906887645e+01,     2.2791762044349533e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.1426711099118858e+01,     3.3416910599045768e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.5387158751643767e+01,     3.3422170263678134e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {   -2.8626186770764239e+01,     1.5488269415258295e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.0054041807187648e+01,    -6.9696027216056227e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.5186740517298174e+01,     2.4664046609590350e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.0082753455907952e+01,     2.3947834514063135e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.8839389268539453e+00,     1.3584208397115992e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.7800024308162278e+01,     7.1090824172641751e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -7.5338081017974474e+01,    -7.1094938477162141e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    6.4766387728715999e+00,     1.3293205974301347e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.1907373804141471e+01,     3.3419869063736725e+04,     5.4516346567282114e-04,     9.9753028034028185e-07,    -4.9733641849259522e-09, } , 
      {    3.8721289554045896e+01,     3.3419211798987177e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.3643550691815907e+01,     2.3252352888550064e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.9421969282211705e+01,     4.0630973464404109e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.4402276497751844e+01,     3.3472509528573144e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    7.3961920109867059e+00,    -1.2232060617425818e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.4256669931351915e+00,     7.8077884047622410e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.8889634513323088e+00,     1.5001197229602603e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.2791884704329565e+01,     9.6104515237868618e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.1846768284661886e+00,     1.3447137559032451e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.6162242925731682e+00,     1.5569532135957519e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.4865544581272179e-01,     3.8875681139341696e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3474529829859655e+01,     4.0636233129036475e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    3.7910483323913155e+01,     2.2397610982796090e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.9523523005835749e+01,     1.3642074315591806e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -2.5400898352910936e+01,     9.6606773332483863e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5106021116343195e+01,     2.5649266813765262e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.7014756676209615e+00,     7.3571215022987872e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.9685400061439669e+01,    -9.5477660843819340e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5642293783675051e+01,     1.7723967559782166e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.9917313754648177e+01,     3.3248688165823260e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    4.5231900875713066e+01,     2.3421207367782252e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.6868316667001579e+01,     2.5582085753700503e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.4855264589560718e+01,    -1.2156005225239310e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.2865689716978679e-01,     3.2853589653346564e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.5951663381062483e+01,     1.4393240775867322e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.4275990690329252e+02,     8.4350880628077775e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.3950409641647832e+02,    -8.4318442521754259e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.8120745197834800e+01,     1.6713706181834306e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9643117606717269e+01,     2.1657945815328396e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -2.3365610418450373e+01,     8.5862694795744392e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.7562528529047000e+01,     1.6935624420454780e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    9.1016700537198787e+00,     6.5442589259113292e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3125084866281679e+01,     1.8240941456360426e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0425248826343923e+01,     2.3975996790315687e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.6766021510686510e+01,     2.3976444104631104e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.2339592215851837e+01,     1.4390561536800784e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -4.2562472263897089e+00,    -2.0180644699372187e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0076960031937494e+01,     2.3992219955697386e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    2.0131959030389591e+01,     8.8571400495432863e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.8035870379098807e+01,     8.3998331470936537e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.7022787655752904e+01,     2.4604175716560865e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.9288533870160975e+01,     2.3914087297600949e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3621831196079377e+01,     1.6066287142159186e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    8.9804660101203471e+00,     3.2633498491975974e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {   -7.5421789346137643e+01,     2.3951947573863916e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.9461359319071960e+01,     1.1667805414465613e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.0486945529472358e+00,    -8.9973393927232864e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.0670518003187265e+01,     3.9482926321051593e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -3.2730250956083395e+01,     8.0050619698553965e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.4260292899085627e+01,     1.1388747965556093e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6461830914895167e+01,     1.7690186132327006e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    7.4789107391068086e+00,     1.4577782437524444e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    3.0943067397238256e+01,    -9.5484233491314772e+03,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -1.3367944287342191e+00,    -1.7363771887115151e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.5674755141441032e+01,     9.5030737669786286e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.2463501710343525e+00,    -8.2319609695942072e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.0887036079788004e+01,     8.7440038339148687e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.9938730634764799e+00,     7.7706657995240203e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5465657313324405e+01,     2.2720316622849186e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4129969711983208e+01,     1.2048207145740613e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0186089935100091e+01,    -2.4120287382907873e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.3136181677362977e+00,     1.1921733300701753e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3583572293724808e+00,     6.2114916912447121e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.1374799560623785e+01,    -1.5322613314889713e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.0658240648770047e+01,     6.9102136230006327e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.5013254015529885e+01,     7.6643012121813372e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.4933601767045857e+01,    -6.2227411060279132e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.4286797129424595e+00,    -3.4492772922036721e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8928748455141449e+01,     4.0082847972686868e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    2.7830137860341914e+01,     2.4198389151627682e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    9.6843100674176341e+00,     1.6832439542061165e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -4.0660059661750452e+01,     1.7627623610070827e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.2825609974723434e+01,     1.5700488644322226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7621906501460511e+00,     6.0104073167395381e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.2494957801612134e+00,    -8.4512480767227626e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    4.2572497562043040e+01,     8.3773932904380072e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0050167107459288e+01,    -1.2325212033315202e+03,    -5.2523741015816222e-04,    -6.2257596114658415e-07,     2.9213417982617303e-09, } , 
      {    7.5507535551562688e+00,     1.6570082949469303e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.1951393340370675e+01,     3.1361135578370442e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    5.1608546123004601e+01,     2.4462893737651029e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.3456046048752086e+01,     1.5018892805429630e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.6180270938041268e+01,     6.0122915994995574e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.0345782883126020e+00,    -1.0280013573116166e+04,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.3905231453670076e+01,     2.5160678850597258e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.2762053904159167e+01,     2.5004098482072146e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    8.8830535539450910e+00,     8.3195026367081337e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2658630879200219e+00,     5.7534796357326218e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.6135124647772408e+01,     8.5021582629631812e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.9220262129795174e+01,    -5.0044978731406864e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -9.9585608787909354e+00,     8.0595900121219602e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.3271661498675797e+00,     2.2230983103221333e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.6974976002484432e+01,     2.3401593305105918e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.9447208527520683e+00,     1.1541551287638929e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.6046174900566314e+01,     8.2806628354087625e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6575159273778873e+01,     2.5659638014712629e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    7.4596055316229153e+00,    -1.0175690956335373e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.0033822157398442e+01,     1.6780191266055761e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    2.8957504892915715e+01,     2.8064164498743703e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5816778107997962e+00,    -1.6838598825525914e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.2342125569078922e+01,     3.3366571334150758e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.7022308956892573e+01,     2.4561868216563234e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.3002864704725805e+01,     1.5996397364140405e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.1069025251401456e+01,     3.0264555962561197e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.1044903562752815e+00,     2.4147072713012094e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.8937921563352418e+01,     1.4576686556656956e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5457607617260857e+01,    -5.7161007970958490e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.7092928607156701e+01,     9.1015886444350563e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -8.2452708321220705e+00,     3.2093061206419130e+04,     6.0699262030195476e-04,     1.0703384354164773e-06,    -5.3225269980610395e-09, } , 
      {    1.5644707592229640e+01,     1.4224024202978260e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -5.0442294498091762e+00,     1.9259968811157545e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.4125597265344965e+01,     1.6118307968781077e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    9.3818858368095334e+01,     3.2289369843808417e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    8.1516951331429759e+00,    -9.2459809970928723e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -1.1344545678958358e+01,     4.1955480121780289e+04,     2.6485872715752774e-04,     7.3612097362895329e-07,    -3.7962365671919993e-09, } , 
      {    9.0987624495251023e+00,     7.2497190998346132e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2102604666934603e+01,     1.5646976733317460e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.0848799400358075e+01,     2.3353946336870606e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.4794276621693619e+01,    -1.3149632660110469e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -9.6227647719173266e+00,     1.5648081312382430e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.3369482301404624e+01,    -8.1824470581397472e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0351013007624957e+02,     3.1190276599281009e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -5.3952905176507457e+01,     3.1190290026382463e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    6.0579394168211635e+00,     2.5718445246317166e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.9952833539273556e+01,     1.0961945568407944e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -4.7453859385896735e+01,     1.6616513749724436e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9915744844147000e+01,     1.9761210223736507e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -9.7229328909402781e+00,     2.0000155264813566e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.7359711835297902e+00,    -1.7659533209968307e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    4.9302699858951200e+01,    -1.0176049985844174e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.9882964767095991e+01,     1.0419594219155237e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5539792123351193e+01,     1.4879501815926860e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.6413313281774997e+01,     1.6761605292582568e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.5304858381140805e+01,     1.6762709871647534e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3743259662260146e+01,     1.7144807350860669e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.7185354714296956e+01,     3.5842100599333731e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -5.6150508258207044e-01,    -2.5999713891096144e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.4618259569589434e+00,     2.4403022844621544e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -4.2037687398944641e+01,     9.1967604765193100e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5160889321790227e+00,    -1.1120178595528658e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    8.9516880791980782e+00,    -8.2363682112326096e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.5998183706713167e+00,     1.4743454882385439e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.2322509786195397e-01,     3.1790784355603191e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.4783955671240800e+00,     1.5071479189548718e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.6463642069342317e+00,     2.3326582954450001e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.6657483145054135e+00,    -1.9027463117048857e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -6.8316592051311375e+01,     9.3039992574890675e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9027187742411613e+01,     2.2826254817271569e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.3124182147701487e+02,     8.4368022610560511e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.7146781639648054e+00,     1.7217311701680483e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1620952586670832e+01,     7.6309461776464777e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.6289927655944069e+01,     7.6689036120641576e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.6755301588148114e+01,     1.8364894383245697e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.1314291005806687e+01,     7.4831642778598334e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7759635624953201e+00,    -9.6179245606154655e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -8.7423484266424212e-01,    -1.6159587146685757e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    1.7293217106954369e+01,     1.4724975096230586e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.4660897692393274e+01,     3.1676604287051305e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    9.6546157703252273e+00,     1.5994130375191966e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2268423333808922e+01,     3.2897171040738795e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.6654470391665377e+01,     2.0360981538779279e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.3269249211911660e+00,     7.6911950516625893e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9296220409322189e+01,     8.9438810984707270e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6480865850803482e+01,     2.8685407591370495e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.2610960822222310e+00,     1.5288289480612117e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    3.4148649643762354e+01,     2.1542843568244756e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -5.2055282594742849e+00,    -8.5070624868446739e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.7105549040207439e+01,     2.3960220939249397e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.3567598798856906e+01,     8.1229284810683330e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.5721569082390095e+01,    -5.8995723010525362e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.7108339909379556e+01,     2.4004606380883655e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.0426524981410971e+01,     8.4307950945852226e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.3682335468225197e+01,    -8.4361372203979790e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0445295616445812e+01,     8.1240577316295921e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.9958979519351900e+01,     4.1960082521663106e+04,    -3.9726088367124645e-05,     2.3468322531964382e-07,    -1.3242685699506974e-09, } , 
      {    2.2107524482797761e+01,     2.4607157864835124e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.0683479495413465e+01,     1.6332924506559069e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.0440329025650129e+01,     9.9660882699897720e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5584968101909936e+01,     3.9454540387641326e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    2.5467199063321431e+01,     2.9830056871426739e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.2684342152526099e+01,     1.1117050090432193e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3811477694256812e+01,     2.5934124335104807e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6458392228171860e+01,     2.3993493444239062e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    5.0778362417943477e+00,     4.3581413446392808e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.7852638763842346e-01,    -1.6945629086071734e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -1.1685754566791132e+02,     1.0442913046080241e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2482702501472131e+02,     1.0512033527549909e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.0585864549912429e+00,     3.1110653509035910e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.1365754536731536e+01,    -1.7940304345007319e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.7221590927735448e-01,     6.1191111728906399e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7611591874559885e+01,     7.3909754137158980e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.0265417936555763e+00,    -3.5207572434706635e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6451049427325728e+01,     1.5038409107560434e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.2966846394860411e+01,     1.6726074954718479e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.9801111901407005e+01,     1.8380642126760147e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.7000043623719847e+00,     5.1625868309309335e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5488709787082218e+01,     1.6275830978064005e+04,    -2.8514563075555195e-04,    -1.9332997579930752e-07,     7.4409688590372669e-10, } , 
      {    1.2232960386791708e+01,    -5.9110133691919600e+02,    -1.5713330000258471e-04,    -1.8263954324263358e-07,     8.0295326679042444e-10, } , 
      {    1.6554049852608552e+00,    -2.5054800590334904e+04,    -6.1134664179665469e-04,    -9.3541255782076041e-07,     4.5653449909041671e-09, } , 
      {    7.4451998539396609e+00,    -1.4088308343177511e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.1939991342058804e+01,     2.2878864884973958e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -6.1919460665360049e+01,     3.2320453900385277e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.0715750493666867e+01,     1.5743088996727736e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.9528727152224494e+00,    -1.0039331463718712e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.3223735944878278e+01,     3.2586599153322550e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -5.7648349117468840e+01,     1.6625439879655645e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -9.2242140759175975e+00,    -4.2522373612098736e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.4065291969195634e+01,     2.4587607351711344e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.3044759255248546e+00,    -9.3216245702128126e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.6765768054041025e+01,     2.5084948952653271e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.3251116771682567e+00,    -8.3106672164216525e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.5042438847837936e+01,     8.2668716476106274e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.8298249334652155e+01,    -8.6000606673725742e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0977075931427738e+01,     4.8609032783639777e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.2107529002630200e+00,     8.1348392439804838e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.1949570230080937e+00,     1.5604869206894242e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0261547160557622e+02,     2.4000493321082875e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.9099515003779000e+01,    -3.2432886422140837e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.7884662493886498e-01,    -3.2620604016103180e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.2754057706863065e+01,     3.8104625213023668e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    3.2182387612086378e+00,    -3.5737263406818552e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.2203254118678039e+01,     1.5263177921233566e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    6.8071524033962323e+00,     1.6151883681010217e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.6788979288172563e+01,    -1.2002156223772245e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    8.2709361028842547e+00,     1.5174325959828650e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.3097584946026593e+01,     8.4269405715862413e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.4321169400479690e+01,    -1.9428827092630536e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6023532934075213e+01,     8.3155735134251827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6735294924134188e-01,     1.6713419821521777e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.3058213695588350e+01,     3.2097663606301958e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -3.4057799718561199e+01,    -1.7877138457338227e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.2785101007350159e+01,     7.1273123406919940e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.3755607376187747e+01,     2.5731775830202030e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    9.9226162508122275e+00,    -7.9940445219254671e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.7202892489982638e+01,     7.8172255213315611e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.3018894882306569e+00,     4.0002994511733341e+04,     3.2668788178666131e-04,     8.0892912870514867e-07,    -4.1453993803270866e-09, } , 
      {    4.6887182618243500e+01,    -1.7876433825422082e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    6.1009067739835334e+00,     7.2482964089008601e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7577821294717168e+01,    -1.0346136865640638e+04,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    7.5227025751057521e+00,    -8.7015315599723226e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.7504777899421839e+01,     8.0891470536131746e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.6634217022113971e+00,     8.4363110715493913e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.9289174948425112e+00,    -3.4197947559507502e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.6991147996934625e+01,     6.9622344496225232e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.4677861521344426e+00,    -1.7127946604213095e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -1.0734180558826314e+01,     1.7687884932385597e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3060696974490451e+01,     8.4306212434338086e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.0260418270630005e+00,    -1.9432708510565901e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.6062934918223853e+01,     8.4157458464105130e+03,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -3.9318745405038065e+01,    -8.4511864685726887e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.8435056713460778e+00,     2.2824832126337817e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1891739765453281e+01,     7.7648181893917636e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0264249094853604e+01,     6.3399603390197317e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8270590714280157e+00,     6.0077774844233554e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2571370167423634e+01,     1.6482718338359966e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.4146043823062854e+01,     4.1812666205799258e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.2341813453563947e+01,     3.3261806077021516e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    4.6808882018241516e+00,    -2.8690394319173301e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.3316133392356548e+01,    -1.6713455716300494e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {   -4.2908487011697446e+01,     7.5723026355013353e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.3107227465094759e+01,    -1.6486363676892379e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.1305941585598394e+00,    -1.0017495010168675e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    9.7376706591888563e+01,     6.5052534210541326e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.7806594999137445e+01,     1.6314685107392341e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6964878248238527e+01,     1.4322639652381664e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.1826986256677653e+01,     2.2690835692890714e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6575376935867826e+01,     1.6134178629444279e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.3389438320374929e+01,     1.5083211161737958e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.9368751308753769e+00,     5.6036972274876902e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.2315097409639236e+01,     5.0689322229545432e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8130399709391309e+01,     1.6694594727204731e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.8706495023710730e+01,     1.4963864507618060e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.5368645054710917e+01,    -2.1380722139402496e+02,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.8718571847483702e+01,     2.3546987371917414e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.4943249572864310e+01,     8.4007893217657584e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.7960210740590759e+00,     1.2036555486188056e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7318611540295894e+01,     7.4685276747280213e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.8181552009171185e+01,     2.1444627790975741e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -4.2639538890257658e+00,    -1.5977269628544394e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    2.5063188717163783e+01,     4.7061624222692662e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {   -1.6753781131889916e+01,     1.7237239034011698e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.5585254211917402e+01,     1.5238775569157659e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1232999549645289e+01,     7.1004279769496507e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.1912648913047274e+01,     7.1181482880307367e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5840666291545888e+01,     1.4304163328092509e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.2886819415431541e+00,    -2.9684936551389774e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8016642461934538e+01,     2.7448974824896424e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.4106249439016469e-01,     1.6133138651602632e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.3636546897407200e+01,    -1.2171020921918462e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -3.5049895374668900e+01,     1.7219861193999623e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.1700837801071602e+00,    -1.5373545207299783e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -1.6052602359785602e+01,     1.5318534851419907e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.8221418184420042e+00,     3.1636789385153392e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    6.1863071247121351e+00,     5.2883551012871194e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2930357473727643e+01,     3.2664151414334672e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    9.4590554531095670e+00,     1.4336468909046805e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.5289680803240707e+01,     1.5008912156548915e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.2804666812617338e+00,     1.4317317523405787e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    6.1576326255657303e+00,     8.0170349612908230e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.9360040272741365e+01,    -8.3647740520200205e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3011389552175231e+01,     2.3972417677864069e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -4.4363851208846810e+01,     8.6806905189586041e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -9.2282251261076489e+00,     7.0826274259055608e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -4.4328391841262449e-01,    -8.3298548861776817e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.5582829668557736e+01,     3.1989443163415985e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3057045238978681e+01,     6.4920011741670141e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.6067012984794999e+01,     1.4324062343315416e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.3945383863066345e+00,    -8.4400775570437054e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.6674596158972502e+01,     1.4497563392648117e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.5395766335902774e+00,     1.3224968854878163e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9673796129818236e+01,     9.5813774714315878e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.5636371423808065e+01,     8.8626992330475823e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0462836020904660e+01,     2.3980023217082722e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.9156103401693612e+00,    -9.4367610282894238e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.1267500870130171e+01,     1.6804817398070754e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.8139718684953898e+01,     9.7575052066545340e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.0722348743915333e+01,     2.4581986127386692e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.0402186407409197e+01,     2.3880660473595606e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.8855097676480952e+01,     8.4327502206243498e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.2110908163295164e+01,    -8.4341820943588518e+03,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    5.1962057626201821e+00,     2.3223959935542578e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    8.5141125789470156e+00,     1.5450431076590954e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5425104715151585e+00,     1.4561766118297557e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.9006802088632444e+01,     8.4361976082273486e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.2262612575446671e+01,    -8.4307347067558549e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.9274058539669703e+00,     5.8128580605011948e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.4283367286079461e+01,    -2.5075116522739205e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.6471117796724020e+01,     8.8456653212017100e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.1434191254644404e+01,     3.3442082336494190e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    7.6435831324953654e+00,    -8.4054690385241302e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -1.6346085347824772e+01,    -7.5077388072210560e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.7994059511306393e+00,     1.7316868837807524e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1214748072859398e+02,     1.6761811979707702e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2953917827695119e+02,     1.6762503184522400e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.7191408895572479e+01,    -7.3995183643350811e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    8.3035481545016321e-01,    -1.1274110180259762e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.2977995847924785e+01,     8.5224836711063908e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.5790267848221685e+01,     3.2159796756174404e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.5574921683728263e+01,     3.2134059606558145e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    5.3148432646658677e+00,    -8.4672532147782440e+03,     9.2256861198141885e-05,     4.2365685930597359e-08,    -1.9959294437598484e-10, } , 
      {    1.8724514144350898e+01,     7.1737224823040524e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    2.1836226848298907e+01,     1.0611702650125444e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4053757290595389e+01,     2.1506760940848188e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.7920563426880609e+01,     1.6065934826201114e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.5955417763409159e+01,     1.3162337032225436e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.8953965697102284e+01,     1.5114822642272386e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -7.0164276350613335e+00,    -9.7636776569357608e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.9197695796526855e+00,     8.1063457892581800e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8845209216808961e+01,     1.1735368269375933e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3903364646900506e+01,     3.1834338641391736e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.9787980487993995e+01,     1.6278466440211672e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.4529929632950854e+01,     7.1094365495474076e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7110175129254763e+01,     2.3753553070538830e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.4754987183998974e+01,     1.5631529514625949e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4112019705272605e+00,    -5.4132558765423369e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.9457562487050033e+01,     3.3402267434596288e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    6.1530240324661598e+00,     7.1091397154329798e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4195552611536879e+01,     2.1478409218430901e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.5886208578323156e+01,     1.7396498641603346e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.4922997272481684e+00,    -1.6757748354944793e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    3.0230650996571178e+01,     1.3589139429373583e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2261043486116073e+01,     1.9244236327158251e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0039305280739923e+01,     4.6400665873147773e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0974148940920387e+01,     3.1596422193722996e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    6.1226856824830271e+00,    -5.9180705255458952e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.8663856407533409e+01,     1.3193488627364415e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4887277516712185e+00,     1.4496140701714365e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.1027600506713403e+01,    -8.4409391943202045e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -6.4667564123186381e+00,     1.8866947841464556e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.2690040856373468e+00,     1.4997891529826551e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.9857351315162298e+01,     1.3740091413772243e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3114875538014312e+01,     1.4391625198225734e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.9389453924001174e+01,     2.5219717706366213e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -1.3990115253208383e+01,     8.2095261740239403e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.4722911653538961e+00,    -1.2172610539114355e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8359754783655781e+01,     1.9278017754613407e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.1608554423396278e+01,     9.6380605009607953e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3163281676311597e+01,    -6.7631665014506343e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.5556160138697384e+01,     1.2441228115433600e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2910938602822638e+01,     9.1656821669885176e+01,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    2.1973812421340984e+01,     3.1659683606243703e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    8.9406468798079608e+00,     1.6187530439747576e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9345822429945152e+01,     2.4289040372360254e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -3.4236792166661090e+00,     3.3705942173559110e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8463145378134001e+01,     1.5025254912247156e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.6294763077237358e+00,    -1.8657350853805985e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7811940640190778e+00,     1.2974001790256816e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -4.0664335263957652e+00,     8.4320434665578468e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0463836666799782e+01,     8.4348888484253548e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.7993561830992931e+01,     1.6276188923853564e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.9911243198602918e+01,     7.1359488390748265e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1171847106521451e+01,     3.1679234119367487e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -7.5025672107089918e+01,     8.9699380140173398e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.2014324171134918e+00,     1.6760994120560819e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.0892514416173817e+00,     2.3967583949090560e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -3.1888220414571254e+01,    -1.3786628997249054e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.2412755884864250e+01,     1.6762512185564869e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -2.0379675472452082e+01,    -1.6761802978665237e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.4510081478651532e+01,     9.0897708615046140e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8044031847155175e+01,     2.1447242363776768e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    3.3082397374997491e+01,     1.5003488953805103e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.6475318964917598e+00,     8.3673341953362251e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8566155723071798e+01,     1.8524221462021706e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.5595355033363617e+01,    -2.4763707781824178e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -6.1875060902537293e-01,     6.7838300177075271e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1409230246294801e+01,     2.2045064745404299e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.3090437200735469e+01,     9.3591935077621456e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.1048237918786924e+01,     2.3292555932241507e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.1783232985555507e+01,     1.5362728496042932e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.3121541903821637e+01,    -1.0178703501743661e+04,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7912436894541113e+01,     2.3208193240550310e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.1316187893061670e+01,    -9.5999098281525748e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    2.8909842595205831e+01,     1.3106550729655366e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.3180826028225507e+01,     8.2325381734380494e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.1404783893580834e+00,     1.6719497766159348e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.5240267033199189e+01,     1.6919686221229458e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0367963061161268e+02,    -9.5479966651828672e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -9.2422187728896148e+01,    -9.5481927683305457e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3367945508092156e+01,     1.5584250644383266e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.0905429494303107e+01,    -2.4149243532755554e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.9816846035635507e-01,     2.7860574658673304e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6936699567013456e+01,     1.9585458928658036e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.4991323704342882e+01,     1.4880312961523015e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.7508038769425383e+01,     3.3425573905804704e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {   -1.1145342249750538e+01,     1.6069945567883678e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.1734532071458290e+01,     8.6358767705456121e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.6067898422149341e+01,     1.4881017593439168e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.0119115433825369e+01,     1.4843453399523261e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1689407631542085e+01,     9.1055445760941075e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0700136167826210e+01,     6.4748577906266291e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    1.9031997497680777e+01,     2.5313017710892789e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.9596537131039199e+01,     1.4550173460350508e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.0808133724290109e+01,     2.1113492149486410e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4405651165682272e+01,     8.4057182044791916e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.7916352991711451e+00,     1.4323679630223312e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    4.6837228065605530e+01,     1.4323022365473767e+04,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    2.5105960977081487e+00,     2.3754051743319105e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.3045638612175857e+01,     9.2947591579482887e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1280923451511359e+01,     1.6921064873748626e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -4.6917946757142097e+00,     1.4321273455064838e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.0534508232542956e+01,     2.7422866323103681e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.8337566430846071e+00,     2.3400170614172170e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4452534985642735e+01,     3.3577274785702386e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.0340248244214422e+01,     2.2939144149220414e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    6.4065672238102165e+00,     6.5574217908555147e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4558390206040574e+01,     1.2360650188065281e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0869118882297524e+01,     1.3853950704436571e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -2.1126125949015471e+00,     1.5032339349002668e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.8320306818481725e+01,     1.7707253413866652e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.3328439168971173e+01,     1.6971354731893276e+04,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {   -1.1675976931606559e+01,    -4.3566255268869281e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.7809464326186500e+01,     1.6972059363809421e+04,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {    4.1454333965195104e+01,     1.4325428540632241e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.3336219872148870e+01,     8.1670279470000696e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0240735855511078e+01,     1.0212450718205848e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2798677818795042e+01,     1.7749704709398422e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5548794888026343e+01,     1.6759312668057260e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.4229825974676222e+00,     8.4306125738029041e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1671721106533992e+00,    -8.4363197411802994e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -4.4405139825808595e+00,     1.6765002496172841e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.2502738364378772e+00,     6.7691934145757141e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5611090287442750e+01,     7.7372670851609637e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.0501231317612778e+01,     1.5330740776143886e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.4682338510929132e+00,     8.4088735179517589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.6914400561344948e+01,     1.0634210756933227e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8900825440381304e+01,     3.1090956517961466e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -1.5494051442060119e+01,     2.4051471508544109e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    9.3460644408838718e+00,     8.8794467486568683e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.7947072420458934e-01,     3.4945471442411827e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.6414677857200886e+00,    -8.4679589304136389e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -4.9654918078415278e+01,     7.4987063061482622e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4186655595748697e+01,     1.7501300182847597e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.4648893525036199e+01,     6.9695685106126521e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    3.6419204321783397e+01,     7.7396732603194378e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7639345321574769e+01,     2.3337603581172363e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -7.5428942800344089e+01,     9.0773157708255731e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.5819129216635264e+00,    -1.5682716334237134e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8998956739263686e+01,     2.3367100532183886e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.8226110241548984e+01,     3.9676709091795623e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    4.5601085763506584e+00,    -9.7768318522490390e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.7398651422105200e+01,     1.4094623338095118e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.5233995555171594e+01,     2.9864334032034676e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2915314693402387e+00,     3.2298878397654091e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    7.5053757578019056e+00,     8.4156842382604391e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.1889725321598096e+01,     1.5647539545053756e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0590401585891559e+01,     1.5647518500646136e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.0013900374193227e+01,     1.0532702240063418e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.1700244647880908e+01,     1.0422244333566732e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.6026664202750940e+00,    -2.4447540326787293e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0272181061082613e+01,     8.8287883271259998e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.6963960073093265e+00,     1.1294177590181766e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -5.6752022077806803e+00,    -2.4110854396302057e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4545144179147409e+01,     3.3941909821985108e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {   -1.1970458796194496e+01,     3.5037938669728192e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1325094820900588e+01,     9.0448530617295510e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.0907898077629973e+01,     6.3750423530222679e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1815892776088363e+01,     1.6834314349082273e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.1055075328661918e+01,     1.6164502919428205e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    6.9446636326564146e+00,     2.1540542368303344e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.0991061861856334e+01,     2.5074639671397781e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.2249892524131738e+00,     7.9264816670151458e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0215918401921114e+00,    -1.1295946633809799e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.1376306392166224e+00,     1.6060253667716435e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.2022518226308048e+00,     6.4934320669219596e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.3271542514834422e+00,     5.8553027947152914e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.6379210676856601e-02,    -8.3900156085825574e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -6.4393801475105192e+00,     3.9187197090731470e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9174145749902927e-01,     9.0080932998590761e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1690179147766500e+01,     1.6697224559520913e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.4059043430805067e+00,    -2.1293981584045023e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.3511585205765208e+01,    -1.6793637809628803e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    9.0879347207643786e+00,     2.8168728122570292e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2113730684768315e+01,     1.6674243595894466e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -4.3782859077681707e+01,     1.5635323098125968e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    6.4186615586394318e+00,     2.5010666913410190e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    8.9028045131502907e+01,     7.7375890039850101e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4891573498804846e+01,     1.5202692941761086e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.0379703008601521e+01,     2.8457185468436692e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.8532635697884778e+00,     1.2392027576838762e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -7.4405910463134362e-01,     7.7249764791177495e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.2226257226261392e+00,    -1.7954531254344845e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.3425223108643189e+00,     1.4057411460137288e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.6181794240135909e+00,    -1.1086397168073498e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    8.8588655204889513e+00,    -1.9268871908873450e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.1309574275322465e+00,    -9.7771151388715822e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.3116708129651826e+00,     1.8080905902078586e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.5411642176419530e+01,     2.4597969076919802e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.9459087870645963e+01,     8.4329674847113256e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -1.2567714765661471e+01,     1.2139735870953828e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.8892167416765524e+00,     1.8473926871771568e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4037393375829016e+01,    -2.1419572187526151e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.5408464979840929e+01,     2.2465173837706403e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.6945433092001359e+01,    -2.5106268117878179e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -7.7732255712504553e+01,     7.4862434620212389e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.7961858107699392e+01,     1.7197424132113785e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.1919654699016505e+01,     1.6038678164985255e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.9786008193334041e+01,     2.4533182411147878e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.0476837178456778e+01,     9.3500199793619813e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9157114180683049e+01,     2.6629151458055287e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    8.2726020022468898e+00,    -1.2892331359920156e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    4.9262278401485933e+00,    -1.8433519852194331e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.6277813456348927e+01,     2.3816960839881747e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.0227452388164146e+01,     6.8886381202694511e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1291302859873767e+01,     2.4533887043064024e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.3384467425717006e+01,     4.9549450261972925e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.3289980808805410e+01,     1.7191390657671032e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8752746945649644e+01,     1.4514866779644130e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.4325421574804995e+01,     1.4388612652817448e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.6121342869246451e+00,     6.0441887441947001e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8149883924018688e+01,    -8.0247127037993141e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.0180490240266721e+01,     1.3695754758269873e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -4.8002179223177777e+01,     5.3400780842413269e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0527102809565937e+01,     1.6782821098371940e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {   -5.6914020263503478e+00,     1.5273652877480305e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.8813992028613626e+01,     3.0935851873155836e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.5744843948278303e+01,     2.3411902586361408e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.8665552278456648e+01,     2.4310281690945121e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.5013494677891384e+00,     2.0439031720236508e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.8878116827782012e+00,     4.8742450161376809e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8846514159289118e+01,    -1.2199555816657398e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.1224595335942130e+01,     8.4437841959681773e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -4.5566334499326082e+00,     1.1717319325920071e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.8780523080579982e+01,     3.1461987749273314e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3210852883263920e+01,     1.6913332980856350e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4181557990978845e+01,     2.0188012620884656e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.2878299473127095e+01,    -1.2188510026007730e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.8461154018206493e+01,     1.6054220193273683e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.5039742941685127e+01,     7.0977981446334670e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.5719392305087119e+01,     7.1207781203469222e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.0022149776963673e+01,     4.7848018478036720e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {   -1.1042715617955519e+01,     4.7847313846120560e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {   -2.0642709713484901e+01,    -5.3975188428336751e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.1366280342877619e+00,     6.5862073964001565e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0750831356331503e+01,     3.2831047748214318e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {   -1.3448367867627340e+01,     1.5754767803819706e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    8.2101221678308764e+00,     6.9503808408566192e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2100488834894943e+01,     2.1763884009750778e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.6770793558635901e+00,    -7.1048789053199353e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -2.4339240890550400e+01,     2.2232931987204673e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.0624926287599543e+01,     2.3503017384452101e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.6987341091058394e+01,     3.9911995707148177e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    1.0649646014750978e+01,     8.4268547579394963e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.9337793358747444e+00,    -9.3838014068171433e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5977355054471751e+01,     1.5549757871461230e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.7353511765684623e+01,     2.4779329668399019e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3529372665260000e+01,     2.0890577753502093e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.5643167535415650e+01,     1.0674025658831139e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.8677470774462478e+01,     1.4391266168716935e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.2157437612361754e+01,     5.9660841936500919e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.5851029466244020e+00,     1.6038290023191717e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.2132374080518034e+00,     1.6709899830370734e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.4309867316310637e+01,     1.5690188838805649e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.3961501528149380e+00,     2.5171031100066808e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.3373287631370957e+01,     1.7391176512627469e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.0535539166574306e+01,    -1.6304702262608203e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.0115457846416309e+02,     2.2861585174657564e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -5.6308452824702826e+01,     2.2861598601759011e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -4.7063372725926790e+01,    -8.4058571803176656e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    6.9208589300222929e+00,     2.0832052689929493e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5551084982823134e+01,     8.7830458442489326e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8408089134499679e+01,     4.8439825975563854e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.7983450842386119e+01,     3.4030927335599903e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    6.1255519884637835e+00,     7.3379366338057025e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -4.7380551392849867e+01,     1.7293562366599101e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.7845311998015987e+00,     1.5630668276202379e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.5304918241884025e+00,     1.5758862711317230e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    2.2090830809842402e+01,     6.4886071310709212e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.4994804342865002e+00,     1.3240236877090156e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.0159463755870828e+00,    -1.6926063551545512e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.5608252407151855e+00,     1.5089111215823186e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.4039575405487454e+00,     1.6631645699630648e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.6980676313001961e+00,    -1.5694428361503371e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    5.3996747260066114e+00,     7.2408407113340691e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.8951623442254322e+00,     1.6814415333859368e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9209239059140387e+01,     3.1260173090850527e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.7687533677695264e+01,     2.6300564838176160e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.5362662880098034e+01,     1.8562969120640424e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.5101139516468201e+01,    -1.5456028500569504e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -5.3673180854851310e+00,     1.1148530317945942e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0133674153252757e+01,    -1.7837211532452441e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.3874798548573118e+01,     8.0381439878572019e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.8482956796110571e+01,     1.5630256026084277e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {   -1.4047977017522417e+01,    -1.6774155192858521e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.5391249518940658e+01,     1.5710807401316624e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.0159881692936166e+00,    -9.5136019438346702e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.9578737314940664e+00,    -1.6167637054356150e+04,    -6.0035546564184289e-05,    -2.0835318822405737e-07,     1.0363910542446658e-09, } , 
      {    2.0464576248471925e+01,     2.3732889554281934e+04,     3.3332238209186915e-04,     6.0530476464526565e-07,    -3.0251404073872819e-09, } , 
      {    7.7552082025748934e+00,    -7.6685811553737244e+02,     2.4987703194628510e-04,     2.9185177585692671e-07,    -1.3266974864930561e-09, } , 
      {    1.7135328469297166e+00,    -1.0352164710251909e+04,     9.7584624183958920e-05,     4.1132901702272025e-08,    -9.0713487872405258e-11, } , 
      {    3.4903322657552721e+01,     4.7813884734623483e+04,    -1.3165532547257906e-05,     4.0601318045345852e-07,    -2.2560319836751145e-09, } , 
      {    2.7879784694248848e+01,     2.2199508505539070e+04,    -4.6737887434132792e-04,    -3.4675983412466832e-07,     1.5063597404386279e-09, } , 
      {    2.1852154104032721e+01,     2.3385124740385683e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1321670717793189e+01,     3.2899432399855745e+04,     5.0353762083667226e-05,     3.4451185004809966e-07,    -1.9024525179051186e-09, } , 
      {    1.9881531334178973e+01,     1.4357143684966664e+04,    -2.4623940896225960e-04,    -1.5877507460369779e-07,     7.3283466409155224e-10, } , 
      {    1.5084170857692307e+01,    -1.0436352538521126e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    1.0910722897312883e+01,     6.5857552803128010e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6569148202795478e+00,    -2.4422257751062170e+03,    -1.8005623283842597e-04,    -2.2089279712321928e-07,     1.1408538581133376e-09, } , 
      {    3.8580734683195761e+00,    -7.6293961639963991e+03,    -3.4034028507947772e-04,    -4.6976249493538599e-07,     2.2135186719786194e-09, } , 
      {    7.0859174976224306e+00,    -8.8860000744245226e+03,    -3.3501252209366066e-04,    -4.7099527916371135e-07,     2.3223981284821988e-09, } , 
      {    2.1176114748763592e+01,     4.1714450428530239e+04,     5.7607804298738790e-04,     1.0339343578783797e-06,    -5.1479455914934963e-09, } , 
      {    1.7553936783585261e+01,     2.4151988485754537e+04,    -5.3187191046337001e-04,    -4.1895159708670113e-07,     1.8010828253219257e-09, } , 
      {    7.9815807403798047e+00,     8.3996734703734764e+03,    -3.8419351047733836e-05,     3.2291645488086085e-08,    -3.1288905351447236e-10, } , 
      {    3.9132262740770869e+01,     8.4549455514501333e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1899487268946938e+01,     2.1970765740215254e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.0642140285567137e+01,    -4.6452583309028578e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2354670096652814e+01,     1.4344680905826392e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -4.3897020774024810e+01,     7.6069129508762762e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.7907795645834277e+00,     2.3014410469806277e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.8358618192723934e+01,     1.5978416842670487e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2821530977215216e+01,     8.2626138919529021e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.2818093498950264e+01,     7.2658685010153040e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -3.3696116807484575e+00,     2.3602344302103753e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.9169104636086898e+00,     6.3908090480145393e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4391591791530445e+01,     1.4029104656274725e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.7047266431579100e+00,     1.6197839721003063e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3089381898306440e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1162258229155210e+01,     2.0128494043813240e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -8.2099728136350180e+01,     3.2304813820523006e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -5.3578464942260684e-01,     3.1190611945206514e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    5.0092998389485437e+01,     3.1189954680456965e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.1400211034617375e+02,     3.2305009923670688e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -2.4610798972186700e+01,    -8.2259942345390009e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.3419744507276475e+01,    -2.3693689223350839e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5830028847772722e+01,     9.0978221589407422e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.5126771319963858e+01,     2.5436970637778060e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.8361532620019982e+00,    -2.5224191280974901e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1168154829680574e-01,    -9.5944953826298552e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.0605549642769354e+00,     1.4872028779098258e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    9.3868889416639334e+00,     1.6769161163594486e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.7253260014238840e+00,     1.6698879203648372e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8186960114331555e+01,     1.5368089779094862e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8580369709713672e+01,    -6.4039269592128067e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.0759394197701742e-01,     1.6186107748813823e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.5545770981500560e+01,     3.1269913116627562e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    7.5454545679656945e+01,    -1.9030678073651416e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1223894982603861e+01,     1.6625086744031512e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    2.1586234154252800e+01,    -8.2218391489715978e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.9874674555662850e+01,     1.7430280069058506e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8941375844445270e+00,     1.5112521442330977e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.1470976409875718e+02,    -1.6763888550592288e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    2.6249102945660554e+00,     1.4460803630777642e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.3654994159220699e+00,     7.7003838395779067e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2899256218474045e+01,    -6.5796949974939489e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -3.3952867182356933e+01,     1.7961684853542556e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.8600684065011613e+01,     2.5599711886090307e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.7770140526704601e+01,     1.6133521364694734e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1639761066233671e+01,    -9.8423410583305213e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.6719735051860951e+01,     1.4108734460070549e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.3285324314171888e+01,     7.5667434519970402e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.8936602710123758e+01,     1.5675526141817418e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8532075344168444e+01,     1.5273686247174081e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -7.0322739825140967e-01,    -5.2424419764426875e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -7.7170672483611398e+01,     9.6320270265180443e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4571894992446603e+00,    -1.1238433182554659e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3875523241729830e+01,     1.5637219741594457e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.5758505756400707e+00,     2.6368202450172487e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2035735321883507e+01,    -1.8580201584342874e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.7941674355342752e+00,    -9.2532895243327366e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -7.8117184878866261e+00,    -5.1496129890631310e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.9677587754495889e+01,     1.7525451901142762e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.2115014676499314e+01,     8.4318382341500619e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.8859204189685087e+01,    -8.4350940808331397e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    6.7768110951428056e+01,     7.3083626419797547e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.9338557540870367e+00,    -1.2620631080889600e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0706615223294641e+01,     9.2839424466914315e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.6334702713207179e+01,     7.1791179763489536e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.6418567472572384e+01,     1.5433048524856047e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    9.0656445866480571e+00,    -5.5731427963255283e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8607845031010886e+01,     3.6235121569026720e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.9207008183497194e+01,     1.7079927490737326e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4272952765745611e+01,     1.2751267119076590e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.2353088535149304e+01,     2.2587608072658131e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    9.3759048687604398e+00,     1.8411793721899125e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.0799515427051709e+00,     2.0046010750543523e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.0901112638880068e+01,     1.2060826384158601e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.3179661197207793e+00,     1.5670070927982189e+04,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {    1.6880544416702705e+01,    -5.3230798274028621e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    8.6576122895782550e+00,     1.4494203263387237e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.6932402616315780e+01,     1.8346860699304987e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.1106376768364090e+01,     1.5644684108792155e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8889632675432466e-01,     1.5429343058730810e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.4106203988865353e+01,     1.1510071060125181e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0261311132494605e+01,     2.3689118720724971e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.8809252081133497e+01,     3.1719264103007008e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -2.0200106330175167e+00,     2.3930364961743444e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.5745174104317456e+00,    -5.3157713001629963e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    4.7797276660582526e+01,     1.5557424262807381e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.6071344573351681e+01,    -6.5851958615955891e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.5970980415226393e+01,    -3.2445834268783119e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.6651318826120551e+00,     1.6825435960581730e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.2329926126096655e+00,    -2.3707916132688360e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4416101969389734e+01,     3.2224729778768538e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.6979507165051920e+01,     1.2834249085126587e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.2158880419342601e+00,     1.6320718581835090e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.1938827110539094e+01,     8.7816231533151822e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0933014984255244e+02,     1.5019319489378242e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3726894689495905e+00,     1.5650373936907736e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5178079714551510e+01,     8.3401661529650264e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.5893428617196026e+00,     1.6288954519093757e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.0195655269637970e+02,     7.1075571640129565e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.3752375826091995e+01,     1.2597335534345484e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8771397774692520e+01,     1.6459308111852173e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.9615162463310156e+00,     1.2831947885185176e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.1278029481322811e+00,     3.2519528409874838e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.5335060685066900e+01,     1.6759854647176253e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -5.3192506247034331e+01,     9.1569455746213989e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1001462790520998e+01,     8.2621073463807188e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2011498771368167e+01,     8.3150764435919100e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9494968590915285e+01,     1.3542765522684977e+02,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    1.5570216122188329e+01,    -7.8182764836443248e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    7.7535510893338628e+00,     7.1611032438860630e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -1.9571405071043364e+00,    -2.9929923588026013e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.8592414524977023e+00,    -2.4753071167574667e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.6012739156802471e+00,     2.4355163664884723e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -2.4607970428302288e+01,     1.5714952901921793e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.7624164669559249e+01,    -1.8500120771202091e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.6702499853812625e+01,     2.3349990405211553e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.5886742202028685e+00,     1.5657838304105433e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.4582289243893619e+01,     5.7985618476479704e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1766038367438236e+00,    -7.7545163983433304e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.7120859343951960e+00,     8.1302586668478114e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.3694016608907077e+01,     1.0300149513461500e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.8619903328493947e+00,     8.2517980651252310e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6656501881369493e+01,     3.3794543728964559e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7258559825421322e+00,     1.6780633906404208e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.8317986091760616e+01,     8.0042330819356193e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.2003216914547565e+01,     7.1453707598867659e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.0544377361817244e+01,     1.5612781192993722e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    4.7098337442271125e+00,    -1.6797102471124443e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    6.0982029038726013e+00,     1.6603250290481476e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.1334000760338732e+01,    -1.1559144412500791e+02,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    5.3779761104365251e+00,     6.1469193039729225e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0241090699399038e+01,     1.6879551553401190e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5488312904581143e+00,     2.1147521398610359e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.1809136939878462e+00,     9.1573337164149343e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.7297278930324929e+00,     2.4463252767159833e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -6.0810392528689272e+00,     1.6750159971371577e+04,    -1.9201849612945083e-04,    -1.6035648835010902e-08,    -8.8284571330046551e-11, } , 
      {    7.7024827683969006e+00,    -1.0308399506526424e+04,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    9.0125500507963476e+00,     1.6516499765815126e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0937920430659041e+01,     1.6217462540900488e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.5689393277517095e+01,     1.1781768935249080e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3033113689040430e+01,     6.9541836104659451e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.9607349900405211e+00,     9.0095159907928282e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0232884640417437e+01,     1.7693149983537310e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.4478608067993299e+00,     1.3917710789737564e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.1833078354943418e+00,    -7.9886300764027437e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.9713118956864015e-01,     1.4011010794264137e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.2522400274426808e+01,     4.0791337651060727e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.9448441492874743e+01,     2.3156397080632261e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2837965841763159e+01,     1.3648642746929847e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0119838190478262e+01,     1.4907887749337118e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.7904702337739980e+01,     3.2615617288300382e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6608381273111529e+01,     2.2370355141246288e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    6.2928037837578534e+00,    -1.8121597627901568e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    6.1707326012433708e+00,     8.9066692205128966e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1157355757254226e+01,     2.3051661966856445e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.1492458641722081e+01,     7.5229791797577454e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.9586619939296330e+00,    -1.5448614069159230e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.3143237244908207e+01,    -8.4960979801443245e+03,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    4.0146367100188733e+00,    -1.8690407851566515e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0008306006877838e+01,     6.4192020010220203e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1805504063027303e+00,    -8.9635579652681281e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -4.7105033674630103e+00,     6.9389345397317747e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.6341959107028330e+01,    -2.3167588546326933e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.3737255488811712e+01,     5.4619531926475302e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.0825481830635262e+01,     1.7390136534785823e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.2491210402099689e+01,     3.0657576932254186e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.8021473215864020e+01,     1.7359313572021620e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.8640996829339098e+00,    -1.7188959979263200e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    8.1334947179175003e+00,     2.1382808013962916e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.6537132751472758e+01,     1.8321123549688735e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0281095136663783e+01,     1.6815117203587331e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5877616439648996e-01,    -2.9154400977904775e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.4595497690582782e+01,     1.8633962426053415e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0215185949455918e+01,     7.1136973596604539e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -4.6141319922268245e+01,    -1.2808120407657088e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.1258723087083418e+00,    -1.7483765171687166e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.4650611319103639e+00,     1.3191187427423005e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.0683444684053338e+01,     7.7653436705481454e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.2264528556135392e+00,     1.6764460517053849e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.3972494003430924e+01,    -9.7627112545346972e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.7885183851739114e+01,     2.4177725635370793e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.0269052792404384e+01,     1.2128389239068922e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.5437467342213478e+00,     6.7940187523562622e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.9260865627459012e+00,    -1.6841718047768842e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3166365506335410e+01,     8.2081689482408474e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.0979788469342196e+01,     2.2312914106958568e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    8.4456897388821623e+00,    -1.0021503607214109e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.3557976445752551e+01,     3.9555023153858783e+04,    -5.6714050927263691e-04,    -3.2042979702908184e-07,     1.2184822247325963e-09, } , 
      {   -9.1707795986269645e+00,     1.0761964692594213e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.8878554758750170e+01,    -8.5416866003775758e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.6013231263163050e+01,    -9.5420612423139537e+03,    -2.7584724895743558e-04,    -3.9757073197335323e-07,     1.9187955870953214e-09, } , 
      {    2.0338480261132368e+01,     8.2365359301732897e+03,     4.3042013553026956e-04,     5.7959111966384185e-07,    -2.7917026199330408e-09, } , 
      {    1.0857228990529954e+01,     1.0192981881035936e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.1385940950353088e+01,    -9.2352747918698460e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -2.3581329754301301e+01,     1.7163642704658625e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0310531356988420e+01,     2.2133957508292468e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.8387575558923563e+00,     1.4895268510919130e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.2332112042348749e+01,     1.7148619596208904e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5050795082126815e+01,     9.3620535585537109e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6705673940695171e+01,     5.9766353650232895e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -1.1949477264012606e+02,    -7.1110191009674327e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -9.9379312268349210e+01,     8.4311716711009158e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1205990071593281e+02,     8.4357606438822859e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.2755676353399323e+01,     1.7860539725085426e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.1138946615695318e+00,     3.1821041019165141e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2565040707989397e+01,     1.6738685435915380e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.6854923304518339e+00,     9.0955251598297255e+03,    -9.7486691820374786e-05,    -4.1176534933571884e-08,     8.9986267350740956e-11, } , 
      {    9.4251019248096632e+00,    -1.0210153719094833e+04,    -2.7328129982811123e-04,    -3.9814349085621605e-07,     1.9739625358687753e-09, } , 
      {    8.2572969983044615e+00,     2.5158363100986629e+04,     1.7894536024620343e-04,     4.2319711049217731e-07,    -2.1536878155928909e-09, } , 
      {    9.5393180127048396e+00,     6.6567785958883715e+03,    -4.0129363058107845e-04,    -4.0883392341288995e-07,     1.9151061549820774e-09, } , 
      {    2.0050636576826204e+01,     4.8996051768467805e+04,     2.9587123683567863e-04,     7.7248141793575112e-07,    -3.9715451942812071e-09, } , 
      {    1.8437995663516336e+01,     3.2966970874434977e+04,    -1.1377460181882234e-05,     2.7166006174060436e-07,    -1.5540169252916956e-09, } , 
      {    1.1037736380401295e+01,     5.8189271749380096e+03,     3.3967397189449667e-05,     1.0267786533893073e-07,    -4.4356573312073666e-10, } , 
      {    2.5736463343293835e+01,     4.7881423209202709e+04,    -7.4896754812807360e-05,     3.3316139214596317e-07,    -1.9075963910616914e-09, } , 
      {    2.1391543418193347e+01,     2.1989983420133864e+04,    -3.4879660427292473e-05,     1.6541197997261493e-07,    -9.0602465539431193e-10, } , 
      {   -2.0220432487661535e+00,    -1.7322945443080986e+04,    -1.4725565079489941e-04,    -3.1888547152927558e-07,     1.6675602894075487e-09, } , 
      {    1.3070227617504436e+01,     2.2753373584169338e+04,    -1.5567822346548288e-04,     1.9092011243461638e-08,    -1.5471374191567517e-10, } , 
      {    7.7423837547782393e+00,     1.6866902304820622e+04,    -1.3057827988229161e-04,    -1.0117673673811129e-08,    -1.1402332966015179e-10, } , 
      {   -5.3877782822955300e-01,     5.9104131659403720e+02,     1.5200140174393594e-04,     1.8378506100835919e-07,    -9.1328716433733273e-10, } , 
      {    2.3472609810614049e+00,    -5.9457491825257603e+02,    -3.0718712587398539e-05,    -3.6491344000697442e-08,     1.7312696552421530e-10, } , 
      {    3.7241080740152368e+00,     9.6563130207956583e+03,    -3.8517283411317964e-05,     3.2335278719385943e-08,    -3.1216183299280803e-10, } , 
      {    1.4332857749898464e+01,     1.6137329208234372e+04,    -3.5464463536435091e-05,     9.8609066520218951e-08,    -5.8114615954600074e-10, } , 
      {    2.1084254555719781e+01,     2.2719556516720113e+04,    -1.2999347677314897e-04,     5.6685239778584825e-08,    -4.3890182550846293e-10, } , 
      {    2.1068482897436486e+01,     3.0595738404741740e+04,    -3.7681768884455116e-04,    -1.6889274827750889e-07,     6.1881133443140029e-10, } , 
      {   -9.9178678984592796e+00,    -3.3352026337113821e+04,    -4.5450434781246031e-04,    -8.1970682772442234e-07,     4.0850885583970603e-09, } , 
      {    1.2594649342204836e+01,     7.9133825063164923e+03,    -4.0662139356689551e-04,    -4.0760113918456459e-07,     1.8062266984784981e-09, } , 
      {    1.7025827798868317e+01,     3.2548218384957207e+04,    -4.4131072496659331e-04,    -2.4108451123954170e-07,     9.1353441931469843e-10, } , 
      {    3.0522036093196220e+01,     3.8228553759577844e+04,    -1.6536000794599995e-04,     1.5525067306750394e-07,    -1.0207752055761279e-09, } , 
      {    2.0423783074235928e+01,     2.3243113749228603e+04,     8.6207164002076686e-05,     3.1279296344287635e-07,    -1.6447304131641006e-09, } , 
      {    1.9035985028082191e+01,     1.3661291995510415e+04,    -1.8717206818961865e-04,    -8.5306894182039792e-08,     3.2995934322633890e-10, } , 
      {    1.4694648477496475e+01,     1.5056457696093330e+04,    -4.3704910013590376e-04,    -3.7715867003896655e-07,     1.6566568297193952e-09, } , 
      {    1.7855799669275122e+01,     1.3100504134544481e+04,    -2.4614147659867547e-04,    -1.5881870783499763e-07,     7.3210744356988791e-10, } , 
      {    1.4883510356935727e+01,     1.5542724279819216e+04,    -6.8749125253158026e-05,     6.2690481402384324e-08,    -4.6318614279523954e-10, } , 
      {    9.8575464837381723e+00,     5.2581393139720749e+03,    -2.5002011219607148e-05,     2.9166051685972912e-08,    -4.1417632777187652e-11, } , 
      {    3.3431641004957307e+01,     3.8370589131066023e+04,    -5.3632386432165419e-04,    -2.8398208625968429e-07,     1.0446280386867167e-09, } , 
      {    1.2382404549908301e+01,     1.6029050883870259e+04,     3.0468274788823650e-04,     5.0139411507800953e-07,    -2.4726952177629659e-09, } , 
      {    2.2205543441051653e+01,     3.1784828220921740e+04,    -3.2051216192840293e-04,    -9.4764542510388419e-08,     1.6222350583606157e-10, } , 
      {    1.5457120129163489e+01,     5.4001146651350900e+03,    -4.0109776585391013e-04,    -4.0892118987548972e-07,     1.9136517139387490e-09, } , 
      {    6.0796593082764607e+00,     1.7985004445419108e+04,     1.1377512435100821e-04,     2.8305415287404066e-07,    -1.5481458316134589e-09, } , 
      {    1.3357120470273074e+01,    -1.7429605246417450e+03,    -3.7067005928490183e-04,    -4.3936365902108781e-07,     2.0632215826978519e-09, } , 
      {    1.5150671007134026e+01,     3.1852402335495022e+04,    -3.7701355357171949e-04,    -1.6880548181490918e-07,     6.2026577547472895e-10, } , 
      {    1.9790687788006810e+01,     3.1014550914544659e+04,     5.8247474198808631e-05,     3.4270630693691158e-07,    -1.7384061126280854e-09, } , 
      {    2.7046328531281061e+00,    -1.8471331049470151e+04,    -4.8720699749240271e-04,    -7.2175778654966593e-07,     3.5576549065811849e-09, } , 
      {    1.3719931160868303e+01,     3.2828469104605334e+04,     2.3830370703723460e-04,     5.6359910406013079e-07,    -2.8792599707414226e-09, } , 
      {    1.0833892875176119e+01,     3.4014085339451951e+04,     4.2102382136856905e-04,     7.8387550906918747e-07,    -3.9656741006029709e-09, } , 
      {    6.8421335779373509e+00,     8.3286614144608720e+03,     1.4972645863300178e-04,     2.5129163303751754e-07,    -1.2911509473941051e-09, } , 
      {    1.4325065051040884e+01,     6.9334957138779555e+03,     3.9960349057928694e-04,     5.4314340889444430e-07,    -2.6178484338871610e-09, } , 
      {    1.1934162152757713e+01,     8.4369397388249818e+03,    -1.9042075279166978e-04,    -1.5149341552027312e-07,     6.0039811082286022e-10, } , 
      {    3.1431085101708451e+01,     6.9284530221673631e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.2457235372019646e+00,    -3.6266954378930463e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.3270311137361858e+01,     1.0753655462626964e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.1065491018502209e+00,    -5.6334775407530469e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.8437561707990131e+01,    -8.9884792371827425e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.1418802882212812e+01,    -6.3375552304341682e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6805089418237014e+01,     1.3302383078202602e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.3250622601647201e+01,     1.8760625995960863e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1269930272259160e+01,     2.1435242154278814e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.9634986966398785e+01,     3.8890314836451565e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    4.1810867844650282e+01,     1.5442910589371244e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.8169391084641671e+00,     1.5703248461246579e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4523048640767209e+01,     2.5406317715419362e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.6815117850271836e+01,     3.8421266858997675e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    3.7781688134666901e+01,     3.2322960815552702e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.6086015701570410e+01,     3.1373754816788431e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.0672776693793001e+01,     1.4302021089870688e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.4182076676821680e+01,    -8.0584941312544761e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.0348009438944302e+01,     7.0922208509505463e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6144207195518271e+02,     8.4326058828944078e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5818626146836851e+02,    -8.4343264320887938e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.4341763942376481e+01,     1.5021298980588102e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.5032830340878380e+00,     5.2516981123835706e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.1396069711857635e+00,    -4.5137608521017012e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.4570448279688694e+01,     6.5943917786956354e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6219175988952905e+00,     2.2666636180171117e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.7144084746341228e+01,    -5.5065513313595475e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.1297924249060841e+01,     7.7301175509666800e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7998680212134740e+01,     1.8083207102019995e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.4360405858498675e+00,     2.2652371054846768e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    4.9192742453057861e+01,     2.2651713790097216e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.8986327202953479e+00,     5.7908944159764769e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.3901000824172758e+01,     8.7904044997880192e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -7.9446460160784440e+01,     9.0037194414725000e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8781374521018151e+01,     1.6210876776925250e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0361173250995357e+01,     1.4969707526349943e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.2095027857299812e+00,     8.4340097488266038e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.1465313272544208e+01,    -8.4329225661565961e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.4703244693286777e+01,     1.4260994190818383e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0112398549160925e+01,    -1.2826816705999356e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.0112182125809994e+01,     9.7973201085524452e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.1999523584728564e+00,     2.3878004670204380e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.6715460322068221e+01,     2.2635121741664392e+04,    -4.3912817851966964e-04,    -3.0973936447240789e-07,     1.2773386056192941e-09, } , 
      {   -2.0090231433056243e+00,     6.5948598856062131e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3595803872486499e+01,     2.4415642083039529e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0269419998902245e+01,     1.6180250662942655e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6974927470301303e+01,     1.2619238417987162e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.1513725269965942e+00,     1.5787229693249668e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.4968049672786002e+01,    -8.8266302178658407e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.1117010971197310e+00,     1.6280903679937177e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -7.9669714490098102e+01,     1.4323252946274699e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.1643212730047095e+02,     1.4323449049422379e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.6553232412378631e+00,    -8.4252991144579355e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -7.7177993878524831e-01,    -1.8668985469348306e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1990783005366417e+01,    -1.6514251728344505e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {   -8.4202902208718946e+00,     1.6431269762294509e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.0088425842082486e+01,     1.3586509597057404e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0151637789717205e+01,     1.4077905688323845e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9788492897285970e+01,     1.6446190200653909e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.8949266718696947e+01,     3.1556072334929791e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    8.6339083843900735e+00,    -1.2193927699294466e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.8623988543765453e+00,     6.5055637112455834e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.3345905858081437e+00,    -1.2194138143370662e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.6242319206237426e+00,     1.9652989780850534e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.2309756839289783e+01,     3.0583305635764045e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    7.2979802801030704e+00,    -2.5103966917936770e+04,    -6.1134664179665469e-04,    -9.3541255782076041e-07,     4.5653449909041671e-09, } , 
      {    5.7934174495048332e+00,    -8.4885774929246145e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.3083412353592305e+01,     1.7638371020931136e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.4639348031748389e+01,     2.5095258233908757e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.1382304039549940e+01,     1.1354966538100933e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7206685704727345e+01,     1.1070253590519033e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3533665737318241e+01,     3.1800557213936572e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    8.8003166154711057e+00,     8.3047671623364222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.7345069665204065e+00,     7.7379243499105096e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -8.2655253460843636e-01,    -8.8308943688229901e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    6.6150914074633192e+01,    -1.2071973674092778e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.0942142285564227e+00,    -8.4695745080552260e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -4.8086377428145553e+01,     8.4330433464255439e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.1342187914959773e+01,    -8.4338889685576614e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -5.9356504367100626e+01,     1.0266368086006340e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.4187473819999248e+00,    -6.4165461976307943e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.6521689648777357e+01,     8.4298930265148356e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7292469882337741e+01,     2.2984450763438268e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.7434207024674961e+01,     1.5806788630441595e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.3086096154953282e+01,     2.4071780421351184e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.3366157006799710e+01,     1.5302992823131477e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.7433957087917264e+01,    -9.5501722595404062e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.6691996811860371e+01,     1.4634870336188022e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0490369346633961e+00,     8.9977840186035883e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.0038706672461313e+00,     3.1015261745481948e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.2230954777834722e+00,     1.5469157960628723e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -2.0640721693936452e+00,     5.6823193696406879e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5311566160744533e+01,     1.6572891470385030e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1194816908509226e+01,     3.1778775995979366e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    5.0377061639640646e+00,     1.6761761748075038e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.9292134236237608e+00,    -1.6762553416155060e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    2.5949085263657622e+01,     3.3432159669779932e+04,     4.5470021253962852e-04,     8.1961956126182262e-07,    -4.0865429994403895e-09, } , 
      {    2.1625306637397603e+00,     1.7336784724482528e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -3.3042509087668051e+01,     7.6662500961646720e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.7719617494672160e+00,     2.2830938965849593e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -2.3386462773495658e+01,     7.1263554140298402e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.4224522896539511e+01,     7.1056853364552771e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.8036265123761424e+01,     7.6669547280808210e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.9883491688981536e-01,     3.4008033114509584e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.2696850595608030e+01,     8.4430558914590201e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8691405680914009e+01,    -9.5460171739730067e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    7.7007045180710083e+00,     1.5394208723556683e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.1499960160226756e+01,     6.3797752409176446e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6531174360226210e+01,    -7.5113126136955543e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.6917223271674398e+01,     1.6205171934857273e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.3711560010391622e+01,     1.6204514670107726e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6518616357724163e+01,     2.1682528979129329e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -6.5783145034957080e-01,     6.0753403393336757e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.3162792161166429e+01,     3.2299980839839245e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    4.7370874202298436e+00,     1.6394314911188165e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.4904472260760855e+01,     7.1128909285251111e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.8779439526283692e+00,    -1.2631741256902201e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8023634000649658e+01,     2.6158578227350179e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.6924750383720255e+00,    -1.0305173426538477e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.6747377331780466e+00,     8.9426726831705764e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.6308239049642115e+00,     1.6123540715814021e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.4457508966244070e+01,     2.3975668157940912e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.6936215800796091e+01,     1.5585395872977499e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.7764963616212327e+01,     8.3716463804787763e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    2.1171610042630977e+01,     1.6068916974475371e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.8616830216394344e+00,     1.5641495548407192e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {   -1.4947351949465357e+01,    -6.3694408342837733e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8921639569697792e+01,     6.3001454371218197e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9191706487539385e+01,     7.1033880784049661e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -7.2664678965054570e+00,     2.3976772737005878e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.0926246111058568e+01,     8.8084381837287274e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7755855001545942e+01,     8.6011899179338361e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.8433870778377781e+01,     7.6701718384638916e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.0808134801126155e+00,    -3.3084610674834657e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.3913530595142714e+00,     2.5106848514962498e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.0354606814268415e+01,     3.2552817725867393e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.8177844742012610e+01,     1.8757915352938686e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.9443081087920433e+01,     7.5721731570349129e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6356660124182831e+01,     1.3852777767143427e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.1763099759104078e+02,     8.4331205550842533e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2402840073144479e+02,     8.4338117598989484e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.8438094535408318e+01,    -1.0056957596108514e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.5355073829183162e+01,     2.4864378860194603e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    2.9237994879227877e+01,     3.9441422476443069e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -3.5767179485436820e+01,     1.5675138000023881e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -4.8507625060075050e+01,    -8.4567726287000223e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.4929576054067805e+01,     1.4655759338814816e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.0673772372538952e+00,     2.5265905699552764e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.6486092214136518e+00,    -9.5131498277473147e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5455584071942184e+01,     1.7468462054053020e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    7.8923240618603989e+00,     1.5524730081779997e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.2867416705203070e+01,    -9.0460412585378454e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3688568805925208e+01,     1.9637257296851232e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.9710957652966687e+01,    -2.6203176366062202e+04,    -7.6363904955898082e-04,    -1.1861314319754151e-06,     5.8013289895248171e-09, } , 
      {    1.0769436920653719e+01,     7.1011210894565265e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1918670752030245e+00,     5.6511677745017114e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5570968975285808e+01,     2.1935864537937745e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.5421478517345093e+01,    -2.4756661462662673e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.7386109205061144e+00,     8.4320265951057499e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0868063284473115e+01,     1.6206006764036731e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -7.4881897643717139e+01,     1.0260334611563589e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0880521058269323e+01,     2.5073216980464029e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.9487396062366740e+00,     1.6805311133482985e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.1168595698010733e+02,     2.3348010914001694e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.4479990408133464e+01,    -8.4231481190150225e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {   -5.9790240714985075e+00,     1.1324298356227082e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -5.0687334069758396e+00,    -1.7025957612563548e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.9603923634861776e+01,     2.1837648760668730e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.1439843164366117e+00,    -1.7338650055764723e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -1.3620009978115075e+01,     8.6598926173036125e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.6875820464929301e+01,    -8.2070396976795910e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6899253742823422e+01,     1.7058056826194315e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.2673961573556559e+01,     1.6686427684171063e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.8978346735094469e+00,     8.4732723897586075e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6119251700442753e+01,     2.0926026958968934e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -9.1880402451505994e+00,     8.4683768355080210e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1109766757729140e+00,    -5.2090301986120721e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.0215365047746747e+00,     4.1496354401471642e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.5934509932794741e+01,     1.7646415298770044e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.5651728000664196e+01,     3.2540198487449401e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.4160295398529593e+01,     1.7408489375362391e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6918304203630480e+01,     2.5287946952997492e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    3.3617252566651125e+01,    -1.6791007977312616e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {   -4.8203162657255554e+01,     1.0306182987904251e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8806053168673358e+01,     1.0561319059496303e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7090243770107598e+00,     2.2781962084412462e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    7.6397789676131438e+01,    -8.2394587200437290e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -1.4171213946400407e+01,    -8.7051705939331732e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.7341528196159011e+01,     6.5516215368995681e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.5517594580213597e+01,     3.9583409087269050e+04,    -5.6714050927263691e-04,    -3.2042979702908184e-07,     1.2184822247325963e-09, } , 
      {    1.3289999463837951e+00,    -8.4349057198774499e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    5.3707022012045940e+01,     6.5523261688157163e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -5.5517974599619642e+01,     8.2825414999794793e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.3013669527000591e+00,     5.9977880783215523e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1276286286822330e+01,    -9.5394582183738730e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.9424588955725255e+01,    -8.2083746634668696e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -8.9431547512786675e+00,    -1.5932794375028964e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.1962713898614632e+01,     2.3492656626173807e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6630514679542600e+01,     2.1079958543700042e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -2.1559518205871427e+01,     8.4032109888345167e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.9900186722669746e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4864560371078326e+01,     8.1271212170951844e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2419680689585775e+01,     8.2496558269034103e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1309551043824293e+01,     3.2410850066519230e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3889820409531971e+01,     1.5595271271105630e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.4748320689502883e+00,    -7.9164922609133428e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.9386724182925118e+01,     1.6275484291937415e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.8054420244722778e+00,     5.7260902031976520e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3590493554626498e+01,     2.5223957229063937e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.7065026340428698e+01,     1.8116988529475155e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.4628921714234757e+01,     8.3026249241146015e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.1689761425117865e+01,     1.5370555678041799e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.1962203436266430e+00,    -7.6169390146876076e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.8742368968951155e+01,     7.8615486444210082e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    7.3719728347815314e+00,     1.5103918313641916e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.6914124134524364e+00,     7.1224509974343791e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.1851059210251538e+01,     1.6761029319466779e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.2959571451429156e+01,     1.6763285844763315e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.1338265806322386e+01,     2.0154231193429492e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    7.3065808981877836e+01,     1.6078493066883166e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.7604302352348604e+00,     2.5017252677385426e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    3.1801852995716999e+01,     2.8850206438129677e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1913703996269440e+00,    -2.9492215252456381e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8402978233982445e+01,     1.3227270054819575e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0430790138135794e+01,     9.4144543291758382e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1051902554318712e+01,    -1.1767434761775528e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.6810139143451890e+00,     2.3676499482306979e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -7.6247244229675637e+00,     1.6388281436745412e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.3074978914295088e+00,     6.6909259398013853e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7862496220361450e+01,     2.2131112594234677e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.2346011391514455e+00,    -4.6985240949932478e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -7.0992569182963159e+00,    -2.8041064093231926e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.0747301960603103e+01,     2.3140664596632963e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -5.7069735924717193e+01,     1.6783733084846237e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.1132824635938732e+01,    -8.4336051333300784e+03,     1.8766447463475076e-04,     1.5096152643072787e-07,    -6.6889743582682677e-10, } , 
      {    1.3319262030305781e+01,    -2.7827578220085240e+02,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {    9.0576655805925395e+00,     6.2430640997759110e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3921474413778739e+01,     2.4796602665164683e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.4086874134323756e+01,     2.4629005510260754e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.5063465810647720e+01,     3.2309842904354442e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.7609040866429275e-01,     1.4719500472637990e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1104441560511612e+01,     2.9044834924163069e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0073366937988606e+01,     1.5787248644727488e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.3182876384276149e+01,    -8.4370392884683642e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    7.8852516770319056e+01,     7.1069002995150431e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.5107243817164171e+01,     3.0822999492755011e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.0500996473677890e+01,     9.5475960439764294e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.0954882663950745e-01,     8.2864229179148447e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.3613275448307451e+00,     6.8805604727367718e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.1121798600258037e+00,     2.0754500428917374e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.0841984148261396e+01,     2.5010168240629915e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    6.5687393316958023e+00,     1.7387835334844407e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -4.3457056800897265e+00,    -1.9378823766089106e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.0203002459825692e+00,     4.5158756989797130e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.4692291007261034e+00,     4.3561886595680869e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.8267845261382787e+00,     1.5025780393403536e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.4326652207436883e+00,     2.4409572324481764e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -5.8173452020903412e+01,     7.1116759654653433e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.5814489796145068e+01,     5.4004128286688456e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.3253187811334936e+01,     6.6557821833247290e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5043378738317166e+01,     5.7659319134716661e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.1366934978631535e+00,    -1.6551896770911870e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -1.5784529691212299e+01,    -5.7905398125266602e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.5894726886935198e+00,    -1.6553319461845622e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -9.2329761207752803e+00,    -5.5426339587561197e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8078876610980132e+01,     3.7724753425894432e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6646033682974135e-01,    -1.6979410513526898e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    5.1352033213677855e+00,    -6.4979012282522444e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.6230177247299011e+01,     6.7303449150788638e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    3.1650458815433076e+01,     1.3982160399066572e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5873371858848877e+01,     1.4996679532672159e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9783256380754519e+01,     1.9671038724306396e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    6.6049435833790220e+00,     6.8082456616937884e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.7385389459376356e+01,     1.3555358001918425e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.4137157673897196e+00,     7.6124698533573228e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8609748407323622e+01,     4.1731311175177805e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.2135923811296728e+01,    -1.1561249136665772e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.7629452570241453e+00,    -1.0229371399013440e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.1222663894216620e+01,     1.8773663096453136e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.6897679344727212e+00,    -1.2362640387808242e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    7.6749674905842307e+00,     6.7964682076033341e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.6429316999280360e+01,     5.3039951811493884e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7271691445208901e+01,     2.3913863640443236e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.6179163031685277e+01,     2.3844223242749216e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -1.2539661361608648e+01,     1.6125213498686673e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.3112498001226385e+01,     1.7406007195449027e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.9570914242772122e+00,    -1.7929755238591348e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    4.8295268927790937e+00,     2.3364833543235443e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -1.1149952130767826e+01,     2.5230534417623072e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -3.4332117663703386e+01,     1.0713840560729052e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5192295693646148e+01,     1.7198949385365006e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.9637073235344374e+01,     1.5693377399190611e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0607429653186647e+02,    -1.8476128256049601e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.9346069338689560e+00,     3.2599158213670664e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -4.2058853517192098e+01,     1.6371008439979743e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.9432289689455580e+01,     2.3811927137533828e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.8019078573546448e+01,     3.1033702944306635e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.7147341567907426e+00,    -1.7823817044168962e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.2966085975017489e+01,    -2.0042912457039081e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {    6.5855705895560206e+00,     7.8588390151241256e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0149001861422320e+01,     8.9100996710155669e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.3541975271448251e+01,     2.2416755807119436e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    8.8345230673527482e+00,    -1.5223855130798260e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.1669356706645916e+01,     1.4732119711055977e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.7673139352643041e+01,     1.6739866142516621e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3745602065677442e+01,    -5.8417035209065013e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.5619431620002878e+01,     2.1871430188123886e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.1204422729418255e+01,     1.3814488118496734e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.3760148107736423e+01,     2.5035737671305487e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.5972927384398442e+01,     8.7720252175265705e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.3959455212561105e+00,     9.0720830237926311e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.5898564295847080e+00,    -1.5766566176992772e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -7.2766622491345956e+01,     9.5982455990628841e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.5050913787459042e+00,    -3.7014820371764526e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7822815748923649e+01,     9.1262080923510002e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    3.8043045353610605e+00,     5.7797779588944559e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.1823839787322399e+01,     7.7896165441576277e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.1263770748487723e+00,     2.9863804087888922e+04,    -1.3454336299501728e-04,     1.9169838383690152e-07,    -1.1946293916220075e-09, } , 
      {    5.4429897581032435e+01,     2.9868406487771743e+04,    -4.3912817851966964e-04,    -3.0973936447240789e-07,     1.2773386056192941e-09, } , 
      {   -6.7896907758659530e+00,     1.0755509348252956e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2305444396922944e+01,     7.7740069773070809e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0657663213186531e+01,    -1.6871190070640925e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {   -8.5300839679548432e+01,     8.3018155270738844e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5438709415747844e+01,     6.4847938392577853e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0628041613770542e+01,    -1.2297125733887442e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.5134640353117046e+00,     1.7235360645136345e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.2059377117922121e+01,    -5.0382793005958465e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2118820661282591e+01,     9.3384853168603786e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1738764969432571e-01,     7.3073628698849197e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6236840959452223e+01,     1.4307808969560110e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.0787188861334485e+01,     1.8804814691592113e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0179361979598099e+01,     2.3767382327203970e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.5664874317272071e+01,    -8.4783481314312030e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    5.6118430298894850e-01,    -1.7056962774539021e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.2631207404362577e+01,     2.1113739971155203e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.2659706325216931e-01,     1.0437889882813943e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7818956772115522e+00,    -1.0517056690816298e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    2.3612716357878600e+01,     1.3697673245119180e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.0893050124170991e+01,     5.4155706037919140e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.9133429958261363e+00,    -8.0224115038579057e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.4554503243242792e+01,     1.5396509923498092e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1431921714335349e+01,     1.7062437398131617e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1316077442983834e+01,     1.7338207415416276e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6206976734288897e+01,     2.3765959636270218e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.4148515913127016e+01,     3.4570217407030650e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    6.0460258849997196e+01,     7.3421440694349149e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5899106094928346e+00,     1.5699786774594260e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.2975432868138824e+01,     2.2233636619120818e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    3.7501809470954752e+00,    -8.1179784973329170e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    5.8169220185060681e+01,     6.7138364843893723e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.1592588230604090e+01,    -2.6207778765945015e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    2.1129665235780504e+01,     3.9535895418262779e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -1.1667907120688078e+01,     9.7522144774551343e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7937978010441363e+01,     3.3612169216290655e+04,    -4.1049408001561064e-04,    -2.0463680047014410e-07,     7.3968023326881881e-10, } , 
      {    3.4036305699119178e+00,     1.7326475443227038e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -4.5111695258366709e+01,     1.7921869951644643e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.3605406508930606e+00,     7.8140369609466006e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.0342569730789766e+00,     1.4807042139751917e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.3070657580157334e+01,     1.4136482413082957e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -5.3844604358322883e+00,     1.2438926915492188e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -8.9454861856872654e+00,     8.4319655757366763e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.2201296672501492e+01,    -8.4349667392465235e+03,     9.2743731943700382e-05,     1.0921223261429318e-07,    -5.2374421970263164e-10, } , 
      {    1.7850788396100615e+01,    -1.2222482061910478e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.1523551370173145e+01,     8.2914642871504657e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    6.3233997787923855e+00,     2.3694533166247693e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    2.5869038880556449e+01,     2.4208109982017049e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    7.9508510265406187e+01,    -5.3907488046582239e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -7.0407269129963259e+00,    -1.1479418137766483e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    8.3354835744234972e+00,     2.1225073659622482e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -8.7943264658348390e+00,     7.2489735433747665e+03,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    7.6783492928469226e+00,     1.0106512523783465e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6414687343282026e+01,     2.5075101263224049e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.6424217569226940e+01,     1.5744431823411438e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.7251417518614520e+00,     1.7302551180940009e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.4287608255772806e+01,    -8.6390137257473762e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.0589393528259485e+00,     2.4038577254503551e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.9580739933747964e+00,     8.5070538172137676e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.4393291350288404e+00,     8.3598784977694340e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5359371188563600e+01,     8.4733078677656158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9439364120559723e+01,     2.7841995794589413e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.8621523839728560e+01,     8.4455331063771046e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.6452991016093037e+00,     7.1742211550843303e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.0656889986029420e+01,     9.1076221188778072e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.2116740283526131e+00,     9.1393260035492676e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8047563049214787e+01,     5.8369252188846576e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -1.8833647044705470e+00,    -1.2165583780754650e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -8.3884772855208567e-02,    -8.8084534432438795e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.3689215086887980e+00,     8.1206272811269218e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -9.0522743713177101e-01,     6.9143419001919319e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8288588605811429e+02,     8.4592665894790425e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7963007557130010e+02,    -8.4076657255041591e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.3340401508691890e+00,    -1.2090940108777686e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.7815465907860478e+01,     1.5361915388393159e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.8308658390836818e+00,     2.2906473862147890e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.4372330125036655e+01,     8.4338594037540424e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -7.9749269846326385e+00,     8.4330729112291592e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.1031797325113637e+01,     8.2279185892358255e+03,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.6224988626817158e+01,     2.6224605301599611e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3685367766786690e+01,     9.9667929019059211e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6481887530238462e+00,     7.7779361005900309e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1333007842196409e+01,     1.7670301110702112e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0072955751040425e+01,     2.5492686445240150e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.5729874070313350e+01,     2.3912942069006716e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.4754805784937918e+01,     1.2162747870367923e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.9476654312030192e+01,     1.4338893026136970e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.6537144781556656e+00,    -7.1151881865754231e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    6.2679482960656578e+00,     1.5526872320001816e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.4035549575247154e+01,     9.9279977752514278e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0884830815468419e+01,     3.2868785107328527e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.1193499286378042e+01,     7.3911579235644581e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.5055326764397154e+00,     1.6759554558174968e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3389649792976801e+00,     1.3655228510905084e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -4.1845346506648795e+00,     1.4949357382952672e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -8.5306794864697153e+00,     8.4088582584366050e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -1.1786489973283935e+01,    -8.4580740565465931e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.3635922466987239e+01,     1.6630605003271176e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.0532690744602380e+01,     8.4347667440389760e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.9301180805779648e+01,     1.5290925874245973e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3138431540614505e+01,    -8.1337451923251883e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.9362786253467856e+00,     2.4382359328364648e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -3.7210814760899979e+01,     1.0947525385593553e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.2866087582696668e+01,    -8.9930816370655630e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7242075466476390e+01,     1.6735849191488491e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.8434937968541256e+01,     1.6024817508543634e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.3740358258611653e+00,    -9.1691514993453748e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -1.3520035631937127e+01,     1.7390793799535364e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.5847610804785385e+01,     1.6065123680604955e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0471840259658709e+00,     6.9787762500057888e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.6767913714297819e+01,     1.5019123386230562e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8959922951872281e+00,     2.5385095348312316e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.4885663080782805e+01,    -5.6072867053595473e+01,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.5031469716232071e+01,     3.0458364456472282e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.0221326728380085e+00,     8.2399161999792959e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    6.6236409827211951e+00,    -9.5033524932554042e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8728676899247290e+01,     1.6763261109509258e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -7.8616465521646326e+00,     3.5256987393801583e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6121143307759187e+01,     1.6023627326203483e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.6648797571358287e+00,     1.6685004993237311e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.2313716275388797e+00,    -1.8399155532003315e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -9.5258687938570290e+00,     2.3345835319644149e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1088384771175093e+01,     7.3718066954376864e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -1.0669345876650311e+01,    -1.6592423018276659e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -6.2503490041874308e+01,     1.5629289623683215e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.3799228894136100e+01,     8.4292084018338792e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.0543418407321873e+01,    -8.4377239131493225e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.1248087824483314e+01,     1.5969556687129943e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.4827031233225885e+00,     5.2892983999477010e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.2777442642457480e+01,     3.0509006630575001e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -2.0161749275819516e+00,    -1.6605577213589939e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    1.0422154191008238e+01,    -1.2013790839314565e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    3.3723838430811483e+01,     8.0957060092123047e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.4124926182565559e+01,     8.0811655148531427e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2222530835136180e+01,     1.7183282452556541e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7001162659997988e+01,     1.6808515279774136e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.2524886703980860e+00,    -2.5264350056071202e+04,    -1.7874949551903517e-04,    -4.2328437695477703e-07,     2.1522333745495625e-09, } , 
      {    9.6022854299382168e+00,     2.4011165336482791e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -3.8373200587800504e+01,     8.4337795303512212e+03,    -5.3187191046337001e-04,    -4.1895159708670113e-07,     1.8010828253219257e-09, } , 
      {   -4.1629011074614716e+01,    -8.4331527846319805e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -9.8507991130288115e+00,     1.6788465972741607e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6176582264151122e+01,     6.9449948225506287e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.0389400296945865e-01,     3.2286862928640981e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    3.4435130837459262e+01,    -8.1494776763244845e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -1.1661909880169226e+01,     9.6138234485317525e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.5311868505742137e+00,     5.6147565147303676e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.2358361422430146e+00,    -1.6933009847653746e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -3.2998478566072038e+01,     1.7859513144614488e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.4343625674857108e+01,     9.1171306727234860e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.2441830923963849e+01,     7.1077628792389751e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.2261671842619009e+01,     2.2716476772285841e+04,    -4.3912817851966964e-04,    -3.0973936447240789e-07,     1.2773386056192941e-09, } , 
      {    1.7312811279707944e+01,     1.4121845809951144e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.9945557232070993e+01,     3.2228470628685980e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.5862521319574201e+00,    -8.6741048927441625e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.4004116314137640e+00,     1.0931277386534097e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.3122585997872058e+01,     7.1108133857414105e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.1560082430438072e+00,     1.2325105715730541e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.4123303245855094e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0433723827999913e+01,     8.1332465195449076e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.4307796799058590e+00,     1.1999006607145775e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.7986238856192699e+01,     5.4722901826019288e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    2.5528161158834081e+01,     1.1903092029818166e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.8165349943174114e+00,     2.3278048807576124e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    3.6094190251873279e+01,     5.3438100830473013e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.4273731333353226e+00,     2.7085052048552075e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3110686707112286e+01,     9.4985655457870944e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8379390105442688e+01,     1.5450598795531634e+04,     2.1194455164410975e-04,     3.9098996802870867e-07,    -1.9637378153341751e-09, } , 
      {    9.1332120849866740e+00,     7.0982675057678298e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1344094838478583e+01,     1.6662298621989339e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    7.7267824156343980e+00,     8.5547892520606874e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6341179828414610e+01,     2.1840263333469757e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    3.6236864841361239e+01,     1.3440611973127094e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    9.1848053217770698e+00,     1.6412907445979028e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8302996069599317e+01,    -4.9300588577098752e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.3158950623072556e+01,     2.3347201516960977e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.1692733668027806e+01,     1.2521410208761909e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.1085794434944802e+01,    -1.8456333343950992e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.2557211691518297e-01,     7.9602630944703060e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.7470197928247195e+01,     3.2385093965425156e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.0742608375199605e+01,     2.3906390603614636e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.1070582578336637e+01,     1.0923218601515209e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2618074149700174e+01,     2.3747492787719973e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.5921445153523564e+01,     1.6451619905691783e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.5529608986076177e+01,    -1.6588690743775322e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.7967323154862427e+01,     3.1127926505801584e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.3616286636772720e+01,     3.0392241163947812e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    4.4657983119340535e+01,     6.0457139974459169e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1799655066119524e+01,     2.4898160287649764e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.1106703830568012e+01,     9.9743984411245729e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1235034848568326e+01,     2.4820787450431962e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.9938891693181446e+00,    -4.5954312824383703e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.6319010427840510e+01,     8.4027979756177483e+03,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {   -4.9186663798014214e+00,    -4.7127510043307740e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.4443160569983791e+00,    -1.1688967603502784e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.7004401598498280e+01,     2.3986529728728885e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5686679880307866e+01,     1.4487644307788103e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.0332743613217822e+01,     1.5638668867309400e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.2812904849474648e+01,     1.5656389178390491e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.0841521491795447e+01,     1.7485636999244845e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.2903561116423411e+01,    -7.5852910682331803e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.1943764374916398e+01,    -1.1103708036073840e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3342511721449526e+01,     3.1050597901947171e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.7966047268084324e+00,     1.6621726614770632e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.0030264225739337e+01,     3.6628142538719709e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.0142910237750776e+01,     1.6791654533126570e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.2168214817015674e+01,     2.9973483044638560e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.2611150250263647e+01,     5.7571129885213177e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8121725063985030e+00,     2.7622143990410393e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4160330572041140e+01,     8.4107053567588046e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.1701959835249582e+01,     1.5572966291095812e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.0933770523887468e+01,    -1.6798240209511623e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.9227656670724365e+00,     8.3253857248473978e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0557699798555014e+01,     2.3136062196750139e+04,     4.5470021253962852e-04,     8.1961956126182262e-07,    -4.0865429994403895e-09, } , 
      {    1.1894413084776943e+01,     9.0552147869380115e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.5157148824144508e+01,     9.0809614122931744e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.2434786523243147e+01,     9.0617804560878685e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.2674440335684078e+00,     2.5075429895598827e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.3487200189218091e+00,     2.2437917996156608e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.4912885548382304e+00,    -1.7877497486847031e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -2.8764042530217623e+01,     8.3252457146056222e+03,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {    1.1671917105005274e+01,    -2.2111729761683860e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.0551588360721764e+00,    -1.7876074795913279e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -2.0525471395502279e+01,    -7.1734983391439710e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5912041878109280e+00,     9.3560200841109599e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.1618480913331739e+00,     5.8400537239811201e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1499593375542672e+01,     7.1203087592125576e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.1730726235683903e+01,     7.3361105949921639e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4912556448933449e+01,     1.6520312011163362e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3754545126472109e+01,    -1.8465278824527784e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.8583590148257709e+01,     3.2288912363872849e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    8.9152248389804933e-01,    -1.8445575149354285e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -2.0926925261681113e+01,     2.4971404301044764e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.1403458519523545e+01,     1.5307467674629237e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8188231689710513e+01,     4.0041344128078345e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    7.9094409221915463e+00,     1.6763321043669283e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3275138941488915e+01,     1.4541912347055322e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.7236827617576481e+01,    -6.1870468426164643e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.1095941943279488e+01,    -1.4293050738276311e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {    4.3654716301745623e+01,     8.4131912413077953e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.0398905814931396e+01,    -8.4537410736754082e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    4.9975190841897955e+01,    -1.4286004419114811e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {   -1.0655520290570156e+01,    -5.9356549284491057e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6233655440477925e+01,     1.5125870977693661e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.8518959136196287e+00,     1.5419930613657783e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.6804833605549954e+00,     2.0361479459224385e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.1960749208776402e+01,     8.8851547231271434e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.2790847294622889e+00,     9.6260916036491326e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6435751200739407e+01,     2.3965911166217906e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.9837901394528522e+01,     1.6083147888806756e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -7.2908959303325602e+00,     1.7610332671373471e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.1218145147051587e+01,     3.8339911828376222e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    4.4408581675389399e+01,     1.5036494434570071e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.1155497046221310e+00,    -3.6842833015314409e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.3573915267876384e+01,    -1.0430833741478153e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.2885416908456058e+01,     8.5267509025030231e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    5.5355344065103127e+00,     1.7486025141038383e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9825641694102622e+01,     1.3620291024512562e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.9135789724763271e+01,     2.9290766799995981e+04,    -2.8683577075734346e-04,    -5.9020490317753199e-08,     4.1354606998643315e-11, } , 
      {    1.9340391924706566e+01,     1.8476228071712983e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.4957774600647360e+00,    -1.6768057636200279e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.0768353902685407e+01,     9.2724677183498570e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5807619697337184e+01,     2.2356143178392722e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.2098404638704103e+00,    -1.6512793214480833e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    1.6521944124661836e+00,     8.3582767189550104e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.3500879716897995e+01,    -6.7216968678350822e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4246060978468041e+01,     1.6093509614015213e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.5321422959685080e+01,    -4.9592387500345922e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -4.0633198179686048e+00,     1.0335204193439807e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.6893205565733940e+00,    -1.6561950953832077e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.7496440828394357e+00,     1.0619742380190343e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.7133863447689833e+01,     8.4330087153015156e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.3878052960875607e+01,    -8.4339235996816824e+03,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    4.0077746278716589e+01,    -1.6747237143755650e+04,    -1.7874949551903517e-04,    -4.2328437695477703e-07,     2.1522333745495625e-09, } , 
      {    1.0316931805002131e+01,     8.2431043615113304e+03,     3.9950555821570275e-04,     5.4318704212574410e-07,    -2.6171212133654968e-09, } , 
      {    3.4405854732493530e+00,    -1.7581980948956185e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.2960650747044930e+01,     8.2702743894143860e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.2336943781904004e+01,     1.2453847353851586e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.2483789083399106e+00,    -9.2936632770808028e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -3.7562592851138721e-01,     8.8402266660573696e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3521644434921477e+01,     4.0556051035708166e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -1.8467596264215860e+01,     8.4597745481181591e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0770504757684257e+01,     2.4505201237648660e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.7019084775337173e+01,     2.9464267849328680e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -2.0281442657287620e+01,     1.6784449021713481e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    8.7544454716897544e+00,     7.0563285110179122e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3045239616874795e+01,     2.5210293712432234e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    4.9375416165844264e+01,     7.0984714212333402e+03,    -5.2523741015816222e-04,    -6.2257596114658415e-07,     2.9213417982617303e-09, } , 
      {   -2.3600330164646898e+01,    -8.0093001539773086e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.0160506790625995e+01,     3.7739990176740946e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    1.6141511327507587e+01,    -1.0764894984949880e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -3.9298794389818124e+00,    -4.0945030068694405e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -9.6479979652351915e+00,     9.2816412467500213e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.3223404179825053e+01,     1.4300809092716294e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.0713353380786181e+01,     7.9935458491451864e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.8614199931531680e+01,    -7.0064973946420932e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.6993987839390826e+01,     5.9416904760138987e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.6661063185810953e+01,     7.6213482418578660e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.3098592988407649e+00,     7.0908118082010387e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4864690623586736e+01,     8.4071577668650425e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4959750200234360e+01,     1.7136033727484690e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.4819214286734017e+01,     1.1779139102932895e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0461038769377993e+01,    -2.5028715856866052e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    6.2688974690834609e+00,     6.8713716848214535e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.6246126300426784e+00,    -1.6761054054720840e+04,    -6.4443822985857152e-04,    -9.0435369656099959e-07,     4.3613353938932741e-09, } , 
      {    1.3546877793875394e+01,     1.4082314927175057e+02,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    2.1051013646333878e+01,     3.8497646182716657e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    1.1603025665760308e+01,    -7.7844950561891656e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.3683830959431369e+01,     1.6950544858814181e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.1837279464802613e+01,    -7.1201048437470472e+03,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -6.6059416333952868e+01,     9.9323068425346119e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4481368012637463e+00,    -9.5356431972793980e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -5.8638263875586134e+01,     1.2635023559933234e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7650749993634303e+01,     2.2860880542741412e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    3.0475527860033651e+00,     7.4339159916918175e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6076878386235663e+01,     1.6623149305704384e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.3539742430875497e+00,    -3.3860133284955887e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.1757885255736689e+00,     1.0096144105501541e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.1453760094767276e+00,     1.0858802468128609e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.6193225701825149e-01,    -6.1909282605518297e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8739256581336949e+01,     2.4446793678178507e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0469509057535969e+01,     7.3717972196987757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9652159219018763e+01,    -7.5035531467936198e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.7324068657553219e+01,    -8.8499230768594280e+03,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    4.5973161206316409e+00,     1.3211850943679896e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0468360356018330e+01,     1.6763580273048803e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.5263242658647330e+01,     3.2911889549164538e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -6.1347815160334676e+01,     1.0928457098507044e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3549139700517852e+01,     1.0441946005473490e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6369331210038744e+00,    -1.0491871010483113e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    6.4009969928229182e-01,     1.6760734891181299e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1491977430444502e+01,     1.5334690146485267e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4109726533354770e+01,     3.0927764749884249e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    5.8697159120219524e+00,     1.4998557921547510e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.3219037240926056e+01,     2.2182642129060023e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    2.1132821558942265e+01,     2.2843558204267581e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.3486488485500183e+00,     1.4354865436355260e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.7530758805565956e+01,     2.9618199434059788e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    3.2585884833534749e+01,     2.0521515013506225e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0910946676632523e+01,     2.2862303233675164e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.1655918275207421e+01,    -3.4662030381984005e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.4869421231827388e+01,    -1.2011638929665260e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.1108529465729100e+01,     2.9775732476629237e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.3487136421027355e+01,    -6.1835607822196489e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.0579760693715695e+01,     2.7022172427748275e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -5.7649862282909767e+00,    -2.3309033463018977e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.7754043232360431e+01,    -7.3626236912986114e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.0323830339688534e+01,     3.3155388161296680e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {   -2.5633799900828258e+00,     7.8574163241903725e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.2270963019249876e+00,    -3.5656234476268708e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -5.8530396720638109e+01,     1.7253747464701188e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.9501256250112419e+01,     4.1654963122824302e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3075725019900855e+00,     1.6127291041470377e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.8399059693976860e+00,    -7.1081846050959857e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    8.7402020039191424e+00,     1.6449318705750371e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.0288243689409562e+00,    -2.5092991244960322e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    2.0705512650916774e+01,     3.2270419099174807e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    1.0222564457676013e+01,    -7.8750483504648428e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    8.0932647004895593e+00,     1.6895265804440489e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    8.7868505797300198e+00,    -9.1731074310044296e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.7695553431465328e+01,     3.8891019468367725e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    1.4583110426491279e+01,    -6.3701455317576474e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.8211936921676944e+01,     1.4558637613201096e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.9327655773245937e+01,     3.6545826985230251e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.9637433952029703e+01,     4.7454645192385651e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {    3.6463456406922603e+01,     2.2607160448532384e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.6821516129391023e+00,     8.4330703234515895e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.7323412183111685e+01,     8.6298432961028611e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5193139423278211e+01,     2.2504988739604320e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.4109772313473554e-02,    -1.6745296835467485e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    9.6038499287494652e+00,     7.5226998766972497e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0187026461933335e+01,     3.2527080576251134e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    5.9379493134981107e+00,     1.3664390490444299e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3863217763863988e+01,     1.6765970873928185e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.9110046857057561e+01,     2.6693585807869149e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    8.1187309746179452e+01,     1.6133843283518781e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2319518538991415e+01,    -9.8351964435051304e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    5.4170015092747065e+01,    -1.0601436845416074e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.5282852042100323e+00,     1.6758344290301917e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.6360884682771726e+00,     1.5828364133172776e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.1765845361777913e+01,     5.9379925943105145e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.9055771324078421e+01,     1.6759504066215566e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.0451285141231921e+02,     8.4341216604067176e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0125704092550501e+02,    -8.4328106545764877e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    2.1312736822968951e+01,     1.6671487409904657e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.2077896765211051e+01,    -6.3286758981420935e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.8923302632002947e+00,     7.0797911814786748e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.2040879529877806e+00,     1.5770387898079920e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.1512787194965370e+01,     1.3979530566750391e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1278160138293231e+01,     6.0476286704362810e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    8.5482471872780295e+00,    -7.9978472915347929e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.8724399467804602e+01,     4.6696989186409941e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {    2.6527927294063787e+01,     5.4061668900774584e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    7.1086252475212248e+01,     6.0915623737865808e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.4067090776146287e+01,     2.4445268424927290e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.3744474739219614e-01,    -1.6499639019167553e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {   -8.8230616357121185e+00,     1.0538256416841108e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.8080426904063831e+00,     7.3162077659103115e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.5385738948614376e+01,     3.3341988170349825e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {   -5.8523678619597888e+01,     7.8267340751772390e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.3681482952979458e+00,     2.2621389500113291e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    5.9845335559006458e+01,     2.4135480055065043e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.5785526027163336e+01,    -7.4171881629652808e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0738569309186989e+01,     2.4317426305770514e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.0634184349537879e+01,    -1.2716610438775713e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.4998108189884476e+01,     7.1099927644063437e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.1614770249276944e+01,     9.9262733680918609e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.4672546522470885e+01,     1.4830416299030985e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8659023228700121e+01,     3.9429957223840400e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -1.0245368574351577e+01,     1.0145235447148119e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.5917905394904707e+01,     7.1261488791377615e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0956846511872804e+01,     7.3214674305426797e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.9391447679210977e+01,     1.5817882615608367e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    6.6424514573582618e+00,     6.2205156333259956e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3618993502611053e+01,     9.0493222230598349e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0272796340735113e+00,    -8.6933169851672683e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.4158393001271015e+00,     1.4502247541226143e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {   -2.3235308567358608e+01,     1.8702527823110436e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.5121814801188705e+01,     1.6847820860353626e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0378111737165153e+01,     7.1103916598844016e+03,    -5.5832899822007905e-04,    -5.9151709988682344e-07,     2.7173322012508378e-09, } , 
      {   -6.6663095131077540e+01,     7.8843924810003591e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.5598369095286518e+01,     2.0581033590577645e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.1179699752851576e+01,     2.5166100067809213e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.4310379929969599e+01,     1.7409652836916626e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.7117983774518180e+01,     1.2174789904942068e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.1097608248476512e+00,    -6.3749637296086075e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.0250990099169293e+01,     6.4813395769534518e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -8.4412125759367115e+01,     2.3347814810854012e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.6456809523292733e+01,     1.6146469235487497e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.1133218478555317e+01,     2.4046110225492179e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -9.3274181945012309e+00,     6.5496726529162315e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.9817352554715917e+01,    -2.7304499019002460e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.3485798252265507e+01,     2.3973375533415605e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -8.6610065400899231e+01,     8.5960618686477010e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.3167297378955141e+01,     2.2941221692004114e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.6645796942914309e+01,     1.5026699604373460e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8005561602905534e+01,     1.4007882289167675e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -6.8080435415416076e+01,     7.1085835005740437e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.0811101059044333e+01,     2.5612738676059202e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.0994642028025899e+01,     5.6993351805053908e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    1.5242708272557307e+01,     6.6812248694345544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1898053763007788e+01,     1.5017143895020701e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.0377915757665619e+01,     4.0620485385522021e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -1.0546795991603851e+01,     7.4340035639026974e+02,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5688941916794263e+00,     7.9073302813741266e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.1899338628227472e+01,     9.7430409490549682e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6933748429931512e+01,     5.3836249849452133e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.9209466229196899e+00,     9.0268288535277534e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.2077047336211510e+00,    -9.7833813321092621e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -7.1657996998159433e+01,     8.4330367851078699e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -7.4913807484973660e+01,    -8.4338955298753335e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.4563971272746342e+01,     1.3108312349470899e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.2241855477696593e+00,    -3.4233543542516031e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.7394229972475763e+00,     2.5090625349580790e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.1080198344703650e+01,     2.5091072663896208e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.6051767601198904e+01,     1.9026983395746403e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.4804198674776373e+01,     8.2576828596953055e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.8060009161590592e+01,    -8.6092494552878979e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.3701917341541566e+01,     2.3629600143653555e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.4367256130407711e+01,     3.1766775786481419e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    3.7042249369354030e+00,     2.3979065361531186e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -2.7741806405726877e-01,    -9.5169431216177327e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.9857113254460160e+01,     1.6417838478236623e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1057560369430922e+01,     1.6702639005043635e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.2070017981385135e+01,    -8.5519444607842761e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.2825274041176328e+01,     2.3836535036588823e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.0209720505018986e+01,     8.4062361954285207e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    3.6086755408121512e+01,     3.1020548748993355e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {   -5.1349864941805308e+01,     1.5669104525581128e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.3542974808092357e+01,     2.4493194344051652e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -4.9029268901357188e+00,     4.8962774302547141e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3272807364359387e+01,     3.0497541377972339e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    7.6447525708493401e+01,     1.6764811098014532e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.7674461982011751e+00,    -9.3565789349611150e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.4910811062749723e+01,    -6.5765817519126358e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.2776718837546372e+00,     6.5758981151446696e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.0691682809728306e+01,     1.6761175920198832e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -5.8658602397316130e+01,    -1.6763139244031270e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.2533909098356094e+01,     7.8466342241592514e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8566635298969857e+01,    -5.1135303616665606e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.5140196375054128e+01,     1.7325532144566456e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.7947861563373085e+01,    -8.2526310471687721e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.7644260470487652e+00,     2.2246402214361013e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.2531584437765243e+01,     1.5778040905334350e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5376956386292328e+01,     1.6564082844720324e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9594006069257333e+01,     7.7099867403618036e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.2628145657802339e+00,    -9.8635634254375709e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.5604025705594797e+01,     1.7894321152540586e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0411013226647569e+01,     8.7795877885311620e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.7737527873685906e+01,     2.4039498825940074e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.5319515971966036e+00,     6.0791431089430025e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6452819674995602e+01,     3.0940918945197529e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    1.3056825778772687e+01,     8.0267056489258312e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.5647602975577382e+00,     1.2503361265306045e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.2363964185195035e+00,     1.4337944755547667e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.0218061428421926e+01,     1.4341399941304400e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.8285510393776455e+01,     1.6529301645170330e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1978181870316646e+00,     7.7095985985682673e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.6221072835608403e+00,     1.7074996458479731e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8088152544414868e+01,    -2.0463369299385840e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -8.2830384645642745e+00,    -2.5074849498514501e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -6.8065081232654983e+00,     1.2045905945799201e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.3480795074834315e+00,     7.8691482963796125e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.2121804937106120e+00,    -2.5235964122660942e+04,    -1.7874949551903517e-04,    -4.2328437695477703e-07,     2.1522333745495625e-09, } , 
      {   -1.7841299692931088e+01,     1.5788549231274863e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -8.2120855362702727e+00,     1.0362488378559967e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.7742637342503489e+00,     8.8683736779815335e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3106878996726090e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9475495836613106e+01,     9.2641843555911564e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1448788212461372e+01,     3.3497092692374077e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.1629486199937809e+01,     6.6501971203337125e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.0578996549330270e+01,    -1.6774363506839029e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    8.7365740099370655e+00,     7.7118585679194830e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.0544028793652046e+01,     8.7101777318148343e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.7690869437501803e+01,     8.3763782588280192e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.7867819245056818e+01,     2.2668963103279584e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -6.0027288526515498e+01,     1.1225004915717636e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0589174426634488e+01,     2.4873596075326652e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -1.3327225758272640e+01,     1.6115801053613646e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -8.5226889201122198e+00,    -3.1971273790161813e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -6.4241355158798868e+01,     2.3363454890716283e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.7443005642689897e+01,     2.4916047370858418e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.5158532638099402e+01,     2.6124796799895019e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -1.3475809158646904e+01,    -5.4313002702888334e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7644548306431169e+01,     1.5458643073370535e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.5115824586111497e+01,     2.0030278266544221e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -4.8507141434550984e+00,     2.3154842351622201e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4205018577760015e+01,     1.8142725679091411e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.2817067496089383e+00,    -1.8034520495720586e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    4.6576705154414810e-02,     1.2798166457730016e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -9.9755721512947204e-02,    -8.4360691814316833e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.9870921794938106e+00,     1.5615889833616606e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.3381059359784508e+00,     2.2649964879688287e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -1.4950208672396350e+01,     2.5159541112210078e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.2550442078548762e+00,     1.9968458489531396e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.7112373041625343e+01,     5.9920297409089062e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.9706177677964813e+01,     2.5712597636184913e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.2123061689994854e+01,     1.0405067487556503e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    1.7674772683285749e+01,     2.5168845893630925e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.8355038987456069e+01,     1.8739881668997976e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.1936115077070415e+00,     8.5032960013503580e+03,    -4.6568873433953641e-04,    -4.8106931960622267e-07,     2.2091020193437112e-09, } , 
      {    1.1139566610495473e+01,     1.1732342603019737e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5269825924293093e+01,     1.5605580552361118e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.3811766699588489e+01,     2.2654119965255693e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    4.3043026457816955e+00,     1.5640050856280888e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4420804654734194e+01,     1.5344679552587071e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.7823059578851772e+01,    -1.3645184080556994e+03,    -9.2640263880542614e-05,    -1.1044778028060079e-07,     5.0823018190712641e-10, } , 
      {    3.8880335841749951e+00,     8.4404697389710363e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9005524809965628e+01,     7.6780771404643228e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.4924616093412141e+01,    -7.1147178375280673e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.6861381716106258e+00,     1.4537967535626531e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.1222025131592424e+01,     5.9972894055412744e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    7.9923494920147569e+00,    -8.9737562785325929e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4315908557007919e+01,    -6.1866740278009620e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -7.9771890628311404e+00,     2.3967360291932848e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    3.5168459399861561e+01,     2.3985080603013943e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.5709851837428126e+01,     3.8970296956205471e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -1.0935597157766100e+01,     8.2603126920114901e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.4191407644580323e+01,    -8.6066196229717116e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.2439145773807962e+01,     7.1103840133576814e+03,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -3.9977202483620147e+01,    -7.1081922516227078e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.7131431827092783e+01,     1.4953959782835493e+04,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {   -6.1762154340756981e+00,    -1.1677234361162277e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4196689181340453e+01,     3.0978104014779259e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {   -1.9607997388001781e+01,     9.2172068969361644e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6785041203482272e+01,     1.8955990090333413e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.9150195050241450e+01,    -8.2924459490666832e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.1186210662130254e+00,     5.9923925842766503e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4934850977869447e+01,     2.4291191235709148e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.8457189307569010e+01,     7.2780562939275205e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1610603998778370e+00,    -1.6742979388097930e+04,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    6.0641178740599262e+00,     8.2511391636113258e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1676919932968353e+01,     1.7899328046512401e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -5.3530553577422957e+00,    -4.4875239765624283e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -5.0646630857571211e+01,     4.2663005161589990e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.3222129564657209e+01,     2.9243227407822666e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.2189814225116805e+01,     1.6557523889121192e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3008811591290801e+01,     1.4329909953447674e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.0446437059675951e+01,     8.4263454366210735e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7190626572861728e+01,    -8.4405868783621281e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    9.3564419526995106e+00,     5.7751207013869835e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.5077853050270576e+01,     1.0502899725238629e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.6764147089900987e+01,     1.0452046848391521e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.1545226339799299e+00,     1.5500336364143801e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.3389675412411984e+01,     1.9562446929243924e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0042548024575112e+01,     2.4061286842055579e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7132175070298494e+01,     6.5030218008893789e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.5309391234556697e+00,    -4.2680239714814797e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5361706760781164e+00,    -1.0432352433411699e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.8270771943883524e+01,     3.0983363679411625e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    1.0489255572548188e+01,     2.3933560631517692e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.0862293824428679e+00,    -9.3128081014041509e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    8.8015493915245422e+00,     1.6440129917835053e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.0074159658110318e+01,     1.0968272000404955e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.5917204666471640e-01,    -1.0256584395130558e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.5944737309703470e+01,     3.3970295755395375e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    3.2657641723804893e+01,     3.1016782263499033e+04,    -2.8683577075734346e-04,    -5.9020490317753199e-08,     4.1354606998643315e-11, } , 
      {   -9.0023798151730318e+01,    -1.8478089287526386e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.7482770499864904e+00,     1.7400774448416083e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.4695675879699799e+01,     2.2673372330449842e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.6540798496386639e+01,    -5.8846587477152934e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.3340886228512989e+00,    -1.1671455403889415e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.5377848123489093e+00,    -8.8330366070448126e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.4129325020055692e+00,     1.3400599949813852e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.4139487005257635e+01,     1.5636039034993215e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.6619648241514462e+01,     1.5659019010706672e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.4169772957784090e+01,     1.5648233654766094e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.0145325414019442e+00,     1.7778210991600149e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.1976091538504747e+01,     6.5542750527990538e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    6.0033175699869332e+00,    -1.7300293667730482e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    3.9393850610054599e+01,    -9.9258082963128663e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.9486985511866138e+01,     2.3282651207458945e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.8189079957895519e+01,     1.5655007189419002e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.6546309900699320e+01,     1.5494302889701048e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.4740991215596978e+01,     1.7123827802760712e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -5.3586056203803984e+00,     1.7277053935909611e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -7.9652483480975036e+01,     1.2939004348347225e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.1447717465662794e+00,     1.2110340295613059e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.8055503521876780e+01,    -1.9544490507854243e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.7198814934211327e+01,     1.6224030972238528e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0497852534561527e+01,     1.9178194017119722e+01,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    5.9736489970380555e+00,    -1.6697723232301192e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {   -1.1016453722969533e+02,     8.4332391856610175e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1342034771650955e+02,    -8.4336931293221842e+03,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    6.3536296416849209e+00,     6.7680822741930760e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.5667951128690836e+01,     1.6486530583708201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2520140418175302e+01,     2.2024446182893324e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.7070129787188669e+01,     3.9219253772288772e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -1.7062698134005657e+01,     8.7374448783157368e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.0318508620819877e+01,    -8.1294874366674649e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0707290092407241e+01,     2.4381860655584373e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    5.3830426856893396e+00,     2.1853381244668024e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.2437282362639465e+01,     1.5950411862806595e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2223187065626227e+01,     1.9197835661285098e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.8042596790320751e+01,     8.9942108876268230e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1645193649916735e+01,    -7.8727214273563768e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    2.4552291817826529e+01,     2.3595818716198395e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -6.8132058075734236e+01,     1.0594395855035316e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6747286165110740e+01,     2.2357796080898173e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    7.9555752292688506e+00,     1.3805266083669019e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.4744509505291159e+00,     1.7544108726522247e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.5959118304291922e+00,    -1.8411519215796659e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1254284936888446e+01,     1.4013311994205549e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.3224214652573213e+01,     3.1878109474948695e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.2814179779843784e+01,     1.1747987507793918e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.5177851572365064e+00,    -8.7025342002993184e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    8.5587789788391149e+00,     1.4521425735243263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6581746214538185e+01,     6.2479951320335040e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.8010782804996106e+00,     8.4242678938373738e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5452677936853880e+00,    -8.4426644211458297e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    4.0266354120450849e+00,     3.8402044978248676e+04,    -4.1484810151031068e-04,    -6.9710922874427147e-08,    -1.7501773888054540e-11, } , 
      {    5.5330151116690971e+01,     3.8406647378131500e+04,    -7.1943291703496304e-04,    -5.7114867118373656e-07,     2.4544662233532473e-09, } , 
      {    3.4237669552329386e+00,    -1.6926450892054614e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.2869530847496584e+00,     1.5794582705717619e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.0171430288323528e+01,     7.0193072277471765e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.5652066411910077e+00,     2.3687041177941268e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    5.8067246211973561e+01,     8.4311582439994654e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -4.5386657764389959e+01,     8.4357740709837362e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.0480956008422076e+01,    -9.3119042500219280e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1347081410960914e+01,     1.6139883471512261e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9470458644635158e+01,     3.5306297716377539e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.4287870571606561e+01,     6.6159672814268179e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9527770655842161e+01,    -4.6714278020244092e+02,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.8472932235108427e+01,     1.7686364411239858e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -7.1696485525922981e+01,     1.5646824390933796e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.3089612187969543e+01,    -2.9054743676624821e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.7390632829555713e+01,     1.2815364420057031e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5496862877588807e+01,     1.0321018744944413e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0888458744479539e+01,     3.9236507413307427e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4778049976477218e+01,     1.3729988301812387e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.2582149504211642e+01,     4.1155972687343448e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -1.2444565709509316e+01,    -8.3985554794751824e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.0798206707377584e+00,     1.5107135424024986e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.9600310832697559e+01,     1.9150936322631671e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.7019453813929040e+01,    -8.3063999850250166e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6774484640659949e+01,     6.5170289638482509e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    4.8254640250852226e+01,     8.6061209501914327e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -4.9775527787556904e+00,     5.6047671086285645e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8516047519579931e+01,     1.7299794994950204e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.2993123303375640e+00,     5.6173863470465512e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.4736868786175179e+01,    -5.9074339112963810e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3671555760334389e+01,     7.2166658892984269e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.2611286709420366e+01,     2.4075547242343669e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.9933580376524503e+01,     7.7206750367281447e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    7.2627397531868008e+00,    -1.6753279193614508e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    2.5099074957794819e+01,     3.9596526998467307e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    3.1613468724160141e+00,    -2.0492160650761948e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2491145021486474e+01,     1.5679168212083285e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4730746643121623e+01,     8.6475905838069848e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.6088785551306202e+01,     1.6092957567297799e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0287335026090933e+01,     4.7865087823779072e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.7987444565518999e+01,     1.8304378804131311e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.3375199030447916e+01,     8.6507038293882943e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.6631009517262136e+01,    -8.2162284855949092e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6833268421235928e+01,     1.4243168904520231e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.3581869162095554e+01,     4.0505454888471809e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8299230463228398e+01,     2.2632864228454862e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -6.3234148860730528e+00,     4.5032564605617263e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.5710192271442063e+00,     1.4527984690842397e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3349502939331239e+00,    -3.6550813713033049e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.5748043447390714e+00,     1.3427168387966947e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5926448466301112e+01,     3.3438728101117980e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    3.5964312213442255e+01,     1.5789530893191079e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1300966842905298e+01,    -2.2149946972555313e+03,     2.7812772776794338e-04,     3.2887224550918713e-07,    -1.5557186213123898e-09, } , 
      {   -7.0821699059192994e+01,     2.5090855720289226e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.6246854515879363e+01,     2.3923962695729078e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    7.7215682449831542e+00,     2.3876893652603118e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    7.8472435914304377e+00,    -9.5694246247345582e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.3115699234841312e+01,     1.2914431178454897e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4661309876569462e+01,     3.3517756208630970e+04,    -8.8115110525578645e-06,     2.7108730285774154e-07,    -1.4988499765182413e-09, } , 
      {    4.9352547283940940e+00,    -5.3384213343288025e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.8660545965753359e+00,     4.0633931929095066e+04,     3.2668788178666131e-04,     8.0892912870514867e-07,    -4.1453993803270866e-09, } , 
      {    4.6762728442332559e+01,     4.0633274664345525e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.3854448409808517e+01,     2.3663381571108715e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.6641328953259048e+01,     2.5090842293187772e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.2494906743586540e+01,     2.3236579173960567e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    5.6664614742137260e+01,     1.4968931579659749e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.5341828305609571e+01,     2.5265650642618581e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    4.8029323263110300e+00,     1.4546018374783107e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.1145313892079329e+01,     1.7531485375585515e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1038850246732544e+01,     8.2860498416277460e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.4793479336482568e+01,    -8.5010290124019193e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    2.2268347882428795e+01,     2.3254612857901277e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.7974221402751091e+01,     1.8714144519381720e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.6592206724119421e+01,     1.5034763466092832e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.5676613858053720e+01,     8.4303376523951392e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.9015674236362546e+01,     2.3355391028996910e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.9755147616417617e+01,     8.5797105239753037e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.3010958103231836e+01,    -8.2872217910078980e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0227670992050101e+01,     4.8998343137747339e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {    4.3961247759931066e+00,     7.7226393843976239e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.0002758735191868e+01,    -6.0109059895198197e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -1.4998379435234258e+02,     8.4307687866132983e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0923745291096957e+01,     4.5585172819754180e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1554485387087567e+01,     9.2501610192362714e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.1026233619486781e+01,     2.2230669730361718e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2591065920750268e+01,     1.6570632324580547e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.2200424165883287e+01,     6.0152973906981777e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1639669771166641e+01,     1.5839603655495692e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.9350518654607285e+00,     4.1761360923283019e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.3460694093901239e+01,    -9.5636367450451362e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.0257063841901360e+01,    -1.4830834161105275e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.2870229310196363e+01,     2.5799338685112351e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    5.4300150794750479e+00,     2.3200720203721710e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    3.2905367157075233e+01,     1.5621220632223387e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.2945956918036131e+01,     2.3976209925269588e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.0439382725822846e+00,    -1.0878507132893801e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8170770897708046e+01,     2.4744228702918659e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.4245227992642619e+01,     2.3976230969677203e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.8004121749649663e+01,     6.6410161017264909e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.3307087968735566e+01,     1.6020438765818522e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.8308909151968731e+00,     8.4092826658250524e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.2491957161734994e-01,    -8.4576496491581511e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.8706774556262449e+01,     2.2181469191766879e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.8644039256063035e+01,     2.5269246877335827e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -9.5408412988131062e+00,     8.6533336617044788e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.2796651785627329e+01,    -8.2135986532787228e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8248624829221491e+01,     2.3974078209251576e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -7.5299898481796363e+00,    -7.9296703501814836e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.4602062938935266e+01,     1.5526997070728436e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    9.2635766154963495e-01,     2.2044897026463619e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    5.5458626904096008e+01,     3.2130110236216766e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.2299304874453476e+00,     9.9107313398034294e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4244975180692983e-01,    -8.8515129313339658e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    3.6993165839587896e+01,     2.2328885507630734e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.3721278837867679e+01,     3.0429978523062022e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {   -3.2447577957351719e+00,     5.3330692335046633e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2302293315031939e+01,     2.5189064784007514e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    5.3941782305389083e+01,     5.7176862284169410e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5863359465443930e+01,    -1.3929043414659582e+03,    -9.2640263880542614e-05,    -1.1044778028060079e-07,     5.0823018190712641e-10, } , 
      {    1.9947260276729249e+01,     1.6957103814413313e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5454964077426533e+01,     2.1082607327494039e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {   -3.8145117109522086e-01,    -1.1863332101449723e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.3273697327592558e+01,     1.5437620527604844e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.8292103860891025e+01,     3.2069625256744293e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    4.3411410779311606e+00,     1.6730518392881710e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.6081786922130696e+01,     7.6961260839201859e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.1600437608093657e+01,     1.5438338586622443e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.3800641023310279e+01,     3.2489424904120515e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8184093524891377e+01,     2.2632854752715957e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -8.4631212855922389e+00,    -1.1872439107459471e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    8.1719762778891560e+00,    -9.6800919214808819e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    4.0522606506648007e+00,    -2.0806414349652823e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2718337115802468e+01,     3.1203401224030007e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.9317496112019327e+01,    -8.4895390245451945e+03,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    3.0774897919984770e+01,     2.4247924883914966e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.1759152011815488e+01,     8.4391473159762772e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5014962498629711e+01,    -8.4277849990069244e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -7.7443874546335252e+00,     4.1102354908687375e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.7667355118063632e+00,     1.6793796771348392e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9725285284680119e+01,    -4.5370378880168873e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.4921427886381387e+01,     2.5063619044675419e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.0352746660178891e+00,    -1.2903909677147640e+03,     3.3578458807475305e-05,     3.5787685423935058e-08,    -1.2014167831575411e-10, } , 
      {    7.2910825348382673e+00,     1.5576541347268438e+04,    -9.4433871945491906e-05,     2.5097252867261127e-08,    -1.7899805920245181e-10, } , 
      {    2.2304457785768896e+01,     4.3166717110237296e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2305815140171157e+01,     1.6904132933278066e+04,    -2.8780951224846043e-04,    -1.9271358368514484e-07,     6.8965715765193682e-10, } , 
      {    9.0500100638776715e+00,     3.7200618294864398e+01,    -1.5979718149549324e-04,    -1.8202315112847090e-07,     7.4851353853863457e-10, } , 
      {    2.6546735265540015e+01,     2.5829991607471049e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.6221087368826530e+00,     1.6112520089091657e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.4177929824480429e+01,     1.8253560694778414e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.2131819614671095e+01,     7.1917714257599127e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.9970556725705940e+01,     6.9384358669514968e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    6.6506492692536732e+00,     8.3286914246234501e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1204059094222487e+01,     2.0064059693999385e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.2168113572625053e+01,     8.3976071872527391e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6773573978792822e+00,    -8.9596020336090751e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.1727368053840635e+01,     8.8697963689152857e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4065356012868072e+01,     2.6852912886645154e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.2290086944501123e+01,     1.5851664043063523e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    7.2809990824580968e+00,     7.7948492911905696e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0632586769871196e+01,     2.3784695189938891e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.4435539793213284e+01,     3.0954996697479182e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    2.3793907118258957e+01,     2.6962653850676863e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -6.7751613355466418e+00,    -4.8805449462554170e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.5157488960889105e-01,     8.4318278158149005e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3032163337192491e+01,     8.4351044991683011e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.3065625258672299e+01,     1.4375181368759560e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9111262920398634e+01,     2.3906330669454608e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.0715887260452725e+01,     9.3369020681637139e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2532893325656721e+01,     1.7705933875841456e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.6728710602057713e+01,    -7.6316917341063254e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2365617205557719e+01,     1.4617597339422355e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    6.5825755450240857e+00,    -1.2374369760739667e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.3332573798450632e+01,     6.0058990955480040e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.1671816486023943e+00,     3.7172145211757515e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.2854013695911668e+00,     1.2216702810816939e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.5963568661330367e+01,     2.2026364669742630e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {   -5.1820545345382129e+01,     1.7587808708172917e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.5855495738742036e+01,    -1.0498044809417206e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.2337292760187403e+01,     2.6569632880983874e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.7042074697728200e+01,     2.2264451157816875e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1332782508505190e+01,     2.5676496775828338e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.2470822047720675e+00,     8.4231568762361130e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.0389209890874707e+00,     2.3929819781600247e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.6842670127382867e+01,     9.8800423772395698e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7619109972587658e+01,     3.1173362632024149e+04,    -7.1943291703496304e-04,    -5.7114867118373656e-07,     2.4544662233532473e-09, } , 
      {    1.0475616385641294e+01,    -8.8396050383828551e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    3.0379410005088332e+01,    -7.5933079067683429e+03,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    7.4615564938745882e+00,    -8.3352503802225838e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.0300183276995256e+01,     6.1523939275655221e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.3431213347077346e+01,     1.4027451753769274e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.2349241079405342e+00,    -1.5322421684979358e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2969174372095685e+01,     1.4276950331975389e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.9269631931204256e+00,    -9.4613441920903551e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.2894076886002033e+01,     1.5760064407372283e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3036973375683552e+01,     1.5332041362691261e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.8505839849309023e+01,    -8.3876177811509369e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.3039167777969261e+01,     2.2630712514494138e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    3.8798775177787824e+01,     1.3948378971611412e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.2254576189077596e+01,     9.1104756083517022e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.4085673346857014e+01,     2.3418553851882767e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.1011988646475800e+01,     8.4326902108014074e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -8.3314001988922044e+00,     8.4342421041817943e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.6962716868679280e+01,     2.3419258483798912e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    6.0044817480464046e+00,    -1.7301447653545805e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    2.0671407349746591e+01,     1.6189086083229138e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.1042229778446970e+01,     4.8386956233509394e+04,     3.2668788178666131e-04,     8.0892912870514867e-07,    -4.1453993803270866e-09, } , 
      {   -7.6342799639793967e+01,     7.7133228439262548e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1254174012551074e+01,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0585877820625903e+01,     3.3241935514827637e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3331260234163699e+01,     6.1589528831646548e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.1675746161226872e+00,     8.4437754387470886e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.4003351618089024e+00,     2.5193020715666571e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0682836367447145e+01,     8.4339107833719027e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    7.4270258806329217e+00,    -8.4330215316112990e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.4352999866412652e+01,     3.3057172384027661e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    4.4751531355760441e+01,     6.4806215179358487e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.4145348043306640e+00,     3.2367268679126995e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    8.7775954780558351e+01,     6.7596848607300362e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4053012310725306e+01,     2.6294495079618391e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.1452620729571830e+01,     3.7021163508412697e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.2657689482416423e+01,     1.6479971628109066e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.8052657757868683e+01,     3.1376088019447661e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3388045671556110e+01,    -1.8402327106097428e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2767953050163531e+01,    -9.2068888584595879e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -1.2007171120504598e+01,     2.9311725817897750e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.2458635716974387e+00,    -8.9727562289223551e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -9.6344198556767680e+00,     9.9694674088669799e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1042312139091237e-01,    -4.7565920193701431e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.6454194561271414e+01,     6.8144829400662238e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.2644544840176607e+01,     2.5164143512092905e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2791814920471367e+01,     2.1600510807631995e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3067536806545448e+00,    -4.3635710496771553e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.0513762761744484e+01,    -3.8251264375247410e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2747660632899553e+01,     1.5614458940861661e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5408470155334831e+01,     2.2563024908857202e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {   -1.0427216803061118e+01,     1.5673837413476504e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.8061005171495278e+01,     7.1791521873419242e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -9.6648975307759084e+00,     7.7355181747520364e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0544973200833473e+01,    -7.3964127351282787e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.2760852150773566e+01,     2.0547252163122481e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.1409531168670133e+01,     1.3255621777236862e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.2789882458464980e+01,     3.2185467166403108e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -1.5343268386656675e+01,     3.2788931646375000e+04,     5.4516346567282114e-04,     9.9753028034028185e-07,    -4.9733641849259522e-09, } , 
      {   -1.6130306043520136e+01,     1.6673115333246202e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.1255847732186037e+00,    -1.6744123898174341e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.9973231975355613e+01,     3.9502054056647597e+04,    -5.6714050927263691e-04,    -3.2042979702908184e-07,     1.2184822247325963e-09, } , 
      {   -8.2289664911306666e+00,     1.1652884976106214e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.0943441569256990e+01,     2.4028478199217712e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.1783932454870829e+01,     2.3964730459616665e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    3.8975202791901260e+01,     2.3987710435330126e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.7065912965511050e+00,    -9.5267648087788548e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {   -2.3417071558175788e+00,     1.5267625964355226e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.5794094487664943e+00,    -1.0762050070892090e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    4.4724941168952039e+00,    -8.2408373725628917e+03,    -7.3935897254962196e-04,    -9.4610299037743436e-07,     4.5064886100174693e-09, } , 
      {    2.1902739247976566e+00,     7.7276063114339340e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.8024585380269762e+01,     3.1939122849998792e+04,    -4.1049408001561064e-04,    -2.0463680047014410e-07,     7.3968023326881881e-10, } , 
      {   -1.4268888744723043e+01,     1.5565045729580228e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.8756929215252455e+01,     7.7871096289901961e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.9824246145739099e+00,     1.2896382234999033e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7836759833260572e+01,     1.6721817199060755e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4235675684436771e+01,     2.1727835593347179e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.2478972607806497e+01,     9.1834254694810061e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.7915779584473537e+00,    -2.5044448340865354e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    4.6997823412367001e+00,    -3.0616662313141819e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.3196667218954472e-01,    -8.4264625760121671e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4368905631339224e+01,     1.3379574704122133e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    2.1853499203988314e+01,     9.8074752988688233e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3243234975546748e+01,     4.1531240929141859e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5222084360137030e+01,     3.6419634601050375e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.8487684346107280e+01,     1.8510009499168140e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.2648165252817375e+01,     2.3112626247075295e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.9949945636452451e+01,     1.5666707216867066e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.4535708021225403e+01,     1.6058598935998794e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.2419198244801912e+01,     1.7203457606556538e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.9955711059420214e+01,     2.2254614211140597e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.0964444330534228e+01,     8.4990544931207787e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.9670819910833632e+00,     1.5365841741624241e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.9366689809658708e+00,    -1.5646425495455738e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.4891643889085632e+01,    -5.8243212399818221e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6685212417374036e+01,     9.0171499430110998e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.2657316357430325e+01,     2.8750997147361822e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.4880122262987499e+00,    -1.0133742485846546e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    6.0082385984269475e+00,     1.1605985637452786e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6472429072941047e+01,     1.8256689199874876e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.5028243572796740e+01,     9.0617670289864182e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.9144040308764971e+01,     1.7157609230215872e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -8.3892431834449624e+01,     9.2979657830463166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.2826414896433036e+00,     3.1326777696871713e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.6366196394748531e+00,     1.7954783028272581e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.2644567695800369e+01,     1.9166684066146125e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.3678134731429821e+00,    -1.1518404372229356e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.2653837206056249e+00,     1.8345929659805324e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -8.3722312462737385e+00,     8.4360807302926332e+03,    -6.8416431822569614e-04,    -6.6967047124135585e-07,     3.0370668239425765e-09, } , 
      {   -1.1628041733087958e+01,    -8.4308515846905684e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    3.8588550471968638e+01,     2.1769303397750540e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.2120969505410413e+01,    -3.0284947026287648e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5311820803673831e+01,     1.0038949668873145e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.3429685874971071e+01,     2.5381516120967899e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.8876002209232743e+00,    -1.8427541465413571e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    3.0752884249772844e+01,     4.0476036682160120e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.9106731486960634e+01,     7.0605862666756384e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.3806529745505518e+00,    -7.8919090971124097e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.5243153180260308e+01,     2.7355674820369852e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.2384542041315909e+01,     2.8559215207190637e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.4105280515521642e+01,    -1.6851199830983904e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6160296534484228e+01,     6.8920352010783481e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.1213262816739224e+01,     1.6839211170346895e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -1.4852103047332402e+01,     2.1451306424038012e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.4124820805186866e+00,    -7.0202706188754037e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    5.5162934846956908e+00,    -8.4142879634744786e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    9.1969528864826149e+00,     2.5875736960309150e+04,    -1.9201849612945083e-04,    -1.6035648835010902e-08,    -8.8284571330046551e-11, } , 
      {    2.7489260847826351e+01,     2.9080063348444353e+04,    -2.8683577075734346e-04,    -5.9020490317753199e-08,     4.1354606998643315e-11, } , 
      {    4.2272248922407968e+01,     7.8200790108054553e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1343814679555974e+01,    -7.6010235522324792e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -3.9539799121562638e+00,    -1.0189021540220238e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.1208880021364294e+01,     2.0720719001462214e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.3769692210239612e+01,     1.2846868323544575e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4927980542019265e+01,     3.1599052026039182e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    8.8306370718913936e+00,     2.3450084571355917e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    5.8127111691496133e+01,    -1.2470122693071899e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.0861976291127235e+01,     2.8235016764282394e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.7317033355317655e+01,     3.1202902551249728e+04,    -2.8683577075734346e-04,    -5.9020490317753199e-08,     4.1354606998643315e-11, } , 
      {    1.6646042533136917e+01,     5.4439042491916707e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    8.0827447923029734e+00,     6.5015991099556250e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.5189814133993416e+00,     2.4115905858357961e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    9.9882894914873752e+00,     7.4765719525669265e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    5.1494645701905206e+00,    -6.1592834647499899e+00,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.7314255321219090e+00,    -1.2510424813665959e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.6795237798482678e+01,     2.0962075375372533e+04,    -4.3912817851966964e-04,    -3.0973936447240789e-07,     1.2773386056192941e-09, } , 
      {    1.3394915153229411e+01,     1.3614861319474687e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.5318109517320794e+01,     1.7773114935906215e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -9.9449869160418523e+00,    -3.5901483487091691e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.5488750100284463e+00,    -2.3209139402000947e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.4082673411326606e+01,     1.6872992597802055e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0262975240741135e+01,     1.4783934822451845e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.9992538609632504e+01,     1.6133856710620232e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    9.4555235521562864e-02,    -1.1513199565221643e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.0209853123490916e+01,     3.1189571967364860e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    1.0381628032047834e+00,    -1.2268814587023132e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5408836120859483e+01,     1.6355397073549280e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.4556826386472551e+01,     2.4886215313744640e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.0848079700484448e+01,     7.4270580411124683e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0503568896488336e+00,     1.3641392210149030e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.1468637970492829e+01,     1.7216964117827891e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.1738024535406606e+01,     7.1273218164309037e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.7049956721918605e+01,    -8.0278259493806245e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -3.6362170990308673e-01,     7.3135543634811199e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.2599374230298189e+01,     1.0051751548228349e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1470211390617866e+01,     9.6877801276412774e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.0509565774093794e+01,    -1.4362323236500060e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.5027217477070618e+00,     1.4949391593945644e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.2957129709872468e+01,    -7.7556446609915984e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.9081699799836183e+00,    -8.4592033071078549e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.7006545846061854e+01,    -7.4252555139513370e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.4935665407956382e+01,    -1.2119251255641996e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.1131733213556380e+01,     1.6845770125946030e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.3162031996021689e+01,     3.1254717662645602e+04,    -7.1943291703496304e-04,    -5.7114867118373656e-07,     2.4544662233532473e-09, } , 
      {    7.4214459002265887e+00,     1.4101182293694248e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.5534336720410611e+01,     8.4034980218495766e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2278526233596388e+01,    -8.4634342931336269e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.4681089323123224e+00,     1.5206090022569986e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -2.2780012783891785e+01,     1.8001499755440469e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -4.7483314034147073e+00,    -3.7790342981885769e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.4588884985658204e+01,     1.6624704949185947e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.2663902176413295e+01,     6.9682679240652751e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0949621765973625e+01,    -7.5234712912203549e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.4337206075389957e+00,    -1.6848908106781404e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -7.6399642705591617e+01,     1.6879871319331549e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.8921019962258159e+00,    -7.7069427951770413e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.8370279703546286e+00,     1.8122384023520051e+04,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {    6.5505149638230815e+00,     3.2292292633678859e+04,    -3.9726088367124645e-05,     2.3468322531964382e-07,    -1.3242685699506974e-09, } , 
      {    1.0271374110926926e+01,     7.3181262527596182e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    2.0924911988737893e+01,     1.4088064382495984e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    3.4644400618502850e+01,     2.9636248377515654e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.3759603969936080e+01,     2.4271025639897365e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -1.7339618843651117e-01,    -6.0108511318951651e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.0477307588839704e+00,     2.5928706057520343e+04,    -1.9201849612945083e-04,    -1.6035648835010902e-08,    -8.8284571330046551e-11, } , 
      {    2.3972224729977690e+01,     1.4930328674916231e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {   -8.5732995538474555e+00,     1.7137129608352177e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.1227092795108877e+00,     2.4007398850988469e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.4008801724488535e+01,     2.0914535983199214e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.4541492414582891e+01,     1.3307452148147884e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -7.1526417164827834e+00,    -8.3836852218124441e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.5117696326714913e+00,     2.5719156591784042e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.1830352402856203e+01,    -9.1293365974474636e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -2.2703484631467262e+01,     1.6274870205494428e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0993094599194890e+00,     2.2255544819716523e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.2351979080548308e+00,    -1.1877641028999169e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4008627997755108e+00,     1.5792644138772386e+04,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {    3.2187468719218771e+01,     4.9026729071157606e+04,    -4.4080109861824687e-05,     3.6960910291536078e-07,    -2.0814505771075709e-09, } , 
      {    1.9690467832689095e+01,     2.2450537234574593e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    6.2964613415153181e+00,     2.4462541421692957e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    2.4715122495362593e+01,     1.8472122043985200e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -6.9693498035698639e+01,     8.0473840873979825e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.6512785287101970e+00,     1.1259864006413225e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.5891073963046942e+00,     8.4336069447646132e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4269695843888297e+01,     8.4333253702185884e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.1160111741438676e+00,     4.9864489296848242e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4818671166414854e+01,     8.4706780354494313e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1562860679600631e+01,    -8.3962542795337704e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -3.0482752953329271e+01,     8.4319322060429004e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.3738563440143480e+01,    -8.4350001089402995e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.5332849357625420e+01,     3.1443618871640159e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.7500145994129980e+01,     1.9420004365439388e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0935171462728100e+01,     1.6141328163638565e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.0200835509642864e+01,     1.0080801097334264e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.1188587431251831e+00,    -1.8661674094950806e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {    8.5686873004828797e+01,     1.5382622626927301e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.3256608487523716e+01,    -9.5792731202717860e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.9000452347968135e+00,     6.5159254364540429e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.9119018201984105e+01,     8.4196749696934639e+03,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -5.2374828688798331e+01,    -8.4472573452897414e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0458252392366113e+01,     8.6621938172450227e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.9918154340938941e+00,    -1.6767587287152925e+04,    -2.4493267164286882e-04,    -3.6116665443525549e-07,     1.7442141805277773e-09, } , 
      {    4.8039964609010568e+01,     1.6777078020474448e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.2969275778189584e+01,    -1.7015493140923470e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.1095165415128648e+01,     1.8001852071398542e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.9028194138873825e+01,     1.4305326789646741e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.9649559922797746e+01,    -1.2457116827598147e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.7645310105493920e+01,     3.0896036971257920e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    2.0036744104709854e+01,    -6.6134764249491063e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    7.8039439375111606e+00,    -2.2381188701097813e+02,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    2.9196405523992283e+01,     2.4591410121320670e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    1.7792242200075538e+01,     3.0631865505804981e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.4538598245349815e+01,     1.3764933190821779e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    5.9695187074629672e+00,     8.3908063415358974e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.6274936423469093e+01,     1.7521096727108143e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6686539885668772e+01,     2.7748695790062840e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.7394875018373837e+00,     8.4761259734473042e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6644957091522247e+01,     4.0349844297980253e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7662791635205195e+01,     1.5962997731530810e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.2250820875259372e+00,     2.3381699919714494e+04,    -1.5844003732197554e-04,     1.9752036588924160e-08,    -2.0842624964580063e-10, } , 
      {    6.1857181098627763e+00,    -1.7424240964784272e+04,    -5.4707783578367268e-05,    -2.0958597245238268e-07,     1.1452705107482456e-09, } , 
      {    5.2423247918728269e+00,    -2.5334179899929964e+04,     2.2293307344401761e-04,     5.2439726373108627e-08,    -8.6296835237497348e-11, } , 
      {    1.2423204879994261e+01,     1.6518826688923586e+04,     5.5179796597802893e-04,     7.9390591628039882e-07,    -3.8531052119861476e-09, } , 
      {    1.1828400164385027e+01,    -1.7910567568835315e+04,    -4.2813965671976176e-04,    -6.4828960612800795e-07,     3.1547795857159722e-09, } , 
      {    1.6582633207685912e+01,    -1.8814866748024763e+03,    -1.2089095970220088e-04,    -1.4746824993286117e-07,     7.3725131672646013e-10, } , 
      {    1.0810551051247934e+01,     4.8974579489074836e+02,     2.4454926896046804e-04,     2.9308456008525207e-07,    -1.4355769429966358e-09, } , 
      {    9.0111819753844529e+00,     1.5053008495091046e+04,    -3.1073244508430170e-04,    -2.3096683756573055e-07,     1.0275577489748503e-09, } , 
      {    4.7688737757406372e+00,    -9.0955607998237865e+03,     9.2256861198141885e-05,     4.2365685930597359e-08,    -1.9959294437598484e-10, } , 
      {    3.2590893795022453e+01,     3.8856891354785970e+04,    -1.6279405881667558e-04,     1.5467791418464115e-07,    -9.6560825680267380e-10, } , 
      {    2.9217498160013122e+01,     4.8928513293888587e+04,     3.5760245910122811e-04,     8.4533320624324642e-07,    -4.3199807868946306e-09, } , 
      {    1.6066064008612955e+01,     1.4985434380517760e+04,    -2.5423105344098514e-04,    -1.5692589826120981e-07,     5.6951547933618284e-10, } , 
      {    1.4596813159309306e+01,     7.8423704504038878e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1992881646113696e+01,     3.2129143833815702e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {   -4.4960002065939211e+00,    -2.6171696138548446e+04,    -6.4226121911122155e-04,    -9.7181663535885805e-07,     4.7399263974717103e-09, } , 
      {    1.2067341630051047e+01,     5.8188915349439476e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    5.3996070518970498e+00,    -9.0955495401608241e+03,     9.7584624183958920e-05,     4.1132901702272025e-08,    -9.0713487872405258e-11, } , 
      {    2.6145816733228466e+00,     7.8669371188686882e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2851284306512056e+01,     1.5410370579939710e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3151745064698773e+01,     8.2817920859700243e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.1715157219856652e+01,     1.9063511533452460e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.2484083105542153e+01,     2.5845522779699359e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.2434664252119685e+00,     8.8172793877914501e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1617291980746053e+01,     9.2386862908946969e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3357636272623578e+01,    -1.6121230758651520e+04,    -2.6457087756708985e-05,    -1.7256550280012231e-07,     9.1624937592891186e-10, } , 
      {   -1.1618259310093171e+01,    -1.0582042509913224e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.2368221256603942e+01,     1.6920908268200924e+02,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    9.4425671647865972e+00,     2.2155287416270319e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1957166007085460e+01,    -1.5911681292915218e+04,    -4.5905423403432856e-04,    -6.8469368366610570e-07,     3.3293609922835159e-09, } , 
      {    1.1127812727929388e+01,     1.6781335776132175e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -8.7719999256596353e+00,    -9.7960005705272488e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -3.8284598222380097e+00,     2.5973588031459949e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -2.7951065723023888e+01,    -6.0601614544710901e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3907569733799996e+01,     2.2868150843807420e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.7792577362334779e+01,     3.1520023918526193e+04,     4.5470021253962852e-04,     8.1961956126182262e-07,    -4.0865429994403895e-09, } , 
      {    3.1202139272637215e+01,     3.1226331729235342e+04,    -7.1943291703496304e-04,    -5.7114867118373656e-07,     2.4544662233532473e-09, } , 
      {    7.8548178603052055e+00,     4.1222095979867925e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {   -1.1073595018851819e+01,     1.0866843036720240e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.8777965968956192e+01,     1.0473822067027251e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.7059626582795268e+01,    -8.2419656352111597e+03,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    2.1355685734047277e+01,     3.3641709135516241e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -7.0511275684115361e+01,     1.0226553184108427e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2112925994944820e+01,     7.7310367619366043e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.5044254454152481e+00,    -8.5181630333291141e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.1436446156068648e+01,     7.8695929222599143e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -7.8941349695259468e+00,     1.5787214433734514e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.1329731733626744e+01,     1.3109395643713156e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.0003453507342419e+01,     2.3971811220303138e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.0459958697744039e+01,     1.5444538512712777e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -3.6410399303855758e+00,     1.3595313638322978e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.9379150034752485e+01,     4.8176252781957766e+04,     3.2668788178666131e-04,     8.0892912870514867e-07,    -4.1453993803270866e-09, } , 
      {    8.3484144040388208e-01,    -1.7894810349581952e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.5168637990650495e+01,     7.0268048392204764e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.2647755861493550e+01,    -5.9144802304578786e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.3683094971506844e+01,    -1.1930949015066981e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.5678593575282873e+00,     1.7468017428172720e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.4938771970105989e+01,     3.2193578183629561e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.0925015107996796e+01,     2.3865997776607372e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.6245270388689885e+01,     5.8676292955372201e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.7350602958991068e+00,     1.3079853738955719e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    9.2727841457777753e+00,     1.4390913852758858e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.0169736589635523e+00,    -2.4760184622243428e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1765041618941556e+01,    -7.8117175280451920e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7310858211399744e+01,     3.1138468201435873e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    3.0191511169989464e+00,     1.3524689820044576e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.8470254047525877e+01,     2.2657472127509860e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.1114601538746687e+00,    -1.0098538263969531e+03,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    1.4975860868574543e+01,     3.3419540431361951e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    2.2423634706345403e+01,     3.9518974737455188e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    7.6131012961504947e+00,     7.8761518778590489e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.3572908093362717e+00,    -8.9907804371241546e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1250170121897209e+01,     7.6328453649986823e+03,     2.1402363002787563e-04,     3.2357066246215001e-07,    -1.5844195912340743e-09, } , 
      {    8.5528080825704791e+00,    -6.2485838404325841e+02,    -1.2631665505160206e-04,    -1.4619183247323603e-07,     6.2909908074454482e-10, } , 
      {    1.8018151448375445e+01,     3.2061605359236481e+04,     4.8551685749061114e-04,     8.5606727203122028e-07,    -4.2603971854862689e-09, } , 
      {    1.7738208412427632e+01,     4.0039058388630307e+04,     1.4624271056626098e-04,     5.2114615166693372e-07,    -2.6811214674087666e-09, } , 
      {    8.4817131446757799e+00,    -1.8120092654240525e+04,     4.3595571942736815e-06,    -1.3611779203072471e-07,     7.4239518988303234e-10, } , 
      {    1.3695553139326535e+01,     4.1991538368845766e+04,     8.1749674444218863e-05,     4.4895438870490091e-07,    -2.3863983825254686e-09, } , 
      {    2.3424035527961113e+01,     3.8924429829365195e+04,    -2.2452528108222501e-04,     8.1826125877145813e-08,    -6.1717266418925054e-10, } , 
      {    1.9381380080327091e+01,     4.0876909809580655e+04,    -2.8901831720426713e-04,     9.6343629151130302e-09,    -3.2244957930595240e-10, } , 
      {    1.5398429534581405e+01,     2.3834185075985217e+04,     2.4077451487533700e-04,     4.9600526556837270e-07,    -2.5028506287279790e-09, } , 
      {    2.4431110070727051e+01,     3.1122769218583613e+04,    -2.8703163548451173e-04,    -5.8933223855153483e-08,     4.2809048041971763e-11, } , 
      {    3.1076086978169283e+01,     3.0041897706442574e+04,    -6.8861627208398042e-04,    -5.3470096041433901e-07,     2.2806120373073679e-09, } , 
      {    2.0211354045128999e+01,     2.1429195559167933e+04,    -9.3849068836349288e-05,     9.1900166319657081e-08,    -5.0387655505076292e-10, } , 
      {    1.8111354386238581e+01,     1.4286120369391094e+04,    -6.3421362267340991e-05,     6.1457697174058990e-08,    -3.5430668629166001e-10, } , 
      {    1.9849989414263625e+01,     2.3456136796298288e+04,    -4.7280456969072912e-04,    -3.4548341666504310e-07,     1.3982075044567121e-09, } , 
      {    1.7506226604718382e+01,     4.0181093760118471e+04,    -2.2472114580939328e-04,     8.1913392339745530e-08,    -6.1571822314592188e-10, } , 
      {    9.4963664387490425e+00,     1.7214667118716872e+04,     4.8740286221957098e-04,     7.2167052008706621e-07,    -3.5591093476245142e-09, } , 
      {    1.4812418560633677e+01,    -1.9525099903780465e+03,     6.1927086992717722e-05,     7.2764521844895592e-08,    -3.4989003365675202e-10, } , 
      {    1.3463570284419735e+01,     4.2133573740333944e+04,    -2.8921418193143540e-04,     9.7216293777127470e-09,    -3.2099513826262385e-10, } , 
      {    1.0164634109748981e+01,     2.5894943380564800e+04,    -1.6386573267137666e-04,     2.1028454048549345e-08,    -3.1657848562771596e-10, } , 
      {    4.1599304198899363e+00,    -1.8680880515206456e+04,    -5.4609851214783133e-05,    -2.0962960568368254e-07,     1.1445432902265811e-09, } , 
      {    1.4132830620643201e+01,     5.0252715699221080e+04,     2.9567537210851036e-04,     7.7256868439835084e-07,    -3.9700907532378787e-09, } , 
      {    6.4410255099381617e+00,     1.5958063208288750e+04,     4.9273062520538804e-04,     7.2043773585874085e-07,    -3.4502298911209345e-09, } , 
      {    3.5787194508146555e+01,     4.6699280555689475e+04,    -3.8403145655932806e-04,    -3.3263212105029557e-08,    -1.9135595993393406e-10, } , 
      {    3.0144495698463487e+01,     4.7185607159740510e+04,    -1.0599583417933510e-05,     4.0544042157059570e-07,    -2.2008650349016606e-09, } , 
      {    7.3570546754452071e+00,     1.6620122210626876e+04,     4.5925009876149678e-04,     6.8460641720350598e-07,    -3.3308154333268447e-09, } , 
      {    1.8875271923102524e+01,     4.1228148204810292e+04,     2.0254823748240929e-04,     5.9527435743405429e-07,    -3.1377092960041052e-09, } , 
      {    7.9266627876951086e+00,    -9.3723022981444701e+03,    -7.0854232759863934e-04,    -9.0965527960803671e-07,     4.3326344239715898e-09, } , 
      {    1.0069977781773130e+01,     7.0720575040327494e+03,     1.5505422161881882e-04,     2.5005884880919218e-07,    -1.1822714908905254e-09, } , 
      {    2.0679150063730493e+01,     1.4499143416460778e+04,    -6.2243309596014677e-04,    -5.9681868293386045e-07,     2.6886312313291530e-09, } , 
      {    7.3961778306592523e+00,     1.4200536132559500e+02,    -3.7352980550497868e-04,    -4.3866000044432536e-07,     2.0102362954893911e-09, } , 
      {    2.2366445146164843e+01,     1.4147558579236302e+04,     1.8112790669312708e-04,     3.5454225725931107e-07,    -1.7898836292882957e-09, } , 
      {    5.0759656940493620e+00,    -1.8018821512868330e+04,    -8.8090377658674317e-05,    -2.4546092433891746e-07,     1.2639577480206711e-09, } , 
      {    5.7098797767715608e+00,    -6.7544104410706495e+01,     5.9067340772640943e-05,     7.3468180421657975e-08,    -4.0287532086521329e-10, } , 
      {    6.6454833629636525e+00,    -1.3614386636214313e+03,     2.1649443786597803e-04,     2.5597682397039192e-07,    -1.2080102492206306e-09, } , 
      {    1.1718813175061593e+00,    -8.3624784819100933e+03,    -1.2404171194066790e-04,    -2.1369840450239435e-07,     1.0069628638013171e-09, } , 
      {    1.0150638314622682e+01,     1.6833175267859137e+04,    -9.7195685801984544e-05,     2.5757278212723649e-08,    -2.3271056693257730e-10, } , 
      {    2.5299501329609209e+00,     3.3757047124063320e+01,    -3.0816644950982666e-05,    -3.6447710769397583e-08,     1.7385418604587960e-10, } , 
      {    8.7210508822029240e+01,     7.1275275316569241e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.1819010084010024e+00,     1.3472874708648706e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.7237949999036921e+00,    -3.9705500799841693e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -5.7446387370619242e+01,     7.7929526477220788e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.0151430371587269e+00,    -2.0829974925313554e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2034585526488325e+01,     2.5319576666491928e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.0445019295391107e+01,     3.0339807023631911e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    1.0524678869633224e+01,     2.2874709799406555e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.1725053017641471e+00,    -2.1610842133543611e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -7.7211555759418919e+00,     8.4259931206629990e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.5352056238450835e+00,     8.4514998414323090e+03,    -4.6568873433953641e-04,    -4.8106931960622267e-07,     2.2091020193437112e-09, } , 
      {    2.6047925041831746e+01,     2.3202797746505410e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.9332174258980039e+01,     6.9979544440229101e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    5.8136104831877624e+00,     4.3883597633987847e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.5189006270279517e+00,     4.7511623143322686e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.5956874076243110e+00,     8.7463050338562789e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.1719474429439476e+00,     1.5453979065337642e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.1702044737422790e+01,     9.7702634209109965e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.9913073456686563e+00,     8.4259879909225438e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.3869061347593394e+00,     8.4409443240606579e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.2826318137574489e+01,     7.0877126297590121e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7265647832694466e+01,     2.2615994006068391e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    3.5911455490352653e+00,    -1.3176190694022744e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -5.2261869924789481e+00,     1.2354028003572537e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5408159081237006e+01,     1.7628011751864367e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7133996314401813e+01,     1.5456013241054356e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.8349019459685838e+00,    -1.5010905733589589e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.2893594344778840e+01,    -8.4118906547604201e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.2432295524862491e+01,     3.2320911380320838e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {   -1.3072702723019560e+01,    -1.5421980048889647e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.2117395431624695e+01,     7.6107943688116370e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0084108726843645e+01,     1.6362577656822932e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.3877369276035118e+01,     7.0080828259631871e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.4845163455816490e+01,     8.5589443376280906e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.4030327470367698e+00,     1.0413948886506021e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.1884888521921155e+00,     5.1288054034757743e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.9937969236995290e+00,     1.5784599860933484e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2974530021969565e+01,     7.5824911955114885e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.9552320029264330e+00,     4.1228547292867261e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -4.2654016690148410e+01,     7.8327675496199899e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.7266629309449657e+00,    -2.2928651060357697e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0055534987768947e+01,     8.3812084057472857e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.5598383104127898e+00,     8.4857239092359159e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0342280902721988e+01,    -8.2047384977381789e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2486604433881281e+01,     1.0734820108829008e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3068512573513043e+01,     1.6860373359384066e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {   -2.8061109066031172e+01,     7.0236248542516169e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.5214966097338261e+00,    -1.6873491270582334e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.8222035713651557e+01,     2.5390569971904915e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.7463109530081624e-01,     1.6428096338643321e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.9492661563104310e+01,     2.3784704665677804e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.9252402535991916e+01,     3.3308206742894668e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    1.5679695027127913e+01,     1.7096218825586780e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -4.5101887774583442e+00,     1.4404031763957126e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.9017650382960642e+00,     8.5044538330731084e+03,    -2.5205404269363512e-04,    -2.2438883705906827e-07,     9.4810648291461947e-10, } , 
      {    6.8882708899709755e-01,     4.5934279599918373e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2655873860841920e+01,    -9.0393458345239287e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.0625132396525785e+01,     3.1078949624364457e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {    7.6646871401916110e+00,    -5.0435465816027181e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.4152141524244151e+00,     8.4018269682582613e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.9464223770298452e+00,     8.4651053467249403e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.6918682240280187e+00,    -8.6249819392871941e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.4741058299467326e+01,     7.2075039097592116e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.4086050875901908e+00,     4.2004069902988504e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4900760397118225e+01,     2.6034625300464904e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.2639305719417404e+00,    -3.5775291102911810e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.5476263449254787e+01,     7.0478977278611001e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.9492218841221263e+00,     1.3289403204692020e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.1981080709692513e+00,     8.4334661574916008e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.0990105768993592e+00,    -1.6663941804846032e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    8.0872855453270400e+00,     2.1618094629315470e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.3022495261019333e+00,     2.9946786053938922e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    5.5568159697136830e+00,     8.4967445359582798e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.0236196281462053e+00,     8.3701877790249218e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2670411043867409e+00,    -2.5202182695205785e+04,    -2.0966407283360197e-04,    -4.5968845449287477e-07,     2.3268147811171065e-09, } , 
      {    1.7653119813690783e+01,     6.3876805429180768e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.1953746696352420e+01,     6.4809861772761333e+03,     4.8687074555849077e-07,     6.6846546683695823e-08,    -3.2415127532664681e-10, } , 
      {    8.7259026670495654e+00,     7.7375900877042541e+03,    -4.8408922402585326e-06,     6.8079330912021156e-08,    -4.3303073183022644e-10, } , 
      {    1.2854002632779840e+01,     1.5019227067635886e+04,    -2.7981786776973489e-04,    -1.9456276002763283e-07,     8.5297634240730622e-10, } , 
      {    1.5209557183166643e+01,     2.3347918492259334e+04,    -1.2752546000740873e-04,     5.6156114127021881e-08,    -3.8300765621334456e-10, } , 
      {    9.5981921459656174e+00,    -1.8477052473473168e+03,    -1.5180553701676770e-04,    -1.8387232747095888e-07,     9.1183272329400397e-10, } , 
      {    7.1681781352402378e+00,     1.6133855626900990e+04,     9.0950123878751090e-05,     2.4475726576215507e-07,    -1.2109724608122099e-09, } , 
      {    1.5847795555505810e+00,     9.0617681127056603e+03,    -6.6670046869392126e-05,    -4.7288241641743030e-09,    -8.3867918695138641e-11, } , 
      {    7.2039825062858753e+00,     8.3996847300364407e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0389318593266449e+01,     8.4357673574330111e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.3991915464104796e+01,     8.4311649575501906e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.7077499483622147e-01,     7.3307668808221115e+02,    -2.1896245463171829e-04,    -2.5544769831882901e-07,     1.1521160799255120e-09, } , 
      {    3.9403341059373833e+00,     1.7390459537329112e+04,     8.5622360892934055e-05,     2.4599004999048043e-07,    -1.3198519173157896e-09, } , 
      {    2.4570690344666328e+00,    -5.2352722234590965e+02,    -2.1363469164590129e-04,    -2.5668048254715437e-07,     1.2609955364290918e-09, } , 
      {    4.8126235848534353e+00,     7.8051642022775404e+03,    -6.1342283883575091e-05,    -5.9616083924996316e-09,     2.5011537808440947e-11, } , 
      {    1.1996872384208951e+01,     5.8526842220620729e+03,     3.1507522384670024e-06,     6.6230154569533156e-08,    -2.6971154707485704e-10, } , 
      {    1.4309301246739222e+01,     1.4809677601899584e+04,     1.5277927850788466e-04,     3.1756542083835050e-07,    -1.5601352739472976e-09, } , 
      {    3.8103440908308803e+00,     8.3997091103675375e+03,    -3.3189520425500942e-05,     3.1102494491060616e-08,    -2.0328237648922845e-10, } , 
      {    9.6261586034769859e+00,     1.6275830978064005e+04,    -2.8514563075555195e-04,    -1.9332997579930752e-07,     7.4409688590372669e-10, } , 
      {    1.1981713153863788e+01,     2.4604522402687457e+04,    -1.3285322299322574e-04,     5.7388898355347215e-08,    -4.9188711271692414e-10, } , 
      {    1.5035431926279584e+01,     7.1092881324901937e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7565111733553444e+01,     3.1676609916882786e+04,     2.4766947754917454e-05,     3.0687498828167661e-07,    -1.6189916548339954e-09, } , 
      {    1.1081457217436368e+01,     1.6066281512327703e+04,     1.4745151552206766e-04,     3.1879820506667586e-07,    -1.6690147304508773e-09, } , 
      {    9.9248390082969777e+00,     1.6833145257696560e+04,    -9.9761634931308941e-05,     2.6330037095586454e-08,    -2.8787751570603142e-10, } , 
      {    6.6690285214827547e+00,    -3.3787057286643176e+01,     2.8250695821658283e-05,     3.7020469652260392e-08,    -2.2902113481933369e-10, } , 
      {    9.5237326856270403e+00,     2.4462547051524438e+04,     2.4324253164107726e-04,     4.9547613991680984e-07,    -2.4469564594328609e-09, } , 
      {    6.3703481166627629e+00,    -5.9110133691919600e+02,    -1.5713330000258471e-04,    -1.8263954324263358e-07,     8.0295326679042444e-10, } , 
      {    1.5252682871023174e+01,     2.2719616537045273e+04,    -1.2486157851450020e-04,     5.5539722012859221e-08,    -3.2856792796155479e-10, } , 
      {    7.2426375955788149e+00,    -1.0176396671970768e+04,    -3.0409794477909385e-04,    -4.3459120162561366e-07,     2.1478167219146547e-09, } , 
      {    6.2958886563241858e+00,     2.5719150961952560e+04,     2.3791476865526025e-04,     4.9670892414513520e-07,    -2.5558359159364402e-09, } , 
      {   -2.1636513581727147e+01,     1.6759856382173639e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.2897128320636371e+01,     1.4390925112421824e+04,    -2.7715398627682641e-04,    -1.9517915214179551e-07,     9.0741607065909609e-10, } , 
      {    3.2745031737106544e+01,     1.6764458782056459e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9995125744278827e+01,     1.3695049042634480e+04,    -2.1798871314060133e-04,    -1.2175460495143737e-07,     5.0381352927221842e-10, } , 
      {    9.6413178338221481e+00,    -2.4760072025613772e+03,    -1.4914165552385917e-04,    -1.8448871958512156e-07,     9.6627245154579374e-10, } , 
      {    2.2350680294665629e+01,     2.2023740467257929e+04,    -6.5696305378275133e-05,     1.2896426920321735e-07,    -7.3217046934843241e-10, } , 
      {   -2.0654308555693898e+01,     8.4337947898663733e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.7051711653362137e+01,     8.4331375251168283e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    7.9877554801444877e+00,     7.0993305412990367e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3250936231093046e+01,     3.0561981357617675e+04,    -3.4600104389356849e-04,    -1.3244503750811132e-07,     4.4495714838552083e-10, } , 
      {    9.5567650290698012e+00,     1.6728376154659891e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0151448407983010e-01,    -8.8522186469693588e+03,    -3.6592709940822741e-04,    -5.0739935670180904e-07,     2.4969795350497424e-09, } , 
      {   -3.1263295452230242e+00,    -7.5956147365412389e+03,    -3.7125486239404448e-04,    -5.0616657247348368e-07,     2.3881000785461631e-09, } , 
      {    1.4447221560326051e+01,     8.4460853959095875e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.6827769791930347e+00,     8.3658920429183145e+03,    -7.5047737331670442e-06,     6.8695723026183823e-08,    -4.8747046008201626e-10, } , 
      {    1.4337267704250591e+01,     3.2933213827310909e+04,     1.9439184769100432e-05,     3.0810777251000197e-07,    -1.7278711113375752e-09, } , 
      {    9.5830329156204552e+00,     1.6904132933278066e+04,    -2.8780951224846043e-04,    -1.9271358368514484e-07,     6.8965715765193682e-10, } , 
      {    7.2113038230967685e+00,     1.5505553671686930e+04,     9.3614005371659594e-05,     2.4414087364799244e-07,    -1.1565327325604202e-09, } , 
      {    2.0895381680706244e+01,     2.2233289932994227e+04,    -4.9829345165589478e-04,    -3.8316391166276606e-07,     1.6809411470061717e-09, } , 
      {    1.2024838841720319e+01,     2.3976220447473395e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    6.3272224288062322e+00,     3.7200618294864398e+01,    -1.5979718149549324e-04,    -1.8202315112847090e-07,     7.4851353853863457e-10, } , 
      {    9.3495230958525397e+00,     7.1430695599453538e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6850942773300698e+01,     7.1069869325487834e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.7530592422689708e+01,     7.1115893324316021e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1643000534538793e+01,     8.2937465356140583e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.1390068269275195e-01,     1.3613786432962715e+03,    -2.2162633612462683e-04,    -2.5483130620466639e-07,     1.0976763516737224e-09, } , 
      {    1.6767281714975969e+01,     1.4951652953062599e+04,    -2.2331647612641834e-04,    -1.2052182072311206e-07,     3.9493407276863884e-10, } , 
      {    2.5606490781479849e+01,     3.8890672782241127e+04,    -1.9370863613124236e-04,     1.1827383664654339e-07,    -7.9102685023513006e-10, } , 
      {    9.6692842913335166e+00,     1.5647529022849945e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.4352426934595753e+01,     1.4181375646685523e+04,     1.5544316000079320e-04,     3.1694902872418788e-07,    -1.5056955456955079e-09, } , 
      {   -1.5950678351569625e+01,     1.5645227822908535e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.6664855797126023e+01,     2.3138369026523033e+04,     3.0507168627021087e-04,     5.6828429499300522e-07,    -2.7961192725679486e-09, } , 
      {    3.8430844292075292e+01,     1.5649830222791355e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.9920666283940250e+01,     4.0005301341506231e+04,     1.7705935551724363e-04,     5.5759386243633127e-07,    -2.8549756534546465e-09, } , 
      {    2.5001947223231631e+00,    -1.1518291775599700e+03,    -2.1097081015299275e-04,    -2.5729687466131700e-07,     1.3154352646808814e-09, } , 
      {    1.5236542780387465e+01,     8.7151534387173033e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3097188860385828e+01,     1.0512704882622563e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -4.1410892731621885e+01,     1.0442241691007587e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.5829672820741120e+01,     7.1089358165321200e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.2792444963699040e+01,     7.1096404484482682e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7608237421409974e+01,     3.1048307961668725e+04,     2.7430829247825965e-05,     3.0625859616751398e-07,    -1.5645519265822058e-09, } , 
      {    1.5416538676940506e+00,     9.6900700679197216e+03,    -6.9333928362300630e-05,    -4.1124320500116387e-09,    -1.3830764694692846e-10, } , 
      {    1.1719179292866844e+01,     8.7927056997294276e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9122836265362771e+01,     2.3280344377686051e+04,    -7.1024068364092168e-05,     1.3019705343154266e-07,    -8.4104992585201194e-10, } , 
      {    1.2039998072065481e+01,     5.2243822668480125e+03,     5.8146337313755150e-06,     6.5613762455370502e-08,    -2.1527181882306722e-10, } , 
      {    3.8972084180808531e+00,     1.8018761492543170e+04,     8.2958479400025551e-05,     2.4660644210464305e-07,    -1.3742916455675793e-09, } , 
      {   -1.3595130811906866e+01,     2.3973919247531987e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    4.0786402720341670e+01,     2.3978521647414807e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -3.6721903342827687e+01,     1.6761805266156978e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.7830419606589089e+01,     1.6762509898073124e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3076343812713223e+01,     8.5574190843768720e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4134738045192936e+00,    -1.2194032921332564e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2280393558683780e+01,     2.5161836682320012e+04,     5.2530772831017247e-05,     2.7704891125024120e-07,    -1.5238615143266822e-09, } , 
      {    4.8303619111593775e+01,     8.4338184734496772e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.5623025991910126e+01,     8.4331138415335245e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1124582905292899e+01,     1.5437979557113644e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    8.6979362095381969e+00,    -1.0385946137707066e+04,     1.2849920149852570e-04,     7.7536979240369744e-08,    -2.6529489443994916e-10, } , 
      {    7.1563862198657535e+00,    -8.9197927615426452e+03,    -3.0942570776491091e-04,    -4.3335841739728830e-07,     2.0389372654110754e-09, } , 
      {    1.1879287236013843e+01,     3.2791238476147890e+04,     3.9553493940340344e-04,     7.4619501407146456e-07,    -3.6829404580535118e-09, } , 
      {   -1.4547895404440778e+00,    -7.1017685744087430e+01,     1.8548192818782712e-04,     2.1961637966359409e-07,    -1.0327016221314225e-09, } , 
      {    5.4621024123126105e+00,     8.3570249140807373e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.1658986412176828e+00,     1.6728400534990986e+04,     1.1910288733682525e-04,     2.8182136864571535e-07,    -1.4392663751098792e-09, } , 
      {    2.0038251432135358e+01,     1.3066747087420419e+04,    -2.1532483164769280e-04,    -1.2237099706560005e-07,     5.5825325752400829e-10, } , 
      {    9.3217883406906505e+00,     8.3467156328252495e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.3568554288032448e+00,     6.6905356430124348e+03,    -4.3211027553206107e-04,    -4.4528163418228755e-07,     2.0889603410279568e-09, } , 
      {    1.3437011767823170e+01,     2.4394972936951155e+04,     2.9974392328439381e-04,     5.6951707922133058e-07,    -2.9049987290715279e-09, } , 
      {    8.6514432067109883e+00,     3.4047842386576012e+04,     3.9020717641758638e-04,     7.4742779829978992e-07,    -3.7918199145570915e-09, } , 
      {    1.5251853335907009e+01,     1.5613747595394785e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.5510529627131753e+00,     8.4332425003338903e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.0129535484870427e+01,     8.4336898146493113e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.2393805982522160e+01,     2.1395438512043867e+04,    -6.3032423885366629e-05,     1.2834787708905467e-07,    -6.7773074109664254e-10, } , 
      {   -1.9280960116508854e+01,     2.5088547806797091e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.2440043690871532e+01,     1.6745236901307457e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.2418631052852334e+00,     7.9291114993313295e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3789746806312463e+01,     8.4210146380142924e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5100589354485365e+01,     2.5093150206679911e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    9.1842192855969120e+00,    -1.2169541367574413e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.4706234845052432e+01,     3.0352431891881381e+04,     8.6596102384051048e-05,     3.7968314335787204e-07,    -1.9681544679690833e-09, } , 
      {    1.0412196706679977e+01,     7.9471395534405556e+03,    -4.3743803851787813e-04,    -4.4404884995396219e-07,     1.9800808845243775e-09, } , 
      {    4.8870830451920124e+00,    -1.8505088096594216e+04,    -4.5639035254142003e-04,    -6.8531007578026838e-07,     3.3838007205353055e-09, } , 
      {    5.4700921802353424e+00,    -9.1293422272789467e+03,     1.2317143851270869e-04,     7.8769763468695077e-08,    -3.7417435094352879e-10, } , 
      {    4.4276918043203821e+00,     8.5044538330731084e+03,    -2.5205404269363512e-04,    -2.2438883705906827e-07,     9.4810648291461947e-10, } , 
      {   -5.5453360401665730e-01,     8.4672232046156641e+03,    -9.4822810327466282e-05,    -4.1792927047734551e-08,     1.4442599560253075e-10, } , 
      {   -4.0302153127621789e+00,    -8.4174666492676042e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -7.7440366223362589e-01,     8.4494656657155974e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.8598514288221750e+00,     8.4023145623526252e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0023092201790195e+01,     3.1818585268045797e+04,    -3.5132880687938555e-04,    -1.3121225327978601e-07,     3.3607769188194119e-10, } , 
      {    1.7639571193892024e+01,     5.3663576180110294e+03,    -3.7028112090292745e-04,    -3.7247347910609211e-07,     1.7397975278928694e-09, } , 
      {    6.5865895074823273e+00,     7.1430451796142570e+03,    -3.2993655698332687e-05,     3.1015228028460899e-08,    -2.0473681753255705e-10, } , 
      {    1.3141942679464680e+01,     9.1857266694224163e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4801629489267860e+00,     7.7713771449908972e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3294061918949577e+01,     2.9933679402403613e+04,    -3.4333716240066001e-04,    -1.3306142962227400e-07,     4.9939687663731064e-10, } , 
      {    1.5295808558879704e+01,     2.2091314581831215e+04,    -1.2219769702159170e-04,     5.4923329898696554e-08,    -2.7412819970976498e-10, } , 
      {    7.2857632834353456e+00,    -1.0804698627184827e+04,    -3.0143406328618538e-04,    -4.3520759373977628e-07,     2.2022564501664448e-09, } , 
      {    1.0668350781255736e+01,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0510165648771119e+01,     6.9515537781497615e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.9407254900682243e+01,     1.6761828949740277e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.8298739230257002e+01,     1.6762486214489822e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.4380393392107122e+01,     3.2304911872096847e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    7.0962564547562472e+00,    -8.4672475849467628e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.6665189578227341e+01,     9.1081744084102920e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.0397319647921552e+00,     8.4772369910485650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5345727184953528e+00,     1.0406338740127441e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.2209303759324035e+00,     1.0548607833502709e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7065962119795962e+01,     1.5508967232695151e+04,    -3.7932480302175347e-05,     9.9138192171781901e-08,    -6.3704032884111916e-10, } , 
      {    9.0245830718695572e+00,     8.2949043673368069e+03,     1.8054310358398448e-04,     2.8773934380691514e-07,    -1.4650051334399845e-09, } , 
      {    3.9834597937939140e+00,     1.6762157582115051e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.8557492727099660e+00,     7.1768622470634800e+03,    -5.8678402390666587e-05,    -6.5780005066622959e-09,     7.9451266060230744e-11, } , 
      {    2.4139433466101021e+00,     1.0477473286815075e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.7077015324187990e-01,     8.3598698281385277e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.0291567299845745e+00,     1.6762868927581927e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.0792205355273863e+00,     1.6761446236648175e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.6769297296250301e+00,     7.1066583001740100e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.6192402875186529e+00,     8.4327548120247247e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.0612274112893552e+00,     8.4341775029584769e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3003344079726425e+01,     7.1119179648063782e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7607407886293878e+01,     2.3942439020018235e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    8.8462627075119737e+00,     8.4798668233647477e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4564658258244165e+01,     9.5787476391154032e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3959686023027153e+01,     9.0279923150819832e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.4897194876449884e+00,     9.0617737425371433e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.0768365275902796e-01,     7.8051585724460574e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.7962045331866651e+01,     4.7219364206864579e+04,    -4.1416228368916176e-05,     3.6899271080119810e-07,    -2.0270108488557812e-09, } , 
      {    1.6810407402832499e+01,     1.4323350997848540e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    1.6692822254637392e+01,     4.1261905251934353e+04,     1.7173159253142662e-04,     5.5882664666465663e-07,    -2.9638551099582258e-09, } , 
      {    1.2940254008492902e+01,     1.3762623157207763e+04,    -2.7449010478391788e-04,    -1.9579554425595816e-07,     9.6185579891088575e-10, } , 
      {    9.5668583734835710e+00,     2.3834245096310377e+04,     2.4590641313398579e-04,     4.9485974780264716e-07,    -2.3925167311810709e-09, } , 
      {   -5.9927829382546491e+00,    -8.3659520632434742e+03,     2.3728754745182761e-06,    -6.7550205260458206e-08,     3.7713656253510807e-10, } , 
      {    1.0150097643189980e+01,     7.2670224868306250e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.8124362006036776e+00,     7.5360905296383407e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2221594387846586e+01,     8.4394996319343518e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.0407347063056136e+00,     6.5585328084567745e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8261213111594607e+01,     8.3659033025812805e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.0584771511528787e+01,     8.4180470463519232e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.3840582649349665e+01,    -8.4488852686312784e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0588663572372758e+01,     6.5869187418670326e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5432359767844778e+01,     7.7375957175357371e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9304616707514732e+01,     1.0250826057717910e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.1889552474821734e+00,    -1.1856218646780960e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0801365171455094e+01,     8.4526538272476319e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.0323343886245056e+01,     8.4212602327676232e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7667537651403393e+01,     2.3489893843422349e+04,    -5.0362121464171173e-04,    -3.8193112743444070e-07,     1.5720616905025920e-09, } , 
      {    1.9165961953219302e+01,     2.2652042422471990e+04,    -6.8360186871183650e-05,     1.2958066131738000e-07,    -7.8661019760022217e-10, } , 
      {    1.1938587466007258e+01,     2.5232824357901518e+04,    -1.3551710448613427e-04,     5.8005290469509875e-08,    -5.4632684096871401e-10, } , 
      {    9.6844435216786788e+00,    -3.1043091577754376e+03,    -1.4647777403095066e-04,    -1.8510511169928422e-07,     1.0207121797975836e-09, } , 
      {    1.2340990012289456e+01,     7.0563190352790025e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4206892319768720e+01,     8.4154172140357387e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.7462702931233956e+01,    -8.4515151009474648e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.9628644893822127e+00,     6.7500485902523669e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.6037996864139075e-01,     7.0694732305922817e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.7306782871966231e+00,     8.0066637603434538e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3143394614844297e+01,     9.6562999001275275e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1650764549610710e+01,     7.1491030343881057e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.0892969145612295e+00,     8.3989733845695646e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2378646752176998e+01,     4.0147276692669249e+04,    -1.9903639911705937e-04,     1.1950662087486871e-07,    -8.9990630673870959e-10, } , 
      {    2.5649616469336380e+01,     3.8262370827027065e+04,    -1.9104475463833383e-04,     1.1765744453238072e-07,    -7.3658712198334019e-10, } , 
      {    2.0938507368562775e+01,     2.1604987977780169e+04,    -4.9562957016298620e-04,    -3.8378030377692869e-07,     1.7353808752579614e-09, } , 
      {    1.4010620841919231e+01,     1.4252363322267032e+04,    -3.2604717316358325e-05,     9.7905407943456567e-08,    -5.2816087233753963e-10, } , 
      {    3.9716678784194301e+00,    -8.2914908063285857e+03,    -3.1208958925781939e-04,    -4.3274202528312567e-07,     1.9844975371592853e-09, } , 
      {   -5.4818840956098267e+00,    -1.5924306161164684e+04,    -5.2354727015637066e-04,    -7.5688544662813840e-07,     3.6240840771668139e-09, } , 
      {   -2.2540400663069722e+00,    -1.7180910071592807e+04,    -5.1821950717055370e-04,    -7.5811823085646376e-07,     3.7329635336703936e-09, } , 
      {    1.4567765297483831e+01,     7.1622572297013849e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5987312568008040e+01,     9.9717686088083901e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0444623266447568e+00,    -8.2576981192104613e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.6389267289497038e+01,     9.6225184726723674e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.2594637438203865e+01,     2.3975868131515323e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    5.6069014111671137e+01,     2.3976572763431468e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -1.4495390939512635e+01,     1.5435678357172234e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    9.2989217753438012e+01,     7.0937461042017640e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2697065331679152e+01,     8.4003960755033168e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.6029185692883942e+01,     7.1248301607786234e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.9886182788052380e+01,     1.5440280757055054e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7966218056223386e+01,     2.4047208123054901e+04,    -3.1823721881746878e-04,    -1.6227111453954675e-07,     5.4008728889283402e-10, } , 
      {    1.1655066291532428e+01,     5.9236718976435814e+03,    -1.8489712507868453e-04,    -1.5281346621119814e-07,     7.0782312628311120e-10, } , 
      {    4.8008316694789510e+00,    -1.7248484186166093e+04,    -4.6171811552723710e-04,    -6.8407729155194302e-07,     3.2749212640317262e-09, } , 
      {    6.8694977304202460e+00,     1.5576541347268438e+04,    -9.4433871945491906e-05,     2.5097252867261127e-08,    -1.7899805920245181e-10, } , 
      {    5.9692417939928255e+00,     7.0383004569086861e+03,     1.8587086656980149e-04,     2.8650655957858978e-07,    -1.3561256769364048e-09, } , 
      {    5.8388796223299622e-02,    -8.2239166917552993e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    6.8283956454586416e+00,    -8.4503868382991932e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.8095573514389230e+01,     9.5011953781032789e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1921583388754282e+01,     2.5057067579283339e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -2.2574652230967125e+01,     8.4285351252340042e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.5255240678550727e+01,     8.4383971897491974e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4282905032382136e+01,     8.9942108876268230e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.3482183901370064e+00,     8.1643981146838842e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1239578181964564e+01,     3.2302610672155432e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.4488785833464132e+01,     8.9504400540698607e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5234350363394249e+01,     1.5647176706891869e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    5.3999548915018138e+01,     1.5647881338808022e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.3141936454479463e+01,     3.2307213072038259e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.5405097180102443e+00,     6.3570276205593791e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2042847669097075e+01,     7.0296583286943696e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1324016804761875e+01,    -8.4177336734923010e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6802758563209657e+01,     1.6774776820533039e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.0852767814417964e+01,     8.3783548220585890e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.0132308999321777e+00,     7.2206218209574781e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3244194671181139e+01,     6.4809805474446503e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.8422873305762185e+01,    -1.2197556080913314e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    5.0676340703885501e+01,    -1.2190509761751814e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    3.2743700046540823e+01,     7.1889179362860177e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7409844667036900e+01,     1.0364789578501379e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1710678796877904e+01,    -8.5010290124019193e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    3.0392059342592031e+01,     2.9237803332616269e+04,    -2.8417188926443493e-04,    -5.9636882431915866e-08,     9.5794335250433106e-11, } , 
      {    2.1478390815749574e+01,     3.1609035802309503e+04,     8.1268339398234013e-05,     3.8091592758619740e-07,    -2.0770339244726627e-09, } , 
      {    1.1851723138757286e+01,     1.5613771975725882e+04,    -2.5166510431166075e-04,    -1.5749865714407260e-07,     6.2468242810963683e-10, } , 
      {    7.9118981063352063e+00,    -2.0572547130836165e+03,     2.8079160926085191e-04,     3.2825585339502445e-07,    -1.5012788930605999e-09, } , 
      {    5.2137299075233727e+00,     1.7576240844965923e+02,    -4.0434645045596130e-04,    -4.7510771121372296e-07,     2.1840904815352705e-09, } , 
      {    1.9579194207091499e+00,    -1.6691169906533542e+04,    -2.7633411970299409e-04,    -4.6441727865704901e-07,     2.2429468624219679e-09, } , 
      {    4.7694978969969046e+00,     8.4334661574916008e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.8068205849519288e+01,     8.4491986414908988e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    9.9594309557541507e+00,    -7.2106324148556760e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    8.1789206745653278e+00,     7.2490077543677362e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.7407019043693914e+00,    -1.6795939009570211e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    9.6645415411483029e+00,     1.6755598626515919e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -4.2302814981448634e+01,     8.4610751346655343e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0086118876539555e+01,     7.6316575231133547e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2981302364934420e+01,     7.0317358714780694e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.1826186950807955e+01,     6.4806282314865757e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.3205606963446517e+01,     1.0247353292674052e+02,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.4085953713170776e+00,    -1.1133368846728445e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.4360555733202474e+01,     2.5090496690780426e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    5.0180194908700869e+01,     2.5091201322696572e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.1575349305885588e+01,     7.6600434565236128e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1538466782457775e+01,     8.2069960109477070e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.7694291451206183e+00,     1.6759527749798865e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.3388470716920793e+00,     1.6764787414431237e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.3513185620487658e+01,     7.1096167648649662e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.4192834983889647e+01,     7.1089595001154212e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.1150002988404615e+00,     5.9640066508663913e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.9021020791752136e+01,     8.4152267583248704e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3834751053747517e+01,     7.1868403935023180e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4564793106648114e+01,     8.4732810593895119e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.7302979666625706e+00,     1.6604423227774620e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.8036504792205228e+01,     2.0909111907992821e+04,    -4.3646429702676111e-04,    -3.1035575658657057e-07,     1.3317783338710840e-09, } , 
      {    2.0066217889646726e+01,     3.1190283312831736e+04,    -3.4866492538647702e-04,    -1.3182864539394866e-07,     3.9051742013373101e-10, } , 
      {    2.0321772606610189e+01,     3.2375899547678353e+04,    -1.6594481105514257e-04,     8.8447759615107979e-08,    -6.9589670972781681e-10, } , 
      {    1.4910876778346651e+01,     2.2790604212626778e+04,    -3.1290945583165172e-04,    -1.6350389876787206e-07,     6.4896674539641355e-10, } , 
      {    1.4710407569409160e+01,     7.1802758080717022e+03,    -1.9022488806450153e-04,    -1.5158068198287283e-07,     5.9894366977953167e-10, } , 
      {    1.2968220466004270e+01,     3.1886159382619080e+04,    -4.0783019852270211e-04,    -2.0525319258430675e-07,     7.9411996152060857e-10, } , 
      {    2.2276220834327049e+01,     4.8333992766129682e+04,     3.2935176327956984e-04,     8.0831273659098599e-07,    -4.0909596520752965e-09, } , 
      {    1.9963791971796780e+01,     3.9376999386292169e+04,     1.7972323701015216e-04,     5.5697747032216859e-07,    -2.8005359252028564e-09, } , 
      {    7.2274783652336527e+00,     8.5754415086546160e+03,    -4.4010192001078661e-04,    -4.4343245783979956e-07,     1.9256411562725874e-09, } , 
      {    4.2310349570955248e+00,    -1.1856462450091931e+03,    -1.8528606346065888e-04,    -2.1970364612619381e-07,     1.0312471810880936e-09, } , 
      {    1.6707981484982554e+01,     2.2510067071308971e+04,     3.0773556776311935e-04,     5.6766790287884254e-07,    -2.7416795443161585e-09, } , 
      {    1.1038331529579837e+01,     1.6694583467541765e+04,     1.4478763402915913e-04,     3.1941459718083849e-07,    -1.7234544587026670e-09, } , 
      {    6.3423816591513944e+00,    -1.8714637562330518e+04,    -2.3793206263800474e-05,    -1.7318189491428496e-07,     9.7068910418070173e-10, } , 
      {    1.4635948109070583e+01,     3.3490528106943457e+04,     2.0482318059334345e-04,     5.2776778540489592e-07,    -2.7598455129473328e-09, } , 
      {    3.6136872436060230e+00,    -1.2903909677147640e+03,     3.3578458807475305e-05,     3.5787685423935058e-08,    -1.2014167831575411e-10, } , 
      {    4.3997253468090136e+00,    -9.6190823923382159e+03,    -1.1871394895485087e-04,    -2.1493118873071965e-07,     1.1158423203048969e-09, } , 
      {    7.9821826959545970e+00,     2.5928700427688862e+04,    -1.9468237762235933e-04,    -1.5419256720848238e-08,    -1.4272429958183634e-10, } , 
      {    6.2527629684676551e+00,     2.6347452917166622e+04,     2.3525088716235172e-04,     4.9732531625929788e-07,    -2.6102756441882303e-09, } , 
      {    6.3390143441807165e+00,     2.5090849006738499e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -3.1694552330795545e+00,    -6.9673127813271785e+03,    -3.7391874388695295e-04,    -5.0555018035932100e-07,     2.3336603502943730e-09, } , 
      {   -8.1672681648159227e+00,     8.3936512555936879e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1411190472629768e+01,     1.5489794668509514e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.6572706049087991e+00,     1.0740456518433530e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.9707810485807258e+00,     1.0214490055196711e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.8943503694555076e+01,     1.6744884585349384e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.9527230916355357e+01,     9.8942163477962658e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8832223536497750e+01,     1.0757810548194364e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0469385369863494e+02,     8.4329581988524860e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.0143804321182071e+02,    -8.4339741161307156e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.3749018252009586e+00,     1.6919891936455482e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -3.2199412528787150e+01,    -1.2204772963381902e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5955364640697994e+01,     9.3434610237628476e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4494729545848051e+00,     6.8276008512644912e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.8421513182053904e+00,    -9.1293365974474636e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {   -1.7294541782510940e+01,    -8.3713330076191542e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.4038732552832698e+01,     8.4955993073640475e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.7477850219438018e-01,     2.6250908720858274e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4742588859143035e+01,     1.6749039670916784e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.4270199201633265e+00,     9.6900813275826877e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.2535685274202764e+00,     7.1768509874005140e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0143613369929696e+00,    -5.2959621472281015e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.5678371781190069e+01,    -8.2216154918138855e+03,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    4.3155261562629166e+00,     7.6136427906504650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6030287972631509e+01,    -7.2444138423108334e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.6167722570475505e+01,     8.0530644262166006e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.6806240616669541e+00,     8.4360959898077836e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.1688650409428334e-01,     8.4308363251754163e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0449704128120686e+01,     6.4816918929115272e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1488045045521799e+01,    -1.1789264406641814e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.0082664386282172e+01,     8.4165454766840085e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    3.7573873916408395e+01,     6.4832817473860596e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5232273539151743e+01,     1.1739397128613791e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -3.1138917355635389e+01,     8.5008900365634454e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.1538326868385909e+00,    -1.3771376464736884e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.1825199167898901e+01,     1.5471760984568806e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.5127703053686297e+00,     6.5147676047312934e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    9.4518110051430657e+00,    -1.3855616032331204e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    8.0651102714398455e+00,     2.4010001874928555e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -5.7418435831058545e+00,     1.6722342680217138e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.9410257651773053e+00,     8.5912005118320321e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.2327537642419770e+00,     7.0966688940722070e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6851831397926475e+01,     1.6801972484012964e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    5.5079735549443839e+01,     1.5438331873071720e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.3998491296243788e+01,     1.6622437960237508e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.9689078717152078e+01,     1.5437627241155567e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -2.8703244867865383e+01,    -8.2262178916967096e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.0535513893177885e+01,     6.5606103512404752e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3767091728409930e+01,     2.3818486093132964e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    9.9445363041410531e+00,     1.5096773698816527e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.9295249074739562e-01,     5.5709856811734044e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5868832764434325e+01,     8.8391063656025763e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.6137815005033147e+00,     1.5644899190533761e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    6.1457839122756299e+00,    -3.4944889009392369e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3865826907008127e+01,     1.5650158855166128e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.4571240721917240e+00,     8.2757318031511695e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.4072507788614661e+01,     8.4246060019510587e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.7328318275428888e+01,    -8.4423263130321466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1489409167017261e+01,     1.5125159632226785e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.6807817045700830e+01,     6.4786793475032409e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    2.0254474848442520e+01,     1.1150831517887353e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5093105699108648e+01,     1.5647200390475175e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -1.2612941100085621e+01,     1.5647857655224718e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.3647869829406261e+01,     4.6104735647599475e+04,    -4.1218422001740215e-04,    -7.0327314988589814e-08,     3.6937954363735257e-11, } , 
      {    3.1292315279019455e+01,     3.7776044222976023e+04,    -5.6447662777972844e-04,    -3.2104618914324452e-07,     1.2729219529843862e-09, } , 
      {    3.2747613892978833e+01,     3.7566494757239721e+04,    -1.3187948150210878e-04,     1.9108199172273887e-07,    -1.1401896633702176e-09, } , 
      {    2.2421772440033529e+01,     3.9518974737455188e+04,    -1.9637251762415087e-04,     1.1889022876070606e-07,    -8.4546657848691982e-10, } , 
      {    2.7061789395439234e+01,     3.8681123316504832e+04,     2.3888851014637722e-04,     6.3040201751252676e-07,    -3.2041384665897342e-09, } , 
      {    2.4749360532908963e+01,     2.9724129936667319e+04,     8.9259983876959552e-05,     3.7906675124370942e-07,    -1.9137147397172933e-09, } , 
      {    1.6724156027119438e+01,     1.5579954908276661e+04,    -2.2598035761932687e-04,    -1.1990542860894938e-07,     3.4049434451684908e-10, } , 
      {    1.5323775016391073e+01,     4.0214850807242532e+04,    -2.5553779076037592e-04,     4.5465681570347947e-08,    -4.4186403710004221e-10, } , 
      {    1.7651363109266505e+01,     3.0420006006454667e+04,     3.0094710740734479e-05,     3.0564220405335130e-07,    -1.5101121983304159e-09, } , 
      {    8.1428935320062354e+00,    -1.6381557816110135e+03,    -5.8440268329438725e-04,    -6.9600050833694232e-07,     3.3249443396486077e-09, } , 
      {    1.4207277689144089e+01,     2.3942463400349334e+04,    -9.9372696549334565e-05,     9.3220217010582131e-08,    -6.1130157051101389e-10, } , 
      {    1.9020410347512829e+01,     3.1467060451146481e+04,     4.5736409403253705e-04,     8.1900316914765994e-07,    -4.0321032711885995e-09, } , 
      {    1.3480137455679701e+01,     2.3766670981737094e+04,     3.0240780477730234e-04,     5.6890068710716790e-07,    -2.8505590008197383e-09, } , 
      {    1.4395552622452284e+01,     1.3553073691471462e+04,     1.5810704149370170e-04,     3.1633263661002520e-07,    -1.4512558174437181e-09, } , 
      {    9.2250522808070485e+00,     2.3905232771891890e+04,     5.7858535816834269e-05,     2.7581612702191584e-07,    -1.4149820578231025e-09, } , 
      {    4.3134739710959522e+00,    -8.3624784819100933e+03,    -1.2404171194066790e-04,    -2.1369840450239435e-07,     1.0069628638013171e-09, } , 
      {    1.0337737246341399e+01,     3.4257391852312314e+04,    -4.2389969860033156e-05,     2.3529961743380648e-07,    -1.3787082982024873e-09, } , 
      {    7.2823959684646677e+00,     3.3000787941884191e+04,    -3.7062206874216134e-05,     2.3406683320548117e-07,    -1.2698288416989075e-09, } , 
      {    2.4850354919780013e+00,     1.7600009003065410e+04,    -3.4697478538468551e-04,    -2.6613813087550296e-07,     1.0932596990388144e-09, } , 
      {    5.7128795212808559e+00,     1.6343405092637287e+04,    -3.4164702239886845e-04,    -2.6737091510382832e-07,     1.2021391555423941e-09, } , 
      {    1.8010209463701452e+00,     1.6795914629239112e+04,     5.7469597434859906e-05,     2.0892594710692014e-07,    -1.0915580030181199e-09, } , 
      {    1.4234841786400645e+01,     4.1119929900771342e+04,     5.4782734716572962e-04,     9.9691388822611917e-07,    -4.9189244566741622e-09, } , 
      {    7.2544295109532984e+00,     1.4877251716472871e+04,     9.6277886864568112e-05,     2.4352448153382976e-07,    -1.1020930043086304e-09, } , 
      {    5.1728699313101112e+00,     1.5774383007934210e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.0829200510662538e+01,     2.3350214062369265e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    7.5439977471858990e+00,     1.6709188484903858e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2127914931543968e+01,     1.4458963476606300e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.0443173534010631e+01,     6.4959830970238500e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.2296119981201727e+01,     2.3425465123439975e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    8.8154767136526644e+00,     8.4203032925474163e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8016807892425759e+00,     5.1805635656959382e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.7404349614626248e+01,     9.7364819934558345e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1043356042795960e+01,     1.5805263377190377e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.8401003178513218e+00,     1.6815126679326244e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -1.7889406796892068e+01,    -8.2232827735906540e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    6.6890968959712902e+00,    -1.7701586161666755e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.4973017161143751e+00,     1.5664449703657538e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -6.4560055530751725e-02,     1.5607714120952032e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.0959985918190256e+01,     1.0287237317489253e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.8242597940665362e+01,     1.6746615553826621e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.1191229681720744e+01,    -8.4208469190736123e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    1.3256047409529748e+01,     1.5594559925638754e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.7044109856495273e+00,     1.6778157090339049e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -7.0850815429917660e+01,     1.6777699610403477e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.2262172370002707e+01,     6.0115802540326813e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2164341191324350e+01,     7.1306180404680454e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4633844840729346e+01,     1.6784699487247297e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    8.5136154897253231e+00,     7.0879582245123420e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3844537710670304e+01,     2.3453851056850235e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    2.6015065487046208e+01,     6.8548566928142900e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -4.3771649797087747e+01,    -8.2242690077133757e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.0746889958385012e+01,    -8.2235643757972230e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.3231483794062342e+00,     8.4864352547027920e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.3545057811305732e+00,    -1.2220331244494391e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -4.6669754489624950e+01,     8.4948565621206944e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0257375002946681e+01,     2.3976549079848173e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    3.7448645339977119e+01,     2.3975891815098621e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0613301348320768e+01,    -1.2167734598170728e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.9371363496444500e+00,     8.9558355481147610e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.2335172084844626e+00,    -1.7417726827564168e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0743465501979226e+00,     8.3804970602804096e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.2557703787591919e+01,     1.5687343924747858e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -7.3157984850781204e+01,     8.4490081857800305e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.8085062411478932e+00,     1.6239788191491891e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.2749315038635887e+00,     1.6211402258081631e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.5838223294454920e+01,     8.4179241292031711e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.7070100679250309e+01,     1.6591305316576352e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.7181677526315362e+01,     6.8946715947122029e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1283955538522791e+01,     1.5646817677383069e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.3552005351520499e+01,     2.3345611662486444e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {   -8.6144367651203780e+01,     7.1091900809163544e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0682392442683005e+02,     7.1093861840640329e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7478991289184869e+01,     1.5648240368316821e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.9067459053208147e+00,     1.6761933924957339e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    1.2484966764710553e+01,     1.6762381239272760e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0935998537631448e+01,     2.3973590615157213e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.1676650353638841e+01,     7.1085767870233167e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.5567612360575197e+01,     1.2031676115658047e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.3537058059245993e+01,     8.9232704579721030e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.6255779571522673e+01,     2.3978850279789578e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.4496764302802001e+00,     8.9842214815250190e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.4285126525008378e+00,     5.9056420596102980e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.1832842343409826e+00,     1.2024826113256631e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    3.7710288334717528e+00,     6.9853352056049225e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.9926366041057513e+00,    -1.2723723893444478e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.5218864963081227e+01,     7.1099994779570707e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.3814487633769128e+01,    -2.7562699840684991e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.6868622984949301e+01,     1.6682527778319225e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.8034255851224923e+01,     1.0707593280956098e+02,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.6880902128016224e+00,     8.3356950061028856e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7890316969349920e+00,    -1.0616689377928246e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -7.3097670628512024e-01,     5.1779647114804166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5601595772986212e+01,     8.5130959612874249e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -6.2623246740435299e+00,    -1.6746158073891056e+04,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.1676607020451002e+01,     1.1543852487580341e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7979779658436524e+01,     1.6841787385910873e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.3650719627313029e+01,     8.2445802080121903e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.3182036587100541e+00,    -1.2592181940311687e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.7168552976634786e-02,     6.4345798815715025e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7591162018172469e+01,     1.7043844863340753e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.7594661624663818e+01,     2.5298097272533392e+04,    -3.9726088367124645e-05,     2.3468322531964382e-07,    -1.3242685699506974e-09, } , 
      {    3.6786871045875870e+01,     2.5302699672416205e+04,    -3.4431090389177698e-04,    -2.6675452298966564e-07,     1.1476994272906043e-09, } , 
      {   -1.9202931411079746e+01,     8.3538363536957768e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.8473955212241442e+01,     1.5021522637745813e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    4.7950596979517242e+00,     5.6949386080586755e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.1246346977652406e+00,     6.5553005690156988e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.8930886471774402e-01,     6.2714412349131123e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    5.2985379885735648e+00,     1.7284526972738211e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.0396520452708131e+00,    -1.2201146376001327e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4212082111154704e+01,    -1.2186919466663801e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.4074129931759227e+01,     1.7121397124352879e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5436991609269933e+01,     1.5700498120061136e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.9299980421513791e+01,    -1.1795883902353441e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.8846372890238953e+01,     1.0129502963148821e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.3880535082696824e+01,     1.5631986994561516e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    1.0374047577327111e+01,     8.4423445459921440e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.8214389532728008e+00,     1.7312912906148467e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.3268655078211680e+01,     1.8440453666397525e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.1660450224141279e+01,     6.6004252531383872e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.5118009926666275e+01,     1.5663071051138375e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -2.1584463903262069e+01,     2.5144929072326249e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.3404146832105482e+01,     2.4133954801813827e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.0026573599296881e+01,    -3.9957992529212106e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.0375738473296474e+00,    -8.4400251130907345e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.2624243344598156e-01,    -4.4598059116526838e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -9.2352052049762712e+00,     8.3200549262406166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.6428958530309856e+00,     8.8690850234484096e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5907551959851418e+01,     1.5016920237862993e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    1.9476160265240310e+00,     7.8827108334581817e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1885985048289355e+01,     8.2944578810809344e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.7385360812981094e+00,     3.8646201409385367e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2393775462380528e+01,     1.0680258287182241e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1085746323970470e+02,    -8.4351971259688398e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3607877428384707e+01,     2.5266617045019640e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    5.3798649683475821e+00,     7.3783561752979103e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.2747688897506691e+01,     6.4813328634027248e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.0760175124465661e+02,     8.4317351890143618e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    8.4974587557826009e+00,     8.3577362595476134e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.1322326357468722e+00,    -8.4465840686898682e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    2.1879587623850782e-01,    -9.7238627550378478e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.0161406885640076e+01,     3.2304559556138771e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    5.8346915026240296e+01,     3.2305264188054920e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    2.8936760728632652e+01,     2.9447352798352578e+04,    -7.1676903554205456e-04,    -5.7176506329789924e-07,     2.5089059516050370e-09, } , 
      {    3.0317599882253454e+01,     5.5548055631488023e+04,     1.1087617939341001e-04,     6.1971158495585282e-07,    -3.2629948474764320e-09, } , 
      {    2.4734201302563800e+01,     4.8475968117292701e+04,    -4.6743991354733197e-05,     3.7022549502952346e-07,    -2.1358903053593605e-09, } , 
      {    2.8005171019723182e+01,     4.6591062251650517e+04,    -3.8752346876007665e-05,     3.6837631868703547e-07,    -1.9725711206039911e-09, } , 
      {    2.3337187606806108e+01,     2.9305377447189556e+04,    -3.4067328090775148e-04,    -1.3367782173643665e-07,     5.5383660488910041e-10, } , 
      {    1.7624411963546862e+01,     2.4118195798636407e+04,    -5.0628509613462032e-04,    -3.8131473532027807e-07,     1.5176219622508023e-09, } , 
      {    1.7710663339259924e+01,     2.2861591888208288e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    2.1521516503606104e+01,     3.0980733847095442e+04,     8.3932220891142531e-05,     3.8029953547203472e-07,    -2.0225941962208730e-09, } , 
      {    2.2436931670378691e+01,     2.0767136556829810e+04,    -6.0368542392458111e-05,     1.2773148497489202e-07,    -6.2329101284485277e-10, } , 
      {    2.0081377119991888e+01,     1.2438445132206358e+04,    -2.1266095015478429e-04,    -1.2298738917976270e-07,     6.1269298577579806e-10, } , 
      {    1.7682696881748555e+01,     4.7380556627969690e+03,    -3.6761723941001898e-04,    -3.7308987122025474e-07,     1.7942372561446590e-09, } , 
      {    1.9094869807851403e+01,     5.1568081522747279e+03,     6.2316025374692084e-05,     1.3965470175989127e-07,    -6.7331408846173465e-10, } , 
      {    1.9421516670182765e+01,     2.3837658657318603e+04,     1.1435992746015083e-04,     3.4985706632643663e-07,    -1.8730243274617701e-09, } , 
      {    1.6735947942493922e+01,     4.0633603296720292e+04,     1.7439547402433512e-04,     5.5821025455049395e-07,    -2.9094153817064361e-09, } , 
      {    1.5792566318209973e+01,     3.2723664361574603e+04,     4.5203633104671999e-04,     8.2023595337598530e-07,    -4.1409827276921796e-09, } , 
      {    1.2083123759922010e+01,     4.5960803116339521e+03,     8.4785152242840258e-06,     6.4997370341207835e-08,    -1.6083209057127743e-10, } , 
      {    2.5315284948052099e+00,    -2.6833779521217668e+04,    -6.0868276030374621e-04,    -9.3602894993492309e-07,     4.6197847191559572e-09, } , 
      {    1.0967239384211938e+01,    -8.0065080265549568e+02,     2.7546384627503485e-04,     3.2948863762334981e-07,    -1.6101583495641797e-09, } , 
      {    5.5132178680918731e+00,    -9.7576441824930062e+03,     1.2583532000561720e-04,     7.8153371354532411e-08,    -3.1973462269173897e-10, } , 
      {    8.7410618973947276e+00,    -1.1014248092921127e+04,     1.3116308299143423e-04,     7.6920587126207077e-08,    -2.1085516618815939e-10, } , 
      {    6.7401206668506539e+00,     1.7461447212910622e+04,    -1.0242551642421745e-04,     2.6946429209749117e-08,    -3.4231724395782118e-10, } , 
      {    1.0054216071866570e+01,     1.4948239392054378e+04,    -9.1769990452583402e-05,     2.4480860753098463e-08,    -1.2455833095066202e-10, } , 
      {    6.7984055850523468e+00,    -1.9186929229288244e+03,     3.6242340300383816e-05,     3.5171293309772398e-08,    -6.5701950063964304e-11, } , 
      {    1.1006997757097791e+01,     4.2376533811199464e+04,     5.4249958417991267e-04,     9.9814667245444453e-07,    -5.0278039131777423e-09, } , 
      {   -8.5702637054928243e-01,     1.9896805985103319e+03,    -2.2429021761753533e-04,    -2.5421491409050371e-07,     1.0432366234219325e-09, } , 
      {    1.4464017193636058e-01,    -9.4805206021834201e+03,    -3.6326321791531894e-04,    -5.0801574881597172e-07,     2.5514192633015325e-09, } , 
      {    8.5214531916044862e+00,     2.5057091959614438e+04,     2.7139529509915140e-04,     5.3254024280037007e-07,    -2.6752503737305300e-09, } , 
      {   -5.7891270243987570e+01,     8.4550416602227833e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3022474039357284e+01,     1.6650823893647768e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1453958628682646e+01,    -6.5589555991336965e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1953746696352420e+01,     6.4809861772761333e+03,     4.8687074555849077e-07,     6.6846546683695823e-08,    -3.2415127532664681e-10, } , 
      {    3.1940031158177835e+01,     8.4357673574330111e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.9259442624643430e+01,     8.4311649575501906e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    8.7259026670495654e+00,     7.7375900877042541e+03,    -4.8408922402585326e-06,     6.8079330912021156e-08,    -4.3303073183022644e-10, } , 
      {    1.5209557183166643e+01,     2.3347918492259334e+04,    -1.2752546000740873e-04,     5.6156114127021881e-08,    -3.8300765621334456e-10, } , 
      {    1.2854002632779840e+01,     1.5019227067635886e+04,    -2.7981786776973489e-04,    -1.9456276002763283e-07,     8.5297634240730622e-10, } , 
      {    9.5981921459656174e+00,    -1.8477052473473168e+03,    -1.5180553701676770e-04,    -1.8387232747095888e-07,     9.1183272329400397e-10, } , 
      {    7.1681781352402378e+00,     1.6133855626900990e+04,     9.0950123878751090e-05,     2.4475726576215507e-07,    -1.2109724608122099e-09, } , 
      {    1.5847795555505810e+00,     9.0617681127056603e+03,    -6.6670046869392126e-05,    -4.7288241641743030e-09,    -8.3867918695138641e-11, } , 
      {    3.9403341059373833e+00,     1.7390459537329112e+04,     8.5622360892934055e-05,     2.4599004999048043e-07,    -1.3198519173157896e-09, } , 
      {    4.8126235848534353e+00,     7.8051642022775404e+03,    -6.1342283883575091e-05,    -5.9616083924996316e-09,     2.5011537808440947e-11, } , 
      {   -7.7077499483622147e-01,     7.3307668808221115e+02,    -2.1896245463171829e-04,    -2.5544769831882901e-07,     1.1521160799255120e-09, } , 
      {    2.4570690344666328e+00,    -5.2352722234590965e+02,    -2.1363469164590129e-04,    -2.5668048254715437e-07,     1.2609955364290918e-09, } , 
      {    1.1996872384208951e+01,     5.8526842220620729e+03,     3.1507522384670024e-06,     6.6230154569533156e-08,    -2.6971154707485704e-10, } , 
      {   -1.6903927285047136e+01,     1.6759856382173639e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.4295619111581516e+01,     1.6764458782056459e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.2215272762656280e+01,     8.4337947898663733e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.8612675903060296e+01,     8.4331375251168283e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.4309301246739222e+01,     1.4809677601899584e+04,     1.5277927850788466e-04,     3.1756542083835050e-07,    -1.5601352739472976e-09, } , 
      {   -1.2118317078128324e+01,     7.1069869325487834e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.9081151311438475e+01,     7.1115893324316021e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.2034509595846018e+01,     1.0512704882622563e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -4.0348183020021523e+01,     1.0442241691007587e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -3.5652204641531711e+01,     1.6761805266156978e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    4.6760656193048888e+01,     1.6762509898073124e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6158972708624972e+01,     8.4460853959095875e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.7384748772114321e+01,     8.4338184734496772e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -3.4704133498227762e+01,     8.4331138415335245e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.1218030627621413e+01,     1.5645227822908535e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    3.9981369282143447e+01,     1.5649830222791355e+04,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -8.8625090875093271e+00,     2.3973919247531987e+04,     2.2103066262008943e-05,     3.0749138039583929e-07,    -1.6734313830857853e-09, } , 
      {    4.2336962861875421e+01,     2.3978521647414807e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    9.0353062556434214e+00,     8.3989733845695646e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9005212682955626e+01,     8.4246060019510587e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.2261023169769846e+01,    -8.4423263130321466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.5956470892488781e+00,     8.2937465356140583e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.4994841985014755e-01,     8.4327548120247247e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.3473515602541637e+00,     8.4341775029584769e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5406170605362565e+01,     8.4023145623526252e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1151613411977706e+01,     7.1096404484482682e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.0476961688462925e+01,     7.1089358165321200e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.3131138017979836e+01,     8.4394996319343518e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.3009929753381630e+00,     7.1066583001740100e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2375722911829342e+01,     7.1119179648063782e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.0968115026462744e+01,     1.6761828949740277e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.9859659949986568e+01,     1.6762486214489822e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    9.1479187827826195e+01,     7.0937461042017640e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.4518690453782483e+01,     7.1248301607786234e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.4548385641568860e+01,     2.5088547806797091e+04,     3.9287105791049491e-04,     7.4681140618562724e-07,    -3.7373801863053014e-09, } , 
      {    1.2504927328413261e+01,     8.4526538272476319e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2123549274855723e+00,     8.3783548220585890e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.8961634141800552e+00,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1922090343899807e+01,     8.3647398410270489e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3296276823162092e+01,     8.3263902246820398e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4290586701908992e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3879936099552321e+01,     8.3931257744373070e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4637661618298718e+01,     8.3117707438158577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7437359142554332e-01,    -8.2949343774993868e+03,    -1.8310905271330885e-04,    -2.8716658492405232e-07,     1.4098381846665302e-09, } , 
      {    1.7438896895435052e+00,     8.3624484717475134e+03,     1.2147576281134351e-04,     2.1427116338525712e-07,    -1.0621298125747714e-09, } , 
      {   -1.5254898793006621e+01,     1.5540453090040384e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2331952597015430e+01,     1.5545055489923205e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0782296009688613e+01,     1.4967415802147443e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5064537788217919e+01,     8.3283391086653737e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6058841305283707e+01,     8.3290437405815264e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7610443106694792e+01,     7.2117616654169342e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9976399005958271e+01,     7.2163640652997528e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.1160340976891874e+00,     1.4756712350595821e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4277976518600450e+00,     6.6387243775239922e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6188676990810691e+00,     7.1780144489547456e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6990915359575052e+01,     1.5542401974023718e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4068029601709227e+01,     1.5543106605939871e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6450670256454769e+00,     7.9356704549304632e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9337861505301781e+01,     7.2137105494002708e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1703831564290098e+01,     7.2144151813164190e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7507598657758208e+00,     7.7533529367890978e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3686940173943247e+01,     1.5149733320288806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5599118685322999e+01,     8.0001048047443182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3492863348135220e+00,     8.7217123943164388e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7613996924682542e+00,     6.4280209259723688e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.2411922690179651e+00,     8.9040299124578014e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7291777483150785e+01,     1.5508997242857731e+04,    -3.5366531172850963e-05,     9.8565433288919105e-08,    -5.8187338006766507e-10, } , 
      {    1.2785067090303437e+01,     1.4392077314313090e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1331398100476823e+01,     6.8210418956653575e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4390748241399915e-01,     7.1496285155444875e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7852743225235166e+00,     1.5576511337105858e+04,    -9.6999821074816303e-05,     2.5670011750123932e-08,    -2.3416500797590593e-10, } , 
      {    5.1822649901576607e+00,     7.2309835461659377e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7950942362113915e+00,     8.5044838432356883e+03,    -2.4948809356431073e-04,    -2.2496159594193106e-07,     1.0032734316880736e-09, } , 
      {    1.4417941908889736e+01,     8.3155735134251827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3976490113691220e+00,     8.2888765227255371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5395314235603161e+01,     8.3685063265213612e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5303048257241230e+01,     8.7861467441302957e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3744726468567725e+00,     8.3294027700903262e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6198032213822442e+00,     8.3279800791565740e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9356211338075342e+00,     8.3467156328252495e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0430917574473169e+01,     6.0633858896896400e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7695867028495806e+00,     8.2949099971682899e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.2708093757403583e+01,     3.6048416403597997e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.1997242663831340e+00,    -8.3624728520786120e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0859893729263035e+01,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2354868452613557e+01,     1.5020384899358634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0892554215778979e+01,     1.4954261606834163e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.4275067144967011e+00,     8.1709570702830179e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2421699413438263e+01,     1.5385019935641363e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9334777449026930e+01,     8.3512333297556943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4936214380650647e+01,     7.1803058182342811e+03,    -1.8765893893517714e-04,    -1.5215344086573563e-07,     6.5411061855298576e-10, } , 
      {   -2.0940722332051283e+01,     1.6655081649305488e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.5639481898113532e+00,     8.4864257789638814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5186367790066271e+00,     8.3260615923072655e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4755017863933197e+00,     8.3313212569396328e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0063847244772138e+01,     7.0563285110179122e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6646141119982047e+01,     1.6659684049188309e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.5105310400401235e+00,     8.3816605218346413e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4620823032673842e+00,     8.2757223274122589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0741236837276458e+00,     7.5426494852374744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.9159430258126271e+00,     9.1147333640094239e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2754871376042701e+01,     7.2140628653583444e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0000471167607484e+01,     6.6916934747351834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0159067489872236e+00,     7.1610937681471532e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1014145706632462e+01,     1.4991998965948376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.4015993268578768e+00,     7.8063220340002890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3287072679778543e+01,     7.2670319625695356e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8531804742384161e+01,     6.6255701822107130e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1452808786180636e+01,     1.5489785192770603e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3917562083020432e+01,     7.7894013531926967e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7615556085157120e+00,     1.5502939388083882e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.8877721035793931e+00,     5.7533848783435178e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4277992804588946e+01,     1.6693431265650499e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1279547707665643e+00,     7.2114330330421608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5467539679570926e+01,     1.5595723387192986e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3237752408838681e+01,     7.2166926976745290e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7954456277679668e+01,     1.6328796229367768e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7359516834500294e+01,     8.3114184278577814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1496279258805950e+01,     1.5540124457665610e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5580094559371894e+01,     1.5545384122297977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3350923882914699e+01,     1.5582569191879707e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4365461831883479e+00,     7.1742479634604324e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6270682209699523e+01,     1.6118092777816146e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4297316403756275e+00,     7.2478199124824077e+03,    -2.4929222883714246e-04,    -2.2504886240453078e-07,     1.0018189906447449e-09, } , 
      {    3.0811348476251037e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9817120318249156e+01,     8.3290200569982226e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.6020432922962335e+00,     8.8510608152466102e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6726516468134172e+01,     9.1791677138232808e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9936452388274049e+00,     3.9302096969298748e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2834733377802712e+01,     7.7401987414758187e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0947683201804253e+01,     7.2538777672562564e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9626100581248265e+00,     7.7779361005900309e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6576154108585861e+00,     6.6633075413249253e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4666061357129315e+01,     6.4434349813856670e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2410491969975661e+00,     9.4793684002921545e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8306544275262446e+00,     8.9171841077710815e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5707963267948966e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2292684376175934e+01,     7.3717972196987757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6994506730698923e+01,     1.6640462168439306e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0303623474082535e+00,     8.8794467486568683e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.1164269516179726e-01,     3.3757047124063320e+01,    -3.0816644950982666e-05,    -3.6447710769397583e-08,     1.7385418604587960e-10, } , 
      {    1.3390854472159115e+01,     1.5935775259674781e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2002398616778342e+01,     8.1578391590847514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1408161054442658e+00,    -1.6920680807593385e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4631166420530946e+01,     1.5700488644322226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5615341074686029e+00,     7.8604193938597496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3223207623757931e+01,     1.4445046411524281e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1811809589612592e+00,    -1.6623625802122835e+04,    -3.3540146047563498e-04,    -5.3788545907870699e-07,     2.6458221832871811e-09, } , 
      {    4.0994444144632327e+00,     1.6691139896370962e+04,     2.7376817057366969e-04,     4.6499003753991189e-07,    -2.2981138111954222e-09, } , 
      {   -1.2899343854058355e+01,     2.3869144514663836e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2967021409963756e+01,     8.9400783288614002e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4687534615077524e+01,     2.3873746914546657e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8148699865073269e+01,     9.5721886835162695e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3288117524145820e+01,     7.1365106043462201e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0728914108200815e+01,     1.4339108217101899e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6576296683582761e+01,     8.5508601287777383e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5319955656011345e+01,     8.2490616208276260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2288121115148506e+01,     7.7364322559815055e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5636027634225021e+01,     1.5465202028969670e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6318122710065410e+01,     8.4083212284192741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6092563442631494e+01,     1.4916747904877662e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1646117618564862e+00,     1.4128404765550276e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.1547233805163408e+00,     8.3910482316536072e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0979362826320305e+00,     7.2916151263704687e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2075551737401192e+01,     1.4994076508732076e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5110888691824371e+01,     1.5542754289981794e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.6475383400118124e+00,     1.4363691380902832e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1371462799205666e+01,     8.4280182194937279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4583609981628793e+00,     8.2511391636113258e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4576947136565801e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5211773030677520e+01,     1.4912145504994842e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4739002980962740e+01,     8.4177089382382401e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.7548541075995487e+01,     7.2140695789090714e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9914492500770123e+01,     7.2140561518076174e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5466754772342988e+00,     8.4062436856355744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1483233995230057e+01,     1.5620306550993919e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.3136306419482402e+00,     8.5044594629045914e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4207231277758078e+01,     8.2588016466046665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6947197594137869e+01,     1.4914094388978176e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.1978602121450237e+00,     1.5463124486185970e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9712130999230439e+01,     6.6653850841086260e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0800994743650662e+01,     1.7114838168753744e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4701458867324925e+01,     1.6065123680604955e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0866042075988281e+01,     6.1163549869008320e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2223287167111309e+01,     1.7507859138446733e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.5413025453604625e+00,     7.1344330615625204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0144735318698217e+01,     8.3200549262406166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.7761828350448035e-01,     7.2271807765566118e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9571292120536306e+01,     9.9652096532092582e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4460518005805429e+00,     1.4521425735243263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2372561543567198e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.1917452020497450e+00,     5.8849268314763094e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3136676234771929e+01,     2.3296107226770895e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.5192992210416961e+01,     1.5542761003532520e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2270037547057939e+01,     1.5542747576431069e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5835404024904793e+00,     1.3763769729267548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3645980216571822e+01,     1.7900880108139721e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8051323730440110e+01,    -8.3226579501806991e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7577536780303952e+01,     1.5288322850305891e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9620849259825604e+01,     8.3347248990662010e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.4009587942463504e+01,     7.7800136433737316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1075899669189724e+01,     1.4935776612914104e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4094890053997720e+00,     7.2133515198914674e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4092395880171239e+01,     1.4914799020894327e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5955468115702695e+01,     7.2147742108252214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4307937367170766e+01,     1.5622384093777619e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9740719110849536e+00,     1.5506705873578196e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.6735530757142163e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1921849645786551e+01,     7.2936926691541685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6233595554796633e+01,     1.6721817199060755e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5222183505572900e+01,     6.9596314256824407e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8615435572746026e+00,     7.8025555485059758e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3649748332574774e+00,     6.0104167924784488e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1259314744290450e+00,     8.3598698281385277e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2242668687123714e+00,     1.6131246973129426e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2708937740911736e+01,     1.6657030533288827e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.8414357531324157e+01,     1.6657735165204973e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.1922710622302741e+00,     8.1643981146838842e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6973700305633219e+00,     8.5639780399760057e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8852440057834087e+01,     1.5542425657607024e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1775397426195170e+01,     1.5543082922356567e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.2150386820028878e+00,    -1.6899670470994574e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9555033628247980e+01,     8.0742599849475464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5068531038528825e+01,     1.8293901077832710e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7847490148721263e+00,    -8.3456121054310406e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.2968603189129153e+00,     6.0349999562793819e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2998496281066576e+01,     8.8773692058731685e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1033600947043603e+01,     7.6070838350513322e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2510001386843900e+01,    -2.3011999414102320e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1470343230313942e+01,     2.3085403775219267e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9647334302465616e+01,     2.3837688667481183e+04,     1.1692587658947522e-04,     3.4928430744357381e-07,    -1.8178573786883158e-09, } , 
      {    2.3051201765978298e+00,     8.0934048092708945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1928364566255098e+01,     1.3816738826478739e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9607579778445015e+00,     7.2206218209574781e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.5636522138994486e+00,     8.5831228642993538e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4640963750297765e+01,     2.3871093398647172e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.6429193586976055e+01,     2.3871798030563317e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.9297138403145265e+01,     6.5854029643547265e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8206430216300937e+00,     9.4263993030809634e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8100329409316265e+00,     5.7997133409268263e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2677593993437917e+01,     1.5525833609174202e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2140826778515216e+01,     2.3905202761729310e+04,     5.5292586687509885e-05,     2.7638888590477866e-07,    -1.4701490065965570e-09, } , 
      {    2.0993758203117260e+01,     1.0358230622902245e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2883682836965733e+01,     7.1915209602261002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.7208443314139448e+00,     6.6070851882906527e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2123393344199034e+00,     8.3500213326013018e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.5262421746037074e+00,     8.2937465356140583e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.7812269604094020e+00,     8.3073615166455984e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2318943522763885e+01,     2.3907494131008843e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.9271555674068748e+00,     8.3286914246234501e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6710594563588153e+00,    -1.2904209778773438e+03,     3.1012509678150921e-05,     3.6360444306797873e-08,    -1.7530862708920817e-10, } , 
      {    2.6491019138031284e+01,     1.8686922047525695e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5484756428361255e+00,    -1.9006704986510808e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6042483448502843e+01,     2.3478424744912256e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7980224743506874e+01,     8.9438810984707270e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6038572439643728e+00,     7.1960386571565450e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.3155064767683387e+00,    -9.6191124025007939e+03,    -1.2127989808417527e-04,    -2.1435842984785683e-07,     1.0606753715314426e-09, } , 
      {    3.0389662891071958e+00,     7.7217137475557583e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1728014760858073e+01,     6.5861075962708755e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4363122550339407e+01,     8.2353914200968757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2895999440985733e+01,     1.5073353996569826e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6181701326288511e+00,     9.9007753033953995e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0006305568552225e+00,     1.6264361879553911e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7334903171007316e+01,     1.4880695287643670e+04,    -3.2702649679942452e-05,     9.7949041174756439e-08,    -5.2743365181587530e-10, } , 
      {   -1.4457867202741696e+01,     8.3266138818397485e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.8522754999882221e+00,     7.4684943050342481e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5452178047504948e+01,     8.3307689674071516e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7176703295770167e+01,     7.2313358621240131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0909816143711506e+00,     6.1927343106198141e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1798850240859586e+00,     9.5077543337024126e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3736416564294608e+01,     6.5880564802542103e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.9935020992113337e+00,     1.1790629090789623e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0186211605273366e+01,    -8.3160721862054634e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.9963643744112083e+01,     1.5578802706385393e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6656067747302130e+00,     6.5497068639092031e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8038989132537530e+01,     8.3413106630414368e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    8.8781602408747489e+00,     8.3640284955601728e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1051251992640010e+00,     1.6082044361412549e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2208208259342086e+01,     1.4915158050403128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4820117930541954e+01,    -1.1149808752231806e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.2474936579960428e+01,    -1.1142762433070307e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8070395712045411e+01,     8.3654511864939250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0918883257988057e+01,     1.4325954021788621e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1624598887917999e+01,     1.4861477607725061e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7644059007124664e+00,     7.2478442928135055e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.7567323335374020e+01,     6.5834540803713917e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7913509628162650e+01,     1.9079943017218680e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6496502040169112e+01,     7.2137342329835719e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2637071055852401e+01,     8.8871092316502090e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0116156761102832e+00,     1.5478319940167938e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.7048607840001324e+00,     1.7050403818939889e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.4130677206756960e+01,     7.2143914977331169e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8284000103800473e+00,     1.4948209381891797e+04,    -9.4335939581907799e-05,     2.5053619635961265e-08,    -1.7972527972411611e-10, } , 
      {   -2.6757422359253518e+01,     8.3237603923658535e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0101876081901533e+01,    -3.2858661987913069e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.6856451561488104e+00,     1.9651048484649373e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.2623941373634411e+00,     1.5720838787719499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9543480006239289e+00,     8.9356691016911409e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2314298690196077e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0920876540767068e-01,     1.5797185729657698e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0542389354598878e+01,     6.7446625719463755e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.7632296362412436e-01,     7.3073628698849197e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2026620289930925e+01,     7.0382704467461062e+03,     1.8330491744047710e-04,     2.8707931846145255e-07,    -1.4112926257098591e-09, } , 
      {    8.6549295441931761e+01,     8.3131493963350204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7492475044763079e+01,     8.3227913705382216e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2416177556536844e+01,     1.0751251592595232e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.5555558602059392e+01,     8.3442334529118798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8051661686739955e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.7457364678276370e+00,     7.2119853225746447e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7696310430952744e+00,     6.7566075458514997e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1111749002589200e+01,     7.2161404081420451e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0595719838647780e+01,     1.7232721337081250e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6090990893192004e+01,     1.6093509614015213e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6406304175175350e+01,     1.5248507948407980e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8739722762108023e+01,     1.5328137752203804e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8617398921827320e+01,     1.7268769753484848e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.7965576702738275e-01,     8.0968018900797906e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.3950136520368126e+00,     1.5540676747198093e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0378266523647337e+00,     8.0132227159425856e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8641242169646475e+00,     6.5864666257796780e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3472238653401675e+01,     1.5544831832765496e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.8342992319406939e+00,    -1.5076495289580932e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.9048182951835209e+00,     7.8592911312114811e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7751343649568817e+01,     8.3336224568810467e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4761808014437257e+01,    -8.3467403680793141e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.3215908166550719e+01,     1.5033891410629989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8321696982388179e+01,     1.4660015265260348e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5350918909716293e+00,     8.4346296190458324e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.3222446781142381e+00,     7.7627406466080629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2910319328104913e+00,     1.3970670411209845e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1040259048514825e+01,     1.0293796273088388e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0622160227050941e+00,     8.7980917180354190e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.3726354468882448e+01,     7.2139648137845052e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0609230208977044e+02,     7.2141609169321837e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3464671437442268e+00,     7.3783561752979103e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.6383901247274856e+01,     6.9994463275803537e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4462699895742928e+00,     8.2227532302010677e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5810337156249476e+01,     1.5508972862526634e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -2.3253127604886672e+01,     7.6980826396364973e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0107140858116503e+01,     2.3581258181579246e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4050721268679734e+01,     6.9198165237845287e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5446192868583598e+01,     8.3124820892827338e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2297038144865127e+00,     5.4350783046440974e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9097345110352046e-01,     6.5213209304989450e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6776774811027330e+01,     1.6644264938048633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9335937213065026e+01,     1.9472963986911669e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4358630773206306e+01,     1.5564084197959646e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0197672992400094e+01,     8.9546978097275824e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.9573367321853201e+00,     8.6441601333043145e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0726391240641618e+01,     8.1140748868454593e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.0804110224997316e+00,     1.4914446704936252e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7311758963957917e+01,     7.7690759450494879e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1445330241159617e+01,     8.2534403635527378e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3991886889101323e+01,     7.7736783449323084e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2705533510894901e+01,     1.5521424382003943e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0045108336251865e+00,     7.1250453517435544e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5369728395028384e+01,     8.3331006517937094e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1921089063532925e+01,     1.5607188639795653e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.2135146584526559e+00,     2.2754515955398729e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.4276856406373891e+01,     2.3935880064419103e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.2979473813006459e+00,     1.7579241861223818e+02,    -4.0178050132663691e-04,    -4.7568047009658573e-07,     2.2392574303087244e-09, } , 
      {    8.6646257547950079e-01,     8.2485093312951431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.8522587123740138e+01,     8.2716035259598684e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5204119905390153e+01,    -1.3117911198267393e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.0373331708330241e+01,     2.2759118355281553e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2038728807464047e+01,     7.2353927733361961e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4582245183637896e+00,    -8.3095037548674209e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.9736012518492544e+01,     8.5433079624014408e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4310990375415933e+01,     8.3478790943794811e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0320377906830483e+01,     7.1927329573804927e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0125528916092877e+01,     8.4088735179517589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5132826109388615e+01,     2.2720768738936542e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0033683728157849e+01,     1.5227285581300930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9350895759963969e+00,    -2.2653055349338092e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3338442793937013e+01,     8.3908245744958967e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1749357479736059e+01,     8.0569869881818795e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4367116524910992e+01,     1.6486530583708201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2085740804826877e+00,     8.3985212684822091e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.2208746738895631e+00,     6.5327861831016098e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0401358256293864e+00,     1.6617567947348987e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7752669160471026e+01,     1.6697197751144813e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2122269724851034e+01,     8.4066883115158762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9426082321282330e+01,     9.3369020681637139e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0855769659734790e+01,     6.7051999860065380e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5495050942413489e+00,     7.3849151308970431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2676159718728911e+01,     1.4598977996255388e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1807815524050802e+01,    -8.5021924739561528e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.6049712113173999e+00,     7.2839526433771289e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5032617327838459e+01,     2.3854525033797650e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.1914843339237322e+00,     1.6799418220735075e+04,    -6.6379040851001885e-05,     6.2204988982121232e-08,    -4.0656475297845690e-10, } , 
      {    1.3346830592474877e+01,     1.4328798935846411e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1529780916787738e+01,     2.7511467878509129e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.9375966705289969e+00,     1.6091432071231513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4540356351553669e+01,     1.5527212261693365e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5233744924169974e+01,     8.9930474260725914e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4998297336949065e+01,     7.7000315236198321e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2173867649714126e+01,     7.1985208370699147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1621996346744448e+01,     8.3110593983489816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2098501222702884e+00,    -8.2926430082198494e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3936847087209834e+01,     7.7648181893917636e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.1370796438029984e+01,     1.5542656238407953e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0844785884751370e+02,     1.5542852341555634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6963276839373314e+01,     9.0214333594828495e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7298918719082970e+00,     1.6658094194713776e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.9751976964490154e+00,     1.6656671503780024e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1701033068479974e+01,     1.4834264611607941e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6126434390451259e+00,     8.5382408903597516e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.7702429191867699e+00,     7.7713771449908972e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3838577000078114e+01,     1.1144272562288221e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6089671501614511e+01,     4.5745531950684426e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.9809133327221460e+01,     7.2296048936467741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.7466643080138567e+01,     1.5558296318270224e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8812659529080136e+01,     2.2181478667505791e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.5515821018962228e+00,     5.4880474018552895e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1105179756809470e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9520990993980497e+01,     1.4699830167158261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0214885227770986e+02,     8.3281834659843353e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0057933583052605e+02,    -8.3291993832625649e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.0363738859581044e+01,     7.4459523999020039e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5363034252062334e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.7897769322884773e+00,     8.9562876642021165e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0322788176294420e+01,     7.1971421845507530e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.0758302293802672e+01,     1.9865984956604658e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7408129171882862e+00,     6.5857552803128010e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2462337160783933e+01,     1.0686817242781375e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1856068247447613e+00,     7.7010951850447836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.6759535641240699e+00,     6.8985941566774809e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0090545622120732e+00,     1.6833175267859137e+04,    -9.7195685801984544e-05,     2.5757278212723649e-08,    -2.3271056693257730e-10, } , 
      {    3.5251054780797144e+00,     1.4874631803038339e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4013305399831964e+01,     7.4286794031363352e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.9266074629548076e-01,     6.6026759611203952e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3658451678800329e+01,     1.4498015508735472e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4717593507364322e+01,     2.3349076323982084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5624022173068806e+01,     1.4917076537252435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7159668407151507e+01,     8.3444239086227499e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.3098837808442241e+00,     7.5083092069321601e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1609972742155787e+01,     8.5359396904183413e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.0020983769497311e+00,    -8.3352503802225838e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.3255469366561460e+01,     1.4957106520891955e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0749052772507575e+01,     7.2501112817619451e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.3253290200544232e+01,     2.3282953031457615e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1269695544491997e+01,     1.5576535717436956e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4780347569071154e+01,     2.3713711360264813e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.8571998557733114e+00,     1.5462572196653486e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5177117847156438e+01,     7.8198285452716436e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6593591462816775e+01,     7.1802814379031852e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.9693682324922012e+01,     8.5405420903011618e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.4776671085985029e+01,     1.5072181059276681e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5976002104678241e+01,     6.3313238406368982e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7218865593853909e+00,     1.8024208201798999e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2952141357824003e+01,     3.1441677575438998e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0290878072329653e+01,     1.6675407057448698e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8560578983245559e+01,     5.9972625971651705e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3124929983241440e-01,    -7.6666324222853264e+03,    -1.8577293420621736e-04,    -2.8655019280988964e-07,     1.3553984564147406e-09, } , 
      {    6.4939945829384307e+00,     7.1338807720300365e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6038505816034728e+01,     7.7007361555359812e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3802662934081212e+01,     8.6454796713295091e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0126925530219566e+01,     1.6623601421791740e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    5.3649600093223038e+00,    -6.5589555991336965e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.8462676530616715e+00,    -1.6691164276702060e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.1539420612778127e+01,     7.4857673017999159e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1673534220328801e+01,     1.5757370827759785e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3215284168741285e+01,     1.6650823893647768e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.8585169709074083e+01,     2.4983773073928940e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.4223686892496660e+01,     9.5154168166957534e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6923134912514737e+00,    -1.1675976564762971e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0975689469330572e+00,     2.3831630812707332e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7153773522289825e+01,     2.1970775215954163e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8549171356679771e+01,     2.2541905132244210e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3499380441630553e+00,     6.5055731869844931e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4161112733827625e+01,     8.8375543039752556e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8625293755450656e+01,     2.4446784202439594e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.8584907375966253e+00,     7.7003838395779067e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0781524896118240e+01,    -1.6369979498882660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2941935095585174e+01,     7.9225525437321949e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7817855565359331e+00,     1.6499648494906469e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.3683163853142233e+00,     8.0344450830496344e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2180635701067089e+01,     2.0259005926297643e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9001696470360059e+01,     2.4988375473811760e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5731052065552110e+01,     2.3911260616503158e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.2546295283989277e+00,     6.9918941612040562e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6477387219250328e+01,     8.2092467189297131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0327304793920057e+01,     6.2702865716319375e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2156718998350151e+01,     1.6168760675085927e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.9517173322726862e+00,     7.7376200978668339e+03,    -2.2749431109341485e-06,     6.7506572029158348e-08,    -3.7786378305677235e-10, } , 
      {    1.3369666525837449e+01,     2.3320690390571825e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5652193186591031e+01,     8.3014614625603699e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3765852937183919e+01,    -8.4738065405458947e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.6633309953480879e+01,     2.5022122690273947e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9312880199146225e+01,    -1.7031212424127366e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.9662454231064865e+00,    -1.0616594620539142e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3484493679814360e+00,    -8.2822907587503032e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.7476686994236914e+01,     8.4481361303171871e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6456002414373739e+01,     1.5778040905334350e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8945764133481298e+01,     2.2756464839382068e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7685189607456316e+01,     2.3924414811816438e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0849171108715986e+01,     9.7299230378567008e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2105587049134755e+01,     2.2757169471298213e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3885028010428462e+01,     2.3818476617394052e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.2447494310341710e+01,     1.4365768923686534e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5260913000631078e+01,     1.1537293531981210e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0918187489293663e+01,     8.3750920904965969e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.7867350267258075e+00,     8.3455521712710179e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0309379032740964e+01,     2.4657487653991222e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.9178436697719619e+00,     1.6815117203587331e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3194262429503461e+01,     8.3106424811675879e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0041318562945152e+01,     1.5542042944514918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0899932115569786e+01,     6.6284150962685044e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3860993546068489e+01,     2.3868815882289062e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.0741442777988874e+00,    -1.0808715121410423e+03,    -4.0158463659946864e-04,    -4.7576773655918550e-07,     2.2378029892653955e-09, } , 
      {    2.6252497486449959e+00,     8.6229377661972649e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2956873666577913e+00,     1.0054706888126506e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9240916289784673e+00,    -8.9232363327134481e+03,    -1.8044517122040035e-04,    -2.8778297703821500e-07,     1.4642779129183203e-09, } , 
      {    1.7926251460466666e+01,     2.3874075546921427e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.5994191925688330e+00,     6.0697767706769139e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4374609571711430e+01,     3.5371887272368867e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0842831420549352e+01,    -1.2723629136055376e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6972760150150737e+01,     1.5543465635448671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.8742347287617491e+00,     1.6654753016930714e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.8310703050976898e+00,     1.6660012681563087e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3884687780717828e+01,     1.1079838212474362e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.8559673646665633e+00,     1.6710351946458093e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0663643045071066e+01,     1.5837000631555609e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2270176173233573e+01,     1.7443424788632878e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0366942706727581e+01,     8.9569990096689926e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4280051868163794e+00,     1.5871340909860926e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.7172521859966231e+01,     6.3711387425348103e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5012406027625033e+01,    -8.3129589406241503e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.8063951725127492e+00,     1.6604413752035707e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9011815772025692e+01,     8.7085944831181696e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7865720459570277e+01,     8.3593443469821468e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.9354586052953602e+00,     5.6419789865863950e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4818721650790675e+01,     7.8423704504038878e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5003351165871173e+01,     1.6640109852481233e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2201234025177081e+00,     7.7430436555336100e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4430521264138534e+00,     9.5323374975033457e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0301027483063718e+01,     7.2490077543677362e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8206738582628617e+01,     1.5622936383310103e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7714018998551069e+00,     1.6135013458623740e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5869536893500102e+01,     7.2942449586866514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1289279681201634e+00,     6.9143419001919319e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2577316673615648e+00,    -1.1146285592651057e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5746497849497416e+01,     2.4264466684298233e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.9147274197862714e+01,     1.6173363074968747e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5331132594825192e+01,     7.7710248290328227e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.2011017211777300e+01,     7.7717294609489718e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7674744979233182e+01,    -8.3441105357631277e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6665223406372565e+01,     8.4706780354494313e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1786330550338203e+01,     7.1517060583281882e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2647181363892155e+01,     8.6852945732274220e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3276877578922711e+01,     6.5883851126289865e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9967009205338410e+01,     7.2226993637411788e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2433595252023874e+01,     6.7434896346532323e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5308312425827893e+01,     8.6284123897898626e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1728729302418643e+01,     2.2363796185647152e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0991529126316820e+01,     6.0001075112229610e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0693555246201619e+01,     1.4739645069056172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4961933047873487e+01,     8.6056647694315961e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2856402311597833e+01,    -1.7272996765668267e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.8565684423532058e+01,     7.7229257447101518e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1194507189050480e+00,     8.3446909328474467e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2662739536641027e+01,     1.5307467674629237e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0840664683080909e+01,     8.3129436811089981e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8733058158288842e+00,     7.6474147423667155e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6979250507228450e+01,     1.6679924754379146e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3602931049251517e+01,     2.0652026895990632e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.3800433232233509e+00,     9.2114304493448963e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1299646441561816e+01,     6.1693240841120232e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7436235177688459e+00,     1.3735383795857289e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4149605343187666e+01,     8.9504400540698607e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.6112356806674946e+00,    -1.1172583915812884e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.9464600084151682e+00,     1.1506769756687036e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1262737382896226e+00,     6.5459403784148890e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5579368238031766e+01,     2.2773737836147735e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.1087065140403993e+01,     8.3286847110727249e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.0092782558287297e+01,     8.3286981381741753e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4043689134238839e+01,    -1.1119987269489220e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.4424036340739192e+00,    -1.6653838832985248e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.1496629710428490e+00,    -8.2611773303753234e+03,    -2.1392569766429153e-04,    -3.2361429569344993e-07,     1.5836923707124101e-09, } , 
      {    4.9899779781786997e+01,     1.5367952654101717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9734978402214525e+00,     1.2036460728798950e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7030382597217741e+00,     8.3962055188715749e+03,     9.0659117860360849e-05,     1.7782345261585954e-07,    -8.8827562652889158e-10, } , 
      {    1.7230705712425177e+01,     1.7272572523094179e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0153268554080348e+01,     8.3265491864016294e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.5837539143002406e+00,    -8.3308336628452726e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2847621644490184e+01,    -8.3418093358217156e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4979339893974252e+01,     6.5520038630202207e+03,    -1.8499505744226863e-04,    -1.5276983297989825e-07,     7.0855034680477553e-10, } , 
      {   -1.0464872915609799e+01,     8.4683768355080210e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5797103102894315e+01,     3.9302096969298746e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7870125848732323e+00,     7.7341465165334521e+03,     1.2413964430425204e-04,     2.1365477127109444e-07,    -1.0076900843229813e-09, } , 
      {   -1.8340448272107700e+01,     1.5423309584288058e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.8579639210242114e+00,     3.1083207380022177e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.2915144397320930e+01,     8.0171720862839666e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7746493243911283e+00,     8.9143391937132892e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8644231678508525e+01,     1.7665593492787164e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2728891421953847e+01,     3.1087809779905005e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.8200552628124917e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8514054370094488e+00,     8.4308631335515183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7543005194778480e+01,     7.0392612294782657e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3304157899364064e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.5687884098197777e+00,     1.4425824530775280e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.5040394201101943e+00,     1.1015106480668383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1664351098084198e+01,     9.3500199793619813e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2221212435920032e+01,     1.3869707923689930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5839133536839698e+01,     1.5648692484404177e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9554888878087553e+00,     1.7179752239870060e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5991190231968604e+01,     1.6149731967049485e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3166954684694197e+01,     1.6657054216872126e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0633652773801305e+00,     6.9853352056049225e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5452276076179476e+00,    -3.3781427455160383e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7461669746232225e+01,     1.6657711481621671e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8901505962225169e+01,     1.5536195334382661e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6458132113750441e+01,     1.3852787242882338e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.0684214862472743e+01,     7.0946181596646074e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6683211195385741e+01,     1.1930314501674195e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5195380247446975e+01,     1.6068890166099269e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.1894719905883051e+01,     8.1538897887433714e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2272302846004809e+01,     1.0122944007549690e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0553565721092426e+01,     9.1716155474469833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0817765318362603e+01,     2.1606140179671438e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.8017778611628017e+01,     1.4430426930658101e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.0098062299566349e+01,     6.9787762500057888e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3146476729778991e+00,     1.6106627525213480e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0022753198827399e+01,     2.2692382805526282e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.4781250872656919e+00,     7.7003894694093897e+03,     1.5495628925523469e-04,     2.5010248204049204e-07,    -1.1815442703688610e-09, } , 
      {    1.3636280496885833e+01,     7.8210405424260362e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6976249725231170e+01,     2.4029180068945676e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0543788897673249e+01,     3.2197835939287281e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.0067123143003734e+01,     1.8058614462480149e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2757249614111359e+01,     8.5574190843768720e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7632246931709389e+01,     1.6171061875027337e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5307041941495713e+01,     1.1472859182167349e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0904548418331288e+01,     8.5034930605035279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8172325589669036e+00,     9.7430409490549682e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2694447278195909e+01,     8.3283323951146504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.1700223458812793e+01,     8.3290504541322498e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2702896205974767e+01,     7.5255822036978279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8427928305337936e+01,     2.3245439329501114e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7043050688313571e+01,     3.2202438339170101e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7511597186175194e+01,     8.5047628919983163e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.8716098128662018e+01,     1.5537823257724198e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2457349895073591e+01,     1.5529636378783527e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.5793251209234974e+01,     1.5547685322239387e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8423543863487390e+00,     8.9907504269615747e+03,     1.1881188131843500e-04,     2.1488755549941980e-07,    -1.1165695408265611e-09, } , 
      {    1.9722834865215450e+01,     1.7844108241319202e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5498940677154209e+00,     1.7260154635030553e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5953779096252809e+00,     1.7808059824915603e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.7080772556707785e+01,     8.3279867927073010e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.8075045331885150e+01,     8.3293960565395992e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2580704685225619e+01,    -1.1483856063891690e+03,    -3.3995134669750335e-04,    -4.0287231502039035e-07,     1.8900946171736367e-09, } , 
      {    3.7002147469349396e+00,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6642560386999854e+00,     8.1889718027459076e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.5333154087616240e+01,     2.2723060108216079e+04,    -2.5384211505901077e-04,    -9.0035718346214099e-08,     2.4609142453120021e-10, } , 
      {    3.5025089770287238e+01,     2.1045047865683620e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0994834174991276e+02,     8.3269604561462111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4909513634169841e+01,    -8.2665582747510034e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.2966530896230246e-01,     1.7245875532394530e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.9904281704400173e+00,    -1.1544434611630179e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4237613630190975e+01,     1.6012154583393763e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7219598767764566e+01,     4.3232306666228633e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.1647716817328231e-01,     8.3284677674657396e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.9684677680235296e+01,     8.2317886240619573e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0753843346136840e+00,    -1.1153399047319820e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0579455338276414e+01,    -1.1139172137982293e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.4058968943494268e+00,     1.4836894443924128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4362639257785386e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1809156497073534e+01,     8.3289150817811606e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0947655860623167e+00,    -8.4684110465009944e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.1489964672481005e+01,     1.8451635432173138e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.2404254761482942e+00,    -8.5551615711673439e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.0450881796855292e+01,     8.4039424856941623e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9350684369021511e+01,     2.4969153593062754e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.4428715733098159e+00,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7317527822536203e+00,    -8.3126919163994535e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1734049757972297e+01,     8.8273902472144641e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2789965197562575e+01,    -1.7085226553869916e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1167258856965162e+01,     3.0510170092129239e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.3061153044235816e+01,     8.1937046906412834e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6120742857587384e+01,     1.6051617169333600e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0910387649279420e+01,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2228738384296811e+01,     1.5560803233437655e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8344538131014112e+01,     6.4109536444327223e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2856218078283987e+01,     2.3240836929618294e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.1296480240886446e+01,     1.4427773414758614e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.5174101743185918e+01,     7.3888645012384241e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9745219307659596e+01,     1.4428478046674763e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.4728571536992332e+00,     6.6195179572683483e+03,    -2.4662834734423393e-04,    -2.2566525451869346e-07,     1.0562587188965348e-09, } , 
      {    3.4758289716440967e+00,     8.9569933798375096e+03,     1.4962852626941768e-04,     2.5133526626881740e-07,    -1.2904237268724407e-09, } , 
      {    2.9873887803120816e+01,     7.2321118088142066e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8166983410992536e+01,     1.7215800656273659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1059602238052285e+01,     1.5407767555999628e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.2070687883526013e+01,     8.4636781586056168e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0109655940353285e+01,     6.0370774990630825e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5802345733270714e+01,     1.6879551553401190e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3380152283325337e+01,     8.3288398416806631e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3858389556084809e+00,     8.3285430075662371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0709078843567154e+01,     1.3710800632056356e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2110184753523065e+01,     1.4911816872620067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.7656827099171064e+00,    -7.9694518823856242e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0387540451484483e+01,     1.7208138173280317e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.1516007027177366e+00,     1.5524705346525934e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5995174668901537e+01,     2.3542859094726118e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.8701515136874093e+01,     7.0790761313761777e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1221931459916217e+00,     1.4205957026562399e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0635253829764073e+00,     1.5208693046510067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4549989648500352e+00,     2.5019831320994414e+04,     4.2606057833599587e-04,     7.1570891169456661e-07,    -3.5340978098160731e-09, } , 
      {   -4.5367356838809867e+00,    -2.4952317226746283e+04,    -4.8769386823796122e-04,    -7.8860433323336171e-07,     3.8818061819078319e-09, } , 
      {    9.2313171579712687e+01,     1.4914439991385527e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0516228536145680e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1107675235473623e+01,     1.4307469027868559e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0580045905255862e+00,    -8.0470041433977476e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7826656337509650e+01,     2.2790574202464199e+04,    -3.1547540496097606e-04,    -1.6293113988500929e-07,     5.9379979662295946e-10, } , 
      {    7.3529900399185006e+00,     9.0371811029973032e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3103758842000156e+01,     2.2667799641725353e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.8223421299469269e+00,     8.3132723134837725e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6207057532319187e+00,     7.6202017462495987e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4799181869467802e+01,     1.3642083791330715e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2537213602894901e+01,     7.4493494807109000e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8611185930402918e+00,    -1.2903965975462468e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.5476822700883730e+01,     1.5662198995675531e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2912736289723590e+01,     1.8844656401866127e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.2352593348695891e+00,     1.4834816901140426e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.8360228036584598e+01,     8.9397260129033239e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3695342057170649e+01,     1.0515964977242678e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3301251187246912e+00,     9.0099681068801838e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8105496818866676e+01,     1.2323335471367182e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6532857167235875e+00,     8.3783548220585890e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.4003376887914616e+00,     9.3734302058697722e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4366015456622158e+01,     1.5073706312527900e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3080601973020059e+01,     7.3335075710520814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7241802127850123e+00,     6.8800016218866167e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5910353066178304e+01,     8.6103787058491525e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.3351522734209276e+00,     8.6879309668612768e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.2835141024642063e+01,     1.5717555925861872e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7985309650879785e+01,     2.3793893453593122e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1628308948815224e+01,    -1.0748136573671934e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2895570615997158e+01,     7.1118911564302762e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8525550810264597e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8642084779574891e+01,     4.7162516363158493e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4638579318227825e+00,    -1.1080696036659720e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0344182797057386e+00,     1.0447387812463210e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6729402801232006e+01,     1.1865880151860336e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1612175225765192e+01,     8.0338862321994802e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4260832193753469e+01,     7.6834631587703134e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4973301688415312e+01,     8.5662380093272204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2908483720469953e+01,     8.4292412282353416e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5341726886027940e+01,     1.6251243968355648e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3640027432735618e+01,     1.5214167670102664e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.6962527232116997e+01,     8.9795409148012368e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5194981151203084e+00,    -9.6190880221696971e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.6447281004487408e+01,     2.1438068835376605e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2230568323629535e+01,     8.9220541206596008e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.8847463579794947e+00,     1.3577649441516856e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6867524181750637e+00,     9.5426467787588335e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1071971866378540e+01,     7.2091318331007478e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1500749834815842e+00,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    6.8277978754219086e+00,    -9.5689420492467389e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1696457543324388e+00,     7.1147360704880666e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2219947908334756e+01,     8.8341401344390179e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.8039224057100594e+00,    -1.1921808202772299e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2077124022268730e+01,     8.0776570657564434e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4506703259632978e+01,     7.8400692504624767e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5950571117418377e+01,     8.9573513256270689e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3280246713417817e+01,     8.3544305597715611e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3806323888899676e+01,     9.4624477194845604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6824085727320426e+01,    -1.1169297592065159e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9508273577164182e+01,     3.0299466640577612e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0383283718126048e+01,     7.6136427906504650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1519087987197633e+01,     2.2457096190173725e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2367818563109411e+01,     1.0107675985337697e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6334828839924171e+01,     7.3490495993405111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2708069081464611e+01,     7.2784972151722031e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.9373060708541765e+01,     1.4035104761023700e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.4226417987126936e+01,     8.2335195925391963e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2820886152611967e+01,     8.3162399051461416e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.3882181823811974e+00,    -1.1326527674669046e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1077973477236876e+01,     2.2216870415428866e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.9681885560622590e+00,    -8.3411429441007604e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.9403540160155925e+00,     7.6605689376799946e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5902346765940919e+01,     1.4288440319832118e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.8531102596559975e+01,     1.0973555624994374e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6550075757493527e+01,     7.2075039097592116e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    6.6050444808266295e+00,     3.1546870868086353e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4335319042727694e+01,     1.9237677371559115e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0002824812917133e+01,     8.3996847300364407e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2961944486766740e+01,     1.6577753045451074e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.5553706548473869e+00,    -1.0808471318099444e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    9.6219714118963502e+00,     1.3500097180504734e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.3234323929352428e+01,     8.4238632567077038e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7027040943451421e+01,     2.4393815105228405e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1021603841231403e+01,     8.8974804325975801e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0459191577118442e+01,     9.1016154528111583e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1717852476306710e+01,     9.1318006455490722e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0385633834027992e+01,     2.2234447764716981e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.8476617210247994e-01,     1.5536720815539044e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.8676459581566718e+01,     1.6737012653042722e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4439119410532172e+01,     2.3322767933355524e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.0783197166304369e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.5149862185631065e+01,     1.4914453418486977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3931356506841489e+01,     6.1138690423794981e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.2240247821715275e+00,     7.0382948270772031e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.4226989883034751e+01,     8.3151849133520955e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2008419061313390e+01,     6.7450148879044500e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5221533826213644e+01,     8.3421979358948047e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6455227104790024e+00,     1.3188431241433196e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4417334022424086e+01,    -1.1146352728158322e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.6762518957924428e+01,    -1.1146218457143791e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6571309377888582e+01,     1.6587493071228117e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5602575377580990e+01,     9.3434610237628476e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4107059141704461e+01,     1.6137299198071792e+04,    -3.8030412665759474e-05,     9.9181825403081772e-08,    -6.3631310831945483e-10, } , 
      {    2.0457616050475057e+01,     2.2599082800999709e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.8949152833201275e+01,     2.2168324472192511e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.3437622114175603e+01,     7.2189938976159428e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.9957639207930725e+01,     6.5857485667620740e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3840517456345546e+01,     7.3200010597807268e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6521005999718859e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1019023006346899e+02,     1.5542246331342680e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.8311317755966300e+02,     1.5543262248620909e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3762505086088311e+01,     1.4405583825584445e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1380343677882987e+00,     1.3893118150197721e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.6122838075643465e+01,     8.5658498675336850e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0585267299215086e+00,     6.5082030193006776e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3995377557478044e+01,     1.5677741023963963e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.0484438565066313e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0175164795733821e+01,     8.4733078677656158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2219858675779420e+01,     1.5447582457897541e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7661819724119368e+01,     1.5548787764424547e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -7.3704726716673434e+00,     6.2356807030156233e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.4209454229757910e+00,    -5.3929007143075387e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0064612058653104e+01,     5.1092726060088371e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6593995706850325e+01,     3.1085156264005516e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.4464962618212454e+01,     3.1085860895921662e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.0141066152646545e+01,    -8.3277250222223847e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    6.1854866405316095e+01,     1.4779459970954085e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7467092004500554e+01,     2.3871445714605245e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.4595834674026371e+01,     2.3242785813601629e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0738323682874805e+01,     1.3241400338644387e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9859593087672081e+01,     7.1188910332740907e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4301996116701092e+00,     8.5719979374838113e+03,    -3.1112138346627602e-04,    -2.9785701748072627e-07,     1.3509818037798328e-09, } , 
      {    1.1314151651396669e+01,     8.8199281715854559e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.8835406688377034e+01,     1.4898904676647822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.7650253813937438e+00,     7.1081246709359621e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2468487620792075e+01,     2.3279186545963301e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.4515984169026819e+00,     7.2096536381880860e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3860563551898215e+01,     2.3948997975617367e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4811610768186327e+01,     1.6170350529560461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3676968362940194e+00,    -2.2123364377226180e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.9429447927676129e+01,     8.8999111110054127e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8517226497634063e+00,    -1.0370762982529814e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.8511462017342915e+00,     2.3791815910809419e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.4922979671146678e+01,     8.9566466937109180e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5663450911506764e+00,     7.7030136718940903e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6806873697178268e+00,    -7.2160758957211817e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5118301824620193e+01,     1.0908985946935667e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5388967182202990e+01,     8.2733344944371092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.6713941728511855e+01,     8.3044185510139669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0418608581155059e+01,     1.5436816095559412e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7708979326981080e+01,     8.3529642982329315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7003616936440785e+01,     1.5858222998662659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.8334273337248476e+01,     7.2711507640219252e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8934494846833672e+01,     1.2434972588928190e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.6791612896351332e+01,     7.8446716503452953e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.4396199142029758e+01,     8.3840483548097927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9484878515048564e+01,     7.5481241088300721e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3186942260459205e+01,     1.4285786803932633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9091235384713885e+00,     7.2302722006990607e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9527788400515941e+01,     1.2716356441060170e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4674529308445937e+01,     3.2236185555632292e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.7969065178897523e+00,     2.2850117159866713e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.9124224111493433e+00,     8.2757318031511695e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.1249969509138440e+00,     8.3003054912131920e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1099470977794251e+01,    -1.1484099867202651e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    4.4373063379946522e+00,     6.6207001693221937e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.7869481429002136e+01,     2.1831089805069594e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.8692723056750635e+00,     8.3570773580337082e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5757941219776299e+01,     1.9630698341252100e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2157424081602642e+01,     9.9326794870277098e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8437301001961995e+01,     9.4144543291758382e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.5077759623174316e+00,     7.1960139219024823e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.2271896363846029e+00,     8.7992646553285613e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3771982492369435e+01,     8.2058325493934753e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1498575666576887e+00,     7.7184080477797061e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.8221476017513059e+00,     1.6579830588234774e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2883741288256779e+01,     1.6204813292319919e+04,    -9.9663702567724807e-05,     2.6286403864286599e-08,    -2.8860473622769569e-10, } , 
      {    4.1391177267489631e+01,     8.3178747133665947e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.6625773923787790e+00,     7.1787257944216226e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4084134146277187e+01,     3.0692487610270600e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.0491622326723533e+01,    -1.6633063405148241e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0965288068013441e+00,    -2.2936914683440682e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.0355870396306610e+00,     8.1114384932116045e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.5663167488515430e+01,     7.1828844618432668e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3555528050938424e+01,     2.2809786252551334e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8151751452104893e+01,     1.2258901121553325e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3286050384595804e+00,     7.2199629194435738e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8544103759657631e+00,     5.5410164990664807e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2802519318699428e+00,     8.9017513614789365e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1127944239885071e+01,     8.9596288419851771e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9405232959035484e+01,     2.2703848058128948e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.7631977419831941e+01,     8.1694318170318002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1208422580828334e+01,     2.3871117082230470e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1036929480005384e+01,     1.5542395260472995e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3955755237812824e+00,     5.4066923712338394e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6521128954728219e+00,     1.3135462144222005e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3960681344000058e+01,     1.5543113319490594e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.4203070601874686e+00,     2.3871774346980023e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.4185213548493358e+00,     5.8564776908983958e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7128107785129991e+00,    -6.9889778018784739e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7526437306848033e+00,     7.1773031034878695e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.7494779036552778e+01,     7.3092346974425982e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2599022113221093e+01,     1.4350128843824263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9102013711871910e+00,     1.6734935110259023e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.5845122030467795e+00,     2.4459938397752874e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0502843103765684e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4850796152463275e+01,     8.3296578270245136e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9063411663220755e+01,     2.2206061831306721e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2998278181462160e+01,     6.5857609101442840e+03,    -2.1581170239325131e-04,    -1.8921754374929586e-07,     8.8240453285065505e-10, } , 
      {    1.8075561586848451e+01,     8.8636990051424182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9531757071168137e+01,     2.2754187323023958e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.8642144262801018e+01,     8.4879510322151000e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1487071260270849e+01,     5.5022935757018258e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7368744681183031e+01,     6.2795526908746854e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5464430873319168e+00,     7.4650972242253520e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3627518298537559e+01,     2.2759446987656323e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0102308061020722e+01,     7.2009449541600789e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    5.7168049887322141e+01,     1.6171414190985413e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1829007009212580e+01,     2.2717002253442228e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.1229435673840733e+00,     8.2331314507456591e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.7505394791948262e+01,     6.5857619938635271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5211414400023429e+01,    -8.3459644213891170e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -5.1001186678506180e+01,     8.5260349656357739e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5491004815109390e+01,     3.1624798651394347e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2890957514319913e+01,     8.3853584035380245e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6294857571159532e+01,     7.8596434471695557e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2934267216757082e+01,     8.3198312690829080e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.0786882892318044e+01,    -8.3375515801639958e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.3379806557603999e+01,     1.5487397359795452e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.5925933825532415e+00,     6.5061254765169770e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1288953993998476e+01,     8.2817920859700243e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    8.1017164862148604e+01,     7.1587059351720027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6680103453315489e+01,     2.3951075518401070e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.1764879895594582e+00,     1.5549313245580928e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0907634364247885e+01,     9.6654886880428458e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8214580285318149e+00,     8.3220800250713455e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.5154658003888350e+01,     1.5637926122066050e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8733103940429450e+00,     8.4242513985012411e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4310835559481042e+01,     2.3332155643174490e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.0922682266870787e+00,     1.7576803828114203e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    5.6454483181352629e+01,     2.3243490445517775e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.0024747284685169e+01,     7.8604193938597491e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8463729404455570e+01,     1.3277448755047984e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.8448387789862011e+01,     7.2200963398010972e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -1.8912732450488878e+00,     9.5454916928166240e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2687381354731180e+01,     9.2339723544771405e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2002889725517043e+01,     3.2166380092104631e+04,     2.6921828435180138e-04,     6.0000318159822853e-07,    -3.0538413773089666e-09, } , 
      {    2.7688711081463993e+01,     3.1051751532839527e+04,    -1.0154970729668459e-04,     1.6068315580844063e-07,    -9.8989257408945052e-10, } , 
      {    9.9770814469669666e-01,     1.5530135051563808e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.1381357180305677e+01,     2.2796632057238054e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2309856402361460e+01,     1.0058509657735831e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8166135676152212e+01,     2.3837664287150084e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    4.8776034385239761e+01,     7.2137038358495429e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1967943839408726e+00,     7.2316948916328138e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5236899527939251e+01,     1.5520212384849550e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.6411405148802398e+01,     7.2144218948671469e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8589666309882137e+01,     2.5050508623684207e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.4281941779251838e+01,     1.1371704643973499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7579754919001260e+00,     5.8772656019389506e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9318805017424644e+01,     1.5525481293216126e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.6155500214184876e+01,     1.5529247778710440e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3170559175521280e+01,     2.9934831604294886e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.4813158472807025e+01,     2.3858327803406981e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.5704259018553604e+00,     7.7315622430929852e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3684583817442393e+01,     8.4493481274715796e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8503993823276399e+01,     7.2005563540869898e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3343719498190609e+01,     1.6000209609488636e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.9933441374120527e+01,     2.3617014274929345e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4578988986753942e+01,     2.5836550563070181e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.8903935197111572e+00,     6.8127668225673460e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7886886307757894e+00,     8.3248886550141251e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2055422785262593e+00,     8.3324941942327750e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7180434039520474e+01,     2.0023719310945089e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3348101194519002e+01,     1.5126323093781017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9226041598273760e+00,     6.5831254479966174e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4496380456237389e+01,     3.2233894186352754e+04,     2.0758499444983606e-04,     5.2710776005943332e-07,    -2.7061330052172078e-09, } , 
      {    5.3860882614542739e+01,     7.5653971055957409e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5641723513609108e+01,    -8.6364983828341337e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.4230552242838307e+01,     8.3973835300950304e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.3688841956856066e+01,     9.5056767909187110e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7842209604195233e+01,     1.4286491435848784e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1732739592721192e+01,     1.4914775337311024e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3157028025494910e+01,     2.5443529593377192e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.5326325288164547e+01,     8.7977394020773445e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6495018725685373e+01,     1.4620200363362435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0870988690061964e+01,     7.2275693766296990e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5836473982269496e+01,     1.5718546708594033e+04,    -4.6796367745047056e-04,    -4.1356274757706424e-07,     1.8312382362869388e-09, } , 
      {    4.8633484195009693e+00,     8.3390007058789379e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3234698744190688e+01,     1.5556260801253149e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1855998286135367e+00,    -7.5764309126926364e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.1292705760112618e+00,     8.3183821433679623e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0183964997918498e+00,     1.4878398288532653e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.9278830904108535e+00,     1.0001737790915315e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7993472828778525e+01,    -8.3373279230062853e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.1295506413115159e+01,     8.2802400243427037e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.5051934981125164e+01,     1.4445398727482358e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.8653463358836518e+01,     7.2694197955446853e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7512558051482383e+00,     5.9787776032451093e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.7899517836588110e+00,     1.6648746350864069e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1562201494376829e+00,     7.7687473126747136e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8544378460001729e+00,    -2.3182746321450009e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.2767792906456510e+00,     1.2168002681931741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9690459815789218e+01,     2.3209386712267122e+04,     1.1958975808238373e-04,     3.4866791532941113e-07,    -1.7634176504365261e-09, } , 
      {    8.9782271623599836e+00,    -1.6429330993956535e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.1454806300421421e+00,     6.3635865761585128e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3523515203941308e+01,     7.7740069773070809e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6720733997736986e+01,     1.4819274872851996e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.1182215715999675e+00,     1.5558486773981091e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    6.6878181032871815e+00,     6.5850439348459240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8599945591771124e+01,     7.4673984241667622e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0182208095369646e+01,     3.1119265627087651e+04,    -1.6318299719864993e-04,     8.7787734269645441e-08,    -6.4218420199769137e-10, } , 
      {   -3.6943517907128527e+00,    -8.4707267961116249e+03,     2.1867144861332805e-04,     1.8851388517253349e-07,    -8.2941924564219393e-10, } , 
      {    8.6507632685094421e+01,     6.5702132520243713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3430208402795961e+01,     2.3264468037537554e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.4353106677620318e+00,     1.4286139119890708e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6737684089280215e+01,     8.3172014367667234e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -6.2837424956289667e+01,     2.3871452428155972e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.4625604849785390e+01,     2.3871439001054518e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.3300128008839609e+00,     2.3835397298201646e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.4590568702366092e+01,    -8.3401814124801767e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.6541160987911894e+01,     1.1302006916628650e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2461468053126229e+01,     7.8416591049370109e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6079593994517069e+01,     1.5555373528399781e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    5.9507177672034750e+01,     6.4507685463306352e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0137842281102944e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0412606280944425e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6195188449596351e+01,     1.5527021805982495e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6728100084708863e+01,     1.4818886731058461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4071597947487298e+01,     8.0310922092562214e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0354806192841554e+02,     8.3287894761972893e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.2553772086368610e+01,     8.3285933730496108e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6833840024188728e+01,     1.7321738850696041e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2909277326340494e+01,     5.8953145453948109e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9291523080299562e+01,     2.2224110774762583e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.0824910890467461e+00,     7.2080293909155935e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8015027187450272e+01,     2.3872157060072121e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3310017489416692e+00,    -9.0617981228682402e+03,     6.4104097740067743e-05,     5.3015830470371089e-09,     2.8700969921684549e-11, } , 
      {   -5.6312026526366047e+01,     1.5598111220168137e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4812158081562252e+00,     8.3570249140807373e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.3788776840136485e+00,     7.9122602284226723e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4020160867423792e+00,     1.3342362826164301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8857448754503551e+01,     1.5558173401121474e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3167270743777429e+01,     1.5514368356571536e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0811834109580012e+01,     7.1856769319480863e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5425778187602965e+01,     8.2196589360975777e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5061361797653454e+01,     8.0753558658150323e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3622019645242087e+01,     2.3905227142060408e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.3766266925658773e+01,     2.3870734369138368e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2266619817741329e+01,     1.4286850465357584e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7025197998279076e+01,     9.7364819934558345e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6001714264968527e+01,     2.6229571532763170e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5013992949249189e+01,    -1.1942583630609302e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0750941808367291e+01,     8.1366167919777035e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5566458545772246e+01,     8.0802868980726271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3774473814077195e+01,     1.4968127147614319e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7803201530002877e+00,     1.5527335178842119e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.6524530946016824e+01,     8.5378885744016752e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.4608199055619977e+01,     8.5385932063178279e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1900106115357907e+01,     9.4946364225041470e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0783467421320876e+02,     7.2135549067192296e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.8546873081017637e+02,     7.2145708239974610e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0949992857286460e+01,     1.3109377410753154e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9371342076235432e+01,     7.6831108428122388e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.8619927838000450e+01,     1.5518481416372311e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3123283464333777e+00,    -2.5289780836966238e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.7090287244010355e+01,     8.4019859299778509e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.4079119246663701e+01,     7.9773571843860536e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9606718381954407e+00,     1.5524730081779997e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.5168574902343973e+01,     1.4283837919949297e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.2289352951325856e+01,     3.2199784823270620e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.8788717922352404e+01,     3.2200489455186769e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5335828384198290e+01,     8.1835763087010055e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.1439202762827108e+01,     6.0768924009609946e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.0962784569758753e+01,     7.1897899917488630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2183952466371746e+01,     2.3276900806515248e+04,     5.7956468180418396e-05,     2.7577249379061598e-07,    -1.4157092783447669e-09, } , 
      {    1.5829341669224156e+01,     1.4392788659779966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0667380879428549e+01,    -1.8995165128357589e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.2078982220309229e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5265211608645060e+01,    -9.4410187839471473e+03,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    8.1815202206989248e+00,     8.3713512405791535e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6007115905556347e+01,    -1.3690599989410089e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1254110766668749e+01,     7.8111920468888093e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5695558007396501e+01,    -1.6651349374804147e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.1976236606630824e+01,     1.6663416323689653e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.4209040790865247e+01,     7.0045133996220429e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.2600409562620092e+01,     8.5846415562329021e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.0769915386008265e+01,     2.4986426589828421e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -3.0353417936737078e+01,     2.4985721957912276e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.9574024782681136e+01,     1.2651922091246313e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9309677969955580e+01,     1.7791139144108009e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3939429216566072e+01,     1.6170709559069264e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1553830283014896e+01,     7.2424487987686025e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8174197584490992e+00,     8.2860316086677467e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0996777366183506e+01,     6.7976316691575666e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7944511795449607e+00,     1.8073704941245055e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0987286636016250e+01,     8.2419503756960075e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.6055436354365522e+00,     8.7451226208242279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.6447766482769808e+00,     1.4391365968846214e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3823994941210515e+01,     1.6128402059071634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8913408018680027e+01,     1.6173691707343522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2823449726679295e+00,     1.6657382849246900e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.7009102696368839e+00,     1.5183514747743968e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3909547394526871e+01,     1.5571140223392053e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2730430705120154e+01,     1.4893116796958400e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2443311216749986e+01,     9.2905014022905671e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.3322100922572480e+00,     1.2139553541353828e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3283068054270242e+00,     7.8243462422020884e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0333194707215718e+02,     7.2383357389678267e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7903514863564318e+00,     1.6425493314703243e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.8969284949921370e+00,     5.5775446367725372e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0570315086873428e+02,     1.5567027163591278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9072823710476774e+00,     7.8581181939183380e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6374827754525047e+01,     1.6108705067997182e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.8896043446186383e+01,     1.4914118072561479e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1475871373887038e+01,     7.7997106344481845e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4532559949114692e+00,     7.2102600957490185e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0021055677521321e+01,     1.6496361397840737e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0409328687756449e+01,     1.6031376464142766e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.3298287189108091e+00,     7.1442330214995854e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.5520872233751504e+01,     1.5113521214425813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.4538170190852497e-01,     7.6977540072617230e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6502605340349348e+01,     7.2294819764980220e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.9019431161337916e+00,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9466036881702077e+01,     8.3290846708858917e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.4717611852147527e+00,     8.3282981783610085e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3736700374020034e+01,     1.0451530627428820e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8800763946242405e+01,     2.1553171082460249e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.7283931956396978e+00,     7.8051341921149615e+03,    -6.3908233012899474e-05,    -5.3888495096368257e-09,    -3.0155410965013152e-11, } , 
      {    1.4912419652116178e+01,     7.2178656349676703e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3848717936627423e+01,     9.1941574525792294e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9926475705994001e+01,     9.8074752988688233e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8602961519283710e+01,     2.0416740280638074e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2989443831015182e+01,     2.4356780791700485e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1056441630727068e+01,     1.6892669464599454e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1604625676693452e+01,     2.0426571094770613e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3824597667495038e+01,     3.1414095199842719e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.2977593254044130e+01,     1.4394368683592627e+04,    -4.0613452282133695e-04,    -3.4075459250086880e-07,     1.4820754231518511e-09, } , 
      {    9.8127078423686829e+00,     1.5538951520372470e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4467214416531167e+01,     2.2145430251102189e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0188937989184790e+01,     7.6408652625064933e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.7203694880933735e+01,     1.5447194316104004e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1376529236885601e+01,     2.1228392028053695e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3153103394415066e+01,     2.2756817155340141e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0758399690595956e+01,     9.0809519365542656e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3147589142276814e-01,     1.6688561252761978e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.1680663133452356e+00,     7.2412928274214255e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7264155158221392e+01,     1.5546557059591121e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1309040280020962e+01,     7.2130964629572827e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.7847423536629080e+01,     1.4859089774749909e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1255621346812173e+01,     2.3864886759006109e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2245859441158828e+01,     7.8423648205724048e+03,    -2.2113946537906831e-04,    -1.8798475952097055e-07,     7.7352507634707552e-10, } , 
      {    1.4376410635493425e+01,     2.8286990488630363e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9475834941632726e+01,     1.5383494682390145e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3417814931945049e+00,     3.1178403515078116e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6424985348792212e+01,     1.4897526024128658e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8102255741519830e+01,     1.4270391376376258e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.4332027924472879e+01,     6.2883355150877996e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4383438558738233e+01,     7.6872659283796393e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5078181437512708e+00,    -1.7216062363327969e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0918418061914810e+01,     1.4966704456680567e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9269051904049164e+00,     7.8243367664631787e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4853258722211031e+01,     7.1185028914805544e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7424323053559661e+01,     2.6622592502456158e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.3167035750959588e+00,     5.1845236670795503e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.1718640761101291e+00,    -8.2642570748095914e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4441002332797979e+01,     1.4904137423680764e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4307622387697112e+00,     1.5069551226960499e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9820464103188291e+00,    -3.4944889009392369e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.0897524106827341e+01,     1.6026774064259946e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.4956145406047344e+01,     1.6097767369672936e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    3.9486157842057160e+01,     1.5467503228911082e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0356967607989375e+01,     1.6365182285858073e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8192028183590448e+01,     2.4047238133217485e+04,    -3.1567126968814438e-04,    -1.6284387342240958e-07,     5.9525423766628801e-10, } , 
      {    1.2050179236551116e+01,     8.3659220530808943e+03,    -4.9388246038426593e-06,     6.8122964143321001e-08,    -4.3230351130856216e-10, } , 
      {   -9.0745086230041849e+00,     8.8613978052010079e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8036235693807200e+01,     1.3905756340093529e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.5196984746349891e+00,     1.5559674970789387e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0473285568439909e+01,     8.3361695911925071e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2217256482286967e-01,     8.3212132580543930e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8346960203077778e+00,     8.3467251085641601e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.4852567700373243e+00,     8.3989733845695646e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2800292663817975e+01,     2.5158601724983564e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4133595483443294e+01,     2.2130810770236010e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0547972827673391e+01,     1.6493089539307337e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5364253342330919e+01,     1.0090755304530103e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2197952949570976e+01,     1.4031707680214806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9417382238858067e+00,     2.2092461153890996e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.3415145121908303e+00,     5.3821092074329063e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6347276784419961e+01,     1.6102369769555757e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8397472935988933e+01,     3.1807116169535708e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.8447956565954112e+01,     2.4422201038638665e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2546137675783603e+01,     1.4978436428869807e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6332627167643267e+01,     1.5475240195733668e+04,    -4.5498862218682970e-06,     1.3501314405831670e-07,    -7.5572756611354469e-10, } , 
      {   -6.0846378821938075e+01,     8.1919737221640462e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1909087274960992e+01,     1.6402951409570996e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0635859213611628e+01,     1.7206060630496620e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.7842181194640794e+01,     1.7283389234351031e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.4292267595333676e+00,    -1.6674303530054498e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.5796473977483661e+01,     3.2217200185560241e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9562556897556213e+01,     9.2567199748354051e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6789077068025364e+01,     1.6564082844720324e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.3744323075142368e+00,     1.6385678412805326e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7023582723198530e+01,     1.4375156633505498e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.7237475962349347e+00,     1.2398586532437894e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.5804164019158080e+01,     8.3284857093974297e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7141856678141210e+00,    -1.9187229330914042e+03,     3.3676391171059432e-05,     3.5744052192635206e-08,    -1.2086889883741836e-10, } , 
      {   -2.2321597982075691e+01,    -9.4456211838299678e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.5896713352908058e+00,     1.2566151700910864e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6588907821158166e+01,     1.3839633047569057e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.3032589711204601e+00,     1.2005565562744909e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0606931925910487e+01,     2.2617131744455568e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.3812551477907355e-01,     7.8554883616021552e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2806651328249529e+01,     7.5251061070712240e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.1830956392311407e+01,     7.0548032577666963e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2867052450559434e+01,     7.9379716548718716e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6841237431942012e+01,     8.9968139115669055e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8730875689061627e+01,     7.6052120074936529e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1420302199751767e+01,     6.6394357229908692e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5506222654189870e+00,     9.1922856250215500e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7963870331296928e+01,     1.1695027886321639e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0018854224599920e+00,    -2.1102199643873823e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.1774410297518703e+00,     1.7755090727704410e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.3164049993762056e+01,     6.7848297898023629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7203141381837540e+01,     2.1342467630908621e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1310986027095080e+01,     1.4481094827927880e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.8901920556663221e+00,    -8.2047384977381789e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.0936772170565206e+01,     7.6848418112894769e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1534563942562798e+02,     8.3283458222161025e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2633993383358118e+02,     8.3290370270307976e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4786152018612171e+01,    -1.7432884602687236e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.2830521270679931e+00,     1.4790493778050981e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.2084670778031352e+00,     1.6911814288922804e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7394792147335632e+01,     3.2183216458421099e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.4376324881526301e+01,     6.4905834482285472e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8611444019717167e+00,    -1.0018658471722905e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.1524154264215269e+01,     9.2737872563750516e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3503703184050082e+01,     1.4916524247719952e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0468812144683870e+01,     8.4526443515087212e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3213689432581404e+01,     8.3281391350909671e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2219439142706749e+01,     8.3292437141559349e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.7525179768649153e+00,     7.1972021187107775e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.9695908935273705e+00,    -5.9225916864194517e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.4013959376986591e+00,     7.5226051193081457e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4366300831596472e+01,     6.4901953064350109e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1751803576233552e+01,     1.5876815533453522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9821624766264271e+01,    -8.3395081358803091e+03,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -4.2879135324163677e+01,     9.0919857436511593e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7217468222673048e+01,     3.6147409882490110e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0243361364927136e+01,     6.0214374192008127e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1823683849691259e+01,     1.6833150887528041e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1589979959392116e+01,     8.8018944876447458e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1118362590175490e+00,     1.3700491350800869e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1914138766851696e+01,     8.7838455441888837e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5755681943122468e+01,     6.6813564847807866e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8537952406806042e-01,     1.5168878144612154e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3510042094421060e+00,    -8.8330460827837233e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.1324145075838459e+00,    -9.5033524932554042e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2355235812859187e+00,    -8.6359420039809891e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8180267577680738e-01,     1.1612544593051920e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4224098546984983e+01,     2.9088811421913433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.4936412968268666e+01,     9.4658618890207999e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.7755876125914689e+00,     1.6719739656277055e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9794461740415041e+00,     3.5923954223782710e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.2178201134326358e+01,     8.4862200637378610e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7198279396177174e+00,     8.5574285601157826e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -7.1703171588331003e+00,     2.2126208370353186e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.2974917879371723e+00,     1.5604887439854243e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6653203950160961e+01,     1.6746400362861688e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4049167403945003e+01,    -8.3061495194912040e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.6761062002308954e+01,     7.2320870735601447e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.7251906584472350e+00,     1.5523566620225765e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8630819375613587e+01,     8.3309584135718887e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.4295485193579374e+01,     2.3307572479373561e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.4700873765769504e+00,    -7.6539831737047607e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7169491252671460e+01,     1.4425495898400506e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.0424005967220115e+01,     8.3297873054909360e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0025454641971269e+01,     2.0809761250331063e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2638924912075129e+01,     2.2914551509680572e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4477993102517964e+01,     8.5021582629631812e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6592176781798383e+01,     7.8450002827200724e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8424297275580850e+00,     8.3282955905834388e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.7291328037427398e-01,    -8.3290872586634614e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4093410781872276e+01,     1.4550984605946665e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8903773364412380e+01,     7.2135105758258615e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5381428194886091e+00,     7.2146151548908274e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1279038235513376e+01,     1.4430755563032873e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.5423308794803155e+01,     1.8136166723492275e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8349849708293650e+01,    -8.1890060137388809e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.8123176427858869e+00,     7.6666080419542295e+03,     1.8587086656980149e-04,     2.8650655957858978e-07,    -1.3561256769364048e-09, } , 
      {   -5.7380535043970564e+00,     4.7915026973865633e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0786130934322209e+00,     8.4875987162570236e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6910813015419413e+00,     6.5677310721110025e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2372383701956892e+01,     1.3502398380446142e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3283715987125477e+00,     7.6996724941110297e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0400402509336200e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1713667766327303e+01,     1.4233170022679516e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5674641741295225e+00,     1.6678712757224752e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3802836561950016e+01,     8.3750491222825367e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2651626672015865e+01,    -1.0349987554692811e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0796630648978152e+01,     1.4428125730716689e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.4447149571595403e+00,    -1.6834175672392344e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6009067990508044e+01,     8.6464613332457266e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0150492153037352e+01,     1.4950495121339849e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5133998003588053e+01,    -9.3300004526574980e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1547039058843460e+01,     2.5128109645358527e+04,     8.5913366911324296e-05,     3.1292386313677598e-07,    -1.6425487515991077e-09, } , 
      {    6.3600404483103814e+00,    -8.3106672164216525e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.3498659263523516e+00,     8.2522501812125865e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -3.7143835833677755e+01,     9.5282186960509571e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1021256891392669e+01,    -1.7350104933270350e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.1993089467324719e+01,    -8.3275955437559660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8846870026584320e+01,     2.7015613472149147e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8330669967284390e+01,     2.3889469922807042e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.5160129920887284e+01,     1.0844551597121805e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1362220566188785e+00,     1.6636052941269048e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0996336887183933e+01,     1.3044943060939302e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.0577008256967588e+01,     8.8600962091075016e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6669663667910566e+00,     1.6422096233894346e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.2374932710030890e+01,     1.5480621140109348e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7234926401684071e+01,     1.6011802267435691e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1278783205793854e+01,     7.1714030494026410e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0848727763146236e+02,     1.4914544756510091e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.0289649064542736e+00,     8.3275279630692185e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.5029311451062508e+00,     7.4313157967701900e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3953524145371784e+01,     1.7743145753799286e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2265459245851780e+00,     1.5542530632824084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9850506579220824e+01,     1.5542977947139501e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7803478393538201e+00,    -8.4051326680343154e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1884792510859343e+01,     1.6622437960237508e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.3388593824038168e+01,    -1.3245303409594821e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0867011397439477e+01,     1.8419446729153555e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6311538288689285e+00,     7.9018890274753030e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.6571763662875467e+00,     6.1282999608059563e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3954085716476811e+01,    -1.3238257090433322e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.6882095057996615e+01,     1.8529187693185264e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.3450023303413801e+00,     6.8548233231205186e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.1258651705682475e+01,     1.5542202000449310e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0916524469727369e+01,     9.5853065947145369e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6632384084339257e+01,     3.2264571489042548e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3712418567019270e+01,     2.2679264894328018e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.1818827308653788e+00,     1.5543306579514279e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4654699334528569e+00,     5.9705656064655250e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7185998503217457e+00,    -9.4095629367644906e+03,    -5.5387704436179476e-04,    -7.2648661071384027e-07,     3.4737869878860464e-09, } , 
      {    1.0744424288965186e+01,     1.5610268384229923e+04,    -1.2781646602579895e-04,    -1.0777699019273651e-08,    -6.0310821930026303e-11, } , 
      {    6.7236195675588482e+00,     5.9336269603615085e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1961330095035390e+01,     1.5003464218551040e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.5308406103554759e+00,     5.2489580168934071e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3162056614075167e+01,     1.5558753798205793e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.8889319953992043e+00,     1.5638314263859585e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1261887328704532e+01,     2.1659109276882631e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5245745606308184e+01,     2.3402045421193277e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4136334632743736e+01,     6.2915089387389853e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5628737485321818e+00,     6.6380130320571161e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8447952839157217e+01,     1.0129502963148821e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0736552847709024e+00,     6.7408598023370496e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.4847388410057558e+00,     1.4388310828818776e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0946090507248897e+01,     1.3697646436743078e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8220678556467487e+00,     7.4061374980040910e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3338872196339299e+01,     9.8876573921971358e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5194823274972888e+00,     7.3096228392361345e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1247373217894694e+01,     1.5383024513167895e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8641528992759540e+01,     4.0077619579419988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9684192041520543e+00,     1.6866932314983202e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.1401026881488754e+01,     1.0200496268561812e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4100433621686967e+00,     1.3107076210811745e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1397248798367183e+00,     1.2791607502130882e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.4021650215500472e-01,     1.1219523623358931e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6449568705788113e+00,     7.2298106088727945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0811577650331262e+01,     7.6846360960634556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3914897394448671e+01,     1.5526754781757798e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1473776508187228e+01,     7.2297953493576424e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.0356015561836973e+01,     2.4593053304177360e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.2737156546320278e+01,     3.0563139189340429e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.8903130713022303e+01,     2.2128157254336526e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3461350665617061e+01,     1.7287991634233851e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2255400827425820e+01,     9.0349958965614187e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1817688279690058e+01,     9.4302020726902883e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5471110688302034e+01,     1.4461882777840750e+04,    -4.6776781272330229e-04,    -4.1365001403966401e-07,     1.8297837952436104e-09, } , 
      {    3.1309700633673067e+01,     3.0497015896815959e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.7299506583369713e+00,     5.5644267255742707e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4029294832778929e+01,     2.3190169032348509e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5698474401413783e+01,     2.4328901034112088e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.2050856310842000e+00,     1.2714055241118758e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3275158431077365e+01,     2.2654645446412069e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2358565501836871e+01,     8.3258465105656578e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0190824863787098e+01,     6.6497450042463561e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7317638288016571e+01,     7.0743774544737735e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3645186305328574e+00,     8.3315363386812405e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8812290900627939e+00,    -2.1359571140036364e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.5538406187037936e+01,     5.9570953793091840e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.9379788328972651e+00,     2.3893987619737491e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.7488629277955097e+00,    -8.2849205910664878e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.8279772832958102e+01,     8.2972111971097002e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8414455182041110e+00,     2.4125877154281145e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.6149685527844348e+01,     9.4260469871228870e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0528195289983312e+01,     7.8761218676964672e+03,    -2.5195611033005099e-04,    -2.2443247029036815e-07,     9.4737926239295504e-10, } , 
      {    1.4382174316718952e+01,     1.5366961871369556e+04,     3.3559732520280330e-04,     5.3779819261610727e-07,    -2.6472766243305099e-09, } , 
      {    1.3376739483085768e+01,     8.1551903752907474e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.5647441021509763e+01,     3.3019021118843320e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6706204857400614e+01,     1.3877370406683269e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3555559205459817e+01,     2.8168728122570292e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2812046869224602e+01,     3.0927774225623158e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -4.1359311350487982e+00,     7.1986437542186668e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.5507821103981257e+01,     6.5303983501264593e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0601155467868210e+01,     8.3724622581804142e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.4793993501142111e+01,     7.8427227663619624e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0622815928771590e+00,     1.4850012355122395e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2090555365584340e+01,     6.5754459990573132e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9713599795907388e+00,     8.2407774384028671e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.7444264845043023e+00,     7.7540642822559748e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1620323365303545e+01,     7.9978036048029080e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4974192078932356e+01,     7.2140733875621536e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3916997480042070e+00,     7.2140523431545353e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8992238084641926e+01,     1.4467940632614602e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0169719529087228e+01,     7.1507844868916654e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1938532767802137e+00,    -1.0229361923274528e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0186597716371683e+01,     8.9577103551358687e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8243674750703427e+00,     1.1452814816238019e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1761318184120313e+01,     8.8690850234484096e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8202678935620739e+01,     1.4071522582112717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8332761086239344e+01,     1.8922208662878253e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9642791433676550e+00,     1.4913735359469376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1307717483003945e+01,     8.5442743648025044e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.8593648714956412e+00,     1.4397472808357994e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0658812679476942e+01,    -1.7422247988437725e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.4593723254378768e+00,     2.4410735786035999e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.2723833064947614e+01,     1.4912369162152550e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5095981676122179e+00,     7.7643735635114617e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6218805748233493e+01,    -1.7425838283525741e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.2409684691013616e+01,     1.5618005351052507e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2734963072893434e+01,     9.2129491412784446e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0225127271025617e+01,    -9.4770770310126172e+03,    -4.9224375445982953e-04,    -6.5359118917504507e-07,     3.1260786157942875e-09, } , 
      {    4.9599522754473879e+00,    -1.7947803827124244e+04,    -2.7357230584650142e-04,    -4.6507730400251161e-07,     2.2966593701520934e-09, } , 
      {    1.1974274646080758e+01,     1.8229466728018848e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2669986847568573e+01,     1.9313234210097771e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1429681496536748e+01,     2.3149838125033129e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1068798230025489e+01,     9.6497409445283938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0186993808498830e+00,    -2.8936131199793522e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.1005510681547584e+01,     8.4111747178931673e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5571287116962582e+01,     8.3297949520176571e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.2351603958187024e+01,     1.5561941959737826e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.3865636958677953e+01,     9.6835223719835540e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.0295294992249566e+00,     7.3369217405883201e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0694972404083407e+01,    -8.3242821974531889e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0354200371028691e+01,     6.5644253723349493e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9691766691931573e+01,     1.7430306877434607e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9622701253369037e+00,     1.0826502653665946e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2609502464584036e+01,     1.3922677020901123e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.7006969061080568e+00,     7.1360659784659183e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7032174179598321e+01,     8.4460853959095875e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.6516350194299783e+00,     9.2544187748939949e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3940250292296490e+00,     1.6050405172179209e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1443838834253359e+01,     7.5449506851788865e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1855386339105092e+00,     7.5360905296383407e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3666980947061758e+00,     2.3807011364791386e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.0066672120856552e+01,     4.4007829276349858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4809677256919809e+00,     1.0630045375960857e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0245193586510331e+00,    -2.0764385369322217e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5482879579767740e+01,     7.6207540357820817e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1054071640468344e+00,     1.5534117791598961e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.2697497508729391e+01,     1.7268417437526776e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1112277824051699e+01,     7.2567226813140478e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1317712488111928e+00,     6.6790552848393763e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4573561883529980e-01,     1.2321034271425769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0060271749708720e+01,     2.5379095243563337e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.7905807579911929e+01,     7.2893139264290567e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8745634355465493e+00,     7.3537824872358879e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.6636269733867621e+00,     3.5337070936721659e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.1977914182654530e+00,     8.3356950061028856e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2191276432711877e+01,     7.2773412438250234e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3475048077847077e+01,     6.0640972351565169e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5488114301407128e+01,     2.1735488600601610e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0867658571596740e+01,     8.9232119523823494e+03,     1.8054310358398448e-04,     2.8773934380691514e-07,    -1.4650051334399845e-09, } , 
      {   -1.2103086081431782e+01,     1.6655305306463200e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7808500932445327e+01,     1.6659460392030600e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.6720512197825698e+01,    -9.6505682496834488e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.6926399179351755e+01,     1.4252333312104452e+04,    -3.5170666445682708e-05,     9.8478166826319389e-08,    -5.8332782111099372e-10, } , 
      {    1.2518998603358225e+01,     6.1518967384727603e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.3626186003301877e+00,     5.1714057558812829e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6162378629421838e+01,     1.6600294950583320e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1356053051066649e+01,     2.3243138129559702e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.2096164941739023e+00,     7.5088680577823143e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.6962668728727778e+01,     2.3854172717839578e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.9386662346423805e+01,     1.2088048856014626e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5489833735116523e+01,     1.6013327520686908e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9874590422731728e+01,     7.6450269093915649e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1448152952220504e+01,     2.1202782220024052e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.1600684013879992e+00,     4.3984817276935755e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2895548168451878e+00,    -1.7171989946943947e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.2135543918189654e+01,     2.3010152714148557e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1327325296023091e+01,    -1.5840288526770746e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.8942642828935689e+01,     8.3278793789835545e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    6.1089941074572280e+01,    -8.3295034702633493e+03,     6.6345003052078436e-06,    -2.0362436405988305e-07,     1.1202589729398046e-09, } , 
      {   -4.0428159076407333e+00,     2.3869368171821541e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.8302747206282859e+01,     9.4884037941530441e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2530402931456770e+01,     1.5479475911515116e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.3209421786545903e+01,     7.3109656659198372e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8255418945800429e+01,     1.7037285907741621e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7335981063563018e+01,     9.0033996755421413e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.8307256405233101e+00,    -1.6644763610828915e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    3.5831206387424494e+01,     2.3873523257388948e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.8940599390735144e+00,     8.9312618600527403e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.2682556073335363e+01,     6.1167073028589075e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0678218800124778e+01,     7.2037535841028566e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2049919073047040e+00,     8.1981795421390452e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.7475208633263421e+01,     7.8994583490674686e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0269343662664888e+01,     2.7408634441842129e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2897287882971877e+00,     6.0626745442227639e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9498465895067305e+00,     8.1697841329898756e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2572802425638743e+01,     3.6759761870474335e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.6881223662555307e+00,     6.8401867199887047e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8952516102422702e+01,     1.5541787887580735e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -8.7610798566098651e+01,     1.4914348653362411e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7907745190646612e+01,     7.2112179513005531e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8770823063399034e+00,     7.7526415913222218e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6648752937579829e+01,     1.5702013897573444e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0895804695366662e+01,    -7.2609622040117729e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0394665935842099e+01,     1.6670002087664889e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.8349728195203337e+01,     3.1871550519349563e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -9.8007578807458344e+00,     2.4497452099709379e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -6.0769518013965538e+00,    -2.3188953454365924e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.4570517845931379e+00,     7.2169077794161358e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1671783387473301e+01,     7.2243721466138322e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.7753977037930554e+01,     9.0521708417532482e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.9153363232890115e+00,     5.9581590407341346e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4552314607311173e+01,     2.3243849475026578e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.3847155605965717e+00,     1.0433481683972957e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5853708035352412e+01,     1.4880670907312575e+04,    -3.2604717316358325e-05,     9.7905407943456567e-08,    -5.2816087233753963e-10, } , 
      {    1.3883079404074596e+00,     7.6228315785657833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1238957291885789e+00,     1.5261067008756092e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1130931484820096e+01,     1.7823327847127595e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1096461914502548e+01,     2.3656829176827254e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4092570627734849e+01,     2.3243443907027640e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0538254070535917e+01,     1.7836445758325859e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.6424569315932231e+00,     2.2299361835833297e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2359191878381520e+01,     9.9783275644075238e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7615847921749026e-01,     2.9424634157381547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4768449424188887e+01,     1.6393230579181625e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.0866205643804083e+01,    -9.6551706495662693e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -8.9223480458061829e+01,    -1.1147266108389445e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3790471965331928e+01,     1.5553063571237282e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0687833459167426e+02,    -1.1145305076912668e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7071179259951638e+01,     3.6949230815773199e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6692985696437162e+01,     2.3892775622583096e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.1205045521650625e+00,     1.5496353624108648e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0947165832530764e+01,     8.9750232178707920e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9778370932343037e+01,     1.9315229632571238e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0287560875492208e+01,     1.5539909375924004e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8278858259864165e+01,     1.6974171095952959e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3700976048907272e+00,     8.9543691773528080e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.7440391029472011e+01,     1.4929988733224682e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7880284235829711e+00,     1.5545599204039585e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0670715931061586e+01,     2.2988706689883795e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3244000484249710e+01,     1.5532445008726307e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8785486983691317e+00,     1.3812936056869414e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.2291784453900050e+00,     9.6474397445869836e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6583406588832485e+01,     1.1237572566814793e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8945185163157658e+00,     8.4617996469488880e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6946484775549258e+00,     1.5636054294508369e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9520706792427035e+01,     2.2834369416352263e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.0142034269433523e+01,     2.2208139374090420e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.2813030819648619e+00,     4.8597398168097461e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2951214442758021e+01,     2.5561412761704698e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0613143850228971e+02,     6.5858533318866412e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1710893734695150e+01,     8.0868458536717608e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2832951669538410e+01,     6.5632133751805577e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.8068215897792292e+00,     1.0040460714279969e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8761223691385727e+01,     2.3577199373031428e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.5260624118153334e-01,     1.1928013301732783e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.6491541188012498e+01,     8.7579245001794316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.0026696534019692e+01,     8.9725410379574223e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2180109839594108e+01,     9.4739729062472525e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.7455712244074739e+00,    -8.6572780445025783e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.4347954467285717e+00,     7.6900745583224189e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7800112150596705e+00,    -8.7554938217715990e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.4402761535079442e+01,     1.6652451816989302e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.1487782492800179e+01,     4.7938038973279736e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7378028858863846e+01,     1.4252393332429610e+04,    -3.0038768187033942e-05,     9.7332649060593772e-08,    -4.7299392356408544e-10, } , 
      {    8.3766652852301089e+00,     4.9127089140209373e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5574650986501503e+01,     2.3362582835253437e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.2468171934545129e+00,     8.9033185669909253e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8793616494219329e+01,     8.3267425406401144e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7901851903782173e+00,     6.5923142359119347e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.4324054037634601e+00,     1.4004451838665005e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5080927391454189e+01,     2.3850115806627393e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.2137243805154924e+01,     2.2128861886252671e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1779514059412136e+01,     2.0295391982787942e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1421075303222270e+01,     3.0534753255930173e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.0586368142416376e+01,     7.2266821037763330e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.2418563853665290e+01,     1.3437964030632289e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9233746954437274e+00,     6.4618023534275308e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.3794534511907464e+01,     1.3895419350139131e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4868338161351780e+00,     1.6730463662918642e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.7388637743753264e+01,    -8.3288971398494687e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.5806154959425690e+01,     3.1138477677174782e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0309622018122180e+01,     1.7264360526314591e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5383314442167265e+01,     1.5021096244825510e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8724437625299437e+00,     8.7251094751253349e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9848331404049187e+01,     1.6651482795161672e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3146956302837367e+00,    -1.0228207937459209e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.5242124038607365e+01,     7.9375422824881407e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2858796074679596e+01,     1.0593517238254801e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6028997457193555e+01,     1.5543720692382856e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.1990620129597161e+01,    -5.9000540852293000e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5478671386148562e+01,     5.9578000112253330e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.3384763294945259e+00,    -8.6019711958920243e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3442359998966419e+01,     5.5841780702675851e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4836905370995371e+01,    -1.4088749008389211e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5815942493392033e+01,     1.4921924871505309e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4879748868139888e+00,     8.8930092857354375e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.1782948078068348e+01,     8.3451998553129433e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.8385534412541737e+01,     8.2998410294258865e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5761060560068577e+01,     7.6917473411950723e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2418429828044701e+01,     1.0042761914221381e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0266594940434022e+00,     8.1916205865399133e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.7188847020191155e+01,     1.7562663779100592e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4239268163777830e+01,     7.3733224729499934e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0397274373459995e+00,    -2.8406440227681610e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.1673882603075540e+01,     7.1967898685926775e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.1143164531877382e+01,     6.5878328230965017e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.5448606197326498e+01,     8.8202813072095887e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4608771723571179e+01,     2.0977832594625892e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6856346967537789e+01,     7.2332505351143764e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.2243059486860890e+00,    -1.5995323846908777e+04,    -3.3806534196854351e-04,    -5.3726906696454431e-07,     2.5913824550353914e-09, } , 
      {    7.9202659795125134e+00,     9.6889178660284560e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.0262643762618268e+01,     1.6136116986017947e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -7.7989366219170613e+00,     8.3306403086067858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6165534907770301e+01,     8.2926087972268797e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.4176352268671808e+01,     3.1045693678065676e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.3881965140195520e+00,    -2.4748550006701107e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.8905003484358886e+01,     1.6641840820958470e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.5742183856063434e+00,     2.4420123495854965e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0691116338946541e+01,     7.1618051136140293e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.0703187038937969e+01,     7.6432959409143259e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4468947039983657e+01,    -1.1059920608822713e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.3199924247501784e+01,     1.6672924877535326e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4537725242845788e+01,     1.5606032668448473e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7472557693558990e+00,     1.5542743767777985e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7329806793261636e+01,     1.5542764812185605e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3599157805789952e+01,     8.3493549408803447e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.5743306831320822e+00,     7.8489294060030215e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5515100367194700e+01,     2.7173653603957523e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.7887557907391214e+00,     9.1135793781941029e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1786875154822908e+01,     3.1032539482752396e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.3153075888259458e+01,     8.2462081313537328e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.5023374205407904e+00,     3.9411898804645629e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    1.2482054149935317e+01,     6.6435600144914122e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.7019410294982311e+00,     1.5707047599921360e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.1006749195715649e+01,     2.5597461178108300e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2567763205283160e+01,     1.8216348816820584e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3567025663006781e+01,     1.5500094474026091e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.1214615262956862e+00,     1.7013985997850876e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6795990519521311e+00,     7.6048238657001166e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8755411030701563e+01,     2.3226217448752108e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.6303696345888692e+01,     7.8420181344458133e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2217688797607975e+01,     1.4914452334767735e+04,    -6.3519294630925112e-05,     6.1501330405358849e-08,    -3.5357946576999568e-10, } , 
      {    7.2744394419446792e+00,     1.6187982555834931e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5969212550075392e+01,     1.5560778498183594e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.4975333474695381e+01,     6.7470825091869074e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8145915566526490e+01,     8.2288324644977438e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    5.5084399105932405e+01,     3.9416501204528453e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.9647376235722032e+00,     1.1534992332039796e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1892898774208167e+01,     3.1082878747647403e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0624198484771460e+01,     8.3325047164365860e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7026065616262116e-01,     8.3248781328103141e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7168326447821798e+01,    -1.3292450970430964e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.3690584606922114e+00,     1.5453736776367004e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3759648516211037e+01,     3.1125323481861502e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3645645976008595e+01,     8.3301141155572022e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6511267691446481e+00,     8.3272687336896961e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0491436937388578e+01,     9.1768665138818706e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5977119610135574e+01,     3.1088138412279775e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1674317799642996e+01,     7.2663206171026595e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1167264500776160e+00,     8.9047412579246775e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2016904544793796e+00,    -8.8668275102388816e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.0508565783528404e+01,     7.2366047704905905e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.1501334341785238e+01,     2.4502054499592199e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1770863812974612e+00,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.6886795854495489e+00,     6.2174862944640950e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1284851667419275e+01,    -5.6387323523168652e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.9294650893651735e+00,     6.3093741736172706e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8715256982365780e+00,     1.4319907426677735e+04,    -9.1672058088999267e-05,     2.4437227521798605e-08,    -1.2528555147232634e-10, } , 
      {    1.3977064065143127e+01,     7.1465487711102178e+03,    -1.5684229398419449e-04,    -1.1570573009633805e-07,     4.8025643250710614e-10, } , 
      {    2.1760108653737575e+01,     8.1764316938756156e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7169271010945826e+00,    -2.6867124380370560e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1039933268353739e+01,     1.3294369435855580e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4045174982501226e+01,     8.3023830339968918e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4775475351546369e+01,     1.0280678361890123e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2896673598619621e+00,    -8.9737562785325929e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8494301433116185e+01,     4.0879440512703059e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8893160071658279e+00,     1.6763321043669283e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7801468491327398e+01,     1.6641173513906182e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2910618791943314e+01,     5.1868248670209614e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5399337895802381e+01,     8.7151534387173033e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.5735393129428459e+01,    -9.6532217655829318e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.1589723180141135e+01,    -9.6525171336667827e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.0572097154629470e+01,     8.3434099330491972e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1220103149229985e+01,     1.9708250602264226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2394604887148766e+01,     2.3555977005924382e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3051297384291852e+01,     8.3549998152500084e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6024510726845711e+00,    -2.4694595066252091e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.9094724344071182e+01,     1.4111337484010630e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9134446167160434e+01,     2.4972956362672085e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.3947596562967846e+00,     1.6341914140566036e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.6549735672654720e-01,     6.5988731915110684e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2980102423635213e+01,    -1.4511511592244483e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.7151581795222370e+01,     2.2756823868890868e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0031145026312028e+02,     2.2756810441789414e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5189024345417060e+01,    -8.0807855708529069e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.0108111924083083e+01,     1.6662313881504499e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8755788519114329e+00,     4.4092271702584291e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9870282886623613e+01,     1.0522523932841810e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6657556422874251e+01,     3.1660847067797939e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -9.2695253898601528e+00,     1.4983138810407832e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3395816704845332e+01,     1.8622487697711837e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3044600267495419e+01,     1.5490496538237479e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5822174702549248e+00,     4.0054607580005868e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2648234854193802e+01,     1.6028722948243283e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8959959533848412e+01,     2.4106732329957798e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.8556391101872052e-01,     1.5085298970474949e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0710176855877247e+01,    -5.0435465816027181e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.4655221665329776e+00,     1.4679160089583695e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1733844479351937e+01,     6.2222931813232144e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4235199716370497e+01,    -1.1167061020488063e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5554577457778507e+00,    -5.5731427963255283e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1307274849719217e+01,     1.6066311522490283e+04,     1.5001746465139205e-04,     3.1822544618381309e-07,    -1.6138477816774232e-09, } , 
      {    3.2870452262606037e+01,     2.1595803189717037e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1802159745716443e+00,     8.5375295448928755e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.9038251572859366e+00,     8.5389522358266277e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.6124105056255758e+00,     7.7184175235186167e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.4155764757018883e+00,     1.7126783142658867e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.7245484413953232e+01,    -2.7170443644157149e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.1890195877539391e+01,    -1.1125510164814050e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7117261463023333e+01,    -1.6675431792702759e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.6486575007534022e+01,     3.0459502194859462e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.1691915275002589e+01,     2.7801655411535117e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6234568847576018e+01,    -1.6209335340717018e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.6028155518653904e-01,     2.2677187351544315e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.4271773005331241e+01,     7.7003771260271797e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6494402511340667e+01,     7.6823348961220454e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0691076773971510e+01,     1.6154141194219745e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.6903726057800966e+01,     2.3855903686316815e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.8901470137886216e+00,     8.4984259884618386e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.5190746020622612e+01,     1.6869233514924610e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.2033198685272424e+01,     1.4987741210290653e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.2722086700760855e+01,     8.0181537482001822e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3290603337512108e+00,     7.3253965538256298e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.0107218386506505e+00,     1.3611430868972020e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8108725149466359e+01,     8.2380212524130602e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2069745454188681e+01,     6.4099684915320468e+03,     1.8596879893338560e-04,     2.8646292634728987e-07,    -1.3568528974580690e-09, } , 
      {    1.3110396156687038e+01,     8.3441212785160733e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.5229807054425081e+01,     7.6329431902770966e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3800114487041704e+01,     1.6582131788176186e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.6513818356812955e+00,     7.2761960152307911e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4303766117085134e+01,    -8.5513588015580190e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.8987710148010084e-01,     6.6556450583315864e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1087895994161855e+01,     6.6030282770784697e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8710213842014234e+00,     7.2138392082006349e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1597803585580724e+01,     8.8663288374586045e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.1599544013184257e+01,     9.8224650376247719e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1489807778086048e+01,     8.4273068740268518e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7493275687752014e+01,     7.2142865225160531e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.3845253043373962e+00,     1.1141971362346809e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7131381735022821e+01,     7.1388118042876304e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.8033113679474004e+00,    -1.0796836702557132e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.3597581406416133e+00,    -8.3095132306063297e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.5641057436976858e+01,     2.1577754246261178e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5523444900867023e+01,     7.5169457053149945e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.5007845997821526e+01,     9.1543425506813164e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9072970336629727e+00,     2.3203323227661789e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.5119558011623553e+01,     2.3886987742893674e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.9966668084206290e+01,     6.5856572287389627e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6937555280931768e+01,     3.1103863300887397e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0809376915951841e+01,     1.2481069825707611e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1161297034419363e+01,     2.9881862507083697e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9584608161203011e+01,     7.9663233772891590e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -1.1628233812918133e+01,     1.7308232339424689e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3837917008172120e+01,     7.1983303813590464e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.7169139956384811e+01,     6.8972746186522854e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.3586319900919444e+00,    -1.0247414357714853e+04,    -1.1861601659126676e-04,    -2.1497482196201951e-07,     1.1151150997832324e-09, } , 
      {   -6.5477972765846548e-01,     6.6205900233812281e+02,    -3.3480526443891184e-05,    -3.5831318655234917e-08,     1.1941445779408981e-10, } , 
      {    4.1638731367629745e+01,    -1.6234914386169121e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.3557126089760992e+01,     2.3403173683841545e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1233898593541861e+01,     1.6692719920183623e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.8909667544491725e+01,     9.0123559398553352e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4031604829301418e+01,     1.5595012041726110e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0891294979195223e+00,    -1.0012108991862686e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.4553823211602548e+01,     8.3284371602392148e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.3559523569089706e+01,     8.3289456890076854e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2032889431750657e+01,    -6.7903889733066608e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0922344577260272e+01,     1.6957103814413313e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4088437493381655e+01,     6.5860839126875744e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2204667110216199e+01,     7.0405618160256417e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -4.4018678354302686e+01,    -5.7087898663580745e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2159798999633136e+01,     8.7025675699930907e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8012535943212740e+00,     1.6551444654824518e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3967539503124048e+01,     1.6362577656822932e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.6656704891912653e+00,    -1.7959052415218403e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6396645909676913e+01,     1.0677292206116781e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.9346919247544960e+00,    -9.2260670524767102e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1422747327677046e+01,     7.6938248839787739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4003328219585100e+01,     1.8609369786513569e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1890573333964358e+00,     8.1452199206667647e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.3384598880491033e+01,     9.4094786222733692e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0805811715121594e+01,     7.2300623735823419e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5288815918533164e+01,     1.0076543341676543e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.8743773412848519e+00,     6.6736692665333858e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.3005675590723360e+00,     4.8067707195985540e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4334646506639565e+01,     5.5798458367139492e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5485609260683271e+00,     8.9673082909244804e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9947315774481709e+00,     8.2764336728791350e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0865083340474087e+00,    -6.2567775272935887e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1313018465056226e+01,     1.6972851557927763e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3833217632453053e+01,     1.0435782883914368e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0528934447766897e+01,     1.6345863510907418e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.2796470520298957e+00,    -1.7251927757336896e+04,    -3.3273757898272645e-04,    -5.3850185119286967e-07,     2.7002619115389707e-09, } , 
      {    5.9995546243819522e+00,     8.3809491763677652e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4375708063007302e+00,     9.5213168707809818e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8959388771519784e+01,     1.5589154955854945e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.3031375292485636e+01,     6.6924048202020604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9550652889948751e+01,     9.7601082305946147e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2527543025418701e+01,     1.3721821258778720e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0178394649988913e+00,     8.3805065360193203e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.2395958435490943e+01,     1.6864631115041790e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.8045492894972789e+01,     9.5683859139069446e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4055749453536958e+01,     1.0371348534100511e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0110325818004220e+01,     2.5025889175768261e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.6300100439105520e+01,     1.5371902024443094e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3080590052945219e+01,     1.6442766311468909e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1654617599457424e-01,     2.4946259371972435e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8006199501914992e+01,     1.1630593536507782e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0426329996267505e+01,     1.6694142611117375e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9917734739377178e+01,     4.4809650209632946e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7901443190280748e+00,     8.0182093328307928e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1639177490568919e+01,     1.7856910120674405e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.6245941193363038e+01,     1.4485213629380269e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0289361198401611e+01,     1.4585860085057122e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.5706876939887175e+00,     8.0066637603434538e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.8035868890347704e+00,     1.0748950392653820e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1239595657682173e+00,     1.4444335066057407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3949298604375439e+01,     8.5780557922576627e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.5838952638795259e+01,     6.8574597167543725e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3315885206367950e+00,     9.0525589835467836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5205560962985622e+01,     8.2484923653491787e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.3277503543189468e+01,     8.9570057232197178e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1565768724924101e+00,    -7.7341708968645498e+03,    -1.2404171194066790e-04,    -2.1369840450239435e-07,     1.0069628638013171e-09, } , 
      {    5.8124687114764590e+00,     6.6195423375994451e+03,    -2.4672627970781806e-04,    -2.2562162128739358e-07,     1.0569859394181992e-09, } , 
      {   -9.0696678000893236e+00,     9.5698874835748593e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6723268247275556e+01,     2.4815222008331650e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1779877143653393e+02,     8.3266559076063750e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0680452244023334e+02,     8.3307269416405252e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1425688805892866e+00,     1.6062837941156900e+04,     2.7643205206657822e-04,     4.6437364542574921e-07,    -2.2436740829436326e-09, } , 
      {    1.6742272787453686e+01,     8.5236376569217118e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.4751111100422705e-01,     6.7514094248126639e+01,    -6.1633289901965333e-05,    -7.2895421538795167e-08,     3.4770837209175919e-10, } , 
      {    2.1333016509283105e+01,     1.4350573469704566e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.4344124757871484e+00,     7.8070333794671660e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0096154966689809e+01,     2.2064075220480736e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6514842238479375e+01,    -8.2799895588088930e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.2178086757765051e+01,     8.1306166872287240e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4379028118185211e+01,     8.4287295649606040e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0958979320533615e+01,     7.9548152792538094e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7972919487355622e+01,     2.3245767961875885e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.3884278044495595e+01,     1.7228602535628863e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3150573845385214e+00,     8.3260884006833676e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.7681434671201053e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2208011736054463e+00,     1.6577200755918595e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.7134676461328510e+01,     8.3283032828299147e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8128974975948324e+01,     8.3290795664169855e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3071572885087258e+01,     7.1824236761250049e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1872234793095686e+01,     2.3028521591781711e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.7380766476391173e+01,    -8.3408973493474296e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.3852854392040710e+00,     7.2065846987892874e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3669494276166056e+01,     7.2150292677594080e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {   -2.5976842669806899e+01,     9.5680335979488682e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1002403853209293e+00,     7.0934061625102149e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.8147661978177752e+00,     3.0454899794976634e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.4955790738033304e+01,     1.0037786207318912e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0700648320755768e+00,     7.9379304242816779e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2657851802839964e+01,     2.0101271571957212e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9868494897499108e+01,     5.7030162555913548e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2071105181082304e+01,     7.1721143948695171e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4307701315172643e+01,     1.0986538207947789e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9946607097199367e+00,     6.9829843858759290e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -4.9527910574432958e+01,     8.3164854998994724e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.7725586284469468e+01,     1.6661792076417158e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2072397382833280e+01,     2.4986785619337224e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.4014964976278685e+01,     6.6812248694345544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.3431936780644111e+00,     2.4985362928403472e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.7745098828455220e+01,     1.7162831559062466e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5468199331869190e+01,     8.4073996569827523e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1025488893187196e+00,    -1.6638195179490867e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.6013272224737211e+01,     2.2826706933358924e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2200696233224274e+01,     1.4455355692779769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5617188826416196e+01,     2.2502385715664241e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.7380476406050583e+01,     1.6871999387024895e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3978063874083798e+01,     1.6639750822972430e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1977486348337624e+01,     1.4978881054750109e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4997440836427455e+01,     7.2215410319274015e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1721301948788869e+00,     6.2860343151463885e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3453723418297141e+01,     6.5932334468818581e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8897136267431716e+00,     8.4798668233647477e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7519730862109153e+00,     9.1327857984497496e+03,    -2.5215197505721926e-04,    -2.2434520382776838e-07,     9.4883370343628348e-10, } , 
      {    1.4570845327096652e+01,     5.9236418874810024e+03,    -1.8746307420800890e-04,    -1.5224070732833534e-07,     6.5265617750965711e-10, } , 
      {    1.1549771283551458e-01,     2.0954946573630150e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.6666442838386089e+01,     1.6676570519002933e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -6.2253062536962904e+00,     1.0355929422960833e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0569620708694769e+00,     5.0136714015408515e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0981616815148456e+01,     1.6719515999119347e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7101462276608566e+00,     5.9213992788636597e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3747400857499711e+01,     7.7375957175357371e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.5094286066408271e+01,     6.5836777375291003e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3727873727974231e+01,     1.6756709644117178e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.6472173902706064e+00,     9.9629084532678462e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.2756204807323883e+01,     9.7826501357268589e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2480583465871964e+01,     1.6737564942575205e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8426188891935809e+01,     9.7999231324925277e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1544700264859884e+00,     8.4557217023113171e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2922875041662625e+00,     7.2457020545916839e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3840813386122786e+01,     1.3830985000325278e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.7143646527745304e+01,    -9.4436722998466303e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0849225445898920e+01,    -8.3406890353669223e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    6.6249427708237310e+00,     7.7996886892425294e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.0087160888702044e+01,    -9.4429676679304775e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.8389053283789295e+01,     6.9370895205501974e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8360009663818609e+01,     3.5034072997817257e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7062389631735659e+01,     3.1677767748605536e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7897707578102178e+00,    -8.3636363136328437e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.7618416371039977e+01,     1.5550232456550852e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0980186269336542e+01,     7.2147674972744944e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7137926202937258e+01,     3.2042402784888265e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.4793852614660654e+00,    -9.8363409535815426e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1656986458007195e+01,     1.7619192826914019e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.5611469370002833e+01,     8.3290460280732677e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6443692247411882e+01,     1.3224479657836793e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7132056402231473e+01,     2.3400872483900133e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3710061909963249e+01,     1.5959185486182572e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5887902346656801e+00,     1.3218409899279031e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.5759115325639282e+01,     5.9728668064069352e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6468015106737699e+01,     8.2698222733270304e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0295353698308361e+01,     7.2560113358471717e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1250691084707540e+00,     1.6100068569614348e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.9774098727846273e+01,     7.7007124719526801e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.3646047492757454e+00,     2.5161866692482588e+04,     5.5096721960341630e-05,     2.7647615236737837e-07,    -1.4686945655532281e-09, } , 
      {    6.9519367444206734e+00,     8.3997091103675375e+03,    -3.3189520425500942e-05,     3.1102494491060616e-08,    -2.0328237648922845e-10, } , 
      {    1.4050046831901968e+01,     2.3162956036231393e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.1246426914263985e+00,    -1.7632026274469709e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.6674983180551592e+01,     1.5153336116323724e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5984451793277573e+01,     2.2738792947138343e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.7180987401213304e+01,    -8.3283368211736342e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -7.3299808834317218e+01,     8.4464051618399481e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6436563569710394e+00,    -1.7429361443106488e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1218610664468157e+01,     2.3791263621276936e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.4954983397998625e+00,     1.5535276123412737e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9558105038556153e+01,     2.9671159055532069e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.5025013944130823e+00,     6.0687813837345439e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.8345568902594628e+01,     1.7522848877202676e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.8415944770759495e+01,     1.5971987365537778e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1769572584978496e+00,    -2.8624804763181965e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4163009678828555e+01,     7.0689477494358998e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.9706642842563706e+01,     7.1877544747317861e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2346217011548688e+01,     2.2836446959135967e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.8621600867748242e+01,     7.2133582334421944e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.4168211257163890e+00,     8.4248296591087674e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1730836055018363e+00,     9.2512453512428092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7663055556273513e+00,    -8.3345914787086767e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.2066327275139599e+01,     1.5516445899355236e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8656860169544849e+01,     2.3700593449066546e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.4522165694671560e+00,    -1.6830883898579606e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    3.3751509522997438e+01,     5.9597488952086660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0004512059309604e+01,     3.6124397883076008e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.1411003784405676e+00,     1.7284979088825567e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0975244324796272e+01,     3.2775475627063046e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.6475215316318739e+00,    -1.0438911389010831e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.5611306335207381e+01,     3.1611644456081067e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.8409752175330127e+01,     1.6029427580159432e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.8927053258535196e+00,     1.5533894134441252e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.1969757829659518e+01,     1.5551614445522340e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2546491792562424e+01,     1.5969556687129943e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.3929981651768031e+01,     7.7713704314401703e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.7249867003121111e+01,     7.7713838585416252e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.8786447039182249e+01,     8.1296169151338891e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.9015243331957606e+01,     2.3871347663031403e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1125829781444338e+01,     7.2625142656390908e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0491933433617078e+01,    -1.9973675839865518e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6061485598103062e+01,     1.6210876776925250e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1080341795144064e+02,     2.3871543766179086e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.6225081558502605e+00,    -8.4929847345630169e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.6693288296415387e+00,     6.2830195521406040e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {   -5.6853044634050764e-01,    -5.9454490808999708e+02,    -2.8152763458074149e-05,    -3.7064102883560250e-08,     2.2829391429766939e-10, } , 
      {    1.5324414858643816e+01,     1.0764369503793499e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8390672180925183e+00,    -2.0584048529915126e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4276054419544215e+01,     2.3345309838487770e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9152373476799216e+01,     8.9407896743282763e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6540509176323745e+01,     6.5854266479380285e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5661168371849916e+00,     8.2654130461567711e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5701613128408031e+01,     2.2657490360469863e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1309571235024203e+01,     1.4948233762222895e+04,    -9.4433871945491906e-05,     2.5097252867261127e-08,    -1.7899805920245181e-10, } , 
      {    2.5437020941072323e+01,     1.9002390756206558e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3603003839399015e+01,     9.8669938759402394e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6238995110488389e+01,     1.4445757756991157e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0609901456354887e+01,     8.3850147572108544e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.0427078184840155e+01,    -8.3601716521372000e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.2570392309373709e+01,     8.7240135942578490e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.4503160215579820e+00,    -1.3971962187754312e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0062989625273286e+00,     8.4852717931485604e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -3.1940023802097471e+01,    -1.1944470569373675e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4281883025741010e+01,     2.1988824159410025e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5437045275822571e+01,     1.5511115100748455e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8542950271648193e+00,    -1.6621334432843305e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.1051390926559131e+01,    -1.1359584672429576e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3114173102412671e+01,     2.8194676381228106e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9041942254483455e+01,     8.1091372932701943e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.4403959835893847e+00,     3.2160322237330784e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -6.9791065013352210e+01,     6.6012973086012298e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4426976360721424e+01,     1.5500805819492967e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8581590073480942e+00,     1.4443873474231139e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3079723646618120e+01,     1.8012213796607008e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.8047013649067392e+00,     1.5538345062811535e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4630857165050180e+01,     1.5764922994136083e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.3191260990429399e+01,     7.7003905531286346e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3058555679122680e+01,    -2.3314288274582796e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4245471774570797e+00,     8.3919698030901291e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5610536225344148e+01,     2.3285797945515405e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0234913148125258e+01,     1.7226171857221030e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.8811832045278507e+00,     9.7173037994387141e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.5616312062672030e+01,     1.1769853662952619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3231885461929505e+00,     5.9044785980560664e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3734351110437153e+01,    -7.8712361051166035e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.0488386568736967e+00,     5.9574476952672576e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1340928226108545e+01,     4.8739859906562815e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.3428218014497437e+00,     7.2403712559849027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1038677459771566e+01,     7.0041610836639693e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    5.6315279862878796e+01,     7.0048657155801175e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -3.3244190113323732e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5288965558479337e+01,     8.2595129920715426e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9111919862215185e+01,     2.2149839478272450e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8355374827880695e+01,     1.4795001999242515e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8102425973583870e+01,     3.2239952041126609e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.8653210750103479e+00,     1.4246324217992797e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.3247850834581811e+00,     2.4086062252383235e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1012329766217888e+00,    -8.9907748072926734e+03,    -1.1871394895485087e-04,    -2.1493118873071965e-07,     1.1158423203048969e-09, } , 
      {    5.9793977934830487e+01,     8.5277659341130111e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2963601647792444e+00,     7.9629024025253993e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1991013991513702e+01,     1.4994628798264559e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2162336796673024e+01,     8.6621938172450227e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.4423943305597007e-01,     6.8341598068636240e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5808027479306315e+00,    -1.0593996959557258e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.7060099582586954e+00,     7.6790539316000550e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6210474860755955e+01,     1.0673699331583108e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6065274191865146e+00,    -1.0932986512872537e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.9822587224869341e+00,     8.2491309589696575e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.9918810696577571e+00,     1.5569062680608353e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2413492437246774e+01,     8.3264139082653419e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.1419133032008322e+01,     8.3309689409815564e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8162815145495234e+01,     7.5733024075961721e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.5253045029882166e+01,     1.0828803853607355e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5963990180082783e+01,     1.6902980731386800e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.3021012987080914e+01,     6.4291749117876907e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.5099376421477398e+01,     1.4199398070963269e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.1328657913098539e+01,     7.6251327785071953e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.9026739102053813e+00,     8.4706667757864652e+03,    -2.2380334687197685e-04,    -1.8736836740680787e-07,     7.1908534809528565e-10, } , 
      {    1.4818055020140159e+01,     1.9015508667404825e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.9068300054388398e+00,     1.5713606555520493e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7180164892752416e+01,     6.3658877760999239e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3040029977541167e+01,     7.6047826351099229e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6189270115560085e+01,     1.6786251548874614e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.4637167119107852e+00,    -8.5447998459588853e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    7.8214564856701516e+00,    -8.5202166821579540e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.0450440682809673e+02,     1.6656874890607785e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.9822378128013932e+02,    -1.6657890807886015e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.6034134266788193e+01,     1.7602478680998502e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4596242416678354e+01,     1.6168432042711152e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7321031195728107e-01,     9.2255392661935866e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7810398675561974e+01,     2.0494292541650200e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5690599246662291e+00,     8.8503494697797341e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9448819310485220e+01,    -8.5156989852275074e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.9436589939572531e+01,     1.6934356194055050e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5909218674573427e+01,     7.2670224868306250e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8752907854111228e+01,     8.0530644262166006e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7142977070061590e+01,     1.4505677991728813e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.5616471775250510e+01,     8.3276277631984995e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6610881457274331e+01,     8.3297550860484007e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1979103329989433e+00,     1.7319441851585023e+04,     2.7110428908076121e-04,     4.6560642965407457e-07,    -2.3525535394472119e-09, } , 
      {    8.7119190257615120e+00,    -1.3241780250014071e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.9338944661478212e+00,    -1.0045890419317846e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.2622205038855490e+01,     1.4929403038074366e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3264830958361264e+01,     9.8232230423832771e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3035219149253361e+01,     1.8764474308537818e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0935480195219034e+00,     1.2949341856471314e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5531856649049779e+01,     1.5536854235896566e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8419596907858782e+01,     8.3263884896846503e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9782421130043581e+01,     3.8964282694747135e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2209005183488017e+01,     7.9219309160576804e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1636297828412673e+01,     1.5574393479215134e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7249997183202321e+00,     1.4868046039063105e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.2843433203301533e+00,     9.2724677183498570e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3202011163390303e+01,     2.4225601679717811e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.6820944274500214e-01,     6.9323755841326420e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3388196839126765e+01,     1.4592419040656254e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.2709857926754129e+01,     1.5774643824525454e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0848079140526846e+01,     7.3019768515789283e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4156751899722545e+01,     9.6059701109714297e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0459296645707830e+01,     1.8366477631942365e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1965747183819762e+00,     8.2419408999570987e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.8592326431890811e+00,    -1.8231182376389570e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1613198863037390e+01,     1.8371453338844833e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3924335093383474e+01,     1.0082106435402638e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8101791313325148e+01,     3.2593158108921685e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.6656997614425165e+01,    -1.1301705875535354e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2231562741174301e+01,     1.2874090795400600e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3787484328237834e+01,     3.1478529549656578e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.3580234998844105e+00,     9.6359487688172649e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1664110424673932e+00,     1.2825388929586043e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1288695857541100e+01,     1.5565296195114039e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3456298016494856e+01,    -6.3973680036136739e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.1959952151807727e+00,     9.4786570548252785e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.2795680308678861e+01,     1.7817095218776492e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7817607934607437e+01,     1.4607501903046659e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.3518939068652902e+01,     3.8838861516752688e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    5.6283730566652688e+00,    -8.4154419492898032e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.4547922868989716e+00,    -1.4300972679459696e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2027039781255986e+01,     8.3375698131239933e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0457525021859869e+01,    -8.3198130361229087e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.9566132808959164e+01,     1.5403068879097222e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7316052014569108e+00,     7.7078620061469637e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2646826636770330e+01,     2.5004098482072146e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.9429121516364848e+01,     1.2023614506200771e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2387080453956393e+01,     9.4800797457590306e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2650235911647757e+01,     1.5584702760470622e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.6994495088440544e+00,     1.5531264302125068e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.4320901366764232e+01,     6.8246446917002750e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5010018105378336e+01,     2.2927669420878839e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5776597328492819e+01,     1.5554244277838521e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.9462836181299963e+00,     8.3460042873583734e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1292895296757820e+01,     1.0915544902534797e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3691657707152221e+00,     4.0698951078144446e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4457915539398620e+01,     1.9157495278230806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2985061340344515e+01,     1.6099716253656274e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.7872978026462469e+00,     4.4629160775074315e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5652699747613390e+01,     8.5502385011032238e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0711415293170804e+01,     7.1430751897768369e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    5.4376155057126539e+01,     1.6100420885572423e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.6878354176587225e+00,     1.5654087978449079e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.0853219882792274e+01,     7.7000552072031342e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6229540758837661e+01,     3.3312464498552392e+04,     7.6146203881163085e-04,     1.2535943707732736e-06,    -6.1799199931032542e-09, } , 
      {   -1.6469006538152637e+00,    -5.2613587611747471e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8988740321553355e+01,     3.3350814114897395e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.4185530269601500e+01,     8.9569922961182674e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5683168738304176e+00,    -1.3433562190185280e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3618118706117176e+01,     2.9987800701506079e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5667496699374205e+01,     1.4288768952206890e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0510815801139870e+00,     7.4559084363100337e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.6095978300563658e+01,     1.8151156462248218e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.4613416533782662e+00,     7.2054263669755110e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.4803995426003546e+01,     1.6673115333246202e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.9921549836240118e+01,     3.2253106236439889e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.4634253529268637e-01,    -1.6663941804846032e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.4920006701638972e+01,     3.1607877970586749e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.5778168704916638e+01,     5.7428311574892668e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2480310131415036e+01,     2.4952292846415188e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    1.4885754826840143e+00,    -2.5019855701325509e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {   -7.4251118392255169e+00,     8.3309943595622499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.8156995035255825e+00,     5.4999923757604138e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0337457071255116e+01,     1.5030694180614122e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6811046206889010e+01,     1.7148619596208904e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.8137616996891692e+00,    -8.2038221769129018e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4848222955828092e+01,     1.3013776206285171e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4503072337238144e+01,     1.8405234766299989e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.0104355674558640e+01,     7.2347941712979718e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5750440783981645e+01,     1.1379559177640776e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.8474236210942934e+00,     7.3008039142857860e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.9542002218505203e+00,     1.5612644068000578e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3909866396808127e+01,     8.9566703772942201e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9883708788149292e+01,     9.1145276487834035e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2951001729573580e+01,    -1.1085950848223529e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -6.9001239926766118e+01,    -1.0990865309766759e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2663885518708884e+01,     3.2986179078614674e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.2459889414628336e+00,     8.1360027055347155e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.9685855546472141e+01,     1.0575406606015249e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7567948678748913e+00,     5.3717999261774185e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0178249477788004e+01,     1.7978432369151844e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8727913684309900e+00,     1.7789845607142468e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5724542528239063e+01,     3.1649381815195276e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.7451827013189440e+00,     8.3559138964794765e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.2231488503824753e+01,     1.5472864511963011e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7766694062999004e+00,     7.2014436269403577e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.1255207918513968e+00,    -8.1271862367260564e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5571169624609986e+01,     2.4979515318271213e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -3.8646096738995297e+01,     8.3257092763491910e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.5161780980273951e+00,     7.1948751956023143e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.5376279775472771e+01,     2.2094758153002018e+04,    -2.5117823356610229e-04,    -9.0652110460376752e-08,     3.0053115278298998e-10, } , 
      {    7.9383347145682777e+00,     9.5639766867366852e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9640394980052619e+01,     8.3316735728977092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6720525275967081e+01,     1.1157390473486486e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7530057876424749e+01,     2.3316432634914101e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    6.1209834560163365e+00,    -8.7934194391255005e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6302536521755226e+01,     3.2147168042017503e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9751402058175117e+01,     7.2604635312314922e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.1254813221579571e+01,     1.3799465829713070e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.7955878940433038e+01,     1.4428119017165964e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.9507150409965966e+01,     1.4428132444267416e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.2450711608887623e+01,     8.3236243525817608e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.3013489179581004e+01,     7.0022121996806327e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.6870052910838012e+01,     1.9395411725899543e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.0047283468094690e+01,     1.4985792326307319e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.7284336016508558e+01,     1.4985087694391166e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.0056360883028933e+01,     2.3311830235031281e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    1.3827809438351601e+01,     7.1934443028473697e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6791878454876219e+00,    -8.5213801437121856e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.6550585490194102e+01,     3.0456848678959974e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.1882957545939011e+01,     7.9150003773558983e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1357083037337162e+01,     3.3317066898435209e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8751883667658582e+01,     9.3964206452351282e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.2763134868385446e+01,     5.2670069603492684e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7559943618145738e+01,     1.6148519969895093e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.2963813196122889e+00,    -1.1206620337078566e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.2337133668171020e+01,     1.3777276240538902e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6223678155263226e+01,     3.2197507306912514e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.3832739413117817e+01,     7.7717057773656707e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.0512858318610615e+01,     7.7710485126161238e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.4322302244120584e+00,    -1.2128443365341227e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.0627563224009693e+00,     6.3504686649602463e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2232371530112173e+01,    -1.8465474156245675e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9577189855484505e+01,     2.2075540473083405e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3107362721498264e+01,     8.3323893063268333e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7146273278750686e+00,     8.4514903656934002e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.5879832142737833e+01,     1.9550516247923795e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0698859428406120e+01,     8.4023145623526252e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1796101355236036e+01,     8.7811710372278249e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8905211294374389e+01,     1.3330417852259176e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.8761524860388644e+01,     8.3275631619751803e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0275261791179268e+01,     3.2202766971544879e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.6711404911015379e+01,     1.0469564311369530e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.0573224145665339e+01,     7.5879390107279842e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8194614293676725e+01,     1.5540612051759974e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.0239939134420526e+01,     5.5939855962776719e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2125950737676634e+00,    -9.4962890810997451e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.0221198900888631e+01,     1.6688035771605599e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.6683870750217080e+00,    -1.0405129961555673e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.6484417961917202e+01,     1.3737461338640989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2723454642031236e+00,     8.6349713453889963e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4996659469830064e+01,     3.1243242934304017e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.6815973063188956e+01,    -8.3575418198210136e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.1427200949831139e+01,     3.2194188186146130e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.5414261864564942e+00,     7.2346814278693191e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.9462287157515938e+01,     9.4485888922551312e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8080803034778645e+00,    -9.3903508866773627e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.7981752758833629e+00,     7.3591779812807890e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    9.9435195621374906e+00,     8.9393669833945241e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2974392720705730e+01,     8.5301966125208455e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4082997592790477e+01,     6.5688345995052096e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.2289586761541912e+01,     7.8510069487867222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.8969490646427552e+01,     2.1540016887146965e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.8602874344967493e+01,     6.7589087457929118e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4632047948208111e-01,     7.0694464222161796e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2299318305406544e+01,     1.5129063242714243e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1550952644018793e+00,     1.4977725083402931e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1680416634739963e+01,    -9.6866264967489187e+03,    -5.9646608182209927e-05,    -1.4146300830906168e-07,     7.1296699943968341e-10, } , 
      {    8.3888847484099518e+00,     7.2815769596064711e+03,    -2.8010887378812518e-04,    -2.6149657317392838e-07,     1.1756731766906245e-09, } , 
      {    2.0778066389481886e+01,    -1.9126707081490381e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.6563407404412402e+01,    -7.6877646011599199e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.3735081711922065e+01,    -2.2647014654389488e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.7986858764564584e+00,     6.9765162806545741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7620603394454804e+01,    -1.1123273593236936e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.8823841999817645e+00,     1.5544896528203617e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.1359598324864937e+00,     2.4828339919529917e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    9.7701544758326726e+00,     8.3041082608225170e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0548952164577486e+01,     1.0984192239243887e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2343290887030535e+02,     7.2120273483412693e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3242067451421224e+00,    -9.0502990141955652e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2579895941026922e+02,     1.5540718772964719e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2193277243601983e+01,     1.6164502919428205e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.5268299872616012e+01,     1.4224005970018261e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1204773977162017e+01,     4.2894492391677013e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.9740437131886077e+00,     9.3204611086585792e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9755525787208089e+01,     8.3298196872717199e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8093180574184593e+01,     2.3977383909027627e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.5457919031352727e-01,    -3.2555014460111843e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0551406425777124e+01,     2.3951627807933553e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0106720464702856e+02,     7.2160983823754195e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8007990374773762e+01,     1.6630152887183820e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7470618075780212e+01,     3.1049460163559994e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.6638865307232507e+01,     1.4681461289525107e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9495860952919255e+01,    -9.0001202148711468e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.8722251660598573e+01,     1.5544789806998868e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1685992687348747e+01,    -1.7002763283549452e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7199955374242137e+01,     8.3363180082497202e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.4821884271379027e+00,     9.0345512706811169e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0101321589636374e+01,     2.1030801691837085e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.0360407875293429e+01,     3.1121556996367188e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.2254081471131815e+00,     8.3532745884243832e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.2056628421391906e+00,     8.3210648409971800e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8217007121441195e-01,     7.1883237302102343e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4951746639653027e+00,    -2.3285839134004882e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.7345131789720556e+00,     2.2757145787714911e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.6894323778630870e+01,     2.2756488522965363e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.5715579008123562e+01,     2.2381845129103014e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8290151057215951e+01,     7.0068145995634513e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.1688958149877298e+00,     5.8930133454534016e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1190194379615695e+01,     8.2580903011377904e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.4474528146073652e+01,     8.4065902599420351e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3010424666920045e+01,     3.1678824670605945e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3172328025459379e-01,    -1.6058653062271096e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.3229502447312473e+00,     8.2691728475520376e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.2828017556395029e+01,    -5.6595077801538696e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.2335525824070004e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5622446829268952e-01,     1.2432367959893056e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.1686166570013778e+01,    -1.9575526820886398e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1777903509601181e+01,     1.3802119345612558e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.2305023273580993e+01,    -1.4819123793418391e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4403158885562576e+01,     2.4790585377054276e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.9243990450062860e+01,     2.0887313511343185e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4380334976786267e+01,     8.2445802080121903e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2474506366924163e+00,     4.1643119620077772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0088740854056049e+01,     2.4499753299650787e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7059937206652208e+01,     1.5229915413617116e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7841861102130643e+01,     3.0064180025225058e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5252242702584587e+01,    -1.0785801428615077e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    4.6143723020928640e+00,     1.5555872201180062e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6178886209417215e+01,     7.8953395476150790e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5305494136556312e+00,     1.6029786609668234e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1499549862210657e+01,     8.7927056997294276e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3050006021940085e+01,     8.7423824718910018e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4536500026652533e+01,     2.8587697350921095e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.7519919768440676e+01,     5.9551464953258474e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3656439242579474e+01,     1.5585414105937498e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.2681051744106711e+00,     9.3136021582729645e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3856394008429170e+01,    -8.4377239131493225e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -5.1734432188533575e-02,     3.6768741381214577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6453409360740793e+01,    -1.1458069627801838e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.6605898559963805e+01,     8.7804664053116758e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.6055925145884274e+01,     1.0116711379198587e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5508628952129190e+01,     3.1085508579963589e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.2337622717106580e+00,     4.7320306302129484e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0083948967756978e+01,     1.6133825616738413e+04,     8.8384174749426707e-05,     2.4533002464501789e-07,    -1.2661394095856640e-09, } , 
      {    1.4869527128180046e+01,     6.4809561671135543e+03,    -2.0790783837658933e-06,     6.7419305566558618e-08,    -3.7931822410010095e-10, } , 
      {    3.7779012102428588e+01,     1.3879447949466970e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6675530774841647e+01,     1.1221824823300343e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2655520651483929e+00,     8.3195026367081337e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.3511074456789824e+00,     6.7500485902523669e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4101058665863910e+01,     8.3474269782921256e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2385369026581404e+01,     3.1021074230149734e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9237313516670746e+01,     1.8537711599976528e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.0953074815866941e+00,     1.4921005660535386e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.3886293344132177e+01,     1.5814458726423365e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7158669023372006e-01,    -8.4082518902772426e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.1823991336913258e+00,     5.1069714060674260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3018512323087574e+01,     2.4165692056179061e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.4875404148819165e+00,    -4.8632097421956246e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.4005570501904856e+01,     7.8266379664045908e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.4489908864207486e+00,     5.4039359882495944e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5925753522567994e+01,     1.8798255735992978e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1773582996097694e+01,     8.7036634508605766e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4510234055133351e+01,     6.6218036967164016e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    5.1840763095842588e+00,     1.0593819442238274e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0372295603176153e+02,     1.4938719578545733e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3297428189620689e+01,     1.3817091142436813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1271832291096633e+01,     2.5143808628210780e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.7301745741655665e+01,     1.9943537217616780e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.7248848403848122e+01,     1.8111341560350309e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.9073421344410406e+01,     1.5542366148188261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2799007612870890e+00,     8.9495208430999355e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.6150510981994969e+01,     1.5543142431775332e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5727219615046693e+00,     1.6860746647567908e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    7.2654736305314884e+00,    -1.1948106525934136e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.7767940164668026e+01,     1.5542548574755774e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.0861851142603818e+01,     1.4712317078690776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3108012376865535e+01,     1.4925467331658616e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7240067949512682e+01,     8.3020307180388172e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6113704110004221e+00,     1.5320585585827506e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0155070975863723e+01,    -2.7229962063081111e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0781802722538165e+00,     5.6757604140415560e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.1728521829623659e+00,    -7.2129593379641401e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.4138319028277884e+01,     1.5480397482951637e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4801321908943089e+01,     8.2911521813048821e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7468569683460608e+01,     7.1315795720886272e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5360644348471748e+00,     5.4842446322459627e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.0603470540765201e+01,     1.5343679780492235e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8345555652717650e+01,     2.4478423391672935e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.1423539732813595e+01,     7.2136747235648090e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2581290837507439e+01,     7.4336169967116020e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8235101579120887e+01,     8.3553521312080829e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3789508209503154e+01,     7.2144510071518798e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7000365446994792e+01,     1.4434684686315824e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -9.5633482971605929e+01,     8.3667753580441240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7710508742496422e+00,     2.3865412240162495e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    9.2382041982832170e+00,     1.6029075264201358e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4784094783185655e+01,     8.2883510415691562e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.1804843115990305e+01,     1.7166245728598708e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1194062010186376e+00,     4.7783847861882959e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8009401045624562e+00,    -2.1593673405114264e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5436797411152794e+00,     8.4334361473290228e+03,    -6.6572114505808005e-05,    -4.7724573954741622e-09,    -8.4595139216802968e-11, } , 
      {    1.7869782025366181e+01,     2.2162272247250137e+04,    -3.1281152346806753e-04,    -1.6354753199917192e-07,     6.4823952487474922e-10, } , 
      {   -3.3981673338226997e+01,     1.0054354572168431e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4845308583586032e+01,     1.5542960005207817e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8587581299905569e+01,     5.3689550121196271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.7319841759307110e+01,     1.6560480048685407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6053594439193660e+01,     3.1007956318951463e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.7212551643188064e+01,     1.0076896477300674e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5202088048814133e+01,     8.1758881025406117e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6746105132231193e+01,     1.4410101522514891e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5025454453049328e+01,     1.0260014845633228e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.9966385959072257e+00,     8.3166938138799778e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.6983500524812122e+01,     2.4093614418759535e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2280530974992164e+01,     1.8875525874528130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1283889849630047e+00,    -8.3298548861776817e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    5.1114068900748224e+00,     8.6959752447001829e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1201320058673780e+01,     2.5039043371081541e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.6718410237780954e+01,     8.9573276420437651e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.5258637410966003e+00,     1.3797516945729738e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0723576128959650e+01,     1.6363136507673085e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4185821038885578e+01,     5.6600279300422571e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4489691841788165e+01,     8.2102131213307766e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3732358189906032e+01,    -4.0613888089125430e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6841249507406658e+01,     9.1081744084102920e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0546287599121609e+00,     8.5912099875709428e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.3481182462554905e+01,     8.3509828642218836e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -3.1332878672064901e+01,    -8.3063999850250166e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.0266530645724231e+01,     6.6007116134509142e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4624937723851460e+01,     2.5772116213256326e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.4931335479583652e+00,     1.4348495926920865e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.5764748915648283e+00,     1.2352852621132345e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5574865064733626e+01,     7.1894797015574113e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1565473316497947e+01,     7.1980633571343469e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7434362528778934e+01,     9.0394823029387098e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6993246854361544e+01,    -8.3137709862640550e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    3.0017786125889977e+01,     2.3877479189047997e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.7642397742135103e+01,     1.1066720301276095e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0786145144603985e+01,     3.1574130754124559e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2469731254467792e+01,     8.4699666899825552e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.2300654731054266e+00,     2.4983444441554166e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2186538210183606e+01,     2.4988704106186531e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.3055800096078922e+01,     8.8951792326561699e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.7619236224924313e-01,     7.7521989509737759e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.6170932001148568e+01,     1.5522135727470819e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4118175406672787e+01,     1.6537938143553165e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4878491408270170e+01,    -6.0043470339206870e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.9412911609827123e+01,     1.5642081084852072e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.6401861480727717e+01,     2.4189215623227523e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.0705591259614593e+01,     9.7202933286967036e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8504124144196794e+00,     7.2139144483011305e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9856206638860661e+00,     6.7020027559906721e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0759892136260376e+00,     8.5356110580435670e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.0065904103631524e+00,     8.5408707226759343e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.4718306464579893e+01,     1.0216244012076262e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8301850341456944e+01,     1.9788432695592532e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2721255945536335e+01,     1.5631771803596585e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6516771536506909e+01,     7.2142112824155583e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1410359500947401e+01,    -1.6802497965169347e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.8181876084520474e+00,     8.3176707979010862e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6639740728034688e+01,     6.5519794826891248e+03,    -1.8489712507868453e-04,    -1.5281346621119814e-07,     7.0782312628311120e-10, } , 
      {    9.1738819174503377e+00,     8.3397120513458140e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7914094385812859e+01,     1.5518171126180861e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6147796779760064e+00,     1.2666061850810034e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0787411530994362e-01,     1.5605111097011952e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.9836669679347978e+01,     1.6776827554940639e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8231010164429726e+01,     1.7541640565901893e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.6361479726362568e+01,     2.3866514682347650e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.2627056879974461e+01,     4.6824702088606900e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6533569211996704e+00,     1.6291593827148850e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.3132032026074944e+01,     1.3406797175978158e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.1382792659045542e+00,     2.4463704883247188e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.7190323254639189e+01,     1.1772580147333763e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4167944555980192e+01,     8.4123039684544274e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    5.5114284688247173e+00,     1.4580385461464524e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1476439518302524e+00,     2.4933105176659155e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.7686519309029443e+00,     8.5912005118320321e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.9930223584361872e+00,    -1.6610982183373751e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -6.2254725293947317e+01,     8.3260379087239653e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.7814511722326447e+00,     2.4200032334484371e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.4952984957052227e+00,     8.2925830740598267e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6462615437420514e+01,     2.4465990622695241e+04,     1.1426199509656672e-04,     3.4990069955773649e-07,    -1.8722971069401059e-09, } , 
      {    7.0521844388588093e+00,     1.6204873312645079e+04,    -9.4531804309076040e-05,     2.5140886098560985e-08,    -1.7827083868078751e-10, } , 
      {    1.7361598443882738e+01,     8.3235027160050977e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6990256239155297e+00,    -8.2975130211083724e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.3654347231392549e+01,     1.3267111765093587e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6540102894956066e+00,     1.5716255339314497e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.6785082312765987e+00,     1.2039346990200069e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2440704189723704e+00,     2.3933802521635404e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.8344745781770069e+00,     6.1125522172915062e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4508095236199363e-01,     1.0969965329906358e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8144194160143062e+01,     1.1550411443179475e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4853488971003275e+00,     1.5243033324815382e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.0230699490153281e+00,     7.1603824226802762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5497481282429533e-01,    -9.1485147914645859e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4705457297719047e+01,     1.5403034668104252e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.0851827748555500e+01,     1.2416635475893758e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4676285697739537e+01,     2.2694460348309982e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3273679041087410e+01,     1.6703783515120049e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.8776019541109250e+01,     1.7585411399458855e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3037247459532011e+01,     2.3068336493679621e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8450060254465235e+01,     7.4957501465840487e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.4356643426895179e+01,     7.4771308034170816e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.3102145759749755e+00,     1.1087285051798765e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6629170907997377e+00,     1.8383398312749956e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0955099011435649e+01,     1.4808508510513870e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2647154418960888e+01,     2.4968801277104682e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.6690685996729087e+01,     5.7826460593871798e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4824787242294361e+01,     7.2936233310121370e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.8695293393090502e+00,    -7.4883953716705537e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.5460098012552820e+01,    -8.3296730865396694e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0838366068948950e+01,     1.6639333905791038e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9288702105857769e+01,     8.7523653166751355e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.0912959742838229e+01,     1.5563372852492770e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.9568171631662850e+00,     8.2152801933724659e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6240598952477640e+01,     1.9408529637097814e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.3718557180030704e+00,     8.6415303009881281e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7934284003136906e+01,     3.1102429260771187e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -8.1881619801281040e+00,     4.0526527363910733e+04,     5.4298645492547107e-04,     1.0649932191381403e-06,    -5.3519551885043885e-09, } , 
      {   -5.1893158862392582e+01,     7.6034810390164130e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1862778265637292e+01,     3.8628158065201060e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.7316655366621678e+01,     8.4246060019510587e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.6444436629171388e+00,     6.6114924299290560e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.3140858953921306e+01,     7.2081628112731169e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.3724858448009151e+01,     2.2805735571458008e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0450047874224886e+00,     7.3663225961061789e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9776904558121998e+01,     1.3800170461629221e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.5274029039583951e+01,     1.3660132734786575e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6140754701529033e+01,     2.9972096516641250e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1335472238834265e+01,    -1.3771471222125986e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.6092972169492242e+01,     8.3053849534150304e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6962233623958156e+01,     2.9306524019249340e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.7901941914011520e+01,     7.2184720925286047e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    8.2561318876866991e+00,    -9.4426086384216778e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.6876995854299608e+00,    -9.4440313293554300e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.7711291100248342e+01,     8.3436477577615642e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1356665769578401e+01,     6.2184714602360673e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.7682140180222596e+00,     6.1156436414339551e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2884560584599559e+01,     2.4482832618843197e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.6075959977284242e+00,     9.9176968403512956e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3038407774093550e+01,     9.2593498071515914e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5692855834789789e+01,     6.2133149872446666e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.7184839475581466e+00,     8.3137350914853359e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.2954531666300696e+01,     7.0149883558687834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7912754113301643e+00,     7.2386460291592775e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9798059945894986e+00,     1.6951629190820713e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.5809395697545941e+01,    -8.4421026558744361e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.4992641958084132e+00,     8.2970522353901106e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.7607398211196937e+01,     8.3277097627072326e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0859177570445292e+01,    -8.3275878972292430e+03,    -7.0844439523505521e-04,    -9.0969891283933662e-07,     4.3319072034499252e-09, } , 
      {    1.0791232035108520e+01,     1.4945316177598261e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.4497083266310895e+01,     3.0457553310876119e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.0121057414278880e+01,     7.2138571501323258e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.4871771616339391e+00,    -9.9345567308505615e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.8459750634184289e+00,     6.1266642681563198e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2849326005740441e+01,     2.8263978489216242e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8726955725171862e+01,     2.0336558187309769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6283295371184852e+01,     7.2677433080364126e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0488267773086285e+01,     1.7896725022572318e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5446035206013796e+01,     2.4535801716054390e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3852998546592739e+00,     7.2162051035801660e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.0063741977874985e+00,     5.7352789911129694e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4936056906832822e+00,     8.3603306138567896e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4355234515251571e+01,     1.5718522328262934e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -7.2008153789020895e+01,     8.1521588202661333e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9432534972490807e+01,     1.5531937578724939e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    9.0559538278996428e+00,    -1.2712089277902157e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.8067014019673433e+00,     8.2550950952703770e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4825458152488956e+01,    -1.1315492400726989e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    6.8646391065658193e+01,     7.9392732509653806e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4313190531926434e+00,    -9.9656893745117140e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.9152187286800466e+01,     1.6799369460072881e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8146397200566831e+01,     2.3876376746862839e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9398380221312181e+01,     4.0531129763793550e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -4.0550443481961615e+01,     1.8445402803822035e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.8407455726482691e+01,    -3.1151933833948351e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.3889950941253169e+01,     6.1565222047568195e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1751489930551941e+01,     7.8086077734483415e+03,    -1.9032282042808564e-04,    -1.5153704875157295e-07,     5.9967089030119600e-10, } , 
      {   -7.2480731494958377e+00,     7.2052027098178014e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.9614016552528231e+01,     7.2229230208988874e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1920800660871675e+00,     8.6687527728441564e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.7241020275623029e-02,     7.2386755892333372e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0235073331891478e+01,     2.2129220915761474e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.0718157557287459e+01,     2.2739896474532547e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    4.5064262699399382e+00,     1.2613092753598843e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5994227209017907e+01,     2.3752001008911509e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0924167055577543e+01,     7.7886900077258197e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2147978766570439e+01,     1.4922073288562522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0675650297692261e+01,     2.1280334481036174e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3951412516947656e+01,     1.5824441571207499e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    7.3249156285831887e+01,     8.3313449405229348e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6187357637004061e+00,    -9.5131498277473147e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4997372712527559e+01,     1.1867253920723017e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.3763577059611265e+00,    -8.4022877539765232e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.3756769833573927e+01,    -8.5555138871254203e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.8061153479704338e+01,     1.5074482259218094e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2257178351946010e+01,     2.3696644078725163e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.7314360987591755e+01,     9.3862320852249759e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1893579695011233e+01,     6.5233984732826448e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1076067858891962e+01,    -1.3499151746176613e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1642472047049052e+01,     7.6304940615591222e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4621788371724705e+01,     2.5008616179002594e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0926843915486444e+01,     2.2287416861928174e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.1133051650552055e+02,    -8.3304223931006873e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2349826610896005e+02,     7.2157938338355834e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4591142374932424e+00,     1.5489073847303727e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1982620195829895e+01,     1.4314168059711651e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6773812001964039e-01,    -3.6485224157041712e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.6458221984352672e+00,     6.6659373736411080e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4457870145190377e+00,     1.5682439700866367e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.1685294265625501e+00,     1.5567337453782728e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5219752825886088e+00,     9.9487686937041217e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4365969227869655e+01,     7.3507805678177492e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0968718536159692e+00,     1.0577076278749664e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.4189309932675132e-01,     6.0273374732860430e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7348433855568906e+01,     1.9191276705685967e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1657273493049527e+01,     9.6078484998467793e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0899765759292222e+01,    -1.8749333490348263e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.4241495686747964e+01,     3.9413847688628957e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    6.6823582721952235e+01,     3.9414552320545117e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.1214209268783389e+01,     1.4198661082598073e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5278539705112976e+01,     8.3267270769539718e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.5944160874534035e+00,     4.7139504363744391e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6459178010518993e+00,     7.1158470880893274e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.0655224954992772e+01,     2.2467921296906388e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.4977573083941429e+01,     2.3796194653534534e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6695433568484592e+01,     1.6083147888806756e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0430086928746404e+01,    -8.4995436901621470e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.4735762075266479e+00,     3.2838531684284699e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.1254500248129176e+01,     6.8366451523188516e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5056023029754499e+01,     2.3636159099252691e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7944303235044632e+01,     1.6408978322696079e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6457566257804448e+00,    -1.3639929268993194e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3514600429864869e+01,     3.7990305849472043e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7112381869069605e+01,     2.2128509570294598e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.0189919517889523e+02,     1.5541023321504554e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0404883903498767e+01,     1.9503719748010637e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8935142133348544e+00,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3938407400255848e+00,     8.2867429541346228e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8980756604664911e+01,     7.2119206271365247e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.2481181488237523e+01,     1.3834763034680538e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.2476209847410281e+01,     1.5019673553891758e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2356731433664310e+00,     7.0727875999992402e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7367861986725899e+01,    -1.6641650365247599e+04,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    2.2715042807142247e+01,     1.1308565872227784e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6953713769734634e+01,    -1.0031812667036185e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1768382203482528e+01,     2.3970772509475522e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.4405894199129762e+01,     1.3498673398216670e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.8202317918963180e+00,     6.6490336587794800e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5607239882381375e+01,     6.0530488997352422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4911193483529814e-01,     2.3862809216222413e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.4579019151545456e+01,     7.7153401727160199e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9323897415804225e+01,     3.2357871493569128e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.7917122269055561e+00,    -1.7403063119944645e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5973436029733641e+01,    -4.9310322575947794e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0026108889717218e+00,     1.7442270802817551e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    4.6008219859311303e+00,     8.3706398951122774e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8097254211428869e+01,     1.1614845792993332e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3525884947696243e-01,     4.3853638164953081e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0500591002867623e+01,     3.1569528354241738e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.4674097990546207e+01,     6.9012239889936654e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8674831211663694e+01,     1.4909515672678655e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.4356648379946513e+01,     8.3271661713722315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5351065046734611e+01,     8.3302166778746669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7835085443480857e+01,     1.5541690628556844e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7746492411837302e+01,     1.7363687321184869e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6214535418459704e+01,     1.5701661581615368e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8651843483477801e+01,     3.0652922358697197e+01,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -3.2537577368227097e+00,     9.0193558166991497e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2355376153539289e+01,     1.5553571001238650e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.4914327499803576e+01,     1.5543817951406745e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9571609426291666e+01,     6.9769044224481095e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2732958377706638e+01,     1.7285690434292443e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8894150542165633e+01,     1.7738170046896819e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4953918042827787e+01,     1.7654780494428928e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6685726924219143e+01,     1.4617026939711250e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2507462794699975e+01,     3.0536830798713869e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.0992323276905864e+00,    -1.3268078573175899e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.2952377490907052e+00,     1.5976106166990166e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3152238441069153e+02,     8.3282410570915345e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4251667881864472e+02,     8.3291417921553657e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6213988322554833e+00,     8.0907749769547099e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4582971857927646e+01,     2.3801555936586461e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5465639662309322e+01,    -1.3215481926852235e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.5454160926586368e+01,     7.7024613823616073e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0022699808639182e+00,     7.4624673919091674e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0271815702258404e+01,     7.2129992039333938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5714615568954864e+00,     1.6071735080157061e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.0029898751807522e+01,    -1.6672801960386576e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    9.3297342837873920e-01,     1.4901827466518265e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.4049343956248553e+01,     5.0754911785536779e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.3728409141809221e+00,    -5.2355723250848860e+02,    -2.1620064077522566e-04,    -2.5610772366429155e-07,     1.2058285876556377e-09, } , 
      {    1.5239293395182624e+01,     2.4533504716943371e+04,     5.2628705194601375e-05,     2.7700527801894134e-07,    -1.5245887348483466e-09, } , 
      {    3.0044266853581274e+01,     3.9380442957462976e+04,     5.0742700465641588e-05,     4.1140202996309535e-07,    -2.2258765727101013e-09, } , 
      {    6.9659320159481970e+00,     1.7461477223073198e+04,    -9.9859567294893075e-05,     2.6373670326886312e-08,    -2.8715029518436709e-10, } , 
      {    1.4450176979839913e+01,     1.4427414385249813e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.2640155746549496e+01,     7.2151265267832960e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1432340439521291e+00,     6.4967377666980119e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3767006221189018e+00,    -8.3985812026422318e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.4583958486722274e+01,     2.3889494658061107e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.1007374159707890e+00,     1.1646326020507080e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.7658814474934481e+01,     8.7181095982815204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9044205862300466e+01,     1.4907887749337118e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.3997153807068088e+01,     1.4428837076183565e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6214668614932652e+01,     8.5379122579849791e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    3.5959378219383247e+01,     2.8980718320614083e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.2125337448052633e+00,     1.3657831534845165e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5152585326578766e+01,     2.3960463228220033e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3262789775272687e+01,     1.4173694291040785e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8133985893170532e+01,     1.0862585281062517e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4275514174396617e+00,     8.2750109819453828e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3883376166114305e+01,     6.1170663323677081e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9292341308995846e+00,    -1.5852017899702171e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1329453741278890e+00,     5.6431786111198778e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5822992218974219e+01,     6.4092682346353013e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -5.1702515646646205e+01,     1.8405587901924126e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0066980235896885e+00,     7.9291209750702401e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8131097793668843e+01,     8.5385695227345241e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    9.5446948291137339e+00,     8.3823718673015173e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.4338707316210701e+01,     9.4087739903572183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8469627931372727e+01,     1.5596434732659862e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7612856575096899e+01,     1.7179272518567603e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.4172728920510309e+00,     1.5314026630228374e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.2382036857195470e+01,     1.5543850170849282e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.9459089361001475e+01,     1.5541658409114307e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5699676940927283e+00,     2.3011248595016041e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5168551473236118e+01,     2.6735945268387886e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7015912202703610e+01,     2.2756105809873265e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.6684566813359517e+01,     7.5567606072129065e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6233010902392483e+01,     1.5378460980042231e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.0444726980738633e+01,     9.8622799395226848e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5450842207816535e+00,     1.6473911345290213e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.0734539133209232e+01,     1.5542902707039008e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3250882948234981e+01,    -1.2843631231034949e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.0706010133531514e+01,     6.4663105746190649e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8542471928979651e-01,     1.4475025069370116e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.5609337608753844e+00,     4.6400665873147773e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.2251806847446858e+01,     1.4932495648392112e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3452885988724645e+00,     1.5542605872924581e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1846190141258898e+01,     3.5660314860397193e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.9967005053714942e+01,     1.4507755534512513e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0428160622966176e+02,     7.2123318968811072e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.0131629294150169e+02,     6.6100281539222833e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1969475253735826e+00,     6.3861686630103100e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6571870820699470e+01,     2.4340846008017212e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7997369302003332e+01,     2.9906445670884630e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3912297847901534e+01,     7.7905553390080167e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1625553506881744e+01,    -1.5747743514451031e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3497936310860776e+00,     7.8166313152557777e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4714806787521670e+00,     1.0873985972020246e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9871425862314695e+01,     7.8887711162770356e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.4624640464735844e+01,     2.8155811376647689e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3800789700295358e+01,     7.7901126986595727e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2820957317251100e+01,     1.0094169474066342e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0393974765683399e+01,     1.6266621848905130e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5884000204846398e+00,     7.8056106885334129e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3281528497034166e+01,     1.0683014473172048e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1823288399314578e+01,     1.4371458751802114e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1417914011823047e+00,     1.6674243595894466e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.9206704887154817e+00,     1.4939029868737185e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.5522894545120511e-01,     5.9743683760748518e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3974650406874169e+01,     1.4151152385908541e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6132714618748459e+01,     2.2757528500807017e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4706115174190046e+01,     6.8003718180907927e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7155562603669896e+01,     1.7897077338530391e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3198172292878418e+01,     1.6641635105732450e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9734308437134843e+01,     2.0181453665285520e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1150845900352152e+01,     1.4525028531278182e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1345682834679433e+01,     2.5508443664493509e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2995775193427322e+01,     9.0136659885835670e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3980695815289288e+01,     6.8176448148564605e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3072905909268897e+01,     3.1068587899155995e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -2.1383571818785981e+01,     4.6827203076370097e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7376539271342651e+01,     1.5901993832219623e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.1900002356742995e+01,     3.1163060840975715e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5576816477905313e+01,     9.9989910806644184e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.2383034579496211e+01,     8.3116241430838018e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6552284230455072e+00,     6.6452738573842162e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0754836263763570e+01,     6.5967759070351649e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8705488975214912e+00,     3.9609654580288402e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7585944466721767e+01,     1.5507809400972403e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.1054933399404923e+01,     7.2025728775016178e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.3420876802437327e+01,     7.2255528532150729e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.0148792077329567e+01,     2.0729579157002758e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6508685590803029e+01,     1.7279131478693311e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.5088835775959639e+00,    -7.7353287285872993e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -6.3914868733715302e+01,     9.7428352338289460e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4339516693822635e+01,     7.7544564641833049e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -3.1389016976669506e+01,     8.3457587061630966e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4745548655461445e+00,     1.6673382357470899e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.5958836381632729e+01,     8.3250886285885317e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9783135198970663e+00,     1.6080890375597228e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.2762846355180946e+00,     1.5294848436211250e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    6.6953199441777500e+01,     8.3322942206583684e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9989258912472003e+01,     8.6126799057905664e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.7612530357999407e+00,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9329052054938394e+01,     8.2396491757545991e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.7158775034616749e+00,     7.2206742649104490e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4171826670508167e+01,     2.7697334480814698e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5347637540847940e+01,     1.4012316132332508e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2460932305621593e+00,    -1.0798150931248660e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.2403769448424100e+01,    -1.8048943455862172e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0278950635328998e+01,     1.8017609290651904e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.0273080017443309e+01,     1.5547163517152054e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4390335019451488e+01,     1.6406348490379893e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1082723827065157e+00,    -9.4367610282894238e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -4.0471069150688777e+01,     8.3245726231710596e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6548844437656465e+00,     5.2827394443485682e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3604565228117935e+00,     7.4121376027530705e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.3734112314928240e+00,     8.5666078722921902e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2319734796297228e+01,    -6.8679412343187851e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.3604642661708688e+00,     8.7397460782571488e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.5484636942515504e+00,     1.4208586858878585e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4366779185486241e+01,     2.3240508297243516e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4334576236553810e+01,     7.5295315740392089e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6439532687722267e+01,     3.9021179034894049e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.5522509242610163e+01,     2.4985745641495578e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.2375080353320929e+00,     7.7357209105146285e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3276522366561064e+01,     1.6117381432349270e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4271477025003605e+01,     2.4333768792286392e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2216056832029794e+01,     9.1945455943727648e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9569363972330372e+01,     1.1943432412872462e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6301029895063607e+01,    -5.6113260642276982e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7921800237131066e+01,     1.6931366664797057e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.7553056431586697e+01,     2.4397581590722719e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.5106115188195318e+01,     2.4986402906245126e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -3.1117453086057324e+01,    -8.3290102282749540e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.3908851426053435e+00,     7.6654389505685149e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7171970309337752e+01,     8.3263035916483004e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.6177770445476384e+01,     8.3310792575986015e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4644359134127747e+00,     9.3242828297457254e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.0226455756834000e+01,     8.3282269628361428e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8625977581490954e+01,     1.2165601117026752e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9652329765992818e+01,     1.7934661535594882e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5155543974317874e+01,    -7.4782151354236166e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.9547944575701127e+01,     8.3283726209719462e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.0325538550751929e+00,     1.7540121873968346e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.1005265906531392e+01,     2.5994284917410616e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1557744415858643e+01,    -5.3813608561543219e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.6484913406985733e+00,     2.2754739612556437e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.0693923112059132e+00,     1.6797068260131469e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.5229069102869666e+00,     1.1253305050814093e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.1386574922888233e+00,     1.6759554558174968e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2174947332103581e+01,     7.7783807264703328e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9598617447607175e+01,     2.5601263947717627e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.4568172935236774e-01,     5.8062722965259600e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8656825349623986e+01,    -8.3291558864107574e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.5660913835102665e+01,     8.2500280232286896e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.1233545310492701e+01,     7.0717239385742896e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1511393908758777e+01,     2.2758894698123844e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.4679472746913529e+00,     8.0158525482587711e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0906834612139100e+01,    -8.4177336734923010e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3293733941816782e+01,     1.3249062821637726e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.0343822448855349e+00,    -1.0803546425293562e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.2366814105960863e+01,    -8.2483293269717251e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6779973184790627e+01,     2.2542200617562150e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.3361643251807564e+01,     1.5089248340816330e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2477240782554052e+01,     8.2330902201554654e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1608860842231277e+01,     8.3347517074423049e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    3.7828818037161561e+01,     8.3288409106041308e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.6259301589977760e+01,    -8.3285419386427693e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -5.1081920005049639e+00,     1.4896397761480392e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8588301321980126e+01,     2.3707152404665681e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.3917370277518657e+00,     1.7259953284676190e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.6680054600371799e+00,     2.4435318949836928e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.4443498400647975e+01,     2.2462570813766324e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.1706771281085899e+00,     1.5516093583397162e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3442633750390883e+01,     1.6657376135696177e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6448032060701451e+01,     1.0653035815326213e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.7737205616425868e+01,     1.6657389562797624e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4909518231662108e+01,     1.1175028323387183e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8473852245765961e+01,     6.8390908391212179e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5471627987462590e+01,     5.4685121482466657e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -7.8850427517279442e-02,     6.9661570115878012e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -7.5633019376100805e+01,     6.2831429859626996e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.1871531810607729e+00,     9.0606102809829117e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.0012428513026014e+01,     4.3209294666814512e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6612057054333363e+01,     3.0666154965317764e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2073986823464949e+01,     2.6398130993836276e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.0823539783546998e+00,     1.1926254461575309e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1519402828686761e+01,     7.1874021587737116e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4485913466706251e+01,     7.8821853523017999e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1815424439295063e+01,     8.2801315545825182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9074874617899940e+01,     1.1459741270969083e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2105369142390195e+01,     2.1673355450729163e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.5623779759130407e-01,     6.5393546144396551e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.3309014088353194e+01,     8.2734306032097575e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    7.9478949700437180e+00,     7.5269017417230243e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.2345174027091641e+00,    -7.2117958764099076e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1911095049665471e+01,     1.5016618413864320e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2156233020176639e+01,    -7.7173045203854990e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2255676499870143e+01,     1.2649141170002442e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8771009065496933e+01,     1.9584297675378955e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.4769927015199187e+00,    -9.6010543382289852e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.0585928101085766e+00,     1.6724896943495027e+04,     2.4295152562268702e-04,     4.2854232677051423e-07,    -2.1242596251495428e-09, } , 
      {   -7.5052175214296515e+00,    -1.6589868754998774e+04,    -3.6621810542661771e-04,    -5.7433316984810465e-07,     2.8196763693330605e-09, } , 
      {    2.2537761249493048e+01,     3.9447957051711099e+04,    -1.0890589436323744e-05,     3.3850660842430015e-07,    -1.8781682006183425e-09, } , 
      {    6.4863414639783405e+00,    -6.6211902266328343e+02,     2.8348628185242411e-05,     3.6976836420960540e-08,    -2.2974835534099799e-10, } , 
      {    1.6993093238606438e+01,     1.4951682963225179e+04,    -2.2075052699709397e-04,    -1.2109457960597486e-07,     4.5010102154209299e-10, } , 
      {    1.0736873431342513e+00,     6.1992932662189469e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7030634579802189e+01,     6.4460698694282310e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6403907271851651e+01,     7.8977273805902296e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2579640252826829e+01,     1.4190579146012918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3760322350657113e+01,     2.4380308593957052e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1460362064935032e+01,     8.8728877930577346e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5494255249743969e+01,     1.4918259996749388e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2508824007822708e+01,     1.6655240611025081e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    6.2281984598652915e+00,    -1.6659525087468719e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.0492103236803723e+01,    -1.6670500760445168e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    4.8325820621171964e+01,    -1.2894301951451844e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0839255377965209e+01,     1.7861028922126792e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0767697370906925e+01,    -7.8892697890573163e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.2274515049995937e+01,     1.2809656445586746e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6710788367506680e+01,     2.4047213752886382e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.6157610752647486e+01,     1.0609264981769251e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9177481332425462e+01,     1.5551390788364630e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.4566313156583863e+00,     2.2534648451185851e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5683892904745987e+01,     1.2619238417987162e+01,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.3577110270762693e+00,     7.5944456451555243e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7824400218408172e+01,     2.4582236592920504e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2214300366937007e+01,     7.8397406180877033e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0354845026127959e+01,     1.4434025784801917e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2850897186329338e+00,    -7.0585984605766726e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9396768638652655e-01,     8.3249249391291360e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5852990861429577e+00,     8.9420426765308785e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5637515686197126e+01,     9.6640240106353303e-01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.2330879205365912e+00,     6.2522528876912284e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.1951384750117278e+01,     1.6170733242652566e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0941401199389436e+00,     1.4676530257267512e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3276274010774722e+01,    -9.3115705530841967e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.4882172973362444e+01,    -8.3839522460371427e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -1.1633631419480437e+01,     9.2882002023491568e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.1346604007151591e+00,    -4.2680239714814797e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.4053383669940551e+00,    -1.7827510462085611e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.6294850361048404e+01,     2.1874078130618691e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.0902189747416031e+00,    -4.0415433853971595e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6158041505630504e+01,     1.6118804123283022e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4521952581747058e+01,    -5.5766659559062771e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.1343319785177357e+00,     1.7243030618336739e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.6490485047493422e+01,    -5.5696196367447794e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4624699113524782e+01,     2.2198399348313378e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1987834889632024e+01,     9.5011953781032789e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2710670063321238e+01,     8.6218534341907289e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0127156975703473e+01,     6.6909821292683073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.7931402079502483e+00,    -8.3341211296613219e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    2.8110650117638336e+01,     1.5010047909404235e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.8956306818207196e+00,     2.8908321987354811e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8035218668857034e+01,     2.3165585868547580e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.2492426096699035e+01,     7.2142685805843648e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3669881337634056e+01,     2.2131139402610781e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.4642717457339165e+00,     8.0656234865647129e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.5354018501295990e+00,     2.2769935066538404e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.8843203442253433e+01,     4.2923307555598194e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3722763153179137e+01,    -8.3449007599641664e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.2004676993062635e+01,     1.8994785216987908e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4560148250952981e+01,     1.5383846998348221e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5005548157931168e+00,     3.9923428468023203e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0169647696089864e+01,     5.3931298341552701e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4643839791800929e+01,     6.9274193485939986e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6813120200553108e+01,     8.3284061966332702e+03,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.0171664640649553e+01,     9.7894416149281151e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.0201805762830141e+00,     2.3305494936589861e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.4628151258129147e+00,     1.3419915087176425e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8382636647741002e+01,    -8.3289766526136300e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7663206604564543e+01,     1.9801550606790799e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0479713809843105e+01,     7.1779802379617740e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    4.1864999705185308e+00,     1.1190377864353643e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2712025422546469e+01,     1.5577699178991188e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.2427281428099313e+01,     2.6387305887103601e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0404365652623143e+01,    -1.0344464659367977e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7248383788525629e+01,     1.0339477931565175e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.2101888188983025e+00,     3.1005878776167763e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.5515868665813315e+01,     7.7246567131873890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1472821674399711e+01,     7.9176215114745992e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3889581293950670e+01,     7.2407235719429773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.8491468955353110e+00,     6.2058522218180806e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1686972260812100e+01,     2.2088694668396685e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.9489088032262600e+00,    -9.8106038039652885e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.9520065314334005e+01,     1.2007866762686321e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3714439549503581e+00,    -1.0988808157506548e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0861837428516150e+01,     1.5530302770504486e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    6.5442113336221812e+00,     6.1151820496076889e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9792375534231809e+01,     6.5933818639390720e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2411061245622076e+01,     2.2778147063317992e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.5451041831872558e+00,    -1.9988862759200979e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6252141653383561e+01,     1.5569414996566427e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7962963599468793e+00,     2.3853396771149382e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.9450886200190567e+00,     1.0860284081121106e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.6494758615434684e+00,     8.8407515339911224e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9961165688121753e+01,     8.6129300045668788e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5046344471193088e+01,     1.7188787633730950e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0761168656187392e+00,    -9.5415357611575746e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0280388899357170e+01,    -7.8117175280451920e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.2189134710930979e+01,     1.5625237583251512e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2897705120550484e+02,     1.5544485258459034e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.7493979079990595e+01,     2.2774090152105808e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.6984021902004272e+01,     1.5541942244341903e+04,     9.2743731943700382e-05,     1.0921223261429318e-07,    -5.2374421970263164e-10, } , 
      {   -4.9906969398198257e+01,     1.5543566335621690e+04,    -2.2511008419136763e-04,     1.5023212424749856e-08,    -2.9229416834093930e-10, } , 
      {    1.6757688873688885e+01,     1.3821148053648998e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.2156975262554495e+00,     7.8231827806478577e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2341098079118987e+01,     8.7681130601895838e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.6106367375776095e+01,     9.9020948414205959e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1271355120853823e+01,     1.4402386595568578e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2451724111192121e+01,     8.2404251224447908e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2318210013603860e+01,     3.1149942929777444e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    5.1268912773280817e+01,     2.2247954275988333e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.7140384155964732e+01,     2.2774866098796003e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9582733661618423e+00,     8.4059769660489983e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2490635090473852e+00,     7.7636622180445856e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.1667031609997579e+01,     1.0371142818874490e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1571319171084639e+01,     2.1122600126695743e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.9503858081812339e+01,     1.7483033975304766e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8856796460260945e+00,     2.5574566957017978e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -7.1041599772433855e+01,     9.0747127468854906e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9451739595625508e+00,     1.4283509287574525e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.5115562844948030e+01,     1.0014539670270518e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1144591535537019e+01,     1.3679161442823017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2858686114770919e+01,    -1.9177377801907278e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.1172685693920164e+00,    -1.7091734673551027e+03,    -3.9892075510656011e-04,    -4.7638412867334818e-07,     2.2922427175171852e-09, } , 
      {    2.7731836943853445e+01,     3.0423449577625466e+04,    -9.8885825803776082e-05,     1.6006676369427795e-07,    -9.3545284583766085e-10, } , 
      {    1.7232864254514766e+01,     2.4013481086093419e+04,    -2.8485462473716171e-04,    -1.2639616265301197e-07,     4.2140005162040839e-10, } , 
      {    4.4251605407603627e+01,     1.6482581213366822e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7405846990956928e+02,     8.3146769547129788e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6620576815520590e+02,    -8.3427058945339213e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.0651697015737915e+02,     7.2137172629509951e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2888291356041157e+02,     7.2144084677656929e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3915864274053451e+00,     1.6440129917835053e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4627442033756729e+01,    -2.2916139255603675e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.0833691131096499e+00,    -9.9940753079219721e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1370789544130929e+01,    -1.1020093208471185e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    1.7677077449950680e+01,     8.3865668188381751e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -4.9581212624557061e+01,     1.5932172463639865e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8208818354404439e+01,     1.7500306972070437e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1363531660662527e+01,     8.5257893708824431e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4528994760304315e+01,     1.6107072151093782e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0958737662153748e+00,     7.7114044663002705e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3081812459404981e+01,     2.2039492056679806e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.7128144770875053e+01,     2.4721922003805077e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.9553667539423444e+01,     1.4919377737193841e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6549915749974229e+00,    -8.5720223178149117e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    4.1998126138020950e+00,    -9.4095385564333919e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    4.9122347661976047e+00,    -1.9186985527603072e+03,     3.3578458807475305e-05,     3.5787685423935058e-08,    -1.2014167831575411e-10, } , 
      {    2.5978765308504265e+01,     1.6157943963829071e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.9144070897792993e+00,    -9.7058385468360484e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.4238712875212940e+00,     8.8517721607134863e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.0513994509174665e+01,     1.0410957720772401e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3517759582654438e+00,     2.3858826476187256e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.0950633126920591e+01,     2.6136751249539051e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.2807770172835660e+01,     8.2162465957735276e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2969729846082334e+00,    -3.0797334077300429e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.1824857735310047e+01,     8.3263919596794276e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1164701436890457e+01,     2.0574474634978505e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8081098478195639e+00,    -8.6753117284432901e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.3810717100489985e+01,     6.5943917786956354e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1257216025317859e+01,    -8.4389140954894738e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.1129738844492270e+00,    -8.3353028241755528e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.6797933693830736e+00,     4.9656780112321285e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0725273397210074e+01,     7.0167193243460215e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5410871771323205e+01,     8.7059646508019869e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.2701427950329283e+00,     8.4646445610066803e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.7097429701725302e+00,    -8.1982336684113557e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4476324213688603e+01,     1.6735379736139326e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.3418625390966298e+01,     2.3736458980623080e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.8412216728752313e+01,     1.0348600913742246e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0631253675248885e+02,     8.3299610139815522e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3490973830966709e+01,     1.5526544954641078e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6500123334942824e+01,     1.0666093792364454e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0730868856547577e+01,     2.2735487247362289e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.8927758716239635e+00,    -9.5999003524136660e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.0706452511168926e+01,     9.0279923150819832e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6893862872858850e+01,     5.8615331179396508e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.1338476134623292e+00,     8.1305833175349526e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4312893257033178e+01,     2.2528089495586715e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.4735130759012835e+01,     7.7692996022071966e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6707556799706964e+01,    -1.2881296085978092e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {   -2.6163321182812460e+01,    -1.2340732649588424e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.3912375209382763e+01,     1.4914805734445052e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2565438775459199e+01,     6.6428431789763818e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0360154162152725e+01,     8.3297960163527678e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7381612906152199e+01,     2.9373739290307072e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.3658918704741012e+00,     8.3275868328941324e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1415310827010117e+01,     7.7734546877745988e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.5006734581481609e+01,     1.8739649145395852e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.6156899348050082e+01,     1.8699834243497942e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.8548947103861636e+01,     1.6832184485126978e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.3672304564039628e+00,     1.0467263111428118e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5460062638085970e+01,     3.0996491066348804e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2638252687176683e+01,     7.9820711208036100e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.4796053523230569e+01,     3.1085515293514312e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1287574719047356e+01,     1.6171056245195854e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {    1.0000732809216879e+01,     1.4441243641914956e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8816617214168279e+00,    -1.0175238840248017e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -8.4445085051680536e+01,     8.9327261360595112e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0233556049653201e-01,     8.9729985178929910e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5693985709989835e+01,     2.0403559095356504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0211489516784571e+01,     9.6812211720421437e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4194932548208261e+01,     1.6894929433950674e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0266697589435202e+02,     3.1085501866412866e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.2385436080966471e+01,     1.5449454285455220e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1705729161906699e+01,     3.3297845017686210e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.5111590309033211e+01,     1.5460270996712077e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.9531853172736021e+01,     2.4500105615608860e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9845984498981386e+02,    -8.3274218352653461e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.2856001764186782e+01,     1.3968541088457096e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2082162090546134e+01,     2.6172799665942654e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.7459771763127279e+00,     5.8538364239533385e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.6254111089044887e+00,     1.0217170892806848e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.1403108050220681e+01,     7.1653609995437891e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.8551201486724509e+01,     2.3167887068488992e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6469405903012220e+00,     1.3816027481011864e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4338277519510793e+01,     2.1184733276568193e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.3329421248427963e+01,     2.2756719103766300e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1648923423924597e+02,     2.2756915206913982e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.5519975469735300e+01,     7.7159258678663373e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7675038527076481e+00,     5.4132513268329731e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.9343067110976349e+01,     1.6657741878755704e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5048482096348870e+01,     1.6657023819738097e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.8436455120256205e+01,     2.3884064953023233e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {    3.0952891677759496e+01,     1.5796089848790210e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2556278848636033e+00,     2.2872765975342213e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.5047894700995364e+01,     1.5854273628321278e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8046734141306569e+01,     3.4596364662247634e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.8680332522179555e+00,     2.3163508325763876e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8326896761907832e+01,     1.4463070619726082e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    7.5197599242642328e+00,    -9.0775309617905059e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.3133706269932626e+00,     7.6208233739241141e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2951956653745640e+00,     9.1267335735073830e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -4.8896321675941564e+01,    -9.6902800561492768e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0385481796054975e+01,     8.5485589288363281e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2819082255497591e+01,     8.3309908895674725e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.7893604788558619e+00,     1.0074242141735131e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.7183612709830008e+01,     3.2187019228030425e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2911850014270126e+01,     9.4271106485478376e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7162276954355143e+01,     2.4499041954183911e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.7855385238747928e+01,     2.3990890420298980e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0994335394552174e+01,     1.2336453382565449e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.3952782056665214e+01,     1.7777280316878579e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.3490183966295239e+00,    -8.3324579101177624e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.0010726192655603e+00,     7.2265143848356538e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2889221908282710e+01,     1.6224030972238528e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6073205914335929e+01,     1.4286132406339982e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6835243244190991e+01,     1.5540805405998459e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4180925961501387e-01,     1.5544703173965130e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5620555805545138e+00,     6.6103384441137341e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9819259341812938e+01,     3.2200137139228696e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.0308516132955031e+01,     2.3314131434972693e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.3849625769882035e+01,     2.6780326856796586e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.8105533407039118e+00,     3.3348522745617869e+04,     5.7835298609832211e-04,     9.6642778584922133e-07,    -4.7700818084367239e-09, } , 
      {   -6.8922900597348642e+00,    -3.3281008651369732e+04,    -6.3998627600028734e-04,    -1.0393232073880164e-06,     5.1177901805284827e-09, } , 
      {    1.3249931336594525e+01,     6.9707563524979814e+03,     2.4493820734244241e-04,     3.5997474000024776e-07,    -1.7590009978016184e-09, } , 
      {    2.0225333608693255e+01,     3.0490963671873589e+04,    -1.6051911570574143e-04,     8.7171342155482774e-08,    -5.8774447374590161e-10, } , 
      {    2.6441538290184097e+00,     1.6900689362107263e+04,    -1.5882897570394989e-04,    -4.7138143326071518e-08,     1.1499780515918190e-10, } , 
      {    6.7025903597136889e+00,     7.0720274938701696e+03,     1.5248827248949445e-04,     2.5063160769205500e-07,    -1.2374384396639795e-09, } , 
      {   -1.1434742786168149e+01,     3.9279084969884639e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4138071397126946e+01,     1.1701586841920773e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3492371087267042e+01,     3.0328340690766154e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.8721405930033498e+01,     8.3285524487849743e+03,     2.7595071702059336e-04,     3.9633518430704559e-07,    -1.9343096248908262e-09, } , 
      {   -2.6574107684396893e+01,    -8.3288304004619240e+03,    -2.8634098504059006e-05,    -1.0510256400226382e-07,     5.3765837235047523e-10, } , 
      {    5.9266396891573351e+01,     2.3314483750930765e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    6.7907617015793491e+00,     1.3264810565152176e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1071820708929501e+01,     1.8327682505287870e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.2150256549593497e+00,     1.5555205809459105e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5414539171468342e+00,     1.5548654344067023e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2243372490617013e+00,    -9.4459498162047366e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.7309506383564468e+00,    -9.6554992819410418e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.1792403411582232e+01,     2.3313779119014620e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.1254578503868620e+02,     2.3870937755966126e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -1.8075762343410659e+02,     2.3871953673244363e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.9266195933867269e+00,     1.5573932693496874e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.2236214572094397e+00,     8.3232617195855764e+03,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.3464496071111183e+01,     2.2636160452492011e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2775706015769341e+01,     2.2286243924635030e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.7973075994200300e+01,     3.0831077140287685e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3123416796712483e+01,    -9.6502396173086745e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1847927379453299e+01,     8.8456653212017100e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9556489363394576e+01,     1.1255606250755505e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.1353774755082338e+00,     1.4082004099677129e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7723531188849662e+01,    -5.2183050945347113e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7392296262756219e+01,     1.7192390429765866e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9064913002530563e+01,     1.4821146700409678e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.3473269157535572e+00,     8.3364063515697617e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0691353117481714e+00,     7.4896898637651939e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.4845819599402446e+01,     8.3436118629828488e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.7161209260684078e+00,    -1.1272477976830928e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.7262466864264077e+01,    -1.6595249699374450e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -8.6773982887906911e+00,     1.6171390507402110e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1719934106426699e+01,    -9.4406901515723694e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.4352582311341294e+01,     1.3856906044334726e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0948882241262128e-01,     7.0720762545323632e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.7439155547428129e+00,    -9.4771014113437141e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {   -2.4046794121330070e+01,    -6.0576567991329921e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.4944617822061463e+01,     1.8190971364146135e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5631677910019137e+00,     1.6727212693105659e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.6766557426348299e+01,     8.3881947421797140e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9344021706499255e+01,     2.4186914423286107e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.3311984202869276e+00,     1.5009364272636270e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.1241654568995969e+00,     1.5506671662585224e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    7.6308680247323188e+00,     8.3926811485570051e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8854108045654812e+00,    -1.6883852995790799e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.6842863644624611e+01,     9.0366288134648166e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8799784656239623e+01,     9.7030203319310349e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9924340298980141e+01,     2.3836500825595853e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    6.9348581132298559e+00,     7.1573958864437709e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.4725089255802061e+01,     1.6656678217330747e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0759330880416303e+02,     1.6655651880769663e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0430466193073073e+01,     1.6658087481163053e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3536610522047866e+01,     2.2066376420422152e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4566833799734876e+01,     2.4493194344051652e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.0192767320958652e+01,     1.9977318645071944e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0043909984475266e+02,     7.7004818911517468e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8415788590370916e+01,    -1.0241362132772489e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.9984887674017280e+00,     8.5820117239167157e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.7187851677896212e+01,     1.4936487958380980e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4474178473113067e+01,     9.6523707768445802e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0058942590901196e+01,     1.2558622086719737e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1672050553421895e+00,     6.2516940368410733e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6742337482656477e+00,     1.7652974254369179e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.2745036559790119e+01,     2.5536829597903768e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.2728962749201731e+00,     5.9176327933693456e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8044706675363244e+01,     7.1969955838186970e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3272076338625425e-01,     4.8897184746555813e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.1393436655127109e+00,     9.6013061029385335e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8670969692502652e+01,     1.4269687628495058e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2997260677313633e+01,     8.4169975927713640e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.0199149652019539e+00,     1.6517663227369358e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.4106907475529539e+01,     7.7080104232041776e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2396476818816108e+01,     1.6850011634175607e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.6387431736859931e+01,     8.3860697490049006e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0404739963575217e+01,     1.5525687008442148e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4025928690024374e+01,     8.9776625259258872e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.9785630757745452e+01,     7.0724285704904387e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3709810661282500e+00,     7.7500472370130456e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0488554215700503e+01,     2.4046247350485326e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3318089244781127e+01,     5.9600775275834421e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1886578632110655e+01,     7.3230953538842186e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6714599598576324e+01,     8.3401566772261158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2496046541588015e+01,     1.4901328793737983e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.6174270211788317e+00,     7.9946301811517224e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6314305748992794e+01,    -3.7387614536963997e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2331104679221227e+01,    -8.1552245862837190e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.3610143914551665e+00,     8.2647017006898950e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0337299704370231e+01,     3.2122584878216574e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.1665893121900428e+00,     1.5531733663259431e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.6681433813109038e+01,     1.7256631475449803e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1349110539909516e+01,     8.3368584676571172e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.4964087855461172e+00,    -8.3205243815897848e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.1625939526056897e+00,     2.6298323161836379e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6824807541325441e+01,     8.2898429251266007e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.8297541894461000e+00,     9.4160900218254756e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9782252998760285e+01,     6.6038042237686641e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6811982402863414e+01,    -1.5925960759164900e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7374016730677194e+01,     2.4564187649464649e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.7030767597804310e+01,     4.0564876980255736e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.8095164707245708e+01,     1.6732633910317611e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -7.9115447093184912e+00,     8.6261111898484523e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.8688417689955756e+01,    -6.2830087149481687e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7536539687152608e+01,     1.0546720738341641e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8316094492716829e+01,     6.2545540876326377e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.9702049130470831e+00,     2.2003914638174929e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0917680196745039e+02,    -1.7309684772390028e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5398829809804759e+00,     1.6657159192089188e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.4165520527746320e+01,     1.6657606506404609e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.5761511754034183e+00,     9.0889962143931698e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.4526459124658729e+01,     8.2173577361561656e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.7370516336762265e+01,    -3.1776990862227649e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.5126611405949255e+00,    -4.4345643550901468e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7392441349845429e+00,    -1.6797102471124443e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.3994604044383010e+01,     9.1170345639508359e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5201271218260480e+01,     1.5578836917378369e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.2729856351773847e+00,    -8.4758840833295944e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.2586285533246262e+01,     1.4301095453028822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3336022517026276e+00,     7.3445747478427493e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.8874732662190130e+00,     1.6029080894032839e+04,     3.0724869701756089e-04,     5.0082135619514676e-07,    -2.4175282689895120e-09, } , 
      {    4.7076955792726032e+01,     7.2032461541014873e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    9.4668455968850253e+01,     2.3243131416008975e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.5680413240612339e+01,     7.2311301468979909e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0257659835268248e+02,     2.2741275127051711e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.7531341411682739e+01,     1.3522844800862998e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.0231239485535951e+01,     7.2104600693234279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1739967380135301e+00,     4.8559370472004193e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4715270951788014e+01,     3.1165138383759415e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.5655086340141917e+00,     7.7985996168469246e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2739374510879498e+01,     1.5942334215273917e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1684695871490387e+01,     1.5083663277825313e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1288607128456597e+01,     1.5916630435351435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9421658020096430e+01,     1.4839195643865540e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7858550940646644e+01,     1.5553774916704158e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3142001847329839e+01,     6.5895685721259360e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.7884098879522647e+01,     1.5539151493946876e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2597550730316073e+01,     7.2176656613932628e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9088400765708498e+00,     3.5993218771093334e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2995994610342237e+01,     2.1515621096388728e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3174521261304616e+01,     1.6207110291430936e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.9656530655957045e+00,    -9.6926843515227702e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.3189157555568798e+00,     2.4978112290424951e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5223509770126327e+01,     1.5440582581053726e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.0860451536186719e+00,    -1.6641383341022902e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.2341971300176375e+00,    -1.1664436706609754e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.5427950018560805e+00,    -1.4486898027368334e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6747181635464198e+01,     1.8875412162965091e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7991412246023614e+01,    -7.6943503651351566e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.4166656081486224e+01,     8.7821973737889148e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1182866312887992e+01,    -7.0793738884136769e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.2562074735069366e+00,     1.4050852504538150e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5519774006010636e+01,     3.8263523028918338e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.7600361415375499e+01,     1.1002285951462240e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7703875369665795e+01,     2.4579935392979096e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.3329762726744921e+01,     1.5559821571521441e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2676181104184758e-01,    -2.9219990533896112e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.4961630990068031e+01,     1.5546357086016711e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9773324500513340e+01,    -8.5820269834318697e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9390879656841975e+01,     3.2722506529851857e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.4043415424097487e+00,     2.3537384471133519e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3696976518747206e+01,     1.3202677415279733e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8371378871698028e+01,     1.0037081575402763e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3583212665071311e+00,    -9.6528694496248572e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.2378186581206130e+00,     1.5002360691156835e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2166087467670728e+01,     1.8071732373678420e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0940783210442696e+01,     1.2400887732379308e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1865647541649579e+01,     1.6726074954718479e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.6966485634950024e+00,     7.8761818880216279e+03,    -2.4682421207140225e-04,    -2.2557798805609374e-07,     1.0577131599398632e-09, } , 
      {   -6.8168347480552214e+00,     1.4908413230493501e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.6725009861000750e+01,     2.3402397737151350e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.5782476369412493e+00,    -8.3006943056559812e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.7516240980860918e+00,     9.5845952492476608e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7870557339495576e+01,     1.1046056785019202e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.0507568665475681e+01,     1.1852762240662069e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8909430077704368e+00,     8.9459783829466287e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1465910853194980e+01,     3.0478530902895898e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3148141711577143e+01,     1.6465308216601152e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5789243554342740e+01,     6.7605569161928797e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8065869748286264e+01,     2.2441348446659278e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3381283348657803e+01,     9.4973926084939540e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.3820338641737120e+00,     1.5989260362303445e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.6288988412929424e+00,     1.5418801363096525e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.5418656293798790e+01,     1.6522396115264732e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.4505380538807788e+00,    -8.4492233767449616e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.1093837423616201e+01,     1.4914087675427450e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9508900486605942e+01,     8.8878205771170833e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3740034847085971e+01,    -6.4749202646257982e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.4335532698869272e+01,     1.5049433438918417e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4546411937068573e+00,     7.0628874666170459e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.2462565641042112e+00,     2.1906340866140308e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.8247724465684829e+00,     7.6911950516625893e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.7047859486789569e+00,     8.0258085846668000e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9035768916267571e+01,     1.7572293488260591e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.9419033154441237e+01,     2.2772359183628570e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.8155231086649097e+00,     9.4256879576140873e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2426554192162477e+01,     8.8041956875861561e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6211816089563548e+01,    -1.0903248852587421e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0529832777505042e+01,     2.5544492080897111e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2606610169625743e+01,     1.5304837842313054e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6500381482521867e+01,     1.4053153704479562e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2104050396148914e+01,     8.3131934347496099e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7922876015646459e+01,     1.4446174674172551e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6639240104055780e+01,     3.1674001263111219e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5011271293542670e+01,     1.6235496224841196e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.4827733782411823e+00,     6.4798170858904186e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8710016174947128e+00,     8.4269072018924671e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.6586141456306080e+01,     3.1636263903997005e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.4263848446046623e+00,     1.3082493047010814e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5558222157859980e+00,     7.2030422386359805e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.8405672161835454e+01,     1.8071526658452400e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1326299450049333e+01,     2.9868708311770413e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.3761965865774583e+01,     8.3470537409389326e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.6977053887705242e+00,     8.8937206312023136e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2480633409460143e+01,     1.7936539924470231e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2309646347673501e+01,     7.7927070529687489e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.6100649495315684e+00,     7.2942619246326167e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8407637424639173e+01,     1.4252357692435549e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.4199370728949425e+01,     8.5459290965201435e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5850066064115786e+01,     8.6748130556630113e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.8034305517911740e+01,    -7.2947436314669321e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -5.4567835607374001e+01,     1.0177257587036129e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2592125926823115e+01,     2.0967495604671494e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5335030518899984e+01,     1.4190967287806454e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2716061660808300e+01,     3.9450248420990632e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.7198919353872011e+01,     1.3746849048459955e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.7830383786153536e-01,    -1.1191171900941648e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8162731854878338e+01,     8.9166586266146987e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.5118750948448078e+01,     1.6886110509000326e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.1226035167309590e+01,     8.3773932904380072e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.3486078130884952e+01,     1.4206509316094884e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8386879574637820e+01,     1.4827705656008811e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    8.2831695113008038e+01,     7.2116750323831939e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1955061616846962e+01,     1.9187669756030985e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9480041714861024e+01,     3.8526574359177503e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.0351906783170932e+00,     1.7642293582896418e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5192549799782611e+01,     1.5540366457006643e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8640975515414357e+01,     6.9875927188144497e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.5777668174506433e+01,     7.2250834920807083e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6418277900055184e+01,     8.6310422221060489e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7822245479501063e+01,     1.5741828799471356e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.0466558287661989e+01,     7.2164506983334941e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5735238582623722e+01,     1.6657531266304115e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.9870327353787474e-02,     1.6657234432189685e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5272963732061108e+01,     2.7173347826489575e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.8116354496923790e+01,     1.5545142122956946e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5252823651300176e+00,     1.5443427495111517e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.4213221333636035e+01,    -8.2872217910078980e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2843961979438387e+01,    -2.5103435886700686e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.9443685444908866e+01,     8.9569009580951533e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.9714989364517209e+01,     1.6792369583229069e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.7471183043530729e+00,     6.3478388326440618e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2502073788802043e+01,     2.9359493116460533e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.0356835119499612e+00,    -9.4613441920903551e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.1303694912648336e+01,     7.0574824968332341e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.4918208948153886e+00,     7.8827108334581817e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.0228706164235533e+01,     3.2200848484695573e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.0652937256269512e+01,     1.0136061918747955e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4100122517022484e+00,    -1.6298143307009072e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.7115954694094782e+01,     1.6473349398426635e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.1474547351826843e+00,     2.3382823540444271e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.8199073953212292e+00,    -1.5541650762587587e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -2.1233086858180073e+01,     1.0965796158092435e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4737430493246837e+01,     7.2151587462258303e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.2775277961018023e+00,     2.3902624118120322e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3506344802255757e+01,     9.0552147869380115e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.1212972677078131e+01,     2.4078693980400138e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    1.8167388609832543e+01,     2.5208242978024638e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.5973568515628420e-02,    -8.3118306779758805e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    2.0829882291693188e+01,    -9.6130545477269443e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.9662239272820109e+01,     1.5525122263707326e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5179101384777768e+01,     8.2107987805570319e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    9.6229816114798208e+00,     8.8863978861833330e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6778117827267494e+00,     7.1430695599453538e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.6999826894620416e+00,     1.2560123656387652e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6434486736031204e+01,     8.3566885435909189e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.5364303187265840e+01,     7.2016113458810369e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.3876545806908840e+01,     3.0785787614797177e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.6267974517483083e+01,     3.2199425793761820e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.4068627688349739e+01,     8.3659033025812805e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.3789316692378870e+00,     3.1085837212338363e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.9249851369670310e+01,     3.1085179947588815e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2729932920879806e+01,    -8.4219914291500227e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -6.0000507755737544e+00,     8.3298206751847101e+03,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {   -1.3852752529937149e+01,    -8.3275621740621882e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    4.3416714488672437e+00,    -2.7876749255569694e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.9246432944307479e+01,     2.4975257562613493e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3194700266825720e+01,    -1.2496152932472723e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.5372093871752739e+00,     2.2221809574821171e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.2201755693335819e+01,     9.3464171833270630e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1143305164901927e+00,    -2.9480519242396940e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.3746886925338082e+01,     1.6646566137990041e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2672087066719540e+00,     5.2165804515628406e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1964323042868340e+01,     3.0404231897706853e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1890975987535723e+01,    -8.7090931558984503e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.5397164475872700e+01,     7.4131373748479054e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4525010347968106e+01,     9.0563258045392704e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2853756913724128e+01,     3.5348875272954765e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.1668421230561968e+01,     1.6306048609009506e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4228787498375347e+01,     3.9071976975157850e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6996660114336528e+01,     1.0392012050357403e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0695524148918121e+01,     2.1813040861613736e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.1389836298552240e+01,     1.4286145833441435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1386310338456241e+01,     2.1542861801204759e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2993400026413369e+01,     8.3272605318685055e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1423883579229557e+01,    -8.3301223173783965e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.7017808856480361e+00,    -1.6375695568021196e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    9.9646907990750169e+00,     1.4949391593945644e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.9280108346779503e+01,    -8.8837045043145918e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    4.0090543536176781e+01,     2.4083296380282958e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.3085687323455463e+01,     9.2910602531407203e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8224411183545111e+00,     5.8592319179982405e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6047099479661966e+01,     3.2086021073577576e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.3031762675711931e+01,     8.4729792353908397e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.7673520415111916e+00,     1.3764481074734424e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.2051689942373631e+00,     8.2735800891904382e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.9984498543590988e+01,     2.3560908038181980e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7102893257927889e+01,     7.2129669844908585e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6994864369961515e+00,     1.7365872527620751e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8935323719201321e+02,     1.4914954663575367e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1865766092790601e+01,     8.1421066750854534e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.0590516788234780e+01,     1.6894541292157133e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0320260008632950e+01,     9.0966844205535635e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6770634654160087e+01,     1.7942525829262377e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.1777426062075961e+01,     7.0565342262439335e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5391918080008736e+01,     2.0990178169437638e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -4.2019351748732362e+01,     2.0919714977822662e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    5.7988755946372565e+00,     7.9652293256338635e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3806484147134030e+00,    -8.9378113399129634e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.4910133122454624e+01,     3.4258550387696032e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.8030234671823830e+00,     1.1581788831952954e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1289657606667841e+01,     8.3963790302603884e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2173590556001193e+00,    -1.7079912553012564e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.1409689509943153e+00,     1.4935065267447228e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7087615318460294e+00,     1.4948228132391412e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.5399301338309819e+00,     5.9414481870432610e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9982377304458538e+01,     5.4485848159154521e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6203149737589385e+01,     7.7558351167024675e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.4661478461135449e+00,     1.6641963738107224e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9738410045279085e+01,     6.6475750573256264e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0093471633063800e+01,     1.8199897325424927e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0250476023607959e+01,     6.9761464176896043e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1935615510137797e+01,     2.1828788605128182e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.2793824673580474e-01,     2.9968578820757073e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.8881562848527693e+00,    -7.1504863397298482e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.0937307346427554e+00,     7.9291114993313295e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8451874229080246e+01,     8.8391063656025763e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.6356513079100612e+00,    -8.3838027600564637e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.2915004242357853e+01,     7.8751854754579854e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9085990292340171e+01,     2.0194571576483788e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.2243625073438054e+01,     3.1571477238225078e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.8731474281872423e+01,     1.4270597091602278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4185626322865836e+00,     8.9783289176468443e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6912101818394195e+01,     1.8230786266044044e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2415718276067317e+01,     1.2729474352258436e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0547153803863431e+01,     1.0196341182994413e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0655556119908038e+01,    -9.3982692338502693e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1670898179166265e+01,     1.6064412335138079e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1886179606963223e+00,     7.2905041087692089e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2623828453219975e+01,    -1.7766875616032303e+03,    -3.3728746520459481e-04,    -4.0348870713455303e-07,     1.9445343454254264e-09, } , 
      {    1.4983741819811819e+01,     2.3347888482096754e+04,    -1.3009140913673313e-04,     5.6728873009884696e-08,    -4.3817460498679860e-10, } , 
      {    1.2215293743769578e+01,    -2.4050495371424477e+03,    -3.3975548197033502e-04,    -4.0295958148299006e-07,     1.8886401761303078e-09, } , 
      {    1.6128162788763952e+01,     1.6534523974016924e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8658507826098294e+00,    -1.3705786908745549e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.1907483977601192e+00,     5.0202303571399843e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6275836652884974e-01,     7.7249670033788398e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9048602300482862e+01,     1.5494052424167241e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -5.0668994343393372e+01,     8.3377268184843397e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2898735285877166e+01,     7.5246005417816123e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2653081902375202e+01,     1.4508324789990962e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1271841881487425e-01,     6.2082941847648647e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9732837530917045e+01,     2.2048327476966289e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0675498663036731e+01,     9.6236882851727091e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7390492683896809e+01,     6.1103458827987470e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.8181848860456586e+00,     1.7285684804460961e+04,     3.0192093403174383e-04,     5.0205414042347212e-07,    -2.5264077254930913e-09, } , 
      {    1.7892457291341557e+01,     1.3800529491138022e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.2401565860792960e+01,     1.7350124784106301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8804393772279710e+01,     2.9766760260000057e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.6099936244432900e+01,     1.5541229036730576e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2953930098173636e+01,     8.3356913014672646e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2557920506045804e+01,     1.6181371156282825e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3178924335465709e+01,     1.5544279543233013e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1122987179929185e+00,     8.9493724260427225e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7095652408747464e+00,     7.1960291814176353e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.7621672878027876e+00,     1.4048222672221968e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7611643804032600e-01,    -1.7436474897775252e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5554092376487088e+00,     1.3688983129984143e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.6800400785840310e+01,     3.1651459357978976e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.8855982236097496e+01,     9.4325032726316986e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7641607149730984e+01,    -1.0954408895090737e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    6.7235765657242457e+00,     1.3763058383800671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3411479669566942e+01,     9.8942163477962658e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3121872101195937e+01,     1.5387649767957548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0329753544741571e+00,     2.5934124335104807e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1580123951086943e+01,     2.4499400983692714e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.5650823289738980e+01,     7.7002857880040665e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8414638200822893e+01,     2.9973181220639894e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -3.5291205119902820e+00,     1.7348047241322602e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9852943892465760e+00,     1.4931367385743844e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0008320444023804e+01,     1.6707046246682039e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.4159391449062124e+01,     5.4087699140175391e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4053629325189933e+01,     1.7678711403985431e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.6812182738951918e+01,     8.4586110865639275e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    8.2426021577578084e+00,     3.3406124347172772e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4630986623024023e+00,    -1.3433656947574382e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7713773736041453e+01,     8.3757604677494128e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.9710744089941116e+01,     7.7712790934170571e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.5374701830976560e+00,     7.6136333149115553e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.9753525854210654e+00,     1.5948893170873049e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0639110351867694e+02,     7.7714751965647365e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.0617464612188591e+01,     1.8994080585071755e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.9027318023329158e+00,    -1.0327577700543548e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.7527794353643387e+00,     1.7702121630493220e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.7508571482570503e+01,     2.1913893032516607e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9818468636004823e+01,     2.2125879737978412e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.4010297025857206e+00,     1.2559038246242101e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0172523289681052e+00,     1.5507417219045072e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0535299163003067e+01,     2.3878004670204380e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -5.5570974186323411e+01,     9.5171832084254220e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1649133488898528e+01,     2.4006432448587409e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.7120595724059683e+01,    -1.0393774981943934e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2267204953088271e+00,    -1.4017302860135314e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.0745126347969162e+01,     2.0199094891187960e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9428017809286953e+01,     1.0143724401741296e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2491623113790304e+01,     1.8720703474980855e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9145983127329288e+01,    -4.8252841248417244e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4946844394433965e+01,     1.3817450171945615e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.8116515301823561e+00,     7.3537482762429154e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.2001340103805527e+00,    -8.3744704628220825e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    8.4460431272511798e+01,    -1.7429428578613756e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.0901223343264066e+00,     1.4428454363091463e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.4538823833211616e+01,     1.4427797098341918e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.0134457633141096e+00,     1.8319536579254382e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9721320628246097e+00,    -2.3919072456130853e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.3025522640430705e+00,     3.2063009074163456e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6346946965119660e+00,     7.7195715093339377e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.9801022566276405e+00,     6.6806728480128204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9737033194826634e+00,     8.0158714997365905e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    7.7488360798627722e+00,    -7.3139218254840534e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.7808908819959459e+00,    -8.3463234508979167e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2891357469424584e+01,    -7.9268539861218023e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1690421994665412e+01,     1.7016622391484729e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.8266355171700866e+01,     1.6939070130472603e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    6.8796457488395703e+01,     6.1963371066547315e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1615149119261851e+01,     2.0370339614764925e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4516536653605837e+02,     7.2141048911249691e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2279959126468393e+02,     7.2140208395917198e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3324687278782292e+00,     6.7549718532018633e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7322631431385297e+01,     7.6600434565236128e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.6340011049023744e+01,     1.4969803635122595e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4676115380944186e+00,     6.0427673271786653e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0521691622802425e+01,     3.2166355711773533e+04,     2.6931621671538551e-04,     5.9995954836692873e-07,    -3.0545685978306309e-09, } , 
      {    1.5977579082550786e+01,     3.2233918566683853e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    1.3810722420612183e+01,     1.4461907158171849e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.5799997756746387e-01,    -1.7947779446793145e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    9.5802331889462415e+00,     1.5366986251700655e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    3.9445848385271390e+01,     1.6730677354601299e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.2094610564180911e+01,     1.6171068588578062e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5176604048012976e+00,    -2.4514258226845000e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8579782763901846e+01,     1.3958725437304720e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3781017064288672e+01,     1.5179292190992208e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4961258681381558e+01,     2.2459397390115137e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4969807682844625e+00,     8.4630514518231612e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1572528048943207e+01,     1.6728370524828410e+04,     1.1653693820750085e-04,     2.8239412752857817e-07,    -1.4944333238833335e-09, } , 
      {    4.1772160428502074e+00,     1.2871789595459191e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.2141626353898296e+00,     8.3570431470407366e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.0678803245784703e+01,     7.0589583433340958e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6186079502193920e+00,     1.1205383863795210e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.7830580149331638e+01,     1.5193151018221637e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.5507977880065738e+01,     9.3689590884593072e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.0647161486414136e+00,     3.2200465771603467e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    4.3563986853994628e+01,     3.2199808506853919e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.1923314368529292e+00,     8.6921535236130367e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0728654341412991e+00,     6.1463336447466672e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.4418206328039091e+01,     2.1908642066081717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3626908156809971e+00,     8.3532840641632938e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.6610578343425217e+01,     1.3211325462523515e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.4614865239348882e+00,     7.7041971313910417e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1958722975163489e+00,     1.4682557170392593e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.9563256067586209e+01,     1.0308786011844333e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.4956082447758803e+01,     1.5543458921897944e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3209877678937087e+01,     7.0697750545909530e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.4580318570056789e+01,     2.3776273882520989e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.4766636566790918e+00,     2.3754658470377126e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.5424843385666485e+00,    -1.0203624773658277e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1433581394424206e+01,     1.3347338533066772e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2619096676187119e+01,     3.3050613428428522e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.7626444175279659e+00,     2.3887178198604543e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -1.3568510965511980e+01,     1.5750002555776686e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -4.5270640324644617e+01,     1.5009618537020508e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.8368833314070351e+01,     7.2125376121071258e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3156985023179764e+01,     2.4100173374358670e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3228597706039562e+01,     1.4329510281313287e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0736484904311929e+01,     7.2155881186095612e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1990817200245498e+00,    -8.8679814960542026e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.6891058037291707e+01,     2.2676635062011832e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1386655223784244e+01,     1.6047099472403157e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -6.9520511290080151e+01,     7.7869191732793279e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.0904194261473918e+01,     1.5860524198604071e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4400410014010578e+00,     4.9663397435138549e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.7928393141965245e+01,     1.5542049658065645e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5103494150532626e+00,    -9.6992701154980059e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.9350677751569716e+00,    -4.8275853247831346e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8517050655493903e+00,     7.0191071573211721e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7650631418354482e+01,     1.5771481949735216e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -9.3771583079273348e+00,     2.3243466761934476e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.1762901966239852e+01,     1.3443214997067174e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1371281980130826e+01,     7.9497482072121202e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.4834181125763674e+01,     6.6077965337575297e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2149373982896218e+01,     3.1178808584490165e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.8244088632741065e+00,    -2.4218859034589195e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.2909658374413667e+01,    -1.7769710208570599e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {   -4.6217871360353541e+01,    -8.3010824474495166e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.2790964877540880e+01,    -1.5836765367190001e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3094646356653694e+01,     7.8430817958707648e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1490212890839164e+01,     1.2951643056412726e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4060551984798469e+01,     2.5739135148110243e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.0354614750175166e+00,     2.1517122666056646e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1084857237660319e+00,     1.5505994528111320e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.6154497308207532e+00,     7.2126401744245923e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.7414671124692873e+00,     2.1047902593495073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7123008098375156e+01,    -8.4242926290914365e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.6304324531790675e+00,     8.3293130522979645e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6755778486824763e+01,     1.5543857817376005e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    3.6659262355597733e+01,     7.9843723207450239e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4753018561733956e+01,     2.3778145710078672e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.0834750223444832e+01,     2.2614478228556083e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0912343249064353e+01,     4.2456784056107390e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5631191432638794e+01,     7.0443283015199549e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.7283667066446764e+01,     1.4932470913138051e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -6.2858519214227357e+01,     1.8365773000026213e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1591299097055050e+01,     1.0182462584621102e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.2032191782974405e+01,     1.5554751900725269e+04,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.6845859680561237e-01,    -1.7609603525124478e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.7973566228673752e+00,     8.3352428900155301e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1671954154062032e+01,     7.3691673873825930e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5745743080310001e+01,     7.2154855562920966e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7188539016675875e+01,     2.3387425940327095e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6240899230656609e+01,     3.2277689400240819e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.7244268165749439e-01,     8.9881774131840721e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.6985967749067257e-01,     7.2136696190959028e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1898284950495093e+00,     4.4966975049625926e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.0532459957142834e+01,     7.7161163235772055e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7263399504281573e+01,     6.0334744427518672e+00,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0945749124830426e+01,     3.2788629822376330e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.1165478041700382e+01,     2.3227596101271272e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6695875328744034e+01,     2.7566368796182564e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6842052501268352e+01,     2.9395541532864136e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -2.2221543221170403e-01,    -8.3280697969489356e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.7271070988303549e+01,     1.0388415815640159e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6125710751513239e+01,     1.5591456155796352e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.4042635728307339e+01,     3.3443634398121510e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.9119269669759653e+00,     1.6031705096517542e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.1296924241066392e+01,     2.2171169386250302e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1794968994932013e+01,     7.2144561116207860e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0686713290659799e+00,     8.4334661574916008e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5718415153248682e+01,     6.5545768767977233e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.3726615467406269e+00,    -8.3132615707308250e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1023335573829854e+02,     1.4913938746297134e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.4400997134984312e+01,     7.2151663927525524e+03,    -7.7462757135888876e-04,    -8.4758119031981508e-07,     3.9238880094281401e-09, } , 
      {    1.3174940614857455e+01,     8.4526538272476319e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1279107440091831e+00,     1.6200772690806727e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6130189174118726e+01,     1.0665389160448305e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4588635946023141e+01,     1.7250745545283051e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.6180031271838349e+01,     1.4900940193664897e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.7766000587303893e+01,     8.7406515034137628e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4473776028338090e+00,     1.0160645082548888e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.2794541104433236e+01,     2.3243144843110425e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5346640528918076e+00,     1.2478768625766203e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4668282786458537e+01,     6.8467790452816107e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -6.0662283617554451e+01,     1.7443219073406850e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0448677595217957e+00,     1.5530756679238322e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.1020044193090373e+00,     8.4022790843456169e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5043956370108731e+00,     3.2120507335432871e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.8916599639507492e+00,     8.2551037649012833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.8552670527468166e+01,     1.4890173831326769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0420795967433492e+00,     1.7871338203382282e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.2051481153502621e+00,     1.9804827290144201e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0409463085389753e+01,     2.1579831789044878e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5175853294375278e+01,     1.6644931329769592e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    3.2361419464889281e+01,     1.2793908702072295e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5103499068041678e+01,    -8.9509655352262416e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.7734988959932792e+01,     1.5754604955659510e+04,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {   -2.2022738435941495e+01,    -8.8883069041974122e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -4.9464842948089029e+01,     1.9033895486969668e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.3251896662343711e+01,     9.9972666735048497e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3391220803609993e+00,     7.2070592838789089e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.0893936579953895e+01,     5.4883997178133641e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0979040187391547e+01,     1.1648627220448492e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.6120492014330807e+00,    -1.6669834368724209e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.6506604057607813e+01,     1.3986017809170671e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.9159657809083477e+00,     9.4897053902465523e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1696044289598987e+01,     2.1712078374093820e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4024796979683729e-02,    -1.1338162290211358e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    8.1496558624489008e+00,    -1.2205667536874885e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4811243439805217e+01,     1.4985440010349243e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6851934483025417e+01,     4.0562585610976210e+04,     3.5987740221216227e-04,     7.7782663421408804e-07,    -3.9421170038378582e-09, } , 
      {    2.4358444799502621e+01,     4.0495071516728080e+04,     4.2151069211412756e-04,     8.5072205575288325e-07,    -4.2898253759296179e-09, } , 
      {    2.2046015238840649e+01,     3.1538078136890570e+04,     2.7188216584470991e-04,     5.9938678948406585e-07,    -2.9994016490571770e-09, } , 
      {    2.2490244456840315e+01,     2.3280374387848628e+04,    -6.8458119234767785e-05,     1.2962429454867986e-07,    -7.8588297707855784e-10, } , 
      {    1.5769783762739163e+01,     1.5019197057473302e+04,    -2.8238381689905929e-04,    -1.9399000114477001e-07,     7.9780939363385223e-10, } , 
      {    1.5022465058232008e+01,     5.9237019078061603e+03,    -1.8233117594936010e-04,    -1.5338622509406093e-07,     7.6299007505656529e-10, } , 
      {    2.0711567937324794e+01,     9.1453862863681206e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -7.0443768262827078e-01,     1.4426048187932989e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5151311822321675e+01,     1.5038409107560434e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.4018340878077076e+01,     2.1360516574364483e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.5700654992026593e+00,     1.5543108893431609e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.4647118003008657e+01,     1.5542399686531979e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.7726413644373755e+00,     7.8696004124669689e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4628467351617459e+01,     7.2132508197184507e+03,    -5.9548675818625800e-05,    -1.4150664154036154e-07,     7.1223977891801919e-10, } , 
      {   -5.2262523948585056e+01,     7.2148749109982382e+03,    -3.7740249195369381e-04,    -2.3569566172990487e-07,     9.4368983027971148e-10, } , 
      {   -3.0951216640603363e-01,     6.9646984977241309e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.6321199651435435e+01,     1.5511274062468045e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.9153118186318096e+01,     1.4430203273500392e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4536881768872453e+01,     9.9848171101077205e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2813020095127680e+01,     2.2612529344572751e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.4191210980045767e+01,     2.3108151395577537e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.6269134796570377e+00,     2.2971433693118131e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4274713889166291e+01,     3.1418665576024887e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5560465125938791e+01,     1.2094607811613761e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.5150574411178059e+00,     7.3511337034418830e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.4647409891012501e+01,     8.3563004017973835e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.3354538007454256e+01,     8.3378649530236162e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    2.7645908898279119e+00,     4.6206504318478637e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7928843079255167e+01,     8.3661625319608011e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0664720399116035e+02,     8.9570970612428318e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5491343388067948e+01,     7.0743432434808028e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -3.4912670634035742e+01,     1.0156526281096500e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.8476423098613111e+01,     1.6520665146787494e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4637349546569745e+00,     1.5310864755438135e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.8359563481407424e+00,    -1.8204884053227729e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.1350195262179064e+01,     1.8525032607617861e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9402549952880470e+00,     2.3933578864477691e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.7725055592871772e+01,     1.2702110267213633e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.8488520827570252e+00,    -6.4522826585313658e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2788326573239150e+01,     1.4851168326469573e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1515929500220096e+01,     2.2130107022354796e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.9118237640512305e+01,    -9.4433132703378287e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.2061659423984707e+01,    -9.4433266974392791e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3644819156377842e+01,     8.8767999503947212e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5119148407828366e+01,     1.3595698384972718e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3957858977405138e+01,     8.4085448855769846e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2490801723647671e+01,     8.1529081268271548e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8665704170279280e+01,    -1.2597436751875505e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    5.3333227770323511e+01,     2.3871086685096441e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.0803018420985913e+00,     1.5341249102084401e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0234029226100876e+01,     2.2779437248296172e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7985064305943965e+01,     2.2163454459303990e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.0801217133095173e+00,     2.3852258044849215e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.2355294537155617e+01,     8.3507326780681778e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3616958812425741e+00,     8.3066501711787223e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1545715549695053e+01,     2.3871804744114048e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.8743827401690680e+00,    -8.3139729161977011e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4299794777341116e+01,     7.7261229747260168e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5115619894375783e+01,     1.7737465414980670e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.6207678493032326e+01,     1.7148972731833037e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2739344662409531e+00,     8.7877824367799312e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2226570018096419e+01,     7.5798518874563952e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    5.9745108665461659e+00,    -1.1844584031238639e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.1834894463263446e+00,     3.8297270245380525e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    4.7168597681589226e+00,    -8.5977594674311676e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.3613151919716657e+01,     3.2193578183629561e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    1.4661554382240780e+01,     1.4461644314216483e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.6865110224494501e+01,    -8.3318046702047614e+03,     1.5892690806753402e-04,     4.7094510094771660e-08,    -1.1572502568084620e-10, } , 
      {    2.2089716002904343e+01,     1.4133879389142874e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4597132783123902e+01,     6.4821440089988819e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.0837354781719558e+01,     1.7195020262082053e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0677998203523522e+01,     1.6101172097008553e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.7552968515240868e+00,     7.2832412979102519e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3492873612433200e+00,     7.0551555737247700e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8217278541522653e+01,     8.3927734584792306e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6327692673574756e+01,     3.8188760084625919e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9039502029368705e+01,     1.1395306921155227e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1545046478458616e+01,     1.5956445337249346e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.8310481620574528e+00,     1.7338659531503636e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0389398858896692e+00,    -1.4919017854436420e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1938719091739404e+01,     1.2245783210355055e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3339822056180441e+01,     8.1528634521822833e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0770256483697779e+01,     3.8301872645263342e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    3.3517064506928684e+00,     7.2729320166547650e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3442782862912850e-01,     1.4976803511966407e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6580121022157208e+01,    -7.0851941657306288e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.2988636219328384e+00,     9.8397380343904388e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9293113339789365e+01,     1.1439077754712191e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7210672314683571e+01,    -1.0501942094512490e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {   -1.9027412885149118e+01,     8.7969634553871510e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3779688636001088e+01,     6.9515443024108517e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    6.6582466596464997e+01,     1.6562211017162648e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.4643702487472332e+00,     7.0018835673058593e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.6478252633712010e+01,     1.3483029898965086e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1432093230052839e+01,     6.8506007663687578e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6548226051062766e+01,     8.3827640492288483e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.7812081679187237e+01,     7.0071432319382275e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    5.9121127737596623e+01,     8.3934780903953815e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2283016244057112e+00,     1.6060253667716435e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8705992310164206e+01,     2.3051063496913957e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7292006176578891e+01,     1.4918249474545579e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5906774106292874e+01,     1.5074065342036702e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5431971587589217e+00,     1.0149624455826524e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0612557558920786e+01,     2.4906444470074523e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.7132234476640669e+00,     7.3047330375687361e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0557227341172439e+01,    -8.3306557722929265e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.5872222500441779e+00,     7.0987675581507574e+01,    -1.8804787731715151e-04,    -2.1904362078073129e-07,     9.7753467335796819e-10, } , 
      {    1.0835311749799686e+01,     2.3543230485485988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9370915762949778e+01,     1.7460492070172520e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.4424354269207967e+01,     1.4213509192938698e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5069178484940034e+01,     9.6213550111181357e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4480267966158216e+01,     7.2121139813750087e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0599168923814766e+00,     7.1857293759010572e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0549885008258615e-01,     7.7953014072779260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3384239311869496e+01,     1.7288320266608629e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.8978671231030768e+00,    -9.4951350952844259e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.1228044175725181e+01,     8.9696182480869793e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.9146106303156984e+01,     8.1754652914745529e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4553637108294039e+01,     1.6248614136039461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5711444687201759e+00,     1.4338396871635023e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5476523102294038e+01,     6.8644595935981870e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3115687741021036e+01,     2.1570091763267836e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2358379363303925e+01,     1.6728376154659891e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.2271676623007309e+01,     1.7588041231775038e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0780564255815772e+00,    -8.7025342002993184e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.8209260491344107e+01,     2.3209362331936023e+04,     1.1968769044596786e-04,     3.4862428209811127e-07,    -1.7641448709581904e-09, } , 
      {    1.8909046171636493e+01,     1.6137274817740694e+04,    -3.7932480302175347e-05,     9.9138192171781901e-08,    -6.3704032884111916e-10, } , 
      {    1.0790246264085246e+01,     7.0993305412990367e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.1038698865902873e+01,     2.5065704077666174e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.9244148731313516e+01,     1.5574234517495548e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.9239720931063989e+01,    -8.2828430482827862e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.9416224059049288e+01,     3.0457200994918046e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -8.6296496902930393e+00,     8.1555379591433393e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9132845167987806e+00,     8.4606886293476273e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.5909559560740242e+01,     1.0045391746537563e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2556787822461501e+00,     1.4349417498357387e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.8784857200097846e+00,     8.9483625112861591e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0369202038515958e+01,     1.4328087590379535e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.9311948065138880e-01,    -9.6720571193808883e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.3838097062692434e+01,     1.3122495321951425e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.4974829364557834e+01,     1.0705204062346218e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.5684994712673756e+01,     1.4428027679142850e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1413368400182240e+02,     1.4428223782290528e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.0878276611141757e+01,     1.6752554681331152e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.4677216812720136e+01,    -1.7774312608453412e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.6998737489621256e+01,    -8.4819175577723472e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.0039592667799440e+01,     7.1519297154858978e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3140683841376234e+01,     8.6627526680951778e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4035601334722703e+01,    -7.8056731579337793e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6833316607041148e+01,     7.1622477539624742e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    8.3920209254158387e+00,    -8.3249935429200650e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3686654530300190e+01,     1.4999054991380783e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6596564620570859e+01,     8.1054383884626250e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5999308894729168e+01,     2.4886215313744640e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.5850359430246872e+00,    -5.2959621472281015e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -4.6117087595545012e+00,     4.6272093874469974e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0393497760653947e+01,     1.7127956079952011e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.3159478005277677e+01,     1.3916019040732526e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.7887189674198865e+01,     1.5542028288429719e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0956868612205184e+01,     8.5797105239753037e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.9151869140052071e+01,     8.3288818803343202e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.7582352692868255e+01,    -8.3285009689125818e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.7689022938736308e+01,     1.4920480179379005e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    8.9256435428851795e+00,     6.0736951709451287e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.7498308151627171e-02,     7.8051585724460574e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.8550831997484856e+01,     2.3855713230605950e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -3.9745405740695858e+00,     1.3295962160291156e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.0953719528591588e+01,     2.2521530539987587e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.2316508330845267e+01,     1.4564843433176095e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.2307493065331414e+01,     1.6186603903315767e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6327566581670609e+01,     6.5861143098216025e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3273328021987014e+00,    -2.5278240978813024e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.2742578155369447e+01,     8.9630324841117454e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.0810334253023868e+01,     8.3745398009641140e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7038770022148498e+01,     7.1027291768910609e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.7888993170599928e+01,     2.3865545660520016e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.7445957808547938e+01,     8.4219761696348742e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.5254734709649842e+00,     8.9531857178558566e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1604424910766269e+01,     7.1975544346688521e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.9225473780597248e+01,     8.1866895542823167e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0904378596680637e+01,     1.4381768033057602e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1212325246411382e+01,     2.3236579173960567e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0105160403664474e+01,     1.0147695991393971e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.4954965929510777e+00,     9.8827263599395374e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2802046363279668e+01,     7.8374073944456359e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6373715707793970e+00,     8.4404697389710363e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.1514551810589957e+00,     7.7224250930226353e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4707562100063420e+01,     2.3890633384361277e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.4872334073499125e+01,     8.6178706941555756e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    8.0014978917277109e+00,     6.1053343601784682e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7285300019440029e+01,     1.6250691678823163e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.4450009218878435e+01,     7.8423771639546158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1405626462494617e+01,     2.3953929007874962e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8028385378322419e+01,     1.4999766336847659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0668065809928414e+01,     2.5368278532306482e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7807284289672616e+00,     6.6063738428237757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5158424222509471e+01,    -6.0818992949328112e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.5597010545621544e+01,     9.6804784267987907e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9833029211429711e+01,     7.2965461586280635e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0257074743049845e+01,     5.8618877213894684e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0142085097404154e+01,     8.9324158458680595e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1213261450328758e+01,     2.3886864825744924e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1652283825454980e+01,     2.6709646945226045e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7906601812475579e+01,     1.3643237777146036e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    5.8814911245781119e+01,     3.1572181870141227e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2095536011114964e+01,     7.7371436014483816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.2829990964105278e+01,     1.0331327916976577e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5556300942224240e+01,     8.2660596019707264e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.2865475876572191e+01,     1.0642847255316061e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4225953280868762e+00,     1.3602264414050051e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    8.2209510825834666e+00,     1.4275829838635220e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1600733383218298e+01,     5.8224609612850918e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8180371054038487e+00,     6.2901893051241950e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6889896912749208e+01,     1.4396947327201611e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6577045607891594e-01,     7.3875639146910471e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1989420192125326e+00,     1.2085747656073214e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1139977417711542e+00,     7.2160117493416792e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2198421531651338e+00,     7.1933993491014517e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.5465305715230002e+01,     3.3836655367814499e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.4717221851242101e+01,     8.3255781790421388e+03,     4.6351172359218639e-04,     5.4853225840408113e-07,    -2.5876930229221479e-09, } , 
      {    2.3139387355844207e+01,     6.4160207164744115e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1756104219070581e+01,     1.3743151166756572e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1872083891459436e+01,     1.6643876337975547e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.4676860775091612e+01,     7.2143171297425788e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6192405540219404e+01,     6.8962929567360679e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1605133107601340e+01,     1.8941816119776715e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3029889500461469e+01,     1.3975646118112314e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6452074043156436e+01,     6.6501896301266597e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.1764861559224506e+00,     8.3029542750071960e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.7966539415805300e+00,     1.1454810238711489e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9813760943376795e+00,     6.4963749302339500e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.9568721182952686e+01,     1.8031711756554483e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2342641054867975e+01,     4.6386993753037259e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7577602025849316e+01,     1.6670889360518257e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.2233069995367458e+00,     1.1061789269018500e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4407188005859936e+01,     2.2992103770692695e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.6004345944022242e+00,    -1.1584192870634633e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.5333022467512606e+01,     2.3715844291023495e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.7042373227653371e+01,     7.2138086009741100e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3004741715010269e+01,     5.0544870981556710e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0201444035519502e+00,     7.1677051676992578e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3914655247205934e+00,     5.4662109483052536e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.4693693529793439e+00,    -9.5208722449006782e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.1251571262166202e+01,     2.3242809497184928e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1293664285334653e+01,     8.8161159262372184e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7845179685884570e+01,     3.9953490076017792e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -5.6687025731485541e-01,     1.4508806496825277e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -6.0734424047899331e+00,     1.0275747329632526e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.7328708072149741e+00,     2.8132799377233582e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2976532093314137e+01,     2.3765507520182862e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1956466938360354e+01,    -1.8728558062511256e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9677392673692399e+01,     7.2260604761018167e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1189566491744674e+01,     1.4982810178033058e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    8.2811350041055825e+00,    -1.2285887522997609e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.4852627949240187e+01,     8.3174519023005341e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.5840355380942427e+01,     2.2301663035774705e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0568388100892683e+01,    -4.4322631551487366e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.3698819365513522e+01,     1.6139883471512261e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.8866608714152800e+01,     7.2163403817164535e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1920515384248908e+01,    -7.4186965583522042e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8802324196170630e+01,     2.0924863497414703e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.2675992966805119e+00,     1.6604423227774620e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.6460865579231047e+01,     1.5540451355042995e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.1618692457495250e-01,     1.5545057224920594e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.6492386146933775e+00,     1.0668768299325513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.8218245234328947e+00,    -2.1725293141184557e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -3.2597013718855138e+01,     8.3993324140783643e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2457775055378120e+01,     8.3007501170934938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0905393576611253e+00,     1.2395292688527138e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.6132742566884154e+01,     1.4927065943354239e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.7421993327218679e+01,     2.5279236283437629e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.9045517968979297e+01,     3.2279766943024522e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.6746791879599243e+01,     6.7207420142949686e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1234806495630838e+01,     2.4502383131966973e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8361312186531606e+01,     1.4915869395870006e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5532388208526406e+01,    -9.7964182528293895e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0888337250905675e+01,     8.4395264403104557e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -7.1636558332565805e+00,     1.7117692896565186e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1572863702499326e+01,     1.4988069842665425e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0944698961871744e+01,     3.3379200048307663e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.6386988687581962e+01,     2.2852418359808125e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2143312904933280e+01,     9.4210132847749701e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2441016842077524e+01,     1.6189110818483197e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9829244297928547e+01,     8.2488379636699156e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4957342010876705e+01,     7.7292402818770734e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0842453640292584e+00,    -8.1528990060112083e+03,    -5.5407290908896314e-04,    -7.2639934425124045e-07,     3.4752414289293752e-09, } , 
      {    1.0339266389900782e+01,     7.3898552839705844e+03,    -6.2025608521279669e-04,    -6.6428162173171901e-07,     3.0672222349075897e-09, } , 
      {    3.0172853165965696e+00,    -8.8522486571319387e+03,    -3.6849304853755181e-04,    -5.0682659781894627e-07,     2.4418125862762885e-09, } , 
      {    1.4539505795028070e+01,     3.1605592231138693e+04,     2.1024887594274456e-04,     5.2649136794527064e-07,    -2.6516932769654181e-09, } , 
      {    8.8124135574659146e-02,    -7.0383304670712660e+03,    -1.8843681569912589e-04,    -2.8593380069572702e-07,     1.3009587281629509e-09, } , 
      {    6.8281500001158166e+00,    -7.3310669824479100e+02,     2.1639650550239392e-04,     2.5602045720169183e-07,    -1.2072830286989661e-09, } , 
      {    8.8418986323590225e+00,     7.6665724019601657e+03,     1.8064103594756859e-04,     2.8769571057561523e-07,    -1.4657323539616488e-09, } , 
      {    1.8027997765868715e+00,    -1.4203537148817486e+02,     3.7096385637565423e-04,     4.3923275932718818e-07,    -2.0654032442628450e-09, } , 
      {    8.6374581453595898e+00,     2.4986074273870348e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.6756515971082369e+01,     7.2332410593754648e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    3.3751857499493234e+01,     7.7828671328476248e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1033380556777082e+01,     6.5808242480552053e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7767173801254614e+00,    -9.2790266739489925e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.6482339937956951e+01,     1.6173139417811039e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2357497688528461e+01,     7.3898461631546397e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5050715061284823e+00,     9.4275722403741056e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.7099710744565328e-01,     1.1847831208404474e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7785511601341859e+01,    -1.6656416446845840e+04,    -7.0626738448770513e-04,    -9.7716185163719508e-07,     4.7104982070283614e-09, } , 
      {   -7.0083337716106193e+00,     5.0267893127391180e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9597465048795225e+01,     2.0597243910167081e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.8280630269440614e+00,     1.2933963975921832e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.3002602073742366e+01,    -1.7429294307599225e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5277600616305058e+00,     1.6097438737298166e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -6.3575433321223169e+00,    -5.2206062944761225e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.2808272654269480e+01,     2.3966617546689500e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7957543708564998e+01,     1.7672152448386296e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.8455325516138892e+00,    -1.3302114994441590e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2945944965667259e+01,     8.1022212780795553e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    7.7740691515908424e+01,     1.6602025919060557e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.8118573943449263e+01,     2.7959389765875552e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.0492189856882099e+01,     1.5485666391318213e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4536848887878172e+01,     1.4603953703157857e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8528227070305856e+01,     1.8472415826364748e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2295107484215556e+01,     1.4946085894169591e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5096115485148591e+01,     3.8891830613963881e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.0204080600932130e+01,     7.8526797316593756e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2044390684279935e+02,     1.5542712264215168e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4752105097860093e+02,     1.5542796315748419e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8644332072452372e+01,     1.3562659702760911e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3037437864353535e+01,     2.0763360584457914e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5863560073387568e+01,     1.6102698401930533e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9372406226368781e+01,     7.7064173140206585e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6824869463276617e+01,     8.3192489103849148e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0226684593630267e+01,     3.0159781229693046e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.3165713627038315e+01,     8.1123439183682203e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8505957442362186e+00,     2.1329907977851814e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3707551901428996e+01,     1.7981062201468027e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9404698642107178e+01,     1.8485217705719948e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3337269775199889e+00,     1.4555207162698425e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3429655029227334e+01,     8.3285506373504377e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4353606063972446e+00,     8.3288322118964625e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9274821153756125e+01,     2.3906782785541967e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7189694479006604e+01,     1.5622314755635587e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2123473563936216e+00,     6.4864284854425241e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0458356941606446e+01,     1.7965314457953573e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.6931023483975634e+00,    -6.1028337684374424e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.8816637073438560e+00,     1.4927564616134518e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.5368415492896276e+01,     1.6171055161476612e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5728860719985548e+01,     4.8701865814555276e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -6.7312031215354793e+01,     1.8660019341600026e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1380481894951442e+01,    -1.2466257639892829e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.3910152057974837e+01,     1.9113724444673840e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.5696547088012457e+01,     2.7488455879095009e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7350124096047892e+01,    -4.1369104726755222e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7636567062513384e+00,    -2.7385275494329253e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.2035209762999358e+01,     1.6712739779433243e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.5740671570861281e+01,     2.3816088784418906e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.3332578665764903e+00,     1.4423716503546430e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0562975610394112e+01,     8.5170787013225781e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.2766751191559351e+01,     2.4360067888766218e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6257079424037059e+01,     2.2836999248668450e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3538169093903011e+01,     2.3242426784092826e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9424239697915326e+01,     1.4957817866358831e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8801254511793161e+01,     6.5853962508039995e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2176787336355718e+01,     2.4908522012858226e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.6278102498316063e+00,     7.9537041388711732e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.4413082582077923e+00,     1.5498983456424834e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.1232075397255215e+01,     7.2117853490002353e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4536594521709478e+01,     8.2975398294844745e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.2280208719892414e+01,     8.1240577316295921e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.1614745227128992e+01,    -8.3103291083079675e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    9.8245233185507992e+00,     2.3639556180061591e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1827026679145316e+01,    -8.1245564044098719e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7572794733557568e+01,     2.3261162337761500e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.1107468237266740e+01,     1.5525893543334229e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    5.0203667728351121e+01,     2.2615182860472232e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.4349377510179878e+01,    -1.2098003913493594e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8919189933651136e+01,     2.4916184495851565e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2938393388858433e+01,     2.6060316719284674e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.3669400472473953e+01,     3.8825707321439404e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    7.2138205738680110e+00,     5.8798954342551342e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4584036304944885e+01,     1.1337562948611103e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6101929855770763e+01,     1.0613124695153263e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8475266037585349e+00,     8.3286809024196391e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1468473636142562e+00,     8.3287019468272611e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0076728768943365e+01,     1.4449455638694541e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3545657741890023e+01,     1.4339819562568775e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2855243981419910e+00,     7.4067515844470799e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.6838136451636068e+00,    -8.3598430197624275e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.3467187457253679e+01,     1.5433721801455922e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.0687028737661048e+01,     3.2218161347430490e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -2.6650514816400083e+01,     7.5809391338841706e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2262691183147902e+01,    -7.9915985058424212e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7999814418204188e+01,     1.5038399631821523e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    6.2143675817922350e-02,     7.7210024020888814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.5123135806168193e+00,     3.3326578729832972e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3717662061544210e+01,     2.2050957309282476e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.6357860232340826e+01,     8.3110661118997068e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2296474579105109e+01,     3.1945705699552796e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.0509399052786531e+01,     2.0587592546176777e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.4303474730732013e+00,     4.6337683430461302e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3785143503776617e+01,    -9.9518385357136867e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2845569727207824e+01,     7.2501454927549166e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.2162773109113999e+01,     4.0183743850164831e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.3061304123770913e+01,     2.1484202477025054e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2532999502522504e+01,     8.0596233818157334e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.4528983304025429e-01,     1.1692726686380227e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2100704261232829e+01,     4.9123674696200533e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.4177303802501786e+00,     9.0617737425371433e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5637605388293274e+01,     1.6683121984395009e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7212018907676754e+00,     1.3367972634193943e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7573714570640391e+01,     1.6858482456470683e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3279000105493711e+01,     3.0616108286551622e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.8467344941706052e+01,     2.3908205476475719e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.3858227299597932e+00,     8.7062932831767575e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4150287218818658e+00,     6.0453616814878424e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.3607251478193128e+00,     5.3422580214199825e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4841243593833433e+01,     2.3230020218361435e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.7032661192907801e+00,    -8.4400251130907345e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.3367658755984198e+01,     1.6764424571063490e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.7592778796976127e+01,     8.2560614976714387e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.4253886712380766e+00,     8.2930351901471822e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2188831132778246e+01,    -7.4962488193643285e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.3707001305930593e+00,     2.3207089713156100e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -3.8606824875334198e+01,     5.7121968638196267e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.6426831503102100e+01,     1.5513903894784231e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    6.2805634450009080e+00,     1.4497304163268598e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.0928360380826074e+00,     8.2309202732347349e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.9348164849997801e+01,     1.5025254912247156e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.4992457714541032e+01,     1.0490587524568227e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5469893095510674e+00,    -1.0086903648427229e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2923487008793984e+01,     1.3344664026105713e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8075301945372487e+01,     1.8055984630163966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3805242335874864e+01,     2.3857939203333892e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0799102537616534e+01,     8.3685331348974651e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -6.5073135208529351e+01,     1.9288326926645575e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3398460340186201e+01,     8.5328264448370282e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.8845148480503035e+01,     2.5672257253130618e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    8.6225302469303475e+00,     7.2846639888440050e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.4423514554830383e+00,     2.3265680034691948e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9358490851339162e+01,     7.3380157922436156e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.5281450998676425e+01,     2.0890577753502093e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.5167148887448032e+01,     3.9256465650246602e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.5045774138061718e+00,     7.9874855663263324e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.3784392750142018e+01,     1.3186929671765280e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5674273050753778e+00,     8.4772369910485650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4775245871828446e+01,     3.2029284873689994e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.8359453193183114e+01,     1.4406795822738837e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7035272782408903e+01,     9.0989856204949738e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1875186122353021e+01,     8.0109215160011736e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.6122639557963474e+00,     4.1036765352696057e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.5743358395140390e+01,     8.7374448783157368e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.6382951324105317e-01,     1.4917291618994042e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2266569640031522e+00,     8.1703982194328637e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.8902088063617795e+00,     8.3080728621124745e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2684053814713909e+00,     2.5063626534882471e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.6178873376415940e+01,     2.4457093483695084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5448465766554470e+01,     2.1753537544057472e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0043745890518757e+01,     1.8313508534731172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2129661357228558e+00,     7.4055786471539368e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.8252002889787482e+00,     1.4886060771525994e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.8537070298526288e+00,     4.2407473733531442e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6088300066299389e+01,     2.2725638751825063e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.9034630806492583e+00,     1.2719030948021227e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5387468801328641e+00,     1.4945625108451330e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.9514795475537362e+01,     1.5732483999299802e+01,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.7744176825295149e+01,     4.2118969781555788e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2699929335419558e+01,     8.3515856457137688e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5792903093356770e+01,     1.8819278949191677e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0903118686302173e+00,     8.6157931513718759e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.1102044991581302e+01,     7.6983062967942060e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.1991814280413919e+00,    -9.6589029240676100e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.8908818899042288e+01,     1.3919262851364881e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.6273726117887286e+01,     1.4898714220936952e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4837377138781976e+01,     1.0287237317489253e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9350046941010945e+01,     2.5650430275319493e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.9046880380300308e+00,     7.8600024766783572e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.5233179227776734e+00,    -8.4264625760121671e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5757942803451819e+01,     2.0949446661215632e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.3197622533484541e+00,    -8.0180863437824667e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -7.9959296552328013e+01,     9.6632054300331220e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2546754158971254e+00,    -8.3444391681378984e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0684159308679209e+01,     8.8620814419689741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8668931389536638e+00,     5.9548178629510739e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9192098245991232e+01,     3.7724232957487584e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.2533450556950946e+01,     1.3562271560967374e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5192778470620379e+01,     2.9184838081312508e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.4762445700346900e+01,     1.8436367409961149e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0607444310017609e+01,    -6.6113995521056310e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.1078903456036135e+00,     8.3493099871344257e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.3666918930745808e+00,    -7.7069427951770413e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.6710136515294401e+01,     2.4820787450431962e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3861158952446162e+01,     1.6960368056572211e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6653409683158273e+01,     6.5791963247136682e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.3081583890442152e+01,     3.0639856642155919e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6927679121894585e+01,    -7.7959850440458940e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5573903255428863e+01,     1.7268058408017972e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.4021432988710842e+01,     1.8325958098128300e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2896126444564560e+01,     9.2659087627507215e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0286569686915978e+01,     8.3904959421211242e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3665144111368152e+01,     2.3276925186846347e+04,     5.7858535816834269e-05,     2.7581612702191584e-07,    -1.4149820578231025e-09, } , 
      {    1.3411876278379625e+01,     7.8085833931172456e+03,    -1.9022488806450153e-04,    -1.5158068198287283e-07,     5.9894366977953167e-10, } , 
      {    2.5566802158107933e+00,    -1.0247389977383758e+04,    -1.1871394895485087e-04,    -2.1493118873071965e-07,     1.1158423203048969e-09, } , 
      {    7.2678043264159680e+00,     6.4099928718631427e+03,     1.8587086656980149e-04,     2.8650655957858978e-07,    -1.3561256769364048e-09, } , 
      {   -3.7256973662500066e+00,     1.5271049853540224e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3553601946409318e+01,     6.7453739174132515e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3716655129206590e+00,     6.8276008512644912e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.5301949939068349e+00,    -2.5032409340803697e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.8011908979086584e+01,     1.5511575886466717e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.8756771249763859e-01,     7.2272170606716236e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8005646762509826e+01,     2.2792865571743740e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -1.0730411576205686e+01,     1.5541626027333525e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7807464080011684e+01,     1.5543882552630064e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5594841563828005e+01,     2.3884952225876597e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1019056949800810e+00,     1.2556320886778327e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5239366363891875e+00,     5.6878900797412950e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.9318871243286004e+01,     8.0907684156370397e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.3072409478357528e+01,     7.5636661371185019e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0135417530184427e+00,    -1.0541027862346067e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2935852172236695e+00,     7.8488795357065328e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.6542302461517647e+01,    -8.5468773887425868e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.2093876002636556e+01,     8.3325793173162019e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.9946577756999950e+01,    -8.3248035319306982e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1330281546360901e+01,    -6.4405532855999445e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8415509763457106e+01,     1.0785033020050392e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.4361620761905897e+01,     1.6633109975637417e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5503366775632422e+01,     8.3021728080272460e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9683664112384327e+00,     7.1993443569325973e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8541969637306146e+01,     2.4355465488883397e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    3.3490850854958552e+01,     8.5689090722336005e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2558258248995777e+01,     7.2009086700450653e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2283116952524118e+01,     1.4927953216207607e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6428622882561580e+01,     1.3991892778316893e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8800723014374224e+01,     1.6295014801912610e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.2531202408581636e+00,    -2.3499138213783403e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0819589833313472e+00,     7.1500731414247894e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0422233540759594e+01,     1.4914998994468737e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0006770868028489e+02,     1.6681655722856383e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.2600590782256109e+01,     2.4426458794296384e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    1.4406107616432907e+01,     6.0814100978914403e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    5.2422134024834044e+00,     1.6344562924360042e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.0797391606942995e+01,     3.2750892463262117e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -4.3924637475303303e+00,    -2.7849282153060731e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.7433575872115235e+00,     2.1671054250787754e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.3395129987254357e+00,     1.3184628471823869e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0463547391159089e+00,     1.4924755986191740e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7478524364156689e+00,    -1.0622382892967516e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3353537092527292e+01,     1.4862188953191937e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8549399518574532e+01,     8.3338119972605855e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7924649976962375e+00,    -4.8632660405104525e+02,    -3.7343187314139455e-04,    -4.3870363367562522e-07,     2.0095090749677264e-09, } , 
      {    2.6545187257430271e+01,     1.3988719354665707e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0365952192496879e+00,     9.4547947122301321e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.4067622475374090e+01,     1.5697858812006043e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9349769691073313e+01,     1.5571604685179362e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    9.8505584856101454e+00,     8.3232959305785480e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.2810420384263335e+00,    -8.3340869186683522e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8193838343968920e+01,     2.1872735526192264e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.9871785588599344e+00,     6.5147619748998113e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2578724051646102e+01,     3.0457912340384923e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3728601503825507e+01,     1.6278439631835570e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.8398730787899105e+00,     8.9349577562242648e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.9419355237829885e+00,     6.1335125190742974e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3773638965448026e+01,     5.0317203449967128e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0554573911056835e+00,     1.2240852178097462e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5260784980988085e+01,     7.5913513510520352e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.7523321529921212e-01,     2.3856026603465565e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.4496375597515261e+00,     1.6153037666825539e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.5379488024467179e+01,    -3.6082627396572207e+01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.6798693762666037e+00,     6.0257043542928909e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.5486628291260665e+01,     2.5612080658974483e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -5.3935007236000416e+00,     1.2902941190598167e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.5403217616046465e+00,     8.3533108725393940e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1160188409020234e+01,     7.5890501511106231e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.1732764601557637e+00,     8.3537192143046850e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4674301629044514e+01,     3.9478634354400900e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.2423511895848280e+01,     1.3030158359642337e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3289768017170354e+01,    -8.9443797712510059e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.2178418506527933e+01,     7.2170450136326044e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1496374781723572e+01,     1.5110466097669896e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5261274639426091e+01,     1.3515516291644411e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5318603083389005e+01,    -1.5419111139677625e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7154974880941420e+00,    -8.1065227204691600e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8137111492506477e+01,     2.8750997147361822e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3374754307288166e+00,     6.8367896391798085e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7489860404190310e+01,     1.6550964933522060e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.2916639325783414e+01,     2.2650878960917758e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9456793108495241e+01,    -6.9017226617739443e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -3.4932762680805727e+00,     5.7533126750536794e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.9935732449563147e+01,     8.0898020132359761e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4942088288721941e+01,     1.5529600094668516e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.8716151926982874e+01,     7.8180975767944055e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1389757017279095e+01,     1.4486994882013110e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6887520942572124e+01,     3.4229676337507488e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    7.8128294521833794e+00,    -8.6081306683785351e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.6417652498672020e+01,     6.5084949017778717e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0902923039055015e+01,     2.9970527704740412e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    5.8738336223822483e+00,     2.3013539513374071e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0145527003583666e+01,     2.9971232336656558e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.1319594844958427e+01,     1.3802447977987331e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.4824118314434571e+00,    -5.0434518242136147e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2075163268350472e+01,    -4.6899338653426639e+01,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    9.9797633611571701e+00,     4.3942163556613468e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.2021982564473056e+01,     1.3228598459289184e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6194185441135776e+01,     7.3096640698263291e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8086463841228575e+01,     3.9467169101798230e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -5.9404041498325597e+00,     1.4540570559566611e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4807033503641209e+00,     1.6628996915836640e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.1379375620524610e+00,     1.5555908485295073e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.5868876778003305e+00,     5.4198102824321068e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3325253641853184e+01,     9.2386862908946969e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.8305602571424899e+01,     2.4486635388452523e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3948922580308292e+01,     1.3545386705995243e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8455494459894503e+01,     1.7269481098951725e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7519808027135078e+00,     1.8849227551366296e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0437673355783488e+01,    -1.2569674860491614e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5139160834816217e+01,     1.8374083171161012e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8807236269081994e+01,     2.2948891787985885e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8886759647840872e+01,     2.2921110465279704e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.0267212955166350e+01,     2.6065278222823606e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    8.2249966171629971e+00,     1.6685768782657156e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.4106564359387015e+00,     5.6666978914573477e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.1186843103760289e-01,     6.0803065704972341e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4045365361102315e+01,     3.0417386093020134e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.9441887365739969e+01,     1.4755187097344602e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5705154276906018e+01,     2.3455014518404467e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.8702938146826142e+01,     2.4431061194179205e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8516128907229092e+01,     1.5666707216867066e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    6.7130914525835292e+00,     1.2507154559176461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0021592528836456e+02,     1.4412583702428259e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.9568481018858265e+01,    -3.5133424023739599e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.3496682381581637e+00,     1.5025780393403536e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.7704641591046766e+01,     1.0291513015078666e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3716917353761630e+01,     5.9574409817165306e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5102321288016443e+01,     2.3221808221581850e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.5867796569946293e+00,     1.3187483199635335e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7594269031069828e+01,     2.3848903809472999e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.8535682152448725e+01,     1.2543139701496766e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.0399693871519730e+01,     8.3083412812261777e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.1969210318703531e+01,    -8.3490415680207225e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2436033618435033e+01,     1.4888138314309694e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.5203596309964666e+01,     2.3141558306926117e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3271608387706960e+01,     1.7994180112666294e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.8860405043826809e+00,    -9.7304217106369815e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.8688089519475417e+00,    -1.5256832128988026e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1359510306290463e+01,     1.6163583708458280e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2876553397470776e+00,     8.3155285596792655e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4349535853836786e+00,    -8.3418542895676364e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.9936880822862936e+01,     4.0528476247894061e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    9.1820177933715090e+00,     9.6808665685923261e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2401716228517451e+01,     1.2041648190141479e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.1147291610390639e+01,     4.0529180879810221e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    5.1422950312615470e+00,     1.3072573441938139e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -5.3963811160700260e+01,     2.3926802644791584e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.4851592412444958e+00,     5.1697389657825875e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1329257861274604e+01,     6.6703635667573317e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2827374677530379e+01,     2.4346949977567951e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0229527193157981e+01,     5.9724040284053708e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9588100758803098e+01,     9.3096795963076875e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.4019013701582104e+01,    -1.8779228782928149e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1008840893887154e+01,     9.0999486627124634e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.8684192369928979e+01,     8.2958763995693516e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4497062195962762e+00,    -9.3657677228764296e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.6313680854739101e+01,    -7.9717530823270317e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4079561094203509e+01,     1.4942832638346510e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7888674567197569e+00,     6.5813460531425435e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2157745734158791e+01,     3.2863237477272378e-01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9751388496387701e+01,     4.0511907883044551e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.5672527084786905e+01,     6.6710749122242087e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3600843499047604e+01,    -1.4038078287972317e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.6525112816307264e+00,     7.7849921260224373e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5842364331293652e+01,     2.3855236379264526e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.5959882339942013e+01,    -2.2517990236624555e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0478265649289238e+01,     1.1788327890848215e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8970582049264877e+00,     1.0529082888440944e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5319054905848528e+00,     6.5731360418948143e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.3470145163100131e+00,    -1.2566639307532805e+03,     1.9586472716825521e-07,    -8.7266462599716787e-11,    -1.4544410433286028e-12, } , 
      {    1.8688184510556869e+01,     2.3803931620357118e+04,     1.4774252154045787e-04,     3.8573201821297136e-07,    -1.9917115647341956e-09, } , 
      {    1.4405738324793976e+01,     1.6694613477704344e+04,     1.4735358315848355e-04,     3.1884183829797577e-07,    -1.6682875099292129e-09, } , 
      {    1.6969525216274135e+01,     1.3624031356890391e+04,    -3.2506784952774190e-05,     9.7861774712156735e-08,    -5.2888809285920396e-10, } , 
      {    2.2548227406416661e+00,     8.0409437382629858e+02,    -4.0444438281954544e-04,    -4.7506407798242305e-07,     2.1848177020569347e-09, } , 
      {    9.7377315244649232e+00,    -5.9107132675661705e+02,    -1.5456735087326034e-04,    -1.8321230212549640e-07,     8.5812021556387854e-10, } , 
      {    2.3020718941900661e+01,     1.3766066728378566e+04,    -4.0347064132842842e-04,    -3.4137098461503148e-07,     1.5365151514036408e-09, } , 
      {    1.1708368606018542e+01,     8.4369097286624019e+03,    -1.9298670192099417e-04,    -1.5092065663741027e-07,     5.4523116204940613e-10, } , 
      {    7.5159826670228389e+00,     5.9912160020542869e+03,    -2.4396446585132548e-04,    -2.2628164663285614e-07,     1.1106984471483244e-09, } , 
      {    2.2423556680770933e+01,     2.2581024736727904e+04,     1.1712174131664347e-04,     3.4919704098097409e-07,    -1.8193118197316447e-09, } , 
      {    1.9588418106769865e+01,     3.2342172510716868e+04,    -1.3256221697483553e-04,     1.2432271150164275e-07,    -8.1458394700024244e-10, } , 
      {    1.6564621192657278e+01,     1.5333204824245493e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    5.6557706046686409e+00,     7.9099389351456912e+03,    -2.7764085702238487e-04,    -2.6202569882549135e-07,     1.2315673459857430e-09, } , 
      {    6.7030410248487566e+00,     7.2452412688734239e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -3.6110055436188393e+00,     7.8363369759611360e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.7118735075663235e+01,     2.3558246182165149e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0715567918186061e+01,     1.1832098724405178e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.3841628798628527e-01,     1.3906236061395986e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.0334021852053663e+00,     1.4896422496734453e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.3191327238665256e-01,     7.1986330114657212e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8843392595708409e+01,     8.4000370459945170e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3370252040455028e+01,     1.2638804180048044e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.1537474151383886e+00,     2.4202589680303709e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8861685309522485e+01,     1.4929865816075930e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6020862080948714e+01,     3.0459830827234233e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.7729377408484599e+01,     1.5543147536244236e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.9313330418326284e-01,     9.5559973877790526e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1161817879964268e+01,     7.0405960270186151e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.5275208697874440e+01,     1.6498123241655252e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.3144065391214980e+01,     1.3657479218887089e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1430495080833829e+01,     5.3270638057755787e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1667113536711806e+01,     8.9853324991262798e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9602049313180629e+01,     8.0192829987614414e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    5.9034386103376422e+00,    -9.1305000590016971e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.4798103049000595e+00,     1.4264809211912856e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0776240150201186e+00,     7.7805659329062155e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4015370371894759e+01,     1.5541590828427563e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.0818842250207039e+00,     8.7973803725685430e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9561798086101856e+01,     3.2681206844291701e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.9273201946663998e+00,    -9.0437495343353457e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.3892194657532201e+01,     6.6826580808742938e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8671184853261238e+00,     1.6028363918734482e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9962938688548267e+01,     2.2821251505153999e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    2.7469273190785469e+01,     3.6016230770507436e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.8840033425742682e+01,     3.8788193619482903e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0035463068068809e+02,     2.2128502856743871e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5784645443140265e+01,     3.1593159462161006e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.0221586570371386e-01,    -8.2669128782008211e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.0524598414209329e+01,    -1.1155949616661680e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8633586024263153e+01,     3.0202769555242143e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.0994169317870949e+01,     1.6816642456838548e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5522637231039015e+01,     2.3843059781194985e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9902856921236616e+00,     1.4115785527132288e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.0065690020079707e+01,     6.8886381202694511e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0943331426146649e+01,     8.9363804471580170e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4360687313762142e+01,     2.9990526416214561e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3260198176042248e+01,     6.4537580755886238e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -6.1767944813678049e+01,     1.4443667759005120e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -6.5685997262119145e-01,     1.5542361043719353e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7857724697123105e+00,     7.0125576774609508e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1857682117050057e+01,     1.5072642651102949e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6855981046986805e+00,     3.2164088722825098e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -5.6414493121914600e-01,     1.1299705716687240e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2806897219117634e+01,     8.0006636555944742e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4674253067807806e+01,     1.8387201082359283e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5049801177578637e+01,     9.4482357566309984e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.8702695865602941e+01,     6.6411122104991427e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3692810055306474e+01,     1.5515524327918714e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.5545392867269747e+00,     8.3904699710460809e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.6252793641466706e+01,     1.3432534325594414e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.6800484358462683e+01,     3.0791262238389776e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.4705087271648615e+01,     8.9619300419265892e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4246659509903523e+00,     7.2981740819696024e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.9812248503739426e+01,     7.2110807170840844e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9538809434367383e+01,     2.8352410735568541e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.7652478922701526e+01,     6.6809271123970557e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6982914888153051e+01,     1.2487628781306747e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4952962107101818e+01,     2.2678820268447715e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.8816932673207596e+00,     3.1339940019639493e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.3365163617959510e+00,     8.1371756428278577e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.1990758181943097e+01,    -4.0392421854557488e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -5.4013227971203349e+01,     1.0603032353418150e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.7785412845134978e+00,    -5.6136272641691103e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5076242925567961e+01,    -1.8225659481064733e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9392774092107263e+01,     8.0155239158839977e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0769904349172322e+01,     7.1191452976583250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0298256749141299e+01,     6.1796333653675110e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3484541519232426e+01,    -7.8735373050580174e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.2285789497315534e+01,     7.3023291675370028e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.2435031557725464e+00,     7.4373159015191695e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.6274343691699443e+01,     2.3847172840995765e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.9906644225760104e+00,     7.7857034714893143e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7371942827630580e+00,     1.4899027593796574e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7411171202543446e+01,     2.0714160045863078e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.3345180783669093e+01,    -1.9528379260050256e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.6710635771156717e+01,     1.6054814399349467e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3235887390554968e+00,     1.3760003243773233e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.7206648051109568e+01,     1.6658349251647964e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6013965713568464e+01,     6.5871779712465532e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7963760549072990e+01,     2.4342018945310356e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8883615958164235e+01,     1.5527006546467344e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.9849894452774166e+01,     1.4218223949022313e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -2.5810254323600361e+01,     1.5527833851622396e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.8060744992008452e+01,     7.7130030779958934e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.3010573748208156e+01,     1.0575483554314091e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.1209252839410297e+01,     1.4218928580938462e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.4188026066917313e+01,     3.9361230907375844e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.3136590084609077e+01,     1.5447836722281776e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8035988089055913e+01,     1.4436762229099524e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -8.1137037986230744e+00,     1.4216275065038979e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {    4.9473002525427034e+01,     1.4220877464921799e+04,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    3.9441431549295247e+00,     1.5793788648848802e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0705234108373539e+01,     4.0200241943973015e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.8364047455253928e+01,     1.4411205049909098e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {   -1.7594260777052675e+00,     3.1711514965067719e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -6.0341505591971565e+00,     4.2341884177540105e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.8682197171887537e+01,     2.1953707934414517e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.3012742244018654e+01,     7.8423637368531608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.4743510406428371e+00,     6.5677063368569397e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.4135901689692290e+01,     9.0148744038837176e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.3907385105670361e+01,     2.2862755349762523e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.7847531788218433e+01,     7.5965755091108185e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.7123621620141192e+00,     7.2103649836549621e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1819086556155936e+01,     1.6913332980856350e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.8678313333968637e+00,     2.6087903140599633e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.6493128807220302e+01,     1.9734354101232293e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1201037169564167e+01,     7.1978535300176300e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.0959363885417957e+01,     8.5675808360109240e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9060428963822332e+01,     2.3147966297475446e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.7796205099822302e-01,     8.4482435357422619e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5439152725350171e+01,     9.3706834956188759e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.4080934482249194e+00,     1.7389301705606362e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.0578712248907127e+01,     1.6648522693706356e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -2.3142523104044127e+01,    -1.6666243004787444e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -5.1887845354710407e+01,     1.9800945872208840e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6667814270809579e+01,     1.4498726854202348e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4477209284622148e+01,     3.4570066339085797e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1874035433972594e+01,     8.7194111943750286e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.3770139407046145e+01,     7.2627647311728997e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    4.6946990950335028e+00,    -1.7455659766268318e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2178122234094136e+01,     1.6650771449694796e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0339082498670722e+01,     2.9864334032034676e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.8487851693163890e+01,     1.8620204439702116e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4250439829634365e+01,    -1.0682278933919570e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    9.8830941833008819e+00,     1.5463193824328004e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.0769959855161176e+01,     1.5232216613558527e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3752135844250644e+01,     8.3283733254190138e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7578414362974524e+00,     8.3290095238278845e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5509680405076676e+01,     7.2074514658062417e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -5.3926125846511866e+01,     1.9328141828543485e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9091361999680394e+01,     2.3147578155681913e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.1765469531568286e+01,     1.8954265683173846e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.0588401549729781e+01,     1.0959635502022122e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7941137514202097e+01,     2.2582015519460059e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.7324119602266677e+01,    -1.7584781725990788e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8135775825284504e+01,    -9.9919977651382724e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.6361627459116821e+01,    -2.4840452655082299e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0393703934563172e+01,     1.6520312011163362e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.5417258280977791e+01,     9.5100555336438247e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5514127254886978e+01,     2.3887445222829243e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.9060642639253871e+00,    -8.8925646598551357e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.8669711856223685e+01,     1.6188334871793006e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.2887300794388011e+01,     1.5557674728341190e+04,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    8.5872633568159330e+01,    -1.3241847385521337e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.5307210239089926e+01,    -1.3241713114506806e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.5429829540565166e+01,     3.9397279323779447e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.9132976472027273e+01,     3.0870892042185595e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.6919844896561330e+01,     1.5578091360918517e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1401277793640309e+01,     8.3244260525912250e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5438725265166482e+01,     1.5565045729580228e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {    4.2515777768972491e+01,     1.5520462850383361e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4613764174772806e+01,     8.4184202837051162e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5711982868200852e+01,     1.4112657022035824e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.0511198722086572e+01,     7.6519324392971612e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.6724091602255733e+01,     7.6954528073203110e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0008486188550773e+01,     9.2572735027054478e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.4335088331577666e+00,     2.4977437775487517e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -1.1275817245324735e+01,     3.8477264036601555e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0939878119901827e+01,     8.1732582702244290e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0685414594171707e+01,     1.5472924446123037e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    4.3841355594022060e+00,     1.3886123748152638e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.6780970861208544e+01,    -1.7452373442520584e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3762200030170055e+01,     9.1611340298825708e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0447253408199906e+01,     6.5224769018461220e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4923362111022261e+01,    -8.3195178962232822e+03,    -5.8706656478729573e-04,    -6.9538411622277964e-07,     3.2705046113968180e-09, } , 
      {    4.0019659053004411e+00,     3.1083431037179889e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.6271870468369869e+01,     9.9747247683726055e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6828787379967046e+01,    -4.8451400280746384e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0473839088906281e+01,     2.3825045048732100e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.4109227662981601e+01,     5.5240958182588884e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.2056641672052811e+01,     2.3512206172367420e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.6112144431804268e+01,     1.5579514051852269e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.1201420938286603e+01,     3.2657592458735537e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.2089718340431133e+01,     2.5761299501999467e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7344874621587120e+01,     1.0438412716230552e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1129595195409134e+00,     7.7706657995240203e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.9595911764045631e+01,     7.8420418180291144e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.0534779188886446e+01,    -3.5231595807488247e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3869169176493983e+01,     3.1087586122747289e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6934727333885299e+01,     3.4165241987693633e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.9160036259533175e+01,     4.6049179478485639e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.5724173200270357e+00,     7.7720884904577742e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2959172705573854e+01,     9.3565095250621780e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6728044378892086e+00,     9.4374199298033272e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9823953200342885e+01,     7.9790881528632926e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3776716559096560e+01,     3.8863444680553614e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.2231268129528740e+01,     1.8045995224062161e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.0496123316585688e-02,     5.0731899786122658e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.7153965065644883e-01,    -7.3568534185377621e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.4205124275891521e+00,     8.6541161697123443e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.4084345503373967e+01,     3.1053767642326784e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2491746076109314e+00,     4.3373704261865851e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0987964993154151e+01,     3.6611416541221588e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.5524400479122704e+01,     7.3905954697156612e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6265341024970677e+01,     2.3899831648015504e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.7878973322179192e+01,     1.3658183850803240e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0516924674326166e+01,     1.5564795543426522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.7835215240598039e+00,    -8.3003397022061636e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    6.7526463988636056e+00,    -8.5202072064190434e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6699050412707727e+01,     2.9513424701191641e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.9439986233998681e+01,     1.4774761294051679e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7470108486614265e+01,    -8.3210032184630600e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.4459871185862124e+01,     2.1156381554150903e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0806455787337543e+02,     2.3895718588214724e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6577361738548902e+01,    -5.6888783252398225e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.1689857760570490e+01,     2.6458299192516592e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -6.1158403554784737e+00,    -9.4831348857864668e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.7782443564804842e+00,     1.4895259035180221e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9401814560998609e+01,     2.0855894891979511e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5198531956738982e+01,     5.4247413146896997e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8892082246432793e+01,     7.9159667797569609e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.7068586639354251e+01,     6.6291264417353814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2234802290062152e+00,     7.7047930667481651e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5475076647262966e+00,    -1.1314893059126725e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1623803355479726e+01,     6.6544473857843823e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3857252238145204e+01,     9.9184131532960964e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2854457772109562e+00,     5.4873360563884125e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.4212839106626518e+01,     1.5540100774082310e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.1289891610432520e+01,     1.5545407805881279e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3360946516740084e+01,     1.7303714642494240e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9348879089467928e+01,     1.5546567581794930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4387823002049124e+01,     8.9549214668852928e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7512364415346138e+00,     7.2102495735452094e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8300877129666109e+00,     8.4236089923234686e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8887785866312758e+01,    -8.2327768472958414e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.6893373718478832e+00,     7.2020652546148740e+03,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    7.8849865483241484e+00,    -7.3411442973400808e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -8.6834430362645548e+01,     6.0403471143606112e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.8543450132240775e+01,     6.5722487690414473e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4618180748301024e+01,     2.2615541889981036e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6135584458770506e+01,     2.3296818572237768e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8000785016900231e+01,    -6.6921731960376410e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.7252530018424238e+01,     2.2694684005467694e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2311962998767999e+01,     1.3658542880312041e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5503627591142390e+01,    -1.1699854894514474e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4753529671465806e+01,     3.1106838487941441e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.2602085772093199e+00,    -1.9207463264574699e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.0773279700472180e+00,     1.7455424998130831e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -8.0895161378061459e+01,     6.5614824067033187e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0018093131087298e+00,     6.7086141555427776e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9603247155300263e+00,     1.3513215091702999e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9352044027405984e+01,     2.2763376110939276e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.5233473234476676e+01,     1.4891904799804008e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.0976460056419164e+00,     8.3426985875823211e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3736421552343288e+00,     2.3599741278163674e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.8792413958629304e+00,     1.5542358455941783e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.7197811107943071e+01,     1.5543150124021806e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.5876937102653415e+00,     3.2222679044360939e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3584147838655760e+01,     9.8140342544679588e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6511789920370930e+01,     3.9374385102689121e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.4105047238495036e+01,     8.2108082562959426e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0431702330289792e+01,     1.3957552500011578e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.6926355656925054e+00,    -6.8782616211695688e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.2900966491390790e+01,     5.8622758631830038e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1648008645222539e+01,     3.0552802199386027e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5191099802131882e+00,     8.1071443481436763e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.5447187587478988e+00,     1.5520713036537067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.0229281176918530e+00,    -1.1977213840327622e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7622410742126615e+01,     8.3651330872894905e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6604842639866133e+00,    -9.7490893738756165e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.2125737059426875e+01,    -1.1133368846728445e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.0790820535334973e+00,     1.4930179188935546e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    9.6981172833431216e+01,     3.2200130425677969e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -6.0481844620962349e+01,     3.2200143852779423e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.6571969600121452e+01,     1.8767104140854000e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5332237520399715e+01,     1.9506745414366829e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.8642669999140878e+01,    -8.2599993191518697e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.9162482718525116e+00,     7.7441546731348708e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.9632482788183808e+01,     6.6563962697677189e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -6.1955131145337816e-01,     8.3643465947646073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7684323608526100e+01,     6.6571009016838671e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.4364958120459150e+01,    -1.7979259674306893e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {   -1.4605905809157605e+01,     3.2182864142463026e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.7575950293668726e+01,    -1.6674655846012571e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.1237711882415667e+01,    -8.6567191936524268e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.3371856709279069e+00,     8.8723907232244619e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1470556367869303e+01,     1.6561822875369111e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.4814714903164043e+00,    -8.6741048927441625e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.4047062724049129e+00,    -7.6811961698218765e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.2329057893839916e+01,     1.6253545168297054e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9675090492314276e+00,     6.6173944695461405e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.2203046204817952e+01,     9.0348978449875794e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9397800220486953e+01,     1.5431420601514510e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.9015457000449555e+01,     3.9989538492421387e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.5206418772625895e+01,     1.3579950641458268e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5128884398102755e+01,     3.0614935349258474e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.8120823885140886e+00,     1.2509920220905180e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6207516469407867e+01,     3.1051727152508429e+04,    -1.0145177493310048e-04,     1.6063952257714078e-07,    -9.9061979461111474e-10, } , 
      {    2.4637995309835066e+01,     1.4394344303261529e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    1.1142592651393507e+01,    -1.7767119419343255e+03,    -3.3718953284101068e-04,    -4.0353234036585289e-07,     1.9438071249037621e-09, } , 
      {    6.5984930265783461e+00,    -1.7091490870240048e+03,    -3.9901868747014424e-04,    -4.7634049544204832e-07,     2.2929699380388498e-09, } , 
      {    1.3223217659839621e+01,     1.7251903377005801e+04,     3.3283551134631064e-04,     5.3845821796156986e-07,    -2.7009891320606354e-09, } , 
      {    1.0167876207429963e+01,     1.5995299466577679e+04,     3.3816327433212770e-04,     5.3722543373324450e-07,    -2.5921096755570561e-09, } , 
      {    6.2707501998208937e+00,     9.1327614181186509e+03,    -2.5205404269363512e-04,    -2.2438883705906827e-07,     9.4810648291461947e-10, } , 
      {    1.6052076765599985e+01,     5.9236662678120983e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.0568947972580716e+01,     8.3658976727497975e+03,    -4.8408922402585326e-06,     6.8079330912021156e-08,    -4.3303073183022644e-10, } , 
      {    8.5120734779480252e+00,     5.9452052775890002e+02,     2.8250695821658283e-05,     3.7020469652260392e-08,    -2.2902113481933369e-10, } , 
      {    8.5983248536610866e+00,    -6.6208338266922078e+02,     3.3578458807475305e-05,     3.5787685423935058e-08,    -1.2014167831575411e-10, } , 
      {    2.2364492911780435e+01,     1.3403400095169261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2912425917156636e+00,     1.8330429215538767e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.4581801747428997e-01,     2.2048879766498772e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.3525189530828069e+01,     8.3304166514490753e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4519483938781260e+01,     8.3269661977978249e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6134078535414183e+01,     3.9454014906484954e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.0274944918795775e+01,     2.2637372449646406e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.3341564261402903e+00,     7.3325411686510188e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.7998069199311523e+01,     3.3314413382535720e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    5.3125685156442117e+01,     3.3315118014451873e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.3094670260388689e+01,     1.5543917751536026e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    6.4535044331263922e+00,     7.3898908377995112e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2745104637053917e+01,     5.8707066463398169e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -3.0238977563612156e+01,     1.9113525290765490e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6390237196031521e+01,     1.5651786778507671e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    4.4109794642312281e+01,     2.1502503185190464e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.7711485765567225e+00,     9.8205932100670925e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6465806205846050e+01,     8.2204709817374751e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.2212652550135630e+01,     9.5856589106726115e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0135921786881923e+00,     2.4044946763937947e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    4.2349667201224745e+00,     1.6752942823124689e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2641706696900913e+01,     5.3500758051896821e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2654666952069578e+00,     2.8333489457286738e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.9565617689106375e+01,     1.5541772628065579e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.6642670192912377e+01,     1.5543735951898010e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4352292506872402e+01,     1.3737684995798698e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3624799790561596e+01,     3.0576645700611782e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.6207628764641782e+01,     1.5894088530219191e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7018969343150438e+01,     8.3155372293101700e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.1715982898713250e+01,     1.5303864878594322e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.9255935990555848e+00,     7.2144174688081603e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    3.2291537002087978e+01,     7.2137082619085268e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.1413823258477237e+01,     9.0057008754835515e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.0188645110849510e+00,     2.1464153568845453e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.4728611209036778e+01,     2.3809312564732798e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.0245900770212044e+00,     8.3418456199367283e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8668530921623310e+00,     5.7221610799147029e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.7284421323792799e+01,     1.0625574258550394e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4778510072122614e+01,    -7.7184327830337716e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0019337337154315e+01,     1.6186809618541789e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4936687562261806e+01,     6.5895580499221278e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.8830297257992910e+01,     8.6782946963836075e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4253677274931356e+01,     3.9411570172270855e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -5.3283700181294282e+00,    -1.0720598670236537e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.0954783823450706e+01,     1.4940755095562808e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.0186787225647622e+00,    -2.2154707647977352e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1644059305436052e+01,     1.7081056741298584e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    7.6524156009062185e+00,     1.5538940998168659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2969077385734792e+01,     1.7126430826700795e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.8925011813075855e+01,     8.7008366015158517e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2380564795305502e+00,     8.1179879730718267e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.0346442095922956e+00,     3.7106555655766188e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8544677290839758e+01,     1.9628036485235261e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8728115556605982e+01,     2.4437396492620628e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.4592118489449426e+00,     3.8783591219600086e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    1.0045016653013432e+01,     1.5010006678814043e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4849802889225323e+00,     1.6647073567991411e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6350086057140448e+01,     1.4722372072290504e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.2225067518443575e+00,     1.6667692130502390e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.3930119690390207e+01,    -3.9532216963439896e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.6793219634458211e+01,     9.6406635249008777e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1560610444573040e+01,     1.6661185618856227e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8573508540058512e+01,     7.2178761571714795e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1449166999476432e+00,     1.6653580079637573e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0751398785905199e+01,     2.2614830544514160e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.4736415726544667e+00,     1.5478764566048239e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9718408966921848e+00,     1.0906684746994251e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8328135380661728e+01,     3.9416829836903220e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.6685074364484741e+01,     1.3908537261337398e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7545693665501467e+01,     3.7513529505935956e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.7398178130588008e+01,     9.5876077946559471e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.0596489698632293e+01,     9.5830053947731267e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.9355235178191386e+01,     1.4897173708170583e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.3965686911704040e+01,     7.1825826378445963e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.2171005340129238e+01,     2.3867642944995918e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.7465295803385629e+01,    -1.5786094646773104e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.8290279923735270e+01,     8.3501708185819825e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.6720763476551454e+01,    -8.3072120306649176e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.9554151298316675e+01,     2.3775885740727455e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.6290219847724757e+01,     1.6146789001417854e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8350119779770013e+00,     1.6685716338704187e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.7877525365660560e+01,     2.3442212639049263e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5815687498052995e+01,     2.5616683058857303e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5512126011653578e+01,     3.3772221018000651e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.6329145743268139e+01,    -8.6595792444439921e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8202039735142281e+00,    -2.9465822171905438e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1916667700938195e+01,     1.0170954363804376e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.8135925964852220e+01,     2.3814357815941665e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5556759081988112e+01,     2.3385124740385683e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.3199653108429214e+01,     1.5056433315762231e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1572303866419297e+01,     7.3089804330583638e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3941434984818088e+01,    -7.5583972287519246e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.9041090323358176e+01,     7.3560647356994796e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8834627435307681e+01,     9.7272932055405181e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0981925680944283e+01,    -6.9300896437063811e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3339874159130584e+01,    -2.4980040799427596e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    2.3100273680133427e+01,     3.1064178671985737e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9616918744616534e+01,     2.3875248484214571e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6181870393917418e+01,     3.9742786624466164e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    8.9938899721964809e+00,     7.8348922838348308e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.3746126281605129e+01,     5.9574544088179846e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3076723904460461e+00,    -1.8347150395668598e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -6.0017639438384435e+01,     3.8941817365806946e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5301981362032738e+01,     7.4427905251117663e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.2169356121548226e+01,     5.3131406019977176e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.7326580275998467e+01,     1.0937093596889878e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.8165849616337852e+01,     7.1991424269989475e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -7.4452495866981891e+00,     3.2826143524332823e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -3.6545031673360434e+01,     8.3425968313728608e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.2413802225802502e+01,     4.0541626238151466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4476994785514288e+01,     1.2065670284812892e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.5325917469883272e-01,     1.4299257031088975e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.9826749474102847e+00,     6.0950250789229804e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8309375145907914e+01,     3.4622697307200477e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1385946061312580e+01,     7.7437550010004870e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2176838274215619e+01,    -1.0816711256855342e+01,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.9213110361848976e+01,     1.3896720946232639e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9045572687432335e+00,     1.4162186193005438e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.5821004749719965e+00,     2.3871222057447536e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.2206061129607608e+01,     2.3871669371762953e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1779458788203604e+01,    -1.5310597554658834e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3009044927346476e+01,     7.3265076942082669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.9801136140443916e+00,    -9.5753171886127329e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.3414306984885657e+01,     2.5221360889222902e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.7970559052401523e+01,     1.4381740324358690e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4331873059589661e+01,     2.4992107748313101e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    2.8790297613234966e+01,     1.4464174147120289e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {   -6.3359212566303171e+01,     9.3066022814291500e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7718443330369180e+01,     1.7523237018996217e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8357288933726132e+01,     3.4558262957386622e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.3455864691415563e+01,    -1.1149875887739072e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5800336417251376e+01,    -1.1142695297563041e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.0417410136156803e+01,     8.4152267583248704e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.5777879372099495e+00,     2.0272379983373830e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0752904179631944e+01,     4.0135807594159152e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -6.2627728491056686e+01,     8.3296396952127507e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -6.4197244938240502e+01,    -8.3277431540341495e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.4534261941880738e+01,     1.4347943637388382e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.0633518185349500e+00,    -8.5731857793691452e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -2.1711146206244784e+01,     7.9514029389297611e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5633031878020763e+01,     7.3975343693150298e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.4385341698553006e+01,     1.6645892861390173e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -2.6949152553690006e+01,    -1.6668872837103627e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.5559197165127431e+01,     7.1404752056361776e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4029169160450309e+01,     2.3363794832407832e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.3221760199717611e+01,    -1.7983862074189718e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.5783122271517161e+01,     3.3920736113144421e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.1931423669735210e+01,     2.0980613515869765e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2890074853304846e+01,     6.4835835713847327e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.3812832506336132e+00,     1.3120194122010013e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0873625732164175e+01,     6.0008188566898380e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2669853435482187e+01,     1.7286401779759319e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.9544094428478360e+01,     3.1716117364950547e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.6162116021596518e+01,     1.3148968655493358e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9152721934402759e+01,     2.1521531893226907e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8940863292352724e+01,     1.6608931448966156e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.1737512212933190e+01,     1.6854726390259224e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3161887264169376e+00,    -9.0683326981362789e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.2848085343527892e+01,     8.1083252476302932e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0365206424298165e+01,     9.5161281621626276e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0684412913341447e+01,     1.5599842188645376e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6098948741181410e+01,     1.8780222052052268e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.8843562360683350e+00,     1.5062333369847462e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5392692998101771e+00,     6.1728695832317919e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.6729193692997939e+01,     5.5356930186341742e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.4410750106481288e+01,     9.9574517716069386e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6163984071892180e+01,     1.7427677045118424e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5206798698263341e+01,     2.3385477056343756e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.9120091200245497e+00,    -5.5994411194873646e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.5213642892916084e+01,    -8.3317506293233619e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5352211287751736e+01,     2.1107181015556067e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.7874769898470419e+01,     8.2646036491160557e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0523879666296551e+01,     8.4099940112919285e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2204637722320086e+01,     8.3260970703142739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2016642783646510e+01,     2.3853813688330774e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.7861389813379507e+00,     5.3504700181995668e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2635356408768059e+00,     7.2516021086769124e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6252359369254982e+01,     7.2780525892918995e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5886266397445667e+01,     1.4293617286459765e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2620980358296812e+01,     2.4464808410641395e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.4200906952459647e+01,    -1.2445482212055826e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.7792800810333000e+00,     8.5110184185037251e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.4712160081555989e+00,     6.5489955184423270e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.6687526964752003e+01,    -1.0194567271808514e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2133963470943200e+01,     9.5384072560611094e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.2348472128402417e+00,     8.2337738569234316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0164610629181720e+01,     1.5620751176874221e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.6843114416744001e+01,     2.2146558513750460e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2306741499261769e+00,     1.2289656882652291e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5380049801414550e+01,     1.6217462540900488e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.6360314824138250e+01,     8.3701610582390022e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.8330814504941806e+01,     8.2016099926417155e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -5.8113675456321854e+00,    -3.1779491849990600e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.1145387298831917e+00,     5.0912236625529749e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6956180641104972e+01,     1.6015957353003090e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1315625238566760e+01,     2.3917846380478393e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    7.1428119973666018e+00,    -3.4662030381984005e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.7781414854244467e+01,     1.6548350360721026e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8354310043684670e+01,     1.3484349436990282e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8201308113791580e+01,     7.2361041188030731e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7918356197265325e+01,     1.5469166630259628e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0278619152782134e+01,     9.8255242423246873e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8430749650305094e+00,    -8.3559213866865284e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.1367336673507289e+01,     1.7834368215542163e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.8364640491159740e+01,     1.5631796538850645e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5145645507768545e+01,     2.1499849669290979e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.5770855828927504e+00,    -9.2855856295481226e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2207526729437971e+01,     1.7342357413207017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0034964147038608e+00,     8.8716793777575858e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3524474648857826e+01,     3.8210553931707145e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.7987111262669035e+00,     6.5504182093760801e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.2008301593334565e+00,    -6.0066482338620954e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.9865253211420981e-01,    -8.1191419588871468e+03,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {    2.8986489419963579e+01,     4.0593262913666003e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.5953198650137487e+01,     2.6294005882577025e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4391289047723351e+01,     1.5466313046570931e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -3.4127520079930616e+00,     1.3722764557439301e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.9736887123427273e+01,     6.5992617915841556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4699212759950733e+00,     1.5400767679155813e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.7198614906306773e+01,     6.5906863125703967e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.9529797805044851e+01,     1.9582573268219388e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1997948498550439e+00,    -8.8860057042560038e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.0746047441775852e+00,     2.4330651286036300e+02,    -4.6341379122860225e-04,    -5.4857589163538094e-07,     2.5869658024004836e-09, } , 
      {    7.8695694045132365e+00,    -1.7805768455636069e+04,    -6.4453616222215565e-04,    -9.0431006332969979e-07,     4.3620626144149388e-09, } , 
      {    3.6304721979712173e-01,    -1.7738254361387942e+04,    -7.0616945212412100e-04,    -9.7720548486849499e-07,     4.7097709865066976e-09, } , 
      {    1.8301391453943880e+00,     7.1058445613193926e+03,     1.2680352579716054e-04,     2.1303837915693181e-07,    -9.5325035607119168e-10, } , 
      {    1.3099976046825185e+01,     2.3938959808853375e+04,     2.4475941736527219e-05,     2.3994117513538105e-07,    -1.2962948205506771e-09, } , 
      {    1.6375752855499798e+01,     1.4846938240519608e+04,    -1.8860047289597794e-06,     1.3439675194415402e-07,    -7.0128783786175493e-10, } , 
      {    1.9733585678178677e+01,     2.2581084757053060e+04,     1.2225363957529226e-04,     3.4805152321524850e-07,    -1.7089779221847365e-09, } , 
      {    4.1307877862561648e+00,    -8.9908104472867344e+03,    -1.2394377957708377e-04,    -2.1374203773369421e-07,     1.0062356432796529e-09, } , 
      {   -1.0245528014451333e+00,    -1.7390489547491692e+04,    -8.8188310022258438e-05,    -2.4541729110761760e-07,     1.2646849685423355e-09, } , 
      {    5.6292138334013293e+00,    -9.8286618682370936e+03,     3.1131724819344434e-04,     2.9776975101812650e-07,    -1.3524362448231613e-09, } , 
      {    1.8235152998782354e+01,     2.3418936178003423e+04,    -3.1300738819523585e-04,    -1.6346026553657220e-07,     6.4969396591807777e-10, } , 
      {    4.4294718562675852e+00,    -8.4334961676541807e+03,     6.1440216247159225e-05,     5.9179751611997757e-09,    -2.5738758330105242e-11, } , 
      {    1.5514236376158564e+01,     1.3833580822626689e+04,    -4.6510393123039382e-04,    -4.1426640615382669e-07,     1.8842235234954001e-09, } , 
      {    1.1091016180140059e+01,     8.3321650059568310e+03,     2.5877820347140000e-05,     1.0457067491271859e-07,    -6.0615769735444168e-10, } , 
      {    6.5865895074823273e+00,     7.1430451796142570e+03,    -3.2993655698332687e-05,     3.1015228028460899e-08,    -2.0473681753255705e-10, } , 
      {    1.3902593609230262e+01,     3.3456801069981972e+04,     2.3820577467365049e-04,     5.6364273729143070e-07,    -2.8785327502197583e-09, } , 
      {    3.1252428207515695e+01,     6.9284530221673631e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1193449686089137e+01,     9.5147054712288773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3987128817576000e+01,    -1.1281350705364596e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.3994241323569074e+00,     5.7195312475985183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5322428129390584e+00,     7.9210338517986493e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.3517092747301319e+01,    -9.6528761631755824e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -6.7662755319127243e+01,    -9.6528627360741320e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.4266856152921733e+01,     2.4731642834194448e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    9.5079306923811195e+00,     6.7439512264794985e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7106462224602819e+00,     1.6240951653046122e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.8178264483798500e+01,     8.3508810137976197e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6265504273615836e+01,     2.3855446206381246e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.0198256414934335e+01,     1.6657284797673063e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.3752830061820294e+01,    -8.5482455559767059e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.0590363799077352e+02,     1.6657480900820738e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.2000756802121884e+01,     5.5282146197112761e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.7236271770891953e+00,    -5.9712918153046508e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.4529624607759416e+01,     2.5900984912884043e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0078312014066773e+00,     6.3570276205593791e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.3184770280532607e+00,     6.7266383637445770e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.1799323715547871e+01,     6.2820953347170359e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0584128585980288e+00,    -5.5468444731636919e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.0180308946906791e+01,     2.3187781199373356e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3689273837052345e+00,     6.5916553343980313e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3053179587119232e+01,     2.2422755911868415e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.8784197659709623e+00,    -1.1856162348466132e+03,    -1.8272011433133451e-04,    -2.2027640500905660e-07,     1.0864141298615479e-09, } , 
      {    1.2473990421990333e+01,     1.4286150379553674e+04,    -6.0855413138016608e-05,     6.0884938291196182e-08,    -2.9913973751820592e-10, } , 
      {    2.4105384832004738e+01,     7.2117599304195455e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4715135348082898e+01,     1.4872498234447425e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0887298192343221e+02,     8.3927792001308444e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5104129271175681e+00,     6.9077829445927982e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1880953301992026e+01,     1.8358335427646562e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6012519614437128e+00,    -8.2577037490419425e+03,    -3.4034028507947772e-04,    -4.6976249493538599e-07,     2.2135186719786194e-09, } , 
      {   -4.6385703068274310e+01,    -8.3857793232870281e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.9678757955066402e+01,     6.6590497856672046e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {   -3.1922684548736729e+01,     9.0528092985369040e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5882975619297184e+01,     3.8500276036015671e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4735918274243975e+01,     1.3799818145671146e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.9703688629231095e+00,     1.4642742268494680e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.3868381004610058e+00,     1.4882807515702912e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2821564323501175e+01,     1.2254635749521094e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1466798851236217e+00,     4.2844013289753939e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3798504775650059e+01,     1.5873642109802337e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9973527049859172e+01,     6.2361520085526436e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.9928461077434108e-01,     8.3258379351495569e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6736721883147851e+01,     3.0474121675725641e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.1031334771369689e+01,    -2.3072540054226365e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.5261618242722896e+00,     8.3471677489126050e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.9566453770884777e+00,    -8.3102151003342969e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.1533506828083153e+00,     7.1920216119136157e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3302408424423980e+01,     9.1857266694224163e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1983169283725957e+01,     1.2856041851944738e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3198900729833777e+01,     8.9923360806057153e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6117866952164178e+01,     7.9220002541997119e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0961936748818717e+01,    -1.9311557020690984e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3128735318750378e+01,     9.6419735736291113e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6923689113777058e+00,     8.9738597563165622e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.1024540591308838e+01,     7.8426990827786603e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0414345043461850e+00,     8.3253857248473978e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.4678291364594998e-01,     4.7740590229269088e+04,     3.2451087103931128e-04,     8.7639206750300724e-07,    -4.5239903839055236e-09, } , 
      {   -1.5527127804154230e+01,     8.5095265349462807e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7331710583333511e+01,     1.0922103858133931e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0054050623463269e+01,     2.3946696775675959e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0960928413912839e+01,     2.8745431705261526e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.6365467465230481e+01,     5.9419056669788279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2972147193450798e+00,     5.1206010936767268e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3610446946870692e+01,     1.4914670362093959e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9430820257905317e+01,     1.0784833866142042e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6261175375644971e+01,     1.0680258287182241e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.2735105867330416e+00,     1.4871786888980549e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1265647841804943e+01,    -1.0966043510633040e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    2.6618454188162069e+01,     5.8177622843826866e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0809457292167766e+00,    -6.0911991528951216e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0613034523891594e+01,     8.4353409645127085e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3413087517200708e+01,    -3.6462212157627619e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7452755707801277e+01,     7.2163298543067849e+03,    -5.5832899822007905e-04,    -5.9151709988682344e-07,     2.7173322012508378e-09, } , 
      {    4.6078464153804166e+00,     6.1217410052068235e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3483406367137388e+01,    -1.1167707974869256e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -1.7096649030605835e+01,    -8.1478563143006204e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4824443743386544e+01,     2.3505656692507197e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    4.9480946299201641e+01,     1.5560027286747463e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3112943550007500e+01,     2.6851320162209580e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0557219287216444e+01,     1.6700042665202604e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.0368787603007874e-01,     6.4411388371706371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2698502637339139e+01,     3.4547054339671686e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4554397421615931e+01,     8.3114526446114214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4642861192982355e+01,     8.6143012678144314e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.2941008010558420e+01,     1.8914450781275933e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7691685784153353e+01,     1.6512267733324454e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -3.1126639890486789e+01,     1.2562628541330114e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8731838112163341e+00,    -1.5540487301033356e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    4.0430874172189689e+01,     7.1577395327709401e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.8580431020518482e+00,     6.5960645615682888e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1670881033363599e+01,     2.6119830568731461e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.8319960016123460e+00,    -1.5905122337316085e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -6.0778176440559051e-01,     8.3572668041984471e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.8861770875596466e+00,     7.8585797857446041e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1714121867634368e+00,    -1.1124863210432839e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.9872695756453140e+01,     9.2321273352955632e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.9645433143024995e+00,     8.3633171500932949e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.9439628382965623e+01,     1.0999450403920033e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7556669474704538e+00,     8.5393948761750726e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3358283362741354e+01,     1.6570641800319459e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.4586533057285545e+00,     6.5886001943705924e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1089502271799008e+01,     1.4914457227140061e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6219553990306601e+01,    -6.2623004226184275e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.7439819102525320e+01,     4.7745192629151890e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.4095156232128069e+01,     7.8444479931875885e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0673770638740852e+00,     1.1377257977699364e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4221342149235241e+00,     2.5048431080900507e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6786229593446695e+01,     2.3232828848304212e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7380714021285307e+01,     1.3719636052342838e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.1689613917891787e+01,     1.4286467752265480e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3189025727603012e+01,     2.1420019891920747e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7788301928325762e+01,     8.3129342053700893e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.1644327463064911e+01,    -1.1011220479937515e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7303396189296830e+01,     7.1964308390838769e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.2212907535474891e+00,     6.9581127337488952e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -8.4273610615036709e+00,    -8.5605809591671077e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.7514234220616789e+00,     8.4831295549267397e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1954129930176517e+01,     1.5426506814303924e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3957060320754255e+00,     1.1770278947392351e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1570826663744498e+01,    -1.1139239273489559e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.7694241156898030e+01,    -1.9521332940888756e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.0575219128273929e+01,     4.9979389175415527e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.7998871061102690e-01,     1.4914053458673810e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5074977368178169e+01,    -6.4142549624381081e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.0756777326010322e+01,     1.5892357561741952e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.7108402696484916e+01,     2.2128516283845322e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.3843724014764447e+01,     4.4471835935081344e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2175621983542491e+01,     6.2752622785344056e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0358560761284618e+01,    -1.6666019347629735e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.9778430298555036e+00,     8.4323026959373692e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.1287584783198746e+01,     1.5453712041112944e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -3.8478390831356766e+00,     2.5240505713546248e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.6920229202970400e+00,    -8.9739196904765849e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -9.6173823646731478e+00,     8.3363796307838420e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.4547337615601592e+01,     2.2360399104838252e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.3784014873310575e+00,     1.0513663777301266e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3273765916166020e+01,     2.3295395881304019e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2519921701535786e+01,     2.3949442601497671e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.2137970836558964e+01,     1.2225119694098164e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.5834219387541602e+00,    -2.3712437293561920e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.7115963462422535e+01,     2.3712186107013596e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.6927192937381136e+01,     1.4230782189704367e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3275798943520623e+00,     8.5169109823818999e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1911193777917354e+01,     1.2181348860541202e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.2157346352911532e+01,     1.0585759356652481e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1729864597122710e+01,     1.0682662157213974e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3249781438739246e+01,     1.5612584133840552e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    7.2145513825376639e+00,     9.4999869628031302e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4250037781689244e+01,     2.4739970947260936e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0341724882206869e+01,     2.1083770789048274e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.3644054943460674e+01,     8.3256322199235365e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0056720940022206e+01,     6.5697557720888035e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2418498896025579e+01,     9.0191321595414393e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.1818876308575128e+01,     1.7403404171508941e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.3922084578212178e+01,    -1.1153331911812554e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.4046807815859133e+00,    -1.1610292251382525e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.4559331910012613e+01,     1.5369253240649092e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.5605482723949338e+01,     2.3941275558464000e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    5.7578916408259948e+00,     7.7702136834366665e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2120675133177361e+00,     8.1179974488107364e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -8.2760796405050527e+00,     2.5534752055120069e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.6964423686839485e+00,     8.5360986521379309e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.8005279742094721e+01,     1.9160125110546989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8153064067676876e+01,     1.4301558231030387e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4936583711141527e+00,     7.0966688940722070e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.3782912222943651e+00,    -1.4481309518866788e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.1158684027329233e+00,    -7.7882978257984914e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.2748583081557685e+00,    -7.1505810971189499e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3895223114470586e+01,     1.5637671857681813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6729905771986893e+01,     8.4697116330483677e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.8299422219170705e+01,    -8.1876712161985324e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.7392526747387027e+00,     7.2163658002971415e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1032885014245979e+01,     6.2637276160328056e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.2826499682203341e+01,     8.2184687537574264e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.6950797296816607e-01,     8.2220418847341916e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.3639495102347148e+00,     5.6502920657886409e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.5501755520427132e+00,     3.8299219129363853e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.8135706967642225e+01,     9.0460165232837826e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2503927877167499e+01,     3.8299923761280013e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.0879268640971674e+01,    -6.5087016920809574e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.8991431096342765e+01,     1.5232965920119550e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7956987380678804e+00,     7.1868403935023180e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2653620082602387e+01,     8.6376011777051790e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6986197433804495e+01,     1.5010075618103147e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9769953319922767e+01,     2.9577859051005496e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    5.6740583332664940e+00,    -7.3964127351282787e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.7989210188565412e+00,     7.2876505250805112e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.9124849419515204e+00,     5.3602917053606925e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.4080398936119977e+01,     6.9751734539708705e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0828208004209209e+01,     1.3825555295287399e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    5.8875313827909341e+00,     7.1643994679232055e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0332109197002591e+01,     7.7018065305116588e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5889627449924451e+00,     2.5017252677385426e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.6479526565868468e+01,     9.8412567263239871e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.5863876653955010e+01,     2.3245215672343402e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5842684665439279e+01,     5.8778178914714335e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.6272118668922261e+01,     1.7697650513082757e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.0335670174125760e+00,    -1.9268871908873450e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.8704812192088749e+00,     8.9415798985293150e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8512065578718135e+00,    -3.4159825106025128e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3063287383724056e+01,     3.0141732286237184e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.8649571058926369e+01,     1.4989697766006964e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1496617422914719e+00,     5.2308073609003679e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9876891359446997e+01,     7.1441730873395618e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.3481923761205525e+00,    -2.3455065797399379e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.8615145476460055e-02,    -2.3117251522847773e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.4269278753176298e+01,     4.5515411956543267e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3148788733348402e+00,    -1.1430144926753642e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.2696715994146111e+00,     1.4914223047778542e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.8593872196469086e+00,     1.6316176990949782e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    8.8349312115109200e+00,     1.6727664809193015e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3565389054840344e+01,     1.6022516308602226e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0421540058573855e+01,     9.6397515384265898e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -7.4555678029791466e+00,     3.8411674480610227e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.5608057086595934e+00,     1.0309174153637870e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0791055523173174e+00,     2.0727277957061342e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7377213394451374e+01,     2.6687026852270013e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.3698444566656933e+00,     7.3008133900246967e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.8476122254139149e-01,     1.0984237008006377e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1362986997643041e+01,     1.4913894415403767e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0089895310081978e+01,    -8.3662306679420180e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.5274512255071837e+00,     7.1345023997045519e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.8110580147515609e+00,     2.3274316533074780e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.1894088724525258e+01,     8.3315449140973433e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4939755697852348e+00,     1.6026445431885171e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4828748991407586e+01,     1.3031825149741031e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0895818846156518e+00,    -3.6578902209805034e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    7.9888610839492671e+00,     1.7073814045447678e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.2295737689731254e+00,     1.2116899251212193e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.3137714111901715e+01,     7.0963491281418464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8396426776070269e+00,    -1.1572883752208093e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2345624165107104e+01,     1.0735983570383240e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4245265701248606e+01,     1.6184845446450963e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8675050653497706e+01,     2.3087704975160683e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1844576569301122e+00,     1.6614723033291197e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8743427810350113e+00,    -3.4727543774230303e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.1560801528469575e+00,     1.3072183765755326e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3141489669497833e+01,     7.2578336989153086e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    6.8982518393473145e+00,     6.0207260737339366e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0567510758321998e+01,    -9.0978563699337137e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.0736008690186342e+01,    -9.0932539700508933e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    7.1027130409449786e+00,     2.5595013160945018e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.5227660690705754e+01,     1.6266663079495320e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1911892460013153e+01,     3.7999850480155517e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    8.5498714000804483e+00,     6.0530766084341521e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1902062737452461e+01,     8.6134919514304638e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    4.0130473229139142e+01,     8.5351276447784403e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.3855805903952984e+01,     9.1972166572791430e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5995336865975530e+01,     2.2110485362092801e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.7802423747434606e+00,     2.3398242651583951e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.4704126397646746e+00,    -3.1031625857156537e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.0144788379663989e+00,     5.9993961657560849e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4089925548787649e+01,     2.2561861447302967e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3824159757120789e+01,     1.2434669159834466e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4901760108359559e+01,     2.3307127853493257e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1064275444778140e+01,    -1.1211875148642366e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {   -1.7439322303241074e+00,     6.6019646156535182e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0156034242985310e+00,     6.5574217908555147e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8645357638509076e+01,     1.5586525123538759e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.4776200061346190e+01,     9.6317072605876838e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.4691650564859629e+00,     6.2068100141927255e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.7843949142250310e+01,    -1.0592716290787639e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0020233999734931e+01,     8.7926723300356552e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    7.8449798533588044e+00,     9.0629372040913750e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.5618214536623150e+00,     7.0978153896804733e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.6228096315958652e+01,     1.9248512024747659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6577454440922264e+00,     2.0334256987368357e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0983059471319475e+01,     7.2128899280652013e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9240040455907888e+01,     2.3638460299194099e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.5721038350891831e+01,     9.7791085680570086e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0018583059793977e+00,     6.6970889687800864e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9080689243510566e+01,     2.4892774269343776e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0893533254147840e+01,    -1.1277464704633730e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    1.1383993215096101e+01,     7.2152358026514867e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3142227094093208e+01,     2.3465306833713992e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.6519125882984813e+01,     1.5397639174059352e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    7.1797270769325898e+00,     7.8379612232336294e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6543776229616803e+01,     8.3643875250689725e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7529641629613366e+01,     8.3311212833652262e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.6535347221660174e+01,     8.3262615658816740e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1230738113877097e+00,    -1.8411519215796659e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.5260471853812106e+01,     8.8220122756868259e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5328745159069550e+00,     6.3482086956090307e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3806728319016045e+01,     8.3283961619424499e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    4.5954026564652651e+01,    -8.3289866873044484e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.9834430379271744e+01,     1.1178053989743381e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -7.9385898742150438e+00,     1.0133984375964255e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2401962954696533e+00,    -8.5070624868446739e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -1.1384301818442127e+01,    -8.7363489974482509e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.4911666149024086e+01,     8.3255629195269867e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    6.4590261409774783e+00,     9.8478156819231190e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.2778159535975858e+01,     6.5863075698452840e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1957673274119943e+01,     8.3151288875448809e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0001267599403130e+01,     1.6342170286443952e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3216798673661110e+00,     8.0842160213555762e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9967322949249589e+01,     1.5697826788305383e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4524574895937565e+00,     1.6156105541889223e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8941014558526557e+01,     1.3842477961626850e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.5171074903461232e+01,    -1.8381079763949028e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.0469995185492216e+01,     7.7490857053924647e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.0248458447754079e+00,     7.9563339711873577e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9857525991874127e+01,     1.7325505336190356e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7756592984343182e+00,     1.5999508223997282e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.6336710344151966e-01,     8.3422539617020193e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4572775185124428e+01,     1.6065835026071831e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1810280367935299e+01,    -1.0719687433094020e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.1999847119885363e+00,     8.2440831381789194e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9778215464236780e+01,     3.4951283927079610e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.9306509226110336e+01,     7.2598419035569759e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.2016128598666285e+00,     1.4952184064050463e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2155343284040555e+01,     3.0454571162601860e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5389548787413823e+01,     1.6755598626515919e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.0338717490504667e+01,     1.4911601790878460e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.7196160073351408e+00,     1.7190089229824458e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.8267591840922606e+01,     1.5542289828194491e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.1190539337116586e+01,     1.5543218751769105e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.3591336076351375e+01,     8.9727314936682924e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.5895759381186227e+00,     9.3346008682223019e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5874645413864368e+01,     9.9809573967237084e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1039446883193030e+01,     1.3188783557391271e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5164961727153326e+01,     1.3585201607893154e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.3505347331367776e+00,     4.9361191405287282e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4471834719731692e+01,     9.0869186716094700e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.6939673976060497e+01,     2.2750258199741005e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    1.2209439900846061e+01,    -1.6892557016325811e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4800390927940313e+01,     1.5790660143752335e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -2.1506561104094654e+01,     1.5542435486330292e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.8583613607900652e+01,     1.5543073093633298e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9126965301959764e+00,     2.3497569569235602e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.8468251344674421e+01,     1.0897278694991965e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6413626388087188e+01,     3.1518860456971961e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.5188058779439572e+01,     6.7224729827722058e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2903527810962272e+01,     2.4821780963930785e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.5882453750468628e+01,     2.1549402523843892e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.0105013501195327e+01,     6.5594468896862436e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2344707201740704e+01,     1.5598015111395487e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0493792038632441e+02,     3.1069966551675163e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.5870197078894812e+01,     1.5542903775962475e+04,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -1.8793144575088800e+01,     1.5542604804001114e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.1025930685184974e+01,     6.5334975285684859e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0635067034704955e+00,    -3.4689516078137044e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.5644051087527870e+00,     1.8422629391809260e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5244956516570829e+00,     7.8773153394132796e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8540836132858615e+01,    -8.9042248868852766e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4996301744123016e+01,     1.0207055224160948e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0736473993583036e+00,    -2.5002994954677950e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {    4.3858277190554631e+01,     3.2830745924215647e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3742131823970890e+01,     2.9871553225828207e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.4927882363003064e-01,     7.2764196723885034e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    8.8648039772500482e+00,    -1.8384961181884394e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.7778283740037817e+00,     8.4132997110679808e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7205343394265562e+01,     3.7850945810074300e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.7837619002852332e+00,     1.0120642807608278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8322610665519093e+01,    -8.2691881070671843e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1527853267967055e+01,     1.4077905688323845e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2445220072066054e+01,     5.3133923667072650e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    5.2731023917464192e+01,     7.6363904110087306e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.9965455545133430e-01,     1.7874183117440076e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.3803393911100841e+00,     1.3868996578223056e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8303441393158380e+01,     2.2539579483443449e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4409055974272524e+01,     7.9202513437907846e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7652919213810692e+00,     4.3677259665946094e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9616731255599536e+01,     2.5802945223122142e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0285892212018480e+01,     1.5332050838430172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7330772760351284e+01,     1.7109862461851277e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5267158332748128e+01,     4.8402045632011232e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5267638922411471e+01,     1.5487493468568102e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -8.0724969693581826e+01,     1.7991896854656574e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3863410318400888e+01,    -8.3235708519863147e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.8054227077266795e+01,     6.2107832540756754e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4034626983458622e+01,     2.3539707325905238e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7613677929290805e+00,     1.6184179786225606e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2213775613781117e+01,     1.4293765703516978e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8405453876729830e+01,     1.2880649750999733e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0418711346677480e+01,     3.0770039871282726e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.0693438771907946e+00,    -7.8930725586666422e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1577718551353504e+01,     6.9269611386099186e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.4071231708990801e+01,     7.1852124701607790e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -8.0151441599608290e+01,    -1.1389014328745880e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.7755258581006530e+01,     2.6464858148115723e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2681830456846459e+01,     7.3303103410362155e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0675805515815561e+01,     2.2751886123082542e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.9233418561380322e+00,     6.1801150722018265e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.1727462254644252e+01,     8.6392291010467179e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.8851023997014886e+00,     1.3095842673022778e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.9179215495038200e-01,     8.9136278482464131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0623139110364864e+01,    -6.3996692035550832e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.5746654058171226e+00,     4.0676050856576967e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7862857600327096e+01,     1.5536142890429690e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3522283658328616e+01,    -2.2266737693456889e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.7861724836160725e+01,     9.0240631917990358e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6143208469452645e+01,     3.9807220974280026e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.0721868721525727e+01,     8.3401661529650264e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.3843915457520280e+01,     8.2909540655092387e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0773950041849609e+01,     7.7664461127333016e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.7454068947342400e+01,     7.7763081772484948e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.6631025985517851e+01,     1.3972971611151257e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1804447505051808e+01,     1.6657776095509344e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.0988778483382262e+00,     1.6656989602984457e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3096148792766972e+01,     7.2129346027100755e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0958457062569789e+01,     1.5087600170416365e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.0973869117319097e+01,     3.1085410528389752e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2623191845321239e+01,     1.5437527441026288e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.6199957527022733e+00,     1.8033683940710034e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.1884479675748027e+02,     3.1085606631537426e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.6374707600599677e+01,     2.2743409287946997e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0529753336149126e+01,     2.6083782152327858e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.7578960268823423e+00,     8.5595707983376033e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -6.7363626640886821e+01,     1.7109157829935124e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.9399685216550282e+01,     1.0450772622670314e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5316547201195526e+01,     1.6792810504473749e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    6.4539562586386445e-01,     8.9555763187352386e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1010898155554315e+01,     9.7027005660006726e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2253839736852392e+01,     2.3967005688483034e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.1408634937469571e+01,     1.3655530334903755e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.4529112154087827e+00,     4.6801690089192789e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.5114695628164379e+00,    -3.3150200230825994e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.3645980653499066e+01,     6.0990819901351670e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.9254725546004625e+00,     6.1686127386451462e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9537372188837976e+00,     1.6047108948142068e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1135431695590363e+01,     3.1554908873375556e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.8853308844261147e+01,     2.8820203045029783e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.4984955573680465e+00,    -9.4083750948791640e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.3912280082021972e+01,     1.4508307824044996e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.4212630832868541e+01,     8.4814947467062884e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.3231833105065549e+00,     2.0883812026942513e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -4.8025890647492062e+00,     1.2727173152317026e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.6957932917918006e+00,     2.1026081120317784e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.2578553740939153e+01,     5.4887587473221656e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1531903663142415e+00,     1.5616341949703961e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9031225663503232e+01,     1.3214170376581305e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.7383473691265806e+01,     6.0997866220513151e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.4819734192510671e+02,     8.3303133299396268e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4034464017074333e+02,    -8.3270695193072752e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9117905440476711e+01,     2.5075091787485137e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.0872095850668337e+00,     8.7988030635022951e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0304613970998993e+01,     7.1088360164028381e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0665068573309700e+01,     7.2136670313183340e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.1700874829722695e+01,     7.2144586993983567e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9413052869066675e+01,     2.2669889484385237e+00,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -5.2856120968534732e+00,     8.3134959706414829e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8105185242212855e+01,     2.9278138085839084e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2743884399971510e+01,     8.2904408358380060e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5280818720986815e+00,     8.5405678134682148e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2317132332696019e+00,    -1.1184313288744311e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.4244406462053840e+00,     5.4357896501109744e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3026925521963930e+00,     7.2544032484126383e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0636062930358111e+01,     2.3119185202674431e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.2059686610541878e+00,     6.0097054470115727e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8912106040518147e+01,     1.6943985903215045e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.1842791471902750e+01,     7.7007428690867082e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5329078554321445e+00,     6.7336419452240116e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9380235705448541e+01,     1.8065173418079285e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -7.2825255351009943e-01,     1.0591216038313389e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6355706688278833e+01,     1.1035273942083563e+00,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    5.3511752737047786e+01,     2.6154320471692456e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.4569772120022773e+00,     3.3176345958836309e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.7069100148228337e+01,     3.1101050608252015e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5766152612936708e+01,     2.3875854941775502e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.2892048156584977e+01,     3.0945823169079013e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3284939767917511e+01,     7.8254497695962955e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.4535402139421976e+01,     2.7244341131902569e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    5.2716015090908869e+01,     1.5996260239147259e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8779179721499268e+01,     1.0831433685923541e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8475579919258500e+00,     8.3664287837376614e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9098217278466961e+01,     2.2621830421357976e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4674440811384033e+01,     2.0349525009580349e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.1876936920361825e+00,     9.7276218379152888e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.6763777508104702e+00,     1.0524107181538473e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0234006667349972e+01,    -7.7871343642442589e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.2782652019074149e+01,    -3.1480227513751743e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4928553618908921e+01,     1.5659001765659665e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5754931183715861e+01,     1.5859542536687853e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.6192782908156083e+01,     8.3259512756902241e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.3871615110941029e+01,     1.8779464047293761e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6750809457487438e+01,     1.9899766384059818e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.5462424399771891e+01,     7.2151911280066115e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.0403283927141533e+00,     6.5797218058700500e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.5198537637873727e+01,     8.3314315735566761e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.7666792933263444e+01,    -1.5438616348210871e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0309133702054046e+01,     2.3853421506403447e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.2338444742996213e+01,    -1.0862426258548471e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0828317944738735e+01,     1.5167782263744666e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7805443744477849e-01,    -1.0646966056768450e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.2705676534858000e+01,     1.6404047290438481e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1271229601608319e+01,     9.4121531292344262e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4854516507197459e+00,     1.2633873147790451e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.8351813881114101e+00,     1.3329743587746314e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.6298814180085667e+01,     2.6071837178422735e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5502292263803806e+01,    -7.4029640743529053e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.4229987899213228e+00,    -1.1108257896557802e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9236735447115578e+01,     7.1179246308730280e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0630992451042630e+01,     2.0150518789739522e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.1952102878218334e+00,     3.2454568578904597e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.7839021127040269e-01,     1.7053800899748785e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4337297784353225e+01,     9.6589297324437102e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3158773799400056e+01,     2.3294953240955569e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.7995276456050338e+01,    -5.2958573555468347e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.2364599393738338e+00,     1.9941236017675372e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.3431620469898412e+01,     7.7138903508492613e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.4865257928882954e+01,    -6.2840563661938234e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4849985215143377e+01,     8.5104929373473424e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4037837778586329e+01,     3.0756921960084463e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -1.5944210231577724e+01,    -9.7324992534206813e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.9731543565936590e+01,     3.5015718276893465e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.0151010362641436e+01,     6.5726373691145354e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.1240961281009545e+00,    -8.9209505932653938e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.1111546017351721e+01,     7.1050303768324738e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.2277055662646056e+01,     1.6153028191086631e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    5.3835547513354491e+01,     2.2761748187597736e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4188970020162726e+01,     8.9510989555837641e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4429610827592910e+01,     1.0968504524007079e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.3581755376228450e+01,     8.2006102205468796e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.6065380724331089e+00,     1.6168984332243635e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4863121351531525e+00,     8.7649243770232461e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0817174318775233e+00,     1.6204837672651018e+04,    -9.9761634931308941e-05,     2.6330037095586454e-08,    -2.8787751570603142e-10, } , 
      {    2.1663380367211349e+01,     3.1119290007418749e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    3.8010109774654701e+00,    -1.6062862321487999e+04,    -2.7633411970299409e-04,    -4.6441727865704901e-07,     2.2429468624219679e-09, } , 
      {    7.4566952505581208e-01,    -1.7319466231916122e+04,    -2.7100635671717702e-04,    -4.6565006288537437e-07,     2.3518263189255473e-09, } , 
      {    1.6109248440616003e+01,     1.6066287142159186e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    5.7087971512179814e-01,    -5.2353285217739244e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    1.0067575877497664e+01,     6.4809805474446503e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.1854135515008265e+01,     1.6204848932313980e+04,    -9.4433871945491906e-05,     2.5097252867261127e-08,    -1.7899805920245181e-10, } , 
      {    1.7485079298432908e+01,     1.3797188313354962e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1161375974076979e+01,     2.9253554922038151e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.9484537218695102e-01,     1.3769165223312450e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0195648870489002e+01,     8.9842214815250190e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.7277171448542379e+01,     4.2430485732945544e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5504543656729393e+00,     6.0507476997938338e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9159192439753259e+00,     1.4930446213160250e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1611255511021161e+01,     9.3772424512180078e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6399707596113203e+01,     1.1659449978806952e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9742857649235376e+01,     8.6038197502500188e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.5883355155135437e+00,     7.1676621994851976e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.4682102848826837e+00,     2.2029182775424317e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2742987624146998e+01,     2.9908523213668326e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.0482915644560933e+01,     1.6566712677036507e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.2510918558322857e+00,     1.4976579854808700e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.2494645831344107e+00,     9.6382756919257281e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0926322099679080e+00,    -8.3464733438546118e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -1.4122270621386878e+01,     3.0616844642741817e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8304224976933170e+01,     9.4920065901879625e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3066247722164739e+01,     1.8247104577919548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9210766737253458e+01,     2.6857879117808712e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1493881365819348e+01,     7.7053148718355005e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4674479052370543e+01,     1.9407771632339311e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1688002110073233e+01,     1.6177620830626473e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -8.7717411526304030e+00,     1.5542469061991611e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.6424015695667023e+01,    -1.5543039517971974e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3300229628584095e+01,     1.5541581352688652e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7855453003313864e+00,     1.7417687639016618e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.0331656580933359e+01,     1.6257812399693690e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2473060120564258e+01,     1.5176965267883746e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.9218985487277529e+01,     3.2811524043466648e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2935805939046237e+01,     1.3887732131891729e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.4891159532054445e+01,    -7.5787321126340466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.0910539813504911e+01,     1.5152034520230216e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7570098102160365e+01,     1.3155511052401605e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2385326248528841e+01,     2.9138452674954515e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.7589712964815227e+00,     1.5541331599048041e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7601186472548328e+01,     3.1730736845816729e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.1257558865757849e+01,     7.0322613526344512e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0504947929933977e+01,     2.2743699244141877e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.8725295219769515e+00,    -3.1572856687421672e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.0751776381822737e+01,    -7.7894260884467585e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8005721222552523e+01,     8.9590765524526942e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3777163973047871e+01,     1.5543927227274937e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4677899443160790e+01,     8.1552093267685668e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3611325297370797e+01,     8.3993560976616664e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.6534541883996976e+00,    -1.0071627568934096e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.5778134108883037e+01,     1.4130705965491685e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0585981334806638e+00,     5.7287200355138357e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9589349352469167e+01,    -1.0201547230874576e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.7820187716666078e+00,    -7.9164922609133428e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.7177788386038899e+00,     7.2626227353992772e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2976091235524224e+01,     8.5482302964615556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9853219916537455e+01,     3.0242584457140052e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.5058332693611426e+01,    -8.2612675455516037e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.5891635505757620e+01,     2.1500554301207128e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.8431131274916204e+01,     8.2480106585148624e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    3.1989719971080699e+01,     5.3909598872345396e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5862742650511318e+01,     2.3384772424427611e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.6439528332621123e+00,     7.0809546430329065e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.8147536657052070e+00,    -1.6592948499433045e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -7.9409306707146214e+00,     1.4905810206553419e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.8177886706948385e+00,     1.5537324584943924e+04,    -2.2282960538085985e-04,    -5.3675274039416252e-08,     7.0782797441992036e-11, } , 
      {   -1.1834485872341787e+01,    -1.5548183995019670e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.5666549713530873e+01,     2.0115055143380841e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.0060540068558517e+01,    -1.3671770245940479e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.1359201531080556e+00,    -7.6441243410864018e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7453749180771654e+00,     5.4519989854516907e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2093801400380407e+01,     3.0377018901589738e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.1422524711624522e+01,     1.2411960589514092e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6632004295896536e+01,     6.9042744954960990e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1726584433047313e+01,     8.8702484850026412e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -8.9499528836495045e+01,     8.3355969545290445e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4835362063356879e+01,    -3.2532002460697740e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.3613727242797538e+01,     2.3870893425072762e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3303769076744942e+01,     2.2026337861366526e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0982794108921645e+01,     1.6659649838195339e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.2412711078771439e+00,     1.4906968538367195e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5166978289297980e+01,     5.1941533762395484e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3871808949264253e+00,     1.2163299917085338e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9441753159094237e+01,     1.9553146080239974e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3709176500388516e+00,     5.4343669591772214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1457800084226406e+00,     1.0198195068620402e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.4453923342991679e+00,    -1.2370234810020611e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5057974578490981e+01,     8.7423759105733316e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8258170109868637e+00,     2.3871998004137728e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1391544769624748e+01,     1.6973456383838925e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    7.9325107901925778e+00,     7.0033594138067210e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0550078700681951e+01,     2.3869303476383426e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    1.1238082903897658e+01,     2.3873587952827067e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.3738098696542288e+01,     1.4898447196712254e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3112243496207913e+01,     1.7409643361177717e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.3736501363131438e+01,     8.4394996319343518e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.9487557148602921e+00,     9.0993142528697463e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7269143803450000e+01,     9.0678340253559982e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.0852861071884382e+02,     1.4286237171464549e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6598564960137320e+01,     2.3870479312204181e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    4.4433749642620839e+00,     6.5573693469025429e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8799238512775617e+01,     2.7080047821963002e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.9540675470102236e+01,     1.4694579200723376e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0770907459711143e+00,     1.4250090703487109e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.8511253846868563e+00,     9.8051740989274149e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6691319484865630e+01,     5.2332255328941064e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.7808721749381633e+01,    -1.0903556856556233e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2277877343757961e+01,     1.6729479682054098e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.8142248395267790e+00,    -1.7363771887115151e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.5632208494933572e+01,     3.0983336871035521e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.8739766410735808e+01,     9.5849542787564606e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9730628504425587e+00,     6.5430954643570976e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4826195856685906e+00,     3.1634186361213309e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.2839333930093368e+01,    -8.3313624875298283e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.4935604219397689e+01,    -6.4558058181121123e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1466951571340974e+02,     1.4978534480443646e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0872311378057436e+01,     8.7643102905802589e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.0497855416076380e+00,    -1.0857669508320072e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -7.2285542986272269e+00,    -3.5709701546920478e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.7394607279165712e+01,     1.9208697122849750e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9040133611565576e+01,     3.2221467047206548e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5096830088294178e+01,     1.5052024088591974e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4357269052394827e+01,     1.5143174364689674e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.8697938072104261e+00,     5.9471384140117698e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3150102335766050e+01,     1.3268413361187095e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0491837664059762e+01,     2.2873713549233338e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    9.0343124396092094e+00,     6.1583034573896593e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2114484717276568e+01,    -8.3381339388619836e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -8.5178493029490028e+01,     1.8286143196230390e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1988606454152961e+00,     1.6675416533187610e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.9770925022609489e+01,     1.5478666514474398e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6014045622683632e+01,     2.3881754995860734e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8209983582515775e+02,     1.5528739820071323e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.5502278332135177e+02,     1.5556768759892266e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3647663615589128e+01,    -9.6977514235644576e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3766357971584551e+01,     7.0820656606341672e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.6455283196850822e+01,     2.2879676030570117e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0353784601713548e+01,     2.3241060586775999e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2856919862093854e+01,     1.6664861015815957e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.0714764831323492e+01,     1.0268971109946422e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4041674162063220e+00,    -8.9132281761120303e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.2846133468882527e+01,     8.4817508925842048e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9839483597472782e+01,     8.3314228753591979e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.6467096875169496e+00,     1.1723878281519204e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.3720521958730529e+00,    -1.2101885331428962e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.3762638221619866e+01,     7.8353705735600724e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2048392793426608e+00,    -7.5234712912203549e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.8427890618915859e+00,     1.6649904682677843e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9773146885929465e+00,    -9.7768223765101284e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.3822175484189835e+01,     8.4575601242511657e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -8.4049690424564893e-01,    -8.8548273007409225e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -4.3074531644169234e+01,     8.6546602795577837e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6447358323196802e+01,    -8.5562898338156156e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.8849070868707258e+00,     5.9567363498003806e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4175555529026905e+01,     2.4951129384860957e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -8.8862585061018478e+01,     1.0251698113180753e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3262930285470436e+01,     1.5906216388971381e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0465941015605900e+01,     3.0493249411321645e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.5997362733617273e+01,     1.4933634374692283e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.9335803823269128e+01,     1.3383386949470367e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0393856429546659e+01,     7.7582592337926308e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0466285119142755e+01,     8.5742893067633540e+03,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {   -1.7604233028058346e+01,     9.1899844250801380e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3468612713213755e+01,    -7.1857111429410597e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.6176874255715862e+01,     1.4852313555063805e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2075973748641580e+01,     7.2673842785276111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.5580074481166868e+00,     1.7112536968812332e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9520274777563525e+01,     7.1763255062441331e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4830529882818464e+00,    -2.7642646990491794e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1878220462371512e+01,     1.5505016930867583e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.4194403228503631e+00,     2.3178703779745840e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8677922917183332e+00,     7.8482705044891172e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7392300008324479e+01,     1.6192391783005189e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7791704764106761e+01,    -8.9750479531248566e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7257926276725151e+01,     1.5544176980915548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7522475675694331e+01,     1.9173243021745257e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4590093052078533e+01,    -8.3337584966651375e+03,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {   -1.4741827781814192e+00,     6.6260956633670958e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0429613614767436e+02,     7.1361640300397585e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.9329970798657907e+00,     1.6127291041470377e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5734345369727020e+01,     2.3861136433349755e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1595194358404362e+01,    -7.9230512165124755e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.2565060712605728e+01,     8.3059438042651836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6682326802230758e+00,     2.1042746665742208e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.6463200860237279e+00,     8.1889907542237288e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.1283751800845025e+01,     6.8854762454792162e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.7448891767314805e+01,     3.2178807231250845e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.7957589851440595e+01,     7.6225029461910099e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9882387122196947e+00,     8.3922065634673836e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6320189931012052e+01,     6.6011743914524786e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0664402016727745e+00,    -1.6906783925663337e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.1738515885386960e+01,     1.0919820600124209e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2514041673336695e+00,     5.4419636843428543e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2607188882754947e+01,     8.0465054706174678e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.2711788586653066e+01,     3.2921744728800812e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -6.3432956900351527e+01,     1.6657177134020880e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8608254801289856e+00,     1.3571090485917724e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3465520297149979e+01,     9.0797979507389446e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.2664632899849849e+01,     7.0355864611625475e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7907651288049738e+01,     7.8074255613944970e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.2406242885281999e+00,    -8.9197627513800653e+03,    -3.0685975863558652e-04,    -4.3393117628015106e-07,     2.0941042141845293e-09, } , 
      {    6.4981237800419391e+00,     2.4391529365780352e+04,     4.2872445982890440e-04,     7.1509251958040393e-07,    -3.4796580815642834e-09, } , 
      {   -4.5798608481387424e+00,    -2.4324015271532222e+04,    -4.9035774973086969e-04,    -7.8798794111919903e-07,     3.8273664536560427e-09, } , 
      {    6.8992320225740551e+00,     1.6762127571952475e+04,     8.5720293256518189e-05,     2.4594641675918057e-07,    -1.3205791378374536e-09, } , 
      {   -2.8809666392536379e+00,    -9.5515382879275076e+03,    -1.7778128972749182e-04,    -2.8839936915237763e-07,     1.5187176411701100e-09, } , 
      {    1.0787549802288792e+01,     1.4981966429015862e+04,    -1.2515258453289047e-04,    -1.1394091133436318e-08,    -5.8710936782364866e-12, } , 
      {    1.3662830272770645e+01,     2.4395002947113735e+04,     3.0230987241371821e-04,     5.6894432033846781e-07,    -2.8498317802980741e-09, } , 
      {    1.2227077979695352e+01,     2.2648598851301187e+04,     6.0620349673326907e-05,     2.7515610167645335e-07,    -1.3612695500929772e-09, } , 
      {    3.5730088558594069e+01,     3.8265814398197865e+04,    -3.2002529118284442e-04,    -2.7917995826692583e-08,    -1.6192776949058513e-10, } , 
      {    1.3827125530534605e+01,     2.4114752227465604e+04,    -3.7730455959010967e-04,    -2.3573929496120473e-07,     9.4296260975804725e-10, } , 
      {    2.0925123303242909e+01,     2.3418876157678260e+04,    -3.1813928645388459e-04,    -1.6231474777084661e-07,     5.3936006837116969e-10, } , 
      {    2.2148435746169913e+01,     2.3351362063430137e+04,    -2.5650599655191930e-04,    -8.9419326232051445e-08,     1.9165169627941039e-10, } , 
      {    9.7573117047365017e+00,    -2.5470248883054646e+03,     3.6340272663967943e-05,     3.5127660078472540e-08,    -6.6429170585628593e-11, } , 
      {    1.6737724329249698e+01,     2.3695653295993005e+04,     4.8788973296512949e-04,     7.8851706677076199e-07,    -3.8832606229511612e-09, } , 
      {    3.3374531739279242e+01,     2.9937122973574420e+04,    -4.7231769894517061e-04,    -2.7863686998134726e-07,     1.0740562291300656e-09, } , 
      {    1.5879598971994326e+01,     1.5090244753379971e+04,    -4.6529979595756203e-04,    -4.1417913969122692e-07,     1.8856779645387285e-09, } , 
      {    1.4425299830042555e+01,     1.4738659916155495e+04,     3.3826120669571184e-04,     5.3718180050194459e-07,    -2.5928368960787203e-09, } , 
      {    7.7715440162374572e+00,     7.1768322369009002e+03,    -6.1244351519990970e-05,    -6.0052416237994891e-09,     2.4284317286776639e-11, } , 
      {    1.6009578515125316e+01,     2.4080995180341546e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.3654023964738514e+01,     1.5752303755718098e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.4252192330127007e+01,     6.3762114444079825e+03,     2.1678544388436827e-04,     3.2291063711668748e-07,    -1.5307070835039487e-09, } , 
      {   -2.8511902846648560e+00,    -8.3659520632434742e+03,     2.3728754745182761e-06,    -6.7550205260458206e-08,     3.7713656253510807e-10, } , 
      {    7.1541949064845536e+00,     7.0720875141953293e+03,     1.5762017074814322e-04,     2.4948608992632941e-07,    -1.1271045421170713e-09, } , 
      {   -1.9170879044252649e+02,     6.5862632389519185e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -5.3779637715648477e+01,     1.5540454825037774e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.0856690219454478e+01,     1.5545053754925815e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2323233646121873e+01,     7.2501712159219687e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8183961650592288e+01,     2.2721480084403418e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.1786942299962185e+01,     6.4264956727211520e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3254771071760175e+01,     1.6013533235912926e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.2713103385903555e+00,     7.2232516532736618e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3639080837709976e+01,     8.9297690476059124e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8192085996805929e+00,     8.3113774975534161e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9785233808410219e+00,    -2.2645941894669331e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3514273613694094e+01,     1.5369287451642063e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0423809916647411e+01,     1.6816290140880476e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0922976957632011e+01,     2.5007404181848200e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.6567825131573116e+01,     7.2114093494588606e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0664208685967646e+01,     8.6457984749810130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.4457715131503051e+00,     8.1878083411916750e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0407682951376254e+00,     6.6129852423758830e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9422814072338525e+01,    -6.2991522263446541e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3457877444499484e+01,     1.6522659721002574e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.3515799197651361e+01,     2.2876261861033876e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.0531725944637060e+01,     7.2289833037177414e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    1.9914232399197498e+01,     1.3067899289311683e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -3.6906081938039101e+01,     8.3109711135423622e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.4758783692402496e+01,    -8.3464117357045379e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3872358311866004e+00,     6.5077583934203758e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1517695297099895e+00,    -1.0050297660956245e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.5864987881515979e+00,     7.2064362817320734e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2045646892580372e+01,    -6.7926901732480710e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1341063246502280e+01,     1.3880017204945420e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5802276651476301e+01,     8.4215117078475650e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3740086156138982e+01,     2.0402494106791542e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.8478881057243690e+01,     6.5917887547555538e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.3560248008818391e+01,     8.3514390449817147e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2794672752592398e+00,     1.3093513673733178e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3066823643657024e+01,     1.7474077710991573e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -7.5048346833253447e+01,     1.1253881843595937e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.4367685894232167e+00,     1.5716221128321522e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.4762449346411172e+01,     2.4936436763421375e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -7.1271942534777217e+00,     2.1497900785307640e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3353377755306919e+01,     2.1373634485562750e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.9207618638506396e+00,     7.7653436705481454e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.4906215521366644e+00,     2.1120298926754331e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.6667441923295002e+01,     9.3291441865613942e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6548358969756649e+01,     1.4645378662128536e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.5986599294817710e+01,     1.6778039552095030e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1158045353541080e+01,     6.0111281379453258e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2765385661772441e-01,     7.7249764791177495e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.0864333044173321e+01,    -5.5660293416567652e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5196364578683486e+01,     1.6886119984739234e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2508005684059061e+01,     8.5236471326606224e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1675204079189868e+01,     7.4121709724468437e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7689570241855094e+01,     1.1073279256875230e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.4945523170536408e+00,     2.4964744365892497e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.7644671545312889e+00,    -8.5029038194230288e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.6902967224625741e+01,     3.1320795195316143e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1545518851803334e+02,     7.1759789319376705e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3956585806582105e+01,     7.1453707598867659e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.0448931204795233e-01,    -9.8543746375222545e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.3283309696458412e+01,     6.6120636709393593e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8793288161776758e+01,    -7.1079087831525976e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2767410654901859e+01,     1.6086950658416079e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5257212859718125e+01,     1.6714049828161475e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.2109759056425048e+01,     7.1499750898509501e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1086975859476809e+00,    -5.5802562509942913e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.5316532968861019e+00,    -1.6533429922361629e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.0661877469801638e+01,     5.6469546934888631e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2834126059225994e+01,     1.6781335776132175e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.0484545769440119e+01,    -7.7587579065729115e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    5.8554355071226425e+00,    -1.5878316222864005e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3279988722062695e+01,    -7.0256755886592173e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.7901465143375326e+00,     1.4914436182732443e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1200544557756317e+01,     3.5344304896772592e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.4780625620119654e+00,    -1.1146180370612958e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.8346109911574242e+00,     7.2927262667531068e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9582215185188641e+01,     2.9042851470486527e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -7.2084143000555017e+00,     7.7533282015350360e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6862391260305589e+01,     9.7637044653118610e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.0392768567091352e+01,     8.3283285864615664e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1387062975044543e+01,     8.3290542627853338e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8051356502590345e+01,     1.3251140364421426e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5864107741713228e+01,     9.8228531794183073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1084341641915549e+02,     2.3243236181133543e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5549957868725800e+01,     5.3145463525225850e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.4070036231238845e+01,     3.0040769798717269e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.8603968770862419e+01,     8.3281927518431712e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.6026720986861001e+01,     6.6037794885146013e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.3012382039389866e+01,     1.5949445460405532e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.8391510684371653e+01,     1.9622388170117301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8154998001902598e+00,     1.9548215047982383e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2131393707798228e+01,     7.3192897143138507e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2487065296259663e+01,     1.5178119253699064e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1777125127978749e+00,    -1.1146390814689155e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.7068784246187167e+00,     1.5103332654415661e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.9600555932673885e+01,    -8.6003958610650188e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.1299008487524118e+02,     1.6657037246839551e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2869548838396798e+02,     1.6657728451654250e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.5603618448390328e+01,     8.0315850322580700e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.2700860624883006e+01,     1.0428230717538070e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0756548607764120e+00,    -2.2660168804006853e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3347088320503449e+01,     1.2574369830234191e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8180680655470169e+00,     7.2735814424297596e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4631691666134294e+01,     6.6813841934796956e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.8233096072718695e+01,     8.3104520254567196e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.9802612519902510e+01,    -8.3469308237901823e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.1398329509433678e+00,     6.6033873065872713e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0921162377148761e+00,     7.8274141172657746e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4787729190945818e+01,     2.2931618791220222e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4644754674450077e+01,     7.3729701569919189e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7319925251843763e+01,    -9.5229497876843780e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.4986330170902654e+01,     8.3260203617170719e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1766690989186081e+01,     6.6141412137230600e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9681170117282615e+01,     7.1607414521890787e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.7653212337130213e+01,     1.5752863911441091e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1982650944041943e+01,     1.7520977049645000e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1761513500930656e+00,     6.2449429323240781e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4663777841829582e+00,    -1.0750577328082371e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.0887506145701437e+01,     2.3367100532183886e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.3539759669930490e+01,     1.4288216662674407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7369940418234474e+01,     3.9571934358927472e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.6693779943232546e+01,     2.3311501602656506e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.3644551050415163e-01,     7.5420906343873212e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -9.8638282033492164e+00,     1.6657935138779383e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5569053763784183e+01,     1.6656830559714417e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.0002961264756003e+00,     2.2720057393469666e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.2599620697262477e+00,    -8.5175773741028606e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0412756744077619e+01,    -1.0616689377928246e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -1.2897540138505846e+02,     7.2136124978264297e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5134134478809085e+02,     7.2145132328902591e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7340863070182717e+00,     6.5857658025166102e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3560327389224963e+01,     1.2618140663791153e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.9465003793921873e+00,     9.6180997766711698e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.4080922979670486e+01,     8.3121639900782993e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9502138518090025e+01,    -8.5322074159169988e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    2.8627673172452997e+01,     4.1781155507004187e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.1408550552637184e+01,    -8.3259599738877041e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0778228020244008e+01,     2.3316761267288879e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.8472476061155369e+01,     8.9614082368392519e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2666092753091963e+01,    -7.5538901702727765e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -1.2122490689200504e+01,     1.6830849687586633e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.0663125808340020e+01,     2.7250900087501701e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4188344172047946e+01,     1.7758893497313737e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.7801270947938246e+00,     6.0372442124461054e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0771925577233095e+00,    -8.9105983437958457e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4314162938255398e+01,     6.1700354295788993e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6304390573369147e+01,    -1.7982930744969908e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.3338517894738828e+00,     1.6709197960642770e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.5096260382049278e+01,     8.1398054751440404e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.1435392578382081e+02,     1.5544023879339898e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8727687328001483e+02,     1.5541484700623694e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2811992074417322e+00,     8.2944578810809344e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7794204841369840e+01,     7.2363543049567779e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    4.0160148244402230e+01,     7.1917714257599127e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.0108675702855399e+00,    -1.6639358641045103e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    6.0553864136114504e+00,     2.4016560830527687e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.1672976762180248e+01,    -8.5014811284892767e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.8505189628789843e+01,     2.4539568201548700e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.7097419795818276e+01,     1.5922657348476514e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.2304601962272326e+00,     8.7735275057123072e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.8113508747154629e+01,     5.6262465025870952e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.4913894139559901e+01,     2.4340493692059139e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4894849085446140e+01,     2.0452869417932452e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1232313596297615e+01,     7.2166658892984269e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.7905184454841532e+01,     1.0234277696016974e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.2655983813836492e+01,     1.4285810487515937e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5555563065364311e+01,     9.4078953735767063e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -6.7486250653806952e+00,    -1.1113619639929524e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.1381264754237517e+01,     3.2435423754581250e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.0056300539903589e+01,     1.5464757403089368e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.7648009480385074e+00,     8.4383275007492157e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.9431172201971210e+00,    -8.3816510460957306e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    5.4804872099362072e+00,     2.3589758433379546e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.7466071491682037e+01,    -1.6668199560503759e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {    9.8161193784985983e+00,     3.2739427210659451e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -1.3736251827502208e+00,    -2.7047461219777651e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.9866195986215818e-01,     1.4456991385429406e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5673757661118927e+01,     2.5132991916953924e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3105024776083013e+01,     2.5507963943191055e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.9640466929440706e+01,     2.6084021722664274e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3163593155637852e+00,    -3.2202766427999723e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.3028004214998319e+00,     4.6357697659085737e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2586559258675404e+01,     1.0606635149453068e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9698244696296143e+01,     8.3279754877561954e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -3.1267761143479945e+01,    -8.3294073614907011e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.0976504151834344e+01,    -1.6789464203770933e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.7304115410502696e+01,     1.5489432876812527e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1924752826271170e+01,     6.0971331061518285e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3200354817817047e+01,     7.7029419587152393e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3637162935423465e+01,    -8.3183733861468736e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.5131836759970199e+00,     7.7320230288112461e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2871110832706101e+01,     9.1949154573377327e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4299994990434751e+01,     9.1304658480087255e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.4933217310621565e+00,     1.5140916851480146e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.8891822303875458e+01,     3.9946440467437315e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.2428899062345593e-01,     7.6267875102248345e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -8.8783021965334719e+00,     3.4481464783680340e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.3660553567365241e+01,     8.9520679774113996e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.7896875191718760e+00,     2.3888366395412839e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.4443023710548455e+00,     8.1168150357786844e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.5203938581206886e+00,    -3.0501934885044625e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.5957882957507479e+01,     2.7637362101595558e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.4016067638403321e+01,     2.4205506958076970e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.0191416083598419e+00,     2.4750787658517795e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.3645306201470298e+01,     3.0499860810873750e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.5994772050329704e+00,    -1.7532454255661366e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.4789888097098626e+01,    -5.0205345821886112e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.2291841085240209e+00,     8.4449389003013202e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.8933820303290446e+01,     7.2167163812578265e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.5885711882058473e+01,     8.1620969147424767e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9249184294149252e+01,     3.1005326486635284e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -8.0528370771535318e-01,    -8.0682265105047954e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4891490947631841e+01,     2.3808167336138566e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.7195295944717586e+01,    -2.2119841217645426e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.4197565259449615e+01,     3.8785540103583415e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.5239877963091514e+01,     1.3870419269156806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.0008303610826532e+00,    -1.7161737507407172e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    5.9022508869203065e+01,     2.4030705322196893e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3476316065304175e+01,     1.0055418233593380e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9422207334968927e+01,    -1.3410987058090004e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    4.7113554012543117e+00,     8.2124439489455799e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.9601963868546672e+00,     7.7960127527448012e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6225725191227859e+01,     1.3424846119434020e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1794460732376937e+00,     1.8209330312030741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6784931564978274e+00,     5.3356990658208488e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5964786647007507e+00,    -2.0371824858844643e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2412711769092311e+01,     2.2287769177886246e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.6899518869242875e+00,     3.2198059596444997e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.3448428646878696e+01,     3.0850221964611035e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3228664551037085e+01,     2.6748138153777003e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.6535234876971314e+00,     9.4690591190366667e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2043098702921764e+00,    -3.1283887636467944e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9807964448990973e+01,     1.5545021278930233e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    1.3944907261813727e+01,     1.7130585912268194e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.3821284287533562e+00,     6.2118791349431613e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5893882653211932e+01,     8.8176269716440438e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0705497558630402e+01,    -2.3705323838893160e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2123956634050927e+01,    -9.4346834855057223e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.3738384946181014e+01,    -8.4952859345044235e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2115354279616671e+01,    -8.1867048137974671e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.4875509126970377e+00,     1.0229613687984078e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9911048938937341e+00,    -1.3051188248440485e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    9.2466839173457878e+00,     1.1152414766584017e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2323234368522407e+00,    -9.4885761396852940e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.6444073209140022e+01,     1.4269218439083115e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.8189198198872667e+01,     3.2202214682012400e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.7866256129557357e+01,     1.0992094481254280e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.4058334846511258e+00,     7.6999905933154650e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6049014325995817e+01,     2.3844215752542164e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.3430822813371407e+01,     1.6020438765818522e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.7667718735821293e+01,     2.3156397080632261e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.3012536328412075e+01,     2.4429863521632007e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.8068451417175783e+01,     1.6658478730114388e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.4238961206440692e+01,     7.5238512352205908e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2793281222514725e+00,     7.0281959946114029e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.0653662587827988e+00,     5.0186471084433197e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8382667253547048e+01,     7.3872163308384552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1204804240650223e+01,     3.7148894469653234e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    5.4408135199734133e+01,     1.5596075703151062e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5071391005014910e+01,     8.5333098581021550e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.3558851327088304e+01,    -7.6643354231743069e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.2301686967109660e+01,    -6.1156807223879705e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.0665891749132328e+01,     2.4031481268887088e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5375549850453545e+01,     7.3668661874411828e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2444608851148814e+01,     7.4619687191288876e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -3.1019304498895810e+01,    -1.2738881668567547e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7087451603217723e+01,     2.4045264840310720e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -7.4742074544571594e+01,    -6.1276555675711370e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6422254155897775e+01,     1.0600076193853933e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7723706908298382e+01,     8.1452532903605370e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5274377618809840e+00,     1.2090723362975685e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8895597383869500e+00,     2.2738768211884279e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.1258603285979724e+01,     2.3855697971090794e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.8093621493636434e+00,    -8.2293646297531704e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.4748893711258646e+01,    -2.3715960453142670e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.9741409551357965e+00,     7.0035814794349278e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6208919516917582e+01,     1.0024728230280671e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.0004543811274473e+01,     7.3222833082443176e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -6.2259087866066594e+00,     1.2334152182624037e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.7062315087453523e+00,     6.6813152541905911e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5622638765711789e+00,    -9.6463104940257253e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    6.2964836789289791e+00,    -9.6535807950917333e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.9376645819213579e+01,     5.1145235724437243e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.2569992016008662e+00,     1.2936722618053327e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    9.5579394954127963e+00,    -9.6521581041579811e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.2656359692993597e+01,     8.7704142601309959e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0190465831190032e+01,     1.2527203467356063e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9893111645387247e+00,     1.4285427774423832e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7418484550693073e+01,     1.6029403896576128e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -8.6611214338021398e+01,     1.0880005698226296e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8477433154668432e+00,     8.3143650981250339e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2495022662497766e+01,    -8.2396739110086583e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -8.9647332606001669e+01,     1.8580389537804207e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4968156549231306e+00,     5.1815111395869735e+01,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.4235467205484367e+01,    -8.6315408948863260e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.5543836477929105e+01,     2.6686634945811929e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8323062271934692e+01,     1.4192839115364133e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.9733378235680298e+01,     2.2932600453136438e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1119490181598689e+01,     3.1905890797654880e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.1483639668900946e+02,     1.1128975907878666e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3303645174247912e+01,     1.8751356397339550e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.1732079880633606e+00,     7.9110967668684398e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5302289121069116e+01,     7.7790920719372089e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6677275204628090e+00,     8.2473363940020008e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4702622576027444e+00,    -7.1834099429996495e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4120543867437952e+01,     1.7197776448071858e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.0704853544128650e+01,    -1.6517697438362324e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.5967649968777291e+00,     7.1967500026234220e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5276103661968143e+00,     2.1639887396133625e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -1.0657934349024419e+01,     1.5330903624304083e+04,     5.1870637791611210e-04,     8.2496477754015965e-07,    -4.0571148089970406e-09, } , 
      {    2.3983162930456068e+01,     8.0404785574923862e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.1337137317358994e+01,     9.3319710359061191e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2125163255826216e+01,     3.7906550475628945e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.7298482864337057e+01,     3.9936569395210194e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.1279748166976361e+01,     8.3270635012819112e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.0476382540907982e+00,     8.5891563387421047e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.1072738842022041e+00,     8.8091123447577829e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3403425614334637e+01,     5.7839808569275283e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5183539677378970e+01,     9.5541549995755595e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.8842296079983917e+01,    -9.4429913515137832e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.1786021281469100e+01,    -9.4436486162633300e+03,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    6.7036979932860987e+00,     8.3846470580711466e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8505688660540102e+01,     2.3353946336870606e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.3503512633536690e-01,     6.2265157380749752e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3902368012470422e+01,     1.6160041248304973e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.1782258380126628e+01,     1.1231339938463692e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8605702364115686e+00,     7.2518002244725558e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4951556740817544e+00,     1.3604862437633974e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9779037510377584e+01,     7.2216894489846154e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7929486977526352e+01,     8.2508105312365533e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3457304582165420e+01,     3.1985520601450706e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.4827635655266926e+01,    -6.6326546189662295e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.9631575066210935e+01,     7.2373693365667641e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.7243378470457804e+00,     3.2135702789414838e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    2.5976027966630078e+01,     3.8316492126129524e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -3.4348644017347993e+01,    -1.6656286968379412e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.2201617119783570e+01,     5.8895325243988618e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.9441530246411361e+01,     1.6801999292389064e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9689671222386657e+01,    -1.1151808487975891e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4329798927387493e+01,     2.6558158152642296e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.8849909227736532e+01,     5.8902371563150118e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.2369687657533609e+01,     6.5438068098239737e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2036207860927030e+01,    -1.1140762697326222e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.7304694924148617e+01,     2.1942423493536880e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6845377012044747e+00,     1.3846717484324574e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.5088048720509391e+01,     2.3258680157848132e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.9077505176797914e+01,     8.3900818292525437e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.0647021623981729e+01,    -8.2673010199943565e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0519087698625572e+01,     8.6622403271428266e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5148161882260128e+01,     7.1791179763489536e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -7.4007334145912154e+01,     1.9876819609793201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3134122593015150e+01,     1.9570123704822257e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1063340188202432e+01,     1.1330857311826217e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.8135804717906669e+00,    -1.3707083808353764e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.3982719439052342e+00,    -6.1027390110483316e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.2858284872799590e+00,     1.3453696514631585e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.3681770946460588e+01,     1.5953824203130644e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0098020272709022e+01,     2.3480725944853668e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9486435491156200e+01,     1.4759013550537229e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3886112758573297e+01,     1.1079080207715857e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0701845320191296e+01,     1.5136615409090540e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.2593839403928964e+01,     1.7794553313644250e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.7988455581860592e-02,     2.4011131125489817e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.2491551504148317e+00,     2.0649725696049223e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.5478881277026781e+00,     1.5734270071777388e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.7737141215156011e+00,     1.4465664974035259e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1141826620573195e+01,    -1.1249378405205935e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3556373348374741e+01,     9.9314282257540981e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -6.1136143651982522e+01,     9.9349098664746944e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.3315479275259179e+00,     4.9672707356677056e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.1920754465174909e+01,     7.2130812034421269e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.4286697868207298e+01,     7.2150445272745619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.3813021149263363e+01,     1.7754738411746337e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2852837753700802e+01,     9.4444235112827628e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.9580035959269910e+00,     5.6577267301008451e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0851282927075179e+00,    -8.2790280271883130e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.9350032508973726e+01,     7.2262687900823230e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.0053621015569635e+02,    -7.7008917982170224e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.1799120101242089e+01,    -9.5732396458290332e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.3367683365163984e+01,    -8.9507442791184985e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.1128283836634285e+00,    -1.5982674598328209e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.6342312056041681e+01,     9.1785460861487663e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.4003464862287345e+00,     5.4144597421331227e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4495395314574189e+01,     2.1066850108240687e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    2.0787779988803814e+02,     6.5852473216736835e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.4719301143321450e+00,     7.3424577946792590e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4463995916382739e+00,     7.9640563883407203e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.9366323068246754e+00,     1.3422544919492608e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6836529198318750e+01,     9.9030765033368116e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0873825154410724e+01,     1.9946167049932963e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8057953789619184e+01,     1.4365992580844240e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.8792744904185277e+00,     2.9246136261906431e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.2101966241835036e+01,     8.9434924983976380e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2858118734522236e+01,     3.1072390668765322e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    7.0462106956636905e+00,     6.5541160910794615e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2394976542565431e+01,     8.9389500662131304e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6535795467630876e+01,     8.9414569813805647e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1161483926946229e+01,     1.5734828922627541e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1931853192401949e+01,     1.3827048107734227e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7615900578521519e+01,     1.4310411993500191e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3237610030076569e+01,     1.7306353950549339e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5672513466075051e+01,     1.8169948150947435e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8673130312969100e+01,     4.6360695429875423e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6545103331299281e+01,     7.1983151218438952e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.2150298449409418e+01,     1.4956395175425079e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9430142412767211e-01,     1.9155194078289394e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9534546983201970e+01,     6.0192674722800839e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1186113996629112e+01,     9.5257880176431227e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0543255662633996e+01,     1.4936988610068496e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1844377083816642e+01,     2.3342464924429980e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5922065470054516e+00,     6.6015030238272520e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2455113603963621e+01,     3.0392766645104188e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.1533128200793739e+01,     7.4585382686262174e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.8921481508996347e+00,     8.4710066678242038e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5110782655190363e+01,    -9.9073008893007609e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.3277259038470799e+00,     8.4100464552448993e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.0694724404175290e+01,     1.9542758366321475e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5477038662212070e+01,     8.8981298583725729e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6773465366316543e+01,     2.2112967542006169e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4139024281805753e+01,     7.1342094044048099e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0223951166423895e+00,     1.4239738454017563e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.3900504507290457e+00,     9.3631209246142826e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0994235501377553e+01,     3.0628053260456749e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.5591775064206157e+00,     6.6234926394270133e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6257723655300978e+01,    -2.8601792763767862e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0355219090599892e+00,    -2.8444467923774873e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0002418694613720e+01,     1.5529191752903225e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1172360891922199e+00,    -8.3772512946643801e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -9.0801872175891285e-01,     1.3376144253619463e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0972446781889477e+01,     7.5775249643479310e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0429511559068111e+01,     1.5864781954261793e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.9227274031413472e+01,     8.4550416602227833e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4572411750893774e+01,     1.5647981138937301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7411608891761396e+01,     8.5013462173232820e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.6009020280826867e+01,     7.3031051142271972e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.4660781976380502e+01,     1.5684740900807776e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.9776521681819638e+01,     5.3140478448405065e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7610395604455004e+01,     1.6107783496560658e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1986707143999723e+01,     1.6230580452098753e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.5246625078146590e+01,     1.2827690129527455e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.5876999689866640e+01,     8.2168531761158392e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4716013682459469e+01,     8.8817479485982803e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6082835586375982e+01,     2.5085401068740626e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1686311434312667e+02,     1.5606842065489189e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7019153008547889e+01,     8.4000133624112132e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.0242186033158932e+01,     8.2234645756679438e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.5223970454070104e+00,    -1.1043192780096178e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6066860343398019e+01,     3.0379648733905920e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.6410097916257655e+01,    -1.6634840944114658e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.7717122803429103e+01,     7.2493999362950681e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.9722998102905986e+00,     2.3884563625803508e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.5607590926653643e+00,     1.6526870966762497e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.6327827478702400e+01,     2.4383079114862339e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.3351956067821561e+01,     8.9950829430896665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.6011094916233226e+00,     1.5899357438585766e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.2150472557003464e+00,     1.5580491649096006e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5376475610055124e+01,     1.6001845302138276e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7574286251612456e+01,     2.2791762044349533e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.2754816571460417e+00,     3.1300125117741583e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1449689818589421e+02,     7.2781506408657388e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5094559763248277e+01,     1.5004618204366361e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    4.0220130694705176e+01,     2.7473068791655991e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7105659457072690e+01,     1.8562969120640424e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1116075822614748e+01,     8.9761866794250236e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.9301335970345811e+01,     8.3244336689657284e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1310451128608822e+01,     1.6990885241868476e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2416109756765904e+01,     3.3707786668186789e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -2.8751765037419119e+00,     4.2871480392262911e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.3808004449889047e+01,     2.9531473644647504e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5916942096711614e+01,     8.5361633475760500e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.0969862747996189e+01,     6.9968251934616510e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1763821225084818e+00,     8.3150764435919100e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.4187039592123822e+00,     8.2937989795670292e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7495737898965107e+00,     2.4379096596802658e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -6.9622313502367263e+01,     8.3313279704903580e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0616607910320454e+01,     8.3260548787565422e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6059084036817353e+01,     2.1644489796016445e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {    1.0858108916904191e+01,     1.5760007221393642e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -2.1258316185521455e+01,     1.4953909290876089e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1924710504936680e+01,     2.3007851514207145e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1505471483974326e+01,     8.5201919469038912e+03,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -2.9070475659358790e+00,     6.2330746936741070e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.2414442835860427e+01,    -8.6169042917545139e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.7906512369457062e+01,     8.4298967311504566e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -3.9476028816640877e+01,    -8.2274861180964454e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.7054539118972549e+00,     1.6580941605836037e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.5839101610383542e+01,     1.6100779915081224e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.5589103913333979e+01,     8.8053249381474161e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.4066643448690424e+01,     1.6505123118499065e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5583518039789027e+01,     1.4253324094836609e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5834377286172565e+01,     8.2637773535071337e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9328405641533530e+00,    -9.5764282062139937e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.0365220495872851e+00,    -1.9152525753450340e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    7.7246764530603329e-01,     8.4339182735789564e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1153501618090630e+01,     3.5408739246586454e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.3344495133178249e+01,    -7.9537193983863253e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0107399255140340e+01,     1.6711455473852297e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.7647678277816816e+01,     7.2005003282797752e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0098384361720170e+01,     1.4910643935326925e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6218066661005786e+01,     8.3820128377927176e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2407634836649763e+01,     6.8053094116660595e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.9605812696481735e+00,    -2.1610842133543611e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.2102474597957993e+01,     2.7643921057194690e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7308676793788024e+01,     1.0373978366416695e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.5499023452366458e+00,     1.6099357224147472e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.2632268559178641e+01,     7.2191299374000337e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.5756087573283732e+00,     8.3635838696798710e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0918428765559085e+01,     7.9199227114160112e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2238379536827487e+01,    -1.1489987856728476e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7910605833688720e+01,     1.6658486376641107e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5302291271251724e+01,     3.3890104186328150e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    4.9788795666840429e+01,     1.4915413107337312e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -3.5216636707052317e+01,     8.2753700114541825e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.5436775198758603e+00,     1.6664386430726336e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1800247977436129e-01,     1.3060675544938600e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4265044022232438e+01,     1.7776927181254447e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.3851386936512025e+01,     8.7190760006825822e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.0555075104439258e+00,     2.3964745719131817e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    9.7548417933045144e+00,     1.4476685600757621e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2537112456766888e+01,     7.9895631091100331e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1366923393536851e+01,     1.3722265884659024e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.8636032072706598e+01,     2.2813905054003721e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3416176675774151e+01,     1.3964111455610711e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.1480979219670733e+02,     8.2871455542483000e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7135916436514691e+00,     8.3258378409347533e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.8608898892880719e+00,    -8.3315450083121486e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6081269807791223e+01,    -6.2396336492732416e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -8.7567617151539409e+01,     1.4286041068316868e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2808773762637390e+01,    -5.5961547957396397e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.2975526073661605e+00,     1.7583110199517447e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7016641789331878e+01,     5.7064133364002537e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    8.3948202500755951e+00,     6.5688945336652350e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4429373516719252e+01,     1.5780670737650536e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8168886208076032e+01,     2.0292787353752807e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.1009866547884020e+00,     1.6082755706879425e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7959701241587695e+01,     3.3794543728964559e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3656910306808943e+01,     1.8439016193755149e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.4001599361766395e+01,     8.5403184331434531e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    7.6191940088412125e+00,     7.0747060868485460e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3399321517802218e+01,     1.2620755236592184e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.2076863155099353e+01,     1.4359433625245110e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.0163779824345148e+01,     2.4988151816654052e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.6966344604620573e+01,     2.4497123467334604e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.0889929879612072e+00,    -3.5219207050248956e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2660428541594673e+01,     2.3868600800547454e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.9270990271735382e+00,     7.6371054611112277e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8626031586992795e+01,     1.7970664998252560e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8397949455424467e+01,     3.9336647743574911e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.5333977884615456e+01,     8.2381830840936909e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6511355628505669e+01,     1.2934352117715367e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.0746899506446876e+00,     1.5556316827060364e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0645574328599992e+01,     1.5335506024186903e+04,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -3.2127411102109910e+01,    -4.5350594916546447e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9838104599433457e+00,     1.0953481246907410e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9027698361397420e+01,    -1.0069326368992690e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    3.0831561338619824e+01,     8.0957060092123047e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2962189525702840e+01,     7.7000248100691051e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.9048646719926214e-01,     1.6660227763304691e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.4566429526636554e+01,     1.6170963823453496e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1632008844799842e+00,     8.3650579402314834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1305106728194939e+01,     1.4936881204643421e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9171629978902892e+01,    -2.8850395197565376e+01,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.3491125944738442e+01,     1.4604658335074006e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.7474025067207908e+00,     2.4983996731086645e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.5029997980553107e+00,    -1.8347349896346357e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -3.8731520986334118e+00,     1.5289418731173379e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.7091631772383256e+00,     4.9492370517269937e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9843643835028722e+01,    -8.3459302046354769e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.4906242646611922e+01,     7.7965220740632249e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.3733206394850370e+00,     1.6151874205271308e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1269786719853439e+01,     1.6569468863026315e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.3149176414346677e+01,     7.5583819692367724e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.1265512278318752e+00,     2.3874290628663035e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.8833962098252783e+00,     1.0240836651616106e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2744891963077292e+01,     6.7233326639685238e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4358357597126902e+01,     1.1258352961006402e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4717823533051885e+01,     7.9333692549890511e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5177289712262063e+01,     7.1376216219474818e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    8.0370240727608774e+01,     1.3936535848130550e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.4336350840851217e-01,     1.4692278000781964e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.5073041253054953e-01,    -1.1147769763223196e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4715264347587652e+01,     1.6654537935189110e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4695684943528457e+01,     1.3069338851697534e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2433068472504289e+01,     1.0095721535693661e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.6902399448627925e+01,     1.5819727634789941e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.4165913284859457e+01,     8.9412512661545425e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7182415779889908e+00,     7.2276254024369136e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0691754270321802e+02,    -1.7428380927368094e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.2315644287505489e+01,    -8.1022649648114402e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.1318960731320473e+01,     1.6155519846738907e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.8909547779713336e+01,     1.4913480302535188e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -2.0769655610896226e+01,     9.4509583175160242e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0803589115876473e+01,     1.4966261816332124e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.7005673666375294e+01,    -1.1144801422078917e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.7702223881076343e+01,     3.2864493140677834e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.0210467865527502e+01,     1.1224454655616524e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4248499880906786e+00,    -1.6733824092657764e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.2771126570502691e+01,    -1.7935783184133761e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.1422159891662309e+00,     8.3978099230153330e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0948268622299146e+01,    -1.1779069377317849e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3643050936380362e+01,     7.1250206164894935e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.6833732348748839e+01,     1.7346683833973111e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.3155982680244342e+01,     8.5431719226173482e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.8440970528287373e+01,     1.1315124827826918e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6687879866034052e+00,    -7.7342270096798802e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4962929662502169e+01,     2.2251368445524571e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.1720478058492745e+01,     2.1619646690942791e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.4761512355636892e+00,    -2.4973455035452360e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    1.6142465678515176e+01,     1.7620638799066537e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.7990756182631522e+01,    -1.9907450948956102e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0131765383076965e+01,     6.8494278290756147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4932782221245340e+01,     2.6547341441385444e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3410401012788228e+01,    -8.3303193479649890e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -5.5146993453861484e+00,     9.4923352225627350e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9776895997784589e+00,     4.5430981708357394e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9827737008232862e+01,     1.3273670720692722e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.9145912664688431e-01,    -1.6613612015689938e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.4462957464266328e+01,     8.5551178844354617e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9413063508770684e+01,    -4.9028363858538487e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.3977656077131373e+00,     1.2037754265764492e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.2925070534651695e+01,     1.1191525036565781e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7926307759045542e+01,     3.1691274259876889e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0049812255595604e+01,     4.5711365203934038e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.0281891081657077e+00,     1.2636502980106634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.8429442706153580e+01,     8.3754118564269575e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4078200913610083e+01,     9.2802855021195569e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3437055011152529e+01,     9.8754712728337029e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.0281866419675438e+00,     1.8762173108596406e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1820392408936282e+01,     8.3644217418226126e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6893651088151870e+01,     2.3934724093071924e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.4157619412028808e+00,     6.8030082117246475e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6518816618480852e+01,     6.2291521317088300e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2957666378205127e+01,     1.6701153682803866e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.3227838615289144e+01,     8.3581883756349689e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.3751368609257462e+00,    -8.2991944736119331e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.4492099487164360e+01,     2.8398848116711451e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.1265847430889821e+01,     6.5847888779117384e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9889585522723664e+01,     8.3685426106363739e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2079824817310890e+01,     2.5758998302058058e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2943798428436841e+01,     2.0791712306875204e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.0811109604760958e-01,     3.2748814920478413e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.5364415754492189e+00,    -9.0643767664772258e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.0732746587892073e+01,     2.2400214006736169e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.4133073689330139e+01,     1.7086615924802882e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1214630204023431e+01,     7.6944837854926773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5328911548026962e+00,     1.2667654575245611e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3222842317880673e+01,     8.8238308531835282e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7578540732731231e+01,     8.3957556067534933e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3978886667611661e+01,     7.7598871571341697e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5390405418477862e+00,     4.6625961670003977e+04,    -4.6257120609174708e-05,     4.3707204171321924e-07,    -2.4600415806860071e-09, } , 
      {    6.3125746355211049e+01,     4.6630564069886794e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -2.0245598224087637e+01,    -1.8700756060405729e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.0635343131572924e+01,     1.6625902621733148e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7107240963610117e+01,     4.0564020811097453e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.2751045858114615e+01,     2.4998693512288337e+04,     5.1870637791611210e-04,     8.2496477754015965e-07,    -4.0571148089970406e-09, } , 
      {    1.4743850654514961e+01,     8.1955232681379875e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4492945577512543e+01,     3.1338844138772001e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.2241449326371363e+00,     1.8451078532387173e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0948790247221048e+01,     8.3063847255098663e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.2172139884331344e+01,     6.7872604682101974e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.2055390571553133e+00,     2.4754184739326694e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4088038081897444e+01,    -3.6236804733429017e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -9.1848933059956209e+01,     1.9502943464423563e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.0502072709211028e+00,     1.0511033944985082e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0022310893399934e+02,     8.3263969382327650e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3357912109849021e+01,     1.6538345695104075e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    9.0991208509303718e+01,     8.0189030547612047e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2461933926284416e-01,     1.2476138793450018e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7940035266305165e+00,     1.5658036852511015e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.8536987661074669e+01,     1.8955990090333413e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.3060686865342905e+00,     2.1435767635435193e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.3058913012545084e+00,    -1.0070916223467220e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    9.7998997765183390e+01,     1.3799811432120419e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9239298561183965e+01,    -8.3294127803940182e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.8509115658662051e+01,     2.7975474537240607e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.9095077148470594e+00,     9.7249920055991079e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1230441099195616e+02,     6.6498430558201962e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2997962758706038e+01,     3.1317398114507247e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.5740272504754236e-01,     4.9426780961278610e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5075330583050047e+01,     1.6346845172823630e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1856273462442459e+01,     1.1661349297606748e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2284203603757268e+01,     2.5092012468292731e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.1252569166294482e+01,     1.1571074959436370e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.6466084146389370e+00,    -3.9639911243850365e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.8723047130603416e+00,     6.9853257298660119e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.3827272952569260e+01,     1.5224984381359518e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0327551690530972e+00,     8.8613700965020980e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.3888853993716204e+00,     5.6550968977846615e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4660262112740936e+01,     1.7699374920242324e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9576901473867441e+01,     1.7844819586786078e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9299909443422990e+01,     3.1078949624364457e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {    2.0613567488404789e+00,     4.7538016223873628e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2448315764557947e+00,     1.2083117823757031e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2622763446738595e+01,     3.5737325866465580e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -8.5254659100996747e+01,     2.3243040077985861e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.9275147739986771e+00,     1.1080171597130000e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7974367832202088e+01,     8.3335614375119694e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2883187105139777e+00,    -1.2477367815905432e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.2479116398449552e+01,     1.0937836342133231e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.2631921302373815e+01,     1.4308681025022952e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7675661315131954e+01,     7.0432105998196475e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.3899717887101332e+01,    -8.2214526436536926e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -5.1235380168957789e-01,     6.0626633160081496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3294410437138074e+01,    -8.2919316627529734e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -6.5168717467411213e+01,     1.0563217451520237e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5024668272162106e+01,     7.6841745042371904e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1218119080156299e+01,     7.6534845009244800e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -1.2890182395924846e+01,     9.5234868177017124e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.5662152629531150e+01,     6.1017355060346490e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    6.3889276037097879e+01,     7.6762053129066426e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.0185425308637868e+00,     1.0344294999908332e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0176405596888102e+01,     5.2335801363439250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4828442200823794e+01,     6.1770018266205152e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.0258363597882465e+01,     2.2661645323255885e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1072456843257381e+01,     1.7267268089699319e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.1067871273316527e+01,     1.5933903432117104e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0452273916404927e+02,    -9.4432219323147146e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -9.1578831830396354e+01,    -9.4434180354623932e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0848819395214916e+01,     1.8958736800584309e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.4811644199664684e+00,     1.0118012975292095e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6393874474873016e+00,     1.6656219387692669e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.5947600665854893e+01,     2.2891803889322306e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.8807571323150416e+00,    -4.7976201862042853e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -3.4491635473686991e+01,     1.5520006669623528e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2005515477799053e+01,     7.2128994038041128e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    1.1882467230272757e+01,     1.5644925998909863e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.3411208803200211e+01,     1.4793914752814739e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.8262742819820289e+01,     2.3902098636963943e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {   -1.8238171210417519e+01,     1.1767617091375523e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8181391327580370e+00,     2.0256704726356234e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2719178336411812e+00,     1.0053995542659630e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.7482420109106499e+01,     8.3288913981978585e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8476714517059676e+01,     8.3284914510490416e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7020450469321471e+00,     1.3815565889185596e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9299008026682376e+00,     1.6163435291401067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2335868239338756e+00,    -2.0054547072581418e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.0527810110759134e+01,     1.7420677168274611e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.9976306687721266e+00,     1.5951523003189232e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8944986636270190e+01,     1.9566263991438245e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.9072296576847005e+00,    -1.0697465853255039e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1194890264628764e+01,     1.5041003461869610e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0219197795253945e+02,     7.6998758809387891e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1212486067788852e+01,    -7.2080025825394923e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    3.7703292950569832e+01,    -8.3913232472761774e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -1.9217549028732759e+02,     8.3284953214757734e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0316978469528078e+02,     8.3288875277711286e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6762295259690561e+01,     7.8358114948047551e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.5866372379587823e+01,     8.5744863659165185e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.0521683928560179e+00,     8.3292350159584530e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6904870147219413e+01,    -8.3281478332884453e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.6105642769378804e+01,     1.4954613922792239e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5743420529069887e+00,     1.2243482010413647e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.4309036304445691e+01,     2.2734275250207895e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0128937810130685e+01,     5.5060716100570880e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.1028743177585934e+01,     1.5933515290323567e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.2687095693048551e+00,     7.0901099384730733e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.8830604547888035e+00,     8.3847283968983265e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0090598094357290e+01,     7.9557816816548730e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -9.3314594610278817e+01,     1.0545944454754568e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7218580051865704e+00,     7.9820900722814313e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.9502887679439063e+00,     1.2274633605552624e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.6477253351669141e+00,     1.1941131212931050e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6687353564365129e+01,     1.7843396895852326e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7380400269364628e+01,     2.8030383071288546e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.1387084760733572e+01,     8.9937587715394675e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9018793964910031e+01,     6.5852029907803180e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5847653844866731e+01,     1.1361525493700068e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -9.2493341494217631e+00,     7.7381211986921189e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3373317339300046e+00,    -1.0278875103376643e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.2554819305092934e+01,     2.3559926376265765e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.3452302948182915e+00,     5.7870941025088387e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.9472407718729464e+01,    -2.5004123217326218e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.8762974984291674e+01,     1.4350840189291141e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6403420471523049e+01,     6.5859036973700149e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7186384749239334e+00,    -8.4602440034673255e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.0416169560884647e+00,     1.5543883540543056e+04,    -3.7445609556616870e-05,     1.6598473885547775e-07,    -9.6119160416776602e-10, } , 
      {   -2.1893842894127744e+01,    -1.5541625039420536e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.1422568140556169e+01,     8.3992326139490851e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.4738090677420317e+00,     8.3133135440739679e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4084254995348132e+01,    -7.0099431046599166e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.5291837093491807e+01,     7.1427172439872802e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3030793669448439e+01,     7.0630447193796851e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7817705032599015e+01,     1.1739397128613791e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    9.4059314705157373e+00,     5.4155706037919140e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.4194593975212236e+01,     1.5540014141048568e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9726744085598650e+01,     9.8850275598809476e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0941632648585188e+01,     6.4122884419730708e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.6913605447946947e+00,     2.2128838202669365e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -9.4099764638303540e+01,     1.8874635879378020e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1477184862730969e+01,     1.6100397201989119e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    3.2868412868997368e+01,     1.6099739937239576e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.9944817386845006e+00,    -1.2543481811426479e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    5.1581784302119624e+00,     1.4632759423710548e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1549145912553612e+01,     7.8313498236815240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.6793495555635474e+00,     1.2543701648360338e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.8731161115863735e+00,     1.7976131169210435e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6179910376058022e+01,     3.1191446774385971e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.7184996672596426e+01,     1.4523726935184673e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1665892979992748e+01,     1.5588533328180427e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -9.4323651663089947e+01,     8.0725290164703074e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8613836312681862e+00,     1.6093064988134913e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.1304070648277751e+01,     2.2688125049868559e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5273648320591473e+01,    -8.2385199251933391e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -3.0296813699888698e+01,     2.4357414372866733e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -4.8929524447827440e+01,     8.3282686135573931e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -5.6782226202190834e+01,    -8.3291142356895070e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5404989722977685e+01,     2.3819187962860928e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.3397095551351779e+00,    -1.6652973622076646e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    4.2427919061213672e+01,     6.9682679240652751e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5044066850838234e+00,    -1.1021770397877963e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.0524457850119491e+00,     1.5007746709462826e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.7840414702772861e+01,     8.3313145433889076e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.6846120294819670e+01,     8.3260683058579925e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3372048453489739e+01,     7.6217204381831452e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.5457473101675898e+00,     8.6933264609061789e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2720517564707025e+01,     1.5461399259360345e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7759050726977819e+01,     2.8313288811792190e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1701932054721368e+01,     1.0218545212017674e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8475330031148779e+01,     1.7685200949685626e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6562747027941981e+01,     7.0841432034178670e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1358855161723687e+01,     8.7282019400166337e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1121707579866019e+02,     8.3309859110141351e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5212687167066745e+00,     8.3440693051729322e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2099086499677348e+01,    -1.9517742645800738e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.3483324660701456e+01,     8.3961153036952946e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.7118497419610545e+01,     1.5545494438915021e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.6839523127066371e+01,     7.1913152450000798e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.8678245109968881e+00,    -1.2763255779713600e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.3210139409078735e+01,     1.6028746631826583e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.7209034039355515e+00,     7.1953273116896689e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6962708541987663e+01,     2.2756425248882078e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2115537449493647e+00,    -1.3679488585583708e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6059892493536739e+02,     8.3278311500262571e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.5902940848818361e+02,    -8.3295516992206431e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0126566800702426e+00,     6.5782771137437439e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7238844770585018e+01,     1.4356780109345624e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.1847924551677362e+01,     8.3462635167378940e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9334630488064228e+01,     1.0627298665709961e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3512174981259363e+01,     2.6152019271751044e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7242977545770206e+01,     2.2127798224827722e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.0275127326762146e+01,     8.3502669273546326e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.1844643773945961e+01,    -8.3071159218922694e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0057563722223273e+01,     2.4035739024544808e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.4982633790342149e+01,     1.3011161633484138e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4343622294809007e+01,     9.2741060600265573e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1485024355844669e+00,     3.2194103664785940e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    8.1391647840664998e+00,    -5.0765088219741426e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2350747562666889e+01,     3.2206170613671446e+04,     2.6931621671538551e-04,     5.9995954836692873e-07,    -3.0545685978306309e-09, } , 
      {   -2.4508579946233731e+00,     1.8369152138903417e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0659944782242074e+02,     7.7644716150853019e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0332540162224742e+01,     1.7337006872908030e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.8209982346781331e+01,     1.7003504480286458e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -7.8802367974798102e+00,     5.2620759280916736e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1292998947652677e+01,     9.6251483049885501e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.8432034478395469e+01,    -8.8601555405439285e+00,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.4148785664084848e+01,     5.1543384743416364e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8824275889764444e+00,    -8.0999637648700282e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    7.6446517468404931e+00,     7.5633130014943699e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.4933732192030810e+01,     1.5231234951642311e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.2688177988171940e+00,     3.1532014652285241e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.2405698337627573e+00,    -2.1063982433002352e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.7679766120249880e+01,     1.3817867089127009e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6265480713009353e+01,     1.7285361801917672e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.9999672137202666e+01,     8.6384797944856946e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6403094734726373e+00,     7.3319555094247626e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.2540353585509676e+00,     1.6498475557613325e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7967518653795782e+01,     6.5829103662550096e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0346817564854156e+01,     2.5597108862150224e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7383660674238079e+00,     1.2869159763143005e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2113155206714996e+00,     2.2726164232981442e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -3.6210919457577980e+01,     1.0976908498787789e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.3816507276009261e+01,     2.4928986375206769e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.8983773632661967e+00,     3.6728382007673349e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -5.7837477284919316e+00,     8.3390094631000266e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.4770359452105538e+01,     1.7736586798200155e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.6858168942125559e+01,     7.9041902274167132e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.1540141125027663e+00,    -8.9635579652681281e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.9001979342303081e+01,     8.3551419052384372e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0770916559031036e+00,     1.6153012931571477e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.9223297428157952e+01,     2.4080642864383473e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -6.3924406994059865e+01,     1.1293696745493849e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8100921652707825e+01,     2.4081347496299619e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.5750971328767923e+01,     2.6951179122335285e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.7382502489754152e+00,     6.5853620340503594e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1504020880755636e+01,     8.2834372543585705e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1927373370526785e+01,     2.2141627481492862e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.8256274932704284e+00,     7.2192515739766968e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.9130709486650204e+01,     1.9702017973913124e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9848366842369487e+01,     2.1993522836312433e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.8169082067707905e+01,    -8.5217324596702601e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2824528891321219e+01,     1.7383906211561462e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.1519241538926533e+01,     8.3931275094346947e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5874561422427302e+01,     4.7167552941376140e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    5.2421312256063018e+01,     8.3931240394399192e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1248672868899490e+01,     6.9248835958262189e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    7.7889354198297154e+00,     2.2333143263288461e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.0303657705736065e+00,     1.6075460670559354e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.6565179880994194e+01,     1.5280332208283835e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2047206971620504e+01,     2.4981143241612754e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    2.7785479868791874e+01,     7.3274740966093304e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.7740697609436967e+01,     2.3349787669448961e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7998460042852386e+01,     9.0764437153627296e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0264700589155673e+00,     1.4370747406335238e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.4789682309646324e+00,    -7.2194925703962181e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.1491107189492517e+01,    -1.1303610432644055e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.6984446215550932e+01,     7.2018569406343649e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.0206805495364360e+01,     3.8392871449848506e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -8.0708035879489159e+00,    -1.4885047046347456e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9664844041865726e+00,    -1.0856515522504753e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    8.2778351410440720e+00,    -1.0190685853873151e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4781068540598717e+01,     1.2967390799927176e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9387229067388752e+01,     1.6774776820533039e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.1564546738321301e+01,     1.5751951439760023e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.4489412248087000e+01,     1.7867098680684558e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    5.5731023568651437e+01,     1.5752656071676172e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    4.5912037290535707e+01,     7.2135984035710389e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -2.3546093887503300e+01,     7.2145273271456535e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.2320466433516479e+00,     1.6081333015945673e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7733655021257375e+00,     8.3448408258041418e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.4450402169839720e+01,    -3.9114914213530847e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2664606375238092e+01,     2.5593051950938039e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.1011946001002304e+00,     9.8957995964929305e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.2308149252434077e+00,     6.9607273065499267e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0367659438554108e+01,     7.1327078347368943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1048887925897006e+01,     8.3362435909997475e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2020700569261109e+01,     1.4923306860476796e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.1118651732972381e+01,     9.6233905281352108e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0951247604007683e+00,    -8.2933543536867255e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1037659930522850e+01,     2.9733154920052002e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3562285341246399e+01,     7.2152263269125760e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    4.1828736110530052e+01,     7.2113227164251193e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3836306021194815e+00,    -1.0267145730445218e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2894338074925255e+01,     2.6165137182949311e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2806615715797442e+01,     1.4988745710125295e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.6367334984875349e+01,     1.1995272714275061e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7623197132334887e+01,     7.1588020439446518e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.9989140535367284e+01,     7.2693236867720352e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    8.5095115641237030e+01,     8.5226988620713219e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1159476539975612e+00,     6.8122254268566967e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.1726885122960384e+00,    -2.8962619037733575e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.0585432500666769e+01,     1.2854722313919543e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6363180058195148e+01,     7.7474528827038685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1237469549872721e+00,     1.1850461040720660e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1835307245697226e+00,    -2.4184557095168930e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.8777962700642927e+00,     1.0173799277862166e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.0146470290095460e+00,     7.7032287536356980e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4572179042245445e+01,     7.6740754489513492e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.7747567351521312e+00,     6.4633179324734215e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.6813086402394575e+00,     1.1690096854064044e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4551674672952625e+01,     8.3538363536957768e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.6856401327867786e+01,     3.8786244735499567e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.3514642528507871e+01,     7.2142123513390270e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {   -2.1148699125475471e+01,     7.2139133793776637e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    3.2306183071788062e+01,     2.0339188019625952e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9346784953019345e+00,     4.5909101215648133e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0648128788134521e+01,     2.5365977332365070e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9463012347332008e+01,     7.2168030142915695e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6894025684651350e+01,     3.1572540899650030e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.4986085408236232e+00,     7.8761575076905319e+03,    -2.4672627970781806e-04,    -2.2562162128739358e-07,     1.0569859394181992e-09, } , 
      {    1.1767884139295189e+01,     1.7461452842742103e+04,    -9.9761634931308941e-05,     2.6330037095586454e-08,    -2.8787751570603142e-10, } , 
      {    7.9675760440743364e+00,    -6.6209464233218637e+02,     2.8250695821658283e-05,     3.7020469652260392e-08,    -2.2902113481933369e-10, } , 
      {    2.3851919158454379e+01,     2.2723035727884977e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {    1.6166298787614643e+01,     2.2790598582795297e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.6382189453364454e+01,     2.3923703466349561e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.3347372318911432e+00,     7.1154474159549436e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1052135532450432e+01,     3.1379754921537402e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.7679978188442295e+01,    -2.4671583066837993e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.3403203043862888e+00,     5.3339001734552448e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3693798044319117e+00,     1.1881612635859636e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7525412523384908e+01,     1.5062992271361367e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -9.2271930835194311e+00,     1.4354831225362290e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.9114146817296902e+01,     1.1466300226568219e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.6295962371176689e+00,     1.4860766262258185e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6165315648826013e+00,     1.4988034364658419e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5080322135941495e+01,     1.8832037163448138e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.1320764357606453e+01,     7.3549459487901195e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0177354769674487e+01,    -8.3905112016362727e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    5.1000615594000376e-01,     8.2829123864248177e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.8665720151484160e+00,     8.8194216260132707e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7846628979308662e+01,     1.0993097163546921e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.9267724525803004e+01,     3.2184595110940267e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.3046729109323500e+00,     4.1500772011427525e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.7019717681506023e+01,     8.5537829186481813e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.5622607959436152e+00,     8.8774385440152000e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.1570123040779194e+01,     1.5565501910340059e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9760513897101632e+01,     1.3602474604658822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5764449565080589e+00,     6.0768902253456770e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6228054521457324e+01,     7.2143816690098483e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.0053559123131039e+00,     1.5533875901481253e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1314156237411760e+01,     3.0421152578514448e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.0137859452210087e+01,     2.3878923881174302e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.0016381350292569e+00,     2.3791644766874015e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4776650706834317e+01,     2.1766655455255735e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2536973091766344e+01,    -7.9798230833329935e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.6489328253974609e+01,     7.9483086448262702e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4491814391556499e+01,     2.1834263228720782e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.6713597457338807e+00,    -1.6550221580900652e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.9207122155994149e+01,     7.2368104857166090e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1298197404464602e+01,     1.6656570803606999e+04,     4.6351172359218639e-04,     5.4853225840408113e-07,    -2.5876930229221479e-09, } , 
      {    5.8734386549327596e+01,    -1.6658194894886801e+04,    -1.4565790745711834e-04,    -4.5434323821453780e-07,     2.3562429715604555e-09, } , 
      {    9.6133250604192693e+00,     2.4516673980458381e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.4069938265782989e+01,     5.6632013536934428e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0008477405763585e+02,     8.6610216996179388e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4002771483559821e+01,     1.0999656119146053e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0301807258824262e+01,     3.0551255086750480e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8551092057118510e+01,     7.5318327739806191e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.5410366976254082e+00,     9.5703502615764228e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4936678933550013e+01,     1.5762308421335050e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9961729011659557e+01,     7.1207628608317709e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.4822975852492926e+01,     1.8307018989030974e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8135258828153020e+01,     9.3657524633612811e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.0316015310761522e+01,     8.1356503895766400e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.2951202610672077e+01,     2.4266767884239645e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5731824851170996e+01,     1.6483881799914197e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    9.1971362525935348e+01,     1.4985433296798517e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -5.9208486007129885e+01,     1.4985446723899968e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.3862119320643973e+01,     7.2137440617068405e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7641375980569400e+01,     1.0417749199973659e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7660809191460014e+01,     1.5115951892833647e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -4.0791804217376777e+00,     9.8853561922557219e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4347532052751646e+00,     6.5857447581089928e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5543898811078183e+00,     9.2674920114473880e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5326847453219017e+01,    -7.9853460953534523e+01,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7096310097481826e+01,     2.5526012886646913e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7184299584286471e+00,     5.9394234870654591e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9965083573761270e+01,     1.9349011060026402e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.3554320985041628e+01,     2.8036942026887671e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3894230422395694e+01,     1.0544815372440242e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6192770349665054e+01,     1.8129607767893140e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0712915558763278e+01,     1.0719062889575645e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5810069269888254e+01,     2.1558294017905428e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    7.1884715384938520e+00,     7.5606831691781854e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8172609344297474e+01,     2.3250616296128759e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.5168893334272703e+01,     1.7039587107683030e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9829940349355937e+01,     1.9047754314199097e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.6029168796730584e+00,     1.7232009991614377e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.6291362105909286e+01,     5.2339682781374604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7787589157742145e+00,    -3.0977670916707521e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.9631409707864606e+01,     1.6480850244889582e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9728520680792059e+00,    -8.5338316631894922e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.5642374507003957e+01,    -1.0661771589843593e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7522451374955253e+01,    -5.8466126795802547e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1752502187703445e+01,     1.4434737130268793e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3615533998316236e+00,    -1.1015403862660503e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.3348342365274295e+01,     2.5285795239036765e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1683245488372247e+01,     2.3863967548036188e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.7043486654680544e+00,    -1.0513501807984267e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9693912031808470e+01,     7.7802373005314412e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.9228993460680508e-01,    -9.6720476436419776e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.5167835538681951e+00,     8.6939310716102555e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7321953174660649e+01,     6.9166431001333422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1073466682122266e+00,     5.9361177872894059e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6589405781287219e+01,     7.9482896933484499e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.3896863955149158e+01,     5.1038429009709616e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    8.1861595709172619e-01,     5.7199011105634872e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.9562503845289267e+00,    -9.0305953390220675e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.9384763608272777e+01,     8.6813720112621450e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7720744130377184e+01,     2.9435872440179515e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    7.0939118646694332e-01,     1.3515844924019184e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3426495875076039e+01,     3.1415379252277162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.9541910983496294e+01,     2.3869714746128004e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3860181049558955e+01,     1.5330877901137030e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6349540446599967e+01,     1.3625016509791067e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6813224751147722e+00,     1.1457440071027671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5371507754667668e+00,     2.3862585559064701e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.4325312380046370e+01,     2.3880305870145792e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.6043290573630237e+01,     2.4287876910806022e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8716780260034938e+01,    -9.4035050819906410e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.0845892240108206e+01,    -5.9061312566516672e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -4.3295192195992387e+00,     5.8400537239811201e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.6253022415370296e+01,    -2.3708914133981166e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0062756821388284e+01,     5.0290905126805292e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.0857111638573373e+00,     7.1257566972104314e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5764506015425340e+01,     1.0051950702136699e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0381326528349888e+01,     1.3781793937469347e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -6.5589575367183357e+01,     1.0229156208048509e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0524731799430288e+02,     8.9564910510298778e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.3421375613992375e+00,     7.1353546329990422e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.6580406206185785e+01,     7.6761109659684244e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0813335591790249e+02,     7.7708691863517815e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9145323701240969e+02,     7.7718851036300130e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8602207848629504e+01,     3.1165690673291894e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3345769958648539e+01,     1.7343565828507166e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.4216585344389856e+01,     7.1986849848088614e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0486237735744517e+01,     3.1084797234496713e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.8756775129768069e+01,     1.7216512001740535e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6150515231126466e+01,    -1.1270800787424132e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -1.7268575125515667e+01,     7.1407683600039454e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.1471443172626710e+01,     9.9165077873946993e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.3399798662267948e+01,     2.4188233961311307e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.3015232317452430e+00,     2.5315926564976562e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -6.4619004885547753e+00,     7.2129582736290258e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1945591441783323e+00,     1.6656987015206887e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -2.0860664037790855e+00,    -1.6657778683286910e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.6878025907841288e-01,    -1.0032948707978203e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.1028656596187751e-01,     1.9863683756663246e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0563117369361420e+01,     1.5837559482405766e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    6.4983068045746570e+00,     9.1452185674274424e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -5.7837834555409664e+00,     1.1488591666166649e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.2768818751046304e+01,     3.2215679167517126e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.1489443301075347e+01,     3.3301647787295537e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.0071674580580350e+01,     1.5551632678482338e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.9106842295191129e+00,     8.3120888430202922e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5307054754243364e+01,     7.4470482807694898e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.1299940970475681e+00,     1.1297075884371056e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4681012928782775e-01,     1.4980569997460721e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9256152862522811e+00,     1.3655201702528982e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4107348639860815e+00,     1.5496975251783158e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    5.2984658894508669e+00,     1.4710311684722674e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5476933119509610e+01,     7.4155680532557408e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.4517588089997599e+01,    -2.5409782931945811e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.8218767152043842e+00,    -1.7642660522885008e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.5902395109847598e+00,     8.1231438563676738e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.2755570709711417e+01,     1.4284061577107008e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9694211135562512e+00,    -3.9957992529212106e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.2794195877975437e+01,     2.9828541093914428e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9317633600628035e+01,     3.7635215443872796e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.6292795051722717e+01,    -1.6184243665752226e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.4313013069636227e+01,     6.4558356183723245e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.1160255762416490e+01,     2.3325621422829419e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2833649335437883e+01,     9.3987218451765384e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.6695471090505997e+01,     6.3087819355046540e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7908785246346135e+01,     6.6610063413835142e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.1903181030860122e+00,     1.0350680936113131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.4865056846808820e+00,     1.5447153085513812e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.8370187462745153e-01,     2.6251856294749285e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.3255115991921333e+01,     2.1641836280116964e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    2.8823926321213300e+01,     7.2151674570876630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6563073226668784e+01,     1.5527376409432312e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7786603319180898e+01,     2.1642540912033110e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -2.5056627201356140e+01,     8.6792610987846710e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3737352453046809e+01,     7.8177778108640441e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.0726048539410973e+01,     7.5268827902452040e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.8981598696870201e+01,     1.4914595121993465e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.7129317246772544e+01,    -1.0071979884892173e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.8490830321071542e+01,     2.9813145666358054e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -8.6194997870922506e+01,     1.1214066941698024e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5694583301905951e+01,    -1.0281529350628476e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.9929883740034377e+01,     3.1905684234170476e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4233808577618965e-01,     1.1064419101334683e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5807061867828455e+01,     7.5272303740977968e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1088989452282084e+01,    -7.7702736175966902e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8530621356310341e+01,     7.9866117780396235e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.6294086919204522e+00,     1.0278377161948709e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5165105662682480e+01,     8.2065438948603514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4455036676654780e+00,     8.7987058044784080e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.9820691645961730e+01,     1.6228149773690919e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.3750390675488937e+00,     8.3298643619165923e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.7279544725496145e+01,     1.6508430689493252e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.6200562305631268e+00,     8.3275184873303078e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3101910693320589e+00,     1.2983123283926476e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2648770804280886e+01,     7.6316575231133547e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -1.1910593578709077e+00,     1.0671398131641698e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6379274822863337e+01,     6.7622878846701187e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8350558976558791e+01,     3.1067484371761791e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.0302126501072678e+01,     2.1500913330715932e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.6921166688428542e+01,     7.8422723988300477e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1471957731350148e+01,     4.9641574900863925e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.6268456889614864e+00,    -7.9560465653792562e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8930973052655986e+01,    -9.9858960125705835e+01,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    9.1260583760999850e+01,     2.4970532245581919e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.2115929657873544e+01,     2.4424502238580077e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4270813931053556e+00,     1.5422097587133665e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.7476231867305962e+00,     3.5191397837810246e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1642070843542236e+01,     2.7866089761348980e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.2923955237484677e+00,    -6.9481233276470930e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0595441496276361e+01,    -9.4372865094458011e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {   -8.5487065380388905e-01,     7.1140247250211905e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3724180212260922e+01,    -5.7226597526949827e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -7.6520090295110990e+00,    -9.4493534583313049e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.4518799615792403e+01,     9.5879364270307196e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6443784136696035e+01,     1.2596172072791251e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8657311190404098e+01,    -1.1174734733228970e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3596490787769216e+01,    -7.1032278496713416e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.8727047478896417e+01,     2.2335444463229869e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6678080996415208e+00,    -1.1222304544602801e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.1034445395187245e+00,     2.0663516256893672e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -6.0813694433078467e+01,     2.5685872703685145e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9676825967204802e+01,     2.1447232888037863e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9653103498199201e+01,     1.5247949097557825e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -7.0843812173515374e+01,     2.5001616302158778e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -9.7894277044772977e+00,     8.3271908428685256e+03,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {   -1.1358944151661099e+01,    -8.3301920063783764e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.7626980061110125e+01,     1.1241432280198806e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5437165772646580e+01,     2.4771457736092358e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -7.1954444271999378e+00,     1.1095570696473662e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9696962066798555e+01,     1.7332091100165591e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    7.4502296313988934e+00,     2.3782428200990456e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.2383956738739855e+01,     1.6460333437491563e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5412664958118918e+00,    -1.0464648538627085e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -9.2625677090016616e+00,     1.6326495029426358e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.0873288044973584e+01,    -7.7092439951184515e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8350206044235660e+00,    -1.8558053347897985e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.6323981199783805e+01,     2.4691269081446382e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.1234966697917763e+01,    -6.2239011652739428e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3074499769155276e+01,     7.2178293508526585e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2657331697206413e+01,    -6.6169221349669315e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.6986056447203470e+00,     5.2647057604078582e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1823160911139533e+02,     2.2756309196701022e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {   -1.7507179851173652e+02,     2.2757325113979259e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.7144792427320317e+00,     1.5687869405904239e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.6844399227956242e+01,     7.9859936827688889e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8628166843347380e+01,    -1.5387945627793979e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.3454802626218729e+00,     4.2314322317642027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0522022124220815e+01,     1.6872965789425954e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3530357565559727e+01,     8.3047671623364222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9103888463079468e+01,     8.3284763428818424e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.1096000700322257e+00,     8.3289065063650560e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8204280950731726e+02,     8.3544918566108918e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.8047329306013344e+02,    -8.3028909926360084e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.6760655766727512e+00,     7.7521894752348662e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.4547771869311305e+01,     1.5638355494449777e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5308324569297856e+01,     1.0151794402747257e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.1927860409055020e+00,    -7.6328753751612639e+03,    -2.1658957915720006e-04,    -3.2299790357928725e-07,     1.5292526424606202e-09, } , 
      {    4.7992293966239084e+00,     9.6190523821756324e+03,     1.1614799982552650e-04,     2.1550394761358242e-07,    -1.1710092690783507e-09, } , 
      {    9.5534652324515967e+00,     2.5648133276208475e+04,     4.2339669684308734e-04,     7.1632530380872929e-07,    -3.5885375380678631e-09, } , 
      {    1.2439503692676702e+01,     2.4462517041361858e+04,     2.4067658251175289e-04,     4.9604889879967261e-07,    -2.5021234082063148e-09, } , 
      {    9.3248633060836390e+00,    -1.8015317921372367e+04,    -2.1193901594453610e-04,    -3.9218188246371640e-07,     1.9489509980603341e-09, } , 
      {   -7.6352019514825491e+00,    -2.5580619181960345e+04,    -4.8502998674505274e-04,    -7.8922072534752439e-07,     3.9362459101596220e-09, } , 
      {    1.4275751625813053e+01,     7.7038630507427697e+03,     2.8541701840048518e-05,     1.0395428279855592e-07,    -5.5171796910265192e-10, } , 
      {    3.0087393239569504e+01,     3.8752141002248914e+04,     5.3406581958550106e-05,     4.1078563784893267e-07,    -2.1714368444583117e-09, } , 
      {    2.4432900943715623e+01,     1.4184819217856326e+04,     2.6462623456282624e-05,     1.7137358836511453e-07,    -9.3103619320275287e-10, } , 
      {    5.4826704351359501e+00,    -4.5250953660182131e+02,    -3.9911661983372838e-04,    -4.7629686221074841e-07,     2.2936971585605140e-09, } , 
      {    2.8223588888625301e+01,     3.8333328492446002e+04,    -3.8165858108480972e-04,    -1.0081341736548776e-07,     1.8578060260117391e-10, } , 
      {    2.0547586573701608e+01,     3.2375929557840933e+04,    -1.6337886192581817e-04,     8.7875000732245171e-08,    -6.4072976095436282e-10, } , 
      {    2.6043952815996319e+00,    -2.6276495251747692e+04,    -4.2586471360882760e-04,    -7.1579617815716632e-07,     3.5326433687727442e-09, } , 
      {    8.9948416238658169e+00,     7.1093181426527735e+03,     3.8893838197436232e-07,     6.6890179914995668e-08,    -3.2342405480498253e-10, } , 
      {    1.4613971538551954e+01,     5.2953399322669429e+03,    -1.8479919271510039e-04,    -1.5285709944249797e-07,     7.0709590576144687e-10, } , 
      {    5.0310437226835854e+00,    -4.5256955692698193e+02,    -4.0424851809237717e-04,    -4.7515134444502282e-07,     2.1833632610136059e-09, } , 
      {    2.5868037130765309e+01,     3.0004637067822547e+04,    -5.3395098884713595e-04,    -3.5153229152014247e-07,     1.4217646012218248e-09, } , 
      {    9.8519687308440531e+00,     1.6275860988226586e+04,    -2.8257968162622755e-04,    -1.9390273468217029e-07,     7.9926383467718089e-10, } , 
      {    1.1471579532261138e+01,     1.5786060802842156e+04,    -5.2959696735243580e-04,    -4.8645816911585939e-07,     2.1789466083786981e-09, } , 
      {   -1.0000885683483807e+00,    -1.1117836452073143e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2250445690196880e+01,     1.4218576264980387e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    4.9010662739476967e+00,    -9.4433199838885539e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8825628692975869e+01,     8.3289882587378779e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5955494754237526e+01,     1.6678046365503793e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.8313641422136726e+00,     8.3283945905090222e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4831232642561030e+01,     8.9503876101168880e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0735763113870878e+00,     2.4880136079447966e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.3155055929576989e+00,     3.9121607534740124e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1154641652649401e+02,     1.6171159926601178e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2062028328099313e+01,    -7.4805163353650296e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3445082554959935e+01,    -8.5342389928792254e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9340463816461440e+01,     3.1729563908523585e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.1547896337904904e+00,     4.5496571264348731e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.4716021250050275e+01,    -5.5732099318327937e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.2747012375476046e+01,    -5.5730756608182628e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.0376908717812803e+01,     3.8235137095508071e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.8704975678854431e+01,     1.6076144307327319e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1922398959291897e+01,    -1.7408586015269484e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6721960825497490e+01,     9.7587734330542680e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5671835342158897e+01,     1.5543375917656313e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1405217161647105e+01,     1.5542132662307280e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.1508222868859690e+00,     7.2294407459078266e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8012928596880400e+00,     2.1217929805248605e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    6.3275834534175388e+00,     8.9253598204356531e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4291461841557052e+00,     2.0691963342011786e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.1127295703016772e+01,     7.2137776373681636e+03,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.1927822852905560e+01,     2.3731760303720672e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0322837436278800e+01,     3.1574459386499337e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.9247975038124991e+00,    -8.2610038189865099e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.4860405168204565e+01,     2.5891355203724044e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.6015180219349979e+01,     8.3694727289224247e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0050593259058452e+01,     1.0309528757087688e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0271010887744976e+02,     3.0457194281367319e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.4325657627211303e+01,     7.6868773283065530e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4068461145284019e+01,    -7.2143480933485253e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.2522949421290541e+01,    -5.5260821413692611e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.9716878892080807e+00,    -9.9127202773005229e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.4942892138274107e+01,     2.6545040241444032e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.1816256383509609e+00,    -1.0831932358703820e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -8.6177806926266349e+00,     1.0702549726780673e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.7405238234457798e+01,     1.4301681148179139e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.0713495454987232e+00,     1.1548110243238063e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9434914049578943e+01,     8.8348514799058921e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5228716244428909e+01,     2.9924494614340481e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.1343146588217188e+00,     1.7505557938505321e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7837651548300119e+01,     1.5589659675736830e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.8360282993262373e+00,     8.3291360505037519e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.2665118521424255e+00,    -8.3282467987431464e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.2575405441496059e+01,     3.5801760216279443e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.7838932275438708e+00,    -7.9260089498510854e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.0013234180792466e+01,     2.2683522649985738e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    1.6769602319579057e+01,     8.2081594725019386e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -4.8173745513036891e+01,     1.3517870021753406e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7061263272152402e+01,     2.3731726092727702e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    3.5278920679785486e+01,     8.2892288386836135e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1468524988521933e+00,     8.6441790847821339e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.0723695417387429e+01,     1.4094064487244961e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.4832986701918600e+01,     1.5058734515703643e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -1.7213997215483097e+01,     1.0722477059111887e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7285508892423735e+01,     3.1086219925430465e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.3521513898704024e+01,     8.8425771665499315e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.6081152273481418e+01,     1.8092395889935313e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4865367636737375e+01,     1.3805680969763578e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0958798686404549e+01,     7.2120985176888662e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.3613965711622505e+01,     1.6018258552944500e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5092230161560263e+00,     7.6481260878335916e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.7667991626546078e-01,    -1.7084185246395049e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.2562982337592318e+01,     1.3206455449634996e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.1193850813443149e+01,     8.2718092411858906e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5141614277994339e+01,     1.7215089310806783e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0769807503156247e+01,     2.2107179662316747e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5381269414916369e+01,     1.4757866336411142e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    8.5643058742500298e+00,     4.3561886595680869e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.8062954276972647e+00,     9.5419354332919575e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4446043207346975e+00,     3.2530974073295042e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4196144954068393e+00,     8.3237157177209792e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.6131723176431464e+01,     7.2117634004143210e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5961754015901462e+01,     2.5302862520576411e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.7526786200727500e+00,     1.6257802923954781e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3533758508360748e+01,     5.9649258618363147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5747574726715268e+00,     8.3336671315259191e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6723861330918766e+00,     8.8172793877914501e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3291293039650649e+01,     1.4360438125079751e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4465129953178756e+01,     1.5507861844925374e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9304248244443858e+01,     9.4170841614920209e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9474609773832981e+01,    -8.0247127037993141e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3359752585355817e+01,     3.3926152602731752e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.5973370524617092e-01,    -9.9615025597243130e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1149171044903476e+01,     1.6655433965263561e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.4211679849337180e+01,     8.3478696186405687e+03,     3.9950555821570275e-04,     5.4318704212574410e-07,    -2.6171212133654968e-09, } , 
      {    1.2217378925776961e+01,     7.3537635357580666e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.2011255578766526e+01,     2.3576640522181271e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -1.0941414518654888e+01,    -8.3532993236784423e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    3.5278197496843667e+01,    -8.0447029434563337e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -9.1065488244895292e+01,     1.1174252039800111e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.1015725280629134e+00,     1.2150680678667351e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.4302872321621614e+01,     8.3681540105632866e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2454785645376894e+01,     2.4991005306127950e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9307737002951564e+00,    -9.4613536678292658e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8437406626215608e+01,     1.5606744013915350e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0802920817290904e+01,     1.8458194387772273e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0671505763802763e+01,    -2.6256751690320953e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.0289193051636992e+01,    -1.1195595915227022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0544405861573443e+01,     2.2930318204672843e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.7615254839147120e+01,     1.8522628737586128e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.1389492920769847e+01,    -8.3329491802811717e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -9.3452995564916854e+00,     4.8690549583986867e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6304992543354690e+01,     6.5236221304403552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1468485910558336e+01,     1.5542613502708782e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0516447421918031e+01,     1.5558132170531280e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1682119739545008e+01,     2.2428230535461014e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    5.6857362143433745e+00,     2.2483848541262159e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6085676631075216e+00,     1.5542895077254807e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6999389943050915e+01,     7.2529113648551938e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.3649325466631570e+00,     4.6052725512983816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7908219414439994e+00,     1.6018765982945870e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1955047630110620e+00,     3.0377571191122221e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.8893788535035441e+01,     2.0455463204002732e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8990191391920987e+01,    -8.1294874366674649e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.9180612257643318e+01,    -1.7430341958844879e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2836246720830587e+01,     2.0856146656689059e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -1.2792637265412935e+01,    -1.3262555677851078e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9175939637803321e+01,     3.2243498508722078e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7974428127477074e+02,     7.2000483954478732e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.5737833787173835e+02,     7.2280773352688175e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -5.5698854538243197e+00,     1.0904054914678069e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8415562778286294e+01,     2.9448510630075325e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.6338509514880045e+01,     7.2876591947114175e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0024263833353032e+01,    -8.1174571930845605e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0789775257722013e+01,     9.4617363740176861e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1445617527237555e+01,     2.3687974210648557e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -4.2660363329995945e+01,     1.6266233707111593e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1969369492303869e+02,     1.4912411187919177e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4356236342839370e+01,     2.1899781910541173e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -4.1647266234301860e+01,     1.5445851489420304e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.9643962860460086e+01,     1.5225966043275736e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9704340965694409e+00,     9.9559140512227668e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.1510234234575137e+01,     2.9850223317850356e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.2934157806006404e+01,     8.3290047974830668e+03,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -4.0786859560369798e+01,    -8.3283780517638297e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {   -2.8496415606060310e+00,     8.3320885054323462e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5030516435206174e+01,    -6.7102068799783538e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.6074041669533266e+00,    -1.0259847126692548e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.1485858559153435e+01,     1.5530548365257817e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -2.3344288721021982e+01,     9.6754113547571014e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.3602252987433364e+01,     2.5021411344807071e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4242506604900667e+01,    -1.3028481170235552e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0533396829340234e+01,     9.4631590649514364e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3358214920723398e+01,    -1.3221004822177065e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.2740408812349703e+01,     6.6759267797429129e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.8284842770110089e+01,     1.0156725435004850e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.7882512793932232e+01,     1.5845637129938445e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.5013163243668766e+00,     8.3298024422247108e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.9317998771830647e+00,    -8.3275804070221911e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.9508923890272939e+01,     1.1548128185169751e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.2974734779435391e+00,     3.8941270695333033e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.3935091830013945e+01,    -8.4706932949645834e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.1042955060345008e+01,     3.0282399359037969e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.7905208550670466e+01,     8.6326701454475860e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -7.7497603300197149e+00,     4.5742497659747169e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.6806340717997754e+00,     1.3310822232543839e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.4622698723005847e+00,     7.2086331603204726e+03,    -3.7512201314318606e-04,    -3.0439414819407097e-07,     1.3067667960626429e-09, } , 
      {    8.3591520817548979e+00,     6.5159254364540429e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0472255624080224e+01,     1.7233432682548126e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2110546593599476e+01,     7.1791704203019235e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7567318542644102e+01,     3.1714168480967204e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.2349714784985247e+01,     7.4247568411710581e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2122442691696858e+01,     1.4270139611666711e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0201225996778458e+01,     8.3855736080610095e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4535516928244817e+01,     1.7802664330870706e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.9547536206719158e+01,     1.4060841910639956e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.3109125656380915e+01,     1.2396944892834931e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.6847409704583086e+01,     2.9831194609813916e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.4111117072152119e+01,     2.9026485138402422e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6876397960030747e+01,     2.3387754572701870e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1830725824840318e+00,     7.5930060827696743e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.4747294802940424e+01,     3.1884668430547834e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5852987488210387e+01,     9.3162385519068193e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.7672412137851810e+00,     7.0912639242883943e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.7601057295570129e+01,     1.2974167019613280e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9760489937310261e+01,     2.3471865789313124e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.0256752565924590e+00,    -9.4601807305361235e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5189065883510299e+01,     2.3888306461252807e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.7004993588260042e+01,     2.9382387337550856e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -6.9709103721840364e+01,    -1.6657588564472924e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -8.4803313196872416e+00,     1.6283506703877259e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.1683524592423122e+00,    -1.1506640609622511e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.9585254907042295e+01,     2.0685808323445795e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.2878580570196423e+01,    -8.2663346175932929e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -4.6925900286643074e-01,     5.8334947683819873e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1659878941095139e+01,     1.3114843581711029e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1417492520367205e+01,     7.7153042779373045e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5726015171392936e+01,     2.0227587306933969e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.8552749755996871e+01,     1.9168882220951833e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0376591096721715e+00,     7.3875449632132277e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4546100836019669e+01,     5.7985618476479704e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.3105593745372435e+00,    -1.3455079329792591e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.3098723527763063e+01,    -1.8330409043532136e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.6937422494113015e+01,     2.2128180937919820e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.7258431693446347e+01,     8.2655700515171193e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0152028940549229e+00,     1.9470662786970257e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6106658680702068e+01,     4.9487573304245361e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -1.3453603983024617e+01,    -1.1341123125098488e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.4097851272515548e+01,     9.3308751550386332e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2212727720117002e+01,     1.5973718334015015e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.7301776559406297e+00,     6.0983307786990335e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6642199658653807e+00,    -1.1169335447391610e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1394909809141886e+01,     1.9742032029719383e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4064902427119401e+01,     1.6415537278295211e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.3614080093254945e+01,     6.6252178662526394e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4137568556250848e+01,     7.6715181848651910e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.0166842159282167e+01,     9.2901055682505539e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6801740365857869e+00,    -1.0169843346203117e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6147153262202163e+01,     1.6703740546905985e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3343834049915660e+01,     4.1104167051686491e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    6.6850235954820093e+01,     5.5680295216091881e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.3043421116008339e+01,     1.9402796853229720e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.5753129571373155e+00,     7.8347438667776169e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0741088891938709e+01,     1.6628532454049335e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.0517891677551681e+01,     1.6557523889121192e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -5.1152901779574883e+01,     1.6226418805213680e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0154440857988931e+01,     1.5291719931114787e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.6208175023167534e+01,     1.5474062184510214e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2547206728219100e+00,    -1.1221067258341625e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1585449861591144e+01,     1.4412695876824066e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7600049965946205e+00,     2.4670605565189486e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.5266573834351036e+01,     9.5902376269721317e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8123835679656839e+00,     1.2305656390876289e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0808775422658123e+01,     2.5455474567282316e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -4.4470531230874020e+00,     2.3237104655116953e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.8424208878149440e+00,     1.7221700710358888e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.5151406418831456e+01,     1.9837004707895292e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.8687109685291947e+01,     1.5332852508287417e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.1676082329981750e+01,     1.5035103407784380e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.4724954208933156e+01,     1.9144377367032539e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.0855964210220508e+00,     7.2102963798640321e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    5.8674811129250784e+01,     1.5333557140203569e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -9.8676909187564590e+01,     1.4916482221953325e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5016939814774489e+01,     2.5200690811648339e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.1665560251864413e+00,    -3.5503066384351541e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1592094474845721e+01,     8.3283215616584821e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4045236020055583e+01,     3.6130346836158569e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.9819838530394065e+01,     1.4896005855022322e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6155351900272272e+01,     2.4910823212799638e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2618180776668027e+01,    -9.0670172210391684e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.0604228324384209e+01,     7.7936685845893317e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.5431141388789584e+01,     7.5111692577237245e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1194017818888209e+01,     1.8888847022565526e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2102808390753175e+01,     2.3871435192401437e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9685337756548421e+01,     2.3871456236809052e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.8732042058632999e+01,    -8.3593596064972990e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -9.5575098727858688e+01,     8.8929112341615983e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5297111607009320e+01,    -8.2354066796120260e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -2.9164112226866987e+01,    -8.3771428249041965e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.4363143246762334e+01,     1.7286019066667213e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.6677589410024886e+01,    -1.1971118525348229e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.4433820711987657e+00,     1.6659331733230236e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6978682230820404e+01,     9.8009163432696932e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    7.8497137614530530e+01,     7.2163623303023678e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3652511917719664e+01,     2.3871594131662459e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1563657477400920e+01,     1.6167248583214201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1278570734869357e+01,    -7.7158029507175843e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.1362811239620179e+00,     2.3871297297548030e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.6304501679751695e+01,     1.9797189805997376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5028903035463657e+00,     1.3720188341875320e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.2158051812597153e+00,     8.7354519331892188e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3809980400133774e+01,     1.6621300221850328e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    1.7560133112397793e+01,     1.6577183789972627e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8802617923308780e+01,     2.8423404040981535e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.7521226139972079e+01,     2.3225865132794035e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.3780845150731380e+01,     8.3305632521811294e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9436887076224288e+01,     4.0006459173228985e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.7873063715862159e+01,     7.0052247450889199e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.0870392533566303e+01,     8.4549455514501333e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.9331921041028810e+00,    -9.2080333685359983e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -9.7826571925750955e+01,     9.2893292846634831e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6844977139322339e+01,     1.6538904545954225e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.1190485721360776e+01,     8.7528574281377423e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.5129812068052173e+00,     2.8385933410258446e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6930434181752778e+00,     5.8008673267421473e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    5.9120205826317147e+01,     2.2756458125831337e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.0642834426715946e-01,    -1.1067218974056375e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.4826677876825638e+00,     6.9984799251792911e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9011754073251399e+01,     2.5005261943626381e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    6.3664720339986127e+00,     2.1940122293595472e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.9748086758959863e+02,    -8.9575069683081110e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.8743457290792168e+01,     1.0766999336109684e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0249171095180081e+01,     8.9192616505547812e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7368265966696961e+00,    -2.4966886604114316e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    4.0908512468225355e-01,    -5.7649247364967346e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    3.3220597733509885e+00,     7.0901004627341626e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.1176520246857407e+01,    -8.4369118675094251e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.2001209454276777e+01,     7.2954178959797946e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5418656645584225e+01,    -1.1071503926960488e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.3438941675064981e+00,     2.3859955726748514e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.8132055772086098e+01,     2.3882935702461975e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.5961556967567919e+01,     2.2757176184848944e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.1894132955501631e+00,     8.3289806289536755e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8048761398451463e+00,     8.3284022202932247e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3804520850729133e+01,     8.3290332183777155e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2810226442775942e+01,     8.3283496308691847e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4995505781249619e+01,     2.8429962996580660e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.9777497014368821e-01,    -8.2588615807646893e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    2.6669236432090329e+01,     1.3220711099220440e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.9446263230565805e+01,     8.3775417203822508e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1788835621487586e+01,     1.5462900829028260e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.9464034848708415e+01,     1.3799824859221870e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.4869959142533844e+01,     3.8865522223337321e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.8666666380066886e+01,     8.6038292259889295e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7106672197244794e+01,     2.0724826579489127e+02,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.5908686293612838e+01,     3.1731865108464990e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.3736425311486336e+01,     2.0732208989318940e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4769174381856196e+00,     2.2773026490680859e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0064079753529754e+01,    -4.3570120940780243e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8939362312828536e+01,    -5.4535917098872678e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.5582591796215603e+01,     7.7774106194336491e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.0117074531035612e+00,     2.3400768747020629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1818994302652381e+00,     5.9068055211645305e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.5906992339247633e+01,     1.5573407212340489e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    1.5564456970531753e+00,     7.9608487536965622e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.3549792986136829e+01,     1.4731008570673339e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.9101876430781324e+01,    -2.0741373369908124e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4134074193904361e+01,     1.4413407222290942e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8753239234881605e+01,     6.3884117393004781e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2781793838073266e+01,     2.5022834035740823e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2904525247514272e+01,     2.2135068525893734e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.2837003885522570e+01,     3.8821940835945090e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -1.6354791276485791e+01,     7.0540273110765011e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5730592360119431e+01,     2.3464685206039474e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8193512708829317e+01,     1.6146478711226404e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1954464319193333e+01,     1.6090879781699030e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1564059815025129e+01,     4.6052924382111036e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.3403706433079289e+01,     2.1623060860479029e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.6290716427874671e+01,     8.3282339824333649e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.4721199980690869e+01,    -8.3291488668135353e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    9.0901223417456691e+00,     1.5359282786025109e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -1.8371958952478607e+01,     1.8826215551952191e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1646316324035794e+01,     1.1617475625309513e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7376037048293533e+01,    -8.3441894144972885e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.4978945380121587e+01,     1.0324449195447087e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.1309350672091806e+01,     1.0110324769131701e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.0101620524233752e-01,     7.7106931208333945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9950012119887091e+01,     1.1916821474969493e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.8949719860431767e+01,     6.6081897800199713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.1680840746188785e+00,     3.1261188140880386e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8179390032779171e+01,     7.6262620290684536e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    6.6745784178917688e+01,     1.4293138996734522e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.3635906025042587e+01,     1.6610483510593473e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.0830581793378034e+01,    -4.5098154161608600e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.3956867897536380e+00,     7.0038020541551659e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.4426079170548302e+01,     2.0556425691522651e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8837718690992489e-01,     1.6655960158313148e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5131662738454907e+01,     1.5616048795336195e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.5896354227884883e+01,     1.6658805540180652e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.1420643896366059e+01,     2.2701460225153802e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.6307760128799600e+01,     2.4153132995830947e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {   -1.8970221331639738e+00,    -7.1134546687633815e-01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.2951653374787142e+01,     7.0045201131727699e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.1920071631732231e+01,    -2.2069170497228533e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -4.7716417068063528e+00,     2.9826592209931092e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    8.5423989172517789e+00,    -1.5589653628635220e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.3032467024619338e+01,     9.7702634209109965e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.8228196498425177e+01,     7.0045066860713159e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.8994151337027887e+01,     2.0911709302101423e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.7134448112065755e+01,     6.5105042192420879e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.8950696153412451e+01,     1.6240925929879068e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.4857224279960981e+01,    -6.3694408342837733e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.6113124517851052e+00,     5.9923925842766503e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4358681294485228e+01,     1.5624109320603244e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1951746516828505e+01,    -1.6880050226181469e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.0366994721016516e+02,     8.3293469275385669e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.0210043076298135e+02,    -8.3280359217083333e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.0869793345346643e+00,     1.6007745356223506e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4263189768678970e+01,     3.9491752265599163e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.2307877734093175e+01,     8.1770173531018727e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -1.7415777584724502e+01,    -2.5333555880841777e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9940272222889345e+01,     8.8037728765200955e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.1509788670073156e+01,    -7.8536099727268047e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.5785572047228129e-01,     2.2575015642616247e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.0797539745848880e+01,     1.6574899555977179e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.9954359198387536e+01,     8.2776370004026212e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    8.8370373647574532e+00,     6.7123120372461590e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5595062645554023e+00,    -8.7278298831341590e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9417502328461440e+00,     2.3413990395098401e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.7021596137511054e+01,     2.8678216434203798e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.6409522110726414e+01,     3.1491647460854845e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.3855712933302371e+01,     1.0488742505386650e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9858272785595801e+01,     5.9650742788935286e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7193149376650325e+01,     1.5542944745692659e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -4.0116096872844324e+01,     1.5542563834270926e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.6270121282572738e+01,     8.0619245817571464e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.2049931275743049e+00,     8.8334287889721418e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1372053936929802e+01,     1.4718508425746183e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.6964764869321073e+01,    -7.9587275351851258e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6184924431274279e+00,     8.8834026803159213e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.2342093112359702e+00,    -7.7739801689309807e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.1279187580020995e+00,     9.4896776815476424e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4491041218663874e+01,     1.4941107411520885e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8515784518446088e+00,     3.6346209898958637e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2836905661528203e+01,     2.3384413394918807e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0500369835188060e+01,     1.6597864272175488e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.0292950363987920e+01,     3.0910706944083511e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.7106087226657678e+01,     2.2851988987424396e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3810344417265719e+01,    -1.6362866044213897e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3007225437666779e+01,     1.6661196141060034e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.7526732053151420e+01,     1.1597123695295941e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6980147801995464e+00,     1.6653569557433766e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.5445877032204166e+01,    -9.0959074859503799e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.9094301543657011e+01,     3.3208347782768957e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9731720053693373e+01,     2.4230685256843073e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.4219713399053941e+01,     4.6956849489824512e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    1.0615324981545706e+01,     4.5892730430743832e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1478313089854755e+01,    -1.1668863110094205e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.0464103739893559e+01,     8.3634392544796719e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.9345151001801440e+01,     1.7324387634490045e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.5689074590917127e+01,     1.0560155597942072e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9887770938169522e+00,     8.8702579607415519e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.5614252996228089e+01,    -9.0952028540342308e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -4.0933686238697238e+00,     6.5703361691731234e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.5364297454802212e+01,    -7.4075241389546568e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5745882604859773e+01,     7.1434218759034284e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.8856871975776507e+01,     8.3140214517978638e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2764721608693872e+01,     9.3741415513366464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3543079929486588e+00,     6.7746412297922107e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.5413280006997702e+01,     1.7380862266376695e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0555968408459544e+01,    -7.7135017507761722e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0493916040346338e+01,     1.1155089273545074e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7404428866898925e+01,     2.9917910923487292e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    7.5046865385766154e+01,     7.7160202148045555e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2833708699961818e+01,    -1.6409476995476358e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.2060897053398520e+01,     2.2836377620993931e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2279559427892686e+00,     7.7041866091872334e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3796318132110528e+01,     1.4357484741261775e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.1966360659591547e+01,     7.8402929076201872e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5329497165647563e+01,     1.3210495199049431e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5251812073358497e+01,     1.5456003765315443e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    5.2758985011578829e+01,     1.5539641044400483e+04,     2.4503613970602655e-04,     3.5993110676894790e-07,    -1.7597282183232826e-09, } , 
      {   -2.5681932507772828e+01,     1.5545867535563102e+04,    -3.7740249195369381e-04,    -2.3569566172990487e-07,     9.4368983027971148e-10, } , 
      {    2.2515510310856477e+01,     1.4271545362191579e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.8464635228625170e+01,     1.6853814403784934e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -6.6412916464317931e+01,     2.3870741082689095e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.4606287436949124e+01,     8.4557575970900343e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.1250948272622935e+01,     1.3666691690385709e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.7442699259639323e+00,     6.6277037508016283e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1891878833847676e+00,     6.3040679990870995e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.5486393266253961e+01,    -5.1749937773464057e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6507386663803004e+01,     8.9084391396280607e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1631159327975062e+01,     9.1642472754638839e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.9491936980739538e+01,     1.1708145797519906e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6404912413327157e+01,     1.3404182603177127e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.0200421073638481e+01,     7.3133896602286241e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    5.6355494003985761e+01,     2.6940362411078433e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1952755298886728e+01,     1.5489794668509514e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -6.7724221807823398e+01,     1.7301232462580876e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.7731308239765973e+01,     6.3485968374025661e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2612059021459579e+01,     1.5577646735038215e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2264884001986150e+01,     1.5136632375036505e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6768230681323981e+00,     9.5433581242257096e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1351135442565212e+01,     3.5835893931100363e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4310441475764092e+01,     2.3323320222888007e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6658302379572238e+01,     7.8881494886025193e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.1568726703503680e-01,     7.3191885635946198e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.5997925248041691e+01,     6.1343791951026315e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    5.0774985209791069e+01,     2.4358119004782879e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.1614111001104519e+01,     3.0023849117909678e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    5.1788190623186672e+01,     2.3860629003348389e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.1606487177226725e+00,     8.5493414368441936e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.2013350472086071e+01,    -8.1080414124027084e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0762783926078839e+01,     1.1020319635402948e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.3047121169118636e+00,     7.9225695096781601e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.9355876673644978e+00,     2.4714369837428778e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.9623050965179672e+01,     1.0857463793094053e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1059124830136550e+01,     6.4652233281912895e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -3.8482872597323841e+01,     2.6482170741643395e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0628444750789932e+01,     7.2508226272288211e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.0255019935899536e+01,     7.2489553104147653e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2115894663428328e-02,    -8.6965340955503398e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.3288067562480123e+01,     1.5389950967898956e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.2720659610608207e-01,    -1.6692327738256292e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.3914406694884656e+01,    -8.3264244356750096e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {   -7.8470513730568030e+01,     2.0171065951367018e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3360571247725026e+01,     1.6739866142516621e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2277447576057682e+01,    -1.0073928768875510e+04,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.2501319775124585e+01,     8.3282620522397192e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.4070836222308401e+01,    -8.3291207970071828e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.7488752569822417e+00,     2.5342677422474320e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.6181492553418934e+01,     1.4914834846729789e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1163803530921284e+01,     1.6011443237926887e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4468237672230959e+01,     9.2106479413370325e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.8102276990518291e-01,    -2.6271938609656409e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.8863284595310823e+01,    -7.6846513555786078e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.5979491425754226e+01,     2.3123693423865963e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.9663988504452789e+01,     8.3159258293832590e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2894100480578452e+01,     5.3571784597793794e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4870441930187305e+01,     1.2385468621239628e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.8196604101097421e+01,     2.3872150346521394e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.7586879385622915e+01,     7.5411242319862577e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8979317915591182e+01,     7.7710181154820957e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6604331029553549e+01,     9.9471759692685482e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.3696175872240950e+00,     1.3548309473601043e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.0368109309470569e+01,     1.9959284961131234e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2643316597823295e+00,    -2.2341728912726530e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.9427822962051096e+01,     2.4251348773099966e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.6959314352515671e+00,     1.7381250408170232e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.7232823317580710e+00,     8.1814987659173057e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -3.2301141691563515e+01,     7.7717361744996988e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.4389926646415674e+00,     1.9077641817277272e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.2390483052213156e+01,    -7.4775229073927994e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0429975148006488e+01,     7.5022757324894083e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.8798319653272380e+01,     7.7266922302044641e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7719345418720778e+00,     7.6584353690890794e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6232321772470748e+01,     1.8700039958723963e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6379265418511118e+01,     2.6938061211137021e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6460590268443664e+01,     1.6628517194534179e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -6.0944234372943363e+01,     4.9158988258613863e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.8176793270848421e+01,     8.4948565621206944e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.2862344486450709e+01,     1.9916634511691118e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1199837672165543e+02,     7.2153324547164475e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.8963243331862301e+02,     7.2127932760002413e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2460960801244362e+01,     1.6175992907284930e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3001900952820602e+00,    -7.1561874711436221e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.2261230090528144e+01,     4.0581797661063334e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.3217294948175343e+01,     2.3858994195127936e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    5.9937884709236240e+00,     7.9794412884874255e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4961028316830646e+01,     1.5458146003537264e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2984537129162508e+00,     1.0128294547848673e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.7768100355975605e+01,     1.0840190796328383e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.6247525805160443e+01,     1.1525586280037509e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5691788369560799e+01,     1.5592417687416933e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.9068864857645138e+01,     1.0810770169666646e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9269873713172323e+01,     1.1386118133239906e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.8344273680947985e+01,     1.1373326549289673e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0523882671999466e+01,     1.1757659708974365e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.6846521257135834e+00,     1.5078740014875817e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1653747012727678e+00,     7.8269405965112646e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9293124468883960e+01,     7.7398464255177450e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.7110405874839891e+00,     1.7898578908198309e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1320581940938570e+01,     7.1868329032952652e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.3749075613733709e+01,     6.6259224981687885e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9933562779761980e+00,    -8.6750524666351751e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.4425257040858369e+01,     2.3845137323978688e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2158258586383193e+00,     7.2063479384120328e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4558607070752302e+01,     2.2784047117403221e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9835533049120873e+00,     9.0929521460522228e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.5550105506818333e+01,     8.0645391545581770e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.2167761841895455e+01,    -8.1824470581397472e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.8405684726765649e+01,     8.2622158161409043e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.3040652508609973e+00,     1.0633857621309095e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -9.7237263662776918e+00,     2.1385716868046684e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3619221066555991e+01,     7.2309236120059122e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {   -3.8726065591618699e+00,     1.5997908409547228e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.1724873247938014e+01,     2.6621045389820601e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.4009404977355565e+01,     7.2181816668107340e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3397340948604022e+01,     2.2112510062070600e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0523050326850480e-01,     4.3051817231670011e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2064515088282139e+01,     8.3268084157811500e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8725668158851420e+01,     4.0475859466640948e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.4252310451166101e+01,     2.3973617423533316e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4230376097647671e+01,    -1.7354579777415920e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9477076339657675e+01,     1.3696206874357227e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.6401708648617909e+01,     1.6631074458620340e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.0450740331835451e+01,     9.1124321640680137e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4428493672238670e+01,     8.3090145578260490e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -8.1110679252471325e+00,     2.2743310644068788e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.3263129619616528e+01,     7.7228183392850769e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6560413457438781e+00,     9.0076411837717205e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.6385442677146589e+01,     3.1692486257031280e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    6.6744954742569931e+00,    -1.1642919567002441e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -5.7520352354753506e+00,     5.4470327542881332e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5843753594004969e+01,     2.3828785898649541e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0973362070136274e+01,     1.5855593166346473e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9749549575115157e+00,     6.8805604727367718e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.0415155947307138e+01,     3.2306075333651079e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -6.7384886534631562e+01,     1.1485771378139596e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2886840297062072e+00,    -2.2679543187278141e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0702046121441795e+01,     3.3314054353026928e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    5.4453197291288555e+00,     7.9628929267864887e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1594299504100842e+01,     1.4889863541135319e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3030616795580167e+01,    -7.5300302468194877e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.4425618658163554e+01,     3.3315477043960673e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    9.8732476664333895e+00,     1.6149020621582609e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5707066134554122e+00,     2.3883897234082557e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.7966447692700861e+01,     3.9940335880704515e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.0251167479245471e+01,     1.0937481738683413e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.2114724557496039e+00,     8.3313059974244788e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.3217015641079776e+01,    -8.2912203172860973e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.9126415000164414e+01,     1.4210888058819995e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2478992709892470e+01,    -9.7375663254623687e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0659255476341125e+01,     1.1157320866593113e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.4473444468313357e+00,     1.2132440086685065e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.7006079492796182e+00,     2.2382941009970498e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.5315576156641256e+01,     1.4914058563142715e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7793383586100362e+01,     2.3839806525371903e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.7735204985568274e+01,    -8.3647740520200205e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.0737908160913193e+01,     1.6177095349470090e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -2.2329919032093969e+01,     8.9085476093882462e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.9974236306625670e+01,     7.9248537436736060e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3181902672580023e+01,     1.5842475255148209e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7505106774216571e+01,     1.8485416859628298e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.0258798787713729e+01,     1.5135993781416026e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1628590499181140e+01,     1.0021837304151832e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0695872146160808e+01,     1.6683691239873457e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6781081557072017e+01,     2.3829497244116417e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1491147132881776e+01,     2.5613670513449015e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.2030428983916103e+01,     1.4996929998205969e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7212509382841965e+01,    -8.5044747224197472e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.9912750333783169e+00,     7.2114598414182637e+03,    -3.7294500239583599e-04,    -3.7185708699192943e-07,     1.6853577996410795e-09, } , 
      {    1.1966795925979966e+01,     1.4366321213219016e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.2514543201086230e+00,     4.3770833556963680e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1530794075544081e+01,     7.0021864765135788e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4373658180047769e+01,     2.4659788853932630e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2300637540780183e+01,    -8.7577950217130074e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1069625467164212e+01,     2.4552722396861980e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2598988866300921e+01,    -8.3338801332418006e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9037924685248118e+01,     1.8915649707279117e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.6210476919114942e+01,     1.3360411769620163e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9822869951224671e+01,     6.7500819599461392e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1612072413027571e+01,     1.4601607828571572e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4955884063422733e+00,     8.8537095990406160e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.1760731303795511e+01,     3.4886834129045695e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.6495450805459850e+01,     1.5545031806339901e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.5570017775958860e+01,     9.9176368697090256e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0401529568578775e+01,     1.0454501267131973e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.5436782287447457e+00,    -8.3631841975454881e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.0991044949352453e+01,     6.3758534986184250e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.6320282589897609e+01,     7.9406014871880579e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8060945937588599e+01,     1.3630083581832756e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    7.4475008936897339e+01,     5.9020907650809158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5332343283151442e+00,     7.1991065322202312e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8563030584245013e+01,     1.5554960214705772e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.1129119911631548e+01,     1.4547570436410426e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.5807298984204232e+00,     4.1566361567418862e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.0043023426490919e+01,     1.0523402549622326e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3893592059962359e+00,    -1.6985969469126034e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    4.2313013718749803e-01,    -1.8668890711959195e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.3880279515345082e+01,     1.5429853892622518e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7139310327805124e+01,     1.4602927366596768e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.9171090529184056e+00,     3.0966063874269854e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.4207176169501452e+01,     3.1357213016405163e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.1580163149517659e+01,     2.2440028908634078e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0114491453574042e+00,    -9.6951344334266082e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.0639979027994919e+01,     2.1564191709182611e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0625564891727606e+01,     6.5983745187307895e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    5.7223132578669102e+01,     8.3263835111313147e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.7884936705459790e+01,    -8.2937807466070317e+03,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    2.0603670609659524e+01,     3.6173708205651965e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5790309488587027e+01,    -8.1114537527267585e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.2902415584853362e+00,    -1.3156276749406587e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.4101915894849979e+00,     2.1278033281094766e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6751975606723356e+01,     1.5753457741533417e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2553583788420235e+01,     7.1294545789138137e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0449293897347848e+01,     6.6600542855018439e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1437318181848838e+01,     6.6014877643120981e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    2.2745054380922141e+01,     9.3626392177799680e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -9.6896755626380298e+00,     2.3457754667337693e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.5435746772403558e+00,    -2.9568914984460312e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5982399362353679e+00,     8.3886641033140768e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.8245617670109510e+01,     2.9829245725830577e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1856903616661256e+01,     1.9178194017119722e+01,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -2.4712012018613031e+01,     7.2248795766152016e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2690142000259222e+01,     2.9412462213671723e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    7.6461434897772769e+00,     9.3844508325921342e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4939508350243280e+01,     1.4902956717079525e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0171081433810855e+01,     7.1437809054122308e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5618608917522607e+01,     1.6166130842769744e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7308445785473197e+01,     8.3485904398463572e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.4557440311097949e+00,    -8.3087924094005448e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2964099339104639e+01,    -1.9807473739472573e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.3099540004767245e+01,     1.4561293887202153e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5708577042452241e+01,     6.7457908345946416e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.6619935098245531e+00,     8.3109095053922883e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -5.3383774773679704e-01,    -3.0202148306586282e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5553529554922958e+01,     2.9748711316544195e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6789370091752083e+01,     1.3109153753595447e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8990225868763488e+01,     1.5781643701369268e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4555053888751658e+01,     1.4166694414196971e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.0921958135750671e+01,     1.7285697147843170e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.0634644008680372e+01,     2.4428407678279724e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.1310721525089246e+01,     8.2838094506838479e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -6.0741423127447041e+00,     1.1306280674891029e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8232303814877618e+01,     9.0133223422563988e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4884819123240867e+01,     2.2050512683402168e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5911960986692222e+01,     1.8260119650377546e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.0510934853060910e+01,     3.3601368752461945e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.6736894974873081e+01,     2.4429112310195869e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.3524840337431449e+01,     2.4817523208273065e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1423255815765820e+00,     8.9040393881967120e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.3772075435443270e+01,    -8.3936244472175858e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.1272284447083713e+01,     2.2770323666611494e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.0480052007789666e+01,     9.0065129211234525e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.1447425597954265e+01,     5.4221114823735170e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.7465989256554035e+00,     7.2986711518028751e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2750248391874216e+01,     1.9517154607611068e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8351900233111543e+01,     3.1155398357982376e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.8630421748109516e+00,     1.8684620847584283e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5414125134936274e+01,     1.1392677088839042e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3842313416027366e+01,    -1.0071275252976022e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1788408381868937e+01,     1.4534063925139071e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6435815949810735e+01,     6.6747727939275928e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.9037949805602381e+01,     1.4960847370809401e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.1888329754898823e+01,     4.7742539113252409e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    3.9403093576456982e+01,     8.3951670331059959e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0029065973407139e+01,     8.3636021026398703e+03,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.8431592606558280e+00,     8.3275374388081291e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.0315648200489626e+01,     1.5065069814145067e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.2820841033246854e+01,     2.0135052999412372e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0919042286738591e+02,     7.8424685019777271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.7206832524909093e+00,     1.5561116606297273e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.2797735756296916e+01,     1.5524391973666312e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9496064342367070e+01,     4.0371094209511706e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.7265879413725244e+01,     1.1754546463393051e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.8565744016965340e+00,    -3.9076100034177193e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.8518127474652040e+01,     2.5919033856339902e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8084611811063599e+01,    -1.1095614872234162e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0758864160182631e+01,     4.0489013661954232e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.1729251151571436e+00,     9.3880058682393815e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.5915826465199707e+00,     2.7330978443950498e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0496036909463080e-01,     3.7570562314497647e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.2904380014404566e+00,     2.5636923764048137e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3374598117099108e+01,     7.2497190998346132e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.0469246012195892e+01,     5.8875836404155270e+03,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.4703691090205659e+01,     6.9913954884237755e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.3326021082095091e+01,     9.7717592633600020e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1014833677691085e+02,     1.5272780822017461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5454034142097186e+01,     8.5662792399174159e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.9035134384188680e+01,     2.3482027540947176e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.9181493669486727e+01,     4.7743243745168569e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {   -2.2044104083615132e+01,     7.4006476148963429e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1653010476528073e+02,     2.2128607621868439e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.4599222400674371e+01,     2.4148419059413391e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9180385319146712e+01,     1.7205491375018169e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.8590449935439057e+00,     4.8716847907148704e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1374766802361437e+01,     2.2772564898854591e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4032758831807062e+00,    -2.2161392073319439e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8472675090430361e+01,     5.4968638706639504e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.4819680876922366e+01,     9.5787476391154032e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4598437531162801e+01,     1.7031258994616539e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.6272847238805657e+00,     3.3274950796595891e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.7853941296838020e+01,     1.6171773220494213e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0042104443333226e+00,    -1.6638204655229780e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -8.6257004770398851e+00,     2.5048207423742795e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.8686684085454043e+01,     9.4608549950489851e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -8.9261112076866933e-01,     2.3872432375801891e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5077710974673952e+01,     7.8637003583817395e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1133078571591266e+00,     8.2941986517014138e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.1188812592259740e+01,     1.0278048529573936e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.7649238750315249e+01,     9.6355964528591885e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.4993809644351535e+00,     8.3804970602804096e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    7.1507266685875956e+00,    -8.2565346576562297e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.6772672404211093e+00,    -1.5119080397930109e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.2669524084289563e+01,     1.1964095929129353e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.9426757119275692e+01,     1.6673891279936393e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.1691918927577504e+01,     1.1016723400685702e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.6010920875226233e+00,     7.2140270395228827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3346250553265065e+01,    -8.5018448901035608e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2514935652366013e+01,     2.3620411355738241e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.5654228875992153e+01,     6.5700227963135039e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1342107758019583e+01,    -7.7550024375168846e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.1100768424951050e+02,     8.3284644527928667e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {   -1.1257720069669432e+02,    -8.3289183964540334e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    2.4967074621978885e+01,     7.2140986911938062e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6054511917716962e+01,     6.4990142313853603e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -8.5913044747628220e+01,    -2.4272873609482303e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7751209089941419e+01,     1.7043844863340753e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.2561129034220251e+01,     5.4004128286688456e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.1748982300276840e+01,     2.2685471533969077e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.1988136651978785e+00,     6.7228261183963405e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.3321142033374365e-01,     4.6868894794509271e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.9673537967397650e+00,     3.2414753677006687e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -4.1841322025816154e+00,    -3.4907880613637394e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.8057750068963198e+01,     3.0986181785093315e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -3.7094835799371433e+00,     1.2590102314233489e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4896478789476195e+01,     1.6318254533733483e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.2603104290034871e+01,     9.9188089873361096e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.7965840182976113e+01,     3.1614489370138857e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.6393887995213696e+01,     3.9085613384707911e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -1.0769800456969122e+01,     4.4760339887056998e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.2871639546111648e+00,     1.8291599877891294e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8492051193986146e+01,     1.4165607167769196e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    3.8687066995125932e+00,     1.4317290715029685e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1004616330037486e+01,     1.6122610474746591e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -7.4518105829963645e+01,     9.2667873795312389e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.0568718279243285e+01,     1.4715372195446689e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8865276524266129e+01,     1.5622607750935327e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1381020308120473e+01,     2.2340385959139363e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.5375431496196423e+00,     4.3903395233977762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2951449897306128e+01,     1.8008115385253718e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.3788998293875402e+01,    -4.7550426296270598e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7321773572486769e-01,     8.9568505926117796e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7497032643891188e+00,    -2.7347058283457782e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1957065046890040e+01,    -8.1088239204105721e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.5213921952849292e+00,     1.5569984252044875e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.7776094133471889e+00,    -2.4242128265673832e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1916303476370281e+01,     1.0762068303852089e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.2084467899454898e+00,    -7.8918996213734990e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.3503762870458362e+00,     5.4298795313413598e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0856947374264362e+01,     7.2229412538588886e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -5.7677057088836658e+00,     5.3940731328158517e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.1819866071682998e+00,    -1.0623708075207905e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.7864977773797097e+00,     7.7746828447669495e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1642728425413441e+01,     6.0879595777516633e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6796766794452072e+01,     1.3192840468603454e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -6.4536742494285022e+01,     7.7557407697642484e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2834103586011292e+00,     7.2295608552321846e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3066567384446017e+01,     2.8259110731041968e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.8367463655253815e+01,     1.9073710388867581e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9007157629334195e+01,     1.6150443312516361e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1548980889939003e+01,     2.2130587113078298e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.5404989758378012e+00,    -7.7025260777997282e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.2714586320818686e+00,     6.6593429400349678e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5330114634223335e+01,    -1.0280824718712325e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.9687997203291516e+00,     5.3298514556885921e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8589509908783995e+01,     2.2774449181614611e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5014878905534612e+01,     8.4008481915906723e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6197848628159637e+01,     2.2159676424948724e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.9584503951709946e+00,    -1.8556899362082659e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    7.4366636029769984e-01,    -3.9433276081281424e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0141799544085153e+01,    -9.3636901800927299e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.7912415622038104e+01,     6.4406401643903564e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.3992396553527813e+01,     2.3903084903838586e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5931310534477283e+00,     7.9312632132920598e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.2497396669460556e+01,     3.3354580600391710e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    2.4254032429995281e+01,     8.1976540609826643e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8255239657001745e+01,     1.0296097473029797e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3426872019852894e+01,     7.8908218506846342e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.6391349606627363e+00,     2.3897754105231801e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4598541254106721e+01,     1.3799106800204270e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.4496969048819217e+01,     7.7578706337195435e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1365965327668842e+01,     1.5847031171899362e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.7182382600751012e+01,     7.3421440694349149e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6911454496491288e+01,     1.6657128584862669e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1865016744630086e+01,     5.9349057901350143e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4000778204662687e+01,     3.6194781185972432e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3847914166261845e+01,     8.3553884153230938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6551838606423345e+01,     7.4529522767458184e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1683398770458155e+01,    -1.4323984678873821e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.1205989986039768e+01,     1.6657637113631132e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.7669110249124209e+00,     1.6970221725611580e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5934424023665912e+01,    -1.3218768250599969e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.7717032145932450e+00,     9.5220282162478579e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7329939898286852e+00,     1.5671081718694813e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.5489851096045598e+01,     5.3287877942636405e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4320746176273509e+01,     8.4017622728201422e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6088088912176779e+00,     6.6518785728372713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9227649327901069e+01,     1.5485872106544235e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0537821561428771e+01,     1.1859321196261202e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.8933541809150860e+00,     7.7172445862254744e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.2169611041634724e+01,     8.9100996710155669e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.9893397410568177e+01,     8.5765972783939942e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1962443376746787e+01,     1.5627362576426325e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7570980290574095e+01,     2.9968250188382299e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.3769995112383208e+01,    -7.1304503215273680e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.1341580190138210e+01,     8.2771450183460111e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3580049672721627e+01,     3.2119955045900388e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.7387149345838054e+01,    -9.6525408172500847e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.3241471775242005e+01,    -9.6531980819996315e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    3.0480410297411346e+01,    -1.0278875834728991e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    5.2923711445221446e+00,     1.0557594079686851e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.0372693231607464e+00,     6.9066439207569965e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8826150587092467e+01,     1.6598513697950479e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4708571980906555e-01,     8.3802378309008873e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9315184043638460e+01,     2.6185601545297857e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.6230835526027036e+01,     8.3309993381155855e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0247950553363847e+01,    -2.5858602671341832e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0617651203764048e+02,     5.9575457468410978e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4547787430397094e+01,     1.6250622340681130e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.2875482615444710e+01,     1.6814911488361307e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.8061587473490121e+01,     7.7405510574338941e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9440514502782094e+00,     2.1547101323902483e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    5.5679209910760150e+00,     3.9334570200791211e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.9292133367573676e+01,    -9.9521828632403594e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.1754285107488933e+01,     1.7730027842601019e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.3119101462504261e+00,     2.3230518891141714e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.0213255703357655e+02,     8.0587179566591167e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5350309833285881e+01,    -7.8834313932738587e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.9150901082575214e+00,    -1.6396467336822711e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.7173800269198885e+01,     2.7344200092028274e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1778856235540681e+01,     1.7601159142973305e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4806933822170983e+01,    -9.6579365216665446e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -9.2524804580623190e+00,    -9.3984380099489535e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.7859056680185432e+00,     5.9110470293941116e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.6817894533575810e+00,     8.7348133395687400e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1685558468785999e+01,     7.6277539126258962e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3874569783456780e+01,     3.3367734795704993e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    3.1671232276090102e+01,     2.9973509853014672e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -8.4379831112743524e+00,     1.9470559050090760e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4298110619698321e+01,     8.4749357911071529e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3687391085302790e+01,     1.3859336722742561e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -8.7921039058478669e+00,     7.3875297036980737e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -1.8743805604785052e+01,     8.3288500165927180e+03,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {   -2.6596507359148454e+01,    -8.3285328326541839e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -4.8264224219985906e+01,     2.2812174085526480e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.0477383876380038e+01,     4.0568643465750050e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -3.1158430483370914e+01,     8.7639579746221843e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -3.2727946930554729e+01,    -7.8934248746247176e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.2352823830125189e+01,     1.6236323529996247e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -1.0300618053760935e+02,     1.9463128562525650e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1730549482627325e+01,     8.2726185575698528e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.3877847728263930e+01,    -8.3847642916770437e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {   -4.4924678252574139e+01,    -8.5953287890233332e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1080449488703282e+01,     1.4785098284006077e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9256475309828449e+00,     1.4916588943158073e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.9668026111509384e+01,     4.8999070346657327e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.7970374038561854e+00,     8.1311421683851058e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5009226916094093e+01,     1.4308573619597877e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0839312320855914e+00,     2.3196737463686557e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.0383514505517314e+02,     8.5377329317206350e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.9889304052872851e+02,    -8.5387488489988700e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    2.0112217431030011e+01,     1.4743594439397551e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.2873854429729761e+01,     8.5946309623347024e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.7741667120712080e+01,     3.2815326813075968e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.3031541979376428e+01,     1.4423194698459094e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.9760318092648448e+00,     5.4369929860101229e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1132764958203673e+01,     1.4905586549395708e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.5391926330713488e+01,     8.8015658552699715e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1092413391079956e+01,     1.4181373862761468e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5054719416487597e+01,     1.8151544604041759e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0149476041962689e+01,     2.2728465432922858e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.4758561544825575e+01,     9.9760263644661136e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.3251540435014881e+00,     1.2146666996022591e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2549907432294678e+02,     8.3135017122930967e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9011575240547556e+00,     1.7295999715547932e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5007508296040708e+01,     2.3913394185094072e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3838111071673103e+00,     9.3727188604028961e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9689315233691129e+00,    -1.7378990438819019e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -7.2400666525536508e+00,     1.5054132115820823e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.6155889872604540e+01,     1.7492938700087332e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.2392656710506714e+01,     3.0521635344731902e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.0598812514082375e+01,     1.7024666669323633e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.6100437172759758e+01,     5.3763741283515174e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3509364380016326e+01,     3.1713463849051059e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.3100531770602561e+00,    -2.0326677033752585e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.0524138008372947e+01,    -1.6811163672978255e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.1450513937316698e+02,     8.3438811369538053e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4587687443745992e+01,     1.6179698373410172e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3113037580331996e+01,     6.7097176829369837e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0060867282688482e+01,     1.1513883211355801e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.1385042644554989e+01,     8.3271574731747496e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2954559091738801e+01,    -8.3302253760721487e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.6592710744680474e+01,     1.7432608077376015e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1329074014450056e+00,     1.2210096416998496e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7117449634113200e+01,     5.8921860402983475e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {    3.4180907287285076e+00,     7.2025976127556787e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5146576659095857e+01,    -5.3296387830019939e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    8.3218640694493331e+00,    -8.3029522894753372e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.6634467931223000e+01,     3.0317515584033470e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0359608380717699e+01,    -5.0605707401942800e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.9083095036667153e+01,     3.3143913432955102e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.1948824406019460e+00,     8.3270530829467498e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5186035293018072e+01,     1.4915551296665570e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9489432459895575e+00,     1.4221704770076851e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.4752924772794117e+01,     3.0457207708468770e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9098695554941671e+01,     3.7724753425894432e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4903880585861220e+00,     1.3262180732835994e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2771599837420574e+01,     3.9766103628030214e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.3077300349755872e+00,    -1.0640416576908228e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.5385005231887071e+01,     8.2668716476106274e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.6870973158132863e+01,     7.1571806819207850e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3477387879771783e+00,     1.6709640600991217e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8019158897843582e+01,     2.2617460376830342e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.3426984954988566e+00,     1.6605125097502583e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0779073981610015e+01,     3.7777202054698777e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.6967768312873929e+01,     7.9068200597328978e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.5285310499389652e+01,     8.3824117332707719e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -3.8377279260903450e+00,     2.3940137820076823e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.7546300493561759e+01,     2.5197924939548062e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.0457096510083783e+01,     4.0921849533545130e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.1208696169779952e+01,     2.2728431221929881e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.4723669942355365e+01,    -7.7556446609915984e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    8.3613522811193022e+00,     1.5665613165211769e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3335032957496733e+01,     1.0369047334159101e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5421683917654040e+01,     3.0751447336491859e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.2206282764932688e+01,     2.4980174219785120e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9416787601256935e+00,    -3.4132358003516165e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3262526619487296e+01,    -7.7315354347168841e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.1234711956689880e+00,     5.5901638751905248e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.8391343497023769e+01,     2.2216423556515183e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.5824867941629673e+01,    -1.5514402567564508e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6614080937496404e+01,     2.3391683695984819e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.4306939300504034e+01,     2.4499424667276016e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7437231708357036e+01,    -8.4423263130321466e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2695633045830066e+01,     1.7408489375362391e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.1919137620002815e+01,     1.5852542659844039e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.3589065935483760e+01,     1.9996264315486937e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5172484191205641e+01,     2.1125229959011929e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2340617841980865e+00,     2.4228048863209217e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.7768201156689855e+01,    -1.3242760765752460e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.1348452377188270e+01,     4.1679505539520840e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.0833363963024200e+02,    -1.3240799734275683e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.2267554362999636e+01,    -5.5131102869586794e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.1736364927038228e+00,    -1.7271884007961978e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3714092121612616e+01,     1.4547162884859519e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    5.9354490716671453e+00,     9.0202773881356716e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5487289344656432e+00,     1.7253041180361790e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7123000851486392e+01,     2.9420124696665069e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.6122265238397201e+01,     2.4092992791085017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5610110797563355e-01,     1.4614725739769839e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.4052496954710286e+01,     1.5546642182674546e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.1129549458516287e+01,     1.5538866397289039e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1841252032647231e+01,     1.7302551180940009e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9865272252623384e+01,     2.9931065118800572e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    4.5791112595115209e+00,    -1.7554758477100157e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    7.9361400785005354e-01,    -1.1634615542319864e+00,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.7919670355152533e+01,     7.9618151560976239e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9154640767969076e+00,    -3.7080409927755863e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6984238025251610e+00,     6.2688489411179046e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.3226251428471194e+00,    -1.2451519477308569e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6430812365673148e+01,     2.8822983966273649e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.4182440817277451e+00,     6.1369735494118067e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.7994067444457471e+00,     7.9804214888393244e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -3.0484086839841904e+01,     8.0509535137391267e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0441137005457605e+01,     2.8766924391717594e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1379543679198273e+01,    -8.7348303055147026e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    9.1134720294120868e+00,     7.2090871584558763e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4863124060151204e+01,     6.9979544440229101e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.2556395541361418e-01,     1.5345205033743452e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0019870784781881e+01,    -4.3335187700837110e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.8143864202219987e+01,    -1.3397200532898369e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1467572584595271e+01,     1.5537778583079324e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6627342074485568e+01,     1.6029068550650631e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5377519148721994e+00,     1.3240688993177513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7640292771511128e+01,     6.9630437660064927e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1373437805210600e+01,     7.3694960197573655e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2006655244802928e+01,     8.7654832278734011e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.0183278931584056e+01,     2.1245770545573148e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.4301893550501632e+01,     5.7501994294723645e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.8658870866067190e-01,     8.3209240537241676e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.3919406247619051e+01,     1.0745018964244129e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3878270059598824e+01,     7.0497695554187794e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.6106753746405808e+01,     5.6323332587405275e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.4633742040437497e+01,     2.3261187073015564e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.4135127825768574e+00,    -8.6779415607594747e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1180885003544486e+01,     8.3364587955227325e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5695116461016863e+00,    -1.1462677484984451e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.4033385895407264e+01,     8.3650921569851234e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0273422930988287e+01,     1.2772000045232418e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1986200445105011e-01,    -1.5786428343710836e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.0119007189378436e+01,     2.5491522983685918e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2577002127990628e+01,     2.5016727196229047e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    3.9625337678457653e+00,     8.6000194367823824e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.7551086237778613e+00,     1.6135724804090616e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0107498568850568e+00,     5.5403051535996037e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.1374455102869295e+01,     8.3299920111708252e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -2.9227156857232693e+01,    -8.3273908380760749e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -3.9298323057536159e+01,     2.3774542914043755e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.1946086071412257e+01,     1.1364638739281376e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    6.8454331724076845e+00,    -1.7195518934862335e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.2793453200018988e+01,     1.9777240868829249e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3252966667929462e+01,     7.2190385722608135e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0546672875609868e+00,     6.6169336838278805e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.2605557562946402e+01,     2.3718642392522404e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.5406691872950553e+01,     2.3871239999379221e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4291459887320336e+01,     7.6731538775148256e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2858630193769356e+01,    -1.1565770297539329e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1278369058276585e+00,    -8.3106577406827382e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    2.0168841542336231e+01,     8.3279154779332566e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.1745471343830403e+00,     8.3294673713136435e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0210386312687069e+01,     9.5595694450982828e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.9411408986603323e+01,     8.9735074403584858e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.0850904780530659e+01,     3.8141837090981498e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -1.6478481889704199e+01,    -1.1142999268903329e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5579800861276382e+01,     1.5639657090543287e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7632525395652873e+01,     1.6588204416694993e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1203752451208798e+01,     3.3261796601282607e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    6.9674700258629381e+00,     6.7254749021903444e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.1097712054676032e+01,    -8.5441409444449801e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -6.5266932602696158e+01,     2.8628336119423302e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5609828564446522e+01,     1.5547729996884265e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1152993076945151e+01,     6.2759669104505556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9122439715231536e+00,     6.8079409504130545e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4703968385898033e+01,     7.8812189499007382e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.5251043422866912e+00,     7.8203291969591592e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0075994255110423e+02,     2.0091436147571192e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8176884874564234e+01,     6.2829058100499242e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -8.3201422979694470e+00,     3.1080577547705991e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    5.6191061998401857e+01,     3.1090439612221187e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4768737718526971e+01,     6.1306011607473683e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.0582817522355153e+01,     1.5764301366461570e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4497729299910048e+01,     8.3127970803769458e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0038280770146521e+01,     2.3249171604002455e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    1.6732150753247609e+01,     2.8885117116146099e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.4159287475573237e+01,    -1.6608680983432343e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    2.5610713687584936e+00,     1.4984336482955036e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6099574678449489e+01,     2.8731185531414991e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.9906323499285566e+01,     4.5842220930559408e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {   -8.9188426586490479e+00,     1.6193603780159581e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.1560635442338530e+01,     2.5557293960252315e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3872871347319617e+01,     9.6206436656512597e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9372914232233825e+01,     3.7711078762174307e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.7811377020798215e+01,     2.7331082180830010e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3610643934126662e+01,     3.6922932492611353e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.4754419043047452e+01,    -7.6382506897054609e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2944917055916154e+00,     1.3533878607959896e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.9389840862007539e+01,     8.9573922559314342e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5275093352628788e+01,     8.4003960755033168e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.6206579173184895e+01,    -1.5225973074783474e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.3503975653164815e+01,     8.3445857688699543e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6485246154795934e+01,     8.4809873167049191e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8637311225724709e+01,     1.4215946432664205e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -1.6814214870024276e+01,     2.2642645981201167e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.0573112004122990e+01,     2.2732544281730661e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5642229218926415e+01,     1.8151914467006725e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.8437429507092222e+00,     8.3751015662355076e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.2082788499356862e+00,     7.8385571585907528e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8496175334388067e+00,    -1.4738681015029319e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.0314425482957557e+01,     4.3876650164367202e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2039813568354216e+01,     4.0103917902581816e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2506466961265090e+01,     8.5840892667004155e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4902091612123566e+01,     2.4298248111753397e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    5.0843830740002169e+01,     1.5265780945173648e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1254892396483822e+01,     6.5859789374705106e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.5467657965862557e+01,     3.6523367805851558e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.2132184278563880e+00,     6.5320748376347328e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1643488172443829e+01,    -8.5256216519417667e+03,     3.4213389314442701e-04,     3.3421746178752410e-07,    -1.5262904308690409e-09, } , 
      {    1.8609729951944853e+00,     4.2276294621548768e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2188851915355796e+01,     8.3303297663001504e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7495852208505713e+01,     1.3171158244667527e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.2367138478069410e+00,     1.5682473911859339e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.8980647823923480e+00,     1.2416588369529729e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.7458217880134796e+00,     5.8493642362296316e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0919684168135166e+00,    -2.6030239400822666e+00,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    7.4713316456892670e+01,     2.2033337738210343e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4643332294080356e+01,     6.1059932616923734e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0814477043073468e+01,     1.3091328467297295e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.2912236830759669e+00,     7.5606642177003641e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.4524497456318400e+01,     6.0682559025781611e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -7.2876250903418791e+00,     7.7665071321023770e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2721183677863948e+01,     1.4221206097296572e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -2.1383514096674148e+01,    -1.6659450988097524e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9482295450781166e+01,     1.5013773499806528e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.4791217573548654e+01,     2.2032949596416809e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.0348832578828235e+01,     1.5540151266041714e+04,    -2.2065259463350983e-04,    -1.2113821283727471e-07,     4.4937380102042866e-10, } , 
      {    4.8269113157453214e+00,     7.7429912115806392e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0508937746801337e+01,     1.4258391166878300e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.1013077670176793e+01,     7.7577621639593572e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8810704097050998e+02,     8.3233134530958141e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.9687681848336087e+00,     1.4310722283691641e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6502776905374120e+01,     1.9225058133141127e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -4.6451522825303933e+00,     2.2126432027510895e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9403551557450271e+01,     7.9935458491451864e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.7581740447213474e+00,     1.3926899577652881e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.4130791203097580e+01,     3.7497202623712656e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7359674239991389e+00,    -1.8340800416486134e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.0460613639976089e+01,     1.9308670676972106e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.0264188461517541e+01,     3.0397872082413414e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.8240493750043569e+00,     6.6541187534096089e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    2.6428286474444684e+00,     1.5517015154833683e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.1160577545450593e+01,     3.4960327176015744e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1928976819776931e+01,     3.3994389722154934e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -4.4779338827990067e+01,     1.6656994707453363e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.0484789667702714e+01,     1.6657770991040437e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.7172356024046074e+01,     1.6645176924522922e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -5.9736166879183074e+01,    -1.6669588773970878e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -7.6659744916201404e+01,     3.2200039087654855e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.1315897804221531e+02,     3.2200235190802538e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.8543064346075184e+01,     8.2760746433703353e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8504460411236760e+01,     2.4030353006238820e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0919496654709333e+01,     1.4178761363082474e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    7.8266939800853876e+00,     8.8304422527356346e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3569195960445384e+01,     1.5540476773623686e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5857068969205587e+01,     5.8718606321551388e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    6.6516215260244804e-01,    -9.8815971093782791e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -5.7197217912682152e+00,     3.5011060998403163e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -6.7581114358595471e+01,    -1.3086359967129774e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.8203294061561488e+00,     2.7043502879377538e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0224864288569023e+01,     2.8816425010674524e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    5.2465576602143891e+00,     3.1067459636507727e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.4590797251186167e+01,     1.4421566775117561e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    1.6450859900964613e+01,    -6.3171859102853659e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.5053724799886659e+01,     4.0853142189541227e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9227715022936202e+01,     6.6593784180419771e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0778963016236504e+02,     6.9440893973940110e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5706210376170270e+01,     5.5770649154700795e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.8466004668405889e+00,     1.0317225451073999e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.1175960764138260e+01,     7.7848836562622519e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.3997722116224516e+00,     6.1479941761341715e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1621232957994675e+01,     7.6121175373992482e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.9496350562925331e+00,    -2.5381333791367897e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.5425136092165744e+01,     8.3230015965078674e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4430910812222814e+01,     8.3343812527390328e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8370455707386753e+01,     1.6975152757869175e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8180676673519837e+01,     1.3667820940946967e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.2463681396702906e+01,     3.3663364616981889e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1565909607016993e+01,    -7.3204807810831844e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2425391089889512e+01,     8.6026752401736121e+03,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {    3.0749624830281459e+01,     1.4492560080530548e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {    4.5522279019006405e+01,     5.3294924261797896e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2030823348534156e+01,     6.0543504958287504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6536433593166580e+00,     1.4712941517038857e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.6486889128845128e-01,     1.3140857638266907e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.7040679169368019e+01,     8.5346714640186074e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.1681008984016081e+00,     4.1812287962817290e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6645302705719594e+01,     7.8217518878929131e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.6581705021170436e+01,     1.7341047364478785e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.8696166394769307e+00,    -1.0036682679924708e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -5.8444627871203103e-01,     8.4580587970314446e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.2379174895899475e+01,    -8.2553969192690492e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.0149874176845010e+01,     2.4969864938529630e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.4664532056671167e+00,     7.2160272130278245e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2556594318268282e+01,    -8.7761638993461638e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.2247030522384165e+01,    -4.1167944464678731e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.7546083197429134e+01,     8.3813082058765649e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2986164326770176e+01,     1.6731005986976077e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.4569055830103924e+01,     8.8451607611613799e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2015608021387870e+02,     1.6655347332229827e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.8438327714872841e+01,     1.5186151141377823e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0445116107632401e+02,     1.6659418366263973e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1481572382253916e+01,    -4.7500330637710085e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.8336686467812484e+00,     6.5819525107034751e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4158302103281191e+00,     8.3920581464101706e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.0704901510100314e+01,     5.9747206920329263e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2358568551865460e+01,     1.5522404146584522e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.9435621055671461e+01,     1.5563104433379067e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.2703342281820390e+01,     8.3114201628551691e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2182531912230743e+01,     3.2992722795681475e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1123553515410114e+00,    -8.4154324735508926e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.0227621566778714e+02,     9.5835756262372979e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2712500832151040e+01,     1.0018169274681537e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.9671752126508695e-01,    -2.2599100408889067e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.9153540166110279e+01,     8.3512315947583047e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6263244478945484e+00,     1.5735443009070530e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.4890167146248963e+01,     1.4915510366361203e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6700642341395806e+01,     7.4304103716135742e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8460727444279271e+01,     7.7009361291103924e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4115427191918741e+01,     7.2866927923103540e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1037716582726684e+01,     1.4322187536294306e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7654438647616363e+01,     1.9093060928416948e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3423980361628635e+01,     2.3137199935137320e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.7827250776996273e+01,     1.3797203572870114e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4079432346372016e+01,     9.0306221473981677e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9120598645715916e+01,     8.7795877885311620e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0735847824567490e+01,     2.9062513098751601e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.8835804121949487e+00,     1.0239583870547140e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1001957882591689e+01,     2.1078829293138780e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7846581984868894e+00,     6.9945239935202399e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9869944294265082e+01,     2.4477211394518541e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.5232981352468940e+01,     1.7415610096232922e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.3079819382003926e+01,     1.2010496595002500e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1053137912798263e+01,     6.9515537781497615e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.1438579743850426e-01,     7.2151921159196063e+03,    -1.8973801731894305e-04,    -8.4734135299176970e-08,     2.7479239445288481e-10, } , 
      {   -1.9538736546881111e+01,    -7.2129336147970844e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    2.6070356991162136e+00,     3.2191500640845858e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -8.9609529620024546e+01,     2.0131251049469105e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0090895081735564e+01,    -1.0829893700317662e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4977412406492132e+01,     2.3258094462697816e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.6026663020433070e+01,     2.5851540301826128e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4286552818403417e+01,     1.4893828142425276e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3204779900772618e+01,    -1.6669380459990371e+04,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {   -1.1145878305606686e+01,     1.7455507171116806e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -5.4235123357824520e+01,     8.2565112605786635e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4533660026804878e+01,     1.6459308111852173e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.9423085744456081e+01,     3.0010694922596398e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.6134604656112508e+00,     7.7739714993000734e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0532364556842673e+01,     1.4611597234673376e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.0330809413465758e-03,     1.0440274357794444e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2596471129757205e+00,     5.6485379421855278e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.3147952213990461e+01,     2.2690835692890714e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1950907359996844e+01,     2.2785203088750401e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.1141490920466865e+01,     8.5986648925877835e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8817519511450218e+01,     2.2597909863706565e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.8190027129272316e+01,     6.4468468040314174e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    7.3390023564503348e+00,     1.6577822383593106e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.3163641162262245e+01,     3.9902822178748014e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.3043323101235190e+01,     9.6036689110300213e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.9152479764638898e+01,     6.4475514359475656e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    2.6974392234172551e+01,     3.2130247361209909e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.6070916104502280e+00,     1.2747836668573918e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.4123613318528694e+01,     3.2299463934098971e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0437277871212729e+01,    -1.0122269743036826e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.0785210068744089e+01,    -8.3260768518224213e+03,    -7.7245056061153879e-04,    -9.1504412911767354e-07,     4.3024790130065763e-09, } , 
      {   -8.0681235296896418e+01,     7.7471042713814149e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.5518904527701025e+01,     1.1468498381373925e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.9875806033287908e+00,    -8.7701304249034110e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.0059087887223278e+01,     1.3959077753262794e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.3873986214717558e+01,     4.1641155923175837e+04,     9.1375444657395709e-04,     1.5043132449279283e-06,    -7.4159039917239050e-09, } , 
      {   -9.0651605845581145e+01,     1.1508313283271838e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.7331679733991319e-01,     4.9960335516401528e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4252826298617848e+01,     2.0528073969105360e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.5127085696026271e+01,     7.8579124786923185e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9112938106710466e+01,     7.2139220780853320e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2530053888900667e+00,     7.2142036526313568e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3741417460622962e+01,     3.1542963899470433e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.5361537754091761e+00,     7.6687446503445672e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0040406265543655e+01,     2.5193322539665241e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {    3.6625837720583121e-01,     5.5180260597011238e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8792280129009626e+01,     8.5262406808617925e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8043807738932756e+01,     9.9024136450720998e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4049361672938986e+01,     2.2105967665162352e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0183684604366121e+01,     1.1160020305802671e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.0033150409132254e+00,     3.7636151870488984e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.1097926624179308e+01,     8.0499871113380650e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.7102954692749694e-01,    -4.3363485778211298e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6499181134396252e+01,     4.4783351886471100e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9873108846842111e+01,     6.4924552757862275e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.9910832065484598e+01,     6.4282266411983910e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1913071102210822e+01,     1.6950544858814181e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.1343367287694370e+00,     7.0165136091200002e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2212152122457322e+01,     4.0830130190127111e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7999202851473271e+01,     1.3308228263085008e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4301027502863029e+01,     1.5350065570893061e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    4.0205176650721688e+00,     6.9925157888785707e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6938752732117528e+01,     3.2218186082684559e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8462546842193861e-01,     7.0617669732768754e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.8170485142963848e+00,    -5.8308801955809549e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7198227701669683e+01,     1.0444971671829688e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.5143274737631794e+01,    -9.6549469924085570e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -8.7452429418847231e+01,     1.7657835611184844e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.7128657916630434e-01,     8.3382893604120618e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4859053977891577e+01,     8.4887269789052934e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3255915607777840e+01,     1.6657737452696718e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.9536541155735254e+01,    -1.6657028245797079e+04,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.9097906445117516e+01,     1.6852329081545162e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.1463918190241360e+01,     8.3190934888348384e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4145907349170429e+01,     7.2050274714974548e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9637424292420910e+01,     1.7396525449979446e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.4016345906581556e+01,    -8.5152761741614468e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.7458283491549089e+01,     7.7871096289901961e+03,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {    3.0997578816904081e+01,    -9.6507919068411575e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.9141170490278121e+00,     6.5855316231550914e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3256591203642950e+01,    -2.7562699840684991e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.4014188820489878e+01,     2.4111663362215393e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3546082047915178e-01,     1.4410076787260830e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.8232467232139253e+01,     8.6289712406400176e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0379765477775855e+01,    -8.0284116086068843e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1139665650742494e+01,     8.3339510892558174e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4537124278930236e-01,     8.3234317599910828e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.8812182684321940e+01,     1.1771373721649798e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5661025707572023e+01,     1.2894537215817456e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.8561061448666223e+00,     1.5540349353160873e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2668390518565493e-01,     5.3990298882404995e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1373639154900486e+02,     2.2781090028949620e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.4234349778238595e+00,    -8.3378802125387665e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.0027721530253849e+01,     5.0747086705458114e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.1744557368159025e+00,     5.0540117845951445e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    6.2971213877087578e+01,     2.1659461592840704e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.5247825611196049e+01,     5.9817205688767408e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2966953020272837e+02,    -1.1142829568577583e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.4514695195380980e+01,     7.2709450487959039e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9650483858853150e+01,     9.1688496753467043e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.9147882708870057e+01,    -8.0493053433391578e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0965993827408774e+01,     1.4663412346069244e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8925074983671326e+01,     1.8878437829321287e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.7376311039018141e+01,    -7.7867962561305740e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.7853992874752208e+01,     2.2753014385730814e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1006765429754246e+01,     2.4650928698392087e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.8585551019714597e+01,     4.0526198731535958e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -8.3001436207898251e+01,     1.7363589269611031e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7487220282911636e+01,     2.3944740219959644e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0704885556107318e+01,     3.2807114816296387e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.5030450272902613e+01,     1.6411279522637487e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2851097833124555e+01,     1.5056785631720308e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.6014800120129451e+01,    -1.0920866541328614e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    5.1480093617891377e+01,     1.4433056762974287e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.1634094925924650e+01,     6.5684822835471350e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -9.4199000681472498e+00,     1.7032354875484027e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3077505841404543e+01,    -8.5536600014994292e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.8267374552618310e+01,    -2.2682246250196977e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.0390723929398260e+00,     1.2354815698880933e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0766709245514923e+00,    -8.4166054108440349e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.6576954994208769e+01,    -9.4453975266722555e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5092423216241855e+00,     7.2051844768578021e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3380198346738322e+01,     5.7375801910543805e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1345002335504910e+01,     5.5306452981191114e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.1824591483259702e+01,     2.2182190012972664e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.0981068349006250e+01,     1.2640105776141552e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5378969737921832e+01,     3.2494383480802513e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.4116558686826107e+01,     9.9318480128965293e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5204697412078358e+00,     1.5321207213502021e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.6087577966350139e+01,     2.4300678790161226e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2624628408549391e+01,     4.0531458396168324e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -1.8731586810674872e+00,     5.4404737986890004e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8130911497213575e+01,    -1.0326059008610000e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.8226105459803215e+00,     8.0830336083235243e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3316047445414494e+01,     1.7321027505229165e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5151602702726475e+01,     2.0914554216159213e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9235365049385063e+01,     6.1524034033044318e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    8.2574138475955259e+00,     8.4952447039142316e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4830175178370233e+01,     8.9258474145300170e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.3970903678835938e+01,     8.3782053360779082e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.6175095012203833e+01,     8.3158895452682464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7414104227605108e+01,     2.1249167626382048e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    2.5305563861815934e+01,     2.2760619924949468e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.0551603507857328e+00,     7.1682838271597111e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    8.2139182088861503e+01,     8.6073957379088351e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.3785467330272088e+00,    -8.2816223814974855e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.0685996011631619e+02,     2.4985566315231230e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    1.9586822546758518e+02,    -2.4986582232509467e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -6.2080017625052664e-02,     7.0191166330600827e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.0914783063807080e+02,     8.3290571676829786e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2014212504602401e+02,     8.3283256815639197e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.5181016384259813e+01,     8.3414933039786538e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9052772249301588e+00,    -8.5190532206037205e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    4.5366589784758112e+01,     2.4504684331908382e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7058027229639332e+01,     1.4378462333281552e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.5251578474274190e+01,    -7.2253023876812622e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -9.2757918713057634e+00,     1.5638668867309400e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.4469490432021246e+00,     1.3924269745336698e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -9.8461667445753456e+00,     7.1184616608903598e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.3125499415189367e+01,     1.0611566181710663e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8022537100840601e+01,     7.6005248794521995e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.5003335143056704e+01,     3.3332789906695594e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {   -1.4325209878830369e+01,    -1.7450136870943493e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9522454274255146e+01,    -9.4412424411048523e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.3812010018710996e+00,    -8.3036636349422133e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.3128080790613637e+01,    -2.1671021478249413e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.7950818434666825e+01,     1.2755079364424824e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8446875912610242e+01,     7.1479395728338741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6623053243634640e+00,     1.6654779825306818e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -1.2268589521363200e+01,     8.9345645099618232e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7030225833797427e+01,     7.6385640625650831e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.7919348866224034e+00,     1.7850008295404430e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.6467541077645599e+01,     1.4434159205159442e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    2.0693325509809036e+01,     1.1779139102932895e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.8467176125978781e+00,    -1.1429620487223924e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2701096561017565e+01,     2.3359385605237574e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1972517816198256e+01,    -7.5606984286933384e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.0393463542222978e+00,     9.0591707185970627e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2199654488350255e+01,     1.1413340605095935e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.7491578241144037e+00,    -3.2892828734663444e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1245179446333655e+01,     1.9494623229826175e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    1.2816137862489339e+02,     2.3869410197588168e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.0284327996179314e+01,     8.9175364237291560e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0753122737551104e+00,     7.5698719570935027e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -4.7036852860182066e+00,    -2.1202093704891854e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6321329923652645e+01,     1.7735890666693451e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.7778892892895314e+01,     2.7333383380771418e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6842779161825536e+01,     8.4188629240535611e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.0505567852357103e+01,     1.1203791139359633e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6860681877422152e+01,     2.8767233947818593e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -2.2489408526089790e+01,     6.2600638510413228e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2159892802790527e+01,     1.5599636473419354e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.3589761943644323e+00,    -3.8062567700446034e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.3963919663582907e+00,    -5.4378592258879689e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.4639981390335972e+00,     1.4998557921547510e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3373673971850748e+01,     1.6086453588582806e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0334570414355710e+01,     3.9159768564911137e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.4005901070089998e+01,     3.2195206106971094e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.0754127286957230e+01,     1.1780722351629560e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9123578557926081e+01,     7.4362315695126345e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -6.8365416519466251e+01,     9.6008486230029666e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0098379039829181e+01,     7.8515592383192052e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.4070448257582015e+01,     1.4913383043511301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0908917173606770e+01,    -7.7622036165907302e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.0575721182952869e-01,    -3.2374677620704751e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.6373494645144234e+01,     2.3873481231622322e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.4180051798207005e+00,     8.9558355481147610e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.5538219590707328e+01,     1.4289952411703844e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3369688391282452e+01,     1.0971617769588391e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.5869676085298678e+00,     7.1367773239327953e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8091419454514902e+01,     1.3613732068913429e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.1201640421426143e+02,    -1.1149741616724539e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7937388953673182e+01,     4.8713561583400970e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0768420967960779e+00,     2.2772549639339442e+04,    -5.3404892121072009e-04,    -3.5148865828884261e-07,     1.4224918217434891e-09, } , 
      {    5.0055650021033770e+01,     6.3765581305345740e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.8987814465203407e+01,     4.0002692687734670e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.8043369607186381e+00,    -1.0726800887762784e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8708789003623394e+01,     1.4979147774336685e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7370487116778266e+00,     7.6797652770669320e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.2363482339677407e+00,     5.8128312521250937e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5278319015611800e+01,     1.0881763475079639e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.1903613089414122e+01,     1.7952081952758665e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1335068896848709e+01,     7.7948298378684149e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.5216706742706840e+00,     2.1154080354209495e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.8320676389238777e+00,     7.6758171711131063e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0080576372408530e+01,     8.9466897284135048e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0002213907648468e+02,     1.0211883211282840e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4551027899336484e+00,     7.1243340062766783e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7752480002744262e+00,    -8.8330366070448126e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.3651646584262572e+01,     2.5931170105621954e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.6318316288124485e+01,     2.3238207097302107e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2517988571320138e+00,    -1.0124596666145288e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0120377891855485e+01,     7.7559580338512205e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.3883283394039793e+01,     3.8352540542533126e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.5147630110933136e+01,     1.7497541099970153e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.8299104999948860e+01,     1.4563112464698856e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -4.4704887394689452e+00,     1.1961794729187945e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2900839117770049e+01,     8.3342735890868498e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1897300169964460e+00,    -8.3231092601600467e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.4192108705466421e+01,     4.3696313324960083e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.0013623783426358e+00,    -1.0674198004363388e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.3625206751609612e+01,     1.6157555363755982e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.2412069459675621e+00,    -1.7947512557065186e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.4188208314912735e+01,     1.4852089897906095e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0063659084468844e+01,     1.4993724192774000e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3772759300256203e+01,     8.9833074002955509e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7213909953128219e+00,    -9.7492788886538301e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.2174577622087305e+01,     2.9344873635594351e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    8.4883435578291966e+01,     1.4914652420162274e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8512839416428065e+01,     2.3850827152094269e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.0471546421643954e+00,     1.5105962486731845e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1094449704146566e+01,     1.4180200925468325e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0196266772680760e+01,     7.6982416013560860e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.0479534805706727e+00,     1.6085742243115930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1703780942992458e+01,     1.5994130375191966e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -7.2292674081918307e+01,     9.8950949645767814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0828991275014154e+01,     1.5476630997457325e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3781262723040149e+00,     5.5049680826628819e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9051921935441126e+01,     2.4411839313430202e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3092434499600213e+01,    -2.9597364125038230e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.1588348035263856e-01,     1.5137114081870819e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.6053232332130364e+01,     1.5557710623119909e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.5876170135121903e+00,    -1.1899661579315500e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.8071523681082667e+01,    -1.6688863076760648e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {   -2.0321715476291448e+01,     1.5476278681499249e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4833845942009933e+01,     9.4521384382290726e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0120876936703203e+01,     8.0174908899354705e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3807981461853956e+01,     4.8202772308699086e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1754534809151332e+01,     1.5121347386878548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0746067695255600e+02,     1.9757374904099466e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1649717528189667e+01,     8.3213686796044694e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5322198255529543e+01,    -8.5544502257004679e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.5002807111913419e+00,     7.7423323100667340e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.2511883478543595e+00,     4.1474473688265680e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6147258051128098e+01,     1.9537398336725528e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.9569495867813501e+01,     2.2128411518720757e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.7659275439669720e+01,    -8.2845919586917116e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.6285537469710519e+01,     1.6100061856063621e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -6.4894311971326232e+01,     1.6100075283165075e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2936966059254969e+01,     8.2625681320989788e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3712732495504966e+01,     4.1645758323058661e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    6.6963062960141411e+00,     7.7570508184924802e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6414809638332052e+01,     8.3267244505719773e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.0123758143811159e+01,    -9.6529675011986965e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.0597808090264614e+02,    -9.6527713980510180e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.4651981772103538e+01,     8.2250202191495064e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.5967190515463239e+01,     6.4122542309800983e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    9.7073694931773993e+00,     1.5535750708502361e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -7.4497084918106310e+01,     1.0817648891196141e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.4802803441104424e+00,     1.3828683800383864e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0653477347735727e+01,    -7.0874953656720409e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.7362718330222933e+00,     1.6615434378758073e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.1088753920590984e+00,    -1.6873491270582334e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.9703678667056419e+01,     1.0549746404697838e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.5420486410919899e+01,     8.3306583986749229e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1786078634385863e+01,     2.0352305930824223e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0624405219175127e+01,     8.2661243916236472e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5795045149323059e+01,     2.5052809823625619e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4217403302309894e+01,     2.3707450565759127e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.4800066526059448e+01,     6.0464652088820494e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.7669881142396772e+01,     8.3152211974671063e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9237704058298650e+00,     1.6594104470780221e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.9993613177466059e+01,     1.5309768874570649e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4219809061469135e+01,     5.5109416229456092e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.6424360375051734e+01,     2.1657936339589487e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.1654879460976701e+00,     8.7489348661724653e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -2.8704090581021493e+00,    -1.9923005119448626e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.9219107471363394e+01,     3.7095925372442041e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.2769654385402340e+01,     1.6095810813956625e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2913627321985651e+01,     1.5780300874685565e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.4521014673060222e+01,     1.5545736438256054e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5738949883238234e+01,     6.1432203991653532e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0502252403396049e+02,     3.9414193291036310e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -5.2440503835253963e+01,     3.9414206718137764e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    6.2612378013858319e+00,     2.4042297980143943e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0020837796646690e+02,     8.3294318823750618e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.9214083558513707e+01,     8.3279509668718383e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8083932129819427e+01,     3.9978073239818717e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.7821282871450741e+01,     7.3030803789731362e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -6.7906586537058189e+01,     8.3294094836410532e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.8900880945011380e+01,     8.3279733656058470e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6914759241040329e-01,     8.3912584576232530e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.7195964869173622e+01,     2.3871651429831269e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.3141406100671606e+01,    -1.9098257940912467e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2631099559207813e+01,    -8.1583378318650311e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.7621326042056619e+01,     8.7305288631250987e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.9725183559646609e+01,     7.2290191984964576e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7638455071590201e+01,     1.3753432739313148e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.4615016347282364e+00,     1.5611446395453375e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.0302869117848203e+01,     8.3271015701489159e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1297163525801395e+01,     8.3302812790979842e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.8626062810082097e+01,    -9.5536895167345665e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1663825374487121e+01,     1.3259424546846187e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.4769774162764270e+01,     8.3293214312631480e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7888016868421623e+01,     8.1416838640193928e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.7925419770401465e+01,     3.3308206742894668e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    4.5287226509373379e+00,     2.7222471856026459e+01,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.2416902039064652e+01,     2.5043949668085075e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.3616533920019709e+00,     7.2420041728883007e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.3558736629857790e+01,     2.2327584079784159e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4385554694503519e+01,     1.6666261237747443e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    8.1049291465461017e+00,    -1.6648504460746357e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.0741345146380553e+01,     9.8101051311850097e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2271676764012263e+00,    -1.9498557777307660e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.1783901891175880e+01,    -4.6675497705012913e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.5248032468303251e+01,     2.2661257181462352e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.7061163018615993e+01,     3.9493829808382863e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -7.4077493147449104e+01,     1.1151710134667868e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3317920734554745e+01,     6.3975357225543512e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.6966111633921188e+00,     1.5514757171014326e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.1616685965759332e+01,     9.9966898807230064e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6883536084542332e+01,     6.6049429500688329e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.4318393026818683e+01,     7.2135641925780692e+03,    -5.2741442090551219e-04,    -5.5511302234872569e-07,     2.5427507946832937e-09, } , 
      {    1.3022365892512973e+01,     1.4759342182912005e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1377644017985041e+01,    -1.6199927529740056e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    2.0658095376419219e+01,    -4.3992030834641400e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.8884396698284377e+01,     1.5649403829871053e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7662288114907891e-01,     3.9414528636961812e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    5.1605405920056945e+01,     3.9413871372212270e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0056023485963106e+01,     8.2439945487859368e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.6888617207339955e+01,     2.2772236266479813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8658490371144172e+00,     1.1712955381796792e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6514560775223742e+01,     1.1898895882310519e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2820927663919498e+01,     2.6695169056565814e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1875669178663749e+01,     1.6676510584842905e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5957883339164727e+01,     1.8195685300563688e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -7.6919982296150096e+01,     7.6996792076617576e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1456812622107634e+01,     3.0950521845981420e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6712562502279894e+01,     8.6410316282078493e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.3251608076611552e+01,     8.9548567714471719e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.4494047507486000e+01,    -8.9724181208086702e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.3128116852704927e+00,     1.5551927818381959e+04,    -5.0095733314880326e-04,    -3.8254751954860338e-07,     1.6265014187543818e-09, } , 
      {    6.9692799912541172e+00,     1.6699331319735727e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.7863258369549015e+00,     1.6720661227713579e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.2739202370692585e+01,    -8.1816377417557796e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    5.5942530900470935e+00,     8.3563135686138612e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1581266502759910e+01,     2.9216004935966637e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.2441143639558049e+01,    -6.3060878498695274e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.8546402198659266e+01,     3.1968948554140579e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.8918690416116615e+00,     9.2698378860336725e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3863793273690572e+01,     2.1606851525138314e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1978705916248492e+01,     1.6884956523184999e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.9582763055023705e+01,     2.4257907728699098e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.0222170493144807e+02,     1.1134437137902198e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5124255823471331e+00,     3.3640352617567773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4165358689096497e+00,     1.1933367916244072e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4948860952255842e+01,     2.5210544177966047e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0516395926314836e+01,     1.4603909028512984e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7903426671603725e+01,     1.4332953898632435e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6818243846416554e+01,     3.0440280314110452e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    4.5635484280136396e+00,     3.2262493946258852e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.8237656322481904e+01,     1.4912304466714431e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    4.1527702684274914e+01,     8.9281486144714290e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7334390164037558e+01,     1.8653140620070535e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    6.2002294554794979e+00,     8.8827000044799497e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.5707201080295938e+00,    -8.8852943587891277e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.7980330906980416e+01,     8.4187961846660146e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.2448438154019179e+01,     2.5001806757869646e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {    5.4530295625857924e+01,     3.7673565060217807e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.2014359714601439e-01,    -1.8922688384180710e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    6.8624177597763198e+00,    -1.7196672920677658e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.4139015610098678e+01,     1.0807871976499189e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7831805185463764e+01,     8.9541540956112021e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2071942318843561e+01,     6.6124159868974348e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0973993478878633e+01,     1.5744259477879190e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    6.9944067861898969e+00,     1.3026894117483438e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8819967505011618e+01,    -1.5040467329231751e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.5029220994090160e+00,     1.8433586488717279e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -6.9000115644263190e+01,    -1.7273941160222194e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6914806075519852e+01,    -8.3985554794751824e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0083007035735912e+01,    -1.0888914096488515e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.1477597839013576e+01,     2.3306416508026381e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.9127688170233883e-01,    -7.2318447845895080e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.8302411398907847e+01,     2.3350179851376291e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    2.0151189751028586e+01,     8.5164570736480619e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.6383206672561030e+00,     3.3350239262157379e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.3136486447376479e+00,    -8.8867170497228799e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.9372810582144037e+01,     5.2643771280330857e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5435345634168542e+01,     2.6340905221230460e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    5.3926711499142339e+00,     8.9776175721799682e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4493543840319433e+01,     2.8652131700734953e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.2576968439266949e+00,    -2.3689168062477279e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.3763354799392404e+00,    -2.1539907979443456e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.7133997048720957e+01,     1.8088799655218063e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.6142964646641556e+01,     1.1310969742259516e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5819626896528717e+01,     1.4925936692792977e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4812471931087234e+00,    -1.0387096277614961e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.2965082155467091e+01,    -1.5533580761581627e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.6493720952942724e+01,     9.1353968802663185e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3331200680829031e+01,     7.8953300718761675e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -9.0617993732445541e+00,     7.0270553047542871e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    3.2338037968271976e+01,     3.0546218508532835e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -5.1072378156558056e+00,     1.2197081344540500e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.9244801208536018e+00,     6.3816202600992228e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3121753251378763e+01,     2.3072285864021003e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.7810432613096880e+00,     1.2726146783150837e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5411958157726584e+01,     5.6648780803271984e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.2000965245643243e+01,     5.3445592213613927e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.0822003871902368e+01,     1.4937736130013218e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0926197307592709e+00,    -1.3117265055240987e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.8666979759885614e+00,     9.6476634017446922e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4927718537406108e+01,     7.5475805174950692e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1183958957235454e+01,     5.9595252380509583e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0823079786284914e+01,    -1.0283478234611812e+04,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -9.6748291773392818e+00,     8.3380791344424160e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7356893349636682e+00,    -8.3566327321534045e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9580813771365055e+00,     1.5769224436525688e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.9296981004738868e+01,     2.2686176165885223e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.5220235721208848e+01,     1.6012865928860640e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4012910549293352e+01,     2.5947884251537471e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.0732033609739652e+01,     1.6153788878261668e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3123084616631542e+02,     1.5543099892389142e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0571583838939544e+01,     1.5117539874129852e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8549433671321875e+01,     1.6315678318169503e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.0485907131877987e+01,    -1.1409369498916640e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.5636220114911168e+01,     3.1062966674831347e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7129757889557542e+00,     1.0325173661861867e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0216040597261845e+01,    -1.7741145478257267e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8401627475974315e+01,     1.0193937312962680e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    8.9421712975297325e+00,     5.5513257803219685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.9016577331774824e+00,     1.1568773759494958e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.3218437384038788e+00,     2.4500081932025565e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.0606800877821354e+00,    -7.6843843313539091e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.2492081888670175e+00,     4.4786638210218825e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9779081952747216e+01,     2.3090878398811867e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.9999910492176326e+01,     2.3882262425862100e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    3.3352004572888532e+01,     3.4387410691847923e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3464939117231864e+01,     4.4034127599511703e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9939989640521006e+01,     2.6312054826032887e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4722506854808666e+00,     1.5663286242103306e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.4101521729065990e+01,     1.8052929513408053e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1173540733206167e+01,    -2.5460453652362703e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2785419527450363e+00,     7.6929056730088496e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9795281551694945e+01,     3.8246602348110740e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -2.2670087599786559e+01,     7.1475872568757986e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0991765358175982e+00,    -1.0092291085190993e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8274416309183223e+00,     1.5571106012399079e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {    4.2519729068701935e+00,    -8.5743492409233768e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7011733826510259e+01,     9.1983459078404048e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    9.7356225215380150e+01,     8.3283811344320002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7245718368170454e+01,     1.1188463182987614e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.6361944817762151e+01,     8.3290017148149000e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3279535699280798e+01,     2.3892064277116220e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9057999225503721e+01,     9.3995338908164395e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.2567987390241800e+01,     1.3525297474186937e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.6721348894253403e+01,     2.3871057572811711e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4178244252877599e+01,    -1.6016456025783373e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    4.1529211068639018e+00,     8.9989474801578199e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.8509542257891596e+01,     2.3871833856398778e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.4603859254945483e+01,     6.1480107377418915e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6401384485095491e+01,     1.0717470165140072e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.7097249204774316e+00,     2.6046491411865509e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1734747342575922e+02,     6.5837197632957259e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5630124957324163e+01,    -1.6023041789758605e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -8.7126500366007793e+00,    -1.3549297719099573e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.5105447947621244e+01,     1.2560475972345728e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1506432008000687e+00,     2.3867036487434987e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.5690678989443683e+00,     7.7006074967356162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2212047650569728e+01,     1.6808558247988200e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.8077648612053274e+01,     3.0710536553726459e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1832438177295209e+01,     1.3400307630277963e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7884689968950212e+01,     7.8274500120444918e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -5.8326084532311278e+00,     4.8855218788534185e+04,     6.9527886268779720e-04,     1.3157120932927950e-06,    -6.5879391871250393e-09, } , 
      {   -8.9919924610770579e+01,     5.9573496436934183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1328869704222924e+02,     1.6170553916388222e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    6.2610400488483560e+00,     2.2741398044200461e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.2908957538861159e+01,     3.2280319232557005e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.2818437013120928e+01,     5.8151324520665048e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9246819280213160e+01,     2.7724103150522995e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.4549965310730926e-01,     1.5542718464146335e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6345430745039437e+01,     1.4750153394996687e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0119334182851983e+01,    -9.8411667678070080e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {    4.1353387390803171e+01,     7.6895671283210522e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.1901919114315920e+00,     1.4601626780049393e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -1.7712685601974769e+02,     8.3340693961510860e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1100020052471862e+01,    -1.4726059428237124e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.7322608760472530e+01,     1.5542790115817257e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.2672931381516108e+01,     1.4059712660078698e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.9621534758137766e+00,     1.7179040894403184e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.6562817626752505e+01,     8.6902321668026889e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3313902199310597e+01,     1.3806377101270280e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.1419929037089434e+00,     7.6546048013792752e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.6723424328389500e+00,    -3.1281992488685898e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    7.4861879595088809e+01,     4.6628615185903465e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    4.9956594270999251e+00,    -5.4812549171723549e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.1970333665565498e+00,     4.6627910553987305e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    4.2670102918597888e+00,    -1.6821676159186463e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    2.4060707163229193e+01,     8.2998257699107344e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.9475864063093375e+01,     1.4903629993679397e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.1183356182583890e+00,    -9.5661189249585059e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.8216200530414667e+01,     1.5687370733123960e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.5485981135148670e+00,     1.0264256353862807e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.6814186107626064e+01,     1.1785698058532031e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2727329501218136e+00,     8.3970548977202561e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.9925919097629521e+00,     9.8615449403400760e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4558444111020371e+01,     1.4787838742453750e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.6629924105469293e+01,    -7.2386707644133385e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.8405828344975447e+01,     1.1587943087067664e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5842710681196564e+01,     2.2073152640108252e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4464725791719665e+01,     2.2111588889487004e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -1.0187668798176395e+01,     1.0389710850415995e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.0987058101077061e+00,    -9.7587886925694183e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4190535782813672e+01,     6.3166872375050862e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.6844703075735374e+01,     3.2106837134702120e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.7966074529485681e+01,     8.4710303514075058e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9063882083962664e+01,     1.6659660365605010e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.6721935562638819e+01,     7.7756349006486198e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.6541079192478051e+01,     1.7285683720741719e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.4769295524051941e+01,     1.6655105332888790e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.0617187018616843e+00,    -1.1150218055275479e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3664806360349388e+01,     3.8197399736393869e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.5418854188196960e+01,     3.6587802155665420e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.7661840452676131e+01,     8.3290217919956103e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.8656134860629322e+01,     8.3283610572512898e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -7.3312526699220566e+00,     1.1175752789801969e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.3316249457141904e+01,     7.2146844930328607e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2625576528530507e+01,     8.9608017792783194e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2000108950744972e+01,     1.4539963979224302e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4828793572997379e+00,     1.5508615116649019e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2712984026313315e+01,    -1.1142353130026636e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.7008053052353809e+00,     3.1640521659654733e+04,     2.0531005133890193e-04,     5.9461433208859170e-07,    -3.0839967882739798e-09, } , 
      {    5.6433831382372768e-01,     1.4910037477765993e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.1551145130830562e+00,     7.7959697845307410e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3938010652998639e+01,    -9.9471157911986720e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.5550938324180507e+01,     7.5137990900399091e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6273651760044736e+01,    -9.8674859874028480e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.8448398180281849e+01,     1.4500755657668698e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.6354124324667765e+01,     1.8246328294332478e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.7710538155812028e+00,     2.6555455833829265e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9269453393085463e+01,     1.9519869372666708e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4483166798124710e+01,     6.8740015171376381e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -2.8595524958884624e+01,     1.4925263416193107e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.1984012282101659e-01,     1.5791158816532617e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.0860364901108648e+00,    -4.7293695475141167e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7874890880840802e+01,     1.8083511032283059e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6407121299322533e+01,     6.0221487646676896e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3772669464920668e+01,     2.6479847886871670e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.7724383644831345e+00,    -4.1992777397375912e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.6595135290613484e+01,     7.9122345052556193e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    3.6590280001158874e+01,     2.1518250928704918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.9756240499149946e+00,    -5.0448382561949802e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.8036377891517816e+01,     3.1091542054406342e+04,    -1.0145177493310048e-04,     1.6063952257714078e-07,    -9.9061979461111474e-10, } , 
      {    1.0507580454078944e+01,     8.3363539076167908e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.1089735579331741e-01,    -2.9400327373303207e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.7598561613081198e+00,     1.0782731820108982e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1513996658002890e+01,    -1.7489696187533998e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -8.1450410147816470e+00,     3.9898219778865190e+04,     5.4298645492547107e-04,     1.0649932191381403e-06,    -5.3519551885043885e-09, } , 
      {    5.1753990324801691e+01,     4.8859821188417001e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -1.4792007020623263e+01,     1.0578596799895402e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.0415540137988772e+02,     1.5542408687574447e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1882504349021094e+01,     1.9701691646665095e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.9886776970138527e+01,     3.1645124059537553e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -4.0478258833312765e+01,     2.6145786643399944e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9440136465177503e+01,     2.3558606838240565e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.0502429351721737e+01,     3.2205068171486291e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.7561452389724383e+00,     7.7675743753815714e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.9462962924326881e+01,     3.7748816121288510e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -5.8358971955615068e+01,     7.8267340751772390e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.6331135742533476e+01,     1.5564233683940327e+04,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -1.9254083238727471e+01,     1.5521274896023262e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    1.8023652990735211e+01,     1.0340196938961537e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.7084758224675255e+01,     3.1023151772933434e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -4.7066332363606548e+01,     8.9168317918130051e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3358059944081231e+00,     3.0457529627292817e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    8.4829589685150619e+00,     4.6848330169521741e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.3925813216606805e+01,     1.6036075141045172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2882512517281516e+01,     5.5417278445333568e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7629948338879487e+01,     2.2552697394641709e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.7599861059008060e+00,     6.6724963292402426e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4838435736187380e+01,     2.5066256367198657e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.6353527710222366e+01,     1.5446839712654189e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.7436851319938746e+00,    -1.4043601183297151e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.0806632081510038e+01,     5.6573980977260726e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.6829749618252485e+00,    -5.8453675148857928e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.0496368539561374e+00,     7.2134412376838291e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2922088349069796e+00,    -1.8767602813634287e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    3.1571654533854542e+01,     2.3121486402615843e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.5072181663056476e+01,     2.2066152763264439e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.8550490935890082e+01,     1.7069342928037215e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4353660846825774e+01,     7.4247663169099669e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.9329644203028380e+00,     3.9641805050188395e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5242179723147082e+01,     2.8556530496266965e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    5.7322474998310966e+00,     1.7312862734922660e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.4689511314091781e+01,     2.1605428834204562e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.4494875225267450e+00,    -1.7635996605675418e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.2148826833238658e+01,     8.3225371061539863e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7437717732465046e+01,    -8.3348457430929120e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.2502482853634262e+00,     7.7002354225206927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0273353379477435e+00,     5.1250050900081360e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.3970732991322635e+01,     1.1842374526743566e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6236811520697138e+01,     8.6394461968867581e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.7751317185564837e-01,    -9.4114646141022286e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.5953937338931837e+01,     8.3276159590613261e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    8.8101235584568428e+01,    -8.3297668901855741e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    2.9330453338670416e+01,     1.5495854951328369e+04,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    3.8209590632323987e+01,     6.3785070145179088e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.2209000664858365e+00,    -3.7664854943142529e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -8.7367274213050266e+00,     1.5602257607538060e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0393297300611067e+01,     1.5982175925547928e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    9.8346841665352027e+00,     3.1079475105520836e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    1.9811312665994958e+00,     1.4422092256273936e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -2.6613009786301532e+01,     9.2027939509620628e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3920610466744705e+01,     8.3914005476116818e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5810644288046074e+01,     8.3754879245366083e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2558702255159661e+01,     8.3292980049657599e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3552964416613086e+01,     8.3280848442811402e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7407592033761281e+00,     7.2075113999662653e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.0026482306808855e+01,     2.3872541595472729e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.1814643911388472e+01,     2.3870349833737760e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6879245503290399e+01,     1.7140704895979306e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0916872423326469e+01,    -9.2598484799318685e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.1284197522640099e+01,     2.2739544158574474e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.2653414375541267e+01,     2.2292802880234161e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.1165505753973495e+01,     8.9968407199430076e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.7315979045043832e+00,     1.4321084008900101e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.3851802604454051e+01,     2.5436970637778060e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    7.3018368523276695e+00,     1.6674695711981825e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.9242075839189567e+00,     7.7751799146002222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9451263735724677e+01,     2.5494937153222156e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.3539444781254247e+01,     1.6586781725761241e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0726967950616707e+01,     1.1026878591002082e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -8.1252703725694566e+00,     9.9419097433185070e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1221095856145297e+01,     1.6728728470617967e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.8905275699383424e-01,     1.6468481640252343e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5042374233878117e+00,    -1.0151809662262407e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.4583968309349807e+01,    -1.7342996459278147e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1766478157838785e+01,     2.1535146874258444e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.4199350972345153e+00,    -2.6529310105818950e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    8.6636210609713959e+00,     8.6184229836880604e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.9648255138159108e+00,     8.3948147171479195e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.2937033268513630e+01,     2.0465312292940835e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0305061922806988e+01,     1.6048202999797362e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.8549271517365838e+01,     2.3078844819620139e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    8.0932677777550168e+00,     1.6335355184966904e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.2481185986265526e+00,     8.3349836606360077e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0659158050419773e+01,     4.9515382516684058e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5396261190893654e+01,     1.5501284656366246e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0706544427866376e+01,    -1.0660120252081728e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.7178319451914412e+01,    -1.1319015560307739e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    1.9377333960555498e-01,     6.1332756677084253e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5374907304736229e+01,     2.7093165733161266e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.3788722572005398e+01,     8.3417940763065635e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.0982185734290209e+00,     1.6684605321102925e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -7.6881563323791909e+01,     9.3809595490778884e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8073972673277297e+01,    -1.7981913190206382e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.8326687449332947e+01,     1.0530721230117644e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.9355726319225841e+00,     7.6827518133034373e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2664878626294936e+01,     8.2964286891018346e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0775041081691548e+01,     1.6655079914308102e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -5.0696375729647603e+00,     1.6659685784185698e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9217172749483325e+01,    -1.7981208558290236e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.8869288623461301e+01,     3.2669537432640664e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -5.7913603651912853e+00,     1.4985768642724015e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.8554237366536888e+01,     1.4985111377974468e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.2670260180317001e+01,     1.4550273260479789e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5823979690215950e+01,     1.5483250972425532e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0102686891578261e+02,     6.5877907973298770e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3822226416077871e+01,     3.8807222327519346e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.1089038665642672e+01,    -2.0554756825577624e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1586959560087067e-01,     1.1499656302018273e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2523785161922163e+01,     1.0236156084892324e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1786950210283425e+01,     3.2370978991055063e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5579722141920360e+00,     6.6017547885367985e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3657827721207582e+01,     1.8464753343371405e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.5841162348996107e+00,     2.4905892180542040e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.4198699269688746e+01,     5.6611571806035154e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -3.1323243819037810e+01,     1.4967063486189367e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2933027119232221e+01,     8.3316343902550798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.1938669868817229e+01,     8.3257484589918186e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.5849886399294659e+00,    -3.8838090310567277e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.5585174129450188e+00,    -2.2130477831894941e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0195648817049122e+01,     1.6059728186560054e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.6685921825996846e+01,     1.6437828717893644e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.4857868942498762e+01,     1.4984728664882367e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    7.8261646203545778e+00,     8.3776434765917129e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.0178720002961740e+01,     6.9300743841912299e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.4448441052872980e+01,     1.3784276117382715e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0125079765452600e+00,     1.4134963721149412e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.1629543837094673e+01,     8.2637584020293143e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.0906663736622750e+00,     7.7163833478019042e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0108324116199917e+01,     8.3511259243306195e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.1140297082467274e+00,     8.3062569249162807e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6325420735433745e+01,     2.4968442247595878e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0674199096914869e+01,     9.0743929809551300e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.8683065989616043e+01,     1.2147567433086040e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.5415223804589075e+01,     6.6342066805935474e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.7170744851168749e-01,     7.7001601824201971e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0590324833028117e+01,     2.3183619552488286e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -2.5228497591092349e+01,     1.2393421733254181e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.8165539240853024e+00,     8.2818949247102919e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7535820414575269e+00,    -1.0280465689203524e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.3176656771017260e+01,     1.5630668276202379e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.5238780522308151e+00,     7.5864203187944386e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.2434407463647560e+01,    -8.8307448828423094e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.6568939968889854e+01,    -4.9366178133090079e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.7192726882772959e+01,     7.8665489770751519e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.3841686891183564e+01,     7.7955556716621613e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8279389648148829e+01,     8.8908757171445232e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7430256462966405e+01,     7.6961260839201859e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.1109242463098077e+01,     1.6381755850840051e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    6.3772220265068178e+00,     8.3254381688003687e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0319561848359424e+01,     8.3739809501139589e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1646862849206912e+01,     2.9209445980367502e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.6170736427826036e+00,     8.3319446804465315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.1765295019712276e+00,     8.3784723603026086e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.9572806332598311e+00,     6.6764254525231927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4408607834278563e+01,     3.8732923322330302e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.6325694483950217e+00,     2.3623539860834702e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.0467530137525826e+02,     1.4644473236971919e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9233625569460912e+01,     8.3448326239829494e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1852331629181403e+01,     7.7997630784011562e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9333637829216180e+01,     8.4713893809163073e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5996553485844229e+01,     1.8718048439383620e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0045560687396307e+01,     2.3982779403072531e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -6.5010155764265264e+01,    -2.0571522602131154e-01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2460513199096301e+01,     2.2118200289039109e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6448324320454347e+01,     7.1894702258185007e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.8526205247100620e+00,     1.5534986919082512e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.6434443937202463e+00,     2.4304797591613613e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -8.4795074944241122e+00,     2.4968025330414490e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -4.0727329160531859e+01,     1.1271154840361605e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.7276415929271884e+00,     3.1594371459315396e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.4073061189398921e+01,     9.1634352298239846e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.6801755244568007e+01,     2.7486186702854255e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8523354016865365e+00,     5.2555169724925408e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.0848756422247323e+00,    -9.6786065992411095e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.6062082907608684e+00,     3.6978817033821088e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9249391517033079e+01,     1.4190224542563103e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.9752531309131349e+01,     3.2892879074088094e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5641363839006218e+01,     2.7637660262688987e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.4984470498765319e+00,    -2.4950025857466757e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {   -1.8153021864538708e+01,    -8.8099453268013240e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.5771304729461267e+01,     5.9416999517528084e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.4455779824663164e+00,     7.3118340167470606e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.3250704511102356e+01,     1.6656319187821951e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.2844061846284632e+01,     7.1569749666947646e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.8957567133683028e+01,     1.6658446510671849e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3188110373717684e+01,     2.3227390386045252e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.4966766428849105e+00,     7.2062954944590610e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1661551791602713e+01,     2.1026998922227758e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.6028372693813147e+00,     1.1984785168553301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6686821330899701e+01,     1.7854479442266573e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2841759655712416e+01,     6.5868598720421196e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0098939451608221e+01,     1.6036553430770415e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.3353636409013863e+00,     7.2691742007913572e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.6185158595788458e+00,     1.6860373359384066e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    1.9015940267028213e+01,    -9.4307007454705672e+03,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {    2.1960800166494810e+01,     1.2252342165954191e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.6310279650075925e+00,     8.9750326936097044e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.0414049573308120e+00,    -5.9518577071413688e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.5275928814291710e+01,     2.5829991607471049e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0250922391881723e+00,     1.3531248775643711e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.3989984985862634e+01,     9.5950877772570675e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.1862710332879249e+00,    -3.3630134133913216e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1322830780924455e+00,    -1.2539853377749032e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3678165025191063e+01,     8.3149002368253095e+03,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -5.1530866779554472e+01,    -8.3424826124215906e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.9095192912717769e+00,    -8.3003579351661647e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.6857939693130472e+01,     2.6733926190923441e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    4.3495046359629992e+00,    -2.2020271564671302e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.2516291008253901e+00,     9.3526498116781640e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.4686568376639393e+01,     8.3512350647530820e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0720997735008888e+00,     7.6030478976972063e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.8404999272675333e+01,     8.4329406763352199e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.7728320049745001e+00,     1.6459274742158399e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.2004980086380588e+00,    -5.8190691917239565e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.2111672641754112e+01,     1.2887208706598867e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4235694679909049e+01,     8.4596574087270674e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7357890228995217e+01,     7.7956500186003796e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0587286265162597e+01,     8.5700383227948605e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    9.9122954514565123e+01,     3.0441658966629613e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.4525493438161362e+01,     9.1669007913633704e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.8755284607914462e+01,     3.2517907047850968e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6890004105276844e+01,     3.6916388775544547e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.0830616199053562e+00,    -1.6736943314900691e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.5153844163824740e+01,     8.2042398249578982e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -4.8473252845593828e+00,     1.3435662830690877e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5301494413362038e+00,     6.4875395030437840e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.5530637766147866e+00,    -7.4988786516805130e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.7928085071303883e+00,     1.7478519170530919e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -1.1278684034466561e+01,     1.1610139656231022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3659198795741972e+01,     3.6899920493197242e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.8059336062156383e+01,     1.5779912732892031e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.3090188269411641e+01,     2.0775427837006646e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6886313969231900e+01,     6.6965634876237045e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    7.0346359724723335e+00,    -2.6906804280771610e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7679299597251024e+01,     2.1666100363623327e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9075223806158199e+01,     1.9486081898109936e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.1844993291134870e+01,     1.1780017719713411e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7594358399297754e+01,     3.6885221920890413e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    5.0736230315563859e+01,     1.4897379423396604e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9575851140729010e+00,    -3.3330537070233086e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.9428909883796961e+00,     1.6259954637915511e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.5888096020816334e+01,     3.1204953285657324e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.2008674973170500e+01,     8.0289551999418873e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.2058227087915100e+01,     1.2679920678039463e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4666850281485351e+00,    -1.6848908106781404e+04,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    1.0793453839355264e+01,    -8.3552100412196542e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9930572038876644e+01,     3.8296941613005751e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.5612459033451863e+01,     4.7626523021889971e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4105730745871961e+01,    -7.3515154982874501e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.3130593005281446e+00,    -1.7772722753978884e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.2001525716894509e+01,     6.8144829400662238e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    5.9196223516674449e+00,    -1.7771300063045132e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.8959536495751980e+01,     2.2180767322038915e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.4263192470888963e+00,     3.3705942173559110e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7611655697038193e+00,     2.2909076886087976e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.7091010345447835e+00,     6.0420559817117883e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6869301451813435e+01,     7.2218302362576278e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3105549860082668e+01,     1.4931513986475898e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8660461021968018e+01,     8.3413296145192580e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3949351921194417e+01,    -8.3160532347276403e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.3181139173814799e+01,     1.8123048812294008e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.1412875642949551e+01,     1.6113973976363759e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.2220391710217708e+01,     3.2165192250219301e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -1.8916988379049887e+01,     1.6867260947357972e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.1711794659857919e+01,     1.6866603682608427e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.6586898608335268e+01,     7.0258433075998946e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    9.3988175610265792e+00,     6.1060457056453433e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.8006052820366946e+01,     8.3303040884498332e+03,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -4.9575569267550762e+01,    -8.3270787607970633e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {    4.3752314532970196e+01,     1.5938076459616193e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2694179496248291e+01,     8.3169512506130177e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1124663049064479e+01,    -8.3404315986338843e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -5.5758254322572922e+00,    -1.3625344130356484e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.2225511976979135e+01,     1.8851215357465262e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.9392183924604353e+01,     2.2144051598583028e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5675576297987124e+01,     2.0921094938798349e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.7808550933656768e+00,     8.7588909025804951e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0057846616079837e+01,    -8.3280614179837539e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.2827715600071732e+01,    -1.0883201686385473e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0219523522488196e+01,     5.6949386080586755e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.6984130913850841e+01,     3.4100807637879778e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.9224288267018665e+01,     1.5550521660881077e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7066170723833736e+01,     2.5263047618678494e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2243336165489097e+01,     6.0504190674190595e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5715906589553377e+01,     1.7090947826954623e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6426062589797038e+01,     2.3809088907575086e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.5098831160922979e+00,     1.3952077876418980e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5310638116283897e+01,     1.6082795572848681e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.4022616924833471e+01,     3.8302201277638116e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -2.1814929980673568e+01,    -1.6844390409850959e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.3967484045911654e+01,    -8.0289704594570358e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.6129848407072970e+01,     7.7270445461625404e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2070571251572888e+00,    -1.1326622432058150e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.6693511673665746e+01,     7.2447157877170430e+03,    -5.5832899822007905e-04,    -5.9151709988682344e-07,     2.7173322012508378e-09, } , 
      {    2.2554916474767342e+01,     1.0671069499266925e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4255492011807540e+01,     9.7917428148695271e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.7606725352207206e+00,     4.7011163887884544e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2210656117970508e+01,     3.8259756543424024e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.0334646142826106e+01,     2.3367091056444973e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.0993724321981322e+00,     2.0761059384516506e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    5.6935112650015345e+00,     1.3187719895966322e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3364137216005537e+01,    -8.3332338568079595e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.8041037405442609e+01,     7.7010884714940566e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2898917157242774e+01,    -5.1430540334639973e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.9106867796936733e+01,     8.3305967905248472e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7537351349752917e+01,    -8.3267860587220512e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    8.7105711590360713e-01,     2.9340271235711531e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.2927725005229046e+01,     1.4297159746613072e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0507787750320285e+01,     1.3805718199756375e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.1902890057192774e+00,     5.0010521631228648e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.7161135892159173e+01,     6.5362413688583401e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.1101449974188220e+01,    -1.0055960707392314e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.7903366045270083e+01,     1.3014930192100492e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -6.5604395280574550e+01,     1.5539695085281883e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.2681447784380538e+01,     1.5545813494681710e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7340297489964756e+00,     6.7308703962352465e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    8.4323566167101049e+01,     1.3642289506556735e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5163813836194270e+01,     1.0003814033715344e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1037684276286189e+01,     2.0413502021357908e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.0478052831026929e+01,     1.5537687217940107e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.1473404455432878e+00,    -9.1655382187037992e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    2.3208900290512023e+01,    -4.2745288008083053e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.4837594826263512e+01,     7.2142533210692118e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -4.2471651423231123e+01,     7.2138724096474780e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.5633674479810660e+00,     6.5665676105567709e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.4976451437901922e+01,     4.1379304853051974e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.7888557471703827e+01,     1.4986151355816119e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.7767687357432838e+00,     6.8084226572473699e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.9055253973394191e+00,     1.3618016632947252e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3529819032489200e+01,     1.3171862876583678e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0672802829917225e+02,     9.8778219678111127e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9062604973745522e+01,     1.6111543297955923e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -1.4569480030508046e+01,     1.8760625995960863e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0255943059729439e+01,     2.3846862550804315e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.6498184196380457e+01,     7.9997524887862446e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1495452227137664e+01,     3.4520309270061107e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4556870358476615e+01,     8.0004571207023937e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.6020344437793774e+01,     7.9063638789730630e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.2490581142086910e+01,     1.4920438359396940e+01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.0956153109478306e+00,    -2.5159575323203051e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {   -1.6030768847295874e+01,     8.8149971393278593e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.3965748256825574e+01,     2.6700144763468281e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.5581963496431737e+00,    -8.3367357024623561e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    3.4919685142957100e+00,     1.6603702406568835e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.5128377463163616e+01,     1.4831963411666535e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6688138526110801e+00,     9.6350875303936955e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.6084119592155659e+00,     6.6478884301852477e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.6135880372805559e+01,    -8.0312716593984478e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2823079160906943e+01,     1.4507686196370483e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.1420362832502118e+00,     3.1080851301473285e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.2482494205968617e+00,     1.5469157960628723e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.4086631349893526e+01,     1.2357116898822342e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.4559971607247650e+00,     8.3103480528821037e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8037867882989133e+01,     7.2171761109396539e+03,    -5.2969489971602004e-04,    -4.8641453588455959e-07,     2.1796738289003623e-09, } , 
      {    1.7874520955029283e+01,    -5.9241649405923781e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6391443578265424e+01,     2.3558625789718386e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.4929115157853474e+01,     2.3492502497193916e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.8000219362671778e+01,     1.3783818637447148e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7313816006188620e+00,    -1.7609698282513584e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.1366658504174580e+00,    -1.1216321407445405e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.1107584826001542e+01,     7.2083730372427608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2655549110604520e+01,    -8.2565536091340473e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {    3.3463607671310122e+01,     1.5537064461866212e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3392856751680227e+01,     1.7352754616422484e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9980753580901212e+00,     5.4868744645621464e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.6968381809032334e+00,     5.9831848448835126e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    8.7458111124145184e+00,     8.4631998688803742e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7996852573413065e+01,     5.6078444235071001e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0351440192340029e+01,     1.5929216304075650e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0037486694329193e+01,    -8.3226311418045952e+03,    -2.7584724895743558e-04,    -3.9757073197335323e-07,     1.9187955870953214e-09, } , 
      {    2.8148744702273220e+01,     1.4415007819518423e+04,    -8.0771915942080559e-04,    -8.1652232906005436e-07,     3.7198784124172471e-09, } , 
      {    2.0143650147992718e+00,     1.5452582790551684e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.1627255709720661e+01,     6.9924464507365392e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3717295778550369e+01,     8.5471192788602948e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.1398041545919540e+01,     2.2597557547748493e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.9247691529819799e+00,    -9.6741993576027089e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.2387400914000347e+01,     7.2067331158465022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.0718011238106140e+00,    -9.4559392223065406e+03,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {    5.4816248824641827e+00,    -1.2374275003350554e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.0553072722835175e+00,    -2.5131189389792791e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    2.8794804538877489e+01,     4.7349870459517500e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {   -1.2699887342953431e+01,     7.6799299593011347e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7557300949759263e+01,     4.7392078646882328e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    4.4236690169546144e+01,     2.2741084671340839e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.2072635639263467e+01,     1.4976266481949075e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.2307597292323905e+01,     8.8325474100034426e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -7.0835696565003431e+01,     1.6029081977752081e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.9025543502744409e+00,    -8.3080279083665555e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    3.4881772533342850e+00,     7.1566845409768939e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2426351598316856e+00,     6.5611721165118688e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6213422605809775e+01,     1.0185575830202415e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -7.1283795209972325e+01,    -8.3463167373471915e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.5712517558363839e+00,    -7.9422294105295969e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    5.4061032248561204e+01,     2.9948338115566239e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.6367934413329330e+01,    -1.8658474953799850e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.4894837848752381e+01,     4.7964337296441554e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.1979545201729870e+01,     1.4539587111034089e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.8016935260932371e+01,     1.7825629047069004e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0079875405232688e+01,     2.4990483501040606e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.2125317365887067e+00,    -6.6944743959790549e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3475454265225569e+01,     8.3130513447611811e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7267773881546994e+01,     9.7092595215998081e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.4568877408972227e+01,     8.3027819294749497e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.8596841982761354e+01,     2.7737221061721262e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.4766324296354497e+01,     3.0616460602509695e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.5611445828030075e+01,     9.0471037697115571e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1110375009879235e+01,     1.7725112069858580e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.0507957206834316e+01,     1.4903426078213888e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.7423493515662312e+00,     7.2197526934739280e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1810745892705199e+01,     3.8339386347219843e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.6959328558198603e+01,     2.3438093837596876e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1762831094358969e+01,     1.4376310619320819e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -1.3971017108921739e+01,    -3.3088781982054206e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0080661644304445e+02,     1.8540574635906290e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0587726520362242e+01,     2.2192499294228153e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.8750304920527974e+00,     8.2862734987854565e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -3.5088035874352386e+00,    -5.1223905172071045e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.7447391768111927e+01,    -3.4431910387842800e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.3751849807184181e+01,     1.3242111684111263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5549058881449245e+00,     1.6570632324580547e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.4469738984824239e+01,     8.3443315044857191e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0346178837407692e+01,     1.9271458799014275e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.3662878721250099e+01,    -3.7237734767748861e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6300084602231941e+01,     1.7790427798641133e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4452926581609985e+01,     1.0163284390603982e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.8410652226488473e+01,    -2.1721692198666306e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5470172605567280e+01,     2.2283614092318847e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.6895527850653949e-01,     9.3534170601708774e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.5904843198303231e+00,    -7.0190391439861583e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.6946890849099088e+00,    -8.0199059274272457e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0521680683327219e+02,     2.0385682489145009e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.3191291508480418e+00,     1.5584223923597347e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    6.3685433058578759e+01,     1.5515145312807861e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.5193299360294095e+00,     5.9367841790103648e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3077190166226602e+01,     1.5325501358569947e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.0889071985351874e+01,     2.3817765271927179e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.8492999782237289e+00,     7.2801861578828148e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2266116662905446e+00,     3.7668962660334983e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    2.8224493472706094e+01,     2.7879207672547243e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3874962226373547e+01,     7.8784188668074876e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    5.2008946375543044e+00,     6.6698664969240590e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1003588444474680e+01,     1.2494187736905880e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1996159629580916e+01,     1.6660364997521159e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0613317170674901e+02,     2.3267411003169182e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.3874204436312620e+00,     1.5548444118097377e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1427143308960027e+01,     2.4517777507852585e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0320061303379507e+01,     1.3136173489688881e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.6290793221027542e+01,     1.6654400700972641e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7540190434552070e+01,     3.1964850523876143e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.7017317455853505e+00,     9.4477292110588151e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2435307080021630e+01,     3.1260310215843667e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.1057636684634566e+01,    -1.7279798111725354e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1928918058141916e+01,    -9.6315395416470055e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.0099510276894224e+00,     8.3213616751116060e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5048405881899733e+01,     7.2805384738408902e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8238081250169724e+00,     1.0345611384484257e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.6493359817866668e+01,     2.0491511620406334e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    3.4034124307066564e+00,     8.0688405969477826e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.0537999450979541e+01,    -8.5797257834904558e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.9829440748279126e+00,     8.3360211741352941e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6092297646988964e+01,     1.6077761870500763e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1258429405389590e+01,     1.4304163328092509e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.0092761341240904e+02,     8.3237671059165805e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8629789588808248e+02,     1.6171569833666455e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9747280221032817e+01,     1.7791850489574885e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3252240328316185e+01,     1.2778489590932615e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.7467190344821506e+00,    -9.6404179301475488e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.5544124649661288e+00,     1.7807348479448727e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.6942452631234360e+01,     1.0530402426466140e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7744240420831023e+01,     2.2397577613102309e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    3.9333519570850521e+00,    -9.6179245606154655e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -1.7371207287238224e+01,     8.6598926173036125e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2101862362080606e+01,     1.5125611748314142e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9737647426199771e+00,     7.2213926148701867e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4986625864154334e+01,     2.4889171473308859e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.2804046691246441e+01,     1.1667572890863488e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4712691826070291e+01,     3.8874909933156283e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {   -3.1414588740298026e+01,     1.7285338118334370e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.0229260769735014e+00,    -4.0765757282480035e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.2751636414343746e+00,    -1.6744133373913253e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.3620305684196556e+01,     8.8385207063763191e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3403826505701804e+01,     1.3280229676291856e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8432429764124585e-01,     1.3199720472417721e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    4.1755782045273307e+01,     7.1499808315025621e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6697707250932094e+01,     2.6223012577164038e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.1045467648628744e+00,     8.4837959466476987e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1135567193821808e+01,     8.3114166928603936e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2134208449295500e+01,     1.6632799685445967e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6272895620721048e+01,     2.2707153757905002e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.1501142416075112e+01,     1.8859093851089587e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.7720131724827212e+01,     1.4914240989710232e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.7426885151254954e+00,     6.7772710621083934e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.3820639790694734e+01,     6.5861434221063373e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3622536488453143e+00,    -1.1352471217760813e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3301091262928788e+00,     8.0404451877986139e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -8.9283279724038422e+00,     3.1068235583197922e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -4.3318063420778870e+01,    -8.3322608509645961e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -1.5361583747200712e+01,     8.7812157118726973e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1201683858718612e-01,     7.3242855362243681e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.2313159384948964e+00,     8.3958714382233047e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.0800000739472125e+01,     1.6441035342991669e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2146019373397959e+01,    -3.7894321741132944e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -7.6745471147259863e+01,     1.0189341306150598e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7878321985604021e+01,     4.6592214453541790e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    1.0648942272245421e+01,     8.3790661675254651e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.2914642565134098e+01,    -8.2922497619574078e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3906644230105449e+01,     8.3247623013405027e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4805594080472201e+01,     6.0968044737770570e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -4.7103098918998114e+01,     8.3906959156955309e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9060427858326349e+01,     1.4372768800530348e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0585042973564352e+01,     3.3312135866177618e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    3.2418122845166735e+01,     2.1471850262831769e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.3633685446774262e+00,     8.8834113499468258e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9027327520940222e+01,     1.5456013241054356e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.0492263416441695e+01,     6.6082971854450479e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.4542454017306618e+01,     3.3317395530809983e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -2.8883180651837087e+01,     8.4703257194913549e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5835689813136604e+01,     2.2291439598433499e+01,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.6841556509853771e+01,    -7.8454989555003522e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -6.9721051724422153e+01,     3.1570799535161459e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8771406519450441e+01,     1.4898698961421800e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -8.9971552685689147e+01,     7.7898835318585952e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.1903588212251393e+01,     3.2828797040232312e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.9725859206457450e+00,     7.8708232359301633e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0945843224342482e+01,     1.5115302467058653e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.9919802155277523e+01,    -6.8539548370912371e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.5974198314830700e+01,     1.8128614557115980e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.2901088718347200e+00,     1.4887785998351617e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3092917651137721e+01,     8.5070892952207760e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0585588143196269e+01,     3.7882078265887430e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.4240992255508953e+00,    -8.3312944485635326e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.4742562856013526e+01,     1.7808771170382479e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.1268866644309298e+01,    -8.4255942251849410e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.1000966967997243e+01,     1.6267964675588832e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.4734588602933147e+00,     8.6617979832050114e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5621925619356105e+00,     5.4983566831107773e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8853483737726240e+01,     1.4516039716937274e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    9.3986140074419673e+00,     6.7926559622550985e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0891117997385928e+01,     6.1067555877107338e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.9299480094541153e+01,     7.1492761995864121e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8933006180822353e+01,     6.1020641384094251e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.0944906648343153e+01,     6.0317467004563005e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.9457814637080148e+01,     1.4523297562800943e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.2436959678327320e+00,    -7.3177340708322909e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.2118813851696064e+00,    -8.5558729166342200e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.8793301366462423e+01,     1.5588602666322460e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    6.9550698458217930e-01,    -1.1352920755219984e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.0783543918665970e+01,     4.9397219365636447e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3675243117154707e+01,     6.4434400371120482e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3409331982728759e+01,     2.4245321859974887e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.4460418737275425e+01,    -1.6686233244444462e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    1.2164794169025594e+01,     1.6711063291924966e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.4851576719070589e+00,    -8.9591412478908151e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.5716607044134932e+01,     7.0065909424057436e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -8.5968996766761787e+00,     4.6609908149021576e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    9.0616885331511394e+00,    -8.1517788762658984e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -6.0716083715934005e+01,     1.1819832621611324e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6601213860323654e+01,     1.5754630678826559e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1336229227228168e+01,     7.0870325876704765e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.7512893182704630e+01,     2.2345260268967868e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2418561313733463e+00,     9.0092567614133077e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.8275724935160440e+00,     8.9150505391801653e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0748580502162731e+01,     7.8466282060616104e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.2369110053156014e+01,     9.0342845510945408e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4766243573598622e+01,     3.9349765654773175e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.1190468931863396e+01,     1.5629495338909237e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.7415022149226374e+01,     1.5609229898464340e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3685659449789393e+01,     2.5301542982551211e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -6.2359812320192134e+00,     2.4522295204783033e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.9487298337096647e+01,     7.2268647447135490e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7924998157285629e+01,     2.2852377129217930e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.3219029728651535e+01,     1.4913350824068764e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4417232046904891e-01,     6.3642082038330282e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.3308527679370563e+00,    -8.2930362544822910e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9424558473788220e+00,     1.2703027768786727e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.7439671287791711e+01,     1.5539772141707535e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2837583468720566e+00,     5.9640066508663913e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.4003575568983280e+00,    -3.7260746767162955e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.7575117782071246e+01,     1.6504579527164064e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4848773187154961e+01,    -2.4021764761014856e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1616432152020852e+01,     1.1553041275495660e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1542115410752814e+01,     2.3880082212988076e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.0264252453558314e+01,     5.9428539375681285e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    3.9299325585391777e+01,     2.2756264865807658e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.6401319780806940e+00,     2.2803716493993568e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    9.1147541329091837e+00,     1.2592450024072705e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9095545012234957e+01,     1.4146453709006135e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1407139345119908e+00,    -1.0657275338023937e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.6621665839686884e+01,     8.3167464507183249e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0634722288478821e+01,     3.2231315542743770e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    1.0747268998230012e+01,    -8.5539886338742017e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.5761166357201049e+00,     1.5550442496142188e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {    2.8653169139526113e+01,     1.5535066083821401e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    9.9219510517909875e+00,     5.1376243284261236e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -1.8218086707172557e+01,     1.5056080999804157e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.7815555879600822e+01,     1.6031152806985056e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4758546121217929e+01,     1.6798658114606005e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4509411080105060e+01,     1.2403517564695489e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3639410298568428e+00,     1.4914298287879039e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9680177342716295e+01,    -1.3506511271354015e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2317833645457441e+01,     8.4359302055932076e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.4858578078064966e+01,     3.7688184541083981e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.8602622754532838e+00,     2.2757369444872624e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.4708372068099621e+01,     7.9076321053727988e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.6416236073745299e+01,     1.6656254586598629e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.2121639582472099e+01,     1.6658511111895165e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6003254004678737e+01,     7.1273123406919940e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    4.3496292649661811e+01,     1.5591205690262541e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -4.7667671708080704e+01,     6.5853671385192665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6762587900832386e+00,    -8.3210289416301093e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5285771616073378e+01,     2.9120403731498653e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5474616080068085e+01,     2.3254158756282064e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.7697448758226280e+00,    -1.6266886686327782e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8043921103250629e+01,     2.3921109112040383e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.4528891493016124e+01,     1.0072731096328305e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.6399725475017263e+01,     4.1772325822744962e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.7751546678997101e+01,    -8.5075879680010530e+03,    -5.9548675818625800e-05,    -1.4150664154036154e-07,     7.1223977891801919e-10, } , 
      {    4.5387220462515245e+01,     1.6801670660014293e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4719227107152996e+01,     1.5718537587778088e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5684837281879027e+01,     8.1858775086424157e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.1850971835476422e+01,     7.2012609860031398e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1359971692490625e+00,     1.5539900800507901e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9941055334556946e+01,     1.5545607779455688e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7085112632362122e+01,     1.8544935436699714e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.5926155268600922e+01,     1.9153340192663403e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1534310079209408e+01,     2.3896028878406174e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2826259009942238e+01,     7.6879323201005982e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.2209153697276172e+00,     6.0840044522006156e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4005029574210823e+00,    -2.2116250922557419e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9626057600557456e+01,     1.8260539437519983e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.2112010862804539e+00,     1.6246312936098049e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6097620359478775e+00,    -8.7282713499155725e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.0263230172533225e+01,     7.5869726083269215e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.1930518878789986e+01,     1.1596812109052622e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.1981251248000337e+01,     1.5282289375863140e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.4426495275681397e+00,     1.3660461367161348e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0966146055725083e+01,    -5.8869151296420654e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.4922912834978725e+01,     1.7052705018881297e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0671346234963650e+01,     1.4583245512256090e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.7456234289698145e+01,     6.5032719870430847e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0676348660342587e+01,     2.8117124120215984e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.0119370915752672e+01,     3.3386862531300998e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1876622342113953e+01,     3.1098626491161856e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.5303638276608808e+01,     1.6889474220115448e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.6478168481955855e-01,     5.5471125569247056e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    8.6819421304680464e+00,     1.3189142586900072e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5913959774785795e+01,     2.9045152670427942e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9503973484821561e+01,     8.3478191602194565e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -4.8052980949574362e+01,     2.0290510657060757e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.8912162497098365e+01,     7.8268284221154581e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9422391313284511e+00,    -9.1278512752076895e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.3616851961115177e+01,     1.1806361574788923e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    8.5668131625055892e+00,     1.5279081741218981e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.0809314019021805e+00,     1.8040565519024291e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.8025554398529906e+01,     7.0720695409816362e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.1427602920695143e+00,     1.4306757682401683e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4665331093509373e+00,    -8.5459443560352938e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {   -3.8256034580657314e+01,     2.3672371205115684e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2204460727503976e+01,     7.5786979016410733e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2114869451766751e+01,     1.2172160072625884e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.4827991578664982e+01,     1.7745775586115469e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5257415934347156e+00,     1.2279894106995066e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.8005352828885684e+01,     1.6878929925726672e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.1513883059696226e+01,     1.3976350750028463e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2909578499549854e+01,     1.5699334658506907e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -5.5562848198558310e+00,    -4.6518172865019924e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2118811735626537e+01,     7.2120958913068716e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0721879315857259e+00,     7.1669938222323826e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4772693179970560e+01,     3.4780431661540912e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.4557047863440623e+01,     1.5361919179658964e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -7.0865400893758590e+00,     1.8564847509515774e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3690520022557386e+01,    -5.1200893172656906e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4720295987245233e+01,     1.4774727083058709e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.6511473736488071e+01,     7.2939163263118789e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4208634862381373e+01,     1.5529952410626591e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2941161603586856e+01,     1.6672101357672647e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3080772762242304e+01,     2.4691827932296532e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.6794706174320684e+01,     8.7857944281722193e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6713372174402288e+00,    -2.9355615904681795e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0525893148187700e+02,     1.8834820977480107e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.6456911877624819e+01,     1.6458308339757339e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1384910057199868e+01,     2.3214272474846985e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.5645294806191441e+01,     2.4446072856972722e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6271888119380840e+01,     7.2287813737840934e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    7.9749503494938425e+00,    -1.0381873158542408e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    9.3038663424672521e+00,    -1.0940099967541303e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.4931449202933123e+01,     2.6440032984973761e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.0543741570513689e+01,     8.7864990600883702e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0944498785844054e+01,     3.2035843829289130e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -2.7666256588517932e+01,     7.7541041482252294e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -5.3005505859261492e-01,     1.5087947754268955e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.7191680493834166e+01,     5.7321504860165051e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    8.6368136450262423e-01,     7.1672663654451844e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3608573291672998e+00,     4.1784631345530106e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1766396827739511e+01,     1.9612649397796238e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.5812483507976443e+00,     1.6681966013047830e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5052343158871516e+01,     1.7291723908735195e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -1.0997335704966385e+02,     7.6986528711006686e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6549424218955728e+00,     1.6127690713604758e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3304838310511109e+01,     2.0094712616358083e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    8.6665942774749407e+00,     6.7784439994015365e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.1191609899848476e+02,     2.0051621245673279e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.8544053781096380e+01,    -8.3807122512453407e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.5429595136364242e+01,     2.3870382053180296e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.7504925442843458e+01,     1.3664831411688980e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9554826284823715e+01,     1.6665009432873172e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.8496036575347219e+00,     1.6649756265620628e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.2535750447575404e+00,     1.3042641860997890e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.0199140915313851e+00,    -8.3870654916184540e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -1.0440156573713100e+01,     7.0024358568383432e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    7.0916116853770372e+00,     6.0596194041953286e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -9.9975254197259261e+01,     1.1762744722947744e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9325968009026646e+01,    -1.0286244106712087e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -4.9351098985172559e+00,    -5.5154114869000914e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.6811772370103540e+01,     1.4427766701207887e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.9045142680280946e+01,     3.7842116125815082e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    5.7219138108564302e+01,     2.3872509376030193e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.8580908495270467e+00,     2.8710172674842579e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.0735095907494433e+01,     2.3925126157283310e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1856855314268955e+01,     1.3173811760567014e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.2000539519617545e+01,     1.4940183854552504e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    8.6039098180973177e+00,     1.8735553668675323e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.7249758113427216e+00,     6.4564068593826287e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3318497089794725e+00,    -1.0028967752978393e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0447436308062893e+02,     1.0506129552856655e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2630595199578718e+01,     2.3382494908069501e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.6189836922574763e+00,     6.6387149017850816e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.8859551149861801e+01,     8.1850681922584499e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.4481258334635267e+00,     4.7319841203151491e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.2855327038154286e+01,     7.4519858743447558e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.3289511902909112e+01,     7.2070629885145299e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2647850146686618e+01,     9.8491073364951490e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2412996275477752e+00,     7.4831309081660620e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.6164690798531268e+01,     8.3243126818983383e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.0403186781343592e+01,     7.2109496197770350e+03,     9.2743731943700382e-05,     1.0921223261429318e-07,    -5.2374421970263164e-10, } , 
      {    1.2590061119119525e+01,     2.5554863281844482e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.7586844913131703e+00,     1.2143692460186876e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6539570556175991e+00,    -7.9460416558778334e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1436434090196332e+01,     3.2792396307870644e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.6331313024848921e+01,     1.3672787867983279e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3755597424594285e+01,    -7.7488352398586539e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -1.8365979091559986e+01,     1.4428484760225490e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -8.5363267918097243e+00,     8.8435877784180084e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.6389028546173122e+01,    -7.8137950708288927e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.3639302445095371e+01,     7.7625169894503533e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9513574247501992e+01,     1.4006753038606415e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6300495478298437e+01,     1.2804226740548869e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    6.0084495826919152e+01,     7.2542300832143310e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1683103342890444e+00,     7.9115488829557953e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6892734751892746e+00,     3.7544263991335811e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4354940571504997e-01,    -2.5224191280974901e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.2534360223768373e+02,     8.3286493988568272e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3633817026653477e+02,     8.3287334503900747e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7622972436570112e+01,     9.7925548605094282e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.9359034542776577e+01,     1.6033991036943797e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -9.7530470248855039e+00,     7.2160298394098154e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3923099434683621e-01,     3.2182088195772831e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.1440935084930194e+01,     1.6285025395810806e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.7141461379511744e+00,    -1.1033437546601215e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2333594094054519e+01,     7.6975389255201153e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.2225102120135247e+00,     8.3256000004810012e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1848107366942457e+01,     1.5542459027300794e+04,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -3.4771054863136442e+01,     1.5543049552662795e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    3.6698612919434815e+01,     9.9832585966651204e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7717439981127052e+00,     8.3317828487658990e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6817170544422400e+00,     6.9831834916441912e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.0987718778104307e+00,     5.9666839581220358e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9994522856466077e+01,     2.3314138148523416e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.7468513142251240e+01,     2.3314124721421969e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.0069371893260950e+01,     8.4486367820047035e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0000638268005190e+01,     1.6728023838701818e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.1677921886895668e+01,     2.2714157339384437e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.0116245541560911e+00,     1.6158442636609350e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.0792145584049024e+01,     3.2212756377646681e+04,     3.0023079402995232e-04,     6.3636362590502636e-07,    -3.2291500043981749e-09, } , 
      {   -1.3208322920906409e+02,     8.3320275281879003e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.3365274565624790e+02,    -8.3253553210589998e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -7.8952028460660188e+01,     1.1111895232769955e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3005155310545629e+00,     7.7989992889813075e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7882366938322338e+00,    -8.3452940062266061e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.2836338554189624e+00,     8.9567753525112839e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6456613926652032e+01,     1.6676118402915574e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.3386744856848107e+00,     9.1658371299384180e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -6.7392499593380222e+01,     2.0210880853264931e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.7905832984420460e+00,    -6.3014534262860670e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9818050603625941e+00,     7.9012396017003084e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9292977044499693e+01,     3.0456872362543272e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -8.0762572008275608e+00,     7.2324251816738260e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    3.9664045672413081e+01,     1.0689118442722785e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5507621508337493e+01,     9.2426088528599739e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5509242613098380e+01,     3.2428864798982118e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    3.1284896705501519e-01,    -1.8733821386179123e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.7950444093484084e-01,    -1.7870227185781023e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.6378079511574903e+01,     3.1103532788165387e+04,    -3.1775034807191027e-04,    -9.5424567855850918e-08,     2.1593601356618716e-10, } , 
      {    9.0668388914219165e+00,     1.5539056408278411e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.7081443825985669e+00,     3.2187517900810708e+04,     1.7657248477168515e-04,     4.9074731575263549e-07,    -2.5308243781279995e-09, } , 
      {   -9.6832756386780119e+00,    -7.4797338273571650e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.2331798226775792e+00,     1.5549365689533899e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8230154620590426e+01,     1.4854928127864836e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -2.1756487954551055e+01,     8.5539733743590477e+03,    -5.2969489971602004e-04,    -4.8641453588455959e-07,     2.1796738289003623e-09, } , 
      {    4.6373988540538448e+01,     2.5603565147659036e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6298866269923138e+01,     8.3290183220008348e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.3045718619699418e+00,     8.3283645272460653e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1382602348280187e+01,     1.5702789844263638e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9200785245781361e+01,     2.7726404350464407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1500838145167528e+01,    -1.6766415337772771e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    9.7511619120795299e+00,     1.0109573537911606e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.5808073361770550e+01,     7.9728823328882927e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -7.2104228275599695e+01,     4.5177498068822570e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.4669457711396543e+00,    -1.6656279321852693e+04,    -8.6073680299738134e-04,    -1.1604177869939913e-06,     5.5678912020705760e-09, } , 
      {    2.0028222219581803e+01,     2.3444643317457096e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {   -9.5301666830998727e+01,     8.3278820275827347e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0629596123895192e+02,     8.3295008216641654e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9384034845139922e+01,     1.3290955266319339e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6036412486021170e+01,    -2.5011633912966690e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.8564787303848966e+01,     2.5114942973498066e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4081180923614935e+01,    -9.4744983874036352e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -6.5017087682150301e+00,     1.1804060374847513e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    5.4631061707091405e+01,     3.2025335503348615e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.7923603925887583e+01,     1.7021538164227168e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.6871396873802613e+00,     6.0415589118785156e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.8903856103825483e+00,     7.3975009996212575e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.7122767004490571e+00,     7.9072845215202033e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.5784659762394348e+01,     5.6406594485611986e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7200791522261568e+01,     8.4756090677070279e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4933553421089156e+01,     8.3241489924389425e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5069698258689414e+01,     4.7778939845614084e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.8536634605619846e+00,     1.4292698075489843e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.7032654907647007e+01,     5.1556732718819840e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.6700875671928209e+01,     1.1274784444772624e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.4345356575716423e+01,     5.5486789820598215e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -6.1913594518940052e+00,    -4.5922987094305790e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.3012593288302341e+01,     2.9609025905659626e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.3660011478650720e+00,    -5.9084324565930792e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8280495852983286e+01,     2.7126947160616430e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.2420968736004946e+01,     6.6650327681505514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0185640789723823e+02,     1.4912715736459011e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0463415791520157e+01,     1.4414619219445334e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.3079147431199498e+00,     2.4097915861149140e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.0990934832868713e+00,     6.4650985774646724e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8992386669872054e-01,     1.3741942751456423e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1080492892507934e+00,     5.5437632093173770e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.5777672229363048e+01,    -9.6684114779132869e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.8310194288822079e+00,    -1.7719042311300815e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    8.8734498390982033e+00,     2.4358478034291682e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.7451037047141497e+01,     1.4879501815926860e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.6952076271250828e+01,    -5.0670720416894710e+00,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -3.7132911541972589e+01,     1.5555556169337000e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.5395994647145088e+01,     7.8583622415995342e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4708163235414208e+01,    -7.3385144650238972e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.7923942354625225e+01,     1.9618079102834115e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.4378874175675051e+01,     1.5570751408949263e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    3.0255859182597906e+01,     2.3378565784786548e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    8.9470915675114266e+00,    -7.8738848889106102e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.4011036235328104e+01,     1.0311829957029098e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6249216274548395e+01,     1.4722930923140661e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.6331526289421205e+01,     5.1894546993371441e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0113037728866929e+02,     3.8083933420673930e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.7063652618096892e+01,     3.1567869959618874e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.9622744501939877e-01,     1.3138227805950723e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.6204524509823184e+00,     1.6648194061331586e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1674630069996429e+01,     1.6963662770012448e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.4632321963060377e+01,    -3.8815078311153175e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.8461244038742040e+00,     2.3571724749438828e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -9.7948272912664791e-01,    -2.9154400977904775e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.6296739400781590e+01,     8.5503078392452553e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8032531774510154e+01,     2.1911271898397907e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9200912051788251e+01,     7.7708248554584134e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.4969317977863696e+01,     8.9303383030843597e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9866633848119175e+00,     9.4820171840861585e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.0395170965187118e+01,    -1.6627540509823411e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0700365081283078e+01,     1.7223870657279622e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    4.7235777015002675e+01,     2.5445830793318608e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0717982935629689e+00,     1.5162851231487075e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1763082578566516e+01,     2.2537278283502037e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3948189276296308e+01,     8.2464585968875435e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    6.2181341653512739e+01,     1.6856457358736461e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -5.8626001889512658e+01,     8.2979119165864286e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2245683183334037e-01,     4.8624960027995539e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8397618795321307e+01,     2.2208070035948389e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2548444523815530e+01,     2.6307123793775296e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.4068610519436476e+01,     2.0425497391042918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9152828233211508e+01,     1.4923083203319085e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1777765233632188e+01,     6.0847157976674916e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2170015985427103e+01,     3.1968247604685035e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.5239600601770462e+00,     1.5890729163357412e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.8422588386033292e+00,    -3.0459519802748819e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -2.4656636621577917e+01,     1.6448035219835489e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.3134971699244517e+01,     6.0941653977266633e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8666037981538452e+01,     8.3612543830978211e+01,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {   -1.0667626568688847e+02,     1.1428683479476016e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0781792445432153e+01,    -8.6402107629629318e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.2522094192782832e+01,     7.7719294345233811e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7427926595097404e+01,     2.2648258136313502e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9646098706940535e+01,     2.8272228642240232e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5064781543204802e+01,     7.9951737724867253e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6652594600709698e+01,     3.1569199721866968e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0106345178410976e+01,     9.8538759647419720e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    5.1362317393485910e+01,     2.6705075795725876e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6098169555585581e+01,     5.8369157431457461e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    4.0740534273356666e+01,    -1.0161732328976665e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.8756833818488630e+01,     1.9046161589763520e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.2517556107265507e+01,     1.4518634071246450e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2522549857769183e+01,    -1.4333038458544135e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.7149386645887652e+01,     7.7014884313072253e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.2271322477485917e+01,     8.3179250622853215e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9528932889522448e+01,     8.9306906190424343e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0815133551477256e+00,    -1.6828235114785602e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -1.6734562798582520e-01,    -1.0562864315896106e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.5104563592309514e+01,     3.2969710796267373e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.4735962673232557e+00,     6.0169662723386728e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.1911897738320461e+00,     1.5608877582506264e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7824147083269697e+00,    -2.2135093750157603e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.9339576452390972e+01,     2.3870633668965347e+04,     2.4503613970602655e-04,     3.5993110676894790e-07,    -1.7597282183232826e-09, } , 
      {   -4.7551414847811358e+01,     2.3872257760245142e+04,    -7.2817676429041446e-05,     2.6574208657940457e-07,    -1.5282781669615902e-09, } , 
      {    1.6352996830192925e+01,     6.6504563497132331e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3265958837783934e+01,     8.3394577869615787e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4445175127981399e+01,    -7.1370092771265008e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.1969527243461798e+01,     1.6035108738644110e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    8.1386826387497067e+00,     3.2528723759107823e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    6.7708610662511326e-01,     2.0368038414823521e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.6375387227951723e+01,     1.5220726625701798e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.3068416339251897e+01,     2.9602466950060490e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.1338521930432428e+01,     1.0824648768039955e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.2967793331921555e+01,     2.9424634157381552e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2200507858703409e+00,     1.5495957790068636e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9391629961813532e+01,     1.5550921333015462e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    7.6854225419924678e+00,     1.5534587246948129e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2515339221500517e+00,     8.9286130762587345e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9204826633197536e+00,     2.9710142920637900e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5660773933649041e+00,    -1.7824980505723201e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.7351761049888662e+01,     3.1256360845502291e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.6353744927195930e+01,     1.6142675941617079e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2295576921952076e+00,     7.2134328587186483e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.5822765083945150e+01,     2.3762413226079370e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.8105499490145966e+00,     1.6861502609945332e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.3203671106044375e+01,     2.0745326900517204e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2056331985426313e+01,     7.2108096095352630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5567882651681415e+01,     2.4970326530355898e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.6500023473244752e+01,     9.1661961594472195e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    7.9749875677293343e+00,    -1.7597968909582160e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.6521078410742149e+01,     6.2689670336067420e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2326335716271517e+01,     1.2020833584956898e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.3885673734331778e-01,     5.5709856811734044e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.8465444498901515e+00,     2.9775732476629237e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1758961321709020e+01,     2.2379543929161602e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1245617558496674e+00,    -1.8165032378204996e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    3.2397133671622299e+01,     2.6451740236917460e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.9531556201629328e+01,     1.3119721002774286e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5076439399184132e+00,     5.9885992904062359e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.9021576706485419e+00,    -9.5235020772168627e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.2977945007741258e+00,     7.0713649090654862e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.6760178897927871e-01,     7.3319460336858519e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.6631656137875233e+00,     1.1687011838705025e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9526239907556572e+01,     4.0515710652653877e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.0170578193757027e+01,     6.4448979200480808e+03,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {    4.1132943322247783e+01,     6.4495003199309031e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    3.4471001300543755e+01,     1.5540787315930322e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5138958748955375e+01,     1.6075890042943083e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9860287322861986e+01,     8.2319943392879777e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4045168468279087e+01,     5.9577763276420310e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1279667423045254e+01,     6.5864599122289510e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3371972736805740e+00,     2.5280320615444165e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0309656718884847e+01,     7.2173161211814258e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3309489999784940e+00,    -8.2940656991536034e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8637734735981681e+01,     1.0837992641522675e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5013494870078475e+01,     7.2210664468377809e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -3.2431231794173691e+01,     8.9573580391777941e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4942034326439254e+01,     1.2772786863479805e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    5.2743263869433417e+00,     2.1460387083351143e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.5432875163160398e+01,     2.3654192783193394e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    9.3880206413940748e-01,    -1.1408424832353492e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.5023185841190081e+01,     4.1314870503238119e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -1.0859837183703274e+01,     2.9342220119694870e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.5895122598596132e+01,     1.4304188063346568e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.3165049477998423e+00,     7.0754733353412603e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4130535057465536e+01,     1.0411190244374528e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.6970519284620913e+00,     2.5125759684754921e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.1505681468067422e+01,     1.0091466649996979e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4803396932584000e+01,     1.6652315777205211e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.8766004733187714e+01,     7.2294927192509695e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -9.5110303957897770e+01,     1.1802559624845653e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1810870323978500e+01,     6.5651367178018263e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.9922901878035873e+01,    -9.6373274213364275e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    4.8912602825302670e+01,     1.4441632241988043e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.8121369695466328e+01,     2.6616033546857027e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.4781977308453502e+01,     1.5535424540469950e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8306572562209489e+00,     9.1776490218897361e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.2971818585627759e+01,    -1.7369026698678963e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -7.3941272833841580e+00,     1.5544721264033267e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.8390697810932117e+00,    -4.1190956464092824e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.2680135845844935e+00,    -1.0299316834275917e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.1101361768744322e+00,    -7.2136706834310162e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.1954645001795019e+01,     1.6959733646729495e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0298262968327997e+01,     1.7892668111360133e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0977096703610492e+01,     2.6058719267224471e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.7068570200616247e+00,     1.7473909992050892e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.1024371311753256e+01,     1.4994428824690151e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0427712074466658e+01,     7.7080987665242183e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.9025395238870445e+01,     8.1060240476888812e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    8.3661148157638543e-01,     1.5527797956843680e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1488083434432870e+00,    -8.2720244457088738e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8513500629419475e+01,     2.4447495547906470e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7543451105173489e+01,     1.7820698014811409e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5453742476793858e+01,     1.6672812703139523e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1192227643863167e+02,     8.3336157433303197e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.5635552356949660e+01,     8.0884737770132997e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.5991151192368950e+01,     1.4830416299030985e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.5784057924815489e+00,    -1.8374581843941298e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    2.5017702212070141e+01,     2.4103335249148899e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.4780394150012757e+01,    -9.5601204467984644e+01,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0464219944040529e+00,    -2.1566206302605297e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4310257350505680e+01,     2.3818486093132964e+04,    -1.3018934150031724e-04,     5.6772506241184548e-08,    -4.3744738446513437e-10, } , 
      {   -9.0813721039248954e+00,    -1.4719265870614527e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -2.0359034887561442e+01,     1.9447586534237220e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.2906834853744385e+01,     6.3866807708232409e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0751660672628098e+00,    -9.1887879153173344e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.8708622039952722e+01,     4.8600921327678207e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3015201558781072e+01,     7.2088741567399939e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    3.4470829848759434e+00,    -1.0488058765134876e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.0508976722473120e+00,     7.3472310218438079e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3358608570347123e+01,     9.3031206407085556e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.7210314256362544e+01,     1.6023175210116127e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6941035508495290e+01,     1.5238775569157659e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.6418239992337424e+00,     1.7278212267723386e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.2207652299829135e+01,     3.0322214260935878e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9198422241715326e+02,     8.3413376612976135e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.9041470596996945e+02,    -8.3160451879492866e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.3756881607315311e+01,     6.1209824970482623e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.9046761235596193e+01,     1.5885452031836357e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.2062428162778032e+00,     6.9503903165955298e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4907594510054704e+01,     8.9572226668267031e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.6618636441542751e+00,     1.5338619269768216e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.9079427138542371e+01,     1.4102020842491856e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.4331898626390611e+00,     9.3871984876260376e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.5384999755873341e+01,    -7.6359494897640488e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.4418646426172552e+01,     8.5228217792200758e+03,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.2864442049126851e+01,     1.6845770125946030e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    8.9150640677528912e+01,     1.3348043164982919e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1628997180735595e+01,     8.4032541563775849e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.6925154068650212e+01,    -1.7212395928938044e+04,    -6.4226121911122155e-04,    -9.7181663535885805e-07,     4.7399263974717103e-09, } , 
      {   -2.9485859068812776e+00,    -1.5560536209212956e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2290501288922691e+01,     1.5550084039493639e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6408051505097283e+01,     7.2179507580510981e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.8773994908129687e+01,     7.2101749726655944e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.7963790693980721e+00,     3.8814049923750472e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5685370700157002e+01,    -6.4635652850140468e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.6268197111736100e+00,     4.0856428513288956e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7120994558292743e+01,     2.1314115908491338e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.7414719938876587e+01,     1.0900206602101098e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4721410167736856e+01,     7.2758826423711689e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.1690875081991088e+00,     6.0481178674776493e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.9775278404269887e+00,     1.5892004426117819e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.8702797962525715e+00,     8.4253885099589224e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0854406004423385e+01,     1.5398466479214401e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.6048229385000056e+01,    -6.9178432551908429e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.9053404299850058e+01,     6.8187406957239455e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3989062706675007e+01,     8.4672809546405351e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4500162830716564e+01,    -8.3406363985285734e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9042482403083444e+01,     2.2945718364334698e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.3566924019453495e+01,     4.5477585894276686e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.6727387089712970e+01,     1.5545357313921877e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.4750742522749007e+01,     1.3673250645984845e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1670178555507441e+01,     4.0686562918192576e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.1222291042769134e+01,     2.2376914096845419e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6709752409306123e+01,     1.5805822228040533e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.7990947090572142e+01,    -4.5435968436160201e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.6617528272534095e+01,     7.3723560705489308e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.9196009774177867e+01,     1.4476577130997437e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.3588130448823222e+01,     7.0080828259631871e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8219090546508522e+01,     1.2178719028225018e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9168225330133509e+01,     2.5477311020832356e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.3643169409655176e+01,     3.2080692433534958e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.8195868981559443e+01,     1.4357843770770576e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.8479571644789509e+01,     2.3255757367977691e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {    3.3513766559820581e+00,     9.5826767623983542e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.9348427748226005e+00,    -1.6658546310801128e+04,    -5.2088338866346218e-04,    -7.5750183874230108e-07,     3.6785238054186036e-09, } , 
      {    1.5239760936842103e+01,     9.0357072420282948e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2028877669930196e+01,     8.3382943254073471e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -5.8283148761780392e+00,     8.3324505075008929e+03,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {   -7.3978313233618511e+00,    -8.3249323417460073e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    5.3036451210731439e+00,     7.7287173290351939e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2000497712046801e+01,     2.0994753275433486e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.7529197400080939e+01,     2.2681566094269430e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.8534053952362640e+01,     1.1445956476241683e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0572158968732658e+01,     2.2693538776873462e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5271925236987272e+01,     2.4108992299309019e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2473125753260959e+01,     5.9715598990656690e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.0752465708161484e+01,     6.2326024625787022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8973566466183364e+01,     1.8213718984504398e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1834855911982856e+01,     8.1512802034856186e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    6.7639859416659824e+00,     4.8042092013433319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.6257386760120646e+01,     1.8138796555808458e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4621747912264686e+01,     1.6121398477592200e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.7204195476509362e+01,     7.0225470835627530e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -4.9589640230926157e+00,     1.5404164759964709e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.6248133075485001e+01,     7.2111198997267147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8015628199397220e+00,     4.7434923411318750e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.8614589243304351e+01,     7.2170058309899750e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9289343818644884e+00,    -8.8363423068208631e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -6.2783373757172086e+01,     1.5264049976696409e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0351298436119755e+01,     3.1494277293171031e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.0347042456584012e+01,     1.9219628428103253e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.5831450021946685e+00,     1.5401479024622689e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.8588559239445566e+00,     1.5323200158628535e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    2.4167628924900335e+01,     9.7556601874729567e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -6.4107198487415307e-01,     1.6645385238503426e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    3.0563106214869421e+01,     2.4016062157747412e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1791576793335457e+02,    -1.1166640762821812e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2961707687932003e+01,    -1.5414309564132527e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.4848696685569221e+01,     2.2752407928169883e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.1401024661736212e+00,    -1.4274674356297851e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0998346038700449e+01,     7.1894434174424005e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7881093560756156e+01,     1.5543198915862098e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.1959589430498454e+00,     1.5542309664101493e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.4913020575129003e+01,     1.5469459784627394e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.2390957833117390e+01,     1.4691193668775428e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -5.5295912611265919e+01,     1.7046801022904969e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8901148278371362e+01,     2.2974070086751988e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {   -3.1185852643772215e+01,     8.2378220985047101e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    7.3325138699864221e+00,     1.7649152533282027e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.9345697454018840e+00,    -8.2584094646773374e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    7.6266645767279764e+00,    -1.6796577658439696e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9294668344112207e+01,    -5.5311439708993912e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2023822977121768e+01,     1.2227734266899195e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.3713562382180683e+01,     2.1502831817565238e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.1317188439500612e+00,     1.0106964639870821e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.2040402037342581e+01,    -8.3328102260758369e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0026607053420931e+02,    -1.1125930422480301e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0024288672330968e+01,     8.5213202095521629e+03,    -6.5107273016377942e-04,    -7.0072933250111656e-07,     3.2410764209534695e-09, } , 
      {    3.8549156120586233e+01,     6.8233430956067659e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.1975105054769593e+00,     8.3263645015149868e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.1058222929473455e+01,     1.7789546419672435e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.4044418243549151e+01,     1.5229934365094938e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    8.2607530345063751e+01,     3.0990336747879337e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.4570002501917569e+01,     1.5907389326264525e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8997139648607799e+00,    -9.6966402831818214e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.4285859181733230e+01,     6.3527698649016547e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6286189687509456e+01,     2.3914105530560948e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.7603394317096424e+00,     6.7036479243792191e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.1812482918994634e+01,    -1.2439959316730992e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5367261987832308e+01,     8.3265578560325339e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3031269798013319e+01,     8.1429187207253526e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -4.3731736271954187e+00,     8.3308249932143644e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2376701528608205e+00,     8.0181984228450547e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9752506567367671e+00,    -9.3062491458050190e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.4269746926833093e+01,    -9.4588620121769836e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.6585697864651756e+00,     1.7647544549331302e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.2461840369482204e+01,     1.3013981921511193e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.8229599623489584e+01,     2.6774094228445487e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.2993501767635630e+01,     8.1901018946063668e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3933746668153180e+01,    -8.3291900974037271e+03,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {    1.3879975485547476e+01,     1.7068437502880599e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.5761955218854652e+01,     1.5401734081556877e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.3314460274555728e+01,     1.5325516618085099e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2785492855362786e+00,     7.1463752597214061e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0503775904261285e+01,     1.9879102867802922e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3088261795252926e+01,     1.7949335090274708e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7403912872583809e+00,    -1.6455877661349503e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.0886915487166808e+01,     1.6282395563494623e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.4116933225106905e+01,     1.4910843908901334e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8416411027188644e+01,     7.7462322159185715e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.9964151320115484e+01,     8.4841245790224712e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7524308101391139e+01,     8.0284907381545772e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8825923910819526e+01,     3.9964955328620461e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.0113017707536638e+01,     8.3301929942913648e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1107312115489830e+01,     8.3271898549555353e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.2609894828962638e-02,     1.5429343058730810e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.4238750346577305e+01,     6.3038512782924363e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.5488313310181372e+00,     6.4430709326954047e+02,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9288053229897439e+00,     1.3110601138031398e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -2.9310402087173436e+01,     7.4344290423515031e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.2526764151366798e+01,    -8.0776723252715965e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -5.5609771077178202e+01,    -4.3096404403322049e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5192847495505628e+01,     3.1713104819542255e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5624132423858345e+02,     1.5544376195297973e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.2916427173477743e+02,     1.5541132384665616e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5582346053223958e+01,    -4.0340383054294080e+01,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -2.0304508414638569e+01,     3.3297492701728137e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.1217632233456921e+01,     1.5805272852929289e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    6.5450431875734871e+00,     5.0938534948691595e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.0614612516561735e+01,     1.4483482660903030e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.6849327276947022e+01,     3.0162954653344234e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.7620359456844277e+00,     6.7969203236906897e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3149783750768169e+01,     9.0531372441543099e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.3627903708698277e+00,     1.4574394832454454e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5646827483259891e+00,     2.7150641604543407e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.7503473699901999e+00,     1.6078738661636497e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.7823375797918160e+01,     4.2165346792437951e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.1499595709784444e+01,     7.2608593652715044e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6091112473407716e+01,     7.8288639391325341e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.1963299263694900e+00,     2.2625246136899386e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4887180792038913e+01,     1.6630722142662267e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.7569754986573130e+01,     1.9930419306418516e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.5679495721882091e+01,     1.7071073896514452e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3687120704258199e+01,     8.2808429000493943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1581686584930084e+01,     8.0779856981312150e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9446317412068126e+01,     2.3393761238768519e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.4487230104548530e+01,     1.3641832026621167e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2699573448050918e+01,     2.7548319852726701e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.1006425589356606e+01,    -1.2292682549008488e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.0668022725787228e+01,     7.7650849089783387e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9416727553052581e+01,     4.5585172819754180e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.8686710388784398e+01,     1.1198974676316811e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.8973392617532809e+01,     7.3672889985072425e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0215601465872274e+02,     8.3191828707777713e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.4303312904359331e+01,    -8.3381999784691288e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7455345270579130e+00,     2.2818950305212587e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.2558140571024030e+01,     4.5247961260437569e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.5138306669925583e+01,     6.5850506483966501e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8100933215902941e+01,     2.6225416447195770e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4350986114703689e+01,    -9.6168210332212566e+03,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {    2.4562288331206524e+01,     7.4116121215966878e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1506263699444126e+01,     3.3157031344153365e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -1.8069697600899001e+01,    -7.6316917341063254e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.7823810378245135e+01,     1.1110491134833057e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    6.8807763789650682e+00,    -7.7844950561891656e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.1923125625445781e+01,     1.6033484491371615e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.0424587904196532e+00,    -1.6626204445731819e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    3.4637692304037017e+01,    -1.6642462410887500e+04,    -3.9504806865784498e-04,    -6.7934846738776867e-07,     3.3587891827268648e-09, } , 
      {    6.4004287264356741e+00,     8.4191398309931847e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.1467765336289368e+01,     7.5328186711049366e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.6839457437371870e+00,     1.6169639184093583e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.5778067497631369e+01,    -1.7186632707011668e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.6658301813090386e+00,     1.2649620891304901e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.3744336330426679e+00,    -7.6823596313761082e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.8128793041922036e+00,     9.7418869632396472e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1124480400245730e+01,     1.0942767374390825e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.8331986112006774e+01,     8.3329939305889638e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.0636547119991580e+00,     3.4415875227689012e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2634714119196223e+01,    -7.1419660325511468e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3870744513849006e+01,     1.7121397124352879e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.2319921114543533e+00,     1.7254523579638313e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.1708342211768237e-01,     5.3267867187864817e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -8.9973811213262778e+01,     7.7782826748964926e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4503452831552593e+01,     2.3250764713185974e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2127611928936933e+01,     2.1765047471305013e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.6626915876756865e+00,     1.4381056687590726e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4097033351135714e+01,     1.4452733629771683e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.2296785185648858e+01,     4.7508145189664629e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    5.9612847268184055e-01,     4.0040206389691171e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    1.6930587789542258e+01,     2.4577305560662910e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7380591389223575e+01,     1.8588706270256676e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    5.1680161860274580e+01,     1.4392429630271166e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.5136932851460067e+01,     5.4582429177582526e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4479762449671924e+01,     4.6235241900252397e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.1124423602460887e+01,     1.6919891936455482e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    1.7432706412934493e+01,     2.4474016150034535e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    3.2798857824711462e+01,     2.3059699995296789e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.6896979836494488e+00,     1.8270601167941961e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6965329337674380e+00,     1.6165028400584586e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.2058716366178237e+02,     3.1085000621324474e+04,    -1.6545794030958409e-04,     1.5529430629880380e-07,    -1.0200479850544637e-09, } , 
      {   -1.7271624396134993e+02,     3.1086016538602711e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    8.0043011709667553e+00,     8.7561432475465917e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7921820468189686e+00,     7.9106446507810851e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -4.6980599855771779e+00,     7.2157880921839705e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7064003388609571e+01,     7.2123376385327183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8148755786442106e+00,     8.7458339662911021e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9285277567154001e+01,     1.4908546650851022e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.3775989132622524e+00,     2.4277584595496497e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.9121544704840869e+00,     4.7648222491097267e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3157987864346214e+00,     6.2729164039481220e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4797855098866339e+00,     2.2750783680897384e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.2327830287063701e+01,     1.0090043959063229e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1444001906591726e+01,     3.2499858104395105e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.7163220669339040e+01,     8.3330701673485619e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6608251778096559e+01,     1.5570363267155730e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.1583656403510187e+01,     7.3623137729174232e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.1967960202068070e+01,     2.5287723684706025e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8312061495730021e+01,     3.7309409745237535e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.8510311719012851e+00,     7.0795544211014203e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.1834419545701884e+00,     8.4895759726628603e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.5976055632074262e+00,     8.1882604572790306e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.0280891160236152e+00,    -8.4691223919678705e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.6868134364756969e+01,     1.5614099911352861e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.1124549762705946e+01,     1.6679674288845330e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {   -3.3688360617842953e+01,    -1.6635091409648470e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    1.9195242468273747e+01,     1.7436865833033742e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1813765295272498e+00,     1.4685719045182830e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.1168549737066975e+00,    -1.1431854010339104e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.6672138919328646e+01,     1.5542255617201516e+04,    -3.7512201314318606e-04,    -3.0439414819407097e-07,     1.3067667960626429e-09, } , 
      {    1.9019864376292016e+01,    -1.5543252962762075e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -8.5072081105021731e+00,     3.7670911544318311e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.3399693648248260e+01,     1.5503291704041958e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4937270067309978e+00,     1.6827117413085289e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.6296162204529516e+01,    -8.2980385022647515e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {    9.3680929726986086e+00,     1.3721109913311844e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9296472332791538e+01,     2.6225768763153843e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9195126059903846e+01,     3.1716445997325318e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1365212626328315e+01,     1.2073802682168089e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.0592453482668041e+01,     1.6684043555831529e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1117887141622541e+02,     1.0172068309384927e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0135312049918859e+01,    -1.3942318688792562e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    8.5640374255511169e+00,     1.5522090773724904e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -1.9805526371607566e+01,     1.3589916363976768e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {   -3.3677189266286767e+01,     6.6383720615659176e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8271486263543313e+00,     1.4744093112177832e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.5078067966951686e+01,    -3.3307525070818974e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.8303193697561979e+00,    -1.0334146131881591e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.0585261925014240e+01,     8.3275012422832988e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -2.2154778372198049e+01,    -8.3298816069636014e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.7211605355926833e+00,     7.7467415372369933e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8237195652069396e+00,     1.2745206836257736e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.3596748593067964e+01,     3.1220495313945758e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5134425785793516e+01,     6.5351131062100740e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.5955399341864229e+01,     8.7108956830595835e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4833306042942777e+01,     2.3348364978515208e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.2681965417190324e+00,     5.0474528289960126e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.6531020227172327e+01,     1.6740995393077879e+04,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {    3.4212016523299866e+01,     2.3424939642283596e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3274534088929140e+00,    -3.1315485191259127e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    5.7208337967905480e+00,     1.5923137069778975e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.8192236778068605e+00,     4.8766604976173394e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2135388101299196e+01,     1.3505571804097330e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.0614486327685370e+01,     1.1605216083833333e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5275808636890707e+01,    -9.4241323141325229e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {   -2.9723416531761835e+01,    -2.4970341789871047e+04,    -7.0626738448770513e-04,    -9.7716185163719508e-07,     4.7104982070283614e-09, } , 
      {    1.7957652864403627e+01,     1.3172221906092478e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -5.9725333864013876e+00,     1.6186794359026633e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -2.1396594891613884e+01,    -2.1955413135325580e+02,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.1059936226300564e+01,     2.8665249611933217e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.1653915665735514e+01,     7.1038401944923207e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.4413110212891000e+01,     1.5539501034158713e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5246267476972577e+01,     7.2468778904124429e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8041706937969167e+01,     8.4057450128552919e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.1766537724303785e+01,     1.9664479768707264e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.1709253006339964e+00,     1.1033041712561204e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.8883362669342375e+01,     1.5400415363197737e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4084044327850311e+01,     1.5594569401377667e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    4.6846075453351773e+01,     3.6980823125358394e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.2580468599303018e+01,     6.5868189417377525e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3321689298651038e+00,    -9.4625076536445831e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    5.5728949569093864e+01,     2.2145782567060265e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5142021510234763e+01,     5.9553701524835578e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9155858733476041e+00,    -8.6507190889034482e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -4.2172190289368729e+01,     1.5543008554366032e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6768945459807227e+00,     8.8768169163406856e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4922493961109303e+01,     1.5709348799862768e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0828456354141551e+01,     2.4500464645117663e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9870657166716807e+01,     3.0979570385541207e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.2079378798849902e+01,     2.9775924312661311e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.4714123102252273e+01,     7.1937127219610738e+03,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.7655288544515287e+01,    -7.2344130087556150e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6932294344126802e+01,     8.4065723180103469e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.0754721167635518e+01,    -8.3193037148044841e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.1696717790183548e+00,     7.7711534878331886e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.1510447126474443e+01,     7.7716008021486050e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.5289055768033997e+01,     1.6858888037144297e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.6470540468227568e+01,     2.3923260826001115e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.0093489307113650e+00,     1.5398137846839629e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.6054848317584227e+00,    -1.8888594947940874e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0468204050213775e+01,     7.7310367619366043e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    3.3814404341452146e+01,     2.6844761206610448e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.7047710216868399e+01,     6.2870083130798514e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.4478165747628324e-01,     4.5404683385195558e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5998348025933566e+00,     1.6464694130158165e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    3.7606860610626720e+00,    -8.6234966170474181e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    4.4504129992971224e+00,    -9.9138932145936651e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.6452667240483905e+01,     1.6804300492330472e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8853266520994527e+01,     1.4399739797306431e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.4363037552795305e+01,     9.0980192180939121e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.5932553999979113e+01,    -7.5593636311529899e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.9824492023983655e+01,     1.0586958282655665e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -1.0680072758186228e+01,     1.5542615314143322e+04,     5.7475133134433546e-05,     2.0773403267191242e-07,    -1.1063448202919609e-09, } , 
      {   -3.4615532608402432e+01,    -1.5542893265820270e+04,     1.8984148538210081e-04,     8.3498587632869385e-08,    -2.9030643224839010e-10, } , 
      {   -5.8075225751232715e+01,     1.4916834537911403e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2970370837633510e+01,     6.3739046146350902e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    1.3880917495748665e+01,     7.2109714412158955e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4703555679155089e+01,     1.5524972370750631e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {    1.0436053844583075e+01,     2.3249697085158838e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.3093480357954039e+00,    -1.2658039580064042e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6276051248244727e+01,     5.8872550080407536e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.2663836933364943e+01,     1.5546007545804876e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1193754854641828e+01,    -7.7193081741054357e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.5932840836760178e+00,     6.7866110424352028e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8136590617019539e+01,     1.8247500411959561e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.6901857490374312e+01,    -1.1633304250796609e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -3.1586061812858759e+01,     1.8383045996791880e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.5235141398909375e+00,    -1.9782227596632047e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6918679886124625e+01,     3.2821885768675107e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -1.1124217548754385e+02,     8.6212067977200277e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.4152583936581014e+00,     1.1283957973172790e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.2567267688936454e+01,     1.5163811072570466e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    1.0504944042450324e+01,     8.7113411933690677e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.1132011205358712e+00,    -1.5512101367623098e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    4.3123306849710215e+00,    -9.3036193134888345e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.1777680868312274e+01,     2.4866629568176613e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2488312796010691e+01,     6.0541781502965041e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -9.8087243684452829e-01,     7.1742211550843303e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.4961226291868204e+01,     1.6753297426574511e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.5055586594446408e+00,    -9.3248416805958823e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.9388790367863308e+01,     5.9754966387231198e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9789337958532517e+01,    -1.9643476694799240e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.0058114469133388e+00,     1.5444538512712777e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    4.4707328964314748e+01,     1.4770823472571252e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0622652422242336e+01,     7.7556199257375374e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.2659967961197314e+00,    -8.2265197156953800e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.1929222507567427e+00,     6.3106269546862313e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.9445564286401385e+01,     1.0733217908654524e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.4848756583450076e+00,     7.2171542895007933e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6116285683960413e+01,     2.2097331166779517e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.7065352793444806e+00,     7.6719979061676486e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3552007265687962e+01,     4.0986283883358992e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    6.8586026513794121e+00,     2.0941784178222293e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5006122046683998e+01,     2.2443978278975461e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.8421777054862361e+01,     6.9035251889350748e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5354288519633842e+01,     2.4028026083130353e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    3.2993755135006239e+01,     2.3514507372308832e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.0007727015610213e+00,    -1.7477206216088034e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.0971178481144301e+02,     1.9129067319053924e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9528733043301095e+01,     3.2827733378807367e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.0510017658122031e+01,     1.4539474678699124e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0371120652859759e+01,     5.8925146726731200e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -6.6907243699201011e+00,     1.4983362467565541e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.8452546445670670e+01,     5.5486942415749727e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7151756225112262e+01,     1.6617042466192604e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {   -3.7439909601629111e+01,    -6.2794855553674165e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.7760254682026059e+01,     5.5824756690301328e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2756027031606777e+01,     2.3788962421335527e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.9190797794577541e+01,     2.3869496830621909e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.5973638100020686e+00,     2.3873394598588584e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.6131590973656316e+00,    -4.9853196791235678e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.0066995450504193e+00,    -4.4598059116526838e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -8.1198432194185187e+01,     1.0483587647724413e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3885952035204582e+01,     3.7593574313911759e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2684335228424644e+01,     2.9829604755339380e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.8079650295886909e+00,    -1.1224953328396809e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    6.8106751994793910e+01,     1.5481363885352703e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    8.9221912638375258e+00,     1.8826607458410268e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1269106285411237e+01,     2.8784154628626180e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.5151109757884839e+01,     6.9353585520729575e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.2733718791016670e-01,     8.5328111853218779e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7039723808806940e+01,     7.1625164590809054e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1203484304051816e+01,     7.7610678637354094e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7404904590304525e+01,     5.2737831800353724e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9761175803082836e+01,     3.1879385911008635e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.7413140797285642e+01,     8.5996312949888470e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.2686621684853684e+01,     9.3075686838302136e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.4322481841763093e+00,     7.8243215069480248e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.7027147576833684e+01,     1.4913634659296360e+04,     9.2743731943700382e-05,     1.0921223261429318e-07,    -5.2374421970263164e-10, } , 
      {    3.4730052920662700e+01,     7.2147257236230544e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1189775816569156e+01,     3.5809595607938509e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.1777006963529777e+01,     1.5496905913641131e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -3.8668072915992635e+00,     4.3389631506221622e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -9.5021247208116719e+00,     2.5054766379341927e+04,     4.2378563522506172e-04,     7.8321548372372488e-07,    -3.9119615928728455e-09, } , 
      {    2.2160919511696473e+01,     1.0203126100877997e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2364185800926165e+01,     7.2134000070936308e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0076088148522029e+00,    -7.3596329353072633e+01,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.5358615303379054e+01,    -8.3155887729403366e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    2.8486491853852648e+01,     3.5498079656548744e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.4091431181728034e+00,     9.9525904147912697e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    8.3671457917433472e+00,     7.9129715738895484e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9542638927959086e+01,     2.7009054516550015e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.9453751865905431e+01,     1.4987517553132944e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.3132031743211360e+01,     3.0435871086940195e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.2675401719310145e+01,     8.0825880980140391e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.2195633156488796e+01,     2.5105518979564084e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.1947379335578539e+01,    -1.2880953976048368e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.0563969596273708e+00,     1.5524720606041086e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -7.7581668342350483e+00,     7.1984227854960754e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9204931316077751e+01,     3.2828092408316166e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -2.0241045275974823e+01,     8.3332091215538949e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.8105891511037047e+01,     7.7796613274156562e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0393838716957697e+00,     7.5710354186477343e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2823084656708295e+00,     8.7500983277266969e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9789142306869508e+01,    -1.4989796608814859e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.4314508685022894e+01,    -5.5360750031569842e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -7.9437658505523999e+01,     7.0720829680830902e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2878067834332370e+01,     8.1896831482127836e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.0253660799689719e+00,    -8.4676997010341183e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -6.9758572366639896e+01,     5.9729897235556873e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5437495936873077e+01,     3.1397027918303073e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.1334712084647558e+00,    -1.8127659881694071e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.9406948652666003e+00,     1.3596052988422536e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.0104713663905208e+01,    -1.1392117230660385e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.3893074427396826e+01,     1.5539811324350163e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1348778875522726e+01,     2.9240400726724871e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.5371046745333217e+01,    -8.0119031779173893e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.9836967184445939e+01,     2.4879656358145512e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0340994863455894e+01,     1.5006769918327094e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.8649519083777406e+01,     6.7102670580482272e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.1540916769919319e+01,    -1.7319155175882847e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8313829704561247e+01,     2.1070914592909185e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0361586780978087e+01,     2.4958844311807268e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.2385453304417574e+01,    -2.5001493385010028e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -2.4274343314187572e+00,    -7.7553776367669007e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.8327482135913318e+01,     2.2753972241282350e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.1524588769165689e+00,     7.1911686442680257e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8330041157964917e+01,     2.7949176214078761e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0764006770601515e+00,     3.8346084924618885e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.4912557898522799e+01,     9.6138234485317525e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -3.6475595832317751e+01,     6.5846916188878495e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5786296840402581e+01,     8.2867989799418374e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6118637338279086e+01,    -1.6884205311748872e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -7.4323269595574235e+00,     4.2276001678976718e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9241855231570923e+01,     1.5542500025597559e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6196046914972548e+01,     3.5173452631233900e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.1638135024083791e+00,     8.8324872319335664e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.3186894381738288e+00,    -1.6850071568335636e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    4.7270173119125714e+00,     1.6919901412194395e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -2.3703005562310899e+01,     9.3884526146881035e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.8574028078549247e+00,    -8.5156062120636830e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -6.2798378077567687e+00,    -4.5121166161022702e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -4.2953995434209766e+01,     1.5740097830994118e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.6200525509703523e+00,    -7.1599902407529489e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    2.4136883463600562e+01,     6.2081534217594926e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7215287253863263e+00,    -8.4360691814316833e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.5530636796226986e+01,     7.1173657800228721e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2759164227294551e+01,     2.7832411172634085e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.3932411516011900e+00,     7.1765917580209934e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0417411093638950e+01,    -9.4382529118468665e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.0097084719535361e+02,     1.5545697255613426e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8248002412038939e+00,     7.7873766532148957e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.0019450524443377e+01,     4.2679698452091707e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2313003945008717e+01,     2.3278083018569094e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -6.9451076492907688e+00,     1.5540083227075416e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.4022160153096770e+01,     1.5545425352888173e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2717913406025501e+01,     2.2762717209425369e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.8295058951962893e+01,     1.8048984753320154e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.3758502050429563e+01,     2.3520111474367848e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4831086874486662e+01,     2.2759662069397931e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6551295858184595e+01,     1.5146336239479910e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0048849914953671e+01,     3.0627992076554597e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.6781051905411076e+01,     8.3705838693050628e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8855530023493888e+01,     2.5607822903316755e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.1617815840592876e+01,     8.1140683255277891e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.5202755331192691e+01,     3.9401082093388766e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.9584990913653840e+01,     7.2599112416990092e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    8.8283662921820767e+00,     5.2255288389077959e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.0099509183074240e+01,     1.2540588402779027e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.0590364925264211e+01,     2.5870331990525345e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    9.9278281010916700e+00,     2.4400426504780509e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.8033700968907084e+01,    -8.9412665256696946e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -2.4744962745289513e+01,     1.9741832875811037e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5267924972511585e+01,     2.2756593498182428e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.7891885627147222e+01,     2.2757040812497849e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.3383191297650662e+01,     1.2645363135647178e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.1684276054181097e+01,     1.2721571983795166e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1937335086258400e+01,     2.3248069161817293e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.5341906647984418e+00,     3.0155508226257325e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.3005557716236858e+01,     3.4911411969878736e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9899236532267011e+01,     1.6654729333347415e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.5604640040993800e+01,     1.6660036365146381e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0364742902197625e+01,     6.5771187819299685e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -6.6962348087249381e+00,     1.5211866470161254e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.3206192963756980e+00,     1.7316868837807524e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.7974450973174942e+00,     5.3019176383656886e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.2583131296782707e+01,     2.2820095533806820e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.4547763568002573e+01,     1.1004587151403652e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.2618414073139391e+01,     3.9739805304868387e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.2281425351050469e+01,     1.2991966194047700e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {   -1.7932097719273401e+01,     7.8374394181462940e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8842214337035895e+01,     6.9699969051712287e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0404577479108854e+01,     1.9928118106477104e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.9109552584476447e+01,     1.4912058871961101e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0073379647782545e+01,     1.6672339182385014e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1354113574168643e+01,     1.4268090176434847e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.2922179883118588e+01,     1.4403852857107207e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.9854350866319628e+00,     8.2691109278601543e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4282837807185782e+00,    -8.3900156085825574e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    4.0984295598134636e+01,     6.7898968618440522e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.5438475783719205e+00,     5.3265102779055314e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3714111816951055e+01,     3.3837135089116957e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -4.3677448069900544e+00,     1.6642426516108786e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3610830947633907e+01,     3.3472500052834235e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    2.3765154190104365e+01,     7.1938375491098113e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.9521262456337904e+00,     2.3649277010450955e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.3133279091906718e+02,     2.3873176683082482e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.3197572111022687e+01,     8.3536126965380663e+03,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {   -2.4767088558206503e+01,    -8.3037701527088338e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -2.6581738137666591e+00,     2.3225089186103840e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.6730539153744139e+00,     6.9158596351222229e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6722901081475946e+01,     1.6573271632635642e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -6.8200071803681093e+00,     2.3844785008020612e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.9385969749575423e+01,     2.6734279326547574e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0479165136083530e+00,     3.1928432702787126e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.8671299045360485e+01,     6.0326987563379698e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6295480393591725e+02,     8.3199605793023948e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3283385296340988e+00,     7.2656092716357834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0123152538800323e+01,     7.2297029452206134e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7840689705683594e+00,     1.3606035374927116e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.0754875227235416e+00,     1.4825429191321462e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.3638584068744244e+00,     7.8335102948633448e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -5.7686078157794753e+00,     2.3835363087208672e+04,     2.6931621671538551e-04,     5.9995954836692873e-07,    -3.0545685978306309e-09, } , 
      {    2.1734726537638423e+01,     3.2515605847909563e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7394910609397658e+02,     8.3374222699445036e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.2340748440765765e+01,     1.4912921451685033e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0751354257651371e+02,     1.8206513392434565e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3376160744586727e+01,     3.4322976342034068e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.6055946278478306e+01,    -3.4884868614223287e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.9312989112763084e+01,     1.4391724998355017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5478496889079896e+00,     1.6997444197467605e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    2.7885856918498860e+01,     1.5446057204646322e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6145456987299617e+01,    -1.8623808500043856e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.1321992344783382e+01,    -9.4277779556001242e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.9011942012560091e+01,     1.4011694534515911e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.7625161670325238e+00,     3.7394183815490669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8266203397023617e+01,     1.5595713911454075e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    1.8436005077632007e+01,     8.8118838937465480e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.2795388820004149e+01,     1.4141024003968265e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    9.4382336324009657e+00,    -1.2182398305790243e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.7659644559438377e-01,     1.3104446378495561e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.8584909329764265e+00,     2.2343126108072589e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1154368450195769e+01,     2.4623706226536062e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {   -1.2385498471444281e+01,     1.6097991026830648e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.4027833918102139e+00,    -3.3084610674834657e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.6719117900074885e+01,     7.6932779347046153e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1587605456100874e+02,     1.4427617772077576e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {   -1.7742735306212353e+02,     1.4428633689355805e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.1451248386119673e+01,     2.3005221681890962e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2946215251932763e+01,     8.6928619991188752e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.4159300105606993e+01,     8.2626138919529021e+03,    -4.3477415702496960e-04,    -4.4466524206812487e-07,     2.0345206127761672e-09, } , 
      {   -4.2339627429541281e+01,     8.7241430727242714e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.3909143876725089e+01,    -7.9332397765226297e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0310394983531342e+01,     1.4918855932106509e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.7890226244380010e+01,     5.9057022593133115e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3854652096971520e+01,     1.5589550789894953e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9703030603204951e+01,     2.7519968130309418e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.3049186613382055e+01,     1.1946062245188648e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7137913272328277e+01,     5.3675154497337762e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    8.9490381047729954e+00,     9.1235522889597742e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4727550077755708e+01,     2.0487733586051072e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    6.7977351905934178e+00,     1.4390654623379338e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5873984522284719e+01,    -6.7439883074335139e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.7538229695659293e+01,     2.0132027333056179e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4988315785506924e+01,     2.2519270570636367e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2006140123573420e+01,     7.9974749724281355e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5055579262055350e+01,     2.3530239856308126e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.6438140989322627e+00,    -8.4592033071078549e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {   -2.8864702890112845e+01,     1.5525033978900707e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.5941755393918847e+01,     1.5560474601062882e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.5251442186645917e+01,     1.6656401187330681e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.7815253041782917e+01,    -1.6658364511163112e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.8157068521928341e+00,     9.4593194921125523e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.4529988601081289e+01,    -9.5394582183738730e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -9.8612797378078568e+01,     1.7618020709286935e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5426266532596031e+01,     7.4840363333226778e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.5972850674934502e+00,     9.1419558358654540e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.3776882917198762e+01,     1.6102146112398052e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1933497490513393e+01,     8.3204072421986912e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.1780945429082298e+01,     2.3642718054851819e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -1.0898543072632918e+02,     9.2495143827655702e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.8004484984498987e+01,     1.4913742073020103e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.8124124292085028e+01,     5.9296997422548493e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    1.2259732177734078e+01,     9.4057807405699859e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0767827512902016e+01,     1.7717408604183034e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -6.4430940826679812e+00,     1.5011143790271724e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.1882148061051332e+00,     3.8264976874561734e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.4380465926814058e+01,    -1.7216998328820864e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.0210985856397972e+01,    -4.0876871320743840e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.8353289754085491e+00,    -8.3282992426961191e+03,    -7.0844439523505521e-04,    -9.0969891283933662e-07,     4.3319072034499252e-09, } , 
      {    9.5086838157131606e+00,     1.6187061383251337e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.8114280117308187e+01,    -9.0704102409199786e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.7899457209563558e+01,    -8.3931410339524591e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.2101666836679037e+01,     2.8510145089908972e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4726287929630887e+01,     2.4011165336482791e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.3355110204119498e+01,     1.1989833078745611e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.8792017922845403e+00,     9.3680536633026913e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.6897815120615640e+01,     2.4018966517148883e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    9.8595206168402925e+00,     1.6621274498683277e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.1048375663415109e+00,     5.2761710130105239e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5184819271020551e+00,    -1.0955885285589091e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.0353129138094303e+01,    -7.2651172895791742e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.3984080696494523e+00,     7.2342881816068775e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4795425516302643e+01,     1.4901292509622972e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4236950401488269e+00,     1.3580279273833041e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4974629877589587e+00,     7.7816864262463851e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.3127853727172081e+00,     1.1615570259408116e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.9869680778543204e+00,    -4.2768300007497146e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.5329322410380229e+01,     8.4086735443773505e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.4517592481941897e-01,     1.9975017445130532e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.3271728768555583e+01,    -1.6189832174253766e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {   -1.1637130582807960e+02,     2.0345867587247092e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6344102264309306e+01,     9.3549510116195779e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.6866155636935154e+01,     2.4897705301601371e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3509939289044953e+01,     7.6496817313151541e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    2.0538930607043490e+01,     8.8047392789211590e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7494576270694218e+00,    -8.3290612875884181e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.1259274918364031e+01,     2.6066861471520271e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1374114113682964e+01,    -1.7406349443692361e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -8.6124294511418285e+00,     4.6080311934298770e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -4.1647503048136208e+01,     8.0111386118412156e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.2958631195457029e+00,    -3.4120585829711763e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    9.9049896092753507e+00,     1.0770893159465368e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -6.0689217216979614e+01,     8.2766705980015595e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    3.9463282674615961e+01,     2.2304292868090895e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4813066851554781e+00,     8.5562650985615492e+03,    -4.6568873433953641e-04,    -4.8106931960622267e-07,     2.2091020193437112e-09, } , 
      {    2.1197507809896056e+01,     2.2772816663564139e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.3350694563317376e+02,     1.5527564577651441e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.4332722798396361e+01,     8.2487093048695497e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6716740975304035e+01,     2.1285585447471065e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.6909807931650469e+01,     1.6199447808437595e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.8440479140456176e+00,     1.5376159780100821e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.2989503366558827e+01,    -6.9274598113901975e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.6755732554524258e+01,    -8.8863580202140784e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.4011810632568753e+01,    -8.8856533882979293e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    5.8128115259281170e+00,     8.9208906591053710e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.6761255330259974e+00,    -1.1683090019431734e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.1975923730303762e+01,     3.3929955372341079e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.3310873284431551e+01,     2.4124781273413661e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.6910661881350904e+01,    -1.1150167010586415e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4566066780029374e+01,    -1.1142404174715703e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.4721248962695832e+01,     2.4816860350008352e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.4299987948184715e+01,    -2.1272872459270293e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    9.6258222328874332e+00,     8.4242773695762844e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2273652644352356e+02,     1.4916177673413489e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.0764657390461672e-01,     2.0098970372015800e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6321508759909602e+01,     2.2004556643409320e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -9.0742556267409888e+00,     6.8831750455378024e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.0306319840509416e+02,     1.7912267050860748e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0967365087581367e+01,     8.7631468290260273e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.9265924171493101e+00,    -8.3360141696424289e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1402123810186939e+01,     1.6737577117587405e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.4330723939014121e+00,     6.5856068632555871e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8826404635668425e+01,     8.5906750306756512e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9905697653012098e+01,     3.2726273015346171e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.2164019804050007e+01,     7.5023091021831806e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.7788633953775950e+01,     2.2756806633136333e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.5371074220920708e+01,     2.2756827677543948e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.3846031250621280e+00,     6.7778299129585484e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5666953680440958e+00,    -9.9683381583057217e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.9058662345486788e+00,     1.7796959061811233e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4044608712543202e+01,     4.3670015001798256e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0874859713009460e+01,     8.9876519320276911e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.6885606026990569e+01,     2.4435763575717232e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6022656065374854e+01,     8.6576855960534886e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.2521547827892551e+01,    -1.5594646403539627e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.0636414654543378e+00,    -7.8401034614554501e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.7337998849246116e+01,     2.1185066567771355e+02,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {    2.0101716937943465e+01,     2.5692378745808321e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.4675823351377289e+01,     2.2916076762931789e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.6893301324570658e+00,    -2.4992633229469484e+04,    -6.4226121911122155e-04,    -9.7181663535885805e-07,     4.7399263974717103e-09, } , 
      {    1.0825152555150298e+01,     2.1748606511799873e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.9534423079637758e+01,     8.3439594468354426e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8583418481799590e+01,     6.9292623385513289e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.4998278119757021e+01,     1.4412126222492692e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.7414998152562077e+01,     2.0759558634514648e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.3513720291651779e+01,     8.3175727463272469e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5510112052324150e+01,     9.4236431170911565e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0163686310530409e+01,     1.2101166767212893e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2489308132552051e+01,     8.2917214367833294e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0154249822107007e+01,    -1.3248893704682835e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5656317873510639e+01,     1.7618796992874006e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7575074838284760e+01,    -7.7364664669744761e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.7099683535131430e+01,     3.5693834229641652e+02,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.1368606904450354e+02,    -1.6659113817724141e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.0450817656125376e+02,     8.3398101029196532e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5910730145446902e+01,     3.1556081810668700e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.1809005437189235e+00,     8.9213427751927247e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.2606466061688137e+01,    -1.7279656959849690e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -3.5740721630693429e+01,     2.4108604157515481e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8702291350582588e+01,     4.0528117218385269e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.9666765623429193e+01,     6.5946154358533440e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.2508011310405134e+01,     4.0529539909319014e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.0726166201789958e+01,     1.8420158074620431e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.4074072802492132e+00,     1.5527114210119526e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0590746697988680e+01,     1.2463020882251752e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.1200968079541596e+01,     2.2746507874084651e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.8921325280542138e+00,     1.1411039405154526e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.2149437957702395e+01,     1.1419899560695068e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    3.5935628402484866e+01,     1.2796538534388477e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0058239949423466e+00,     4.4694750331065652e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.0118858577835965e+01,     1.7258460472229362e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.2097666981830017e+01,     8.3312232053657935e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0309196138978321e+02,     8.3261596438811066e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7455572741070302e+00,     8.6391844264018437e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.5694632155949346e+00,     1.4815119910065974e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.6184401317407151e+01,    -8.7397613377723028e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.8385850724704493e+00,     7.2145363861128562e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.3527358330561949e+01,     7.2135893446038344e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1490133916424284e+02,     3.2199629180589578e+04,     2.0531005133890193e-04,     5.9461433208859170e-07,    -3.0839967882739798e-09, } , 
      {   -1.7840206845888963e+02,     3.2200645097867808e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    7.5080599519930100e+00,     9.5491982441509135e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.3484043159478261e+01,     8.3656614124635707e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1321257595532142e+01,     7.2144550472856763e+03,    -7.7462757135888876e-04,    -8.4758119031981508e-07,     3.9238880094281401e-09, } , 
      {   -2.2739613364091877e+01,     8.3225874550727149e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3535801721089349e+01,     1.2565181042318873e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0962691129941170e+01,     2.7402075486243004e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3362272577103191e+01,     2.4133964277552739e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    3.2101306177679705e+00,     6.3842500924154074e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.5527149134314772e+01,     7.2145074912386481e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.8387942687176224e+00,     7.2136182394780426e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2685613755921612e+01,     1.5619195533392660e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.4439078383601185e+01,     3.0002046875352615e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.9320484772315403e+01,    -9.1245641781419617e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8209166496396236e+01,     7.4356099418381182e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4272711739560545e+01,     1.4904848769147640e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0661932825323980e+01,     6.4306507582885533e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6521753362737414e+01,     2.9039501099337504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2750580664317109e+01,     3.3315094330868575e+04,     7.6146203881163085e-04,     1.2535943707732736e-06,    -6.1799199931032542e-09, } , 
      {    3.7878202374591119e+01,     3.3314437066119026e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.2714232430412507e+00,     2.6250908720858274e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -8.3271544580946077e-01,     1.3348921781763434e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    5.8888596854180456e+01,     8.3269101719906103e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.7894314415763027e+01,     8.3304726772562899e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0495661528865281e+01,     1.6174864644636664e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0464622530731042e+01,     2.3338739334027687e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -6.1686629403419531e+00,     8.9558944179396749e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6461823283351904e+01,     2.2479843810531995e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6895623550015756e+01,     1.5740303546220137e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0326120944170139e+01,     2.4737669747319527e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6060193266438901e+01,     4.8319246912309836e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.1376007496939611e+00,     8.9640025911484281e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9406046112160091e+01,     6.8692105471578543e+01,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.5782873098586208e+01,     1.1118895109613770e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1066347207394969e+01,     7.1746002794185069e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6954728567831914e+01,     1.6419459840260490e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    9.1284370871719631e+01,     8.9577036415851417e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7588459131263591e+01,     6.8021027865680308e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.3717365027714106e+00,    -5.3009180777652546e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    7.4075031895866097e+00,    -1.6663282903332132e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -4.8633513084143544e+01,     7.2142628389327519e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.0999456487175948e+01,     7.2138628917839351e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6241197367736806e+01,     1.5539662865839346e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1403171892159548e+01,     1.5684756160322928e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.3333008825907193e+01,    -1.0419986323130954e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.6351907641983288e+01,     1.5127034439247893e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5218906579707507e+01,     1.4950529332332824e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0547374635038904e+02,     8.9552680411917554e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4134309259420377e+01,     7.8730386322777376e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.0200003001841669e+00,     2.4620285251772300e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.1947956052446642e+01,     2.2767126436595630e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.2653632648546306e+01,     5.9798821949744279e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.4160629623387536e+01,     1.7323774367713115e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2338911182366164e+01,     1.4915542585803740e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.8294631794129668e+00,     2.5002073782094347e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.3945437104751322e+01,    -1.3940017488851146e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    4.7774002825993279e+01,     1.7286042750250515e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7704983336766862e+00,    -1.6663994248799008e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    3.5242810016606640e+01,     2.7237782176303437e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.1656865648369759e+01,     1.5542038353114543e+04,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    4.8733918152175761e+01,     1.5543470226849046e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {    2.5041583256106691e+01,     1.2199382544481912e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    5.7967889707198657e+00,    -8.8848422427017722e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.5126985573318272e+01,     6.8364610068050360e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.3720922642918747e+01,    -9.1041916683751388e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.0967421981889332e+02,     2.0679928830718822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2087759493770564e+01,     8.9227654661264769e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.7030041460359740e+01,     4.7807325779024352e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {   -2.8281792188525031e+01,     8.4173566222801637e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.9967894168420756e+01,     6.9738452177481940e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7730371625657817e+00,    -8.0178968290042667e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2631530761693486e+00,    -1.7841841252370763e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.1416641958659289e+02,     1.9423313660627737e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.0664847111635112e+01,     8.9336925384605747e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.5503112399290039e+01,     1.2750137868515329e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.5560006025497842e+01,     1.4845754599464670e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8398632999931561e+01,     1.3778488237693295e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3113980394473774e+02,     7.1988731530279892e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9392732270118280e+01,     1.5491207883704357e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4864250198624745e+01,     2.4983932035648530e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    3.8725155498937220e+00,    -2.4988216512092167e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    3.0482566350416622e+00,     7.7003943617817158e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8843569838033279e+01,     2.4957208619157631e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3906844841640353e+01,     9.9017695959955290e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1909196743912874e+01,     1.7363242695304569e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.8615473679124186e+00,     7.6124698533573228e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.1199268417897279e+01,     8.4793145338322647e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7698336471186618e+01,     1.8531817525501443e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1224078906063752e+01,     1.4962898105216998e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.3832592165332116e+00,    -9.7330515429531642e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -3.8349447730558033e+01,     8.3245363390560487e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9343179890303233e+01,     8.3328465101908514e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.2759027352909360e+00,     2.5845522779699359e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.8208198135873461e+00,     9.8795529362883517e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2845050300980780e+00,     9.3947601138476239e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2275867066725546e+01,     1.3466315753049574e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.3656757241776809e+01,     8.0293780110079451e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.9077039028353848e+00,     8.3326473562825013e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -3.4772203500191967e+00,    -8.3247354929643989e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.2526970415719433e+01,     1.5475330410909950e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.0835547615413621e+01,     1.5545845714124247e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5971773974964858e+00,     5.8500699384590253e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.5764628023534023e+01,     3.1793998258337440e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5650534100951045e+01,     2.8917926953212600e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    2.5983869873936957e+01,     3.9178913389234483e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -1.1808569502236852e+01,     1.5545485740717544e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.8885622006042851e+01,     1.5540022839246045e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3800921836449938e+00,     1.5889374593801635e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1842417945304277e+01,     1.4740945655603547e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.0337600311150789e+01,     8.6103976573269738e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0019743609349739e+01,     2.8130242031414244e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.6839544664883919e+00,     1.6145322739879228e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9926959444306387e+01,     2.3871838960867688e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.0394264679650167e+00,     2.2744197916922152e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.8230652447968705e+01,     7.7685322309331059e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.8784040851631945e+00,     2.3772118919734967e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.6462702295512678e+01,     7.2044649295697327e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0414202655628607e+01,     1.8606739954197386e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6310664888543457e+01,    -9.5884350998109985e+03,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {   -8.1678234648103931e-01,    -3.6304887317634611e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2009920513953688e+01,     1.5305595847071560e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4547723925920877e+01,     1.4443544841856366e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6631923230413857e+01,     9.0256911151405748e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.5713101618273804e-01,     3.0485665530759143e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2002845629378402e+01,     3.3951868568957543e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4599291508303057e+01,     9.5330488429702218e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4490507397869692e+00,     1.6390929379240217e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -7.1220324448612828e+01,    -1.7653260435731681e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7136570058746194e+00,     1.7976354826368144e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4756842919848165e+00,    -8.3216878431440164e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.3494828628203550e+01,     1.5316327830169785e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.8606036730977706e+00,     2.3871052468342801e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7828088040301800e+01,    -6.2227411060279132e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.7171945752168871e+00,     1.5740829027376520e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.2482097101233421e+01,     2.9058270581626202e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.2447824274951103e+00,     1.2352185866564749e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.6112361559338693e+00,     8.8640276375171925e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6771170590695199e+01,    -1.1813641090203746e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1421948912346309e+01,     1.3774079010523035e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4764374009049519e+01,     1.4918049500971167e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5665801089946143e+00,     1.4518795902927081e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7352603971335672e+00,     5.8526824381380175e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.4092937526970815e+00,     1.3921965675434247e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4696965375408606e+00,     1.3848378015712078e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4599907913493333e+01,     1.8516069781986997e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -6.5381630050615072e+01,     2.4986080987421075e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -7.4788709060592886e+00,     1.5723589400304625e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    8.5798196414409134e+01,     2.4986067560319621e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.0392797215910949e+00,     8.3720625860460295e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.0996786111876133e+00,     2.2612200712197973e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3435212379118859e+01,     2.2235159110183857e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.5489748947543065e+00,    -1.0900453954641728e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0365776117560149e+01,    -1.3234666795345308e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.4491084014814668e+01,     2.9995487919753479e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -2.4742197918548278e+01,    -1.4436227306951439e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.7831609230678609e+01,     3.9325182490972249e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.3502102938630964e+01,     2.2007852867446465e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.1378124820598199e+00,    -9.3373817894661715e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.4554158192761246e+00,    -8.1775160258821525e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    6.8665833041795480e+01,     6.6657374000667005e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2307946607538353e+01,    -6.9209008557910665e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    9.5983714009716490e+00,    -7.1720034793371576e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.2616906880709262e-01,     8.8922979402685596e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9068561220900605e+01,    -8.3216915477796338e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.6676663258050802e+01,     1.5147432120347396e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4991583445836838e+01,     1.4318489654590923e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1651256951472222e+00,     8.4864352547027920e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    8.2152672532879706e+01,     8.3377688442509643e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9201768566555977e+00,     6.9117120678757474e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9777642029406239e+00,    -9.2311277746951259e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.1737921708396492e+01,     2.9893327759686359e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.5253746292986428e+00,     3.9769649662528382e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9830816390893098e+00,     2.0176522633027926e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.9438229202127353e+01,     1.8218650016761992e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1715346269098930e+01,     8.7835169118141112e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2534764836210329e+01,     1.5582921507837782e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3251089906469367e+01,     7.6179005463081903e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6247432142460280e+01,     1.5487642954548783e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -7.6687267681153095e+00,     7.2232363937585105e+03,    -6.5324974091112938e-04,    -6.3326639370325810e-07,     2.8624854173750325e-09, } , 
      {    2.1089543672018191e+01,     2.2704559403595824e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.7706907472326264e+01,     1.6658908102498121e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.0637241193276466e+01,     8.3274158172399857e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.9067724746092651e+01,    -8.3299670320069145e+03,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    7.6175039465189096e+00,     6.3232367173653083e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.6169840534129039e+00,     8.7444112753573518e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9595790547988440e+01,     1.4456511664126949e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0084438547208084e+01,     1.6075432961860442e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2000369553140224e+01,     1.6655857595995683e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.4827448874149063e+00,     1.3169209360684194e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.7992631541666260e+00,     8.3306099114727567e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1950312537865662e+00,     8.3267729377741416e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.6118978161237919e+01,    -8.2024372977967669e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.0769928282813236e+01,     2.3417939765439784e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0811285905872520e+01,     9.5956158759700247e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5987876953886419e+01,     6.5461597567210140e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9055199223571986e+01,     2.5596310060553205e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.0385049008734544e+01,    -1.6527646448805378e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {   -1.0892684197973969e+02,     1.0800375894430470e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3673877827666828e+01,     5.3314413101631235e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.1115628206730150e+00,     3.3614054294405942e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7294135250329074e+01,     6.4257197260309586e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8225792137321299e-01,    -3.8657753471160186e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3591333006215191e+01,    -3.2802776902897585e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.5505653332938722e+00,     7.7742220590486886e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.5837341675392493e-01,     8.0650188758606355e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1087173941658898e+01,     1.6668403475969262e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.8511523102209296e+01,     7.1340807456044449e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5918676927826901e+01,     8.9445474901916841e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.2080089456138463e+00,     1.8070578387863094e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.8814441842561767e+00,     7.2236608011469561e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5470130088598253e+01,     4.7600224698728125e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1137268466963671e+02,     1.4452398604326170e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -8.3080249539730758e+01,     1.5812727757946126e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6156863687363732e+00,     1.6646362222524538e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.1412556803606879e+00,    -2.4999192185068619e+04,    -8.2764521493546451e-04,    -1.1914766482537522e-06,     5.7719007990814690e-09, } , 
      {    4.6602954480506199e+01,     2.4811272637990267e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0004621177164015e+01,     1.3806429545223251e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2643320825562565e+00,     1.4351072142484845e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -3.3517056005336080e+01,     4.3526654999873381e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -3.7193362115758291e+00,     1.5917726316218923e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.1267414484120325e+01,     2.3267721293360632e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3666903517374762e+00,    -8.9389748014671950e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    5.9792655101511443e+00,     9.5316261520364696e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2733146061119085e+01,     1.1725039534798289e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.9490208567158209e+00,     2.6247898222839613e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5451698820903738e+01,     3.0368183481303258e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -4.5007840066828342e+00,     5.2930916938181163e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0301641161061195e+01,     1.4929824585485736e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.7963975327762910e+01,     5.6421980953814818e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.2485705812238422e+01,     1.5558394369844063e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4432855157607676e+01,     6.2866661455434496e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4631574681359412e+01,     9.6551364385732959e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -2.9184988710032549e+01,     2.0036079217384853e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.1899648978865329e+01,     4.0044808789573988e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    5.1014442409985588e+00,     1.6239336075404535e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    9.5638679849695230e-01,    -1.1160512501988583e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3068300285040216e+01,     3.8457305799662368e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    7.3028086997737240e+00,     1.5305207705278024e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.8699768264162626e+02,     2.3243646088198821e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.5703870933292862e+01,     5.3013921572093059e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    9.1558930283253730e+00,     8.0092935926596365e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1236487165783828e+01,     6.0145355939308383e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1572315071840578e-01,    -2.5496415999535166e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.5467666927177461e+01,     1.2345859434567737e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6622842147265526e+00,     1.5037305580166227e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.0202215191334808e+01,    -4.4747247472271260e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.2916682986030472e+02,     1.6656932481714986e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4487223336903153e+02,     1.6657833216778814e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.5592588893905706e+01,     9.0582043161959991e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -3.7162105341089514e+01,    -7.5991785330509019e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1150702156122128e+01,    -7.9487883661287278e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.8403465725100581e+01,     6.5686879987731536e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4971002316175994e-01,    -1.5542362108054464e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.1303907732955947e+01,    -6.5846517529185967e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    4.7335429804959240e+01,     2.9438173640120931e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.3602302375840853e-01,     7.6467033968998385e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1138456645977971e+01,     8.3158962588189715e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0970891375030325e+00,     5.6239453026456849e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.2000843604684924e+01,     8.2222735088986101e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.6303534417245633e+01,     7.1514958323585415e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.0369938383311315e+00,    -5.3783406488165519e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.3278986567549349e+01,     1.5494302889701048e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.3171464334010780e+01,     2.0005670367489227e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.8711759416919485e+00,     8.3040558168695461e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1630354186902185e+01,     7.3099774426859503e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {   -2.0090145418701020e+01,    -1.5283196065326579e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9884447770153201e+01,     8.4007882574306495e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.0624987899921720e+01,     1.3768639742156069e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.5913964503413174e+01,     1.4844556926917467e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.9557580566315846e+01,     2.5665698297531482e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.5416904356227732e+01,    -8.1222552044684599e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -1.1196455981459805e+02,     1.8500759734008381e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8304706849459567e+01,     2.3173248351540919e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.2132934657098680e+01,     8.3414865904279286e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.5773984683943256e+00,     2.3314460067347467e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    4.4051384570513534e+01,     2.3313802802597918e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    7.4030968112664661e+00,     9.0268288535277534e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2243374152817104e+01,     3.9668631444262937e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -7.7142625723248726e+00,    -4.9051375857952580e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.2071839002386309e+00,    -6.3486654064467712e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9806689832291099e+01,     5.9417152112679614e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.2231275966089337e+00,     1.7579936775866256e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4509892341447420e+01,    -8.3125502252639490e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0643301998246154e+02,     1.5557944002312150e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.3405571108217046e+01,     1.1406141574343770e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0004354504787777e+01,     2.2208691663622903e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.7730964200347231e+01,     9.1629790490641499e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7828979252158017e+01,     1.8418735383686679e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.0895942968149981e+01,     7.2940449851122439e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3833626659665974e+01,     2.1864871232524758e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.5629156555676463e+01,     3.8944799711175066e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.5446090098940751e+01,     6.6028225618524484e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1868005620343176e+00,     7.1706917039357641e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6291830504003329e+01,    -7.2632634039531813e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.6086507107104257e+01,     1.4066271615677833e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.9284851708279476e+00,     1.7996031232252224e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1688172408680359e+01,     1.4329909953447674e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.0694350504806639e+01,     2.3450079069596577e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.5496147324700225e+00,     1.5923156021256795e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.8123371736487982e+01,     1.1667805414465613e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.5677275302671916e+00,    -9.0437400585964351e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.6114059466827669e+01,     7.7567071721653119e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    6.6899821046904000e+00,     2.3260058810367293e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.8676739117073936e+01,     2.3839965487091493e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    4.6493923122065993e+01,     7.1345621371065477e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.8248384295505240e+01,    -1.5099507288995046e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2425437857515824e+00,     2.4078365348025363e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    5.8284361373194704e+00,    -7.9448781943236008e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    2.9909107105119222e+01,     3.9428289353478613e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.6026801671003703e+01,    -7.3734007357141672e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.2541497462198265e+01,     1.5526859945735290e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.8571716820875793e+01,     8.5375362584436025e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    8.6655414000910213e+01,     8.5389455222759007e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -4.6943305366792728e+01,    -7.2001574586089355e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    9.9662782060869635e+00,    -1.5196632658942233e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -2.6127415503658867e+01,     1.5191420049744398e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.7728519687332689e+00,    -8.5477773158943808e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0959465183566117e+01,    -1.0609481165870379e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.9578967293540295e+00,     1.9535097136784119e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -9.0925136110957201e+00,     1.2256599921611913e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.0753141860607656e-01,     3.1108050485095835e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    5.0176085029300282e+01,     3.2910928017543956e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -7.3165373747475144e+01,     1.6044617292489786e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.1476531866624299e+01,     8.0958354876787289e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -4.9329233620987694e+01,    -8.5615473615681731e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -3.1151432005335344e-01,     5.1507422396243901e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    6.1208105049896933e+00,     8.5136482508199097e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    2.8434315089982647e+01,     9.5913668775333917e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    7.8041219620009992e+00,     7.8669371188686882e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.4139449835202839e+01,    -1.1149571916398766e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.9607493667809578e+01,     1.6849659318217535e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.9692001402213275e+01,     7.3526523953754295e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1433180800997576e+01,     3.1085149550454786e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5366070303778945e+00,     4.9073020138982088e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2287353023991910e+01,     1.0108387330804573e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1125514021691671e+01,     2.7912989100002407e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -4.2156117827373784e+01,    -1.3137030687546667e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0646744347987820e+00,     1.2711425408802575e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7342037318325545e+00,     1.3858687296967568e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4267889021107472e+00,     8.4552481815568080e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.1512828312817597e+01,     2.3243497159068505e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.8654468465087298e+01,     2.5838851763011589e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.3917583842810849e+01,     9.2677537819323024e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.3563386474218280e+01,     3.1085867609472392e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -7.6305930925655971e+01,     1.2074264061287226e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.8440244000581316e+01,     1.2136620868317381e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2176099272630942e-01,    -1.0485312054883980e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1718732053498854e+01,     8.9996588256246960e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4628307306963560e+01,     2.1138347870210193e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9634549456660451e+01,     2.4083625012657729e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    2.1695628069335612e+01,     1.5419895414751820e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.3975581192146635e+01,     7.2355422593168787e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {   -2.1609637789114235e+01,     7.1925834713998120e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    2.3496818225340114e+01,     7.6998315500454237e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6013818688093089e+00,     6.0376297885955646e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5648748854389725e+00,     8.2021346676900921e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.4179564119407544e+01,     3.4513262950899607e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0877595638791475e+02,     7.2292525776886987e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6835785563760673e+01,     7.3207124052476038e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4284398609644839e+01,     6.1339802996245753e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.0243767239939572e+01,    -1.1143742948808699e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.7876550211587137e+01,     1.6678958351978086e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -6.4157175759544558e+01,    -1.6635807346515718e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.9781970233025561e+00,     3.4006134822070646e+02,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {   -4.3160308981097781e+00,    -4.1010619624685733e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.8051900711692412e+01,     2.4674554935530869e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.2810657769730440e-01,     7.2424145877756309e+03,    -4.0603659045775281e-04,    -3.4079822573216866e-07,     1.4813482026301869e-09, } , 
      {    9.4753417032457765e+00,     8.2529615266794626e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.9058252560619646e+00,    -8.4044213225674393e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.9873919917976405e+01,     8.3295960301140094e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8879625510023214e+01,     8.3277868191328907e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8612100353841718e+01,     2.3898106421189877e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.7173535669008579e+01,     3.1558711642984883e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3683706829927736e+01,     1.5543146471909129e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    1.5073481216380060e+01,     1.7514418094045868e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.4754173456373479e+01,     3.8850326769355357e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    5.5685530426388610e+00,     1.7283060601976256e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.7479620319819944e+01,    -3.0954658917293436e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.7457219612017994e+01,     8.9812718832784758e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0715657999613462e+01,    -5.1381230012064043e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.8685111851540483e+00,     1.5749673923401915e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    8.6407822363377349e+01,     1.6115704944840996e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0182733317013735e+01,     7.8473014826614854e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5431370898777427e+01,     1.4946797239636468e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0178847839369688e+01,     1.9439182559456509e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.6292538452512535e+01,     9.0143773340504431e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8864664775545918e+01,     2.6578467833243376e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3451709310921196e+01,    -8.4761077404873049e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.3232656395842497e+01,     8.9566399801601929e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.6221071317488693e+01,     8.8618271775847388e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0179998239419504e+01,     2.9342924751611015e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -1.1756820428455930e+01,     8.3212183877948482e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.3326336875639742e+01,    -8.3361644614520537e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.4084771923625937e+01,     8.9562943777528435e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6706933615440974e+01,    -1.1132058683313530e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2165585629353721e+01,     9.4674139506481188e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.8525963970600714e+01,     1.8937956406392703e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.8224915822040604e+00,     8.1206272811269218e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.5230196123733307e+01,     1.4980922313418798e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7508365957210096e+01,     1.4762186974188417e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.9645528440929098e+01,     2.4579383103446613e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.0007079399571035e+00,     1.4666540851165708e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.0003691642405453e+01,    -1.6045995945008952e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.5463902104953917e+01,     2.0088756820219010e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.9148431756432442e+00,    -9.3586231080510424e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -5.2640551364151307e+00,    -3.4389729499678688e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -2.7075665373188027e+00,    -1.2320225305512436e+03,    -2.1629857313880979e-04,    -2.5606409043299169e-07,     1.2065558081773019e-09, } , 
      {    6.9260036274742411e+00,     1.0660686892241729e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.6036278770405531e+01,     8.7266434265740318e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7298180338331100e+01,     1.5754933588034282e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -2.6182985574360703e+01,     2.5160875909750430e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.5485449415773484e+01,     8.7589242722742674e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    8.3051995102327822e+00,     7.2593170356232249e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2045520618973926e+00,     7.8346555234575762e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.7788641306408245e+01,     2.5996586117352024e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2721186154336333e+01,     8.2150565362147572e+03,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -8.8924690950156517e+01,    -5.5741233120639174e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0729909370683160e+01,     1.6465867067451309e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -9.8533468438365034e+01,     2.0719743732616735e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0089396824595697e+02,    -5.5721622805871391e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.9364212486043854e+01,     8.3293542828881618e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8369993274545251e+01,     8.3280285663587383e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3536664434982704e+01,     2.8949551465959954e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    8.4988815272287965e+00,     4.7446033587331367e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -2.1012452371156932e+01,     1.7248872008547860e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1653799549718332e+01,     1.3932986302156611e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4768875109578084e+02,     8.3259940537451475e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.4925826754296463e+02,    -8.3313887955017526e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.9623527633612472e+00,     8.2853202632008706e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.1669460308051882e+01,     1.4985341958775405e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.5914546436928646e+01,     6.7700945086009961e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.1443238323151655e+02,     1.4985538061923082e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.8715739796327476e+01,     3.2758554946255455e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.4451192319535231e+01,     6.3769171600433765e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -9.5345285503884263e+00,     7.0048420319968154e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    3.4811069181339590e+01,     7.0041847672472704e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.3532251182944542e+01,     9.1585041975663880e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.1801395785371938e+01,     2.5059368779224747e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    1.3195942275900240e+01,     7.7725406065451289e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.8789805775336959e+00,     1.6625743660013559e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.2565954424862056e+01,     7.2099077797909431e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0760018666107143e+01,     1.4967768118105518e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1923654281642072e+01,     2.0272123837495910e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.8428641608584442e+01,     2.2614823830963433e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.8716240622664316e+01,     4.1469633615550265e+01,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6119117600434990e+01,     1.5522528973733261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0214402811217045e+01,     6.9945087340050868e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.6914499653927141e+01,     3.0527090772936830e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7576778883737301e+01,     2.3769274005677173e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.4927611945906278e+01,     7.2182179509257458e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6746712290320112e+01,     8.3121230597739304e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1713987850655680e+00,     1.0857921273029620e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3956224552194083e+01,     1.1064090468959910e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6653799319719607e+00,    -7.0601123485661077e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0662252422836183e+01,    -4.4650607232165021e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.8012582933160601e-01,     8.3250448783050815e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -8.1328275836950041e+00,    -8.3323379709418186e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -9.6936682770364442e+00,    -1.1507111866616760e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.2117405578979850e+01,     9.3396487784146120e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.6492567913386470e+01,    -2.9377315373889123e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.8873664809040598e+01,     9.4252710404326936e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2672261376228415e+00,     4.1948470488827297e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8544024060528894e+01,     2.1707136878184327e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.8136703247538748e+01,    -5.1887086183529902e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.5808601281870054e+01,     8.3327483358356330e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.6870247196296724e+00,     1.5594300696259232e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6068068808440144e+00,     5.0120330598641513e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1429595932537303e+01,     3.2709000018580500e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.8779862723231822e+01,     2.6073475520099437e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7898096080328614e+01,    -1.1148828236493409e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2032878721510960e+01,     2.5694563952244200e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0231575986738169e+02,     6.3157818123484685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.1230205562613760e+00,     8.4979079975125114e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.6357201751321313e+01,     7.2164927241001196e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.3991258348288923e+01,     7.2116330066165692e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0916138232845412e+01,     2.0643622015498840e+02,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.4178721981581988e+01,     2.2700150176425566e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.3314013144167784e+00,     7.8409477594701357e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0707901425036157e+00,     1.4374053106111292e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    8.8569591493621004e+01,     1.6084526541325919e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    6.6730019626119006e+00,    -1.2735358508986797e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.3846852990719682e+00,     7.7467692459359023e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.5471424987020026e+01,     1.5558648634228299e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6895175950667550e+01,     5.1530434395658003e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.1094359575791501e+01,     7.2119292967674292e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5408551018554775e-01,     1.9783501663334937e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5890534726442048e+01,     3.7278242890583402e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.6145003253871263e+01,     2.1484454241734606e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -2.5169204201657891e+00,     3.1492199750387328e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.9100309851216110e+01,     8.8777215218312431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4693209514423220e+01,     1.0723878096975795e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.7052727219110967e+01,     3.2219324808984726e+04,     2.6931621671538551e-04,     5.9995954836692873e-07,    -3.0545685978306309e-09, } , 
      {    3.1955837921675162e+01,     1.6186480986167018e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7271542026127054e+00,     1.4356028897909491e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -7.7482348151362984e+01,     8.2712512100017939e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3275403669865696e+01,     1.5622736409735695e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4779180516827925e+01,     1.2382854048438596e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.9392008971579209e+00,    -8.9409995014449960e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -9.9870183427182440e+00,     2.3220431907613538e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    2.3442127557763285e+01,     1.5540620721390880e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2698700216119234e+00,     7.2161964339492597e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.8205611791382568e-01,     7.6474242181056252e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    4.2669190352591757e-01,    -2.3870342187211038e+04,    -6.4226121911122155e-04,    -9.7181663535885805e-07,     4.7399263974717103e-09, } , 
      {    2.3247614985566873e+01,     1.6768217864933904e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.4471802725740321e+00,    -1.7004003153066740e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.6937025884058400e+01,     1.6044031597339472e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.5257057770941998e+01,     8.8505810939441544e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    2.2995163105935564e+01,     2.9892883133806059e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5634202216326251e+01,     8.7337459735081684e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1040666516083371e+00,    -1.9126842355872112e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.9017069061706636e+01,     1.5533156354193183e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1098585654077580e+01,     1.4182527848576789e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    8.2471486064101995e-01,     6.2084820541342651e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.1241385272604898e+00,     1.3699780005333992e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5679352930437584e+00,     8.2719558419179448e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0752040049084993e+01,     1.6399789534780761e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0486115817254856e+01,     1.5055721970295355e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    9.3940606878962409e+00,     1.3795408918500887e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0577722068427622e+01,    -1.7353095606843781e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9413478630667012e+01,    -4.1505758739230314e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    3.3223259884822465e+00,    -3.6823038431593318e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.6265360056663656e+01,     9.3188683842230057e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.1041027783281905e+01,     8.3931190608865818e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6047613057453979e+01,     9.0033567073280792e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8282948741069085e+01,     1.3235500284559157e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0544144768598144e+01,     9.1726087582241489e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -6.0138772990729024e+01,     8.3931324879880322e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0278629207844379e+01,     2.2009064864600859e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.7872254692665948e+00,    -8.1874161592643441e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0031775918338070e+01,     3.6926218816359078e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0149145380693170e+01,     1.4286498149399509e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2884255469628427e+00,     7.2235053795968806e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.3738130352031163e+01,     1.6634076378038480e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1640838760317360e+01,     6.5590945737281681e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0285681955108338e+01,     3.9431120685394628e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    7.3514129721053294e+01,     2.2853719955901630e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -6.5470825814254612e+00,     1.6094035095171595e+04,     8.8286242385842573e-05,     2.4537365787631775e-07,    -1.2654121890639998e-09, } , 
      {    3.3536731657369572e+01,     2.8903166059601957e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.0186830948362626e+01,     7.1981685211118402e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7929085673915441e+01,     9.5284178499593072e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.6218043178553767e+00,    -1.6377092953551426e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9780370751312446e+01,     1.5300908719030107e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.6788909185764203e+00,    -1.7790035616713802e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.9929685244661943e+01,     2.3208193240550310e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.9480263104438489e+00,    -9.3838014068171433e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -6.0469150951818804e+00,     1.3800146778045921e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.6325447664458994e+00,     1.5544887858572709e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4792486464673132e+01,     3.4715997311727049e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.6140419646478506e+01,     7.2316881780820868e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.5896628079347153e+00,     1.9188975505744558e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    1.7878657656426117e+01,     7.6097136673675177e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3659306090535345e+01,     2.3897184849753354e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.9820759078010326e+00,    -9.2532895243327366e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.7154898867983576e+01,     1.6619843605928330e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7175005856979336e+01,     2.2632854752715957e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.2739404269320076e+00,     1.5486087311067220e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.2466427504668287e-01,     6.5290883013982284e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2272536402700680e+01,     3.3337307603626046e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.0334575535183145e+01,     1.5784599860933484e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.0429583982922409e+01,     1.8422860898689793e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.6454384584965140e+00,     6.2112725546008505e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0622257035474341e+01,     2.8119425320157396e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5222086103568884e+01,    -8.8127111989016048e+03,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.0036830633043223e+01,     3.0711632434593950e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.6081429961497779e+01,     2.1119819205451877e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    3.5092072462285515e+01,    -8.6730105285018763e+03,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.1685527871944487e+01,     2.3824546375951817e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {    1.0066017022015451e+01,     3.5050000242173023e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -4.3281799418518654e+00,     2.5779933223708022e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6513616797330073e+01,     6.1392492079911526e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -2.4021295491329340e+01,     8.3145716044505389e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.5590811938513156e+01,    -8.3428112447963613e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1207801899554296e+01,    -7.7128353590552160e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.6412973893984621e+01,     3.8746077517643585e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -4.5279010663996573e+01,     1.7880199545751366e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.5985404554657805e-01,     1.5562979606230328e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5528384438461558e-02,     5.5955783207132481e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.5112886651423427e+01,    -4.7270683475727046e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -6.2276252486729931e+00,     7.6692169849237771e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.5479560379596457e+01,     3.9392870096609186e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.1034766075519869e+01,     1.5541323397226850e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    6.0422864282861308e+00,     1.5544185182736741e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.4902115145697419e+01,     8.2081689482408474e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    7.6901262213457784e+00,     7.7713666227870872e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.1089172498888942e+01,     6.4680415430963030e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0202226538612173e+01,     7.0875061084249865e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.9163988110293637e+00,     7.5710012076547628e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    8.9897891494035473e+00,     7.7713876671947064e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.3446866163123596e+01,     1.4028615215323505e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5908981376452115e+01,     2.3623041188054423e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.6173575163925634e+01,     1.5462772170227896e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3142572109640660e+01,     7.6888938517211809e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0485813962279181e+01,     7.1765236220397783e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.1209554886527080e+01,     8.5378818608509519e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.0936549480699124e+01,     1.4399774008299404e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.9101605296138451e+00,     7.6893632128555428e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0017357379884970e+01,     1.5542091431717083e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4369791937273218e+01,     8.4460948716484982e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5580905666471772e+01,     6.2546267098775184e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6973405765691666e+01,     1.6568340600378047e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    8.8312246889234700e+00,     1.1099445669372826e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1128585156868846e+01,    -9.5572126763153153e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0363663792708838e+01,     1.3120868386522874e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.9995425981795549e+01,     1.6708699287862488e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0138723965056766e+01,     8.5443011731786064e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.4604235360888048e+01,    -1.8360504091659637e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -3.3126167622158889e+01,     8.5385999198685513e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.8660206475579109e+01,     1.5480187256981992e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.9724640906836896e+01,    -8.3196560307625587e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -3.1644415109014055e+01,    -1.4968328177012354e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6580492321201923e+01,     8.2304603878392791e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1291601421975535e+01,    -8.4269224614076193e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.7094229817961207e+01,     1.5543417148246506e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2609269783309252e+00,     8.9886381989023321e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.6497910723767632e+01,     5.7601041542549356e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8630824546623685e+01,     1.4281208087633113e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0984720986319267e+01,     7.7005322566351206e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6167544846447285e+01,     1.5695557612064633e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.4146565865424165e+01,     4.6056606930919170e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4737978496410769e+01,     1.6640315567707257e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1562939838993589e+02,     1.0466314650958742e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5717948535784503e+00,     7.0507463465545115e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2529446549018104e+00,    -8.5654708524228317e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    7.4282028826841664e+00,     1.1677348805830745e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3868931230225929e+01,     3.0157480029751634e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    8.4322688475145711e+00,     1.2669244513465742e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.0506550691554935e+01,     6.9889713713336132e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    5.8390052605280154e+00,     1.6689022038480241e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.2781899845582956e+00,     1.7616891626972611e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.3389622186094394e+00,    -1.3361756357448792e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3362293198366721e+00,     2.4409581800220672e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -3.1869043103300765e+00,     9.5776800110882668e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9034200911741937e+01,     1.6674450130786550e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.6154690308933031e+00,     5.9820308590681907e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4375182453221464e+01,     2.1938325082183601e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    2.3857416609942259e+01,     2.2135987736863655e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.2524770167713143e+01,     2.0584756998634570e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4362346245704227e+00,     3.2180949469472660e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {   -5.1619853704106930e+01,     2.6105971741502031e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1280887652927351e+01,     2.7155298883033713e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.2622174358280695e+01,     2.6519662788769587e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.3603199967915145e+01,     1.6653780053211980e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.5343221093640963e+01,     1.3831979292925407e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    7.2385603343544972e+01,     1.3704646313586893e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.9309102442887010e+01,     1.6660985645281820e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.7138438786082077e+01,     1.2791570752233301e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3245258943010612e+00,     2.4404152095182802e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.1673476598251717e+00,     1.9581996475437547e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    3.9246482207960028e+01,     4.2558367762130933e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -4.3020200422495193e+01,     1.9367956730441394e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3544892753128014e+01,     1.5294349763430972e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.0904952023767898e+01,     1.4901995185458944e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.4285276991753713e+00,     6.5878975185346226e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.3894631477691163e+01,     7.2222299083920125e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {   -1.1113006366383016e+02,     1.1722929821049831e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7851240641958711e+01,     2.3246951421372836e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.6990947527269256e+01,     1.5656165521232781e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.0443138610372723e+02,     1.2056991064521559e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2368877510930908e+01,     8.7130425262398294e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.3291074470623407e+01,     1.3701202696267745e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5177092605582345e+00,     1.6723350208249876e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7877029073955249e+01,     1.7197321462023461e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.5090963445368118e+01,     1.4915151336852401e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.9813782726225337e+00,     4.5562160820340068e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5495433687605246e+01,     1.4424322961107364e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1031624250821668e+01,    -8.1859934683305901e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4002756782001022e+01,     6.7983430146244427e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.4163205536945558e-01,     5.1779647114804166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.4756720010885171e+00,    -9.4646498918664056e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3363262155518710e+01,     1.4807068948128019e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.2069519924994278e+01,     2.2809074907084458e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.3386749991200571e+00,    -1.0246232145661010e+04,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {   -3.2462541891609611e+00,     8.3750768309814466e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.4755693783813371e+01,     3.1180680412047841e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.9461094006851347e+01,     1.3853498588349214e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    4.7325276656113296e+01,     3.2124886078157979e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2144761768961787e+01,    -6.1620813882611183e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.2007240585309852e+01,     1.5003488953805103e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.0280691665431156e+01,     5.9571190628924851e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.4457756781094786e+00,     1.7301689942516441e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.5451853092594874e+01,     6.4956505202702365e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.0329820634637176e+01,     1.7780829862852523e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9164476145119608e+01,     3.9119953663013221e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -8.7972496482545175e+00,     6.6824523656482725e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3666910692551326e+00,     1.4054249585347048e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.9553679061551716e+00,     7.3259011138659562e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2581518086890231e+00,     1.4926898224413560e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.5382481080249228e+00,    -1.9716638040640712e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    3.5910815332257968e+01,     3.0395067845045603e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2891046657667488e+01,     1.4332565756838900e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5500203256210039e+00,     2.4041180278443633e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.2248051912910794e+01,     1.5412242407497388e+04,    -2.8248174926264342e-04,    -1.9394636791347018e-07,     7.9853661415551646e-10, } , 
      {   -1.5860702188789311e+01,    -8.8173135987844253e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2952885247807643e+01,     1.4431928500326014e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.6217958482102709e+01,     2.4143150151046815e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.6269169628959634e+00,     2.5088245982798420e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.9940855481002817e+00,     1.0890937003479801e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.7601678794980341e+01,     1.4954263341831551e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0314820354693476e+01,     6.2215818358563374e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.4914186765703779e+00,     9.2632789304345388e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.6790698246396052e+00,     1.4927600900249530e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3768989107777102e+01,     3.1030151649777246e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.0681929466197548e+01,     9.7456707813711528e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1417470276371759e+02,     1.3799916197244986e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.7811007416419464e+00,     8.3936512555936879e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -8.5651010628955817e+01,     1.0777833989298228e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.0540933974396047e+00,     7.2766298983581473e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3040839153477677e+01,     7.1725932317427942e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.0621426465445660e+00,    -1.8294380799135168e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -1.2179878341401771e+01,     8.2588273697717195e+03,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    2.2012352978204177e+01,     2.6263352960218333e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.8025333609044782e+01,    -8.8948658597965477e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.5627381213142484e+00,     6.8651945240678879e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.0317892686430810e+01,     7.6159003038599922e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    3.6050937764190877e+01,     8.3318886546393169e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9588787217768740e+01,    -1.0535608474346303e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -2.5056263637242083e+01,     8.3254941946075833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7148316924693248e+01,     4.0357976298313442e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.0820585857097083e+01,     7.0878087385316630e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -6.5229024738108052e+01,     7.0200554279104726e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    5.5457993585110028e+00,     1.8635928471587777e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.7621986878514619e+01,     3.5566473600926889e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.1077553555652798e+01,     8.3954269743787172e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.8260896136039353e+00,    -3.7014820371764526e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.8097037253121506e+00,    -2.9972243912932709e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.7998361960094893e+01,     1.4102700985627796e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.0086664519852150e+01,     8.9968501956819182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4407586441808586e+00,    -1.6848898631042492e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    1.7542749550635971e+01,    -1.0954503652479841e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    1.4592836706088752e+01,     1.3197247710241856e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    4.8278788927150174e+01,     3.7373844095051390e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3188152255473270e+01,     2.0057500738400249e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -1.1356252639396963e+01,     7.2827549708299248e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -4.6556995088349240e+01,     7.2032965030202158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9216544230761471e+01,     8.0027346370605046e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9112716714108475e+00,     8.3778940363403271e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3709355128587612e+01,     1.2439591704266140e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    6.2520585507032692e+01,     1.4954259871836775e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5814806059715803e+01,     5.3391295163235181e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -7.1548068868415058e+01,     8.5382476039104768e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.2383355733116758e+01,     2.7795096455935993e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    9.5412939176950058e+00,     8.2795250970215839e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5822063769225407e+01,     3.0964851877115463e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    7.2539544099985633e+01,     3.7671616176234471e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    4.4752334921030766e+01,     1.5782971937591945e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2007188164389930e-01,     8.3192789795504232e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -6.4944456553991614e-01,    -8.3381038696964770e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.3597416602404266e+01,     1.5480122467329067e+04,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    9.6801476234002983e+00,    -1.1910698026759687e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    1.4974458206514646e+01,     1.6310762545427060e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    6.1240034269116862e+01,     1.3590620995892919e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.1021833521751425e+01,     1.7226821282996021e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0653222933625239e+01,    -9.1888885442126502e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.3311708551038764e+01,     3.4465442674162499e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -1.5730459887598901e+01,    -5.9290959728499729e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7531296913475387e+01,     9.3065838252813955e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6920019183155102e+01,     2.3712538422971669e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.5815227616890873e+01,     1.6408220317937572e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -9.6991737006605359e+00,     2.1915313082769480e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.7175274866834222e+01,     3.2289154652843481e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.6555082604647183e+01,     8.2514914795694003e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4192598494227230e+01,     1.5613395279436709e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.9313184337137031e+01,     1.5541126366640252e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -2.2236131833331029e+01,     1.5544382213323333e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    4.3849255340816029e+01,     7.2447310472321951e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    8.8665775061794321e+00,     2.1593501989775628e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5679768341282774e+01,     2.2762850629782890e+04,    -2.5374418269542669e-04,    -9.0079351577513944e-08,     2.4536420400953609e-10, } , 
      {    1.0470949850008772e+01,    -1.1022267086602440e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.0292595001143296e+01,     2.3941335492624028e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -1.1313762371374404e+01,    -1.0359651578703435e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.9054751706583684e+00,    -1.1348906255282080e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.0490211901682422e+01,     1.4915550232330463e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    5.2778216505509967e+01,     2.7098096765418864e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3730255067159689e+01,     8.3347953941741853e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.2938877288258544e+01,    -7.9059314078336829e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.2815837811467503e+00,     3.9459421809291744e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.0274947197813776e+02,     8.2247887472181428e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2045812033429449e+01,     1.5681343819998878e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2645845772540284e+01,     4.0451276302840022e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    1.2153550400729504e+01,     1.9910480256576404e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2586847819755405e+01,     9.3898616896359945e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    5.9573963399205550e+00,     6.5360918828776630e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.6399443044687576e+00,     1.9705949402322814e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.5759771475946888e+01,    -7.4171881629652808e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.6911104600450358e+01,     5.8380029895735215e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.8422363326051165e+00,    -1.8263248155474008e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -6.7178508736018102e+01,     1.6115610597902780e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.1515282818941177e+01,     1.2933609372472016e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.1829708408190083e+01,     3.8692592415014922e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.4550138439926819e+00,     8.3778577522253163e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.0655137142327860e-01,     1.5239871450025146e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.3473007636437821e+01,    -1.6657701652898402e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.9631780203070321e+01,     8.5382341768090264e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    7.9527051173637389e+01,     1.6654995016271751e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.3933388099464850e+01,     2.4285136761872796e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.1785479762746039e+01,     2.3378124863541867e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1215417642239252e+01,     2.1408728867460795e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.3494936378548486e+01,     7.7158136934705299e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.7953832422980803e+01,     2.5109291402407634e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2894544795875769e+00,     9.4367085843364512e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2700274914688670e+01,    -8.1560366319236218e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    8.0030223588520801e+00,     6.1799830379005539e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1213874149150387e+00,    -1.1378884913127913e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.7268863974253897e+01,     1.1961819232437305e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.3822403504559929e+01,     1.6659770682222050e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.4719906855488336e+00,    -9.5726873562965484e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.6068472386062780e+00,     2.3331052115780287e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {   -4.6124271451523970e+00,     5.9176059849932444e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    5.1524785982193135e+00,    -1.6064020153210748e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -1.0458389982443059e+01,    -5.7713616185095398e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.2796437612497932e+01,    -7.8942008213149093e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.0155993898703407e+01,     5.6396777866449829e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.8834535245223691e+01,     8.8943671870162689e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -3.7951689916353303e+01,     7.2951289157254905e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.5791285662824251e+01,     3.2065150405246532e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.8925469347616215e+01,     7.2248292276964730e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0723277791558504e+01,     9.0066624071041315e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.1641674922077354e+02,     1.8795006075582194e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1835673189078477e+01,    -6.6864796809983727e+02,    -1.5446941850967620e-04,    -1.8325593535679623e-07,     8.5739299504221421e-10, } , 
      {   -8.9709002379818841e+01,     1.7029528026139302e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9975976336066066e+00,    -9.1485242672034965e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    9.9195772677864387e+00,     2.5055904117729111e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.9136353780906376e+01,     1.8924838495194432e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0427361659005600e+01,     1.1727753254418369e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8138996052521446e+00,    -8.9701169208672582e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0505918903892029e+01,     1.1863578951918927e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.2379596488435185e+00,     4.2659815955703628e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1091050954021199e+01,     7.2735661829146084e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.0179287095041573e+01,     3.2319581844922432e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    2.9472948942974828e+01,     3.0363900990391474e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -4.6653775126474635e+00,     1.1959164896871760e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.8995965206704151e+00,    -1.8657350853805985e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.3909552919047592e+01,     2.9451291551319191e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6501496762221141e+01,     1.2126771779379280e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -3.6492736923237274e+01,    -8.3121829939339550e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    4.0909109717565883e+01,     3.1496578493112436e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.9843374689984241e+00,    -1.1184418510782409e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0698712422149395e+01,     2.1136739886259467e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.2942302815372699e+01,     1.5471408668610731e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -5.1192345768454297e+00,    -1.1426458516294204e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.8802881087446487e+01,     1.8981727239949665e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -1.4211076108184574e+01,    -7.9720882760194763e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0430514399384935e+01,     8.5923639733862638e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2362725989703751e+00,     7.2482964089008601e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.0227096199240769e+01,     1.5538599204414393e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2505335471317974e+01,     8.3052192784237777e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.6526337169545755e+00,    -8.3521635708231242e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.9715575337591723e+01,    -8.3755907632768758e+03,    -4.9214582209624539e-04,    -6.5363482240634487e-07,     3.1253513952726233e-09, } , 
      {    7.1206385868535605e+01,     1.3998892655160707e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.7192390025320872e+01,     1.6657064045595398e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0195427002436634e+00,    -2.6091601770249317e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5585228705378908e+01,    -3.7539243318568836e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.3905700875738489e+01,     3.0459278537701743e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.6693118002560510e+01,     2.3822743848790684e+04,     2.0748706208625195e-04,     5.2715139329073324e-07,    -2.7054057846955432e-09, } , 
      {   -3.1141618738596200e+01,     1.0635741350442768e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2211579831332848e+01,     8.1824223228856845e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2534644183310384e+01,     2.9574461970196600e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.8331729355933586e+00,    -2.4668037032339826e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.4635453400041222e+01,     2.1053695912927407e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    3.9734138714744887e+01,     3.7702430714930524e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.4607660896597551e+01,     9.2080181090208480e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.1156677776372316e+01,     2.5914102824082307e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.9397249995943973e+01,    -7.7176568363435736e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.0889810653915493e+01,    -8.2070396976795910e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2758039387876501e+01,     1.2917964467697835e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2076602591962036e+01,     6.7615385781090972e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.0753397228738308e+01,     8.3394928763651587e+03,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -5.2322913675922123e+01,    -8.3178899728817451e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    9.2695573157869990e+00,    -2.0300378710590744e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.5439418924480936e+01,     2.3716341192580996e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.9831351987231809e+00,    -2.6609752884208015e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    2.4367122793718437e+01,     2.7323476641611356e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5066827623579197e+01,     2.1589219498863844e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.0183291217232302e+00,     4.9334893082125436e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7297557865936710e+01,     1.5546909375549198e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2581767343382850e+01,     1.6656918387459595e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.6301141795425437e+01,    -1.6657847311034206e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    4.6227822511183714e+00,     7.3362103951214431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6370575937453498e+01,     8.3340527076753788e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2950241201136716e+00,     9.8750381537791473e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    5.2443971562143776e+00,     3.3903258381641426e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -9.2779828305080674e+00,     1.1018018435461538e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.8692137621224996e+00,     5.2784979361189880e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -4.4206290882080978e+01,     1.5531987927643668e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1557534739839234e+01,    -1.4335339658485547e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.4406674532328395e+01,     8.8289178055924222e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.1258891028868555e+02,     2.3242630170920584e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {    2.6007658139800711e+01,     1.6715258243461623e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.3356274836500663e+01,     1.4460940755770787e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3775152705119469e+01,     2.4375790897026603e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    2.0069319272365039e+01,     1.1231013611215660e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0991046953185744e+01,     7.7191193932465821e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.6150684440206469e+01,     2.0880754555744054e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.8996489947349229e+01,     1.5978373155938605e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    2.6847935695729653e+01,    -8.9111506333283305e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -7.6421572275474972e+01,     2.8230187100444182e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.3327277566830141e+01,     8.9289606601113301e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.6668133266326961e+01,     2.7630803145996422e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.1481780879866195e+00,    -8.9378208156518740e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    1.4880361321079977e+01,     1.5542465085651569e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3964415392753633e+01,     1.6663443132065753e+04,     1.8102997432954297e-04,     3.5458589049061093e-07,    -1.7891564087666313e-09, } , 
      {    5.8952559462436307e+00,    -1.0123442680329968e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.2196559210110923e+01,     1.5543043494312020e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4574266274308059e+00,     7.0169649190993514e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.8234629931897661e+00,     1.5523576095964674e+04,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    7.9488196161201641e+00,     6.6011331608622831e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9044374335223413e+00,     1.3521751439688769e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.3228791938915485e+00,     9.1238886594495907e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -3.9361305446247215e+01,     7.7530006208310233e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.0085029087318425e+01,     7.1721704206767317e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4859977228814174e+01,     1.4285780090381908e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.9863843723368774e+01,     1.4915258750576148e+04,    -2.2511008419136763e-04,     1.5023212424749856e-08,    -2.9229416834093930e-10, } , 
      {    2.4589627117849773e+01,     8.3619741792983295e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.0663877187057224e+01,     9.8215777269017963e+01,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    4.5677287682083126e+01,     1.0184763784562516e+04,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.1331969735147382e+01,     4.3358499050408500e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.9345567086749774e+01,     2.0011100072527101e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.4501899361701156e+00,    -1.0068997736617912e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -1.0643145388187115e+01,     1.8695036439969517e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7472347452308025e+01,     1.6470782840193751e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -2.2553030168196511e+00,     1.2955900812070449e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -3.1991324143338815e+00,     7.3107599506938150e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2369415660291580e+01,     1.4964110102371391e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.8372030330220863e+01,     3.8788522251857677e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.1486687419524650e+01,     2.1742047556200745e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.6319087232846616e+01,     8.3343725831081265e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -2.4171788987210014e+01,    -8.3230102661387737e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    5.4144065832658697e+01,     1.6170702845518537e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9569813505154467e+00,    -1.0361359127998709e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.3911143218991345e+01,    -9.9067563237983336e+02,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    2.4980792016576604e+01,     3.6520586884607692e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    3.9341258595843591e+00,    -1.0596645743351264e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.6892008408800088e+00,     1.3958051172791858e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    5.8645019450236706e+00,     9.2467305687336047e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    6.3787658050647185e+00,    -9.7755772118156665e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.0953463419951170e+01,     1.6602122027833204e+04,     2.4057865014816875e-04,     4.9609253203097252e-07,    -2.5013961876846506e-09, } , 
      {    3.2719400443730869e+01,     6.7277571506539371e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    2.7101146852440788e+01,     3.9435529912564889e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {   -2.3258658505375614e+01,    -1.6746425098115753e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.5872877451825071e+01,     1.0556305360296970e+04,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -8.2991130929023456e+01,     1.1740202817815498e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4031325612461819e+01,     2.4656333668175896e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {   -6.9701856557398997e+01,     8.5200014911930211e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.3353325785075967e+00,    -1.7183529805097162e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1264782357249583e+00,     1.5033539094671913e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6177097879425098e+01,     7.2139238895198705e+03,    -9.4817274627892635e-05,    -4.2984841482742304e-08,     1.2963917832868989e-10, } , 
      {   -2.9118263321688111e+01,    -7.2142018411968211e+03,     3.4213389314442701e-04,     3.3421746178752410e-07,    -1.5262904308690409e-09, } , 
      {    3.9394508704182005e+00,    -8.5705369955751394e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    3.8883867909535921e+00,    -2.0769973877823768e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.8995711023297119e+01,     9.5610337211050537e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0686117394185118e+01,     2.0951524203999335e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.4091852949291273e+01,     3.8222019184309815e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    1.0401779014368714e+02,     8.3314193085134721e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.3023495735733945e+01,     8.3259635407334263e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0605347147544716e+01,     3.0128614375038916e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.8400496850902499e+01,     2.4406453295124214e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2699891395241728e+01,     1.5201548431684678e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    2.9246181844973986e+01,     1.1503512104526046e+04,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.6977025653770212e+01,     1.4308180373335435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5989197433047636e+01,     1.4669971301668378e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.4984554591171854e+01,    -2.9199215106059110e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9293915390886532e+01,     1.4803113016468968e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -6.2297972768998235e+00,     8.3040835255684560e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    4.5499264387787861e+01,     9.5564561995169715e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.3369467450921947e+01,    -6.5551023579541061e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.8415336875629556e+01,     2.5445167935053894e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1441773922198079e+01,     3.8749488755161829e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {    1.2757840585291733e+00,     7.2089422927212090e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.4945757218816240e+00,     1.7929997527561984e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.9309610148478423e+01,     2.8873172142240976e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {   -1.6669039892304312e+01,     1.4624436648370101e+02,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.5876280555401785e+01,     2.3899779204062532e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    2.0441791731272559e+01,     2.4022621113346544e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -8.5198602963121903e+00,    -8.2789104889442915e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.4365128411062122e+01,     7.5798613631953049e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    3.8115846280496051e+01,     1.0376279566358109e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4898895003972591e+01,     2.1611535673716331e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -1.1861818215708576e+02,     1.9717560002201550e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1580889555210897e+01,     5.2965221443207865e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.6051770923344435e+01,     1.6696080049444503e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -8.1425137127835811e+01,     8.3230110858048320e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7732459854626342e+01,     1.5500861845300180e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9612016573229610e+01,     1.2571739997918005e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8613180232631508e+00,     6.7025605553437049e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.8606130409399288e+01,     2.3051622347764111e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    9.2419421386817618e+01,     8.3343717634420682e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9688558975447879e+01,     1.5988734881147062e+04,     1.5011539701497616e-04,     3.1818181295251318e-07,    -1.6145750021990875e-09, } , 
      {    3.4264328012481116e+00,    -1.3341674311032102e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5143706376723131e+01,     1.6544938020396978e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4724094070181977e+00,     1.6627370850363764e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.9332675140936686e+01,    -1.1148342744911270e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    6.8967836517460448e+00,     1.6134302113156864e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8680544791008206e+01,     1.0102280491292797e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.5973687647393922e+01,     7.2398000149745985e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.2647594101475256e+01,     2.8191895459984236e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.1037875096114487e+01,     2.0321139076170090e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4901710289910874e+01,     3.8786603765008367e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.4060227326777817e+01,     7.8139845169936289e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2581240549517041e+00,    -1.9808525919793883e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.4924911901479909e+00,     1.2318404439109587e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6227251486750109e+01,     7.9755121652044763e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.2193201778303315e+00,     2.2410695524300580e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -1.9487616897794737e+01,    -7.3691826468977415e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    3.3109690534263770e+01,     1.6703231225587566e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.8904377451554286e+01,    -2.7024449220363549e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.6829051843737563e+01,    -1.6611534472906234e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    8.0155161022111265e+00,     1.4973965282007666e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    7.4926454948971983e+00,     6.8497456396011671e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.2595387136185352e+01,     1.5944591728483443e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    1.6609414449770599e+00,     6.7615832527539687e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0490298313708227e+01,     2.5571722042960188e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    2.2532707051979777e+01,     1.8118133039551569e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -2.3191438722871474e+01,     8.2642418152944410e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.5441766161848703e+01,    -1.2505816956483345e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    3.1285468549231595e+01,     6.3347361809609483e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2243155122585975e+01,     1.2043574183467715e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3648465891522036e+01,     1.9244236327158251e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -1.8307632389353898e+01,    -7.6778419344456615e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5804303206856423e+01,     7.8177872866029547e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0457466204345973e+01,     3.0941220769196196e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -4.4070807598720236e+00,     3.2215869623227994e+04,    -1.0988521799907885e-05,     3.3855024165560006e-07,    -1.8774409800966779e-09, } , 
      {    1.0785923945974091e+01,    -1.0238517218714696e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1068919269830495e+02,     7.0046114511958822e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -8.5412651070246994e+01,     7.0044153480482037e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.3460449587852096e+01,     1.6487850121733398e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.9408320879983769e+01,     1.7414559133920156e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    8.7146091628917688e-01,     8.5380172332020411e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.2122362637452433e+00,     8.5384645475174621e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    4.2344702397473974e+01,     1.6941371330414015e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.3208692477189583e+01,     1.0857629045523875e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1187811320510027e+01,     1.5543227810736304e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5889241183295972e+01,     1.5542280769227285e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.1921178343373995e+01,     6.5761573445241893e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2977344740681360e+01,     7.7803659593318062e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5294398995813070e+01,    -1.7068877279070509e+03,    -7.4153598329697204e-04,    -8.7864005157957579e-07,     4.1278976064390322e-09, } , 
      {    1.3189885964577753e+01,    -3.0141450721008650e+03,    -3.0893883701935240e-04,    -3.6651187071359246e-07,     1.7147859900844284e-09, } , 
      {   -6.4659894165039979e+00,     2.2065001222074284e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.2889385994289206e+01,     8.0627366273970456e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    6.0788472554573190e+00,    -8.1790013481219239e+03,     3.1121931582986021e-04,     2.9781338424942636e-07,    -1.3517090243014970e-09, } , 
      {   -2.5211430620614166e+01,    -8.6852523265569107e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.5549159658507225e+01,     1.0804211214067516e+04,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    1.9553260446006011e+01,     1.8640521381652550e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -2.7241557631022069e+01,    -1.6712643670660589e+04,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    4.0059337121904264e+01,    -6.2631822652725532e+01,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.3182300103571594e+01,     5.9612609870803926e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6218567835762197e+01,     3.5109018281420038e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.6227391949795766e+01,     3.9114479039420621e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -2.8085391033403187e+01,    -1.1123510429069970e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.2959406314691320e+00,     1.4273519881472723e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.2478509531461881e+01,     1.8182567389365424e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.9835255247858896e-01,     1.6162425376644504e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    1.0033771083635965e+01,     3.2764010374460380e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    3.0584138811525555e+01,     6.3685176084161085e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.9759900091850128e+01,     1.1033890369421893e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.6224218789602620e+01,     3.1571118208716278e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5368038113919424e+01,    -1.3546996519158158e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    2.0484761528064389e+01,     1.6504564267648911e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    2.2545660089728560e+01,     1.1335788344083810e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.4300119842693183e+01,     8.3390284145778478e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4761454721281115e+01,     1.3039513355901425e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.8694024398203005e+01,     8.3493615021980149e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7698864548016481e+01,     8.3080213470488852e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.7956703825702851e+00,     7.1770928775182238e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.9615810330424893e+01,     6.6567418721750655e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {   -6.1564037819885172e+01,     6.6567552992765195e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.0472020422110404e+01,     1.8905767703373116e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.3084602234334966e+01,    -1.8254194375803654e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3721359130355490e+01,     1.1168955482135432e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.5912778586777989e+01,     3.0388508889446464e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    4.8542333590398151e+01,     9.1162586172606425e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6152893648372725e+01,    -7.3434454972814892e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.2798321177782613e+01,    -1.5806906560047071e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -1.8131116438547899e+01,     3.2374938775108771e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.8475368809204024e+01,     6.8402200896824779e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.4403590359876862e+01,     1.5152363152604994e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7410404184589694e+01,     3.8943626773881930e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.7305147222815318e-01,     9.5206055253141039e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3562299979007690e+01,     1.7298309672710431e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {   -7.1157944930039491e+01,     8.3416304289717973e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {   -5.5393699200265196e+00,     1.2489885864648156e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5393455403453869e+01,     1.4456067038246645e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0548217425370831e+02,     8.0327141145723963e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.0615806817630062e+00,    -9.3631378905602469e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.2395081754561904e+01,     6.0602828188539206e+00,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {    5.6938536162885455e+01,     1.5441548983454790e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    2.6297733967886387e+01,     7.2063746591979525e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    1.6775262424321564e+01,     1.7762188989855294e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    6.5916765513560902e+00,     1.4580944312314679e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.8135149055289119e+01,     9.0340525979008362e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    8.1141878359158923e+00,     1.1403676944132160e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.8993104418537598e+01,     2.0323440276111502e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -6.1486947583228186e+00,    -1.6662812554284774e+04,    -4.6123124478167859e-04,    -6.1723074486824723e-07,     2.9507699887050792e-09, } , 
      {   -2.3721068534208499e+01,    -9.1508159914059961e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.7296742802087060e+01,    -6.3509673377405252e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -1.0112707576187736e+00,     1.5543297881316799e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.8088323261424776e+01,     1.5542210698646793e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.2505267480674256e+00,     2.1954157569150248e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3289142527736407e+01,     2.6189720346750240e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.8631252243416604e+01,     1.4296128525992512e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    2.9371020705140740e+01,     2.1695157693286230e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.1022269107878579e+01,     1.7169442958614571e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.2711990008163660e+01,     7.4800176625847507e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.5740166356758365e+01,    -1.1169060756232134e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.3402400992720196e+01,     2.3242779100050899e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0863967862768202e+01,     1.1126238878347509e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -1.5724682781456977e+02,     7.2053320200372909e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.4766686350851685e+00,     6.6214115147890707e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.7577193908246013e+01,     9.3473835857281265e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.9304613442451739e+00,     5.5617968932580870e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0276652498100081e+00,     7.1404665360052732e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.2397518707670944e+01,     1.4041663716622836e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.4387755132542441e+01,     2.3883443325348715e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    3.3529123635653368e+00,    -8.3113785618885286e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {   -2.4356462360213271e+00,     4.0764540634135774e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.7489468585135590e+00,     7.7003733173740984e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2143974926244411e+01,     1.5401119995113890e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1224804110479507e+00,    -3.7498728575227688e+02,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    1.7090152127420382e+01,     8.3302470681050127e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    7.1829668023615739e+01,     7.1918732761841748e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.0038122267458803e+01,     2.4515295327939217e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6477309645633532e+01,     3.1098127818381578e+04,    -7.0537197618533671e-05,     1.9704360011523847e-07,    -1.1652012011786588e-09, } , 
      {    1.1393610054798883e+01,     3.1072889341545604e+04,    -1.9419550687680085e-04,     5.1427289962847581e-08,    -4.6687557490848320e-10, } , 
      {    5.9492552816555644e+01,     7.2137676026773461e+03,    -1.2355484119510939e-04,    -1.4685185781869851e-07,     6.8281158847467036e-10, } , 
      {   -3.7126609413523241e+01,     7.2143581280393428e+03,    -3.1339632657721023e-04,    -2.3035044545156790e-07,     9.7311802072306041e-10, } , 
      {    2.4474240961349590e+01,     1.2339083214881632e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2423433945490910e+01,    -1.2685703518701477e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -4.3973705967734844e-01,     1.6583786519893827e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -8.0657027953812133e-01,    -2.9426625696465053e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.0898671066906505e+01,     5.1489635379455831e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -9.9569153159955803e+01,     1.2096805966419472e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3764183567691290e+01,     5.3268389102803048e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.4135057376498299e+00,    -1.6217961213680765e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -4.0026843835411725e+00,    -8.3298454104387692e+03,    -5.8488955403994576e-04,    -7.6284705502063811e-07,     3.6490956149752546e-09, } , 
      {    9.5968612854089148e+00,     3.0235032290763760e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.3338922832144334e+01,    -6.5574035578955154e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -2.2124214824799751e+01,    -1.6910718408055312e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.3186199121276644e+01,     8.8487785667830194e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.5626798768951913e+01,     2.5032474939743497e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {    4.6350534171383746e+00,    -2.4939673607997200e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    5.6275183841326538e+01,     7.6074361510094068e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0076319860668328e+01,     1.5910038110058531e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -3.1063732563837636e+01,     7.6067315190932577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0537230773545790e+01,     8.9571474267262056e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0670345566033383e+01,     4.2951388731823921e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    3.1246312909199020e+01,     1.0979979252348654e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    4.9687932571661495e+00,     8.3010168366800681e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.2012576627305069e+00,     6.4154016875543821e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.7961243203112568e+02,     7.2227937106793979e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4411744820202825e+00,     1.5505784302141674e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3929939495387931e+01,    -1.7391228524975136e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0069540939698530e+01,     1.6561781644778916e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6835055249839677e+01,     2.5064071160762775e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4926794319176754e+01,     2.4510062580906277e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5280999255117822e+01,     7.1648965377564782e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1280523252564040e+01,     1.8592947778486257e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -5.6072081299708920e+01,     2.6400218083075844e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.7441791150410268e+01,     3.2367097573119700e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.1338251274501584e+02,     1.1094622236004288e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.9162822333306977e+01,     8.3197026102825403e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.8152680950965614e+00,     3.0863339875809303e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.7812340077932767e+01,     1.5258765808814682e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    2.2547906556059843e+01,     2.8306010069695396e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    7.0081205446445702e+00,     6.2457034078310053e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6894864138112133e+00,     7.8236348967352124e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -7.8560283434643345e+00,     1.8009912596665596e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.6520307645391725e+01,     1.5605386112634518e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    6.0255869730159581e+00,     8.3563660125668321e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8005667565590578e+01,     7.0781097289751160e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.5320972420964779e+01,     2.8492096146453110e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.0862922555707044e+00,     1.1566143927178773e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.0897367150522768e+01,     2.2697313837783873e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0759337528629898e+00,     8.2995941457463159e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8292932702339257e+01,     2.5290726271294356e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8286621007781605e+00,    -2.3686138970400089e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0782434237380848e+01,     1.5715907755461907e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2359862702533604e+01,     1.5650974732867769e+04,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    4.0985295654429905e+01,     3.8531460979865595e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    2.7554753924855369e+01,     3.0048432281710611e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.5464031933342875e+01,    -8.2112974533373126e+03,    -3.0676182627200238e-04,    -4.3397480951145098e-07,     2.0933769936628650e-09, } , 
      {    1.3996644039234408e+01,     8.3369034214030344e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.4613794054084401e+00,     1.5054166326813795e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.3749340511657406e+00,     4.9088966686727008e+03,    -2.1770107473500209e-06,     6.7462938797858489e-08,    -3.7859100357843662e-10, } , 
      {    2.4804759553498091e+01,     1.3038384105340167e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.9549947749280099e+00,     9.0863474305991676e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.8922527699437177e+00,    -1.6140408952668640e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {   -4.2743166289144817e+01,     2.5517479058354402e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2453688915038029e+01,     7.2559553100399571e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.9413958756153846e+00,     1.7425375845177012e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.4442594774571401e+01,     3.4501491090566102e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    5.9076116973043767e+01,     3.1731089161774802e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    3.7363692449435206e+01,     1.3189559504081464e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6466666811178346e+01,     1.2592403514174897e+04,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -4.7190814627538941e+01,     2.5811725399928218e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2575451151980026e+00,     1.5553072328458373e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {    2.4819507388607999e+01,     1.5532436251505220e+04,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.1586352884306542e+01,     2.2799476971295844e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.2160596957022761e+01,    -1.0869669717818031e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.4986247555639736e+00,     7.9640221773477497e+03,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    5.3453458671206788e-01,     8.5542403985837500e+03,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {   -2.9676152550339026e+01,     1.5335885148992338e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.7202157848166006e+00,    -1.6730979178599970e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    4.3824797194147976e+01,    -8.3203299418631832e+03,    -4.2813965671976176e-04,    -6.4828960612800795e-07,     3.1547795857159722e-09, } , 
      {   -1.0614164796897137e+01,     1.0624997465768549e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    3.1341849765310350e+00,     1.5765421666916363e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -5.0117499942517547e+00,     1.8795954536051569e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {   -2.2310540062417189e+01,     1.5543315439902108e+02,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.6861448393021234e-01,     1.9390480693641948e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8256925150157194e+01,     2.1655306507273301e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.3226695762416476e+01,     7.7007770858403483e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.3721964900215525e+01,    -8.8159788012440767e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -2.4560191368289999e+01,     7.8531871616607441e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.8472505472091264e+01,     5.7875394214725929e+01,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    9.8917641827607738e+00,    -8.1124058086084242e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    5.4825367339578657e-02,     6.7592563296455046e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.2489281494611211e-01,    -7.2025881370167690e+03,    -3.6859098090113594e-04,    -5.0678296458764636e-07,     2.4425398067979527e-09, } , 
      {    2.1046901943192147e+00,     7.0668702066521982e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {   -1.0034690690614298e+01,     4.2150102237368883e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    4.9105752217042209e+00,     1.5193885948691337e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -4.0811392612462924e+00,     2.4044912552944974e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {   -2.8915909800151738e+00,    -8.3460053516934840e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.8116070888491297e+01,     2.2740817647116142e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.6035345460780270e+00,     2.9683844597476054e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9145484688472330e+01,     2.0508432301213925e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.0897252093177045e+01,     7.0612595432755079e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.4337838079436835e+00,     1.8402933566358581e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    4.8782423788700818e+01,     2.3842595319407683e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -5.2145109488158461e+00,     2.3871800318055059e+04,    -1.6328092956223407e-04,     8.7831367500945312e-08,    -6.4145698147602705e-10, } , 
      {    3.7002672553395456e+01,     2.3871091111155427e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    2.3304633678846034e+01,     4.1707891472931107e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {   -5.5368868989029973e+00,     7.1032546580474418e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.7179299233626182e-01,     2.3234501631176867e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.9403138504379193e+01,     2.1179081705477747e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4581867943726124e+01,     1.3799489513296376e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -4.6645929654254799e+00,     4.6544318593030248e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.8117105894936280e+01,     1.7035359275841893e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.8862271839646398e+00,     8.0170254855519124e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.1714214605198009e+01,     1.6954802614471904e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -5.9151573620685198e+01,    -2.3888941138747350e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.3804820639358752e+01,     2.8188117425628978e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    8.4395293044474595e+00,     2.9055969381684794e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2906924844089641e+01,     9.2420682912480879e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    7.2919925450539882e+00,    -3.7384277567586776e+02,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    2.7643559685881655e+01,     3.5502039251113027e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.6785667150126178e+00,    -1.9132237849917012e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    6.1081495034309938e+00,    -1.7929745762852439e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    2.7477689339919227e+01,     1.7894321152540586e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.2430239318265190e+00,    -3.6463503628272872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.5140248912459917e+01,     8.0802934593902992e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.3570732465276102e+01,    -8.5770893898566028e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    3.1734420647934442e+01,     4.7806859861297053e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3131655553449711e+01,     8.4629898292890393e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    4.0247297446994557e+00,    -2.2833297431356082e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1441382328470615e+00,     7.7638989784218402e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.3013704007644247e+00,     8.1993430036932768e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9597194442568409e+01,     1.4291070152148299e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.3820494758550534e+00,    -4.6698509704427024e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.4319154940700635e+01,     1.6488831783649613e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5375339776092778e+01,     2.6308390626558236e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6777426649866509e+00,     1.2562879842377461e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.9164942423722703e+01,     7.2510328531984651e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7628092708008744e+01,     1.5493587962379930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.3139084990256400e+01,     1.4141438092054968e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.8652792767469279e+01,    -1.2566084565403598e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4036251946018810e+01,     7.2838927092171043e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {    8.4086881121143513e+00,     1.5605321322981597e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4458747145104297e+01,     7.0640989454915712e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.6763454903275914e+01,     6.5833674473376504e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7750117750930691e+01,     8.5449770406384741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1178578102005217e+02,     7.6983483225608315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1661227608854791e+01,     3.7201863566864420e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    2.2252645762380321e+00,     5.4700231936534901e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.2291454735341745e+00,     1.7270734101154329e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.4430107874307687e+00,     9.6628588557266594e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3851808469966160e+00,    -8.7913986220583752e+01,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    5.0858815347266237e+01,     1.8611670986454981e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6673898367256665e+01,     3.0856780920210163e+04,     5.3017643576575737e-05,     3.4389545793393698e-07,    -1.8480127896533289e-09, } , 
      {   -3.2242208591726254e+00,     1.3748501707055557e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.1442300119956421e+01,     2.8523263001107232e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.6019719684166120e+01,     1.8909090751679985e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    3.2754659525990718e+01,     4.7288708747338351e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.3803466534420403e+00,    -1.8034529971459502e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.2740481746336416e+01,     6.5836130420909813e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.3511929571637079e+01,     6.6049334743299223e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {    1.5814509920448158e+01,     6.6574599311926695e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    1.5085688169813910e+01,     2.4271025639897365e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    8.3189330205490055e+00,     2.5770962227440999e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    2.1402642203391434e+01,    -7.2145615381386233e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.2246083899400539e+01,     6.7687605636937548e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.4965636220488108e+01,     2.9296187029294939e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7215316809499754e+01,     2.9316200956728153e+02,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    2.3474245139070558e+01,     2.2725177966106799e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.6851703167165262e+00,     2.5868813298591798e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1328179086142896e+02,     8.0985328585570296e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0107739307018065e+01,     2.2730508764713584e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -4.6630424407923243e+00,     2.3618110155796829e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -6.1146371633132084e+00,     1.3565030203098870e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.2926120357237789e+01,     1.5231088350910255e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4283605098111831e+01,     6.1217743749005967e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    2.4264832709130335e+01,     2.0145462192656611e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.6743994890622904e+01,    -8.9344571045367484e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.2069979690053616e+00,     1.4303059800698300e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.3601908600821401e+01,     1.5917741575734075e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {   -8.1925134386748894e+01,     1.3799720094097307e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.6503782246240007e+01,     2.0633977952534769e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9533654262749991e+00,     6.0945185333507961e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    3.3592476698392471e-01,     6.5600161451646918e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0937067906713585e+01,     2.3747492787719973e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    4.5887170720795698e+00,     6.6199888238553176e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -4.5913660346546791e+01,     3.1140865510149932e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.9465462213188104e+01,     1.6655083384302878e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.5170865721914907e+01,     1.6659682314190926e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8983724601432648e+01,    -1.5567362189036790e+04,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.2013471843274548e+01,     7.5819055362852332e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.6051923839083727e+01,     2.1531368839903182e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.1504128555502531e+00,     1.4408997995120684e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.5961020730094894e+01,    -9.6181216197686317e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -5.2487923384610209e+00,    -4.0945030068694405e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.0230915040772306e+00,    -1.4913343177542043e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    7.0953091232007068e+00,     7.7397379557575578e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7232761278820998e+00,     1.6344543972882220e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.4628657941091564e+01,     6.0587308533837302e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.2343035082767486e+01,     3.1889599462805425e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.0835659702175064e+01,    -3.7575549042300463e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.8594980428747938e+00,     2.6762277913340724e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -1.5739606553924141e+01,     2.5061325334941062e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    8.9187316357570143e+00,     8.3577887035005842e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.5517103666898002e+00,     1.7818396814870001e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.1597531986654452e+01,     3.3303948987236945e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9475697345869115e+01,    -9.3680689228178453e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.2069180771837376e+00,     1.5652976960847818e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    1.0256654888206626e+01,    -2.2859240974447848e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.5717980149568227e+00,     1.0497916033786814e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -7.9571647785282362e-01,    -7.0560767463083666e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.3045556835113114e+01,     4.0546852772053942e+04,     2.0531005133890193e-04,     5.9461433208859170e-07,    -3.0839967882739798e-09, } , 
      {    7.8179507341222552e+00,     6.6725058049791533e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -8.3448985079883172e+01,     9.8552800626788703e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7987829217507315e+01,     1.5726225793938482e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    1.2409217979293187e+01,     1.3722532604245598e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9040736990760763e+01,     3.5959494570619878e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.2150363185246782e+00,     7.0655172989332314e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0066471900543906e+01,     2.8237233419605682e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2338349136633715e+00,     3.1639189233339554e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7088471604271582e+00,     1.8604438754255978e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3488319508590422e+01,     6.5768951247722580e+03,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -1.9621653284462894e+00,     7.2405814819545485e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.7898825065507431e+00,    -1.9551154423631333e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.4986444657679764e+01,     8.4042024269668855e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4412217840907726e+01,    -6.5344388416972106e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4200821089417111e+01,     1.7279079034740338e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2579089722516284e+02,     1.5018349382341559e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.4812132209361650e+01,     1.5584646734663407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.3178992185717199e+01,     1.5322104277761053e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -5.1631284695429017e+01,     8.2847577212731485e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0288534716801550e+01,     3.0508677530173245e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9402805483581066e+01,     6.0476191946973695e+03,    -1.5664642925702622e-04,    -1.1579299655893774e-07,     4.7880199146377759e-10, } , 
      {    2.6059031244489013e+01,     1.7501300182847597e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -9.6100986293284585e-01,     6.6659543395870742e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8714885832976343e+01,     1.6290506580721074e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4119784068092898e+01,     2.2769436393758129e+04,    -2.2282960538085985e-04,    -5.3675274039416252e-08,     7.0782797441992036e-11, } , 
      {    3.8962759774323779e+00,     2.4985850616712640e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {    1.6520236632068023e+01,     2.4986297931028057e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9683117798675120e+01,     2.2788456344573482e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4806465935573161e+01,     1.4446149938918492e+04,    -6.2233516359656263e-04,    -5.9686231616516036e-07,     2.6879040108074888e-09, } , 
      {    3.1940503355417320e+00,     7.5944645966333446e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.2404303173800095e+01,     1.4549811668653521e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9211804502413109e+01,     2.6389607087045013e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2754742359287743e+00,     1.2627314192191317e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -2.5724664440966492e+01,     3.2143049240565113e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -7.2205855558366387e+00,     1.7589356525346461e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3634133733358617e+01,    -2.2036617699315866e+02,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    2.2511128061420145e+01,     7.2046203511198100e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -1.0828388283406651e+01,     9.4209799150811978e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    2.0103773498746648e+01,     7.0945107542395335e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.8089853157585765e+01,     2.8023824115689411e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.3529456666719124e+01,     1.4705368286767043e+02,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -7.4427929295468989e+00,    -2.4970074765646357e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    1.2460910000729983e+01,     7.7467845054510544e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.4590463801128477e+01,     1.5908543312079844e+04,     3.6641397015378598e-04,     5.7424590338550493e-07,    -2.8211308103763894e-09, } , 
      {    1.6555074134839181e+01,     1.5125150156487873e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7340059884306882e+00,    -1.0141457412792859e+04,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.5336639179661287e-01,    -9.1490736423147391e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.9299782914365387e+01,     1.7590342431716450e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1727271649072284e+00,     6.5664107450282995e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1905324726240023e+01,    -7.3657369368799236e+03,     9.4920742691050403e-05,     4.1749293816434692e-08,    -1.4515321612419505e-10, } , 
      {    4.7372250777100993e+00,    -1.8322766732545428e+04,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {    7.3691698228407070e+00,    -8.2506945377310221e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.3564205408279097e+01,     3.1057122646553333e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -5.1311320234002409e+00,     1.8211417784562989e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5227457387193311e+01,     2.8581138395321959e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.3569297710248623e+01,     8.1971388457795747e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.3417593351628044e+01,     6.1967108666270906e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.2006236320719150e+00,    -1.0834501557500248e+03,    -5.5615198747272897e-04,    -6.5898003868468190e-07,     3.0959232048292744e-09, } , 
      {    3.1521486269387342e+01,     1.5579724277821915e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5530934187711022e+01,     2.2764295321909198e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.1321114039036896e+01,     1.6452749156253041e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {   -1.8541616062154272e+01,     8.3228998402984071e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1278498094207698e+01,    -8.5490576016166106e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.7642042571750189e+01,     2.2749338988771084e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    6.4101266352842163e+00,     6.6879269892408720e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.2635499287976252e+01,     2.6346300715275353e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {   -2.0203839166094973e+01,     1.5547012045639516e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.3218753668538845e+01,    -9.5792731202717860e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.2176731699033823e+01,     9.7152262566550125e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -1.6578144852794033e+01,     1.5582216875921631e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5209948145019192e+00,     8.1781376535566678e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    4.7274381973640253e+01,     1.5538496534324073e+04,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    3.5990324977029786e+01,     4.7154398746062856e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    9.3025548725271108e+00,     4.7861521570875793e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8395044254082446e+01,     2.1799922950415468e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {   -2.2505716675974945e+00,    -4.0235097014564490e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -6.6711975127430740e+00,    -4.4875239765624283e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.7852734976401150e+00,    -9.1021141255914372e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.6460404811879005e+01,     1.6539499680919394e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0243224464454553e+01,     2.1669418558138117e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.4306628271486151e+01,     3.1113894513373845e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3413378809803113e+01,     1.7304818169888447e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.6365851379646706e+01,     1.6657243873408428e+04,     4.2824312478291954e-04,     6.4705405846170031e-07,    -3.1702936235114770e-09, } , 
      {   -2.8929662234783706e+01,    -1.6657521825085372e+04,    -1.8092650626638519e-04,    -3.5582143815691857e-07,     1.7736423709711261e-09, } , 
      {    4.2594626618658047e+01,     3.1103557523419448e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.1914404054813751e+01,     1.7265479269983007e+02,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    8.1784789383776456e+01,     7.2323022645250758e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    4.5287378562031613e+01,     8.2403224523544741e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8401456547667827e+01,     1.1803335908432728e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2597406930742663e+00,    -1.7765452452912872e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {    1.3191036228842634e-01,     1.6651953144209030e+04,     1.4793838626762614e-04,     3.8564475175037170e-07,    -1.9931660057775241e-09, } , 
      {   -8.0810407732240499e+00,     1.4878364077539682e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    2.0575788207934718e+01,     2.0935884124137065e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.1702089510629897e+01,     4.6039770186797759e+04,    -1.9854952837150089e-04,     1.8635316755856455e-07,    -1.2240575820653564e-09, } , 
      {    1.7221334830556547e+01,     8.8145326775405520e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1362303952512246e+01,     1.3999056344620103e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9940222575559538e+01,     8.3619473709222293e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.7586057976037523e+01,     5.7999190561528476e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4749181223180699e+01,     2.1091433272041617e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    3.0498273227932602e+00,     2.2783125545966697e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.0504209326362297e+01,    -7.1676774590003497e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.7334416335199919e+01,     3.0781529859139453e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -3.0779147540119220e+00,    -8.4792451956902332e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -2.4891783252509690e+01,     1.5543067662841418e+04,    -5.3404892121072009e-04,    -3.5148865828884261e-07,     1.4224918217434891e-09, } , 
      {   -4.8827243102725895e+01,    -1.5542440917122174e+04,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {   -4.6488710870114502e+00,     1.3355480737362568e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.4250624591751887e+01,    -1.7932260024553016e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {   -1.3320562224128274e+00,     1.5518146390926802e+04,    -4.7004275583423645e-04,    -3.4614344201050564e-07,     1.4519200121868380e-09, } , 
      {    2.2847057425585085e+01,     1.4378923118999812e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.1277533476755494e+01,     1.5984193290261755e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    7.1790648749696437e+00,     2.3852267520588121e+04,    -1.6110391881488405e-04,     2.0368428703086823e-08,    -2.6286597789759042e-10, } , 
      {   -3.4496138010810711e+01,     8.2507868476532512e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3052815098366560e+01,     8.6585416974931445e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3351232801231987e+01,     1.5396509923498092e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.2350182642863366e+01,    -2.5062304633383583e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.9856787588012232e+01,     9.9843878472263787e+03,     2.8737566567216773e-05,     1.0386701633595621e-07,    -5.5317241014598047e-10, } , 
      {   -2.3592542634407756e+01,     5.7172300476571072e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0023507366315584e+01,     2.5981665678992627e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.2397696663544416e+01,     2.4238729534681977e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    3.7291356022083249e+01,     3.0788088814738589e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.3047826293039137e+01,     1.3771484656213859e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.7572618361884523e+01,     2.1273775525437042e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    3.6521620339833866e+01,     1.5222199557781421e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5971326929970537e+01,     1.1397608121096637e+04,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5713959564040628e+01,     2.4867037119727524e+04,    -3.4648791463912706e-04,    -1.9929158419180713e-07,     7.6910842371216763e-10, } , 
      {    1.3940864971201060e+01,     7.7621883570755799e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5033708387868757e+01,     3.1307677284117879e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    1.3085191450694065e+01,     2.0163404721829662e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.5345713412879121e+01,    -1.2047333193076702e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    4.9692622122396138e+01,     3.7766865064744372e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.1659496784225176e+00,    -8.2823337269643616e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {   -2.0823095371529305e+01,     1.6171420904536140e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3837479724583368e+01,     8.6590805716637151e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    3.0010511738398240e+01,     7.7450687543643398e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.5391902212204567e+00,    -1.7037784580521904e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -1.9424069946520994e+00,     4.7849437417874296e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.5837867735678373e+00,    -1.8139295228588740e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {   -3.8623640040798932e+00,     3.6834330937205914e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.0525312925754022e+01,     2.6694464424649661e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.6665311469680397e+01,     3.0914656314424890e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    7.3717048301445010e+00,     9.7521962444951350e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {   -2.7583465259654226e+01,     7.0414080726585144e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    2.9379860762166146e+01,     4.8893568404879188e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    2.7535829084733688e+01,     4.7585157074870054e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    3.5163785419818993e+01,     3.1935984869163422e+04,    -5.0313434389615322e-04,    -3.1508458075074492e-07,     1.2479104151759452e-09, } , 
      {    2.6535294501994308e+01,    -4.3340473778797168e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.6776741575771894e+01,     3.1400977288644452e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.5590195731294784e+01,     1.0148681157165944e+04,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.9775468076577454e+01,     3.9378151588183442e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    8.9009826493321214e+00,     2.4802602769913661e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    1.4591420825205109e+01,     2.6374686648685612e+04,     2.4280077009358964e-05,     2.4002844159798077e-07,    -1.2948403795073485e-09, } , 
      {    9.0003060240364494e+00,    -7.5492084408366100e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    1.5919733998900591e+01,     5.6218429252107273e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.1025029616642961e+00,     1.2169858872684474e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.8498630241365746e+01,     4.8135912398903478e+04,     1.7221846327698510e-04,     6.2567319334835252e-07,    -3.2880063852848728e-09, } , 
      {    7.0142738795420581e+00,     2.8662948411991812e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.2895373167813583e+01,    -9.4072715674849587e+03,    -8.9382839105929817e-04,    -1.1293589257342305e-06,     5.3638816050596831e-09, } , 
      {    8.4375544644992715e+00,     2.6369256943647739e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.3995339772530013e+01,     2.4171166679771661e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.5944435693289538e+00,    -1.0866824803760241e+04,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {   -4.0890809941401347e+00,    -4.2587963168090046e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7456064653502629e+01,     2.5214801933623774e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    5.5681072012917383e+00,     1.6813963217772009e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -4.0208519417329480e-01,     1.6323321605775172e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.6634673834302465e+00,     2.9296615363829505e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4110549323073586e+01,     2.7941340822419690e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    3.6619138826064059e+00,     5.2257751744315044e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.4147568221489530e+00,     1.4527984690842397e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.7481367133770629e+00,     2.4768821342458505e+04,     5.5194654323925758e-05,     2.7643251913607852e-07,    -1.4694217860748923e-09, } , 
      {    8.8710907608244423e+00,     2.1513319896447319e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7985478897429385e+01,     6.2571686604336728e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.5414548168494264e+01,     1.6991444092718630e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.6895803036992689e+01,     1.6546049160779618e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    1.7225531903546468e+01,     2.3889479398545951e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.7053124359288649e+01,     1.6155062366803340e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9550304531843036e+01,     1.2964300719889986e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5506865073999451e+01,     2.8334361792112675e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.8517085311965840e+00,    -9.8881560649774128e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.7867719406006611e+01,     3.2216136647452688e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.7837564784237305e+01,    -1.2259622477323892e+03,    -3.3985341433391921e-04,    -4.0291594825169021e-07,     1.8893673966519725e-09, } , 
      {    4.0276537235705963e+00,     6.3278378466679108e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2001717442153748e+01,     7.4519793130270846e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1100498964128533e+00,     8.1735869025992015e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8424797369419288e+00,    -1.0551099821936912e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    4.3177692610459451e+00,    -1.6768716537714183e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    3.2083411274636493e+00,     6.4345798815715025e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    2.3553810308667185e+01,     3.1101508088187584e+04,    -3.5084193613382710e-04,    -6.4365706596090173e-08,     1.1926416555294388e-11, } , 
      {    6.3420986998917659e+00,     2.0885012311401777e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.2190630303659717e+01,     7.6675688145238091e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0831862269764448e+01,     1.1676978942865775e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.4345043484027958e+01,     3.1069509071739594e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.2720359787981916e+00,     2.3259137637783700e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.5657678708936359e+01,     7.2320965492990545e+03,    -6.2015815284921266e-04,    -6.6432525496301882e-07,     3.0664950143859254e-09, } , 
      {    3.4169608471239652e+00,     2.0569543602720914e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2189501232987439e+01,     8.4143547028620269e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.0619984785803631e+01,    -8.2430281463848751e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -5.8603709493473744e+01,     8.5598163930909341e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.4883030371512836e+01,     2.1890962985590824e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0852052930338942e+01,     1.5107135424024986e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    6.0318880130241332e+00,     1.3890488317881536e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2326452933166600e+01,     1.3413356131577293e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.4243970553520136e+00,    -1.7588539904555320e+04,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    1.8273639274876190e+00,    -8.5222413821357550e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    1.8839953672960750e+01,     1.4444125238940689e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.8625131842468132e+01,     3.2184137631004698e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.2572540087344036e+01,     9.7518598740053185e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.8702252589988682e+00,     1.1609862569241914e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.2945283837163508e+00,     1.7782732152473704e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.2226146018552193e+01,     1.5564329792712977e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.6146887645105185e+01,     2.3227138621335704e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.0143765068818329e-01,     6.6549337128647094e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9646500402798583e+00,     6.5677215963720919e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    5.4736678495719886e+00,     1.6678001411757876e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.9313437697013541e+01,     1.5521178787250614e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.7200640683919719e+01,     2.1893222954942041e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9786174535343470e+01,     2.4344648777626539e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.1222881713133157e+00,     6.6563564037984625e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7781492585474239e-01,    -9.4112750993240218e+02,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    7.7187814485461539e+00,     2.4072921457597899e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4247547190902292e+00,     4.6982026928599880e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6134994660935611e+01,     7.5552687236554621e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -1.4231963096249654e+01,     2.0995392929146205e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5059819289032365e+01,     1.3564531530318591e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.2602691044285123e+01,     9.6251214966124480e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8894444464358990e+00,     1.5156292275887941e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0070997844215366e+01,     2.5851928443619665e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.8802940522077236e+00,     2.0962564572413903e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.8550227805324067e+00,     2.9448990351377783e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    6.6870302093443623e+00,     2.0491991341708788e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.8352309845518517e+00,     8.5110089427648145e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0147669972957319e+01,     1.6636764286735925e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.3101865134112671e+00,     1.3497467348188549e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.3564706473318644e+00,    -1.5510452202424858e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9322649609092348e+01,     3.0221914379565489e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    8.9286366485117199e+00,     1.8637865578261320e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.5211359564071468e+01,     7.9461787808709760e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    4.1600981940589952e+00,     8.1463739064820857e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.9185521171438054e+01,     2.4634740033632952e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.2202982029876514e+01,     1.2069999912558764e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.6278511187081346e+00,     1.3020335161884304e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    2.1720551928199939e+01,     2.3760112026137958e+04,     1.1702380895305935e-04,     3.4924067421227395e-07,    -1.8185845992099802e-09, } , 
      {    1.3711193154259082e+01,     6.7692267842694873e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    9.0678310557212463e+00,     5.8254504905430804e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    6.5637136596439785e+00,     1.7964640193440711e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.1509552045186144e+01,    -8.8946422026388354e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.9347911132409745e+00,    -1.9151578179559237e+02,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {    1.1939010098405554e+01,     1.6145324725411087e+04,    -4.9878032240145318e-04,    -4.5001045834646184e-07,     2.0050924223328187e-09, } , 
      {    9.2276616688973920e+00,     2.1355585542106888e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4067309018239110e+01,     2.5929850567596754e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.7800700999003549e+00,    -3.0951112882795260e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0150597792915097e+01,     1.6408608459731109e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.5411546470594382e+00,    -1.6428655189493478e+04,    -4.8996881134889532e-04,    -7.2109776120420344e-07,     3.5039423988510599e-09, } , 
      {   -8.1774969224807048e-01,    -2.1293981584045023e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.1141409611456027e+01,     7.0797028381586342e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7084983557022596e+01,     2.2035209565768026e+04,    -6.5542675165847946e-04,    -5.6580345490539964e-07,     2.4838944137965963e-09, } , 
      {    1.1304170011629536e+01,     2.9842011321070771e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.3551461114754167e+01,     1.9770383752136673e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.2436729458856606e+00,    -8.2304756473544294e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    1.6407892936152180e+01,     2.8727382761805664e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.7999107516338846e+00,     1.5001197229602603e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    7.2160456720131672e-01,     7.4405330119022392e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.7992868955281541e+01,     1.5640970067250808e+04,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    1.6332997111432270e+01,     1.5351238508186205e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    1.3176653842348896e+01,     2.0398691337182216e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.1255119702935545e+01,     8.8687327074903351e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.0921310043052870e+00,     5.4324295208500935e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.9644413138490444e+00,    -2.5097407962337635e+04,    -4.2596264597241174e-04,    -7.1575254492586641e-07,     3.5333705892944084e-09, } , 
      {    1.9353727256493915e+01,     2.4874740585403062e+04,     4.8779180060154535e-04,     7.8856070000206190e-07,    -3.8825334024294965e-09, } , 
      {   -5.1647872516551885e+01,     7.2356383680895269e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.7426340399870433e+01,     2.3969661491874260e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {    1.2010855612964635e+01,     1.4914452334767735e+04,    -6.3519294630925112e-05,     6.1501330405358849e-08,    -3.5357946576999568e-10, } , 
      {    9.6553010625778324e+00,     6.5857609101442840e+03,    -2.1581170239325131e-04,    -1.8921754374929586e-07,     8.8240453285065505e-10, } , 
      {    3.9694765650382298e+00,     7.7003894694093897e+03,     1.5495628925523469e-04,     2.5010248204049204e-07,    -1.1815442703688610e-09, } , 
      {    7.0248178429149615e+00,     8.9569933798375096e+03,     1.4962852626941768e-04,     2.5133526626881740e-07,    -1.2904237268724407e-09, } , 
      {    4.6692632925281590e+00,     6.2830195521406040e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.5066196890841367e+01,     1.6171056245195854e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {    1.2710642340454564e+01,     7.8423648205724048e+03,    -2.2113946537906831e-04,    -1.8798475952097055e-07,     7.7352507634707552e-10, } , 
      {    1.2053981300821166e+01,     1.4286150379553674e+04,    -6.0855413138016608e-05,     6.0884938291196182e-08,    -2.9913973751820592e-10, } , 
      {    6.7261374380949697e+00,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    1.4366410163351437e+01,     2.3243143759391183e+04,     8.8773113131401069e-05,     3.1222020456001358e-07,    -1.5895634643906465e-09, } , 
      {    1.0441339165780823e+01,    -1.7429305144791660e+03,    -3.6810411015557749e-04,    -4.3993641790395058e-07,     2.1183885314713058e-09, } , 
      {    9.6984267504343631e+00,     5.9574589549302236e+03,    -2.1314782090034278e-04,    -1.8983393586345854e-07,     9.3684426110244481e-10, } , 
      {    6.3250311154250323e+00,     1.6029080894032839e+04,     3.0724869701756089e-04,     5.0082135619514676e-07,    -2.4175282689895120e-09, } , 
      {   -1.8551860995578064e+01,     8.3263902246820398e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9546155599817300e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3803723933017640e+00,     1.7285684804460961e+04,     3.0192093403174383e-04,     5.0205414042347212e-07,    -2.5264077254930913e-09, } , 
      {    1.5140732560123102e+01,     1.5542754289981794e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2667516652598033e+01,     8.4706667757864652e+03,    -2.2380334687197685e-04,    -1.8736836740680787e-07,     7.1908534809528565e-10, } , 
      {    4.0126022528947605e+00,     7.0720875141953293e+03,     1.5762017074814322e-04,     2.4948608992632941e-07,    -1.1271045421170713e-09, } , 
      {    2.0052234660891042e+01,     2.2128515200126079e+04,    -2.8199487851708491e-04,    -1.2709982122977434e-07,     4.7438533882886960e-10, } , 
      {    1.7421751441228167e+01,     2.4499747669819306e+04,     8.3445350145584034e-05,     3.1345298878833894e-07,    -1.6984429208942262e-09, } , 
      {    1.2756849290004153e+01,     7.2140628653583444e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9816921550584308e+00,     9.5852953350515709e+03,     1.4696464477650915e-04,     2.5195165838298003e-07,    -1.3448634551242304e-09, } , 
      {   -3.3625969635303704e+01,     8.3283391086653737e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4620264854995426e+01,     8.3290437405815264e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6261376046716283e+00,     1.2566039104281208e+03,    -5.3277629858170233e-06,     1.2327842283253282e-09,    -1.0887945650357959e-10, } , 
      {    1.7696680110504239e+01,     1.3799823775502628e+04,    -4.3428728627941109e-04,    -3.7781869538442909e-07,     1.7103693374495204e-09, } , 
      {   -1.0510498664410537e+01,     1.5540453090040384e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7587553259885084e+01,     1.5545055489923205e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2407789211277844e+01,     3.0457206624749528e+04,    -1.2970247075475876e-04,     1.2361905292488038e-07,    -7.6159865979178123e-10, } , 
      {    1.4409535851207968e+01,     2.2614841804177122e+04,     9.1436994624309573e-05,     3.1160381244585095e-07,    -1.5351237361388568e-09, } , 
      {    1.2097106988677696e+01,     1.3657848424339612e+04,    -5.8191531645108090e-05,     6.0268546177033528e-08,    -2.4470000926641610e-10, } , 
      {   -1.2866068997715065e+01,     7.2117616654169342e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5524561692490163e+01,     8.3117707438158577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5232013625723155e+01,     7.2163640652997528e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.0816919884817722e+00,     1.6728370524828410e+04,     1.1653693820750085e-04,     2.8239412752857817e-07,    -1.4944333238833335e-09, } , 
      {    7.0679435307714922e+00,     8.3286914246234501e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5707963267948966e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.5121755412979603e+00,     7.0987675581507574e+01,    -1.8804787731715151e-04,    -2.1904362078073129e-07,     9.7753467335796819e-10, } , 
      {    1.6721964713738238e+01,     3.1571835184014635e+04,     2.4106552089372724e-04,     5.6293907871466830e-07,    -2.8255474630112975e-09, } , 
      {    5.7968678798744220e+00,     8.3294027700903262e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1973121976171264e+00,     8.3279800791565740e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0484464853637354e+01,    -2.3712324696932264e+03,    -3.6544022866266901e-04,    -4.4055281001811320e-07,     2.1728282597230958e-09, } , 
      {    1.1896637779740205e+01,    -1.9524799802154675e+03,     6.4493036122042105e-05,     7.2191762962032783e-08,    -2.9472308488329798e-10, } , 
      {    2.3107575938767770e+01,     2.3385119110554202e+04,    -2.8732264150290197e-04,    -1.2586703700144904e-07,     3.6550588232529002e-10, } , 
      {    2.0095360348747572e+01,     2.1500213244912018e+04,    -2.7933099702417643e-04,    -1.2771621334393702e-07,     5.2882506708065947e-10, } , 
      {   -2.5782739540198193e+01,     1.5542401974023718e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2859760044521892e+01,     1.5543106605939871e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.9539814673978242e+00,     7.1430751897768369e+03,    -3.0427706569008304e-05,     3.0442469145598091e-08,    -1.4956986875910296e-10, } , 
      {    8.0857846153940205e+00,    -1.0071621939102613e+04,    -5.2039651791790367e-04,    -6.9065529205860530e-07,     3.3543725300919566e-09, } , 
      {    1.7464877129084698e+01,     2.3871445714605245e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.7415524382908938e+00,     5.3291569997161632e+03,    -2.1048393940743427e-04,    -1.9045032797762119e-07,     9.9128398935423468e-10, } , 
      {    3.2489987486013668e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.1495255704391905e+01,     8.3290200569982226e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.6805856658118348e+00,     2.4357772318656287e+04,     4.5954110477988707e-04,     7.5154023034980148e-07,    -3.6535122676101628e-09, } , 
      {   -1.6196478547968542e+01,     1.6655081649305488e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {   -2.8398241218945376e+01,     7.2137105494002708e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.0766097446684597e+01,     7.2144151813164190e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1735926943688566e+01,     2.5614376229084410e+04,     4.5421334179407001e-04,     7.5277301457812684e-07,    -3.7623917241137426e-09, } , 
      {    3.1901896553271929e+01,     1.6659684049188309e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1566209909111578e+00,    -8.2577037490419425e+03,    -3.4034028507947772e-04,    -4.6976249493538599e-07,     2.2135186719786194e-09, } , 
      {    5.5695192289555795e+00,     9.1665428455738111e+03,    -2.8296862000820193e-04,    -2.6079291459716599e-07,     1.1226878894821633e-09, } , 
      {    9.5410832293534025e+00,    -1.0281171404838919e+04,    -8.7799371640284076e-05,    -1.7852711119262194e-07,     9.4126091373735280e-10, } , 
      {    2.5240291356676052e+00,     8.1709570702830179e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5023071202984836e+01,     1.6799358200409915e+04,    -7.1510939109650651e-05,     6.3350506747846849e-08,    -5.1689865052536508e-10, } , 
      {    1.2748929644220318e+01,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.4684077329711336e+00,     8.4864257789638814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3681568032815630e+00,     1.5400778938818779e+04,     3.0991257851046937e-04,     5.0020496408098408e-07,    -2.3630885407377219e-09, } , 
      {    5.8387617567423504e+00,     8.3260615923072655e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1556417088187452e+00,     8.3313212569396328e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5852470481080260e+01,     8.3114184278577814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2450914899134375e+01,     2.9828904669535470e+04,    -1.2703858926185022e-04,     1.2300266081071770e-07,    -7.0715893153999136e-10, } , 
      {    1.7995360515324229e+01,     1.4357138055135183e+04,    -2.4890329045516808e-04,    -1.5815868248953511e-07,     6.7839493583976237e-10, } , 
      {    2.5141779510788478e+00,     7.9099389351456912e+03,    -2.7764085702238487e-04,    -2.6202569882549135e-07,     1.2315673459857430e-09, } , 
      {    2.5463130489154572e+01,     3.1713810535177650e+04,    -1.3503023374057576e-04,     1.2485183715320568e-07,    -8.7047811629536076e-10, } , 
      {    2.0752021388380967e+01,     1.5056427685930750e+04,    -4.3961504926522815e-04,    -3.7658591115610373e-07,     1.6014898809459407e-09, } , 
      {    1.0567710737206211e+01,     1.5385019935641363e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3679998116531475e+00,     8.2949099971682899e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.7984833644693357e+00,    -8.3624728520786120e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.2957101057916134e+00,     7.2309835461659377e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.9425167295239547e+01,     1.8684292215209513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1703694486214378e+00,     1.4197535116301515e+02,    -3.7609575463430303e-04,    -4.3808724156146259e-07,     1.9550693467159364e-09, } , 
      {    1.7739805798360770e+01,     1.3171521820288568e+04,    -4.3162340478650261e-04,    -3.7843508749859172e-07,     1.7648090657013101e-09, } , 
      {    8.2119561543371145e+00,     7.0563285110179122e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4767516486021375e+01,     1.5613741965563302e+04,    -2.5423105344098514e-04,    -1.5692589826120981e-07,     5.6951547933618284e-10, } , 
      {    1.9777305991614973e+01,     3.2828439094442758e+04,     2.3573775790791023e-04,     5.6417186294299366e-07,    -2.9344269195148769e-09, } , 
      {    1.4252192330127007e+01,     6.3762114444079825e+03,     2.1678544388436827e-04,     3.2291063711668748e-07,    -1.5307070835039487e-09, } , 
      {    1.7841057228502947e+01,     8.3155735134251827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.6465195267449584e+00,     8.2888765227255371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3641915607549786e+01,     8.3685063265213612e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.0992997091095873e+00,     1.4991998965948376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.4763343761664647e+01,     3.8785898049372976e+04,     2.2589937007567433e-05,     3.7433792707953507e-07,    -1.9975826584124323e-09, } , 
      {    4.8697325014656503e+00,     1.6238630359769140e+04,    -1.2534844926005871e-04,    -1.1306824670836598e-08,    -4.4166526349079032e-12, } , 
      {    1.0645835362723428e+01,     1.5020384899358634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9995332396809591e+00,    -8.8150180286744944e+03,    -5.2572428090372062e-04,    -6.8942250783027994e-07,     3.2454930735883773e-09, } , 
      {   -1.7820660703358278e+01,     8.3512333297556943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4018905974347618e+01,     1.5540124457665610e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3058744901350700e+01,     1.5545384122297977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.1550951965151661e+00,     2.3869144514663836e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.1696640771728305e+01,     7.2114330330421608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7209322412121239e+01,     2.2685829479758631e+04,    -9.6610882692841927e-05,     9.2560191665119606e-08,    -5.5758906278088846e-10, } , 
      {    4.7834811257525889e+00,     1.7495234270197263e+04,    -1.3067621224587572e-04,    -1.0074040442511271e-08,    -1.1329610913848749e-10, } , 
      {    1.0668828323743412e+01,     7.2166926976745290e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9943214677382230e+01,     2.3873746914546657e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9668670834276735e+01,     1.7427677045118424e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0786179873479504e+00,     7.7779361005900309e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0355087790067762e+01,    -4.8632660405104525e+02,    -3.7343187314139455e-04,    -4.3870363367562522e-07,     2.0095090749677264e-09, } , 
      {    6.7458926270879607e+00,     6.6633075413249253e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3150701626624301e+01,     2.2756817155340141e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    8.2904469195065698e+00,     6.6916934747351834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5047278514157625e+01,     8.3131493963350204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.4053457214904356e+01,     8.3442334529118798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.3806932919289867e+00,     1.5502939388083882e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7629994350566299e+01,     1.4916747904877662e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0053368369053965e+01,     7.1610937681471532e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.6687937504373505e+00,    -6.9587606978734675e+02,     5.9165273136225077e-05,     7.3424547190358117e-08,    -4.0360254138687761e-10, } , 
      {    3.0209384462297084e+00,     7.1742479634604324e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3784430839036061e+01,     8.2490616208276260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4781553164544182e+01,     8.4083212284192741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7794365809177695e+01,     1.6799369460072881e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6199074580478903e+00,     7.8063220340002890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3874116378250372e+01,     8.1578391590847514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1707232881757154e+01,     1.5582569191879707e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.4234980811582947e+00,     1.6657382849246900e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.9357984718475929e+01,     7.2538777672562564e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2400264826084987e+01,     1.5489785192770603e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1270445664370772e+01,     1.6657030533288827e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.6975844754455721e+01,     1.6657735165204973e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.9087069796993568e+00,     8.8794467486568683e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0903820620904892e+00,     7.2133515198914674e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4275083010149055e+01,     7.2147742108252214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6487428692185077e+01,     1.5700488644322226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0528598416305414e+01,     1.5542425657607024e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3451546091755748e+01,     1.5543082922356567e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0585239172018056e+01,     8.3640284955601728e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4119372846266938e+01,     7.3717972196987757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6369692757461085e+01,     8.3654511864939250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6750707343287218e+01,     1.4912145504994842e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.5984269170110217e+00,    -1.1856162348466132e+03,    -1.8272011433133451e-04,    -2.2027640500905660e-07,     1.0864141298615479e-09, } , 
      {    4.0557279407512903e+00,     6.4437855589812689e+03,     1.6028405224105172e-04,     2.4886969781216673e-07,    -1.0726648138652814e-09, } , 
      {    1.2309536017784627e+01,     1.5471766614400287e+04,     1.2186470119331788e-04,     2.8116134330025281e-07,    -1.3855538673797537e-09, } , 
      {    8.3734825302567799e+00,     8.8510608152466102e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.7123071036408177e+01,     2.3942433390186754e+04,    -1.0193864567865896e-04,     9.3792975893444940e-08,    -6.6646851928446799e-10, } , 
      {    1.2243678856527948e+01,     7.2670319625695356e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3037358582364806e+00,     8.2757223274122589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.3372467054452333e+00,     1.7913986759675019e+04,     2.9925705253883535e-04,     5.0267053253763480e-07,    -2.5808474537448814e-09, } , 
      {    9.3046169461803601e+01,     1.5527212261693365e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0681474169533573e+01,     7.1985208370699147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6819127127821805e+00,     8.3816605218346413e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8316660084938093e+01,     7.2296048936467741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.5971863735021458e+01,     1.5558296318270224e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8428920196466237e+01,     8.5508601287777383e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2819475377110031e+01,     1.9312599800255055e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4616676313693985e+01,     1.5595723387192986e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1772548222012826e+01,     1.4914094388978176e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.7026600866719264e+00,     1.5463124486185970e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6765090401594769e+01,     3.0943533228800574e+04,     2.4372940238663577e-04,     5.6232268660050562e-07,    -2.7711077347595075e-09, } , 
      {    1.7251662454074584e+01,     1.0355600790586061e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.0532913170068312e+00,     7.1344330615625204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.6660998548641164e+00,     8.3446909328474467e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4452661539064499e+01,     2.1986539848963064e+04,     9.4100876117218091e-05,     3.1098742033168827e-07,    -1.4806840078870670e-09, } , 
      {    6.9519088315957180e+00,     8.3110593983489816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5506256177011103e+01,     3.1085508579963589e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    9.8246044038282321e+00,     9.0279810554190190e+03,    -3.8419351047733836e-05,     3.2291645488086085e-08,    -3.1288905351447236e-10, } , 
      {    4.8855046833477234e+00,     8.3624484717475134e+03,     1.2147576281134351e-04,     2.1427116338525712e-07,    -1.0621298125747714e-09, } , 
      {   -2.9671970710156748e+00,    -8.2949343774993868e+03,    -1.8310905271330885e-04,    -2.8716658492405232e-07,     1.4098381846665302e-09, } , 
      {    3.8177604357064197e+01,     7.2137342329835719e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.5811654336556938e+01,     7.2143914977331169e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0432789436957812e+01,     7.2936926691541685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3611055934501362e+01,     8.3124820892827338e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2801003225519452e+01,     1.5622384093777619e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7880340806594990e+01,     1.6640462168439306e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6790446002015962e+01,     1.4915158050403128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7081799566365383e+00,     8.3284677674657396e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.2861436450850867e+00,     8.3289150817811606e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0027379065247532e+01,    -1.6920680807593385e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.4048996009149025e+01,    -2.3011999414102320e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8093613708817443e+01,     2.9342578065484424e+04,    -5.0047046240324475e-04,    -3.1570097286490760e-07,     1.3023501434277351e-09, } , 
      {    1.0660307930583668e+01,     7.1365106043462201e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5094041139096911e+01,     8.2588016466046665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2140232676534225e+01,     1.3029546469125553e+04,    -5.5527650152199586e-05,     5.9652154062870862e-08,    -1.9026028101462631e-10, } , 
      {    4.5830119168150985e+00,     1.8849058656421812e+03,    -7.9916444787255359e-06,     1.8491763424879922e-09,    -1.6331918475536940e-10, } , 
      {    1.2318353098126806e+01,     1.4445046411524281e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6310207728553578e+01,     8.4706780354494313e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2993508316858373e+01,     1.5465202028969670e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1860614231075441e+00,    -8.3126919163994535e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0594627671479103e+01,     1.4994076508732076e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9077519264125044e+01,     3.9900526608638080e+04,     3.9335792865605347e-04,     8.1365795286932302e-07,    -4.0615314616319476e-09, } , 
      {    1.1437246538868575e+01,     2.5057061949451858e+04,     2.6882934596982700e-04,     5.3311300168323284e-07,    -2.7304173225039839e-09, } , 
      {    6.8555145016645618e+00,     6.5147732345627765e+03,    -2.7763825076099793e-05,     2.9826077031435431e-08,    -9.5130140507313155e-11, } , 
      {    1.1582223880652048e+01,     7.2916151263704687e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4450662769803026e+01,     6.5864666257796780e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3220709048675214e+00,     1.4286139119890708e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2768914494478908e+01,     1.4339108217101899e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.6400270486616577e+00,     8.5044594629045914e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9151978724463621e+01,     1.3590274309766326e+04,    -1.6901400017915301e-06,     1.3430948548155433e-07,    -7.0274227890508348e-10, } , 
      {    1.1024348300824153e+01,     7.6328153548361033e+03,     2.1145768089855126e-04,     3.2414342134501284e-07,    -1.6395865400075284e-09, } , 
      {    1.1939763467596736e+01,    -2.5807819354295279e+03,     6.7156917614950609e-05,     7.1575370847870116e-08,    -2.4028335663150822e-10, } , 
      {    6.9385664672019010e+00,     1.0213597290265632e+04,     1.4430076328360064e-04,     2.5256805049714271e-07,    -1.3993031833760202e-09, } , 
      {    5.0387240930492609e+00,     8.2511391636113258e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1134826068630435e+01,     1.4128404765550276e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9189206967087031e+00,     8.4062436856355744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.2564558603974270e+00,     6.2830758504554319e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.3916531806078911e+01,     1.5620306550993919e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0449168259204246e+01,     3.7671269490107872e+04,    -3.4817805464091857e-04,    -6.4982098710252840e-08,     6.6366144807084178e-11, } , 
      {    2.0708895700524437e+01,     1.5684729641144810e+04,    -4.4227893075813663e-04,    -3.7596951904194110e-07,     1.5470501526941510e-09, } , 
      {    1.5639805964937427e+01,     6.0284466305117321e+03,    -4.0119569821749426e-04,    -4.0887755664418986e-07,     1.9143789344604132e-09, } , 
      {    7.2372024978984095e-02,     8.3785142095036190e+02,    -4.3526102777052811e-04,    -5.1151178875182071e-07,     2.3586718881028141e-09, } , 
      {    7.5446240707431143e+00,     5.5075532403341913e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1878249135215652e+01,     6.5857552803128010e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5275203063599470e+01,     6.5880564802542103e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.4106486688449209e+01,     6.5854029643547265e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5983273785144165e+00,     8.9569990096689926e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3959670294387756e+00,     7.7003838395779067e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0845497685833115e+02,     8.3269604561462111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0138486036604103e+01,     2.0871911289697960e+04,    -2.7666711553126790e-04,    -1.2833260545809968e-07,     5.8326479533244923e-10, } , 
      {    2.0795147076237498e+01,     1.4428125730716689e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.3132392000505480e+00,    -9.0245674944107959e+03,    -9.3127134626101111e-05,    -1.7729432696429663e-07,     8.3238145723377327e-10, } , 
      {    1.8890110477462336e+01,     1.8055984630163966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.9707311657390207e+01,     7.7000315236198321e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8224165893434314e+01,     6.6653850841086260e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7244234872630093e+00,     7.2478442928135055e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    7.0726248580445192e+00,     7.2206218209574781e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.4098124777288314e+01,     7.7364322559815055e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5142365376755350e+01,     2.3871798030563317e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.3354439308509630e+01,     2.3871093398647172e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.5668650630876598e+01,     7.2313358621240131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.6397914269211071e+00,     8.3200549262406166e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -1.9106121309128575e+01,     6.5834540803713917e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.4242629829224782e+00,     1.2566151700910864e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.7019649508168513e+01,     8.2716035259598684e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0250162812261232e+00,     5.2236939062316048e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5341125560117437e+01,     5.4711323508791802e+03,    -5.8657969404173733e-04,    -6.2853756953908375e-07,     2.9463533360701711e-09, } , 
      {    1.1513201906841902e+01,     1.5542042944514918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.3460213738049820e+00,     1.0977078784575124e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2327363921087764e+01,     8.2353914200968757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.9579664793087758e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5512117782365605e+01,     1.5543465635448671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6426949578441700e+01,     1.6065123680604955e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2025699262528349e+00,     8.9562876642021165e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6072270882240698e+01,     1.5288322850305891e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.8442712956796981e+01,     7.2139648137845052e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1080865238120211e+02,     7.2141609169321837e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7628178513743684e+00,     7.7010951850447836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5553554589224365e+01,     7.2850505409398529e+03,    -4.0652346120331132e-04,    -4.0764477241586450e-07,     1.8054994779568337e-09, } , 
      {    2.4710522632223171e+00,     8.5382408903597516e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    8.1521863710316289e+00,     1.6658094194713776e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.5523106401612150e+00,     1.6656671503780024e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.2496051874228655e+01,     7.7800136433737316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8633572173377839e+00,     7.2353927733361961e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2500485594653517e+01,     7.1927329573804927e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7238648091029333e+01,     1.5328137752203804e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9887258578411654e+01,     8.9438810984707270e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.3716728586938309e+01,     6.9596314256824407e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4792078298579987e+01,     7.6980826396364973e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.9661066418998168e+00,     6.1163549869008320e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5750476136431271e+00,     1.3763769729267548e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7818685039541375e+01,     4.0042501959801099e+04,     1.7262174021750411e-05,     3.7557071130786043e-07,    -2.1064621149160116e-09, } , 
      {    2.4806469449521177e+01,     3.8157596094158915e+04,     2.5253818500475944e-05,     3.7372153496537244e-07,    -1.9431429301606422e-09, } , 
      {    2.3064450250911239e+01,     2.4013421065768263e+04,    -2.8998652299581045e-04,    -1.2525064488728636e-07,     3.1106615407350025e-10, } , 
      {    1.6052704202981847e+01,     2.3452693225127485e+04,    -3.4382403314621852e-04,    -1.9990797630596978e-07,     8.2354815196395740e-10, } , 
      {    1.9820431679471504e+01,     3.2200137139228696e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {   -1.1477784941349885e+01,     8.8773692058731685e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4854218945061374e+01,     1.5521424382003943e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1737431166589410e+01,     8.9546978097275824e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.4420579047433346e+00,     1.4521425735243263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1718555464441181e+01,     1.5033891410629989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.9654609929806615e+00,     7.2271807765566118e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0073875781406056e+01,     9.7272932055405181e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2221203744911792e+01,     1.5564084197959646e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.8850705876503920e+01,     7.7690759450494879e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.4882924523013521e+01,     6.9994463275803537e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.5530826768938240e+01,     7.7736783449323084e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.2996909837895183e+01,    -8.3441105357631277e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.1452487878695207e+01,     1.5073353996569826e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.4692630740339112e+00,     2.2754515955398729e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.5629078069825511e+01,     2.2759118355281553e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5579774556930088e+01,     8.3647398410270489e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -8.5983789386579161e+00,     8.3908245744958967e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2816231589154881e+01,     1.4914799020894327e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8028597545228028e+01,     8.0742599849475464e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0413005477137977e+01,     6.0104167924784488e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3697149652595044e+01,     1.5124001800504037e+04,    -4.9611644090854471e-04,    -4.5062685046062452e-07,     2.0595321505846083e-09, } , 
      {    1.0527590541493884e+01,    -2.9995344249072868e+03,    -3.6277634716976048e-04,    -4.4116920213227588e-07,     2.2272679879748855e-09, } , 
      {    8.1289103032505512e+00,    -1.0699923894316675e+04,    -5.1773263642499508e-04,    -6.9127168417276798e-07,     3.4088122583437467e-09, } , 
      {    9.9108557795412935e+00,     7.7713771449908972e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.6119196048705398e+00,    -8.4672532147782440e+03,     9.2256861198141885e-05,     4.2365685930597359e-08,    -1.9959294437598484e-10, } , 
      {    3.3002160542818384e+00,    -4.1875248947775890e+02,    -4.2993326478471105e-04,    -5.1274457298014607e-07,     2.4675513446063935e-09, } , 
      {   -8.6087316479113809e+01,     1.5542656238407953e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1316445975432046e+02,     1.5542852341555634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9207274997513451e+01,     8.1140748868454593e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0355894228466189e+01,     1.8849227551366296e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2034330675257261e+01,    -8.3160721862054634e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.9886999845714598e+01,     8.3413106630414368e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {   -7.0345915621151640e+00,     8.5831228642993538e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4957179969186942e+01,     8.2092467189297131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8214190388296245e+01,     8.5433079624014408e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1217646516480681e+01,     8.2937465356140583e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.5957243981328119e+01,     8.4481361303171871e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3121177063271986e+01,     1.4861477607725061e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4253800183559658e+01,     8.4066883115158762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.7815142048818426e+00,     5.7997133409268263e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9357456557054554e+01,     6.7051999860065380e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5787644874290722e+01,     8.3227913705382216e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6621659064646430e+01,    -8.3467403680793141e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2645288632361968e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2645098662422637e+01,     1.6833150887528041e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.6796424174076819e+01,     5.2615828851428769e+03,    -1.5398254776411772e-04,    -1.1640938867310041e-07,     5.3324171971556735e-10, } , 
      {    1.8038486203180760e+01,     1.3728836099921122e+04,    -2.4623940896225960e-04,    -1.5877507460369779e-07,     7.3283466409155224e-10, } , 
      {    1.1036140216198637e+01,     3.2686463743279739e+04,     6.1183351254221320e-04,     1.0022591045044562e-06,    -4.8894962662308132e-09, } , 
      {    6.0784077884676320e+00,     1.5773435434043176e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.1019028476208028e+00,     6.7446625719463755e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3834531528667792e+01,     1.9170613189429074e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.1234224278941873e+00,     2.5132303401821728e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1368913406280509e+01,     7.1915209602261002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.6767049272395518e+00,    -8.3456121054310406e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.0174947625647530e+01,     1.5072181059276681e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4523933539967659e+00,     7.8592911312114811e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.3148889345273947e+01,     8.5359396904183413e+03,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {    3.0161202386349917e+00,    -1.2903965975462468e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -1.8911304745463774e+01,     8.3444239086227499e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {   -8.5215255372450372e+01,     8.3044185510139669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6210095479954674e+01,     8.3529642982329315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.2517848734822113e+01,     7.4286794031363352e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1232607970033730e+01,     8.5405420903011618e+03,    -4.3259714627761958e-04,    -5.1212818086598339e-07,     2.4131116163546038e-09, } , 
      {    7.7105972441774853e+00,    -1.1146285592651057e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7898045671208870e+01,     1.6093509614015213e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.5738059158430641e+01,     2.1013886640860976e+04,    -6.5276287016557098e-04,    -5.6641984701956232e-07,     2.5383341420483859e-09, } , 
      {    2.2494040586990906e+01,     2.9200602714321409e+04,    -1.2437470776894172e-04,     1.2238626869655504e-07,    -6.5271920328820160e-10, } , 
      {    1.5254874184404375e+01,     6.7277362613073010e+03,    -5.9190745702755428e-04,    -6.2730478531075839e-07,     2.8374738795665917e-09, } , 
      {    1.4810642173877905e+01,     1.4985440010349243e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {    5.7302300650072180e+00,    -1.8400313363726069e+04,    -6.7268892568022990e-04,    -9.4137416621326002e-07,     4.5903565287126083e-09, } , 
      {    1.4091481494075369e+01,     3.3943067653707862e+04,     6.0650574955639625e-04,     1.0034918887327816e-06,    -4.9983757227343934e-09, } , 
      {    3.4847282175000032e+01,     1.6657054216872126e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7020103986325239e+00,     9.4263993030809634e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.3565695433694469e+01,     1.5525833609174202e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {   -1.9141871792679918e+01,     1.6657711481621671e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    7.5269911981034010e+00,     8.7980917180354190e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6038717904870801e+01,     3.7698455102732592e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.3737511238527045e+00,     1.4914446704936252e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3418981958160295e+01,     8.3237603923658535e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.1678291543634565e+00,    -1.7429361443106488e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.3660362226114707e+01,     7.8198285452716436e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3840924013901482e+01,     2.4983773073928940e+04,     6.0916963104930472e-04,     1.0028754966186189e-06,    -4.9439359944826033e-09, } , 
      {   -2.2650114556238119e+01,     8.8375543039752556e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0955836272517693e+01,     1.4598977996255388e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.3540742634066589e+00,     7.4684943050342481e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2714195876608500e+01,     8.2335195925391963e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.8401215801460999e+01,     1.5367952654101717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9562606106067996e+01,     1.5407767555999628e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.8188005263451828e+01,     8.2317886240619573e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3007652340531570e+00,     7.3073628698849197e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2385563093947654e+01,     8.4177089382382401e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1549827326810430e+01,     8.1937046906412834e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.7732470153033937e-01,     7.3849151308970431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.3594696788046903e+01,     1.4898904676647822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.1721703709620918e+01,     8.4238632567077038e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4257488625849874e+01,     2.4988375473811760e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.3695013405188583e+01,     1.6168760675085927e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.3880160958909229e+01,     8.2733344944371092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3729589341941946e+00,     8.0132227159425856e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1576367106415272e+01,     3.1415379252277162e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.7204932946811184e+01,     7.0790761313761777e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0804062542200455e+01,     6.5327861831016098e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.0557683870313738e+01,     8.4636781586056168e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9875639741022219e+01,     3.4227046505191305e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    2.1275270380567768e+01,     9.3369020681637139e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.2887261627176322e+01,     8.3840483548097927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6225760302705432e+01,     1.6486530583708201e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.6044848830626869e+01,     7.0392612294782657e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7118898312051449e+01,     4.7114589473996428e+04,     1.7488234476989361e-04,     6.2505680123418984e-07,    -3.2335666570330831e-09, } , 
      {    2.2207320002340349e+01,     1.4846878220194449e+04,    -7.0179029876085535e-06,     1.3554226970987963e-07,    -8.1162173540866301e-10, } , 
      {    1.7252448099977769e+01,     2.2057527524544574e+04,    -9.3947001199933423e-05,     9.1943799550956940e-08,    -5.0314933452909859e-10, } , 
      {    1.7378625753371637e+01,     2.5128049625033367e+04,     8.0781468652675530e-05,     3.1406938090250157e-07,    -1.7528826491460159e-09, } , 
      {    4.8148148982346353e+00,    -8.1867160734604331e+03,    -5.2838816239662921e-04,    -6.8880611571611726e-07,     3.1910533453365872e-09, } , 
      {    1.4150168772531874e+01,     1.5508997242857731e+04,    -3.5366531172850963e-05,     9.8565433288919105e-08,    -5.8187338006766507e-10, } , 
      {    1.6607746880513808e+01,     1.4704902869031434e+04,     3.6907785164669451e-04,     5.7362951127134225e-07,    -2.7666910821245997e-09, } , 
      {    8.7119194382938812e+00,    -1.3241780250014071e+03,     6.1829154629133588e-05,     7.2808155076195450e-08,    -3.4916281313508780e-10, } , 
      {    6.2701135121940172e+00,    -8.3962655391967364e+03,    -9.5791016119009615e-05,    -1.7667793485013395e-07,     7.7794172898198340e-10, } , 
      {    9.5842089172099332e+00,    -1.0909473360052978e+04,    -8.5135490147375572e-05,    -1.7914350330678462e-07,     9.9570064198914267e-10, } , 
      {    8.7237113536683655e+00,     2.3729470363442226e+04,     4.6220498627279555e-04,     7.5092383823563880e-07,    -3.5990725393583728e-09, } , 
      {   -1.1427312853951024e+01,     8.3132723134837725e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0168374053607916e+01,     1.5757370827759785e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.0380094602509104e+01,     8.1538897887433714e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7492867903177096e+00,     1.5542530632824084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.7327765713488294e+01,     1.5542977947139501e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7550298969681670e+01,     1.5508972862526634e+04,    -3.5268598809266836e-05,     9.8521800057619234e-08,    -5.8260060058932940e-10, } , 
      {    3.4413624337578206e+01,     8.3336224568810467e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.7216956621152839e+00,     7.2839526433771289e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0011460988861494e+00,     1.5797185729657698e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.1195674596966025e+00,     6.1927343106198141e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8994037387288900e+00,    -8.3352503802225838e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    7.8212367219151426e+00,     6.7434896346532323e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6849670326639249e+01,     1.5423309584288058e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9387490434700990e+01,     8.5034930605035279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.7126519160879724e+01,     1.5518481416372311e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0480863888597042e+01,    -8.3129589406241503e+03,    -4.0168256896305277e-04,    -4.7572410332788564e-07,     2.2385302097870602e-09, } , 
      {    6.6195530968006882e+00,     8.6441601333043145e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.9471791072726234e+01,     7.1897899917488630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2684836132473531e+01,     1.4917076537252435e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.0723751817480604e+01,     1.5447582457897541e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9199974589522053e+01,     7.0946181596646074e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2830031295225046e+01,     1.5558753798205793e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.2885462259928225e+01,     1.4325954021788621e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.2022700309964520e+00,     8.3500213326013018e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7912876357650021e+00,     8.3073615166455984e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0686168089481981e+01,     1.6173363074968747e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.8364361323777089e+01,     7.1188910332740907e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4247005638175430e+01,     1.5526754781757798e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3957652483576677e-01,     6.2759623957866688e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0184042362781332e+02,     7.2383357389678267e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1073962438201495e+00,    -9.4433199838885539e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.2298974975696868e+01,     3.0943521969137611e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.0420974151578378e+02,     1.5567027163591278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0452096118630241e+00,     1.4874631803038339e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.8146756497959977e+00,     7.7627406466080629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2765392398746418e+01,     1.4498015508735472e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.2315592990041598e+01,     1.0983908375631603e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.8796353423257344e+00,     1.6499648494906469e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    8.7393498675183618e+00,    -1.1153399047319820e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.2515452801552209e+00,    -6.5589555991336965e+00,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    8.9152505705988183e+00,    -1.1139172137982293e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.2804722809591048e+01,     4.5999960914731324e+04,    -1.9588564687859238e-04,     1.8573677544440189e-07,    -1.1696178538135666e-09, } , 
      {    1.7782931486217301e+01,     1.2543219865074509e+04,    -4.2895952329359408e-04,    -3.7905147961275440e-07,     1.8192487939531000e-09, } , 
      {    1.9851765451953547e+01,     6.5181867955709977e+03,    -1.5931031074993473e-04,    -1.1517660444477507e-07,     4.2436226321198777e-10, } , 
      {    1.9478625586794980e+01,     3.2271124814810202e+04,     5.0353762083667226e-05,     3.4451185004809966e-07,    -1.9024525179051186e-09, } , 
      {    1.7095104578896809e+01,     5.8188971647754306e+03,     3.1401448060125284e-05,     1.0325062422179354e-07,    -4.9873268189419070e-10, } , 
      {    1.2926883731274128e+01,     1.5576511337105858e+04,    -9.6999821074816303e-05,     2.5670011750123932e-08,    -2.3416500797590593e-10, } , 
      {    2.2132860542001772e+01,     4.1157130519066202e+04,     3.8803016567023641e-04,     8.1489073709764838e-07,    -4.1704109181355277e-09, } , 
      {    9.7846781261474227e+00,     4.7008550445021028e+03,    -2.0782005791452577e-04,    -1.9106672009178387e-07,     1.0457237176060245e-09, } , 
      {    5.6439786892941566e+00,    -1.7143709453297946e+04,    -6.7801668866604686e-04,    -9.4014138198493466e-07,     4.4814770722090281e-09, } , 
      {    1.4295318017983538e+01,     5.7479094891939221e+03,     2.1944932537727680e-04,     3.2229424500252485e-07,    -1.4762673552521590e-09, } , 
      {    7.1855286789666000e+00,    -1.8609862829462370e+04,    -2.4009177940261027e-04,    -4.2924598534727668e-07,     2.1772449123580036e-09, } , 
      {    4.4137085755646979e+00,    -5.5731427963255283e+02,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    7.6415526048675524e+00,    -1.8139181900606736e+03,    -1.8005623283842597e-04,    -2.2089279712321928e-07,     1.1408538581133376e-09, } , 
      {    2.8010664405243553e+00,    -1.6586395173665391e+04,    -4.9263269284180379e-04,    -7.2048136909004076e-07,     3.4495026705992703e-09, } , 
      {    4.6534911106460859e+00,     8.5044838432356883e+03,    -2.4948809356431073e-04,    -2.2496159594193106e-07,     1.0032734316880736e-09, } , 
      {    4.7134896401900775e+01,    -1.1142762433070307e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5104477585293935e+01,     1.6650823893647768e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.9472952495173310e+01,    -1.1149808752231806e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.1884948486744193e+01,     1.5487397359795452e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9523244570966995e+01,     7.1587059351720027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.5825097048916007e+01,     8.3279867927073010e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6819340630509259e+01,     8.3293960565395992e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2711448778852837e+01,     1.0213614179760079e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7171543708996460e+00,     6.4437686694868207e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2177198032185377e+00,     5.4350783046440974e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9266145084230004e+00,     1.4756712350595821e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3522751009149063e+01,     8.8871092316502090e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.3503030570065171e+01,     1.6640109852481233e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2385648423586151e+00,     7.1787257944216226e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0824182219368993e+01,     1.6815117203587331e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4906843351485611e+01,    -6.3017820586608377e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    4.9158099326703699e+01,     1.4739645069056172e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4842918483529544e+01,     7.3490495993405111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2923985344214543e+01,     2.3713711360264813e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4216484098684031e+01,     7.8423704504038878e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1371345466574567e-01,     3.1083207380022177e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    4.7984635197611851e+01,     3.1087809779905005e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.8336337072884611e+01,     7.1802814379031852e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    9.5267535341243832e+00,     1.5576535717436956e+04,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    1.1864474472142014e+01,     1.6623601421791740e+04,     3.3549939283921917e-04,     5.3784182584740718e-07,    -2.6465494038088457e-09, } , 
      {    5.5838489241845988e+00,    -1.6691164276702060e+04,    -2.7367023821008555e-04,    -4.6503367077121169e-07,     2.2973865906737576e-09, } , 
      {    6.5852357084070858e+00,     7.2316948916328138e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.8312709929703397e+00,     7.1773031034878695e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.8248072790912921e+00,     1.4425824530775280e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -3.3449970250915143e+01,     8.6056647694315961e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1269710559165773e+01,     6.5702132520243713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1919936073314547e+01,     9.5853065947145369e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.2564166536138837e-01,     7.0720762545323632e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.4484894822417331e+00,     1.4363691380902832e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2497069737500311e+01,     1.5677741023963963e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7019143458739340e+01,     7.7229257447101518e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3860703188831739e+01,     5.4867778124971869e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.1956519064271003e+00,     1.6654753016930714e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.5097204949312157e+00,     1.6660012681563087e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.2318720414265201e+00,     7.6474147423667155e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.6002381297761140e+01,     7.3092346974425982e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5960451934808741e+00,     6.2702865716319375e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.7160840030972984e+01,     7.2694197955446853e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.3972079473193872e+01,     1.5662198995675531e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.1356165950663751e+01,    -1.7272996765668267e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.3681926043009685e+01,     7.3888645012384241e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2867923945806510e+01,     1.5073706312527900e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.3594016615555091e-02,     7.2302722006990607e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3273518120710449e+01,     1.4430426930658101e+04,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    4.1591918440302720e+01,     7.3335075710520814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.3657472499365056e+01,     1.5637926122066050e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.6203549993198596e+00,     1.4911816872620067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.4815993385071060e+01,     1.5598111220168137e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.7995417509416889e+00,     3.2197835939287281e+04,     3.9069404716314494e-04,     8.1427434498348570e-07,    -4.1159711898837376e-09, } , 
      {    2.0385071197924745e+02,     8.3281834659843353e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    2.0228119553206363e+02,    -8.3291993832625649e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.6530586494019882e+01,     8.3931257744373070e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.3443758524471683e+00,     8.3985212684822091e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.0170539287020908e+01,     1.6644264938048633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.0785470158693986e+00,     7.1081246709359621e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1471638188889438e+01,     1.5436816095559412e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3467100228198913e+01,     8.6284123897898626e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2697245670823307e+01,     9.7299230378567008e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2298770886498119e+01,     3.2202438339170101e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.7949050093880132e+01,     1.4699830167158261e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1333751852347188e+01,     1.5717555925861872e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1448699366722419e+01,     2.3320690390571825e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.0245856876831205e+01,     8.0569869881818795e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0627349130807904e+01,     8.9566466937109180e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2667961473235017e+01,     1.5227285581300930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.6451162193007129e+01,    -8.2665582747510034e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0247387190241444e+01,     6.5883851126289865e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2329884849092808e+01,    -1.3117911198267393e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.0006114922710061e+01,     2.3245439329501114e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4117922755280707e+01,     1.4392788659779966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.4116270288540598e+00,     1.6091432071231513e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5055019914646387e+01,     8.3106424811675879e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1416484500393597e+01,     8.0171720862839666e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3934881144361206e+00,     7.2138392082006349e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.3195071577815838e+01,     8.3290504541322498e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4189365985769030e+01,     8.3283323951146504e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4972487934973106e+01,     7.2142865225160531e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.8364056512909691e+01,     1.4968127147614319e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9967044576756834e+00,    -1.2723629136055376e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0492293947060777e+01,     3.7042967534893811e+04,    -3.4551417314801009e-04,    -6.5598490824415507e-08,     1.2080587305887397e-10, } , 
      {    2.8136739396673974e+01,     2.8714276110270366e+04,    -4.9780658091033616e-04,    -3.1631736497907023e-07,     1.3567898716795247e-09, } , 
      {    2.7861810727397906e+01,     3.9414200004587037e+04,     1.9926055514658922e-05,     3.7495431919369775e-07,    -2.0520223866642219e-09, } , 
      {    1.7166196724264708e+01,     2.3314131434972693e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    1.1067473988680684e+01,     7.0045133996220429e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.4665090568171429e+01,     2.3800458039023735e+04,     2.7415710895564407e-04,     5.3188021745490748e-07,    -2.6215378660004046e-09, } , 
      {    1.2180158954215035e+01,     1.7356672480042471e+04,     1.1387305671459234e-04,     2.8301051964274080e-07,    -1.5488730521351231e-09, } , 
      {    3.5565783269938080e+00,    -9.7238571252063666e+03,     9.7584624183958920e-05,     4.1132901702272025e-08,    -9.0713487872405258e-11, } , 
      {    4.9128581893221810e+00,     1.5610328404555081e+04,    -1.2268456776715018e-04,    -1.1923216784999263e-08,     5.0023075616881894e-11, } , 
      {    1.1779052631545097e+01,     2.4986074273870348e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.3958685831366076e+00,     6.9918941612040562e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3001931867251288e+01,     2.3349076323982084e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.3456987584321194e+00,     1.4391365968846214e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1195985622123537e+01,     7.5255822036978279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8229604917713926e+00,     7.8581181939183380e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.2968364173506144e+00,     8.4346296190458324e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6351564974163804e+01,     2.3868815882289062e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.1894618840684645e+01,     1.4834816901140426e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4596023488610527e+01,     1.5307467674629237e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5437507909883884e+01,     2.3874075546921427e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.4487843588877027e+01,     7.7007361555359812e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0929384165804326e+00,     6.9143419001919319e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2292766774051978e+01,     6.9787762500057888e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5921411612773564e+01,     2.3854525033797650e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    5.0510049516969737e+01,     6.7450148879044500e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.5775827961785893e+01,     8.5260349656357739e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0309427246774122e+01,     6.8985941566774809e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.4232045724008877e+00,     6.5459403784148890e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5919110685884203e+01,     3.1571829554183154e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.0226718890919187e+01,     1.6170350529560461e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4971941344040332e+00,     8.2485093312951431e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4395154947504899e+01,     2.3240836929618294e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    9.1075129825082595e+00,     1.7285690434292443e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0473613554252447e+01,     7.2300623735823419e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    7.5031655609311958e+00,     8.4088735179517589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.7371847186885088e+00,     8.3853584035380245e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.3303166956101897e+00,     1.4966704456680567e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0018404405979219e+01,     7.4857673017999159e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.2888676272794413e+01,     2.3871117082230470e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.2864447354331325e+00,     1.4834264611607941e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1100512058784791e+01,     2.3871774346980023e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.8051457307133649e+00,     8.4308631335515183e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2282660124054317e+01,     8.6454796713295091e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5911168266681388e+00,     8.2227532302010677e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.4815085244331385e+01,     7.8596434471695557e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.6817645638027074e+01,     6.4109536444327223e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.4611657815002317e+01,     8.5658498675336850e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1207672430490970e+01,     7.1971421845507530e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    1.0409514947184986e+01,     6.1693240841120232e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2851860284051938e+01,     7.8427227663619624e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0331705195897442e+01,     1.4779459970954085e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5341666922769894e+00,    -1.6653838832985248e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.0790145319978180e+01,     1.5536195334382661e+04,    -2.5156717194807661e-04,    -1.5754229037537246e-07,     6.2395520758797261e-10, } , 
      {   -3.4122593264537400e+01,     7.7710248290328227e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.2616243757163446e+01,     1.4859089774749909e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.6128202803927252e+01,     8.1694318170318002e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0802715645135791e+01,     7.7717294609489718e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.5123897303409812e+00,     6.5850439348459240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.9468689221439792e+00,    -8.8150123988430114e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    7.1488182595125210e+01,     1.4819274872851996e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2890039483352629e+00,     9.5323374975033457e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4916209411613881e+01,     8.3198312690829080e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.2769289962754524e+01,    -8.3375515801639958e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    9.6564442470465899e+00,     9.3500199793619813e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7820986222936192e+01,     2.2756464839382068e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    2.0534680282283134e+01,     5.9972625971651705e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0981167044109597e+01,     2.2757169471298213e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -2.9684409315393800e-01,     1.6617567947348987e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -2.8158878797837648e+01,     7.8420181344458133e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.5060691409946955e+01,     8.5378885744016752e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.6003334836470902e+01,     1.6697197751144813e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.3144414530652853e+01,     8.5385932063178279e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.0463586434491063e+01,     1.5558173401121474e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.5392854321795568e+01,    -7.9587275351851258e+02,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5706968380873320e+01,     7.6848418112894769e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3794227135438643e+01,     8.0776570657564434e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.7133131419470928e+01,     8.4879510322151000e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6683111487186473e+00,     7.0382948270772031e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.1560939435524937e-01,    -9.6190880221696971e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.6026537844425405e+01,     7.8400692504624767e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8847929296931078e+01,     2.4029180068945676e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.6466781084215745e+00,     9.0099681068801838e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8239421826207298e+00,     1.4392077314313090e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.3865453391622236e+00,     1.5527335178842119e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.0771089303090964e+00,    -1.6369979498882660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.9053979055258786e+01,     1.7272572523094179e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.5903989929792004e+01,     1.4929988733224682e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6009527287806002e+01,     6.6394357229908692e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7363405741569924e+00,     2.3831630812707332e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4751544983891135e+01,     2.3818476617394052e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.0488784879316341e+01,     1.7665593492787164e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6691251991602307e+01,     7.9225525437321949e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1380772730976972e+01,     8.3347248990662010e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.9811256283793160e+01,    -8.3226579501806991e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    7.4529990753835254e+00,     6.4280209259723688e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.3412950931388902e+01,     1.4914775337311024e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.0298203182106093e+01,     8.9573513256270689e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7228309770119745e+00,     7.1250453517435544e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.4085493498749720e+01,     1.5560803233437655e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7046530982297270e+01,     2.2541905132244210e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5626644573347974e+01,     1.6012154583393763e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1729298191113763e+01,     7.2321118088142066e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4118780554855537e+01,     1.0122944007549690e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.1177871286889172e+00,    -8.1529233863423060e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {   -3.3816005495287158e+01,     8.7977394020773445e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.0086550365107385e+00,     1.5524705346525934e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.9408037839304924e+01,     8.3593443469821468e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    8.3212832006049950e+00,    -8.1867048137974671e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.8108603395237353e+01,     7.2294819764980220e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.4060943039341254e+01,     2.3911260616503158e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.4021850873249903e+01,     1.5113521214425813e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6785921593052866e+01,     1.6171061875027337e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9332198386894692e+00,    -4.8632097421956246e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2584319930401024e+01,     7.3200010597807268e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.8363031770860648e+01,    -1.1169297592065159e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.9350953376567304e+01,     8.1919737221640462e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4657462502765936e+01,    -1.7085226553869916e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.5613062189306895e+01,     6.3711387425348103e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.1593111838128383e+01,     1.4285786803932633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5996360321200047e+00,     7.2490077543677362e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    6.2545819999293570e+00,    -1.1172583915812884e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.8832099577279688e+00,     9.7430409490549682e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1910476333119178e+01,     1.8058614462480149e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.2602673417637291e+00,     6.5831254479966174e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1399090913892259e+01,    -1.1119987269489220e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.4424532623398312e+00,     1.3500097180504734e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4598718113416712e+01,     1.6051617169333600e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.9751495244770894e+00,     6.6380130320571161e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2295032602640998e+01,     8.3129436811089981e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.5016158395717966e+01,     5.0886287935180644e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    7.8880372596261825e+00,     7.8416591049370109e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.6938205420808941e+01,     8.4862200637378610e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3504509537080978e+01,     3.8927873400535995e+04,    -3.5350581762673563e-04,    -6.3749314481927506e-08,    -4.2513311696495410e-11, } , 
      {    3.1148954986694179e+01,     3.0599181975912546e+04,    -5.0579822538906170e-04,    -3.1446818863658224e-07,     1.1934706869241553e-09, } , 
      {    1.9195104412320152e+01,     1.2961972354552267e+04,     9.7374149111698154e-07,     1.3369309336739165e-07,    -6.4830255065329361e-10, } , 
      {    1.9564876962508041e+01,     3.1014520904382080e+04,     5.5681525069484248e-05,     3.4327906581977430e-07,    -1.7935730614015391e-09, } , 
      {    1.3867260549593954e+01,     7.0755010752035514e+03,     2.6073685074308262e-05,     1.0448340845011887e-07,    -6.0761213839777023e-10, } , 
      {    1.1794614222145071e+01,     7.1803058182342811e+03,    -1.8765893893517714e-04,    -1.5215344086573563e-07,     6.5411061855298576e-10, } , 
      {    9.1248176763383029e+00,     1.6100068569614348e+04,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    7.4690498534414296e+00,     6.9928963079556797e+02,    -1.9071175881006004e-04,    -2.1842722866656861e-07,     9.2309494510617842e-10, } , 
      {    2.0581540251778954e+00,    -8.8860057042560038e+03,    -3.3767640358656919e-04,    -4.7037888704954867e-07,     2.2679584002304091e-09, } , 
      {    7.2252870518524528e+00,     2.4567321784392589e+04,     2.6943958502267475e-05,     2.3941204948381809e-07,    -1.2404006512555586e-09, } , 
      {    5.5263935410990488e+00,     9.7948448007878724e+03,    -2.8563250150111041e-04,    -2.6017652248300336e-07,     1.0682481612303737e-09, } , 
      {    6.4112824911380928e+00,     1.4772476983604720e+04,     3.1257646000337790e-04,     4.9958857196682140e-07,    -2.3086488124859323e-09, } , 
      {    5.2077608380319722e+00,     4.6940029341196907e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.7402114996110498e+01,     1.6641840820958470e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.0335583737383093e+01,     7.7540642822559748e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.1696425539229764e+01,     1.6672924877535326e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.6846567180057910e+01,     7.2711507640219252e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1882061789559604e+01,     8.3301141155572022e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.8782996239073420e-01,     8.3272687336896961e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0912764942549641e+01,     7.4493494807109000e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.8914554767441530e+01,     2.4985721957912276e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    4.9331105232610113e+01,     2.4986426589828421e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    5.0764646463673330e+01,     6.5861075962708755e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1328096341382476e+00,     1.4205957026562399e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.6252709989486540e+00,     1.3735383795857289e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0371728023992420e+01,     1.4967415802147443e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.4484837675956275e+00,     1.3710800632056356e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.2545544614544983e+00,     7.1338807720300365e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1613972212389889e+01,     1.5462572196653486e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.9340554889994888e+00,     6.5055731869844931e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5902121357010712e+01,     8.7085944831181696e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.7987321477191831e+01,     1.5383494682390145e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.9430605371980402e+01,     2.3242785813601629e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.6388508465175921e+01,     2.3872157060072121e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.4734996768180332e+01,     1.5648692484404177e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4239811640097306e+01,     6.5061254765169770e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5388502656699124e+01,     2.3870734369138368e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    6.0113448967319760e+00,     6.0349999562793819e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0847728798130776e+01,    -1.1675976564762971e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6990419529673677e+01,     2.3924414811816438e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.3331942645552619e+01,     1.8451635432173138e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9647754754378060e+00,     1.6135013458623740e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.4286984435978294e+00,     1.6106627525213480e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    7.3594137447425627e+00,     1.0213285547385307e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.7081281600209293e+00,     7.7526415913222218e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.1664661604558198e+01,     6.7848297898023629e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.4293069477674109e+01,     1.4914118072561479e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1961350861553621e+01,     8.3265491864016294e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    4.1086491071902245e+00,    -8.3308336628452726e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.5408002822719908e+01,     2.3855903686316815e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1763247471378136e+01,     6.0640972351565169e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.8453177374759676e+01,     7.2226993637411788e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.0278393782695062e+01,     6.5303983501264593e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.4794285887641774e+00,    -1.1544434611630179e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.9040821652293204e+01,    -6.2865990100361807e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1564756518516605e+01,     1.4913735359469376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.9146974584852458e+01,     6.4905834482285472e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0339076459316779e+01,     7.0548032577666963e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2423268268258129e+01,     1.4836894443924128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.3622720196213315e+01,     2.3886987742893674e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.1311888827152107e+01,     1.4957106520891955e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8330607016082922e+01,     7.8446716503452953e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.7990316640042039e+01,     6.4507685463306352e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8630659379635411e+01,    -8.3277250222223847e+03,    -5.5397497672537901e-04,    -7.2644297748254036e-07,     3.4745142084077110e-09, } , 
      {    5.9911248862781257e+00,     6.0626745442227639e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5979942935712863e+01,     7.8994583490674686e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1630481442029885e+02,     8.3266559076063750e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0531056971816051e+02,     8.3307269416405252e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.0789886656667309e+00,     8.7992646553285613e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3369138580312914e+00,     7.7030136718940903e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.7665603736739627e+01,     1.4288440319832118e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.9627859083239079e+00,     7.4650972242253520e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3680493415682189e+01,     1.5021096244825510e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.1429036885512739e+01,     1.6577753045451074e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.0317818623191313e+00,     8.5639780399760057e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4166185200419946e+01,     7.6834631587703134e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5752580713903965e+00,    -1.1080696036659720e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    6.5554955595822140e-01,     8.9033185669909253e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9278895676077170e+01,    -7.9629803795824500e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.5478046416188306e+01,     1.6679924754379146e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.7137412328562014e+01,     1.6737012653042722e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.0156358095970759e+01,     8.3307689674071516e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.9161970162366863e+01,     8.3266138818397485e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.5292767815907951e+00,     1.4143495244315181e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.3987826794413909e+00,     2.5443143967590322e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7643973105633570e+01,     1.6879551553401190e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.7976155639207853e+01,     7.6831108428122388e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.7927582275773069e+01,     8.8999111110054127e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.0153389631143277e+01,    -1.0748136573671934e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    8.6214831619092109e+00,    -9.5689420492467389e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.3146462331707356e+01,     7.2942449586866514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5376350549799788e+01,     1.5537823257724198e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    4.2452826105155815e+01,     1.5547685322239387e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.3510296123003229e+00,     2.3791815910809419e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.0080478718430498e+01,     8.0338862321994802e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -7.4816360228885884e+01,     8.9725410379574223e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.4220940219825152e+01,     8.3284857093974297e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.5541018974940343e+01,     1.0515964977242678e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2585149462227648e+01,     7.9773571843860536e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3219255444537186e+01,     1.5778040905334350e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0739717311887409e+01,     1.7179752239870060e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    6.7451438634855183e+00,     1.1015106480668383e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.2378148350342557e-01,     6.5497068639092031e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.4753512135994640e+01,     1.8844656401866127e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1892572981072128e+01,     7.1980633571343469e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.5669799445799697e+00,     1.4893116796958400e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.7047100928084305e+01,     1.0973555624994374e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2262457976559755e+01,     1.7208138173280317e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    4.3269188824100970e+00,     6.6026759611203952e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.8249698164302657e+01,     6.6012973086012298e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3438945460682652e+01,     8.8636990051424182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1330584247248241e+01,     1.3869707923689930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2701270120998529e+01,     8.9047412579246775e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.1846803651263009e+00,     8.3989733845695646e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    1.0429039725625710e+01,     1.4286850465357584e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9922849344995925e+00,     7.7533529367890978e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0453958659019944e+01,     1.4350128843824263e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3476078784514733e+00,     8.9577103551358687e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.4984163977967995e+01,     8.7579245001794316e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.2029251522271824e+00,    -3.9814901897912250e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -5.7426392184208037e+00,     7.1986437542186668e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.5484755847732197e+01,     1.5622936383310103e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2936624226441465e+01,     2.2216870415428866e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.6604430621149247e+00,     1.6604413752035707e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.9762344197555226e+00,    -8.3418093358217156e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.8432812154298777e+01,     7.2075039097592116e+03,    -4.0385957971040279e-04,    -4.0826116453002718e-07,     1.8599392062086235e-09, } , 
      {    8.2086625886522988e+00,     6.6387243775239922e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6417767688333833e+00,     7.6996724941110297e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0207333884158203e+01,     9.1318006455490722e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3501915360435326e+01,     7.6052120074936529e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.1146892887558032e+01,     1.4954261606834163e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.3271607101938585e+00,     7.6605689376799946e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2750697320666754e+01,     9.6654886880428458e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4649223469346680e+01,     7.6450269093915649e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.9077324436959442e+00,     8.9040299124578014e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.0401312670491727e+00,     1.6710351946458093e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    9.5200519608436949e+00,     8.2764336728791350e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4738789961104288e+00,     8.3809491763677652e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.6946448818549316e+00,     1.5208693046510067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -9.3639378095463215e+00,     7.1960139219024823e+03,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    5.1137750369508073e+00,     8.0934048092708945e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0198107106805833e+02,     8.3287894761972893e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -9.0987082829444532e+01,     8.3285933730496108e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1327923837501240e+01,     6.6924048202020604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0569803112286500e+01,     2.1461653777799097e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0701641481060793e+01,     8.3220800250713455e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.2793641343291942e+00,     9.3734302058697722e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.5815161411387688e+00,     1.6579830588234774e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.2341108370404093e+01,     7.5653971055957409e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.9563546049087925e+00,     6.6284150962685044e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0799915537486866e+01,     8.1889718027459076e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0726597922786217e+01,     9.0371811029973032e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0854834742982042e+00,     1.6734935110259023e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.5162109126814357e+01,     2.3951075518401070e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.5168066140056766e+01,    -1.1301705875535354e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    2.6174924581163186e+01,     1.9237677371559115e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6802474377260674e+00,     7.8070333794671660e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0892839929309233e+00,     3.5337070936721659e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.0869985064207349e+01,     1.5558486773981091e+04,    -3.1557333732456025e-04,    -1.6288750665370941e-07,     5.9452701714462378e-10, } , 
      {   -3.3711562181290304e+01,    -8.3459644213891170e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.4492551914840108e+01,     1.5858222998662659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.7069242789947596e+00,    -1.0616594620539142e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2194149163209680e+02,     7.2120273483412693e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7425028025265071e+01,     8.3267425406401144e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -9.9575775955502195e+01,     7.2160983823754195e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3101460154935680e+01,     8.8341401344390179e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.0943527717947905e+00,    -7.7552261012124291e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.9472235053904186e+00,    -8.4684110465009944e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2430640944010985e+02,     1.5540718772964719e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.7947037772132269e+01,     1.5527021805982495e+04,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    7.1446128363779327e+00,     1.6673382357470899e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -9.7229692438156931e+01,     1.5544789806998868e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5778343267560935e+01,     8.3973835300950304e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    9.9751486288625255e+00,     6.5082030193006776e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9670028602962959e+00,     1.2036460728798950e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.9895793145116201e+01,     7.5328186711049366e+03,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -4.0203386009950883e+01,     8.3276277631984995e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.1197726404909709e+01,     8.3297550860484007e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2063398147402559e+01,     1.5126323093781017e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5142931620527115e+01,     1.5702013897573444e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.7514013114192522e+01,    -1.0990865309766759e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.7156785646192823e+01,     3.6759761870474335e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.2749201970124055e+01,     7.3733224729499934e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.6707582025907872e+01,     1.4283837919949297e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.3060898059101511e+00,    -7.9694518823856242e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    4.3392682840038495e+00,     6.8800016218866167e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.4315543465320921e+00,     8.3306403086067858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1073312034152933e+01,    -8.3345914787086767e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.5417782146571524e+01,     2.3793893453593122e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.5872135027404703e+01,     1.5529636378783527e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.3908199658516931e+01,     1.4679160089583695e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8608729116720543e+01,     2.2599082800999709e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.8629293405386846e+01,     8.4019859299778509e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -1.9029712783096567e+01,     9.1716155474469833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.3753026836596213e+01,     7.9375422824881407e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.1369203525493852e+00,    -2.3528661535255571e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9240844863961839e+00,     1.0447387812463210e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.0521054613219963e+01,    -9.4410187839471473e+03,    -6.7535280717313838e-04,    -9.4075777409909734e-07,     4.5359168004608182e-09, } , 
      {   -1.3540705211442406e+01,    -1.1942583630609302e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7818123965399920e+01,     1.5525481293216126e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.2873004071725575e+01,     8.3290795664169855e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.1878719017532219e+01,     8.3283032828299147e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.2663649502274748e+00,     7.7430436555336100e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.6367485939339836e+00,     1.3135462144222005e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3075130475387228e+01,     1.4307469027868559e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.1932607307584448e+01,     1.6031376464142766e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.6743444218924473e+01,     1.4978436428869807e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.0981606770738176e+02,    -8.3304223931006873e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    6.5406934568681692e+00,    -8.0470041433977476e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    2.6961932967896420e+01,     1.0908985946935667e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.3378329391134871e+00,     6.0633858896896400e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0410818523531715e+01,     1.3893118150197721e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7730539343138496e+01,     7.2091318331007478e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.9074259173874019e+01,     8.8600962091075016e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.5284079658035488e+00,     7.6136427906504650e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.5782567005690936e+01,     1.6171414190985413e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.8135488478343440e+00,     2.0426571094770613e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -9.4137482299221816e+01,     8.3667753580441240e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.2289405194903722e+00,     2.4356780791700485e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.9816334771251263e+01,     8.2972111971097002e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -6.0231744401923450e+01,     8.8202813072095887e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3458272866937662e+01,     1.4445398727482358e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.8099369030495708e+01,     8.4464051618399481e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4670876187240019e+01,     2.2773737836147735e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    8.0953588662931413e+00,     1.7789845607142468e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    6.6337395825898504e-01,     7.6202017462495987e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -8.2977156759937813e+01,     8.4065902599420351e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9632656089120122e+00,     9.1922856250215500e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0875554257328021e+01,     8.6879309668612768e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.7446056756013423e+01,     1.6587493071228117e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.3160393978529630e+00,     8.8503494697797341e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.9666332188989948e+00,    -1.7422247988437725e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.5209303760521800e+00,     7.5426494852374744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.5386087869285287e+01,     3.1085156264005516e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    4.0969112778834011e+00,     7.9122602284226723e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.3257312940937950e+01,     3.1085860895921662e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {    1.4622320408059288e+01,     1.4445757756991157e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4494399265059347e+01,     7.8953395476150790e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.2538006192391478e+01,     9.0521708417532482e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4393534382255362e+01,     8.6103787058491525e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -3.7748507527514143e+01,    -9.1245641781419617e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0355476961280750e+01,     1.4427773414758614e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.8009827507579402e+01,     7.3109656659198372e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.8805092935195944e+01,     1.4428478046674763e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    6.2092515430749788e+00,     6.5644253723349493e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7596227690536729e+01,     1.9630698341252100e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4062267474366354e+00,     1.6496361397840737e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.6454174285387229e+00,     2.8286990488630363e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4177087942099375e+01,     1.0058509657735831e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.4077075325948059e+00,     8.2419503756960075e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    3.3320805071262861e+01,     7.2135105758258615e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0955187961590001e+01,     7.2146151548908274e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.3695756412478637e+01,     9.0123559398553352e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.7131520148729944e+00,     6.7976316691575666e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0461559322905730e+01,     1.0107675985337697e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.8054613175004519e+00,     8.3162399051461416e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.2359448703166400e+00,    -8.3411429441007604e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {   -1.6495155905951211e+01,     1.6097767369672936e+04,     2.7149322746273553e-04,     5.3249660956907016e-07,    -2.6759775942521942e-09, } , 
      {    1.6207462039861085e+01,     2.3948997975617367e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -2.6957972524173098e+01,     8.9397260129033239e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0224956876045178e+02,     1.4938719578545733e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2436538196576052e+01,     1.6026774064259946e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.7512346488722649e+01,     1.5490496538237479e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0964399538046990e+01,     3.2199784823270620e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.5147744338011012e+01,     7.1618051136140293e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.7869270748109898e+00,     5.8772656019389506e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.5176699460062039e+01,     1.5153336116323724e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.5672116023166275e+01,     2.2130810770236010e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.2161829415207752e+01,     8.9400783288614002e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7463868458539025e+01,     3.2200489455186769e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    1.7124883509986073e+01,     2.2206061831306721e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    1.4847675838671385e+01,     9.2567199748354051e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4145752991718791e+01,     7.8450002827200724e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.0096605748977218e+01,     7.2189938976159428e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -4.2331184172708284e+01,     9.1941574525792294e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.2070397961795891e+01,    -8.3288971398494687e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    3.7886389884652104e+01,     1.6102369769555757e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    9.1320064796024933e+00,     8.2691728475520376e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.9124032280644386e+01,     2.3243849475026578e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.9381241226998604e+01,     1.7283389234351031e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    8.4306002267525528e+00,     1.3107076210811745e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1063615929243610e+01,     3.2217200185560241e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7156325855827117e+01,     9.1016154528111583e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9048909576808896e+01,     1.7215800656273659e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9958445397278595e+01,     7.6207540357820817e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.2205060553791078e+00,     7.2663206171026595e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0040312975217506e+02,     1.5541023321504554e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.1715045698486280e+01,     7.6846360960634556e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2119854050122065e+01,     1.4405583825584445e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.4424902875004060e+00,     8.2757318031511695e+03,    -6.4006165376483609e-05,    -5.3452162783369673e-09,    -2.9428190443348850e-11, } , 
      {   -1.7577030882264598e+01,    -9.4456211838299678e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.3138693043616314e+00,    -8.2047384977381789e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.8898744034835406e+00,     8.7451226208242279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1285286097387042e+00,     5.2969097211191368e+01,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.0637199543595095e+00,     1.6657159192089188e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.1641683554367287e+01,     1.6657606506404609e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.8122086490142053e+01,     7.9379716548718716e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -2.2724621499018678e+01,     8.3151849133520955e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.3719069569498316e+01,     8.3421979358948047e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.7282076154260837e+00,     8.9143391937132892e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4627780251185619e+01,     8.2058325493934753e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2966407837759725e+01,     2.3322767933355524e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    9.5607728912293659e+00,     1.5595012041726110e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.1374548336597407e+01,     9.0919857436511593e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1449338618913636e+00,     8.9543691773528080e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9041217301363638e+00,     8.4273068740268518e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.1189208571942919e+02,     1.5542246331342680e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {   -1.8481503321562317e+02,     1.5543262248620909e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    5.1381962846934126e-01,    -8.6359420039809891e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2165002214270650e+01,     8.4526443515087212e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.2759070351529314e+00,     8.3356950061028856e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    8.8020602879189298e+01,     1.4914439991385527e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.9700820476806165e+00,     1.4444335066057407e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.1762352359812844e+00,     8.7838455441888837e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8686685022136256e+01,     2.2234447764716981e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.9364984722081431e+00,     8.3478790943794811e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.3669820250243312e+00,    -8.3095037548674209e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2559090742503887e+01,     8.8974804325975801e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {   -1.9814367729334627e+01,     8.2802400243427037e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -5.5729392347136653e-01,     1.5604887439854243e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.0719957866519472e+01,     8.3288409106041308e+03,     2.4721315045337657e-04,     2.9246816797108939e-07,    -1.3811372147448460e-09, } , 
      {    3.2867256112156078e+01,    -8.3285419386427693e+03,    -5.7371665071275779e-05,    -2.0896958033822003e-07,     1.0908307824964557e-09, } , 
      {    6.6352538010469999e+00,     8.3460042873583734e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -8.5103995356196727e-01,     1.5530135051563808e+04,    -1.2801233075296722e-04,    -1.0690432556673935e-08,    -5.8856380886697700e-11, } , 
      {   -1.1409456628787321e+02,     8.3283458222161025e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2508886069582640e+02,     8.3290370270307976e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.3907693798189413e+01,     5.9570953793091840e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.3676006109711394e+00,     6.8127668225673460e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.1880814322962701e+01,     1.7298182331123817e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.4662797664434898e+01,     1.6600294950583320e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2555558015037697e+01,     8.0310922092562214e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.7468089098663143e+01,     8.9407896743282763e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4386323674031626e+01,     6.6497450042463561e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.9533385463645271e+01,     8.3297873054909360e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2483714889800002e+01,     3.6147409882490110e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    9.2980949719569228e+00,     1.6639750822972430e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.5878927832507834e+01,    -8.3408973493474296e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    9.9154182402828521e+00,     7.7894013531926967e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.5124483658651698e+01,     2.3243490445517775e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    2.2075089349462935e+01,     2.2754187323023958e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -5.4187648423123242e+00,    -1.6641383341022902e+04,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    1.1394086201300203e+01,    -1.1921808202772299e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.4309342020126365e+01,     8.3164854998994724e+03,     2.1412156239145977e-04,     3.2352702923085016e-07,    -1.5851468117557385e-09, } , 
      {    1.2197772733028341e+02,     7.2157938338355834e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.7634766552210799e+01,     1.5480621140109348e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.7109943851014947e+01,     6.4384961333397314e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.1084554227197565e+01,     2.2759446987656323e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    3.1199513634400720e+01,    -1.0349987554692811e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.1107650469599253e+01,    -8.3275955437559660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.0221933665994083e+00,     1.2714055241118758e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.7093228163868233e+00,     2.2126208370353186e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -3.7392797442499287e+01,     1.6028722948243283e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    5.5719366946168467e+00,     7.1360659784659183e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5766294057456587e+01,     1.6011802267435691e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9017475809267264e+01,     2.0023719310945089e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.9057652644470604e+00,     9.4786570548252785e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -6.9442446012224792e+01,     1.4914453418486977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5941262638045572e+01,     1.4914348653362411e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.2822836772442912e+01,     6.8246446917002750e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.5768531983737695e+01,     6.5860839126875744e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6855724550010569e+00,     8.2750109819453828e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8967185534265049e+01,     8.4287295649606040e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5125911781187503e+01,     2.2667799641725353e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.5228814722196368e+01,     9.0214333594828495e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.2861274161193492e+00,     8.3823718673015173e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.5709477627753756e+00,     8.1114384932116045e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.0687037458827534e+01,     9.4800797457590306e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.8779264427298990e+01,     2.4393815105228405e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.0930188120129065e+00,     1.4233170022679516e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.0981962529168392e+00,     6.0214374192008127e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3487021579793769e+01,     2.2457096190173725e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.4407860866893078e+01,     8.2330902201554654e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.7938825289071598e+00,     8.8199281715854559e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    2.8383237883126817e+01,     1.1302006916628650e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5961448221285103e+01,     1.6641173513906182e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.7006846102931554e+00,     7.7643735635114617e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.7143833190597704e+01,     7.9392732509653806e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0234281430039253e+01,     2.4969153593062754e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    7.2825976233328333e+00,     5.4066923712338394e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.7928457894400225e+01,     1.5555373528399781e+04,    -4.3540214947000418e-06,     1.3492587759571698e-07,    -7.5718200715687324e-10, } , 
      {    8.6613745851933377e+00,     7.6408652625064933e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    3.5651296543246673e+01,     1.5542202000449310e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.5744511683451101e+00,     1.5543306579514279e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2213828292295405e+01,     1.4935776612914104e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.2363114922558452e+01,     1.5541690628556844e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5920347741917054e+00,     1.3188431241433196e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1363969508005106e+01,     8.4292412282353416e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.9441778800820352e+01,     1.5543817951406745e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.7107945036415373e+01,     8.4733078677656158e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2400366965464448e+01,     8.3474269782921256e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -3.8638474909597438e+00,     9.5454916928166240e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.6393597927296497e-01,     7.6977540072617230e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3904586162115629e+01,     4.0077619579419988e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    8.0784520573617193e+00,     5.9581590407341346e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9839542402129354e+01,     6.6100281539222833e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3241014953195263e+01,     2.1228392028053695e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0364905540702130e+01,     7.2119853225746447e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.2729773377959603e+01,     7.2161404081420451e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.2726803510156770e+01,     1.1371704643973499e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0275887918553181e+02,     7.2123318968811072e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {   -3.4783706151641475e+01,     7.2129992039333938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1619844622437135e+01,     6.3504686649602463e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1671561929739134e+01,     8.9393669833945241e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.7151164448189562e+01,     7.2151265267832960e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3081553983792553e+01,     8.0001048047443182e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6771188458067940e+01,     9.4144543291758382e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.7256702879118251e+00,    -1.7632026274469709e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.2748082696379456e+02,     1.5544485258459034e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.9001565496482037e+01,    -1.6633063405148241e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.3225557056407006e+01,     7.2297953493576424e+03,    -4.6786574508688643e-04,    -4.1360638080836410e-07,     1.8305110157652746e-09, } , 
      {    3.5000361262375563e+01,     1.7287991634233851e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.3161385274160455e+01,     7.9978036048029080e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.0271264372647838e+01,     2.4422201038638665e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    1.8851279035486082e+00,    -4.7057323070511143e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4854593825184601e+01,     7.1118911564302762e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.0886286137747838e+00,     5.3821092074329063e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.6063427820341856e+00,     1.2321034271425769e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.0734377659613255e+01,     8.0957060092123047e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2226114869932665e+01,     8.3544305597715611e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5594473927143115e+01,     1.0451530627428820e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.4703520276662733e+01,     9.4624477194845604e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2940993408703683e+01,     1.6692719920183623e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.6785814325397119e+00,     7.7687473126747136e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {    7.9644019835585826e+00,     1.3812936056869414e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6267440505889130e+01,     9.6497409445283938e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -4.7291892544329500e+01,     8.1296169151338891e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.6176031111487340e+01,     4.6904797745389442e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.9809637861174366e+01,     8.2817920859700243e+03,    -1.8756100657159303e-04,    -1.5219707409703549e-07,     6.5338339803132143e-10, } , 
      {    1.8611475176684188e+01,     6.6255701822107130e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.1256833390820677e+00,    -8.5551615711673439e+03,    -9.0463253133192593e-05,    -1.7791071907845928e-07,     8.8682118548556303e-10, } , 
      {    1.1001227081478824e+01,     7.7740069773070809e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -2.1454304118499152e+01,     7.7007124719526801e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.4386205543192709e+01,    -1.6209335340717018e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6144591473937012e+01,     8.9930474260725914e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.1428184337223595e+00,     7.6228315785657833e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1168836302075938e+01,     7.9018890274753030e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    8.3909077727143178e+00,     7.8243367664631787e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {   -5.6149336645548949e+01,     8.7181095982815204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.0446224596666912e+01,     2.2703848058128948e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    4.2862803940853391e+00,     8.5389522358266277e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    3.7972767117092268e+00,     8.5375295448928755e+03,    -2.8030473851529340e-04,    -2.6140930671132867e-07,     1.1771276177339532e-09, } , 
      {    1.3419153209660037e+01,     2.2717002253442228e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    7.6415546899325708e+00,     7.1603824226802762e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.4148057632114543e+00,     1.3342362826164301e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.9829102054253418e+01,     8.3172014367667234e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.1398618501437234e+01,    -8.3401814124801767e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {   -7.1387000770963724e+01,     8.7804664053116758e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4528485811410359e+01,     2.2914551509680572e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {    9.4009626564839294e+01,     7.7246567131873890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.6698050065495668e+01,     7.5449506851788865e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0066588945287505e+01,     1.5489073847303727e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3292565926846045e+01,     7.7648181893917636e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3207750043759635e+01,     1.4550984605946665e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.3236759841475251e+01,    -5.6387323523168652e+02,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.8310213378800668e+01,     8.5277659341130111e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.8753154173214615e+00,     1.5168878144612154e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8724369241840918e+01,     1.6694142611117375e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.3844764145785344e+00,     7.0848967837385135e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -3.6083081876795816e+01,     6.2795526908746854e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.5325640864174337e+01,     4.4007829276349858e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6977750249885570e+00,     5.7462714236747547e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.4650103384964693e+01,     7.2677433080364126e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.4728730556448735e+00,    -2.2123364377226180e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.0439558631656823e+01,     2.0416740280638074e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.7656501299518908e+01,    -1.1147266108389445e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    7.2954982348669528e-01,     7.1496285155444875e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.3939962379381271e+01,     7.7159258678663373e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0531329362385316e+02,    -1.1145305076912668e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.4662240130959225e+01,     2.5158601724983564e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    3.1976340509861057e+01,    -9.6505682496834488e+03,    -2.4275566089551877e-04,    -4.2862959323311400e-07,     2.1228051841062140e-09, } , 
      {    6.9687708515968012e+00,     7.4313157967701900e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    9.5841037066134174e+00,    -6.9889778018784739e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -3.1736659003310713e+01,     1.7480163588007872e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.1888948263275116e+00,     1.1928013301732783e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.3027617340287946e+01,     1.4795001999242515e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8252858992877215e+01,     9.8074752988688233e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.3498774199687141e+01,     9.1543425506813164e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.2056060498303204e+01,     7.8821853523017999e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.1937541429249606e+01,     6.5854266479380285e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -7.0509474410656551e+01,     8.1521588202661333e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.6252269512192555e+01,     1.4270391376376258e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    1.0022726857614082e+01,     7.8397406180877033e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.6958915181519110e+01,     8.2196589360975777e+03,     3.0914577314566794e-05,     3.6404077538097725e-08,    -1.7458140656754390e-10, } , 
      {   -8.8297612793960070e+01,     6.5856572287389627e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9815049014396209e+00,     2.4985362928403472e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    1.0435105947500261e+01,     2.4986785619337224e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    6.1993312962187819e+00,    -1.0370762982529814e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.6289406813304740e+01,     1.6173691707343522e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.0344378673635235e+01,     5.7604983330122809e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.0727394504684529e+01,    -7.5764309126926364e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    6.8312702165011006e+00,    -8.8330460827837233e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    9.5875083256239407e+01,     7.2147674972744944e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0608644267754148e+02,     1.6655651880769663e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -4.2520734190941234e+01,    -5.7087898663580745e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.4699601849198444e+01,     1.4485213629380269e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.3512089919125600e+01,     7.2133582334421944e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.1987686994887916e+00,     6.8401867199887047e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.6251664568931709e+01,     7.7000552072031342e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.3515353222832061e+01,     7.2009449541600789e+03,    -5.8924357553464580e-04,    -6.2792117742492107e-07,     2.8919136078183814e-09, } , 
      {    6.2778034105008835e+00,     8.3080728621124745e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7176362113940815e+00,     8.3493099871344257e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.3717503069585042e+01,     7.6329431902770966e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.0765578957326676e+02,    -1.7309684772390028e+00,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    9.0848103114716618e+00,     9.4377629372640649e+02,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    2.8226195924615340e+01,     2.3858327803406981e+04,    -2.8465876000999344e-04,    -1.2648342911561169e-07,     4.1994561057707983e-10, } , 
      {   -1.4674206261888285e+00,     7.9946301811517224e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1793366600298013e+01,     9.2724677183498570e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2433936081786054e+01,     8.6627526680951778e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.3037998743922046e+00,     8.4269072018924671e+03,    -2.4939016120072659e-04,    -2.2500522917323092e-07,     1.0025462111664093e-09, } , 
      {    1.2897964899144206e+01,     8.2173577361561656e+03,     1.8320698507689299e-04,     2.8712295169275246e-07,    -1.4105654051881947e-09, } , 
      {    1.0634198891089058e+01,     7.5295315740392089e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.9678070421870890e+00,    -8.4400251130907345e+03,    -1.2137783044775939e-04,    -2.1431479661655700e-07,     1.0614025920531070e-09, } , 
      {    1.2010855612964635e+01,     1.4914452334767735e+04,    -6.3519294630925112e-05,     6.1501330405358849e-08,    -3.5357946576999568e-10, } , 
      {    9.6553010625778324e+00,     6.5857609101442840e+03,    -2.1581170239325131e-04,    -1.8921754374929586e-07,     8.8240453285065505e-10, } , 
      {    3.9694765650382298e+00,     7.7003894694093897e+03,     1.5495628925523469e-04,     2.5010248204049204e-07,    -1.1815442703688610e-09, } , 
      {    7.0248178429149615e+00,     8.9569933798375096e+03,     1.4962852626941768e-04,     2.5133526626881740e-07,    -1.2904237268724407e-09, } , 
      {    4.6692632925281590e+00,     6.2830195521406040e+02,    -2.6638814929085117e-06,     6.1639211416266408e-10,    -5.4439728251789797e-11, } , 
      {    1.5066196890841367e+01,     1.6171056245195854e+04,    -6.8847057616742147e-05,     6.2734114633684182e-08,    -4.6245892227357531e-10, } , 
      {    1.2710642340454564e+01,     7.8423648205724048e+03,    -2.2113946537906831e-04,    -1.8798475952097055e-07,     7.7352507634707552e-10, } , 
      {   -2.0102667314297992e+01,     8.3263902246820398e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.1096952431353994e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    1.2053981300821166e+01,     1.4286150379553674e+04,    -6.0855413138016608e-05,     6.0884938291196182e-08,    -2.9913973751820592e-10, } , 
      {    6.7261374380949697e+00,     8.3996791002049576e+03,    -3.5755469554825326e-05,     3.1675253373923424e-08,    -2.5844932526268254e-10, } , 
      {    1.4366410163351437e+01,     2.3243143759391183e+04,     8.8773113131401069e-05,     3.1222020456001358e-07,    -1.5895634643906465e-09, } , 
      {   -3.2559810619081802e+01,     8.3283391086653737e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.3554037704501027e+01,     8.3290437405815264e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.6984267504343631e+00,     5.9574589549302236e+03,    -2.1314782090034278e-04,    -1.8983393586345854e-07,     9.3684426110244481e-10, } , 
      {    1.0441339165780823e+01,    -1.7429305144791660e+03,    -3.6810411015557749e-04,    -4.3993641790395058e-07,     2.1183885314713058e-09, } , 
      {    6.3250311154250323e+00,     1.6029080894032839e+04,     3.0724869701756089e-04,     5.0082135619514676e-07,    -2.4175282689895120e-09, } , 
      {   -1.2061209843137572e+01,     1.5540453090040384e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.9138261209120017e+01,     1.5545055489923205e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5109322578697897e+01,     1.5542754289981794e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.3803723933017640e+00,     1.7285684804460961e+04,     3.0192093403174383e-04,     5.0205414042347212e-07,    -2.5264077254930913e-09, } , 
      {    2.0052234660891042e+01,     2.2128515200126079e+04,    -2.8199487851708491e-04,    -1.2709982122977434e-07,     4.7438533882886960e-10, } , 
      {    1.2667516652598033e+01,     8.4706667757864652e+03,    -2.2380334687197685e-04,    -1.8736836740680787e-07,     7.1908534809528565e-10, } , 
      {    4.0126022528947605e+00,     7.0720875141953293e+03,     1.5762017074814322e-04,     2.4948608992632941e-07,    -1.1271045421170713e-09, } , 
      {   -1.4416685123360839e+01,     7.2117616654169342e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.6782626783905926e+01,     7.2163640652997528e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    1.0490530153540323e+01,     8.3294027700903262e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.0350339882464556e-01,     8.3279800791565740e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0508975159033648e+01,     8.3117707438158577e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.7421751441228167e+01,     2.4499747669819306e+04,     8.3445350145584034e-05,     3.1345298878833894e-07,    -1.6984429208942262e-09, } , 
      {    2.7769301084726568e+01,     8.3283627922486776e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {   -1.6775119027572888e+01,     8.3290200569982226e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.7845419650298261e+01,     8.3155735134251827e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.2745925720367595e-01,     8.3260615923072655e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0566343328242020e+01,     8.3313212569396328e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.3528013738011850e+01,     8.3131493963350204e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.2534055802379257e+01,     8.3442334529118798e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.7747112739289154e+01,     1.6655081649305488e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.3452507009725196e+01,     1.6659684049188309e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.9737565596938751e+00,     7.2114330330421608e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.3557361313207714e+00,     1.5540124457665610e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7478476566798420e+01,     1.5543106605939871e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.6389136132644161e+01,     7.2166926976745290e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0406443229812584e+01,     1.5542401974023718e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.8717856643643351e+01,     1.5545384122297977e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    9.1535859219528973e+01,     1.5527212261693365e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.9173011312966409e+01,     7.1985208370699147e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.6808878741678143e+01,     7.2296048936467741e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -6.4461842135847959e+01,     1.5558296318270224e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2294293157827692e+01,     8.3640284955601728e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.4662165672593963e+01,     8.3654511864939250e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.3296219461270013e+01,     7.2144151813164190e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0936488815428742e+01,     7.2137105494002708e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.0597927019090690e+01,     8.3114184278577814e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    4.5910020379944072e+01,     1.6657735165204973e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -3.0204799867286003e+01,     1.6657030533288827e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -8.7338564485088028e+00,     1.5543082922356567e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    3.5810908952314804e+01,     1.5542425657607024e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.7905782583552234e+00,     7.2133515198914674e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2574215912925125e+01,     7.2147742108252214e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.0264754313416593e+00,     7.2309835461659377e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.1093161454708387e+00,     8.1709570702830179e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0693414706548147e+02,     8.3269604561462111e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    8.1203069316127277e+00,     8.3221324690243164e+03,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -9.7056814930944775e+00,     2.3869144514663836e+04,     2.3840163940081873e-04,     5.6355547082883098e-07,    -2.8799871912630872e-09, } , 
      {    4.1493905078323991e+01,     2.3873746914546657e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    8.5553868304556033e+00,     8.3110593983489816e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.8828957023227977e+00,     8.4864257789638814e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.9181387107049297e+01,     1.4916747904877662e+04,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3472396619671759e+01,     8.3647398410270489e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.5143545529983928e+01,     1.5385019935641363e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3455472612088265e+01,     7.2137342329835719e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.1089646631458930e+01,     7.2143914977331169e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2788563340522931e+01,     7.0563285110179122e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2725510399484714e+01,     1.5465202028969670e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2018083947064923e+01,     1.4912145504994842e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -1.3564574959625421e+01,     8.3198312690829080e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {   -2.1417276713988816e+01,    -8.3375515801639958e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    1.0372067502311969e+01,     7.1365106043462201e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8040309688202676e+01,     8.3124820892827338e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3822557506652910e+01,     1.4991998965948376e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7223001748351674e+00,     8.2511391636113258e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.0117666329427333e+01,     1.4914094388978176e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.2792810970431248e+00,     8.4062436856355744e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.8511436495392850e+01,     8.2490616208276260e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    2.9508184968667710e+01,     8.4083212284192741e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.5060647910080442e+01,     1.4915158050403128e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5456071699838672e+01,     1.5020384899358634e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1371611494712793e+01,     7.1610937681471532e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2035842259619885e+01,     7.2916151263704687e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3726003691721454e+01,     1.5489785192770603e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.4392913612092807e+01,     1.5620306550993919e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -8.3700526473293877e+01,     8.3044185510139669e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.4695717608492643e+01,     8.3529642982329315e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.1459716241530934e+01,     6.6633075413249253e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.9752562200571671e+00,     7.2271807765566118e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -7.4059037336156308e+00,     8.2888765227255371e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2726047624015257e+01,     6.5864666257796780e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.8399937025680369e+01,     8.3685063265213612e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.5505113392093364e+00,     7.3717972196987757e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.0450252915807637e+01,     1.5463124486185970e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.5571069999167426e+01,     8.1578391590847514e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0968834306448134e+01,     7.2670319625695356e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.2802427748963433e+01,     7.1344330615625204e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3100225105139449e+01,     6.6916934747351834e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.3328960180852091e+01,     1.5595723387192986e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1914123246531247e+01,     1.5700488644322226e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.7390274682510194e+00,     7.7779361005900309e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.5599954687709474e+01,    -2.3011999414102320e+00,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    5.6908852473847418e+00,     8.2757223274122589e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.4225959070947782e+00,     7.8063220340002890e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6262121702973531e+01,     8.3512333297556943e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.2987036835043089e+00,     8.3816605218346413e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4560886876113530e+01,    -8.3441105357631277e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    3.5179669856014279e+01,     7.2936926691541685e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -7.5619384790954953e+01,     1.5518481416372311e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -7.7964829593029862e+01,     7.1897899917488630e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.7541078257828971e+01,     1.5622384093777619e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    6.5872041425140715e-01,     1.5502939388083882e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.7130588318488535e+01,    -1.1142762433070307e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.9476608692720887e+01,    -1.1149808752231806e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    5.2835390131035895e+00,     8.8794467486568683e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.6913660967381929e+00,     7.1742479634604324e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.2175164116029606e+01,     6.5854029643547265e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0033558263086663e+02,     7.2383357389678267e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.0269999774941373e+02,     1.5567027163591278e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.8330382461950066e+01,     7.7000315236198321e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3193011574970395e+01,     1.5542042944514918e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.5680382578126109e+00,     8.8510608152466102e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3853608112306761e+01,     8.5508601287777383e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    2.4054197953482301e+01,     7.2538777672562564e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    3.9242125688301339e+00,     8.9562876642021165e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.0287627068522722e+00,     7.7010951850447836e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -3.5491710780446084e+01,     8.2716035259598684e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    7.2359592243244748e+01,     8.2733344944371092e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    9.2049799288769108e+01,     1.4898904676647822e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.6416965923340811e+01,     1.5582569191879707e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -6.1366403050817574e+01,     8.3840483548097927e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.3816239203474373e+01,     1.5543465635448671e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    2.9758573101089638e+00,     1.6656671503780024e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    1.2729023994758302e+01,     1.6658094194713776e+04,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.6825818243459061e+01,     6.5880564802542103e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -2.1985039739287686e+01,     2.3871093398647172e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {    5.3771680815546588e+01,     2.3871798030563317e+04,     8.6109231638492552e-05,     3.1283659667417626e-07,    -1.6440031926424364e-09, } , 
      {   -3.1554252056952325e+01,     8.3297873054909360e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.1035838203219168e+01,     1.4445046411524281e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -3.3123769125486490e+01,    -8.3275955437559660e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    6.1189925197427272e+01,     8.2335195925391963e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.4373575604487058e+01,     6.5834540803713917e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -5.0197136261324260e+01,     8.4238632567077038e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    3.1145226431118196e+01,     7.7026850395193178e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    5.0017271658963402e+01,     8.1937046906412834e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.0370779739960071e+01,     1.5487397359795452e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.8011360701063225e+01,     7.1587059351720027e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -3.9024620241944881e+01,     8.4636781586056168e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2859143111663888e+01,     1.6640462168439306e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.4419508090977779e+01,     1.6657711481621671e+04,     4.5687722328697854e-04,     7.5215662246396416e-07,    -3.7079519958619525e-09, } , 
      {    3.0124911599704571e+01,     1.6657054216872126e+04,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    6.9205578568906162e+01,     1.5447582457897541e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.2991885352035171e+01,     8.3132723134837725e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    7.5289271427743270e+00,     7.1787257944216226e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    8.9731227414554525e+01,     6.5702132520243713e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    6.6847957204145089e+01,     7.1188910332740907e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    5.8042136863028446e+01,     1.5407767555999628e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0059541622536941e+01,     7.6980826396364973e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -4.6662267554201328e+01,     8.2317886240619573e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -5.5651333418991598e+01,     7.2694197955446853e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.9632009970904765e+00,     8.2588016466046665e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.1290452989563661e+01,    -1.6920680807593385e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -1.7004774959544328e+01,     8.9546978097275824e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {   -1.1217693891360936e+00,     7.1773031034878695e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -5.3301575922825755e+01,     1.5598111220168137e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    5.5682501268355459e+01,     7.0790761313761777e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -2.0401268728448873e+01,     7.7690759450494879e+03,     1.1920081970040937e-04,     2.8177773541441549e-07,    -1.4399935956315436e-09, } , 
      {    3.7081387633941659e+01,     7.7736783449323084e+03,    -1.8538399582424301e-04,    -2.1966001289489395e-07,     1.0319744016097581e-09, } , 
      {    4.4137075152033283e+01,     7.2313358621240131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.0198089932479242e+00,     2.2754515955398729e+04,    -1.3236635224766726e-04,     1.2423544503904303e-07,    -8.1603838804357100e-10, } , 
      {   -2.3414583322969190e+01,     8.2092467189297131e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    4.7179613844882823e+01,     2.2759118355281553e+04,    -4.3695116777231962e-04,    -3.7720230327026641e-07,     1.6559296091977306e-09, } , 
      {    2.4583731126599639e+02,     8.3303133299396268e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4412582702287203e+01,     8.4481361303171871e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -1.0165885732100733e+01,     8.3908245744958967e+03,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8491847319211701e+00,    -8.3463234508979167e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    2.4425294302210955e+02,    -8.3270695193072752e+03,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {   -5.5237138319737227e+00,     1.5533894134441252e+04,    -9.9274764185750444e-05,     9.3176583779282273e-08,    -6.1202879103267822e-10, } , 
      {    3.2600766335779724e+01,     1.5551614445522340e+04,    -3.3091588061916815e-05,     3.1058861259760758e-08,    -2.0400959701089275e-10, } , 
      {   -6.4352956633964226e+01,     1.4929988733224682e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    4.1979654784099594e+01,     1.5328137752203804e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.3942233503498819e+01,     8.1140748868454593e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.4255748687762100e+01,     8.3237603923658535e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    2.0206568765545711e+01,     1.6644264938048633e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.8842158166099530e+01,     8.1538897887433714e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -4.4490202116975347e+01,     7.3092346974425982e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    1.2405217967178359e+01,     1.4392788659779966e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.0470441727800477e+01,     8.3444239086227499e+03,    -9.7097753438400423e-05,     2.5713644981423790e-08,    -2.3343778745424160e-10, } , 
      {    4.6471937715397189e+01,     1.5033891410629989e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.2948732273943236e+01,     8.5433079624014408e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {   -2.7849362452189091e+01,     8.5034930605035279e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.0204503992548977e+01,     8.3301141155572022e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    7.8969472750927139e-01,     8.3272687336896961e+03,     1.5229240776232618e-04,     2.5071887415465472e-07,    -1.2359839986206508e-09, } , 
      {    1.2351941353953011e+01,    -1.3117911198267393e+01,    -3.7076799164848602e-04,    -4.3932002578978790e-07,     2.0639488032195161e-09, } , 
      {    4.6872302411250061e+01,     1.5367952654101717e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    3.4195419378112796e+01,     8.9593002096104028e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
      {    4.4514869503662972e+01,     7.0392612294782657e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {   -1.1478877768761816e+02,    -8.3304223931006873e+03,    -5.2306039941081215e-04,    -6.9003889994444262e-07,     3.2999328018401669e-09, } , 
      {    8.2816019329640884e+00,     1.4834264611607941e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.5327813943191533e+01,     1.5423309584288058e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -1.7678545252081697e+01,     7.0946181596646074e+03,    -2.1847558388615981e-04,    -1.8860115163513320e-07,     8.2796480459886528e-10, } , 
      {    9.2068852205906691e+00,     1.4911816872620067e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {    1.2917081399717340e+01,     1.5227285581300930e+04,    -6.6183176123833630e-05,     6.2117722519521515e-08,    -4.0801919402178550e-10, } , 
      {   -2.1736365956369216e+01,     8.3263902246820398e+03,     3.0458481552465236e-04,     5.0143774830930944e-07,    -2.4719679972413017e-09, } , 
      {    3.2730503217768330e+01,     8.3309926245648603e+03,     0.0000000000000000e+00,     0.0000000000000000e+00,     0.0000000000000000e+00, } , 
    } ,  
  } ;  
  
/*  
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  
 */  

/*
 * --- Initialization of time powers ------------------------------------
 */

  jstat = 0 ;
  
  tj = ( epoch1 - GAL_J2000 ) + epoch2 ;
  
  t[0] = 1.0 ;
  t[1] = tj / GAL_DJC ;
  t[2] = t[1] * t[1] ;
  t[3] = t[2] * t[1] ;
  t[4] = t[3] * t[1] ;
  
/*
 * --- Evaluation of the series: substitution of time in the series -----
 *     Variable iv=1 : Longitude
 *     Variable iv=2 : Latitude
 *     Variable iv=3 : Distance
 */

  for ( iv = 0; iv < 3; iv++ ) {  

    v[iv]   = 0.0 ;
    v[iv+3] = 0.0 ;

/*
 * ------ Periodic series ----------------------------------------------
 */

    for ( n = nmpb[1][iv]; n <= nmpb[2][iv]; n++ ) {  
      x = cmpb[icor][n] ;
      y = fmpb[icor][n][0] ;
      yp = 0.0 ;
      for ( k = 1; k <= 4; k++ ) {
        y = y + fmpb[icor][n][k] /* (k,n) */ * t[k] ;
        yp = yp + k * fmpb[icor][n][k] /* (k,n) */ * t[k-1] /* (k-1) */ ;
      }
      v[iv] += x * sin ( y ) ;
      v[iv+3] += x * yp * cos ( y ) ;
    }

/*
 * ------ Poisson series  ----------------------------------------------
 */
 
    for ( it = 0; it <= 3; it++ ) {
      for ( n = nper[1][it][iv]; n <= nper[2][it][iv]; n++ ) {
        x = cper[icor][n] ;
        y = fper[icor][n][0] ;
        xp = 0.0 ;
        yp = 0.0 ;
        if ( it != 0 ) {
          xp = it * x * t[it-1] ;
        } 
        for ( k = 1; k <= 4; k++ ) { 
          y = y + fper[icor][n][k] * t[k] ;
          yp = yp + k * fper[icor][n][k] * t[k-1] ;
        }
        v[iv] += x * t[it] * sin ( y ) ;
        v[iv+3] += xp * sin ( y ) + x * t[it] * yp * cos ( y ) ;
      }
    }

  }

/*
 * --- Computation of the rectangular coordinates (Epoch J2000) ---------           
 */
 
  v[0] = v[0] / GAL_R2AS + w[icor][0][0] + w[icor][1][0] * t[1] + w[icor][2][0] * t[2] + w[icor][3][0] * t[3] + w[icor][4][0] * t[4] ;
  v[1] = v[1] / GAL_R2AS ; 
  v[2] = v[2] * a405 / aelp ;
  v[3] = v[3] / GAL_R2AS + w[icor][1][0] + 2.0 * w[icor][2][0] * t[1] + 3.0 * w[icor][3][0] * t[2] + 4.0 * w[icor][4][0] * t[3] ;
  v[4] = v[4] / GAL_R2AS ;

  clamb  = cos ( v[0] ) ;
  slamb  = sin ( v[0] ) ;
  cbeta  = cos ( v[1] ) ;
  sbeta  = sin ( v[1] ) ;
  cw     = v[2] * cbeta ; ;
  sw     = v[2] * sbeta ; ;

  x1     = cw * clamb ;
  x2     = cw * slamb ;
  x3     = sw ;
  xp1    = ( v[5] * cbeta - v[4] * sw ) * clamb - v[3] * x2 ;
  xp2    = ( v[5] * cbeta - v[4] * sw ) * slamb + v[3] * x1 ;
  xp3    = v[5] * sbeta + v[4] * cw ;

  pw     = ( p1 + p2 * t[1] + p3 * t[2] + p4 * t[3] + p5 * t[4] ) * t[1] ;
  qw     = ( q1 + q2 * t[1] + q3 * t[2] + q4 * t[3] + q5 * t[4] ) * t[1] ;
  ra     = 2.0 * sqrt ( 1 - pw * pw - qw * qw ) ;
  pwqw   = 2.0 * pw * qw ;
  pw2    = 1.0 - 2.0 * pw * pw ;
  qw2    = 1.0 - 2.0 * qw * qw ;
  pwra   = pw * ra ;
  qwra   = qw * ra ;

  pv[0][0] = ( pw2 * x1 + pwqw * x2 + pwra * x3 ) * GAL_KM2M ;
  pv[0][1] = ( pwqw * x1 + qw2 * x2 - qwra * x3 ) * GAL_KM2M ;
  pv[0][2] = ( -pwra * x1 + qwra * x2 + ( pw2 + qw2 - 1 ) * x3 ) * GAL_KM2M ;

  ppw    = p1 + ( 2.0 * p2 + 3.0 * p3 * t[1] + 4.0 * p4 * t[2] + 5.0 * p5 * t[3] ) * t[1] ;
  qpw    = q1 + ( 2.0 * q2 + 3.0 * q3 * t[1] + 4.0 * q4 * t[2] + 5.0 * q5 * t[3] ) * t[1] ;
  ppw2   = -4.0 * pw * ppw ;
  qpw2   = -4.0 * qw * qpw ;
  ppwqpw = 2.0 * ( ppw * qw + pw * qpw ) ;
  rap    = ( ppw2 + qpw2 ) / ra ;
  ppwra  = ppw * ra + pw * rap ;
  qpwra  = qpw * ra + qw * rap ;

  pv[1][0] = ( pw2 * xp1 + pwqw * xp2 + pwra * xp3 + ppw2 * x1 + ppwqpw * x2 + ppwra * x3 ) / GAL_DJC * GAL_KM2M ;
  pv[1][1] = ( pwqw * xp1 + qw2 * xp2 - qwra * xp3 + ppwqpw * x1 + qpw2 * x2 - qpwra * x3 ) / GAL_DJC * GAL_KM2M ;  
  pv[1][2] = ( -pwra * xp1 + qwra * xp2 + ( pw2 + qw2 -1.0 ) * xp3 - ppwra * x1 + qpwra * x2 + ( ppw2 + qpw2 ) * x3 ) / GAL_DJC * GAL_KM2M ;

  return jstat ;
    
/*  
 * Finished.  
 */  

}

/*  
 *  gal - General Astrodynamics Library  
 *  Copyright (C) 2008 Paul C. L. Willmott  
 *  
 *  This program is free software; you can redistribute it and/or modify  
 *  it under the terms of the GNU General Public License as published by  
 *  the Free Software Foundation; either version 2 of the License, or  
 *  (at your option) any later version.  
 *  
 *  This program is distributed in the hope that it will be useful,  
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of  
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the  
 *  GNU General Public License for more details.  
 *  
 *  You should have received a copy of the GNU General Public License along  
 *  with this program; if not, write to the Free Software Foundation, Inc.,  
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.  
 *  
 *  Contact:  
 *  
 *  Paul Willmott  
 *  vp9mu@amsat.org  
 */  
